!**************************************************************************************************
!
!    ####    ####   #    #   ####  #####
!   #    #  #    #  ##   #  #        #
!   #       #    #  # #  #   ####    #
!   #       #    #  #  # #       #   #
!   #    #  #    #  #   ##  #    #   #
!    ####    ####   #    #   ####    # _module
!
!   Global physical and mathematical constants and units
!
!**************************************************************************************************
!   This modules does not provide any routine but a number of physical and mathematical constants.
!   In addition, there are a number of unit strings for several quantities useful for the output
!   of data into files. This module is based on cgs units.
!**************************************************************************************************
!   Fortran90
!   Bernd Freytag:         Copenhagen, Uppsala, Lyon
!   2013-09-27
!**************************************************************************************************
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!
!**************************************************************************************************


!------************--------------------------------------------------------------------------------
module const_module
!--------------------------------------------------------------------------------------------------
! NAME:
!   const_module ('constants_module')
!
! PURPOSE:
!   Provide global physical and mathematical constants and units.
!
! CATEGORY:
!   Constants
!
! CALLING SEQUENCE:
!   use const_module
!
! VARIABLES:
!   G_grav_drk:  ('G_gravity_double_real_kind') 6.6726E-08_drk, from Unsoeld p.402, corrected
!
! RESTRICTIONS:
!   This module should contain all constants and units used in CO5BOLD.
!   However, there are lots of exceptions -- typically in routines not written by B.F.
!   Therefore, to change a constant (or a unit) system you have to change things here and then
!   to go through all routines to check what is needed.
!
! TODO:
!   Put all constants here and use only them.
!
! MODIFICATION HISTORY:
!   2000-01-08 (B.F. Copenhagen) Written
!   2000-02-25 (B.F.) sigma=5.6705E-05 -> sigma=5.6705E-05
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2002-08-07 (B.F. Uppsala) rhd_const_module -> const_module
!   2002-11-01 (B.F.) Gauss added
!   2002-11-09 (B.F.) Bugfix: rhov_unit ='g/cm^' -> 'g/cm^2/s'
!   2006-10-27 (B.F. Lyon) Corrected typos in comments
!   2007-01-31 (B.F.) frequency_unit
!   2007-04-16 (B.F.) k_Bol_drk, amu_drk, R_gas_drk from rossow_sed.f90
!   2010-02-03 (B.F.) B_unit= 'G' -> 'G*sqrt(4pi)'
!   2013-07/08 (B.F.) Units for moment of inertia and angula momentum (density)
!--------------------------------------------------------------------------------------------------
implicit none
!
! --- Global parameters ---
!!$integer, parameter, private             :: srk=kind(0.0E+00), drk=kind(0.0D+00)
integer, parameter, private             :: srk=kind(0.0E+00), drk=selected_real_kind(10)
!
! --- Double precision ---
! --- Fundamental mathematical constants ---
real (kind=drk), parameter              :: pi_drk     =3.1415926535897932384626433_drk
!
! --- Derived mathematical constants ---
real (kind=drk), parameter              :: halfpi_drk =0.5_drk*pi_drk, &
                                           twopi_drk  =2.0_drk*pi_drk, &
                                           fourpi_drk =4.0_drk*pi_drk
!
! --- Fundamental physical constants ---
real (kind=drk), parameter              :: sigma_drk  =5.6705E-05_drk    , &
                                           G_grav_drk =6.6726E-08_drk    , &
                                           c_light_drk=2.99792458E+10_drk, &
                                           k_Bol_drk  =1.380658E-16_drk  , &
                                           amu_drk    =1.6605402E-24_drk , &
                                           R_gas_drk  =8.3145E+07_drk
!
! --- Derived physical constants ---
real (kind=drk), parameter              :: sigopi_drk =sigma_drk/pi_drk
!
! --- Single precision ---
real (kind=srk), parameter              :: pi_srk     =pi_drk     , &
                                           halfpi_srk =halfpi_drk , &
                                           twopi_srk  =twopi_drk  , &
                                           fourpi_srk =fourpi_drk , &
                                           sigma_srk  =sigma_drk  , &
                                           G_grav_srk =G_grav_drk , &
                                           c_light_srk=c_light_drk, &
                                           k_Bol_srk  =k_Bol_drk  , &
                                           amu_srk    =amu_drk    , &
                                           R_gas_srk  =R_gas_drk  , &
                                           sigopi_srk =sigopi_drk
!
real (kind=srk), parameter              :: pi         =pi_drk     , &
                                           halfpi     =halfpi_drk , &
                                           twopi      =twopi_drk  , &
                                           fourpi     =fourpi_drk , &
                                           sigma      =sigma_drk  , &
                                           G_grav     =G_grav_drk , &
                                           c_light    =c_light_drk, &
                                           k_Bol      =k_Bol_srk  , &
                                           amu        =amu_drk    , &
                                           R_gas      =R_gas_drk  , &
                                           sigopi     =sigopi_drk
!
! --- Unit names ---
character(len=*), parameter             :: time_unit       ='s'         , &
                                           frequency_unit  ='1/s'       , &
                                           x1_unit         ='cm'        , &
                                           x2_unit         ='cm'        , &
                                           x3_unit         ='cm'        , &
                                           v_unit          ='cm/s'      , &
                                           grav_unit       ='cm/s^2'    , &
                                           mass_unit       ='g'         , &
                                           rho_unit        ='g/cm^3'    , &
                                           rhov_unit       ='g/cm^2/s'  , &
                                           rhovv_unit      ='g/cm/s^2'  , &
                                           MomIne_unit     ='g/cm'      , &
                                           AngMom_unit     ='g/cm/s'    , &
                                           e_unit          ='erg/g'     , &
                                           rhoe_unit       ='erg/cm^3'  , &
                                           fe_unit         ='erg/cm^2/s', &
                                           luminpervol_unit='erg/cm^3/s', &
                                           p_unit          ='dyn/cm^2'  , &
                                           T_unit          ='K'         , &
                                           s_unit          ='erg/K/g'   , &
                                           rhos_unit       ='erg/K/cm^3', &
                                           kappa_unit      ='cm^2/g'    , &
                                           kapparho_unit   ='1/cm'      , &
                                           u_unit          ='u'         , &
                                           B_unit          ='G*sqrt(4pi)'
!
end module const_module
