!****************************************************************************************************
!
!   #      # #####  #####    ##   #####  #   # 
!   #      # #    # #    #  #  #  #    #  # #  
!   #      # #####  #    # #    # #    #   #   
!   #      # #    # #####  ###### #####    #   
!   #      # #    # #   #  #    # #   #    #   
!   ###### # #####  #    # #    # #    #   # _routines
!
!   Interfaces for Fortran library routines
!
!****************************************************************************************************
!   This file contains minimal interface definitions for some standard Fortran, almost-standard,
!   OpenMP, and machine-specific subroutines.
!****************************************************************************************************
!   Bernd Freytag:         Lyon
!   2014-09-12
!****************************************************************************************************
! MODULES:
!   library_module:        Minimal interface definitions for some subroutines
!
!****************************************************************************************************


!------**************--------------------------------------------------------------------------------
module library_module
!----------------------------------------------------------------------------------------------------
! NAME:
!   library_module ('library_module')
!
! PURPOSE:
!   Provide some minimal interface definitions for some subroutines
!
!----------------------------------------------------------------------------------------------------
!
contains

!----------*************-----------------------------------------------------------------------------
subroutine date_and_time(date, time, zone, values)
!----------------------------------------------------------------------------------------------------
! NAME:
!   date_and_time ('date_and_time')
!
! PURPOSE:
!   Give date, time, and time zone.
!
! CATEGORY:
!   Standard Fortran intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/DATE_005fAND_005fTIME.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine date_and_time


!----------*****-------------------------------------------------------------------------------------
subroutine flush(unit)
!----------------------------------------------------------------------------------------------------
! NAME:
!   flush ('flush')
!
! PURPOSE:
!   Flush Fortran unit(s)
!
! CATEGORY:
!   Standard Fortran subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/FLUSH.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine flush


!----------*************-----------------------------------------------------------------------------
subroutine random_number(result)
!----------------------------------------------------------------------------------------------------
! NAME:
!   random_number ('random_number')
!
! PURPOSE:
!   Compute next pseudo-random number(s).
!
! CATEGORY:
!   Standard Fortran intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/RANDOM_005fNUMBER.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine random_number


!----------***********-------------------------------------------------------------------------------
subroutine random_seed(size, put, get)
!----------------------------------------------------------------------------------------------------
! NAME:
!   random_seed ('random_seed')
!
! PURPOSE:
!   Initialize random number generator.
!
! CATEGORY:
!   Standard Fortran intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/RANDOM_005fSEED.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine random_seed


!----------************------------------------------------------------------------------------------
subroutine system_clock(count, count_rate, count_max)
!----------------------------------------------------------------------------------------------------
! NAME:
!   system_clock ('system_clock')
!
! PURPOSE:
!   Give the system time.
!
! CATEGORY:
!   Standard Fortran intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/SYSTEM_005fCLOCK.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine system_clock


!****************************************************************************************************


!----------****************--------------------------------------------------------------------------
subroutine omp_get_schedule(kind, modifier)
!----------------------------------------------------------------------------------------------------
! NAME:
!   omp_get_schedule ('open_multi-processing_get_schedule')
!
! PURPOSE:
!   Optain the runtime scheduling method.
!
! CATEGORY:
!   Standard Fortran OpenMP subroutine
!
! PROCEDURE:
!   See
!   \href{https://gcc.gnu.org/onlinedocs/libgomp/omp_005fget_005fschedule.html}{gfortran docs}.
!   \href{http://pic.dhe.ibm.com/infocenter/compbg/v121v141/index.jsp?topic=%2Fcom.ibm.xlf141.bg.doc%2Fproguide%2Fompgetschedule.html}{IBM documentation}
!
!----------------------------------------------------------------------------------------------------
!
! --- I/O parameters ---
integer(kind=omp_sched_kind), &
                           intent(out)  :: kind
integer,                   intent(out)  :: modifier
!----------------------------------------------------------------------------------------------------
!
end subroutine omp_get_schedule


!----------*******************-----------------------------------------------------------------------
subroutine omp_set_num_threads(num_threads)
!----------------------------------------------------------------------------------------------------
! NAME:
!   omp_set_num_threads ('open_multi-processing_set_number_threads')
!
! PURPOSE:
!   Set the number of threads.
!
! CATEGORY:
!   Standard Fortran OpenMP subroutine
!
! PROCEDURE:
!   See
!   \href{https://gcc.gnu.org/onlinedocs/libgomp/omp_005fset_005fnum_005fthreads.html}{gfortran docs}.
!   \href{http://pic.dhe.ibm.com/infocenter/comphelp/v111v131/index.jsp?topic=%2Fcom.ibm.xlc111.aix.doc%2Fcompiler_ref%2Fbif_omp_set_num_threads.html}{IBM documentation}
!
!----------------------------------------------------------------------------------------------------
!
! --- I/O parameters ---
integer,                   intent(in)   :: num_threads
!----------------------------------------------------------------------------------------------------
!
end subroutine omp_set_num_threads


!****************************************************************************************************


!----------*******-----------------------------------------------------------------------------------
subroutine asnunit(channel, string, ierr)
!----------------------------------------------------------------------------------------------------
! NAME:
!   asnunit ('assign_unit')
!
! PURPOSE:
!   Assign attribute to unit.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{https://fs.hlrs.de/projects/craydoc/docs_merged/man/ffiom/74/cat3/assign.3f.html}{HLRS docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine asnunit


!----------******------------------------------------------------------------------------------------
subroutine assign(cmd, ierr)
!----------------------------------------------------------------------------------------------------
! NAME:
!   assign ('assign')
!
! PURPOSE:
!   Assign attibute to unit.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{https://fs.hlrs.de/projects/craydoc/docs_merged/man/ffiom/74/cat3/assign.3f.html}{HLRS docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine assign


!----------*****-------------------------------------------------------------------------------------
subroutine clock(p, q)
!----------------------------------------------------------------------------------------------------
! NAME:
!   clock ('clock')
!
! PURPOSE:
!   Measure the CPU time and the elapsed time of any part of the program.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{\detokenize{http://www.hitachi.co.jp/Prod/comp/hpc/foruser/sr8000/tebiki-e/clock.html}}{Hitachi docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine clock


!----------*****-------------------------------------------------------------------------------------
subroutine etime(tarray, result)
!----------------------------------------------------------------------------------------------------
! NAME:
!   etime ('execution_time')
!
! PURPOSE:
!   Give the elapsed execution time of runtime in seconds.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/ETIME.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine etime


!----------*****-------------------------------------------------------------------------------------
subroutine HF_SH(cmd, ireturn)
!----------------------------------------------------------------------------------------------------
! NAME:
!   HF_SH ('HF_SHell')
!
! PURPOSE:
!   Execute string as shell command.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
!----------------------------------------------------------------------------------------------------
!
end subroutine HF_SH




!----------*********---------------------------------------------------------------------------------
subroutine pxfstrget(handle, compnam, value, ilen, ierror)
!----------------------------------------------------------------------------------------------------
! NAME:
!   pxfstrget ('pxf_structure_get')
!
! PURPOSE:
!   Extract individual values from structure.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{https://fs.hlrs.de/projects/craydoc/docs_merged/man/pxfm/74/cat3/pxfstrget.3f.html}{HLRS docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine pxfstrget


!----------***************---------------------------------------------------------------------------
subroutine pxfstructcreate(structname, jhandle, ierror)
!----------------------------------------------------------------------------------------------------
! NAME:
!   pxfstructcreate ('pxf_structure_create')
!
! PURPOSE:
!   Create an instance of the desired structure.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{https://fs.hlrs.de/projects/craydoc/docs_merged/man/pxfm/74/cat3/pxfstructcreate.3f.html}{HLRS docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine pxfstructcreate


!----------*************-----------------------------------------------------------------------------
subroutine pxfstructfree(structname, jhandle, ierror)
!----------------------------------------------------------------------------------------------------
! NAME:
!   pxfstructfree ('pxf_structure_free')
!
! PURPOSE:
!   Free the instance of the structure referenced by jhandle.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{https://fs.hlrs.de/projects/craydoc/docs_merged/man/pxfm/74/cat3/pxfstructfree.3f.html}{HLRS docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine pxfstructfree


!----------********----------------------------------------------------------------------------------
subroutine pxfuname(structname, jhandle, ierror)
!----------------------------------------------------------------------------------------------------
! NAME:
!   pxfuname ('pxf_uname')
!
! PURPOSE:
!   Retrieve the operating-system name.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{https://fs.hlrs.de/projects/craydoc/docs_merged/man/pxfm/74/cat3/pxfuname.3f.html}{HLRS docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine pxfuname


!----------******------------------------------------------------------------------------------------
subroutine system(command, status)
!----------------------------------------------------------------------------------------------------
! NAME:
!   system ('system')
!
! PURPOSE:
!   Execute a shell command
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{http://gcc.gnu.org/onlinedocs/gfortran/SYSTEM.html}{gfortran docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine system


!----------*******-----------------------------------------------------------------------------------
subroutine tremain(time)
!----------------------------------------------------------------------------------------------------
! NAME:
!   tremain ('time_remain')
!
! PURPOSE:
!   Give the CPU time (in seconds, of type real) remaining for the program.
!
! CATEGORY:
!   Nonstandard intrinsic subroutine
!
! PROCEDURE:
!   See
!   \href{\detokenize{http://docs.cray.com/books/S-2310-51/html-S-2310-51/fortranlibs.html}}{Cray docs}.
!
!----------------------------------------------------------------------------------------------------
!
end subroutine tremain

end module library_module
