!******************************************************************************
!       EOS Test Routines
!       Fortran90 version
!       17-Mar-1998 18:25:51
!       2015-04-03
!******************************************************************************
! Non-standard FORTRAN:
!   Hopefully none!
!******************************************************************************
! MODULES:
!   eostest_module:        Routines for testing of EOS routines
!
! PROGRAM:
!   eostest:               Compare GAS and interpolated EOS
!
!******************************************************************************


!------**************----------------------------------------------------------
module eostest_module
!------------------------------------------------------------------------------
! NAME:
!   eostest_module ('equation_of_state_test_module')
!
! PURPOSE:
!   Provide routines for testing of EOS routines.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   use eostest_module
!
! VARIABLES:
!
! ROUTINES: (contained)
!   eostest_Check:         Check accuracy of interpolation table and speed of interpolation
!
! MODIFICATION HISTORY:
!   19. 5.97 Put into module by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
contains

subroutine eostest_Check(ncpri, nx1_check,nx2_check)
!------------------------------------------------------------------------------
! NAME:
!   eostest_Check ('equation_of_state_check')
!
! PURPOSE:
!   Check accuracy of interpolation table and speed of interpolation.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eostest_Check(ncpri, nx1_check,nx2_check)
!
! INPUT:
!   ncpri:        ('number_channel_print') integer, logical output channel number
!   nx1_check:    ('number_x_1_check') integer, number of test points in x1 direction
!   nx2_check:    ('number_x_2_check') integer, number of test points in x2 direction
!
! OUTPUT:
!   None
!
! ROUTINES:
!   eosinter:
!   gas:                   EOS routine of RHD-code (or gasgas)
!   gasinter:
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   19. 5.97 Written by Bernd Freytag, Astrophysik Kiel
!   24. 6.97 (B.F.) Write GAS and interpolated quantities into file
!------------------------------------------------------------------------------
use tabinter_coeff_module
use gasinter_module
use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
integer,                     intent(in) :: ncpri, nx1_check,nx2_check
!
! --- Local variables ---
character*(10)                          :: time
integer                                 :: ierror, ifent, kenter
real, dimension(nx1_check,nx2_check)    :: P,  T, &
                                           rho,  cpd,  Gamma1,  delta,  h, s, &
                                           cs, e, kout
real, dimension(nx1_check,nx2_check)    :: P1, T1, &
                                           rho1, cpd1, Gamma11, delta1, h1,s1, &
                                           cs1, e1, &
                                           dPdrho1, dPde1, dTde1
real, dimension(nx1_check)              :: lnx11d_h, x11d_h
real, dimension(nx2_check)              :: lnx21d_h, x21d_h
integer                                 :: ix1, ix2, ncerr
!------------------------------------------------------------------------------
!
! --- Initialize GAS routine by calling it with kenter=-1 ---
kenter=-1
ifent=0
call gas(8000.0, 16000.0, &
         rho(1,1), cpd(1,1), Gamma1(1,1), delta(1,1), h(1,1), s(1,1), cs(1,1), &
         ierror, EOS%ifion, ifent, kenter, EOS%abuX, EOS%abuY, EOS%Qmol)
!
! --- Array with test values ---
lnx11d_h=alog(EOS%x1min+EOS%x1_shift) + &
         (/(ix1,ix1=0,nx1_check-1)/)/real(nx1_check-1)* &
         (alog(EOS%x1max+EOS%x1_shift)-alog(EOS%x1min+EOS%x1_shift))
x11d_h=exp(lnx11d_h)-EOS%x1_shift
lnx21d_h=alog(EOS%x2min+EOS%x2_shift) + &
         (/(ix2,ix2=0,nx2_check-1)/)/real(nx2_check-1)* &
         (alog(EOS%x2max+EOS%x2_shift)-alog(EOS%x2min+EOS%x2_shift))
x21d_h=exp(lnx21d_h)-EOS%x2_shift
!
if (EOS%variables == 'lnP,lnT->lnrho,h,s') then
  !
  ! --- Arrays of independent variables ---
  P=spread(x11d_h,2,nx2_check)
  T=spread(x21d_h,1,nx1_check)
  P1=P
  T1=T
  !
  ! --- Compute GAS quantities with new GASINTER routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of gasinter:       ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      call gasinter(P1(ix1,ix2), T1(ix1,ix2), &
                    rho1(ix1,ix2), cpd1(ix1,ix2), Gamma11(ix1,ix2), &
                    delta1(ix1,ix2), h1(ix1,ix2), s1(ix1,ix2), cs1(ix1,ix2), &
                    ierror, ifent)
      e1(ix1,ix2)=h1(ix1,ix2)-P1(ix1,ix2)/rho1(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after gasinter:                 ' // time
  !
  ! --- Compute GAS quantities with old GAS routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of GAS:            ' // time
  !
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      kenter=0
      call gas(P(ix1,ix2), T(ix1,ix2), &
               rho(ix1,ix2), cpd(ix1,ix2), Gamma1(ix1,ix2), &
               delta(ix1,ix2), h(ix1,ix2), s(ix1,ix2), cs(ix1,ix2), &
               ierror, EOS%ifion, ifent, kenter, &
               EOS%abuX, EOS%abuY, EOS%Qmol)
      kout(ix1,ix2)=real(kenter)
      e(ix1,ix2)=h(ix1,ix2)-P(ix1,ix2)/rho(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GAS:                      ' // time
  !
else if (EOS%variables == 'lnrho,lne->s,lnP,lnT') then
  !
  ! --- Arrays of independent variables ---
  rho1=spread(x11d_h,2,nx2_check)
  e1  =spread(x21d_h,1,nx1_check)
  !
  ! --- Compute EOS quantities with new EOSINTER routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of eosinter0:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    do ix1=1,nx1_check
!DIR$ INLINE
      call eosinter(rho1(ix1,ix2), e1(ix1,ix2), &
                    P=P1(ix1,ix2), dPdrho=dPdrho1(ix1,ix2), &
                                   dPde=dPde1(ix1,ix2), &
                    T=T1(ix1,ix2), dTde=dTde1(ix1,ix2), &
                    s=s1(ix1,ix2))
      cpd1(ix1,ix2)=P1(ix1,ix2)/(rho1(ix1,ix2)*T1(ix1,ix2)) + &
                    rho1(ix1,ix2)*dPdrho1(ix1,ix2)/(T1(ix1,ix2)*dPde1(ix1,ix2))
      Gamma11(ix1,ix2)=T1(ix1,ix2)*cpd1(ix1,ix2)*dPde1(ix1,ix2)/P1(ix1,ix2)
      delta1(ix1,ix2) =1.0/(cpd1(ix1,ix2)*dTde1(ix1,ix2)- &
                           (Gamma11(ix1,ix2)*P1(ix1,ix2))/ &
                           (T1(ix1,ix2)*rho1(ix1,ix2)*cpd1(ix1,ix2)))
      cs1(ix1,ix2)=sqrt(Gamma11(ix1,ix2)*P1(ix1,ix2)/rho1(ix1,ix2))
      h1(ix1,ix2)=e1(ix1,ix2)+P1(ix1,ix2)/rho1(ix1,ix2)
     end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after eosinter0:                ' // time
  !
  ! --- Compute EOS quantities with new EOSINTER1 routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of eosinter1:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
!DIR$ INLINE
    call eosinter(rho1(:,ix2), e1(:,ix2), &
                  P=P1(:,ix2), dPdrho=dPdrho1(:,ix2), dPde=dPde1(:,ix2), &
                  T=T1(:,ix2), dTde=dTde1(:,ix2), &
                  s=s1(:,ix2))
    cpd1(:,ix2)=P1(:,ix2)/(rho1(:,ix2)*T1(:,ix2)) + &
                  rho1(:,ix2)*dPdrho1(:,ix2)/(T1(:,ix2)*dPde1(:,ix2))
    Gamma11(:,ix2)=T1(:,ix2)*cpd1(:,ix2)*dPde1(:,ix2)/P1(:,ix2)
    delta1(:,ix2) =1.0/(cpd1(:,ix2)*dTde1(:,ix2)- &
                         (Gamma11(:,ix2)*P1(:,ix2))/ &
                         (T1(:,ix2)*rho1(:,ix2)*cpd1(:,ix2)))
    cs1(:,ix2)=sqrt(Gamma11(:,ix2)*P1(:,ix2)/rho1(:,ix2))
    h1(:,ix2)=e1(:,ix2)+P1(:,ix2)/rho1(:,ix2)
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after eosinter1:                ' // time
  !
  P=P1
  T=T1
  !
  ! --- Compute EOS quantities with old GAS routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of GAS:            ' // time
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      kenter=0
      call gas(P(ix1,ix2), T(ix1,ix2), &
               rho(ix1,ix2), cpd(ix1,ix2), Gamma1(ix1,ix2), &
               delta(ix1,ix2), h(ix1,ix2), s(ix1,ix2), cs(ix1,ix2), &
               ierror, EOS%ifion, ifent, kenter, &
               EOS%abuX, EOS%abuY, EOS%Qmol)
      kout(ix1,ix2)=real(kenter)
      e(ix1,ix2)=h(ix1,ix2)-P(ix1,ix2)/rho(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GAS:                      ' // time
  !
endif
!
! --- Results ---
write(ncpri,*)
write(ncpri,'(A)') 'Ranges of variables'
write(ncpri,'(A)') '-------------------'
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   P: ', minval(P),   maxval(P)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   T: ', minval(T),   maxval(T)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range: rho: ', minval(rho), maxval(rho)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   e: ', minval(e),   maxval(e)
write(ncpri,'(A)') 'Accuracy of interpolation'
write(ncpri,'(A)') '-------------------------'
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    s: ', &
  maxval(abs(   s-   s1))/maxval(abs(s))
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    h: ', &
  maxval(abs(   h-   h1))/maxval(abs(h))
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    e: ', &
  maxval(abs(   e-   e1))/maxval(abs(e))
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:      P: ', &
  maxval(abs(   P-   P1)/  P)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:      T: ', &
  maxval(abs(   T-   T1)/  T)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:    rho: ', &
  maxval(abs( rho- rho1)/rho)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:    cpd: ', &
  maxval(abs( cpd- cpd1)/cpd)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:  delta: ', &
  maxval(abs( delta- delta1)/delta)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation: Gamma1: ', &
  maxval(abs(Gamma1-Gamma11)/Gamma1)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:     cs: ', &
  maxval(abs(cs-cs1)/cs)
write(ncpri,'(A,F13.8)') 'RMS     relative deviation:    rho: ', &
  sqrt(sum(( rho- rho1)**2/rho**2)/real(nx1_check*nx2_check))
write(ncpri,'(A,F13.8)') 'RMS     relative deviation:    cpd: ', &
  sqrt(sum(( cpd- cpd1)**2/cpd**2)/real(nx1_check*nx2_check))
!
! --- Write original and interpolated arrays into file ---
call uio_openwr(ncerr, 'eostest_err.dat', form=EOS%form, conv=EOS%conv)
!
! --- Write file_id ---
call uio_wr(ncerr, 'eos-tables: error', 'file_id', name='File identification')
!
call uio_wr(ncerr, p, 'p', format=EOS%number_format)
call uio_wr(ncerr, p1, 'p1', format=EOS%number_format)
call uio_wr(ncerr, t, 't', format=EOS%number_format)
call uio_wr(ncerr, t1, 't1', format=EOS%number_format)
call uio_wr(ncerr, s, 's', format=EOS%number_format)
call uio_wr(ncerr, s1, 's1', format=EOS%number_format)
call uio_wr(ncerr, h, 'h', format=EOS%number_format)
call uio_wr(ncerr, h1, 'h1', format=EOS%number_format)
call uio_wr(ncerr, rho, 'rho', format=EOS%number_format)
call uio_wr(ncerr, rho1, 'rho1', format=EOS%number_format)
call uio_wr(ncerr, delta, 'delta', format=EOS%number_format)
call uio_wr(ncerr, delta1, 'delta1', format=EOS%number_format)
call uio_wr(ncerr, Gamma1, 'gamma1', format=EOS%number_format)
call uio_wr(ncerr, Gamma11, 'gamma11', format=EOS%number_format)
call uio_wr(ncerr, cpd, 'cpd', format=EOS%number_format)
call uio_wr(ncerr, cpd1, 'cpd1', format=EOS%number_format)
!
! --- Close file ---
call uio_closwr(ncerr)
!
end subroutine eostest_Check

end module eostest_module


program eostest
!------------------------------------------------------------------------------
! NAME:
!   eostest ('equation_of_state_test')
!
! PURPOSE:
!   Compare GAS and interpolated EOS.
!
! CATEGORY:
!   EOS
!
! VARIABLES:
!   file:         ('file') character, output file name
!   ncpri:        ('number_channel_print') integer, logical number for output
!   parfile:      ('parameter_file') character, parameter file name
!   time:         ('time') character, time string
!
! SUBROUTINES:
!   tabinter_rdcoeff:      Read coefficients for bicubic interpolation of EOS quantities from file
!   readpar:               Read parameter from input parameter file
!   uio_init:              Initialization of UIO routines
!
! MODULES:
!   uio_bulk_module:       Main set of UIO routines
!   tabinter_module:       Routines for handling of interpolation coefficients
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   gasinter_setup_module: Routines for setting up coefficients for EOS interpolation routine
!
! PROCEDURE:
!   Read parameter file,
!   clear EOS table,
!   read interpolation coefficients back from file,
!   compare GAS and interpolated EOS.
!
! MODIFICATION HISTORY:
!   19. 5.97 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_bulk_module
use tabinter_module
use gasinter_module
use eostest_module
!
implicit none
!
! --- Variables ---
integer, parameter                      :: ncpri=6
character*(*), parameter                :: parfile='eostest_par.txt'
character*(80)                          :: file
character(len=10)                       :: time
integer                                 :: nx1_check,nx2_check
!------------------------------------------------------------------------------
!
write(ncpri,*)
write(ncpri,*) ' EOSTEST'
write(ncpri,*) ' ======='
write(ncpri,*)
!
! --- Initialize uio routines ---
call uio_init(progrm='eostest')
!
! --- Choose table number, set coefficients to zero, deallocate arrays ---
call tabinter_reset(1, ncpri)
!
! --- Read parameter file, result in global variable EOS ---
call readpar(parfile, ncpri, file, nx1_check,nx2_check)
!
! --- Reload coefficients from file ---
call tabinter_rdcoeff(file, ncpri)
!
! --- Check accuracy of interpolation table and speed of interpolation ---
call eostest_Check(ncpri, nx1_check,nx2_check)
!
! --- Deallocate tables ---
call tabinter_delete(1, ncpri)
!
contains

!----------*******-------------------------------------------------------------
subroutine readpar(parfile, ncpri, file, nx1_check,nx2_check, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   readpar ('read_parameter')
!
! PURPOSE:
!   Read parameter from input parameter file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call readpar(parfile, ncpri, outstr,ierr)
!
! INPUT:
!   parfile:      ('parameter_file') name of parameter file
!   ncpri:        ('number_channel_print') integer, logical number for output
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   file:         ('file') character, name of file for coefficients
!   outstr:       ('output_string') character, optional
!   ierr:         ('integer_error') integer, optional, contains exit error status
!                   0: correct operation
!                  >0: occurance of an error
!
! ROUTINES:
!   uio_closrd:            Close file after reading
!   uio_err:               Handle UIO errors
!   uio_filcon:            Determine file contents: list of all entries with its positions
!   uio_openrd:            Open file for reading, read header
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_srhd:              Search entry header in file or in entry list
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   uio_siz_module:        String length, table size for UIO
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   A file is opened, read, and closed. Parameters are stored in saved global
!   variable EOS.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Open requested file (in uio format).
!   Determine file contents.
!   Search each parameter, read it and put it into variable EOS.
!   Close file again.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   19. 5.97 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
character*(*),           intent(in)     :: parfile
integer,                 intent(in)     :: ncpri
character*(*),           intent(out)    :: file
integer,                 intent(out)    :: nx1_check, nx2_check
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: ncpar, ierr0, ntt
character*(80)                          :: outstr0
character*(let)                         :: termt(2,nttmx)
character*(let)                         :: ident
integer, parameter                      :: nentmx=200
integer                                 :: nentry
integer                                 :: poslist(nentmx)
character*(let)                         :: identlist(3,nentmx)
integer                                 :: iup(2)
!------------------------------------------------------------------------------
!
! --- Open parameter file for reading ---
call uio_openrd(ncpar, parfile)
!
write(ncpri,'(A,I3)') 'Parameter file ''' // trim(parfile) // &
                      ''' opened on channel ', ncpar
!
! --- Determine parameter file contents ---
call uio_filcon(ncpar, identlist,poslist,nentry)
!
!write(ncpri,*) 'nentry=', nentry
!write(ncpri,'(A)') identlist(:,:nentry)
!write(ncpri,'(I4)') poslist(:nentry)
!
! --- Read file_id ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='file_id', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%file_id,ident)
  if (EOS%file_id /= 'eostest-parameter') then
    outstr0='''' // trim(parfile) // ''' is not eostest paramter file'
    ierr0=1
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='readpar', &
                 errlevel=3)
  endif
else
  outstr0='Type of ''' // trim(parfile) // ''' can not be determined'
  ierr0=2
  call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='readpar', &
               errlevel=3)
endif
!
! --- Read description ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='description', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%description,ident,mode=2,iup=iup(1:1))
  EOS%ndes=iup(1)
else
  EOS%ndes=0
endif
write(ncpri,'(A)') &
  'Description: --------------------------------------------------------------'
write(ncpri,'(A)') EOS%description(:EOS%ndes)
!
! --- Read history ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='history', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%history,ident,mode=2,iup=iup(1:1))
  EOS%nhis=iup(1)
else
  EOS%nhis=0
endif
write(ncpri,'(A)') &
  'History: ------------------------------------------------------------------'
write(ncpri,'(A)') EOS%history(:EOS%nhis)
write(ncpri,'(A)') &
  '---------------------------------------------------------------------------'
!
! --- Read output file name ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='file', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%file,ident)
else
  EOS%file='eostest_out.txt'
endif
write(ncpri,'(A)') 'Output filename:   ' // trim(EOS%file)
file=EOS%file
!
! --- Some default values ---
nx1_check=67
nx2_check=93
!
! --- Read test table size: first dimension ---
call uio_srhd(ncpar, termt,ntt, type=intena, ident='nx1', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, nx1_check,ident)
endif
write(ncpri,'(A,I15)') 'nx1:   ', nx1_check
!
! --- Read test table size: second dimension ---
call uio_srhd(ncpar, termt,ntt, type=intena, ident='nx2', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, nx2_check,ident)
endif
write(ncpri,'(A,I15)') 'nx2:   ', nx2_check
!
! --- Close parameter file after reading ---
call uio_closrd(ncpar)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine readpar
!
end program eostest
