!***************************************************************************************************
!
!    ####    ##    ####   #####  ######  ####   ####  
!   #    #  #  #  #            # #      #    # #      
!   #      #    #  ####   #####  #####  #    #  ####  
!   #  ### ######      # #       #      #    #      # 
!   #    # #    # #    # #       #      #    # #    # 
!    ####  #    #  ####  ####### ######  ####   ####  
!
!   EOS table generation routines routines
!
!***************************************************************************************************
!   This program computes the equation of state with the gasgas routine or a simple gamma law,
!   inverts the equation if necessary, tabulates the coefficients, and writes them into a file
!   in UIO format.
!***************************************************************************************************
!   Fortran90
!   Bernd Freytag:         Kiel, Uppsala, Lyon
!   2016-03-16
!***************************************************************************************************
! MODULES:
!   gasinter_setup_module: Routines for setting up coefficients for EOS interpolation routine
!
! PROGRAM:
!   gas2eos:               Put EOS from GAS-routine into table
!
!***************************************************************************************************


!------*********************------------------------------------------------------------------------
module gasinter_setup_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_setup_module ('gas_interpolation_set_up_module')
!
! PURPOSE:
!   Provide routines for setting up coefficients for EOS interpolation routine.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   use gasinter_setup_module
!
! VARIABLES:
!
! ROUTINES: (contained)
!   gasinter_ReadPar:      Read parameters from input parameter file
!   gasinter_AllDerivatives: Compute numerically 1st and 2nd derivatives for 2D array function
!   gasinter_ModifyGas:    Modify precomputed set of EOS quantities
!   gasinter_SetUpGas:     Compute coefficients for later bicubic interpolation of GAS quantities
!   gasinter_SetUpgamma:   Compute coeffs for bicubic interpolation of EOS qs with simple gamma law
!   gasinter_SetUp:        Compute coefficients for bicubic interpolation of GAS/EOS quantities
!   gasinter_CheckGas:     Check accuracy of interpolation table for GAS quantities
!   gasinter_Checkgamma:   Check accuracy of interpolation table for gamma law
!   gasinter_Check:        Check accuracy of interpolation table
!
! MODIFICATION HISTORY:
!   1997-03-11 (B.F. Kiel) Put into module
!   1997-04-29 (B.F.) gasinter_check
!   1997-05-23 (B.F.) gasinter_ReadPar into module
!   2004-02-26 (B.F. Uppsala) Call supergas instead of gasgas
!   2014-09-06 (B.F.) Compute and output additional arrays
!   2014-09-09 (B.F.) Some cleanup; option to get EOS quantities only from s and
!                     its numerical derivatives
!---------------------------------------------------------------------------------------------------
contains

!----------****************-------------------------------------------------------------------------
subroutine gasinter_ReadPar(parfile, ncpri, file, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_ReadPar ('gas_interpolation_Read_Parameters')
!
! PURPOSE:
!   Read parameters from input parameter file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call gasinter_ReadPar(parfile, ncpri, file, outstr,ierr)
!
! INPUT:
!   parfile:      ('parameter_file') character, name of parameter file
!   ncpri:        ('number_channel_print') integer, logical number for text output
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   file:         ('file') character, name of file for coefficients
!   outstr:       ('output_string') character, optional
!   ierr:         ('integer_error') integer, optional, contains exit error status
!                  0: correct operation
!                 >0: occurance of an error
!
! ROUTINES:
!   uio_closrd:            Close file after reading
!   uio_err:               Handle UIO errors
!   uio_filcon:            Determine file contents: list of all entries with its positions
!   uio_openrd:            Open file for reading, read header
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_srhd:              Search entry header in file or in entry list
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   uio_siz_module:        String length, table size for UIO
!   uio_nam_module:        Definition of names for UIO
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   A file is opened, read, and closed. Parameters are stored in saved global variable EOS.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Open requested file (in uio format).
!   Determine file contents.
!   Search each parameter, read it and put it into variable EOS.
!   Close file again.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-02-22 (B.F. Kiel) Written
!   1997-04-09 (B.F.) Correction: x1max/x2max
!   1997-04-10 (B.F.) Correction: abuX -> par%abuX
!   1997-04-16 (B.F.) Check existence of parameters -> choose default or stop with error message
!   1997-04-30 (B.F.) ifion in parameter file
!   1997-05-06 (B.F.) parameter file added
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: parfile
integer,                   intent(in)   :: ncpri
character(len=*),          intent(out)  :: file
character(len=*), optional,intent(out)  :: outstr
integer,          optional,intent(out)  :: ierr
!
! --- Local variables ---
integer                                 :: ncpar, ierr0, ntt
character(len=80)                       :: outstr0
character(len=let)                      :: termt(2,nttmx)
character(len=let)                      :: ident
integer, parameter                      :: nentmx=200
integer                                 :: nentry
integer                                 :: poslist(nentmx)
character(len=let)                      :: identlist(3,nentmx)
integer                                 :: iup(2)
!---------------------------------------------------------------------------------------------------
!
! --- Open parameter file for reading ---
call uio_openrd(ncpar, parfile)
!
write(ncpri,'(A,I3)') 'Parameter file ''' // trim(parfile) // &
                      ''' opened on channel ', ncpar
!
! --- Determine parameter file contents ---
call uio_filcon(ncpar, identlist,poslist,nentry)
!
!write(ncpri,*) 'nentry=', nentry
!write(ncpri,'(A)') identlist(:,:nentry)
!write(ncpri,'(I4)') poslist(:nentry)
!
! --- Read file_id ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='file_id', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%file_id,ident)
  if (EOS%file_id /= 'gas2eos-parameter') then
    outstr0='''' // trim(parfile) // ''' is not gas2eos parameter file'
    ierr0=1
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='gasinter_ReadPar',&
                 errlevel=3)
  endif
else
  outstr0='Type of ''' // trim(parfile) // ''' can not be determined'
  ierr0=2
  call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='gasinter_ReadPar', &
               errlevel=3)
endif
!
! --- Read description ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='description', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%description,ident,mode=2,iup=iup(1:1))
  EOS%ndes=iup(1)
else
  EOS%ndes=0
endif
write(ncpri,'(A)') &
  'Description: --------------------------------------------------------------'
write(ncpri,'(A)') EOS%description(:EOS%ndes)
!
! --- Read history ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='history', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%history,ident,mode=2,iup=iup(1:1))
  EOS%nhis=iup(1)
else
  EOS%nhis=0
endif
write(ncpri,'(A)') &
  'History: ------------------------------------------------------------------'
write(ncpri,'(A)') EOS%history(:EOS%nhis)
write(ncpri,'(A)') &
  '---------------------------------------------------------------------------'
!
! --- Read variable names ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='variables', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%variables,ident)
else
  EOS%variables='lnP,lnT->lnrho,h,s'
endif
write(ncpri,'(A)') 'Variables:         ' // trim(EOS%variables)
!
! --- Read output file name ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%file,ident)
else
  EOS%file='gas2eos_out.txt'
endif
write(ncpri,'(A)') 'Output filename:   ' // trim(EOS%file)
file=EOS%file
!
! --- Read output format ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='outform', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%form,ident)
else
  EOS%form='formatted'
endif
write(ncpri,'(A)') 'Output format:     ' // trim(EOS%form)
!
! --- Read output conversion type ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='outconv', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%conv,ident)
else
  EOS%conv='ieee_4'
endif
write(ncpri,'(A)') 'Output conversion: ' // trim(EOS%conv)
!
! --- Read number format ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='number_format', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%number_format,ident)
else
  EOS%conv=''
endif
write(ncpri,'(A)') 'Number format:     ' // trim(EOS%number_format)
!
! --- Source ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='source', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%source,ident)
else
  EOS%source=''
endif
write(ncpri,'(A)') 'Source:            ' // trim(EOS%source)
!
! --- Derivative mode ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='derivative_mode', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%derivative_mode,ident)
else
  EOS%derivative_mode=''
endif
write(ncpri,'(A)') 'Derivative_mode:   ' // trim(EOS%derivative_mode)
!
! --- Read hydrogen abundance ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='abux', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%abuX,ident)
else
  outstr0='hydrogen abundance is not specified'
  ierr0=3
  call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='gasinter_ReadPar', &
               errlevel=3)
endif
write(ncpri,'(A,F15.12)') 'abuX:  ', EOS%abuX
!
! --- Read helium abundance ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='abuy', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%abuY,ident)
else
  outstr0='helium abundance is not specified'
  ierr0=4
  call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='gasinter_ReadPar', &
               errlevel=3)
endif
write(ncpri,'(A,F15.12)') 'abuY:  ', EOS%abuY
!
! --- Compute metal abundance ---
EOS%abuZ=1.0-EOS%abuX-EOS%abuY
write(ncpri,'(A,F15.12)') 'abuZ:  ', EOS%abuZ
!
! --- Read mean molecular weight ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='qmol', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%Qmol,ident)
else
  outstr0='mean molecular weight is not specified'
  ierr0=5
  call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='gasinter_ReadPar', &
               errlevel=3)
endif
write(ncpri,'(A,E15.8)')  'Qmol:  ', EOS%Qmol
!
! --- Read ifion: handling of ionization ---
call uio_srhd(ncpar, termt,ntt, type=intena, ident='ifion', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%ifion,ident)
else
  EOS%ifion=1
endif
write(ncpri,'(A,I15)')    'ifion: ', EOS%ifion
!
! --- Read Gamma ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='gamma', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%Gamma,ident)
else
  EOS%Gamma=0.0
endif
write(ncpri,'(A,E15.8)')  'Gamma: ', EOS%Gamma
!
! --- Some default values ---
EOS%nx1=10
EOS%x1min=1.0E+03
EOS%x1max=1.0E+05
EOS%nx2=20
EOS%x2min=5.0E+03
EOS%x2max=2.0E+04
!
! --- Read table size: first dimension ---
call uio_srhd(ncpar, termt,ntt, type=intena, ident='nx1', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%nx1,ident)
endif
write(ncpri,'(A,I15)') 'nx1:   ', EOS%nx1
!
! --- Read lower boundary of table: first dimension ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='x1min', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%x1min,ident)
endif
write(ncpri,'(A,E15.8)') 'x1min: ', EOS%x1min
!
! --- Read upper boundary of table: first dimension ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='x1max', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%x1max,ident)
endif
write(ncpri,'(A,E15.8)') 'x1max: ', EOS%x1max
!
! --- Read table size: second dimension ---
call uio_srhd(ncpar, termt,ntt, type=intena, ident='nx2', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%nx2,ident)
endif
write(ncpri,'(A,I15)') 'nx2:   ', EOS%nx2
!
! --- Read lower boundary of table: second dimension ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='x2min', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%x2min,ident)
endif
write(ncpri,'(A,E15.8)') 'x2min: ', EOS%x2min
!
! --- Read upper boundary of table: second dimension ---
call uio_srhd(ncpar, termt,ntt, type=realna, ident='x2max', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, EOS%x2max,ident)
endif
write(ncpri,'(A,E15.8)') 'x2max: ', EOS%x2max
write(ncpri,*)
!
! --- Set number of independent variables ---
EOS%nx=2
!
! --- Close parameter file after reading ---
call uio_closrd(ncpar)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine gasinter_ReadPar


!----------***********************------------------------------------------------------------------
subroutine gasinter_AllDerivatives(mode, nx1, nx2, &
                                   x1, x2, &
                                   y1, &
                                   dy1dx1, dy1dx2, &
                                   ddy1dx1dx1, ddy1dx2dx2, ddy1dx1dx2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_AllDerivatives ('gas_interpolation_All_Derivatives')
!
! PURPOSE:
!   Compute numerically first and second derivatives for a function given by 2D array.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter_AllDerivatives(mode, nx1, nx2, &
!                                x1, x2, &
!                                y1, &
!                                dy1dx1, dy1dx2, &
!                                ddy1dx1dx1, ddy1dx2dx2, ddy1dx1dx2)
!
! INPUT:
!   mode:         ('mode') character, not used so far
!   nx1:          ('number_x_1') integer, vector length of first dimension
!   nx2:          ('number_x_2') integer, vector length of second dimension
!   x1:           ('x_1) real 2D array with first independent variable
!   x2:           ('x_2) real 2D array with second independent variable
!   y1:           ('y_1) real 2D array with first (and only) dependent variable
!
! OUTPUT:
!   dy1dx1:       ('delta_y_1_delta_x_1') real 2D array with first derivative
!   dy1dx2:       ('delta_y_1_delta_x_2') real 2D array with first derivative
!   ddy1dx1dx1:   ('delta_delta_y_1_delta_x_1_delta_x_1') real 2D array with second derivative
!   ddy1dx2dx2:   ('delta_delta_y_2_delta_x_1_delta_x_2') real 2D array with second derivative
!   ddy1dx1dx2:   ('delta_delta_y_1_delta_x_1_delta_x_2') real 2D array with cross derivative
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For finite-difference coefficients see e.g.
!   \href{http://en.wikipedia.org/wiki/Finite_difference_coefficient#Central_finite_difference}{Wikipedia}
!   \href{\detokenize{http://www.trentfguidry.net/post/2010/09/04/Numerical-differentiation-formulas.aspx}}{Trent Guidry}
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-09-09 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: mode
integer,                   intent(in)   :: nx1, nx2
real, dimension(nx1+1,nx2+1), &
                           intent(in)   :: x1, x2, &
                                           y1
real, dimension(nx1+1,nx2+1), &
                           intent(out)  :: dy1dx1, dy1dx2, &
                                           ddy1dx1dx1, ddy1dx2dx2, ddy1dx1dx2
!
! --- Local variables ---
integer                                 :: i1, i2
real                                    :: dx1, dx2
!---------------------------------------------------------------------------------------------------
!
! --- Compute dx1 and dx2 assuming equidistant grid ---
dx1=(x1(nx1+1,    1)-x1(1,1))/real(nx1)
dx2=(x2(1    ,nx2+1)-x2(1,1))/real(nx2)
!
! --- Compute first derivatives (bulk) ---
do i1=2,nx1
  do i2=1,nx2+1
    dy1dx1(i1,i2)=(y1(i1+1,i2  )-y1(i1-1,i2  )) / &
                  (x1(i1+1,i2  )-x1(i1-1,i2  ))
  end do ! i2
end do ! i1
do i1=1,nx1+1
  do i2=2,nx2
    dy1dx2(i1,i2)=(y1(i1  ,i2+1)-y1(i1  ,i2-1)) / &
                  (x2(i1  ,i2+1)-x2(i1  ,i2-1))
  end do ! i2
end do ! i1
!
! --- Compute second derivatives (bulk) ---
do i1=2,nx1
  do i2=1,nx2+1
    ddy1dx1dx1(i1,i2)=((y1(i1+1,i2  )-y1(i1  ,i2  )) / &
                       (x1(i1+1,i2  )-x1(i1  ,i2  ))     - &
                       (y1(i1  ,i2  )-y1(i1-1,i2  )) / &
                       (x1(i1  ,i2  )-x1(i1-1,i2  ))         ) / &
                      ((x1(i1+1,i2  )-x1(i1-1,i2  ))*0.5     )
  end do ! i2
end do ! i1
do i1=1,nx1+1
  do i2=2,nx2
    ddy1dx2dx2(i1,i2)=((y1(i1  ,i2+1)-y1(i1  ,i2  )) / &
                       (x2(i1  ,i2+1)-x2(i1  ,i2  ))     - &
                       (y1(i1  ,i2  )-y1(i1  ,i2-1)) / &
                       (x2(i1  ,i2  )-x2(i1  ,i2-1))         ) / &
                      ((x2(i1  ,i2+1)-x2(i1  ,i2-1))*0.5     )
  end do ! i2
end do ! i1
do i1=2,nx1
  do i2=2,nx2
    ddy1dx1dx2(i1,i2)=((y1(i1+1,i2+1)-y1(i1-1,i2+1)) / &
                       (x1(i1+1,i2+1)-x1(i1-1,i2+1))    - &
                       (y1(i1+1,i2-1)-y1(i1-1,i2-1)) / &
                       (x1(i1+1,i2-1)-x1(i1-1,i2-1))     ) / &
                      ( x2(i1  ,i2+1)-x2(i1  ,i2-1)      )
  end do ! i2
end do ! i1
!
! --- Recompute first derivatives with 5-point stencil (bulk) ---
do i1=3,nx1-3
  do i2=1,nx2+1
    dy1dx1(i1,i2)=(  1.0/12.0*y1(i1-2,i2  ) &
                   - 2.0/3.0 *y1(i1-1,i2  ) &
                   + 2.0/3.0 *y1(i1+1,i2  ) &
                   - 1.0/12.0*y1(i1+2,i2  )  ) / dx1
  end do ! i2
end do ! i1
do i1=1,nx1+1
  do i2=3,nx2-1
    dy1dx2(i1,i2)=(  1.0/12.0*y1(i1  ,i2-2) &
                   - 2.0/3.0 *y1(i1  ,i2-1) &
                   + 2.0/3.0 *y1(i1  ,i2+1) &
                   - 1.0/12.0*y1(i1  ,i2+2)  ) / dx2
  end do ! i2
end do ! i1
!
! --- Recompute second derivatives with 5-point stencil (bulk) ---
do i1=3,nx1-1
  do i2=1,nx2+1
    ddy1dx1dx1(i1,i2)=(- 1.0/12.0*y1(i1-2,i2  ) &
                       + 4.0/3.0 *y1(i1-1,i2  ) &
                       - 5.0/2.0 *y1(i1  ,i2  ) &
                       + 4.0/3.0 *y1(i1+1,i2  ) &
                       - 1.0/12.0*y1(i1+2,i2  )  ) / dx1**2
  end do ! i2
end do ! i1
do i1=1,nx1+1
  do i2=3,nx2-1
    ddy1dx2dx2(i1,i2)=(- 1.0/12.0*y1(i1  ,i2-2) &
                       + 4.0/3.0 *y1(i1  ,i2-1) &
                       - 5.0/2.0 *y1(i1  ,i2  ) &
                       + 4.0/3.0 *y1(i1  ,i2+1) &
                       - 1.0/12.0*y1(i1  ,i2+2)  ) / dx2**2
  end do ! i2
end do ! i1
do i1=3,nx1-1
  do i2=3,nx2-1
    ddy1dx1dx2(i1,i2)=(  1.0/12.0*dy1dx1(i1  ,i2-2) &
                       - 2.0/3.0 *dy1dx1(i1  ,i2-1) &
                       + 2.0/3.0 *dy1dx1(i1  ,i2+1) &
                       - 1.0/12.0*dy1dx1(i1  ,i2+2)  ) / dx2
  end do ! i2
end do ! i1
!
!!$! --- Extrapolate second derivatives at boundaries keeping them constant ---
!!$do i1=1,nx1+1
!!$  ddy1dx2dx2(i1,1    )=ddy1dx2dx2(i1,2  )
!!$  ddy1dx2dx2(i1,nx2+1)=ddy1dx2dx2(i1,nx2)
!!$end do ! i1
!!$do i1=2,nx1
!!$  ddy1dx1dx2(i1,1    )=ddy1dx1dx2(i1,2  )
!!$  ddy1dx1dx2(i1,nx2+1)=ddy1dx1dx2(i1,nx2)
!!$end do ! i1
!!$do i2=1,nx2+1
!!$  ddy1dx1dx1(1    ,i2)=ddy1dx1dx1(2  ,i2)
!!$  ddy1dx1dx2(1    ,i2)=ddy1dx1dx2(2  ,i2)
!!$  ddy1dx1dx1(nx1+1,i2)=ddy1dx1dx1(nx1,i2)
!!$  ddy1dx1dx2(nx1+1,i2)=ddy1dx1dx2(nx1,i2)
!!$end do ! i2
!
! --- Extrapolate second derivatives at boundaries linearly ---
do i1=1,nx1+1
  ddy1dx2dx2(i1,1    )=ddy1dx2dx2(i1,2    ) - &
                      (ddy1dx2dx2(i1,3    )-ddy1dx2dx2(i1,2    )) / &
                      (        x2(i1,3    )-        x2(i1,2    )) * &
                      (        x2(i1,2    )-        x2(i1,1    ))
  ddy1dx2dx2(i1,nx2+1)=ddy1dx2dx2(i1,nx2  ) + &
                      (ddy1dx2dx2(i1,nx2  )-ddy1dx2dx2(i1,nx2-1)) / &
                      (        x2(i1,nx2  )-        x2(i1,nx2-1)) * &
                      (        x2(i1,nx2+1)-        x2(i1,nx2  ))
end do ! i1
do i1=2,nx1
  ddy1dx1dx2(i1,1    )=ddy1dx1dx2(i1,2    ) - &
                      (ddy1dx1dx2(i1,3    )-ddy1dx1dx2(i1,2    )) / &
                      (        x2(i1,3    )-        x2(i1,2    )) * &
                      (        x2(i1,2    )-        x2(i1,1    ))
  ddy1dx1dx2(i1,nx2+1)=ddy1dx1dx2(i1,nx2  ) + &
                      (ddy1dx1dx2(i1,nx2  )-ddy1dx1dx2(i1,nx2-1)) / &
                      (        x2(i1,nx2  )-        x2(i1,nx2-1)) * &
                      (        x2(i1,nx2+1)-        x2(i1,nx2  ))
end do ! i1
do i2=1,nx2+1
  ddy1dx1dx1(1    ,i2)=ddy1dx1dx1(2    ,i2) - &
                      (ddy1dx1dx1(3    ,i2)-ddy1dx1dx1(2    ,i2)) / &
                      (        x1(3    ,i2)-        x1(2    ,i2)) * &
                      (        x1(2    ,i2)-        x1(1    ,i2))
  ddy1dx1dx2(1    ,i2)=ddy1dx1dx2(2    ,i2) - &
                      (ddy1dx1dx2(3    ,i2)-ddy1dx1dx2(2    ,i2)) / &
                      (        x1(3    ,i2)-        x1(2    ,i2)) * &
                      (        x1(2    ,i2)-        x1(1    ,i2))
  ddy1dx1dx1(nx1+1,i2)=ddy1dx1dx1(nx1  ,i2) + &
                      (ddy1dx1dx1(nx1  ,i2)-ddy1dx1dx1(nx1-1,i2)) / &
                      (        x1(nx1  ,i2)-        x1(nx1-1,i2)) * &
                      (        x1(nx1+1,i2)-        x1(nx1  ,i2))
  ddy1dx1dx2(nx1+1,i2)=ddy1dx1dx2(nx1  ,i2) + &
                      (ddy1dx1dx2(nx1  ,i2)-ddy1dx1dx2(nx1-1,i2)) / &
                      (        x1(nx1  ,i2)-        x1(nx1-1,i2)) * &
                      (        x1(nx1+1,i2)-        x1(nx1  ,i2))
end do ! i2
!
! --- Extrapolate first derivatives linearly at boundaries ---
do i2=1,nx2+1
  dy1dx1(1    ,i2)=dy1dx1(2  ,i2) - &
                   0.5*(ddy1dx1dx1(2    ,i2)+ddy1dx1dx1(1  ,i2  )) * &
                       (        x1(2    ,i2)-        x1(1  ,i2  ))
  dy1dx1(nx1+1,i2)=dy1dx1(nx1,i2) + &
                   0.5*(ddy1dx1dx1(nx1+1,i2)+ddy1dx1dx1(nx1,i2)) * &
                       (        x1(nx1+1,i2)-        x1(nx1,i2))
end do ! i1
do i1=1,nx1+1
  dy1dx2(i1,1    )=dy1dx2(i1,2  ) - &
                   0.5*(ddy1dx2dx2(i1,2    )+ddy1dx2dx2(i1,1  )) * &
                       (        x2(i1,2    )-        x2(i1,1  ))
  dy1dx2(i1,nx2+1)=dy1dx2(i1,nx2) + &
                   0.5*(ddy1dx2dx2(i1,nx2+1)+ddy1dx2dx2(i1,nx2)) * &
                       (        x2(i1,nx2+1)-        x2(i1,nx2))
end do ! i1
!
end subroutine gasinter_AllDerivatives


!----------******************-----------------------------------------------------------------------
subroutine gasinter_ModifyGas(mode, nx1, nx2, x2_shift, &
                              rho, e, s, &
                              P, T, dsdlnrho, dsdlne, &
                              dlnPdlnrho, dlnPdlne, dlnTdlnrho, dlnTdlne)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_ModifyGas ('gas_interpolation_Modify_Gas')
!
! PURPOSE:
!   Modify precomputed set of EOS quantities.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter_ModifyGas(mode, nx1, nx2, x2_shift, &
!                           rho, e, s, &
!                           P, T, dsdlnrho, dsdlne, &
!                           dlnPdlnrho, dlnPdlne, dlnTdlnrho, dlnTdlne)
!
! INPUT:
!   mode:         ('mode') character, not used so far
!   nx1:          ('number_x_1') integer, vector length of first independent variable
!   nx2:          ('number_x_2') integer, vector length of second independent variable
!   x2_shift:     ('x_2_shift') real, shift value for x2 axis
!   rho:          ('rho') real 2D array with density
!   e:            ('energy') real 2D array with internal energy
!   s:            ('s') real 2D array with entropy
!
! OUTPUT:
!   P:            ('Pressure') real 2D array with pressure
!   T:            ('Temperature') real 2D array with temperature
!   dsdlnrho:     ('delta_s_delta_log_normal_rho') real 2D array with entropy derivative
!   dsdlne:       ('delta_s_delta_log_normal_energy') real 2D array with entropy derivative
!   dlnPdlnrho:   ('delta_log_normal_Pressure_delta_log_normal_rho') real 2D array with press. deriv.
!   dlnPdlne:     ('delta_log_normal_Pressure_delta_log_normal_energy') real 2D array with p. deriv.
!   dlnTdlnrho:   ('delta_log_normal_Temperature_delta_log_normal_rho') real 2D array with t. deriv.
!   dlnTdlne:     ('delta_log_normal_Temperature_delta_log_normal_energy') real 2D array with t. der.
!
! ROUTINES:
!   gasinter_AllDerivatives: Compute numerically 1st and 2nd derivatives for 2D array function
!   uio_closwr:            Close file after writing
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!
! MODULES:
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-09-09 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use uio_bulk_module
use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: mode
integer,                   intent(in)   :: nx1, nx2
real,                      intent(in)   :: x2_shift
real, dimension(nx1+1,nx2+1), &
                           intent(in)   :: rho, e, s
!real, dimension(nx1+1,nx2+1), &
!                           intent(out)  :: P, T, cpd, Gamma1, delta, h, cs
real, dimension(nx1+1,nx2+1), &
                           intent(out)  :: P, T, dsdlnrho, dsdlne, &
                                           dlnPdlnrho, dlnPdlne, dlnTdlnrho, dlnTdlne
!
! --- Local variables ---
integer                                 :: i1, i2, nc
real, dimension(nx1+1,nx2+1)            :: ddsdlnrhodlnrho, ddsdlnedlne, ddsdlnrhodlne
!---------------------------------------------------------------------------------------------------
!
! --- Compute first and second derivatives of entropy ---
call gasinter_AllDerivatives(mode, nx1, nx2, &
                             x1        =alog(rho), &
                             x2        =alog(e+x2_shift), &
                             y1        =s, &
                             dy1dx1    =dsdlnrho, &
                             dy1dx2    =dsdlne, &
                             ddy1dx1dx1=ddsdlnrhodlnrho, &
                             ddy1dx2dx2=ddsdlnedlne, &
                             ddy1dx1dx2=ddsdlnrhodlne)
!
! --- Compute quantities from first derivatives ---
T=(e+x2_shift)/dsdlne
P=-T*rho*dsdlnrho
!h=e+P/rho
!
! --- Compute derivatives of T and P ---
!dTdlne  =(e+x2_shift)/dsdlne * (1.0-ddsdlnedlne/dsdlne)
!dTdlnrho=-(e+x2_shift)/dsdlne**2*ddsdlnrhodlne
!dlnPdlne  =rho/P*(-dTdlne*dsdlnrho - T*ddsdlnrhodlne)
!dPdlnrho=-dTdlnrho*rho*dsdlnrho - T*rho*dsdlnrho - T*rho*ddsdlnrhodlnrho
dlnTdlne  =1.0-ddsdlnedlne/dsdlne
dlnTdlnrho=-ddsdlnrhodlne/dsdlne
dlnPdlne  =dlnTdlne         + ddsdlnrhodlne/dsdlnrho
dlnPdlnrho=dlnTdlnrho + 1.0 + ddsdlnrhodlnrho/dsdlnrho
!
!Gamma1=1.0/P*dPdlnrho + 1.0/(rho*(e+x2_shift))*dPdlne
!cs    =sqrt(Gamma1*P/rho)
!
call uio_openwr(nc, 'gasinter_test.dat', form='unformatted', conv='ieee_4',position='append')
call uio_wrlabl(nc, dataset_ident, date='now' )
call uio_wrlabl(nc, box_ident                 )
call uio_wr(nc, 'z'               , box_id_ident        )
call uio_wr(nc, x2_shift          , 'x2_shift'          )
call uio_wr(nc, e                 , 'e'                 )
call uio_wr(nc, rho               , 'rho'               )
call uio_wr(nc, T                 , 't'                 )
call uio_wr(nc, p                 , 'p'                 )
call uio_wr(nc, s                 , 's'                 )
call uio_wr(nc, dsdlnrho          , 'dsdlnrho'          )
call uio_wr(nc, dsdlne            , 'dsdlne'            )
call uio_wr(nc, ddsdlnrhodlnrho   , 'ddsdlnrhodlnrho'   )
call uio_wr(nc, ddsdlnedlne       , 'ddsdlnedlne'       )
call uio_wr(nc, ddsdlnrhodlne     , 'ddsdlnrhodlne'     )
call uio_wr(nc, dlnTdlne          , 'dlntdlne'          )
call uio_wr(nc, dlnTdlnrho        , 'dlntdlnrho'        )
call uio_wr(nc, dlnPdlne          , 'dlnpdlne'          )
call uio_wr(nc, dlnPdlnrho        , 'dlnpdlnrho'        )
call uio_wrlabl(nc, endbox_ident)
call uio_wrlabl(nc, enddataset_ident)
call flush(nc)
call uio_closrd(nc)
!
end subroutine gasinter_ModifyGas


!----------*****************------------------------------------------------------------------------
subroutine gasinter_SetUpGas()
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_SetUpGas ('gas_interpolation_Set_Up_Gas')
!
! PURPOSE:
!   Compute coefficients for later bicubic interpolation of GAS quantities.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter_SetUpGas()
!
! INPUT:
!   None (Parameters are used from saved global variable.)
!
! OUTPUT:
!   None (Coefficients are in saved global variable.)
!
! ROUTINES:
!   bicubic_coeff:         Compute bicubic interpolation coefficients from tabulated values
!   gas:                   EOS routine of RHD-code (or gasgas)
!   gasinv:                Inversion of GAS routine: rho/e -> P,T,...
!   uio_dattim:            Get string with current date and time
!
! MODULES:
!   bicubic_module:        Bicubic interpolation
!   tabinter_coeff_module: Interpolation coefficients
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   The current EOS table with interpolation coefficients and additional information is changed.
!
! RESTRICTIONS:
!   The results of the interpolation can only be as good as the underlying EOS equation.
!   An interpolation table has to be choosen and filled properly before calling the routine.
!
! PROCEDURE:
!   According to input parameters a P/T or a rho/e grid is generated.
!   For each pair the EOS is solved by calling GAS or GASINV -> h, s, Gamma1,...
!   Some first derivatives of quantities to be tabulated are computed by thermodynamic relations.
!   Cross derivatives are computed by numerical differentiation with
!   an appropriate handling of boundaries.
!   Coefficients for later bicubic interpolation are computed by bicubic_coeff.
!
! EXAMPLE:
!   ! --- Select and clear table 1 ---
!   call gasinter_reset(1)
!   ! --- Specify parameters ---
!   ...
!   ! --- Set up interpolation coefficients according to specified parameters --
!   call gasinter_SetUpGas()
!
! MODIFICATION HISTORY:
!   1996-03-19 (B.F. Kiel) Written
!   1996-03-21 (B.F.) Boundary values for cross derivatives
!   1996-03-23 (B.F.) T -> ln(T)
!   1997-04-07 (B.F.) ncP_i, ... as input parameters
!   1997-04-10 (B.F.) C(:,:,i,:,:) -> C(:,:,i,1:ncP,1:ncT)
!   1997-04-14 (B.F.) More comments
!   1997-04-28 (B.F.) Global variable par, less input parameters
!   1997-05-02 (B.F.) lnrho->y1...
!   1997-05-06 (B.F.) Correction of dP/drho|e
!   1997-05-07 (B.F.) dP/drho -> dlnP/dlnrho, dT/drho -> dlnT/dlnrho
!   1997-05-09 (B.F.) shift for e added
!   1997-05-13 (B.F.) from high to low energy
!   1997-05-14 (B.F.) Some arrays in par are pointers. They are allocated.
!   1997-05-16 (B.F.) EOS%ny, EOS%nC added
!   1997-05-21 (B.F.) EOS%x_name, EOS%x_unit allocated
!   1997-05-22 (B.F.) Boundary values of cross derivatives computed differently
!   1997-09-11 (B.F.) After big step in P/T initialize GASGAG by setting kenter=0
!   1999-04-23 (B.F.) recompute cv0 after ix1-loop
!   2004-02-25 (B.F.) Call GASGAS instead of GAS (was implicitely done before)
!   2004-02-26 (B.F.) supergas
!---------------------------------------------------------------------------------------------------
use bicubic_module
use tabinter_coeff_module
use uio_bulk_module
use gas_module
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
real, dimension(EOS%nx1+1)              :: x11d
real, dimension(EOS%nx2+1)              :: x21d
real, dimension(EOS%nx1+1,EOS%nx2+1)    :: P, T, rho, cpd, Gamma1, delta, &
                                           h, s, cs, e, &
                                           x1, x2, y1, y2, y3
real, dimension(EOS%nx1+1,EOS%nx2+1)    :: dy1dx1, dy1dx2, ddy1dx1dx2, &
                                           dy2dx1, dy2dx2, ddy2dx1dx2, &
                                           dy3dx1, dy3dx2, ddy3dx1dx2
real, dimension(1:4, 1:4, &
                EOS%nx1  ,EOS%nx2  )    :: Ctmp
integer                                 :: ierror, ifent, kenter, ix1, ix2
character(len=24)                       :: time
real                                    :: P0, T0, cv0, &
                                           cpd0, Gamma10, delta0, h0, s0, cs0,&
                                           T_rhomin_emin, T_rhomax_emin, &
                                           T_rhomin_emax, T_rhomax_emax, &
                                           Tmin, Tmax
!---------------------------------------------------------------------------------------------------
!
EOS%nx=2
EOS%ny=3
EOS%nC=16
!
! --- Allocate arrays for coefficients and additional information ---
allocate(EOS%x_name(EOS%nx))
allocate(EOS%x_unit(EOS%nx))
allocate(EOS%lnx11d(1:EOS%nx1+1), EOS%lnx21d(1:EOS%nx2+1))
allocate(EOS%C_name(EOS%ny), EOS%C_unit(EOS%ny), EOS%C_shift(EOS%ny), &
         EOS%C_offset(EOS%ny))
allocate(EOS%C(1:EOS%nC,1:EOS%ny,1:EOS%nx1,1:EOS%nx2))
!
! --- Initialize GAS routine by calling it with kenter=-1 ---
kenter=-1
ifent=0
call supergas(8000.0, 16000.0, &
        rho(1,1), cpd(1,1), Gamma1(1,1), delta(1,1), h(1,1), s(1,1), cs(1,1), &
        ierror, EOS%ifion, ifent, kenter, EOS%abuX, EOS%abuY, EOS%Qmol)
!
! --- Shifts ---
!
! --- All shifts (and offsets) are still zero ---
EOS%x1_shift =0.0
EOS%x2_shift =0.0
EOS%C_shift  =0.0
EOS%x1_offset=0.0
EOS%x2_offset=0.0
EOS%C_offset =0.0
!
if ((EOS%variables == 'lnrho,lne->s,lnP,lnT') .and. (EOS%x2min <= 0.0)) then
  ! --- Problem: Determine shift for e ---
  ! --- (x2max+x2_shift)/(x2min+x2_shift) = Tmax/Tmin  ->  x2_shift ---
  !
  kenter=0
  call supergasinv(EOS%x1max, EOS%x2max, P0, T_rhomax_emax, &
              cpd0, Gamma10, delta0, h0, s0, cs0, &
              ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
  kenter=0
  call supergasinv(EOS%x1min, EOS%x2min, P0, T_rhomin_emin, &
              cpd0, Gamma10, delta0, h0, s0, cs0, &
              ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
!!$  kenter=0
!!$  call gasinv(EOS%x1max, EOS%x2min, P0, T_rhomax_emin, &
!!$              cpd0, Gamma10, delta0, h0, s0, cs0, &
!!$              ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
!!$  kenter=0
!!$  call gasinv(EOS%x1min, EOS%x2max, P0, T_rhomin_emax, &
!!$              cpd0, Gamma10, delta0, h0, s0, cs0, &
!!$              ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
!!$  Tmin=min(T_rhomin_emin, T_rhomax_emin, T_rhomin_emax, T_rhomax_emax)
!!$  Tmax=max(T_rhomin_emin, T_rhomax_emin, T_rhomin_emax, T_rhomax_emax)
  Tmin=min(T_rhomin_emin, T_rhomax_emax)
  Tmax=max(T_rhomin_emin, T_rhomax_emax)
  !
  EOS%x2_shift=(Tmin*EOS%x2max - Tmax*EOS%x2min) / (Tmax - Tmin)
  write(6,'(A,E15.8)') 'EOS%x2_shift=', EOS%x2_shift
  !
endif
!
! --- Compute offsets, ... ---
EOS%lnx1off=alog(EOS%x1min+EOS%x1_shift)
EOS%lnx1fac=real(EOS%nx1)/ &
            (alog(EOS%x1max+EOS%x1_shift)-alog(EOS%x1min+EOS%x1_shift))
EOS%lnx11d(1:EOS%nx1+1)=EOS%lnx1off + (/(ix1,ix1=0,EOS%nx1)/)/EOS%lnx1fac
x11d=exp(EOS%lnx11d(1:EOS%nx1+1))-EOS%x1_shift
x1=spread(x11d,2,EOS%nx2+1)
!
EOS%lnx2off=alog(EOS%x2min+EOS%x2_shift)
EOS%lnx2fac=real(EOS%nx2)/ &
            (alog(EOS%x2max+EOS%x2_shift)-alog(EOS%x2min+EOS%x2_shift))
EOS%lnx21d(1:EOS%nx2+1)=EOS%lnx2off + (/(ix2,ix2=0,EOS%nx2)/)/EOS%lnx2fac
x21d=exp(EOS%lnx21d(1:EOS%nx2+1))-EOS%x2_shift
x2=spread(x21d,1,EOS%nx1+1)
!
if (EOS%variables == 'lnP,lnT->lnrho,h,s') then
  ! --- Put infos into EOS ---
  ! --- x1=P, x2=T ---
  ! --- y1=lnrho, y2=h, y3=s ---
  ! --- Give description ---
  EOS%description(1)='Coefficients for bicubic interpolation of EOS quantities.'
  EOS%description(2)='lnrho, h, s as function of lnP, lnT.'
  EOS%description(3)='Source: GASGAS_23jan98 (supergas).'
  EOS%ndes=3
  !
  ! --- Expand history ---
  EOS%nhis=min(EOS%nhis,nhismax)
  EOS%history(1:EOS%nhis)=EOS%history(1:EOS%nhis)
  if (EOS%nhis < nhismax) then
    call uio_dattim(time)
    EOS%nhis=EOS%nhis+1
    EOS%history(EOS%nhis)='gas2eos: ' // time
  endif
  !
  ! --- Data type ---
  EOS%interpolation='bicubic'
  EOS%source='gasgas'
  !  
  ! --- 1D vectors ---
  EOS%x_name(1)='P'                 ; EOS%x_unit(1)='dyn/cm^2'
  EOS%x_name(2)='T'                 ; EOS%x_unit(2)='K'
  !
  ! --- Coefficients ---
  EOS%C_name(1)='rho'               ; EOS%C_unit(1)='g/cm^3'
  EOS%C_name(2)='h'                 ; EOS%C_unit(2)='erg/g'
  EOS%C_name(3)='s'                 ; EOS%C_unit(3)='erg/K/g'
  !
  ! --- Fill P and T array ---
  P=x1
  T=x2
  !
  ! --- Solve EOS by calling GAS for each P/T combination ---
  do ix2=1,EOS%nx2+1
    do ix1=1,EOS%nx1+1
      kenter=0
      ifent=1
      call supergas(P(ix1,ix2),   T(ix1,ix2), &
               rho(ix1,ix2), cpd(ix1,ix2), Gamma1(ix1,ix2),   delta(ix1,ix2), &
               h(ix1,ix2),   s(ix1,ix2),   cs(ix1,ix2), &
               ierror, EOS%ifion, ifent, kenter, EOS%abuX, EOS%abuY, EOS%Qmol)
    end do ! ix1
  end do ! ix2
  !
  ! --- Tabulated quantities ---
  y1=alog(rho)
  y2=h
  y3=s
  !
  ! --- Compute first derivatives of quantities to be tabulated ---
  ! --- dlnrhodlnP ---
  dy1dx1=1.0/Gamma1 + delta*P/(T*rho*cpd)
  ! --- dlnrhodlnT ---
  dy1dx2=-delta
  ! --- dhdlnP ---
  dy2dx1=(1.0-delta)*P/rho
  ! --- dhdlnT ---
  dy2dx2=cpd*delta*T
  ! --- dsdlnP ---
  dy3dx1=-(delta*P)/(rho*T)
  ! --- dsdlnT ---
  dy3dx2=cpd*delta
  !
  ! --- Some logarithms ---
  x1=alog(P)
  x2=alog(T)
else if (EOS%variables == 'lnrho,lne->s,lnP,lnT') then
  ! --- Put infos into EOS ---
  ! --- x1=rho, x2=e(+shift) ---
  ! --- y1=s, y2=lnP, y3=lnT ---
  ! --- Give description ---
  EOS%description(1)='Coefficients for bicubic interpolation of EOS quantities.'
  EOS%description(2)='s, lnP, lnT as function of lnrho, lne.'
  EOS%description(3)='e shifted to positiv values.'
  EOS%description(4)='Source: Inversion of GASGAS by GASINV (supergas).'
  EOS%ndes=4
  !
  ! --- Expand history ---
  EOS%nhis=min(EOS%nhis,nhismax)
  EOS%history(1:EOS%nhis)=EOS%history(1:EOS%nhis)
  if (EOS%nhis < nhismax) then
    call uio_dattim(time)
    EOS%nhis=EOS%nhis+1
    EOS%history(EOS%nhis)='gas2eos: ' // time
  endif
  !
  ! --- Data type ---
  EOS%interpolation='bicubic'
  EOS%source='gasgas'
  !
  ! --- 1D vectors ---
  EOS%x_name(1)='rho'               ; EOS%x_unit(1)='g/cm^3'
  EOS%x_name(2)='e'                 ; EOS%x_unit(2)='erg/g'
  !
  ! --- Coefficients ---
  EOS%C_name(1)='s'                 ; EOS%C_unit(1)='erg/K/g'
  EOS%C_name(2)='P'                 ; EOS%C_unit(2)='dyn/cm^2'
  EOS%C_name(3)='T'                 ; EOS%C_unit(3)='K'
  !
  ! --- Fill rho and e array ---
  rho=x1
  e=x2
  !
  ! --- Solve EOS by calling GASINV for each rho/e combination ---
  ifent=1
  kenter=0
  !
!  do ix2=1,EOS%nx2+1
!    do ix1=1,EOS%nx1+1
!      call gasinv(rho(ix1,ix2), e(ix1,ix2), P0, T0, &
!                  cpd(ix1,ix2), Gamma1(ix1,ix2), delta(ix1,ix2), &
!                  h(ix1,ix2),   s(ix1,ix2),      cs(ix1,ix2), &
!                  ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
!      P(ix1,ix2)=P0
!      T(ix1,ix2)=T0
!      cv0=(cpd(ix1,ix2)*delta(ix1,ix2))/(1.0+ &
!                                (Gamma1(ix1,ix2)*delta(ix1,ix2)*P(ix1,ix2))/ &
!                                 (T(ix1,ix2)*rho(ix1,ix2)*cpd(ix1,ix2)))
!      if (ix1 <= EOS%nx1) then
!        kenter=1
!        P0=P(ix1,ix2) * (rho(ix1+1,ix2)/rho(ix1,ix2)) ** ( &
!            Gamma1(ix1,ix2) * &
!            (1.0-P(ix1,ix2)/(rho(ix1,ix2)*cpd(ix1,ix2)*T(ix1,ix2))) )
!        T0=T(ix1,ix2) * (rho(ix1+1,ix2)/rho(ix1,ix2)) ** ( &
!            -P(ix1,ix2)/(rho(ix1,ix2)*T(ix1,ix2)) * &
!           (1.0/cv0-Gamma1(ix1,ix2)/cpd(ix1,ix2)) )
!      endif
!    end do ! ix1
!    if (ix2 <= EOS%nx2) then
!      kenter=1
!      P0=P(1,ix2) + P(1,ix2)*Gamma1(1,ix2)/(T(1,ix2)*cpd(1,ix2)) * &
!                                                          (e(1,ix2+1)-e(1,ix2))
!      T0=T(1,ix2) + 1.0/cv0 *                             (e(1,ix2+1)-e(1,ix2))
!    endif
!  end do ! ix2
  !
  do ix2=EOS%nx2+1,1,-1
    do ix1=EOS%nx1+1,1,-1
      call supergasinv(rho(ix1,ix2), e(ix1,ix2), P0, T0, &
                  cpd(ix1,ix2), Gamma1(ix1,ix2), delta(ix1,ix2), &
                  h(ix1,ix2),   s(ix1,ix2),      cs(ix1,ix2), &
                  ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
      write(6,'(A,2I4)') 'ierror, kenter=', ierror, kenter
      P(ix1,ix2)=P0
      T(ix1,ix2)=T0
      cv0=(cpd(ix1,ix2)*delta(ix1,ix2))/(1.0+ &
                                (Gamma1(ix1,ix2)*delta(ix1,ix2)*P(ix1,ix2))/ &
                                 (T(ix1,ix2)*rho(ix1,ix2)*cpd(ix1,ix2)))
      if ((ix1 > 1) .and. (ierror .eq. 0)) then
        kenter=1
        P0=P(ix1,ix2) * (rho(ix1-1,ix2)/rho(ix1,ix2)) ** ( &
            Gamma1(ix1,ix2) * &
            (1.0-P(ix1,ix2)/(rho(ix1,ix2)*cpd(ix1,ix2)*T(ix1,ix2))) )
        T0=T(ix1,ix2) * (rho(ix1-1,ix2)/rho(ix1,ix2)) ** ( &
            -P(ix1,ix2)/(rho(ix1,ix2)*T(ix1,ix2)) * &
            (1.0/cv0-Gamma1(ix1,ix2)/cpd(ix1,ix2)) )
      else
        kenter=0
      endif
    end do ! ix1
    if ((ix2 > 1) .and. (ierror .eq. 0)) then
!     --- 11.09.1997 ---
!     --- Make big step in P/T -> set kenter=0 to initialize GASGAS properly ---
!      kenter=0
!     --- 23.04.1999 ---
      kenter=1
      cv0=(cpd(EOS%nx1+1,ix2)*delta(EOS%nx1+1,ix2))/(1.0+ &
                (Gamma1(EOS%nx1+1,ix2)*delta(EOS%nx1+1,ix2)*P(EOS%nx1+1,ix2))/ &
                (T(EOS%nx1+1,ix2)*rho(EOS%nx1+1,ix2)*cpd(EOS%nx1+1,ix2)))
      P0=P(EOS%nx1+1,ix2) + &
                            P(EOS%nx1+1,ix2)*Gamma1(EOS%nx1+1,ix2)/ &
                             (T(EOS%nx1+1,ix2)*cpd(EOS%nx1+1,ix2)) * &
                                          (e(EOS%nx1+1,ix2-1)-e(EOS%nx1+1,ix2))
      T0=T(EOS%nx1+1,ix2) + 1.0/cv0 *     (e(EOS%nx1+1,ix2-1)-e(EOS%nx1+1,ix2))
    else
      kenter=0
    endif
  end do ! ix2
  !
  ! --- Tabulated quantities ---
  x1=alog(rho)
  x2=alog(e+EOS%x2_shift)
  y1=s
  !
  if (EOS%derivative_mode == 's_numerical1') then
    ! --- Use only entropy and its numerical derivatives for rho end e ---
    !
    call gasinter_ModifyGas(EOS%derivative_mode, EOS%nx1, EOS%nx2, EOS%x2_shift, &
                            rho=rho, e=e, s=s, &
                            P=P, T=T, &
                            dsdlnrho  =dy1dx1, &
                            dsdlne    =dy1dx2, &
                            dlnPdlnrho=dy2dx1, &
                            dlnPdlne  =dy2dx2, &
                            dlnTdlnrho=dy3dx1, &
                            dlnTdlne  =dy3dx2   ) 
    !
  else ! (EOS%derivative_mode)
    ! --- Use original values from supergasinv for all thermodynamic quantities ---
    !
    ! --- Compute first derivatives of quantities to be tabulated ---
    ! --- Shift in e taken into account ---
    ! --- dsdlnrho ---
    dy1dx1=-P/(T*rho)
    ! --- dsdlne ---
    dy1dx2=(e+EOS%x2_shift)/T
    ! --- dlnPdlnrho ---
    dy2dx1=Gamma1*(1.0-P/(rho*cpd*T))
    ! --- dlnPdlne ---
    dy2dx2=(e+EOS%x2_shift)*Gamma1/(T*cpd)
    ! --- dlnTdlnrho ---
    dy3dx1=-P/(rho*T)*( (1.0+Gamma1*delta*P/(T*rho*cpd))/(cpd*delta)-Gamma1/cpd )
    ! --- dlnTdlne ---
    dy3dx2=(e+EOS%x2_shift)*(1+Gamma1*delta*P/(T*rho*cpd))/(T*cpd*delta)
    !
  endif ! (EOS%derivative_mode)
  !
  y2=alog(P)
  y3=alog(T)
  !
else ! (EOS%variables)
  stop 'Unknown interpolation mode'
  !
endif ! (EOS%variables)
!
! --- Compute cross derivatives ---
ddy1dx1dx2(2:EOS%nx1,  2:EOS%nx2  )= 0.5*( &
   (dy1dx1(2:EOS%nx1  ,3:EOS%nx2+1)-dy1dx1(2:EOS%nx1  ,1:EOS%nx2-1))/  &
       (x2(2:EOS%nx1  ,3:EOS%nx2+1)-    x2(2:EOS%nx1  ,1:EOS%nx2-1)) + &
   (dy1dx2(3:EOS%nx1+1,2:EOS%nx2  )-dy1dx2(1:EOS%nx1-1,2:EOS%nx2  ))/  &
       (x1(3:EOS%nx1+1,2:EOS%nx2  )-    x1(1:EOS%nx1-1,2:EOS%nx2  ))  )
ddy2dx1dx2(2:EOS%nx1,  2:EOS%nx2  )= 0.5*( &
   (dy2dx1(2:EOS%nx1  ,3:EOS%nx2+1)-dy2dx1(2:EOS%nx1  ,1:EOS%nx2-1))/  &
       (x2(2:EOS%nx1  ,3:EOS%nx2+1)-    x2(2:EOS%nx1  ,1:EOS%nx2-1)) + &
   (dy2dx2(3:EOS%nx1+1,2:EOS%nx2  )-dy2dx2(1:EOS%nx1-1,2:EOS%nx2  ))/  &
       (x1(3:EOS%nx1+1,2:EOS%nx2  )-    x1(1:EOS%nx1-1,2:EOS%nx2  ))  )
ddy3dx1dx2(2:EOS%nx1,  2:EOS%nx2  )= 0.5*( &
   (dy3dx1(2:EOS%nx1  ,3:EOS%nx2+1)-dy3dx1(2:EOS%nx1  ,1:EOS%nx2-1))/  &
       (x2(2:EOS%nx1  ,3:EOS%nx2+1)-    x2(2:EOS%nx1  ,1:EOS%nx2-1)) + &
   (dy3dx2(3:EOS%nx1+1,2:EOS%nx2  )-dy3dx2(1:EOS%nx1-1,2:EOS%nx2  ))/  &
       (x1(3:EOS%nx1+1,2:EOS%nx2  )-    x1(1:EOS%nx1-1,2:EOS%nx2  ))  )
!
! --- Boundary values of cross derivatives ---
!ddy1dx1dx2(2:EOS%nx1,1          )=ddy1dx1dx2(2:EOS%nx1,2          )
!ddy1dx1dx2(2:EOS%nx1,EOS%nx2+1  )=ddy1dx1dx2(2:EOS%nx1,EOS%nx2    )
!ddy1dx1dx2(1        ,1:EOS%nx2+1)=ddy1dx1dx2(2        ,1:EOS%nx2+1)
!ddy1dx1dx2(EOS%nx1+1,1:EOS%nx2+1)=ddy1dx1dx2(EOS%nx1  ,1:EOS%nx2+1)
!ddy2dx1dx2(2:EOS%nx1,1          )=ddy2dx1dx2(2:EOS%nx1,2          )
!ddy2dx1dx2(2:EOS%nx1,EOS%nx2+1  )=ddy2dx1dx2(2:EOS%nx1,EOS%nx2    )
!ddy2dx1dx2(1        ,1:EOS%nx2+1)=ddy2dx1dx2(2        ,1:EOS%nx2+1)
!ddy2dx1dx2(EOS%nx1+1,1:EOS%nx2+1)=ddy2dx1dx2(EOS%nx1  ,1:EOS%nx2+1)
!ddy3dx1dx2(2:EOS%nx1,1          )=ddy3dx1dx2(2:EOS%nx1,2          )
!ddy3dx1dx2(2:EOS%nx1,EOS%nx2+1  )=ddy3dx1dx2(2:EOS%nx1,EOS%nx2    )
!ddy3dx1dx2(1        ,1:EOS%nx2+1)=ddy3dx1dx2(2        ,1:EOS%nx2+1)
!ddy3dx1dx2(EOS%nx1+1,1:EOS%nx2+1)=ddy3dx1dx2(EOS%nx1  ,1:EOS%nx2+1)
!
ddy1dx1dx2(2:EOS%nx1  ,1          )= &
   (dy1dx2(3:EOS%nx1+1,1          )-dy1dx2(1:EOS%nx1-1,1          ))/  &
       (x1(3:EOS%nx1+1,1          )-    x1(1:EOS%nx1-1,1          ))
ddy1dx1dx2(2:EOS%nx1  ,EOS%nx2+1  )= &
   (dy1dx2(3:EOS%nx1+1,EOS%nx2+1  )-dy1dx2(1:EOS%nx1-1,EOS%nx2+1  ))/  &
       (x1(3:EOS%nx1+1,EOS%nx2+1  )-    x1(1:EOS%nx1-1,EOS%nx2+1  ))
ddy1dx1dx2(1          ,1:EOS%nx2+1)= &
   (dy1dx1(1          ,3:EOS%nx2+1)-dy1dx1(1          ,1:EOS%nx2-1))/  &
       (x2(1          ,3:EOS%nx2+1)-    x2(1          ,1:EOS%nx2-1))
ddy1dx1dx2(EOS%nx1+1  ,1:EOS%nx2+1)= &
   (dy1dx1(EOS%nx1+1  ,3:EOS%nx2+1)-dy1dx1(EOS%nx1+1  ,1:EOS%nx2-1))/  &
       (x2(EOS%nx1+1  ,3:EOS%nx2+1)-    x2(EOS%nx1+1  ,1:EOS%nx2-1))
ddy1dx1dx2(1        ,1        )= &
   0.5*(ddy1dx1dx2(1        ,2        )+ddy1dx1dx2(2        ,1        ))
ddy1dx1dx2(1        ,EOS%nx2+1)= &
   0.5*(ddy1dx1dx2(1        ,EOS%nx2  )+ddy1dx1dx2(2        ,EOS%nx2+1))
ddy1dx1dx2(EOS%nx1+1,1        )= &
   0.5*(ddy1dx1dx2(EOS%nx1+1,2        )+ddy1dx1dx2(EOS%nx1  ,1        ))
ddy1dx1dx2(EOS%nx1+1,EOS%nx2+1)= &
   0.5*(ddy1dx1dx2(EOS%nx1+1,EOS%nx2  )+ddy1dx1dx2(EOS%nx1  ,EOS%nx2+1))
!
ddy2dx1dx2(2:EOS%nx1  ,1          )= &
     (dy2dx2(3:EOS%nx1+1,1          )-dy2dx2(1:EOS%nx1-1,1          ))/  &
       (x1(3:EOS%nx1+1,1          )-    x1(1:EOS%nx1-1,1          ))
ddy2dx1dx2(2:EOS%nx1  ,EOS%nx2+1  )= &
     (dy2dx2(3:EOS%nx1+1,EOS%nx2+1  )-dy2dx2(1:EOS%nx1-1,EOS%nx2+1  ))/  &
       (x1(3:EOS%nx1+1,EOS%nx2+1  )-    x1(1:EOS%nx1-1,EOS%nx2+1  ))
ddy2dx1dx2(1          ,1:EOS%nx2+1)= &
   (dy2dx1(1          ,3:EOS%nx2+1)-dy2dx1(1          ,1:EOS%nx2-1))/  &
   (x2(1          ,3:EOS%nx2+1)-    x2(1          ,1:EOS%nx2-1))
ddy2dx1dx2(EOS%nx1+1  ,1:EOS%nx2+1)= &
   (dy2dx1(EOS%nx1+1  ,3:EOS%nx2+1)-dy2dx1(EOS%nx1+1  ,1:EOS%nx2-1))/  &
       (x2(EOS%nx1+1  ,3:EOS%nx2+1)-    x2(EOS%nx1+1  ,1:EOS%nx2-1))
ddy2dx1dx2(1        ,1        )= &
   0.5*(ddy2dx1dx2(1        ,2        )+ddy2dx1dx2(2        ,1        ))
ddy2dx1dx2(1        ,EOS%nx2+1)= &
   0.5*(ddy2dx1dx2(1        ,EOS%nx2  )+ddy2dx1dx2(2        ,EOS%nx2+1))
ddy2dx1dx2(EOS%nx1+1,1        )= &
   0.5*(ddy2dx1dx2(EOS%nx1+1,2        )+ddy2dx1dx2(EOS%nx1  ,1        ))
ddy2dx1dx2(EOS%nx1+1,EOS%nx2+1)= &
   0.5*(ddy2dx1dx2(EOS%nx1+1,EOS%nx2  )+ddy2dx1dx2(EOS%nx1  ,EOS%nx2+1))
!
ddy3dx1dx2(2:EOS%nx1  ,1          )= &
   (dy3dx2(3:EOS%nx1+1,1          )-dy3dx2(1:EOS%nx1-1,1          ))/  &
       (x1(3:EOS%nx1+1,1          )-    x1(1:EOS%nx1-1,1          ))
ddy3dx1dx2(2:EOS%nx1  ,EOS%nx2+1  )= &
   (dy3dx2(3:EOS%nx1+1,EOS%nx2+1  )-dy3dx2(1:EOS%nx1-1,EOS%nx2+1  ))/  &
       (x1(3:EOS%nx1+1,EOS%nx2+1  )-    x1(1:EOS%nx1-1,EOS%nx2+1  ))
ddy3dx1dx2(1          ,1:EOS%nx2+1)= &
   (dy3dx1(1          ,3:EOS%nx2+1)-dy3dx1(1          ,1:EOS%nx2-1))/  &
       (x2(1          ,3:EOS%nx2+1)-    x2(1          ,1:EOS%nx2-1))
ddy3dx1dx2(EOS%nx1+1  ,1:EOS%nx2+1)= &
   (dy3dx1(EOS%nx1+1  ,3:EOS%nx2+1)-dy3dx1(EOS%nx1+1  ,1:EOS%nx2-1))/  &
       (x2(EOS%nx1+1  ,3:EOS%nx2+1)-    x2(EOS%nx1+1  ,1:EOS%nx2-1))
ddy3dx1dx2(1        ,1        )= &
   0.5*(ddy3dx1dx2(1        ,2        )+ddy3dx1dx2(2        ,1        ))
ddy3dx1dx2(1        ,EOS%nx2+1)= &
   0.5*(ddy3dx1dx2(1        ,EOS%nx2  )+ddy3dx1dx2(2        ,EOS%nx2+1))
ddy3dx1dx2(EOS%nx1+1,1        )= &
   0.5*(ddy3dx1dx2(EOS%nx1+1,2        )+ddy3dx1dx2(EOS%nx1  ,1        ))
ddy3dx1dx2(EOS%nx1+1,EOS%nx2+1)= &
   0.5*(ddy3dx1dx2(EOS%nx1+1,EOS%nx2  )+ddy3dx1dx2(EOS%nx1  ,EOS%nx2+1))
!
! --- Compute coefficients for later bicubic interpolation ---
call bicubic_coeff(EOS%lnx11d, EOS%lnx21d, &
                   y1, dy1dx1, dy1dx2, ddy1dx1dx2, &
                   Ctmp, constant_spacing=.true.)
EOS%C(:,1,:,:)=reshape(Ctmp, [16, EOS%nx1, EOS%nx2])
call bicubic_coeff(EOS%lnx11d, EOS%lnx21d, &
                   y2, dy2dx1, dy2dx2, ddy2dx1dx2, &
                   Ctmp, constant_spacing=.true.)
EOS%C(:,2,:,:)=reshape(Ctmp, [16, EOS%nx1, EOS%nx2])
call bicubic_coeff(EOS%lnx11d, EOS%lnx21d, &
                   y3, dy3dx1, dy3dx2, ddy3dx1dx2, &
                   Ctmp, constant_spacing=.true.)
EOS%C(:,3,:,:)=reshape(Ctmp, [16, EOS%nx1, EOS%nx2])
!
end subroutine gasinter_SetUpGas


!----------*******************----------------------------------------------------------------------
subroutine gasinter_SetUpgamma()
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_SetUpgamma ('gas_interpolation_Set_Up_gamma')
!
! PURPOSE:
!   Compute coefficients for later bicubic interpolation of EOS quantities with simple gamma law.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter_SetUpgamma()
!
! INPUT:
!   None (Parameters are used from saved global variable.)
!
! OUTPUT:
!   None (Coefficients are in saved global variable.)
!
! ROUTINES:
!   bicubic_coeff:         Compute bicubic interpolation coefficients from tabulated values
!   uio_dattim:            Get string with current date and time
!
! MODULES:
!   bicubic_module:        Bicubic interpolation
!   tabinter_coeff_module: Interpolation coefficients
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   The current EOS table with interpolation coefficients and additional information is changed.
!
! RESTRICTIONS:
!   The results of the interpolation can only be as good as the underlying EOS equation.
!   An interpolation table has to be choosen and filled properly before calling the routine.
!
! PROCEDURE:
!   According to input parameters a P/T or a rho/e grid is generated.
!   For each pair the EOS is solved by calling GAS or GASINV -> h, s, Gamma1,...
!   Some first derivatives of quantities to be tabulated are computed by thermodynamic relations.
!   Cross derivatives are computed by numerical differentiation with
!   an appropriate handling of boundaries.
!   Coefficients for later bicubic interpolation are computed by bicubic_coeff.
!
! EXAMPLE:
!   ! --- Select and clear table 1 ---
!   call gasinter_reset(1)
!   ! --- Specify parameters ---
!   ...
!   ! --- Set up interpolation coefficients according to specified parameters --
!   call gasinter_SetUpgamma()
!
! MODIFICATION HISTORY:
!   1997-10-15 (B.F. Kiel) Written
!   2005-06-07 (B.F.) x1,x2,C_offset
!---------------------------------------------------------------------------------------------------
use bicubic_module
use tabinter_coeff_module
use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
real, dimension(EOS%nx1+1)              :: x11d
real, dimension(EOS%nx2+1)              :: x21d
real, dimension(EOS%nx1+1,EOS%nx2+1)    :: P, T, rho, cpd, Gamma1, delta, &
                                           h, s, cs, e, &
                                           x1, x2, y1, y2, y3
real, dimension(EOS%nx1+1,EOS%nx2+1)    :: dy1dx1, dy1dx2, ddy1dx1dx2, &
                                           dy2dx1, dy2dx2, ddy2dx1dx2, &
                                           dy3dx1, dy3dx2, ddy3dx1dx2
real, dimension(1:4, 1:4, &
                EOS%nx1  ,EOS%nx2  )    :: Ctmp
integer                                 :: ierror, ifent, kenter, ix1, ix2
character(len=24)                       :: time
real                                    :: P0, T0, cv0, &
                                           cpd0, Gamma10, delta0, h0, s0, cs0, &
                                           T_rhomin_emin, T_rhomax_emin, &
                                           T_rhomin_emax, T_rhomax_emax, &
                                           Tmin, Tmax
real, parameter                         :: Rgas=8.3145E+07, s_sigma=0.0
!---------------------------------------------------------------------------------------------------
!
EOS%nx=2
EOS%ny=3
EOS%nC=16
!
! --- Allocate arrays for coefficients and additional information ---
allocate(EOS%x_name(EOS%nx), EOS%x_unit(EOS%nx))
allocate(EOS%lnx11d(1:EOS%nx1+1), EOS%lnx21d(1:EOS%nx2+1))
allocate(EOS%C_name(EOS%ny), EOS%C_unit(EOS%ny), EOS%C_shift(EOS%ny), &
         EOS%C_offset(EOS%ny))
allocate(EOS%C(1:EOS%nC,1:EOS%ny,1:EOS%nx1,1:EOS%nx2))
!
! --- Shifts ---
!
! --- All shifts are zero ---
EOS%x1_shift=0.0
EOS%x2_shift=0.0
EOS%C_shift =0.0
EOS%x1_offset=0.0
EOS%x2_offset=0.0
EOS%C_offset =0.0
!
! --- Compute offsets, ... ---
EOS%lnx1off=alog(EOS%x1min+EOS%x1_shift)
EOS%lnx1fac=real(EOS%nx1)/ &
            (alog(EOS%x1max+EOS%x1_shift)-alog(EOS%x1min+EOS%x1_shift))
EOS%lnx11d(1:EOS%nx1+1)=EOS%lnx1off + (/(ix1,ix1=0,EOS%nx1)/)/EOS%lnx1fac
x11d=exp(EOS%lnx11d(1:EOS%nx1+1))-EOS%x1_shift
x1=spread(x11d,2,EOS%nx2+1)
!
EOS%lnx2off=alog(EOS%x2min+EOS%x2_shift)
EOS%lnx2fac=real(EOS%nx2)/ &
            (alog(EOS%x2max+EOS%x2_shift)-alog(EOS%x2min+EOS%x2_shift))
EOS%lnx21d(1:EOS%nx2+1)=EOS%lnx2off + (/(ix2,ix2=0,EOS%nx2)/)/EOS%lnx2fac
x21d=exp(EOS%lnx21d(1:EOS%nx2+1))-EOS%x2_shift
x2=spread(x21d,1,EOS%nx1+1)
!
if (EOS%variables == 'lnP,lnT->lnrho,h,s') then
  ! --- Put infos into EOS ---
  ! --- x1=P, x2=T ---
  ! --- y1=lnrho, y2=h, y3=s ---
  ! --- Give description ---
  EOS%description(1)='Coefficients for bicubic interpolation of EOS quantities.'
  EOS%description(2)='lnrho, h, s as function of lnP, lnT.'
  EOS%description(3)='Source: Gamma-law'
  EOS%ndes=3
  !
  ! --- Expand history ---
  EOS%nhis=min(EOS%nhis,nhismax)
  EOS%history(1:EOS%nhis)=EOS%history(1:EOS%nhis)
  if (EOS%nhis < nhismax) then
    call uio_dattim(time)
    EOS%nhis=EOS%nhis+1
    EOS%history(EOS%nhis)='gas2eos: ' // time
  endif
  !
  ! --- Data type ---
  EOS%interpolation='bicubic'
  EOS%source='gamma'
  !  
  ! --- 1D vectors ---
  EOS%x_name(1)='P'                 ; EOS%x_unit(1)='dyn/cm^2'
  EOS%x_name(2)='T'                 ; EOS%x_unit(2)='K'
  !
  ! --- Coefficients ---
  EOS%C_name(1)='rho'               ; EOS%C_unit(1)='g/cm^3'
  EOS%C_name(2)='h'                 ; EOS%C_unit(2)='erg/g'
  EOS%C_name(3)='s'                 ; EOS%C_unit(3)='erg/K/g'
  !
  ! --- Fill P and T array ---
  P=x1
  T=x2
  !
  ! --- Solve EOS for simple Gamma-law ---
  rho=P/(Rgas/EOS%Qmol*T)
  cpd=EOS%Gamma/(EOS%Gamma-1.0) * Rgas/EOS%Qmol
  Gamma1=EOS%Gamma
  delta=1.0
  h=Rgas/EOS%Qmol/(EOS%Gamma-1.0)*T + P/rho
  s=Rgas/EOS%Qmol * (1.0/(EOS%Gamma-1.0)*alog(T) - alog(rho) + s_sigma)
  !
  ! --- Tabulated quantities ---
  y1=alog(rho)
  y2=h
  y3=s
  !
  ! --- Compute first derivatives of quantities to be tabulated ---
  ! --- dlnrhodlnP ---
  dy1dx1=1.0/Gamma1 + delta*P/(T*rho*cpd)
  ! --- dlnrhodlnT ---
  dy1dx2=-delta
  ! --- dhdlnP ---
  dy2dx1=(1.0-delta)*P/rho
  ! --- dhdlnT ---
  dy2dx2=cpd*delta*T
  ! --- dsdlnP ---
  dy3dx1=-(delta*P)/(rho*T)
  ! --- dsdlnT ---
  dy3dx2=cpd*delta
  !
  ! --- Some logarithms ---
  x1=alog(P)
  x2=alog(T)
else if (EOS%variables == 'lnrho,lne->s,lnP,lnT') then
  ! --- Put infos into EOS ---
  ! --- x1=rho, x2=e(+shift) ---
  ! --- y1=s, y2=lnP, y3=lnT ---
  ! --- Give description ---
  EOS%description(1)='Coefficients for bicubic interpolation of EOS quantities.'
  EOS%description(2)='s, lnP, lnT as function of lnrho, lne.'
  EOS%description(3)='e shifted to positiv values.'
  EOS%description(4)='Source: Gamma-law'
  EOS%ndes=4
  !
  ! --- Expand history ---
  EOS%nhis=min(EOS%nhis,nhismax)
  EOS%history(1:EOS%nhis)=EOS%history(1:EOS%nhis)
  if (EOS%nhis < nhismax) then
    call uio_dattim(time)
    EOS%nhis=EOS%nhis+1
    EOS%history(EOS%nhis)='gas2eos: ' // time
  endif
  !
  ! --- Data type ---
  EOS%interpolation='bicubic'
  EOS%source='gamma'
  !
  ! --- 1D vectors ---
  EOS%x_name(1)='rho'               ; EOS%x_unit(1)='g/cm^3'
  EOS%x_name(2)='e'                 ; EOS%x_unit(2)='erg/g'
  !
  ! --- Coefficients ---
  EOS%C_name(1)='s'                 ; EOS%C_unit(1)='erg/K/g'
  EOS%C_name(2)='P'                 ; EOS%C_unit(2)='dyn/cm^2'
  EOS%C_name(3)='T'                 ; EOS%C_unit(3)='K'
  !
  ! --- Fill rho and e array ---
  rho=x1
  e=x2
  !
  ! --- Solve EOS for simple Gamma-law ---
  T=(EOS%Gamma-1.0)/(Rgas/EOS%Qmol)*e
  P=(EOS%Gamma-1.0)*e*rho
  cpd=EOS%Gamma/(EOS%Gamma-1.0) * Rgas/EOS%Qmol
  Gamma1=EOS%Gamma
  delta=1.0
  h=e + P/rho
  s=Rgas/EOS%Qmol * (1.0/(EOS%Gamma-1.0)*alog(T) - alog(rho) + s_sigma)
  !
  ! --- Tabulated quantities ---
  y1=s
  y2=alog(P)
  y3=alog(T)
  !
  ! --- Compute first derivatives of quantities to be tabulated ---
  ! --- Shift in e taken into account ---
  ! --- dsdlnrho ---
  dy1dx1=-P/(T*rho)
  ! --- dsdlne ---
  dy1dx2=(e+EOS%x2_shift)/T
  ! --- dlnPdlnrho ---
  dy2dx1=Gamma1*(1.0-P/(rho*cpd*T))
  ! --- dlnPdlne ---
  dy2dx2=(e+EOS%x2_shift)*Gamma1/(T*cpd)
  ! --- dlnTdlnrho ---
  dy3dx1=-P/(rho*T)*( (1.0+Gamma1*delta*P/(T*rho*cpd))/(cpd*delta)-Gamma1/cpd )
  ! --- dlnTdlne ---
  dy3dx2=(e+EOS%x2_shift)*(1+Gamma1*delta*P/(T*rho*cpd))/(T*cpd*delta)
  !
  ! --- Some logarithms ---
  x1=alog(rho)
  x2=alog(e+EOS%x2_shift)
else
  stop 'Unknown interpolation mode'
endif
!
! --- Compute cross derivatives ---
ddy1dx1dx2(2:EOS%nx1,  2:EOS%nx2  )= 0.5*( &
   (dy1dx1(2:EOS%nx1  ,3:EOS%nx2+1)-dy1dx1(2:EOS%nx1  ,1:EOS%nx2-1))/  &
       (x2(2:EOS%nx1  ,3:EOS%nx2+1)-    x2(2:EOS%nx1  ,1:EOS%nx2-1)) + &
   (dy1dx2(3:EOS%nx1+1,2:EOS%nx2  )-dy1dx2(1:EOS%nx1-1,2:EOS%nx2  ))/  &
       (x1(3:EOS%nx1+1,2:EOS%nx2  )-    x1(1:EOS%nx1-1,2:EOS%nx2  ))  )
ddy2dx1dx2(2:EOS%nx1,  2:EOS%nx2  )= 0.5*( &
   (dy2dx1(2:EOS%nx1  ,3:EOS%nx2+1)-dy2dx1(2:EOS%nx1  ,1:EOS%nx2-1))/  &
       (x2(2:EOS%nx1  ,3:EOS%nx2+1)-    x2(2:EOS%nx1  ,1:EOS%nx2-1)) + &
   (dy2dx2(3:EOS%nx1+1,2:EOS%nx2  )-dy2dx2(1:EOS%nx1-1,2:EOS%nx2  ))/  &
       (x1(3:EOS%nx1+1,2:EOS%nx2  )-    x1(1:EOS%nx1-1,2:EOS%nx2  ))  )
ddy3dx1dx2(2:EOS%nx1,  2:EOS%nx2  )= 0.5*( &
   (dy3dx1(2:EOS%nx1  ,3:EOS%nx2+1)-dy3dx1(2:EOS%nx1  ,1:EOS%nx2-1))/  &
       (x2(2:EOS%nx1  ,3:EOS%nx2+1)-    x2(2:EOS%nx1  ,1:EOS%nx2-1)) + &
   (dy3dx2(3:EOS%nx1+1,2:EOS%nx2  )-dy3dx2(1:EOS%nx1-1,2:EOS%nx2  ))/  &
       (x1(3:EOS%nx1+1,2:EOS%nx2  )-    x1(1:EOS%nx1-1,2:EOS%nx2  ))  )
!
! --- Boundary values of cross derivatives ---
!ddy1dx1dx2(2:EOS%nx1,1          )=ddy1dx1dx2(2:EOS%nx1,2          )
!ddy1dx1dx2(2:EOS%nx1,EOS%nx2+1  )=ddy1dx1dx2(2:EOS%nx1,EOS%nx2    )
!ddy1dx1dx2(1        ,1:EOS%nx2+1)=ddy1dx1dx2(2        ,1:EOS%nx2+1)
!ddy1dx1dx2(EOS%nx1+1,1:EOS%nx2+1)=ddy1dx1dx2(EOS%nx1  ,1:EOS%nx2+1)
!ddy2dx1dx2(2:EOS%nx1,1          )=ddy2dx1dx2(2:EOS%nx1,2          )
!ddy2dx1dx2(2:EOS%nx1,EOS%nx2+1  )=ddy2dx1dx2(2:EOS%nx1,EOS%nx2    )
!ddy2dx1dx2(1        ,1:EOS%nx2+1)=ddy2dx1dx2(2        ,1:EOS%nx2+1)
!ddy2dx1dx2(EOS%nx1+1,1:EOS%nx2+1)=ddy2dx1dx2(EOS%nx1  ,1:EOS%nx2+1)
!ddy3dx1dx2(2:EOS%nx1,1          )=ddy3dx1dx2(2:EOS%nx1,2          )
!ddy3dx1dx2(2:EOS%nx1,EOS%nx2+1  )=ddy3dx1dx2(2:EOS%nx1,EOS%nx2    )
!ddy3dx1dx2(1        ,1:EOS%nx2+1)=ddy3dx1dx2(2        ,1:EOS%nx2+1)
!ddy3dx1dx2(EOS%nx1+1,1:EOS%nx2+1)=ddy3dx1dx2(EOS%nx1  ,1:EOS%nx2+1)
!
ddy1dx1dx2(2:EOS%nx1  ,1          )= &
   (dy1dx2(3:EOS%nx1+1,1          )-dy1dx2(1:EOS%nx1-1,1          ))/  &
       (x1(3:EOS%nx1+1,1          )-    x1(1:EOS%nx1-1,1          ))
ddy1dx1dx2(2:EOS%nx1  ,EOS%nx2+1  )= &
   (dy1dx2(3:EOS%nx1+1,EOS%nx2+1  )-dy1dx2(1:EOS%nx1-1,EOS%nx2+1  ))/  &
       (x1(3:EOS%nx1+1,EOS%nx2+1  )-    x1(1:EOS%nx1-1,EOS%nx2+1  ))
ddy1dx1dx2(1          ,1:EOS%nx2+1)= &
   (dy1dx1(1          ,3:EOS%nx2+1)-dy1dx1(1          ,1:EOS%nx2-1))/  &
       (x2(1          ,3:EOS%nx2+1)-    x2(1          ,1:EOS%nx2-1))
ddy1dx1dx2(EOS%nx1+1  ,1:EOS%nx2+1)= &
   (dy1dx1(EOS%nx1+1  ,3:EOS%nx2+1)-dy1dx1(EOS%nx1+1  ,1:EOS%nx2-1))/  &
       (x2(EOS%nx1+1  ,3:EOS%nx2+1)-    x2(EOS%nx1+1  ,1:EOS%nx2-1))
ddy1dx1dx2(1        ,1        )= &
   0.5*(ddy1dx1dx2(1        ,2        )+ddy1dx1dx2(2        ,1        ))
ddy1dx1dx2(1        ,EOS%nx2+1)= &
   0.5*(ddy1dx1dx2(1        ,EOS%nx2  )+ddy1dx1dx2(2        ,EOS%nx2+1))
ddy1dx1dx2(EOS%nx1+1,1        )= &
   0.5*(ddy1dx1dx2(EOS%nx1+1,2        )+ddy1dx1dx2(EOS%nx1  ,1        ))
ddy1dx1dx2(EOS%nx1+1,EOS%nx2+1)= &
   0.5*(ddy1dx1dx2(EOS%nx1+1,EOS%nx2  )+ddy1dx1dx2(EOS%nx1  ,EOS%nx2+1))
!
ddy2dx1dx2(2:EOS%nx1  ,1          )= &
   (dy2dx2(3:EOS%nx1+1,1          )-dy2dx2(1:EOS%nx1-1,1          ))/  &
       (x1(3:EOS%nx1+1,1          )-    x1(1:EOS%nx1-1,1          ))
ddy2dx1dx2(2:EOS%nx1  ,EOS%nx2+1  )= &
   (dy2dx2(3:EOS%nx1+1,EOS%nx2+1  )-dy2dx2(1:EOS%nx1-1,EOS%nx2+1  ))/  &
       (x1(3:EOS%nx1+1,EOS%nx2+1  )-    x1(1:EOS%nx1-1,EOS%nx2+1  ))
ddy2dx1dx2(1          ,1:EOS%nx2+1)= &
   (dy2dx1(1          ,3:EOS%nx2+1)-dy2dx1(1          ,1:EOS%nx2-1))/  &
       (x2(1          ,3:EOS%nx2+1)-    x2(1          ,1:EOS%nx2-1))
ddy2dx1dx2(EOS%nx1+1  ,1:EOS%nx2+1)= &
   (dy2dx1(EOS%nx1+1  ,3:EOS%nx2+1)-dy2dx1(EOS%nx1+1  ,1:EOS%nx2-1))/  &
       (x2(EOS%nx1+1  ,3:EOS%nx2+1)-    x2(EOS%nx1+1  ,1:EOS%nx2-1))
ddy2dx1dx2(1        ,1        )= &
   0.5*(ddy2dx1dx2(1        ,2        )+ddy2dx1dx2(2        ,1        ))
ddy2dx1dx2(1        ,EOS%nx2+1)= &
   0.5*(ddy2dx1dx2(1        ,EOS%nx2  )+ddy2dx1dx2(2        ,EOS%nx2+1))
ddy2dx1dx2(EOS%nx1+1,1        )= &
   0.5*(ddy2dx1dx2(EOS%nx1+1,2        )+ddy2dx1dx2(EOS%nx1  ,1        ))
ddy2dx1dx2(EOS%nx1+1,EOS%nx2+1)= &
   0.5*(ddy2dx1dx2(EOS%nx1+1,EOS%nx2  )+ddy2dx1dx2(EOS%nx1  ,EOS%nx2+1))
!
ddy3dx1dx2(2:EOS%nx1  ,1          )= &
   (dy3dx2(3:EOS%nx1+1,1          )-dy3dx2(1:EOS%nx1-1,1          ))/  &
       (x1(3:EOS%nx1+1,1          )-    x1(1:EOS%nx1-1,1          ))
ddy3dx1dx2(2:EOS%nx1  ,EOS%nx2+1  )= &
   (dy3dx2(3:EOS%nx1+1,EOS%nx2+1  )-dy3dx2(1:EOS%nx1-1,EOS%nx2+1  ))/  &
       (x1(3:EOS%nx1+1,EOS%nx2+1  )-    x1(1:EOS%nx1-1,EOS%nx2+1  ))
ddy3dx1dx2(1          ,1:EOS%nx2+1)= &
   (dy3dx1(1          ,3:EOS%nx2+1)-dy3dx1(1          ,1:EOS%nx2-1))/  &
       (x2(1          ,3:EOS%nx2+1)-    x2(1          ,1:EOS%nx2-1))
ddy3dx1dx2(EOS%nx1+1  ,1:EOS%nx2+1)= &
   (dy3dx1(EOS%nx1+1  ,3:EOS%nx2+1)-dy3dx1(EOS%nx1+1  ,1:EOS%nx2-1))/  &
       (x2(EOS%nx1+1  ,3:EOS%nx2+1)-    x2(EOS%nx1+1  ,1:EOS%nx2-1))
ddy3dx1dx2(1        ,1        )= &
   0.5*(ddy3dx1dx2(1        ,2        )+ddy3dx1dx2(2        ,1        ))
ddy3dx1dx2(1        ,EOS%nx2+1)= &
   0.5*(ddy3dx1dx2(1        ,EOS%nx2  )+ddy3dx1dx2(2        ,EOS%nx2+1))
ddy3dx1dx2(EOS%nx1+1,1        )= &
   0.5*(ddy3dx1dx2(EOS%nx1+1,2        )+ddy3dx1dx2(EOS%nx1  ,1        ))
ddy3dx1dx2(EOS%nx1+1,EOS%nx2+1)= &
   0.5*(ddy3dx1dx2(EOS%nx1+1,EOS%nx2  )+ddy3dx1dx2(EOS%nx1  ,EOS%nx2+1))
!
! --- Compute coefficients for later bicubic interpolation ---
call bicubic_coeff(EOS%lnx11d, EOS%lnx21d, &
                   y1, dy1dx1, dy1dx2, ddy1dx1dx2, &
                   Ctmp, constant_spacing=.true.)
EOS%C(:,1,:,:)=reshape(Ctmp, [16, EOS%nx1, EOS%nx2])
call bicubic_coeff(EOS%lnx11d, EOS%lnx21d, &
                   y2, dy2dx1, dy2dx2, ddy2dx1dx2, &
                   Ctmp, constant_spacing=.true.)
EOS%C(:,2,:,:)=reshape(Ctmp, [16, EOS%nx1, EOS%nx2])
call bicubic_coeff(EOS%lnx11d, EOS%lnx21d, &
                   y3, dy3dx1, dy3dx2, ddy3dx1dx2, &
                   Ctmp, constant_spacing=.true.)
EOS%C(:,3,:,:)=reshape(Ctmp, [16, EOS%nx1, EOS%nx2])
!
end subroutine gasinter_SetUpgamma


!----------**************---------------------------------------------------------------------------
subroutine gasinter_SetUp()
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_SetUp ('gas_interpolation_Set_Up')
!
! PURPOSE:
!   Compute coefficients for later bicubic interpolation of GAS/EOS quantities.
!   Use GAS routines or simple gamma law.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter_SetUp()
!
! INPUT:
!   None (Parameters are used from saved global variable.)
!
! OUTPUT:
!   None (Coefficients are in saved global variable.)
!
! ROUTINES:
!   gasinter_SetUpGas:     Compute coefficients for later bicubic interpolation of GAS quantities
!   gasinter_SetUpgamma:   Compute coeffs for bicubic interpolation of EOS qs with simple gamma law
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!
! SIDE EFFECTS:
!   See the called gasinter_SetUpXXX routines.
!
! RESTRICTIONS:
!   See the called gasinter_SetUpXXX routines.
!
! PROCEDURE:
!
! EXAMPLE:
!   ! --- Select and clear table 1 ---
!   call gasinter_reset(1)
!   ! --- Specify parameters ---
!   ...
!   ! --- Set up interpolation coefficients according to specified parameters --
!   call gasinterSetUp()
!
! MODIFICATION HISTORY:
!   1997-10-15 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
!---------------------------------------------------------------------------------------------------
!
select case (EOS%source)
  case ('gasgas', 'GASGAS', 'GasGas', 'Gasgas')
    call gasinter_SetUpGas()
  case ('gamma', 'GAMMA', 'Gamma')
    call gasinter_SetUpgamma()
  case default
    stop 'Unknown source type'
end select
!
end subroutine gasinter_SetUp


!----------*****************------------------------------------------------------------------------
subroutine gasinter_CheckGas(ncpri, nx1_check0,nx2_check0)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_CheckGas ('gas_interpolation_Check_Gas')
!
! PURPOSE:
!   Check accuracy of interpolation table for GAS quantities.
!
! CATEGORY:
!   Interpolation, EOS
!
! CALLING SEQUENCE:
!   call gasinter_CheckGas(ncpri, nx1_check0,nx2_check0)
!
! INPUT:
!   ncpri:        ('number_channel_print') integer, logical output channel number
!   nx1_check0:   ('number_x_1_check_0') integer, number of test points in
!                 x1 direction: 0 -> nx1_check=EOS%nx1-3
!   nx2_check0:   ('number_x_2_check_0') integer, number of test points in
!                 x2 direction: 0 -> nx2_check=EOS%nx2-3
!
! OUTPUT:
!   None
!
! ROUTINES:
!   gasgas:                EOS routine of HDW-code
!   gasinter:              Solve EOS by bicubic interpolation: P  , T -> rho, h, ...
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   uio_closwr:            Close file after writing
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   supergas:              Extended gasgas routine: P, T -> rho, h,...
!   supergasinv:           Extended inverted gasgas routine: rho, e -> P, T, ...
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   Text is printed on channel ncpri.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For a grid of specified number of elements, solve original gas equation
!   and interpolate with new table.
!   Measure time and compare results.
!   Print results on channel ncpri.
!
! EXAMPLE:
!   call gasinter_CheckGas(6, 20,30)
!
! MODIFICATION HISTORY:
!   1997-04-29 (B.F. Kiel) Written
!   1997-05-05 (B.F.) Check alse new interpolation mode (EOS%variables = 'lnrho,lne->s,lnP,lnT')
!   1997-05-09 (B.F.) Variables may be shifted
!   1997-05-21 (B.F.) Don't check boundary cells
!   1997-05-23 (B.F.) nx1_check -> nx1_check0
!   1997-07-27 (B.F.) Call of eosinter changed
!   2004-02-25 (B.F. Uppsala) Call GASGAS instead of GAS (was implicitely done before)
!   2013-02-15 (B.F. Lyon) Removed ierr parameter from eosinter call
!   2014-09-06 (B.F. Uppsala) Compute and output additional arrays
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
use gasinter_module
use uio_bulk_module
use gas_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncpri, nx1_check0,nx2_check0
!
! --- Local variables ---
character(len=10)                       :: time
real                                    :: P0, T0, cv0
integer                                 :: ierror, ifent, kenter, &
                                           nx1_check, nx2_check
real, allocatable, dimension(:,:)       :: P,  T, &
                                           rho,  cpd,  Gamma1,  delta,  h,  s, &
                                           cs,  e
real, allocatable, dimension(:,:)       :: P1, T1, &
                                           rho1, cpd1, Gamma11, delta1, h1, s1,&
                                           cs1, e1, &
                                           dPdrho1, dPde1, dTde1
real, allocatable, dimension(:,:)       :: P2, T2, &
                                           rho2, cpd2, Gamma12, delta2, h2, s2,&
                                           cs2, e2, &
                                           dPdrho2, dPde2, dTde2
real, allocatable, dimension(:,:)       :: P3, T3, &
                                           rho3, cpd3, Gamma13, delta3, h3, s3,&
                                           cs3, e3, &
                                           dPdrho3, dPde3, dTde3
real, allocatable, dimension(:)         :: lnx11d_h, x11d_h
real, allocatable, dimension(:)         :: lnx21d_h, x21d_h
integer                                 :: ix1, ix2, ncerr
!---------------------------------------------------------------------------------------------------
!
! --- Test if default array size is to be used ---
if (nx1_check0 <= 0) then
  nx1_check=EOS%nx1-3
else
  nx1_check=nx1_check0
endif
if (nx2_check0 <= 0) then
  nx2_check=EOS%nx2-3
else
  nx2_check=nx2_check0
endif
!
! --- Allocate arrays ---
allocate(     P(nx1_check,nx2_check),     T(nx1_check,nx2_check), &
            rho(nx1_check,nx2_check),   cpd(nx1_check,nx2_check), &
         Gamma1(nx1_check,nx2_check), delta(nx1_check,nx2_check), &
              h(nx1_check,nx2_check),     s(nx1_check,nx2_check), &
             cs(nx1_check,nx2_check),     e(nx1_check,nx2_check))
allocate(     P1(nx1_check,nx2_check),     T1(nx1_check,nx2_check), &
            rho1(nx1_check,nx2_check),   cpd1(nx1_check,nx2_check), &
         Gamma11(nx1_check,nx2_check), delta1(nx1_check,nx2_check), &
              h1(nx1_check,nx2_check),     s1(nx1_check,nx2_check), &
             cs1(nx1_check,nx2_check),     e1(nx1_check,nx2_check), &
         dPdrho1(nx1_check,nx2_check),  dPde1(nx1_check,nx2_check), &
           dTde1(nx1_check,nx2_check) )
allocate(     P2(nx1_check,nx2_check),     T2(nx1_check,nx2_check), &
            rho2(nx1_check,nx2_check),   cpd2(nx1_check,nx2_check), &
         Gamma12(nx1_check,nx2_check), delta2(nx1_check,nx2_check), &
              h2(nx1_check,nx2_check),     s2(nx1_check,nx2_check), &
             cs2(nx1_check,nx2_check),     e2(nx1_check,nx2_check), &
         dPdrho2(nx1_check,nx2_check),  dPde2(nx1_check,nx2_check), &
           dTde2(nx1_check,nx2_check) )
allocate(     P3(nx1_check,nx2_check),     T3(nx1_check,nx2_check), &
            rho3(nx1_check,nx2_check),   cpd3(nx1_check,nx2_check), &
         Gamma13(nx1_check,nx2_check), delta3(nx1_check,nx2_check), &
              h3(nx1_check,nx2_check),     s3(nx1_check,nx2_check), &
             cs3(nx1_check,nx2_check),     e3(nx1_check,nx2_check), &
         dPdrho3(nx1_check,nx2_check),  dPde3(nx1_check,nx2_check), &
           dTde3(nx1_check,nx2_check) )
allocate(lnx11d_h(nx1_check), x11d_h(nx1_check))
allocate(lnx21d_h(nx2_check), x21d_h(nx2_check))
!
! --- Initialize GAS routine by calling it with kenter=-1 ---
kenter=-1
ifent=0
call supergas(8000.0, 16000.0, &
        rho(1,1), cpd(1,1), Gamma1(1,1), delta(1,1), h(1,1), s(1,1), cs(1,1), &
        ierror, EOS%ifion, ifent, kenter, EOS%abuX, EOS%abuY, EOS%Qmol)
!
! --- Array with test values ---
!lnx11d_h=alog(EOS%x1min+EOS%x1_shift) + &
!         (/(ix1,ix1=0,nx1_check-1)/)/real(nx1_check-1)* &
!         (alog(EOS%x1max+EOS%x1_shift)-alog(EOS%x1min+EOS%x1_shift))
!x11d_h=exp(lnx11d_h)-EOS%x1_shift
!lnx21d_h=alog(EOS%x2min+EOS%x2_shift) + &
!         (/(ix2,ix2=0,nx2_check-1)/)/real(nx2_check-1)* &
!         (alog(EOS%x2max+EOS%x2_shift)-alog(EOS%x2min+EOS%x2_shift))
!x21d_h=exp(lnx21d_h)-EOS%x2_shift
lnx11d_h=EOS%lnx1off + 1.5/EOS%lnx1fac + &
         (/(ix1,ix1=0,nx1_check-1)/)/real(nx1_check-1)* &
         (alog(EOS%x1max+EOS%x1_shift)-EOS%lnx1off-3.0/EOS%lnx1fac)
x11d_h=exp(lnx11d_h)-EOS%x1_shift
lnx21d_h=EOS%lnx2off + 1.5/EOS%lnx2fac + &
         (/(ix2,ix2=0,nx2_check-1)/)/real(nx2_check-1)* &
         (alog(EOS%x2max+EOS%x2_shift)-EOS%lnx2off-3.0/EOS%lnx2fac)
x21d_h=exp(lnx21d_h)-EOS%x2_shift
!
if (EOS%variables == 'lnP,lnT->lnrho,h,s') then
  !
  ! --- Arrays of independent variables ---
  P=spread(x11d_h, 2, nx2_check)
  T=spread(x21d_h, 1, nx1_check)
  P1=P
  T1=T
  !
  ! --- Compute GAS quantities with new GASINTER routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of gasinter0:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      call gasinter(P1(ix1,ix2), T1(ix1,ix2), &
                    rho1(ix1,ix2), cpd1(ix1,ix2), Gamma11(ix1,ix2), &
                    delta1(ix1,ix2), h1(ix1,ix2), s1(ix1,ix2), cs1(ix1,ix2), &
                    ierror, ifent)
      e1(ix1,ix2)=h1(ix1,ix2)-P1(ix1,ix2)/rho1(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after gasinter0:                ' // time
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of gasinter1:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    !DIR$ INLINE
    call gasinter(P1(:,ix2), T1(:,ix2), &
                  rho1(:,ix2), cpd1(:,ix2), Gamma11(:,ix2), &
                  delta1(:,ix2), h1(:,ix2), s1(:,ix2), cs1(:,ix2), &
                  ierror, ifent)
    e1(:,ix2)=h1(:,ix2)-P1(:,ix2)/rho1(:,ix2)
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after gasinter1:                ' // time
  !
  ! --- Compute GAS quantities with old GAS routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of GAS:            ' // time
  !
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      kenter=0
      !DIR$ INLINE
      call supergas(P(ix1,ix2), T(ix1,ix2), &
               rho(ix1,ix2), cpd(ix1,ix2), Gamma1(ix1,ix2), &
               delta(ix1,ix2), h(ix1,ix2), s(ix1,ix2), cs(ix1,ix2), &
               ierror, EOS%ifion, ifent, kenter, &
               EOS%abuX, EOS%abuY, EOS%Qmol)
      e(ix1,ix2)=h(ix1,ix2)-P(ix1,ix2)/rho(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GAS:                      ' // time
  !
else if (EOS%variables == 'lnrho,lne->s,lnP,lnT') then
  !
!!$  ! --- Compute EOS quantities with new EOSINTER routine ---
!!$  call date_and_time(time=time)
!!$  write(ncpri,'(A)') 'Time before start of eosinter0:      ' // time
!!$  !
!!$  ifent=1
!!$  do ix2=1,nx2_check
!!$    do ix1=1,nx1_check
!!$      !DIR$ INLINE
!!$      call eosinter(rho1(ix1,ix2), e1(ix1,ix2), &
!!$                    P=P1(ix1,ix2), dPdrho=dPdrho1(ix1,ix2), &
!!$                                   dPde=dPde1(ix1,ix2), &
!!$                    T=T1(ix1,ix2), dTde=dTde1(ix1,ix2), &
!!$                    s=s1(ix1,ix2))
!!$      cpd1(ix1,ix2)=P1(ix1,ix2)/(rho1(ix1,ix2)*T1(ix1,ix2)) + &
!!$                    rho1(ix1,ix2)*dPdrho1(ix1,ix2)/(T1(ix1,ix2)*dPde1(ix1,ix2))
!!$      Gamma11(ix1,ix2)=T1(ix1,ix2)*cpd1(ix1,ix2)*dPde1(ix1,ix2)/P1(ix1,ix2)
!!$      delta1(ix1,ix2) =1.0/(cpd1(ix1,ix2)*dTde1(ix1,ix2)- &
!!$                           (Gamma11(ix1,ix2)*P1(ix1,ix2))/ &
!!$                           (T1(ix1,ix2)*rho1(ix1,ix2)*cpd1(ix1,ix2)))
!!$      cs1(ix1,ix2)=sqrt(Gamma11(ix1,ix2)*P1(ix1,ix2)/rho1(ix1,ix2))
!!$      h1(ix1,ix2)=e1(ix1,ix2)+P1(ix1,ix2)/rho1(ix1,ix2)
!!$    end do ! ix1
!!$  end do ! ix2
!!$  !
!!$  call date_and_time(time=time)
!!$  write(ncpri,'(A)') 'Time after eosinter0:                ' // time
  !
  ! === Compute EOS quantities with new EOSINTER1 routine from regular rho/e array ===
  !
  ! --- Arrays of independent variables ---
  rho1=spread(x11d_h, 2, nx2_check)
  e1  =spread(x21d_h, 1, nx1_check)
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of eosinter1:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    !DIR$ INLINE
    call eosinter(rho1(:,ix2), e1(:,ix2), &
                  P=P1(:,ix2), dPdrho=dPdrho1(:,ix2), dPde=dPde1(:,ix2), &
                  T=T1(:,ix2), dTde=dTde1(:,ix2), &
                  s=s1(:,ix2))
    cpd1(:,ix2)=P1(:,ix2)/(rho1(:,ix2)*T1(:,ix2)) + &
                  rho1(:,ix2)*dPdrho1(:,ix2)/(T1(:,ix2)*dPde1(:,ix2))
    Gamma11(:,ix2)=T1(:,ix2)*cpd1(:,ix2)*dPde1(:,ix2)/P1(:,ix2)
    delta1(:,ix2) =1.0/(cpd1(:,ix2)*dTde1(:,ix2)- &
                         (Gamma11(:,ix2)*P1(:,ix2))/ &
                         (T1(:,ix2)*rho1(:,ix2)*cpd1(:,ix2)))
    cs1(:,ix2)=sqrt(Gamma11(:,ix2)*P1(:,ix2)/rho1(:,ix2))
    h1(:,ix2)=e1(:,ix2)+P1(:,ix2)/rho1(:,ix2)
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after eosinter1:                ' // time
  !
  ! === Compute EOS quantities with old GAS routine from results of EOSINTER ===
  !
  P=P1
  T=T1
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of GAS:            ' // time
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      kenter=0
      call supergas(P(ix1,ix2), T(ix1,ix2), &
               rho(ix1,ix2), cpd(ix1,ix2), Gamma1(ix1,ix2), &
               delta(ix1,ix2), h(ix1,ix2), s(ix1,ix2), cs(ix1,ix2), &
               ierror, EOS%ifion, ifent, kenter, &
               EOS%abuX, EOS%abuY, EOS%Qmol)
      e(ix1,ix2)=h(ix1,ix2)-P(ix1,ix2)/rho(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GAS:                      ' // time
  !
  ! === Compute EOS quantities with superGASinv from regular rho/e array ===
  !
  rho2=rho1
  e2  =e1
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of GASinv:          ' // time
  ifent=1
  kenter=0
  do ix2=nx2_check,1,-1
    do ix1=nx1_check,1,-1
      call supergasinv(rho2(ix1,ix2), e2(ix1,ix2), P0, T0, &
                  cpd2(ix1,ix2), Gamma12(ix1,ix2), delta2(ix1,ix2), &
                  h2(ix1,ix2),   s2(ix1,ix2),      cs2(ix1,ix2), &
                  ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
      write(6,'(A,2I4)') 'ierror, kenter=', ierror, kenter
      P2(ix1,ix2)=P0
      T2(ix1,ix2)=T0
      cv0=(cpd2(ix1,ix2)*delta2(ix1,ix2))/(1.0+ &
                                (Gamma12(ix1,ix2)*delta2(ix1,ix2)*P2(ix1,ix2))/ &
                                 (T2(ix1,ix2)*rho2(ix1,ix2)*cpd2(ix1,ix2)))
      if ((ix1 > 1) .and. (ierror .eq. 0)) then
        kenter=1
        P0=P2(ix1,ix2) * (rho2(ix1-1,ix2)/rho2(ix1,ix2)) ** ( &
            Gamma12(ix1,ix2) * &
            (1.0-P2(ix1,ix2)/(rho2(ix1,ix2)*cpd2(ix1,ix2)*T2(ix1,ix2))) )
        T0=T2(ix1,ix2) * (rho2(ix1-1,ix2)/rho2(ix1,ix2)) ** ( &
            -P2(ix1,ix2)/(rho2(ix1,ix2)*T2(ix1,ix2)) * &
            (1.0/cv0-Gamma12(ix1,ix2)/cpd2(ix1,ix2)) )
      else
        kenter=0
      endif
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GASinv:                   ' // time
  !
  ! === Compute EOS quantities with superGASinv from results of superGAS ===
  !
  rho3=rho
  e3  =e
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of GASinv:          ' // time
  ifent=1
  kenter=0
  do ix2=nx2_check,1,-1
    do ix1=nx1_check,1,-1
      call supergasinv(rho3(ix1,ix2), e3(ix1,ix2), P0, T0, &
                  cpd3(ix1,ix2), Gamma13(ix1,ix2), delta3(ix1,ix2), &
                  h3(ix1,ix2),   s3(ix1,ix2),      cs3(ix1,ix2), &
                  ierror, EOS%ifion, ifent, kenter, EOS%abuX,EOS%abuY,EOS%Qmol)
      write(6,'(A,2I4)') 'ierror, kenter=', ierror, kenter
      P3(ix1,ix2)=P0
      T3(ix1,ix2)=T0
      cv0=(cpd3(ix1,ix2)*delta3(ix1,ix2))/(1.0+ &
                                (Gamma13(ix1,ix2)*delta3(ix1,ix2)*P3(ix1,ix2))/ &
                                 (T3(ix1,ix2)*rho3(ix1,ix2)*cpd3(ix1,ix2)))
      if ((ix1 > 1) .and. (ierror .eq. 0)) then
        kenter=1
        P0=P3(ix1,ix2) * (rho3(ix1-1,ix2)/rho3(ix1,ix2)) ** ( &
            Gamma13(ix1,ix2) * &
            (1.0-P3(ix1,ix2)/(rho3(ix1,ix2)*cpd3(ix1,ix2)*T3(ix1,ix2))) )
        T0=T3(ix1,ix2) * (rho3(ix1-1,ix2)/rho3(ix1,ix2)) ** ( &
            -P3(ix1,ix2)/(rho3(ix1,ix2)*T3(ix1,ix2)) * &
            (1.0/cv0-Gamma13(ix1,ix2)/cpd3(ix1,ix2)) )
      else
        kenter=0
      endif
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GASinv:                   ' // time
  !
endif
!
! --- Results ---
write(ncpri,*)
write(ncpri,'(A)') 'Ranges of variables'
write(ncpri,'(A)') '-------------------'
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   P: ', minval(P),   maxval(P)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   T: ', minval(T),   maxval(T)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range: rho: ', minval(rho), maxval(rho)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   e: ', minval(e),   maxval(e)
write(ncpri,'(A)') 'Accuracy of interpolation'
write(ncpri,'(A)') '-------------------------'
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    s: ', &
  maxval(abs(   s-   s1))/maxval(abs(s))
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    h: ', &
  maxval(abs(   h-   h1))/maxval(abs(h))
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    e: ', &
  maxval(abs(   e-   e1))/maxval(abs(e))
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:      P: ', &
  maxval(abs(   P-   P1)/  P)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:      T: ', &
  maxval(abs(   T-   T1)/  T)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:    rho: ', &
  maxval(abs( rho- rho1)/rho)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:    cpd: ', &
  maxval(abs( cpd- cpd1)/cpd)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:  delta: ', &
  maxval(abs( delta- delta1)/delta)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation: Gamma1: ', &
  maxval(abs(Gamma1-Gamma11)/Gamma1)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:     cs: ', &
  maxval(abs(cs-cs1)/cs)
write(ncpri,'(A,F13.8)') 'RMS     relative deviation:    rho: ', &
  sqrt(sum(( rho- rho1)**2/rho**2)/real(nx1_check*nx2_check))
write(ncpri,'(A,F13.8)') 'RMS     relative deviation:    cpd: ', &
  sqrt(sum(( cpd- cpd1)**2/cpd**2)/real(nx1_check*nx2_check))
!
! --- Write original and interpolated arrays into file ---
call uio_openwr(ncerr, 'gas2eos_err.dat', form=EOS%form, conv=EOS%conv)
!
! --- Write file_id ---
call uio_wr(ncerr, 'eos-tables: error', 'file_id', name='File identification')
!
! --- Write all arrays ---
call uio_wr(ncerr, p      , 'p'      , format=EOS%number_format)
call uio_wr(ncerr, t      , 't'      , format=EOS%number_format)
call uio_wr(ncerr, s      , 's'      , format=EOS%number_format)
call uio_wr(ncerr, h      , 'h'      , format=EOS%number_format)
call uio_wr(ncerr, rho    , 'rho'    , format=EOS%number_format)
call uio_wr(ncerr, delta  , 'delta'  , format=EOS%number_format)
call uio_wr(ncerr, Gamma1 , 'gamma1' , format=EOS%number_format)
call uio_wr(ncerr, cpd    , 'cpd'    , format=EOS%number_format)
call uio_wr(ncerr, p1     , 'p1'     , format=EOS%number_format)
call uio_wr(ncerr, t1     , 't1'     , format=EOS%number_format)
call uio_wr(ncerr, s1     , 's1'     , format=EOS%number_format)
call uio_wr(ncerr, h1     , 'h1'     , format=EOS%number_format)
call uio_wr(ncerr, rho1   , 'rho1'   , format=EOS%number_format)
call uio_wr(ncerr, delta1 , 'delta1' , format=EOS%number_format)
call uio_wr(ncerr, Gamma11, 'gamma11', format=EOS%number_format)
call uio_wr(ncerr, cpd1   , 'cpd1'   , format=EOS%number_format)
call uio_wr(ncerr, p2     , 'p2'     , format=EOS%number_format)
call uio_wr(ncerr, t2     , 't2'     , format=EOS%number_format)
call uio_wr(ncerr, s2     , 's2'     , format=EOS%number_format)
call uio_wr(ncerr, h2     , 'h2'     , format=EOS%number_format)
call uio_wr(ncerr, rho2   , 'rho2'   , format=EOS%number_format)
call uio_wr(ncerr, delta2 , 'delta2' , format=EOS%number_format)
call uio_wr(ncerr, Gamma12, 'gamma12', format=EOS%number_format)
call uio_wr(ncerr, cpd2   , 'cpd2'   , format=EOS%number_format)
call uio_wr(ncerr, p3     , 'p3'     , format=EOS%number_format)
call uio_wr(ncerr, t3     , 't3'     , format=EOS%number_format)
call uio_wr(ncerr, s3     , 's3'     , format=EOS%number_format)
call uio_wr(ncerr, h3     , 'h3'     , format=EOS%number_format)
call uio_wr(ncerr, rho3   , 'rho3'   , format=EOS%number_format)
call uio_wr(ncerr, delta3 , 'delta3' , format=EOS%number_format)
call uio_wr(ncerr, Gamma13, 'gamma13', format=EOS%number_format)
call uio_wr(ncerr, cpd3   , 'cpd3'   , format=EOS%number_format)
!
! --- Close file ---
call uio_closwr(ncerr)
!
! --- Deallocate arrays ---
deallocate(P , T , rho , cpd , Gamma1 , delta , h , s , cs , e )
deallocate(P1, T1, rho1, cpd1, Gamma11, delta1, h1, s1, cs1, e1)
deallocate(P2, T2, rho2, cpd2, Gamma12, delta2, h2, s2, cs2, e2)
deallocate(P3, T3, rho3, cpd3, Gamma13, delta3, h3, s3, cs3, e3)
deallocate(lnx11d_h, x11d_h)
deallocate(lnx21d_h, x21d_h)
!
end subroutine gasinter_CheckGas


!----------*******************----------------------------------------------------------------------
subroutine gasinter_Checkgamma(ncpri, nx1_check0,nx2_check0)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_Checkgamma ('gas_interpolation_check_gamma')
!
! PURPOSE:
!   Check accuracy of interpolation table for gamma law.
!
! CATEGORY:
!   Interpolation, EOS
!
! CALLING SEQUENCE:
!   call gasinter_Checkgamma(ncpri, nx1_check0,nx2_check0)
!
! INPUT:
!   ncpri:        ('number_channel_print') integer, logical output channel number
!   nx1_check0:   ('number_x_1_check_0') integer, number of test points in
!                 x1 direction: 0 -> nx1_check=EOS%nx1-3
!   nx2_check0:   ('number_x_2_check_0') integer, number of test points in
!                 x2 direction: 0 -> nx2_check=EOS%nx2-3
!
! OUTPUT:
!   None
!
! ROUTINES:
!   gasinter:              Solve EOS by bicubic interpolation: P  , T -> rho, h, ...
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   uio_closwr:            Close file after writing
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   Text is printed on channel ncpri.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For a grid of specified number of elements solve original gas equation
!   and interpolate with new table.
!   Measure time and compare results.
!   Print results on channel ncpri.
!
! EXAMPLE:
!   call gasinter_Checkgamma(6, 20,30)
!
! MODIFICATION HISTORY:
!   1997-10-15 (B.F. Kiel) Written
!   2013-02-15 (B.F. Lyon) Removed ierr parameter from eosinter call
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
use gasinter_module
use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncpri, nx1_check0,nx2_check0
!
! --- Local variables ---
character(len=10)                       :: time
integer                                 :: ierror, ifent, kenter, &
                                           nx1_check, nx2_check
real, allocatable, dimension(:,:)       :: P,  T, &
                                           rho,  cpd,  Gamma1,  delta,  h, s, &
                                           cs, e
real, allocatable, dimension(:,:)       :: P1, T1, &
                                           rho1, cpd1, Gamma11, delta1, h1,s1, &
                                           cs1, e1, &
                                           dPdrho1, dPde1, dTde1
real, allocatable, dimension(:)         :: lnx11d_h, x11d_h
real, allocatable, dimension(:)         :: lnx21d_h, x21d_h
integer                                 :: ix1, ix2, ncerr
real, parameter                         :: Rgas=8.3145E+07, s_sigma=0.0
!---------------------------------------------------------------------------------------------------
!
! --- Test if default array size is to be used ---
if (nx1_check0 <= 0) then
  nx1_check=EOS%nx1-3
else
  nx1_check=nx1_check0
endif
if (nx2_check0 <= 0) then
  nx2_check=EOS%nx2-3
else
  nx2_check=nx2_check0
endif
!
! --- Allocate arrays ---
allocate(     P(nx1_check,nx2_check),     T(nx1_check,nx2_check), &
            rho(nx1_check,nx2_check),   cpd(nx1_check,nx2_check), &
         Gamma1(nx1_check,nx2_check), delta(nx1_check,nx2_check), &
              h(nx1_check,nx2_check),     s(nx1_check,nx2_check), &
             cs(nx1_check,nx2_check),     e(nx1_check,nx2_check))
allocate(     P1(nx1_check,nx2_check),     T1(nx1_check,nx2_check), &
            rho1(nx1_check,nx2_check),   cpd1(nx1_check,nx2_check), &
         Gamma11(nx1_check,nx2_check), delta1(nx1_check,nx2_check), &
              h1(nx1_check,nx2_check),     s1(nx1_check,nx2_check), &
             cs1(nx1_check,nx2_check),     e1(nx1_check,nx2_check), &
         dPdrho1(nx1_check,nx2_check),  dPde1(nx1_check,nx2_check), &
           dTde1(nx1_check,nx2_check) )
allocate(lnx11d_h(nx1_check), x11d_h(nx1_check))
allocate(lnx21d_h(nx2_check), x21d_h(nx2_check))
!
! --- Array with test values ---
lnx11d_h=EOS%lnx1off + 1.5/EOS%lnx1fac + &
         (/(ix1,ix1=0,nx1_check-1)/)/real(nx1_check-1)* &
         (alog(EOS%x1max+EOS%x1_shift)-EOS%lnx1off-3.0/EOS%lnx1fac)
x11d_h=exp(lnx11d_h)-EOS%x1_shift
lnx21d_h=EOS%lnx2off + 1.5/EOS%lnx2fac + &
         (/(ix2,ix2=0,nx2_check-1)/)/real(nx2_check-1)* &
         (alog(EOS%x2max+EOS%x2_shift)-EOS%lnx2off-3.0/EOS%lnx2fac)
x21d_h=exp(lnx21d_h)-EOS%x2_shift
!
if (EOS%variables == 'lnP,lnT->lnrho,h,s') then
  !
  ! --- Arrays of independent variables ---
  P=spread(x11d_h,2,nx2_check)
  T=spread(x21d_h,1,nx1_check)
  P1=P
  T1=T
  !
  ! --- Compute GAS quantities with new GASINTER routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of gasinter0:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      call gasinter(P1(ix1,ix2), T1(ix1,ix2), &
                    rho1(ix1,ix2), cpd1(ix1,ix2), Gamma11(ix1,ix2), &
                    delta1(ix1,ix2), h1(ix1,ix2), s1(ix1,ix2), cs1(ix1,ix2), &
                    ierror, ifent)
      e1(ix1,ix2)=h1(ix1,ix2)-P1(ix1,ix2)/rho1(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after gasinter0:                ' // time
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of gasinter1:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    !DIR$ INLINE
    call gasinter(P1(:,ix2), T1(:,ix2), &
                  rho1(:,ix2), cpd1(:,ix2), Gamma11(:,ix2), &
                  delta1(:,ix2), h1(:,ix2), s1(:,ix2), cs1(:,ix2), &
                  ierror, ifent)
    e1(:,ix2)=h1(:,ix2)-P1(:,ix2)/rho1(:,ix2)
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after gasinter1:                ' // time
  !
  ! --- Compute GAS quantities with Gamma-law ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of Gamma-law:      ' // time
  !
  ! --- Solve EOS for simple Gamma-law ---
  rho=P/((Rgas/EOS%Qmol)*T)
  cpd=EOS%Gamma/(EOS%Gamma-1.0) * Rgas/EOS%Qmol
  Gamma1=EOS%Gamma
  delta=1.0
  h=Rgas/EOS%Qmol/(EOS%Gamma-1.0)*T + P/rho
  s=Rgas/EOS%Qmol * (1.0/(EOS%Gamma-1.0)*alog(T) - alog(rho) + s_sigma)
  cs=sqrt(EOS%Gamma*P/rho)
  e=h-P/rho
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GAS:                      ' // time
  !
else if (EOS%variables == 'lnrho,lne->s,lnP,lnT') then
  !
  ! --- Arrays of independent variables ---
  rho1=spread(x11d_h,2,nx2_check)
  e1  =spread(x21d_h,1,nx1_check)
  !
  ! --- Compute EOS quantities with new EOSINTER routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of eosinter0:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    do ix1=1,nx1_check
      !DIR$ INLINE
      call eosinter(rho1(ix1,ix2), e1(ix1,ix2), &
                    P=P1(ix1,ix2), dPdrho=dPdrho1(ix1,ix2), &
                                   dPde=dPde1(ix1,ix2), &
                    T=T1(ix1,ix2), dTde=dTde1(ix1,ix2), &
                    s=s1(ix1,ix2))
      cpd1(ix1,ix2)=P1(ix1,ix2)/(rho1(ix1,ix2)*T1(ix1,ix2)) + &
                    rho1(ix1,ix2)*dPdrho1(ix1,ix2)/(T1(ix1,ix2)*dPde1(ix1,ix2))
      Gamma11(ix1,ix2)=T1(ix1,ix2)*cpd1(ix1,ix2)*dPde1(ix1,ix2)/P1(ix1,ix2)
      delta1(ix1,ix2) =1.0/(cpd1(ix1,ix2)*dTde1(ix1,ix2)- &
                           (Gamma11(ix1,ix2)*P1(ix1,ix2))/ &
                           (T1(ix1,ix2)*rho1(ix1,ix2)*cpd1(ix1,ix2)))
      cs1(ix1,ix2)=sqrt(Gamma11(ix1,ix2)*P1(ix1,ix2)/rho1(ix1,ix2))
      h1(ix1,ix2)=e1(ix1,ix2)+P1(ix1,ix2)/rho1(ix1,ix2)
    end do ! ix1
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after eosinter0:                ' // time
  !
  ! --- Compute EOS quantities with new EOSINTER1 routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of eosinter1:      ' // time
  !
  ifent=1
  do ix2=1,nx2_check
    !DIR$ INLINE
    call eosinter(rho1(:,ix2), e1(:,ix2), &
                  P=P1(:,ix2), dPdrho=dPdrho1(:,ix2), dPde=dPde1(:,ix2), &
                  T=T1(:,ix2), dTde=dTde1(:,ix2), &
                  s=s1(:,ix2))
    cpd1(:,ix2)=P1(:,ix2)/(rho1(:,ix2)*T1(:,ix2)) + &
                  rho1(:,ix2)*dPdrho1(:,ix2)/(T1(:,ix2)*dPde1(:,ix2))
    Gamma11(:,ix2)=T1(:,ix2)*cpd1(:,ix2)*dPde1(:,ix2)/P1(:,ix2)
    delta1(:,ix2) =1.0/(cpd1(:,ix2)*dTde1(:,ix2)- &
                         (Gamma11(:,ix2)*P1(:,ix2))/ &
                         (T1(:,ix2)*rho1(:,ix2)*cpd1(:,ix2)))
    cs1(:,ix2)=sqrt(Gamma11(:,ix2)*P1(:,ix2)/rho1(:,ix2))
    h1(:,ix2)=e1(:,ix2)+P1(:,ix2)/rho1(:,ix2)
  end do ! ix2
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after eosinter1:                ' // time
  !
  P=P1
  T=T1
  !
  ! --- Compute EOS quantities with old GAS routine ---
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time before start of Gamma-law:      ' // time
  !
  ! --- Solve EOS for simple Gamma-law ---
  rho=P/((Rgas/EOS%Qmol)*T)
  cpd=EOS%Gamma/(EOS%Gamma-1.0) * Rgas/EOS%Qmol
  Gamma1=EOS%Gamma
  delta=1.0
  h=Rgas/EOS%Qmol/(EOS%Gamma-1.0)*T + P/rho
  s=Rgas/EOS%Qmol * (1.0/(EOS%Gamma-1.0)*alog(T) - alog(rho) + s_sigma)
  cs=sqrt(EOS%Gamma*P/rho)
  e=h-P/rho
  !
  call date_and_time(time=time)
  write(ncpri,'(A)') 'Time after GAS:                      ' // time
  !
endif
!
! --- Results ---
write(ncpri,*)
write(ncpri,'(A)') 'Ranges of variables'
write(ncpri,'(A)') '-------------------'
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   P: ', minval(P),   maxval(P)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   T: ', minval(T),   maxval(T)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range: rho: ', minval(rho), maxval(rho)
write(ncpri,'(A,E13.6,'' ...'',E13.6)') 'Range:   e: ', minval(e),   maxval(e)
write(ncpri,'(A)') 'Accuracy of interpolation'
write(ncpri,'(A)') '-------------------------'
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    s: ', &
  maxval(abs(   s-   s1))/maxval(abs(s))
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    h: ', &
  maxval(abs(   h-   h1))/maxval(abs(h))
write(ncpri,'(A,F13.8)') 'Maximum (relative) deviation:    e: ', &
  maxval(abs(   e-   e1))/maxval(abs(e))
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:      P: ', &
  maxval(abs(   P-   P1)/  P)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:      T: ', &
  maxval(abs(   T-   T1)/  T)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:    rho: ', &
  maxval(abs( rho- rho1)/rho)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:    cpd: ', &
  maxval(abs( cpd- cpd1)/cpd)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:  delta: ', &
  maxval(abs( delta- delta1)/delta)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation: Gamma1: ', &
  maxval(abs(Gamma1-Gamma11)/Gamma1)
write(ncpri,'(A,F13.8)') 'Maximum relative deviation:     cs: ', &
  maxval(abs(cs-cs1)/cs)
write(ncpri,'(A,F13.8)') 'RMS     relative deviation:    rho: ', &
  sqrt(sum(( rho- rho1)**2/rho**2)/real(nx1_check*nx2_check))
write(ncpri,'(A,F13.8)') 'RMS     relative deviation:    cpd: ', &
  sqrt(sum(( cpd- cpd1)**2/cpd**2)/real(nx1_check*nx2_check))
!
! --- Write original and interpolated arrays into file ---
call uio_openwr(ncerr, 'gas2eos_err.dat', form=EOS%form, conv=EOS%conv)
!
! --- Write file_id ---
call uio_wr(ncerr, 'eos-tables: error', 'file_id', name='File identification')
!
! --- Write all arrays ---
call uio_wr(ncerr, p      , 'p'      , format=EOS%number_format)
call uio_wr(ncerr, t      , 't'      , format=EOS%number_format)
call uio_wr(ncerr, s      , 's'      , format=EOS%number_format)
call uio_wr(ncerr, h      , 'h'      , format=EOS%number_format)
call uio_wr(ncerr, rho    , 'rho'    , format=EOS%number_format)
call uio_wr(ncerr, delta  , 'delta'  , format=EOS%number_format)
call uio_wr(ncerr, Gamma1 , 'gamma1' , format=EOS%number_format)
call uio_wr(ncerr, cpd    , 'cpd'    , format=EOS%number_format)
call uio_wr(ncerr, p1     , 'p1'     , format=EOS%number_format)
call uio_wr(ncerr, t1     , 't1'     , format=EOS%number_format)
call uio_wr(ncerr, s1     , 's1'     , format=EOS%number_format)
call uio_wr(ncerr, h1     , 'h1'     , format=EOS%number_format)
call uio_wr(ncerr, rho1   , 'rho1'   , format=EOS%number_format)
call uio_wr(ncerr, delta1 , 'delta1' , format=EOS%number_format)
call uio_wr(ncerr, Gamma11, 'gamma11', format=EOS%number_format)
call uio_wr(ncerr, cpd1   , 'cpd1'   , format=EOS%number_format)
!
! --- Close file ---
call uio_closwr(ncerr)
!
! --- Deallocate arrays ---
deallocate(P, T, rho, cpd, Gamma1, delta, h, s, cs, e)
deallocate(P1, T1, rho1, cpd1, Gamma11, delta1, h1, s1, cs1, e1)
deallocate(lnx11d_h, x11d_h)
deallocate(lnx21d_h, x21d_h)
!
end subroutine gasinter_Checkgamma


!----------**************---------------------------------------------------------------------------
subroutine gasinter_Check(ncpri, nx1_check0,nx2_check0)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_Check ('gas_interpolation_Check')
!
! PURPOSE:
!   Check accuracy of interpolation table.
!
! CATEGORY:
!   Interpolation, EOS
!
! CALLING SEQUENCE:
!   call gasinter_Check(ncpri, nx1_check0,nx2_check0)
!
! INPUT:
!   ncpri:        ('number_channel_print') integer, logical output channel number
!   nx1_check0:   ('number_x_1_check_0') integer, number of test points in x1 direction:
!                 0 -> nx1_check=EOS%nx1-3
!   nx2_check0:   ('number_x_2_check_0') integer, number of test points in x2 direction:
!                 0 -> nx2_check=EOS%nx2-3
!
! OUTPUT:
!   None
!
! ROUTINES:
!   gasinter_CheckGas:     Check accuracy of interpolation table for GAS quantities
!   gasinter_Checkgamma:   Check accuracy of interpolation table for gamma law
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!
! SIDE EFFECTS:
!   Text is printed on channel ncpri.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For a grid of specified number of elements solve original gas equation
!   and interpolate with new table.
!   Measure time and compare results.
!   Print results on channel ncpri.
!
! EXAMPLE:
!   call gasinter_Check(6, 20,30)
!
! MODIFICATION HISTORY:
!   1997-10-15 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncpri, nx1_check0,nx2_check0
!
!---------------------------------------------------------------------------------------------------
!
select case (EOS%source)
  case ('gasgas', 'GASGAS', 'GasGas', 'Gasgas')
    call gasinter_CheckGas(ncpri, nx1_check0,nx2_check0)
  case ('gamma', 'GAMMA', 'Gamma')
    call gasinter_Checkgamma(ncpri, nx1_check0,nx2_check0)
  case default
    stop 'Unknown source type'
end select
!
end subroutine gasinter_Check

end module gasinter_setup_module


!-------*******-------------------------------------------------------------------------------------
program gas2eos
!---------------------------------------------------------------------------------------------------
! NAME:
!   gas2eos ('gas_to_equation_of_state')
!
! PURPOSE:
!   Put EOS from GAS-routine into table.
!
! CATEGORY:
!   EOS, Interpolation
!
! LOCAL VARIABLES:
!   file:         ('file') character, output file name
!   ncpri:        ('number_channel_print') integer, logical number for output
!   parfile:      ('parameter_file') character, parameter file name
!   time:         ('time') character, time string
!
! ROUTINES:
!   tabinter_rdcoeff:      Read coefficients for bicubic interpolation of EOS quantities from file
!   gasinter_SetUp:        Compute coefficients for bicubic interpolation of GAS/EOS quantities
!   tabinter_wrcoeff:      Write coefficients for bicubic interpolation of GAS quantities into file
!   gasinter_ReadPar:      Read parameters from input parameter file
!   uio_init:              Initialization of UIO routines
!
! MODULES:
!   uio_bulk_module:       Main set of UIO routines
!   tabinter_module:       Routines for handling of interpolation coefficients
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   gasinter_setup_module: Routines for setting up coefficients for EOS interpolation routine
!
! PROCEDURE:
!   Read parameter file,
!   derive interpolation coefficients for EOS table in form (P,T->... or rho,e->...),
!   write result into file,
!   clear EOS table,
!   read interpolation coefficients back from file,
!   compare original and interpolated EOS.
!
! MODIFICATION HISTORY:
!   1997-02-22 (B.F. Kiel) Written
!   1997-02-24 (B.F.) type (par_type)
!   1997-04-07 (B.F.)
!   1997-04-09 (B.F.) Correction in readpar
!   1997-04-10 (B.F.) Correction
!   1997-04-11 (B.F.) Set ifion=1
!   1997-04-16 (B.F.) Rearrangement; nP, nT increased
!   1997-04-17 (B.F.) Modifications
!   1997-04-28 (B.F.) Everything in global variable par in tabinter_coeff_module
!   1997-04-30 (B.F.)
!   1997-05-06 (B.F.)
!   1997-05-14 (B.F.) More than one parameter set possible
!   1997-05-19 (B.F.) ncpri in call of routines
!   1997-05-23 (B.F.) readpar -> gasinter_ReadPar
!---------------------------------------------------------------------------------------------------
use uio_bulk_module
use tabinter_module
use gasinter_module
use gasinter_setup_module
!
implicit none
!
! --- Variables ---
integer, parameter                      :: ncpri=6
character(len=*), parameter             :: parfile='gas2eos_par.txt'
character(len=80)                       :: file
character(len=10)                       :: time
!---------------------------------------------------------------------------------------------------
!
write(ncpri,*)
write(ncpri,*) ' GAS2EOS'
write(ncpri,*) ' ======='
write(ncpri,*)
!
! --- Initialize uio routines ---
call uio_init(progrm='gas2eos')
!
! --- Set coefficients to zero, deallocate arrays ---
call tabinter_reset(1, ncpri)
!
! --- Read parameter file, result in global variable EOS ---
call gasinter_ReadPar(parfile, ncpri, file)
!
! --- Compute coefficients for later interpolation ---
!
call date_and_time(time=time)
write(ncpri,'(A)') 'Time before start of gasinter_SetUp: ' // time
!
call gasinter_SetUp()
!
call date_and_time(time=time)
write(ncpri,'(A)') 'Time after gasinter_SetUp:           ' // time
!
! --- Write coefficients into file ---
write(ncpri,'(A)') 'Write coefficients into ' // trim(file)
call tabinter_wrcoeff()
!
! --- Select another table, set coefficients to zero, deallocate arrays ---
call tabinter_reset(2, ncpri)
!
! --- Reload coefficients from file ---
call tabinter_rdcoeff(file, ncpri)
!
! --- Check accuracy of interpolation table ---
!call gasinter_Check(ncpri, 0,0)
!call gasinter_Check(ncpri, 191, 7213)
call gasinter_Check(ncpri, 191, 1213)
!
! --- Deallocate tables ---
call tabinter_delete(1, ncpri)
call tabinter_delete(2, ncpri)
!
end program gas2eos
