      SUBROUTINE BROYDN( MDIMB, NX, MIMPS, PRECIS, RNRN, 
     >                   BINV, X, Y, IS, IMPS, ISTAT)
********************************************************************************
*
* Version: 18-MAY-1992 12:42:19
*
* Author : HGL
*
* Description: BROYDN improves the approximate solution of a set of non-linear
*   (linear too) equations 
*   
*   y := F(x) = 0, F: R_n -> R_n is a vector function of vector x.
*   
*   BROYDN improves a given solution x_k (maximum MIMPS times) by computing
*
*   x_k+1 = x_k - y_k * BINV.
*
*   If the prescribed precision of solution x is not reached BINV is modified
*   and a following solution x is computed. BROYDN uses the Broyden method 
*   described in
*   L. Koesterke, W.-R. Hamann, P. Kosmol (1992): 
*   Astron. & Astrophys. Vol. 255, pp. 490-494
* 
*   BROYDN uses BINV, X, Y, IS and IMPS as work area. These variables must
*   correspond to different physical memory locations.
*
*   The FORTRAN formulation coded here interprets vectors as row-vectors. The
*   BINV array indices correspond to mathematical index notation 
*   ( BINV( row index, column index)). This is important if one (like the
*   author) is trying to sort the bunch of different indices correctly. The
*   author found for the BINV update formula (adopting Einstein's summation 
*   convention):
*
*                       (BINV_ik * Dx_k) * (Dx_j - Dy_k * BINV_kj)
*   BINV_ij = BINV_ij + ------------------------------------------,  D = delta.
*                                 Dx_j * Dy_k * BINV_kj
*
* Arguments, access, type, dimension, description:
*   MDIMB  R I : First dimension of BINV array
*   NX     R I : Number of equations actually used
*   MIMPS  R I : Maximum number of improvements of the solution, i.e. 
*                maximum number of iterations
*   PRECIS R D1: Absolute (positive) correction (somewhat like the precision)
*                where the iteration should stop; this value has to be 
*                specified for every vector component
*   RNRN   R E : Name of subroutine which contains the vector function to be 
*                solved, RNRN( NX, <input vector>, <output vector>). Double
*                precision!
*   BINV   * D2: Inverse Broyden matrix. A suitable first approximation must be
*                specified! It is modified during the iteration cycle.
*   X      * D2: X(IS) contains the current solution of the set of equations.
*                A suitable first approximation must be specified!
*                Dimension of X: X( MDIMB, >=2), only two x-vectors are needed!
*                Care should be taken that IS points in the calling routine
*                actually to the correct memory location in array X!
*   Y      * D2: Working area during iteration cycle, dimension Y( MDIMB, >=3)
*                At least three scratch vectors are needed.
*   IS     * I : Pointer to current solution, IS must be 1 or 2!
*   IMPS   W I : Number of improvement steps executed when routine has
*                finished, work variable!
*   ISTAT  W I : Exit status:
*                = +1: Iteration has not converged in MIMPS cycles
*                =  0: Everything fine, iteration has converged
*                = -2: Scalar product in denominator of BINV update formula
*                      equals zero. Perhaps iteration has converged, but the
*                      numerical noise is greater than the precision. Try again
*                      with a somewhat relaxed claim of precision.
*                = -3: Illegal pointer IS encountered!
*                = -999: Semantic error in FORTRAN code, contact the author!
*
* Common blocks (access): None
*
* External procedures   : RNRN (external)
*
* File units (access)   : None
*
*
* Comments: The routine is designed in DOUBLE PRECISION!
*
*   Since a lot of index gymnastics is performed a scetch of the memory
*   management seems to be useful:
*   (here: B=BINV, #=variable is used in current computation)
*
*   ---------------------------------------------------------------------------
*                            X(IS)     X(IW)     Y(IS)     Y(IW)     Y(IW2)
*   ---------------------------------------------------------------------------
*   Start                    x_k       indef     indef     indef     indef
*   ---------------------------------------------------------------------------
*   Compute F(x_k)           #                   y_k
*   ===========================================================================
*   Start iteration          x_k       indef     y_k       indef     indef
*   ---------------------------------------------------------------------------
*   Update solution          #         x_k+1     #
*   ---------------------------------------------------------------------------
*   Exchange IS<->IW         x_k+1     x_k       indef     y_k       indef
*   ---------------------------------------------------------------------------
*   Compute x_k+1-x_k        #         #Dx_k
*   ---------------------------------------------------------------------------
*   Compute F(x_k+1)         #                   y_k+1
*   ---------------------------------------------------------------------------
*   Convergence check
*   ---------------------------------------------------------------------------
*   Compute y_k+1-y_k                            #         #Dy_k
*   ---------------------------------------------------------------------------
*   Compute Dy_k*B_k                                       #         Dy_k*B_k
*   ---------------------------------------------------------------------------
*   Compute Dy_k*B_k*(Dx_k)T           #                             #
*   -> DUM
*   ---------------------------------------------------------------------------
*   Check DUM=0 ?
*   ---------------------------------------------------------------------------
*   Compute B_k*(Dx_k)T/DUM            #                   B_k*(Dx_k)T
*                                                          /DUM
*   ---------------------------------------------------------------------------
*   Compute Dx_k-Dy_k*B_k              #                             #Dx_k-
*                                                                     Dy_k*B_k
*   ---------------------------------------------------------------------------
*   Update B_k                                             #         #
*   ---------------------------------------------------------------------------
*   Interpret k+1 as k       x_k       Dx_k-1    y_k       indef     indef
*   ===========================================================================
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
      DOUBLE PRECISION BINV( MDIMB, *), X( MDIMB, *), Y( MDIMB, *)
      DOUBLE PRECISION PRECIS(*)
      DOUBLE PRECISION R00, DUM
      PARAMETER( R00 = 0.0)
      LOGICAL ISCONV
***   Work indicex IW2 is fixed
      PARAMETER( IW2 = 3)

      EXTERNAL RNRN

***   Preset status indicator
      ISTAT = -999

***   Definition of work indices
***   IS is 1 or 2, IW has to be 2 or 1 resp.
      IF ( IS .EQ. 1) THEN
        IW = 2
      ELSEIF( IS .EQ. 2) THEN
        IW = 1
      ELSE
        ISTAT = -3
        RETURN
      ENDIF

***   Compute y_k
      CALL RNRN( NX, X( 1, IS), Y( 1, IS))


***   Broyden iteration starts here
***   =============================
      DO 900 IMPS = 1, MIMPS

***   Improve solution
      DO 20 I = 1, NX
        DUM = R00
        DO 10 K = 1, NX
          DUM = DUM + Y( K, IS) * BINV( K, I) 
10      CONTINUE
        X( I, IW) = X( I, IS) - DUM
20    CONTINUE

***   Change the role of IS and IW
      IDUM = IW
      IW   = IS
      IS   = IDUM

***   Compute Dx_k
      DO 30 K = 1, NX
        X( K, IW) = X( K, IS) - X( K, IW)
30    CONTINUE

***   Compute y_k+1
      CALL RNRN( NX, X( 1, IS), Y( 1, IS))

***   Convergence check ...
***   Test if correction of solution is smaller than preset precision in every
***   vector component:
      ISCONV = .TRUE.
      DO 40 K = 1, NX
        ISCONV = ISCONV .AND. ( ABS( X( K, IW)) .LT. PRECIS( K))
40    CONTINUE
      IF ( ISCONV) THEN
        ISTAT = 0
        RETURN
      ENDIF

***   Compute Dy_k
      DO 60 K = 1, NX
        Y( K, IW) = Y( K, IS) - Y( K, IW)
60    CONTINUE


***   Update inverse Broyden matrix      
***   -----------------------------

***   Dy_k * BINV_k (stored in the Y scratch area 2)
***   Currently Y scratch area 1 contains Dy_k
      DO 110 I = 1, NX
        DUM = R00
        DO 100 K = 1, NX
          DUM = DUM + Y( K, IW) * BINV( K, I)
100     CONTINUE
        Y( I, IW2) = DUM
110   CONTINUE

***   Compute scalar product (Dy_k * BINV)_k * (Dx_k)T
      DUM = R00
      DO 120 K = 1, NX
        DUM = DUM + X( K, IW) * Y( K, IW2)
120   CONTINUE

***   Scalar product less or equal zero?
***   Probably due to the fact that Dx_k is zero.
      IF ( DUM .EQ. R00) THEN
        ISTAT = -2
        RETURN
      ENDIF

***   Compute BINV_k * (Dx_k)T / scalar product
      DO 140 I = 1, NX
        Y( I, IW) = R00
        DO 130 K = 1, NX
          Y( I, IW) = Y( I, IW) + BINV( I, K) * X( K, IW)
130     CONTINUE
          Y( I, IW) = Y( I, IW) / DUM
140   CONTINUE

***   Compute Dx_k - Dy_k * BINV_k
      DO 150 K = 1, NX
        Y( K, IW2) = X( K, IW) - Y( K, IW2)
150   CONTINUE

***   Update BINV
      DO 170 J = 1, NX
        DO 160 I = 1, NX
          BINV( I, J) = BINV( I, J) + Y( I, IW) * Y( J, IW2)
160     CONTINUE
170   CONTINUE

***   Broyden iteration ends here
***   ===========================
900   CONTINUE

***   Number of Broyden iterations executed, finish!
      IMPS  = MIMPS
      ISTAT = 1
      RETURN

      END
