      SUBROUTINE GAS (XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
C
C***********************************************************************
C
C   IONIZATION BALANCE AND THERMODYNAMIC DERIVATIVES  (B.W., 6.9.1983)
C
C   modified, corrected and converted to CRAY/VAX-FORTRAN  by D.G.
C
C
C   Last Update:    12-AUG-1992 12:23:56 BY H.G.L: Entropy computation 
C                   corrected. (Domains 1, 6 and 7)
C
C                   3-APR-1992 13:08:41 by M.S.
C
C                   14-Nov-1994 19:08:45: common block / EE / added by B.F.
C
C***********************************************************************
C
C-----------------------------------------------------------------------
C   This subroutine is based on  IONIS.FOR   by B. E. Wolf (Heidelberg)
C   Astron. Astrophys. 127, 93 (1983); 130, 202 (1984); 145, 278 (1985)
C-----------------------------------------------------------------------
C
C   The subroutine computes thermodynamic quantities and derivatives
C   for given temperature T and gas pressure p
C
C   Ions: H2, H, H+, He, He+, He++, M (representative metal, here Mg;
C         the metal is assumed neutral throughout the subroutine)
C
C   Each temperature section (see below) is divided in 3 subsections:
C      1.: computation of density and energy
C      2.: computation of entropy (omitted if IFENTR = 0)
C      3.: computation of derivatives (omitted if IFDERI = 0)
C
C   Warning: do not use the subroutine for temperatures less than 1500 K
C           log10 (gas pressure) must be greater than -4 and less than 8
C-----------------------------------------------------------------------
C
C   Input parameters are
C       XP     = gas pressure [dyn/cm2]
C       XT     = temperature [K]
C
C   Control parameters are
C       IFION  - determines how to treat ionization processes [0...16]
C       IFENT  - entropy is computed if = 1
C       KENTER - must be < 0 in initial call, otherwise >= 0
C                   computational domain determined by GAS if = 0
C                   computational domain KENTER assumed if > 0
C
C   Output parameters are
C       RHO    = density [g/cm3]
C       XCP    = Cp' = d(S) / d(ln rho) [erg/g/K]
C       XGAM   = gamma' = Cp' / Cv'
C       CXT    = alpha * T = (-1/rho) * d(rho)/d(T) * T
C       ENTALP = enthalpy per mass unit [erg/g/K]
C       S      = entropy per mass unit [erg/g/K]
C       CS     = adiabatic sound velocity [cm/s]
C       IERROR = error indicator (= 0 if no errors encountered)
C       KENTER = index of computational domain
C
C
C***********************************************************************
C
      DIMENSION BORDER (12,4,3), TT (4,3)
C
C
C-----------------------------------------------------------------------
C   COMMON block / GFACT / is not shared by other routines. It is only
C   used to ensure a proper placement of the 'G' variables.
C   Attention: Do not change variable arrangement within GFACT !!!
C-----------------------------------------------------------------------
C
      COMMON / GFACT / G1,   G13,  G25,  G37,  G49,  G61,  G73,  G85,
     >                 G97,  G109, G121, G133, G4,   G16,  G28,  G40,
     >                 G52,  G64,  G76,  G88,  G100, G112, G124, G136,
     >                 G7,   G19,  G31,  G43,  G55,  G67,  G79,  G91,
     >                 G103, G115, G127, G139, G10,  G22,  G34,  G46,
     >                 G58,  G70,  G82,  G94,  G106, G118, G130, G142,
     >                 G2,   G14,  G26,  G38,  G50,  G62,  G74,  G86,
     >                 G98,  G110, G122, G134, G5,   G17,  G29,  G41,
     >                 G53,  G65,  G77,  G89,  G101, G113, G125, G137,
     >                 G8,   G20,  G32,  G44,  G56,  G68,  G80,  G92,
     >                 G104, G116, G128, G140, G11,  G23,  G35,  G47,
     >                 G59,  G71,  G83,  G95,  G107, G119, G131, G143,
     >                 G3,   G15,  G27,  G39,  G51,  G63,  G75,  G87,
     >                 G99,  G111, G123, G135, G6,   G18,  G30,  G42,
     >                 G54,  G66,  G78,  G90,  G102, G114, G126, G138,
     >                 G9,   G21,  G33,  G45,  G57,  G69,  G81,  G93,
     >                 G105, G117, G129, G141, G12,  G24,  G36,  G48,
     >                 G60,  G72,  G84,  G96,  G108, G120, G132, G144
C
      common / EE / anhn, anhp, anhen, anhep, anhepp, ct
C
C
C-----------------------------------------------------------------------
C   Definitions of basic parameters
C-----------------------------------------------------------------------
C
      PARAMETER ( EPSIT = 1.0E-05, HLIMIT = 1.0E-02)
C
      PARAMETER ( A1 = 5.797083E-03, A2 = 4.676132E-08,
     >            A3 = 1.088582E-11 )
C
      PARAMETER ( QCON1 =  9.55469E-01, QCON2 = 1.039473E-04,
     >            QCON3 = -7.19848E-08, QCON4 = 1.154352E-11,
     >            QCON5 = -6.403472E-16 )
C
      PARAMETER ( QQ0 = 0.0, QQ1 = 1.0, QQ2 = 2.0, QQ3 = 3.0, QQ4 = 4.0,
     >            QQ5 = 5.0, QQ6 = 6.0 )
C
      PARAMETER ( Q04 = 0.4, QI2 = 0.5, Q15 = 1.5, Q25 = 2.5, Q35 = 3.5,
     >            Q45 = 4.5, QI4 = 0.25)
C
      PARAMETER ( QLM4 = 1.0E-04, QLM3 = 1.0E-03, QLM2 = 1.0E-02,
     >            QLM1 = 1.0E-01, QLP1 = 1.0E+01, QLP2 = 1.0E+02,
     >            QLP3 = 1.0E+03, QLP4 = 1.0E+04, QLP5 = 1.0E+05,
     >            QLP6 = 1.0E+06, QLP7 = 1.0E+07, QMAX = 1.0E+10 )
C
      PARAMETER ( Q13 = 1.3125E+01, Q14 = -1.375E+01, Q17 = 1.95E+01,
     >            Q37 =   3.75E+00, Q52 =  5.0E+02 )
C
      PARAMETER ( QF0 =  2.7E+01, QF1 =  4.0E+01, QF2 = -1.5E+01,
     >            QF3 = -1.4E+02, QF4 = 1.65E+02, QF5 = -2.5E+01,
     >            QF6 = -1.5E+02, QF7 =  5.0E+01 )
C
      PARAMETER ( QT1 = 2.5E+03, QT2 = 1.1E+04, QT3 =  5.0E+03,
     >            QT4 = 8.0E+03, QT5 = 2.0E+04, QV1 = 9.14E+03,
     >            QV2 = 1.53E+04 )
C
      PARAMETER ( QPOT = -8.7179E-11,   QPOT1 = 2.4587E+01,
     >            QPOT2 = 3.830669E+01, QPOT3 = 5.4416E+01,
     >            QPOT4 = 2.853292E+05, QPOT5 = 5.706584E+05,
     >            QPOT6 = 1.3598E+01,   QPOT7 = 3.692039E+01,
     >            QPOT8 = 1.578032E+05, QPOT9 = 3.156063E+05,
     >            QPOT10 = 6.317E+05,   QPOT11 = 1.263E+06 )
C
      PARAMETER ( BK =    1.38054E-16,   QH = 6.6256E-27,
     >            QMASS = 1.6604E-24,  QGAS = 8.3144E+07,
     >            QQQ =   1.6021E-12 )
C
      PARAMETER ( QPE = 1.8228E+03, QBKT = 1.160488E+04 )
C
      PARAMETER ( QPI2 = 6.283185307179586E+00, QEL = 6.6531E-25 )
C
      PARAMETER ( QHYD = 1.0080, QHEL = 4.0026 )
C
      PARAMETER ( POTH = 2.1785E-11, POTHE = 3.9391E-11,
     >            POTHI = 1.2657E-10 )
C
      PARAMETER ( Q2P1 = 5.19678E+04, Q2P2 = 6.33262E+03,
     >            Q2P3 = -4.7041E+01 )
C
C
C-----------------------------------------------------------------------
C   Save computed quantities for subsequent calls of subroutine
C-----------------------------------------------------------------------
C
      SAVE WGHTMT, Z, ELCON, ALG2, RO
C
      SAVE C0, C1, C2, C3, C4, C5, C6, C7, C8, C9, C11, C12, C13,
     >     C14, C15, C16, C17, C21, C22, C23, C30, C31, C32, C33
C
      SAVE D0, D1, D2, D3, D4, D5, D6, D7, D8, D9, D10
C
C
C-----------------------------------------------------------------------
C   Map BORDER array onto 'G' constants
C-----------------------------------------------------------------------
C
      EQUIVALENCE (G1, BORDER (1,1,1))
C
C
C-----------------------------------------------------------------------
C   Definition of control variables which are to remain unchanged
C-----------------------------------------------------------------------
C
C-----------------------------------------------------------------------
C   Error counter set equal to zero at beginning
C-----------------------------------------------------------------------
C
      IERROR = 0
C
C-----------------------------------------------------------------------
C   Input variables which may be changed are shifted to local variables
C-----------------------------------------------------------------------
C
      P = XP
      T = XT
      IFENTR = IFENT
C
C
C-----------------------------------------------------------------------
C   Computation of auxiliary variables in first GAS call (KENTER < 0)
C-----------------------------------------------------------------------
C
      KP = 0
C
      IF (KENTER .GE. 0) GOTO 777
      IFENTR = 0
C
      T      = 5000.0
      P      = 1000.0
C
      TINV   = QQ1 / T
      T2INV  = TINV**2
      SQRTT  = SQRT(T)
      DNDP   = TINV / BK
      BKEVTI = DNDP * QQQ
      ANTOT  = P * DNDP
      DNDT   =  - ANTOT * TINV
C
      Z = AMAX1(1.0E-25, QQ1 - Y - X)
      WGHTMT = (QMOL - QHYD * X - QHEL * Y) / Z
C
      C0 = X * POTH + Y * POTHI
      C1 = QI2 / (QQ2 - Z)
      C2 = QQ1 + X
      C3 = QQ1 + Y
      C4 = QQ2 - Z
      C5 = QQ1 - Z
      C6 = QQ1 + Y - Z
      C7 = C6 + QQ1
      C8 = QQ1 / C7
      C9 = QQ1 / (QQ2 - X)
      C11 = (X + X) * (QQ4 - X - X)
      C12 = QI2 * X
      C13 = QI2 * C9
      C14 = C5 / C4
      C16 = QI2 * C2
      C15 = C16 * C16
      C17 = (C4 + C4) * (C6 + C6)
C
      RO = QMASS * (QHYD * X + QHEL * Y + WGHTMT * Z)
      C30 = C11 * QI2
      C21 = QMASS * QGAS / RO
      C22 = QQ1 + QQ1 / (X + Y + Y)
      C23 = QQ2 / X
      IF (IFION .EQ. 0  .OR.  IFION .EQ. 2  .OR.  IFION .EQ. 15)
     >   C22 = QQ0
      IF (IFION .EQ .0  .OR.  (IFION .GT. 5  .AND.  IFION .NE. 13) )
     >   C23 = QQ0
      ELCON = QEL * (X + Y + Y) / RO
      C31 = RO * C9
      C32 = C17 * QI2
      C33 = C8 * RO
      ALG2 = ALOG (QQ2)
C
      D0 = Q15 * ALOG (QPI2 * QMASS / QH * BK / QH)
      D1 = D0 + Q15 * ALG2
      D2 = D0 + ALG2 + ALG2 + ALG2
      D3 = D0
      IF (WGHTMT .GT. QQ0) D3 = D3 + Q15 * ALOG (WGHTMT)
      D4 = QQ3 * (D0 - QI2 * ALOG(QPE) + ALG2 + ALG2) + ALG2
      D5 = D0 + ALG2
      D6 = D4 - D2 + ALG2
      D7 = D6 - ALG2 * QQ4
      D8 = Q25
      IF (X .GT. QQ0) D8 = D8 + X * (D5 - ALOG(X))
      IF (Y .GT. QQ0) D8 = D8 + Y * (D2 - ALOG(Y))
      IF (Z .GT. QQ0) D8 = D8 + Z * (D3 - ALOG(Z))
      D9 = X + X + Y + Y + Y + Z
      D10 = QQ0
      IF (X .GT. QQ0) D10 = D10 + X * (D7 - ALOG(X*C6))
      IF (Y .GT. QQ0) D10 = D10 + Y * (D4 - ALOG(Y*C6*C6) - 
     >                                 QQ2 * ALG2 - Q15 * ALOG(QPE))
      IF (Z .GT. QQ0) D10 = D10 + Z * (D3 - ALOG(Z))
      IF (Y .EQ. QQ0) RETURN
C
C
C-----------------------------------------------------------------------
C   Determination of boundary lines for different P/T regions
C-----------------------------------------------------------------------
C
 1000 KP = KP + 1
      IF (KP .GT. 12) GOTO 1240
      PP = QLP1**(KP - 4)
      LP = 1
      IF (KP .GT. 1) LP = 2
      IP = LP - 1
C
 1010 IP = IP + 1
      IF (IP .EQ. 4) GOTO 1225
      IF (LP .EQ. 1) GOTO 1020
      DO 1011 I = 1,4
         TT (I,1) = TT (I,3)
 1011 CONTINUE
C
 1020 Y1 = QQ0
      Y2 = QQ0
      P = PP
      IF (IP .EQ. 1) P = QLM1 * P
      IF (IP .EQ. 2) P = Q04 * P
      LOW = 1
      IT = 0
C
 1030 IT = IT + 1
      IF (IT .GT. 5000) GOTO 1215
      T = FLOAT (IT*500 + 500)
      TINV = QQ1 / T
      T2INV = TINV**2
      SQRTT = SQRT(T)
      DNDP = TINV / BK
      BKEVTI = QBKT * TINV
      ANTOT = DNDP * P
      ANTOT2 = ANTOT * QI2
      ANE = QQ0
      ANH2 = QQ0
      ANHEN = QQ0
      ANHEP = QQ0
      ANHEPP = QQ0
      KENTER = LOW
      IBACK = 0
      IF (KENTER .EQ. 0) GOTO 1035
      GOTO (100, 300, 500, 500, 600), KENTER
C
 1035 IF (LOW .EQ. 1) ANHEN = Y * (ANTOT + ANH2)
      IF (LOW .EQ. 2) ANHEN = Y * (ANTOT - ANE)
      IF (LOW .EQ. 3) GOTO 1090
      IF (LOW .EQ. 5) GOTO 1130
      XNH2 = ANH2
      XNHEN = ANHEN
      ANE = QQ0
      ANH2 = QQ0
      ANHEN = QQ0
      ANHEP = QQ0
      ANHEPP = QQ0
      KENTER = LOW + 1
      IBACK = 1
      IF (KENTER .EQ. 0) GOTO 1040
      GOTO (100, 300, 500, 500, 600), KENTER
C
 1040 IF (LOW .EQ. 1) GOTO 1050
      IF (LOW .EQ. 4) GOTO 1110
      stop 'Fehler in Gas-Routine: 1040'
C
 1050 IF (XNH2 .LT. ANE  .OR.  XNH2 .LE. QQ0) GOTO 1060
      Y2 = XNH2
      Y1 = ANE
      GOTO 1030
C
 1060 IF (XNH2 .LE. QQ0  .OR.  IT .EQ. 1) GOTO 1070
      TT (1,IP) = T - Q52 + Q52*(Y2 - Y1) / (ANE + Y2 - Y1 - XNH2)
      GOTO 1080
C
 1070 TT (1,IP) = T
C
 1080 LOW = 3
      GOTO 1030
C
 1090 H1 = SQRT(F2) * SQRT ( F2*C15 + ANTOT*X ) - F2 * C16
      IF ( ABS(ANE-H1) .GT. ABS(ANE-ANTOT2) ) GOTO 1100
      Y1 = ABS(ANE-ANTOT2)
      Y2 = ABS(ANE-H1)
      GOTO 1030
C
 1100 TT (2,IP) = Q52 * ( ABS(ANE-ANTOT2) - Y1) /
     >                  ( Y2 + ABS(ANE-ANTOT2) - Y1- ABS(ANE-H1) ) +
     >            T - Q52
      LOW = 4
      GOTO 1030
C
 1110 IF (XNHEN .LT. ANHEPP) GOTO 1120
      Y3 = XNHEN
      Y5 = ANHEPP
      GOTO 1030
C
 1120 TT (3,IP) = T - Q52 + Q52*(Y3 - Y5) / (ANHEPP + Y3 - Y5 - XNHEN)
      LOW = 5
      GOTO 1030
C
 1130 IF (ANHEP .LT. (QLM4*ANHEPP) ) GOTO 1140
      Y4 = ANHEP
      Y5 = QLM4 * ANHEPP
      GOTO 1030
C
 1140 TT (4,IP) = T - Q52 + Q52*(Y4 - Y5) /
     >                          (QLM4*ANHEPP + Y4 - Y5 - ANHEP)
      GOTO 1010
C
 1200 IF (IBACK .EQ. 0) GOTO 1035
      GOTO 1040
C
 1215 stop 'Fehler in Gas-Routine: 1215'
C
 1225 Q = QQ1 / (QF0 * PP)
      DO 1230 I = 1,4
         BORDER (KP,I,1) = ( QF1 * TT(I,1) + QF2 * TT(I,2) +
     >                    QQ2 * TT(I,3) ) * Q * PP
         BORDER (KP,I,2) = ( QF3 * TT(I,1) + QF4 * TT(I,2) +
     >                    QF5 * TT(I,3) ) * Q
         BORDER (KP,I,3) = ( QLP2 * TT(I,1) + QF6 * TT(I,2) +
     >                    QF7 * TT(I,3) ) * Q / PP
 1230 CONTINUE
      GOTO 1000
C
 1240 P = WGHTMT
      T = ELCON
      RETURN
C
C
C-----------------------------------------------------------------------
C   Compute common thermodynamical quantities (KENTER >= 0)
C-----------------------------------------------------------------------
C
  777 IRUN = 0
      TINV   = QQ1 / T
      T2INV  = TINV**2
      SQRTT  = SQRT(T)
      DNDP   = TINV / BK
      BKEVTI = DNDP * QQQ
      ANTOT  = P * DNDP
      DNDT   =  - ANTOT * TINV
C----      IF (NEWTON .LE. 0) GOTO 99
C----      D2NDT2 =  - (DNDT + DNDT)*TINV
C----      D2NDPT =  - DNDP * TINV
   99 IF (IFION .EQ. 0) GOTO 200
C
C-----------------------------------------------------------------------
C   Jump to appropriate formula in subsequent GAS calls (KENTER > 0)
C-----------------------------------------------------------------------
C
      GOTO (100, 200, 300, 500, 500, 600, 700), KENTER
C
C
C-----------------------------------------------------------------------
C   Recompute KENTER in first call of GAS for grid points (KENTER = 0)
C-----------------------------------------------------------------------
C
      IF (IFION .GT. 12) GOTO 56
C
      IF (P .GE. QLP2) GOTO 12
      IF (P .GE. QQ1)  GOTO 8
      IF (P .GE. QLM2) GOTO 4
      IF (P .GE. QLM3) GOTO 2
      IF (P .LE. QLM4) GOTO 25
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -4 < log p < -3
C++++++++++++++++++++++++++++++++++++++
      T1 = G1 + P * (G2 + P * G3)
      IF (T .LE. T1) GOTO 51
      T3 = G7 + P * (G8 + P * G9)
      IF (T .GE. T3) GOTO 1
      T2 = G4 + P * (G5 + P * G6)
      GOTO 24
    1 T4 = G10 + P * (G11 + P * G12)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -3 < log p < -2
C++++++++++++++++++++++++++++++++++++++
    2 T1 = G13 + P * (G14 + P * G15)
      IF (T .LE. T1) GOTO 51
      T3 = G19 + P * (G20 + P * G21)
      IF (T .GE. T3) GOTO 3
      T2 = G16 + P * (G17 + P * G18)
      GOTO 24
    3 T4 = G22 + P * (G23 + P * G24)
      IF (T .LT. T4) GOTO 54
      GOTO 55
    4 IF (P .GE. QLM1) GOTO 6
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -2 < log p < -1
C++++++++++++++++++++++++++++++++++++++
      T1 = G25 + P * (G26 + P * G27)
      IF (T .LE. T1) GOTO 51
      T3 = G31 + P * (G32 + P * G33)
      IF (T .GE. T3) GOTO 5
      T2 = G28 + P * (G29 + P * G30)
      GOTO 24
    5 T4 = G34 + P * (G35 + P * G36)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -1 < log p < 0
C++++++++++++++++++++++++++++++++++++++
    6 T1 = G37 + P * (G38 + P * G39)
      IF (T .LE. T1) GOTO 51
      T3 = G43 + P * (G44 + P * G45)
      IF (T .GE. T3) GOTO 7
      T2 = G40 + P * (G41 + P * G42)
      GOTO 24
    7 T4 = G46 + P * (G47 + P * G48)
      IF (T .LT. T4) GOTO 54
      GOTO 55
    8 IF (P .GE. QLP1) GOTO 10
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  0  < log p < +1
C++++++++++++++++++++++++++++++++++++++
      T1 = G49 + P * (G50 + P * G51)
      IF (T .LE. T1) GOTO 51
      T3 = G55 + P * (G56 + P * G57)
      IF (T .GE. T3) GOTO 9
      T2 = G52 + P * (G53 + P * G54)
      GOTO 24
    9 T4 = G58 + P * (G59 + P * G60)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +1 < log p < +2
C++++++++++++++++++++++++++++++++++++++
   10 T1 = G61 + P * (G62 + P * G63)
      IF (T .LE. T1) GOTO 51
      T3 = G67 + P * (G68 + P * G69)
      IF (T .GE. T3) GOTO 11
      T2 = G64 + P * (G65 + P * G66)
      GOTO 24
   11 T4 = G70 + P * (G71 + P * G72)
      IF (T .LT. T4) GOTO 54
      GOTO 55
   12 IF (P .GE. QLP4) GOTO 16
      IF (P .GE. QLP3) GOTO 14
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +2 < log p < +3
C++++++++++++++++++++++++++++++++++++++
      T1 = G73 + P * (G74 + P * G75)
      IF (T .LE. T1) GOTO 51
      T3 = G79 + P * (G80 + P * G81)
      IF (T .GE. T3) GOTO 13
      T2 = G76 + P * (G77 + P * G78)
      GOTO 24
   13 T4 = G82 + P * (G83 + P * G84)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +3 < log p < +4
C++++++++++++++++++++++++++++++++++++++
   14 T1 = G85 + P * (G86 + P * G87)
      IF (T .LE. T1) GOTO 51
      T3 = G91 + P * (G92 + P * G93)
      IF (T .GE. T3) GOTO 15
      T2 = G88 + P * (G89 + P * G90)
      GOTO 24
   15 T4 = G94 + P * (G95 + P * G96)
      IF (T .LT. T4) GOTO 54
      GOTO 55
   16 IF (P .GE. QLP6) GOTO 20
      IF (P .GE. QLP5) GOTO 18
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +4 < log p < +5
C++++++++++++++++++++++++++++++++++++++
      T1 = G97 + P * (G98 + P * G99)
      IF (T .LE. T1) GOTO 51
      T3 = G103 + P * (G104 + P * G105)
      IF (T .GE. T3) GOTO 17
      T2 = G100 + P * (G101 + P * G102)
      GOTO 24
   17 T4 = G106 + P * (G107 + P * G108)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +5 < log p < +6
C++++++++++++++++++++++++++++++++++++++
   18 T1 = G109 + P * (G110 + P * G111)
      IF (T .LE. T1) GOTO 51
      T3 = G115 + P * (G116 + P * G117)
      IF (T .GE. T3) GOTO 19
      T2 = G112 + P * (G113 + P * G114)
      GOTO 24
   19 T4 = G118 + P * (G119 + P * G120)
      IF (T .LT. T4) GOTO 54
      GOTO 55
   20 IF (P .GE. QLP7) GOTO 22
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +6 < log p < +7
C++++++++++++++++++++++++++++++++++++++
      T1 = G121 + P * (G122 + P * G123)
      IF (T .LE. T1) GOTO 51
      T3 = G127 + P * (G128 + P * G129)
      IF (T .GE. T3) GOTO 21
      T2 = G124 + P * (G125 + P * G126)
      GOTO 24
   21 T4 = G130 + P * (G131 + P * G132)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +7 < log p < +8
C++++++++++++++++++++++++++++++++++++++
   22 T1 = G133 + P * (G134 + P * G135)
      IF (T .LE. T1) GOTO 51
      T3 = G139 + P * (G140 + P * G141)
      IF (T .GE. T3) GOTO 23
      T2 = G136 + P * (G137 + P * G138)
      GOTO 24
   23 T4 = G142 + P * (G143 + P * G144)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
   24 IF (T .LT. (T1+T1)) GOTO 52
      KENTER = 4
      IF (T .GE. T2) KENTER = 5
      GOTO 53
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  log p < -4
C++++++++++++++++++++++++++++++++++++++
   25 IF (T .LE. QT1) GOTO 51
      IF (T .GE. QT2) GOTO 26
      IF (T .LT. QT3) GOTO 52
      KENTER = 4
      IF (T .GE. QT4) KENTER = 5
      GOTO 53
   26 IF (T .LT. QT5) GOTO 54
      GOTO 55
C
   51 GOTO (100, 100, 100, 100, 100, 300, 300, 300, 300, 500, 600, 700),
     >     IFION
   52 GOTO (300, 100, 300, 300, 300, 300, 300, 300, 300, 500, 600, 700),
     >     IFION
   53 GOTO (500, 100, 300, 500, 500, 300, 500, 500, 500, 500, 600, 700),
     >     IFION
   54 GOTO (600, 100, 300, 500, 600, 300, 500, 600, 600, 600, 600, 700),
     >     IFION
   55 GOTO (700, 100, 300, 500, 600, 300, 500, 600, 700, 700, 700, 700),
     >     IFION
   56 IF (IFION .EQ. 16) GOTO 700
      IF (IFION .EQ. 15) GOTO 200
      GOTO 300
C
C
C-----------------------------------------------------------------------
C   Low temperature: The atmosphere consists of H2, H, He, M
C-----------------------------------------------------------------------
C
  100 SQRTTI=TINV/SQRTT
      PAR=QCON1+T*(QCON2+T*(QCON3+T*(QCON4+T*QCON5)))
      IF(PAR.LE.QQ0.AND.IFION.NE.2.AND.KP.EQ.0) GOTO 300
      IF(PAR.LE.QQ0.AND.IFION.EQ.2.AND.KP.EQ.0) GOTO 200
      PAR=QQ1/PAR
      H1=Q2P1*TINV
      H2=Q2P2*TINV
      PARM=(PAR-QQ1)*H2
      EFKT=EXP(H1+Q2P3)
      F11=T*(A1+T*(A2+T*A3))
      QH2=PAR*F11
      F1=QH2*SQRTTI*EFKT
      F1INV=QQ1/F1
      H3=C13*F1INV
      C11F1A = C11 * F1 * ANTOT
      W = SQRT( QQ1 + C11F1A )
      IF (C11F1A .LE. HLIMIT) SQRTM1 = C11F1A * QI2 *
     >                       (QQ1 - C11F1A * QI4 * (QQ1 - C11F1A * QI2))
      IF (C11F1A .GT. HLIMIT) SQRTM1 = W - QQ1
      ANHN = SQRTM1 * F1INV * C13
      IF (IFION .EQ .13  .AND.  (F1*ANHN) .LE. QLM3  .AND.  KP .EQ. 0)
     >   GOTO 200
      ANH2=F1*ANHN*ANHN
      ANA=(ANTOT+ANTOT-ANHN)*C9
      ANHEN=ANA*Y
      ANMN=ANA*Z
      ANE=QQ0
      ANHP=QQ0
      ANHEP=QQ0
      ANHEPP=QQ0
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      DF11=A1+T*(A2+A2+(A3+A3+A3)*T)
      DPR=PAR*PARM*TINV
      DQH2=DPR*F11+DF11*PAR
      DF1DT=EFKT*SQRTTI*TINV*(T*DQH2-QH2*(H1+Q15))
      C=DF1DT*F1INV
      F2=(T*C+Q15)*T*BK
      ENERGY=(Q15*P+ANH2*F2)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=1
C
      IF(IFENTR.EQ.0) GOTO 150
      S = ANTOT * ( Q25 + Q15 * ALOG(T) ) -
     >    ANH2 * (ALOG(ANH2/(QH2*QH2)) - T / QH2 * DQH2 - D1) -
     >    ANHN * (ALOG(ANHN) - D5)
      IF (ANHEN .NE. QQ0) S = S - ANHEN * (ALOG(ANHEN) - D2)
      IF (ANMN .NE. QQ0) S = S - ANMN * (ALOG(ANMN) - D3)
      S = BK * RHOINV * S
C
  150 CONTINUE
      D2PAR=TINV*((PAR+PAR-QQ1)*DPR*H2-(PAR+PAR)*TINV*PARM)
      D2F11=QQ2*(A2+(A3+A3+A3)*T)
      D2QH2=D2PAR*F11+(DPR+DPR)*DF11+PAR*D2F11
      DF1DT2=TINV*(EFKT*SQRTTI*(T*D2QH2-DQH2*(H1+QQ3)+
     >   TINV*QH2*(Q37+Q35*H1))-DF1DT*H1)
      WINV=QQ1/W
      DWDP=C30*WINV*F1*DNDP
      DWDT=C30*WINV*(DF1DT*ANTOT+F1*DNDT)
      DCDP=H3*DWDP
      DCDT = H3 * (DWDT - C * SQRTM1)
      DRDT=C31*(DNDT+DNDT-DCDT)
      DRDP=C31*(DNDP+DNDP-DCDP)
      DF2DT=TINV*F2+BK*T*T*(DF1DT2*F1INV+C*(TINV-C))
      DEDT=RHOINV*(ANHN*(ANHN*(DF1DT*F2+F1*DF2DT)+DCDT*(F1+F1)*F2)-
     >   ENERGY*DRDT)
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      D3PAR=H2*TINV*(D2PAR*(PAR+PAR-QQ1)+(DPR+DPR)*(DPR-
C----     >   (TINV+TINV)*(PAR+PAR-QQ1))+QQ6*(PAR-QQ1)*PAR*T2INV)
C----      D3QH2=QQ6*A3*PAR+QQ3*(D2F11*DPR+DF11*D2PAR)+F11*D3PAR
C----      DF1DT3=TINV*(H1*((TINV+TINV)*DF1DT-DF1DT2)+EFKT*SQRTTI*
C----     >   (T*D3QH2-D2QH2*(Q45+H1+H1)+TINV*(((H1+QQ5)**2+Q14)*
C----     >   DQH2-TINV*QH2*((Q45*H1+Q17)*H1+Q13))))
C----      D2WDP2=-DWDP**2*WINV
C----      D2WDPT=(C30*(DF1DT*DNDP+F1*D2NDPT)-DWDP*DWDT)*WINV
C----      D2WDT2=WINV*(C30*(DF1DT2*ANTOT+(DNDT+DNDT)*DF1DT+F1*D2NDT2)-
C----     >   DWDT**2)
C----      D2CDP2=H3*D2WDP2
C----      D2CDPT=H3*(D2WDPT-C*DWDP)
C----      D2CDT2=H3*(D2WDT2-(C+C)*DWDT+(W-QQ1)*((C+C)*C-DF1DT2*F1INV))
C----      D2RDT2=C31*(D2NDT2+D2NDT2-D2CDT2)
C----      D2RDPT=C31*(D2NDPT+D2NDPT-D2CDPT)
C----      D2RDP2=-C31*D2CDP2
C----      DF2DT2=TINV*(DF2DT-TINV*F2)+
C----     >   BK*T*(QQ2*F1INV*(DF1DT2-DF1DT*(TINV-C))+
C----     >         T*(F1INV*DF1DT3-F1INV*F1INV*DF1DT2*DF1DT+
C----     >            (F1INV*DF1DT2-C*C)*(TINV-QQ2*C)-TINV*TINV*C))
C----      D2EDT2=RHOINV*(ANHN*(ANHN*(DF1DT2*F2+(DF1DT+DF1DT)*DF2DT+
C----     >   DF2DT2*F1)+QQ4*DCDT*(F1*DF2DT+F2*DF1DT)+D2CDT2*(F1+F1)*F2)+
C----     >   (F1+F1)*F2*DCDT**2-ENERGY*D2RDT2-(DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Special Case.
C   The atmosphere consists of uncharged particles: H, He, M
C-----------------------------------------------------------------------
C
  200 RHO=RO*ANTOT
      RHOINV=QQ1/RHO
      ENERGY=Q15*P*RHOINV
      ENTALP=ENERGY+P*RHOINV
      ANA=ANTOT
      ANHN=ANA*X
      ANHEN=ANA*Y
      ANMN=ANA*Z
      ANH2=QQ0
      ANHP=QQ0
      ANHEP=QQ0
      ANHEPP=QQ0
      ANE=QQ0
      IF(KP.EQ.0) KENTER=2
C
      IF(IFENTR.NE.0) S=BK*RHOINV*(ANTOT*(Q15*ALOG(T)-ALOG(ANTOT)+D8))
C
      DRDP=RO*DNDP
      DRDT=RO*DNDT
      DEDT=ENERGY*TINV
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      D2RDT2=RO*D2NDT2
C----      D2RDPT=RO*D2NDPT
C----      D2RDP2=QQ0
C----      D2EDT2=QQ0
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Moderate temperature: The atmosphere consists of H, H+, He, M
C-----------------------------------------------------------------------
C
  300 F12=QPOT6*BKEVTI+Q15
      F2=EXP(QPOT7-F12)*SQRTT*T
      SRF2 = SQRT(F2)
      WFAC = SQRT(F2*C15+ANTOT*X)
      W    = SRF2*WFAC
      ANE=W-F2*C16
      IF(IFION.EQ.13.AND.ANE.LE.(EPSIT*ANTOT).AND.KP.EQ.0) GOTO 100
      ANA=ANTOT-ANE
      ANMN=ANA*Z
      ANHEN=ANA*Y
      ANHN=ANA*X-ANE
      ANH2=QQ0
      ANHP=ANA*X-ANHN
      ANHEP=QQ0
      ANHEPP=QQ0
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      ENERGY=(Q15*P+ANE*POTH)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=3
C
      IF(IFENTR.EQ.0) GOTO 350
      S=ANTOT*(Q25+Q15*ALOG(T))-
     >   ANHN*(ALOG(ANHN)-D5)-ANE*(ALOG(ANE**2)-D7)
      IF(ANHEN.NE.QQ0) S=S-ANHEN*(ALOG(ANHEN)-D2)
      IF(ANMN.NE.QQ0) S=S-ANMN*(ALOG(ANMN)-D3)
      S=BK*RHOINV*S
C
  350 CONTINUE
C--- DF2DT --> DF2DT / F2
C--- DWDP  --> DWDP  / SRF2
C--- DWDT  --> DWDT  / SRF2
      F2WINV=SRF2/WFAC
      DF2DT=T2INV*(Q15*T+QPOT8)
      DWDP=C12*DNDP/WFAC
      DWDT=(C15*F2*DF2DT + C12*(DF2DT*ANTOT + DNDT))/WFAC
      DCDT=SRF2*DWDT-F2*DF2DT*C16
      DRDT=RO*(DNDT-DCDT)
      DRDP=RO*(DNDP-SRF2*DWDP)
      DEDT=RHOINV*(POTH*DCDT-ENERGY*DRDT)
C
C--- DF2DT2  --> DF2DT2  / F2
C----      IF(NEWTON.LE.0) GOTO 999
C----      DF2DT2=T2INV*(DF2DT*(Q15*T+QPOT8)-(Q15+TINV*QPOT9))
C----      D2WDP2=-F2WINV*DWDP**2
C----      D2WDPT=F2WINV*(C12*(DF2DT*DNDP+D2NDPT)-DWDT*DWDP)
C----      D2WDT2=F2WINV*(C15*F2*(DF2DT**2 + DF2DT2) +
C----     >       C12*(DF2DT2*ANTOT+(DNDT+DNDT)*DF2DT+D2NDT2)-DWDT**2)
C----      D2CDT2=D2WDT2-F2*DF2DT2*C16
C----      D2RDT2=RO*(D2NDT2-D2CDT2)
C----      D2RDPT=RO*(D2NDPT-D2WDPT)
C----      D2RDP2=-RO*D2WDP2
C----      D2EDT2=RHOINV*(POTH*D2CDT2-ENERGY*D2RDT2-(DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Medium temperature: The atmosphere consists of H, H+, He, He+, M
C-----------------------------------------------------------------------
C
  500 F12 = QPOT6 * BKEVTI + Q15
      F2 = EXP( QPOT7-F12 ) * SQRTT * T
      F13 = QPOT1 * BKEVTI + Q15
      F3 = EXP( QPOT2-F13 ) * SQRTT * T
C
      F2PF3 = (F2 + F3)
      IF (F2PF3 .GT. 1.0) FACT = F2PF3
      IF (F2PF3 .LE. 1.0) FACT = 1.0 / F2PF3
      F2 = F2 / FACT
      F3 = F3 / FACT
      ANTOT = ANTOT / FACT
C
      F11 = C4 * F2 * F3
      F14 = F11 * C14
      F23 = F2 * X + F3 * Y
      B1 = F2 * C2 + F3 * C3
      B2 = F11 - ANTOT * F23
      ANE = QI2 * ANTOT
      IF (KENTER .EQ. 4) ANE = SQRT( F2 * (F2*C15 + ANTOT*X) )
     >                         - F2 * C16
  543 H2 = (ANE+B1) * ANE
      H3 = H2 + B2
      FDIVDF = ( H3*ANE - ANTOT*F14 ) / ( ANE*ANE + H2 + H3 )
      ANE = ANE - FDIVDF
      IF (ABS(FDIVDF) .GE. (EPSIT*ANE)) GOTO 543
C
      ANTOT = FACT * ANTOT
      ANE = FACT * ANE
      F2 = FACT * F2
      F3 = FACT * F3
C
      ANA = ANTOT - ANE
      H2 = QQ1 / (ANE + F2)
      H3 = QQ1 / (ANE + F3)
      ANHN = ANA * X * H2 * ANE
      ANHP = ANA * X - ANHN
      ANHEN = ANA * Y * H3 * ANE
      ANHEP = ANA * Y - ANHEN
      ANMN = ANA * Z
      ANH2 = QQ0
      ANHEPP = QQ0
      RHO = RO * ANA
      RHOINV = QQ1 / RHO
      ENERGY = (Q15*P + ANHP*POTH + ANHEP*POTHE) * RHOINV
      ENTALP = ENERGY + P*RHOINV
C
      IF (IFENTR .EQ. 0) GOTO 550
      S=ANTOT*(Q25+Q15*ALOG(T))-
     >   ANHN*(ALOG(ANHN)-D5)-ANHP*(ALOG(ANHP*ANE)-D7)
      IF(ANHEN.GT.QQ0) S=S-ANHEN*(ALOG(ANHEN)-D2)
      IF(ANHEP.GT.QQ0) S=S-ANHEP*(ALOG(ANHEP*ANE)-D6)
      IF(ANMN.GT.QQ0) S=S-ANMN*(ALOG(ANMN)-D3)
      S=BK*RHOINV*S
C
  550 CONTINUE
C
      ANE = ANE / FACT
      ANTOT = ANTOT / FACT
      ANA = ANA / FACT
      DNDT = DNDT / FACT
      DNDP = DNDP / FACT
      ANHP = ANHP / FACT
      ANHEP = ANHEP / FACT
      F2 = F2 / FACT
      F3 = F3 / FACT
      H2 = H2 * FACT
      H3 = H3 * FACT
C
      DF2DT=F2*T2INV*(Q15*T+QPOT8)
      DF3DT=F3*T2INV*(Q15*T+QPOT4)
      B1T=DF2DT*C2+DF3DT*C3
      B2T=C4*(DF2DT*F3+F2*DF3DT)-DNDT*(F2*X+F3*Y)-
     >   ANTOT*(DF2DT*X+DF3DT*Y)
      B3T=C5*(DNDT*F2*F3+ANTOT*(DF2DT*F3+F2*DF3DT))
      B2P=-DNDP*F23
      FDIVDF=-QQ1/(ANE*(ANE+ANE+ANE+B1+B1)+B2)
      DCDP=FDIVDF*(ANE*B2P-F14*DNDP)
      DCDT=FDIVDF*((ANE*B1T+B2T)*ANE-B3T)
      DRDT=RO*(DNDT-DCDT)
      DRDP=RO*(DNDP-DCDP)
      DXDT=H2*(X*(F2*(DNDT-DCDT)+DF2DT*ANA)-ANHP*(DCDT+DF2DT))
      DYDT=H3*(Y*(F3*(DNDT-DCDT)+DF3DT*ANA)-ANHEP*(DCDT+DF3DT))
      DEDT=RHOINV*(POTH*DXDT+POTHE*DYDT-ENERGY*DRDT)
C
      DCDP = FACT * DCDP
      DCDT = FACT * DCDT
      DRDP = FACT * DRDP
      DRDT = FACT * DRDT
      DXDT = FACT * DXDT
      DYDT = FACT * DYDT
      DEDT = FACT * DEDT
C
      ANE = FACT * ANE
      ANTOT = FACT * ANTOT
      ANA = FACT * ANA
      DNDT = FACT * DNDT
      DNDP = FACT * DNDP
      ANHP = FACT * ANHP
      ANHEP = FACT * ANHEP
      H2 = FACT * H2
      H3 = FACT * H3
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      DF2DT2=T2INV*(DF2DT*(Q15*T+QPOT8)-F2*(Q15+TINV*QPOT9))
C----      DF3DT2=T2INV*(DF3DT*(Q15*T+QPOT4)-F3*(Q15+TINV*QPOT5))
C----      D2FDE2=QQ2*(ANE+ANE+ANE+B1)
C----      B1TT=DF2DT2*C2+DF3DT2*C3
C----      B2TT=C4*(DF2DT2*F3+(DF2DT+DF2DT)*DF3DT+F2*DF3DT2)-D2NDT2*F23-
C----     >   (DNDT+DNDT)*(DF2DT*X+DF3DT*Y)-ANTOT*(DF2DT2*X+DF3DT2*Y)
C----      B3TT=C5*(ANTOT*(DF2DT2*F3+(DF2DT+DF2DT)*DF3DT+F2*DF3DT2)+
C----     >   (DNDT+DNDT)*(DF2DT*F3+F2*DF3DT)+F2*F3*D2NDT2)
C----      D2FDT2=ANE*(ANE*B1TT+B2TT)-B3TT
C----      D2FDPT=-ANE*(D2NDPT*F23+DNDP*(DF2DT*X+DF3DT*Y))-
C----     >   C5*(DNDP*(DF2DT*F3+F2*DF3DT)+F2*F3*D2NDPT)
C----      D2FDTE=(ANE+ANE)*B1T+B2T
C----      D2CDT2=FDIVDF*((D2FDE2*DCDT+D2FDTE+D2FDTE)*DCDT+D2FDT2)
C----      D2CDPT=FDIVDF*((D2FDE2*DCDT+D2FDTE)*DCDP+B2P*DCDT+D2FDPT)
C----      D2CDP2=FDIVDF*(D2FDE2*DCDP+B2P+B2P)*DCDP
C----      D2RDT2=RO*(D2NDT2-D2CDT2)
C----      D2RDP2=-RO*D2CDP2
C----      D2RDPT=RO*(D2NDPT-D2CDPT)
C----      D2XDT2=H2*(X*(F2*(D2NDT2-D2CDT2)+(DF2DT+DF2DT)*(DNDT-DCDT)+
C----     >   ANA*DF2DT2)-(DXDT+DXDT)*(DCDT+DF2DT)-ANHP*(D2CDT2+DF2DT2))
C----      D2YDT2=H3*(Y*(F3*(D2NDT2-D2CDT2)+(DF3DT+DF3DT)*(DNDT-DCDT)+
C----     >   ANA*DF3DT2)-(DYDT+DYDT)*(DCDT+DF3DT)-ANHEP*(D2CDT2+DF3DT2))
C----      D2EDT2=RHOINV*(POTH*D2XDT2+POTHE*D2YDT2-ENERGY*D2RDT2-
C----     >   (DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   High temperature: The atmosphere consists of H+, He+, He++, M
C-----------------------------------------------------------------------
C
  600 F14 = QPOT3 * BKEVTI + Q15
      F4 = EXP( QPOT7-F14 ) * SQRTT * T
      IF (F4 .GT. 1.0) FACT = F4
      IF (F4 .LE. 1.0) FACT = 1.0 / F4
      F4 = F4 / FACT
      ANTOT = ANTOT / FACT
      F3 = ( F4*C7 - ANTOT*C5 )
      W = SQRT( C17*ANTOT*F4 + F3*F3 )
      ANE = ABS( W-F3 ) * C1
C
      F4 = F4 * FACT
      ANTOT = ANTOT * FACT
      F3 = F3 * FACT
      W = W * FACT
      ANE = ANE * FACT
C
      ANA = ANTOT - ANE
      H2=QQ1/(ANE+F4)
      ANHP=ANA*X
      ANHEP=ANA*Y*H2*ANE
      ANHEPP=ANA*Y-ANHEP
      IF(ANHEPP.LT.QQ0) ANHEPP=QQ0
      ANMN=ANA*Z
      ANH2=QQ0
      ANHN=QQ0
      ANHEN=QQ0
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      ENERGY=(ANHP*POTH+ANHEP*POTHE+ANHEPP*POTHI+Q15*P)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=6
C
      IF(IFENTR.EQ.0) GOTO 650
      S=ANTOT*(Q25+Q15*ALOG(T))-ANHP*(ALOG(ANHP*ANE)-D7)
      IF(ANHEP.NE.QQ0) S=S-ANHEP*(ALOG(ANHEP*ANE)-D6)
      IF (ANHEPP .NE. QQ0)
     >   S = S - ANHEPP * (ALOG(ANHEPP) + 2.0 * ALOG(ANE) - D4 +
     >                      QQ2 * ALG2 + Q15 * ALOG(QPE))
      IF(ANMN.NE.QQ0) S=S-ANMN*(ALOG(ANMN)-D3)
      S=BK*RHOINV*S
C
  650 CONTINUE
C
      F3 = F3 / FACT
      F4 = F4 / FACT
      W = W / FACT
      H2 = H2 * FACT
      ANA = ANA / FACT
      ANE = ANE / FACT
      ANTOT = ANTOT / FACT
      ANHEP = ANHEP / FACT
      DNDT = DNDT / FACT
      DNDP = DNDP / FACT
C
      DF4DT=F4*T2INV*(Q15*T+QPOT10)
      WINV=QQ1/W
      DF3DT=C7*DF4DT-C5*DNDT
      DF3DP=-C5*DNDP
      DWDT=WINV*(C32*(DNDT*F4+ANTOT*DF4DT)+F3*DF3DT)
      DWDP=WINV*(C32*DNDP*F4+F3*DF3DP)
      DCDT=C1*(DWDT-DF3DT)
      DCDP=C1*(DWDP-DF3DP)
      DXDT=DNDT-DCDT
      DRDT=RO*DXDT
      DRDP=RO*(DNDP-DCDP)
      DYDT=H2*(Y*(DXDT*ANE+ANA*DCDT)-ANHEP*(DCDT+DF4DT))
      DEDT=RHOINV*(DXDT*C0+QPOT*DYDT-ENERGY*DRDT)
C
      F3 = F3 * FACT
      F4 = F4 * FACT
      W = W * FACT
      H2 = H2 / FACT
      ANA = ANA * FACT
      ANE = ANE * FACT
      ANTOT = ANTOT * FACT
      ANHEP = ANHEP * FACT
      DNDT = DNDT * FACT
      DNDP = DNDP * FACT
C
      DF4DT = DF4DT * FACT
      WINV = WINV / FACT
      DF3DT = DF3DT * FACT
      DF3DP = DF3DP * FACT
      DWDT = DWDT * FACT
      DWDP = DWDP * FACT
      DCDT = DCDT * FACT
      DCDP = DCDP * FACT
      DXDT = DXDT * FACT
      DRDT = DRDT * FACT
      DRDP = DRDP * FACT
      DYDT = DYDT * FACT
      DEDT = DEDT * FACT
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      DF4DT2=T2INV*(DF4DT*(Q15*T+QPOT10)-F4*(Q15+TINV*QPOT11))
C----      DF3DT2=C7*DF4DT2-C5*D2NDT2
C----      DF3DPT=-C5*D2NDPT
C----      D2WDT2=WINV*(C32*(D2NDT2*F4+(DNDT+DNDT)*DF4DT+ANTOT*DF4DT2)+
C----     >   DF3DT**2+F3*DF3DT2-DWDT**2)
C----      D2WDP2=WINV*(DF3DP**2-DWDP**2)
C----      D2WDPT=WINV*(C32*(D2NDPT*F4+DNDP*DF4DT)+DF3DT*DF3DP+
C----     >   F3*DF3DPT-DWDP*DWDT)
C----      D2CDP2=C1*D2WDP2
C----      D2CDT2=C1*(D2WDT2-DF3DT2)
C----      D2CDPT=C1*(D2WDPT-DF3DPT)
C----      D2XDT2=D2NDT2-D2CDT2
C----      D2RDT2=RO*D2XDT2
C----      D2RDPT=RO*(D2NDPT-D2CDPT)
C----      D2RDP2=-RO*D2CDP2
C----      D2YDT2=H2*(Y*(D2XDT2*ANE+(DXDT+DXDT)*DCDT+ANA*D2CDT2)-
C----     >   (DYDT+DYDT)*(DCDT+DF4DT)-ANHEP*(D2CDT2+DF4DT2))
C----      D2EDT2=RHOINV*(D2XDT2*C0+QPOT*D2YDT2-ENERGY*D2RDT2-
C----     >   (DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Very high temperature: The atmosphere consists of H+, He++, M
C-----------------------------------------------------------------------
C
  700 ANA=ANTOT*C8
      ANE=ANA*C6
      ANH2=QQ0
      ANHN=QQ0
      ANHP=ANA*X
      ANHEN=QQ0
      ANHEP=QQ0
      ANHEPP=ANA*Y
      ANMN=ANA*Z
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      ENERGY=(Q15*P+ANA*C0)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=7
C
      IF(IFENTR.NE.0) S=BK*RHOINV*(ANTOT*(Q25+Q15*ALOG(T))-
     >   ANA*(ALOG(ANA)*D9-D10))
C
      DRDT=C33*DNDT
      DRDP=C33*DNDP
      DEDT=Q15*P*TINV*RHOINV
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      D2RDT2=C33*D2NDT2
C----      D2RDPT=C33*D2NDPT
C----      D2RDP2=QQ0
C----      D2EDT2=QQ0
C
C
C-----------------------------------------------------------------------
C   Computation of common thermodynamical quantities
C-----------------------------------------------------------------------
C
  999 IF (KP .NE. 0) GOTO 1200
C
      RHOIN2 = RHOINV * RHOINV
      DSDT   = TINV * (DEDT - P * DRDT * RHOIN2)
      DTDS   = QQ1 / DSDT
      DSDP   = RHOIN2 * DRDT
      Y1     = -DSDP * DTDS
      H      = DRDP + DRDT * Y1
C
      IF (H .GT. QQ0  .AND.  DRDP .GE. QQ0) GOTO 888
      IF (IRUN .EQ. 3  .OR.  IFION .EQ. 0) GOTO 888
      IERROR = IERROR + 1
      IRUN = IRUN + 1
      IF (IRUN .EQ. 1) KENNEW = KENTER + 1
      IF (IRUN .EQ. 2) KENNEW = KENTER - 1
      IF (KENNEW .LE. 0) KENNEW = 1
      IF (KENNEW .GT. 7) KENNEW = 7
      KENTER = KENNEW
      GOTO 99
C
  888 CONTINUE
      CS   = QQ1 / SQRT(H)
      XCP  = -RHO * DSDT / DRDT
      XGAM = CS*CS * RHO / P
      CXT  = -RHOINV * DRDT * T
C----      CT   = QQ1 / SQRT(DRDP)
C----      XMU  = QMOL * ANA / ANTOT
C----      H    = -RHOINV * DRDT * DTDS * CS
C----      DTDP = Y1
C
C----      IF(NEWTON.LE.0) RETURN
C----      D2SDPT=RHOIN2*(D2RDT2-(RHOINV+RHOINV)*DRDT**2)
C----      D2SDP2=RHOIN2*(D2RDPT-(RHOINV+RHOINV)*DRDP*DRDT)
C----      D2SDT2=-TINV*(TINV*DEDT-D2EDT2-P*RHOIN2*(TINV*DRDT+
C----     >   (RHOINV+RHOINV)*DRDT**2-D2RDT2))
C----      H2=-DTDS**2
C----      DTDPP=H2*(DSDT*D2SDP2-DSDP*D2SDPT)
C----      DTDPT=H2*(DSDT*D2SDPT-DSDP*D2SDT2)
C----      DTDSP=H2*D2SDPT
C----      DTDST=H2*D2SDT2
C----      HP=D2RDP2+D2RDPT*DTDP+DRDT*DTDPP
C----      HT=D2RDPT+D2RDT2*DTDP+DRDT*DTDPT
C----      CSP=-QI2*HP*CS**3
C----      CST=-QI2*HT*CS**3
C----      CTP=-QI2*D2RDP2*CT**3
C----      CTT=-QI2*D2RDPT*CT**3
C----      HP=RHOINV*(DRDT*(CS*(RHOINV*DTDS*DRDP-DTDSP)-DTDS*CSP)-
C----     >   D2RDPT*CS*DTDS)
C----      HT=RHOINV*(DRDT*(CS*(RHOINV*DTDS*DRDT-DTDST)-DTDS*CST)-
C----     >   D2RDT2*CS*DTDS)
C
      RETURN
      END
