C*******************************************************************************
C
C    ####     ##     ####
C   #    #   #  #   #
C   #       #    #   ####
C   #  ###  ######       #
C   #    #  #    #  #    #
C    ####   #    #   #### _module
C
C*******************************************************************************
C
C Routines (contained):
C   broydn
C   eoscnd
C   fcsaha (fcsaha_23jan98.f)
C   gas    (gas_23jan98.f)
C   gasgas (gasgas_23jan98.f)
C   gas_h
C   gas_sundeep
C   gasinv
C   owninv
C   qh2    (qh2_23jan98.f)
C   smthlm
C   supergas
C   supergasinv
C
C History:
C   old
C   2004-02-24 (B.F.) Put into module,
C                     "end" -> "end subroutine name"
C                     declared QH2 as variable in gas()
C                     removed exp10() internal function
C   2004-02-25 (B.F.) gas_h, gas_sundeep included
C   2004-02-26 (B.F.) supergas
C   2008-12-08 (B.F.) Initial guess for supergasinv changed
C   2014-09-06 (B.F. Uppsala) epsiter smaller
C*******************************************************************************

      module gas_module

      contains

      SUBROUTINE BROYDN( MDIMB, NX, MIMPS, PRECIS, RNRN, 
     >                   BINV, X, Y, IS, IMPS, ISTAT)
********************************************************************************
*
* Version: 18-MAY-1992 12:42:19
*
* Author : HGL
*
* Description: BROYDN improves the approximate solution of a set of non-linear
*   (linear too) equations 
*   
*   y := F(x) = 0, F: R_n -> R_n is a vector function of vector x.
*   
*   BROYDN improves a given solution x_k (maximum MIMPS times) by computing
*
*   x_k+1 = x_k - y_k * BINV.
*
*   If the prescribed precision of solution x is not reached BINV is modified
*   and a following solution x is computed. BROYDN uses the Broyden method 
*   described in
*   L. Koesterke, W.-R. Hamann, P. Kosmol (1992): 
*   Astron. & Astrophys. Vol. 255, pp. 490-494
* 
*   BROYDN uses BINV, X, Y, IS and IMPS as work area. These variables must
*   correspond to different physical memory locations.
*
*   The FORTRAN formulation coded here interprets vectors as row-vectors. The
*   BINV array indices correspond to mathematical index notation 
*   ( BINV( row index, column index)). This is important if one (like the
*   author) is trying to sort the bunch of different indices correctly. The
*   author found for the BINV update formula (adopting Einstein's summation 
*   convention):
*
*                       (BINV_ik * Dx_k) * (Dx_j - Dy_k * BINV_kj)
*   BINV_ij = BINV_ij + ------------------------------------------,  D = delta.
*                                 Dx_j * Dy_k * BINV_kj
*
* Arguments, access, type, dimension, description:
*   MDIMB  R I : First dimension of BINV array
*   NX     R I : Number of equations actually used
*   MIMPS  R I : Maximum number of improvements of the solution, i.e. 
*                maximum number of iterations
*   PRECIS R D1: Absolute (positive) correction (somewhat like the precision)
*                where the iteration should stop; this value has to be 
*                specified for every vector component
*   RNRN   R E : Name of subroutine which contains the vector function to be 
*                solved, RNRN( NX, <input vector>, <output vector>). Double
*                precision!
*   BINV   * D2: Inverse Broyden matrix. A suitable first approximation must be
*                specified! It is modified during the iteration cycle.
*   X      * D2: X(IS) contains the current solution of the set of equations.
*                A suitable first approximation must be specified!
*                Dimension of X: X( MDIMB, >=2), only two x-vectors are needed!
*                Care should be taken that IS points in the calling routine
*                actually to the correct memory location in array X!
*   Y      * D2: Working area during iteration cycle, dimension Y( MDIMB, >=3)
*                At least three scratch vectors are needed.
*   IS     * I : Pointer to current solution, IS must be 1 or 2!
*   IMPS   W I : Number of improvement steps executed when routine has
*                finished, work variable!
*   ISTAT  W I : Exit status:
*                = +1: Iteration has not converged in MIMPS cycles
*                =  0: Everything fine, iteration has converged
*                = -2: Scalar product in denominator of BINV update formula
*                      equals zero. Perhaps iteration has converged, but the
*                      numerical noise is greater than the precision. Try again
*                      with a somewhat relaxed claim of precision.
*                = -3: Illegal pointer IS encountered!
*                = -999: Semantic error in FORTRAN code, contact the author!
*
* Common blocks (access): None
*
* External procedures   : RNRN (external)
*
* File units (access)   : None
*
*
* Comments: The routine is designed in DOUBLE PRECISION!
*
*   Since a lot of index gymnastics is performed a scetch of the memory
*   management seems to be useful:
*   (here: B=BINV, #=variable is used in current computation)
*
*   ---------------------------------------------------------------------------
*                            X(IS)     X(IW)     Y(IS)     Y(IW)     Y(IW2)
*   ---------------------------------------------------------------------------
*   Start                    x_k       indef     indef     indef     indef
*   ---------------------------------------------------------------------------
*   Compute F(x_k)           #                   y_k
*   ===========================================================================
*   Start iteration          x_k       indef     y_k       indef     indef
*   ---------------------------------------------------------------------------
*   Update solution          #         x_k+1     #
*   ---------------------------------------------------------------------------
*   Exchange IS<->IW         x_k+1     x_k       indef     y_k       indef
*   ---------------------------------------------------------------------------
*   Compute x_k+1-x_k        #         #Dx_k
*   ---------------------------------------------------------------------------
*   Compute F(x_k+1)         #                   y_k+1
*   ---------------------------------------------------------------------------
*   Convergence check
*   ---------------------------------------------------------------------------
*   Compute y_k+1-y_k                            #         #Dy_k
*   ---------------------------------------------------------------------------
*   Compute Dy_k*B_k                                       #         Dy_k*B_k
*   ---------------------------------------------------------------------------
*   Compute Dy_k*B_k*(Dx_k)T           #                             #
*   -> DUM
*   ---------------------------------------------------------------------------
*   Check DUM=0 ?
*   ---------------------------------------------------------------------------
*   Compute B_k*(Dx_k)T/DUM            #                   B_k*(Dx_k)T
*                                                          /DUM
*   ---------------------------------------------------------------------------
*   Compute Dx_k-Dy_k*B_k              #                             #Dx_k-
*                                                                     Dy_k*B_k
*   ---------------------------------------------------------------------------
*   Update B_k                                             #         #
*   ---------------------------------------------------------------------------
*   Interpret k+1 as k       x_k       Dx_k-1    y_k       indef     indef
*   ===========================================================================
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
      DOUBLE PRECISION BINV( MDIMB, *), X( MDIMB, *), Y( MDIMB, *)
      DOUBLE PRECISION PRECIS(*)
      DOUBLE PRECISION R00, DUM
      PARAMETER( R00 = 0.0)
      LOGICAL ISCONV
***   Work indicex IW2 is fixed
      PARAMETER( IW2 = 3)

      EXTERNAL RNRN

***   Preset status indicator
      ISTAT = -999

***   Definition of work indices
***   IS is 1 or 2, IW has to be 2 or 1 resp.
      IF ( IS .EQ. 1) THEN
        IW = 2
      ELSEIF( IS .EQ. 2) THEN
        IW = 1
      ELSE
        ISTAT = -3
        RETURN
      ENDIF

***   Compute y_k
      CALL RNRN( NX, X( 1, IS), Y( 1, IS))


***   Broyden iteration starts here
***   =============================
      DO 900 IMPS = 1, MIMPS

***   Improve solution
      DO 20 I = 1, NX
        DUM = R00
        DO 10 K = 1, NX
          DUM = DUM + Y( K, IS) * BINV( K, I) 
10      CONTINUE
        X( I, IW) = X( I, IS) - DUM
20    CONTINUE

***   Change the role of IS and IW
      IDUM = IW
      IW   = IS
      IS   = IDUM

***   Compute Dx_k
      DO 30 K = 1, NX
        X( K, IW) = X( K, IS) - X( K, IW)
30    CONTINUE

***   Compute y_k+1
      CALL RNRN( NX, X( 1, IS), Y( 1, IS))

***   Convergence check ...
***   Test if correction of solution is smaller than preset precision in every
***   vector component:
      ISCONV = .TRUE.
      DO 40 K = 1, NX
        ISCONV = ISCONV .AND. ( ABS( X( K, IW)) .LT. PRECIS( K))
40    CONTINUE
      IF ( ISCONV) THEN
        ISTAT = 0
        RETURN
      ENDIF

***   Compute Dy_k
      DO 60 K = 1, NX
        Y( K, IW) = Y( K, IS) - Y( K, IW)
60    CONTINUE


***   Update inverse Broyden matrix      
***   -----------------------------

***   Dy_k * BINV_k (stored in the Y scratch area 2)
***   Currently Y scratch area 1 contains Dy_k
      DO 110 I = 1, NX
        DUM = R00
        DO 100 K = 1, NX
          DUM = DUM + Y( K, IW) * BINV( K, I)
100     CONTINUE
        Y( I, IW2) = DUM
110   CONTINUE

***   Compute scalar product (Dy_k * BINV)_k * (Dx_k)T
      DUM = R00
      DO 120 K = 1, NX
        DUM = DUM + X( K, IW) * Y( K, IW2)
120   CONTINUE

***   Scalar product less or equal zero?
***   Probably due to the fact that Dx_k is zero.
      IF ( DUM .EQ. R00) THEN
        ISTAT = -2
        RETURN
      ENDIF

***   Compute BINV_k * (Dx_k)T / scalar product
      DO 140 I = 1, NX
        Y( I, IW) = R00
        DO 130 K = 1, NX
          Y( I, IW) = Y( I, IW) + BINV( I, K) * X( K, IW)
130     CONTINUE
          Y( I, IW) = Y( I, IW) / DUM
140   CONTINUE

***   Compute Dx_k - Dy_k * BINV_k
      DO 150 K = 1, NX
        Y( K, IW2) = X( K, IW) - Y( K, IW2)
150   CONTINUE

***   Update BINV
      DO 170 J = 1, NX
        DO 160 I = 1, NX
          BINV( I, J) = BINV( I, J) + Y( I, IW) * Y( J, IW2)
160     CONTINUE
170   CONTINUE

***   Broyden iteration ends here
***   ===========================
900   CONTINUE

***   Number of Broyden iterations executed, finish!
      IMPS  = MIMPS
      ISTAT = 1
      RETURN

      END SUBROUTINE BROYDN


      SUBROUTINE EOSCND( NEQ, XDN, YDN)
********************************************************************************
*
* Version: 20-MAY-1992 12:20:58
*
* Author : HGL
*
* Description: EOSCND gives the equations which govern the different ionization
*   and dissoziation processes in the EOS.
*   EOSJCB determines the Jacobi-matrix of the system of equations:
*   YJACOB( i, j) = d y( j) / d x( i)
* 
*
* Arguments, access, type, dimension, description:
*   NEQ    R I : Number of equations actually regarded of calling routine
*   XDN    R D1: Array containing the number densities of various species.
*   YDN    W D1: Array containing the deviations of the constrain equations
*
*
* Common blocks (access): EOSPAR(R)
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: The routine is designed in DOUBLE PRECISION.
*   Some variables are passed to EOSCND by common block EOSPAR. This is done to
*   ensure the correct form of the parameter list of the SUBROUTINE statement.
*   BROYDN assumes such a simple form of the external vector function.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
      INCLUDE 'EOSPAR.INC'

      DOUBLE PRECISION XDN(*), YDN(*), YJACOB( MEQ, *), 
     >                 DYDNDT(*), DYDNDP(*)

***   Constants
      DOUBLE PRECISION C10, C20, C100
      PARAMETER( C10 = 1.0, C20 = 2.0, C100 = 10.0)

***   Definition of symbolic names for XDN array elements:
***   Somewhat slow but better to understand ...
***   Here EQUIVALENCE would be better but it doesn't work since XDN is a
***   formal parameter.
      DOUBLE PRECISION DNA, DNH2, DNHN, DNHP, DNHEN, DNHEP, DNHEPP, 
     >                 DNEL, DNM
      PARAMETER( INA=1, INH2=2, INHN=3, INHP=4, INHEN=5, INHEP=6, 
     >           INHEPP=7, INEL=8, INM=9)
      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
***   Modify DNEL: From numerical reasons a minimum electron density is adopted
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

***   Here the constrain equations ...
***   (see Gigas, Dissertation, p.272)
      YDN(1) = C20*DNH2 + DNHN + DNHP - X*DNA 
      YDN(2) = DNHEN + DNHEP + DNHEPP - Y*DNA 
      YDN(3) = DNA + DNEL - DNH2 - DNTOT
      YDN(4) = DNHP + DNHEP + C20*DNHEPP - DNEL
      YDN(5) = DNHN*DNHN*GT - DNH2
      YDN(6) = DNHN/DNEL*F1T - DNHP
      YDN(7) = DNHEN/DNEL*F2T - DNHEP
      YDN(8) = DNHEP/DNEL*F3T - DNHEPP
      YDN(9) = DNM - Z*DNA

      RETURN

      ENTRY EOSJCB( XDN, YJACOB)
***   ==========================
***   In this section the Jacobi-matrix of the system of constrain equations is 
***   computed:
***   XDN    R D1: Array containing the number densities of various species.
***   YJACOB W D2: Jacobi-matrix

      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

      YJACOB(1, 1)=-X
      YJACOB(2, 1)=C20
      YJACOB(3, 1)=1.0
      YJACOB(4, 1)=1.0
      YJACOB(5, 1)=0.0
      YJACOB(6, 1)=0.0
      YJACOB(7, 1)=0.0
      YJACOB(8, 1)=0.0
      YJACOB(9, 1)=0.0
      YJACOB(1, 2)=-Y
      YJACOB(2, 2)=0.0
      YJACOB(3, 2)=0.0
      YJACOB(4, 2)=0.0
      YJACOB(5, 2)=1.0
      YJACOB(6, 2)=1.0
      YJACOB(7, 2)=1.0
      YJACOB(8, 2)=0.0
      YJACOB(9, 2)=0.0
      YJACOB(1, 3)=1.0
      YJACOB(2, 3)=-1.0
      YJACOB(3, 3)=0.0
      YJACOB(4, 3)=0.0
      YJACOB(5, 3)=0.0
      YJACOB(6, 3)=0.0
      YJACOB(7, 3)=0.0
      YJACOB(8, 3)=1.0
      YJACOB(9, 3)=0.0
      YJACOB(1, 4)=0.0
      YJACOB(2, 4)=0.0
      YJACOB(3, 4)=0.0
      YJACOB(4, 4)=1.0
      YJACOB(5, 4)=0.0
      YJACOB(6, 4)=1.0
      YJACOB(7, 4)=C20
      YJACOB(8, 4)=-1.0
      YJACOB(9, 4)=0.0
      YJACOB(1, 5)=0.0
      YJACOB(2, 5)=-1.0
      YJACOB(3, 5)=2.0*DNHN*GT
      YJACOB(4, 5)=0.0
      YJACOB(5, 5)=0.0
      YJACOB(6, 5)=0.0
      YJACOB(7, 5)=0.0
      YJACOB(8, 5)=0.0
      YJACOB(9, 5)=0.0
      YJACOB(1, 6)=0.0
      YJACOB(2, 6)=0.0
      YJACOB(3, 6)=F1T/DNEL
      YJACOB(4, 6)=-1.0
      YJACOB(5, 6)=0.0
      YJACOB(6, 6)=0.0
      YJACOB(7, 6)=0.0
      YJACOB(8, 6)=-(DNHN*F1T)/DNEL/DNEL
      YJACOB(9, 6)=0.0
      YJACOB(1, 7)=0.0
      YJACOB(2, 7)=0.0
      YJACOB(3, 7)=0.0
      YJACOB(4, 7)=0.0
      YJACOB(5, 7)=F2T/DNEL
      YJACOB(6, 7)=-1.0
      YJACOB(7, 7)=0.0
      YJACOB(8, 7)=-(DNHEN*F2T)/DNEL/DNEL
      YJACOB(9, 7)=0.0
      YJACOB(1, 8)=0.0
      YJACOB(2, 8)=0.0
      YJACOB(3, 8)=0.0
      YJACOB(4, 8)=0.0
      YJACOB(5, 8)=0.0
      YJACOB(6, 8)=F3T/DNEL
      YJACOB(7, 8)=-1.0
      YJACOB(8, 8)=-(DNHEP*F3T)/DNEL/DNEL
      YJACOB(9, 8)=0.0
      YJACOB(1, 9)=-Z
      YJACOB(2, 9)=0.0
      YJACOB(3, 9)=0.0
      YJACOB(4, 9)=0.0
      YJACOB(5, 9)=0.0
      YJACOB(6, 9)=0.0
      YJACOB(7, 9)=0.0
      YJACOB(8, 9)=0.0
      YJACOB(9, 9)=1.0

      RETURN

      ENTRY EOSDDT( XDN, DYDNDT)
***   ==========================
***   This section computes the temperature derivatives DYDNDT of every
***   constrain equation.

      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

      DYDNDT(1)=0.0
      DYDNDT(2)=0.0
      DYDNDT(3)=DNTOT/DT
      DYDNDT(4)=0.0
      DYDNDT(5)=DNHN**2*DGTDT
      DYDNDT(6)=(DNHN*DF1TDT)/DNEL
      DYDNDT(7)=(DNHEN*DF2TDT)/DNEL
      DYDNDT(8)=(DNHEP*DF3TDT)/DNEL
      DYDNDT(9)=0.0

      RETURN

      ENTRY EOSDDP( XDN, DYDNDP)
***   ==========================
***   This section computes the pressure derivatives DYDNDP of every
***   constrain equation.

      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

      DYDNDP(1)=0.0
      DYDNDP(2)=0.0
      DYDNDP(3)=-DNTOT/DP
      DYDNDP(4)=0.0
      DYDNDP(5)=0.0
      DYDNDP(6)=0.0
      DYDNDP(7)=0.0
      DYDNDP(8)=0.0
      DYDNDP(9)=0.0

      RETURN

      END SUBROUTINE EOSCND


      SUBROUTINE FCSAHA( T, FACGT, FACF1T, FACF2T, FACF3T, 
     >                   EIH, EIHE, EIHEP, EDH2, EDH2DT,
     >                   QH2T, DLQDLT, GT, F1T, F2T, F3T, 
     >                   DGTDT, DF1TDT, DF2TDT, DF3TDT)
********************************************************************************
*
* Version: Fri Jan 16 14:28:06 MET 1998
*
* Author : HGL
*
* Description: FCSAHA is an auxiliary routine of the GASTST program which
*   computes some particle density independent factors which are used in the 
*   Saha-equations.
* 
*
* Arguments, access, type, dimension, description:
*   T      R D : Temperature
*   FACGT  R D : Control factor of GT
*   FACF1T R D : Control factor of F1T
*   FACF2T R D : Control factor of F2T
*   FACF3T R D : Control factor of F3T
*   EIH    R D : Ionization energy H
*   EIHE   R D : Ionization energy He
*   EIHEP  R D : Ionization energy He+
*   EDH2   R D : Dissoziation energy H_2
*   EDH2DT R D : Temperature derivative of H_2 dissoziation energy
*   QH2T   R D : q(T)
*   DLQDLT R D : d ln q(T) / d ln T
*   GT     W D : g(T)
*   F1T    W D : f1(T)
*   F2T    W D : f2(T)
*   F3T    W D : f3(T)
*   DGTDT  W D : d g(T) / d T
*   DF1TDT W D : d f1(T) / d T
*   DF2TDT W D : d f2(T) / d T
*   DF3TDT W D : d f3(T) / d T
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: Designed in DOUBLE PRECISION.
*
* Modification history:
*   08/05/92/hgl: coded
*   16/01/98/hgl: eih, eihe, eihep, edh2 become variable
*   19/01/98/hgl: edh2dt
* 
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
      DOUBLE PRECISION T, FACGT, FACF1T, FACF2T, FACF3T, QH2T, DLQDLT,
     >                 GT, F1T, F2T, F3T, DGTDT, DF1TDT, DF2TDT, DF3TDT

      INCLUDE 'matnat_23jan98.inc'

      DOUBLE PRECISION EIH, EIHE, EIHEP, EDH2, EDH2DT

***   Auxiliary paramters
      DOUBLE PRECISION AUX1, R15, R20, R40, RO40
***   AUX1 = 2 * pi * WTUNIT / h**2
      PARAMETER( AUX1 = D20PI * WTUNIT / PLNCKH / PLNCKH)
      PARAMETER( R15 = 1.5, R20 = 2.0, R40 = 4.0, RO40 = 0.25)

      DOUBLE PRECISION KT, AUX2, AUX3

***   Temperature is measured in kT
      KT = BOLTZK * T
      AUX2 = ( AUX1 * WTEL * KT)**R15

***   F1(T)
      AUX3   = EIH / KT
      F1T    = FACF1T * AUX2 * EXP( - AUX3)
      DF1TDT = FACF1T * ( R15 + AUX3) * F1T / T

***   F2(T)
      AUX3   = EIHE / KT
      F2T    = FACF2T * R40 * AUX2 * EXP( - AUX3)
      DF2TDT = FACF2T * ( R15 + AUX3) * F2T / T

***   F3(T)
      AUX3   = EIHEP / KT
      F3T    = FACF3T * AUX2 * EXP( - AUX3)
      DF3TDT = FACF3T * ( R15 + AUX3) * F3T / T

***   G(T)
      AUX3   = EDH2 / KT
      GT     = FACGT * RO40 * ( AUX1 * WTH * KT / R20)**(-R15) * QH2T
     >       * EXP( AUX3)
      DGTDT  = FACGT * ( - R15 - AUX3 + EDH2DT/BOLTZK + DLQDLT) * GT / T
C      DGTDT  = FACGT * ( - R15 - AUX3 + DLQDLT) * GT / T

      RETURN
      END SUBROUTINE FCSAHA



      SUBROUTINE gas(XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >               IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
C
C***********************************************************************
C
C   IONIZATION BALANCE AND THERMODYNAMIC DERIVATIVES  (B.W., 6.9.1983)
C
C   modified, corrected and converted to CRAY/VAX-FORTRAN  by D.G.
C
C   Last Update:    23-JAN-1998 15:33:00 by hgl: entropy bug in H_2
C                   molecule formation fixed 
C
C                   12-FEB-1993 10:34:00 BY H.G.L: GAS writes 
C                   identification message.
C
C                   12-AUG-1992 12:23:56 BY H.G.L: Entropy computation 
C                   corrected. (Domains 1, 6 and 7)
C
C                   28-APR-1992 11:19:43 BY M.S.
C
C   um common-Block / E / erweitert fuer Anwendungen auf VAX
C      1-MAY-1989 17:32:11 D.G.
C
C***********************************************************************
C
C-----------------------------------------------------------------------
C   This subroutine is based on  IONIS.FOR   by B. E. Wolf (Heidelberg)
C   Astron. Astrophys. 127, 93 (1983); 130, 202 (1984); 145, 278 (1985)
C-----------------------------------------------------------------------
C
C   The subroutine computes thermodynamic quantities and derivatives
C   for given temperature T and gas pressure p
C
C   Ions: H2, H, H+, He, He+, He++, M (representative metal, here Mg;
C         the metal is assumed neutral throughout the subroutine)
C
C   Each temperature section (see below) is divided in 3 subsections:
C      1.: computation of density and energy
C      2.: computation of entropy (omitted if IFENTR = 0)
C      3.: computation of derivatives (omitted if IFDERI = 0)
C
C   Warning: do not use the subroutine for temperatures less than 1500 K
C           log10 (gas pressure) must be greater than -4 and less than 8
C-----------------------------------------------------------------------
C
C   Input parameters are
C       XP     = gas pressure [dyn/cm2]
C       XT     = temperature [K]
C
C   Control parameters are
C       IFION  - determines how to treat ionization processes [0...16]
C       IFENT  - entropy is computed if = 1
C       KENTER - must be < 0 in initial call, otherwise >= 0
C                   computational domain determined by GAS if = 0
C                   computational domain KENTER assumed if > 0
C
C   Output parameters are
C       RHO    = density [g/cm3]
C       XCP    = Cp' = d(S) / d(ln rho) [erg/g/K]
C       XGAM   = gamma' = Cp' / Cv'
C       CXT    = alpha * T = (-1/rho) * d(rho)/d(T) * T
C       ENTALP = enthalpy per mass unit [erg/g/K]
C       S      = entropy per mass unit [erg/g/K]
C       CS     = adiabatic sound velocity [cm/s]
C       IERROR = error indicator (= 0 if no errors encountered)
C       KENTER = index of computational domain
C       
C        
C***********************************************************************
C
      DIMENSION BORDER (12,4,3), TT (4,3)
C
C
C-----------------------------------------------------------------------
C   COMMON block / GFACT / is not shared by other routines. It is only
C   used to ensure a proper placement of the 'G' variables.
C   Attention: Do not change variable arrangement within GFACT !!!
C-----------------------------------------------------------------------
C
      COMMON / GFACT / G1,   G13,  G25,  G37,  G49,  G61,  G73,  G85,
     >                 G97,  G109, G121, G133, G4,   G16,  G28,  G40,
     >                 G52,  G64,  G76,  G88,  G100, G112, G124, G136,
     >                 G7,   G19,  G31,  G43,  G55,  G67,  G79,  G91,
     >                 G103, G115, G127, G139, G10,  G22,  G34,  G46,
     >                 G58,  G70,  G82,  G94,  G106, G118, G130, G142,
     >                 G2,   G14,  G26,  G38,  G50,  G62,  G74,  G86,
     >                 G98,  G110, G122, G134, G5,   G17,  G29,  G41,
     >                 G53,  G65,  G77,  G89,  G101, G113, G125, G137,
     >                 G8,   G20,  G32,  G44,  G56,  G68,  G80,  G92,
     >                 G104, G116, G128, G140, G11,  G23,  G35,  G47,
     >                 G59,  G71,  G83,  G95,  G107, G119, G131, G143,
     >                 G3,   G15,  G27,  G39,  G51,  G63,  G75,  G87,
     >                 G99,  G111, G123, G135, G6,   G18,  G30,  G42,
     >                 G54,  G66,  G78,  G90,  G102, G114, G126, G138,
     >                 G9,   G21,  G33,  G45,  G57,  G69,  G81,  G93,
     >                 G105, G117, G129, G141, G12,  G24,  G36,  G48,
     >                 G60,  G72,  G84,  G96,  G108, G120, G132, G144
C
      common / E / anhn, anhp, anhen, anhep, anhepp, ct
C
C-----------------------------------------------------------------------
C   Definitions of basic parameters
C-----------------------------------------------------------------------
C
      PARAMETER ( EPSIT = 1.0E-05, HLIMIT = 1.0E-02)
C
      PARAMETER ( A1 = 5.797083E-03, A2 = 4.676132E-08, 
     >            A3 = 1.088582E-11 )
C
      PARAMETER ( QCON1 =  9.55469E-01, QCON2 = 1.039473E-04,
     >            QCON3 = -7.19848E-08, QCON4 = 1.154352E-11,
     >            QCON5 = -6.403472E-16 )
C
      PARAMETER ( QQ0 = 0.0, QQ1 = 1.0, QQ2 = 2.0, QQ3 = 3.0, QQ4 = 4.0,
     >            QQ5 = 5.0, QQ6 = 6.0 )
C
      PARAMETER ( Q04 = 0.4, QI2 = 0.5, Q15 = 1.5, Q25 = 2.5, Q35 = 3.5,
     >            Q45 = 4.5, QI4 = 0.25)
C
      PARAMETER ( QLM4 = 1.0E-04, QLM3 = 1.0E-03, QLM2 = 1.0E-02,
     >            QLM1 = 1.0E-01, QLP1 = 1.0E+01, QLP2 = 1.0E+02,
     >            QLP3 = 1.0E+03, QLP4 = 1.0E+04, QLP5 = 1.0E+05,
     >            QLP6 = 1.0E+06, QLP7 = 1.0E+07, QMAX = 1.0E+10 )
C
      PARAMETER ( Q13 = 1.3125E+01, Q14 = -1.375E+01, Q17 = 1.95E+01, 
     >            Q37 =   3.75E+00, Q52 =  5.0E+02 )
C
      PARAMETER ( QF0 =  2.7E+01, QF1 =  4.0E+01, QF2 = -1.5E+01,
     >            QF3 = -1.4E+02, QF4 = 1.65E+02, QF5 = -2.5E+01,
     >            QF6 = -1.5E+02, QF7 =  5.0E+01 )
C
      PARAMETER ( QT1 = 2.5E+03, QT2 = 1.1E+04, QT3 =  5.0E+03,
     >            QT4 = 8.0E+03, QT5 = 2.0E+04, QV1 = 9.14E+03,
     >            QV2 = 1.53E+04 )
C
      PARAMETER ( QPOT = -8.7179E-11,   QPOT1 = 2.4587E+01,
     >            QPOT2 = 3.830669E+01, QPOT3 = 5.4416E+01, 
     >            QPOT4 = 2.853292E+05, QPOT5 = 5.706584E+05,
     >            QPOT6 = 1.3598E+01,   QPOT7 = 3.692039E+01,
     >            QPOT8 = 1.578032E+05, QPOT9 = 3.156063E+05,
     >            QPOT10 = 6.317E+05,   QPOT11 = 1.263E+06 )
C
      PARAMETER ( BK =    1.38054E-16,   QH = 6.6256E-27, 
     >            QMASS = 1.6604E-24,  QGAS = 8.3144E+07,
     >            QQQ =   1.6021E-12 )
C
      PARAMETER ( QPE = 1.8228E+03, QBKT = 1.160488E+04 )
C
      PARAMETER ( QPI2 = 6.283185307179586E+00, QEL = 6.6531E-25 )
C
      PARAMETER ( QHYD = 1.0080, QHEL = 4.0026 )
C
      PARAMETER ( POTH = 2.1785E-11, POTHE = 3.9391E-11,
     >            POTHI = 1.2657E-10 )
C
      PARAMETER ( Q2P1 = 5.19678E+04, Q2P2 = 6.33262E+03,
     >            Q2P3 = -4.7041E+01 )
C
C     --- QH2 is an internal variable (and not the routine in this module)
      real :: QH2
C
C-----------------------------------------------------------------------
C   Save computed quantities for subsequent calls of subroutine
C-----------------------------------------------------------------------
C
      SAVE WGHTMT, Z, ELCON, ALG2, RO
C
      SAVE C0, C1, C2, C3, C4, C5, C6, C7, C8, C9, C11, C12, C13,
     >     C14, C15, C16, C17, C21, C22, C23, C30, C31, C32, C33
C
      SAVE D0, D1, D2, D3, D4, D5, D6, D7, D8, D9, D10
C
C
C-----------------------------------------------------------------------
C   Map BORDER array onto 'G' constants
C-----------------------------------------------------------------------
C
      EQUIVALENCE (G1, BORDER (1,1,1))
C
C
C-----------------------------------------------------------------------
C   Definition of control variables which are to remain unchanged
C-----------------------------------------------------------------------
C
C-----------------------------------------------------------------------
C   Error counter set equal to zero at beginning
C-----------------------------------------------------------------------
C
      IERROR = 0
C
C-----------------------------------------------------------------------
C   Input variables which may be changed are shifted to local variables
C-----------------------------------------------------------------------
C   
      P = XP
      T = XT
      IFENTR = IFENT
C
C
C-----------------------------------------------------------------------
C   Computation of auxiliary variables in first GAS call (KENTER < 0) 
C-----------------------------------------------------------------------
C
      KP = 0
C
      IF (KENTER .GE. 0) GOTO 777
C
c      WRITE( 6, '( /, 1X, 72(''=''))')
c      WRITE( 6, '( 1X, ''GAS version 23-Jan-1998 (STD) busy!'')')
c      WRITE( 6, '( 1X, 72(''=''))')
c      WRITE( 6, '( /)')
C
      IFENTR = 0
C
      T      = 5000.0
      P      = 1000.0
C
      TINV   = QQ1 / T
      T2INV  = TINV**2
      SQRTT  = SQRT(T)
      DNDP   = TINV / BK
      BKEVTI = DNDP * QQQ
      ANTOT  = P * DNDP
      DNDT   =  - ANTOT * TINV
C
      Z = AMAX1(1.0E-25, QQ1 - Y - X)
      WGHTMT = (QMOL - QHYD * X - QHEL * Y) / Z
C
      C0 = X * POTH + Y * POTHI
      C1 = QI2 / (QQ2 - Z)
      C2 = QQ1 + X
      C3 = QQ1 + Y
      C4 = QQ2 - Z
      C5 = QQ1 - Z
      C6 = QQ1 + Y - Z
      C7 = C6 + QQ1
      C8 = QQ1 / C7
      C9 = QQ1 / (QQ2 - X)
      C11 = (X + X) * (QQ4 - X - X)
      C12 = QI2 * X
      C13 = QI2 * C9
      C14 = C5 / C4
      C16 = QI2 * C2
      C15 = C16 * C16
      C17 = (C4 + C4) * (C6 + C6)
C
      RO = QMASS * (QHYD * X + QHEL * Y + WGHTMT * Z)
      C30 = C11 * QI2
      C21 = QMASS * QGAS / RO
      C22 = QQ1 + QQ1 / (X + Y + Y)
      C23 = QQ2 / X
      IF (IFION .EQ. 0  .OR.  IFION .EQ. 2  .OR.  IFION .EQ. 15) 
     >   C22 = QQ0
      IF (IFION .EQ .0  .OR.  (IFION .GT. 5  .AND.  IFION .NE. 13) )
     >   C23 = QQ0
      ELCON = QEL * (X + Y + Y) / RO
      C31 = RO * C9
      C32 = C17 * QI2
      C33 = C8 * RO
      ALG2 = ALOG (QQ2)
C
      D0 = Q15 * ALOG (QPI2 * QMASS / QH * BK / QH)
      D1 = D0 + Q15 * ALG2
      D2 = D0 + ALG2 + ALG2 + ALG2
      D3 = D0
      IF (WGHTMT .GT. QQ0) D3 = D3 + Q15 * ALOG (WGHTMT)
      D4 = QQ3 * (D0 - QI2 * ALOG(QPE) + ALG2 + ALG2) + ALG2
      D5 = D0 + ALG2
      D6 = D4 - D2 + ALG2
      D7 = D6 - ALG2 * QQ4
      D8 = Q25
      IF (X .GT. QQ0) D8 = D8 + X * (D5 - ALOG(X))
      IF (Y .GT. QQ0) D8 = D8 + Y * (D2 - ALOG(Y))
      IF (Z .GT. QQ0) D8 = D8 + Z * (D3 - ALOG(Z))
      D9 = X + X + Y + Y + Y + Z
      D10 = QQ0
      IF (X .GT. QQ0) D10 = D10 + X * (D7 - ALOG(X*C6))
      IF (Y .GT. QQ0) D10 = D10 + Y * (D4 - ALOG(Y*C6*C6) - 
     >                                 QQ2 * ALG2 - Q15 * ALOG(QPE))
      IF (Z .GT. QQ0) D10 = D10 + Z * (D3 - ALOG(Z))
      IF (Y .EQ. QQ0) RETURN
C
C
C-----------------------------------------------------------------------
C   Determination of boundary lines for different P/T regions
C-----------------------------------------------------------------------
C
 1000 KP = KP + 1
      IF (KP .GT. 12) GOTO 1240
      PP = QLP1**(KP - 4)
      LP = 1
      IF (KP .GT. 1) LP = 2
      IP = LP - 1
C
 1010 IP = IP + 1
      IF (IP .EQ. 4) GOTO 1225
      IF (LP .EQ. 1) GOTO 1020
      DO 1011 I = 1,4
         TT (I,1) = TT (I,3)
 1011 CONTINUE
C
 1020 Y1 = QQ0
      Y2 = QQ0
      P = PP
      IF (IP .EQ. 1) P = QLM1 * P
      IF (IP .EQ. 2) P = Q04 * P
      LOW = 1
      IT = 0
C
 1030 IT = IT + 1
      IF (IT .GT. 5000) GOTO 1215
      T = FLOAT (IT*500 + 500)
      TINV = QQ1 / T
      T2INV = TINV**2
      SQRTT = SQRT(T)
      DNDP = TINV / BK
      BKEVTI = QBKT * TINV
      ANTOT = DNDP * P
      ANTOT2 = ANTOT * QI2
      ANE = QQ0
      ANH2 = QQ0
      ANHEN = QQ0
      ANHEP = QQ0
      ANHEPP = QQ0
      KENTER = LOW
      IBACK = 0
      IF (KENTER .EQ. 0) GOTO 1035
      GOTO (100, 300, 500, 500, 600), KENTER
C
 1035 IF (LOW .EQ. 1) ANHEN = Y * (ANTOT + ANH2)
      IF (LOW .EQ. 2) ANHEN = Y * (ANTOT - ANE)
      IF (LOW .EQ. 3) GOTO 1090
      IF (LOW .EQ. 5) GOTO 1130
      XNH2 = ANH2
      XNHEN = ANHEN
      ANE = QQ0
      ANH2 = QQ0
      ANHEN = QQ0
      ANHEP = QQ0
      ANHEPP = QQ0
      KENTER = LOW + 1
      IBACK = 1
      IF (KENTER .EQ. 0) GOTO 1040
      GOTO (100, 300, 500, 500, 600), KENTER
C
 1040 IF (LOW .EQ. 1) GOTO 1050
      IF (LOW .EQ. 4) GOTO 1110
      CALL EXIT
C
 1050 IF (XNH2 .LT. ANE  .OR.  XNH2 .LE. QQ0) GOTO 1060
      Y2 = XNH2
      Y1 = ANE
      GOTO 1030
C
 1060 IF (XNH2 .LE. QQ0  .OR.  IT .EQ. 1) GOTO 1070
      TT (1,IP) = T - Q52 + Q52*(Y2 - Y1) / (ANE + Y2 - Y1 - XNH2)
      GOTO 1080
C
 1070 TT (1,IP) = T
C
 1080 LOW = 3
      GOTO 1030
C
 1090 H1 = SQRT(F2) * SQRT ( F2*C15 + ANTOT*X ) - F2 * C16
      IF ( ABS(ANE-H1) .GT. ABS(ANE-ANTOT2) ) GOTO 1100
      Y1 = ABS(ANE-ANTOT2)
      Y2 = ABS(ANE-H1)
      GOTO 1030
C
 1100 TT (2,IP) = Q52 * ( ABS(ANE-ANTOT2) - Y1) / 
     >                  ( Y2 + ABS(ANE-ANTOT2) - Y1- ABS(ANE-H1) ) +
     >            T - Q52
      LOW = 4
      GOTO 1030
C
 1110 IF (XNHEN .LT. ANHEPP) GOTO 1120
      Y3 = XNHEN
      Y5 = ANHEPP
      GOTO 1030
C
 1120 TT (3,IP) = T - Q52 + Q52*(Y3 - Y5) / (ANHEPP + Y3 - Y5 - XNHEN)
      LOW = 5
      GOTO 1030
C
 1130 IF (ANHEP .LT. (QLM4*ANHEPP) ) GOTO 1140
      Y4 = ANHEP
      Y5 = QLM4 * ANHEPP
      GOTO 1030
C
 1140 TT (4,IP) = T - Q52 + Q52*(Y4 - Y5) / 
     >                          (QLM4*ANHEPP + Y4 - Y5 - ANHEP)
      GOTO 1010
C
 1200 IF (IBACK .EQ. 0) GOTO 1035
      GOTO 1040
C
 1215 CALL EXIT
C
 1225 Q = QQ1 / (QF0 * PP)
      DO 1230 I = 1,4
         BORDER (KP,I,1) = ( QF1 * TT(I,1) + QF2 * TT(I,2) + 
     >                    QQ2 * TT(I,3) ) * Q * PP
         BORDER (KP,I,2) = ( QF3 * TT(I,1) + QF4 * TT(I,2) + 
     >                    QF5 * TT(I,3) ) * Q
         BORDER (KP,I,3) = ( QLP2 * TT(I,1) + QF6 * TT(I,2) + 
     >                    QF7 * TT(I,3) ) * Q / PP
 1230 CONTINUE
      GOTO 1000
C
 1240 P = WGHTMT
      T = ELCON
      RETURN
C
C
C-----------------------------------------------------------------------
C   Compute common thermodynamical quantities (KENTER >= 0)   
C-----------------------------------------------------------------------
C
  777 IRUN = 0
      TINV   = QQ1 / T
      T2INV  = TINV**2
      SQRTT  = SQRT(T)
      DNDP   = TINV / BK
      BKEVTI = DNDP * QQQ
      ANTOT  = P * DNDP
      DNDT   =  - ANTOT * TINV
C----      IF (NEWTON .LE. 0) GOTO 99
C----      D2NDT2 =  - (DNDT + DNDT)*TINV
C----      D2NDPT =  - DNDP * TINV
   99 IF (IFION .EQ. 0) GOTO 200
C
C-----------------------------------------------------------------------
C   Jump to appropriate formula in subsequent GAS calls (KENTER > 0)
C-----------------------------------------------------------------------
C
      GOTO (100, 200, 300, 500, 500, 600, 700), KENTER
C
C
C-----------------------------------------------------------------------
C   Recompute KENTER in first call of GAS for grid points (KENTER = 0)
C-----------------------------------------------------------------------
C
      IF (IFION .GT. 12) GOTO 56
C
      IF (P .GE. QLP2) GOTO 12
      IF (P .GE. QQ1)  GOTO 8
      IF (P .GE. QLM2) GOTO 4
      IF (P .GE. QLM3) GOTO 2
      IF (P .LE. QLM4) GOTO 25
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -4 < log p < -3 
C++++++++++++++++++++++++++++++++++++++
      T1 = G1 + P * (G2 + P * G3)
      IF (T .LE. T1) GOTO 51
      T3 = G7 + P * (G8 + P * G9)
      IF (T .GE. T3) GOTO 1
      T2 = G4 + P * (G5 + P * G6)
      GOTO 24
    1 T4 = G10 + P * (G11 + P * G12)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -3 < log p < -2
C++++++++++++++++++++++++++++++++++++++
    2 T1 = G13 + P * (G14 + P * G15)
      IF (T .LE. T1) GOTO 51
      T3 = G19 + P * (G20 + P * G21)
      IF (T .GE. T3) GOTO 3
      T2 = G16 + P * (G17 + P * G18)
      GOTO 24
    3 T4 = G22 + P * (G23 + P * G24)
      IF (T .LT. T4) GOTO 54
      GOTO 55
    4 IF (P .GE. QLM1) GOTO 6
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -2 < log p < -1 
C++++++++++++++++++++++++++++++++++++++
      T1 = G25 + P * (G26 + P * G27)
      IF (T .LE. T1) GOTO 51
      T3 = G31 + P * (G32 + P * G33)
      IF (T .GE. T3) GOTO 5
      T2 = G28 + P * (G29 + P * G30)
      GOTO 24
    5 T4 = G34 + P * (G35 + P * G36)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  -1 < log p < 0 
C++++++++++++++++++++++++++++++++++++++
    6 T1 = G37 + P * (G38 + P * G39)
      IF (T .LE. T1) GOTO 51
      T3 = G43 + P * (G44 + P * G45)
      IF (T .GE. T3) GOTO 7
      T2 = G40 + P * (G41 + P * G42)
      GOTO 24
    7 T4 = G46 + P * (G47 + P * G48)
      IF (T .LT. T4) GOTO 54
      GOTO 55
    8 IF (P .GE. QLP1) GOTO 10
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  0  < log p < +1 
C++++++++++++++++++++++++++++++++++++++
      T1 = G49 + P * (G50 + P * G51)
      IF (T .LE. T1) GOTO 51
      T3 = G55 + P * (G56 + P * G57)
      IF (T .GE. T3) GOTO 9
      T2 = G52 + P * (G53 + P * G54)
      GOTO 24
    9 T4 = G58 + P * (G59 + P * G60)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +1 < log p < +2 
C++++++++++++++++++++++++++++++++++++++
   10 T1 = G61 + P * (G62 + P * G63)
      IF (T .LE. T1) GOTO 51
      T3 = G67 + P * (G68 + P * G69)
      IF (T .GE. T3) GOTO 11
      T2 = G64 + P * (G65 + P * G66)
      GOTO 24
   11 T4 = G70 + P * (G71 + P * G72)
      IF (T .LT. T4) GOTO 54
      GOTO 55
   12 IF (P .GE. QLP4) GOTO 16
      IF (P .GE. QLP3) GOTO 14
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +2 < log p < +3 
C++++++++++++++++++++++++++++++++++++++
      T1 = G73 + P * (G74 + P * G75)
      IF (T .LE. T1) GOTO 51
      T3 = G79 + P * (G80 + P * G81)
      IF (T .GE. T3) GOTO 13
      T2 = G76 + P * (G77 + P * G78)
      GOTO 24
   13 T4 = G82 + P * (G83 + P * G84)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +3 < log p < +4 
C++++++++++++++++++++++++++++++++++++++
   14 T1 = G85 + P * (G86 + P * G87)
      IF (T .LE. T1) GOTO 51
      T3 = G91 + P * (G92 + P * G93)
      IF (T .GE. T3) GOTO 15
      T2 = G88 + P * (G89 + P * G90)
      GOTO 24
   15 T4 = G94 + P * (G95 + P * G96)
      IF (T .LT. T4) GOTO 54
      GOTO 55
   16 IF (P .GE. QLP6) GOTO 20
      IF (P .GE. QLP5) GOTO 18
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +4 < log p < +5 
C++++++++++++++++++++++++++++++++++++++
      T1 = G97 + P * (G98 + P * G99)
      IF (T .LE. T1) GOTO 51
      T3 = G103 + P * (G104 + P * G105)
      IF (T .GE. T3) GOTO 17
      T2 = G100 + P * (G101 + P * G102)
      GOTO 24
   17 T4 = G106 + P * (G107 + P * G108)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +5 < log p < +6 
C++++++++++++++++++++++++++++++++++++++
   18 T1 = G109 + P * (G110 + P * G111)
      IF (T .LE. T1) GOTO 51
      T3 = G115 + P * (G116 + P * G117)
      IF (T .GE. T3) GOTO 19
      T2 = G112 + P * (G113 + P * G114)
      GOTO 24
   19 T4 = G118 + P * (G119 + P * G120)
      IF (T .LT. T4) GOTO 54
      GOTO 55
   20 IF (P .GE. QLP7) GOTO 22
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +6 < log p < +7 
C++++++++++++++++++++++++++++++++++++++
      T1 = G121 + P * (G122 + P * G123)
      IF (T .LE. T1) GOTO 51
      T3 = G127 + P * (G128 + P * G129)
      IF (T .GE. T3) GOTO 21
      T2 = G124 + P * (G125 + P * G126)
      GOTO 24
   21 T4 = G130 + P * (G131 + P * G132)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  +7 < log p < +8 
C++++++++++++++++++++++++++++++++++++++
   22 T1 = G133 + P * (G134 + P * G135)
      IF (T .LE. T1) GOTO 51
      T3 = G139 + P * (G140 + P * G141)
      IF (T .GE. T3) GOTO 23
      T2 = G136 + P * (G137 + P * G138)
      GOTO 24
   23 T4 = G142 + P * (G143 + P * G144)
      IF (T .LT. T4) GOTO 54
      GOTO 55
C
   24 IF (T .LT. (T1+T1)) GOTO 52
      KENTER = 4
      IF (T .GE. T2) KENTER = 5
      GOTO 53
C
C++++++++++++++++++++++++++++++++++++++
C   Pressure  log p < -4 
C++++++++++++++++++++++++++++++++++++++
   25 IF (T .LE. QT1) GOTO 51
      IF (T .GE. QT2) GOTO 26
      IF (T .LT. QT3) GOTO 52
      KENTER = 4
      IF (T .GE. QT4) KENTER = 5
      GOTO 53
   26 IF (T .LT. QT5) GOTO 54
      GOTO 55
C
   51 GOTO (100, 100, 100, 100, 100, 300, 300, 300, 300, 500, 600, 700),
     >     IFION
   52 GOTO (300, 100, 300, 300, 300, 300, 300, 300, 300, 500, 600, 700),
     >     IFION
   53 GOTO (500, 100, 300, 500, 500, 300, 500, 500, 500, 500, 600, 700),
     >     IFION
   54 GOTO (600, 100, 300, 500, 600, 300, 500, 600, 600, 600, 600, 700),
     >     IFION
   55 GOTO (700, 100, 300, 500, 600, 300, 500, 600, 700, 700, 700, 700),
     >     IFION
   56 IF (IFION .EQ. 16) GOTO 700
      IF (IFION .EQ. 15) GOTO 200
      GOTO 300
C
C
C-----------------------------------------------------------------------
C   Low temperature: The atmosphere consists of H2, H, He, M
C-----------------------------------------------------------------------
C
  100 SQRTTI=TINV/SQRTT
      PAR=QCON1+T*(QCON2+T*(QCON3+T*(QCON4+T*QCON5)))
      IF(PAR.LE.QQ0.AND.IFION.NE.2.AND.KP.EQ.0) GOTO 300
      IF(PAR.LE.QQ0.AND.IFION.EQ.2.AND.KP.EQ.0) GOTO 200
      PAR=QQ1/PAR
      H1=Q2P1*TINV
      H2=Q2P2*TINV
      PARM=(PAR-QQ1)*H2
      EFKT=EXP(H1+Q2P3)
      F11=T*(A1+T*(A2+T*A3))
      QH2=PAR*F11
      F1=QH2*SQRTTI*EFKT
      F1INV=QQ1/F1
      H3=C13*F1INV
      C11F1A = C11 * F1 * ANTOT
      W = SQRT( QQ1 + C11F1A )
      IF (C11F1A .LE. HLIMIT) SQRTM1 = C11F1A * QI2 * 
     >                       (QQ1 - C11F1A * QI4 * (QQ1 - C11F1A * QI2))
      IF (C11F1A .GT. HLIMIT) SQRTM1 = W - QQ1
      ANHN = SQRTM1 * F1INV * C13
      IF (IFION .EQ .13  .AND.  (F1*ANHN) .LE. QLM3  .AND.  KP .EQ. 0)
     >   GOTO 200
      ANH2=F1*ANHN*ANHN
      ANA=(ANTOT+ANTOT-ANHN)*C9
      ANHEN=ANA*Y
      ANMN=ANA*Z
      ANE=QQ0
      ANHP=QQ0
      ANHEP=QQ0
      ANHEPP=QQ0
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      DF11=A1+T*(A2+A2+(A3+A3+A3)*T)
      DPR=PAR*PARM*TINV
      DQH2=DPR*F11+DF11*PAR
      DF1DT=EFKT*SQRTTI*TINV*(T*DQH2-QH2*(H1+Q15))
      C=DF1DT*F1INV
      F2=(T*C+Q15)*T*BK
      ENERGY=(Q15*P+ANH2*F2)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=1
C
      IF(IFENTR.EQ.0) GOTO 150
      S = ANTOT * ( Q25 + Q15 * ALOG(T) ) - 
     >    ANH2 * (ALOG(ANH2/QH2) - T / QH2 * DQH2 - D1) -
     >    ANHN * (ALOG(ANHN) - D5)
      IF (ANHEN .NE. QQ0) S = S - ANHEN * (ALOG(ANHEN) - D2)
      IF (ANMN .NE. QQ0) S = S - ANMN * (ALOG(ANMN) - D3)
      S = BK * RHOINV * S
C
  150 CONTINUE
      D2PAR=TINV*((PAR+PAR-QQ1)*DPR*H2-(PAR+PAR)*TINV*PARM)
      D2F11=QQ2*(A2+(A3+A3+A3)*T)
      D2QH2=D2PAR*F11+(DPR+DPR)*DF11+PAR*D2F11
      DF1DT2=TINV*(EFKT*SQRTTI*(T*D2QH2-DQH2*(H1+QQ3)+
     >   TINV*QH2*(Q37+Q35*H1))-DF1DT*H1)
      WINV=QQ1/W
      DWDP=C30*WINV*F1*DNDP
      DWDT=C30*WINV*(DF1DT*ANTOT+F1*DNDT)
      DCDP=H3*DWDP
      DCDT = H3 * (DWDT - C * SQRTM1)
      DRDT=C31*(DNDT+DNDT-DCDT)
      DRDP=C31*(DNDP+DNDP-DCDP)
      DF2DT=TINV*F2+BK*T*T*(DF1DT2*F1INV+C*(TINV-C))
      DEDT=RHOINV*(ANHN*(ANHN*(DF1DT*F2+F1*DF2DT)+DCDT*(F1+F1)*F2)-
     >   ENERGY*DRDT)
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      D3PAR=H2*TINV*(D2PAR*(PAR+PAR-QQ1)+(DPR+DPR)*(DPR-
C----     >   (TINV+TINV)*(PAR+PAR-QQ1))+QQ6*(PAR-QQ1)*PAR*T2INV)
C----      D3QH2=QQ6*A3*PAR+QQ3*(D2F11*DPR+DF11*D2PAR)+F11*D3PAR
C----      DF1DT3=TINV*(H1*((TINV+TINV)*DF1DT-DF1DT2)+EFKT*SQRTTI*
C----     >   (T*D3QH2-D2QH2*(Q45+H1+H1)+TINV*(((H1+QQ5)**2+Q14)*
C----     >   DQH2-TINV*QH2*((Q45*H1+Q17)*H1+Q13))))
C----      D2WDP2=-DWDP**2*WINV
C----      D2WDPT=(C30*(DF1DT*DNDP+F1*D2NDPT)-DWDP*DWDT)*WINV
C----      D2WDT2=WINV*(C30*(DF1DT2*ANTOT+(DNDT+DNDT)*DF1DT+F1*D2NDT2)-
C----     >   DWDT**2)
C----      D2CDP2=H3*D2WDP2
C----      D2CDPT=H3*(D2WDPT-C*DWDP)
C----      D2CDT2=H3*(D2WDT2-(C+C)*DWDT+(W-QQ1)*((C+C)*C-DF1DT2*F1INV))
C----      D2RDT2=C31*(D2NDT2+D2NDT2-D2CDT2)
C----      D2RDPT=C31*(D2NDPT+D2NDPT-D2CDPT)
C----      D2RDP2=-C31*D2CDP2
C----      DF2DT2=TINV*(DF2DT-TINV*F2)+
C----     >   BK*T*(QQ2*F1INV*(DF1DT2-DF1DT*(TINV-C))+
C----     >         T*(F1INV*DF1DT3-F1INV*F1INV*DF1DT2*DF1DT+
C----     >            (F1INV*DF1DT2-C*C)*(TINV-QQ2*C)-TINV*TINV*C))
C----      D2EDT2=RHOINV*(ANHN*(ANHN*(DF1DT2*F2+(DF1DT+DF1DT)*DF2DT+
C----     >   DF2DT2*F1)+QQ4*DCDT*(F1*DF2DT+F2*DF1DT)+D2CDT2*(F1+F1)*F2)+
C----     >   (F1+F1)*F2*DCDT**2-ENERGY*D2RDT2-(DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Special Case. 
C   The atmosphere consists of uncharged particles: H, He, M
C-----------------------------------------------------------------------
C
  200 RHO=RO*ANTOT
      RHOINV=QQ1/RHO
      ENERGY=Q15*P*RHOINV
      ENTALP=ENERGY+P*RHOINV
      ANA=ANTOT
      ANHN=ANA*X
      ANHEN=ANA*Y
      ANMN=ANA*Z
      ANH2=QQ0
      ANHP=QQ0
      ANHEP=QQ0
      ANHEPP=QQ0
      ANE=QQ0
      IF(KP.EQ.0) KENTER=2
C
      IF(IFENTR.NE.0) S=BK*RHOINV*(ANTOT*(Q15*ALOG(T)-ALOG(ANTOT)+D8))
C
      DRDP=RO*DNDP
      DRDT=RO*DNDT
      DEDT=ENERGY*TINV
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      D2RDT2=RO*D2NDT2
C----      D2RDPT=RO*D2NDPT
C----      D2RDP2=QQ0
C----      D2EDT2=QQ0
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Moderate temperature: The atmosphere consists of H, H+, He, M
C-----------------------------------------------------------------------
C
  300 F12=QPOT6*BKEVTI+Q15
      F2=EXP(QPOT7-F12)*SQRTT*T
      SRF2 = SQRT(F2)
      WFAC = SQRT(F2*C15+ANTOT*X)
      W    = SRF2*WFAC
      ANE=W-F2*C16
      IF(IFION.EQ.13.AND.ANE.LE.(EPSIT*ANTOT).AND.KP.EQ.0) GOTO 100
      ANA=ANTOT-ANE
      ANMN=ANA*Z
      ANHEN=ANA*Y
      ANHN=ANA*X-ANE
      ANH2=QQ0
      ANHP=ANA*X-ANHN
      ANHEP=QQ0
      ANHEPP=QQ0
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      ENERGY=(Q15*P+ANE*POTH)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=3
C
      IF(IFENTR.EQ.0) GOTO 350
      S=ANTOT*(Q25+Q15*ALOG(T))-
     >   ANHN*(ALOG(ANHN)-D5)-ANE*(ALOG(ANE**2)-D7)
      IF(ANHEN.NE.QQ0) S=S-ANHEN*(ALOG(ANHEN)-D2)
      IF(ANMN.NE.QQ0) S=S-ANMN*(ALOG(ANMN)-D3)
      S=BK*RHOINV*S
C
  350 CONTINUE
C--- DF2DT --> DF2DT / F2
C--- DWDP  --> DWDP  / SRF2
C--- DWDT  --> DWDT  / SRF2
      F2WINV=SRF2/WFAC
      DF2DT=T2INV*(Q15*T+QPOT8)
      DWDP=C12*DNDP/WFAC
      DWDT=(C15*F2*DF2DT + C12*(DF2DT*ANTOT + DNDT))/WFAC
      DCDT=SRF2*DWDT-F2*DF2DT*C16
      DRDT=RO*(DNDT-DCDT)
      DRDP=RO*(DNDP-SRF2*DWDP)
      DEDT=RHOINV*(POTH*DCDT-ENERGY*DRDT)
C
C--- DF2DT2  --> DF2DT2  / F2
C----      IF(NEWTON.LE.0) GOTO 999
C----      DF2DT2=T2INV*(DF2DT*(Q15*T+QPOT8)-(Q15+TINV*QPOT9))
C----      D2WDP2=-F2WINV*DWDP**2
C----      D2WDPT=F2WINV*(C12*(DF2DT*DNDP+D2NDPT)-DWDT*DWDP)
C----      D2WDT2=F2WINV*(C15*F2*(DF2DT**2 + DF2DT2) +
C----     >       C12*(DF2DT2*ANTOT+(DNDT+DNDT)*DF2DT+D2NDT2)-DWDT**2)
C----      D2CDT2=D2WDT2-F2*DF2DT2*C16
C----      D2RDT2=RO*(D2NDT2-D2CDT2)
C----      D2RDPT=RO*(D2NDPT-D2WDPT)
C----      D2RDP2=-RO*D2WDP2
C----      D2EDT2=RHOINV*(POTH*D2CDT2-ENERGY*D2RDT2-(DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Medium temperature: The atmosphere consists of H, H+, He, He+, M
C-----------------------------------------------------------------------
C
  500 F12 = QPOT6 * BKEVTI + Q15
      F2 = EXP( QPOT7-F12 ) * SQRTT * T
      F13 = QPOT1 * BKEVTI + Q15
      F3 = EXP( QPOT2-F13 ) * SQRTT * T
C
      F2PF3 = (F2 + F3)
      IF (F2PF3 .GT. 1.0) FACT = F2PF3
      IF (F2PF3 .LE. 1.0) FACT = 1.0 / F2PF3
      F2 = F2 / FACT
      F3 = F3 / FACT
      ANTOT = ANTOT / FACT
C
      F11 = C4 * F2 * F3
      F14 = F11 * C14
      F23 = F2 * X + F3 * Y
      B1 = F2 * C2 + F3 * C3
      B2 = F11 - ANTOT * F23
      ANE = QI2 * ANTOT
      IF (KENTER .EQ. 4) ANE = SQRT( F2 * (F2*C15 + ANTOT*X) ) 
     >                         - F2 * C16
  543 H2 = (ANE+B1) * ANE
      H3 = H2 + B2
      FDIVDF = ( H3*ANE - ANTOT*F14 ) / ( ANE*ANE + H2 + H3 )
      ANE = ANE - FDIVDF
      IF (ABS(FDIVDF) .GE. (EPSIT*ANE)) GOTO 543
C
      ANTOT = FACT * ANTOT
      ANE = FACT * ANE
      F2 = FACT * F2
      F3 = FACT * F3
C
      ANA = ANTOT - ANE
      H2 = QQ1 / (ANE + F2)
      H3 = QQ1 / (ANE + F3)
      ANHN = ANA * X * H2 * ANE
      ANHP = ANA * X - ANHN
      ANHEN = ANA * Y * H3 * ANE
      ANHEP = ANA * Y - ANHEN
      ANMN = ANA * Z
      ANH2 = QQ0
      ANHEPP = QQ0
      RHO = RO * ANA
      RHOINV = QQ1 / RHO
      ENERGY = (Q15*P + ANHP*POTH + ANHEP*POTHE) * RHOINV
      ENTALP = ENERGY + P*RHOINV
C
      IF (IFENTR .EQ. 0) GOTO 550
      S=ANTOT*(Q25+Q15*ALOG(T))-
     >   ANHN*(ALOG(ANHN)-D5)-ANHP*(ALOG(ANHP*ANE)-D7)
      IF(ANHEN.GT.QQ0) S=S-ANHEN*(ALOG(ANHEN)-D2)
      IF(ANHEP.GT.QQ0) S=S-ANHEP*(ALOG(ANHEP*ANE)-D6)
      IF(ANMN.GT.QQ0) S=S-ANMN*(ALOG(ANMN)-D3)
      S=BK*RHOINV*S
C
  550 CONTINUE
C
      ANE = ANE / FACT
      ANTOT = ANTOT / FACT
      ANA = ANA / FACT
      DNDT = DNDT / FACT
      DNDP = DNDP / FACT
      ANHP = ANHP / FACT
      ANHEP = ANHEP / FACT
      F2 = F2 / FACT
      F3 = F3 / FACT
      H2 = H2 * FACT
      H3 = H3 * FACT
C
      DF2DT=F2*T2INV*(Q15*T+QPOT8)
      DF3DT=F3*T2INV*(Q15*T+QPOT4)
      B1T=DF2DT*C2+DF3DT*C3
      B2T=C4*(DF2DT*F3+F2*DF3DT)-DNDT*(F2*X+F3*Y)-
     >   ANTOT*(DF2DT*X+DF3DT*Y)
      B3T=C5*(DNDT*F2*F3+ANTOT*(DF2DT*F3+F2*DF3DT))
      B2P=-DNDP*F23
      FDIVDF=-QQ1/(ANE*(ANE+ANE+ANE+B1+B1)+B2)
      DCDP=FDIVDF*(ANE*B2P-F14*DNDP)
      DCDT=FDIVDF*((ANE*B1T+B2T)*ANE-B3T)
      DRDT=RO*(DNDT-DCDT)
      DRDP=RO*(DNDP-DCDP)
      DXDT=H2*(X*(F2*(DNDT-DCDT)+DF2DT*ANA)-ANHP*(DCDT+DF2DT))
      DYDT=H3*(Y*(F3*(DNDT-DCDT)+DF3DT*ANA)-ANHEP*(DCDT+DF3DT))
      DEDT=RHOINV*(POTH*DXDT+POTHE*DYDT-ENERGY*DRDT)
C
      DCDP = FACT * DCDP
      DCDT = FACT * DCDT
      DRDP = FACT * DRDP
      DRDT = FACT * DRDT
      DXDT = FACT * DXDT
      DYDT = FACT * DYDT
      DEDT = FACT * DEDT
C
      ANE = FACT * ANE
      ANTOT = FACT * ANTOT
      ANA = FACT * ANA
      DNDT = FACT * DNDT
      DNDP = FACT * DNDP
      ANHP = FACT * ANHP
      ANHEP = FACT * ANHEP
      H2 = FACT * H2
      H3 = FACT * H3
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      DF2DT2=T2INV*(DF2DT*(Q15*T+QPOT8)-F2*(Q15+TINV*QPOT9))
C----      DF3DT2=T2INV*(DF3DT*(Q15*T+QPOT4)-F3*(Q15+TINV*QPOT5))
C----      D2FDE2=QQ2*(ANE+ANE+ANE+B1)
C----      B1TT=DF2DT2*C2+DF3DT2*C3
C----      B2TT=C4*(DF2DT2*F3+(DF2DT+DF2DT)*DF3DT+F2*DF3DT2)-D2NDT2*F23-
C----     >   (DNDT+DNDT)*(DF2DT*X+DF3DT*Y)-ANTOT*(DF2DT2*X+DF3DT2*Y)
C----      B3TT=C5*(ANTOT*(DF2DT2*F3+(DF2DT+DF2DT)*DF3DT+F2*DF3DT2)+
C----     >   (DNDT+DNDT)*(DF2DT*F3+F2*DF3DT)+F2*F3*D2NDT2)
C----      D2FDT2=ANE*(ANE*B1TT+B2TT)-B3TT
C----      D2FDPT=-ANE*(D2NDPT*F23+DNDP*(DF2DT*X+DF3DT*Y))-
C----     >   C5*(DNDP*(DF2DT*F3+F2*DF3DT)+F2*F3*D2NDPT)
C----      D2FDTE=(ANE+ANE)*B1T+B2T
C----      D2CDT2=FDIVDF*((D2FDE2*DCDT+D2FDTE+D2FDTE)*DCDT+D2FDT2)
C----      D2CDPT=FDIVDF*((D2FDE2*DCDT+D2FDTE)*DCDP+B2P*DCDT+D2FDPT)
C----      D2CDP2=FDIVDF*(D2FDE2*DCDP+B2P+B2P)*DCDP
C----      D2RDT2=RO*(D2NDT2-D2CDT2)
C----      D2RDP2=-RO*D2CDP2
C----      D2RDPT=RO*(D2NDPT-D2CDPT)
C----      D2XDT2=H2*(X*(F2*(D2NDT2-D2CDT2)+(DF2DT+DF2DT)*(DNDT-DCDT)+
C----     >   ANA*DF2DT2)-(DXDT+DXDT)*(DCDT+DF2DT)-ANHP*(D2CDT2+DF2DT2))
C----      D2YDT2=H3*(Y*(F3*(D2NDT2-D2CDT2)+(DF3DT+DF3DT)*(DNDT-DCDT)+
C----     >   ANA*DF3DT2)-(DYDT+DYDT)*(DCDT+DF3DT)-ANHEP*(D2CDT2+DF3DT2))
C----      D2EDT2=RHOINV*(POTH*D2XDT2+POTHE*D2YDT2-ENERGY*D2RDT2-
C----     >   (DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   High temperature: The atmosphere consists of H+, He+, He++, M
C-----------------------------------------------------------------------
C
  600 F14 = QPOT3 * BKEVTI + Q15
      F4 = EXP( QPOT7-F14 ) * SQRTT * T
      IF (F4 .GT. 1.0) FACT = F4
      IF (F4 .LE. 1.0) FACT = 1.0 / F4
      F4 = F4 / FACT
      ANTOT = ANTOT / FACT
      F3 = ( F4*C7 - ANTOT*C5 )
      W = SQRT( C17*ANTOT*F4 + F3*F3 ) 
      ANE = ABS( W-F3 ) * C1
C
      F4 = F4 * FACT
      ANTOT = ANTOT * FACT
      F3 = F3 * FACT
      W = W * FACT
      ANE = ANE * FACT
C
      ANA = ANTOT - ANE
      H2=QQ1/(ANE+F4)
      ANHP=ANA*X
      ANHEP=ANA*Y*H2*ANE
      ANHEPP=ANA*Y-ANHEP
      IF(ANHEPP.LT.QQ0) ANHEPP=QQ0
      ANMN=ANA*Z
      ANH2=QQ0
      ANHN=QQ0
      ANHEN=QQ0
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      ENERGY=(ANHP*POTH+ANHEP*POTHE+ANHEPP*POTHI+Q15*P)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=6
C
      IF(IFENTR.EQ.0) GOTO 650
      S=ANTOT*(Q25+Q15*ALOG(T))-ANHP*(ALOG(ANHP*ANE)-D7)
      IF(ANHEP.NE.QQ0) S=S-ANHEP*(ALOG(ANHEP*ANE)-D6)
      IF (ANHEPP .NE. QQ0) 
     >   S = S - ANHEPP * (ALOG(ANHEPP) + 2.0 * ALOG(ANE) - D4 +
     >                      QQ2 * ALG2 + Q15 * ALOG(QPE))
      IF(ANMN.NE.QQ0) S=S-ANMN*(ALOG(ANMN)-D3)
      S=BK*RHOINV*S
C
  650 CONTINUE
C
      F3 = F3 / FACT
      F4 = F4 / FACT
      W = W / FACT
      H2 = H2 * FACT
      ANA = ANA / FACT
      ANE = ANE / FACT
      ANTOT = ANTOT / FACT
      ANHEP = ANHEP / FACT
      DNDT = DNDT / FACT
      DNDP = DNDP / FACT
C
      DF4DT=F4*T2INV*(Q15*T+QPOT10)
      WINV=QQ1/W
      DF3DT=C7*DF4DT-C5*DNDT
      DF3DP=-C5*DNDP
      DWDT=WINV*(C32*(DNDT*F4+ANTOT*DF4DT)+F3*DF3DT)
      DWDP=WINV*(C32*DNDP*F4+F3*DF3DP)
      DCDT=C1*(DWDT-DF3DT)
      DCDP=C1*(DWDP-DF3DP)
      DXDT=DNDT-DCDT
      DRDT=RO*DXDT
      DRDP=RO*(DNDP-DCDP)
      DYDT=H2*(Y*(DXDT*ANE+ANA*DCDT)-ANHEP*(DCDT+DF4DT))
      DEDT=RHOINV*(DXDT*C0+QPOT*DYDT-ENERGY*DRDT)
C
      F3 = F3 * FACT
      F4 = F4 * FACT
      W = W * FACT
      H2 = H2 / FACT
      ANA = ANA * FACT
      ANE = ANE * FACT
      ANTOT = ANTOT * FACT
      ANHEP = ANHEP * FACT
      DNDT = DNDT * FACT
      DNDP = DNDP * FACT
C
      DF4DT = DF4DT * FACT
      WINV = WINV / FACT
      DF3DT = DF3DT * FACT
      DF3DP = DF3DP * FACT
      DWDT = DWDT * FACT
      DWDP = DWDP * FACT
      DCDT = DCDT * FACT
      DCDP = DCDP * FACT
      DXDT = DXDT * FACT
      DRDT = DRDT * FACT
      DRDP = DRDP * FACT
      DYDT = DYDT * FACT
      DEDT = DEDT * FACT
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      DF4DT2=T2INV*(DF4DT*(Q15*T+QPOT10)-F4*(Q15+TINV*QPOT11))
C----      DF3DT2=C7*DF4DT2-C5*D2NDT2
C----      DF3DPT=-C5*D2NDPT
C----      D2WDT2=WINV*(C32*(D2NDT2*F4+(DNDT+DNDT)*DF4DT+ANTOT*DF4DT2)+
C----     >   DF3DT**2+F3*DF3DT2-DWDT**2)
C----      D2WDP2=WINV*(DF3DP**2-DWDP**2)
C----      D2WDPT=WINV*(C32*(D2NDPT*F4+DNDP*DF4DT)+DF3DT*DF3DP+
C----     >   F3*DF3DPT-DWDP*DWDT)
C----      D2CDP2=C1*D2WDP2
C----      D2CDT2=C1*(D2WDT2-DF3DT2)
C----      D2CDPT=C1*(D2WDPT-DF3DPT)
C----      D2XDT2=D2NDT2-D2CDT2
C----      D2RDT2=RO*D2XDT2
C----      D2RDPT=RO*(D2NDPT-D2CDPT)
C----      D2RDP2=-RO*D2CDP2
C----      D2YDT2=H2*(Y*(D2XDT2*ANE+(DXDT+DXDT)*DCDT+ANA*D2CDT2)-
C----     >   (DYDT+DYDT)*(DCDT+DF4DT)-ANHEP*(D2CDT2+DF4DT2))
C----      D2EDT2=RHOINV*(D2XDT2*C0+QPOT*D2YDT2-ENERGY*D2RDT2-
C----     >   (DEDT+DEDT)*DRDT)
      GOTO 999
C
C
C-----------------------------------------------------------------------
C   Very high temperature: The atmosphere consists of H+, He++, M
C-----------------------------------------------------------------------
C
  700 ANA=ANTOT*C8
      ANE=ANA*C6
      ANH2=QQ0
      ANHN=QQ0
      ANHP=ANA*X
      ANHEN=QQ0
      ANHEP=QQ0
      ANHEPP=ANA*Y
      ANMN=ANA*Z
      RHO=RO*ANA
      RHOINV=QQ1/RHO
      ENERGY=(Q15*P+ANA*C0)*RHOINV
      ENTALP=ENERGY+P*RHOINV
      IF(KP.EQ.0) KENTER=7
C
      IF(IFENTR.NE.0) S=BK*RHOINV*(ANTOT*(Q25+Q15*ALOG(T))-
     >   ANA*(ALOG(ANA)*D9-D10))
C
      DRDT=C33*DNDT
      DRDP=C33*DNDP
      DEDT=Q15*P*TINV*RHOINV
C
C----      IF(NEWTON.LE.0) GOTO 999
C----      D2RDT2=C33*D2NDT2
C----      D2RDPT=C33*D2NDPT
C----      D2RDP2=QQ0
C----      D2EDT2=QQ0
C
C
C-----------------------------------------------------------------------
C   Computation of common thermodynamical quantities
C-----------------------------------------------------------------------
C
  999 IF (KP .NE. 0) GOTO 1200
C
      RHOIN2 = RHOINV * RHOINV
      DSDT   = TINV * (DEDT - P * DRDT * RHOIN2)
      DTDS   = QQ1 / DSDT
      DSDP   = RHOIN2 * DRDT
      Y1     = -DSDP * DTDS
      H      = DRDP + DRDT * Y1
C
      IF (H .GT. QQ0  .AND.  DRDP .GE. QQ0) GOTO 888
      IF (IRUN .EQ. 3  .OR.  IFION .EQ. 0) GOTO 888
      IERROR = IERROR + 1
      IRUN = IRUN + 1
      IF (IRUN .EQ. 1) KENNEW = KENTER + 1
      IF (IRUN .EQ. 2) KENNEW = KENTER - 1
      IF (KENNEW .LE. 0) KENNEW = 1
      IF (KENNEW .GT. 7) KENNEW = 7
      KENTER = KENNEW
      GOTO 99
C
  888 CONTINUE
      CS   = QQ1 / SQRT(H)
      XCP  = -RHO * DSDT / DRDT
      XGAM = CS*CS * RHO / P
      CXT  = -RHOINV * DRDT * T
      CT   = QQ1 / SQRT(DRDP)
C----      XMU  = QMOL * ANA / ANTOT
C----      H    = -RHOINV * DRDT * DTDS * CS
C----      DTDP = Y1
C
C----      IF(NEWTON.LE.0) RETURN
C----      D2SDPT=RHOIN2*(D2RDT2-(RHOINV+RHOINV)*DRDT**2)
C----      D2SDP2=RHOIN2*(D2RDPT-(RHOINV+RHOINV)*DRDP*DRDT)
C----      D2SDT2=-TINV*(TINV*DEDT-D2EDT2-P*RHOIN2*(TINV*DRDT+
C----     >   (RHOINV+RHOINV)*DRDT**2-D2RDT2))
C----      H2=-DTDS**2
C----      DTDPP=H2*(DSDT*D2SDP2-DSDP*D2SDPT)
C----      DTDPT=H2*(DSDT*D2SDPT-DSDP*D2SDT2)
C----      DTDSP=H2*D2SDPT
C----      DTDST=H2*D2SDT2
C----      HP=D2RDP2+D2RDPT*DTDP+DRDT*DTDPP
C----      HT=D2RDPT+D2RDT2*DTDP+DRDT*DTDPT
C----      CSP=-QI2*HP*CS**3
C----      CST=-QI2*HT*CS**3
C----      CTP=-QI2*D2RDP2*CT**3
C----      CTT=-QI2*D2RDPT*CT**3
C----      HP=RHOINV*(DRDT*(CS*(RHOINV*DTDS*DRDP-DTDSP)-DTDS*CSP)-
C----     >   D2RDPT*CS*DTDS)
C----      HT=RHOINV*(DRDT*(CS*(RHOINV*DTDS*DRDT-DTDST)-DTDS*CST)-
C----     >   D2RDT2*CS*DTDS)
C
      RETURN
      END SUBROUTINE gas


C      SUBROUTINE gas(P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
      SUBROUTINE gasgas(P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                  IERROR, IFION, IFENT, KENTER, RX, RY, RQMOL)
C     >                  ANHN, ANHP, ANHEN, ANHEP, ANHEPP, ANH2)
********************************************************************************
*
* Version: Fri Jan 23 11:42:12 MET 1998
*
* Author : HGL
*
* Description: GASGAS is (almost) a functional twin to the famous routine GAS. 
*   A GAS-call may be replaced by a GASGAS-call. If the GAS common block /E/ 
*   is in use it has to be replaced by a similar common block /EE/. The meaning 
*   of the control parameters IERROR and KENTER is somewhat different. If these
*   parameters are used in a critical way in the calling module one has to
*   check the consequences.
* 
*   The internal method of determining the requested thermodynamic parameters 
*   is completely different from GAS. While GAS is a time-optimized routine
*   GASGAS is constructed to allow an easy change in the basic physics of the
*   fundamental set of equations which govern the various
*   ionization/dissoziation processes. GASGAS solves the given set of
*   fundamental particel density equations to a preset accuracy (Parameter
*   PREREL). From these densities the various thermodynamic parameters are
*   computed. Input parameters are the gas-pressure and the temperature.
*
* Arguments, access, type, dimension, description:
*   P      R R : Gas pressure
*   T      R R : Temperature
*   RHO    W R : Mass density
*   XCP    W R : C_p(s): Modified specific heat per massunit at constant p
*   XGAM   W R : C_p(s)/C_v(s): Modified ratio of specific heats
*   CXT    W R : C_p = CXT * C_p(s), conversion factor between C_p and C_p(s)
*   ENTALP W R : Enthalpy per massunit
*   S      W R : Entropy per massunit
*   CS     W R : Adiabatic sound velocity
*   IERROR W I : Error indicator:
*                =  0: No error detected
*                <> 0: No convergence during solution of fundamental equations;
*                      in this case don't trust anything which is returned by 
*                      GASGAS!
*   IFION  R I : Controls which ionization/dissoziation processes are
*                considered, see IFION translation table below.
*   KENTER R I : Controls the way GASGAS works:
*                <  0: Nothing is done, except IERROR is set to zero
*                =  0: Compute new initial particel densities, solve
*                      fundamental equations, compute thermodynamic parameters
*                      (this is the standard case)
*                >  0: Start solution of fundamental equations from particel 
*                      densities of previous GASGAS call, compute thermodynamic
*                      quantities. This KENTER value is useful if the requested
*                      p/T-parameters are not much different from the ones of
*                      the last GASGAS call.
*   RX     R R : Hydrogen abundance
*   RY     R R : Helium abundance
*   RQMOL  R R : Mean molecular weight of particels in the case of vanishing
*                ionization and H2-molecule formation
*   
*
* Common blocks (access): EE (W)
*
* External procedures   : BROYDN, EOSCND, FCSAHA, OWNINV, QH2
*
* File units (access)   : * (W)
*
*
* Comments: GASGAS is written in DOUBLE PRECISION!
*
*   Modification history:
*   03/11/97/hgl: coded
*   23/01/98/hgl: variable H_2 dissociation energy introduced but actually 
*     not used. 
*     Common block /EE/ extended to make ANH2 accessible
*     Switch to entropy computation analogous GAS possible via STGGMD
*   02/02/98/bf: PREREL: 1.0e-10 -> 1.0e-13
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************

***   Commons related EOSCND
      INCLUDE 'EOSPAR.INC'

      DOUBLE PRECISION DPART, DNCCM, WT32, DUM, DUMA, DUMB
      DOUBLE PRECISION BINV( MEQ, MEQ), DN( MEQ, 2), YWORK( MEQ, 3)
      DOUBLE PRECISION DNTOTS
***   Define indices of various densities
      PARAMETER( INA=1, INH2=2, INHN=3, INHP=4, INHEN=5, INHEP=6, 
     >           INHEPP=7, INEL=8, INM=9)
      DOUBLE PRECISION PREREL
      PARAMETER( PREREL = 1.0E-13)
      PARAMETER( MIMPS = 30)

      REAL P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS
***   declaration of DT and DP in EOSPAR.INC!
      DOUBLE PRECISION DRHO, DXCP, DXGAM, DCXT, DS, 
     >                 DCS
      DOUBLE PRECISION DE, DH, WTM, QMOL
      REAL DUM1, DUM2, DUM3, DUM4, DUM5, DUM6, DUM7, DUM8, DUM9

***   Thermodynamic auxiliary variables
      DOUBLE PRECISION DEVDTP, DEMDTP, DRODTP, DRODPT, DSDPT, DCVH2, 
     >                 DEH2, DCP
***   QH2 auxiliary variables
      DOUBLE PRECISION D2QDT2
***   QH2T, DLQDLT declared in EOSPAR.INC

***   Ionization energies including possible modifications
      DOUBLE PRECISION EIH, EIHE, EIHEP, EDH2, EDH2DT

***   Double precision real constants
      DOUBLE PRECISION D00, D05, D10, D15, D20, D25, D40, D80
      PARAMETER( D00 = 0.0, D05 = 0.5, D10 = 1.0, D15 = 1.5, D20 = 2.0, 
     >           D25 = 2.5, D40 = 4.0, D80 = 8.0)
C      DOUBLE PRECISION LN10, EXP10
      DOUBLE PRECISION LN10

      INCLUDE 'matnat_23jan98.inc'

***   Commons related to GAS
      REAL ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT, ANH2
      COMMON / EE / ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT, ANH2
      REAL RX, RY, RQMOL

      LOGICAL DUMLOG, GASCMP

      SAVE DN, BINV, GASCMP

C      EXTERNAL EOSCND

***   Statement functions ...
C      EXP10( X) = EXP( LN10 * X)

      DATA LN10 / 2.3025850929940457 /
      DATA GASCMP / .FALSE. /

***====================================================================
      IERROR = 0

***   This routine needs no initialization
      IF ( KENTER .LT. 0) RETURN 

***   Initialize working variables
      X      = DBLE( RX)
      Y      = DBLE( RY)
      Z      = D10 - X - Y
      QMOL   = DBLE( RQMOL)
      IF( Z .LE. D00) THEN
        Z      = D00
        WTM    = D00
      ELSE
        WTM    = (QMOL - WTH * X - WTHE * Y) / Z
      ENDIF

      IS = 1 
      IW = 2
      NEQ = 8

      DP = DBLE( P)
      DT = DBLE( T)

***   Translate IFION to FAC??? parameters:
***      IFION  |  FACGT     FACF1T    FACF2T    FACF3T
***             |   H2        H         He        He+
***      -------|--------------------------------------
***        0    |   0.0       0.0       0.0       0.0
***        1    |   1.0       1.0       1.0       1.0
***        2    |   1.0       0.0       0.0       0.0
***        3    |   1.0       1.0       0.0       0.0
***        4    |   1.0       1.0       1.0       0.0
***        5    |   1.0       1.0       1.0       1.0
***        6    |   0.0       1.0       0.0       0.0
***        7    |   0.0       1.0       1.0       1.0
***      -------|--------------------------------------
***      ELSE   |   Error report and stop
***      -------|--------------------------------------
       IF ( IFION .EQ. 1) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D10
         FACF3T = D10
       ELSEIF ( IFION .EQ. 3) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 0) THEN
         FACGT  = D00
         FACF1T = D00
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 2) THEN
         FACGT  = D10
         FACF1T = D00
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 4) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D10
         FACF3T = D00
       ELSEIF ( IFION .EQ. 5) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D10
         FACF3T = D10
       ELSEIF ( IFION .EQ. 6) THEN
         FACGT  = D00
         FACF1T = D10
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 7) THEN
         FACGT  = D00
         FACF1T = D10
         FACF2T = D10
         FACF3T = D10
       ELSE
         WRITE( *, *) 'Illegal IFION encountered in GASGAS!'
         WRITE( *, *) 'IFION = ', IFION
         STOP
       ENDIF

***   Something like pressure ionization should go here
***   Obs that if it is dependent on particle densities some major
***   rearrangements must be done.
      EIH    = EIH0
      EIHE   = EIHE0
      EIHEP  = EIHEP0
      EDH2   = EDH20
      EDH2DT = D00

***   Set particle density independend quantities ...
***   Compute quantities related to the H2 partition function
      CALL QH2( DT, QH2T, DUM, D2QDT2)
      DLQDLT = DUM / QH2T * DT
***   Compute Saha factors
      CALL FCSAHA( DT, FACGT, FACF1T, FACF2T, FACF3T, 
     >             EIH, EIHE, EIHEP, EDH2, EDH2DT,
     >             QH2T, DLQDLT, GT, F1T, F2T, F3T, 
     >             DGTDT, DF1TDT, DF2TDT, DF3TDT)

***   Compute total particle density
      DNTOT           = DP / ( BOLTZK * DT)
***   New unit of particle density: DNTOT
      GT  = GT * DNTOT
      F1T = F1T / DNTOT
      F2T = F2T / DNTOT
      F3T = F3T / DNTOT
      DGTDT  = DGTDT * DNTOT
      DF1TDT = DF1TDT / DNTOT
      DF2TDT = DF2TDT / DNTOT
      DF3TDT = DF3TDT / DNTOT
***   Save value of DNTOT ...
      DNTOTS = DNTOT
***   and reset DNTOT.
      DNTOT  = D10

***   Set precision of computation ...
      DO 50 I = 1, NEQ
        PRECIS( I) = PREREL * DNTOT
50    CONTINUE

      IF ( KENTER .EQ. 0) THEN
***   Compute start values for exact solution of EOS.
***   Estimate n(H2)/n(HN) ratio:
      DUM = X * DNTOT * GT
      IF ( ( DUM .GE. D10) .OR. 
     >     ( F1T .LT. PRECIS( INHP))) THEN
***     H2-molecules dominant or H-ionization negligible
        DUM = ( D20 - X) * GT
        IF ( DUM .EQ. D00) THEN
***       No H2 - molecules, assume neutral plasma
          DN( INHN, IS) = X * DNTOT
        ELSE
          DN( INHN, IS) = D05 * 
     >              ( SQRT( D10 + D40 * DNTOT * X * DUM) - D10) / DUM
        ENDIF
        DN( INH2, IS) = ( DN( INHN, IS) * GT) * DN( INHN, IS)
        DN( INEL, IS) = D00
        DN( INHP, IS) = D00
      ELSE
***     Take into account: H-ionization, approximate H2-molecule formation 
        DUM             = ( D10 + X) * F1T
        DUMA            = D10 + ( DNTOT * GT) * X * ( D20 - X)
        DUMB            = D40 * DNTOT * X / ( D10 + X) * DUMA
        IF ( DUM .GT. ( 100.0 * DUMB)) THEN
***       Hydrogen is almost completely ionized ...
***       and H2 hopefully completely dissoziated, so DUMA = 1
          DN( INEL, IS)   = D05 * D05 * DUMB
        ELSE
          DN( INEL, IS)   = D05 * ( SQRT( DUM)*SQRT( DUM + DUMB) - DUM)
     >                    / DUMA
        ENDIF
        DN( INHP, IS)   = DN( INEL, IS)
        DN( INHN, IS)   = DN( INEL, IS) / F1T * DN( INHP, IS)
        DN( INH2, IS)   = DN( INHN, IS) * X * DNTOT * GT
      ENDIF
***   Remaining species ...
      DN( INA, IS)    = DNTOT - DN( INEL, IS) + DN( INH2, IS)
      DN( INM, IS)    = Z * DN( INA, IS)
      DN( INHEN, IS)  = Y * DN( INA, IS)
      DN( INHEP, IS)  = D00
      DN( INHEPP, IS) = D00

***   Compute Jacobi matrix of system of equations ...
***   B(i, j) = d y(j) / d x(i)
      CALL EOSJCB( DN( 1, IS), BINV)

***   Invert BINV
      CALL OWNINV( NEQ, MEQ, BINV)
***   Following ENDIF related to ( KENTER .EQ. 0)
      ENDIF

***   Compute improved solution of EOS constrain equations
      CALL BROYDN( MEQ, NEQ, MIMPS, PRECIS, EOSCND, 
     >             BINV, DN, YWORK, IS, IMPS, ISTAT)
      IF ( ISTAT .NE. 0) IERROR = 1
      DN( INM, IS) = Z * DN( INA, IS) 
***   Until now 9 equations are regarded
      NEQ = 9

***   Temperature and pressure derivatives of number densities (in DNTOTS)
***   ===================================================================
***   Work index
      IW  = 1
***   Temperature derivatives, index
      IDT = 2
***   Pressure derivatives, index
      IDP = 3
***   Compute Jacobi matrix of actual particle densities
      CALL EOSJCB( DN( 1, IS), BINV)
***   Invert it
      CALL OWNINV( NEQ, MEQ, BINV)

***   Compute temperature derivatives
      CALL EOSDDT( DN( 1, IS), YWORK( 1, IW))
      DO 110 I = 1, NEQ
        DUM = D00
        DO 100 J = 1, NEQ
          DUM = DUM - YWORK( J, IW) * BINV( J, I)
100     CONTINUE
        YWORK( I, IDT) = DUM
110   CONTINUE

***   Compute pressure derivatives
      CALL EOSDDP( DN( 1, IS), YWORK( 1, IW))
      DO 130 I = 1, NEQ
        DUM = D00
        DO 120 J = 1, NEQ
          DUM = DUM - YWORK( J, IW) * BINV( J, I)
120     CONTINUE
        YWORK( I, IDP) = DUM
130   CONTINUE

***   Density
***   =======
      DRHO = QMOL * DN( INA, IS) * ( WTUNIT * DNTOTS)

      IF ( IFENT .EQ. 1) THEN
***   Entropy
***   =======
***   Only those species are considered which are more abundand than the
***   numerical accuracy to which they are computed.
      DUM = D20PI * WTUNIT / PLNCKH * BOLTZK / PLNCKH
      DUM = DUM * SQRT( DUM)
      DS  = D00
***   H2-molecule
      IF ( DN( INH2, IS) .GT. PRECIS( INH2)) THEN
        DPART = QH2T
        DNCCM = DN( INH2, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D20 * SQRT(D20)
        ELSE
          WT32  = WTH2 * SQRT( WTH2)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
***     Add entropy fraction due to internal rotational and vibrational niveaus
        DS = DS + DNCCM * DLQDLT
C GAS entropy bug was:  DS = DS + DNCCM * ( LOG( DPART) + DLQDLT)
      ENDIF
***   H I
      IF ( DN( INHN, IS) .GT. PRECIS( INHN)) THEN
        DPART = D20
        DNCCM = DN( INHN, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D10
        ELSE
          WT32  = WTH * SQRT( WTH)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   H II
      IF ( DN( INHP, IS) .GT. PRECIS( INHP)) THEN
        DPART = D10
        DNCCM = DN( INHP, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D10
        ELSE
          WT32  = WTHP * SQRT( WTHP)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   He I
      IF ( DN( INHEN, IS) .GT. PRECIS( INHEN)) THEN
        DPART = D10
        DNCCM = DN( INHEN, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D80
        ELSE
          WT32  = WTHE * SQRT( WTHE)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   He II
      IF ( DN( INHEP, IS) .GT. PRECIS( INHEP)) THEN
        DPART = D20
        DNCCM = DN( INHEP, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D80
        ELSE
          WT32  = WTHEP * SQRT( WTHEP)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   He III
      IF ( DN( INHEPP, IS) .GT. PRECIS( INHEPP)) THEN
        DPART = D10
        DNCCM = DN( INHEPP, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D80
        ELSE
          WT32  = WTHEPP * SQRT( WTHEPP)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   Electrons
      IF ( DN( INEL, IS) .GT. PRECIS( INEL)) THEN
        DPART = D20
        DNCCM = DN( INEL, IS) * DNTOTS
        WT32  = WTEL * SQRT( WTEL)
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   Representative metal
      IF ( DN( INM, IS) .GT. PRECIS( INM)) THEN
***     Magnesium like, statistical weight 1
        DPART = D10
        DNCCM = DN( INM, IS) * DNTOTS
        WT32  = WTM * SQRT( WTM)
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   Add fractions due to total number ...
      DS = DS + DNTOTS * ( D15 * LOG( DT) + D25)
***   Convert to erg/g/K
      DS = DS * BOLTZK / DRHO 
***   Following ENDIF related to ( IFENT .EQ. 1)
      ENDIF

***   Internal energy and temperature derivative of internal energy per volume
***   ========================================================================
      DE    = D00
***   d e(per volume) / d T at constant pressure
      DEVDTP = D00
***   Ionization energy of H I
      IF ( DN( INHP, IS) .GT. PRECIS( INHP)) THEN
        DE = DE + DN( INHP, IS) * EIH
        DEVDTP = DEVDTP + YWORK( INHP, IDT) * EIH
      ENDIF
***   Ionization energy of He I
      IF ( DN( INHEP, IS) .GT. PRECIS( INHEP)) THEN
        DE = DE + DN( INHEP, IS) * EIHE
        DEVDTP = DEVDTP + YWORK( INHEP, IDT) * EIHE
      ENDIF
***   Ionization energy of He II
      IF ( DN( INHEPP, IS) .GT. PRECIS( INHEPP)) THEN
        DE = DE + DN( INHEPP, IS) * ( EIHE + EIHEP)
        DEVDTP = DEVDTP + YWORK( INHEPP, IDT) * ( EIHE + EIHEP)
      ENDIF
***   Energy of H2-molecule
      IF ( DN( INH2, IS) .GT. PRECIS( INH2)) THEN
        DEH2 = BOLTZK * DT * DLQDLT - EDH2
        DE = DE + DN( INH2, IS) * DEH2
        DCVH2 = BOLTZK * ( DLQDLT * ( D20 - DLQDLT) 
     >        + DT * DT / QH2T * D2QDT2)
        DEVDTP = DEVDTP + YWORK( INH2, IDT) * DEH2 
     >         + DN( INH2, IS) * DCVH2
      ENDIF
***   Kinetic energy of particles and/or conversion to cgs
      DE = DE * DNTOTS + D15 * DP
      DEVDTP = DNTOTS * DEVDTP
***   Conversion to energy per mass unit
      DE = DE / DRHO

***   Enthalpy
***   ========
      DH = DE + DP / DRHO

***   d rho / d T at constant pressure
***   ================================
      DRODTP = QMOL * WTUNIT * DNTOTS 
     >       * ( - D10 / DT - YWORK( INEL, IDT) + YWORK( INH2, IDT))

***   d rho / d p at constant temperature
***   ===================================
      DRODPT = QMOL * WTUNIT * DNTOTS 
     >       * ( D10 / DP - YWORK( INEL, IDP) + YWORK( INH2, IDP))

***   C_p / C_p(s)
***   ============
      DCXT = - DRODTP / DRHO * DT

***   d s / d p at constant temperature
***   =================================
      DSDPT = DRODTP / DRHO / DRHO

***   d e (per mass) / d T at constant pressure
***   =========================================
      DEMDTP = ( DEVDTP - DE * DRODTP) / DRHO
 
***   d h (per mass) / d T at constant pressure = C_p
***   ===============================================
      DCP = DEMDTP - DRODTP / DRHO * DP / DRHO

***   C_p(s)
***   ======
      DXCP = DCP / DCXT

***   Velocity of sound and ratio C_p(s) / C_v(s)
***   ===========================================
      DCS   = D10 / ( DRODPT - DRODTP * DSDPT / DCP * DT)
      DXGAM = DCS / DP * DRHO
      DCS   = SQRT( DCS)

***   Return single precision values ...
      ANHN   = REAL( DNTOTS * DN( INHN, IS))
      ANHP   = REAL( DNTOTS * DN( INHP, IS))
      ANHEN  = REAL( DNTOTS * DN( INHEN, IS))
      ANHEP  = REAL( DNTOTS * DN( INHEP, IS))
      ANHEPP = REAL( DNTOTS * DN( INHEPP, IS))
      ANH2   = REAL( DNTOTS * DN( INH2, IS))
      CT     = REAL( D10 / SQRT( DRODPT))
      RHO    = REAL( DRHO)
      XCP    = REAL( DXCP)
      XGAM   = REAL( DXGAM)
      CXT    = REAL( DCXT)
      ENTALP = REAL( DH)
      IF ( IFENT .EQ. 1) S = REAL( DS)
      CS     = REAL( DCS)

      RETURN

      ENTRY STGGMD( DUMLOG)
***   Set GASGAS compatibility (to GAS) mode
***   DUMLOG = .FALSE.: Normal GASGAS operation (default)
***          = .TRUE. : Entropy computation analogous to GAS, i.e. with integer
***                     atomic weights
      GASCMP = DUMLOG
      RETURN

      END SUBROUTINE gasgas


      SUBROUTINE GAS_H ( P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                  IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
********************************************************************************
*
* Version: 11-JUN-1993 11:15:02
*
* Author : HGL
*
* Description: This routine computes thermodynamic quantities needed in the
*   HD context. This routine is derived from a more complete version!
*   The following ionization equilibrium is considered:
*   H    <->  H+   + e-
*   He ionization is not taken into account in this version!
*   H_2 molecule formation is not taken into account. Nonideal effects are
*   neglected. In the routine it is assumed that the plasma is primarily
*   composed of hydrogen.
* 
*
* Arguments, access, type, dimension, description:
*   P      R R : Gas pressure
*   T      R R : Temperature
*   RHO    W R : Mass density
*   XCP    W R : HD specific heat
*   XGAM   W R : HD ratio of specific heats
*   CXT    W R : CXT = CP / XCP
*   ENTALP W R : Enthalpy per mass unit
*   S      W R : Entropy per mass unit
*   CS     W R : Sound speed
*   IERROR W I : Error indicator, 0: no error, 1: illegal IFION specified
*   IFENT  R I : Entropy requested? 1: yes, else: no
*   KENTER * I : Input : < 0 : Initialization of the routine
*                        else: Compute thermodynamic quantities
*                Output: =0
*   X      R R : Hydrogen abundance by number fraction
*   Y      R R : Helium abundance by number fraction, must be zero!
*   QMOL   R R : Mean molecular weight of neutral gas
*
* Common blocks (access): EE (VAX only)
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: None
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
***   EPSIT is the relative accuracy (related to total number density)
***   to which the electron density is computed
      PARAMETER( EPSIT = 1.0E-8)
***   EPSNTR is the limit at which the plasma is regarded as neutral
      PARAMETER( EPSNTR = 0.25 * EPSIT * EPSIT)
***
      PARAMETER( BOLTZK = 1.38054E-16, PLNCKH = 6.6256E-27)
      PARAMETER( WTUNIT = 1.6604E-24,  UGC = 8.3144E+07)
***   Atomic weights
      PARAMETER( WTHN = 1.0080, WTHEN = 4.0026, WTEL = 5.48609E-4)
      PARAMETER( WTHP = WTHN - WTEL, WTHEP = WTHEN - WTEL, 
     >           WTHEPP = WTHEP - WTEL)
***   Ionization energies
      PARAMETER( EIH = 2.1785E-11, EIHE = 3.9391E-11, 
     >           EIHEP = 8.7179E-11)
      PARAMETER( EIHEPP = EIHE + EIHEP)
***
      PARAMETER( R20PI = 6.283185307179586E+00)
***   AUX0 = 2 * pi * WTUNIT * k / h**2
      PARAMETER( AUX0 = R20PI * WTUNIT / PLNCKH * BOLTZK / PLNCKH)
***   AUX1 = 2 * pi * WTUNIT / h**2 * WTEL
      PARAMETER( AUX1 = R20PI * WTUNIT / PLNCKH / PLNCKH * WTEL)
***   Partitions functions of various species
***   Metal magnesium like ...
      PARAMETER( QHN = 2.0, QHP = 1.0, QHEN = 1.0, QHEP = 2.0,
     >           QHEPP = 1.0, QEL = 2.0, QM = 1.0)


***   VAX only: (and Sun)
      COMMON / EE / ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT

      SAVE QMA, Z, WTM, SAHN, SAHP, SAHEN, SAHEP, SAHEPP, SAEL, SAM
      SAVE P01, P02, P03, P04, P05, P06, P07, P08, P09, P10
      SAVE EPSFUL

      REAL LN10, KT
      DATA LN10 / 2.3025850929940457 /
***   Statement functions ...
C      EXP10( XX) = EXP( LN10 * XX)

      IF ( KENTER .LT. 0 ) THEN
        QMA = WTUNIT * QMOL
***     Auxiliary parameters for entropy computation
***     These parameters are known before run time and could be computed
***     previously.
        SAHN   = 1.5 * LOG( AUX0 * WTHN)   + LOG( QHN)
        SAHP   = 1.5 * LOG( AUX0 * WTHP)   + LOG( QHP) 
        SAHEN  = 1.5 * LOG( AUX0 * WTHEN)  + LOG( QHEN) 
        SAHEP  = 1.5 * LOG( AUX0 * WTHEP)  + LOG( QHEP) 
        SAHEPP = 1.5 * LOG( AUX0 * WTHEPP) + LOG( QHEPP) 
        SAEL   = 1.5 * LOG( AUX0 * WTEL)   + LOG( QEL) 
***     The next statement is introduced because ANEL = ANHP in this context.
***     In fact it is bad programming style ...
        SAEL   = SAEL + SAHP
***
        Z   = 1.0 - X - Y
        IF ( Z .GT. 0.0) THEN
          WTM = ( QMOL - WTHN * X - WTHEN * Y) / Z
          SAM = 1.5 * LOG( AUX0 * WTM) + LOG( QM)
        ELSE
          Z   = 0.0
          WTM = 0.0
          SAM = 0.0
        ENDIF
        P01 = 1.0 + X
        P02 = X + 2.0 * Y
        P03 = 1.0 + X + 2.0 * Y
        P04 = X + Y
        P05 = 1.0 + X + Y
        P06 = 1.0 + Y
        P07 = 1.0 + 2.0 * Y
        P08 = - ( X * EIH + Y * EIHEPP)
        P09 = 2.0 * Y
        P10 = X / ( 1.0 + X)
        IF ( Y .NE. 0.0) THEN
          WRITE( *, *) 'GAS> Gas version in effect does not support ',
     >                 'He ionization!'
          WRITE( *, *) 'GAS> Set Y = 0 or use another version!'
          STOP
        ENDIF
        WRITE( 6, '( /, 1X, 72(''=''))')
        WRITE( 6, '( 1X, ''GAS version 6-Nov-1992 (WD) busy!'')')
        WRITE( 6, '( 1X, 72(''=''))')
        WRITE( 6, '( /)')
        EPSFUL = SQRT( EPSIT)
        IERROR = 0
        RETURN
      ENDIF

      KENTER = 0

      IF ( IFION .NE. 1) THEN
        IERROR = 1
      ELSE
        IERROR = 0
      ENDIF

***   Temperature measured in kT
      KT = BOLTZK * T
      ANTOT = P / KT


      AUX2 = ( AUX1 * KT)**1.5 / ANTOT

***   F1(T)
      AUX3   = EIH / KT
      F1T    = AUX2 * EXP( - AUX3)
      DTF1T  = ( 1.5 + AUX3) * F1T / T

***   Check whether the plasma could be regarded as neutral
      IF ( F1T .LT. EPSNTR) THEN
***     In this if block all particle densities are measured in external units!
        ANA    = ANTOT
        ANHN   = X * ANTOT
        ANEL   = 0.0
        ANHP   = 0.0
***     VAX only:
        ANHEN  = 0.0
        ANHEP  = 0.0
        ANHEPP = 0.0
***
        RHO    = ANTOT * QMA
        OHR    = 1.0 / RHO
        DTRHO  = - RHO / T 
        DPRHO  =   RHO / P 
        E      = 1.5 * P * OHR
        DTEV   = 0.0
        GOTO 30
      ENDIF

***   The prefix DT denotes the temperature derivative at constant pressure
***   The prefix DP denotes the pressure derivative at constant temperature
      DTNTOT = - 1.0 / T
      DPNTOT =   1.0 / P

***   Compute of electron pressure.
      DUM  = P01 * F1T
      DUM2 = 4.0/DUM * (F1T/DUM) * X
      IF ( DUM2 .LT. EPSFUL) THEN
***     Plasma nearly completely ionized
        DUM3 = 1.0 + 0.5 * DUM2
        ANEL = P10 * ( 1.0 - 0.25 * DUM2)
      ELSE
        DUM3 = SQRT( 1.0 + DUM2)
        ANEL = 0.5 * DUM * ( DUM3 - 1.0)
      ENDIF
      ONEL  = 1.0 / ANEL
      DUM3  = P10 / DUM3
      DTLF1T = DTF1T / F1T
      DTNEL = ANEL * DTLF1T + DUM3 * ( DTNTOT - DTLF1T)
      DPNEL = DPNTOT * DUM3

***   Compute some auxiliary parameters
C      DTNENE = DTNEL * ONEL
      F1NE   = F1T * ONEL
C      DTF1NE = DTF1T * ONEL
***   Prefix TD denotes the temperature derivative of the fraction F?/ANEL
C      TDF1NE = DTF1NE - DTNENE * F1NE
C      HTERM  = 1.0 + F1NE

***   Until now external units are used ...
***   Particle densities
      ANEL   = ANEL * ANTOT
      ANA    = ANTOT - ANEL
      ANHP   = ANEL
      ANHN   = ANEL / F1NE
***   VAX only:
C      ANHEN  = 0.0
C      ANHEP  = 0.0
C      ANHEPP = 0.0
***   Temperature derivatives of particle densities at constant pressure
      DTNTOT = DTNTOT * ANTOT
      DPNTOT = DPNTOT * ANTOT
      DTNEL  = DTNEL * ANTOT
      DPNEL  = DPNEL * ANTOT
      DTNA   = DTNTOT - DTNEL
      DTNHP  = DTNEL
      DTNHN  = X * DTNTOT - P01 * DTNEL

***   Mass density
      RHO = QMA * ANA
      OHR = 1.0 / RHO
***   Temperature derivative
      DTRHO = DTNA * QMA
***   Pressure derivative
      DPRHO = ( DPNTOT - DPNEL) * QMA
***   Internal energy
      E =( ANHP * EIH + 1.5 * P) * OHR
***   Temperature derivative of internal energy per volume at const. pressure
      DTEV = DTNHP * EIH

***   =========================================================================
***   The remaining program section is used in any case (neutral or partial
***   ionization).
***   Temperature derivative of internal energy per mass at const. pressure
30    DTEM = ( DTEV - E * DTRHO) * OHR

      POHR   = P * OHR
      DTLRHO = DTRHO * OHR

***   Enthalpy
      ENTALP = E + POHR

***   CXT
      CXT = - T * DTLRHO

***   (dS/dp)_T
      DPS = DTLRHO * OHR

***   C_p
      CP =  DTEM - DTLRHO * POHR

***   C_p(s)
      XCP = CP / CXT

      CS   = 1.0 / ( DPRHO - DTRHO * DPS / CP * T)
      XGAM = CS / P * RHO
      CS = SQRT( CS)

***   VAX only:
C      CT = 1.0 / SQRT( DPRHO)

      IF ( IFENT .EQ. 1) THEN
***     Precision converted to external units
        PRECIS = EPSIT * ANTOT
        ANM    = Z * ANA
        S = ANTOT * ( 2.5 + 1.5 * LOG( T))
        IF ( ANHN  .GT. PRECIS) S = S + ANHN  * ( SAHN  - LOG( ANHN))
C        IF ( ANHP  .GT. PRECIS) S = S + ANHP  * ( SAHP  - LOG( ANHP))
        IF ( ANEL  .GT. PRECIS) 
     >  S = S + ANEL  * ( SAEL  - 2.0 * LOG( ANEL))
        IF ( ANM   .GT. PRECIS) S = S + ANM   * ( SAM   - LOG( ANM))
        S = S * BOLTZK * OHR
      ENDIF
      RETURN
      END SUBROUTINE GAS_H


      SUBROUTINE GAS_SUNDEEP( P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                       IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
********************************************************************************
*
* Version: 29-OCT-1993 15:39:42
*
* Author : HGL
*
* Description: This routine computes thermodynamic quantities needed in the
*   HD context. The following ionization equilibria are considered:
*   H    <->  H+   + e-
*   He   <->  He+  + e-
*   He+  <->  He++ + e-
*   H_2 molecule formation is not taken into account. Nonideal effects are
*   neglected. In the routine it is assumed that the plasma is primarily
*   composed of hydrogen.
* 
*
* Arguments, access, type, dimension, description:
*   P      R R : Gas pressure
*   T      R R : Temperature
*   RHO    W R : Mass density
*   XCP    W R : HD specific heat
*   XGAM   W R : HD ratio of specific heats
*   CXT    W R : CXT = CP / XCP
*   ENTALP W R : Enthalpy per mass unit
*   S      W R : Entropy per mass unit
*   CS     W R : Sound speed
*   IERROR W I : Error indicator, 0: no error, 1: illegal IFION specified
*   IFENT  R I : Entropy requested? 1: yes, else: no
*   KENTER * I : Input : < 0 : Initialization of the routine
*                        else: Compute thermodynamic quantities
*                Output: Number of Newton-iterations
*   X      R R : Hydrogen abundance by number fraction
*   Y      R R : Helium abundance by number fraction
*   QMOL   R R : Mean molecular weight of neutral gas
*
* Common blocks (access): E (ifndef CONTEXT_HD)
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Preprocessor macros   : CONTEXT_HD
*
* Comments: None
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
***   EPSIT is the relative accuracy (related to total number density)
***   to which the electron density is computed
      PARAMETER( EPSIT = 1.0E-5)
***   EPSFUL determines the limit at which the plasma is regarded as 
***   fully ionozed.
      PARAMETER( EPSFUL = 1.0 / ( 0.5 * EPSIT))
***   EPSNTR is the limit at which the plasma is regarded as neutral
      PARAMETER( EPSNTR = 0.25 * EPSIT * EPSIT)
***
      PARAMETER( BOLTZK = 1.38054E-16, PLNCKH = 6.6256E-27)
      PARAMETER( WTUNIT = 1.6604E-24,  UGC = 8.3144E+07)
***   Atomic weights
      PARAMETER( WTHN = 1.0080, WTHEN = 4.0026, WTEL = 5.48609E-4)
      PARAMETER( WTHP = WTHN - WTEL, WTHEP = WTHEN - WTEL, 
     >           WTHEPP = WTHEP - WTEL)
***   Ionization energies
      PARAMETER( EIH = 2.1785E-11, EIHE = 3.9391E-11, 
     >           EIHEP = 8.7179E-11)
      PARAMETER( EIHEPP = EIHE + EIHEP)
***
      PARAMETER( R20PI = 6.283185307179586E+00)
***   AUX0 = 2 * pi * WTUNIT * k / h**2
      PARAMETER( AUX0 = R20PI * WTUNIT / PLNCKH * BOLTZK / PLNCKH)
***   AUX1 = 2 * pi * WTUNIT / h**2 * WTEL
      PARAMETER( AUX1 = R20PI * WTUNIT / PLNCKH / PLNCKH * WTEL)
***   Partitions functions of various species
***   Metal magnesium like ...
      PARAMETER( QHN = 2.0, QHP = 1.0, QHEN = 1.0, QHEP = 2.0,
     >           QHEPP = 1.0, QEL = 2.0, QM = 1.0)


C.#ifndef CONTEXT_HD
      COMMON / E / ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT
C.#endif

      SAVE QMA, Z, WTM, SAHN, SAHP, SAHEN, SAHEP, SAHEPP, SAEL, SAM
      SAVE P01, P02, P03, P04, P05, P06, P07, P08, P09, P10

      REAL LN10, KT
      DATA LN10 / 2.3025850929940457 /
***   Statement functions ...
C     EXP10( XX) = EXP( LN10 * XX)

      IF ( KENTER .LT. 0 ) THEN
        QMA = WTUNIT * QMOL
***     Auxiliary parameters for entropy computation
***     These parameters are known before run time and could be computed
***     previously.
        SAHN   = 1.5 * LOG( AUX0 * WTHN)   + LOG( QHN)
        SAHP   = 1.5 * LOG( AUX0 * WTHP)   + LOG( QHP) 
        SAHEN  = 1.5 * LOG( AUX0 * WTHEN)  + LOG( QHEN) 
        SAHEP  = 1.5 * LOG( AUX0 * WTHEP)  + LOG( QHEP) 
        SAHEPP = 1.5 * LOG( AUX0 * WTHEPP) + LOG( QHEPP) 
        SAEL   = 1.5 * LOG( AUX0 * WTEL)   + LOG( QEL) 
***
        Z   = 1.0 - X - Y
        IF ( Z .GT. 0.0) THEN
          WTM = ( QMOL - WTHN * X - WTHEN * Y) / Z
          SAM = 1.5 * LOG( AUX0 * WTM) + LOG( QM)
        ELSE
          Z   = 0.0
          WTM = 0.0
          SAM = 0.0
        ENDIF
        P01 = 1.0 + X
        P02 = X + 2.0 * Y
        P03 = 1.0 + X + 2.0 * Y
        P04 = X + Y
        P05 = 1.0 + X + Y
        P06 = 1.0 + Y
        P07 = 1.0 + 2.0 * Y
        P08 = - ( X * EIH + Y * EIHEPP)
        P09 = 2.0 * Y
        P10 = X / ( 1.0 + X)
***
        WRITE( 6, '( /, 1X, 72(''=''))')
        WRITE( 6, '( 1X, ''GAS version 20-Oct-1992 (UR) busy!'')')
        WRITE( 6, '( 1X, 72(''=''))')
        WRITE( 6, '( /)')
        IERROR = 0
        RETURN
      ENDIF

      KENTER = 0

***   In this routine all ionization equilibria are solved.
***   Don't try to switch off one of these by IFION!
      IF ( IFION .NE. 1) THEN
        IERROR = 1
      ELSE
        IERROR = 0
      ENDIF

***   Temperature measured in kT
      KT = BOLTZK * T
      ANTOT = P / KT

***   ANTOT is the unit of particle number density which is used subsequently.
***   ANTOT is termed as the 'internal' unit while normal cgs units are 
***   termed 'external'.

***   The F?T factors should be measured in internal units.
      AUX2 = ( AUX1 * KT)**1.5 / ANTOT

***   F1(T)
      AUX3   = EIH / KT
      F1T    = AUX2 * EXP( - AUX3)
      DTF1T  = ( 1.5 + AUX3) * F1T / T

***   Check whether the plasma could be regarded as neutral
      IF ( F1T .LT. EPSNTR) THEN
***     In this if block all particle densities are measured in external units!
        ANA    = ANTOT
        ANHN   = X * ANTOT
        ANHEN  = Y * ANTOT
        ANEL   = 0.0
        ANHP   = 0.0
        ANHEP  = 0.0
        ANHEPP = 0.0
        RHO    = ANTOT * QMA
        OHR    = 1.0 / RHO
        DTRHO  = - RHO / T 
        DPRHO  =   RHO / P 
        E      = 1.5 * P * OHR
        DTEV   = 0.0
        GOTO 30
      ENDIF

***   Compute first guess of electron pressure.
***   The electron density is computed by regarding the ionization
***   equilibrium of hydrogen only.
      DUM  = P01 * F1T
      DUM2 = 4.0/DUM * (F1T/DUM) * X
      IF ( DUM2 .LT. EPSIT) THEN
***     Plasma nearly completely ionized
        ANEL = P10
      ELSE
        ANEL = 0.5 * DUM * (SQRT( 1.0 + DUM2) - 1.0)
      ENDIF

***   F3(T)
      AUX3   = EIHEP / KT
      F3T    = AUX2 * EXP( - AUX3)
      DTF3T  = ( 1.5 + AUX3) * F3T / T

      IF ( F3T / ANEL .GT. EPSFUL) THEN
***     In this if block all particle densities are measured in external units!
        ANA    = ANTOT / P03
        ANHP   = X * ANA
        ANHEPP = Y * ANA
        ANHN   = 0.0
        ANHEN  = 0.0
        ANHEP  = 0.0
        ANEL   = ANHP + ANHEPP + ANHEPP
        RHO    = ANA * QMA
        OHR    = 1.0 / RHO
        DTRHO  = - RHO / T 
        DPRHO  =   RHO / P 
        E      = ( ANHP * EIH + ANHEPP * EIHEPP + 1.5 * P) * OHR
        DTEV   = ANA * P08 / T
        GOTO 30
      ENDIF

***   F2(T)
      AUX3   = EIHE / KT
      F2T    = 4.0 * AUX2 * EXP( - AUX3)
      DTF2T  = ( 1.5 + AUX3) * F2T / T

***   The prefix DT denotes the temperature derivative at constant pressure
***   The prefix DP denotes the pressure derivative at constant temperature
      DTNTOT = - 1.0 / T
      DPNTOT =   1.0 / P

***   Some auxiliary parameters ...
      F1F3   = F1T * F3T
      F1F2F3 = F1F3 * F2T
      AUX4   = F1T * X + F2T * Y
      AUX5   = F1T * P04 + 2.0 * F3T * Y
      AUX6   = F1F3 * P03 - AUX5
      AUX7   = F1T * P05 + F3T * P07

***   Compute coefficients of the polynomial
***   A4 * ANEL**4 + A3 * ANEL**3 + A2 * ANEL**2 + A1 * ANEL + A0 = 0
***   which has to be solved in order to determine the electron density. 
      A0 = - F1F2F3 * P02
      A1 = F2T * AUX6
      A2 = F2T * AUX7 - AUX4
      A3 = F1T * P01 + F2T * P06
***   A4 is not used explicitly, so 
C      A4 = 1.0

      AUX8 = A2 + A2
      AUX9 = 3.0 * A3

***   Solve polynomial by Newton iteration
20    DANEL = (A0 + ANEL * ( A1 + ANEL * ( A2 + ANEL * ( A3 + ANEL ))))/
     >( A1 + ANEL * ( AUX8 + ANEL * (AUX9 + 4.0*ANEL)))
      ANEL = ANEL - DANEL
      KENTER = KENTER + 1
      IF ( ABS( DANEL) .GE. EPSIT) GOTO 20
      ONEL = 1.0 / ANEL

***   Compute derivatives of electron density ( d/dT, d/dp)
      DTA3 = P01 * DTF1T + P06 * DTF2T
      DTA2 = DTF2T * AUX7 + F2T * ( DTF1T * P05 + DTF3T * P07)
     >     - DTNTOT * AUX4 - ( DTF1T * X + DTF2T * Y)
      DTA1 = DTF2T * AUX6 + F2T * ( ( DTF1T * F3T + F1T * DTF3T) * P03
     >     - DTNTOT * AUX5 - ( DTF1T * P04 + P09 * DTF3T))
      DTA0 = - P02 * (
     >  ( F2T * ( DTF1T * F3T + F1T * DTF3T) + F1F3 * DTF2T)
     >   + F1F2F3 * DTNTOT )
C      DPA3 = 0.0
      DPA2 = - DPNTOT * AUX4
      DPA1 = - F2T * DPNTOT * AUX5
      DPA0 = - DPNTOT * P02 * F1F2F3

      DUM = 1.0 / 
     >    ( 4.0 + ONEL * ( AUX9 + ONEL * ( AUX8 + ONEL * A1)))
      DTNEL = - ( DTA3 + ONEL * ( DTA2 + ONEL * ( DTA1 + ONEL * DTA0)))
     >      * DUM
      DPNEL = - ONEL * ( DPA2 + ONEL * ( DPA1 + ONEL * DPA0))
     >      * DUM

***   Compute some auxiliary parameters
      DTNENE = DTNEL * ONEL
      F1NE   = F1T * ONEL
      F2NE   = F2T * ONEL
      F3NE   = F3T * ONEL
      DTF1NE = DTF1T * ONEL
      DTF2NE = DTF2T * ONEL
      DTF3NE = DTF3T * ONEL
***   Prefix TD denotes the temperature derivative of the fraction F?/ANEL
      TDF1NE = DTF1NE - DTNENE * F1NE
      TDF2NE = DTF2NE - DTNENE * F2NE
      TDF3NE = DTF3NE - DTNENE * F3NE
      HTERM  = 1.0 + F1NE
      HETERM = 1.0 + F2NE * ( 1.0 + F3NE)

***   Until now external units are used ...
***   Particle densities
      ANEL   = ANEL * ANTOT
      ANA    = ANTOT - ANEL
      ANHN   = X * ANA / HTERM
      ANHP   = ANHN * F1NE
      ANHEN  = Y * ANA / HETERM
      ANHEP  = ANHEN * F2NE
      ANHEPP = ANHEP * F3NE
***   Temperature derivatives of particle densities at constant pressure
      DTNTOT = DTNTOT * ANTOT
      DPNTOT = DPNTOT * ANTOT
      DTNEL  = DTNEL * ANTOT
      DPNEL  = DPNEL * ANTOT
      DTNA   = DTNTOT - DTNEL
      DTNHN  = ( X * DTNA - ANHN * TDF1NE) / HTERM
      DTNHP  = DTNHN * F1NE + ANHN * TDF1NE
      DTNHEN = ( Y * DTNA - ANHEN * 
     >           ( TDF2NE * ( 1.0 + F3NE) + F2NE * TDF3NE)) / HETERM
      DTNHEP  = DTNHEN * F2NE + ANHEN * TDF2NE
      DTNHPP  = DTNHEP * F3NE + ANHEP * TDF3NE

***   Mass density
      RHO = QMA * ANA
      OHR = 1.0 / RHO
***   Temperature derivative
      DTRHO = DTNA * QMA
***   Pressure derivative
      DPRHO = ( DPNTOT - DPNEL) * QMA
***   Internal energy
      E =( ANHP * EIH + ANHEP * EIHE + ANHEPP * EIHEPP + 1.5 * P) * OHR
***   Temperature derivative of internal energy per volume at const. pressure
      DTEV = DTNHP * EIH + DTNHEP * EIHE + DTNHPP * EIHEPP

***   =========================================================================
***   The remaining program section is used in any case (neutral or partial
***   ionization).
***   Temperature derivative of internal energy per mass at const. pressure
30    DTEM = ( DTEV - E * DTRHO) * OHR

      POHR   = P * OHR
      DTLRHO = DTRHO * OHR

***   Enthalpy
      ENTALP = E + POHR

***   CXT
      CXT = - T * DTLRHO

***   (dS/dp)_T
      DPS = DTLRHO * OHR

***   C_p
      CP =  DTEM - DTLRHO * POHR

***   C_p(s)
      XCP = CP / CXT

      CS   = 1.0 / ( DPRHO - DTRHO * DPS / CP * T)
      XGAM = CS / P * RHO
      CS = SQRT( CS)

C.#ifndef CONTEXT_HD
      CT = 1.0 / SQRT( DPRHO)
C.#endif

      IF ( IFENT .EQ. 1) THEN
***     Precision converted to external units
        PRECIS = EPSIT * ANTOT
        ANM    = Z * ANA
        S = ANTOT * ( 2.5 + 1.5 * LOG( T))
        IF ( ANHN  .GT. PRECIS) S = S + ANHN  * ( SAHN  - LOG( ANHN))
        IF ( ANHP  .GT. PRECIS) S = S + ANHP  * ( SAHP  - LOG( ANHP))
        IF ( ANHEN .GT. PRECIS) S = S + ANHEN * ( SAHEN - LOG( ANHEN))
        IF ( ANHEP .GT. PRECIS) S = S + ANHEP * ( SAHEP - LOG( ANHEP))
        IF ( ANHEPP .GT. PRECIS) 
     >    S = S + ANHEPP * ( SAHEPP - LOG( ANHEPP))
        IF ( ANEL  .GT. PRECIS) S = S + ANEL  * ( SAEL  - LOG( ANEL))
        IF ( ANM   .GT. PRECIS) S = S + ANM   * ( SAM   - LOG( ANM))
        S = S * BOLTZK * OHR
      ENDIF

      RETURN
      END SUBROUTINE GAS_SUNDEEP


C----------------******------------------------------------------------+-------
      subroutine gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
     &                  ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C----------------------------------------------------------------------+-------
C NAME:
C   gasinv ('gas_invert')
C
C PURPOSE:
C   Inversion of GAS routine: rho/e -> P,T,...
C
C CATEGORY:
C   EOS
C
C CALLING SEQUENCE:
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C
C INPUT:
C   rho:    ('rho') density [g/cm^3]
C   e:      ('energy') internal energy per mass unit [erg/g]
C   ifion:  ('i_flag_ionization') integer, determines how to treat ionization
C           processes (0...16)
C   ifent:  ('i_flag_entropy') entropy is computed if ifent=1 (0...1)
C   abuX:   ('abundance_X') real, hydrogen abundance (number fraction)
C   abuY:   ('abundance_Y') real, helium abundance (number fraction)
C   abuZ:   ('abundance_Z') real, metal abundance (number fraction)
C   Qmol:   ('Q_molecular') real, mean molecular weight [u]
C
C INPUT/OUTPUT:
C   P:      ('Pressure') pressure [dyn/cm^2]
C   T:      ('Temperature') temperature [K]
C   kenter: ('k_enter') original GAS: must be <0 in initial call, otherwise >= 0
C
C OUTPUT:
C   cpd:    ('cp_dash') Cp' = d(S)/d(ln rho) | P [erg/g/K]
C           GAS: xcp
C   Gamma1: ('Gamma_1') 1. adiabatic exponent: Gamma1 = Cp'/Cv'
C                                                     = d(ln P)/d(ln rho) | s
C           GAS: xgam
C   delta:  ('delta') expansion coefficient: delta = alpha*T
C                                                  =-d(ln rho)/d(ln T) | P
C           GAS: cxt
C   h:      ('h') enthalpy per mass unit [erg/g]
C           GAS: entalp
C   s:      ('s') entropy per mass unit [erg/g/K]
C   cs:     ('c_sound') adiabatic sound velocity [cm/s]
C   ierror: ('index_error') error number
C
C ROUTINES:
C   gas: EOS routine of RHD-code (or gasgas)
C
C INCLUDES:
C   None
C
C COMMON BLOCK:
C   None
C
C SIDE EFFECTS:
C   None
C
C RESTRICTIONS:
C
C PROCEDURE:
C
C EXAMPLE:
C   kenter=-1
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C   kenter=0
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C
C MODIFICATION HISTORY:
C    4. 5.97 Written by Bernd Freytag, Astrophysik Kiel
C    6. 5.97 (B.F.) Correction in dp/drho|e
C   14. 5.97 (B.F.) nitermax=200
C   16. 9.97 (B.F.) call GAS with kenter0=0
C   15. 6.99 (B.F.) nitermax=30, return with error indication,
C            formula for initial guess of temperature changed
C   16. 6.99 (B.F.) Save start values, change in error formula
C   25. 2.04 (B.F.) Call GASGAS instead of GAS (was implicitely done before)
C   2014-09-06 (B.F. Uppsala) epsiter=1.0E-05 -> 1.0E-09
C----------------------------------------------------------------------+-------
      implicit none
C
C --- I/O parameters ---
      real rho, e
      real P, T, cpd, Gamma1, delta, h, s, cs
      integer ifion, ifent
      integer kenter
      integer ierror
      real abuX, abuY, Qmol
C
C --- Local variables ---
      real rhoiter, eiter, error, error0, cv
      real Piter, Titer, Pstep, Tstep
      real P0, T0, rho0, cpd0, Gamma10, delta0, h0, s0, cs0
      integer niter, kenter0
      real Rgas, epsiter, stepfac
      integer nitermax
      parameter(Rgas=8.3145E+07,
     &          epsiter=1.0E-9,
     &          nitermax=30,
     &          stepfac=10.0)
C----------------------------------------------------------------------+-------
C
      if (kenter .lt. 0) then
C       -- Initialize GAS ---
        call gasgas(8000.0, 16000.0,
     &              rhoiter, cpd, Gamma1, delta, h, s, cs,
     &              ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
      else
C       --- Initial values for interation ---
        if (kenter .gt. 0) then
C         --- Assume that in P and T good initial guesses for iteration are ---
C         --- present.                                                      ---
          Piter=P
          Titer=T
        else
C         --- Try simple initial guess for P and T from rho and e ---
          Titer=(e+1.5E+12)/(1.5*Rgas)
C         Titer=max(Titer,1500.)
          Piter=Rgas/Qmol*Titer*rho
        endif
C
        niter=0
C
C       --- Iteration ---
C
   10   niter=niter+1
C       --- Solve EOS by call of GAS ---
        kenter0=0
        call gasgas(Piter, Titer,
     &              rhoiter, cpd, Gamma1, delta, h, s, cs,
     &              ierror, ifion, ifent, kenter0, abuX, abuY, Qmol)
        if (niter .eq. 1) then
          P0     =Piter
          T0     =Titer
          rho0   =rhoiter
          cpd0   =cpd
          Gamma10=Gamma1
          delta0 =delta
          h0     =h
          s0     =s
          cs0    =cs
        endif
        cv=(cpd*delta)/(1.0+(Gamma1*delta*Piter)/(Titer*rhoiter*cpd))
        eiter=h-Piter/rhoiter
C       --- Perform step: next guess ---
        Pstep=Piter*Gamma1/rhoiter*(1.0-Piter/(rhoiter*Titer*cpd))
     &                                                 * (rhoiter-rho) +
     &        Piter*Gamma1/(Titer*cpd)                 * (eiter  -e  )
        Tstep=-Piter/(rhoiter*rhoiter)*(1.0/cv-Gamma1/cpd) 
     &                                                 * (rhoiter-rho) +
     &        1.0/cv                                   * (eiter  -e  )
C       --- Limit step size ---
        Pstep=min(max(Pstep,-0.5*Piter),0.5*Piter)
        Tstep=min(max(Tstep,-0.5*Titer),0.5*Titer)
        write(6,'(I3,1X,4(E17.10,1X))') niter, Piter, Titer, Pstep, 
     &                                  Tstep
        Piter=Piter-Pstep
        Titer=Titer-Tstep
C       --- Compute residual difference in rho and e ---
        error=abs(rhoiter-rho)/rho0 + abs(eiter  -e  )/(Rgas*T0)
        if (niter .eq. 1) then
          error0=error
        else
          if (error .gt. stepfac*error0) then
C           --- 'Step too large' ---
            ierror=1
            goto 999
          endif
        endif
C       --- Check number of iterations ---
        if (niter .gt. nitermax) then
C         --- 'Too many iterations' ---
          ierror=2
          goto 999
        endif
C       --- Check residual error ---
        if (error .gt. epsiter) goto 10
C
C       --- Iteration finished ---
        P  =Piter
        T  =Titer
        ierror=0
C
      endif
C
 999  if (ierror .gt. 0) then
C       --- In case of error give back start values ---
        P     =P0
        T     =T0
        rho   =rho0
        cpd   =cpd0
        Gamma1=Gamma10
        delta =delta0
        h     =h0
        s     =s0
        cs    =cs0
      endif
      kenter=niter
C
      end subroutine gasinv


C----------------***********-----------------------------------------+-------
      subroutine supergasinv(rho, e, 
     &                       P, T, cpd, Gamma1, delta, h, s, cs,
     &                       ierror, ifion, ifent, kenter, 
     &                       abuX, abuY, Qmol)
C--------------------------------------------------------------------+-------
C NAME:
C   supergasinv ('super_gas_invert')
C
C PURPOSE:
C   Inversion of GAS routine: rho/e -> P,T,...
C
C CATEGORY:
C   EOS
C
C CALLING SEQUENCE:
C   call supergasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &                 ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C
C INPUT:
C   rho:    ('rho') density [g/cm^3]
C   e:      ('energy') internal energy per mass unit [erg/g]
C   ifion:  ('i_flag_ionization') integer, determines how to treat ionization
C           processes (0...16)
C   ifent:  ('i_flag_entropy') entropy is computed if ifent=1 (0...1)
C   abuX:   ('abundance_X') real, hydrogen abundance (number fraction)
C   abuY:   ('abundance_Y') real, helium abundance (number fraction)
C   abuZ:   ('abundance_Z') real, metal abundance (number fraction)
C   Qmol:   ('Q_molecular') real, mean molecular weight [u]
C
C INPUT/OUTPUT:
C   P:      ('Pressure') pressure [dyn/cm^2]
C   T:      ('Temperature') temperature [K]
C   kenter: ('k_enter') original GAS: must be <0 in initial call, otherwise >= 0
C
C OUTPUT:
C   cpd:    ('cp_dash') Cp' = d(S)/d(ln rho) | P [erg/g/K]
C           GAS: xcp
C   Gamma1: ('Gamma_1') 1. adiabatic exponent: Gamma1 = Cp'/Cv'
C                                                     = d(ln P)/d(ln rho) | s
C           GAS: xgam
C   delta:  ('delta') expansion coefficient: delta = alpha*T
C                                                  =-d(ln rho)/d(ln T) | P
C           GAS: cxt
C   h:      ('h') enthalpy per mass unit [erg/g]
C           GAS: entalp
C   s:      ('s') entropy per mass unit [erg/g/K]
C   cs:     ('c_sound') adiabatic sound velocity [cm/s]
C   ierror: ('index_error') error number
C
C ROUTINES:
C   gas: EOS routine of RHD-code (or gasgas)
C
C INCLUDES:
C   None
C
C COMMON BLOCK:
C   None
C
C SIDE EFFECTS:
C   None
C
C RESTRICTIONS:
C
C PROCEDURE:
C
C EXAMPLE:
C   kenter=-1
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C   kenter=0
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C
C MODIFICATION HISTORY:
C    4. 5.97 Written by Bernd Freytag, Astrophysik Kiel
C    6. 5.97 (B.F.) Correction in dp/drho|e
C   14. 5.97 (B.F.) nitermax=200
C   16. 9.97 (B.F.) call GAS with kenter0=0
C   15. 6.99 (B.F.) nitermax=30, return with error indication,
C            formula for initial guess of temperature changed
C   16. 6.99 (B.F.) Save start values, change in error formula
C   25. 2.04 (B.F.) Call GASGAS instead of GAS (was implicitely done before)
C   26. 2.04 (B.F.) Call SUPERGAS instead of GASGAS,
C                   nitermax=20->300, epsiter=1.0E-5 -> 1.0E-07
C                   Initial guess: e+1.5E+12 -> e+1.51E+12
C   2008-12-08 (B.F.) Temperatur of intital guess: > 200K
C   2014-09-06 (B.F. Uppsala) epsiter=1.0E-07 -> 1.0E-09
C   2014-09-10 (B.F.) Simple step-size control depending on number of
C                     iterations
C----------------------------------------------------------------------+-------
      implicit none
C
C --- I/O parameters ---
      real rho, e
      real P, T, cpd, Gamma1, delta, h, s, cs
      integer ifion, ifent
      integer kenter
      integer ierror
      real abuX, abuY, Qmol
C
C --- Local variables ---
      real rhoiter, eiter, error, error0, cv
      real Piter, Titer, Pstep, Tstep
      real P0, T0, rho0, cpd0, Gamma10, delta0, h0, s0, cs0
      integer niter, kenter0
      real Rgas, epsiter, stepfac, StepSize
      integer nitermax
      parameter(Rgas=8.3145E+07,
     &          epsiter=1.0E-9,
     &          nitermax=300,
     &          stepfac=10.0)
C----------------------------------------------------------------------+-------
C
C     --- Set initial step size ---
      StepSize=1.0
C
      if (kenter .lt. 0) then
C       -- Initialize GAS ---
        call supergas(8000.0, 16000.0,
     &                rhoiter, cpd, Gamma1, delta, h, s, cs,
     &                ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
      else
C       --- Initial values for interation ---
        if (kenter .gt. 0) then
C         --- Assume that in P and T good initial guesses for iteration are ---
C         --- present.                                                      ---
          Piter=P
          Titer=T
        else
C         --- Try simple initial guess for P and T from rho and e ---
C         Titer=(e+1.5E+12)/(1.5*Rgas)
          Titer=(e+1.51E+12)/(1.5*Rgas)
C         Titer=max(Titer,1500.)
          Titer=max(Titer,200.)
          Piter=Rgas/Qmol*Titer*rho
        endif
C
        niter=0
C
C       --- Iteration ---
C
   10   niter=niter+1
C       --- Solve EOS by call of GAS ---
        kenter0=0
        call supergas(Piter, Titer,
     &                rhoiter, cpd, Gamma1, delta, h, s, cs,
     &                ierror, ifion, ifent, kenter0, abuX, abuY, Qmol)
        if (niter .eq. 1) then
          P0     =Piter
          T0     =Titer
          rho0   =rhoiter
          cpd0   =cpd
          Gamma10=Gamma1
          delta0 =delta
          h0     =h
          s0     =s
          cs0    =cs
        endif
        cv=(cpd*delta)/(1.0+(Gamma1*delta*Piter)/(Titer*rhoiter*cpd))
        eiter=h-Piter/rhoiter
C       --- Perform step: next guess ---
        Pstep=Piter*Gamma1/rhoiter*(1.0-Piter/(rhoiter*Titer*cpd))
     &                                                 * (rhoiter-rho) +
     &        Piter*Gamma1/(Titer*cpd)                 * (eiter  -e  )
        Tstep=-Piter/(rhoiter*rhoiter)*(1.0/cv-Gamma1/cpd) 
     &                                                 * (rhoiter-rho) +
     &        1.0/cv                                   * (eiter  -e  )
C       --- Limit step size ---
        Pstep=min(max(Pstep,-0.5*Piter),0.5*Piter)
        Tstep=min(max(Tstep,-0.5*Titer),0.5*Titer)
C       --- Compute residual difference in rho and e ---
        error=abs(rhoiter-rho)/rho0 + abs(eiter  -e  )/(Rgas*T0)
        ! --- Check if step size should be reduced ---
        if (2*niter > nitermax) then
          StepSize=0.25
        else if (3*niter > nitermax) then
          StepSize=0.5
        endif
        write(6,'(I3,4(1X,E17.10),3X,E17.10,1X,F5.2)') 
     &      niter, Piter, Titer, Pstep, Tstep, error, StepSize
        Piter=Piter-StepSize*Pstep
        Titer=Titer-StepSize*Tstep
        if (niter .eq. 1) then
          error0=error
        else
          if (error .gt. stepfac*error0) then
C           --- 'Step too large' ---
            ierror=1
            goto 999
          endif
        endif
C       --- Check number of iterations ---
        if (niter .gt. nitermax) then
C         --- 'Too many iterations' ---
          ierror=2
          goto 999
        endif
C       --- Check residual error ---
        if (error .gt. epsiter) goto 10
C
C       --- Iteration finished ---
        P  =Piter
        T  =Titer
        ierror=0
C
      endif
C
 999  if (ierror .gt. 0) then
C       --- In case of error give back start values ---
        P     =P0
        T     =T0
        rho   =rho0
        cpd   =cpd0
        Gamma1=Gamma10
        delta =delta0
        h     =h0
        s     =s0
        cs    =cs0
      endif
      kenter=niter
C
      end subroutine supergasinv


      SUBROUTINE OWNINV(N,NDIM,A)
C*******************************************************************************
C***  MATRIX INVERSION 
C***  MATRIX A
C***  N = RANK OF SUBMATRIX (LEFT UPPER BLOCK) TO BE INVERTED
C***  NDIM = ROW DIMENSION OF TOTAL MATRIX
C***  N .LE. NDIM .LE. 100
C***
C***  Converted to standard FORTRAN 77 by HGL 31-MAR-1992 15:26:33
C***  Converted to double precision by HGL 4-APR-1992 15:58:29
C*******************************************************************************
      DOUBLE PRECISION A(NDIM,NDIM)
      DOUBLE PRECISION AMAX, ABAMAX, SAVE, AIK
      INTEGER IK(100),J K(100)
      IF (N .GT. NDIM) THEN
            WRITE( *, *) 'N .GT. NDIM'
            WRITE( *, *) 'ERROR IN OWNINV'
            STOP 
            ENDIF
      IF (NDIM .GT. 100) THEN
            WRITE( *, *) 'NDIM .GT. 100'
            WRITE( *, *) 'ERROR IN OWNINV'
            STOP 
            ENDIF
C
      DO 100  K=1,N
C
C     SUCHE MAXIMALES MATRIXELEMENT
      AMAX=0.
      ABAMAX=0.
      DO 30 J=K,N
        L=N+1-K
      IMAX=ISAMAX(L,A(K,J),1)+K-1
      IF(ABAMAX.GT.ABS(A(IMAX,J))) GOTO 30
      AMAX=A(IMAX,J)
      ABAMAX=ABS(AMAX)
      IK(K)=IMAX
      JK(K)=J
   30 CONTINUE
C
C***  STOP IN CASE OF SINGULARITY (PIVOT ELEMENT = 0 )
      IF (AMAX .EQ. .0) THEN
            WRITE( *, *) 'SINGULARITY DISCOVERED'
            WRITE( *, *) 'ERROR IN OWNINV'
            STOP 
            ENDIF
C
C     VERTAUSCHEN DER ZEILEN I UND K
      I=IK(K)
      IF(I.EQ.K) GOTO 51
      DO 50 J=1,N
      SAVE =A(K,J)
      A(K,J)=A(I,J)
   50 A(I,J)=-SAVE
C
C     VERTAUSCHEN DER SPALTEN J UND K
   51 J=JK(K)
      IF(J.EQ.K) GOTO 61
      DO 60 I=1,N
      SAVE = A(I,K)
      A(I,K)=A(I,J)
   60 A(I,J)=-SAVE
C
C     DIVISION DER SPALTE K DURCH AMAX
   61 DO 70  I=1,N
      A(I,K)=-A(I,K)/AMAX
   70 CONTINUE
      A(K,K)=0.
C
C     UMFORMEN DER ZEILEN UND SPALTEN
C***  ELEMENTARE UMFORMUNG:  ZEILE I = ZEILE I + A(I,K) * ZEILE K
      DO 81 I=1,N
      IF (I .EQ. K) GOTO 81
      AIK=A(I,K)
      DO 80 J=1,N
      A(I,J)=A(I,J)+AIK*A(K,J)
   80 CONTINUE
   81 CONTINUE
C
C***  SPALTE K: DIVISION DUCH AMAX
      DO 90 J=1,N
      A(K,J)=A(K,J)/AMAX
  90  CONTINUE
C
C***  DIAGONALELEMENT
      A(K,K)=1./AMAX
  100 CONTINUE
C
C
C     ZEILEN UND SPALTEN RUECKTAUSCHOPERATIONEN
      DO 130 L=1,N
      K=N-L+1
      J=IK(K)
      IF(J.LE.K) GOTO 111
C***  VERTAUSCHEN DER SPALTEN J UND K
      DO 110 I=1,N
       SAVE=A(I,K)
      A(I,K)=-A(I,J)
  110 A(I,J)=SAVE
  111 I=JK(K)
      IF(I .LE. K) GOTO 130
C***  VERTAUSCHEN DER ZEILEN I UND K
      DO 120 J=1,N
      SAVE=A(K,J)
      A(K,J)=-A(I,J)
  120 A(I,J)=SAVE
  130 CONTINUE
C
      RETURN
      END SUBROUTINE OWNINV


      INTEGER FUNCTION ISAMAX( N, SX, INCX)
***   ISAMAX finds the first index of the largest absolute value of the
***   elements of a vector. This function exists as CRAY routine.
***   Double precision version!
      DOUBLE PRECISION SX(*), CURMAX, TMP
      ISAMAX = 1
      CURMAX   = ABS( SX( 1))
      DO 10 I = 1, N, INCX
        TMP = ABS( SX( I))
        IF ( TMP .GT. CURMAX) THEN
          CURMAX = TMP
          ISAMAX = I
        ENDIF          
10    CONTINUE
      RETURN
      END FUNCTION ISAMAX


      SUBROUTINE QH2( TT, U, DUDT, D2UDT2)
********************************************************************************
*
* Version: Thu Jan 22 16:46:07 MET 1998
*
* Author : HGL
*
* Description: QH2 computes approximately the partition function of the H_2
*   molecule. See Bohn & Wolf (1984), A&A Vol.130, p.202!
* 
*
* Arguments, access, type, dimension, description:
*   T       R D : Temperature
*   U       W D : Partition function at temperature T
*   DUDT    W D : Derivative of U 
*   D2UDT2  W D : Second derivative of U
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: Designed in DOUBLE PRECISION
*   Version was corrected with respect to the one from 11-MAY-1992 13:39:58
*   to ensure that derivatives are consistent with U by replacing polynomial
*   expression F4 by the exact term. Now the function can savely be used over 
*   a much wider range in temperature. 
*   A more sophisticated T limiter is noy in use to guarantee a smooth 
*   transition between regions with and without H_2 molecule formation.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
*******************************************************************************
      IMPLICIT NONE
      DOUBLE PRECISION T, TT, U, DUDT, D2UDT2
      DOUBLE PRECISION DOT, DOTDOT, F2, F4, DF2, DDF2 

***   Constants
      DOUBLE PRECISION T0, TSMTH, TD, TDD
      PARAMETER( T0=6000.0, TSMTH=20000.0)
      DOUBLE PRECISION D00, D10, D20, D30, D40, D60
      PARAMETER( D00=0.0, D10=1.0, D20=2.0, D30=3.0, D40=4.0, D60=6.0)

***   Parameters are set to values adopted in GAS
      DOUBLE PRECISION A1, A2, A3      
      PARAMETER ( A1 = 5.797083E-03, A2 = 4.676132E-08, 
     >            A3 = 1.088582E-11 )
      DOUBLE PRECISION QCON1, QCON2, QCON3, QCON4, QCON5
      PARAMETER ( QCON1 =  9.55469E-01, QCON2 = 1.039473E-04,
     >            QCON3 = -7.19848E-08, QCON4 = 1.154352E-11,
     >            QCON5 = -6.403472E-16 )
      DOUBLE PRECISION D
***   This parameter is called Q2P2 in GAS
      PARAMETER ( D = 6.33262E+03)

***   Argument limiting:
***   Smoothly adopt boundary values for TT > T0
      CALL smthlm( T0, TSMTH, TT, T, TD, TDD)

      F2 = A1 + T * ( A2 + T * A3)
C      F4 = QCON1 + T* ( QCON2 + T * ( QCON3 + T * ( QCON4 + T * QCON5)))
***   The line above corresponds to the old version
***   The polynomial representation of F4 was replaced by the exact term
***   according to the paper by Bohn & Wolf. It is less efficient (exp)
***   but this does not seem to be a problem here.
***   The following is the update. The computation of DOT was moved here.
***   
      DOT    = D / T
      F4 = D10 - EXP(-DOT)
 
      U = T * F2 / F4

      DDF2 = D20 * A3
      DF2  = A2 + T * DDF2
C      DF4  = QCON2 + T * ( D20*QCON3 + T * ( D30*QCON4 + T*D40*QCON5))
C      DDF4 = D20 * QCON3 + T * ( D60 * QCON4 + T * D30 * D40 * QCON5)

      DOTDOT = DOT * DOT

      DUDT = ( F2 * ( DOT / F4 + ( D10 - DOT)) + T * DF2) / F4
      D2UDT2 = (( DOTDOT * F2 * 
     >          ( D20 / T / F4 - D30 / T) + D20 * DOT * DF2) / F4
     >       + F2 * DOTDOT / T + D20 * ( D10 - DOT) * DF2 + T * DDF2)
     >       / F4

***   Correct derivatives for TT -> T transformation
      DUDT   = DUDT * TD
      D2UDT2 = D2UDT2 * TD * TD + DUDT * TDD

      RETURN
      END SUBROUTINE QH2


      subroutine smthlm(x0, dx, x, xs, xsd, xsdd)
*******************************************************************************
* Version: Fri Jan 23 11:01:56 MET 1998
*
* Author : HGL
*
* Description: smthlm (SMooTH LiMiter) is intended to delimit smoothly 
*   the range of function arguments. I.e. if a function is well defined
*   in a certain interval of arguments only one can work around this problem
*   by modifying the argument before it is put into that function. Effectively
*   this defines a certain extrapolation procedure. The modified value xs, 
*   and the derivatives xsd = dxs/dx, xsdd = d2xsdx^2 are
*   derived from a 4th order polynomial having the following properties:
*   
*   x <= x0:      xs   = x
*                 xsd  = 1.0 (linear)
*                 xsdd = 0.0
*   x >= x0 + dx: xs   = x0 + 0.5*dx (asymtotic value for x->inf) 
*                 xsd  = 0.0
*                 xsdd = 0.0
*   x0 < x < x0 + dx: smooth connection of other two regions with continous
*                     first and second derivative.
*
* Arguments, access, type, dimension, description:
*   x0     R D : threshold where smoothing sets in 
*   dx     R D : smoothing length, for x>x0+dx the asymtotic plateau is reached
*   x      R D : original argument
*   xs     W D : modified argument
*   xsd    W D : d xs / d x
*   xsdd   W D : d2 xs / d x^2
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
* 
* Preprocessor macros   : None
*
* Comments:
*   Double precision!!
*
*   The relations above are given for positive x0 and dx. actually things work
*   as well for arbitrary combinations of sign of dx and x0!
*
* Modification history:
*   21/01/98/hgl: coded
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
*******************************************************************************
      implicit none
      double precision x0, dx, x, xs, xsd, xsdd
      double precision xn, xn2

      xn = (x-x0)/dx

      if (xn .ge. 1.0D0) then
        xs   = x0 + 0.5D0 * dx
        xsd  = 0.0
        xsdd = 0.0
        return
      endif 

      if (xn .le. 0.0D0) then
        xs   = x
        xsd  = 1.0D0
        xsdd = 0.0D0
        return
      endif  

      xn2  = xn*xn
      xs   = dx*(0.5D0*(xn-2.0D0)*xn2+1.0D0)*xn + x0
      xsd  = (2.0D0*xn-3.0D0)*xn2 + 1.0D0
      xsdd = 6.0D0*(xn2-xn)/dx

      return
      end subroutine smthlm


C----------------********----------------------------------------------+-------
      subroutine supergas(XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     &                    IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
C----------------------------------------------------------------------+-------
C NAME:
C   supergas ('super_gas')
C
C PURPOSE:
C   Compute thermodynamic quantities and derivatives
C   for given temperature and gas pressure
C
C CATEGORY:
C   EOS
C
C CALLING SEQUENCE:
C
C PROCEDURE:
c   Depending on temperature either gasgas is called
C   or the EOS for a perfect gas is computed.
C
C MODIFICATION HISTORY:
C   2004-02-26 (B.F.)
C----------------------------------------------------------------------+-------
C
      parameter(T_switch=500.0,
     &          Rgas=8.3145E+07)
      real mu
C
C----------------------------------------------------------------------+-------
c      kenter0=kenter
C
      if (kenter < 0) then
C       --- Initialization of gas and gasgas ---
C
c        call gas(XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
c     &           IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
C
c        kenter=kenter0
        call gasgas(XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     &              IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
      else
        if (xt >= T_switch) then
C         --- Call gasgas for high temperatures ---
C
          call gasgas(XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     &                IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
        else
C         --- Call gas for low temperatures ---
C
c         call gas(XP, XT, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
c     &             IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
C
C         --- Solve EOS for too switch temperature to get mu and gamma --- 
          call gasgas(XP, T_switch, RHO_switch, 
     &                XCP, XGAM, CXT, ENTALP, S, CS,
     &                IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
C
C         --- Compute mu and Gammas (and keep them constant) ---
          mu    =Rgas * (rho_switch*T_switch)/xP
          Gamma1=XGAM
          Gamma3=(xP * Gamma1) / (rho_switch * XCP * T_switch) + 1.0
          delta =CXT
          gamma =1.0/delta + (Gamma1*xP)/(T_switch*rho_switch*XCP)
C
C         --- Only one gamma, constants for perfect gas ---
          Gamma1=gamma
          Gamma3=gamma
          delta =1.0
          h_const=ENTALP - Rgas/mu/(gamma-1.0)*T_switch - xP/rho_switch
          s_sigma=mu/Rgas*s - alog(T_switch)/(gamma-1.0) +
     &            alog(rho_switch)
C
C         --- Perfect gas ---
          rho=xP/(Rgas/mu*xT)
          cpd=gamma/(gamma-1.0) * Rgas/mu
          h  =Rgas/mu/(gamma-1.0)*xT + xP/rho + h_const
          s  =Rgas/mu * (alog(xT)/(gamma-1.0) - alog(rho) + s_sigma)
C
C         --- GAS naming conventions ---
          XCP   =cpd
          XGAM  =gamma
          CXT   =delta
          ENTALP=h
          CS    =sqrt(gamma*xP/rho)
        endif
      endif
C
      end subroutine supergas

      end module gas_module
