      SUBROUTINE GAS ( P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                 IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
********************************************************************************
*
* Version: 29-OCT-1993 15:39:42
*
* Author : HGL
*
* Description: This routine computes thermodynamic quantities needed in the
*   HD context. The following ionization equilibria are considered:
*   H    <->  H+   + e-
*   He   <->  He+  + e-
*   He+  <->  He++ + e-
*   H_2 molecule formation is not taken into account. Nonideal effects are
*   neglected. In the routine it is assumed that the plasma is primarily
*   composed of hydrogen.
* 
*
* Arguments, access, type, dimension, description:
*   P      R R : Gas pressure
*   T      R R : Temperature
*   RHO    W R : Mass density
*   XCP    W R : HD specific heat
*   XGAM   W R : HD ratio of specific heats
*   CXT    W R : CXT = CP / XCP
*   ENTALP W R : Enthalpy per mass unit
*   S      W R : Entropy per mass unit
*   CS     W R : Sound speed
*   IERROR W I : Error indicator, 0: no error, 1: illegal IFION specified
*   IFENT  R I : Entropy requested? 1: yes, else: no
*   KENTER * I : Input : < 0 : Initialization of the routine
*                        else: Compute thermodynamic quantities
*                Output: Number of Newton-iterations
*   X      R R : Hydrogen abundance by number fraction
*   Y      R R : Helium abundance by number fraction
*   QMOL   R R : Mean molecular weight of neutral gas
*
* Common blocks (access): E (ifndef CONTEXT_HD)
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Preprocessor macros   : CONTEXT_HD
*
* Comments: None
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
***   EPSIT is the relative accuracy (related to total number density)
***   to which the electron density is computed
      PARAMETER( EPSIT = 1.0E-5)
***   EPSFUL determines the limit at which the plasma is regarded as 
***   fully ionozed.
      PARAMETER( EPSFUL = 1.0 / ( 0.5 * EPSIT))
***   EPSNTR is the limit at which the plasma is regarded as neutral
      PARAMETER( EPSNTR = 0.25 * EPSIT * EPSIT)
***
      PARAMETER( BOLTZK = 1.38054E-16, PLNCKH = 6.6256E-27)
      PARAMETER( WTUNIT = 1.6604E-24,  UGC = 8.3144E+07)
***   Atomic weights
      PARAMETER( WTHN = 1.0080, WTHEN = 4.0026, WTEL = 5.48609E-4)
      PARAMETER( WTHP = WTHN - WTEL, WTHEP = WTHEN - WTEL, 
     >           WTHEPP = WTHEP - WTEL)
***   Ionization energies
      PARAMETER( EIH = 2.1785E-11, EIHE = 3.9391E-11, 
     >           EIHEP = 8.7179E-11)
      PARAMETER( EIHEPP = EIHE + EIHEP)
***
      PARAMETER( R20PI = 6.283185307179586E+00)
***   AUX0 = 2 * pi * WTUNIT * k / h**2
      PARAMETER( AUX0 = R20PI * WTUNIT / PLNCKH * BOLTZK / PLNCKH)
***   AUX1 = 2 * pi * WTUNIT / h**2 * WTEL
      PARAMETER( AUX1 = R20PI * WTUNIT / PLNCKH / PLNCKH * WTEL)
***   Partitions functions of various species
***   Metal magnesium like ...
      PARAMETER( QHN = 2.0, QHP = 1.0, QHEN = 1.0, QHEP = 2.0,
     >           QHEPP = 1.0, QEL = 2.0, QM = 1.0)


C.#ifndef CONTEXT_HD
      COMMON / E / ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT
C.#endif

      SAVE QMA, Z, WTM, SAHN, SAHP, SAHEN, SAHEP, SAHEPP, SAEL, SAM
      SAVE P01, P02, P03, P04, P05, P06, P07, P08, P09, P10

      REAL LN10, KT
      DATA LN10 / 2.3025850929940457 /
***   Statement functions ...
      EXP10( XX) = EXP( LN10 * XX)

      IF ( KENTER .LT. 0 ) THEN
        QMA = WTUNIT * QMOL
***     Auxiliary parameters for entropy computation
***     These parameters are known before run time and could be computed
***     previously.
        SAHN   = 1.5 * LOG( AUX0 * WTHN)   + LOG( QHN)
        SAHP   = 1.5 * LOG( AUX0 * WTHP)   + LOG( QHP) 
        SAHEN  = 1.5 * LOG( AUX0 * WTHEN)  + LOG( QHEN) 
        SAHEP  = 1.5 * LOG( AUX0 * WTHEP)  + LOG( QHEP) 
        SAHEPP = 1.5 * LOG( AUX0 * WTHEPP) + LOG( QHEPP) 
        SAEL   = 1.5 * LOG( AUX0 * WTEL)   + LOG( QEL) 
***
        Z   = 1.0 - X - Y
        IF ( Z .GT. 0.0) THEN
          WTM = ( QMOL - WTHN * X - WTHEN * Y) / Z
          SAM = 1.5 * LOG( AUX0 * WTM) + LOG( QM)
        ELSE
          Z   = 0.0
          WTM = 0.0
          SAM = 0.0
        ENDIF
        P01 = 1.0 + X
        P02 = X + 2.0 * Y
        P03 = 1.0 + X + 2.0 * Y
        P04 = X + Y
        P05 = 1.0 + X + Y
        P06 = 1.0 + Y
        P07 = 1.0 + 2.0 * Y
        P08 = - ( X * EIH + Y * EIHEPP)
        P09 = 2.0 * Y
        P10 = X / ( 1.0 + X)
***
        WRITE( 6, '( /, 1X, 72(''=''))')
        WRITE( 6, '( 1X, ''GAS version 20-Oct-1992 (UR) busy!'')')
        WRITE( 6, '( 1X, 72(''=''))')
        WRITE( 6, '( /)')
        IERROR = 0
        RETURN
      ENDIF

      KENTER = 0

***   In this routine all ionization equilibria are solved.
***   Don't try to switch off one of these by IFION!
      IF ( IFION .NE. 1) THEN
        IERROR = 1
      ELSE
        IERROR = 0
      ENDIF

***   Temperature measured in kT
      KT = BOLTZK * T
      ANTOT = P / KT

***   ANTOT is the unit of particle number density which is used subsequently.
***   ANTOT is termed as the 'internal' unit while normal cgs units are 
***   termed 'external'.

***   The F?T factors should be measured in internal units.
      AUX2 = ( AUX1 * KT)**1.5 / ANTOT

***   F1(T)
      AUX3   = EIH / KT
      F1T    = AUX2 * EXP( - AUX3)
      DTF1T  = ( 1.5 + AUX3) * F1T / T

***   Check whether the plasma could be regarded as neutral
      IF ( F1T .LT. EPSNTR) THEN
***     In this if block all particle densities are measured in external units!
        ANA    = ANTOT
        ANHN   = X * ANTOT
        ANHEN  = Y * ANTOT
        ANEL   = 0.0
        ANHP   = 0.0
        ANHEP  = 0.0
        ANHEPP = 0.0
        RHO    = ANTOT * QMA
        OHR    = 1.0 / RHO
        DTRHO  = - RHO / T 
        DPRHO  =   RHO / P 
        E      = 1.5 * P * OHR
        DTEV   = 0.0
        GOTO 30
      ENDIF

***   Compute first guess of electron pressure.
***   The electron density is computed by regarding the ionization
***   equilibrium of hydrogen only.
      DUM  = P01 * F1T
      DUM2 = 4.0/DUM * (F1T/DUM) * X
      IF ( DUM2 .LT. EPSIT) THEN
***     Plasma nearly completely ionized
        ANEL = P10
      ELSE
        ANEL = 0.5 * DUM * (SQRT( 1.0 + DUM2) - 1.0)
      ENDIF

***   F3(T)
      AUX3   = EIHEP / KT
      F3T    = AUX2 * EXP( - AUX3)
      DTF3T  = ( 1.5 + AUX3) * F3T / T

      IF ( F3T / ANEL .GT. EPSFUL) THEN
***     In this if block all particle densities are measured in external units!
        ANA    = ANTOT / P03
        ANHP   = X * ANA
        ANHEPP = Y * ANA
        ANHN   = 0.0
        ANHEN  = 0.0
        ANHEP  = 0.0
        ANEL   = ANHP + ANHEPP + ANHEPP
        RHO    = ANA * QMA
        OHR    = 1.0 / RHO
        DTRHO  = - RHO / T 
        DPRHO  =   RHO / P 
        E      = ( ANHP * EIH + ANHEPP * EIHEPP + 1.5 * P) * OHR
        DTEV   = ANA * P08 / T
        GOTO 30
      ENDIF

***   F2(T)
      AUX3   = EIHE / KT
      F2T    = 4.0 * AUX2 * EXP( - AUX3)
      DTF2T  = ( 1.5 + AUX3) * F2T / T

***   The prefix DT denotes the temperature derivative at constant pressure
***   The prefix DP denotes the pressure derivative at constant temperature
      DTNTOT = - 1.0 / T
      DPNTOT =   1.0 / P

***   Some auxiliary parameters ...
      F1F3   = F1T * F3T
      F1F2F3 = F1F3 * F2T
      AUX4   = F1T * X + F2T * Y
      AUX5   = F1T * P04 + 2.0 * F3T * Y
      AUX6   = F1F3 * P03 - AUX5
      AUX7   = F1T * P05 + F3T * P07

***   Compute coefficients of the polynomial
***   A4 * ANEL**4 + A3 * ANEL**3 + A2 * ANEL**2 + A1 * ANEL + A0 = 0
***   which has to be solved in order to determine the electron density. 
      A0 = - F1F2F3 * P02
      A1 = F2T * AUX6
      A2 = F2T * AUX7 - AUX4
      A3 = F1T * P01 + F2T * P06
***   A4 is not used explicitly, so 
C      A4 = 1.0

      AUX8 = A2 + A2
      AUX9 = 3.0 * A3

***   Solve polynomial by Newton iteration
20    DANEL = (A0 + ANEL * ( A1 + ANEL * ( A2 + ANEL * ( A3 + ANEL ))))/
     >( A1 + ANEL * ( AUX8 + ANEL * (AUX9 + 4.0*ANEL)))
      ANEL = ANEL - DANEL
      KENTER = KENTER + 1
      IF ( ABS( DANEL) .GE. EPSIT) GOTO 20
      ONEL = 1.0 / ANEL

***   Compute derivatives of electron density ( d/dT, d/dp)
      DTA3 = P01 * DTF1T + P06 * DTF2T
      DTA2 = DTF2T * AUX7 + F2T * ( DTF1T * P05 + DTF3T * P07)
     >     - DTNTOT * AUX4 - ( DTF1T * X + DTF2T * Y)
      DTA1 = DTF2T * AUX6 + F2T * ( ( DTF1T * F3T + F1T * DTF3T) * P03
     >     - DTNTOT * AUX5 - ( DTF1T * P04 + P09 * DTF3T))
      DTA0 = - P02 * (
     >  ( F2T * ( DTF1T * F3T + F1T * DTF3T) + F1F3 * DTF2T)
     >   + F1F2F3 * DTNTOT )
C      DPA3 = 0.0
      DPA2 = - DPNTOT * AUX4
      DPA1 = - F2T * DPNTOT * AUX5
      DPA0 = - DPNTOT * P02 * F1F2F3

      DUM = 1.0 / 
     >    ( 4.0 + ONEL * ( AUX9 + ONEL * ( AUX8 + ONEL * A1)))
      DTNEL = - ( DTA3 + ONEL * ( DTA2 + ONEL * ( DTA1 + ONEL * DTA0)))
     >      * DUM
      DPNEL = - ONEL * ( DPA2 + ONEL * ( DPA1 + ONEL * DPA0))
     >      * DUM

***   Compute some auxiliary parameters
      DTNENE = DTNEL * ONEL
      F1NE   = F1T * ONEL
      F2NE   = F2T * ONEL
      F3NE   = F3T * ONEL
      DTF1NE = DTF1T * ONEL
      DTF2NE = DTF2T * ONEL
      DTF3NE = DTF3T * ONEL
***   Prefix TD denotes the temperature derivative of the fraction F?/ANEL
      TDF1NE = DTF1NE - DTNENE * F1NE
      TDF2NE = DTF2NE - DTNENE * F2NE
      TDF3NE = DTF3NE - DTNENE * F3NE
      HTERM  = 1.0 + F1NE
      HETERM = 1.0 + F2NE * ( 1.0 + F3NE)

***   Until now external units are used ...
***   Particle densities
      ANEL   = ANEL * ANTOT
      ANA    = ANTOT - ANEL
      ANHN   = X * ANA / HTERM
      ANHP   = ANHN * F1NE
      ANHEN  = Y * ANA / HETERM
      ANHEP  = ANHEN * F2NE
      ANHEPP = ANHEP * F3NE
***   Temperature derivatives of particle densities at constant pressure
      DTNTOT = DTNTOT * ANTOT
      DPNTOT = DPNTOT * ANTOT
      DTNEL  = DTNEL * ANTOT
      DPNEL  = DPNEL * ANTOT
      DTNA   = DTNTOT - DTNEL
      DTNHN  = ( X * DTNA - ANHN * TDF1NE) / HTERM
      DTNHP  = DTNHN * F1NE + ANHN * TDF1NE
      DTNHEN = ( Y * DTNA - ANHEN * 
     >           ( TDF2NE * ( 1.0 + F3NE) + F2NE * TDF3NE)) / HETERM
      DTNHEP  = DTNHEN * F2NE + ANHEN * TDF2NE
      DTNHPP  = DTNHEP * F3NE + ANHEP * TDF3NE

***   Mass density
      RHO = QMA * ANA
      OHR = 1.0 / RHO
***   Temperature derivative
      DTRHO = DTNA * QMA
***   Pressure derivative
      DPRHO = ( DPNTOT - DPNEL) * QMA
***   Internal energy
      E =( ANHP * EIH + ANHEP * EIHE + ANHEPP * EIHEPP + 1.5 * P) * OHR
***   Temperature derivative of internal energy per volume at const. pressure
      DTEV = DTNHP * EIH + DTNHEP * EIHE + DTNHPP * EIHEPP

***   =========================================================================
***   The remaining program section is used in any case (neutral or partial
***   ionization).
***   Temperature derivative of internal energy per mass at const. pressure
30    DTEM = ( DTEV - E * DTRHO) * OHR

      POHR   = P * OHR
      DTLRHO = DTRHO * OHR

***   Enthalpy
      ENTALP = E + POHR

***   CXT
      CXT = - T * DTLRHO

***   (dS/dp)_T
      DPS = DTLRHO * OHR

***   C_p
      CP =  DTEM - DTLRHO * POHR

***   C_p(s)
      XCP = CP / CXT

      CS   = 1.0 / ( DPRHO - DTRHO * DPS / CP * T)
      XGAM = CS / P * RHO
      CS = SQRT( CS)

C.#ifndef CONTEXT_HD
      CT = 1.0 / SQRT( DPRHO)
C.#endif

      IF ( IFENT .EQ. 1) THEN
***     Precision converted to external units
        PRECIS = EPSIT * ANTOT
        ANM    = Z * ANA
        S = ANTOT * ( 2.5 + 1.5 * LOG( T))
        IF ( ANHN  .GT. PRECIS) S = S + ANHN  * ( SAHN  - LOG( ANHN))
        IF ( ANHP  .GT. PRECIS) S = S + ANHP  * ( SAHP  - LOG( ANHP))
        IF ( ANHEN .GT. PRECIS) S = S + ANHEN * ( SAHEN - LOG( ANHEN))
        IF ( ANHEP .GT. PRECIS) S = S + ANHEP * ( SAHEP - LOG( ANHEP))
        IF ( ANHEPP .GT. PRECIS) 
     >    S = S + ANHEPP * ( SAHEPP - LOG( ANHEPP))
        IF ( ANEL  .GT. PRECIS) S = S + ANEL  * ( SAEL  - LOG( ANEL))
        IF ( ANM   .GT. PRECIS) S = S + ANM   * ( SAM   - LOG( ANM))
        S = S * BOLTZK * OHR
      ENDIF

      RETURN
      END
