C      SUBROUTINE GASGAS(P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
      SUBROUTINE GAS(P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                  IERROR, IFION, IFENT, KENTER, RX, RY, RQMOL)
C     >                  ANHN, ANHP, ANHEN, ANHEP, ANHEPP, ANH2)
********************************************************************************
*
* Version: Fri Jan 23 11:42:12 MET 1998
*
* Author : HGL
*
* Description: GASGAS is (almost) a functional twin to the famous routine GAS. 
*   A GAS-call may be replaced by a GASGAS-call. If the GAS common block /E/ 
*   is in use it has to be replaced by a similar common block /EE/. The meaning 
*   of the control parameters IERROR and KENTER is somewhat different. If these
*   parameters are used in a critical way in the calling module one has to
*   check the consequences.
* 
*   The internal method of determining the requested thermodynamic parameters 
*   is completely different from GAS. While GAS is a time-optimized routine
*   GASGAS is constructed to allow an easy change in the basic physics of the
*   fundamental set of equations which govern the various
*   ionization/dissoziation processes. GASGAS solves the given set of
*   fundamental particel density equations to a preset accuracy (Parameter
*   PREREL). From these densities the various thermodynamic parameters are
*   computed. Input parameters are the gas-pressure and the temperature.
*
* Arguments, access, type, dimension, description:
*   P      R R : Gas pressure
*   T      R R : Temperature
*   RHO    W R : Mass density
*   XCP    W R : C_p(s): Modified specific heat per massunit at constant p
*   XGAM   W R : C_p(s)/C_v(s): Modified ratio of specific heats
*   CXT    W R : C_p = CXT * C_p(s), conversion factor between C_p and C_p(s)
*   ENTALP W R : Enthalpy per massunit
*   S      W R : Entropy per massunit
*   CS     W R : Adiabatic sound velocity
*   IERROR W I : Error indicator:
*                =  0: No error detected
*                <> 0: No convergence during solution of fundamental equations;
*                      in this case don't trust anything which is returned by 
*                      GASGAS!
*   IFION  R I : Controls which ionization/dissoziation processes are
*                considered, see IFION translation table below.
*   KENTER R I : Controls the way GASGAS works:
*                <  0: Nothing is done, except IERROR is set to zero
*                =  0: Compute new initial particel densities, solve
*                      fundamental equations, compute thermodynamic parameters
*                      (this is the standard case)
*                >  0: Start solution of fundamental equations from particel 
*                      densities of previous GASGAS call, compute thermodynamic
*                      quantities. This KENTER value is useful if the requested
*                      p/T-parameters are not much different from the ones of
*                      the last GASGAS call.
*   RX     R R : Hydrogen abundance
*   RY     R R : Helium abundance
*   RQMOL  R R : Mean molecular weight of particels in the case of vanishing
*                ionization and H2-molecule formation
*   
*
* Common blocks (access): EE (W)
*
* External procedures   : BROYDN, EOSCND, FCSAHA, OWNINV, QH2
*
* File units (access)   : * (W)
*
*
* Comments: GASGAS is written in DOUBLE PRECISION!
*
*   Modification history:
*   03/11/97/hgl: coded
*   23/01/98/hgl: variable H_2 dissociation energy introduced but actually 
*     not used. 
*     Common block /EE/ extended to make ANH2 accessible
*     Switch to entropy computation analogous GAS possible via STGGMD
*   02/02/98/bf: PREREL: 1.0e-10 -> 1.0e-13
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************

***   Commons related EOSCND
      INCLUDE 'EOSPAR.INC'

      DOUBLE PRECISION DPART, DNCCM, WT32, DUM, DUMA, DUMB
      DOUBLE PRECISION BINV( MEQ, MEQ), DN( MEQ, 2), YWORK( MEQ, 3)
      DOUBLE PRECISION DNTOTS
***   Define indices of various densities
      PARAMETER( INA=1, INH2=2, INHN=3, INHP=4, INHEN=5, INHEP=6, 
     >           INHEPP=7, INEL=8, INM=9)
      DOUBLE PRECISION PREREL
      PARAMETER( PREREL = 1.0E-13)
      PARAMETER( MIMPS = 30)

      REAL P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS
***   declaration of DT and DP in EOSPAR.INC!
      DOUBLE PRECISION DRHO, DXCP, DXGAM, DCXT, DS, 
     >                 DCS
      DOUBLE PRECISION DE, DH, WTM, QMOL
      REAL DUM1, DUM2, DUM3, DUM4, DUM5, DUM6, DUM7, DUM8, DUM9

***   Thermodynamic auxiliary variables
      DOUBLE PRECISION DEVDTP, DEMDTP, DRODTP, DRODPT, DSDPT, DCVH2, 
     >                 DEH2, DCP
***   QH2 auxiliary variables
      DOUBLE PRECISION D2QDT2
***   QH2T, DLQDLT declared in EOSPAR.INC

***   Ionization energies including possible modifications
      DOUBLE PRECISION EIH, EIHE, EIHEP, EDH2, EDH2DT

***   Double precision real constants
      DOUBLE PRECISION D00, D05, D10, D15, D20, D25, D40, D80
      PARAMETER( D00 = 0.0, D05 = 0.5, D10 = 1.0, D15 = 1.5, D20 = 2.0, 
     >           D25 = 2.5, D40 = 4.0, D80 = 8.0)
      DOUBLE PRECISION LN10, EXP10

      INCLUDE 'matnat_23jan98.inc'

***   Commons related to GAS
      REAL ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT, ANH2
      COMMON / EE / ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT, ANH2
      REAL RX, RY, RQMOL

      LOGICAL DUMLOG, GASCMP

      SAVE DN, BINV, GASCMP

      EXTERNAL EOSCND

***   Statement functions ...
      EXP10( X) = EXP( LN10 * X)

      DATA LN10 / 2.3025850929940457 /
      DATA GASCMP / .FALSE. /

***====================================================================
      IERROR = 0

***   This routine needs no initialization
      IF ( KENTER .LT. 0) RETURN 

***   Initialize working variables
      X      = DBLE( RX)
      Y      = DBLE( RY)
      Z      = D10 - X - Y
      QMOL   = DBLE( RQMOL)
      IF( Z .LE. D00) THEN
        Z      = D00
        WTM    = D00
      ELSE
        WTM    = (QMOL - WTH * X - WTHE * Y) / Z
      ENDIF

      IS = 1 
      IW = 2
      NEQ = 8

      DP = DBLE( P)
      DT = DBLE( T)

***   Translate IFION to FAC??? parameters:
***      IFION  |  FACGT     FACF1T    FACF2T    FACF3T
***             |   H2        H         He        He+
***      -------|--------------------------------------
***        0    |   0.0       0.0       0.0       0.0
***        1    |   1.0       1.0       1.0       1.0
***        2    |   1.0       0.0       0.0       0.0
***        3    |   1.0       1.0       0.0       0.0
***        4    |   1.0       1.0       1.0       0.0
***        5    |   1.0       1.0       1.0       1.0
***        6    |   0.0       1.0       0.0       0.0
***        7    |   0.0       1.0       1.0       1.0
***      -------|--------------------------------------
***      ELSE   |   Error report and stop
***      -------|--------------------------------------
       IF ( IFION .EQ. 1) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D10
         FACF3T = D10
       ELSEIF ( IFION .EQ. 3) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 0) THEN
         FACGT  = D00
         FACF1T = D00
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 2) THEN
         FACGT  = D10
         FACF1T = D00
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 4) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D10
         FACF3T = D00
       ELSEIF ( IFION .EQ. 5) THEN
         FACGT  = D10
         FACF1T = D10
         FACF2T = D10
         FACF3T = D10
       ELSEIF ( IFION .EQ. 6) THEN
         FACGT  = D00
         FACF1T = D10
         FACF2T = D00
         FACF3T = D00
       ELSEIF ( IFION .EQ. 7) THEN
         FACGT  = D00
         FACF1T = D10
         FACF2T = D10
         FACF3T = D10
       ELSE
         WRITE( *, *) 'Illegal IFION encountered in GASGAS!'
         WRITE( *, *) 'IFION = ', IFION
         STOP
       ENDIF

***   Something like pressure ionization should go here
***   Obs that if it is dependent on particle densities some major
***   rearrangements must be done.
      EIH    = EIH0
      EIHE   = EIHE0
      EIHEP  = EIHEP0
      EDH2   = EDH20
      EDH2DT = D00

***   Set particle density independend quantities ...
***   Compute quantities related to the H2 partition function
      CALL QH2( DT, QH2T, DUM, D2QDT2)
      DLQDLT = DUM / QH2T * DT
***   Compute Saha factors
      CALL FCSAHA( DT, FACGT, FACF1T, FACF2T, FACF3T, 
     >             EIH, EIHE, EIHEP, EDH2, EDH2DT,
     >             QH2T, DLQDLT, GT, F1T, F2T, F3T, 
     >             DGTDT, DF1TDT, DF2TDT, DF3TDT)

***   Compute total particle density
      DNTOT           = DP / ( BOLTZK * DT)
***   New unit of particle density: DNTOT
      GT  = GT * DNTOT
      F1T = F1T / DNTOT
      F2T = F2T / DNTOT
      F3T = F3T / DNTOT
      DGTDT  = DGTDT * DNTOT
      DF1TDT = DF1TDT / DNTOT
      DF2TDT = DF2TDT / DNTOT
      DF3TDT = DF3TDT / DNTOT
***   Save value of DNTOT ...
      DNTOTS = DNTOT
***   and reset DNTOT.
      DNTOT  = D10

***   Set precision of computation ...
      DO 50 I = 1, NEQ
        PRECIS( I) = PREREL * DNTOT
50    CONTINUE

      IF ( KENTER .EQ. 0) THEN
***   Compute start values for exact solution of EOS.
***   Estimate n(H2)/n(HN) ratio:
      DUM = X * DNTOT * GT
      IF ( ( DUM .GE. D10) .OR. 
     >     ( F1T .LT. PRECIS( INHP))) THEN
***     H2-molecules dominant or H-ionization negligible
        DUM = ( D20 - X) * GT
        IF ( DUM .EQ. D00) THEN
***       No H2 - molecules, assume neutral plasma
          DN( INHN, IS) = X * DNTOT
        ELSE
          DN( INHN, IS) = D05 * 
     >              ( SQRT( D10 + D40 * DNTOT * X * DUM) - D10) / DUM
        ENDIF
        DN( INH2, IS) = ( DN( INHN, IS) * GT) * DN( INHN, IS)
        DN( INEL, IS) = D00
        DN( INHP, IS) = D00
      ELSE
***     Take into account: H-ionization, approximate H2-molecule formation 
        DUM             = ( D10 + X) * F1T
        DUMA            = D10 + ( DNTOT * GT) * X * ( D20 - X)
        DUMB            = D40 * DNTOT * X / ( D10 + X) * DUMA
        IF ( DUM .GT. ( 100.0 * DUMB)) THEN
***       Hydrogen is almost completely ionized ...
***       and H2 hopefully completely dissoziated, so DUMA = 1
          DN( INEL, IS)   = D05 * D05 * DUMB
        ELSE
          DN( INEL, IS)   = D05 * ( SQRT( DUM)*SQRT( DUM + DUMB) - DUM)
     >                    / DUMA
        ENDIF
        DN( INHP, IS)   = DN( INEL, IS)
        DN( INHN, IS)   = DN( INEL, IS) / F1T * DN( INHP, IS)
        DN( INH2, IS)   = DN( INHN, IS) * X * DNTOT * GT
      ENDIF
***   Remaining species ...
      DN( INA, IS)    = DNTOT - DN( INEL, IS) + DN( INH2, IS)
      DN( INM, IS)    = Z * DN( INA, IS)
      DN( INHEN, IS)  = Y * DN( INA, IS)
      DN( INHEP, IS)  = D00
      DN( INHEPP, IS) = D00

***   Compute Jacobi matrix of system of equations ...
***   B(i, j) = d y(j) / d x(i)
      CALL EOSJCB( DN( 1, IS), BINV)

***   Invert BINV
      CALL OWNINV( NEQ, MEQ, BINV)
***   Following ENDIF related to ( KENTER .EQ. 0)
      ENDIF

***   Compute improved solution of EOS constrain equations
      CALL BROYDN( MEQ, NEQ, MIMPS, PRECIS, EOSCND, 
     >             BINV, DN, YWORK, IS, IMPS, ISTAT)
      IF ( ISTAT .NE. 0) IERROR = 1
      DN( INM, IS) = Z * DN( INA, IS) 
***   Until now 9 equations are regarded
      NEQ = 9

***   Temperature and pressure derivatives of number densities (in DNTOTS)
***   ===================================================================
***   Work index
      IW  = 1
***   Temperature derivatives, index
      IDT = 2
***   Pressure derivatives, index
      IDP = 3
***   Compute Jacobi matrix of actual particle densities
      CALL EOSJCB( DN( 1, IS), BINV)
***   Invert it
      CALL OWNINV( NEQ, MEQ, BINV)

***   Compute temperature derivatives
      CALL EOSDDT( DN( 1, IS), YWORK( 1, IW))
      DO 110 I = 1, NEQ
        DUM = D00
        DO 100 J = 1, NEQ
          DUM = DUM - YWORK( J, IW) * BINV( J, I)
100     CONTINUE
        YWORK( I, IDT) = DUM
110   CONTINUE

***   Compute pressure derivatives
      CALL EOSDDP( DN( 1, IS), YWORK( 1, IW))
      DO 130 I = 1, NEQ
        DUM = D00
        DO 120 J = 1, NEQ
          DUM = DUM - YWORK( J, IW) * BINV( J, I)
120     CONTINUE
        YWORK( I, IDP) = DUM
130   CONTINUE

***   Density
***   =======
      DRHO = QMOL * DN( INA, IS) * ( WTUNIT * DNTOTS)

      IF ( IFENT .EQ. 1) THEN
***   Entropy
***   =======
***   Only those species are considered which are more abundand than the
***   numerical accuracy to which they are computed.
      DUM = D20PI * WTUNIT / PLNCKH * BOLTZK / PLNCKH
      DUM = DUM * SQRT( DUM)
      DS  = D00
***   H2-molecule
      IF ( DN( INH2, IS) .GT. PRECIS( INH2)) THEN
        DPART = QH2T
        DNCCM = DN( INH2, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D20 * SQRT(D20)
        ELSE
          WT32  = WTH2 * SQRT( WTH2)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
***     Add entropy fraction due to internal rotational and vibrational niveaus
        DS = DS + DNCCM * DLQDLT
C GAS entropy bug was:  DS = DS + DNCCM * ( LOG( DPART) + DLQDLT)
      ENDIF
***   H I
      IF ( DN( INHN, IS) .GT. PRECIS( INHN)) THEN
        DPART = D20
        DNCCM = DN( INHN, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D10
        ELSE
          WT32  = WTH * SQRT( WTH)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   H II
      IF ( DN( INHP, IS) .GT. PRECIS( INHP)) THEN
        DPART = D10
        DNCCM = DN( INHP, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D10
        ELSE
          WT32  = WTHP * SQRT( WTHP)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   He I
      IF ( DN( INHEN, IS) .GT. PRECIS( INHEN)) THEN
        DPART = D10
        DNCCM = DN( INHEN, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D80
        ELSE
          WT32  = WTHE * SQRT( WTHE)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   He II
      IF ( DN( INHEP, IS) .GT. PRECIS( INHEP)) THEN
        DPART = D20
        DNCCM = DN( INHEP, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D80
        ELSE
          WT32  = WTHEP * SQRT( WTHEP)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   He III
      IF ( DN( INHEPP, IS) .GT. PRECIS( INHEPP)) THEN
        DPART = D10
        DNCCM = DN( INHEPP, IS) * DNTOTS
        IF (GASCMP) THEN
          WT32 = D80
        ELSE
          WT32  = WTHEPP * SQRT( WTHEPP)
        ENDIF
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   Electrons
      IF ( DN( INEL, IS) .GT. PRECIS( INEL)) THEN
        DPART = D20
        DNCCM = DN( INEL, IS) * DNTOTS
        WT32  = WTEL * SQRT( WTEL)
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   Representative metal
      IF ( DN( INM, IS) .GT. PRECIS( INM)) THEN
***     Magnesium like, statistical weight 1
        DPART = D10
        DNCCM = DN( INM, IS) * DNTOTS
        WT32  = WTM * SQRT( WTM)
        DS = DS + DNCCM * LOG( DUM * WT32 * DPART / DNCCM)
      ENDIF
***   Add fractions due to total number ...
      DS = DS + DNTOTS * ( D15 * LOG( DT) + D25)
***   Convert to erg/g/K
      DS = DS * BOLTZK / DRHO 
***   Following ENDIF related to ( IFENT .EQ. 1)
      ENDIF

***   Internal energy and temperature derivative of internal energy per volume
***   ========================================================================
      DE    = D00
***   d e(per volume) / d T at constant pressure
      DEVDTP = D00
***   Ionization energy of H I
      IF ( DN( INHP, IS) .GT. PRECIS( INHP)) THEN
        DE = DE + DN( INHP, IS) * EIH
        DEVDTP = DEVDTP + YWORK( INHP, IDT) * EIH
      ENDIF
***   Ionization energy of He I
      IF ( DN( INHEP, IS) .GT. PRECIS( INHEP)) THEN
        DE = DE + DN( INHEP, IS) * EIHE
        DEVDTP = DEVDTP + YWORK( INHEP, IDT) * EIHE
      ENDIF
***   Ionization energy of He II
      IF ( DN( INHEPP, IS) .GT. PRECIS( INHEPP)) THEN
        DE = DE + DN( INHEPP, IS) * ( EIHE + EIHEP)
        DEVDTP = DEVDTP + YWORK( INHEPP, IDT) * ( EIHE + EIHEP)
      ENDIF
***   Energy of H2-molecule
      IF ( DN( INH2, IS) .GT. PRECIS( INH2)) THEN
        DEH2 = BOLTZK * DT * DLQDLT - EDH2
        DE = DE + DN( INH2, IS) * DEH2
        DCVH2 = BOLTZK * ( DLQDLT * ( D20 - DLQDLT) 
     >        + DT * DT / QH2T * D2QDT2)
        DEVDTP = DEVDTP + YWORK( INH2, IDT) * DEH2 
     >         + DN( INH2, IS) * DCVH2
      ENDIF
***   Kinetic energy of particles and/or conversion to cgs
      DE = DE * DNTOTS + D15 * DP
      DEVDTP = DNTOTS * DEVDTP
***   Conversion to energy per mass unit
      DE = DE / DRHO

***   Enthalpy
***   ========
      DH = DE + DP / DRHO

***   d rho / d T at constant pressure
***   ================================
      DRODTP = QMOL * WTUNIT * DNTOTS 
     >       * ( - D10 / DT - YWORK( INEL, IDT) + YWORK( INH2, IDT))

***   d rho / d p at constant temperature
***   ===================================
      DRODPT = QMOL * WTUNIT * DNTOTS 
     >       * ( D10 / DP - YWORK( INEL, IDP) + YWORK( INH2, IDP))

***   C_p / C_p(s)
***   ============
      DCXT = - DRODTP / DRHO * DT

***   d s / d p at constant temperature
***   =================================
      DSDPT = DRODTP / DRHO / DRHO

***   d e (per mass) / d T at constant pressure
***   =========================================
      DEMDTP = ( DEVDTP - DE * DRODTP) / DRHO
 
***   d h (per mass) / d T at constant pressure = C_p
***   ===============================================
      DCP = DEMDTP - DRODTP / DRHO * DP / DRHO

***   C_p(s)
***   ======
      DXCP = DCP / DCXT

***   Velocity of sound and ratio C_p(s) / C_v(s)
***   ===========================================
      DCS   = D10 / ( DRODPT - DRODTP * DSDPT / DCP * DT)
      DXGAM = DCS / DP * DRHO
      DCS   = SQRT( DCS)

***   Return single precision values ...
      ANHN   = REAL( DNTOTS * DN( INHN, IS))
      ANHP   = REAL( DNTOTS * DN( INHP, IS))
      ANHEN  = REAL( DNTOTS * DN( INHEN, IS))
      ANHEP  = REAL( DNTOTS * DN( INHEP, IS))
      ANHEPP = REAL( DNTOTS * DN( INHEPP, IS))
      ANH2   = REAL( DNTOTS * DN( INH2, IS))
      CT     = REAL( D10 / SQRT( DRODPT))
      RHO    = REAL( DRHO)
      XCP    = REAL( DXCP)
      XGAM   = REAL( DXGAM)
      CXT    = REAL( DCXT)
      ENTALP = REAL( DH)
      IF ( IFENT .EQ. 1) S = REAL( DS)
      CS     = REAL( DCS)

      RETURN

      ENTRY STGGMD( DUMLOG)
***   Set GASGAS compatibility (to GAS) mode
***   DUMLOG = .FALSE.: Normal GASGAS operation (default)
***          = .TRUE. : Entropy computation analogous to GAS, i.e. with integer
***                     atomic weights
      GASCMP = DUMLOG
      RETURN

      END
