C******************************************************************************
C     EOS routine: invert GAS to give function: rho/e -> P,T
C     17-Sep-1997 16:16:43
C     16.06.1999
C******************************************************************************


C----------------******------------------------------------------------+-------
      subroutine gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
     &                  ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C----------------------------------------------------------------------+-------
C NAME:
C   gasinv ('gas_invert')
C
C PURPOSE:
C   Inversion of GAS routine: rho/e -> P,T,...
C
C CATEGORY:
C   EOS
C
C CALLING SEQUENCE:
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C
C INPUT:
C   rho:    ('rho') density [g/cm^3]
C   e:      ('energy') internal energy per mass unit [erg/g]
C   ifion:  ('i_flag_ionization') integer, determines how to treat ionization
C           processes (0...16)
C   ifent:  ('i_flag_entropy') entropy is computed if ifent=1 (0...1)
C   abuX:   ('abundance_X') real, hydrogen abundance (number fraction)
C   abuY:   ('abundance_Y') real, helium abundance (number fraction)
C   abuZ:   ('abundance_Z') real, metal abundance (number fraction)
C   Qmol:   ('Q_molecular') real, mean molecular weight [u]
C
C INPUT/OUTPUT:
C   P:      ('Pressure') pressure [dyn/cm^2]
C   T:      ('Temperature') temperature [K]
C   kenter: ('k_enter') original GAS: must be <0 in initial call, otherwise >= 0
C
C OUTPUT:
C   cpd:    ('cp_dash') Cp' = d(S)/d(ln rho) | P [erg/g/K]
C           GAS: xcp
C   Gamma1: ('Gamma_1') 1. adiabatic exponent: Gamma1 = Cp'/Cv'
C                                                     = d(ln P)/d(ln rho) | s
C           GAS: xgam
C   delta:  ('delta') expansion coefficient: delta = alpha*T
C                                                  =-d(ln rho)/d(ln T) | P
C           GAS: cxt
C   h:      ('h') enthalpy per mass unit [erg/g]
C           GAS: entalp
C   s:      ('s') entropy per mass unit [erg/g/K]
C   cs:     ('c_sound') adiabatic sound velocity [cm/s]
C   ierror: ('index_error') error number
C
C ROUTINES:
C   gas: EOS routine of RHD-code (or gasgas)
C
C INCLUDES:
C   None
C
C COMMON BLOCK:
C   None
C
C SIDE EFFECTS:
C   None
C
C RESTRICTIONS:
C
C PROCEDURE:
C
C EXAMPLE:
C   kenter=-1
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C   kenter=0
C   call gasinv(rho, e, P, T, cpd, Gamma1, delta, h, s, cs,
C  &            ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
C
C MODIFICATION HISTORY:
C    4. 5.97 Written by Bernd Freytag, Astrophysik Kiel
C    6. 5.97 (B.F.) Correction in dp/drho|e
C   14. 5.97 (B.F.) nitermax=200
C   16. 9.97 (B.F.) call GAS with kenter0=0
C   15. 6.99 (B.F.) nitermax=30, return with error indication,
C            formula for initial guess of temperature changed
C   16. 6.99 (B.F.) Save start values, change in error formula
C----------------------------------------------------------------------+-------
      implicit none
C
C --- I/O parameters ---
      real rho, e
      real P, T, cpd, Gamma1, delta, h, s, cs
      integer ifion, ifent
      integer kenter
      integer ierror
      real abuX, abuY, Qmol
C
C --- Local variables ---
      real rhoiter, eiter, error, error0, cv
      real Piter, Titer, Pstep, Tstep
      real P0, T0, rho0, cpd0, Gamma10, delta0, h0, s0, cs0
      integer niter, kenter0
      real Rgas, epsiter, stepfac
      integer nitermax
      parameter(Rgas=8.3145E+07,
     &          epsiter=1.0E-5,
     &          nitermax=30,
     &          stepfac=10.0)
C----------------------------------------------------------------------+-------
C
      if (kenter .lt. 0) then
C       -- Initialize GAS ---
        call gas(8000.0, 16000.0,
     &           rhoiter, cpd, Gamma1, delta, h, s, cs,
     &           ierror, ifion, ifent, kenter, abuX, abuY, Qmol)
      else
C       --- Initial values for interation ---
        if (kenter .gt. 0) then
C         --- Assume that in P and T good initial guesses for iteration are ---
C         --- present.                                                      ---
          Piter=P
          Titer=T
        else
C         --- Try simple initial guess for P and T from rho and e ---
          Titer=(e+1.5E+12)/(1.5*Rgas)
C         Titer=max(Titer,1500.)
          Piter=Rgas/Qmol*Titer*rho
        endif
C
        niter=0
C
C       --- Iteration ---
C
   10   niter=niter+1
C       --- Solve EOS by call of GAS ---
        kenter0=0
        call gas(Piter, Titer,
     &           rhoiter, cpd, Gamma1, delta, h, s, cs,
     &           ierror, ifion, ifent, kenter0, abuX, abuY, Qmol)
        if (niter .eq. 1) then
          P0     =Piter
          T0     =Titer
          rho0   =rhoiter
          cpd0   =cpd
          Gamma10=Gamma1
          delta0 =delta
          h0     =h
          s0     =s
          cs0    =cs
        endif
        cv=(cpd*delta)/(1.0+(Gamma1*delta*Piter)/(Titer*rhoiter*cpd))
        eiter=h-Piter/rhoiter
C       --- Perform step: next guess ---
        Pstep=Piter*Gamma1/rhoiter*(1.0-Piter/(rhoiter*Titer*cpd))
     &                                                 * (rhoiter-rho) +
     &        Piter*Gamma1/(Titer*cpd)                 * (eiter  -e  )
        Tstep=-Piter/(rhoiter*rhoiter)*(1.0/cv-Gamma1/cpd) 
     &                                                 * (rhoiter-rho) +
     &        1.0/cv                                   * (eiter  -e  )
C       --- Limit step size ---
        Pstep=min(max(Pstep,-0.5*Piter),0.5*Piter)
        Tstep=min(max(Tstep,-0.5*Titer),0.5*Titer)
        write(6,'(I3,1X,4(E17.10,1X))') niter, Piter, Titer, Pstep, 
     &                                  Tstep
        Piter=Piter-Pstep
        Titer=Titer-Tstep
C       --- Compute residual difference in rho and e ---
        error=abs(rhoiter-rho)/rho0 + abs(eiter  -e  )/(Rgas*T0)
        if (niter .eq. 1) then
          error0=error
        else
          if (error .gt. stepfac*error0) then
C           --- 'Step too large' ---
            ierror=1
            goto 999
          endif
        endif
C       --- Check number of iterations ---
        if (niter .gt. nitermax) then
C         --- 'Too many iterations' ---
          ierror=2
          goto 999
        endif
C       --- Check residual error ---
        if (error .gt. epsiter) goto 10
C
C       --- Iteration finished ---
        P  =Piter
        T  =Titer
        ierror=0
C
      endif
C
 999  if (ierror .gt. 0) then
C       --- In case of error give back start values ---
        P     =P0
        T     =T0
        rho   =rho0
        cpd   =cpd0
        Gamma1=Gamma10
        delta =delta0
        h     =h0
        s     =s0
        cs    =cs0
      endif
      kenter=niter
C
      end
