      SUBROUTINE QH2( TT, U, DUDT, D2UDT2)
********************************************************************************
*
* Version: Thu Jan 22 16:46:07 MET 1998
*
* Author : HGL
*
* Description: QH2 computes approximately the partition function of the H_2
*   molecule. See Bohn & Wolf (1984), A&A Vol.130, p.202!
* 
*
* Arguments, access, type, dimension, description:
*   T       R D : Temperature
*   U       W D : Partition function at temperature T
*   DUDT    W D : Derivative of U 
*   D2UDT2  W D : Second derivative of U
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: Designed in DOUBLE PRECISION
*   Version was corrected with respect to the one from 11-MAY-1992 13:39:58
*   to ensure that derivatives are consistent with U by replacing polynomial
*   expression F4 by the exact term. Now the function can savely be used over 
*   a much wider range in temperature. 
*   A more sophisticated T limiter is noy in use to guarantee a smooth 
*   transition between regions with and without H_2 molecule formation.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
*******************************************************************************
      IMPLICIT NONE
      DOUBLE PRECISION T, TT, U, DUDT, D2UDT2
      DOUBLE PRECISION DOT, DOTDOT, F2, F4, DF2, DDF2 

***   Constants
      DOUBLE PRECISION T0, TSMTH, TD, TDD
      PARAMETER( T0=6000.0, TSMTH=20000.0)
      DOUBLE PRECISION D00, D10, D20, D30, D40, D60
      PARAMETER( D00=0.0, D10=1.0, D20=2.0, D30=3.0, D40=4.0, D60=6.0)

***   Parameters are set to values adopted in GAS
      DOUBLE PRECISION A1, A2, A3      
      PARAMETER ( A1 = 5.797083E-03, A2 = 4.676132E-08, 
     >            A3 = 1.088582E-11 )
      DOUBLE PRECISION QCON1, QCON2, QCON3, QCON4, QCON5
      PARAMETER ( QCON1 =  9.55469E-01, QCON2 = 1.039473E-04,
     >            QCON3 = -7.19848E-08, QCON4 = 1.154352E-11,
     >            QCON5 = -6.403472E-16 )
      DOUBLE PRECISION D
***   This parameter is called Q2P2 in GAS
      PARAMETER ( D = 6.33262E+03)

***   Argument limiting:
***   Smoothly adopt boundary values for TT > T0
      CALL SMTHLM( T0, TSMTH, TT, T, TD, TDD)

      F2 = A1 + T * ( A2 + T * A3)
C      F4 = QCON1 + T* ( QCON2 + T * ( QCON3 + T * ( QCON4 + T * QCON5)))
***   The line above corresponds to the old version
***   The polynomial representation of F4 was replaced by the exact term
***   according to the paper by Bohn & Wolf. It is less efficient (exp)
***   but this does not seem to be a problem here.
***   The following is the update. The computation of DOT was moved here.
***   
      DOT    = D / T
      F4 = D10 - EXP(-DOT)
 
      U = T * F2 / F4

      DDF2 = D20 * A3
      DF2  = A2 + T * DDF2
C      DF4  = QCON2 + T * ( D20*QCON3 + T * ( D30*QCON4 + T*D40*QCON5))
C      DDF4 = D20 * QCON3 + T * ( D60 * QCON4 + T * D30 * D40 * QCON5)

      DOTDOT = DOT * DOT

      DUDT = ( F2 * ( DOT / F4 + ( D10 - DOT)) + T * DF2) / F4
      D2UDT2 = (( DOTDOT * F2 * 
     >          ( D20 / T / F4 - D30 / T) + D20 * DOT * DF2) / F4
     >       + F2 * DOTDOT / T + D20 * ( D10 - DOT) * DF2 + T * DDF2)
     >       / F4

***   Correct derivatives for TT -> T transformation
      DUDT   = DUDT * TD
      D2UDT2 = D2UDT2 * TD * TD + DUDT * TDD

      RETURN
      END
