!=======================================================================
      module dust_Al2O3AGB_module
!=======================================================================
C
      contains
C
C-----------------------------------------------------------------------
      SUBROUTINE dust_Al2O3AGB_1Dchange(
     #            NPT,NEQ,YOLD,TSTEP,RHOINT,TEMINT,
     #            CNDNH,EPSINT,R0DUST,YNEW,DV)
C-----------------------------------------------------------------------
C FORTRAN-Subroutine:
C
C Author  : S. Hoefner
C Created : 19-MAY-2017 
C
C Description:
C
C    calculates dust quantities at the new time level
C    from values at old time level
C
C      - integration of grain growth equations if appropriate
C      - no dust if conditions are not suitable for dust
C
C-----------------------------------------------------------------------
C Input variables:
C
C    NPT       : number of points
C    TSTEP     : time step 
C
C    YOLD      : K3/RHO at old time level
C    RHOINT    : gas density
C    TEMINT    : gas temperature
C
C Output variables:
C
C    YNEW      : K3/RHO at new time level
C
C Local variables and constants:
C
C    TCH       : gas  temperature for chemistry, minimum value 400 K
C    TDCH      : dust temperature for chemistry, minimum value 400 K
C
C-----------------------------------------------------------------------
C Subroutines and functions referenced:
C
C    DUSTAL: grain growth rates
C
C-----------------------------------------------------------------------
C Further comments:
C
C  - T_dust = T_gas  
C  - minimum temperatures in DUSTAL: 400 K (chemistry)
C
C***********************************************************************
C-----------------------------------------------------------------------
C Declarations:

      IMPLICIT NONE

C Double Precision:

      DOUBLE PRECISION
     # Z0,Z1,Z2,Z3,Z4,
     # ZNL,PMASS,SMASS

C Numerical constants:

      PARAMETER (Z0=0., Z1=1., Z2=2., Z3=3., Z4=4.)

C Constants and parameters for dust formation:

      PARAMETER ( ZNL   = 1.D+03 ) 
      PARAMETER ( PMASS = 1.6726E-24 )          ! proton mass

      INTEGER 
     #  NPT,NEQ,L,J,
     #  ITCH,ITDCH,IXNOC,IXNSI,IXNAL,ISDUS,ITAU3

      PARAMETER (ITCH=1,ITDCH=2,IXNOC=3,IXNSI=4,IXNAL=5,
     #           ISDUS=6,ITAU3=7)

      DOUBLE PRECISION
     #  YOLD(NPT,NEQ),YNEW(NPT,NEQ),TSTEP,
     #  RHOINT(NPT),TEMINT(NPT),HEH,EPSINT,R0DUST,
     #  EPSC,EPSO,EPSAL,EPSSI,HELPOC,HELPSI,HELPAL,
     #  XK0D,XK2D,XK3K0,CNDNH(NEQ),
     #  DV(NPT,7),FCLIM

      CHARACTER*70 TXDUS

C-----------------------------------------------------------------------
C Set abundances locally, just for testing ...

      EPSC   = 3.303E-04               
      EPSO   = 6.607E-04   
      EPSSI  = 3.550E-05   
      EPSAL  = 2.950E-06

      HEH    = 10.D00**(-1.00D00)      ! abundance of He

C     auxiliary variables

      SMASS = PMASS *( 1.0D00 + HEH*Z4 )

      HELPOC = ( EPSO - EPSC ) / SMASS
      HELPSI = EPSSI / SMASS
      HELPAL = EPSAL / SMASS

      FCLIM  = Z1 - 1.0D-02  ! maximum allowed f_Al (numerics)

C-----------------------------------------------------------------------
C Calculate moments at new time level:

      DO J=1,NEQ   ! loop over "species"

        DO L=1,NPT

!         enforce upper and lower limit for f_cond 

          YOLD(L,J) = MIN( YOLD(L,J), HELPAL*FCLIM/Z2 )
          YOLD(L,J) = MAX( YOLD(L,J), Z0 )

C         gas and dust temperatures

          DV(L,ITCH)  = MAX ( TEMINT(L), 4.0D+02 ) 
          DV(L,ITDCH) = DV(L,ITCH)               !!! T_dust = T_gas !!!

C         number densities of O-C and Al in the gas phase

          DV(L,IXNOC) = ( HELPOC - 3.0*YOLD(L,J) ) * RHOINT(L) 
          DV(L,IXNAL) = ( HELPAL - 2.0*YOLD(L,J) ) * RHOINT(L)

C         a few adjustments, just in case ...

          DV(L,IXNOC)  = MAX ( DV(L,IXNOC), Z0 )
          DV(L,IXNAL)  = MAX ( DV(L,IXNAL), Z0 )

        END DO

C       compute growth rates

        CALL DUSTAL
     #     ( NPT,DV(1,ITCH),RHOINT,
     #       DV(1,IXNOC),DV(1,IXNSI),DV(1,IXNAL),DV(1,ITDCH),
     #       DV(1,ITAU3),DV(1,ISDUS),R0DUST,TXDUS )

C       integration of grain growth equations between two time levels

        DO L=1,NPT

C         replace moments K0/rho and K2/rho in the source terms for K3/rho

C                  replacing K0/rho by a constant (parameter)

          XK0D   = CNDNH(J) / SMASS

          XK3K0  = YOLD(L,J) / XK0D
          XK3K0  = MAX( XK3K0, Z0 )

C                  replacing K2 by K0 * N**2/3

          XK2D   = XK0D * ( XK3K0 + ZNL )**(Z2/Z3)

C         check if dust equations should be integrated in this interval

          IF ( ( DV(L,ITAU3) .LE. Z0 ) .AND.
     #       ( ( YOLD(L,J) .LT. (EPSINT*HELPAL) ) .OR. 
     #           ( YOLD(L,J) .LT. (ZNL*XK0D) ) ) ) THEN
          
C           no dust here and now

            YNEW(L,J) = Z0

          ELSE

C           new values of the moments

            YNEW(L,J) = YOLD(L,J) + DV(L,ITAU3) * XK2D  * TSTEP

!           enforce upper and lower limits for f_cond 

            YNEW(L,J) = MIN( YNEW(L,J), HELPAL*FCLIM/Z2 )
            YNEW(L,J) = MAX( YNEW(L,J), Z0 )

          END IF

        END DO

      END DO

C-----------------------------------------------------------------------
C End of dust_Al2O3AGB_1Dchange

      RETURN
      END SUBROUTINE


C======================================================================
C
C   Dust microphysics & chemistry: Al2O3
C
C   15-APR-2015: DARWIN version (materd_al2o3_d399_al_notridi.f)
C   19-MAY-2017: CO5BOLD version
C
C======================================================================

      SUBROUTINE  DUSTAL(
     #                NPT,TEMP,RHOG,XNOC,XNSI,XNAL,TDUS,
     #                TAU3,SDUS,R0,TXDUST )
C
C----------------------------------------------------------------------
C Author :   Susanne Hoefner
C Date   :   15-APR-2015: DARWIN version
C            19-MAY-2017: CO5BOLD version
C
C Description :
C
C     calculates the net grain growth rate for corundum (Al2O3)
C
C     current version assumes 
C       - that all gas-phase Al is in atoms 
C       - formation of corundum: 2 Al + 3 H2O -> Al2O3 + 3 H2
C       (see Gail & Sedlmayr, "Phys. & Chem. of CDSs", p.307f)
C
C......................................................................
C
C  input:
C
C     NPT    :  number of points
C     TEMP   :  gas temperature
C     RHOG   :  gas density  [g/cm^3]
C     XN..   :  number densities of relevant elements in the gas phase
C               note: XNOC = XNO_gas - XNCO
C     TDUS   :  dust temperature
C
C  output:
C
C     SDUS   :  supersaturation ratio at T_dust
C     TAU3   :  net growth rate
C     R0     :  monomer radius
C     TXDUST :  identification text
C
C  parameters:
C
C     AW     :  'atomic weight' of dust monomers
C     RHOD   :  density of condensed material
C     STICK  :  sticking coefficent
C
C.......................................................................
C
C  subroutines called :
C
C     LNKPAL :  ln(K_p) for Al2O3
C
C.......................................................................
C
C  further comments :
C
C     molecules considered: H2O, SiO (for n_H2O) 
C           (note: CO is already assumed subtracted from n_O)
C
C-----------------------------------------------------------------------
C declarations and parameters:

      IMPLICIT NONE

      INTEGER  NPT,L

      DOUBLE PRECISION
     #   AW,RHOD,STICK,R0,R02,R024PI,WTTD,CT3AL,
     #   XLNBLZ,XLNTG,XNH2,XNH2O,XNSIO,
     #   XLNH2,XLNH2O,XLNAL,ELNAL,SDAL

      DOUBLE PRECISION
     #   TEMP(NPT),RHOG(NPT),TDUS(NPT),
     #   XNOC(NPT),XNSI(NPT),XNAL(NPT),
     #   TAU3(NPT),
     #   SDUS(NPT),XLKPAL(NPT)

      CHARACTER*10 TXRD
      CHARACTER*50 TXHELP
      CHARACTER*70 TXDUST

C     numerical and physical constants

      DOUBLE PRECISION
     #   PI,BOLTZ,PMAS,Z0,Z1,Z2,Z3,Z4,Z4PI,HEH,SMASS

      PARAMETER (PI     = 3.1415926535898,
     #           BOLTZ  = 1.38062E-16,
     #           PMAS   = 1.6726E-24)

      PARAMETER (Z0=0., Z1=1., Z2=2., Z3=3., Z4=4.,
     #           Z4PI  = Z4*PI,
     #           HEH   = 1.D-01,
     #           SMASS = PMAS *( 1.0D00 + HEH*Z4 ))

C-----------------------------------------------------------------------
C setting various constants:

C     sticking coeff., grain material density, atomic weight of monomer

      STICK = Z1      !   whatever ...
      RHOD  = 3.99    !   Al2O3 (G&S Ph+Ch_CDS Tab.12.1/CRC-HB)
      AW    = Z2*27.0 + Z3*16.0

      TXHELP =
     #   'DUSTAL_A_RD  : O-rich (H2O, SiO; Al2O3; rate: 2 Al)'
      WRITE (TXRD, '(1PE10.3)') RHOD
      TXDUST = TXHELP // ', RHOD=' // TXRD

C     calculate monomer radius and other useful quantities

      R0     = ( Z3 * AW * PMAS / ( Z4PI * RHOD ) )**(Z1/Z3)
      R02    = R0**2
      R024PI = Z4PI * R02

      CT3AL  = R024PI * STICK * SQRT ( BOLTZ / ( Z2*PI*PMAS*27.0 ) )
     #         / Z2             ! 2 Al per monomer

      XLNBLZ = LOG(BOLTZ)

C-----------------------------------------------------------------------
C chemistry and net growth rate:

      DO L=1,NPT

C       gas phase chemistry (rather simple):

        XNH2  = RHOG(L)/ (Z2 * SMASS)    ! n_H2
        XNSIO = XNSI(L)                  ! all Si in SiO
        XNH2O = XNOC(L) - XNSIO          ! H2O : (O-CO) - SiO

C       partial pressures (number densities) of Al 
C                         in equilibrium with dust phase
C           ... better do this in ln ...

        CALL  LNKPAL ( 1,TDUS(L),XLKPAL(L) ) 

        XLNH2  = LOG(XNH2)
        XLNH2O = LOG(XNH2O)
        XLNAL  = LOG(XNAL(L))

        XLNTG  = LOG(TEMP(L))      ! to convert pps to number densities

C       number densities of Al in CE with Al2O3 (ln)

        ELNAL  = ( Z3*XLNH2 - Z3*XLNH2O
     #              - XLKPAL(L)
     #              - Z2* ( XLNTG + XLNBLZ ) ) / Z2

C       supersaturation ratios for Al over Al2O3

        SDAL    = EXP( XLNAL  - ELNAL  )

C       assembling the net growth rate for corundum

        WTTD   = SQRT( TEMP(L)/TDUS(L) )

        SDUS(L) = SDAL

C       net growth rate (addition of 2 Al to grain surface)

        TAU3(L)   = CT3AL
     #              * SQRT (TEMP(L)) * XNAL(L)
     #              * ( Z1 - Z1/SDUS(L) * WTTD )

      END DO

C----------------------------------------------------------------------

      RETURN
      END SUBROUTINE

C----------------------------------------------------------------------


      SUBROUTINE LNKPAL ( N,TEMP,XLKPAL )
C
C----------------------------------------------------------------------
C Author :   Susanne Hoefner
C Date   :   13-APR-2015: DARWIN version
C            18-MAY-2017: CO5BOLD version
C
C Description :
C
C     calculating ln(Kp_Al2O3) for given T 
C......................................................................
C
C  input :
C
C     N          : number of points
C     TEMP       : temperature
C
C  output:
C
C     XLKPAL     : ln(Kp_Al2O3)
C
C  parameters:
C
C     DGEFAL     : Gibbs energy of formation of Al2O3 (corundum)
C     DGEF(i,j)  : coeff. j for species i in polynomial fit
C
C.......................................................................
C
C  called subroutines :
C     *** none ***
C.......................................................................
C
C  further comments :
C
C     following Gail & Sedlmayr, "Phys. & Chem. of CDSs" (p.307f) 
C        assuming that all Al in the gas is in atomic form ->
C        formation of corundum:  2 Al + 3 H2O -> Al2O3 + 3 H2
C
C     polynomial fits for free energies from Sharp & Huebner (1990)
C
C     ln (Kp_Al2O3) = delta G / (R*T)
C
C                     delta G =   3 delta G_H2O
C                               - 3 delta G_H2  - delta G_Al2O3
C
C-----------------------------------------------------------------------
C declarations and parameters:

      IMPLICIT NONE

      INTEGER   N,I,L,NSPEC,NCOEF,
     #          IH2O,IH2,ICRN,IGA,IGB,IGC,IGD,IGE

      PARAMETER ( NSPEC=3, NCOEF=5,
     #            IH2O=1, IH2=2, ICRN=3,
     #            IGA=1, IGB=2, IGC=3, IGD=4, IGE=5 )

      DOUBLE PRECISION
     #    TEMP(N),XLKPAL(N),
     #    DGEF(NSPEC,NCOEF),DGEFAL(NCOEF),
     #    RCALMOL,ADYCM2,Z0,Z1,Z2,Z3

      PARAMETER ( Z0=0., Z1=1., Z2=2., Z3=3. )

      PARAMETER ( RCALMOL = 1.98D+00,         ! gas const. [cal/mole/K]
     #            ADYCM2 = Z1/1.01325D+06 )   ! [atm / (dyn/cm2)]

C     coefficients for polynomials (Sharp & Huebner 1990):
C        delta G = a/T + b + c*T + d*T**2 + e**3  [cal/mole]
C                                                 -> pressures in [atm]

      DATA (DGEF(IH2O,I), I=1,NCOEF) /  8.66184E+05, -2.27857E+05,
     #      5.61473E+01,  7.62548E-04, -4.95254E-08 /

      DATA (DGEF(IH2, I), I=1,NCOEF) /  4.25321E+05, -1.07123E+05,
     #      2.69980E+01,  5.48280E-04, -3.81498E-08 /

      DATA (DGEF(ICRN,I), I=1,NCOEF) /  0.00000E+00, -7.32976E+05,  
     #      1.84782E+02, -2.57313E-03,  0.00000E+00 /

C-----------------------------------------------------------------------

      DO L=1,N

        DO I=1,NCOEF

C         calculating polynomial coefficients for corundum (Al2O3):
C             3 H2O - 3 H2 - Al2O3

          DGEFAL(I) = ( Z3*DGEF(IH2O,I) - Z3*DGEF(IH2,I)
     #                    - DGEF(ICRN,I) )
     #                / RCALMOL
        END DO

C       calculating ln(Kp_Al2O3) = delta G /(R*T)
C                                ( polynomial(T) )

        XLKPAL(L) =   DGEFAL(IGA)/TEMP(L)**2    + DGEFAL(IGB)/TEMP(L)
     #              + DGEFAL(IGC)
     #              + DGEFAL(IGD)*TEMP(L)       + DGEFAL(IGE)*TEMP(L)**2
     #              + Z2* LOG(ADYCM2)    ! converting to [dyn/cm2]

      END DO

C-----------------------------------------------------------------------

      RETURN
      END SUBROUTINE

C-----------------------------------------------------------------------

!=======================================================================
      end module dust_Al2O3AGB_module
!=======================================================================
