!***************************************************************************************************
!
!   #####   #    #   ####   #####         #####    #   #    #   ####
!   #    #  #    #  #         #           #    #   #   ##   #  #
!   #    #  #    #   ####     #           #####    #   # #  #   ####
!   #    #  #    #       #    #           #    #   #   #  # #       #
!   #    #  #    #  #    #    #           #    #   #   #   ##  #    #
!   #####    ####    ####     #  #######  #####    #   #    #   #### _module
!
!   Dust-bin scheme
!
!***************************************************************************************************
!   Fortran90
!   2017-12-06
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Lyon, Uppsala
!   France Allard:         Lyon
!   Tristan Guillot:       Lyon
!   Derek Homeier:         Goettingen
!***************************************************************************************************
! MODULES:
!   dust_bins_module:      Dust-bin scheme
!
!***************************************************************************************************


!------****************-----------------------------------------------------------------------------
module dust_bins_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   dust_bins_module ('dust_bins_module')
!
! PURPOSE:
!   Provide routines for dust bin scheme.
!
! CATEGORY:
!   Hydrodynamics, dust formation
!
! CALLING SEQUENCE:
!   use dust_bins_module
!
! VARIABLES:
!   rs:          ('radius_solid') scalar with radius of grains consisting of single monomer
!
! ROUTINES: (all contained routines; public ones are marked with '>')
!   dust_bins_rossow78:    Compute characteristic rates for formation of Mg2SiO4 dust
! > dust_bins_SourceStep:  Update model by applying dust source term step (dust bin scheme).
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!
! CALLING TREE:
!
! MODIFICATION HISTORY:
!   2000-04-18 (T.G., F.A.)
!   2006-11-07 (F.A.) Version of rossow78_mod for CO5BOLD
!   2007-06-05 (B.F.) Fortran 90 version
!   2007-08-13 (B.F.) Formula for saturation pressure from Curtis S. Cooper routine
!   2007-12-15 (B.F.) New value for sig, according to Derek,
!                     R_gas -> k_Bol/amu
!   2008-09-04 (B.F.) alpha_stick_nucl introduced
!---------------------------------------------------------------------------------------------------
use const_module, only: pi_drk, k_Bol_drk, amu_drk
!
implicit none
!
!real*8, parameter                       :: amu_drk  =1.6605402d-24, &
!                                           k_Bol_drk=1.380658d-16, &
!                                           pi_drk   =3.1415926535897932384626433_drk
!
! --- Parameters for dirty forsterite (Mg2SiO4) ---
real*8                                  :: rop, eps, ms, rs, alpha_stick_cond, alpha_stick_coag, &
                                           alpha_stick_coal, alpha_stick_nucl
real*8, parameter                       :: &
                       ! rop   = 3.3d0, &          !mass density of monomer
                       ! stock = 2.d0, &           !for Mg
                       ! stock1= 1.d0, &           !for Si
                       ! stock2= 4.d0, &           !for O
                       ! eps   = 1.2427d-5, &      ! for Mg, old questionable values
                       ! eps1  = 1.1868d-5, &      ! for Si, old questionable values
                       ! eps2  = 1.6764d-4, &      ! for O, old questionable values
                       ! eps   = 3.1187D-05, &     ! for Mg according to phoenix_Rainout-2007.GN.abu
                       ! eps1  = 2.9783D-05, &     !for Si
                       ! eps2  = 6.8230D-04, &     !for O
                       mv    = 24.31d0*amu_drk, &  !for Mg
                       ! mv1   = 28.09d0*amu_drk, &!for Si
                       ! mv2   = 16.d0*amu_drk, &  !for O
                       ! ms    =stock*mv+stock1*mv1+stock2*mv2, & !mass of monomer
                       ! ms    = 140.71d0*amu_drk  !for       forsterite
                       ! rs    =(3.d0*ms/(rop*4.d0*pi_drk))**(1.d0/3.d0),& !radius
                       ! rs    =( 3.d0*(stock*mv+stock1*mv1+stock2*mv2)/(rop*4.d0*pi) )**(1.d0/3.d0)
                       gamma_surf=250.d0, &        ! Surface tension
                       f_CorrFiniteMass=5.d0, &    ! Correction of finite mass
                       a_ps=42.179466259637d00, &  ! Coefficient for saturation pressure formula
                       b_ps=-2.94064111396886d04, &! Coefficient for saturation pressure formula
                       c_ps=-4.52457333892826d07, &! Coefficient for saturation pressure formula
                       d_ps= 2.00362363225697d10   ! Coefficient for saturation pressure formula
!character(len=9), parameter             :: cname='Mg2SiO4/c'
!real*8, parameter, dimension(5)         :: coef=(/ 7.52334e+04, -9.38369e+05, 2.47581e+02, &
!                                                  -3.14980e-03,  0.00000e+00 /)
                                                !radius of single monomer
!character(len=18) cname
!real*8 coef(5)
!data cname,coef/'Mg2SiO4/c', 7.52334e+04, -9.38369e+05, 2.47581e+02, -3.14980e-03, 0.00000e+00/
!
! --- Parameters for gas ---
real*8, parameter                       :: &
                      ! sig=pi_drk*(1.d-8)**2
                      ! sig= pi_drk*(2.d-8)**2    !cfa06: radius of H2 molecule, not HI!
                      sig=4.d0*(pi_drk*1.2d-8**2+2.d0/3.d0*2.d0*0.74d-8*1.2d-8) !cross-section of H2
!
private
!
public dust_bins_SourceStep
!
contains

!----------******************-----------------------------------------------------------------------
subroutine dust_bins_rossow78(grav, p, T, ro, ros, rov, size, &
                              rovs, rnucl, rcond, rcoag, rcoal)
!---------------------------------------------------------------------------------------------------
! NAME:
!   dust_bins_rossow78 ('dust_bins_Rossow_1978')
!
! PURPOSE:
!   Compute characteristic rates for formation of Mg2SiO4 dust.
!
! CATEGORY:
!   HD, source terms, dust formation
!
! CALLING SEQUENCE:
!   call dust_bins_rossow78(grav, p, T, ro, ros, rov, size, &
!                           rovs, rnucl, rcond, rcoag, rcoal)
!
! INPUT:
!   grav:   ('gravity') double precision scalar with gravity [cm/s^2]
!   p:      ('pressure') double precision scalar with gas pressure [dyn/cm^2]
!   T:      ('Temperature') double precision scalar with gas temperature [K]
!   ro:     ('rho') double precision scalar with gas mass density [g/cm^3]
!   ros:    ('rho_solid') double precision scalar with grain mass density [g/cm^3]
!   rov:    ('rho_vapor') double precision scalar with mass density of monomers
!           in the gas phase [g/cm^3]
!   size:   ('size') double precision scalar with grain radius, Rossow: a [cm]
!
! OUTPUT:
!   rovs:   ('rho_vapor_saturation') double precision scalar with saturation density of monomers
!           [g/sm^3]
!   rnucl:  ('rate_nucleation') double precision scalar with nucleation rate [1/s].
!           Proper nucleation rate is:
!             rnucl * (rov-rovs) [g/cm^3/s]
!   rcond:  ('rate_condensation') double precision scalar with condensation rate [cm^3/g/s].
!           A negative values means net evaporation.
!           Proper condensation rate is:
!             rcond * (rov-rovs)*ros [g/cm^3/s]
!   rcoag:  ('rate_coagulation') rate of coagulation [g/cm^3/s]
!   rcoal:  ('rate_coalescence') rate of coalescence [g/cm^3/s]
!
! LOCAL VARIABLES:
!   dgvcrit:    ('delta_Gibbs_volume') barrier for vapor going to critical nucleus [erg]
!   dgv:        ('delta_Gibbs_volume_critical') Change in free energy per unit volume [erg/cm^2]
!   eta:        ('eta') dynamical viscosity rho*nu, was: nu, Rossow: eta, [g/s/cm]
!   gibbs:      ('Gibbs') Gibbs free energy [erg]
!   Kn:         ('Knudsen_number') Knudsen number, lambda_mfp/size, Rossow Eq. (1): Kn [1]
!   lambda_mfp: ('lambda_mean_free_path') gas mean free path, Rossow: lambda_mfp [cm]
!   mcrit:      ('mass_critical') not used: critical mass for a grain to be viable [g]
!   ncrit:      ('number_critical') critical number density of seed molecules [1/cm^3]
!   mu:         ('mu') mean molecular weight of the gas [1]
!   nn:         ('number') number density of monomers in the condensed phase [1/cm^3]
!   omega:      ('omega') flux of gas molecules/atoms onto grains, divided by
!               oversaturation (rov-rovs), [cm/g/s];
!               the original omega=omega * (rov-rovs) [1/cm^2/s]
!   rcrit:      ('radius_critical') critical radius for grains to start nucleating [cm]
!   Re:         ('Reynolds_number') Reynolds number, Rossow Eq. (3): Re [1]
!   rn:         ('rho_number') atmospheric density by number of particles per volume [1/cm^3]
!   T400:       ('Temperature_400') max(T, 400.0) [K]
!   smax:       ('saturation_maximum') not used: supersaturation, Rossow: S_max [1]
!
! PARAMETERS:
!   alpha_stick_cond: ('alpha_sticking_condensation') sticking coefficient for condensation,
!                     molecular sticking efficiency,
!                     condensation coefficient, "collage efficiency", Rossow: alpha [1]
!   alpha_stick_coag: ('alpha_sticking_coagulation') sticking coefficient for coagulation [1]
!   alpha_stick_coal: ('alpha_sticking_coalescence') sticking coefficient for coalescence [1]
!   alpha_stick_nucl: ('alpha_sticking_nucleation') sticking coefficient for nucleation,
!                     implemented as factor to the rates
!   a_ps:             ('a_pressure_saturation') Coefficient for saturation pressure formula,
!                     from Curtis S. Cooper vapor.c, see
!                     http://www.lpl.arizona.edu/~curtis/research/clouds/doxygen/vapor_8c-source.html
!                     00146         a =  42.179466259637;
!                     00147         b = -2.94064111396886e4;
!                     00148         c = -4.52457333892826e7;
!                     00149         d = 2.00362363225697e10;
!                     00151         return exp( a + b/x + c/pow(x,2.0) + d/pow(x,3.0) );
!   b_ps:             ('b_pressure_saturation') Coefficient for saturation pressure formula,
!   c_ps:             ('c_pressure_saturation') Coefficient for saturation pressure formula,
!   d_ps:             ('d_pressure_saturation') Coefficient for saturation pressure formula,
!   eps:              ('epsilon') number fraction of rarest component in gas;
!                     for Mg2SiO4 that is Mg -> eps = eps_Mg/2 = 3.1187D-05 * 0.5,
!                     value according to phoenix_Rainout-2007.GN.abu
!   f_CorrFiniteMass: ('factor_Correction_Finite_Mass') Correction of finite mass,
!                     see Rossow Eq. (14) and text below [1]
!   gamma_surf:       ('gamma_surface') surface tension, surface energy per unit area or flux units
!                     [erg/cm^2]
!   ms:               ('mass_solid') molar mass of a monomer [g]
!   mv:               ('mass_vapor') molar mass of constituting element: Mg [g]
!   mv1:              ('mass_vapor_1') not used: molar mass of constituting element: Si [g]
!   mv2:              ('mass_vapor_2') not used: molar mass of constituting element: O [g]
!   rop:              ('rho_particle') averaged grain material density [g/cm^3]
!   sig:              ('sigma') cross-section of the gas, effectively H_2 [cm^2]
!   stock:            ('Stockiometric') not used: stoichiometric coefficient of element in monomer:
!                     Mg: 2
!   stock1:           ('Stockiometric_1') not used: stoichiometric coefficient of element in
!                     monomer:
!                     Si: 1
!   stock2:           ('Stockiometric_2') not used: stoichiometric coefficient of element in
!                     monomer:
!                     O: 4
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Calculation of the characteristical rates of cloud formation
!   (after Rossow (1978), Icarus 30, 1-50).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-04-18 (T.G., F.A.)
!   2006-11-07 (F.A.) Version of rossow78_mod for CO5BOLD
!   2007-06-05 (B.F.) Fortran 90 version
!   2007-11-13 (B.F.) Debugging
!   2007-11-22 (B.F.) Debugging
!   2007-11-26 (B.F.) Output of rovs, change in definition of rnucl and rconv,
!                     Limit T in formula for P_sat (T400)
!   2007-11-29 (B.F.) Factor ros in rconv removed
!   2007-12-12 (B.F.) Condensation rate test: if (Kn < alpha_stick)  ->  if (Kn < 1.0D+00)
!   2007-12-15 (B.F.) Compute nn from ros and not from rovs
!   2007-12-17 (B.F.) New formula for lambda_mfp and nu; nu->eta; f->f_CorrFiniteMass; reordering
!   2007-12-18 (B.F.) smax removed; more comments;
!                     sticking coefficients for coagulation and coalescence
!   2008-01-08 (B.F.) Case for large Reynolds numbers outcommented
!   2008-09-04 (B.F.) alpha_stick_nucl introduced,
!                     alpha_stick_cond in both branches of formula for condensation rate,
!                     if (Kn < alpha_stick_cond)  ->  if (Kn < 1.0D+00)
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
real*8,                   intent(in)    :: grav, p, T, ro, ros, rov, size
real*8,                   intent(out)   :: rovs, rnucl, rcond, rcoag, rcoal
!
! --- Local variables ---
integer                                 :: i,j
real*8                                  :: eta, mu, rn, lambda_mfp, T400, &
                                           Kn, nn, &
!                                           a70, Re, &
                                           dgv, dgvcrit, rcrit, ncrit, omega, gibbs
!real*8 p0, p1, p2, dkgas, mcrit
!real*8 dklog,t1,su,th,th1,th2,y1,y2
!---------------------------------------------------------------------------------------------------
!
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Some basic quantities describing gas thermodynamics, grains, and momomers   c
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
! --- Saturation pressure according to Curtis S. Cooper vapor.c ---
!rovs=ms/(k_Bol_drk*T)*exp(a_ps + b_ps/T + c_ps/T**2 + d_ps/T**3 )
!rovs=0.5*eps*ms/(k_Bol_drk*T)*exp(a_ps + b_ps/T + c_ps/T**2 + d_ps/T**3 )
! --- Limit temperature to be above 400 K: below the fit grows and then diverges ---
T400=max(T, 400.0D+00)
!rovs=0.5*eps*ms/(k_Bol_drk*T)*exp(a_ps + b_ps/T400 + c_ps/T400**2 + d_ps/T400**3 )
rovs=eps*ms/(k_Bol_drk*T)*exp(a_ps + b_ps/T400 + c_ps/T400**2 + d_ps/T400**3 )
!
! --- Mean molecular weight of the gas ---
mu=ro*(k_Bol_drk*T)/(p*amu_drk)
!
! --- Number density of monomers in the condensed phase ---
!nn=rovs/((4.d0/3.d0*pi_drk)*size**3*rop)
nn=ros/((4.d0/3.d0*pi_drk)*size**3*rop)
!
! --- Atmospheric density by number of particles per volume ---
rn=ro/(mu*amu_drk)
! --- Mean free path of the gas ---
!lambda_mfp=sqrt(3.d0*pi_drk)/(8.d0*rn*sig)
!lambda_mfp=(sqrt(3.d0*pi_drk)/8.d0)/(rn*sig)
lambda_mfp=(1.d0/sqrt(2.d0))/(rn*sig)
!
! --- Dynamical viscosity ---
! eta=ro/sqrt(t)
!fa06 une Bernd's viscosity!
!eta=ro*lambda_mfp*sqrt((8.d0/pi_drk)*(k_Bol_drk*T)/(mu*amu_drk))
!eta = sqrt(pi*kbol*t*mu*amu) * 5.d0/(16.d0*sig)
! --- Reif ("Statistical and Thermal Physics") 14.8.33 ---
eta=(sqrt(pi_drk)*5.d0/16.d0) *sqrt((k_Bol_drk*T)*(mu*amu_drk))/sig
!
! --- Knudsen number ---
Kn=lambda_mfp/size
!
! --- Reynolds number ---
! --- a70: size of particles for which Re=70                           ---
! --- Rossow Eq. (20)                                                  ---
! --- Corrected according to Carlson et al. 1988 (1/3 instead of 1/2?) ---
!a70=(9.d0*270.d0*eta*eta/(4.d0*rop*ro*grav))**(1.d0/3.d0)
!a70=((9.d0*270.d0/4.d0)*eta*eta/(rop*ro*grav))**(1.d0/3.d0)
!Re =70.d0*size/a70
! --- D.H.'s new formulation: ---
! --- Re=4*ro*rop*grav*size**3/(9*eta**2) ---
! --- if (Re > 18.0) Re=sqrt(18*4*ro*rop*grav*size**3/(9*eta**2)) ---
!Re=(4.d0/9.d0)*(ro*rop*grav*size**3/(eta**2))
!if (Re > 18.0) Re=sqrt((18.d0*4.d0/9.d0)*(ro*rop*grav*size**3/(eta**2)))
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Nucleation rate:
!     -Supersaturation [smax] (Rossow p.2,3)       	               c
!         s>0: growth ; s<0: evaporation                               c
!     -change in free energy per unit volume  (erg/cm**2) [dgv]        c
!     -Gibbs free energy of formation of the grain (pressure) [dklog]  c
!        Keq/kgas < 1 for grains to form ==> dgv & dgvcrit < 0         c
!     -barrier for vapor going to critical nucleus (erg) [dgvcrit]     c
!     -critical number density of seed molecules (cm^-3)   [ncrit]     c
!     -critical radius for grains to start nucleating (cm)   [rcrit]   c
!     -flux of gas molecules/atoms onto grains (cm g^-1 sec^-1) [omega]c
!     -rate of nucleation [rnucl]                                      c 
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
if ((rov/rovs) > 1.d0) then
  ! --- Positive supersaturation -> there is nucleation ---
  !
  !smax = rov/rovs-1.d0
  !out  gibbs=k_Bol_drk*t*log(rov/rovs)    !according to lecture
  !     gibbs=k_Bol_drk*t*(dklog-dkgas)    !according to derivation
  !gibbs  = (k_Bol_drk*T)*log(1.d0+smax)   !according to Arizona Cloud code
  gibbs  =(k_Bol_drk*T)*log(rov/rovs)
  dgv    =gibbs/((4.d0/3.d0*pi_drk)*rs**3) 
  dgvcrit=(16.d0/3.d0*pi_drk)*gamma_surf**3/dgv**2
  ncrit  =(rov/ms)*exp(-dgvcrit/(k_Bol_drk*T))   
  rcrit  =2.d0*gamma_surf/dgv
  ! rcrit  =-2.d0*gamma_surf/dgv             ! wrong sign
  ! mcrit  =rop*(4.d0*pi_drk*rcrit**3/3.d0)  ! not needed
  ! omega  =alpha_stick_cond*(rov-rovs)/ms*sqrt(R_gas_drk*t/(2.d0*pi_drk*mv))
  ! omega  =alpha_stick_cond/ms*sqrt(R_gas_drk*t/(2.d0*pi_drk*mv)) ! Factor (rov-rovs) removed
  ! omega  =alpha_stick_cond/ms*sqrt((k_Bol_drk*T)/((2.d0*pi_drk)*mv*amu_drk))
  omega  =alpha_stick_nucl/ms*sqrt((k_Bol_drk*T)/((2.d0*pi_drk)*mv*amu_drk))
  !
  ! --- Nucleation rate = dros/dt/(rov-rovs) ---
  !rnucl=max((4.d0*pi_drk*ms)*ncrit*rcrit**2 *omega, 0.0D+00)
  rnucl=(4.d0*pi_drk)*ms*ncrit*rcrit**2 *omega
else
  rnucl=0.0D+00
endif
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Rates of condensation, coagulation, coalescence                      c 
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! --- Condensation rate = dros/dt/(rov-rovs) ---
!if (Kn < alpha_stick_cond) then
if (Kn < 1.0D+00) then
  ! --- Laminar viscous flow (Stokes), Rossow Eq. (16) ---
  !rcond=ros*(2.d0*f*eta*rovs*smax/rop)/(ro*size**2)
  !rcond=ros*(2.d0*f*eta*(rov-rovs)/rop)/(ro*size**2)
  !rcond=ros*(2.d0*f*eta/rop)/(ro*size**2)
  !rcond=(2.d0*f_CorrFiniteMass*eta/rop)/(ro*size**2)
  rcond=(2.d0*alpha_stick_cond*f_CorrFiniteMass*eta/rop)/(ro*size**2)
!  if (Re > 1.d0) then
!    ! --- Turbulent flow (Newton), apply ventilation factor, Rossow p. 10 bottom right ---
!    rcond=rcond*(1.d0+2.d-1*sqrt(Re))
!  endif
else
  ! --- Subsonic free molecular flow, Rossow Eq. (17) ---
  !rcond=ros*sqrt(2.d0*k_Bol_drk*t/(pi_drk*amu_drk*mu))*(3.d0*alpha_stick_cond*f_CorrFiniteMass*rovs*smax/(2.d0*rop))/size
  !rcond=ros*sqrt(2.d0*k_Bol_drk*t/(pi_drk*amu_drk*mu))*(3.d0*alpha_stick_cond*f_CorrFiniteMass*(rov-rovs)/(2.d0*rop))/size
  !rcond=ros*sqrt(2.d0*k_Bol_drk*t/(pi_drk*amu_drk*mu))*(3.d0*alpha_stick_cond*f_CorrFiniteMass/(2.d0*rop))/size
  !rcond=sqrt(2.d0*(k_Bol_drk*T)/(pi_drk*amu_drk*mu))* &
  !      (3.d0*alpha_stick_cond*f_CorrFiniteMass/(2.d0*rop))/size
  rcond=sqrt((2.d0/pi_drk)*(k_Bol_drk*T)/(amu_drk*mu))* &
        ((3.d0/2.d0)*alpha_stick_cond*f_CorrFiniteMass/rop)/size
endif
!
! --- Coagulation rate = dros/dt ---
if (Kn < 1.d0) then
  ! --- Laminar viscous flow (Stokes); Rossow Eq. (32) ---
  !rcoag=ros*(4.d0*(k_Bol_drk*T)/(3.d0*eta))*nn
  rcoag=alpha_stick_coag*ros*((4.d0/3.d0)*(k_Bol_drk*T)/eta)*nn
else
  ! --- Subsonic free molecular flow; Rossow Eq. (33) ---
  rcoag=alpha_stick_coag*ros*4.d0*sqrt(3.d0*size*(k_Bol_drk*T)/rop)*nn
endif
!
! --- Coalescence rate = dros/dt ---
if (Kn > 1.d0) then
  ! --- Subsonic free molecular flow; Rossow Eq. (37) ---
  !rcoal=ros*(4.d0*pi_drk*rop*grav/(27.d0*ro))* &
  !      sqrt(pi_drk*amu_drk*mu/(2.d0*(k_Bol_drk*T)))*size**3*nn
  rcoal=alpha_stick_coal*ros*((4.d0*pi_drk/27.d0)*rop*grav/ro)* &
        sqrt((pi_drk/2.d0)*(amu_drk*mu)/(k_Bol_drk*T))*size**3*nn
!elseif (Re > 70.d0) then
!  ! --- Turbulent flow (Newton); Rossow Eq. (36) ---
!  !rcoal=ros*sqrt(10.d0*pi_drk*pi_drk*rop*grav*size**5/(3.d0*ro))*nn
!  rcoal=alpha_stick_coal*ros*sqrt((10.d0/3.d0*pi_drk*pi_drk)*rop*grav*size**5/ro)*nn
else
  ! --- Laminar viscous flow (Stokes); Rossow Eq. (35) ---
  !rcoal=ros*(pi_drk*rop*grav/(9.d0*eta))*size**4*nn
  rcoal=alpha_stick_coal*ros*((pi_drk/9.d0)*rop*grav/eta)*size**4*nn
endif
!
!write(*,'(1p,10(1PD11.3))') t,p,size,rcrit,Kn,Re,rnucl,rcond,rcoag,rcoal
!
end subroutine dust_bins_rossow78


!----------********************---------------------------------------------------------------------
subroutine dust_bins_SourceStep_old(model, action, dtime, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   dust_bins_SourceStep ('dust_bins_Source_Step')
!
! PURPOSE:
!   Update model by applying dust source term step (dust bin scheme).
!   
! CATEGORY:
!   Dust, Hydrodynamics
!
! CALLING SEQUENCE:
!   call dust_bins_SourceStep(model, action, dtime, outstr=outstr,ierr=ierr)
!
! INPUT:
!   action: ('action') derived type with collection of control parameters
!   dtime:  ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:  ('model') derived type, contains all model data (e.g. model%rho)
!     m1:        ('m_number_1') integer, 1. dimension, start index
!     n1:        ('number_1')   integer, 1. dimension, end index
!     m2:        ('m_number_2') integer, 2. dimension, start index
!     n2:        ('number_2')   integer, 2. dimension, end index
!     m3:        ('m_number_3') integer, 3. dimension, start index
!     n3:        ('number_3')   integer, 3. dimension, end index
!     nghost1:   ('number_ghost_1') integer, 1. dimension, number of ghost cells
!     nghost2:   ('number_ghost_2') integer, 2. dimension, number of ghost cells
!     nghost3:   ('number_ghost_3') integer, 3. dimension, number of ghost cells
!     itime:     ('index_time') integer, time step number
!     nquc:      ('number_quantities_center') integer, number of additional fields
!     time:      ('time') real, time
!     xc1:       ('x_center_1') real, 1. coordinate of cell centers
!     xc2:       ('x_center_2') real, 2. coordinate of cell centers
!     xc3:       ('x_center_3') real, 3. coordinate of cell centers
!     xb1:       ('x_boundary_1') real, 1. coordinate of cell boundaries
!     xb2:       ('x_boundary_2') real, 2. coordinate of cell boundaries
!     xb3:       ('x_boundary_3') real, 3. coordinate of cell boundaries
!     rho:       ('rho') real, dimension(:,:,:), density
!     ei:        ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!     v1:        ('v_1') real, dimension(:,:,:), velocity, 1. coordinate
!     v2:        ('v_2') real, dimension(:,:,:), velocity, 2. coordinate
!     v3:        ('v_3') real, dimension(:,:,:), velocity, 3. coordinate
!     quc:       ('quantity_center') real, dimension(:,:,:,:), cell centered set of quantities
!                                     e.g. quc(i1,i2,i3,iquc)
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('index_error') integer, optional error number:
!           =0: normal run
!           1-99:  "small" error (e.g. time step limit violation)
!           >=100: "severe" error:
!
! LOCAL VARIABLES:
!   m1:          ('m_1') integer, lower index, 1st dimension
!   n1:          ('n_1') integer, upper index, 1st dimension
!   m2:          ('m_2') integer, lower index, 2nd dimension
!   n2:          ('n_2') integer, upper index, 2nd dimension
!   m3:          ('m_3') integer, lower index, 3rd dimension
!   n3:          ('n_3') integer, upper index, 3rd dimension
!   nquc:        ('number_quantities_center') integer, number of additional fields
!   iquc_monom:  ('integer_quantity_center_monomer') index of monomer bin
!   iquc_dustm:  ('integer_quantity_center_dust_m') index of first dust bin,
!                it is assumed that iquc_dustm=iquc_monom+1: the dust bins follow the monomer bin
!   iquc_dustn:  ('integer_quantity_center_dust_n') index of last dust bin
!
! ROUTINES:
!   eosinter_PT:           Solve EOS by bicubic interpolation: rho, e -> P, T
!   dust_bins_rossow78:    Compute characteristic rates for formation of Mg2SiO4 dust
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   There is no time step control.
!   Gravity is assumed to be constant.
!
! PROCEDURE:
!   Control parameters:
!     eps_AlmostNoDust:         From par%C_dust04, typically 1.0E-09 or so,
!                               If the density in a bin relative to the gas density falls below this
!                               value, the bin is completely emptied and the material is added to
!                               the bin with the largest dust or monomer density,
!     T_limit_AbsolutelyNoDust: Parameter: 2500.0,
!                               If all temperature values in a 1D vector are above this limit, the
!                               possibly remaining dust is removed and the material is put into the
!                               monomer bin. There is no detailed treatment of dust.
!                               In the other vectors, the temperature is set to
!                                 T=min(T,T_limit_AbsolutelyNoDust).
!                               No coagulation of coalescence is considered above this value.
!     dtimefac_reserve:         Parameter: 1.5
!                               The total change rate in each bin is limited to
!                                 1/dtimefac_reserve
!                               E.g. a value of 1.5=3/2 means that the rate is limited to 2/3
!                               of the total contents of the bin: at least 1/3 remains.
!
!   Binning scheme (condensation):
!     f(iquc+1)/f(iquc) = 1/r3      (larger grains have larger flux according to volume ratio)
!     f(iquc+1)-f(iquc) = rcond     (the condensation rate -- flux of monomers -- is the difference)
!    =>
!     f(iquc  ) = rcond * r3/(1-r3)
!     f(iquc+1) = rcond *  1/(1-r3)
!
! TO DO:
!   Test, verify, ...
!
! MODIFICATION HISTORY:
!   2007-06-05 (B.F. Lyon) Writing started
!   2007-06-07 (B.F.) First complete version
!   2007-11-13 (B.F.) Debugging, rcoag included
!   2007-11-22 (B.F.) rcoal included
!   2007-11-26 (B.F.) Change integration scheme to account of small nucl./cond. time-scales
!   2007-11-29 (B.F.) Symmetrisation (centering) of condensation rate
!   2007-11-30 (B.F.) Superbee slopes
!   2007-12-02 (B.F.) Skip detailed dust treatment for vectors with too large temperatures,
!                     Abbreviations for dust bin indices
!   2007-12-18 (B.F.) Re-arrangement and addition of parameters;
!                     factor 2.0*r3_ratio(iquc) in coagulation rate
!   2008-09-05 (B.F.) High temperatures: actually change densities, not only rates
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use const_module, only: pi_drk, amu_drk
!
!# ifdef rhd_dust_t01
!!
!use timing_module
!!
!# endif
!
implicit none
!
! --- I/O parameters ---
type(box_type),           intent(inout) :: model
type(action_type),        intent(in)    :: action
real,                     intent(in)    :: dtime
integer,        optional, intent(out)   :: ierr
character(len=*),optional,intent(out)   :: outstr
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=kind(0.0D+00)
real, parameter                         :: T_limit_AbsolutelyNoDust=2500.0, &
                                           dtimefac_reserve=1.5
integer                                 :: m1,n1, m2,n2, m3,n3, nquc, i1, i2, i3, iquc, &
                                           iquc_monom, iquc_dustm, iquc_dustn
real                                    :: eps_AlmostNoDust, rcondeff
integer, dimension(model%m1:model%n1)   :: iqucmax
real, dimension(model%m1:model%n1)      :: rho, P, T, rhovs, dtimefac1, dtimefac
real, dimension(model%m1:model%n1, &
                1       :model%nquc)    :: quc, qucslope, rnucl, rcond, rcoag, rfast, rtot
real, dimension(1       :model%nquc-1)  :: r3_ratio
real(kind=drk)                          :: gravdb, rhovsdb, rnucldb, rconddb, rcoagdb, rcoaldb
real(kind=drk), &
      dimension(model%m1:model%n1)      :: rhodb, Pdb, Tdb
real(kind=drk), &
      dimension(model%m1:model%n1, &
                1       :model%nquc)    :: qucdb
real(kind=drk), &
     dimension(1:par%n_dustGrainRadius) :: dustGrainRadiusdb
integer                                 :: ierr0
character(len=80)                       :: outstr0
!character(len=2)                        :: sepstr
!---------------------------------------------------------------------------------------------------
!
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
nquc=model%nquc
iquc_monom=1
iquc_dustm=2
iquc_dustn=nquc
!
! --- Input parameters: take basic parameters from parameter file ---
rop             =par%C_dust01
ms              =par%C_dust02 * amu_drk
eps             =par%C_dust03
eps_AlmostNoDust=par%C_dust04
! Not used:      par%C_dust05
alpha_stick_cond=par%C_dust06
alpha_stick_coag=par%C_dust07
alpha_stick_coal=par%C_dust08
alpha_stick_nucl=par%C_dust09
!
rs              =(3.d0*ms/(rop*4.d0*pi_drk))**(1.d0/3.d0)
!
! --- Grain radius ratios (actually volume ratios) ---
dustGrainRadiusdb=real(par%ar_dustGrainRadius, kind=drk)
r3_ratio(iquc_monom)=(rs/par%ar_dustGrainRadius(iquc_dustm))**3
do iquc=iquc_dustm,iquc_dustn-1
  r3_ratio(iquc)=(par%ar_dustGrainRadius(iquc)/par%ar_dustGrainRadius(iquc+1))**3
end do ! iquc
!
!write(6,*) 'rs=', rs
!write(6,*) 'eps_AlmostNoDust=', eps_AlmostNoDust
!do iquc=1,nquc-1
!  write(6,*) 'r3_ratio(', iquc ,')=', r3_ratio(iquc)
!end do ! iquc

! --- Dust formation: update ---
!$OMP PARALLEL DEFAULT(SHARED), &
!$OMP   PRIVATE(i1, i2, rho, quc, P, T, Pdb, Tdb, rhodb, qucdb, gravdb, iqucmax, &
!$OMP           rhovs, rnucl, rcond, rcoag, rfast, rtot, rcondeff, &
!$OMP           qucslope, rhovsdb, rnucldb, rconddb, rcoagdb, rcoaldb, dtimefac1, dtimefac)
!
!$OMP DO
do i3=m3,n3
  do i2=m2,n2
   !
   ! --- Make thread-local copies ---
   rho=model%rho(m1:n1,i2,i3)
   quc=model%quc(m1:n1,i2,i3,1:nquc)
   !
   ! --- Solve EOS ---
   call eosinter_PT(rho, model%ei(m1:n1,i2,i3), P=P, T=T)
   !
   if (minval(T) < T_limit_AbsolutelyNoDust) then
    ! === Temperatures low enough to form dust (possibly) somewhere: treat it in detail ===
    !
    ! --- Limit temperature to reasonable values ---
    T=min(T, T_limit_AbsolutelyNoDust)
    !
    ! --- Transform to double precision ---
    Pdb  =real(P  , kind=drk)
    Tdb  =real(T  , kind=drk)
    rhodb=real(rho, kind=drk)
    qucdb=real(quc, kind=drk)
    !
    ! --- Assume constant gravity ---
    gravdb=real(par%grav, kind=drk)
    !
    ! --- Bin index with maximum dust/monomer density ---
    iqucmax=maxloc(quc,2)
    !
    ! --- Slope of linear dust distribution within each bin ---
    qucslope=0.0
    if (nquc > 2) then
      ! --- First and last bin: slope between neighbor cells ---
      qucslope(:,iquc_dustm)=quc(:,iquc_dustm+1)-quc(:,iquc_dustm  )
      qucslope(:,iquc_dustn)=quc(:,iquc_dustn  )-quc(:,iquc_dustn-1)
      ! --- Interior bins: Superbee ---
      do iquc=iquc_dustm+1,iquc_dustn-1
        do i1=m1,n1
          qucslope(i1,iquc)=(     sign(1.0,quc(i1,iquc  ) - quc(i1,iquc-1)) + &
                                  sign(1.0,quc(i1,iquc+1) - quc(i1,iquc  )) &
                            ) * &
                            min( abs(      quc(i1,iquc  ) - quc(i1,iquc-1)  ), &
                                 abs(      quc(i1,iquc+1) - quc(i1,iquc  )  ), &
                                 max(  abs(quc(i1,iquc  ) - quc(i1,iquc-1)), &
                                       abs(quc(i1,iquc+1) - quc(i1,iquc  )) &
                                    )*0.5 &
                               )
        end do ! i1
      end do ! iquc
    endif
    !
    ! --- Initialize total rate ---
    rfast=0.0
    rtot =0.0
    rnucl=0.0
    rcond=0.0
    rcoag=0.0
    !
    ! --- Call Rossow routine to get nucleation and condensation rate ---
    ! --- Add rates to get total rate of change ---
    do iquc=iquc_dustm,iquc_dustn
      do i1=m1,n1
        if (quc(i1,iqucmax(i1)) > 0.0) then
          ! --- Call Rossow routine ---
          call dust_bins_rossow78(gravdb, Pdb(i1), Tdb(i1), &
                                  rhodb(i1), qucdb(i1,iquc), qucdb(i1,iquc_monom), &
                                  dustGrainRadiusdb(iquc), &
                                  rhovsdb, rnucldb, rconddb, rcoagdb, rcoaldb)
          !
          ! --- Transform to single precision ---
          rcond(i1,iquc)=real(rconddb        , kind=srk)
          rcoag(i1,iquc)=real(rcoagdb+rcoaldb, kind=srk)
          if (T(i1) >= T_limit_AbsolutelyNoDust) rcoag(i1,iquc)=0.0
          rhovs(i1)     =real(rhovsdb        , kind=srk)
          !
          ! --- Add nucleation rate to rate of first dust bin and to total fast rate ---
          if (iquc == iquc_dustm) then
            rnucl(i1,iquc)=real(rnucldb, kind=srk)
            rfast(i1,iquc)=rfast(i1,iquc)+rnucl(i1,iquc)
          endif
          !
          ! --- Add evaporation/condensation rate to total fast rate ---
          if (iquc > iquc_dustm) then
            ! --- Rate between two dust bins ---
            ! --- Find effective condensation rate from "upwind" bin density ---
            if (quc(i1,iquc_monom) < rhovs(i1)) then
              ! --- Evaporation ---
              rcondeff=0.5 * (rcond(i1,iquc-1)+rcond(i1,iquc)) * &
                       (quc(i1,iquc  )-0.5*qucslope(i1,iquc  ))
            else
              ! --- Condensation ---
              rcondeff=0.5 * (rcond(i1,iquc-1)+rcond(i1,iquc)) * &
                       (quc(i1,iquc-1)+0.5*qucslope(i1,iquc-1))
            endif
            rfast(i1,iquc  )=rfast(i1,iquc  ) + rcondeff                  /(1.0-r3_ratio(iquc-1))
            rfast(i1,iquc-1)=rfast(i1,iquc-1) - rcondeff*(r3_ratio(iquc-1)/(1.0-r3_ratio(iquc-1)))
            !
          else
            ! --- Rate between monomer bin and lowest radius bin (no condensation) ---
            if (quc(i1,iquc_monom) < rhovs(i1)) then
              ! --- Evaporation ---
              rcondeff=rcond(i1,iquc) * quc(i1,iquc)
              rfast(i1,iquc)=rfast(i1,iquc)   + rcondeff/(1.0-r3_ratio(iquc-1))
            endif
            !
          endif
          !
          ! --- Add coagulation rate to total rate (dust in largest bin does not coagulate) ---
          if (iquc < iquc_dustn) then
            rtot(i1,iquc  )=rtot(i1,iquc  )-2.0*r3_ratio(iquc)*rcoag(i1,iquc)
            rtot(i1,iquc+1)=rtot(i1,iquc+1)+2.0*r3_ratio(iquc)*rcoag(i1,iquc)
          endif
          !
        endif
      end do ! i1
    end do ! iquc
    !
    ! --- Monomer change rate is negative of sum of grain change rate, fast processes only ---
    rfast(:,iquc_monom)=-sum(rfast(:,iquc_dustm:iquc_dustn),2)
    !
    ! --- Compute rate reduction factor due to rapid nucleation/condensation ---
    do i1=m1,n1
      if (abs(dtime*rfast(i1,1)) < 1.0E-02) then
        dtimefac1(i1)=1.0
      else
        dtimefac1(i1)=(exp(dtime*rfast(i1,1))-1.0)/(dtime*rfast(i1,1))
      endif
    end do ! i1
    !
    ! --- Update total rate, account for rate reduction factor ---
    do iquc=iquc_dustm,iquc_dustn
      do i1=m1,n1
        rtot(i1,iquc)=rtot(i1,iquc) + rfast(i1,iquc)*dtimefac1(i1)*(quc(i1,iquc_monom)-rhovs(i1))
      end do ! i1
    end do ! iquc
    !
    ! --- Monomer change rate is negative of sum of grain change rate ---
    rtot(:,iquc_monom)=-sum(rtot(:,iquc_dustm:iquc_dustn),2)
    !
    ! --- Detect bins with too small residual density ---
    do iquc=iquc_monom,iquc_dustn
      do i1=m1,n1
        if (model%quc(i1,i2,i3,iquc) <= eps_AlmostNoDust*rho(i1)) then
          if ((iquc /= iqucmax(i1)) .and. ((dtime*rtot(i1,iquc)) < -model%quc(i1,i2,i3,iquc))) then
            ! --- Almost no dust left in bin and rate is negative: remove everything from bin ---
            model%quc(i1,i2,i3,iqucmax(i1))=model%quc(i1,i2,i3,iqucmax(i1))+model%quc(i1,i2,i3,iquc)
            model%quc(i1,i2,i3,iquc       )=0.0
            rtot(i1,iqucmax(i1))=rtot(i1,iqucmax(i1))+rtot(i1,iquc)
            rtot(i1,iquc       )=0.0
          endif
          ! --- Increase density by arbitrary large factor: only used in rtot/quc ratio ---
          quc( i1,iquc       )=rho(i1)*1.0E+08
        endif
      end do ! i1
    end do ! iquc
    !
    ! --- Find proper dtime factor (redefine previous value) ---
    dtimefac=1.0/max(dtimefac_reserve*maxval(-rtot/quc,2), 1.0/dtime)
    !
    ! --- Apply changes ---
    do iquc=iquc_monom,iquc_dustn
      do i1=m1,n1

!if ((i1 == 1) .and. (i3 > -1)) then
!  if (iquc == 1) then
!    sepstr='--'
!  else
!    sepstr='  '
!  endif
!  write(6,'(4I4,A,6(1PE11.3),A,4(1PE11.3),A,1PE11.3)') i1,i2,i3,iquc, sepstr, Tdb(i1), Pdb(i1), rhodb(i1), quc(i1,iquc), model%quc(i1,i2,i3,iquc), rhovs(i1), sepstr, rnucl(i1,iquc), rcond(i1,iquc), rcoag(i1,iquc), rtot(i1,iquc), sepstr, dtimefac(i1)/dtime
!endif

        model%quc(i1,i2,i3,iquc)=model%quc(i1,i2,i3,iquc)+dtimefac(i1)*rtot(i1,iquc)
      end do ! i1
    end do ! iquc
    !
   else
    ! === Temperatures too high: skip detailed treatment of dust ===
    !
    ! --- Destroy all dust grains and put material into monomer bin ---
!    rtot(:,iquc_monom           )=rtot(:,iquc_monom)+sum(rtot(:,iquc_dustm:iquc_dustn),2)
!    rtot(:,iquc_dustm:iquc_dustn)=0.0
    !
    model%quc(:,i2,i3,iquc_monom           )=model%quc(:,i2,i3,iquc_monom) + &
                                             sum(model%quc(:,i2,i3,iquc_dustm:iquc_dustn),2)
    model%quc(:,i2,i3,iquc_dustm:iquc_dustn)=0.0
    !
   endif
   !
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
end subroutine dust_bins_SourceStep_old


!----------********************---------------------------------------------------------------------
subroutine dust_bins_SourceStep(model, action, dtime, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   dust_bins_SourceStep ('dust_bins_Source_Step')
!
! PURPOSE:
!   Update model by applying dust source term step (dust bin scheme).
!   
! CATEGORY:
!   Dust, Hydrodynamics
!
! CALLING SEQUENCE:
!   call dust_bins_SourceStep(model, action, dtime, outstr=outstr,ierr=ierr)
!
! INPUT:
!   action: ('action') derived type with collection of control parameters
!   dtime:  ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:  ('model') derived type, contains all model data (e.g. model%rho)
!     m1:        ('m_number_1') integer, 1. dimension, start index
!     n1:        ('number_1')   integer, 1. dimension, end index
!     m2:        ('m_number_2') integer, 2. dimension, start index
!     n2:        ('number_2')   integer, 2. dimension, end index
!     m3:        ('m_number_3') integer, 3. dimension, start index
!     n3:        ('number_3')   integer, 3. dimension, end index
!     nghost1:   ('number_ghost_1') integer, 1. dimension, number of ghost cells
!     nghost2:   ('number_ghost_2') integer, 2. dimension, number of ghost cells
!     nghost3:   ('number_ghost_3') integer, 3. dimension, number of ghost cells
!     itime:     ('index_time') integer, time step number
!     nquc:      ('number_quantities_center') integer, number of additional fields
!     time:      ('time') real, time
!     xc1:       ('x_center_1') real, 1. coordinate of cell centers
!     xc2:       ('x_center_2') real, 2. coordinate of cell centers
!     xc3:       ('x_center_3') real, 3. coordinate of cell centers
!     xb1:       ('x_boundary_1') real, 1. coordinate of cell boundaries
!     xb2:       ('x_boundary_2') real, 2. coordinate of cell boundaries
!     xb3:       ('x_boundary_3') real, 3. coordinate of cell boundaries
!     rho:       ('rho') real, dimension(:,:,:), density
!     ei:        ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!     v1:        ('v_1') real, dimension(:,:,:), velocity, 1. coordinate
!     v2:        ('v_2') real, dimension(:,:,:), velocity, 2. coordinate
!     v3:        ('v_3') real, dimension(:,:,:), velocity, 3. coordinate
!     quc:       ('quantity_center') real, dimension(:,:,:,:), cell centered set of quantities
!                                     e.g. quc(i1,i2,i3,iquc)
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('index_error') integer, optional error number:
!           =0: normal run
!           1-99:  "small" error (e.g. time step limit violation)
!           >=100: "severe" error:
!
! LOCAL VARIABLES:
!   srk:         ('single_real_kind) integer with kind of single real
!   drk:         ('double_real_kind) integer with kind of double real
!   m1:          ('m_1') integer, lower index, 1st dimension
!   n1:          ('n_1') integer, upper index, 1st dimension
!   m2:          ('m_2') integer, lower index, 2nd dimension
!   n2:          ('n_2') integer, upper index, 2nd dimension
!   m3:          ('m_3') integer, lower index, 3rd dimension
!   n3:          ('n_3') integer, upper index, 3rd dimension
!   nquc:        ('number_quantities_center') integer, number of additional fields
!   i1:          ('integer_1') integer, running index, 1st dimension
!   i2:          ('integer_2') integer, running index, 2nd dimension
!   i3:          ('integer_3') integer, running index, 3rd dimension
!   iquc:        ('integer_quantity_centered') integer, running index, "dust type index"
!   iquc_monom:  ('integer_quantity_center_monomer') index of monomer bin
!   iquc_dustm:  ('integer_quantity_center_dust_m') index of first dust bin,
!                  it is assumed that iquc_dustm=iquc_monom+1: the dust bins follow the monomer bin
!   iquc_dustn:  ('integer_quantity_center_dust_n') index of last dust bin
!   rcondeff:    ('rate_condensation_effective') temporary real scalar for condensation rate
!                  "centered" at boundary between adjacent bins
!   dquc:        ('delta_quantity_centered') temporary real scalar for change rate of dust density
!   iqucmax:     ('integer_quantity_centered_maximum') integer vector with index of bin with
!                  maximum density
!   rho:         ('rho') real vector with gas density
!   P:           ('Pressure') real vector with gas pressure
!   T:           ('Tempearure') real vector with gas temperature
!   rhovs:       ('rho_vapor_saturation') real vector with monomer saturation density
!   dtimefac:    ('delta_time_factor') real vector with time step, reduced by factor due to
!                  limited availability of monomers
!   quc:         ('quantity_center') real vector with cell centered set of quantities
!                  quc(:,iquc_monom): monomers
!                  quc(:,iquc_dustm): first dust bin with smallest grains
!                  quc(:,iquc_dustn): last dust bin with largest grains
!   qucslope:    ('quantity_center_slope') real vector with slopes of quc's, made with SuperBee
!   rnucl:       ('rate_nucleation') real vector with nucleation rate [1/s]
!   rcond:       ('rate_condensation') real vector with condensation rate [cm^3/g/s]
!   rcoag:       ('rate_coagulation') real vector with rate of coagulation + coalescence [g/cm^3/s]
!   rfast:       ('rate_fast') real vector with change rate per bin due to fast processes such as
!                   nucleation and condensation
!   r3_ratio:    ('radius_3_ratio') real vector with ratio of volumes of grains in bin to
!                  grains in next bin; computed from input array with grain sizes
!   gravdb:      ('gravity_double') double real scalar with gravity
!   rhovsdb:     ('rho_vapor_saturation_double') double vector with monomer saturation density
!   rnucldb:     ('rate_nucleation_double') double vector with nucleation rate [1/s]
!   rconddb:     ('rate_condensation_double') double vector with condensation rate [cm^3/g/s]
!   rcoagdb:     ('rate_coagulation_double') double vector with rate of coagulation [g/cm^3/s]
!   rcoaldb:     ('rate_coalescence_double') double vector with rate of coalescence [g/cm^3/s]
!
! ROUTINES:
!   eosinter_PT:           Solve EOS by bicubic interpolation: rho, e -> P, T
!   dust_bins_rossow78:    Compute characteristic rates for formation of Mg2SiO4 dust
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   There is no time step control.
!   Gravity is assumed to be constant.
!
! PROCEDURE:
!   Control parameters:
!     T_limit_AbsolutelyNoDust: Parameter: 2500.0,
!                               If all temperature values in a 1D vector are above this limit, the
!                               possibly remaining dust is removed and the material is put into the
!                               monomer bin. There is no detailed treatment of dust.
!                               In the other vectors, the temperature is set to
!                                 T=min(T,T_limit_AbsolutelyNoDust).
!                               No coagulation of coalescence is considered above this value.
!
!   Binning scheme (condensation):
!     f(iquc+1)/f(iquc) = 1/r3      (larger grains have larger flux according to volume ratio)
!     f(iquc+1)-f(iquc) = rcond     (the condensation rate -- flux of monomers -- is the difference)
!    =>
!     f(iquc  ) = rcond * r3/(1-r3)
!     f(iquc+1) = rcond *  1/(1-r3)
!
! TO DO:
!   Test, verify, ...
!
! MODIFICATION HISTORY:
!   2007-06-05 (B.F. Lyon) Writing started
!   2007-06-07 (B.F.) First complete version
!   2007-11-13 (B.F.) Debugging, rcoag included
!   2007-11-22 (B.F.) rcoal included
!   2007-11-26 (B.F.) Change integration scheme to account of small nucl./cond. time-scales
!   2007-11-29 (B.F.) Symmetrisation (centering) of condensation rate
!   2007-11-30 (B.F.) Superbee slopes
!   2007-12-02 (B.F.) Skip detailed dust treatment for vectors with too large temperatures,
!                     Abbreviations for dust bin indices
!   2007-12-18 (B.F.) Re-arrangement and addition of parameters;
!                     factor 2.0*r3_ratio(iquc) in coagulation rate
!   2008-09-05 (B.F.) High temperatures: actually change densities, not only rates
!   2008-10-15 (B.F.) First lines of new version
!   2008-10-16 (B.F.) New version
!   2008-10-20 (B.F.) Error handling (thread save)
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use const_module, only: pi_drk, amu_drk
!
!# ifdef rhd_dust_t01
!!
!use timing_module
!!
!# endif
!
implicit none
!
! --- I/O parameters ---
type(box_type),           intent(inout) :: model
type(action_type),        intent(in)    :: action
real,                     intent(in)    :: dtime
integer,        optional, intent(out)   :: ierr
character(len=*),optional,intent(out)   :: outstr
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=kind(0.0D+00)
real, parameter                         :: T_limit_AbsolutelyNoDust=2500.0
integer                                 :: m1,n1, m2,n2, m3,n3, nquc, i1, i2, i3, iquc, &
                                           iquc_monom, iquc_dustm, iquc_dustn, ierr_thread
real                                    :: rcondeff, dquc
integer, dimension(model%m1:model%n1)   :: iqucmax
real, dimension(model%m1:model%n1)      :: rho, P, T, rhovs, dtimefac
real, dimension(model%m1:model%n1, &
                1       :model%nquc)    :: quc, qucslope, rnucl, rcond, rcoag, rfast
real, dimension(1       :model%nquc-1)  :: r3_ratio
real(kind=drk)                          :: gravdb, rhovsdb, rnucldb, rconddb, rcoagdb, rcoaldb
real(kind=drk), &
      dimension(model%m1:model%n1)      :: rhodb, Pdb, Tdb
real(kind=drk), &
      dimension(model%m1:model%n1, &
                1       :model%nquc)    :: qucdb
real(kind=drk), &
     dimension(1:par%n_dustGrainRadius) :: dustGrainRadiusdb
integer                                 :: ierr0
character(len=80)                       :: outstr0
!character(len=2)                        :: sepstr
!---------------------------------------------------------------------------------------------------
!
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
nquc=model%nquc
iquc_monom=1
iquc_dustm=2
iquc_dustn=nquc
!
! --- Input parameters: take basic parameters from parameter file ---
rop             =par%C_dust01
ms              =par%C_dust02 * amu_drk
eps             =par%C_dust03
! Not used anymore:
!   eps_AlmostNoDust=par%C_dust04
! Not used:      par%C_dust05
alpha_stick_cond=par%C_dust06
alpha_stick_coag=par%C_dust07
alpha_stick_coal=par%C_dust08
alpha_stick_nucl=par%C_dust09
rs              =(3.d0*ms/(rop*4.d0*pi_drk))**(1.d0/3.d0)
!
! --- Grain radius ratios (actually volume ratios) ---
dustGrainRadiusdb=real(par%ar_dustGrainRadius, kind=drk)
r3_ratio(iquc_monom)=(rs/par%ar_dustGrainRadius(iquc_dustm))**3
do iquc=iquc_dustm,iquc_dustn-1
  r3_ratio(iquc)=(par%ar_dustGrainRadius(iquc)/par%ar_dustGrainRadius(iquc+1))**3
end do ! iquc
!
!write(6,*) 'rs=', rs
!do iquc=1,nquc-1
!  write(6,*) 'r3_ratio(', iquc ,')=', r3_ratio(iquc)
!end do ! iquc
!
write(gl%nc_p,'(A)') 'SOURCE: dust_bins_01'
call flush(gl%nc_p)
!
! --- Dust formation: update ---
!$OMP PARALLEL DEFAULT(SHARED), &
!$OMP   PRIVATE(i1, i2, ierr_thread, rho, quc, P, T, Pdb, Tdb, rhodb, qucdb, gravdb, iqucmax, &
!$OMP           rhovs, rnucl, rcond, rcoag, rfast, rcondeff, dquc, &
!$OMP           qucslope, rhovsdb, rnucldb, rconddb, rcoagdb, rcoaldb, dtimefac)
!
ierr_thread=0
!
!$OMP DO
do i3=m3,n3
  do i2=m2,n2
   !
   ! --- Make thread-local copies ---
   rho=model%rho(m1:n1,i2,i3)
   quc=max(model%quc(m1:n1,i2,i3,1:nquc), 0.0)
   !
   ! --- Solve EOS ---
   call eosinter_PT(rho, model%ei(m1:n1,i2,i3), P=P, T=T)
   !
!write(6,*) i2,i3,minval(T)
   if (minval(T) < T_limit_AbsolutelyNoDust) then
    ! === Temperatures low enough to form dust (possibly) somewhere: treat it in detail ===
    !
    ! --- Limit temperature to reasonable values ---
    T=min(T, T_limit_AbsolutelyNoDust)
    !
    ! --- Transform to double precision ---
    Pdb  =real(P  , kind=drk)
    Tdb  =real(T  , kind=drk)
    rhodb=real(rho, kind=drk)
    qucdb=real(quc, kind=drk)
    !
    ! --- Assume constant gravity ---
    gravdb=real(par%grav, kind=drk)
    !
    ! --- Bin index with maximum dust/monomer density ---
    iqucmax=maxloc(quc,2)
    !
    ! --- Slope of linear dust distribution within each bin ---
    qucslope=0.0
    if (nquc > 2) then
      ! --- First and last bin: slope between neighbor cells ---
      qucslope(:,iquc_dustm)=quc(:,iquc_dustm+1)-quc(:,iquc_dustm  )
      qucslope(:,iquc_dustn)=quc(:,iquc_dustn  )-quc(:,iquc_dustn-1)
      ! --- Interior bins: Superbee ---
      do iquc=iquc_dustm+1,iquc_dustn-1
        do i1=m1,n1
          qucslope(i1,iquc)=(     sign(1.0,quc(i1,iquc  ) - quc(i1,iquc-1)) + &
                                  sign(1.0,quc(i1,iquc+1) - quc(i1,iquc  )) &
                            ) * &
                            min( abs(      quc(i1,iquc  ) - quc(i1,iquc-1)  ), &
                                 abs(      quc(i1,iquc+1) - quc(i1,iquc  )  ), &
                                 max(  abs(quc(i1,iquc  ) - quc(i1,iquc-1)), &
                                       abs(quc(i1,iquc+1) - quc(i1,iquc  )) &
                                    )*0.5 &
                               )
        end do ! i1
      end do ! iquc
    endif
    !
    ! --- Initialize total rate ---
    rfast=0.0
    rnucl=0.0
    rcond=0.0
    rcoag=0.0
    !
    ! --- Call Rossow routine to get nucleation, condensation, and coaXXX rates ---
    ! --- Add rates to get total rate of change ---
    do iquc=iquc_dustm,iquc_dustn
      do i1=m1,n1
        if (quc(i1,iqucmax(i1)) > 0.0) then
          ! --- Call Rossow routine ---
          call dust_bins_rossow78(gravdb, Pdb(i1), Tdb(i1), &
                                  rhodb(i1), qucdb(i1,iquc), qucdb(i1,iquc_monom), &
                                  dustGrainRadiusdb(iquc), &
                                  rhovsdb, rnucldb, rconddb, rcoagdb, rcoaldb)
          !
          ! --- Transform to single precision ---
          rcond(i1,iquc)=real(rconddb        , kind=srk)
          rcoag(i1,iquc)=real(rcoagdb+rcoaldb, kind=srk)
          if (T(i1) >= T_limit_AbsolutelyNoDust) rcoag(i1,iquc)=0.0
          rhovs(i1)     =real(rhovsdb        , kind=srk)
          !
          ! --- Add nucleation rate to fast rate of first dust bin ---
          if (iquc == iquc_dustm) then
            rnucl(i1,iquc)=real(rnucldb, kind=srk)
            rfast(i1,iquc)=rfast(i1,iquc)+rnucl(i1,iquc)
          endif
          !
          ! --- Add evaporation/condensation rate to total fast rate ---
          if (iquc > iquc_dustm) then
            ! --- Rate between two dust bins ---
            ! --- Find effective condensation rate from "upwind" bin density ---
            if (quc(i1,iquc_monom) < rhovs(i1)) then
              ! --- Evaporation ---
              rcondeff=0.5 * (rcond(i1,iquc-1)+rcond(i1,iquc)) * &
                       (quc(i1,iquc  )-0.5*qucslope(i1,iquc  ))
            else
              ! --- Condensation ---
              rcondeff=0.5 * (rcond(i1,iquc-1)+rcond(i1,iquc)) * &
                       (quc(i1,iquc-1)+0.5*qucslope(i1,iquc-1))
            endif
            rfast(i1,iquc  )=rfast(i1,iquc  ) + rcondeff                  /(1.0-r3_ratio(iquc-1))
            rfast(i1,iquc-1)=rfast(i1,iquc-1) - rcondeff*(r3_ratio(iquc-1)/(1.0-r3_ratio(iquc-1)))
            !
          else
            ! --- Rate between monomer bin and lowest radius bin (no condensation) ---
            if (quc(i1,iquc_monom) < rhovs(i1)) then
              ! --- Evaporation ---
              rcondeff=rcond(i1,iquc) * quc(i1,iquc)
!              rcondeff=rcond(i1,iquc) * (quc(i1,iquc)-0.5*qucslope(i1,iquc  ))
              rfast(i1,iquc)=rfast(i1,iquc)   + rcondeff/(1.0-r3_ratio(iquc-1))
            endif
            !
          endif
          !
        endif
      end do ! i1
    end do ! iquc
    !
    ! --- Monomer change rate is negative of sum of grain change rate, fast processes only ---
    rfast(:,iquc_monom)=-sum(rfast(:,iquc_dustm:iquc_dustn),2)
    !
    ! --- Compute rate reduction factor due to rapid nucleation/condensation ---
    do i1=m1,n1
      if (abs(dtime*rfast(i1,1)) < 1.0E-02) then
        dtimefac(i1)=dtime
      else
        dtimefac(i1)=(exp(dtime*rfast(i1,1))-1.0)/rfast(i1,1)
      endif
    end do ! i1
!if ((minval(quc) < 0.0) .and. (ierr_thread == 0)) ierr_thread=100
!write(6,*) 'quc:  ', minval(quc), maxval(quc), minval(quc-model%quc(m1:n1,i2,i3,1:nquc)), maxval(quc-model%quc(m1:n1,i2,i3,1:nquc))
!write(6,*) 'dtime:', dtime, minval(dtimefac), maxval(dtimefac)
    !
    ! --- Account for condensation ---
    ! --- Start with largest grains ---
    do iquc=iquc_dustn,iquc_dustm+1,-1
      do i1=m1,n1
        if (quc(i1,iquc_monom) > rhovs(i1)) then
          ! --- There are excess momomers: condensation is possible ---
          dquc=dtimefac(i1)* &
               0.5*(rcond(i1,iquc-1)+rcond(i1,iquc))* &
               (model%quc(i1,i2,i3,iquc-1)+0.5*qucslope(i1,iquc-1))* &
               (r3_ratio(iquc-1)/(1.0-r3_ratio(iquc-1)))* &
               (quc(i1,iquc_monom)-rhovs(i1))
          dquc=min(dquc, quc(i1,iquc-1), (1.0/r3_ratio(iquc-1)-1.0)*(quc(i1,iquc_monom)-rhovs(i1)))
          quc(i1,iquc-1)    =max(quc(i1,iquc-1    )-dquc, 0.0)
          quc(i1,iquc  )    =    quc(i1,iquc      )     +dquc/r3_ratio(iquc-1)
          quc(i1,iquc_monom)=    quc(i1,iquc_monom)+dquc-dquc/r3_ratio(iquc-1)
        endif
      end do ! i1
    end do ! iquc
!if ((minval(quc) < 0.0) .and. (ierr_thread == 0)) ierr_thread=101
!write(6,*) 'quc:  ', minval(quc), maxval(quc), minval(quc-model%quc(m1:n1,i2,i3,1:nquc)), maxval(quc-model%quc(m1:n1,i2,i3,1:nquc))
    !
    ! --- Account for nucleation ---
    iquc=iquc_dustm
      do i1=m1,n1
        if (quc(i1,iquc_monom) > rhovs(i1)) then
          ! --- There are excess momomers: nucleation is possible ---
          ! --- Limit the nucleation rate so that at most all excess monomers nucleate ---
          dquc              =min(dtimefac(i1)*rnucl(i1,iquc), 1.0) * (quc(i1,iquc_monom)-rhovs(i1))
          quc(i1,iquc      )=quc(i1,iquc      )+dquc
          quc(i1,iquc_monom)=quc(i1,iquc_monom)-dquc
        endif
      end do ! i1
!if ((minval(quc) < 0.0) .and. (ierr_thread == 0)) ierr_thread=102
!write(6,*) 'quc:  ', minval(quc), maxval(quc), minval(quc-model%quc(m1:n1,i2,i3,1:nquc)), maxval(quc-model%quc(m1:n1,i2,i3,1:nquc))
    !
    ! --- Account for evaporation: start with smallest grains ---
    iquc=iquc_dustm
      do i1=m1,n1
        if (quc(i1,iquc_monom) < rhovs(i1)) then
          ! --- There is a deficit in momomers: evoparation occurs ---
          dquc=-dtimefac(i1)* &
               rcond(i1,iquc)* &
!               (model%quc(i1,i2,i3,iquc  )-0.5*qucslope(i1,iquc  ))* &
               model%quc(i1,i2,i3,iquc  )* &
               (1.0/(1.0-r3_ratio(iquc-1)))* &
               (quc(i1,iquc_monom)-rhovs(i1))
          dquc=min(dquc, quc(i1,iquc))
          quc(i1,iquc  )    =max(quc(i1,iquc      )-dquc, 0.0)
          quc(i1,iquc_monom)=    quc(i1,iquc_monom)+dquc
        endif
      end do ! i1
!if ((minval(quc) < 0.0) .and. (ierr_thread == 0)) ierr_thread=103
!write(6,*) 'quc:  ', minval(quc), maxval(quc), minval(quc-model%quc(m1:n1,i2,i3,1:nquc)), maxval(quc-model%quc(m1:n1,i2,i3,1:nquc))
    !
    ! --- Account for evaporation: from small to large grains ---
    do iquc=iquc_dustm+1,iquc_dustn
      do i1=m1,n1
        if (quc(i1,iquc_monom) < rhovs(i1)) then
          ! --- There is a deficit in momomers: evoparation occurs ---
          dquc=-dtimefac(i1)* &
               0.5*(rcond(i1,iquc-1)+rcond(i1,iquc))* &
               (model%quc(i1,i2,i3,iquc  )-0.5*qucslope(i1,iquc  ))* &
               (1.0/(1.0-r3_ratio(iquc-1)))* &
               (quc(i1,iquc_monom)-rhovs(i1))
          dquc=min(dquc, quc(i1,iquc))
          quc(i1,iquc  )    =max(quc(i1,iquc      )-dquc, 0.0)
          quc(i1,iquc-1)    =    quc(i1,iquc-1    )     +dquc*r3_ratio(iquc-1)
          quc(i1,iquc_monom)=    quc(i1,iquc_monom)+dquc-dquc*r3_ratio(iquc-1)
        endif
      end do ! i1
    end do ! iquc
!if ((minval(quc) < 0.0) .and. (ierr_thread == 0)) ierr_thread=104
!write(6,*) 'quc:  ', minval(quc), maxval(quc), minval(quc-model%quc(m1:n1,i2,i3,1:nquc)), maxval(quc-model%quc(m1:n1,i2,i3,1:nquc))
    !
    ! --- Account for coagulation and coalescence from bin iquc-1 to iquc ---
    ! --- Start with largest grains ---
    do iquc=iquc_dustn,iquc_dustm+1,-1
      do i1=m1,n1
        dquc=dtime* &
             2.0*r3_ratio(iquc-1)* &
             rcoag(i1,iquc-1)
        dquc=min(dquc, quc(i1,iquc-1))
        quc(i1,iquc-1)=max(quc(i1,iquc-1)-dquc, 0.0)
        quc(i1,iquc  )=    quc(i1,iquc  )+dquc
      end do ! i1
    end do ! iquc
!if ((minval(quc) < 0.0) .and. (ierr_thread == 0)) ierr_thread=105
!write(6,*) 'rcoag:', minval(rcoag), maxval(rcoag)
!write(6,*) 'quc:  ', minval(quc), maxval(quc), minval(quc-model%quc(m1:n1,i2,i3,1:nquc)), maxval(quc-model%quc(m1:n1,i2,i3,1:nquc))
    !
    ! --- Apply changes: copy temporary array to main array ---
    do iquc=iquc_monom,iquc_dustn
      do i1=m1,n1
!if ((i1 == 1) .and. (i3 > -1)) then
!  if (iquc == 1) then
!    sepstr='--'
!  else
!    sepstr='  '
!  endif
!  write(6,'(4I4,A,6(1PE11.3),A,3(1PE11.3),A,1PE11.3)') i1,i2,i3,iquc, sepstr, Tdb(i1), Pdb(i1), rhodb(i1), quc(i1,iquc), model%quc(i1,i2,i3,iquc), rhovs(i1), sepstr, rnucl(i1,iquc), rcond(i1,iquc), rcoag(i1,iquc), sepstr, dtimefac(i1)/dtime
!endif
        model%quc(i1,i2,i3,iquc)=quc(i1,iquc)
      end do ! i1
    end do ! iquc
    !
   else
    ! === Temperatures too high: skip detailed treatment of dust ===
    !
    ! --- Destroy all dust grains and put material into monomer bin ---
    !
    model%quc(:,i2,i3,iquc_monom           )=model%quc(:,i2,i3,iquc_monom) + &
                                             sum(model%quc(:,i2,i3,iquc_dustm:iquc_dustn),2)
    model%quc(:,i2,i3,iquc_dustm:iquc_dustn)=0.0
    !
   endif
   !
  end do ! i2
end do ! i3
!$OMP END DO
!
!OMP CRITICAL (DUST_ERROR_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0='Error in dust_bins_SourceStep'
  endif
!OMP END CRITICAL (DUST_ERROR_REDUCTION)
!
!$OMP END PARALLEL
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
end subroutine dust_bins_SourceStep

end module dust_bins_module
