C**************************************************************************************************
C
C   ######  #     #  #####  #######        #    #   #####  #     #  #####  #     #
C   #     # #     # #     #    #           #   #   #     # ##   ## #     # ##    #
C   #     # #     # #          #           #  #          # # # # # #     # # #   #
C   #     # #     #  #####     #           ###      #####  #  #  # #     # #  #  #
C   #     # #     #       #    #           #  #          # #     # #     # #   # #
C   #     # #     # #     #    #           #   #   #     # #     # #     # #    ##
C   ######   #####   #####     #   ####### #    #   #####  #     #  #####  #     # _module
C
C   Dust physics module
C
C**************************************************************************************************
C   Fortran90
C   Susanne Hoefner:       Uppsala
C   Bernd Freytag:         Uppsala, Lyon
C   Hans-Guenter Ludwig:   Lyon
C   2007-12-18
C**************************************************************************************************
C MODULES:
C   dust_k3mon_module:     simplified dust formation model
C
C**************************************************************************************************


C------------*****************---------------------------------------------------------------------
      module dust_k3mon_module
C--------------------------------------------------------------------------------------------------
C NAME:
C   rhd_k3mon_module ('rhd_k3_monomer_module')
C
C PURPOSE:
C   Provide routines for a simplified dust formation model.
C
C CATEGORY:
C   Dust
C
C CALLING SEQUENCE:
C   use dust_k3mon_module
C
C VARIABLES:
C   None
C
C ROUTINES: (contained)
C   DUSTFORM_3:     simplified dust formation model for CO5BOLD (1 moment type)
C   DUSTFORM_4:     simplified dust formation model for CO5BOLD (2 moments type)
C   DUSTFORM_5:     simplified dust formation model for CO5BOLD (2 moments type)
C                   modified version of DUSTFORM_4 for forsterite
C   DUSTFORM_6:     bugfix version of DUSTFORM_5
C   PSATC1:         saturation pressure of C over graphite
C   PSATFORSTERITE: Saturation pressure of forsterite
C
C MODIFICATION HISTORY:
C   28. 6.91 (Susanne Hoefner) PSATC1.FOR
C   18.11.02 (S.H.) dust_k3mon.f
C   20.11.02 (B.F.) Routines put into module
C   11.10.06 (H.G.L.) dustform_5 and psatforsterite added
C   2007-12-03 (B.F.) fosterite -> forsterite
C   2007-12-18 (B.F.) Use constants from const_module
C--------------------------------------------------------------------------------------------------
      implicit none
C
      contains

C         SUBROUTINE DUSTFORM ( NPT, RHOGAS, TEMP, DUSTK3, CZUO, XNDMAX, 
C        #                      TAUI, DUSTK0, XNCTOT, XNCGAS, R1DUST )
C---------------------------------------------------------------------------
C     file:    dust_k3mon.f
C     autor:   S. Hoefner
C     created: 18/11/2002
C---------------------------------------------------------------------------
C     variables:
C    
C      input: 
C    
C        NPT    : number of spatial points
C        RHOGAS : gas density [g/cm3]
C        TEMP   : temperature [K]
C        DUSTK3 : 3rd moment of the grain size distribution [1/cm3]
C        CZUO   : C/O ratio (by number)
C        XNDMAX : maximum possible number of atoms per grain
C    
C      output:
C    
C        TAUI   : net growth rate [1/s]
C        DUSTK0 : 0th moment of the grain size distribution [1/cm3]
C                 (= number density of grains)
C        XNCTOT : total number density of C atoms (gas+dust)
C        XNCGAS : actual number density of C atoms in the gas
C        R1DUST : monomer radius
C    
C---------------------------------------------------------------------------
C     comments:
C    
C      >>> simplified dust formation model for CO5BOLD <<<
C    
C      This routine provides the growth rate required for the source term
C      in a one-equation dust formation model. It assumes that the grains 
C      are formed and destroyed by reactions involving monomers (atoms) 
C      only, and that all condensible material in the gas phase is present
C      in the form of monomers. 
C    
C      The material-dependent quantities are given for the case of carbon 
C      grains in a C-rich environment
C    
C          <-> CZUO > 1 !!!
C    
C      The one-equation model assumes that the number density of grains
C      DUSTK0 is proportional to the gas density (no formation or 
C      destruction of the seed nuclei) and that all grains at a given 
C      point and time have the same size (size distribution function =
C      delta function). The temporal evolution of the grain size (growth
C      and evaporation) is described by the equation for DUSTK3. The 
C      source term at a given point is
C    
C        TAUI * DUSTK3**(2.0/3.0) * DUSTK0**(1.0/3.0)  = d/dt (DUSTK3)
C    
C      in units [1/(s cm3)]. The grain size in monomers (atoms) and the 
C      grain radius in [cm] are 
C    
C        NGRAIN = DUSTK3 / DUSTK0
C        RGRAIN = R1DUST * NGRAIN**(1.0/3.0)
C    
C      respectively.
C    
C---------------------------------------------------------------------------
C   
C     Declaration, Include, Parameter, Dimension:
C   
C         IMPLICIT NONE
C   
C     Integer:
C   
C         INTEGER I,NPT
C   
C     Double Precision:
C   
C          DOUBLE PRECISION
C         REAL
C        # RHOGAS(NPT),TEMP(NPT),DUSTK3(NPT),
C        # DUSTK0(NPT),XNCTOT(NPT),XNCGAS(NPT),TAUI(NPT),
C        # PMASS,BOLTZ,PI,OHS,HEH,CZUO,HELPNC,XNDMAX,
C        # Z03,Z4PI,AGW,RHOD,STICK1,R1DUST,A1DUST,
C        # VTH,PSAT,DUMMY
C   
C---------------------------------------------------------------------------
C     fundamental constants and parameters:   (adopt your preferred values)
C   
C         PMASS  = 1.6726E-24              ! proton mass
C         BOLTZ  = 1.38062E-16             ! Boltzmann constant (k)
C         PI     = 3.1415926535898
C   
C         OHS    = 1.0D01**(-3.18D00)      ! abundance of O (solar)
C         HEH    = 10.D00**(-1.00D00)      ! abundance of He
C   
C   ccc      CZUO   = 1.8                     ! C/O ratio (by number)
C   
C---------------------------------------------------------------------------
C     calculate total and actual carbon number density:
C   
C         HELPNC = ( CZUO - 1.0D00 )*OHS / ( PMASS*( 1.0D00 + HEH*4 ) )
C   
C         DO I=1,NPT
C           XNCTOT(I) = HELPNC * RHOGAS(I)        ! total  (<-> abundance)
C           XNCGAS(I) = XNCTOT(I) - DUSTK3(I)     ! actual (minus condensed)
C         END DO
C   
C---------------------------------------------------------------------------
C     calculate number density of dust grains: 
C       (normally given by equation; here: based on available carbon and 
C       maximum allowed size of grains XNDMAX)
C   
C   ccc      XNDMAX = 1.0D+07     ! max. number of atoms per grain (free par.)
C   
C         DO I=1,NPT
C           DUSTK0(I) = XNCTOT(I) / XNDMAX
C         END DO
C   
C---------------------------------------------------------------------------
C     calculated net growth rate (may be negative!):
C       (normally done by a bunch of subroutines involving chemistry, etc.
C       here: reactions involving C atoms, only)
C   
C         Z03    = 1.0 / 3.0
C         Z4PI   = 4.0 * PI
C   
C         AGW    = 12.01115D00      ! atomic weight of carbon
C         RHOD   = 2.25D00          ! density of grain material
C         STICK1 = 0.37             ! sticking coeff. [0-1]
C   
C         R1DUST = ( 3.0 * AGW * PMASS / ( Z4PI * RHOD ) )**Z03
C         A1DUST = Z4PI * R1DUST**2
C   
C         DO I=1,NPT
C   
C           CALL PSATC1 ( 1, TEMP(I), PSAT, DUMMY )
C   
C           VTH     = SQRT( BOLTZ * TEMP(I) / (2.0 * PI * PMASS * AGW) )
C           TAUI(I) = A1DUST * STICK1 * VTH * 
C        #                   ( XNCGAS(I) - PSAT/(BOLTZ*TEMP(I)) )
C   
C         END DO
C   
C---------------------------------------------------------------------------
C   
C         RETURN
C         END SUBROUTINE DUSTFORM


C         SUBROUTINE DUSTFORM_2 ( NPT, XNCGAS, TEMP, DUSTK3, CZUO, XNDMAX, 
C        #                        TAUI, DUSTK0, XNCTOT, R1DUST )
C---------------------------------------------------------------------------
C     file:    dust_k3mon.f
C     autor:   S. Hoefner
C     created: 18/11/2002
C              20/11/2002: derived from DUSTFORM by B.F.
C                          XNCGAS instead of RHOGAS becomes input parameter
C---------------------------------------------------------------------------
C     variables:
C    
C      input: 
C    
C        NPT    : number of spatial points
C        XNCGAS : actual number density of C atoms in the gas
C        TEMP   : temperature [K]
C        DUSTK3 : 3rd moment of the grain size distribution [1/cm3]
C        CZUO   : C/O ratio (by number)
C        XNDMAX : maximum possible number of atoms per grain
C    
C      output:
C    
C        TAUI   : net growth rate [1/s]
C        DUSTK0 : 0th moment of the grain size distribution [1/cm3]
C                 (= number density of grains)
C        XNCTOT : total number density of C atoms (gas+dust)
C        R1DUST : monomer radius
C    
C---------------------------------------------------------------------------
C     comments:
C    
C      >>> simplified dust formation model for CO5BOLD <<<
C    
C      This routine provides the growth rate required for the source term
C      in a one-equation dust formation model. It assumes that the grains 
C      are formed and destroyed by reactions involving monomers (atoms) 
C      only, and that all condensible material in the gas phase is present
C      in the form of monomers. 
C    
C      The material-dependent quantities are given for the case of carbon 
C      grains in a C-rich environment
C    
C          <-> CZUO > 1 !!!
C    
C      The one-equation model assumes that the number density of grains
C      DUSTK0 is proportional to the gas density (no formation or 
C      destruction of the seed nuclei) and that all grains at a given 
C      point and time have the same size (size distribution function =
C      delta function). The temporal evolution of the grain size (growth
C      and evaporation) is described by the equation for DUSTK3. The 
C      source term at a given point is
C    
C        TAUI * DUSTK3**(2.0/3.0) * DUSTK0**(1.0/3.0)  = d/dt (DUSTK3)
C    
C      in units [1/(s cm3)]. The grain size in monomers (atoms) and the 
C      grain radius in [cm] are 
C    
C        NGRAIN = DUSTK3 / DUSTK0
C        RGRAIN = R1DUST * NGRAIN**(1.0/3.0)
C    
C      respectively.
C    
C---------------------------------------------------------------------------
C   
C     Declaration, Include, Parameter, Dimension:
C   
C         IMPLICIT NONE
C   
C     Integer:
C   
C         INTEGER I,NPT
C   
C     Double Precision:
C   
C          DOUBLE PRECISION
C         REAL
C        # TEMP(NPT),DUSTK3(NPT),
C        # DUSTK0(NPT),XNCTOT(NPT),XNCGAS(NPT),TAUI(NPT),
C        # PMASS,BOLTZ,PI,OHS,HEH,CZUO,HELPNC,XNDMAX,
C        # Z03,Z4PI,AGW,RHOD,STICK1,R1DUST,A1DUST,
C        # VTH,PSAT,DUMMY
C   
C---------------------------------------------------------------------------
C     fundamental constants and parameters:   (adopt your preferred values)
C   
C         PMASS  = 1.6726E-24              ! proton mass
C         BOLTZ  = 1.38062E-16             ! Boltzmann constant (k)
C         PI     = 3.1415926535898
C   
C         OHS    = 1.0D01**(-3.18D00)      ! abundance of O (solar)
C         HEH    = 10.D00**(-1.00D00)      ! abundance of He
C   
C   ccc      CZUO   = 1.8                     ! C/O ratio (by number)
C   
C---------------------------------------------------------------------------
C     calculate total and actual carbon number density:
C   
C         HELPNC = ( CZUO - 1.0D00 )*OHS / ( PMASS*( 1.0D00 + HEH*4 ) )
C   
C         DO I=1,NPT
C           XNCTOT(I) = XNCGAS(I) + DUSTK3(I)     ! total: gas + dust
C         END DO
C   
C---------------------------------------------------------------------------
C     calculate number density of dust grains: 
C       (normally given by equation; here: based on available carbon and 
C       maximum allowed size of grains XNDMAX)
C   
C   ccc      XNDMAX = 1.0D+07     ! max. number of atoms per grain (free par.)
C   
C         DO I=1,NPT
C           DUSTK0(I) = XNCTOT(I) / XNDMAX
C         END DO
C   
C---------------------------------------------------------------------------
C     calculated net growth rate (may be negative!):
C       (normally done by a bunch of subroutines involving chemistry, etc.
C       here: reactions involving C atoms, only)
C   
C         Z03    = 1.0 / 3.0
C         Z4PI   = 4.0 * PI
C   
C         AGW    = 12.01115D00      ! atomic weight of carbon
C         RHOD   = 2.25D00          ! density of grain material
C         STICK1 = 0.37             ! sticking coeff. [0-1]
C   
C         R1DUST = ( 3.0 * AGW * PMASS / ( Z4PI * RHOD ) )**Z03
C         A1DUST = Z4PI * R1DUST**2
C   
C         DO I=1,NPT
C   
C           CALL PSATC1 ( 1, TEMP(I), PSAT, DUMMY )
C   
C           VTH     = SQRT( BOLTZ * TEMP(I) / (2.0 * PI * PMASS * AGW) )
C           TAUI(I) = A1DUST * STICK1 * VTH * 
C        #                   ( XNCGAS(I) - PSAT/(BOLTZ*TEMP(I)) )
C   
C         END DO
C---------------------------------------------------------------------------
C         RETURN
C         END SUBROUTINE DUSTFORM_2
C---------------------------------------------------------------------------


      SUBROUTINE DUSTFORM_3( NPT, RHOGAS, TEMP, DUSTK3, CZUO, XNDMAX, 
     #                       TAUI, DUSTK0, XNCTOT, XNCGAS, R1DUST,
     #                       rateK3, equK3)
C-----------------------------------------------------------------------
C file:    dust_k3mon.f
C autor:   S. Hoefner
C created: 18/11/2002
C-----------------------------------------------------------------------
C variables:
C
C  input: 
C
C    NPT    : number of spatial points
C    RHOGAS : gas density [g/cm3]
C    TEMP   : temperature [K]
C    DUSTK3 : 3rd moment of the grain size distribution [1/cm3]
C    CZUO   : C/O ratio (by number)
C    XNDMAX : maximum possible number of atoms per grain
C
C  output:
C
C    TAUI   : net growth rate [1/s]
C    DUSTK0 : 0th moment of the grain size distribution [1/cm3]
C             (= number density of grains)
C    XNCTOT : total number density of C atoms (gas+dust)
C    XNCGAS : actual number density of C atoms in the gas
C    R1DUST : monomer radius
C
C-----------------------------------------------------------------------
C comments:
C
C  >>> simplified dust formation model for CO5BOLD <<<
C
C  This routine provides the growth rate required for the source term
C  in a one-equation dust formation model. It assumes that the grains 
C  are formed and destroyed by reactions involving monomers (atoms) 
C  only, and that all condensible material in the gas phase is present
C  in the form of monomers. 
C
C  The material-dependent quantities are given for the case of carbon 
C  grains in a C-rich environment
C
C      <-> CZUO > 1 !!!
C
C  The one-equation model assumes that the number density of grains
C  DUSTK0 is proportional to the gas density (no formation or 
C  destruction of the seed nuclei) and that all grains at a given 
C  point and time have the same size (size distribution function =
C  delta function). The temporal evolution of the grain size (growth
C  and evaporation) is described by the equation for DUSTK3. The 
C  source term at a given point is
C
C    TAUI * DUSTK3**(2.0/3.0) * DUSTK0**(1.0/3.0)  = d/dt (DUSTK3)
C
C  in units [1/(s cm3)]. The grain size in monomers (atoms) and the 
C  grain radius in [cm] are 
C
C    NGRAIN = DUSTK3 / DUSTK0
C    RGRAIN = R1DUST * NGRAIN**(1.0/3.0)
C
C  respectively.
C
C-----------------------------------------------------------------------

C Declaration, Include, Parameter, Dimension:

      use const_module, only: pi_drk, k_Bol_drk, amu_drk

      IMPLICIT NONE

C Integer:

      INTEGER I,NPT

C Double Precision:

C      DOUBLE PRECISION
      REAL
     # RHOGAS(NPT),TEMP(NPT),DUSTK3(NPT),
     # DUSTK0(NPT),XNCTOT(NPT),XNCGAS(NPT),TAUI(NPT),
     # PMASS,BOLTZ,PI,OHS,HEH,CZUO,HELPNC,XNDMAX,
     # Z03,Z4PI,AGW,RHOD,STICK1,R1DUST,A1DUST,
     # VTH, PSAT(NPT), DPSATT(NPT), DUMMY, 
     # rateK3(NPT), equK3(NPT)

C-----------------------------------------------------------------------
C fundamental constants and parameters:   (adopt your preferred values)

c      PMASS  = 1.6726E-24              ! proton mass
c      BOLTZ  = 1.38062E-16             ! Boltzmann constant (k)
c      PI     = 3.1415926535898
      PARAMETER ( PI     = pi_drk,
     #            BOLTZ  = k_Bol_drk,
     #            PMASS  = amu_drk )

      OHS    = 1.0D01**(-3.18D00)      ! abundance of O (solar)
      HEH    = 10.D00**(-1.00D00)      ! abundance of He

ccc      CZUO   = 1.8                     ! C/O ratio (by number)

C-----------------------------------------------------------------------
C calculate total and actual carbon number density:

      HELPNC = ( CZUO - 1.0D00 )*OHS / ( PMASS*( 1.0D00 + HEH*4 ) )

      DO I=1,NPT
        XNCTOT(I) = HELPNC * RHOGAS(I)        ! total  (<-> abundance)
        XNCGAS(I) = XNCTOT(I) - DUSTK3(I)     ! actual (minus condensed)
      END DO

C-----------------------------------------------------------------------
C calculate number density of dust grains: 
C   (normally given by equation; here: based on available carbon and 
C   maximum allowed size of grains XNDMAX)

ccc      XNDMAX = 1.0D+07     ! max. number of atoms per grain (free par.)

      DO I=1,NPT
        DUSTK0(I) = XNCTOT(I) / XNDMAX
      END DO

C-----------------------------------------------------------------------
C calculated net growth rate (may be negative!):
C   (normally done by a bunch of subroutines involving chemistry, etc.
C   here: reactions involving C atoms, only)

      Z03    = 1.0 / 3.0
      Z4PI   = 4.0 * PI

      AGW    = 12.01115D00      ! atomic weight of carbon
      RHOD   = 2.25D00          ! density of grain material
      STICK1 = 0.37             ! sticking coeff. [0-1]

      R1DUST = ( 3.0 * AGW * PMASS / ( Z4PI * RHOD ) )**Z03
      A1DUST = Z4PI * R1DUST**2

      CALL PSATC1 ( NPT, TEMP, PSAT, DPSATT )

      DO I=1,NPT
        VTH     = SQRT( BOLTZ * TEMP(I) / (2.0 * PI * PMASS * AGW) )
        TAUI(I) = A1DUST * STICK1 * VTH * 
     #                   ( XNCGAS(I) - PSAT(I)/(BOLTZ*TEMP(I)) )

        rateK3(I) = A1DUST * STICK1 * VTH *
     #               ((1.0E-09*DUSTK3(I))**2 * DUSTK0(I))**Z03 * 1.0E+06
        equK3( I) = XNCTOT(I) - PSAT(I)/(BOLTZ*TEMP(I))
      END DO
C-----------------------------------------------------------------------
      END SUBROUTINE DUSTFORM_3


      SUBROUTINE DUSTFORM_4( NPT, XNCGAS, TEMP, DUSTK3, CZUO, XNDMAX, 
     #                       TAUI, DUSTK0, XNCTOT, R1DUST,
     #                       rateK3, equK3)
C-----------------------------------------------------------------------
C file:    dust_k3mon.f
C autor:   S. Hoefner
C created: 18/11/2002
C-----------------------------------------------------------------------
C variables:
C
C  input: 
C
C    NPT    : number of spatial points
C    TEMP   : temperature [K]
C    DUSTK3 : 3rd moment of the grain size distribution [1/cm3]
C    CZUO   : C/O ratio (by number)
C    XNDMAX : maximum possible number of atoms per grain
C
C  output:
C
C    TAUI   : net growth rate [1/s]
C    DUSTK0 : 0th moment of the grain size distribution [1/cm3]
C             (= number density of grains)
C    XNCTOT : total number density of C atoms (gas+dust)
C    XNCGAS : actual number density of C atoms in the gas
C    R1DUST : monomer radius
C
C-----------------------------------------------------------------------
C comments:
C
C  >>> simplified dust formation model for CO5BOLD <<<
C
C  This routine provides the growth rate required for the source term
C  in a one-equation dust formation model. It assumes that the grains 
C  are formed and destroyed by reactions involving monomers (atoms) 
C  only, and that all condensible material in the gas phase is present
C  in the form of monomers. 
C
C  The material-dependent quantities are given for the case of carbon 
C  grains in a C-rich environment
C
C      <-> CZUO > 1 !!!
C
C  The one-equation model assumes that the number density of grains
C  DUSTK0 is proportional to the gas density (no formation or 
C  destruction of the seed nuclei) and that all grains at a given 
C  point and time have the same size (size distribution function =
C  delta function). The temporal evolution of the grain size (growth
C  and evaporation) is described by the equation for DUSTK3. The 
C  source term at a given point is
C
C    TAUI * DUSTK3**(2.0/3.0) * DUSTK0**(1.0/3.0)  = d/dt (DUSTK3)
C
C  in units [1/(s cm3)]. The grain size in monomers (atoms) and the 
C  grain radius in [cm] are 
C
C    NGRAIN = DUSTK3 / DUSTK0
C    RGRAIN = R1DUST * NGRAIN**(1.0/3.0)
C
C  respectively.
C
C-----------------------------------------------------------------------

C Declaration, Include, Parameter, Dimension:

      use const_module, only: pi_drk, k_Bol_drk, amu_drk

      IMPLICIT NONE

C Integer:

      INTEGER I,NPT

C Double Precision:

C      DOUBLE PRECISION
      REAL
     # TEMP(NPT),DUSTK3(NPT),
     # DUSTK0(NPT),XNCTOT(NPT),XNCGAS(NPT),TAUI(NPT),
     # PMASS,BOLTZ,PI,OHS,HEH,CZUO,HELPNC,XNDMAX,
     # Z03,Z4PI,AGW,RHOD,STICK1,R1DUST,A1DUST,
     # VTH, PSAT(NPT), DPSATT(NPT), DUMMY, 
     # rateK3(NPT), equK3(NPT)

C-----------------------------------------------------------------------
C fundamental constants and parameters:   (adopt your preferred values)

c      PMASS  = 1.6726E-24              ! proton mass
c      BOLTZ  = 1.38062E-16             ! Boltzmann constant (k)
c      PI     = 3.1415926535898
      PARAMETER ( PI     = pi_drk,
     #            BOLTZ  = k_Bol_drk,
     #            PMASS  = amu_drk )

      OHS    = 1.0D01**(-3.18D00)      ! abundance of O (solar)
      HEH    = 10.D00**(-1.00D00)      ! abundance of He

ccc      CZUO   = 1.8                     ! C/O ratio (by number)

C-----------------------------------------------------------------------
C calculate total and actual carbon number density:

      HELPNC = ( CZUO - 1.0D00 )*OHS / ( PMASS*( 1.0D00 + HEH*4 ) )

      DO I=1,NPT
        XNCTOT(I) = XNCGAS(I) + DUSTK3(I)     ! total: gas + dust
      END DO

C-----------------------------------------------------------------------
C calculate number density of dust grains: 
C   (normally given by equation; here: based on available carbon and 
C   maximum allowed size of grains XNDMAX)

ccc      XNDMAX = 1.0D+07     ! max. number of atoms per grain (free par.)

      DO I=1,NPT
        DUSTK0(I) = XNCTOT(I) / XNDMAX
      END DO

C-----------------------------------------------------------------------
C calculated net growth rate (may be negative!):
C   (normally done by a bunch of subroutines involving chemistry, etc.
C   here: reactions involving C atoms, only)

      Z03    = 1.0 / 3.0
      Z4PI   = 4.0 * PI

      AGW    = 12.01115D00      ! atomic weight of carbon
      RHOD   = 2.25D00          ! density of grain material
      STICK1 = 0.37             ! sticking coeff. [0-1]

      R1DUST = ( 3.0 * AGW * PMASS / ( Z4PI * RHOD ) )**Z03
      A1DUST = Z4PI * R1DUST**2

      CALL PSATC1 ( NPT, TEMP, PSAT, DPSATT )

      DO I=1,NPT
        VTH     = SQRT( BOLTZ * TEMP(I) / (2.0 * PI * PMASS * AGW) )
        TAUI(I) = A1DUST * STICK1 * VTH * 
     #                   ( XNCGAS(I) - PSAT(I)/(BOLTZ*TEMP(I)) )

        rateK3(I) = A1DUST * STICK1 * VTH *
     #               ((1.0E-09*DUSTK3(I))**2 * DUSTK0(I))**Z03 * 1.0E+06
        equK3( I) = XNCTOT(I) - PSAT(I)/(BOLTZ*TEMP(I))
      END DO
C-----------------------------------------------------------------------
      END SUBROUTINE DUSTFORM_4


      SUBROUTINE PSATC1 ( NPT, TEMP, PSAT, DPSATT )
C
C----------------------------------------------------------------------
C File  :    PSATC1.FOR
C Autor :    Susanne Hoefner
C Datum :    28.6.91                   letzte Aenderung : 3.7.91
C
C Verwendungszweck :
C     
C     Berechnung des Saettigungsdruckes von C ueber Graphit
C......................................................................
C
C  Parameter :
C
C     TEMP       : Temperatur
C     PSAT       : Saettigungsdruck
C     DPSATT     : Abl. von Psat nach der Temperatur
C.......................................................................
C
C  verwendete Unterprogramme :
C     *** keine ***
C.......................................................................
C
C  weitere Kommentare :
C
C     Naeherungsformel fuer PSAT von C ueber Graphit 
C     (nach Gail & Sedlmayr, Paper IV, A+A 1988) 
C     gueltig im Temperaturbereich 500 K <= T <= 1500 K
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INTEGER   NPT,L
C
C      DOUBLE PRECISION  
      REAL
     #    TEMP(NPT),PSAT(NPT),DPSATT(NPT),XLNPS,DLNPST,APSAT,BPSAT
C
C     Koeffizienten fuer C ( 500 K - 1500 K )
C
      APSAT  = 86300D00
      BPSAT  = 32.89D00
C
      DO L=1,NPT
C
C       Formel fuer ln(Psat) nach G&S (Paper IV)
C
        XLNPS     = - APSAT / TEMP(L) + BPSAT
        DLNPST    =   APSAT / TEMP(L)**2
C
C       Saettigungsdruck
C
        PSAT(L)   = EXP (XLNPS)
        DPSATT(L) = PSAT(L) * DLNPST
C
      END DO
C
      END SUBROUTINE PSATC1


      SUBROUTINE DUSTFORM_5( NPT, XNCGAS, TEMP, DUSTK3, CZUO, XNDMAX, 
     #                       TAUI, DUSTK0, XNCTOT, R1DUST,
     #                       rateK3, equK3)
C-----------------------------------------------------------------------
C dustform_4 adapted for forsterite
C created: 18/11/2002
C-----------------------------------------------------------------------

      use const_module, only: pi_drk, k_Bol_drk, amu_drk

      IMPLICIT NONE

C Integer:

      INTEGER I,NPT

C Double Precision:

C      DOUBLE PRECISION
      REAL
     # TEMP(NPT),DUSTK3(NPT),
     # DUSTK0(NPT),XNCTOT(NPT),XNCGAS(NPT),TAUI(NPT),
     # PMASS,BOLTZ,PI,OHS,HEH,CZUO,HELPNC,XNDMAX,
     # Z03,Z4PI,AGW,RHOD,STICK1,R1DUST,A1DUST,
     # VTH, PSAT(NPT), DPSATT(NPT), DUMMY, 
     # rateK3(NPT), equK3(NPT)

C-----------------------------------------------------------------------
C fundamental constants and parameters:   (adopt your preferred values)

c      PMASS  = 1.6726E-24              ! proton mass
c      BOLTZ  = 1.38062E-16             ! Boltzmann constant (k)
c      PI     = 3.1415926535898
      PARAMETER ( PI     = pi_drk,
     #            BOLTZ  = k_Bol_drk,
     #            PMASS  = amu_drk )

      DO I=1,NPT
        XNCTOT(I) = max(XNCGAS(I) + DUSTK3(I),0.0)     ! total: gas + dust
      END DO

      DO I=1,NPT
        DUSTK0(I) = XNCTOT(I) / XNDMAX
      END DO

      Z03    = 1.0 / 3.0
      Z4PI   = 4.0 * PI

      AGW    = 172.25          ! atomic weight of forsterite
      RHOD   = 3.8D00          ! density of grain material
      STICK1 = 0.37            ! sticking coeff. [0-1]

      R1DUST = ( 3.0 * AGW * PMASS / ( Z4PI * RHOD ) )**Z03
      A1DUST = Z4PI * R1DUST**2

      CALL PSATFORSTERITE(NPT, TEMP, PSAT, DPSATT)

      DO I=1,NPT
        VTH     = SQRT( BOLTZ * TEMP(I) / (2.0 * PI * PMASS * AGW) )
        TAUI(I) = A1DUST * STICK1 * VTH * 
     #                   ( XNCGAS(I) - PSAT(I)/(BOLTZ*TEMP(I)) )

        rateK3(I) = A1DUST * STICK1 * VTH *
     #           ((1.0E-09*DUSTK3(I))**2 * DUSTK0(I))**Z03 * 1.0E+06
        equK3( I) = max(XNCTOT(I) - PSAT(I)/(BOLTZ*TEMP(I)), 0.0)
      END DO
      END SUBROUTINE DUSTFORM_5
C-----------------------------------------------------------------------


      SUBROUTINE PSATFORSTERITE(NPT, TEMP, PSAT, DPSATT)
C......................................................................
C     Saturation pressure of forsterite MgFeSiO_4 (or Mg2SiO4 ?)
C     MODIFICATION HISTORY:
C       2006-11-XX (HGL)
C       2007-12-12 (B.F.) Limit temperature from below to avoid singularity
C......................................................................

      IMPLICIT NONE
C
      INTEGER   NPT,L
C
C      DOUBLE PRECISION  
      REAL
     #    TEMP(NPT),PSAT(NPT),DPSATT(NPT),XLNPS,DLNPST,APSAT,BPSAT,T400,
     #    a, b, c, d 
C
         a      =  42.179466259637
         b      = -2.94064111396886e4
         c      = -4.52457333892826e7
         d      = 2.00362363225697e10

      DO L=1,NPT
         T400=max(temp(L), 400.0)
         psat(L)   = exp(a + b/T400 + c/T400**2 + d/T400**3)
         dpsatt(L) = 0.0  ! ad-hoc value, not used
      END DO
C
      END SUBROUTINE PSATFORSTERITE
C-----------------------------------------------------------------------


C----------------**********------------------------------------------------------------------------
      SUBROUTINE DUSTFORM_6(NPT, XNCGAS, TEMP, DUSTK3, CZUO, XNDMAX,
     #                      TAUI, DUSTK0, XNCTOT, R1DUST,
     #                      rateK3, equK3)
C--------------------------------------------------------------------------------------------------
C NAME:
C   dustform_6 ('dust_formation_6')
C
C PURPOSE:
C   Compute condensation or evaporation rate for forsterite
C
C CATEGORY:
C   Hydrodynamics, dust formation
C
C CALLING SEQUENCE:
C
C INPUT:
C        NPT    : number of spatial points
C        XNCGAS : actual number density of C atoms in the gas
C        TEMP   : temperature [K]
C        DUSTK3 : 3rd moment of the grain size distribution [1/cm3]
C        CZUO   : C/O ratio (by number): not used
C        XNDMAX : maximum possible number of atoms per grain
C    
C OUTPUT:
C        TAUI   : net growth rate [1/s]
C        DUSTK0 : 0th moment of the grain size distribution [1/cm3]
C                 (= number density of grains)
C        XNCTOT : total number density of C atoms (gas+dust)
C        R1DUST : monomer radius
C        rateK3 : rate for 3rd dust moment
C        equK3  : equilibrium value for 3rd dust moment
C
C LOCAL VARIABLES:
C   vth:    ('velocity_thermal')
C
C ROUTINES:
C   PSATFORSTERITE: Compute saturation pressure (pressure of the gas)
C
C MODIFICATION HISTORY:
C   2002-10-06 (S.H.) dustform created
C   2006-11-18 (HGL) dustform_4 -> dustform_5 adapted for forsterite
C   2007-12-03 (B.F.) dustform_5 -> dustform_6: bugfixes
C   2007-12-11 (B.F.) Replaced PMASS,BOLTZ,PI by amu,k_Bol,pi from const_module
C   2007-12-12 (B.F.) STICK1=0.37 -> 1.0 (to be consistent with dust_bins_module).
C                     Take basic parameters from parameter file
C   2007-12-13 (B.F.) Parameter NumMonoMini added
C   2007-12-18 (B.F.) Re-arrangement of input parameters
C--------------------------------------------------------------------------------------------------
      use const_module, only: pi, k_Bol, amu
      use rhd_gl_module
C
      IMPLICIT NONE
C
C     --- I/O parameters ---
      INTEGER, intent(in) :: NPT
      REAL, intent(in)    ::
     # XNCGAS(NPT),TEMP(NPT),DUSTK3(NPT),CZUO,XNDMAX
      REAL, intent(out)   ::
     # TAUI(NPT),DUSTK0(NPT),XNCTOT(NPT),R1DUST,
     # rateK3(NPT), equK3(NPT)
C
C     --- Local variables ---
      INTEGER             :: I
      REAL                :: VTH, A1DUST, PSAT(NPT), DPSATT(NPT)
      real                :: AGW, RHOD, eps, STICK1, NumMonoMini
C     --- Fundamental constants and parameters:   (adopt your preferred values) ---
C      REAL, parameter ::
C     # AGW    = 140.71,            ! atomic weight of forsterite, was: 172.25
C     # RHOD   = 3.3E+00,           ! density of grain material, was: 3.8D00
C     # eps    = 3.1187E-05,        ! number fraction of Mg, from phoenix_Rainout-2007.GN.abu
C     # STICK1 = 1.0                ! sticking coeff. [0-1], was 0.37
C--------------------------------------------------------------------------------------------------
C     --- Take basic parameters from parameter file ---
      RHOD       =par%C_dust01
      AGW        =par%C_dust02
      eps        =par%C_dust03
      NumMonoMini=max(par%C_dust05,1.0)
      STICK1     =par%C_dust06
C
C     --- Radius and surface area of single momomer ---
      R1DUST = ( AGW * amu / ( (4.0*pi/3.0) * RHOD ) )**(1.0/3.0)
      A1DUST = (4.0*pi) * R1DUST**2
C
C     --- Compute total number of momomers from gas + dust ---
      DO I=1,NPT
        XNCTOT(I) = max(XNCGAS(I) + DUSTK3(I),0.0)
      END DO ! I
C
C     --- Define number of dust grains as fixed fraction of total number of momomers ---
      DO I=1,NPT
        DUSTK0(I) = XNCTOT(I) / XNDMAX
      END DO ! I
C
C     --- Compute saturation pressure (pressure of the gas) ---
      CALL PSATFORSTERITE(NPT, TEMP, PSAT, DPSATT)
C
C     --- Compute partial pressure of "forsterite momomers" ---
      DO I=1,NPT
C        PSAT(I)=0.5*eps*PSAT(I)
        PSAT(I)=eps*PSAT(I)
      END DO ! I
C
C     --- Compute rate and equilibrium value ---
      DO I=1,NPT
        VTH     = SQRT( k_Bol * TEMP(I) / ((2.0*pi) * amu * AGW) )
        TAUI(I) = A1DUST * STICK1 * VTH * 
     #                   ( XNCGAS(I) - PSAT(I)/(k_Bol*TEMP(I)) )
C        rateK3(I) = A1DUST * STICK1 * VTH *
C     #         ((1.0E-09*DUSTK3(I))**2 * DUSTK0(I))**(1.0/3.0) * 1.0E+06
        rateK3(I) = A1DUST * STICK1 * VTH *
     #         ((1.0E-09*max(DUSTK3(I),NumMonoMini*DUSTK0(I)))**2 *
     #          DUSTK0(I))**(1.0/3.0) * 1.0E+06
        equK3( I) = XNCTOT(I) - PSAT(I)/(k_Bol*TEMP(I))
      END DO ! I
C
      END SUBROUTINE DUSTFORM_6
C-----------------------------------------------------------------------

      end module dust_k3mon_module
