!***************************************************************************************************
!
!                       #####                  #                            #     # #######
!        #    #  ####  #     #  ####  #  ####  #    #    ##    ####  #####  #     #    #
!        ##  ## #    #       # #      # #    # #    #   #  #  #    # #    # #     #    #
!        # ## # #       #####   ####  # #    # #    #  #    # #      #####  #     #    #
!        #    # #  ### #            # # #    # ####### ###### #  ### #    # #     #    #
!        #    # #    # #       #    # # #    #      #  #    # #    # #    # #     #    #
!   dust_#    #  ####  #######  ####  #  ####       #  #    #  ####  #####   #####     # _module
!
!***************************************************************************************************
!   Fortran90
!   2017-05-19
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   dust_Mg2SiO4AGBUT_module: Unit-test routines for dust routines
!
!***************************************************************************************************


!------************************---------------------------------------------------------------------
module dust_Mg2SiO4AGBUT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_Mg2SiO4AGBUT_module ('rhd_Mg2_Si_O4_AGB_Unit_Test_module')
!
! PURPOSE:
!   Provide unit-test routines for dust test routines.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhd_Mg2SiO4AGBUT_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   dust_Mg2SiO4AGBUT_TestAll: Perform unit tests of basic radiation-transport routines
!   dust_Mg2SiO4AGB_1DchangeUTsimple: Perform simple unit tests for dust_Mg2SiO4AGB_1Dchange
!   dust_Mg2SiO4AGB_1DchangeUT: Perform unit tests for dust_Mg2SiO4AGB_1Dchange
!
! MODIFICATION HISTORY:
!   2017-05-17 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------**************************---------------------------------------------------------------
subroutine dust_Mg2SiO4AGB_1DchangeUT(TestPrefix, nPt, YoldFrac, rhomin, rhomax, Tmin, Tmax)
!---------------------------------------------------------------------------------------------------
! NAME:
!   dust_Mg2SiO4AGB_1DchangeUT ('dust_Mg2_Si_O4_AGB_1D_change_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for dust_Mg2SiO4AGB_1Dchange.
!
! CATEGORY:
!   Dust, testing
!
! CALLING SEQUENCE:
!   call dust_Mg2SiO4AGB_1DchangeUT(TestPrefix, nPt, YoldFrac, rhomin, rhomax, Tmin, Tmax)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nPt:          ('number_Points) integer with number of points in test vectors
!   YoldFrac:     ('Y_old_Fraction') real with fraction of dust concentration,
!                 usually: 0.0 <= YoldFrac <=1.0
!   Tmin:         ('Temperature_minimum') real with minimum temperature
!   Tmax:         ('Temperature_maximum') real with maximum temperature
!   rhomin:       ('rho_minimum') real with minimum density
!   rhomax:       ('rho_maximum') real with maximum density
!
! OUTPUT:
!   None
!
! ROUTINES:
!   dust_Mg2SiO4AGB_1Dchange: Calculate the Mg2SiO4-dust concentration at the new time level
!   UT_Assert:             Check array or relation between two arrays
!   UT_PrintSeparator:     Print separator line
!
! MODULES:
!   dust_Mg2SiO4AGB_module: Routines for Mg2SiO4 dust for AGB stars
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   By calling unit-test assert routines global counters are incremented.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call dust_Mg2SiO4AGB_1DchangeUT('Mg2SiO4AGB', 13, 0.5, 1.0E-12, 1.0E-12, 800.0, 1400.0)
!
! MODIFICATION HISTORY:
!   2017-05-17 (B.F. Uppsala) First version
!   2017-05-18 (B.F.) Add parameters rhomin, rhomax, YoldFrac
!---------------------------------------------------------------------------------------------------
use dust_Mg2SiO4AGB_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: srk=kind(0.0E+00), drk=kind(0.0D+00)
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nPt
real(kind=srk),            intent(in)   :: YoldFrac, rhomin, rhomax, Tmin, Tmax
!
! --- Local variables ---
integer, parameter                      :: nquc=3
character(len=128)                      :: TestName
character(len=80)                       :: YoldFracStr, rhominStr, rhomaxStr, TminStr, TmaxStr, &
                                           dtimeStr
integer                                 :: i, iquc
real(kind=drk), parameter               :: PMASS=1.6726E-24_drk
real(kind=drk)                          :: dtime, epsint, R0Dust, &
                                           EPSSI, HEH, HELPSI
real(kind=drk), dimension(1:nquc)       :: CNDNH
real(kind=drk), dimension(1:nPt)        :: rho, T
real(kind=drk), dimension(1:nPt,nquc)   :: Yold, Yold0, Ynew
real(kind=drk), dimension(1:nPt,7)      :: DV_scratch
!---------------------------------------------------------------------------------------------------
!
! --- Set abundances locally as in dust_Mg2SiO4AGB_1Dchange ---
EPSSI  = 3.550E-05_drk
HEH    = 10.0_drk**(-1.00E00_drk)
HELPSI = EPSSI / ( PMASS*( 1.0_drk + HEH*4.0_drk ) )
!
!
! --- Set start values for the concentration ---
do i=1, nPt
  do iquc=1, nquc
!    Yold0(i, iquc)=1.0E+03_drk
!    Yold0(i, iquc)=1.0001E+00_drk*HELPSI
!    Yold0(i, iquc)=0.5_drk*HELPSI
    Yold0(i, iquc)=real(YoldFrac, kind=drk) * HELPSI
  end do ! iquc
end do ! i
!
! --- Compute the density vector (exponential increase) ---
do i=1, nPt
!  rho(i)=1.0E-12_drk
  rho(i)=exp(real(i-1, kind=drk)/real(nPt-1, kind=drk) * log(real(rhomax/rhomin, kind=drk))) * &
         real(rhomin, kind=drk)
end do ! i
!
! --- Compute the temperature vector (linear increase) ---
do i=1, nPt
  T(i)  =    real(i-1, kind=drk)/real(nPt-1, kind=drk) *     real(Tmax-Tmin    , kind=drk)      + &
         real(Tmin, kind=drk)
end do ! i
!
do iquc=1, nquc
  CNDNH(iquc)=1.0E-15_drk * 10.0_drk**(iquc-1)
end do ! iquc
epsint=1.0D-03

! --- Construct global strings used for test names ---
write(YoldFracStr, '(1PE10.3)') YoldFrac ; YoldFracStr=adjustl(YoldFracStr)
write(rhominStr  , '(1PE9.2)' ) rhomin   ; rhominStr  =adjustl(rhominStr  )
write(rhomaxStr  , '(1PE9.2)' ) rhomax   ; rhomaxStr  =adjustl(rhomaxStr  )
write(TminStr    , '(F9.1)'   ) Tmin     ; TminStr    =adjustl(TminStr    )
write(TmaxStr    , '(F9.1)'   ) Tmax     ; TmaxStr    =adjustl(TmaxStr    )
!
call UT_PrintSeparator()
!
!
dtime =1.0E+03_drk
write(dtimeStr , '(1PE8.1)') dtime   ; dtimeStr =adjustl(dtimeStr  )
!
Yold=Yold0
call dust_Mg2SiO4AGB_1Dchange(nPt, nquc, Yold, dtime, rho, T, CNDNH, epsint, &
                              R0dust, Ynew, DV_scratch)
!
TestName=trim(TestPrefix) // &
         '_dtime='    // trim(dtimeStr   ) // &
         '_YoldFrac=' // trim(YoldFracStr) // &
         '_rho='      // trim(rhominStr  ) // '-' // trim(rhomaxStr) // &
         '_T='        // trim(TminStr    ) // '-' // trim(TmaxStr  ) // &
         '_ValuesArePositive'
!
call UT_Assert('>=', TestName, real(Ynew, kind=srk), 0.0E+00)
!
if (UT%AllVecInPrint_flag) then
  write(6,'(A,1PE13.6)') 'HELPSI=', HELPSI
  call UT_PrintArray('Yol='   , real(Yold0(:,1:1), kind=srk))
  call UT_PrintArray('rho=   ', real(rho, kind=srk))
  call UT_PrintArray('T=     ', real(T, kind=srk))
  call UT_PrintArray('CNDNH= ', real(CNDNH, kind=srk))
endif
!
TestName=trim(TestPrefix) // &
         '_dtime='    // trim(dtimeStr   ) // &
         '_YoldFrac=' // trim(YoldFracStr) // &
         '_rho='      // trim(rhominStr  ) // '-' // trim(rhomaxStr) // &
         '_T='        // trim(TminStr    ) // '-' // trim(TmaxStr  ) // &
         '_ValuesAreBelowLimit'
!
call UT_Assert('<=', TestName, real(Ynew, kind=srk), real(HELPSI, kind=srk))
!
!
dtime =1.0E+09_drk
write(dtimeStr , '(1PE8.1)') dtime   ; dtimeStr =adjustl(dtimeStr  )
!
Yold=Yold0
call dust_Mg2SiO4AGB_1Dchange(nPt, nquc, Yold, dtime, rho, T, CNDNH, epsint, &
                              R0dust, Ynew, DV_scratch)
!
TestName=trim(TestPrefix) // &
         '_dtime='    // trim(dtimeStr   ) // &
         '_YoldFrac=' // trim(YoldFracStr) // &
         '_rho='      // trim(rhominStr  ) // '-' // trim(rhomaxStr) // &
         '_T='        // trim(TminStr    ) // '-' // trim(TmaxStr  ) // &
         '_ValuesArePositive'
!
call UT_Assert('>=', TestName, real(Ynew, kind=srk), 0.0E+00)
!
TestName=trim(TestPrefix) // &
         '_dtime='    // trim(dtimeStr   ) // &
         '_YoldFrac=' // trim(YoldFracStr) // &
         '_rho='      // trim(rhominStr  ) // '-' // trim(rhomaxStr) // &
         '_T='        // trim(TminStr    ) // '-' // trim(TmaxStr  ) // &
         '_ValuesAreBelowLimit'
!
call UT_Assert('<=', TestName, real(Ynew, kind=srk), real(HELPSI, kind=srk))
!
end subroutine dust_Mg2SiO4AGB_1DchangeUT


!----------********************************---------------------------------------------------------
subroutine dust_Mg2SiO4AGB_1DchangeUTsimple(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   dust_Mg2SiO4AGB_1DchangeUTsimple ('dust_Mg2_Si_O4_AGB_1D_change_Unit_Test_simple')
!
! PURPOSE:
!   Perform simple unit tests for dust_Mg2SiO4AGB_1Dchange.
!
! CATEGORY:
!   Dust, testing
!
! CALLING SEQUENCE:
!   call dust_Mg2SiO4AGB_1DchangeUTsimple(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   dust_Mg2SiO4AGB_1Dchange: Calculate the Mg2SiO4-dust concentration at the new time level
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   dust_Mg2SiO4AGB_module: Routines for Mg2SiO4 dust for AGB stars
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   By calling unit-test assert routines global counters are incremented.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call dust_Mg2SiO4AGB_1DchangeUTsimple('Mg2SiO4AGB')
!
! MODIFICATION HISTORY:
!   2017-05-17 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use dust_Mg2SiO4AGB_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=kind(0.0D+00)
integer, parameter                      :: nPt=13, nquc=3
character(len=128)                      :: TestName
integer                                 :: i, iquc
real(kind=drk), parameter               :: PMASS=1.6726E-24_drk
real(kind=drk)                          :: dtime, epsint, R0Dust, &
                                           EPSSI, HEH, HELPSI
real(kind=drk), dimension(1:nquc)       :: CNDNH
real(kind=drk), dimension(1:nPt)        :: rho, T
real(kind=drk), dimension(1:nPt,nquc)   :: Yold, Ynew
real(kind=drk), dimension(1:nPt,7)      :: DV_scratch
!---------------------------------------------------------------------------------------------------
!
! --- Set abundances locally as in dust_Mg2SiO4AGB_1Dchange ---
EPSSI  = 3.550E-05_drk
HEH    = 10.0_drk**(-1.00E00_drk)
HELPSI = EPSSI / ( PMASS*( 1.0_drk + HEH*4.0_drk ) )
write(6,'(A,1PE13.6)') 'HELPSI=', HELPSI
!
do i=1, nPt
  do iquc=1, nquc
!    Yold(i,iquc)=1.0E+03_drk
    Yold(i,iquc)=0.5_drk*HELPSI
!    Yold(i,iquc)=1.0001_drk*HELPSI
  end do ! iquc
end do ! i
write(6,'(A)') 'Yold='
write(6,'(13E14.6)') Yold(:,1)

!dtime =1.0E+06_drk
dtime =1.0E+03_drk
do i=1, nPt
  rho(i)=1.0E-12_drk
end do ! i
write(6,'(A)') 'rho='
write(6,'(13E14.6)') rho

do i=1, nPt
  T(i)=real(i-1, kind=drk)/real(nPt-1, kind=drk) * 0.6E+03_drk + 0.8E+03_drk
end do ! i
write(6,'(A)') 'T='
write(6,'(13E14.6)') T

do iquc=1, nquc
  CNDNH(iquc)=1.0E-15_drk * 10.0_drk**(iquc-1)
end do ! iquc
write(6,'(A)') 'CNDNH='
write(6,'(13E14.6)') CNDNH

epsint=1.0D-03
!
call dust_Mg2SiO4AGB_1Dchange(nPt, nquc, Yold, dtime, rho, T, CNDNH, epsint, &
                              R0dust, Ynew, DV_scratch)
!
TestName=trim(TestPrefix) // &
         '_ValuesArePositive'
!
call UT_Assert('>=', TestName, real(Ynew, kind=srk), 0.0E+00)
!
TestName=trim(TestPrefix) // &
         '_ValuesAreBelowLimit'
!
call UT_Assert('<=', TestName, real(Ynew, kind=srk), real(HELPSI, kind=srk))
!
end subroutine dust_Mg2SiO4AGB_1DchangeUTsimple


!----------*************************----------------------------------------------------------------
subroutine dust_Mg2SiO4AGBUT_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_Mg2SiO4AGBUT_TestAll ('dust_Mg2_Si_O4_AGB_Unit_Test_Test_All')
!
! PURPOSE:
!   Perform unit tests of Mg2SiO4AGB-dust routines.
!
! CATEGORY:
!   Dust, testing
!
! CALLING SEQUENCE:
!   call dust_Mg2SiO4AGBUT_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_baserad_RotationMatrixUT: Perform unit tests for rhd_baserad_RotationMatrix
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Perform all unit tests of Mg2SiO4AGB dust routine.
!
! EXAMPLE:
!   call dust_Mg2SiO4AGBUT_TestAll()
!
! MODIFICATION HISTORY:
!   2017-05-17 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='Mg2SiO4AGB'
!
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13,-0.001 , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 0.0   , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 0.001 , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 0.1   , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 0.5   , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 0.95  , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 1.0   , 1.0E-12, 1.0E-12,  800.0, 1400.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_TempSeq_', 13, 1.001 , 1.0E-12, 1.0E-12,  800.0, 1400.0)
!
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_rhoseq_' , 13, 0.5   , 1.0E-15, 1.0E-09,  900.0,  900.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_rhoseq_' , 13, 0.5   , 1.0E-15, 1.0E-09, 1000.0, 1000.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_rhoseq_' , 13, 0.5   , 1.0E-15, 1.0E-09, 1130.0, 1130.0)
call dust_Mg2SiO4AGB_1DchangeUT( &
             trim(TestPrefix) // '_rhoseq_' , 13, 0.5   , 1.0E-15, 1.0E-09, 1400.0, 1400.0)
!
! call dust_Mg2SiO4AGB_1DchangeUTsimple(TestPrefix)
!
end subroutine dust_Mg2SiO4AGBUT_TestAll

end module dust_Mg2SiO4AGBUT_module
