!=======================================================================
      module dust_Mg2SiO4AGB_module
!=======================================================================
C
C MODIFICATION HISTORY:
C   2011-12-02 (S.H.) Adapted from 1D code
C   2017-05-16 (S.H.) Limit maximum of dust concentration
C   2017-05-18 (S.H.) DUSTPH: SiO or 2 Mg as key species
C
      contains
C
C-----------------------------------------------------------------------
      SUBROUTINE dust_Mg2SiO4AGB_1Dchange(
     #            NPT,NEQ,YOLD,TSTEP,RHOINT,TEMINT,
     #            CNDNH,EPSINT,R0DUST,YNEW,DV)
C-----------------------------------------------------------------------
C FORTRAN-Subroutine:
C
C Author  : S. Hoefner
C Created : 08-MAR-2012 
C Modified: 2017-05-16 (S.H.) Limit maximum of dust concentration
C           2017-05-18 (S.H.) Limit input values of dust concentration
C           2017-05-18 (S.H.) New DUSTPH version: SiO or Mg growth rate
C
C Description:
C
C    calculates the dust moments at the new time level
C    from values at old time level
C
C      - integration of grain growth equations if appropriate
C      - no dust if conditions are not suitable for dust
C
C-----------------------------------------------------------------------
C Input variables:
C
C    NPT       : number of points
C    TSTEP     : time step 
C
C    YOLD      : K3/RHO at old time level
C    RHOINT    : gas density
C    TEMINT    : gas temperature
C
C Output variables:
C
C    YNEW      : K3/RHO at new time level
C
C Local variables and constants:
C
C    TCH       : gas  temperature for chemistry, minimum value 400 K
C    TDCH      : dust temperature for chemistry, minimum value 400 K
C
C-----------------------------------------------------------------------
C Subroutines and functions referenced:
C
C    DUSTPH: nucleation and grain growth rates
C
C-----------------------------------------------------------------------
C Further comments:
C
C  - T_dust = T_gas  
C  - minimal temperatures in STAUPH: 400 K (chemistry)
C
C***********************************************************************
C-----------------------------------------------------------------------
C Declarations:

      IMPLICIT NONE

C Double Precision:

      DOUBLE PRECISION
     # Z0,Z1,Z2,Z3,Z4,
     # ZNL,PMASS,SMASS

C Numerical constants:

      PARAMETER (Z0=0., Z1=1., Z2=2., Z3=3., Z4=4.)

C Constants and parameters for dust formation:

      PARAMETER ( ZNL   = 1.D+03 ) 
      PARAMETER ( PMASS = 1.6726E-24 )          ! proton mass

      INTEGER 
     #  NPT,NEQ,L,J,
     #  ITCH,ITDCH,IXNOC,IXNSI,IXNMG,ISDUS,ITAU3,IDUST

      PARAMETER (ITCH=1,ITDCH=2,IXNOC=3,IXNSI=4,IXNMG=5,
     #           ISDUS=6,ITAU3=7)

      DOUBLE PRECISION
     #  YOLD(NPT,NEQ),YNEW(NPT,NEQ),TSTEP,
     #  RHOINT(NPT),TEMINT(NPT),HEH,EPSINT,R0DUST,
     #  EPSC,EPSO,EPSMG,EPSSI,HELPOC,HELPSI,HELPMG,
     #  XK0D,XK2D,XK3K0,CNDNH(NEQ),
     #  DV(NPT,7)

      CHARACTER*70 TXDUS

C-----------------------------------------------------------------------
C Set abundances locally, just for testing ...

      EPSC   = 3.303E-04               
      EPSO   = 6.607E-04   
      EPSSI  = 3.550E-05   

      EPSMG  = 3.800E-05    ! solar
      IDUST  = -2           ! Mg in growth rate

!!!      EPSMG  = 7.100E-05    ! eps_Si*2 (x=1)!!!
!!!      IDUST  = -1           ! SiO rate, testing ...

      HEH    = 10.D00**(-1.00D00)      ! abundance of He

C     auxiliary variables

      HELPOC = ( EPSO - EPSC ) / ( PMASS*( 1.0D00 + HEH*4.0 ) )
      HELPSI = EPSSI / ( PMASS*( 1.0D00 + HEH*4.0 ) )
      HELPMG = EPSMG / ( PMASS*( 1.0D00 + HEH*4.0 ) )

      SMASS = PMASS *( 1.0D00 + HEH*Z4 )

C-----------------------------------------------------------------------
C Calculate moments at new time level:

      DO J=1,NEQ   ! loop over "species"

        DO L=1,NPT

!         --- Enforce upper limit for f_cond (2017-05-16 S.H.) ---

          IF (IDUST.EQ.-2) THEN 
            YOLD(L,J) = MIN( YOLD(L,J), HELPMG*(1.0D+00-1.0D-02)/Z2 )
          ELSE
            YOLD(L,J) = MIN( YOLD(L,J), HELPSI*(1.0D+00-1.0D-02) )
          END IF

          YOLD(L,J) = MAX( YOLD(L,J), Z0 )

!         --- and lower limit: no negative f_cond (2017-05-18 S.H.) ---

C         gas and dust temperatures

          DV(L,ITCH)  = MAX ( TEMINT(L), 4.0D+02 ) 
          DV(L,ITDCH) = DV(L,ITCH)               !!! T_dust = T_gas !!!

C         number densities of O-C, Si, Mg in the gas phase

          DV(L,IXNOC) = ( HELPOC - 4.0*YOLD(L,J) ) * RHOINT(L) 
          DV(L,IXNSI) = ( HELPSI - 1.0*YOLD(L,J) ) * RHOINT(L)
          DV(L,IXNMG) = ( HELPMG - 2.0*YOLD(L,J) ) * RHOINT(L)

C         a few adjustments, just in case ...

          DV(L,IXNOC)  = MAX ( DV(L,IXNOC), Z0 )
          DV(L,IXNSI)  = MAX ( DV(L,IXNSI), Z0 )
          DV(L,IXNMG)  = MAX ( DV(L,IXNMG), Z0 )

        END DO

C       compute growth rates

        CALL DUSTPH
     #     ( NPT,DV(1,ITCH),RHOINT,
     #       DV(1,IXNOC),DV(1,IXNSI),DV(1,IXNMG),DV(1,ITDCH),IDUST,
     #       DV(1,ITAU3),DV(1,ISDUS),R0DUST,TXDUS )


C       integration of grain growth equations between two time levels

        DO L=1,NPT

C         replace moments K0/rho and K2/rho in the source terms for K3/rho

C                  replacing K0/rho by a constant (parameter)

          XK0D   = CNDNH(J) / SMASS

          XK3K0  = YOLD(L,J) / XK0D
          XK3K0  = MAX( XK3K0, Z0 )

C                  replacing K2 by K0 * N**2/3

          XK2D   = XK0D * ( XK3K0 + ZNL )**(Z2/Z3)

C         check if dust equations should be integrated in this interval

          IF ( ( DV(L,ITAU3) .LE. Z0 ) .AND.
     #       ( ( YOLD(L,J) .LT. (EPSINT*HELPSI) ) .OR. 
     #           ( YOLD(L,J) .LT. (ZNL*XK0D) ) ) ) THEN
          
C           no dust here and now

            YNEW(L,J) = Z0

          ELSE

C           new values of the moments

            YNEW(L,J) = YOLD(L,J) + DV(L,ITAU3) * XK2D  * TSTEP
            YNEW(L,J) = MAX( YNEW(L,J), Z0 )

!           --- Enforce upper limit for f_cond (2017-05-16 S.H.) ---

            IF (IDUST.EQ.-2) THEN
              YNEW(L,J) = MIN( YNEW(L,J), HELPMG*(1.0D+00-1.0D-02)/Z2 )
            ELSE
              YNEW(L,J) = MIN( YNEW(L,J), HELPSI*(1.0D+00-1.0D-02) )
            ENDIF

          END IF

        END DO

      END DO

C-----------------------------------------------------------------------
C End of dust_Mg2SiO4AGB_1Dchange

      RETURN
      END SUBROUTINE


C======================================================================
C
C   Dust microphysics & chemistry: O-rich composition
C
C   subroutine DUSTPH: from materd_ol_awx1d327_sio_mg_notridi.f
C                      28-JAN-2015: DARWIN version
C                      17-MAY-2017: CO5BOLD version
C
C======================================================================

      SUBROUTINE  DUSTPH(
     #                NPT,TEMP,RHOG,XNOC,XNSI,XNMG,TDUS,IDUST,
     #                TAU3,SDUS,R0,TXDUST )
C
C----------------------------------------------------------------------
C Author :   Susanne Hoefner
C Date   :   29-MAR-2006
C            03-OCT-2006: corrected error in AW
C                         version for x=1 (forsterite)
C            28-JAN-2015: added growth rate with Mg as key species
C            17-MAY-2017: CO5BOLD version
C
C Description :
C
C     calculates the net grain growth rates for Mg2SiO4 grains
C
C     IDUST can be used to select the key species:
C           = -1 -> SiO
C           = -2 -> Mg
C
C......................................................................
C
C  input:
C
C     NPT    :  number of points
C     TEMP   :  gas temperature
C     RHOG   :  gas density  [g/cm^3]
C     XN..   :  number densities of relevant elements in the gas phase
C               note: XNOC = XNO_gas - XNCO
C     TDUS   :  dust temperature
C
C  output:
C
C     SDUS   :  supersaturation ratio at T_dust
C     TAU3   :  net growth rate
C     R0     :  monomer radius
C     TXDUST :  identification text
C
C  parameters:
C
C     AW     :  'atomic weight' of dust monomers
C     RHOD   :  density of condensed material
C     STICK  :  sticking coefficent
C
C.......................................................................
C
C  subroutines called :
C
C     LNKPOL :  ln(K_p) for olivine
C
C.......................................................................
C
C  further comments :
C
C     molecules considered: H2O, SiO
C           (note: CO is already assumed subtracted from n_O)
C
C-----------------------------------------------------------------------
C declarations and parameters:

      IMPLICIT NONE

      INTEGER  NPT,L,IDUST

      DOUBLE PRECISION
     #   AW,RHOD,STICK,R0,R02,R024PI,WTTD,
     #   CT3SIO,CT3MG,XLNBLZ,XLNTG,ELNSIO,ELNMG,SDSIO,SDMG,
     #   XNH2,XNH2O,XNSIO,XLNH2,XLNH2O,XLNSIO,XLNMG

      DOUBLE PRECISION
     #   TEMP(NPT),RHOG(NPT),TDUS(NPT),
     #   XNOC(NPT),XNSI(NPT),XNMG(NPT),
     #   TAU3(NPT),SDUS(NPT),
     #   XLKPOL(NPT),X1(NPT)

      CHARACTER*10 TXRD
      CHARACTER*50 TXHELP
      CHARACTER*70 TXDUST

C     numerical and physical constants

      DOUBLE PRECISION
     #   PI,BOLTZ,PMAS,Z0,Z1,Z2,Z3,Z4,Z4PI,HEH,SMASS

      PARAMETER (PI     = 3.1415926535898,
     #           BOLTZ  = 1.38062E-16,
     #           PMAS   = 1.6726E-24)

      PARAMETER (Z0=0., Z1=1., Z2=2., Z3=3., Z4=4., 
     #           Z4PI  = Z4*PI,
     #           HEH   = 1.D-01,
     #           SMASS = PMAS *( 1.0D00 + HEH*Z4 ))

C-----------------------------------------------------------------------
C setting various constants:

C     sticking coeff., grain material density, atomic weight of monomer

      STICK = Z1           ! whatever ...
      RHOD  = 3.27         ! Mg2SiO4 density (Jager et al. 2003)
      AW    = Z2*24.0 + 28.0 + Z4*16.0 

      TXHELP =
     #   'DUSTPH_O_RD  : O-rich (H2O, SiO; Mg2SiO4; rates: SiO, Mg'
      WRITE (TXRD, '(1PE10.3)') RHOD
      TXDUST = TXHELP // ', RHOD=' // TXRD

C     calculate monomer radius and other useful quantities

      R0     = ( Z3 * AW * PMAS / ( Z4PI * RHOD ) )**(Z1/Z3)
      R02    = R0**2
      R024PI = Z4PI * R02

      CT3SIO = R024PI * STICK * SQRT ( BOLTZ / ( Z2*PI*PMAS*44.0 ) )
      CT3MG  = R024PI * STICK * SQRT ( BOLTZ / ( Z2*PI*PMAS*24.0 ) )

      XLNBLZ = LOG(BOLTZ)

C-----------------------------------------------------------------------
C chemistry and net growth rate:

      DO L=1,NPT

C       gas phase chemistry (rather simple):

        XNH2  = RHOG(L) / (Z2 * SMASS)   ! n_H2
        XNSIO = XNSI(L)                  ! all Si in SiO
        XNH2O = XNOC(L) - XNSIO          ! H2O : (O-CO) - SiO

        X1(L) = Z1 

C       partial pressures (number densities) of SiO and Mg  
C                         in equilibrium with dust phase
C           ... better do this in ln ...

        CALL  LNKPOL ( 1,TDUS(L),X1(L),XLKPOL(L) )

        XLNH2  = LOG(XNH2)
        XLNH2O = LOG(XNH2O)
        XLNSIO = LOG(XNSIO)
        XLNMG  = LOG(XNMG(L))

        XLNTG  = LOG(TEMP(L))      ! to convert pps to number densities

C       number densities of SiO and Mg in CE with Mg2SiO4 (ln)

        ELNSIO = Z3*XLNH2 - Z3*XLNH2O - Z2*XLNMG
     #              - XLKPOL(L)
     #              - Z3* ( XLNTG + XLNBLZ )

        ELNMG  = ( Z3*XLNH2 - Z3*XLNH2O - XLNSIO
     #              - XLKPOL(L)
     #              - Z3* ( XLNTG + XLNBLZ ) ) / Z2

C       supersaturation ratios for SiO and Mg over Mg2SiO4

        SDSIO   = EXP( XLNSIO - ELNSIO )
        SDMG    = EXP( XLNMG  - ELNMG  )

C       assembling the net growth rate for Mg2SiO4

        WTTD   = SQRT( TEMP(L)/TDUS(L) )

        IF ( IDUST .EQ. -2 ) THEN    ! Mg as key species

          SDUS(L) = SDMG

C         net growth rate (addition of 2x Mg to grain surface)

          TAU3(L)   = CT3MG
     #              * SQRT (TEMP(L)) * XNMG(L)
     #              * ( Z1 - Z1/SDUS(L) * WTTD ) / Z2

        ELSE    ! SiO rate ... default during test phase

          SDUS(L) = SDSIO

C         net growth rate (addition of SiO to grain surface)

          TAU3(L)   = CT3SIO
     #              * SQRT (TEMP(L)) * XNSIO
     #              * ( Z1 - Z1/SDUS(L) * WTTD )

        END IF

      END DO

C----------------------------------------------------------------------

      RETURN
      END SUBROUTINE

C----------------------------------------------------------------------


      SUBROUTINE LNKPOL ( N,TEMP,XMGFEO,XLKPOL )
C
C----------------------------------------------------------------------
C File   :   LNKPOL.FOR
C Author :   Susanne Hoefner
C Date   :   02-APR-2006
C            01-DEC-2011  CO5BOLD version
C
C Description :
C
C     calculating ln(Kp_olivine) for given T and x (Mg/(Mg+Fe))
C......................................................................
C
C  input :
C
C     N          : number of points
C     TEMP       : temperature
C     XMGFEO     : Mg/(Fe+Mg) in olivine
C
C  output:
C
C     XLKPOL     : ln(Kp_olivine)
C
C  parameters:
C
C     DGEFOL     : Gibbs energy of formation of olivine
C     DGEF(i,j)  : coeff. j for species i in polynomial fit
C
C.......................................................................
C
C  called subroutines :
C     *** none ***
C.......................................................................
C
C  further comments :
C
C     following Gail & Sedlmayr (1999, A&A 347, 594)
C     polynomial fits for free energies from Sharp & Huebner (1990)
C
C     ln (Kp_ol) = delta G / (R*T)
C
C                  delta G =     delta G_SiO + 3 delta G_H2O
C                            - 3 delta G_H2  -   delta G_ol
C
C     delta G_ol  = x * delta G_for + (1-x) * delta G_fay + delta G_mix
C
C     delta G_mix = 2 R*T ( x ln x + (1-x) ln (1-x ))
C
C-----------------------------------------------------------------------
C declarations and parameters:

      IMPLICIT NONE

      INTEGER   N,I,L,NSPEC,NCOEF,
     #          ISIO,IH2O,IH2,IFOR,IFAY,IGA,IGB,IGC,IGD,IGE

      PARAMETER ( NSPEC=5, NCOEF=5,
     #            ISIO=1, IH2O=2, IH2=3, IFOR=4, IFAY=5,
     #            IGA=1, IGB=2, IGC=3, IGD=4, IGE=5 )

      DOUBLE PRECISION
     #    TEMP(N),XMGFEO(N),XLKPOL(N),
     #    DGEF(NSPEC,NCOEF),DGEFOL(NCOEF),
     #    DGMIX,RCALMOL,ADYCM2,Z0,Z1,Z2,Z3

      PARAMETER ( Z0=0., Z1=1., Z2=2., Z3=3. )

      PARAMETER ( RCALMOL = 1.98D+00,         ! gas const. [cal/mole/K]
     #            ADYCM2 = Z1/1.01325D+06 )   ! [atm / (dyn/cm2)]

C     coefficients for polynomials (Sharp & Huebner 1990):
C        delta G = a/T + b + c*T + d*T**2 + e**3  [cal/mole]
C                                                 -> pressures in [atm]

      DATA (DGEF(ISIO,I), I=1,NCOEF) /  1.82780E+05, -1.92839E+05,
     #      3.03804E+01,  4.16079E-04, -1.91035E-08 /

      DATA (DGEF(IH2O,I), I=1,NCOEF) /  8.66184E+05, -2.27857E+05,
     #      5.61473E+01,  7.62548E-04, -4.95254E-08 /

      DATA (DGEF(IH2, I), I=1,NCOEF) /  4.25321E+05, -1.07123E+05,
     #      2.69980E+01,  5.48280E-04, -3.81498E-08 /

      DATA (DGEF(IFOR,I), I=1,NCOEF) /  7.52334E+04, -9.38369E+05,
     #      2.47581E+02, -3.14980E-03,  0.00000E-00 /

      DATA (DGEF(IFAY,I), I=1,NCOEF) /  6.84477E+04, -9.02146E+05,
     #      2.51045E+02, -4.44028E-03,  0.00000E-00 /

C-----------------------------------------------------------------------

      DO L=1,N

        DO I=1,NCOEF

C         calculating polynomial coefficients for olivine:
C             SiO + 3 H2O - 3 H2 - x forsterite - (1-x) fayalite

          DGEFOL(I) = ( DGEF(ISIO,I) + Z3*DGEF(IH2O,I) - Z3*DGEF(IH2,I)
     #                    - XMGFEO(L)        * DGEF(IFOR,I)
     #                    - (Z1 - XMGFEO(L)) * DGEF(IFAY,I) )
     #                / RCALMOL
        END DO

        IF ( (XMGFEO(L).EQ.Z1) .OR. (XMGFEO(L).EQ.Z0) ) THEN
          DGMIX  = Z0
        ELSE
          DGMIX  = Z2 * ( XMGFEO(L)      * LOG(XMGFEO(L))
     #                  + (Z1-XMGFEO(L)) * LOG(Z1-XMGFEO(L)) )
        END IF

C       calculating ln(Kp_ol) = delta G /(R*T)
C                               ( polynomial(T) - G_mix )

        XLKPOL(L) =   DGEFOL(IGA)/TEMP(L)**2    + DGEFOL(IGB)/TEMP(L)
     #              + DGEFOL(IGC)
     #              + DGEFOL(IGD)*TEMP(L)       + DGEFOL(IGE)*TEMP(L)**2
     #              - DGMIX
     #              + Z3* LOG(ADYCM2)    ! converting to [dyn/cm2]

      END DO

C-----------------------------------------------------------------------

      RETURN
      END SUBROUTINE

C-----------------------------------------------------------------------

!=======================================================================
      end module dust_Mg2SiO4AGB_module
!=======================================================================
