C------------********************---------------------------------------
      module dust_momentc2_module
C-----------------------------------------------------------------------
C
C MODIFICATION HISTORY:
C   2003-08-25 (S.H.) Adapted from 1D code
C   2007-12-18 (B.F.) Use constants from const_module
C
C-----------------------------------------------------------------------
      contains


C-----------------------------------------------------------------------
      SUBROUTINE MOMENT (NPT,YOLD,TSTEP,RHOINT,TEMINT,CONSNC,
     #                   HEH,SGMIN,EPSINT,R0DUST,YNEW,DV,SCRD)
C-----------------------------------------------------------------------
C FORTRAN-Subroutine:
C
C File    : MOMENT.FOR
C Author  : S. Hoefner
C Created : 25-AUG-2003 
C Modified: 
C
C Description:
C
C    calculates the dust moments at the new time level (TNEW)
C    from values at old time level (TOLD)
C
C      - integration of moment equations if appropriate
C      - no dust if conditions are not suitable for dust
C
C-----------------------------------------------------------------------
C Input variables:
C
C    NPT       : number of points
C
C    YOLD(IK0) : K0/RHO at TOLD
C    YOLD(IK1) : K1/RHO at TOLD
C    YOLD(IK2) : K2/RHO at TOLD
C    YOLD(IK3) : K3/RHO at TOLD
C
C    TSTEP     : time step 
C
C    RHOINT    : gas density
C    TEMINT    : gas temperature
C
C Output variables:
C
C    YNEW(IK0) : K0/RHO at TOLD
C    YNEW(IK1) : K1/RHO at TOLD
C    YNEw(IK2) : K2/RHO at TOLD
C    YNEW(IK3) : K3/RHO at TOLD
C
C Local variables and constants:
C
C    TCH       : gas  temperature for chemistry, minimum value 400 K
C    TDCH      : dust temperature for chemistry, minimum value 400 K
C
C    XNC       : total number density of C in gas (sum of all species)
C
C    TAUI      : grain growth rate / d  (see STAUPH)
C    SJK       : nucleation rate        (see STAUPH)
C
C-----------------------------------------------------------------------
C Subroutines and functions referenced:
C
C    STAUPH: nucleation and grain growth rates
C
C-----------------------------------------------------------------------
C Further comments:
C
C  - T_dust = T_gas  
C
C  - minimal temperatures in STAUPH: 400 K (chemistry)
C  - minimal supersaturation ratio for nucleation (SGMIN)
C
C***********************************************************************
C-----------------------------------------------------------------------
C Declarations:

      IMPLICIT NONE
      INCLUDE 'DINDEX.INC'

C Double Precision:

      DOUBLE PRECISION
     # Z0,Z1,Z2,Z3,
     # ZNL,ZNL13,ZNL23

C Numerical constants:

      PARAMETER (Z0=0., Z1=1., Z2=2., Z3=3.)

C Materical constants and parameters for dust formation:

      PARAMETER ( ZNL   = 1.D+03, 
     #            ZNL13 = 1.D+01,
     #            ZNL23 = 1.D+02 )

      INTEGER 
     #  NPT,L,J,
     #  ITCH,ITDCH,IXNC,IXP1,IPSAT,IPVAP,ISGAS,ISDUS,ITAUI,ICCS,ISJK
      PARAMETER (ITCH=1, ITDCH=2,IXNC=3, IXP1=4, IPSAT=5,IPVAP=6,
     #           ISGAS=7,ISDUS=8,ITAUI=9,ICCS=10,ISJK=11)

      DOUBLE PRECISION
     #  YOLD(NPT,NEQ),YNEW(NPT,NEQ),TSTEP,
     #  RHOINT(NPT),TEMINT(NPT),CONSNC,HEH,SGMIN,EPSINT,R0DUST,
     #  SCRD(NPT,36),DV(NPT,11),DYDT(NEQ)

      CHARACTER*70 TXDUS

C-----------------------------------------------------------------------
C Calculate moments at TNEW:

      DO L=1,NPT

C       call chemistry and dust physics routines (COBOLD version)

        DV(L,ITCH)  = MAX ( TEMINT(L), 4.0D+02 ) 
        DV(L,ITDCH) = DV(L,ITCH)               !!! T_dust = T_gas !!!

        DV(L,IXNC)  = ( CONSNC - YOLD(L,IK3) ) * RHOINT(L)
        DV(L,IXNC)  = MAX ( DV(L,IXNC), Z0 )

      END DO

      CALL STAUPH ( NPT,DV(1,ITCH),RHOINT,DV(1,IXNC),DV(1,ITDCH),HEH,
     #              DV(1,IXP1),SCRD(1,1),DV(1,IPSAT),DV(1,IPVAP),
     #              DV(1,ISGAS),DV(1,ISDUS),
     #              DV(1,ITAUI),DV(1,ICCS),DV(1,ISJK),R0DUST,
     #              TXDUS )

      DO L=1,NPT

        IF ( DV(L,ISGAS) .LT. SGMIN ) DV(L,ISJK) = Z0 

C       check if dust equations should be integrated in this interval

        IF ( ( DV(L,ITAUI) .LE. Z0 ) .AND.
     #       ( ( YOLD(L,IK3) .LT. (EPSINT*CONSNC) ) .OR. 
     #         ( YOLD(L,IK3) .LT. (ZNL*YOLD(L,IK0)) ) ) ) THEN
          
C         no dust here and now

          DO J=1,NEQ
            YNEW(L,J) = Z0
          END DO

        ELSE

C       integration of moment equations between two time levels:

C         time derivatives of the normalized moments (source terms)

          DYDT(IK0) =                       DV(L,ISJK) / RHOINT(L)
          DYDT(IK1) =       DV(L,ITAUI) * YOLD(L,IK0) 
     #                            + ZNL13 * DV(L,ISJK) / RHOINT(L)
          DYDT(IK2) =  Z2 * DV(L,ITAUI) * YOLD(L,IK1) 
     #                            + ZNL23 * DV(L,ISJK) / RHOINT(L)
          DYDT(IK3) =  Z3 * DV(L,ITAUI) * YOLD(L,IK2) 
     #                            + ZNL   * DV(L,ISJK) / RHOINT(L)

C         new values of the moments

          DO J=1,NEQ
            YNEW(L,J) = YOLD(L,J) + DYDT(J) * TSTEP
            YNEW(L,J) = MAX( YNEW(L,J), Z0 )
          END DO

        END IF

      END DO

C-----------------------------------------------------------------------
C End of MOMENT (MOMENT.FOR).

      RETURN
      END SUBROUTINE


C======================================================================
C
C   Staub-Materialfunktionen: C2 - Version fuer COBOLD
C
C   File erstellt am  17.12.2002
C   Modifiziert am    17.01.2003
C
C   Enthaelt:  STAUPH_C2    (ohne Abl.)
C              ABKOC2
C              CHEMC2
C              DISKON_C2
C              NETRAT_2
C              NUKRAT_2
C              PSATC1
C              SPLINT & TRIDI
C
C======================================================================


C----------------------------------------------------------------------
      SUBROUTINE  STAUPH ( NPT,TEMP,RHOG,XNC,TDUS,HEH,
     #                     XP1,SCRD,PSAT,PVAP,SGAS,SDUS,
     #                     TAUI,CCS,SJK,R0,
     #                     TXDUST )
C----------------------------------------------------------------------
C File  :    STAUPH_C2
C Autor :    Susanne Hoefner
C Datum :    15.01.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     Berechnung der fuer die Staubkomponente relevanten Groessen 
C     fuer C-reiche Chemie
C......................................................................
C
C  Eingabe:
C
C     NPT    :  Anzahl der Punkte
C     TEMP   :  Gastemperatur
C     RHOG   :  Gasdichte
C     XNC    :  Anzahldichte der freien (d.h. weder in CO noch im 
C               Staub gebundenen) C-Atome: XNC = XNC_gas - XNCO
C     TDUS   :  Staubtemperatur
C     HEH    :  Elementhaeufigkeit von He
C
C  Ausgabe:
C
C     XP1    :  Monomerpartialdruck
C     PSAT   :  Saettigungsdruck bei Tgas
C     PVAP   :  Saettigungsdruck bei Tdust
C     SGAS   :  Uebersaettigungsverhaeltnis bei Tgas
C     SDUS   :  Uebersaettigungsverhaeltnis bei Tdust
C     TAUI   :  Netto-Wachstumsrate / DICL            !!!  Achtung !!!
C     CCS    :  krit. Clustergroesse
C     SJK    :  Keimbildungsrate
C     R0     :  Monomerradius
C     TXDUST :  Identifikationstext
C
C  Parameter :
C
C     AGW    :  Atomgewicht des staubbildenden Materials
C     RHOD   :  Dichte des kondensierten Materials
C     SIG    :  Oberflaechenspannung
C     NSIG   :  Parameter fuer die N-Abh. von SIG
C     STICK  :  Sticking coefficent fuer Ci
C     STICKH :  Sticking coefficent fuer CiHj
C     DICL   :  raeuml. Dimension der Cluster (3=kugelf.)
C
C     NATMAX :  max. Zahl der pro Reaktion angelagerten C-Atome
C     NHMAX  :  max. Zahl der H-Atome in den CiHj
C
C     BOLTZ  :  Boltzmannkonstante 
C     PMASS  :  Protonmasse
C
C.......................................................................
C
C  verwendete Unterprogramme :
C
C     CHEMC2 :  Berechnung der relevanten Partialdruecke aus dem 
C               chem. Glgw. zwischen H,H2,C,C2,C2H,C2H2
C     PSATC1 :  Saettigungsdruck von C 
C     ABKOC2 :  Abweichungskoeff. fuer NTE
C     NETRAT :  Netto-Wachstumsrate  
C     NUKRAT :  Keimbildungsrate  
C
C.......................................................................
C
C  weitere Kommentare :
C
C     verwendete Molekuele:  H2,C2,C2H,C2H2  (fuer eff. Partialdruecke)
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:
C     -------------------------

      use const_module, only: pi_drk, k_Bol_drk, amu_drk

      IMPLICIT NONE

C     Parameter fuer Chemie und eff. Partialdruecke

      INCLUDE 'CHPAR_CT.INC'

      INTEGER  NPT,NATMAX,NHMAX,I,J,L,NSIG

      PARAMETER ( NATMAX = 2, NHMAX = 2, NSIG = 5 )

      DOUBLE PRECISION
     #   AGW,RHOD,STICK(NATMAX),STICKH(NATMAX,NHMAX),
     #   Z0,Z1,Z2,Z3,Z03,Z2PI,Z4PI,PI,BOLTZ,PMASS,HEH,
     #   DICL,R0,R02,R024PI,CKI,CBET,FKOR(NATMAX,NHMAX),SIG,
     #   Z1SD,Z1SD2,WTTD

      DOUBLE PRECISION
     #   TEMP(NPT),RHOG(NPT),XNC(NPT),TDUS(NPT),
     #   XP1(NPT),SCRD(NPT,36),PSAT(NPT),PVAP(NPT),
     #   SGAS(NPT),SDUS(NPT),TAUI(NPT),CCS(NPT),SJK(NPT) 

      CHARACTER*70 TXDUST

c      PARAMETER ( PI     = 3.1415926535898, 
c     #            BOLTZ  = 1.38062E-16,
c     #            PMASS  = 1.6726E-24 )
      PARAMETER ( PI     = pi_drk,
     #            BOLTZ  = k_Bol_drk,
     #            PMASS  = amu_drk )

      PARAMETER ( Z0 = 0.0D00, Z1 = 1.0D00, Z2 = 2.0D00, Z3 = 3.0D00,
     #            Z03 = Z1/Z3, Z2PI = 2.0*PI, Z4PI = 4.0*PI )

      PARAMETER ( AGW  = 12.01115D00, RHOD = 2.25D00, SIG = 1.4D03,
     #            DICL = Z3 )

C.......................................................................

C     Materialkonstante und Hilfsgroessen
C     -----------------------------------

      TXDUST = 'STAUPH_C     : C-Chemie (H2,C2,C2H,C2H2)'

      STICK(1) = 0.37D00
      STICK(2) = 0.34D00
      STICKH (2,1) = 0.34D00
      STICKH (2,2) = 0.34D00 

      DO I=1,NATMAX
        IF ( STICK(I).NE.Z0 ) THEN
          DO J=1,NHMAX
            FKOR(I,J) = STICKH(I,J)/STICK(I) / SQRT( 1 + J/(I*AGW) )
          END DO
        END IF
      END DO

      R0     = ( Z3 * AGW * PMASS / ( Z4PI * RHOD ) )**Z03
      R02    = R0**2
      R024PI = Z4PI * R02
      CKI    = R024PI / DICL
      CBET   = Z1 / SQRT( BOLTZ * Z2PI * PMASS * AGW )

C.......................................................................

C     Chemie  (C-reich, Molekuele: H2, C2, C2H, C2H2)
C     -----------------------------------------------

      CALL  CHEMC2  ( NPT,TEMP,RHOG,XNC,HEH,XP1,SCRD )

C.......................................................................

C     Saettigungsdruck fuer Tgas (SJK) und Tdus (TAUI)
C     ------------------------------------------------

      CALL  PSATC1  ( NPT, TDUS, PVAP )
      CALL  PSATC1  ( NPT, TEMP, PSAT )

C.......................................................................

C     Hilfsvektoren fuer Keimbildungsrate
C     -----------------------------------

      DO L=1,NPT

C        Uebersaettigungsverhaeltnis fuer Tgas 

         SGAS(L)       = XP1(L) / PSAT(L)

C        effektive Partialdruecke fuer Keimbildungsrate
C        (fuer CE, T=Tgas, heteromolekulare Beitraege, NATMAX=2)

         SCRD(L,IPP1)   = XP1(L)
         SCRD(L,IPP2)   = SCRD(L,IPC2) + SCRD(L,IPC2H)  * FKOR(2,1) 
     #                                 + SCRD(L,IPC2H2) * FKOR(2,2)

      END DO

C     Keimbildungsrate 
C     ----------------

      CALL  NUKRAT ( NPT,TEMP,SGAS,PSAT,XP1,SCRD,
     #               NATMAX,STICK,CBET,R024PI,SIG,NSIG,
     #               CCS,SJK )

C.......................................................................

C     Hilfsvektoren fuer die Netto-Wachstumsrate
C     ------------------------------------------

C     NTE-Abweichungskoeff. fuer die Netto-Wachstumsrate

      CALL  ABKOC2  ( NPT, TEMP, TDUS, SCRD )

      DO L=1,NPT

C        Uebersaettigungsverhaeltnis fuer Tdus 

         SDUS(L)       = XP1(L) / PVAP(L)

         Z1SD   = Z1 / SDUS(L)
         Z1SD2  = Z1SD**2

         WTTD   = SQRT( TEMP(L)/TDUS(L) )

C        effektive Partialdruecke fuer Netto-Wachstumsrate
C        (NTE, heteromolekulare Beitraege, NATMAX=2)

         SCRD(L,IPP1)   = XP1(L) * ( Z1 - Z1SD * WTTD )

         SCRD(L,IPP2)   = SCRD(L,IPC2)   * ( Z1 - Z1SD2*SCRD(L,KC2) )
     #    + FKOR(2,1) *   SCRD(L,IPC2H)  * ( Z1 - Z1SD2*SCRD(L,KC2H) ) 
     #    + FKOR(2,2) *   SCRD(L,IPC2H2) * ( Z1 - Z1SD2*SCRD(L,KC2H2) )

      END DO

C     Netto-Wachstumsrate
C     -------------------

      CALL  NETRAT ( NPT,TEMP,SCRD,NATMAX,STICK,CKI,CBET,TAUI )

C...................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  ABKOC2  ( NPT, TEMP, TDUS, SCRD )
C----------------------------------------------------------------------
C File  :    ABKOC2
C Autor :    Susanne Hoefner
C Datum :    15.01.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     NTE-Abweichungskoeffizienten fuer Netto-Wachstumsrate 
C     fuer C-reiche Chemie
C......................................................................
C
C  Eingabe :
C
C     NPT    :  Anzahl der Punkte
C     TEMP   :  Gastemperatur
C     TDUS   :  Staubtemperatur
C
C  Ausgabe :
C
C     SCRD(,Ki) :  Abweichungskoeff. fuer das Molekuel 'i'
C
C.......................................................................
C
C  verwendete Unterprogramme :
C
C     DISKON :  Dissoziationskonstanten  
C
C.......................................................................
C
C  weitere Kommentare :
C
C     Departure coefficients nach Gauger et al., A+A 1990
C
C     verwendete Molekuele:  H2,C2,C2H,C2H2  (fuer eff. Partialdruecke)
C
C
C     ACHTUNG:  ABKOC2 NACH CHEMIE aufrufen, DISKON benutzt die
C               gleichen SCRD-Vektoren ! ( SCRD(L,KH2), ... )
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:
C     -------------------------

      IMPLICIT NONE

C     Parameter fuer Chemie und eff. Partialdruecke

      INCLUDE 'CHPAR_CT.INC'

      INTEGER  NPT,NKP,L

      DOUBLE PRECISION
     #   TEMP(NPT),TDUS(NPT),SCRD(NPT,36),
     #   AC2,AC2H,AC2H2

      PARAMETER ( NKP = 4 )

C.......................................................................

C     Dissoziationskonstanten
C     -----------------------

      CALL  DISKON  ( NPT, NKP, TDUS, SCRD )

      DO L=1,NPT

        SCRD(L,IPP1)   = SCRD(L,KH2)
        SCRD(L,IDPP1D) = SCRD(L,KC2)
        SCRD(L,IDPP1P) = SCRD(L,KC2H)
        SCRD(L,IDPP1T) = SCRD(L,KC2H2)

      END DO

      CALL  DISKON  ( NPT, NKP, TEMP, SCRD )

C.......................................................................

C     Abweichungskoeffizienten
C     ------------------------

      DO L=1,NPT

        AC2     =  ( SCRD(L,IDPP1D) / SCRD(L,KC2) ) 
     #                * SQRT( TEMP(L)/TDUS(L) )

        AC2H    =  SCRD(L,IDPP1P) / SCRD(L,KC2H)

        AC2H2   =  ( SCRD(L,KH2) / SCRD(L,IPP1) ) 
     #             * ( SCRD(L,IDPP1T) / SCRD(L,KC2H2) )

        SCRD(L,KC2)    = AC2
        SCRD(L,KC2H)   = AC2H
        SCRD(L,KC2H2)  = AC2H2

      END DO

C...................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  CHEMC2  ( NPT,TEMP,RHOG,XNC,HEH,XP1,SCRD )
C----------------------------------------------------------------------
C File  :    CHEMC2
C Autor :    Susanne Hoefner
C Datum :    15.01.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     Bestimmung der Anzahldichten der Molekuele 
C     fuer C-reiche Chemie
C......................................................................
C
C  Eingabe:
C
C     NPT        :  Anzahl der Punkte
C     TEMP       :  Gastemperatur
C     RHOG       :  Gasdichte
C     XNC        :  Anzahldichte der freien (d.h. weder in CO noch im 
C                   Staub gebundenen) C-Atome: XNC = XNC_gas - XNCO
C     HEH        :  Elementhaeufigkeit von He
C
C  Ausgabe:
C
C     XP1        :  Partialdruck der C-Monomere
C     SCRD(,IPi) :  Partialdruck des Molekuels 'i'
C
C  Parameter und Konstante:
C
C     BOLTZ      :  Boltzmannkonstante
C     PMASS      :  Protonmasse
C
C.......................................................................
C
C  verwendete Unterprogramme :
C     
C     DISKON  :  Berechnung der Dissoziationskonstanten
C
C.......................................................................
C
C  weitere Kommentare :
C
C     Annahme:   chem. Gleichgewicht fuer H,H2,C,C2,C2H,C2H2 
C
C     Molekuele :  1  H2   
C                  2  C2    
C                  3  C2H   
C                  4  C2H2  
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:
C     -------------------------

      use const_module, only: k_Bol_drk, amu_drk

      IMPLICIT NONE

      INTEGER  NPT,NKP,L

      PARAMETER  ( NKP = 4 )

C     Parameter fuer Partialdruecke:

      INCLUDE 'CHPAR_CT.INC'

      DOUBLE PRECISION
     #    TEMP(NPT),RHOG(NPT),XNC(NPT),XP1(NPT),SCRD(NPT,36),
     #    BOLTZ,PMASS,HEH,SMASS,Z0,Z1,Z2,Z3,Z4,Z8,Z05,
     #    DKP1,DKP2,DKP3,DKP4,XNHTOT,PRHTOT,HRAD,PRH1,
     #    P1K2,CRAD,HGC2,BOLTZ2,BOLTZ4,BOLTZ8

      PARAMETER ( Z0 = 0.0D00, Z1 = 1.0D00, Z2 = 2.0D00, Z3 = 3.0D00,
     #            Z4 = 4.0D00, Z8 = 8.0D00, Z05 = Z1/Z2 )

c      PARAMETER ( BOLTZ = 1.38062E-16, PMASS  = 1.6726E-24 )
      PARAMETER ( BOLTZ = k_Bol_drk, PMASS  = amu_drk )

      PARAMETER ( BOLTZ2 = Z2*BOLTZ, BOLTZ4 = Z4*BOLTZ, 
     #            BOLTZ8 = Z8*BOLTZ )

C.......................................................................

C     Dissoziationskonstanten
C     -----------------------

      CALL  DISKON ( NPT,NKP,TEMP,SCRD )

C.......................................................................

C     Partialdrucke der Monomere und der relevanten Molekuele :
C     ---------------------------------------------------------

      SMASS  = PMASS *( Z1 + HEH*4 )

      DO L=1,NPT

C        DK's 'lesbar' machen 

         DKP1      = SCRD(L,KH2)
         DKP2      = SCRD(L,KC2)
         DKP3      = SCRD(L,KC2H)
         DKP4      = SCRD(L,KC2H2)

C        Wasserstoff (gesamt): Anzahldichte und Druck

         XNHTOT    = RHOG(L) / SMASS
         PRHTOT    = BOLTZ * XNHTOT * TEMP(L)

C        Partialdruck von H1

         HRAD      = Z1 + Z8*DKP1*PRHTOT 
         PRH1      = Z2 * PRHTOT / ( SQRT( HRAD ) + Z1 )

C        Partialdruck von C1

         P1K2      = DKP2   + PRH1   * DKP3 + PRH1**2        * DKP4
         CRAD      = Z1 + BOLTZ8 * TEMP(L) * XNC(L)* P1K2

         XP1(L)    = ( BOLTZ2 * TEMP(L) * XNC(L) ) 
     #                  / ( SQRT( CRAD ) + Z1 )

C        Partialdruecke der Molekuele

         HGC2      = XP1(L)**2

         SCRD(L,IPC2)   = HGC2                        * DKP2
         SCRD(L,IPC2H)  = HGC2  *   PRH1              * DKP3
         SCRD(L,IPC2H2) = HGC2  *   PRH1**2           * DKP4

      END DO

C.......................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  DKINIT()
C----------------------------------------------------------------------
C File  :    DKINTI_C2
C Autor :    Susanne Hoefner
C Datum :    26.8.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     Dissoziationskonstanten fuer C-reiche Chemie - Initialisierung:
C     Berechnung der Spline-Koeffizienten mit SPLINI fuer SPLINT 
C     (wegen Parallelisierung)
C
C......................................................................
C
C  Parameter und Variable :
C
C        NKPL       :  verwendet zur Dimensionierung lokaler Vektoren
C        NTTAB      :  Anzahl der DK-Daten pro Molekuel 
C        DKPTAB(,K) :  Tabelle der DK-Daten fuer das K-te Molekuel
C        TTAB       :  zugehoerige Temperaturtabelle
C        PP         :  Glaettungsparameter fuer Splines
C
C.......................................................................
C
C  verwendete Unterprogramme :
C     
C     SPLINI  :  Berechnung der Spline-Koeffizienten fuer die 
C                Spline-Interpolation der Dissoziationskonstanten 
C                aus den Daten in den INCLUDE-Files
C                Interpolation der Daten in SPLINT !
C
C.......................................................................
C
C  weitere Kommentare :
C
C     verwendete Molekuele :
C
C                 Index  Molekuel  Datensatznr.
C                -------------------------------
C                   1      H2        K14
C                   2      C2        K08
C                   3      C2H       K09
C                   4      C2H2      K10
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:

      IMPLICIT NONE

      INCLUDE 'CHPAR_CT.INC'

      INTEGER

     #    NKPL,K08,K09,K10,K14,I,L

      DOUBLE PRECISION

     #    TEMOG,TEMUG,DELTEM

      INCLUDE 'DKSPLINT.INC'

      PARAMETER  ( NKPL = 4,
C                  C2       C2H      C2H2     H2
     #             K08 = 2, K09 = 3, K10 = 4, K14 = 1 )

      PARAMETER  ( TEMUG = 1.0D2, TEMOG = 6.0D3 )

      DOUBLE PRECISION 
     #          TTAB(NTTAB), DKPTAB(NTTAB,NKPL),
     #          DKPV01(NTTAB),DKPV02(NTTAB),DKPV03(NTTAB),DKPV04(NTTAB)

      CHARACTER * 15 STRMOL(NKPL)

C.......................................................................

C     Inkludieren der files mit den DKP-Daten

      INCLUDE 'H2.INC'
      INCLUDE 'C2.INC'
      INCLUDE 'C2H.INC'
      INCLUDE 'C2H2.INC'

C.......................................................................

C     Berechnen der zugehoerigen Temperaturtabelle und umspeichern
C     der DK-Daten der verschiedenen Molekule auf einzele Vektoren      

      DELTEM = ( TEMOG - TEMUG ) / (NTTAB-1)
      DO L = 1,NTTAB
         TTAB(L)   = TEMUG + (L-1) * DELTEM
         DKPV01(L) = DKPTAB(L,1)
         DKPV02(L) = DKPTAB(L,2)
         DKPV03(L) = DKPTAB(L,3)
         DKPV04(L) = DKPTAB(L,4)
      END DO

C.......................................................................

C     Berechnung der Spline-Koeffizienten fuer die Interpolation der 
C     Dissoziationskonstanten mit SPLINT

      CALL SPLINI (NTTAB, TTAB, DKPV01, SPLA1, SPLB1, SPLC1, SPLD1, PP)
      CALL SPLINI (NTTAB, TTAB, DKPV02, SPLA2, SPLB2, SPLC2, SPLD2, PP)
      CALL SPLINI (NTTAB, TTAB, DKPV03, SPLA3, SPLB3, SPLC3, SPLD3, PP)
      CALL SPLINI (NTTAB, TTAB, DKPV04, SPLA4, SPLB4, SPLC4, SPLD4, PP)

C.......................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  DISKON ( NPT,NKP,TEMP,SCRD )
C----------------------------------------------------------------------
C File  :    DISKON_C2
C Autor :    Susanne Hoefner
C Datum :    17.12.2002 (COBOLD)
C            26.08.2003 (COBOLD, DKINIT, SPLINI, SPLINT)
C
C Verwendungszweck :
C     
C     Bestimmung der Dissoziationskonstanten fuer C-reiche Chemie
C
C......................................................................
C
C  Parameter und Variable :
C
C     Eingabe:
C  
C        NPT        :  Anzahl der Temperaturwerte 
C        NKP        :  Anzahl der Molekuele
C        TEMP(L)    :  Vektor der Temperaturwerte
C
C     Ausgabe:
C
C        DKPi       :  DK des i-ten Molekuels 
C
C     intern:     
C
C        NKPL       :  = NKP, verwendet zur Dimensionierung lokaler 
C                      Vektoren ( Gleichheit wird ueberprueft )
C        NTTAB      :  Anzahl der DK-Daten pro Molekuel 
C        DKPTAB(,K) :  Tabelle der DK-Daten fuer das K-te Molekuel
C        TTAB       :  zugehoerige Temperaturtabelle
C        PP         :  Glaettungsparameter fuer Splines
C
C.......................................................................
C
C  verwendete Unterprogramme :
C     
C     SPLINT  :  Berechnung der DK aus den Daten in den INCLUDE-
C                Files unter Verwendung von Splines
C
C.......................................................................
C
C  weitere Kommentare :
C
C     verwendete Molekuele :
C
C                 Index  Molekuel  Datensatznr.
C                -------------------------------
C                   1      H2        K14
C                   2      C2        K08
C                   3      C2H       K09
C                   4      C2H2      K10
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:

      IMPLICIT NONE

      INCLUDE 'CHPAR_CT.INC'

      INTEGER

     #    NPT,NKP,NKPL,K08,K09,K10,K14,I,L

      DOUBLE PRECISION

     #    TEMOG,TEMUG,DELTEM,XLN10,XLDKP,DLDKPT,DKP1,DKP2,DKP3,DKP4

      INCLUDE 'DKSPLINT.INC'

      PARAMETER  ( NKPL = 4,
C                  C2       C2H      C2H2     H2
     #             K08 = 2, K09 = 3, K10 = 4, K14 = 1 )

      PARAMETER  ( TEMUG = 1.0D2, TEMOG = 6.0D3 )

      DOUBLE PRECISION 
     #          TEMP(NPT),SCRD(NPT,36),
     #          TTAB(NTTAB), DKPTAB(NTTAB,NKPL),
     #          DKPV01(NTTAB),DKPV02(NTTAB),DKPV03(NTTAB),DKPV04(NTTAB)

      CHARACTER * 15 STRMOL(NKPL)

C.......................................................................

C     Inkludieren der files mit den DKP-Daten

      INCLUDE 'H2.INC'
      INCLUDE 'C2.INC'
      INCLUDE 'C2H.INC'
      INCLUDE 'C2H2.INC'

C.......................................................................

C     Ueberpruefung der Konsistenz der Parameter NKP und NKPL
C     (Anzahl der Molekuele)

      IF ( NKP.NE.NKPL ) THEN
         WRITE (*,*) ' DISKON: Anzahl der Molekuele <> NKP !'
         STOP
      END IF

C.......................................................................

C     Berechnen der zugehoerigen Temperaturtabelle und umspeichern
C     der DK-Daten der verschiedenen Molekule auf einzele Vektoren      

      DELTEM = ( TEMOG - TEMUG ) / (NTTAB-1)
      DO L = 1,NTTAB
         TTAB(L)   = TEMUG + (L-1) * DELTEM
         DKPV01(L) = DKPTAB(L,1)
         DKPV02(L) = DKPTAB(L,2)
         DKPV03(L) = DKPTAB(L,3)
         DKPV04(L) = DKPTAB(L,4)
      END DO

C.......................................................................

      XLN10 = LOG(10.0D00)

C     Berechnung der Dissoziationskonstanten + Ableitungen mit Splines

      DO L=1,NPT

         CALL  SPLINT (NTTAB, TTAB, DKPV01, TEMP(L), XLDKP, DLDKPT, 
     #                 SPLA1, SPLB1, SPLC1, SPLD1, PP)
         DKP1   = 10.D00**XLDKP

         CALL  SPLINT (NTTAB, TTAB, DKPV02, TEMP(L), XLDKP, DLDKPT, 
     #                 SPLA2, SPLB2, SPLC2, SPLD2, PP)
         DKP2   = 10.D00**XLDKP

         CALL  SPLINT (NTTAB, TTAB, DKPV03, TEMP(L), XLDKP, DLDKPT, 
     #                 SPLA3, SPLB3, SPLC3, SPLD3, PP)
         DKP3   = 10.D00**XLDKP

         CALL  SPLINT (NTTAB, TTAB, DKPV04, TEMP(L), XLDKP, DLDKPT, 
     #                 SPLA4, SPLB4, SPLC4, SPLD4, PP)
         DKP4   = 10.D00**XLDKP

C        Umspeichern der DK's auf scratch-Vektoren

         SCRD(L,KH2)    = DKP1
         SCRD(L,KC2)    = DKP2
         SCRD(L,KC2H)   = DKP3
         SCRD(L,KC2H2)  = DKP4

      END DO

C.......................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  NETRAT ( NPT,TEMP,SCRD,NATMAX,STICK,CKI,CBET,TAUI )
C----------------------------------------------------------------------
C File  :    NETRAT_2
C Autor :    Susanne Hoefner
C Datum :    15.01.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     Berechnung der Netto-Wachstumsrate fuer die Staubbildung
C......................................................................
C
C  Eingabe:
C
C     NPT         :  Anzahl der Punkte
C     TEMP        :  Gastemperatur
C     SCRD(,IPPi) :  eff. Partialdruck der Spezies mit i Monomeren
C     NATMAX      :  max. Zahl der pro Reaktion angelagerten Monomere,
C                    muss = NATML sein 
C     STICK(I)    :  Sticking coefficient der I-mere 
C     CKI         :  Materialkonstante (siehe STAUPH)
C     CBET        :  Materialkonstante (siehe STAUPH) 
C
C  Ausgabe:
C
C     TAUI   :  Netto-Wachstumsrate 1/Tau
C
C.......................................................................
C
C  verwendete Unterprogramme :
C     *** keine ***
C.......................................................................
C
C  weitere Kommentare :
C
C     Netto-Wachstumsrate 1/Tau nach Gauger et al., A+A 1990
C
C     Verdampfungs- und NTE-Terme sind in den eff. Partialdruecken
C     enthalten (siehe STAUPH).
C
C     Nur Beitraege von Molekuelen mit maximal 2 Monomeren werden 
C     beruecksichtigt. (NATML=2)
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:

      IMPLICIT NONE

      INCLUDE  'CHPAR_CT.INC'

      INTEGER  
     #   NPT,NATMAX,NATML,L

      PARAMETER ( NATML = 2 )

      DOUBLE PRECISION
     #   TEMP(NPT),SCRD(NPT,36),TAUI(NPT),STICK(NATMAX),CKI,CBET,
     #   PARTPR(NATML),TAUSUM,CTSUM,TCBET,CTCBET

C.......................................................................

C     Konsistenz mit STAUPH ueberpruefen:

      IF ( NATMAX.NE.NATML ) THEN
         WRITE(*,*) ' NETRAT: NATMAX <> NATML '
         STOP
      END IF 

C...................................................................

      DO L=1,NPT

C        eff. Partialdruecke 'lesbar' machen:

         PARTPR(1) = SCRD(L,IPP1)
         PARTPR(2) = SCRD(L,IPP2)

C        Netto-Wachstumsrate
C        -------------------

         CTSUM     = SQRT(2.0D00) * STICK(2)
         TAUSUM    = STICK(1) * PARTPR(1) + CTSUM * PARTPR(2) 
         TCBET     = CBET / SQRT(TEMP(L))
         CTCBET    = CKI * TCBET

         TAUI(L)   = CTCBET * TAUSUM

      END DO

C...................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  NUKRAT ( NPT,TEMP,SGAS,PSAT,XP1,SCRD,
     #                     NATMAX,STICK,CBET,R024PI,SIG,NSIG,
     #                     CCS,SJK )
C----------------------------------------------------------------------
C File  :    NUKRAT_2
C Autor :    Susanne Hoefner
C Datum :    15.01.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     Berechnung der Keimbildungsrate fuer die Staubbildung
C......................................................................
C
C  Eingabe:
C
C     NPT         :  Anzahl der Punkte
C     TEMP        :  Gastemperatur
C     SGAS        :  Uebersattigungsverhaeltnis bez. Psat(Tgas) 
C     PSAT        :  Saettigungsdruck (Tgas)
C     XP1         :  Monomer-Partialdruck
C     SCRD(,IPPi) :  eff. Partialdruck der Spezies mit i Monomeren
C     NATMAX      :  max. Zahl der pro Reaktion angelagerten Monomere,
C                    muss = NATML sein 
C     STICK(I)    :  Sticking coefficient der I-mere 
C     CBET        :  Materialkonstante (siehe STAUPH) 
C     R024PI      :  Materialkonstante (siehe STAUPH)
C     SIG         :  Oberflaechenspannung
C     NSIG        :  Parameter fuer die N-Abh. der Oberflaechenspannung
C
C  Ausgabe:
C
C     CCS         :  kritische Clustergroesse
C     SJK         :  stationaere Keimbildungsrate
C
C.......................................................................
C
C  verwendete Unterprogramme :
C     *** keine ***
C.......................................................................
C
C  weitere Kommentare :
C
C     Keimbildungsrate nach der klass. Keimbildungstheorie, erweitert
C     fuer heteromolekulare Reaktionen
C 
C     Nur Beitraege von Molekuelen mit maximal 2 Monomeren werden 
C     beruecksichtigt.
C
C     ACHTUNG:  Konsistenz mit STAUPH, CHEMIE und NETRAT beachten !
C
C     SJK wird fuer SGAS <= 1 null gesetzt !  
C
C-----------------------------------------------------------------------

C     Deklarationen, Parameter:

      use const_module, only: pi_drk, k_Bol_drk, amu_drk

      IMPLICIT NONE

      INCLUDE 'CHPAR_CT.INC'

      INTEGER  
     #   NPT,NATMAX,NATML,NSIG,L,NAT,ICCSK2,ININUK

      PARAMETER ( NATML = 2 )

      DOUBLE PRECISION
     #   TEMP(NPT),SGAS(NPT),XP1(NPT),SCRD(NPT,36),PSAT(NPT),
     #   STICK(NATMAX),CBET,R024PI,CCS(NPT),SJK(NPT),
     #   Z0,Z1,Z2,Z3,Z05,Z32,Z03,Z23,Z8,Z9,Z29,Z2PI,Z4PI,
     #   PI,BOLTZ,SIG,PARTPR(NATML)

      DOUBLE PRECISION
     #   XLNS,CCSU,HG2,HG4,BETSUM,CBSUM,TCBET,BET,
     #   ANK,CCSM,HG5,THEK,DTHEK,C0EX,XNC1,C0NK,RAD,ZEF,THEU,THEU23

c      PARAMETER ( PI = 3.1415926535898, BOLTZ  = 1.38062E-16 )
      PARAMETER ( PI = pi_drk, BOLTZ  = k_Bol_drk )

      PARAMETER ( Z0 = 0.0D00, Z1 = 1.0D00, Z2 = 2.0D00, Z3 = 3.0D00,
     #            Z8 = 8.0D00, Z9 = 9.0D00, 
     #            Z05 = Z1/Z2, Z32 = Z3/Z2, Z03 = Z1/Z3, Z23 = Z2/Z3,
     #            Z29 = Z2/Z9, Z2PI = Z2*PI, Z4PI = 4.0*PI )

      DATA  ININUK /0/

C.......................................................................

C     Konstante und Parameter setzen

      THEU   = SIG * R024PI / BOLTZ
      THEU23 = Z23 * THEU

C.......................................................................

C     Konsistenz mit STAUPH ueberpruefen:

      IF ( NATMAX.NE.NATML ) THEN
         WRITE(*,*) ' NUKRAT: NATMAX <> NATML '
         STOP
      END IF 

C...................................................................

C     Kritische Clustergroesse (CCS):
C     -------------------------------

      ICCSK2 = 0

      DO L=1,NPT

        IF ( SGAS(L).GT.Z1 ) THEN

C         Hilfsgroessen

          XLNS      = LOG ( SGAS(L) )
          CCSU      = ( THEU23 / ( TEMP(L)*XLNS ) )** 3

          HG4       = NSIG/CCSU

          IF ( HG4.EQ.Z0 ) THEN
            HG2    = Z2
          ELSE
            HG2    = Z1 + SQRT( Z1 + Z2*HG4**Z03 )
     #                    - Z2*HG4**Z03
          END IF

C         kritische Clustergroesse

          CCS(L)    = Z1 + CCSU/Z8 * HG2**3
          IF ( CCS(L).LT.Z2 ) THEN
            ICCSK2 = 1
          END IF

        END IF

      END DO

C     minimale krit. Clustergroesse = 2

      IF ( ICCSK2.EQ.1 ) THEN
         DO L=1,NPT
            CCS(L) = CCS(L) + Z2
         END DO
      END IF      

C.......................................................................

C     Keimbildungsrate (SJK) :
C     ------------------------

      DO L=1,NPT

        IF ( SGAS(L).LE.Z1 ) THEN

C         S <= 1 :  SJK = 0 setzen
          SJK(L)    = Z0

        ELSE

C        eff. Partialdruecke 'lesbar' machen:

         PARTPR(1) = SCRD(L,IPP1)
         PARTPR(2) = SCRD(L,IPP2)

         XLNS      = LOG ( SGAS(L) )

C        Beta 

         BETSUM    = Z0
         DO NAT=1,NATMAX
            CBSUM  = NAT**(Z32) * STICK(NAT)
            BETSUM = BETSUM + CBSUM * PARTPR(NAT)
         END DO

         TCBET     = CBET / SQRT(TEMP(L))
         BET       = TCBET * BETSUM

C        Oberflaeche der kritischen Cluster

         ANK       = R024PI * CCS(L)**Z23

C        Hilfsgroessen

         CCSM      = CCS(L) - Z1
         HG5       = Z1 + (NSIG/CCSM)**Z03
         THEK      = THEU / HG5
         DTHEK     = THEU * HG5**(-2) * Z03*(NSIG/CCSM)**(-Z23)
     #               * NSIG*CCSM**(-2)
         C0EX      = EXP( CCSM*XLNS - THEK * CCSM**Z23 / TEMP(L) )

C        Gleichgewichtsdichte fuer kritische Cluster

         XNC1      = XP1(L) / ( BOLTZ * TEMP(L) )
         C0NK      = XNC1 * C0EX

C        Zeldovich-Faktor

         RAD       = - Z2 / TEMP(L) * CCSM**Z23
     #                 * ( DTHEK**2 / THEK - THEK/Z9 * CCSM**(-2) )

         ZEF       = SQRT( RAD / Z2PI )

C        stationaere Keimbildungsrate

         SJK(L)    = BET * ANK * ZEF * C0NK

        END IF

      END DO

C...................................................................

      RETURN
      END SUBROUTINE


C----------------------------------------------------------------------
      SUBROUTINE  PSATC1 ( NPT, TEMP, PSAT )
C----------------------------------------------------------------------
C File  :    PSATC1
C Autor :    Susanne Hoefner
C Datum :    15.01.2003 (COBOLD)
C
C Verwendungszweck :
C     
C     Berechnung des Saettigungsdruckes von C ueber Graphit
C......................................................................
C
C  Variable :
C
C     TEMP       : Temperatur
C     PSAT       : Saettigungsdruck
C
C.......................................................................
C
C  verwendete Unterprogramme :
C     *** keine ***
C.......................................................................
C
C  weitere Kommentare :
C
C     Naeherungsformel fuer PSAT von C ueber Graphit 
C     (nach Gail & Sedlmayr, Paper IV, A+A 1988) 
C     gueltig im Temperaturbereich 500 K <= T <= 1500 K
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE

      INTEGER   NPT,L

      DOUBLE PRECISION  TEMP(NPT),PSAT(NPT),XLNPS,APSAT,BPSAT

C     Koeffizienten fuer C ( 500 K - 1500 K )

      PARAMETER ( APSAT = 86300D00, BPSAT = 32.89D00 )

C.......................................................................

      DO L=1,NPT

C       Formel fuer ln(Psat) nach G&S (Paper IV)

        XLNPS     = - APSAT / TEMP(L) + BPSAT

C       Saettigungsdruck

        PSAT(L)   = EXP (XLNPS)

      END DO

C-----------------------------------------------------------------------

      RETURN
      END SUBROUTINE


C-----------------------------------------------------------------------
      SUBROUTINE SPLINI (NX, XTAB, YTAB, SPLA, SPLB, SPLC, SPLD, PP)
C
C-----------------------------------------------------------------------
C File  :   SPLINI.FOR
C Autor :   S.Hoefner 
C Datum :   26.8.03
C
C modifizierte Version von 
C
C File  :   SPLINT.FOR
C Autor :   A. Hajek 
C Datum :   3.6.91      (minor modifications by SH, 17.01.2003)
C
C modifizierte Version von 
C
C File  :   EAD:CEPH.DIPROF2                           1.  V E R S I O N
C Autor :   Ernst Dorfi, MPA, Muenchen
C Datum :   05/02/89                       letzte Modifikation: 07/02/89
C
C  Verwendungszweck
C
C     Spline-Initialisierung, Berechnung der Splinekoefizienten
C     fuer SPLINT (wegen Parallelisierung)
C.......................................................................
C
C  Eingabeparameter
C
C     NX       : Anzahl der Funktionswerte
C     XTAB     : X - Werte
C     YTAB     : Funktion Y(X)
C     PP       : Glaettungsparameter fuer die rationalen Splines
C
C  Ausgabeparameter
C
C     SPLA     : Splinekoeffizienten
C     SPLB     : Splinekoeffizienten
C     SPLC     : Splinekoeffizienten
C     SPLD     : Splinekoeffizienten
C
C  Interne Parameter
C
C     NXMAX    : Maximale Anzahl der Datenpunkte
C     IX       : Intervall der Splineinterpolation in Y
C
C.......................................................................
C
C  Verwendete Unterprogramme
C
C     TRIDI      : Loesung eines tridiagonalen Gleichungssystem
C.......................................................................
C
C  Weitere Kommentare
C
C     Erstellung der Splinetabelle fuer Interpolation
C     Dieses Programm basiert auf dem Buch von H. Spaeth: 'Spline
C     Algorithmen zur Konstruktion glatter Kurven und Flaechen',
C     Oldenburg Verlag, Muenchen 1973.
C     Der Steifheitsparameter PP ist links und rechts gleich und
C     wird bei allen Punkten verwendet.
C
C     Diese Initialisierungroutine war ein Teil von SPLINT, 
C     Aufteilung wegen Parallelisierung
C
C-----------------------------------------------------------------------
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( NXMAX = 1000 )
C
      DIMENSION XTAB(NX), YTAB(NX),
     #          SPLA(NX), SPLB(NX), SPLC(NX), SPLD(NX),
     #          SCR1(NXMAX), SCR2(NXMAX), SCR3(NXMAX)
C
C-----------------------------------------------------------------------

C     Dimensionierung testen

      IF ( NX.GT.NXMAX ) THEN
         WRITE(*,*) ' !!! ERROR IN SUBROUTINE SPLINT: NX > NXMAX !!! '
         STOP
      END IF 

C-----------------------------------------------------------------------
C     Erstellen der Zeigertabelle,
C     Berechnung der Splinekoeffizienten,
C     Setzen der Parameter,
C     Setzen der Randbedingungen
C-----------------------------------------------------------------------
C
C     Glaettungsparameter fuer die rationalen Splines
C       PP   = 1.0E+00
C
        PP1P = PP + 1.0
        PP2P = PP + 2.0
        PP3P = PP + 3.0
        PPQ2 = 1.0 / ( PP2P ** 2 - 1.0 )
C
C     Aufstellen des Gleichungssystems fuer die Splinekoeffizienten
C     (Randbedingung : einfache Gradienten)
C
        SPLA(1) =  0.0
        SPLB(1) =  1.0
        SPLC(1) =  0.0
C       SPLD(1) =  0.0
        SPLD(1) =  ( YTAB(2)-YTAB(1) ) / ( XTAB(2)-XTAB(1) )
C
        DO 200 I = 2,NX-1
          SPLA(I) = 1.0 / ( XTAB(I  ) - XTAB(I-1) )
          SPLC(I) = 1.0 / ( XTAB(I+1) - XTAB(I  ) )
C
          SPLB(I) = PP2P * ( SPLA(I) + SPLC(I) )
          SPLD(I) = PP3P * ( SPLA(I) ** 2
     #                            * ( YTAB(I  ) - YTAB(I-1) )
     #                     + SPLC(I) ** 2
     #                            * ( YTAB(I+1) - YTAB(I  ) ) )
  200   CONTINUE
C
        SPLA(NX) =  0.0
        SPLB(NX) =  1.0
        SPLC(NX) =  0.0
        SPLD(NX) =  0.0
C       SPLD(NX) =  ( YTAB(NX) - YTAB(NX-1) )
C    #           /  ( XTAB(NX) - XTAB(NX-1) )
C
C     Loesung des Gleichungssystems und Zusammensetzen der
C     Splinekoeffizienten
C
        CALL TRIDI (NX, SPLA, SPLB, SPLC, SPLD, SCR1, SCR2, SCR3)
C
        DO 300 I = 1,NX-1
           DIPRAT = PP3P * ( YTAB (I+1) - YTAB (I) )
           XII    =          XTAB(I+1) - XTAB(I)
           R2P    = PP2P * XII
C
           SPLC(I) = (   DIPRAT - R2P * SCR1(I  )
     #                          - XII * SCR1(I+1) ) * PPQ2
           SPLD(I) = ( - DIPRAT + R2P * SCR1(I+1)
     #                          + XII * SCR1(I  ) ) * PPQ2
C
           SPLA(I) = YTAB(I  ) - SPLC(I)
           SPLB(I) = YTAB(I+1) - SPLD(I)
  300   CONTINUE
C
      RETURN
      END SUBROUTINE


C-----------------------------------------------------------------------
      SUBROUTINE SPLINT (NX, XTAB, YTAB, X, Y, DYDX, 
     #                   SPLA, SPLB, SPLC, SPLD, PP)
C
C-----------------------------------------------------------------------
C File  :   SPLINT.FOR
C Autor :   A. Hajek 
C Datum :   3.6.91   (minor modifications by SH, 17.01.2003)
C           26.8.03  initialisation in separate routine SPLINI (SH) !!!
C
C modifizierte Version von 
C
C File  :   EAD:CEPH.DIPROF2                           1.  V E R S I O N
C Autor :   Ernst Dorfi, MPA, Muenchen
C Datum :   05/02/89                       letzte Modifikation: 07/02/89
C
C  Verwendungszweck
C
C     Spline - Interpolation einer Funktion Y(X)
C.......................................................................
C
C  Eingabeparameter
C
C     NX       : Anzahl der Funktionswerte
C     XTAB     : X - Werte
C     YTAB     : Funktion Y(X)
C     X        : Stuetzstelle
C     PP       : Glaettungsparameter fuer die rationalen Splines
C     INIT     : = 0 fuer ersten Aufruf, wird danach auf 1 gesetzt
C
C  Ausgabeparameter
C
C     Y       : Interpolierter Funktionswert an der Stelle X
C     DYDX    : Ableitung der Funktion Y(X) nach X an der Stelle X
C
C  Interne Parameter
C
C     NXMAX    : Maximale Anzahl der Datenpunkte
C     SPLA     : Splinekoeffizienten
C     SPLB     : Splinekoeffizienten
C     SPLC     : Splinekoeffizienten
C     SPLD     : Splinekoeffizienten
C     IX       : Intervall der Splineinterpolation in Y
C
C.......................................................................
C
C  Verwendete Unterprogramme
C
C     TRIDI      : Loesung eines tridiagonalen Gleichungssystem
C.......................................................................
C
C  Weitere Kommentare
C
C     Spline-Interpolation
C     Dieses Programm basiert auf dem Buch von H. Spaeth: 'Spline
C     Algorithmen zur Konstruktion glatter Kurven und Flaechen',
C     Oldenburg Verlag, Muenchen 1973.
C     Der Steifheitsparameter PP ist links und rechts gleich und
C     wird bei allen Punkten verwendet.
C-----------------------------------------------------------------------
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( NXMAX = 1000 )
C
      DIMENSION XTAB(NX), YTAB(NX),
     #          SPLA(NX), SPLB(NX), SPLC(NX), SPLD(NX) 
C
C-----------------------------------------------------------------------

C     Dimensionierung testen

      IF ( NX.GT.NXMAX ) THEN
         WRITE(*,*) ' !!! ERROR IN SUBROUTINE SPLINT: NX > NXMAX !!! '
         STOP
      END IF 

C-----------------------------------------------------------------------
C     Interpolation der Funktionswerte  ( mit Intervallkontrolle )
C-----------------------------------------------------------------------
C
      XSRCH = X
C
C     Logarithmisch:
C     XSRCH = LOG10(X)
C
C     Bestimme den Index des Interpolationsintervalls
C
      IX = 0
      DO 400 I = 1,NX
        IF ( XTAB(I) .GT. XSRCH ) THEN
          IX = I
          GOTO 410
        END IF
  400 CONTINUE
C
  410 CONTINUE
C
      IX = MIN( IX, NX)
      IX = MAX( IX, 2 )
C
      IXM1 = IX - 1
      XKN  = XTAB(IXM1)
      XKP  = XTAB(IX)
C
      IF ( XSRCH .GE. XKN  .AND.  XSRCH .LE. XKP ) THEN
C
C     Interpolation mittels rationaler Splines
C
        DX  = 1.0 / ( XKP - XKN )
        TRE = ( XSRCH - XKN ) * DX
        URE = 1.0 - TRE
C
        TRE2 = TRE  * TRE
        TRE3 = TRE2 * TRE
        PTP  = PP * TRE  + 1.0
        DTP  = 1.0 / PTP
        URE2 = URE  * URE
        URE3 = URE2 * URE
        PUP  = PP * URE + 1.0
        DUP  = 1.0 / PUP
C
        RTPQ = URE2 * ( 3.0 * PTP + URE * PP ) * DTP ** 2
        RUPQ = TRE2 * ( 3.0 * PUP + TRE * PP ) * DUP ** 2
C
C     Funktionswert mit Ableitung
C
        Y    =   SPLA(IXM1) * URE        + SPLB(IXM1) * TRE
     #         + SPLC(IXM1) * URE3 * DTP + SPLD(IXM1) * TRE3 * DUP
C
        DYDX = ( - SPLA(IXM1)        + SPLB(IXM1)
     #           - SPLC(IXM1) * RTPQ + SPLD(IXM1) * RUPQ ) * DX
C
C       Logarithmisch:
C       Y    = 10.0 ** Y
C       DYDX = Y / X * DYDX
C
      ELSEIF ( XSRCH .GT. XTAB(NX) )  THEN
C
C     Y wird ausserhalb der Tabelle konstant gesetzt
C
        Y    = YTAB(NX)
        DYDX = 0.0
C
C       Logarithmisch:
C       Y   = 10.0 ** YTAB(NX)
C
      ELSE
C
C     Interpolationsfehler
C
        WRITE(6,990) XSRCH, IX, XKP, XKN
        STOP
      ENDIF
C
      RETURN
C
  990 FORMAT(' Interpolationsfehler'/' X-Wert :',1PE21.14,
     #       ' Tab.index :',I5/' X(ind) : ',1PE21.14,' X(ind+1) : ',
     #       1PE21.14/' S T O P  in SPLINT.')
C
      END SUBROUTINE


C-----------------------------------------------------------------------
      SUBROUTINE  TRIDI (N,A,B,C,D,X,Q,R)                              
C                                                                      
C-----------------------------------------------------------------------
C File  :   EAD:SNR.TRIDI                              8.  V E R S I O N
C Autor :   Ernst Dorfi, MKP, Heidelberg                                
C Datum :   01/12/85                       letzte Modifikation: 10/05/88
C                                                                       
C   Verwendungszweck                                                    
C                                                                       
C     Loesung eines tridiagonalen Gleichungssystems der Form            
C     A(I) * X(I-1) + B(I) * X(I) + C(I) * X(I+1) = D(I)                
C-----------------------------------------------------------------------
C                                                                       
C   Eingabeparameter                                                    
C                                                                       
C     N      : Dimension der Matrix                                     
C     A(I)   : Vektor der Koeffizienten von X(I-1)                      
C     B(I)   : Vektor der Koeffizienten von X(I), d.h. Diagonalelemente 
C              der Matrix                                               
C     C(I)   : Vektor der Koeffizienten von X(I+1)                      
C     D(I)   : Rechte Seite                                             
C     Q(I)   : Hilfsvektor                                              
C     R(I)   :    --"--                                                 
C                                                                       
C   Ausgabeparameter                                                    
C                                                                       
C     X(I)   : Loesungsvektor                                           
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
        IMPLICIT  REAL*8  (A-H,O-Z)                           
C                                                                       
        DIMENSION  A(N),B(N),C(N),D(N),X(N),Q(N),R(N)                   
        DATA       Z1 /1.0E+00/                                         
C                                                                       
           R(1) = C(1) / B(1)                                           
           Q(1) = D(1) / B(1)                                           
        DO 300 I=2,N-1                                                  
           S    = Z1 / ( B(I) - A(I) * R(I-1) )                         
           R(I) =        C(I)                   * S                     
           Q(I) =      ( D(I) - A(I) * Q(I-1) ) * S                     
  300   CONTINUE                                                        
           Q(N) = ( D(N) - A(N) * Q(N-1) ) / ( B(N) - A(N) * R(N-1) )   
C                                                                       
           X(N) = Q(N)                                                  
        DO 301 I=N-1,1,-1                                               
           X(I) = Q(I) - R(I) * X(I+1)                                  
  301   CONTINUE                                                        
C                                                                       
        RETURN 
        END SUBROUTINE

      end module dust_momentc2_module
