!**************************************************************************************************
!
!   ######  ####    #####  #     #   ####   
!   #       #   #   #      ##    #  #       
!   ####    #    #  ####   # #   #   ####   
!   #       #    #  #      #  #  #       #  
!   #       #   #   #      #   # #       #  
!   #####   ####    #####  #     #  #####    _module
!
!   Radiation-Hydrodynamics-Code: LTE electron densities 
!
!**************************************************************************************************
!   Fortran90
!   Sven Wedemeyer-Boehm, Freiburg
!   17.10.2005
!**************************************************************************************************
! Modules:
!   edens_module: routines for calculating LTE electron densities
!**************************************************************************************************


!------**************------------------------------------------------------------------------------
module edens_module
!--------------------------------------------------------------------------------------------------
! NAME:
!   edens  _module ('edens_module')
!
! PURPOSE:
!   Provide routines for calculating LTE electron densities.
!
! CATEGORY:
!   Hydrodynamics, equation of state
!
! CALLING SEQUENCE:
!
! VARIABLES:
! (see definition section below)
!
! ROUTINES: (contained)
!   edens_init      :        Read input data, initialise quantities
!
! MODIFICATION HISTORY:
!   31.03.05 (S W  B ) First version
!   08.08.05 (SWB) Revised version
!--------------------------------------------------------------------------------------------------
!
implicit none
!
save 
!
! --- double precision kind definition
integer, parameter :: drk=selected_real_kind(10,200)
!----------------------------------------
! --- physical constants in SI units ---
real(kind=drk), parameter :: HPLANCK    = 6.6261D-34 
real(kind=drk), parameter :: PI         = 3.14159
real(kind=drk), parameter :: M_ELECTRON = 9.1094D-31
real(kind=drk), parameter :: KBOLTZMANN = 1.3807D-23
real(kind=drk), parameter :: C1 = (HPLANCK/(2.0*PI*M_ELECTRON)) * (HPLANCK/KBOLTZMANN)
!--------------------------------------------------------------------------------
! --- conversion factor for energy units --- 
!     (required for ionization potential and energy levels) ---
!     cm^-1 --> J
!     (1 m^-1 ) hc = x J
!     x = {hc} 
!     c=299792458., h= 6.6261D-34 
!     --> Value of conversion factor:  	x = 1.986 445 61(34) x 10-25  
!----------------------------------------
real, parameter              :: c_minv2j= 1.98644561E-25*1E2  
!
!----------------------------------------
! --- type definitions ---
!
! --- type for chemical element data ---
type chemel_type
  integer          :: i   ! --- element number --- 
  character(len=2) :: id  ! --- element ID --- 
  real             :: abu ! --- abundance in epsilon notation---
end type chemel_type
!
! --- type for partition function table and related data ---
type pftable_type
  integer                              :: nelements, ntemp, nstage_total
  integer, dimension(:),   pointer     :: nstage, index
  real,    dimension(:),   pointer     :: T
  real,    dimension(:),   pointer     :: ionpot
  real,    dimension(:,:), pointer     :: lnpf
end type pftable_type
!
! --- electron density table (pretabulated) ---
type netable_type
  ! --- table limits ---
  real(kind=drk)                 :: Tmin,     Tmax,  &
                                    lnnhmin,  lnnhmax, &
                                    lnfionmin, lnfionmax
  ! --- table sampling ---
  real(kind=drk)                 :: dT, dlnnh, dlnfion
  ! --- number sampling points ---
  integer                        :: nT, nlnnh, nlnfion
  ! --- axes ---
  real(kind=drk), dimension(:), pointer       :: T, lnnh, lnfion
  ! --- electron density ---
  real(kind=drk), dimension(:,:,:), pointer   :: n_e
end type netable_type
!
!----------------------------------------
! --- chemical element data ---
type(chemel_type), dimension(:), pointer :: chem_el
!
! --- partition function table and related data ---
type(pftable_type)                     :: pftable
!
! --- electron density table (pretabulated) ---
type(netable_type) :: netable 
!
!----------------------------------------
! --- hard-wired filenames ---
!
character(len=80), parameter :: filename_abu='abundance.input'
character(len=80), parameter :: filename_pf ='pf_kurucz.dat'
!
!----------------------------------------
! --- number of chemical elements ---
integer, parameter           :: n_chemel=99
!
integer, parameter           :: n_max_element             = 26
real,    parameter           :: MAX_ELECTRON_ERROR        = 1.0E-2
integer, parameter           :: N_MAX_ELECTRON_ITERATIONS = 100
!
integer                      :: nstagemax
!----------------------------------------
!
contains
!
!----------**********------------------------------------------------------------------------------
subroutine edens_init()
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   31. 3.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!
!--------------------------------------------------------------------------------
! --- read abundance file ---
!
allocate(chem_el(n_chemel)) ! --- change this later !! ---
!
call edens_read_abu()
!
!--------------------------------------------------------------------------------
! --- read partition function table ---
!
call edens_read_pf()
!
nstagemax=maxval(pftable%nstage)
!
!--------------------------------------------------------------------------------
end subroutine edens_init

!----------**************--------------------------------------------------------------------------
subroutine edens_read_abu()
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   31. 3.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!
integer, parameter:: nc=30 ! --- file unit 
logical           :: exist_flag
integer           :: ios, ierr
integer           :: i, ncount
character(len=80) :: cdummy
real              :: rmax
!
!--------------------------------------------------------------------------------------------------
ncount=0
ierr=0
!--------------------------------------------------------------------------------------------------
! --- create file name ---
!filename_abu=adjustl(trim(par%strtest2))
!
!if (filename_abu(1:1).ne.' ') then 
!  ctmp1=filename_abu(len_trim(filename_abu):len(filename_abu))
!  if (ctmp1(1:1).ne.'/') filename_abu=adjustl(trim(filename_abu))//'/'
!end if  
!filename_abu=adjustl(trim(filename_abu))//adjustl(trim(par%strtest1))
!
!--------------------------------------------------------------------------------------------------

rmax=0.0

! --- read reaction table ---
!
write(*,'(A,A)') '> Reading abundance file: ',trim(filename_abu)
!   
inquire(file=filename_abu, exist=exist_flag)
!
if (exist_flag) then 
  !====================================================================================================
  ! ATTENTION! Unit number is always 30 ... Change in later version!
  !====================================================================================================
  ! --- open input file ---
  open(unit=nc, file=filename_abu, form='FORMATTED', iostat=ios) 
  !
  if (ios == 0) then 
    do      
      read(nc, fmt='(A80)', err=11, end=12) cdummy
      !
      ! --- process input string ---
      cdummy=adjustl(cdummy)
      ! 
      if ((len_trim(cdummy) .ne. 0) .and. (scan(cdummy, '#')  ==  0)) then 
        ! --- found a valid data entry --- 
        ncount=ncount+1
        chem_el(ncount)%id=cdummy(1:2)
        read(cdummy(3:len_trim(cdummy)),*) chem_el(ncount)%abu
        !
        ! --- get maximum abundance read so far ---
        rmax=max(rmax, chem_el(ncount)%abu)
        !
      end if
      !
      !
      ! --- Finished line, read next line ---
    end do
    !
    ! --- Error occured ---
11  ierr=102
    ! 
12  close(nc) 
    !
  else 
    ierr=101
  end if
else 
  ierr=100
end if
!
! --- error handling ---
if (ierr .ne. 0) then 
  select case(ierr)
  case(100) 
    write(6,'(A)') 'EDENS: ERROR: Abundance input file not found!'
  case(101) 
    write(6,'(A)') 'EDENS: ERROR while opening abundance input file!'
  case(102) 
    write(6,'(A)') 'EDENS: ERROR while reading abundance input file!'
  end select
  stop
end if
!
! --- convert to epsilon notation if necessary ---
!f (maxval(chem_el.abu) == 12.0) then 
if (rmax                == 12.0) then 
  do i=1,size(chem_el)
    chem_el(i)%abu = 10.0**(chem_el(i)%abu-12.0)
  end do
end if 
!
! --- check results ---
!do i=1,size(chem_el)
!  write(*,*) i,' [',chem_el(i)%id,'] = ',chem_el(i)%abu
!end do
!
end subroutine edens_read_abu

!----------*************---------------------------------------------------------------------------
subroutine edens_read_pf()
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   31. 3.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
use uio_base_module
use uio_bulk_module
!
implicit none
!
logical            :: exist_flag
integer            :: ios, ierr
character*(80)     :: outstr0
integer            :: ierr0
character(len=80)  :: cdummy
!
! --- for UIO (most of these variables should be provided by uio_base_module! Check reason later! ---
integer :: ncpf ! --- file unit 
integer, parameter                      :: let=80, nttmx=20 
integer                                 :: ntt
character*(let)                         :: termt(2,nttmx)
character*(let)                         :: ident
character, parameter                    :: realna*(*)='real', &
                                           intena*(*)='integer' 
!
!--------------------------------------------------------------------------------------------------
ierr=0
!
ierr0=0
outstr0=''
!
!--------------------------------------------------------------------------------------------------
! --- create file name ---
!filename_abu=adjustl(trim(par%strtest2))
!
!if (filename_abu(1:1).ne.' ') then 
!  ctmp1=filename_abu(len_trim(filename_abu):len(filename_abu))
!  if (ctmp1(1:1).ne.'/') filename_abu=adjustl(trim(filename_abu))//'/'
!end if  
!filename_abu=adjustl(trim(filename_abu))//adjustl(trim(par%strtest1))
!
!--------------------------------------------------------------------------------------------------
! --- read partition function and ionization potential ---
!
write(*,'(A,A)') '> Reading partition function file: ',trim(filename_pf)
!   
inquire(file=filename_pf, exist=exist_flag)
!
if (exist_flag) then 
  ! --- open input file ---
  !
  call uio_openrd(ncpf, filename_pf, outstr=outstr0, ierr=ierr0)
  ! 
  ! -- searching entry and read data if no error occurred ---
   call uio_srhd(ncpf, termt, ntt, type=intena, ident='nelements', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt, pftable%nelements, ident)
  else
    ierr=102
    outstr0='nelements'
    pftable%nelements=0
  end if
  !  
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=intena, ident='ntemp', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt, pftable%ntemp, ident)
  else
    ierr=102
    outstr0='ntemp'
    pftable%ntemp=0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=intena, ident='nstage_total', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  pftable%nstage_total, ident)
  else
    ierr=102
    outstr0='nstage_total'
     pftable%nstage_total=0
  end if
  !
  ! --- now allocate arrays --- 
  if ((pftable%nelements    .ne. 0) .and. & 
      (pftable%ntemp        .ne. 0) .and. &  
      (pftable%nstage_total .ne. 0)) then 
    allocate(pftable%nstage(pftable%nelements                  )) 
    allocate(pftable%index (pftable%nelements                  )) 
    allocate(pftable%T     (pftable%ntemp                      ))     
    allocate(pftable%ionpot(pftable%nstage_total               ))     
    allocate(pftable%lnpf  (pftable%ntemp, pftable%nstage_total))     
  else 
    ierr=110
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=intena, ident='nstage', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, pftable%nstage, ident, mode=2)
  else
    ierr=102
    outstr0='nstage'
    pftable%nstage(:)=0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=intena, ident='index', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, pftable%index, ident, mode=2)
  else
    ierr=102
    outstr0='index'
    pftable%index(:)=0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='T', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, pftable%T, ident, mode=2)
  else
    ierr=102
    outstr0='T'
    pftable%T(:)=0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='ionpot', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, pftable%ionpot, ident, mode=2)
  else
    ierr=102
    outstr0='ionpot'
    pftable%ionpot(:)=0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='pf', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, pftable%lnpf, ident, mode=2)
    pftable%lnpf=alog(pftable%lnpf)
  else
    ierr=102
    outstr0='pf'
    pftable%lnpf(:,:)=0
  end if
  ! 
  ! --- close file ---
  call uio_closrd(ncpf)
else 
  ierr=100
end if
!
! --- error handling ---
if (ierr .ne. 0) then 
  select case(ierr)
  case(100) 
    write(6,'(A)') 'EDENS: ERROR: Partition Function input file not found!'
  case(101) 
    write(6,'(A)') 'EDENS: ERROR while opening partition function input file!'
  case(102) 
    write(6,'(A)') 'EDENS: ERROR while reading partition function input file! ' // outstr0
  end select
  stop
end if
!
end subroutine edens_read_pf
!
!----------*************---------------------------------------------------------------------------
subroutine edens_calc_ne(n_h, T, n_e, si_units)
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!      n_h :      total number density of hydrogen in cm^-3 or m^-3
!      T   :      gas temperature in K 
!      n_e :      electron densities in cm^-3 or m^-3; if all entries are zero, initial electron 
!                 densities are calculated from hydrogen only, otherwise the provided electron 
!                 densities are used
!      si_units : true if the input electron densities are in SI units (m^-3) or false in case 
!                 of cgs (cm^-3)    
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
! partition function stored in a sequential array, 
! individual elements can be addressed via the provided indices
! Example: get partition function for neutral hydrogen (HI):
!   ielement=0
!   istage=0
!   pf_HI=spf.pf(ielement,spf.index(ielement,0)+istage)
!
! MODIFICATION HISTORY:
!   31. 3.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real(kind=drk), dimension(:,:,:), intent(in)    :: T
real(kind=drk), dimension(:,:,:), intent(inout) :: n_H
real(kind=drk), dimension(:,:,:), intent(inout) :: n_e
logical, optional, intent(in)                   :: si_units
!
integer,  dimension(3)                          :: ndim
integer                                         :: i1,i2,i3, &
                                                   ielement, istage, & 
                                                   niter ! --- number of iterations ---
real(kind=drk), dimension(:,:,:), allocatable   :: n_e_old
real(kind=drk)                                  :: sum, PhiH, lnUk, ionpot, error, &
                                                   dn_e, akj
!!$real(kind=drk), dimension(:), allocatable       :: fjk, dfjk
real(kind=drk), dimension(1:nstagemax)          :: fjk, dfjk
!
!!$integer :: OMP_GET_THREAD_NUM, OMP_GET_NUM_THREADS
!--------------------------------------------------------------------------------
!
! --- get array dimensions ---
ndim=shape(n_e)
!
!--------------------------------------------------------------------------------
! --- convert input hydrogen number density to SI units ---
if (.not. si_units) then 
  n_H = n_H * 1E6
  n_e = n_e * 1E6
end if
!
!--------------------------------------------------------------------------------
! --- check if old electron densities should be used as initial guess ---
! --- otherwise Use original electron density as starting guess ---
!
allocate(n_e_old(ndim(1),ndim(2),ndim(3)))
!
if ((maxval(n_e) == 0.0) .and. (minval(n_e) == 0.0)) then 
  ! --- Get the initial solution from ionization of H only ---
  ! --- partition function for H ---
  !
  ielement=1
  istage=1
  !
  !$OMP PARALLEL DEFAULT(shared), &
  !$OMP          PRIVATE(i1,i2,i3, lnUk, ionpot, PhiH)
  !$OMP DO
  do i3=1,ndim(3)
    !!$write(*,'(A,I2,"/",I2," : i3 = ",I3)') 'OMP THREAD ',OMP_GET_THREAD_NUM(),OMP_GET_NUM_THREADS(), i3
    do i2=1,ndim(2)
      do i1=1,ndim(1) 
        ! --- Get the initial solution from ionization of H only ---
        ! --- partition function for H ---
        call edens_get_pf(lnUk, T(i1,i2,i3), ielement, istage) 
        !
        ! --- function Phi for H (refer to Mihalas p.73, eq. 3-23) ---
        ionpot=pftable%ionpot(pftable%index(ielement)+istage-1)
        PhiH = 0.5 * ((C1/T(i1,i2,i3))**1.5) * exp(lnUk + ionpot/(KBOLTZMANN*T(i1,i2,i3))) 
        !
        ! --- store old electron density ---
        n_e_old(i1,i2,i3) = (sqrt(1.0 + 4.0*n_H(i1,i2,i3)*PhiH) - 1.0) / (2.0*PhiH)
        !
        ! --- Copy into n_e as well to calculate first fij and dfij - - */
        n_e(i1,i2,i3) = n_e_old(i1,i2,i3)
        !
      end do
    end do
  end do
  !$OMP END DO
  !$OMP END PARALLEL
else
  n_e_old = n_e
end if

!!$allocate(fjk(maxval(pftable%nstage)), dfjk(maxval(pftable%nstage)))
!
!--------------------------------------------------------------------------------
! --- loop over grid cells ---
!
!!$!$OMP PARALLEL DEFAULT(shared), &
!!$!$OMP          PRIVATE(i1,i2,i3,ielement,istage,niter,error,sum,fjk,dfjk, akj, dn_e)
!!$!$OMP DO SCHEDULE(GUIDED)
do i3=1,ndim(3)
!!$  write(*,'(A,I2,"/",I2," : i3 = ",I3)') 'OMP THREAD ',OMP_GET_THREAD_NUM(),OMP_GET_NUM_THREADS(), i3
  !
  do i2=1,ndim(2)
    do i1=1,ndim(1)
      ! --- start iteration loop ---
      niter = 0
      !
      do 
        error = n_e_old(i1,i2,i3) / n_H(i1,i2,i3)
        sum   = 0.D0
        !
        ! --- loop over all chemical elements ---
        do ielement=1,n_max_element
  	  call edens_get_fjk(fjk, dfjk, T(i1,i2,i3), n_e(i1,i2,i3), ielement)
          !
          ! --- loop over ionisation stages of current element ---
          !     (neglect neutral stage, start with second stage ---
!$OMP CRITICAL(edens_calc_stageloop)
          do istage=2,pftable%nstage(ielement)
	    akj = chem_el(ielement)%abu * (istage-1)
            error = error - akj * fjk(istage)
	    sum   = sum   + akj * dfjk(istage)
          end do
!$OMP END CRITICAL(edens_calc_stageloop)
          !if (ielement == 1) write(*,'(A,E12.2,A)') 'n_e due to HII = ', fjk(2)* akj * n_H(i1,i2,i3)/1E6,' cm^-3'   
        end do 	
        !
        ! --- new electron density ---
        n_e(i1,i2,i3) = n_e_old(I1,i2,i3) - n_H(i1,i2,i3) * error / (1.0 - n_H(i1,i2,i3) * sum)
        !
        ! --- difference with regard to result of last iteration ---
        dn_e = abs((n_e(i1,i2,i3) - n_e_old(i1,i2,i3))/n_e_old(i1,i2,i3))
        !
        ! --- leave iteration loop if requested accurracy is reached ---
        if (dn_e .le. MAX_ELECTRON_ERROR) exit
        !  
        ! --- count number of iterations done ---
        niter=niter+1
        if (niter .gt. N_MAX_ELECTRON_ITERATIONS) exit
        !
        ! --- store electron density as reference for next iteration ---
        n_e_old(i1,i2,i3) = n_e(i1,i2,i3)
        !
      end do 
      !
      if (dn_e .gt. MAX_ELECTRON_ERROR) then 
        write(*,'("ERROR: Electron density iteration not converged:  ")')  
        write(*,'("       spatial location: ",3(I3))')            i1,i2,i3
        write(*,'("       temperature     : ",F6.1," [K]")')      T(i1,i2,i3)
        write(*,'("       density         : ",E12.3," [m^-3]")')  n_H(i1,i2,i3)
        write(*,'("       max dev. in n_e : ",E12.3)')            dn_e
      end if
    end do
  end do
  !
  ! 
end do
!!$!$OMP END DO
!!$!$OMP END PARALLEL
!
deallocate(n_e_old)
!!$  deallocate(fjk, dfjk)
! 
! --- convert densities to unit system of input densities (SI/cgs) ---
if (.not. si_units) then 
  n_e=n_e/1E6
  n_H=n_H/1E6
end if
!
end subroutine edens_calc_ne
!
!----------********************--------------------------------------------------------------------
subroutine edens_calc_ne_hconst(n_h, T, n_e, si_units)
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!      n_h :      total number density of hydrogen in cm^-3 or m^-3
!      T   :      gas temperature in K 
!      n_e :      electron densities in cm^-3 or m^-3; if all entries are zero, initial electron 
!                 densities are calculated from hydrogen only, otherwise the provided electron 
!                 densities are used
!      si_units : true if the input electron densities are in SI units (m^-3) or false in case 
!                 of cgs (cm^-3)    
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!    4. 4.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real(kind=drk), dimension(:,:,:), intent(in)    :: T
real(kind=drk), dimension(:,:,:), intent(inout) :: n_H
real(kind=drk), dimension(:,:,:), intent(inout) :: n_e
logical, optional, intent(in)                   :: si_units
!
integer,  dimension(3)                          :: ndim
integer                                         :: i1,i2,i3, &
                                                   ielement, istage, & 
                                                   niter ! --- number of iterations ---
real(kind=drk), dimension(:,:,:), allocatable   :: n_e_old, n_e_h
real(kind=drk)                                  :: sum, PhiH, lnUk, ionpot, error, &
                                                   dn_e, akj, fH
real(kind=drk), dimension(:), allocatable       :: fjk, dfjk
!
!--------------------------------------------------------------------------------
!
! --- get array dimensions ---
ndim=shape(n_e)
!
!--------------------------------------------------------------------------------
! --- convert input hydrogen number density to SI units ---
if (.not. si_units) then 
  n_H=n_H*1E6
  n_e=n_e*1E6
end if
!
!--------------------------------------------------------------------------------
!
allocate(n_e_old(ndim(1),ndim(2),ndim(3)),n_e_h(ndim(1),ndim(2),ndim(3)))
!
n_e_old = n_e
n_e_h   = n_e
!
allocate(fjk(maxval(pftable%nstage)), dfjk(maxval(pftable%nstage)))
!
!--------------------------------------------------------------------------------
! --- loop over grid cells ---
!
do i3=1,ndim(3)
  do i2=1,ndim(2)
    do i1=1,ndim(1)
      ! --- start iteration loop ---
      niter = 0
      !
      do 
        ! 
        error = n_e_old(i1,i2,i3) / n_H(i1,i2,i3)
        sum   = 0.D0
        !
        ! --- loop over all chemical elements except hydrogen ---
        do ielement=1,n_max_element
          if (ielement == 1) then 
             fH     =n_e_h(i1,i2,i3)/(n_h(i1,i2,i3)-n_e_h(i1,i2,i3))
             fjk(1) =1.0 / (1.0 + fH)
             fjk(2) =fH  / (1.0 + fH)
             dfjk(1)=0D0
             dfjk(2)=0D0
          else 
            call edens_get_fjk(fjk, dfjk, T(i1,i2,i3), n_e(i1,i2,i3), ielement)
          end if
          !
          ! --- loop over ionisation stages of current element ---
          !     (neglect neutral stage, start with second stage ---
          do istage=2,pftable%nstage(ielement)
	    akj = chem_el(ielement)%abu * (istage-1)
            error = error - akj * fjk(istage)
	    sum   = sum   + akj * dfjk(istage)
          end do
        !if (ielement == 1) write(*,'(A,E12.2,A)') 'n_e due to HII = ', fjk(2)* akj * n_H(i1,i2,i3)/1E6,' cm^-3'   


        end do 	
        !
        ! --- new electron density ---
        n_e(i1,i2,i3) = n_e_old(i1,i2,i3) - n_H(i1,i2,i3) * error / (1.0 - n_H(i1,i2,i3) * sum)
        !
        ! --- difference with regard to result of last iteration ---
        dn_e = abs((n_e(i1,i2,i3) - n_e_old(i1,i2,i3))/n_e_old(i1,i2,i3))
        !
        ! --- store electron density as reference for next iteration ---
        n_e_old(i1,i2,i3) = n_e(i1,i2,i3)
        !
        ! --- leave iteration loop if requested accurracy is reached ---
        if (dn_e .le. MAX_ELECTRON_ERROR) exit
        !  
        ! --- count number of iterations done ---
        niter=niter+1
        if (niter .gt. N_MAX_ELECTRON_ITERATIONS) exit
      end do 
      !
      if (dn_e .gt. MAX_ELECTRON_ERROR) then 
        write(*,'("ERROR: Electron density iteration not converged:  ")')  
        write(*,'("       spatial location: ",3(I3))')       i1,i2,i3
        write(*,'("       temperature     : ",F6.1," [K]")')    T(i1,i2,i3)
        write(*,'("       density         : ",E12.3," [m^-3]")')  n_H(i1,i2,i3)
        write(*,'("       dn_emax         : ",E12.3)')  dn_e
      end if
    end do
  end do
end do
!
deallocate(n_e_old, n_e_h, fjk, dfjk)
! 
! --- convert densities to unit system of input densities (SI/cgs) ---
if (.not. si_units) then 
  n_H=n_H/1E6
  n_e=n_e/1E6
end if
!
end subroutine edens_calc_ne_hconst
!
!----------************----------------------------------------------------------------------------
subroutine edens_get_pf(lnUk, T, ielement, istage)
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
! returns natural logarithm of partition function (ln U_k) for
! chemical element number ielement and ionization stage istage
! 
!--------------------------------------------------------------------------------
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!      T  : gas temperature in K 
!      ielement
!      istage 
! OUTPUT:
!      lnUk
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
! Check  the partition function table with regard to neutral hydrogen.
! The Hydrogen pf tabulated by Kurucz has a sharp cusp at
! about T = 9000 K. This leads to problems with the derived
! electron density near that temperature. Therefore, a correct table 
! should contain data derived from a hydrogen atomic model instead.
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
!   partition function stored in a sequential array, 
!   individual elements can be addressed via the provided indices
!   Example: get partition function for neutral hydrogen (HI):
!   ielement=1
!   istage=1
!   lnpf_HI=pftable%lnpf(iT, pftable%index(ielement)+istage-1)
!
! MODIFICATION HISTORY:
!    4. 4.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real(kind=drk),  intent(out)    :: lnUk
real(kind=drk),  intent(in)     :: T
integer, intent(in)             :: ielement, istage 
!
real(kind=drk), dimension(:), allocatable :: ln_pf
real(kind=drk)                            :: fx
integer                                   :: j, ju, jl, jm
!
!--------------------------------------------------------------------------------
lnUk=0D0
!
! --- get partition function for requested element ---
allocate(ln_pf(pftable%ntemp))
ln_pf=pftable%lnpf(1:pftable%ntemp, pftable%index(ielement)+istage-1)
!write(*,*) '> T = ', T, ielement, istage
!write(*,*) 10.0**ln_pf
!
! --- interpolate in temperature ---
!
if (T.le.pftable%T(1)) then
  ! --- temperature at lower table limit ---
  lnUk=ln_pf(1)
else if (T.ge.pftable%T(pftable%ntemp)) then
  ! --- temperature at upper table limit ---
  lnUk=ln_pf(pftable%ntemp)
else 
  ! --- locate index in table ---
  jl=0
  ju=pftable%ntemp+1
  !    
  do
    if (ju-jl.gt.1) then 
      jm=(ju+jl)/2
      if( (pftable%T(pftable%ntemp).ge.pftable%T(1)).eqv.(T.ge.pftable%T(jm)) ) then
        jl=jm
      else 
        ju=jm
      end if
    else 
      exit
    end if
  end do
  !
  if (T == pftable%T(1)) then
    j=1
  else if (T == pftable%T(pftable%ntemp)) then
    j=pftable%ntemp-1
  else
    j=jl
  end if
  !
  fx = (pftable%T(j+1) - T) / (pftable%T(j+1) - pftable%T(j))
  lnUk= fx*ln_pf(j) + (1 - fx)*ln_pf(j+1)
end if
!
end subroutine edens_get_pf
!
!----------*************---------------------------------------------------------------------------
subroutine edens_get_fjk(fjk, dfjk, T, n_e, ielement)
!--------------------------------------------------------------------------------------------------
! NAME:
!   
!
! PURPOSE:
!   
! CATEGORY:
!
! CALLING SEQUENCE:
!
! INPUT:
!      T  : gas temperature in K 
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:

! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   04. 4.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
real(kind=drk), dimension(1:nstagemax), intent(out) :: fjk, dfjk
real(kind=drk),  intent(in)                 :: T, n_e
integer, intent(in)                         :: ielement
!
integer                                     :: istage 
real(kind=drk)                              :: CT_n_e,sum1,sum2
real(kind=drk)                              :: lnUk, lnUkp1
!
!--------------------------------------------------------------------------------------------------
  CT_n_e  = 2.0 * ((C1/T)**(-1.5)) / n_e
  sum1    = 1.0
  sum2    = 0.0
  fjk(:)  = 0.0
  fjk(1)  = 1.0
  dfjk(:) = 0.0
  !
  ! --- get log. partition function for the ground state (stage=0) ---
  !     (lnUk always refers to the next lower stage, i.e. the previous one ...p1)
  istage=1
  call edens_get_pf(lnUk, T, ielement, istage) 
  !
  do istage=2,pftable%nstage(ielement) 
    ! --- get log. partition function for state istage (stage=istage) ---   
    call edens_get_pf(lnUkp1,  T, ielement, istage) 
    !
    fjk(istage) = fjk(istage-1) * CT_n_e * & 
                  exp(lnUkp1 - lnUk - pftable%ionpot(pftable%index(ielement)+istage-1-1)/(KBOLTZMANN*T))
    !
    dfjk(istage) = -(istage-1) * fjk(istage) / n_e
    sum1    = sum1 + fjk(istage)
    sum2    = sum2 + dfjk(istage)
    !
    ! --- store partition function to be the one of the next lower stage ---
    lnUk    = lnUkp1
  end do
  !
  do istage=1,pftable%nstage(ielement) 
    fjk(istage)   =  fjk(istage)/sum1
    dfjk(istage)  = (dfjk(istage) - fjk(istage) * sum2) / sum1
  end do
  !
end subroutine edens_get_fjk
!
!----------********************--------------------------------------------------------------------
subroutine edens_read_ne_table(filename)
!
use uio_base_module
use uio_bulk_module
use uio_filedef_module
!
implicit none
!
character*(*)      :: filename
logical            :: exist_flag
integer            :: ios, ierr
character*(80)     :: outstr0
integer            :: ierr0
character(len=80)  :: cdummy
!
! --- for UIO (most of these variables should be provided by uio_base_module! Check reason later! ---
integer :: ncpf ! --- file unit 
integer, parameter                      :: let=80, nttmx=20 
integer                                 :: ntt
character*(let)                         :: termt(2,nttmx)
character*(let)                         :: ident
character, parameter                    :: realna*(*)='real', &
                                           intena*(*)='integer' 
!
!--------------------------------------------------------------------------------------------------
ierr=0
!
ierr0=0
outstr0=''

! --- read electron density table ---
write(*,'(A,A)') '> Reading electron density table: ',trim(filename)
inquire(file=filename, exist=exist_flag)

if (exist_flag) then 
  ! --- open input file ---
  call uio_openrd(ncpf, filename, outstr=outstr0, ierr=ierr0)
  ! -- searching entry and read data if no error occurred ---
   call uio_srhd(ncpf, termt, ntt, type=intena, ident='nt', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt, netable%nt, ident)
  else
    ierr=102
    outstr0='nt'
    netable%nt=0
  end if

  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=intena, ident='nlnnh', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt, netable%nlnnh, ident)
  else
    ierr=102
    outstr0='nlnnh'
    netable%nlnnh=0
  end if

  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=intena, ident='nlnfion', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%nlnfion, ident)
  else
    ierr=102
    outstr0='nlnfion'
    netable%nlnfion=0
  end if

  ! --- now allocate arrays --- 
  if ((netable%nt      .ne. 0) .and. & 
      (netable%nlnnh   .ne. 0) .and. &  
      (netable%nlnfion .ne. 0)) then 
    allocate(netable%T     (netable%nT                         )) 
    allocate(netable%lnnh  (netable%nlnnh                      )) 
    allocate(netable%lnfion(netable%nlnfion                    ))     
    allocate(netable%n_e(netable%nT,netable%nlnnh,netable%nlnnh))
  else 
    ierr=110
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='tmin', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%tmin, ident)
  else
    ierr=102
    outstr0='tmin'
    netable%tmin=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='tmax', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%tmax, ident)
  else
    ierr=102
    outstr0='tmax'
    netable%tmax=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='lnnhmin', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%lnnhmin, ident)
  else
    ierr=102
    outstr0='lnnhmin'
    netable%lnnhmin=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='lnnhmax', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%lnnhmax, ident)
  else
    ierr=102
    outstr0='lnnhmax'
    netable%lnnhmax=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='lnfionmin', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%lnfionmin, ident)
  else
    ierr=102
    outstr0='lnfionmin'
    netable%lnfionmin=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='lnfionmax', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%lnfionmax, ident)
  else
    ierr=102
    outstr0='lnfionmax'
    netable%lnfionmax=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='dt', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%dt, ident)
  else
    ierr=102
    outstr0='dt'
    netable%dt=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='dlnnh', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%dlnnh, ident)
  else
    ierr=102
    outstr0='dlnnh'
    netable%dlnnh=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='dlnfion', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt, ntt,  netable%dlnfion, ident)
  else
    ierr=102
    outstr0='dlnfion'
    netable%dlnfion=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='t', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, netable%t, ident, mode=2)
  else
    ierr=102
    outstr0='t'
    netable%t(:)=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='lnnh', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, netable%lnnh, ident, mode=2)
  else
    ierr=102
    outstr0='lnnh'
    netable%lnnh(:)=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='lnfion', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, netable%lnfion, ident, mode=2)
  else
    ierr=102
    outstr0='lnfion'
    netable%lnfion(:)=0.0
  end if
  !
  ! -- searching entry and read data if no error occurred ---
  call uio_srhd(ncpf, termt, ntt, type=realna, ident='n_e', &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpf, termt,ntt, netable%n_e, ident, mode=2)
  else
    ierr=102
    outstr0='n_e'
    netable%n_e(:,:,:)=0.0
  end if
  !
  ! --- close file ---
  call uio_closrd(ncpf)
else 
  ierr=100
end if
!
! --- error handling ---
if (ierr .ne. 0) then 
  select case(ierr)
  case(100) 
    write(6,'(A)') 'EDENS: ERROR: Partition Function input file not found!'
  case(101) 
    write(6,'(A)') 'EDENS: ERROR while opening partition function input file!'
  case(102) 
    write(6,'(A)') 'EDENS: ERROR while reading partition function input file! ' // outstr0
  end select
  stop
end if
!
! --- text output ---
write(*,'(A)') '> Axes:'
write(*,'("  T     : dT      = ",E12.2,", minT      = ", E12.2,", maxT      = ", E12.2)') & 
   netable%dt, netable%tmin, netable%tmax
write(*,'("  lnnh  : dlnnh   = ",E12.2,", minlnnh   = ", E12.2,", maxlnnh   = ", E12.2)') & 
   netable%dlnnh, netable%lnnhmin, netable%lnnhmax
write(*,'("  lnfion: dlnfion = ",E12.2,", minlnfion = ", E12.2,", maxlnfion = ", E12.2)') & 
   netable%dlnfion, netable%lnfionmin, netable%lnfionmax
!
end subroutine edens_read_ne_table
!
!----------********************--------------------------------------------------------------------
subroutine edens_interpol_ne_hconst(n_e, n_h, T, fion, si_units)
  !--------------------------------------------------------------------------------------------------
  ! NAME:
  !   
  !
  ! PURPOSE:
  !   
  ! CATEGORY:
  !
  ! CALLING SEQUENCE:
  !
  ! INPUT:
  !      n_h :      total number density of hydrogen in cm^-3 or m^-3
  !      T   :      gas temperature in K 
  !      fion:      ionization degree of hydrogen (from 0.0 to 1.0)
  !      si_units : true if the input electron densities are in SI units (m^-3) or false in case 
  !                 of cgs (cm^-3)    
  !
  ! OUTPUT:
  !      n_e :      electron densities in cm^-3 or m^-3
  !      
  !
  ! ROUTINES:
  !
  ! MODULES:
  !
  ! SIDE EFFECTS:
  !
  ! RESTRICTIONS:

  ! PROCEDURE:
  !
  ! EXAMPLE:
  !
  ! MODIFICATION HISTORY:
  !    7. 4.05 Written by S.Wedemeyer-Boehm, KIS, Freiburg
  !--------------------------------------------------------------------------------------------------
  !
  implicit none
  !
  ! --- I/O parameters ---
  real(kind=drk), dimension(:,:,:), intent(in)    :: n_H, T, fion
  real(kind=drk), dimension(:,:,:), intent(inout) :: n_e
  real(kind=drk)                                  :: lnnh, lnfion
  logical, optional, intent(in)                   :: si_units
  !
  integer,  dimension(3)                          :: ndim
  integer                                         :: i1, i2, i3, & 
                                                     iT, ilnnh, ilnfion 
  integer, dimension(6)                           :: nerr 
  real(kind=drk)                                  :: dT, fT, dlnnh, flnnh, dlnfion, flnfion
  !
  !!$integer :: OMP_GET_THREAD_NUM, OMP_GET_NUM_THREADS
  !--------------------------------------------------------------------------------
  !
  ! --- get array dimensions ---
  ndim=shape(n_e)
  !
  ! --- init error counter ---
  nerr(:)=0
  !--------------------------------------------------------------------------------
  !
  !$OMP PARALLEL DEFAULT(shared), &
  !$OMP          PRIVATE(i1,i2,i3, iT, fT, dT, lnnH, ilnnH, flnnH, dlnnh, & 
  !$OMP                  lnfion, ilnfion, flnfion, dlnfion)
  !$OMP DO
  do i3=1,ndim(3)
     !!$write(*,'(A,I2,"/",I2," : i3 = ",I3)') 'OMP THREAD ',OMP_GET_THREAD_NUM(),OMP_GET_NUM_THREADS(), i3
     do i2=1,ndim(2)
        do i1=1,ndim(1)

           ! --- temperature ---   
           if (T(i1,i2,i3) .le. netable%Tmin) then 
              iT=1
              fT=0.0
              if (T(i1,i2,i3) .lt. netable%Tmin) then
                ! --- Exceeded lower table limit for temperature ---
                nerr(1)=nerr(1)+1 
              endif
           else if (T(i1,i2,i3) .ge. netable%Tmax) then 
              iT=netable%nT-1
              fT=1.0
              if (T(i1,i2,i3) .gt. netable%Tmax) then
                ! --- Exceeded upper table limit for temperature ---
                nerr(2)=nerr(2)+1 
              endif
           else 
              dT=(T(i1,i2,i3)-netable%Tmin)/netable%dT
              iT=floor(dT)+1
              fT=dT-iT+1
           end if

           ! --- H number density ---         
           lnnH=n_H(i1,i2,i3)         
           ! --- convert input hydrogen number density to cgs units if needed ---
           if (si_units) lnnH=lnnH/1E6 ! m^-3 to cm^-3         
           lnnH =log(lnnH)
         
           if (lnnh .le. netable%lnnhmin) then 
              ilnnh=1
              flnnh=0.0
              if (lnnh .lt. netable%lnnhmin) then
                ! --- Exceeded lower table limit for hydrogen number density ---
                nerr(3)=nerr(3)+1 
              endif
           else if (lnnh .ge. netable%lnnhmax) then 
              ilnnh=netable%nlnnh-1
              flnnh=1.0
              if (lnnh .lt. netable%lnnhmax) then
                ! --- Exceeded upper table limit for hydrogen number density ---
                nerr(4)=nerr(4)+1 
              endif
           else 
              dlnnh=(lnnh-netable%lnnhmin)/netable%dlnnh
              ilnnh=floor(dlnnh)+1
              flnnh=dlnnh-ilnnh+1
           end if

           ! ---ionization degree ---        
           lnfion=log(dble(fion(i1,i2,i3)))

           if (lnfion .le. netable%lnfionmin) then 
              ilnfion=1
              flnfion=0.0
              if (lnfion .lt. netable%lnfionmin) then
                ! --- Exceeded lower table limit for ionization degree ---
                nerr(5)=nerr(5)+1 
              endif
           else if (lnfion .ge. netable%lnfionmax) then 
              ilnfion=netable%nlnfion-1
              flnfion=1.0
              if (lnfion .lt. netable%lnfionmax) then
                ! --- Exceeded upper table limit for ionization degree ---
                nerr(6)=nerr(6)+1 
              endif
           else 
              dlnfion=(lnfion-netable%lnfionmin)/netable%dlnfion
              ilnfion=floor(dlnfion)+1
              flnfion=dlnfion-ilnfion+1
           end if

           ! --- trilinear interpolation ---
           n_e(i1,i2,i3) = &
                (1-fT) * (1-flnnh) * (1-flnfion) * netable%n_e(iT  ,ilnnh  ,ilnfion  ) + & 
                (1-fT) * (1-flnnh) *    flnfion  * netable%n_e(iT  ,ilnnh  ,ilnfion+1) + & 
                (1-fT) *    flnnh  * (1-flnfion) * netable%n_e(iT  ,ilnnh+1,ilnfion  ) + & 
                (1-fT) *    flnnh  *    flnfion  * netable%n_e(iT  ,ilnnh+1,ilnfion+1) + & 
                fT     * (1-flnnh) * (1-flnfion) * netable%n_e(iT+1,ilnnh  ,ilnfion  ) + & 
                fT     * (1-flnnh) *    flnfion  * netable%n_e(iT+1,ilnnh  ,ilnfion+1) + & 
                fT     *    flnnh  * (1-flnfion) * netable%n_e(iT+1,ilnnh+1,ilnfion  ) + & 
                fT     *    flnnh  *    flnfion  * netable%n_e(iT+1,ilnnh+1,ilnfion+1)
           
        end do
     end do
  end do
  !$OMP END DO NOWAIT
  !$OMP END PARALLEL
  !
  ! --- convert densities back to SI if requested ---
  if (si_units) n_e=n_e*1E6
  !
  ! --- error information ---
  if (nerr(1) .gt. 0) write(*,'(A,I9,A)') &
     'EDENS: WARNING! Exceeded lower table limit for temperature ',nerr(1),' times.'
  if (nerr(2) .gt. 0) write(*,'(A,I9,A)') & 
     'EDENS: WARNING! Exceeded upper table limit for temperature ',nerr(2),' times.' 
  if (nerr(3) .gt. 0) write(*,'(A,I9,A)') & 
     'EDENS: WARNING! Exceeded lower table limit for hydrogen number density ',nerr(3),' times.' 
  if (nerr(4) .gt. 0) write(*,'(A,I9,A)') & 
     'EDENS: WARNING! Exceeded upper table limit for hydrogen number density ',nerr(4),' times.' 
  if (nerr(5) .gt. 0) write(*,'(A,I9,A)') &
     'EDENS: WARNING! Exceeded lower table limit for ionization degree ',nerr(5),' times.' 
  if (nerr(6) .gt. 0) write(*,'(A,I9,A)') &
     'EDENS: WARNING! Exceeded upper table limit for ionization degree ',nerr(6),' times.' 
  !
  !--------------------------------------------------------------------------------
end subroutine edens_interpol_ne_hconst

!----------------------------------------------------------------------------------
end module edens_module
