module hion_def_module
  !---------------------------------------------------------------------------
  ! NAME:
  !   hion_def_module ('hion_def_module')
  !
  ! PURPOSE:
  !   contains atomic structure definitions and various useful constants
  !
  ! CATEGORY:
  !   Radiation transport
  !
  ! CALLING SEQUENCE:
  !   use hion_def_module
  !
  ! VARIABLES:
  !
  ! ROUTINES: (contained)
  !
  ! MODIFICATION HISTORY:
  !   Feb 20 2005 : Written by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
  !   Apr  1 2005 : (J.L.) Added 'ntd' component to 'atom_type',
  !   May 10 2005 : (J.L.) Added 'chunk_type' 
  !   Aug 08 2005 : (SWB)  changed attribute allocatable in pointer for derived
  !                        types
  !---------------------------------------------------------------------------

  implicit none
  save

  ! --- double precision kind definition
!!$  integer, parameter :: dbl=selected_real_kind(10,200)
  integer, parameter :: dbl=kind(0.0D+00)

  ! --- physical constants and other useful quantities
  real(kind=dbl),parameter :: PI=3.14159265358979
  real(kind=dbl),parameter :: CLIGHT=2.99792458E+08   ! Speed of light [m/s]
  real(kind=dbl),parameter :: HPLANCK=6.6260755E-34    !  Planck's constant [Js]
  real(kind=dbl),parameter :: KBOLTZMANN=1.380658E-23     !  Boltzman's cst. [J/K]
  real(kind=dbl),parameter :: AMU=1.6605402E-27    !  Atomic mass unit [kg] 
  real(kind=dbl),parameter :: M_ELECTRON=9.1093897E-31    !  Electron mass [kg] 
  real(kind=dbl),parameter :: Q_ELECTRON=1.60217733E-19   !  Electron charge [C]
  real(kind=dbl),parameter :: EPSILON_0=8.854187817E-12  !  Vac. permittivity [F/m]
  real(kind=dbl),parameter :: MU_0=1.2566370614E-06 !  Magn. induct. of vac.
  real(kind=dbl),parameter :: RBOHR=5.29177349E-11   !  Bohr radius [m]
  real(kind=dbl),parameter :: E_RYDBERG=2.1798741E-18    !  Ion. pot. Hydrogen [J]
  real(kind=dbl),parameter :: EV_TO_J=1.60217733E-19   !  One electronVolt [J]
  real(kind=dbl),parameter :: THETA0=5.03974756E+03   !  log10(e) * eV/k [K^-1]
  real(kind=dbl),parameter :: WGHT_PER_H=1.4350D0 ! amu per hydrogen atom

  ! --- Unit conversions 
  real(kind=dbl),parameter :: NM_TO_M=1.0E-09
  real(kind=dbl),parameter :: CM_TO_M=1.0E-02
  real(kind=dbl),parameter :: KM_TO_M=1.0E+03
  real(kind=dbl),parameter :: ERG_TO_JOULE=1.0E-07
  real(kind=dbl),parameter :: G_TO_KG=1.0E-03
  real(kind=dbl),parameter :: MICRON_TO_NM=1.0E+03
  real(kind=dbl),parameter :: MEGABARN_TO_M2=1.0E-22
  real(kind=dbl),parameter :: ATM_TO_PA=1.0135E+05 ! --- Atm to Pascal (N/m^2)

  ! ---  Type definitions 

  ! --- bound-bound transition
  type line_type
     integer          :: i,j          ! --- lower, upper level
     real(kind=dbl)   :: f,Trad,H,nu0 ! --- oscillator strength,Trad,H,
                                      ! --- line center frequency
     character(len=5) :: tr_type      ! --- either bbbRE or FIXED
     real(kind=dbl), dimension(:,:), pointer :: nTr  
  end type line_type


  ! --- bound-free transition
  type cont_type
     integer          :: i,j              ! --- lower,upper level
     real(kind=dbl)   :: alpha,H,nu0,Trad ! --- cross section at edge,H,
                                          ! --- edge frequency,Trad
     character(len=5) :: tr_type          ! --- either bbbRE or HYDRO
     real(kind=dbl), dimension(:,:), pointer :: nTr 
  end type cont_type


  ! --- bound-bound collision coefficient struct
  type bbcol_type
     integer                                 :: Nline,Ntemp ! --- number of lines,temperatures
     integer,        dimension(:),   pointer :: i,j ! --- line identifiers
     real(kind=dbl), dimension(:),   pointer :: temp ! --- temperature arrays
     real(kind=dbl), dimension(:,:), pointer :: Cij ! --- collisional coefficients
  end type bbcol_type


  ! --- bound-free collisions coefficient struct
  type bfcol_type
     integer                                 :: Ncont,Ntemp ! --- number of continua,temperatures
     integer,        dimension(:),   pointer :: i,j  ! --- line identifiers
     real(kind=dbl), dimension(:),   pointer :: temp ! --- temperature arrays
     real(kind=dbl), dimension(:,:), pointer :: Cij  ! --- collisional coefficients
  end type bfcol_type


  ! --- atom
  type atom_type
     character(len=2)                             :: id      ! atomic ID
     real(kind=dbl)                               :: abun    ! number abundance relative to hydrogen
     integer                                      :: Nlevel,Nline,Ncont,ielement              
     integer,          dimension(:),      pointer :: stage,levelNo   ! ion stage, levelNo
     real(kind=dbl),   dimension(:),      pointer :: E,g     ! energy [eV],
                                                             ! statistical weight
     character(len=20),dimension(:),      pointer :: label   ! atomic level label
     type(line_type),  dimension(:),      pointer :: line    ! bb data
     type(cont_type),  dimension(:),      pointer :: cont    ! bf data
     real(kind=dbl),   dimension(:,:,:),  pointer :: ntot    ! total number density of atom
     real(kind=dbl),   dimension(:,:,:,:),pointer :: n,nstar ! NLTE/LTE level populations
     real,             dimension(:,:,:,:),pointer :: ntd     ! time dependent NLTE level populations
                                                             ! Is single precision so that it can
                                                             ! be advected by the quc pointer in the 
                                                             !'model' struct
     type(bbcol_type)                             :: bbcol
     type(bfcol_type)                             :: bfcol
  end type atom_type


  ! --- atmos  
  type atmos_type
     integer                                 :: n1,n2,n3 ! --- # of grid points in each direction
     real(kind=dbl),dimension(:,:,:),pointer :: ne,temp,rho,cmass 
     ! electron density, temperature, mass density, column mass
  end type atmos_type


  ! --- chunk
  type chunk_type
     integer                                       :: nchunks,szn, szl ! --- # of chunks, normal size, size of last one.
     integer,        dimension(:,:),       pointer :: index   ! chunk x2 indices
     real(kind=dbl), dimension(:,:,:,:,:), pointer :: rij,cij ! radiative and collisional rate coefficients	
  end type chunk_type


  ! --- output
  type output_type
     logical        :: time,step ! .TRUE. if output after fixed time intervals, or fixed # if time steps
     real(kind=dbl) :: timediff  ! time interval between output
     integer        :: stepdiff  ! number of steps beteen output
  end type output_type


end module hion_def_module
