
  module hion_io_module
  !---------------------------------------------------------------------------
  ! NAME:
  !   hion_io_module ('rhd_chromospheric_radiation_module')
  !
  ! PURPOSE:
  !   Chromospheric radiative transfer routines for rhd.
  !
  ! CATEGORY:
  !   Radiation transport - utility, I/O
  !     contains basic i/o routines and an error message printing routine
  !
  ! CALLING SEQUENCE:
  !   use hion_io_module
  !
  ! VARIABLES:
  !
  ! ROUTINES: (contained)
  !           PrError(RoutineName,Message,ErrLevel)
  !           NextEntry(fileunit)
  !           ReadAtom(atm,atomfile)
  !           GetInput(Natoms,AtomFiles)
  ! MODIFICATION HISTORY:
  !   Feb 22 2005  Written by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
  !---------------------------------------------------------------------------
	use hion_def_module
	use rhd_box_module
	use rhd_sub_module

	implicit none

  contains

 
    subroutine PrError(RoutineName,Message,ErrLevel)
      !---------------------------------------------------------------------
      ! --- Prints simple error messages, ErrLevel 2 stops program 
      ! --- execution.
      ! --- Feb 21 2005 : 
      ! ---   created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
      !---------------------------------------------------------------------
      implicit none

      integer,intent(in) :: ErrLevel
      character(len=*),intent(in) :: RoutineName,Message

      select case(ErrLevel) 
      case(0)
         write(unit=*,fmt='(A)') "MESSAGE in routine ", RoutineName,": "
         write(unit=*,fmt='(A)') Message
      case(1)
         write(unit=*,fmt='(A)') "WARNING in routine ", RoutineName,": "
         write(unit=*,fmt='(A)') Message
      case(2)
         write(unit=*,fmt='(A)') "FATAL ERROR in routine ", RoutineName,": "
         write(unit=*,fmt='(A)') Message
         write(unit=*,fmt='(A)') "Exiting..."
         stop
      end select

    end subroutine PrError
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine WriteLines(lines,iounit)
    !---------------------------------------------------------------------
    ! --- Writes array of line_type structure to iounit in formatted 
    ! --- output.
    ! --- Feb 22 2005: 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl) 
    !---------------------------------------------------------------------
    implicit none

    integer,intent(in) :: iounit
    type(line_type),dimension(:),intent(in) :: lines
    integer ::k,l

    write(unit=iounit,fmt='("Line data:")')
    write(unit=iounit,&
         fmt='("  i  j          f   Trad   H  type        nu0")')
    do k=1,size(lines)
       write(unit=iounit,&
            fmt='(1X,I2,1X,I2,1X,ES10.4,1X,F6.0,1X,F3.1,1X,A5,1X,ES10.4)') &
            lines(k)%i,lines(k)%j,lines(k)%f,lines(k)%Trad, &
            lines(k)%H,lines(k)%tr_type,lines(k)%nu0
    end do

  end subroutine WriteLines
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine WriteContinua(cont,iounit)
    !---------------------------------------------------------------------
    ! --- Writes array of cont_type structures to iounit in formatted 
    ! --- output.
    ! --- Feb 22 2005: 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl) 
    !---------------------------------------------------------------------
    implicit none
    integer,intent(in) :: iounit
    type(cont_type),dimension(:),intent(in) :: cont
    integer :: k

    write(unit=iounit,fmt='("Continuum data:")')
    write(unit=iounit,&
         fmt='("  i  j      alpha   Trad   H  type        nu0")')
    do k=1,size(cont)
       write(unit=iounit,&
            fmt='(1X,I2,1X,I2,1X,ES10.4,1X,F6.0,1X,F3.1,1X,A5,1X,ES10.4)') &
            cont(k)%i,cont(k)%j,cont(k)%alpha, &
            cont(k)%Trad, cont(k)%H,cont(k)%tr_type, &
            cont(k)%nu0
    end do

  end subroutine WriteContinua
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine WriteBbcol(bbcol,iounit)
    !---------------------------------------------------------------------
    ! --- Writes bbcol structure to iounit in formatted output
    ! --- Feb 22 2005: 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl) 
    !---------------------------------------------------------------------
    implicit none

    integer,intent(in) :: iounit
    type(bbcol_type),intent(in) :: bbcol 
    integer :: k,l

    write(unit=iounit,fmt='("bound-bound collision data:")')
    write(unit=iounit,fmt='("Nline: ",I3," Ntemp: ",I3)') &
         bbcol%Nline,bbcol%Ntemp
    write(unit=iounit,fmt='( 7X,20(1X,F10.1) )') &
         (bbcol%temp(k), k=1,bbcol%Ntemp)
    do k=1,bbcol%Nline
       write(unit=iounit,fmt='( 1X,I2,1X,I2,1X,20(:,1X,ES10.4) )') &
            bbcol%i(k),bbcol%j(k), &
            (bbcol%Cij(k,l), l=1, bbcol%Ntemp)
    end do

  end subroutine WriteBbcol
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine WriteBfcol(bfcol,iounit)
    !---------------------------------------------------------------------
    ! --- Writes bfcol structure to iounit in formatted output
    ! --- Feb 22 2005: 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl) 
    !---------------------------------------------------------------------
    implicit none
    integer,intent(in) :: iounit
    type(bfcol_type),intent(in) :: bfcol
    integer :: k,l

    write(unit=iounit,fmt='("bound-free collision data:")')
    write(unit=iounit,fmt='("Ncont: ",I3," Ntemp: ",I3)') &
         bfcol%Ncont,bfcol%Ntemp
    write(unit=iounit,fmt='( 7X,20(1X,f10.1) )') &
         (bfcol%temp(k), k=1, bfcol%Ntemp)

    do k=1,bfcol%Ncont
       write(unit=iounit,fmt='(1X,I2,1X,I2,1X,20(1X,ES10.4))') &
            bfcol%i(k),bfcol%j(k), &
            (bfcol%Cij(k,l), l=1, bfcol%Ntemp)
    end do

  end subroutine WriteBfcol
  !---------------------------------------------------------------------
