!**************************************************************************************************
!
!   #    #  #   ###   #    #                
!   #    #  #  #   #  ##   #          
!   ######  #  #   #  # #  #          
!   #    #  #  #   #  #  # #          
!   #    #  #  #   #  #   ##          
!   #    #  #   ###   #    #  _ dvode_module
!
!   Radiation-Hydrodynamics-Code: Solver for systems of differential equations. 
!
!**************************************************************************************************
!   Fortran90
!   Sven Wedemeyer-Boehm, Freiburg
!   15.11.2005
!**************************************************************************************************
! Modules:
!   hion_dvode_module: call DVODE solver package
!**************************************************************************************************

!------********************------------------------------------------------------------------------
module hion_dvode_module 
!--------------------------------------------------------------------------------------------------
! NAME:
!   hion_dvode_module ('hion_module')
!
! PURPOSE:
!   Call solver for ordinary differential equations dy/dt=f(y,t), 
!   with y and f vectors
!   Here used to solve rate equations for hydrogen model atom.
!
! CATEGORY:
!   Dust/Molecules/Hydrogen
!
! CALLING SEQUENCE:
!
! VARIABLES:
!
! ROUTINES: 
!
! MODIFICATION HISTORY:
! 15.11.2005 (SWB) created module
!
!---------------------------------------------------------------------
use hion_def_module, only: dbl

implicit none

save

real(kind=dbl), dimension(:,:), allocatable :: jacobian
!$OMP THREADPRIVATE(jacobian)

!---------------------------------------------------------------------

contains 

!---------------------------------------------------------------------
  subroutine hion_dvode(nlevel, ndummy, jacin, dbldtime, istate)
    !
    use dvode_module
    ! 
    implicit none

    ! --- I/O ---
    integer,                             intent(in)    :: nlevel
    real(kind=dbl), dimension(1:nlevel), intent(inout) :: ndummy
    real(kind=dbl), dimension(:,:),      intent(in)    :: jacin
    real(kind=dbl),                      intent(in)    :: dbldtime
    integer,                             intent(out)   :: istate

    ! --- DVODE parameter ---
    integer, parameter :: itol   = 1,  & 
                          itask  = 1,  &
                          iopt   = 0,  & 
                          mf     = 21  ! solver method: explicitly give Jacobian matrix.
    
    ! --- more DVODE I/O ---
    real(kind=dbl)                            :: dbltime0
    integer                                   :: lrw,liw
    integer                                   :: ipar
    integer,        dimension(:), allocatable :: iwork
    real(kind=dbl)                            :: rpar
    real(kind=dbl), dimension(:), allocatable :: rwork
    real(kind=dbl), dimension(:), allocatable :: atol,rtol
    !---------------------------------------------------------------------

    dbltime0=0.0D0

    ! --- set dimensions of work arrays ---
    lrw = 22+9*nlevel+2*nlevel**2 
    liw = 30+  nlevel 

    ! --- allocate arrays ---
    allocate( rtol(1:nlevel), atol(1:nlevel), iwork(1:liw), rwork(1:lrw))

    ! --- set arrays for accuracy ---
    rtol(1:nlevel) = 1.0D-4    ! relative accuracy
    atol(1:nlevel) = 1.0D-30   ! absolute accuracy

    iwork(1:liw)=0
    rwork(1:lrw)=0.0D0
    ipar=0
    rpar=0.0D0

    ! --- provide Jacobian to whole module ---
    allocate( jacobian(1:nlevel,1:nlevel))
    jacobian(:,:)=jacin(:,:)

    ! --- init error state ---
    istate = 1

    ! --- call solver ---
    call dvode(rn_sub, nlevel, ndummy(1:nlevel), dbltime0, dbldtime, & 
               itol, rtol, atol, itask, istate, iopt, rwork, lrw, &
               iwork, liw, jac_sub, mf, rpar, ipar)

    ! --- free memory ---
    deallocate( rtol, atol, iwork, rwork )
    deallocate(jacobian)

  end subroutine hion_dvode

  !---------------------------------------------------------------------
  subroutine rn_Sub(neq, t, y, ydot,  rpar, ipar)
    !---------------------------------------------------------------------
    ! --- Calculate R_ij * n_i and put them into ydot. Needed by the
    ! --- dvode routine. Basicly an empty routine. R_ij is calculated
    ! --- in GetPopulations and made available via the global parameter
    ! --- 'gamma'.
    ! --- Apr  1 2005 : 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------

    implicit none

    integer,       intent(in)                    :: neq,ipar
    real(kind=dbl),intent(in)                    :: t, rpar
    real(kind=dbl),intent(in),    dimension(neq) :: y
    real(kind=dbl),intent(inout), dimension(neq) :: ydot

    !---------------------------------------------------------------------

    ydot = matmul(jacobian, y)

  end subroutine rn_Sub
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine jac_sub(neq, t, y, ml, mu, pd, nrowpd, rpar, ipar)  
    !---------------------------------------------------------------------
    ! --- Calculate R_ij  and put them into PD. Needed by the
    ! --- dvode routine. Basicly an empty routine. R_ij is calculated
    ! --- in GetPopulations and made available via the global parameter
    ! --- 'gamma'.
    ! --- Apr  1 2005 : 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    implicit none

    integer,intent(in)                              :: neq, ml, mu, & 
                                                       nrowpd, ipar
    real(kind=dbl),intent(in)                       :: t, rpar
    real(kind=dbl),intent(in),   dimension(neq)     :: y
    real(kind=dbl),intent(inout),dimension(neq,neq) :: pd
    !---------------------------------------------------------------------

    pd(:,:)=jacobian(:,:)

  end subroutine jac_sub
  !---------------------------------------------------------------------

end module hion_dvode_module

