!***************************************************************************
!   Fortran90
!   Jorrit Leenaarts, Utrecht, The Netherlands
!   Created: Feb 22 2005 
!   Last modification: Aug 15 2006
!***************************************************************************

module hion_io_module
  !---------------------------------------------------------------------------
  ! NAME:
  !   hion_io_module ('rhd_chromospheric_radiation_module')
  !
  ! PURPOSE:
  !   Chromospheric radiative transfer routines for rhd.
  !
  ! CATEGORY:
  !   Radiation transport - utility, I/O
  !     contains basic i/o routines and an error message printing routine
  !     for HION.
  !
  ! CALLING SEQUENCE:
  !   use hion_io_module
  !
  ! VARIABLES:
  !
  ! ROUTINES: (contained)
  !           PrError(RoutineName,Message,ErrLevel)
  !           NextEntry(fileunit)
  !           ReadAtom(atm,atomfile)
  !           GetInput(Natoms,AtomFiles)
  ! MODIFICATION HISTORY:
  !   Feb 22 2005: Written by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
  !   May 19 2005: (JL) removed unused routines
  !   Aug 08 2005: (SWB) minor changes
  !   Nov  9 2005: (JL) minor changes 
  !   Aug 15 2005: (SWB) removed hion.input file I/O
  !---------------------------------------------------------------------------
  use hion_def_module
  use rhd_box_module
  use rhd_sub_module

  implicit none

contains

  subroutine PrError(RoutineName,Message,ErrLevel)
    !---------------------------------------------------------------------
    ! --- Prints simple error messages, ErrLevel 2 stops program 
    ! --- execution.
    ! --- Feb 21 2005 : 
    ! ---   created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    implicit none

    integer,intent(in) :: ErrLevel
    character(len=*),intent(in) :: RoutineName,Message

    select case(ErrLevel) 
    case(0)
       write(unit=*,fmt='(A)') "HION: MESSAGE in routine ", RoutineName,": "
       write(unit=*,fmt='(A)') "      ", Message
    case(1)
       write(unit=*,fmt='(A)') "HION: WARNING in routine ", RoutineName,": "
       write(unit=*,fmt='(A)') "      ", Message
    case(2)
       write(unit=*,fmt='(A)') "HION: FATAL ERROR in routine ", RoutineName,": "
       write(unit=*,fmt='(A)') "      ", Message
       write(unit=*,fmt='(A)') "      Exiting..."
       stop
    end select

  end subroutine PrError
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine NextEntry(fileunit)
    !---------------------------------------------------------------------
    ! --- Reads first token from input line, returns when it is not a 
    ! --- comment character.  
    ! --- Feb 21 2005: 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    implicit none

    integer,intent(in)         :: fileunit
    character(len=1)           :: firstchar
    character(len=1),parameter :: comment_char='#'

    do 
       read(unit=fileunit,fmt='(A1)',advance='NO') firstchar
       if (firstchar.ne.comment_char) exit
       ! --- still in comment region, advance to next record
       read(unit=fileunit,fmt='(A1)') firstchar 
    end do

  end subroutine NextEntry
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine WriteAtom(atom,atmos,model,nx,ny,nz,filename,quc_prop)
    !---------------------------------------------------------------------
    ! --- Writes atom structure to file in UIO format. UIO is not capable 
    ! --- of outputting arrays of structures, so some reshuffling of the
    ! --- data is done before writing.
    ! --- INPUT:
    ! ---   atom       : atom_type
    ! ---   atmos      : atmos_type
    ! ---   model      : box_type
    ! ---   nz         : integer, number of points in z direction
    ! ---   filename   : char, name of file to write to.
    ! ---   quc_prop   : char, 'hiontd' or 'hion'
    ! --- Mar 17 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Apr  3 2005 : (JL) Added rate switch.
    ! --- May 19 2005 : (JL) Removed rate switch, they are no longer 
    ! ---               available in 3D version. Maybe output per chunk
    ! ---               if necessary
    ! --- Aug 16 2006 : (JL) added output of velocities
    !---------------------------------------------------------------------
    use hion_def_module
    use uio_bulk_module
    use uio_filedef_module
    use rhd_box_module

    implicit none

    type(atmos_type),intent(in) :: atmos
    type(atom_type),intent(in) :: atom
    type(box_type),intent(in) :: model
    integer,intent(in) :: nx,ny,nz 
    character(len=*),intent(in) :: filename,quc_prop
   
    integer :: k,nc,err
    character(len=9),parameter :: Routinename='WriteAtom'

    real(kind=dbl),dimension(atom%Nline) :: l_i,l_j,l_f,l_H, &
                                            l_nu0,l_Trad
    real(kind=dbl),dimension(nx,ny,atom%Nline) :: l_nTr
    character(len=5),dimension(atom%Nline) :: l_tr_type

    real(kind=dbl),dimension(atom%Ncont) :: c_i,c_j,c_alpha,c_H, &
                                            c_nu0,c_Trad
    real(kind=dbl),dimension(nx,ny,atom%Ncont) :: c_nTr
    character(len=5),dimension(atom%Ncont) :: c_tr_type

    ! --- put data in UIO output format
    do k=1,atom%Nline
       l_i(k)       = atom%line(k)%i
       l_j(k)       = atom%line(k)%j
       l_nTr(:,:,k) = atom%line(k)%nTr(:,:)
       l_f(k)       = atom%line(k)%f
       l_H(k)       = atom%line(k)%H
       l_nu0(k)     = atom%line(k)%nu0
       l_Trad(k)    = atom%line(k)%Trad
       l_tr_type(k) = atom%line(k)%tr_type
    end do

    do k=1,atom%Ncont
       c_i(k)           = atom%cont(k)%i
       c_j(k)           = atom%cont(k)%j
       c_nTr(:,:,k)     = atom%cont(k)%nTr(:,:)
       c_alpha(k)       = atom%cont(k)%alpha
       c_H(k)           = atom%cont(k)%H
       c_nu0(k)         = atom%cont(k)%nu0
       c_Trad(k)        = atom%cont(k)%Trad
       c_tr_type(k)     = atom%cont(k)%tr_type
    end do

    call uio_openwr(nc,filename,form='unformatted', &
         conv='ieee_4',position='rewind',ierr=err)
    if (err.gt.0) call PrError(RoutineName, &
         'Failed to open atomic output file',2) 
    call uio_wrlabl(nc,dataset_ident,date='now')

    call uio_wrlabl(nc,box_ident,date='now')
    call uio_wr(nc,'atmos',      box_id_ident )
    call uio_wr(nc, atmos%rho,  'rho'         )  
    call uio_wr(nc, atmos%temp, 'temp'        )  
    call uio_wr(nc, atmos%ne,   'nelec'       )
    call uio_wr(nc, model%v1,     'v1'          )
    call uio_wr(nc, model%v2,     'v2'          )
    call uio_wr(nc, model%v3,     'v3'          )
    call uio_wr(nc, model%itime,'itime'       )
    call uio_wr(nc, model%time, 'time'        )
    call uio_wr(nc, model%xc1*CM_TO_M,  'xc1'         )
    call uio_wr(nc, model%xc2*CM_TO_M,  'xc2'         )
    call uio_wr(nc, model%xc3*CM_TO_M,  'xc3'         )
    call uio_wr(nc, model%xb1*CM_TO_M,  'xb1'         )
    call uio_wr(nc, model%xb2*CM_TO_M,  'xb2'         )
    call uio_wr(nc, model%xb3*CM_TO_M,  'xb3'         )
    call uio_wrlabl(nc,endbox_ident)

    call uio_wrlabl(nc,box_ident,date='now')
    call uio_wr(nc, 'atom',     box_id_ident   )
    call uio_wr(nc, model%itime     ,'itime'   )
    call uio_wr(nc, model%time      ,'time'    )
    call uio_wr(nc, atom%id         ,'id'      )                 
    call uio_wr(nc, atom%abun       ,'abun'    )
    call uio_wr(nc, atom%Nlevel     ,'Nlevel'  )
    call uio_wr(nc, atom%Nline      ,'Nline'   )
    call uio_wr(nc, atom%Ncont      ,'Ncont'   )
    call uio_wr(nc, atom%stage      ,'stage'   )
    call uio_wr(nc, atom%levelNo    ,'levelNo' )
    call uio_wr(nc, atom%E          ,'E'       )
    call uio_wr(nc, atom%g          ,'g'       )
    call uio_wr(nc, atom%label      ,'label'   )
    call uio_wr(nc, atom%ntot       ,'ntot'    )
!   call uio_wr(nc, atom%n          ,'n'       )

    if (quc_prop.eq.'hiontd') then
       ! --- write out time dependent number densities
       call uio_wr(nc, atom%ntd, 'ntd')
  !  else
  !     call uio_wr(nc, 0, 'ntd')
    end if

    call uio_wr(nc, atom%nstar, 'nstar')
    call uio_wr(nc, l_i,'l_i')
    call uio_wr(nc, l_j,'l_j')
    call uio_wr(nc, l_nTr,'l_nTr')
    call uio_wr(nc, l_f,'l_f')
    call uio_wr(nc, l_H,'l_H')
    call uio_wr(nc, l_nu0,'l_nu0')
    call uio_wr(nc, l_Trad,'l_Trad')
    call uio_wr(nc, l_tr_type,'l_tr_type')
    call uio_wr(nc, c_i,'c_i')
    call uio_wr(nc, c_j,'c_j')
    call uio_wr(nc, c_nTr,'c_nTr')
    call uio_wr(nc, c_alpha,'c_alpha')
    call uio_wr(nc, c_H,'c_H')
    call uio_wr(nc, c_nu0,'c_nu0')
    call uio_wr(nc, c_Trad,'c_Trad')
    call uio_wr(nc, c_tr_type,'c_tr_type')

    call uio_wrlabl(nc,endbox_ident)
    call uio_wrlabl(nc,enddataset_ident)
    call uio_closwr(nc)
   
    write(*,'(A)') 'HION: wrote ',filename,'.'

  end subroutine WriteAtom
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine ReadAtom(atom,atomfile,atmos,DustModel)
    !---------------------------------------------------------------------
    ! --- Reads atomic input file from atomfile into atom, a structure of 
    ! --- type atom_type.
    ! --- INPUT:
    ! ---   atom: atom_type
    ! ---   atomfile: char
    ! ---   atmos : atmos_type
    ! ---   DustModel : char, 'hion' or 'hiontd', if the latter, space
    ! ---               will be allocated for time dependent populations. 
    ! --- Feb 18 2005: 
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Feb 23 2005 : Added line/edge frequency calculation.
    ! --- Nov  9 2005 : Removed possibility of instantaneous NLTE
    ! ---               populations.
    !---------------------------------------------------------------------
    implicit none

    type(atom_type), intent(inout) :: atom
    type(atmos_type),intent(in)    :: atmos

    integer,          parameter    :: fileunit=20
    character(len=8), parameter    :: routinename='ReadAtom'


    character(len=*),intent(in)    :: atomfile,DustModel

    integer                        :: ios,k,l

    !---------------------------------------------------------------------

    open(unit=fileunit,file=atomfile,iostat=ios,status='OLD', &
         form='FORMATTED')
    if (ios.ne.0) call PrError(RoutineName, &
         'Failed to open '// atomfile, 2)

    ! --- read atom name and abundance ---
    call NextEntry(fileunit)
    read(unit=fileunit,fmt=*,iostat=ios) atom%id, atom%abun, atom%ielement
    if (ios.ne.0) call PrError(RoutineName,&
         'Failed to read atom ID,abundance or element number in ' // atomfile, 2)

    ! ---  read Nlevel,Nline,Ncont ---
    call NextEntry(fileunit)
    read(unit=fileunit,fmt=*,iostat=ios) atom%Nlevel, &
         atom%Nline,atom%Ncont
    if (ios.ne.0) call PrError(RoutineName, &
         'Failed to read Nlevel,Nline and/or Ncont in ' // atomfile, 2)

    ! --- allocate space for part of atom structure ---
    allocate( atom%stage(   1:atom%Nlevel))  ! --- ionization stage
    allocate( atom%levelNo( 1:atom%Nlevel))  ! --- level No
    allocate( atom%E(       1:atom%Nlevel))  ! --- level energy (eV)
    allocate( atom%g(       1:atom%Nlevel))  ! --- statistical weights
    allocate( atom%label(   1:atom%Nlevel))  ! --- level label
    allocate( atom%line(    1:atom%Nline) )  ! --- line info structs
    allocate( atom%cont(    1:atom%Ncont) )  ! --- continuum info structs 

    ! --- read level information ---
    call NextEntry(fileunit)
    do k=1,atom%Nlevel
       read(unit=fileunit,fmt=*,iostat=ios) &
            atom%E(k),atom%g(k),atom%label(k),atom%stage(k),atom%levelNo(k)
       if (ios.ne.0) call PrError(RoutineName, &
            'Failed to read level information in ' // atomfile, 2)    
    end do

    ! --- read bb transitions ---
    call NextEntry(fileunit)
    do k=1,atom%Nline
       read(unit=fileunit,fmt=*,iostat=ios) &
            atom%line(k)%j, atom%line(k)%i, atom%line(k)%f, &
            atom%line(k)%Trad, atom%line(k)%H, atom%line(k)%tr_type
       if (ios.ne.0) call PrError(RoutineName,              &
            'Failed to read bb transition in ' // atomfile,2)
       ! --- Calculate line frequency (Hz)
       atom%line(k)%nu0 = &
            ( atom%E(atom%line(k)%j) - atom%E(atom%line(k)%i) ) &
            * EV_TO_J / HPLANCK 
    end do

    ! --- read bf transitions ---
    call NextEntry(fileunit)
    do k=1,atom%Ncont
       read(unit=fileunit,fmt=*,iostat=ios) &
            atom%cont(k)%j, atom%cont(k)%i, atom%cont(k)%alpha, &
            atom%cont(k)%Trad, atom%cont(k)%H, &
            atom%cont(k)%tr_type
       if (ios.ne.0) call PrError(RoutineName, &
            'Failed to bf transition in ' // atomfile, 2)
       ! --- Calculate ionization edge frequency (Hz)
       atom%cont(k)%nu0 = &
            ( atom%E(atom%cont(k)%j) - atom%E(atom%cont(k)%i) ) &
            * EV_TO_J / HPLANCK 
    end do

    ! ---  read bound-bound collisional data ---
    call NextEntry(fileunit)
    read(unit=fileunit,fmt=*,iostat=ios) atom%bbcol%Ntemp
    atom%bbcol%Nline = atom%Nline

    allocate( atom%bbcol%i(atom%bbcol%Nline)               )
    allocate( atom%bbcol%j(atom%bbcol%Nline)               )
    allocate( atom%bbcol%temp(atom%bbcol%Ntemp)            )
    allocate( atom%bbcol%Cij(atom%Nline,atom%bbcol%Ntemp ) )

    read(unit=fileunit,fmt=*,iostat=ios) &
         (atom%bbcol%temp(k), k=1, atom%bbcol%Ntemp)

    do k=1,atom%bbcol%Nline
       read(unit=fileunit,fmt=*,iostat=ios) &           
            atom%bbcol%i(k),atom%bbcol%j(k), &
            (atom%bbcol%Cij(k,l),l=1,atom%bbcol%Ntemp) 
       if (ios.ne.0) call PrError(RoutineName, &
            'Failed to read bb collision coefficient in ' // &
            atomfile, 2)
    end do

    ! ---  read bound-free collisional data ---
    call NextEntry(fileunit)
    read(unit=fileunit,fmt=*,iostat=ios) atom%bfcol%Ntemp
    atom%bfcol%Ncont = atom%Ncont

    allocate( atom%bfcol%i(atom%bfcol%Ncont)                     )
    allocate( atom%bfcol%j(atom%bfcol%Ncont)                     )
    allocate( atom%bfcol%temp(atom%bfcol%Ntemp)                  )
    allocate( atom%bfcol%Cij(atom%bfcol%Ncont, atom%bfcol%Ntemp) )

    read(unit=fileunit,fmt=*,iostat=ios)  &
         (atom%Bfcol%temp(k), k=1, atom%Bfcol%Ntemp)

    do k=1,atom%bfcol%Ncont
       read(unit=fileunit,fmt=*,iostat=ios)  &
            atom%bfcol%i(k),atom%bfcol%j(k), &
            (atom%bfcol%Cij(k,l),l=1,atom%bfcol%Ntemp) 
       if (ios.ne.0) call PrError(RoutineName, &
            'Failed to read bf collision coefficient in ' // &
            atomfile, 2)
    end do

    close(fileunit)

    ! --- total number density ---
    allocate( atom%ntot(1:Atmos%n1, 1:Atmos%n2, 1:Atmos%n3) )
    atom%ntot=0.0
    
    ! --- time independent populations ---
    ! --- if (DustModel.eq.'hion') then
    ! ---    allocate( atom%n(1:Atmos%n1, 1:Atmos%n2, 1:Atmos%n3, 1:atom%Nlevel))
    ! ---    atom%n=0.0
    ! --- end if

    ! --- time dependent populations ---
    if (DustModel.eq.'hiontd') then
       allocate( atom%ntd(1:Atmos%n1, 1:Atmos%n2, 1:Atmos%n3, 1:atom%Nlevel))
       atom%ntd=0.0
    end if

    ! --- LTE populations
    allocate( atom%nstar(1:Atmos%n1, 1:Atmos%n2, 1:Atmos%n3, 1:atom%Nlevel)) 
    atom%nstar=0.0

    do l=1,atom%Nline
       allocate( atom%line(l)%nTr( atmos%n1, atmos%n2) )
       atom%line(l)%nTr=0.0
    end do

    do l=1,atom%Ncont
       allocate( atom%cont(l)%nTr( atmos%n1, atmos%n2 ) )
       atom%cont(l)%nTr=0.0
    end do

  end subroutine ReadAtom
  !---------------------------------------------------------------------

end module hion_io_module
!---------------------------------------------------------------------
