module hion_main_module
  !---------------------------------------------------------------------------
  ! NAME:
  !   hion_main_module ('hion_main_module')
  !
  ! PURPOSE:
  !   Chromospheric radiative transfer routines for rhd.
  !
  ! CATEGORY:
  !   Radiation transport
  !
  ! CALLING SEQUENCE:
  !   use hion_main_module
  !
  ! VARIABLES:
  !
  ! ROUTINES: (contained)
  !           hion_sourcestep(model,action,dtime,outst,ierr)
  !           InitHion(model)
  !           CalcHion(model,dtime)
  !           GetNstar(atom)
  !           GetColRates(atom)
  !           GetPopulations(dtime)
  !           CoolRate(atom,ii,jj)
  !           Rn_Sub(neq, t, y, ydot, rpar, ipar)
  !           Jac_Sub(neq, t, y, ml, mu, pd, nrowpd, rpar, ipar)
  !           OutputControl(dtime,model)
  !           InitChunk()
  ! MODIFICATION HISTORY:
  !  Apr  1 2005 : Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
  !  Apr  5 2005 : (JL) added new electron density and partition function 
  !                     calculation routines. Make sure Atoms(1) is Hydrogen.
  !                     This is needed in 'edens_calc_ne'
  !  May 10 2005 :(JL)  added chunk support
  !  Aug 08 2005: (SWB) small "bug" fix
  !  Nov  9 2005: (JL)  hiontd now intializes with LTE instead of 
  !                     instantaneous NLTE populations.
  !                     Atoms(1)%ntd is now multiplied by a factor
  !                     before being put in model%quc to avoid
  !                     numerical overflows
  !  Nov 15 2005: (SWB) changed quc input factor to cgs-SI conversion 
  !  Nov 16 2005: (SWB) OpenMP parallelisation, changed call to DVODE
  !  Aug 12 2006: (SWB) replaced input file with parameters 
  !---------------------------------------------------------------------
  use hion_def_module
  use hion_util_module
  use hion_io_module
  use rhd_gl_module ! - -- makes par%dustscheme parameter available
  use edens_module
  use timing_module

  implicit none

  save

  type(atom_type), dimension(:), pointer :: Atoms 
  type(atmos_type)  :: Atmos
  type(output_type) :: output 
  type(chunk_type)  :: chunk 
  !
  integer           :: Natoms, nchunks ! --- # of atoms to use, number of chunks            
  !
  logical                                    :: first_step=.TRUE.
  !
contains

  !---------------------------------------------------------------------
  subroutine hion_sourcestep(model,action,dtime,outstr,ierr)
  !--------------------------------------------------------------------- 

  use rhd_action_module
  use rhd_box_module
  use hion_def_module

  implicit none

  type(box_type), intent(inout)      :: model
  type(action_type),intent(in)       :: action
  real, intent(in)                   :: dtime
  integer,optional,intent(out)       :: ierr
  character*(*),optional,intent(out) :: outstr
 
  !---------------------------------------------------------------------

    write(*,'(A)') 'HION: beginning sourcestep...'

    if (first_step) then
       write(*,'(A)') 'HION: initializing...'
       call timing_start('HION: initialization')
       call edens_init()
       call edens_read_ne_table(par%hion_edensfile)
       call InitHion(model)
       call timing_stop('HION: initialization')
    end if

    ! --- put the advected level populations in the ntd array.
    if (par%dustscheme.eq.'hiontd') then 
       ! --- convert input from cgs (cm^-3) to in SI (m^-3) by 
       !     multiplying with 10^6 ---
       Atoms(1)%ntd = model%quc*1e6
    end if

    ! --- main routine call
    call CalcHion(model,dtime)

    ! --- output
    call OutputControl(dtime,model)

    ! --- if time dependent, put level pops in quc to be advected.
    if (par%dustscheme.eq.'hiontd') then 
       ! --- convert output from SI (m^-3) to cgs (cm^-3) to by 
       !     multiplying with 10^-6 ---
       model%quc= Atoms(1)%ntd*1e-6
    end if

    ierr = 0
    outstr = ''
    if (first_step) first_step = .FALSE.
    write(*,'(A)') 'HION: sourcestep finished.'

  end subroutine hion_sourcestep

  !---------------------------------------------------------------------
  subroutine InitHion(model)
    !---------------------------------------------------------------------
    ! --- Initializes variables for the hion approximate radiative 
    ! --- transfer module. In particular, it reads the atom files and
    ! --- allocates space for the populations and rates
    ! --- calculates all rates that are independent of time, electron 
    ! --- density and electron temperature
    ! --- Feb 23 2005 : 
    ! ---    Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !     Aug 08 2005 : (SWB) changed definition of AtomFiles
    !     Aug 12 2006 : replaced hion.input file with parameters 
    !---------------------------------------------------------------------
    ! RESTRICTIONS: 
    ! The current version handles only a single atomic species. 
    ! Will be extended later (change AtomFiles again). 
    !---------------------------------------------------------------------
    use hion_io_module
    use hion_def_module
    use rhd_box_module
    use gasinter_module

    implicit none

    type(box_type),intent(in)                    :: model
    integer                                      :: n1,n2,n3,k,l
    !character(len=80), dimension(:),     pointer :: AtomFiles
    character(len=80), dimension(1)              :: AtomFiles
    real,              dimension(:,:,:), pointer :: TT
    !---------------------------------------------------------------------

    ! --- define number of points in each dimension
    Atmos%n1 = model%n1 - model%m1 + 1
    Atmos%n2 = model%n2 - model%m2 + 1
    Atmos%n3 = model%n3 - model%m3 + 1

    ! --- Allocate memory for atmos struct

    allocate( Atmos%temp(  Atmos%n1, Atmos%n2, Atmos%n3) )
    allocate( Atmos%rho(   Atmos%n1, Atmos%n2, Atmos%n3) )
    allocate( Atmos%ne(    Atmos%n1, Atmos%n2, Atmos%n3) )
    allocate( Atmos%cmass( Atmos%n1, Atmos%n2, Atmos%n3) )

    ! --- prepare some important parameters (used to be read 
    !     from input file chromo.input) --- 
    Natoms=1 
    ! --- currently only one atom possible
    Atomfiles=par%hion_atomfile 
    nchunks=par%hion_chunks
    ! 
    ! --- output control ---    
    output%step=.FALSE.  
    output%time=.FALSE.
    output%stepdiff=0
    output%timediff=0.0

    if (par%dtime_out_hion > 0.0) then  
      output%time=.TRUE.
      output%timediff=par%dtime_out_hion
    end if
    if (par%dtime_out_hion < 0.0) then 
      output%step=.TRUE.
      output%stepdiff=par%dtime_out_hion
    end if
    !
    ! --- allocate the Atoms array
    allocate( Atoms(Natoms) )

    ! --- read atomic input files
    do k=1,Natoms
       call ReadAtom(Atoms(k),AtomFiles(k),Atmos,par%dustscheme)
    end do

    ! --- set up chunk arrays
    call InitChunk()

  end subroutine InitHion
  !---------------------------------------------------------------------
 
  !---------------------------------------------------------------------
  subroutine CalcHion(model,dtime)
    !---------------------------------------------------------------------
    ! --- Does all the necessary calculations to calculate the level
    ! --- populations and the cooling rates at a timestep
    ! --- Feb 23 2005 :
    ! ---    Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    use hion_def_module
    use rhd_box_module

    implicit none

    type(box_type),intent(in) :: model
    real,intent(in)           :: dtime

    integer                    :: k,l,m,y1,y2,ny
    character(len=8),parameter :: RoutineName='CalcHion'
    !---------------------------------------------------------------------

    ! --- mass densities, column mass, number density of atoms, electron density.
    call GetAtmosQuants(model)
   
    ! --- calculate the rate coefficients
    do k=1,Natoms

       ! --- Get the LTE populations
       call GetNstar(k)  

       do l=1,chunk%nchunks
          
          y1=chunk%index(1,l)
          y2=chunk%index(2,l)

          ! --- allocate rate coefficient chunks
          allocate(chunk%Rij(Atmos%n1,y2-y1+1,Atmos%n3,Atoms(k)%Nlevel,Atoms(k)%Nlevel))
          allocate(chunk%Cij(Atmos%n1,y2-y1+1,Atmos%n3,Atoms(k)%Nlevel,Atoms(k)%Nlevel))
          chunk%Rij=0.0
          chunk%Cij=0.0

          call timing_start('HION: collision rate coefficients')
          ! --- calculate the collisonal rate coefficients
          call GetColRates(k,y1,y2)
          call timing_stop('HION: collision rate coefficients')

          call timing_start('HION: radiative rate coefficients')
          ! --- calculate the radiative rate coefficients
          call GetRadRates(k,y1,y2)
          call timing_stop('HION: radiative rate coefficients')

          ! --- calculate the population numbers
          call GetPopulations(dtime,model,y1,y2)

          deallocate(chunk%cij,chunk%rij)

       end do
    end do

  end subroutine CalcHion
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine GetAtmosQuants(model)
    !---------------------------------------------------------------------
    ! --- Calculates mass densities, column mass, number density of atoms, 
    ! --- electron density.
    ! --- May 10 2005 : Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Apr 01 2009 : Corrected by Sven Wedemeyer-Boehm
    !---------------------------------------------------------------------  
    use hion_def_module
    use rhd_box_module
    use gasinter_module

    implicit none

    type(box_type),intent(in)      :: model 
    real,dimension(:,:,:), pointer :: TT
    real(kind=dbl), parameter      :: vacuum=1e-20 ! --- [m^-3] small atomic number density,
                                                   ! --- essentially vacuum 
    integer                        :: k,l,m,n
    !---------------------------------------------------------------------

    ! --- temperature from interpolation of eos table
    allocate (TT(atmos%n1,atmos%n2,atmos%n3) )
!    call eosinter3_T(model%rho, model%ei, T=TT)
    ! --- Call OpenMP version of EOS routines (apparently in non-OpenMP region) ---
    call eosinter_TOMP(model%rho, model%ei, TT, & 
                       model%m1, model%n1, & 
                       model%m2, model%n2, & 
                       model%m3, model%n3 )

    atmos%temp=TT
    deallocate(TT)
    nullify(TT)

    ! --- density in kg/m^3
    atmos%rho=model%rho * G_TO_KG / CM_TO_M**3

    ! --- column mass in kg/m^2
    ! --- uses model%m3/n3, because start index may differ between models    
    atmos%cmass(:,:,atmos%n3) = 1.0D-50 ! avoid log(zero) problems later
    do k = model%n3-1, model%m3, -1
       atmos%cmass(:,:,k-model%m3+1)  = atmos%cmass(:,:,k-model%m3+2)  + &
            0.5 * (atmos%rho(:,:,k+1) + atmos%rho(:,:,k) ) * &
            ( model%xc3(k+1)  - model%xc3(k) ) * CM_TO_M
    end do

    ! --- Total number density of atoms [m^-3]
    do k=1,Natoms
       Atoms(k)%ntot=atmos%rho * Atoms(k)%abun / AMU / WGHT_PER_H 
    end do

    ! --- Calculate electron density [m^-3] 
    if (.not.first_step) then
       ! --- use time dependent populations
       call timing_start('HION: interpolate electron densities')
       call edens_interpol_ne_hconst(Atmos%ne,Atoms(1)%ntot,atmos%temp, &
                                     Atoms(1)%ntd(:,:,:,6) / Atoms(1)%ntot, .TRUE. )
       call timing_stop('HION: interpolate electron densities')
    else
       ! --- first time step
       ! --- check whether quc arrays are non-empty
       if (maxval(model%quc) .ge. vacuum) then 
          ! --- non-empty
          write(*,'(A)') 'HION: Initializing electron densities with NLTE values'
          call timing_start('HION: Interpolate electron densities')
          call edens_interpol_ne_hconst(Atmos%ne,Atoms(1)%ntot,atmos%temp, &
                                        Atoms(1)%ntd(:,:,:,6) / Atoms(1)%ntot, .TRUE. )
          call timing_stop('HION: Interpolate electron densities')
       else 
          ! --- empty
          write(*,'(A)') 'HION: model%quc = 0, initializing electron densities with LTE values.'
          Atmos%ne=0.0
          call timing_start('HION: calculate electron dens.')
          call edens_calc_ne(Atoms(1)%ntot, Atmos%temp, Atmos%ne, .TRUE.)
          call timing_stop('HION: calculate electron dens.')
          write(*,'(A)') 'HION: finished initializing electron densities with LTE values.'
       endif
    endif



  end subroutine GetAtmosQuants
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine GetNstar(q)
    !---------------------------------------------------------------------
    ! --- GetNstar: Calculates the LTE populations. Adapted from Han 
    ! --- Uitenbroeks 'ltepops' c routine.
    ! --- Apr  5 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    use hion_def_module

    implicit none

    integer,intent(in)                                   :: q
    real(kind=dbl),dimension(atmos%n1,atmos%n2,atmos%n3) :: cNe_T,dE_kT,sum
    real(kind=dbl)                                       :: lnUk,gi0,dE
    integer                                              :: k,l,m,o,p,dz 
    real(kind=dbl),parameter                             :: k_b_ev= KBOLTZMANN / EV_TO_J, & ! [eV/K]
                                                            c1 = ( HPLANCK / (2.0*PI*M_ELECTRON) ) * (HPLANCK/KBOLTZMANN)
    !---------------------------------------------------------------------

    cNe_T = 0.5 * atmos%ne * (c1/atmos%temp)**1.5
    sum   = 1.0

    do k=2,Atoms(q)%Nlevel

       dE  = Atoms(q)%E(k) - Atoms(q)%E(1)
       gi0 = Atoms(q)%g(k) / Atoms(q)%g(1)
       dZ  = Atoms(q)%stage(k) - Atoms(q)%stage(1)      
       dE_kT = dE / (k_b_ev * atmos%temp)

       Atoms(q)%nstar(:,:,:,k) = gi0 * exp(-1.0*dE_kT)

       do p=1,dZ
          Atoms(q)%nstar(:,:,:,k) = Atoms(q)%nstar(:,:,:,k) / cNe_T
       end do
       sum = sum + Atoms(q)%nstar(:,:,:,k) 

    end do

    Atoms(q)%nstar(:,:,:,1) = Atoms(q)%ntot / sum

    do k=2,Atoms(q)%Nlevel
       Atoms(q)%nstar(:,:,:,k) = Atoms(q)%nstar(:,:,:,k) * &
            Atoms(q)%nstar(:,:,:,1)
    end do

  end subroutine GetNstar
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine GetColRates(q,y1,y2)
    !---------------------------------------------------------------------
    ! --- Calculates the collisional rate coefficients, given the
    ! --- temperature, the electron density and the atom struct.
    ! --- Based on Han Uitenbroeks rh-code.
    ! --- Feb 28 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Mar 23 2005 : (J.L.) replaced forall loops by whole array operations.
    ! --- May 10 2005 : (J.L.) Works now per chunk.
    !---------------------------------------------------------------------

    use hion_def_module

    implicit none

    integer,intent(in)                                  :: q,y1,y2 ! --- atom index,chunk y indices
    real(kind=dbl),dimension(atmos%n1,y2-y1+1,atmos%n3) :: C,Cij
    integer                                             :: k,l,m,n,i,j,ny
    real(kind=dbl)                                      :: dE  
    !---------------------------------------------------------------------

    ! --- Rij(i,j) holds the transition j->i
    
    ! --- bound-bound transitions
    do k=1,Atoms(q)%bbcol%nline

       call Linear(Atoms(q)%bbcol%Ntemp, Atoms(q)%bbcol%temp, &
            Atoms(q)%bbcol%cij(k,:), atmos%n1, y2-y1+1, &
            atmos%n3,atmos%temp(:,y1:y2,:),C)

       i  = min( Atoms(q)%bbcol%i(k), Atoms(q)%bbcol%j(k) )
       j  = max( Atoms(q)%bbcol%i(k), Atoms(q)%bbcol%j(k) )

       ! --- downward rate j->i
       Cij= C * atmos%ne(:,y1:y2,:) * sqrt( atmos%temp(:,y1:y2,:) ) * &
            Atoms(q)%g(i) / Atoms(q)%g(j)
       chunk%Cij(:,:,:,i,j) = Cij     

       ! --- upward rate i->j
       chunk%Cij(:,:,:,j,i) = & 
            Cij * Atoms(q)%nstar(:,y1:y2,:,j) / Atoms(q)%nstar(:,y1:y2,:,i)

    end do

    ! --- bound-free transitions
    do k=1,Atoms(q)%bfcol%ncont

       call Linear(Atoms(q)%bfcol%Ntemp, Atoms(q)%bfcol%temp, &
            Atoms(q)%bfcol%cij(k,:), atmos%n1, y2-y1+1, &
            atmos%n3,atmos%temp(:,y1:y2,:),C)

       i  = min( Atoms(q)%bfcol%i(k), Atoms(q)%bfcol%j(k) )
       j  = max( Atoms(q)%bfcol%i(k), Atoms(q)%bfcol%j(k) )
       dE = Atoms(q)%E(j) - Atoms(q)%E(i)

       ! --- upward rate i->j
       Cij = C * atmos%ne(:,y1:y2,:) * sqrt( atmos%temp(:,y1:y2,:) ) * &
            exp( (-1.0*dE*EV_TO_J) / ( KBOLTZMANN*atmos%temp(:,y1:y2,:) ) )
       chunk%Cij(:,:,:,j,i) = Cij

       ! --- downward rate j->i
       chunk%Cij(:,:,:,i,j) = &
            Cij * Atoms(q)%nstar(:,y1:y2,:,i) / Atoms(q)%nstar(:,y1:y2,:,j)

    end do

  end subroutine GetColRates
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine GetRadRates(k,y1,y2)
    !---------------------------------------------------------------------
    ! --- GetRadRates: Calculates the fixed radiative rate coefficients
    ! ---              for Atoms(k) and chunk delimited by (y1,y2)
    ! --- Apr  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    implicit none

    integer,intent(in)                      :: k,y1,y2
    real(kind=dbl),dimension(:,:,:),pointer :: Tr
    integer                                 :: l
    !---------------------------------------------------------------------

    allocate( Tr(Atmos%n1,y1:y2,atmos%n3) )

    ! --- loop over bound-bound transitions
    do l=1,Atoms(k)%Nline
       if (Atoms(k)%line(l)%tr_type.eq.'FIXED') then 

          Tr=0.0
          ! --- get radiation temperature array
          call InitTrBb(k,l,y1,y2,Tr)

          ! --- get the bound-bound radiative rate coefficients
          call FixedBbRadRates(k,l,y1,y2,Tr)
 
       end if
    end do

    ! --- loop over bound-free transitions
    do l=1,Atoms(k)%Ncont
       if (Atoms(k)%cont(l)%tr_type.eq.'HYDRO') then

          Tr=0.0
          ! --- get radiation temperature array
          call InitTrBf(k,l,y1,y2,Tr)

          ! --- get the bound-free radiative rate coefficients
          call FixedBfRadRates(k,l,y1,y2,Tr)

       end if
    end do

    deallocate(Tr)

  end subroutine GetRadRates
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine InitTrBb(k,l,y1,y2,Tr)
    !---------------------------------------------------------------------
    ! --- Sets up the radiation temperature array for bb transitions
    ! --- Apr  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Apr 27 2005 : Added code so that program no longer stops if
    ! ---   a column is hotter than max(Trad). Added Trad=Te below nz0
    !---------------------------------------------------------------------
    use hion_def_module
    use hion_io_module
    use hion_util_module 
    !
    implicit none
    !
    ! --- I/O ---
    integer,intent(in)              :: k,l, & ! --- atom and line identifier
                                       y1,y2  ! --- indices for current chunk 
    real(kind=dbl), dimension(:,:,:),       &
                    pointer         :: Tr     ! --- radiation temperature array
    !
    ! --- parameter ---
    character(len=8)                    :: RoutineName='InitTrBb' 
    !
    ! --- local variables ---
    integer                             :: m,n,o,nz0
    real(kind=dbl), dimension(Atmos%n3) :: J
    real(kind=dbl)                      :: nu0,J_tau0,Tr_max,cmass0
    !
    !---------------------------------------------------------------------

    nu0 = Atoms(k)%line(l)%nu0
    ! --- get maximum radiation temperature
    J_tau0 = Planck(Atoms(k)%line(l)%Trad,nu0)
    Tr_max = Trad(2.0*J_tau0,nu0)

    ! --- loop over horizontal positions
    !$OMP PARALLEL DEFAULT(shared), &
    !$OMP          PRIVATE(m,n,o,nz0,cmass0,J)
    !$OMP DO
    do m = 1, atmos%n1
       do n = y1, y2

          ! --- get index of lowest point in atmos where 
          ! --- T_e < Trad_max
          do o=1,atmos%n3
             if (atmos%temp(m,n,o).le.Tr_max) then
                nz0=o
                cmass0=atmos%cmass(m,n,nz0)
                Atoms(k)%line(l)%nTr(m,n)=nz0
                exit
             end if
             if (o.eq.atmos%n3) then
                  ! --- entire column is hotter, seems fishy but continue
                  call PrError(RoutineName,'Failed to find Te < max(Trad)',1)
                  write(*,*) '      Transition',Atoms(k)%line(l)%i,'->',Atoms(k)%line(l)%j,','
                  write(*,*) '      at (x,y)=(',m,',',n,').'     
                  nz0=atmos%n3
                  cmass0=atmos%cmass(m,n,nz0)
                  Atoms(k)%line(l)%nTr(m,n)=nz0
                  exit
             end if          
          end do

          ! --- Now calculate Trad(z) at and above nz0
          if (nz0.ne.atmos%n3) then
             J(nz0:) = J_tau0 + ( Planck( Atmos%temp(m,n,nz0),nu0 ) - J_tau0 ) * &
                  exp( Atoms(k)%line(l)%H * log( Atmos%cmass(m,n,nz0:) / cmass0 ) )
             do o=nz0,atmos%n3
                Tr(m,n,o) = Trad(J(o),nu0)
             end do
          end if

          ! --- Below nz0 Trad=Te
          if (nz0.gt.1) Tr(m,n,1:nz0) = Atmos%temp(m,n,1:nz0)

       end do
    end do
    !$OMP END DO NOWAIT
    !$OMP END PARALLEL

  end subroutine InitTrBb
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine InitTrBf(k,l,y1,y2,Tr)
    !---------------------------------------------------------------------
    ! --- Sets up the radiation temperature array for bf transitions
    ! --- Apr  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Apr 27 2005 : Added code so that program no longer stops if
    ! ---   a column is hotter than max(Trad).
    ! --- Apr 27 2005 : Added Trad=Te below nz0
    !---------------------------------------------------------------------
    use hion_def_module
    use hion_io_module
    use hion_util_module 
    !
    implicit none
    !
    ! --- I/O ---
    integer,intent(in)              :: k,l, & ! --- atom and cont identifier
                                       y1,y2  ! --- indices for current chunk 
    real(kind=dbl), dimension(:,:,:),       &
                    pointer         :: Tr     ! --- radiation temperature array
    !
    ! --- parameter ---
    character(len=8),parameter         :: RoutineName='InitTrBf' 
    !
    ! --- local variables ---
    integer                            :: m,n,o,nz0
    real(kind=dbl),dimension(atmos%n3) :: J
    real(kind=dbl)                     :: nu0,J_tau0,Tr_max,cmass0
    !---------------------------------------------------------------------

    nu0 = Atoms(k)%cont(l)%nu0
    ! --- get maximum radiation temperature
    J_tau0 = Planck(Atoms(k)%cont(l)%Trad,nu0)
    Tr_max = Trad(2.0*J_tau0,nu0)

    ! --- loop over horizontal positions
    !$OMP PARALLEL DEFAULT(shared), &
    !$OMP          PRIVATE(m,n,o,nz0,cmass0,J)
    !$OMP DO
    do m = 1, atmos%n1
       do n = y1, y2

          ! --- get index and column mass of lowest point in atmos where 
          ! --- Trad_max > T_e
          do o=1,Atmos%n3
             if (Atmos%temp(m,n,o).le.Tr_max) then
                nz0=o
                cmass0=atmos%cmass(m,n,nz0)
                Atoms(k)%cont(l)%nTr(m,n)=nz0
                exit
             end if
             if (o.eq.atmos%n3) then
                ! --- entire column hotter, seems fishy but continue
                call PrError(RoutineName,'Failed to find Te < max(Trad)',1)
                write(*,*) '      Transition',Atoms(k)%cont(l)%i,'->',Atoms(k)%cont(l)%j,','
                write(*,*) '      at (x,y)=(',m,',',n,').'     
                nz0=atmos%n3
                cmass0=atmos%cmass(m,n,nz0)
                Atoms(k)%cont(l)%nTr(m,n)=nz0
                exit
             end if
          end do

          ! --- Now calculate Trad(z) at and above nz0
          if (nz0.ne.atmos%n3) then
             J(nz0:) = J_tau0 + ( Planck( Atmos%temp(m,n,nz0),nu0 ) - J_tau0 ) * &
                  exp( Atoms(k)%cont(l)%H * log( Atmos%cmass(m,n,nz0:) / cmass0 ) )
             do o=nz0,atmos%n3
                Tr(m,n,o) = Trad(J(o),nu0)
             end do
          end if
          ! --- Below nz0 Trad=Te
          if (nz0.gt.1) Tr(m,n,1:nz0) = Atmos%temp(m,n,1:nz0)

       end do
    end do
    !$OMP END DO NOWAIT
    !$OMP END PARALLEL

  end subroutine InitTrBf
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine FixedBbRadRates(k,l,y1,y2,Tr)
    !---------------------------------------------------------------------
    ! --- Calculates the bound bound radiative rates for the fixed 
    ! --- radiative temperature area. 
    ! --- Mar  7 2005 :
    ! ---   created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- Apr 27 2005 : Added bb rad rates for deep layers to avoid too 
    ! --- slow, because only collisional, rates in the LTE area.
    !---------------------------------------------------------------------
    use hion_def_module
    use hion_util_module
    use hion_io_module
    !
    implicit none
    !
    ! --- I/O ---
    integer,intent(in)              :: k,l, & ! --- atom and line identifier
                                       y1,y2  ! --- indices for current chunk 
    real(kind=dbl), dimension(:,:,:),       &
                    pointer         :: Tr     ! --- radiation temperature array
    !
    ! --- parameter ---
    real(kind=dbl),    parameter :: &
                C = (8.0 * PI**2 * Q_ELECTRON**2) / (M_ELECTRON * CLIGHT**3)
    character(len=15), parameter :: RoutineName='FixedBbRadRates'
    !
    ! --- local variables ---
    integer :: m,n,o,i,j,ntr
    real(kind=dbl) :: hnu_k, C0, gij 
    !
    !---------------------------------------------------------------------

    ! --- radrates(i,j) holds the transition j->i

    ! --- set lower and upper level identifier
    i = min( Atoms(k)%line(l)%i, Atoms(k)%line(l)%j )
    j = max( Atoms(k)%line(l)%i, Atoms(k)%line(l)%j )

    ! --- (semi-)constants
    C0     = C * Atoms(k)%line(l)%nu0**2 * Atoms(k)%line(l)%f
    hnu_k  = HPLANCK * Atoms(k)%line(l)%nu0 / KBOLTZMANN
    gij    = Atoms(k)%g(i) / Atoms(k)%g(j)

    ! --- upward rate i->j 
    chunk%Rij(:,:,:,j,i) = C0 / ( exp( hnu_k / Tr ) - 1.0 )    
    ! --- downward rate j->i 
    chunk%Rij(:,:,:,i,j) = chunk%Rij(:,:,:,j,i) * gij * exp( hnu_k / Tr )
        
  end subroutine FixedBbRadRates
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine FixedBfRadRates(k,l,y1,y2,Tr)
    !---------------------------------------------------------------------
    ! --- Calculates the upward bound free radiative rates for the fixed 
    ! --- radiative temperature area. 
    ! --- Apr  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl).
    ! --- Apr 27 2005 : Added bb rad rates for deep layers to avoid too 
    ! ---               slow, because only collisional, rates in the LTE 
    ! ---               area.
    !---------------------------------------------------------------------
    use hion_def_module
    use hion_util_module
    use hion_io_module
    ! 
    implicit none
    !
    ! --- I/O ---
    integer,intent(in)              :: k,l, & ! --- atom and cont identifier
                                       y1,y2  ! --- indices for current chunk 
    real(kind=dbl), dimension(:,:,:),       &
                    pointer         :: Tr     ! --- radiation temperature array
    !
    ! --- parameter ---
    real(kind=dbl), parameter       :: C = 8.0 * PI / CLIGHT**2, &
                                       tolerance = 1.0E-5
    integer,parameter               :: MaxSum = 100
    character(len=15),parameter     :: RoutineName='FixedBfRadRates'
    !
    ! --- local variables ---
    integer                         :: m,n,o,p,i,j,nTrMin
    real(kind=dbl)                  :: c0,sigma,En,hnu_kT ,hnu_kTr,hnu_kTe, &
                                       hnu_k
    !---------------------------------------------------------------------

    ! --- lower, upper level identifier
    i = min( Atoms(k)%cont(l)%i, Atoms(k)%cont(l)%j )
    j = max( Atoms(k)%cont(l)%i, Atoms(k)%cont(l)%j )

    ! --- (semi-)constants
    hnu_k = HPLANCK * Atoms(k)%cont(l)%nu0  / KBOLTZMANN
    c0  = C *  Atoms(k)%cont(l)%alpha *  Atoms(k)%cont(l)%nu0**3
    nTrMin=minval(Atoms(k)%cont(l)%nTr)

    !$OMP PARALLEL DEFAULT(shared), &
    !$OMP          PRIVATE(m,n,o,p, hnu_kT, sigma, En, hnu_kTr, hnu_kTe)
    !$OMP DO
    do o=1,atmos%n3           ! --- z
       do n=y1,y2             ! --- y
          do m=1,atmos%n1     ! --- x
             ! --- upward rate coefficients i->j
             ! --- get sum of exponential integrals
             hnu_kT = hnu_k / Tr(m,n,o) 
             sigma  = 0.0     
             do p=1, MaxSum
                   En    = E1(  p * hnu_kT )        
                sigma = sigma + En
                if ( (En/sigma).le.tolerance ) exit           
                if (p.eq.MaxSum) then
                   call PrError(RoutineName, &
                     'Sum of exponential integrals did not converge.', 1)
                   write(*,*) 'Upward: x=',m,' y=',n,' z=',o
                end if
             end do
             ! --- Update rate coefficient
             chunk%Rij(m,n,o,j,i) = C0 * sigma

             ! --- downward rate coefficients j->i
             hnu_kTr= hnu_k / Tr(m,n,o)
             hnu_kTe = hnu_k / atmos%temp(m,n,o)
             ! --- get sum of exponential integrals 
             sigma  = E1(hnu_kTe)
             do p=1,MaxSum
                En    = E1( p*hnu_kTr + hnu_kTe)
                sigma = sigma + En
                if (En/sigma.le.tolerance) exit
                if (p.eq.MaxSum) then
                   call PrError(RoutineName,&
                        'Sum of exponential integrals did not converge.', 1) 
                   write(*,*) 'Downward: x=',m,' y=',n,' z=',o
                end if
             end do
             ! --- Update rate coefficient
             chunk%Rij(m,n,o,i,j) = &
                  C0 * Atoms(k)%nstar(m,n,o,i) / Atoms(k)%nstar(m,n,o,j) * sigma

          end do
       end do
    end do
    !$OMP END DO NOWAIT
    !$OMP END PARALLEL

  end subroutine FixedBfRadRates
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine GetPopulations(dtime,model,y1,y2)
    !---------------------------------------------------------------------
    ! --- Solves the statistical equilibrium equations for all atoms
    ! --- at each spatial location. Uses BLAS, LAPACK and DVODE routines. 
    ! --- Mar 09 2005 :
    ! ---    Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! --- May 19 2005 : (JL) Various updates in the last months
    ! --- Nov  9 2005 : (JL) more updates in later months 
    !---------------------------------------------------------------------
    use hion_def_module
    use hion_util_module
    use hion_dvode_module
    use rhd_box_module

    implicit none

    ! --- I/O ---
    integer, intent(in) :: y1,y2 ! chunk lower and upper indices
    real,    intent(in) :: dtime
    type(box_type)      :: model

    ! --- local variables ---
    integer                                  :: k,i1,i2,i3,o,Nlevel,irepl,info,ny
    real(kind=dbl)                           :: nsum, dev

    character(len=14),parameter :: RoutineName='GetPopulations'
    real(kind=dbl),parameter    :: vacuum=1.0e-20

    real(kind=dbl), dimension(:,:), allocatable :: gamma

    ! --- I/O for call to DVODE package ---
    real(kind=dbl)                            :: dbldtime
    integer                                   :: istate
   !real(kind=dbl), dimension(:), allocatable :: ndummy
    real(kind=dbl), dimension(1:6)            :: ndummy


    ! integer :: OMP_GET_THREAD_NUM, OMP_GET_NUM_THREADS
    logical :: OMP_GET_DYNAMIC

    !---------------------------------------------------------------------

    dbldtime=dble(dtime)
 
    do k=1,Natoms

       nlevel = Atoms(k)%Nlevel

       if (par%dustscheme == 'hiontd') then

          ! --- do the time dependent calculation
          if (first_step) then
             if (maxval(model%quc).le.vacuum) then
                Atoms(k)%ntd=real(Atoms(k)%nstar)
             end if
          end if

          ! --- loop over atmosphere points
          call timing_start('HION: DVODE loop')
          !$OMP PARALLEL DEFAULT(PRIVATE), &
          !$OMP          SHARED(k,nlevel,chunk,dbldtime,atoms,atmos,y1,y2)
          !$OMP DO SCHEDULE(DYNAMIC,1)
          do i3=1,atmos%n3
             do i2=y1,y2
                do i1=1,atmos%n1
                   !
                   ! --- fill rate matrix
                   allocate( gamma(1:nlevel,1:nlevel))
                   gamma(1:nlevel,1:nlevel) = chunk%rij(i1,i2,i3,1:nlevel,1:nlevel) & 
                                            + chunk%cij(i1,i2,i3,1:nlevel,1:nlevel)
                   do o=1,nlevel 
                      gamma(o,o) = -1.0*sum(gamma(1:nlevel,o))
                   end do

                   ! --- fill vector with level populations ---
                   ndummy(1:nlevel) = dble(Atoms(k)%ntd(i1,i2,i3,1:nlevel))

                   ! --- call solver ---
                   istate=1
                   call hion_dvode(nlevel, ndummy(1:nlevel), gamma(1:nlevel,1:nlevel), & 
                                   dbldtime, istate)

                   ! --- free memory ---
                   deallocate(gamma) 

                   ! --- check for proper function of DVODE ---
                   if (istate .ne. 2) then
                      write(*,'(A,I3,A,3(I3,1X),A)') 'HION: Error in DVODE, istate = ',istate, & 
                                                     ' at (x,y,z):', i1,i2,i3,'with'
                      write(*,*) '     T,rho,ne',real(atmos%temp(i1,i2,i3)), & 
                                 real(atmos%rho(i1,i2,i3)), real(atmos%ne(i1,i2,i3))
                      write(*,'(A)') '      Setting populations to values from previous timestep.'
                      ndummy(1:nlevel) = dble(Atoms(k)%ntd(i1,i2,i3,1:nlevel))
                   end if

                   ! --- total number density ---
                   nsum = sum(ndummy)

                   ! --- deviation ---
                   dev = nsum/Atoms(k)%ntot(i1,i2,i3)-1.0

                   ! --- check for particle conservation
                   if (dev .ge. 1.0E-4) then
                      write(*,*) 'HION: Particle density is not conserved at:', &
                           i1,i2,i3,'with dn/n =', dev
                   end if

                   ! --- copy back into atom struct, but enforce exact particle
                   ! --- conservation by scaling with ntot.
                   Atoms(k)%ntd(i1,i2,i3,:) = real(ndummy(:) * Atoms(k)%ntot(i1,i2,i3) / nsum )
                   !              
                end do
             end do
          end do
          !$OMP END DO
          !$OMP END PARALLEL
          call timing_stop('HION: DVODE loop')

       end if

     end do ! --- loop over atoms ---

  end subroutine GetPopulations
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
!!$  function CoolRate(atom,ii,jj)
!!$    !---------------------------------------------------------------------
!!$    ! --- Calculates the cooling rate in the transition between i and j.
!!$    ! --- This is the cooling rate, so a negative cooling rate means the
!!$    ! --- gas is heated up in this particular transition.
!!$    ! --- Mar 23 2005: 
!!$    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
!!$    ! --- Apr  7 2005 :
!!$    ! --- DUE TO CHANGES IN THE REST OF THE CODE, THE RADRATES ARE NO 
!!$    ! --- LONGER EXPLICITLY CALCULATED, SO THIS ROUTINE WON'T WORK
!!$    !---------------------------------------------------------------------
!!$    use hion_def_module
!!$    implicit none
!!$
!!$    real(kind=dbl),dimension(atmos%n1,atmos%n2,atmos%n3) :: CoolRate
!!$    type(atom_type),intent(in) :: atom
!!$
!!$    integer,intent(in) :: ii,jj
!!$
!!$    integer :: i,j,k,l,m,n,ntr
!!$    real(kind=dbl) :: C,C0,term1,term2,term3,Hnu_K,x0,fdummy
!!$
!!$    real(kind=dbl),parameter :: H_K = HPLANCK / KBOLTZMANN,tolerance=1e-5
!!$    character(len=8),parameter :: RoutineName='CoolRate'
!!$    !---------------------------------------------------------------------
!!$
!!$    ! --- make sure j>i
!!$    if (jj.lt.ii) then 
!!$       j=ii
!!$       i=jj
!!$    else if (jj.gt.ii) then
!!$       i=ii
!!$       j=jj
!!$    else 
!!$       call PrError(Routinename,'Upper and lower level i and j are the same',2)
!!$    end if
!!$
!!$    ! --- check for bb or bf 
!!$    if (atom%stage(j).gt.atom%stage(i)) then
!!$
!!$       ! --- bound free transition    
!!$
!!$       ! --- determine transition number
!!$       do k=1,atom%Ncont
!!$          if (j.eq.atom%cont(k)%j .and. i.eq.atom%cont(k)%i) exit
!!$       end do
!!$
!!$       C = 8.0 * PI * atom%cont(k)%nu0**3 * atom%cont(k)%alpha * &
!!$            KBOLTZMANN / CLIGHT**2
!!$       Hnu_K = H_K * atom%cont(k)%nu0
!!$
!!$       do m=nTr,atmos%n3
!!$
!!$          C0 = C * atom%cont(k)%Tr(m)
!!$          x0= Hnu_K / atom%cont(k)%Tr(m)
!!$
!!$          do l=1,atmos%n2
!!$             do k=1,atmos%n1
!!$
!!$                term1 = atom%n(k,l,m,j) / atom%nstar(k,l,m,j) * &
!!$                     atom%nstar(k,l,m,i) / atom%n(k,l,m,i) * &
!!$                     exp(Hnu_K/Atmos%temp(k,l,m))
!!$                term2 = 0.0
!!$
!!$                do n=0,100
!!$
!!$                   fdummy = exp(-1.0*n*x0) / &
!!$                        ( n + atom%cont(k)%Tr(m) / Atmos%temp(k,l,m) )
!!$                   term2 = term2 + fdummy
!!$                   if (fdummy/term2.le.tolerance) exit
!!$                   if (n.eq.100) call PrError(RoutineName, & 
!!$                        'Sum in bf cooling rate did not converge',1)
!!$
!!$                end do
!!$
!!$                term3 = log( 1.0- exp(x0) )
!!$                CoolRate(k,l,m) = C0 * ( ( term1 * term2 ) + term3 )
!!$
!!$             end do
!!$          end do
!!$       end do
!!$
!!$       return
!!$
!!$    else if (atom%stage(j).eq.atom%stage(i)) then
!!$
!!$       ! --- bound-bound transition    
!!$       ! --- determine transition number
!!$       do k=1,atom%Nline
!!$          if (j.eq.atom%line(k)%j .and. i.eq.atom%line(k)%i) exit
!!$       end do
!!$       ! --- remember radrates(j,i) has the rate i->j 
!!$       CoolRate = HPLANCK * atom%line(k)%nu0 * &
!!$            ( atom%n(:,:,:,j) * atom%radrates(:,:,:,i,j) &
!!$            - atom%n(:,:,:,i) * atom%radradrates(:,:,:,j,i) )
!!$       return       
!!$
!!$    else
!!$       call PrError(Routinename, &
!!$            'Stage(j) smaller than stage(i). Mistake in atom file?',2)
!!$    end if
!!$
!!$  end function CoolRate
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine OutputControl(dtime,model)
!---------------------------------------------------------------------
! --- Controls the special HION output.
! --- one can choose to produce output after a fixed amount of solar 
! --- time, a fixed number of timesteps, both, or none
!---------------------------------------------------------------------
    use rhd_box_module
    implicit none

    real, intent(in)          :: dtime
    type(box_type),intent(in) :: model

    real,    save        :: simtime=0.0
    integer, save        :: timestep=0, stepnout=1, timenout=1
!!    real,                :: simtime=0.0
!!    integer              :: timestep=0, stepnout=1, timenout=1
    integer              :: k
    character(len=4)     :: outstr
    character(len=80)    :: filename
    !---------------------------------------------------------------------

    ! --- update the time and step counters
    simtime = simtime + dtime
    timestep = timestep + 1
    
    ! --- output after number of steps
    if (output%step) then
       if (mod(timestep,output%stepdiff).eq.0) then
          
          write(outstr,fmt='(i4.4)') stepnout

          do k=1,Natoms
             filename= 'HION.step.' // trim(adjustl(Atoms(k)%ID))//'.' // outstr // '.out'
             call WriteAtom(atoms(k),Atmos,model,Atmos%n1,Atmos%n2,Atmos%n3, &
                  trim(adjustl(filename)), par%dustscheme )
          end do

          stepnout=stepnout+1

       endif
    endif

    ! --- output after fixed amount of solar time
    if (output%time) then
       if (simtime.ge.(timenout*output%timediff)) then
          
          write(outstr,fmt='(i4.4)') timenout

          do k=1,Natoms
             filename= 'HION.time.' // trim(adjustl(Atoms(k)%ID))//'.' // outstr // '.out'
             call WriteAtom(atoms(k),Atmos,model,Atmos%n1,Atmos%n2,Atmos%n3, &
                  trim(adjustl(filename)), par%dustscheme)
          end do

          timenout=timenout+1

       endif
    endif


  end subroutine OutputControl
!---------------------------------------------------------------------

!---------------------------------------------------------------------
  subroutine InitChunk()
    !---------------------------------------------------------------------
    ! --- initializes chunk sizes and indices
    ! --- 
    !---------------------------------------------------------------------

    implicit none

    integer :: k,a1,a2,s1,s2
    character(len=9),parameter :: RoutineName='InitChunk'
    !---------------------------------------------------------------------

    if ((nchunks.eq.1).or.(Atmos%n2.eq.1)) then
      ! --- whole atmos at once
       chunk%nchunks=1
       allocate(chunk%index(2,1))
       chunk%index(1,1)=1
       chunk%index(2,1)=Atmos%n2
    else if (nchunks.gt.Atmos%n2) then
       ! --- to many chunks requested
       call PrError(RoutineName,&
            'N chunks  greater than number of cells in x2 direction, exiting...',2)
    else
       ! --- sane number of chunks requested
       chunk%nchunks=nchunks
       allocate(chunk%index(2,chunk%nchunks))
       s1=atmos%n2/chunk%nchunks

       if (float(atmos%n2)/float(chunk%nchunks)-s1.le.1e-5) then
          ! --- all chunks same size
          do k=1,chunk%nchunks
             chunk%index(1,k)= (k-1) * s1 + 1
             chunk%index(2,k)= k * s1
          end do
       else
          ! --- 2 sizes
          s2=s1+1
          a1 = -1.0*atmos%n2 + chunk%nchunks * s2
          a2 = chunk%nchunks - a1
          do k=1,a1
             chunk%index(1,k) = (k-1) * s1 + 1
             chunk%index(2,k) = k * s1
          end do
          do k=a1+1,chunk%nchunks
             chunk%index(1,k) = chunk%index(2,a1)  + (k-a1-1) * s2 + 1
             chunk%index(2,k) = chunk%index(2,a1)  +  (k-a1) * s2
          end do
       end if

    end if

    write(*,'(A)')        'HION: chunk info:'
    write(*,'(6X,A8,I4)') 'nchunks:',chunk%nchunks
    write(*,'(6X,A14)')   'chunk indices:'
    write(*,'(6X,2I5.4)') chunk%index

  end subroutine InitChunk
!---------------------------------------------------------------------

end module hion_main_module
!---------------------------------------------------------------------
