module hion_util_module
!---------------------------------------------------------------------------
! NAME:
!   hion_util_module ('hion_util_module')
!
! PURPOSE:
!   contains various useful routines for HION
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   use hion_util_module
!
! VARIABLES:
!
! ROUTINES: (contained)
!   Planck(temp,nu),
!   Trad(I,nu),
!   Linear(Ntable, xtable, ytable, n1,n2,n3, x, y), 
!   Locate(n,xx,x,j),
!   E1(x)
!   imaxloc(x)
!
! MODIFICATION HISTORY:
!   Feb 20 2005 : Written by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
!---------------------------------------------------------------------------
  implicit none

contains

  !---------------------------------------------------------------------
  function Planck(temp,nu)
    !---------------------------------------------------------------------
    ! --- calculates Planck function [J/ (s m^2 ster Hz)]
    ! --- INPUT: 
    ! ---   temp: temperature [K]
    ! ---   nu: frequency [Hz]
    ! --- Mar  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl). 
    !---------------------------------------------------------------------
    use hion_def_module
    implicit none
    real(kind=dbl) :: Planck 
    real(kind=dbl),intent(in) :: temp,nu
    real(kind=dbl) ::  twohnu3_c2,hnu_k

    twohnu3_c2 = 2.0*HPLANCK*nu**3/CLIGHT**2
    hnu_k = HPLANCK*nu /KBOLTZMANN

    Planck = twohnu3_c2 / (exp(hnu_k/temp) - 1.0)

  end function Planck
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  function Trad(I,nu)
    !---------------------------------------------------------------------
    ! --- Calculates radiation temperature from given intensity at 
    ! --- frequency nu
    ! --- INPUT:
    ! ---   I: Intensity [J/ (s m^2 ster Hz)]
    ! ---   nu: Frequency [Hz] 
    ! --- OUTPUT: 
    ! ---   Trad: Radiation temperature [K] 
    ! --- Mar  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl). 
    !---------------------------------------------------------------------
    use hion_def_module
    implicit none

    real(kind=dbl) :: Trad
    real(kind=dbl),intent(in) :: I,nu

    Trad = HPLANCK*nu / &
         (KBOLTZMANN * log(1.0 + (2.0*HPLANCK*nu**3)/(CLIGHT**2 * I) ) )

  end function Trad
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine Linear(Ntable, xtable, ytable, n1, n2, n3, x, y)
    !---------------------------------------------------------------------
    ! --- Linear interpolation routine. Accepts only 3D arrays for x and y
    ! --- Based on Han Uitenbroeks "Linear" C routine. 
    ! --- Generates interpolates y at values x using the 
    ! --- (xtable,ytable) pair to interpolate. 
    ! --- Feb 28 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    ! 18.10.2004 (SWB) fix for array bounds (start with 1 instead of 0)
    !---------------------------------------------------------------------
    use hion_def_module

    implicit none

    integer,intent(in)                            :: Ntable, n1,n2,n3
    real(kind=dbl),dimension(1:Ntable),intent(in) :: xtable, ytable
    real(kind=dbl),dimension(:,:,:),intent(in)    :: x
    real(kind=dbl),dimension(:,:,:),intent(inout) :: y

    logical :: ascend
    integer :: k,l,m,j
    real(kind=dbl) :: xmin, xmax, fx
    !
    !---------------------------------------------------------------------

    if (xtable(2).ge.xtable(1)) then ! --- altered by SWB
       ascend=.true.
    else
       ascend=.false.
    end if

    if (ascend) then 
       xmin=xtable(1)
       xmax=xtable(Ntable)
    else
       xmin=xtable(Ntable)
       xmax=xtable(1)
    endif

    !$OMP PARALLEL DEFAULT(shared), &
    !$OMP          PRIVATE(m,l,k,j,fx)
    !$OMP DO
    do m=1,n3  
       do l=1,n2 
          do k=1,n1 

             if (x(k,l,m).le.xmin) then
                if (ascend) then 
                   y(k,l,m) = ytable(1)
                else
                   y(k,l,m) = ytable(Ntable)
                endif
             else if (x(k,l,m).ge.xmax) then
                if (ascend) then 
                   y(k,l,m) = ytable(Ntable)
                else
                   y(k,l,m) = ytable(1)
                endif
             else 
                call Locate(Ntable, xtable, x(k,l,m), j)
                fx = (xtable(j+1) - x(k,l,m)) / (xtable(j+1) - xtable(j))
                y(k,l,m) = fx*ytable(j) + (1 - fx)*ytable(j+1)
             end if

          end do
       end do
    end do
    !$OMP END DO NOWAIT
    !$OMP END PARALLEL

  end subroutine Linear
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  subroutine Locate(n,xx,x,j)
    !---------------------------------------------------------------------
    ! --- Given an array xx(1:n) and given a value x, returns a value such 
    ! --- that x is between xx(j) and xx(j+1), xx must be monotonic, j=0 
    ! --- or j=n is returned if x is out of range. See Numerical Recipes.
    ! --- Mar  2 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl)
    !---------------------------------------------------------------------
    use hion_def_module
    implicit none

    integer,intent(in) :: n
    integer,intent(inout) :: j
    real(kind=dbl),dimension(:),intent(in) :: xx
    real(kind=dbl),intent(in) :: x

    integer :: jl,jm,ju

    jl=0
    ju=n+1

    do
       if (ju-jl.gt.1) then 
          jm=(ju+jl)/2
          if( (xx(n).ge.xx(1)).eqv.(x.ge.xx(jm)) ) then
             jl=jm
          else 
             ju=jm
          end if
       else 
          exit
       end if
    end do

    if (x.eq.xx(1)) then
       j=1
    else if (x.eq.xx(n)) then
       j=n-1
    else
       j=jl
    endif

  end subroutine Locate
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  function E1(x)
    !---------------------------------------------------------------------
    ! --- Gives an approximation to the first exponential
    ! --- integral. Uses polynomial approximations from Abramowitz and 
    ! --- Stegun, 1970, Handbook of Mathematical Functions, p. 231
    ! --- Absolute accuracy is 2e-7 for 0<x<1 and 2e-8 for 1<x<infinity. 
    ! --- Mar  7 2005 :
    ! ---   Created by Jorrit Leenaarts (j.leenaarts@astro.uu.nl). 
    !---------------------------------------------------------------------
    use hion_def_module
    use hion_io_module
    implicit none

    real(kind=dbl) :: E1
    real(kind=dbl),intent(in) :: x
    character(len=2),parameter :: RoutineName='E1'
    real(kind=dbl),parameter :: & 
         s0 = -0.57721566, &
         s1 =  0.99999193, &
         s2 = -0.24991055, &
         s3 =  0.05519968, &
         s4 = -0.00967004, &
         s5 =  0.00107857
    real(kind=dbl),parameter :: &
         a1 =  8.5733287401, &
         a2 = 18.0590169730, &
         a3 =  8.6347608925, &
         a4 =  0.2677737343, &
         b1 =  9.5733223454, &
         b2 = 25.6329561486, &
         b3 = 21.0996530827, &
         b4 =  3.9584969228 


    if (x.ge.1.0) then
       E1 = ( x**4 + a1*x**3 + a2*x**2 + a3*x + a4 ) / &
            ( ( x**4 + b1*x**3 + b2*x**2 + b3*x + b4 ) * x * exp(x) )
    else if ((x.ge.0.0).and.(x.le.1.0)) then
       E1= s0 + s1*x + s2*x**2 + s3*x**3 + s4*x**4 + s5*x**5 - log(x)
    else 
       call PrError(RoutineName,'argument must be >= 0',2)
    end if

  end function E1
  !---------------------------------------------------------------------

  !---------------------------------------------------------------------
  function imaxloc(x)
    !---------------------------------------------------------------------
    ! --- returns index of 1D with the highest value in the array.
    ! --- Use instead of the internal maxloc() function, which gives
    ! --- an rank 1 array as return value. See Numerical Recipes
    !---------------------------------------------------------------------
    use hion_def_module
    implicit none

    real(kind=dbl),dimension(:),intent(in) :: x
    integer,dimension(1) :: imax
    integer :: imaxloc

    imax=maxloc(x)
    imaxloc=imax(1)

  end function imaxloc
  !---------------------------------------------------------------------

end module hion_util_module
