!***************************************************************************************************
!
!   ######  #     # ######           #####
!   #     # #     # #     #         #     #   ####   #####   ######
!   #     # #     # #     #         #        #    #  #    #  #
!   ######  ####### #     #  #####  #        #    #  #    #  #####
!   #   #   #     # #     #         #        #    #  #    #  #
!   #    #  #     # #     #         #     #  #    #  #    #  #
!   #     # #     # ######           #####    ####   #####   ######
!
!   Radiation-(Magneto-)Hydrodynamics Code "CO5BOLD"
!
!***************************************************************************************************
!   Fortran90
!   2019-03-01
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Matthias Steffen:      Potsdam
!   Hans-Guenter Ludwig:   Lund, Meudon
!   Sven Wedemeyer-Boehm:  Freiburg, Oslo
!   Inga Kamp:             Leiden, Baltimore
!   Susanne Hoefner:       Uppsala
!   Werner Schaffenberger: Potsdam, Freiburg, East Lansing, Exeter
!   France Allard:         Lyon
!***************************************************************************************************
! PROGRAM:
!   rhd:                   Radiation-(Magneto-)Hydrodynamics Code
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Number of magnetic field arrays --- */
# ifndef rhd_box_bmag01
#   define rhd_box_bmag01 0
# endif
!#
!# /* --- MPI flag activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!-------***-----------------------------------------------------------------------------------------
program rhd
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd ('radiation_hydrodynamics_code')
!
! PURPOSE:
!   Solve the coupled equations of 3D compressible (magneto-)hydrodynamics and
!   non-local radiation transport in a gravitationally stratified medium
!   in the presence of optional source terms.
!
! CATEGORY:
!   Radiation (Magneto-)Hydrodynamics
!
! VARIABLES:
!   outstr:         ('output_string') character, string with exit status information
!   ierr:           ('integer_error') integer, contains exit error status
!                     0: correct operation
!                    >0: occurrence of an error
!   filename:       ('file_name') character, complete filename (including path)
!   fileopen:       ('file_open') character, control opening of output files: 'open', 'append'
!   date_start:     ('date_start') character, date string describing begin of computation
!   date_end:       ('date_end') character, date string describing end of computation
!   system:         ('system') character describing current operating system
!   machin:         ('machine') character with name of current machine
!   hrdwar:         ('hardware') character describing current hardware
!   radraystar:     ('radiation_ray_star') character to describe the list of the ray directions
!   ncin:           ('number_channel_input') integer, channel number for start model
!   ncend:          ('number_channel_end') integer, channel number for end model
!   ncfull:         ('number_channel_full') integer, channel number for full output
!   ncmean:         ('number_channel_mean') integer, channel number for output of mean data
!   ncmcyl:         ('number_channel_mean_cylindrical') integer, channel number for output of
!                   cylindrical-mean data
!   ncchu1,2,3      ('number_channel_chunk_1,2,3') integer, channel number for output of chunk i
!   ncopa:          ('number_channel_opacity') integer, channel number for opacity table
!   nband:          ('number_band') integer, number of opacity bands
!   m1,2,3:         ('m_1,2,3') integer, first index value
!   n1,2,3:         ('n_1,2,3') integer, last index value
!   m1,2,3c:        ('m_1,2,3_chunk') integer, first index value of chunk
!   n1,2,3c:        ('n_1,2,3_chunk') integer, last index value of chunk
!   itime_start:    ('integer_time_start') integer, time step number of start model
!   trem:           ('time_remain') real, remaining CPU seconds for current job
!   tela:           ('time_elapse') real, elapsed CPU seconds for current job
!   dtime:          ('delta_time') real, size of time step
!   dtime_old:      ('delta_time_old') real, size of previous time step
!   dtime_new:      ('delta_time_new') real, size of next time step
!   dtime_limit:    ('delta_time_limit') real, time step limit from individual sub step
!   dtime_limit_hd: ('delta_time_limit_hydrodynamics') real, HD time step limit
!   dtime_limit_rad: ('delta_time_limit_radiation') real, RAD time step limit
!   dtime_limit_vis: ('delta_time_limit_viscosity') real, viscous time step limit
!   time_out_full_last: ('time_out_full_last') real, time of last full output
!   time_out_mean_last: ('time_out_mean_last') real, time of last output of averaged data
!   time_out_end_last: ('time_out_end_last') real, time of last full output into end file
!   time_start:     ('time_start') real, time of start model
!   new_timestep_flag: ('new_time_step') logical, true if advance of model has to be recomputed
!                      with reduced timestep
!   out_mean_flag:  ('output_mean_flag') logical, true if *.mean file is to be produced
!   exist_flag:     ('exist_flag') logical, locally used flag that says whether files exists
!   nhis_in:        ('number_history_input') integer, number of entries in input history array
!   nhis_full:      ('number_history_full') integer, number of entries in full output history
!   nhis_end:       ('number_history_end') integer, number of entries in history in end model
!   history_in:     ('history_in') character array with input history entries
!   history_full:   ('history_full') character array with full output history entries
!   history_end:    ('history_end') character array with history entries for end model
!   model:          ('model') type(box_type), structure with all data of current (some substeps
!                   advanced) model
!   model0:         ('model_0') type(box_type), structure with all data of model before each
!                   first subtime step
!   slice:          ('slice') type(box_type), slice (or subblock) of current model
!   dat:            ('data') type(dat_type), structure for additional output data
!   action:         ('action') type(action_type), structure with commands and parameters
!
! ROUTINES:
!   dfopta:                Read opacity table data and store it in common blocks
!   dfoptad:               Read dust opacity table data and store it in common blocks
!   eosinter_SwitchInfo:   Print information about used compiler switches
!   flush:                 Flush output buffer
!   mscrad_switchinfo:     Print information about used compiler switches
!   lcfrad_switchinfo:     Print information about used compiler switches
!   omp_get_schedule:      OpenMP system routine to get scheduling settings
!   opta_Modify:           Manipulate opacity data
!   opta_SwitchInfo:       Print information about used compiler switches
!   random_seed:           Set the random-generator seed
!   rhd_bound_3DCenter:    Modify energy in central cells to keep entropy constant
!   rhd_bound_3DHeatBottom: Modify energy in bottom cells to keep entropy constant
!   rhd_bound_SwitchInfo:  Print information about used compiler switches
!   rhd_box_AllCopyOMP:    Copy entire box, in parallel if box is 3D
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_Read:          Read box from file in UIO form
!   rhd_box_SwitchInfo:    Print information about used compiler switches
!   rhd_box_Write:         Write box with additional infos into file in UIO form
!   rhd_datc_Write:        Write additional RHD data into file in UIO form
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_dat_SwitchInfo:    Print information about used compiler switches
!   rhd_dat_Write:         Write additional RHD data into file in UIO form
!   rhd_dust_PropInit:     Initialize property structure and set e.g. number of dust components
!   rhd_dust_SourceStep:   Update model by applying source term step
!   rhd_hyd_Step:          Perform entire 3D hydro step, possibly composed of multiple time-steps
!   rhd_hyd_SwitchInfo:    Print information about used compiler switches
!   rhd_mscrad_cls:        Deallocate arrays used by MSCrad by calling radcls
!   rhd_mscrad_init:       Initialize MSCrad radiative transfer equations by calling radini
!   rhd_mscrad_step:       Perform 3d radiative energy transfer step
!   rhd_mhd_Step:          Perform entire 2D or 3D magneto-hydrodynamics step, employ HLLE scheme
!   rhd_mhd_SwitchInfo:    Print information about used compiler switches
!   rhd_par_Read:          Read parameters from parameter file and put them into global variable
!   rhd_par_Toaction:      Prepare action variable from input parameters
!   rhd_par_ToSubpar:      Set values in parameter sub structure based on input parameters
!   rhd_prop_Init:         Initialize property structure
!   rhd_prop_Print:        Print information about property structure
!   rhd_rad3d_Step:        Perform 3d radiative energy transfer step
!   rhd_rad3d_SwitchInfo:  Print information about used compiler switches
!   rhd_lcfrad_cls:        Deallocate arrays used by LCFrad by calling radcls
!   rhd_lcfrad_init:       Initialize LCFrad radiative transfer equations by calling radini
!   rhd_lcfrad_step:       Perform 3d radiative energy transfer step
!   rhd_shortrad_Step:     Perform 3d radiative energy transfer step using short characteristics
!   rhd_shortrad_SwitchInfo: Print information about used compiler switches
!   rhd_tweak_Final:       Finalization of module
!   rhd_tweak_Init:        Initialization of module
!   rhd_tweak_Step:        Tweak model for one time step
!   rhd_tweak_SwitchInfo:  Print information about used compiler switches
!   rhd_vis_Step:          Complete viscosity step
!   rhd_vis_SwitchInfo:    Print information about used compiler switches
!   str_pathandfile:       Merge path name and file name
!   tabinter_delete:       Deallocate EOS table completely
!   tabinter_rdcoeff:      Read coefficients for bicubic interpolation of EOS quantities from file
!   tabinter_reset:        Set all quantities in EOS table to zero
!   timing_Init:           Initialize arrays for timing routines
!   timing_Job:            Return the remaining CPU or wall-clock time
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_SwitchInfo:     Print information about used compiler switches.
!   timing_Write:          Write results of counting process into table
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dattim:            Get string with current date and time
!   uio_getenv:            Get information about environment
!   uio_init:              Initialization of UIO routines
!   uio_SwitchInfo:        Print information about used compiler switches
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   mscrad_module:         MSC Radiation Transport Routines
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   lcfrad_module          Long-characteristics radiative transfer routines (local models)
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box-handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_dust_module:       Routines for handling of source terms due to dust or molecules
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_io_module:         Input/output routines
!   rhd_hyd_module:        Hydrodynamics routines
!   rhd_lhdrad_module:     Long-characteristics radiative transfer routines (global models)
!   rhd_mscrad_module:     Radiative transfer routines with multi-short-characteristics scheme
!   rhd_mhd_module:        Magneto-hydrodynamics routines
!   rhd_MPI_module:        MPI routines specific for RHD code
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_lcfrad_module:     Long-characteristics radiative transfer routines (local models)
!   rhd_shortrad_module:   Short-characteristics radiative transfer routines (global models)
!   rhd_sub_module:        Additional subroutines for RHD
!   rhd_tweak_module:      Universal module to do any tweaking
!   rhd_vis_module:        Artificial-tensor-viscosity routines
!   rhd_wrk_module:        Provide persistent work copies of model data
!   str_module:            String handling routines
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!   uio_bulk_module:       Main set of UIO routines
!
! PROCEDURE:
!   Basic initialization
!   Read parameter file (with control parameters and file names of EOS and opacity tables and
!                        of start model)
!   Read EOS table
!   Read opacity table
!   Read start model
!   Open output files
!   Start loop over all time steps until exit condition is reached
!     Start loop over time steps of decreasing size until time step is small enough
!                        to fulfill all stability conditions
!       (Magneto-)Hydrodynamics step
!       Tensor viscosity step (2D or 3D version, optional)
!       Source terms, e.g., for dust or molecules, optional
!       Radiation transport step (LHDrad, MSrad, or SHORTrad)
!     End loop => this time step is done
!     Output of data (if requested for this time step)
!   End loop => this time sequence is done
!   Output of final model
!   Close output files
!
! MODIFICATION HISTORY:
!   1997-05-20 (B.F. Kiel) Writing started
!   1997-06-09 (B.F.) Read parameters from file
!   1997-10-16 (B.F.) 1D -> 3D
!   1999-04    (B.F. Copenhagen) Some extensions
!   1999-04-15 (B.F.) First glimpse of radiative transfer
!   1999-04-19 (B.F.) Load opacity
!   1999-05-03 (B.F.) Radiation transport with lhdrad
!   1999-05-12 (B.F.) First 2D convection
!   1999-05-25 (B.F.) First HD substep: loop over i2 instead of i3
!   1999-05-27 (B.F.) Tensor viscosity
!   1999-05-31 (B.F.) Check viscous time step
!   1999-08-20 (B.F.) Additional output ('*.mean')
!   1999-12-08 (B.F.) 3D radiation transport
!   1999-12-28 (B.F.) Some rearrangements because of 3D radiation transport
!   2000-01-19 (B.F.) Timing routines
!   2000-03-23 (B.F.) Check for existence of stop file
!   2000-04-08 (B.F.) Check for existence of continuation file
!   2000-05-26 (B.F.) Modify opacity table by call of opta_modify
!   2000-12-10 (B.F.) Loop over i_chunk in hydro part to improve cache efficiency
!   2000-12-28 (B.F.) Preprocessor directives added (rhd.f90 -> rhd.F90)
!   2001-01-23 (B.F.) Loop over i_chunk in viscosity part to improve cache efficiency
!   2001-03-14 (B.F.) OpenMP parallelization directives
!   2002-05-27 (B.F.) dtime_limit_vis/hd -> C_vis/hdCourant*dtime_limit_vis/hd: apply Courant
!                     factors already in subroutines
!   2002-10-17 (B.F.) Include file 'compiler_flags.info'
!   2002-11-05 (B.F.) Additional control parameters in some subroutines to account for additional
!                     density arrays and magnetic fields
!   2002-11-13 (B.F.) Three copy_box calls parallelized
!   2004-02-12 (B.F.) Some flush commands (re)moved
!   2004-03-25 (B.F.) Call of rhd_mhd_Roe1Dstep if compiler macro is set
!   2004-05-21 (B.F.) Tweak module
!   2005-06-03 (SWB ) Merged versions MHD/HD
!   2006-12-07 (B.F.) Chunk handling put into subroutines
!   2006-12-12 (B.F.) Call multistep version of hydro routines
!   2006-12-14 (B.F.) Box copying code put into rhd_box_copyall_OMP
!   2007-10-22 (B.F. Lyon) model%time_db introduced
!   2009-01-03 (M.S.) Call of rhd_mhd_Step adjusted
!   2009-07-17 (B.F.) Read dust opacity table
!   2009-09-29 (B.F.) Correct handling of mean output step counter
!   2010-03-31 (B.F.) Call timing_SwitchInfo
!   2010-05-06 (B.F.) Call uio_SwitchInfo
!   2012-07-18 (B.F.) Seed for MPI version
!   2012-07-20 (B.F.) Call routine in rhd_MPI_module;
!                     small changes to keep standard version consistent with MPI branch
!   2012-10-11 (B.F.) Outer time and dtime loops in MPI version
!   2012-10-23 (B.F.) Call rhd_bound_3DCenter_MPI
!   2012-11-07 (B.F.) Some modifications for MPI; call rhd_bound_3DHeatBottom_MPI
!   2012-11-16 (B.F.) Call rhd_hyd_Step in MPI version
!   2012-11-20 (B.F.) Call rhd_vis_Step in MPI version
!   2013-08-07 (B.F.) Additional output into chu2 and chu3 files
!   2014-03-01 (B.F. Uppsala) Call rhd_bound_3DCenter
!   2015-04-28 (B.F. Uppsala) Bugfix in index computation of chu1,2,3 files
!   2016-02-22 (B.F.) Copy update comment from rhd.F90 into MPI version
!   2016-04-15 (B.F.) Reorganize global world-geometry variables for MPI
!   2017-07-15 (B.F.) Initialize UIO earlier; allow console output to file
!   2017-07-28 (B.F.) Re-organize chunk output
!   2017-07-30 (B.F.) Add comments; do small modifications to harmonize MPI and non-MPI version
!   2017-07-31 (B.F.) Write full datasets, collecting data from all MPI processes
!   2017-08-02 (B.F.) Write chunks, collecting data from all MPI processes
!   2017-08-03 (B.F.) Print OpenMP info in separate routine
!   2017-09-12 (B.F.) Write mean data also in MPI version
!   2017-09-13 (B.F.) Bring MPI version and non-MPI version more in line
!   2017-11-05 (B.F.) Optionally call MCrad routines
!   2017-12-01 (M.S.) Rename MSrad and MCrad modules and routines
!   2017-12-04 (B.F.) Write error message and stop if radscheme=MSrad is requested
!---------------------------------------------------------------------------------------------------
use uio_bulk_module
use tabinter_module
use gasinter_module
use str_module
use opta_module
use timing_module
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
use rhd_bound_module
use rhd_box_module
use rhd_wrk_module
use rhd_dat_module
use rhd_io_module
use rhd_sub_module
use rhd_hyd_module
use rhd_vis_module
! --- MPI, use 'include "mpif.h"' if the MPI module is not available ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
use rhd_MPI_module
!#
# if (rhd_box_quc01 != 0)
!#
use rhd_dust_module
!#
# endif
!#      /* rhd_box_quc01 */
!#
# if (rhd_box_bmag01 == 1)
!#
use rhd_mhd_module
!#
# endif
!#      /* rhd_box_bmag01 */
!#
# ifdef rhd_r01
!#
use rhd_lhdrad_module
!#
# endif
!#      /* rhd_r01 */
!#
# ifdef rhd_r02
!#
use rhd_lcfrad_module
use lcfrad_module
use rhd_mscrad_module
use mscrad_module
!#
# endif
!#      /* rhd_r02 */
!#
# ifdef rhd_r03
!#
use rhd_shortrad_module
!#
# endif
!#      /* rhd_r03 */
!#
# ifdef rhd_tweak01
!#
use rhd_tweak_module
use rhd_scale_module
!#
# endif
!#      /* rhd_tweak01 */
!
implicit none
!
! --- Variables ---
character(len=80)                       :: outstr
integer                                 :: ierr
character(len=256)                      :: filename
character(len=80)                       :: fileopen, &
                                           date_start, date_end, system, machin, hrdwar, &
                                           radraystar, SubBoxMode
integer                                 :: ncin, ncend, ncfull, ncmean, ncmcyl, &
                                           ncchu1, ncchu2, ncchu3, &
                                           ncopa, ncstatus
integer                                 :: nband, nbanddust
integer                                 :: m1,n1, m2,n2, m3,n3, i3
integer                                 :: m1c,n1c, m2c,n2c, m3c,n3c
integer                                 :: itime_start, i, NPoint_max
real                                    :: trem, tela, &
                                           dtime, dtime_old, dtime_new, &
                                           dtime_limit, &
                                           dtime_limit_hd, &
                                           dtime_limit_rad, dtime_limit_vis, &
                                           time_out_full_last, &
                                           time_out_mean_last, time_out_mean_lastold, &
                                           time_out_end_last, &
                                           time_start, tmp
logical                                 :: new_timestep_flag, out_mean_flag, &
                                           exist_flag, write_flag
integer                                 :: nhis_in, nhis_full, nhis_end
character(len=80), dimension(nhismax)   :: history_in, history_full, history_end
character(len=84)                       :: line1, line2, line3
character(len=60)                       :: line4
character(len=3)                        :: prefix
type(box_type)                          :: model, model0, slice
type(dat_type)                          :: dat, datLrg, datRadLrg
type(action_type)                       :: action
! --- MPI variables ---
integer                                 :: MyProcWo, ierrMPI, ProcErr, idProc, NProcsWO
integer                                 :: reqs(100), stats(MPI_STATUS_SIZE,100)
real                                    :: buf(5358150)
!#
# if (rhd_mpi_l01 != 1)
!#                      /* MPI not activated ------------- */
integer                                 :: MPI_COMM_WORLD
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
! --- Determine/set some global MPI parameters ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
call MPI_Init(ierrMPI)
call MPI_Comm_Rank(MPI_COMM_WORLD, MyProcWo, ierrMPI)
call MPI_Comm_Size(MPI_COMM_WORLD, NProcsWO, ierrMPI)!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
MyProcWo      =0
MPI_COMM_WORLD=0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
! --- Initialize UIO routines ---
call uio_init(progrm='RHD')
!
! --- Initial call of time routines, get system information ---
call uio_dattim(date_start)
call uio_getenv(MyProc=MyProcWo, &
                system=system, machin=machin, hrdwar=hrdwar)
!
! --- Set important global parameters ---
gl%lw_p        =84                          ! --- Line width for text output       ---
gl%parFile     ='rhd.par'                   ! --- Name of parameter input file     ---
gl%par2File    ='rhd.par2'                  ! --- Name of 2nd parameter input file ---
gl%StopFile    ='rhd.stop'                  ! --- Name of STOP request file        ---
gl%ContFile    ='rhd.cont'                  ! --- Name of CONTINUE request file    ---
gl%StatusFile  ='rhd.done'                  ! --- Name of status output file       ---
gl%DumpFile    ='rhd.dump'                  ! --- Name of dump request file        ---
gl%SnapshotFile='rhd.snap'                  ! --- Name of snapshot output file     ---
gl%OutFile     =''                          ! --- Name of text log output file     ---
gl%version     ='RHD 003.03.2019.03.19'     ! --- RHD code version                 ---
if ((gl%OutFile == '') .or. (MyProcWo > 0)) then
  ! --- Set channel number explicitly ---
  gl%nc_p      =6                           ! --- Channel nc_p for text output     ---
else ! (gl%OutFile /= '')
  ! --- Get free channel via UIO routine and open file ---
  call uio_chunit('get', nc1out=gl%nc_p)
  open(gl%nc_p, file=gl%OutFile, form='formatted', status='unknown', iostat=ierr)
endif ! (gl%OutFile)
if (MyProcWo == 0) then
  gl%nc_m      =gl%nc_p                     ! --- Channel nc_m for text output     ---
else
  gl%nc_m      =-1                          ! --- Channel nc_m for text output     ---
endif
!
! --- Define look of separator lines ---
line1=repeat('*', gl%lw_p)
line2=repeat('=', gl%lw_p)
line3=repeat('-', gl%lw_p)
line4=repeat('.', 60)
!
if (MyProcWo == 0) then
  write(gl%nc_p,'(A)')
  write(gl%nc_p,'(A)') '                 ####                                '
  write(gl%nc_p,'(A)') '  #####   #####  #    ######   #####  #       ###### '
  write(gl%nc_p,'(A)') ' #     # #     # ###  #     # #     # #       #     #'
  write(gl%nc_p,'(A)') ' #       #     #    # #     # #     # #       #     #'
  write(gl%nc_p,'(A)') ' #       #     # ###  ######  #     # #       #     #'
  write(gl%nc_p,'(A)') ' #       #     #      #     # #     # #       #     #'
  write(gl%nc_p,'(A)') ' #     # #     #      #     # #     # #       #     #'
  write(gl%nc_p,'(A)') '  #####   #####       ######   #####  ####### ###### '
  write(gl%nc_p,'(A)')
  write(gl%nc_p,'(A)') ' ' // trim(gl%version)
  write(gl%nc_p,'(A)') ' ' // repeat('=', len_trim(gl%version))
  write(gl%nc_p,'(A)')
  !
  write(gl%nc_p,'(A)') &
    '================================= Initialization ==================================='
  write(gl%nc_p,*)
endif ! (MyProcWo == 0)
!
! --- Set seed for random number generator ---
call random_seed()
!
! --- Initial call of time routines ---
call timing_Job()
!
! --- Starting timing measurement ---
call timing_Init()
call timing_start('RHD code')
!!$call timing_SelfTest()
!
if (MyProcWo == 0) then
  ! --- Print information about compiler switches ---
  !
  write(gl%nc_p,'(A)') line3
  write(gl%nc_p,'(A)') 'Compiler call: ............................................'
  include 'compiler_flags.info'
  call timing_SwitchInfo(gl%nc_p)
!#
# ifdef uio_switch_native_l01
!#
  call uio_SwitchInfo(gl%nc_p)
!#
# endif
!#
  call rhd_box_SwitchInfo(gl%nc_p)
  call rhd_bound_SwitchInfo(gl%nc_p)
  call rhd_dat_SwitchInfo(gl%nc_p)
  call eosinter_SwitchInfo(gl%nc_p)
  call opta_SwitchInfo(gl%nc_p)
  call rhd_hyd_SwitchInfo(gl%nc_p)
!#
# if (rhd_box_bmag01 == 1)
!#
  call rhd_mhd_SwitchInfo(gl%nc_p)
!#
# endif
!#
  call rhd_vis_SwitchInfo(gl%nc_p)
!#
# ifdef rhd_r01
!#
  call rhd_rad3d_SwitchInfo(gl%nc_p)
!#
# endif
!#
# ifdef rhd_r02
!#
  call lcfrad_switchinfo(gl%nc_p)
  call mscrad_switchinfo(gl%nc_p)
!#
# endif
!#
# ifdef rhd_r03
!#
  call rhd_shortrad_SwitchInfo(gl%nc_p)
!#
# endif
!#
# ifdef rhd_tweak01
!#
  call rhd_tweak_SwitchInfo(gl%nc_p)
  call rhd_scale_SwitchInfo(gl%nc_p)
!#
# endif
!#      /* rhd_tweak01 */
  !
  ! --- Print information about MPI environment ---
  write(gl%nc_p,*)
  write(gl%nc_p,'(A)') line3
  write(gl%nc_p,'(A)') 'MPI: ......................................................'
  call rhd_MPI_PrintInfo(gl%nc_m, MyProcWo)
  call flush(gl%nc_p)
  !
endif ! (MyProcWo == 0)
!
! --- Initialize global name arrays ---
call rhd_action_InitNameAr(-1)
!
! --- Print information about OpenMP environment ---
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
if (MyProcWo == 0) then
  write(gl%nc_p,'(A)') 'OpenMP: ...................................................'
endif
call rhd_OMP_PrintInfo(gl%nc_p)
call flush(gl%nc_p)
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
if (MyProcWo == 0) write(gl%nc_p,*)
!
! --- Read parameter file(s) ---
if (MyProcWo == 0) then
  write(gl%nc_p,'(A)') line3
  write(gl%nc_p,'(A)') 'ACTION: Read parameter file <<<<<<<<'
endif
call rhd_par_Read(gl%parFile, gl%nc_m, par)
par%A%IsRead_flag=.true.
!!$ if (MyProcWo == 0) call rhd_par_PrintInfo(par)
inquire(file=gl%par2File, exist=exist_flag)
if (exist_flag) then
  ! --- Read parameter file -- again, but into other structure ---
  if (MyProcWo == 0) then
    write(gl%nc_p,'(A)') line3
    write(gl%nc_p,'(A)') 'ACTION: Read parameter file into par1 <<<<<<<<'
  endif
  call rhd_par_Read(gl%parFile, gl%nc_m, par1)
  par1%A%IsRead_flag=.true.
  ! --- Read 2nd parameter file ---
  if (MyProcWo == 0) then
    write(gl%nc_p,'(A)') line3
    write(gl%nc_p,'(A)') 'ACTION: Read 2nd parameter file into par2 <<<<<<<<'
  endif
  call rhd_par_Read(gl%par2File, gl%nc_m, par2)
  par2%A%IsRead_flag=.true.
else ! (.not. exist_flag)
  par1%A%IsRead_flag=.false.
  par2%A%IsRead_flag=.false.
endif ! (exist_flag)
!
! --- Fill some entries in par%A and prepare action variable ---
call rhd_par_ToSubpar(par)
call rhd_par_Toaction(par, action)
!
! \label{f90:cputime__use1}
! --- Call timing routine to set maximum CPU time ---
call timing_Job(CPUtime=par%CPUtime)
!
! --- Initialize box property data ---
call rhd_prop_Init(par%hdScheme)
call rhd_wg_Init()
!#
# if (rhd_box_quc01 != 0)
!#
! \label{f90:dustscheme__use1}
call rhd_dust_PropInit(par%dustScheme)
!#
# endif
!#      /* rhd_box_quc01 */
!
! \label{f90:eosfile__use1}\label{f90:eospath__use1}
! --- Read EOS table into table 1 ---
if (MyProcWo == 0) then
  write(gl%nc_p,*)
  write(gl%nc_p,'(A)') line3
  write(gl%nc_p,'(A)') 'ACTION: Load EOS data <<<<<<<<'
endif
call tabinter_reset(1, gl%nc_m)
call str_pathandfile(par%EOSPath, par%EOSFile, filename)
call tabinter_rdcoeff(filename, gl%nc_m, outstr,ierr)
call rhd_MPI_ErrorDist(MPI_COMM_WORLD, MyProcWo, 0, ierr, outstr, ProcErr)
if (ierr /= 0) then
  if (MyProcWo == 0) then
    write(gl%nc_p,'(A,I4,A,I6)') &
        'Error during reading of EOS table: ' // trim(outstr) // &
        ' no. ', ierr, &
        ' on process ', ProcErr
  endif
  goto 900
endif
if (MyProcWo == 0) then
  write(gl%nc_p,*)
endif
!
! \label{f90:opafile__use1}\label{f90:opapath__use1}
! --- Read opacity table ---
if (par%OpaFile /= '') then
  call uio_chunit('get', nc1out=ncopa)
  call str_pathandfile(par%OpaPath, par%OpaFile, filename)
  if (MyProcWo == 0) then
    write(gl%nc_p,'(A)') line3
    write(gl%nc_p,'(A)') 'ACTION: Load opacity tables <<<<<<<<'
    write(gl%nc_p,'(A)') 'Opacity: loading ' // trim(filename)
  endif
  open(ncopa, file=filename, form='formatted', status='old', iostat=ierr)
  outstr=''
  call rhd_MPI_ErrorDist(MPI_COMM_WORLD, MyProcWo, 0, ierr, outstr, ProcErr)
  if (ierr /= 0) then
    if (MyProcWo == 0) then
      write(gl%nc_p,'(A,I4,A,I6)') 'Error during reading of opacity table, no. ', ierr, &
        ' on process ', ProcErr
    endif
    goto 900
  endif
  call dfopta(merge(2, 4, (MyProcWo == 0)), gl%nc_p, ncopa, nband)
  close(ncopa)
  call uio_chunit('return', nc1in=ncopa)
  !
  ! --- Manipulate opacity table ---
  call opta_Modify(kappaMin=par%kappaMin, kappaMax=par%kappaMax)
  !
  nband=min(par%N_radband, 2)
  !
  if (MyProcWo == 0) then
    write(gl%nc_p,*)
  endif
else ! (par%OpaFile == '')
  nband=1
endif ! (par%OpaFile)
!
! \label{f90:dustopafile__use1}\label{f90:opapath__use2}
! --- Read dust opacity table ---
if (par%dustOpaFile /= '') then
  call uio_chunit('get', nc1out=ncopa)
  call str_pathandfile(par%OpaPath, par%dustOpaFile, filename)
  if (MyProcWo == 0) then
    write(gl%nc_p,'(A)') line3
    write(gl%nc_p,'(A)') 'ACTION: Load dust opacity tables <<<<<<<<'
    write(gl%nc_p,'(A)') 'Dust Opacity: loading ' // trim(filename)
  endif
  open(ncopa, file=filename, form='formatted', status='old', iostat=ierr)
  outstr=''
  call rhd_MPI_ErrorDist(MPI_COMM_WORLD, MyProcWo, 0, ierr, outstr, ProcErr)
  if (ierr /= 0) then
    if (MyProcWo == 0) then
      write(gl%nc_p,'(A,I4,A,I6)') 'Error during reading of dust opacity table, no. ', ierr, &
        ' on process ', ProcErr
    endif
    goto 900
  endif
  call dfoptad(merge(2, 4, (MyProcWo == 0)), gl%nc_p, ncopa, nbanddust)
  close(ncopa)
  call uio_chunit('return', nc1in=ncopa)
  !
  if (MyProcWo == 0) then
    write(gl%nc_p,*)
  endif
endif ! (par%dustOpaFile /= '')
!
! --- Initialize global box structures ---
call rhd_wrk_boxGloInit()
!
call timing_start('IO: start model')
!
! \label{f90:infile_start__use1}
! --- Read start model ---
if (MyProcWo == 0) then
  write(gl%nc_p,'(A)') line3
  write(gl%nc_p,'(A)') 'ACTION: Read start model <<<<<<<<'
  inquire(file=gl%ContFile, exist=exist_flag)
  fileopen='open'
  filename=par%infile_start
  ! \label{f90:outfile_end__use1}
  if (exist_flag) then
    inquire(file=par%OutFile_end, exist=exist_flag)
    if (exist_flag) then
      ! --- Take existing final model as start model ---
      fileopen='append'
      filename=par%OutFile_end
    endif ! (exist_flag)
  endif ! (exist_flag)
endif ! (MyProcWo == 0)
!#
# if (rhd_mpi_l01 == 11)
!#                      /* MPI activated ----------------- */
! --- There is more than one MPI process: read the start model and distribute it      ---
! --- prop, wg: global-box boundary conditions; action: local-box boundary conditions ---
call rhd_MPI_boxRead(ncin, MyProcWo, action, &
                     box=model, &
                     Bb_flag= ((par%A%hdScheme == hdScheme_RoeMHD) .or. &
                               (par%A%hdScheme == hdScheme_HLLMHD) .or. &
                               (par%A%hdScheme == hdScheme_CenMHD)       ), &
                     dtime=dtime, time_out_full_last=time_out_full_last, &
                     time_out_mean_last=time_out_mean_last, &
                     file=filename, imodel=par%istep_in_start, nhis=nhis_in, history=history_in)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
! --- Just read the start model ---\label{f90:istep_in_start__use1}
if (MyProcWo == 0) then
   call rhd_box_Read('open,model,close', ncin, &
                  box=model, &
                  Bb_flag= ((par%A%hdScheme == hdScheme_RoeMHD) .or. &
                            (par%A%hdScheme == hdScheme_HLLMHD) .or. &
                            (par%A%hdScheme == hdScheme_CenMHD)       ), &
                  dtime=dtime, time_out_full_last=time_out_full_last, &
                  time_out_mean_last=time_out_mean_last, &
                  file=filename, imodel=par%istep_in_start, nhis=nhis_in, history=history_in)
endif ! (MyProcWo == 0)
prop%MyProcWo=0
!
! --- Modify action structure: a direction with only one element has periodic boundaries ---
if (model%n1 == model%m1) then
  action=rhd_action_New(action, bound_l1=bound_periodic, bound_r1=bound_periodic)
endif
if (model%n2 == model%m2) then
  action=rhd_action_New(action, bound_l2=bound_periodic, bound_r2=bound_periodic)
endif
if (model%n3 == model%m3) then
  action=rhd_action_New(action, bound_l3=bound_periodic, bound_r3=bound_periodic)
endif


! --- Copy global boundary information from action to prop structure --- \label{f90:bound__proc1}
prop%bound_l1=action%bound_l1
prop%bound_r1=action%bound_r1
prop%bound_l2=action%bound_l2
prop%bound_r2=action%bound_r2
prop%bound_l3=action%bound_l3
prop%bound_r3=action%bound_r3
prop%bound_mag_l1=action%bound_mag_l1
prop%bound_mag_r1=action%bound_mag_r1
prop%bound_mag_l2=action%bound_mag_l2
prop%bound_mag_r2=action%bound_mag_r2
prop%bound_mag_l3=action%bound_mag_l3
prop%bound_mag_r3=action%bound_mag_r3
! --- Copy global information from prop to wg%hyd structure ---
wg%hyd%m(1)=prop%m1
wg%hyd%n(1)=prop%n1
wg%hyd%m(2)=prop%m2
wg%hyd%n(2)=prop%n2
wg%hyd%m(3)=prop%m3
wg%hyd%n(3)=prop%n3
wg%hyd%xb_l(1)=prop%xb_l1
wg%hyd%xb_r(1)=prop%xb_r1
wg%hyd%xb_l(2)=prop%xb_l2
wg%hyd%xb_r(2)=prop%xb_r2
wg%hyd%xb_l(3)=prop%xb_l3
wg%hyd%xb_r(3)=prop%xb_r3
wg%hyd%bound_l(1)=prop%bound_l1
wg%hyd%bound_r(1)=prop%bound_r1
wg%hyd%bound_l(2)=prop%bound_l2
wg%hyd%bound_r(2)=prop%bound_r2
wg%hyd%bound_l(3)=prop%bound_l3
wg%hyd%bound_r(3)=prop%bound_r3
wg%hyd%bound_mag_l(1)=prop%bound_mag_l1
wg%hyd%bound_mag_r(1)=prop%bound_mag_r1
wg%hyd%bound_mag_l(2)=prop%bound_mag_l2
wg%hyd%bound_mag_r(2)=prop%bound_mag_r2
wg%hyd%bound_mag_l(3)=prop%bound_mag_l3
wg%hyd%bound_mag_r(3)=prop%bound_mag_r3
wg%hyd%MyProcIsInCOMM_flag=.true.
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */

! here hacked stuff by HGL

#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!#
#   endif

if (MyProcWo == 0) then
  write(gl%nc_p, *)
  write(gl%nc_p, '(A)') 'HGL: statistics of input model'
  call rhd_box_PrintStat( model, gl%nc_p, 'HGL: ')
  write(gl%nc_p, *)
endif

call timing_start('MPI transfer')

do i=1,1000
  if (MyProcWo == 0) then
     do idProc = 1,NProcsWo-1
        call MPI_ISEND(model%rho, 189*189*150, MPI_REAL, idProc, 0, MPI_COMM_WORLD, reqs(idproc), ierrMPI)
     enddo
  else
     call MPI_IRECV(buf, 189*189*150, MPI_REAL, 0, 0, MPI_COMM_WORLD, reqs(MyProcWo), ierrMPI)   
  endif

  call MPI_WAITALL(NProcsWo-1, reqs(1:NProcsWo-1), stats(:,1:NProcsWo-1), ierrMPI)
enddo

call timing_stop('MPI transfer')
!

! here end hacked stuff by HGL ...


if (MyProcWo == 0) then
  call timing_Job(tela=tela)
  write(gl%nc_p,'(A,I8,A,1PE14.8,A,1PE14.6,A)')  &
                 '=== Time step number: itime= ', model%itime, &
                 '  time= ', model%time_db, &
                 '  t_job=', tela, &
                 ' ==='
  write(gl%nc_p,'(A)') &
    '================================ End Computation ==================================='
endif
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!
if (ierr >= 100) then
  ! --- Severe error ---
  !
  ! --- Recall unchanged backup model ---
  if (rhd_box_Exist(model0)) then
    dtime=dtime_old
    call rhd_box_AllCopyOMP(model0, model)
  endif
  if (MyProcWo == 0) then
    write(gl%nc_p,*)
    write(gl%nc_p,'(A)') line1
    write(gl%nc_p,'(A)')    'SEVERE ERROR: ' // trim(outstr)
    write(gl%nc_p,'(A,I8)') 'Exit status: ', ierr
    write(gl%nc_p,'(A)')    'Interrupt computation'
    write(gl%nc_p,'(A)') line1
  endif
else ! (ierr < 100)
  ! --- Clean exit ---
  !
  if (MyProcWo == 0) then
    write(gl%nc_p,*)
    write(gl%nc_p,'(A)') line3
    write(gl%nc_p,'(A)')    'Exit information: ' // trim(outstr)
    write(gl%nc_p,'(A,I8)') 'Exit status: ', ierr
    write(gl%nc_p,'(A)') line3
    !
    call uio_chunit('get', nc1out=ncstatus)
    open(ncstatus, file=gl%StatusFile, form='formatted', status='unknown')
    call uio_dattim(date_end)
    write(ncstatus, '(A)') date_end
    close(ncstatus)
    call uio_chunit('return', nc1in=ncstatus)
  endif
endif ! (ierr)
if (MyProcWo == 0) write(gl%nc_p,*)
!
call timing_start('IO: end model')
call timing_start('uio output routines')
call timing_stop('uio output routines')
!
!#
# ifdef rhd_tweak01
!#
call rhd_tweak_Final(gl%nc_m, model, action)
call rhd_scale_Final(gl%nc_m, model, action)
!#
# endif
!#      /* rhd_tweak01 */
!#
# ifdef rhd_r02
!#
! --- Finish timing measurement ---
call timing_stop('RHD code')
if (MyProcWo == 0) write(gl%nc_p,'(A)') line2
call timing_SelfTest()
call timing_Write(gl%nc_p, nThread=1, nProcess=8)
!
# endif
!#      /* rhd_r02 */

900 continue
!
if ((gl%OutFile /= '') .and. (MyProcWo == 0)) then
  ! --- Close file and return channel number via UIO routine ---
  close(gl%nc_p)
  call uio_chunit('return', nc1in=gl%nc_p)
endif ! (gl%OutFile)
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
call MPI_Finalize(ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end program rhd
