!***************************************************************************************************
!
!                                                                               #     # #######
!   #####  #    # #####        #####    ##    ####  ###### #####    ##   #####  #     #    #
!   #    # #    # #    #       #    #  #  #  #      #      #    #  #  #  #    # #     #    #
!   #    # ###### #    #       #####  #    #  ####  #####  #    # #    # #    # #     #    #
!   #####  #    # #    #       #    # ######      # #      #####  ###### #    # #     #    #
!   #   #  #    # #    #       #    # #    # #    # #      #   #  #    # #    # #     #    #
!   #    # #    # #####  ##### #####  #    #  ####  ###### #    # #    # #####   #####     # _module
!
!   Radiation-Hydrodynamics-Code: unit-test routines for base radiation-transport module
!
!***************************************************************************************************
!   Fortran90
!   2017-08-26
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_baseradUT_module:  Unit-test routines for base radiation-transport routines
!
!***************************************************************************************************


!------********************-------------------------------------------------------------------------
module rhd_baseradUT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baseradUT_module ('rhd_base_radiation_Unit_Test_module')
!
! PURPOSE:
!   Provide unit-test routines for base radiation-transport routines.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhd_baseradUT_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_baserad_RotationMatrixUT: Perform unit tests for rhd_baserad_RotationMatrix
!   rhd_baserad_RayDirectionsUT: Perform unit tests for rhd_baserad_RayDirections
!   rhd_baserad_PipeIndexUT: Perform unit tests for rhd_baserad_PipeIndex
!   rhd_baseradUT_TestAll: Perform all unit tests of basic radiation-transport routines
!   rhd_baseradUT_TestSome: Perform some unit tests of basic radiation-transport routines
!
! MODIFICATION HISTORY:
!   2015-02-28 (B.F. Uppsala) First version
!   2015-03-06 (B.F.) Put into own file
!   2017-08-26 (B.F. Lyon) Add routine for partial unit tests
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------****************************-------------------------------------------------------------
subroutine rhd_baserad_RotationMatrixUT(TestPrefix, AngleType)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_RotationMatrixUT ('radiation_hydro_base_radiation_Rotation_Matrix_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_baserad_RotationMatrix.
!
! CATEGORY:
!   Matrix algebra, testing
!
! CALLING SEQUENCE:
!   call rhd_baserad_RotationMatrixUT(TestPrefix, AngleType)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   AngleType:    ('Angle_Type') character
!                   'TaitBryanXYZ': Tait-Bryan X-Y-Z angles
!                   'EulerZXZ':     Euler Z-X-Z angles (default)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_baserad_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_baserad_module:    Basic routines for radiation transport
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Construct a number of rotation matrices and perform basic consistency checks.
!
! EXAMPLE:
!   call rhd_baserad_RotationMatrixUT('baserad', 'EulerZXZ')
!
! MODIFICATION HISTORY:
!   2015-02-28 (B.F. Uppsala) First version
!   2016-05-16 (B.F.) Optionally, print some input data; add some tests
!---------------------------------------------------------------------------------------------------
use rhd_baserad_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix, AngleType
!
! --- Local variables ---
integer                                 :: i, j
real                                    :: theta, phi, psi
real, dimension(3,3)                    :: M, Munity, Mtheta, Mphi, Mpsi, Mminus
character(len=80)                       :: AngleSetName
character(len=128)                      :: TestName
real                                    :: delta
!---------------------------------------------------------------------------------------------------
!
! --- Create unity matrix ---
Munity(1,1)=1.0  ;  Munity(2,1)=0.0  ;  Munity(3,1)=0.0
Munity(1,2)=0.0  ;  Munity(2,2)=1.0  ;  Munity(3,2)=0.0
Munity(1,3)=0.0  ;  Munity(2,3)=0.0  ;  Munity(3,3)=1.0
!
! === Create test case =============================================================================
AngleSetName='AnglesAllZero'
theta       =0.0
phi         =0.0
psi         =0.0
delta       =1.0E-05
! --- Construct the rotation matrix ---
call rhd_baserad_RotationMatrix(theta, phi, psi, M, AngleType=AngleType)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'MatrixIsUnity'
call UT_Assert('==', TestName, M, Munity, delta=delta)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'MatrixIsNotUnity'
call UT_Assert('>=', TestName, (/ sum(abs(M-Munity)) /), 1.0E-05, delta=0.0, Ignore_flag=.true.)
!
! === Create test case =============================================================================
AngleSetName='AnglethetaOne'
theta       =1.0
phi         =0.0
psi         =0.0
delta       =1.0E-05
! --- Construct the rotation matrix ---
call rhd_baserad_RotationMatrix( theta, phi, psi, M     , AngleType=AngleType)
call rhd_baserad_RotationMatrix(-theta, phi, psi, Mminus, AngleType=AngleType)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'MatrixIsOrthonormal'
call UT_Assert('==', TestName, matmul(M, transpose(M)), Munity, delta=delta)
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', M)
endif
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'NegativeAngleGivesInverse'
call UT_Assert('==', TestName, matmul(M, Mminus), Munity, delta=delta)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'AbsMatrixIsNotUnity'
call UT_Assert('>=', TestName, (/ sum(abs(abs(M)-Munity)) /), 1.0E-05, delta=0.0)
!
! === Create test case =============================================================================
AngleSetName='AnglephiOne'
theta       =0.0
phi         =1.0
psi         =0.0
delta       =1.0E-05
! --- Construct the rotation matrix ---
call rhd_baserad_RotationMatrix(theta,  phi, psi, M     , AngleType=AngleType)
call rhd_baserad_RotationMatrix(theta, -phi, psi, Mminus, AngleType=AngleType)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'MatrixIsOrthonormal'
call UT_Assert('==', TestName, matmul(M, transpose(M)), Munity, delta=delta)
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', M)
endif
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'NegativeAngleGivesInverse'
call UT_Assert('==', TestName, matmul(M, Mminus), Munity, delta=delta)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'AbsMatrixIsNotUnity'
call UT_Assert('>=', TestName, (/ sum(abs(abs(M)-Munity)) /), 1.0E-05, delta=0.0)
!
! === Create test case =============================================================================
AngleSetName='AnglepsiOne'
theta       =0.0
phi         =0.0
psi         =1.0
delta       =1.0E-05
! --- Construct the rotation matrix ---
call rhd_baserad_RotationMatrix(theta, phi,  psi, M     , AngleType=AngleType)
call rhd_baserad_RotationMatrix(theta, phi, -psi, Mminus, AngleType=AngleType)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'MatrixIsOrthonormal'
call UT_Assert('==', TestName, matmul(M, transpose(M)), Munity, delta=delta)
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', M)
endif
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'NegativeAngleGivesInverse'
call UT_Assert('==', TestName, matmul(M, Mminus), Munity, delta=delta)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'AbsMatrixIsNotUnity'
call UT_Assert('>=', TestName, (/ sum(abs(abs(M)-Munity)) /), 1.0E-05, delta=0.0)
!
! === Create test case =============================================================================
AngleSetName='AnglesAllNonZero'
theta       =0.5
phi         =0.6
psi         =0.7
delta       =1.0E-05
! --- Construct the rotation matrix ---
call rhd_baserad_RotationMatrix(theta, 0.0, 0.0, Mtheta, AngleType=AngleType)
call rhd_baserad_RotationMatrix(0.0  , phi, 0.0, Mphi  , AngleType=AngleType)
call rhd_baserad_RotationMatrix(0.0  , 0.0, psi, Mpsi  , AngleType=AngleType)
call rhd_baserad_RotationMatrix(theta, phi, psi, M     , AngleType=AngleType)
!
! M(1,1)=2.0  ! --- Attention: forced error ---
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'MatrixMultiplicationWorks'
call UT_Assert('==', TestName, matmul(Mpsi, matmul(Mtheta, Mphi)), M, delta=delta)
!
TestName=trim(TestPrefix) // '_RotationMatrix_' // trim(AngleType) // '_' // &
         trim(AngleSetName) // '_' // &
         'AbsMatrixIsNotUnity'
call UT_Assert('>=', TestName, (/ sum(abs(abs(M)-Munity)) /), 1.0E-05, delta=0.0)
!
!do j=1,5
!  M=Munity
!  call rhd_baserad_DirShuffle(M)
!  do i=1,3
!    write(6,'(A,1X,I2,1X,3F4.0)') 'M_shuffle', i, M(:,i)
!  end do ! i
!  write(6,*)
!end do ! j
!
end subroutine rhd_baserad_RotationMatrixUT


!----------***************************--------------------------------------------------------------
subroutine rhd_baserad_RayDirectionsUT(TestPrefix, radraystar)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_RayDirectionsUT ('rhd_base_radiation_Ray_Directions_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_baserad_RayDirections.
!
! CATEGORY:
!   Radiation transport, testing
!
! CALLING SEQUENCE:
!   call rhd_baserad_RayDirectionsUT(TestPrefix, radraystar)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   radraystar:   ('radiation_ray_star') character, name of axis system, e.g.:
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_Assert:             Check array or relation between two arrays
!   rhd_baserad_RayDirections: Create list of basic direction vectors
!
! MODULES:
!   rhd_baserad_module:    Basic routines for radiation transport
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_baserad_RayDirectionsUT('baserad', 'cube')
!
! MODIFICATION HISTORY:
!   2016-05-16 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use rhd_baserad_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix, radraystar
!
! --- Local variables ---
character(len=128)                      :: TestName
real                                    :: delta
real, dimension(:,:), pointer           :: ray_dir0
real, dimension(:),   pointer           :: ray_wei

!character(len=80)                       :: nRayStr
!logical                                 :: Ignore_flag
!integer                                 :: iRay
!integer, dimension(:,:), allocatable    :: ia
!---------------------------------------------------------------------------------------------------
!
! --- Create the list of basic ray-direction vectors ---
call rhd_baserad_RayDirections(radraystar, ray_dir0, ray_wei)
!
! === Create test case =============================================================================
delta=1.0E-05
TestName=trim(TestPrefix) // '_RayDirections_' // trim(radraystar) // '_' // &
         'VectorsHaveLengthUnity'
call UT_Assert('==', TestName, sum(ray_dir0**2, 1), 1.0, delta=delta)
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', transpose(ray_dir0))
endif
!
! --- Deallocate temporary pointer arrays ---
deallocate(ray_dir0, ray_wei)
!
end subroutine rhd_baserad_RayDirectionsUT


!----------***********************------------------------------------------------------------------
subroutine rhd_baserad_PipeIndexUT(TestPrefix, nRay, nChunk)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_PipeIndexUT ('rhd_base_radiation_Pipe_Index_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_baserad_PipeIndex.
!
! CATEGORY:
!   Radiation transport, parallelization, testing
!
! CALLING SEQUENCE:
!   call rhd_baserad_PipeIndexUT(TestPrefix, nRay, nChunk)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nRay:         ('number_Ray') integer with total number of rays (or integrations)
!   nChunk:       ('number_Chunk') integer with number of chunks in current integration direction
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_Assert:             Check array or relation between two arrays
!   rhd_baserad_PipeIndex: Compute the ray index for job pipelining
!
! MODULES:
!   rhd_baserad_module:    Basic routines for radiation transport
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_baserad_PipeIndexUT('baserad', 3, 4)
!
! MODIFICATION HISTORY:
!   2016-04-16 (B.F. Uppsala) First version
!   2016-04-17 (B.F.) Add some tests
!---------------------------------------------------------------------------------------------------
use rhd_baserad_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nRay, nChunk
!
! --- Local variables ---
character(len=128)                      :: TestName
character(len=80)                       :: nRayStr, nChunkStr
logical                                 :: Ignore_flag, Up_flag
integer                                 :: iRay, nJob, iJob, iChunk, nJob1, nChunk1
integer, dimension(:,:), allocatable    :: ia
!---------------------------------------------------------------------------------------------------
!
! --- Compute total number of jobs and allocate array correspondingly, with one extra layer ---
call rhd_baserad_PipeIndex(nRay, nChunk, 1, 1, &
                           iRay, nJob, Up_flag)
nJob1  =max(nJob  +1,1)
nChunk1=max(nChunk+1,1)
allocate(ia(0:nJob1, 0:nChunk1))
!
! --- Transform number of rays and number of chunkgs into characters ---
write(nRayStr, *) nRay
nRayStr=adjustl(nRayStr)
write(nChunkStr, *) nChunk
nChunkStr=adjustl(nChunkStr)
!
! === Compute and store the ray indices (unsigned) =================================================
do iChunk=0,nChunk1
  do iJob=0,nJob1
    call rhd_baserad_PipeIndex(nRay, nChunk, iJob, iChunk, &
                               iRay, nJob, Up_flag)
    ia(iJob, nChunk1-iChunk)=iRay
  end do ! iJob
end do ! iChunk
!
! --- Test that all values are zero or larger ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
          '_nChunk=' // trim(nChunkStr) // '_' // &
         'ValuesArePositive'
!
call UT_Assert('>=', TestName, ia, 0, delta=0)
!
! --- Test that no values are above the total number of rays ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
          '_nChunk=' // trim(nChunkStr) // '_' // &
         'ValuesAreNotLargerThanRayNumber'
!
call UT_Assert('<=', TestName, ia, nRay, delta=0)
!
! === Compute and store the ray indices (signed) ===================================================
do iChunk=0,nChunk1
  do iJob=0,nJob1
    call rhd_baserad_PipeIndex(nRay, nChunk, iJob, iChunk, &
                               iRay, nJob, Up_flag)
    if (.not. Up_flag) iRay=-iRay
    ia(iJob, nChunk1-iChunk)=iRay
  end do ! iJob
end do ! iChunk
!
! --- Check number of upward ray jobs ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'NumberOfUpwardRayJobsIsCorrect'
!
call UT_Assert('==', TestName, (/sum(min(max(ia, 0), 1))/), nRay*nChunk, delta=0)
!
! --- Check number of downward ray jobs ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'NumberOfDownwardRayJobsIsCorrect'
!
call UT_Assert('==', TestName, (/sum(max(min(ia, 0), -1))/), -nRay*nChunk, delta=0)
!
! --- Check sum of sign of ray indices (each ray is assigned 1 or -1) ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'SignedNumberOfSignedValuesIsZeroForEachChunk'
!
call UT_Assert('==', TestName, &
               sum(ia(1:nJob1-1,1:nChunk1-1)/max(abs(ia(1:nJob1-1,1:nChunk1-1)), 1), 1), &
               0, delta=0)
!
! --- Check sum of signed ray indices ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'SumOfSignedValuesIsZeroForEachChunk'
!
call UT_Assert('==', TestName, sum(ia,1), 0, delta=0)
!
! --- Check if all values in first row are zero ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'AllValuesInFirstRowAreZero'
!
call UT_Assert('==', TestName, ia(:,0), 0, delta=0)
!
! --- Check if all values in last row are zero ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'AllValuesInLastRowAreZero'
!
call UT_Assert('==', TestName, ia(:,nChunk1), 0, delta=0)
!
! --- Check if all values in first column are zero ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'AllValuesInFirstColumnAreZero'
!
call UT_Assert('==', TestName, ia(0,:), 0, delta=0)
!
! --- Check if all values in last column are zero ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'AllValuesInLastColumnAreZero'
!
call UT_Assert('==', TestName, ia(nJob1,:), 0, delta=0)
!
! --- Do ignored test to mark all rays in output ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'SignedValuesAreZero'
!
call UT_Assert('==', TestName, ia, 0, delta=0, Ignore_flag=.true.)
!
! --- Do ignored test to mark all upward rays in output ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'SignedValuesAreNonPositive'
!
call UT_Assert('<=', TestName, ia, 0, delta=0, Ignore_flag=.true.)
!
! --- Do ignored test to mark all downward rays in output ---
TestName=trim(TestPrefix) // '_PipeIndex' // '_nRay=' // trim(nRayStr) // &
         '_nChunk=' // trim(nChunkStr) // '_' // &
         'SignedValuesAreNonNegative'
!
call UT_Assert('>=', TestName, ia, 0, delta=0, Ignore_flag=.true.)
!
deallocate(ia)
!
end subroutine rhd_baserad_PipeIndexUT


!----------*********************--------------------------------------------------------------------
subroutine rhd_baseradUT_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baseradUT_TestAll ('radiation_hydrodynamics_base_radiation_Unit_Test_Test_All')
!
! PURPOSE:
!   Perform unit tests of basic radiation-transport routines.
!
! CATEGORY:
!   Algebra, testing
!
! CALLING SEQUENCE:
!   call rhd_baseradUT_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_baserad_PipeIndexUT: Perform unit tests for rhd_baserad_PipeIndex
!   rhd_baserad_RayDirectionsUT: Perform unit tests for rhd_baserad_RayDirections
!   rhd_baserad_RotationMatrixUT: Perform unit tests for rhd_baserad_RotationMatrix
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in the test subroutines.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Perform all unit tests of basic radiation-transport routines.
!
! EXAMPLE:
!   call rhd_baseradUT_TestAll()
!
! MODIFICATION HISTORY:
!   2015-02-28 (B.F. Uppsala) First version
!   2016-04-16 (B.F.) Test rhd_baserad_PipeIndex
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='baserad'
!
call rhd_baserad_RotationMatrixUT(TestPrefix, 'EulerZXZ'    )
call rhd_baserad_RotationMatrixUT(TestPrefix, 'TaitBryanXYZ')
!
call rhd_baserad_RayDirectionsUT('baserad', 'oktaeder')
call rhd_baserad_RayDirectionsUT('baserad', 'oktaederr-dd1(1)')
call rhd_baserad_RayDirectionsUT('baserad', 'tetraeder')
call rhd_baserad_RayDirectionsUT('baserad', 'cube')
call rhd_baserad_RayDirectionsUT('baserad', 'ikosaeder')
call rhd_baserad_RayDirectionsUT('baserad', 'dodekaeder')
!
call rhd_baserad_PipeIndexUT(TestPrefix, 1, 1)
call rhd_baserad_PipeIndexUT(TestPrefix, 3, 1)
call rhd_baserad_PipeIndexUT(TestPrefix, 1, 2)
call rhd_baserad_PipeIndexUT(TestPrefix, 2, 2)
call rhd_baserad_PipeIndexUT(TestPrefix, 2, 3)
call rhd_baserad_PipeIndexUT(TestPrefix, 3, 4)
call rhd_baserad_PipeIndexUT(TestPrefix, 5, 4)
call rhd_baserad_PipeIndexUT(TestPrefix, 3, 5)
!
end subroutine rhd_baseradUT_TestAll


!----------**********************-------------------------------------------------------------------
subroutine rhd_baseradUT_TestSome()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baseradUT_TestSome ('radiation_hydrodynamics_base_radiation_Unit_Test_Test_Some')
!
! PURPOSE:
!   Perform unit tests of basic radiation-transport routines.
!
! CATEGORY:
!   Algebra, testing
!
! CALLING SEQUENCE:
!   call rhd_baseradUT_TestSome()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_baserad_RayDirectionsUT: Perform unit tests for rhd_baserad_RayDirections
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in the test subroutines.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Perform all unit tests of basic radiation-transport routines.
!
! EXAMPLE:
!   call rhd_baseradUT_TestSome()
!
! MODIFICATION HISTORY:
!   2015-02-28 (B.F. Uppsala) First version
!   2016-04-16 (B.F.) Test rhd_baserad_PipeIndex
!   2017-08-26 (B.F. Lyon) Adapted from rhd_baseradUT_TestAll
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='baserad'
!
call rhd_baserad_RayDirectionsUT('baserad', 'oktaeder')
call rhd_baserad_RayDirectionsUT('baserad', 'tetraeder')
call rhd_baserad_RayDirectionsUT('baserad', 'cube')
call rhd_baserad_RayDirectionsUT('baserad', 'ikosaeder')
!
end subroutine rhd_baseradUT_TestSome

end module rhd_baseradUT_module
