!***************************************************************************************************
!
!   #####  #    # #####        #####    ##    ####  ###### #####    ##   #####
!   #    # #    # #    #       #    #  #  #  #      #      #    #  #  #  #    #
!   #    # ###### #    #       #####  #    #  ####  #####  #    # #    # #    #
!   #####  #    # #    #       #    # ######      # #      #####  ###### #    #
!   #   #  #    # #    #       #    # #    # #    # #      #   #  #    # #    #
!   #    # #    # #####  ##### #####  #    #  ####  ###### #    # #    # ##### _module
!
!   Radiation-Hydrodynamics Code: basic routines for radiation transport
!
!***************************************************************************************************
!   Fortran90
!   2018-11-29
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Uppsala, Montpellier, Lyon, Naples
!***************************************************************************************************
! MODULES:
!   rhd_baserad_module:    Basic routines for radiation transport
!
!***************************************************************************************************


!------******************---------------------------------------------------------------------------
module rhd_baserad_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_module ('rhd_base_radiation_module')
!
! PURPOSE:
!   Provide based routines for radiation transport
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   use rhd_baserad_module
!
! VARIABLES:
!   None
!
! ROUTINES:
!   rhd_baserad_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!   rhd_baserad_RotationMatrixAxisAngle: Create matrix for rotation from rotation axis and one angle
!   rhd_baserad_DirShuffle: Shuffle and revert axis directions randomly
!   rhd_baserad_BaseAxisSystem: Generate random or unity basic orthonormal axis system
!   rhd_baserad_RayDirections: Create list of basic direction vectors
!   rhd_baserad_RaySystem: Create list of direction vectors for actual rotation matrix
!   rhd_baserad_PipeIndex: Compute the ray index for job pipelining
!
! CALLING TREE:
!   rhd_baserad_RaySystem
!   + rhd_baserad_RayDirections
!   + rhd_baserad_BaseAxisSystem
!     . rhd_baserad_RotationMatrix
!     . rhd_baserad_RotationMatrixAxisAngle
!     . rhd_baserad_DirShuffle
!
! MODIFICATION HISTORY:
!   2015-02-28 (B.F. Uppsala) Routines moved from rhd_shortrad_module
!   2016-04-16 (B.F.) Add routine rhd_baserad_PipeIndex
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------**************************---------------------------------------------------------------
subroutine rhd_baserad_RotationMatrix(theta, phi, psi, M, AngleType)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_RotationMatrix ('rhd_base_radiation_Rotation_Matrix')
!
! PURPOSE:
!   Create matrix for rotation in 3D from 3 angles.
!
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_baserad_RotationMatrix(theta, phi, psi, M, AngleType=AngleType)
!
! INPUT:
!   theta:        ('theta') real, angle for rotation matrix
!   phi:          ('phi') real, angle for rotation matrix
!   psi:          ('psi') real, angle for rotation matrix
!   AngleType:    ('Angle_Type') optional character
!                   'TaitBryanXYZ': Tait-Bryan X-Y-Z angles
!                   'EulerZXZ':     Euler Z-X-Z angles (default)
!
! OUTPUT:
!   M:            ('Matrix') 3x3 rotation matrix
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   See Herbert Goldstein, "Klassische Mechanik", p. 121
!   See \href{http://en.wikipedia.org/wiki/Euler_angles#Rotation_matrix}{Wikipedia}
!
! EXAMPLE:
!                               / 1 0 0 \
!   theta=0, phi=0, psi=0 -> M= | 0 1 0 |
!                               \ 0 0 1 /
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Copenhagen) First version
!   2014-01-20 (B.F. Lyon) Parameter AngleType added
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: theta, phi, psi
real, dimension(3,3),      intent(out)  :: M
character(len=*), optional,intent(in)   :: AngleType
!
! --- Local variables ---
character(len=80)                       :: AngleType0
!---------------------------------------------------------------------------------------------------
!
! --- Set default values for keyword ---
if (present(AngleType)) then
  AngleType0=AngleType
else
  AngleType0='EulerZXZ'
endif
!
if (AngleType0 == 'TaitBryanXYZ') then
  ! --- Tait-Bryan angles ---
  !
  M(1,1)=          cos(theta)*cos(psi)
  M(1,2)= cos(phi)*           sin(psi) + sin(phi)*sin(theta)*cos(psi)
  M(1,3)= sin(phi)*           sin(psi) - cos(phi)*sin(theta)*cos(psi)
  !
  M(2,1)=         -cos(theta)*sin(psi)
  M(2,2)= cos(phi)*           cos(psi) - sin(phi)*sin(theta)*sin(psi)
  M(2,3)= sin(phi)*           cos(psi) + cos(phi)*sin(theta)*sin(psi)
  !
  M(3,1)=          sin(theta)
  M(3,2)=-sin(phi)*cos(theta)
  M(3,3)= cos(phi)*cos(theta)
  !
else ! (AngleType0 == 'EulerZXZ')
  ! --- Euler angles ---
  !
  M(1,1)= cos(phi)*cos(psi) - cos(theta)*sin(phi)*sin(psi)
  M(1,2)= sin(phi)*cos(psi) + cos(theta)*cos(phi)*sin(psi)
  M(1,3)=                     sin(theta)*         sin(psi)
  !
  M(2,1)=-cos(phi)*sin(psi) - cos(theta)*sin(phi)*cos(psi)
  M(2,2)=-sin(phi)*sin(psi) + cos(theta)*cos(phi)*cos(psi)
  M(2,3)=                     sin(theta)*         cos(psi)
  !
  M(3,1)=                     sin(theta)*sin(phi)
  M(3,2)=                    -sin(theta)*cos(phi)
  M(3,3)=                     cos(theta)
  !
endif ! (AngleType0)
!
end subroutine rhd_baserad_RotationMatrix


!----------***********************************------------------------------------------------------
subroutine rhd_baserad_RotationMatrixAxisAngle(phi, x, M)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_RotationMatrixAxisAngle ('rhd_short-char_radiation_Rotation_Matrix_Axis_Angle')
!
! PURPOSE:
!   Create matrix for rotation in 3D from axis and one angle.
!
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_baserad_RotationMatrixAxisAngle(phi, x, M)
!
! INPUT:
!   phi:          ('phi') real, angle for rotation matrix
!   x:            ('x') real 3-element unit vector
!
! OUTPUT:
!   M:            ('Matrix') 3x3 rotation matrix
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The vector x has to have length unity. The length is not checked.
!
! PROCEDURE:
!   Compute 3x3 rotation matrix from axis (3-element unit vector) and angle.
!   See \href{http://en.wikipedia.org/wiki/Rotation_matrix#Axis_and_angle}{Wikipedia}
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-01-20 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: phi
real, dimension(3),        intent(in)   :: x
real, dimension(3,3),      intent(out)  :: M
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
M(1,1)=x(1)*x(1) * (1.0-cos(phi)) +        cos(phi)
M(1,2)=x(2)*x(1) * (1.0-cos(phi)) + x(3) * sin(phi)
M(1,3)=x(3)*x(1) * (1.0-cos(phi)) - x(2) * sin(phi)
!
M(2,1)=x(1)*x(2) * (1.0-cos(phi)) - x(3) * sin(phi)
M(2,2)=x(2)*x(2) * (1.0-cos(phi)) +        cos(phi)
M(2,3)=x(3)*x(2) * (1.0-cos(phi)) + x(1) * sin(phi)
!
M(3,1)=x(1)*x(3) * (1.0-cos(phi)) + x(2) * sin(phi)
M(3,2)=x(2)*x(3) * (1.0-cos(phi)) - x(1) * sin(phi)
M(3,3)=x(3)*x(3) * (1.0-cos(phi)) +        cos(phi)
!
end subroutine rhd_baserad_RotationMatrixAxisAngle


!----------**********************-------------------------------------------------------------------
subroutine rhd_baserad_DirShuffle(M)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_DirShuffle ('rhd_base_radiation_Direction_Shuffle')
!
! PURPOSE:
!   Shuffle and revert axes directions randomly.
!
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_baserad_DirShuffle(M)
!
! INPUT:
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   M:            ('Matrix') 3x3 input/output matrix to shuffle
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   The function random_number is called 6 times.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Randomly shuffle elements of input matrix.
!   The shuffle matrix M_shuffle has exactly one non-zero element (1 or -1) per row and colum.
!   The distribution of position and sign should be random.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-01-17 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real, dimension(3,3),      intent(inout):: M
!
! --- Local variables ---
integer                                 :: i, ir, ii
integer, parameter                      :: n=3
integer, dimension(3)                   :: IndexArray
real                                    :: r, rs
real, dimension(3,3)                    :: M_shuffle
!---------------------------------------------------------------------------------------------------
!
! --- Initialize index array ---
do i=1,n
  IndexArray(i)=i
end do ! i
!
! --- Initialize shuffle matrix ---
M_shuffle=0.0
!
do i=1,n
  call random_number(r)
  call random_number(rs)
  ir=floor(float(n-i+1)*r)+1
  ii=IndexArray(ir)
  IndexArray(ir:n-1)=IndexArray(ir+1:n)
  !write(6,'(A,I3,2X,2I2,2X,3I2)') 'i, ir, ii:', i, ir, ii, IndexArray(1:n-i+1)
  if (rs > 0.5) then
    M_shuffle(ii,i)= 1.0
  else
    M_shuffle(ii,i)=-1.0
  endif
  !write(6,'(A,1X,I2,1X,3F4.0)') 'M_shuffle', i, M_shuffle(:,i)
end do ! i
!
M=matmul(M_shuffle, M)
!
end subroutine rhd_baserad_DirShuffle


!----------**************************---------------------------------------------------------------
subroutine rhd_baserad_BaseAxisSystem(radraybase, nc_m, itime, M_axis)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_BaseAxisSystem ('rhd_base_radiation_Base_Axis_System')
!
! PURPOSE:
!   Generate random or unity basic orthonormal axis system.
!
! CATEGORY:
!   Vector algebra
!
! CALLING SEQUENCE:
!   call rhd_baserad_BaseAxisSystem(radraybase, nc_m, itime, M_axis)
!
! INPUT:
!   radraybase:   ('radiation_ray_base') character, name of basic axis system.
!                   'unity'                : unity matrix
!                   'random', 'randomgroup': rotation matrix with random orientation,
!                                            same probability for all orientations
!                   'randomcube'           : rotation matrix with random orientation, diagonals
!                                            preferred to account for non-symmetric weights
!                   'drift01'              : Slow drift, algorithm 1
!                   'drift02'              : Slow drift, algorithm 2
!                   'drift03'              : Slow drift, algorithm 3
!   nc_m:         ('number_channel_MPI_print') integer, channel for text output, typically 6
!   itime:        ('index_time') integer, time step number
!
! OUTPUT:
!   M_axis:       ('Matrix_axis') 3x3 real array, specifying the base axes of the ray system
!
! ROUTINES:
!   rhd_baserad_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!   rhd_baserad_RotationMatrixAxisAngle: Create matrix for rotation from rotation axis and one angle
!   rhd_baserad_DirShuffle: Shuffle and revert axis directions randomly
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Random number generator has to be initialized.
!
! PROCEDURE:
!   A 3x3 rotation matrix is produced: three orthonormal vectors
!   The unity matrix (radraybase='unity')
!   or a matrix with random orientation (radraybase='random' or 'randomgroup')
!   is produced. In the latter case the probability for all orientations should be the same.
!
! EXAMPLE:
!
! TODO:
!   given theta, phi, psi
!   save theta, phi, psi
!   given delta-(theta, phi, psi)
!
! MODIFICATION HISTORY:
!   1999-11-22 (B.F. Copenhagen) Put into subroutine
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2001-11-24 (B.F.) srk removed
!   2008-06-20 (B.F.) Parameter itime added
!   2008-06-27 (B.F.) Option drift02
!   2012-03-08 (B.F.) Option drift03
!   2014-01-20 (B.F.) Option randomcube
!---------------------------------------------------------------------------------------------------
use const_module, only: pi
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: radraybase
integer,                   intent(in)   :: nc_m, itime
real, dimension(3,3),      intent(out)  :: M_axis
!
! --- Local variables ---
!real, parameter                         :: pi=3.1415926535897932384626433
real                                    :: alpha, theta, phi, psi
real, dimension(3)                      :: x
!---------------------------------------------------------------------------------------------------
!
! --- Create base axis system ---
select case (radraybase)
  case('unity')
    ! --- Unity matrix ---
    !
    M_axis     =0.0
    M_axis(1,1)=1.0
    M_axis(2,2)=1.0
    M_axis(3,3)=1.0
  case('random', 'randomgroup')
    ! --- Matrix with random orientation (but equally distributed over all spatial angles) ---
    !
    call random_number(theta)
    call random_number(phi)
    call random_number(psi)
    theta=acos(2.0*theta-1.0)
    phi  =2.0*pi*phi
    psi  =2.0*pi*psi
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis)
    !call rhd_baserad_DirShuffle(M_axis)
  case('randomcube_bak01')
    ! --- Matrix with random orientation (but corrected for 'cube' weights) ---
    !
    call random_number(theta)
    call random_number(phi)
    call random_number(psi)
    !
    !theta=acos(2.0*theta-1.0)
    !theta=acos(0.02*(theta-0.5))
    !theta=acos(1.0-0.157*theta**3)
    theta=acos(1.0-0.157*theta**2)
    !theta=0.0
    !
    !phi  =2.0*pi*phi
    !phi  =0.1 *pi * (2.0*(phi-0.5))*abs(2.0*(phi-0.5))
    phi  =1.0 *pi * (2.0*(phi-0.5))
    !phi  =0.0
    !
    !psi  =2.0*pi*psi
    psi  =0.25*pi * (2.0*(psi-0.5))*abs(2.0*(psi-0.5))**2
    !psi  =0.8 *pi * (2.0*(psi-0.5))*abs(2.0*(psi-0.5))
    !psi  =0.5 *pi * (2.0*(psi-0.5))
    !psi  =0.0
    !
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis)
    call rhd_baserad_DirShuffle(M_axis)
  case('randomcube_bak02')
    ! --- Matrix with random orientation (but corrected for 'cube' weights) ---
    !
    call random_number(theta)
    call random_number(phi)
    call random_number(psi)
    !
    !theta=acos(2.0*theta-1.0)
    !theta=acos(0.02*(theta-0.5))
    !theta=acos(1.0-0.157*theta**3)
    !theta=acos(1.0-0.157*theta**2)
    theta=0.2*pi * (2.0*(theta-0.5))*abs(2.0*(theta-0.5))
    !theta=0.0
    !
    !phi  =2.0*pi*phi
    phi  =0.2*pi * (2.0*(phi  -0.5))*abs(2.0*(phi  -0.5))
    !phi  =1.0 *pi * (2.0*(phi-0.5))
    !phi  =0.0
    !
    !psi  =2.0*pi*psi
    psi  =0.2*pi * (2.0*(psi  -0.5))*abs(2.0*(psi  -0.5))
    !psi  =0.8 *pi * (2.0*(psi-0.5))*abs(2.0*(psi-0.5))
    !psi  =0.5 *pi * (2.0*(psi-0.5))
    !psi  =0.0
    !
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis, AngleType='TaitBryanXYZ')
    call rhd_baserad_DirShuffle(M_axis)
  case('randomcube')
    ! --- Matrix with random orientation (but corrected for 'cube' weights) ---
    !
    call random_number(theta)
    call random_number(phi)
    call random_number(psi)
    !
    theta=acos(2.0*theta-1.0)
    phi  =2.0*pi*phi
    psi  =2.0*pi*psi
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis)
    !call rhd_baserad_DirShuffle(M_axis)
    !
    call random_number(phi)
    !phi=phi
    !phi  =10.0*pi * (2.0*(phi  -0.5))
    !phi  =pi * (2.0*(phi  -0.5))
    !phi  =pi * (2.0*(phi))
    !phi  =0.3*pi * (2.0*(phi  -0.5))*abs(2.0*(phi  -0.5))
    !phi  =0.3*pi * (2.0*(phi))*(2.0*(phi))
    !phi=0.5
    !phi=0.8*phi
    phi=acos(1.0-2.0*phi)
    phi=0.26*phi
    !phi=0.35*phi
    x=M_axis(:,3)
    call rhd_baserad_RotationMatrixAxisAngle(phi, x, M_axis)
!    call rhd_baserad_DirShuffle(M_axis)
  case('drift01')
    ! --- Matrix with gradually changing orientation, depending on time step number ---
    !
    theta=mod(real(itime)/456.789, 1.0)
    phi  =mod(theta*((sqrt(5.0)+1.0)*0.5), 1.0)
    psi  =mod(phi/sqrt(2.0), 1.0)
    theta=acos(2.0*theta-1.0)
    phi  =2.0*pi*phi
    psi  =2.0*pi*psi
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis)
    if (nc_m >= 0) then
      write(nc_m,*) 'theta, phi, psi:', theta, phi, psi
    endif
  case('drift02')
    ! --- Matrix with gradually changing orientation, depending on time step number ---
    !
    theta=mod(real(itime)/678.901, 1.0)
    phi  =mod(theta*((sqrt(5.0)+1.0)*0.5), 1.0)
    psi  =mod(phi/sqrt(2.0), 1.0)
    theta=acos(2.0*theta-1.0)
    phi  =2.0*pi*phi
    psi  =2.0*pi*psi
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis)
    if (nc_m >= 0) then
      write(nc_m,*) 'theta, phi, psi:', theta, phi, psi
    endif
  case('drift03')
    ! --- Matrix with gradually changing orientation, depending on time step number ---
    !
    alpha=real(itime)/456.789
    theta=mod(alpha, 1.0)
    alpha=alpha*((sqrt(5.0)+1.0)*0.5)
    phi  =mod(alpha, 1.0)
    alpha=alpha/sqrt(2.0)
    psi  =mod(alpha, 1.0)
    theta=acos(2.0*theta-1.0)
    phi  =2.0*pi*phi
    psi  =2.0*pi*psi
    call rhd_baserad_RotationMatrix(theta, phi, psi, M_axis)
    if (nc_m >= 0) then
      write(nc_m,*) 'theta, phi, psi:', theta, phi, psi
    endif
  case default
    ! --- Default: unity matrix ---
    !
    M_axis     =0.0
    M_axis(1,1)=1.0
    M_axis(2,2)=1.0
    M_axis(3,3)=1.0
end select
!
end subroutine rhd_baserad_BaseAxisSystem


!----------*************************----------------------------------------------------------------
subroutine rhd_baserad_RayDirections(radraystar, x, w)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_RayDirections ('rhd_base_radiation_Ray_Directions')
!
! PURPOSE:
!   Create list of basic direction vectors.
!
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_baserad_RayDirections(radraystar, x, w)
!
! INPUT:
!   radraystar:   ('radiation_ray_star') character, name of axis system, e.g.:
!                   x1:         ray along x1 axis (N=1)
!                   x2:         ray along x2 axis (N=1)
!                   x3:         ray along x3 axis (N=1)
!                   oktaeder:   rays along x1, x2, and x3 axes (N=3)
!                   tetraeder:  (N=4)
!                   cube:       (N=4)
!                   ikosaeder:  (N=6)
!                   dodekaeder: (N=10)
!                   ...
!
! OUTPUT:
!   x:            ('x') real 3xN matrix with N as number of rays according to radraystar
!   w:            ('weight') real N-element vector with ray weights (usually 1/Number_of_Rays)
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Allocate pointer arrays.
!
! RESTRICTIONS:
!   The arrays x and w are allocated. They have to be deallocated properly by the user.
!
! PROCEDURE:
!   An array with 3xN elements is allocated and filled with vector components
!   according to radraystar.
!   Each vector has length unity.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Copenhagen) First version
!   1999-11-22 (B.F.) Some more ray systems
!   1999-12-22 (B.F.) oktaeder210, 410
!   2000-11-24 (B.F. Uppsala) oktaeder110
!   2000-11-27 (B.F.) oktaederXXX -> oktaeder-XXX
!   2001-11-23 (B.F.) A lot more single-ray possibilities
!   2002-07-30 (B.F.) oktaeder-ddX, cube-ddd added
!   2005-03-08 (B.F.) 'oktaeder-dd1(1)', 'oktaeder-d1d(1)'
!   2013-12-04 (B.F.) Rename rhd_rad3d_RayDirections ->  rhd_shortrad_RayDirections; set ray weight
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: radraystar
real, dimension(:,:), pointer           :: x
real, dimension(:)  , pointer           :: w
!
! --- Local variables ---
integer                                 :: n_v, i
real                                    :: gold
real, parameter                         :: eps=3.0E-6
!---------------------------------------------------------------------------------------------------
!
select case (radraystar)
  case ('x1')
    ! --- Only rays along x1 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 0.0
    w(:)=1.0/real(max(n_v,3))
  case ('x2')
    ! --- Only rays along x2 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0        ; x(3,1) = 0.0
    w(:)=1.0/real(max(n_v,3))
  case ('x2-142')
    ! --- Only rays mainly along x2 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 4.0        ; x(3,1) = 2.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3')
    ! --- Only rays along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 0.0        ; x(3,1) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3-012')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0        ; x(3,1) = 2.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3-m1m24')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) =-1.0            ; x(2,1) =-2.0        ; x(3,1) = 4.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3-m124')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) =-1.0            ; x(2,1) = 2.0        ; x(3,1) = 4.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3-1m24')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) =-2.0        ; x(3,1) = 4.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3-124')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 2.0        ; x(3,1) = 4.0
    w(:)=1.0/real(max(n_v,3))
  case ('x3-101')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.999999       ; x(2,1) = 0.0        ; x(3,1) = 1.000001
    w(:)=1.0/real(max(n_v,3))
  case ('x3-011')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 0.999999   ; x(3,1) = 1.000001
    w(:)=1.0/real(max(n_v,3))
  case ('x3-mm1')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) =-0.999999       ; x(2,1) =-0.999999   ; x(3,1) = 1.000001
    w(:)=1.0/real(max(n_v,3))
  case ('x3-mp1')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) =-0.999999       ; x(2,1) = 0.999999   ; x(3,1) = 1.000001
    w(:)=1.0/real(max(n_v,3))
  case ('x3-pm1')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.999999       ; x(2,1) =-0.999999   ; x(3,1) = 1.000001
    w(:)=1.0/real(max(n_v,3))
  case ('x3-pp1')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.999999       ; x(2,1) = 0.999999   ; x(3,1) = 1.000001
    w(:)=1.0/real(max(n_v,3))
  case ('duo-012')
    ! --- Two rays ---
    n_v=2
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0        ; x(3,1) = 2.00000
    x(1,2) = 0.0            ; x(2,2) =-1.0        ; x(3,2) = 2.00000
    w(:)=1.0/real(max(n_v,3))
  case ('duo-102')
    ! --- Two rays ---
    n_v=2
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 2.00000
    x(1,2) =-1.0            ; x(2,2) = 0.0        ; x(3,2) = 2.00000
    w(:)=1.0/real(max(n_v,3))
  case ('duo-210')
    ! --- Two rays ---
    n_v=2
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 2.0            ; x(2,1) = 1.0        ; x(3,1) = 0.00000
    x(1,2) = 2.0            ; x(2,2) =-1.0        ; x(3,2) = 0.00000
    w(:)=1.0/real(max(n_v,3))
  case ('duo-201')
    ! --- Two rays ---
    n_v=2
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 2.0            ; x(2,1) = 0.0        ; x(3,1) = 1.00000
    x(1,2) = 2.0            ; x(2,2) = 0.0        ; x(3,2) =-1.00000
    w(:)=1.0/real(max(n_v,3))
  case ('duo-120')
    ! --- Two rays ---
    n_v=2
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 2.0        ; x(3,1) = 0.00000
    x(1,2) =-1.0            ; x(2,2) = 2.0        ; x(3,2) = 0.00000
    w(:)=1.0/real(max(n_v,3))
  case ('duo-021')
    ! --- Two rays ---
    n_v=2
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 2.0        ; x(3,1) = 1.00000
    x(1,2) = 0.0            ; x(2,2) = 2.0        ; x(3,2) =-1.00000
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder')
    ! --- Oktaeder: rays along x1, x2, and x3 axis (3 directions) ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 0.0
    x(1,2) = 0.0            ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-dd1(1)')
    ! --- Oktaeder: diagonal,diagonal,1 ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0+eps        ; x(2,1) =-1.0+eps    ; x(3,1) = 0.0
    x(1,2) = 1.0+eps        ; x(2,2) = 1.0-eps    ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-dd1', 'oktaeder-dd1(2)')
    ! --- Oktaeder: diagonal,diagonal,1 ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0-eps        ; x(2,1) =-1.0-eps    ; x(3,1) = 0.0
    x(1,2) = 1.0-eps        ; x(2,2) = 1.0+eps    ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-d1d(1)')
    ! --- Oktaeder: diagonal,1,diagonal  ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0        ; x(3,1) = 0.0
    x(1,2) = 1.0+eps        ; x(2,2) = 0.0        ; x(3,2) = 1.0-eps
    x(1,3) = 1.0+eps        ; x(2,3) = 0.0        ; x(3,3) =-1.0+eps
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-d1d', 'oktaeder-d1d(3)')
    ! --- Oktaeder: diagonal,1,diagonal  ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0        ; x(3,1) = 0.0
    x(1,2) = 1.0-eps        ; x(2,2) = 0.0        ; x(3,2) = 1.0+eps
    x(1,3) = 1.0-eps        ; x(2,3) = 0.0        ; x(3,3) =-1.0-eps
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-1dd(2)')
    ! --- Oktaeder: 1,diagonal,diagonal  ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 0.0
    x(1,2) = 0.0            ; x(2,2) = 1.0+eps    ; x(3,2) = 1.0-eps
    x(1,3) = 0.0            ; x(2,3) = 1.0+eps    ; x(3,3) =-1.0+eps
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-1dd', 'oktaeder-1dd(3)')
    ! --- Oktaeder: 1,diagonal,diagonal  ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 0.0
    x(1,2) = 0.0            ; x(2,2) = 1.0-eps    ; x(3,2) = 1.0+eps
    x(1,3) = 0.0            ; x(2,3) = 1.0-eps    ; x(3,3) =-1.0-eps
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-110')
    ! --- Oktaeder ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) =-1.0        ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-210')
    ! --- Oktaeder ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 2.0            ; x(2,1) =-1.0        ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 2.0        ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('oktaeder-410')
    ! --- Oktaeder ---
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 4.0            ; x(2,1) =-1.0        ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 4.0        ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('tetraeder')
    ! --- Tetraeder: rays in 4 directions ---
    n_v=4
    allocate(x(3,n_v), w(n_v))
    x(1,1) = sqrt(2.0)      ; x(2,1) = 1.0        ; x(3,1) = sqrt(6.0)
    x(1,2) =-2.0*sqrt(2.0)  ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = sqrt(2.0)      ; x(2,3) = 1.0        ; x(3,3) =-sqrt(6.0)
    x(1,4) = 0.0            ; x(2,4) =-3.0        ; x(3,4) = 0.0
    w(:)=1.0/real(max(n_v,3))
  case ('cube')
    ! --- Cube: rays in 4 directions ---
    n_v=4
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 1.0        ; x(3,1) = 1.0
    x(1,2) =-1.0            ; x(2,2) = 1.0        ; x(3,2) = 1.0
    x(1,3) = 1.0            ; x(2,3) =-1.0        ; x(3,3) = 1.0
    x(1,4) =-1.0            ; x(2,4) =-1.0        ; x(3,4) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('cube-ddd', 'cube-ddd(3)')
    ! --- Cube: rays in 4 directions: diagonal,diagonal,diagonal; main direction 3 ---
    n_v=4
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0-eps        ; x(2,1) = 1.0        ; x(3,1) = 1.0+eps
    x(1,2) =-1.0+eps        ; x(2,2) = 1.0        ; x(3,2) = 1.0+eps
    x(1,3) = 1.0-eps        ; x(2,3) =-1.0        ; x(3,3) = 1.0+eps
    x(1,4) =-1.0+eps        ; x(2,4) =-1.0        ; x(3,4) = 1.0+eps
    w(:)=1.0/real(max(n_v,3))
  case ('cube-ddd(2)')
    ! --- Cube: rays in 4 directions: diagonal,diagonal,diagonal; main direction 2 ---
    n_v=4
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0-eps        ; x(2,1) = 1.0+eps    ; x(3,1) = 1.0
    x(1,2) =-1.0+eps        ; x(2,2) = 1.0+eps    ; x(3,2) = 1.0
    x(1,3) = 1.0-eps        ; x(2,3) =-1.0-eps    ; x(3,3) = 1.0
    x(1,4) =-1.0+eps        ; x(2,4) =-1.0-eps    ; x(3,4) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('cube-ddd(1)')
    ! --- Cube: rays in 4 directions: diagonal,diagonal,diagonal; main direction 1 ---
    n_v=4
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0+eps        ; x(2,1) = 1.0-eps    ; x(3,1) = 1.0
    x(1,2) =-1.0-eps        ; x(2,2) = 1.0-eps    ; x(3,2) = 1.0
    x(1,3) = 1.0+eps        ; x(2,3) =-1.0+eps    ; x(3,3) = 1.0
    x(1,4) =-1.0-eps        ; x(2,4) =-1.0+eps    ; x(3,4) = 1.0
    w(:)=1.0/real(max(n_v,3))
  case ('ikosaeder')
    ! --- Ikosaeder: rays in 6 directions ---
    n_v=6
    allocate(x(3,n_v), w(n_v))
    gold = (1.0+sqrt(5.0))/2.0
    x(1,1)  = 0.0           ; x(2,1)  = gold      ; x(3,1)  = 1.0
    x(1,2)  = 0.0           ; x(2,2)  = gold      ; x(3,2)  =-1.0
    x(1,3)  = 1.0           ; x(2,3)  = 0.0       ; x(3,3)  = gold
    x(1,4)  = 1.0           ; x(2,4)  = 0.0       ; x(3,4)  =-gold
    x(1,5)  = gold          ; x(2,5)  = 1.0       ; x(3,5)  = 0.0
    x(1,6)  = gold          ; x(2,6)  =-1.0       ; x(3,6)  = 0.0
    w(:)=1.0/real(max(n_v,3))
  case ('duooktaeder-x11')
    ! --- Duo-Oktaeder: X,1,1 ---
    n_v=6
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 0.0        ; x(3,1) = 1.0
    x(1,2) = 0.0            ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = 1.0-eps        ; x(2,3) =-1.0-eps    ; x(3,3) = 0.0
    x(1,4) = 1.0-eps        ; x(2,4) = 1.0+eps    ; x(3,4) = 0.0
    x(1,5) = 1.0-eps        ; x(2,5) = 0.0        ; x(3,5) = 1.0+eps
    x(1,6) = 1.0-eps        ; x(2,6) = 0.0        ; x(3,6) =-1.0-eps
    w(:)=1.0/real(max(n_v,3))
  case ('duooktaeder-1x1','duooktaeder-1x1(3)')
    ! --- Duo-Oktaeder: 1,X,1 ---
    n_v=6
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 0.0        ; x(3,1) = 1.0
    x(1,2) = 1.0            ; x(2,2) = 0.0        ; x(3,2) = 0.0
    x(1,3) = 1.0-eps        ; x(2,3) =-1.0-eps    ; x(3,3) = 0.0
    x(1,4) = 1.0-eps        ; x(2,4) = 1.0+eps    ; x(3,4) = 0.0
    x(1,5) = 0.0            ; x(2,5) = 1.0-eps    ; x(3,5) = 1.0+eps
    x(1,6) = 0.0            ; x(2,6) = 1.0-eps    ; x(3,6) =-1.0-eps
    w(:)=1.0/real(max(n_v,3))
  case ('duooktaeder-11x','duooktaeder-11x(3)')
    ! --- Duo-Oktaeder: 1,1,X  ---
    n_v=6
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0        ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 0.0        ; x(3,2) = 0.0
    x(1,3) = 1.0-eps        ; x(2,3) = 0.0        ; x(3,3) = 1.0+eps
    x(1,4) = 1.0-eps        ; x(2,4) = 0.0        ; x(3,4) =-1.0-eps
    x(1,5) = 0.0            ; x(2,5) = 1.0-eps    ; x(3,5) = 1.0+eps
    x(1,6) = 0.0            ; x(2,6) = 1.0-eps    ; x(3,6) =-1.0-eps
    w(:)=1.0/real(max(n_v,3))
  case ('dodekaeder')
    ! --- Dodekaeder: rays in 10 directions ---
    n_v=10
    allocate(x(3,n_v), w(n_v))
    gold = (1.0+sqrt(5.0))/2.0
    x(1,1)  = 0.0           ; x(2,1)  = 1.0/gold  ; x(3,1)  = gold
    x(1,2)  = 0.0           ; x(2,2)  = 1.0/gold  ; x(3,2)  =-gold
    x(1,3)  = gold          ; x(2,3)  = 0.0       ; x(3,3)  = 1.0/gold
    x(1,4)  = gold          ; x(2,4)  = 0.0       ; x(3,4)  =-1.0/gold
    x(1,5)  = 1.0/gold      ; x(2,5)  = gold      ; x(3,5)  = 0.0
    x(1,6)  = 1.0/gold      ; x(2,6)  =-gold      ; x(3,6)  = 0.0
    x(1,7)  = 1.0           ; x(2,7)  = 1.0       ; x(3,7)  = 1.0
    x(1,8)  = 1.0           ; x(2,8)  = 1.0       ; x(3,8)  =-1.0
    x(1,9)  = 1.0           ; x(2,9)  =-1.0       ; x(3,9)  = 1.0
    x(1,10) = 1.0           ; x(2,10) =-1.0       ; x(3,10) =-1.0
    w(:)=1.0/real(max(n_v,3))
  case default
    n_v=3
    allocate(x(3,n_v), w(n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 0.0
    x(1,2) = 0.0            ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
    w(:)=1.0/real(max(n_v,3))
end select
!
! --- Normalize vectors ---
do i=1,n_v
  x(:,i)=x(:,i)/sqrt(sum(x(:,i)**2))
end do ! i
!
end subroutine rhd_baserad_RayDirections


!----------*********************--------------------------------------------------------------------
subroutine rhd_baserad_RaySystem(radraystar, radraybase, nc_m, itime, n_subdtime, &
                                 n_dir, ray_dir0, ray_dir, ray_wei, M_axis)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_RaySystem ('rhd_base_radiation_Ray_System')
!
! PURPOSE:
!   Create list of direction vectors for actual rotation matrix.
!
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_baserad_RaySystem(radraystar, radraybase, nc_m, itime, n_subdtime, &
!                              n_dir, ray_dir0, ray_dir, ray_wei, M_axis)
!
! INPUT:
!   nc_m:         ('number_channel_MPI_print') integer, channel for text output, typically 6
!   radraystar:   ('radiation_ray_star') character, name of axis system, e.g.:
!                   'list-01(1)': 'oktaeder-dd1(1)', 'oktaeder-d1d(1)', 'oktaeder-1dd','cube-ddd(1)'
!                   'list-01(2)': 'oktaeder-dd1', 'oktaeder-d1d', 'oktaeder-1dd(2)', 'cube-ddd(2)'
!                   'list-01(3)': 'oktaeder-dd1', 'oktaeder-d1d', 'oktaeder-1dd(3)', 'cube-ddd(3)'
!                   'list-01'   = 'list-01(3)'
!                   'list-02(3)': 'duooktaeder-11x', 'cube-ddd(3)', 'duooktaeder-1x1',
!                                 'cube-ddd(3)', 'duooktaeder-x11'
!                   'list-02'   = 'list-02(3)'
!                   'list-03(3)': 'oktaeder-dd1', 'oktaeder-d1d', 'oktaeder-1dd(3)'
!                   'list-03'   = 'list-03(3)'
!                   'list-04(3)': 'oktaeder-dd1', 'cube-ddd(3)', 'oktaeder-d1d', 'cube-ddd(3)',
!                                 'oktaeder-1dd(3)', 'cube-ddd(3)'
!                   'list-04'   = 'list-04(3)'
!                   otherwise   : value is passed to rhd_baserad_RayDirections
!   radraybase:   ('radiation_ray_base') character, name of basic axis system.
!                   'unity'                : unity matrix
!                   'random', 'randomgroup': rotation matrix with random orientation
!                                            same probability for all orientations
!                   'randomcube'           : rotation matrix with random orientation, diagonals
!                                            preferred to account for non-symmetric weights
!                   'alternate'            : Alternate between unity and random orientation matrix
!   itime:        ('index_time') integer, time step number
!   n_subdtime:   ('number_sub_delta_time') integer, number of sub time step
!
! OUTPUT:
!   n_dir:        ('number_directions') integer, total number of rays in ray_dir, ray_dir0,...
!   ray_dir0:     ('ray_directions_0') real 3xN matrix with N rays according to radraystar
!   ray_dir:      ('ray_directions') real 3xN matrix; ray_dir=matmul(M_axis,ray_dir0)
!   ray_wei:      ('ray_weights') real N-element vector with ray weights
!
! INPUT/OUTPUT:
!   M_axis:       ('Matrix_axis') 3x3 real array, specifying the base axes of the ray system,
!                 it is generated for n_subdtime=1 but may be reused for n_subdtime>1 depending on
!                 value of radraybase.
!
! ROUTINES:
!   rhd_baserad_BaseAxisSystem: Generate random or unity basic orthonormal axis system
!   rhd_baserad_RayDirections: Create list of basic direction vectors
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The pointers ray_dir0 and ray_dir are assumed to be initialized.
!   They are allocated and have to be deallocated properly by the user.
!
! PROCEDURE:
!   An array with 3xN elements is allocated and filled with vector components
!   according to radraystar.
!   Each vector has length unity.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-08-02 (B.F.) Put into own routine
!   2003-03-07 (B.F.) radraybase = 'alternate'
!   2005-03-08 (B.F.) 'list-01(1)'
!   2007-03-02 (B.F.) 'list-03'
!   2013-05-02 (B.F.) 'list-04'
!   2013-12-04 (B.F.) Get and output ray weight
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: radraystar, radraybase
integer,                   intent(in)   :: nc_m, itime, n_subdtime
integer,                   intent(out)  :: n_dir
real, dimension(:,:), pointer           :: ray_dir0, ray_dir
real, dimension(:),   pointer           :: ray_wei
real, dimension(3,3),      intent(inout):: M_axis
!
! --- Local variables ---
integer                                 :: ilist, irandom
!---------------------------------------------------------------------------------------------------
!
! --- Free existing pointers ---
if (associated(ray_dir0)) deallocate(ray_dir0)
if (associated(ray_dir )) deallocate(ray_dir )
if (associated(ray_wei )) deallocate(ray_wei )
!
if (radraybase == 'alternate') then
  irandom=mod(itime+n_subdtime, 2)
else
  irandom=0
endif
!
! --- Find current base ray directions ---
select case (radraystar)
  case ('list-01(1)')
    ! --- List of 4 ray stars ---
    !
    ! --- Number in list from time step ---
    if (radraybase == 'alternate') then
      ilist=mod( (itime+n_subdtime)/2, 4)
    else
      ilist=mod(  itime+n_subdtime   , 4)
    endif
    !
    ! --- Create base ray star ---
    select case (ilist)
      case (0)
        call rhd_baserad_RayDirections('oktaeder-dd1(1)', ray_dir0, ray_wei)
      case (1)
        call rhd_baserad_RayDirections('oktaeder-d1d(1)', ray_dir0, ray_wei)
      case (2)
        call rhd_baserad_RayDirections('oktaeder-1dd'   , ray_dir0, ray_wei)
      case default
        call rhd_baserad_RayDirections('cube-ddd(1)'    , ray_dir0, ray_wei)
    end select
    !
  case ('list-01(2)')
    ! --- List of 4 ray stars ---
    !
    ! --- Number in list from time step ---
    if (radraybase == 'alternate') then
      ilist=mod( (itime+n_subdtime)/2, 4)
    else
      ilist=mod(  itime+n_subdtime   , 4)
    endif
    !
    ! --- Create base ray star ---
    select case (ilist)
      case (0)
        call rhd_baserad_RayDirections('oktaeder-dd1'   , ray_dir0, ray_wei)
      case (1)
        call rhd_baserad_RayDirections('oktaeder-d1d'   , ray_dir0, ray_wei)
      case (2)
        call rhd_baserad_RayDirections('oktaeder-1dd(2)', ray_dir0, ray_wei)
      case default
        call rhd_baserad_RayDirections('cube-ddd(2)'    , ray_dir0, ray_wei)
    end select
    !
  case ('list-01', 'list-01(3)')
    ! --- List of 4 ray stars ---
    !
    ! --- Number in list from time step ---
    if (radraybase == 'alternate') then
      ilist=mod( (itime+n_subdtime)/2, 4)
    else
      ilist=mod(  itime+n_subdtime   , 4)
    endif
    !
    ! --- Create base ray star ---
    select case (ilist)
      case (0)
        call rhd_baserad_RayDirections('oktaeder-dd1'   , ray_dir0, ray_wei)
      case (1)
        call rhd_baserad_RayDirections('oktaeder-d1d'   , ray_dir0, ray_wei)
      case (2)
        call rhd_baserad_RayDirections('oktaeder-1dd(3)', ray_dir0, ray_wei)
      case default
        call rhd_baserad_RayDirections('cube-ddd(3)'    , ray_dir0, ray_wei)
    end select
    !
  case ('list-02', 'list-02(3)')
    ! --- List of 5 ray stars ---
    !
    ! --- Number in list from time step ---
    if (radraybase == 'alternate') then
      ilist=mod( (itime+n_subdtime)/2, 5)
    else
      ilist=mod(  itime+n_subdtime   , 5)
    endif
    !
    ! --- Create base ray star ---
    select case (ilist)
      case (0)
        call rhd_baserad_RayDirections('duooktaeder-11x', ray_dir0, ray_wei)
      case (1)
        call rhd_baserad_RayDirections('cube-ddd(3)'    , ray_dir0, ray_wei)
      case (2)
        call rhd_baserad_RayDirections('duooktaeder-1x1', ray_dir0, ray_wei)
      case (3)
        call rhd_baserad_RayDirections('cube-ddd(3)'    , ray_dir0, ray_wei)
      case default
        call rhd_baserad_RayDirections('duooktaeder-x11', ray_dir0, ray_wei)
    end select
    !
  case ('list-03', 'list-03(3)')
    ! --- List of 3 ray stars ---
    !
    ! --- Number in list from time step ---
    if (radraybase == 'alternate') then
      ilist=mod( (itime+n_subdtime)/2, 3)
    else
      ilist=mod(  itime+n_subdtime   , 3)
    endif
    !
    ! --- Create base ray star ---
    select case (ilist)
      case (0)
        call rhd_baserad_RayDirections('oktaeder-dd1'   , ray_dir0, ray_wei)
      case (1)
        call rhd_baserad_RayDirections('oktaeder-d1d'   , ray_dir0, ray_wei)
      case default
        call rhd_baserad_RayDirections('oktaeder-1dd(3)', ray_dir0, ray_wei)
    end select
    !
  case ('list-04', 'list-04(3)')
    ! --- List of 6 ray stars ---
    !
    ! --- Number in list from time step ---
    if (radraybase == 'alternate') then
      ilist=mod( (itime+n_subdtime)/2, 6)
    else
      ilist=mod(  itime+n_subdtime   , 6)
    endif
    !
    ! --- Create base ray star ---
    select case (ilist)
      case (0)
        call rhd_baserad_RayDirections('oktaeder-dd1'   , ray_dir0, ray_wei)
      case (1)
        call rhd_baserad_RayDirections('cube-ddd(3)'    , ray_dir0, ray_wei)
      case (2)
        call rhd_baserad_RayDirections('oktaeder-d1d'   , ray_dir0, ray_wei)
      case (3)
        call rhd_baserad_RayDirections('cube-ddd(3)'    , ray_dir0, ray_wei)
      case (4)
        call rhd_baserad_RayDirections('oktaeder-1dd(3)', ray_dir0, ray_wei)
      case default
        call rhd_baserad_RayDirections('cube-ddd(3)'    , ray_dir0, ray_wei)
    end select
    !
  case default
    ! --- Always the same ray star ---
    !
    ! --- Create base ray star ---
    call rhd_baserad_RayDirections(radraystar, ray_dir0, ray_wei)
    !
end select
!
! --- Number of directions ---
n_dir=size(ray_dir0,2)
!
allocate(ray_dir(3,n_dir))
!
! --- Create base axis system (possibly different for all sub time steps) ---
if ((n_subdtime <= 1           ) .or. &
    (radraybase == 'random'    ) .or. &
    (radraybase == 'randomcube') .or. &
    (radraybase == 'alternate' )       ) then
  if (radraybase == 'alternate') then
    if (irandom == 1) then
      call rhd_baserad_BaseAxisSystem('random', nc_m, itime, M_axis)
    else
      call rhd_baserad_BaseAxisSystem('unity' , nc_m, itime, M_axis)
    endif
  else
    call rhd_baserad_BaseAxisSystem(radraybase, nc_m, itime, M_axis)
  endif
endif
!
! --- Determine resulting ray directions ---
ray_dir=matmul(M_axis, ray_dir0)
!
!!$! --- Print some statistics about the ray system ---
!!$write(nc_p,'(2A)') 'radraystar: ', radraystar
!!$write(nc_p,*) 'n_dir:   ', n_dir
!!$!
!!$write(nc_p,*) 'M_axis:  ', shape(M_axis)
!!$do i=1,3
!!$  write(nc_p,'(3(E13.6,1X))') M_axis(:,i)
!!$end do ! i
!!$write(nc_p,*)
!!$!
!!$write(nc_p,*) 'ray_dir0:', shape(ray_dir0)
!!$do i=1,n_dir
!!$  write(nc_p,'(3(E13.6,1X))') ray_dir0(:,i)
!!$end do ! i
!!$write(nc_p,*)
!!$!
!!$write(nc_p,*) 'ray_dir: ', shape(ray_dir)
!!$do i=1,n_dir
!!$  write(nc_p,'(3(E13.6,1X))') ray_dir(:,i)
!!$end do ! i
!!$write(nc_p,*)
!!$!
!!$write(nc_p,*) 'ray_wei: ', shape(ray_wei)
!!$do i=1,n_dir
!!$  write(nc_p,'(E13.6)') ray_wei(i)
!!$end do ! i
!!$write(nc_p,*)
!!$!
!!$write(nc_p,*) 'Projection weights:', sum((ray_dir)**2,2)
!!$write(nc_p,*) 'Projection weights:', sum(ray_dir(:,1)**2), sum(ray_dir(:,2)**2), sum(ray_dir(:,3)**2)
!!$write(nc_p,*) 'Projection weights:', sum(ray_wei*ray_dir(:,1)**2), &
!!$                                     sum(ray_wei*ray_dir(:,2)**2), &
!!$                                     sum(ray_wei*ray_dir(:,3)**2)
!!$write(nc_p,*)
!
end subroutine rhd_baserad_RaySystem


!----------*********************--------------------------------------------------------------------
subroutine rhd_baserad_PipeIndex(nRay, nChunk, iJob, iChunk, &
                                 iRay, nJob, Up_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_baserad_PipeIndex ('rhd_base_radiation_Pipe_Index')
!
! PURPOSE:
!   Compute the ray index for job pipelining.
!
! CATEGORY:
!   Radiation transport, parallelization
!
! CALLING SEQUENCE:
!   call rhd_baserad_PipeIndex(nRay, nChunk, iJob, iChunk, &
!                              iRay, nJob, Up_flag)
!
! INPUT:
!   nRay:         ('number_Ray') integer with total number of rays (or integrations);
!                 up and down counts as one
!   nChunk:       ('number_Chunk') integer with number of chunks in current integration direction
!   iJob:         ('index_Job') integer with number of current job (or step in the pipeline),
!                 starting with 1
!   iChunk:       ('index_Chunk') integer with number of current chunk (1 <= iChunk <= nChunk)
!
! OUTPUT:
!   iRay:         ('index_Ray') integer with index of current ray;
!                   0               : nothing to do (or error in range specification)
!                   1 <= iRay <=nRay: normal range
!   nJob:         ('number_Job') integer with total number of jobs (or steps in the pipeline)
!   Up_flag:      ('Upward_flag') logical indicating if integration direction is upward
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   1 <= nRay
!   1 <= nChunk
!   1 <= iJob   <= nJob
!   1 <= iChunk <= nChunk
!
! PROCEDURE:
!   The integration interval is split in nChunk (>=1) chunks (or processes).
!   Integration can start from either side of the interval, even at the same time.
!   One process (or chunk) can handle only one integration job and direction at the time.
!
! EXAMPLE:
!   nChunk=1 (no parallelization)     : nJob=2*nRay
!   nRay=1 (just one ray), nChunk even: nJob=nChunk
!   nRay=1 (just one ray), nChunk odd : nJob=nChunk+1
!
! MODIFICATION HISTORY:
!   2016-04-16 (B.F. Uppsala) Written
!   2017-07-03 (B.F.) Correct typos, add comments
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nRay, nChunk, iJob, iChunk
integer,                   intent(out)  :: iRay, nJob
logical,                   intent(out)  :: Up_flag
!
! --- Local variables ---
integer                                 :: i0, i1
!---------------------------------------------------------------------------------------------------
!
! --- Compute necessary number of jobs ---
nJob=nChunk + 2*(nRay-1) + mod(nChunk,2)
!
! --- Determine direction (up or down) ---
i0     =iJob-iChunk
Up_flag=(mod(i0, 2) == 0)
!
if (Up_flag) then
  ! --- Compute ray number for upward integration ---
  if (i0 >=0) then
    iRay=max(i0/2 + 1, 0)
  else
    iRay=0
  endif
  !
else ! (.not. Up_flag)
  ! --- Compute ray number for downward integration ---
  i1  =iJob - (nChunk-iChunk) + mod(nChunk,2)
  if (i1 >=0) then
    iRay=max(i1/2 + 1, 0) - mod(nChunk,2)
  else
    iRay=0
  endif
  !
endif ! (Up_flag)
!
! --- Check if ray index is too large ---
if (iRay   >   nRay) iRay=0
if (iJob   <      1) iRay=0
if (iJob   >   nJob) iRay=0
if (iChunk <      1) iRay=0
if (iChunk > nChunk) iRay=0
!
end subroutine rhd_baserad_PipeIndex

end module rhd_baserad_module
