!***************************************************************************************************
!
!   #####   #    #  #####           ##     ####   #####  #    ####   #    #
!   #    #  #    #  #    #         #  #   #    #    #    #   #    #  ##   #
!   #    #  ######  #    #        #    #  #         #    #   #    #  # #  #
!   #####   #    #  #    #        ######  #         #    #   #    #  #  # #
!   #   #   #    #  #    #        #    #  #    #    #    #   #    #  #   ##
!   #    #  #    #  #####  ###### #    #   ####     #    #    ####   #    # _module
!
!***************************************************************************************************
!   Fortran90
!   2017-12-07
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!   Werner Schaffenberger: Potsdam, Exeter
!   Matthias Steffen:      Potsdam
!***************************************************************************************************
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
!***************************************************************************************************


!------*****************----------------------------------------------------------------------------
module rhd_action_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_action_module ('rhd_action_module')
!
! PURPOSE:
!   Routines for rhd to handle the parameter structure 'action'.
!
! CATEGORY:
!   Parameter handling
!
! CALLING SEQUENCE:
!   use rhd_action_module
!
! TYPES:
!   action_type:  ('action_type') Structure for basic control parameters
!     bound_l1:        ('boundary_left_1') integer, left boundary type for dimension 1;
!                      values:
!                        bound_None
!                        bound_reflective
!                        bound_periodic
!                        bound_constant
!                        bound_closed
!                        bound_closed2
!                        bound_inoutflow
!                        bound_inoutflow2
!                        bound_inoutflow3
!                        bound_closedbottom
!                        bound_closedbottom2
!                        bound_transmitting
!                        bound_transmitting2
!                        bound_transmitting3
!                        bound_InternalNext
!                        bound_InternalPeriodic
!                        bound_AnyOpen
!     bound_l2:        ('boundary_left_2') integer, left boundary type for dimension 2;
!                      values: see bound_l1
!     bound_l3:        ('boundary_left_3') integer, left boundary type for dimension 3;
!                      values: see bound_l1
!     bound_r1:        ('boundary_right_1') integer, right boundary type for dimension 1;
!                      values: see bound_l1
!     bound_r2:        ('boundary_right_2') integer, right boundary type for dimension 2;
!                      values: see bound_l1
!     bound_r3:        ('boundary_right_3') integer, right boundary type for dimension 3;
!                      values: see bound_l1
!     bound_mag_l1,2,3:('boundary_magnetic_left_1,2,3') integer, left boundary type of the
!                      magnetic field for dimension 1,2,3;
!                      values:
!                        bound_mag_None
!                        bound_mag_periodic
!                        bound_mag_constant
!                        bound_mag_vertical
!                        bound_mag_inoutflow
!                        bound_mag_oblique
!                        bound_mag_fixed
!                        bound_mag_reflective
!                        bound_mag_vertical2
!                        bound_mag_constant2
!                        bound_mag_InternalNext
!                        bound_mag_InternalPeriodic
!     bound_mag_r1,2,3:('boundary_magnetic_right_1,2,3') integer, right boundary type of the
!                      magnetic for dimension 1,2,3;
!                      values: see bound_mag_l1
!     dim:             ('dimension') integer, dimension the solution is to be advanced by 1D step
!     dimV:            ('dimension_Vector') 3-element integer vector with original dimensions
!     C_???:           see rhd_par_RdData in rhd_io_module
!     N_???:           see rhd_par_RdData in rhd_io_module
!     radraybase:      ('radiation_ray_base') character to describe the orientation of the
!                      base axis system:
!                        'unity' : Unity matrix 
!                        'random': Matrix with random orientation
!     radraystar:      ('radiation_ray_star') character to describe the list of the ray directions
!                      (e.g. the number of rays) relative to the base axis system:
!                        'x1'        : Only rays along x1 axis (1 direction)
!                        'x2'        : Only rays along x2 axis (1 direction)
!                        'x3'        : Only rays along x3 axis (1 direction)
!                        'oktaeder'  : Oktaeder: rays along x1, x2, and x3 axis (3 directions)
!                        'tetraeder' : Tetraeder: rays in 4 directions
!                        'cube'      : Cube: rays in 4 directions
!                        'ikosaeder' : Ikosaeder: rays in 6 directions
!                        'dodekaeder': Dodekaeder: rays in 10 directions
!
! VARIABLES:
!   Some constants used as names in CASE distinctions. For an explanation see 'TYPES:' above.
!
! ROUTINES: (contained)
!   rhd_action_StrToRecon: Analyze string with reconstruction scheme
!   rhd_action_InitNameAr: Initialize global name arrays
!   rhd_action_New:        Function: Change contents of action variable
!
! MODIFICATION HISTORY:
!   1995-05-28 (B.F. Copenhagen) Written
!   1999-05-31 (B.F.) Parameters C_visCourant, C_visCourantMax added
!   1999-12-27 (B.F. Kiel) Constants for boundary conditions...
!   2001-02-12 (B.F. Uppsala) Different numbers for all reconstruction types
!   2001-11-15 (B.F.) radScheme_shortrad=3 added
!   2002-11-13 (B.F.) bound_transmitting2
!   2003-01-22 (B.F.) C_SlopeRed
!   2004-05-13 (B.F. Montpellier)
!   2006-09-12 (W.S. Freiburg) New boundary condition bound_inoutflow2 added
!   2007-01-18 (B.F. Lyon) reconstruction_quctype
!   2008-04-27 (B.F.) Parameters C_vis* removed
!   2008-06-10 (W.S. Freiburg) Magnetic boundary conditions and new hdScheme HLLMHD added
!   2008-08-30 (B.F.) Removed from action structure: radpressure, N_radtaurefine, N_radThinPoint,
!              N_radThickPoint, N_radtheta, N_radphi, N_radsubray, N_radband, C_SlopeRed,
!              C_raditereps, C_raditerstep, C_radkappasmooth, C_radTstep, C_radTsmooth,
!              C_radTintminfac
!   2008-10-22 (W.S. Lansing) New magnetic boundary condition bound_mag_oblique
!   2008-11-30 (W.S. Lansing) Removed from action structure: C_resB, C_resEpsilon
!   2009-01-08 (M.S.) Merged for use with latest version of MHD-module
!   2010-02-18 (W.S. Freiburg) New magnetic boundary conditions bound_mag_fixed and 
!                              bound_mag_reflective
!   2010-04-28 (B.F. Lyon) bound_transmitting3
!   2010-06-12 (B.F.) bound_inoutflow2, bound_closedbottom2
!   2010-06-30 (B.F.) hdScheme_CenMHD
!   2011-03-28 (W.S.) bound_mag_vertical2
!   2011-04-27 (B.F.) reconstruction_type_PPL, reconstruction_type_PPLAW
!   2011-04-28 (B.F.) bound_mag_constant2
!   2011-07-07 (B.F.) reconstruction_type_FRmo, reconstruction_type_FRen
!   2011-09-05 (B.F.) reconstruction_type_FRce
!   2011-09-08 (B.F.) Move action%radScheme to par%A%radScheme
!   2011-11-18 (B.F.) Remove PPAW scheme
!   2011-11-21 (B.F.) Reconstruction schemes: PPL->PPmimo, FRmo->FRmono, FRen->FRweno, FRce->FRcont
!   2011-11-15 (B.F.) bound_inoutflow3
!   2012-03-21 (B.F.) hdScheme_Roe -> hdScheme_RoeWave
!   2012-07-30 (B.F.) Some values removed from action structure (left in par or put into par%A)
!   2012-09-05 (B.F.) Internal boundary conditions
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2014-08-19 (B.F. Uppsala) HBweno, VAweno
!   2014-09-21 (B.F.) bound_closed2
!   2014-12-05 (B.F.) New tag of action structure: dimV
!   2016-04-28 (B.F. Uppsala) Outcomment definition of C_radmaxeiChange, C_radimplicitmu
!   2016-05-13 (B.F.) Remove Ziegler scheme (was for testing only); rename bound_no to bound_None
!   2016-05-14 (B.F.) Add global variable NameAr_reconstruction_type
!   2017-11-05 (B.F.) Add option radscheme_MCrad
!   2017-12-01 (M.S.) Rename options radscheme_**rad
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- Action data type ---
!
type action_type
  integer                               :: dim
  integer, dimension(3)                 :: dimV
  integer                               :: bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3
  integer                               :: bound_mag_l1, bound_mag_r1, &
                                           bound_mag_l2, bound_mag_r2, &
                                           bound_mag_l3, bound_mag_r3
  integer                               :: N_radminiter, N_raditer, N_radmaxiter
!  real                                  :: C_radmaxeiChange, C_radimplicitmu
  character(len=80)                     :: radraybase, radraystar
end type action_type
!
integer, parameter                      :: hdScheme_First    =0, &
                                           hdScheme_None     =0, &
                                           hdScheme_RoeWave  =1, &
                                           hdScheme_RoeState =2, &
                                           hdScheme_RoeMagKin=3, &
                                           hdScheme_RoeMHD   =4, &
                                           hdScheme_HLLMHD   =5, &
                                           hdScheme_CenMHD   =6, &
                                           hdScheme_Last     =6
integer, parameter                      :: radScheme_First   =0, &
                                           radScheme_None    =0, &
                                           radScheme_LHDrad  =1, &
                                           radScheme_MSCrad  =2, &
                                           radScheme_LCFrad  =3, &
                                           radScheme_SHORTrad=4, &
                                           radScheme_Last    =4
integer, parameter                      :: reconstruction_type_First   = 0, &
                                           reconstruction_type_Constant= 0, &
                                           reconstruction_type_Minmod  = 1, &
                                           reconstruction_type_VanLeer = 2, &
                                           reconstruction_type_Superbee= 3, &
                                           reconstruction_type_PP      = 4, &
                                           reconstruction_type_PPmimo  = 5, &
                                           reconstruction_type_FRmimo  = 6, &
                                           reconstruction_type_FRmono  = 7, &
                                           reconstruction_type_FRmono2 = 8, &
                                           reconstruction_type_FRweno  = 9, &
                                           reconstruction_type_FRweno2 =10, &
                                           reconstruction_type_FRcont  =11, &
                                           reconstruction_type_VAweno  =12, &
                                           reconstruction_type_HBweno  =13, &
                                           reconstruction_type_Last    =13
integer, parameter                      :: bound_First           =-1, &
                                           bound_None            =-1, &
                                           bound_reflective      = 0, &
                                           bound_periodic        = 1, &
                                           bound_constant        = 2, &
                                           bound_closed          = 3, &
                                           bound_closed2         = 4, &
                                           bound_inoutflow       = 5, &
                                           bound_inoutflow2      = 6, &
                                           bound_inoutflow3      = 7, &
                                           bound_closedbottom    = 8, &
                                           bound_closedbottom2   = 9, &
                                           bound_transmitting    =10, &
                                           bound_transmitting2   =11, &
                                           bound_transmitting3   =12, &
                                           bound_InternalNext    =13, &
                                           bound_InternalPeriodic=14, &
                                           bound_Last            =14
integer, parameter                      :: bound_mag_First           = 0, &
                                           bound_mag_None            = 0, &
                                           bound_mag_reflective      = 1, &
                                           bound_mag_periodic        = 2, &
                                           bound_mag_constant        = 3, &
                                           bound_mag_constant2       = 4, &
                                           bound_mag_inoutflow       = 5, &
                                           bound_mag_vertical        = 6, &
                                           bound_mag_vertical2       = 7, &
                                           bound_mag_oblique         = 8, &
                                           bound_mag_fixed           = 9, &
                                           bound_mag_InternalNext    =10, &
                                           bound_mag_InternalPeriodic=11, &
                                           bound_mag_Last            =11
character(len=20), save, &
  dimension(hdScheme_First: &
            hdScheme_Last    )          :: NameAr_hdScheme
character(len=20), save, &
  dimension(radScheme_First: &
            radScheme_Last    )         :: NameAr_radScheme
character(len=20), save, &
  dimension(reconstruction_type_First: &
            reconstruction_type_Last  ) :: NameAr_reconstruction_type
character(len=20), save, &
  dimension(bound_First: &
            bound_Last    )             :: NameAr_bound
character(len=20), save, &
  dimension(bound_mag_First: &
            bound_mag_Last    )         :: NameAr_bound_mag
!
contains

!----------*********************--------------------------------------------------------------------
subroutine rhd_action_InitNameAr(nc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_action_InitNameAr ('rhd_action_Initialize_Name_Arrays')
!
! PURPOSE:
!   Initialize global name arrays.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_action_InitNameAr()
!
! INPUT:
!   nc:           ('number_channel') integer, number of channel unit for screen output;
!                   a negative value suppresses output
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables in the local module are modified.
!   Data may be printed on channel nc.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_action_InitNameAr()
!
! MODIFICATION HISTORY:
!   2016-05-14 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc
!
! --- Local variables ---
integer                                 :: i
!---------------------------------------------------------------------------------------------------
!
! --- Fill global array with hydro-scheme names ---
NameAr_hdScheme(:)                 ='*** Undefined ***'
NameAr_hdScheme(hdScheme_None     )='None'
NameAr_hdScheme(hdScheme_RoeWave  )='Roe'
NameAr_hdScheme(hdScheme_RoeState )='RoeState'
NameAr_hdScheme(hdScheme_RoeMagKin)='RoeMagKin'
NameAr_hdScheme(hdScheme_RoeMHD   )='RoeMHD'
NameAr_hdScheme(hdScheme_HLLMHD   )='HLLMHD'
NameAr_hdScheme(hdScheme_CenMHD   )='CenMHD'
!
! --- Fill global array with radiation-transport-scheme names ---
NameAr_radScheme(:)                 ='*** Undefined ***'
NameAr_radScheme(radScheme_None    )='None'
NameAr_radScheme(radScheme_LHDrad  )='LHDrad'
NameAr_radScheme(radScheme_MSCrad  )='MSCrad'
NameAr_radScheme(radScheme_LCFrad  )='LCFrad'
NameAr_radScheme(radScheme_SHORTrad)='SHORTrad'
!
! --- Fill global array with reconstruction names ---
NameAr_reconstruction_type(:)                           ='*** Undefined ***'
NameAr_reconstruction_type(reconstruction_type_Constant)='Constant'
NameAr_reconstruction_type(reconstruction_type_Minmod  )='MinMod'
NameAr_reconstruction_type(reconstruction_type_VanLeer )='VanLeer'
NameAr_reconstruction_type(reconstruction_type_Superbee)='SuperBee'
NameAr_reconstruction_type(reconstruction_type_PP      )='PP'
NameAr_reconstruction_type(reconstruction_type_PPmimo  )='PPmimo'
NameAr_reconstruction_type(reconstruction_type_FRmimo  )='FRmimo'
NameAr_reconstruction_type(reconstruction_type_FRmono  )='FRmono'
NameAr_reconstruction_type(reconstruction_type_FRmono2 )='FRmono2'
NameAr_reconstruction_type(reconstruction_type_FRweno  )='FRweno'
NameAr_reconstruction_type(reconstruction_type_FRweno2 )='FRweno2'
NameAr_reconstruction_type(reconstruction_type_FRcont  )='FRcont'
NameAr_reconstruction_type(reconstruction_type_VAweno  )='VAweno'
NameAr_reconstruction_type(reconstruction_type_HBweno  )='HBweno'
!
! --- Fill global array with boundary-condition names ---
NameAr_bound(:)                     ='*** Undefined ***'
NameAr_bound(bound_None            )='None'
NameAr_bound(bound_reflective      )='reflective'
NameAr_bound(bound_periodic        )='periodic'
NameAr_bound(bound_constant        )='constant'
NameAr_bound(bound_closed          )='closed'
NameAr_bound(bound_closed2         )='closed2'
NameAr_bound(bound_inoutflow       )='inoutflow'
NameAr_bound(bound_inoutflow2      )='inoutflow2'
NameAr_bound(bound_inoutflow3      )='inoutflow3'
NameAr_bound(bound_closedbottom    )='closedbottom'
NameAr_bound(bound_closedbottom2   )='closedbottom2'
NameAr_bound(bound_transmitting    )='transmitting'
NameAr_bound(bound_transmitting2   )='transmitting2'
NameAr_bound(bound_transmitting3   )='transmitting3'
NameAr_bound(bound_InternalNext    )='InternalNext'
NameAr_bound(bound_InternalPeriodic)='InternalPeriodic'
!
! --- Fill global array with magnetic boundary-condition names ---
NameAr_bound_mag(:)                         ='*** Undefined ***'
NameAr_bound_mag(bound_mag_None            )='None'
NameAr_bound_mag(bound_mag_reflective      )='reflective'
NameAr_bound_mag(bound_mag_periodic        )='periodic'
NameAr_bound_mag(bound_mag_constant        )='constant'
NameAr_bound_mag(bound_mag_constant2       )='constant2'
NameAr_bound_mag(bound_mag_inoutflow       )='inoutflow'
NameAr_bound_mag(bound_mag_vertical        )='vertical'
NameAr_bound_mag(bound_mag_vertical2       )='vertical2'
NameAr_bound_mag(bound_mag_oblique         )='oblique'
NameAr_bound_mag(bound_mag_fixed           )='fixed'
NameAr_bound_mag(bound_mag_InternalNext    )='InternalNext'
NameAr_bound_mag(bound_mag_InternalPeriodic)='InternalPeriodic'
!
if (nc >= 0) then
  ! --- Print contents of name arrays ---
  !
  write(nc, '(A)') 'hdScheme:'
  do i=hdScheme_First,hdScheme_Last
    write(nc, '(I3,2X,A)') i, trim(NameAr_hdScheme(i))
  end do ! i
  write(nc, '(A)') ''
  !
  write(nc, '(A)') 'radScheme:'
  do i=radScheme_First,radScheme_Last
    write(nc, '(I3,2X,A)') i, trim(NameAr_radScheme(i))
  end do ! i
  write(nc, '(A)') ''
  !
  write(nc, '(A)') 'Reconstruction type:'
  do i=reconstruction_type_First,reconstruction_type_Last
    write(nc, '(I3,2X,A)') i, trim(NameAr_reconstruction_type(i))
  end do ! i
  write(nc, '(A)') ''
  !
  write(nc, '(A)') 'Boundary conditions:'
  do i=bound_First,bound_Last
    write(nc, '(I3,2X,A)') i, trim(NameAr_bound(i))
  end do ! i
  write(nc, '(A)') ''
  !
  write(nc, '(A)') 'Magnetic boundary conditions:'
  do i=bound_mag_First,bound_mag_Last
    write(nc, '(I3,2X,A)') i, trim(NameAr_bound_mag(i))
  end do ! i
  write(nc, '(A)') ''
  !
endif
!
end subroutine rhd_action_InitNameAr


!----------*********************--------------------------------------------------------------------
subroutine rhd_action_StrToRecon(reconstruction, reconstruction_order, reconstruction_type)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_action_StrToRecon ('rhd_action_String_To_Reconstruction')
!
! PURPOSE:
!   Analyze string with reconstruction scheme.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_action_StrToRecon(reconstruction, reconstruction_order, reconstruction_type)
!
! INPUT:
!   reconstruction: ('reconstruction') character with reconstruction scheme
!
! OUTPUT:
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   The routines stops with an error message if the reconstruction is unknown.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2007-01-16 (B.F. Lyon) Taken from rhd_par_Toaction
!   2011-04-27 (B.F.) reconstruction_type_PPL, reconstruction_type_PPLAW;
!                     stop for unknown reconstruction
!   2011-07-07 (B.F.) reconstruction_type_FRmo, reconstruction_type_FRen
!   2011-09-05 (B.F.) reconstruction_type_FRce
!   2015-02-23 (B.F. Uppsala) Moved from rhd_sub_module into rhd_action_module and renamed
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: reconstruction
integer,                   intent(out)  :: reconstruction_order, reconstruction_type
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! \label{f90:reconstruction__proc1}
! --- Check reconstruction method ---
select case (reconstruction)
  ! --- Order 0 ---
  case ('constant', 'Constant')
    reconstruction_order=0
    reconstruction_type =reconstruction_type_Constant
  ! --- Order 1 ---
  case ('superbee', 'Superbee', 'SuperBee', 'SUPERBEE')
    reconstruction_order=1
    reconstruction_type =reconstruction_type_Superbee
  case ('minmod', 'Minmod', 'MinMod', 'MINMOD')
    reconstruction_order=1
    reconstruction_type =reconstruction_type_Minmod
  case ('vanleer', 'vanLeer', 'Vanleer', 'VanLeer', 'VANLEER', 'van Leer', 'Van Leer')
    reconstruction_order=1
    reconstruction_type =reconstruction_type_VanLeer
  ! --- Order 2 ---
  case ('pp', 'Pp', 'PP')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_PP
  case ('ppmimo', 'Ppmimo', 'PPmimo', 'PPMimo', 'PPMiMo', 'PPMIMO')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_PPmimo
  case ('frmimo', 'Frmimo', 'FRmimo', 'FrMimo', 'FrMiMo', 'FrMIMO', 'FRMimo', 'FRMiMo', 'FRMIMO')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_FRmimo
  case ('frmono', 'Frmono', 'FRmono', 'FrMono',           'FrMONO', 'FRMono',           'FRMONO' )
    reconstruction_order=2
    reconstruction_type =reconstruction_type_FRmono
  case ('frmono2', 'Frmono2', 'FRmono2', 'FrMono2',       'FrMONO2','FRMono2',          'FRMONO2')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_FRmono2
  case ('frweno', 'Frweno', 'FRweno', 'FrWeno', 'FrWEno', 'FrWENO', 'FRWeno', 'FRWEno', 'FRWENO')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_FRweno
  case ('frweno2', 'Frweno2', 'FRweno2', 'FrWeno2', 'FrWEno2', 'FrWENO2', 'FRWeno2', 'FRWEno2', &
        'FRWENO2')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_FRweno2
  case ('frcont', 'Frcont', 'FRcont', 'FrCont',           'FRCont',           'FRCONT')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_FRcont
  case ('vaweno', 'Vaweno', 'VAweno', 'VaWeno', 'VaWEno', 'VaWENO', 'VAWeno', 'VAWEno', 'VAWENO')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_VAweno
  case ('hbweno', 'Hbweno', 'HBweno', 'HbWeno', 'HbWEno', 'HbWENO', 'HBWeno', 'HBWEno', 'HBWENO')
    reconstruction_order=2
    reconstruction_type =reconstruction_type_HBweno
  ! --- Default ---
  case default
    reconstruction_order=0
    reconstruction_type =reconstruction_type_Constant
    stop 'Unknown reconstruction method'
end select
!
end subroutine rhd_action_StrToRecon


!--------**************-----------------------------------------------------------------------------
function rhd_action_New(action, &
                        dim, dimV, &
                        bound_l1, bound_r1, &
                        bound_l2, bound_r2, &
                        bound_l3, bound_r3, &
                        bound_mag_l1, bound_mag_r1, &
                        bound_mag_l2, bound_mag_r2, &
                        bound_mag_l3, bound_mag_r3, &
                        radraybase, radraystar)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_action_New ('rhd_action_New')
!
! PURPOSE:
!   Change contents of action variable.
!   
! CATEGORY:
!   Action handling
!
! CALLING SEQUENCE:
!   naction=rhd_action_New(action, &
!                          dim, dimV, &
!                          bound_l1, bound_r1, &
!                          bound_l2, bound_r2, &
!                          bound_l3, bound_r3, &
!                          bound_mag_l1, bound_mag_r1, &
!                          bound_mag_l2, bound_mag_r2, &
!                          bound_mag_l3, bound_mag_r3, &
!                          radraybase, radraystar)
!
! INPUT:
!   action:       ('action') action_type, input structure with rhd control parameters
!   XXX:          list of control parameters, see comments for the module
!
! OUTPUT:
!   rhd_action_New: ('rhd_action_New') action_type, output structure with rhd control parameters
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The value of each optional parameter which is actually present is put into the control
!   parameter structure.
!
! EXAMPLE:
!   naction=rhd_action_New(action, radraybase='unity', radraystar='oktaeder')
!
! MODIFICATION HISTORY:
!   1997-06-19 Written by B.F., Astrophysik Kiel
!   1997-07-24 (B.F.) reconstruction_order, reconstruction_type
!   1997-10-03 (B.F.) Parameters C_Courant, C_CourantMax, C_Pderiv added
!   1997-10-24 (B.F.) C_interaction added
!   1997-11-14 (B.F.) C_visNeu1, C_visNeu2 added
!   1999-04-19 (B.F.) C_visDrag added
!   1999-05-18 (B.F.) Parameters C_radCourant, C_radCourantMax added
!   1999-05-19 (B.F.) Parameter C_radmaxeiChange added
!   1999-05-27 (B.F.) Parameters for artificial viscosity tensor added
!   1999-05-31 (B.F.) Parameters C_visCourant, C_visCourantMax added
!   1999-06-08 (B.F.) Parameter C_maxeiChange added
!   1999-06-20 (B.F. Kiel) Parameter C_visPrturb added
!   1999-06-30 (B.F. Copenhagen) Parameter N_raditer added
!   1999-08-31 (B.F.) Parameters for radiation transport added
!   1999-09-08 (B.F.) Parameter hdScheme added
!   1999-12-08 (B.F.) Parameters radraybase, radraystar added
!   2000-03-21 (B.F.) Parameter C_visBound added
!   2000-04-13 (B.F.) Parameters for Matthias Steffens radiation transport added
!   2002-05-07 (B.F.) Parameter rho_min added
!   2008-04-27 (B.F.) Parameters C_vis* removed
!   2008-06-10 (W.S. Freiburg) parameters for magnetic boundary conditions added
!   2008-11-30 (W.S. Lansing) parameters C_resB, C_resEpsilon removed
!   2010-04-20 (B.F. Lyon) Rename newaction -> rhd_action_New
!   2014-12-05 (B.F.) New parameter dimV
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(action_type)                       :: rhd_action_New
type(action_type),         intent(in)   :: action
integer,         optional, intent(in)   :: dim
integer, dimension(3), &
                 optional, intent(in)   :: dimV
integer,         optional, intent(in)   :: bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3
integer,         optional, intent(in)   :: bound_mag_l1, bound_mag_r1, &
                                           bound_mag_l2, bound_mag_r2, &
                                           bound_mag_l3, bound_mag_r3
character(len=*),optional, intent(in)   :: radraybase, radraystar
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
rhd_action_New=action
!
if (present(dim                   )) rhd_action_New%dim                   =dim
if (present(dimV                  )) rhd_action_New%dimV                  =dimV
if (present(bound_l1              )) rhd_action_New%bound_l1              =bound_l1
if (present(bound_r1              )) rhd_action_New%bound_r1              =bound_r1
if (present(bound_l2              )) rhd_action_New%bound_l2              =bound_l2
if (present(bound_r2              )) rhd_action_New%bound_r2              =bound_r2
if (present(bound_l3              )) rhd_action_New%bound_l3              =bound_l3
if (present(bound_r3              )) rhd_action_New%bound_r3              =bound_r3
if (present(bound_mag_l1          )) rhd_action_New%bound_mag_l1          =bound_mag_l1
if (present(bound_mag_r1          )) rhd_action_New%bound_mag_r1          =bound_mag_r1
if (present(bound_mag_l2          )) rhd_action_New%bound_mag_l2          =bound_mag_l2
if (present(bound_mag_r2          )) rhd_action_New%bound_mag_r2          =bound_mag_r2
if (present(bound_mag_l3          )) rhd_action_New%bound_mag_l3          =bound_mag_l3
if (present(bound_mag_r3          )) rhd_action_New%bound_mag_r3          =bound_mag_r3
if (present(radraybase            )) rhd_action_New%radraybase            =radraybase
if (present(radraystar            )) rhd_action_New%radraystar            =radraystar
!
end function rhd_action_New

end module rhd_action_module
