!***************************************************************************************************
!
!                                                     #     # #######
!   #####  #    # #####          #####   ####  #    # #     #    #
!   #    # #    # #    #         #    # #    #  #  #  #     #    #
!   #    # ###### #    #         #####  #    #   ##   #     #    #
!   #####  #    # #    #         #    # #    #   ##   #     #    #
!   #   #  #    # #    #         #    # #    #  #  #  #     #    #
!   #    # #    # #####  ####### #####   ####  #    #  #####     #  _module
!
!   Radiation-Hydrodynamics Code: unit-test routines for box-handling routines
!
!***************************************************************************************************
!   Fortran90
!   2019-03-06
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!***************************************************************************************************
! MODULES:
!   rhd_boxUT_module:      Unit-test routines for box-handling routines
!
!***************************************************************************************************


!------****************-----------------------------------------------------------------------------
module rhd_boxUT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_boxUT_module ('rhd_box_Unit_Test_module')
!
! PURPOSE:
!   Provide unit-test routines for box-handling routines.
!
! CATEGORY:
!   Testing, box handling
!
! CALLING SEQUENCE:
!   use rhd_boxUT_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_boxUT_Test:        Do unit-test checks on arrays in box structure
!   rhd_box_ChunkSizeOneUT: Perform unit test for rhd_box_ChunkSize
!   rhd_box_ChunkSizeUT:   Perform unit tests for rhd_box_ChunkSize
!   rhd_box_RestIndicesOneUT: Perform unit test for rhd_box_RestIndices
!   rhd_box_RestIndicesUT: Perform unit tests for rhd_box_RestIndices
!   rhd_bound_1DGhostOneUT: Perform unit test for rhd_bound_1DGhost.
!   rhd_bound_1DGhostUT:   Perform unit tests for rhd_bound_1DGhost.
!   rhd_boxUT_TestAll:     Perform all unit tests of box-handling routines
!   rhd_boxUT_TestSome:    Perform some unit tests of box-handling routines
!
! CALLING TREE:
!   rhd_boxUT_TestAll
!     | 
!     * rhd_box_ChunkSizeUT
!     | + rhd_box_ChunkSizeOneUT
!     |   * rhd_box_ChunkSize
!     |   * UT_Assert
!     | 
!     * rhd_box_RestIndicesUT
!     | + rhd_box_RestIndicesOneUT
!     |   * rhd_box_RestIndices
!     |   * UT_Assert
!     | 
!     * rhd_bound_1DGhostUT
!       + rhd_bound_1DGhostOneUT
!         * rhd_box_Make
!         * rhd_box_FillTestOMP
!         * rhd_bound_1DGhost
!         * rhd_boxUT_Test
!         | + UT_Assert
!         * rhd_box_Delete
!
! MODIFICATION HISTORY:
!   2016-04-18 (B.F. Uppsala) First version; test rhd_box_RestIndices
!   2016-04-23 (B.F.) Test rhd_box_ChunkSize
!   2016-04-28 (B.F.) Test rhd_bound_1DGhost
!   2017-08-26 (B.F. Lyon) Add routine for partial unit tests
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------**************---------------------------------------------------------------------------
subroutine rhd_boxUT_Test(TestPrefix, box, FillValues, dx)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_boxUT_Test ('rhd_box_Unit_Test_Test')
!
! PURPOSE:
!   Do unit-test checks on arrays in box structure.
!
! CATEGORY:
!   Box handling, testing
!
! CALLING SEQUENCE:
!   call rhd_boxUT_Test(TestPrefix, nTest, box, FillValues, dx)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nTest:        ('number_Test') integer with test counter
!   box:          ('box') type(box_type), input computational box
!   FillValues:   ('Fill_Values') optional real vector with values the arrays should have been
!                 filled with
!   dx:           ('delta_x) optional real scalar with grid size
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   An array with (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements is allocated (and deallocated),
!   so the number of elements should not be too big.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2016-04-28 (B.F. Uppsala) First version
!   2016-04-29 (B.F.) Check more quantities
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
type(box_type),            intent(in)   :: box
real, dimension(:), &
                 optional, intent(in)   :: FillValues
real,            optional, intent(in)   :: dx
!
! --- Local variables ---
real                                    :: delta
character(len=128)                      :: TestName
character(len=20)                       :: qID

integer                                 :: i_chunk, n_chunks, i1, i2, i3
integer, dimension(:    ), allocatable  :: Vola
integer, dimension(:,:,:), allocatable  :: ia
logical                                 :: Ignore_flag
!---------------------------------------------------------------------------------------------------
!
! --- Set the maximum allowed error for comparison (assuming all values are of order 1.0)
delta=1.0E-04
!
!
! === Check axes =======================
!
! --------------------------------------
qID='xc1'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%xc1, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%xc1, 0.0, delta=delta)
!
! --------------------------------------
qID='xb1'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%xb1, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%xb1, 0.0, delta=delta)
!
! --------------------------------------
qID='xc2'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%xc2, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%xc2, 0.0, delta=delta)
!
! --------------------------------------
qID='xb2'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%xb2, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%xb2, 0.0, delta=delta)
!
! --------------------------------------
qID='xc3'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%xc3, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%xc3, 0.0, delta=delta)
!
! --------------------------------------
qID='xb3'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%xb3, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%xb3, 0.0, delta=delta)
!
!
! === Check HD bulk quantities =========
!
! --------------------------------------
qID='rho'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%rho, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%rho, 0.0, delta=delta)
!
! --------------------------------------
qID='ei'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%ei, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%ei, 0.0, delta=delta)
!
! --------------------------------------
qID='v1'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%v1, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%v1, 0.0, delta=delta)
!
! --------------------------------------
qID='v2'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%v2, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%v2, 0.0, delta=delta)
!
! --------------------------------------
qID='v3'
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsLargerThanZero'
!
call UT_Assert('>=', TestName, box%v3, 0.0, delta=delta)
!
TestName=trim(TestPrefix) // '_BoxFill' // '_' // &
         trim(qID) // '_' // &
         'DeltaIsSmallerThanZero'
!
call UT_Assert('<=', TestName, box%v3, 0.0, delta=delta)
!


!
end subroutine rhd_boxUT_Test


!----------**********************-------------------------------------------------------------------
subroutine rhd_box_ChunkSizeOneUT(TestPrefix, nTest, &
                                  edge_mode, n_elements_per_chunk, m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_ChunkSizeOneUT ('rhd_box_Chunk_Size_One_Unit_Test')
!
! PURPOSE:
!   Perform unit test for rhd_box_ChunkSize.
!
! CATEGORY:
!   Box handling, index arithmetic, testing
!
! CALLING SEQUENCE:
!   call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, &
!                               edge_mode, n_elements_per_chunk, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nTest:        ('number_Test') integer with test counter
!   edge_mode:    ('edge_mode') integer, mode to describe aspect ratio of chunk:
!                   0: always complete columns (n1), prefer complete slices (n2)
!                   1: always complete columns (n1), prefer n2=n3
!                   2: likely partial columns (n1), prefer n1=n2=n3 (under construction)
!   n_elements_per_chunk: ('number_elements_per_chunk') integer, requested total number of
!                         elements per chunk
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_ChunkSize:   Compute indices of boxes surrounding small box in large box
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   An array with (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements is allocated (and deallocated),
!   so the number of elements should not be too big.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2016-04-23 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nTest, &
                                           edge_mode, &
                                           n_elements_per_chunk, m1,n1, m2,n2, m3,n3
!
! --- Local variables ---

character(len=128)                      :: TestName, TestBase
integer                                 :: i_chunk, n_chunks, i1, i2, i3, &
                                           m1c,n1c, m2c,n2c, m3c,n3c
integer, dimension(:    ), allocatable  :: Vola
integer, dimension(:,:,:), allocatable  :: ia
character(len=20)                       :: nTestStr, TestStr
logical                                 :: Ignore_flag
!---------------------------------------------------------------------------------------------------
!
! --- Prepare strings for test name ----
write(nTestStr, *) nTest
nTestStr=adjustl(nTestStr)
!
TestBase=' '
write(TestStr, *) edge_mode
TestBase=trim(TestBase) //        'EdgeMode=' // adjustl(TestStr) 
write(TestStr, *) n_elements_per_chunk
TestBase=trim(TestBase) // '_' // 'nPerChunk=' // adjustl(TestStr) 
write(TestStr, *) m1
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n1
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) m2
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n2
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) m3
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n3
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
!
if ((m1>n1) .or. (m2>n2) .or. (m3>n3)) then
  ! === The box is empty: check only that all chunks are empty, too ===
  !
  ! --- The routine rhd_box_ChunkSize does not check for a well-specified box: skip tests ---
  !
else ! (.not. ((m1>n1) .or. (m2>n2) .or. (m3>n3)))
  ! === The box is non-empty: test that all the chunks sum up properly ===
  !
  call rhd_box_ChunkSize(edge_mode, n_elements_per_chunk, m1,n1, m2,n2, m3,n3, 1, &
                         m2c,n2c, m3c,n3c, n_chunks=n_chunks)
  !
  TestName=trim(TestPrefix) // '_ChunkSize' // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'NumberOfChunksIsGreaterThanZero'
  !
  call UT_Assert('>=', TestName, (/n_chunks/), 1, delta=0)
  !
  if (n_chunks > 0) then
    ! === Test if the chunks sum up properly to give the entire box ===
    !
    ! --- Allocate 3D test array and initialize it with zeros ---
    allocate(ia(m1:n1, m2:n2, m3:n3))
    ia(:,:,:)=0
    !
    ! --- Allocate 1D volume array and initialize it ---
    allocate(Vola(0:n_chunks+1))
    Vola(:)=-1
    !
    ! --- Loop over all chunks (one extra at both ends) ---
    do i_chunk=0,n_chunks+1
      call rhd_box_ChunkSize(edge_mode, n_elements_per_chunk, m1,n1, m2,n2, m3,n3, i_chunk, &
                             m2c,n2c, m3c,n3c, m1c=m1c,n1c=n1c)
      ! write(6,'(I4,2X,3(2I4,X))') i_chunk, m1c,n1c, m2c,n2c, m3c,n3c
      !
      ! --- Compute the volume of the chunk ---
      Vola(i_chunk)=(max(n1c, m1c-1)+1-m1c) * &
                    (max(n2c, m2c-1)+1-m2c) * &
                    (max(n3c, m3c-1)+1-m3c)
      !
      ! --- Set all points within the chunk to the chunk index if it is the first access ---
      do i3=m3c,n3c
        do i2=m2c,n2c
          do i1=m1c,n1c
            if (ia(i1,i2,i3) == 0) then
              ia(i1,i2,i3)=i_chunk
            else if (ia(i1,i2,i3) > 0) then
              ia(i1,i2,i3)=-i_chunk
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !
    end do ! i_chunk
    !
    TestName=trim(TestPrefix) // '_ChunkSize' // '_nTest=' // trim(nTestStr) // '_' // &
             trim(TestBase) // '_' // &
            'OutSideChunksHaveZeroVolume'
    !
    call UT_Assert('==', TestName, (/Vola(0), Vola(n_chunks+1)/), 0, delta=0)
    !
    TestName=trim(TestPrefix) // '_ChunkSize' // '_nTest=' // trim(nTestStr) // '_' // &
             trim(TestBase) // '_' // &
            'InSideChunksHavePositiveVolume'
    !
    call UT_Assert('>=', TestName, Vola(1:n_chunks), 1, delta=0)
    !
    TestName=trim(TestPrefix) // '_ChunkSize' // '_nTest=' // trim(nTestStr) // '_' // &
             trim(TestBase) // '_' // &
            'ChunkVolumeRatioIsNotLarge'
    !
    call UT_Assert('<=', TestName, (/real(maxval(Vola(1:n_chunks))) / &
                                     real(minval(Vola(1:n_chunks)))    /), &
                                                                 3.0, delta=0.0, Ignore_flag=.true.)
    !
    ! write(6,*) 'Max/min ratio= ', real(maxval(Vola(1:n_chunks))) / &
    !                               real(minval(Vola(1:n_chunks)))
    !
    TestName=trim(TestPrefix) // '_ChunkSize' // '_nTest=' // trim(nTestStr) // '_' // &
             trim(TestBase) // '_' // &
            'AllIndicesAreCovered'
    !
    call UT_Assert('>=', TestName, abs(ia), 1, delta=0)
    !
    TestName=trim(TestPrefix) // '_ChunkSize' // '_nTest=' // trim(nTestStr) // '_' // &
             trim(TestBase) // '_' // &
             'NoIndexIsCoveredMoreThanOnce'
    !
    call UT_Assert('>=', TestName, ia, 0, delta=0)
    !
    ! --- Free the memory for the allocated array ---
    deallocate(Vola, ia)
    !
  endif ! (n_chunks > 0)
  !
endif ! ((m1>n1) .or. (m2>n2) .or. (m3>n3))
!
end subroutine rhd_box_ChunkSizeOneUT


!----------*******************----------------------------------------------------------------------
subroutine rhd_box_ChunkSizeUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_ChunkSizeUT ('rhd_box_Chunk_Size_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_box_ChunkSize.
!
! CATEGORY:
!   Testing, box handling
!
! CALLING SEQUENCE:
!   call rhd_box_ChunkSizeUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_ChunkSizeOneUT: Perform unit tests for rhd_box_ChunkSize
!
! MODULES:
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in rhd_box_ChunkSizeOneUT.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_ChunkSizeUT('box')
!
! MODIFICATION HISTORY:
!   2016-04-18 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: nTest, i_edgemode, i_nperchunk
!---------------------------------------------------------------------------------------------------
!
nTest=0
!
! --- Check small boxes ---
do i_edgemode=2,2
  do i_nperchunk=1,8
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, 1,2, 1,2, 1,2)
  end do ! i_nperchunk
  !
  do i_nperchunk=1,28
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, 1,4, 1,4, 1,4)
  end do ! i_nperchunk
  !
  do i_nperchunk=1,28
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, 1,7, 1,7, 1,7)
  end do ! i_nperchunk
  !
  do i_nperchunk=1,34
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, -8,8, -8,8, -8,8)
  end do ! i_nperchunk
  !
  do i_nperchunk=1,27
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, 1,5, 11,16, 21,27)
  end do ! i_nperchunk
  !
  do i_nperchunk=1,27
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, 0,5, -5,6, -4,0)
  end do ! i_nperchunk
  !
  do i_nperchunk=100,800,100
    nTest=nTest+1
    call rhd_box_ChunkSizeOneUT(TestPrefix, nTest, i_edgemode, i_nperchunk, 1,100, 1,100, 1,100)
  end do ! i_nperchunk
  !
end do ! i_edgemode
!
end subroutine rhd_box_ChunkSizeUT


!----------************************-----------------------------------------------------------------
subroutine rhd_box_RestIndicesOneUT(TestPrefix, nTest, &
                                    ml1,nl1, ml2,nl2, ml3,nl3, &
                                    ms1,ns1, ms2,ns2, ms3,ns3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_RestIndicesOneUT ('rhd_box_Rest_Indices_One_Unit_Test')
!
! PURPOSE:
!   Perform unit test for rhd_box_RestIndices.
!
! CATEGORY:
!   Box handling, index arithmetic, testing
!
! CALLING SEQUENCE:
!   call rhd_box_RestIndicesOneUT(TestPrefix, nTest, &
!                                 ml1,nl1, ml2,nl2, ml3,nl3, &
!                                 ms1,ns1, ms2,ns2, ms3,ns3)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   ml1:          ('m_number_large_1') integer, 1. dimension, start index of large box
!   nl1:          ('number_large_1')   integer, 1. dimension, end index of large box
!   ml2:          ('m_number_large_2') integer, 2. dimension, start index of large box
!   nl2:          ('number_large_2')   integer, 2. dimension, end index of large box
!   ml3:          ('m_number_large_3') integer, 3. dimension, start index of large box
!   nl3:          ('number_large_3')   integer, 3. dimension, end index of large box
!   ms1:          ('m_number_small_1') integer, 1. dimension, start index of small box
!   ns1:          ('number_small_1')   integer, 1. dimension, end index of small box
!   ms2:          ('m_number_small_2') integer, 2. dimension, start index of small box
!   ns2:          ('number_small_2')   integer, 2. dimension, end index of small box
!   ms3:          ('m_number_small_3') integer, 3. dimension, start index of small box
!   ns3:          ('number_small_3')   integer, 3. dimension, end index of small box
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_RestIndices:   Compute indices of boxes surrounding small box in large box
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   An array with (nl1-ml1+1)*(nl2-ml2+1)*(nl3-ml3+1) elements is allocated (and deallocated),
!   so the number of elements should not be too big.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2016-04-18 (B.F. Uppsala) First version
!   2016-04-23 (B.F.) Renamed
!   2017-07-28 (B.F.) Test also for ibl=0 (original small box)
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nTest, &
                                           ml1,nl1, ml2,nl2, ml3,nl3, &
                                           ms1,ns1, ms2,ns2, ms3,ns3
!
! --- Local variables ---
character(len=128)                      :: TestName, TestBase
integer                                 :: i1, i2, i3, ibl, &
                                           mb1,nb1, mb2,nb2, mb3,nb3
integer, dimension(0:6)                 :: Vola
integer, dimension(:,:,:), allocatable  :: ia
character(len=20)                       :: nTestStr, TestStr
!---------------------------------------------------------------------------------------------------
!
! --- Prepare strings for test name ----
write(nTestStr, *) nTest
nTestStr=adjustl(nTestStr)
!
TestBase=' '
write(TestStr, *) ml1
TestBase=trim(TestBase) //        adjustl(TestStr) 
write(TestStr, *) nl1
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) ml2
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) nl2
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) ml3
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) nl3
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) ms1
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) ns1
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) ms2
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) ns2
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) ms3
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) ns3
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
!
if ((ml1>nl1) .or. (ml2>nl2) .or. (ml3>nl3)) then
  ! === The large box is empty: check only that all rest boxes are empty, too ===
  !
  ! --- Compute the volume of all the sub boxes ---
  do ibl=0,6
    ! --- Compute the indices of next sub box ---
    call rhd_box_RestIndices(ibl, &
                             ml1,nl1, ml2,nl2, ml3,nl3, &
                             ms1,ns1, ms2,ns2, ms3,ns3, &
                             mb1,nb1, mb2,nb2, mb3,nb3)
    !
    ! --- Compute the volume of the rest box (should be zero) ---
    Vola(ibl)=(max(nb1, mb1-1)+1-mb1) * &
              (max(nb2, mb2-1)+1-mb2) * &
              (max(nb3, mb3-1)+1-mb3)
    !
  end do ! ibl
  !
  TestName=trim(TestPrefix) // '_RestIndices' // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
           'AllVolumesAreZero'
  !
  call UT_Assert('==', TestName, Vola, 0, delta=0)
  !
else ! (.not. ((ml1>nl1) .or. (ml2>nl2) .or. (ml3>nl3)))
  ! === The large box is non-empty: test that the small box and the rest boxes sum up properly ===
  !
  ! --- Allocate and initialize 3D test array with zeros ---
  allocate(ia(ml1:nl1, ml2:nl2, ml3:nl3))
  ia(:,:,:)=0
  !
  ! --- Incremenent values in the 7 sub boxes ---
  do ibl=0,6
    ! --- Compute the indices of next sub box ---
    call rhd_box_RestIndices(ibl, &
                             ml1,nl1, ml2,nl2, ml3,nl3, &
                             ms1,ns1, ms2,ns2, ms3,ns3, &
                             mb1,nb1, mb2,nb2, mb3,nb3)
    !
    ! --- Increment all values within the rest box ---
    do i3=mb3,nb3
      do i2=mb2,nb2
        do i1=mb1,nb1
          ia(i1,i2,i3)=ia(i1,i2,i3)+1
        end do ! i1
      end do ! i2
    end do ! i3
    !
  end do ! ibl
  !
  TestName=trim(TestPrefix) // '_RestIndices' // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'AllIndicesAreCovered'
  !
  call UT_Assert('>=', TestName, ia, 1, delta=0)
  !
  TestName=trim(TestPrefix) // '_RestIndices' // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
           'NoIndexIsCoveredMoreThanOnce'
  !
  call UT_Assert('<=', TestName, ia, 1, delta=0)
  !
  ! --- Free the memory for the test array ---
  deallocate(ia)
  !
endif ! ((ml1>nl1) .or. (ml2>nl2) .or. (ml3>nl3))
!
end subroutine rhd_box_RestIndicesOneUT


!----------*********************--------------------------------------------------------------------
subroutine rhd_box_RestIndicesUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_RestIndicesUT ('rhd_box_Rest_Indices_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_box_RestIndices.
!
! CATEGORY:
!   Testing, box handling
!
! CALLING SEQUENCE:
!   call rhd_box_RestIndicesUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_RestIndicesOneUT: Perform unit tests for rhd_box_RestIndices
!
! MODULES:
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in rhd_box_RestIndicesOneUT.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_RestIndicesUT('box')
!
! MODIFICATION HISTORY:
!   2016-04-18 (B.F. Uppsala) First version
!   2019-03-06 (B.F.) Add additional tests
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: nTest, i1, i2, i3
!---------------------------------------------------------------------------------------------------
!
nTest=0
!
! --- Check handling of empty large boxes ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 2,0, 0,2, 0,2, 1,1, 1,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 2,0, 0,2, 1,1, 1,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 2,0, 1,1, 1,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 2,0, 2,0, 0,2, 1,1, 1,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 2,0, 2,0, 2,0, 1,1, 1,1, 1,1)
!
!
! --- Check handling of empty small boxes ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 2,0, 0,2, 0,2, 3,1, 1,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 2,0, 0,2, 0,2, 1,1, 3,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 2,0, 0,2, 0,2, 1,1, 1,1, 3,1)
!
!
! --- Check handling of 1-point small boxes around 0 ---
do i3=-2,2
  do i2=-2,2
    do i1=-2,2
      nTest=nTest+1
      call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,0, 0,0, 0,0, i1,i1, &
                                                                      i2,i2, &
                                                                      i3,i3   )
      !
    end do ! i1
  end do ! i2
end do ! i3
!
!
! --- Check handling of 1-point small boxes ---
do i3=3,7
  do i2=5,9
    do i1=2,6
      nTest=nTest+1
      call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 4,4, 7,7, 5,5, i1,i1, &
                                                                      i2,i2, &
                                                                      i3,i3   )
      !
    end do ! i1
  end do ! i2
end do ! i3
!
!
! --- Check handling of small boxes completely outside large box ---
do i3=-1,1
  do i2=-1,1
    do i1=-1,1
      nTest=nTest+1
      call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 4*i1,4*i1+1, &
                                                                      4*i2,4*i2+1, &
                                                                      4*i3,4*i3+1   )
      !
    end do ! i1
  end do ! i2
end do ! i3
!
!
! --- Check some small boxes with various sizes relative to large box ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,2, 0,2, 0,2)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,1, 1,1, 1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,0, 0,0, 0,0)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2,-1,3,-1,3,-1,3)
!
!
! --- Check handling of small boxes partly outside large box ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2,-1,1,-1,1,-1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,3,-1,1,-1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2,-1,1, 1,3,-1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,3, 1,3,-1,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2,-1,1,-1,1, 1,3)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,3,-1,1, 1,3)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2,-1,1, 1,3, 1,3)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,3, 1,3, 1,3)
!
!
! --- Check handling of flat small boxes ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,0, 0,0, 0,0)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,0, 1,0, 0,0)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,0, 0,0, 1,0)
!
!
! --- Check handling of small boxes larger than large box ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,3, 0,1, 0,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,3, 0,3, 0,1)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,3, 0,3, 0,3)
!
!
! --- Check handling of small boxes in large box that require only one rest box ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 1,2, 0,2, 0,2)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,1, 0,2, 0,2)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,2, 1,2, 0,2)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,2, 0,1, 0,2)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,2, 0,2, 1,2)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, 0,2, 0,2, 0,2, 0,2, 0,2, 0,1)
!
!
! --- Check treatment of a bit larger boxes ---
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest,  0,7, 1,8, 2,9,  1,7, 3,5, 6,8)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, -2,7, 1,8, 2,9,  1,3, 3,5, 6,7)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, -8,7, 1,7, 3,9, -1,3, 3,5, 4,7)
!
nTest=nTest+1
call rhd_box_RestIndicesOneUT(TestPrefix, nTest, -5,9,-3,5, 3,9, -1,3, 3,5, 4,8)
!
end subroutine rhd_box_RestIndicesUT


!----------**********************-------------------------------------------------------------------
subroutine rhd_bound_1DGhostOneUT(TestPrefix, nTest, &
                                  m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_1DGhostOneUT ('rhd_boundary_conditions_1D_Ghost_cells_One_Unit_Test')
!
! PURPOSE:
!   Perform unit test for rhd_bound_1DGhost.
!
! CATEGORY:
!   Box handling, boundary conditions, testing
!
! CALLING SEQUENCE:
!   call rhd_bound_1DGhostOneUT(TestPrefix, nTest, &
!                               m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nTest:        ('number_Test') integer with test counter
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box-handling routines for RHD
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   A box structure with several arrays of at least (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements
!   is allocated (and deallocated), so the number of elements should not be too big.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2016-04-28 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_bound_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nTest, &
                                           m1,n1, m2,n2, m3,n3
!
! --- Local variables ---
character(len=128)                      :: TestName, TestBase
character(len=20)                       :: nTestStr, TestStr
integer                                 :: i1, i2, i3, nghost
real                                    :: dx
type(box_type)                          :: box
type(action_type)                       :: action
!---------------------------------------------------------------------------------------------------
!
! --- Prepare strings for test name ----
write(nTestStr, *) nTest
nTestStr=adjustl(nTestStr)
!
TestBase=' '
!write(TestStr, *) edge_mode
!TestBase=trim(TestBase) //        'EdgeMode=' // adjustl(TestStr) 
!write(TestStr, *) n_elements_per_chunk
!TestBase=trim(TestBase) // '_' // 'nPerChunk=' // adjustl(TestStr) 
write(TestStr, *) m1
TestBase=trim(TestBase) //        adjustl(TestStr) 
write(TestStr, *) n1
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) m2
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n2
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) m3
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n3
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
!
if ((m1>n1) .or. (m2>n2) .or. (m3>n3)) then
  ! === The requested box is empty: skip all checks ===
  !
else ! (.not. ((m1>n1) .or. (m2>n2) .or. (m3>n3)))
  ! === The box is non-empty ===
  !
  nghost=2
  dx    =1.0
  !
  ! --- Define some entries in the global parameter structure 'par' ---
  par%C_Hptopfactor=0.0
  par%rho_min      =0.0
  par%C_magthetaB  =0.0
  par%C_magphiB    =0.0
  !
  ! --- Define entries in the local control structure 'action' ---
  action=rhd_action_New(action, &
                        dim=1, dimV=((/1, 2, 3/)), &
                        bound_l1    =bound_constant    , bound_r1    =bound_constant    , &
                        bound_l2    =bound_None        , bound_r2    =bound_None        , &
                        bound_l3    =bound_None        , bound_r3    =bound_None        , &
                        bound_mag_l1=bound_mag_constant, bound_mag_r1=bound_mag_constant, &
                        bound_mag_l2=bound_mag_None    , bound_mag_r2=bound_mag_None    , &
                        bound_mag_l3=bound_mag_None    , bound_mag_r3=bound_mag_None    , &
                        radraybase  =''                , radraystar=''                     )
  !
  ! --- Create the test box ---
  call rhd_box_Make(box, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost, &
                         quc_flag=.true., Bc_flag=.true.)
  !
  ! --- Fill the test box with bulk values but omit the ghost cells ---
  call rhd_box_FillTestOMP(box, AddMode=0, dx=dx, nghost1=0, nghost2=0, nghost3=0)
  !
  ! --- Fill the ghost cells according to the selected boundary conditions ---
  call rhd_bound_1DGhost(box, action, grid_mode=1, bulk_mode=1)
  !
  ! --- Subtract the initial values from all points in the test box, including the ghost cells ---
  call rhd_box_FillTestOMP(box, AddMode=2, dx=dx, nghost1=nghost, nghost2=0, nghost3=0)
  !
  TestName=trim(TestPrefix) // '_bound_1DGhost' // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase)
  !
  ! --- Check the test box ---
  call rhd_boxUT_Test(TestName, box)
  !
  ! --- Delete the test box ---
  call rhd_box_Delete(box)
  !
endif ! ((m1>n1) .or. (m2>n2) .or. (m3>n3))
!
end subroutine rhd_bound_1DGhostOneUT


!----------*******************----------------------------------------------------------------------
subroutine rhd_bound_1DGhostUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_1DGhostUT ('rhd_boundary_conditions_1D_Ghost_cells_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_bound_1DGhost.
!
! CATEGORY:
!   Testing, boundary conditions, box handling
!
! CALLING SEQUENCE:
!   call rhd_bound_1DGhostUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_bound_1DGhostOneUT: Perform unit tests for rhd_bound_1DGhost
!
! MODULES:
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in rhd_bound_1DGhostOneUT.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_bound_1DGhostUT('box')
!
! MODIFICATION HISTORY:
!   2016-04-28 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: nTest
!---------------------------------------------------------------------------------------------------
!
nTest=0
!
! --- Check ---
nTest=nTest+1
call rhd_bound_1DGhostOneUT(TestPrefix, nTest, 1,4, 1,3, 1,2)
!
end subroutine rhd_bound_1DGhostUT


!----------*****************------------------------------------------------------------------------
subroutine rhd_boxUT_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_boxUT_TestAll ('radiation_hydrodynamics_box_Unit_Test_Test_All')
!
! PURPOSE:
!   Perform all unit tests of box-handling routines.
!
! CATEGORY:
!   Testing, box handling
!
! CALLING SEQUENCE:
!   call rhd_boxUT_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_bound_1DGhostUT:   Perform unit tests for rhd_bound_1DGhost
!   rhd_box_ChunkSizeUT:   Perform unit tests for rhd_box_ChunkSize
!   rhd_box_RestIndicesUT: Perform unit tests for rhd_box_RestIndices
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in rhd_box_RestIndicesOneUT.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_boxUT_TestAll()
!
! MODIFICATION HISTORY:
!   2016-04-18 (B.F. Uppsala) First version
!   2016-04-23 (B.F.) Move list of test calls into another subroutine
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='box'
!
call rhd_box_ChunkSizeUT(TestPrefix)
call rhd_box_RestIndicesUT(TestPrefix)
call rhd_bound_1DGhostUT(TestPrefix)
!
end subroutine rhd_boxUT_TestAll


!----------******************-----------------------------------------------------------------------
subroutine rhd_boxUT_TestSome()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_boxUT_TestSome ('radiation_hydrodynamics_box_Unit_Test_Test_Some')
!
! PURPOSE:
!   Perform some unit tests of box-handling routines.
!
! CATEGORY:
!   Testing, box handling
!
! CALLING SEQUENCE:
!   call rhd_boxUT_TestSome()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in rhd_box_RestIndicesOneUT.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_boxUT_TestSome()
!
! MODIFICATION HISTORY:
!   2016-04-18 (B.F. Uppsala) First version
!   2016-04-23 (B.F.) Move list of test calls into another subroutine
!   2017-08-26 (B.F. Lyon) Adapted from rhd_boxUT_TestAll
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='box'
!
call rhd_box_RestIndicesUT(TestPrefix)
!call rhd_bound_1DGhostUT(TestPrefix)
!
end subroutine rhd_boxUT_TestSome

end module rhd_boxUT_module
