!***************************************************************************************************
!
!   #####   #    #  #####           ####   #
!   #    #  #    #  #    #         #    #  #
!   #    #  ######  #    #         #       #
!   #####   #    #  #    #         #  ###  #
!   #   #   #    #  #    #         #    #  #
!   #    #  #    #  #####  #######  ####   ###### _module
!
!   Radiation-Hydrodynamics-Code: global variables
!
!***************************************************************************************************
!   Fortran90
!   2019-03-05
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!   Werner Schaffenberger: Potsdam, Exeter
!   Sven Wedemeyer-Boehm:  Kiel, Freiburg
!***************************************************************************************************
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
!***************************************************************************************************


!------*************--------------------------------------------------------------------------------
module rhd_gl_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_gl_module ('rhd_global_parameters_module')
!
! PURPOSE:
!   Save global type definitions and parameters for rhd.
!
! CATEGORY:
!   Global type definitions and parameters
!
! CALLING SEQUENCE:
!   use rhd_gl_module
!
! TYPES:
!   gl_type:      ('global_type') structure type for global data:
!     nc_p:       ('number_channel_print') integer, channel for text output, typically 6
!     nc_m:       ('number_channel_my_print') integer, channel for text output, typically 6 or -1
!     lw_p:       ('line_width_print') integer, desired maximal line width
!     parFile:    ('parameter_File') character, name of parameter file, typically 'rhd.par'
!                 Its contents is described in some detail in 'rhd_io_module.f90'.
!     par2File:   ('parameter_2_File') character, name of 2nd parameter file, typically 'rhd.par2'
!                 The contents is used only in special cases of smooth transitions between
!                 between parameter values with time. Normally, the file should not exist.
!     StopFile:   ('Stop_File') character, name of stop file, typically 'rhd.stop'
!                 The existence of this file is checked at the beginning of each new time step.
!                 If the file exists the simulation is interrupted gracefully (and an end file is
!                 produced).
!     ContFile:   ('Continuation_File') character, name of continuation file, typically 'rhd.cont'
!                 This file is somewhat complementary to the StopFile. If the ContFile exists
!                 at the beginning of a simulation, the startmodel is not read from the file
!                 specified by the entry 'infile_start' in the parameter file but from the file
!                 specified by 'OutFile_end'. The data written into the full and mean file is
!                 appended to existing files.
!     StatusFile: ('Status_File') character, name of exit status file, typically 'rhd.done'
!                 If the simulation ends regularly this file is produced.
!     DumpFile:   ('Dump_File') character, name of dump request file, typically 'rhd.dump'
!                 If this file exists, the current model is written into a file specified
!                 by SnapshotFile.
!     SnapshotFile: ('Snapshot_File') character, name of snapshot output file, typically 'rhd.snap'
!     version:    ('version') character, rhd code version
!   par_type:     ('parameter_type') structure type for data from input parameter file,
!                 the description of the components can be found in rhd_io_module.f90
!
! VARIABLES:
!   gl:           ('global') structure for global data
!   par:          (parameter') structure for data from input parameter file.
!                 See rhd_io_module.F90 for a description of the components.
!
! MODIFICATION HISTORY:
!   1997-05-29 (B.F. Kiel) Written
!   1997-06-05 (B.F.) Next version
!   1997-06-10 (B.F.) type box_type
!   1997-06-19 (B.F.) type action_type, grav_type, bottom_type, top_type
!   1997-07-23 (B.F.) type action_type, par_type extended
!   1997-07-24 (B.F.) type action_type, par_type extended
!   1997-10-14 (B.F.) type par_type extended
!   1997-10-21 (B.F.) type gl changed
!   1999-03-25 (B.F. Copenhagen) Name of energy in type mean_type change, outcommented
!                     New basic variables in type box_type
!   1999-04-06 (B.F.) Put into own file
!   1999-04-07 (B.F.) Parameter grav_type -> grav_mode; parameters mass_star, r0_grav added
!   1999-04-16 (B.F.) Parameter C_visDrag added
!   1999-05-14 (B.F.) Parameter dtime_incmax added
!   1999-05-18 (B.F.) Parameters C_radCourant, C_radCourantMax added
!   1999-05-19 (B.F.) Parameters C_radmaxeiChange, s_InFlow, C_sChange, C_PChange added
!   1999-05-25 (B.F.) gravcorr_terms
!   1999-05-27 (B.F.) Parameters for artificial viscosity
!   1999-05-28 (B.F.) box_type into rhd_box_module, action_type into rhd_action_module
!   1999-05-31 (B.F.) Parameters C_visCourant, C_visCourantMax added
!   1999-06-08 (B.F.) Parameter C_maxeiChange added
!   1999-06-30 (B.F.) Parameter N_raditer added
!   1999-08-31 (B.F.) Parameters for radiative transfer added
!   1999-12-08 (B.F.) Parameters radraybase, radraystar added
!   2000-01-22 (B.F.) grav_type outcommented
!   2000-02-03 (B.F.) Type scale_type introduced
!   2000-03-23 (B.F.) gl%StopFile added
!   2000-04-02 (B.F.) Parameter N_radThin/ThickPoint added
!   2000-04-08 (B.F.) gl%ContFile added
!   2000-04-13 (B.F.) Parameters for Matthias Steffens radiation transport routines added
!   2000-04-28 (B.F.) Parameter r1_grav added
!   2000-05-29 (B.F.) Parameter kappaMax added
!   2000-05-31 (B.F.) Parameter C_radTintminfac added
!   2000-12-10 (B.F. Uppsala) Parameter N_CellsPerChunk added
!   2000-12-28 (B.F.) Parameter dtime_start, starttime added
!   2001-01-11 (B.F.) Parameter starttimestep added
!   2001-01-23 (B.F.) Parameters N_XXXCellsPerChunk and test parameters added
!   2001-04-10 (B.F.) Parameters C_Hptopfactor, C_radTvisdtau, C_radTvis added
!   2001-06-29 (B.F.) Parameter C_radthintimefac added
!   2002-05-07 (B.F.) Parameters rho_min, LuminosityPerVolume, istep_in_start added
!   2002-11-07 (B.F.) gl%lw_p
!   2002-11-08 (B.F.) Parameters r1_rad, dustScheme, N_outslicedim_mean added
!   2002-11-28 (B.F.) gl%StatusFile; C_dust0X
!   2003-01-16 (B.F. Montpellier) gravcorr_terms removed
!   2003-01-22 (B.F.) Parameter C_SlopeRed added
!   2004-02-27 (B.F.) Parameter C_CoreDrag added
!   2004-05-13 (B.F.) Parameters C_visP2Pcoeff, C_visP2Pincl(later: 1,2) added
!   2004-08-18 (B.F.) DumpFile, SnapshotFile
!   2005-03-08 (B.F.) dtime_out_end
!   2005-03-11 (B.F.) chem_abumetal, chem_reacfile, chem_reacpath
!   2005-05-31 (WS&SWB) Parameters C_resB, C_resEpsilon added
!   2006-08-07 (SWB)  hion_datapath, hion_atomfile, hion_abufile, hion_edensfile, 
!                     hion_pffile, dtime_out_hion, hion_chunks
!   2006-11-07 (B.F. Lyon) New parameters added:
!                     N_radrsyslevel, N_radoutput, C_radtcool, C_radscool, C_raddcool,
!                     N_test4, N_test5, C_test4, C_test5, strtest4, strtest5,
!                     B1_inflow
!   2006-12-14 (B.F.) New parameter: heat_mode
!   2007-01-31 (B.F.) New parameter: nu_rotation
!   2007-02-08 (B.F.) New parameter: hdsplit
!   2007-04-16 (B.F.) New parameters: dustOpaFile, ar_dustGrainRadius
!   2007-12-12 (B.F.) More C_dust0X
!   2008-03-11 (B.F.) Parameters C_hydExpCourant, C_hydExpCourantMax added
!   2008-03-17 (B.F.) Parameters C_ReconstructiondTimeFactor, C_hydPredfactor added
!   2008-08-12 (B.F.) Parameters C_radTinci, C_radDinci added
!   2008-06-10 (W.S. Freiburg) side_bound_mag, top_bound_mag, bottom_bound_mag added. Magnetic 
!                     boundary conditions are now separated from hydrodynamic boundary conditions
!   2008-10-22 (W.S. Lansing)  parameters beta_inv, B_theta, B_phi, N_side, N_top, N_bottom
!   2008-10-31 (B.F.) New parameter: dustopamode
!   2009-06-16 (B.F.) New parameter: ar_RotationAxis
!   2010-02-15 (W.S. Freiburg) side_bound_mag split into side_bound_max_x and side_bound_max_y
!   2010-02-16 (W.S. Freiburg) New parameter: va_max
!   2010-04-05 (B.F. Lyon) Parameter C_rhoChangeTop added
!   2010-04-11 (W.S. Freiburg) Parameter C_resBconstant added
!   2010-04-14 (B.F.) B_theta, B_phi, N_side, N_top, N_bottom =>
!                     C_magthetaB, C_magphiB, N_magDiffSide, N_magDiffTop, N_magDiffBottom
!   2010-05-10 (B.F.) New parameters: OutFile_chu1, OutDim_chu1
!   2010-06-03 (B.F. Freiburg) New parameter: hdTimeIntegrationScheme
!   2010-06-09 (B.F. Lyon) New parameter: hdCheckFlux
!   2011-01-06 (W.S. Exeter) New parameters N_resCellsPerChunk, C_resCourant, C_resCourantMax
!   2011-01-27 (W.S. Lyon) New parameter N_orderConstrainedTransport
!   2011-06-20 (W.S.) New parameters r0_core, centrifugal_force
!   2011-08-05 (W.S.) New parameter r2_grav
!   2011-08-26 (B.F.) New parameter hdTransVeloMode
!   2011-09-01 (B.F.) New parameters C_hydTdiff1, C_hydTdiff2
!   2011-09-02 (B.F.) New params C_visP2PhypSmagorinsky, N_visP2PhypSmooth, visP2Phypreconstruction
!   2011-09-07 (B.F.) Add sub structure 'A' to 'par'
!   2011-11-23 (B.F.) New parameter dustReconstruction
!   2011-12-24 (B.F.) Parameters C_hydSoundCourant, C_hydSoundCourantMax added
!   2012-01-01 (B.F.) Parameters C_v3ChangeLinBottom, C_v3ChangeSqrBottom added
!   2012-01-09 (B.F.) Renamed parameters C_hydTdiff1, C_hydTdiff2 -> C_hydTdiffLin, C_hydTdiffMach
!   2012-01-17 (B.F.) New parameter C_visP2Pdivrhov
!   2012-03-07 (B.F.) New parameter ar_dustGrainAbu
!   2012-04-03 (B.F.) New parameter C_hydsDiffVelo
!   2012-04-04 (B.F.) New parameter C_hydvDiffVelo
!   2012-05-04 (B.F.) New parameter C_hydLowMachcsFactor
!   2012-06-05 (B.F.) New parameters hdEntropyWaveMode, hdEnthalpyAvgMode
!   2012-07-09 (B.F.) New Parameter hdCoreDragProfile
!   2012-07-18 (B.F.) New global parameter nc_m
!   2012-07-27 (B.F.) Move scale%length to prop%length
!   2012-07-30 (B.F.) More parameters from action structure to par%A structure;
!                     new parameter aN_MPIProcsPerDim
!   2013-08-07 (B.F.) New parameters: OutFile_chu2/3, OutDim_chu2/3
!   2013-08-08 (B.F.) New parameter: OutFile_mcyl
!   2013-12-05 (B.F.) New parameter: kappaMin
!   2014-03-01 (B.F. Uppsala) New parameter: s_InFlGrad
!   2014-04-08 (B.F.) New parameter: hdCoreHeatProfile
!   2014-06-16 (B.F.) New parameters s_InFlRadGrad, s_InFlLatGrad replace s_InFlGrad
!   2014-07-11 (B.F.) New parameter: C_gravmu
!   2014-08-11 (B.F.) New parameter: C_hydSmallFluctFactor
!   2014-08-15 (B.F.) New parameters: N_hydRKIter, N_hydCellsPerChunk3
!   2014-08-20 (B.F.) New parameters: C_RecWenoWeightCenter, C_RecWenoWeightRight,C_RecWenoPsiFactor
!   2014-08-24 (B.F.) New parameter: N_hydCellsPerChunk2
!   2014-08-25 (B.F.) New parameter: hdTmpStructKeepMode
!   2014-09-04 (B.F.) Remove parameter: C_ReconstructiondTimeFactor
!   2014-09-22 (B.F.) New parameter: C_hydLowMachcsBase
!   2014-12-13 (B.F.) New parameter: radThinHeatScheme; removed: C_visNeu1, C_visNeu2, C_visDrag
!   2014-12-17 (B.F.) New parameter: N_visIter
!   2015-04-25 (B.F.) Additional global parameter structures par1 and par2; add gl%par2file
!   2015-04-27 (B.F.) New parameters: time_ScaleStart, time_ScaleEnd
!   2016-03-15 (B.F.) New parameter: C_radAccEnergyFraction
!   2017-07-15 (B.F.) New parameter: OutFile_out; new global variable gl%OutFile
!   2017-10-05 (B.F.) Remove parameter aN_MPIProcsPerDim from Ainpar_type
!   2017-10-30 (B.F.) New parameter: N_radFeautrier
!   2018-04-24 (B.F.) New parameters: grav_type, C_gravCorePotDepth, C_gravCorePotQuad
!   2018-06-11 (B.F.) New parameter C_gravCorer0Heat=r0_core/r0_grav replaces r0_core
!   2019-02-26 (B.F.) New parameters C_radthintimeRelax, radThinPressScheme, N_radThinHeatPoint,
!                     N_radThinPressPoint
!   2019-03-05 (B.F.) New parameter C_radthinTFactor
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- Global parameters ---
!
type gl_type
  integer                               :: nc_p, nc_m, lw_p
  character(len=80)                     :: parFile, par2File, StopFile, ContFile, StatusFile, &
                                           DumpFile, SnapshotFile, OutFile
  character(len=80)                     :: version
end type gl_type
!
type(gl_type), save                     :: gl
!
! --- Derived parameters ---
type Ainpar_type
  logical                               :: IsRead_flag
  integer                               :: reconstruction_order, &
                                           reconstruction_type, &
                                           reconstruction_quctype, &
                                           visP2Phypreconstruction_order, &
                                           visP2Phypreconstruction_type, &
                                           hdScheme, radScheme
end type Ainpar_type
!
! --- All parameters from parameter file ---
!
integer, parameter                      :: ndesmax=20, nhismax=20, &
                                           n_dustGrainRadiusMax=100
!
type par_type
  integer                               :: ndes
  character(len=80), dimension(ndesmax) :: description
  integer                               :: nhis
  character(len=80), dimension(nhismax) :: history
  !
  type(Ainpar_type)                     :: A
  !
  real                                  :: Teff, &
                                           grav, C_gravmu, C_gravCorePotDepth, C_gravCorePotQuad, &
                                           mass_star, &
                                           r0_grav, r1_grav, r2_grav, r1_rad, C_gravCorer0Heat, &
                                           nu_rotation
  real, dimension(3)                    :: ar_RotationAxis
  character(len=80)                     :: heat_mode, grav_mode, grav_type, centrifugal_force
  !
  character(len=80)                     :: side_bound, top_bound, bottom_bound, &
                                           side_bound_mag_x1, side_bound_mag_x2, &
                                           top_bound_mag, bottom_bound_mag, &
                                           hdCoreHeatProfile, hdCoreDragProfile
  real                                  :: s_InFlow, s_InFlRadGrad, s_InFlLatGrad, &
                                           C_sChange, C_PChange, C_v3ChangeLinBottom, &
                                           C_v3ChangeSqrBottom, &
                                           C_TChange, C_rhoChangeTop, &
                                           C_Tsurf, &
                                           C_TminLimit, C_TminRate, &
                                           C_Hptopfactor, rho_min, LuminosityPerVolume, C_CoreDrag,&
                                           C_magthetaB, C_magphiB, B1_inflow
  integer                               :: N_magDiffSide, N_magDiffTop, N_magDiffBottom
  !
  real                                  :: abuX, abuY, abuZ, Qmol, gamma
  character(len=80)                     :: EOSFile, EOSPath
  real                                  :: kappaMin, kappaMax
  character(len=80)                     :: OpaFile, OpaPath
  real                                  :: chem_abumetal
  character(len=80)                     :: chem_reacfile, chem_reacpath
  !
  character(len=80)                     :: hion_datapath, hion_atomfile, hion_abufile, &
                                           hion_edensfile, hion_pffile
  real                                  :: dtime_out_hion
  integer                               :: hion_chunks
  !
  character(len=80)                     :: hdScheme, hdTimeIntegrationScheme, hdsplit, &
                                           hdTmpStructKeepMode, &
                                           hdCheckFlux, reconstruction, &
                                           hdTransVeloMode, hdEntropyWaveMode, hdEnthalpyAvgMode
  integer                               :: N_hydIter, N_hydMaxIter, N_hydRKIter, &
                                           N_orderConstrainedTransport, &
                                           N_hydCellsPerChunk, N_hydCellsPerChunk2,&
                                           N_hydCellsPerChunk3, &
                                           N_visCellsPerChunk, &
                                           N_resCellsPerChunk, N_radCellsPerChunk
  integer, dimension(3)                 :: aN_MPIProcsPerDim
  real                                  :: C_SlopeRed, C_RecContShift, C_RecContSteep, &
                                           C_RecWenoWeightCenter, C_RecWenoWeightRight, &
                                           C_RecWenoPsiFactor, &
                                           C_hydPredfactor, &
                                           C_hydTDiffLin, C_hydTDiffMach, &
                                           C_hydsDiffVelo, C_hydvDiffVelo, &
                                           C_hydLowMachcsFactor, C_hydLowMachcsBase, &
                                           C_hydSmallFluctFactor
  real                                  :: C_visBound
  real                                  :: beta_inv, va_max
  !
  character(len=80)                     :: dustScheme, dustOpaFile, dustopamode, dustReconstruction
  integer                               :: n_dustGrainRadius, n_dustGrainAbu
  real                                  :: C_dust01, C_dust02, C_dust03, C_dust04, C_dust05, &
                                           C_dust06, C_dust07, C_dust08, C_dust09, C_dust10
  real, &
      dimension(1:n_dustGrainRadiusMax) :: ar_dustGrainRadius, ar_dustGrainAbu
  !
  character(len=80)                     :: radScheme, radThinHeatScheme, radThinPressScheme, &
                                           radraybase, radraystar, radpressure
  integer                               :: N_radminiter, N_raditer, N_radmaxiter, &
                                           N_radtaurefine, &
                                           N_radThinPoint, N_radThinHeatPoint, N_radThinPressPoint,&
                                           N_radThickPoint, &
                                           N_radtheta, N_radphi, N_radsubray, N_radband, &
                                           N_radrsyslevel, N_radoutput, N_radFeautrier
  real                                  :: C_radimplicitmu, C_raditereps, C_raditerstep, &
                                           C_radkappasmooth, C_radTstep, C_radTsmooth, &
                                           C_radTintminfac, &
                                           C_radHtautop, C_radTvisdtau, C_radTvis, &
                                           C_radthintimefac, C_radthintimeRelax, C_radthinTFactor, &
                                           C_radtcool, C_radscool, C_raddcool, &
                                           C_radTinci, C_radDinci, C_radAccEnergyFraction
  !
  character(len=80)                     :: visP2Phypreconstruction
  integer                               :: N_visIter, N_visP2PhypSmooth
  real                                  :: C_visLinear, &
                                           C_visSmagorinsky, C_visArtificial, C_visExpansion, &
                                           C_visPrturb, &
                                           C_visTensordiag, C_visTensoroff, C_visTensordiv, &
                                           C_visP2Pcoeff, C_visP2Pincl1, C_visP2Pincl2, &
                                           C_visP2PhypSmagorinsky, C_visP2PhypArtificial, &
                                           C_visP2PhypExpansion, C_visP2Pdivrhov
  real                                  :: C_resB, C_resBconstant, C_resEpsilon
  !
  real                                  :: starttime, endtime, plustime, &
                                           time_ScaleStart, time_ScaleEnd, &
                                           CPUtime, CPUtime_remainlimit
  integer                               :: starttimestep, endtimestep, plustimestep
  !
  real                                  :: dtime_min, dtime_max, dtime_min_stop, dtime_incmax, &
                                           dtime_start, &
                                           C_Courant        , C_CourantMax        , C_maxeiChange, &
                                           C_hydExpCourant  , C_hydExpCourantMax  , &
                                           C_hydSoundCourant, C_hydSoundCourantMax, &
                                           C_radCourant     , C_radCourantMax    ,C_radmaxeiChange,&
                                           C_visCourant     , C_visCourantMax     , &
                                           C_resCourant     , C_resCourantMax
  !
  real                                  :: dtime_out_full, dtime_out_mean, dtime_out_end
  integer                               :: dtimestep_out_fine, istep_in_start, N_outslicedim_mean
  character(len=80)                     :: infile_start, &
                                           OutFile_end , OutForm_end , OutConv_end , &
                                           OutFile_full, OutForm_full, OutConv_full, &
                                           OutFile_mean, OutForm_mean, OutConv_mean, &
                                           OutFile_fine, OutForm_fine, OutConv_fine, &
                                           OutFile_mcyl, &
                                           OutFile_chu1, &
                                           OutFile_chu2, &
                                           OutFile_chu3, &
                                           OutFile_out
  integer, dimension(2,3)               :: OutDim_chu1, OutDim_chu2, OutDim_chu3
  !
  integer                               :: N_test1, N_test2, N_test3, N_test4, N_test5
  real                                  :: C_test1, C_test2, C_test3, C_test4, C_test5
  character(len=80)                     :: strtest1, strtest2, strtest3, strtest4, strtest5
end type par_type
!
type(par_type), save                    :: par, par1, par2
!
end module rhd_gl_module
