!***************************************************************************************************
!
!                                                          #     # ####### 
!   #####  #    # #####        #    # ######   ##   #    # #     #    #    
!   #    # #    # #    #       ##  ## #       #  #  ##   # #     #    #    
!   #    # ###### #    #       # ## # #####  #    # # #  # #     #    #    
!   #####  #    # #    #       #    # #      ###### #  # # #     #    #    
!   #   #  #    # #    #       #    # #      #    # #   ## #     #    #    
!   #    # #    # ##### ###### #    # ###### #    # #    #  #####     #  _module  
!
!   Radiation-Hydrodynamics-Code: unit-test routines for averaging routines
!
!***************************************************************************************************
!   Fortran90
!   2018-11-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_meanUT_module:     Unit-test routines for averaging routines
!
!***************************************************************************************************


!------*****************----------------------------------------------------------------------------
module rhd_meanUT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_meanUT_module ('rhd_mean_Unit_Test_module')
!
! PURPOSE:
!   Provide unit-test routines for averaging routines.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhd_meanUT_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_mean_x12meanUT:    Perform unit tests of rhd_mean_x12mean
!   rhd_mean_x23meanUT:    Perform unit tests of rhd_mean_x23mean
!   rhd_mean_a2rindexOneUT: Perform one set of unit tests for rhd_mean_a2rindex
!   rhd_mean_rindexOneUT:  Perform one set of unit tests for rhd_mean_rindex
!   rhd_mean_a2rindexPairUT: Perform pairwise unit tests for rhd_mean_a2rindex
!   rhd_mean_rindexUT:     Perform unit tests for rhd_mean_rindex and rhd_mean_a2rindex
!   rhd_mean_SphericalHarmonicUT: Perform unit tests of rhd_mean_SphericalHarmonic
!   rhd_meanUT_TestAll:    Perform unit tests of all averaging routines
!   rhd_meanUT_TestSome:   Perform unit tests of some averaging routines
!
! MODIFICATION HISTORY:
!   2015-03-11 (B.F. Uppsala) First version
!   2017-08-26 (B.F. Lyon) Add routine for partial unit tests
!   2017-08-27 (B.F.) Test rhd_mean_rindex
!   2018-06-11 (B.F. Uppsala) Test rhd_mean_SphericalHarmonic
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_mean_x12meanUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_x12meanUT ('rhd_mean_x12_mean_Unit_Test')
!
! PURPOSE:
!   Perform unit tests of rhd_mean_x12mean.
!   
! CATEGORY:
!   Testing, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_x12meanUT(Testprefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   q:            ('quantity') 3D real array with input data
!   weight:       ('weight') 2D real array with weights
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   UT_Assert:             Check array or relation between two arrays
!   UT_PrintArray:         Print a 1D/2D/3D array
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_mean_module:       Averaging routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   size(q,1) = size(weight,1)
!   size(q,2) = size(weight,2)
!   size(q,3) = size(qmean)
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_mean_x12meanUT('rhd_mean_x12mean_')
!
! MODIFICATION HISTORY:
!   2015-03-11 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
use rhd_mean_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, nb1
real                                    :: TestValue
real, dimension(:,:,:), allocatable     :: q
real, dimension(:,:  ), allocatable     :: weight
real(kind=mArk), &
      dimension(    :), allocatable     :: qmean, qmtmp
character(len=8)                        :: bStr
character(len=128)                      :: TestName
real                                    :: delta
!---------------------------------------------------------------------------------------------------
!
! === Create test case: array filled with constant value ===========================================
m1=1 ; n1=10
m2=1 ; n2=10
m3=1 ; n3=4
delta    =1.0E-06
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_10x10_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case: array filled with constant value ---
m1=1 ; n1=100
m2=1 ; n2=100
m3=1 ; n3=4
delta    =1.0E-04
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_100x100_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case: array filled with constant value ---
m1=1 ; n1=200
m2=1 ; n2=200
m3=1 ; n3=4
delta    =1.0E-03
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_200x200_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case: array filled with constant value ---
m1=1 ; n1=400
m2=1 ; n2=400
m3=1 ; n3=4
delta    =1.0E-03
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_400x400_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case: array filled with constant value ---
m1=1 ; n1=800
m2=1 ; n2=800
m3=1 ; n3=4
delta    =6.0E-03
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_800x800_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! === Create test case: array with slowly increasing values, use different block sizes =============
bStr='b01'
m1=1 ; nb1=1 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='b02'
m1=1 ; nb1=2 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmtmp(            m3:n3 ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    qmtmp=0.0
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmtmp, weight=weight(i1:i1+nb1-1,i2:i2))
    qmean=qmean+qmtmp
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b04'
m1=1 ; nb1=4 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmtmp(            m3:n3 ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    qmtmp=0.0
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmtmp, weight=weight(i1:i1+nb1-1,i2:i2))
    qmean=qmean+qmtmp
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b08'
m1=1 ; nb1=8 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmtmp(            m3:n3 ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    qmtmp=0.0
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmtmp, weight=weight(i1:i1+nb1-1,i2:i2))
    qmean=qmean+qmtmp
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b16'
m1=1 ; nb1=16 ; n1=(400/nb1)*nb1
m2=1 ;          n2=400
m3=1 ;          n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmtmp(            m3:n3 ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    qmtmp=0.0
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmtmp, weight=weight(i1:i1+nb1-1,i2:i2))
    qmean=qmean+qmtmp
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b40'
m1=1 ; nb1=40 ; n1=(400/nb1)*nb1
m2=1 ;          n2=400
m3=1 ;          n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmtmp(            m3:n3 ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    qmtmp=0.0
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmtmp, weight=weight(i1:i1+nb1-1,i2:i2))
    qmean=qmean+qmtmp
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! === Create test case: array with slowly increasing values, use different block sizes =============
bStr='n01'
m1=1 ; nb1=1 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
!
call rhd_mean_x12mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n02'
m1=1 ; nb1=2 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmean, weight=weight(i1:i1+nb1-1,i2:i2))
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n04'
m1=1 ; nb1=4 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmean, weight=weight(i1:i1+nb1-1,i2:i2))
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n08'
m1=1 ; nb1=8 ; n1=(400/nb1)*nb1
m2=1 ;         n2=400
m3=1 ;         n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmean, weight=weight(i1:i1+nb1-1,i2:i2))
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n16'
m1=1 ; nb1=16 ; n1=(400/nb1)*nb1
m2=1 ;          n2=400
m3=1 ;          n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmean, weight=weight(i1:i1+nb1-1,i2:i2))
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n40'
m1=1 ; nb1=40 ; n1=(400/nb1)*nb1
m2=1 ;          n2=400
m3=1 ;          n3=8
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(m1:n1,m2:n2      ), &
         qmean(            m3:n3 ))
!
TestValue=1.0
do i3=m3,n3
  q(:,:,i3)=TestValue+0.1*float(i3-m3)
end do ! i3
weight   =1.0/float((n1-m1+1)*(n2-m2+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i2=m2,n2
  do i1=m1,n1,nb1
    call rhd_mean_x12mean(q(i1:i1+nb1-1,i2:i2,:), qmean, weight=weight(i1:i1+nb1-1,i2:i2))
  end do ! i1
end do ! i2
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m3), kind=kind(0.0E+00)), q(m1,m2,:), delta=delta)
!
deallocate(q,qmean,weight)
!
end subroutine rhd_mean_x12meanUT


!----------******************-----------------------------------------------------------------------
subroutine rhd_mean_x23meanUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_x23meanUT ('rhd_mean_x23_mean_Unit_Test')
!
! PURPOSE:
!   Perform unit tests of rhd_mean_x23mean.
!   
! CATEGORY:
!   Testing, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_x23meanUT(Testprefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   q:            ('quantity') 3D real array with input data
!   weight:       ('weight') 2D real array with weights
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   UT_Assert:             Check array or relation between two arrays
!   UT_PrintArray:         Print a 1D/2D/3D array
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_mean_module:       Averaging routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   size(q,1) = size(qmean)
!   size(q,2) = size(weight,1)
!   size(q,3) = size(weight,2)
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_mean_x23meanUT('rhd_mean_x23mean_')
!
! MODIFICATION HISTORY:
!   2015-03-11 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
use rhd_mean_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, nb2
real                                    :: TestValue
real, dimension(:,:,:), allocatable     :: q
real, dimension(  :,:), allocatable     :: weight
real(kind=mArk), &
      dimension(:    ), allocatable     :: qmean, qmtmp
character(len=8)                        :: bStr
character(len=128)                      :: TestName
real                                    :: delta
!---------------------------------------------------------------------------------------------------
!
! === Create test case: array filled with constant value ===========================================
m1=1 ; n1=4
m2=1 ; n2=10
m3=1 ; n3=10
delta    =1.0E-06
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_10x10_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
m1=1 ; n1=4
m2=1 ; n2=100
m3=1 ; n3=100
delta    =1.0E-04
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_100x100_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
m1=1 ; n1=4
m2=1 ; n2=200
m3=1 ; n3=200
delta    =1.0E-03
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_200x200_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
m1=1 ; n1=4
m2=1 ; n2=400
m3=1 ; n3=400
delta    =1.0E-03
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_400x400_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), TestValue, delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case: large array with slowly increasing values ---
m1=1 ; n1=8
m2=1 ; n2=800
m3=1 ; n3=800
delta    =3.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
q        =TestValue
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_800x800_One_AverageIsOne'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! === Create test case: array with slowly increasing values, use different block sizes =============
bStr='b01'
m1=1 ;         n1=8
m2=1 ; nb2=1 ; n2=nb2*400
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='b02'
m1=1 ;         n1=8
m2=1 ; nb2=2 ; n2=nb2*200
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmtmp( m1:n1            ), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    qmtmp=0.0
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmtmp, weight=weight(i2:i2+nb2-1,i3:i3))
    qmean=qmean+qmtmp
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b04'
m1=1 ;         n1=8
m2=1 ; nb2=4 ; n2=nb2*100
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmtmp( m1:n1            ), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    qmtmp=0.0
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmtmp, weight=weight(i2:i2+nb2-1,i3:i3))
    qmean=qmean+qmtmp
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b08'
m1=1 ;         n1=8
m2=1 ; nb2=8 ; n2=nb2*50
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmtmp( m1:n1            ), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    qmtmp=0.0
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmtmp, weight=weight(i2:i2+nb2-1,i3:i3))
    qmean=qmean+qmtmp
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b16'
m1=1 ;          n1=8
m2=1 ; nb2=16 ; n2=nb2*25
m3=1 ;          n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmtmp( m1:n1            ), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    qmtmp=0.0
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmtmp, weight=weight(i2:i2+nb2-1,i3:i3))
    qmean=qmean+qmtmp
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! --- Create test case ---
bStr='b40'
m1=1 ;          n1=8
m2=1 ; nb2=40 ; n2=nb2*10
m3=1 ;          n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmtmp( m1:n1            ), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    qmtmp=0.0
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmtmp, weight=weight(i2:i2+nb2-1,i3:i3))
    qmean=qmean+qmtmp
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmtmp,qmean,weight)
!
!
! === Create test case: array with slowly increasing values, use different block sizes =============
bStr='n01'
m1=1 ;         n1=8
m2=1 ; nb2=1 ; n2=nb2*400
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
call rhd_mean_x23mean(q, qmean, weight=weight)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n02'
m1=1 ;         n1=8
m2=1 ; nb2=2 ; n2=nb2*200
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmean, weight=weight(i2:i2+nb2-1,i3:i3))
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n04'
m1=1 ;         n1=8
m2=1 ; nb2=4 ; n2=nb2*100
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmean, weight=weight(i2:i2+nb2-1,i3:i3))
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n08'
m1=1 ;         n1=8
m2=1 ; nb2=8 ; n2=nb2*50
m3=1 ;         n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmean, weight=weight(i2:i2+nb2-1,i3:i3))
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n16'
m1=1 ;          n1=8
m2=1 ; nb2=16 ; n2=nb2*25
m3=1 ;          n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmean, weight=weight(i2:i2+nb2-1,i3:i3))
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
!
! --- Create test case ---
bStr='n40'
m1=1 ;          n1=8
m2=1 ; nb2=40 ; n2=nb2*10
m3=1 ;          n3=400
delta    =1.0E-02
!
allocate(q(     m1:n1,m2:n2,m3:n3), &
         weight(      m2:n2,m3:n3), &
         qmean( m1:n1            ))
!
TestValue=1.0
do i1=m1,n1
  q(i1,:,:)=TestValue+0.1*float(i1-m1)
end do ! i1
weight   =1.0/float((n2-m2+1)*(n3-m3+1))
qmean    =0.0
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('q  =', q     )
  call UT_PrintArray('wei=', weight)
endif
!
do i3=m3,n3
  do i2=m2,n2,nb2
    call rhd_mean_x23mean(q(:,i2:i2+nb2-1,i3:i3), qmean, weight=weight(i2:i2+nb2-1,i3:i3))
  end do ! i2
end do ! i3
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_AverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean, kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
TestName=trim(TestPrefix) // '_400x400' // trim(bStr) // '_Inc_ScaledAverageIsAlmostEqual'
call UT_Assert('==', TestName, real(qmean/qmean(m1), kind=kind(0.0E+00)), q(:,m2,m3), delta=delta)
!
deallocate(q,qmean,weight)
!
end subroutine rhd_mean_x23meanUT


!----------**********************-------------------------------------------------------------------
subroutine rhd_mean_a2rindexOneUT(TestPrefix, nTest, &
                                  m1,n1, m2,n2, m3,n3, nghost1, m1_global,n1_global, &
                                  nxFlux, dxFlux)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_a2rindexOneUT ('rhd_mean_array_2_radius_index_One_Unit_Test')
!
! PURPOSE:
!   Perform one set of unit tests for rhd_mean_a2rindex.
!
! CATEGORY:
!   Averaging, testing
!
! CALLING SEQUENCE:
!   call rhd_mean_a2rindexOneUT(TestPrefix, nTest, &
!                               m1,n1, m2,n2, m3,n3, nghost1, m1_global, n1_global, &
!                               nxFlux, dxFlux)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nTest:        ('number_Test') integer with test counter
!   m1:           ('m_number_1') integer, 1st dimension, start index
!   n1:           ('number_1')   integer, 1st dimension, end index
!   m2:           ('m_number_2') integer, 2nd dimension, start index
!   n2:           ('number_2')   integer, 2nd dimension, end index
!   m3:           ('m_number_3') integer, 3rd dimension, start index
!   n3:           ('number_3')   integer, 3rd dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1st dimension, number of ghost cells (> 0)
!   m1_global:    ('m_1_global') integer, 1st dimension, global start index
!   n1_global:    ('n_1_global') integer, 1st dimension, global end index
!   nxflux:       ('number_x_flux') integer with size of xflux vector
!   dxflux:       ('delta_x_flux') real with step size of xflux vector [cm]
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_FillTestOMP:   Set bulk arrays in box structure to some test values
!   rhd_box_Make:          Create computational box of requested size
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_rb23mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rb23absmean:  Average boundary-centered scalar quantity over spheres in asymmetric box
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_mean_module:       Averaging routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   A box structure with several arrays of at least (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements
!   is allocated (and deallocated), so the number of elements should not be too big.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-27 (B.F. Lyon) First version
!   2017-09-02 (B.F.) Add input parameters
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use rhd_mean_module
use rhd_meankind_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nTest, &
                                           m1,n1, m2,n2, m3,n3, nghost1, m1_global,n1_global, nxFlux
real,                      intent(in)   :: dxFlux
!
! --- Local variables ---
character(len=128)                      :: TestName, TestBase
character(len=20)                       :: nTestStr, TestStr
integer                                 :: i1, i2, i3
real                                    :: dx
type(box_type)                          :: box
real,            dimension(nxFlux  )    :: xFlux
real(kind=mArk), dimension(nxFlux-1)    :: qcmean
real(kind=mArk), dimension(nxFlux  )    :: qbmean
integer, dimension(m1:n1, m2:n2, m3:n3) :: cindex
integer, dimension(6    , m2:n2, m3:n3) :: nb6index
integer, dimension(-nxFlux:nxflux, &
                          m2:n2, m3:n3) :: bindex
real,    dimension(       m2:n2, m3:n3) :: weight
real,    dimension(m1:n1, m2:n2, m3:n3) :: x123weight
!---------------------------------------------------------------------------------------------------
!
! --- Set parameter that doesn't change (grid size) ---
dx=1.0
!
! --- Prepare strings for test name ----
write(nTestStr, *) nTest
nTestStr=adjustl(nTestStr)
!
TestBase=' '
write(TestStr, *) m1
TestBase=trim(TestBase) //          adjustl(TestStr) 
write(TestStr, *) n1
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) m2
TestBase=trim(TestBase) // '_'   // adjustl(TestStr) 
write(TestStr, *) n2
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) m3
TestBase=trim(TestBase) // '_'   // adjustl(TestStr) 
write(TestStr, *) n3
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) m1_global
TestBase=trim(TestBase) // '_g=' // adjustl(TestStr) 
write(TestStr, *) n1_global
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) nxFlux
TestBase=trim(TestBase) // '_nf='// adjustl(TestStr) 
write(TestStr, '(F5.3)') dxFlux
TestBase=trim(TestBase) // '_df='// adjustl(TestStr) 
!
if ((m1>n1) .or. (m2>n2) .or. (m3>n3)) then
  ! === The requested box is empty: skip all tests ===
  !
else ! (.not. ((m1>n1) .or. (m2>n2) .or. (m3>n3)))
  ! === The box is non-empty: perform tests ===
  !
  ! --- Fill reference axes ---
  do i1=1,nxFlux
    xFlux(i1)=real(i1-1)*dxFlux
  end do ! i1
  !
  ! --- Create the test box ---
  call rhd_box_Make(box, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost1, nghost2=0, nghost3=0, &
                         quc_flag=.true., Bc_flag=.true.)
  !
  ! --- Fill the test box with bulk values but omit the ghost cells ---
  call rhd_box_FillTestOMP(box, AddMode=0, dx=dx, nghost1=nghost1, nghost2=0, nghost3=0, &
                           FillValues=(/ 2.0, 3.0, 1.0, 0.0, 0.0 /))
  !
  ! --- Produce index arrays for averaging over spheres ---
  call rhd_mean_a2rindex(box%m1,box%n1, box%m2,box%n2, box%m3,box%n3, &
                         nghost1, m1_global, n1_global, &
                         box%xc1, box%xc2, box%xc3, &
                         xFlux, nb6index, bindex, cindex)
  !
  !
  ! --- Test cindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesWithinInputAxis'
  !
  call UT_Assert('<=', TestName, cindex, nXFlux-1, delta=0)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesArePositive'
  !
  call UT_Assert('>=', TestName, cindex, 0, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotOne'
  !
  call UT_Assert('/=', TestName, cindex, 1, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotTwo'
  !
  call UT_Assert('/=', TestName, cindex, 2, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotThree'
  !
  call UT_Assert('/=', TestName, cindex, 3, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotFour'
  !
  call UT_Assert('/=', TestName, cindex, 4, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotLast'
  !
  call UT_Assert('/=', TestName, cindex, nxFlux-1, delta=0, Ignore_flag=.true.)
  !
  !
  ! --- Test bindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BIndicesWithinInputAxis'
  !
  call UT_Assert('>=', TestName, bindex, -nXFlux-1, delta=0, Ignore_flag=.true.)
  !
  !
  ! --- Test nbindex --- 
  !nb6index(1,:,:)=mlbindex
  !nb6index(2,:,:)=nlbindex
  !nb6index(4,:,:)=mrbindex
  !nb6index(5,:,:)=nrbindex
  TestName=trim(TestPrefix)  // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'nB4IndicesAreLargerThanZero'
  !
  call UT_Assert('>=', TestName, nb6index, 0, delta=0, Ignore_flag=.true.)
  !
  !
  ! --- Compute rc mean and test it ---
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(box, 1.0, weight, x123weight)
  !
  qcmean=0.0
  call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, nxFlux, &
                         1.0+0.0*box%rho(m1:n1, m2:n2, m3:n3), qcmean, &
                         cindex, x123weight)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rc23meanOfOneIsPositive'
  !
  call UT_Assert('>=', TestName, real(qcmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  ! --- Compute rb mean and test it ---
  !
  qbmean=0.0
  call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, nxFlux, &
                            0.0*box%v1(m1:n1+1, m2:n2, m3:n3), qbmean, &
                            nb6index, bindex, weight, unity=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rbmeanOfUnityIsPositive'
  !
  call UT_Assert('>=', TestName, real(qbmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  qbmean=0.0
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, nxFlux, &
                       0.5*(box%v1(m1-1:n1, m2:n2, m3:n3)+ box%v1(m1:n1+1, m2:n2, m3:n3)), qbmean, &
                       nb6index, bindex, weight)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rbmeanOfOneIsZero'
  !
  call UT_Assert('==', TestName, real(qbmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05, &
                 Ignore_flag=.true.)


  !
  ! --- Delete the test box ---
  call rhd_box_Delete(box)
  !
endif ! ((m1>n1) .or. (m2>n2) .or. (m3>n3))
!
end subroutine rhd_mean_a2rindexOneUT


!----------********************---------------------------------------------------------------------
subroutine rhd_mean_rindexOneUT(TestPrefix, nTest, &
                                m1,n1, m2,n2, m3,n3, nxFlux, dxFlux)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rindexOneUT ('rhd_mean_radius_index_One_Unit_Test')
!
! PURPOSE:
!   Perform one set of unit tests for rhd_mean_rindex.
!
! CATEGORY:
!   Averaging, testing
!
! CALLING SEQUENCE:
!   call rhd_mean_rindexOneUT(TestPrefix, nTest, &
!                             m1,n1, m2,n2, m3,n3, nxFlux, dxFlux)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nTest:        ('number_Test') integer with test counter
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_FillTestOMP:   Set bulk arrays in box structure to some test values
!   rhd_box_Make:          Create computational box of requested size
!   rhd_mean_arindex:      Produce index arrays for averaging over spheres in a symmetric box
!   rhd_mean_rbmean:       Average boundary-centered quantity over spheres in a symmetric box
!   rhd_mean_rcmean:       Average cell-centered quantity over spheres in a symmetric box
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_mean_module:       Averaging routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   A box structure with several arrays of at least (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements
!   is allocated (and deallocated), so the number of elements should not be too big.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-27 (B.F. Lyon) First version
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use rhd_mean_module
use rhd_meankind_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nTest, &
                                           m1,n1, m2,n2, m3,n3, nxFlux
real,                      intent(in)   :: dxFlux
!
! --- Local variables ---
character(len=128)                      :: TestName, TestBase
character(len=20)                       :: nTestStr, TestStr
integer                                 :: i1, i2, i3, nghost
real                                    :: dx
type(box_type)                          :: box
real,            dimension(nxFlux  )    :: xFlux
real(kind=mArk), dimension(nxFlux-1)    :: qcmean
real(kind=mArk), dimension(nxFlux  )    :: qbmean
integer, dimension(m1:n1, m2:n2, m3:n3) :: cindex
integer, dimension(       m2:n2, m3:n3) :: nbindex
integer, dimension(1:1,   m2:n2, m3:n3) :: nbindex1
integer, dimension(-nxFlux:nxflux, &
                          m2:n2, m3:n3) :: bindex
real,    dimension(       m2:n2, m3:n3) :: weight
real,    dimension(m1:n1, m2:n2, m3:n3) :: x123weight
!---------------------------------------------------------------------------------------------------
!
! --- Prepare strings for test name ----
write(nTestStr, *) nTest
nTestStr=adjustl(nTestStr)
!
TestBase=' '
write(TestStr, *) m1
TestBase=trim(TestBase) //        adjustl(TestStr) 
write(TestStr, *) n1
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) m2
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n2
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) m3
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, *) n3
TestBase=trim(TestBase) // ':' // adjustl(TestStr) 
write(TestStr, *) nxFlux
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
write(TestStr, '(F5.3)') dxFlux
TestBase=trim(TestBase) // '_' // adjustl(TestStr) 
!
if ((m1>n1) .or. (m2>n2) .or. (m3>n3)) then
  ! === The requested box is empty: skip all checks ===
  !
else ! (.not. ((m1>n1) .or. (m2>n2) .or. (m3>n3)))
  ! === The box is non-empty ===
  !
  nghost=1
  dx    =1.0
  !
  ! --- Create the test box ---
  call rhd_box_Make(box, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost, nghost2=nghost, nghost3=nghost, &
                         quc_flag=.true., Bc_flag=.true.)
  !
  ! --- Fill the test box with bulk values but omit the ghost cells ---
  call rhd_box_FillTestOMP(box, AddMode=0, dx=dx, nghost1=nghost, nghost2=nghost, nghost3=nghost, &
                           FillValues=(/ 2.0, 3.0, 1.0, 0.0, 0.0 /))
  !
  ! --- Fill reference axes ---
  do i1=1,nxFlux
    xFlux(i1)=real(i1-1)*dxFlux
  end do ! i1
  !
  ! --- Produce index arrays for averaging over spheres ---
  !call rhd_mean_rindex(box, xFlux, nbindex, bindex, cindex)
  call rhd_mean_arindex(box%m1,box%n1, box%m2,box%n2, box%m3,box%n3, &
                        box%xc1(m1:n1), box%xc2(m2:n2), box%xc3(m3:n3), &
                        xFlux, nbindex, bindex, cindex)
  !
  !
  ! --- Test cindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesWithinInputAxis'
  !
  call UT_Assert('<=', TestName, cindex, nXFlux-1, delta=0)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreLargerThanZero'
  !
  call UT_Assert('>=', TestName, cindex, 0, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotOne'
  !
  call UT_Assert('/=', TestName, cindex, 1, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotTwo'
  !
  call UT_Assert('/=', TestName, cindex, 2, delta=0, Ignore_flag=.true.)
  !
!  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
!           trim(TestBase) // '_' // &
!          'CIndicesAreNotFour'
!  !
!  call UT_Assert('/=', TestName, cindex, 4, delta=0, Ignore_flag=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'CIndicesAreNotLast'
  !
  call UT_Assert('/=', TestName, cindex, nxFlux-1, delta=0, Ignore_flag=.true.)
  !
  !
  ! --- Test bindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BIndicesWithinInputAxis'
  !
  call UT_Assert('>=', TestName, bindex, -nXFlux-1, delta=0, Ignore_flag=.true.)
  !
  !
  ! --- Test nbindex --- 
  nbindex1(1,:,:)=nbindex
  TestName=trim(TestPrefix)  // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'nBIndicesAreLargerThanZero'
  !
  call UT_Assert('>=', TestName, nbindex1, 0, delta=0, Ignore_flag=.true.)
  !
  !
  ! --- Compute rc mean and test it ---
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(box, 1.0, weight, x123weight)
  !
  qcmean=0.0
  call rhd_mean_rcmean(m1,n1, m2,n2, m3,n3, nxFlux, &
                       1.0+0.0*box%rho(m1:n1, m2:n2, m3:n3), qcmean, &
                       nbindex, bindex, cindex, x123weight)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rcmeanIsPositive'
  !
  call UT_Assert('>=', TestName, real(qcmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  ! --- Compute rb mean and test it ---
  !
  qbmean=0.0
  call rhd_mean_rbmean(m1,n1, m2,n2, m3,n3, nxFlux, &
                       0.0*box%v1(m1:n1+1, m2:n2, m3:n3), qbmean, &
                       nbindex, bindex, weight, unity=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rbunityIsPositive'
  !
  call UT_Assert('>=', TestName, real(qbmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  qbmean=0.0
  call rhd_mean_rbmean(m1,n1, m2,n2, m3,n3, nxFlux, &
                       0.5*(box%v1(m1-1:n1, m2:n2, m3:n3)+ box%v1(m1:n1+1, m2:n2, m3:n3)), qbmean, &
                       nbindex, bindex, weight)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rbmeanIsZero'
  !
  call UT_Assert('==', TestName, real(qbmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  ! --- Delete the test box ---
  call rhd_box_Delete(box)
  !
endif ! ((m1>n1) .or. (m2>n2) .or. (m3>n3))
!
end subroutine rhd_mean_rindexOneUT


!----------***********************------------------------------------------------------------------
subroutine rhd_mean_a2rindexPairUT(TestPrefix, nTest, &
                                   m1,n1, m2,n2, m3,n3, nghost1, m1_split, &
                                   m1f,n1f, nxFlux, dxFlux)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_a2rindexPairUT ('rhd_mean_array_2_radius_index_One_Unit_Test')
!
! PURPOSE:
!   Perform pairwise unit tests for rhd_mean_a2rindex.
!
! CATEGORY:
!   Averaging, testing
!
! CALLING SEQUENCE:
!   call rhd_mean_a2rindexPairUT(TestPrefix, nTest, &
!                                m1,n1, m2,n2, m3,n3, nghost1, m1_split, &
!                                nxFlux, dxFlux)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!   nTest:        ('number_Test') integer with test counter
!   m1:           ('m_number_1') integer, 1st dimension, start index
!   n1:           ('number_1')   integer, 1st dimension, end index
!   m2:           ('m_number_2') integer, 2nd dimension, start index
!   n2:           ('number_2')   integer, 2nd dimension, end index
!   m3:           ('m_number_3') integer, 3rd dimension, start index
!   n3:           ('number_3')   integer, 3rd dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1st dimension, number of ghost cells (> 0)
!   m1_split:     ('m_1_split') integer, 1st dimension, index at which box is split
!   m1f:          ('m_1_fill') integer, 1st dimension, start of interval of qc filled with 1
!   n1f:          ('number_1_fill') integer, 1st dimension, end of interval of qc filled with 1
!   nxFlux:       ('number_x_Flux') integer with size of xFlux vector
!   dxFlux:       ('delta_x_Flux') real with step size of xFlux vector [cm]
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_FillTestOMP:   Set bulk arrays in box structure to some test values
!   rhd_box_Make:          Create computational box of requested size
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_rb23mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rb23absmean:  Average boundary-centered scalar quantity over spheres in asymmetric box
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_mean_module:       Averaging routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!   A box structure with several arrays of at least (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements
!   is allocated (and deallocated), so the number of elements should not be too big.
!
! PROCEDURE:
!   Check if sum of averages over parts of box equals average over entire box.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-09-02 (B.F. Lyon) First version
!   2017-09-04 (B.F.) Use only box%rho for qc mean, fill only a part
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use rhd_mean_module
use rhd_meankind_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
integer,                   intent(in)   :: nTest, &
                                           m1,n1, m2,n2, m3,n3, nghost1, m1_split, m1f,n1f, nxFlux
real,                      intent(in)   :: dxFlux
!
! --- Local variables ---
character(len=128)                      :: TestName, TestBase
character(len=20)                       :: nTestStr, TestStr
integer                                 :: i1, i2, i3, m1t,n1t
real                                    :: dx
type(box_type)                          :: box, boxL, boxR
real,            dimension(nxFlux  )    :: xFlux
real(kind=mArk), dimension(nxFlux-1)    :: qcmean, qcmeanL, qcmeanR
real(kind=mArk), dimension(nxFlux  )    :: qbmean, qbmeanL, qbmeanR
integer, dimension(6    , m2:n2, m3:n3) :: nb6index, nb6indexL, nb6indexR
integer, dimension(-nxFlux:nxFlux, &
                          m2:n2, m3:n3) :: bindex, bindexL, bindexR
real,    dimension(       m2:n2, m3:n3) :: weight
integer, dimension(m1:n1, m2:n2, m3:n3) :: cindex
real,    dimension(m1:n1, m2:n2, m3:n3) :: x123weight, qc
integer, dimension(m1      :m1_split-1, &
                          m2:n2, m3:n3) :: cindexL
real,    dimension(m1      :m1_split-1, &
                          m2:n2, m3:n3) :: x123weightL
integer, dimension(m1_split:n1, &
                          m2:n2, m3:n3) :: cindexR
real,    dimension(m1_split:n1, &
                          m2:n2, m3:n3) :: x123weightR
!---------------------------------------------------------------------------------------------------
!
! --- Set parameter that doesn't change (grid size) ---
dx=1.0
!
! --- Prepare strings for test name ----
write(nTestStr, *) nTest
nTestStr=adjustl(nTestStr)
!
TestBase=' '
write(TestStr, *) m1
TestBase=trim(TestBase) //          adjustl(TestStr) 
write(TestStr, *) n1
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) m2
TestBase=trim(TestBase) // '_'   // adjustl(TestStr) 
write(TestStr, *) n2
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) m3
TestBase=trim(TestBase) // '_'   // adjustl(TestStr) 
write(TestStr, *) n3
TestBase=trim(TestBase) // ':'   // adjustl(TestStr) 
write(TestStr, *) m1_split
TestBase=trim(TestBase) // '_split=' // adjustl(TestStr) 
write(TestStr, *) m1f
TestBase=trim(TestBase) // '_fill=' // adjustl(TestStr) 
write(TestStr, *) n1f
TestBase=trim(TestBase) // ':'      // adjustl(TestStr) 
write(TestStr, *) nxFlux
TestBase=trim(TestBase) // '_nf='// adjustl(TestStr) 
write(TestStr, '(F5.3)') dxFlux
TestBase=trim(TestBase) // '_df='// adjustl(TestStr) 
!
if ((m1>n1) .or. (m2>n2) .or. (m3>n3) .or. (m1_split <= m1) .or. (m1_split > n1)) then
  ! === The requested box is empty: skip all tests ===
  !
else
  ! === The box is non-empty: perform tests ===
  !
  ! --- Fill reference axes ---
  do i1=1,nxFlux
    xFlux(i1)=real(i1-1)*dxFlux
  end do ! i1
  !
  !
  ! --- Create the global box and test it separately -----------------------------------------------
  !
  m1t=m1
  n1t=n1
  !
  ! --- Create the global test box ---
  call rhd_box_Make(box, m1=m1t, n1=n1t, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost1, nghost2=0, nghost3=0, &
                         quc_flag=.true., Bc_flag=.true.)
  !
  ! --- Fill the test box with bulk values but omit the ghost cells ---
  call rhd_box_FillTestOMP(box, AddMode=0, dx=dx, nghost1=nghost1, nghost2=0, nghost3=0, &
                           FillValues=(/ 0.0, 3.0, 0.0, 0.0, 0.0 /))
  box%rho(max(m1f,m1-nghost1):min(n1f,n1+nghost1),:,:)=1.0
  box%v1( max(m1f,m1-nghost1):min(n1f,n1+nghost1),:,:)=1.0
  !
  ! --- Produce index arrays for averaging over spheres ---
  call rhd_mean_a2rindex(box%m1,box%n1, box%m2,box%n2, box%m3,box%n3, &
                         nghost1, m1,n1, &
                         box%xc1, box%xc2, box%xc3, &
                         xFlux, nb6index, bindex, cindex)
  !
  ! --- Test cindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxG:CIndicesWithinInputAxis'
  !
  call UT_Assert('<=', TestName, cindex, nXFlux-1, delta=0)
  !
  ! --- Compute rc mean and test it ---
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(box, 1.0, weight, x123weight)
  !
  qcmean=0.0
  call rhd_mean_rc23mean(m1t,n1t, m2,n2, m3,n3, nxFlux, &
                         box%rho(m1t:n1t, m2:n2, m3:n3), qcmean, &
                         cindex, x123weight)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxG:rc23meanOfOneIsPositive'
  !
  call UT_Assert('>=', TestName, real(qcmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  ! --- Compute rb mean and test it ---
  !
  qbmean=0.0
  call rhd_mean_rb23absmean(m1t,n1t, m2,n2, m3,n3, nxFlux, &
                            box%v1(m1t:n1t+1, m2:n2, m3:n3), qbmean, &
                            nb6index, bindex, weight, unity=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxG:rbmeanOfUnityIsPositive'
  !
  call UT_Assert('>=', TestName, real(qbmean,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  ! --- Create the left box and test it separately -------------------------------------------------
  !
  m1t=m1
  n1t=m1_split-1
  !
  ! --- Create the global test box ---
  call rhd_box_Make(boxL, m1=m1t, n1=n1t, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost1, nghost2=0, nghost3=0, &
                         quc_flag=.true., Bc_flag=.true.)
  !
  ! --- Fill the test box with bulk values but omit the ghost cells ---
  call rhd_box_FillTestOMP(boxL, AddMode=0, dx=dx, nghost1=nghost1, nghost2=0, nghost3=0, &
                           FillValues=(/ 1.0, 3.0, 1.0, 0.0, 0.0 /))
  !
  ! --- Produce index arrays for averaging over spheres ---
  call rhd_mean_a2rindex(boxL%m1,boxL%n1, boxL%m2,boxL%n2, boxL%m3,boxL%n3, &
                         nghost1, m1,n1, &
                         boxL%xc1, boxL%xc2, boxL%xc3, &
                         xFlux, nb6indexL, bindexL, cindexL)
  !
  ! --- Test cindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxL:CIndicesWithinInputAxis'
  !
  call UT_Assert('<=', TestName, cindexL, nXFlux-1, delta=0)
  !
  ! --- Compute rc mean and test it ---
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(boxL, 1.0, weight, x123weightL)
  !
  qcmeanL=0.0
  call rhd_mean_rc23mean(m1t,n1t, m2,n2, m3,n3, nxFlux, &
                         box%rho(m1t:n1t, m2:n2, m3:n3), qcmeanL, &
                         cindexL, x123weightL)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxL:rc23meanOfOneIsPositive'
  !
  call UT_Assert('>=', TestName, real(qcmeanL,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  ! --- Compute rb mean and test it ---
  !
  qbmeanL=0.0
  call rhd_mean_rb23absmean(m1t,n1t, m2,n2, m3,n3, nxFlux, &
                            box%v1(m1t:n1t+1, m2:n2, m3:n3), qbmeanL, &
                            nb6indexL, bindexL, weight, unity=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxL:rbmeanOfUnityIsPositive'
  !
  call UT_Assert('>=', TestName, real(qbmeanL,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  ! --- Create the right box and test it separately ------------------------------------------------
  !
  m1t=m1_split
  n1t=n1
  !
  ! --- Create the global test box ---
  call rhd_box_Make(boxR, m1=m1t, n1=n1t, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost1, nghost2=0, nghost3=0, &
                         quc_flag=.true., Bc_flag=.true.)
  !
  ! --- Fill the test box with bulk values but omit the ghost cells ---
  call rhd_box_FillTestOMP(boxR, AddMode=0, dx=dx, nghost1=nghost1, nghost2=0, nghost3=0, &
                           FillValues=(/ 1.0, 3.0, 1.0, 0.0, 0.0 /))
  !
  ! --- Produce index arrays for averaging over spheres ---
  call rhd_mean_a2rindex(boxR%m1,boxR%n1, boxR%m2,boxR%n2, boxR%m3,boxR%n3, &
                         nghost1, m1,n1, &
                         boxR%xc1, boxR%xc2, boxR%xc3, &
                         xFlux, nb6indexR, bindexR, cindexR)
  !
  ! --- Test cindex ---
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxR:CIndicesWithinInputAxis'
  !
  call UT_Assert('<=', TestName, cindexR, nXFlux-1, delta=0)
  !
  ! --- Compute rc mean and test it ---
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(boxR, 1.0, weight, x123weightR)
  !
  qcmeanR=0.0
  call rhd_mean_rc23mean(m1t,n1t, m2,n2, m3,n3, nxFlux, &
                         box%rho(m1t:n1t, m2:n2, m3:n3), qcmeanR, &
                         cindexR, x123weightR)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxR:rc23meanOfOneIsPositive'
  !
  call UT_Assert('>=', TestName, real(qcmeanR,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  ! --- Compute rb mean and test it ---
  !
  qbmeanR=0.0
  call rhd_mean_rb23absmean(m1t,n1t, m2,n2, m3,n3, nxFlux, &
                            box%v1(m1t:n1t+1, m2:n2, m3:n3), qbmeanR, &
                            nb6indexR, bindexR, weight, unity=.true.)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'BoxR:rbmeanOfUnityIsPositive'
  !
  call UT_Assert('>=', TestName, real(qbmeanR,kind=kind(0.0E+00)), 0.0, delta=1.0E-05)
  !
  !
  ! --- Check if partial averages sum up properly --------------------------------------------------
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rc23meanSumUp'
  !
  call UT_Assert('==', TestName, real(qcmean         ,kind=kind(0.0E+00)), &
                                 real(qcmeanL+qcmeanR,kind=kind(0.0E+00)), delta=1.0E-05)
  !
  TestName=trim(TestPrefix) // '_nTest=' // trim(nTestStr) // '_' // &
           trim(TestBase) // '_' // &
          'rb23meanSumUp'
  !
  call UT_Assert('==', TestName, real(qbmean         ,kind=kind(0.0E+00)), &
                                 real(qbmeanL+qbmeanR,kind=kind(0.0E+00)), delta=1.0E-05)
  !
  !
  ! --- Delete the test boxes ---
  call rhd_box_Delete(boxR)
  call rhd_box_Delete(boxL)
  call rhd_box_Delete(box )
  !
endif
!
end subroutine rhd_mean_a2rindexPairUT


!----------*****************------------------------------------------------------------------------
subroutine rhd_mean_rindexUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rindexUT ('rhd_mean_radius_index_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for rhd_mean_rindex and rhd_mean_a2rindex.
!
! CATEGORY:
!   Testing, boundary conditions, box handling
!
! CALLING SEQUENCE:
!   call rhd_mean_rindexUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_mean_a2rindexOneUT: Perform one set of unit tests for rhd_mean_a2rindex
!   rhd_mean_a2rindexPairUT: Perform pairwise unit tests for rhd_mean_a2rindex
!   rhd_mean_rindexOneUT:  Perform one set of unit tests for rhd_mean_rindex
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in rhd_mean_rindexOneUT and rhd_mean_a2rindexOneUT.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2016-04-28 (B.F. Uppsala) First version
!   2017-09-04 (B.F. Lyon) Add detailed (single-point) tests for rindex and rcmean
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: nTest, m1,n1, m1_split, i1f, i2, i3
!---------------------------------------------------------------------------------------------------
!
nTest=0
!
! --- Check ---
!
nTest=nTest+1
call rhd_mean_rindexOneUT(  TestPrefix // '_arindex_' , nTest, -3, 3, 0,5, 0,0, 6, 0.9)
!
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -3, 3, 0,5, 0,0, 1, -3,3, 6, 0.9)
!
!
nTest=nTest+1
call rhd_mean_rindexOneUT(  TestPrefix // '_arindex_' , nTest, -4, 4, -1,5, 0,0, 6, 0.9)
!
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -4, 4, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -4, 1, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest,  2, 4, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -4, 0, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest,  1, 4, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -4,-1, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest,  0, 4, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -4,-2, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -1, 4, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -4,-3, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
nTest=nTest+1
call rhd_mean_a2rindexOneUT(TestPrefix // '_a2rindex_', nTest, -2, 4, -1,5, 0,0, 1, -4,4, 6, 0.9)
!
!
m1=-4 ; n1= 4
do m1_split=m1+1,n1
  nTest=nTest+1
  call rhd_mean_a2rindexPairUT(TestPrefix // '_a2rindexPair_', &
                                               nTest, m1,n1, 0,5, 0,0, 1, m1_split, m1,n1+1, 6, 0.9)
end do ! m1_split
!
do m1_split=m1+1,n1
  nTest=nTest+1
  do i3=0,4
    do i2=0,5
      do i1f=m1,n1+1
        call rhd_mean_a2rindexPairUT(TestPrefix // '_a2rindexPairSingleColumn_', &
                                           nTest, m1,n1, i2,i2, i3,i3, 1, m1_split, i1f,i1f, 6, 0.9)
      end do ! i1f
    end do ! i2
  end do ! i3
end do ! m1_split
!
m1=-6 ; n1= 6
do m1_split=m1+1,n1
  nTest=nTest+1
  call rhd_mean_a2rindexPairUT(TestPrefix // '_a2rindexPair_', &
                                               nTest, m1,n1, 0,6, 0,0, 1, m1_split, m1,n1+1, 6, 1.1)
end do ! m1_split
!
end subroutine rhd_mean_rindexUT


!----------****************************-------------------------------------------------------------
subroutine rhd_mean_SphericalHarmonicUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_SphericalHarmonicUT ('rhd_mean_Spherical_Harmonic_Unit_Test')
!
! PURPOSE:
!   Perform unit tests of rhd_mean_SphericalHarmonic.
!   
! CATEGORY:
!   Testing, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_SphericalHarmonicUT(Testprefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   UT_Assert:             Check array or relation between two arrays
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_mean_SphericalHarmonicUT('rhd_mean_SphericalHarmonic_')
!
! MODIFICATION HISTORY:
!   2018-06-11 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_mean_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer                                 :: i1, i2, i3, i, j, nPoints
real                                    :: r, r2, rMax, rMin, r2Max, r2Min, Y0Amp
integer, parameter                      :: n=10
integer, parameter                      :: nPointsMax=(2*n+1)**3
real, dimension(nPointsMax)             :: x1, x2, x3
real, parameter                         :: dx=1.0
character(len=128)                      :: TestName
real                                    :: delta
integer, parameter                      :: nHarmonic=16
real, dimension(nPointsMax, &
                0:nHarmonic-1)          :: Y
real, dimension(0:nHarmonic-1, &
                0:nHarmonic-1   )       :: M, Munity
!---------------------------------------------------------------------------------------------------
!
! --- Compute inner and outer radius of shell ---
rMax =real(n)
rMin =rMax-dx
r2Max=rMax**2
r2Min=rMin**2
!
i=0
!
! --- Loop over all grid points to find all points within shell ---
do i3=-n,n
  do i2=-n,n
    do i1=-n,n
      r2=real(i1**2 + i2**2 + i3**2)
      if ((r2 <= r2Max) .and. (r2 > r2Min)) then
        i=i+1
        r=sqrt(r2)
        x1(i)=real(i1)/r
        x2(i)=real(i2)/r
        x3(i)=real(i3)/r
      endif
    end do ! i1
  end do ! i2
end do ! i3
nPoints=i
!
! --- Compute all spherical harmonics ---
do i=0,nHarmonic-1
  call rhd_mean_SphericalHarmonic(1, nPoints, i, x1(1:nPoints), x2(1:nPoints), x3(1:nPoints), &
                                  Y(1:nPoints,i))
end do ! i
!
! --- Compute amplitude of mode 0 ---
Y0Amp=sqrt(sum(Y(1:nPoints,0)**2))
!
! --- Normalize all modes ---
do i=0,nHarmonic-1
  Y(1:nPoints,i)=Y(1:nPoints,i)/Y0Amp
end do ! i
!
! --- Compute all products ---
do j=0,nHarmonic-1
  do i=0,nHarmonic-1
    M(i,j)=sum(Y(1:nPoints,i)*Y(1:nPoints,j))
  end do ! i
end do ! j
!
! --- Create unity matrix ---
Munity=0.0
do i=0,nHarmonic-1
  Munity(i,i)=1.0
end do ! i
!
TestName=trim(TestPrefix) // '_n=10_dx=1.0_' // &
         'FuncionsAreOrthonormal'
delta=0.1
call UT_Assert('==', TestName, M, Munity, delta=delta)
!
end subroutine rhd_mean_SphericalHarmonicUt


!----------******************-----------------------------------------------------------------------
subroutine rhd_meanUT_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_meanUT_TestAll ('radiation_hydrodynamics_mean_Unit_Test_Test_All')
!
! PURPOSE:
!   Perform unit tests of all averaging routines.
!
! CATEGORY:
!   Averaging, testing
!
! CALLING SEQUENCE:
!   call rhd_meanUT_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_mean_rindexUT:     Perform unit tests for rhd_mean_rindex and rhd_mean_a2rindex
!   rhd_mean_x12meanUT:    Perform unit tests of rhd_mean_x12mean
!   rhd_mean_x23meanUT:    Perform unit tests of rhd_mean_x23mean
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Perform all unit tests of averaging routines.
!
! EXAMPLE:
!   call rhd_meanUT_TestAll()
!
! MODIFICATION HISTORY:
!   2015-03-11 (B.F. Uppsala) First version
!   2017-09-04 (B.F. Lyon) Add unit tests of old and new spherical-shell averages
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='rhd_mean'
!
call rhd_mean_x12meanUT(trim(TestPrefix) // '_x12mean_')
call rhd_mean_x23meanUT(trim(TestPrefix) // '_x23mean_')
!
call rhd_mean_rindexUT( trim(TestPrefix) // '_rindex_' )
!
end subroutine rhd_meanUT_TestAll


!----------*******************----------------------------------------------------------------------
subroutine rhd_meanUT_TestSome()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_meanUT_TestSome ('radiation_hydrodynamics_mean_Unit_Test_Test_Some')
!
! PURPOSE:
!   Perform unit tests of some averaging routines.
!
! CATEGORY:
!   Averaging, testing
!
! CALLING SEQUENCE:
!   call rhd_meanUT_TestSome()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Perform selected unit tests of some averaging routines.
!
! EXAMPLE:
!   call rhd_meanUT_TestSome()
!
! MODIFICATION HISTORY:
!   2015-03-11 (B.F. Uppsala) First version
!   2017-08-26 (B.F. Lyon) Adapted from rhd_meanUT_TestAll
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='rhd_mean'
!call rhd_mean_rindexUT(trim(TestPrefix) // '_rindex_')
call rhd_mean_SphericalHarmonicUT(Testprefix // '__SphericalHarmonic_')
!
end subroutine rhd_meanUT_TestSome

end module rhd_meanUT_module
