!***************************************************************************************************
!
!   #####   #    #  #####          #    #  ######    ##    #    #
!   #    #  #    #  #    #         ##  ##  #        #  #   ##   #
!   #    #  ######  #    #         # ## #  #####   #    #  # #  #
!   #####   #    #  #    #         #    #  #       ######  #  # #
!   #   #   #    #  #    #         #    #  #       #    #  #   ##
!   #    #  #    #  #####  ####### #    #  ######  #    #  #    #_module
!
!   Radiation-Hydrodynamics-Code: averaging module
!
!***************************************************************************************************
!   Fortran90
!   Bernd Freytag:         Copenhagen, Lyon, Uppsala
!   2019-01-31
!***************************************************************************************************
! MODULES:
!   rhd_mean_module:       Averaging routines
!
!***************************************************************************************************

!------***************------------------------------------------------------------------------------
module rhd_mean_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_module ('rhd_mean_module')
!
! PURPOSE:
!   Provide averaging routines for rhd.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_mean_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_weight:       Produce weight array for horizontal averaging
!   rhd_mean_ax23weight:   Produce weight arrays for averaging over spheres
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_arindex:      Produce index arrays for averaging over spheres in a symmetric box
!   rhd_mean_rindex:       Produce index arrays for averaging over spheres in a symmetric box
!   rhd_mean_c1Index:      Produce index array for azimuthal averaging over cylinders
!   rhd_mean_rbmean:       Average boundary-centered vector quantity over spheres in symmetric box
!   rhd_mean_rb12mean:     Average boundary-centered vector quantity over spheres in asymmetric box
!   rhd_mean_rb13mean:     Average boundary-centered vector quantity over spheres in asymmetric box
!   rhd_mean_rb23mean:     Average boundary-centered vector quantity over spheres in asymmetric box
!   rhd_mean_rb23absmean:  Average boundary-centered scalar quantity over spheres in asymmetric box
!   rhd_mean_SphericalHarmonic: Compute real spherical harmonic for points in space
!   rhd_mean_rb23shmean:   Compute spherical-harmonic amplitude of boundary-centered quantity
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_rcmean:       Average cell-centered quantity over spheres in a symmetric box
!   rhd_mean_ccmean        Average cell-centered quantity over cylinders (azimuthal mean)
!   rhd_mean_ccamean:      Average cell-centered quantity over cylinders (azimuthal mean)
!   rhd_mean_VecProj:      Project vector on radial ray
!   rhd_mean_VecbProjUnity: Project radial unity vector on x1 axis direction
!   rhd_mean_c1VecProj:    Project vector on radial vector perpendicular to x1 axis
!
! MODIFICATION HISTORY:
!   2000-01-24 (B.F. Copenhagen) Written
!   2000-02-07 (B.F.) Put into module
!   2013-08-19 (B.F. Lyon) Azimuthal averages over cylindrical surfaces
!   2015-03-21 (B.F. Uppsala) Kind of mean data determined by parameter in module
!   2017-09-05 (B.F. Lyon) Add comments for new routines for spherical averages in an asymmetric box
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_aweight(m2, n2, m3, n3, xb2, xb3, AreaFraction, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_aweight ('rhd_mean_array_weight')
!
! PURPOSE:
!   Produce weight array for horizontal averaging.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_aweight(m2, n2, m3, n3, xb2, xb3, AreaFraction, weight)
!
! INPUT:
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! OUTPUT:
!   weight:       ('weight') 2D real array with weights
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-03-11 (B.F.) Routine not using box
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m2, n2, m3,n3
real, dimension(m2:n2+1),  intent(in)   :: xb2
real, dimension(m3:n3+1),  intent(in)   :: xb3
real,                      intent(in)   :: AreaFraction
real, dimension(m2:n2, &
                m3:n3  ),  intent(out)  :: weight
!
! --- Local variables ---
integer                                 :: i3
real                                    :: normfactor
!---------------------------------------------------------------------------------------------------
!
normfactor=AreaFraction/((xb2(n2+1)-xb2(m2)) * & 
                         (xb3(n3+1)-xb3(m3))) 
!
do i3=m3,n3
  weight(:,i3)=(xb2(m2+1:n2+1)-xb2(m2  :n2  )) * &
               (xb3(i3+1     )-xb3(i3       )) * normfactor
end do ! i3
!
end subroutine rhd_mean_aweight


!----------***************--------------------------------------------------------------------------
subroutine rhd_mean_weight(model, AreaFraction, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_weight ('rhd_mean_weight')
!
! PURPOSE:
!   Produce weight array for horizontal averaging.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_weight(model, AreaFraction, weight)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! OUTPUT:
!   weight:       ('weight') 2D real array with weights
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-02-03 (B.F.) Separate routines for weight and x12weight
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real,                      intent(in)   :: AreaFraction
real, dimension(model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(out)  :: weight
!
! --- Local variables ---
integer                                 :: i3
real                                    :: normfactor
!---------------------------------------------------------------------------------------------------
!
normfactor=AreaFraction/((model%xb2(model%n2+1)-model%xb2(model%m2)) * & 
                         (model%xb3(model%n3+1)-model%xb3(model%m3))) 
!
do i3=model%m3,model%n3
  weight(:,i3)=(model%xb2(model%m2+1:model%n2+1)- &
                model%xb2(model%m2  :model%n2  ) ) * &
               (model%xb3(i3+1)-model%xb3(i3)) * normfactor
end do ! i3
!
end subroutine rhd_mean_weight


!----------*******************----------------------------------------------------------------------
subroutine rhd_mean_ax23weight(m1,n1, m2,n2, m3,n3, xb1, xb2, xb3, scalelength, &
                               x23weight, x123weight, factor)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_ax23weight ('rhd_mean_array_x23_weight')
!
! PURPOSE:
!   Produce weight arrays for averaging over spheres.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_ax23weight(m1,n1, m2,n2, m3,n3, xb1, xb2, xb3, scalelength, &
!                            x23weight, x123weight, factor=factor)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   scalelength:  ('scale_length') real, typical length scale
!   factor:       ('factor') optional real, normalization factor, default=1.0
!
! OUTPUT:
!   x23weight:    ('x_23_weight') 2D real array with weights
!   x123weight:   ('x_123_weight') 3D real array with weights
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-24 (B.F. Copenhagen) Written
!   2000-01-28 (B.F.) x123weight
!   2000-02-03 (B.F.) Parameter scalelength added
!   2000-02-11 (B.F.) Keyword factor added
!   2001-08-29 (B.F.) x123weight not optional anymore, no box input
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1, n1, m2, n2, m3,n3
real, dimension(m1:n1+1),  intent(in)   :: xb1
real, dimension(m2:n2+1),  intent(in)   :: xb2
real, dimension(m3:n3+1),  intent(in)   :: xb3
real,                      intent(in)   :: scalelength
real, dimension(m2:n2, &
                m3:n3),    intent(out)  :: x23weight
real, dimension(m1:n1, &
                m2:n2, &
                m3:n3),    intent(out)  :: x123weight
real,            optional, intent(in)   :: factor
!
! --- Local variables ---
integer                                 :: i1, i3
real                                    :: normfactor
!---------------------------------------------------------------------------------------------------
!
if (present(factor)) then
  normfactor=factor/scalelength**2
else
  normfactor=1.0   /scalelength**2
endif
!
do i3=m3,n3
  x23weight(:,i3)=(xb2(m2+1:n2+1)-xb2(m2  :n2  )) * &
                  (xb3(i3+1     )-xb3(i3       )) * normfactor
end do ! i3
!
normfactor=1.0/scalelength
do i1=m1,n1
  x123weight(i1,:,:)=x23weight * (xb1(i1+1)-xb1(i1)) * normfactor
end do ! i1
!
end subroutine rhd_mean_ax23weight


!----------******************-----------------------------------------------------------------------
subroutine rhd_mean_x23weight(model, scalelength, x23weight, x123weight, factor)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_x23weight ('rhd_mean_x23_weight')
!
! PURPOSE:
!   Produce weight arrays for averaging over spheres.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_x23weight(model, scalelength, x23weight, x123weight, factor=factor)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   scalelength:  ('scale_length') real, typical length scale
!   factor:       ('factor') optional real, normalization factor, default=1.0
!
! OUTPUT:
!   x23weight:    ('x_23_weight') 2D real array with weights
!   x123weight:   ('x_123_weight') 3D real array with weights
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-24 (B.F. Copenhagen) Written
!   2000-01-28 (B.F.) x123weight
!   2000-02-03 (B.F.) Parameter scalelength added
!   2000-02-11 (B.F.) Keyword factor added
!   2001-08-29 (B.F.) x123weight not optional anymore
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real,                      intent(in)   :: scalelength
real, dimension(model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(out)  :: x23weight
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(out)  :: x123weight
real,            optional, intent(in)   :: factor
!
! --- Local variables ---
integer                                 :: i1, i3
real                                    :: normfactor
!---------------------------------------------------------------------------------------------------
!
if (present(factor)) then
  normfactor=factor/scalelength**2
else
  normfactor=1.0   /scalelength**2
endif
!
do i3=model%m3,model%n3
  x23weight(:,i3)=(model%xb2(model%m2+1:model%n2+1)- &
                   model%xb2(model%m2  :model%n2  ) ) * &
                  (model%xb3(i3+1)-model%xb3(i3)) * normfactor
end do ! i3
!
normfactor=1.0/scalelength
do i1=model%m1,model%n1
  x123weight(i1,:,:)=x23weight * (model%xb1(i1+1)-model%xb1(i1)) * normfactor
end do ! i1
!
end subroutine rhd_mean_x23weight


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_x12mean(q, qmean, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_x12mean ('rhd_mean_x12_mean')
!
! PURPOSE:
!   Average quantity horizontally in x12 plane.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_x12mean(q, qmean, weight)
!
! INPUT:
!   q:            ('quantity') 3D real array with input data
!   weight:       ('weight') 2D real array with weights
!
! OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   size(q,1) = size(weight,1)
!   size(q,2) = size(weight,2)
!   size(q,3) = size(qmean)
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-24 (B.F. Copenhagen) Written
!   2001-02-13 (B.F. Uppsala) Index order changed, weight not optional anymore
!   2001-03-11 (B.F.) x23 -> x12
!   2001-05-24 (B.F.) intent(out) -> intent(inout)
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: q
real(kind=mArk), &
      dimension(    :),    intent(inout):: qmean
real, dimension(:,:  ),    intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: m3,n3, i3
!---------------------------------------------------------------------------------------------------
!
m3=1
n3=size(q,3)
!
do i3=m3,n3
  qmean(i3)=qmean(i3)+sum(q(:,:,i3)*weight)
end do ! i3
!
end subroutine rhd_mean_x12mean


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_x13mean(q, qmean, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_x13mean ('rhd_mean_x13_mean')
!
! PURPOSE:
!   Average quantity horizontally in x13 plane.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_x13mean(q, qmean, weight)
!
! INPUT:
!   q:            ('quantity') 3D real array with input data
!   weight:       ('weight') 2D real array with weights
!
! OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   size(q,1) = size(weight,1)
!   size(q,2) = size(qmean)
!   size(q,3) = size(weight,2)
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-24 (B.F. Copenhagen) Written
!   2001-02-13 (B.F. Uppsala) Index order changed, weight not optional anymore
!   2001-03-11 (B.F.) x23 -> x12
!   2001-05-22 (B.F.) x12 -> x13
!   2001-05-24 (B.F.) intent(out) -> intent(inout)
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: q
real(kind=mArk), &
      dimension(  :  ),    intent(inout):: qmean
real, dimension(:,  :),    intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: m2,n2, i2
!---------------------------------------------------------------------------------------------------
!
m2=1
n2=size(q,2)
!
do i2=m2,n2
  qmean(i2)=qmean(i2)+sum(q(:,i2,:)*weight)
end do ! i2
!
end subroutine rhd_mean_x13mean


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_x23mean(q, qmean, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_x23mean ('rhd_mean_x23_mean')
!
! PURPOSE:
!   Average quantity horizontally in x23 plane.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_x23mean(q, qmean, weight=weight)
!
! INPUT:
!   q:            ('quantity') 3D real array with input data
!   weight:       ('weight') 2D real array with weights
!
! OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input arrays have to be compatible:
!     size(q,1) = size(qmean)
!     size(q,2) = size(weight,1)
!     size(q,3) = size(weight,2)
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-24 (B.F. Copenhagen) Written
!   2001-02-13 (B.F. Uppsala) Index order changed, weight not optional anymore
!   2001-05-24 (B.F.) intent(out) -> intent(inout)
!   2015-03-11 (B.F.) Add up averaging results first into temporary array
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: q
real(kind=mArk), &
      dimension(:    ),    intent(inout):: qmean
real, dimension(  :,:),    intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1,i2,i3
real                                    :: factor
real, dimension(1:size(q,1))            :: qmtmp
!---------------------------------------------------------------------------------------------------
!
m1=1
n1=size(q,1)
m2=1
n2=size(q,2)
m3=1
n3=size(q,3)
!
!!$do i3=m3,n3
!!$  do i2=m2,n2
!!$    factor=weight(i2,i3)
!!$    do i1=m1,n1
!!$      qmean(i1)=qmean(i1) + q(i1,i2,i3)*factor
!!$    end do ! i1
!!$  end do ! i2
!!$end do ! i3
!
qmtmp=0.0
do i3=m3,n3
  do i2=m2,n2
    factor=weight(i2,i3)
    do i1=m1,n1
      qmtmp(i1)=qmtmp(i1) + q(i1,i2,i3)*factor
    end do ! i1
  end do ! i2
end do ! i3
qmean=qmean+qmtmp
!
end subroutine rhd_mean_x23mean


!----------*****************------------------------------------------------------------------------
subroutine rhd_mean_a2rindex(m1,n1, m2,n2, m3,n3, nghost1, m1_global,n1_global, &
                             xc1, xc2, xc3, xflux, &
                             nb6index, bindex, cindex)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_a2rindex ('rhd_mean_array_2_radius_index')
!
! PURPOSE:
!   Produce index arrays for averaging over spheres in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_a2rindex(m1,n1, m2,n2, m3,n3, nghost1, m1_global,n1_global, &
!                          xc1, xc2, xc3, xflux, &
!                          nb6index, bindex, cindex)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1st dimension, number of ghost cells (> 0)
!   m1_global:    ('m_1_global') integer, 1st dimension, global start index
!   n1_global:    ('n_1_global') integer, 1st dimension, global end index
!   xc1:          ('x_center_1') real, 1. coordinate of cell centers
!   xc2:          ('x_center_2') real, 2. coordinate of cell centers
!   xc3:          ('x_center_3') real, 3. coordinate of cell centers
!   xflux:        ('x_flux') real vector with radius values for flux calculation
!
! OUTPUT:
!   nb6index:     ('number_boundary_6_index') 3D integer array
!   bindex:       ('boundary_index') 3D integer array
!   cindex:       ('center_index') 3D integer array
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that
!     xflux(1)=0.0 (center) .
!   No flux is computed here.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-26 (B.F. Copenhagen) Written
!   2000-02-07 (B.F.) Corrections
!   2001-08-29 (B.F.) Version without box input
!   2001-09-04 (B.F.) m1_global, n1_global
!   2017-08-27 (B.F. Lyon) Change initial cindex values from 1 to -1
!   2017-08-28 (B.F.) Merge index arrays to single array nb6index
!   2017-09-02 (B.F.) First version that passes a number of unit tests
!---------------------------------------------------------------------------------------------------
!!$use uio_bulk_module
!!$use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1, n1, m2, n2, m3,n3, nghost1, m1_global, n1_global
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: xc1
real, dimension(m2:n2),    intent(in)   :: xc2
real, dimension(m3:n3),    intent(in)   :: xc3
real, dimension(:),        intent(in)   :: xflux
integer, dimension(6, &
                   m2:n2, &
                   m3:n3), intent(out)  :: nb6index
integer, dimension(-size(xflux):size(xflux), &
                   m2:n2, &
                   m3:n3), intent(out)  :: bindex
integer, dimension(m1:n1, &
                   m2:n2, &
                   m3:n3), intent(out)  :: cindex
!
! --- Local variables ---
integer                                 :: nxflux, ixflux, numxflux, i1, i2, i3, m1l, m1l0, cil, cir
real                                    :: sqr_xc23, sqr_xc3, sqr_r
real, dimension(size(xflux))            :: sqr_xflux
!!$integer                                 :: nc
integer, dimension(m2:n2, &
                   m3:n3)               :: nbindex
!---------------------------------------------------------------------------------------------------
!
nxflux   =size(xflux)
sqr_xflux=xflux**2
!
! --- Fill index arrays with something: dummy values for cindex and bindex, ---
! --- outer shell for nb6index                                              ---
cindex  =-1
bindex  =m1-100
nb6index(1,:,:)=nxflux
nb6index(2,:,:)=nxflux
nb6index(3,:,:)=0
nb6index(4,:,:)=nxflux
nb6index(5,:,:)=nxflux
nb6index(6,:,:)=0
!
! --- Find i1 coordinate value of point in every column which is closest to center ---
m1l =minval(minloc(abs(xc1(m1  :n1  ))))-1+m1
m1l0=minval(minloc(abs(xc1(m1-1:n1+1))))-1+m1-1
bindex(0,:,:)=m1l
!
! --- Loop over columns in x3 direction ---
do i3=m3,n3
  sqr_xc3=xc3(i3)**2
  !
  ! --- Loop over columns in x2 direction ---
  do i2=m2,n2
    sqr_xc23=sqr_xc3+xc2(i2)**2
    !
    ! --- Get number of flux points further away than this point ---
    numxflux=count(sqr_xflux(2:) > (sqr_xc23+xc1(m1l)**2))
    nbindex(i2,i3)=numxflux
    nb6index(1,i2,i3)=nxflux-numxflux+1
    nb6index(4,i2,i3)=nxflux-numxflux+1
    !
    if (numxflux > 0) then
      ! --- Sphere with maximum radius has points within this column ---
      !
      ! === Handle right/rear/upper part of sphere =================================================
      !
      ixflux=nxflux
      cir=nxflux+5
      cil=nxflux+5
      !
      ! --- Loop over right/rear/upper part of sphere ---
      do i1=n1+nghost1+1, m1l, -1
        if (numxflux < nbindex(i2,i3)) then
          if (i1 <= n1) then
            cindex(i1,i2,i3)=ixflux
            !
            if (i1 == n1) then
              ! --- Rightmost inside point: Check the outside point on the right ---
              if ((sqr_xc23+xc1(i1+1)**2) > sqr_xflux(ixflux+1)) then
                cir=ixflux+1
              else
                cir=ixflux
              endif
              !write(6,'(A,3I3,2X,3I3)') 'i1,i2,i3, n1, ixflux, cirr=', i1,i2,i3, n1, ixflux, cir
            endif
            !
            if (i1 == (m1l)) then
              ! --- Most central inside point: check if central point is included ---
              if (m1l > m1l0) then
                ! --- Take current point ---
                cil=ixflux
              else
                ! --- Check the next point on the left ---
                if ((sqr_xc23+xc1(i1  )**2) < sqr_xflux(ixflux)) then
                  cil=ixflux-1
                else
                  cil=ixflux
                endif
              endif
              !write(6,'(A,3I3,2X,3I3)') 'i1,i2,i3, n1, ixflux, cirl=', i1,i2,i3, n1, ixflux, cil
            endif
            !
          endif ! (i1 <= n1)
        endif ! (numxflux < nbindex(i2,i3))
        !
        sqr_r=sqr_xc23+xc1(i1-1)**2
        !
        ! --- Loop over all boundary-value spheres to exit at the first one that lies within ---
        NUMXFLUXR_LOOP: do while (numxflux > 0)
          if (sqr_xflux(ixflux) < sqr_r) then
            exit NUMXFLUXR_LOOP
          endif
          !
          bindex(ixflux,i2,i3)=i1
          !
          if (i1 > (n1+1)) then
            if (n1 == n1_global) then
              ! --- Use flux at boundary of box ---
              bindex(ixflux,i2,i3)=n1+1
              !write(6,'(A,3I3,2X,3I3,2X,I3)') &
              !  'a2r_r: i1,i2,i3, m1,n1,n1_global, ixflux=', &
              !  i1,i2,i3, m1, n1_global, ixflux
            else
              ! --- Ignore this flux ---
              nb6index(5,i2,i3)=nb6index(5,i2,i3)-1
              !write(6,'(A,3I3,2X,3I3,2X,I3)') &
              !  'a2r_r: i1,i2,i3, m1,n1,n1_global, nb6index(5,i2,i3)=', &
              !  i1,i2,i3, m1,n1,n1_global, nb6index(5,i2,i3)
            endif
          endif
          !
          numxflux=numxflux-1
          ixflux=nxflux-nbindex(i2,i3)+numxflux
        end do NUMXFLUXR_LOOP
        !
      end do ! i1
      !
      ! --- Find the left (inside) flux boundary ---
      i1=m1l+1
      CIRL_LOOP: do while (i1 <= n1)
        if ((cindex(i1,i2,i3) >= 0) .and. (cindex(i1,i2,i3) > cil)) then
          nb6index(4,i2,i3)=cindex(i1,i2,i3)
          exit CIRL_LOOP
        endif
        !
        i1=i1+1
      end do CIRL_LOOP
      !
      ! --- Find the right (outside) flux boundary ---
      if (nb6index(4,i2,i3) > nxflux) then
        ! --- Left boundary lies outside flux-index interval: there are no points ---
        nb6index(6,i2,i3)=nxflux
      else if (n1 == n1_global) then
        ! --- Right box boundary is global boundary and there are points inside ---
        nb6index(6,i2,i3)=nxflux
      else
        i1=n1
        CIRR_LOOP: do while (i1 >= m1l)
          if ((cindex(i1,i2,i3) >= 0) .and. (cindex(i1,i2,i3) < cir)) then
            nb6index(6,i2,i3)=cindex(i1,i2,i3)+1
            exit CIRR_LOOP
          endif
          !
          i1=i1-1
        end do CIRR_LOOP
      endif
      !
      !
      ! === Handle left/front/lower part of sphere =================================================
      !
      numxflux=nbindex(i2,i3)
      ixflux=nxflux
      cir=nxflux+5
      cil=nxflux+5
      !
      ! --- Loop over left/front/lower part of sphere ---
      do i1=m1-nghost1, m1l+1
        if (numxflux < nbindex(i2,i3)) then
          if (i1 > m1) then
            cindex(i1-1,i2,i3)=ixflux
            !
            if (i1 == (m1+1)) then
              ! --- Leftmost inside point: Check the outside point on the left ---
              if ((sqr_xc23+xc1(i1-2)**2) > sqr_xflux(ixflux+1)) then
                cil=ixflux+1
              else
                cil=ixflux
              endif
              ! write(6,'(A,3I3,2X,3I3)') 'i1,i2,i3, n1, ixflux, cill=', i1,i2,i3, n1, ixflux, cil
            endif
            !
            if (i1 == (m1l+1)) then
              ! --- Most central inside point: check if central point is included ---
              if (m1l < m1l0) then
                ! --- Take current point ---
                cir=ixflux
              else
                ! --- Check the next point on the right ---
                if ((sqr_xc23+xc1(i1  )**2) < sqr_xflux(ixflux)) then
                  cir=ixflux-1
                else
                  cir=ixflux
                endif
              endif
              ! write(6,'(A,3I3,2X,3I3)') 'i1,i2,i3, n1, ixflux, cilr=', i1,i2,i3, n1, ixflux, cir
            endif
            !
          endif ! (i1 > m1)
        endif ! (numxflux < nbindex(i2,i3))
        !
        sqr_r=sqr_xc23+xc1(i1)**2
        !
        ! --- Loop over all boundary-value spheres to exit at the first one that lies within ---
        NUMXFLUXL_LOOP: do while (numxflux > 0)
          if (sqr_xflux(ixflux) < sqr_r) then
            exit NUMXFLUXL_LOOP
          endif
          !
          bindex(-ixflux,i2,i3)=i1
          !
          if (i1 < m1) then
            if (m1 == m1_global) then
              ! --- Use flux at boundary of box ---
              bindex(-ixflux,i2,i3)=m1
              ! write(6,'(A,3I3,2X,3I3,2X,2I3)') &
              !   'a2r_l: i1,i2,i3, m1, m1_global, ixflux, cil,r=', &
              !   i1,i2,i3, m1, m1_global, ixflux, cil, cir
            else
              ! --- Ignore this flux ---
              nb6index(2,i2,i3)=nb6index(2,i2,i3)-1
              ! write(6,'(A,3I3,2X,3I3,2X,2I3)') &
              !   'a2r_l: i1,i2,i3, m1, m1_global, nb6index(2,i2,i3), cil,r=', &
              !   i1,i2,i3, m1, m1_global, nb6index(2,i2,i3), cil, cir
            endif
          endif
          !
          numxflux=numxflux-1
          ixflux=nxflux-nbindex(i2,i3)+numxflux
        end do NUMXFLUXL_LOOP
        !
      end do ! i1
      !
      ! --- Find the right (inside) flux boundary ---
      i1=m1l
      CILR_LOOP: do while (i1 >= m1)
        if ((cindex(i1,i2,i3) >= 0) .and. (cindex(i1,i2,i3) > cir)) then
          nb6index(1,i2,i3)=cindex(i1,i2,i3)
          exit CILR_LOOP
        endif
        !
        i1=i1-1
      end do CILR_LOOP
      !
      ! --- Find the left (outside) flux boundary ---
      if (nb6index(1,i2,i3) > nxflux) then
        ! --- Right boundary lies outside flux-index interval: there are no points ---
        nb6index(3,i2,i3)=nxflux
      else if (m1 == m1_global) then
        ! --- Left box boundary is global boundary and there are points inside ---
        nb6index(3,i2,i3)=nxflux
      else
        i1=m1
        CILL_LOOP: do while (i1 <= m1l)
          if ((cindex(i1,i2,i3) >= 0) .and. (cindex(i1,i2,i3) < cil)) then
            nb6index(3,i2,i3)=cindex(i1,i2,i3)+1
            exit CILL_LOOP
          endif
          !
          i1=i1+1
        end do CILL_LOOP
      endif
      !
    endif ! (numx_flux > 0)
  end do ! i2
end do ! i3
!
!!$call uio_openwr(nc, 'rhd_mean_a2rindex_test.out', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now'    )
!!$call uio_wrlabl(nc, box_ident                    )
!!$call uio_wr(    nc, 'z'          , box_id_ident  )
!!$call uio_wr(    nc, m1           , 'm1'          )
!!$call uio_wr(    nc, n1           , 'n1'          )
!!$call uio_wr(    nc, m2           , 'm2'          )
!!$call uio_wr(    nc, n2           , 'n2'          )
!!$call uio_wr(    nc, m3           , 'm3'          )
!!$call uio_wr(    nc, n3           , 'n3'          )
!!$call uio_wr(    nc, xc1          , 'xc1'         )
!!$call uio_wr(    nc, xc2          , 'xc2'         )
!!$call uio_wr(    nc, xc3          , 'xc3'         )
!!$call uio_wr(    nc, nghost1      , 'nghost1'     )
!!$call uio_wr(    nc, m1_global    , 'm1_global'   )
!!$call uio_wr(    nc, n1_global    , 'n1_global'   )
!!$call uio_wr(    nc, nbindex      , 'nbindex'     )
!!$call uio_wr(    nc, nb6index     , 'nb6index'    )
!!$call uio_wr(    nc, bindex       , 'bindex'      )
!!$call uio_wr(    nc, cindex       , 'cindex'      )
!!$call uio_wrlabl(nc, endbox_ident                 )
!!$call uio_wrlabl(nc, enddataset_ident             )
!!$call uio_closrd(nc)
!
end subroutine rhd_mean_a2rindex


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_arindex(m1,n1, m2,n2, m3,n3, xc1, xc2, xc3, xflux, nbindex, bindex, cindex)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_arindex ('rhd_mean_array_radius_index')
!
! PURPOSE:
!   Produce index arrays for averaging over spheres in a symmetric box.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_arindex(m1,n1, m2,n2, m3,n3, xc1, xc2, xc3, xflux, &
!                         nbindex, bindex, cindex)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xc1:          ('x_center_1') real, 1. coordinate of cell centers
!   xc2:          ('x_center_2') real, 2. coordinate of cell centers
!   xc3:          ('x_center_3') real, 3. coordinate of cell centers
!   xflux:        ('x_flux') real vector with radius values for flux calculation
!
! OUTPUT:
!   nbindex:      ('number_boundary_index') 2D integer array
!   bindex:       ('boundary_index') 3D integer array
!   cindex:       ('center_index') 3D integer array
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that
!     xflux(1)=0.0 (center) .
!   No flux is computed here.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-26 (B.F. Copenhagen) Written
!   2000-02-07 (B.F.) Corrections
!   2001-08-29 (B.F. Uppsala) Version without box input
!   2017-08-27 (B.F. Lyon) Change initial cindex values from 1 to -1
!---------------------------------------------------------------------------------------------------
!!$use uio_bulk_module
!!$use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1, n1, m2, n2, m3,n3
real, dimension(m1:n1),    intent(in)   :: xc1
real, dimension(m2:n2),    intent(in)   :: xc2
real, dimension(m3:n3),    intent(in)   :: xc3
real, dimension(:),        intent(in)   :: xflux
integer, dimension(m2:n2, &
                   m3:n3), intent(out)  :: nbindex
integer, dimension(-size(xflux):size(xflux), &
                   m2:n2, &
                   m3:n3), intent(out)  :: bindex
integer, dimension(m1:n1, &
                   m2:n2, &
                   m3:n3), intent(out)  :: cindex
!
! --- Local variables ---
integer                                 :: nxflux, ixflux, numxflux, i1, i2, i3, m1l
real                                    :: sqr_xc23, sqr_xc3, sqr_r
real, dimension(size(xflux))            :: sqr_xflux
integer                                 :: nc
!---------------------------------------------------------------------------------------------------
!
nxflux   =size(xflux)
sqr_xflux=xflux**2
!
! --- Fill index arrays with something ---
cindex=-1
bindex=m1-100
!
! --- Find i1 coordinate value of point in every column which is closest to center ---
m1l=minval(minloc(abs(xc1(m1:n1))))-1+m1
bindex(0,:,:)=m1l
!
! --- Loop over columns in x3 direction ---
do i3=m3,n3
  sqr_xc3=xc3(i3)**2
  !
  ! --- Loop over columns in x2 direction ---
  do i2=m2,n2
    sqr_xc23=sqr_xc3+xc2(i2)**2
    !
    ! --- Get number of flux points further away than this point ---
    numxflux=count(sqr_xflux(2:) > (sqr_xc23+xc1(m1l)**2))
    nbindex(i2,i3)=numxflux
    ixflux=nxflux-nbindex(i2,i3)+numxflux
    !
    if (numxflux > 0) then
      ! --- Sphere with maximum radius has points within this column ---
      !
      ! --- Loop over right/rear/upper part of sphere ---
      i1=n1+1
      do while ((i1 >= m1l) .and. (i1 > m1))
        if (numxflux < nbindex(i2,i3)) cindex(i1,i2,i3)=ixflux
        !
        sqr_r=sqr_xc23+xc1(i1-1)**2
        !
        ! --- Loop over all boundary-value spheres that go through this boundary ---
        NUMXFLUXR_LOOP: do while (numxflux > 0)
          if (sqr_xflux(ixflux) < sqr_r) then
            exit NUMXFLUXR_LOOP
          endif
          bindex(ixflux,i2,i3)=i1
          numxflux=numxflux-1
          ixflux=nxflux-nbindex(i2,i3)+numxflux
        end do NUMXFLUXR_LOOP
        !
        i1=i1-1
      end do ! i1
      !
      ! --- Handle left/front/lower part of sphere ---
      !
      numxflux=nbindex(i2,i3)
      ixflux=nxflux-nbindex(i2,i3)+numxflux
      !
      ! --- Loop over left/front/lower part of sphere ---
      i1=m1
      do while (i1 <= m1l)
        if (numxflux < nbindex(i2,i3)) cindex(i1-1,i2,i3)=ixflux
        !
        sqr_r=sqr_xc23+xc1(i1)**2
        !
        ! --- Loop over all boundary-value spheres that go through this boundary ---
        NUMXFLUXL_LOOP: do while (numxflux > 0)
          ixflux=nxflux-nbindex(i2,i3)+numxflux
          if (sqr_xflux(ixflux) < sqr_r) then
            exit NUMXFLUXL_LOOP
          endif
          !
          bindex(-ixflux,i2,i3)=i1
          !
          numxflux=numxflux-1
        end do NUMXFLUXL_LOOP
        ixflux=nxflux-nbindex(i2,i3)+numxflux
        !
        i1=i1+1
      end do ! i1
      !
    endif ! (numx_flux > 0)
  end do ! i2
end do ! i3
!
!!$call uio_openwr(nc, 'rhd_mean_arindex_test.out', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now')
!!$call uio_wrlabl(nc, box_ident                )
!!$call uio_wr(nc, 'z'          , box_id_ident  )
!!$call uio_wr(nc, xc1          , 'xc1'         )
!!$call uio_wr(nc, xc2          , 'xc2'         )
!!$call uio_wr(nc, xc3          , 'xc3'         )
!!$call uio_wr(nc, nbindex      , 'nbindex'     )
!!$call uio_wr(nc, bindex       , 'bindex'      )
!!$call uio_wr(nc, cindex       , 'cindex'      )
!!$call uio_wrlabl(nc, endbox_ident             )
!!$call uio_wrlabl(nc, enddataset_ident         )
!!$call uio_closrd(nc)
!
end subroutine rhd_mean_arindex


!----------***************--------------------------------------------------------------------------
subroutine rhd_mean_rindex(model, xflux, nbindex, bindex, cindex)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rindex ('rhd_mean_radius_index')
!
! PURPOSE:
!   Produce index arrays for averaging over spheres in a symmetric box.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_rindex(model, xflux, nbindex, bindex, cindex)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   xflux:        ('x_flux') real vector with radius values for flux calculation
!
! OUTPUT:
!   nbindex:      ('number_boundary_index') 2D integer array
!   bindex:       ('boundary_index') 3D integer array
!   cindex:       ('center_index') 3D integer array
!
! VARIABLES:
!   ixflux:       ('integer_x_flux') integer with current xflux element
!   nxflux:       ('number_x_flux') integer with size of xflux vector
!   numxflux:     ('number_x_flux') integer with number of xflux points beyond
!                                  (xc1(i1),xc2(i2),xc3(i3))
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   sqr_r:        ('square_radius') real with square of radius
!   sqr_xc23:     ('square_x_center_2_3') real with xc2(i2)**2+xc3(i3)**2
!   sqr_xc3:      ('square_x_center_3') real with square of xc3(i3)
!   sqr_xflux:    ('square_x_flux') real vector with square of xflux vector
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that
!     xflux(1)=0.0 (center) .
!   No flux is computed here.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-26 (B.F. Copenhagen) Written
!   2000-02-07 (B.F.) Corrections
!   2017-08-27 (B.F. Lyon) Change initial cindex values from 1 to -1
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real, dimension(:),        intent(in)   :: xflux
integer, dimension(model%m2:model%n2, &
                   model%m3:model%n3), &
                           intent(out)  :: nbindex
integer, dimension(-size(xflux):size(xflux), &
                   model%m2:model%n2, &
                   model%m3:model%n3), &
                           intent(out)  :: bindex
integer, dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3), &
                           intent(out)  :: cindex
!
! --- Local variables ---
integer                                 :: nxflux, ixflux, numxflux, i1, i2, i3, m1l
real                                    :: sqr_xc23, sqr_xc3, sqr_r
real, dimension(size(xflux))            :: sqr_xflux
!!$integer                                 :: nc
!---------------------------------------------------------------------------------------------------
!
nxflux   =size(xflux)
sqr_xflux=xflux**2
!
! --- Fill index array with something ---
cindex=-1
bindex=model%m1-100
!
! --- Determine i1 coordinate value of point in every column which is closest to center ---
! --- ToDo: why the minval()? ---
m1l          =minval(minloc(abs(model%xc1(model%m1:model%n1))))-1+model%m1
bindex(0,:,:)=m1l
!
! --- Loop over columns in x3 direction ---
do i3=model%m3,model%n3
  sqr_xc3=model%xc3(i3)**2
  !
  ! --- Loop over columns in x2 direction ---
  do i2=model%m2,model%n2
    sqr_xc23=sqr_xc3+model%xc2(i2)**2
    !
    ! --- Number of flux points further away than this point ---
    numxflux      =count(sqr_xflux(2:) > (sqr_xc23+model%xc1(m1l)**2))
    nbindex(i2,i3)=numxflux
    ixflux        =nxflux-nbindex(i2,i3)+numxflux
    !
    if (numxflux > 0) then
      ! --- Sphere with maximum radius has points within this column ---
      !
      ! --- Loop over right part of sphere ---
      i1=model%n1+1
      do while ((i1 >= m1l) .and. (i1 > model%m1))
        if (numxflux < nbindex(i2,i3)) cindex(i1,i2,i3)=ixflux
        !
        sqr_r=sqr_xc23+model%xc1(i1-1)**2
        !
        ! --- Loop over all boundary-value spheres that go through this boundary ---
        NUMXFLUXR_LOOP: do while (numxflux > 0)
          if (sqr_xflux(ixflux) < sqr_r) then
            exit NUMXFLUXR_LOOP
          endif
          !
          bindex(ixflux,i2,i3)=i1
          numxflux            =numxflux-1
          ixflux              =nxflux-nbindex(i2,i3)+numxflux
        end do NUMXFLUXR_LOOP
        !
        i1=i1-1
      end do
      !
      numxflux=nbindex(i2,i3)
      ixflux  =nxflux-nbindex(i2,i3)+numxflux
      !
      ! --- Loop over left part of sphere ---
      i1=model%m1
      do while (i1 <= m1l)
        if (numxflux < nbindex(i2,i3)) cindex(i1-1,i2,i3)=ixflux
        !
        sqr_r=sqr_xc23+model%xc1(i1)**2
        !
        ! --- Loop over all boundary-value spheres that go through this boundary ---
        NUMXFLUXL_LOOP: do while (numxflux > 0)
          ixflux=nxflux-nbindex(i2,i3)+numxflux
          if (sqr_xflux(ixflux) < sqr_r) then
            exit NUMXFLUXL_LOOP
          endif
          !
          bindex(-ixflux,i2,i3)=i1
          numxflux             =numxflux-1
        end do NUMXFLUXL_LOOP
        ixflux                 =nxflux-nbindex(i2,i3)+numxflux
        !
        i1=i1+1
      end do
      !
    endif ! (numx_flux > 0)
  end do ! i2
end do ! i3
!
!!$call uio_openwr(nc, 'rhd_mean_rindex_test.out', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now')
!!$call uio_wrlabl(nc, box_ident                )
!!$call uio_wr(nc, 'z'          , box_id_ident  )
!!$call uio_wr(nc, model%xc1    , 'xc1'         )
!!$call uio_wr(nc, model%xb1    , 'xb1'         )
!!$call uio_wr(nc, model%xc2    , 'xc2'         )
!!$call uio_wr(nc, model%xb2    , 'xb2'         )
!!$call uio_wr(nc, model%xc3    , 'xc3'         )
!!$call uio_wr(nc, model%xb3    , 'xb3'         )
!!$call uio_wr(nc, nbindex      , 'nbindex'     )
!!$call uio_wr(nc, bindex       , 'bindex'      )
!!$call uio_wr(nc, cindex       , 'cindex'      )
!!$call uio_wrlabl(nc, endbox_ident             )
!!$call uio_wrlabl(nc, enddataset_ident         )
!!$call uio_closrd(nc)
!
end subroutine rhd_mean_rindex


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_c1Index(model, xflux, ccindex)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_c1Index ('rhd_mean_cylindrical_3_index')
!
! PURPOSE:
!   Produce index array for azimuthal averaging over cylinders.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_c1Index(model, xflux, ccindex)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   x_flux:       ('x_flux') real vector with radius values for flux calculation
!
! OUTPUT:
!   ccindex:      ('cylinder_center_index') 2D integer array with indices of cylinder columns
!
! VARIABLES:
!   ixflux:       ('integer_x_flux') integer with current xflux element
!   nbindex:      ('number_boundary_index') integer
!   nxflux:       ('number_x_flux') integer with size of xflux vector
!   numxflux:     ('number_x_flux') integer with number of xflux points beyond (xc2(i2),xc3(i3))
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   sqr_r:        ('square_radius') real with square of distance to cylinder axis
!   sqr_xc3:      ('square_x_center_3') real with square of xc3(i3)
!   sqr_xflux:    ('square_x_flux') real vector with square of xflux vector
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that
!     xflux(1)=0.0 (center) .
!   No flux is computed here.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-08-07 (B.F. Lyon) Derived from rhd_mean_rindex
!---------------------------------------------------------------------------------------------------
!!$use uio_bulk_module
!!$use uio_filedef_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real, dimension(:),        intent(in)   :: xflux
integer, dimension(model%m2:model%n2, &
                   model%m3:model%n3), &
                           intent(out)  :: ccindex
!
! --- Local variables ---
integer                                 :: nxflux, ixflux, numxflux, i2, i3, nbindex
real                                    :: sqr_xc3, sqr_r
real, dimension(size(xflux))            :: sqr_xflux
!!$integer                                 :: nc
!---------------------------------------------------------------------------------------------------
!
nxflux   =size(xflux)
sqr_xflux=xflux**2
!
! --- Fill index array with value indicating no corresponding index ---
!ccindex=-1
!
! --- Loop over columns in x3 direction ---
do i3=model%m3,model%n3
  sqr_xc3=model%xc3(i3)**2
  !
  ! --- Loop over columns in x2 direction ---
  do i2=model%m2,model%n2
    sqr_r=sqr_xc3+model%xc2(i2)**2
    !
    ! --- Count number of flux points further away than this point ---
    numxflux=count(sqr_xflux > sqr_r)
    !
    ! --- Compute target cylinder index for current grid point ---
    if (numxflux > 0) then
      ccindex(i2,i3)=nxflux-numxflux
    else
      ccindex(i2,i3)=0
    endif
!!$    nbindex       =numxflux
!!$    !ixflux        =nxflux-nbindex+numxflux
!!$    ixflux        =nxflux
!!$    !
!!$    if (numxflux > 0) then
!!$      ! --- Cylinder with maximum radius has points within this column ---
!!$      !
!!$      ! --- Loop over all boundary-value spheres that go through this boundary ---
!!$      NUMXFLUXR_LOOP: do while (numxflux > 0)
!!$        if (sqr_xflux(ixflux) < sqr_r) then
!!$          exit NUMXFLUXR_LOOP
!!$        endif
!!$        !
!!$        numxflux            =numxflux-1
!!$        ixflux              =nxflux-nbindex+numxflux
!!$      end do NUMXFLUXR_LOOP
!!$      !
!!$      numxflux=nbindex
!!$      !ixflux  =nxflux-nbindex+numxflux
!!$      ixflux  =nxflux
!!$      !
!!$      if (numxflux < nbindex) ccindex(i2,i3)=ixflux
!!$      !
!!$    endif ! (numx_flux > 0)
  end do ! i2
end do ! i3
!
!!$call uio_openwr(nc, 'rhd_mean_c1Index_test.out', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now')
!!$call uio_wrlabl(nc, box_ident                )
!!$call uio_wr(nc, 'z'          , box_id_ident  )
!!$call uio_wr(nc, model%xc1    , 'xc1'         )
!!$call uio_wr(nc, model%xb1    , 'xb1'         )
!!$call uio_wr(nc, model%xc2    , 'xc2'         )
!!$call uio_wr(nc, model%xb2    , 'xb2'         )
!!$call uio_wr(nc, model%xc3    , 'xc3'         )
!!$call uio_wr(nc, model%xb3    , 'xb3'         )
!!$call uio_wr(nc, ccindex       , 'ccindex'      )
!!$call uio_wrlabl(nc, endbox_ident)
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call uio_closrd(nc)
!
end subroutine rhd_mean_c1Index


!----------*****************------------------------------------------------------------------------
subroutine rhd_mean_rb12mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
                             nb6index, bindex, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rb12mean ('rhd_mean_radius_boundary_12_mean')
!
! PURPOSE:
!   Average boundary centered quantity over spheres in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_rb12mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
!                          nb6index, bindex, weight)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nb:           ('number_boundary')   integer, number of points in average flux vector
!   q:            ('quantity') 3D real array with input data
!   nb6index:     ('number_boundary_6_index') 3D integer array
!   bindex:       ('boundary_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2001-08-29 (B.F. Uppsala) nbindex -> mlbindex...nrbindex
!   2015-03-21 (B.F.) Give the mean data array its own kind
!   2017-08-28 (B.F. Lyon) Correct use of index arrays
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real, dimension(m1:n1,m2:n2,m3:n3+1), &
                           intent(in)   :: q
real(kind=mArk), &
         dimension(1:nb) ,intent(inout):: qmean
integer, dimension(6     ,m1:n1,m2:n2), &
                           intent(in)   :: nb6index
integer, dimension(-nb:nb,m1:n1,m2:n2), &
                           intent(in)   :: bindex
real,    dimension(       m1:n1,m2:n2), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i1, i2, mlb, nlb, mrb, nrb
!---------------------------------------------------------------------------------------------------
!
! --- Integrate input quantity ---
do i2=m2,n2
  do i1=m1,n1
    mlb=nb6index(1,i1,i2)
    nlb=nb6index(3,i1,i2)
    qmean(mlb:nlb)=qmean(mlb:nlb) - q(i1,i2,bindex(-mlb:-nlb:-1,i1,i2)) * weight(i1,i2)
    !
    mrb=nb6index(4,i1,i2)
    nrb=nb6index(6,i1,i2)
    qmean(mrb:nrb)=qmean(mrb:nrb) + q(i1,i2,bindex( mrb: nrb   ,i1,i2)) * weight(i1,i2)
  end do ! i1
end do ! i2
!
end subroutine rhd_mean_rb12mean


!----------*****************------------------------------------------------------------------------
subroutine rhd_mean_rb13mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
                             nb6index, bindex, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rb13mean ('rhd_mean_radius_boundary_13_mean')
!
! PURPOSE:
!   Average boundary centered quantity over spheres in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_rb13mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
!                          nb6index, bindex, weight)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nb:           ('number_boundary')   integer, number of points in average flux vector
!   q:            ('quantity') 3D real array with input data
!   nb6index:     ('number_boundary_6_index') 3D integer array
!   bindex:       ('boundary_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2001-08-29 (B.F. Uppsala) nbindex -> mlbindex...nrbindex
!   2015-03-21 (B.F.) Give the mean data array its own kind
!   2017-08-28 (B.F. Lyon) Correct use of index arrays
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real, dimension(m1:n1,m2:n2+1,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), &
         dimension(1:nb)  ,intent(inout):: qmean
integer, dimension(6     ,m1:n1,m3:n3), &
                           intent(in)   :: nb6index
integer, dimension(-nb:nb,m1:n1,m3:n3), &
                           intent(in)   :: bindex
real,    dimension(       m1:n1,m3:n3), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i1, i3, mlb, nlb, mrb, nrb
!---------------------------------------------------------------------------------------------------
!
! --- Integrate input quantity ---
do i3=m3,n3
  do i1=m1,n1
    mlb=nb6index(1,i1,i3)
    nlb=nb6index(3,i1,i3)
    qmean(mlb:nlb)=qmean(mlb:nlb) - q(i1,bindex(-mlb:-nlb:-1,i1,i3),i3) * weight(i1,i3)
    !
    mrb=nb6index(4,i1,i3)
    nrb=nb6index(6,i1,i3)
    qmean(mrb:nrb)=qmean(mrb:nrb) + q(i1,bindex( mrb: nrb   ,i1,i3),i3) * weight(i1,i3)
  end do ! i1
end do ! i3
!
end subroutine rhd_mean_rb13mean


!----------*****************------------------------------------------------------------------------
subroutine rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
                             nb6index, bindex, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rb23mean ('rhd_mean_radius_boundary_23_mean')
!
! PURPOSE:
!   Average boundary-centered vector quantity over spheres in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics, averaging
!
! CALLING SEQUENCE:
!   call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
!                          nb6index, bindex, weight)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nb:           ('number_boundary') integer, number of points in average flux vector
!   q:            ('quantity') 3D real array with input data
!   nb6index:     ('number_boundary_6_index') 3D integer array
!   bindex:       ('boundary_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2001-08-29 (B.F. Uppsala) nbindex -> mlbindex...nrbindex
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!   2017-08-28 (B.F. Lyon) Correct use of index arrays
!   2018-06-13 (B.F. Uppsala) Remove optional parameter unity (use rhd_mean_rb23absmean)
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real, dimension(m1:n1+1,m2:n2,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), &
         dimension(1:nb)  ,intent(inout):: qmean
integer, dimension(6     ,m2:n2,m3:n3), &
                           intent(in)   :: nb6index
integer, dimension(-nb:nb,m2:n2,m3:n3), &
                           intent(in)   :: bindex
real,    dimension(       m2:n2,m3:n3), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i2, i3, mlb, nlb, mrb, nrb
!---------------------------------------------------------------------------------------------------
!
! --- Integrate input quantity ---
do i3=m3,n3
  do i2=m2,n2
    mlb=nb6index(1,i2,i3)
    nlb=nb6index(3,i2,i3)
    qmean(mlb:nlb)=qmean(mlb:nlb) - q(bindex(-mlb:-nlb:-1,i2,i3),i2,i3) * weight(i2,i3)
    !
    mrb=nb6index(4,i2,i3)
    nrb=nb6index(6,i2,i3)
    qmean(mrb:nrb)=qmean(mrb:nrb) + q(bindex( mrb: nrb   ,i2,i3),i2,i3) * weight(i2,i3)
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_rb23mean


!----------********************---------------------------------------------------------------------
subroutine rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
                                nb6index, bindex, weight, unity)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rb23absmean ('rhd_mean_radius_boundary_23_absolute_mean')
!
! PURPOSE:
!   Average boundary-centered scalar quantity over spheres in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics, averaging
!
! CALLING SEQUENCE:
!   call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
!                             nb6index, bindex, weight, unity=unity)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nb:           ('number_boundary') integer, number of points in average flux vector
!   q:            ('quantity') 3D real array with input data
!   nb6index:     ('number_boundary_6_index') 3D integer array
!   bindex:       ('boundary_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!   unity:        ('unity') optional logical, if present and true, the input quantity is ignored
!                 and only the weight factors are integrated
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2001-08-29 (B.F. Uppsala) nbindex -> mlbindex...nrbindex
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!   2017-08-28 (B.F. Lyon) Correct use of index arrays
!   2018-06-13 (B.F. Uppsala) Derived from rhd_mean_rb23mean
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real, dimension(m1:n1+1,m2:n2,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), &
         dimension(1:nb)  ,intent(inout):: qmean
integer, dimension(6     ,m2:n2,m3:n3), &
                           intent(in)   :: nb6index
integer, dimension(-nb:nb,m2:n2,m3:n3), &
                           intent(in)   :: bindex
real,    dimension(       m2:n2,m3:n3), &
                           intent(in)   :: weight
logical,         optional, intent(in)   :: unity
!
! --- Local variables ---
logical                                 :: unity_flag
integer                                 :: i2, i3, mlb, nlb, mrb, nrb
!---------------------------------------------------------------------------------------------------
!
if (present(unity)) then
  unity_flag=unity
else
  unity_flag=.false.
endif
!
if (unity_flag) then
  ! --- Integrate only weights - with proper sign ---
  do i3=m3,n3
    do i2=m2,n2
      mlb=nb6index(1,i2,i3)
      nlb=nb6index(3,i2,i3)
      qmean(mlb:nlb)=qmean(mlb:nlb) + weight(i2,i3)
      !
      mrb=nb6index(4,i2,i3)
      nrb=nb6index(6,i2,i3)
      qmean(mrb:nrb)=qmean(mrb:nrb) + weight(i2,i3)
    end do ! i2
  end do ! i3
else
  ! --- Integrate input quantity ---
  do i3=m3,n3
    do i2=m2,n2
      mlb=nb6index(1,i2,i3)
      nlb=nb6index(3,i2,i3)
      qmean(mlb:nlb)=qmean(mlb:nlb) + q(bindex(-mlb:-nlb:-1,i2,i3),i2,i3) * weight(i2,i3)
      !
      mrb=nb6index(4,i2,i3)
      nrb=nb6index(6,i2,i3)
      qmean(mrb:nrb)=qmean(mrb:nrb) + q(bindex( mrb: nrb   ,i2,i3),i2,i3) * weight(i2,i3)
    end do ! i2
  end do ! i3
endif
!
end subroutine rhd_mean_rb23absmean


!----------**************************---------------------------------------------------------------
subroutine rhd_mean_SphericalHarmonic(m,n, i, x1, x2, x3, Y)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_SphericalHarmonic ('rhd_mean_Spherical_Harmonic')
!
! PURPOSE:
!   Compute real spherical harmonic for points in space.
!   
! CATEGORY:
!   Math
!
! CALLING SEQUENCE:
!   call rhd_mean_SphericalHarmonic(m,n, i, x1, x2, x3, Y)
!
! INPUT:
!   m:            ('m_number') integer with lower boundary index of vectors
!   n:            ('number') integer with upper boundary index of vectors
!   i:            ('index') mode index: l_mode^2 + l_mode  + m_mode
!   x1:           ('x_1') real vector with x1 ('x') coordinates
!   x2:           ('x_2') real vector with x2 ('y') coordinates
!   x3:           ('x_3') real vector with x3 ('z') coordinates
!
! OUTPUT:
!   Y:            ('Y') real vector with mode amplitude
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   See \href{https://en.wikipedia.org/wiki/Table_of_spherical_harmonics}{Wikipedia: Table of Spherical Harmonics}
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2018-06-11 (B.F. Uppsala) Extracted from rhd_mean_rb23shmean
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m, n, i
real, dimension(m:n),      intent(in)   :: x1, x2, x3
real, dimension(m:n),      intent(out)  :: Y
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (i == 0) then
  ! --- Spherical harmonic: l=0, m= 0 ---
  Y=1.0
  !
else if (i == 1) then
  ! --- Spherical harmonic: l=1, m=-1 ---
  Y=sqrt(3.0) * x2
else if (i == 2) then
  ! --- Spherical harmonic: l=1, m= 0 ---
  Y=sqrt(3.0) * x3
else if (i == 3) then
  ! --- Spherical harmonic: l=1, m= 1 ---
  Y=sqrt(3.0) * x1
  !
else if (i == 4) then
  ! --- Spherical harmonic: l=2, m=-2 ---
  Y=sqrt(15.0    ) * x1 * x2
else if (i == 5) then
  ! --- Spherical harmonic: l=2, m=-1 ---
  Y=sqrt(15.0    ) * x2 * x3
else if (i == 6) then
  ! --- Spherical harmonic: l=2, m= 0 ---
  Y=sqrt( 5.0/4.0) * (-x1**2 - x2**2 + 2.0*x3**2)
else if (i == 7) then
  ! --- Spherical harmonic: l=2, m= 1 ---
  Y=sqrt(15.0    ) * x3 * x1
else if (i == 8) then
  ! --- Spherical harmonic: l=2, m= 2 ---
  Y=sqrt(15.0/4.0) * (x1**2 - x2**2)
  !
else if (i ==  9) then
  ! --- Spherical harmonic: l=3, m=-3 ---
  Y=sqrt( 35.0/8.0) * (3.0*x1**2 -     x2**2) * x2
else if (i == 10) then
  ! --- Spherical harmonic: l=3, m=-2 ---
  Y=sqrt(105.0    ) * x1 * x2 * x3
else if (i == 11) then
  ! --- Spherical harmonic: l=3, m=-1 ---
  Y=sqrt( 21.0/8.0) * x2 * (4.0*x3**2 -     x1**2 -     x2**2)
else if (i == 12) then
  ! --- Spherical harmonic: l=3, m= 0 ---
  Y=sqrt(  7.0/4.0) * x3 * (2.0*x3**2 - 3.0*x1**2 - 3.0*x2**2)
else if (i == 13) then
  ! --- Spherical harmonic: l=3, m= 1 ---
  Y=sqrt( 21.0/8.0) * x1 * (4.0*x3**2 -     x1**2 -     x2**2)
else if (i == 14) then
  ! --- Spherical harmonic: l=3, m= 2 ---
  Y=sqrt(105.0/4.0) * (    x1**2 -     x2**2) * x3
else if (i == 15) then
  ! --- Spherical harmonic: l=3, m= 3 ---
  Y=sqrt( 35.0/8.0) * (    x1**2 - 3.0*x2**2) * x1
  !
endif ! (i)
!
end subroutine rhd_mean_SphericalHarmonic


!----------*******************----------------------------------------------------------------------
subroutine rhd_mean_rb23shmean(model, m1,n1, m2,n2, m3,n3, m,n, na, nWave, i_wave, dim, q, qmean, &
                               nb6index, bindex, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rb23shmean ('rhd_mean_radius_boundary_23_spherical_harmonics_mean')
!
! PURPOSE:
!   Compute spherical-harmonic amplitude of boundary-centered quantity in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics, averaging
!
! CALLING SEQUENCE:
!   call rhd_mean_rb23shmean(m1,n1, m2,n2, m3,n3, m,n, na, nWave, i_wave, dim, q, qmean, &
!                            nb6index, bindex, weight)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   m:            ('m_number') integer, spatial start index
!   n:            ('number')   integer, spatial end index
!   na:           ('number_array') number of averaged quantities in array
!   nWave:        ('number_Wave') integer with number of wave components to store
!   i_wave:       ('index_wave') integer with start index of waves in data array qmean
!   dim:          ('dimension') integer to specify dimension of mean vectors (1, 2, 3)
!   q:            ('quantity') 3D real array with input data
!   nb6index:     ('number_boundary_6_index') 3D integer array
!   bindex:       ('boundary_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 2D real array with averaged data
!
! ROUTINES:
!   rhd_mean_SphericalHarmonic: Compute spherical harmonic for points in space
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2018-06-04 (B.F. Uppsala) Derived from rhd_mean_rb23mean
!   2018-06-12 (B.F.) New parameter nWave
!   2018-06-17 (B.F. Kiel) Bugfix in index range
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, m, n, na, nWave, i_wave, dim
real, dimension(m1:n1+1,m2:n2,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), dimension(m:n+1,na), &
                           intent(inout):: qmean
integer, dimension(6     ,m2:n2,m3:n3), &
                           intent(in)   :: nb6index
integer, dimension(-(n+1):n+1,m2:n2,m3:n3), &
                           intent(in)   :: bindex
real,    dimension(       m2:n2,m3:n3), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i2, i3, mlb, nlb, mrb, nrb, i, mlb0, nlb0, mrb0, nrb0
real, dimension(1:2*(n-m+3))            :: x1mrg, x2mrg, x3mrg, rmrg, qmrg, fmrg
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! --- Integrate input quantity ---
do i3=m3,n3
  do i2=m2,n2
    ! --- Define abbreviations for the ranges of cell surfaces to be taken into account ---
    mlb=    nb6index(1,i2,i3)
    nlb=max(nb6index(3,i2,i3), mlb-1)
    mrb=    nb6index(4,i2,i3)
    nrb=max(nb6index(6,i2,i3), mrb-1)
    !
    ! --- Construct indices for contiguous data values ---
    mlb0=1
    nlb0=nlb-mlb+mlb0
    mrb0=nlb0+1
    nrb0=nrb-mrb+mrb0
    !
    if (nrb0 >= mlb0) then
      ! --- Construct merged contiguous axes depending on actual stencil direction ---
      if (dim == 1) then
        x1mrg(mlb0:nlb0)=model%xb1(bindex(-mlb:-nlb:-1,i2,i3))
        x1mrg(mrb0:nrb0)=model%xb1(bindex( mrb: nrb   ,i2,i3))
        x2mrg(mlb0:nlb0)=model%xc2(i2)
        x2mrg(mrb0:nrb0)=model%xc2(i2)
        x3mrg(mlb0:nlb0)=model%xc3(i3)
        x3mrg(mrb0:nrb0)=model%xc3(i3)
      else if (dim == 2) then
        x1mrg(mlb0:nlb0)=model%xc2(i2)
        x1mrg(mrb0:nrb0)=model%xc2(i2)
        x2mrg(mlb0:nlb0)=model%xb1(bindex(-mlb:-nlb:-1,i2,i3))
        x2mrg(mrb0:nrb0)=model%xb1(bindex( mrb: nrb   ,i2,i3))
        x3mrg(mlb0:nlb0)=model%xc3(i3)
        x3mrg(mrb0:nrb0)=model%xc3(i3)
      else ! (dim == 3)
        x1mrg(mlb0:nlb0)=model%xc3(i3)
        x1mrg(mrb0:nrb0)=model%xc3(i3)
        x2mrg(mlb0:nlb0)=model%xc2(i2)
        x2mrg(mrb0:nrb0)=model%xc2(i2)
        x3mrg(mlb0:nlb0)=model%xb1(bindex(-mlb:-nlb:-1,i2,i3))
        x3mrg(mrb0:nrb0)=model%xb1(bindex( mrb: nrb   ,i2,i3))
      endif ! (dim)
      !
      ! --- Compute distance to origin ---
      rmrg(mlb0:nrb0)=max(sqrt(x1mrg(mlb0:nrb0)**2 + x2mrg(mlb0:nrb0)**2 + x3mrg(mlb0:nrb0)**2), &
                          eps_tiny)
      !
      ! --- Normalize axes ---
      x1mrg(mlb0:nrb0)=x1mrg(mlb0:nrb0) / rmrg(mlb0:nrb0)
      x2mrg(mlb0:nrb0)=x2mrg(mlb0:nrb0) / rmrg(mlb0:nrb0)
      x3mrg(mlb0:nrb0)=x3mrg(mlb0:nrb0) / rmrg(mlb0:nrb0)
      !
      ! --- Construct merged contiguous data vector ---
      qmrg( mlb0:nlb0)=q(bindex(-mlb:-nlb:-1,i2,i3),i2,i3)
      qmrg( mrb0:nrb0)=q(bindex( mrb: nrb   ,i2,i3),i2,i3)
      !
      do i=0,nWave-1
        call rhd_mean_SphericalHarmonic(mlb0,nrb0, i, &
                                        x1mrg(mlb0:nrb0), x2mrg(mlb0:nrb0), x3mrg(mlb0:nrb0), &
                                        fmrg(mlb0:nrb0))
        !
        qmean(mlb:nlb,i_wave+i)=qmean(mlb:nlb,i_wave+i) &
                                  - fmrg(mlb0:nlb0)*qmrg(mlb0:nlb0) * weight(i2,i3)
        qmean(mrb:nrb,i_wave+i)=qmean(mrb:nrb,i_wave+i) &
                                  + fmrg(mrb0:nrb0)*qmrg(mrb0:nrb0) * weight(i2,i3)
      enddo ! i
      !
    endif ! (nrb0 >= mlb0)
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_rb23shmean


!----------***************--------------------------------------------------------------------------
subroutine rhd_mean_rbmean(m1,n1, m2,n2, m3,n3, nb, q, qmean, nbindex, bindex, weight, unity)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rbmean ('rhd_mean_radius_boundary_mean')
!
! PURPOSE:
!   Average boundary-centered quantity over spheres in a symmetric box.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_rbmean(m1,n1, m2,n2, m3,n3, nb, q, qmean, nbindex, bindex, weight, unity)
!
! INPUT:
!   q:            ('quantity') 3D real array with input data
!   weight:       ('weight') 2D real array with weights
!   unity:        ('unity') optional logical, if present and true, the input quantity is ignored
!                 and only the weight factors are integrated
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Assumes equidistant, symmetric grid. The latter makes it not MPI safe.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real, dimension(m1:n1+1,m2:n2,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), &
      dimension(1:nb)     ,intent(inout):: qmean
integer, dimension(m2:n2,m3:n3), &
                           intent(in)   :: nbindex
integer, dimension(-nb:nb,m2:n2,m3:n3), &
                           intent(in)   :: bindex
real, dimension(m2:n2,m3:n3), &
                           intent(in)   :: weight
logical,          optional,intent(in)   :: unity
!
! --- Local variables ---
logical                                 :: unity_flag
integer                                 :: i2, i3, mb
!---------------------------------------------------------------------------------------------------
!
if (present(unity)) then
  unity_flag=unity
else
  unity_flag=.false.
endif
!
if (unity_flag) then
  ! --- Integrate only weights - with proper sign ---
  do i3=m3,n3
    do i2=m2,n2
      mb=nb-nbindex(i2,i3)+1
      qmean(mb:nb)=qmean(mb:nb) + weight(i2,i3)
      qmean(mb:nb)=qmean(mb:nb) + weight(i2,i3)
    end do ! i2
  end do ! i3
else
  ! --- Integrate input quantity ---
  do i3=m3,n3
    do i2=m2,n2
      mb=nb-nbindex(i2,i3)+1
      qmean(mb:nb)=qmean(mb:nb) + q(bindex( mb: nb   ,i2,i3),i2,i3) * weight(i2,i3)
      qmean(mb:nb)=qmean(mb:nb) - q(bindex(-mb:-nb:-1,i2,i3),i2,i3) * weight(i2,i3)
!!$    write(6,*)
!!$    write(6,*) 'bindex+:', bindex( mb: nb   ,i2,i3)
!!$    write(6,*) 'bindex-:', bindex(-mb:-nb:-1,i2,i3) 
    end do ! i2
  end do ! i3
endif
!
end subroutine rhd_mean_rbmean


!----------*****************------------------------------------------------------------------------
subroutine rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
                             cindex, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rc23mean ('rhd_mean_radius_center_23_mean')
!
! PURPOSE:
!   Average cell-centered quantity over spheres in an asymmetric box.
!   
! CATEGORY:
!   Hydrodynamics, averaging
!
! CALLING SEQUENCE:
!   call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, nb, q, qmean, &
!                          cindex, weight)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nb:           ('number_boundary') integer, number of points in average flux vector
!   q:            ('quantity') 3D real array with input data
!   cindex:       ('center_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!   2017-08-28 (B.F. Lyon) Adapted from rhd_mean_rcmean
!   2017-08-30 (B.F.) Use only cindex array (discarded points have -1)
!   2017-09-19 (B.F. Uppsala) Remove two unused index parameters
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real,    dimension(m1:n1 ,m2:n2,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), &
         dimension(1:nb-1),intent(inout):: qmean
integer, dimension( m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: cindex
real,    dimension( m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i1, i2, i3, mlb, nlb, mrb, nrb
!---------------------------------------------------------------------------------------------------
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      if (cindex(i1,i2,i3) >= 0) then
        qmean(cindex(i1,i2,i3))=qmean(cindex(i1,i2,i3)) + q(i1,i2,i3)*weight(i1,i2,i3)
      endif
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_rc23mean


!----------***************--------------------------------------------------------------------------
subroutine rhd_mean_rcmean(m1,n1, m2,n2, m3,n3, nb, q, qmean, nbindex, bindex, cindex, weight)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_rcmean ('rhd_mean_radius_center_mean')
!
! PURPOSE:
!   Average cell-centered quantity over spheres in a symmetric box.
!   
! CATEGORY:
!   Hydrodynamics, averaging
!
! CALLING SEQUENCE:
!   call rhd_mean_rcmean(m1,n1, m2,n2, m3,n3, nb, q, qmean, nbindex, bindex, cindex, weight)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nb:           ('number_boundary') integer, number of points in average flux vector
!   q:            ('quantity') 3D real array with input data
!   nbindex:      ('number_boundary_index') 2D integer array
!   bindex:       ('boundary_index') 3D integer array
!   cindex:       ('center_index') 3D integer array
!   weight:       ('weight') 2D real array with weights
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 1D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input arrays have to be symmetric in first dimension (m1=-n1 and corresponding axis).
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-31 (B.F. Copenhagen) Written
!   2015-03-21 (B.F. Uppsala) Give the mean data array its own kind
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nb
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: q
real(kind=mArk), &
      dimension(1:nb-1)   ,intent(inout):: qmean
integer, dimension(m2:n2,m3:n3), &
                           intent(in)   :: nbindex
integer, dimension(-nb:nb,m2:n2,m3:n3), &
                           intent(in)   :: bindex
integer, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: cindex
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i1, i2, i3
!---------------------------------------------------------------------------------------------------
!
do i3=m3,n3
  do i2=m2,n2
    if (nbindex(i2,i3) > 0) then
      ! --- Lower/left part of sphere ---
      do i1=bindex(-nb,i2,i3)  , bindex(0,i2,i3)
        qmean(cindex(i1,i2,i3))=qmean(cindex(i1,i2,i3)) + q(i1,i2,i3)*weight(i1,i2,i3)
      end do ! i1
      !
      ! --- ToDo: why split into two loops? ---
      !
      ! --- Upper/right part of sphere ---
      do i1=bindex( nb,i2,i3)-1, bindex(0,i2,i3)+1, -1
        qmean(cindex(i1,i2,i3))=qmean(cindex(i1,i2,i3)) + q(i1,i2,i3)*weight(i1,i2,i3)
      end do ! i1
    endif
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_rcmean


!----------***************--------------------------------------------------------------------------
subroutine rhd_mean_ccmean(m1,n1, m2,n2, m3,n3, mc1,nc1,mc2,nc2, ccindex, weight, q, qmean)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_ccmean ('rhd_mean_cylinder_center_mean')
!
! PURPOSE:
!   Average cell-centered quantity over cylinders (azimuthal mean).
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_ccmean(m1,n1, m2,n2, m3,n3, mc1,nc1,mc2,nc2, ccindex, weight, q, qmean)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index of input array
!   n1:           ('number_1')   integer, 1. dimension, end index of input array
!   m2:           ('m_number_2') integer, 2. dimension, start index of input array
!   n2:           ('number_2')   integer, 2. dimension, end index of input array
!   m3:           ('m_number_3') integer, 3. dimension, start index of input array
!   n3:           ('number_3')   integer, 3. dimension, end index of input array
!   mc1:          ('m_number_cylindrical_1') integer, 1. dimension, start index of output array
!   nc1:          ('number_cylindrical_1') integer, 1. dimension, end index of output array
!   mc2:          ('m_number_cylindrical_2') integer, 2. dimension, start index of output array
!   nc2:          ('number_cylindrical_2') integer, 2. dimension, end index of output array
!   ccindex:      ('cylinder_center_index') 2D integer array with indices of cylinder columns
!   weight:       ('weight') 2D real array with weights
!   q:            ('quantity') 3D real array with input data
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 2D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-08-19 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, mc1, nc1, mc2, nc2
integer, dimension(m2:n2,m3:n3), &
                           intent(in)   :: ccindex
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: weight
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: q
real, dimension(mc1:nc1,mc2:nc2), &
                           intent(inout):: qmean
!
! --- Local variables ---
integer                                 :: i1, i2, i3
!---------------------------------------------------------------------------------------------------
!
do i3=m3,n3
  do i2=m2,n2
    if (ccindex(i2,i3) > 0) then
      !DIR$ SIMD
      do i1=m1,n1
        qmean(i1,ccindex(i2,i3))=qmean(i1,ccindex(i2,i3)) + q(i1,i2,i3)*weight(i1,i2,i3)
      end do ! i1
    endif ! (ccindex(i2,i3) > 0)
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_ccmean


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, mc1,nc1,mc2,nc2,na, ccindex, weight, qmean, iq, q)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_ccamean ('rhd_mean_cylinder_center_array_mean')
!
! PURPOSE:
!   Average cell-centered quantity over cylinders (azimuthal mean).
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, mc1,nc1,mc2,nc2,na, ccindex, weight, qmean, iq, q)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index of input array
!   n1:           ('number_1')   integer, 1. dimension, end index of input array
!   m2:           ('m_number_2') integer, 2. dimension, start index of input array
!   n2:           ('number_2')   integer, 2. dimension, end index of input array
!   m3:           ('m_number_3') integer, 3. dimension, start index of input array
!   n3:           ('number_3')   integer, 3. dimension, end index of input array
!   mc1:          ('m_number_cylindrical_1') integer, 1. dimension, start index of output array
!   nc1:          ('number_cylindrical_1') integer, 1. dimension, end index of output array
!   mc2:          ('m_number_cylindrical_2') integer, 2. dimension, start index of output array
!   nc2:          ('number_cylindrical_2') integer, 2. dimension, end index of output array
!   na:           ('nunber_array') integer, 3. dimension of output array
!   ccindex:      ('cylinder_center_index') 2D integer array with indices of cylinder columns
!   weight:       ('weight') 2D real array with weights
!   iq:           ('index_quantity') integer with index of 2D array of quantity in 3D array
!   q:            ('quantity') 3D real array with input data
!
! INPUT/OUTPUT:
!   qmean:        ('quantity_mean') 3D real array with output data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-08-19 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, mc1, nc1, mc2, nc2, na, iq
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: q
real, dimension(mc1:nc1,mc2:nc2,1:na), &
                           intent(inout):: qmean
integer, dimension(m2:n2,m3:n3), &
                           intent(in)   :: ccindex
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: weight
!
! --- Local variables ---
integer                                 :: i1, i2, i3
!---------------------------------------------------------------------------------------------------
!
do i3=m3,n3
  do i2=m2,n2
    if (ccindex(i2,i3) > 0) then
      !DIR$ SIMD
      do i1=m1,n1
        qmean(i1,ccindex(i2,i3),iq)=qmean(i1,ccindex(i2,i3),iq) + q(i1,i2,i3)*weight(i1,i2,i3)
      end do ! i1
    endif ! (ccindex(i2,i3) > 0)
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_ccamean


!----------****************-------------------------------------------------------------------------
subroutine rhd_mean_VecProj(model, y1, y2, y3, yr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_VecProj ('rhd_mean_Vector_Projection')
!
! PURPOSE:
!   Project vector on radial ray.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_VecProj(model, y1, y2, y3, yr)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   y1:           ('y_1') real 3D array with 1st component of input vector
!   y2:           ('y_2') real 3D array with 2nd component of input vector
!   y3:           ('y_3') real 3D array with 3rd component of input vector
!
! OUTPUT:
!   yr:           ('y_radial') real 3D array with projected vector
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Take the axis data from 'model' and project the vector field given by (y1,y2,y3) onto
!   a radial ray.
!
! EXAMPLE:
!   call rhd_mean_VecProj(model, y1, y2, y3, yr)
!
! MODIFICATION HISTORY:
!   2000-02-21 (B.F. Copenhagen) Written
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: y1, y2, y3
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(out)  :: yr
!
! --- Local variables ---
integer                                 :: i2, i3
real                                    :: sqr_xc23, sqr_xc3
real, dimension(model%m1:model%n1)      :: r
!---------------------------------------------------------------------------------------------------
!
do i3=model%m3,model%n3
  sqr_xc3=model%xc3(i3)**2
  do i2=model%m2,model%n2
    sqr_xc23=sqr_xc3+model%xc2(i2)**2
    !
    r=sqrt(sqr_xc23+model%xc1(model%m1:model%n1)**2)
    where (r < 100.0*tiny(r))
      r=1.0
    end where
    !
    yr(:,i2,i3)=(model%xc1(model%m1:model%n1)*y1(:,i2,i3) + &
                 model%xc2(i2               )*y2(:,i2,i3) + &
                 model%xc3(i3               )*y3(:,i2,i3)    ) / r
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_VecProj


!----------**********************-------------------------------------------------------------------
subroutine rhd_mean_VecbProjUnity(model, yr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_VecbProjUnity ('rhd_mean_Vector_boundary_Projection_Unity')
!
! PURPOSE:
!   Project radial unity vector on x1 axis direction.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_VecProj(model, yr)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!
! OUTPUT:
!   yr:           ('y_radial') real 3D array with projected vector
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Take the axis data from 'model' and project a radial unity vector onto x1 direction.
!
! EXAMPLE:
!   call rhd_mean_VecbProjUnity(model, yr)
!
! MODIFICATION HISTORY:
!   2018-05-31 (B.F. Uppsala) Derived from rhd_mean_VecProj
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3     ), &
                           intent(out)  :: yr
!
! --- Local variables ---
integer                                 :: i2, i3
real                                    :: sqr_xc23, sqr_xc3
real, dimension(model%m1:model%n1+1)    :: r
!---------------------------------------------------------------------------------------------------
!
do i3=model%m3,model%n3
  sqr_xc3=model%xc3(i3)**2
  do i2=model%m2,model%n2
    sqr_xc23=sqr_xc3+model%xc2(i2)**2
    !
    r=sqrt(sqr_xc23+model%xb1(model%m1:model%n1+1)**2)
    where (r < 100.0*tiny(r))
      r=1.0
    end where
    !
    yr(:,i2,i3)=(model%xb1(model%m1:model%n1+1))/r
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_VecbProjUnity


!----------******************-----------------------------------------------------------------------
subroutine rhd_mean_c1VecProj(model, y2, y3, yr, ya)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mean_c1VecProj ('rhd_mean_cylinder_1_Vector_Projection')
!
! PURPOSE:
!   Project vector on radial vector perpendicular to x1 axis.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mean_VecProj(model, y2, y3, yr, ya)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   y2:           ('y_2') real 3D array with 2nd component of input vector
!   y3:           ('y_3') real 3D array with 3rd component of input vector
!
! OUTPUT:
!   yr:           ('y_radial') real 3D array with radially projected vector
!   ya:           ('y_azimuthal') real 3D array with azimuthally projected vector
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Take the cylinder axis data from 'model' and project the vector field given by (y2,y3) onto
!   a radial ray perpendicular to the cylinder axis.
!
! EXAMPLE:
!   call rhd_mean_VecProj(model, y2, y3, yr, ya)
!
! MODIFICATION HISTORY:
!   2013-08-20 (B.F. Lyon) Adapted from rhd_mean_VecProj
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: y2, y3
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(out)  :: yr, ya
!
! --- Local variables ---
integer                                 :: i2, i3
real                                    :: sqr_xc23, sqr_xc3, r
!---------------------------------------------------------------------------------------------------
!
do i3=model%m3,model%n3
  sqr_xc3=model%xc3(i3)**2
  do i2=model%m2,model%n2
    sqr_xc23=sqr_xc3+model%xc2(i2)**2
    !
    r=sqrt(sqr_xc23)
    if (r < 100.0*tiny(r)) r=1.0
    !
    yr(:,i2,i3)=(model%xc2(i2)*y2(:,i2,i3) + &
                 model%xc3(i3)*y3(:,i2,i3)    ) / r
    ya(:,i2,i3)=(model%xc3(i3)*y2(:,i2,i3) - &
                 model%xc2(i2)*y3(:,i2,i3)    ) / r
  end do ! i2
end do ! i3
!
end subroutine rhd_mean_c1VecProj

end module rhd_mean_module
