!***************************************************************************************************
!
!   #####   #    #  #####         #    #  #####   #
!   #    #  #    #  #    #        ##  ##  #    #  #
!   #    #  ######  #    #        # ## #  #    #  #
!   #####   #    #  #    #        #    #  #####   #
!   #   #   #    #  #    #        #    #  #       #
!   #    #  #    #  #####  ###### #    #  #       # _module
!
!   MPI routines for Radiation-Hydrodynamics Code
!
!***************************************************************************************************
!   Fortran90, MPI
!   2018-11-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_MPI_module:        MPI routines specific for RHD code
!
!***************************************************************************************************

!# /* --- Default values for preprocessor macros ------------------------------------------------ */
!#
!# /* --- MPI flag activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------**************-------------------------------------------------------------------------------
module rhd_MPI_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_module ('rhd_MPI_module')
!
! PURPOSE:
!   Provide MPI routines for RHD.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   use rhd_MPI_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   --- General (MPI and non-MPI) routines ---
!   rhd_MPI_ErrorDist:     Distribute error code and output string among processes
!   rhd_MPI_DtimeErrorDist: Distribute time step, error code, and output string among processes
!   rhd_MPI_AllReduceReals: Reduce and broadcast a number of real scalars in one go
!   rhd_MPI_BCastReals:    Broadcast a number of real scalars in one go
!   rhd_MPI_DimsCreate:    Find and limit number of MPI processes per dimension
!   rhd_MPI_PrintInfo:     Write infomation about MPI environment to output channel
!   rhd_OMP_PrintInfo:     Write infomation about OpenMP environment to output channel
!   rhd_MPI_boxWrite:      Write box with additional information into file in UIO form
!   --- MPI-only routines ---
!   rhd_MPI_CartSetup:     Set up Cartesian MPI communicator and put data into geometry variable
!   rhd_MPI_boxRead:       Read box from file in UIO form and distribute it to other processes
!
! MODIFICATION HISTORY:
!   2012-07-20 (B.F. Lyon) First version
!   2016-04-15 (B.F.) Reorganize global world-geometry variables for MPI
!   2017-08-11 (B.F.) Move and rename rhd_MPI_PCoords2mn
!   2017-09-18 (B.F. Uppsala) Move MPI routines for prop handling into rhd_prop_module;
!                             Move MPI routines for box handling into rhd_box_module
!   2017-11-17 (B.F.) Add routine rhd_MPI_AllReduceReals
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------*****************------------------------------------------------------------------------
subroutine rhd_MPI_ErrorDist(rhdMPI_COMM, MyProc, MaProc, ierr, outstr, ProcErr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_ErrorDist ('rhd_MPI_Error_Distribute')
!
! PURPOSE:
!   Distribute error code and output string among processes.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_ErrorDist(rhdMPI_COMM, MyProc, Maproc, ierr, outstr, ProcErr)
!
! INPUT:
!   rhdMPI_COMM:  ('rhd_MPI_COMMunicator') MPI communicator
!   MyProc:       ('My_Process') integer with rank of current process
!   MaProc:       ('Master_Process') integer with rank of master process
!
! INPUT/OUTPUT:
!   ierr:         ('integer_error') integer with exit error status.
!                 The input is the error status of the current process.
!                 The output is the largest error status value of all processes.
!   outstr:       ('output_string') character with description of error.
!                 The input is the description of the error of the current process, possibly empty.
!                 For MyRank /= 0: output=input,
!                 For MyRank == 0: output=description of the error with the largest status value
!
! OUTPUT:
!   ProcErr:      ('Process_Error') integer with the rank of the process with the largest error
!
! ROUTINES:
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Recv:              Perform a blocking receive
!   MPI_Send:              Perform a blocking send
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Only 80 bytes of outstr are sent.
!
! PROCEDURE:
!   Find largest error code and send it to all processes.
!   Send corresponding error-info string to master process.
!
!   Early version of MPI command:
!     call MPI_AllReduce(ierr, ierrAll, 1, MPI_INTEGER, MPI_MAX, MPI_COMM_WORLD, ierrMPI)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-07-20 (B.F. Lyon) Written
!   2012-07-31 (B.F.) Non-MPI version
!   2012-08-01 (B.F.) New Parameter: MPI communicator
!   2012-10-23 (B.F.) Account for non-zero master process rank
!---------------------------------------------------------------------------------------------------
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: rhdMPI_COMM, MyProc, MaProc
integer,                   intent(inout):: ierr
character(len=*),          intent(inout):: outstr
integer,                   intent(out)  :: ProcErr
!
! --- Local variables ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
integer, dimension(2)                   :: ierrAndProcMy, ierrAndProcAll
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
character(len=80)                       :: outstr0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Put local error code and process rank into temporary array ---
ierrAndProcMy(1)=ierr
ierrAndProcMy(2)=MyProc
!
! --- Find global maximum error among all processes ---
call MPI_AllReduce(ierrAndProcMy, ierrAndProcAll, 1, MPI_2INTEGER, MPI_MAXLOC, rhdMPI_COMM, &
                   ierrMPI)
!
! --- Copy global maximum error and corresponding process rank into output parameters ---
ierr   =ierrAndProcAll(1)
ProcErr=ierrAndProcAll(2)
!
if ((MyProc == MaProc) .and. (MyProc == ProcErr)) then
  ! --- The local process is the master and caused the error: nothing to do ---
  !
else
  !
  if (MyProc == ProcErr) then
    ! --- The local process caused the error: send output string to the master process ---
    !
    outstr0=outstr
    call MPI_Send(outstr0, 80, MPI_CHARACTER, MaProc, 0, rhdMPI_COMM, &
                  ierrMPI)
    !
  endif
  !
  if (MyProc == MaProc) then
    ! --- Receive output string from the process that caused the error ---
    !
    call MPI_Recv(outstr0, 80, MPI_CHARACTER, MPI_ANY_SOURCE, MPI_ANY_TAG, rhdMPI_COMM, &
                  StatusMPI, ierrMPI)
    outstr=outstr0
    !
  endif
  !
endif
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
ProcErr=MyProc
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_MPI_ErrorDist


!----------**********************-------------------------------------------------------------------
subroutine rhd_MPI_DtimeErrorDist(rhdMPI_COMM, MyProc, Maproc, dtime_limit, ierr, outstr, ProcErr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_DtimeErrorDist ('rhd_MPI_Delta_time_Error_Distribute')
!
! PURPOSE:
!   Distribute time step, error code, and output string among processes.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_ErrorDist(rhdMPI_COMM, MyProc, Maproc, dtime_limit, ierr, outstr, ProcErr)
!
! INPUT:
!   rhdMPI_COMM:  ('rhd_MPI_COMMunicator') MPI communicator
!   MyProc:       ('My_Process') integer with rank of current process
!   MaProc:       ('Master_Process') integer with rank of master process
!
! INPUT/OUTPUT:
!   dtime_limit:  ('delta_time_limit') real with time step limit
!   ierr:         ('integer_error') integer with exit error status.
!                 The input is the error status of the current process.
!                 The output is the largest error status value of all processes.
!   outstr:       ('output_string') character with description of error.
!                 The input is the description of the error of the current process, possibly empty.
!                 For MyRank /= 0: output=input,
!                 For MyRank == 0: output=description of the error with the largest status value
!
! OUTPUT:
!   ProcErr:      ('Process_Error') integer with the rank of the process with the largest error
!
! ROUTINES:
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Recv:              Perform a blocking receive
!   MPI_Send:              Perform a blocking send
!   rhd_MPI_ErrorDist:     Distribute error code and output string among processes
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Only 80 bytes of outstr are sent.
!
! PROCEDURE:
!   Find largest error code and send it to all processes.
!   Send corresponding error-info string to master process.
!
!   Early version of MPI command:
!     call MPI_AllReduce(ierr, ierrAll, 1, MPI_INTEGER, MPI_MAX, MPI_COMM_WORLD, ierrMPI)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-07-20 (B.F. Lyon) Written
!   2012-07-31 (B.F.) Non-MPI version
!   2012-08-01 (B.F.) New Parameter: MPI communicator
!   2012-10-23 (B.F.) Account for non-zero master process rank
!   2012-11-08 (B.F.) Adapted from rhd_MPI_ErrorDist
!---------------------------------------------------------------------------------------------------
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: rhdMPI_COMM, MyProc, MaProc
real,                      intent(inout):: dtime_limit
integer,                   intent(inout):: ierr
character(len=*),          intent(inout):: outstr
integer,                   intent(out)  :: ProcErr
!
! --- Local variables ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
real                                    :: mydtime_limit
integer                                 :: ierrMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
!
! --- Find global minimum time-step limit among all processes ---
mydtime_limit=dtime_limit
call MPI_AllReduce(mydtime_limit, dtime_limit, 1, MPI_REAL, MPI_MIN, rhdMPI_COMM, ierrMPI)
!
! --- Distribute error info ---
call rhd_MPI_ErrorDist(rhdMPI_COMM, MyProc, Maproc, ierr, outstr, ProcErr)
!
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
ProcErr=MyProc
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_MPI_DtimeErrorDist


!----------**********************-------------------------------------------------------------------
subroutine rhd_MPI_AllReduceReals(rhdMPI_Op, Comm, r1, r2, r3, r4)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_AllReduceReals ('rhd_MPI_BroadCast_Reals')
!
! PURPOSE:
!   Reduce and broadcast a number of real scalars in one go.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_AllReduceReals(rhdMPI_Op, Comm, r1=r1, r2=r2, r3=r3, r4=r4)
!
! INPUT:
!   rhdMPI_Op:    ('rhd_MPI_Operator') integer with MPI operator, e.g., MPI_SUM
!   Comm:         ('Communicator') integer with MPI communicator
!
! OUTPUT:
!   None
!
! INPUT/OUTPUT:
!   r1:           ('real_1') real scalar to be reduced and broadcasted over entire MPI communicator
!   r2:           ('real_2') real scalar to be reduced and broadcasted over entire MPI communicator
!   r3:           ('real_3') real scalar to be reduced and broadcasted over entire MPI communicator
!   r4:           ('real_4') real scalar to be reduced and broadcasted over entire MPI communicator
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!
! ROUTINES:
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!   Via MPI reduction.
!
! RESTRICTIONS:
!   For a not-present optional parameter, a zero is used.
!   Called without MPI activated, the routine does nothing.
!
! PROCEDURE:
!   Pack input scalars into vector,
!   reduce and broadcast over all MPI processes in input communicator,
!   unpack the vector.
!
! EXAMPLE:
!   call rhd_MPI_AllReduceReals(MPI_SUM, MPI_COMM_WORLD, r1=rho_mean)
!
! MODIFICATION HISTORY:
!   2017-11-17 (B.F. Uppsala) Adapt from rhd_MPI_BCastReals
!---------------------------------------------------------------------------------------------------
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: rhdMPI_Op, Comm
real,            optional, intent(inout):: r1, r2, r3, r4
!
! --- Local variables ---
integer                                 :: ierrMPI
real, dimension(4)                      :: rVec
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
if (present(r1) .or. present(r2) .or. present(r3) .or. present(r4)) then
  ! --- Fill the temporary vector ---
  rVec=0.0
  if (present(r1)) rVec(1)=r1
  if (present(r2)) rVec(2)=r2
  if (present(r3)) rVec(3)=r3
  if (present(r4)) rVec(4)=r4
  !
  ! --- Reduce and broadcast the temporaty vector ---
  call MPI_AllReduce(MPI_IN_PLACE, rVec, 4, MPI_REAL, rhdMPI_Op, COMM, ierrMPI)
  !
  ! --- Extract values from the temporary vector ---
  if (present(r1)) r1=rVec(1)
  if (present(r2)) r2=rVec(2)
  if (present(r3)) r3=rVec(3)
  if (present(r4)) r4=rVec(4)
endif ! (present(r1) .or. present(r2) .or. present(r3) .or. present(r4))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_MPI_AllReduceReals


!----------******************-----------------------------------------------------------------------
subroutine rhd_MPI_BCastReals(MyProc, MaProc, Comm, r1, r2, r3, r4)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_BCastReals ('rhd_MPI_BroadCast_Reals')
!
! PURPOSE:
!   Broadcast a number of real scalars in one go.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_BCastReals(MyProc, MaProc, Comm, r1=r1, r2=r2, r3=r3, r4=r4)
!
! INPUT:
!   MyProc:       ('My_Process') integer with MPI rank of local process in communicator Comm
!   MaProc:       ('Master_Process') integer with MPI rank of master process
!   Comm:         ('Communicator') integer with MPI communicator
!
! OUTPUT:
!   None
!
! INPUT/OUTPUT:
!   r1:           ('real_1') real scalar to be broadcasted to entire MPI communicator
!   r2:           ('real_2') real scalar to be broadcasted to entire MPI communicator
!   r3:           ('real_3') real scalar to be broadcasted to entire MPI communicator
!   r4:           ('real_4') real scalar to be broadcasted to entire MPI communicator
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!
! ROUTINES:
!   MPI_BCast:             Broadcast a message from one process to all others in the group
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!   Via MPI broadcast.
!
! RESTRICTIONS:
!   For a not-present optional parameter, a zero is broadcasted.
!   Called without MPI activated, the routine does nothing.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_MPI_BCastReals(MyProcWo, 0, MPI_COMM_WORLD, r1=dtime)
!
! MODIFICATION HISTORY:
!   2017-08-08 (B.F. Uppsala) Cut out from rhd_MPI_boxRead and generalize
!---------------------------------------------------------------------------------------------------
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: MyProc, MaProc, Comm
real,            optional, intent(inout):: r1, r2, r3, r4
!
! --- Local variables ---
integer                                 :: ierrMPI
real, dimension(4)                      :: rVec
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
if (present(r1) .or. present(r2) .or. present(r3) .or. present(r4)) then
  if (MyProc == MaProc) then
    ! --- Fill the temporary vector ---
    rVec=0.0
    if (present(r1)) rVec(1)=r1
    if (present(r2)) rVec(2)=r2
    if (present(r3)) rVec(3)=r3
    if (present(r4)) rVec(4)=r4
  endif
  !
  ! --- Broadcast the temporaty vector ---
  call MPI_BCast(rVec, 4, MPI_REAL, MaProc, Comm, ierrMPI)
  !
  if (MyProc /= MaProc) then
    ! --- Extract values from the temporary vector ---
    if (present(r1)) r1=rVec(1)
    if (present(r2)) r2=rVec(2)
    if (present(r3)) r3=rVec(3)
    if (present(r4)) r4=rVec(4)
  endif
endif ! (present(r1) .or. present(r2) .or. present(r3) .or. present(r4))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_MPI_BCastReals


!----------******************-----------------------------------------------------------------------
subroutine rhd_MPI_DimsCreate(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                              nProcsTot, nProcsPDin, nProcsPDout)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_DimsCreate ('rhd_MPI_Dimensions_Create')
!
! PURPOSE:
!   Find and limit number of MPI processes per dimension.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_DimsCreate(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
!                           nProcsTot, nProcsPDin, nProcsPDout)
!
! INPUT:
!   m1:           ('m_1')      integer, 1. dimension, start index
!   n1:           ('number_1') integer, 1. dimension, end index
!   m2:           ('m_2')      integer, 2. dimension, start index
!   n2:           ('number_2') integer, 2. dimension, end index
!   m3:           ('m_3')      integer, 3. dimension, start index
!   n3:           ('number_3') integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   nProcsTot:    ('number_Processes_Total') integer with total number of MPI processes
!   nProcsPDin:   ('number_Process_Per_Dimension_input') 3-element integer vector with
!                 requested number of Cartesian MPI processes in each dimension
!
! OUTPUT:
!   nProcsPDout:  ('number_Process_Per_Dimension_output') 3-element integer vector with
!                 output number of Cartesian MPI processes in each dimension
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   None
!
! PROCEDURE:
!   Find reasonable number of MPI processes per dimension.
!   Make sure that there is a minimum number of cells per dimension to allow
!   boundary-condition handling in one step.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-10-05 (B.F. Uppsala) Extracted from rhd_MPI_boxRead
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           nProcsTot
integer, dimension(3),     intent(in)   :: nProcsPDin
integer, dimension(3),     intent(out)  :: nProcsPDout
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (nProcsPDin(1) <= 0) then
  nProcsPDout(1)=nProcsTot/(max(nProcsPDin(2), 1) * &
                            max(nProcsPDin(3), 1)  )
else
  nProcsPDout(1)=nProcsPDin(1)
endif
nProcsPDout(1)=max(min(nProcsPDout(1), &
                       (n1-m1+1)/max(nghost1, 1), &
                       nProcsTot), &
                   1)
!
if (nProcsPDin(2) <= 0) then
  nProcsPDout(2)=nProcsTot/(   nProcsPDout(1)    * &
                            max(nProcsPDin(3), 1) )
else
  nProcsPDout(2)=nProcsPDin(2)
endif
nProcsPDout(2)=max(min(nProcsPDout(2), &
                       (n2-m2+1)/max(nghost2, 1), &
                       nProcsTot/nProcsPDout(1)), &
                   1)
!
if (nProcsPDin(3) <= 0) then
  nProcsPDout(3)=nProcsTot/(nProcsPDout(1) * &
                            nProcsPDout(2)  )
else
  nProcsPDout(3)=nProcsPDin(3)
endif
nProcsPDout(3)=max(min(nProcsPDout(3), &
                       (n3-m3+1)/max(nghost3, 1), &
                       nProcsTot/(nProcsPDout(1) * &
                                  nProcsPDout(2)  )), &
                   1)
!
end subroutine rhd_MPI_DimsCreate


!----------*****************------------------------------------------------------------------------
subroutine rhd_MPI_PrintInfo(ncp, MyProc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_PrintInfo ('rhd_MPI_Print_Info')
!
! PURPOSE:
!   Write infomation about MPI environment to output channel.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_PrintInfo(ncp, MyProc)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel
!   MyProc:       ('My_Process') integer with rank of current process
!
! OUTPUT:
!   None
!
! ROUTINES:
!   MPI_Comm_Get_Attr:     Get an attribute value
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_Get_Library_Version: Get the name of the MPI library version
!   MPI_Get_Processor_Name: Get processor name for current process
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!   Write into channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write a number of pre-defined constants, inquired parameters, and attribute values
!   to output channel, if channel number >= -1.
!
! EXAMPLE:
!   call rhd_MPI_PrintInfo(6, 0)
!
! MODIFICATION HISTORY:
!   2012-07-26 (B.F. Lyon) Written
!   2017-07-15 (B.F. Uppsala) Print more information
!---------------------------------------------------------------------------------------------------
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp, MyProc
!
! --- Local variables ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
logical                                 :: flagMPI
integer                                 :: ierrMPI, ProcessorNameMPIlen, &
                                           NProcsWO, LibraryVersionMPIlen
integer(kind=MPI_ADDRESS_KIND)          :: rhdMPI_WTIME_IS_GLOBAL_Value, rhdMPI_TAG_UB_Value
character(len=MPI_MAX_PROCESSOR_NAME)   :: ProcessorNameMPI
character(len=MPI_MAX_LIBRARY_VERSION_STRING) &
                                        :: LibraryVersionMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
if (ncp >= 0) then
  write(ncp,'(A,I1,A,I1)') &
                         'Version:                   MPI-', &
                           MPI_VERSION, '.', MPI_SUBVERSION
  !
  call MPI_Get_Library_Version(LibraryVersionMPI, LibraryVersionMPIlen, ierrMPI)
  write(ncp,'(A)')       'Library version:           ' // trim(LibraryVersionMPI)
  !
  call MPI_Get_Processor_Name(ProcessorNameMPI, ProcessorNameMPIlen, ierrMPI)
  write(ncp,'(A)')       'Processor name for rank 0: ' // trim(ProcessorNameMPI)
  !
  call MPI_Comm_Size(MPI_COMM_WORLD, NProcsWO, ierrMPI)
  write(ncp,'(A,I13)')   'Running on processes:      ', NProcsWO
  !
  call MPI_Comm_Get_Attr(MPI_COMM_WORLD, MPI_TAG_UB, rhdMPI_TAG_UB_Value, &
                         flagMPI, ierrMPI)
  if (flagMPI) then
    write(ncp,'(A,I20)') 'Largest message tag value: ', rhdMPI_TAG_UB_Value
  endif
  !
  call MPI_Comm_Get_Attr(MPI_COMM_WORLD, MPI_WTIME_IS_GLOBAL, rhdMPI_WTIME_IS_GLOBAL_Value, &
                         flagMPI, ierrMPI)
  if (flagMPI) then
    if (rhdMPI_WTIME_IS_GLOBAL_Value == 1) then
      write(ncp,'(A)')   'MPI wallclock times are:   synchronized between processes'
    else
      write(ncp,'(A)')   'MPI wallclock times are:   not synchronized between processes'
    endif
  endif ! (flagMPI)
  !
  if (MPI_SUBARRAYS_SUPPORTED) then
    write(ncp,'(A)')     'MPI subarrays are:         supported'
  else ! (.not. MPI_SUBARRAYS_SUPPORTED)
    write(ncp,'(A)')     'MPI subarrays are:         not supported'
  endif ! (MPI_SUBARRAYS_SUPPORTED)
  !
endif
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
  write(ncp,'(A)') '  ... is not activated: only one process running'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_MPI_PrintInfo


!----------*****************------------------------------------------------------------------------
subroutine rhd_OMP_PrintInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_OMP_PrintInfo ('rhd_OpenMP_Print_Info')
!
! PURPOSE:
!   Write infomation about OpenMP environment to output channel.
!
! CATEGORY:
!   OpenMP
!
! CALLING SEQUENCE:
!   call rhd_OMP_PrintInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   MyProcWo:     ('My_Process_World') integer with rank of current process
!
! ROUTINES:
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   omp_lib:               Library of system OpenMP definitions and routines
!
! SIDE EFFECTS:
!   Write into channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_OMP_PrintInfo(6, 0)
!
! MODIFICATION HISTORY:
!   2017-08-03 (B.F. Uppsala) Move code from rhd.F90 and rhdmpi.F90 into separate routine
!   2017-08-04 (B.F.) For recent OpenMP versions, print process binding type (thread affinity)
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
integer                                 :: MyProcWo
character(len=4)                        :: prefix
character(len=80)                       :: OpenMPStr
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
integer                                 :: ierrMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
! --- OpenMP variables ---
!$ integer                              :: omp_MaxThreads, omp_SchedModifier
!$ integer(kind=omp_sched_kind)         :: omp_Sched
!#
# ifdef _OPENMP
# if (_OPENMP >= 201511)
!#
integer(kind=omp_proc_bind_kind)     :: omp_ProcBind
# endif
!#      /* (_OPENMP > 201511) */
# endif
!#      /* _OPENMP */
!---------------------------------------------------------------------------------------------------
!
! --- Determine ID of current MPI process and compose the prefix ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
call MPI_Comm_Rank(MPI_COMM_WORLD, MyProcWo, ierrMPI)
write(prefix, '(I3)') MyProcWo
prefix='#'//prefix
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
MyProcWo=0
prefix=''
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!#
# ifdef _OPENMP
!#
! --- Get and print information about OpenMP environment ---
!
write(OpenMPStr,*) _OPENMP
write(ncp,'(A)')       trim(prefix)//'  Compiler switch _OPEN'//'MP:   '//trim(OpenMPStr)
!
omp_MaxThreads=omp_get_max_threads()
write(ncp,'(A,I4)')    trim(prefix)//'  Maximum number of threads: ', omp_MaxThreads
!
call omp_get_schedule(omp_Sched, omp_SchedModifier)
if      (omp_Sched == omp_sched_auto   ) then
  write(ncp,'(A,I4)' ) trim(prefix)//'  Scheduling type: (auto)    ', omp_Sched
else if (omp_Sched == omp_sched_static ) then
  write(ncp,'(A,2I4)') trim(prefix)//'  Scheduling type: (static)  ', omp_Sched, omp_SchedModifier
else if (omp_Sched == omp_sched_dynamic) then
  write(ncp,'(A,2I4)') trim(prefix)//'  Scheduling type: (dynamic) ', omp_Sched, omp_SchedModifier
else if (omp_Sched == omp_sched_guided ) then
  write(ncp,'(A,2I4)') trim(prefix)//'  Scheduling type: (guided)  ', omp_Sched, omp_SchedModifier
else ! (omp_Sched)
  write(ncp,'(A,2I4)') trim(prefix)//'  Scheduling type:           ', omp_Sched, omp_SchedModifier
endif ! (omp_Sched)
!
!#
# if (_OPENMP >= 201511)
!#
!write(ncp,'(A,I4)'   ) trim(prefix)//'  Process binding type:      ', omp_get_proc_bind()
omp_ProcBind=omp_get_proc_bind()
if      (omp_ProcBind == omp_proc_bind_false ) then
  write(ncp,'(A,I4)' ) trim(prefix)//'  Process binding: (false)   ', omp_ProcBind
else if (omp_ProcBind == omp_proc_bind_true  ) then
  write(ncp,'(A,I4)' ) trim(prefix)//'  Process binding: (true)    ', omp_ProcBind
else if (omp_ProcBind == omp_proc_bind_master) then
  write(ncp,'(A,I4)' ) trim(prefix)//'  Process binding: (master)  ', omp_ProcBind
else if (omp_ProcBind == omp_proc_bind_close ) then
  write(ncp,'(A,I4)' ) trim(prefix)//'  Process binding: (close)   ', omp_ProcBind
else if (omp_ProcBind == omp_proc_bind_spread) then
  write(ncp,'(A,I4)' ) trim(prefix)//'  Process binding: (spread)  ', omp_ProcBind
else ! (omp_ProcBind)
  write(ncp,'(A,I4)' ) trim(prefix)//'  Process binding:           ', omp_ProcBind
endif ! (omp_ProcBind)
!#
# endif
!#      /* (_OPENMP > 201511) */
!
write(ncp,'(A,L8)')    trim(prefix)//'  Nesting enabled:       ', omp_get_nested()
!
!#
# else
!#      /* _OPENMP */
  write(ncp,'(A)')       trim(prefix)//'  ... is not activated'
!#
# endif
!#      /* _OPENMP */
!
end subroutine rhd_OMP_PrintInfo


!----------****************-------------------------------------------------------------------------
subroutine rhd_MPI_boxWrite(command, ncout, &
                            box_id, box, dtime, time_out_full_last, &
                            time_out_mean_last, &
                            file, form, conv, &
                            ndes, description, nhis, history, version, &
                            SubBoxDim, SubBoxMode, SubBoxName)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_boxWrite ('rhd_MPI_box_Write')
!
! PURPOSE:
!   Write box with additional information into file in UIO form.
!
! CATEGORY:
!   I/O, box handling
!
! CALLING SEQUENCE:
!   call rhd_MPI_boxWrite(command, ncout, &
!                         box_id=box_id, box=box, &
!                         dtime=dtime, time_out_full_last=time_out_full_last, &
!                         time_out_mean_last=time_out_mean_last, &
!                         file=file, form=form, conv=conv, &
!                         ndes=ndes, description=description, &
!                         nhis=nhis, history=history, version=version, &
!                         SubBoxDim=SubBoxDim, SubBoxMode=SubBoxmode, SubBoxName=SubBoxName)
!
! INPUT:
!   command:      ('command') character, command string:
!                   'open'  : Open new file and write header
!                   'append': Open existing file
!                   'model' : Write 'dataset' label and general model data at beginning of dataset
!                   'box':    Write box
!                   'endmo':  Write 'end dataset' label
!                   'close':  Close the file
!   --- optional: ---
!   file:         ('file') character, name of data file
!   form:         ('format') character, name of data file
!   conv:         ('conversion') character, conversion type of data file
!   box_id:       ('box_identifier') character, identifier for box, default='z'
!   box:          ('box') type(box_type), input computational box
!   dtime:        ('delta_time') real, time step
!   time_out_full_last: ('time_output_full_last') real, time of last output of full model
!   time_out_mean_last: ('time_output_mean_last') real, time of last output of averaged data
!   ndes:         ('number_description') integer, number of lines of description
!   description:  ('description') character, dimension(:), description of model
!   nhis:         ('number_history') integer, number of lines of history
!   history:      ('history') character, dimension(:), history of model
!   version:      ('version') character, program version
!   SubBoxDim:    ('Sub_Box_Dimension') optional 2x3 real array with output dimension
!   SubBoxMode:   ('Sub_Box_Mode') optional character with mode how to find or interpret
!                 the output dimension
!   SubBoxName:   ('Sub_box_Name') optional character with name of output if SubBoxDim is specified,
!                 e.g., 'Chunk 1'
!
! OUTPUT:
!   None
!
! INPUT/OUTPUT:
!   ncout:        ('number_channel_output') integer, unit number for data output
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in cartesian topology from rank in group
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_MPICreateType: Create MPI data type for box structure
!   rhd_box_SubBoxIndices: Compute indices of box representing the section of both input boxes
!   rhd_box_WrData:        Write RHD model into file in UIO form
!   rhd_box_Write:         Write box with additional infos into file in UIO form
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   MPI:                   Standard MPI routines and definitions
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_io_module:         Input/output routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_sub_module:        Additional subroutines for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   If the command string contains 'open', the optional parameter 'file' has to be specified.
!   If the command string contains 'model', the optional parameters
!   'box', 'dtime', 'time_out_full_last' have to be specified.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-09-06 (B.F. Lyon) First version
!   2017-07-29 (B.F. Uppsala) Make the routine work also without MPI activated
!   2017-08-02 (B.F.) Allow output of sub box
!   2017-08-08 (B.F.) Add timing routines
!---------------------------------------------------------------------------------------------------
use const_module
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_io_module
use rhd_sub_module
use timing_module
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
use MPI
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(inout):: ncout
character(len=*),optional, intent(in)   :: file, form, conv, box_id
integer,         optional, intent(in)   :: ndes
character(len=*), dimension(:), &
                 optional, intent(in)   :: description
integer,         optional, intent(in)   :: nhis
character(len=*), dimension(:), &
                 optional, intent(in)   :: history
character(len=*),optional, intent(in)   :: version, SubBoxMode, SubBoxName
type(box_type),  optional, intent(in)   :: box
real,            optional, intent(in)   :: dtime, time_out_full_last, time_out_mean_last
integer, dimension(2,3), &
                 optional, intent(in)   :: SubBoxDim
!
! --- Local variables ---
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
integer                                 :: ierrMPI, &
                                           NProcsHD, SoProcHD, iSoProcHD, &
                                           m1,n1, m2,n2, m3,n3, &
                                           m1c,n1c, m2c,n2c, m3c,n3c, &
                                           m1b,n1b, m2b,n2b, m3b,n3b
logical                                 :: IsEmpty_flag, IsComplete_flag
integer, dimension(1)                   :: NewRanks
integer, dimension(3)                   :: PCoords
integer, dimension(:), allocatable      :: rhdMPI_boxLrgTypes, rhdMPI_boxTypes, RequestsMPI
integer, dimension(:,:), allocatable    :: StatusMPI
type(box_type)                          :: boxLrg
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
if (wg%hyd%MyProcIsInCOMM_flag) then
  !
  ! write(gl%nc_p, '(A,I3,A,2(1PE14.6))') '#', prop%MyProcWo, &
  !     ' rhd_MPI_boxWrite: max(box%quc(-8:8..,1;2)): ', &
  !     maxval(box%quc(max(box%m1,-8):min(box%n1,8), &
  !                    max(box%m2,-8):min(box%n2,8), &
  !                    max(box%m3,-8):min(box%n3,8),  1)), &
  !     maxval(box%quc(max(box%m1,-8):min(box%n1,8), &
  !                    max(box%m2,-8):min(box%n2,8), &
  !                    max(box%m3,-8):min(box%n3,8),  2))
  !
  if (prop%MyProcWo == 0) then
    ! --- Only the master process opens the file and writes to it ---
    !
    if (index(command,'open') /= 0) then
      ! --- Open file for writing in uio-form ---
      call rhd_box_Write('open', ncout, file=file, form=form, conv=conv, &
                         ndes=ndes, description=description, nhis=nhis, history=history, &
                         version=version)
    else if (index(command,'append') /= 0) then
      ! --- Append data to file ---
      call rhd_box_Write('append', ncout, file=file, form=form, conv=conv, &
                         ndes=ndes, description=description, nhis=nhis, history=history, &
                         version=version)
    endif
    !
    if (index(command,'model') /= 0) then
      ! --- Write model header (time, time step, etc.) ---
      call rhd_box_Write('model', ncout, &
                         box=box, &
                         dtime=dtime, time_out_full_last=time_out_full_last, &
                         time_out_mean_last=time_out_mean_last)
    endif
  endif ! (prop%MyProcWo == 0)
  !
  !
  if (index(command,'box') /= 0) then
    ! === Collect boxes from the compute processes ===
    !
    ! --- Compute indices of global output box ---
    call rhd_box_SubBoxIndices(prop%m1,prop%n1, prop%m2,prop%n2, prop%m3,prop%n3, &
                               0  ,0  , 0  ,0  , 0  ,0  , &
                               m1c,n1c, m2c,n2c, m3c,n3c, &
                               SubBoxMode=SubBoxMode, SubBoxDim=SubBoxDim, &
                               IsEmpty_flag=IsEmpty_flag)
    !
    ! if ((prop%MyProcWo == 0) .and. (present(SubBoxName))) then
    !   write(gl%nc_p,'(A,3(I5,1X,I5,3X),L1)') &
    !                            merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
    !                            ' output; m1c,n1c, m2c,n2c, m3c,n3c: ', &
    !                                      m1c,n1c, m2c,n2c, m3c,n3c, (.not. IsEmpty_flag)
    !   call flush(gl%nc_p)
    ! endif
    !
    if (.not. IsEmpty_flag) then
      !
      call timing_start('IO:   collect')
      !
      if (prop%MyProcWo == 0) then
        ! --- The current process is the master that will collect and write the complete data set --
        !
        ! write(gl%nc_p, '(A)') 'Collect boxes'
        ! flush(gl%nc_p)
        !
        ! --- Create the output box ---
        ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, &
        !   ' Create large box'
        call rhd_box_Make(boxLrg, m1c,n1c, m2c,n2c, m3c,n3c, &
                          quc_flag=.true., Bb_flag=(prop%nBb /= 0))
        !
        ! --- Create temporary arrays ---
        call MPI_Comm_Size(wg%hyd%Comm, NProcsHD, ierrMPI)
        iSoProcHD=NProcsHD-1
        allocate(rhdMPI_boxLrgTypes(        1:iSoProcHD), &
                 RequestsMPI(               1:iSoProcHD), &
                 StatusMPI(MPI_STATUS_SIZE, 1:iSoProcHD))
        !
        ! --- Loop over the source processes ---
        iSoProcHD=0
        do SoProcHD=0,NProcsHD-1
          ! --- Get source-process coordinates ---
          call MPI_Cart_Coords(wg%hyd%Comm, SoProcHD, 3, PCoords, ierrMPI)
          call rhd_geo_Coords2mn(wg%hyd, PCoords, m1,n1, m2,n2, m3,n3)
          !
          ! --- Compute indices of local output box ---
          call rhd_box_SubBoxIndices(m1,n1, m2,n2, m3,n3, &
                                     m1c,n1c, m2c,n2c, m3c,n3c, &
                                     m1b,n1b, m2b,n2b, m3b,n3b, &
                                     SubBoxMode='Dimensions-absolute', &
                                     IsEmpty_flag=IsEmpty_flag, IsComplete_flag=IsComplete_flag)
          !
          ! write(gl%nc_p, '(A,I3,A,3(2X,2I5),A,I3)') '#', wg%hyd%MyProc, ' Dim: ', &
          !                                           m1b,n1b, m2b,n2b, m3b,n3b, &
          !                                           ' -> P:', SoProcHD
          !
          if (.not. IsEmpty_flag) then
            if (SoProcHD == wg%hyd%MyProc) then
              ! --- Target is identical to source: just copy the box locally ---
              ! --- ToDoMPI: OMP ---
              !
              ! write(gl%nc_p, '(A,I3,A,I3)') '#', wg%hyd%MyProc, &
              !   ' Start copying small local box into large box ', SoProcHD
              call rhd_box_Copy(box   , m1b,n1b, m2b,n2b, m3b,n3b, &
                                boxLrg, m1b,     m2b,     m3b, &
                                trans_mode=1, time_mode=1, grid_mode=1, bulk_mode=1, flux_mode=0)
              ! write(gl%nc_p, '(A,I3,A,I3)') '#', wg%hyd%MyProc, &
              !   ' Stop copying small local box into large box  ', SoProcHD
              !
            else ! (SoProcHD /= wg%hyd%MyProc)
              ! --- Data has to be received from different process ---
              !
              iSoProcHD=iSoProcHD+1
              ! --- Create MPI data type ---
              ! write(gl%nc_p, '(A,I3,A,I3)') '#', wg%hyd%MyProc, &
              !   ' Create MPI type for part of large box ', SoProcHD
              call rhd_box_MPICreateType(boxLrg, m1b,n1b, m2b,n2b, m3b,n3b, &
                                         rhdMPI_boxLrgTypes(iSoProcHD), &
                                         time_mode=1, grid_mode=1, bulk_mode=2)
              !
              ! --- Request to get the box ---
              ! write(gl%nc_p, '(A,I3,A,2I3)') '#', wg%hyd%MyProc, &
              !   ' Receive part of large box from ', SoProcHD
              call MPI_IRecv(boxLrg, 1, rhdMPI_boxLrgTypes(iSoProcHD), SoProcHD, &
                             MPI_ANY_TAG, wg%hyd%Comm, RequestsMPI(iSoProcHD), ierrMPI)
              !
            endif ! (SoProcHD)
          endif ! (.not. IsEmpty_flag)
          !
        end do ! SoProcHD
        !
        ! --- Wait until all boxes are copied ---
        ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Wait until all boxes are received'
        call MPI_WaitAll(iSoProcHD, RequestsMPI, StatusMPI, ierrMPI)
        ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, &
        !   ' Finished waiting that all boxes are received'
        !
        ! --- Free data types ---
        do SoProcHD=1,iSoProcHD
          call MPI_Type_Free(rhdMPI_boxLrgTypes(SoProcHD), ierrMPI)
        end do ! SoProcHD
        !
        ! --- Deallocate arrays ---
        deallocate(rhdMPI_boxLrgTypes, &
                   RequestsMPI, &
                   StatusMPI)
        !
      else ! (prop%MyProcWo /= 0)
        ! --- The current process is one of the HD compute processes but definitely not WORLD(0) ---
        !
        ! --- Create temporary arrays ---
        allocate(rhdMPI_boxTypes(1:1), &
                 RequestsMPI(    1:1), &
                 StatusMPI(MPI_STATUS_SIZE, 1))
        !
        ! --- Get local process coordinates ---
        call MPI_Cart_Coords(wg%hyd%Comm, wg%hyd%MyProc, 3, PCoords, ierrMPI)
        call rhd_geo_Coords2mn(wg%hyd, PCoords, m1,n1, m2,n2, m3,n3)
        !
        ! --- Compute indices of local output box ---
        call rhd_box_SubBoxIndices(m1,n1, m2,n2, m3,n3, &
                                   m1c,n1c, m2c,n2c, m3c,n3c, &
                                   m1b,n1b, m2b,n2b, m3b,n3b, &
                                   SubBoxMode='Dimensions-absolute', &
                                   IsEmpty_flag=IsEmpty_flag)
        if (.not. IsEmpty_flag) then
          !
          ! --- Create MPI data type ---
          ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, &
          !   ' Create MPI type for part of small local box'
          call rhd_box_MPICreateType(box, m1b,n1b, m2b,n2b, m3b,n3b, &
                                     rhdMPI_boxTypes(1), &
                                     time_mode=1, grid_mode=1, bulk_mode=2)
          !
          ! --- Receive it ---
          ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Send small local box'
          call MPI_ISend(box, 1, rhdMPI_boxTypes(1), wg%hyd%MaProcWoInCO, &
                         wg%hyd%MyProc, wg%hyd%Comm, RequestsMPI(1), ierrMPI)
          !
          ! --- Wait until all boxes are sent ---
          ! write(gl%nc_p, '(A,I3,A)') &
          !                        '#', wg%hyd%MyProc, ' Wait until small local box has been sent'
          call MPI_WaitAll(1, RequestsMPI, StatusMPI, ierrMPI)
          ! write(gl%nc_p, '(A,I3,A)') &
          !                   '#', wg%hyd%MyProc, ' Finished waiting for small local box to be sent'
          !
          ! --- Free data types ---
          call MPI_Type_Free(rhdMPI_boxTypes(1), ierrMPI)
          !
          ! --- Deallocate arrays ---
          deallocate(rhdMPI_boxTypes, &
                     RequestsMPI, &
                     StatusMPI)
          !
        endif ! (IsEmpty_flag)
      endif ! (prop%MyProcWo)
      !
      call timing_stop('IO:   collect')
      !
      if (prop%MyProcWo == 0) then
        ! --- Write box corresponding to entire model ---
        ! write(gl%nc_p, '(A,I3,A)') '#', prop%MyProcWo, ' Write large box to file'
        ! write(gl%nc_p, '(A,I3,A,2(1PE14.6))') '#', prop%MyProcWo, &
        !     ' rhd_MPI_boxWrite: max(boxLrg%quc(-8:8..,1;2)): ', &
        !     maxval(boxLrg%quc(max(boxLrg%m1,-8):min(boxLrg%n1,8), &
        !                       max(boxLrg%m2,-8):min(boxLrg%n2,8), &
        !                       max(boxLrg%m3,-8):min(boxLrg%n3,8),  1)), &
        !     maxval(boxLrg%quc(max(boxLrg%m1,-8):min(boxLrg%n1,8), &
        !                       max(boxLrg%m2,-8):min(boxLrg%n2,8), &
        !                       max(boxLrg%m3,-8):min(boxLrg%n3,8),  2))
        !
        call timing_start('IO: write UIO box')
        call rhd_box_WrData('box', gl%nc_p, ncout, &
                           time_unit=time_unit, &
                           x1_unit=x1_unit, x2_unit=x2_unit, x3_unit=x3_unit, &
                           rho_unit=rho_unit, v_unit=v_unit, e_unit=e_unit, &
                           box_id=box_id, &
                           m1=boxLrg%m1, m2=boxLrg%m2, m3=boxLrg%m3, &
                           n1=boxLrg%n1, n2=boxLrg%n2, n3=boxLrg%n3, &
                           itime=boxLrg%itime, time=boxLrg%time, time_db=boxLrg%time_db, &
                           dtime=dtime, &
                           xc1=boxLrg%xc1, xc2=boxLrg%xc2, xc3=boxLrg%xc3, &
                           xb1=boxLrg%xb1, xb2=boxLrg%xb2, xb3=boxLrg%xb3, &
                           rho=boxLrg%rho, ei=boxLrg%ei, v1=boxLrg%v1, v2=boxLrg%v2, v3=boxLrg%v3, &
                           quc=boxLrg%quc, Bb1=boxLrg%Bb1, Bb2=boxLrg%Bb2, Bb3=boxLrg%Bb3)
        call timing_stop('IO: write UIO box')
        !
        ! --- Delete large input box ---
        call rhd_box_Delete(boxLrg)
      endif ! (prop%MyProcWo == 0)
    endif ! (.not. IsEmpty_flag)
  endif ! (index(command,'box') /= 0)
  !
  !
  if (prop%MyProcWo == 0) then
    ! --- Only the master process writes data to the file and closes it ---
    !
    if (index(command,'endmo') /= 0) then
      ! --- Close file ---
      call rhd_box_Write('endmo', ncout)
    endif
    !
    if (index(command,'close') /= 0) then
      ! --- Close file ---
      call rhd_box_Write('close', ncout)
    endif
  endif ! (prop%MyProcWo == 0)
  !
endif ! (wg%hyd%MyProcIsInCOMM_flag)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
! --- Just write the model ---
if (index(command,'box') /= 0) call timing_start('IO: write UIO box')
call rhd_box_Write(command, ncout, box_id=box_id, box=box, &
                   dtime=dtime, time_out_full_last=time_out_full_last, &
                   time_out_mean_last=time_out_mean_last, &
                   file=file, form=form, conv=conv, &
                   ndes=ndes, description=description, &
                   nhis=nhis, history=history, version=version, &
                   SubBoxDim=SubBoxDim, SubBoxMode=SubBoxmode, SubBoxName=SubBoxName)
if (index(command,'box') /= 0) call timing_stop('IO: write UIO box')
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_MPI_boxWrite


!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ======================================================== */


!----------*****************------------------------------------------------------------------------
subroutine rhd_MPI_CartSetup(MyProcWo, geo, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_CartSetup ('rhd_MPI_Cartesian_Setup')
!
! PURPOSE:
!   Set up Cartesian MPI communicator and put data into geometry variable.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_MPI_CartSetup(MyProcWo, geo, action)
!
! INPUT:
!   MyProcWo:     ('My_Process_World') MPI process ID within world communicator
!
! OUTPUT:
!   None
!
! INPUT/OUTPUT:
!   geo:          ('geometry') structure of type(geo_type) with geometry data
!   action:       ('action') derived type with collection of control parameters
!
! ROUTINES:
!   flush:                 Flush output buffer
!   MPI_Barrier:           Block untill all processes have reached this routine
!   MPI_BCast:             Broadcast a message from one process to all others in the group
!   MPI_Cart_Coords:       Determine process coordinates in cartesian topology from rank in group
!   MPI_Cart_Create:       Create a new communicator with attached cartesian topology information
!   MPI_Cart_Shift:        Compute the shifted source and destination ranks from direction a. amount
!   MPI_Comm_Create:       Create a new communicator
!   MPI_Comm_Free:         Free a communicator
!   MPI_Comm_Get_Errhandler: Retrieve error handler associated with a communicator
!   MPI_Comm_Group:        Return the group associated with a communicator
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!   MPI_Comm_Set_ErrHandler: Attach a new error handler to a communicator
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_Group_Excl:        Produce a group by removing listed members
!   MPI_Group_Free:        Free a group
!   MPI_Group_Incl:        Produce a group by taking only listed members
!   MPI_Group_Translate_Ranks: Translate the ranks of processes in one group to those in another one
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-08 (B.F. Uppsala) Extract from rhd_MPI_boxRead
!   2017-09-10 (B.F.) Modify insertion of world master process into Cartesian communicator
!   2017-10-05 (B.F.) Work on new parameter geo instead of global structure wg%hyd
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_gl_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: MyProcWo
type(geo_type),            intent(inout):: geo
type(action_type),         intent(inout):: action
!
! --- Local variables ---
integer                                 :: ierrMPI, &
                                           rhdMPI_GROUP_WORLD, NProcsWO, &
                                           rhdMPI_GROUP_MASTER, &
                                           rhdMPI_GROUP_GEO, NProcsGEO, TaProcGEO, &
                                           rhdMPI_GROUP_GEOm1, &
                                           rhdMPI_GROUP_GEOtmp, rhdMPI_COMM_GEOtmp, &
                                           rhdMPI_ERRORS_HANDLE, &
                                           m1,n1, m2,n2, m3,n3
integer, dimension(1)                   :: NewRanks
integer, dimension(3)                   :: PCoords
logical, dimension(3)                   :: IsPeriodic_Flags
logical                                 :: MyProcIsInCOMM_GEO_flag
!---------------------------------------------------------------------------------------------------
!
! --- Create communicator describing Cartesian geometry ---
IsPeriodic_Flags=(geo%bound_l == bound_periodic)
call MPI_Cart_Create(MPI_COMM_WORLD, 3, geo%nProcsPD, IsPeriodic_Flags, .true., &
                     geo%Comm, ierrMPI)
call MPI_Comm_Size(geo%Comm, NProcsGEO, ierrMPI)
!
! --- Check if world master process is in rhdMPI_COMM_GEO ---
call MPI_Comm_Get_Errhandler(MPI_COMM_WORLD, rhdMPI_ERRORS_HANDLE, ierrMPI)
call MPI_Comm_Set_ErrHandler(MPI_COMM_WORLD, MPI_ERRORS_RETURN, ierrMPI)
if (MyProcWo == 0) then
  call MPI_Comm_Rank(geo%Comm, geo%MyProc, ierrMPI)
  MyProcIsInCOMM_GEO_flag=(ierrMPI == 0)
endif
call MPI_Comm_Set_ErrHandler(MPI_COMM_WORLD, rhdMPI_ERRORS_HANDLE, ierrMPI)
! --- Tell the other processes about it ---
call MPI_BCast(MyProcIsInCOMM_GEO_flag, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, ierrMPI)
!
call MPI_Comm_Group(MPI_COMM_WORLD, rhdMPI_GROUP_WORLD, ierrMPI)
!
if (.not. MyProcIsInCOMM_GEO_flag) then
  ! --- World master process is initially not in GEO communicator: add it ---
  !
  if (MyProcWo == 0) then
    write(gl%nc_p, '(A)') 'World master process is not yet in GEO communicator: adding it.'
  endif
  !
  ! --- Remove last process from rhdMPI_COMM_GEO ---
  call MPI_Comm_Group(geo%Comm, rhdMPI_GROUP_GEO, ierrMPI)
  call MPI_Group_Excl(rhdMPI_GROUP_GEO, 1, (/ NProcsGEO /), rhdMPI_GROUP_GEOm1, ierrMPI)
  call MPI_Group_Free(rhdMPI_GROUP_GEO, ierrMPI)
  !
  ! --- Create group with master only ---
  call MPI_Group_Incl(rhdMPI_GROUP_WORLD, 1, (/ 0 /), rhdMPI_GROUP_MASTER, ierrMPI)
  !
  ! --- Merge master and GEO-1 group ---
  call MPI_Group_Union(rhdMPI_GROUP_MASTER, rhdMPI_GROUP_GEOm1, rhdMPI_GROUP_GEOtmp, ierrMPI)
  call MPI_Group_Free(rhdMPI_GROUP_GEOm1, ierrMPI)
  call MPI_Group_Free(rhdMPI_GROUP_MASTER, ierrMPI)
  !
  ! --- Recreate rhdMPI_COMM_GEO (via tmp...) ---
  call MPI_Comm_Free(geo%Comm, ierrMPI)
  call MPI_Comm_Create(MPI_COMM_WORLD, rhdMPI_GROUP_GEOtmp, rhdMPI_COMM_GEOtmp, ierrMPI)
  call MPI_Cart_Create(rhdMPI_COMM_GEOtmp, 3, geo%nProcsPD, IsPeriodic_Flags, .true., &
                       geo%Comm, ierrMPI)
  call MPI_Group_Free(rhdMPI_GROUP_GEOtmp, ierrMPI)
  call MPI_Comm_Free(rhdMPI_COMM_GEOtmp, ierrMPI)
endif ! (MyProcIsInCOMM_GEO_flag)
!
! --- Find ID of world master process in rhdMPI_COMM_GEO ---
call MPI_Comm_Group(geo%Comm, rhdMPI_GROUP_GEO   , ierrMPI)
call MPI_Group_Translate_Ranks(rhdMPI_GROUP_WORLD, 1, (/ 0 /), rhdMPI_GROUP_GEO, &
                               NewRanks, ierrMPI)
geo%MaProcWoInCO=NewRanks(1)
!
! --- Find out if current process is in communicator rhdMPI_COMM_GEO ---
call MPI_Comm_Get_Errhandler(MPI_COMM_WORLD, rhdMPI_ERRORS_HANDLE, ierrMPI)
call MPI_Comm_Set_ErrHandler(MPI_COMM_WORLD, MPI_ERRORS_RETURN, ierrMPI)
call MPI_Comm_Rank(geo%Comm, geo%MyProc, ierrMPI)
geo%MyProcIsInCOMM_flag=(ierrMPI == 0)
!write(gl%nc_p, '(A,2I5)') 'geo%MyProc, ierrMPI: ', geo%MyProc, ierrMPI
!call MPI_Comm_Set_ErrHandler(MPI_COMM_WORLD, MPI_ERRORS_ARE_FATAL, ierrMPI)
call MPI_Comm_Set_ErrHandler(MPI_COMM_WORLD, rhdMPI_ERRORS_HANDLE, ierrMPI)
!call MPI_Comm_Group(geo%Comm, rhdMPI_GROUP_GEO, ierrMPI)
!write(gl%nc_p, '(A,2I5)') 'IsInGroup A MyProcWo, ierrMPI: ', MyProcWo, ierrMPI
!call MPI_Group_Rank(rhdMPI_GROUP_GEO, geo%MyProc, ierrMPI)
!write(gl%nc_p, '(A,2I5)') 'IsInGroup B MyProcGEO, ierrMPI: ', geo%MyProc, ierrMPI
!call MPI_Group_Free(rhdMPI_GROUP_GEO, ierrMPI)
!write(gl%nc_p, '(A,2I5)') 'IsInGroup C MyProcGEO, ierrMPI: ', geo%MyProc, ierrMPI
!
! --- Free some group handles ---
call MPI_Group_Free(rhdMPI_GROUP_GEO   , ierrMPI)
call MPI_Group_Free(rhdMPI_GROUP_WORLD , ierrMPI)
!
! --- Set dummy values for left and right neighbors and Cartesian process coordinates ---
geo%Proc_l =MPI_PROC_NULL
geo%Proc_r =MPI_PROC_NULL
geo%iProcPD=-1
!
if (geo%MyProcIsInCOMM_flag) then
  ! --- Change boundary-condition identifier to internal if necessary ---
  !
  call MPI_Cart_Coords(geo%Comm, geo%MyProc, 3, PCoords, ierrMPI)
  geo%iProcPD=PCoords
  if (geo%nProcsPD(1) > 1) then
    ! --- More than one process along this dimension ---
    if (PCoords(1) == 0) then
      ! --- Leftmost box ---
      if (action%bound_l1 == bound_periodic) then
        ! --- Periodic boundary conditions ---
        action=rhd_action_New(action, bound_l1=bound_InternalPeriodic)
      endif
    else
      ! --- Not leftmost box and non-periodic boundary conditions ---
      action=rhd_action_New(action, bound_l1=bound_InternalNext)
    endif
    !
    if (PCoords(1) == geo%nProcsPD(1)-1) then
      ! --- Rightmost box ---
      if (action%bound_r1 == bound_periodic) then
        ! --- Periodic boundary conditions ---
        action=rhd_action_New(action, bound_r1=bound_InternalPeriodic)
      endif
    else
      ! --- Not rightmost box and non-periodic boundary conditions ---
      action=rhd_action_New(action, bound_r1=bound_InternalNext)
    endif
  endif ! (geo%nProcsPD(1) > 1)
  !
  if (geo%nProcsPD(2) > 1) then
    ! --- More than one process along this dimension ---
    if (PCoords(2) == 0) then
      ! --- Leftmost box ---
      if (action%bound_l2 == bound_periodic) then
        ! --- Periodic boundary conditions ---
        action=rhd_action_New(action, bound_l2=bound_InternalPeriodic)
      endif
    else
      ! --- Not leftmost box and non-periodic boundary conditions ---
      action=rhd_action_New(action, bound_l2=bound_InternalNext)
    endif
    !
    if (PCoords(2) == geo%nProcsPD(2)-1) then
      ! --- Rightmost box ---
      if (action%bound_r2 == bound_periodic) then
        ! --- Periodic boundary conditions ---
        action=rhd_action_New(action, bound_r2=bound_InternalPeriodic)
      endif
    else
      ! --- Not rightmost box and non-periodic boundary conditions ---
      action=rhd_action_New(action, bound_r2=bound_InternalNext)
    endif
  endif ! (geo%nProcsPD(2) > 1)
  !
  if (geo%nProcsPD(3) > 1) then
    ! --- More than one process along this dimension ---
    if (PCoords(3) == 0) then
      ! --- Leftmost box ---
      if (action%bound_l3 == bound_periodic) then
        ! --- Periodic boundary conditions ---
        action=rhd_action_New(action, bound_l3=bound_InternalPeriodic)
      endif
    else
      ! --- Not leftmost box and non-periodic boundary conditions ---
      action=rhd_action_New(action, bound_l3=bound_InternalNext)
    endif
    !
    if (PCoords(3) == geo%nProcsPD(3)-1) then
      ! --- Rightmost box ---
      if (action%bound_r3 == bound_periodic) then
        ! --- Periodic boundary conditions ---
        action=rhd_action_New(action, bound_r3=bound_InternalPeriodic)
      endif
    else
      ! --- Not rightmost box and non-periodic boundary conditions ---
      action=rhd_action_New(action, bound_r3=bound_InternalNext)
    endif
  endif ! (geo%nProcsPD(3) > 1)
  !
  !write(gl%nc_p, '(A,I3,A,3(2I3,1X))') '#', geo%MyProc, ' Boundaries: ', &
  !                                      action%bound_l1, action%bound_r1, &
  !                                      action%bound_l2, action%bound_r2, &
  !                                      action%bound_l3, action%bound_r3
  !write(gl%nc_p, '(A,I3,A,3(A,1X,A,3X))') '#', geo%MyProc, ' action: bound: ', &
  !            trim(NameAr_bound(action%bound_l1)), trim(NameAr_bound(action%bound_r1)), &
  !            trim(NameAr_bound(action%bound_l2)), trim(NameAr_bound(action%bound_r2)), &
  !            trim(NameAr_bound(action%bound_l3)), trim(NameAr_bound(action%bound_r3))
  !
  ! --- Find left and right neighbors in all relevant dimensions ---
  if (geo%nProcsPD(1) > 1) then
    ! --- Find neighbors in  x1 direction ---
    call MPI_Cart_Shift(geo%Comm, 0,  1, geo%Proc_l(1), geo%Proc_r(1), ierrMPI)
  endif
  if (geo%nProcsPD(2) > 1) then
    ! --- Find neighbors in  x2 direction ---
    call MPI_Cart_Shift(geo%Comm, 1,  1, geo%Proc_l(2), geo%Proc_r(2), ierrMPI)
  endif
  if (geo%nProcsPD(3) > 1) then
    ! --- Find neighbors in  x3 direction ---
    call MPI_Cart_Shift(geo%Comm, 2,  1, geo%Proc_l(3), geo%Proc_r(3), ierrMPI)
  endif
endif ! (geo%MyProcIsInCOMM_flag)
!
call flush(gl%nc_p)
call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!
if (MyProcWo == 0) then
  ! --- Print requested and resulting number of MPI processes per dimension ---
  write(gl%nc_p, '(A,3I5)') 'Requested process partition: ', par%aN_MPIProcsPerDim
  write(gl%nc_p, '(A,3I5)') '               -> partition: ', geo%nProcsPD
endif
!
!!$! --- Print some additional output ---
!!$if (MyProcWo == 0) then
!!$  do TaProcGEO=0,NProcsGEO-1
!!$    ! --- Get process coordinates ---
!!$    call MPI_Cart_Coords(geo%Comm, TaProcGEO, 3, PCoords, ierrMPI)
!!$    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!!$    !
!!$    write(gl%nc_p, '(A,I3,A,3I3,1X,3(2X,2I5),A,I3)') &
!!$      '#', geo%MyProc, ' Dim:    ', &
!!$      PCoords, m1,n1, m2,n2, m3,n3, ' -> P: ', TaProcGEO
!!$    !
!!$  end do ! TaProcGEO
!!$endif
!!$if (geo%MyProcIsInCOMM_flag) then
!!$  if (geo%Proc_l(1) /= MPI_PROC_NULL) then
!!$    ! --- Get and print coordinates of neighbors ---
!!$    call MPI_Cart_Coords(geo%Comm, geo%Proc_l(1), 3, PCoords, ierrMPI)
!!$    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!!$    write(gl%nc_p, '(A,I3,A,3I3,1X,3(2X,2I5),A,I3)') &
!!$      '#', geo%MyProc, ' Dim: l1 ', &
!!$      PCoords, m1,n1, m2,n2, m3,n3, ' -> P: ', geo%Proc_l(1)
!!$  endif
!!$  !
!!$  if (geo%Proc_r(1) /= MPI_PROC_NULL) then
!!$    ! --- Get and print coordinates of neighbors ---
!!$    call MPI_Cart_Coords(geo%Comm, geo%Proc_r(1), 3, PCoords, ierrMPI)
!!$    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!!$    write(gl%nc_p, '(A,I3,A,3I3,1X,3(2X,2I5),A,I3)') &
!!$      '#', geo%MyProc, ' Dim: r1 ', &
!!$      PCoords, m1,n1, m2,n2, m3,n3, ' -> P: ', geo%Proc_r(1)
!!$  endif
!!$  !
!!$  if (geo%Proc_l(3) /= MPI_PROC_NULL) then
!!$    ! --- Get and print coordinates of neighbors ---
!!$    call MPI_Cart_Coords(geo%Comm, geo%Proc_l(3), 3, PCoords, ierrMPI)
!!$    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!!$    write(gl%nc_p, '(A,I3,A,3I3,1X,3(2X,2I5),A,I3)') &
!!$      '#', geo%MyProc, ' Dim: l3 ', &
!!$      PCoords, m1,n1, m2,n2, m3,n3, ' -> P: ', geo%Proc_l(3)
!!$  endif
!!$  !
!!$  if (geo%Proc_r(3) /= MPI_PROC_NULL) then
!!$    ! --- Get and print coordinates of neighbors ---
!!$    call MPI_Cart_Coords(geo%Comm, geo%Proc_r(3), 3, PCoords, ierrMPI)
!!$    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!!$    write(gl%nc_p, '(A,I3,A,3I3,1X,3(2X,2I5),A,I3)') &
!!$      '#', geo%MyProc, ' Dim: r3 ', &
!!$      PCoords, m1,n1, m2,n2, m3,n3, ' -> P: ', geo%Proc_r(3)
!!$  endif
!!$endif ! (geo%MyProcIsInCOMM_flag)
!
end subroutine rhd_MPI_CartSetup


!----------***************--------------------------------------------------------------------------
subroutine rhd_MPI_boxRead(ncin, MyProcWo, action, box, &
                           Bb_flag, &
                           dtime, time_out_full_last, time_out_mean_last, &
                           file, imodel, nhis, history)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_MPI_boxRead ('rhd_MPI_box_Read')
!
! PURPOSE:
!   Read box from file in UIO form and distribute it to other processes.
!
! CATEGORY:
!   MPI, I/O, box handling
!
! CALLING SEQUENCE:
!   call rhd_MPI_boxRead(ncin, MyProcWo, action, box, &
!                        Bb_flag=Bb_flag, &
!                        dtime=dtime, time_out_full_last=time_out_full_last, &
!                        time_out_mean_last=time_out_mean_last, &
!                        file=file, imodel=imodel, nhis=nhis, history=history)
!
! INPUT:
!   MyProcWo:     ('My_Process_World') MPI process ID within world communicator
!   Bb_flag:      ('B_boundary_flag') optional logical indicating to read boundary-located
!                 magnetic fields, if available in file
!   file:         ('file') optional character, name of data file
!   imodel:       ('indes_model') optional integer with snapshot number (default: 1 -- the first)
!
! OUTPUT:
!   box:          ('box') type(box_type), computational box
!   dtime:        ('delta_time')  optional real, time step
!   time_out_full_last: ('time_output_full_last') optional real, time of last output of full model
!   time_out_mean_last: ('time_output_mean_last') optional real, time of last output of mean data
!   nhis:         ('number_history') optional integer with number of lines of history
!   history:      ('history') optional character vector with history of model
!
! INPUT/OUTPUT:
!   ncin:         ('number_channel_input') integer, unit number for data input
!   action:       ('action') derived type with collection of control parameters
!
! ROUTINES:
!   flush:                 Flush output buffer
!   MPI_Barrier:           Block untill all processes have reached this routine
!   MPI_Cart_Coords:       Determine process coordinates in cartesian topology from rank in group
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_FillGhostAxes: Fill ghost cells of box axes for real boundaries
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_MPICreateType: Create MPI data type for box structure
!   rhd_box_MPIFillGhostAxes: Fill axes in ghost layers by communicating with neighbor MPI processes
!   rhd_box_Read:          Read box from file in UIO form
!   rhd_MPI_BCastReals:    Broadcast a number of real scalars in one go
!   rhd_MPI_CartSetup:     Set up Cartesian MPI communicator and put data into geometry variable
!   rhd_MPI_DimsCreate:    Find and limit number of MPI processes per dimension
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!   rhd_prop_MPIBCast:     Broadcast some data in the prop structure to other MPI processes
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_io_module:         Input/output routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_sub_module:        Additional subroutines for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Entries in global prop and wg%hyd structures are changed.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-07-27 (B.F. Lyon) First version
!   2012-08-03 (B.F.) Next version
!   2012-09-05 (B.F.) Boundary conditions
!   2012-10-12 (B.F.) Output: MyProcIsInCOMM_HD_flag
!   2012-10-19 (B.F.) Some changes to ensure that the world master process is in the HD communicator
!   2012-10-23 (B.F.) Ensure that HD communicator contains world master
!   2012-10-25 (B.F.) Bugfixes
!   2014-10-27 (B.F.) Set minimum number of ghost cells differently according the (M)HD solver
!   2016-03-20 (B.F. Uppsala) Put some geometrical data into global structure
!   2017-07-31 (B.F.) Set nhis to zero for non-master processes; distribute some scalars
!   2017-08-08 (B.F.) Put broadcast of prop and scalar data into separate routines;
!                     copy Cartesian geometry data to wg%hyd; 
!                     put setup of Cartesian communicator into separate routine
!   2017-10-05 (B.F.) Find number of processes per dimension by calling rhd_MPI_DimsCreate
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_io_module
use rhd_sub_module
use timing_module
use MPI
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(inout):: ncin
integer,                   intent(in)   :: MyProcWo
type(action_type),         intent(inout):: action
type(box_type),            intent(out)  :: box
logical,         optional, intent(in)   :: Bb_flag
character(len=*),optional, intent(in)   :: file
integer,         optional, intent(in)   :: imodel
real,            optional, intent(out)  :: dtime, time_out_full_last, time_out_mean_last
integer,         optional, intent(out)  :: nhis
character(len=*), dimension(:), &
                 optional, intent(out)  :: history
!
! --- Local variables ---
integer                                 :: ierrMPI, &
                                           NProcsHD, TaProcHD, iTaProcHD, nProcsWo, &
                                           m1,n1, m2,n2, m3,n3, &
                                           nghost1, nghost2, nghost3, nghost
integer, dimension(3)                   :: PCoords
integer, dimension(:), allocatable      :: rhdMPI_boxLrgTypes, rhdMPI_boxTypes, RequestsMPI
integer, dimension(:,:), allocatable    :: StatusMPI
type(box_type)                          :: boxLrg
logical                                 :: MyProcIsInCOMM_HD_flag
!---------------------------------------------------------------------------------------------------
!
prop%MyProcWo=MyProcWo
!
! === Read start model and put some quantities into global structure ===
if (MyProcWo == 0) then
  ! --- Read entire start model into one large box structure ---
  ! write(gl%nc_p, '(A,I3,A)') '#', MyProcWo, &
  !     ' Read start model from file into large box'
  call timing_start('IO: read UIO box')
  call rhd_box_Read('open,model,close', ncin, &
                    box=boxLrg, &
                    Bb_flag=Bb_flag, &
                    dtime=dtime, &
                    time_out_full_last=time_out_full_last, &
                    time_out_mean_last=time_out_mean_last, &
                    file=file, imodel=par%istep_in_start, nhis=nhis, history=history)
  call timing_stop('IO: read UIO box')
  !
  ! write(gl%nc_p, '(A,I3,A,2(1PE14.6))') '#', MyProcWo, &
  !     ' rhd_MPI_boxRead: max(boxLrg%quc(-8:8..,1;2)): ', &
  !     maxval(boxLrg%quc(max(boxLrg%m1,-8):min(boxLrg%n1,8), &
  !                       max(boxLrg%m2,-8):min(boxLrg%n2,8), &
  !                       max(boxLrg%m3,-8):min(boxLrg%n3,8),  1)), &
  !     maxval(boxLrg%quc(max(boxLrg%m1,-8):min(boxLrg%n1,8), &
  !                       max(boxLrg%m2,-8):min(boxLrg%n2,8), &
  !                      max(boxLrg%m3,-8):min(boxLrg%n3,8),  2))
  !
  ! --- Put some global properties of input model in prop structure  ---
  ! --- (the reading routine itself partly fills the prop structure) ---
  !
  ! --- Define typical length scale ---
  prop%length=(boxLrg%xb1(boxLrg%n1+1)-boxLrg%xb1(boxLrg%m1))**(1.0/3.0) * &
              (boxLrg%xb2(boxLrg%n2+1)-boxLrg%xb2(boxLrg%m2))**(1.0/3.0) * &
              (boxLrg%xb3(boxLrg%n3+1)-boxLrg%xb3(boxLrg%m3))**(1.0/3.0)
  !
  ! --- Modify action structure for master process:               ---
  ! --- A direction with only one element has periodic boundaries ---
  if (boxLrg%n1 == boxLrg%m1) then
    action=rhd_action_New(action, bound_l1=bound_periodic, bound_r1=bound_periodic)
  endif
  if (boxLrg%n2 == boxLrg%m2) then
    action=rhd_action_New(action, bound_l2=bound_periodic, bound_r2=bound_periodic)
  endif
  if (boxLrg%n3 == boxLrg%m3) then
    action=rhd_action_New(action, bound_l3=bound_periodic, bound_r3=bound_periodic)
  endif
  !
  ! --- Copy global boundary information from action to prop structure ---\label{f90:bound__proc2}
  prop%bound_l1    =action%bound_l1
  prop%bound_r1    =action%bound_r1
  prop%bound_l2    =action%bound_l2
  prop%bound_r2    =action%bound_r2
  prop%bound_l3    =action%bound_l3
  prop%bound_r3    =action%bound_r3
  prop%bound_mag_l1=action%bound_mag_l1
  prop%bound_mag_r1=action%bound_mag_r1
  prop%bound_mag_l2=action%bound_mag_l2
  prop%bound_mag_r2=action%bound_mag_r2
  prop%bound_mag_l3=action%bound_mag_l3
  prop%bound_mag_r3=action%bound_mag_r3
  !
else ! (MyProcWo /= 0)
  if (present(nhis)) nhis=0
endif ! (MyProcWo)
!
! === Distribute some scalars and the global prop structure ===
!
call rhd_MPI_BCastReals(MyProcWo, 0, MPI_COMM_WORLD, &
                        r1=dtime, r2=time_out_full_last, r3=time_out_mean_last)
!
call rhd_prop_MPIBCast(MyProcWo, 0, MPI_COMM_WORLD)
!
if (MyProcWo /= 0) then
  ! --- Modify action structure for the rest of the processes:    ---
  ! --- A direction with only one element has periodic boundaries ---
  if (prop%n1 == prop%m1) then
    action=rhd_action_New(action, bound_l1=bound_periodic, bound_r1=bound_periodic)
  endif
  if (prop%n2 == prop%m2) then
    action=rhd_action_New(action, bound_l2=bound_periodic, bound_r2=bound_periodic)
  endif
  if (prop%n3 == prop%m3) then
    action=rhd_action_New(action, bound_l3=bound_periodic, bound_r3=bound_periodic)
  endif
endif
!
! --- Copy global information from prop to wg%hyd structure ---
wg%hyd%m(1)=prop%m1
wg%hyd%n(1)=prop%n1
wg%hyd%m(2)=prop%m2
wg%hyd%n(2)=prop%n2
wg%hyd%m(3)=prop%m3
wg%hyd%n(3)=prop%n3
wg%hyd%xb_l(1)=prop%xb_l1
wg%hyd%xb_r(1)=prop%xb_r1
wg%hyd%xb_l(2)=prop%xb_l2
wg%hyd%xb_r(2)=prop%xb_r2
wg%hyd%xb_l(3)=prop%xb_l3
wg%hyd%xb_r(3)=prop%xb_r3
wg%hyd%bound_l(1)=prop%bound_l1
wg%hyd%bound_r(1)=prop%bound_r1
wg%hyd%bound_l(2)=prop%bound_l2
wg%hyd%bound_r(2)=prop%bound_r2
wg%hyd%bound_l(3)=prop%bound_l3
wg%hyd%bound_r(3)=prop%bound_r3
wg%hyd%bound_mag_l(1)=prop%bound_mag_l1
wg%hyd%bound_mag_r(1)=prop%bound_mag_r1
wg%hyd%bound_mag_l(2)=prop%bound_mag_l2
wg%hyd%bound_mag_r(2)=prop%bound_mag_r2
wg%hyd%bound_mag_l(3)=prop%bound_mag_l3
wg%hyd%bound_mag_r(3)=prop%bound_mag_r3
!
! --- Determine necessary number of ghost cells ---
if ((par%A%hdScheme == hdScheme_RoeMHD) .or. &
    (par%A%hdScheme == hdScheme_HLLMHD) .or. &
    (par%A%hdScheme == hdScheme_CenMHD)       ) then
  ! --- For MHD solver: number of ghost cells one larger than reconstruction order ---
  nghost=par%A%reconstruction_order+1
else
  ! --- For HD solver: minimum number of ghost cells is two ---
  nghost=max(par%A%reconstruction_order+1, 2)
endif
!
if (prop%n1 > prop%m1) then
  nghost1=nghost
else
  nghost1=0
endif
if (prop%n2 > prop%m2) then
  nghost2=nghost
else
  nghost2=0
endif
if (prop%n3 > prop%m3) then
  nghost3=nghost
else
  nghost3=0
endif
!
! call flush(gl%nc_p)
! call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!
! write(gl%nc_p, '(A,I3,A,3(I5,1X,I5,1X,I5,3X))') '#', MyProcWo, &
!   ' Prop: ', prop%m1, prop%n1, nghost1, &
!              prop%m2, prop%n2, nghost2, &
!              prop%m3, prop%n3, nghost3
!
! --- ToDoMPI: read quc quantity information (not necessary if other processes don't do I/O) ---
!
!
! === Distribute processes properly over the global 3D array ===
!
! --- Determine total number of MPI processes ---
call MPI_Comm_Size(MPI_COMM_WORLD, NProcsWO, ierrMPI)
!
! --- Find reasonable number of MPI processes per dimension                   ---\label{f90:an_mpiprocsperdim__proc1}
! --- Make sure that there is a minimum number of cells per dimension to make ---
! --- boundary-condition handling possible (in one step)                      ---
call rhd_MPI_DimsCreate(prop%m1,prop%n1, prop%m2,prop%n2, prop%m3,prop%n3, &
                        nghost1, nghost2, nghost3, &
                        NProcsWO, par%aN_MPIProcsPerDim, wg%hyd%nProcsPD)
!
! === Set up Cartesian communicator ===
call rhd_MPI_CartSetup(MyProcWo, wg%hyd, action)
!
! === Send boxes to the compute processes ===
!
if (MyProcWo == 0) then
  write(gl%nc_p, '(A)') 'Distribute boxes'
endif
call MPI_Comm_Size(wg%hyd%Comm, NProcsHD, ierrMPI)
call timing_start('IO:   distribute')
if (MyProcWo == 0) then
  ! --- The current process is the master that has read the original data ---
  !
  ! --- Create temporary arrays of the appropriate size ---
  iTaProcHD=NProcsHD-1
  allocate(rhdMPI_boxLrgTypes(1:iTaProcHD), &
           RequestsMPI(       1:iTaProcHD), &
           StatusMPI(MPI_STATUS_SIZE, iTaProcHD))
  !
  iTaProcHD=0
  do TaProcHD=0,NProcsHD-1
    ! --- Get process coordinates ---
    call MPI_Cart_Coords(wg%hyd%Comm, TaProcHD, 3, PCoords, ierrMPI)
    call rhd_geo_Coords2mn(wg%hyd, PCoords, m1,n1, m2,n2, m3,n3)
    !
    ! write(gl%nc_p, '(A,I3,A,3I3,1X,3(2X,2I5),A,I3)') '#', MyProcWo, &
    !   ' Dim:    ', PCoords, &
    !   m1,n1, m2,n2, m3,n3, &
    !   ' -> P', TaProcHD
    !
    if (wg%hyd%MyProc == TaProcHD) then
      ! --- Create "small" process-local box (including ghost cells) ---
      ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, &
      !   ' Create small local box'
      call rhd_box_Make(box, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                             nghost1=nghost1, &
                             nghost2=nghost2, &
                             nghost3=nghost3, &
                             quc_flag=.true., Bb_flag=.false.)
      !
      ! --- Target is identical to source: just copy the box ---
      ! --- ToDoMPI: OMP ---
      ! write(gl%nc_p, '(A,I3,A,I3)') '#', wg%hyd%MyProc, &
      !   ' Start copying part of large box to local small one ', TaProcHD
      call rhd_box_Copy( boxLrg, m1,n1, m2,n2, m3,n3, &
                         box   , m1,     m2,     m3, &
                         trans_mode=1, time_mode=1, grid_mode=1, bulk_mode=1, flux_mode=0)
      ! write(gl%nc_p, '(A,I3,A,I3)') '#', wg%hyd%MyProc, &
      !   ' Stop copying part of large box to local small one  ', TaProcHD
      !
    else
      ! --- Data has to be sent to different process ---
      iTaProcHD=iTaProcHD+1
      ! --- Create source MPI data type ---
      ! write(gl%nc_p, '(A,I3,A,I3)') '#', wg%hyd%MyProc, &
      !   ' Create MPI type for part of large box ', TaProcHD
      call rhd_box_MPICreateType(boxLrg, m1,n1, m2,n2, m3,n3, &
                                 rhdMPI_boxLrgTypes(iTaProcHD), &
                                 time_mode=1, grid_mode=1, bulk_mode=2)
      !
      ! --- Send it ---
      ! write(gl%nc_p, '(A,I3,A,2I3)') '#', wg%hyd%MyProc, ' Send part of large box ', TaProcHD
      call MPI_ISend(boxLrg, 1, rhdMPI_boxLrgTypes(iTaProcHD), TaProcHD, &
                     TaProcHD, wg%hyd%Comm, RequestsMPI(iTaProcHD), ierrMPI)
      !
    endif
    !
  end do ! TaProcHD
  !
  ! --- Wait until all boxes are copied ---
  ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Wait until all boxes have been sent'
  call MPI_WaitAll(iTaProcHD, RequestsMPI, StatusMPI, ierrMPI)
  ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Finished waiting that all boxes have been sent'
  !
  ! --- Free data types ---
  do TaProcHD=1,iTaProcHD
    call MPI_Type_Free(rhdMPI_boxLrgTypes(TaProcHD), ierrMPI)
  end do ! TaProcHD
  !
  ! --- Deallocate arrays ---
  deallocate(rhdMPI_boxLrgTypes, &
             RequestsMPI, &
             StatusMPI)
  !
else if (wg%hyd%MyProcIsInCOMM_flag) then
  ! --- The current process is one of the HD compute processes but definitely not WORLD(0) ---
  !
  ! --- Create temporary arrays of the appropriate size ---
  allocate(rhdMPI_boxTypes(1:1), &
           RequestsMPI(    1:1), &
           StatusMPI(MPI_STATUS_SIZE, 1))
  !
  ! --- Get process coordinates ---
  call MPI_Cart_Coords(wg%hyd%Comm, wg%hyd%MyProc, 3, PCoords, ierrMPI)
  call rhd_geo_Coords2mn(wg%hyd, PCoords, m1,n1, m2,n2, m3,n3)
  !
  ! --- Create "small" process-local box (including ghost cells) ---
  call rhd_box_Make(box, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                         nghost1=nghost1, &
                         nghost2=nghost2, &
                         nghost3=nghost3, &
                         quc_flag=.true., Bb_flag=.false.)
  !
  ! --- Create target MPI data type ---
  ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Create MPI type for part of small local box'
  call rhd_box_MPICreateType(box, m1,n1, m2,n2, m3,n3, &
                             rhdMPI_boxTypes(1), &
                             time_mode=1, grid_mode=1, bulk_mode=2)
  !
  ! --- Receive it ---
  ! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Receive small local box'
  call MPI_IRecv(box, 1, rhdMPI_boxTypes(1), wg%hyd%MaProcWoInCO, &
                 MPI_ANY_TAG, wg%hyd%Comm, RequestsMPI(1), ierrMPI)
  !
  ! --- Wait until all boxes are copied ---
  ! write(gl%nc_p, '(A,I3,A)') &
  !                         '#', wg%hyd%MyProc, ' Wait for reception of small local box'
  call MPI_WaitAll(1, RequestsMPI, StatusMPI, ierrMPI)
  ! write(gl%nc_p, '(A,I3,A)') &
  !                        '#', wg%hyd%MyProc, ' Finished waiting for reception of small local box'
  !
  ! --- Free data types ---
  call MPI_Type_Free(rhdMPI_boxTypes(1), ierrMPI)
  !
  ! --- Deallocate arrays ---
  deallocate(rhdMPI_boxTypes, &
             RequestsMPI, &
             StatusMPI)
  !
endif
call timing_stop('IO:   distribute')
!
! --- Fill ghost cells of axes for MPI-internal and real boundaries ---
call rhd_box_MPIFillGhostAxes(action, wg%hyd, box, merge(nghost, 0, (prop%n1 > prop%m1)), &
                                                   merge(nghost, 0, (prop%n2 > prop%m2)), &
                                                   merge(nghost, 0, (prop%n3 > prop%m3))   )
call rhd_box_FillGhostAxes(action, box, nghost, nghost, nghost)
!
if (MyProcWo == 0) then
  ! --- Delete large input box ---
  call rhd_box_Delete(boxLrg)
endif
!
call flush(gl%nc_p)
!
call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
!
end subroutine rhd_MPI_boxRead

!#
# endif
!#      /* rhd_mpi_l01 ========================================================================== */

end module rhd_MPI_module
