!***************************************************************************************************
!
!   #####  #    # #####          #####  #    # #   #  ####  
!   #    # #    # #    #         #    # #    #  # #  #      
!   #    # ###### #    #         #    # ######   #    ####  
!   #####  #    # #    #         #####  #    #   #        # 
!   #   #  #    # #    #         #      #    #   #   #    # 
!   #    # #    # #####  ######  #      #    #   #    #### _module
!
!   Radiation-Hydrodynamics Code: phycics routines (source terms: gravity, Coriolis, dust)
!
!***************************************************************************************************
!   Fortran90
!   2018-11-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Werner Schaffenberger: Potsdam, Freiburg, East Lansing, Exeter
!   Hans-Guenter Ludwig:   Lund, Meudon, Heidelberg
!   Tristan Guillot:       Lyon
!   France Allard:         Lyon
!***************************************************************************************************
! MODULES:
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!
!***************************************************************************************************

!------***************------------------------------------------------------------------------------
module rhd_phys_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_phys_module ('rhd_physics_module')
!
! PURPOSE:
!   Provide physics (source term) routines for rhd.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_phys_module
!
! VARIABLES:
!   None
!
! ROUTINES: (all contained routines; public ones are marked with '>')
! > rhd_grav:              Compute gravitational potential
! > rhd_hyd_ApplyGrav:     Apply gravitational acceleration.
! > rhd_hyd_Coriolis:      Apply Coriolis rotation
! > rhd_hyd_CoriolisOMP:   Apply Coriolis rotation (with local OpenMP directives)
! > rhd_hyd_Rossow78tfall: Calculate dust sedimentation velocity
! > rhd_box_AllAddDeltaOMP: Add change to entire box, in parallel if box is 3D
!
! MODIFICATION HISTORY:
!   2012-02-21 (B.F. Lyon) Move some routines from rhd_hyd_module.F90 into this one
!---------------------------------------------------------------------------------------------------
implicit none
!
public
!
contains

!----------********---------------------------------------------------------------------------------
subroutine rhd_grav(model, action, grav_flag, phic_grav, phib_grav, gc_grav)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_grav ('rhd_gravitation')
!
! PURPOSE:
!   Compute gravitational potential.
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_grav(model, action, grav_flag, phic_grav, phib_grav, gc_grav=gc_grav)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!   phic_grav:    ('phi_central_gravity') real 3D array, gravitational potential at cell centers
!   phib_grav:    ('phi_boundary_gravity') real 3D array, gravitational potential at cell boundaries
!
! OUTPUT (OPTIONAL):
!   gc_grav:      ('gravity_center_gravity') real 3D array, gravitational acceleration, cell centers
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   g1:           ('gravity_1') real, component of gravity in first dimension
!
! ROUTINES:
!   None
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The routine actually computes -phi_grav, i.e. the negative of the gravitational potential.
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   1999-04-07 (B.F.) Spatially varying potential
!   1999-06-10 (B.F.) Formula for phib_grav corrected
!   2000-01-24 (B.F.) Put into own routine
!   2000-04-28 (B.F.) Parameter r1_grav added with new form of potential
!   2000-01-23 (B.F.) Substitution: x**0.25 -> sqrt(sqrt(x))
!   2000-02-06 (B.F.) x**4 -> (x**2)**2
!   2000-02-15 (B.F.) smooth central potential: phib_grav is (linearly) interpolated
!   2002-01-28 (B.F.) Introduce local copies of xb1 and xc1 because of PGI 3.3-2 compiler
!   2004-07-26 (HGL, B.F.) Branch for 'localboxtide'
!   2007-01-31 (B.F. Lyon) centrifugal force
!   2007-02-01 (B.F.) Renamed: wl -> wr
!   2010-05-22 (B.F.) Centrifugal force only for 'central' potential
!   2011-06-03 (W.S.) Allows nonzero values for nghost2 and nghost3 necessary for unsplit
!                     Hancock scheme
!   2011-06-20 (W.S.) Outer smoothing radius for old spherical potential,
!                     centrifugal force only treated when it is activated
!   2012-02-01 (B.F.) For r0<>0, r1<>0, r2=0: account for nonzero nghost2/3 properly,
!                     include NOVECTOR directive because of Intel 12.1 compiler bug? Or overflow?
!   2012-02-15 (B.F.) Compute gc_grav for potential with r0_grav, r1_grav
!   2012-02-21 (B.F.) Computation of gc_grav outcommented
!   2012-04-12 (B.F.) Move constant out of loop, caused nonsense results: same compiler bug again?
!   2013-02-22 (B.F.) Remove NOVECTOR tentatively
!   2018-04-24 (B.F.) Use new parameter grav_type to distinguish between potential formulas;
!                     use only r1_grav (not r2_grav anymore);
!                     do not interpolate phib_grav
!   2018-04-25 (B.F.) Bugfix in calculation of new phib_grav
!   2018-05-02 (B.F.) Reduce gradient in new potential by changing r^12/r^8 -> r^8/r^4
!---------------------------------------------------------------------------------------------------
use const_module, only: G_grav, pi
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
logical,                   intent(out)  :: grav_flag
real, dimension( &
      model%m1-model%nghost1:&
      model%n1+model%nghost1,&
      model%m2-model%nghost2:&
      model%n2+model%nghost2,&
      model%m3-model%nghost3:&
      model%n3+model%nghost3), &
                           intent(out)  :: phic_grav
real, dimension( &
      model%m1-model%nghost1:&
      model%n1+model%nghost1+1,&
      model%m2-model%nghost2:&
      model%n2+model%nghost2,&
      model%m3-model%nghost3:&
      model%n3+model%nghost3), &
                           intent(out)  :: phib_grav
real, dimension( &
      model%m1-model%nghost1:&
      model%n1+model%nghost1,&
      model%m2-model%nghost2:&
      model%n2+model%nghost2,&
      model%m3-model%nghost3:&
      model%n3+model%nghost3), &
                 optional, intent(out)  :: gc_grav
!
! --- Local variables ---
real, dimension( &
      model%m1-model%nghost1:&
      model%n1+model%nghost1  )         :: xc1
real, dimension( &
      model%m1-model%nghost1:&
      model%n1+model%nghost1+1)         :: xb1
!real, dimension( &
!      model%m1-model%nghost1:&
!      model%n1+model%nghost1,&
!      model%m2-model%nghost2:&
!      model%n2+model%nghost2,&
!      model%m3-model%nghost3:&
!      model%n3+model%nghost3)           :: gc_grav0
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3
integer                                 :: i1, i2, i3
real                                    :: g1, wr, r2, rovr02, rovr12, rovr18, r1ovr04, quadroot, &
                                           R_cent, R2_grav, eps_R2, gr_cent, Cd, Cq
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
nghost1=model%nghost1
nghost2=model%nghost2
nghost3=model%nghost3
!
! \label{f90:grav_mode__use1}
! === Gravitational potential ===
if (par%grav_mode == 'constant') then
  ! \label{f90:grav__use1}
  ! --- Compute trivial potential for constant gravity ---
  !
  ! --- Select current gravity component ---
  select case (action%dim)
    case (1,2)
      g1       =0.0
      grav_flag=.false.
    case (3)
      g1       =par%grav
      grav_flag=.true.
  end select
  if (grav_flag) then
    xc1=model%xc1
    phic_grav                             =spread(spread(-xc1*g1,2,n2+nghost2-m2+nghost2+1),&
                                                                 3,n3+nghost3-m3+nghost3+1)
    xb1=model%xb1
    phib_grav(m1-nghost1:n1+nghost1+1,:,:)=spread(spread(-xb1*g1,2,n2+nghost2-m2+nghost2+1),&
                                                                 3,n3+nghost3-m3+nghost3+1)
  else
    phic_grav=0.0
    phib_grav=0.0
  endif
  !gc_grav0=g1
  !
else if (par%grav_mode == 'central') then
  ! \label{f90:mass_star__use1}
  ! --- Compute spherical potential ---
  !
  ! \label{f90:grav_type__use1}
  if (par%grav_type == 'classic2') then
    if (par%r1_grav == 0.0) then
      ! \label{f90:r0_grav__use1}
      ! --- Compute the 'very old' RSG ^2 potential only from r0_grav ---
      !
      do i3=m3-nghost3,n3+nghost3
        do i2=m2-nghost2,n2+nghost2
          phic_grav(:,i2,i3)= G_grav*par%mass_star / &
                    sqrt(model%xc1**2 + model%xc2(i2)**2 + model%xc3(i3)**2 + par%r0_grav**2)
          phib_grav(m1-nghost1:n1+nghost1+1,i2,i3)= &
                              G_grav*par%mass_star / &
                    sqrt(model%xb1**2 + model%xc2(i2)**2 + model%xc3(i3)**2 + par%r0_grav**2)
        end do ! i2
      end do ! i3
      !
    else ! (par%r1_grav /= 0.0)
      ! \label{f90:r1_grav__use1}
      ! --- Compute the near-polytropic ^2 potential from r0_grav and r1_grav (formerly r2_grav) ---
      !
      do i3=m3-nghost3,n3+nghost3
        do i2=m2-nghost2,n2+nghost2
          ! --- Compute the cell-center potential directly from formula ---
          phic_grav(:,i2,i3)= &
            G_grav*par%mass_star / &
            sqrt((model%xc1**2 + model%xc2(i2)**2 + model%xc3(i3)**2) / &
                 sqrt(sqrt(1.0 + &
                           (((model%xc1**2 + model%xc2(i2)**2 + model%xc3(i3)**2)/ &
                             par%r1_grav**2)**2)**2)) + &
                 par%r0_grav**2)
          !
          ! --- Compute the cell-boundary potential directly from formula ---
          phib_grav(m1-nghost1:n1+nghost1+1,i2,i3)= &
            G_grav*par%mass_star / &
            sqrt((model%xb1**2 + model%xc2(i2)**2 + model%xc3(i3)**2) / &
                 sqrt(sqrt(1.0 + &
                           (((model%xb1**2 + model%xc2(i2)**2 + model%xc3(i3)**2)/ &
                              par%r1_grav**2)**2)**2)) + &
                 par%r0_grav**2)
!!$          ! --- Interpolate the cell-boundary potential linearly from cell-center potential ---
!!$          do i1=m1-nghost1+1,n1+nghost1
!!$            wr=(model%xb1(i1)-model%xc1(i1-1)) / (model%xc1(i1)-model%xc1(i1-1))
!!$            phib_grav(i1,i2,i3)=     wr  * phic_grav(i1  ,i2,i3) + &
!!$                                (1.0-wr) * phic_grav(i1-1,i2,i3)
!!$          end do ! i1
!!$          phib_grav(m1-nghost1  ,i2,i3)=phic_grav(m1-nghost1  ,i2,i3)
!!$          phib_grav(n1+nghost1+1,i2,i3)=phic_grav(n1+nghost1  ,i2,i3)
        end do ! i2
      end do ! i3
    endif ! (par%r1_grav)
    !
  else ! (par%grav_type == 'classic4')
    ! \label{f90:r1_grav__use1}
    ! --- Compute the ^4 RSG potential ---
    !
    if ((par%C_gravCorePotDepth-1.0) <= 0.0) then
      ! --- Compute the standard ^4 RSG potential from r0_grav and r1_grav ---
      !
      r1ovr04=((par%r1_grav/par%r0_grav)**2)**2
      !
      ! --- Compute the cell-center potential directly from formula ---
      do i3=m3-nghost3,n3+nghost3
        do i2=m2-nghost2,n2+nghost2
!!$       !DIR$ NOVECTOR
          do i1=m1-nghost1,n1+nghost1
            rovr12             =(model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2)/ &
                                par%r1_grav**2
            quadroot           =sqrt(sqrt(1.0 + &
                                          r1ovr04 * sqrt((rovr12**2)**2 / (1.0+(rovr12**2)**2)) &
                                         ))
            phic_grav(i1,i2,i3)=G_grav*par%mass_star / (par%r0_grav * quadroot)
            !gc_grav0( i1,i2,i3)=G_grav*par%mass_star * rovr12*sqrt(rovr12) * r1ovr04 / &
            !                    (par%r0_grav*par%r1_grav* quadroot**5 *sqrt(1.0+(rovr12**2)**2)**3)
          end do ! i1
        end do ! i2
      end do ! i3
      !
      ! --- Compute the cell-boundary potential directly from formula ---
      do i3=m3-nghost3,n3+nghost3
        do i2=m2-nghost2,n2+nghost2
          do i1=m1-nghost1,n1+nghost1+1
            rovr12             =(model%xb1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2)/ &
                                par%r1_grav**2
            quadroot           =sqrt(sqrt(1.0 + &
                                          r1ovr04 * sqrt((rovr12**2)**2 / (1.0+(rovr12**2)**2)) &
                                         ))
            phib_grav(i1,i2,i3)=G_grav*par%mass_star / (par%r0_grav * quadroot)
          end do ! i1
!!$        ! --- Interpolate the cell-boundary potential linearly from cell-center potential ---
!!$        do i1=m1-nghost1+1,n1+nghost1
!!$          wr=(model%xb1(i1)-model%xc1(i1-1)) / (model%xc1(i1)-model%xc1(i1-1))
!!$          phib_grav(i1,i2,i3)=     wr  * phic_grav(i1  ,i2,i3) + &
!!$                              (1.0-wr) * phic_grav(i1-1,i2,i3)
!!$        end do ! i1
!!$        phib_grav(m1-nghost1  ,i2,i3)=phic_grav(m1-nghost1  ,i2,i3)
!!$        phib_grav(n1+nghost1+1,i2,i3)=phic_grav(n1+nghost1  ,i2,i3)
        end do ! i2
      end do ! i3
      !
    else ! (C_gravCorePotDepth > 1.0)
      ! --- Compute the ^4 RSG potential from r0_grav, r1_grav, C_gravCorePotDepth, and ---
      ! --- C_gravCorePotQuad                                                           ---
      !
      Cd=1.0/max(par%C_gravCorePotDepth, 1.0)
      Cq=max(par%C_gravCorePotQuad, 0.0) * (1.0-Cd) * Cd**3
      !
      ! --- Compute the cell-center potential directly from formula ---
      do i3=m3-nghost3,n3+nghost3
        do i2=m2-nghost2,n2+nghost2
          do i1=m1-nghost1,n1+nghost1
            r2                 =model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2
            rovr02             =  r2/par%r0_grav**2
            rovr18             =((r2/par%r1_grav**2)**2)**2
            quadroot           =sqrt(sqrt(Cd**4 + &
                                          (rovr02**4/((1.0-Cd)+rovr02**2) + Cq*rovr02) / &
                                          sqrt(1.0+rovr18) &
                                         ))
            phic_grav(i1,i2,i3)=(G_grav*par%mass_star/par%r0_grav) / quadroot
          end do ! i1
        end do ! i2
      end do ! i3
      !
      ! --- Compute the cell-boundary potential directly from formula ---
      do i3=m3-nghost3,n3+nghost3
        do i2=m2-nghost2,n2+nghost2
          do i1=m1-nghost1,n1+nghost1+1
            r2                 =model%xb1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2
            rovr02             =  r2/par%r0_grav**2
            rovr18             =((r2/par%r1_grav**2)**2)**2
            quadroot           =sqrt(sqrt(Cd**4 + &
                                          (rovr02**4/((1.0-Cd)+rovr02**2) + Cq*rovr02) / &
                                          sqrt(1.0+rovr18) &
                                         ))
            phib_grav(i1,i2,i3)=(G_grav*par%mass_star/par%r0_grav) / quadroot
          end do ! i1
        end do ! i2
      end do ! i3
      !
    endif ! (C_gravCorePotDepth)
  endif ! (par%grav_type)
  !
  grav_flag=.true.
  !
  ! \label{f90:centrifugal_force__use1}
  if (par%nu_rotation /= 0.0 .and. par%centrifugal_force == 'on') then
    ! --- Add centrifugal force due to rotation around global x3 axis  ---
    ! --- phi_centrifugal = (-) (2*pi*nu)^2 * r^2/2                    ---
    !
    select case (action%dim)
      case (1,2)
        ! \label{f90:nu_rotation__use1}
        ! --- Local indices = global indices ---
        ! --- or: Transposed 2 -> 1          ---
        !
        eps_R2=0.001*sqrt(abs(model%xb1(i1+1)-model%xb1(i1)))
        do i3=m3-nghost3,n3+nghost3
          do i2=m2-nghost2,n2+nghost2
            do i1=m1-nghost1,n1+nghost1
              phic_grav(i1,i2,i3)=phic_grav(i1,i2,i3) + &
                                 (2.0*(pi*par%nu_rotation)**2)*(model%xc1(i1)**2 + model%xc2(i2)**2)
              R_cent =sqrt(model%xc1(i1)**2 + model%xc2(i2)**2)
              R2_grav=     model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)
              gr_cent=(4.0*(pi*par%nu_rotation)**2) * R_cent
              !gc_grav0(i1,i2,i3)=sqrt(gc_grav0(i1,i2,i3)**2 + gr_cent**2 + &
              !                        2.0*gc_grav0(i1,i2,i3)*gr_cent*R_cent/sqrt(R2_grav+eps_R2))
            end do ! i1
            do i1=m1-nghost1,n1+nghost1+1
              phib_grav(i1,i2,i3)=phib_grav(i1,i2,i3) + &
                                 (2.0*(pi*par%nu_rotation)**2)*(model%xb1(i1)**2 + model%xc2(i2)**2)
            end do ! i1
          end do ! i2
        end do ! i3
      case (3)
        ! --- Transposed 3 -> 1 ---
        !
        do i3=m3-nghost3,n3+nghost3
          do i2=m2-nghost2,n2+nghost2
            do i1=m1-nghost1,n1+nghost1
              phic_grav(i1,i2,i3)=phic_grav(i1,i2,i3) + &
                                 (2.0*(pi*par%nu_rotation)**2)*(model%xc3(i3)**2 + model%xc2(i2)**2)
              R_cent =sqrt(model%xc3(i3)**2 + model%xc2(i2)**2)
              R2_grav=     model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)
              gr_cent=(4.0*(pi*par%nu_rotation)**2) * R_cent
              !gc_grav0(i1,i2,i3)=sqrt(gc_grav0(i1,i2,i3)**2 + gr_cent**2 + &
              !                        2.0*gc_grav0(i1,i2,i3)*gr_cent*R_cent/sqrt(R2_grav+eps_R2))
            end do ! i1
            do i1=m1-nghost1,n1+nghost1+1
              phib_grav(i1,i2,i3)=phib_grav(i1,i2,i3) + &
                                 (2.0*(pi*par%nu_rotation)**2)*(model%xc3(i3)**2 + model%xc2(i2)**2)
            end do ! i1
          end do ! i2
        end do ! i3
    end select
    !
  endif ! (par%nu_rotation /= 0.0 .and. par%centrifugal_force == 'on')
  !
else if (par%grav_mode == 'localboxtide') then
  ! --- Compute experimental time-dependent potential ---
  !
  ! --- Select current gravity component ---
  select case (action%dim)
    case (1)
      xc1=model%xc1
      phic_grav = &
        spread(spread(par%C_test1*cos(par%C_test2*xc1+par%C_test3*model%time),&
                                           2,n2+nghost2-m2+nghost2+1),3,n3+nghost3-m3+nghost3+1)
      xb1=model%xb1
      phib_grav(m1-nghost1:n1+nghost1+1,:,:)= &
        spread(spread(par%C_test1*cos(par%C_test2*xb1+par%C_test3*model%time),&
                                           2,n2+nghost2-m2+nghost2+1),3,n3+nghost3-m3+nghost3+1)
      grav_flag=.true.
    case (2)
      g1=0.0
      phic_grav=0.0
      phib_grav=0.0
      grav_flag=.false.
    case (3)
      g1=par%grav
      xc1=model%xc1
      phic_grav =spread(spread(-xc1*g1,2,n2+nghost2-m2+nghost2+1),3,n3+nghost3-m3+nghost3+1)
      xb1=model%xb1
      phib_grav(m1-nghost1:n1+nghost1+1,:,:)=spread(spread(-xb1*g1,2,n2+nghost2-m2+nghost2+1),&
                                                                   3,n3+nghost3-m3+nghost3+1)
      grav_flag=.true.
  end select
  !
else ! (par%grav_mode)
  ! --- Set potential to zero ---
  !
  phic_grav=0.0
  phib_grav=0.0
  grav_flag=.false.
  !
endif ! (par%grav_mode)
!
!if (present(gc_grav)) gc_grav=gc_grav0
!
end subroutine rhd_grav


!----------*****************------------------------------------------------------------------------
subroutine rhd_hyd_ApplyGrav(box, m1,n1, m2,n2, m3,n3, &
                             boxout, &
                             action, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_ApplyGrav ('rhd_hyd_Apply_Gravity')
!
! PURPOSE:
!   Apply gravitational acceleration.
!   
! CATEGORY:
!   Hydrodynamics, box handling
!
! CALLING SEQUENCE:
!
! INPUT:
!   box:          ('box') type(box_type), input computational box, contains the delta of the
!                 conserved quantities
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box, contains the old and new
!                 state variables
!
! OUTPUT:
!
! ROUTINES:
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_grav:              Compute gravitational potential
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   boxout has to contain the normal variables: rho, v1,2,3, ei
!   box has to contain the changes of the corresponding conserved variables:
!     drho, drhov1,2,3, drhoei
!
! PROCEDURE:
!   Short: boxout=boxout+(box)
!
! EXAMPLE:
!
! TODO:
!
! MODIFICATION HISTORY:
!   2012-02-28 (B.F.) 
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
!
! --- Local variables ---
logical                                 :: grav_flag
integer                                 :: i1, i2, i3, bound_l, bound_r
real                                    :: rho_old, drho, rho, wgrav_old, wgrav_new, &
                                           source_rhov1, source_rhov2, source_rhov3
type(box_type)                          :: box1, box2, box3
real, dimension(m1:n1  ,&
                m2:n2  ,&
                m3:n3)                  :: phic1_grav
real, dimension(m2:n2  ,&
                m1:n1  ,&
                m3:n3)                  :: phic2_grav
real, dimension(m3:n3  ,&
                m2:n2  ,&
                m1:n1)                  :: phic3_grav
real, dimension(m1:n1+1,&
                m2:n2  ,&
                m3:n3  )                :: phib1_grav
real, dimension(m2:n2+1,&
                m1:n1,&
                m3:n3  )                :: phib2_grav
real, dimension(m3:n3+1,&
                m2:n2  ,&
                m1:n1   )               :: phib3_grav
!---------------------------------------------------------------------------------------------------
!
! --- Compute gravitational potential ---
!
! --- Handle x1 direction ---
!
! --- Create dummy box structure with only axes filled ---
call rhd_box_Make(box1, &
                  m1=box%m1, n1=box%n1, m2=box%m2, n2=box%n2, m3=box%m3, n3=box%n3, &
                  quc_flag=.false., Bc_flag=.false., Bb_flag=.false.)
!
! --- Copy axes into new box ---
call rhd_box_Copy(box , m1,n1, m2,n2, m3,n3, &
                  box1, m1    , m2    , m3    , &
                  trans_mode=1, time_mode=0, grid_mode=1, bulk_mode=0, flux_mode=0)
!
! --- Compute gravitational potential with centering to get gravity in x1 direction ---
call rhd_grav(box1, rhd_action_New(action,dim=1), grav_flag, phic1_grav, phib1_grav)
!
! --- Delete box ---
call rhd_box_Delete(box1)
!
! --- Apply boundary conditions ---
bound_l=action%bound_l1
bound_r=action%bound_r1
if (.not. ((bound_l == bound_periodic     ) .or. &
           (bound_l == bound_None         ) .or. &
           (bound_l == bound_transmitting ) .or. &
           (bound_l == bound_transmitting2) .or. &
           (bound_l == bound_inoutflow2   ) .or. &
           (bound_l == bound_inoutflow3   ) .or. &
           (bound_l == bound_closedbottom2)))   then
  do i1=m1,m1
    phib1_grav(i1,:,:)=phic1_grav(m1,:,:)
  end do ! i1
endif
if (.not. ((bound_r == bound_periodic     ) .or. &
           (bound_r == bound_None         ) .or. &
           (bound_r == bound_transmitting ) .or. &
           (bound_r == bound_transmitting2))) then
  do i1=n1+1,n1+1
    phib1_grav(i1,:,:)=phic1_grav(n1,:,:)
  end do ! i1
endif
!
!
! --- Handle x2 direction ---
!
! --- Create dummy box structure with only axes filled ---
call rhd_box_Make(box2, &
                  m1=box%m2, n1=box%n2, m2=box%m1, n2=box%n1, m3=box%m3, n3=box%n3, &
                  quc_flag=.false., Bc_flag=.false., Bb_flag=.false.)
!
! --- Copy axes into new box ---
call rhd_box_Copy(box , m1,n1, m2,n2, m3,n3, &
                  box2, m2    , m1    , m3    , &
                  trans_mode=2, time_mode=0, grid_mode=1, bulk_mode=0, flux_mode=0)
!
! --- Compute gravitational potential with centering to get gravity in x1 direction ---
call rhd_grav(box2, rhd_action_New(action, dim=2, dimV=((/2, 1, 3/))), &
              grav_flag, phic2_grav, phib2_grav)
!
! --- Delete box ---
call rhd_box_Delete(box2)
!
! --- Apply boundary conditions ---
bound_l=action%bound_l2
bound_r=action%bound_r2
if (.not. ((bound_l == bound_periodic     ) .or. &
           (bound_l == bound_None         ) .or. &
           (bound_l == bound_transmitting ) .or. &
           (bound_l == bound_transmitting2) .or. &
           (bound_l == bound_inoutflow2   ) .or. &
           (bound_l == bound_inoutflow3   ) .or. &
           (bound_l == bound_closedbottom2)))   then
  do i2=m2,m2
    phib2_grav(i2,:,:)=phic2_grav(m2,:,:)
  end do ! i2
endif
if (.not. ((bound_r == bound_periodic     ) .or. &
           (bound_r == bound_None         ) .or. &
           (bound_r == bound_transmitting ) .or. &
           (bound_r == bound_transmitting2))) then
  do i2=n2+1,n2+1
    phib2_grav(i2,:,:)=phic2_grav(n2,:,:)
  end do ! i2
endif
!
!
! --- Handle x3 direction ---
!
call rhd_box_Make(box3, &
                  m1=box%m3, n1=box%n3, m2=box%m2, n2=box%n2, m3=box%m1, n3=box%n1, &
                  quc_flag=.false., Bc_flag=.false., Bb_flag=.false.)
!
! --- Copy axes into new box ---
call rhd_box_Copy(box , m1,n1, m2,n2, m3,n3, &
                  box3, m3    , m2    , m1    , &
                  trans_mode=3, time_mode=0, grid_mode=1, bulk_mode=0, flux_mode=0)
!
! --- Compute gravitational potential with centering to get gravity in x1 direction ---
call rhd_grav(box3, rhd_action_New(action, dim=3, dimV=((/3, 2, 1/))), &
              grav_flag, phic3_grav, phib3_grav)
!
! --- Delete box ---
call rhd_box_Delete(box3)
!
! --- Apply boundary conditions ---
bound_l=action%bound_l3
bound_r=action%bound_r3
if (.not. ((bound_l == bound_periodic     ) .or. &
           (bound_l == bound_None         ) .or. &
           (bound_l == bound_transmitting ) .or. &
           (bound_l == bound_transmitting2) .or. &
           (bound_l == bound_inoutflow2   ) .or. &
           (bound_l == bound_inoutflow3   ) .or. &
           (bound_l == bound_closedbottom2)))   then
  do i3=m3,m3
    phib3_grav(i3,:,:)=phic3_grav(m3,:,:)
  end do ! i2
endif
if (.not. ((bound_r == bound_periodic     ) .or. &
           (bound_r == bound_None         ) .or. &
           (bound_r == bound_transmitting ) .or. &
           (bound_r == bound_transmitting2))) then
  do i3=n3+1,n3+1
    phib3_grav(i3,:,:)=phic3_grav(n3,:,:)
  end do ! i3
endif
!
! \label{f90:c_gravmu__use2}
! --- Choose time-centering of gravity source terms ---
if (par%C_gravmu < 0.0) then
  ! --- Use old defaults ---
  if ((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
      (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) then
    ! --- For Runge-Kutta use only first-order terms (density at old time step) ---
    wgrav_new=0.0
  else
    ! --- Use time-centered density ---
    wgrav_new=0.5
  endif
else
  ! --- Use input parameter as weight ---
  wgrav_new=par%C_gravmu
endif
wgrav_old=1.0-wgrav_new
!
! --- Add acceleration due to gravity ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      !
      rho_old            =boxout%rho(i1,i2,i3)
      drho               =box%rho(   i1,i2,i3)
      !rho                =wgrav_old*rho_old + wgrav_new*(rho_old+drho)
      rho                =rho_old + wgrav_new*drho
      source_rhov1       =rho * ((phib1_grav(i1+1,i2,i3)-phib1_grav(i1,i2,i3))/ &
                                 (box%xb1(   i1+1      )-box%xb1(   i1      )) )
      boxout%v1(i1,i2,i3)=box%v1(i1,i2,i3) + dtime*source_rhov1
      source_rhov2       =rho * ((phib2_grav(i2+1,i1,i3)-phib2_grav(i2,i1,i3))/ &
                                 (box%xb2(      i2+1   )-box%xb2(      i2   )) )
      boxout%v2(i1,i2,i3)=box%v2(i1,i2,i3) + dtime*source_rhov2
      source_rhov3       =rho * ((phib3_grav(i3+1,i2,i1)-phib3_grav(i3,i2,i1))/ &
                                 (box%xb3(         i3+1)-box%xb3(         i3)) )
      boxout%v3(i1,i2,i3)=box%v3(i1,i2,i3) + dtime*source_rhov3
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_hyd_ApplyGrav


!----------**********************-------------------------------------------------------------------
subroutine rhd_box_AllAddDeltaOMP(box, boxout, action, dtime, ApplyGrav_flag, eiTable_min, &
                                  outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AllAddDeltaOMP       ('rhd_box_All_Add_Delta_OpenMP')
!
! PURPOSE:
!   Add change to entire box, in parallel if box is 3D.
!   
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AllAddDeltaOMP(box, boxout, eiTable_min, &
!                               outstr=outstr,ierr=ierr)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   eiTable_min:  ('energy_internal_Table_minimum') real with minimum allowed internal-energy value
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error: (currently none)
!
! ROUTINES:
!   rhd_box_AddDelta:      Add delta of (part of) box into another. Transpose, if requested
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create box and boxout with rhd_box_Make first.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-08-08 (B.F. Lyon) Written
!   2012-02-29 (B.F.) Apply acceleration due to gravity if requested
!   2012-03-01 (B.F.) Gravity terms outcommented
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
type(box_type),            intent(inout):: boxout
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: ApplyGrav_flag
real,                      intent(in)   :: eiTable_min
!integer,  optional, intent(in)          :: flux
integer,          optional,intent(out)  :: ierr
character(len=*), optional,intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i3
integer                                 :: ierr0, ierr_chunk, ierr_thread
character(len=80)                       :: outstr0, outstr_chunk, outstr_thread
!---------------------------------------------------------------------------------------------------
!
ierr0  =0
outstr0=''
!
if (box%n2 > box%m2) then
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(i3, outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  ! --- Error handling (initialization) ---
  outstr_thread=''
  ierr_thread  =0
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=box%m3,box%n3
!    if (ApplyGrav_flag) then
!      stop 'Not implemented, yet, in 3D.'
!    endif
    call rhd_box_AddDelta(box,   box%m1, box%n1, box%m2, box%n2, i3, i3, &
                          boxout,box%m1,         box%m2,         i3    , &
                          eiTable_min, bulk_mode=1, trans_mode=1, &
                          outstr=outstr_chunk, ierr=ierr_chunk)
    !
    ! --- Error handling ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    !
  end do ! i3
  !$OMP END DO NOWAIT
  !
  ! --- Error handling ---
  !$OMP CRITICAL (ADD_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  !$OMP END CRITICAL (ADD_REDUCTION)
  !
  !$OMP END PARALLEL
  !
else
!  if (ApplyGrav_flag) then
!    call rhd_hyd_ApplyGrav(box, box%m1, box%n1, box%m2, box%n2, box%m3, box%n3, &
!                           boxout, &
!                           action, dtime)
!
!  endif
  call rhd_box_AddDelta(box,   box%m1, box%n1, box%m2, box%n2, box%m3, box%n3, &
                        boxout,box%m1,         box%m2,         box%m3        , &
                        eiTable_min, bulk_mode=1, trans_mode=1, &
                        outstr=outstr0, ierr=ierr0)

endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_box_AllAddDeltaOMP


!----------****************-------------------------------------------------------------------------
subroutine rhd_hyd_Coriolis(model, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Coriolis ('rhd_hydrodynamics_Coriolis')
!
! PURPOSE:
!   Apply Coriolis rotation
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Coriolis(model, dtime)
!
! INPUT:
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   None
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The routine assumes a rotation around the x3 axis
!   The axes should not be transposed.
!
! PROCEDURE:
!   Instead of applying a Coriolis acceleration, the velocities are rotated.
!   That conserves the kinetic energy exactly.
!
! MODIFICATION HISTORY:
!   2007-02-01 (B.F. Lyon) Written
!   2009-06-16 (B.F.) Account for rotation axes x2 or x3 (old choice)
!   2011-12-08 (W.S.) Arbitrary rotation axis
!   2013-06-04 (B.F.) Reinstall old rotation axes as special cases
!---------------------------------------------------------------------------------------------------
use const_module, only: pi
use rhd_gl_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
real,                      intent(in)   :: dtime
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3
real                                    :: phi, sinphi, cosphi, vn, v1t, v2t, v3t
!---------------------------------------------------------------------------------------------------
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set Coriolis angle = 2 * rotation angle ---\label{f90:nu_rotation__use2}
phi   =4.0*pi*par%nu_rotation*dtime
sinphi=sin(phi)
cosphi=cos(phi)
!
! --- Apply Coriolis rotation ---\label{f90:ar_rotationaxis__use1}
if (sum(abs(par%ar_RotationAxis - (/0.0,0.0,1.0/))) < 1.0E-5) then
  ! --- Use rotation axis x3 ---
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        v1t=model%v1(i1,i2,i3)
        v2t=model%v2(i1,i2,i3)
        model%v1(i1,i2,i3)= cosphi*v1t + sinphi*v2t
        model%v2(i1,i2,i3)=-sinphi*v1t + cosphi*v2t
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (sum(abs(par%ar_RotationAxis - (/0.0,1.0,0.0/))) < 1.0E-5) then
  ! --- Use rotation axis x2 ---
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        v1t=model%v1(i1,i2,i3)
        v3t=model%v3(i1,i2,i3)
        model%v1(i1,i2,i3)= cosphi*v1t + sinphi*v3t
        model%v3(i1,i2,i3)=-sinphi*v1t + cosphi*v3t
      end do ! i1
    end do ! i2
  end do ! i3
else
  ! --- Use arbitrary rotation axis ---
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        v1t=model%v1(i1,i2,i3)
        v2t=model%v2(i1,i2,i3)
        v3t=model%v3(i1,i2,i3)
        vn=model%v1(i1,i2,i3)*par%ar_RotationAxis(1) + &
           model%v2(i1,i2,i3)*par%ar_RotationAxis(2) + &
           model%v3(i1,i2,i3)*par%ar_RotationAxis(3)
        model%v1(i1,i2,i3)=       vn*par%ar_RotationAxis(1) + &
                           (v1t - vn*par%ar_RotationAxis(1))*cosphi + &
                           (v2t*par%ar_RotationAxis(3) - v3t*par%ar_RotationAxis(2))*sinphi
        model%v2(i1,i2,i3)=       vn*par%ar_RotationAxis(2) + &
                           (v2t - vn*par%ar_RotationAxis(2))*cosphi + &
                           (v3t*par%ar_RotationAxis(1) - v1t*par%ar_RotationAxis(3))*sinphi
        model%v3(i1,i2,i3)=       vn*par%ar_RotationAxis(3) + &
                           (v3t - vn*par%ar_RotationAxis(3))*cosphi + &
                           (v1t*par%ar_RotationAxis(2) - v2t*par%ar_RotationAxis(1))*sinphi
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
end subroutine rhd_hyd_Coriolis


!----------*******************----------------------------------------------------------------------
subroutine rhd_hyd_CoriolisOMP(model, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_CoriolisOMP ('rhd_hydrodynamics_Coriolis_OpenMP')
!
! PURPOSE:
!   Apply Coriolis rotation
!   
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_CoriolisOMP(model, dtime)
!
! INPUT:
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   None
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_box_module:        Box-handling routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The routine assumes a rotation around the x3 axis
!   The axes should not be transposed.
!
! PROCEDURE:
!   Instead of applying a Coriolis acceleration, the velocities are rotated.
!   That conserves the kinetic energy exactly.
!
! MODIFICATION HISTORY:
!   2007-02-01 (B.F. Lyon) Written
!   2009-06-16 (B.F.) Account for rotation axes x2 or x3 (old choice)
!   2011-08-06 (B.F.) Adapted from rhd_hyd_Coriolis
!   2011-12-08 (W.S.) Arbitrary rotation axis
!   2013-06-04 (B.F.) Reinstall old rotation axes as special cases
!---------------------------------------------------------------------------------------------------
use const_module, only: pi
use rhd_gl_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
real,                      intent(in)   :: dtime
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3
real                                    :: phi, sinphi, cosphi, vn, v1t, v2t, v3t
!---------------------------------------------------------------------------------------------------
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set Coriolis angle = 2 * rotation angle ---\label{f90:nu_rotation__use3}
phi   =4.0*pi*par%nu_rotation*dtime
sinphi=sin(phi)
cosphi=cos(phi)
!
! --- Apply Coriolis rotation ---\label{f90:ar_rotationaxis__use2}
if (sum(abs(par%ar_RotationAxis - (/0.0,0.0,1.0/))) < 1.0E-5) then
  ! --- Use rotation axis x3 ---
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(v1t, v2t, i1, i2, i3)
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        v1t=model%v1(i1,i2,i3)
        v2t=model%v2(i1,i2,i3)
        model%v1(i1,i2,i3)= cosphi*v1t + sinphi*v2t
        model%v2(i1,i2,i3)=-sinphi*v1t + cosphi*v2t
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
  !
else if (sum(abs(par%ar_RotationAxis - (/0.0,1.0,0.0/))) < 1.0E-5) then
  ! --- Use rotation axis x2 ---
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(v1t, v3t, i1, i2, i3)
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        v1t=model%v1(i1,i2,i3)
        v3t=model%v3(i1,i2,i3)
        model%v1(i1,i2,i3)= cosphi*v1t + sinphi*v3t
        model%v3(i1,i2,i3)=-sinphi*v1t + cosphi*v3t
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
  !
else
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(v1t, v2t, v3t, vn, i1, i2, i3)
  !$OMP DO SCHEDULE(RUNTIME)
  ! --- Use arbitrary rotation axis ---
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        v1t=model%v1(i1,i2,i3)
        v2t=model%v2(i1,i2,i3)
        v3t=model%v3(i1,i2,i3)
        vn=model%v1(i1,i2,i3)*par%ar_RotationAxis(1) + &
           model%v2(i1,i2,i3)*par%ar_RotationAxis(2) + &
           model%v3(i1,i2,i3)*par%ar_RotationAxis(3)
        model%v1(i1,i2,i3)=       vn*par%ar_RotationAxis(1) + &
                           (v1t - vn*par%ar_RotationAxis(1))*cosphi + &
                           (v2t*par%ar_RotationAxis(3) - v3t*par%ar_RotationAxis(2))*sinphi
        model%v2(i1,i2,i3)=       vn*par%ar_RotationAxis(2) + &
                           (v2t - vn*par%ar_RotationAxis(2))*cosphi + &
                           (v3t*par%ar_RotationAxis(1) - v1t*par%ar_RotationAxis(3))*sinphi
        model%v3(i1,i2,i3)=       vn*par%ar_RotationAxis(3) + &
                           (v3t - vn*par%ar_RotationAxis(3))*cosphi + &
                           (v1t*par%ar_RotationAxis(2) - v2t*par%ar_RotationAxis(1))*sinphi
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  !$OMP END PARALLEL
endif
!
end subroutine rhd_hyd_CoriolisOMP


!----------*********************--------------------------------------------------------------------
subroutine rhd_hyd_Rossow78tfall(p_in, T_in, rho_in, grav_in, radius_in, rop, vfall_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Rossow78tfall ('rhd_hydrodynamics_Rossow_1978_time_fall')
!
! PURPOSE:
!   Calculation of dust sedimentation velocity
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Rossow78tfall(p_in, T_in, rho_in, grav_in, radius_in, rop, vfall_out)
!
! INPUT:
!   p_in:         ('pressure_input') real scalar with gas pressure (-> p)
!   T_in:         ('Temperature_input') real scalar with gas temperature (-> t)
!   rho_in:       ('rho_input') real scalar with gas density (-> ro)
!   grav_in:      ('gravity_input') real scalar with acceleration due to gravity (-> grav)
!   rop:          ('rho_particle') averaged grain material density [g/cm^3]
!   radius_in:    ('radius_input') real scalar with grain radius (-> size)
!
! OUTPUT:
!   vfall_out:    ('velocity_fall_output') real scalar with sedimentation velocity (-> vfall)
!
! LOCAL VARIABLES:
!   a70:          ('a_70') size of particles for which Re=70
!   grav:         ('gravity') gravity
!   Kn:           ('Knudsen') Knudsen number
!   lambda:       ('lambda') mean free path of the gas
!   mu:           ('mu') mean molecular weight of gas
!   nu:           ('nu') dynamical viscosity
!   p:            ('pressure') gas pressure
!   size:         ('size') radius of dust grain
!   Re:           ('Reynolds') Reynolds number
!   rn:           ('rho_number') atmospheric density by number of particules/cm3
!   ro:           ('rho') gas density
!   sig:          ('sigma') gas cross section (from radius of H2 molecule, not HI!)
!   T:            ('Temperature') gas temperature
!   vfall:        ('velocity_fall') sedimentation velocity
!
! ROUTINES:
!   None
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Some constants are set within this routine. They have to consistent with definitions elsewhere.
!   The regime distinction should be done based on the total gravity, not only on the component in
!   the current direction.
!
! COMMENTS:
!   Original:
!     constants:
!      real*8 pi,kbol,amu,rgas
!      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/
!      data rgas/8.3145d7/
!      rop = 3.8d0               !for dirty forsterite (MgFeSiO4)

! PROCEDURE:
!     Calculation of the sedimentation velocity in the cloud
!     (according to Rossow (1978), Icarus 30, 1-50, Eq. (19))
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-04    Auteur: Tristan Guillot d'origine (Avril 2000) 
!   2006-11-10 (F.A.) Routine rossow78_tfall in file rossow_sed.f90
!   2007-04-16 (B.F.) F90 version, rop=3.8d0 -> 3.3d0
!   2007-12-12 (B.F.) real*8 -> kind=kind(0.0D+00); rop as input parameter
!   2007-12-14 (B.F.) Kn -> invKn (Kn is infinity when size=0.0);
!                     case Re>>70,Kn<<1: changed formula
!   2007-12-15 (B.F.) New value for sig, according to Derek,
!                     R_gas -> k_Bol/amu
!   2007-12-17 (B.F.) New formula for lambda and nu; nu->eta; reordering of some formulas
!   2008-01-08 (B.F.) Case for large Reynolds numbers outcommented
!---------------------------------------------------------------------------------------------------
use const_module, only: pi_drk, k_Bol_drk, amu_drk
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: p_in, T_in, rho_in, grav_in, radius_in
real(kind=kind(0.0D+00)),  intent(in)   :: rop
real,                      intent(out)  :: vfall_out
!
! --- Local variables ---
real(kind=kind(0.0D+00))                :: p,t,ro,grav,size,vfall, &
                                           eta,mu,rn,lambda,invKn
!                                           a70, Re
real(kind=kind(0.0D+00)), parameter     :: sig=4.d0*(pi_drk*1.2d-8**2+2.d0/3.d0*2.d0*0.74d-8*1.2d-8)
                                           !sig=pi_drk*(2.d-8)**2 !radius of H2 molecule, not HI!
                                           ! rop=3.3d0, &          !for forsterite (Mg2SiO4)
!---------------------------------------------------------------------------------------------------
!
! --- Transform type of input parameters (single -> double precision) ---
p   =p_in
T   =T_in
ro  =rho_in
grav=grav_in
size=radius_in
!
! --- Mean molecular weight of gas ---
mu=ro*(k_Bol_drk*T)/(p*amu_drk)
!
! --- Atmospheric density by number of particles/cm^3 ---
rn=ro/(mu*amu_drk)
!!! rn=p/(k_Bol_drk*T)
!
! --- Mean free path of the gas ---
!lambda=sqrt(3.d0*pi_drk)/(8.d0*rn*sig)
lambda=(1.d0/sqrt(2.d0))/(rn*sig)
!
! --- Dynamical viscosity ---
!      eta=ro/sqrt(T)
!fa06 une Bernd's viscosity!
!eta=ro*lambda*sqrt((8.d0/pi_drk)*(k_Bol_drk*T)/(mu*amu_drk))
! --- Reif ("Statistical and Thermal Physics") 14.8.33 ---
eta=(sqrt(pi_drk)*5.d0/16.d0) *sqrt((k_Bol_drk*T)*(mu*amu_drk))/sig
!!! eta=(sqrt(pi_drk)*5.d0/16.d0) * (k_Bol_drk*T)*sqrt(ro/p)/sig
!
! --- Size of particles for which Re=70          ---
! --- Corrected according to Carlson et al. 1988 ---
!a70=(9.d0*270.d0*eta*eta/(4.d0*ro*rop*abs(grav)))**(1.d0/3.d0)
!a70=((9.d0*270.d0/4.d0)*eta*eta/(ro*rop*abs(grav)))**(1.d0/3.d0)
! --- D.H.'s new formulation: ---
! --- Re=4*ro*rop*grav*size**3/(9*eta**2) ---
! --- if (Re > 18.0) Re=sqrt(18*4*ro*rop*grav*size**3/(9*eta**2)) ---
!Re=(4.d0/9.d0)*(ro*rop*grav*size**3/(eta**2))
!if (Re > 18.0) Re=sqrt((18.d0*4.d0/9.d0)*(ro*rop*grav*size**3/(eta**2)))
!
! --- Knudsen number Kn ---
!Kn=lambda/size
invKn=size/lambda
! --- Reynolds number Re ---
!Re=70.d0*size/a70
!
! --- Sedimentation velocity for 3 regimes; Rossow Eq. (19) ---
if (invKn < 1.d0) then
  ! --- Subsonic free molecular flow ---
  !vfall=(16.d0*rop*grav*size)*sqrt(pi_drk*(mu*amu_drk)/(8.d0*(k_Bol_drk*T))) / (27.d0*ro)
  vfall=((16.d0/27.d0)*rop*grav*size)*sqrt((pi_drk/8.d0)*(mu*amu_drk)/(k_Bol_drk*T)) / ro
  !!! vfall=((16.d0/27.d0)*sqrt(pi_drk/8.d0)) *(rop*grav*size)*sqrt(ro/p)/ro
  !
!elseif (Re > 70.d0) then
!  ! --- Turbulent flow (Newton) ---
!  ! --- The factor 40/3 = 4/3 / (1/2 * C_d) = 4/3 / (1/2 * 0.2)                    ---
!  ! --- where C_d: quadratic drag coefficient, Stroemungswiderstandskoeffizient    ---
!  ! ---            about 0.1 to 0.45 for a sphere                                  ---
!  ! --- from F_grav = F_fric                                                       ---
!  ! ---      m * grav = 4/3*pi*size^3*rop * grav = 1/2*C_d * rho*pi*size^2*vfall^2 ---
!  !vfall=sign(sqrt(4.d0*rop*grav*grav*size)/(3.d0*ro),grav)
!  !vfall=sign(sqrt((40.d0*rop*abs(grav)*size)/(3.d0*ro)),grav)
!  vfall=sign(sqrt(((40.d0/3.d0)*rop*abs(grav)*size)/ro),grav)
!  !
else
  ! --- Laminar viscous flow (Stokes) ---
  ! --- from F_grav = F_fric                                                       ---
  ! ---      m * grav = 4/3*pi*size^3*rop * grav = 6*pi*eta*size*vfall             ---
  vfall=(2.d0/9.d0)*(rop*grav*size*size)/eta
  !
endif
!write(6,'(1p,4d11.3,1X,4d11.3,1X,d11.3)') grav,ro,p,T, size,lambda,eta,invKn, vfall
!
! --- Transform type of output parameter (double -> single precision) ---
vfall_out=vfall
!
end subroutine rhd_hyd_Rossow78tfall


!----------*********************--------------------------------------------------------------------
subroutine rhd_hyd_Rossow78tfall_new(p_in, T_in, rho_in, grav_in, radius_in, rop, vfall_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Rossow78tfall ('rhd_hydrodynamics_Rossow_1978_time_fall')
!
! PURPOSE:
!   Calculation of dust sedimentation velocity
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Rossow78tfall(p_in, T_in, rho_in, grav_in, radius_in, rop, vfall_out)
!
! INPUT:
!   p_in:         ('pressure_input') real scalar with gas pressure (-> p)
!   T_in:         ('Temperature_input') real scalar with gas temperature (-> t)
!   rho_in:       ('rho_input') real scalar with gas density (-> ro)
!   grav_in:      ('gravity_input') real scalar with acceleration due to gravity (-> grav)
!   rop:          ('rho_particle') averaged grain material density [g/cm^3]
!   radius_in:    ('radius_input') real scalar with grain radius (-> size)
!
! OUTPUT:
!   vfall_out:    ('velocity_fall_output') real scalar with sedimentation velocity (-> vfall)
!
! LOCAL VARIABLES:
!   a70:          ('a_70') size of particles for which Re=70
!   grav:         ('gravity') gravity
!   Kn:           ('Knudsen') Knudsen number
!   lambda:       ('lambda') mean free path of the gas
!   mu:           ('mu') mean molecular weight of gas
!   nu:           ('nu') dynamical viscosity
!   p:            ('pressure') gas pressure
!   size:         ('size') radius of dust grain
!   Re:           ('Reynolds') Reynolds number
!   rn:           ('rho_number') atmospheric density by number of particules/cm3
!   ro:           ('rho') gas density
!   sig:          ('sigma') gas cross section (from radius of H2 molecule, not HI!)
!   T:            ('Temperature') gas temperature
!   vfall:        ('velocity_fall') sedimentation velocity
!
! ROUTINES:
!   None
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Some constants are set within this routine. They have to consistent with definitions elsewhere.
!   The regime distinction should be done based on the total gravity, not only on the component in
!   the current direction.
!
! COMMENTS:
!   Original:
!     constants:
!      real*8 pi,kbol,amu,rgas
!      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/
!      data rgas/8.3145d7/
!      rop = 3.8d0               !for dirty forsterite (MgFeSiO4)

! PROCEDURE:
!     Calculation of the sedimentation velocity in the cloud
!     (according to Rossow (1978), Icarus 30, 1-50, Eq. (19))
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-04    Auteur: Tristan Guillot d'origine (Avril 2000) 
!   2006-11-10 (F.A. Lyon) Routine rossow78_tfall in file rossow_sed.f90
!   2007-04-16 (B.F.) F90 version, rop=3.8d0 -> 3.3d0
!   2007-12-12 (B.F.) real*8 -> kind=kind(0.0D+00); rop as input parameter
!   2007-12-14 (B.F.) Kn -> invKn (Kn is infinity when size=0.0);
!                     case Re>>70,Kn<<1: changed formula
!   2007-12-15 (B.F.) New value for sig, according to Derek,
!                     R_gas -> k_Bol/amu
!   2007-12-17 (B.F.) New formula for lambda and nu; nu->eta; reordering of some formulas
!   2008-01-08 (B.F.) Case for large Reynolds numbers outcommented
!   2013-08-26 (B.F.) New version
!---------------------------------------------------------------------------------------------------
use const_module, only: pi_drk, k_Bol_drk, amu_drk
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: p_in, T_in, rho_in, grav_in, radius_in
real(kind=kind(0.0D+00)),  intent(in)   :: rop
real,                      intent(out)  :: vfall_out
!
! --- Local variables ---
real(kind=kind(0.0D+00))                :: p,t,ro,grav,size,vfall, &
                                           eta,mu,rn,lambda,invKn
!                                           a70, Re
real(kind=kind(0.0D+00)), parameter     :: sig=4.d0*(pi_drk*1.2d-8**2+2.d0/3.d0*2.d0*0.74d-8*1.2d-8)
                                           !sig=pi_drk*(2.d-8)**2 !radius of H2 molecule, not HI!
                                           ! rop=3.3d0, &          !for forsterite (Mg2SiO4)
!---------------------------------------------------------------------------------------------------
!
! --- Transform type of input parameters (single -> double precision) ---
p   =p_in
T   =T_in
ro  =rho_in
grav=grav_in
size=radius_in
!
! --- Mean molecular weight of gas ---
!mu=ro*(k_Bol_drk*T)/(p*amu_drk)  ! --- prev
!
! --- Atmospheric density by number of particles/cm^3 ---
!rn=ro/(mu*amu_drk)  ! --- prev
rn=p/(k_Bol_drk*T)
!
! --- Mean free path of the gas ---
!lambda=sqrt(3.d0*pi_drk)/(8.d0*rn*sig)
lambda=(1.d0/sqrt(2.d0))/(rn*sig)
!
! --- Dynamical viscosity ---
!      eta=ro/sqrt(T)
!fa06 une Bernd's viscosity!
!eta=ro*lambda*sqrt((8.d0/pi_drk)*(k_Bol_drk*T)/(mu*amu_drk))
! --- Reif ("Statistical and Thermal Physics") 14.8.33 ---
!eta=(sqrt(pi_drk)*5.d0/16.d0) *sqrt((k_Bol_drk*T)*(mu*amu_drk))/sig  ! --- prev
eta=(sqrt(pi_drk)*5.d0/16.d0) * (k_Bol_drk*T)*sqrt(ro/p)/sig
!
! --- Size of particles for which Re=70          ---
! --- Corrected according to Carlson et al. 1988 ---
!a70=(9.d0*270.d0*eta*eta/(4.d0*ro*rop*abs(grav)))**(1.d0/3.d0)
!a70=((9.d0*270.d0/4.d0)*eta*eta/(ro*rop*abs(grav)))**(1.d0/3.d0)
! --- D.H.'s new formulation: ---
! --- Re=4*ro*rop*grav*size**3/(9*eta**2) ---
! --- if (Re > 18.0) Re=sqrt(18*4*ro*rop*grav*size**3/(9*eta**2)) ---
!Re=(4.d0/9.d0)*(ro*rop*grav*size**3/(eta**2))
!if (Re > 18.0) Re=sqrt((18.d0*4.d0/9.d0)*(ro*rop*grav*size**3/(eta**2)))
!
! --- Knudsen number Kn ---
!Kn=lambda/size
invKn=size/lambda
! --- Reynolds number Re ---
!Re=70.d0*size/a70
!
! --- Sedimentation velocity for 3 regimes; Rossow Eq. (19) ---
if (invKn < 1.d0) then
  ! --- Subsonic free molecular flow ---
  !vfall=(16.d0*rop*grav*size)*sqrt(pi_drk*(mu*amu_drk)/(8.d0*(k_Bol_drk*T))) / (27.d0*ro)
  !vfall=((16.d0/27.d0)*rop*grav*size)*sqrt((pi_drk/8.d0)*(mu*amu_drk)/(k_Bol_drk*T))/ro ! --- prev
  vfall=((16.d0/27.d0)*sqrt(pi_drk/8.d0)) * (rop*grav*size)*sqrt(ro/p)/ro
  !
!elseif (Re > 70.d0) then
!  ! --- Turbulent flow (Newton) ---
!  ! --- The factor 40/3 = 4/3 / (1/2 * C_d) = 4/3 / (1/2 * 0.2)                    ---
!  ! --- where C_d: quadratic drag coefficient, Stroemungswiderstandskoeffizient    ---
!  ! ---            about 0.1 to 0.45 for a sphere                                  ---
!  ! --- from F_grav = F_fric                                                       ---
!  ! ---      m * grav = 4/3*pi*size^3*rop * grav = 1/2*C_d * rho*pi*size^2*vfall^2 ---
!  !vfall=sign(sqrt(4.d0*rop*grav*grav*size)/(3.d0*ro),grav)
!  !vfall=sign(sqrt((40.d0*rop*abs(grav)*size)/(3.d0*ro)),grav)
!  vfall=sign(sqrt(((40.d0/3.d0)*rop*abs(grav)*size)/ro),grav)
!  !
else
  ! --- Laminar viscous flow (Stokes) ---
  ! --- from F_grav = F_fric                                                       ---
  ! ---      m * grav = 4/3*pi*size^3*rop * grav = 6*pi*eta*size*vfall             ---
  vfall=(2.d0/9.d0)*(rop*grav*size*size)/eta
  !
endif
!write(6,'(1p,4d11.3,1X,4d11.3,1X,d11.3)') grav,ro,p,T, size,lambda,eta,invKn, vfall
!
! --- Transform type of output parameter (double -> single precision) ---
vfall_out=vfall
!
end subroutine rhd_hyd_Rossow78tfall_new

end module rhd_phys_module
