!***************************************************************************************************
!
!   #####   #    #  #####          #####   #####    ####   #####
!   #    #  #    #  #    #         #    #  #    #  #    #  #    #
!   #    #  ######  #    #         #    #  #    #  #    #  #    #
!   #####   #    #  #    #         #####   #####   #    #  #####
!   #   #   #    #  #    #         #       #   #   #    #  #
!   #    #  #    #  #####  ####### #       #    #   ####   #     _module
!
!   Provide global properties of quantities in box for Radiation-Hydrodynamics Code
!
!***************************************************************************************************
!   Fortran90
!   2018-11-18
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
!***************************************************************************************************

!# /* --- Default values for preprocessor macros ------------------------------------------------ */
!#
!# /* --- MPI flag activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------***************------------------------------------------------------------------------------
module rhd_prop_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_prop_module ('rhd_properties_module')
!
! PURPOSE:
!   Provide global properties of quantities in box.
!
! CATEGORY:
!   Global type definitions and parameters
!
! CALLING SEQUENCE:
!   use rhd_prop_module
!
! TYPES:
!   prop_type:    ('properties_type') structure type for properties of quantities in box
!     m1:           ('m_number_1') integer, 1. dimension, start index of global box
!     n1:           ('number_1')   integer, 1. dimension, end index of global box
!     m2:           ('m_number_2') integer, 2. dimension, start index of global box
!     n2:           ('number_2')   integer, 2. dimension, end index of global box
!     m3:           ('m_number_3') integer, 3. dimension, start index of global box
!     n3:           ('number_3')   integer, 3. dimension, end index of global box
!     nquc:         ('number_quantities_center') integer, number of additional density arrays
!     nBb:          ('number_B_boundary') integer, box can contain magnetic-field arrays
!                     0: no magnetic-field arrays
!                     1: magnetic field arrays can be there
!     bound_l1,2,3: ('boundary_left_1,2,3') integer, left boundary type for dimension 1,2,3
!                      of global box; values: see action structure
!     bound_r1,2,3: ('boundary_right_1,2,3') integer, right boundary type for dimension 1,2,3
!                      of global box; values: see action structure
!     bound_mag_l1,2,3:('boundary_magnetic_left_1,2,3') integer, left boundary type of the
!                      magnetic field for dimension 1,2,3
!                      of global box; values: see action structure
!     bound_mag_r1,2,3:('boundary_magnetic_right_1,2,3') integer, right boundary type of the
!                      magnetic for dimension 1,2,3
!                      of global box; values: see action structure
!     xb_l1:        ('x_boundary_left_1') real with left boundary value of xb1 axis
!     xb_r1:        ('x_boundary_right_1') real with right boundary value of xb1 axis
!     xb_l2:        ('x_boundary_left_2') real with left boundary value of xb2 axis
!     xb_r2:        ('x_boundary_right_2') real with right boundary value of xb2 axis
!     xb_l3:        ('x_boundary_left_3') real with left boundary value of xb3 axis
!     xb_r3:        ('x_boundary_right_3') real with right boundary value of xb3 axis
!     length:       ('length') real, typical length scale
!     xc1:          ('x_center_1') real, 1. coordinate of cell centers
!     xc2:          ('x_center_2') real, 2. coordinate of cell centers
!     xc3:          ('x_center_3') real, 3. coordinate of cell centers
!     xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!     xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!     xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!     quc_prop:     ('quantities_centered_properties') character with name of dust model
!     XYX_ident:    ('XYZ_identifier') character with identifier of quantity,
!                   e.g. 'rho', 'v1'
!     XYZ_name:     ('XYZ_name') character with name of quantity,
!                   e.g. 'Density', 'Velocity 1'
!     XYZ_unit:     ('XYZ_unit') character with unit of quantity,
!                   e.g. 'g/cm^3', 'cm/s'
!     quc_ident:    ('quantity_center_identifier') character vector w. identifiers of quantities,
!                   e.g. 'quc001'
!     quc_name:     ('quantity_center_name') character vector with names of quantities,
!                   e.g. 'Dust moment 3'
!     quc_unit:     ('quantity_center_unit') character vector with units of quantities,
!                   e.g. '1/cm^3'
!     quc_bound:    ('quantity_center_boundary') integer vector to control the handling of each
!                   quc array:
!                     0: extrapolate quc quantities properly but do not rescale inside cells
!                     2: scale quc values to new mass density (inside cells)
!     quc_advect:   ('quantity_center_advect') integer vector to control the type of advection
!                   of each quc array:
!                     0: no advection
!                     1: simple advection
!                     2: advection + settling (HGL version)
!                     3: advection + settling (rhd_hyd_rossow78tfall, grain radii from model)
!                     4: advection + settling (rhd_hyd_rossow78tfall, grain radii from input)
!     quc_positive: ('quantity_center_positive') integer vector to control check for positivity
!                   in some advection schemes:
!                     0: no test
!                     1: check for positivity (the old standard)
!     Bb1_IsInFile_flag: ('B_boundary_1_Is_In_File_flag') logical that indicates if the
!                        magnetic field in x1 direction has been read from file (for MHD runs)
!     Bb2_IsInFile_flag: ('B_boundary_2_Is_In_File_flag') logical that indicates if the
!                        magnetic field in x2 direction has been read from file (for MHD runs)
!     Bb3_IsInFile_flag: ('B_boundary_3_Is_In_File_flag') logical that indicates if the
!                        magnetic field in x3 direction has been read from file (for MHD runs)
!     MyProcWo:     ('My_Process_World') MPI rank in world communicator
!
!   geo_type:     ('geometry_type') structure for geometrical properties of partitioning
!     Comm:         ('Communicator') integer with MPI communicator for all processes in partitioning
!     MyProc:       ('My_Process') integer with identifier of local process in communicator Comm
!     MaProcWoInCO: ('Master_Process_World_In_COmmunicator') integer with identifier
!                   of master process in communicator Comm
!     m:            ('m_indices') 3-element integer vector with start indices of entire geo box
!     n:            ('number_indices') 3-element integer vector with end indices of entire geo box
!     bound_l:      ('boundary_left') 3-element integer vector with left boundary type
!     bound_r:      ('boundary_right') 3-element integer vector with right boundary type
!     bound_mag_l:  ('boundary_magnetic_left') 3-element integer vector with left boundary type
!                   of the magnetic field; values: see action structure
!     bound_mag_r:  ('boundary_magnetic_right') 3-element integer vector with right boundary type
!                   of the magnetic field; values: see action structure
!     Proc_l:       ('Process_left') 3-element integer vector with identifier of process
!                   holding the box on the "left" (in front, at the bottom)
!     Proc_r:       ('Process_right') 3-element integer vector with identifier of process
!                   holding the box on the "right" (behind, on top)
!     iProcPD:      ('index_Process_Per_Dimension') 3-element integer vector with
!                   Cartesian MPI process coordinates, from 0 to nProcsPD-1
!     nProcsPD:     ('number_Process_Per_Dimension') 3-element integer vector with
!                   number of Cartesian MPI processes in each dimension
!     xb_l:         ('x_boundary_left') 3-element real vector with left boundary values
!                   of entire geo box
!     xb_r:         ('x_boundary_right') 3-element real vector with right boundary values
!                   of entire geo box
!     MyProcIsInCOMM_flag: ('My_Process_Is_In_COMMunicator_flag') logical to
!                   indicate if current process is member of the communicator Comm
!
! VARIABLES:
!   prop:         ('properties') structure for storage of properties of quantities in box
!   wg:           ('world_geometry') structure to store information about various decompositions
!
! ROUTINES: (contained)
!   rhd_prop_Init:         Initialize property structure
!   rhd_prop_Print:        Print information about property structure
!   rhd_geo_Init:          Initialize world-geometry structure
!   rhd_geo_Copy:          Copy world-geometry structure
!   rhd_geo_Print:         Print information about geometry structure
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!   rhd_geo_Coords2mnVec:  Compute box boundary indices from MPI process coordinates
!   rhd_geo_Toaction:      Copy boundary-condition settings from geo structure to action structure
!   rhd_wg_Init:           Initialize global collection of world-geometry structures
!   --- MPI-only routines ---
!   rhd_prop_MPICreateType: Create MPI data type for prop structure
!   rhd_prop_MPICreateTypex: Create MPI data type for axes in the prop structure
!   rhd_prop_MPIBCast:     Broadcast some data in the prop structure to other MPI processes
!
! MODIFICATION HISTORY:
!   2002-10-29 (B.F. Uppsala) First version
!   2002-11-01 (B.F.) Magnetic field data
!   2002-11-06 (B.F.) More information
!   2010-03-08 (B.F. Lyon) Properties Bb1_IsInFile_flag etc. added
!   2010-04-09 (B.F.) Put m1,...,n3 into structure
!   2012-07-27 (B.F.) Move scale%length to prop%length; put bound_XYZ into structure
!   2012-09-05 (B.F.) Put xb_l/r1/2/3 into structure
!   2012-11-12 (B.F.) Put some MPI parameters into structure
!   2014-03-19 (B.F. Uppsala) New types and global variable
!   2014-04-06 (B.F.) Put complete axis vectors xb1, xb2, xb3 into prop structure
!   2016-04-15 (B.F.) Reorganize global world-geometry variables for MPI: move from prop to wg%hyd
!   2017-05-22 (B.F.) Add array quc_positive to control check for positivity of quc arrays
!   2017-08-07 (B.F.) Define axes as pointer instead of allocatable; add xc1, xc2, xc3 axes
!   2017-08-08 (B.F.) Add iProcPD and nProcsPD in geo structure
!   2017-08-11 (B.F.) Copy and rename routine from rhd_MPI_module: rhd_geo_Coords2mn
!   2017-08-13 (B.F.) Modify print routines
!   2017-09-18 (B.F.) Move MPI routines for prop handling here and rename them
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- Global parameters ---
!
type prop_type
  integer                               :: m1,n1, m2,n2, m3,n3, &
                                           nquc, nBb, &
                                           bound_l1    , bound_r1    , &
                                           bound_l2    , bound_r2    , &
                                           bound_l3    , bound_r3    , &
                                           bound_mag_l1, bound_mag_r1, &
                                           bound_mag_l2, bound_mag_r2, &
                                           bound_mag_l3, bound_mag_r3
  real                                  :: xb_l1, xb_r1, &
                                           xb_l2, xb_r2, &
                                           xb_l3, xb_r3, &
                                           length
  real,           dimension(:), pointer :: xc1, xc2, xc3, xb1, xb2, xb3
  character(len=80)                     :: quc_prop, &
                                           time_ident, time_name, time_unit, &
                                           xc1_ident , xc1_name , xc1_unit , &
                                           xc2_ident , xc2_name , xc2_unit , &
                                           xc3_ident , xc3_name , xc3_unit , &
                                           xb1_ident , xb1_name , xb1_unit , &
                                           xb2_ident , xb2_name , xb2_unit , &
                                           xb3_ident , xb3_name , xb3_unit , &
                                           rho_ident , rho_name , rho_unit , &
                                           ei_ident  , ei_name  , ei_unit  , &
                                           v1_ident  , v1_name  , v1_unit  , &
                                           v2_ident  , v2_name  , v2_unit  , &
                                           v3_ident  , v3_name  , v3_unit  , &
                                           B1_ident  , B1_name  , B1_unit  , &
                                           B2_ident  , B2_name  , B2_unit  , &
                                           B3_ident  , B3_name  , B3_unit
  logical                               :: Bb1_IsInFile_flag, Bb2_IsInFile_flag, Bb3_IsInFile_flag
  character(len=80), &
                  dimension(:), pointer :: quc_ident, quc_name, quc_unit
  integer,        dimension(:), pointer :: quc_bound, quc_advect, quc_positive
  !
  integer                               :: MyProcWo
end type prop_type
!
type geo_type
  integer                               :: Comm, MyProc, MaProcWoInCO
  integer, dimension(3)                 :: m, n, &
                                           bound_l, bound_r, &
                                           bound_mag_l, bound_mag_r, &
                                           Proc_l, Proc_r, &
                                           iProcPD, nProcsPD
  real, dimension(3)                    :: xb_l, xb_r
  logical                               :: MyProcIsInCOMM_flag
end type geo_type
!
integer, parameter, private             :: n_wg_radMax=3, n_wg_chuMax=3
!
type wg_type
  integer                               :: n_radMax, n_chuMax, n_rad, n_chu
  type(geo_type)                        :: wld, mas, hyd, mean
  type(geo_type), dimension(n_wg_radMax):: rad
  type(geo_type), dimension(n_wg_chuMax):: chu
end type wg_type
!
type(prop_type), save                   :: prop
type(wg_type  ), save                   :: wg
!
contains

!----------*************----------------------------------------------------------------------------
subroutine rhd_prop_Init(hdScheme)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_prop_Init ('rhd_properties_Initialize')
!
! PURPOSE:
!   Initialize property structure.
!
! CATEGORY:
!   box handling, global parameters
!
! CALLING SEQUENCE:
!   call rhd_prop_Init(hdScheme)
!
! INPUT:
!   hdScheme:     ('hydrodynamics_Scheme') character, hydrodynamics scheme
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Modify values of components of global variable.
!
! RESTRICTIONS:
!   Only part of the prop structure is initialized in this routine.
!   Most values are set elsewhere.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_prop_Init('Roe')
!
! MODIFICATION HISTORY:
!   2002-10-29 (B.F. Copenhagen) Written
!   2002-11-08 (B.F.) Perform only basic initialization
!   2004-03-27 (B.F.) Branch for hdScheme="RoeMHD"
!   2010-02-01 (B.F. Lyon) Set nBb property for all MHD schemes
!   2010-03-08 (B.F.) Properties Bb1_IsInFile_flag etc. added
!   2010-07-01 (B.F.) hdScheme_CenMHD
!   2017-08-07 (B.F.) Nullify axes
!   2017-08-22 (B.F.) Set Bb1_IsInFile_flags per default to .false.
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: hdScheme
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
prop%m1=0
prop%n1=0
prop%m2=0
prop%n2=0
prop%m3=0
prop%n3=0
!
prop%quc_prop=''
!
prop%time_ident='' ; prop%time_name='' ; prop%time_unit=''
prop%xc1_ident ='' ; prop%xc1_name ='' ; prop%xc1_unit =''
prop%xc2_ident ='' ; prop%xc2_name ='' ; prop%xc2_unit =''
prop%xc3_ident ='' ; prop%xc3_name ='' ; prop%xc3_unit =''
prop%xb1_ident ='' ; prop%xb1_name ='' ; prop%xb1_unit =''
prop%xb2_ident ='' ; prop%xb2_name ='' ; prop%xb2_unit =''
prop%xb3_ident ='' ; prop%xb3_name ='' ; prop%xb3_unit =''
prop%rho_ident ='' ; prop%rho_name ='' ; prop%rho_unit =''
prop%ei_ident  ='' ; prop%ei_name  ='' ; prop%ei_unit  =''
prop%v1_ident  ='' ; prop%v1_name  ='' ; prop%v1_unit  =''
prop%v2_ident  ='' ; prop%v2_name  ='' ; prop%v2_unit  =''
prop%v3_ident  ='' ; prop%v3_name  ='' ; prop%v3_unit  =''
prop%B1_ident  ='' ; prop%B1_name  ='' ; prop%B1_unit  =''
prop%B2_ident  ='' ; prop%B2_name  ='' ; prop%B2_unit  =''
prop%B3_ident  ='' ; prop%B3_name  ='' ; prop%B3_unit  =''
prop%Bb1_IsInFile_flag=.false.
prop%Bb2_IsInFile_flag=.false.
prop%Bb3_IsInFile_flag=.false.
!
select case (hdScheme)
  case ('roe', 'Roe', 'ROE')
    prop%nBb=0
  case ('roemagkin', 'RoeMagKin', 'ROEMAGKIN')
    prop%nBb=1
  case ('roemhd', 'Roemhd', 'RoeMHD', 'ROEMHD')
    prop%nBb=1
  case ('hllmhd', 'HllMHD', 'HLLmhd', 'HLLMHD')
    prop%nBb=1
  case ('centralmhd', 'CentralMHD', 'Centralmhd', 'CENTRALMHD')
    prop%nBb=1
!  case ('vanleer', 'vanLeer', 'Vanleer', 'VanLeer', 'VANLEER')
!    prop%nBb=0
  case default
    prop%nBb=0
endselect ! (hdScheme)
!
prop%nquc=0
nullify(prop%xc1, prop%xc2, prop%xc3, prop%xb1, prop%xb2, prop%xb3, &
        prop%quc_ident, prop%quc_name  , prop%quc_unit, &
        prop%quc_bound, prop%quc_advect, prop%quc_positive)
!
prop%MyProcWo=0
!
end subroutine rhd_prop_Init


!----------**************---------------------------------------------------------------------------
subroutine rhd_prop_Print(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!    rhd_prop_Print ('rhd_properties_Print')
!
! PURPOSE:
!   Print information about property structure.
!
! CATEGORY:
!   box handling
!
! CALLING SEQUENCE:
!   call rhd_prop_Print(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None (data is written to channel ncp)
!
! ROUTINES:
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Data is written to channel ncp, possibly into a file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_prop_Init('Roe')
!   call rhd_prop_Print(6)
!
! MODIFICATION HISTORY:
!   2002-10-29 (B.F. Copenhagen) Written
!   2012-09-05 (B.F. Lyon) Print axes ranges
!   2017-08-13 (B.F.) Add processor ID as prefix under MPI; print information about boundaries
!---------------------------------------------------------------------------------------------------
use rhd_action_module
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
integer                                 :: iquc
integer, parameter                      :: linewidth=84
character(len=linewidth)                :: line, blanks
!character(len=*), parameter             :: inuformat='2X,A10,1X,A42,1X,A13'
!character(len=*), parameter             :: inuformat='1X,A8,1X,A38,1X,A13'
character(len=*), parameter             :: inuformat='A,1X,A8,1X,A38,1X,A13'
character(len=20)                       :: IsInFile_str
integer                                 :: MyProcWo, ierrMPI
character(len=16)                       :: prefix
!---------------------------------------------------------------------------------------------------
!
! --- Determine ID of current MPI process and compose the prefix ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
call MPI_Comm_Rank(MPI_COMM_WORLD, MyProcWo, ierrMPI)
write(prefix, '(I3)') MyProcWo
prefix='#' // trim(prefix) // ' prop:'
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
MyProcWo=0
prefix='prop:'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ncp >= 0) then
  blanks=repeat(' ', linewidth)
!!$  line=repeat('-', linewidth)
!!$  write(ncp, *)
!!$  write(ncp,'(A)') line
  write(ncp,'(A)') trim(prefix) // ' Properties of start model:'
  !
  ! --- Print information about axes ---
  write(ncp,'(A,1X,A,1PE14.6)') trim(prefix), 'Typical length scale: ', prop%length
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
                   prop%time_ident, &
       '"' // trim(prop%time_name) // '"' // blanks, &
       '[' // trim(prop%time_unit) // ']' // blanks
  write(ncp,'(' // inuformat // ',1X,1PE11.4,A2,1PE11.4' // ')') &
       trim(prefix), &
                   prop%xc1_ident, &
       '"' // trim(prop%xc1_name) // '"' // blanks, &
       '[' // trim(prop%xc1_unit) // ']' // blanks, &
       minval(prop%xc1), '..', maxval(prop%xc1)
  write(ncp,'(' // inuformat // ',1X,1PE11.4,A2,1PE11.4' // ')') &
       trim(prefix), &
                   prop%xc2_ident, &
       '"' // trim(prop%xc2_name) // '"' // blanks, &
       '[' // trim(prop%xc2_unit) // ']' // blanks, &
       minval(prop%xc2), '..', maxval(prop%xc2)
  write(ncp,'(' // inuformat // ',1X,1PE11.4,A2,1PE11.4' // ')') &
       trim(prefix), &
                   prop%xc3_ident, &
       '"' // trim(prop%xc3_name) // '"' // blanks, &
       '[' // trim(prop%xc3_unit) // ']' // blanks, &
       minval(prop%xc3), '..', maxval(prop%xc3)
  write(ncp,'(' // inuformat // ',1X,1PE11.4,A2,1PE11.4' // ')') &
       trim(prefix), &
                   prop%xb1_ident, &
       '"' // trim(prop%xb1_name) // '"' // blanks, &
       '[' // trim(prop%xb1_unit) // ']' // blanks, &
       prop%xb_l1, '..', prop%xb_r1
  write(ncp,'(' // inuformat // ',1X,1PE11.4,A2,1PE11.4' // ')') &
       trim(prefix), &
                   prop%xb2_ident, &
       '"' // trim(prop%xb2_name) // '"' // blanks, &
       '[' // trim(prop%xb2_unit) // ']' // blanks, &
       prop%xb_l2, '..', prop%xb_r2
  write(ncp,'(' // inuformat // ',1X,1PE11.4,A2,1PE11.4' // ')') &
       trim(prefix), &
                   prop%xb3_ident, &
       '"' // trim(prop%xb3_name) // '"' // blanks, &
       '[' // trim(prop%xb3_unit) // ']' // blanks, &
       prop%xb_l3, '..', prop%xb_r3
  !
  ! --- Print information about basic hydrodynamics quantities ---
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
                   prop%rho_ident, &
       '"' // trim(prop%rho_name) // '"' // blanks, &
       '[' // trim(prop%rho_unit) // ']' // blanks
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
                   prop%ei_ident, &
       '"' // trim(prop%ei_name) // '"' // blanks, &
       '[' // trim(prop%ei_unit) // ']' // blanks
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
                   prop%v1_ident, &
       '"' // trim(prop%v1_name) // '"' // blanks, &
       '[' // trim(prop%v1_unit) // ']' // blanks
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
                   prop%v2_ident, &
       '"' // trim(prop%v2_name) // '"' // blanks, &
       '[' // trim(prop%v2_unit) // ']' // blanks
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
                   prop%v3_ident, &
       '"' // trim(prop%v3_name) // '"' // blanks, &
       '[' // trim(prop%v3_unit) // ']' // blanks
  !
  ! --- Print information about additional density fields ---
  do iquc=1,prop%nquc
    write(ncp,'(' // inuformat // ',A,I1,A,A,I1,A)') &
       trim(prefix), &
                   prop%quc_ident(iquc), &
       '"' // trim(prop%quc_name(iquc)) // '"' // blanks, &
       '[' // trim(prop%quc_unit(iquc)) // ']' // blanks, &
       ' advect(', prop%quc_advect(iquc), ') ', &
       ' positive(', prop%quc_positive(iquc), ')'
  end do ! iquc
  !
  if (prop%nBb > 0) then
    ! --- Print information about magnetic fields ---
    if (prop%Bb1_IsInFile_flag) then
      !IsInFile_str=' ( Is in file)'
      IsInFile_str=''
    else
      IsInFile_str=' (Not in file)'
    endif
    write(ncp,'(' // inuformat // ',A)') &
       trim(prefix), &
                     prop%B1_ident, &
         '"' // trim(prop%B1_name) // '"' // blanks, &
         '[' // trim(prop%B1_unit) // ']' // blanks, &
         IsInFile_str
    if (prop%Bb2_IsInFile_flag) then
      !IsInFile_str=' ( Is in file)'
      IsInFile_str=''
    else
      IsInFile_str=' (Not in file)'
    endif
    write(ncp,'(' // inuformat // ',A)') &
       trim(prefix), &
                     prop%B2_ident, &
         '"' // trim(prop%B2_name) // '"' // blanks, &
         '[' // trim(prop%B2_unit) // ']' // blanks, &
         IsInFile_str
    if (prop%Bb3_IsInFile_flag) then
      !IsInFile_str=' ( Is in file)'
      IsInFile_str=''
    else
      IsInFile_str=' (Not in file)'
    endif
    write(ncp,'(' // inuformat // ',A)') &
       trim(prefix), &
                     prop%B3_ident, &
         '"' // trim(prop%B3_name) // '"' // blanks, &
         '[' // trim(prop%B3_unit) // ']' // blanks, &
         IsInFile_str
  endif! (prop%nBb > 0)
  !
  ! --- Print information about boundary conditions ---
  write(ncp,'(A,1X,A,A20,A,A20)') &
    trim(prefix), &
    'bound_l,r1: ' , trim(NameAr_bound(prop%bound_l1)), &
    ' .. '         , trim(NameAr_bound(prop%bound_r1))
  write(ncp,'(A,1X,A,A20,A,A20)') &
    trim(prefix), &
    'bound_l,r2: ' , trim(NameAr_bound(prop%bound_l2)), &
    ' .. '         , trim(NameAr_bound(prop%bound_r2))
  write(ncp,'(A,1X,A,A20,A,A20)') &
    trim(prefix), &
    'bound_l,r3: ' , trim(NameAr_bound(prop%bound_l3)), &
    ' .. '         , trim(NameAr_bound(prop%bound_r3))
  !
!!$  write(ncp,'(A)') line
endif
!
end subroutine rhd_prop_Print


!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ======================================================== */


!----------**********************-------------------------------------------------------------------
subroutine rhd_prop_MPICreateType(rhdMPI_propDataType)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_prop_MPICreateType ('rhd_properties_MPI_Create_Type')
!
! PURPOSE:
!   Create MPI data type for parts of the prop structure.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_prop_MPICreateType(rhdMPI_propDataType)
!
! INPUT:
!   None
!
! OUTPUT:
!   rhdMPI_propDataType: ('rhd_MPI_properties_Data_Type') integer with MPI data type handle
!                        for prop structure
!
! ROUTINES:
!   MPI_Get_Address:       Get the address of a location in memory
!   MPI_Type_Commit:       Commit an MPI data type
!   MPI_Type_Create_Struct: Create an MPI structured data type
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Not the complete prop structure is put into the MPI data type.
!   Some entries have to be filled by each process individually (e.g., by calling
!   rhd_dust_PropInit).
!
! PROCEDURE:
!   Create and fill arrays for displacement, length, and type of variables in prop structure.
!   Put the result into MPI structure and give handle to resulting MPI data type as output.
!
! EXAMPLE:
!   call rhd_prop_MPICreateType(rhdMPI_propDataType)
!
! MODIFICATION HISTORY:
!   2012-07-27 (B.F. Lyon) Written
!   2012-11-19 (B.F.) Slight rearrangements; include Bb1/2/3_IsInFile_flag
!---------------------------------------------------------------------------------------------------
use MPI
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(out)  :: rhdMPI_PropDataType
!
! --- Local variables ---
integer, parameter                      :: nBlocks=4
integer, dimension(nBlocks)             :: a_BlockLengths, a_Types
integer(kind=MPI_ADDRESS_KIND), &
         dimension(nBlocks)             :: a_Displacements
integer                                 :: i, ierrMPI
!---------------------------------------------------------------------------------------------------
!
i=0
!
! --- Describe parts of the prop structure the MPI way ---
i=i+1
call MPI_Get_Address(prop%m1               , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   6+2+6*2
a_Types(i)       =   MPI_INTEGER
!
i=i+1
call MPI_Get_Address(prop%xb_l1            , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   6+1
a_Types(i)       =   MPI_REAL
!
i=i+1
call MPI_Get_Address(prop%quc_prop         , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   (1+(1+6+5+3)*3)*80
a_Types(i)       =   MPI_CHARACTER
!
i=i+1
call MPI_Get_Address(prop%Bb1_IsInFile_flag, a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   3
a_Types(i)       =   MPI_LOGICAL
!
! --- Shift displacements ---
a_Displacements=a_Displacements-a_Displacements(1)
!
! --- Use the constructed arrays to create MPI structure data type ---
call MPI_Type_Create_Struct(nBlocks, a_BlockLengths, a_Displacements, a_Types, &
                            rhdMPI_PropDataType, ierrMPI)
!
! --- Commit new MPI data type ---
call MPI_Type_Commit(rhdMPI_propDataType, ierrMPI)
!
end subroutine rhd_prop_MPICreateType


!----------***********************------------------------------------------------------------------
subroutine rhd_prop_MPICreateTypex(rhdMPI_propDataType)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_prop_MPICreateTypex ('rhd_properties_MPI_Create_Type_x')
!
! PURPOSE:
!   Create MPI data type for axes in the prop structure.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_prop_MPICreateTypex(rhdMPI_propDataType)
!
! INPUT:
!   None
!
! OUTPUT:
!   rhdMPI_propDataType: ('rhd_MPI_properties_Data_Type') integer with MPI data-type handle
!                        for prop structure
!
! ROUTINES:
!   MPI_Get_Address:       Get the address of a location in memory
!   MPI_Type_Commit:       Commit an MPI data type
!   MPI_Type_Create_Struct: Create an MPI structured data type
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Not the complete prop structure is put into the MPI data type.
!   Some entries have to be filled by each process individually (e.g., by calling
!   rhd_dust_PropInit).
!   This routine assumes that the axis dimensions are already present in the prop structure.
!
! PROCEDURE:
!   Create and fill arrays for displacement, length, and type of axes in prop structure.
!   Put the result into MPI structure and give handle to resulting MPI data type as output.
!
! EXAMPLE:
!   call rhd_prop_MPICreateTypex(rhdMPI_propDataType)
!
! MODIFICATION HISTORY:
!   2014-04-07 (B.F. Lyon) Adapt from rhd_prop_MPICreateType
!   2017-08-07 (B.F. Uppsala) Account for xc1, xc2, xc3 axes;
!              Bugfix: compute displacements relative to displacement of m1
!---------------------------------------------------------------------------------------------------
use MPI
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(out)  :: rhdMPI_PropDataType
!
! --- Local variables ---
integer, parameter                      :: nBlocks=6
integer, dimension(nBlocks)             :: a_BlockLengths, a_Types
integer(kind=MPI_ADDRESS_KIND)          :: Displacement_m1
integer(kind=MPI_ADDRESS_KIND), &
         dimension(nBlocks)             :: a_Displacements
integer                                 :: i, ierrMPI
!---------------------------------------------------------------------------------------------------
!
i=0
!
! --- Describe parts of the prop structure the MPI way ---
i=i+1
call MPI_Get_Address(prop%xc1              , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   prop%n1-prop%m1+1
a_Types(i)       =   MPI_REAL
!
i=i+1
call MPI_Get_Address(prop%xc2              , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   prop%n2-prop%m2+1
a_Types(i)       =   MPI_REAL
!
i=i+1
call MPI_Get_Address(prop%xc3              , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   prop%n3-prop%m3+1
a_Types(i)       =   MPI_REAL
!
i=i+1
call MPI_Get_Address(prop%xb1              , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   prop%n1-prop%m1+2
a_Types(i)       =   MPI_REAL
!
i=i+1
call MPI_Get_Address(prop%xb2              , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   prop%n2-prop%m2+2
a_Types(i)       =   MPI_REAL
!
i=i+1
call MPI_Get_Address(prop%xb3              , a_Displacements(i), ierrMPI)
a_BlockLengths(i)=   prop%n3-prop%m3+2
a_Types(i)       =   MPI_REAL
!
! --- Shift displacements ---
call MPI_Get_Address(prop%m1               , Displacement_m1   , ierrMPI)
a_Displacements=a_Displacements-Displacement_m1
!
! --- Use the constructed arrays to create MPI structure data type ---
call MPI_Type_Create_Struct(nBlocks, a_BlockLengths, a_Displacements, a_Types, &
                            rhdMPI_PropDataType, ierrMPI)
!
! --- Commit new MPI data type ---
call MPI_Type_Commit(rhdMPI_propDataType, ierrMPI)
!
end subroutine rhd_prop_MPICreateTypex


!----------*****************------------------------------------------------------------------------
subroutine rhd_prop_MPIBCast(MyProc, MaProc, Comm)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_prop_MPIBCast ('rhd_properties_MPI_BroadCast')
!
! PURPOSE:
!   Broadcast some data in the prop structure to other MPI processes.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_prop_MPIBCast((MyProc, MaProc, Comm)
!
! INPUT:
!   MyProc:       ('My_Process') integer with MPI rank of local process in communicator Comm
!   MaProc:       ('Master_Process') integer with MPI rank of master process
!   Comm:         ('Communicator') integer with MPI communicator
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   rhdMPI_propDataType: ('rhd_MPI_properties_Data_Type') integer with MPI data-type handle
!                        for prop structure
!
! ROUTINES:
!   MPI_BCast:             Broadcast a message from one process to all others in the group
!   MPI_Type_Free:         Free an MPI data type
!   rhd_prop_MPICreateType: Create MPI data type for prop structure
!   rhd_prop_MPICreateTypex: Create MPI data type for axes in the prop structure
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!   The global prop structure is modified via MPI broadcasts.
!
! RESTRICTIONS:
!   Not the complete prop structure is put into the MPI data type.
!   Some entries have to be filled by each process individually (e.g., by calling
!   rhd_dust_PropInit).
!
! PROCEDURE:
!   (Most of) the entries of the prop structure are distributed from the master process
!   (usually 0) to the rest of the members of the communicator (usually MPI_COMM_WORLD).
!
! EXAMPLE:
!   call rhd_prop_MPIBCast(MyProcWo, 0, MPI_COMM_WORLD)
!
! MODIFICATION HISTORY:
!   2017-08-08 (B.F. Uppsala) Cut out from rhd_MPI_boxRead
!---------------------------------------------------------------------------------------------------
use MPI
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: MyProc, MaProc, Comm
!
! --- Local variables ---
integer                                 :: ierrMPI, rhdMPI_propDataType
!---------------------------------------------------------------------------------------------------
!
! === Distribute global structure prop, in two steps ===
!
! --- Create MPI data type for prop structure ---
call rhd_prop_MPICreateType(rhdMPI_propDataType)
!
! --- Broadcast prop data from rank-0 process to the rest of the world ---
call MPI_BCast(prop, 1, rhdMPI_propDataType, MaProc, Comm, ierrMPI)
!
! --- Free MPI data type for prop structure again ---
call MPI_Type_Free(rhdMPI_propDataType, ierrMPI)
!
if (MyProc /= MaProc) then
  allocate(prop%xc1(prop%m1:prop%n1  ), &
           prop%xc2(prop%m2:prop%n2  ), &
           prop%xc3(prop%m3:prop%n3  ), &
           prop%xb1(prop%m1:prop%n1+1), &
           prop%xb2(prop%m2:prop%n2+1), &
           prop%xb3(prop%m3:prop%n3+1))
endif
!
! --- Create MPI data type for axes in prop structure ---
call rhd_prop_MPICreateTypex(rhdMPI_propDataType)
!
! --- Broadcast prop data from rank-0 process to the rest of the world ---
call MPI_BCast(prop, 1, rhdMPI_propDataType, MaProc, Comm, ierrMPI)
!
! --- Free MPI data type for prop structure again ---
call MPI_Type_Free(rhdMPI_propDataType, ierrMPI)
!
end subroutine rhd_prop_MPIBCast


!#
# endif
!#      /* rhd_mpi_l01 ========================================================================== */


!----------************-----------------------------------------------------------------------------
subroutine rhd_geo_Init(geo)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_geo_Init ('rhd_geometry_Initialize')
!
! PURPOSE:
!   Initialize world-geometry structure.
!
! CATEGORY:
!   box handling, global parameters
!
! CALLING SEQUENCE:
!   call rhd_geo_Init(geo)
!
! INPUT:
!   None
!
! OUTPUT:
!   geo:          ('geometry') structure of type(geo_type) with geometry data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Initialize components of global structure with zeros or .false.
!
! EXAMPLE:
!   call rhd_geo_Init(wg%hyd)
!
! MODIFICATION HISTORY:
!   2014-03-19 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(out)  :: geo
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- MPI communicator and process IDs ---
geo%COMM        =0
geo%MyProc      =0
geo%MaProcWoInCO=0
geo%iProcPD     =0
geo%NProcsPD    =0
!
! --- Basic geometry and boundary conditions of the box ---
geo%m           =0
geo%n           =0
geo%bound_l     =bound_None
geo%bound_r     =bound_None
geo%bound_mag_l =bound_mag_None
geo%bound_mag_r =bound_mag_None
!
! --- Neighbor processes ---
geo%Proc_l      =0
geo%Proc_r      =0
!
! --- Axes boundary values ---
geo%xb_l        =0.0
geo%xb_r        =0.0
!
geo%MyProcIsInCOMM_flag=.false.
!
end subroutine rhd_geo_Init


!----------************-----------------------------------------------------------------------------
subroutine rhd_geo_Copy(geo, geoOut)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_geo_Copy ('rhd_geometry_Copy')
!
! PURPOSE:
!   Copy world-geometry structure.
!
! CATEGORY:
!   box handling, global parameters
!
! CALLING SEQUENCE:
!   call rhd_geo_Copy(geo, geoOut)
!
! INPUT:
!   geo:          ('geometry') structure of type(geo_type) with geometry data
!
! OUTPUT:
!   geoOut:       ('geometry_Output') structure of type(geo_type) with geometry data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_geo_Copy(geoIn, geoOut)
!
! MODIFICATION HISTORY:
!   2014-03-19 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(geo_type),            intent(out)  :: geoOut
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- MPI communicator and process IDs ---
geoOut%COMM        =geo%COMM
geoOut%MyProc      =geo%MyProc
geoOut%MaProcWoInCO=geo%MaProcWoInCO
geoOut%iProcPD     =geo%iProcPD
geoOut%nProcsPD    =geo%nProcsPD
!
! --- Basic geometry and boundary conditions of the box ---
geoOut%m           =geo%m
geoOut%n           =geo%n
geoOut%bound_l     =geo%bound_l
geoOut%bound_r     =geo%bound_r
geoOut%bound_mag_l =geo%bound_mag_l
geoOut%bound_mag_r =geo%bound_mag_r
!
! --- Neighbor processes ---
geoOut%Proc_l      =geo%Proc_l
geoOut%Proc_r      =geo%Proc_r
!
! --- Axes boundary values ---
geoOut%xb_l        =geo%xb_l
geoOut%xb_r        =geo%xb_r
!
geoOut%MyProcIsInCOMM_flag=geo%MyProcIsInCOMM_flag
!
end subroutine rhd_geo_Copy


!----------*************----------------------------------------------------------------------------
subroutine rhd_geo_Print(ncp, geo, geoName)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_geo_Print ('rhd_geometry_Print')
!
! PURPOSE:
!   Print information about geometry structure.
!
! CATEGORY:
!   box handling
!
! CALLING SEQUENCE:
!   call rhd_geo_Print(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   geo:          ('geometry') structure of geo_type
!   geoName:      ('geometry_Name') character with name of structure
!
! OUTPUT:
!   None (data is written to channel ncp)
!
! ROUTINES:
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Data is written to channel ncp, possibly into a file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_geo_Init(geo)
!   call rhd_geo_Print(6, geo, 'Hydro')
!
! MODIFICATION HISTORY:
!   2014-12-08 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_action_module
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(geo_type),            intent(in)   :: geo
character(len=*),          intent(in)   :: geoName
!
! --- Local variables ---
integer, parameter                      :: linewidth=84
! character(len=linewidth)                :: line, blanks
character(len=*), parameter             :: inuformat='A,1X,2(I3,A),2(A,I5,A2,1PE13.6,A,A17)'
integer                                 :: MyProcWo, ierrMPI
character(len=16)                       :: prefix
!---------------------------------------------------------------------------------------------------
!
! --- Determine ID of current MPI process and compose the prefix ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
call MPI_Comm_Rank(MPI_COMM_WORLD, MyProcWo, ierrMPI)
write(prefix, '(I3)') MyProcWo
prefix='#' // trim(prefix) // ' ' // trim(geoName) // ':'
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
MyProcWo=0
prefix=trim(geoName) // ':'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ncp >= 0) then
  ! blanks=repeat(' ', linewidth)
  ! line=repeat('-', linewidth)
  ! write(ncp, *)
  ! write(ncp,'(A)') line
  write(ncp,'(A)') trim(prefix) // ' Properties of geometry structure'
  !
  ! --- Print information about axes ---
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
         geo%iProcPD(1), ' [', geo%nProcsPD(1), '];', &
         ' xb1(' , geo%m(1)  , ')=', geo%xb_l(1), ', ', trim(NameAr_bound(geo%bound_l(1))), &
         ' ... (', geo%n(1)+1, ')=', geo%xb_r(1), ', ', trim(NameAr_bound(geo%bound_r(1)))
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
         geo%iProcPD(2), ' [', geo%nProcsPD(2), '];', &
         ' xb2(' , geo%m(2)  , ')=', geo%xb_l(2), ', ', trim(NameAr_bound(geo%bound_l(2))), &
         ' ... (', geo%n(2)+1, ')=', geo%xb_r(2), ', ', trim(NameAr_bound(geo%bound_r(2)))
  write(ncp,'(' // inuformat // ')') &
       trim(prefix), &
         geo%iProcPD(3), ' [', geo%nProcsPD(3), '];', &
         ' xb3(' , geo%m(3)  , ')=', geo%xb_l(3), ', ', trim(NameAr_bound(geo%bound_l(3))), &
         ' ... (', geo%n(3)+1, ')=', geo%xb_r(3), ', ', trim(NameAr_bound(geo%bound_r(3)))
  !
  ! --- Print information about process ---
  ! write(ncp,'(A,1X,A,1X,I3,3X,3I3,3X,3I3)') &
  !      trim(prefix), &
  !      'Process; my, iPD(), nPD(): ', &
  !      geo%MyProc, geo%iProcPD, geo%nProcsPD
  !
  ! write(ncp,'(A)') line
endif
!
end subroutine rhd_geo_Print


!----------*****************------------------------------------------------------------------------
subroutine rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_geo_Coords2mn ('rhd_geometry_Coordinates_2_m_n')
!
! PURPOSE:
!   Compute box boundary indices from MPI process coordinates.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   PCoords:      ('Process_Coordinates') 3-element integer vector with process coordinates
!
! OUTPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-08-04 (B.F. Lyon) Put into own routine
!   2017-08-11 (B.F. Uppsala) Use global information from input geo structure and not from prop;
!                      Move routine into rhd_prop_module
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
integer, dimension(3),     intent(in)   :: PCoords
integer,                   intent(out)  :: m1,n1, m2,n2, m3,n3
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
m1=min((1+10*(geo%n(1)-geo%m(1)+1)/geo%nProcsPD(1))*(PCoords(1)  )/10+geo%m(1)  , geo%n(1)+1)
n1=min((1+10*(geo%n(1)-geo%m(1)+1)/geo%nProcsPD(1))*(PCoords(1)+1)/10+geo%m(1)-1, geo%n(1)  )
!
m2=min((1+10*(geo%n(2)-geo%m(2)+1)/geo%nProcsPD(2))*(PCoords(2)  )/10+geo%m(2)  , geo%n(2)+1)
n2=min((1+10*(geo%n(2)-geo%m(2)+1)/geo%nProcsPD(2))*(PCoords(2)+1)/10+geo%m(2)-1, geo%n(2)  )
!
m3=min((1+10*(geo%n(3)-geo%m(3)+1)/geo%nProcsPD(3))*(PCoords(3)  )/10+geo%m(3)  , geo%n(3)+1)
n3=min((1+10*(geo%n(3)-geo%m(3)+1)/geo%nProcsPD(3))*(PCoords(3)+1)/10+geo%m(3)-1, geo%n(3)  )
!
end subroutine rhd_geo_Coords2mn


!----------********************---------------------------------------------------------------------
subroutine rhd_geo_Coords2mnVec(geo, PCoords, m, n)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_geo_Coords2mnVec ('rhd_geometry_Coordinates_2_m_n_Vector')
!
! PURPOSE:
!   Compute box boundary indices from MPI process coordinates.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_geo_Coords2mnVec(geo, PCoords, m, n)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   PCoords:      ('Process_Coordinates') 3-element integer vector with process coordinates
!
! OUTPUT:
!   m:            ('m_number') integer vector with start indices
!   n:            ('number')   integer vector with end indices
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-09-11 (B.F. Uppsala) Adapt from rhd_geo_Coords2mn
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
integer, dimension(3),     intent(in)   :: PCoords
integer, dimension(3),     intent(out)  :: m, n
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
m=min((1+10*(geo%n-geo%m+1)/geo%nProcsPD)*(PCoords  )/10+geo%m  , geo%n+1)
n=min((1+10*(geo%n-geo%m+1)/geo%nProcsPD)*(PCoords+1)/10+geo%m-1, geo%n  )
!
end subroutine rhd_geo_Coords2mnVec


!----------****************-------------------------------------------------------------------------
subroutine rhd_geo_Toaction(geo, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_geo_Toaction ('rhd_geoametry_To_action')
!
! PURPOSE:
!   Copy boundary-condition settings from geo structure to action structure.
!   
! CATEGORY:
!   Parameter handling
!
! CALLING SEQUENCE:
!   call rhd_geo_Toaction(par, action)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   action:       ('action') action_type, structure with rhd control parameters
!
! ROUTINES:
!   rhd_action_New:        Function: Change contents of action variable
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Theboundary-condition settings from the geo struckture are copied into the action structure.
!   The other entries of the action structure are not touched.
!
! EXAMPLE:
!   ! --- Fill all entries of action structure according to input parameters ---
!   call rhd_par_Toaction(par   , action)
!   ! --- Overwrite boundary-conditions settings in action with values from wg%hyd ---
!   call rhd_geo_Toaction(wg%hyd, action)
!
! MODIFICATION HISTORY:
!   2018-11-18 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(inout):: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
action=rhd_action_New(action, &
                      bound_l1    =geo%bound_l(    1), &
                      bound_r1    =geo%bound_r(    1), &
                      bound_l2    =geo%bound_l(    2), &
                      bound_r2    =geo%bound_r(    2), &
                      bound_l3    =geo%bound_l(    3), &
                      bound_r3    =geo%bound_r(    3), &
                      bound_mag_l1=geo%bound_mag_l(1), &
                      bound_mag_r1=geo%bound_mag_r(1), &
                      bound_mag_l2=geo%bound_mag_l(2), &
                      bound_mag_r2=geo%bound_mag_r(2), &
                      bound_mag_l3=geo%bound_mag_l(3), &
                      bound_mag_r3=geo%bound_mag_r(3)   )
!
end subroutine rhd_geo_Toaction


!----------***********------------------------------------------------------------------------------
subroutine rhd_wg_Init()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_wg_Init ('rhd_world_geometry_Initialize')
!
! PURPOSE:
!   Initialize global collection of world-geometry structures.
!
! CATEGORY:
!   box handling, global parameters
!
! CALLING SEQUENCE:
!   call rhd_wg_Init()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Modify values of components of global variable.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Initialize components of global structure with zeros or .false.
!
! EXAMPLE:
!   call rhd_wg_Init()
!
! MODIFICATION HISTORY:
!   2014-03-19 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
integer                                 :: i
!---------------------------------------------------------------------------------------------------
!
wg%n_radMax=n_wg_radMax
wg%n_chuMax=n_wg_chuMax
wg%n_rad   =0
wg%n_chu   =0
!
call rhd_geo_Init(wg%mas )
call rhd_geo_Init(wg%hyd )
call rhd_geo_Init(wg%mean)
!
do i=1,wg%n_radMax
  call rhd_geo_Init(wg%rad(i))
end do ! i
!
do i=1,wg%n_chuMax
  call rhd_geo_Init(wg%chu(i))
end do ! i
!
end subroutine rhd_wg_Init

end module rhd_prop_module
