!***************************************************************************************************
!
!                                                     #     # #######
!   #####  #    # #####          #####  ######  ####  #     #    #
!   #    # #    # #    #         #    # #      #    # #     #    #
!   #    # ###### #    #         #    # #####  #      #     #    #
!   #####  #    # #    #         #####  #      #      #     #    #
!   #   #  #    # #    #         #   #  #      #    # #     #    #
!   #    # #    # #####          #    # ######  ####   #####     #  _module
!
!   Radiation-Hydrodynamics-Code: unit-test routines for reconstruction routines
!
!***************************************************************************************************
!   Fortran90
!   2018-05-09
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_recUT_module:      Unit-test routines for reconstruction routines
!
!***************************************************************************************************


!------****************-----------------------------------------------------------------------------
module rhd_recUT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_recUT_module ('rhd_reconstruction_Unit_Test_module')
!
! PURPOSE:
!   Provide unit-test routines for reconstruction routines.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhd_recUT_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_recUT_Vector:      Fill arrays with test values
!   rhd_recUT_Wrap:        Reconstruct and compute boundary values will different methods
!   rhd_recUT_OneRec:      Perform unit tests for one reconstruction type
!   rhd_recUT_TestAll:     Perform unit tests of reconstruction routines
!
! CALLING TREE:
!   rhd_recUT_TestAll
!     rhd_recUT_OneRec
!     + rhd_action_StrToRecon
!     + rhd_recUT_Vector
!     |   UT_PrintArray
!     + rhd_recUT_Wrap
!     |   rhd_rec_VecEq
!     |   rhd_rec_Vec
!     + UT_Assert
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-02-26 (B.F.) More comments
!   2015-03-06 (B.F.) Put into own file
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------****************-------------------------------------------------------------------------
subroutine rhd_recUT_Vector(VectorName, TestValue1, TestValue2, TestValue3, &
                            m1,n1, m2,n2, m3,n3, nghost1, av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_recUT_Vector ('radiation_hydrodynamics_reconstruction_Unit_Test_Vector')
!
! PURPOSE:
!   Fill arrays with test values.
!
! CATEGORY:
!   Reconstruction, testing
!
! CALLING SEQUENCE:
!   call rhd_recUT_Vector(VectorName, TestValue1, TestValue2, TestValue3, &
!                         m1,n1, m2,n2, m3,n3, nghost1, av, aLv, aRv)
!
! INPUT:
!   VectorName:   ('Vector_Name') character with name of vector to be produced
!   TestValue1:   ('Test_Value_1') real with 1st input parameter
!   TestValue2:   ('Test_Value_2') real with 2nd input parameter
!   TestValue3:   ('Test_Value_3') real with 3rd input parameter
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!
! OUTPUT:
!   av:           ('a_vector') 3D array with cell-centered values
!   aLv:          ('a_Left_vector') 3D array with left-side boundary values
!   aRv:          ('a_Right_vector') 3D array with right-side boundary values
!
! ROUTINES:
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   The routine just stops if the input vector name is unknown.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   According to the requested type (VectorName) and possibly the modification parameters
!   (TestValue1,2,3) a vector of values is produced and put into the 3D output arrays.
!   So far, values for identical i1 are identical, independent of i2 and i3.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use const_module, only: pi
use UT_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nghost2=0, nghost3=0, d2=0, d3=0
character(len=*),          intent(in)   :: VectorName
real,                      intent(in)   :: TestValue1, TestValue2, TestValue3
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(out)  :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3, ib1, p
real                                    :: x
!---------------------------------------------------------------------------------------------------
!
if (VectorName == 'Zero') then
  av =0.0
  aLv=0.0
  aRv=0.0
  !
else if (VectorName == 'Constant') then
  av =TestValue1
  aLv=TestValue1
  aRv=TestValue1
  !
else if (VectorName == 'Linear') then
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      do i1=m1-nghost1,n1+nghost1
        x           =float(i1-m1)/float(n1-m1)
        av(i1,i2,i3)=TestValue1 + x*TestValue2
      end do ! i1
    end do ! i2
  end do ! i3
  !
  do i3=m3-d3,n3+d3
    do i2=m2-d2,n2+d2
      do i1=m1-1,n1+1
        x            =(float(i1-m1)-0.5)/float(n1-m1)
        aLv(i1,i2,i3)=TestValue1 + x*TestValue2
        x            =(float(i1-m1)+0.5)/float(n1-m1)
        aRv(i1,i2,i3)=TestValue1 + x*TestValue2
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (VectorName == 'Parabola') then
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      do i1=m1-nghost1,n1+nghost1
        x           =float(i1-m1)/float(n1-m1)
        av(i1,i2,i3)=TestValue1 + x*(TestValue2 + x*TestValue3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  do i3=m3-d3,n3+d3
    do i2=m2-d2,n2+d2
      do i1=m1-1,n1+1
        x            =(float(i1-m1)-0.5)/float(n1-m1)
        aLv(i1,i2,i3)=TestValue1 + x*(TestValue2 + x*TestValue3)
        x            =(float(i1-m1)+0.5)/float(n1-m1)
        aRv(i1,i2,i3)=TestValue1 + x*(TestValue2 + x*TestValue3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (VectorName == 'Sin') then
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      do i1=m1-nghost1,n1+nghost1
        av(i1,i2,i3)=TestValue1+TestValue2*sin((float(i1-m1)   )*(Testvalue3/float(n1-m1+1)*2.0*pi))
      end do ! i1
    end do ! i2
  end do ! i3
  !
  do i3=m3-d3,n3+d3
    do i2=m2-d2,n2+d2
      do i1=m1-1,n1+1
        aLv(i1,i2,i3)=TestValue1+ &
                      TestValue2*sin((float(i1-m1)-0.5)*(Testvalue3/float(n1-m1+1)*2.0*pi))
        aRv(i1,i2,i3)=TestValue1+ &
                      TestValue2*sin((float(i1-m1)+0.5)*(Testvalue3/float(n1-m1+1)*2.0*pi))
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (VectorName == 'Rectangles') then
  ! --- Produce a vector like 00010110011100011110000... ---
  do i1=m1-nghost1,n1+nghost1
    av(i1,:,:)=TestValue1 + TestValue3*float(i1-m1)
  end do ! i1
  !
  ! --- Increment certain points by TestValue2
  i1 =m1+1
  p  =1
  do while (i1 <= n1+nghost1)
    ib1=i1
    do while ((i1 <= n1+nghost1) .and. ((i1-ib1) < p))
      av(i1,:,:)=av(i1,:,:)+TestValue2
      i1=i1+1
    end do
    i1=i1+p
    p=p+1
  end do
  !
  ! --- Copy the cell-center values to both cell boundaries ---
  aLv=av(m1-1:n1+1, m2-d2:n2+d2, m3-d3:n3+d3)
  aRv=av(m1-1:n1+1, m2-d2:n2+d2, m3-d3:n3+d3)
  !
else
  stop 'rhd_recUT_Vector: ERROR: unknown vector name'
  !
endif
!
if (UT%AllVecInPrint_flag) then
  call UT_PrintArray('Vi1=', av)
endif
!
end subroutine rhd_recUT_Vector


!----------**************---------------------------------------------------------------------------
subroutine rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                          reconstruction_order, reconstruction_type, &
                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                          av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_recUT_Wrap ('radiation_hydrodynamics_reconstruction_Unit_Test_Wrapper')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values will different reconstruction methods
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call  rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
!                        reconstruction_type, &
!                        SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                        av, aLv, aRv)
!
! INPUT:
!   RecMethod:    ('Reconstruction_Method') character with reconstruction method
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! VARIABLES:
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! ROUTINES:
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   rhd_rec_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   rhd_rec_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!
! MODULES:
!   rhd_rec_module:        Reconstruction routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Call the individual reconstruction methods (equidistand and general) with an equidistant grid.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2015-03-05 (B.F. Uppsala) Written
!   2018-05-09 (B.F.) Pass dimension specifications as parameters;
!                     Bugfix: upwind indices i1/i1+1 -> i1-1,i1
!---------------------------------------------------------------------------------------------------
use rhd_rec_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: RecMethod
integer, parameter                      :: nghost2=0, nghost3=0, d2=0, d3=0
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3, j
real                                    :: dtime, dxb1_avg, dummy
real, dimension(m1-nghost1:n1+nghost1)  :: dxb1
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2    , &
                m3:n3       )           :: alpha
integer, dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3     )          :: i_up
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3     )             :: lambda, flux_alpha_out
!---------------------------------------------------------------------------------------------------
!
if (RecMethod == 'VecEq') then
  ! --- Reconstruct vector with equidistant grid ---
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     av, aLv, aRv)
  !
else if (RecMethod == 'VecNe') then
  ! --- Reconstruct vector with general grid (set to be equidistant in this case) ---
  dxb1=1.0
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   av, dxb1, aLv, aRv)
  !
else if (RecMethod == 'FluxEq') then
  ! --- Get reconstructed boundary states from flux ---
  dtime   =0.0
  dxb1_avg=1.0
  !
  ! --- Get right boundary values from flux with positive velocities ---
  lambda=1.0
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order,reconstruction_order
        do i1=m1,n1+1
          alpha(i1,j,i2,i3)=av(i1+j-1,i2,i3)
        end do ! i1
        !alpha(m1-1,j,i2,i3)=alpha(m1  ,j,i2,i3)
      end do ! j
    end do ! i2
  end do ! i3
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        i_up(i1,i2,i3)=i1-1
      end do ! i1
    end do ! i2
  end do ! i3
  !
  call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                           reconstruction_order, reconstruction_type, &
                           dtime, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           alpha, dxb1_avg, lambda, &
                           flux_alpha_out)
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        aRv(i1-1,i2,i3)=flux_alpha_out(i1,i2,i3)/lambda(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  ! --- Get left boundary values from flux with negative velocities ---
  lambda=-1.0
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order,reconstruction_order
        do i1=m1,n1+1
          alpha(i1,j,i2,i3)=av(i1+j,i2,i3)
        end do ! i1
        ! alpha(n1+2,j,i2,i3)=alpha(n1+1,j,i2,i3)
      end do ! j
    end do ! i2
  end do ! i3
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        i_up(i1,i2,i3)=i1
      end do ! i1
    end do ! i2
  end do ! i3
  !
  call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                           reconstruction_order, reconstruction_type, &
                           dtime, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           alpha, dxb1_avg, lambda, &
                           flux_alpha_out)
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        aLv(i1  ,i2,i3)=flux_alpha_out(i1,i2,i3)/lambda(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (RecMethod == 'FluxNe') then
  ! --- Get reconstructed boundary states from flux ---
  dtime=0.0
  dxb1 =1.0
  !
  ! --- Get right boundary values from flux with positive velocities ---
  lambda=1.0
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order,reconstruction_order
        do i1=m1,n1+1
          alpha(i1,j,i2,i3)=av(i1+j-1,i2,i3)
        end do ! i1
      end do ! j
    end do ! i2
  end do ! i3
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        i_up(i1,i2,i3)=i1-1
      end do ! i1
    end do ! i2
  end do ! i3
  !
  call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         dtime, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         alpha, dxb1, i_up, lambda, &
                         flux_alpha_out)
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        aRv(i1-1,i2,i3)=flux_alpha_out(i1,i2,i3)/lambda(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  ! --- Get left boundary values from flux with negative velocities ---
  lambda=-1.0
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order,reconstruction_order
        do i1=m1,n1+1
          alpha(i1,j,i2,i3)=av(i1+j,i2,i3)
        end do ! i1
      end do ! j
    end do ! i2
  end do ! i3
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        i_up(i1,i2,i3)=i1
      end do ! i1
    end do ! i2
  end do ! i3
  !
  call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         dtime, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         alpha, dxb1, i_up, lambda, &
                         flux_alpha_out)
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        aLv(i1  ,i2,i3)=flux_alpha_out(i1,i2,i3)/lambda(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
else
  stop 'rhd_recUT_Wrap: ERROR: unknown reconstruction method'
endif
!
end subroutine rhd_recUT_Wrap


!----------****************-------------------------------------------------------------------------
subroutine rhd_recUT_OneRec(TestPrefix, reconstruction, RecMethod, &
                            SlopeRed_flag, SlopeCon_flag, SlopeRed_value, SlopeCon_value)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_recUT_OneRec ('radiation_hydrodynamics_reconstruction_Unit_Test_One_Reconstruction')
!
! PURPOSE:
!   Perform unit tests for one reconstruction type.
!
! CATEGORY:
!   Reconstruction, testing
!
! CALLING SEQUENCE:
!   call rhd_recUT_OneRec(TestPrefix, reconstruction, RecMethod, &
!                         SlopeRed_flag, SlopeCon_flag, SlopeRed_value, SlopeCon_value)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character describing the current group of tests
!   reconstruction: ('reconstruction') character with reconstruction scheme
!   RecMethod:    ('Reconstruction_Method') character with reconstruction method
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_action_StrToRecon: Analyze string with reconstruction scheme
!   rhd_recUT_Vector:      Fill arrays with test values
!   rhd_recUT_Wrap:        Reconstruct and compute boundary values will different methods
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Create a number of test vectors.
!   Reconstruct each of them and compute the reconstructed boundary values.
!   Check if the boundary values are consistent with various expectations e.g., about linearity:
!     Some properties should be fulfilled: adding or multiplying a constant.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-05 (B.F.) Ignore the outcome of some tests instead of giving a huge error limit
!   2018-05-09 (B.F.) Use non-zero delta also in check for zero.
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix, reconstruction, RecMethod
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real,                      intent(in)   :: SlopeRed_value, SlopeCon_value
!
! --- Local variables ---
integer, parameter                      :: nghost2=0, nghost3=0, d2=0, d3=0
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, d, &
                                           reconstruction_order, reconstruction_type
real                                    :: TestValue, delta
logical                                 :: Ignore_flag
character(len=80)                       :: VectorName
character(len=128)                      :: TestName
real, dimension(:,:,:), allocatable     :: SlopeRed, SlopeCon
real, dimension(:,:,:), allocatable     :: av
real, dimension(:,:,:), allocatable     :: aLv, aRv, aLv2, aRv2, aLvRef, aRvRef
!---------------------------------------------------------------------------------------------------
!
! --- Choose array size ---
m1=1
n1=18
m2=1
n2=1
m3=1
n3=1
!
d=0
!
call rhd_action_StrToRecon(reconstruction, reconstruction_order, reconstruction_type)
nghost1=reconstruction_order+1
!
! --- Allocate necessary arrays ---
allocate(SlopeRed(m1-1 :n1+2, m2-d2:n2+d2, m3-d3:n3+d3), &
         SlopeCon(m1-1 :n1+2, m2-d2:n2+d2, m3-d3:n3+d3), &
         av(  m1-nghost1:n1+nghost1, m2-nghost2:n2+nghost2, m3-nghost3:n3+nghost3), &
         aLv(     m1-1 :n1+1, m2-d2:n2+d2, m3-d3:n3+d3), &
         aRv(     m1-1 :n1+1, m2-d2:n2+d2, m3-d3:n3+d3), &
         aLv2(    m1-1 :n1+1, m2-d2:n2+d2, m3-d3:n3+d3), &
         aRv2(    m1-1 :n1+1, m2-d2:n2+d2, m3-d3:n3+d3), &
         aLvRef(  m1-1 :n1+1, m2-d2:n2+d2, m3-d3:n3+d3), &
         aRvRef(  m1-1 :n1+1, m2-d2:n2+d2, m3-d3:n3+d3))
!
SlopeRed=SlopeRed_value
SlopeCon=SlopeCon_Value
!
!
! === Create test cases: constants =================================================================
VectorName='Zero'
delta    =1.0E-07
TestValue=0.0
call rhd_recUT_Vector('Constant', TestValue, 0.0, 0.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSideIs' // VectorName
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), TestValue, delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSideIs' // VectorName
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), TestValue, delta=delta)
!
!
! === Create test case ===
VectorName='One'
delta    =1.0E-05
TestValue=1.0
call rhd_recUT_Vector('Constant', TestValue, 0.0, 0.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSideIs' // VectorName
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), TestValue, delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSideIs' // VectorName
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), TestValue, delta=delta)
!
!
! === Create test case ===
VectorName='MinusOne'
delta    =1.0E-05
TestValue=-1.0
call rhd_recUT_Vector('Constant', TestValue, 0.0, 0.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSideIs' // VectorName
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), TestValue, delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSideIs' // VectorName
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), TestValue, delta=delta)
!
!
! === Create test cases: linear slopes =============================================================
Ignore_flag=((reconstruction_order <= 0) .or. &
             (SlopeRed_flag .and. (SlopeRed_value < 1.0)))
VectorName='LinearUp'
delta=1.0E-05
call rhd_recUT_Vector('Linear', -0.2, 1.2, 0.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRvRef(m1-d:n1+d,:,:), delta=delta, &
                Ignore_flag=Ignore_flag)
!
!
! === Create test case ===
VectorName='LinearDown'
delta=1.0E-05
call rhd_recUT_Vector('Linear', 0.2, -1.2, 0.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
!
!
! === Create test cases: parabolas =================================================================
Ignore_flag=(((reconstruction_type /= reconstruction_type_PP     ) .and. &
              (reconstruction_type /= reconstruction_type_FRmono ) .and. &
              (reconstruction_type /= reconstruction_type_FRmono2) .and. &
              (reconstruction_type /= reconstruction_type_FRweno ) .and. &
              (reconstruction_type /= reconstruction_type_FRweno2) .and. &
              (reconstruction_type /= reconstruction_type_FRcont ) .and. &
              (reconstruction_type /= reconstruction_type_VAweno ) .and. &
              (reconstruction_type /= reconstruction_type_HBweno )       ) .or. &
             (SlopeRed_flag .and. (SlopeRed_value < 1.0))                        )
delta=2.0E-05
!
VectorName='ParabolaUp'
call rhd_recUT_Vector('Parabola', 0.0, 1.0, 0.05, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
!
!
! === Create test case ===
VectorName='ParabolaDown'
call rhd_recUT_Vector('Parabola', 1.0, -1.0, -0.05, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRvRef(m1-d:n1+d,:,:), delta=delta, &
               Ignore_flag=Ignore_flag)
!
!
! === Create test case =============================================================================
VectorName='Sin1'
TestValue=1.0
delta=1.0E-05
call rhd_recUT_Vector('Sin', 0.0, 1.0, 1.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av+TestValue, aLv2, aRv2)
aLv2=aLv2-TestValue
aRv2=aRv2-TestValue
!
! aLv2(m1,m2,m3)=20.0  ! --- Attention: forced error ---
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'AddedLeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLv2(m1-d:n1+d,:,:), delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'AddedRightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRv2(m1-d:n1+d,:,:), delta=delta)
!
!
! === Create test case ===
TestValue=3.0
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av*TestValue, aLv2, aRv2)
aLv2=aLv2/TestValue
aRv2=aRv2/TestValue
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'MultipliedLeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLv2(m1-d:n1+d,:,:), delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'MultipliedRightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRv2(m1-d:n1+d,:,:), delta=delta)
!
!
! === Create test case ===
TestValue=-1.0
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av*TestValue, aLv2, aRv2)
aLv2=aLv2/TestValue
aRv2=aRv2/TestValue
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'InvertedLeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLv2(m1-d:n1+d,:,:), delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'InvertedRightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRv2(m1-d:n1+d,:,:), delta=delta)
!
!
! === Create test case ===
delta=1.0E-05
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av(n1+nghost1:m1-nghost1:-1,:,:), aLv2, aRv2)
aLvRef=aRv2(n1+1:m1-1:-1,:,:)
aRv2  =aLv2(n1+1:m1-1:-1,:,:)
aLv2  =aLvRef
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'ReversedLeftSidesAreAlmostEqual'
call UT_Assert('==', TestName, aLv(m1-d:n1+d,:,:), aLv2(m1-d:n1+d,:,:), delta=delta)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'ReversedRightSidesAreAlmostEqual'
call UT_Assert('==', TestName, aRv(m1-d:n1+d,:,:), aRv2(m1-d:n1+d,:,:), delta=delta)
!
!
! === Create test cases: rectangles ================================================================
Ignore_flag=((reconstruction_type /= reconstruction_type_Constant) .and. &
             (reconstruction_type /= reconstruction_type_MinMod  ) .and. &
             (reconstruction_type /= reconstruction_type_VanLeer ) .and. &
             (reconstruction_type /= reconstruction_type_SuperBee) .and. &
             (reconstruction_type /= reconstruction_type_PPmimo  ) .and. &
             (reconstruction_type /= reconstruction_type_PP      ) .and. &
             (reconstruction_type /= reconstruction_type_FRmimo  ) .and. &
             (reconstruction_type /= reconstruction_type_FRmono  ) .and. &
             (reconstruction_type /= reconstruction_type_FRmono2 )        )
delta=1.0E-06
if (nghost1 > 1) then
  ! d=1
else
  d=0
endif
VectorName='Rectangles'
call rhd_recUT_Vector('Rectangles', 0.0, 1.0, 0.0, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! aLv(m1     ,m2,m3)= 20.0  ! --- Attention: forced error ---
! aRv(m1:m1:2,m2,m3)=-20.0  ! --- Attention: forced error ---
! aLv=aLv+2.0*delta         ! --- Attention: forced error ---
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesDontOvershoot'
call UT_Assert('<=', TestName, &
       aLv(m1-d:n1+d,:,:), max(av(m1-1-d:n1-1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesDontOvershoot'
call UT_Assert('<=', TestName, &
       aRv(m1-d:n1+d,:,:), max(av(m1+1-d:n1+1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)

TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesDontUndershoot'
call UT_Assert('>=', TestName, &
       aLv(m1-d:n1+d,:,:), min(av(m1-1-d:n1-1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesDontUndershoot'
call UT_Assert('>=', TestName, &
       aRv(m1-d:n1+d,:,:), min(av(m1+1-d:n1+1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
!
! --- Create test ---
VectorName='RectanglesUp'
call rhd_recUT_Vector('Rectangles', 0.0, 1.0, 0.1, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesDontOvershoot'
call UT_Assert('<=', TestName, &
       aLv(m1-d:n1+d,:,:), max(av(m1-1-d:n1-1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesDontOvershoot'
call UT_Assert('<=', TestName, &
       aRv(m1-d:n1+d,:,:), max(av(m1+1-d:n1+1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesDontUndershoot'
call UT_Assert('>=', TestName, &
       aLv(m1-d:n1+d,:,:), min(av(m1-1-d:n1-1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesDontUndershoot'
call UT_Assert('>=', TestName, &
       aRv(m1-d:n1+d,:,:), min(av(m1+1-d:n1+1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
!
! --- Create test ---
VectorName='RectanglesDown'
call rhd_recUT_Vector('Rectangles', 0.0, 1.0, -0.1, m1,n1, m2,n2, m3,n3, nghost1, &
                      av, aLvRef, aRvRef)
!
! --- Call the reconstruction routine to be tested ---
call rhd_recUT_Wrap(RecMethod, m1,n1, m2,n2, m3,n3, nghost1, &
                    reconstruction_order, reconstruction_type, &
                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                    av, aLv, aRv)
!
! --- Check ---
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesDontOvershoot'
call UT_Assert('<=', TestName, &
       aLv(m1-d:n1+d,:,:), max(av(m1-1-d:n1-1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesDontOvershoot'
call UT_Assert('<=', TestName, &
       aRv(m1-d:n1+d,:,:), max(av(m1+1-d:n1+1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'LeftSidesDontUndershoot'
call UT_Assert('>=', TestName, &
       aLv(m1-d:n1+d,:,:), min(av(m1-1-d:n1-1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
TestName=trim(TestPrefix) // trim(reconstruction) // '_' // trim(RecMethod) // '_' // &
         trim(VectorName) // '_' // &
         'RightSidesDontUndershoot'
call UT_Assert('>=', TestName, &
       aRv(m1-d:n1+d,:,:), min(av(m1+1-d:n1+1+d,:,:), av(m1-d:n1+d,:,:)), &
       delta=delta, Ignore_flag=Ignore_flag)
!
! ==================================================================================================
!
! --- Deallocate arrays ---
deallocate(SlopeRed, SlopeCon, &
           av, &
           aLv, aRv, aLv2, aRv2, aLvRef, aRvRef)
!
end subroutine rhd_recUT_OneRec


!----------*****************------------------------------------------------------------------------
subroutine rhd_recUT_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_recUT_TestAll ('radiation_hydrodynamics_reconstruction_Unit_Test_Test_All')
!
! PURPOSE:
!   Perform unit tests of reconstruction routines.
!
! CATEGORY:
!   Reconstruction, testing
!
! CALLING SEQUENCE:
!   call rhd_recUT_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_recUT_OneRec:      Perform unit tests for one reconstruction type
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Set global parameters in par structure.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Perform all unit tests of reconstruction routines.
!
! EXAMPLE:
!   call rhd_recUT_TestAll()
!
! MODIFICATION HISTORY:
!   2015-02-26 (B.F. Uppsala) First version
!   2015-02-27 (B.F.) Vary SlopeCon and SlopeRed
!   2016-05-14 (B.F.) Replace local array RecName with global array NameAr_reconstruction_type
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- Local variables ---
integer                                 :: iRec, iSlope
logical                                 :: SlopeRed_flag , SlopeCon_flag
real                                    :: SlopeRed_value, SlopeCon_value
character(len=80)                       :: TestPrefix, RecMethod
!---------------------------------------------------------------------------------------------------
!
! --- Set global parameters in par structure ---
par%C_RecWenoPsiFactor   =5.0
par%C_RecWenoWeightCenter=0.6
par%C_RecWenoWeightRight =2.0/3.0
!
! --- Loop over all tests ---
do iSlope=1,5
  if (iSlope == 1) then
    TestPrefix='Reconstruct_RfCf_'
    SlopeRed_flag =.false.
    SlopeCon_flag =.false.
    SlopeRed_value=1.0
    SlopeCon_value=1.0
  else if (iSlope == 2) then
    TestPrefix='Reconstruct_Rt1Cf_'
    SlopeRed_flag =.true.
    SlopeCon_flag =.false.
    SlopeRed_value=1.0
    SlopeCon_value=1.0
  else if (iSlope == 3) then
    TestPrefix='Reconstruct_Rt1Ct1_'
    SlopeRed_flag =.true.
    SlopeCon_flag =.true.
    SlopeRed_value=1.0
    SlopeCon_value=1.0
  else if (iSlope == 4) then
    TestPrefix='Reconstruct_Rt0.5Ct1_'
    SlopeRed_flag =.true.
    SlopeCon_flag =.true.
    SlopeRed_value=0.5
    SlopeCon_value=1.0
  else if (iSlope == 5) then
    TestPrefix='Reconstruct_Rt1Ct0.5_'
    SlopeRed_flag =.true.
    SlopeCon_flag =.true.
    SlopeRed_value=1.0
    SlopeCon_value=0.5
  else
    stop 'rhd_recUT_module: ERROR: Too many slope modifiers'
  endif
  !
  do iRec=reconstruction_type_First, reconstruction_type_Last
    RecMethod='VecEq'
    call rhd_recUT_OneRec(TestPrefix, trim(NameAr_reconstruction_type(iRec)), trim(RecMethod), &
                          SlopeRed_flag, SlopeCon_flag, SlopeRed_value, SlopeCon_value)
    !
    RecMethod='VecNe'
    call rhd_recUT_OneRec(TestPrefix, trim(NameAr_reconstruction_type(iRec)), trim(RecMethod), &
                          SlopeRed_flag, SlopeCon_flag, SlopeRed_value, SlopeCon_value)
    !
    RecMethod='FluxEq'
    call rhd_recUT_OneRec(TestPrefix, trim(NameAr_reconstruction_type(iRec)), trim(RecMethod), &
                          SlopeRed_flag, SlopeCon_flag, SlopeRed_value, SlopeCon_value)
    !
    RecMethod='FluxNe'
    call rhd_recUT_OneRec(TestPrefix, trim(NameAr_reconstruction_type(iRec)), trim(RecMethod), &
                          SlopeRed_flag, SlopeCon_flag, SlopeRed_value, SlopeCon_value)
  end do ! iRec
end do ! iSlope
!
end subroutine rhd_recUT_TestAll

end module rhd_recUT_module
