!***************************************************************************************************
!
!   #####   #    #  #####           ####   #    #  #####
!   #    #  #    #  #    #         #       #    #  #    #
!   #    #  ######  #    #          ####   #    #  #####
!   #####   #    #  #    #              #  #    #  #    #
!   #   #   #    #  #    #         #    #  #    #  #    #
!   #    #  #    #  #####  #######  ####    ####   ##### _module
!
!***************************************************************************************************
!   Fortran90
!   2019-03-05
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!   Werner Schaffenberger: Potsdam, Exeter
!   Sven Wedemeyer-Boehm:  Kiel, Freiburg
!***************************************************************************************************
! MODULES:
!   rhd_sub_module:        Additional subroutines for RHD
!
!***************************************************************************************************


!------**************-------------------------------------------------------------------------------
module rhd_sub_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_sub_module ('rhd_sub_module')
!
! PURPOSE:
!   Additional routines for RHD.
!
! CATEGORY:
!   Mixed
!
! CALLING SEQUENCE:
!   use rhd_sub_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_par_PrintInfo:     Print information about RHD parameters
!   rhd_par_Write:         Write RHD parameter into file in UIO form
!   rhd_par_Read:          Read parameters from parameter file and put them into global variable
!   rhd_par_ToSubpar:      Set values in parameter sub structure based on input parameters
!   rhd_par_Toaction:      Prepare action variable from input parameters
!   rhd_box_Read:          Read box from file in UIO form
!   rhd_box_Write:         Write box with additional infos into file in UIO form
!
! MODIFICATION HISTORY:
!   1997-06-09 (B.F. Kiel) Written
!   1999-05-28 (B.F. Copenhagen) Several routines put into other modules
!   2002-10-21 (B.F.) I/O of quc
!   2002-11-01 (B.F.) I/O of magnetic field
!   2005-05-31 (W.S.&SWB) MHD parameters
!   2006-08-07 (SWB)  hion_datapath, hion_atomfile, hion_abufile, hion_edensfile, 
!                     hion_pffile, dtime_out_hion, hion_chunks
!   2007-10-22 (B.F. Lyon) box%time_db introduced
!   2010-05-10 (B.F.) Some routines renamed
!   2015-02-23 (B.F. Uppsala) Move rhd_par_ToactionRecon into rhd_action_module and rename it
!   2015-04-25 (B.F.) Make control structure input parameter in a few routines
!   2018-11-17 (B.F.) Split subroutine rhd_par_Toaction
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------*****************------------------------------------------------------------------------
subroutine rhd_par_PrintInfo(parL)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_par_PrintInfo ('rhd_parameters_Print_Information')
!
! PURPOSE:
!   Print information about RHD parameters.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_par_PrintInfo(parL)
!
! INPUT:
!   parL:         ('parameter_Local') structure with control parameters
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   Print text on channel gl%nc_p.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-10 (B.F. Kiel) Written
!   2006-08-07 (SWB)  hion_datapath, hion_atomfile, hion_abufile, hion_edensfile, 
!                     hion_pffile, dtime_out_hion, hion_chunks
!   2008-06-10 (W.S. Freiburg) Magnetic boundary conditions
!   2010-05-10 (B.F. Lyon) Rename rhd_infopar -> rhd_par_PrintInfo
!   2015-04-25 (B.F. Uppsala) Make control structure input parameter
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(par_type),            intent(in)   :: parL
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
write(gl%nc_p,*)
write(gl%nc_p,'(A)') repeat('-',79)
write(gl%nc_p,'(A)') 'Parameter file: ' // trim(gl%parFile)
!
! --- Description ---
write(gl%nc_p,'(A)') 'Description:'
write(gl%nc_p,'(A)') parL%description(:parL%ndes)
!
! --- History ---
write(gl%nc_p,'(A)') 'History:'
write(gl%nc_p,'(A)') parL%history(:parL%nhis)
!
! --- Effective temperature ---
write(gl%nc_p,'(A,F15.6)')  'Teff:           ', parL%Teff
write(gl%nc_p,'(A,A)')      'heat_mode:      ', parL%heat_mode
!
! --- Gravity ---
write(gl%nc_p,'(A,A)')      'grav_mode:      ', parL%grav_mode
write(gl%nc_p,'(A,A)')      'grav_type:      ', parL%grav_type
write(gl%nc_p,'(A,E15.8)')  'grav:           ', parL%grav
write(gl%nc_p,'(A,E15.8)')  'mass_star:      ', parL%mass_star
write(gl%nc_p,'(A,E15.8)')  'C_gravmu:       ', parL%C_gravmu
write(gl%nc_p,'(A,E15.8)')  'C_gravCorePotDepth: ', parL%C_gravCorePotDepth
write(gl%nc_p,'(A,E15.8)')  'C_gravCorePotQuad:  ', parL%C_gravCorePotQuad
write(gl%nc_p,'(A,E15.8)')  'r0_grav:        ', parL%r0_grav
write(gl%nc_p,'(A,E15.8)')  'r1_grav:        ', parL%r1_grav
write(gl%nc_p,'(A,E15.8)')  'r2_grav:        ', parL%r2_grav
write(gl%nc_p,'(A,E15.8)')  'r1_rad:         ', parL%r1_rad
write(gl%nc_p,'(A,E15.8)')  'C_gravCorer0Heat: ', parL%C_gravCorer0Heat
write(gl%nc_p,'(A)') 'centrifugal_force: ' // trim(parL%centrifugal_force)
write(gl%nc_p,'(A,E15.8)')  'nu_rotation:    ', parL%nu_rotation
write(gl%nc_p,'(A,3(1PE13.6))')'ar_RotationAxis:', parL%ar_RotationAxis(1:3)
!
! --- Abundances ---
write(gl%nc_p,'(A,F15.12)') 'abuX:           ', parL%abuX
write(gl%nc_p,'(A,F15.12)') 'abuY:           ', parL%abuY
write(gl%nc_p,'(A,F15.12)') 'abuZ:           ', parL%abuZ
write(gl%nc_p,'(A,E15.8)')  'Qmol:           ', parL%Qmol
write(gl%nc_p,'(A,E15.8)')  'gamma:          ', parL%gamma
!
! --- EOS file ---
write(gl%nc_p,'(A)') 'EOS file name:     ' // trim(parL%EOSFile)
write(gl%nc_p,'(A)') 'EOS path name:     ' // trim(parL%EOSPath)
!
! --- Opacity file ---
write(gl%nc_p,'(A)') 'opacity file name: ' // trim(parL%OpaFile)
write(gl%nc_p,'(A)') 'opacity path name: ' // trim(parL%OpaPath)
write(gl%nc_p,'(A,E15.8)')  'kappaMin:       ', parL%kappaMin
write(gl%nc_p,'(A,E15.8)')  'kappaMax:       ', parL%kappaMax
!
! --- Chemical reaction network data file ---
write(gl%nc_p,'(A,E15.8)')  'chem_abumetal:   ', parL%chem_abumetal
write(gl%nc_p,'(A)') 'reaction file name:' // trim(parL%chem_reacfile)
write(gl%nc_p,'(A)') 'reaction path name:' // trim(parL%chem_reacpath)
! 
! --- HION data files etc ---
write(gl%nc_p,'(A)')         'HION data path:    ' // trim(parL%hion_datapath)
write(gl%nc_p,'(A)')         'HION atom file:    ' // trim(parL%hion_atomfile)
write(gl%nc_p,'(A)')         'HION abund. file:  ' // trim(parL%hion_abufile)
write(gl%nc_p,'(A)')         'HION el.dens. file:' // trim(parL%hion_edensfile)
write(gl%nc_p,'(A)')         'HION part.f. file: ' // trim(parL%hion_pffile)
write(gl%nc_p,'(A,1PE15.8)') 'dtime_out_hion:    ', parL%dtime_out_hion
write(gl%nc_p,'(A,I15)')     'hion_chunks:       ', parL%hion_chunks
!
! --- Boundary conditions ---
write(gl%nc_p,'(A)') 'Side boundaries:   ' // trim(parL%side_bound)
write(gl%nc_p,'(A)') 'Top boundaries:    ' // trim(parL%top_bound)
write(gl%nc_p,'(A)') 'Bottom boundaries: ' // trim(parL%bottom_bound)
write(gl%nc_p,'(A)') 'Side bound mag x1: ' // trim(parL%side_bound_mag_x1)
write(gl%nc_p,'(A)') 'Side bound mag x2: ' // trim(parL%side_bound_mag_x2)
write(gl%nc_p,'(A)') 'Top bound mag:     ' // trim(parL%top_bound_mag)
write(gl%nc_p,'(A)') 'Bottom bound mag:  ' // trim(parL%bottom_bound_mag)
write(gl%nc_p,'(A,E15.8)')  's_InFlow:                ', parL%s_InFlow
write(gl%nc_p,'(A,E15.8)')  's_InFlRadGrad:           ', parL%s_InFlRadGrad
write(gl%nc_p,'(A,E15.8)')  's_InFlLatGrad:           ', parL%s_InFlLatGrad
write(gl%nc_p,'(A,E15.8)')  'C_sChange:               ', parL%C_sChange
write(gl%nc_p,'(A,E15.8)')  'C_PChange:               ', parL%C_PChange
write(gl%nc_p,'(A,E15.8)')  'C_v3ChangeLinBottom:     ', parL%C_v3ChangeLinBottom
write(gl%nc_p,'(A,E15.8)')  'C_v3ChangeSqrBottom:     ', parL%C_v3ChangeSqrBottom
write(gl%nc_p,'(A,E15.8)')  'C_TChange:               ', parL%C_TChange
write(gl%nc_p,'(A,E15.8)')  'C_rhoChangeTop:          ', parL%C_rhoChangeTop
write(gl%nc_p,'(A,E15.8)')  'C_Tsurf:                 ', parL%C_Tsurf
write(gl%nc_p,'(A,E15.8)')  'C_Hptopfactor:           ', parL%C_Hptopfactor
write(gl%nc_p,'(A,E15.8)')  'rho_min:                 ', parL%rho_min
write(gl%nc_p,'(A,E15.8)')  'LuminosityPerVolume:     ', parL%LuminosityPerVolume
write(gl%nc_p,'(A)') 'Core heat profile: ' // trim(parL%hdCoreHeatProfile)
write(gl%nc_p,'(A)') 'Core drag profile: ' // trim(parL%hdCoreDragProfile)
write(gl%nc_p,'(A,E15.8)')  'C_CoreDrag:              ', parL%C_CoreDrag
write(gl%nc_p,'(A,E15.8)')  'B1_inflow:               ', parL%B1_inflow
write(gl%nc_p,'(A,E15.8)')  'C_magthetaB:             ', parL%C_magthetaB
write(gl%nc_p,'(A,E15.8)')  'C_magphiB:               ', parL%C_magphiB
write(gl%nc_p,'(A,I15)')    'N cells diff side layer: ', parL%N_magDiffSide
write(gl%nc_p,'(A,I15)')    'N cells diff top layer:  ', parL%N_magDiffTop
write(gl%nc_p,'(A,I15)')    'N cells diff bot layer:  ', parL%N_magDiffBottom
!
! --- Hydrodynamics control ---
write(gl%nc_p,'(A,I15)')    'Cells per chunk (HYD):   ', parL%N_hydCellsPerChunk
write(gl%nc_p,'(A,I15)')    'Cells per chunk (HYD2):  ', parL%N_hydCellsPerChunk2
write(gl%nc_p,'(A,I15)')    'Cells per chunk (HYD3):  ', parL%N_hydCellsPerChunk3
write(gl%nc_p,'(A,I15)')    'Cells per chunk (VIS):   ', parL%N_visCellsPerChunk
write(gl%nc_p,'(A,I15)')    'Cells per chunk (RES):   ', parL%N_resCellsPerChunk
write(gl%nc_p,'(A,I15)')    'Cells per chunk (RAD):   ', parL%N_radCellsPerChunk
write(gl%nc_p,'(A,3(I5))')  'aN_MPIProcsPerDim:       ', parL%aN_MPIProcsPerDim
write(gl%nc_p,'(A)')        'Hydro scheme:      ' // trim(parL%hdScheme)
write(gl%nc_p,'(A)')        'Hydro splitting:   ' // trim(parL%hdsplit)
write(gl%nc_p,'(A)')        'Tmp-Struct Retain Mode:  ' // trim(parL%hdTmpStructKeepMode)
write(gl%nc_p,'(A)')        'MHD CheckFlux:     ' // trim(parL%hdCheckFlux)
write(gl%nc_p,'(A)')        'Reconstruction:    ' // trim(parL%reconstruction)
write(gl%nc_p,'(A)')        'hdTransVeloMode:   ' // trim(parL%hdTransVeloMode)
write(gl%nc_p,'(A)')        'hdEntropyWaveMode: ' // trim(parL%hdEntropyWaveMode)
write(gl%nc_p,'(A)')        'hdEnthalpyAvgMode: ' // trim(parL%hdEnthalpyAvgMode)
write(gl%nc_p,'(A)')        'MHD time-integration: ' // trim(parL%hdTimeIntegrationScheme)
write(gl%nc_p,'(A,I15)')    'Hydro iterations:        ', parL%N_hydIter
write(gl%nc_p,'(A,I15)')    'Hydro iterations (max):  ', parL%N_hydMaxIter
write(gl%nc_p,'(A,I15)')    'Hydro RK iterations:     ', parL%N_hydRKIter
write(gl%nc_p,'(A,E15.8)')  'C_hydTDiffLin:           ', parL%C_hydTDiffLin
write(gl%nc_p,'(A,E15.8)')  'C_hydTDiffMach:          ', parL%C_hydTDiffMach
write(gl%nc_p,'(A,E15.8)')  'C_hydsDiffVelo:          ', parL%C_hydsDiffVelo
write(gl%nc_p,'(A,E15.8)')  'C_hydvDiffVelo:          ', parL%C_hydvDiffVelo
write(gl%nc_p,'(A,E15.8)')  'C_hydLowMachcsFactor:    ', parL%C_hydLowMachcsFactor
write(gl%nc_p,'(A,E15.8)')  'C_hydLowMachcsBase:      ', parL%C_hydLowMachcsBase
write(gl%nc_p,'(A,E15.8)')  'C_hydSmallFluctFactor:   ', parL%C_hydSmallFluctFactor
write(gl%nc_p,'(A,E15.8)')  'C_SlopeRed:              ', parL%C_SlopeRed
write(gl%nc_p,'(A,E15.8)')  'C_RecContShift:          ', parL%C_RecContShift
write(gl%nc_p,'(A,E15.8)')  'C_RecContSteep:          ', parL%C_RecContSteep
write(gl%nc_p,'(A,E15.8)')  'C_RecWenoWeightCenter:   ', parL%C_RecWenoWeightCenter
write(gl%nc_p,'(A,E15.8)')  'C_RecWenoWeightRight:    ', parL%C_RecWenoWeightRight
write(gl%nc_p,'(A,E15.8)')  'C_RecWenoPsiFactor:      ', parL%C_RecWenoPsiFactor
write(gl%nc_p,'(A,E15.8)')  'C_hydPredfactor:         ', parL%C_hydPredfactor
write(gl%nc_p,'(A,E15.8)')  'C_visBound:              ', parL%C_visBound
!
! --- Parameters for MHD module ---
write(gl%nc_p,'(A,E15.8)')  'C_resB:                  ', parL%C_resB
write(gl%nc_p,'(A,E15.8)')  'C_resBconstant:          ', parL%C_resBconstant
write(gl%nc_p,'(A,E15.8)')  'C_resEpsilon:            ', parL%C_resEpsilon
write(gl%nc_p,'(A,E15.8)')  'beta_inv:                ', parL%beta_inv
write(gl%nc_p,'(A,E15.8)')  'va_max:                  ', parL%va_max
write(gl%nc_p,'(A,I15)')    'Order of Const. Transp.: ', parL%N_orderConstrainedTransport
!
! --- Dust/molecule formation scheme ---
write(gl%nc_p,'(A)')        'Dust scheme:       ' // trim(parL%dustScheme)
write(gl%nc_p,'(A)')        'Dust reconstruct.: ' // trim(parL%dustReconstruction)
write(gl%nc_p,'(A)')        'Dust opacity file: ' // trim(parL%dustOpaFile)
write(gl%nc_p,'(A)')        'Dust opacity mode: ' // trim(parL%dustopamode)
write(gl%nc_p,'(A,5(1PE15.8))') 'ar_dustGrainRadius: ', &
                                                   parL%ar_dustGrainRadius(1:parL%n_dustGrainRadius)
write(gl%nc_p,'(A,5(1PE15.8))') 'ar_dustGrainAbu:    ', &
                                                   parL%ar_dustGrainAbu(1:parL%n_dustGrainAbu)
write(gl%nc_p,'(A,E15.8)')  'C_dust01:                ', parL%C_dust01
write(gl%nc_p,'(A,E15.8)')  'C_dust02:                ', parL%C_dust02
write(gl%nc_p,'(A,E15.8)')  'C_dust03:                ', parL%C_dust03
write(gl%nc_p,'(A,E15.8)')  'C_dust04:                ', parL%C_dust04
write(gl%nc_p,'(A,E15.8)')  'C_dust05:                ', parL%C_dust05
write(gl%nc_p,'(A,E15.8)')  'C_dust06:                ', parL%C_dust06
write(gl%nc_p,'(A,E15.8)')  'C_dust07:                ', parL%C_dust07
write(gl%nc_p,'(A,E15.8)')  'C_dust08:                ', parL%C_dust08
write(gl%nc_p,'(A,E15.8)')  'C_dust09:                ', parL%C_dust09
write(gl%nc_p,'(A,E15.8)')  'C_dust10:                ', parL%C_dust10
!
! --- Radiative transfer control ---
write(gl%nc_p,'(A)')        'Radiation trans. Scheme: ' // trim(parL%radScheme)
write(gl%nc_p,'(A,I15)')    'Rad. iterations (min):   ', parL%N_radminiter
write(gl%nc_p,'(A,I15)')    'Radiation iterations:    ', parL%N_raditer
write(gl%nc_p,'(A,I15)')    'Rad. iterations (max):   ', parL%N_radmaxiter
write(gl%nc_p,'(A,I15)')    'T step grid refinement f:', parL%N_radtaurefine
write(gl%nc_p,'(A)')        'Thin Rad. Heat Scheme:   ' // trim(parL%radThinHeatScheme)
write(gl%nc_p,'(A)')        'Thin Rad. Press. Scheme: ' // trim(parL%radThinPressScheme)
write(gl%nc_p,'(A,I15)')    'N_radThinPoint:          ', parL%N_radThinPoint
write(gl%nc_p,'(A,I15)')    'N_radThinHeatPoint:      ', parL%N_radThinHeatPoint
write(gl%nc_p,'(A,I15)')    'N_radThinPressPoint:     ', parL%N_radThinPressPoint
write(gl%nc_p,'(A,I15)')    'N_radThickPoint:         ', parL%N_radThickPoint
write(gl%nc_p,'(A,I15)')    'Number of theta angles:  ', parL%N_radtheta
write(gl%nc_p,'(A,I15)')    'Number of phi angles:    ', parL%N_radphi
write(gl%nc_p,'(A,I15)')    'Number of sub rays:      ', parL%N_radsubray
write(gl%nc_p,'(A,I15)')    'Number of bands:         ', parL%N_radband
write(gl%nc_p,'(A,I15)')    'Zero point of ray system:', parL%N_radrsyslevel
write(gl%nc_p,'(A,I15)')    'Additional MSrad output: ', parL%N_radoutput
write(gl%nc_p,'(A,I15)')    'MSrad Feautrier scheme:  ', parL%N_radFeautrier
write(gl%nc_p,'(A,E15.8)')  'Implicitness parameter:  ', parL%C_radimplicitmu
write(gl%nc_p,'(A,E15.8)')  'Rad. iteration accuracy: ', parL%C_raditereps
write(gl%nc_p,'(A,E15.8)')  'Rad. iteration step:     ', parL%C_raditerstep
write(gl%nc_p,'(A,E15.8)')  'Opacity smoothing param.:', parL%C_radkappasmooth
write(gl%nc_p,'(A,E15.8)')  'Temperature interpol. p.:', parL%C_radTstep
write(gl%nc_p,'(A,E15.8)')  'Temp. smoothing param.:  ', parL%C_radTsmooth
write(gl%nc_p,'(A,E15.8)')  'Temp. interpol. param.:  ', parL%C_radTintminfac
write(gl%nc_p,'(A,E15.8)')  'Opt. depth sc. hei., top:', parL%C_radHtautop
write(gl%nc_p,'(A,E15.8)')  'Opt. depth limit f. Tvis:', parL%C_radTvisdtau
write(gl%nc_p,'(A,E15.8)')  'Value of Tvis:           ', parL%C_radTvis
write(gl%nc_p,'(A,E15.8)')  'Time scale red. fac.:    ', parL%C_radthintimefac
write(gl%nc_p,'(A,E15.8)')  'C_radthintimeRelax:      ', parL%C_radthintimeRelax
write(gl%nc_p,'(A,E15.8)')  'C_radthinTFactor:        ', parL%C_radthinTFactor
write(gl%nc_p,'(A,E15.8)')  'P1 MSrad surface cooling:', parL%C_radtcool
write(gl%nc_p,'(A,E15.8)')  'P2 MSrad surface cooling:', parL%C_radscool
write(gl%nc_p,'(A,E15.8)')  'P3 MSrad surface cooling:', parL%C_raddcool
write(gl%nc_p,'(A,E15.8)')  'Temp. of incident rad.:  ', parL%C_radTinci
write(gl%nc_p,'(A,E15.8)')  'Dil. factor incid. rad.: ', parL%C_radDinci
write(gl%nc_p,'(A,E15.8)')  'C_radAccEnergyFraction:  ', parL%C_radAccEnergyFraction
write(gl%nc_p,'(A)')        'Base asis system:  ' // trim(parL%radraybase)
write(gl%nc_p,'(A)')        'Ray directions:    ' // trim(parL%radraystar)
write(gl%nc_p,'(A)')        'Radiation pressure:' // trim(parL%radpressure)
!
! --- Parameters for artificial tensor viscosity ---
write(gl%nc_p,'(A,E15.8)')  'C_visLinear:             ', parL%C_visLinear
write(gl%nc_p,'(A,E15.8)')  'C_visSmagorinsky:        ', parL%C_visSmagorinsky
write(gl%nc_p,'(A,E15.8)')  'C_visArtificial:         ', parL%C_visArtificial
write(gl%nc_p,'(A,E15.8)')  'C_visExpansion:          ', parL%C_visExpansion
write(gl%nc_p,'(A,E15.8)')  'C_visPrturb:             ', parL%C_visPrturb
write(gl%nc_p,'(A,E15.8)')  'C_visTensordiag:         ', parL%C_visTensordiag
write(gl%nc_p,'(A,E15.8)')  'C_visTensoroff:          ', parL%C_visTensoroff
write(gl%nc_p,'(A,E15.8)')  'C_visTensordiv:          ', parL%C_visTensordiv
write(gl%nc_p,'(A,E15.8)')  'C_visP2Pcoeff:           ', parL%C_visP2Pcoeff
write(gl%nc_p,'(A,E15.8)')  'C_visP2Pincl1:           ', parL%C_visP2Pincl1
write(gl%nc_p,'(A,E15.8)')  'C_visP2Pincl2:           ', parL%C_visP2Pincl2
write(gl%nc_p,'(A,E15.8)')  'C_visP2PhypSmagorinsky:  ', parL%C_visP2PhypSmagorinsky
write(gl%nc_p,'(A,E15.8)')  'C_visP2PhypArtificial:   ', parL%C_visP2PhypArtificial
write(gl%nc_p,'(A,E15.8)')  'C_visP2PhypExpansion:    ', parL%C_visP2PhypExpansion 
write(gl%nc_p,'(A,E15.8)')  'C_visP2Pdivrhov:         ', parL%C_visP2Pdivrhov
write(gl%nc_p,'(A,I15)')    'N_visIter:               ', parL%N_visIter
write(gl%nc_p,'(A,I15)')    'N_visP2PhypSmooth:       ', parL%N_visP2PhypSmooth
write(gl%nc_p,'(A)')        'visP2Phypreconstruction: ' // trim(parL%visP2Phypreconstruction)
!
! --- Timing information ---
write(gl%nc_p,'(A,E15.8)')  'Start time:              ', parL%starttime
write(gl%nc_p,'(A,E15.8)')  'Simulation time:         ', parL%endtime
write(gl%nc_p,'(A,E15.8)')  'Advance time:            ', parL%plustime
write(gl%nc_p,'(A,E15.8)')  'Scale time Start:        ', parL%time_ScaleStart
write(gl%nc_p,'(A,E15.8)')  'Scale time End:          ', parL%time_ScaleEnd
write(gl%nc_p,'(A,E15.8)')  'CPU time:                ', parL%CPUtime
write(gl%nc_p,'(A,E15.8)')  'CPU time (remain):       ', parL%CPUtime_remainlimit
write(gl%nc_p,'(A,I15)')    'Start step number:       ', parL%starttimestep
write(gl%nc_p,'(A,I15)')    'End time step number:    ', parL%endtimestep
write(gl%nc_p,'(A,I15)')    'Advance time step number:', parL%plustimestep
!
! --- Numerical constants ---
write(gl%nc_p,'(A,E15.8)')  'minimum time step:       ', parL%dtime_min
write(gl%nc_p,'(A,E15.8)')  'maximum time step:       ', parL%dtime_max
write(gl%nc_p,'(A,E15.8)')  'absolute min. time step: ', parL%dtime_min_stop
write(gl%nc_p,'(A,E15.8)')  'maximum time step factor:', parL%dtime_incmax
write(gl%nc_p,'(A,E15.8)')  'initial time step:       ', parL%dtime_start
write(gl%nc_p,'(A,E15.8)')  'HD Courant factor:       ', parL%C_Courant
write(gl%nc_p,'(A,E15.8)')  'max. HD Courant factor:  ', parL%C_CourantMax
write(gl%nc_p,'(A,E15.8)')  'max. HD energy change:   ', parL%C_maxeiChange
write(gl%nc_p,'(A,E15.8)')  'HD Exp. Courant factor:  ', parL%C_hydExpCourant
write(gl%nc_p,'(A,E15.8)')  'max. HD Exp. Courant fac:', parL%C_hydExpCourantMax
write(gl%nc_p,'(A,E15.8)')  'HD Snd. Courant factor:  ', parL%C_hydSoundCourant
write(gl%nc_p,'(A,E15.8)')  'max. HD Snd. Courant fac:', parL%C_hydSoundCourantMax
write(gl%nc_p,'(A,E15.8)')  'RAD Courant factor:      ', parL%C_radCourant
write(gl%nc_p,'(A,E15.8)')  'max. RAD Courant factor: ', parL%C_radCourantMax
write(gl%nc_p,'(A,E15.8)')  'max. RAD energy change:  ', parL%C_radmaxeiChange
write(gl%nc_p,'(A,E15.8)')  'VIS Courant factor:      ', parL%C_visCourant
write(gl%nc_p,'(A,E15.8)')  'max. VIS Courant factor: ', parL%C_visCourantMax
write(gl%nc_p,'(A,E15.8)')  'RES Courant factor:      ', parL%C_resCourant
write(gl%nc_p,'(A,E15.8)')  'max. RES Courant factor: ', parL%C_resCourantMax
!
! --- Test parameters ---
write(gl%nc_p,'(A,I15)')    'N_test1:                 ', parL%N_test1
write(gl%nc_p,'(A,I15)')    'N_test2:                 ', parL%N_test2
write(gl%nc_p,'(A,I15)')    'N_test3:                 ', parL%N_test3
write(gl%nc_p,'(A,I15)')    'N_test4:                 ', parL%N_test4
write(gl%nc_p,'(A,I15)')    'N_test5:                 ', parL%N_test5
write(gl%nc_p,'(A,E15.8)')  'C_test1:                 ', parL%C_test1
write(gl%nc_p,'(A,E15.8)')  'C_test2:                 ', parL%C_test2
write(gl%nc_p,'(A,E15.8)')  'C_test3:                 ', parL%C_test3
write(gl%nc_p,'(A,E15.8)')  'C_test4:                 ', parL%C_test4
write(gl%nc_p,'(A,E15.8)')  'C_test5:                 ', parL%C_test5
write(gl%nc_p,'(A)')        'Test string (1):   ' // trim(parL%strtest1)
write(gl%nc_p,'(A)')        'Test string (2):   ' // trim(parL%strtest2)
write(gl%nc_p,'(A)')        'Test string (3):   ' // trim(parL%strtest3)
write(gl%nc_p,'(A)')        'Test string (4):   ' // trim(parL%strtest4)
write(gl%nc_p,'(A)')        'Test string (5):   ' // trim(parL%strtest5)
!
! --- I/O files ---
write(gl%nc_p,'(A,I15)')    'N_outslicedim_mean:', parL%N_outslicedim_mean
write(gl%nc_p,'(A)')        'Start model file:  ' // trim(parL%infile_start)
write(gl%nc_p,'(A)')        'End model file:    ' // trim(parL%OutFile_end)
!
write(gl%nc_p,'(A)') repeat('-',79)
!
end subroutine rhd_par_PrintInfo


!----------*************----------------------------------------------------------------------------
subroutine rhd_par_Write(nc, file, form, conv, parl)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_par_Write ('rhd_parameters_Write')
!
! PURPOSE:
!   Write RHD parameter into file in UIO form.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_par_Write(nc=nc, file=file, form=form, conv=conv)
!
! INPUT:
!   nc:           ('number_channel') integer, optional, number of channel unit
!   file:         ('file_name') character, optional, name of output file;
!                 either nc or file have to be specified.
!   form:         ('format') character, optional, format of output file
!   conv:         ('conversion') character, optional, conversion type of output file
!   parl:         ('parameter_local') structure with control parameters
!
! OUTPUT:
!   None (write only to file)
!
! ROUTINES:
!   uio_closwr:            Close file after writing
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!   UIO routines have to be initialized.
!
! PROCEDURE:
!   This routine writes the RHD parameters into a file in UIO form.
!   If the input parameter nc is present, it is assumed that the corresponding
!   channel is opened for output in UIO form.
!   If instead the input parameter file is present a file with this name is
!   opened with format and conversion type corresponding to form and conv,
!   respectively.
!
! EXAMPLE:
!
! TODO:
!   Write the values of all parameters.
!   Write label statements.
!
! MODIFICATION HISTORY:
!   1997-06-13 (B.F. Kiel) Written
!   1997-07-24 (B.F.) Parameters added
!   2006-08-07 (SWB)  hion_datapath, hion_atomfile, hion_abufile, hion_edensfile, 
!                     hion_pffile, dtime_out_hion, hion_chunks
!   2008-06-10 (W.S. Freiburg) Magnetic boundary conditions
!   2010-05-10 (B.F. Lyon) Rename rhd_wrpar_par -> rhd_par_Write
!   2015-04-25 (B.F. Uppsala) Make control structure input parameter
!---------------------------------------------------------------------------------------------------
use uio_bulk_module
use const_module
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,         optional, intent(in)   :: nc
character(len=*),optional, intent(in)   :: file, form, conv
type(par_type),            intent(in)   :: parl
!
! --- Local variables ---
integer                                 :: ncout
!---------------------------------------------------------------------------------------------------
!
if (present(nc)) then
  ncout=nc
else
  ! --- Open file in uio-form ---
  call uio_openwr(ncout, file, form=form, conv=conv)
endif
!
! --- Write file_id ---
call uio_wr(ncout, 'rhd-parameter', 'file_id', name='File identification')
!
! --- Write file description ---
call uio_wr(ncout, parL%description(1:parL%ndes), 'description', name='File description')
!
! --- Write file history ---
call uio_wr(ncout, parL%history(1:parL%nhis), 'history', name='File history')
!
! --- Write effective temperature ---
call uio_wr(ncout, parL%Teff, 'teff', name='Effective temperature', unit=T_unit)
call uio_wr(ncout, parL%heat_mode, 'heat_mode', name='Heating mode'                         )
!
! --- Write gravity ---
call uio_wr(ncout, parL%grav_mode, 'grav_mode', name='Gravity mode'                         )
call uio_wr(ncout, parL%grav_type, 'grav_type', name='Gravity type'                         )
call uio_wr(ncout, parL%grav     , 'grav'     , name='Gravity'              , unit=grav_unit)
call uio_wr(ncout, parL%mass_star, 'mass_star', name='Star Mass'            , unit=mass_unit)
call uio_wr(ncout, parL%C_gravmu , 'c_gravmu' , name='Gravity time centering', unit='1')
call uio_wr(ncout, parL%C_gravCorePotDepth , 'c_gravcorepotdepth', &
                                             name='Depth factor of core potential', unit='1')
call uio_wr(ncout, parL%C_gravCorePotQuad , 'c_gravcorepotquad' , &
                                             name='Amplitude of quadratic terms in core', unit='1')
call uio_wr(ncout, parL%r0_grav  , 'r0_grav'  , name='Inner gravity smoothing radius' &
                                                                           , unit=x1_unit  )
call uio_wr(ncout, parL%r1_grav  , 'r1_grav'  , name='Outer gravity smoothing radius' &
                                                                           , unit=x1_unit  )
call uio_wr(ncout, parL%r2_grav  , 'r2_grav', name='Outer gravity smoothing radius (old potential)'&
                                                                           , unit=x1_unit  )
call uio_wr(ncout, parL%r1_rad   , 'r1_rad'   , name='Radiation cutting radius' &
                                                                           , unit=x1_unit  )
call uio_wr(ncout, parL%C_gravCorer0Heat, 'c_gravcorer0heat', name='Core radius fraction', unit='1')
call uio_wr(ncout, parL%centrifugal_force, 'centrifugal_force', name='Centrifugal force'    )
call uio_wr(ncout, parL%nu_rotation , 'nu_rotation', name='Radiation cutting radius' &
                                                                           , unit=frequency_unit)
call uio_wr(ncout, parL%ar_RotationAxis, 'ar_rotationaxis', name='Rotation axis' )
!
! --- Write abundances ---
call uio_wr(ncout, parL%abuX , 'abux' , name='Hydrogen abundance',    unit='1'   )
call uio_wr(ncout, parL%abuY , 'abuy' , name='Helium abundance',      unit='1'   )
call uio_wr(ncout, parL%abuZ , 'abuz' , name='Metal abundance',       unit='1'   )
call uio_wr(ncout, parL%Qmol , 'qmol' , name='Mean molecular weight', unit=u_unit)
call uio_wr(ncout, parL%gamma, 'gamma', name='Adiabatic coefficient', unit='1'   )
!
! --- Write EOS file name and path ---
call uio_wr(ncout, parL%EOSFile, 'eosfile', name='EOS file name')
call uio_wr(ncout, parL%EOSPath, 'eospath', name='EOS path name')
!
! --- Write opacity file name and path ---
call uio_wr(ncout, parL%kappaMin, 'kappamin', name='Minimum opacity'  , unit=kappa_unit)
call uio_wr(ncout, parL%kappaMax, 'kappamax', name='Maximum opacity'  , unit=kappa_unit)
call uio_wr(ncout, parL%OpaFile , 'opafile' , name='Opacity file name')
call uio_wr(ncout, parL%OpaPath , 'opapath' , name='Opacity path name')
!
! --- Write chemical reaction file name and path ---
call uio_wr(ncout, parL%chem_abumetal, 'chem_abumetal' , name='Chemistry metal abundance', unit='1')
call uio_wr(ncout, parL%chem_reacfile, 'chem_reacfile' , name='Chemical reaction network file name')
call uio_wr(ncout, parL%chem_reacpath, 'chem_reacpath' , name='Chemical reaction network path name')
!
! --- Write HION parameters ---
call uio_wr(ncout, parL%hion_datapath,  'hion_datapath',  name='HION data path')
call uio_wr(ncout, parL%hion_atomfile,  'hion_atomfile',  name='HION atom file name')
call uio_wr(ncout, parL%hion_abufile,   'hion_abufile',   name='HION abundance file name')
call uio_wr(ncout, parL%hion_edensfile, 'hion_edensfile', name='HION electron density file name')
call uio_wr(ncout, parL%hion_pffile,    'hion_pffile',    name='HION partition function file name')
call uio_wr(ncout, parL%dtime_out_hion, 'dtime_out_hion', name='Output file time step (HION)', &
                                                         unit=time_unit)
call uio_wr(ncout, parL%hion_chunks,    'hion_chunks',    name='Number of HION chunks')
!
! --- Write boundary conditions ---
call uio_wr(ncout, parL%side_bound,    'side_bound',   name='Side boundaries')
call uio_wr(ncout, parL%top_bound,     'top_bound',    name='Top boundaries')
call uio_wr(ncout, parL%bottom_bound,  'bottom_bound', name='Bottom boundaries')
call uio_wr(ncout, parL%side_bound_mag_x1, 'side_bound_mag_x1', &
                                                  name='Side boundaries magnetic field x direction')
call uio_wr(ncout, parL%side_bound_mag_x2, 'side_bound_mag_x2', &
                                                  name='Side boundaries magnetic field y direction')
call uio_wr(ncout, parL%top_bound_mag,  'top_bound_mag',       name='Top boundaries magnetic field')
call uio_wr(ncout, parL%bottom_bound_mag, 'bottom_bound_mag', &
                                                 name='Bottom boundaries magnetic field')
call uio_wr(ncout, parL%s_InFlow , 's_inflow',   name='Entropy of inflowing material' , unit=s_unit)
call uio_wr(ncout, parL%s_InFlRadGrad, 's_inflradgrad', name='Radial entropy gradient in core', &
                                                       unit=s_unit)
call uio_wr(ncout, parL%s_InFlLatGrad, 's_infllatgrad', name='Lateral entropy gradient in core', &
                                                       unit=s_unit)
call uio_wr(ncout, parL%C_sChange, 'c_schange',  name='Rate of entropy change at bottom', unit='1')
call uio_wr(ncout, parL%C_PChange, 'c_pchange',  name='Rate of pressure change at bottom', unit='1')
call uio_wr(ncout, parL%C_v3ChangeLinBottom, 'c_v3changelinbottom', &
                                 name='Rate of linear vertical velocity change at bottom', unit='1')
call uio_wr(ncout, parL%C_v3ChangeSqrBottom, 'c_v3changesqrbottom', &
                              name='Rate of quadratic vertical velocity change at bottom', unit='1')
call uio_wr(ncout, parL%C_TChange, 'c_tchange',  name='Rate of temperature change at top', unit='1')
call uio_wr(ncout, parL%C_rhoChangeTop, 'c_rhochangetop', name='Rate of density change at top', &
                                                                                           unit='1')
call uio_wr(ncout, parL%C_Tsurf  , 'c_tsurf',     name='Surface temperature factor'      , unit='1')
call uio_wr(ncout, parL%C_Hptopfactor, 'c_hptopfactor', &
                               name='Correction factor for surface pressure scale height', unit='1')
call uio_wr(ncout, parL%rho_min  , 'rho_min',    name='Minimum boundary density'    , unit=rho_unit)
call uio_wr(ncout, parL%LuminosityPerVolume, 'luminositypervolume', &
                                         name='Luminosity per core volume'  , unit=luminpervol_unit)
call uio_wr(ncout, parL%hdCoreHeatProfile, 'hdcoreheatprofile', &
                                                           name='Radial profile of core heating'   )
call uio_wr(ncout, parL%hdCoreDragProfile, 'hdcoredragprofile', &
                                                           name='Radial profile of core drag force')
call uio_wr(ncout, parL%C_CoreDrag , 'C_CoreDrag',   name='Core drag parameter'  ,         unit='1')
call uio_wr(ncout, parL%B1_inflow, 'b1_inflow', &
                                            name='Inflowing horizontal magnetic field', unit=B_unit)
call uio_wr(ncout, parL%C_magthetaB, 'c_magthetab',       name='theta of magnetic field',  unit='1')
call uio_wr(ncout, parL%C_magphiB,    'c_magphib',           name='phi of magnetic field', unit='1')
call uio_wr(ncout, parL%N_magDiffSide, 'n_magdiffside', &
                                                   name='Number of cells for diffusive side layers')
call uio_wr(ncout, parL%N_magDiffTop,  'n_magdifftop' &
                                                   , name='Number of cells for diffusive top layer')
call uio_wr(ncout, parL%N_magDiffBottom, 'n_magdiffbottom', &
                                                  name='Number of cells for diffusive bottom layer')
!
! --- Write (magneto-)hydrodynamics parameters ---
call uio_wr(ncout, parL%hdScheme,     'hdscheme'        , name='Hydrodynamics scheme' )
call uio_wr(ncout, parL%hdTimeIntegrationScheme, 'hdtimeintegrationscheme', &
                                                         name='MHD time-integration scheme' )
call uio_wr(ncout, parL%hdsplit,      'hdsplit'        , name='Hydro directional splitting scheme' )
call uio_wr(ncout, parL%hdTmpStructKeepMode, 'hdtmpstructkeepmode', &
                                                     name='Retaining mode for temporary strucutes' )
call uio_wr(ncout, parL%N_hydIter,    'n_hyditer'       , name='Number of hydro iterations')
call uio_wr(ncout, parL%N_hydMaxIter, 'n_hydmaxiter',     name='Maximum number of hydro iterations')
call uio_wr(ncout, parL%N_hydRKIter, 'n_hydrkiter',     name='Number of Runge-Kutta sub steps')
call uio_wr(ncout, parL%C_hydTDiffLin    , 'c_hydtdifflin', &
                    name='Parameter for energy diffusion near local temperature minimum', unit='1')
call uio_wr(ncout, parL%C_hydTDiffMach    , 'c_hydtdiffmach', &
                    name='Parameter for energy diffusion near local temperature minimum', unit='1')
call uio_wr(ncout, parL%C_hydsDiffVelo    , 'c_hydsdiffvelo', &
                    name='Parameter for energy diffusion down an entropy step', unit='1')
call uio_wr(ncout, parL%C_hydvDiffVelo    , 'c_hydvdiffvelo', &
                    name='Parameter for momentum diffusion down a transversal velocity step', &
                                                                                           unit='1')
call uio_wr(ncout, parL%C_hydLowMachcsFactor, 'c_hydlowmachcsfactor', &
                    name='Parameter for low-Mach-number sound-speed reduction',            unit='1')
call uio_wr(ncout, parL%C_hydLowMachcsBase, 'c_hydlowmachcsbase', &
                    name='Base value for low-Mach-number sound-speed reduction',           unit='1')
call uio_wr(ncout, parL%C_hydSmallFluctFactor, 'c_hydsmallfluctfactor', &
                    name='Parameter for small/large-fluctation estimate',                  unit='1')
call uio_wr(ncout, parL%N_hydCellsPerChunk, 'n_hydcellsperchunk', &
                                                             name='Number of cells per hydro chunk')
call uio_wr(ncout, parL%N_hydCellsPerChunk2, 'n_hydcellsperchunk2', &
                                             name='Number of cells per hydro chunk in x2 direction')
call uio_wr(ncout, parL%N_hydCellsPerChunk3, 'n_hydcellsperchunk3', &
                                             name='Number of cells per hydro chunk in x3 direction')
call uio_wr(ncout, parL%N_visCellsPerChunk, 'n_viscellsperchunk', &
                                                         name='Number of cells per viscosity chunk')
call uio_wr(ncout, parL%N_resCellsPerChunk, 'n_rescellsperchunk', &
                                                       name='Number of cells per resistivity chunk')
call uio_wr(ncout, parL%N_radCellsPerChunk, 'n_radcellsperchunk', &
                                               name='Number of cells per radiation transport chunk')
call uio_wr(ncout, parL%aN_MPIProcsPerDim, 'an_mpiprocsperdim', &
                                                       name='Number of MPI processes per dimension')
call uio_wr(ncout, parL%hdCheckFlux, 'hdcheckflux', name='Activation of CheckFlux')
call uio_wr(ncout, parL%reconstruction, 'reconstruction', name='Reconstruction method')
call uio_wr(ncout, parL%hdTransVeloMode, 'hdtransvelomode', &
                                                       name='Mode of transverse velocity advection')
call uio_wr(ncout, parL%hdEntropyWaveMode, 'hdentropywavemode', &
                                                       name='Mode of entropy wave treatment')
call uio_wr(ncout, parL%hdEnthalpyAvgMode, 'hdenthalpyavgmode', &
                                                       name='Mode of enthalpy averaging')
call uio_wr(ncout, parL%C_SlopeRed    , 'c_slopered', &
                    name='Slope reduction parameter in case of strong density contrast', unit='1')
call uio_wr(ncout, parL%C_RecContShift    , 'c_reccontshift', &
                    name='Enhanced reconstruction continuity 1 for shifting', unit='1')
call uio_wr(ncout, parL%C_RecContSteep    , 'c_reccontsteep', &
                    name='Enhanced reconstruction continuity 2 for steepening', unit='1')
call uio_wr(ncout, parL%C_RecWenoWeightCenter, 'c_recwenoweightcenter', &
                    name='Weight of central polynomial', unit='1')
call uio_wr(ncout, parL%C_RecWenoWeightRight , 'c_recwenoweightright', &
                    name='Weight factor of right polynomial', unit='1')
call uio_wr(ncout, parL%C_RecWenoPsiFactor   , 'c_recwenopsifactor', &
                    name='Smoothing factor in case of bad pairwise smoothness match', unit='1')
call uio_wr(ncout, parL%C_hydPredfactor   , 'c_hydpredfactor', &
                    name='Hydrostatic pressure reduction factor', unit='1')
call uio_wr(ncout, parL%C_visBound    , 'c_visbound',    name='Boundary drag viscosity parameter', &
                                                                                         unit='1')
call uio_wr(ncout, parL%beta_inv  , 'beta_inv', &
                    name='inverse plasma beta for dual energy method', unit='1')
call uio_wr(ncout, parL%va_max  , 'va_max', &
                    name='maximum Alfven speed', unit=v_unit)
call uio_wr(ncout, parL%N_orderConstrainedTransport, 'N_orderconstrainedtransport', &
                                                         name='Order of Constrained Transport')
call uio_wr(ncout, parL%C_resB, 'c_resB', &
            name='Resistivity parameter', unit='1')
call uio_wr(ncout, parL%C_resBconstant, 'c_resBconstant', &
            name='Resistivity parameter for constant resistivity', unit='cm^2/s')
call uio_wr(ncout, parL%C_resEpsilon, 'c_resepsilon', &
            name='Parameter for additional thermal energy diffusion', unit='1')
!
!
! --- Write dust parameters ---
call uio_wr(ncout, parL%dustScheme    , 'dustscheme'    , name='Dust formation scheme')
call uio_wr(ncout, parL%dustReconstruction, 'dustreconstruction', name='Dust reconstruction scheme')
call uio_wr(ncout, parL%dustOpaFile   , 'dustopafile'   , name='Dust opacity file name')
call uio_wr(ncout, parL%dustopamode   , 'dustopamode'   , name='Dust opacity mode')
call uio_wr(ncout, parL%ar_dustGrainRadius, 'ar_dustgrainradius', name='Dust grain radii',unit='cm')
call uio_wr(ncout, parL%ar_dustGrainAbu   , 'ar_dustgrainabu'   , name='Dust grain abundances')
call uio_wr(ncout, parL%C_dust01      , 'c_dust01', name='Dust parameter 01')
call uio_wr(ncout, parL%C_dust02      , 'c_dust02', name='Dust parameter 02')
call uio_wr(ncout, parL%C_dust03      , 'c_dust03', name='Dust parameter 03')
call uio_wr(ncout, parL%C_dust04      , 'c_dust04', name='Dust parameter 04')
call uio_wr(ncout, parL%C_dust05      , 'c_dust05', name='Dust parameter 05')
call uio_wr(ncout, parL%C_dust06      , 'c_dust06', name='Dust parameter 06')
call uio_wr(ncout, parL%C_dust07      , 'c_dust07', name='Dust parameter 07')
call uio_wr(ncout, parL%C_dust08      , 'c_dust08', name='Dust parameter 08')
call uio_wr(ncout, parL%C_dust09      , 'c_dust09', name='Dust parameter 09')
call uio_wr(ncout, parL%C_dust10      , 'c_dust10', name='Dust parameter 10')
!
! --- Write timing information ---
call uio_wr(ncout, parL%starttime,           'starttime', name='Start time',        unit=time_unit)
call uio_wr(ncout, parL%endtime,             'endtime', &
                                            name='total simulation time limit',     unit=time_unit)
call uio_wr(ncout, parL%plustime,            'plustime', &
                                            name='simulation advance time',         unit=time_unit)
call uio_wr(ncout, parL%time_ScaleStart,     'time_scalestart', &
                                            name='Start time of scaling interval',  unit=time_unit)
call uio_wr(ncout, parL%time_ScaleEnd,       'time_scaleend', &
                                            name='End time of scaling interval'  ,  unit=time_unit)
call uio_wr(ncout, parL%CPUtime,             'cputime', name='CPU time',                unit='s')
call uio_wr(ncout, parL%CPUtime_remainlimit, 'cputime_remainlimit', &
                                            name='maximum remaining CPU time',          unit='s')
call uio_wr(ncout, parL%starttimestep,       'starttimestep', &
                                            name='Start time step number',              unit='1')
call uio_wr(ncout, parL%endtimestep,         'endtimestep', &
                                            name='total simulation time step number',   unit='1')
call uio_wr(ncout, parL%plustimestep,        'plustimestep', &
                                            name='simulation advance time step number', unit='1')
!
! --- Write numerical timestep constants ---
call uio_wr(ncout, parL%dtime_min, 'dtime_min', name='minimum time step', unit=time_unit)
call uio_wr(ncout, parL%dtime_max, 'dtime_max', name='maximum time step', unit=time_unit)
call uio_wr(ncout, parL%dtime_min_stop, 'dtime_min_stop', name='absolute minimum time step', &
                                                                                unit=time_unit)
call uio_wr(ncout, parL%dtime_incmax, 'dtime_incmax', name='maximum time step factor', unit='1')
call uio_wr(ncout, parL%dtime_start, 'dtime_start', name='initial time step',    unit=time_unit)
call uio_wr(ncout, parL%C_Courant, 'c_courant', name='Courant factor',                 unit='1')
call uio_wr(ncout, parL%C_CourantMax, 'c_courantmax', &
            name='maximum Courant factor',                                            unit='1')
call uio_wr(ncout, parL%C_maxeiChange, 'c_maxeichange', &
            name='maximum allowed change of internal energy due to hydro step',       unit='1')
call uio_wr(ncout, parL%C_hydExpCourant, 'c_hydexpcourant', &
            name='expansion Courant factor',                                          unit='1')
call uio_wr(ncout, parL%C_hydExpCourantMax, 'c_hydexpcourantmax', &
            name='maximum expansion Courant factor',                                  unit='1')
call uio_wr(ncout, parL%C_hydSoundCourant, 'c_hydsoundcourant', &
            name='sound-speed Courant factor',                                        unit='1')
call uio_wr(ncout, parL%C_hydSoundCourantMax, 'c_hydsoundcourantmax', &
            name='maximum sound-speed Courant factor',                                unit='1')
call uio_wr(ncout, parL%C_radCourant, 'c_radcourant', name='radiative Courant factor', unit='1')
call uio_wr(ncout, parL%C_radCourantMax, 'c_radcourantmax', &
            name='maximum radiative Courant factor',                                  unit='1')
call uio_wr(ncout, parL%C_radmaxeiChange, 'c_radmaxeichange', &
            name='maximum allowed change of internal energy due to radiation',        unit='1')
call uio_wr(ncout, parL%C_visCourant, 'c_viscourant', name='viscous Courant factor',   unit='1')
call uio_wr(ncout, parL%C_visCourantMax, 'c_viscourantmax', &
            name='maximum viscous Courant factor',                                    unit='1')
call uio_wr(ncout, parL%C_resCourant, 'c_rescourant', name='resistive Courant factor', unit='1')
call uio_wr(ncout, parL%C_resCourantMax, 'c_rescourantmax', &
            name='maximum resistive Courant factor',                                  unit='1')
!
! --- Write radiation-transport parameters ---
call uio_wr(ncout, parL%radScheme   , 'radscheme'      , name='Radiation transport scheme' )
call uio_wr(ncout, parL%N_radminiter, 'n_radminiter', &
            name='Minimum number of radiation transport iterations')
call uio_wr(ncout, parL%N_raditer, 'n_raditer', &
            name='Number of radiation transport iterations')
call uio_wr(ncout, parL%N_radmaxiter, 'n_radmaxiter', &
            name='Maximum number of radiation transport iterations')
call uio_wr(ncout, parL%N_radtaurefine, 'n_radtaurefine', &
            name='Grid refinement factor near large temperature steps')
call uio_wr(ncout, parL%radThinHeatScheme, 'radthinheatscheme', &
                                   name='Radiation transport heating scheme in optically thin' )
call uio_wr(ncout, parL%radThinPressScheme, 'radthinpressscheme', &
                                   name='Radiation pressure scheme in optically thin' )
call uio_wr(ncout, parL%N_radThinPoint, 'n_radthinpoint', &
            name='Number of grid points with optically thin approximation')
call uio_wr(ncout, parL%N_radThinHeatPoint, 'n_radthinheatpoint', &
            name='Number of transition grid points with optically thin heating')
call uio_wr(ncout, parL%N_radThinPressPoint, 'n_radthinpresspoint', &
            name='Number of transition grid points with optically thin rad. acceleration')
call uio_wr(ncout, parL%N_radThickPoint, 'n_radthickpoint', &
            name='Number of grid points with optically thick (diffusion) approximation')
call uio_wr(ncout, parL%N_radtheta, 'n_radtheta', &
            name='Number of ray sets in theta direction')
call uio_wr(ncout, parL%N_radphi, 'n_radphi', &
            name='Number of ray sets in phi direction')
call uio_wr(ncout, parL%N_radsubray, 'n_radsubray', &
            name='Number of rays per cell per direction')
call uio_wr(ncout, parL%N_radband, 'n_radband', &
            name='Number of frequency bands')
call uio_wr(ncout, parL%N_radrsyslevel, 'n_radrsyslevel', &
            name='Zero point of ray system')
call uio_wr(ncout, parL%N_radoutput, 'n_radoutput', &
            name='Additional output for radiation transport')
call uio_wr(ncout, parL%N_radFeautrier, 'n_radfeautrier', &
            name='MSrad Feautrier scheme')
call uio_wr(ncout, parL%C_radimplicitmu, 'c_radimplicitmu', &
            name='Implicitness parameter for radiation transport', unit='1')
call uio_wr(ncout, parL%C_raditereps, 'c_raditereps', &
            name='Relative accuracy for radiation iteration', unit='1')
call uio_wr(ncout, parL%C_raditerstep, 'c_raditerstep', &
            name='Step size of radiation iteration', unit='1')
call uio_wr(ncout, parL%C_radkappasmooth, 'c_radkappasmooth', &
            name='Opacity smoothing parameter', unit='1')
call uio_wr(ncout, parL%C_radTstep, 'c_radtstep', &
            name='Temperature interpolation parameter', unit='1')
call uio_wr(ncout, parL%C_radTsmooth, 'c_radtsmooth', &
            name='Temperature smoothing parameter', unit='1')
call uio_wr(ncout, parL%C_radTintminfac, 'c_radtintminfac', &
            name='Temperature interpolation parameter', unit='1')
call uio_wr(ncout, parL%C_radHtautop, 'c_radhtautop', &
            name='Scale height of optical depth at top', unit=x1_unit)
call uio_wr(ncout, parL%C_radTvisdtau, 'c_radtvisdtau', &
            name='Optical depth limit for temperature viscosity', unit='1')
call uio_wr(ncout, parL%C_radTvis    , 'c_radtvis', &
            name='Temperature viscosity', unit='1')
call uio_wr(ncout, parL%C_radthintimefac    , 'c_radthintimefac', &
            name='Timescale reduction factor in optically thin', unit='1')
call uio_wr(ncout, parL%C_radthintimeRelax  , 'c_radthintimerelax', &
            name='Radiative relaxation timescale in optically thin', unit=time_unit)
call uio_wr(ncout, parL%C_radthinTFactor    , 'c_radthintfactor', &
            name='Temperature-correction factor in optically thin', unit='1')
call uio_wr(ncout, parL%C_radtcool    , 'c_radtcool', &
            name='Surface cooling parameter')
call uio_wr(ncout, parL%C_radscool    , 'c_radscool', &
            name='Surface cooling parameter')
call uio_wr(ncout, parL%C_raddcool    , 'c_raddcool', &
            name='Surface cooling parameter')
call uio_wr(ncout, parL%C_radTinci    , 'c_radtinci', &
            name='Temperature of black body that sends incident radiation')
call uio_wr(ncout, parL%C_radDinci    , 'c_raddinci', &
            name='Dilution factor of black body that sends incident radiation')
call uio_wr(ncout, parL%C_radAccEnergyFraction, 'c_radaccenergyfraction', &
            name='Fraction of accumulated radiative energy change', unit='1')
call uio_wr(ncout, parL%radraybase , 'radraybase'    , name='Base axis system')
call uio_wr(ncout, parL%radraystar , 'radraystar'    , name='List of relative ray directions')
call uio_wr(ncout, parL%radpressure, 'radpressure'   , name='Radiation pressure mode')
!
! --- Write parameters for classical tensor viscosity ---
call uio_wr(ncout, parL%C_visLinear, 'c_vislinear', &
            name='Linear viscosity parameter', unit='1')
call uio_wr(ncout, parL%C_visSmagorinsky, 'c_vissmagorinsky', &
            name='Turbulent eddy viscosity parameter (Smagorinsky type)', unit='1')
call uio_wr(ncout, parL%C_visArtificial, 'c_visartificial', &
            name='Artificial viscosity tensor parameter', unit='1')
call uio_wr(ncout, parL%C_visExpansion, 'c_visexpansion', &
            name='Expansion viscosity tensor parameter', unit='1')
call uio_wr(ncout, parL%C_visPrturb, 'c_visprturb', &
            name='Viscous Prandtl number', unit='1')
call uio_wr(ncout, parL%C_visTensordiag, 'c_vistensordiag', &
            name='Diagonal factor for viscous stress tensor', unit='1')
call uio_wr(ncout, parL%C_visTensoroff, 'c_vistensoroff', &
            name='Off-diagonal factor for viscous stress tensor', unit='1')
call uio_wr(ncout, parL%C_visTensordiv, 'c_vistensordiv', &
            name='Divergence factor for viscous stress tensor', unit='1')
!
! --- Write parameters for point-to-point tensor viscosity ---
call uio_wr(ncout, parL%C_visP2Pcoeff, 'c_visp2pcoeff', &
            name='Viscosity coefficient for point 2 point viscosity', unit='1')
call uio_wr(ncout, parL%C_visP2Pincl1, 'c_visp2pincl1', &
            name='Factor 1 for inclined cells for point 2 point viscosity', unit='1')
call uio_wr(ncout, parL%C_visP2Pincl2, 'c_visp2pincl2', &
            name='Factor 2 for inclined cells for point 2 point viscosity', unit='1')
call uio_wr(ncout, parL%C_visP2PhypSmagorinsky, 'c_visp2phypsmagorinsky', &
            name='Smagorinsky parameter for P2P hyperviscosity', unit='1')
call uio_wr(ncout, parL%C_visP2PhypArtificial, 'c_visp2phypartificial', &
            name='Artificial parameter for P2P hyperviscosity', unit='1')
call uio_wr(ncout, parL%C_visP2PhypExpansion, 'c_visp2phypexpansion', &
            name='Expansion parameter for P2P hyperviscosity', unit='1')
call uio_wr(ncout, parL%C_visP2Pdivrhov, 'c_visp2pdivrhov', &
            name='Viscosity coefficient for point 2 point mass-flux-divergence viscosity', unit='1')
call uio_wr(ncout, parL%N_visIter , 'n_visiter' , &
            name='Number of iteration for tensor viscosity')
call uio_wr(ncout, parL%N_visP2PhypSmooth , 'n_visp2phypsmooth' , &
            name='Number of smoothing iterations for point-to-point hyperviscosity')
call uio_wr(ncout, parL%visP2Phypreconstruction, 'visp2phypreconstruction', name='')
!
! --- Write test parameters ---
call uio_wr(ncout, parL%N_test1 , 'n_test1' , name='Test integer 1')
call uio_wr(ncout, parL%N_test2 , 'n_test2' , name='Test integer 2')
call uio_wr(ncout, parL%N_test3 , 'n_test3' , name='Test integer 3')
call uio_wr(ncout, parL%N_test4 , 'n_test4' , name='Test integer 4')
call uio_wr(ncout, parL%N_test5 , 'n_test5' , name='Test integer 5')
call uio_wr(ncout, parL%C_test1 , 'c_test1' , name='Test real 1'   )
call uio_wr(ncout, parL%C_test2 , 'c_test2' , name='Test real 2'   )
call uio_wr(ncout, parL%C_test3 , 'c_test3' , name='Test real 3'   )
call uio_wr(ncout, parL%C_test4 , 'c_test4' , name='Test real 4'   )
call uio_wr(ncout, parL%C_test5 , 'c_test5' , name='Test real 5'   )
call uio_wr(ncout, parL%strtest1, 'strtest1', name='Test string 1' )
call uio_wr(ncout, parL%strtest2, 'strtest2', name='Test string 2' )
call uio_wr(ncout, parL%strtest3, 'strtest3', name='Test string 3' )
call uio_wr(ncout, parL%strtest4, 'strtest4', name='Test string 4' )
call uio_wr(ncout, parL%strtest5, 'strtest5', name='Test string 5' )
!
! --- Write I/O parameters ---
call uio_wr(ncout, parL%istep_in_start, 'istep_in_start', name='Input data set number')
call uio_wr(ncout, parL%infile_start,   'infile_start'  , name='File name of start model')
call uio_wr(ncout, parL%N_outslicedim_mean, 'n_outslicedim_mean', &
                                                         name='Plane index of output slice')
call uio_wr(ncout, parL%OutFile_end ,   'outfile_end'   , name='Output file name (final model)')
call uio_wr(ncout, parL%OutForm_end ,   'outform_end'   , name='Output file format (final model)')
call uio_wr(ncout, parL%OutConv_end ,   'outconv_end' , name='Output file conversion (final model)')
call uio_wr(ncout, parL%dtime_out_end,  'dtime_out_end',  name='Output file time step (end data)', &
                                                         unit=time_unit)
call uio_wr(ncout, parL%OutFile_full,   'outfile_full'  , name='Output file name (full data)')
call uio_wr(ncout, parL%OutForm_full,   'outform_full'  , name='Output file format (full data)')
call uio_wr(ncout, parL%OutConv_full,   'outconv_full'  , name='Output file conversion (full data)')
call uio_wr(ncout, parL%dtime_out_full, 'dtime_out_full', name='Output file time step (full data)',&
                                                         unit=time_unit)
call uio_wr(ncout, parL%OutFile_mean,   'outfile_mean'  , name='Output file name (mean data)')
call uio_wr(ncout, parL%OutForm_mean,   'outform_mean'  , name='Output file format (mean data)')
call uio_wr(ncout, parL%OutConv_mean,   'outconv_mean'  , name='Output file conversion (mean data)')
call uio_wr(ncout, parL%dtime_out_mean, 'dtime_out_mean', name='Output file time step (mean data)',&
                                                         unit=time_unit)
call uio_wr(ncout, parL%OutFile_mcyl,   'outfile_mcyl', &
                                                    name='Output file name (cylindrical mean data)')
call uio_wr(ncout, parL%OutFile_chu1,   'outfile_chu1'  , name='Output file name (chunk 1)')
call uio_wr(ncout, parL%OutDim_chu1 ,   'outdim_chu1'   , name='Dimension of output chunk 1')
call uio_wr(ncout, parL%OutFile_chu2,   'outfile_chu2'  , name='Output file name (chunk 2)')
call uio_wr(ncout, parL%OutDim_chu2 ,   'outdim_chu2'   , name='Dimension of output chunk 2')
call uio_wr(ncout, parL%OutFile_chu3,   'outfile_chu3'  , name='Output file name (chunk 3)')
call uio_wr(ncout, parL%OutDim_chu3 ,   'outdim_chu3'   , name='Dimension of output chunk 3')
!
call uio_wr(ncout, parL%OutFile_out,    'outfile_out'   , name='Output file name (text log)')
!
call uio_wr(ncout, parL%OutFile_fine,   'outfile_fine'  , name='Output file name (fine data)')
call uio_wr(ncout, parL%OutForm_fine,   'outform_fine'  , name='Output file format (fine data)')
call uio_wr(ncout, parL%OutConv_fine,   'outconv_fine'  , name='Output file conversion (fine data)')
call uio_wr(ncout, parL%dtimestep_out_fine, &
                                       'dtime_out_fine', name='Output file time step (fine data)', &
                                                         unit='1')
!
if (.not. present(nc)) then
  ! --- Close file ---
  call uio_closwr(ncout)
endif
!
end subroutine rhd_par_Write


!----------************-----------------------------------------------------------------------------
subroutine rhd_par_Read(file, nc_p, parL)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_par_Read ('rhd_parameters_Read')
!
! PURPOSE:
!   Read parameters from parameter file for RHD code and put it into global variable.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_par_Read(nc_p, file, parL)
!
! INPUT:
!   nc_p:         ('number_channel_print') integer, unit number for text output
!   file:         ('file_name') character with parameter file name
!   parL:         ('parameter_Local') structure with control parameters
!
! OUTPUT:
!
! ROUTINES:
!   rhd_par_RdData:        Read parameters from parameter file in UIO form for RHD code
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_io_module:         Input/output routines
!
! SIDE EFFECTS:
!   Open file, read data from it, close file.
!
! RESTRICTIONS:
!   UIO routines have to be initialized.
!
! PROCEDURE:
!   Call rhd_par_RdData with appropriate parameters.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-09 (B.F. Kiel) Written
!   1997-10-02 (B.F.) plus/endtimestep read
!   2003-09-11 (B.F. Uppsala) Name of parameter file from input parameter and not from global
!                    variable
!   2008-06-10 (W.S. Freiburg) Magnetic boundary conditions
!   2010-05-10 (B.F. Lyon) Rename rhd_rdpar_par -> rhd_par_Read
!   2015-04-25 (B.F. Uppsala) Make control structure input parameter
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_io_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc_p
character(len=*),          intent(in)   :: file
type(par_type),            intent(out)  :: parL
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
call rhd_par_RdData(file, nc_p, n_dustGrainRadiusMax, &
ndes=parL%ndes, description=parL%description, nhis=parL%nhis, &
history=parL%history, Teff=parL%Teff, heat_mode=parL%heat_mode, &
  grav_mode=parL%grav_mode, grav_type=parL%grav_type, &
  grav=parL%grav, C_gravmu=parL%C_gravmu, C_gravCorePotDepth=parL%C_gravCorePotDepth, &
  C_gravCorePotQuad=parL%C_gravCorePotQuad, mass_star=parL%mass_star, r0_grav=parL%r0_grav, &
  r1_grav=parL%r1_grav, r2_grav=parL%r2_grav, r1_rad=parL%r1_rad, &
  C_gravCorer0Heat=parL%C_gravCorer0Heat, centrifugal_force=parL%centrifugal_force, &
  nu_rotation=parL%nu_rotation, ar_RotationAxis=parL%ar_RotationAxis, &
side_bound=parL%side_bound, top_bound=parL%top_bound, bottom_bound=parL%bottom_bound, &
  side_bound_mag_x1=parL%side_bound_mag_x1, side_bound_mag_x2=parL%side_bound_mag_x2, &
  top_bound_mag=parL%top_bound_mag, bottom_bound_mag=parL%bottom_bound_mag, &
s_InFlow=parL%s_InFlow, s_InFlRadGrad=parL%s_InFlRadGrad, s_InFlLatGrad=parL%s_InFlLatGrad, &
  C_sChange=parL%C_sChange, C_PChange=parL%C_PChange, &
  C_v3ChangeLinBottom=parL%C_v3ChangeLinBottom, C_v3ChangeSqrBottom=parL%C_v3ChangeSqrBottom, &
  C_TChange=parL%C_TChange, C_rhoChangeTop=parL%C_rhoChangeTop, C_Tsurf=parL%C_Tsurf, &
  C_TminLimit=parL%C_TminLimit, C_TminRate=parL%C_TminRate, &
C_Hptopfactor=parL%C_Hptopfactor, rho_min=parL%rho_min, &
LuminosityPerVolume=parL%LuminosityPerVolume, &
  hdCoreHeatProfile=parL%hdCoreHeatProfile, &
  hdCoreDragProfile=parL%hdCoreDragProfile, C_CoreDrag=parL%C_CoreDrag, &
B1_inflow=parL%B1_inflow, C_magthetaB=parL%C_magthetaB, C_magphiB=parL%C_magphiB, &
  N_magDiffSide=parL%N_magDiffSide, N_magDiffTop=parL%N_magDiffTop, &
  N_magDiffBottom=parL%N_magDiffBottom,&
abuX=parL%abuX, abuY=parL%abuY, abuZ=parL%abuZ, Qmol=parL%Qmol, gamma=parL%gamma, &
EOSFile=parL%EOSFile, EOSPath=parL%EOSPath, &
  kappaMin=parL%kappaMin, kappaMax=parL%kappaMax, OpaFile=parL%OpaFile, OpaPath=parL%OpaPath, &
chem_abumetal=parL%chem_abumetal, chem_reacfile=parL%chem_reacfile, &
  chem_reacpath=parL%chem_reacpath, &
hion_datapath=parL%hion_datapath, hion_atomfile=parL%hion_atomfile, hion_abufile=parL%hion_abufile,&
  hion_edensfile=parL%hion_edensfile, hion_pffile=parL%hion_pffile, &
  dtime_out_hion=parL%dtime_out_hion, & 
  hion_chunks=parL%hion_chunks, &
hdScheme=parL%hdScheme, hdTimeIntegrationScheme=parL%hdTimeIntegrationScheme, &
  hdsplit=parL%hdsplit, hdTmpStructKeepMode=parL%hdTmpStructKeepMode, &
  hdCheckFlux=parL%hdCheckFlux, reconstruction=parL%reconstruction, &
  hdTransVeloMode=parL%hdTransVeloMode, hdEntropyWaveMode=parL%hdEntropyWaveMode, &
  hdEnthalpyAvgMode=parL%hdEnthalpyAvgMode, &
  N_orderConstrainedTransport=parL%N_orderConstrainedTransport, &
  N_hydIter=parL%N_hydIter, N_hydMaxIter=parL%N_hydMaxIter, N_hydRKIter=parL%N_hydRKIter, &
  C_hydTDiffLin=parL%C_hydTDiffLin, C_hydTDiffMach=parL%C_hydTDiffMach, &
  C_hydsDiffVelo=parL%C_hydsDiffVelo, C_hydvDiffVelo=parL%C_hydvDiffVelo, &
  C_hydLowMachcsFactor=parL%C_hydLowMachcsFactor, C_hydLowMachcsBase=parL%C_hydLowMachcsBase, &
  C_hydSmallFluctFactor=parL%C_hydSmallFluctFactor, &
  C_SlopeRed=parL%C_SlopeRed, C_RecContShift=parL%C_RecContShift, &
  C_RecContSteep=parL%C_RecContSteep, &
  C_RecWenoWeightCenter=parL%C_RecWenoWeightCenter, C_RecWenoWeightRight=parL%C_RecWenoWeightRight,&
  C_RecWenoPsiFactor=parL%C_RecWenoPsiFactor, &
  C_hydPredfactor=parL%C_hydPredfactor, &
  C_visBound=parL%C_visBound, &
  beta_inv=parL%beta_inv, va_max=parL%va_max, &
N_hydCellsPerChunk=parL%N_hydCellsPerChunk, N_hydCellsPerChunk2=parL%N_hydCellsPerChunk2, &
  N_hydCellsPerChunk3=parL%N_hydCellsPerChunk3, N_visCellsPerChunk=parL%N_visCellsPerChunk, &
  N_resCellsPerChunk=parL%N_resCellsPerChunk, N_radCellsPerChunk=parL%N_radCellsPerChunk, &
  aN_MPIProcsPerDim=parL%aN_MPIProcsPerDim, &
dustScheme=parL%dustScheme, dustReconstruction=parL%dustReconstruction, &
  dustOpaFile=parL%dustOpaFile, &
  dustopamode=parL%dustopamode, &
  n_dustGrainRadius=parL%n_dustGrainRadius, ar_dustGrainRadius=parL%ar_dustGrainRadius, &
  n_dustGrainAbu=parL%n_dustGrainAbu, ar_dustGrainAbu=parL%ar_dustGrainAbu, &
  C_dust01=parL%C_dust01, C_dust02=parL%C_dust02, C_dust03=parL%C_dust03, &
  C_dust04=parL%C_dust04, C_dust05=parL%C_dust05, C_dust06=parL%C_dust06, C_dust07=parL%C_dust07, &
  C_dust08=parL%C_dust08, C_dust09=parL%C_dust09, C_dust10=parL%C_dust10, &
radScheme=parL%radScheme, N_radminiter=parL%N_radminiter, &
  N_raditer=parL%N_raditer, N_radmaxiter=parL%N_radmaxiter, N_radtaurefine=parL%N_radtaurefine, &
  radThinHeatScheme=parL%radThinHeatScheme, radThinPressScheme=parL%radThinPressScheme, &
  N_radThinPoint=parL%N_radThinPoint, N_radThinHeatPoint=parL%N_radThinHeatPoint, &
  N_radThinPressPoint=parL%N_radThinPressPoint, N_radThickPoint=parL%N_radThickPoint, &
  N_radtheta=parL%N_radtheta, &
  N_radphi=parL%N_radphi, N_radsubray=parL%N_radsubray, N_radband=parL%N_radband, &
  N_radrsyslevel=parL%N_radrsyslevel, N_radoutput=parL%N_radoutput, &
  N_radFeautrier=parL%N_radFeautrier, &
  C_radimplicitmu=parL%C_radimplicitmu, C_raditereps=parL%C_raditereps, &
  C_raditerstep=parL%C_raditerstep, &
  C_radkappasmooth=parL%C_radkappasmooth, C_radTstep=parL%C_radTstep, &
  C_radTsmooth=parL%C_radTsmooth,&
  C_radTintminfac=parL%C_radTintminfac, C_radHtautop=parL%C_radHtautop, &
  C_radTvisdtau=parL%C_radTvisdtau, &
  C_radTvis=parL%C_radTvis, C_radthintimefac=parL%C_radthintimefac, &
  C_radthintimeRelax=parL%C_radthintimeRelax, C_radthinTFactor=parL%C_radthinTFactor, &
  C_radtcool=parL%C_radtcool, &
  C_radscool=parL%C_radscool, C_raddcool=parL%C_raddcool, &
  C_radTinci=parL%C_radTinci, C_radDinci=parL%C_radDinci, &
  C_radAccEnergyFraction=parL%C_radAccEnergyFraction, &
  radraybase=parL%radraybase, &
  radraystar=parL%radraystar, radpressure=parL%radpressure, C_visLinear=parL%C_visLinear, &
C_visSmagorinsky=parL%C_visSmagorinsky, C_visArtificial=parL%C_visArtificial, &
  C_visExpansion=parL%C_visExpansion, C_visPrturb=parL%C_visPrturb, &
  C_visTensordiag=parL%C_visTensordiag, &
  C_visTensoroff=parL%C_visTensoroff, C_visTensordiv=parL%C_visTensordiv, & 
  C_visP2Pcoeff=parL%C_visP2Pcoeff, &
  C_visP2Pincl1=parL%C_visP2Pincl1, C_visP2Pincl2=parL%C_visP2Pincl2, &
  C_visP2PhypSmagorinsky=parL%C_visP2PhypSmagorinsky, &
  C_visP2PhypArtificial=parL%C_visP2PhypArtificial, C_visP2PhypExpansion=parL%C_visP2PhypExpansion,&
  C_visP2Pdivrhov=parL%C_visP2Pdivrhov, &
  N_visIter=parL%N_visIter, N_visP2PhypSmooth=parL%N_visP2PhypSmooth, &
  visP2Phypreconstruction=parL%visP2Phypreconstruction, &
C_resB=parL%C_resB, C_resBconstant=parL%C_resBconstant, C_resEpsilon=parL%C_resEpsilon, &
starttime=parL%starttime, &
plustime=parL%plustime, starttimestep=parL%starttimestep, endtimestep=parL%endtimestep, &
  CPUtime=parL%CPUtime, CPUtime_remainlimit=parL%CPUtime_remainlimit, endtime=parL%endtime, &
  time_ScaleStart=parL%time_ScaleStart, time_ScaleEnd=parL%time_ScaleEnd, &
  plustimestep=parL%plustimestep, dtime_min=parL%dtime_min, dtime_max=parL%dtime_max, &
dtime_min_stop=parL%dtime_min_stop, dtime_incmax=parL%dtime_incmax, dtime_start=parL%dtime_start, &
C_Courant=parL%C_Courant, C_CourantMax=parL%C_CourantMax, C_maxeiChange=parL%C_maxeiChange, &
  C_hydExpCourant=parL%C_hydExpCourant, C_hydExpCourantMax=parL%C_hydExpCourantMax, &
  C_hydSoundCourant=parL%C_hydSoundCourant, C_hydSoundCourantMax=parL%C_hydSoundCourantMax, &
  C_radCourant=parL%C_radCourant, C_radCourantMax=parL%C_radCourantMax, &
  C_radmaxeiChange=parL%C_radmaxeiChange, &
  C_visCourant=parL%C_visCourant, C_visCourantMax=parL%C_visCourantMax, &
  C_resCourant=parL%C_resCourant, C_resCourantMax=parL%C_resCourantMax, &
dtime_out_full=parL%dtime_out_full, dtime_out_mean=parL%dtime_out_mean, &
  dtime_out_end=parL%dtime_out_end, dtimestep_out_fine=parL%dtimestep_out_fine, &
istep_in_start=parL%istep_in_start, N_outslicedim_mean=parL%N_outslicedim_mean, &
  infile_start=parL%infile_start, &
OutFile_end=parL%OutFile_end, OutForm_end=parL%OutForm_end, OutConv_end=parL%OutConv_end, &
  OutFile_full=parL%OutFile_full, OutForm_full=parL%OutForm_full, OutConv_full=parL%OutConv_full, &
  OutFile_mean=parL%OutFile_mean, OutForm_mean=parL%OutForm_mean, OutConv_mean=parL%OutConv_mean, &
  OutFile_fine=parL%OutFile_fine, OutForm_fine=parL%OutForm_fine, OutConv_fine=parL%OutConv_fine, &
  OutFile_mcyl=parL%OutFile_mcyl, &
  OutFile_chu1=parL%OutFile_chu1, OutDim_chu1=parL%OutDim_chu1, &
  OutFile_chu2=parL%OutFile_chu2, OutDim_chu2=parL%OutDim_chu2, &
  OutFile_chu3=parL%OutFile_chu3, OutDim_chu3=parL%OutDim_chu3, &
  OutFile_out=parL%OutFile_out, &
N_test1=parL%N_test1, N_test2=parL%N_test2, N_test3=parL%N_test3, &
  N_test4=parL%N_test4, N_test5=parL%N_test5, C_test1=parL%C_test1, C_test2=parL%C_test2, &
  C_test3=parL%C_test3, C_test4=parL%C_test4, C_test5=parL%C_test5, strtest1=parL%strtest1, &
  strtest2=parL%strtest2, strtest3=parL%strtest3, strtest4=parL%strtest4, strtest5=parL%strtest5)
!
end subroutine rhd_par_Read


!----------****************-------------------------------------------------------------------------
subroutine rhd_par_ToSubpar(parL)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_par_ToSubpar ('rhd_parameters_To_Sub_parameters')
!
! PURPOSE:
!   Set values in parameter sub structure based on input parameters.
!   
! CATEGORY:
!   Parameter handling
!
! CALLING SEQUENCE:
!   call rhd_par_ToSubpar(parL)
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! INPUT/OUTPUT:
!   parL:         ('parameter_Local') structure with control parameters
!
! ROUTINES:
!   rhd_action_StrToRecon: Analyze string with reconstruction scheme
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_par_ToSubpar(par)
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written
!   1997-10-06 (B.F.) Superbee: reconstruction_order=1
!   1997-10-08 (B.F.) MinMod, vanLeer
!   1997-10-21 (B.F.) PP
!   1997-10-25 (B.F.) Superbeemod
!   1999-08-19 (B.F.) 'closedbottom'
!   1999-09-28 (B.F.) 'transmitting'
!   2001-11-19 (B.F.) SHORTrad
!   2002-11-06 (B.F.) RoeMagKin
!   2002-11-13 (B.F.) bound_transmitting2
!   2007-01-16 (B.F. Lyon) reconstruction scheme analysis in subroutine, hdSchemes with space
!   2008-06-10 (W.S. Freiburg) Magnetic boundary conditions and new hdScheme HLLMHD added
!   2010-03-31 (W.S. Freiburg) Magnetic boundary condition 'fixed' also for bottom and top
!                     Default magnetic boundary conditions
!   2010-04-28 (B.F.) bound_transmitting3
!   2010-06-12 (B.F.) bound_inoutflow2, bound_closedbottom2
!   2010-06-30 (B.F.) hdScheme_CenMHD
!   2011-09-08 (B.F.) Modify sub structure 'A' of 'par'
!   2011-11-23 (B.F.) Analyze dustReconstruction
!   2011-12-15 (B.F.) bound_inoutflow3
!   2014-09-21 (B.F.) bound_closed2
!   2015-04-25 (B.F. Uppsala) Make control structure input parameter
!   2018-11-17 (B.F.) Extract from rhd_par_Toaction
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
type(par_type),            intent(inout):: parL
!
! --- Local variables ---
integer                                 :: hdScheme, &
                                           reconstruction_order, reconstruction_type, &
                                           reconstruction_qucorder, reconstruction_quctype, &
                                           radScheme
character(len=80)                       :: reconstruction
!---------------------------------------------------------------------------------------------------
!
! \label{f90:hdscheme__proc1}
! --- Check hydrodynamics scheme ---
select case (parL%hdScheme)
  case ('roe', 'Roe', 'ROE', 'roewave', 'RoeWave')
    hdScheme=hdScheme_RoeWave
  case ('roestate', 'RoeState')
    hdScheme=hdScheme_RoeState
  case ('roemagkin', 'RoeMagKin', 'ROEMAGKIN')
    hdScheme=hdScheme_RoeMagKin
  case ('roemhd', 'Roemhd', 'RoeMHD', 'ROEMHD')
    hdScheme=hdScheme_RoeMHD
  case ('hllmhd', 'HllMHD', 'HLLmhd', 'HLLMHD')
    hdScheme=hdScheme_HLLMHD
  case ('centralmhd', 'CentralMHD', 'Centralmhd', 'CENTRALMHD')
    hdScheme=hdScheme_CenMHD
  case default
    hdScheme=hdScheme_None
endselect
!
! --- Put value into global parameter structure ---
parL%A%hdScheme=hdScheme
!
! --- Check hydro reconstruction method ---
!write(6,*)
!write(6,*) 'parL%reconstruction:     ', trim(parL%reconstruction)
!write(6,*) 'parL%dustReconstruction: ', trim(parL%dustReconstruction)
!write(6,*) index(trim(parL%reconstruction), ' ')
!
if (parL%dustReconstruction /= '') then
  ! --- Some dust reconstruction is specified ---
  !
  call rhd_action_StrToRecon(parL%dustReconstruction, &
                             reconstruction_qucorder, reconstruction_quctype)
  !write(6,'(2A,2I3)') '  D: ', trim(parL%dustReconstruction), reconstruction_qucorder, &
  !                             reconstruction_quctype
  !
endif
!
! \label{f90:reconstruction__proc2}
if (index(trim(parL%reconstruction), ' ') == 0) then
  ! --- Reconstruction method is single word ---
  !
  reconstruction=parL%reconstruction
  call rhd_action_StrToRecon(reconstruction, reconstruction_order, reconstruction_type)
  !
  if (parL%dustReconstruction == '') then
    reconstruction_qucorder=reconstruction_order
    reconstruction_quctype =reconstruction_type
    !write(6,'(2A,2I3)') '  D: ', trim(reconstruction), reconstruction_qucorder, &
    !                             reconstruction_quctype
  endif
  !
else
  ! --- Multiple words ---
  !
  reconstruction=parL%reconstruction(1:index(trim(parL%reconstruction), ' ')-1)
  call rhd_action_StrToRecon(reconstruction, reconstruction_order, reconstruction_type)
  !
  if (parL%dustReconstruction == '') then
    call rhd_action_StrToRecon(adjustl(parL%reconstruction( &
                                         index(trim(parL%reconstruction),' ')+1:)), &
                               reconstruction_qucorder, reconstruction_quctype)
    !write(6,'(2A,2I3)') '  D: ', &
    !                    adjustl(parL%reconstruction(index(trim(parL%reconstruction), ' ')+1:)), &
    !                    reconstruction_qucorder, reconstruction_quctype
  endif
  !
endif
!
!write(6,'(2A,2I3)') '  H: ', trim(reconstruction), reconstruction_order, reconstruction_type
!
if (reconstruction_qucorder /= reconstruction_order) then
  stop 'Reconstruction orders of HD and dust scheme have to be the same.'
endif
!
! --- Put values into global parameter structure ---
parL%A%reconstruction_order  =reconstruction_order
parL%A%reconstruction_type   =reconstruction_type
parL%A%reconstruction_quctype=reconstruction_quctype
!
! --- Check VisP2P reconstruction method ---
call rhd_action_StrToRecon(parL%visP2Phypreconstruction, &
                           parL%A%visP2Phypreconstruction_order, &
                           parL%A%visP2Phypreconstruction_type)
!write(6,'(2A,2I3)') '  V: ', trim(parL%visP2Phypreconstruction), &
!                               parL%A%visP2Phypreconstruction_order, &
!                               parL%A%visP2Phypreconstruction_type
if ((parL%A%visP2Phypreconstruction_type /= reconstruction_type_Constant) .and. &
    (parL%A%visP2Phypreconstruction_type /= reconstruction_type_Minmod)) then
  stop 'Reconstruction method for P2P viscosity not supported. Use either constand or MinMod!'
endif
!
!\label{f90:radscheme__proc1}
! --- Check radiation transport scheme ---
select case (parL%radScheme)
  case ('lhdrad', 'LHDrad', 'LHDRAD')
    radScheme=radScheme_LHDrad
  case ('mscrad', 'MSCrad', 'MSCRAD')
    radScheme=radScheme_MSCrad
  case ('lcfrad', 'LCFrad', 'LCFRAD')
    radScheme=radScheme_LCFrad
  case ('shortrad', 'SHORTrad', 'SHORTRAD')
    radScheme=radScheme_SHORTrad
  case default
    radScheme=radScheme_None
endselect
!
! --- Put value into global parameter structure ---
parL%A%radScheme=radScheme
!
end subroutine rhd_par_ToSubpar


!----------****************-------------------------------------------------------------------------
subroutine rhd_par_Toaction(parL, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_par_Toaction ('rhd_parameters_To_action')
!
! PURPOSE:
!   Prepare action variable from input parameters.
!   
! CATEGORY:
!   Parameter handling
!
! CALLING SEQUENCE:
!   call rhd_par_Toaction(parL, action)
!
! INPUT:
!   parL:         ('parameter_Local') structure with control parameters
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   action:       ('action') action_type, input structure with rhd control parameters
!
! ROUTINES:
!   rhd_action_New:        Function: Change contents of action variable
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_par_Toaction(par, action)
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written
!   1997-10-06 (B.F.) Superbee: reconstruction_order=1
!   1997-10-08 (B.F.) MinMod, vanLeer
!   1997-10-21 (B.F.) PP
!   1997-10-25 (B.F.) Superbeemod
!   1999-08-19 (B.F.) 'closedbottom'
!   1999-09-28 (B.F.) 'transmitting'
!   2001-11-19 (B.F.) SHORTrad
!   2002-11-06 (B.F.) RoeMagKin
!   2002-11-13 (B.F.) bound_transmitting2
!   2007-01-16 (B.F. Lyon) reconstruction scheme analysis in subroutine, hdSchemes with space
!   2008-06-10 (W.S. Freiburg) Magnetic boundary conditions and new hdScheme HLLMHD added
!   2010-03-31 (W.S. Freiburg) Magnetic boundary condition 'fixed' also for bottom and top
!                     Default magnetic boundary conditions
!   2010-04-28 (B.F.) bound_transmitting3
!   2010-06-12 (B.F.) bound_inoutflow2, bound_closedbottom2
!   2010-06-30 (B.F.) hdScheme_CenMHD
!   2011-09-08 (B.F.) Modify sub structure 'A' of 'par'
!   2011-11-23 (B.F.) Analyze dustReconstruction
!   2011-12-15 (B.F.) bound_inoutflow3
!   2014-09-21 (B.F.) bound_closed2
!   2015-04-25 (B.F. Uppsala) Make control structure input parameter
!   2018-11-17 (B.F.) Put modification of parameter structure into separate routine rhd_par_ToSubpar
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
type(par_type),            intent(in)   :: parL
type(action_type),         intent(inout):: action
!
! --- Local variables ---
integer                                 :: bound_l, bound_r, bound_b, bound_t, &
                                           bound_mag_l1, bound_mag_r1, bound_mag_l2, bound_mag_r2, &
                                           bound_mag_b, bound_mag_t
character(len=80)                       :: reconstruction
!---------------------------------------------------------------------------------------------------
!
! \label{f90:side_bound__proc1}
! --- Check lateral boundary conditions ---
if       (parL%side_bound == 'reflective'  )  then
  bound_l=bound_reflective
  bound_r=bound_reflective
else if ((parL%side_bound == 'closed'      ) .or. &
         (parL%side_bound == 'closedtop'   )) then
  bound_l=bound_closed
  bound_r=bound_closed
else if ((parL%side_bound == 'closed2'     ) .or. &
         (parL%side_bound == 'closedtop2'  )) then
  bound_l=bound_closed2
  bound_r=bound_closed2
else if  (parL%side_bound == 'periodic'    )  then
  bound_l=bound_periodic
  bound_r=bound_periodic
else if  (parL%side_bound == 'constant'    )  then
  bound_l=bound_constant
  bound_r=bound_constant
!else if  (parL%side_bound == 'closedbottom')  then
!  bound_l=bound_closedbottom
!  bound_r=bound_closedbottom
!else if  (parL%side_bound == 'closedbottom2')  then
!  bound_l=bound_closedbottom2
!  bound_r=bound_closedbottom2
else if ((parL%side_bound == 'transmitting' ) .or. &
         (parL%side_bound == 'transmitting1')) then
  bound_l=bound_transmitting
  bound_r=bound_transmitting
else if (parL%side_bound == 'transmitting2' ) then
  bound_l=bound_transmitting2
  bound_r=bound_transmitting2
else if (parL%side_bound == 'transmitting3' ) then
  bound_l=bound_transmitting3
  bound_r=bound_transmitting3
else
  bound_l=bound_reflective
  bound_r=bound_reflective
endif
!
! \label{f90:bottom_bound__proc1}
! --- Check bottom boundary condition ---
if       (parL%bottom_bound == 'reflective'   ) then
  bound_b=bound_reflective
else if ((parL%bottom_bound == 'closed'       ) .or. &
         (parL%bottom_bound == 'closedtop'    )) then
  bound_b=bound_closed
else if ((parL%bottom_bound == 'closed2'      ) .or. &
         (parL%bottom_bound == 'closedtop2'   )) then
  bound_b=bound_closed2
else if  (parL%bottom_bound == 'periodic'     )  then
  bound_b=bound_periodic
else if  (parL%bottom_bound == 'constant'     )  then
  bound_b=bound_constant
else if  (parL%bottom_bound == 'inoutflow'    )  then
  bound_b=bound_inoutflow
else if  (parL%bottom_bound == 'inoutflow2'   )  then
  bound_b=bound_inoutflow2
else if  (parL%bottom_bound == 'inoutflow3'   )  then
  bound_b=bound_inoutflow3
else if  (parL%bottom_bound == 'closedbottom' )  then
  bound_b=bound_closedbottom
else if  (parL%bottom_bound == 'closedbottom2')  then
  bound_b=bound_closedbottom2
else if ((parL%bottom_bound == 'transmitting' ) .or. &
         (parL%bottom_bound == 'transmitting1')) then
  bound_b=bound_transmitting
else if  (parL%bottom_bound == 'transmitting2')  then
  bound_b=bound_transmitting2
else if  (parL%bottom_bound == 'transmitting3')  then
  bound_b=bound_transmitting3
else
  bound_b=bound_reflective
endif
!
! \label{f90:top_bound__proc1}
! --- Check top boundary condition ---
if       (parL%top_bound == 'reflective'   )  then
  bound_t=bound_reflective
else if ((parL%top_bound == 'closed'       ) .or. &
         (parL%top_bound == 'closedtop'    )) then
  bound_t=bound_closed
else if ((parL%top_bound == 'closed2'      ) .or. &
         (parL%top_bound == 'closedtop2'   )) then
  bound_t=bound_closed2
else if  (parL%top_bound == 'periodic'     )  then
  bound_t=bound_periodic
else if  (parL%top_bound == 'constant'     )  then
  bound_t=bound_constant
else if ((parL%top_bound == 'transmitting' ) .or. &
         (parL%top_bound == 'transmitting1')) then
  bound_t=bound_transmitting
else if  (parL%top_bound == 'transmitting2')  then
  bound_t=bound_transmitting2
else if  (parL%top_bound == 'transmitting3')  then
  bound_t=bound_transmitting3
else
  bound_t=bound_reflective
endif
!
! --- Check lateral magnetic boundary conditions ---
if       (parL%side_bound_mag_x1 == 'periodic'   )  then
  bound_mag_l1=bound_mag_periodic
  bound_mag_r1=bound_mag_periodic
else if  (parL%side_bound_mag_x1 == 'constant'   )  then
  bound_mag_l1=bound_mag_constant
  bound_mag_r1=bound_mag_constant
else if  (parL%side_bound_mag_x1 == 'constant2'  )  then
  bound_mag_l1=bound_mag_constant2
  bound_mag_r1=bound_mag_constant2
else if  (parL%side_bound_mag_x1 == 'fixed'      )  then
  bound_mag_l1=bound_mag_fixed
  bound_mag_r1=bound_mag_fixed
else if  (parL%side_bound_mag_x1 == 'vertical'   )  then
  bound_mag_l1=bound_mag_vertical
  bound_mag_r1=bound_mag_vertical
else if  (parL%side_bound_mag_x1 == 'vertical2'  )  then
  bound_mag_l1=bound_mag_vertical2
  bound_mag_r1=bound_mag_vertical2
else if  (parL%side_bound_mag_x1 == 'reflective' )  then
  bound_mag_l1=bound_mag_reflective
  bound_mag_r1=bound_mag_reflective
else
  bound_mag_l1=bound_mag_periodic
  bound_mag_r1=bound_mag_periodic
endif
if       (parL%side_bound_mag_x2 == 'periodic'   )  then
  bound_mag_l2=bound_mag_periodic
  bound_mag_r2=bound_mag_periodic
else if  (parL%side_bound_mag_x2 == 'constant'   )  then
  bound_mag_l2=bound_mag_constant
  bound_mag_r2=bound_mag_constant
else if  (parL%side_bound_mag_x2 == 'constant2'  )  then
  bound_mag_l2=bound_mag_constant2
  bound_mag_r2=bound_mag_constant2
else if  (parL%side_bound_mag_x2 == 'fixed'      )  then
  bound_mag_l2=bound_mag_fixed
  bound_mag_r2=bound_mag_fixed
else if  (parL%side_bound_mag_x2 == 'vertical'   )  then
  bound_mag_l2=bound_mag_vertical
  bound_mag_r2=bound_mag_vertical
else if  (parL%side_bound_mag_x2 == 'vertical2'  )  then
  bound_mag_l2=bound_mag_vertical2
  bound_mag_r2=bound_mag_vertical2
else if  (parL%side_bound_mag_x2 == 'reflective' )  then
  bound_mag_l2=bound_mag_reflective
  bound_mag_r2=bound_mag_reflective
else
  bound_mag_l2=bound_mag_periodic
  bound_mag_r2=bound_mag_periodic
endif
!
! --- Check bottom magnetic boundary condition ---
if       (parL%bottom_bound_mag == 'periodic'    )  then
  bound_mag_b=bound_mag_periodic
else if  (parL%bottom_bound_mag == 'constant'    )  then
  bound_mag_b=bound_mag_constant
else if  (parL%bottom_bound_mag == 'constant2'   )  then
  bound_mag_b=bound_mag_constant2
else if  (parL%bottom_bound_mag == 'fixed'       )  then
  bound_mag_b=bound_mag_fixed
else if  (parL%bottom_bound_mag == 'vertical'    )  then
  bound_mag_b=bound_mag_vertical
else if  (parL%bottom_bound_mag == 'vertical2'   )  then
  bound_mag_b=bound_mag_vertical2
else if  (parL%bottom_bound_mag == 'inoutflow'   )  then
  bound_mag_b=bound_mag_inoutflow
else if  (parL%bottom_bound_mag == 'oblique'     )  then
  bound_mag_b=bound_mag_oblique
else
  bound_mag_b=bound_mag_constant
endif
!
! --- Check top magnetic boundary condition ---
if       (parL%top_bound_mag == 'periodic'    )  then
  bound_mag_t=bound_mag_periodic
else if  (parL%top_bound_mag == 'constant'    )  then
  bound_mag_t=bound_mag_constant
else if  (parL%top_bound_mag == 'constant2'   )  then
  bound_mag_t=bound_mag_constant2
else if  (parL%top_bound_mag == 'fixed'       )  then
  bound_mag_t=bound_mag_fixed
else if  (parL%top_bound_mag == 'vertical'    )  then
  bound_mag_t=bound_mag_vertical
else if  (parL%top_bound_mag == 'vertical2'   )  then
  bound_mag_t=bound_mag_vertical
else if  (parL%top_bound_mag == 'oblique'     )  then
  bound_mag_t=bound_mag_oblique
else
  bound_mag_t=bound_mag_constant
endif
!
! --- Modify all fields of action variable, e.g., create a new one ---
action=rhd_action_New(action, &
                      dim=1, dimV=((/1, 2, 3/)), &
                      bound_l1    =bound_l     , bound_r1    =bound_r     , &
                      bound_l2    =bound_l     , bound_r2    =bound_r     , &
                      bound_l3    =bound_b     , bound_r3    =bound_t     , &
                      bound_mag_l1=bound_mag_l1, bound_mag_r1=bound_mag_r1, &
                      bound_mag_l2=bound_mag_l2, bound_mag_r2=bound_mag_r2, &
                      bound_mag_l3=bound_mag_b , bound_mag_r3=bound_mag_t , &
                      radraybase=parL%radraybase, radraystar=parL%radraystar )
!
end subroutine rhd_par_Toaction


!----------*************----------------------------------------------------------------------------
subroutine rhd_box_Write(command, ncout, box_id, box, dtime, &
                         time_out_full_last, time_out_mean_last, &
                         file, form, conv, &
                         ndes, description, nhis, history, version, &
                         SubBoxDim, SubBoxMode, SubBoxName)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Write ('rhd_box_Write')
!
! PURPOSE:
!   Write box with additional information into file in UIO form.
!   
! CATEGORY:
!   I/O, box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Write(command, ncout, box_id=box_id, box=box, &
!                      dtime=dtime, time_out_full_last=time_out_full_last, &
!                      time_out_mean_last=time_out_mean_last, &
!                      file=file, form=form, conv=conv, &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version=version, &
!                      SubBoxDim=SubBoxDim, SubBoxMode=SubBoxmode, SubBoxName=SubBoxName)
!
! INPUT:
!   command:      ('command') character, command string: 
!                   'open'  : Open new file and write header
!                   'append': Open existing file
!                   'model' : Write 'dataset' label and general model data at beginning of dataset
!                   'box':    Write box
!                   'endmo':  Write 'end dataset' label
!                   'close':  Close the file
!   --- optional: ---
!   file:         ('file') character, name of data file
!   form:         ('format') character, name of data file
!   conv:         ('conversion') character, conversion type of data file
!   box_id:       ('box_identifier') character, identifier for box, default='z'
!   box:          ('box') type(box_type), input computational box
!   dtime:        ('delta_time') real, time step
!   time_out_full_last: ('time_output_full_last') real, time of last output of full model
!   time_out_mean_last: ('time_output_mean_last') real, time of last output of averaged data
!   ndes:         ('number_description') integer, number of lines of description
!   description:  ('description') character, dimension(:), description of model
!   nhis:         ('number_history') integer, number of lines of history
!   history:      ('history') character, dimension(:), history of model
!   version:      ('version') character, program version
!   SubBoxDim:    ('Sub_Box_Dimension') optional 2x3 real array with output dimension
!   SubBoxMode:   ('Sub_Box_Mode') optional character with mode how to find or interpret
!                 the output dimension
!   SubBoxName:   ('Sub_box_Name') optional character with name of output if SubBoxDim is specified,
!                 e.g., 'Chunk 1'
!
! INPUT/OUTPUT:
!   ncout:        ('number_channel_output') integer, unit number for data output
!
! LOCAL VARIABLES:
!   m1,2,3c:      ('m_1,2,3_chunk') integer, first index value of chunk
!   n1,2,3c:      ('n_1,2,3_chunk') integer, last index value of chunk
!   slice:        ('slice') type(box_type), slice (or subblock) of current model
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_SubBoxIndices: Compute indices of box representing the section of both input boxes
!   rhd_box_WrData:        Write RHD model into file in UIO form
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_io_module:         Input/output routines
!
! SIDE EFFECTS:
!   Write data to file.
!
! RESTRICTIONS:
!   If the command string contains 'open', the optional parameter
!   file has to be specified.
!   If the command string contains 'model', the optional parameters
!   box, dtime, time_out_full_last have to be specified.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-22 (B.F. Kiel) Written
!   1997-10-15 (B.F.) Parameters dtime, time_out_full_last added
!   1999-04-08 (B.F. Copenhagen) Call rhd_wrboxdata also with 'endmo'
!   1999-06-03 (B.F.) Parameters history and description added
!   2000-01-22 (B.F.) New commands 'box', 'endmo' added, parameter box_id added
!   2010-05-10 (B.F. Lyon) Rename write_box -> rhd_box_Write
!   2017-07-28 (B.F. Uppsala) Add optional keywords OutDim and OutName to specify sub box
!   2017-07-28 (B.F.) Add optional keywords OutMode to specify sub box
!   2017-08-01 (B.F.) Rename new keywords: Out... -> SubBox...; call rhd_box_SubBoxIndices
!   2017-12-18 (B.F.) Remove merge command because of compiler warning
!---------------------------------------------------------------------------------------------------
use const_module
use rhd_gl_module
use rhd_box_module
use rhd_io_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(inout):: ncout
character(len=*),optional, intent(in)   :: file, form, conv, box_id
integer,         optional, intent(in)   :: ndes
character(len=*), dimension(:), &
                 optional, intent(in)   :: description
integer,         optional, intent(in)   :: nhis
character(len=*), dimension(:), &
                 optional, intent(in)   :: history
character(len=*),optional, intent(in)   :: version, SubBoxMode, SubBoxName
type(box_type),  optional, intent(in)   :: box
real,            optional, intent(in)   :: dtime, time_out_full_last, time_out_mean_last
integer, dimension(2,3), &
                 optional, intent(in)   :: SubBoxDim
!
! --- Local variables ---
integer                                 :: m1c,n1c, m2c,n2c, m3c,n3c
logical                                 :: IsEmpty_flag, IsComplete_flag
!character(len=80)                       :: SubBoxMode0
type(box_type)                          :: slice
!---------------------------------------------------------------------------------------------------
!
if (index(command,'open') /= 0) then
  ! --- Open file for writing in uio-form ---
  call rhd_box_WrData('open', gl%nc_p, ncout, file=file, form=form, conv=conv, &
                     ndes=ndes, description=description, nhis=nhis, history=history, &
                     version=version)
else if (index(command,'append') /= 0) then
  ! --- Append data to file ---
  call rhd_box_WrData('append', gl%nc_p, ncout, file=file, form=form, conv=conv, &
                     ndes=ndes, description=description, nhis=nhis, history=history, &
                     version=version)
endif
!
!
if (index(command,'model') /= 0) then
  ! --- Write model header to file ---
  call rhd_box_WrData('model', gl%nc_p, ncout, &
                     time_unit=time_unit, &
                     itime=box%itime, time=box%time, time_db=box%time_db, &
                     dtime=dtime, time_out_full_last=time_out_full_last, &
                       time_out_mean_last=time_out_mean_last)
endif ! (index(command,'model') /= 0)
!
if (index(command,'box') /= 0) then
  ! --- Write box data to file ---
  !
  call rhd_box_SubBoxIndices(box%m1,box%n1, box%m2,box%n2, box%m3,box%n3, &
                             0  ,0  , 0  ,0  , 0  ,0  , &
                             m1c,n1c, m2c,n2c, m3c,n3c, &
                             SubBoxMode=SubBoxMode, SubBoxDim=SubBoxDim, &
                             IsEmpty_flag=IsEmpty_flag, IsComplete_flag=IsComplete_flag)
  !
  if (present(SubBoxName)) then
    write(gl%nc_p,'(A,3(I5,1X,I5,3X),L1)') &
                             trim(SubBoxName) // &
                             ' output; m1c,n1c, m2c,n2c, m3c,n3c: ', &
                                       m1c,n1c, m2c,n2c, m3c,n3c, (.not. IsEmpty_flag)
    call flush(gl%nc_p)
  endif
  !
  if (.not. IsEmpty_flag) then
    if ((IsComplete_flag) .and. &
        (box%nghost1 == 0) .and. (box%nghost2 == 0) .and. (box%nghost3 == 0)) then
      ! --- Write entire model box ---
      !
      call rhd_box_WrData('box', gl%nc_p, ncout, &
                          time_unit=time_unit, &
                          x1_unit=x1_unit, x2_unit=x2_unit, x3_unit=x3_unit, &
                          rho_unit=rho_unit, v_unit=v_unit, e_unit=e_unit, &
                          box_id=box_id, &
                          m1=box%m1, m2=box%m2, m3=box%m3, &
                          n1=box%n1, n2=box%n2, n3=box%n3, &
                          itime=box%itime, time=box%time, time_db=box%time_db, &
                          dtime=dtime, &
                          xc1=box%xc1, xc2=box%xc2, xc3=box%xc3, &
                          xb1=box%xb1, xb2=box%xb2, xb3=box%xb3, &
                          rho=box%rho, ei=box%ei, v1=box%v1, v2=box%v2, v3=box%v3, &
                          quc=box%quc, Bb1=box%Bb1, Bb2=box%Bb2, Bb3=box%Bb3)
      !
    else ! (.not. IsComplete_flag)
      ! --- Create, copy, write, and delete the requested sub box ---
      !
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                        quc_flag=.true., Bb_flag=.true.)
      call rhd_box_Copy(box  ,    m1c,    n1c,    m2c,    n2c,    m3c,    n3c, &
                        slice,    m1c,            m2c,            m3c, &
                        time_mode=1, grid_mode=1)
      call rhd_box_WrData('box', gl%nc_p, ncout, &
                          time_unit=time_unit, &
                          x1_unit=x1_unit, x2_unit=x2_unit, x3_unit=x3_unit, &
                          rho_unit=rho_unit, v_unit=v_unit, e_unit=e_unit, &
                          box_id=box_id, &
                          m1=slice%m1, m2=slice%m2, m3=slice%m3, &
                          n1=slice%n1, n2=slice%n2, n3=slice%n3, &
                          itime=slice%itime, time=slice%time, time_db=slice%time_db, &
                          dtime=dtime, &
                          xc1=slice%xc1, xc2=slice%xc2, xc3=slice%xc3, &
                          xb1=slice%xb1, xb2=slice%xb2, xb3=slice%xb3, &
                          rho=slice%rho, ei=slice%ei, v1=slice%v1, v2=slice%v2, v3=slice%v3, &
                          quc=slice%quc, Bb1=slice%Bb1, Bb2=slice%Bb2, Bb3=slice%Bb3)
      call rhd_box_Delete(slice)
    endif ! (IsComplete_flag)
    !
    call flush(ncout)
  endif ! (.not. IsEmpty_flag)
  !
endif ! (index(command,'box') /= 0)
!
if (index(command,'endmo') /= 0) then
  ! --- Write end-model label to file ---
  call rhd_box_WrData('endmo', gl%nc_p, ncout)
endif
!
!
if (index(command,'close') /= 0) then
  ! --- Close file ---
  call rhd_box_WrData('close', gl%nc_p, ncout)
endif
!
end subroutine rhd_box_Write


!----------************-----------------------------------------------------------------------------
subroutine rhd_box_Read(command, ncin, box, Bb_flag, dtime, &
                        time_out_full_last, time_out_mean_last, &
                        file, imodel, nhis, history, nmodel)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Read ('rhd_box_Read')
!
! PURPOSE:
!   Read box from file in UIO form.
!   
! CATEGORY:
!   I/O, box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Read(command, ncin, box=box, Bb_flag=Bb_flag, &
!                     dtime=dtime, time_out_full_last=time_out_full_last, &
!                       time_out_mean_last=time_out_mean_last, &
!                     file=file, imodel=imodel, &
!                     nhis=nhis, history=history, nmodel=nmodel)
!
! INPUT:
!   command:      ('command') character, command string:
!                   'open'  : Open file and read identifiers
!                   'model' : Read general model data at beginning of dataset
!                   'box':    Read box
!                   'close':  Close the file
!   --- optional: ---
!   Bb_flag:      ('B_boundary_flag') logical to indicate that magnetic fields should be read
!   file:         ('file') character, name of data file
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), input computational box
!   ncin:         ('number_channel_output') integer, unit number for data output
!
! OUTPUT:
!   dtime:        ('delta_time') optional real, time step
!   time_out_full_last: ('time_output_full_last') optional real, time of last output of full model
!   time_out_mean_last: ('time_output_mean_last') optional real, time of last output of avg. data
!   nhis:         ('number_history') optional integer, number of lines of history
!   history:      ('history') optional character, dimension(:), history of model
!   nmodel:       ('number_model') number of model datasets in file
!
! ROUTINES:
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_RdData:        Read RHD model from file in UIO form
!   rhd_box_ZeroOMP:       Set bulk arrays explicitly to zero
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_io_module:         Input/output routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Call rhd_box_RdData with appropriate parameters.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-22 (B.F. Kiel) Written
!   1997-10-02 (B.F.) Some changes
!   1997-10-15 (B.F.) Parameters dtime and time_out_full_last added
!   1999-06-03 (B.F.) Parameter history added
!   2000-02-09 (B.F.) Parameter time_out_mean_last added
!   2000-03-27 (B.F.) Parameter imodel added
!   2002-10-21 (B.F.) Read additional density fields (quc)
!   2002-11-05 (B.F.) Read magnetic fields
!   2003-09-11 (B.F.) Save some lists; add new optional output parameter nmodel; nmodelmx=15->200
!   2010-05-10 (B.F. Lyon) Rename read_box -> rhd_box_Read
!   2011-08-25 (B.F.) New optional parameter Bb_flag
!   2012-09-05 (B.F.) Put axes end coordinates into prop structure
!   2013-05-31 (B.F.) Set bulk arrays to zero before reading them
!   2014-03-20 (B.F.) Move assignment of values of prop structure entries into subroutine
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_box_module
use rhd_prop_module
use rhd_io_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(inout):: ncin
logical,         optional, intent(in)   :: Bb_flag
character(len=*),optional, intent(in)   :: file
integer      ,   optional, intent(in)   :: imodel
type(box_type),  optional, intent(out)  :: box
real,            optional, intent(out)  :: dtime, time_out_full_last, time_out_mean_last
integer,         optional, intent(out)  :: nhis, nmodel
character(len=*), dimension(:), &
                 optional, intent(out)  :: history
!
! --- Local variables ---
logical                                 :: Bb_flag0
integer                                 :: imodel0
integer, parameter                      :: nmodelmx=200
integer,                      save      :: nmodel0
integer, dimension(nmodelmx), save      :: modellist, itimelist
real,    dimension(nmodelmx), save      :: timelist
!---------------------------------------------------------------------------------------------------
!
! --- Check optional parameters ---
if (present(imodel)) then
  imodel0=max(imodel, 1)
else
  imodel0=1
endif
if (present(Bb_flag)) then
  Bb_flag0=Bb_flag
else
  Bb_flag0=.true.
endif
!
if (index(command,'open') /= 0) then
  ! --- Open file for reading in uio-form ---
  call rhd_box_RdData('open', gl%nc_p, ncin, &
                      modellist, timelist, itimelist, nmodel0, imodel0, file=file, &
                      nhis=nhis, history=history)
endif
!
if (index(command,'model') /= 0) then
  ! --- Read general model data ---   
  call rhd_box_RdData('model', gl%nc_p, ncin, &
                      modellist, timelist, itimelist, nmodel0, imodel0, &
                      m1=box%m1, m2=box%m2, m3=box%m3, &
                      n1=box%n1, n2=box%n2, n3=box%n3, &
                      itime=box%itime, time=box%time, time_db=box%time_db, &
                      dtime=dtime, time_out_full_last=time_out_full_last, &
                      time_out_mean_last=time_out_mean_last)
  !
  ! --- Create computational box of requested size: ---
  ! --- Allocate memory for bulk data               ---
  call rhd_box_Make(box, quc_flag=.true., Bb_flag=Bb_flag0)
  !
  ! --- Set bulk arrays explicitly to zero trying to aid proper memory page placement ---
  call rhd_box_ZeroOMP(box)
  !
  ! --- Read box ---
  call rhd_box_RdData('box', gl%nc_p, ncin, &
                      modellist, timelist, itimelist, nmodel0, imodel0, &
                      xc1=box%xc1, xc2=box%xc2, xc3=box%xc3, &
                      xb1=box%xb1, xb2=box%xb2, xb3=box%xb3, &
                      rho=box%rho, ei=box%ei, v1=box%v1, v2=box%v2, v3=box%v3, &
                      quc=box%quc, Bb1=box%Bb1, Bb2=box%Bb2, Bb3=box%Bb3)
  !
endif
!
if (index(command,'close') /= 0) then
  ! --- Close file ---
  call rhd_box_RdData('close', gl%nc_p, ncin, modellist, timelist, itimelist, nmodel0, imodel0)
endif
!
if (present(nmodel)) nmodel=nmodel0
!
end subroutine rhd_box_Read

end module rhd_sub_module
