!***************************************************************************************************
!
!   #####  #    # #####         #    # #####  #    # 
!   #    # #    # #    #        #    # #    # #   #  
!   #    # ###### #    #        #    # #    # ####   
!   #####  #    # #    #        # ## # #####  #  #   
!   #   #  #    # #    #        ##  ## #   #  #   #  
!   #    # #    # #####  ###### #    # #    # #    # _module
!
!   Radiation-Hydrodynamics Code: Provide global work copies of model data
!
!***************************************************************************************************
!   Fortran90
!   2017-10-07
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!***************************************************************************************************
! MODULES:
!   rhd_wrk_module:        Provide persistent work copies of model data
!
!***************************************************************************************************


!------**************-------------------------------------------------------------------------------
module rhd_wrk_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_wrk_module ('rhd_work_module')
!
! PURPOSE:
!   Provide persistent work copies of model data.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_wrk_module
!
! VARIABLES:
!   model1:       ('model_1') type(box_type), model data: rho, ei, v, x, time, ...
!   model2:       ('model_2') type(box_type), model data: rho, ei, v, x, time, ...
!   model3:       ('model_3') type(box_type), model data: rho, ei, v, x, time, ...
!   modelA:       ('model_A') type(box_type), model data: rho, ei, v, x, time, ...
!   modelB:       ('model_B') type(box_type), model data: rho, ei, v, x, time, ...
!   modelVis:     ('model_Viscosity') type(box_type), model data: rho, ei, v, x, time, ...
!
! ROUTINES: (contained)
!   rhd_wrk_boxGloInit:    Initialize global computational boxes. Nullify pointers
!   rhd_wrk_boxGloDelete:  Delete global computational boxes. Deallocate memory
!
! MODIFICATION HISTORY:
!   2017-10-07 (B.F. Uppsala) Extract definitions and routines from rhd_box_module
!---------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
type(box_type), save                    :: model1, model2, model3, modelA, modelB, modelVis
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_wrk_boxGloInit()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_wrk_boxGloInit ('rhd_work_box_Global_Initialize')
!
! PURPOSE:
!   Initialize global computational boxes. Nullify pointers.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_wrk_boxGloInit()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Modify global (saved) box structures.
!
! RESTRICTIONS:
!   This routine should be called once during the initialization phase of the main program.
!   Allthough the structures are global, they should NOT be used to transfer information
!   from one module to another other or from one time step to the next.
!
! PROCEDURE:
!   Set scalar values to zero, nullify pointers.
!
! EXAMPLE:
!   call rhd_wrk_boxGloInit()
!
! MODIFICATION HISTORY:
!   2014-08-17 (B.F. Uppsala) Adapted from rhd_box_Init
!   2017-10-07 (B.F.) Rename routine rhd_box_GloInit -> rhd_wrk_boxGloInit
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Initialize all global boxes ---
call rhd_box_Init(model1  )
call rhd_box_Init(model2  )
call rhd_box_Init(model3  )
call rhd_box_Init(modelA  )
call rhd_box_Init(modelB  )
call rhd_box_Init(modelVis)
!
end subroutine rhd_wrk_boxGloInit


!----------********************---------------------------------------------------------------------
subroutine rhd_wrk_boxGloDelete()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_wrk_boxGloDelete ('rhd_work_box_Global_Delete')
!
! PURPOSE:
!   Delete global computational boxes. Deallocate memory.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_wrk_boxGloDelete()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Exist:         Function: Test if box is already filled
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Modify global (saved) box structures.
!
! RESTRICTIONS:
!   This routine should be called once during the finalization phase of the main program.
!
! PROCEDURE:
!   Delete all global computational boxes one by one.
!
! EXAMPLE:
!   call rhd_wrk_boxGloDelete()
!
! MODIFICATION HISTORY:
!   2014-08-17 (B.F. Uppsala) Adapted from rhd_box_GloInit
!   2017-10-07 (B.F.) Rename routine rhd_box_GloDelete -> rhd_wrk_boxGloDelete
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Delete all global boxes ---
if (rhd_box_Exist(modelVis)) call rhd_box_Delete(modelVis)
if (rhd_box_Exist(modelB  )) call rhd_box_Delete(modelB  )
if (rhd_box_Exist(modelA  )) call rhd_box_Delete(modelA  )
if (rhd_box_Exist(model3  )) call rhd_box_Delete(model3  )
if (rhd_box_Exist(model2  )) call rhd_box_Delete(model2  )
if (rhd_box_Exist(model1  )) call rhd_box_Delete(model1  )
!
end subroutine rhd_wrk_boxGloDelete

end module rhd_wrk_module
