!***************************************************************************************************
!
!   #####  #    # #####          ####   ####    ##   #      ###### 
!   #    # #    # #    #        #      #    #  #  #  #      #      
!   #    # ###### #    #         ####  #      #    # #      #####  
!   #####  #    # #    #             # #      ###### #      #      
!   #   #  #    # #    #        #    # #    # #    # #      #      
!   #    # #    # #####  ######  ####   ####  #    # ###### ###### _module
!
!   Radiation-Hydrodynamics-Code: Scaling module
!
!***************************************************************************************************
!   Fortran90
!   2018-06-11
!***************************************************************************************************
!   Bernd Freytag:         Uppsala
!***************************************************************************************************
! MODULES:
!   rhd_scale_module:      Scale control parameters and model data with time
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Example --- */
# ifndef rhd_scale_l01
#   define rhd_scale_l01 1
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------****************-----------------------------------------------------------------------------
module rhd_scale_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_scale_module ('rhd_scale_module')
!
! PURPOSE:
!   Scale control parameters and model data with time.
!
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   use rhd_scale_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_scale_SwitchInfo:  Print information about used compiler switches
!   rhd_scale_Init:        Initialization of module
!   rhd_scale_Step:        Scale model for one time step
!   rhd_scale_Final:       Finalization of module
!
! MODIFICATION HISTORY:
!   2015-05-02 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhd_scale_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_scale_SwitchInfo ('rhd_scale_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_scale_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_scale_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)')    'Compiler switches: rhd_scale_module .......................'
  !
  write(ncp,'(A,I6)') '  rhd' // '_scale_l01:        ', rhd_scale_l01
  !
!#
# ifdef rhd_scale_t01
!#
  write(ncp,'(A,A)' ) '  rhd' // '_scale_t01:           ', defstr
!#
# else
!#
  write(ncp,'(A,A)' ) '  rhd' // '_scale_t01:           ', undefstr
!#
# endif
!#
endif
!
end subroutine rhd_scale_SwitchInfo


!----------**************---------------------------------------------------------------------------
subroutine rhd_scale_Init(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_scale_Init ('rhd_scale_Initialize')
!
! PURPOSE:
!   Initialization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_scale_Init(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Write some information about used parameters to channel ncp.
!
! EXAMPLE:
!   call rhd_scale_Init(6, model, action)
!
! MODIFICATION HISTORY:
!   2015-05-02 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Initialize scale module <<<<<<<<'
  !
  if (par2%A%IsRead_flag) then
    write(ncp,'(A,E14.6)')     'time_ScaleStart:                       ', par%time_ScaleStart
    write(ncp,'(A,E14.6)')     'time_ScaleEnd:                         ', par%time_ScaleEnd
    !
    ! \label{f90:time_scalestart__use1}
    if ((par%time_ScaleStart >= 0.0               ) .and. &
        (par%time_ScaleEnd   >= par%time_ScaleStart)       ) then
      ! --- Scaling parameters are properly specified: print what's going to happen ---
      !
      if (par%C_test5 > 0.0) then
        ! --- Density scaling is requested ---
        write(ncp,'(A,E14.6)') 'Adjust entropy by C_test4:             ', par%C_test4
        write(ncp,'(A,E14.6)') 'Scale total mass by C_test5:           ', par%C_test5
      endif
      !
      if (par2%Teff /= par1%Teff) then
        ! --- A change of the effective temperature is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale effective temperature Teff from: ', par1%Teff, &
                                                                 ' to: ', par2%Teff
      endif
      !
      if (par2%mass_star /= par1%mass_star) then
        ! --- A change of the stellar mass is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale stellar mass mass_star from:     ', par1%mass_star, &
                                                                 ' to: ', par2%mass_star
      endif
      !
      if (par2%r0_grav /= par1%r0_grav) then
        ! --- A change of the central radius is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale central radius r0_grav from:     ', par1%r0_grav, &
                                                                 ' to: ', par2%r0_grav
      endif
      !
      if (par2%C_gravCorer0Heat /= par1%C_gravCorer0Heat) then
        ! --- A change of the central heating radius fraction is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                            'Scale central radius C_gravCorer0Heat from: ', par1%C_gravCorer0Heat, &
                                                                   ' to: ', par2%C_gravCorer0Heat
      endif
      !
      if (par2%C_gravCorePotDepth /= par1%C_gravCorePotDepth) then
        ! --- A change of the core potential depth factor is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale central C_gravCorePotDepth from: ', par1%C_gravCorePotDepth, &
                                                                 ' to: ', par2%C_gravCorePotDepth
      endif
      !
      if (par2%LuminosityPerVolume /= par1%LuminosityPerVolume) then
        ! --- A change of the stellar mass is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale LuminosityPerVolume from:        ', par1%LuminosityPerVolume,&
                                                                 ' to: ', par2%LuminosityPerVolume
      endif
      !
      if (par2%nu_rotation /= par1%nu_rotation) then
        ! --- A change of the rotation rate is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale rotation rate nu_rotation from:  ', par1%nu_rotation, &
                                                                 ' to: ', par2%nu_rotation
      endif
      !
      if (par2%C_CoreDrag /= par1%C_CoreDrag) then
        ! --- A change of the core drag force is requested ---
        write(ncp,'(2(A,1PE14.6))') &
                               'Scale core drag C_CoreDrag from:       ', par1%C_CoreDrag, &
                                                                 ' to: ', par2%C_CoreDrag
      endif
      !
      ! --- r1_grav, r2_grav, grav? ---
      ! --- s_InFlow, C_sChange?    ---
      ! --- C_vis?, C/N_test?       ---
      ! ...
      !
    else
      write(ncp,'(A)')         'No proper time interval specified: don''t scale anything'
    endif
  else
    write(ncp,'(A)')           'No 2nd parameter file found: don''t scale anything'
  endif
  !
  write(ncp,*)
endif ! (ncp)
!
end subroutine rhd_scale_Init


!----------**************---------------------------------------------------------------------------
subroutine rhd_scale_Step(ncp, model, action, dtime, outstr, ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_scale_Step ('rhd_scale_Step')
!
! PURPOSE:
!   Scale control parameters and model data for one time step.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_scale_Step(ncp, model, action, dtime, outstr=outstr, ierr=ierr)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') type (box_type), model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   For the computation of the total mass, an equidistant grid is assumed.
!   Scaling is controlled with test parameters par%C_test4 and par%C_test5, so far.
!
! PROCEDURE:
!                           + Value2 (par2%Value)
!                          /
!                         /
!                        /
!                       + ValueNew (par%Value)
!                      /                ^
!                     + ValueOld   (par%Value)
!                    /
!                   /
!                  + Value1 (par1%Value)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2015-05-02 (B.F. Uppsala) Written
!   2017-09-23 (B.F.) Change formula for r0_grav scaling
!   2017-09-24 (B.F.) Bugfix in limiting of mu_time
!   2017-10-23 (B.F.) Adjust internal energy when density is scaled
!   2017-11-25 (B.F.) Add C_CoreDrag
!   2018-04-24 (B.F.) Add C_gravCorePotDepth
!   2018-06-11 (B.F.) Add C_gravCorer0Heat; rearrange some scalings
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
use gasinter_module
!#
# ifdef rhd_scale_t01 
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: drhoOrho, rho_factor, rho_pre, rho_post, &
                                           ds, rhoei_pre, rhoei_post, &
                                           volume_per_cell, &
                                           mu_time, &
                                           Value1, Value2, ValueOld, ValueNew, Scale_factor
real(kind=kind(0.0D+00))                :: Volume_factor
real, dimension(model%m1:model%n1)      :: P, T, dxb1rel
real, dimension(model%m2:model%n2)      :: dxb2rel
real, dimension(model%m3:model%n3)      :: dxb3rel
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- Indicate that no error occurred so far ---
ierr0=0
outstr0=''
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! \label{f90:time_scalestart__use2}
if ((par2%A%IsRead_flag                        ) .and. &
    (par%time_ScaleStart >= 0.0                ) .and. &
    (par%time_ScaleEnd   >= par%time_ScaleStart) .and. &
    (model%time          >  par%time_ScaleStart)        ) then
  ! --- Scaling parameters are properly specified: check what to scale and do it ---
  !
!#
# ifdef rhd_scale_t01 
!#
  call timing_start('SCALE:')
!#
# endif
!#
  !
  if ((par%C_test5 >  0.0              ) .and. &
      (model%time  <= par%time_ScaleEnd)        ) then
    ! === Scale the density everywhere (i.e., the total mass) and adjust the internal energy =======
    !
    Scale_factor =1.0/prop%length
    Volume_factor=real(prop%length, kind=kind(0.0D+00))**3
    dxb1rel=(model%xb1(m1+1:n1+1)-model%xb1(m1:n1)) * Scale_factor
    dxb2rel=(model%xb2(m2+1:n2+1)-model%xb2(m2:n2)) * Scale_factor
    dxb3rel=(model%xb3(m3+1:n3+1)-model%xb3(m3:n3)) * Scale_factor
    !
    ! --- Compute density factor and entropy change for time step ---
    drhoOrho  =max(0.0 + alog(par%C_test5) * (dtime/(par%time_ScaleEnd-par%time_ScaleStart)), -1.0)
    rho_factor=drhoOrho + 1.0
    ds        =               par%C_test4  * (dtime/(par%time_ScaleEnd-par%time_ScaleStart))
    !
    rho_pre   =0.0
    rho_post  =0.0
    rhoei_pre =0.0
    rhoei_post=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, P, T, volume_per_cell) &
    !$OMP    REDUCTION(+: rho_pre, rho_post, rhoei_pre, rhoei_post)
    do i3=m3,n3
      do i2=m2,n2
        ! --- Compute pressure P and temperature T with equation of state for one vector ---
        call eosinter_PT(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), P, T)
        !
        ! --- Modify density rho and internal energy ei ---
        do i1=m1,n1
          ! volume_per_cell=((model%xb1(i1+1)-model%xb1(i1)) * Scale_factor) * &
          !                 ((model%xb2(i2+1)-model%xb2(i2)) * Scale_factor) * &
          !                 ((model%xb3(i3+1)-model%xb3(i3)) * Scale_factor)
          volume_per_cell=dxb1rel(i1) * dxb2rel(i2) * dxb3rel(i3)
          !
          rho_pre            =rho_pre   + model%rho(i1,i2,i3)                   *volume_per_cell
          rhoei_pre          =rhoei_pre + model%rho(i1,i2,i3)*model%ei(i1,i2,i3)*volume_per_cell
          !
          model%rho(i1,i2,i3)=model%rho(i1,i2,i3) * rho_factor
          model%ei( i1,i2,i3)=model%ei( i1,i2,i3) + P(i1)/model%rho(i1,i2,i3) * drhoOrho &
                                                  + T(i1)                     * ds
          !
          rho_post           =rho_post  + model%rho(i1,i2,i3)                   *volume_per_cell
          rhoei_post         =rhoei_post+ model%rho(i1,i2,i3)*model%ei(i1,i2,i3)*volume_per_cell
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
    if (ncp >= 0) then
      ! --- Write result for master ---
      write(ncp,'(A,E15.7,A,F10.7,A,E15.7)') &
        'Scale step: box mass    = ', real(rho_pre   , kind=kind(0.0D+00))*Volume_factor, &
                               ' * ', rho_factor, &
                              ' -> ', real(rho_post  , kind=kind(0.0D+00))*Volume_factor
      write(ncp,'(A,E15.7,A,E15.7)') &
        '            total energy= ', real(rhoei_pre , kind=kind(0.0D+00))*Volume_factor, &
                 '              -> ', real(rhoei_post, kind=kind(0.0D+00))*Volume_factor
    endif
    !
  endif ! (par%C_test5 > 0.0)
  !
  ! --- Compute time-interpolation parameter (account for zero-length interval) ---
  if (par%time_ScaleEnd > par%time_ScaleStart) then
    mu_time=max(min((model%time       -par%time_ScaleStart) / &
                    (par%time_ScaleEnd-par%time_ScaleStart), &
                    1.0), &
                0.0)
  else
    mu_time=1.0
  endif
  !
  if (par2%Teff /= par1%Teff) then
    ! === Change the effective temperature in the control-parameter structure ======================
    !
    Value1       =par1%Teff
    Value2       =par2%Teff
    ValueOld     = par%Teff
    ValueNew     =Value1 * sqrt(sqrt(1.0 + ((Value2/Value1)**4-1.0) * mu_time))
    par%Teff     =ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: Teff=              ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%Teff /= par1%Teff)
  !
  if (par2%mass_star /= par1%mass_star) then
    ! === Change the stellar mass in the control-parameter structure ===============================
    !
    Value1       =par1%mass_star
    Value2       =par2%mass_star
    ValueOld     = par%mass_star
    ValueNew     =Value1 * exp((alog(Value2)-alog(Value1)) * mu_time)
    par%mass_star=ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: mass_star=         ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%mass_star /= par1%mass_star)
  !
  if (par2%r0_grav /= par1%r0_grav) then
    ! === Change the central radius in the control-parameter structure =============================
    !
    Value1          =par1%r0_grav
    Value2          =par2%r0_grav
    ValueOld        = par%r0_grav
    ValueNew        =Value1 / (1.0 + ((Value1/Value2)**3.0 - 1.0) * mu_time)**(1.0/3.0)
    par%r0_grav     =ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: r0_grav=           ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%r0_grav /= par1%r0_grav)
  !
  if (par2%C_gravCorer0Heat /= par1%C_gravCorer0Heat) then
    ! === Change the central radius fraction for heating in the control-parameter structure ========
    !
    Value1              =merge(par1%C_gravCorer0Heat, 1.0, (par1%C_gravCorer0Heat > 0.0))
    Value2              =merge(par2%C_gravCorer0Heat, 1.0, (par2%C_gravCorer0Heat > 0.0))
    ValueOld            =       par%C_gravCorer0Heat
    ValueNew            =Value1 / (1.0 + ((Value1/Value2)**3.0 - 1.0) * mu_time)**(1.0/3.0)
    par%C_gravCorer0Heat=ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: C_gravCorer0Heat=  ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%C_gravCorer0Heat /= par1%C_gravCorer0Heat)
  !
  if (par2%C_gravCorePotDepth /= par1%C_gravCorePotDepth) then
    ! === Change the core potential depth factor in the control-parameter structure ================
    !
    Value1                =max(par1%C_gravCorePotDepth,1.0)
    Value2                =max(par2%C_gravCorePotDepth,1.0)
    ValueOld              =     par%C_gravCorePotDepth
    ValueNew              =Value1 * exp((alog(Value2)-alog(Value1)) * mu_time)
    par%C_gravCorePotDepth=ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: C_gravCorePotDepth=' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%C_gravCorePotDepth /= par1%C_gravCorePotDepth)
  !
  if (par2%LuminosityPerVolume /= par1%LuminosityPerVolume) then
    ! === Change the central radius in the control-parameter structure =============================
    !
    Value1                 =par1%LuminosityPerVolume
    Value2                 =par2%LuminosityPerVolume
    ValueOld               = par%LuminosityPerVolume
    ValueNew               =Value1 + (Value2-Value1) * mu_time
    par%LuminosityPerVolume=ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: LumPerVolume=      ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%LuminosityPerVolume /= par1%LuminosityPerVolume)
  !
  if (par2%nu_rotation /= par1%nu_rotation) then
    ! === Change the rotation rate in the control-parameter structure ==============================
    !
    Value1         =par1%nu_rotation
    Value2         =par2%nu_rotation
    ValueOld       = par%nu_rotation
    ValueNew       =Value1 + (Value2-Value1) * mu_time
    par%nu_rotation=ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: nu_rotation=       ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%nu_rotation /= par1%nu_rotation)
  !
  if (par2%C_CoreDrag /= par1%C_CoreDrag) then
    ! === Change the core drag force in the control-parameter structure ============================
    !
    Value1        =par1%C_CoreDrag
    Value2        =par2%C_CoreDrag
    ValueOld      = par%C_CoreDrag
    ValueNew      =Value1 + (Value2-Value1) * mu_time
    par%C_CoreDrag=ValueNew
    !
    if (ncp >= 0) then
      write(ncp,'(4(A,1PE15.7),A)') &
        'Scale step: C_CoreDrag=        ' // &
                                 '(', Value1, '), ', &
                                      ValueOld, &
                              ' -> ', ValueNew, &
                               ', (', Value2, ')'
    endif
  endif ! (par2%C_CoreDrag /= par1%C_CoreDrag)
  !
  ! --- Add more parameter scalings here ---
  ! ...
  !
  !
  outstr0='Scaling step done'
  !
!#
# ifdef rhd_scale_t01 
!#
  call timing_stop('SCALE:')
!#
# endif
!#
else ! --- Scaling parameters not set ---
  !
  if (ncp >= 0) then
    write(ncp,'(A)') 'Skip scaling step'
  endif
endif ! --- Scaling ---
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_scale_Step


!----------***************--------------------------------------------------------------------------
subroutine rhd_scale_Final(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_scale_Final ('rhd_scale_Finalize')
!
! PURPOSE:
!   Finalization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_scale_Final(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write message about finalization of scale step to channel ncp.
!
! EXAMPLE:
!   call rhd_scale_Final(6, model, action)
!
! MODIFICATION HISTORY:
!   2015-05-02 (B.F.) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Finalize scale module <<<<<<<<'
  write(ncp,*)
endif
!
end subroutine rhd_scale_Final

end module rhd_scale_module
