!***************************************************************************************************
!
!   #####   #    #  #####         #####  #    #  ######    ##    #    #
!   #    #  #    #  #    #          #    #    #  #        #  #   #   #
!   #    #  ######  #    #          #    #    #  #####   #    #  ####
!   #####   #    #  #    #          #    # ## #  #       ######  #  #
!   #   #   #    #  #    #          #    ##  ##  #       #    #  #   #
!   #    #  #    #  #####  #######  #    #    #  ######  #    #  #    # _module
!
!   Radiation-Hydrodynamics-Code: tweak module
!
!***************************************************************************************************
!   Fortran90
!   2016-03-16
!***************************************************************************************************
!   Bernd Freytag:         Uppsala, Montpellier
!***************************************************************************************************
! MODULES:
!   rhd_tweak_module:      Reduce the mass by a tiny amount per time step
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Example --- */
# ifndef rhd_tweak_l01
#   define rhd_tweak_l01 1
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------****************-----------------------------------------------------------------------------
module rhd_tweak_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_module ('rhd_tweak_module')
!
! PURPOSE:
!   Reduce the mass by a tiny amount per time step.
!
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   use rhd_tweak_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_tweak_SwitchInfo:  Print information about used compiler switches
!   rhd_tweak_Init:        Initialization of module
!   rhd_tweak_Step:        Tweak model for one time step
!   rhd_tweak_Final:       Finalization of module
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) First version
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhd_tweak_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_SwitchInfo ('rhd_tweak_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_tweak_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)')    'Compiler switches: rhd_tweak_module .......................'
  !
  write(ncp,'(A,I6)') '  rhd' // '_tweak_l01:        ', rhd_tweak_l01
  !
!#
# ifdef rhd_tweak_t01
!#
  write(ncp,'(A,A)' ) '  rhd' // '_tweak_t01:           ', defstr
!#
# else
!#
  write(ncp,'(A,A)' ) '  rhd' // '_tweak_t01:           ', undefstr
!#
# endif
!#
endif ! (ncp)
!
end subroutine rhd_tweak_SwitchInfo


!----------**************---------------------------------------------------------------------------
subroutine rhd_tweak_Init(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Init ('rhd_tweak_Initialize')
!
! PURPOSE:
!   Initialization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Init(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write some information about used parameters to channel ncp.
!
! EXAMPLE:
!   call rhd_tweak_Init(6, model, action)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  !write(ncp,'(A)') 'ACTION: Initialization of tweak demo module <<<<<<<<'
  !write(ncp,'(A)') 'This demo module does nothing to the model data.'
  write(ncp,'(A)') 'ACTION: Initialize tweak module <<<<<<<<'
  write(ncp,'(A,E13.5)') 'Reduce mass in each cell by dtime/', par%C_test1
  write(ncp,*)
endif ! (ncp)
!
end subroutine rhd_tweak_Init


!----------**************---------------------------------------------------------------------------
subroutine rhd_tweak_Step(ncp, model, action, dtime, outstr, ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Step ('rhd_tweak_Step')
!
! PURPOSE:
!   Reduce density in each cell by a tiny amount per time step.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Step(ncp, model, action, dtime, outstr=outstr, ierr=ierr)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') type (box_type), model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!   2004-05-22 (B.F.) Mass reduction
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!#
# ifdef rhd_tweak_t01 
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: red_factor, volume_per_cell, &
                                           rho_pre, rho_pre_thread, rho_post, rho_post_thread
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- Indicate that no error occured so far ---
ierr0=0
outstr0=''
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
!write(ncp,'(A)') 'Tweak demo step'
!write(ncp,'(A)') ' Don''t do anything at all!'
!
if (par%C_test1 > 0.0) then
  !
!#
# ifdef rhd_tweak_t01 
!#
  call timing_start('TWEAK: mass reduction')
!#
# endif
!#
  if (ncp >= 0) then
    write(ncp,'(A)') 'Tweak step'
  endif
  !
  ! --- Compute reduction factor for time step ---
  red_factor=max(1.0 - dtime/par%C_test1, 0.0)
  !
  rho_pre =0.0
  rho_post=0.0
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP   PRIVATE(i1, i2, rho_pre_thread, rho_post_thread)
  !
  rho_pre_thread =0.0
  rho_post_thread=0.0
  !
  !$OMP DO
  do i3=m3, n3
    do i2=m2, n2
      do i1=m1, n1
        rho_pre_thread     =rho_pre_thread  + model%rho(i1,i2,i3)
        model%rho(i1,i2,i3)=model%rho(i1,i2,i3)*red_factor
        rho_post_thread    =rho_post_thread + model%rho(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  !$OMP CRITICAL
  rho_pre =rho_pre + rho_pre_thread
  rho_post=rho_post+ rho_post_thread
  !$OMP END CRITICAL
  !
  !$OMP END PARALLEL
  !
  ! --- Compute total mass from integrated density ---
  volume_per_cell=(model%xb1(m1+1)-model%xb1(m1)) * &
                  (model%xb2(m2+1)-model%xb2(m2)) * &
                  (model%xb3(m3+1)-model%xb3(m3))
  rho_pre =rho_pre *volume_per_cell
  rho_post=rho_post*volume_per_cell
  !
  if (ncp >= 0) then
    write(ncp,'(A,1PE15.6,A,1PE15.6)') '  Total mass: ', rho_pre, ' -> ', rho_post
  endif
  !
  outstr0='mass reduction done'
!#
# ifdef rhd_tweak_t01 
!#
  call timing_stop('TWEAK: mass reduction')
!#
# endif
!#
else
  !
  if (ncp >= 0) then
    write(ncp,'(A)') 'Skip tweak step'
  endif
endif
!
!write(ncp,*)
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_tweak_Step


!----------***************--------------------------------------------------------------------------
subroutine rhd_tweak_Final(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Final ('rhd_tweak_Finalize')
!
! PURPOSE:
!   Finalization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Final(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write message about finalization of tweak step to channel ncp.
!
! EXAMPLE:
!   call rhd_tweak_Final(6, model, action)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Finalize tweak module <<<<<<<<'
  write(ncp,*)
endif ! (ncp)
!
end subroutine rhd_tweak_Final

end module rhd_tweak_module
