!***************************************************************************************************
!
!   #####   #    #  #####         #####  #    #  ######    ##    #    #
!   #    #  #    #  #    #          #    #    #  #        #  #   #   #
!   #    #  ######  #    #          #    #    #  #####   #    #  ####
!   #####   #    #  #    #          #    # ## #  #       ######  #  #
!   #   #   #    #  #    #          #    ##  ##  #       #    #  #   #
!   #    #  #    #  #####  #######  #    #    #  ######  #    #  #    # _module
!
!   Radiation-Hydrodynamics-Code: tweak module
!
!***************************************************************************************************
!   Fortran90
!   2016-03-16
!***************************************************************************************************
!   Bernd Freytag:         Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_tweak_module:      Damp plane-parallel and/or all motions
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Example --- */
# ifndef rhd_tweak_l01
#   define rhd_tweak_l01 1
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------****************-----------------------------------------------------------------------------
module rhd_tweak_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_module ('rhd_tweak_module')
!
! PURPOSE:
!   Damp plane-parallel and/or all motions.
!
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   use rhd_tweak_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_tweak_SwitchInfo:  Print information about used compiler switches
!   rhd_tweak_Init:        Initialization of module
!   rhd_tweak_Step:        Tweak model for one time step
!   rhd_tweak_Final:       Finalization of module
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) First version
!   2004-06-01 (B.F.) Damp plane-parallel (vertical) modes
!   2008-02-07 (B.F. Lyon) Damp plane-parallel (horizontal) modes
!   2008-03-13 (B.F.) Damp everything in layers near the top with drag force
!   2009-04-02 (B.F.) Allow gradual onset of damping
!   2014-12-20 (B.F. Uppsala) Option to reduce update of int. energy in total-damping step (C_test4)
!   2015-01-03 (B.F. Kiel) Option for vertical-only damping (controlled with par%strtest1)
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhd_tweak_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_SwitchInfo ('rhd_tweak_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_tweak_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)')    'Compiler switches: rhd_tweak_module .......................'
  !
  write(ncp,'(A,I6)') '  rhd' // '_tweak_l01:              ', rhd_tweak_l01
  !
!#
# ifdef rhd_tweak_t01
!#
  write(ncp,'(A,A)' ) '  rhd' // '_tweak_t01:           ', defstr
!#
# else
!#
  write(ncp,'(A,A)' ) '  rhd' // '_tweak_t01:           ', undefstr
!#
# endif
!#
endif ! (ncp)
!
end subroutine rhd_tweak_SwitchInfo


!----------**************---------------------------------------------------------------------------
subroutine rhd_tweak_Init(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Init ('rhd_tweak_Initialize')
!
! PURPOSE:
!   Initialization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Init(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write some information about used parameters to channel ncp.
!
! EXAMPLE:
!   call rhd_tweak_Init(6, model, action)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Initialize tweak module <<<<<<<<'
  write(ncp,'(A,1PE13.5)') 'Damp plane-parallel modes (v3 ) on time-scale C_test1=', par%C_test1
  write(ncp,'(A,1PE13.5)') 'Damp plane-parallel modes (v12) on time-scale C_test2=', par%C_test2
  write(ncp,'(A,I13)'    ) '  in points from bottom                       N_test1=', par%N_test1
  write(ncp,'(A,1PE13.5)') 'Damp everything on time-scale                 C_test3=', par%C_test3
  write(ncp,'(A,A)'      ) '  acting on direction(s)                     strtest1= ', &
                                                                                  trim(par%strtest1)
  write(ncp,'(A,1PE13.5)') '  with energy-update fraction                 C_test4=', par%C_test4
  write(ncp,'(A,I13)'    ) '  in points from top                          N_test2=', par%N_test2
  write(ncp,'(A,I13)'    ) '  with linear transition over points          N_test3=', par%N_test3
  write(ncp,*)
endif ! (ncp)
!
end subroutine rhd_tweak_Init


!----------**************---------------------------------------------------------------------------
subroutine rhd_tweak_Step(ncp, model, action, dtime, outstr, ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Step ('rhd_tweak_Step')
!
! PURPOSE:
!   Damp plane-parallel and/or all motions.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Step(ncp, model, action, dtime, outstr=outstr, ierr=ierr)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') type (box_type), model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute mean vertical (v3) and horizontal (v1, v2) velocity from mean mass fluxes and
!   mean mass for each layer.
!   Subtract some fraction of it from the velocity of each point.
!   Additionally, damp local velocities (all or vertical-only) in top layers.
!   Control parameters for damping of plane-parallel (usually global) modes:
!   C_test1:      ('Constant_test_1') real with damping time scale for plane-parallel v3 modes
!                 the vertical sound-crossing time through the computational box can serve as
!                 a guide line for a typical value,
!                 a value <=0.0 switches off this damping
!   C_test2:      ('Constant_test_2') real with damping time scale for plane-parallel v1,v2 modes,
!                 a typical value is much larger than C_test1 or even 0.0 to switch off this damping
!   N_test1:      ('N_test_1) integer with number of damped layers counted from bottom:
!                 <=0: Switch off plane-parallel damping (default)
!                 >0 : Activate plane-parallel damping in this number of layers counted from bottom,
!                      A very large number means all layers are considered
!   Additionally, damp local velocities (all or vertical-only) in top layers.
!   Control parameters for top-layer damping:
!   C_test3:      ('Constant_test_3') real with damping time scale for top-layer flows,
!                 the vertical sound-crossing time through the top drag layers can serve as
!                 a guide line for a typical value
!   C_test4:      ('Constant_test_4') real with energy-update fraction for internal energy
!                 0.0      : internal energy is not updated, the damping does not cause any heating
!                 0.0<C<1.0: the dissipated kinetic energy is partly added to the internal energy
!                 1.0      : the dissipated kinetic energy is fully added to the internal energy
!   strtest1:     ('string_test_1') character with direction of motion to be damped:
!                 'All', ''           : All components are damped with the same rate (old, default)
!                 'Radial', 'Vertical': Only vertical motions (v3) are damped
!   N_test2:      ('N_test_2) integer with number of damped layers counted from top,
!                 typical values are 8 to 20
!   N_test3:      ('N_test_3) integer with number of linear-transition layers,
!                 a typical value is N_test2/2
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!   2004-05-22 (B.F.) Mass reduction
!   2004-06-01 (B.F.) Damping of plane-parallel (vertical) modes
!   2008-02-07 (B.F.) Damp plane-parallel (horizontal) modes
!   2009-04-02 (B.F.) Allow gradual onset of damping
!   2014-12-20 (B.F.) Allow non-complete update of internal energy in total-damping step (C_test4)
!   2015-01-03 (B.F. Kiel) Option for vertical-only damping (controlled with par%strtest1)
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!#
# ifdef rhd_tweak_t01 
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3, m3s
real                                    :: rho_mean, rhov_mean, v_mean, &
                                           red_factorv3, red_factorv1, v_factor_drag, &
                                           TransLayerThickness, eiUpdateFactor
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: area
logical                                 :: tweak_flag
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- Indicate that no error occurred so far ---
ierr0=0
outstr0=''
tweak_flag=.false.
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! === Damp plane-parallel modes ===
if (((par%C_test1 > 0.0) .or. (par%C_test2 > 0.0)) .and. &
     (par%N_test1 > 0  )                                  ) then
  !
!#
# ifdef rhd_tweak_t01 
!#
  call timing_start('TWEAK: pp damping')
!#
# endif
!#
  if (ncp >= 0) then
    write(ncp,'(A)') 'Tweak step: pp damping'
  endif
  !
  ! --- Compute relative cell (column) areas ---
  do i2=m2,n2
    do i1=m1,n1
      area(i1,i2)= (model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2)) / &
                  ((model%xb1(n1+1)-model%xb1(m1)) * (model%xb2(n2+1)-model%xb2(m2)))
    end do ! i1
  end do ! i2
  !
  ! --- Compute reduction factor for time step ---
  if (par%C_test1 > 0.0) then
    red_factorv3=dtime/par%C_test1
  else
    red_factorv3=0.0
  endif
  if (par%C_test2 > 0.0) then
    red_factorv1=dtime/par%C_test2
  else
    red_factorv1=0.0
  endif
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP   PRIVATE(rho_mean, rhov_mean, v_mean)
  !
  !$OMP DO
  do i3=m3, min(n3,m3+par%N_test1-1)
    rho_mean                =sum(area*model%rho(m1:n1,m2:n2,i3)                         )
    rhov_mean               =sum(area*model%rho(m1:n1,m2:n2,i3)*model%v1(m1:n1,m2:n2,i3))
    v_mean                  =rhov_mean/rho_mean
    model%v1(m1:n1,m2:n2,i3)=model%v1(m1:n1,m2:n2,i3) - v_mean*red_factorv1
    rhov_mean               =sum(area*model%rho(m1:n1,m2:n2,i3)*model%v2(m1:n1,m2:n2,i3))
    v_mean                  =rhov_mean/rho_mean
    model%v2(m1:n1,m2:n2,i3)=model%v2(m1:n1,m2:n2,i3) - v_mean*red_factorv1
    rhov_mean               =sum(area*model%rho(m1:n1,m2:n2,i3)*model%v3(m1:n1,m2:n2,i3))
    v_mean                  =rhov_mean/rho_mean
    model%v3(m1:n1,m2:n2,i3)=model%v3(m1:n1,m2:n2,i3) - v_mean*red_factorv3
  end do ! i3
  !$OMP END DO
  !
  !$OMP END PARALLEL
  !
  outstr0='plane-parallel damping step done'
  tweak_flag=.true.
!#
# ifdef rhd_tweak_t01 
!#
  call timing_stop('TWEAK: pp damping')
!#
# endif
!#
endif
!
! === Damp everything ===
if ((par%C_test3 > 0.0) .and. (par%N_test2 > 0)) then
  !
!#
# ifdef rhd_tweak_t01 
!#
  call timing_start('TWEAK: top-layer drag force')
!#
# endif
!#
  !
  ! --- Determine start index for damped region ---
  m3s=max(m3,n3-par%N_test2+1)
  ! --- Compute thickness of transition layer (minimum=1 means no points) ---
  TransLayerThickness=real(min(max(par%N_test3, 0), n3-m3s)+1)
  ! --- Set update factor for internal energy (BF: 2014-12-20) ---
  eiUpdateFactor=min(max(par%C_test4, 0.0), 1.0)
  !
  if ((par%strtest1 == 'radial'  ) .or. &
      (par%strtest1 == 'Radial'  ) .or. &
      (par%strtest1 == 'vertical') .or. &
      (par%strtest1 == 'Vertical')       ) then
    !
    if (ncp >= 0) then
      write(ncp,'(A)') 'Tweak step: top-layer vertical drag force'
    endif
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, v_factor_drag)
    do i3=m3s, n3
      ! --- Set reduction factor for time step (now possibly height dependent) ---
      !v_factor_drag=max(1.0-dtime/par%C_test3, 0.0)
      v_factor_drag=max(1.0-dtime/par%C_test3*min(real(i3-m3s+1)/TransLayerThickness, 1.0), 0.0)
      do i2=m2,n2
        do i1=m1,n1
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + (0.5 * eiUpdateFactor * &
                                                   (1.0 - v_factor_drag**2)) * &
                                                  (model%v3(i1,i2,i3)**2    )
          model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  else ! (par%strtest1)
    !
    if (ncp >= 0) then
      write(ncp,'(A)') 'Tweak step: top-layer drag force'
    endif
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, v_factor_drag)
    do i3=m3s, n3
      ! --- Set reduction factor for time step (now possibly height dependent) ---
      !v_factor_drag=max(1.0-dtime/par%C_test3, 0.0)
      v_factor_drag=max(1.0-dtime/par%C_test3*min(real(i3-m3s+1)/TransLayerThickness, 1.0), 0.0)
      do i2=m2,n2
        do i1=m1,n1
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + (0.5 * eiUpdateFactor * &
                                                   (1.0 - v_factor_drag**2)) * &
                                                  (model%v1(i1,i2,i3)**2 + &
                                                   model%v2(i1,i2,i3)**2 + &
                                                   model%v3(i1,i2,i3)**2    )
          model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
          model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
          model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  endif ! (par%strtest1)
  !
  outstr0='Top-layer drag force step done'
  tweak_flag=.true.
!#
# ifdef rhd_tweak_t01 
!#
  call timing_stop('TWEAK: top-layer drag force')
!#
# endif
!#
endif
!
if (.not. tweak_flag) then
  !
  if (ncp >= 0) then
    write(ncp,'(A)') 'Skip tweak step'
  endif
endif
!
!write(ncp,*)
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_tweak_Step


!----------***************--------------------------------------------------------------------------
subroutine rhd_tweak_Final(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Final ('rhd_tweak_Finalize')
!
! PURPOSE:
!   Finalization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Final(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write message about finalization of tweak step to channel ncp.
!
! EXAMPLE:
!   call rhd_tweak_Final(6, model, action)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Finalize tweak module <<<<<<<<'
  write(ncp,*)
endif ! (ncp)
!
end subroutine rhd_tweak_Final

end module rhd_tweak_module
