!***************************************************************************************************
!
!   #####   #    #  #####         #####  #    #  ######    ##    #    #
!   #    #  #    #  #    #          #    #    #  #        #  #   #   #
!   #    #  ######  #    #          #    #    #  #####   #    #  ####
!   #####   #    #  #    #          #    # ## #  #       ######  #  #
!   #   #   #    #  #    #          #    ##  ##  #       #    #  #   #
!   #    #  #    #  #####  #######  #    #    #  ######  #    #  #    # _module
!
!   Radiation-Hydrodynamics-Code: tweak module
!
!***************************************************************************************************
!   Fortran90
!   2016-03-16
!***************************************************************************************************
!   Bernd Freytag:         Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_tweak_module:      Damp velocities in regions with very small density
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Example --- */
# ifndef rhd_tweak_l01
#   define rhd_tweak_l01 1
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------****************-----------------------------------------------------------------------------
module rhd_tweak_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_module ('rhd_tweak_module')
!
! PURPOSE:
!   Damp velocities in regions with very small density.
!
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   use rhd_tweak_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_tweak_SwitchInfo:  Print information about used compiler switches
!   rhd_tweak_Init:        Initialization of module
!   rhd_tweak_Step:        Tweak model for one time step
!   rhd_tweak_Final:       Finalization of module
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) First version
!   2004-06-01 (B.F.) Damp motions in regions with very small density
!   2013-07-10 (B.F. Lyon) More comments
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhd_tweak_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_SwitchInfo ('rhd_tweak_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_tweak_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!   2013-07-10 (B.F. Lyon) Check for positive channel number to allow printing by only one process
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)')    'Compiler switches: rhd_tweak_module .......................'
  !
  write(ncp,'(A,I6)') '  rhd' // '_tweak_l01:        ', rhd_tweak_l01
  !
!#
# ifdef rhd_tweak_t01
!#
  write(ncp,'(A,A)' ) '  rhd' // '_tweak_t01:           ', defstr
!#
# else
!#
  write(ncp,'(A,A)' ) '  rhd' // '_tweak_t01:           ', undefstr
!#
# endif
!#
endif
!
end subroutine rhd_tweak_SwitchInfo


!----------**************---------------------------------------------------------------------------
subroutine rhd_tweak_Init(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Init ('rhd_tweak_Initialize')
!
! PURPOSE:
!   Initialization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Init(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write some information about used parameters to channel ncp.
!
! EXAMPLE:
!   call rhd_tweak_Init(6, model, action)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!   2013-07-10 (B.F. Lyon) Check for positive channel number to allow printing by only one process
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Initialize tweak module <<<<<<<<'
  write(ncp,'(A,E13.5)') 'Damp motions on time-scale        ', par%C_test1
  write(ncp,'(A,E13.5)') 'in points with mass below rho_min*', par%C_test2
  write(ncp,'(A)'      ) 'and limit density from below.'
  write(ncp,*)
endif
!
end subroutine rhd_tweak_Init


!----------**************---------------------------------------------------------------------------
subroutine rhd_tweak_Step(ncp, model, action, dtime, outstr, ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Step ('rhd_tweak_Step')
!
! PURPOSE:
!   Tweak model for one time step.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Step(ncp, model, action, dtime, outstr=outstr, ierr=ierr)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') type (box_type), model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, tweak step successfully completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Reduce all velocity components in cells with very low mass.
!   Ignore energy conservation.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!   2004-05-22 (B.F.) Mass reduction
!   2004-06-01 (B.F.) Damping of plane-parallel modes
!   2004-08-18 (B.F.) Set density at least to minimu value par%rho_min
!   2013-07-10 (B.F. Lyon) Check for positive channel number to allow printing by only one process
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!#
# ifdef rhd_tweak_t01 
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: rho_thresh, red_factor
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
if ((par%C_test1 > 0.0) .and. (par%C_test2 > 0.0)) then
  !
!#
# ifdef rhd_tweak_t01 
!#
  call timing_start('TWEAK: small rho damping')
!#
# endif
!#
  if (ncp >= 0) write(ncp,'(A)') 'Tweak step'
  !
  ! --- Reduction factor for time step ---
  red_factor=max(1.0-dtime/par%C_test1, 0.0)
  rho_thresh=par%C_test2 * par%rho_min
  !
  !$OMP PARALLEL DEFAULT(SHARED)
  !
  !$OMP DO
  do i3=m3, n3
    do i2=m2, n2
      do i1=m1, n1
        if (model%rho(i1,i2,i3) <= rho_thresh) then
          model%v1(i1,i2,i3) =model%v1(i1,i2,i3) * red_factor
          model%v2(i1,i2,i3) =model%v2(i1,i2,i3) * red_factor
          model%v3(i1,i2,i3) =model%v3(i1,i2,i3) * red_factor
          model%rho(i1,i2,i3)=max(model%rho(i1,i2,i3), par%rho_min)
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  !$OMP END PARALLEL
  !
  outstr0='low mass damping step done'
!#
# ifdef rhd_tweak_t01 
!#
  call timing_stop('TWEAK: small rho damping')
!#
# endif
!#
else
  !
  if (ncp >= 0) write(ncp,'(A)') 'Skip tweak step'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_tweak_Step


!----------***************--------------------------------------------------------------------------
subroutine rhd_tweak_Final(ncp, model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_tweak_Final ('rhd_tweak_Finalize')
!
! PURPOSE:
!   Finalization of module.
!   
! CATEGORY:
!   Simulations
!
! CALLING SEQUENCE:
!   call rhd_tweak_Final(ncp, model, action)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_box_module:        Box-handling routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   Just write to channel ncp.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just write message about finalization of tweak step to channel ncp.
!
! EXAMPLE:
!   call rhd_tweak_Final(6, model, action)
!
! MODIFICATION HISTORY:
!   2004-05-21 (B.F.) Written
!   2013-07-10 (B.F. Lyon) Check for positive channel number to allow printing by only one process
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (ncp >= 0) then
  write(ncp,'(A)') 'ACTION: Finalize tweak module <<<<<<<<'
  write(ncp,*)
endif
!
end subroutine rhd_tweak_Final

end module rhd_tweak_module
