!***************************************************************************************************
!
!   #     # #######
!   #     #    #
!   #     #    #
!   #     #    #
!   #     #    #
!   #     #    #
!    #####     #  _module
!
!   Basic unit-test variables and routines
!
!***************************************************************************************************
!   Fortran90
!   2017-07-27
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   UT_module:             Global variables and basic routines for unit testing
!
!***************************************************************************************************

!# /* --- Set default values for some preprocessor macros --------------------------------------- */
!#
!# /* --- Activate color output: 0 (no color), 1 (color; default) --- */
# ifndef ut_switch_color
#   define ut_switch_color 1
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*********------------------------------------------------------------------------------------
module UT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_module ('Unit_Test_module')
!
! PURPOSE:
!   Provide global variables and basic routines for unit testing.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use UT_module
!
! TYPES:
!   UT_type:      ('Unit_Test_type') structure for global data for unit tests:
!     nTestTotal: ('number_Test_Total') counter for total number of performed tests
!     nTestFailed: ('number_Test_Failed') counter for number of failed tests
!     nc_p:       ('number_channel_print') integer with output channel number
!     nMax1:      ('number_Maximum_1') integer with maximum i1 number of elements to print
!     nMax2:      ('number_Maximum_2') integer with maximum i2 number of elements to print
!     nMax3:      ('number_Maximum_3') integer with maximum i3 number of elements to print
!     AllTestPrint_flag: ('All_Test_Print_flag') logical to indicate the amount of printed messages:
!                          .true.:  print a message for each test
!                          .false.: print a message only for failed tests (default)
!     AllVecInPrint_flag: ('All_Vector_Input_Print_flag') logical to indicate if the input vectors
!                         for tests should be printed:
!                          .true.:  print each input vector (according to the restrictions
!                                   by nMax1,2,3)
!                          .false.: don't print any test input data
!     AllVecOutPrint_flag: ('All_Vector_Output_Print_flag') logical to indicate if the
!                          output vectors of tests should be printed:
!                           .true.:  print each output vector (according to the restrictions
!                                    by nMax1,2,3)
!                           .false.: don't print any test output data
!                          The data is only printed if a test failed or AllTestPrint_flag=T.
!     LogicalFormat: ('Logical_Format') character with output format for logical variables
!     IntegerFormat: ('Integer_Format') character with output format for integers
!     RealFormat: ('Real_Format') character with output format for reals
!     CharacterFormat: ('Character_Format') character with output format for strings
!
! VARIABLES:
!   UT:           ('Unit_Test') structure for global unit-test data
!
! ROUTINES: (contained)
!   UT_Init:               Initialize unit-test counters and control variables
!   UT_PrintSummary:       Print values of unit-test counters and summarize tests
!   UT_PrintSeparator:     Print separator line
!   UT_PrintArray:         Print a 1D/2D/3D array
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_Assert:             Check array or relation between two arrays
!   UT_DemoExamplePassAll: Call a number of assert routines to perform tests that are all passed
!   UT_DemoExampleFailAll: Call a number of assert routines to perform tests that all fail
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-02-26 (B.F.) More comments
!   2015-03-01 (B.F.) Renamed and reorganized
!   2015-03-04 (B.F.) Renamed and reorganized
!   2015-03-06 (B.F.) Module put into own file
!   2016-04-16 (B.F.) Add routines to process integer arrays
!   2016-04-29 (B.F.) Add routines to process character arrays
!   2016-05-01 (B.F.) Add routines to process logical arrays
!   2017-05-18 (B.F.) Add routine to print separator line
!---------------------------------------------------------------------------------------------------
implicit none
!
type UT_type
  integer                               :: nTestTotal, nTestPassed, nTestFailed, &
                                           nc_p, &
                                           nMax1, nMax2, nMax3
  logical                               :: AllTestPrint_flag, &
                                           AllVecInPrint_flag, &
                                           AllVecOutPrint_flag
  character(len=80)                     :: LogicalFormat, IntegerFormat, RealFormat, CharacterFormat
end type UT_type
!
type(UT_type), save                     :: UT
!#
# if (ut_switch_color == 0)
!#
character(len=*), parameter             :: UT_NoColor='', &
                                           UT_PassedColor='', &
                                           UT_FailedColor='', &
                                           UT_FMarkColor='', &
                                           UT_SepColor=''
character(len=9), parameter             :: UT_PassedStr=' : passed', &
                                           UT_FailedStr=' : failed'
!#
# else
!#     /* ut_switch_color */
character(len=*), parameter             :: UT_NoColor='\e[0m', &
                                           UT_PassedColor='\e[0;34m', &
                                           UT_FailedColor='\e[101m', &
                                           UT_FMarkColor='\e[48;5;228m', &
                                           UT_SepColor='\e[0;37m'
!                                           UT_FMarkColor='\e[48;5;194m'
!                                           UT_FMarkColor='\e[48;5;225m'
!                                           UT_FMarkColor='\e[48;5;195m'
!                                           UT_FMarkColor='\e[48;5;221m'
character(len=30), parameter            :: UT_PassedStr=' : ' // UT_PassedColor // 'passed' // &
                                                        UT_NoColor, &
                                           UT_FailedStr=' : ' // UT_FailedColor // 'failed' // &
                                                        UT_NoColor, &
                                           UT_IgnoredFailStr=' : ignoredFail', &
                                           UT_IgnoredPassStr=' : ignoredPass'
!#
# endif
!#      /* ut_switch_color */
!
interface UT_PrintArray
  module procedure UT_PrintLogical1  , &
                   UT_PrintInteger1  , UT_PrintInteger2, UT_PrintInteger3, &
                   UT_PrintReal1     , UT_PrintReal2   , UT_PrintReal3   , &
                   UT_PrintCharacter1
end interface ! UT_PrintArray
!
interface UT_Assert
  module procedure UT_AssertLogical11  , &
                   UT_AssertInteger10  , UT_AssertInteger20, UT_AssertInteger30, &
                   UT_AssertInteger11  , UT_AssertInteger22, UT_AssertInteger33, &
                   UT_AssertReal10     , UT_AssertReal20   , UT_AssertReal30   , &
                   UT_AssertReal11     , UT_AssertReal22   , UT_AssertReal33   , &
                   UT_AssertCharacter11
end interface ! UT_Assert
!
contains

!----------*******----------------------------------------------------------------------------------
subroutine UT_Init()
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_Init ('Unit_Test_Initialize')
!
! PURPOSE:
!   Initialize unit-test counters and control variables.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_Init()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Set global counters, format strings, and output control variables.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_Init()
!
! MODIFICATION HISTORY:
!   2015-03-01 (B.F. Uppsala) Put into own routine
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Initialize unit-test counters ---
UT%nTestTotal         =0
UT%nTestPassed        =0
UT%nTestFailed        =0
!
! --- Initialize unit-test parameters ---
UT%nMax1              =15
UT%nMax2              =8
UT%nMax3              =2
UT%LogicalFormat      ='L3'
UT%IntegerFormat      ='I6'
UT%RealFormat         ='1PE13.6'
UT%CharacterFormat    ='A13'
UT%AllTestPrint_flag  =.false.
UT%AllVecInPrint_flag =.false.
UT%AllVecOutPrint_flag=.false.
!
end subroutine UT_Init


!----------***************--------------------------------------------------------------------------
subroutine UT_PrintSummary()
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintSummary ('Unit_Test_Print_Summary')
!
! PURPOSE:
!   Print values of unit-test counters and summarize tests.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintSummary()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintSummary()
!
! MODIFICATION HISTORY:
!   2015-03-01 (B.F. Uppsala) Put into own routine
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
character(len=80)                       :: MarkColor
!---------------------------------------------------------------------------------------------------
!
! --- Print test summary ---
if (UT%nTestTotal <= 0) then
  write(UT%nc_p, '(A)') 'No test was performed.'
  !
else
  if (UT%nTestFailed > 0) then
    MarkColor=UT_FailedColor
  else
    MarkColor=UT_PassedColor
  endif
  !
!  write(UT%nc_p, '(A,I10,A,I10,A,F7.3,A)') &
!    'Tests: # ' // trim(MarkColor) // 'failed: ', UT%nTestFailed, &
!    UT_NoColor // '   out of # total: '         , UT%nTestTotal, &
!    '   with fraction: ' // trim(MarkColor), &
!      float(UT%nTestFailed)/float(UT%nTestTotal)*100.0, ' %' // UT_NoColor
  !
  write(UT%nc_p, '(3(A,I10))') &
    'Tests: # ' // trim(MarkColor) // 'failed: ', UT%nTestFailed, &
    UT_NoColor // ' , # ' // trim(UT_PassedColor) // 'passed: ', UT%nTestPassed, &
    UT_NoColor // '   out of # total: '         , UT%nTestTotal
  !
endif
!
end subroutine UT_PrintSummary


!----------*****************------------------------------------------------------------------------
subroutine UT_PrintSeparator()
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintSeparator ('Unit_Test_Print_Separator')
!
! PURPOSE:
!   Print separator line.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintSeparator()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintSeparator()
!
! MODIFICATION HISTORY:
!   2017-05-18 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
character(len=80)                       :: MarkColor
!---------------------------------------------------------------------------------------------------
!
if (UT%AllTestPrint_flag) then
  write(UT%nc_p, '(A)') '       ' // trim(UT_SepColor) // '====================' // UT_NoColor
endif
!
end subroutine UT_PrintSeparator


!----------******************-----------------------------------------------------------------------
subroutine UT_PrintLogical1(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintLogical1 ('Unit_Test_Print_Logical_1')
!
! PURPOSE:
!   Print a 1D logical array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintLogical1(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') logical 1D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 1D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintLogical1('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2016-04-29 (B.F.) Adapt character version from integer version
!   2016-05-01 (B.F.) Adapt logical version from character version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
logical, dimension(:),     intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, i1, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%LogicalFormat)//')'
  !
  write(UT%nc_p, '(A)', advance='no') PrefStr
  do i1=m1, nPri1
    write(aStr, FormatStr) a(i1)
    if (.not. PassMask(i1)) then
      aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
    endif
    write(UT%nc_p, '(A)', advance='no') trim(aStr)
  end do ! i1
  write(UT%nc_p, '(A)') trim(DotDotDotStr)
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,' // trim(n1Str) // '(1X,'//trim(UT%LogicalFormat)//'),A)'
  !
  write(UT%nc_p, FormatStr) PrefStr, a(m1:nPri1), trim(DotDotDotStr)
endif
!
end subroutine UT_PrintLogical1


!----------****************-------------------------------------------------------------------------
subroutine UT_PrintInteger1(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintInteger1 ('Unit_Test_Print_Integer_1')
!
! PURPOSE:
!   Print a 1D integer array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintInteger1(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') integer 1D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 1D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintInteger1('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!   2016-04-16 (B.F.) Adapt integer version from real version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
integer, dimension(:),     intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, i1, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%IntegerFormat)//')'
  !
  write(UT%nc_p, '(A)', advance='no') PrefStr
  do i1=m1, nPri1
    write(aStr, FormatStr) a(i1)
    if (.not. PassMask(i1)) then
      aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
    endif
    write(UT%nc_p, '(A)', advance='no') trim(aStr)
  end do ! i1
  write(UT%nc_p, '(A)') trim(DotDotDotStr)
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,' // trim(n1Str) // '(1X,'//trim(UT%IntegerFormat)//'),A)'
  !
  write(UT%nc_p, FormatStr) PrefStr, a(m1:nPri1), trim(DotDotDotStr)
endif
!
end subroutine UT_PrintInteger1


!----------****************-------------------------------------------------------------------------
subroutine UT_PrintInteger2(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintInteger2 ('Unit_Test_Print_Integer_2')
!
! PURPOSE:
!   Print a 2D integer array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintInteger2(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') integer 2D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 2D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintInteger2('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!   2016-04-16 (B.F.) Adapt integer version from real version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
integer, dimension(:,:),   intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:,:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, i1, i2, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
m2=1
n2=size(a,2)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%IntegerFormat)//')'
  !
  do i2=m2, min(n2, UT%nMax2)
    write(UT%nc_p, '(A,1X,I3)', advance='no') PrefStr, i2
    do i1=m1, nPri1
      write(aStr, FormatStr) a(i1,i2)
      if (.not. PassMask(i1,i2)) then
        aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
      endif
      write(UT%nc_p, '(A)', advance='no') trim(aStr)
    end do ! i1
    write(UT%nc_p, '(A)') trim(DotDotDotStr)
  end do ! i2
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,I3,' // trim(n1Str) // '(1X,'//trim(UT%IntegerFormat)//'),A)'
  !
  do i2=m2, min(n2, UT%nMax2)
    write(UT%nc_p, FormatStr) PrefStr, i2, a(m1:nPri1, i2), trim(DotDotDotStr)
  end do ! i2
endif
!
end subroutine UT_PrintInteger2


!----------****************-------------------------------------------------------------------------
subroutine UT_PrintInteger3(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintInteger3 ('Unit_Test_Print_Integer_3')
!
! PURPOSE:
!   Print a 3D integer array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintInteger3(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') integer 3D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 3D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintInteger3('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!   2016-04-16 (B.F.) Adapt integer version from real version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
integer, dimension(:,:,:), intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:,:,:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
m2=1
n2=size(a,2)
m3=1
n3=size(a,3)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%IntegerFormat)//')'
  !
  do i3=m3, min(n3, UT%nMax3)
    do i2=m2, min(n2, UT%nMax2)
      write(UT%nc_p, '(A,1X,2I3)', advance='no') PrefStr, i3, i2
      do i1=m1, nPri1
        write(aStr, FormatStr) a(i1,i2,i3)
        if (.not. PassMask(i1,i2,i3)) then
          aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
        endif
        write(UT%nc_p, '(A)', advance='no') trim(aStr)
      end do ! i1
      write(UT%nc_p, '(A)') trim(DotDotDotStr)
    end do ! i2
  end do ! i3
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,2I3,' // trim(n1Str) // '(1X,'//trim(UT%IntegerFormat)//'),A)'
  !
  do i3=m3, min(n3, UT%nMax3)
    do i2=m2, min(n2, UT%nMax2)
      write(UT%nc_p, FormatStr) PrefStr, i3, i2, a(m1:nPri1, i2, i3), trim(DotDotDotStr)
    end do ! i2
  end do ! i3
endif
!
end subroutine UT_PrintInteger3


!----------*************----------------------------------------------------------------------------
subroutine UT_PrintReal1(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintReal1 ('Unit_Test_Print_Real_1')
!
! PURPOSE:
!   Print a 1D real array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintReal1(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') real 1D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 1D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintReal1('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
real, dimension(:),        intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, i1, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%RealFormat)//')'
  !
  write(UT%nc_p, '(A)', advance='no') PrefStr
  do i1=m1, nPri1
    write(aStr, FormatStr) a(i1)
    if (.not. PassMask(i1)) then
      aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
    endif
    write(UT%nc_p, '(A)', advance='no') trim(aStr)
  end do ! i1
  write(UT%nc_p, '(A)') trim(DotDotDotStr)
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,' // trim(n1Str) // '(1X,'//trim(UT%RealFormat)//'),A)'
  !
  write(UT%nc_p, FormatStr) PrefStr, a(m1:nPri1), trim(DotDotDotStr)
endif
!
end subroutine UT_PrintReal1


!----------*************----------------------------------------------------------------------------
subroutine UT_PrintReal2(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintReal2 ('Unit_Test_Print_Real_2')
!
! PURPOSE:
!   Print a 2D real array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintReal2(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') real 2D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 2D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintReal2('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
real, dimension(:,:),      intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:,:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, i1, i2, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
m2=1
n2=size(a,2)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%RealFormat)//')'
  !
  do i2=m2, min(n2, UT%nMax2)
    write(UT%nc_p, '(A,1X,I3)', advance='no') PrefStr, i2
    do i1=m1, nPri1
      write(aStr, FormatStr) a(i1,i2)
      if (.not. PassMask(i1,i2)) then
        aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
      endif
      write(UT%nc_p, '(A)', advance='no') trim(aStr)
    end do ! i1
    write(UT%nc_p, '(A)') trim(DotDotDotStr)
  end do ! i2
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,I3,' // trim(n1Str) // '(1X,'//trim(UT%RealFormat)//'),A)'
  ! FormatStr='(A,"\e[48;5;221m \e[0m",I3,' // &
  !           trim(n1Str) // '("\e[48;5;221m \e[0m",'//trim(UT%RealFormat)//'),A)'
  !
  do i2=m2, min(n2, UT%nMax2)
    write(UT%nc_p, FormatStr) PrefStr, i2, a(m1:nPri1, i2), trim(DotDotDotStr)
  end do ! i2
endif
!
end subroutine UT_PrintReal2


!----------*************----------------------------------------------------------------------------
subroutine UT_PrintReal3(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintReal3 ('Unit_Test_Print_Real_3')
!
! PURPOSE:
!   Print a 3D real array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintReal3(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') real 3D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 3D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintReal3('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
real, dimension(:,:,:),    intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:,:,:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
m2=1
n2=size(a,2)
m3=1
n3=size(a,3)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%RealFormat)//')'
  !
  do i3=m3, min(n3, UT%nMax3)
    do i2=m2, min(n2, UT%nMax2)
      write(UT%nc_p, '(A,1X,2I3)', advance='no') PrefStr, i3, i2
      do i1=m1, nPri1
        write(aStr, FormatStr) a(i1,i2,i3)
        if (.not. PassMask(i1,i2,i3)) then
          aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
        endif
        write(UT%nc_p, '(A)', advance='no') trim(aStr)
      end do ! i1
      write(UT%nc_p, '(A)') trim(DotDotDotStr)
    end do ! i2
  end do ! i3
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,2I3,' // trim(n1Str) // '(1X,'//trim(UT%RealFormat)//'),A)'
  !
  do i3=m3, min(n3, UT%nMax3)
    do i2=m2, min(n2, UT%nMax2)
      write(UT%nc_p, FormatStr) PrefStr, i3, i2, a(m1:nPri1, i2, i3), trim(DotDotDotStr)
    end do ! i2
  end do ! i3
endif
!
end subroutine UT_PrintReal3


!----------******************-----------------------------------------------------------------------
subroutine UT_PrintCharacter1(PrefStr, a, Pass_flag, PassMask)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_PrintCharacter1 ('Unit_Test_Print_Character_1')
!
! PURPOSE:
!   Print a 1D character array.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_PrintCharacter1(PrefStr, a, Pass_flag, PassMask)
!
! INPUT:
!   PrefStr:      ('Prefix_String') character with prefix to be plotted at beginning of each line
!   a:            ('array') character 1D array to be printed
!   Pass_flag:    ('Passed_flag') optional logical to indicate that the unit test has been passed
!   PassMask:     ('Passed_Mask') logical 1D array (same shape as a) to indicate which elements of a
!                 are above the error limit (and should be highlighted)
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_PrintCharacter1('Vi1=', v1)
!
! MODIFICATION HISTORY:
!   2015-03-02 (B.F. Uppsala) Written
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2016-04-29 (B.F.) Adapt character version from integer version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: PrefStr
character(len=*), dimension(:), &
                           intent(in)   :: a
logical, optional,         intent(in)   :: Pass_flag
logical, dimension(:), optional, &
                           intent(in)   :: PassMask
!
! --- Local variables ---
integer                                 :: m1,n1, i1, nPri1
logical                                 :: PrintColor_flag
character(len=80)                       :: FormatStr, n1Str, aStr
character(len=4)                        :: DotDotDotStr
!---------------------------------------------------------------------------------------------------
!
! --- Get array size ---
m1=1
n1=size(a,1)
nPri1=min(n1, UT%nMax1)
!
! --- Compose indicator string for too long lines ---
if (nPri1 < n1) then
  DotDotDotStr=' ...'
else
  DotDotDotStr=''
endif
!
! --- Check of color output is necessary ---
if ((present(Pass_flag)) .and. (present(PassMask)) .and. (UT_FMarkColor /= '')) then
  PrintColor_flag=(.not. Pass_flag)
else
  PrintColor_flag=.false.
endif
if (PrintColor_flag) then
  ! --- Print array in compact format, possibly with colors ---
  !
  FormatStr='(1X,'//trim(UT%CharacterFormat)//')'
  !
  write(UT%nc_p, '(A)', advance='no') PrefStr
  do i1=m1, nPri1
    write(aStr, FormatStr) a(i1)
    if (.not. PassMask(i1)) then
      aStr=trim(UT_FMarkColor) // trim(aStr) // UT_NoColor
    endif
    write(UT%nc_p, '(A)', advance='no') trim(aStr)
  end do ! i1
  write(UT%nc_p, '(A)') trim(DotDotDotStr)
else
  ! --- Print array in compact format ---
  !
  write(n1Str,*) nPri1
  FormatStr='(A,1X,' // trim(n1Str) // '(1X,'//trim(UT%CharacterFormat)//'),A)'
  !
  write(UT%nc_p, FormatStr) PrefStr, a(m1:nPri1), trim(DotDotDotStr)
endif
!
end subroutine UT_PrintCharacter1


!--------**************-----------------------------------------------------------------------------
function UT_StrPassFail(Pass_flag, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_StrPassFail ('Unit_Test_String_Passed_Failed')
!
! PURPOSE:
!   Produce status string corresponding to the input state.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   Result=UT_StrPassFail(Pass_flag, Ignore_flag)
!
! INPUT:
!
! OUTPUT:
!   Result:       Character with appropriate test-result status message
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2015-03-05 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
logical,                   intent(in)   :: Pass_flag, Ignore_flag
character(len=40)                       :: UT_StrPassFail
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (Ignore_flag) then
  if (Pass_flag) then
    UT_StrPassFail=UT_IgnoredPassStr
  else
    UT_StrPassFail=UT_IgnoredFailStr
  endif
else
  if (Pass_flag) then
    UT_StrPassFail=UT_PassedStr
  else
    UT_StrPassFail=UT_FailedStr
  endif
endif
!
end function UT_StrPassFail


!----------********************---------------------------------------------------------------------
subroutine UT_AssertLogical11(Relation, TestName, aXv1, aXv2, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertLogical11 ('Unit_Test_Assert_Logical_1_1')
!
! PURPOSE:
!   Compare two arrays and check if they are equal.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertLogical11(Relation, TestName, aXv1, aXv2, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity
!                 '/=': check for inequality
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 1D array for test
!   aXv2:         ('a_X_vector_2') Second 1D array for test
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertLogical11('==', 'ArraysAreEqual', a, b)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2016-04-29 (B.F.) Adapt character version from integer version
!   2016-05-01 (B.F.) Adapt logical version from character version
!   2017-05-17 (B.F.) Change comparision operators
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
logical, dimension(:),     intent(in)   :: aXv1, aXv2
logical,          optional,intent(in)   :: Ignore_flag
!
! --- Local variables ---
logical                                 :: TestPass_flag, Ignore_flag0
logical  , dimension(1:size(aXv1,1))    :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='eq'
    PassMask     =(aXv1 .eqv. aXv2)
    TestPass_flag=all(PassMask)
    !
  else if (Relation == '/=') then
    NormName     ='neq'
    PassMask     =(aXv1 .neqv. aXv2)
    TestPass_flag=all(PassMask)
    !
  else
    stop 'UT_AssertLogical11: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(A)') trim(TestName) // ' ' // &
      trim(NormName) // &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if ((((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. &
     (UT%AllTestPrint_flag)                                   ) .and. &
    ( UT%AllVecOutPrint_flag)                                          ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertLogical11


!----------******************-----------------------------------------------------------------------
subroutine UT_AssertInteger10(Relation, TestName, aXv, TestValue, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertInteger10 ('Unit_Test_Assert_Integer_1_0')
!
! PURPOSE:
!   Compare input array to a constant value and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertInteger10(Relation, TestName, aXv, TestValue, &
!                           delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv:          ('a_X_vector') 1D array to test
!   TestValue:    ('Test_Value') integer with constant to test against
!   delta:        ('delta') optional integer scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertInteger10('==', 'ArrayIsZero', a, 0.0)
!   call UT_AssertInteger10('==', 'ArrayIsAlmostZero', a, 0.0, delta=1.0E-05)
!   call UT_AssertInteger10('>=', 'ArrayIsPositive', a, 0.0)
!   call UT_AssertInteger10('>=', 'ArrayIsPositive', a, 1.0, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2017-08-27 (B.F. Lyon) Add '/=' relation
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
integer,                   intent(in)   :: TestValue
integer, dimension(:),     intent(in)   :: aXv
integer, optional,         intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
integer                                 :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
integer, dimension(1:size(aXv,1))       :: ErrorAr
logical, dimension(1:size(aXv,1))       :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (Relation == '==') then
  NormName     ='L_max'
  ErrorAr      =abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '/=') then
  NormName     ='L_maxneq'
  ErrorAr      =-abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr   < delta1)
  TestPass_flag=(TestNorm  < delta1)
  !
else if (Relation == '>=') then
  NormName     ='mindiff'
  ErrorAr      =TestValue-aXv
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '<=') then
  NormName     ='maxdiff'
  ErrorAr      =aXv-TestValue
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else
  stop 'UT_AssertInteger10: ERROR: Unknown relation type.'
  !
endif
!
! --- Store that one more test has been performed ---
UT%nTestTotal=UT%nTestTotal+1
!
! --- Store that one more test has failed, if so ---
if (.not. Ignore_flag0) then
  if (TestPass_flag) then
    UT%nTestPassed=UT%nTestPassed+1
  else
    UT%nTestFailed=UT%nTestFailed+1
  endif
endif
!
if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
  ! --- Print the test result ---
  !
  write(UT%nc_p, '(2(A,'//trim(UT%IntegerFormat)//'),A)') trim(TestName) // ' ' // &
    trim(NormName) // '=', TestNorm, ' <= ', delta1, &
    trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
  !
  if (UT%AllVecOutPrint_flag) then
    ! --- Print (part of) the test data ---
    call UT_PrintArray('Vo1=', aXv, Pass_flag=TestPass_flag, PassMask=PassMask)
  endif
endif
!
end subroutine UT_AssertInteger10


!----------******************-----------------------------------------------------------------------
subroutine UT_AssertInteger20(Relation, TestName, aXv, TestValue, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertInteger20 ('Unit_Test_Assert_Integer_2_0')
!
! PURPOSE:
!   Compare input array to a constant value and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertInteger20(Relation, TestName, aXv, TestValue, &
!                           delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv:          ('a_X_vector') 2D array to test
!   TestValue:    ('Test_Value') integer with constant to test against
!   delta:        ('delta') optional integer scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertInteger20('==', 'ArrayIsZero', a, 0.0)
!   call UT_AssertInteger20('==', 'ArrayIsAlmostZero', a, 0.0, delta=1.0E-05)
!   call UT_AssertInteger20('>=', 'ArrayIsPositive', a, 0.0)
!   call UT_AssertInteger20('>=', 'ArrayIsPositive', a, 1.0, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2017-08-27 (B.F. Lyon) Add '/=' relation
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
integer,                   intent(in)   :: TestValue
integer, dimension(:,:),   intent(in)   :: aXv
integer, optional,         intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
integer                                 :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
integer, dimension(1:size(aXv,1), &
                   1:size(aXv,2))       :: ErrorAr
logical, dimension(1:size(aXv,1), &
                   1:size(aXv,2))       :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (Relation == '==') then
  NormName     ='L_max'
  ErrorAr      =abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '/=') then
  NormName     ='L_maxneq'
  ErrorAr      =-abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr   < delta1)
  TestPass_flag=(TestNorm  < delta1)
  !
else if (Relation == '>=') then
  NormName     ='mindiff'
  ErrorAr      =TestValue-aXv
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '<=') then
  NormName     ='maxdiff'
  ErrorAr      =aXv-TestValue
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else
  stop 'UT_AssertInteger20: ERROR: Unknown relation type.'
  !
endif
!
! --- Store that one more test has been performed ---
UT%nTestTotal=UT%nTestTotal+1
!
! --- Store that one more test has failed, if so ---
if (.not. Ignore_flag0) then
  if (TestPass_flag) then
    UT%nTestPassed=UT%nTestPassed+1
  else
    UT%nTestFailed=UT%nTestFailed+1
  endif
endif
!
if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
  ! --- Print the test result ---
  !
  write(UT%nc_p, '(2(A,'//trim(UT%IntegerFormat)//'),A)') trim(TestName) // ' ' // &
    trim(NormName) // '=', TestNorm, ' <= ', delta1, &
    trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
  !
  if (UT%AllVecOutPrint_flag) then
    ! --- Print (part of) the test data ---
    call UT_PrintArray('Vo1=', aXv, Pass_flag=TestPass_flag, PassMask=PassMask)
  endif
endif
!
end subroutine UT_AssertInteger20


!----------******************-----------------------------------------------------------------------
subroutine UT_AssertInteger30(Relation, TestName, aXv, TestValue, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertInteger30 ('Unit_Test_Assert_Integer_3_0')
!
! PURPOSE:
!   Compare input array to a constant value and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertInteger30(Relation, TestName, aXv, TestValue, &
!                           delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv:          ('a_X_vector') 3D array to test
!   TestValue:    ('Test_Value') integer with constant to test against
!   delta:        ('delta') optional integer scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertInteger30('==', 'ArrayIsZero', a, 0.0)
!   call UT_AssertInteger30('==', 'ArrayIsAlmostZero', a, 0.0, delta=1.0E-05)
!   call UT_AssertInteger30('>=', 'ArrayIsPositive', a, 0.0)
!   call UT_AssertInteger30('>=', 'ArrayIsPositive', a, 1.0, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2017-08-27 (B.F. Lyon) Add '/=' relation
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
integer,                   intent(in)   :: TestValue
integer, dimension(:,:,:), intent(in)   :: aXv
integer, optional,         intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
integer                                 :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
integer, dimension(1:size(aXv,1), &
                   1:size(aXv,2), &
                   1:size(aXv,3))       :: ErrorAr
logical, dimension(1:size(aXv,1), &
                   1:size(aXv,2), &
                   1:size(aXv,3))       :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (Relation == '==') then
  NormName     ='L_max'
  ErrorAr      =abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '/=') then
  NormName     ='L_maxneq'
  ErrorAr      =-abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr   < delta1)
  TestPass_flag=(TestNorm  < delta1)
  !
else if (Relation == '>=') then
  NormName     ='mindiff'
  ErrorAr      =TestValue-aXv
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '<=') then
  NormName     ='maxdiff'
  ErrorAr      =aXv-TestValue
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else
  stop 'UT_AssertInteger30: ERROR: Unknown relation type.'
  !
endif
!
! --- Store that one more test has been performed ---
UT%nTestTotal=UT%nTestTotal+1
!
! --- Store that one more test has failed, if so ---
if (.not. Ignore_flag0) then
  if (TestPass_flag) then
    UT%nTestPassed=UT%nTestPassed+1
  else
    UT%nTestFailed=UT%nTestFailed+1
  endif
endif
!
if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
  ! --- Print the test result ---
  !
  write(UT%nc_p, '(2(A,'//trim(UT%IntegerFormat)//'),A)') trim(TestName) // ' ' // &
    trim(NormName) // '=', TestNorm, ' <= ', delta1, &
    trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
  !
  if (UT%AllVecOutPrint_flag) then
    ! --- Print (part of) the test data ---
    call UT_PrintArray('Vo1=', aXv, Pass_flag=TestPass_flag, PassMask=PassMask)
  endif
endif
!
end subroutine UT_AssertInteger30


!----------******************-----------------------------------------------------------------------
subroutine UT_AssertInteger11(Relation, TestName, aXv1, aXv2, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertInteger11 ('Unit_Test_Assert_Integer_1_1')
!
! PURPOSE:
!   Compare two arrays and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertInteger11(Relation, TestName, aXv1, aXv2, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 1D array for test
!   aXv2:         ('a_X_vector_2') Second 1D array for test
!   delta:        ('delta') optional integer scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertInteger11('==', 'ArraysAreEqual', a, b)
!   call UT_AssertInteger11('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-05)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
integer, dimension(:),     intent(in)   :: aXv1, aXv2
integer, optional,         intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
integer                                 :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
integer, dimension(1:size(aXv1,1))      :: ErrorAr
logical, dimension(1:size(aXv1,1))      :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='L_max'
    ErrorAr      =abs(aXv1-aXv2)
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '>=') then
    NormName     ='mindiff'
    ErrorAr      =aXv2-aXv1
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '<=') then
    NormName     ='maxdiff'
    ErrorAr      =aXv1-aXv2
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else
    stop 'UT_AssertInteger11: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(2(A,'//trim(UT%IntegerFormat)//'),A)') trim(TestName) // ' ' // &
      trim(NormName) // '=', TestNorm, ' <= ', delta1, &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if ((((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. &
     (UT%AllTestPrint_flag)                                   ) .and. &
    ( UT%AllVecOutPrint_flag)                                          ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertInteger11


!----------******************-----------------------------------------------------------------------
subroutine UT_AssertInteger22(Relation, TestName, aXv1, aXv2, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertInteger22 ('Unit_Test_Assert_Integer_2_2')
!
! PURPOSE:
!   Compare two arrays and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertInteger22(Relation, TestName, aXv1, aXv2, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 2D array for test
!   aXv2:         ('a_X_vector_2') Second 2D array for test
!   delta:        ('delta') optional integer scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertInteger22('==', 'ArraysAreEqual', a, b)
!   call UT_AssertInteger22('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-05)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
integer, dimension(:,:),   intent(in)   :: aXv1, aXv2
integer, optional,         intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
integer                                 :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
integer, dimension(1:size(aXv1,1), &
                   1:size(aXv1,2))      :: ErrorAr
logical, dimension(1:size(aXv1,1), &
                   1:size(aXv1,2))      :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='L_max'
    ErrorAr      =abs(aXv1-aXv2)
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '>=') then
    NormName     ='mindiff'
    ErrorAr      =aXv2-aXv1
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '<=') then
    NormName     ='maxdiff'
    ErrorAr      =aXv1-aXv2
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else
    stop 'UT_AssertInteger22: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(2(A,'//trim(UT%IntegerFormat)//'),A)') trim(TestName) // ' ' // &
      trim(NormName) // '=', TestNorm, ' <= ', delta1, &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if ((((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. &
     (UT%AllTestPrint_flag)                                   ) .and. &
    ( UT%AllVecOutPrint_flag)                                          ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertInteger22


!----------******************-----------------------------------------------------------------------
subroutine UT_AssertInteger33(Relation, TestName, aXv1, aXv2, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertInteger33 ('Unit_Test_Assert_Integer_3_3')
!
! PURPOSE:
!   Compare two arrays and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertInteger33(Relation, TestName, aXv1, aXv2, delta=delta)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 3D array for test
!   aXv2:         ('a_X_vector_2') Second 3D array for test
!   delta:        ('delta') optional integer scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertInteger33('==', 'ArraysAreEqual'      , a, b)
!   call UT_AssertInteger33('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-05)
!   call UT_AssertInteger33('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-06, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
integer, dimension(:,:,:), intent(in)   :: aXv1, aXv2
integer, optional,         intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
integer                                 :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
integer, dimension(1:size(aXv1,1), &
                   1:size(aXv1,2), &
                   1:size(aXv1,3))      :: ErrorAr
logical, dimension(1:size(aXv1,1), &
                   1:size(aXv1,2), &
                   1:size(aXv1,3))      :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='L_max'
    ErrorAr      =abs(aXv1-aXv2)
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '>=') then
    NormName     ='mindiff'
    ErrorAr      =aXv2-aXv1
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '<=') then
    NormName     ='maxdiff'
    ErrorAr      =aXv1-aXv2
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else
    stop 'UT_AssertInteger33: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(2(A,'//trim(UT%IntegerFormat)//'),A)') trim(TestName) // ' ' // &
      trim(NormName) // '=', TestNorm, ' <= ', delta1, &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if (((.not. TestPass_flag) .or. (UT%AllTestPrint_flag)) .and. &
    ( UT%AllVecOutPrint_flag)                                  ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertInteger33


!----------***************--------------------------------------------------------------------------
subroutine UT_AssertReal10(Relation, TestName, aXv, TestValue, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertReal10 ('Unit_Test_Assert_Real_1_0')
!
! PURPOSE:
!   Compare input array to a constant value and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertReal10(Relation, TestName, aXv, TestValue, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv:          ('a_X_vector') 1D array to test
!   TestValue:    ('Test_Value') real with constant to test against
!   delta:        ('delta') optional real scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertReal10('==', 'ArrayIsZero', a, 0.0)
!   call UT_AssertReal10('==', 'ArrayIsAlmostZero', a, 0.0, delta=1.0E-05)
!   call UT_AssertReal10('>=', 'ArrayIsPositive', a, 0.0)
!   call UT_AssertReal10('>=', 'ArrayIsPositive', a, 1.0, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
real,                      intent(in)   :: TestValue
real, dimension(:),        intent(in)   :: aXv
real, optional,            intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
real                                    :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
real, dimension(1:size(aXv,1))          :: ErrorAr
logical, dimension(1:size(aXv,1))       :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (Relation == '==') then
  NormName     ='L_max'
  ErrorAr      =abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '>=') then
  NormName     ='mindiff'
  ErrorAr      =TestValue-aXv
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '<=') then
  NormName     ='maxdiff'
  ErrorAr      =aXv-TestValue
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else
  stop 'UT_AssertReal10: ERROR: Unknown relation type.'
  !
endif
!
! --- Store that one more test has been performed ---
UT%nTestTotal=UT%nTestTotal+1
!
! --- Store that one more test has failed, if so ---
if (.not. Ignore_flag0) then
  if (TestPass_flag) then
    UT%nTestPassed=UT%nTestPassed+1
  else
    UT%nTestFailed=UT%nTestFailed+1
  endif
endif
!
if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
  ! --- Print the test result ---
  !
  write(UT%nc_p, '(2(A,'//trim(UT%RealFormat)//'),A)') trim(TestName) // ' ' // &
    trim(NormName) // '=', TestNorm, ' <= ', delta1, &
    trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
  !
  if (UT%AllVecOutPrint_flag) then
    ! --- Print (part of) the test data ---
    call UT_PrintArray('Vo1=', aXv, Pass_flag=TestPass_flag, PassMask=PassMask)
  endif
endif
!
end subroutine UT_AssertReal10


subroutine UT_AssertReal20(Relation, TestName, aXv, TestValue, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertReal20 ('Unit_Test_Assert_Real_2_0')
!
! PURPOSE:
!   Compare input array to a constant value and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertReal20(Relation, TestName, aXv, TestValue, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv:          ('a_X_vector') 2D array to test
!   TestValue:    ('Test_Value') real with constant to test against
!   delta:        ('delta') optional real scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertReal20('==', 'ArrayIsZero', a, 0.0)
!   call UT_AssertReal20('==', 'ArrayIsAlmostZero', a, 0.0, delta=1.0E-05)
!   call UT_AssertReal20('>=', 'ArrayIsPositive', a, 0.0)
!   call UT_AssertReal20('>=', 'ArrayIsPositive', a, 1.0, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
real,                      intent(in)   :: TestValue
real, dimension(:,:),      intent(in)   :: aXv
real, optional,            intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
real                                    :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
real, dimension(1:size(aXv,1), &
                1:size(aXv,2))          :: ErrorAr
logical, dimension(1:size(aXv,1), &
                   1:size(aXv,2))       :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (Relation == '==') then
  NormName     ='L_max'
  ErrorAr      =abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '>=') then
  NormName     ='mindiff'
  ErrorAr      =TestValue-aXv
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '<=') then
  NormName     ='maxdiff'
  ErrorAr      =aXv-TestValue
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else
  stop 'UT_AssertReal20: ERROR: Unknown relation type.'
  !
endif
!
! --- Store that one more test has been performed ---
UT%nTestTotal=UT%nTestTotal+1
!
! --- Store that one more test has failed, if so ---
if (.not. Ignore_flag0) then
  if (TestPass_flag) then
    UT%nTestPassed=UT%nTestPassed+1
  else
    UT%nTestFailed=UT%nTestFailed+1
  endif
endif
!
if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
  ! --- Print the test result ---
  !
  write(UT%nc_p, '(2(A,'//trim(UT%RealFormat)//'),A)') trim(TestName) // ' ' // &
    trim(NormName) // '=', TestNorm, ' <= ', delta1, &
    trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
  !
  if (UT%AllVecOutPrint_flag) then
    ! --- Print (part of) the test data ---
    call UT_PrintArray('Vo1=', aXv, Pass_flag=TestPass_flag, PassMask=PassMask)
  endif
endif
!
end subroutine UT_AssertReal20


!----------***************--------------------------------------------------------------------------
subroutine UT_AssertReal30(Relation, TestName, aXv, TestValue, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertReal30 ('Unit_Test_Assert_Real_3_0')
!
! PURPOSE:
!   Compare input array to a constant value and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertReal30(Relation, TestName, aXv, TestValue, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv:          ('a_X_vector') 3D array to test
!   TestValue:    ('Test_Value') real with constant to test against
!   delta:        ('delta') optional real scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertReal30('==', 'ArrayIsZero', a, 0.0)
!   call UT_AssertReal30('==', 'ArrayIsAlmostZero', a, 0.0, delta=1.0E-05)
!   call UT_AssertReal30('>=', 'ArrayIsPositive', a, 0.0)
!   call UT_AssertReal30('>=', 'ArrayIsPositive', a, 1.0, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
real,                      intent(in)   :: TestValue
real, dimension(:,:,:),    intent(in)   :: aXv
real, optional,            intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
real                                    :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
real, dimension(1:size(aXv,1), &
                1:size(aXv,2), &
                1:size(aXv,3))          :: ErrorAr
logical, dimension(1:size(aXv,1), &
                   1:size(aXv,2), &
                   1:size(aXv,3))       :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (Relation == '==') then
  NormName     ='L_max'
  ErrorAr      =abs(aXv-TestValue)
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '>=') then
  NormName     ='mindiff'
  ErrorAr      =TestValue-aXv
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else if (Relation == '<=') then
  NormName     ='maxdiff'
  ErrorAr      =aXv-TestValue
  TestNorm     =maxval(ErrorAr)
  delta1       =delta0
  PassMask     =(ErrorAr  <= delta1)
  TestPass_flag=(TestNorm <= delta1)
  !
else
  stop 'UT_AssertReal30: ERROR: Unknown relation type.'
  !
endif
!
! --- Store that one more test has been performed ---
UT%nTestTotal=UT%nTestTotal+1
!
! --- Store that one more test has failed, if so ---
if (.not. Ignore_flag0) then
  if (TestPass_flag) then
    UT%nTestPassed=UT%nTestPassed+1
  else
    UT%nTestFailed=UT%nTestFailed+1
  endif
endif
!
if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
  ! --- Print the test result ---
  !
  write(UT%nc_p, '(2(A,'//trim(UT%RealFormat)//'),A)') trim(TestName) // ' ' // &
    trim(NormName) // '=', TestNorm, ' <= ', delta1, &
    trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
  !
  if (UT%AllVecOutPrint_flag) then
    ! --- Print (part of) the test data ---
    call UT_PrintArray('Vo1=', aXv, Pass_flag=TestPass_flag, PassMask=PassMask)
  endif
endif
!
end subroutine UT_AssertReal30


!----------***************--------------------------------------------------------------------------
subroutine UT_AssertReal11(Relation, TestName, aXv1, aXv2, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertReal11 ('Unit_Test_Assert_Real_1_1')
!
! PURPOSE:
!   Compare two arrays and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertReal11(Relation, TestName, aXv1, aXv2, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 1D array for test
!   aXv2:         ('a_X_vector_2') Second 1D array for test
!   delta:        ('delta') optional real scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertReal11('==', 'ArraysAreEqual', a, b)
!   call UT_AssertReal11('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-05)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
real, dimension(:),        intent(in)   :: aXv1, aXv2
real, optional,            intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
real                                    :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
real, dimension(1:size(aXv1,1))         :: ErrorAr
logical, dimension(1:size(aXv1,1))      :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='L_max'
    ErrorAr      =abs(aXv1-aXv2)
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '>=') then
    NormName     ='mindiff'
    ErrorAr      =aXv2-aXv1
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '<=') then
    NormName     ='maxdiff'
    ErrorAr      =aXv1-aXv2
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else
    stop 'UT_AssertReal11: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(2(A,'//trim(UT%RealFormat)//'),A)') trim(TestName) // ' ' // &
      trim(NormName) // '=', TestNorm, ' <= ', delta1, &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if ((((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. &
     (UT%AllTestPrint_flag)                                   ) .and. &
    ( UT%AllVecOutPrint_flag)                                          ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertReal11


!----------***************--------------------------------------------------------------------------
subroutine UT_AssertReal22(Relation, TestName, aXv1, aXv2, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertReal22 ('Unit_Test_Assert_Real_2_2')
!
! PURPOSE:
!   Compare two arrays and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertReal22(Relation, TestName, aXv1, aXv2, delta=delta, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 2D array for test
!   aXv2:         ('a_X_vector_2') Second 2D array for test
!   delta:        ('delta') optional real scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertReal22('==', 'ArraysAreEqual', a, b)
!   call UT_AssertReal22('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-05)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
real, dimension(:,:),      intent(in)   :: aXv1, aXv2
real, optional,            intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
real                                    :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
real, dimension(1:size(aXv1,1), &
                1:size(aXv1,2))         :: ErrorAr
logical, dimension(1:size(aXv1,1), &
                   1:size(aXv1,2))      :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='L_max'
    ErrorAr      =abs(aXv1-aXv2)
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '>=') then
    NormName     ='mindiff'
    ErrorAr      =aXv2-aXv1
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '<=') then
    NormName     ='maxdiff'
    ErrorAr      =aXv1-aXv2
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else
    stop 'UT_AssertReal22: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(2(A,'//trim(UT%RealFormat)//'),A)') trim(TestName) // ' ' // &
      trim(NormName) // '=', TestNorm, ' <= ', delta1, &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if ((((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. &
     (UT%AllTestPrint_flag)                                   ) .and. &
    ( UT%AllVecOutPrint_flag)                                          ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertReal22


!----------***************--------------------------------------------------------------------------
subroutine UT_AssertReal33(Relation, TestName, aXv1, aXv2, delta, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertReal33 ('Unit_Test_Assert_Real_3_3')
!
! PURPOSE:
!   Compare two arrays and check if the given relation is true.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertReal33(Relation, TestName, aXv1, aXv2, delta=delta)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity, possibly with error margin given by delta
!                 '>=': check if array is larger than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit smaller;
!                         delta < 0.0: there is a safety margin, the array is definitely larger
!                 '<=': check if array is smaller than TestValue, possibly with an error margin
!                       given by delta:
!                         delta > 0.0: the array could be a bit larger;
!                         delta < 0.0: there is a safety margin, the array is definitely smaller
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 3D array for test
!   aXv2:         ('a_X_vector_2') Second 3D array for test
!   delta:        ('delta') optional real scalar with error margin
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertReal33('==', 'ArraysAreEqual'      , a, b)
!   call UT_AssertReal33('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-05)
!   call UT_AssertReal33('==', 'ArraysAreAlmostEqual', a, b, delta=1.0E-06, Ignore_flag=.true.)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
real, dimension(:,:,:),    intent(in)   :: aXv1, aXv2
real, optional,            intent(in)   :: delta
logical, optional,         intent(in)   :: Ignore_flag
!
! --- Local variables ---
real                                    :: TestNorm, delta0, delta1
logical                                 :: TestPass_flag, Ignore_flag0
real, dimension(1:size(aXv1,1), &
                1:size(aXv1,2), &
                1:size(aXv1,3))         :: ErrorAr
logical, dimension(1:size(aXv1,1), &
                   1:size(aXv1,2), &
                   1:size(aXv1,3))      :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(delta)) then
  delta0=delta
else
  delta0=0.0
endif
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='L_max'
    ErrorAr      =abs(aXv1-aXv2)
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '>=') then
    NormName     ='mindiff'
    ErrorAr      =aXv2-aXv1
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else if (Relation == '<=') then
    NormName     ='maxdiff'
    ErrorAr      =aXv1-aXv2
    TestNorm     =maxval(ErrorAr)
    delta1       =delta0
    PassMask     =(ErrorAr  <= delta1)
    TestPass_flag=(TestNorm <= delta1)
    !
  else
    stop 'UT_AssertReal33: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(2(A,'//trim(UT%RealFormat)//'),A)') trim(TestName) // ' ' // &
      trim(NormName) // '=', TestNorm, ' <= ', delta1, &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if (((.not. TestPass_flag) .or. (UT%AllTestPrint_flag)) .and. &
    ( UT%AllVecOutPrint_flag)                                  ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertReal33


!----------********************---------------------------------------------------------------------
subroutine UT_AssertCharacter11(Relation, TestName, aXv1, aXv2, Ignore_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_AssertCharacter11 ('Unit_Test_Assert_Character_1_1')
!
! PURPOSE:
!   Compare two arrays and check if they are equal.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_AssertCharacter11(Relation, TestName, aXv1, aXv2, Ignore_flag=Ignore_flag)
!
! INPUT:
!   Relation:     ('Relation') character with relation that is to be tested:
!                 '==': check for identity
!                 '/=': check for inequality
!   TestName:     ('Test_Name') character with name of test
!   aXv1:         ('a_X_vector_1') First 1D array for test
!   aXv2:         ('a_X_vector_2') Second 1D array for test
!   Ignore_flag:  ('Ignore_flag) optional logical to indicate that a test fail should be ignored
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   UT_PrintArray:         Print a 1D/2D/3D array
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Increment global counters.
!   Write to output channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call UT_AssertCharacter11('==', 'ArraysAreEqual', a, b)
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-04 (B.F.) Adapted from other routines
!   2015-03-05 (B.F.) Add paramter Ignore_flag
!   2016-04-16 (B.F.) Adapt integer version from real version
!   2016-04-29 (B.F.) Adapt character version from integer version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: Relation, TestName
character(len=*), dimension(:), &
                           intent(in)   :: aXv1, aXv2
logical,          optional,intent(in)   :: Ignore_flag
!
! --- Local variables ---
logical                                 :: TestPass_flag, Ignore_flag0
logical  , dimension(1:size(aXv1,1))    :: PassMask
character(len=80)                       :: NormName
!---------------------------------------------------------------------------------------------------
!
! --- Check optional keywords ---
if (present(Ignore_flag)) then
  Ignore_flag0=Ignore_flag
else
  Ignore_flag0=.false.
endif
!
! --- Do the test according to the selected relation ---
if (any(shape(aXv1) /= shape(aXv2))) then
  TestPass_flag=.false.
  !
  write(UT%nc_p, '(A,A)') trim(TestName) // ' ' // 'ArrayMismatch', UT_FailedStr
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed ---
  UT%nTestFailed=UT%nTestFailed+1
  !
else ! (.not. (any(shape(aXv1) /= shape(aXv2))))
  if (Relation == '==') then
    NormName     ='lexeq'
    PassMask     =(aXv1 == aXv2)
    TestPass_flag=all(PassMask)
    !
  else if (Relation == '/=') then
    NormName     ='lexneq'
    PassMask     =(aXv1 /= aXv2)
    TestPass_flag=all(PassMask)
    !
  else
    stop 'UT_AssertCharacter11: ERROR: Unknown relation type.'
    !
  endif
  !
  ! --- Store that one more test has been performed ---
  UT%nTestTotal=UT%nTestTotal+1
  !
  ! --- Store that one more test has failed, if so ---
  if (.not. Ignore_flag0) then
    if (TestPass_flag) then
      UT%nTestPassed=UT%nTestPassed+1
    else
      UT%nTestFailed=UT%nTestFailed+1
    endif
  endif
  !
  if (((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. (UT%AllTestPrint_flag)) then
    ! --- Print the test result ---
    !
    write(UT%nc_p, '(A)') trim(TestName) // ' ' // &
      trim(NormName) // &
      trim(UT_StrPassFail(TestPass_flag, Ignore_flag0))
    !
  endif
endif ! (any(shape(aXv1) /= shape(aXv2)))
!
if ((((.not. TestPass_flag) .and. (.not. Ignore_flag0)) .or. &
     (UT%AllTestPrint_flag)                                   ) .and. &
    ( UT%AllVecOutPrint_flag)                                          ) then
  ! --- Print (part of) the test data ---
  call UT_PrintArray('Vo1=', aXv1, Pass_flag=TestPass_flag, PassMask=PassMask)
  call UT_PrintArray('Vo2=', aXv2, Pass_flag=TestPass_flag, PassMask=PassMask)
endif
!
end subroutine UT_AssertCharacter11


!----------*********************--------------------------------------------------------------------
subroutine UT_DemoExamplePassAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_DemoExamplePassAll ('Unit_Test_Demo_Example_Pass_All')
!
! PURPOSE:
!   Call a number of assert routines to perform tests that are all passed.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_DemoExamplePassAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_Init:               Initialize unit-test counters and control variables
!   UT_PrintSummary:       Print values of unit-test counters and summarize tests
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Set global variable UT.
!   Print data.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Initialize the unit-test routines: set counters to zero.
!   Construct a few demo "unit tests" to show use of calls to UT_Assert().
!   Print the result.
!
! EXAMPLE:
!   call UT_DemoExamplePassAll()
!
! MODIFICATION HISTORY:
!   2015-03-04 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
real                                    :: delta
real, dimension(2,2)                    :: M1, M2, Munity
!---------------------------------------------------------------------------------------------------
!
UT%nc_p=6
!
! --- Initialize unit-test counters ---
call UT_Init()
UT%AllTestPrint_flag  =.true.
UT%AllVecOutPrint_flag=.true.
!
! === Perform some simple demo tests ===
!
! --- Create unity matrix ---
Munity(1,1)=1.0  ;  Munity(2,1)=0.0
Munity(1,2)=0.0  ;  Munity(2,2)=1.0
!
! --- Create demo matrices ---
M1(1,1)=1.1  ;  M1(2,1)=2.1
M1(1,2)=1.2  ;  M1(2,2)=2.2
!
M2=M1 ; M2(1,1)=M2(1,1)+1.0E-06
!
!call UT_Assert('==', 'Matrix1IsLargerOrEqualThanZero'    , M1, 0.0)
!
call UT_Assert('==', 'Matrix1IsIdenticalToItself'        , M1, M1)
call UT_Assert('==', 'Matrix1IsAlmostEqualToMatrix2'     , M1, M2, delta=1.0E-05)
call UT_Assert('<=', 'Matrix1IsSmallerOrEqualThanMatrix2', M1, M2)
call UT_Assert('>=', 'Matrix2IsLargerOrEqualThanMatrix1' , M2, M1)
call UT_Assert('>=', 'Matrix1IsLargerOrEqualThanUnity'   , M1, Munity)
!
call UT_Assert('>=', 'Matrix2IsLargerOrEqualThanUnity'   , M1, Munity, Ignore_flag=.true.)
!
! --- Print test summary ---
call UT_PrintSummary()
!
end subroutine UT_DemoExamplePassAll


!----------*********************--------------------------------------------------------------------
subroutine UT_DemoExampleFailAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   UT_DemoExampleFailAll ('Unit_Test_Demo_Example_Fail_All')
!
! PURPOSE:
!   Call a number of assert routines to perform tests that all fail.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call UT_DemoExampleFailAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   UT_Init:               Initialize unit-test counters and control variables
!   UT_PrintSummary:       Print values of unit-test counters and summarize tests
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Set global variable UT.
!   Print data.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Initialize the unit-test routines: set counters to zero.
!   Construct a few demo "unit tests" to show use of calls to UT_Assert().
!   Print the result.
!
! EXAMPLE:
!   call UT_DemoExampleFailAll()
!
! MODIFICATION HISTORY:
!   2015-03-04 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
real                                    :: delta
real, dimension(2,2)                    :: M1, M2, Munity
!---------------------------------------------------------------------------------------------------
!
UT%nc_p=6
!
! --- Initialize unit-test counters ---
call UT_Init()
UT%AllTestPrint_flag  =.true.
UT%AllVecOutPrint_flag=.true.
!
! === Perform some simple demo tests ===
!
! --- Create unity matrix ---
Munity(1,1)=1.0  ;  Munity(2,1)=0.0
Munity(1,2)=0.0  ;  Munity(2,2)=1.0
!
! --- Create demo matrices ---
M1(1,1)=1.1  ;  M1(2,1)=2.1
M1(1,2)=1.2  ;  M1(2,2)=2.2
!
M2=M1 ; M2(1,1)=M2(1,1)+1.0E-04 ; M2(2,2)=M2(2,2)-1.0E-04
!
call UT_Assert('==', 'Matrix1IsEqualToMatrix2'           , M1, M2)
call UT_Assert('==', 'Matrix1IsAlmostEqualToMatrix2'     , M1, M2, delta=1.0E-05)
call UT_Assert('<=', 'Matrix1IsSmallerOrEqualThanMatrix2', M1, M2)
call UT_Assert('>=', 'Matrix2IsLargerOrEqualThanMatrix1' , M2, M1)
call UT_Assert('<=', 'Matrix1IsSmallerOrEqualThanUnity'  , M1, Munity)
!
call UT_Assert('<=', 'Matrix2IsSmallerOrEqualThanUnity'  , M2, Munity, Ignore_flag=.true.)
!
! --- Print test summary ---
call UT_PrintSummary()
!
end subroutine UT_DemoExampleFailAll

end module UT_module
