!***************************************************************************************************
!
!                        ######  #######                     
!   #####  #    # #####  #     #    #
!   #    # #    # #    # #     #    #
!   #    # ###### #    # ######     #
!   #####  #    # #    # #          #
!   #   #  #    # #    # #          #
!   #    # #    # #####  #          #
!
!   Radiation-Hydrodynamics-Code: performance tests
!
!***************************************************************************************************
!   Fortran90
!   2016-03-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhdPT_simple_module:   Provide simple performance-test routines for RHD code
!   rhdPT_master_module:   Master performance-test routine for RHD code
!
! PROGRAM:
!   rhdPT:                 Do performance tests for RHD code
!
!***************************************************************************************************

!# /* --- Set default values for some preprocessor macros --------------------------------------- */
!#
!# /* --- Select which modules to include in the test (default: all) --- */
!#
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*******************--------------------------------------------------------------------------
module rhdPT_simple_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_module ('radiation_hydrodynamics_Performance_Test_simple_module')
!
! PURPOSE:
!   Provide simple performance-test routines for RHD code.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhdPT_simple_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhdPT_simple_InitArrays: Initialize some arrays
!   rhdPT_simple_TestFew:  Do a few simple performance tests
!   rhdPT_simple_TestMany: Do a number of simple performance tests
!   rhdPT_simple_TestCallAdd1: Do a simple algebraic operation
!   rhdPT_simple_TestCallAdd2: Do a simple algebraic operation
!   rhdPT_simple_TestCall: Do a some performance tests with loop kernels containing calls
!   rhdPT_simple_TestStencil1D: Do a performance tests of simple stencil operations
!   rhdPT_simple_TestAll:  Do all performance tests for RHD code
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------***********************------------------------------------------------------------------
subroutine rhdPT_simple_InitArrays(n1, n2, a, b, c)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_InitArrays ('radiation_hydrodynamics_Performance_Test_simple_Init_Arrays')
!
! PURPOSE:
!   Initialize some arrays.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_InitArrays(n1, n2, a, b, c)
!
! INPUT:
!
! OUTPUT:
!
! ROUTINES:
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_InitArrays(128, 32, a, b, c)
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: n1, n2
real, dimension(n1  ,n2),  intent(out)  :: a, b, c
!
! --- Local variables ---
integer                                 :: i1, i2
!---------------------------------------------------------------------------------------------------
!
! --- Interrupt timing counter ---
call timing_stop('Performance Test total', cycles=0)
!
!do i1=1,n1
!  ri1(i1)=real(i1)
!end do ! i1
!
!$OMP PARALLEL DEFAULT(SHARED)
!
!$OMP DO PRIVATE(i1,i2)
do i2=1,n2
!  ri2=real(i2)
  do i1=1,n1
!    a(i1,i2)=0.2+0.00001*ri1(i1)
!    b(i1,i2)=0.3+0.0001 *ri1(i1)
!    c(i1,i2)=0.4+0.0001 *ri2
    a(i1,i2)=0.2+0.00001*real(i1)
    !b(i1,i2)=0.3+0.0001 *real(i1)
    b(i1,i2)=0.9999+1.0E-8*real(i1)
    !c(i1,i2)=0.4+0.0001 *real(i2)
    c(i1,i2)=0.1+sin(0.1 *real(i2))
!    a(i1,i2)=0.2
!    b(i1,i2)=0.3
!    c(i1,i2)=0.4
!write(6,*) 'i1', i1, i2, a(i1,i2), b(i1,i2), c(i1,i2)
  end do ! i1
end do ! i2
!$OMP END DO
!
!$OMP END PARALLEL
!
! --- Restart timing counter ---
call timing_start('Performance Test total')
!
end subroutine rhdPT_simple_InitArrays


!----------********************---------------------------------------------------------------------
subroutine rhdPT_simple_TestFew(nc, n1, n2, nRepeat1, nRepeat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestFew ('radiation_hydrodynamics_Performance_Test_simple_Test_Few')
!
! PURPOSE:
!   Do a few simple performance tests.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestFew(nc, n1, n2, nRepeat1, nRepeat2)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   n1:           ('number_1') integer with number of elements in 1st dimension
!   n2:           ('number_2') integer with number of elements in 2nd dimension
!   nRepeat1:     ('number_Repeat_1') integer with number of (inner) repetitions
!   nRepeat2:     ('number_Repeat_2') integer with number of (outer) repetitions
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! ROUTINES:
!   rhdPT_simple_InitArrays: Initialize some arrays
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_Write:          Write results of counting process into table
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestFew(6, 256, 128, 100, 10)
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, n1, n2, nRepeat1, nRepeat2
!
! --- Local variables ---
integer                                 :: i1, i2, iRepeat1, iRepeat2, timing_handle
real                                    :: out, timing_factor
real, dimension(n1  ,n2)                :: a, b, c
!---------------------------------------------------------------------------------------------------
!
out=0.0
!
! --- Initialize timing routines ---
call timing_Init()
call timing_start('Performance Test total')
!
timing_factor=real(n1*n2*nRepeat1)
!
! --- Repeat whole test parcour nrepeat2 times ---
do iRepeat2=1,nRepeat2
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+b+c', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)+c(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+b*c', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)*c(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
end do ! iRepeat2
!
call timing_stop('Performance Test total', factor=0.0)
!
write(6,'(7E8.1)') out, minval(a), maxval(a), minval(b), maxval(b), minval(c), maxval(c)
write(6,'(A,I7)') 'nRepeat1 =', nRepeat1
write(6,'(A,I7)') 'nRepeat2 =', nRepeat2
write(6,'(A,I7)') 'n1       =', n1
write(6,'(A,I7)') 'n2       =', n2
!
call timing_Write(nc, nThread=1, ScaleExponent=-9)
!
end subroutine rhdPT_simple_TestFew


!----------*********************--------------------------------------------------------------------
subroutine rhdPT_simple_TestMany(nc, n1, n2, nRepeat1, nRepeat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestMany ('radiation_hydrodynamics_Performance_Test_simple_Test_Many')
!
! PURPOSE:
!   Do a number of simple performance tests.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestMany(nc, n1, n2, nRepeat1, nRepeat2)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   n1:           ('number_1') integer with number of elements in 1st dimension
!   n2:           ('number_2') integer with number of elements in 2nd dimension
!   nRepeat1:     ('number_Repeat_1') integer with number of (inner) repetitions
!   nRepeat2:     ('number_Repeat_2') integer with number of (outer) repetitions
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! ROUTINES:
!   rhdPT_simple_InitArrays: Initialize some arrays
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_Write:          Write results of counting process into table
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestMany(6, 128, 126, 100, 10)
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, n1, n2, nRepeat1, nRepeat2
!
! --- Local variables ---
integer                                 :: i1, i2, iRepeat1, iRepeat2, timing_handle
real                                    :: out, timing_factor
real, dimension(n1  ,n2)                :: a, b, c
real, parameter                         :: Clog0234=alog(0.234)
!---------------------------------------------------------------------------------------------------
!
out=0.0
!
! --- Initialize timing routines ---
call timing_Init()
call timing_start('Performance Test total')
!
timing_factor=real(n1*n2*nRepeat1)
!
! --- Repeat whole test parcour nrepeat2 times ---
do iRepeat2=1,nRepeat2
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=0', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=0.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a-1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)-1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a*0.999', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)*0.999
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a/1.001', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)/1.001
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a*(1/1.001)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)*(1.0/1.1)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a*0.999+0.001', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)*0.999+0.001
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=a*a', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)*a(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=1/a', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=1.0/a(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=p1(a)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=0.09*a(i1,i2)-0.05
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=p2(a)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(0.09*a(i1,i2)-0.05)*a(i1,i2)-0.03
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=p3(a)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=((0.09*a(i1,i2)-0.05)*a(i1,i2)-0.03)*a(i1,i2)-0.04
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=p4(a)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.09*a(i1,i2)-0.05)*a(i1,i2)-0.03)*a(i1,i2)-0.04)*a(i1,i2)-0.06
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=p5(a)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=((((0.09*a(i1,i2)-0.05)*a(i1,i2)-0.03)*a(i1,i2)-0.04)*a(i1,i2)-0.06)*a(i1,i2)-0.01
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b+c', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)+c(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=a+b+c', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)+c(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a*b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)*b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=a/b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)/b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a*(1/b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)*(1.0/b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=p4x1(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.094
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x2(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.094
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x4(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.094
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
        a(i1,i2)=(((0.070*a(i1,i2)-0.071)*b(i1,i2)-0.072)*b(i1,i2)-0.073)*b(i1,i2)-0.074
        a(i1,i2)=(((0.060*a(i1,i2)-0.061)*b(i1,i2)-0.062)*b(i1,i2)-0.063)*b(i1,i2)-0.064
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x8(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.0941
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
        a(i1,i2)=(((0.070*a(i1,i2)-0.071)*b(i1,i2)-0.072)*b(i1,i2)-0.073)*b(i1,i2)-0.074
        a(i1,i2)=(((0.060*a(i1,i2)-0.061)*b(i1,i2)-0.062)*b(i1,i2)-0.063)*b(i1,i2)-0.064
        a(i1,i2)=(((0.050*a(i1,i2)-0.051)*b(i1,i2)-0.052)*b(i1,i2)-0.053)*b(i1,i2)-0.054
        a(i1,i2)=(((0.040*a(i1,i2)-0.041)*b(i1,i2)-0.042)*b(i1,i2)-0.043)*b(i1,i2)-0.044
        a(i1,i2)=(((0.030*a(i1,i2)-0.031)*b(i1,i2)-0.032)*b(i1,i2)-0.033)*b(i1,i2)-0.034
        a(i1,i2)=(((0.020*a(i1,i2)-0.021)*b(i1,i2)-0.022)*b(i1,i2)-0.023)*b(i1,i2)-0.024
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x8/2(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.0942
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
        a(i1,i2)=(((0.070*a(i1,i2)-0.071)*b(i1,i2)-0.072)*b(i1,i2)-0.073)*b(i1,i2)-0.074
        a(i1,i2)=(((0.060*a(i1,i2)-0.061)*b(i1,i2)-0.062)*b(i1,i2)-0.063)*b(i1,i2)-0.064
      end do ! i1
      do i1=1,n1
        a(i1,i2)=(((0.050*a(i1,i2)-0.051)*b(i1,i2)-0.052)*b(i1,i2)-0.053)*b(i1,i2)-0.054
        a(i1,i2)=(((0.040*a(i1,i2)-0.041)*b(i1,i2)-0.042)*b(i1,i2)-0.043)*b(i1,i2)-0.044
        a(i1,i2)=(((0.030*a(i1,i2)-0.031)*b(i1,i2)-0.032)*b(i1,i2)-0.033)*b(i1,i2)-0.034
        a(i1,i2)=(((0.020*a(i1,i2)-0.021)*b(i1,i2)-0.022)*b(i1,i2)-0.023)*b(i1,i2)-0.024
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x16(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.0943
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
        a(i1,i2)=(((0.070*a(i1,i2)-0.071)*b(i1,i2)-0.072)*b(i1,i2)-0.073)*b(i1,i2)-0.074
        a(i1,i2)=(((0.060*a(i1,i2)-0.061)*b(i1,i2)-0.062)*b(i1,i2)-0.063)*b(i1,i2)-0.064
        a(i1,i2)=(((0.050*a(i1,i2)-0.051)*b(i1,i2)-0.052)*b(i1,i2)-0.053)*b(i1,i2)-0.054
        a(i1,i2)=(((0.040*a(i1,i2)-0.041)*b(i1,i2)-0.042)*b(i1,i2)-0.043)*b(i1,i2)-0.044
        a(i1,i2)=(((0.030*a(i1,i2)-0.031)*b(i1,i2)-0.032)*b(i1,i2)-0.033)*b(i1,i2)-0.034
        a(i1,i2)=(((0.020*a(i1,i2)-0.021)*b(i1,i2)-0.022)*b(i1,i2)-0.023)*b(i1,i2)-0.024
        a(i1,i2)=(((0.190*a(i1,i2)-0.191)*b(i1,i2)-0.192)*b(i1,i2)-0.193)*b(i1,i2)-0.194
        a(i1,i2)=(((0.180*a(i1,i2)-0.181)*b(i1,i2)-0.182)*b(i1,i2)-0.183)*b(i1,i2)-0.184
        a(i1,i2)=(((0.170*a(i1,i2)-0.171)*b(i1,i2)-0.172)*b(i1,i2)-0.173)*b(i1,i2)-0.174
        a(i1,i2)=(((0.160*a(i1,i2)-0.161)*b(i1,i2)-0.162)*b(i1,i2)-0.163)*b(i1,i2)-0.164
        a(i1,i2)=(((0.150*a(i1,i2)-0.151)*b(i1,i2)-0.152)*b(i1,i2)-0.153)*b(i1,i2)-0.154
        a(i1,i2)=(((0.140*a(i1,i2)-0.141)*b(i1,i2)-0.142)*b(i1,i2)-0.143)*b(i1,i2)-0.144
        a(i1,i2)=(((0.130*a(i1,i2)-0.131)*b(i1,i2)-0.132)*b(i1,i2)-0.133)*b(i1,i2)-0.134
        a(i1,i2)=(((0.120*a(i1,i2)-0.121)*b(i1,i2)-0.122)*b(i1,i2)-0.123)*b(i1,i2)-0.124
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x16/4(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.0944
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
        a(i1,i2)=(((0.070*a(i1,i2)-0.071)*b(i1,i2)-0.072)*b(i1,i2)-0.073)*b(i1,i2)-0.074
        a(i1,i2)=(((0.060*a(i1,i2)-0.061)*b(i1,i2)-0.062)*b(i1,i2)-0.063)*b(i1,i2)-0.064
      end do ! i1
      do i1=1,n1
        a(i1,i2)=(((0.050*a(i1,i2)-0.051)*b(i1,i2)-0.052)*b(i1,i2)-0.053)*b(i1,i2)-0.054
        a(i1,i2)=(((0.040*a(i1,i2)-0.041)*b(i1,i2)-0.042)*b(i1,i2)-0.043)*b(i1,i2)-0.044
        a(i1,i2)=(((0.030*a(i1,i2)-0.031)*b(i1,i2)-0.032)*b(i1,i2)-0.033)*b(i1,i2)-0.034
        a(i1,i2)=(((0.020*a(i1,i2)-0.021)*b(i1,i2)-0.022)*b(i1,i2)-0.023)*b(i1,i2)-0.024
      end do ! i1
      do i1=1,n1
        a(i1,i2)=(((0.190*a(i1,i2)-0.191)*b(i1,i2)-0.192)*b(i1,i2)-0.193)*b(i1,i2)-0.194
        a(i1,i2)=(((0.180*a(i1,i2)-0.181)*b(i1,i2)-0.182)*b(i1,i2)-0.183)*b(i1,i2)-0.184
        a(i1,i2)=(((0.170*a(i1,i2)-0.171)*b(i1,i2)-0.172)*b(i1,i2)-0.173)*b(i1,i2)-0.174
        a(i1,i2)=(((0.160*a(i1,i2)-0.161)*b(i1,i2)-0.162)*b(i1,i2)-0.163)*b(i1,i2)-0.164
      end do ! i1
      do i1=1,n1
        a(i1,i2)=(((0.150*a(i1,i2)-0.151)*b(i1,i2)-0.152)*b(i1,i2)-0.153)*b(i1,i2)-0.154
        a(i1,i2)=(((0.140*a(i1,i2)-0.141)*b(i1,i2)-0.142)*b(i1,i2)-0.143)*b(i1,i2)-0.144
        a(i1,i2)=(((0.130*a(i1,i2)-0.131)*b(i1,i2)-0.132)*b(i1,i2)-0.133)*b(i1,i2)-0.134
        a(i1,i2)=(((0.120*a(i1,i2)-0.121)*b(i1,i2)-0.122)*b(i1,i2)-0.123)*b(i1,i2)-0.124
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=p4x32(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO DEFAULT(SHARED)
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=(((0.090*b(i1,i2)-0.091)*b(i1,i2)-0.092)*b(i1,i2)-0.093)*b(i1,i2)-0.0945
        a(i1,i2)=(((0.080*a(i1,i2)-0.081)*b(i1,i2)-0.082)*b(i1,i2)-0.083)*b(i1,i2)-0.084
        a(i1,i2)=(((0.070*a(i1,i2)-0.071)*b(i1,i2)-0.072)*b(i1,i2)-0.073)*b(i1,i2)-0.074
        a(i1,i2)=(((0.060*a(i1,i2)-0.061)*b(i1,i2)-0.062)*b(i1,i2)-0.063)*b(i1,i2)-0.064
        a(i1,i2)=(((0.050*a(i1,i2)-0.051)*b(i1,i2)-0.052)*b(i1,i2)-0.053)*b(i1,i2)-0.054
        a(i1,i2)=(((0.040*a(i1,i2)-0.041)*b(i1,i2)-0.042)*b(i1,i2)-0.043)*b(i1,i2)-0.044
        a(i1,i2)=(((0.030*a(i1,i2)-0.031)*b(i1,i2)-0.032)*b(i1,i2)-0.033)*b(i1,i2)-0.034
        a(i1,i2)=(((0.020*a(i1,i2)-0.021)*b(i1,i2)-0.022)*b(i1,i2)-0.023)*b(i1,i2)-0.024
        a(i1,i2)=(((0.190*a(i1,i2)-0.191)*b(i1,i2)-0.192)*b(i1,i2)-0.193)*b(i1,i2)-0.194
        a(i1,i2)=(((0.180*a(i1,i2)-0.181)*b(i1,i2)-0.182)*b(i1,i2)-0.183)*b(i1,i2)-0.184
        a(i1,i2)=(((0.170*a(i1,i2)-0.171)*b(i1,i2)-0.172)*b(i1,i2)-0.173)*b(i1,i2)-0.174
        a(i1,i2)=(((0.160*a(i1,i2)-0.161)*b(i1,i2)-0.162)*b(i1,i2)-0.163)*b(i1,i2)-0.164
        a(i1,i2)=(((0.150*a(i1,i2)-0.151)*b(i1,i2)-0.152)*b(i1,i2)-0.153)*b(i1,i2)-0.154
        a(i1,i2)=(((0.140*a(i1,i2)-0.141)*b(i1,i2)-0.142)*b(i1,i2)-0.143)*b(i1,i2)-0.144
        a(i1,i2)=(((0.130*a(i1,i2)-0.131)*b(i1,i2)-0.132)*b(i1,i2)-0.133)*b(i1,i2)-0.134
        a(i1,i2)=(((0.120*a(i1,i2)-0.121)*b(i1,i2)-0.122)*b(i1,i2)-0.123)*b(i1,i2)-0.124
        a(i1,i2)=(((0.290*a(i1,i2)-0.291)*b(i1,i2)-0.292)*b(i1,i2)-0.293)*b(i1,i2)-0.294
        a(i1,i2)=(((0.280*a(i1,i2)-0.281)*b(i1,i2)-0.282)*b(i1,i2)-0.283)*b(i1,i2)-0.284
        a(i1,i2)=(((0.270*a(i1,i2)-0.271)*b(i1,i2)-0.272)*b(i1,i2)-0.273)*b(i1,i2)-0.274
        a(i1,i2)=(((0.260*a(i1,i2)-0.261)*b(i1,i2)-0.262)*b(i1,i2)-0.263)*b(i1,i2)-0.264
        a(i1,i2)=(((0.250*a(i1,i2)-0.251)*b(i1,i2)-0.252)*b(i1,i2)-0.253)*b(i1,i2)-0.254
        a(i1,i2)=(((0.240*a(i1,i2)-0.241)*b(i1,i2)-0.242)*b(i1,i2)-0.243)*b(i1,i2)-0.244
        a(i1,i2)=(((0.230*a(i1,i2)-0.231)*b(i1,i2)-0.232)*b(i1,i2)-0.233)*b(i1,i2)-0.234
        a(i1,i2)=(((0.220*a(i1,i2)-0.221)*b(i1,i2)-0.222)*b(i1,i2)-0.223)*b(i1,i2)-0.224
        a(i1,i2)=(((0.390*a(i1,i2)-0.391)*b(i1,i2)-0.392)*b(i1,i2)-0.393)*b(i1,i2)-0.394
        a(i1,i2)=(((0.380*a(i1,i2)-0.381)*b(i1,i2)-0.382)*b(i1,i2)-0.383)*b(i1,i2)-0.384
        a(i1,i2)=(((0.370*a(i1,i2)-0.371)*b(i1,i2)-0.372)*b(i1,i2)-0.373)*b(i1,i2)-0.374
        a(i1,i2)=(((0.360*a(i1,i2)-0.361)*b(i1,i2)-0.362)*b(i1,i2)-0.363)*b(i1,i2)-0.364
        a(i1,i2)=(((0.350*a(i1,i2)-0.351)*b(i1,i2)-0.352)*b(i1,i2)-0.353)*b(i1,i2)-0.354
        a(i1,i2)=(((0.340*a(i1,i2)-0.341)*b(i1,i2)-0.342)*b(i1,i2)-0.343)*b(i1,i2)-0.344
        a(i1,i2)=(((0.330*a(i1,i2)-0.331)*b(i1,i2)-0.332)*b(i1,i2)-0.333)*b(i1,i2)-0.334
        a(i1,i2)=(((0.320*a(i1,i2)-0.321)*b(i1,i2)-0.322)*b(i1,i2)-0.323)*b(i1,i2)-0.324
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=sqrt(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=sqrt(b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=sqrt(sqrt(b))', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=sqrt(sqrt(b(i1,i2)))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=exp(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=exp(b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=exp(2*b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=exp(2.0*b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=exp(10*b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=exp(10.0*b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=exp(sqrt(b))', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=exp(sqrt(b(i1,i2)))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=log(b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=log(b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=log(2*b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=log(2.0*b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=log(100*b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=log(100.0*b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b^1/2', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)**(1.0/2.0)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b^1/3', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)**(1.0/3.0)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b^1/4', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)**(1.0/4.0)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b^0.234', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)**0.234
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=exp(log(0.234)*log(b))', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=exp(log(0.234)*log(b(i1,i2)))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=log(b),a=log(0.234)*a,a=exp(a)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=log(b(i1,i2))
      end do ! i1
      do i1=1,n1
        a(i1,i2)=log(0.234)*a(i1,i2)
      end do ! i1
      do i1=1,n1
        a(i1,i2)=exp(a(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
!!$  call timing_GetHandle('a=exp(Clog0234*alog(b))', timing_handle)
!!$  call timing_start(timing_handle)
!!$  do iRepeat1=1,nRepeat1
!!$    !$OMP PARALLEL DO
!!$    do i2=1,n2
!!$      do i1=1,n1
!!$        a(i1,i2)=exp(Clog0234*alog(b(i1,i2)))
!!$      end do ! i1
!!$    end do ! i2
!!$    !$OMP END PARALLEL DO
!!$    !
!!$    out=0.001*out+a(1,1)
!!$  end do ! iRepeat1
!!$  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=abs(c)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=abs(c(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=max(c,0)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=max(c(i1,i2),0.0)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=min(max(c,0),1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=min(max(c(i1,i2),0.0),1.0)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=sign(1,c)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=sign(1.0,c(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=real(int(c)+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=real(int(c(i1,i2))+1)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=sgl(dbl(c)+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=real(real(c(i1,i2), kind=kind(0.0D+00))+1.0D+00, kind=kind(0.0E+00))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=if(b,c)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        if (c(i1,i2) > 0.0) then
          a(i1,i2)=c(i1,i2)
        else
          a(i1,i2)=b(i1,i2)
        endif
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=ifif(b,c)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        if (c(i1,i2) > 0.0) then
          a(i1,i2)=c(i1,i2)
        else
          if (b(i1,i2) > 0.0) then
            a(i1,i2)=b(i1,i2)
          else
            a(i1,i2)=c(i1,i2)
          endif
        endif
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
end do ! iRepeat2
!
call timing_stop('Performance Test total', factor=0.0)
!
write(6,'(7E9.1)') out, minval(a), maxval(a), minval(b), maxval(b), minval(c), maxval(c)
write(6,'(A,I7)') 'nRepeat1 =', nRepeat1
write(6,'(A,I7)') 'nRepeat2 =', nRepeat2
write(6,'(A,I7)') 'n1       =', n1
write(6,'(A,I7)') 'n2       =', n2
!
call timing_Write(nc, nThread=1, ScaleExponent=-9)
!
end subroutine rhdPT_simple_TestMany


!----------****************************-------------------------------------------------------------
subroutine rhdPT_simple_TestShortVecAdd(nc, n2, nRepeat1, nRepeat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestShortVecAdd ('rhd_Performance_Test_simple_Test_Short_Vector_Add')
!
! PURPOSE:
!   Do a few simple performance tests on arrays with small first dimension.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestShortVecAdd(nc, n2, nRepeat1, nRepeat2)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   n2:           ('number_2') integer with number of elements in 2nd dimension
!   nRepeat1:     ('number_Repeat_1') integer with number of (inner) repetitions
!   nRepeat2:     ('number_Repeat_2') integer with number of (outer) repetitions
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! ROUTINES:
!   rhdPT_simple_InitArrays: Initialize some arrays
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_Write:          Write results of counting process into table
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestShortVecAdd(6, 128, 100, 10)
!
! MODIFICATION HISTORY:
!   2015-10-28 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, n2, nRepeat1, nRepeat2
!
! --- Local variables ---
integer                                 :: i1, i2, iRepeat1, iRepeat2, timing_handle
real                                    :: out, timing_factor
real, dimension( 1, n2   )              :: a01
real, dimension( 2, n2/ 2)              :: a02
real, dimension( 4, n2/ 4)              :: a04
real, dimension( 8, n2/ 8)              :: a08
real, dimension(16, n2/16)              :: a16
real, dimension(32, n2/32)              :: a32
real, dimension(64, n2/64)              :: a64
!---------------------------------------------------------------------------------------------------
!
out=0.0
!
! --- Initialize timing routines ---
call timing_Init()
call timing_start('Performance Test total')
!
timing_factor=real(n2*nRepeat1)
!
! --- Repeat whole test parcour nrepeat2 times ---
do iRepeat2=1,nRepeat2
  !
  ! --- Initialize arrays ---
  a01=0.0
  a02=0.0
  a04=0.0
  a08=0.0
  a16=0.0
  a32=0.0
  a64=0.0
  !
  call timing_GetHandle('a01=a01+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,1
        a01(i1,i2)=a01(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a01(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a02=a02+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2/2
      do i1=1,2
        a02(i1,i2)=a02(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a02(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a04=a04+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2/4
      do i1=1,4
        a04(i1,i2)=a04(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a04(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a08=a08+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2/8
      do i1=1,8
        a08(i1,i2)=a08(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a08(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a16=a16+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2/16
      do i1=1,16
        a16(i1,i2)=a16(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a16(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a32=a32+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2/32
      do i1=1,32
        a32(i1,i2)=a32(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a32(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a64=a64+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2/64
      do i1=1,64
        a64(i1,i2)=a64(i1,i2)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a64(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
end do ! iRepeat2
!
call timing_stop('Performance Test total', factor=0.0)
!
write(6,'(8E8.1)') out, &
                   maxval(a01), maxval(a02), maxval(a04), maxval(a08), &
                   maxval(a16), maxval(a32), maxval(a64)
write(6,'(A,I7)') 'nRepeat1 =', nRepeat1
write(6,'(A,I7)') 'nRepeat2 =', nRepeat2
write(6,'(A,I7)') 'n2       =', n2
!
call timing_Write(nc, nThread=1, ScaleExponent=-9)
!
end subroutine rhdPT_simple_TestShortVecAdd


!----------****************************-------------------------------------------------------------
subroutine rhdPT_simple_TestShortVecInt(nc, n2, nRepeat1, nRepeat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestShortVecInt ('rhd_Performance_Test_simple_Test_Short_Vector_Integrate')
!
! PURPOSE:
!   Do a few simple performance tests on arrays with small first dimension.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestShortVecInt(nc, n2, nRepeat1, nRepeat2)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   n2:           ('number_2') integer with number of elements in 2nd dimension
!   nRepeat1:     ('number_Repeat_1') integer with number of (inner) repetitions
!   nRepeat2:     ('number_Repeat_2') integer with number of (outer) repetitions
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! ROUTINES:
!   rhdPT_simple_InitArrays: Initialize some arrays
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_Write:          Write results of counting process into table
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestShortVecInt(6, 128, 100, 10)
!
! MODIFICATION HISTORY:
!   2015-10-28 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, n2, nRepeat1, nRepeat2
!
! --- Local variables ---
integer                                 :: i1, i2, iRepeat1, iRepeat2, timing_handle
real                                    :: out, timing_factor
real, dimension( 1, n2   )              :: a01
real, dimension( 2, n2/ 2)              :: a02
real, dimension( 4, n2/ 4)              :: a04
real, dimension( 8, n2/ 8)              :: a08
real, dimension(16, n2/16)              :: a16
real, dimension(32, n2/32)              :: a32
real, dimension(64, n2/64)              :: a64
!---------------------------------------------------------------------------------------------------
!
out=0.0
!
! --- Initialize timing routines ---
call timing_Init()
call timing_start('Performance Test total')
!
timing_factor=real(n2*nRepeat1)
!
! --- Repeat whole test parcour nrepeat2 times ---
do iRepeat2=1,nRepeat2
  !
  ! --- Initialize arrays ---
  a01=0.0
  a02=0.0
  a04=0.0
  a08=0.0
  a16=0.0
  a32=0.0
  a64=0.0
  !
  call timing_GetHandle('a01=a01(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2
      do i1=1,1
        a01(i1,i2)=a01(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a01(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a02=a02(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2/2
      do i1=1,2
        a02(i1,i2)=a02(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a02(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a04=a04(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2/4
      do i1=1,4
        a04(i1,i2)=a04(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a04(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a08=a08(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2/8
      do i1=1,8
        a08(i1,i2)=a08(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a08(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a16=a16(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2/16
      do i1=1,16
        a16(i1,i2)=a16(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a16(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a32=a32(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2/32
      do i1=1,32
        a32(i1,i2)=a32(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a32(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a64=a64(-1)+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=2,n2/64
      do i1=1,64
        a64(i1,i2)=a64(i1,i2-1)+1.0
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a64(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
end do ! iRepeat2
!
call timing_stop('Performance Test total', factor=0.0)
!
write(6,'(8E8.1)') out, &
                   maxval(a01), maxval(a02), maxval(a04), maxval(a08), &
                   maxval(a16), maxval(a32), maxval(a64)
write(6,'(A,I7)') 'nRepeat1 =', nRepeat1
write(6,'(A,I7)') 'nRepeat2 =', nRepeat2
write(6,'(A,I7)') 'n2       =', n2
!
call timing_Write(nc, nThread=1, ScaleExponent=-9)
!
end subroutine rhdPT_simple_TestShortVecInt


!----------*************************----------------------------------------------------------------
subroutine rhdPT_simple_TestCallAdd1(a, b)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestCallAdd1 ('radiation_hydrodynamics_Performance_Test_simple_Test_Call_Add_1')
!
! PURPOSE:
!   Do a simple algebraic operation.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestCallAdd1(a, b)
!
! INPUT:
!   b:            ('b') real with 2nd operand
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! INPUT/OUTPUT:
!   a:            ('a') real with 1st operand and result
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   None
!
! PROCEDURE:
!   Simply compute a=a+b.
!
! EXAMPLE:
!   call rhdPT_simple_TestCallAdd1(a, 2.0)
!
! MODIFICATION HISTORY:
!   2015-05-21 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(inout):: a
real,                      intent(in)   :: b
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
a=a+b
!
end subroutine rhdPT_simple_TestCallAdd1


!----------*************************----------------------------------------------------------------
subroutine rhdPT_simple_TestCallAdd2(a, b)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestCallAdd2 ('radiation_hydrodynamics_Performance_Test_simple_Test_Call_Add_2')
!
! PURPOSE:
!   Do a simple algebraic operation.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestCallAdd2(a, b)
!
! INPUT:
!   b:            ('b') real with 2nd operand
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! INPUT/OUTPUT:
!   a:            ('a') real with 1st operand and result
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   None
!
! PROCEDURE:
!   Simply compute a=a+b*b.
!
! EXAMPLE:
!   call rhdPT_simple_TestCallAdd2(a, 2.0)
!
! MODIFICATION HISTORY:
!   2015-05-21 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(inout):: a
real,                      intent(in)   :: b
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
a=a+b*b
!
end subroutine rhdPT_simple_TestCallAdd2


!----------*********************--------------------------------------------------------------------
subroutine rhdPT_simple_TestCall(nc, n1, n2, nRepeat1, nRepeat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestCall ('radiation_hydrodynamics_Performance_Test_simple_Test_Call')
!
! PURPOSE:
!   Do a some performance tests with loop kernels containing calls.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestCall(nc, n1, n2, nRepeat1, nRepeat2)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   n1:           ('number_1') integer with number of elements in 1st dimension
!   n2:           ('number_2') integer with number of elements in 2nd dimension
!   nRepeat1:     ('number_Repeat_1') integer with number of (inner) repetitions
!   nRepeat2:     ('number_Repeat_2') integer with number of (outer) repetitions
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! ROUTINES:
!   rhdPT_simple_InitArrays: Initialize some arrays
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_Write:          Write results of counting process into table
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestCall(6, 256, 128, 100, 10)
!
! MODIFICATION HISTORY:
!   2015-05-21 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, n1, n2, nRepeat1, nRepeat2
!
! --- Local variables ---
integer                                 :: i1, i2, iRepeat1, iRepeat2, timing_handle
real                                    :: out, timing_factor
real, dimension(n1  ,n2)                :: a, b, c
!---------------------------------------------------------------------------------------------------
!
out=0.0
!
! --- Initialize timing routines ---
call timing_Init()
call timing_start('Performance Test total')
!
timing_factor=real(n1*n2*nRepeat1)
!
! --- Repeat whole test parcour nrepeat2 times ---
do iRepeat2=1,nRepeat2
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=a+1', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+1
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=a+b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('call(a=a+b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        call rhdPT_simple_TestCallAdd1(a(i1,i2), b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('a=a+b*b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=a(i1,i2)+b(i1,i2)*b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !
  call timing_GetHandle('call(a=a+b*b)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        call rhdPT_simple_TestCallAdd2(a(i1,i2), b(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
end do ! iRepeat2
!
call timing_stop('Performance Test total', factor=0.0)
!
write(6,'(7E8.1)') out, minval(a), maxval(a), minval(b), maxval(b), minval(c), maxval(c)
write(6,'(A,I7)') 'nRepeat1 =', nRepeat1
write(6,'(A,I7)') 'nRepeat2 =', nRepeat2
write(6,'(A,I7)') 'n1       =', n1
write(6,'(A,I7)') 'n2       =', n2
!
call timing_Write(nc, nThread=1, ScaleExponent=-9)
!
end subroutine rhdPT_simple_TestCall


!----------**************************---------------------------------------------------------------
subroutine rhdPT_simple_TestStencil1D(nc, n1, n2, nRepeat1, nRepeat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestStencil1D ('rhd_Performance_Test_simple_Test_Stencil_1_Dimension')
!
! PURPOSE:
!   Do a performance tests of simple stencil operations.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestStencil1D(nc, n1, n2, nRepeat1, nRepeat2)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   n1:           ('number_1') integer with number of elements in 1st dimension
!   n2:           ('number_2') integer with number of elements in 2nd dimension
!   nRepeat1:     ('number_Repeat_1') integer with number of (inner) repetitions
!   nRepeat2:     ('number_Repeat_2') integer with number of (outer) repetitions
!
! OUTPUT:
!   None (only the global timing counters are incremented)
!
! ROUTINES:
!   rhdPT_simple_InitArrays: Initialize some arrays
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_Write:          Write results of counting process into table
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestStencil1D(6, 256, 128, 100, 10)
!
! MODIFICATION HISTORY:
!   2016-03-10 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, n1, n2, nRepeat1, nRepeat2
!
! --- Local variables ---
integer                                 :: i1, i2, iRepeat1, iRepeat2, timing_handle
real                                    :: out, timing_factor
real, dimension(n1  ,n2)                :: a, b, c
!---------------------------------------------------------------------------------------------------
!
out=0.0
!
! --- Initialize timing routines ---
call timing_Init()
call timing_start('Performance Test total')
!
timing_factor=real(n1*n2*nRepeat1)
!
! --- Repeat whole test parcour nrepeat2 times ---
do iRepeat2=1,nRepeat2
  !
  ! --- Initialize arrays ---
  call rhdPT_simple_InitArrays(n1, n2, a, b, c)
  !  
  call timing_GetHandle('a=b', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b(+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1-1
        a(i1,i2)=b(i1+1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b(-1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=2,n1
        a(i1,i2)=b(i1-1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(2,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b(0)+b(+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1-1
        a(i1,i2)=b(i1,i2)+b(i1+1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b(-1)+b(+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=2,n1-1
        a(i1,i2)=b(i1-1,i2)+b(i1+1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=2*b(0)-b(-1)-b(+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=2,n1-1
        a(i1,i2)=2.0*b(i1,i2)-b(i1-1,i2)-b(i1+1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b+c', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=1,n1
        a(i1,i2)=b(i1,i2)+c(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
  call timing_GetHandle('a=b(-1)+c(+1)', timing_handle)
  call timing_start(timing_handle)
  do iRepeat1=1,nRepeat1
    !$OMP PARALLEL DO
    do i2=1,n2
      do i1=2,n1-1
        a(i1,i2)=b(i1-1,i2)+c(i1+1,i2)
      end do ! i1
    end do ! i2
    !$OMP END PARALLEL DO
    !
    out=0.001*out+a(1,1)
  end do ! iRepeat1
  call timing_stop(timing_handle, factor=timing_factor)
  !
end do ! iRepeat2
!
call timing_stop('Performance Test total', factor=0.0)
!
write(6,'(7E8.1)') out, minval(a), maxval(a), minval(b), maxval(b), minval(c), maxval(c)
write(6,'(A,I7)') 'nRepeat1 =', nRepeat1
write(6,'(A,I7)') 'nRepeat2 =', nRepeat2
write(6,'(A,I7)') 'n1       =', n1
write(6,'(A,I7)') 'n2       =', n2
!
call timing_Write(nc, nThread=1, ScaleExponent=-9)
!
end subroutine rhdPT_simple_TestStencil1D


!----------********************---------------------------------------------------------------------
subroutine rhdPT_simple_TestAll(nc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_simple_TestAll ('radiation_hydrodynamics_Performance_Test_simple_Test_All')
!
! PURPOSE:
!   Do all simple performance tests for RHD code.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_simple_TestAll(nc)
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   omp_lib:               Library of system OpenMP definitions and routines
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_simple_TestAll(nc)
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc
!
! --- Local variables ---
character(len=84)                       :: line1, line2, line3
character(len=60)                       :: line4
! --- OpenMP variables ---
!$ integer                              :: omp_MaxThreads, omp_SchedModifier
!$ integer(kind=omp_sched_kind)         :: omp_SchedKind
!---------------------------------------------------------------------------------------------------
!
line1=repeat('*', 84)
line2=repeat('=', 84)
line3=repeat('-', 84)
line4=repeat('.', 60)
!
write(nc,'(A)') line3
write(nc,'(A)')       'OpenMP: ...................................................'
!#
# ifdef _OPENMP
!#
! --- Get and print information about OpenMp environment ---
omp_MaxThreads=omp_get_max_threads()
call omp_get_schedule(omp_SchedKind, omp_SchedModifier)
write(nc,'(A,I4)')    '  Maximum number of threads: ', omp_MaxThreads
if (omp_SchedKind == omp_sched_auto) then
  write(nc,'(A,I4)')  '  Scheduling type: (auto)    ', omp_SchedKind
else if (omp_SchedKind == omp_sched_static) then
  write(nc,'(A,2I4)') '  Scheduling type: (static)  ', omp_SchedKind, omp_SchedModifier
else if (omp_SchedKind == omp_sched_dynamic) then
  write(nc,'(A,2I4)') '  Scheduling type: (dynamic) ', omp_SchedKind, omp_SchedModifier
else if (omp_SchedKind == omp_sched_guided) then
  write(nc,'(A,2I4)') '  Scheduling type: (guided)  ', omp_SchedKind, omp_SchedModifier
else
  write(nc,'(A,2I4)') '  Scheduling type:           ', omp_SchedKind, omp_SchedModifier
endif
write(nc,'(A,L8)')    '  Nesting enabled:       ', omp_get_nested()
!#
# else
!#      /* _OPENMP */
write(nc,'(A)')       '  ... is not activated'
!#
# endif
!#      /* _OPENMP */
!
write(nc,*)
!
!call rhdPT_simple_TestMany(nc, 256,  512,  1024,  8)
!
! --- Measure performance of a few arithmetic operations on small arrays of different aspect ratios
call rhdPT_simple_TestFew( nc,  23,    714,  8192,  8)
call rhdPT_simple_TestFew( nc,  24,    684,  8192,  8)
call rhdPT_simple_TestFew( nc,  25,    656,  8192,  8)
call rhdPT_simple_TestFew( nc,  31,    528,  8192,  8)
call rhdPT_simple_TestFew( nc,  32,    512,  8192,  8)
call rhdPT_simple_TestFew( nc,  33,    496,  8192,  8)
call rhdPT_simple_TestFew( nc,  63,    260,  8192,  8)
call rhdPT_simple_TestFew( nc,  64,    256,  8192,  8)
call rhdPT_simple_TestFew( nc,  65,    252,  8192,  8)
call rhdPT_simple_TestFew( nc, 128,    128,  8192,  8)
call rhdPT_simple_TestFew( nc, 256,     64,  8192,  8)
call rhdPT_simple_TestFew( nc, 256,     32,  8192, 16)
call rhdPT_simple_TestFew( nc, 256,     16,  8192, 32)
call rhdPT_simple_TestFew( nc, 512,      8,  8192, 32)
!
! --- Measure performance of a few arithmetic operations on large arrays ---
!call rhdPT_simple_TestFew( nc,4096,   2048,    16,  8)
!call rhdPT_simple_TestFew( nc,4096,   4096,     8,  8)
!call rhdPT_simple_TestFew( nc,4096,   8192,     4,  8)
!
! --- Measure performance of a few arithmetic operations on arrays of different sizes ---
call rhdPT_simple_TestFew( nc, 256,     32, 16384,  8)
call rhdPT_simple_TestFew( nc, 256,     64,  8192,  8)
call rhdPT_simple_TestFew( nc, 256,    128,  4096,  8)
call rhdPT_simple_TestFew( nc, 256,    256,  2048,  8)
call rhdPT_simple_TestFew( nc, 256,    512,  1024,  8)
call rhdPT_simple_TestFew( nc, 256,   1024,   512,  8)
call rhdPT_simple_TestFew( nc, 256,   2048,   256,  8)
!call rhdPT_simple_TestFew( nc, 256,   4096,   128,  8)
!call rhdPT_simple_TestFew( nc, 256,   8192,    64,  8)
!call rhdPT_simple_TestFew( nc, 256,  16384,    32,  8)
!call rhdPT_simple_TestFew( nc, 256,  32768,    16,  8)
!call rhdPT_simple_TestFew( nc, 256,  65536,     8,  8)
!call rhdPT_simple_TestFew( nc, 256, 131072,     4,  8)
!call rhdPT_simple_TestFew( nc, 256, 262144,     2,  8)
!call rhdPT_simple_TestFew( nc, 256, 524288,     1,  8)
!
! --- Measure performance of lots of arithmetic operations on small arrays ---
!call rhdPT_simple_TestMany(nc, 256,     32, 16384,   8)
!call rhdPT_simple_TestMany(nc, 256,    128,  4096,   8)
!call rhdPT_simple_TestMany(nc, 256,    512,  1024,   8)
!call rhdPT_simple_TestMany(nc, 256,   2048,   256,   8)
!call rhdPT_simple_TestMany(nc, 256,   8192,    64,   8)
!
! --- Test inlining ---
!call rhdPT_simple_TestCall(nc, 256,    512,  1024,   80)
!
! --- Test vectorization of short loop over first dimension ---
!call rhdPT_simple_TestShortVecAdd(nc,     512,  8192, 256)
!call rhdPT_simple_TestShortVecAdd(nc,    4096,  8192,  32)
!
! --- Test vectorization of short integration loop ---
!call rhdPT_simple_TestShortVecInt(nc,     512,  8192, 256)
!call rhdPT_simple_TestShortVecInt(nc,    4096,  8192,  32)
!
! --- Measure performance of simple 1D stencil operations ---
call rhdPT_simple_TestStencil1D( nc, 256,    256,  2048,  8*4)
call rhdPT_simple_TestStencil1D( nc, 256,    512,  1024,  8*4)
call rhdPT_simple_TestStencil1D( nc, 256,   1024,   512,  8*4)
!
end subroutine rhdPT_simple_TestAll

end module rhdPT_simple_module


!------*******************--------------------------------------------------------------------------
module rhdPT_master_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_master_module ('radiation_hydrodynamics_Performance_Test_master_module')
!
! PURPOSE:
!   Provide master performance-test routines for RHD code.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhdPT_master_module
!
! TYPES:
!
! VARIABLES:
!
! ROUTINES: (contained)
!   rhdPT_master_TestAll:  Do all performance tests for RHD code
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhdPT_master_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT_master_TestAll ('radiation_hydrodynamics_Performance_Test_master_Test_All')
!
! PURPOSE:
!   Do all performance tests for RHD code.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdPT_master_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhdPT_master_TestAll()
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use timing_module
use rhdPT_simple_module
!
implicit none
!---------------------------------------------------------------------------------------------------
!
call rhdPT_simple_TestAll(6)
!
end subroutine rhdPT_master_TestAll

end module rhdPT_master_module


!-------*****---------------------------------------------------------------------------------------
program rhdPT
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdPT ('radiation_hydrodynamics_Performance_Test')
!
! PURPOSE:
!   Do performance tests for RHD code.
!
! CATEGORY:
!   Testing
!
! VARIABLES:
!
! ROUTINES:
!   rhdPT_master_TestAll:  Do all performance tests for RHD code
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhdPT_master_module:   Master performance-test routine for RHD code
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! PROCEDURE:
!   Initialize global variables and call performance-test master routine.
!   Compile e.g. with
!     ifort -assume byterecl -fpp -openmp -fast -xHost timing_module.F90 rhdPT.F90 -o rhdPT.exe
!     gfortran -fopenmp -Ofast -march=native  timing_module.F90 rhdPT.F90 -o rhdPT.exe
!   Likely, you have to set
!     ulimit -s unlimited
!
! MODIFICATION HISTORY:
!   2015-05-15 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!use rhd_gl_module
use timing_module
use rhdPT_master_module
!
implicit none
!---------------------------------------------------------------------------------------------------
!
! --- Set important global parameters ---
!gl%nc_p        =6                           ! --- Channel nc_p for text output  ---
!gl%nc_m        =6                           ! --- Channel nc_p for text output  ---
!gl%lw_p        =84                          ! --- Line width for text output    ---
!
call timing_SwitchInfo(6)
!
call rhdPT_master_TestAll()
!
end program rhdPT
