pro extremum, x=x, y, order, xout, yout, $
  i0=i0, irange=irange, degree=degree, verbose=verbose, coeff=c0, $
  measure_errors=measure_errors
;
;------------------------------------------------------------------------------
;+
; NAME:
;   extremum
;
; PURPOSE:
;   Find "exact" position of zero, minimum, maximum, ... of function given by
;   vector.
;
; CATEGORY:
;   Calculation
;
; CALLING SEQUENCE:
;   extremum [, x=x] y, order, xout, yout $
;             [, i0=i0, irange=irange, degree=degree, verbose=verbose ]
;
; INPUTS:
;   y       - Vector with ordinate values
;   order   - kind of 'extremum': 0 -> zero (default)
;                                 1 -> minimum, maximum
;                                 2 -> turning point
;
; OUTPUTS:
;   xout    - Location of extremum
;   yout    - y value at extremum
;
; KEYWORDS:
;   x       - vector with abscissa values, default: 0,1,...,n_elements(y)-1
;   i0      - start index value for search of extremum (0 -> estimate start
;             value)
;   irange  - half index range for search interval,
;             number of interval points = 2*irange+1
;   degree  - degree of polynomial which is fitted (degree >= order+1)
;   verbose - if set, print additional information
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   deriv: Compute approximate derivative
;   poly_fit: Fit polynomial
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   No warranty that the minimum or whatever is indeed found
;
; PROCEDURE:
;   First find estimate of position of extremum (i0). Then fit polynomial (of
;   degree degree) in interval (irange) around this point. Compute desired
;   (order) derivative of polynomial. Then iteratively search zero of this
;   derivative.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20.11.95 Written by Bernd Freytag, Astrophysik Kiel
;   13.03.96 (B.F.) More comments
;    7.11.97 (B.F.) poly_fit(double(...))
;-
;------------------------------------------------------------------------------
if (keyword_set(verbose)) then print, 'Find extremum with extremum: ----------'
;
; --- Number of elements of input vector ---
ny=n_elements(y)
;
; --- Default values for keywords ---
if (n_elements(order) eq 0) then order=0
if (n_elements(x) eq 0) then begin
  x=findgen(ny)
endif else if (n_elements(x) ne ny) then begin
  message, 'x and y have different numbers of elements'
endif
if (n_elements(irange) eq 0) then irange=ny/2
if (n_elements(degree) eq 0) then degree=order+1 else degree=degree > (order+1)
;
; --- Find approximate position of extremum ---
if (not keyword_set(i0)) then begin
  if (order eq 0) then begin
    yymin=min(y^2,i0)
  endif else begin
    dy=y
    for i=1,order do dy=deriv(x,dy)
    yymin=min(dy^2,i0)
  endelse
endif
;
; --- Define range ---
i1=(i0-irange) > 0
i2=(i0+irange) < (ny-1)
if (keyword_set(verbose)) then help,i0,i1,i2
;
; --- Fit polynomial ---
;c0=poly_fit(x(i1:i2),y(i1:i2),degree)
if (keyword_set(measure_errors)) then begin
  c0=float(poly_fit(double(x(i1:i2)),double(y(i1:i2)),degree, measure_errors=measure_errors(i1:i2)))
endif else begin
  c0=float(poly_fit(double(x(i1:i2)),double(y(i1:i2)),degree))
endelse
if (keyword_set(verbose)) then print, 'Polynomial:     ', reform(c0)
cd=c0
;
; --- Compute desired derivative of polynomial ---
for i=0,order do begin
  c=cd
  cd=cd*findgen(n_elements(cd))
  cd=cd(1:*)
  if (keyword_set(verbose)) then print, 'Polynomial (deriv):', cd
endfor
;
; --- Start value ---
x0=x(i0)
dx=deriv(x)
xeps=dx(i0)*1e-9
iter=0
xnew=x0
;
; --- Perform iteration to find zero ---
repeat begin
  iter=iter+1
  xold=xnew
  xnew=xold-poly(xold,c)/poly(xold,cd)
  if (keyword_set(verbose)) then print, iter, xold, xnew
endrep until ((iter ge 20) or (abs(xnew-xold) lt xeps))
;
; --- Output values ---
xout=xnew
yout=poly(xout,c0)
if (keyword_set(verbose)) then help, xout, yout
;
if (keyword_set(verbose)) then print, 'End of extremum: ----------------------'
;
end ; extremum
