function komega, q, $
  t_axis=t_axis, x_axis=x_axis, $
  meansub=meansub, $
  t_trenddegree=t_trenddegree, t_smooth=t_smooth, t_cosinebell=t_cosinebell, $
  x_smooth=x_smooth, $
  f_smooth=f_smooth, $
  q0=q0, $
  f_axis=f_axis, o_axis=o_axis, $
  l_axis=l_axis, k_axis=k_axis
;------------------------------------------------------------------------------
;+
; NAME:
;   komega ('k_omega')
;
; PURPOSE:
;   Compute 2D powerspectrum and produce data for k-omega diagram.
;
; CATEGORY:
;   FFT
;
; CALLING SEQUENCE:
;
; INPUT:
;   q:            ('quantity') 2D (or - later - 3D) array with amplitude(space,time)
;
; OUTPUT:
;   result:       Resulting powerspectrum (only one quadrant, up to Nyquist frequencies)
;
; KEYWORDS:
;   t_axis:       ('time_axis') vector with as many elements as last dimension of q
;   meansub:      ('mean_subtract') If set the space-time average is subtracted
;   t_trenddegree:('time_trend_degree') Degree of polynomial to subtract,
;                 default: -1 (no subtraction)
;   t_smooth:     ('time_smooth') Number of times a time-smoothing filter is applied,
;                 default: 0 (no filtering)
;   t_cosinebell: ('time_cosine_bell') Relative intervall length at beginning and
;                 end of dataset that is multiplied with cosinebell function.
;                 This is useful for all signals that are not periodic in time.
;   x_smooth:     ('x_smooth')  Number of times a space-smoothing filter is applied,
;                 default: 0 (no filtering)
;   f_smooth:     ('frequency_smooth')  Number of times a frequency-smoothing filter
;                 is applied, default: 0 (no filtering).
;   q0:           ('quantity_0') Output array: the input array after filtering, but
;                 before Fourier analysis
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;  cosinebell: Compute the cosine-bell function
;  smooth2d:   Smooth 2D array
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   n1=50 & nt=100
;   x=findgen(n1)/float(n1) & t=findgen(nt)/float(nt)
;   x2d=rebin(reform(x,n1,1 ),n1,nt)
;   t2d=rebin(reform(t,1 ,nt),n1,nt)
;
;   q=sin(2.0*!pi*t2d)*cos(2.0*!pi*x2d);
;   qp=komega(q)
;   print,avg(q^2),max(qp)
;
;   q=sin(2.0*!pi*t2d);
;   qp=komega(q)
;   print,avg(q^2),max(qp)
;
;   qp=komega(q, t_trenddegree=-1, t_cosinebell=0.2, f_smooth=1, q0=q0)
;
; MODIFICATION HISTORY:
;   2005-08-17 Written by Bernd Freytag, LANL
;   2005-08-18 (B.F.) Keywords t_axis, t_trenddegree added
;-
;------------------------------------------------------------------------------

compile_opt strictarr

q0=q

t_kernel=reform([0.25,0.5,0.25],1,3)
x_kernel=reform([0.25,0.5,0.25],3,1)

; --- Check dimension of input array ---
n1=0L
n2=0L
qndim=size(q0, /n_dimensions)
qdim =size(q0, /dimension)
if (qndim eq 2) then begin
  n1=qdim[0]
  nt=qdim[1]
endif else if (qndim eq 3) then begin
  message, 'Only 2D arrays supported, so far.'
  n1=qdim[0]
  n2=qdim[1]
  nt=qdim[2]
endif else begin
  message, 'Only 2D arrays supported, so far.'
endelse

; --- Check axes ---
if (keyword_set(t_axis)) then begin
  t_axis0=t_axis
endif else begin
  t_axis0=findgen(nt)
endelse
if (keyword_set(x_axis)) then begin
  x_axis0=x_axis
endif else begin
  x_axis0=findgen(n1)
endelse

; --- Subtract average ---
if (keyword_set(meansub)) then begin
  message, 'Subtract total mean', /informational
  q0=q0-avg(q0)
endif

; --- Remove temporal trend by subtraction polynomial ---
if (n_elements(t_trenddegree) ne 0) then begin
  if (t_trenddegree ge 0) then begin
    message, 'Remove trend in time: subtract polynomial:', /informational
    ;
    ; --- Compute time-dependent space-average ---
    if (qndim eq 2) then begin
      qtavg=avg(q0, 0)
    endif else if (qndim eq 3) then begin  
      qtavg=avg(avg(q0, 0), 0)
    endif else begin
      qtavg=q0
    endelse
    ;
    ; --- Fit polynomial to it ---
    coeffpoly=poly_fit(t_axis0, qtavg, t_trenddegree, yfit=qtfit)
    strcoeffpoly=''
    for i=0, t_trenddegree do strcoeffpoly=strcoeffpoly + string(coeffpoly[i])
    message, 'Polynomial coefficients :' + strcoeffpoly, /informational
    ;
    ; --- Subtract fit ---
    if (qndim eq 2) then begin
      qtfit=rebin(reform(qtfit, 1, n_elements(qtfit)), n1, nt)
    endif else if (qndim eq 3) then begin  
      qtfit=rebin(reform(qtfit, 1, 1,  n_elements(qtfit)), n1, n2, nt)
    endif
    q0=q0-qtfit
  endif
endif

; --- Smooth small temporal scales ---
if (keyword_set(t_smooth)) then begin
  message, 'Smooth small temporal scales', /informational
  q0=smooth2d(q0, q_smooth=t_smooth, kernel=t_kernel)
endif

; --- Smooth small spatial scales ---
if (keyword_set(x_smooth)) then begin
  message, 'Smooth small spatial scales', /informational
  q0=smooth2d(q0, q_smooth=x_smooth, kernel=x_kernel, /periodic)
endif

; --- Smooth at start and end ---
if (keyword_set(t_cosinebell)) then begin
  message, 'Smooth in time with cosinebell', /informational
  normtime=findgen(nt)/float(nt)
  cosinebellmask=cosinebell(normtime, 0.0 > t_cosinebell < 0.5)
  if (qndim eq 2) then begin
    cosinebellmask=rebin(reform(cosinebellmask, 1, nt), n1, nt)
  endif
  q0=q0*temporary(cosinebellmask)
endif

; --- Fourier transform in all dimensions, powerspectrum ---
qpower=abs(fft(q0))^2

; --- Smooth temporal frequencies ---
if (keyword_set(f_smooth)) then begin
  message, 'Smooth temporal frequencies', /informational
  qpower=smooth2d(qpower, q_smooth=f_smooth, kernel=t_kernel, /periodic)
endif

; --- Fold everything into one quadrant ---
if (qndim eq 2) then begin
  n1out=(n1+2L)/2L
  ntout=(nt+2L)/2L
  n1rest=n1-n1out
  ntrest=nt-ntout
  ;
  qout=fltarr(n1out, ntout)
  ;
  qout                    =qout                     +         qpower[0    :n1out-1, 0     :ntout-1]
  qout[0:*     , 1:ntrest]=qout[0:*     , 1:ntrest] + reverse(qpower[0    :n1out-1, ntout:*       ], 2)
  qout[1:n1rest, 0:*     ]=qout[1:n1rest, 0:*     ] + reverse(qpower[n1out:*      , 0     :ntout-1], 1)
  qout[1:n1rest, 1:ntrest]=qout[1:n1rest, 1:ntrest] + rotate( qpower[n1out:*      , ntout:*       ], 2)
endif

if (arg_present(f_axis)) then begin
  f_axis=findgen(ntout) * (1.0    /(t_axis[nt-1]-t_axis[0]))
endif
if (arg_present(o_axis)) then begin
  o_axis=findgen(ntout) * (2.0*!PI/(t_axis[nt-1]-t_axis[0]))
endif

if (arg_present(l_axis)) then begin
  l_axis=findgen(n1out) * (1.0    /(x_axis[n1-1]-x_axis[0]))
endif
if (arg_present(k_axis)) then begin
  k_axis=findgen(n1out) * (2.0*!PI/(x_axis[n1-1]-x_axis[0]))
endif


return, qout

end ; komega
