;+
; Name:
;      	LEGEND 
;
; Purpose:
;	This procedure makes a legend for a plot.  The legend can contain
;	a mixture of symbols, linestyles, Hershey characters (vectorfont),
;	and filled polygons (usersym).
;       New: Default is CLIP. to avoid clipping set !P.NOCLIP=1.
;
; Examples:
;	The call:
;		legend, ['diamond','asterisk','square'], psym=[4,2,6]
;	produces:
;		-----------------
;		|		|
;		|  <> diamond	|
;		|  *  asterisk	|
;		|  [] square	|
;		|		|
;		-----------------
;	Each symbol is drawn with a plots command, so they look OK.
;	Other examples are given in usage.
;
; Usage:
;	legend,items,linestyle=linestyle	; vertical legend at upper left
;	legend,items,psym=psym			; ditto except using symbols
;       legend,items,thick=thick                ; ditto except using thickness
;	legend,items,psym=psym,/horizontal	; horizontal format
;	legend,items,psym=psym,box=0		; sans border
;	legend,items,psym=psym,delimiter='='	; embed an '=' betw psym & text
;	legend,items,psym=psym,margin=2		; 2-character margin
;	legend,items,psym=psym,position=pos	; position of legend
;	legend,items,psym=psym,number=2		; plot two symbols, not one
;	legend,items,/fill,psym=[8,8,8],colors=[10,20,30]; 3 filled squares
;
; Inputs:
;	items = text for the items in the legend, a string array.
;		You can omit items if you don't want any text labels.
;		For example, items = ['diamond','asterisk','square'].
;
; Optional Keywords:
;	linestyle = array of linestyle numbers  If linestyle(i) < 0, then omit 
;		ith symbol or line to allow a multi-line entry.
;	psym = array of plot symbol numbers.  If psym(i) is negative, then a
;		line connects pts for ith item.  If psym(i) = 8, then the
;		procedure usersym is called with vertices defined in the 
;		keyword usersym.
;               By use of the function PSYMUSER additional symbols are
;               available.
;	N. B.: Choose either linestyle, psym, thick, or a combination.If none is
;		present, only the text is output.  linestyle, psym and/or
;		thick parameters don't have to have the
;		same number of elements any more.
;		By default, if psym is positive, you get one point so there is
;		no connecting line.
;	colors = array of colors for plot symbols/lines (D=!P.COLOR)
;	textcolors = array of colors for text (D=!P.COLOR)
;	thick = array of thickness numbers
;       symsize = array of symbol sizes (D=charsize)
;	vectorfont = vector-drawn characters for the sym/line column, e.g.,
;		['!9B!3','!9C!3','!9D!3'] produces an open square, a checkmark,
;		and a partial derivative, which might have accompanying items
;		['BOX','CHECK','PARTIAL DERIVATIVE'].  If vectorfont(i) = '',
;		then plots is called to make a symbol or a line, but if
;		vectorfont(i) is a non-null string, then xyouts is called.
;		There is no check that !p.font is set properly, e.g., -1 for
;		X and 0 for PostScript.  This can produce an error, e.g., use
;		!20 with PostScript and !p.font=0, but allows use of Hershey
;		*AND* PostScript fonts together.
;	/help = flag to print header
;	/horizontal = flag to make the legend horizontal
;	/vertical = flag to make the legend vertical (D=vertical)
;	box = flag to include/omit box around the legend (D=include)
;	delimiter = embedded character(s) between symbol and text (D=none)
;	margin = margin around text measured in characters and lines
;	spacing = line spacing (D=bit more than character height)
;	pspacing = psym spacing (D=3 characters)
;	charsize = just like !p.charsize for plot labels
;	number = number of plot symbols to plot or length of line (D=1)
;	usersym = 2-D array of vertices, cf. usersym in IDL manual. (D=square)
;	fill = flag to fill the usersym
;	position = coordinates of the upper left of the legend (default:
;                  normalized)
;       /data = flag to indicate that the position is given in data coordinates
;       /device = flag to indicate that the position is given in device
;                 coordinates
;       yalignment = alignment parameter in vertical direction, active if the
;                    position keyword is not specified.
;                    (0: bottom, 0.5: center, 1: top (default))
;       xalignment = alignment parameter in horizontal direction, active if the
;                    position keyword is not specified. 
;                    (0: left(default), 0.5: center, 1: right)
;                    The legend box is plotted two times: The first time it is
;                    plotted outside the clipping box and the horizontal
;                    width of the legend box is determined. With this quantity
;                    the position of the box can be determined for the second
;                    plot inside the plot frame.
;       itemwidth = estimated textwidth in characters (default=8), necessary if
;                   position keyword is not present and xalignment <> 0
;                   (not necessary any more)
;
; Outputs:
;	None
;
; Optional Output Keywords:
;	corners = 4-element array, like !p.position, of the normalized
;	  coords for the box (even if box=0): [llx,lly,urx,ury].
;	  Useful for multi-column or multi-line legends, for example,
;	  to make a 2-column legend, you might do the following:
;	    c1_items = ['diamond','asterisk','square']
;	    c1_psym = [4,2,6]
;	    c2_items = ['solid','dashed','dotted']
;	    c2_line = [0,2,1]
;	    legend,c1_items,psym=c1_psym,corners=c1,box=0
;	    legend,c2_items,line=c2_line,corners=c2,box=0,pos=[c1(2),c1(3)]
;	    c = [c1(0)<c2(0),c1(1)<c2(1),c1(2)>c2(2),c1(3)>c2(3)]
;	    plots,[c(0),c(0),c(2),c(2),c(0)],[c(1),c(3),c(3),c(1),c(1)],/norm
;	  Useful also to place the legend.  Here's an automatic way to place
;	  the legend in the lower right corner.  The difficulty is that the
;	  legend's width is unknown until it's is plotted.  In this example,
;	  the legend is plotted twice: the first time in the upper left, the
;	  second time in the lower right.
;	    legend,['1','22','333','4444'],linestyle=indgen(4),corners=corners
;			; BOGUS LEGEND---FIRST TIME TO REPORT CORNERS
;	    xydims = [corners(2)-corners(0),corners(3)-corners(1)]
;			; SAVE WIDTH AND HEIGHT
;	    chdim=[!d.x_ch_size/float(!d.x_size),!d.y_ch_size/float(!d.y_size)]
;			; DIMENSIONS OF ONE CHARACTER IN NORMALIZED COORDS
;	    pos = [!x.window(1)-chdim(0)-xydims(0) $
;			,!y.window(0)+chdim(1)+xydims(1)]
;			; CALCULATE POSITION FOR LOWER RIGHT
;	    plot,findgen(10)	; SIMPLE PLOT; YOU DO WHATEVER YOU WANT HERE.
;	    legend,['1','22','333','4444'],linestyle=indgen(4),pos=pos
;			; REDO THE LEGEND IN LOWER RIGHT CORNER
;	  You can modify the pos calculation to place the legend where you
;	  want.  For example to place it in the upper right:
;	    pos = [!x.window(1)-chdim(0)-xydims(0),!y.window(1)-xydims(1)]
;
; Common blocks:
;	none
;
; Procedure:
;	If keyword help is set, call doc_library to print header.
;	See notes in the code.
;
; Routines:
;       The function PSYMUSER is used. It provides a number of additional
;       plot symbols. If it is not available use the identity
;         function psymuser, psym
;         return, psym
;         end
;       instead or remove it: 
;         plots, ..., psym=psymuser(psym(i)), ...
;           =>
;         plots, ..., psym=psym(i), ...
;       The function REPLICATE1 is used. It replicates a vector until the output
;       has a specified number of elements. If it is not available use the
;       identity but then you produce error messages if you use vectors
;       with different numbers of elements in ITEMS, LINESTYLE, PSYM, ...
;
; Restrictions:
;	Here are some things that aren't implemented.
;	- An orientation keyword would allow lines at angles in the legend.
;	- An order option to interchange symbols and text might be nice.
;	- Somebody might like double boxes, e.g., with box = 2.
;	- Another feature might be a continuous bar with ticks and text.
;	- There is no provision for multi-line text, e.g., '1st line!c2nd line'
;	  Sensing !c would be easy, but !c isn't implemented for PostScript.
;	  A better way might be to simply output the 2nd line as another item
;	  but without any accompanying symbol or linestyle.  A flag to omit
;	  the symbol and linestyle is linestyle(i) = -1.
;
; Side Effects:
;         If the keyword USERSYM or a non-standard plot symbol is used,
;         a new user symbol is defined.
;         A legend is plotted on the current graphic device.
;   
; Modification history:
;	write, 24-25 Aug 92, F K Knight (knight@ll.mit.edu)
;	allow omission of items or omission of both psym and linestyle, add 
;	  corners keyword to facilitate multi-column legends, improve place-
;	  ment of symbols and text, add guards for unequal size, 26 Aug 92, FKK
;	add linestyle(i)=-1 to suppress a single symbol/line, 27 Aug 92, FKK
;	add keyword vectorfont to allow characters in the sym/line column,
;	  28 Aug 92, FKK
;	Removed default Usersym definition  W. Landsman  HSTX   July, 1993
;       Added keyword THICK: Bernd Freytag, Astrophysik Kiel, Sep. 94
;       Added routine call to PSYMUSER: B. F., Oct. 94
;       Added keywords DATA, DEVICE, and X/YALIGNMENT: B. F., Oct. 94
;       Default is now: Do CLIPPING. B.F. Oct. 94
;       Use REPLICATE1 to equalize the number of elements in ITEMS, LINESTYLE,
;         PSYM, ... B.F. Oct. 94
;       FILL may be vector. B.F. Jul. 96
;  22. 3.02 (B.F. Uppsala) Keyword symsize added
;  21. 6.05 (B.F. LA) Keyword nfromitems added
;  19.11.10 (B.F. Lyon) !COLOR -> !P.COLOR
;-
pro legend ,items0, help=help $
  ,linestyle=linestyle0, psym=psym0, thick=thick0, vectorfont=vectorfont0 $
  ,colors=colors0, textcolors=textcolors0, symsize=symsize0 $
  ,horizontal=horizontal, vertical=vertical, box=box, margin=margin $
  ,delimiter=delimiter, spacing=spacing, charsize=charsize, pspacing=pspacing $
  ,position=position0, data=data, device=device $
  ,number=number $
  ,fill=fill0, usersym=usersym, corners=corners $
  ,yalignment=yalignment, xalignment=xalignment, itemwidth=itemwidth, nfromitems=nfromitems
;
; --- Help ---
if keyword_set(help) then begin & doc_library,'legend' & return & endif
;
; --- Set defaults for symbols, linestyles, and items.
ni  = n_elements(items0)
np  = n_elements(psym0)
nf  = n_elements(fill0)
nl  = n_elements(linestyle0)
nv  = n_elements(vectorfont0)
nc  = n_elements(colors0)
ntc = n_elements(textcolors0)
nss = n_elements(symsize0)
nt  = n_elements(thick0)
if (keyword_set(nfromitems)) then begin
  n = ni
endif else begin
  n = max([ni,np,nl,nv,nc,ntc,nss,nt])			; NUMBER OF ENTRIES
endelse
;
; --- No inputs => error message ---
if n eq 0 then message,'No inputs!  For help, type legend,/help.'
;
; --- Replicate vectors to get the same size for all ---
if (ni eq 0) then begin
  items = replicate('',n)				; DEFAULT BLANK ARRAY
endif else begin
  szt = size(items0)
  if (szt(szt(0)+1) ne 7) then $
    message,'First parameter must be a string array.  For help, type legend,/help.'
  items=replicate1(items0,n)
endelse
if (nf eq 0) then fill=replicate(0,n) $
             else fill=replicate1(fill0,n)
if (nl eq 0) then linestyle=intarr(n) $
             else linestyle=replicate1(linestyle0,n)
if (np eq 0) then psym=intarr(n) $
             else psym=replicate1(psym0,n)
if (nc eq 0) then colors=replicate(!P.COLOR,n) $
             else colors=replicate1(colors0,n)
if (ntc eq 0) then textcolors=replicate(!P.COLOR,n) $
              else textcolors=replicate1(textcolors0,n)
if (nv eq 0) then vectorfont=replicate('',n) $
             else vectorfont=replicate1(vectorfont0,n)
if (nt eq 0) then thick = intarr(n) $
             else thick = replicate1(thick0,n)
symline = (np ne 0) or (nl ne 0) or (nt ne 0)		; FLAG TO PLOT SYM/LINE
;
; --- Choose vertical or horizontal orientation.
if n_elements(horizontal) eq 0 then begin 		; D=VERTICAL
  if n_elements(vertical) eq 0 then vertical = 1
endif else begin
  if n_elements(vertical) eq 0 then vertical = not horizontal
endelse
;
; --- Set defaults for other options.
if n_elements(number) eq 0 then number = 1
if n_elements(box) eq 0 then box = 1
if n_elements(margin) eq 0 then margin = 0.5
if n_elements(delimiter) eq 0 then delimiter = ''
if n_elements(charsize) eq 0 then charsize = !p.charsize
if charsize eq 0 then charsize = 1
if n_elements(spacing) eq 0 then spacing = 1.2
if n_elements(pspacing) eq 0 then pspacing = 3
xspacing = !d.x_ch_size/float(!d.x_size) * (spacing > charsize)
yspacing = !d.y_ch_size/float(!d.y_size) * (spacing > charsize)
;
if (nss eq 0) then symsize=replicate(charsize,n) $
              else symsize=replicate1(symsize0,n)
;
if !x.window(0) eq !x.window(1) then begin
  plot,/nodata,xstyle=4,ystyle=4,[0],/noerase
endif
;
; --- Calculate offset xt for start of ITEMS ---
xt = 0
for i = 0,n-1 do begin
  if psym(i) eq 0 then num = (number + 1) > 3 else num = number
  if psym(i) lt 0 then num = number > 2                ; TO SHOW CONNECTING LINE
  if psym(i) eq 0 then expand = 1 else expand = 2
  if vertical then begin
    xt = (expand*pspacing*(num-1)*xspacing) > xt
  endif else begin
    xt = xt + (expand*pspacing*(num-1)*xspacing)
  endelse
endfor
;
; --- Next line takes care of weirdness with small windows. ---
pos = [min(!x.window),min(!y.window),max(!x.window),max(!y.window)]
;
; --- Plot the legend list two times: First a testplot outside the
; --- clipping frame at which the exact horizontal width is determined for the
; --- final plot. ---
for l=0,1 do begin
  ; --- Set some variables depending on test/final plot ---
  if (l eq 0) then begin
    ; --- Make the first (test) plot ---
    ; --- Shift the legend outside the plot frame ---
    deltaposx=(pos(2)-pos(0))*10.0
    deltaposy=-(pos(3)-pos(1))*10.0
    ; --- Save old system NOCLIP keyword ---
    clipdump=!P.NOCLIP
    ; --- Do clip ---
    !P.NOCLIP=0
    noclip=0
  endif else begin
    ; --- Make the second (final) plot ---
    ; --- No shifting ---
    deltaposx=0.0
    deltaposy=0.0
    ; --- Reset system NOCLIP keyword ---
    !P.NOCLIP=clipdump
    noclip=!P.NOCLIP
  endelse
  ;
  ; --- Determine position of upper left corner of box ---
  if n_elements(position0) eq 0 then begin
    ; --- Default position ---
    position = [pos(0),pos(3)] + [xspacing,-yspacing]
    ; --- Alignment in vertical direction ---
    if (n_elements(yalignment) ne 0) then begin
      ; --- Compute the vertical extent of the box ---
      if vertical then bottom = n else bottom = 1
      ywidth = - (2*margin+bottom-0.5)*yspacing
      ; --- Set aligned vertical position ---
      position(1) = (pos(3)-yspacing       ) *      yalignment + $
                    (pos(1)+yspacing-ywidth) * (1.0-yalignment)
    endif else begin
      position(1) = pos(3)-yspacing
    endelse
    ; --- Alignment in horizontal direction ---
    if (n_elements(xalignment) ne 0) then begin
      if (l eq 0) then begin
        ; --- Try an estimate of the horizontal extent of the box ---
        if vertical then bottom = n else bottom = 1
        right=(float(n)/float(bottom))
        xwidth=margin*xspacing
        xwidth=xwidth + xspacing                            ; First spacing
        xwidth=xwidth + xt                                  ; Offset for symbols
        if symline then xwidth = xwidth + right*xspacing    ; Extra spacing
        if (delimiter ne '') then xwidth = xwidth + right*xspacing ; Delimiter
        if (n_elements(itemwidth) eq 0) then itemwidth=8
        ; --- Estimated space for text ---
        xwidth = xwidth + itemwidth*xspacing*right
        if not vertical then xwidth = xwidth + (n-1)*2*xspacing ; Inter-item sp.
        xwidth = xwidth + xspacing*margin                       ; Last spacing
      endif else begin
        xwidth=maxx-oldposition(0)
      endelse
      ; --- Set aligned horizontal position ---
      position(0) = (pos(0)+xspacing       ) * (1.0-xalignment) + $
                    (pos(2)-xspacing-xwidth) *      xalignment
    endif else begin
      position(0) = pos(0)+xspacing
    endelse
  endif else if (keyword_set(data)) then begin
    position=convert_coord(position0, /data, /to_norm)
  endif else if (keyword_set(device)) then begin
    position=convert_coord(position0, /device, /to_norm)
  endif else begin
    position=position0
  endelse
  oldposition=position
  ;  
  ; --- Initialize positions
  yoff = 0.25*yspacing				; VERT. OFFSET FOR SYM/LINE.
  maxx = 0					; SAVED WIDTH FOR DRAWING BOX
  x0 = position(0) + (margin)*xspacing		; INITIAL X & Y POSITIONS
  y0 = position(1) - (margin-0.5)*yspacing
  y = y0                                        ; STARTING X & Y POSITIONS
  x = x0
  ;
  ;	=====>> OUTPUT TEXT FOR LEGEND, ITEM BY ITEM.
  ;	=====>> FOR EACH ITEM, PLACE SYM/LINE, THEN DELIMITER,
  ;	=====>> THEN TEXT---UPDATING X & Y POSITIONS EACH TIME.
  ;	=====>> THERE ARE A NUMBER OF EXCEPTIONS DONE WITH IF STATEMENTS.
  ;
  for i = 0,n-1 do begin
    ; --- Reset either x or y ---
    if vertical then x = x0 else y = y0
    ; --- Calculate position of next plot symbol ---
    x = x + xspacing				; UPDATE X & Y POSITIONS
    y = y - yspacing
    if (psym(i) eq 0) and (vectorfont(i) eq '') then num = (number + 1) > 3 $
                                                else num = number
    if psym(i) lt 0 then num = number > 2              ; TO SHOW CONNECTING LINE
    if psym(i) eq 0 then expand = 1 else expand = 2
    xp = x + expand*pspacing*indgen(num)*xspacing
    if (psym(i) gt 0) and (num eq 1) and vertical then xp = x + xt/2.
    yp = y + intarr(num)
    if vectorfont(i) eq '' then yp = yp + yoff
    if psym(i) eq 0 then begin
      xp = [min(xp),max(xp)]			; TO EXPOSE LINESTYLES
      yp = [min(yp),max(yp)]			; DITTO
    endif
    ; --- Define user symbol ---
    if (psym(i) eq 8) and keyword_set(usersym) then $
                  usersym,usersym,fill=fill(i),color=colors(i)
    if vectorfont(i) ne '' then begin
      ; --- Plot vectorfont symbol ---
      if (num eq 1) and vertical then xp = x + xt/2
      xyouts, xp+deltaposx, yp+deltaposy, vectorfont(i), /norm, $
         width=width,color=colors(i) $
        ,size=charsize, align=0.5, noclip=noclip
    endif else begin
      ; --- Plot line ---
      if symline and ((linestyle(i) ge 0) or (thick(i) gt 0)) then $
        plots, xp+deltaposx, yp+deltaposy, /norm, $
          color=colors(i), linestyle=linestyle(i), $
          psym=psymuser(psym(i),fill=fill(i)), thick=thick(i), $
          symsize=symsize(i), noclip=noclip
    endelse
    if vertical then x = x + xt else x = max(xp)
    if symline then x = x + xspacing
    ; --- Plot delimiter ---
;help,textcolors
;help,/str,textcolors
    xyouts, x+deltaposx, y+deltaposy, delimiter, /norm, $
      width=width, color=textcolors[i], $
      size=charsize, noclip=noclip
    x = x + width
    if width gt 0 then x = x + 0.5*xspacing
    ; --- Plot item string ---
    xyouts, x+deltaposx, y+deltaposy, items[i], /norm, $
      width=width, color=textcolors[i], $
      size=charsize, noclip=noclip
    x = x + width                                          ; Add space for item
    if not vertical and (i lt (n-1)) then x = x+2*xspacing ; ADD INTER-ITEM SPACE
    maxx = (x + xspacing*margin) > maxx		         ; UPDATE MAXIMUM X
  endfor ; i
endfor ; l
;
;	=====>> OUTPUT BORDER
;
x = position(0)
y = position(1)
if vertical then bottom = n else bottom = 1
ywidth = - (2*margin+bottom-0.5)*yspacing
corners = [x,y+ywidth,maxx,y]
if box then plots, [x,maxx,maxx,x,x], y+[0,0,ywidth,ywidth,0], /norm, $
  noclip=noclip
;
return
end ; legend
