function monobicubic, yin, x1in, x2in, x1out, x2out
;
;------------------------------------------------------------------------------
;+
; NAME:
;   monobicubic
;
; PURPOSE:
;   Perform two dimensional piecewise monotonic cubic interpolation
;
; CATEGORY:
;   Interpolation
;
; CALLING SEQUENCE:
;   yout = monobicubic(yin, x1in, x2in, x1out, x2out)
;
; INPUTS:
;   yin - Y values of input function: Y(X1,X2)
;   x1in - X1 values of input function (strictly ascending or descending)
;   x2in - X2 values of input function (strictly ascending or descending)
;   x1out - X1 values of output function (any order, any array size)
;   x2out - X2 values of output function (any order, any array size)
;
; OUTPUTS:
;   "monobicubic = yin(xout)"
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Abscissa values have to be strictly monotonic and in- or decreasing.
;   At least 4 values are necessary for each dimension.
;   Function is very slow.
;
; PROCEDURE:
;   Uses simple monotonic cubic polynomials in both directions
;   (see Matthias Steffen (1990), A&A 239, 443).
;
; EXAMPLE:
;  nx1in=4
;  nx2in=5
;  x1in=findgen(nx1in)
;  x2in=findgen(nx2in)
;  yin=rebin(reform(x1in^2,nx1in,1),nx1in,nx2in) + $
;      rebin(reform(x2in^2,1,nx2in),nx1in,nx2in)
;  nxout=5
;  x1out=findgen(nxout)*0.5
;  x2out=findgen(nxout)*0.5
;  yout=monobicubic(yin, x1in,x2in, x1out,x2out)
;  plot, x1out, yout
;  oplot,x1out, x1out^2+x2out^2, thick=2,linestyle=2
;
; MODIFICATION HISTORY:
;    8. 5.96 Written by Bernd Freytag, Astrophysik Kiel
;    9.12.96 (B.F.) More comments, allow less than 4 entries per dimension
;   12.12.96 (B.F.) Corrections: yint(.) -> yint(.-kmin)
;                   nx=1 allowed
;-
;------------------------------------------------------------------------------
;
; --- Number of elements of input function ---
nx1=n_elements(x1in)
nx2=n_elements(x2in)
ny=n_elements(yin)
if (nx1*nx2 eq ny) then begin
  ; --- Input seems o.k. ---
endif else begin
  message, 'x1in, x2in, and yin must have a proper number of elements.'
endelse
;
; --- Determine interval indices and relative x value ---
if (nx1 eq 1) then begin
  i1out=[1]
endif else begin
  i1out=1 > (long(interpol(findgen(nx1),x1in,x1out)) < (nx1-3))
endelse
if (nx2 eq 1) then begin
  i2out=[1]
endif else begin
  i2out=1 > (long(interpol(findgen(nx2),x2in,x2out)) < (nx2-3))
endelse
;
; --- Compute result ---
yout=x1out*0.0
xint=fltarr(4)
yint=fltarr(4)
for j=0,n_elements(x1out)-1 do begin
  i1=i1out(j)
  i2=i2out(j)
;  print,i1,nx1,i2,nx2
  kmin=0>(i2-1)
  kmax=(i2+2)<(nx2-1)
;  help,kmin,kmax
  for k=kmin,kmax do begin
    xint(k-kmin)=x2in(k)   
    yint(k-kmin)=monocubic(reform( yin(0>(i1-1):(i1+2)<(nx1-1),k )), $
                              x1in(0>(i1-1):(i1+2)<(nx1-1)),      x1out(j))
  endfor ; k
;  print,'yint=',yint
;  print,'xint=',xint
  yout(j)=monocubic(yint(0:kmax-kmin), xint(0:kmax-kmin), x2out(j))
endfor
;
return, yout
;
end ; monobicubic
