function monocubic, yinput, xinput, xout, $
  integral=integral, derivative=derivative
;
;------------------------------------------------------------------------------
;+
; NAME:
;   MONOCUBIC ('monotonic_cubic')
;
; PURPOSE:
;   Perform monotonic piecewise cubic interpolation
;
; CATEGORY:
;   Interpolation
;
; CALLING SEQUENCE:
;   yout = monocubic(yin, xin [, xout, integral=integral, derivative=derivative])
;
; INPUTS:
;   yin:        ('y_input') real vector, Y values of input function
;   xin:        ('x_input') real vector, X values of input function
;               (strictly ascending or descending)
;   xout:       ('x_output') read vector, X values of output function (any order),
;               if this parameter is omitted, the input function is integrated
;
; OUTPUTS:
;   "monocubic = yin(xout)"
;
; KEYWORDS:
;   integral:   ('integral') input flag: if set, the integral at the requested
;               grip points is computed.
;   derivative: ('derivative') input flag: if set, the derivative at the requested
;               grip points is computed.
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   signum: function to compute sign
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Abscissa values have to be strictly monotonic and ascending or descending.
;   The extrapolation algorithm at both boundaries is different.
;
; PROCEDURE:
;   Uses simple monotonic cubic polynomials
;   (see Matthias Steffen (1990), A&A 239, 443).
;   yin(xin):   1 element  -> constant extrapolation
;   yin(xin): 2,3 elements -> linear interpolation
;   yin(xin): >=4 elements -> cubic interpolation
;
; EXAMPLE:
;   Interpolate a rather nasty function:
;     xin=[0.1, 1.0, 1.9, 2.0, 2.5, 3.5]
;     yin=[1.0, 1.0, 1.0, 3.0, 3.2, 3.3]
;     plot, xin, yin, xstyle=3, ystyle=3
;     xout=findgen(33)/10.0
;     yout=monocubic(yin, xin, xout)
;     oplot, xout, yout, psym=-1
;
; MODIFICATION HISTORY:
;    3. 3.95 Written by Bernd Freytag, Astrophysik Kiel
;   16. 3.95 (B.F.) Integration
;   24. 3.95 (B.F.) Keyword DERIVATIVE added
;   12. 4.95 (B.F.) Boundary handling changed
;    8. 5.96 (B.F.) Data vectors are reversed if necessary
;    9.12.96 (B.F.) Allow input functions with less than 4 elements.
;   17. 2.98 (B.F.) Keyword yintegral added
;   18. 2.98 (B.F.) Keyword integral added, keyword yintegral removed
;   14.12.01 (B.F.) Uppsala, case n0=1,2,3 for integral computation corrected
;-
;------------------------------------------------------------------------------
;
; --- Number of elements of input function ---
nx=n_elements(xinput)
ny=n_elements(yinput)
if (nx eq ny) then n0=nx $
              else message, 'xin and yin must have the same number of elements.'
if (xinput(0) le xinput(nx-1)) then begin
  xin=xinput
  yin=yinput
endif else begin
  xin=reverse(xinput)
  yin=reverse(yinput)
endelse
;
;print, 'monocubic:'
;print, xout
;print, xin,yin
;
n=n0
if (n0 eq 1) then begin
  ; --- Only one element: constant function ---
  if (keyword_set(derivative)) then begin
    ; --- Derivative ---
    yout=xin*0.0
  endif else if (keyword_set(integral)) then begin
    ; --- Integration ---
    yout=xin*0.0
  endif else begin
    if (n_elements(xout) ne 0) then begin
      ; --- Interpolation ---
      yout=xin*0.0+yin(0)
    endif else begin
      ; --- Integration ---
      yout=xin*0.0
    endelse
  endelse  
endif else begin
  if (n0 eq 2) then begin
    ; --- Only two elements: linear extrapolation ---
    xin=[2.0*xin(0)-xin(1),xin,2.0*xin(1)-xin(0)]
    yin=[2.0*yin(0)-yin(1),yin,2.0*yin(1)-yin(0)]
    n=4
  endif else if (n0 eq 3) then begin
    ; --- Only three elements: linear extrapolation ---
    xin=[xin,2.0*xin(2)-xin(1)]
    yin=[yin,2.0*yin(2)-yin(1)]
    n=4
  endif
  ;
  ; --- Now: at least four elements: cubic interpolation ---
  ;
  ; --- Compute x-step h, linear slope s and slope of parabola p ---
  h=shift(xin,-1)-xin
  s=(shift(yin,-1)-yin)/h
  p=(shift(s,1)*h+s*shift(h,1)) / (shift(h,1)+h)
  ;
  ; --- Monotonization step ---
;  yl=(sign(shift(s,1))+sign(s)) * ((abs(shift(s,1)) < abs(s)) < (0.5*abs(p)))
  yl=(signum(shift(s,1))+signum(s)) * ((abs(shift(s,1)) < abs(s)) < (0.5*abs(p)))
  ;
  ; --- Boundary handling ---
  yl(0)=s(0)
  yl(n-1)=s(n-2)
  ;
  ; --- Compute coefficients for cubic polynomial ---
  a=(yl+shift(yl,-1)-2.0*s)/h^2
  b=(3.0*s-2.0*yl-shift(yl,-1))/h
  c=yl
  d=yin
  ;
  if ((n_elements(xout) ne 0) or (keyword_set(integral))) then begin
    ; --- Interpolation ---
    ;
    ; --- Linear extrapolation at upper boundary ---
    a(n-1)=0.0
    b(n-1)=0.0
    ;
    ; --- Linear extrapolation at lower boundary ---
    a=[0.0,a]
    b=[0.0,b]
    c=[c(0),c]
    d=[d(0),d]
    ;
    ; print,'a=',a
    ; print,'b=',b
    ; print,'c=',c
    ; print,'d=',d
    ;
    if (not keyword_set(integral)) then begin
      ; --- Determine interval indices and relative x value ---
      iout=(0 > long(interpol(findgen(n)+1.0,xin,xout))) < n
      xrel=xout-xin((iout-1)>0)
      ; print,'xrel=',xrel
    endif else begin
      iout=indgen(n)
    endelse
    ; print,'iout=',iout
    ;
    ; --- Compute result ---
    if (keyword_set(derivative)) then begin
      ; --- Derivative ---
      yout=(3.0*a(iout)*xrel+2.0*b(iout))*xrel+c(iout)
    endif else if (keyword_set(integral)) then begin
      ; --- Integration ---
      ;
      ; --- Integration in all intervalls ---
      delx=xin-shift(xin,1)
      delx(0)=delx(1)
      ; print, 'delx=', delx
      yout=(((a(iout)*delx/4.0+b(iout)/3.0)*delx+c(iout)/2.0)*delx+d(iout))*delx
      ;
      if (n0 eq 2) then begin
        ; --- Only two elements ---
        yout=yout(1:2)
      endif else if (n0 eq 3) then begin
        ; --- Only three elements ---
        yout=yout(0:2)
      endif
      ;
      ; --- Reverse if input x vector is non-increasing ---
      if (xinput(0) gt xinput(n0-1)) then begin
        yout(1:*)=-reverse(yout(1:*))
      endif
      ;
      yout(0)=0.0
      ;
      ; --- Integrate over contribution of all intervals ---
      for i=1,n0-1 do begin
        yout(i)=yout(i-1)+yout(i)
      endfor
      ;
    endif else begin
      ; --- Interpolation ---
      yout=((a(iout)*xrel+b(iout))*xrel+c(iout))*xrel+d(iout)
    endelse
    ;
  endif else begin
    ; --- Integration: total integral ---
    ;
    delx=shift(xin,-1)-xin
    yout=(((a*delx/4.0+b/3.0)*delx+c/2.0)*delx+d)*delx
    if (xinput(0) gt xinput(n0-1)) then begin
      yout=-yout
    endif
    ;
    if (n0 eq 2) then begin
      ; --- Only two elements ---
      yout=total(yout(1:n-3))
    endif else if (n0 eq 3) then begin
      ; --- Only three elements ---
      yout=total(yout(0:n-3))
    endif else begin
      ; --- More than three elements ---
      yout=total(yout(0:n-2))
    endelse
    ;
  endelse
endelse
;
; print,yout
return, yout
;
end ; monocubic
