function psymuser, psin, fill=fill
;+
; NAME:
;   psymuser ('plot_symbol_user')
;
; PURPOSE:
;   Define a user symbol.
;
; CATEGORY:
;   Image display
;
; CALLING SEQUENCE:
;   psout=psymuser(psin, fill=fill)
;
; INPUTS:
;   psin: ('plot_symbol_input') integer,
;        Number of plot symbol: (Negative value => connection with line)
;        Standard symbols:
;           1  Plus sign (+)
;           2  Asterisk (*)
;           3  Period (.)
;           4  Diamond
;           5  Triangle
;           6  Square
;           7  X
;           8  User-defined.
;           9  Undefined
;          10  Histogram mode.
;        Extra symbols:
;         101  Smily with arms but without eyes (Don't Panic)
;         102  Circle (o)
;         103  Skull
;         104  (Sun)
;         120-300 Polygons
;         320-500 Stars
;           x  Question mark (?) without point
;
; OUTPUTS:
;   The function PSYMUSER gives a value which can be fed directly into
;   the PLOT procedure via the PSYM keyword: If the input is the number of a
;   standard plotting symbol, the output is this number. Otherwise the output
;   is 8 (or -8).
;
; KEYWORDS:
;   fill: ('fill') If set the plot symbol is filled
;         
; SIDE EFFECTS:
;   Defines a user symbol with usersym.
;
; PROCEDURES:
;   None
;
; EXAMPLE:
;   plot, [0.5], [0.5], psym=psymuser(101), xrange=[0,1], yrange=[0,1], $
;     symsize=10
;   
; MODIFICATION HISTORY:
;   12.10.94: Written by Bernd Freytag, Astrophysik Kiel
;-
;
; --- Check keyword ---
if (not keyword_set(fill)) then fill=0
;
; --- Standard or extra plot symbol? ---
if ((psin ge -10) and (psin le 10)) then begin
  ; --- Standard plot symbol ---
  psout=psin
  ;
endif else begin
  ; --- Extra plot symbol ---
  psout=signum(psin)*8
  ;
  ; --- Look if number of symbol is known ---
  if (abs(psin) eq 101) then begin
    ; --- Smily with arms but without eyes (Don't Panic) ---
    ;
    ; --- Arm ---
    nmax=20
    xarm=fltarr(nmax)
    yarm=xarm
    ;
    n=0
    ;
    xarm(n)=0.55     & yarm(n)=0.3
    n=n+1
    xrh=0.15         & yrh=0.9
    xarm(n)=xrh      & yarm(n)=yrh
    n=n+1
    xarm(n)=xrh-0.25 & yarm(n)=yrh-0.05
    n=n+1
    xarm(n)=xrh      & yarm(n)=yrh
    n=n+1
    xarm(n)=xrh-0.15 & yarm(n)=yrh+0.15
    n=n+1
    xarm(n)=xrh      & yarm(n)=yrh
    n=n+1
    xarm(n)=xrh+0.0  & yarm(n)=yrh+0.2
    n=n+1
    xarm(n)=xrh      & yarm(n)=yrh
    n=n+1
    xarm(n)=xrh+0.12 & yarm(n)=yrh+0.12
    n=n+1
    xarm(n)=xrh      & yarm(n)=yrh
    n=n+1
    xarm(n)=xarm(0)  & yarm(n)=yarm(0)
    n=n+1
    xarm(n)=0.0      & yarm(n)=0.0
    n=n+1
    ;
    narm=n
    ;
    xarm=xarm(0:narm-1)
    yarm=yarm(0:narm-1)
    ;
    ; --- Face --------------------------
    nmax=50
    x=fltarr(nmax)
    y=x
    ;
    n=0
    ;
    ; --- Circle ---
    ndel=17
    angle=(findgen(ndel)) * (!PI*2./(ndel-1))
    x(n:n+ndel-1)=cos(angle)
    y(n:n+ndel-1)=sin(angle)
    n=n+ndel
    ; --- Mouth (lower lip) ---
    x(n)=0.3     & y(n)=-0.65
    n=n+1
    x(n)=-0.3    & y(n)=-0.65
    n=n+1
    x(n)=x(8)    & y(n)=y(8)
    n=n+1
    ; --- Left arm ---
    x(n:n+narm-1)=x(n-1)-xarm
    y(n:n+narm-1)=y(n-1)+yarm
    n=n+narm
    ; --- Mouth (upper lip) ---
    x(n)=0.0     & y(n)=-0.35
    n=n+1
    x(n)=x(16)   & y(n)=y(16)
    n=n+1
    ; --- Right arm ---
    x(n:n+narm-1)=x(n-1)+xarm
    y(n:n+narm-1)=y(n-1)+yarm
    n=n+narm
    ;
    x=x(0:n-1)
    y=y(0:n-1)
    ;
  endif else if (abs(psin) eq 102) then begin
    ; --- Circle ---
    ndel=17
    angle=(findgen(ndel)) * (!PI*2./(ndel-1))
    x=cos(angle)
    y=sin(angle)
  endif else if (abs(psin) eq 103) then begin
    ; --- Skull ---
    x=fltarr(27)
    y=x
    ; --- Eyes ---
    xsp=[-1.0,-0.5,-0.75,-0.25,-0.5, 0.5, 0.25, 0.75, 0.5, 1.0]
    ysp=[ 0.5, 0.5, 0.2 , 0.2 , 0.5, 0.5, 0.2 , 0.2 , 0.5, 0.5]
    ; --- Head ---
    xhe=[-1.0,-0.5 , 0.0, 0.5 , 1.0, 1.0, 0.9, 0.4, 0.3, 0.25, 0.0, $
       -0.25,-0.3,-0.4,-0.9,-1.0,-1.0]
    yhe=[ 0.5, 0.95, 1.0, 0.95, 0.5, 0.1,-0.1,-0.4,-0.6,-0.85,-1.0, $
       -0.85,-0.6,-0.4,-0.1 , 0.1, 0.5]
    x(0:9)=xsp
    y(0:9)=ysp
    x(10:26)=xhe
    y(10:26)=yhe
  endif else if (abs(psin) eq 104) then begin
    ; --- Sun ---
    ndel=15
    x=fltarr(ndel*3)
    y=x
    angle=(findgen(ndel)) * (!PI*2./(ndel-1))
    x(0:ndel-1)=cos(angle)
    y(0:ndel-1)=sin(angle)
    fac1=0.8
    fac2=0.2
    x(ndel:2*ndel-1)=fac1*cos(angle)
    y(ndel:2*ndel-1)=-fac1*sin(angle)
    x(2*ndel:*)=fac2*cos(angle)
    y(2*ndel:*)=fac2*sin(angle)
  endif else if (abs(psin) ge 120 and abs(psin) le 300) then begin
    ; --- Polygon ---
    ndel=fix(abs(psin))/10-9
    alpha=(abs(psin) mod 10)/8.0
    angle=(findgen(ndel) + alpha) * (!PI*2./(ndel-1))
    x=cos(angle)
    y=sin(angle)
  endif else if (abs(psin) ge 320 and abs(psin) le 500) then begin
    ; --- Star ---
    ndel=2*(fix(abs(psin))/10-30)+1
    alpha=(abs(psin) mod 10)/4.0
    angle=(findgen(ndel) + alpha) * (!PI*2./(ndel-1))
    x=cos(angle)
    y=sin(angle)
    fac=0.25
    nrange=bindgen((ndel-1)/2)*2+1
    x(nrange)=fac*x(nrange)
    y(nrange)=fac*y(nrange)
  endif else begin
    ; --- Question mark (?) without point ---
    x=[-0.6, 0.0, 0.6, 0.6, 0.0, 0.0]
    y=[ 0.7, 1.0, 0.7, 0.3, 0.0,-0.8]
  endelse
  ;
  ; --- Set new user symbol ---  
  usersym, x, y, fill=fill
  ;
endelse
;
return, psout
end ; psymuser
