pro setcon
;
;+
; NAME:
;   SETCON ('Set_Constants')
;
; PURPOSE:
;   Define a new system variable "!CON", that is a structure that contains
;   a list of important constants.
;
; CALLING SEQUENCE:
;   setcon
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORD PARAMETERS:
;   None
;
; SIDE EFFECTS:
;   A new system variable '!CON' is created. A variable con1 is created and
;   deleted.
;
; SUBROUTINES:
;   None
;
; PROCEDURE:
;   Straightforward:
;   First a structure variable is created, that contains a list of important
;   physical and astronomical units (from Unsoeld, Baschek 1988, "Der neue
;   Kosmos", 4. Auflage, Seite 402, 403 (G(cgs) corrected)).
;
; EXAMPLE:
;   To created the new system variabel "!con" type:
;     @setcon
;     help, /struct, !CON
;
; MODIFICATION HISTORY:
;   14. 2.94 Written by Bernd Freytag, Astrophysik Kiel
;    9. 4.94 (B.F.) Special characters
;    9.10.98 (B.F.) Rewritten as procedure
;   2007-02-21 (B.F. Lyon) mHydr:  1.6736E-28 -> 1.6736E-24
;   2008-01-31 (B.F.) muAtom
;   2008-12-12 (B.F.) Jupiter (source: http://space.about.com/cs/solarsystem/p/jupiterinfo.htm, Wikipedia)
;-
;
; --- Define structure for new system variable ---
con1={contyp, C___physical___ : 0, $
              G:        6.6726E-08, $
              clight:   2.9979E+10, $
              hPlanck:  6.6261E-27, $
              muAtom:   1.6605402E-24, $
              mProt:    1.6726E-24, $
              mNeut:    1.6749E-24, $
              mHydr:    1.6736E-24, $
              mElek:    9.1094E-28, $
              NAvogadro:6.0221E+23, $
              RGas:     8.3145E+07, $
              kBoltz:   1.3807E-16, $
              alphafine:7.2974E-03, $
              Rydberg:  1.0974E+05, $
              aBohr:    5.2918E-09, $
              cPlanck:  1.4388E+00, $
              sigmaSB:  5.6705E-05, $
              C___astronom___ : 1, $
              au:       1.496E+13,  $
              pc:       3.086E+18,  $
              ylight:   9.45418e+17,$
              ysid:     3.15581e+07,$
              y:        365.2425*24.0*3600.0,$
              C___Earth______ : 2, $
              REarth:   6.378E+08,  $
              MEarth:   5.973E+27,  $
              C___Jupiter____ : 3, $
              RJupiter: 7.1492E+09, $
              MJupiter: 1.8986E+30, $
              C___Sun________ : 4, $
              RSun:     6.980E+10,  $
              MSun:     1.989E+33,  $
              LSun:     3.85E+33,   $
              TeffSun:  5770.0,     $
              gravSun:  2.736E+04,  $
              MagSun:   4.87,       $
              C___Characters_ : 5, $
              AA : string(byte(197)),$   ; Angstrom
              aes : string(byte(228)),$  ; ae
              aeB : string(byte(196)),$  ; AE
              oes : string(byte(246)),$  ; oe
              oeB : string(byte(214)),$  ; OE
              ues : string(byte(252)),$  ; ue
              ueB : string(byte(220)),$  ; UE
              ss : string(byte(223))$   ; ss
                                                   }
;
; --- Define new system variable ---
defsysv, '!con', con1 
;
; --- Show new system variable ---
;help, /structure, !con
;
; --- Delete temporary variables ---
;delvar, con1
;
end ; setcon
