function smooth2d, im, $
  q_smooth=q_smooth, kernel=kernel, q_drop=q_drop, $
  sigma=sigma, $
  q_median=q_median, $
  extend=extend, keepextend=keepextend, periodic=periodic
;
;------------------------------------------------------------------------------
;+
; NAME:
;   smooth2d ('smooth_2_dimensions')
;
; PURPOSE:
;   Smooth image
;
; CATEGORY:
;   Signal processing
;
; CALLING SEQUENCE:
;   im_smooth=smooth2d(im,  $
;                      q_smooth=q_smooth, kernel=kernel, $
;                      sigma=sigma, $
;                      q_median=q_median, $
;                      extend=extend, periodic=periodic)
;
; INPUTS:
;   im:       ('image') 2D array typically with image
;
; OUTPUTS:
;   smooth2d(): smoothed image
;
; KEYWORDS:
;   q_smooth:   ('quantity_smooth') integer counter how often smoothing
;               by convolution with [ 1 2 1 ]/12 should be performed
;                                   [ 2 4 2 ]
;                                   [ 1 2 1 ]
;   kernel:     ('kernel') smoothing kernel matrix to be used if q_smooth is set
;   q_median:   ('quantity_median'), integer (0, 1, 2, ...) to specify
;               half width (-1) of median filtering box
;   sigma:      ('sigma') real parameter: width of Gaussian for Fourier smoothing
;   extend:     ('extend') integer factor specifying number of padding copies (1 is a good value).
;   keepextend: ('keep_extend') keyword, if set extension cells are not removed
;   periodic:   ('periodic') keyword to enable periodic boundary handling for
;               convolution or median smoothing (always on for Fourier smoothing)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Only 2D input data allowed.
;
; PROCEDURE:
;
; EXAMPLE:
;   im_smooth=smooth2d(im, q_median=1, q_smooth=1)
;
; MODIFICATION HISTORY:
;   2002-05-27 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2005-05-15 (B.F. LA) More comments
;   2012-04-04 (B.F. Lyon) Normalize kernel always
;-
;------------------------------------------------------------------------------
;
; --- Define 3x3 smoothing kernel ---
if (not keyword_set(kernel)) then begin
  if (keyword_set(q_drop)) then begin
    kernel=[[q_drop^2, q_drop, q_drop^2], $
            [q_drop  , 1.0   , q_drop  ], $
            [q_drop^2, q_drop, q_drop^2]]
  endif else begin
    ; --- q_drop=0.5 ---
    kernel=[[1.0, 2.0, 1.0], $
            [2.0, 4.0, 2.0], $
            [1.0, 2.0, 1.0]]
  endelse
endif
kernel=kernel/total(kernel)
;
n1=(size(im))(1)
n2=(size(im))(2)
;
; --- Extend array with zeros or by dublication ---
if (keyword_set(extend)) then begin
  nfac=1+2*long(extend > 0)
  n1new=n1 * nfac
  n2new=n2 * nfac
help, extend, nfac, n1,n2, n1new,n2new
  out=make_array(n1new, n2new, value=0.0)
  if (keyword_set(periodic)) then begin
    for i1=0L,nfac-1L do begin
      for i2=0L,nfac-1L do begin
        out(i1*n1:i1*n1+n1-1L, i2*n2:i2*n2+n2-1L)=im
      endfor ; i2
    endfor ; i1
  endif else begin
    out((nfac/2)*n1:(nfac/2)*n1+n1-1L, (nfac/2)*n2:(nfac/2)*n2+n2-1L)=im
  endelse
endif else begin
  nfac=1
  n1new=n1
  n2new=n2
  out=float(reform(im))
endelse
;
; --- Smoothing by repeated application of 3x3 kernel ---
if (keyword_set(q_smooth)) then begin
  if (keyword_set(periodic)) then begin
    edge_truncate=0
    edge_wrap    =1
  endif else begin
    edge_truncate=1
    edge_wrap    =0
  endelse
  for i=1,q_smooth do begin
    out=convol(out, kernel, edge_truncate=edge_truncate, edge_wrap=edge_wrap, /center)
  endfor 
endif
;
; --- Smoothing with help of Fourier analysis/synthesis ---
if (keyword_set(sigma)) then begin
  xarr=dist(n1new, n2new)
  gkernel=exp(-0.5*(xarr/sigma)^2)
  gkernel=gkernel/total(gkernel)*float(n1new)*float(n2new)
  fft_gkernel=fft(gkernel)
  ;
  fft_out=fft(out)
  fft_out=fft_out*fft_gkernel
  out=abs(fft(fft_out, /inverse))
endif
;
; --- Smoothing by median filtering ---
if (keyword_set(q_median)) then begin
  if (q_median gt 1) then begin
    out=median(out, q_median)
  endif
endif
;
; --- Take central part from extended image ---
if (keyword_set(extend) and not keyword_set(keepextend)) then begin
  out=out((nfac/2)*n1:(nfac/2)*n1+n1-1L, (nfac/2)*n2:(nfac/2)*n2+n2-1L)
endif
;
return, out
;
end ; smooth2d
