pro smoothweight, x0, y0, x1, y1, w0=w00, nRepeat=nRepeat0

;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   smoothweight ('smooth_weight')
;
; PURPOSE:
;   Smooth x- and y-axis values of 1D vector accounting for weights.
;
; CATEGORY:
;   Data analysis
;
; CALLING SEQUENCE:
;   smoothweight, x0, y0, x1, y1, w0=w00, nRepeat=nRepeat0
;    
; INPUT:
;   x0:               ('x_0') real vector with input x-axis
;   y0:               ('y_0') real vector with input y-axis
;
; OUTPUT:
;   x1:               ('x_1') real vector with smoothed x-axis
;   y1:               ('y_1') real vector with smoothed y-axis
;
; KEYWORDS:
;   w0:               ('weight_0') real vector with weights, default: all weights=1.0
;   nRepeat:          ('number_Repeat') integer with number of times the smoothing is to be repeated
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; TODO:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2015-12-15 (B.F. Uppsala) First version
;-
;---------------------------------------------------------------------------------------------------

; --- Set common (but non-default) compiler options
compile_opt defint32, strictarr


; === Initialize ===

; --- Check optional keywords ---
if (keyword_set(w00)) then begin
  w0=w00
endif else begin
  w0=x0*0.0+1.0
endelse
if (keyword_set(nRepeat0)) then begin
  nRepeat=long(nRepeat0) > 0
endif else begin
  nRepeat=0
endelse

; --- Check sizes of input arrays ---
nx=n_elements(x0)
ny=n_elements(y0)
nw=n_elements(w0)
if (nx ne ny) then message, 'Sizes of input arrays x and y do not agree.'
if (nx ne nw) then message, 'Sizes of input arrays x and w do not agree.'

; help, x0, y0, w0


; --- Construct smoothing kernel ---
kcw   =0.5
kernel3=[(1.0-kcw)*0.5, kcw, (1.0-kcw)*0.5]
; print, kernel3
kernel2=[kcw*0.5, (1.0-kcw)*0.5]
; print, kernel2

w1=w0
x1=x0
y1=y0

for iRepeat=0,nRepeat-1 do begin
  ;
  x1=x1*w1
  y1=y1*w1
  ;
  w1m=kernel2[0]*w1[0] + kernel2[1]*w1[1]
  w1=convol(w1, kernel3, /center)
  w1[ 0]=w1m
  ;w1[ 0]=w0[ 0]
  w1[-1]=w0[-1]
  ;
  x1m=(kernel2[0]*x1[0] + kernel2[1]*x1[1])/w1[0]
  y1m=(kernel2[0]*y1[0] + kernel2[1]*y1[1])/w1[0]
  x1 =convol(x1, kernel3, /center)/w1
  y1 =convol(y1, kernel3, /center)/w1
  ;
  x1[ 0]=x1m
  ;x1[ 0]=x0[ 0]
  x1[-1]=x0[-1]
  y1[ 0]=y1m
  ;y1[ 0]=y0[ 0]
  y1[-1]=y0[-1]
  ; print, w1[0:3], x1[0:3], y1[0:3], format='(4F7.4,3X,4F7.4,3X,4E11.4)'
endfor ; iRepeat

end ; smoothweight
