function chunkbox, box, dim0, ZeroStart=ZeroStart0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   chunkbox ('chunk_box')
;
; PURPOSE:
;   Cut out chunk of a box
;
; CATEGORY:
;   Box handling
;
; CALLING SEQUENCE:
;   boxout=chunkbox(box, dim, ZeroStart=ZeroStart)
;
; INPUT:
;   box:       ('box') data structure with CO5BOLD data
;   dim0:      ('dimension_0') new dimension array, long[2,3]:
;                 [[m1:n1], [m2:n2], [m3:n3]]
;
; OUTPUT:
;   chunkbox:  ('chunk_box') new data structure with part of the original box
;
; KEYWORDS:
;   ZeroStart: ('Zero_Start') if set the array dimensions in dim0 start counting from 0
;              and do not refer to the range in box.z.dimension
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2005-07-15 (B.F. LA)
;   2006-07-10 (B.F.) () -> []
;   2009-03-13 (B.F. Lyon) Just copy head data
;   2010-03-11 (T.S. Napels) handling of zdata changed to admit "incomplete" boxes
;   2010-03-12 (B.F. Napels) Magnetic fields, qucs, time_db
;   2018-06-07 (B.F. Uppsala) Account for non-existing box tags
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

; --- Restrict input dimensions to allowed values ---
if (n_elements(dim0) ne 0) then begin
  ZeroStart=keyword_set(ZeroStart0)
  dim      =long(dim0)
  if (ZeroStart) then begin
    ; --- Shift dimension to range in input data structure ---
    dim[0,*]=dim[0,*] + box.z.dimension[0,*]
    dim[1,*]=dim[1,*] + box.z.dimension[0,*]
  endif
  dim[0,*]=dim[0,*] > box.z.dimension[0,*]
  dim[1,*]=dim[1,*] < box.z.dimension[1,*]
endif else begin
  ZeroStart=0
  dim      =box.z.dimension
endelse

m1=dim[0,0]-box.z.dimension[0,0]
n1=dim[1,0]-box.z.dimension[0,0]
m2=dim[0,1]-box.z.dimension[0,1]
n2=dim[1,1]-box.z.dimension[0,1]
m3=dim[0,2]-box.z.dimension[0,2]
n3=dim[1,2]-box.z.dimension[0,2]

;print, dim
;print, box.z.dimension
;print, m1,n1, m2,n2, m3,n3
;print, m2,n2
;print, m3,n3
;help,/str,box
;help,/str,box.head

; --- Append comment to history ---
;if (n_elements(box.head.history) lt 14) then begin
;  history=[box.head.history, $
;           'Processed by chunkbox: ' + strtrim(systime(0),2) $
;          ]
;endif else begin
;  history=[box.head.history[0:8], $
;           '...', $
;           box.head.history[12:*], $
;           'Processed by chunkbox: ' + strtrim(systime(0),2) $
;          ]
;endelse

version='chunkbox 2018-06-078'

; --- Head data structure ---
;headdata=create_struct( $
;  'type'       , 'uio', $
;  'file_id'    , 'rhd-data', $
;  'description', box.head.description, $
;  'history'    , history, $
;  'version'    , version $
;)

; --- Check basic volume data ---
boxtags  =tag_names(box)
ztags    =tag_names(box.z)
indquctag=where(strmid(ztags,0,3) eq 'QUC', nquc)

zdata=create_struct( $
  'type'       , 'uio', $
  'box_id'     , 'z', $
  'dimension'  , dim, $
  'time'       , box.z.time, $
  'time_db'    , ((where(ztags eq 'TIME_DB'))[0] ne -1) ? box.z.time_db : double(box.z.time), $
  'itime'      , box.z.itime, $
  'xc1'        , reform(box.z.xc1[m1:n1   ,0       ,0      ], n1-m1+1, 1      , 1      ), $
  'xc2'        , reform(box.z.xc2[0       ,m2:n2   ,0      ], 1      , n2-m2+1, 1      ), $
  'xc3'        , reform(box.z.xc3[0       ,0       ,m3:n3  ], 1      , 1      , n3-m3+1), $
  'xb1'        , reform(box.z.xb1[m1:n1+1 ,0       ,0      ], n1-m1+2, 1      , 1      ), $
  'xb2'        , reform(box.z.xb2[0       ,m2:n2+1 ,0      ], 1      , n2-m2+2, 1      ), $
  'xb3'        , reform(box.z.xb3[0       ,0       ,m3:n3+1], 1      , 1      , n3-m3+2))
if ((where(ztags eq 'RHO'))[0] ne -1) then zdata=create_struct(zdata,'rho', reform(box.z.rho[m1:n1   ,m2:n2   ,m3:n3   ], n1-m1+1, n2-m2+1, n3-m3+1))
if ((where(ztags eq 'EI' ))[0] ne -1) then zdata=create_struct(zdata,'ei' , reform(box.z.ei[ m1:n1   ,m2:n2   ,m3:n3   ], n1-m1+1, n2-m2+1, n3-m3+1))
if ((where(ztags eq 'V1' ))[0] ne -1) then zdata=create_struct(zdata,'v1' , reform(box.z.v1[ m1:n1   ,m2:n2   ,m3:n3   ], n1-m1+1, n2-m2+1, n3-m3+1))
if ((where(ztags eq 'V2' ))[0] ne -1) then zdata=create_struct(zdata,'v2' , reform(box.z.v2[ m1:n1   ,m2:n2   ,m3:n3   ], n1-m1+1, n2-m2+1, n3-m3+1))
if ((where(ztags eq 'V3' ))[0] ne -1) then zdata=create_struct(zdata,'v3' , reform(box.z.v3[ m1:n1   ,m2:n2   ,m3:n3   ], n1-m1+1, n2-m2+1, n3-m3+1))
if ((where(ztags eq 'BB1'))[0] ne -1) then zdata=create_struct(zdata,'bb1', reform(box.z.bb1[m1:n1+1 ,m2:n2   ,m3:n3   ], n1-m1+2, n2-m2+1, n3-m3+1))
if ((where(ztags eq 'BB2'))[0] ne -1) then zdata=create_struct(zdata,'bb2', reform(box.z.bb2[m1:n1   ,m2:n2+1 ,m3:n3   ], n1-m1+1, n2-m2+2, n3-m3+1))
if ((where(ztags eq 'BB3'))[0] ne -1) then zdata=create_struct(zdata,'bb3', reform(box.z.bb3[m1:n1   ,m2:n2   ,m3:n3+1 ], n1-m1+1, n2-m2+1, n3-m3+2))
for iquc=1,nquc do begin                                         
  ident='00' + strtrim(string(iquc, format='(I3)'),2)            
  ident='quc' + strmid(ident,strlen(ident)-3,3)                  
  dummy=execute('zdata=create_struct(zdata,ident, reform(box.z.' + ident + '[m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1))')
endfor ; iquc 

; --- Put the box together ---
outbox=create_struct( $
  'type'              , 'uio', $
;  'head'              , headdata, $
  'head'             , ((where(boxtags eq 'DTIME'))[0] ne -1) ? box.head : '', $
  'dataset_id'        , 'single_box', $
  'modeltime'         , box.modeltime, $
  'modeltime_db'      , ((where(boxtags eq 'MODELTIME_DB'))[0] ne -1) ? box.modeltime_db : double(box.modeltime), $
  'modelitime'        , box.modelitime, $
  'dtime'             , ((where(boxtags eq 'DTIME'))[0] ne -1) ? box.dtime : 0.0, $
  'time_out_full_last', ((where(boxtags eq 'TIME_OUT_FULL_LAST'))[0] ne -1) ? box.time_out_full_last : -1.0, $
  'time_out_mean_last', ((where(boxtags eq 'TIME_OUT_MEAN_LAST'))[0] ne -1) ? box.time_out_mean_last : -1.0, $
  'z'                 , temporary(zdata) $
)

if (total(tag_names(box) eq 'EOS') gt 0.0) then begin
  ; --- Append EOS data to box ---
  eosdata=create_struct( $
    'P'     , reform(box.eos.P[     m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
    'dPdrho', reform(box.eos.dPdrho[m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
    'dPdei' , reform(box.eos.dPdei[ m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
    'T'     , reform(box.eos.T[     m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
    'dTdei' , reform(box.eos.dTdei[ m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
    's'     , reform(box.eos.s[     m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1)   )
  outbox=create_struct(temporary(outbox), 'eos', temporary(eosdata))
endif

if (total(tag_names(box) eq 'OPA') gt 0.0) then begin
  ; --- Append opacity data to box ---
  opadata=create_struct( $
    'kappa' , reform(box.opa.kappa[ m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1))
  outbox=create_struct(temporary(outbox), 'opa', temporary(opadata))
endif

return, outbox

end ; chunkbox
