function combox, command, box, $
  par=par, noaxis=noaxis, meansub=meansub, norm=norm, execute=execute
;
;------------------------------------------------------------------------------
;+
; NAME:
;   combox ('compute_box')
;
; PURPOSE:
;   Compute derived quantities from independent variables and EOS/opacity data
;
; CATEGORY:
;   EOS
;
; CALLING SEQUENCE:
;   out=combox(command, box)
;
; INPUT:
;   command: ('command') string with requested quantity
;   box:     ('box') data structure with CO5BOLD data (including EOS and opacities)
;
; OUTPUT:
;   out:     ('output') 3D array
;
; KEYWORDS:
;   par:     ('parameters') structure containing data from a .par file, needed for some variables 
;   noaxis:  ('no_axis') If set the dimensions (m1,n1,...) are not taken from axes but from
;              dimension tag in box structure
;   meansub: ('mean_subtract') If set the horizontal mean (over 1st and 2nd dimension) is subtracted
;   norm:    ('normalize') If set (in addition to meansub) the result is normalized:
;              out=(out-out_mean)/out
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   opta_info:
;   xbnu:
;   xkaros:
;   tau_int:  Integrate optical depth scale
;   derivbox: Compute derivatives from independent variables
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   To get all all possible results, the sub-structures box.eos and box.opa
;   need to be properly defined.
;
; PROCEDURE:
;
; EXAMPLE:
;   ; --- Assume: modelfile is defined; EOS and opacity table are read ---
;
;   ; --- Load data set ---
;   ful=uio_dataset_rd(modelfile)
;
;   ; --- Solve EOS, compute opacities ---
;   eosbox, ful, eos=EOS, opa=1
;
;   ; --- Compute heat capacity ---
;   cp=combox('cp', ful)
;
; MODIFICATION HISTORY:
;    2. 3.01 Written by Bernd Freytag, Uppsala
;   12. 3.01 (B.F.) More items
;    8. 1.02 (B.F.) cs not in box.eos anymore
;   27. 3.02 (S.W.) new keyword PAR
;   12. 5.03 (B.F.) new keyword noaxis
;   10.11.03 (B.F.) rearranged formula for cp
;   10.12.03 (B.F.) from Svens version: mass3l/r, message
;   29. 7.04 (B.F. Montpellier) time_rad3
;   27. 7.05 (B.F. LA) Dimensions from DIMENSION entry
;   14.11.05 (B.F. Uppsala) v_vec added
;   2006-05-22 (B.F. LA) Compute horizontal mean in double precision
;   2006-06-02 (B.F.) omegaBVsqrograv added
;   2007-02-01 (B.F. Lyon) mach_v3
;   2007-02-02 (B.F.) Keyword norm added; call derivbox if command not found;
;                     First version of 'execute' functionality'
;   2009-10-30 (B.F.) Some square brackets, time_rad3nongrey
;   2010-02-06 (B.F.) Magnetic field: 'bc', unit_B_value
;   2018-04-17 (B.F.) Some dust quantities
;   2019-02-27 (B.F.) Radiative acceleration onto dust grains
;-
;------------------------------------------------------------------------------
;
unit_B_value=1.0/sqrt(4.0*!PI)
;
if (not keyword_set(par)) then par=0
;
if (not keyword_set(noaxis)) then begin
  if (total(tag_names(box.z) eq 'DIMENSION')) then begin
    ; --- Dimensions from DIMENSION entry ---
    m1=0
    n1=box.z.dimension[1,0]-box.z.dimension[0,0]
    m2=0
    n2=box.z.dimension[1,1]-box.z.dimension[0,1]
    m3=0
    n3=box.z.dimension[1,2]-box.z.dimension[0,2]
  endif else begin
    ; --- Dimensions from axes ---
    m1=0
    n1=n_elements(box.z.xc1)-1
    m2=0
    n2=n_elements(box.z.xc2)-1
    m3=0
    n3=n_elements(box.z.xc3)-1
  endelse
endif
;
cm=strlowcase(command)
;
if (keyword_set(execute)) then begin
  ; --- Interpret command als function (test mode) ---
  ;
  ; --- Parse command string ---
  spos=strsplit(command, '+-/*^()', len=slen, count=snum)
  ;
  if (snum gt 0) then begin
    ; --- Some sub strings were found ---
    ;
    ; --- First separaror, e.g. leading '(' ---
    cm_sepfirst=strmid(command, 0, spos[0])
    ;help, cm_sepfirst
    if (snum ge 2) then begin
      ; --- Inner separators ---
      cm_sep=strmid(command,(spos+slen)[0:snum-2],spos[1:snum-1]-(spos+slen)[0:snum-2])
      ;print, 'cm_sep: ' + cm_sep
    endif
    ; --- Last separator, e.g. trailing '(' ---
    cm_seplast=strmid(command, spos[snum-1]+slen[snum-1])
    ;help, cm_seplast
    ;
    ; --- Add recursive calls of combox ---
    cm_subcms='combox(''' + strmid(command, spos, slen) + ''',box,par=par)'
    ;print, 'cm_subcms: ' + cm_subcms
    ;
    ; --- Start of execute string ---
    cm_full='out=' + cm_sepfirst + cm_subcms[0]
    ; --- Inner commands and separators ---
    for i=1,n_elements(cm_subcms)-1 do begin
      cm_full=cm_full + cm_sep[i-1] + cm_subcms[i]
    endfor
    ; --- Last separator ---
    cm_full=cm_full + cm_seplast
    ;
    help, cm_full
    ;
    ; --- Execute the produced string ---
    dummy=execute(cm_full)
    ;
  endif else begin
    message, 'Command parsing not successful', /informational
  endelse
endif else if (cm eq 'bc') then begin
  ; --- Cell averaged magnetic field ---
  ;
  out=      (box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ] + box.z.Bb1[m1:n1,m2:n2,m3:n3])^2
  out=out + (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ] + box.z.Bb2[m1:n1,m2:n2,m3:n3])^2
  out=out + (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1] + box.z.Bb3[m1:n1,m2:n2,m3:n3])^2
  out=(0.5/unit_B_value) * sqrt(out)
  ;
endif else if (cm eq 'pmag') then begin
  ; --- Magnetic pressure from cell averaged magnetic field ---
  ;
  out=      (box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ] + box.z.Bb1[m1:n1,m2:n2,m3:n3])^2
  out=out + (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ] + box.z.Bb2[m1:n1,m2:n2,m3:n3])^2
  out=out + (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1] + box.z.Bb3[m1:n1,m2:n2,m3:n3])^2
  out=0.125 * out
  ;
endif else if (cm eq 'bc1') then begin
  ; --- Cell-averaged magnetic field 1 ---
  ;
  out=(0.5/unit_B_value) * (box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ] + box.z.Bb1[m1:n1,m2:n2,m3:n3])
  ;
endif else if (cm eq 'bc2') then begin
  ; --- Cell-averaged magnetic field 2 ---
  ;
  out=(0.5/unit_B_value) * (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ] + box.z.Bb2[m1:n1,m2:n2,m3:n3])
  ;
endif else if (cm eq 'bc3') then begin
  ; --- Cell-averaged magnetic field 3 ---
  ;
  out=(0.5/unit_B_value) * (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1] + box.z.Bb3[m1:n1,m2:n2,m3:n3])
  ;
endif else if (cm eq 'bb1') then begin
  ; --- Boundary-located magnetic field 1 ---
  ;
  out=(1.0/unit_B_value) * box.z.Bb1
  ;
endif else if (cm eq 'bb2') then begin
  ; --- Boundary-located magnetic field 2 ---
  ;
  out=(1.0/unit_B_value) * box.z.Bb2
  ;
endif else if (cm eq 'bb3') then begin
  ; --- Boundary-located magnetic field 3 ---
  ;
  out=(1.0/unit_B_value) * box.z.Bb3
  ;
endif else if (cm eq 'bc_radial') then begin
  ; --- Radial magnetic field ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out=(0.5/unit_B_value) * $
      ((box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ] + box.z.Bb1[m1:n1,m2:n2,m3:n3]) * xc1a + $
       (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ] + box.z.Bb2[m1:n1,m2:n2,m3:n3]) * xc2a + $
       (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1] + box.z.Bb3[m1:n1,m2:n2,m3:n3]) * xc3a  )/ $
       sqrt(eps^2 + xc1a^2 + xc2a^2 + xc3a^2)
  ;
endif else if (cm eq 'bc3overbc') then begin
  ; --- Cell averaged magnetic field 3 ---
  ;
  bc3=      (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1] + box.z.Bb3[m1:n1,m2:n2,m3:n3])
  out=      (box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ] + box.z.Bb1[m1:n1,m2:n2,m3:n3])^2
  out=out + (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ] + box.z.Bb2[m1:n1,m2:n2,m3:n3])^2
  out=out + (bc3)^2
  out=bc3/sqrt(out > 1.0E-30)
  ;
endif else if (cm eq 'vac1') then begin
  ; --- Alven speed from cell averaged magnetic field 1 ---
  ;
  out=0.5 * (box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ] + box.z.Bb1[m1:n1,m2:n2,m3:n3])/sqrt(box.z.rho)
  ;
endif else if (cm eq 'vac2') then begin
  ; --- Alven speed from cell averaged magnetic field 2 ---
  ;
  out=0.5 * (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ] + box.z.Bb2[m1:n1,m2:n2,m3:n3])/sqrt(box.z.rho)
  ;
endif else if (cm eq 'vac3') then begin
  ; --- Alven speed from cell averaged magnetic field 3 ---
  ;
  out=0.5 * (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1] + box.z.Bb3[m1:n1,m2:n2,m3:n3])/sqrt(box.z.rho)
  ;
endif else if (cm eq 'cp') then begin
  ; --- Heat capacity (constant pressure) ---
  ;
;  out=1.0/( box.eos.dTdei - $
;            ( (box.eos.T * box.eos.dPdei^2) / $
;              (box.z.rho^2*box.eos.dPdrho + box.eos.P*box.eos.dPdei) $
;            ) $
;          )
  out=1.0/( box.eos.dTdei - $
            ( (box.eos.T * box.eos.dPdei) / $
              ((box.z.rho*box.eos.dPdrho)/box.eos.dPdei*box.z.rho + box.eos.P) $
            ) $
          )
  ;
endif else if ((cm eq 'cpdash') or (cm eq 'cpd')) then begin
  ; --- c_p'=P/(rho*T) * Gamma_1/(Gamma_3-1) ---
  ;
  out=box.eos.dPdrho*box.z.rho/box.eos.P + box.eos.dPdei/box.z.rho ; Gamma-1
  out=out*box.eos.P/(box.eos.T*box.eos.dPdei)
  ;
endif else if (cm eq 'cs') then begin
  ; --- Adiabatic sound speed ---
  ;
  out=sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  ;
endif else if (cm eq 'cv') then begin
  ; --- Heat capacity (constant volume or density) ---
  ;
  out=1.0/box.eos.dTdei
  ;
endif else if ((cm eq 'cvdash') or (cm eq 'cvd')) then begin
  ; --- c_v'=P/(rho*T) * 1/(Gamma_3-1) ---
  ;
  out=box.eos.P/(box.eos.T*box.eos.dPdei)
  ;
endif else if (cm eq 'divb') then begin
  ; --- Divergence of magnetic field ---
  ;
  if (m2 eq n2) then begin
    dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
    dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
    out=(box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ]-box.z.Bb1[m1:n1,m2:n2,m3:n3])/dxb1a + $
        (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1]-box.z.Bb3[m1:n1,m2:n2,m3:n3])/dxb3a
  endif else begin
    ;dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
    ;dxb2a=rebin(box.z.xb2[*,m2+1:n2+1,*]-box.z.xb2[*,m2:n2,*],n1-m1+1, n2-m2+1, n3-m3+1)
    ;dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
    ;out=(box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ]-box.z.Bb1[m1:n1,m2:n2,m3:n3])/dxb1a + $
    ;    (box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ]-box.z.Bb2[m1:n1,m2:n2,m3:n3])/dxb2a + $
    ;    (box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1]-box.z.Bb3[m1:n1,m2:n2,m3:n3])/dxb3a
    out=    (box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ]-box.z.Bb1[m1:n1,m2:n2,m3:n3])/rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
    out=out+(box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ]-box.z.Bb2[m1:n1,m2:n2,m3:n3])/rebin(box.z.xb2[*,m2+1:n2+1,*]-box.z.xb2[*,m2:n2,*],n1-m1+1, n2-m2+1, n3-m3+1)
    out=out+(box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1]-box.z.Bb3[m1:n1,m2:n2,m3:n3])/rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  endelse
  ;
endif else if (cm eq 'delta') then begin
  ; --- delta ---
  ;
  Gamma_1=box.eos.dPdrho*box.z.rho/box.eos.P + box.eos.dPdei/box.z.rho
  out=1.0/((Gamma_1*box.eos.P*box.eos.dTdei)/(box.eos.T*box.eos.dPdei) - (box.eos.dPdei/box.z.rho))
  ;
endif else if (cm eq 'dtau1') then begin
  ; --- Delta optical depth in 1 direction ---
  ;
  dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
  out  =box.z.rho * box.opa.kappa * dxb1a
  ;
endif else if (cm eq 'dtau2') then begin
  ; --- Delta optical depth in 2 direction ---
  ;
  dxb2a=rebin(box.z.xb2[*,m2+1:n2+1,*]-box.z.xb2[*,m2:n2,*],n1-m1+1, n2-m2+1, n3-m3+1)
  out  =box.z.rho * box.opa.kappa * dxb2a
  ;
endif else if (cm eq 'dtau3') then begin
  ; --- Delta optical depth in 3 direction ---
  ;
  dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  out  =box.z.rho * box.opa.kappa * dxb3a
  ;
endif else if ((cm eq 'dtde') or (cm eq 'dtdei')) then begin
  ; --- Temperature derivative dT/dei|rho ---
  ;
  out=box.eos.dTdei
  ;
endif else if (cm eq 'ei') then begin
  ; --- Internal energy per mass ---
  ;
  out=box.z.ei
  ;
endif else if (cm eq 'eip') then begin
  ; --- Enthalpy per mass ---
  ;
  out=box.z.ei+box.eos.P/box.z.rho
  ;
endif else if (cm eq 'ek') then begin
  ; --- Kinetic energy per volume ---
  ;
  out=0.5*(box.z.v1^2+box.z.v2^2+box.z.v3^2)
  ;
endif else if ((cm eq 'em') or (cm eq 'em_ver1')) then begin
  ; --- Magnetic energy per mass ---
  ;
  out=0.125*((box.z.Bb1[m1:n1,m2:n2,m3:n3] + box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ])^2 + $
             (box.z.Bb2[m1:n1,m2:n2,m3:n3] + box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ])^2 + $
             (box.z.Bb3[m1:n1,m2:n2,m3:n3] + box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1])^2  )
  ;
endif else if (cm eq 'em_ver2') then begin
  ; --- Magnetic energy per mass ---
  ;
  out=0.25*(box.z.Bb1[m1:n1,m2:n2,m3:n3]^2 + box.z.Bb1[m1+1:n1+1,m2  :n2  ,m3  :n3  ]^2 + $
            box.z.Bb2[m1:n1,m2:n2,m3:n3]^2 + box.z.Bb2[m1  :n1  ,m2+1:n2+1,m3  :n3  ]^2 + $
            box.z.Bb3[m1:n1,m2:n2,m3:n3]^2 + box.z.Bb3[m1  :n1  ,m2  :n2  ,m3+1:n3+1]^2  )
  ;
endif else if (cm eq 'emission') then begin
  out=box.z.rho*box.eos.T^4*box.opa.kappa
  ;
endif else if ((cm eq 'gamma1') or (cm eq 'gamma_1')) then begin
  ; --- First adiabatic coefficient ---
  ;
  out=box.eos.dPdrho*box.z.rho/box.eos.P + box.eos.dPdei/box.z.rho
  ;
endif else if ((cm eq 'gamma3') or (cm eq 'gamma_3')) then begin
  ; --- Third adiabatic coefficient ---
  ;
  out=box.eos.dPdei/box.z.rho + 1.0
  ;
endif else if (cm eq 'gravraddust001') then begin
  ; --- Approximate radiative acceleration ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Grain properties: forsterite ---
    HEH   = 1.0E-01                     ; abundance of He
    SMASS = !con.MProt*(1.0 + HEH*4.0)
    ZNL   = 1.0E+03                     ; number of monomers in seed particle
    RHOD  = 3.27                        ; density of dust monomer, Mg2SiO4 density (Jager et al. 2003)
    AW    = 2.0*24.0 + 28.0 + 4.0*16.0  ; mass of monomer, Mg2SiO4, in proton masses
    ;
    ; --- Toy model for forsterite grain opacities (Hoefner 2008) ---
    q1        = 2.00E-03                ; Efficiency
    q4        = 4.68E+02
    qmax      = 1.0
    lambdamax = 1.08E-4
    ;
    n_MonoPerGrain = ZNL + (SMASS/par.ar_dustGrainAbu[0]) * box.z.quc001/box.z.rho
    a_Grain        = (((3.0*AW*!con.MProt)/(4.0*!PI*RHOD)) * n_MonoPerGrain)^(1.0/3.0)
    Q_rp           = (q1*(a_Grain/lambdamax) + q4*(a_Grain/lambdamax)^4) < qmax
    ;n_GrainProVol  = box.z.quc001/n_MonoPerGrain
    ;kapparho_rp    = !pi * a_Grain^2 * Q_rp * n_GrainProVol
    kapparho_rp    = !pi * a_Grain^2 * Q_rp * box.z.quc001 / n_MonoPerGrain
    ;
    r_core = par.r0_grav
    if (total(tag_names(par) eq 'C_GRAVCORER0HEAT')) then begin
      if (par.C_gravCorer0Heat > 0.0) then begin
        r_core = par.C_gravCorer0Heat*par.r0_grav
      endif
    endif
    F_CoreLum = 1.0/3.0 * r_core * par.LuminosityPerVolume
    F_rad     = F_CoreLum * (r_core^2/(combox('radius', box)^2 > (r_core^2)))
    ;
    ; out=sqrt(sqrt(F_rad/!con.sigmaSB))
    out       = F_rad * kapparho_rp / (!con.cLight * box.z.rho)
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'kappa') then begin
  ; --- Opacity per mass unit ---
  ;
  out=box.opa.kappa
  ;
endif else if ((cm eq 'kapparho') or (cm eq 'rhokappa')) then begin
  ; --- Opacity per volume ---
  ;
  out=box.z.rho * box.opa.kappa
  ;
endif else if (cm eq 'l2') then begin
  ; --- Angular momentum 2 ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  out =box.z.rho*(xc3a*box.z.v1 - xc1a*box.z.v3)
  ;
endif else if (cm eq 'l3') then begin
  ; --- Angular momentum 3 ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  out =box.z.rho*(xc1a*box.z.v2 - xc2a*box.z.v1)
  ;
endif else if ((cm eq 'mach') or (cm eq 'ma')) then begin
  ; --- Mach number ---
  out=sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  out=sqrt(box.z.v1^2+box.z.v2^2+box.z.v3^2)/out
  ;
endif else if ((cm eq 'mach_v1') or (cm eq 'machv1') or (cm eq 'mach1') or (cm eq 'ma1')) then begin
  ; --- Mach number in x1 direction ---
  out=sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  out=box.z.v1/out
  ;
endif else if ((cm eq 'mach_v2') or (cm eq 'machv2') or (cm eq 'mach2') or (cm eq 'ma2')) then begin
  ; --- Mach number in x2 direction ---
  out=sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  out=box.z.v2/out
  ;
endif else if ((cm eq 'mach_v3') or (cm eq 'machv3') or (cm eq 'mach3') or (cm eq 'ma3')) then begin
  ; --- Mach number in x3 direction ---
  out=sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  out=box.z.v3/out
  ;
endif else if ((cm eq 'mach_radial') or (cm eq 'ma_radial') or (cm eq 'mar')) then begin
  ; --- 'Radial' Mach number ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out=sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  out=(box.z.v1*xc1a + box.z.v2*xc2a + box.z.v3*xc3a) / $
      (sqrt(eps^2 + xc1a^2 + xc2a^2 + xc3a^2)*out)
  ;
endif else if (cm eq 'mach_tangential') then begin
  ; --- 'Tangential' Mach number ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out=(box.z.v1*xc1a + box.z.v2*xc2a + box.z.v3*xc3a) / $
       sqrt(eps^2 + xc1a^2 + xc2a^2 + xc3a^2)
  out=sqrt((box.z.v1^2+box.z.v2^2+box.z.v3^2) - out^2)/ $
      sqrt((box.eos.P/box.z.rho)*(box.eos.dPdei/box.z.rho) + box.eos.dPdrho)
  ;
endif else if (cm eq 'mass1l') then begin
  ; --- Integrated mass ---
  ;
  dxb1a=rebin(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*),n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho
  for i1=m1+1,n1 do begin
    out(i1,*,*)=out(i1-1,*,*) + 0.5*(box.z.rho(i1-1,*,*)+box.z.rho(i1  ,*,*))
  endfor
  ;
endif else if (cm eq 'mass1r') then begin
  ; --- Integrated mass ---
  ;
  dxb1a=rebin(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*),n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho
  for i1=n1-1,m1,-1 do begin
    out(i1,*,*)=out(i1+1,*,*) + 0.5*(box.z.rho(i1+1,*,*)+box.z.rho(i1  ,*,*))
  endfor
  ;
endif else if (cm eq 'mass3l') then begin
  ; --- Integrated mass ---
  ;
  dxb3a=rebin(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3),n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho
  for i3=m3+1,n3 do begin
    out(*,*,i3)=out(*,*,i3-1) + 0.5*(box.z.rho(*,*,i3-1)+box.z.rho(*,*,i3))*dxb3a(i3)
  endfor
  ;
endif else if (cm eq 'mass3r') then begin
  ; --- Integrated mass ---
  ;
  dxb3a=rebin(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3),n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho
  for i3=n3-1,m3,-1 do begin
    out(*,*,i3)=out(*,*,i3+1) + 0.5*(box.z.rho(*,*,i3+1)+box.z.rho(*,*,i3))*dxb3a(i3)
  endfor
  ;
endif else if (cm eq 'mu') then begin
  ; --- Mean particle weight ---
  ;
  out=!con.Rgas * (box.z.rho*box.eos.T)/box.eos.P
  ;
endif else if (cm eq 'omegabvsqrograv') then begin
  ; --- Brunt-Vaeisaelae frequency squared over gravity: delta/c_p * ds/dz ---
  ;
  xc3=reform(box.z.xc3)
  Gamma_1=box.eos.dPdrho*box.z.rho/box.eos.P + box.eos.dPdei/box.z.rho
  delta  =1.0/((temporary(Gamma_1)*box.eos.P*box.eos.dTdei)/(box.eos.T*box.eos.dPdei) - $
               (box.eos.dPdei/box.z.rho))
  cp     =1.0/( box.eos.dTdei - $
                ( (box.eos.T * box.eos.dPdei) / $
                 ((box.z.rho*box.eos.dPdrho)/box.eos.dPdei*box.z.rho + box.eos.P) $
                ) $
              )
  out=temporary(delta)/temporary(cp)
  for i1=m1,n1 do begin
    for i2=m2,n2 do begin
      out[i1,i2,*]=out[i1,i2,*]*deriv(xc3, box.eos.s[i1,i2,*])
    endfor ; i2
  endfor ; i1
  ;
endif else if (cm eq 'p') then begin
  ; --- Pressure ---
  ;
  out=box.eos.P
  ;
endif else if (cm eq 'quc001') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc001
  ;
endif else if (cm eq 'quc001orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc001/box.z.rho
  ;
endif else if (cm eq 'quc002') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc002
  ;
endif else if (cm eq 'quc002m001') then begin
  ; --- Additional tracer, only where quc001 is not condensed ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Condensation fraction: forsterite ---
    HEH    = 1.0E-01                    ; abundance of He
    SMASS  = !con.MProt*(1.0 + HEH*4.0)
    EPSMG  = 3.800E-05                  ; solar
    ; IDUST  = -2                       ; Mg in growth rate
    HELPMG = EPSMG / SMASS
    ;
    f=1.0-box.z.quc001/((HELPMG/2.0)*box.z.rho)
    ;
    ; --- Density: corundum ---
    ;
    out=f*box.z.quc002
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'quc002orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc002/box.z.rho
  ;
endif else if (cm eq 'quc003') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc003
  ;
endif else if (cm eq 'quc003orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc003/box.z.rho
  ;
endif else if (cm eq 'quc004') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc004
  ;
endif else if (cm eq 'quc004orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc004/box.z.rho
  ;
;endif else if (cm eq 'quc004orhonorm') then begin
;  ; --- Additional tracer ---
;  ;
;  out=box.z.quc004/(box.z.rho*2.89750e+20*(par.c_dust01-1.0))
;  ;
endif else if (cm eq 'quc005') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc005
  ;
endif else if (cm eq 'quc005orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc005/box.z.rho
  ;
endif else if (cm eq 'quc006') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc006
  ;
endif else if (cm eq 'quc006orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc006/box.z.rho
  ;
endif else if (cm eq 'quc007') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc007
  ;
endif else if (cm eq 'quc007orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc007/box.z.rho
  ;
endif else if (cm eq 'quc008') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc008
  ;
endif else if (cm eq 'quc008orho') then begin
  ; --- Additional tracer ---
  ;
  out=box.z.quc008/box.z.rho
  ;
endif else if (cm eq 'rdust') then begin
  ; --- Grain radius ---
  ;
  if (par.dustscheme eq 'dust_k3mon_03') then begin
    out=par.ar_dustGrainRadius[1]*(box.z.quc002/(box.z.quc001+box.z.quc002))^(1.0/3.0)
    ;
  endif else if (par.dustscheme eq 'dust_bins_01') then begin
    tags=tag_names(box.z)
    indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
    outr=0.0*box.z.rho
    out=0.0*box.z.rho
    for iquc=1,nquc-1 do begin
      qucid='00'+trim(string(iquc+1),2)
      qucid=strmid(qucid, strlen(qucid)-1, 3, /reverse_offset)
      dummy=execute('outr=outr+box.z.quc' + qucid + '*par.ar_dustGrainRadius[' + string(iquc) + ']')
      dummy=execute('out=out+box.z.quc' + qucid)
    endfor ; i
    ;
    ind=where(out ne 0.0, nind)
    if (nind ge 1) then begin
      out[ind]=outr[ind]/out[ind]
    endif
    ;
  endif ; (par.dustscheme)
  ;
endif else if (cm eq 'rdust001') then begin
  ; --- Grain radius ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Grain radius: forsterite ---
    HEH   = 1.0E-01                     ; abundance of He
    SMASS = !con.MProt*(1.0 + HEH*4.0)
    ZNL   = 1.0E+03                     ; number of monomers in seed particle
    RHOD  = 3.27                        ; density of dust monomer, Mg2SiO4 density (Jager et al. 2003)
    AW    = 2.0*24.0 + 28.0 + 4.0*16.0  ; mass of monomer, Mg2SiO4, in proton masses
    ;
    out=(((3.0*AW*!con.MProt)/(4.0*!PI*RHOD)) * $
        (ZNL + (SMASS/par.ar_dustGrainAbu[0]) * box.z.quc001/box.z.rho))^(1.0/3.0)
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'rdust002') then begin
  ; --- Grain radius ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Grain radius: corundum ---
    HEH   = 1.0E-01                     ; abundance of He
    SMASS = !con.MProt*(1.0 + HEH*4.0)
    ZNL   = 1.0E+03                     ; number of monomers in seed particle
    RHOD  = 3.99                        ; density of dust monomer, Al2O3 (G&S Ph+Ch_CDS Tab.12.1/CRC-HB)
    AW    = 2.0*27.0 + 3.0*16.0         ; mass of monomer, Al2O3, in proton masses
    ;
    out=(((3.0*AW*!con.MProt)/(4.0*!PI*RHOD)) * $
        (ZNL + (SMASS/par.ar_dustGrainAbu[1]) * box.z.quc002/box.z.rho))^(1.0/3.0)
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'condfrac001') then begin
  ; --- Condensation fraction ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Condensation fraction: forsterite ---
    HEH    = 1.0E-01                    ; abundance of He
    SMASS  = !con.MProt*(1.0 + HEH*4.0)
    EPSMG  = 3.800E-05                  ; solar
    ; IDUST  = -2                       ; Mg in growth rate
    HELPMG = EPSMG / SMASS
    ;
    out=box.z.quc001/((HELPMG/2.0)*box.z.rho)
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'condfrac002') then begin
  ; --- Condensation fraction ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Condensation fraction: corundum ---
    HEH    = 1.0E-01                     ; abundance of He
    SMASS  = !con.MProt*(1.0 + HEH*4.0)
    EPSAL  = 2.950E-06
    HELPAL = EPSAL / SMASS
    ;
    out=box.z.quc002/((HELPAL/2.0)*box.z.rho)
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'condfrac002m001') then begin
  ; --- Condensation fraction ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Condensation fraction: forsterite ---
    HEH    = 1.0E-01                    ; abundance of He
    SMASS  = !con.MProt*(1.0 + HEH*4.0)
    EPSMG  = 3.800E-05                  ; solar
    ; IDUST  = -2                       ; Mg in growth rate
    HELPMG = EPSMG / SMASS
    ;
    f=1.0-box.z.quc001/((HELPMG/2.0)*box.z.rho)
    ;
    ; --- Condensation fraction: corundum ---
    HEH    = 1.0E-01                     ; abundance of He
    SMASS  = !con.MProt*(1.0 + HEH*4.0)
    EPSAL  = 2.950E-06
    HELPAL = EPSAL / SMASS
    ;
    out=f*box.z.quc002/((HELPAL/2.0)*box.z.rho)
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'qucsat') then begin
  ; --- Saturation density ---
  ;
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    psatc1, box.eos.T, Psat
    out=(par.C_dust03*par.C_dust02*!con.muatom)/(!con.kBoltz*box.eos.T)*Psat
  endif else begin
    message, 'quantity not defined for selected dustscheme'
  endelse
  ;
endif else if (cm eq 'sat') then begin
  ; --- Saturation ---
  ;
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    psatc1, box.eos.T, Psat
    out=box.z.quc001/((par.C_dust03*par.C_dust02*!con.muatom)/(!con.kBoltz*box.eos.T)*Psat)
  endif else begin
    message, 'quantity not defined for selected dustscheme'
  endelse
  ;
endif else if (cm eq 'qucall') then begin
  ; --- Additional tracers: add all ---
  ;
  tags=tag_names(box.z)
  indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
  out=0.0*box.z.rho
  for iquc=0,nquc-1 do begin
    dummy=execute('out=out+box.z.' + tags[indquctag[iquc]])
  endfor
  ;
endif else if (cm eq 'qucallorho') then begin
  ; --- Additional tracers: add all, divide by rho ---
  ;
  tags=tag_names(box.z)
  indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
  out=0.0*box.z.rho
  for iquc=0,nquc-1 do begin
    dummy=execute('out=out+box.z.' + tags[indquctag[iquc]])
  endfor
  out=out/box.z.rho
  ;
endif else if (cm eq 'qucallm1') then begin
  ; --- Additional tracers: add all but 'QUC001' ---
  ;
  tags=tag_names(box.z)
  indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
  out=0.0*box.z.rho
  for iquc=0,nquc-1 do begin
    if (tags[indquctag[iquc]] ne 'QUC001') then begin
      dummy=execute('out=out+box.z.' + tags[indquctag[iquc]])
    endif
  endfor
  ;
endif else if (cm eq 'qucallm1orho') then begin
  ; --- Additional tracers: add all but 'QUC001', divide by rho ---
  ;
  tags=tag_names(box.z)
  indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
  out=0.0*box.z.rho
  for iquc=0,nquc-1 do begin
    if (tags[indquctag[iquc]] ne 'QUC001') then begin
      dummy=execute('out=out+box.z.' + tags[indquctag[iquc]])
    endif
  endfor
  out=out/box.z.rho
  ;
endif else if (cm eq 'ratenucl') then begin
  ; --- Nucleation rate ---
  ;
  if (par.dustscheme eq 'dust_bins_01') then begin
    dust_bins_rossow78, par, box.eos.P, box.eos.T, box.z.rho, box.z.quc002, box.z.quc001, 1.0E-04, $
      rovs=rovs, rnucl=out
    out=out*(box.z.quc001-rovs)
  endif else begin
    message, 'Quantity not supported for given dustscheme.'
  endelse
  ;
endif else if (cm eq 'rcritnucl') then begin
  ; --- Critical nucleation radius ---
  ;
  if (par.dustscheme eq 'dust_bins_01') then begin
    dust_bins_rossow78, par, box.eos.P, box.eos.T, box.z.rho, box.z.quc002, box.z.quc001, 1.0E-04, $
      rcrit=out
  endif else begin
    message, 'Quantity not supported for given dustscheme.'
  endelse
  ;
endif else if (cm eq 'radius') then begin
  ; --- Radius of cell centers (distance from xc1=xc2=xc3=0) ---
  ;
  xc1a=rebin([box.z.xc1^2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2^2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3^2], n1-m1+1, n2-m2+1, n3-m3+1)
  out =sqrt(xc1a + xc2a + xc3a)
  ;
endif else if (cm eq 'radiuscyl3') then begin
  ; --- Cylindrical radius of cell centers (distance from xc1=xc2=0) ---
  ;
  xc1a=rebin([box.z.xc1^2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2^2], n1-m1+1, n2-m2+1, n3-m3+1)
  out =sqrt(xc1a + xc2a)
  ;
endif else if (cm eq 'radiusnorm') then begin
  ; --- Radius of cell centers (distance from xc1=xc2=xc3=0) ---
  ;
  xc1a=rebin([box.z.xc1^2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2^2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3^2], n1-m1+1, n2-m2+1, n3-m3+1)
  out =sqrt(xc1a + xc2a + xc3a)
  out=out/max(out)
  ;
endif else if (cm eq 'radiuscube') then begin
  ; --- Maximum coordinate values of cell centers ---
  ;
  xc1a=rebin(abs([box.z.xc1]), n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin(abs([box.z.xc2]), n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin(abs([box.z.xc3]), n1-m1+1, n2-m2+1, n3-m3+1)
  out =xc1a > xc2a > xc3a
  ;
endif else if (cm eq 'radiuscuber') then begin
  ; --- Maximum coordinate values of cell centers, randomized ---
  ;
  xc1a=rebin(abs([box.z.xc1]), n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin(abs([box.z.xc2]), n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin(abs([box.z.xc3]), n1-m1+1, n2-m2+1, n3-m3+1)
  out =xc1a > xc2a > xc3a
  dxb3=reform(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3])
  for i3=m3,n3 do begin
    out[*,*,i3]=out[*,*,i3] + dxb3[i3]*(randomu(seed, n1-m1+1, n2-m2+1)-0.5)
  endfor ; i3
  ;
endif else if (cm eq 'radiuscubenorm') then begin
  ; --- Normalized maximum coordinate values of cell centers ---
  ;
  xc1a=rebin(abs([box.z.xc1]), n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin(abs([box.z.xc2]), n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin(abs([box.z.xc3]), n1-m1+1, n2-m2+1, n3-m3+1)
  out =xc1a > xc2a > xc3a
  out=out/max(out)
  ;
;endif else if (cm eq 'radiusb') then begin
;  ; --- Radius of cell corners (distance from xc1=xc2=xc3=0) ---
;  ;
;  xb1a=rebin(box.z.xb1^2, n1-m1+2, n2-m2+2, n3-m3+2)
;  xb2a=rebin(box.z.xb2^2, n1-m1+2, n2-m2+2, n3-m3+2)
;  xb3a=rebin(box.z.xb3^2, n1-m1+2, n2-m2+2, n3-m3+2)
;  out =sqrt(xb1a + xb2a + xb3a)
  ;
endif else if (cm eq 'randomu') then begin
  ; --- Box of model size with random numbers
  ;
  out=randomu(seed, n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else if (cm eq 'rho') then begin
  ; --- Mass density ---
  ;
  out=box.z.rho
  ;
endif else if (cm eq 'rhoei') then begin
  ; --- Internal energy per volume ---
  ;
  out=box.z.rho * box.z.ei
  ;
endif else if (cm eq 'rhoeip') then begin
  ; --- Enthalpy per volume ---
  ;
  out=box.z.rho*box.z.ei + box.eos.P
  ;
endif else if (cm eq 'rhoeik') then begin
  ; --- Internal + kinetic energy per volume ---
  ;
  out=box.z.rho * (box.z.ei + 0.5*(box.z.v1^2 + box.z.v2^2 + box.z.v3^2))
  ;
endif else if (cm eq 'rhoeikg') then begin
  ; --- Internal + kinetic +potential (gravity) energy per volume ---
  ;
  if (par.grav_mode eq 'central') then begin
    eg=spherepot(combox('radius', box), r0=par.r0_grav, r1=par.r1_grav, C=!con.G*par.mass_star)
  endif else begin
    message, 'Potential not computed'
  endelse
  out=box.z.rho * (box.z.ei + 0.5*(box.z.v1^2 + box.z.v2^2 + box.z.v3^2) + eg)
  ;
endif else if (cm eq 'rhoek') then begin
  ; --- Kinetic energy per volume ---
  ;
  out=box.z.rho * 0.5*(box.z.v1^2 + box.z.v2^2 + box.z.v3^2)
  ;
endif else if (cm eq 'rhos') then begin
  ; --- Entropy per volume ---
  ;
  out=box.z.rho * box.eos.s
  ;
endif else if (cm eq 'rhovv_radial') then begin
  ; --- Radial turbulent pressure ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out= float(box.z.rho * (1.0D+00*box.z.v1*xc1a + box.z.v2*xc2a + box.z.v3*xc3a)^2 / $
                         (eps^2 + xc1a^2 + xc2a^2 + xc3a^2))
  ;
endif else if (cm eq 'rhov1') then begin
  ; --- Momentum in 1 direction ---
  ;
  out=box.z.rho * box.z.v1
  ;
endif else if (cm eq 'rhov2') then begin
  ; --- Momentum in 2 direction ---
  ;
  out=box.z.rho * box.z.v2
  ;
endif else if (cm eq 'rhov3') then begin
  ; --- Momentum in 3 direction ---
  ;
  out=box.z.rho * box.z.v3
  ;
endif else if (cm eq 'rhov1v1') then begin
  ; --- Momentum flux in 1 direction ---
  ;
  out=box.z.rho * box.z.v1^2
  ;
endif else if (cm eq 'rhov2v2') then begin
  ; --- Momentum flux in 2 direction ---
  ;
  out=box.z.rho * box.z.v2^2
  ;
endif else if (cm eq 'rhov3v3') then begin
  ; --- Momentum flux in 3 direction ---
  ;
  out=box.z.rho * box.z.v3^2
  ;
endif else if (cm eq 'rhov_radial') then begin
  ; --- Radial mass flux ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out=box.z.rho*(box.z.v1*xc1a + box.z.v2*xc2a + box.z.v3*xc3a) / $
                 sqrt(eps^2 + xc1a^2 + xc2a^2 + xc3a^2)
  ;
endif else if (cm eq 'rhox1') then begin
  ; --- rho * x1 ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho * xc1a
  ;
endif else if (cm eq 'rhox2') then begin
  ; --- rho * x2 ---
  ;
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho * xc2a
  ;
endif else if (cm eq 'rhox3') then begin
  ; --- rho * x3 ---
  ;
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  out=box.z.rho * xc3a
  ;
endif else if (cm eq 's') then begin
  ; --- Entropy per mass unit ---
  ;
  out=box.eos.s
  ;
endif else if (cm eq 't') then begin
  ; --- Temperature ---
  ;
  out=box.eos.T
  ;
endif else if (cm eq 'tauharm') then begin
  ; --- Estimate for "absolute" optical depth for spherical models: harmonic sum ---
  ;
  if (n_elements(box.z.xc1) eq 1) then begin
    ; --- 2D ---
    out=1.0/(1.0/combox('tau2l', box) + 1.0/combox('tau2r', box) + $
             1.0/combox('tau3l', box) + 1.0/combox('tau3r', box)  )
  endif else if (n_elements(box.z.xc2) eq 1) then begin
    ; --- 2D ---
    out=1.0/(1.0/combox('tau1l', box) + 1.0/combox('tau1r', box) + $
             1.0/combox('tau3l', box) + 1.0/combox('tau3r', box)  )
  endif else if (n_elements(box.z.xc3) eq 1) then begin
    ; --- 2D ---
    out=1.0/(1.0/combox('tau1l', box) + 1.0/combox('tau1r', box) + $
             1.0/combox('tau2l', box) + 1.0/combox('tau2r', box)  )
  endif else begin
    ; --- 3D ---
    out=1.0/(1.0/combox('tau1l', box) + 1.0/combox('tau1r', box) + $
             1.0/combox('tau2l', box) + 1.0/combox('tau2r', box) + $
             1.0/combox('tau3l', box) + 1.0/combox('tau3r', box)  )
  endelse
  ;
endif else if (cm eq 'tau1lo') then begin
  ; --- Optical depth, 1 direction, left ---
  ;
  dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau1=box.z.rho * box.opa.kappa * dxb1a
  out=dtau1
  for i1=m1+1,n1 do begin
    out[i1,*,*]=out[i1-1,*,*] + 0.5*(dtau1[i1-1,*,*]+dtau1[i1  ,*,*])
  endfor
  ;
endif else if (cm eq 'tau1l') then begin
  ; --- Optical depth, 1 direction, left ---
  ;
  dxb1a=rebin(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*),n1-m1+1, n2-m2+1, n3-m3+1)
  dtau1=box.z.rho * box.opa.kappa * dxb1a
  dtau1(m1+1:n1,*,*)=0.5*(dtau1(m1:n1-1,*,*)+dtau1(m1+1:n1,*,*))
  out=total(dtau1, 1, /cumulative)
  ;
endif else if (cm eq 'tau1r') then begin
  ; --- Optical depth, 1 direction, right ---
  ;
  dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau1=box.z.rho * box.opa.kappa * dxb1a
  out=dtau1
  for i1=n1-1,m1,-1 do begin
    out[i1,*,*]=out[i1+1,*,*] + 0.5*(dtau1[i1+1,*,*]+dtau1[i1  ,*,*])
  endfor
  ;
endif else if (cm eq 'tau1rn') then begin
  ; --- Optical depth, 1 direction, right ---
  ;
  dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau1=box.z.rho * box.opa.kappa * dxb1a
  dtau1[m1:n1-1,*,*]=0.5*(dtau1[m1+1:n1,*,*]+dtau1[m1:n1-1,*,*])
  dtau1=reverse(dtau1, 1)
  out=total(dtau1, 1, /cumulative)
  out=reverse(out, 1)
  ;
endif else if (cm eq 'tau2l') then begin
  ; --- Optical depth, 2 direction, left ---
  ;
  dxb2a=rebin(box.z.xb2[*,m2+1:n2+1,*]-box.z.xb2[*,m2:n2,*],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau2=box.z.rho * box.opa.kappa * dxb2a
  out=dtau2
  for i2=m2+1,n2 do begin
    out[*,i2,*]=out[*,i2-1,*] + 0.5*(dtau2[*,i2-1,*]+dtau2[*,i2  ,*])
  endfor
  ;
endif else if (cm eq 'tau2ln') then begin
  ; --- Optical depth, 2 direction, left ---
  ;
  dxb2a=rebin(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*),n1-m1+1, n2-m2+1, n3-m3+1)
  dtau2=box.z.rho * box.opa.kappa * dxb2a
  dtau2[*,m2+1:n2,*]=0.5*(dtau2[*,m2:n2-1,*]+dtau2[*,m2+1:n2,*])
  out=total(dtau2, 2, /cumulative)
  ;
endif else if (cm eq 'tau2r') then begin
  ; --- Optical depth, 3 direction, right ---
  ;
  dxb2a=rebin(box.z.xb2[*,m2+1:n2+1,*]-box.z.xb2[*,m2:n2,*],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau2=box.z.rho * box.opa.kappa * dxb2a
  out=dtau2
  for i2=n2-1,m2,-1 do begin
    out[*,i2,*]=out[*,i2+1,*] + 0.5*(dtau2[*,i2+1,*]+dtau2[*,i2  ,*])
  endfor
  ;
endif else if (cm eq 'tau3l') then begin
  ; --- Optical depth, 3 direction, left ---
  ;
  dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau3=box.z.rho * box.opa.kappa * dxb3a
  out=dtau3
  for i3=m3+1,n3 do begin
    out[*,*,i3]=out[*,*,i3-1] + 0.5*(dtau3[*,*,i3-1]+dtau3[*,*,i3  ])
  endfor
  ;
endif else if (cm eq 'tau3ln') then begin
  ; --- Optical depth, 3 direction, left ---
  ;
  dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau3=box.z.rho * box.opa.kappa * dxb3a
  dtau3[*,*,m3+1:n3]=0.5*(dtau3[*,*,m3:n3-1]+dtau3[*,*,m3+1:n3])
  out=total(dtau3, 3, /cumulative)
  ;
endif else if ((cm eq 'tau3r') or (cm eq 'tau')) then begin
  ; --- Optical depth, 3 direction, right ---
  ;
  dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  dtau3=box.z.rho * box.opa.kappa * dxb3a
  out=dtau3
  for i3=n3-1,m3,-1 do begin
    out[*,*,i3]=out[*,*,i3+1] + 0.5*(dtau3[*,*,i3+1]+dtau3[*,*,i3  ])
  endfor
  ;
endif else if ((cm eq 'tau_int') or (cm eq 'tauint') or (cm eq 'tauint_xdust')) then begin
  ; --- Optical depth, using TAU_INT.PRO by M.Steffen ---
  ;
  out=-1
  ; check if opacity is present in data structure
  opa_flag=0
  tags=tag_names(box) 
  for i=0,n_elements(tags)-1 do begin 
    if strlowcase(tags[i]) eq 'opa' then opa_flag=1
  endfor ; i
  ;
  ; check if c_radhtautop is present in structure PAR
  htau_flag=0
  if n_tags(par) gt 0 then begin 
    parTags=tag_names(par) 
    for i=0,n_elements(parTags)-1 do begin 
      if strlowcase(parTags[i]) eq 'c_radhtautop' then begin 
        c_radhtautop=par.c_radhtautop
        if (c_radhtautop le 0.0) then begin
          ; --- Invent a proxy for the scale height ---
          c_radhtautop=box.z.xb3[-1]-box.z.xb3[-2]
        endif
        htau_flag=1
      endif
    endfor ; i
  endif else begin 
    message, 'Structure PAR is not defined or does not contain valid data!'
  endelse
  ;
  if htau_flag then begin 
    ;
    if opa_flag then begin
      if (cm eq 'tauint_xdust') then begin
        ;
        if (total(parTags eq 'DUSTOPAMODE') gt 0.0) then begin
          DustOpaMode=par.DustOpaMode
        endif else begin
          DustOpaMode=''
        endelse
        ;
        if ((par.dustscheme eq 'dust_k3mon_03') and (DustOpaMode eq 'largeparticlelimit')) then begin
          kaprho=1.5/(par.C_dust01*par.ar_dustGrainRadius[1]) * $
                  ( box.z.quc002^2 * $
                   (box.z.quc001+box.z.quc002))^(1.0/3.0)
          ; print, 'dust: ', minmax(kaprho)
        endif else begin
          message,'This formula is probably not current: check or avoid!'
          ; kappa=box.opa.kappa + xdust(box.z.quc001, box.z.quc002, box.z.rho, par.c_dust02, par.c_dust05)
          ; kappa=box.opa.kappa + xdust(box.z.quc001, box.z.quc002, box.z.rho, par.c_dust02, par.c_dust05)
          ; kaprho=rebin(box.z.rho*kappa, n1-m1+1, n2-m2+1, n3-m3+1)
        endelse
        ;
        kaprho=kaprho + box.z.rho*box.opa.kappa
        ; print, 'gas+dust: ', minmax(kaprho)
        ;
      endif else begin
        kaprho=box.z.rho*box.opa.kappa
        ;
      endelse
      ;
      z=reform(box.z.xc3) 
      out=kaprho
;      if (n2 gt n2) then begin
        for i2=m2,n2 do begin
            out[*,i2,*]=tau_int(reform(kaprho[*,i2,*], n1-m1+1, n3-m3+1), z, n1-m1+1, n3-m3+1, $
                                c_radhtautop, mode=1)
        endfor
;      endif else begin
;        for i1=m1,n1 do begin
;            out[i1,*,*]=tau_int(reform(kaprho[i1,*,*]), z, n2-m2+1, n3-m3+1, $
;                                c_radhtautop, mode=1)
;        endfor
;      endelse
    endif else begin 
      message,'ERROR: Tagname OPA is not present in data structure!'
    endelse
  endif else begin 
    message,'Tag  c_radhtautop not present in structure PAR!'
  endelse
  ;
endif else if ((cm eq 'time_rad') or (cm eq 'time_rad1'))then begin
  ; --- Numerical radiative time scale: spatial scale from xb1 axis ---
  ; --- 2013-02-04 (B.F.): geom-factor: 1.5 -> 3/pi^2 ---
  ;
  dxb1a=rebin(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*],n1-m1+1, n2-m2+1, n3-m3+1)
  out  =box.z.rho * box.opa.kappa * dxb1a
  out  =1.0/(16.0*!con.sigmasb)/ $
        (box.eos.dTdei*box.opa.kappa*box.eos.T^3) * $
        (1.0+(3.0/!pi^2)*out^2)
  ;
endif else if (cm eq 'time_rad3') then begin
  ; --- Numerical radiative time scale: spatial scale from xb3 axis  ---
  ; --- 2013-02-04 (B.F.): geom-factor: 1.5 -> 3/pi^2 ---
  ;
  dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  out  =box.z.rho * box.opa.kappa * dxb3a
  out  =1.0/(16.0*!con.sigmasb)/ $
        (box.eos.dTdei*box.opa.kappa*box.eos.T^3) * $
        (1.0+(3.0/!pi^2)*out^2)
  ;
endif else if (cm eq 'time_rad3nongrey') then begin
  ; --- Numerical non-grey radiative time scale: spatial scale from xb3 axis  ---
  ;
;  ; --- Grey: ---
;  dtau =box.z.rho * dxb3a * box.opa.kappa
;  out  =dtau/(1.0+1.5*dtau^2)
;  out  =box.z.rho*dxb3a/(16.0*!con.sigmasb)/(box.eos.dTdei*box.eos.T^3*out)
  opta_info, nband=nband
  dxb3a=rebin(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3],n1-m1+1, n2-m2+1, n3-m3+1)
  out  =box.z.rho*0.0
  for iband=1,nband do begin
    dtau=box.z.rho * dxb3a * xkaros(box.eos.P, box.eos.T, iband)
    ;out =out + xbnu(box.eos.T, iband) * dtau/(1.0+1.5*dtau^2)
    out =out + xbnu(box.eos.T, iband) * dtau/(1.0+(3.0/!pi^2)*dtau^2)
  endfor ; iband
  out=box.z.rho*dxb3a/(16.0*!con.sigmasb)/(box.eos.dTdei*box.eos.T^3*out)
  ;
endif else if (cm eq 'v') then begin
  ; --- Amount of velocity ---
  ;
  out=sqrt(box.z.v1^2+box.z.v2^2+box.z.v3^2)
  ;
endif else if (cm eq 'v1') then begin
  ; --- Velocity in 1 direction ---
  ;
  out=box.z.v1
  ;
endif else if (cm eq 'v2') then begin
  ; --- Velocity in 2 direction ---
  ;
  out=box.z.v2
  ;
endif else if (cm eq 'v3') then begin
  ; --- Velocity in 3 direction ---
  ;
  out=box.z.v3
  ;
endif else if ((cm eq 'vradial') or (cm eq 'v_radial')) then begin
  ; --- Radial velocity ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out=(box.z.v1*xc1a + box.z.v2*xc2a + box.z.v3*xc3a) / $
       sqrt(eps^2 + xc1a^2 + xc2a^2 + xc3a^2)
  ;
endif else if (cm eq 'v_tangential') then begin
  ; --- Amount of tangential velocity ---
  ;
  xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  xc2a=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  xc3a=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  eps=0.01* (min(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*)) < $
             min(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*)) < $
             min(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3))  )
  out=(box.z.v1*xc1a + box.z.v2*xc2a + box.z.v3*xc3a) / $
       sqrt(eps^2 + xc1a^2 + xc2a^2 + xc3a^2)
  out=sqrt(((box.z.v1^2+box.z.v2^2+box.z.v3^2) - out^2) > 0.0)
  ;
endif else if (cm eq 'v_vec') then begin
  ; --- Velocity vector ---
  ;
  out=fltarr(n1-m1+1 ,n2-m2+1, n3-m3+1, 3)
  out(*,*,*,0)=box.z.v1
  out(*,*,*,1)=box.z.v2
  out(*,*,*,2)=box.z.v3
  ;
endif else if (cm eq 'xc1') then begin
  ; --- xc1 coordinate ---
  ;
  out=reform([box.z.xc1], n1-m1+1, 1, 1)
  ;
endif else if (cm eq 'xc2') then begin
  ; --- xc2 coordinate ---
  ;
  out=rebin([box.z.xc2], 1, n2-m2+1, 1)
  ;
endif else if (cm eq 'xc3') then begin
  ; --- xc3 coordinate ---
  ;
  out=rebin([box.z.xc3], 1, 1, n3-m3+1)
  ;
endif else if (cm eq 'xc1a') then begin
  ; --- xc1 coordinate as 3D array ---
  ;
  out=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else if (cm eq 'xc1ar') then begin
  ; --- xc1 coordinate as 3D array, randomized ---
  ;
  out=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
  dxb1=reform(box.z.xb1[m1+1:n1+1,*,*]-box.z.xb1[m1:n1,*,*])
  for i1=m1,n1 do begin
    out[i1,*,*]=out[i1,*,*] + dxb1[i1]*(randomu(seed, n2-m2+1, n3-m3+1)-0.5)
  endfor ; i1
  ;
endif else if (cm eq 'xc2a') then begin
  ; --- xc2 coordinate as 3D array ---
  ;
  out=rebin([box.z.xc2], n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else if (cm eq 'xc3a') then begin
  ; --- xc3 coordinate as 3D array ---
  ;
  out=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else if (cm eq 'xc3ar') then begin
  ; --- xc3 coordinate as 3D array, randomized ---
  ;
  out=rebin([box.z.xc3], n1-m1+1, n2-m2+1, n3-m3+1)
  dxb3=reform(box.z.xb3[*,*,m3+1:n3+1]-box.z.xb3[*,*,m3:n3])
  for i3=m3,n3 do begin
    out[*,*,i3]=out[*,*,i3] + dxb3[i3]*(randomu(seed, n1-m1+1, n2-m2+1)-0.5)
  endfor ; i3
  ;
endif else if (cm eq 'dxb1a') then begin
  ; --- delta xb1 step as 3D array ---
  ;
  out=rebin(box.z.xb1(m1+1:n1+1,*,*)-box.z.xb1(m1:n1,*,*),n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else if (cm eq 'dxb2a') then begin
  ; --- delta xb2 step as 3D array ---
  ;
  out=rebin(box.z.xb2(*,m2+1:n2+1,*)-box.z.xb2(*,m2:n2,*),n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else if (cm eq 'dxb3a') then begin
  ; --- delta xb3 step as 3D array ---
  ;
  out=rebin(box.z.xb3(*,*,m3+1:n3+1)-box.z.xb3(*,*,m3:n3),n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endif else begin
  ; --- Call derivbox ---
  out=derivbox(command, box)
  ;
  ; --- Default: zero ---
  ;message, 'Unknown command. Returning an empty array.', /informational
  ;out=box.z.rho*0.0
  ;
endelse
;
if (keyword_set(meansub)) then begin
  ; --- Subtract horizontal mean ---
    outmean=float(rebin(reform(avg(avg(double(out),0),0),1,1,n3+1),n1+1,n2+1,n3+1))
    if (keyword_set(norm)) then begin
      out=(out-outmean)/out
    endif else begin
      out=out-outmean
    endelse
endif
;
return, out
;
end ; combox
