function derivbox, command, box, qin=qin, ierr=ierr
;
;------------------------------------------------------------------------------
;+
; NAME:
;   derivbox ('derivative_box')
;
; PURPOSE:
;   Compute derivatives from independent variables
;
; CATEGORY:
;   Hydrodynamics, derivatives
;
; CALLING SEQUENCE:
;   out=derivbox(command, box)
;
; INPUT:
;   command:      ('command') string with requested quantity
;   box:          ('box') data structure with CO5BOLD data
;
; OUTPUT:
;   out:          ('output') 3D array
;
; KEYWORDS:
;   qin:          ('quantity_input') 3D array, input
;   ierr:         ('integer_error') integer with error indicator
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   ; --- Assume: modelfile is defined ---
;
;   ; --- Load data set ---
;   ful=uio_dataset_rd(modelfile)
;
;   ; --- Compute divergence ---
;   divv=derivbox('divv', ful)
;
; MODIFICATION HISTORY:
;   2002-06-10 (B.F. Uppsala)
;   2004-08-26 (B.F.) gradv13, gradv23, gradv33
;   2005-06-01 (B.F.) divv 2D
;   2006-05-31 (B.F.) absrotv for 2D case
;   2010-02-24 (B.F.) helv
;   2018-08-31 (B.F.) Add optional input parameter qin
;-
;------------------------------------------------------------------------------
;
; --- No error occured so far ---
ierr=0
;
m1=0
n1=n_elements(box.z.xc1)-1
m2=0
n2=n_elements(box.z.xc2)-1
m3=0
n3=n_elements(box.z.xc3)-1
;
cm=strlowcase(command)
;
xc1a=rebin([box.z.xc1], n1-m1+1, n2-m2+1, n3-m3+1)
;
if (cm eq 'divv') then begin
  ; --- Divergence: v ---
  ;
  if ((n2-m2) gt 0) then begin
    out=derivbox('gradv11', box) + derivbox('gradv22', box) + derivbox('gradv33', box)
  endif else begin
    out=derivbox('gradv11', box)                            + derivbox('gradv33', box)
  endelse
  ;
endif else if (cm eq 'divv13') then begin
  ; --- Divergence: v (only 1/3 plane) ---
  ;
  out=derivbox('gradv11', box) + derivbox('gradv33', box)
  ;
endif else if (cm eq 'gradv11') then begin
  ; --- Gradient: v1 direction 1 ---
  ;
  out=deriv(xc1a, box.z.v1)
  ;
endif else if (cm eq 'gradv21') then begin
  ; --- Gradient: v2 direction 1 ---
  ;
  out=deriv(xc1a, box.z.v2)
  ;
endif else if (cm eq 'gradv31') then begin
  ; --- Gradient: v3 direction 1 ---
  ;
  out=deriv(xc1a, box.z.v3)
  ;
endif else if (cm eq 'gradv12') then begin
  ; --- Gradient: v1 direction 2 ---
  ;
  out=box.z.rho
  xc2=reform(box.z.xc2)
  for i3=m3,n3 do begin
    for i1=m1,n1 do begin
      out[i1,*,i3]=deriv(xc2, reform(box.z.v1[i1,*,i3]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradv22') then begin
  ; --- Gradient: v2 direction 2 ---
  ;
  out=box.z.rho
  xc2=reform(box.z.xc2)
  for i3=m3,n3 do begin
    for i1=m1,n1 do begin
      out[i1,*,i3]=deriv(xc2, reform(box.z.v2[i1,*,i3]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradv32') then begin
  ; --- Gradient: v3 direction 2 ---
  ;
  out=box.z.rho
  xc2=reform(box.z.xc2)
  for i3=m3,n3 do begin
    for i1=m1,n1 do begin
      out[i1,*,i3]=deriv(xc2, reform(box.z.v3[i1,*,i3]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradv13') then begin
  ; --- Gradient: v1 direction 3 ---
  ;
  out=box.z.rho
  xc3=reform(box.z.xc3)
  for i2=m2,n2 do begin
    for i1=m1,n1 do begin
      out[i1,i2,*]=deriv(xc3, reform(box.z.v1[i1,i2,*]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradv23') then begin
  ; --- Gradient: v2 direction 3 ---
  ;
  out=box.z.rho
  xc3=reform(box.z.xc3)
  for i2=m2,n2 do begin
    for i1=m1,n1 do begin
      out[i1,i2,*]=deriv(xc3, reform(box.z.v2[i1,i2,*]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradv33') then begin
  ; --- Gradient: v3 direction 3 ---
  ;
  out=box.z.rho
  xc3=reform(box.z.xc3)
  for i2=m2,n2 do begin
    for i1=m1,n1 do begin
      out[i1,i2,*]=deriv(xc3, reform(box.z.v3[i1,i2,*]))
    endfor
  endfor
  ;
endif else if (cm eq 'rotv1') then begin
  ; --- Vorticity: v, component 1 ---
  ;
  out=derivbox('gradv32', box) - derivbox('gradv23', box)
  ;
endif else if (cm eq 'rotv2') then begin
  ; --- Vorticity: v, component 2 ---
  ;
  out=derivbox('gradv13', box) - derivbox('gradv31', box)
  ;
endif else if (cm eq 'rotv3') then begin
  ; --- Vorticity: v, component 3 ---
  ;
  out=derivbox('gradv21', box) - derivbox('gradv12', box)
  ;
endif else if (cm eq 'helv') then begin
  ; --- Helicity ---
  out=derivbox('rotv1',box)*box.z.v1 + $
      derivbox('rotv2',box)*box.z.v2 + $
      derivbox('rotv3',box)*box.z.v3
  ;
endif else if (cm eq 'absrotv') then begin
  ; --- Vorticity: v, amount ---
  ;
  if (n1 eq m1) then begin
    out=sqrt(derivbox('rotv1', box)^2)
  endif else if (n2 eq m2) then begin
    out=sqrt(derivbox('rotv2', box)^2)
  endif else if (n3 eq m3) then begin
    out=sqrt(derivbox('rotv3', box)^2)
  endif else begin
    ; --- 3D case ---
    out=sqrt(derivbox('rotv1', box)^2 + derivbox('rotv2', box)^2 + derivbox('rotv3', box)^2)
  endelse
  ;
  ;
endif else if (cm eq 'divrhov') then begin
  ; --- Divergence: v ---
  ;
  out=derivbox('gradrhov11', box) + derivbox('gradrhov22', box) + derivbox('gradrhov33', box)
  ;
endif else if (cm eq 'gradrhov11') then begin
  ; --- Gradient: rhov1 direction 1 ---
  ;
  out=deriv(xc1a, box.z.rho*box.z.v1)
  ;
endif else if (cm eq 'gradrhov22') then begin
  ; --- Gradient: rhov2 direction 2 ---
  ;
  out=box.z.rho
  xc2=reform(box.z.xc2)
  for i3=m3,n3 do begin
    for i1=m1,n1 do begin
      out[i1,*,i3]=deriv(xc2, reform(box.z.rho[i1,*,i3]*box.z.v2[i1,*,i3]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradrhov33') then begin
  ; --- Gradient: rhov3 direction 3 ---
  ;
  out=box.z.rho
  xc3=reform(box.z.xc3)
  for i2=m2,n2 do begin
    for i1=m1,n1 do begin
      out[i1,i2,*]=deriv(xc3, reform(box.z.rho[i1,i2,*]*box.z.v1[i1,i2,*]))
    endfor
  endfor
  ;


endif else if (cm eq 'gradq1') then begin
  ; --- Gradient: q direction 1 ---
  ;
  out=deriv(xc1a, qin)
  ;
endif else if (cm eq 'gradq2') then begin
  ; --- Gradient: q direction 2 ---
  ;
  out=qin
  xc2=reform(box.z.xc2)
  for i3=m3,n3 do begin
    for i1=m1,n1 do begin
      out[i1,*,i3]=deriv(xc2, reform(qin[i1,*,i3]))
    endfor
  endfor
  ;
endif else if (cm eq 'gradq3') then begin
  ; --- Gradient: q direction 3 ---
  ;
  out=qin
  xc3=reform(box.z.xc3)
  for i2=m2,n2 do begin
    for i1=m1,n1 do begin
      out[i1,i2,*]=deriv(xc3, reform(qin[i1,i2,*]))
    endfor
  endfor
  ;
endif else if (cm eq 'absgradq') then begin
  ; --- Gradient: q absolute value ---
  ;
  out=sqrt(derivbox('gradq1', box, qin=qin)^2 + derivbox('gradq2', box, qin=qin)^2 + derivbox('gradq3', box, qin=qin)^2)


endif else begin
  ; --- Default: zero ---
  ;
  out=box.z.rho*0.0
  ierr=1
  ;
endelse
;
return, out
;
end ; derivbox
