pro eosbox, model, eos=eos, opa=opa, $
  status_bar=status_bar, out_quantities=out_quantities0, n_max=n_max, ierror=ierror
;
;------------------------------------------------------------------------------
;+
; NAME:
;   eosbox ('equation_of_state_box')
;
; PURPOSE:
;   Solve EOS by bicubic interpolation for data in idl structure: rho, e -> P, T, ...
;
; CATEGORY:
;   EOS, Interpolation
;
; CALLING SEQUENCE:
;   eosbox, model, eos=eos, opa=opa, ierror=ierror
;
; INPUT/OUTPUT:
;   model:  ('model') IDL structure with HD data including
;     z.rho:    ('rho') density [g/cm^3]
;     z.e:      ('energy') internal energy per mass unit [erg/g]
;
; OUTPUT:
;   data are appended to IDL structure model:
;     eos.P:       ('Pressure') pressure [dyn/cm^2]
;     eos.dPdrho:  ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
;     eos.dPde:    ('delta_Pressure_delta_energy') dP/de|rho
;     eos.T:       ('Temperature') temperature [K]
;     eos.dTde:    ('delta_Temperature_delta_energy') dT/de|rho
;     eos.s:       ('s') entropy per mass unit [erg/g/K]
;
; KEYWORDS:
;   eos:           ('equation_of_state') input, data structure with EOS data, read in with
;                  tabinter_rdcoeff
;   opa:           ('opacities') input keyword, if set opacities are calculated with xkaros,
;                  the opacity tables have to be read in with dfopta
;   status_bar:    ('status_bar') integer flag to switch on status bar
;   out_quantities:('output_quantities') integer to specify output amount:
;                  0: all quantities (default, see above)
;                  1: P and T only (sufficient e.g. for computation of kappa)
;                  2: T only
;   n_max          ('number_maximum') integer specifying the maximum number of
;                  array elements for which the EOS is solved by a single call
;                  of eosinter
;   ierror:        ('index_error') integer, optional output, error number
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   eosinter: ('equation_of_state_interpolate')
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   14. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   17. 5.99 (B.F.) Sound velocity included
;   23. 3.00 (B.F.) Computation of opacities
;   30.11.00 (B.F. Uppsala) version for large 3D arrays
;   23.11.01 (B.F.) Formula for cs rearranged to account for low densities
;    8. 1.02 (B.F.) Formula for cs removed to save memory
;                   4D -> 3D
;    5. 3.02 (B.F.) Loop index last index in slices
;    6. 3.02 (B.F.) Keywords status_bar, out_quantities added
;    8. 7.03 (B.F.) n_max as keyword added
;    7. 4.05 (B.F.) out_quantities=2 possible
;-
;------------------------------------------------------------------------------
;
if (n_elements(n_max) eq 0) then n_max=400000
;
; --- Check keywords ---
if (n_elements(eos) eq 0) then begin
  message, 'No EOS table specified'
endif
if (n_elements(out_quantities0) eq 0) then begin
  out_quantities=0
endif else begin
  out_quantities=out_quantities0
endelse
;
n=n_elements(model.z.rho)
;
if (out_quantities eq 0) then begin
  ; --- Compute all quantities ---
  ;
  if (n le n_max) then begin
   ; --- Solve EOS in one step ---
    eosinter, model.z.rho, model.z.ei, P, dPdrho, dPdei, T, dTdei, s, $
              eos=eos, ierror=ierror
  endif else begin
    P=model.z.rho*0.0
    dPdrho=P
    dPdei =P
    T     =P
    dTdei =P
    s     =P
    if (keyword_set(status_bar)) then statusbar,'create',winID,0.
    n3=n_elements(model.z.rho[0,0,*])
    for i=0,n3-1 do begin
      if ((i mod 10) eq 0) then print, i
      eosinter, model.z.rho[*,*,i], model.z.ei[*,*,i], P0, dPdrho0, dPdei0, T0, dTdei0, s0, $
                eos=eos, ierror=ierror
      P[     *,*,i]=P0
      dPdrho[*,*,i]=dPdrho0
      dPdei[ *,*,i]=dPdei0
      T[     *,*,i]=T0
      dTdei[ *,*,i]=dTdei0
      s[     *,*,i]=s0
      if (keyword_set(status_bar)) then statusbar,'update',winID,1.*i/n3
    endfor
    if (keyword_set(status_bar)) then statusbar,'delete',winID,0.
  endelse
  ;
  ; --- Put data into convenient structure ---
  eosdata=create_struct( $
    'P'     , temporary(P)     , $
    'dPdrho', temporary(dPdrho), $
    'dPdei' , temporary(dPdei) , $
    'T'     , temporary(T)     , $
    'dTdei' , temporary(dTdei) , $
    's'     , temporary(s)      )
  ;
endif else if (out_quantities eq 1) then begin
  ; --- Compute P and T only ---
  ;
  if (n le n_max) then begin
   ; --- Solve EOS in one step ---
    eosinter, model.z.rho, model.z.ei, P, dPdrho, dPdei, T, dTdei, s, $
              eos=eos, out_quantities=out_quantities, ierror=ierror
  endif else begin
    P=model.z.rho*0.0
    T=P
    if (keyword_set(status_bar)) then statusbar,'create',winID,0.
    n3=n_elements(model.z.rho[0,0,*])
    for i=0,n3-1 do begin
      if ((i mod 10) eq 0) then print, i
      eosinter, model.z.rho[*,*,i], model.z.ei[*,*,i], P0, dPdrho0, dPdei0, T0, dTdei0, s0, $
                eos=eos, out_quantities=out_quantities, ierror=ierror
      P[     *,*,i]=P0
      T[     *,*,i]=T0
      if (keyword_set(status_bar)) then statusbar,'update',winID,1.*i/n3
    endfor
    if (keyword_set(status_bar)) then statusbar,'delete',winID,0.
  endelse
  ;
  ; --- Put data into convenient structure ---
  eosdata=create_struct( $
    'P'     , temporary(P)     , $
    'T'     , temporary(T)      )
  ;
endif else begin
  ; --- Compute T only ---
  ;
  if (n le n_max) then begin
   ; --- Solve EOS in one step ---
    eosinter, model.z.rho, model.z.ei, P, dPdrho, dPdei, T, dTdei, s, $
              eos=eos, out_quantities=out_quantities, ierror=ierror
  endif else begin
    T=model.z.rho*0.0
    if (keyword_set(status_bar)) then statusbar,'create',winID,0.
    n3=n_elements(model.z.rho[0,0,*])
    for i=0,n3-1 do begin
      if ((i mod 10) eq 0) then print, i
      eosinter, model.z.rho[*,*,i], model.z.ei[*,*,i], P0, dPdrho0, dPdei0, T0, dTdei0, s0, $
                eos=eos, out_quantities=out_quantities, ierror=ierror
      T[     *,*,i]=T0
      if (keyword_set(status_bar)) then statusbar,'update',winID,1.*i/n3
    endfor
    if (keyword_set(status_bar)) then statusbar,'delete',winID,0.
  endelse
  ;
  ; --- Put data into convenient structure ---
  eosdata=create_struct( $
    'T'     , temporary(T)      )
endelse
;
if (keyword_set(opa)) then begin
  kappa=xkaros(eosdata.P, eosdata.T, 0)
  opadata=create_struct( $
  'kappa'     , temporary(kappa))
  model=create_struct(temporary(model), 'eos', temporary(eosdata), 'opa', temporary(opadata))
endif else begin
  model=create_struct(temporary(model), 'eos', temporary(eosdata))
endelse
;
end ; eosbox
