pro eosinter, rho, e, P, dPdrho, dPde, T, dTde, s, $
              eos=eos, out_quantities=out_quantities0, $
              dsdrho=dsdrho, dsde=dsde, dTdrho=dTdrho, ierror=ierror
;
;------------------------------------------------------------------------------
;+
; NAME:
;   eosinter ('equation_of_state_interpolation')
;
; PURPOSE:
;   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
;
; CATEGORY:
;   EOS, Interpolation
;
; CALLING SEQUENCE:
;   eosinter, rho, e, P, dPdrho, dPde, T, dTde, s, $
;             eos=eos, out_quantities=out_quantities, ierror=ierror
;
; INPUT:
;   rho:    ('rho') density [g/cm^3]
;   e:      ('energy') internal energy per mass unit [erg/g]
;
; OUTPUT:
;   P:      ('Pressure') pressure [dyn/cm^2]
;   dPdrho: ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
;   dPde:   ('delta_Pressure_delta_energy') dP/de|rho
;   T:      ('Temperature') temperature [K]
;   dTde:   ('delta_Temperature_delta_energy') dT/de|rho
;   s:      ('s') entropy per mass unit [erg/g/K]
;
; KEYWORDS:
;   eos:    ('equation_of_state') data structure with EOS data, read in with
;           tabinter_rdcoeff
;   ierror: ('index_error') integer, optional output, error number
;   out_quantities: ('output_quantities') integer, specifying set of output quantities
;           0: P, dPdrho, dPde, T, dTde, s (default)
;           1: P, T
;           2: T
;           3: P, dPdrho, dPde, T, dTde, s + dsdrho, dsde, dTdrho
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   rho-e combination has to be within allowed range.
;   Otherwise extrapolation is performed with possibly strange results and
;   ierror is non-zero.
;
; PROCEDURE:
;
; EXAMPLE:
;   Gamma3=dPdei/rho + 1.0
;   Gamma1=dPdrho*rho/P + dPdei/rho
;   cs=sqrt(P*dPdei/rho**2 + dPdrho)
;   ; ---
;   rho=[0.002]& ei=[1.0e13]
;   eosinter, rho, ei,  P, dPdrho, dPde, T, dTde, s, EOS=EOS, out_q=3,
;   dsdrho=dsdrho, ierror=ierror
;   print, ierror, rho, ei, P, dPdrho, dPde, T, dTde, s, dsdrho, -P/(T*rho^2), (dsdrho+P/(T*rho^2))/dsdrho
;
; MODIFICATION HISTORY:
;   1999-04-26 (B.F. Copenhagen) Written
;   2001-10-23 (B.F. Uppsala) Reordering of dimensions of Ct: better performance
;                     New timing information, controlled by pa (printamount)
;                     Reordering of dimensions of eos.box.C
;   2002-02-06 (B.F.) Keyword out_quantities added
;   2005-04-07 (B.F.) out_quantities=2 possible
;   2010-12-17 (B.F. Lyon) () -> []
;   2014-09-05 (B.F. Uppsala) Optional output of dsdrho, dsde
;-
;------------------------------------------------------------------------------
;
; --- Check optional keyword and set default value if necessary ---
if (n_elements(out_quantities0) eq 0) then begin
  out_quantities=0
endif else begin
  out_quantities=out_quantities0
endelse
;
; --- Set print amount (0, 1, 2) ---
pa=0
;
st0=systime(1)
if (pa gt 1) then print, '0:        ', systime(1)-st0, format='(A,F10.5)'
;
if (n_elements(eos) eq 0) then begin
  message, 'No EOS table specified'
endif
;
n  =n_elements(rho)
dim=(size(rho))[1:(size(rho))[0]]
;
lnx1=reform(alog(rho                 ),n)
lnx2=reform(alog(e  +eos.box.x2_shift),n)
;print, 'minmax(lnx1), minmax(lnx2): ', minmax(lnx1), minmax(lnx2)
if (pa gt 1) then print, 'lnx1/2:   ', systime(1)-st0, format='(A,F10.5)'
;
; --- Determine indices of interpolation coefficients ---
i1=fix((lnx1-eos.box.lnx1off)*eos.box.lnx1fac)
i2=fix((lnx2-eos.box.lnx2off)*eos.box.lnx2fac)
;print, 'minmax(i1), minmax(i2): ', minmax(i1), minmax(i2)
;print, 'i1,i2' & print, i1 & print, i2
ierror=0L
dummy =where(i1 lt 0, count)
ierror=ierror+count
dummy =where(i1 gt eos.box.nx1, count)
ierror=ierror+count
dummy =where(i2 lt 0, count)
ierror=ierror+count
dummy =where(i2 gt eos.box.nx2, count)
ierror=ierror+count
i1=((i1 > 0) < (eos.box.nx1-1))
i2=((i2 > 0) < (eos.box.nx2-1))
if (pa gt 1) then print, 'i1/2:     ', systime(1)-st0, format='(A,F10.5)'
;
;help,i1,i2
;print, 'i1,i2' & print, i1 & print, i2
;print, 'minmax(i1), minmax(i2): ', minmax(i1), minmax(i2)

; --- Compute relative position in cell ---
x1t=lnx1-eos.box.lnx11d[i1]
x2t=lnx2-eos.box.lnx21d[i2]
;print, 'x1t,x2t' & print, x1t & print, x2t
;print, 'minmax(x1t), minmax(x2t): ', minmax(x1t), minmax(x2t)
if (pa gt 1) then print, 'x1/2t:    ', systime(1)-st0, format='(A,F10.5)'
;
ind=reform( $
    rebin(reform(eos.box.nc*(i1+eos.box.nx1*i2),n,1),n,eos.box.nc) + $
    rebin(reform(indgen(eos.box.nc),1,eos.box.nc),n,eos.box.nc), $
           n*eos.box.nc)

if (pa gt 1) then print, 'ind:      ', systime(1)-st0, format='(A,F10.5)'
;
if ((out_quantities eq 0) or (out_quantities eq 3)) then begin
  ; --- Compute all quantities ---
  Ct=reform(eos.box.c[ind],n,eos.box.nc)
  if (pa gt 1) then print, 'ct:       ', systime(1)-st0, format='(A,F10.5)'
  ;print, 'minmax(ct)', minmax(ct)
  if (pa gt 1) then print, 'tr(ct):   ', systime(1)-st0, format='(A,F10.5)'
  ;
  s=$
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  )))
  if (out_quantities eq 3) then begin
    dsdrho=1.0/rho* ( $
             Ct[*, 1] + x1t*(2.*Ct[*, 2] + x1t*3.*Ct[*, 3]) + $
       x2t*( Ct[*, 5] + x1t*(2.*Ct[*, 6] + x1t*3.*Ct[*, 7]) + $
       x2t*( Ct[*, 9] + x1t*(2.*Ct[*,10] + x1t*3.*Ct[*,11]) + $
       x2t*( Ct[*,13] + x1t*(2.*Ct[*,14] + x1t*3.*Ct[*,15])  ))) )
   dsde=1.0/(e+eos.box.x2_shift) * ( $
             Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
   2.0*x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
   1.5*x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  )) )
    dsdrho=reform(dsdrho       ,dim)
    dsde  =reform(dsdrho       ,dim)
  endif
  if (pa gt 1) then print, 's:        ', systime(1)-st0, format='(A,F10.5)'
  s       =reform(s       ,dim)
  if (pa gt 1) then print, 'rf(s):    ', systime(1)-st0, format='(A,F10.5)'
  ;
  ind=ind+eos.box.nc*eos.box.nx1*eos.box.nx2
  Ct=reform(eos.box.c[ind],n,eos.box.nc)
  P=exp( $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))) )
  dPdrho=P/rho* ( $
             Ct[*, 1] + x1t*(2.*Ct[*, 2] + x1t*3.*Ct[*, 3]) + $
       x2t*( Ct[*, 5] + x1t*(2.*Ct[*, 6] + x1t*3.*Ct[*, 7]) + $
       x2t*( Ct[*, 9] + x1t*(2.*Ct[*,10] + x1t*3.*Ct[*,11]) + $
       x2t*( Ct[*,13] + x1t*(2.*Ct[*,14] + x1t*3.*Ct[*,15])  ))) )
  dPde=P/(e+eos.box.x2_shift) * ( $
             Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
   2.0*x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
   1.5*x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  )) )
  if (pa gt 1) then print, 'P,dP:     ', systime(1)-st0, format='(A,F10.5)'
  P     =reform(P     ,dim)
  dPdrho=reform(dPdrho,dim)
  dPde  =reform(dPde  ,dim)
  if (pa gt 1) then print, 'rf(P,dP): ', systime(1)-st0, format='(A,F10.5)'
  ;
  ind=ind+eos.box.nc*eos.box.nx1*eos.box.nx2
  Ct=reform(eos.box.c[ind],n,eos.box.nc)
  T=exp( $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))) )
  if (out_quantities eq 3) then begin
    dTdrho=T/rho* ( $
             Ct[*, 1] + x1t*(2.*Ct[*, 2] + x1t*3.*Ct[*, 3]) + $
       x2t*( Ct[*, 5] + x1t*(2.*Ct[*, 6] + x1t*3.*Ct[*, 7]) + $
       x2t*( Ct[*, 9] + x1t*(2.*Ct[*,10] + x1t*3.*Ct[*,11]) + $
       x2t*( Ct[*,13] + x1t*(2.*Ct[*,14] + x1t*3.*Ct[*,15])  ))) )
    dTdrho=reform(dTdrho,dim)
  endif
  dTde=T/(e+eos.box.x2_shift) * ( $
             Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
   2.0*x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
   1.5*x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  )) )
  if (pa gt 1) then print, 'T,dT:     ', systime(1)-st0, format='(A,F10.5)'
  T   =reform(T   ,dim)
  dTde=reform(dTde,dim)
  if (pa gt 0) then print, 'rf(T,dT): ', systime(1)-st0, format='(A,F10.5)'
  ;
endif else if (out_quantities eq 1) then begin
  ; --- Compute P and T only ---
  ;
  ind=ind+eos.box.nc*eos.box.nx1*eos.box.nx2
  Ct=reform(eos.box.c[ind],n,eos.box.nc)
  P=exp( $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))) )
  if (pa gt 1) then print, 'P:        ', systime(1)-st0, format='(A,F10.5)'
  P     =reform(P     ,dim)
  if (pa gt 1) then print, 'rf(P):    ', systime(1)-st0, format='(A,F10.5)'
  ;
  ind=ind+eos.box.nc*eos.box.nx1*eos.box.nx2
  Ct=reform(eos.box.c[ind],n,eos.box.nc)
  T=exp( $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))) )
  if (pa gt 1) then print, 'T:        ', systime(1)-st0, format='(A,F10.5)'
  T   =reform(T   ,dim)
  if (pa gt 0) then print, 'rf(T):    ', systime(1)-st0, format='(A,F10.5)'
  ;
endif else begin
  ; --- Compute T only ---
  ;
  ind=ind+eos.box.nc*eos.box.nx1*eos.box.nx2*2L
  Ct=reform(eos.box.c(ind),n,eos.box.nc)
  T=exp( $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))) )
  if (pa gt 1) then print, 'T:        ', systime(1)-st0, format='(A,F10.5)'
  T   =reform(T   ,dim)
  if (pa gt 0) then print, 'rf(T):    ', systime(1)-st0, format='(A,F10.5)'
endelse
;
end ; eosinter
