pro eosslice, model, adm, eos=eos, opa=opa, ierror=ierror
;
;------------------------------------------------------------------------------
;+
; NAME:
;   eossubbox ('equation_of_state_box')
;
; PURPOSE:
;   Solve EOS by bicubic interpolation for data in idl structure: rho, e -> P, T, ...
;
; CATEGORY:
;   EOS, Interpolation
;
; CALLING SEQUENCE:
;   eosbox, model, eos=eos, opa=opa, ierror=ierror
;
; INPUT/OUTPUT:
;   model:  ('model') IDL structure with HD data including
;     z.rho:    ('rho') density [g/cm^3]
;     z.e:      ('energy') internal energy per mass unit [erg/g]
;   adm:     lower and upper indices for x,y,z : INTARR(3,2)
;
; OUTPUT:
;   data are appended to IDL structure model:
;     eos.P:      ('Pressure') pressure [dyn/cm^2]
;     eos.dPdrho: ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
;     eos.dPde:   ('delta_Pressure_delta_energy') dP/de|rho
;     eos.T:      ('Temperature') temperature [K]
;     eos.dTde:   ('delta_Temperature_delta_energy') dT/de|rho
;     eos.s:      ('s') entropy per mass unit [erg/g/K]
;
; KEYWORDS:
;   eos:    ('equation_of_state') input, data structure with EOS data, read in with
;           tabinter_rdcoeff
;   opa:    ('opacities') input keyword, if set opacities are calculated with xkaros,
;           the opacity tables have to be read in with dfopta
;   ierror: ('index_error') integer, optional output, error number
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   14. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   17. 5.99 (B.F.) Sound velocity included
;   23. 3.00 (B.F.) Computation of opacities
;   30.11.00 (B.F. Uppsala) version for large 3D arrays
;   08.01.01 S.W., Kiel: version for subarrays
;-
;------------------------------------------------------------------------------
;
n_max=250000
;




; --- Check if structure model.eos already exists ---
lexist=0
tagstr=TAG_NAMES(model)
for i=0,n_elements(tagstr)-1 do if STRUPCASE(tagstr(i)) eq 'EOS' then lexist=1




; --- Check keywords ---
if (n_elements(eos) eq 0) then begin
  message, 'No EOS table specified'
endif
;
n=n_elements(model.z.rho(adm(0,0):adm(0,1),adm(1,0):adm(1,1),adm(2,0):adm(2,1)))
print,'  n(EOS) = ',n
;
print,adm






; --- Check array dimensions --- 
if (adm(0,0) ne adm(0,1)) and (adm(1,0) ne adm(1,1)) and (adm(2,0) ne adm(2,1)) then begin &$
  print,'> WARNING: More than a 2D slice!!' &$
  help,adm &$
  print,adm&$
  print,'> Eosslice aborted.' &$
  stop &$
endif


P0     =FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)
dPdrho0=FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)
dPdei0 =FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)
T0     =FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)
dTdei0 =FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)
s0     =FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)
cs0    =FLTARR(ABS(adm(0,1)-adm(0,0))+1,ABS(adm(1,1)-adm(1,0))+1,ABS(adm(2,1)-adm(2,0))+1)

; --- Solve EOS in one step ---
print,'  Solve EOS in one step'
eosinter, model.z.rho(adm(0,0):adm(0,1),adm(1,0):adm(1,1),adm(2,0):adm(2,1)), $
          model.z.ei(adm(0,0):adm(0,1),adm(1,0):adm(1,1),adm(2,0):adm(2,1)), $
          P0, dPdrho0, dPdei0, T0, dTdei0, s0, $
          eos=eos, ierror=ierror

; sound velocity
cs0=sqrt(P0*dPdei0/model.z.rho(adm(0,0):adm(0,1),adm(1,0):adm(1,1),adm(2,0):adm(2,1))^2 + dPdrho0) 


; --- Define array dimensions
;if lexist eq 0 then begin 
  xs=n_elements(model.z.xc1(*)) 
  ys=n_elements(model.z.xc2(0,*)) 
  zs=n_elements(model.z.xc3(0,0,*)) 
  P     =FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
  dPdrho=FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
  dPdei =FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
  T     =FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
  dTdei =FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
  s     =FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
  cs    =FLTARR(MAX([xs,ys,zs]),MAX([xs,ys,zs]))
;endif

; --- 

if adm(0,0) eq adm(0,1) then begin
  P     (adm(1,0):adm(1,1),adm(2,0):adm(2,1))=P0
  dPdrho(adm(1,0):adm(1,1),adm(2,0):adm(2,1))=dPdrho0
  dPdei (adm(1,0):adm(1,1),adm(2,0):adm(2,1))=dPdei0
  T     (adm(1,0):adm(1,1),adm(2,0):adm(2,1))=T0
  dTdei (adm(1,0):adm(1,1),adm(2,0):adm(2,1))=dTdei0
  s     (adm(1,0):adm(1,1),adm(2,0):adm(2,1))=s0
  cs    (adm(1,0):adm(1,1),adm(2,0):adm(2,1))=cs0
endif 
if adm(1,0) eq adm(1,1) then begin
  P     (adm(0,0):adm(0,1),adm(2,0):adm(2,1))=P0
  dPdrho(adm(0,0):adm(0,1),adm(2,0):adm(2,1))=dPdrho0
  dPdei (adm(0,0):adm(0,1),adm(2,0):adm(2,1))=dPdei0
  T     (adm(0,0):adm(0,1),adm(2,0):adm(2,1))=T0
  dTdei (adm(0,0):adm(0,1),adm(2,0):adm(2,1))=dTdei0
  s     (adm(0,0):adm(0,1),adm(2,0):adm(2,1))=s0
  cs    (adm(0,0):adm(0,1),adm(2,0):adm(2,1))=cs0
endif 
if adm(2,0) eq adm(2,1) then begin
  P     (adm(0,0):adm(0,1),adm(1,0):adm(1,1))=P0
  dPdrho(adm(0,0):adm(0,1),adm(1,0):adm(1,1))=dPdrho0
  dPdei (adm(0,0):adm(0,1),adm(1,0):adm(1,1))=dPdei0
  T     (adm(0,0):adm(0,1),adm(1,0):adm(1,1))=T0
  dTdei (adm(0,0):adm(0,1),adm(1,0):adm(1,1))=dTdei0
  s     (adm(0,0):adm(0,1),adm(1,0):adm(1,1))=s0
  cs    (adm(0,0):adm(0,1),adm(1,0):adm(1,1))=cs0
endif 



; --- Prepare structure for /opa ---
if (keyword_set(opa)) then begin
  kappa=xkaros(P, T, 0)
  opadata=create_struct( $
  'kappa'     , kappa)
endif 




; --- Put data into convenient structure ---
eosdata=create_struct( $
    'P'     , P     , $
    'dPdrho', dPdrho, $
    'dPdei' , dPdei , $
    'T'     , T     , $
    'dTdei' , dTdei , $
    's'     , s     , $
    'cs'    , cs)

;help,eosdata,/structure

if lexist eq 0 then begin 
  print,'> New structure'
  model=create_struct(model, 'eos', eosdata)
;  if (keyword_set(opa)) then begin
;    model=create_struct(model, 'eos', eosdata, 'opa', opadata)
;  endif else begin
endif 
if lexist eq 1 then begin 
  print,'  Structure already exists.'
  model.eos.P    =P
  model.eos.dPdei=dPdei
  model.eos.T    =T
  model.eos.dTdei=dTdei 
  model.eos.s    =s
  model.eos.cs   =cs
endif


;    
;help,model.eos,/structure





;  if (keyword_set(opa)) then begin 
;     model=create_struct(model,'opa', opadata)
;  endif 

end ; eosbox
