pro gasinter, P, T, rho, xcp, xgam, cxt, h, s, csound, $
              EOS=EOS, ierror=ierror
;
;------------------------------------------------------------------------------
;+
; NAME:
;   gasinter ('gas_interpolation')
;
; PURPOSE:
;   Interpolate in EOS table: P,T -> ...
;
; CATEGORY:
;   EOS, Interpolation
;
; CALLING SEQUENCE:
;   gasinter, P, T, rho, xcp, xgam, cxt, h, s, csound, $
;             EOS=EOS, ierror=ierror
;
; INPUT:
;   P:      ('Pressure') pressure [dyn/cm^2]
;   T:      ('Temperature') temperature [K]
;
; OUTPUT:
;   rho:    ('rho') density [g/cm^3]
;   xcp:    ('x_cp') Cp' = d(S)/d(ln rho) | P [erg/g/K]
;   xgam:   ('x_gamma') 1. adiabatic exponent: xgam = Gamma1 = Cp'/Cv'
;                                                   = d(ln P)/d(ln rho) | s
;   cxt:    ('cxt') expansion coefficient: cxt = delta = alpha*T
;                                              =-d(ln rho)/d(ln T) | P
;   h:      ('h') enthalpy per mass unit [erg/g]
;   s:      ('s') entropy per mass unit [erg/g/K]
;   cs:     ('c_sound') adiabatic sound velocity [cm/s]
;
; KEYWORDS:
;   EOS:    ('Equation_Of_State') data structure with EOS data, read in with tabinter_rdcoeff.
;           A simplified version is:
;           sEOS={ $
;             head: { $
;               variables: 'lnP,lnT->lnrho' $
;               variables:     'lnP,lnT->lnrho', $
;               interpolation: 'bicubic_direct' $
;                   }, $
;             box: { $
;               nx1:    n_elements(P), $
;               nx2:    n_elements(T), $
;               lnx11d: alog(P), $
;               lnx21d: alog(T), $
;               Ct:     alog(transpose(rho)) $
;                  } $
;                }
;           In this case a simpler interpolation is used and only rho is computed.
;   ierror: ('index_error') integer, optional output, error number
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   P, T combination has to be within allowed range.
;   Otherwise extrapolation is performed with possibly strange results and
;   ierror is non-zero.
;
; PROCEDURE:
;        EOS.box.c( 0,0,i1,i2) + x1t*(EOS.box.c( 1,0,i1,i2) + x1t*(EOS.box.c( 2,0,i1,i2) + x1t*EOS.box.c( 3,0,i1,i2))) + $
;  x2t*( EOS.box.c( 4,0,i1,i2) + x1t*(EOS.box.c( 5,0,i1,i2) + x1t*(EOS.box.c( 6,0,i1,i2) + x1t*EOS.box.c( 7,0,i1,i2))) + $
;  x2t*( EOS.box.c( 8,0,i1,i2) + x1t*(EOS.box.c( 9,0,i1,i2) + x1t*(EOS.box.c(10,0,i1,i2) + x1t*EOS.box.c(11,0,i1,i2))) + $
;  x2t*( EOS.box.c(12,0,i1,i2) + x1t*(EOS.box.c(13,0,i1,i2) + x1t*(EOS.box.c(14,0,i1,i2) + x1t*EOS.box.c(15,0,i1,i2)))  ))) )
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-04-21 (B.F. Kiel) Written
;   1999-04-26 (B.F. Copenhagen) Corrections
;   2001-12-05 (B.F. Uppsala) Reordering of dimensions of Ct: better performance
;                           New timing information, controlled by pa (printamount)
;                           Reordering of dimensions of EOS.box.C
;   2014-06-03 (B.F.) Some modernization of expressions: ()->[],...
;   2014-06-04 (B.F.) Add branch for interpolation in simple EOS structure (P,T->rho)
;-
;------------------------------------------------------------------------------
;
; --- Set some compiler options ---
compile_opt defint32, strictarr
;
;st0=systime(1)
;
if (n_elements(EOS) eq 0) then begin
  message, 'No EOS table specified'
endif
;
if (EOS.head.interpolation ne 'bicubic_direct') then begin
  ; --- Interpolate in a standard EOS table ---
  ;
  n  =n_elements(P)
  dim=size(P, /DIMENSIONS)
  ;
  lnP=reform(alog(P), n)
  lnT=reform(alog(T), n)
  ;print, systime(1)-st0
  ;
  ; --- Determine indices of interpolation coefficients ---
  i1=fix((lnP-EOS.box.lnx1off)*EOS.box.lnx1fac)
  i2=fix((lnT-EOS.box.lnx2off)*EOS.box.lnx2fac)
  ;print, 'i1,i2' & print, i1 & print, i2
  ierror=0
  dummy =where(i1 lt 0, count)
  ierror=ierror+count
  dummy =where(i1 gt EOS.box.nx1, count)
  ierror=ierror+count
  dummy =where(i2 lt 0, count)
  ierror=ierror+count
  dummy =where(i2 gt EOS.box.nx2, count)
  ierror=ierror+count
  i1=((i1 > 0) < (EOS.box.nx1-1))
  i2=((i2 > 0) < (EOS.box.nx2-1))
  ;print, systime(1)-st0
  ;
  ;help,i1,i2
  ;print, 'i1,i2' & print, i1 & print, i2
  ;
  ; --- Compute relative position in cell ---
  x1t=lnP-EOS.box.lnx11d[i1]
  x2t=lnT-EOS.box.lnx21d[i2]
  ;print, 'x1t,x2t' & print, x1t & print, x2t
  ;print, systime(1)-st0
  ind=reform(rebin(reform(EOS.box.nc*(i1+EOS.box.nx1*i2), n, 1         ), n, EOS.box.nc) + $
             rebin(reform(indgen(EOS.box.nc)            , 1, EOS.box.nc), n, EOS.box.nc), $
             n*EOS.box.nc)
  ;print, systime(1)-st0
  ;help,ind0
  ;print, 'ind0' & print,ind0
  ;help,ind
  ;print, 'ind' & print,ind
  Ct =reform(EOS.box.c[ind], n, EOS.box.nc)
  ;help, ct
  ;print, 'ct[0,*]' & print,ct[0,*]
  ;c0=reform(EOS.box.c[0,0,*,*])
  ;print, 'c0' & print, c0[i1,i2]
  ;print, systime(1)-st0
  ;
  rho=exp( $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))) )
  dlnrhodlnP= $
             Ct[*, 1] + x1t*(2.*Ct[*, 2] + x1t*3.*Ct[*, 3]) + $
       x2t*( Ct[*, 5] + x1t*(2.*Ct[*, 6] + x1t*3.*Ct[*, 7]) + $
       x2t*( Ct[*, 9] + x1t*(2.*Ct[*,10] + x1t*3.*Ct[*,11]) + $
       x2t*( Ct[*,13] + x1t*(2.*Ct[*,14] + x1t*3.*Ct[*,15])  )))
  dlnrhodlnT= $
             Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
   2.0*x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
   1.5*x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))
  ;print, systime(1)-st0
  rho       =reform(rho       , dim)
  dlnrhodlnP=reform(dlnrhodlnP, dim)
  dlnrhodlnT=reform(dlnrhodlnT, dim)
  ;print, systime(1)-st0
  ;
  ind=ind+EOS.box.nc*EOS.box.nx1*EOS.box.nx2
  Ct =reform(EOS.box.c[ind], n, EOS.box.nc)
  h= $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  )))
  dhdlnT= $
             Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
   2.0*x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
   1.5*x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  ))
  ;print, systime(1)-st0
  h     =reform(h     , dim)
  dhdlnT=reform(dhdlnT, dim)
  ;print, systime(1)-st0
  ;
  ind=ind+EOS.box.nc*EOS.box.nx1*EOS.box.nx2
  Ct =reform(EOS.box.c[ind],n,EOS.box.nc)
  s= $
             Ct[*, 0] + x1t*(Ct[*, 1] + x1t*(Ct[*, 2] + x1t*Ct[*, 3])) + $
       x2t*( Ct[*, 4] + x1t*(Ct[*, 5] + x1t*(Ct[*, 6] + x1t*Ct[*, 7])) + $
       x2t*( Ct[*, 8] + x1t*(Ct[*, 9] + x1t*(Ct[*,10] + x1t*Ct[*,11])) + $
       x2t*( Ct[*,12] + x1t*(Ct[*,13] + x1t*(Ct[*,14] + x1t*Ct[*,15]))  )))
  ;print, systime(1)-st0
  s=reform(s, dim)
  ;print, systime(1)-st0
  ;
  ; --- Compute all desired quantities from tabulated values ---
  cxt     =-dlnrhodlnT
  xcp     = dhdlnT/(cxt*T)
  xgam_rho= dhdlnT/((rho*dhdlnT)*dlnrhodlnP-cxt*cxt*P)
  csound  = sqrt(xgam_rho*P)
  xgam    = xgam_rho*rho
  ;
endif else begin
  ; --- Interpolate in a simple EOS structure ---
  ;
  n  =n_elements(P)
  dim=size(P, /DIMENSIONS)
  ;
  lnP=reform(alog(P), n)
  lnT=reform(alog(T), n)
  ;print, systime(1)-st0
  ;
  lnx1off=EOS.box.lnx11d[0]
  lnx2off=EOS.box.lnx21d[0]
  lnx1fac=float(EOS.box.nx1-1)/(EOS.box.lnx11d[EOS.box.nx1-1]-lnx1off)
  lnx2fac=float(EOS.box.nx2-1)/(EOS.box.lnx21d[EOS.box.nx2-1]-lnx2off)
  ;
  ; --- Determine indices of interpolation coefficients ---
  ;i1=fix((lnP-EOS.box.lnx1off)*EOS.box.lnx1fac)
  ;i2=fix((lnT-EOS.box.lnx2off)*EOS.box.lnx2fac)
  ind1=(lnP-lnx1off)*lnx1fac
  ind2=(lnt-lnx2off)*lnx2fac
  ;print, 'i1,i2' & print, i1 & print, i2
  ierror=0
  ;dummy =where(i1 lt 0, count)
  ;ierror=ierror+count
  ;dummy =where(i1 gt EOS.box.nx1, count)
  ;ierror=ierror+count
  ;dummy =where(i2 lt 0, count)
  ;ierror=ierror+count
  ;dummy =where(i2 gt EOS.box.nx2, count)
  ;ierror=ierror+count
  ;i1=((i1 > 0) < (EOS.box.nx1-1))
  ;i2=((i2 > 0) < (EOS.box.nx2-1))
  ;print, systime(1)-st0
  ;
  ;help,i1,i2
  ;print, 'i1,i2' & print, i1 & print, i2
  ;
  rho=exp(interpolate(eos.box.Ct, ind1, ind2, CUBIC=-0.5))
  rho=reform(rho, dim)
  ;
endelse
;
end ; gasinter
