function interbox, q, x, $
  x_regular=x_regular, orientation=orientation0, boundary=boundary0, $
  out_x_regular=out_x_regular
;
;------------------------------------------------------------------------------
;+
; NAME:
;   interbox ('interpolate_box')
;
; PURPOSE:
;   Interpolate box data onto new grid (e.g. equidistant in log tau)
;
; CATEGORY:
;   RHD/Box handling
;
; CALLING SEQUENCE:
;   function interbox, q, x, $
;     x_regular=x_regular, orientation=orientation0, boundary=boundary0, $
;     out_x_regular=out_x_regular
;
; INPUT:
;   q:           ('quantity') 3D array with quantity to interpolate (e.g. temperature)
;   x:           ('x_axis') 3D array with old axis
;
; OUTPUT:
;
; KEYWORDS:
;   orientation:   ('orientation') integer with orientation:
;                    1: along dimension 1 (default)
;                    2: along dimension 2
;                    3: along dimension 3
;   boundary:      ('boundary_condition') integer (0, 1) to specify treatment of boundaries
;   out_x_regular: ('out_x-axis_regular') output: new regular x axis
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2001-08-21 Written by Bernd Freytag, Uppsala
;   2006-01-19 (B.F.) More comments
;   2017-01-17 (B.F.) Change brackets for vectors: () -> []
;-
;------------------------------------------------------------------------------
;
m1=0
n1=n_elements(q[*,0,0])-1
m2=0
n2=n_elements(q[0,*,0])-1
m3=0
n3=n_elements(q[0,0,*])-1
;
; --- Default values for keywords ---
if (keyword_set(x_regular)) then begin
  n_x_regular=n_elements(x_regular)
endif else begin
  n_x_regular=101
  x_regular=findgen(n_x_regular)*0.1-5.0
endelse
out_x_regular=x_regular
;
if (keyword_set(orientation0)) then begin
  orientation=orientation0
endif else begin
  orientation=1
endelse
;
if (keyword_set(boundary0)) then begin
  boundary=boundary0
endif else begin
  boundary=0
endelse
;
; --- Find reasonable step in x ---
if (n_x_regular eq 1) then begin
  if (x_regular(0) eq 0.0) then begin
    dx_regular=1.0
  endif else begin
    dx_regular=abs(x_regular)
  endelse
endif else begin
  dx_regular=abs(max(x_regular) - min(x_regular))/float(n_x_regular-1)
endelse
;
if (orientation eq 1) then begin
  ; --- Interpolation direction: 1 ---
  ;
  ; --- Extend x and q to prevent extrapolation ---
  x_norm=fltarr(n1-m1+3, n2-m2+1, n3-m3+1)
  x_norm[1:n1+1,*,*]=x
  if (x[0,0,0] lt x[n1,0,0]) then begin
    x_norm[0   ,*,*]=(min(x) < min(x_regular)) - dx_regular
    x_norm[n1+2,*,*]=(max(x) > max(x_regular)) + dx_regular
  endif else begin
    x_norm[0   ,*,*]=(max(x) > max(x_regular)) + dx_regular
    x_norm[n1+2,*,*]=(min(x) < min(x_regular)) - dx_regular
  endelse
  q_norm=fltarr(n1-m1+3, n2-m2+1, n3-m3+1)
  q_norm[1:n1+1,*,*]=q
  if (boundary eq 0) then begin
    q_norm[0     ,*,*]=q[0 ,*,*]
    q_norm[  n1+2,*,*]=q[n1,*,*]
  endif
  ;
  ; --- Create empty output array ---
  q_regular=fltarr(n_x_regular, n2-m2+1, n3-m3+1)
  ;
  ; --- Interpolate ---
  for i3=m3,n3 do begin
    for i2=m2,n2 do begin
      q_regular[*,i2,i3]=interpol(reform(q_norm[*,i2,i3]), reform(x_norm[*,i2,i3]), x_regular)
    endfor
  endfor
  ;
endif else if (orientation eq 2) then begin
  ; --- Interpolation direction: 2 ---
  ;
  ; --- Extend x and q to prevent extrapolation ---
  x_norm=fltarr(n1-m1+1, n2-m2+3, n3-m3+1)
  x_norm[*,1:n2+1,*]=x
  if (x(0,0,0) lt x(0,n2,0)) then begin
    x_norm[*,0   ,*]=(min(x) < min(x_regular)) - dx_regular
    x_norm[*,n2+2,*]=(max(x) > max(x_regular)) + dx_regular
  endif else begin
    x_norm[*,0   ,*]=(max(x) > max(x_regular)) + dx_regular
    x_norm[*,n2+2,*]=(min(x) < min(x_regular)) - dx_regular
  endelse
  q_norm=fltarr(n1-m1+1, n2-m2+3, n3-m3+1)
  q_norm[*,1:n2+1,*]=q
  if (boundary eq 0) then begin
    q_norm[*,0     ,*]=q[*,0 ,*]
    q_norm[*,  n2+2,*]=q[*,n2,*]
  endif
  ;
  ; --- Create empty output array ---
  q_regular=fltarr(n1-m1+1, n_x_regular, n3-m3+1)
  ;
  ; --- Interpolate ---
  for i3=m3,n3 do begin
    for i1=m1,n1 do begin
      q_regular[i1,*,i3]=interpol(reform(q_norm[i1,*,i3]), reform(x_norm[i1,*,i3]), x_regular)
    endfor
  endfor
  ;
endif else begin
  ; --- Interpolation direction: 3 ---
  ;
  ; --- Extend x and q to prevent extrapolation ---
  x_norm=fltarr(n1-m1+1, n2-m2+1, n3-m3+3)
  x_norm[*,*,1:n3+1]=x
  if (x[0,0,0] lt x[0,0,n3]) then begin
    x_norm[*,*,0   ]=(min(x) < min(x_regular)) - dx_regular
    x_norm[*,*,n3+2]=(max(x) > max(x_regular)) + dx_regular
  endif else begin
    x_norm[*,*,0   ]=(max(x) > max(x_regular)) + dx_regular
    x_norm[*,*,n3+2]=(min(x) < min(x_regular)) - dx_regular
  endelse
  q_norm=fltarr(n1-m1+1, n2-m2+1, n3-m3+3)
  q_norm[*,*,1:n3+1]=q
  if (boundary eq 0) then begin
    q_norm[*,*,0     ]=q[*,*,0 ]
    q_norm[*,*,  n3+2]=q[*,*,n3]
  endif
  ;
  ; --- Create empty output array ---
  q_regular=fltarr(n1-m1+1, n2-m2+1, n_x_regular)
  ;
  ; --- Interpolate ---
  for i2=m2,n2 do begin
    for i1=m1,n1 do begin
      q_regular[i1,i2,*]=interpol(reform(q_norm[i1,i2,*]), reform(x_norm[i1,i2,*]), x_regular)
    endfor
  endfor
  ;
endelse
;
return, q_regular
;
end ; interbox
