function meopta, Opa0, Opa1, Name=Name0, Author=Author0, Comment=Comment0
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   meopta ('merge_opacity_table')
;
; PURPOSE:
;   Merge two opacity tables.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   result=meopta()
;
; INPUT:
;   Opa0:         ('Opacities_0') Structure with input opacity data
;   Opa1:         ('Opacities_1') Structure with input opacity data
;
; OUTPUT:
;   result:       Structure with output opacity data
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   It is assumed that Opa0 is the low-pressure/low-temperature table with band data
;   and Opa1 is the grey high-pressure/high-temperature table.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2025-08-25 (B.F. Uppsala)
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

; --- Check optional keywords and set default values ---
if (keyword_set(Name0)) then begin
  Name=Name0
endif else begin
  Name='meopta_opacity_table'
endelse

if (keyword_set(Author0)) then begin
  Author=Author0
endif else begin
  Author='-'
endelse

if (keyword_set(Comment0)) then begin
  Comment=Comment0
endif else begin
  Comment=''
endelse


; === Generate the entries for the new opacity table ===============================================

; --- Construct date and date/time string ---
caldat, systime(/julian), mm,dd,yyyy,hh,mmin,ss
ss=round(ss)
yyyy=strmid('000'+strtrim(string(yyyy),2),3,4,/rev)
mm  =strmid('0'  +strtrim(string(mm  ),2),1,2,/rev)
dd  =strmid('0'  +strtrim(string(dd  ),2),1,2,/rev)
hh  =strmid('0'  +strtrim(string(hh  ),2),1,2,/rev)
mmin=strmid('0'  +strtrim(string(mmin),2),1,2,/rev)
ss  =strmid('0'  +strtrim(string(ss  ),2),1,2,/rev)
DateString      =                   yyyy + '-' + mm   + '-' + dd
DateTimeString  =DateString + ' ' + hh   + ':' + mmin + ':' + ss
;DateShString    =                   yyyy       + mm         + dd
;DateTimeShString=DateShString     + hh         + mmin       + ss

; --- Construct the merged temperature vector ---
logTmin=min(Opa0.TabT) < min(Opa1.TabT)
logTmax=max(Opa0.TabT) > max(Opa1.TabT)
help, logTmin, logTmax
nTtoAdd=round(((logTmax-Opa0.TabT[-1]) > 0.0) / Opa0.TabdTB[-1])
help, nTtoAdd
nTOut  =Opa0.nT + nTtoAdd
TabTOut=fltarr(nTOut)
TabTOut[0:Opa0.nT-1]=Opa0.TabT
if (nTtoAdd gt 0) then begin
  TabTOut[Opa0.nT:*]=Opa0.TabT[-1] + (1.0+findgen(nTtoAdd)) * Opa0.TabdTB[-1]
endif
TabTBnOut    =Opa0.TabTBn
TabTBnOut[-1]=logTmax
IdxTBnOut    =Opa0.IdxTBn
IdxTBnOut[-1]=nTOut
TabdTBout    =Opa0.TabdTB

; --- Construct the merged pressure vector ---
logPmin=min(Opa0.TabP) < min(Opa1.TabP)
logPmax=max(Opa0.TabP) > max(Opa1.TabP)
help, logPmin, logPmax
nPtoAdd=round(((logPmax-Opa0.TabP[-1]) > 0.0) / Opa0.TabdPB[-1])
help, nPtoAdd
nPOut  =Opa0.nP + nPtoAdd
TabPOut=fltarr(nPOut)
TabPOut[0:Opa0.nP-1]=Opa0.TabP
if (nPtoAdd gt 0) then begin
  TabPOut[Opa0.nP:*]=Opa0.TabP[-1] + (1.0+findgen(nPtoAdd)) * Opa0.TabdPB[-1]
endif
TabPBnOut    =Opa0.TabPBn
TabPBnOut[-1]=logPmax
IdxPBnOut    =Opa0.IdxPBn
IdxPBnOut[-1]=nPOut
TabdPBout    =Opa0.TabdPB

; --- Construct the merged opacity table ---
nBand=Opa0.nBand
; --- Create empty table ---
xOpTaOut=fltarr(nTOut, nPOut, nBand+1)
; --- Copy low-temperature table ---
xOptaOut[0:Opa0.nT-1, 0:Opa0.nP-1, *]=Opa0.xOpTa
if (nPtoAdd gt 0) then begin
  ; --- Extrapolate the low-temperature table to high pressures at low temperatures ---
  xOptaOut[0:Opa0.nT-1, Opa0.nP:*  , *]=rebin(Opa0.xOpTa[*, Opa0.nP-1, *], Opa0.nT, nPtoAdd, nBand+1)
endif
if (nTtoAdd gt 0) then begin
  ; --- Extrapolate the low-temperature table to high temperatures at low pressures ---
  xOptaOut[Opa0.nT:*  , 0:Opa0.nP-1, *]=rebin(Opa0.xOpTa[Opa0.nT-1, *, *], nTtoAdd, Opa0.nP, nBand+1)
endif
if (nTtoAdd gt 0) then begin
  ; --- Find a temperature-index range with boundaries just outside the overlap, not larger than 10 points ---
  iTHigh=Opa0.nT
  iTLow =floor((interpol(findgen(nTout), TabTout, Opa1.TabT[0:0]))[0]-0.0001) > (iTHigh - 9)
  Tlow  =TabTOut[iTLow ]
  THigh =TabTOut[iTHigh]
  help, iTLow, iTHigh, TLow, THigh
  ;
  ; --- Construct a weight mask to get a smooth transition in temperature ---
  xT=0.0 > ((TabTOut-TLow)/(THigh-TLow)) < 1.0
  Mask=rebin(reform(3.0*xT^2 - 2.0*xT^3, nTOut, 1, 1), nTOut, nPOut, nBand+1)
  ;
  ; --- Interpolate the grey high-temperature opacities onto the output grid ---
  ;logkappaGrey=alog10(xkaros(10.0^TabPOut, 10.0^TabTOut, 0, Opa=Opa1, /grid))
  logkappaGrey=alog10(xkaros(10.0^TabPOut, 10.0^TabTOut, 0, Opa=Opa1, /grid, /spline))
  ;
  ; --- Blend ---
  xOptaOut=(1.0-Mask)*xOptaOut + Mask*rebin(reform(logkappaGrey, nTout, nPout, 1), nTout, nPout, nBand+1)
endif

; --- Construct the merged source-function weights ---
WTbbdOut=fltarr(nTOut, nBand+1)
WTbbdOut[0:Opa0.nT-1, *]=Opa0.WTbbd
if (nTtoAdd gt 0) then begin
  WTbbdOut[Opa0.nT:*, *]=rebin(Opa0.WTbbd[-1, *], nTtoAdd, nBand+1)
endif


; === Create the new opta structure ================================================================

; --- Put together the opacity structure (without header, so far) ---
Opa={ $
  IdVers: 5, $
  OpTaNa: Name, $
  DateX:  DateTimeString, $
  IdMean: Opa0.IdMean, $
  nT:     nTOut, $
  nTBn:   n_elements(IdxTBnOut), $
  nP:     nPOut, $
  nPBn:   n_elements(IdxPBnOut), $
  nBand:  Opa0.nBand, $
  mStep:  Opa0.mStep, $  
  nuLo:   Opa0.nuLo, $
  nuHi:   Opa0.nuHi, $
  numnu:  Opa0.numnu, $
  Intmdx: Opa0.Intmdx, $
  ;
  TabT:   TabTOut, $
  TabTBn: TabTBnOut, $
  IdxTBn: IdxTBnOut, $
  TabdTB: TabdTBOut, $
  ;
  TabP:   TabPOut, $
  TabPBn: TabPBnOut, $
  IdxPBn: IdxPBnOut, $
  TabdPB: TabdPBOut, $
  ;
  xOpTa:  xOptaOut, $
  ;
  WTbbd:  WTbbdOut, $
  ;
  InBand: Opa0.InBand $
    }

; --- Assemble the header ---
Header=[ $
;  '************************************************************************', $
  '*', $
  '*     Opacity table name: ' + Opa.OpTaNa + '.opta', $
  '*     Created           : ' + Opa.DateX, $ 
  '*     Contents          : grey Rosseland opacities', $
  '*                            + band means', $
  '*     Merger: ' + Opa0.OptaNa + ' + ' + Opa1.OptaNa, $
  '*', $
  '*     Opacity source data', $
  '*     -------------------', $
  '*     IFOP        = ' + 'merged opacity tables', $
  '*     Code        : ' + 'meopta.pro', $
  '*     Author      : ' + Author, $
  '*', $
  '*     Band definition parameters (cgs units)', $
  '*     -------------------------------------', $
  '*     nband       = ' + string(Opa0.nBand           , format='(I3)'  ) , $
  '*', $
  '*     P-T domain definition parameters (log cgs units)', $
  '*     ------------------------------------------------', $
  '*     ntbn        =' +         string(Opa.nTBn      , format='(I3)'  ) , $
  '*     TABTBN      =' + strjoin(string(Opa.TabTBn    , format='(F8.3)')), $
  '*     TABDTB      =' + strjoin(string(opa.TabdTB    , format='(F8.3)')), $
  '*     npbn        =' +         string(Opa.nPBn      , format='(I3)'  ) , $
  '*     TABPBN      =' + strjoin(string(Opa.TabPBn    , format='(F8.3)')), $
  '*     TABDPB      =' + strjoin(string(opa.TabdPB    , format='(F8.3)')), $
  '*', $
  '*     Merging parameters (log cgs units)', $
  '*     ----------------------------------', $
  '*     TMergeLow   =' +         string(TLow          , format='(F8.3)') , $
  '*     TMergeHigh  =' +         string(THigh         , format='(F8.3)') , $
  '*     nMerge      =' +         string(iTHigh-iTLow+1, format='(I3)'  ) , $
;  '*', $
;  '*     Recommended CUBIT interpolation mode', $
;  '*     ------------------------------------', $
;  '*     intmdx      = ' + string(Opa.Intmdx, format='(I3)'), $
  '*', $
;  '************************************************************************', $
  '* -----------------------------------------------------------------------', $
  Opa0.Header, $
;  '************************************************************************', $
  '* -----------------------------------------------------------------------', $
  Opa1.Header $
;  '************************************************************************' $
       ]

; --- Merge header and existing structure ---
Opa=create_struct('Header', Header, Opa)


;; --- Modify header ---
;TimeCreate=systime(0)
;Opa.Header[1]='*     Opacity table name: ' + Name
;Opa.Header[2]='*     Created           : ' + TimeCreate
;Opa.Header[5]='*     '                     + Comment
;Opa.OptaNa   =Name
;Opa.DateX    =TimeCreate



; --- Return result structure ---
return, Opa

end ; meopta
