pro rhd_manipulate_addpoints, addn, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, $
      rhofac=rhofac
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_addpoints ('rhd_manipulate_add_points')
;
; PURPOSE:
;   Add number of points on all sides of simulation data.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_addpoints, addn, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   addn:   ('add_number') integer, number of points to be added on all sides
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;   rhofac: ('rho_factor') real scalar with density decline factor (0.0 ... 1.0)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Extrapolate density possibly decrasing and internal energy constant.
;   Set extrapolated velocity and additional density arrays to zero.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2001-12-11 (B.F Kiel) Written
;   2003-06-10 (B.F.) Keyword rhofac added
;   2013-05-06 (B.F. Lyon) Keyword quc added
;-
;------------------------------------------------------------------------------
;
if (not keyword_set(rhofac)) then rhofac=1.0
;
; --- New index values ---
m1new=m1-addn
n1new=n1+addn
m2new=m2-addn
n2new=n2+addn
m3new=m3-addn
n3new=n3+addn
;
; --- Empty axes ---
xc1new=fltarr(n1new-m1new+1)
xb1new=fltarr(n1new-m1new+2)
xc2new=fltarr(n2new-m2new+1)
xb2new=fltarr(n2new-m2new+2)
xc3new=fltarr(n3new-m3new+1)
xb3new=fltarr(n3new-m3new+2)
;
; --- Put old values into new vectors ---
xc1new(addn:n1new-m1new-addn  )=reform(xc1)
xb1new(addn:n1new-m1new-addn+1)=reform(xb1)
xc2new(addn:n2new-m2new-addn  )=reform(xc2)
xb2new(addn:n2new-m2new-addn+1)=reform(xb2)
xc3new(addn:n3new-m3new-addn  )=reform(xc3)
xb3new(addn:n3new-m3new-addn+1)=reform(xb3)
;
xc1=temporary(xc1new)
xb1=temporary(xb1new)
xc2=temporary(xc2new)
xb2=temporary(xb2new)
xc3=temporary(xc3new)
xb3=temporary(xb3new)
;
; --- Add points to axes: extrapolate equidistant grid ---
xc1(0                 :addn-1)=xc1(addn              ) + $
                               (findgen(addn)-float(addn))*(xc1(addn+1            )-xc1(addn              ))
xc1(n1new-m1new-addn+1:*     )=xc1(n1new-m1new-addn  ) + $
                               (findgen(addn)+1.0        )*(xc1(n1new-m1new-addn  )-xc1(n1new-m1new-addn-1))
xb1(0                 :addn-1)=xb1(addn              ) + $
                               (findgen(addn)-float(addn))*(xb1(addn+1            )-xb1(addn              ))
xb1(n1new-m1new-addn+2:*     )=xb1(n1new-m1new-addn+1) + $
                               (findgen(addn)+1.0        )*(xb1(n1new-m1new-addn+1)-xb1(n1new-m1new-addn  ))
;
xc2(0                 :addn-1)=xc2(addn              ) + $
                               (findgen(addn)-float(addn))*(xc2(addn+1            )-xc2(addn              ))
xc2(n2new-m2new-addn+1:*     )=xc2(n2new-m2new-addn  ) + $
                               (findgen(addn)+1.0        )*(xc2(n2new-m2new-addn  )-xc2(n2new-m2new-addn-1))
xb2(0                 :addn-1)=xb2(addn              ) + $
                               (findgen(addn)-float(addn))*(xb2(addn+1            )-xb2(addn              ))
xb2(n2new-m2new-addn+2:*     )=xb2(n2new-m2new-addn+1) + $
                               (findgen(addn)+1.0        )*(xb2(n2new-m2new-addn+1)-xb2(n2new-m2new-addn  ))
;
xc3(0                 :addn-1)=xc3(addn              ) + $
                               (findgen(addn)-float(addn))*(xc3(addn+1            )-xc3(addn              ))
xc3(n3new-m3new-addn+1:*     )=xc3(n3new-m3new-addn  ) + $
                               (findgen(addn)+1.0        )*(xc3(n3new-m3new-addn  )-xc3(n3new-m3new-addn-1))
xb3(0                 :addn-1)=xb3(addn              ) + $
                               (findgen(addn)-float(addn))*(xb3(addn+1            )-xb3(addn              ))
xb3(n3new-m3new-addn+2:*     )=xb3(n3new-m3new-addn+1) + $
                               (findgen(addn)+1.0        )*(xb3(n3new-m3new-addn+1)-xb3(n3new-m3new-addn  ))
;
; --- Bulk arrays: copy old part ---
rhonew=replicate(rho(0,0,0), n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
rhonew(addn:n1new-m1new-addn, addn:n2new-m2new-addn, addn:n3new-m3new-addn)=rho
rho=temporary(rhonew)
;
einew=replicate(ei(0,0,0), n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
einew(addn:n1new-m1new-addn, addn:n2new-m2new-addn, addn:n3new-m3new-addn)=ei
ei=temporary(einew)
;
v1new=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v1new(addn:n1new-m1new-addn, addn:n2new-m2new-addn, addn:n3new-m3new-addn)=v1
v1=temporary(v1new)
;
v2new=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v2new(addn:n1new-m1new-addn, addn:n2new-m2new-addn, addn:n3new-m3new-addn)=v2
v2=temporary(v2new)
;
v3new=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v3new(addn:n1new-m1new-addn, addn:n2new-m2new-addn, addn:n3new-m3new-addn)=v3
v3=temporary(v3new)
;
if (keyword_set(quc)) then begin
  nquc=n_elements(quc[0,0,0,*])
  qucnew=replicate(quc[0,0,0,0], n1new-m1new+1, n2new-m2new+1, n3new-m3new+1, nquc)
  for iquc=0,nquc-1 do begin
    qucnew[addn:n1new-m1new-addn, addn:n2new-m2new-addn, addn:n3new-m3new-addn, 0:nquc-1]=quc
  endfor ; iquc
  quc=temporary(qucnew)
endif
;
; --- Bulk arrays: boundaries ---
; --- rho ---
if ((rhofac eq 0.0) or (rhofac eq 1.0)) then begin
  rho(        0:addn-1, addn:n2new-m2new-addn, addn:n3new-m3new-addn)= $
    rebin(rho(addn    , addn:n2new-m2new-addn, addn:n3new-m3new-addn), $
          addn, n2-m2+1, n3-m3+1)
  rho(        n1new-m1new-addn+1:*, addn:n2new-m2new-addn, addn:n3new-m3new-addn)= $
    rebin(rho(n1new-m1new-addn    , addn:n2new-m2new-addn, addn:n3new-m3new-addn), $
          addn, n2-m2+1, n3-m3+1)
  rho(        *                   , 0:addn-1, addn:n3new-m3new-addn)= $
    rebin(rho(*                   , addn    , addn:n3new-m3new-addn), $
          n1new-m1new+1, addn, n3-m3+1)
  rho(        *                   , n2new-m2new-addn+1:*, addn:n3new-m3new-addn)= $
    rebin(rho(*                   , n2new-m2new-addn    , addn:n3new-m3new-addn), $
          n1new-m1new+1, addn, n3-m3+1)
  rho(        *                   , *                   , 0:addn-1)= $
    rebin(rho(*                   , *                   , addn    ), $
          n1new-m1new+1, n2new-m2new+1, addn)
  rho(        *                   , *                   , n3new-m3new-addn+1:*)= $
    rebin(rho(*                   , *                   , n3new-m3new-addn    ), $
          n1new-m1new+1, n2new-m2new+1, addn)
endif else begin
  for i=1,addn do begin
    rho(  addn-i              , addn:n2new-m2new-addn, addn:n3new-m3new-addn)= $
      rho(addn-i+1            , addn:n2new-m2new-addn, addn:n3new-m3new-addn) * rhofac
    rho(  n1new-m1new-addn+i  , addn:n2new-m2new-addn, addn:n3new-m3new-addn)= $
      rho(n1new-m1new-addn+i-1, addn:n2new-m2new-addn, addn:n3new-m3new-addn) * rhofac
  endfor
  for i=1,addn do begin
    rho(        *             , addn-i               , addn:n3new-m3new-addn)= $
      rho(      *             , addn-i+1             , addn:n3new-m3new-addn) * rhofac
    rho(        *             , n2new-m2new-addn+i   , addn:n3new-m3new-addn)= $
      rho(      *             , n2new-m2new-addn+i-1 , addn:n3new-m3new-addn) * rhofac
  endfor
  for i=1,addn do begin
    rho(        *             , *                    , addn-i              )= $
      rho(      *             , *                    , addn-i+1            ) * rhofac
    rho(        *             , *                    , n3new-m3new-addn+i  )= $
      rho(      *             , *                    , n3new-m3new-addn+i-1) * rhofac
  endfor
endelse
;
; --- ei ---
ei(        0:addn-1, addn:n2new-m2new-addn, addn:n3new-m3new-addn)= $
  rebin(ei(addn    , addn:n2new-m2new-addn, addn:n3new-m3new-addn), $
        addn, n2-m2+1, n3-m3+1)
ei(        n1new-m1new-addn+1:*, addn:n2new-m2new-addn, addn:n3new-m3new-addn)= $
  rebin(ei(n1new-m1new-addn    , addn:n2new-m2new-addn, addn:n3new-m3new-addn), $
        addn, n2-m2+1, n3-m3+1)
ei(        *                   , 0:addn-1, addn:n3new-m3new-addn)= $
  rebin(ei(*                   , addn    , addn:n3new-m3new-addn), $
        n1new-m1new+1, addn, n3-m3+1)
ei(        *                   , n2new-m2new-addn+1:*, addn:n3new-m3new-addn)= $
  rebin(ei(*                   , n2new-m2new-addn    , addn:n3new-m3new-addn), $
        n1new-m1new+1, addn, n3-m3+1)
ei(        *                   , *                   , 0:addn-1)= $
  rebin(ei(*                   , *                   , addn    ), $
        n1new-m1new+1, n2new-m2new+1, addn)
ei(        *                   , *                   , n3new-m3new-addn+1:*)= $
  rebin(ei(*                   , *                   , n3new-m3new-addn    ), $
        n1new-m1new+1, n2new-m2new+1, addn)
;
; --- New indices ---
m1=m1new
n1=n1new
m2=m2new
n2=n2new
m3=m3new
n3=n3new
;
end ; rhd_manipulate_addpoints
