pro rhd_manipulate_addpointsbottom, addn, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, $
      grav=grav, EOS=EOS, noshiftx3=noshiftx3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_addpointsbottom ('rhd_manipulate_add_points_bottom')
;
; PURPOSE:
;   Add number of points on bottom of data box.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_addpointsbottom, addn, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   addn:   ('add_number') integer, number of points to be added on all sides
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   11.12.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   10. 6.03 (B.F.) Keyword rhofac added
;   17. 3.04 (B.F.) addpoints -> addpointstop
;   19. 3.04 (B.F.) New formula for xb3 extrapolation
;   2007-01-09 (B.F. Lyon)
;   2014-12-09 (B.F. Uppsala) Keyword noshiftx3 added
;-
;------------------------------------------------------------------------------

; --- New index values ---
m1new=m1
n1new=n1
m2new=m2
n2new=n2
m3new=m3-addn
n3new=n3

; --- Empty axes ---
xc3new=fltarr(n3new-m3new+1)
xb3new=fltarr(n3new-m3new+2)

; --- Put old values into new vectors ---
xc3new[addn:n3new-m3new  ]=reform(xc3)
xb3new[addn:n3new-m3new+1]=reform(xb3)

xc3=temporary(xc3new)
xb3=temporary(xb3new)

; --- Add points to axes: extrapolate equidistant grid ---
xc3[0:addn-1]=xc3[addn] + $
              (findgen(addn)-float(addn))*(xc3[addn+1]-xc3[addn])
;xc3(n3new-m3new-addn+1:*     )=xc3(n3new-m3new-addn) + $
;                               (findgen(addn)+1.0)*(xc3(n3new-m3new-addn  )-xc3(n3new-m3new-addn-1))

xb3[1:addn]=0.5 * (xc3[0:addn-1] + xc3[1:addn]  )
xb3[0]=xb3[1] + (xb3[2]-xb3[1])^2/(xb3[2]-xb3[3])
;xb3(n3new-m3new-addn+2:n3new-m3new)=0.5 * (xc3(n3new-m3new-addn+1:n3new-m3new-1) + $
;                                           xc3(n3new-m3new-addn+2:n3new-m3new  )  )
;xb3(n3new-m3new+1)=xb3(n3new-m3new) + (xb3(n3new-m3new)-xb3(n3new-m3new-1))^2/(xb3(n3new-m3new-1)-xb3(n3new-m3new-2))


if (not keyword_set(noshiftx3)) then begin
  ; --- Shift axes: zero point at top of xb3 scale ---
  xc3=xc3-xb3[n3new-m3new+1]
  xb3=xb3-xb3[n3new-m3new+1]
  ; --- Shift axes: zero point at bottom of xc3 scale ---
  ;xb3=xb3-xc3[0]
  ;xc3=xc3-xc3[0]
endif

; --- Bulk arrays: copy old part ---
rhonew=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
rhonew[0:n1new-m1new, 0:n2new-m2new, addn:n3new-m3new]=rho
rho=temporary(rhonew)

einew=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
einew[0:n1new-m1new, 0:n2new-m2new, addn:n3new-m3new]=ei
ei=temporary(einew)

v1new=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v1new[0:n1new-m1new, 0:n2new-m2new, addn:n3new-m3new]=v1
v1=temporary(v1new)

v2new=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v2new[0:n1new-m1new, 0:n2new-m2new, addn:n3new-m3new]=v2
v2=temporary(v2new)

v3new=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v3new[0:n1new-m1new, 0:n2new-m2new, addn:n3new-m3new]=v3
v3=temporary(v3new)

; --- Integrate adiabatic stratification starting from old bottom layer ---
dz    =-0.01*(xc3[1]-xc3[0])
help,dz
npoint=110L*(addn+1L)
stratintegrate, avg(ei[*,*,addn]), avg(rho[*,*,addn]), xc3[addn], eifine, rhofine, zfine, $
  dz=dz, grav=grav, npoint=npoint, eos=EOS

; --- New 1D stratification ---
eistrat =interpol(eifine , zfine, xc3[0:addn-1])
rhostrat=interpol(rhofine, zfine, xc3[0:addn-1])

; --- Bulk arrays (velocities in extension are zero) ---
ei[ *,*,0:addn-1]=rebin(reform(eistrat , 1,1,addn), n1new-m1new+1, n2new-m2new+1, addn)
rho[*,*,0:addn-1]=rebin(reform(rhostrat, 1,1,addn), n1new-m1new+1, n2new-m2new+1, addn)
if (keyword_set(quc)) then begin
  nquc=n_elements(quc[0,0,0,*])
  qucnew=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1, nquc)
  qucnew[0:n1new-m1new, 0:n2new-m2new, addn:n3new-m3new, 0:nquc-1]=quc
  quc=temporary(qucnew)
  for iquc=0,nquc-1 do begin
    quc[*,*,0:addn-1,iquc]=rho[*,*,0:addn-1]/avg(rho[*,*,addn])*avg(quc[*,*,addn,iquc])
  endfor ; iquc
endif

; --- New indices ---
m1=m1new
n1=n1new
m2=m2new
n2=n2new
m3=m3new
n3=n3new

end ; rhd_manipulate_addpointsbottom
