pro rhd_manipulate_addpointstop, addn, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, $
      rhofac=rhofac, noshiftx3=noshiftx3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_addpointstop ('rhd_manipulate_add_points_top')
;
; PURPOSE:
;   Add number of points on top of data box.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_addpointstop, addn, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, $
;     rhofac=rhofac, noshiftx3=noshiftx3
;
; INPUT:
;   addn:   ('add_number') integer, number of points to be added on all sides
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;   quc:    ('quantity_centered') real 4D arrays with additional quantities
;   rhofac: ('rho_factor') real scalar with density decline factor (0.0 ... 1.0)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   11.12.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   10. 6.03 (B.F.) Keyword rhofac added
;   17. 3.04 (B.F.) addpoints -> addpointstop
;   19. 3.04 (B.F.) New formula for xb3 extrapolation
;   2007-10-23 (B.F.) Keyword noshiftx3 added
;   2008-10-22 (B.F.) Keyword quc added
;-
;------------------------------------------------------------------------------
;
if (not keyword_set(rhofac)) then rhofac=1.0
;
; --- New index values ---
m1new=m1
n1new=n1
m2new=m2
n2new=n2
m3new=m3
n3new=n3+addn
;
; --- Empty axes ---
xc3new=fltarr(n3new-m3new+1)
xb3new=fltarr(n3new-m3new+2)
;
; --- Put old values into new vectors ---
xc3new[0:n3new-m3new-addn  ]=reform(xc3)
xb3new[0:n3new-m3new-addn+1]=reform(xb3)
;
xc3=temporary(xc3new)
xb3=temporary(xb3new)
;
; --- Add points to axes: extrapolate equidistant grid ---
xc3[n3new-m3new-addn+1:*     ]=xc3[n3new-m3new-addn] + $
                               (findgen(addn)+1.0)*(xc3[n3new-m3new-addn  ]-xc3[n3new-m3new-addn-1])
;
xb3[n3new-m3new-addn+2:n3new-m3new]=0.5 * (xc3[n3new-m3new-addn+1:n3new-m3new-1] + $
                                           xc3[n3new-m3new-addn+2:n3new-m3new  ]  )
xb3[n3new-m3new+1]=xb3[n3new-m3new] + (xb3[n3new-m3new]-xb3[n3new-m3new-1])^2/(xb3[n3new-m3new-1]-xb3[n3new-m3new-2])
if (not keyword_set(noshiftx3)) then begin
  ; --- Shift axes: zero point at top of xb3 scale ---
  xc3=xc3-xb3[n3new-m3new+1]
  xb3=xb3-xb3[n3new-m3new+1]
endif
;
; --- Bulk arrays: copy old part ---
rhonew=replicate(rho[0,0,0]  , n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
rhonew[0:n1new-m1new, 0:n2new-m2new, 0:n3new-m3new-addn]=rho
rho=temporary(rhonew)
;
einew=replicate(ei[0,0,0]    , n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
einew[0:n1new-m1new, 0:n2new-m2new, 0:n3new-m3new-addn]=ei
ei=temporary(einew)
;
v1new=replicate(v1[0,0,0]*0.0, n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v1new[0:n1new-m1new, 0:n2new-m2new, 0:n3new-m3new-addn]=v1
v1=temporary(v1new)
;
v2new=replicate(v2[0,0,0]*0.0, n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v2new[0:n1new-m1new, 0:n2new-m2new, 0:n3new-m3new-addn]=v2
v2=temporary(v2new)
;
v3new=replicate(v3[0,0,0]*0.0, n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v3new[0:n1new-m1new, 0:n2new-m2new, 0:n3new-m3new-addn]=v3
v3=temporary(v3new)
;
if (keyword_set(quc)) then begin
  nquc=n_elements(quc[0,0,0,*])
  qucnew=replicate(quc[0,0,0,0], n1new-m1new+1, n2new-m2new+1, n3new-m3new+1, nquc)
  for iquc=0,nquc-1 do begin
    qucnew[0:n1new-m1new, 0:n2new-m2new, 0:n3new-m3new-addn, 0:nquc-1]=quc
  endfor ; iquc
  quc=temporary(qucnew)
endif
;
; --- Bulk arrays: boundaries ---
; --- rho ---
if ((rhofac eq 0.0) or (rhofac eq 1.0)) then begin
  rho[        *                   , *                   , n3new-m3new-addn+1:*]= $
    rebin(rho[*                   , *                   , n3new-m3new-addn    ], $
          n1new-m1new+1, n2new-m2new+1, addn)
endif else begin
  for i=1,addn do begin
    rho[        *             , *                    , n3new-m3new-addn+i  ]= $
      rho[      *             , *                    , n3new-m3new-addn+i-1] * rhofac
  endfor ; i
endelse
;
; --- Additional densities: keep concentration as in old top layer ---
if (keyword_set(quc)) then begin
  nquc=n_elements(quc[0,0,0,*])
  for iquc=0,nquc-1 do begin
    for i=1,addn do begin
      quc[*, *, n3new-m3new-addn+i, iquc]= quc[*, *, n3new-m3new-addn  , iquc] * $
                                          (rho[*, *, n3new-m3new-addn+i]/rho[*, *, n3new-m3new-addn])
    endfor ; i
  endfor ; iquc
endif
;
; --- ei ---
ei[        *                   , *                   , n3new-m3new-addn+1:*]= $
  rebin(ei[*                   , *                   , n3new-m3new-addn    ], $
        n1new-m1new+1, n2new-m2new+1, addn)
;
; --- New indices ---
m1=m1new
n1=n1new
m2=m2new
n2=n2new
m3=m3new
n3=n3new
;
end ; rhd_manipulate_addpointstop
