pro rhd_manipulate_interpolate, num1, num2, value3, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, $
      quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3, $
      indexstart=indexstart, $
      x1_command=x1_command0, x2_command=x2_command0, x3_command=x3_command0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_interpolate ('rhd_manipulate_interpolate')
;
; PURPOSE:
;   Interpolate simulation data to new (equidistant) grid.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_interpolate, num1, num2, num3, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, $
;     indexstart=indexstart, $
;     x1_command=x1_command, x2_command=x2_command, x3_command=x3_command
;
; INPUT:
;   num1:   ('number_1') integer, number of points of new grid in x1 direction
;   num2:   ('number_2') integer, number of points of new grid in x2 direction
;   num3:   ('number_3') integer, number of points of new grid in x3 direction
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('velocity_1') real 3D array, velocity, 1st coordinate
;   v2:     ('velocity_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('velocity_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;   quc:    ('quantity_centered') real 4D arrays with additional quantities
;   Bb1:    ('B_boundary_1') real 3D array, magnetic field, 1st coordinate
;   Bb2:    ('B_boundary_2') real 3D array, magnetic field, 2nd coordinate
;   Bb3:    ('B_boundary_3') real 3D array, magnetic field, 3rd coordinate
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   An interpolated magnetic field might be not divergence free.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2001-12-11 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2003-07-01 (B.F.)
;   2007-01-09 (B.F. Lyon) Interpolation of quc
;   2010-02-25 (B.F.) Interpolation of Bb: warning: result might be not divergence free!
;   2010-12-20 (B.F.) Non-equidistant xc3 axis
;-
;------------------------------------------------------------------------------

; --- Default value for keywords ---
; --- Start index ---
if (keyword_set(indexstart)) then begin
  indexstart0=indexstart
endif else begin
  indexstart0='box'
endelse
; --- Commands for axis handling ---
if (n_elements(x1_command0) eq 0) then begin
  x1_command=''
endif else begin
  x1_command=x1_command0
endelse
if (n_elements(x2_command0) eq 0) then begin
  x2_command=''
endif else begin
  x2_command=x2_command0
endelse
if (n_elements(x3_command0) eq 0) then begin
  x3_command=''
endif else begin
  x3_command=x3_command0
endelse

; --- New number of elements of axes ---
if (num1 eq 0) then begin
  num1=n1-m1+1
  if (x1_command eq '') then x1_command='nochange'
endif
if (num2 eq 0) then begin
  num2=n2-m2+1
  if (x2_command eq '') then x2_command='nochange'
endif
sizevalue3=size(value3)
if ((n_elements(value3) eq 1) and $
    ((sizevalue3[1] eq 1) or (sizevalue3[1] eq 2) or (sizevalue3[1] eq 3)) $
   ) then begin
  num3=value3
  value3_numberflag=1
  if (num3 eq 0) then begin
    num3=n3-m3+1
    if (x3_command eq '') then x3_command='nochange'
  endif
endif else begin
  if (x3_command eq 'newxc3') then begin
    num3=n_elements(value3)
  endif else begin
    num3=n_elements(value3)-1
  endelse
help,x3_command, num3,value3_numberflag
  value3_numberflag=0
endelse

; --- Check if change is necessary at all ---
if ((num1 ne (n1-m1+1)) and (x1_command eq 'nochange')) then x1_command=''
if ((num2 ne (n2-m2+1)) and (x2_command eq 'nochange')) then x2_command=''
if ((num3 ne (n3-m3+1)) and (x3_command eq 'nochange')) then x3_command=''

help, num1, num2, num3

; --- Box size: indices ---
if ((indexstart0 eq 'central') or (indexstart0 eq 'center')) then begin
  ; --- Indices (more or less) symmetric around 0 ---
  m1new=-num1/2
  m2new=-num2/2
  m3new=-num3/2
endif else if (indexstart0 eq '1') then begin
  ; --- Indices starting with 1 ---
  m1new=1
  m2new=1
  m3new=1
endif else if (indexstart0 eq '0') then  begin
  ; --- Indices starting with 0 ---
  m1new=0
  m2new=0
  m3new=0
endif else begin
  ; --- Old start indices ---
  m1new=m1
  m2new=m2
  m3new=m3
endelse
n1new=num1-1+m1new
n2new=num2-1+m2new
n3new=num3-1+m3new

; --- Create new (finer or coarser) axes ---
; --- x1 ---
if (x1_command eq 'nochange') then begin
  ; --- Keep axis (might be non-equidistant) ---
  ic1new=findgen(num1)
endif else begin
  ; --- Change number of grid points in this direction, keep total extent ---
  dx1   =(xb1(n1-m1+1)-xb1(0))/float(n1-m1+1)
  dx1new=(xb1(n1-m1+1)-xb1(0))/float(num1)
  xb1new=xb1(0) + dx1new * findgen(num1+1)
  xc1new=0.5*(xb1new(0:num1-1)+xb1new(1:num1))
  xb1new(1:num1-1)=0.5*(xc1new(0:num1-2)+xc1new(1:num1-1))
  ic1new = 0.0 > ((xc1new-xc1(0))/dx1) < float(n1-m1  )
  ib1new = 0.0 > ((xb1new-xb1(0))/dx1) < float(n1-m1+1)
  ;
  m1 =m1new
  n1 =n1new
  xc1=xc1new
  xb1=xb1new
endelse

; --- x2 ---
if (x2_command eq 'nochange') then begin
  ; --- Keep axis (might be non-equidistant) ---
  ic2new=findgen(num2)
endif else begin
  ; --- Change number of grid points in this direction, keep total extent ---
  dx2   =(xb2(n2-m2+1)-xb2(0))/float(n2-m2+1)
  dx2new=(xb2(n2-m2+1)-xb2(0))/float(num2)
  xb2new=xb2(0) + dx2new * findgen(num2+1)
  xc2new=0.5*(xb2new(0:num2-1)+xb2new(1:num2))
  xb2new(1:num2-1)=0.5*(xc2new(0:num2-2)+xc2new(1:num2-1))
  ic2new = 0.0 > ((xc2new-xc2(0))/dx2) < float(n2-m2  )
  ib2new = 0.0 > ((xb2new-xb2(0))/dx2) < float(n2-m2+1)
  ;
  m2 =m2new
  n2 =n2new
  xc2=xc2new
  xb2=xb2new
endelse

; --- x3 ---
if (value3_numberflag) then begin
  ; --- Only number of new elements specified ---
  ;
  if (x3_command eq 'nochange') then begin
    ; --- Keep axis (might be non-equidistant) ---
    ic3new=findgen(num3)
  endif else begin
    ; --- Change number of grid points in this direction, keep total extent ---
    dx3   =(xb3[n3-m3+1]-xb3[0])/float(n3-m3+1)
    dx3new=(xb3[n3-m3+1]-xb3[0])/float(num3)
    xb3new=xb3[0] + dx3new * findgen(num3+1)
    xc3new=0.5*(xb3new[0:num3-1]+xb3new[1:num3])
    xb3new[1:num3-1]=0.5*(xc3new[0:num3-2]+xc3new[1:num3-1])
;    ic3new = 0.0 > ((xc3new-xc3new(0))/dx3) < float(n3-m3  )
;    ib3new = 0.0 > ((xb3new-xb3new(0))/dx3) < float(n3-m3+1)
    ic3new=interpol(findgen(n3-m3+1), xc3, xc3new)
    ib3new=interpol(findgen(n3-m3+2), xb3, xb3new)
    ;
    m3 =m3new
    n3 =n3new
    xc3=xc3new
    xb3=xb3new
  endelse
endif else if (x3_command eq 'newxc3') then begin
  ; --- New xc3 axis specified ---
  xc3new=value3
  ;
  xb3new=fltarr(n3new-m3new+2)
  xb3new[1:num3-1]=0.5*(xc3new(0:num3-2)+xc3new(1:num3-1))
  xb3new[0]=xb3new[1] + (xb3new[2]-xb3new[1])^2/(xb3new[2]-xb3new[3])
  xb3new[num3]=xb3new[num3-1] + (xb3new[num3-2]-xb3new[num3-1])^2/(xb3new[num3-2]-xb3new[num3-3])
  ic3new=interpol(findgen(n3-m3+1), xc3, xc3new)
  ib3new=interpol(findgen(n3-m3+2), xb3, xb3new)
  ;
  m3 =m3new
  n3 =n3new
  xc3=xc3new
  xb3=xb3new
endif else begin
  ; --- New xb3 axis specified ---
  xb3new=value3
  ;
  xc3new=0.5*(xb3new(0:num3-1)+xb3new(1:num3))
  ic3new=interpol(findgen(n3-m3+1), xc3, xc3new)
  ib3new=interpol(findgen(n3-m3+2), xb3, xb3new)
  ;
  m3 =m3new
  n3 =n3new
  xc3=xc3new
  xb3=xb3new
endelse

; --- Interpolate bulk arrays ---
rho=interpolate(rho, ic1new,ic2new,ic3new, /grid)
ei =interpolate(ei , ic1new,ic2new,ic3new, /grid)
v1 =interpolate(v1 , ic1new,ic2new,ic3new, /grid)
v2 =interpolate(v2 , ic1new,ic2new,ic3new, /grid)
v3 =interpolate(v3 , ic1new,ic2new,ic3new, /grid)
if (keyword_set(quc)) then begin
  ; --- Additional density fields ---
  nquc=n_elements(quc[0,0,0,*])
  qucnew=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1, nquc)
  for iquc=0,nquc-1 do begin
    qucnew[*,*,*,iquc]=interpolate(quc[*,*,*,iquc],  ic1new,ic2new,ic3new, /grid)
  endfor ; iquc
  quc=temporary(qucnew)
endif
if (keyword_set(Bb1)) then begin
  ; --- Magnetic field at cell boundaries ---
  Bb1=interpolate(Bb1, ib1new,ic2new,ic3new, /grid)
  Bb2=interpolate(Bb2, ic1new,ib2new,ic3new, /grid)
  Bb3=interpolate(Bb3, ic1new,ic2new,ib3new, /grid)
endif

end ; rhd_manipulate_interpolate
