pro rhd_manipulate_makebox, box, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, $
      quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3, $
      inbox=inbox, $
      time=time, itime=itime, dtime=dtime, time_out_full_last=time_out_full_last, $
      time_out_mean_last=time_out_mean_last, description=description, history=history
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_makebox ('rhd_manipulate_make_box')
;
; PURPOSE:
;   Put individual arrays into RHD data structure.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_makebox, box, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('velocity_1') real 3D array, velocity, 1st coordinate
;   v2:     ('velocity_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('velocity_3') real 3D array, velocity, 3rd coordinate
;
; OUTPUT:
;   box:    ('box') structure with RHD data
;
; KEYWORDS:
;   inbox:  ('input_box') structure with original RHD data (to get e.g. time...)
;   quc:    ('quantity_centered') real 4D array with additional quantities
;   Bb1:    ('B_boundary_1') real 3D array, magnetic field, 1st coordinate
;   Bb2:    ('B_boundary_2') real 3D array, magnetic field, 2nd coordinate
;   Bb3:    ('B_boundary_3') real 3D array, magnetic field, 3rd coordinate
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-06-26 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2007-01-09 (B.F. Lyon) quc array
;-
;------------------------------------------------------------------------------
;
if (keyword_set(inbox)) then begin
  time=inbox.z.time
  itime=inbox.z.itime
  dtime=inbox.dtime
  time_out_full_last=inbox.time_out_full_last
  time_out_mean_last=inbox.time_out_mean_last
  description=inbox.head.description
  history    =[inbox.head.history(0:8), $
               '...', $
               inbox.head.history(12:*), $
               'Processed by rhd_manipulate_makebox: ' + strtrim(systime(0),2) $
              ]
endif else begin
  if (n_elements(time) eq 0) then begin
    time=0.0
  endif
  if (n_elements(itime) eq 0) then begin
    itime=0L
  endif
  if (n_elements(dtime) eq 0) then begin
    dtime=1.0
  endif
  if (n_elements(time_out_full_last) eq 0) then begin
    time_out_full_last=-1.0E20
  endif
  if (n_elements(time_out_mean_last) eq 0) then begin
    time_out_mean_last=-1.0E20
  endif
  if (n_elements(description) eq 0) then begin
    description=['Composed with rhd_manipulate_makebox.pro']
  endif
  if (n_elements(history) eq 0) then begin
    history    =['Composed with rhd_manipulate_makebox.pro ' + strtrim(systime(0),2)]
  endif
endelse
;
version    ='rhd_manipulate_makebox 2003-07-01'
;
headdata=create_struct( $
  'type'       , 'uio', $
  'file_id'    , 'rhd-data', $
  'description', description, $
  'history'    , history, $
  'version'    , version $
)
zdata=create_struct( $
  'type'       , 'uio', $
  'box_id'     , 'z', $
  'dimension'  , long([[m1,n1], [m2,n2], [m3,n3]]), $
  'time'       , time, $
  'itime'      , itime, $
  'xc1'        , xc1, $
  'xc2'        , xc2, $
  'xc3'        , xc3, $
  'xb1'        , xb1, $
  'xb2'        , xb2, $
  'xb3'        , xb3, $
  'rho'        , rho, $
  'ei'         , ei, $
  'v1'         , v1, $
  'v2'         , v2, $
  'v3'         , v3 $
)
if (keyword_set(quc)) then begin
  nquc=n_elements(quc[0,0,0,*])
  for iquc=1,nquc do begin
     ident='00' + strtrim(string(iquc, format='(I3)'),2)
     ident='quc' + strmid(ident,strlen(ident)-3,3)
     zdata=create_struct(zdata, ident, quc[*,*,*,iquc-1])
  endfor ; iquc
endif
if (keyword_set(Bb1)) then begin
  zdata=create_struct(zdata, 'Bb1', Bb1, 'Bb2', Bb2, 'Bb3', Bb3)
endif

box=create_struct( $
  'type'       , 'uio', $
  'head'       , temporary(headdata), $
  'dataset_id' , 'single_box', $
  'modeltime'  , time, $
  'modelitime' , itime, $
  'dtime'      , dtime, $
  'time_out_full_last', time_out_full_last, $
  'time_out_mean_last', time_out_mean_last, $
  'z'          , temporary(zdata) $
)
;
end ; rhd_manipulate_makebox
