pro rhd_manipulate_parsebox, box, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, $
      quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3, $
      indexstart=indexstart
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_parsebox ('rhd_manipulate_parse_box')
;
; PURPOSE:
;   Take data from RHD data structure and put it into individual arrays.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_parsebox, box, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, $
;     quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3, $
;     indexstart=indexstart
;
; INPUT:
;   box:     ('box') structure with RHD data
;
; OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('velocity_1') real 3D array, velocity, 1st coordinate
;   v2:     ('velocity_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('velocity_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;   quc:    ('quantity_centered) real 4D array with additional quantities, output
;   Bb1:    ('B_boundary_1') real 3D array, magnetic field, 1st coordinate
;   Bb2:    ('B_boundary_2') real 3D array, magnetic field, 2nd coordinate
;   Bb3:    ('B_boundary_3') real 3D array, magnetic field, 3rd coordinate
;   indexstart: ('index_start') input, string to specify where indices should start:
;               '0'                : m1=m2=m3=0 
;               '1'                : m1=m2=m3=1
;               'box' (default)    : use m1, m2, m3 from box structure
;               'central', 'center': indices (more or less) symmetric around 0
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2001-12-11 (B.F. Uppsala) Written
;   2004-07-29 (B.F. Montpellier) Long integers
;   2007-01-05 (B.F. Lyon) quc array as output parameter
;   2010-02-24 (B.F.) BbX output parameters
;-
;------------------------------------------------------------------------------
;
; --- Default value for keyword ---
if (keyword_set(indexstart)) then begin
  indexstart0=indexstart
endif else begin
  indexstart0='box'
endelse
;
; --- Box size: indices ---
if ((indexstart0 eq 'central') or (indexstart0 eq 'center')) then begin
  ; --- Indices (more or less) symmetric around 0 ---
  m1=-n_elements(box.z.xc1)/2L
  n1= n_elements(box.z.xc1)-1L+m1
  m2=-n_elements(box.z.xc2)/2L
  n2= n_elements(box.z.xc2)-1L+m2
  m3=-n_elements(box.z.xc3)/2L
  n3= n_elements(box.z.xc3)-1L+m3
endif else if (indexstart0 eq '1') then begin
  ; --- Indices starting with 1 ---
  m1=1L
  n1=n_elements(box.z.xc1)
  m2=1L
  n2=n_elements(box.z.xc2)
  m3=1L
  n3=n_elements(box.z.xc3)
endif else if (indexstart0 eq '0') then begin
  ; --- Indices starting with 0 ---
  m1=0L
  n1=n_elements(box.z.xc1)-1L
  m2=0L
  n2=n_elements(box.z.xc2)-1L
  m3=0L
  n3=n_elements(box.z.xc3)-1L
endif else begin
  ; --- Indices from dimension array ---
  m1=box.z.dimension(0,0)
  n1=box.z.dimension(1,0)
  m2=box.z.dimension(0,1)
  n2=box.z.dimension(1,1)
  m3=box.z.dimension(0,2)
  n3=box.z.dimension(1,2)
endelse
;
; --- Axes ---
; xc1=reform(box.z.xc1)
; xc2=reform(box.z.xc2)
; xc3=reform(box.z.xc3)
; xb1=reform(box.z.xb1)
; xb2=reform(box.z.xb2)
; xb3=reform(box.z.xb3)
xc1=reform([box.z.xc1], n1-m1+1)
xc2=reform([box.z.xc2], n2-m2+1)
xc3=reform([box.z.xc3], n3-m3+1)
xb1=reform([box.z.xb1], n1-m1+2)
xb2=reform([box.z.xb2], n2-m2+2)
xb3=reform([box.z.xb3], n3-m3+2)
;
; --- Bulk arrays ---
rho=rebin(box.z.rho, n1-m1+1, n2-m2+1, n3-m3+1)
ei =rebin(box.z.ei , n1-m1+1, n2-m2+1, n3-m3+1)
v1 =rebin(box.z.v1 , n1-m1+1, n2-m2+1, n3-m3+1)
v2 =rebin(box.z.v2 , n1-m1+1, n2-m2+1, n3-m3+1)
v3 =rebin(box.z.v3 , n1-m1+1, n2-m2+1, n3-m3+1)
;
; --- Tag names ---
Tags=tag_names(box.z)
;
; --- Additional centered quantities: quc ---
nquc=round(total(strmid(Tags, 0, 3) eq 'QUC'))
if (nquc gt 0) then begin
  quc=fltarr(n1-m1+1, n2-m2+1, n3-m3+1, nquc)
  for iquc=1,nquc do begin
    ident='00' + strtrim(string(iquc, format='(I3)'),2)
    ident='quc' + strmid(ident,strlen(ident)-3,3)
    dummy=execute("quc[*,*,*,iquc-1]=box.z." + ident)
  endfor ; iquc
endif
;
; --- Additional centered quantities: Bb1, Bb2, Bb3 ---
if (round(total(strmid(Tags, 0, 3) eq 'BB1')) gt 0) then begin
  Bb1=rebin(box.z.Bb1, n1-m1+2, n2-m2+1, n3-m3+1)
  Bb2=rebin(box.z.Bb2, n1-m1+1, n2-m2+2, n3-m3+1)
  Bb3=rebin(box.z.Bb3, n1-m1+1, n2-m2+1, n3-m3+2)
endif
;
end ; rhd_manipulate_parsebox
