pro rhd_manipulate_repeat, nRep10, nRep20, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_repeat ('rhd_manipulate_add_points_bottom')
;
; PURPOSE:
;   Replicate the box horizontally.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_repeat, nRep10, nRep20, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc
;
; INPUT:
;   nRep10: ('number_Repeat_1_0') integer, number of times to repeat the box in x1 direction
;   nRep20: ('number_Repeat_2_0') integer, number of times to repeat the box in x2 direction
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;   quc:    ('quantity_centered') real 4D array with additional cell-centered quantities
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   No treatment of magnetic fields, so far.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2015-05-19 (B.F. Uppsala) Written based on rhd_manipulate_addpointsbottom
;-
;------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr

; --- Limit input parameters ---
nRep1=long(nRep10) > 1
nRep2=long(nRep20) > 2

; --- Determine number of additional arrays ---
if (keyword_set(quc)) then begin
  nquc=n_elements(quc[0,0,0,*])
endif else begin
  nquc=0
endelse

; --- New index values ---
m1new=m1
n1new=(n1-m1+1)*nRep1 + m1-1
m2new=m2
n2new=(n2-m2+1)*nRep2 + m2-1
m3new=m3
n3new=n3

; --- Create empty axes, copy x3 axes ---
xc1new=fltarr(n1new-m1new+1)
xb1new=fltarr(n1new-m1new+2)
xc2new=fltarr(n2new-m2new+1)
xb2new=fltarr(n2new-m2new+2)
xc3new=reform(xc3)
xb3new=reform(xb3)

; --- Create empty bulk arrays ---
rhonew=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
einew =fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v1new =fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v2new =fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
v3new =fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1)
if (nquc lt 0) then begin
  qucnew=fltarr(n1new-m1new+1, n2new-m2new+1, n3new-m3new+1, nquc)
endif

; --- Put old values into new vectors ---
for iRep1=0,nRep1-1 do begin
  iSh1=      iRep1 * (n1-m1+1)
  xSh1=float(iRep1)* (xb1[n1]-xb1[0])
  xc1new[iSh1:iSh1+n1-m1  ]=reform(xc1)+xSh1
  xb1new[iSh1:ish1+n1-m1+1]=reform(xb1)+xSh1
endfor ; iRep1
;
for iRep2=0,nRep2-1 do begin
  iSh2=iRep2       * (n2-m2+1)
  xSh2=float(iRep2)* (xb2[n2]-xb2[0])
  xc2new[iSh2:iSh2+n2-m2  ]=reform(xc2)+xSh2
  xb2new[iSh2:ish2+n2-m2+1]=reform(xb2)+xSh2
  ;
  for iRep1=0,nRep1-1 do begin
    iSh1=iRep1 * (n1-m1+1)
    ;
    ; --- Copy old part of bulk arrays ---
    rhonew[iSh1:iSh1+n1-m1, iSh2:iSh2+n2-m2, *]=rho
    einew[ iSh1:iSh1+n1-m1, iSh2:iSh2+n2-m2, *]=ei
    v1new[ iSh1:iSh1+n1-m1, iSh2:iSh2+n2-m2, *]=v1
    v2new[ iSh1:iSh1+n1-m1, iSh2:iSh2+n2-m2, *]=v2
    v3new[ iSh1:iSh1+n1-m1, iSh2:iSh2+n2-m2, *]=v3
    if (nquc gt 0) then begin
      qucnew[iSh1:iSh1+n1-m1, iSh2:iSh2+n2-m2, quc]=quc
    endif
    ;
  endfor ; iRep1
endfor ; iRep2

; --- Copy all arrays ---
xc1=temporary(xc1new)
xb1=temporary(xb1new)
xc2=temporary(xc2new)
xb2=temporary(xb2new)
xc3=temporary(xc3new)
xb3=temporary(xb3new)

rho=temporary(rhonew)
ei =temporary(einew )
v1 =temporary(v1new )
v2 =temporary(v2new )
v3 =temporary(v3new )
if (nquc gt 0) then begin
  quc=temporary(qucnew)
endif

; --- Copy indices ---
m1=m1new
n1=n1new
m2=m2new
n2=n2new
m3=m3new
n3=n3new

end ; rhd_manipulate_repeat
