pro rhd_wrbox2d, $
      filename=filename, fileform=fileform, fileconv=fileconv, $
      xc10=xc10, xc30=xc30, v10=v10, v30=v30, rho0=rho0, ei0=ei0, dtime0=dtime0, $
      v2ampl=v2ampl, $
      version=version, description=description, history=history
;+
; NAME:
;   rhd_wrbox2d ('radiation_hydrodynamics_write_box_2_dimensions')
;
; PURPOSE:
;   Write a 2D stratification into file in format appropriate for RHD.
;
; CATEGORY:
;   I/O
;
; CALLING SEQUENCE:
;   rhd_wrbox2d, $
;     filename=filename, fileform=fileform, fileconv=fileconv, $
;     xc10=xc1, xc30=xc3, v10=v1, v30=v3, rho0=rho, ei0=ei
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORDS:
;   filename:('file_name') (Input) String, name of output file
;   fileform:('file_format') (input) string, output file format ('formatted'/'unformatted'/'lnawe')
;   fileconv:('file_conversion') (input) string, output file conversion type ('ieee_4', ...)
;   zarr:    ('z_array') (input) real array, z axis [cm]
;   rhoarr:  ('rho_array') (input) 1D real array, density [g/cm^3]
;   eiarr:   ('energy_internal_array') (input) 1D real array, internal energy per mass [erg/g]
;   version: ('version') (input) string with version ID
;   description: ('description') (input) 1D string array with description of data
;   history: ('history') (input) 1D string array with history information
;
; SIDE EFFECTS:
;   A file is written.
;
; SUBROUTINES:
;   rhd_wrbox2d
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   12. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   22. 7.00 (B.F. Uppsala) 3D version
;   17. 3.04 (B.F.) out_time_mean_last=-1.0E8
;-
;
; --- Check keywords ---
if (not keyword_set(filename)) then filename=''
if (not keyword_set(fileform)) then fileform=''
if (not keyword_set(fileconv)) then fileconv=''
if (not keyword_set(version)) then begin
  version='rhd_wrbox2d 2000.07.22'
endif
if (not keyword_set(v2ampl)) then begin
  D3_flag=0
  v2ampl=0.0
  numD_string='2D'
endif else begin
  D3_flag=1
  numD_string='3D'
endelse
if (not keyword_set(description)) then begin
  description=[numD_string + ' model', $
               strtrim(systime(0),2)]
  ndes=2
endif else begin
  ndes=n_elements(description)
endelse
if (not keyword_set(history)) then begin
  history=['Written by rhd_wrbox2d.pro: ' + strtrim(systime(0),2)]
  nhis=1
endif else begin
  nhis=n_elements(history)
endelse
;
nc_p=-1
;
; --- Physical units ---
unit_time='s'
unit_x1  ='cm'
unit_x2  =unit_x1
unit_x3  =unit_x1
unit_rho ='g/cm^3'
unit_v   ='cm/s'
unit_e   ='erg/g'
;
; --- Box size ---
m1=1
n1=n_elements(rho0(*,0))
m2=1
if (D3_flag) then begin
  n2=n1
endif else begin
  n2=1
endelse
m3=1
n3=n_elements(rho0(0,*))
;
; --- Time information ---
itime=0
time=0.0
if (keyword_set(dtime0)) then begin
  dtime=dtime0
endif else begin
  dtime=1.0
endelse
out_time_full_last=-1.0E8
out_time_mean_last=-1.0E8
;out_time_full_last=0.0
;
; --- Axes ---
xfac=xc10(1)-xc10(0)
xc1=xc10
xb1=fltarr(n1+1)
xb1(1:n1-1)=0.5*(xc1(1:n1-1)+xc1(0:n1-2))
xb1(0     )=xb1(1   ) - (xb1(2   )-xb1(1   ))^2/(xb1(3   )-xb1(2   ))
xb1(n1    )=xb1(n1-1) + (xb1(n1-1)-xb1(n1-2))^2/(xb1(n1-2)-xb1(n1-3))
xc2=xfac*(findgen(n2-m2+1)+float(m2))
xb2=xfac*(findgen(n2-m2+2)+float(m2)-0.5)
xc3=xc30
xb3=fltarr(n3+1)
xb3(1:n3-1)=0.5*(xc3(1:n3-1)+xc3(0:n3-2))
xb3(0     )=xb3(1   ) - (xb3(2   )-xb3(1   ))^2/(xb3(3   )-xb3(2   ))
xb3(n3    )=xb3(n3-1) + (xb3(n3-1)-xb3(n3-2))^2/(xb3(n3-2)-xb3(n3-3))
;
; --- Arrays with bulk data ---
rho=rho0
ei =ei0
v1 =v10
;v2 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
v3 =v30
rho=rebin(reform(rho, n1-m1+1, 1, n3-m3+1), n1-m1+1, n2-m2+1, n3-m3+1)
ei =rebin(reform(ei , n1-m1+1, 1, n3-m3+1), n1-m1+1, n2-m2+1, n3-m3+1)
v1 =rebin(reform(v1 , n1-m1+1, 1, n3-m3+1), n1-m1+1, n2-m2+1, n3-m3+1)
;v2 =rebin(reform(v2 , n1-m1+1, 1, n3-m3+1), n1-m1+1, n2-m2+1, n3-m3+1)
v3 =rebin(reform(v3 , n1-m1+1, 1, n3-m3+1), n1-m1+1, n2-m2+1, n3-m3+1)
;
if (D3_flag) then begin
  v2=(randomu(seed, n1-m1+1, n2-m2+1, n3-m3+1)-0.5)*(2.0*v2ampl)*v1
endif else begin
  v2=fltarr(n1-m1+1, n2-m2+1, n3-m3+1)
endelse
;
help, xc1, xb1, xc2, xb2, xc3, xb3, rho, ei, v1, v2, v3
;
; --- Write data into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
   nc_p, ncout, file=filename, form=fileform, conv=fileconv, $
   ndes=ndes, description=description, $
   nhis=nhis, history=history, $
   version=version, $
   unit_time=unit_time, $
   unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
   unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
   m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
   itime=itime, time=time, dtime=dtime, $
   out_time_full_last=out_time_full_last, $
   out_time_mean_last=out_time_mean_last, $
   xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
   rho=rho, v1=v1, v2=v2, v3=v3, ei=ei
;
print, 'Data written into file ''', filename, ''' in UIO format for RHD'
;
end ; rhd_wrbox2d
