pro rhd_wrboxstrat, $
      filename=filename, fileform=fileform, fileconv=fileconv, double_flag=double_flag, $
      zarr=zarr, rhoarr=rhoarr, eiarr=eiarr, $
      qucarr=qucarr, unit_quc=unit_quc, name_quc=name_quc, $
      vampl=vampl, $
      dx1=dx1, dx2=dx2, dtime=dtime, n1=n1, n2=n2, $
      version=version, description=description, history=history
;+
; NAME:
;   rhd_wrboxstrat ('radiation_hydrodynamics_write_box_stratification')
;
; PURPOSE:
;   Write a 1D stratification into file in format appropriate for RHD.
;
; CATEGORY:
;   I/O
;
; CALLING SEQUENCE:
;   rhd_wrboxstrat, $
;     filename=filename, fileform=fileform, fileconv=fileconv, $
;     zarr=zarr, rhoarr=rhoarr, eiarr=eiarr
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORDS:
;   filename:('file_name') (Input) String, name of output file
;   fileform:('file_format') (input) string, output file format ('formatted'/'unformatted'/'lnawe')
;   fileconv:('file_conversion') (input) string, output file conversion type ('ieee_4', ...)
;   zarr:    ('z_array') (input) real array, z axis [cm]
;   rhoarr:  ('rho_array') (input) 1D real array, density [g/cm^3]
;   eiarr:   ('energy_internal_array') (input) 1D real array, internal energy per mass [erg/g]
;   qucarr:  ('quantity_centered_array') (input) 2D real array, additional densities
;   vampl:   ('velocity_amplitude) (input) real scalar with amplitude of random velocity field
;   dx1:     ('delta_x_1') (input) real scalar, horizontal grid size
;   dx2:     ('delta_x_2') (input) real scalar, horizontal grid size
;   version: ('version') (input) string with version ID
;   description: ('description') (input) 1D string array with description of data
;   history: ('history') (input) 1D string array with history information
;
; SIDE EFFECTS:
;   A file is written.
;
; SUBROUTINES:
;   rhd_wrboxstrat
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   29. 4.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;    7. 5.99 (B.F.) xb3 for irregular grid xc3
;   11. 5.99 (B.F.) improved xb3 for irregular grid xc3
;   23. 2.00 (B.F.) dx1, dx2
;    5. 5.00 (B.F.) Keyword dtime added
;   24. 7.02 (B.F.) Keywords n1, n2, added
;   27. 7.04 (B.F.) Keyword vampl added
;   2007-11-22 (B.F. Lyon) Keywords qucarr, unit_quc, and name_quc added
;   2008-07-29 (B.F.) Parameter double_flag added
;-
;
; --- Check keywords ---
if (not keyword_set(filename)) then filename=''
if (not keyword_set(fileform)) then fileform=''
if (not keyword_set(fileconv)) then fileconv=''
if (not keyword_set(vampl)) then vampl=0.0
if (not keyword_set(version)) then begin
  version='rhd_wrboxstrat 2007.11.22'
endif
if (not keyword_set(description)) then begin
  description=['Stratification', $
               strtrim(systime(0),2)]
  ndes=2
endif else begin
  ndes=n_elements(description)
endelse
if (not keyword_set(history)) then begin
  history=['Written by rhd_wrboxstrat.pro: '+strtrim(systime(0),2)]
  nhis=1
endif else begin
  nhis=n_elements(history)
endelse
;
nc_p=-1
;
; --- Physical units ---
unit_time='s'
unit_x1  ='cm'
unit_x2  =unit_x1
unit_x3  =unit_x1
unit_rho ='g/cm^3'
unit_v   ='cm/s'
unit_e   ='erg/g'
;
; --- Box size ---
m1=1
if (not keyword_set(n1)) then begin
  n1=1
endif
;
m2=1
if (not keyword_set(n2)) then begin
  n2=1
endif
;
m3=1
n3=n_elements(rhoarr)
;
; --- Time information ---
itime=0
time=0.0
if (not keyword_set(dtime)) then dtime=1.0
out_time_full_last=-1.0E20
;out_time_full_last=0.0
;
; --- Axes ---
xfac=zarr(1)-zarr(0)
if (keyword_set(dx1)) then begin
  xfac1=dx1
endif else begin
  xfac1=xfac
endelse
if (keyword_set(dx2)) then begin
  xfac2=dx2
endif else begin
  xfac2=xfac
endelse
xc1=xfac1*(findgen(n1-m1+1)+float(m1)-0.5)
xb1=xfac1*(findgen(n1-m1+2)+float(m1)-1.0)
xc2=xfac2*(findgen(n2-m2+1)+float(m2)-0.5)
xb2=xfac2*(findgen(n2-m2+2)+float(m2)-1.0)
xc3=zarr
xb3=fltarr(n3+1)
xb3[1:n3-1]=0.5*(xc3[1:n3-1]+xc3[0:n3-2])
xb3[0     ]=xb3[1   ] - (xb3[2   ]-xb3[1   ])^2/(xb3[3   ]-xb3[2   ])
xb3[n3    ]=xb3[n3-1] + (xb3[n3-1]-xb3[n3-2])^2/(xb3[n3-2]-xb3[n3-3])
;
; === Arrays with bulk data ===
; --- Thermodynamic quantities ---
rho=reform(rhoarr,1,1,n3-m3+1)
ei =reform(eiarr ,1,1,n3-m3+1)
rho=rebin( rho, n1-m1+1, n2-m2+1, n3-m3+1)
ei =rebin( ei , n1-m1+1, n2-m2+1, n3-m3+1)
rho=reform(rho, n1-m1+1, n2-m2+1, n3-m3+1)
ei =reform(ei , n1-m1+1, n2-m2+1, n3-m3+1)
; --- Velocities ---
v1 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
v2 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
v3 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
if (vampl ne 0.0) then begin
  if (n1 gt m1) then v1=2.0*vampl*(randomu(seed,n1-m1+1, n2-m2+1, n3-m3+1)-0.5)
  if (n2 gt m2) then v2=2.0*vampl*(randomu(seed,n1-m1+1, n2-m2+1, n3-m3+1)-0.5)
  if (n3 gt m3) then v3=2.0*vampl*(randomu(seed,n1-m1+1, n2-m2+1, n3-m3+1)-0.5)
endif
v1=reform(v1 , n1-m1+1, n2-m2+1, n3-m3+1)
v2=reform(v2 , n1-m1+1, n2-m2+1, n3-m3+1)
v3=reform(v3 , n1-m1+1, n2-m2+1, n3-m3+1)
; --- Additional densities ---
if (keyword_set(qucarr)) then begin
  nquc=n_elements(qucarr[0,*])
  quc=reform(qucarr, 1,1,n3-m3+1,nquc)
  quc=rebin( quc,    n1-m1+1, n2-m2+1, n3-m3+1, nquc)
  quc=reform(quc,    n1-m1+1, n2-m2+1, n3-m3+1, nquc)
endif
;
help, xc1, xb1, xc2, xb2, xc3, xb3, rho, ei, v1, v2, v3, quc
;
; --- Write data into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
   nc_p, ncout, file=filename, form=fileform, conv=fileconv, double_flag=double_flag, $
   ndes=ndes, description=description, $
   nhis=nhis, history=history, $
   version=version, $
   unit_time=unit_time, $
   unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
   unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, unit_quc=unit_quc, name_quc=name_quc, $
   m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
   itime=itime, time=time, dtime=dtime, out_time_full_last=out_time_full_last, out_time_mean_last=out_time_full_last, $
   xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
   rho=rho, v1=v1, v2=v2, v3=v3, ei=ei, quc=quc
;
print, 'Data written into file ''', filename, ''' in UIO format for RHD'
;
end ; rhd_wrboxstrat
