function slicebox, box, NormalId=NormalId0, Slope=Slope0
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   slicebox ('slice_box')
;
; PURPOSE:
;   Cut out slice of a box.
;
; CATEGORY:
;   Box handling
;
; CALLING SEQUENCE:
;   boxout=slicebox(box, NormalId=NormalID)
;
; INPUT:
;   box:          ('box') data structure with CO5BOLD data
;
; OUTPUT:
;   slicebox:     ('slice_box') new data structure with part of the original box
;
; KEYWORDS:
;   NormalId:     ('Normal_Identity') string with name of normal,
;                 contains at least one '0' and one '+';
;                 contains no more than two identical characters;
;                 the first occurrence of '+' or '-' is '+';
;                   '0+-':
;   Slope:        ('Slope') real scalar with slope; -1.0 < Slope < 1.0:
;                   0.0: default                
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   An equidistant grid is assumed.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2017-05-31 (B.F. Uppsala) Adapt from chunkbox.pro
;   2018-06-07 (B.F) Add keyword Slope
;-
;---------------------------------------------------------------------------------------------------

; --- Set compiler options ---
compile_opt defint32, strictarr


; === Initialize ===

; --- Check input keyword and set default value if necessary ---
if (keyword_set(NormalId0)) then begin
  NormalId=NormalId0
endif else begin
  NormalId='0+0'
endelse
if (keyword_set(Slope0)) then begin
  Slope=Slope0
endif else begin
  Slope=(-1.0 > Slope0) < 1.0
endelse

; --- Define abbreviations for box dimensions ---
m1=box.z.dimension[0,0]
n1=box.z.dimension[1,0]
m2=box.z.dimension[0,1]
n2=box.z.dimension[1,1]
m3=box.z.dimension[0,2]
n3=box.z.dimension[1,2]

; --- Check basic volume data ---
ztags=tag_names(box.z)
indquctag=where(strmid(ztags,0,3) eq 'QUC', nquc)


; === Handle the various directions ===

if ((NormalId eq '00+') or (NormalId eq '00-')) then begin ;========================================
  ;
  OutBox=chunkbox(box, [[-99999, 99999], [-99999, 99999], [0, 0]])
  ;
endif else if ((NormalId eq '0+0') or (NormalId eq '0-0')) then begin ;=============================
  ;
  OutBox=chunkbox(box, [[-99999, 99999], [0, 0], [-99999, 99999]])
  ;
  if (keyword_set(Slope)) then begin
    ; --- Construct interpolation indices and weights ---
    ind_real=real(n2-m2)/2.0 + (findgen(n1-m1+1)-real(n1-m1)/2.0) * Slope
    ind_low =floor(ind_real) > 0
    ind_up  =(ind_low+1) < (n2-m2)
    w_up    =ind_real - real(ind_low)
;print, 'ind_low=', ind_low
;print, 'ind_up =', ind_up
;print, 'w_up   =', w_up
    ;
    ; --- Compute components of rotation matrix ---
    w2=Slope/sqrt(1.0+Slope^2)
    w1=sqrt(1.0-w2^2)
    ;
    ; --- Handle the axes ---
    OutBox.z.xc1=   box.z.xc1/w1
    OutBox.z.xb1=   box.z.xb1/w1
    OutBox.z.xc2=OutBox.z.xc2*w1
    OutBox.z.xb2=OutBox.z.xb2*w1
    ;
    ; --- Extract bulk data ---
    for i1=0,n1-m1 do begin
      ilow=(ind_low[i1])[0]
      iup =(ind_up[ i1])[0]
      wup =(w_up[   i1])[0]
;help, ind_low[i1], ind_up[ i1], ilow, iup
      OutBox.z.rho[i1, 0, *]= (1.0-wup) * box.z.rho[i1,ilow,*] + (wup) * box.z.rho[i1,iup,*]
;help, i1, reform(box.z.rho[i1,ind_low[i1],*]), reform(box.z.rho[i1,ind_up[ i1],*]), OutBox.z.rho[i1, 0, *]
;help, i1, reform(box.z.rho[i1,ilow,*]), reform(box.z.rho[i1,iup,*]), OutBox.z.rho[i1, 0, *]
;print, i1, box.z.rho[i1,ilow,*], box.z.rho[i1,iup,*], (1.0-wup) * box.z.rho[i1,ilow,*], (wup) * box.z.rho[i1,iup,*], OutBox.z.rho[i1, 0, *], format='(I3,5E15.6)'
      OutBox.z.ei[ i1, 0, *]= (1.0-wup) * box.z.ei[ i1,ilow,*] + (wup) * box.z.ei[ i1,iup,*]
      OutBox.z.v1[ i1, 0, *]=((1.0-wup) * box.z.v1[ i1,ilow,*] + (wup) * box.z.v1[ i1,iup,*]) *   w1 + $
                             ((1.0-wup) * box.z.v2[ i1,ilow,*] + (wup) * box.z.v2[ i1,iup,*]) *   w2
      OutBox.z.v2[ i1, 0, *]=((1.0-wup) * box.z.v1[ i1,ilow,*] + (wup) * box.z.v1[ i1,iup,*]) * (-w2) + $
                             ((1.0-wup) * box.z.v2[ i1,ilow,*] + (wup) * box.z.v2[ i1,iup,*]) *   w1
      OutBox.z.v3[ i1, 0, *]= (1.0-wup) * box.z.v3[ i1,ilow,*] + (wup) * box.z.v3[ i1,iup,*]
    endfor ; i1
    ;
    if (nquc ge 1) then begin
      for i1=0,n1-m1 do begin
        ilow=(ind_low[i1])[0]
        iup =(ind_up[ i1])[0]
        wup =(w_up[   i1])[0]
        OutBox.z.quc001[i1, 0, *]= (1.0-wup) * box.z.quc001[i1,ilow,*] + (wup) * box.z.quc001[i1,iup,*]
      endfor ; i1
    endif
    if (nquc ge 2) then begin
      for i1=0,n1-m1 do begin
        ilow=(ind_low[i1])[0]
        iup =(ind_up[ i1])[0]
        wup =(w_up[   i1])[0]
        OutBox.z.quc002[i1, 0, *]= (1.0-wup) * box.z.quc002[i1,ilow,*] + (wup) * box.z.quc002[i1,iup,*]
      endfor ; i1
    endif
    if (nquc ge 3) then begin
      for i1=0,n1-m1 do begin
        ilow=(ind_low[i1])[0]
        iup =(ind_up[ i1])[0]
        wup =(w_up[   i1])[0]
        OutBox.z.quc003[i1, 0, *]= (1.0-wup) * box.z.quc003[i1,ilow,*] + (wup) * box.z.quc003[i1,iup,*]
      endfor ; i1
    endif
    if (nquc ge 4) then begin
      for i1=0,n1-m1 do begin
        ilow=(ind_low[i1])[0]
        iup =(ind_up[ i1])[0]
        wup =(w_up[   i1])[0]
        OutBox.z.quc004[i1, 0, *]= (1.0-wup) * box.z.quc004[i1,ilow,*] + (wup) * box.z.quc004[i1,iup,*]
      endfor ; i1
    endif
    ;
  endif
  ;
endif else if ((NormalId eq '+00') or (NormalId eq '-00')) then begin ;=============================
  ;
  OutBox=chunkbox(box, [[0, 0], [-99999, 99999], [-99999, 99999]])
  ;
endif else if ((NormalId eq '++0') or (NormalId eq '+-0') or $
               (NormalId eq '+0+') or (NormalId eq '+0-') or $
               (NormalId eq '0++') or (NormalId eq '0+-')     ) then begin ;========================
  ;
  ; --- Create box with appropriate dimensions by calling chunkbox and scale the axes ---
  if ((NormalId eq '++0') or (NormalId eq '+-0')) then begin
    OutBox=chunkbox(box, [[-99999, 99999], [0, 0], [-99999, 99999]])
    ;
    OutBox.z.xc1=   box.z.xc1*sqrt(2.0)
    OutBox.z.xb1=   box.z.xb1*sqrt(2.0)
    OutBox.z.xc2=OutBox.z.xc2*sqrt(0.5)
    OutBox.z.xb2=OutBox.z.xb2*sqrt(0.5)
    ;
  endif else if ((NormalId eq '+0+') or (NormalId eq '+0-')) then begin
    OutBox=chunkbox(box, [[-99999, 99999], [-99999, 99999], [0, 0]])
    ;
    OutBox.z.xc1=   box.z.xc1*sqrt(2.0)
    OutBox.z.xb1=   box.z.xb1*sqrt(2.0)
    OutBox.z.xc3=OutBox.z.xc3*sqrt(0.5)
    OutBox.z.xb3=Outbox.z.xb3*sqrt(0.5)
    ;
  endif else begin
    OutBox=chunkbox(box, [[-99999, 99999], [-99999, 99999], [0, 0]])
    ;
    OutBox.z.xc2=   box.z.xc2*sqrt(2.0)
    OutBox.z.xb2=   box.z.xb2*sqrt(2.0)
    OutBox.z.xc3=OutBox.z.xc3*sqrt(0.5)
    OutBox.z.xb3=OutBox.z.xb3*sqrt(0.5)
    ;
  endelse
  ;
  ; --- Extract bulk data ---
  if (NormalId eq '+-0') then begin ;---------------------------------------------------------------
    ;
    for i1=0,n1-m1 do begin
      OutBox.z.rho[i1, 0, *]= box.z.rho[i1, i1, *]
      OutBox.z.ei[ i1, 0, *]= box.z.ei[ i1, i1, *]
      OutBox.z.v1[ i1, 0, *]=(box.z.v1[ i1, i1, *] + box.z.v2[ i1, i1, *]) * sqrt(0.5)
      OutBox.z.v2[ i1, 0, *]=(box.z.v1[ i1, i1, *] - box.z.v2[ i1, i1, *]) * sqrt(0.5)
      OutBox.z.v3[ i1, 0, *]= box.z.v3[ i1, i1, *]
    endfor ; i1
    ;
    if (nquc ge 1) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc001[i1, 0, *]= box.z.quc001[i1, i1, *]
      endfor ; i1
    endif
    if (nquc ge 2) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc002[i1, 0, *]= box.z.quc002[i1, i1, *]
      endfor ; i1
    endif
    if (nquc ge 3) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc003[i1, 0, *]= box.z.quc003[i1, i1, *]
      endfor ; i1
    endif
    if (nquc ge 4) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc004[i1, 0, *]= box.z.quc004[i1, i1, *]
      endfor ; i1
    endif
    ;
  endif else if (NormalId eq '++0') then begin ;----------------------------------------------------
    ;
    for i1=0,n1-m1 do begin
      OutBox.z.rho[i1, 0, *]= box.z.rho[i1, n1-m1-i1, *]
      OutBox.z.ei[ i1, 0, *]= box.z.ei[ i1, n1-m1-i1, *]
      OutBox.z.v1[ i1, 0, *]=(box.z.v1[ i1, n1-m1-i1, *] + box.z.v2[ i1, n1-m1-i1, *]) * sqrt(0.5)
      OutBox.z.v2[ i1, 0, *]=(box.z.v1[ i1, n1-m1-i1, *] - box.z.v2[ i1, n1-m1-i1, *]) * sqrt(0.5)
      OutBox.z.v3[ i1, 0, *]= box.z.v3[ i1, n1-m1-i1, *]
    endfor ; i1
    ;
    if (nquc ge 1) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc001[i1, 0, *]= box.z.quc001[i1, n1-m1-i1, *]
      endfor ; i1
    endif
    if (nquc ge 2) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc002[i1, 0, *]= box.z.quc002[i1, n1-m1-i1, *]
      endfor ; i1
    endif
    if (nquc ge 3) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc003[i1, 0, *]= box.z.quc003[i1, n1-m1-i1, *]
      endfor ; i1
    endif
    if (nquc ge 4) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc004[i1, 0, *]= box.z.quc004[i1, n1-m1-i1, *]
      endfor ; i1
    endif
    ;
  endif else if (NormalId eq '+0-') then begin ;----------------------------------------------------
    ;
    for i1=0,n1-m1 do begin
      OutBox.z.rho[i1, *, 0]= box.z.rho[i1, *, i1]
      OutBox.z.ei[ i1, *, 0]= box.z.ei[ i1, *, i1]
      OutBox.z.v1[ i1, *, 0]=(box.z.v1[ i1, *, i1] + box.z.v3[ i1, *, i1]) * sqrt(0.5)
      OutBox.z.v2[ i1, *, 0]= box.z.v2[ i1, *, i1]
      OutBox.z.v3[ i1, *, 0]=(box.z.v1[ i1, *, i1] - box.z.v3[ i1, *, i1]) * sqrt(0.5)
    endfor ; i1
    ;
    if (nquc ge 1) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc001[i1, *, 0]= box.z.quc001[i1, *, i1]
      endfor ; i1
    endif
    if (nquc ge 2) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc002[i1, *, 0]= box.z.quc002[i1, *, i1]
      endfor ; i1
    endif
    if (nquc ge 3) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc003[i1, *, 0]= box.z.quc003[i1, *, i1]
      endfor ; i1
    endif
    if (nquc ge 4) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc004[i1, *, 0]= box.z.quc004[i1, *, i1]
      endfor ; i1
    endif
    ;
  endif else if (NormalId eq '+0+') then begin ;----------------------------------------------------
    ;
    for i1=0,n1-m1 do begin
      OutBox.z.rho[i1, *, 0]= box.z.rho[i1, *, n1-m1-i1]
      OutBox.z.ei[ i1, *, 0]= box.z.ei[ i1, *, n1-m1-i1]
      OutBox.z.v1[ i1, *, 0]=(box.z.v1[ i1, *, n1-m1-i1] + box.z.v3[ i1, *, n1-m1-i1]) * sqrt(0.5)
      OutBox.z.v2[ i1, *, 0]= box.z.v2[ i1, *, n1-m1-i1]
      OutBox.z.v3[ i1, *, 0]=(box.z.v1[ i1, *, n1-m1-i1] - box.z.v3[ i1, *, n1-m1-i1]) * sqrt(0.5)
    endfor ; i1
    ;
    if (nquc ge 1) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc001[i1, *, 0]= box.z.quc001[i1, *, n1-m1-i1]
      endfor ; i1
    endif
    if (nquc ge 2) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc002[i1, *, 0]= box.z.quc002[i1, *, n1-m1-i1]
      endfor ; i1
    endif
    if (nquc ge 3) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc003[i1, *, 0]= box.z.quc003[i1, *, n1-m1-i1]
      endfor ; i1
    endif
    if (nquc ge 4) then begin
      for i1=0,n1-m1 do begin
        OutBox.z.quc004[i1, *, 0]= box.z.quc004[i1, *, n1-m1-i1]
      endfor ; i1
    endif
    ;
  endif else if (NormalId eq '0+-') then begin ;----------------------------------------------------
    ;
    for i2=0,n2-m2 do begin
      OutBox.z.rho[*, i2, 0]= box.z.rho[i2, i2, *]
      OutBox.z.ei[ *, i2, 0]= box.z.ei[ i2, i2, *]
      OutBox.z.v1[ *, i2, 0]= box.z.v1[ i2, i2, *]
      OutBox.z.v2[ *, i2, 0]=(box.z.v2[ i2, i2, *] + box.z.v3[ i2, i2, *]) * sqrt(0.5)
      OutBox.z.v3[ *, i2, 0]=(box.z.v2[ i2, i2, *] - box.z.v3[ i2, i2, *]) * sqrt(0.5)
    endfor ; i2
    ;
    if (nquc ge 1) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc001[*, i2, 0]= box.z.quc001[i2, i2, *]
      endfor ; i2
    endif
    if (nquc ge 2) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc002[*, i2, 0]= box.z.quc002[i2, i2, *]
      endfor ; i2
    endif
    if (nquc ge 3) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc003[*, i2, 0]= box.z.quc003[i2, i2, *]
      endfor ; i2
    endif
    if (nquc ge 4) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc004[*, i2, 0]= box.z.quc004[i2, i2, *]
      endfor ; i2
    endif
    ;
  endif else if (NormalId eq '0++') then begin ;----------------------------------------------------
    ;
    for i2=0,n2-m2 do begin
      OutBox.z.rho[*, i2, 0]= box.z.rho[*, i2, n2-m2-i2]
      OutBox.z.ei[ *, i2, 0]= box.z.ei[ *, i2, n2-m2-i2]
      OutBox.z.v1[ *, i2, 0]= box.z.v1[ *, i2, n2-m2-i2]
      OutBox.z.v2[ *, i2, 0]=(box.z.v2[ *, i2, n2-m2-i2] + box.z.v3[ *, i2, n2-m2-i2]) * sqrt(0.5)
      OutBox.z.v3[ *, i2, 0]=(box.z.v2[ *, i2, n2-m2-i2] - box.z.v3[ *, i2, n2-m2-i2]) * sqrt(0.5)
    endfor ; i2
    ;
    if (nquc ge 1) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc001[*, i2, 0]= box.z.quc001[*, i2, n2-m2-i2]
      endfor ; i2
    endif
    if (nquc ge 2) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc002[*, i2, 0]= box.z.quc002[*, i2, n2-m2-i2]
      endfor ; i2
    endif
    if (nquc ge 3) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc003[*, i2, 0]= box.z.quc003[*, i2, n2-m2-i2]
      endfor ; i2
    endif
    if (nquc ge 4) then begin
      for i2=0,n2-m2 do begin
        OutBox.z.quc004[*, i2, 0]= box.z.quc004[*, i2, n2-m2-i2]
      endfor ; i2
    endif
    ;
  endif
  ;
endif else begin

message, 'Type of normal not supported.'

;if (total(tag_names(box) eq 'EOS') gt 0.0) then begin
;  ; --- Append EOS data to box ---
;  eosdata=create_struct( $
;    'P'     , reform(box.eos.P[     m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
;    'dPdrho', reform(box.eos.dPdrho[m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
;    'dPdei' , reform(box.eos.dPdei[ m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
;    'T'     , reform(box.eos.T[     m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
;    'dTdei' , reform(box.eos.dTdei[ m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1), $
;    's'     , reform(box.eos.s[     m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1)   )
;  OutBox=create_struct(temporary(OutBox), 'eos', temporary(eosdata))
;endif

;if (total(tag_names(box) eq 'OPA') gt 0.0) then begin
;  ; --- Append opacity data to box ---
;  opadata=create_struct( $
;    'kappa' , reform(box.opa.kappa[ m1:n1   ,m2:n2   ,m3:n3  ], n1-m1+1, n2-m2+1, n3-m3+1))
;  OutBox=create_struct(temporary(OutBox), 'opa', temporary(opadata))
;endif

endelse



return, OutBox

end ; slicebox
