function spheregrav, r, r0=r0, r1=r1, r2=r2, C=C, gravCorePotDepth=gravCorePotDepth
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   spheregrav ('sphere_grav')
;
; PURPOSE:
;   Compute spherical potential with various approximations
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;    result=spheregrav(r, r0=r0, r1=r1, r2=r2, C=C, gravCorePotDepth=gravCorePotDepth)
;
; INPUT:
;   r:            ('radius') real 1D vector with radius values
;
; OUTPUT:
;   spherepot:    ('sphere_potential') real array of same shape as r with output potential
;
; KEYWORDS:
;   r0:           ('radius_0') real parameter with inner smoothing radius
;   r1:           ('radius_1') real parameter with outer smoothing radius
;   r2:           ('radius_2') real parameter
;   C:            ('Constant') real constant, usually G_grav*M_star
;   gravCorePotDepth: ('gravity_Core_Potential_Depth') real with deepening factor for ^1/4 potential
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   spherepot: Compute spherical potential in various approximations
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The input array r has to be a scalar or a 1D vector.
;
; PROCEDURE:
;   Compute the numerical derivative of the potential.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-11-25 (B.F. Uppsala) First (recorded) version: approximate potential numerically
;   2012-02-05 (B.F. Lyon) Compute gravity for one case analytically
;   2013-07-12 (W.S.) Outcomment case with analytic computation
;   2018-04-18 (B.F. Uppsala) Add comments; replace some () with []
;   2018-04-19 (B.F.) Add keyword gravDeep
;   2018-04-25 (B.F.) Rename parameter gravDeep -> gravCorePotDepth
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

grav=r*0.0
;
indmin=(where(r gt 0.0))[0]
;
;if (indmin ge 0) then begin
;  if ((keyword_set(mode)) and (r0 gt 0.0) and (r1 gt 0.0)) then begin
;    rovr12             =(r/r1)^2
;    r1ovr04            =((r1/r0)^2)^2
;    quadroot           =sqrt(sqrt(1.0 + $
;                                  r1ovr04 * sqrt((rovr12^2)^2 / (1.0 + (rovr12^2)^2)) $
;                                 ))
;    grav=C * rovr12*sqrt(rovr12) * r1ovr04 / $
;         (r0*r1 * quadroot^5 * sqrt(1.0 + (rovr12^2)^2)^3)
;  endif else begin
;  endelse
;endif
;
if (indmin ge 0) then begin
  rsub=r[indmin:*]
  if (n_elements(rsub) eq 1) then begin
    rsub=[0.98, 1.0, 1.02]*rsub[0]
    phi=spherepot(rsub, r0=r0, r1=r1, r2=r2, C=C, gravCorePotDepth=gravCorePotDepth)
    grav[indmin:*]=(deriv(rsub, phi))[1]
  endif else begin
    phi=spherepot(rsub, r0=r0, r1=r1, r2=r2, C=C, gravCorePotDepth=gravCorePotDepth)
    ;
    grav[indmin:*]=deriv(rsub, phi)
  endelse
endif
;
return, grav
;
end ; spheregrav
