function spherepot, r, r0=r0, r1=r1, r2=r2, C=C, gravCorePotDepth=gravCorePotDepth
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   spherepot ('sphere_potential')
;
; PURPOSE:
;   Compute spherical potential in various approximations
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;    result=spherepot(r, r0=r0, r1=r1, r2=r2, C=C, gravCorePotDepth=gravCorePotDepth)
;
; INPUT:
;   r:            ('radius') real array with radius values
;
; OUTPUT:
;   spherepot:    ('sphere_potential') real array of same shape as r with output potential
;
; KEYWORDS:
;   r0:           ('radius_0') real parameter with inner smoothing radius
;   r1:           ('radius_1') real parameter with outer smoothing radius
;   r2:           ('radius_2') real parameter
;   C:            ('Constant') real constant, usually G_grav*M_star
;   gravCorePotDepth: ('gravity_Core_Potential_Depth') real with deepening factor for ^1/4 potential
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-11-25 (B.F. Uppsala) First (recorded) version
;   2012-02-06 (B.F. Lyon) Rearrange terms in formula
;   2013-07-12 (W.S.) Add potential controlled with r2
;   2018-04-17 (B.F. Uppsala) Add comments
;   2018-04-18 (B.F.) Some rearrangements
;   2018-04-19 (B.F.) Add keyword gravDeep and new potential
;   2018-04-25 (B.F.) Rename parameter gravDeep -> gravCorePotDepth
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

; --- Set default values for keywords, if necessary ---
if (n_elements(r0) eq 0) then begin
  r0=0.0
endif
if (n_elements(r1) eq 0) then begin
  r1=0.0
endif
if (n_elements(r2) eq 0) then begin
  r2=0.0
endif
if (n_elements(C) eq 0) then begin
  C=1.0
endif
if (n_elements(gravCorePotDepth) eq 0) then begin
  gravCorePotDepth=0.0
endif
;
; --- Compute the potential ---
if (r0 eq 0.0) then begin
  ;print, 'Raw 1/r potential'
  ;
  phi=-C/r
  ;
endif else if ((r1 eq 0.0) and (r2 eq 0.0)) then begin
  ;print, 'Old smoothed potential'
  ;
  ;phi=-C/sqrt(r0^2 + r^2)
  phi=-C/(r0*sqrt(1.0 + (r/r0)^2))
  ;
endif else if ((r1 eq 0.0) and (r2 gt 0.0)) then begin
  ;print, 'Smoothed potential for M dwarfs'
  ;
  n1=2.0
  n2=8.0
  phi=-C/(r0*(1.0 + ((r/r0)/(1.0+(r/r2)^n2)^(1.0/n2))^n1)^(1.0/n1))
  ;
endif else if ((gravCorePotDepth-1.0) gt 0.0) then begin
  ;print, 'Smoothed potential with central deepening for RSGs'
  ;
  d1=1.0/(gravCorePotDepth > 1.0)
;  phi=-C/(r0*(sqrt(sqrt(d1^4 + $
;                        ((r/r0)^12/((1.0-d1)+(r/r0)^8) + $
;                         0.5*(1.0-d1)*d1^3*(r/r0)^2      ) / $
;                        sqrt(1.0 + (r/r1)^8) $
;                       ))))
  phi=-C/(r0*(sqrt(sqrt(d1^4 + $
                        ((r/r0)^8/((1.0-d1)+(r/r0)^4) + $
                         0.5*(1.0-d1)*d1^3*(r/r0)^2      ) / $
                        sqrt(1.0 + (r/r1)^8) $
                       ))))
  ;
endif else begin
  ;print, 'Smoothed potential with inner and outer smoothing radius for RSGs'
  ;
  ; --- Large r1: phi=-C/(r0*(1.0 + (r/r0)^4)^0.25)
  ; ---              =-C/(r0^4 + r^4)^0.25
  ; ---              =-C/sqrt(sqrt((r0^4 + r^4))
  ; --- Previous: phi=-C/(r0*sqrt(sqrt(1.0 + (r1/r0)^4 * sqrt( (r/r1)^8/(1.0+(r/r1)^8) ) )))
  ; ---           phi=-C/(r0*(1.0 + (r1/r0)^4 * ( (r/r1)^8/(1.0+(r/r1)^8) )^0.5 )^0.25)
  ;
  n=4
  phi=-C/(r0*(1.0 + (r1/r0)^4 * ( (r/r1)^(2*n)/(1.0+(r/r1)^(2*n)) )^(2.0/float(n)) )^0.25)
  ;
endelse
;
return, phi
;
end ; spherepot
