pro stratintegrate, ei0, rho0, z0, ei, rho, z, $
  dz=dz0, grav=grav0, npoint=npoint0, eos=EOS
;------------------------------------------------------------------------------
;+
; NAME:
;   stratintegrate ('stratification_integrate')
;
; PURPOSE:
;   Integrate stratification of density and internal energy
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;   stratintegrate, ei0, rho0, z0, ei, rho, z, $
;     dz=dz0, grav=grav0, npoint=npoint0, eos=EOS
;
; INPUT:
;   ei0:    ('energy_internal_0') real scalar with starting point for internal energy
;   rho0:   ('rho_0') real scalar with starting point for density
;   z0:     ('z_0') real scalar with starting point for geometrical axis
;
; OUTPUT:
;   ei:     ('energy_internal') 1D array with energy stratification
;   rho:    ('rho') 1D array with density stratification
;   z:      ('z') 1D array with geometrical axis
;
; KEYWORDS:
;   dz:     ('delta_z') real scalar with geometrical step size
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   eosinter: Compute equation of state
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The integration is done with a simple 2-step scheme. Use many points!
;
; PROCEDURE:
;   Use the hydrostatic equation and assume constant entropy
;     dP/dz = - grav rho dz
;     ds/dz = 0
;   to integrate density and internal
;      drho/dz = drho/dP|s * dP/dz
;      dei/dz  = dei/dP|s * dP/dz
;   by using
;     drho/dP|s = rho/(P*Gamma_1) = 1/cs^2
;     dei/dP|s  = 1/(rho*Gamma1)
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2007-01-05 Written by Bernd Freytag, CRAL-ENS Lyon
;   2007-01-09 (B.F.) Second-order scheme
;-
;------------------------------------------------------------------------------

; --- Default values for optional keywords ---
if (keyword_set(dz0)) then begin
  dz=dz0
endif else begin
  dz=-1.0
endelse
if (keyword_set(grav0)) then begin
  grav=-abs(grav0)
endif else begin
  grav=0.0
endelse
if (keyword_set(npoint0)) then begin
  npoint=long(npoint0) > 1L
endif else begin
  npoint=100L
endelse

help, dz, npoint, grav

; --- Output arrays ---
ei =fltarr(npoint)
rho=fltarr(npoint)
z  =fltarr(npoint)
ei[npoint-1] =ei0
rho[npoint-1]=rho0
z[  npoint-1]=z0

mu=1.0

; --- Loop over all grid points to be integrated ---
for j=npoint-2L,0L,-1L do begin
  ; --- First step with half size ---
  ;
  ; --- Compute EOS ---
  eosinter, [rho[j+1]], [ei[j+1]], P, dPdrho, dPdei, T, dTdei, s, eos=EOS
  Gamma_1l =dPdrho*rho[j+1]/P + dPdei/rho[j+1]
  if (j eq (npoint-2L)) then begin
    Gamma_1=Gamma_1l
  endif else begin
    Gamma_1=mu*Gamma_1l + (1.0-mu)*Gamma_1
  endelse
  ;Gamma_3=dPdei/rho[j+1] + 1.0
  ;drhodP_s=rho[j+1]/(P*Gamma_1)
  dlnrhodlnP_s=1.0/Gamma_1
  deidP_s     =1.0/(rho[j+1]*Gamma_1)
  ;
  ; --- Hydrostatic equation ---
  dPdz=rho[j+1]*grav
  dlnPdz=dPdz/P
  ;
  ; --- Integrate ---
  z_1  =z[  j+1] +                           (0.5*dz)
  rho_1=rho[j+1] * exp(dlnrhodlnP_s*dlnPdz * (0.5*dz))
  ei_1 =ei[ j+1] +      deidP_s *dPdz      * (0.5*dz)
  ;
  ; --- Second step with full size ---
  ;
  ; --- Compute EOS ---
  eosinter, rho_1, ei_1, P, dPdrho, dPdei, T, dTdei, s, eos=EOS
  Gamma_1l =dPdrho*rho_1/P + dPdei/rho_1
  if (j eq (npoint-2L)) then begin
    Gamma_1=Gamma_1l
  endif else begin
    Gamma_1=mu*Gamma_1l + (1.0-mu)*Gamma_1
  endelse
  ;Gamma_3=dPdei/rho_1 + 1.0
  ;drhodP_s=rho_1/(P*Gamma_1)
  dlnrhodlnP_s=1.0/Gamma_1
  deidP_s     =1.0/(rho_1*Gamma_1)
  ;
  ; --- Hydrostatic equation ---
  dPdz=rho_1*grav
  dlnPdz=dPdz/P
  ;
  ; --- Integrate ---
  z[  j]=z[  j+1] +                           dz
  ;rho[j]=rho[j+1] +    drhodP_s*dPdz        * dz
  rho[j]=rho[j+1] * exp(dlnrhodlnP_s*dlnPdz * dz)
  ei[ j]=ei[ j+1] +      deidP_s *dPdz      * dz
  ;
  if ((j mod 100L) eq 0L) then begin
    print, j, z[j], rho[j], ei[j], P, T, Gamma_1, format='(I5,5E13.5,2X,F9.5)'
  endif
endfor ; j

end ; stratintegrate
