function tau_int, kappa, z, istop, jstop, htau0, mode=mode
;
;------------------------------------------------------------------------------
;+
; NAME:
;   tau_int
;
; PURPOSE:  calculates optical depth
;
; CATEGORY: radiation transport
;
; CALLING SEQUENCE:
;
; INPUT:    kappa (istop, jstop): opacity in [cm^-1]
;           z(jstop)            : geometrical height in [cm]
;           istop               : dimension in x direction
;           jstop               : dimension in z direction
;           htau0               : optical depth scale height
;
; OUTPUT:   tau_int(istop,jstop): optical depth along vertical rays.
;
; KEYWORDS: mode:  0: linear kappa(z); 1: cubic kappa(z)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   xx.xx.xx Written by Matthias Steffen, AIP, Germany
;-
;------------------------------------------------------------------------------
;
if (not keyword_set(mode)) then mode=0
;
dz  =dblarr(jstop)
tau =dblarr(istop,jstop)
dkds=dblarr(istop,jstop)
dxds=dblarr(istop,jstop)
;
;tau(top)=kappa*H_tau0
tau(*,jstop-1)=kappa(*,jstop-1)*htau0
;
  if (mode eq 0) then begin
    for j=jstop-2,0,-1 do begin
      tau(*,j)=tau(*,j+1) + 0.5*(z(j+1)-z(j))*(kappa(*,j+1)+kappa(*,j))
    endfor
  endif
;
   if (mode ne 0) then begin
     for j=0,jstop-2 do begin
       dz(j)     = z(j+1)-z(j)
       dkds(*,j) = -(kappa(*,j+1)-kappa(*,j))/dz(j)
     endfor
;
;TOP:
;
     k1=jstop-1
     k2=jstop-2
     k3=jstop-3
     s3=(dkds(*,k2)*dz(k3)+dkds(*,k3)*dz(k2))/(2.0*(dz(k2)+dz(k3)))
     s4=s3<dkds(*,k2)<dkds(*,k3)
     s5=s3>dkds(*,k2)>dkds(*,k3)
     dxds(*,k1)=1.5*dkds(*,k2) - ((s4>0.)+(s5<0.))
;
;INTERIOR:
;
     for j=jstop-2,1,-1 do begin
       k1=j
       k2=j-1
       s3=(dkds(*,k1)*dz(k2)+dkds(*,k2)*dz(k1))/(2.0*(dz(k1)+dz(k2)))
       s4=s3<dkds(*,k1)<dkds(*,k2)
       s5=s3>dkds(*,k1)>dkds(*,k2)
       dxds(*,k1)=2.0*((s4>0.)+(s5<0.))
     endfor
;
;BOTTOM:
;
     dxds(*,0)=1.5*dkds(*,0) - 0.5*dxds(*,1)
;
     for j=jstop-2,0,-1 do begin
       tau(*,j)=tau(*,j+1) + $
                dz(j)*(0.5*(kappa(*,j+1)+kappa(*,j)) + $
                0.083333333*dz(j)*(dxds(*,j+1)-dxds(*,j)))
     endfor
;
   endif
;   
RETURN, FLOAT(tau)
;
end
