pro wrbox, model, file=file, form=form, conv=conv, position=position0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   wrbox ('write_box')
;
; PURPOSE:
;   Write RHD model box into file in uio form.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   wrbox, model, file=file [, form=form, conv=conv, position=position]
;
; INPUTS:
;   model:  ('model') structure of type 'box' with CO5BOLD full data set
;
; OUTPUTS:
;   None
;
; KEYWORDS:
;   file:     ('filename') character, filename
;   form:     ('format') character, 'formatted' or 'unformatted'
;   conv:     ('conversion') character, type of conversion procedure
;   position: ('position') character, position for writing:
;               'asis':   Create new file, delete existing one (default)
;               'append': Append data to existing file
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model box into file in uio form
;
; SIDE EFFECTS:
;   Write data into file.
;
; RESTRICTIONS:
;   uio routines have to be initialized.
;   The names and units of 'quc' variables are not set properly.
;
; PROCEDURE:
;   The Routine rhd_wrboxdata is called with appropriate parameters.
;
; EXAMPLE:
;   ful=uio_dataset_rd('SUN_3D.end')
;   wrbox, ful, file='copy_of_SUN_3D.end'
;
; MODIFICATION HISTORY:
;   2003-07-24 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2005-06-02 (B.F. LA) version: integer -> character
;-
;------------------------------------------------------------------------------
;
; --- Default value for keyword ---
if (not keyword_set(file)) then file='idl.full'
if (not keyword_set(form)) then form='unformatted'
if (not keyword_set(position0)) then begin
  position='asis'
endif else begin
  position='append'
endelse
if (position eq 'append') then begin
  command='append,model,box,endmo,close'
endif else begin
  command='open,model,box,endmo,close'
endelse
;
; --- Indices from dimension array ---
m1=model.z.dimension(0,0)
n1=model.z.dimension(1,0)
m2=model.z.dimension(0,1)
n2=model.z.dimension(1,1)
m3=model.z.dimension(0,2)
n3=model.z.dimension(1,2)
;
; --- Physical units ---
unit_time='s'
unit_x1  ='cm'
unit_x2  =unit_x1
unit_x3  =unit_x1
unit_rho ='g/cm^3'
unit_v   ='cm/s'
unit_e   ='erg/g'
name_quc =['']
unit_quc =['']
unit_B   ='G'
;
tags=tag_names(model.head)
if (total(tags eq 'DESCRIPTION') gt 0.0) then begin
  description=           model.head.description
  ndes       =n_elements(model.head.description)
endif else begin
  description=['']
  ndes       =0
endelse
if (total(tags eq 'HISTORY') gt 0.0) then begin
  history=           model.head.history
  nhis   =n_elements(model.head.history)
endif else begin
  history=['']
  nhis   =0
endelse
if (total(tags eq 'VERSION') gt 0.0) then begin
  version=model.head.version
endif else begin
  version='0'
endelse
;
if (keyword_set(quc)) then begin
  if (keyword_set(Bb1)) then begin
    rhd_wrboxdata, command, $
      -1, ncout, file=file, form=form, conv=conv, $
      ndes=ndes, description=description, $
      nhis=nhis, history=history, $
      version=version, $
      unit_time=unit_time, $
      unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
      unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
      unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
      m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
      itime=model.z.itime, time=model.z.time, dtime=model.dtime, $
      out_time_full_last=model.time_out_full_last, out_time_mean_last=model.time_out_mean_last, $
      xc1=model.z.xc1, xc2=model.z.xc2, xc3=model.z.xc3, $
      xb1=model.z.xb1, xb2=model.z.xb2, xb3=model.z.xb3, $
      rho=model.z.rho, v1=model.z.v1, v2=model.z.v2, v3=model.z.v3, ei=model.z.ei, $
      quc=model.z.quc, Bb1=model.z.Bb1, Bb2=model.z.Bb2, Bb3=model.z.Bb3
  endif else begin
    rhd_wrboxdata, command, $
      -1, ncout, file=file, form=form, conv=conv, $
      ndes=ndes, description=description, $
      nhis=nhis, history=history, $
      version=version, $
      unit_time=unit_time, $
      unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
      unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
      unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
      m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
      itime=model.z.itime, time=model.z.time, dtime=model.dtime, $
      out_time_full_last=model.time_out_full_last, out_time_mean_last=model.time_out_mean_last, $
      xc1=model.z.xc1, xc2=model.z.xc2, xc3=model.z.xc3, $
      xb1=model.z.xb1, xb2=model.z.xb2, xb3=model.z.xb3, $
      rho=model.z.rho, v1=model.z.v1, v2=model.z.v2, v3=model.z.v3, ei=model.z.ei, $
      quc=model.z.quc
  endelse
endif else begin
  if (keyword_set(Bb1)) then begin
    rhd_wrboxdata, command, $
      -1, ncout, file=file, form=form, conv=conv, $
      ndes=ndes, description=description, $
      nhis=nhis, history=history, $
      version=version, $
      unit_time=unit_time, $
      unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
      unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
      unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
      m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
      itime=model.z.itime, time=model.z.time, dtime=model.dtime, $
      out_time_full_last=model.time_out_full_last, out_time_mean_last=model.time_out_mean_last, $
      xc1=model.z.xc1, xc2=model.z.xc2, xc3=model.z.xc3, $
      xb1=model.z.xb1, xb2=model.z.xb2, xb3=model.z.xb3, $
      rho=model.z.rho, v1=model.z.v1, v2=model.z.v2, v3=model.z.v3, ei=model.z.ei, $
      Bb1=model.z.Bb1, Bb2=model.z.Bb2, Bb3=model.z.Bb3
  endif else begin
    rhd_wrboxdata, command, $
      -1, ncout, file=file, form=form, conv=conv, $
      ndes=ndes, description=description, $
      nhis=nhis, history=history, $
      version=version, $
      unit_time=unit_time, $
      unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
      unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
      unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
      m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
      itime=model.z.itime, time=model.z.time, dtime=model.dtime, $
      out_time_full_last=model.time_out_full_last, out_time_mean_last=model.time_out_mean_last, $
      xc1=model.z.xc1, xc2=model.z.xc2, xc3=model.z.xc3, $
      xb1=model.z.xb1, xb2=model.z.xb2, xb3=model.z.xb3, $
      rho=model.z.rho, v1=model.z.v1, v2=model.z.v2, v3=model.z.v3, ei=model.z.ei
  endelse
endelse
;
end ; wrbox
