function xkaros, P, T, iBand, Opa=Opa, grid=grid, clip=clip, spline=spline
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   xkaros ('x_kappa_rosseland')
;
; PURPOSE:
;   Interpolate opacities from data in opta_common.
;
; CATEGORY:
;   Opacities, Interpolation
;
; CALLING SEQUENCE:
;   kappa=xkaros(P, T, iBand, grid=grid, clip=clip)
;
; INPUTS:
;   P:            ('Pressure') real array with pressure data
;   T:            ('Temperature') real array with temperature data
;   iBand:        ('index_Band') integer
;
; OUTPUTS:
;   xkaros:       ('x_kappa_rosseland') real array with opacity data (per mass)
;
; KEYWORDS:
;   Opa:          ('Opacity') If supplied, the data in this structure is used for the interpolation
;   grid:         ('grid') If set, compute a 2D grid for each combination of input P and T 1D vectors.
;   clip:         ('clip') If not set, in case of extrapolation the outermost values are used (0 or n-1);
;                          if set, the values 1 gridpoint away from the boundary are used (1 or n-2).
;   spline:       ('spline') If set, a higher order interpolation is used.
;
; COMMON BLOCKS:
;   opta_common
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None: read-only from common block.
;
; RESTRICTIONS:
;   Only linear interpolation is used per default.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;              Written by Hans-Guenter Ludwig
;   1999-06-30 (B.F. Copenhagen) Add keyword 'clip'
;   1999-10-11 (B.F.) Return data in same format as T
;   2014-06-03 (B.F. Uppsala) Add more comments
;   2015-08-25 (B.F.) Add keywords 'spline' and 'Opa'
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr

; --- Access common block ---
common opta_common

; --- Handle optional keywords ---
if (keyword_set(spline)) then begin
  cubic=-0.5
endif else begin
  cubic=0.0
endelse

; --- Check if opacity data is supplied in input structure ---
if (keyword_set(opa) and (size(opa, /type) eq 8)) then begin
  ; --- Use data in supplied structure for interpolation -----------------------
  ;
  ; --- Determine index coordinates ---
  iT = interpol(findgen(Opa.nT), Opa.TabT, alog10(T), spline=spline)
  iP = interpol(findgen(Opa.nP), Opa.TabP, alog10(P), spline=spline)
  ;
  ; --- Delimit coordinates ---
  if (keyword_set(clip)) then begin
    iT = 1.0 > iT < float(Opa.nT-2)
    iP = 1.0 > iP < float(Opa.nP-2)
  endif
  ;
  ; --- Compute opacities ---
  if not keyword_set(grid) then begin
    ; --- Compute opacities for all input P/T combinations ---
    xka   =T
    xka[*]=10.0^interpolate(Opa.xopta[*, *, iBand], iT, iP, cubic=cubic)
    ;
  endif else begin
    ; --- Compute opacities for a rectangular grid ---
    xka   =fltarr(n_elements(iT), n_elements(iP))
    xka[*]=10.0^interpolate(Opa.xopta[*, *, iBand], iT, iP, cubic=cubic, /grid)
    ;
  endelse
  ;
endif else begin
  ; --- Use data in common block for interpolation -----------------------------
  ;
  ; --- Determine index coordinates ---
  iT = interpol(findgen(nT), Topta, alog10(T), spline=spline)
  iP = interpol(findgen(nP), Popta, alog10(P), spline=spline)
  ;
  ; --- Delimit coordinates ---
  if (keyword_set(clip)) then begin
    iT = 1.0 > iT < float(nT-2)
    iP = 1.0 > iP < float(nP-2)
  endif
  ;
  ; --- Compute opacities ---
  if not keyword_set(grid) then begin
    ; --- Compute opacities for all input P/T combinations ---
    xka   =T
    xka[*]=10.0^interpolate(xopta[*, *, iBand], iT, iP, cubic=cubic)
    ;
  endif else begin
    ; --- Compute opacities for a rectangular grid ---
    xka   =fltarr(n_elements(iT), n_elements(iP))
    xka[*]=10.0^interpolate(xopta[*, *, iBand], iT, iP, cubic=cubic, /grid)
    ;
  endelse
  ;
endelse

return, xka

end ; xkaros
