pro bubble2d_operator, u, dtdx=dtdx0, v1=v1, v2=v2
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bubble2d_operator ('bubble_2_dimension_operator')
;
; PURPOSE:
;   Solve "bubble equation" in 2D for single time step.
;
; CATEGORY:
;   Hydrodynamics, Plotting.
;
; CALLING SEQUENCE:
;   bubble2d_operator, u, dtdx=dtdx0, v1=v1, v2=v2
;
; INPUT/OUTPUT:
;   u:    ('u') 2D real array with bubble amplitude
;
; KEYWORDS:
;   dtdx: ('delta_time_delta_x') real, normalized time step
;   v1:   ('velocity_1') 2D real array with velocity data in x1 direction
;   v2:   ('velocity_2') 2D real array with velocity data in x2 direction
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Advance u one time step by applyind 2D bubble operator.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20. 3.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   23. 3.01 (B.F.) Perform only one time step
;-
;------------------------------------------------------------------------------
;
; --- Coefficients ---
;C0=0.2 & C2=1.0 & C4=0.35 & Cm=0.6 & dtdx_limit=0.11
;C0=0.2 & C2=1.0 & C4=0.30 & Cm=0.2 & dtdx_limit=0.15
;C0=0.2 & C2=1.0 & C4=0.30 & Cm=0.6 & dtdx_limit=0.15
C0=0.25 & C2=1.2 & C4=0.35 & Cm=0.6 & dtdx_limit=0.1
;
n1=n_elements(u(*,0))
n2=n_elements(u(0,*))
;
if (not keyword_set(v1)) then begin
  v1=u*0.0
endif
if (not keyword_set(v2)) then begin
  v2=u*0.0
endif
if (keyword_set(dtdx0)) then begin
  dtdx=abs(dtdx0) < dtdx_limit
endif else begin
  dtdx=dtdx_limit
endelse
;
; --- Derivatives ---
dudx1_p  =shift(u     ,-1, 0)-u
dudx1_m  =u                  -shift(u     , 1, 0)
dudx1    =0.5*(dudx1_p+dudx1_m)
dudx2_p  =shift(u     , 0,-1)-u
dudx2_m  =u                  -shift(u     , 0, 1)
dudx2    =0.5*(dudx2_p+dudx2_m)
d2udx1_2=dudx1_p             -dudx1_m
d2udx2_2=dudx2_p             -dudx2_m 
d2udx_2 =d2udx1_2 + d2udx2_2
d4udx_4 =shift(d2udx_2,-1, 0) + shift(d2udx_2, 1, 0) + $
         shift(d2udx_2, 0,-1) + shift(d2udx_2, 0, 1) - 4*d2udx_2
;
; --- Update ---
;u=u - dtdx * (v1*dudx1 + v2*dudx2 + C4*d4udx_4 + C2*d2udx_2 + C0*u^3)
;u=u - dtdx * (v1*dudx1 + v2*dudx2 + C4*d4udx_4 + C2*d2udx_2*((u>0.0)/(1.0E-04+abs(u))) + C0*u^3)
;u=u - dtdx * (v1*dudx1 + v2*dudx2 + C4*d4udx_4 + C2*d2udx_2 + C0*u^3 + Cm*(d2udx1_2-d2udx2_2)^2*(u>0.0))
u=u - dtdx * (v1*dudx1 + v2*dudx2 + C4*d4udx_4 + C2*d2udx_2*((u>0.0)/(1.0E-04+abs(u))) + C0*u^3 + Cm*(d2udx1_2-d2udx2_2)^2*(u>0.0))
;
end ; bubble2d_operator
