pro bubble3d_operator, u, dtdx=dtdx0, v1=v1, v2=v2, v3=v3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bubble3d_operator ('bubble_3_dimension_operator')
;
; PURPOSE:
;   Solve "bubble equation" in 3D for single time step.
;
; CATEGORY:
;   Hydrodynamics, Plotting.
;
; CALLING SEQUENCE:
;   bubble3d_operator, u, dtdx=dtdx0, v1=v1, v2=v2, v3=v3
;
; INPUT/OUTPUT:
;   u:    ('u') 3D real array with bubble amplitude
;
; KEYWORDS:
;   dtdx: ('delta_time_delta_x') real, normalized time step
;   v1:   ('velocity_1') 3D real array with velocity data in x1 direction
;   v2:   ('velocity_2') 3D real array with velocity data in x2 direction
;   v3:   ('velocity_3') 3D real array with velocity data in x3 direction
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Advance u one time step by applyind 3D bubble operator.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20. 3.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   23. 3.01 (B.F.) Perform only one time step
;   12. 4.01 (B.F.) 3D version
;-
;------------------------------------------------------------------------------
;
; --- Coefficients ---
;C0=0.2 & C2=1.0 & C4=0.35 & Cm=0.6 & dtdx_limit=0.11
;C0=0.2 & C2=1.0 & C4=0.30 & Cm=0.2 & dtdx_limit=0.15
;C0=0.2 & C2=1.0 & C4=0.30 & Cm=0.6 & dtdx_limit=0.15
;C0=0.25 & C2=1.2 & C4=0.35 & Cm=0.6 & dtdx_limit=0.1
;C0=0.25 & C2=1.2 & C4=0.25 & Cm=0.4 & dtdx_limit=0.05
C0=0.25 & C2=1.2 & C4=0.25 & Cm=0.4 & dtdx_limit=0.08
;
n1=n_elements(u(*,0,0))
n2=n_elements(u(0,*,0))
n3=n_elements(u(0,0,*))
;
if (not keyword_set(v1)) then begin
  v1=u*0.0
endif
if (not keyword_set(v2)) then begin
  v2=u*0.0
endif
if (not keyword_set(v3)) then begin
  v3=u*0.0
endif
if (keyword_set(dtdx0)) then begin
  dtdx=abs(dtdx0) < dtdx_limit
endif else begin
  dtdx=dtdx_limit
endelse
;
; --- Derivatives ---
dudx_p   =shift(u     ,-1, 0, 0)-u
dudx_m   =u                     -shift(u     , 1, 0, 0)
dudx1    =0.5*(dudx_p+dudx_m)
d2udx1_2 =dudx_p                -dudx_m
dudx_p   =shift(u     , 0,-1, 0)-u
dudx_m   =u                     -shift(u     , 0, 1, 0)
dudx2    =0.5*(dudx_p+dudx_m)
d2udx2_2 =dudx_p                -dudx_m 
dudx_p   =shift(u     , 0, 0,-1)-u
dudx_m   =u                     -shift(u     , 0, 0, 1)
dudx3    =0.5*(dudx_p+dudx_m)
d2udx3_2 =dudx_p                -dudx_m 
;
d2udx_2 =d2udx1_2 + d2udx2_2 + d2udx3_2
;
d4udx_4 =shift(d2udx_2,-1, 0, 0) + shift(d2udx_2, 1, 0, 0) + $
         shift(d2udx_2, 0,-1, 0) + shift(d2udx_2, 0, 1, 0) + $
         shift(d2udx_2, 0, 0,-1) + shift(d2udx_2, 0, 0, 1) - 6.0*d2udx_2
;kernel4=[[[0.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 0.0]], $
;         [[0.0, 1.0, 0.0], [1.0,-6.0, 1.0], [0.0, 1.0, 0.0]], $
;         [[0.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 0.0]]]
;d4udx_4=convol(d2udx_2, kernel4, /edge_wrap, /center)
;
; --- Update ---
u=u - dtdx * (v1 * dudx1 + v2 * dudx2 + v3 * dudx3 + $
              C4 * d4udx_4 + $
              C2 * d2udx_2*((u>0.0)/(1.0E-04+abs(u))) + $
              C0 * u^3 + $
              Cm * ((d2udx1_2-d2udx2_2)^2 + (d2udx1_2-d2udx3_2)^2 + (d2udx2_2-d2udx3_2)^2)*(u>0.0) $
             )
;
end ; bubble3d_operator
