pro bubble3d_sequence, u, nframe, dtime, x1, x2, x3, v1, v2, v3, u_arr
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bubble3d_sequence ('bubble_3_dimension_sequence')
;
; PURPOSE:
;   Solve "bubble equation" in 3D for time intervall.
;
; CATEGORY:
;   Hydrodynamics, Plotting.
;
; CALLING SEQUENCE:
;   bubble3d_sequence, u, dtime, x1, x2, x3, v1, v2, v3, u_arr
;
; INPUT:
;   dtime:  ('delta_time') real, time step
;   x1:     ('x_1') 1D real vector with x1 axis values
;   x2:     ('x_2') 1D real vector with x2 axis values
;   x3:     ('x_3') 1D real vector with x3 axis values
;   v1:     ('velocity_1') 3D real array with velocity data in x1 direction
;   v2:     ('velocity_2') 3D real array with velocity data in x2 direction
;   v3:     ('velocity_3') 3D real array with velocity data in x3 direction
;
; INPUT/OUTPUT:
;   u:      ('u') 3D real array with bubble amplitude
;
; OUTPUT:
;   u_arr:  ('u_array') 4D real array with bubble amplitude
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   bubble3d_operator:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   26. 3.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   12. 3.01 (B.F.) 3D version
;-
;------------------------------------------------------------------------------
;
; --- Parameters ---
dtimedx_limit=0.08
C_Courant    =0.25
;
; --- Array size ---
n1=n_elements(u(*,0,0))
n2=n_elements(u(0,*,0))
n3=n_elements(u(0,0,*))
u_arr=fltarr(n1,n2,n3,nframe)
;
; --- Scaling ---
dx              =(max(x1)-min(x1))/float(n_elements(x1)-1)
v_max           =max(sqrt(v1^2+v2^2+v3^2))
l_grid          =v_max*abs(dtime)/dx
nframe_min      =long(l_grid/C_Courant) + 1
;
nframe_factor   =((nframe_min-1)/nframe)+1
nframe_total    =nframe_factor*nframe
dtime_singlestep=abs(dtime)/float(nframe_total)
dtdx            =dtimedx_limit*l_grid/(float(nframe_total)*C_Courant)
v_scalefactor   =abs(dtime)/(dtdx*dx*float(nframe_total))
;
help, dx, v_max, l_grid, nframe_min, nframe_factor, nframe_total, dtime_singlestep, dtdx, v_scalefactor
;
; --- Compute all requested frames ---
for iframe=1,nframe do begin
  ;
  ; --- Iterate bubble equation ---
  for i=1,nframe_factor do begin
    v1_scale=v1*v_scalefactor
    v2_scale=v2*v_scalefactor
    v3_scale=v3*v_scalefactor
    print, 'v_scale: ', minmax(v1_scale), minmax(v2_scale), minmax(v3_scale), format='(A,6E14.5)'
    bubble3d_operator, u, dtdx=dtdx, v1=v1_scale, v2=v2_scale, v3=v3_scale
  endfor ; i
  ;
  u_arr(*,*,*,iframe-1)=u
endfor ; iframe
;
end ; bubble3d_sequence
