pro NORMCT, colortable, multifactor=multifactor, gamma=gamma, drange=drange, $
            colorrange=colorrange, nonorm=nonorm, phaser=phaser, rgbmask=rgbmask, $
            file=file
;
;-------------------------------------------------------------------------------
;+
; NAME:
;   NORMCT
;
; PURPOSE:
;   Load color table (if specified), set first color=black, last color=white.
;
; CATEGORY:
;   Image display
;
; CALLING SEQUENCE:
;   normct, colortable [,multifactor=multifactor]
;
; INPUTS:
;   colortable - Number of color table as used in LOADCT (optional)
;
; OUTPUTS:
;   none
;
; KEYWORDS:
;   multifactor - The number of times the (new or old) color table will wrap.
;                 This parameter does not need to be an integer.
;   gamma -
;   drange -
;   rgbmask -
;   colorrange - If not specified: Complete color table is changed
;                If one element: Only colors from 0 to this value are changed
;                If two elements: This two elements define the color range
;   nonorm - If disabled: Set first color=black, last color=white.
;   phaser -
;
; SIDE EFFECTS:
;   Color tables are loaded. Some information is printed.
;
; EXAMPLE:
;   Load color table PRISM (6), wrap table around two times, make last color
;   white:
;     tvscl,dist(256)              ; Display a simple image
;     NORMCT, 1, MULTIFACTOR=2.0   ; Change color table
;
; MODIFICATION HISTORY:
;   1993-05-11 Written (Bernd Freytag, Astrophysik, Kiel)
;   1993-06-01 (B.F.) Keyword MULTIFACTOR added
;   1993-07-19 (B.F.) Keywords COLORRANGE and NONORM added
;   1993-12-13 (B.F.) Colortable -2 added
;   1994-04-20 (B.F.) Keyword PHASER added
;   1994-05-10 (B.F.) Colortable -3 added (B.F.)
;   1994-05-23 (B.F.) Keyword rgbmask added
;   1997-02-05 (B.F.) gamma correction for standard color tables
;   2018-06-05 (B.F. Uppsala) Add keyword file
;-
;-------------------------------------------------------------------------------
;
; common colors, old_red, old_green, old_blue, cur_red, cur_green, cur_blue
;
; --- Determine color range ---
if (n_elements(colorrange) eq 0) then begin
  color1=0
  color2=!D.TABLE_SIZE-1           ; Number of available colors (not always 256)
endif else if (n_elements(colorrange) eq 1) then begin
  color1=0
  color2=0 > fix(colorrange(0)) < (!D.TABLE_SIZE-1)
endif else begin
  color1=0 > fix(colorrange(0)) < (!D.TABLE_SIZE-1)
  color2=color1 > fix(colorrange(1)) < (!D.TABLE_SIZE-1)
endelse
color_number=color2-color1+1
;print, 'Number of available colors (!D.TABLE_SIZE)=', fix(!D.TABLE_SIZE)
;print, 'Color range:', color1, ' to', color2
;
tvlct, red, green, blue, /GET                   ; Fetch current color table
;
if (n_params() gt 0) then begin
  if (colortable ge 0) then begin
    ; --- If specified load new table ---
    if (keyword_set(file)) then begin
      loadct, colortable, file=file
    endif else begin
      loadct, colortable
    endelse
    ;
    if (keyword_set(gamma)) then gamma_ct, gamma     ; Gamma correction
;    tvlct, red_n, green_n, blue_n, /GET             ; Fetch current color table
;    tvlct, red, green, blue                         ; Load modified color table
    ;
  endif else if (colortable eq -1) then begin
    lastindex=!D.TABLE_SIZE-1
    tvlct, red_n, green_n, blue_n, /GET             ; Fetch current color table
    midindex=lastindex/2
    ;
    if (not keyword_set(gamma)) then gamma=0.5
    shindex=findgen(lastindex+1)-float(midindex)
    maxindex=max(shindex)
    signindex=1.0-2.0*(shindex lt 0.0)
    red_n(0:lastindex)=byte( ((abs(shindex)/maxindex)^gamma)* $
                           maxindex*signindex + midindex )
    green_n(0:lastindex)=red_n(0:lastindex) < midindex
    blue_n(0:lastindex)=midindex
    ;    
;    red(0:lastindex)=bindgen(lastindex+1)
;    green(0:lastindex)=bindgen(lastindex+1) < midindex
;    blue(0:lastindex)=midindex
    ;
    tvlct, red_n, green_n, blue_n                   ; Load modified color table
  endif else if (colortable eq -2) then begin
    lastindex=!D.TABLE_SIZE-1
    tvlct, red_n, green_n, blue_n, /GET             ; Fetch current color table
    x=findgen(lastindex+1)/(lastindex+1)
;    cb=255.0*(1.0-cosinebell(x,0.33,0.17))
;    cb=255.0*(1.0-cosinebell(x,0.36,0.18))
    cb=255.0*(1.0-cosinebell(x,0.37,0.19))
    phi=-lastindex/6
    red_n  =shift(0.92*cb, 0          +phi)
    green_n=shift(0.75*cb,-lastindex/3+phi)
    blue_n =shift(0.93*cb, lastindex/3+phi)
    red_n  =red_n  *(1.0-0.12*sin(6.0*!pi*x)^2)
    green_n=green_n*(1.0-0.12*sin(6.0*!pi*x)^2)
    blue_n =blue_n *(1.0-0.12*sin(6.0*!pi*x)^2)
    red_n  =fix(red_n  *( (findgen(lastindex)/lastindex*16.)<1.0 ))
    green_n=fix(green_n*( (findgen(lastindex)/lastindex*16.)<1.0 ))
    blue_n =fix(blue_n *( (findgen(lastindex)/lastindex*16.)<1.0 ))
    tvlct, red_n, green_n, blue_n                   ; Load modified color table
  endif else if (colortable eq -3) then begin
    lastindex=!D.TABLE_SIZE-1
;    tvlct, red_n, green_n, blue_n, /GET             ; Fetch current color table
    red_n  =fltarr(!D.TABLE_SIZE)
    green_n=fltarr(!D.TABLE_SIZE)
    blue_n =fltarr(!D.TABLE_SIZE)
    ;
    if (keyword_set(drange)) then begin
      midindex=fix(abs(float(drange(0)))/(drange(1)-drange(0))*float(lastindex))
    endif else begin
      midindex=lastindex/2
    endelse
    ;
    if (not keyword_set(rgbmask)) then rgbmask=[[0.,0.,1.],[1.,0.,0.]]
    if (not keyword_set(gamma)) then gamma=1.0
    ;
    cval=(1.0-findgen(midindex+1)/float(midindex))^gamma*255.0
    red_n(0:midindex)  =rgbmask(0,0)*cval
    green_n(0:midindex)=rgbmask(1,0)*cval
    blue_n(0:midindex) =rgbmask(2,0)*cval
    cval=(findgen(lastindex-midindex+1)/float(lastindex-midindex))^gamma*255.0
    red_n(midindex:lastindex)  =rgbmask(0,1)*cval
    green_n(midindex:lastindex)=rgbmask(1,1)*cval
    blue_n(midindex:lastindex) =rgbmask(2,1)*cval
    ;
    rgbmin=min([red_n,green_n,blue_n], max=rgbmax)
    if (rgbmin lt 0.0) then begin
      red_n  =(red_n  -rgbmin)/(rgbmax-rgbmin)*255.
      green_n=(green_n-rgbmin)/(rgbmax-rgbmin)*255.
      blue_n =(blue_n -rgbmin)/(rgbmax-rgbmin)*255.
    endif
;    red_n(midindex)  =0
;    green_n(midindex)=0
;    blue_n(midindex) =0
    ;
;    if (keyword_set(gamma)) then begin
;      red_n  =(float(red_n)/255.0)^gamma*255.0
;      green_n=(float(green_n)/255.0)^gamma*255.0
;      blue_n =(float(blue_n)/255.0)^gamma*255.0
;    endif
    ;
    tvlct, red_n, green_n, blue_n                   ; Load modified color table
  endif
endif
;
; Gamma correction ?
;
tvlct, red_n, green_n, blue_n, /GET             ; Fetch current color table
;
if (n_elements(multifactor) gt 0) then m=float(multifactor) else m=1.0
mfac=m*float(!D.TABLE_SIZE)/float(color_number)
;print,'m=',m,' mfac=', mfac
tabsiz=n_elements(red_n)                        ; Wrap table (like MULTI)
indlist=(findgen(tabsiz)*mfac) mod tabsiz
red_n=red_n(indlist)
green_n=green_n(indlist)
blue_n=blue_n(indlist)
;
if (keyword_set(phaser)) then begin             ; Correct color table for PHASER
  red_o=float(red_n)
  green_o=float(green_n)
  blue_o=float(blue_n)
  ; --- Add some white if blue is present ---
;  red_n  =byte(red_o   + 0.5*blue_o*(255.0-red_o  )/255.0)
;  green_n=byte(green_o + 0.5*blue_o*(255.0-green_o)/255.0)
;  blue_n=byte(blue_o)
  red_n  =byte(red_o   + 0.3*(blue_o-red_o  )*(255.0-red_o  )/255.0)
  green_n=byte(green_o + 0.3*(blue_o-green_o)*(255.0-green_o)/255.0)
  blue_n=byte(blue_o)
endif
;
red(color1:color2)=red_n(0:color_number-1)      ; Shift color range
green(color1:color2)=green_n(0:color_number-1)  ;          "
blue(color1:color2)=blue_n(0:color_number-1)    ;          "
;
if (not keyword_set(nonorm)) then begin
  red(0)=0                                      ; First color index <- black
  green(0)=0                                    ;          "
  blue(0)=0                                     ;          " 
  ;
  red(!D.TABLE_SIZE-1)=255                      ; Last color index <- white
  green(!D.TABLE_SIZE-1)=255                    ;         "
  blue(!D.TABLE_SIZE-1)=255                     ;         "
endif
;
tvlct, red, green, blue                         ; Load modified color table
;
end ; NORMCT
