pro dust_bins_rossow78, par, p, T, ro, ros, rov, size, $
                        rovs=rovs, rnucl=rnucl, rcond=rcond, rcoag=rcoag, rcoal=rcoal, $
                        Kn=Kn, eta=eta, lambda_mfp=lambda_mfp, rcrit=rcrit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   dust_bins_rossow78 ('dust_bins_Rossow_1978')
;
; PURPOSE:
;   Compute characteristic rates for formation of Mg2SiO4 dust.
;
; CATEGORY:
;   HD, source terms, dust formation
;
; CALLING SEQUENCE:
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2000-04-18 (T.G., F.A.)
;   2006-11-07 (F.A.) Version of rossow78_mod for CO5BOLD
;   2007-06-05 (B.F.) Fortran 90 version
;   2007-11-13 (B.F.) Debugging
;   2007-11-22 (B.F.) Debugging
;   2007-11-26 (B.F.) Output of rovs, change in definition of rnucl and rconv,
;                     Limit T in formula for P_sat (T400)
;   2007-11-29 (B.F.) Factor ros in rconv removed
;   2007-12-12 (B.F.) Condensation rate test: if (Kn < alpha_stick)  ->  if (Kn < 1.0D+00)
;   2007-12-15 (B.F.) Compute nn from ros and not from rovs
;   2007-12-17 (B.F.) New formula for lambda_mfp and nu; nu->eta; f->f_CorrFiniteMass; reordering
;   2007-12-18 (B.F.) smax removed; more comments;
;                     sticking coefficients for coagulation and coalescence
;   2008-01-08 (B.F.) Case for large Reynolds numbers outcommented
;   2008-09-03 (B.F.) IDL version
;-
;------------------------------------------------------------------------------

; --- Constants ---
k_Bol_drk=double(!con.kBoltz)
amu_drk  =double(!con.muatom)
pi_drk   =!DPI
mv    = 24.31d0* amu_drk                                 ; for Mg
gamma_surf=250.d0                                        ;  Surface tension
f_CorrFiniteMass=5.d0                                    ;  Correction of finite mass
a_ps=42.179466259637d00                                  ;  Coefficient for saturation pressure formula
b_ps=-2.94064111396886d04                                ;  Coefficient for saturation pressure formula
c_ps=-4.52457333892826d07                                ;  Coefficient for saturation pressure formula
d_ps= 2.00362363225697d10                                ;  Coefficient for saturation pressure formula
sig=4.d0*(pi_drk*1.2d-8^2+2.d0/3.d0*2.d0*0.74d-8*1.2d-8) ; Cross-section of H2

; --- Input parameters: take basic parameters from parameter file ---
rop             =double(par.C_dust01)
ms              =double(par.C_dust02 * amu_drk)
eps             =double(par.C_dust03)
eps_AlmostNoDust=double(par.C_dust04)
; Not used:      par.C_dust05
alpha_stick     =double(par.C_dust06)
alpha_stick_coag=double(par.C_dust07)
alpha_stick_coal=double(par.C_dust08)

rs              =(3.d0*ms/(rop*4.d0*pi_drk))^(1.d0/3.d0)

; --- Saturation pressure according to Curtis S. Cooper vapor.c ---
; --- Limit temperature to be above 400 K: below the fit grows and then diverges ---
T400=T > 400.0D+00
rovs=eps*ms/(k_Bol_drk*T)*exp(a_ps + b_ps/T400 + c_ps/T400^2 + d_ps/T400^3 )

; --- Mean molecular weight of the gas ---
mu=ro*(k_Bol_drk*T)/(p*amu_drk)

; --- Number density of monomers in the condensed phase ---
nn=ros/((4.d0/3.d0*pi_drk)*size^3*rop)

; --- Atmospheric density by number of particles per volume ---
rn=ro/(mu*amu_drk)
; --- Mean free path of the gas ---
lambda_mfp=(1.d0/sqrt(2.d0))/(rn*sig)

; --- Dynamical viscosity ---
;fa06 une Bernd's viscosity!
; --- Reif ("Statistical and Thermal Physics") 14.8.33 ---
eta=(sqrt(pi_drk)*5.d0/16.d0) *sqrt((k_Bol_drk*T)*(mu*amu_drk))/sig

; --- Knudsen number ---
Kn=lambda_mfp/size

; --- Nucleation rate:
;     -Supersaturation [smax] (Rossow p.2,3)       	               c
;         s>0: growth ; s<0: evaporation                               c
;     -change in free energy per unit volume  (erg/cm**2) [dgv]        c
;     -Gibbs free energy of formation of the grain (pressure) [dklog]  c
;        Keq/kgas < 1 for grains to form ==> dgv & dgvcrit < 0         c
;     -barrier for vapor going to critical nucleus (erg) [dgvcrit]     c
;     -critical number density of seed molecules (cm^-3)   [ncrit]     c
;     -critical radius for grains to start nucleating (cm)   [rcrit]   c
;     -flux of gas molecules/atoms onto grains (cm g^-1 sec^-1) [omega]c
;     -rate of nucleation [rnucl]                                      c 
rnucl  =T*0.0D+00
gibbs  =T*0.0D+00
dgv    =T*0.0D+00
dgvcrit=T*0.0D+00
ncrit  =T*0.0D+00
rcrit  =T*0.0D+00
omega  =T*0.0D+00
;
ind=where((rov/rovs) gt 1.0D+00, nind)
if (nind gt 0) then begin
  ; --- Positive supersaturation -> there is nucleation ---
  ;
  gibbs[ind]  =(k_Bol_drk*T[ind])*alog(rov[ind]/rovs[ind])
  dgv[ind]    =gibbs[ind]/((4.d0/3.d0*pi_drk)*rs[ind]^3) 
  dgvcrit[ind]=(16.d0/3.d0*pi_drk)*gamma_surf^3/dgv[ind]^2
  ncrit[ind]  =(rov[ind]/ms)*exp(-dgvcrit[ind]/(k_Bol_drk*T[ind]))   
  rcrit[ind]  =2.d0*gamma_surf/dgv[ind]
  ;
  omega[ind]  =alpha_stick/ms*sqrt((k_Bol_drk*T[ind])/((2.d0*pi_drk)*mv*amu_drk))
  ;
  ; --- Nucleation rate = dros/dt/(rov-rovs) ---
  rnucl[ind]=(4.d0*pi_drk)*ms*ncrit[ind]*rcrit[ind]^2 *omega[ind]
endif



end ; dust_bins_rossow78
