function rhd_hyd_rossow78tfall, p_in, T_in, rho_in, grav_in, radius_in, rop_in, $
  invKn=invKn
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_hyd_rossow78tfall ('rhd_hydrodynamics_rossow_1978_time_fall')
;
; PURPOSE:
;   Calculation of dust sedimentation velocity
;
; CATEGORY:
;   RHD, dust
;
; CALLING SEQUENCE:
;   vfall=rhd_hyd_rossow78tfall(p, T, rho, grav, radius, rop)
;
; INPUT:
;   p:         ('pressure') real scalar with gas pressure (-> p)
;   T:         ('Temperature') real scalar with gas temperature (-> t)
;   rho:       ('rho') real scalar with gas density (-> ro)
;   grav:      ('gravity') real scalar with acceleration due to gravity (-> grav)
;   rop:       ('rho_particle') averaged grain material density [g/cm^3]
;   radius:    ('radius') real scalar with grain radius (-> size)
;
; OUTPUT:
;   vfall: ('velocity_fall') real scalar with sedimentation velocity (-> vfall)
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Some constants are set within this routine. They have to consistent with definitions elsewhere.
;   The regime distinction should be done based on the total gravity, not only on the component in
;   the current direction.
;
; COMMENTS:
;   Original:
;     constants:
;      real*8 pi,kbol,amu,rgas
;      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/
;      data rgas/8.3145d7/
;      rop = 3.8d0               !for dirty forsterite (MgFeSiO4)
;
; PROCEDURE:
;     Calculation of the sedimentation velocity in the cloud
;     (according to Rossow (1978), Icarus 30, 1-50, Eq. (19))
;
; EXAMPLE:
;   mtime=1500L
;   p=seqavg('p',seq,mtime=mtime)
;   T=seqavg('t',seq,mtime=mtime)
;   rho=seqavg('rho',seq,mtime=mtime)
;   radius=seqavg('rdust',seq,mtime=mtime)
;   grav=seq.par.grav
;   rop=seq.par.C_dust01
;
; MODIFICATION HISTORY:
;   2000-04    Auteur: Tristan Guillot d'origine (Avril 2000) 
;   2006-11-10 (F.A.) Routine rossow78_tfall in file rossow_sed.f90
;   2007-04-16 (B.F.) F90 version, rop=3.8d0 -> 3.3d0
;   2007-12-12 (B.F.) real*8 -> kind=kind(0.0D+00); rop as input parameter
;   2007-12-14 (B.F.) Kn -> invKn (Kn is infinity when size=0.0);
;                     case Re>>70,Kn<<1: changed formula
;   2007-12-15 (B.F.) New value for sig, according to Derek,
;                     R_gas -> k_Bol/amu
;   2007-12-17 (B.F.) New formula for lambda and nu; nu->eta; reordering of some formulas
;   2008-01-08 (B.F.) Case for large Reynolds numbers outcommented
;   2008-08-06 (B.F.) IDL version
;-
;------------------------------------------------------------------------------

; === Physical constants ===
; Fortran: use const_module, only: pi_drk, k_Bol_drk, amu_drk
; --- atomic weight unit ---
amu_drk=double(1.0/!CON.NAvogadro)
; --- Boltzmann constant ---
k_Bol_drk=double(!CON.kBoltz)
;help, uAtom, kBoltzmann

; === Other constants ===
; --- cross section of H_2 [cm^2] ---
; --- Radius of H2 molecule, not HI!
sig=4.d0*(!DPI*1.2d-8^2+2.d0/3.d0*2.d0*0.74d-8*1.2d-8)

; --- Transform type of input parameters (single -> double precision) ---
p   =double(p_in)
T   =double(T_in)
ro  =double(rho_in)
grav=double(grav_in)
size=double(radius_in)
rop =double(rop_in)

; --- Mean molecular weight of gas ---
mu=ro*(k_Bol_drk*T)/(p*amu_drk)
; --- Atmospheric density by number of particles/cm^3 ---
rn=ro/(mu*amu_drk)
; --- Mean free path of the gas ---
lambda=(1.d0/sqrt(2.d0))/(rn*sig)
; --- Dynamical viscosity ---
; --- Reif ("Statistical and Thermal Physics") 14.8.33 ---
eta=(sqrt(!DPI)*5.d0/16.d0) *sqrt((k_Bol_drk*T)*(mu*amu_drk))/sig

; --- Knudsen number Kn ---
;Kn=lambda/size
invKn=size/lambda

; === Settling velocity in different regimes; Rossow Eq. (19) ===
vfall=0.0D+00*T

ind=where(invKn lt 1.0, nind)
if (nind gt 0) then begin
  ; --- Subsonic free molecular flow ---
  ;vSettle[ind]=(16.0/27.0) * rhoDust*grav*radiusGrain[ind]/rho[ind] * $
  ;        sqrt((!PI/8.0) * mu[ind]*uAtom/(kBoltzmann*T[ind]))
  vfall[ind]=((16.d0/27.d0)*rop[ind]*grav*size[ind])*sqrt((!DPI/8.d0)*(mu[ind]*amu_drk)/(k_Bol_drk*T[ind])) / ro[ind]
endif

ind=where(invKn ge 1.0, nind)
if (nind gt 0) then begin
  ; --- Laminar viscous flow (Stokes) ---
  ; --- from F_grav = F_fric                                                       ---
  ; ---      m * grav = 4/3*pi*size^3*rop * grav = 6*pi*eta*size*vfall             ---
  vfall[ind]=(2.d0/9.d0)*(rop[ind]*grav*size[ind]^2)/eta[ind]
endif

;prinf, format='(1p,4d11.3,1X,4d11.3,1X,d11.3)' grav,ro,p,T, size,lambda,eta,invKn, vfall

; --- Transform type of output parameter (double -> single precision) ---
return, float(vfall)

end ; function rhd_hyd_rossow78tfall
