; pro spectra_phoenix2gaia
;
;------------------------------------------------------------------------------
;+
; NAME:
;   spectra_phoenix2gaia ('spectra_phoenix_to_gaia')
;
; PURPOSE:
;   Read PHOENIX spectra and transform into GAIA format
;
; CATEGORY:
;   IO, data processing, spectra
;
; CALLING SEQUENCE:
;   @spectra_phoenix2gaia
;
; INPUT:
;   No input parameters.
;   The script has to be edited to change the control parameters.
;   It reads the input data from files.
;
; OUTPUT:
;   No parameter.
;   The scripte produces an output file.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A file is written.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
;   Headers according to GAIA standard.
;     (Interface Control Document for the Gaia spectral libraries, GAIA-C8-OCA-FT-002)
;   The strings '$STRING$' in the template will be replaced in the script.
;     Object          star            STRING          for us there are only stars
;     Type            RVS             STRING          RPBP or RVS
;     Provider        ALLARD          STRING          use your Gaia initials or your last name         
;     Date            $DATE$          STRING          date of production, not ddmmyy, now yyyymmdd
;     Version         $VERSION$       STRING          your own versioning for Gaia
;     Code            PHOENIX         STRING          name of your code, if applicable
;     fluxnorm        0               INT             fluxed: 0;      normalized: 1
;     resolution      -999.           FLOAT           power of resolution (what's that?)
;     ntot            50001           LONG            RPBP: 8001 (300 - 1100 nm), RVS: 50001 (840 - 890 nm)
;     lambda0         840.0           FLOAT           start value of vacuum wavelength [nm]
;     lambdaEnd       890.0           FLOAT           end value of vacuum wavelength [nm]
;     dlambda         1.0E-3          FLOAT           wavelength increment [nm], RPBP: 0.1, RVS: 1E-3
;     Teff            $TEFF$          FLOAT           T_eff, [K]
;     logg            $LOGG$          FLOAT           log (g), [g in cm s^-2]
;     vturb           0.0             FLOAT           microturbulence, [km s^-1]
;     vrot            0.0             FLOAT           equatorial v_rotational (needed in later steps), [km/s]
;     Y               0.28            FLOAT           helium mass fraction
;     Zmetal          0.012           FLOAT           metal (A>2) mass fraction
;     FeRef           7.50            FLOAT           logarithmic iron abundance (H = 12)
;     alphaFe         0.0             FLOAT           logarithmic alpha-element abundance relative to solar
;     CAbund          8.55            FLOAT           logarithmic carbon abundance (H = 12)
;     NAbund          7.97            FLOAT           logarithmic nitrogen abundance (H = 12)
;     OAbund          8.87            FLOAT           logarithmic oxygen abundance (H = 12)
;     MgAbund         7.58            FLOAT           logarithmic magnesium abundance (H = 12)
;     SiAbund         7.55            FLOAT           logarithmic silicon abundance (H = 12)
;     CaAbund         6.34            FLOAT           logarithmic calcium abundance (H = 12)
;     FeAbund         7.50            FLOAT           logarithmic iron abundance (H = 12)
;     mass            0.0             FLOAT           stellar mass (for spherical models), plane-parallel: 0
;     Gmag            -999.0          FLOAT           computed G magnitude [mag], otherwise '-999'
;     bcG             -999.0          FLOAT           computed bolometric correction to G [mag], otherwise '-999'
;     aV              -999.0          FLOAT           reddening once applied, otherwise '-999'
;
;     flux            $$              FLOAT[ntot]     computed flux [W m^-2 nm^-1]
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2008-12-09 Written by Bernd Freytag, CRAL-ENS Lyon
;   2008-12-10 v01: First version
;   2008-12-11 v02: Convolution with Gaussian
;-
;------------------------------------------------------------------------------

; === Control parameters that could be changed for different data sets ========
MetString     ='0.0'
InSpecDir     =getenv('HOME') + '/dat/atm/phoenix/Spectra/Cond-2000-RVS_GAIA08_' + MetString
;InSpecFiles   ='lte30.00-5.50-' + MetString + '.AMES-Cond-2000.server.spec.7'
InSpecFiles   ='lte30.00-5.*-' + MetString + '.AMES-Cond-2000.server.spec.7'
;InSpecFiles   ='lte*-' + MetString + '.AMES-Cond-2000.server.spec.7'
OutSpecVersion='02'
OutSpecFoldFWHM=0.003
OutFileName   ='Allard_M_' + MetString + '_v' + OutSpecVersion + '.txt'
; =============================================================================

Type='RVS' & dlambda=1.0E-3
OutFileHeader1 ='Object   Type     Provider Date     Version  Code     fluxnorm resolution ntot    lambda0  lambdaEnd dlambda  Teff     logg     vturb    vrot     Y        Zmetal   FeRef    alphaFe  CAbund   NAbund   OAbund   MgAbund  SiAbund  CaAbund  FeAbund  mass     GMag     bcG      aV'
OutFileHeader2 ='STRING   STRING   STRING   STRING   STRING   STRING   INTEGER  FLOAT      LONG    FLOAT    FLOAT     FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT    FLOAT'
OutSpecTemplate='star     RVS      ALLARD   $DATE$   $VERSION$ PHOENIX 0        -999.0     50001   840.0    890.0     1.0E-3   $TEFF$   $LOGG$   0.0      0.0      0.28     0.012    7.50     0.0      8.55     7.97     8.87     7.58     7.55     6.34     7.50     0.0     -999.0   -999.0   -999.0'


; --- Put together string with current date ---
caldat, systime(/julian), mm,dd,yyyy 
;yy=yyyy mod 100 & DateString=strmid('0'+strtrim(string(dd),2),1,2,/rev) + strmid('0'+strtrim(string(mm),2),1,2,/rev) + strmid('0'+strtrim(string(yy),2),1,2,/rev)
DateString=strmid('000'+strtrim(string(yyyy),2),3,4,/rev) + strmid('0'+strtrim(string(mm),2),1,2,/rev) + strmid('0'+strtrim(string(dd),2),1,2,/rev)
help, DateString

; --- Open output file and write 2-line file header ---
openw, ncout, OutFileName, /get_lun
;printf, ncout, OutFileHeader1, format='(A)'
;printf, ncout, OutFileHeader2, format='(A)'

; --- Get list of all input files ---
FileList=file_search(InSpecDir + '/' + InSpecFiles, count=nFile)
print, 'Number of input Files:', nFile

; === Loop over all files ========================================
for iFile=0,nFile-1L do begin &$
  ;
  FileName=FileList[iFile] &$
  FileWithoutPath=strmid(FileName, strpos(FileName,'/',/REVERSE_SEARCH)+1) &$
  print, iFile, '  ', FileWithoutPath &$
  ;
  ; --- Determine logg and Teff from filename ---
  ItemsBetweenMinus=strsplit(FileWithoutPath, '-', /extract) &$
  if (n_elements(ItemsBetweenMinus) ge 2) then begin &$
    logg=double(ItemsBetweenMinus[1]) &$
  endif else begin &$
    print, 'Could not determine gravity.' &$
    logg=0.0 &$
  endelse &$
  if (n_elements(ItemsBetweenMinus) ge 1) then begin &$
    Teff=double(strmid(ItemsBetweenMinus[0],3,20))*1.0D+02 &$
  endif else begin &$
    print, 'Could not determine effective temperature.' &$
    Teff=0.0 &$
  endelse &$
  ;
  ; --- Create specific spectrum header from template and write it into file ---
  OutSpecHeader=OutSpecTemplate &$
  OutSpecHeader=strmid(OutSpecHeader, 0, strpos(OutSpecHeader, '$DATE$'   )) + DateString                     + strmid(OutSpecHeader, strpos(OutSpecHeader, '$DATE$'   )+strlen('$DATE$'   )) &$
  OutSpecHeader=strmid(OutSpecHeader, 0, strpos(OutSpecHeader, '$VERSION$')) + OutSpecVersion                 + strmid(OutSpecHeader, strpos(OutSpecHeader, '$VERSION$')+strlen('$VERSION$')) &$
  OutSpecHeader=strmid(OutSpecHeader, 0, strpos(OutSpecHeader, '$TEFF$'   )) + strtrim(string(float(Teff)),2) + strmid(OutSpecHeader, strpos(OutSpecHeader, '$TEFF$'   )+strlen('$TEFF$'   )) &$
  OutSpecHeader=strmid(OutSpecHeader, 0, strpos(OutSpecHeader, '$LOGG$'   )) + strtrim(string(float(logg)),2) + strmid(OutSpecHeader, strpos(OutSpecHeader, '$LOGG$'   )+strlen('$LOGG$'   )) &$
  print, OutSpecHeader &$
  printf, ncout, OutSpecHeader, format='(A)' &$
  ;
  ; --- Read PHOENIX spectrum into double precision 2D array     ---
  ; --- Wavelength is in first column, spectrum in second column ---
  ;rdarray, FileName, a, type=size(0.0D+00, /type) & OutSpec=reform(a[*,1]) &$
  a=read_ascii(FileName) & OutSpec=double(reform(a.field01[1,*])) &$
  ; --- Transform units from PHOENIX convention to [erg cm^-2 s^-1 Aangstroem^-1] ---
  OutSpec=10.0^(Outspec-8.0) &$
  ; --- Transform from erg/cm^2/sec/Aangstroem to W m^-2 nm^-1 ---
  ; ---   erg cm^-2 s^-1 Aangstroem^-1                         ---
  ; --- = (10^-7 Ws) (10^-2 m)^-2 s^-1 (10^-1 nm)^-1           ---
  ; --- = 10^-2 W m^-2 nm^-1                                   ---
  ; --- Account for PHOENIX unit factor                        ---
  OutSpec=OutSpec * 1.0D-2 &$
  if (OutSpecFoldFWHM > 0.0) then begin &$
    OutSpec=[OutSpec, OutSpec[n_elements(OutSpec)-1L]] &$
OutSpec0=OutSpec &$
    ; --- Convolve spectrum with Gaussian ---
    nkernel=51L &$
    sigma  =OutSpecFoldFWHM/dlambda/sqrt(alog(2.0)) &$
    kernel =exp(-((findgen(nkernel)-float(nkernel/2L))/sigma)^2) &$
    kernel =kernel/total(kernel) &$
    OutSpec=convol(OutSpec, kernel, /center, /edge_truncate) &$
  endif else begin &$
    ; --- Add one point (Attention: change for other types of input our output data) ---
    OutSpec=[OutSpec, 0.0D+00] &$
  endelse &$
  ; --- Write into file ---
  printf, ncout, OutSpec, format='(' + strtrim(string(n_elements(OutSpec)),2) +'E12.5)' &$
  ;
endfor ; iFile
; === End of loop over all files ========================================


; --- Close output file ---
close, ncout
free_lun, ncout

;end ; spectra_phoenix2gaia
