function vsettledust, rho, T, P, quc001, quc002, $
  grav, xnDmax, $
  mu=mu, $
  numberGrain=numberGrain, radiusGrain=radiusGrain, $
  lambdaMeanFreePath=lambdaMeanFreePath, Kn=Kn, nuDynamical=nuDynamical, $
  a70=a70, Re=Re
;
;------------------------------------------------------------------------------
;+
; NAME:
;   v_settle_dust ('velocity_settle_dust')
;
; PURPOSE:
;   Compute the dust settling velocity
;
; CATEGORY:
;   RHD, dust
;
; CALLING SEQUENCE:
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2006-11-06 (B.F., F.A.)
;-
;------------------------------------------------------------------------------

; === Physical constants ===
; --- gas constant ---
RGas =!CON.RGas
; --- atomic weight unit ---
uAtom=1.0/!CON.NAvogadro
; --- Boltzmann constant ---
kBoltzmann=!CON.kBoltz
help, RGas, uAtom, kBoltzmann

; === Other constants ===
; --- cross section of H_2 [cm^2] ---
sigmaCross=!PI * 2.0E-08^2
; --- density of dust grain for fosterite [g/cm^3] ---
rhoDust=3.8
; --- atomic weight of fosterite ---
AGW=172.25

; === Basic quantities ===
; --- Mean molecular weight of gas ---
mu=Rgas * (rho*T)/P

; === Further quantities ===

; --- Radius of dust grain made of single monomer ---
R1Dust=((3.0/(4.0*!PI)) * AGW*uAtom/rhoDust)^0.33333333
help, AGW, rhoDust, R1Dust

; --- Number density of grains ---
DustK0=(quc001+quc002)/xnDmax

; --- Number of monomers in grain ---
NumberGrain=quc001/DustK0

; --- Radius of dust grains ---
radiusGrain=R1Dust * (NumberGrain)^0.33333333

; --- Mean free path length [cm] ---
lambdaMeanFreePath=sqrt(3.0*!PI) / (8.0*rho*sigmaCross/(mu*uAtom))

; --- Knudsen number ---
Kn=lambdaMeanFreePath/(radiusGrain > 1.0E-20)

; --- dynamical viscosity ---
nuDynamical=rho/3.0 * lambdaMeanFreePath * sqrt((8.0/!PI)*RGas*T/mu)

; --- a70 ---
a70=(9.0*270.0*mu^2/(4.0*rhoDust*rho*grav))^0.3333333333
; --- Reynolds number ---
Re=70.0 * radiusGrain/a70

; === Settling velocity in different regimes ===

vSettle=0.0*T

; --- Subsonic free molecular flow ---
ind=where(Kn ge 1.0, nind)
if (nind gt 0) then begin
  vSettle[ind]=(16.0/27.0) * rhoDust*grav*radiusGrain[ind]/rho[ind] * $
          sqrt((!PI/8.0) * mu[ind]*uAtom/(kBoltzmann*T[ind]))
endif

; --- Laminar flow ---
ind=where((Kn lt 1.0) and (Re le 70.0), nind)
if (nind gt 0) then begin
  vSettle[ind]=(2.0/9.0)*rhoDust*grav*radiusGrain[ind]^2/nuDynamical[ind]
endif

; --- Turbulent flow ---
ind=where((Kn lt 1.0) and (Re gt 70.0), nind)
if (nind gt 0) then begin
  vSettle[ind]=sqrt((4.0/3.0)*rhoDust*radiusGrain[ind]/rho[ind])*grav
endif


return, vSettle

end ; vsettledust
