FUNCTION fufi_curvefit, X0, Y, W, A, sigmaA, Function_Name = Function_Name, $
  itermax=itermax, dchieps=dchieps, chieps=chieps, Amask=Amask
;+
; NAME:
;   fufi_CURVEFIT
;
; PURPOSE:
;   Non-linear least squares fit to a function of an arbitrary 
;   number of parameters.  The function may be any non-linear 
;   function where the partial derivatives are known or can be 
;   approximated.
;
; CATEGORY:
;   E2 - Curve and Surface Fitting.
;
; CALLING SEQUENCE:
;   Result = fufi_CURVEFIT(X, Y, W, A, sigmaA, FUNCTION_NAME = name
;                          [, itermax=itermax, dchieps=dchieps, chieps=chieps
;                           , Amask=Amask])
;
; INPUTS:
;   X:  A row vector of independent variables.
;
;   Y:  A row vector of dependent variable, the same length as x.
;
;   W:  A row vector of weights, the same length as x and y.
;       For no weighting,
;         w(i) = 1.0.
;       For instrumental weighting,
;         w(i) = 1.0/y(i), etc.
;
;   A:  A vector, with as many elements as the number of terms, that 
;       contains the initial estimate for each parameter.  If A is double-
;       precision, calculations are performed in double precision, 
;       otherwise they are performed in single precision.
;
; KEYWORDS:
;   FUNCTION_NAME:  The name of the function (actually, a procedure) to 
;     fit.  If omitted, "FUNCT" is used.
;     This function,
;     (actually written as a procedure) must accept values of
;     X (the independent variable), and A (the fitted function's
;     parameter values), and return F (the function's value at X),
;     and PDER (a 2D array of partial derivatives).
;     For an example, see FUNCT in the IDL User's Libaray.
;     A call to FUNCT is entered as:
;       FUNCT, X, A, F, PDER
;     where:
;     X = Vector of NPOINT independent variables, input.
;     A = Vector of NTERMS function parameters, input.
;     F = Vector of NPOINT values of function, y(i) = funct(x(i)), output.
;     PDER = Array, (NPOINT, NTERMS), of partial derivatives of funct.
;     PDER(I,J) = DErivative of function at ith point with
;     respect to jth parameter.  Optional output parameter.
;     PDER should not be calculated if the parameter is not
;     supplied in call.
;   ITERMAX: Maximum number of iterations (default: 20)
;   DCHIEPS: 
;   CHIEPS:
;   Amask:
;
; OUTPUTS:
;   Returns a vector of calculated values.
;   A:  A vector of parameters containing fit.
;
; OPTIONAL OUTPUT PARAMETERS:
;   Sigmaa:  A vector of standard deviations for the parameters in A.
;	
; COMMON BLOCK:
;  common fufi_io
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Convergence might be slow so that the default number of iterations might
;   not suffice and a 'Failed to converge' is sent.
;   If the Y values are small the test if the initial fit is good might be
;   nonsense. Use for the weigths (no weighting) w(i)=1.0/avg(y^2).   
;
; PROCEDURE:
;  Copied from "CURFIT", least squares fit to a non-linear
;  function, pages 237-239, Bevington, Data Reduction and Error
;  Analysis for the Physical Sciences.
;
;  "This method is the Gradient-expansion algorithm which
;  combines the best features of the gradient search with
;  the method of linearizing the fitting function."
;
;  Iterations are performed until the chi square changes by
;  only 0.1% (chieps) or until itermax iterations have been performed.
;
;  The initial guess of the parameter values should be
;  as close to the actual values as possible or the solution
;  may not converge.
;
; MODIFICATION HISTORY:
;  Written, DMS, RSI, September, 1982.
;  Does not iterate if the first guess is good.  DMS, Oct, 1990.
;  Added CALL_PROCEDURE to make the function's name a parameter.
;    (Nov 1990)
;  12/14/92 - modified to reflect the changes in the 1991
;    edition of Bevington (eq. II-27) (jiy-suggested by CreaSo)
;  30.11.94 (B.F.) Keyword itermax added
;   8. 9.95 (B.F.) Counter subiter added, keyword dchieps added
;  11. 9.95 (B.F.) Message 'Failed to converge' removed
;  19. 9.95 (B.F.) Renamed: 'curvefit' -> 'fufi_curvefit'
;  12. 5.05 (B.F.) Check for Inf and NaN
;-
; === Common block ===
; --- Common block with I/O switches
common fufi_io, print_terms, print_blank, print_F, print_PDER, print_a, $
       print_time, print_iter, $
       print_init, plot_test, plot_fit
;
;ON_ERROR,2		;RETURN TO CALLER IF ERROR
;
; --- Default keywords ---
; --- Name of function to fit ---
if n_elements(function_name) le 0 then function_name = "FUNCT"
; --- Maximum number of iterations (default: 20) ---
if (n_elements(itermax) eq 0) then itermax=20
;
;A = 1.*A		;MAKE PARAMS FLOATING
A = 1.0D+00*A		;MAKE PARAMS DOUBLE
Ainitial=A
X=double(X0)
;
; --- Look which parameters are to be varied ---
if (not keyword_set(Amask)) then begin
  Amask=replicate(1,n_elements(A))
endif
Aind=where(Amask eq 1)
Ared=A(Aind)
Bred=Ared
;CHISQR=1.0
;CHISQ1last=1.0E38
CHISQR=1.0D+00
CHISQ1last=1.0D+38
;
NTERMS = N_ELEMENTS(Ared)                         ;# OF variable PARAMS.
NFREE = (N_ELEMENTS(Y)<N_ELEMENTS(X(*,0)))-NTERMS ;Degs of freedom
IF (NFREE LE 0) THEN STOP,'Curvefit - not enough data points.'
;FLAMBDA = 0.001		                          ;Initial lambda
;FLAMBDAinc=10.0
;FLAMBDAdec=50.0
FLAMBDA = 0.001D+00		                          ;Initial lambda
FLAMBDAinc=10.0D+00
FLAMBDAdec=50.0D+00
;FLAMBDA = 100.0		                          ;Initial lambda
;FLAMBDAinc=2.0
;FLAMBDAdec=4.0
DIAG = INDGEN(NTERMS)*(NTERMS+1)                  ;SUBSCRIPTS OF DIAGONAL ELEMENTS
; --- Epsilon for chi ---
;if (n_elements(dchieps) eq 0) then dchieps=0.001
;if (n_elements(chieps) eq 0) then chieps=total(abs(y))/1e7/NFREE
if (n_elements(dchieps) eq 0) then dchieps=0.001D+00
if (n_elements(chieps) eq 0) then chieps=total(abs(y))/1D+07/NFREE
;
B=A
;
; --- Iteration loop ---
FOR ITER = 1,itermax DO BEGIN
  ;
  if (ITER gt 1) then begin
    ; --- Keep values ---
    Alast     =A
    YFIT0last =YFIT0
    PDERlast  =PDER
    BETAlast  =BETA
    ALPHAlast =ALPHA
    CHISQ1last=CHISQ1
    Clast     =C
    ARRAY0last=ARRAY0
  endif
  ;
  ; --- COMPUTE FUNCTION AT A with partial derivatives. ---
  CALL_PROCEDURE, Function_name, X, A, YFIT0, PDER, Amask=Amask
  ;
  ; --- EVALUATE ALPHA AND BETA MATRICIES. ---
  BETA = (Y-YFIT0)*W # PDER
;  ALPHA = TRANSPOSE(PDER) # (W # replicate(1.0,NTERMS) * PDER)
  ALPHA = TRANSPOSE(PDER) # (W # replicate(1.0D+00,NTERMS) * PDER)
  ;
  ; --- PRESENT CHI SQUARED. ---
  CHISQ1 = TOTAL(W*(Y-YFIT0)^2)/NFREE
  ;
  ; --- INVERT MODIFIED CURVATURE MATRIX TO FIND NEW PARAMETERS.
  ;
  C = SQRT(ALPHA(DIAG) # ALPHA(DIAG))
  ARRAY0 = ALPHA/C
  ;
  if (ITER eq 1) then begin
    ; --- Keep initial values ---
    YFIT0initial =YFIT0
    ALPHAinitial =ALPHA
    CHISQ1initial=CHISQ1
  endif
  ;
  if (finite(total(ARRAY0)) eq 0) then begin
    if (ITER eq 1) then begin
      ; --- Initial values already give useless matrix ---
      if (print_iter ge 2) then message, 'Initial values useless', /INFORMATIONAL
      sigmaA=!VALUES.F_INFINITY
      YFIT  =YFIT0
      goto, DONE_ITER
    endif else begin
;      FLAMBDA=FLAMBDA*100.0
      FLAMBDA=FLAMBDA*100.0D+00
      if (FLAMBDA eq !VALUES.F_INFINITY) then begin
        ; --- Step size too small ---
        if (print_iter ge 2) then message, 'Step size too small', /INFORMATIONAL
        sigmaA=!VALUES.F_INFINITY
        YFIT=YFIT0
        goto, DONE_ITER
      endif else begin
        ; --- Restore last useful values ---
        A     =Alast
        YFIT0 =YFIT0last
        PDER  =PDERlast
        BETA  =BETAlast
        ALPHA =ALPHAlast
        CHISQ1=CHISQ1last
        C     =Clast
        ARRAY0=ARRAY0last
      endelse
    endelse    
  endif
  ;
  itersub=0
  REPEAT BEGIN
    itersub=itersub+1
    ;
    Bredlast   =Bred
    FLAMBDAlast=FLAMBDA
    CHISQRlast =CHISQR
    ;
    ARRAY1=ARRAY0
;    if (finite(total(array1)) eq 0) then begin & print, C & print & print, array1 & print & endif
;    ARRAY1(DIAG)=ARRAY1(DIAG)*(1.+FLAMBDA)
    ARRAY1(DIAG)=ARRAY1(DIAG)*(1.0D+00+FLAMBDA)
;    if (finite(total(array1)) eq 0) then print, array1
    ARRAY1 = INVERT(ARRAY1)
;    if (finite(total(array1)) eq 0) then print, array1
;    if (finite(total(array1)) eq 0) then stop
    Bred = Ared+ ARRAY1/C # TRANSPOSE(BETA)      ;NEW PARAMS
;    Bred = Ared+ 1.0/(FLAMBDA)* TRANSPOSE(BETA)/sqrt(total(BETA^2))      ;NEW PARAMS
    B(Aind)=Bred
    CALL_PROCEDURE, Function_name, X, B, YFIT	 ;EVALUATE FUNCTION
    CHISQR =TOTAL(W*(Y-YFIT)^2)/NFREE           ;NEW CHISQR
    FLAMBDA=FLAMBDA*FLAMBDAinc                 ;ASSUME FIT GOT WORSE
;print,A
;print
;print,alpha
    if (print_iter ge 3) then begin
;      print, '   itersub=', itersub, flambda, total(alpha), total(c), minmax(PDER), minmax(alpha), minmax(C), total(alpha/c), total(array1), total((Y-YFIT)^2), chisqr, format='(A,I4,14E12.5)'
      print, '   itersub=', itersub, flambda, total(Y), total(Yfit), total((Y-YFIT)^2), chisqr, format='(A,I4,5E13.6)'
    endif
;    print, '          ', B, format='(A,4X,14E12.5)'
    ;
;  ENDREP UNTIL (((itersub gt 1) and (CHISQRlast le CHISQ1) and (CHISQR ge CHISQRlast)) or $
;  ENDREP UNTIL ((CHISQR le CHISQ1) or $
;                (abs(CHISQR) eq !VALUES.F_INFINITY) or $
;  ENDREP UNTIL ((CHISQR LE CHISQ1) or (abs(CHISQR) eq !VALUES.F_INFINITY) or (finite(CHISQR) eq 0) or (itersub gt itermax))
;    if ((itersub gt 1) and (CHISQRlast le CHISQ1) and (CHISQR ge CHISQRlast)) then begin
    if ((itersub gt 1) and (CHISQRlast le CHISQ1) and (CHISQR ge (CHISQRlast-(0.0001>0.0)*CHISQ1))) then begin
      ; --- Restore last value (better than the current one) ---
      Bred   =Bredlast
      FLAMBDA=FLAMBDAlast
      CHISQR =CHISQRlast
      goto, DONE_ITERSUB
    endif
  ENDREP UNTIL ((abs(CHISQR) eq !VALUES.F_INFINITY) or $
                (finite(CHISQR) eq 0) or $
                (itersub gt itermax))
  DONE_ITERSUB:
  ;
  FLAMBDA=FLAMBDA/FLAMBDAdec                          ;DECREASE FLAMBDA BY FACTOR 50
  Ared   =Bred			                 ;SAVE NEW PARAMETER ESTIMATE.
  A(Aind)=Ared
  ;
  if (print_iter ge 1) then begin
;    print, 'ITERATION =',ITER, ' :CHISQR =',CHISQR, ' ,CHISQ1 =',CHISQ1, ' ,CHISQ1last =',CHISQ1last, ' ,DCHISQR =',(CHISQ1-CHISQR)/CHISQ1, $
    print, 'ITER=',ITER, ' :CHISQR 0,1,1l,d =', CHISQR, CHISQ1, CHISQ1last, (CHISQ1-CHISQR)/CHISQ1, A, $
      format='(A,I4,A, 4E12.5, 1X, 4(1X,E13.6))'
  endif
  ;
  if ((abs(CHISQR) eq !VALUES.F_INFINITY) or (finite(CHISQR) eq 0)) then begin
    ; --- Divergence, restore at least start values ---
    if (print_iter ge 2) then message, 'Divergence', /INFORMATIONAL
    sigmaA=!VALUES.F_INFINITY
    A     =Ainitial
    YFIT  =YFIT0initial
    goto, DONE_ITER
  endif
  if (finite(total(A)) eq 0) then begin
    ; --- Divergence, restore at least start values ---
    if (print_iter ge 2) then message, 'Divergence in parameters', /INFORMATIONAL
    sigmaA=!VALUES.F_INFINITY
    A     =Ainitial
    YFIT  =YFIT0initial
    goto, DONE_ITER
  endif
  if (max(abs(A)) gt 1.0D+38) then begin
    ; --- Divergence, restore at least start values ---
    if (print_iter ge 2) then message, 'Parameters too large', /INFORMATIONAL
    sigmaA=!VALUES.F_INFINITY
    A     =Ainitial
    YFIT  =YFIT0initial
    goto, DONE_ITER
  endif
  if (CHISQR le chieps) then begin
    ; --- Convergence ---
    if (print_iter ge 2) then message, 'Convergence', /INFORMATIONAL
    sigmaA=SQRT(ARRAY1(DIAG)/ALPHA(DIAG))
    goto, DONE_ITER
  endif
  if ((dchieps gt 0.0) and ((abs(CHISQ1-CHISQ1last)/CHISQ1) le dchieps)) then begin
    ; --- Marginal convergence ---
    if (print_iter ge 2) then message, 'Marginal convergence', /INFORMATIONAL
    sigmaA=SQRT(ARRAY1(DIAG)/ALPHA(DIAG))
    goto, DONE_ITER
  endif
  if (ITER eq itermax) then begin
    ; --- Maximum iteration count reached ---
    if (print_iter ge 2) then message, 'Failed to converge', /INFORMATIONAL
    sigmaA = SQRT(ARRAY1(DIAG)/ALPHA(DIAG))
  endif
ENDFOR ; ITERATION LOOP
DONE_ITER:

;wait, 0.7

A=float(A)

RETURN, YFIT

END ; fufi_curvefit
