pro fufi_termdef1d
;
;------------------------------------------------------------------------------
;+
; NAME:
;   fufi_termdef1d
;
; PURPOSE:
;   Define the terms used by FUNCTIONFIND.
;
; CATEGORY:
;   Curve fitting
;
; CALLING SEQUENCE:
;   fufi_termdef1d
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None but some quantities in common block fufi_recur are defined.)
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   fufi_recur -
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Quantities in common block fufi_recur are defined.
;
; RESTRICTIONS:
;   This routine is usually called bY FUNCTIONFIND. Outside it is only of
;   limited use.
;
; PROCEDURE:
;    The quantities varname, idterm, terms00, terms0, nterms0 are defined.
;    Possible terms:
;    0:straight line, 1:parabola, 2:exponential, 3:logarithm, 4:rational,
;    5:Gaussian,      6:sine,     7:cosine,      8:atangens,  9:square root
;
; EXAMPLE:
;   fufi_termdef1d
;
; MODIFICATION HISTORY:
;  20. 9.95 Derived from FUNCTIONFIND by Bernd Freytag, Astrophysik Kiel
;  22. 2.05 (B.F. Uppsala) More comments
;-
;------------------------------------------------------------------------------
;
; === Common block ===
; --- Arrays describing the extension posibilities ---
common fufi_recur, varname, idterm, terms00, terms0, nterms0, termmask0, nmask
;
; === Put list of extension terms with partial derivatives into common blocks ==
;
; --- Descriptor of parameter ---
varname='A'
; --- Term used to describe the identity ('1.0' is not allowed) ---
idterm='(0.0*X+1.0)'
; --- List of extension terms ---
;     Maximum: 4 free parameters (npar)
;              4 sets of initial values (nextra)
;
; --- Template: constant ---
terms00={te1prop, $
  name:     'constant', $                        ; Name of term
  term:     '0.0*X+A(00)', $                     ; String with term
  npar:     1, $                                 ; Number of parameters
  allow:    [2, 0, 0, 0], $                      ; Allowed substitutions; 0: no substitution
                                                 ;                        1: allow anything except straight line
                                                 ;                        2: allow any substitution
  vary:     [1, 0, 0, 0], $                      ; Varied parameters
  termpders:[idterm   , ''        , ''        , ''         ], $ ; part. deriv.
  nextra:   1, $                                 ; Number of initial value sets
  aextra:  [[[0.0,1.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $ ; initial values
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
nterms0=10
terms0=replicate(terms00,nterms0)
;
; --- Straight line (recommended) ---
terms0(0)={te1prop, $
  name:     'straight line', $
  term:     'A(00)+A(01)*X', $
  npar:     2, $
  allow:    [1, 2, 0, 0], $
  vary:     [1, 1, 0, 0], $
  termpders:[idterm    , 'X'       , ''        , ''         ], $
  nextra:   1, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.6], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
; --- Parabola (not recommended) ---
terms0(1)={te1prop, $
  name:     'parabola', $
  term:     'A(00)+A(01)*X+A(02)*(X^2)', $
  npar:     3, $
  allow:    [1, 1, 2, 0], $
  vary:     [1, 1, 1, 0], $
  termpders:[idterm    , 'X'       , '(X^2)'   , ''         ], $
  nextra:   2, $
  aextra:  [[[0.0, 1.0], [0.0, 0.1], [0.0,-0.2], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.1], [0.0, 0.2], [0.0, 0.0]], $
            [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
; --- Exponential + constant (useful) ---
;terms0(2)={te1prop, $
;  name:     'exponential', $
;  term:     'A(00)+A(01)*exp(A(02)*X)', $
;  npar:     3, $
;  allow:    [2, 2, 2, 0], $
;  vary:     [1, 1, 1, 0], $
;  termpders:[idterm   ,'exp(A(02)*X)', 'A(01)*X*exp(A(02)*X)', ''       ], $
;  nextra:   2, $
;  aextra:  [[[0.0,0.9],[0.0, 0.1],      [0.8, 0.0],             [0.0, 0.0]], $
;            [[0.0,1.0],[0.0,-0.3],     [-0.8, 0.0],             [0.0, 0.0]], $
;            [[0.0,1.1],[0.0,-0.1],      [0.8, 0.0],             [0.0, 0.0]], $
;            [[0.0,1.0],[0.0, 0.3],     [-0.8, 0.0],             [0.0, 0.0]]] }
;
terms0(2)={te1prop, $
  name:     'exponential', $
  term:     'A(00)+A(01)*exp(A(02)+A(03)*X)', $
  npar:     4, $
  allow:    [2, 2, 1, 2], $
;  allow:    [2, 0, 0, 0], $
  vary:     [1, 1, 0, 1], $
  termpders:[idterm, $
               'exp(A(02)+A(03)*X)', $
                 'A(01)*exp(A(02)+A(03)*X)', $
                   'A(01)*X*exp(A(02)+A(03)*X)' ], $
  nextra:   2, $
  aextra:  [[[0.0,0.9], [0.0, 0.1],   [0.0, 0.0],  [ 0.8, 0.0]], $
            [[0.0,1.0], [0.0,-0.3],   [0.0, 0.0],  [-0.8, 0.0]], $
            [[0.0,1.1], [0.0,-0.1],   [0.0, 0.0],  [ 0.8, 0.0]], $
            [[0.0,1.0], [0.0, 0.3],   [0.0, 0.0],  [-0.8, 0.0]]] }
;
; --- Logarithm + constant (attention!) ---
terms0(3)={te1prop, $
  name:     'logarithm', $
  term:     'A(00)+A(01)*alog(abs(A(02)+A(03)*X))', $
  npar:     4, $
  allow:    [2, 2, 1, 2], $
  vary:     [1, 1, 0, 1], $
  termpders:[idterm, $
              'alog(abs(A(02)+A(03)*X))', $
               'A(01)/(A(02)+A(03)*X)', $
                'A(01)*X/(A(02)+A(03)*X)'], $
  nextra:   4, $
  aextra:  [[[0.0,0.9], [0.0, 0.1], [0.0, 1.0],  [0.2, 0.0]], $
            [[0.0,1.2], [0.0,-0.3], [0.0, 1.0], [-0.1, 0.0]], $
            [[0.0,1.0], [0.0,-0.1], [0.0, 1.0],  [0.3, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [0.0, 1.0], [-0.3, 0.0]]] }
;
; --- Simple rational function (attention!) ---
terms0(4)={te1prop, $
  name:     'rational', $
  term:     'A(00)+A(01)/(A(02)+A(03)*X)', $
  npar:     4, $
  allow:    [2, 2, 1, 2], $
  vary:     [1, 1, 0, 1], $
  termpders:[idterm, $
               '1.0/(A(02)+A(03)*X)', $
                  '(-A(01))/(A(02)+A(03)*X)^2', $
                     '(-A(01))*X/(A(02)+A(03)*X)^2'     ], $
  nextra:   2, $
  aextra:  [[[0.0,0.9], [0.0, 0.2], [1.0, 0.0], [ 0.4, 0.0]], $
            [[0.0,1.2], [0.0, 0.1], [1.0, 0.0], [ 0.5, 0.0]], $
            [[0.0,1.0], [0.0,-0.2], [1.0, 0.0], [-0.6, 0.0]], $
            [[0.0,0.9], [0.0,-0.1], [1.0, 0.0], [ 0.7, 0.0]]] }
;
; --- Gaussian + constant (might be usefull instead of exponential) ---
terms0(5)={te1prop, $
  name:     'Gaussian', $
  term:     'A(00)+A(01)*exp(A(02)*(X-A(03))^2)', $
  npar:     4, $
  allow:    [2, 2, 2, 2], $
  vary:     [1, 1, 1, 1], $
  termpders:[idterm, $
               'exp(A(02)*(X-A(03))^2)', $
                 'A(01)*(X-A(03))^2*exp(A(02)*(X-A(03))^2)', $
                   'A(01)*A(02)*(-2.0)*(X-A(03))*exp(A(02)*(X-A(03))^2)'], $
  nextra:   4, $
  aextra:  [[[0.0,1.1], [0.0,-0.2], [ -3.0, 0.0], [0.8, 0.0]], $
            [[0.0,0.5], [0.0, 0.5], [ -8.0, 0.0], [0.2, 0.0]], $
            [[0.0,0.1], [0.0, 0.9], [ -4.0, 0.0], [0.3, 0.0]], $
            [[0.0,1.0], [0.0,-0.4], [-10.0, 0.0], [0.5, 0.0]]] }
;
; --- Sine + constant ---
terms0(6)={te1prop, $
  name:     'sine', $
  term:     'A(00)+A(01)*sin(A(02)+A(03)*X)', $
  npar:     4, $
;  allow:    [2, 2, 1, 2], $
  allow:    [2, 0, 0, 0], $
  vary:     [1, 1, 1, 1], $
  termpders:[idterm, $
               'sin(A(02)+A(03)*X)', $
                  'A(01)*cos(A(02)+A(03)*X)', $
                     'A(01)*X*cos(A(02)+A(03)*X)'], $
  nextra:   4, $
  aextra:  [[[0.0,1.0], [0.0, 0.2], [ 0.2, 0.0], [ 6.3, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [-0.1, 0.0], [12.6, 0.0]], $
            [[0.0,0.9], [0.0, 0.5], [ 0.2, 0.0], [-4.5, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [-0.3, 0.0], [25.2, 0.0]]] }
;
; --- Cosine + constant ---
terms0(7)={te1prop, $
  name:     'cosine', $
  term:     'A(00)+A(01)*cos(A(02)+A(03)*X)', $
  npar:     4, $
  allow:    [2, 2, 1, 2], $
  vary:     [1, 1, 1, 1], $
  termpders:[idterm, $
               'cos(A(02)+A(03)*X)', $
                  'A(01)*(-sin(A(02)+A(03)*X))', $
                     'A(01)*X*(-sin(A(02)+A(03)*X))'], $
  nextra:   4, $
  aextra:  [[[0.0,0.8], [0.0, 0.2], [ 0.2, 0.0], [ 6.3, 0.0]], $
            [[0.0,1.3], [0.0,-0.3], [-0.1, 0.0], [12.6, 0.0]], $
            [[0.0,0.5], [0.0, 0.5], [ 0.2, 0.0], [-4.5, 0.0]], $
            [[0.0,0.7], [0.0, 0.3], [-0.3, 0.0], [25.2, 0.0]]] }
;
; --- Atan + constant ---
terms0(8)={te1prop, $
  name:     'atangens', $
  term:     'A(00)+A(01)*atan(A(02)+A(03)*X)', $
  npar:     4, $
  allow:    [2, 2, 1, 2], $
  vary:     [1, 1, 1, 1], $
  termpders:[idterm, $
               'atan(A(02)+A(03)*X)', $
                  'A(01)/(1.0+(A(02)+A(03)*X)^2)', $
                     'A(01)*X/(1.0+(A(02)+A(03)*X)^2)'], $
  nextra:   2, $
  aextra:  [[[0.0,1.0], [0.0, 0.2], [ 0.2, 0.0], [ 6.3, 0.0]], $
            [[0.0,1.0], [0.0,-0.3], [-0.1, 0.0], [12.6, 0.0]], $
            [[0.0,0.9], [0.0, 0.5], [ 0.2, 0.0], [-4.5, 0.0]], $
            [[0.0,0.7], [0.0, 0.3], [-0.3, 0.0], [25.2, 0.0]]] }
;
; --- Square root + constant (not recommended) ---
terms0(9)={te1prop, $
  name:     'square root', $
  term:     'A(00)+A(01)*sqrt(abs(A(02)+A(03)*X))', $
;  term:     'A(00)+A(01)*sqrt(abs(A(02)+X))', $
  npar:     4, $
  allow:    [2, 2, 1, 2], $
  vary:     [1, 1, 0, 1], $
  termpders:[idterm, $
              'sqrt(abs(A(02)+A(03)*X))', $
               '0.5*A(01)*sign(A(02)+A(03)*X)/sqrt(abs(A(02)+A(03)*X))', $
                 '0.5*A(01)*X*sign(A(02)+A(03)*X)/sqrt(abs(A(02)+A(03)*X))'], $
;               'sqrt(abs(A(02)+X))', $
;                 '0.5*A(01)*sign(A(02)+X)/sqrt(abs(A(02)+X))', $
;                   '0.5*X*A(01)*sign(A(02)+X)/sqrt(abs(A(02)+X))'], $
  nextra:   4, $
  aextra:  [[[0.0,0.3], [0.0, 0.7],  [0.1, 0.0], [0.8, 0.0]], $
            [[0.0,1.1], [0.0,-0.3],  [0.2, 0.0], [0.2, 0.0]], $
            [[0.0,0.2], [0.0, 0.3],  [0.1, 0.0], [2.5, 0.0]], $
            [[0.0,1.3], [0.0,-0.5],  [0.4, 0.0], [0.6, 0.0]]] }
;
end ; fufi_termdef1d
