pro fufi_termdef2d
;
;------------------------------------------------------------------------------
;+
; NAME:
;   fufi_termdef2d
;
; PURPOSE:
;   Define the (2D) terms used by FUNCTIONFIND.
;
; CATEGORY:
;   Curve fitting
;
; CALLING SEQUENCE:
;   fufi_termdef2d
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None (But some quantities in common block fufi_recur are defined.)
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   fufi_recur -
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Quantities in common block fufi_recur are defined.
;
; RESTRICTIONS:
;   This routine is usually called by FUNCTIONFIND. Outside, it is only of
;   limited use.
;
; PROCEDURE:
;    The quantities varname, idterm, terms00, terms0, nterms0 are defined.
;    Possible terms:
;      0: straight line,
;      1: parabola,
;      2: exponential,
;      3: polynomial_4
;      ... and more
;
; EXAMPLE:
;   fufi_termdef2d
;
; MODIFICATION HISTORY:
;  1995-09-20 (B.F. Kiel) Derived from FUFI_TERMDEF1D
;  1997-10-31 (B.F.) Parabola (No 1) added
;  1997-11-05 (B.F.) Polynomial 4th order added
;  2017-01-30 (B.F. Uppsala) Changed parantheses
;-
;------------------------------------------------------------------------------
;
; === Common block ===
; --- Arrays describing the extension posibilities ---
common fufi_recur, varname, idterm, terms00, terms0, nterms0, termmask0, nmask
;
; === Put list of extension terms with partial derivatives into common blocks ==
;
; --- Descriptor of parameter ---
varname='A'
; --- Term used to describe the identity ('1.0' is not allowed) ---
idterm='(0.0*X(*,0)+1.0)'
; --- List of extension terms ---
;     Maximum: 6 free parameters (npar)
;              4 sets of initial values (nextra)
;
; --- Template: constant ---
terms00={te2prop, $
  name:     'constant', $                        ; Name of term
  term:     '0.0*X(*,0)+A(00)', $                ; String with term
  npar:     1, $                                 ; Number of parameters
  allow:    [2, 0, 0, 0, 0, 0], $                ; Allowed substitutions
  vary:     [1, 0, 0, 0, 0, 0], $                ; Varied parameters (0: no variation, 1: variation possible)
  termpders:[idterm   , ''        , ''        , $
                  ''        ,''         , ''        ], $ ; part. derivatives
  nextra:   1 ,  $                                 ; Number of initial value sets
  aextra:  [[[0.0,1.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $ ; initial values
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
nterms0=17
terms0=replicate(terms00,nterms0)
;
; --- Plane surface (recommended) ---
terms0[0]={te2prop, $
  name:     'plane_surface', $
  term:     'A(00)+A(01)*X(*,0)+A(02)*X(*,1)', $
  npar:     3, $
  allow:    [1, 2, 2, 0, 0, 0], $
  vary:     [1, 1, 1, 0, 0, 0], $
  termpders:[idterm    , 'X(*,0)'  , 'X(*,1)'  , $
                  ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
; --- Parabola ---
terms0[1]={te2prop, $
  name:     'parabola', $
  term:     'A(00)+A(01)*(X(*,0)+A(02)*X(*,1))^2', $
  npar:     3, $
  allow:    [1, 2, 2, 0, 0, 0], $
  vary:     [1, 1, 1, 0, 0, 0], $
  termpders:[idterm    , '(X(*,0)+A(02)*X(*,1))^2'  , $
                         'A(01)*2.0*X(*,1)*(X(*,0)+A(02)*X(*,1))'  , $
                      ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 0.5, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [ 0.2, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [ 0.5, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.5], [-0.2, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
; --- Exponential + constant (useful) ---
terms0[2]={te2prop, $
  name:     'exponential', $
  term:     'A(00)+A(01)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1))', $
  npar:     5, $
  allow:    [2, 2, 1, 2, 2, 0], $
  vary:     [1, 1, 0, 1, 1, 0], $
  termpders:[idterm, $
              'exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1))', $
               'A(01)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1))', $
                'A(01)*X(*,0)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1))', $
                 'A(01)*X(*,1)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1))', $
                  ''                                                  ], $
  nextra:   4, $
  aextra:  [[[0.0,0.9], [0.0, 0.1], [0.0, 0.0], [ 0.8, 0.0], [ 0.8, 0.0], [0.0, 0.0]], $
            [[0.0,1.0], [0.0,-0.3], [0.0, 0.0], [-0.8, 0.0], [ 0.5, 0.0], [0.0, 0.0]], $
            [[0.0,1.1], [0.0,-0.1], [0.0, 0.0], [ 0.7, 0.0], [-0.4, 0.0], [0.0, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [0.0, 0.0], [-0.2, 0.0], [-0.6, 0.0], [0.0, 0.0]]] }
;
; --- Polynomial: 4. order ---
terms0[3]={te2prop, $
  name:     'polynomial_4', $
  term:   'A(00)+A(01)*(X(*,0)+A(02)*X(*,1))^2+' + $
                'A(03)*(X(*,0)+A(02)*X(*,1))^3+' + $
                'A(04)*(X(*,0)+A(02)*X(*,1))^4', $
  npar:     5, $
  allow:    [1, 2, 2, 2, 2, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
              '(X(*,0)+A(02)*X(*,1))^2'  , $
               'A(01)*2.0*X(*,1)*(X(*,0)+A(02)*X(*,1))+'   + $
               'A(03)*3.0*X(*,1)*(X(*,0)+A(02)*X(*,1))^2+' + $
               'A(04)*4.0*X(*,1)*(X(*,0)+A(02)*X(*,1))^3', $
                '(X(*,0)+A(02)*X(*,1))^3'  , $
                 '(X(*,0)+A(02)*X(*,1))^4'  , $
                  ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 0.5, 0.0], [0.0, 0.4], [0.0, 0.4], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [ 0.2, 0.0], [0.0,-0.1], [0.0,-0.1], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [ 0.5, 0.0], [0.0, 0.2], [0.0, 0.2], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.5], [-0.2, 0.0], [0.0,-0.3], [0.0,-0.3], [0.0, 0.0]]] }
;
; --- 2D Gaussian exp(-x^2) ---
terms0[4]={te2prop, $
  name:     'gaussian2', $
  term:   'A(00)+A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
              'exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))'  , $
               'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
               '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))' , $
                'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                '(-A(02)^2*(-2.0*(X(*,0)-A(03))))', $
                 'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                 '(-A(02)^2*(-2.0*(X(*,1)-A(04))))', $
                  ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 1.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.3], [ 1.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;
; --- 2D Gaussian +exp(-x^2) ---
terms0[5]={te2prop, $
  name:     'gaussian2p', $
  term:   'A(00)+A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
              '2.0*A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))'  , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
               '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))' , $
                'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                '(-A(02)^2*(-2.0*(X(*,0)-A(03))))', $
                 'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                 '(-A(02)^2*(-2.0*(X(*,1)-A(04))))', $
                  ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.3], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;
; --- 2D Gaussian -exp(-x^2) ---
terms0[6]={te2prop, $
  name:     'gaussian2m', $
  term:   'A(00)-A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
              '(-2.0*A(01))*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))'  , $
               '(-A(01)^2)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
               '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))' , $
                '(-A(01)^2)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                '(-A(02)^2*(-2.0*(X(*,0)-A(03))))', $
                 '(-A(01)^2)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                 '(-A(02)^2*(-2.0*(X(*,1)-A(04))))', $
                  ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.3], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;
; --- 2D Gaussian exp(-x^4) ---
terms0[7]={te2prop, $
  name:     'gaussian4', $
  term:   'A(00)+A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
              'exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)'  , $
               'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)*' + $
               '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)' , $
                'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)*' + $
                '(-A(02)^2*2.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)*(-2.0*(X(*,0)-A(03))))', $
                 'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)*' + $
                 '(-A(02)^2*2.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)*(-2.0*(X(*,1)-A(04))))', $
                  ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [10.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [20.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [30.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [40.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.5, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 1.5, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.3], [ 2.0, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;
; --- 2D Gaussian +exp(-x^4) ---
terms0[8]={te2prop, $
  name:     'gaussian4p', $
  term:   'A(00)+A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
             '2.0*A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)'  , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)*' +   '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)' , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)*' + '(-A(02)^2*2.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)*(-2.0*(X(*,0)-A(03))))', $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^2)*' + '(-A(02)^2*2.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)*(-2.0*(X(*,1)-A(04))))', $
             ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [10.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [20.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [30.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [40.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.5, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 2.5, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]]] }
;
; --- 2D Gaussian +exp(-x^8) ---
terms0[9]={te2prop, $
  name:     'gaussian8p', $
  term:   'A(00)+A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
             '2.0*A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)'  , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)*' +   '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)' , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)*' + '(-A(02)^2*4.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^3*(-2.0*(X(*,0)-A(03))))', $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)*' + '(-A(02)^2*4.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^3*(-2.0*(X(*,1)-A(04))))', $
              ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.5, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 2.5, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]]] }
;
;
; --- 2D Gaussian +exp(-x^8) ---
terms0[10]={te2prop, $
  name:     'gaussian8p', $
  term:   'A(00)+A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)', $
  npar:     5, $
  allow:    [1, 0, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
             '2.0*A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)'  , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)*' +   '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)' , $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)*' + '(-A(02)^2*4.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^3*(-2.0*(X(*,0)-A(03))))', $
               'A(01)^2*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^4)*' + '(-A(02)^2*4.0*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2)^3*(-2.0*(X(*,1)-A(04))))', $
              ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 103.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 144.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 195.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 305.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.5, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 2.5, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]]] }
;
; --- 2D Gaussian exp(-x^2) ---
terms0[11]={te2prop, $
  name:     'gaussian2a', $
  term:   'A(00)+A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))', $
  npar:     5, $
  allow:    [1, 2, 2, 2, 2, 0], $
  vary:     [1, 1, 1, 1, 1, 0], $
  termpders:[idterm    , $
              'exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))'  , $
               'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
               '(-2.0*A(02)*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))' , $
                'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                '(-A(02)^2*(-2.0*(X(*,0)-A(03))))', $
                 'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2+(X(*,1)-A(04))^2))*' + $
                 '(-A(02)^2*(-2.0*(X(*,1)-A(04))))', $
                  ''        ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 5.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 1.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.3], [ 1.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;
; --- 1D Gaussian exp(-x^2) ---
terms0[12]={te2prop, $
  name:     'gaussian1a', $
  term:   'A(00)+A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2))', $
  npar:     4, $
  allow:    [1, 2, 2, 2, 0, 0], $
  ;allow:    [1, 2, 0, 0, 0, 0], $
  vary:     [1, 1, 1, 1, 0, 0], $
  termpders:[idterm    , $
              'exp(-A(02)^2*((X(*,0)-A(03))^2))'  , $
               'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2))*' + $
               '(-2.0*A(02)*((X(*,0)-A(03))^2))' , $
                'A(01)*exp(-A(02)^2*((X(*,0)-A(03))^2))*' + $
                '(-A(02)^2*(-2.0*(X(*,0)-A(03))))', $
                  '',  ''      ], $
  nextra:   4, $
  aextra:  [[[0.2, 0.0], [0.3, 0.0], [ 3.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 4.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.2, 0.0], [0.3, 0.0], [ 5.3, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.3, 0.0], [0.2, 0.0], [ 5.8, 0.0], [-0.3, 0.0], [ 0.0, 0.0], [0.0, 0.0]]] }
;  aextra:  [[[0.0, 0.9], [0.0, 0.2], [ 1.0, 0.0], [ 0.0, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 0.9], [0.0, 0.5], [ 1.3, 0.0], [ 0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.4], [ 1.3, 0.0], [ 0.0, 0.0], [ 0.2, 0.0], [0.0, 0.0]], $
;            [[0.0, 1.0], [0.0, 0.3], [ 1.8, 0.0], [-0.3, 0.0], [-0.3, 0.0], [0.0, 0.0]]] }
;
terms0[13]={te2prop, $
  name:     'linear_1D_1', $
  term:     'A(00)+A(01)*X(*,0)', $
  npar:     2, $
  allow:    [1, 2, 0, 0, 0, 0], $
  vary:     [1, 1, 0, 0, 0, 0], $
  termpders:[idterm    , 'X(*,0)'  , ''  , $
                  ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
terms0[14]={te2prop, $
  name:     'linear_1D_2', $
  term:     'A(00)+A(01)*X(*,1)', $
  npar:     2, $
  allow:    [1, 2, 0, 0, 0, 0], $
  vary:     [1, 1, 0, 0, 0, 0], $
  termpders:[idterm    , 'X(*,1)'  , ''  , $
                  ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
; --- Exponential 1D + constant ---
terms0[15]={te2prop, $
  name:     'exponential_1D_1', $
  term:     'A(00)+A(01)*exp(A(02)+A(03)*X(*,0))', $
  npar:     4, $
  allow:    [2, 2, 1, 2, 0, 0], $
  vary:     [1, 1, 0, 1, 0, 0], $
  termpders:[idterm, $
              'exp(A(02)+A(03)*X(*,0))', $
               'A(01)*exp(A(02)+A(03)*X(*,0))', $
                'A(01)*X(*,0)*exp(A(02)+A(03)*X(*,0))', $
                 '', $
                  ''                                                  ], $
  nextra:   4, $
  aextra:  [[[0.0,0.9], [0.0, 0.1], [0.0, 0.0], [ 0.8, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.0,1.0], [0.0,-0.3], [0.0, 0.0], [-0.8, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.0,1.1], [0.0,-0.1], [0.0, 0.0], [ 0.7, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [0.0, 0.0], [-0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]]] }
;
; --- Exponential 1D + constant ---
terms0[16]={te2prop, $
  name:     'exponential_1D_2', $
  term:     'A(00)+A(01)*exp(A(02)+A(03)*X(*,1))', $
  npar:     4, $
  allow:    [2, 2, 1, 2, 0, 0], $
  vary:     [1, 1, 0, 1, 0, 0], $
  termpders:[idterm, $
              'exp(A(02)+A(03)*X(*,1))', $
               'A(01)*exp(A(02)+A(03)*X(*,1))', $
                'A(01)*X(*,1)*exp(A(02)+A(03)*X(*,1))', $
                 '', $
                  ''                                                  ], $
  nextra:   4, $
  aextra:  [[[0.0,0.9], [0.0, 0.1], [0.0, 0.0], [ 0.8, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.0,1.0], [0.0,-0.3], [0.0, 0.0], [-0.8, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.0,1.1], [0.0,-0.1], [0.0, 0.0], [ 0.7, 0.0], [ 0.0, 0.0], [0.0, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [0.0, 0.0], [-0.2, 0.0], [ 0.0, 0.0], [0.0, 0.0]]] }
;
end ; fufi_termdef2d
