pro fufi_termdef3d
;
;------------------------------------------------------------------------------
;+
; NAME:
;   fufi_termdef3d ('function_find_term_definition_3_dimensions')
;
; PURPOSE:
;   Define the (2D) terms used by FUNCTIONFIND.
;
; CATEGORY:
;   Curve fitting
;
; CALLING SEQUENCE:
;   fufi_termdef3d
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None (But some quantities in common block fufi_recur are defined.)
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   fufi_recur -
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Quantities in common block fufi_recur are defined.
;
; RESTRICTIONS:
;   This routine is usually called by FUNCTIONFIND. Outside, it is only of
;   limited use.
;
; PROCEDURE:
;    The quantities varname, idterm, terms00, terms0, nterms0 are defined.
;    Possible terms:
;    0:straight line, 1:parabola, 2:exponential, 3:polynomial_4
;
; EXAMPLE:
;   fufi_termdef3d
;
; MODIFICATION HISTORY:
;   20. 9.95 Derived from FUFI_TERMDEF1D by Bernd Freytag, Astrophysik Kiel
;   31.10.97 (B.F.) Parabola (No 1) added
;    5.11.97 (B.F.) Polynomial 4th order added
;   2009-04-21 (B.F. Lyon) 3D case
;   2011-10-19 (B.F.) exponential
;   2018-02-13 (B.F. Uppsala) linear_1D terms added
;-
;------------------------------------------------------------------------------
;
; === Common block ===
; --- Arrays describing the extension posibilities ---
common fufi_recur, varname, idterm, terms00, terms0, nterms0, termmask0, nmask
;
; === Put list of extension terms with partial derivatives into common blocks ==
;
; --- Descriptor of parameter ---
varname='A'
; --- Term used to describe the identity ('1.0' is not allowed) ---
idterm='(0.0*X(*,0)+1.0)'
; --- List of extension terms ---
;     Maximum: 6 free parameters (npar)
;              4 sets of initial values (nextra)
;
; --- Template: constant ---
terms00={te2prop, $
  name:     'constant', $                        ; Name of term
  term:     '0.0*X(*,0)+A(00)', $                ; String with term
  npar:     1, $                                 ; Number of parameters
  allow:    [2, 0, 0, 0, 0, 0], $                ; Allowed substitutions
  vary:     [1, 0, 0, 0, 0, 0], $                ; Varied parameters (0: no variation, 1: variation possible)
  termpders:[idterm   , ''        , ''        , ''        ,''         , ''        ], $ ; part. derivatives
  nextra:   1 ,  $                                 ; Number of initial value sets
  aextra:  [[[0.0,1.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $ ; initial values
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0,0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }

nterms0=5
terms0=replicate(terms00,nterms0)

; --- Plane surface (recommended) ---
terms0[0]={te2prop, $
  name:     'plane surface', $
  term:     'A(00)+A(01)*X(*,0)+A(02)*X(*,1)+A(03)*X(*,2)', $
  npar:     4, $
  allow:    [1, 2, 2, 2, 0, 0], $
  vary:     [1, 1, 1, 1, 0, 0], $
  termpders:[idterm    , 'X(*,0)'  , 'X(*,1)'  , 'X(*,2)'   , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.2], [0.0, 0.1], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.2], [0.0,-0.3], [0.0, 0.0], [0.0, 0.0]]] }

;
; --- Exponential + constant (useful) ---
terms0[1]={te2prop, $
  name:     'exponential', $
  term:     'A(00)+A(01)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1)+A(05)*X(*,2))', $
  npar:     6, $
  allow:    [2, 2, 1, 2, 2, 2], $
  vary:     [1, 1, 0, 1, 1, 1], $
  termpders:[idterm, $
              'exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1)+A(05)*X(*,2))', $
               'A(01)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1)+A(05)*X(*,2))', $
                'A(01)*X(*,0)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1)+A(05)*X(*,2))', $
                 'A(01)*X(*,1)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1)+A(05)*X(*,2))', $
                  'A(01)*X(*,2)*exp(A(02)+A(03)*X(*,0)+A(04)*X(*,1)+A(05)*X(*,2))'   ], $
  nextra:   4, $
  aextra:  [[[0.0,0.9], [0.0, 0.1], [0.0, 0.0], [ 0.8, 0.0], [ 0.8, 0.0], [ 0.8, 0.0]], $
            [[0.0,1.0], [0.0,-0.3], [0.0, 0.0], [-0.8, 0.0], [ 0.5, 0.0], [ 0.3, 0.0]], $
            [[0.0,1.1], [0.0,-0.1], [0.0, 0.0], [ 0.7, 0.0], [-0.4, 0.0], [ 0.2, 0.0]], $
            [[0.0,1.0], [0.0, 0.3], [0.0, 0.0], [-0.2, 0.0], [-0.6, 0.0], [-0.4, 0.0]]] }
;
terms0[2]={te2prop, $
  name:     'linear_1D_1', $
  term:     'A(00)+A(01)*X(*,0)', $
  npar:     2, $
  allow:    [1, 2, 0, 0, 0, 0], $
  vary:     [1, 1, 0, 0, 0, 0], $
  termpders:[idterm    , 'X(*,0)'  , ''  , $
                  ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
terms0[3]={te2prop, $
  name:     'linear_1D_2', $
  term:     'A(00)+A(01)*X(*,1)', $
  npar:     2, $
  allow:    [1, 2, 0, 0, 0, 0], $
  vary:     [1, 1, 0, 0, 0, 0], $
  termpders:[idterm    , 'X(*,1)'  , ''  , $
                  ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
terms0[4]={te2prop, $
  name:     'linear_1D_3', $
  term:     'A(00)+A(01)*X(*,2)', $
  npar:     2, $
  allow:    [1, 2, 0, 0, 0, 0], $
  vary:     [1, 1, 0, 0, 0, 0], $
  termpders:[idterm    , 'X(*,2)'  , ''  , $
                  ''        , ''        , ''        ], $
  nextra:   4, $
  aextra:  [[[0.0, 0.9], [0.0, 0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 0.9], [0.0, 0.5], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0,-0.2], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]], $
            [[0.0, 1.0], [0.0, 0.5], [0.0,-0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]] }
;
end ; fufi_termdef3d
