pro fufi_wridl, filename, varname, xoffset, xfactor, doffset, dfactor, dname=dname0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_wridl (FUnction_FInd_WRite_IDL)
;
; PURPOSE:
;   Create a file with IDL function
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   fufi_wridl, filename, varname, xoffset, xfactor, doffset, dfactor
;
; INPUT:
;   filename - Name of file to be written ('.pro' omitted).
;   varname - String vector with names of variables
;   xoffset - Real vector with x offsets
;   xfactor - Real vector with x factors
;   doffset - Real vector with data offsets
;   dfactor - Real vector with data factors
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   funct_string_common
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Create a file with an IDL function
;
; RESTRICTIONS:
;   The variables in funct_string_common have to be defined properly.
;
; PROCEDURE:
;   Simple: write the function given by F_string with the parameters given
;   by a_out into file in IDL format.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  31.10.97 Written by Bernd Freytag, Astrophysik Kiel
;  2009-05-05 (B.F. Lyon) Do scaling only if necessary
;  2009-05-09 (B.F.) Double precision parameter array
;-
;------------------------------------------------------------------------------

; --- Common block with function and partial derivatives ---
common funct_string_common, F_string, PDER_strings, a_out

if (keyword_set(dname0)) then begin
  dname=dname0
endif else begin
  dname='f0'
endelse

; --- Number of dimensions ---
nvar=n_elements(varname)
; --- Number of parameters ---
nA=n_elements(PDER_strings)

; --- Open new file for FORTRAN routine ---
get_lun, cunit
openw, cunit, filename + '.pro'

c=''

; --- Print headline ---
outvarname=strcompress(varname,/rem)
vars=strcompress(varname[0],/rem)
for n=1,nvar-1 do begin
  vars =vars  + ', ' + strcompress(varname[n],/rem)
endfor ; n
printf, cunit, c, 'function ' + filename + ', ' + vars

printf, cunit, '; --- ' + systime(0) + ' --- B.F. ---'
printf, cunit, c

; --- Declare empty parameter array ---
;printf, cunit, c, 'A=findgen(' + strcompress(string(nA),/rem) + ')'
printf, cunit, c, 'A=dindgen(' + strcompress(string(nA),/rem) + ')'
; --- Set parameter array ---
for n=0,nA-1 do begin
  printf, cunit, c, 'A(' + strmid('0' + strcompress(string(n),/rem), 1,/reverse) + ')=' + $
    strcompress(string(a_out(n),format='(E14.7)'),/rem) 
;    strcompress(string(a_out(n)),/rem) 
endfor ; n
printf, cunit, c

; --- Scale input parameters, if necessary ---
for n=0,nvar-1 do begin
  if ((xfactor[n] ne 1.0) or (xoffset[n] ne 0.0)) then begin
    printf, cunit, c, $
      varname[n] + '0=(' + varname[n] + $
                      '-' + strcompress(string(xoffset[n]),/rem) + $
                     ')/' + strcompress(string(xfactor[n]),/rem)
    outvarname[n]=outvarname[n] + '0'
  endif else begin
    ;printf, cunit, c, $
    ;  varname[n] + '0=' + varname[n]
  endelse
endfor ; n

; --- Compose the function string ---
Fstrnew=F_string
for n=0,nvar-1 do begin
  if (nvar eq 1) then parstrold='X' $
                 else parstrold='X(*,' + string(n,format='(I1)') + ')'
  ;
  if (n eq 0) then begin
    ; --- Simplify function string at first round ---
    ;
    ; --- Chop off first "null term" if it exists ---
    NullTerm='0.0*' + parstrold + '+'
    if (strmid(Fstrnew, 0, strlen(NullTerm)) eq NullTerm) then begin
      Fstrnew=strmid(Fstrnew, strlen(NullTerm))
    endif
    ;
    ; --- Remove leading and trailing round bracket ---
    if ((strmid(Fstrnew, 0, 1          ) eq '(') and $
        (strmid(Fstrnew, 0, 1, /reverse) eq ')')) then begin
      Fstrnew=strmid(Fstrnew, 1, strlen(Fstrnew)-2)
    endif
  endif
  ;
  pos=strpos(Fstrnew, parstrold, 0)
  ; --- Insert new variable instead of old term at all its positions ---
  termnew=outvarname[n]
  while (pos ge 0) do begin
    Fstrnew=strmid(Fstrnew, 0    , pos) + termnew  + $
            strmid(Fstrnew, pos+strlen(parstrold))
    pos=strpos(Fstrnew, parstrold, pos-strlen(parstrold)+strlen(termnew))
  endwhile
endfor
printf, cunit, c + dname + '=' +  Fstrnew
printf, cunit, c

; --- Return function value, including final scaling if necessary ---
if ((dfactor ne 1.0) or (doffset ne 0.0)) then begin
  printf, cunit, c, 'return, ' + dname + '*' + strcompress(dfactor,/rem) + $
                              '+' + strcompress(doffset,/rem)
endif else begin
  printf, cunit, c, 'return, ' + dname
endelse

; --- End of function ---
printf, cunit, c
printf, cunit, c, 'end ; ' + filename

; --- Close file ---
free_lun, cunit
close, cunit

end ; fufi_wridl
