pro funct_poly, X, A, F, Pder
;
;------------------------------------------------------------------------------
;+
; NAME:
;   funct_poly (FUNCTion_POLYnomial)
;
; PURPOSE:
;   Evaluate a polynomial
;   evaluate its partial derivatives.
;   Normally, this function is used by CURVEFIT.
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   funct_poly, X, A, F [, Pder]
;
; INPUTS:
;   X -	The values of the independent variable.
;   A - Parameters (1-20) of the equation described in PROCEDURE below.
;
; OUTPUTS:
;   F - The value of the function at each X(i).
;
; OPTIONAL OUTPUT PARAMETERS:
;   Pder - An array of the size (N_ELEMENTS(X),N_ELEMENTS(A)) that contains the
;          partial derivatives.  Pder(i,j) represents the derivative
;          at the i'th point with respect to j'th parameter.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;   F=A(0) + A(1)*X + A(2)*X^2 + ...
;
; EXAMPLE:
;   x=findgen(100)/10.
;   a=[-1.0,-5.0,1.0]
;   funct_poly, x, a, f, pder
;   plot, x, f
;
; MODIFICATION HISTORY:
;   6. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Number of parameters determines degree of polynomial ---
nA=n_elements(A)
; --- Number of elements ---
nX=n_elements(X)
;
; --- Compute function ---
F=replicate(A(nA-1),nX)
for n=2,nA do F=F*X + A(nA-n)
;
if (n_params() gt 3) then begin
  ; --- Compute partial derivatives ---
  PDER=replicate(1.0+A(0)*0.0,nX,nA)
  for n=1,nA-1 do PDER(*,n)=PDER(*,n-1)*X
endif
;
end ; funct_poly
