pro funct_string, X, A, F, Pder, Amask=Amask
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUNCT_STRING (FUNCTion_STRING)
;
; PURPOSE:
;   Evaluate a fuction given by a string.
;   evaluate its partial derivatives.
;   Normally, this function is used by CURVEFIT.
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   funct_string, X, A, F [, Pder, Amask=Amask]
;
; INPUTS:
;   X -	The values of the independent variable.
;   A - Parameters of the equation described in PROCEDURE below.
;
; OUTPUTS:
;   F - The value of the function at each X(i).
;
; OPTIONAL OUTPUT PARAMETERS:
;   Pder - An array of the size (N_ELEMENTS(X),N_ELEMENTS(A)) that contains the
;          partial derivatives.  Pder(i,j) represents the derivative
;          at the i'th point with respect to j'th parameter.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   funct_string_common
;   fufi_time
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Variable time_funct in common block fufi_time is updated.
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;   result=execute('F='+string)
;
; EXAMPLE:
;   x=findgen(100)/10.
;   a=[-1.0,-5.0,1.0]
;   funct_string, x, a, f, pder
;   plot, x, f
;
; MODIFICATION HISTORY:
;   7. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;  11. 9.95 (B.F.) Common block fufi_time added
;  15. 9.95 (B.F.) a_out added
;  19. 9.95 (B.F.) Keyword Amask added
;-
;------------------------------------------------------------------------------
;
; === Common blocks ===
; --- Common block with function and partial derivatives ---
common funct_string_common, F_string, PDER_strings, a_out
; --- Common block with runtime data ---
common fufi_time, time_fufi, time_cv, time_funct
;
; --- Save time of start of procedure ---
time_funct_start=systime(1)
;
if (n_params() eq 2) then a=a_out
;
; --- Number of parameters ---
nA=n_elements(A)
; --- Number of elements ---
;nX=n_elements(X)
nX=n_elements(X(*,0))
;
; --- Compute function ---
result=execute('F=' + F_string)
;
if (n_params() gt 3) then begin
  if (not keyword_set(Amask)) then begin
    Aind=indgen(n_elements(A))
  endif else begin
    Aind=where(Amask eq 1)
  endelse
  ; --- Compute partial derivatives ---
  nAind=n_elements(Aind)
  PDER=replicate(A(0)*0.0,nX,nAind)
  for n=0,nAind-1 do result=execute('PDER(*,n)=' + PDER_strings(Aind(n)))
endif
;
; --- Check math error status, disable printing of math errors ---
cm=check_math(0,1)
;
; --- Update time counter for this procedure ---
time_funct=time_funct + (systime(1)-time_funct_start)
;
end ; funct_string
