pro functionfind, X, Y, W, $
  help=help, $
  termmask=termmask00, nparmax=nparmax00, depthmax=depthmax00, $
    qopt_eps=qopt_eps00, nextramax=nextramax00, nextrarandom=nextrarandom00, $
    function_name=function_name00, $
  Fstrstart=Fstrstart, PDERstrstart=PDERstrstart, astart=astart, $
    allowtermstart=allowtermstart, varytermstart=varytermstart, $
  qopt=qopt, Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
    allowtermopt=allowtermopt, varytermopt=varytermopt, $
  print_terms0=print_terms0, print_blank0=print_blank0, print_F0=print_F0, $
    print_PDER0=print_PDER0, print_a0=print_a0, $
    print_time0=print_time0, print_iter0=print_iter0, $
    print_init0=print_init0, plot_test0=plot_test0, plot_fit0=plot_fit0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   functionfind ('function_find')
;
; PURPOSE:
;   Find a function and modify its parameters to yield an optimum fit for given X/Y/W values.
;
; CATEGORY:
;   Curve fitting
;
; CALLING SEQUENCE:
;   functionfind
;
; INPUT:
;   X:              ('X_values') Float vector or (nvalue x ndim) 2D array with nvalue X values
;   Y:              ('Y_values') Float vector with nvalue Y values
;   W:              ('Weights') Float vector with nvalue weights (optional, default=1.0)
;
; OUTPUT:
;   Output by keywords, see below
;
; KEYWORDS (INPUT):
;   help:           ('help') If set a short message is printed to the screen and the routine exits.
;   termmask:       ('term_mask') 2D integer array with codes of terms to be used in different substitution depths.
;                   For the codes, try 'functionfind, /help' and look into fufi_termdef1d.pro, fufi_termdef2d.pro, ...
;   nparmax:        ('number_parameters_maximum') Integer with maximum number of parameters to be considered.
;                   Used as condition to terminate the function search. Default: 5
;   depthmax:       ('depth_maximum') Integer with maximum substitution depth.
;                   Used as condition to terminate the function search. Default: nparmax-1
;   qopt_eps:       ('quantity_optimum_epsilon') Float with fitting accuracy to be achieved.
;                   Used as condition to terminate the function search. Default: 1.0E-14
;   nextramax:      ('number_extra_maximum') Integer with number of extra parameter combinations to try as
;                   start values for the non-linear parameter fitting. The maximum is typically 4. Default: 2.
;                   Larger values mean more trials and longer execution time. Linear fits don't need extra starts.
;   nextrarandom:   ('number_extra_random') Integer with number of extra random parameter combinations to try as
;                   start values for the non-linear parameter fitting. Default: 0.
;                   Larger values mean more trials and longer execution time. Linear fits don't need extra starts.
;   function_name:  ('function_name') String with first part of function (and file) name for candidate functions,
;                   default: 'funct_temp'
;   Fstrstart:      ('Function_string_start') String with the function to start with.
;                   Usually determined from termmask, so it has not to be specified on the input line.
;   PDERstrstart:   ('Partial_DERivatives_string_start') Partial derivatives of function to start with.
;                   Usually determined from termmask, so it has not to be specified on the input line.
;   astart:         ('a_start') Parameter vector to start with
;                   Usually determined from termmask, so it has not to be specified on the input line.
;   allowtermstart: ('allow_term_start') Integer vector with markers to indicate which parameters can be
;                   substituted (input).
;   varytermstart:  ('valy_term_start') Integer vector with markers to indicate which parameters can be
;                   varied during the parameter optimization (input).
;   VARIOUS FLAGS TO CONTROL PRINT AND PLOT OUTPUT:
;   print_terms0:
;   print_blank0:
;   print_F0:
;   print_PDER0:
;   print_a0:
;   print_time0:
;   print_iter0:
;   print_init0:
;   plot_test0:
;   plot_fit0:
;
; KEYWORDS (OUTPUT):
;   qopt:           ('quantity_optimum') Scalar float with the residual error of the optimum fit
;   Fstropt:        ('Function_string_optimum') The optimum function as (long) string (main result!)
;   PDERstropt      ('Partial_DERivatives_string_optimum') Partial derivatives of the optimum fit
;   aopt:           ('a_optimum') The real vector of optimum parameters (main result!)
;   allowtermopt:   ('allow_term_optimum') Integer vector with markers to indicate which parameters can be
;                   substituted (output).
;   varytermopt:    ('vary_term_optimum') Integer vector with markers to indicate which parameters can be
;                   varied during the parameter optimization (input).
;
; COMMON BLOCKS:
;   funct_string_common: Function and partial derivatives for CURVEFIT & FUNCT_STRING
;   fufi_common:         X/Y values to be fitted (with weights)
;   fufi_recur:          Arrays describing the extension posibilities
;   fufi_opt:            Arrays with global optimization quantities
;   fufi_time:           Runtime data
;   fufi_io:             I/O switches
;
; ROUTINES:
;   fufi_termdef1d: Defines the possible extension terms in 1D
;   fufi_termdef2d: Defines the possible extension terms in 2D
;   fufi_termdef3d: Defines the possible extension terms in 3D
;   fufi_termdef4d: Defines the possible extension terms in 4D
;   fufi_recur:     Does the recursive term substitution and searches best fit
;
; SIDE EFFECTS:
;   The routines writes lots of temporary IDL functions into the working directory!
;   Plotting, printing, modification of common blocks.
;
; RESTRICTIONS:
;   Be pessimistic when waiting for a fit (well, written in 1995...).
;     Still, the execution time increases drastically with number of
;     substitution levels and fitting parameters.
;   The routine is designed to do non-linear fitting. It is not quaranteed that it
;     finds the optimum solution or a reasonable solution at all (therefore all the
;     output options and the control parameters: you can help it).
;
; PROCEDURE:
;   Not really straightforward...
;
;   The idea is not just to find to given input data the optimum fitting parameters
;   for a given function, but to find the optimum function (i.e., polynomial or sine or
;   exponential) itself.
;
;   The important input parameters are X and Y with the discrete data points to fit.
;   The most important control parameters are termmask, nparmax, and depthmax.
;   The most important ouput is in aopt and Fstropt.
;   Lots of things are printed to the terminal and/or plotted in a window.
;
;   Usually, the routines is started once to find a certain fit.
;   However, the pair of parameter sets (Fstropt, ...) and (Fstrstart, ...) allow to
;   feed the output of one search (Fstropt, ...) as input (Fstrstart, ...) into a follow-up
;   search, possibly with different control parameters.
;
;   Unless specified otherwise, the first candidate function to fit is just a constant
;     A00 .
;   That is easy.  ;-)
;   In the next step, all parameters (unless specified otherwise) are substituted,
;   one after the other by a more complex term, e.g.:
;     A00 -> (A00 + A01*x + A02*x^2)
;   or
;     A00 -> (A00 + A01*sin(A02*x)) .
;   For the new set of parameters, optimum fits are searched based on the old parameter (A00)
;   and several combinations of possible start values.
;   Once the optimum solution (over all substitution terms and allowed start values) is found,
;   the process starts again with all parameters of the new optimum and so on, until the
;   maximum substitution depth, the maximum number of parameters, or the specified accuracy is reached.
;
;   The devil is in the detail, e.g., in the first substitution term example the parameter A00
;   should not be subsituted again by the same polynomial: there would be no extra degree of freedom.
;   The code accounts for some of these issues -- but does not take care of everything.
;
; EXAMPLE:
;   x=findgen(11)/10.0
;   y=sin(2*x)
;   w=x*0.0+1.0
;   functionfind, X, Y, W, aopt=A
;
; THINGS TO DO:
;  straight line
;  rational
;  parabola
;  square root
;  exponential
;  logarithm
;  asin
;  acos
;  tan
;  atangens
;  Gaussian
;  Bessel?
;  Voigt?
;  x^a? (5th parameter?)
;
; MODIFICATION HISTORY:
;   8. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;  10. 9.95 (B.F.) New form of terms (structure)
;  13. 9.95 (B.F.)
;  15. 9.95 (B.F.) Keywords and control parameter added, nextrarandom
;  19. 9.95 (B.F.) Property 'vary' of term added
;  20. 9.95 (B.F.) Procedure fufi_termdef1d introduced
;  22. 9.95 (B.F.) Procedure fufi_termdef2d introduced
;  24. 9.95 (B.F.) 2D plot, keywords for start function
;  27. 9.95 (B.F.) Keyword help added
;  21. 2.05 (B.F. Uppsala) More comments
;  2009-04-21 (B.F. Lyon) 3D and 4D case
;  2009-04-22 (B.F) More comments
;-
;------------------------------------------------------------------------------
;
; === Common blocks ===
; --- Function and partial derivatives for CURVEFIT & FUNCT_STRING
common funct_string_common, F_string, PDER_strings, a_out
; --- X/Y values to be fitted (with weights) ---
common fufi_common, Xvals, Yvals, Wvals
; --- Arrays describing the extension posibilities ---
common fufi_recur, varname, idterm, terms00, terms0, nterms0, termmask0, nmask
; --- Arrays with global optimization quantities ---
common fufi_opt, nparmax, depthmax, qopt_eps, qopt_global, function_name0, $
       nextramax, nextrarandom
; --- Common block with runtime data ---
common fufi_time, time_fufi, time_cv, time_funct
; --- Common block with I/O switches
common fufi_io, print_terms, print_blank, print_F, print_PDER, print_a, $
       print_time, print_iter, $
       print_init, plot_test, plot_fit
;
if (keyword_set(help)) then begin
  print, '1D: Possible terms (termmask):'
  print, '    0:straight line, 1:parabola, 2:exponential, 3:logarithm, 4:rational,'
  print, '    5:Gaussian,      6:sine,     7:cosine,      8:atangens,  9:square root'
  print, '2D: Possible terms (termmask):'
  print, '    0:plane surface, 1:parabola, 2:exponential, 3:polynomial_4'
  print, '3D: Possible terms (termmask):'
  print, '    0:plane surface'
  print, '4D: Possible terms (termmask):'
  print, '    0:plane surface, 1:plane surface (no further parameter substitution), 2:polynomial_4'
  return
endif
;
print, '---------------------------------------------------------------------'
;
; === Put X/Y/W values into common block variables ===
Xvals=X
Yvals=Y
if (n_params() le 2) then Wvals=Y*0.0+1.0 $
                     else Wvals=W
;
xsize=size(x)
ndim=xsize[0]
if (ndim eq 2) then ndim=xsize[2]
help,ndim
ndata=n_elements(y)
;
; === Put list of extension terms with partial derivatives into common blocks ==
if (ndim eq 1) then begin
  print, 'X is 1-dimensional.'
  fufi_termdef1d
endif else if (ndim eq 2) then begin
  print, 'X is 2-dimensional.'
  fufi_termdef2d
endif else if (ndim eq 3) then begin
  print, 'X is 3-dimensional.'
  fufi_termdef3d
endif else if (ndim eq 4) then begin
  print, 'X is 4-dimensional.'
  fufi_termdef4d
endif else begin
  message, 'X is >4 dimensional.'
endelse
;
if (n_elements(print_terms0) gt 0) then print_terms=keyword_set(print_terms0) $
                                   else print_terms=1
if (n_elements(print_blank0) gt 0) then print_blank=keyword_set(print_blank0) $
                                   else print_blank=1
if (n_elements(print_F0    ) gt 0) then print_F=keyword_set(print_F0) $
                                   else print_F=1
if (n_elements(print_PDER0 ) gt 0) then print_PDER=keyword_set(print_PDER0) $
                                   else print_PDER=0
if (n_elements(print_a0    ) gt 0) then print_a=keyword_set(print_a0) $
                                   else print_a=1
if (n_elements(print_time0 ) gt 0) then print_time=keyword_set(print_time0) $
                                   else print_time=1
if (n_elements(print_iter0 ) gt 0) then print_iter=print_iter0 $
                                   else print_iter=0
if (n_elements(print_init0 ) gt 0) then print_init=keyword_set(print_init0) $
                                   else print_init=0
if (n_elements(plot_test0  ) gt 0) then plot_test=keyword_set(plot_test0) $
                                   else plot_test=(ndim eq 1)
if (n_elements(plot_fit0   ) gt 0) then plot_fit=keyword_set(plot_fit0) $
                                   else plot_fit=1
;
; === Extension terms to be used ===
if (n_elements(termmask00) eq 0) then begin
  nmask=2
  termmask0=[[0,2], [0,2]]
endif else begin
  termmask0=termmask00
  termmask0size=size(termmask0)
  if (termmask0size(0) le 1) then begin
    nmask=1
  endif else begin
    nmask=termmask0size(2)
  endelse
endelse
;
; === Set start values ===
; --- Start with constant ---
if (keyword_set(Fstrstart) and keyword_set(PDERstrstart)) then begin
  ; --- Start function from input keywords ---
  Fstr=Fstrstart
  PDERstr=PDERstrstart
  nold=n_elements(PDERstr)
  if (keyword_set(allowtermstart)) then begin
    allowterm=allowtermstart
  endif else begin
    allowterm=replicate(2,nold)
  endelse
  if (keyword_set(varytermstart)) then begin
    varyterm=varytermstart
  endif else begin
    varyterm=replicate(1,nold)
  endelse
  if (keyword_set(astart)) then begin
    aold=astart
  endif else begin
    aold=replicate(0.5,nold)
  endelse
endif else begin
  ; --- Default start function ---
  nold     =terms00.npar
  Fstr     =terms00.term
  PDERstr  =terms00.termpders(0:nold-1)
  allowterm=terms00.allow(0:nold-1)
  varyterm =terms00.vary(0:nold-1)
  aold     =replicate(0.5,nold)
endelse
;
; === Set parameters to control the tree of term extensions ===
; --- Maximum number of parameters ---
if (n_elements(nparmax00) eq 0) then nparmax=5 $
                                else nparmax=0 > nparmax00 < 100
; --- Maximum depth of extension tree ---
if (n_elements(depthmax00) eq 0) then depthmax=0 > (nparmax-1) $
                                 else depthmax=0 > depthmax00 < 100
; --- Limit for quality: q<qopt_eps -> exit ---
if (n_elements(qopt_eps00) eq 0) then qopt_eps=1E-14 $
                                 else qopt_eps=qopt_eps00
; --- Maximum number of sets of initial values ---
if (n_elements(nextramax00) eq 0) then nextramax=2 $
                                  else nextramax=1 > nextramax00 < 4
; --- Maximum number of random initial values guesses ---
if (n_elements(nextrarandom00) eq 0) then nextrarandom=0 $
                                     else nextrarandom=0 > nextrarandom00
; --- Name of procedure with fit function ('funct_string' or 'funct_temp') ---
if (n_elements(function_name00) eq 0) then function_name0='funct_temp' $
                                      else function_name0=function_name00
;
; --- Extremely bad initial global optimum ---
qopt_global=1e36
;
if (print_init) then help, Fstr, PDERstr, allowterm, varyterm, aold, nold, $
  depthmax, termmask0, function_name0
if (print_terms) then for n=0,nmask-1 do $
  print, string(n,format='(X,I2)'), ' ', $
         terms0(termmask0(where(termmask0(*,n) ge 0),n)).name
;
; === Plot function to be fitted ===
if (plot_fit or plot_test) then begin
  if (ndim eq 1) then begin
    plot, x,y, ystyle=3
    oplot, x, y, thick=5, color=!ctv.red
    oplot, x, y, thick=1, color=!ctv.yellow
  endif else if (ndim eq 2) then begin
    ; --- Determine range of axes and data ---
    x0min=min(x(*,0),max=x0max)
    x1min=min(x(*,1),max=x1max)
    ymin=min(y,max=ymax)
    ;
    ; --- Produce regular grid ---
    nreg=25
    x0reg=x0min+(x0max-x0min)*findgen(nreg)/(nreg-1)
    x1reg=x1min+(x1max-x1min)*findgen(nreg)/(nreg-1)
    xreg=fltarr(nreg*nreg,2)
    xreg(*,0)=reform(reform(rebin(reform(x0reg,nreg,1),nreg,nreg),nreg*nreg,1))
    xreg(*,1)=reform(reform(rebin(reform(x1reg,1,nreg),nreg,nreg),nreg*nreg,1))
    ;
    ; --- 2 Plots on top of each other ---
    !P.MULTI=[0,0,2,0,0]
    !P.CHARSIZE=1.5
    !X.MARGIN=[8,2]
    ;
    ; --- Plot 3D coordinate system ---
    surface, [ymin,ymax]#[0.,1.], [x0min,x0max], [x1min, x1max], $
      /save, /nodata, $
      xtitle='X!D0!N', ytitle='X!D1!N', ztitle='Y', $
      xrange=[x0min,x0max], yrange=[x1min, x1max], zrange=[ymin,ymax], $
      xstyle=1, ystyle=1, zstyle=3
    ;
    ; --- Plot data points as set of crosses ---
    plots, reform(x(*,0)), reform(x(*,1)), y, psym=1, /t3d, symsize=2.5
  endif
endif
;
; === Reset global time counter ===
time_fufi_start=systime(1)
time_fufi=0.0
time_cv=0.0
time_funct=0.0
;
; *** Start Recursion ***
fufi_recur, Fstr, PDERstr, aold, allowterm, varyterm, nold, 0, $
  qopt, Fstropt, PDERstropt, aopt, allowtermopt, varytermopt
;
; === Determine runtime ---
time_fufi=systime(1)-time_fufi_start
if (print_time) then begin
  print, '---------------------------------------------------------------------'
  print, 'Runtime:  ', time_fufi,  ' s', format='(A,F9.3,A)'
  print, 'CURVEFIT: ', time_cv,    ' s', format='(A,F9.3,A)'
  print, 'FUNCT:    ', time_funct, ' s', format='(A,F9.3,A)'
endif
;
; === Put optimum fit into common block for FUNCT_STRING ===
F_string=Fstropt
PDER_strings=PDERstropt
a_out=aopt
;
; === Print optimum values ---
print, '---------------------------------------------------------------------'
print, 'qopt=    ', qopt
print, 'Fstropt= ', Fstropt
print, 'aopt=    ', aopt
print, '---------------------------------------------------------------------'
;
; === Plot optimum fit ===
if (plot_fit) then begin
  funct_string, x, aopt, y_fit
  if (ndim eq 1) then begin
    oplot, x, y, thick=1, color=!ctv.yellow
    oplot, x, y_fit, thick=2, color=!ctv.blue
  endif else if (ndim eq 2) then begin
    funct_string, xreg, aopt, y_fitreg
    ;
    for n=0,ndata-1 do begin 
      if (y(n) lt y_fit(n)) then begin 
        plots, [x(n,0),x(n,0)], [x(n,1),x(n,1)], [y(n),y_fit(n)], $
          color=!ctv.gray, /t3d 
        plots, [x(n,0)], [x(n,1)], [y(n)], $
          color=!ctv.gray, /t3d, psym=1, symsize=2.5
      endif 
    endfor
    ;
    !P.MULTI=[2,0,2,0,0]
    surface, reform(y_fitreg,nreg,nreg), x0reg, x1reg, $
      xrange=[x0min,x0max], yrange=[x1min, x1max], zrange=[ymin,ymax], $
      xstyle=5, ystyle=5, zstyle=7, $
      /save, /noerase, color=!ctv.red
    ;
    for n=0,ndata-1 do begin 
      if (y(n) ge y_fit(n)) then begin 
        plots, [x(n,0),x(n,0)], [x(n,1),x(n,1)], [y(n),y_fit(n)], /t3d 
        plots, [x(n,0)], [x(n,1)], [y(n)], $
          color=!P.COLOR, /t3d, psym=1, symsize=2.5 
      endif
    endfor
    ;
    ; --- Other plot ---
    !P.MULTI=[1,0,2,0,0]
    dyfmin=min(y-y_fit,max=dyfmax)
    ;
    surface, [ymin,ymax]#[0.,0.], [x0min,x0max], [x1min, x1max], $
      /save, /nodata, $
      xtitle='X!D0!N', ytitle='X!D1!N', ztitle='Y-Y!Dfit!N', $
      xrange=[x0min,x0max], yrange=[x1min, x1max], zrange=[dyfmin,dyfmax], $
      xstyle=1, ystyle=1, zstyle=3
    ;
    for n=0,ndata-1 do begin 
      if (y(n) lt y_fit(n)) then begin 
        plots, [x(n,0),x(n,0)], [x(n,1),x(n,1)], [y(n)-y_fit(n),0.0], $
               color=!ctv.gray, /t3d 
        plots, [x(n,0)], [x(n,1)], [y(n)-y_fit(n)], $
          color=!ctv.gray, /t3d, psym=1, symsize=2.5 
      endif 
    endfor
    ;
    !P.MULTI=[1,0,2,0,0]
    ;
    surface, 0.0*reform(y_fitreg,nreg,nreg), x0reg, x1reg, $
      xrange=[x0min,x0max], yrange=[x1min, x1max], zrange=[dyfmin,dyfmax], $
      xstyle=5, ystyle=5, zstyle=7, $
      /save, /noerase, color=!ctv.red
    ;
    for n=0,ndata-1 do begin 
      if (y(n) ge y_fit(n)) then begin 
        plots, [x(n,0),x(n,0)], [x(n,1),x(n,1)], [y(n)-y_fit(n),0.0], $
          color=!P.COLOR, /t3d 
        plots, [x(n,0)], [x(n,1)], [y(n)-y_fit(n)], $
          color=!P.COLOR, /t3d, psym=1, symsize=2.5 
      endif 
    endfor
    ;
    !P.MULTI=0
    !P.CHARSIZE=0
    !X.MARGIN=[12,3]
  endif
endif
;
end ; functionfind
