pro ima3_analyseregions, vol_in, count, resultslong, resultsfloat, $
  minpoints=minpoints0, shell=shell0, cellsizes=cellsizes
;   
;---------------------------------------------------------------------------------------------------
;+
; NAME: 
;   ima3_analyseregions ('image_analysis_3D_analyse_regions')
;
; PURPOSE:
;   Find isolated regions in a 3D mask and analyse it.
;
; CATEGORY:
;   3D image analysis
;
; CALLING SEQUENCE:
;   
; 
; INPUT:
;   vol_in: ('volume_input') Elements of value 1 are defined as inside the
;           mask, elements outside have value 0. No other values should exist in the
;           mask. The mask should have type BYTE, other types are untested.
; 
; OUTPUT:
;   findregions: ('find_regions') The returned array contains one element for
;                each isolated region in the mask. The elements value is the number of
;                voxels in the region. The returned array is of type LONG.
;   count:       ('count') Long integer, number of regions found
;   
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; PROCEDURE:   
;   The number of voxels in each region is counted and put in an array which is returned.
;
; RESTRICTIONS:
;   Don't use arrays which are larger or equal to 1000x1000x1000 elements:
;     As of now the returned array of region sizes is of type LONG, which means it can handle 
;     numbers up to ca. 2e9. This can be a limitation when using 3D arrays which are 
;     larger than 1000x1000x1000, i.e. when the size of each volume patch can exceed biggest
;     number possible. Then this function should be extended using LON64 (64 bits).
;
; EXAMPLE:
;   a=index2array('ind','g',36,cm) & sh=index2array('ind','gl',36,cm)
;
; MODIFICATION HISTORY:
;   3. april 2002 (Sigurdur Finnsson) The script is created.
;   5. 8.03 (B.F.) Some modifications, keyword 'count' added,
;                  keyword '/diagonal' in search3d
;   7. 8.03 (B.F.) Compute and analyse distance transform
;   8. 8.03 (B.F.) Compute mean position of each cell, more detailed output
;-
;---------------------------------------------------------------------------------------------------
;
if (keyword_set(minpoints0)) then begin
  minpoints=minpoints0
endif else begin
  minpoints=2L
endelse
;
; --- Remove superfluous external zeros ---
nvvec=size(vol_in, /dimensions)
if (keyword_set(shell0)) then begin
  shell=ima3_smallestbox(BYTE(shell0), border=1, mvec=mvecvol, nvec=nvecvol)
  vol=reform((BYTE(vol_in))(mvecvol(0):nvecvol(0)  , $
                            mvecvol(1):nvecvol(1)  , $
                            mvecvol(2):nvecvol(2)  ), $
                            nvecvol(0)-mvecvol(0)+1, $
                            nvecvol(1)-mvecvol(1)+1, $
                            nvecvol(2)-mvecvol(2)+1)
endif else begin
  vol=ima3_smallestbox(BYTE(vol_in), border=1, mvec=mvecvol)
endelse
;
; --- Coordinates of center ---
icvec=nvvec/2L
;
; --- Dummy index into returned data vectors ---
cellsizes=-999L
resultslong=lonarr(7)
resultsfloat=fltarr(10)
   
arrsize=size(vol, /DIMENSIONS)

for k=0L,(arrsize[2]-1L) do begin
  if (NOT ARRAY_EQUAL(vol[*,*,k], 0B)) then begin
    for j=0L,(arrsize[1]-1L) do begin
      if (NOT ARRAY_EQUAL(vol[*,j,k], 0B)) then begin
        for i=0L,(arrsize[0]-1L) do begin
          if (vol[i,j,k] ne 0B) then begin
            ; --- Find connected region ---
            region=search3d(vol, i,j,k, 1B, 1B, /diagonal)
            npoints=N_ELEMENTS(region)
            print, 'Found region at ', i+mvecvol(0), j+mvecvol(1), k+mvecvol(2), $
                   '  with ', npoints, ' points', $
                   format='(A,3I5,A,I10,A)'
            ;
            if (npoints ge minpoints) then begin
              ; === Region contains enough points: analyse further ===
              ;
              ; --- Extract relevant sub volume around region ---
              subvol=ima3_smallestbox(vol, indexlist=region, outindexlist=subregion, $
                                      border=1, mvec=mvecsubvol, nvec=nvecsubvol)
              nsvvec=size(subvol, /dimensions)
              subvol(*)=0B
              subvol(subregion)=1B
              subshell=reform(shell(mvecsubvol(0):nvecsubvol(0)  , $
                                    mvecsubvol(1):nvecsubvol(1)  , $
                                    mvecsubvol(2):nvecsubvol(2)  ), $
                                    nvecsubvol(0)-mvecsubvol(0)+1, $
                                    nvecsubvol(1)-mvecsubvol(1)+1, $
                                    nvecsubvol(2)-mvecsubvol(2)+1)
              ;
              ; --- Mean position ---
              pointpos=ima3_vectorindex(subregion, nsvvec) + $
                       rebin(reform(mvecsubvol+mvecvol - icvec, 1,3), npoints,3)
              pointpos_abs=sqrt(3.0*avg(float(pointpos)^2))
              pointpos_avg=avg(float(pointpos), 0)
              print, '  CM: ', pointpos_abs, pointpos_avg
              ;
              ; --- Distance transform of region (distance from boundary) ---
              DT=ima3_distancetransform(subvol, constrain=subshell)
              maxDT=max(DT, maxlocDT)
              ;
              maxDT_direct=maxDT
              maxlocDT_direct=ima3_scalarindex(ima3_vectorindex(maxlocDT, nsvvec)+ $
                                               mvecsubvol+mvecvol, nvvec)
              DTavg_direct=total(DT)/float(npoints)
              nDTlt1d5_direct=long(total(subvol(where(DT lt 1.5)))+0.5)
              print, '  DTd: max:', maxDT_direct, $
                     ' avg:', DTavg_direct, $
                     ' n(DT<1.5):', nDTlt1d5_direct, $
                     format='(A,F11.6,A,F11.6,A,I6)'
;              print, maxlocDT_direct, $
;                     ima3_vectorindex(maxlocDT_direct, nvvec), $
;                     subvol(maxlocDT), vol_in(maxlocDT_direct), $
;                     format='(2X,I10,2X,3I5,2X,2I5)'
              ;
              ; --- Distance transform of region (distance from maximum) ---
              lastmaxDT=maxDT
              repeat begin
                lastlastmaxDT=lastmaxDT
                lastmaxDT=maxDT
                lastmaxlocDT=maxlocDT
                subvolhole=subvol
                subvolhole(maxlocDT)=0B
                DT=ima3_distancetransform(subvolhole, constrain=subvol)
                maxDT=max(DT, maxlocDT)
                print, '  DTv: max:', maxDT, $
                       ' avg:', total(DT)/float(npoints), $
                       format='(A,F11.6,A,F11.6)'
              endrep until ((maxDT le (lastmaxDT    *1.000001)) or $
                            (maxDT le (lastlastmaxDT*1.000001)))
              maxDT_vol=maxDT
              maxlocDT_vol=ima3_scalarindex(ima3_vectorindex(maxlocDT, nsvvec)+ $
                                            mvecsubvol+mvecvol, nvvec)
              lastmaxlocDT_vol=ima3_scalarindex(ima3_vectorindex(lastmaxlocDT, nsvvec)+ $
                                                mvecsubvol+mvecvol, nvvec)
              DTavg_vol=total(DT)/float(npoints)
              ;
              ; --- Distance transform of shell (distance from maximum) ---
              maxDT=0.0
              lastmaxDT=0.0
              repeat begin
                lastlastmaxDT=lastmaxDT
                lastmaxDT=maxDT
                lastmaxlocDT=maxlocDT
                subshellhole=subshell
                subshellhole(maxlocDT)=0B
                DT=ima3_distancetransform(subshellhole, constrain=subshell, Fboundary=0B)
                DT=DT * float(subvol)
                maxDT=max(DT, maxlocDT)
                print, '  DTs: max:', maxDT, $
                       ' avg:', total(DT)/float(npoints), $
                       format='(A,F11.6,A,F11.6)'
              endrep until ((maxDT le (lastmaxDT    *1.000001)) or $
                            (maxDT le (lastlastmaxDT*1.000001)))
              maxDT_shl=maxDT
              maxlocDT_shl=ima3_scalarindex(ima3_vectorindex(maxlocDT, nsvvec)+ $
                                            mvecsubvol+mvecvol, nvvec)
              lastmaxlocDT_shl=ima3_scalarindex(ima3_vectorindex(lastmaxlocDT, nsvvec)+ $
                                                mvecsubvol+mvecvol, nvvec)
              DTavg_shl=total(DT)/float(npoints)
              ;
              ; --- Put results into vectors ---
              ; --- Volume (or surface) of the region found ---
              cellsizes=[cellsizes, npoints]
              reslong= [npoints, $
                        maxlocDT_direct, nDTlt1d5_direct, $
                        maxlocDT_vol,    lastmaxlocDT_vol, $
                        maxlocDT_shl,    lastmaxlocDT_shl]
              resfloat=[maxDT_direct, DTavg_direct, $
                        maxDT_vol,    DTavg_vol, $
                        maxDT_shl,    DTavg_shl, $
                        pointpos_abs, pointpos_avg]
;              print, reslong, format='(2X, 7I9)'
;              print, resfloat, format='(2X, 10F9.4)'
              resultslong =[[resultslong ], [reslong ]]
              resultsfloat=[[resultsfloat], [resfloat]]
            endif else begin
              print, '  ignored'
            endelse
            ;
            ; --- Zero the region so it wont be found again ---
            vol[region]=0B
            ;
          endif ; Endof checking each element in a line      
        endfor 
      endif  ; Endof zeroline 
    endfor
  endif ; Endof zeroplane
endfor
   
; cut away the extra cell I had to add in the beginning
if (n_elements(cellsizes) gt 1) then begin
  cellsizes=cellsizes(1:*)
  resultslong =resultslong(*,1:*)
  resultsfloat=resultsfloat(*,1:*)
  count=n_elements(cellsizes)
  ;
  sortind=reverse(sort(resultslong(0,*)))
  cellsizes   =cellsizes(     sortind)
  resultslong =resultslong( *,sortind)
  resultsfloat=resultsfloat(*,sortind)
endif else begin
  cellsizes=-1L
  resultslong=-1L
  resultsfloat=-1L
  count=0L
endelse

end ; ima3_analyseregions
