function ima3_distancetransform_sweep2d, Fin, DTin, constrain=constrain, $
  DTtype=DTtype, initial=initial, neighbors=neighbors
;---------------------------------------------------------------------------------------------------
; NAME:
;   ima3_distancetransform_sweep2d ('image_analysis_2D_distance_transform_sweep_2D')
;
; PURPOSE:
;   Perform single directional update sweep for distance transformation.
;
; CATEGORY:
;   2D/3D image processing
;
; CALLING SEQUENCE:
;   DTout=ima3_distancetransform_sweep2d(Fin, DTin [, constrain=constrain, $
;             initial=initial, DTtype=DTtype])
;
; INPUT:
;   Fin:       ('F_input') 2D byte array with set of points to be transformed
;   DTin:      ('Distance_Transform_input') 2D float array with result of previous sweeps
;
; OUTPUT:
;   DTout:     ('Distance_Transform_output') 2D float array with contribution of next sweep
;
; KEYWORDS:
;   constrain: ('constrain') 2D byte array with same size as Fin containing constraining mask.
;   DTtype:    ('Distance_Transform_type') scalar string with type of transformation
;                'cityblock'
;                'chessboard'
;                'quasieuclidian'
;                'closetoeuclidian2': Optimal local distances (default)
;   initial:   ('initial') if this flag is set DTin is ignored and a freshly initialized array
;              is used.
;   neighbors: ('neighbors') integer (0,1,2) limiting the number of neighbor cells considered,
;              useful for follow-up sweeps; 0 means all neighbors in mask
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   13. 8.03 Written by B.F.
;---------------------------------------------------------------------------------------------------
;
; --- Check input arrays ---
if (((n_params() lt 2) and (not keyword_set(initial))) or (n_params() lt 1)) then begin
  message, 'Not enough input parameters specified.'
endif
;
size_Fin=size(Fin)
if (size_Fin(0) ne 2) then begin
  message, 'Input array Fin does not have exactly 2 dimensions.'
endif
n1=size_Fin(1)
n2=size_Fin(2)
;
if (not keyword_set(initial)) then begin
  size_DTin=size(DTin)
  if (size_DTin(0) ne 2) then begin
    message, 'Input array DTin does not have exactly 2 dimensions.'
  endif
  if (not array_equal(size_Fin(1:2), size_Dtin(1:2))) then begin
    message, 'Input arrays are not conform.'
  endif
endif
;
; --- Type of distance transform ---
if (not keyword_set(DTtype)) then DTtype='closetoeuclidian2'
;
if ((strlowcase(DTtype)) eq 'cityblock') then begin
  n_a=1
  a10=1.0
  ;
endif else if ((strlowcase(DTtype)) eq 'chessboard') then begin
  n_a=2
  a10=1.0
  a11=1.0
  ;
endif else if ((strlowcase(DTtype)) eq 'quasieuclidian') then begin
  n_a=2
  a10=1.0
  a11=sqrt(2.0)
  ;
endif else if ((strlowcase(DTtype)) eq 'closetoeuclidian3') then begin
  ; --- Optimal local 3D distances (see Gunilla Borgefors 1984, Computer Vision, ---
  ; --- Graphics and Image Processing, 27, pp. 321-345)                       ---
  n_a=2
  sqrtterm=sqrt( 2.0 * sqrt(2.0) * (1.0 + sqrt(3.0)) - 7.0)
  a10=0.5 * (1.0 + sqrtterm)
  a11=0.5 * (2.0 * sqrt(2.0) - 1.0 + sqrtterm)
  ;
endif else begin
  ; --- Optimal local distances (see Gunilla Borgefors 1984, Computer Vision, ---
  ; --- Graphics and Image Processing, 27, pp. 321-345)                       ---
  n_a=2
  sqrtterm=sqrt(2.0 * sqrt(2.0) - 2.0)
  a10=0.5 * (sqrtterm + 1.0)
  a11=0.5 * (sqrtterm - 1.0) + sqrt(2.0)
  ;
endelse
;
; --- Limit number of neighbors cells considered ---
if (keyword_set(neighbors)) then begin
  n_a=n_a < (fix(neighbors) > 1)
endif
;
; --- Initialization of DTout from Fin (initial call) or DTin (otherwise) ---
fardistance=float(n1+n2)
if (keyword_set(initial)) then begin
  DTout=(float(Fin)) * fardistance
endif else begin
  DTout=DTin
endelse
;
;print, 'DTtype=', DTtype
;print, 'minmax(DTout)=', minmax(DTout)
;
if (keyword_set(constrain)) then begin
  DTout(0:n1-1,0)=DTout(0:n1-1,0) > $
                          ((1.0-constrain(0:n1-1,0))*fardistance)
endif
;
; --- One sweep ---
if (n_a eq 1) then begin
  for i2=1,n2-1 do begin
  ;  print, i2
;DTran=0.5*(randomu(seed, n1)-0.5)
;DTout(*,i2-1)=DTout(*,i2-1)+DTran
    DTout(1:n1-2,i2)=      DTout(1:n1-2, i2  ) < $
                      (a10+DTout(1:n1-2, i2-1))
;DTout(*,i2-1)=DTout(*,i2-1)-DTran
    if (keyword_set(constrain)) then begin
      DTout(0:n1-1,i2)=DTout(0:n1-1,i2) > $
                              ((1.0-constrain(0:n1-1,i2))*fardistance)
    endif
  endfor
endif else begin
  for i2=1,n2-1 do begin
  ;  print, i2
;DTran=0.5*(randomu(seed, n1)-0.5)
;DTout(*,i2-1)=DTout(*,i2-1)+DTran
    DTout(1:n1-2,i2)=      DTout(1:n1-2, i2  )  < $
                      (a10+DTout(1:n1-2, i2-1)) < $
                      (a11+DTout(0:n1-3, i2-1)) < $
                      (a11+DTout(2:n1-1, i2-1))
;DTout(*,i2-1)=DTout(*,i2-1)-DTran
    if (keyword_set(constrain)) then begin
      DTout(0:n1-1,i2)=DTout(0:n1-1,i2) > $
                              ((1.0-constrain(0:n1-1,i2))*fardistance)
    endif
  endfor
endelse
;
;print, 'minmax(DTout)=', minmax(DTout)
;
return, DTout
;
end ; ima3_distancetransform_sweep2d


function ima3_distancetransform_sweep3d, Fin, DTin, constrain=constrain, $
  DTtype=DTtype, initial=initial, neighbors=neighbors
;---------------------------------------------------------------------------------------------------
; NAME:
;   ima3_distancetransform_sweep3d ('image_analysis_3D_distance_transform_sweep_3D')
;
; PURPOSE:
;   Perform single directional update sweep for distance transformation.
;
; CATEGORY:
;   3D image processing
;
; CALLING SEQUENCE:
;   DTout=ima3_distancetransform_sweep3d(Fin, DTin [, constrain=constrain, $
;             initial=initial, DTtype=DTtype])
;
; INPUT:
;   Fin:       ('F_input') 3D byte array with set of points to be transformed
;   DTin:      ('Distance_Transform_input') 3D float array with result of previous sweeps
;
; OUTPUT:
;   DTout:     ('Distance_Transform_output') 3D float array with contribution of next sweep
;
; KEYWORDS:
;   constrain: ('constrain') 3D byte array with same size as Fin containing constraining mask.
;   DTtype:    ('Distance_Transform_type') scalar string with type of transformation
;                'cityblock'
;                'chessboard18'
;                'chessboard26'
;                'quasieuclidian'
;                'closetoeuclidian3': Optimal local distances (default)
;   initial:   ('initial') if this flag is set DTin is ignored and a freshly initialized array
;              is used.
;   neighbors: ('neighbors') integer (0,1,2,3) limiting the number of neighbor cells considered,
;              useful for follow-up sweeps; 0 means all neighbors in mask.
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    9. 7.03 Written by B.F.
;---------------------------------------------------------------------------------------------------
;
; --- Check input arrays ---
if (((n_params() lt 2) and (not keyword_set(initial))) or (n_params() lt 1)) then begin
  message, 'Not enough input parameters specified.'
endif
;
size_Fin=size(Fin)
if (size_Fin(0) ne 3) then begin
  message, 'Input array Fin does not have exactly 3 dimensions.'
endif
n1=size_Fin(1)
n2=size_Fin(2)
n3=size_Fin(3)
;
if (not keyword_set(initial)) then begin
  size_DTin=size(DTin)
  if (size_DTin(0) ne 3) then begin
    message, 'Input array DTin does not have exactly 3 dimensions.'
  endif
  if (not array_equal(size_Fin(1:3), size_Dtin(1:3))) then begin
    message, 'Input arrays are not conform.'
  endif
endif
;
; --- Type of distance transform ---
if (not keyword_set(DTtype)) then DTtype='closetoeuclidian3'
;
if ((strlowcase(DTtype)) eq 'cityblock') then begin
  n_a=1
  a100=1.0
  ;
endif else if ((strlowcase(DTtype)) eq 'chessboard18') then begin
  n_a=2
  a100=1.0
  a110=1.0
  ;
endif else if ((strlowcase(DTtype)) eq 'chessboard26') then begin
  n_a=3
  a100=1.0
  a110=1.0
  a111=1.0
  ;
endif else if ((strlowcase(DTtype)) eq 'quasieuclidian') then begin
  n_a=3
  a100=1.0
  a110=sqrt(2.0)
  a111=sqrt(3.0)
  ;
endif else begin
  ; --- Optimal local distances (see Gunilla Borgefors 1984, Computer Vision, ---
  ; --- Graphics and Image Processing, 27, pp. 321-345)                       ---
  n_a=3
  sqrtterm=sqrt( 2.0 * sqrt(2.0) * (1.0 + sqrt(3.0)) - 7.0)
  a100=0.5 * (1.0 + sqrtterm)
  a110=0.5 * (2.0 * sqrt(2.0) - 1.0 + sqrtterm)
  a111=0.5 * (2.0 * sqrt(3.0) - 1.0 + sqrtterm)
  ;
endelse
;
; --- Limit number of neighbors cells considered ---
if (keyword_set(neighbors)) then begin
  n_a=n_a < (fix(neighbors) > 1)
endif
;
; --- Initialization of DTout from Fin (initial call) or DTin (otherwise) ---
fardistance=float(n1+n2+n3)
if (keyword_set(initial)) then begin
  DTout=(float(Fin)) * fardistance
endif else begin
  DTout=DTin
endelse
;
;print, 'DTtype=', DTtype
;print, 'minmax(DTout)=', minmax(DTout)
;
if (keyword_set(constrain)) then begin
  DTout(0:n1-1,0:n2-1,0)=DTout(0:n1-1,0:n2-1,0) > $
                          ((1.0-constrain(0:n1-1,0:n2-1,0))*fardistance)
endif
;
; --- One sweep ---
if (n_a eq 1) then begin
  for i3=1,n3-1 do begin
  ;  print, i3
    DTout(1:n1-2,1:n2-2,i3)=      DTout(1:n1-2, 1:n2-2, i3  ) < $
                            (a100+DTout(1:n1-2, 1:n2-2, i3-1))
    if (keyword_set(constrain)) then begin
      DTout(0:n1-1,0:n2-1,i3)=DTout(0:n1-1,0:n2-1,i3) > $
                              ((1.0-constrain(0:n1-1,0:n2-1,i3))*fardistance)
    endif
  endfor
endif else if (n_a eq 2) then begin
  for i3=1,n3-1 do begin
  ;  print, i3
    DTout(1:n1-2,1:n2-2,i3)=      DTout(1:n1-2, 1:n2-2, i3  )  < $
                            (a100+DTout(1:n1-2, 1:n2-2, i3-1)) < $
                            (a110+DTout(0:n1-3, 1:n2-2, i3-1)) < $
                            (a110+DTout(2:n1-1, 1:n2-2, i3-1)) < $
                            (a110+DTout(1:n1-2, 0:n2-3, i3-1)) < $
                            (a110+DTout(1:n1-2, 2:n2-1, i3-1))
    if (keyword_set(constrain)) then begin
      DTout(0:n1-1,0:n2-1,i3)=DTout(0:n1-1,0:n2-1,i3) > $
                              ((1.0-constrain(0:n1-1,0:n2-1,i3))*fardistance)
    endif
;    if (keyword_set(constrain)) then begin
;      DTout(1:n1-2,1:n2-2,i3)=DTout(1:n1-2,1:n2-2,i3) > $
;                              ((1.0-constrain(1:n1-2,1:n2-2,i3))*fardistance)
;    endif
  endfor
endif else begin
  for i3=1,n3-1 do begin
  ;  print, i3
    DTout(1:n1-2,1:n2-2,i3)=      DTout(1:n1-2, 1:n2-2, i3  )  < $
                            (a100+DTout(1:n1-2, 1:n2-2, i3-1)) < $
                            (a110+DTout(0:n1-3, 1:n2-2, i3-1)) < $
                            (a110+DTout(2:n1-1, 1:n2-2, i3-1)) < $
                            (a110+DTout(1:n1-2, 0:n2-3, i3-1)) < $
                            (a110+DTout(1:n1-2, 2:n2-1, i3-1)) < $
                            (a111+DTout(0:n1-3, 0:n2-3, i3-1)) < $
                            (a111+DTout(2:n1-1, 0:n2-3, i3-1)) < $
                            (a111+DTout(0:n1-3, 2:n2-1, i3-1)) < $
                            (a111+DTout(2:n1-1, 2:n2-1, i3-1))
    if (keyword_set(constrain)) then begin
      DTout(0:n1-1,0:n2-1,i3)=DTout(0:n1-1,0:n2-1,i3) > $
                              ((1.0-constrain(0:n1-1,0:n2-1,i3))*fardistance)
    endif
  endfor
endelse
;
;print, 'minmax(DTout)=', minmax(DTout)
;
return, DTout
;
end ; ima3_distancetransform_sweep3d


function ima3_distancetransform, Fin, constrain=constrain, DTtype=DTtype, $
  Fboundary=Fboundary0, periodic=periodic
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   ima3_distancetransform ('image_analysis_3D_distance_transform')
;
; PURPOSE:
;   Compute a 2D/3D distance transformation.
;
; CATEGORY:
;   3D image processing
;
; CALLING SEQUENCE:
;   DT=ima3_distancetransform(Fin [, constrain=constrain, DTtype=DTtype, Fboundary=Fboundary,
;                                  periodic=periodic])
;
; INPUT:
;   Fin:       ('F_input') 2D/3D byte array with set of points to be transformed
;
; OUTPUT:
;   DT:        Real 2D/3D array of same size as Fin, with distance transform of Fin
;
; KEYWORDS:
;   constrain: ('constrain') 2D/3D byte array with same size as Fin containing constraining mask.
;              The distance transform is only computed within the mask, i.e. where (constrain eq 1B).
;              Points outside the mask, i.e. where (constrain eq 0B), are infinitely far away.
;   DTtype:    ('Distance_Transform_type') scalar string with type of transformation:
;              2D case:
;                'cityblock'        : Only the distance to the 4 immediate neighbors is 1
;                'chessboard'       : The distance to all 8 neighbors is 1
;                'quasieuclidian'   : Weights: 1.0, sqrt(2.0)
;                'closetoeuclidian2': Optimal local distances (default)
;              3D case:
;                'cityblock'        : Only the distance to the 6 immediate neighbors is 1
;                'chessboard18';    : The distance to 18 neirest neighbors is 1
;                'chessboard26'     : The distance to all 26 neighbors is 1
;                'quasieuclidian'   : Weights: 1.0, sqrt(2.0), sqrt(3.0)
;                'closetoeuclidian3': Optimal local distances (default)
;   Fboundary: ('F_boundary') scalar byte with value for boundary ghost zone
;   periodic:  ('periodic') if set periodic boundaries are assumed (only valid for 2D case)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   ima3_distancetransform_sweep2d: Compute one 2D sweep of transformation
;   ima3_distancetransform_sweep3d: Compute one 3D sweep of transformation
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The distance computed is only an approximation to the Euclidian distance.
;
; PROCEDURE:
;   Just do enough transformation sweeps by calling
;     ima3_distancetransform_sweep2d
;   or
;     ima3_distancetransform_sweep3d
;   appropriately.
;
; REFERENCES:
;   Gunilla Borgefors 1984, Computer Vision, Graphics and Image Processing, 27, pp. 321-345
;   Gunilla Borgefors 1994, Aspects of Visual Form Processing,
;     eds. Arcelli, Cordella, and Sanniti di Baja, World Scientific, Singapore 1994, pp.83-108
;
; EXAMPLE:
;   ; --- Artificial data ---
;   F=bytarr(8,8) & F(1:6,2:5)=1B
;   DT=ima3_distancetransform(F)
;   ;
;   ; --- CO5BOLD data: sign vertical velocity for horizontally periodic model ---
;   mask=reform(ful.z.v3(*,*,63)) gt 0.0
;   DT=ima3_distancetransform(mask, /periodic)
;   print, total(DT)/total(mask)
;
; MODIFICATION HISTORY:
;    9. 7.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;    7. 8.03 (B.F.) Renamed: distance_transform -> ima3_distancetransform
;   12. 8.03 (B.F.) 2D case included; periodic boundaries for 2D case allowed
;-
;---------------------------------------------------------------------------------------------------
;
; --- Check input arrays ---
if (n_params() lt 1) then begin
  message, 'No input parameter specified.'
endif
;
size_Fin=size(Fin)
if ((size_Fin(0) ne 2) and (size_Fin(0) ne 3)) then begin
  message, 'Input array Fin does not have 2 or 3 dimensions.'
endif
;
n1=size_Fin(1)
n2=size_Fin(2)
;
if (size_Fin(0) eq 2) then begin
  ; === 2D ===
  ;
  ; --- Create array with one extra element on all sides ---
  if (n_elements(Fboundary0) ne 0) then begin
    if (Fboundary0 ne 255B) then begin
      Fboundary=Fboundary0
    endif else begin
      Fboundary=Fin(0,0)
    endelse
  endif else begin
    Fboundary=Fin(0,0)
  endelse
  ;help, Fboundary0, Fboundary
  ;
  if (keyword_set(periodic)) then begin
    ; --- Periodic boundaries ---
    ;
    ; --- Fill increased temporary array ---
    Ftmp=make_array(3*n1+2,3*n2+2, value=Fboundary)
    Ftmp(     1:  n1,     1:  n2)=Fin
    Ftmp(  n1+1:2*n1,     1:  n2)=Fin
    Ftmp(2*n1+1:3*n1,     1:  n2)=Fin
    Ftmp(     1:  n1,  n2+1:2*n2)=Fin
    Ftmp(  n1+1:2*n1,  n2+1:2*n2)=Fin
    Ftmp(2*n1+1:3*n1,  n2+1:2*n2)=Fin
    Ftmp(     1:  n1,2*n2+1:3*n2)=Fin
    Ftmp(  n1+1:2*n1,2*n2+1:3*n2)=Fin
    Ftmp(2*n1+1:3*n1,2*n2+1:3*n2)=Fin
    ;
    ; --- Is there a mask for constrainment? ---
    if (keyword_set(constrain)) then begin
      ctmp=make_array(3*n1+2,3*n2+2, value=Fboundary)
      ctmp(     1:  n1,     1:  n2)=constrain
      ctmp(  n1+1:2*n1,     1:  n2)=constrain
      ctmp(2*n1+1:3*n1,     1:  n2)=constrain
      ctmp(     1:  n1,  n2+1:2*n2)=constrain
      ctmp(  n1+1:2*n1,  n2+1:2*n2)=constrain
      ctmp(2*n1+1:3*n1,  n2+1:2*n2)=constrain
      ctmp(     1:  n1,2*n2+1:3*n2)=constrain
      ctmp(  n1+1:2*n1,2*n2+1:3*n2)=constrain
      ctmp(2*n1+1:3*n1,2*n2+1:3*n2)=constrain
    endif else begin
      ctmp=0
    endelse
    ;
  endif else begin
    ; --- Standard boundaries ---
    ;
    ; --- Fill increased temporary array ---
    Ftmp=make_array(n1+2,n2+2, value=Fboundary)
    Ftmp(1:n1,1:n2)=Fin
    ;
    ; --- Is there a mask for constrainment? ---
    if (keyword_set(constrain)) then begin
      ctmp=make_array(n1+2,n2+2, value=Fboundary)
      ctmp(1:n1,1:n2)=constrain
    endif else begin
      ctmp=0
    endelse
  endelse
  ;
  initial_call=1
  ;
  ; --- Do the sweeps ---
  ;
  irun=0
  ;
  repeat begin
    ;
    DTout   =ima3_distancetransform_sweep2d(Ftmp     , DTout, constrain=ctmp     , $
                DTtype=DTtype, initial=initial_call)
    initial_call=0
    ;
    DTout   =reverse(DTout, 2)
    Ftmptrans=reverse(Ftmp, 2)
    if (keyword_set(constrain)) then ctmptrans=reverse(ctmp, 2)
    DTout   =ima3_distancetransform_sweep2d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype)
    ;
    DTout   =reverse(DTout, 2)
    ;
    ;
    DTout   =transpose(DTout)
    Ftmptrans=transpose(Ftmp)
    if (keyword_set(constrain)) then ctmptrans=transpose(ctmp)
    DTout   =ima3_distancetransform_sweep2d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype, neighbors=1)
    ;
    DTout   =reverse(DTout, 2)
    Ftmptrans=reverse(Ftmptrans, 2)
    if (keyword_set(constrain)) then ctmptrans=reverse(ctmptrans, 2)
    DTout   =ima3_distancetransform_sweep2d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype, neighbors=1)
    ;
    DTout   =reverse(DTout, 2)
    DTout   =transpose(DTout)
    ;
    if (keyword_set(constrain)) then begin
      if (irun eq 0) then begin
        end_flag=0
        ;print, irun, format='(I2)'
      endif else begin
        DT_change=total(abs(DTout-DTout_old))
        end_flag=(DT_change eq 0.0)
        ;print, irun, ' Change in DT:', DT_change, format='(I2, A, E12.5)'
      endelse
      if (not end_flag) then DTout_old=DTout
    endif else begin
      end_flag=1
    endelse
    ;
    irun=irun+1
    ;
  endrep until (end_flag)
  ;
  if (keyword_set(constrain)) then Dtout=Dtout*ctmp
  ;
  ; --- Remove extra layers ---
  if (keyword_set(periodic)) then begin
    DTout=DTout(n1+1:2*n1,n2+1:2*n2)
  endif else begin
    DTout=DTout(   1:  n1,   1:n2  )
  endelse
  ;
endif else begin
  ; === 3D ===
  ;
  n3=size_Fin(3)
  ;
  ; --- Create array with one extra element on all sides ---
  if (n_elements(Fboundary0) ne 0) then begin
    if (Fboundary0 ne 255B) then begin
      Fboundary=Fboundary0
    endif else begin
      Fboundary=Fin(0,0,0)
    endelse
  endif else begin
    Fboundary=Fin(0,0,0)
  endelse
  ;help, Fboundary0, Fboundary
  ;
  Ftmp=make_array(n1+2,n2+2,n3+2, value=Fboundary)
  Ftmp(1:n1,1:n2,1:n3)=Fin
  ;
  ; --- Is there a mask for constrainment? ---
  if (keyword_set(constrain)) then begin
    ctmp=make_array(n1+2,n2+2,n3+2, value=Fboundary)
    ctmp(1:n1,1:n2,1:n3)=constrain
  endif else begin
    ctmp=0
  endelse
  ;
  initial_call=1
  ;
  ; --- Do the sweeps ---
  ;
  irun=0
  ;
  repeat begin
    ;
    DTout   =ima3_distancetransform_sweep3d(Ftmp     , DTout, constrain=ctmp     , $
               DTtype=DTtype, initial=initial_call)
    initial_call=0
    ;
    DTout   =reverse(DTout, 3)
    Ftmptrans=reverse(Ftmp, 3)
    if (keyword_set(constrain)) then ctmptrans=reverse(ctmp, 3)
    DTout   =ima3_distancetransform_sweep3d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype)
    ;
    DTout   =reverse(DTout, 3)
    ;
    ;
    DTout   =transpose(DTout, [1,2,0])
    Ftmptrans=transpose(Ftmp, [1,2,0])
    if (keyword_set(constrain)) then ctmptrans=transpose(ctmp, [1,2,0])
    DTout   =ima3_distancetransform_sweep3d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype, neighbors=2)
    ;
    DTout   =reverse(DTout, 3)
    Ftmptrans=reverse(Ftmptrans, 3)
    if (keyword_set(constrain)) then ctmptrans=reverse(ctmptrans, 3)
    DTout   =ima3_distancetransform_sweep3d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype, neighbors=2)
    ;
    DTout   =reverse(DTout, 3)
    DTout   =transpose(DTout, [2,0,1])
    ;
    ;
    DTout   =transpose(DTout, [2,0,1])
    Ftmptrans=transpose(Ftmp, [2,0,1])
    if (keyword_set(constrain)) then ctmptrans=transpose(ctmp, [2,0,1])
    DTout   =ima3_distancetransform_sweep3d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype, neighbors=1)
    ;
    DTout   =reverse(DTout, 3)
    Ftmptrans=reverse(Ftmptrans, 3)
    if (keyword_set(constrain)) then ctmptrans=reverse(ctmptrans, 3)
    DTout   =ima3_distancetransform_sweep3d(Ftmptrans, DTout, constrain=ctmptrans, $
               DTtype=DTtype, neighbors=1)
    ;
    DTout   =reverse(DTout, 3)
    DTout   =transpose(DTout, [1,2,0])
    ;
    if (keyword_set(constrain)) then begin
      if (irun eq 0) then begin
        end_flag=0
        ;print, irun, format='(I2)'
      endif else begin
        DT_change=total(abs(DTout-DTout_old))
        end_flag=(DT_change eq 0.0)
        ;print, irun, ' Change in DT:', DT_change, format='(I2, A, E12.5)'
      endelse
      if (not end_flag) then DTout_old=DTout
    endif else begin
      end_flag=1
    endelse
    ;
    irun=irun+1
    ;
  endrep until (end_flag)
  ;
  if (keyword_set(constrain)) then Dtout=Dtout*ctmp
  ;
  ; --- Remove extra layers ---
  DTout=DTout(1:n1,1:n2,1:n3)
  ;
endelse

return, DTout
;
end ; ima3_distancetransform
