function ima3_scalarindex, vectorindex, nvec
;
;------------------------------------------------------------------------------
;+
; NAME:
;   ima3_scalarindex ('image_analysis_3D_vector_index')
;
; PURPOSE:
;   Transform vector index into index number.
;
; CATEGORY:
;   3D image analysis
;
; CALLING SEQUENCE:
;   scalarindex=ima3_scalarindex(vectorindex, nvec)
;
; INPUT:
;   vectorindex: ('vector_index') Long integer 2-D array with
;                  (n_elements(scalarindex), n_elements(nvec))
;                elements containing the dimensional array indices
;   nvec:        ('number_vector') (Long) integer vector with array size,
;                e.g. from "size(array, /dimension)"
;
; OUTPUT:
;   scalarindex: ('scalar_index') Long integer vector with array index numbers,
;                useful e.g. in "array(scalarindex)=0.0"
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Straightforward.
;
; EXAMPLE:
;   vectorindex=long([[1,1,2],[0,1,1]])
;   nvec=[3,4]
;   scalarindex=ima3_scalarindex(vectorindex, nvec)
;   print, scalarindex
;     1           4           5
;
; MODIFICATION HISTORY:
;    6. 8.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
; --- Dimension checks ---
ndim=n_elements(nvec)
sizenv=size(nvec)
sizevi=size(vectorindex)
if (sizenv(0) ne 1) then begin
  message, 'Dimension vector must be one-dimensional'
endif
if (ndim eq 1) then begin
  if (not ((sizevi(0) eq 1) or ((sizevi(0) eq 2) and (sizevi(2) eq ndim)))) then begin
    message, 'Index array must be one- or two-dimensional'
  endif
  if (1 ne ndim) then begin
    message, 'Number of indices per element of index array disagrees with length of dimension vector'
  endif
endif else begin
  if (sizevi(0) ne 2) then begin 
    message, 'Index array must be two-dimensional'
  endif
  if (sizevi(2) ne ndim) then begin
    message, 'Number of indices per element of index array disagrees with length of dimension vector'
  endif
endelse
;
; --- Transform indices ---
scalarindex=vectorindex(*,ndim-1)
for i=ndim-2,0,-1 do begin
  scalarindex=scalarindex*nvec(i) + vectorindex(*,i)
endfor
;
return, scalarindex
;
end ; ima3_scalarindex
