pro coma_opac_rd, path, model, $
  P=P, T=T, rho=rho, $
  EleSym=EleSym, EleAbu1=EleAbu1, EleAbu2=EleAbu2, $
  kappaRoss=kappaRoss, kappaPlan=kappaPlan, kappanu=kappanu, $
  WaveNumber=WaveNumber, lambda=lambda, nu=nu
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   coma_opac_rd ('coma_opacity_read_all')
;
; PURPOSE:
;   Read list of COMA raw opacity data files and sort them into 3D array
;
; CATEGORY:
;   I/O
;
; CALLING SEQUENCE:
;   coma_opac_rd, path, model, $
;     P=P, T=T, rho=rho, $
;     EleSym=EleSym, EleAbu1=EleAbu1, EleAbu2=EleAbu2, $
;     kappaRoss=kappaRoss, kappaPlan=kappaPlan, kappanu=kappanu, $
;     WaveNumber=WaveNumber, lambda=lambda, nu=nu
;
; INPUT:
;   path:         ('path') string with path to opacity data
;   model:        ('model') string with model name. If the data is spread over several files
;                 wildcards can be used. The file suffices ".out" and ".dat" have to be removed.
;
; KEYWORDS (OUTPUT):
;   T:            ('Temperature') 1D real vector with temperature values
;   P:            ('Pressure') 1D real vector with pressure values
;   rho:          ('rho') 2D real array with density values
;   EleSym:       ('Element_Symbol') 1D string vector with names of elements
;   EleAbu1:      ('Element_Abundance_1') 1D real vector with low-precision element abundance
;   EleAbu2:      ('Element_Abundance_2') 1D real vector with high-precision element abundance
;   kappaRoss:    ('kappa_Rosseland') 2D real array with Rosseland opacities
;   kappaPlan:    ('kappa_Planck') 2D real array with Planck opacities
;   kappanu:      ('kappa_nu') 3D real array with frequency-dependent opacities (large)
;                 of the form kappa(T,P,nu) as function of temperature, pressure, and wavelength
;   WaveNumber:   ('Wave_Number') 1D real vector with wavenumber values
;   lambda:       ('lambda') 1D real vector with wavelength values
;   nu:           ('nu') 1D real vector with frequency values
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   Files are opened and closed.
;
; RESTRICTIONS:
;   Some data file properties are "hardwired" into the routine.
;
; PROCEDURE:
;
; EXAMPLE:
;   coma_opac_rd, '/Users/befre677/dat/opa/BernhardAringer/', 'kapnu_solabu_p[123]', P=P, T=T, rho=rho, kappanu=kappanu, kappaRoss=kappaRoss, kappaPlan=kappaPlan, lambda=lambda, nu=nu
;
; MODIFICATION HISTORY:
;   2007-09-20 (B.F. Lyon) rd_sort_opac.pro -> phoenix_opac_rdall.pro
;                          output not into file but arrays
;   2014-04-15 (B.F. Uppsala) phoenix_opac_rdall.pro -> coma_opac_rd.pro
;   2015-08-26 (B.F.) Rename paramter EleNam -> EleSym
;-
;---------------------------------------------------------------------------------------------------

; --- Get initial time ---
time0=systime(/SECONDS)

; --- Produce expanded list of '.out' input files ---
get_lun, Unit
PathAndFile=path + model + '.out'
print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, look for out files ', PathAndFile, format='(A,F7.2,A,A)'
Files=findfile(PathAndFile, count=nFiles)

nPTarr=lonarr(nFiles)

Line=''
nPTall=0L

; --- Loop over all input files ---
for iFile=0,nFiles-1 do begin
  ; --- Get unit number and open file ---
  print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, handling ', Files[iFile], format='(A,F7.2,A,A)'
  openr, Unit, Files[iFile]
  ;
  iline=0L
  ;
  ; --- Read Header ---
  readf, Unit, Line & iLine=iLine+1L
  Header=Line
  print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, header: ', Header, format='(A,F7.2,A,A)'
  ;
  ; --- Find number of depth points ---
  while not eof(Unit) do begin
    ; --- Read next line ---
    readf, Unit, Line & iLine=iLine+1L
    ;
    if (strpos(Line, 'revised number of depth points') gt 0) then begin
      nPT          =long(strmid(Line, 0, 5))
      nPTarr[iFile]=nPT
      nPTall       =nPTall+nPT
      print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, number of P/T points ', nPT, format='(A,F7.2,A,I6)'
      break
    endif
  endwhile
  ;
  ; --- Find number of frequency points ---
  while not eof(Unit) do begin
    ; --- Read next line ---
    readf, Unit, Line & iLine=iLine+1L
    ;
    if (strpos(Line, 'vspmin,vspmax,nwnl:') gt 0) then begin
      nnu=long(strmid(Line, 50, 20))
      print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, number of frequency points ', nnu, format='(A,F7.2,A,I8)'
      break
    endif
  endwhile
  ;
  close, Unit
endfor ; iFile

print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, total number of P/T points ', nPTall, format='(A,F7.2,A,I6)'

; --- Allocate arrays ---
T        =fltarr(nPTall)
P        =fltarr(nPTall)
rho      =fltarr(nPTall)
kappaRoss=fltarr(nPTall)
kappaPlan=fltarr(nPTall)

iPTall=0L

; --- Loop over all input files ---
for iFile=0,nFiles-1 do begin
  ; --- Get unit number and open file ---
  print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, handling ', Files[iFile], format='(A,F7.2,A,A)'
  openr, Unit, Files[iFile]
  ;
  iline=0L
  ;
  ; --- Read Header ---
  readf, Unit, Line & iLine=iLine+1L
  Header=Line
  print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, header: ', Header, format='(A,F7.2,A,A)'
  ;
  ; --- Find number of depth points ---
  while not eof(Unit) do begin
    ; --- Read next line ---
    readf, Unit, Line & iLine=iLine+1L
    ;
    if (strpos(Line, 'revised number of depth points') gt 0) then begin
      nPT=long(strmid(Line, 0, 5))
      print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, number of P/T points ', nPT, format='(A,F7.2,A,I6)'
      break
    endif
  endwhile
  ;
  ; --- Skip 7 lines ---
  readf, unit, Line & iLine=iLine+1L
  readf, unit, Line & iLine=iLine+1L
  readf, unit, Line & iLine=iLine+1L
  readf, unit, Line & iLine=iLine+1L
  readf, unit, Line & iLine=iLine+1L
  readf, unit, Line & iLine=iLine+1L
  readf, unit, Line & iLine=iLine+1L
  ;
  ; --- Read table with temperature and pressure ---
  a=fltarr(5, nPT)
  readf, Unit, a
  T1=reform(a[2,*])
  P1=reform(a[3,*])
  T[iPTall:iPTall+nPT-1]=T1
  P[iPTall:iPTall+nPT-1]=P1
  ;
  ; --- Find and read abundance info block ---
  while not eof(Unit) do begin
    readf, Unit, Line & iLine=iLine+1L
    ;
    if (strpos(Line, 'Input abundances 1-') gt 0) then begin
      nEle=75
      EleSym =strarr(nEle)
      EleAbu1=fltarr(nEle)
      EleAbu2=fltarr(nEle)
      ;
      s15=strarr(15)
      f15=fltarr(15)
      ;
      for i=0,4 do begin
        if (i gt 0) then readf, Unit, Line & iLine=iLine+1L
        print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, ', Line, format='(A,F7.2,A,A)'
        ;
        readf, Unit, Line & iLine=iLine+1L
        ;
        readf, Unit, s15, format='(15(3X,A2,3X))' & iLine=iLine+1L
        EleSym[i*15:i*15+14]=s15
        readf, Unit, f15, format='(15(F7.4,1X))' & iLine=iLine+1L
        EleAbu1[i*15:i*15+14]=f15
        readf, Unit, f15, format='(15(F7.4,1X))' & iLine=iLine+1L
        EleAbu2[i*15:i*15+14]=f15
        ;
        readf, Unit, Line & iLine=iLine+1L
      endfor ; i
      ;
      break
    endif
  endwhile
  if (n_elements(EleSym) gt 0) then EleSym=strtrim(EleSym, 2)
  ;
  ; --- Find and read mean kappa block ---
  while not eof(Unit) do begin
    readf, Unit, Line & iLine=iLine+1L
    ;
    if (strpos(Line, 'K          T(K)         RO(K)         PG(K)         PE(K)    KAPPA_ross  KAPPA_planck') gt 0) then begin
      a=fltarr(9, nPT)
      readf, Unit, a
      rho[      iPTall:iPTall+nPT-1]=reform(a[2,*])
      kappaRoss[iPTall:iPTall+nPT-1]=reform(a[5,*])
      kappaPlan[iPTall:iPTall+nPT-1]=reform(a[6,*])
      ;
      break
    endif
  endwhile
  ;
  iPTall=iPTall+nPT
  ;
  close, Unit
endfor

; --- Produce expanded list of '.dat' input files ---
PathAndFile=path + model + '.dat'
print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, look for dat files ', PathAndFile, format='(A,F7.2,A,A)'
Files=findfile(PathAndFile, count=ndatFiles)
if (ndatFiles ne nFiles) then begin
  message, 'Numbers of out and dat files are not the same.'
endif else begin
  nFiles=ndatFiles
endelse

; --- Allocate arrays ---
WaveNumber=fltarr(        nnu)
kappanu   =fltarr(nPTall, nnu)

iPTall=0L

; --- Loop over all input files ---
for iFile=0,nFiles-1 do begin
  ; --- Get unit number and open file ---
  print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, handling ', Files[iFile], format='(A,F7.2,A,A)'
  openr, Unit, Files[iFile]
  ;
  iline=0L
  ;
  ; --- Find first block with wavelength-dependent opacities ---
  while not eof(Unit) do begin
    ; --- Read next line ---
    readf, Unit, Line & iLine=iLine+1L
    ;
    if (strpos(Line, 'kappa [cm2/g] at wave number WNL') gt 0) then begin
      print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, first wavelength block found', format='(A,F7.2,A)'
      break
    endif
  endwhile
  ;
  nPT=nPTarr[iFile]
  a=fltarr(nPT)
  ;
  ; --- Loop over all frequency points ---
  for inu=0L,(nnu-1L) do begin
    readf, Unit, Line
    ;
    readf, Unit, wnl, format='(7X,E20.12)'
    WaveNumber[inu]=wnl
    linu=alog(float(inu+1L))/alog(2.0)
    if (linu eq long(linu)) then begin
      print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, wavenumber: ', inu+1, wnl, format='(A,F7.2,A,I7,E15.6)'
    endif
    ;
    readf, Unit, a
    kappanu[iPTall:iPTall+nPT-1, inu]=a
  endfor ; inu
  ;
  iPTall=iPTall+nPT
  ;
  close, Unit
endfor ; iFile

; --- Compute additional axes ---
lambda=1.0/WaveNumber
nu=!con.clight/lambda

; --- Try to deduce size of temperature axis and reform and transpose arrays ---
ind=where(T eq T[0], nP)
if (nP gt 0) then begin
  nT=nPTall/nP
  print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, number of P/T points ', nPTall, nP, nT, format='(A,F7.2,A,I6,3X,2I4)'
  ;
  ; --- Rebin ---
  T=reform(rebin(reform(T, nP, nT),  1, nT))
  P=reform(rebin(reform(P, nP, nT), nP,  1))
  rho      =reform(rho, nP, nT)
  kappaRoss=reform(kappaRoss, nP, nT,      /OVERWRITE)
  kappaPlan=reform(kappaPlan, nP, nT,      /OVERWRITE)
  kappanu  =reform(kappanu  , nP, nT, nnu, /OVERWRITE)
  ;
  ; --- Transpose ---
  rho      =transpose(rho      )
  kappaRoss=transpose(kappaRoss)
  kappaPlan=transpose(kappaPlan)
  kappanu  =transpose(kappanu, [1, 0, 2])
endif ; (nP gt 0)

; --- Free I/O unit number ---
free_lun, Unit

print, '% coma_opac_rd: ', systime(/SECONDS)-time0, ' s, finished ----------------------------', format='(A,F7.2,A)'

end ; coma_opac_rd
