function mdspectrum_rd, filename, $
  channel=channel, ndataset=ndataset, $
  outstr=outstr, ierr=ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   mdspectrum_rd ('md_spectrum_read')
;
; PURPOSE:
;   Read uio file and put data into anonymous structure
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   data_structure=mdspectrum_rd(filename, channel=channel, ndataset=ndataset, $
;                                outstr=outstr, ierr=ierr)
;
; INPUT:
;   filename:   ('file_name') string, name of input file
;
; OUTPUT:
;
; KEYWORDS:
;   channel:    ('channel') integer, channel number (if file is already open
;                           and should stay so)
;   ndataset:   ('number_data_set') integer (default: 0): dataset to read
;   outstr:     ('output_string') output string
;   ierr:       ('integer_error') output integer
;                                 =0: correct operation
;                                 >0: an error occured while reading (e.g. the
;                                     specified dataset was not found)
;
; ROUTINES:
;   uio_struct_rd:
;
; INCLUDES:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   The file has to be in uio format and has to contain the proper
;   box/endbox and dataset/enddataset labels.
;
; PROCEDURE:
;   An uio file with the entries
;
;     fileform uio
;     headentry1
;     headentry2
;     headentry3
;     ...
;     label dataset
;       datasetentry1
;       datasetentry2
;       datasetentry3
;       label box
;         character box_id=box1
;         boxentry1
;         boxentry2
;         boxentry3
;         ...
;       label endbox
;       label box
;         character box_id=box2
;         boxentry1
;         boxentry2
;         boxentry3
;         ...
;       label endbox
;       ...
;     label enddataset
;
;   is read into a corresponding IDL structure
;
;     data.head.headentry1
;              .headentry2
;              .headentry3
;              ...
;         .datasetentry1
;         .datasetentry2
;         .datasetentry3
;         ...
;         .box1.boxentry1
;              .boxentry2
;              .boxentry3
;              ...
;         .box2.boxentry1
;              .boxentry2
;              .boxentry3
;              ...
;         ...
;
;   It is read only one dataset at a time. There might be more than one box.
;
; EXAMPLE:
;   You can read
;
;     the first dataset by omitting the 'ndataset' and 'channel' keywords,
;       e.g.
;         gt45g35n1_bin=mdspectrum_rd('gt45g35n1.bin1')
;
;     the dataset number 'nlabel' (starting with 0) by specifying 'ndataset' and
;       omitting the 'channel' keyword (slow for datasets in files with many
;       entries),
;       e.g.
;         zt142g80n1_cut=mdspectrum_rd('zt142g80n1.cut', ndataset=1)
;
;     read each dataset one by one by opening the file with uio_openrd and
;       specifying the 'channel' keyword,
;       e.g.
;         uio_openrd, 'zt142g80n1.cut', nc
;         zt142g80n1_cut0=mdspectrum_rd('zt142g80n1.cut', channel=nc)
;         zt142g80n1_cut1=mdspectrum_rd('zt142g80n1.cut', channel=nc)
;         ; ...
;         uio_closrd, nc
;
; MODIFICATION HISTORY:
;    6. 3.00 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;-
;------------------------------------------------------------------------------
;
data_structure=uio_struct_rd(filename, /head, $
                             startlabel='mdspectrum', endlabel='enddataset', $
                             nlabel=ndataset, channel=channel, $
                             outstr=outstr, ierr=ierr)

;
return, data_structure
end ; mdspectrum_rd
