pro phoenix_opac_rdall, filespec, logP, logT, wl, wt, kappa
;
;------------------------------------------------------------------------------
;+
; NAME:
;   phoenix_opac_rdall ('phoenix_opacity_read_all')
;
; PURPOSE:
;   Read list of PHOENIX raw opacity data files and sort them into 3D array
;
; CATEGORY:
;   I/O
;
; CALLING SEQUENCE:
;   phoenix_opac_rdall, filespec, logP, logT, wl, wt, kappa
;
; INPUT:
;   filespec: ('file_specifiations') string with path and filename(s). Wildcards are possible.
;
; OUTPUT:
;   logP:     ('logarithm_Pressure') 1D real vector with log_10(Pressure) values
;   logT:     ('logarithm_Temperature') 1D real vector with log_10(Temperature) values
;   wl:       ('wavelength') 1D double real vector with wavelength values
;   wt:       ('weight') 1D real vector with wavelength point weights
;   kappa:    ('kappa') 3D array with opacity data as function of pressure, temperature, and wavelength
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   phoenix_opac_filterwavelength: Go through all files and find entire wavelength axis
;
; SIDE EFFECTS:
;   Files are opened and closed.
;
; RESTRICTIONS:
;   Some data file properties are "hardwired" into the routine.
;
; PROCEDURE:
;   Temperature and pressure axes are defined.
;   The number of data entries per PHOENIX block is set.
;   All input files are (partially) read to get the total wavelength grid.
;   An (empty) 3D output opacity array is created.
;   All input files are read (again) one by one.
;   From each EOS block the indices for the following opacity data are derived.
;   Each opacity block is read and put into the proper position in the output array.
;
; EXAMPLE:
;   phoenix_opac_rdall, '/Users/bf/dat/opa/Rainout-2007.AGS.wlopactable.outputs/fort.73.000*', logP, logT, wl, wt, kappa
;
; MODIFICATION HISTORY:
;   2007-03-06 (HGL, Meudon) rd_sort_opac.pro
;   2007-09-20 (BF, Lyon) rd_sort_opac.pro -> phoenix_opac_rdall.pro
;                         output not into file but arrays
;-
;------------------------------------------------------------------------------

; --- Parameters to specify pressure and temperature axes ---
; Full version
;  lgTmin = 2.5
;  lgTmax = 4.0
;  lgTstep= 0.04
;  lgPmin = -4.0
;  lgPmax = 10.0
;  lgPstep= 0.2

;  lgTmin = 2.5
;  lgTmax = 4.0
;  lgTstep= 0.1
;  lgPmin = 0.0
;  lgPmax = 10.0
;  lgPstep= 0.5
  
;  ; --- Standard for 'Cond-2007.GN' & file_dir=file_id + '.wlopactable.outputs' (see *.5 file)
;  lgTmin = 2.5
;  lgTmax = 4.0
;  lgTstep= 0.05
;  lgPmin = -4.0
;  lgPmax = 10.0
;  lgPstep=  0.2

;  ; --- Standard for (see z-0.0a+0.0.BT-dusty-2011.agss.wlopactestg.5): g, l
;  lgTmin = 2.5
;  lgTmax = 4.4
;  lgTstep= 0.1
;  lgPmin = -2.7
;  lgPmax = 10.01
;  lgPstep=  0.1

;  ; --- Standard for (see z-0.0a+0.0.BT-dusty-2011.agss.wlopactable.5):
;  lgTmin = 2.5
;  lgTmax = 4.4
;  lgTstep= 0.02
;  lgPmin = -2.7
;  lgPmax = 10.01
;  lgPstep=  0.1

  ; --- Standard for (see z-0.0a+0.0.BT-dusty-2011.agss.wlopactablo.5):
  lgTmin = 2.5
  lgTmax = 4.4
  lgTstep= 0.02
  lgPmin = -9.0
  lgPmax = -2.69
  lgPstep=  0.1

  ; --- Number of data entries in one PHOENIX block ---
  n_OpacLayer=128

  ; --- Get initial time ---
  time0=systime(/SECONDS)

  ; --- Number of pressure and temperature points ---
  nP=long(nint((lgPmax-lgPmin)/lgPstep)+1)
  nT=long(nint((lgTmax-lgTmin)/lgTstep)+1)

  ; --- Define structure adequate to present one block of PHOENIX data ---
  opacity_wl_block = {opac_data, $
       wl: double(0.0), $
       xc:   fltarr(n_OpacLayer), sc: fltarr(n_OpacLayer), $ 
       xd:   fltarr(n_OpacLayer), sd: fltarr(n_OpacLayer), $ 
       xal:  fltarr(n_OpacLayer), xml:fltarr(n_OpacLayer), $ 
       xnlte:fltarr(n_OpacLayer)}

  ; --- Setup temperature, pressure, and wavelength grids ---
  logT = lgTmin+findgen(nT)*lgTstep
  logP = lgPmin+findgen(nP)*lgPstep
  phoenix_opac_filterwavelength, filespec, nwl, wl, wt
  wt = float(wt)
  print, '% phoenix_opac_rdall: number of detected wavelengths ', nwl
  ; --- Create 3D output array ---
  kappa=fltarr(nP,nT,nwl)

  ; --- Get unit number and expanded list of input files ---
  get_lun, unit
  files = findfile(filespec, count=nfiles)

  ; --- Loop over all input files ---
  for ifile=0,nfiles-1 do begin
    print, '% phoenix_opac_rdall: ', systime(/SECONDS)-time0, ' s, Handling ', files[ifile], format='(A,F7.2,A,A)'
    openr, unit, files[ifile], /swap_if_big_endian
    ;
    while not eof(unit) do begin
      ; --- Read next PHOENIX data block ---
      readu, unit, opacity_wl_block
      ;
      ; --- Check if the block contains thermodynamic or opacity data ---
      if (opacity_wl_block.wl lt 0.0) then begin
        ; === Block containing thermodynamic data is encountered ===
        ;
        ; --- Check if there are identical entries and find last index to be used, avoiding doubles ---
        if ((opacity_wl_block.xc[n_OpacLayer-2] eq opacity_wl_block.xc[n_OpacLayer-1]) and $
            (opacity_wl_block.sc[n_OpacLayer-2] eq opacity_wl_block.sc[n_OpacLayer-1])    ) then begin
          nd=(where( (opacity_wl_block.xc eq opacity_wl_block.xc[n_OpacLayer-1]) and $
                     (opacity_wl_block.sc eq opacity_wl_block.sc[n_OpacLayer-1])     ))[0]
        endif else begin
          nd=n_OpacLayer-1
        endelse
        ;print, '  Block with entries: ', nd+1
        ;
        ; --- Setup pressure and temperature indices ---
        indP = round((opacity_wl_block.sc[0:nd]-lgPmin)/lgPstep)
        indT = round((opacity_wl_block.xc[0:nd]-lgTmin)/lgTstep)
        ;
print, 'log P:', minmax(opacity_wl_block.sc), minmax(indP), nP, $
    '   log T:', minmax(opacity_wl_block.xc), minmax(indT), nT, nd+1, format='(A,2E12.4,3I4,A,2E14.4,4I4)'
        ;
;        ii = ip + it*np ; 1D index
;       Check whether 1D mapping forms a consecutive sequence
;        if max(ii-shift(ii,1)) gt 1 then begin
;          print, '% phoenix_opac_rdall: error: ii non consecutive'
;          stop
;        endif
        ;
        ; --- Extract density ---
        rho=10.0^opacity_wl_block.xd
        ;
      endif else begin
        ; === Opacity block encountered. It is assumed that the index arrays (indT, indP) are set  ===
        ;
        ; --- Find opacity index and expand it so that all three index arrays have the same length ---
        iw = (where(opacity_wl_block.wl eq wl))[0]
        indw=replicate(iw,nd+1)
        ; print, '    % phoenix_opac_rdall: iw', iw
        ;
        ; --- Compute total gas opacity: add all gas contributions and divide by rho ---
        ; --- Contributions due to continuum opacity, continuum scattering,          ---
        ; ---                      atomic lines, molecular lines, NLTE lines         ---
;        xgas =(opacity_wl_block.xc  + opacity_wl_block.sc + $
;               opacity_wl_block.xal + opacity_wl_block.xml + $
;               opacity_wl_block.xnlte)/rho
        xgas =opacity_wl_block.xnlte/rho
        kappa[indP,indT,indw]=xgas[0:nd]
        ;
        ; --- Dust opacity (optional) ---
        ; xdust= (opacity_wl_block.xd  + opacity_wl_block.sd)/rho
      endelse
      ;
    endwhile
    close, unit
  endfor

  print, 'Finished after ', systime(/SECONDS)-time0, ' seconds', format='(A,F7.2,A)'

end ; phoenix_opac_rdall
