pro RdArray, filename, a, s, $
             notransform=notransform, type=type0, $
             separator=separator, $
             skiplines=skiplines0, maxlines=maxlines0, stopatemptyline=stopatemptyline, $
             verbose=verbose, $
             filenumber=filenumber, filelist=filelist
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   RdArray ('Read_Array')
;
; PURPOSE:
;   Read array from ASCII file.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   RdArray, filename, a, s, $
;            notransform=notransform, type=type, $
;            separator=separator, $
;            skiplines=skiplines, maxlines=maxlines, stopatemptyline=stopatemptyline, $
;            verbose=verbose, $
;            filenumber=filenumber, filelist=filelist
;
; INPUT:
;   filename:     ('filename') character, name of file(s) with array,
;                 wildscards are possible
;
; OUTPUT:
;   a:            ('array') real, output array
;   s:            ('string') character, original array
;
; KEYWORDS:
;   notransform:  ('no_transform') if set the input is not transformed to float
;                 and the array a will be undefined.
;   separator:    ('separator') input character used to separate to entries,
;                    default: ' '
;   skiplines:    ('skip_lines') input, number of header lines to skip,
;                 empty (header) lines are skipped anyway,
;                   default: 0
;   type:         ('type') input integer for output type code (4: float, 5: double)
;   verbose:      ('verbose') input, if set the screen output is verbose
;   filelist:     ('filelist') output character array with expanded list of filenames
;   filenumber:   ('file_number') output integer with number of files read
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   str_sep:               Parse string into terms (from USERLIB)
;
; SIDE EFFECTS:
;   A channel unit number is fetched and returned, a file is opened and closed.
;
; RESTRICTIONS:
;   The routine is not exactly fast.
;   For each single transformation failure (string -> float) there is an
;   error message printed to screen.
;
; PROCEDURE:
;
; EXAMPLES:
;   Simplest call:
;     rdarray, 'tab.txt', a
;   Additional output of original string array:
;     rdarray, 'tab.txt', a, s
;   Fource double precision output:
;     rdarray, 'tab.txt', type=size(1.0D00,/type)
;     rdarray, 'tab.txt', type=5
;
; MODIFICATION HISTORY:
;   1996-02-16 Written by Bernd Freytag, Astrophysik Kiel
;   1996-05-06 (B.F.) strcompress -> strtrim
;   2001-04-02 (B.F. Uppsala) Keyword notransform added
;   2005-05-18 (B.F. LA) Keywords skiplines, type added; more comments
;   2007-10-11 (B.F. Lyon) Keyword maxlines added
;   2008-10-01 (B.F.) Keyword stopatemptyline added
;-
;---------------------------------------------------------------------------------------------------

; === Keyword handling ===
if (keyword_set(skiplines0)) then begin
  skiplines=skiplines0
endif else begin
  skiplines=0L
endelse
if (keyword_set(maxlines0)) then begin
  maxlines=maxlines0
endif else begin
  maxlines=2L^30
endelse
if (keyword_set(type0)) then begin
  type=type0
endif else begin
  type=4
endelse

; --- Check separator character ---
if (n_elements(separator) eq 0) then begin
  sep=' '
endif else if (separator eq '') then begin
  sep=' '
endif else begin
  sep=separator
endelse

; --- Expand file list, stop if no files found  ---
;filelist=findfile(filename, count=filenumber)
filelist=file_search(filename, count=filenumber)
if (filenumber eq 0) then message, 'No files ('+filename+') found.'

if (keyword_set(verbose)) then begin
  help, notransform
  help, sep
  help, skiplines, maxlines
  help, type
endif

; --- Initial array ---
line=''
n0max=32L
n1max=8L
s=strarr(n0max,n1max)
n0=0L
n1=0L

; --- Get free channel number ---
get_lun, nc

; === Loop over all files ===
for nfile=0,filenumber-1 do begin
  ; --- Open file from list ---
  if (keyword_set(verbose)) then print, 'Reading: '+filelist[nfile]
  openr, nc, filelist[nfile]
  ;
  ; --- Loop through lines of current file ---
  nline=0L
  continue_flag=1
  while ((not EOF(nc)) and (continue_flag)) do begin
    ; --- Read next line from file ---
    nline=nline+1L
    readf, nc, line
    line=strtrim(line,2)
    line=strcompress(line)
    ;
    continue_flag=(not ((keyword_set(stopatemptyline)) and (line eq ''))) or (nline le skiplines)
    ;
    if ((line ne '') and (nline gt skiplines) and (nline le maxlines)) then begin 
      ; --- Line is non-empty: process it ---
      n0=n0+1
      ;
      ; --- Paste line into separate terms ---
      linesep=str_sep(line,sep)
      ;
      if (n0 ge n0max) then begin
        ; --- Make array larger: more lines ---
        shelp=s
	s=strarr(2*n0max,n1max)
	s[0:n0-1,0:n1-1]=shelp[0:n0-1,0:n1-1]
        n0max=2*n0max
        shelp=''
      endif
      ;
      nsep=n_elements(linesep)
      if (nsep ge n1max) then begin
        ; --- Make array larger: more columns ---
        shelp=s
        n1max_new=(2*n1max) > nsep
	s=strarr(n0max,n1max_new)
        if (n1 gt 0) then s[0:n0-1,0:n1-1]=shelp[0:n0-1,0:n1-1]
        n1max=n1max_new
        shelp=''
      endif
      ;
      ; --- Put terms from new line into string array ---
      s[n0-1,0:nsep-1]=linesep
      n1=n1 > nsep
    endif
  endwhile
  ;
  ; --- Close file ---
  close, nc
endfor

; --- Return channel number ---
free_lun, nc

; --- Reduce string array to proper size ---
s=strcompress(s[0:n0-1,0:n1-1], /remove_all)

; --- Transform string array into real array ---
if (not keyword_set(notransform)) then begin
  if (type eq 5) then begin
    a=double(s)
  endif else begin
    a=float(s)
  endelse
endif

end ; RdArray
