pro RDATLAS, filename, r, head=head, jstop=jstop
;
;+
; NAME:
;   RDATLAS
;
; PURPOSE:
;   Load data array from a file produced by the ATLAS6 (or ATLAS9) program.
;
; CATEGORY:
;   Input/Output
;
; INPUT:
;   filename - Name of data file (including path)
;
; OUTPUT:
;   r - Array with output data, r(jstop,24)
;       r(*, 0) - RHOX
;       r(*, 1) - TEMP
;       r(*, 2) - PRESSURE
;       r(*, 3) - ELECTRON NUMBER
;       r(*, 4) - DENSITY
;       r(*, 5) - ROSSELAND MEAN
;       r(*, 6) - HEIGHT (KM), better: depth
;       r(*, 7) - ROSSELAND DEPTH
;       r(*, 8) - FRACTION CONV FLUX
;       r(*, 9) - RADIATIVE ACCELERATION
;       r(*,10) - PER CENT ERROR
;       r(*,11) - FLUX DERIV
;       r(*,12) - HEIGHT (KM), shifted
;       r(*,16) - GRDADB
;       r(*,22) - VCONV (CM/S)
;
; KEYWORDS:
;   head - Output: string array with some informations
;   jstop - 
;
; MODIFICATION HISTORY:
;   23. 9.93 Written by Bernd Freytag, Astrophysik Kiel
;    8.12.94 (B.F.)
;   10.12.94 (B.F.)
;   21. 1.95 (B.F.) ATLAS9
;-
;
print, 'Loading: ', filename
;
; --- Get free unit number ---
get_lun, cunit
; --- Open file ---
openr, cunit, filename
;
; --- Set maximum number of depth points ---
if (n_elements(jstop) eq 0) then jstop=40
; --- Create data array ---
r=fltarr(jstop,24)
; --- Create input string variable ---
line=' '
; --- Create string array for header data ---
head=strarr(3)
;
; --- Look through entire file ---
while (not EOF(cunit)) do begin
  ; --- Read line ---
  readf, cunit, line
  ;print, line
  if (strpos(line,'VCONV') ne -1) then begin
    print, 'strpos(VCONV): ', strpos(line,'VCONV')
    jind=0
    tmparr=fltarr(11)
    for j=0,jstop-1 do begin
      readf, cunit, jind, tmparr
;      print, jind, tmparr
      r(j,13:23)=tmparr
    endfor
  endif  
  ;
;  if (strpos(line,'TEFF') eq 1) then begin
  if (strpos(line,'TEFF') ne -1) then begin
    print, 'strpos(TEFF): ', strpos(line,'TEFF')
    if (strpos(line,'ITERATION') ge 0) then begin
      head(0)=line
      readf, cunit, line
      if (line eq '') then readf, cunit, line ; --- Skip line in ATLAS9 ---
      head(1)=line
      readf, cunit, line
      head(2)=line
      jind=0
      tmparr=fltarr(12)
      for j=0,jstop-1 do begin
        readf, cunit, jind, tmparr
;        print, jind, tmparr
        r(j,0:11)=tmparr
      endfor
    endif
  endif
endwhile
;
; --- Shift z axis to achieve: z(tau=1)=0 ---
ztau1=interpol(r(*,6),r(*,7),1.0)
r(*,12)=-r(*,6)+ztau1(0)
;
; --- Usual order of values ---
print, 'rotate r'
r=rotate(r,5)
;
; --- Close file ---
close, cunit
; --- Free unit ---
free_lun, cunit
;
end ; RDATLAS
