pro RDEVOLU, filename, r, rout, ratm, head=head, jstop=jstop, $
      MLTalpha=MLTalpha, totalMass=totalMass, Teff=Teff, $
      vm=vm, P=P, Temp=Temp, Radiusin=Radiusin, RADM=RADM, ADIM=ADIM, rho=rho, $
      alpha=alpha, delta=delta, cap=cap, beta=beta, $
      grav=grav, Hp=Hp, Gamma1=Gamma1, Csound=Csound, MLTvel=MLTvel, $
      Radiusatm=Radiusatm
;
;+
; NAME:
;   RDEVOLU
;
; PURPOSE:
;   Load data array from a file produced by the stellar evolution code from
;   Thomas bloecker et al.
;
; INPUTS:
;   filename - String: Name of input data file created by the stellar evolution
;              code
;
; OUTPUTS:
;   r - Real array: Table with quantities which describe the interior of a star
;
; KYEWORD PARAMETERS:
;   totalMass -
;   Teff -
;    ...
; CATEGORY:
;   Input/Output
;
; MODIFICATION HISTORY:
;   26. 1.94 Written (B.F.)
;    5. 6.00 (B.F. Copenhagen) New version of data file
;-
;
; --- Open file ---
print, 'Loading: ', filename
get_lun, cunit
openr, cunit, filename
;
; --- Some initializations ---
;jstop=314
jstop=1909
r=fltarr(jstop,38)
head=strarr(2)
jout=27
rout=fltarr(jout,27)
headout=strarr(2)
jatm=51
ratm=fltarr(jatm,11)
headatm=strarr(1)
;
line=' '
;
; --- Search headline of table ---
endflag=0
while (not EOF(cunit) and not endflag) do begin
  readf, cunit, line
  ;
  if (strpos(line,'MISCHUNGSWEGPARAMETER ALPHA =') eq 1) then begin
    ; --- Read MLT parameter alpha ---
    reads, line, MLTalpha, format='(30X,F8.4)'
    print, 'alpha_MLT=', MLTalpha
  endif
  ;
  if (strpos(line,'GMS=') eq 1) then begin
    ; --- Find mass and effective temperature ---
    reads, line, gms, Teff, format='(5X,F8.3,53X,F8.0)'
    totalMass=gms*1.989E+33   ; cgs units
    print, 'Mass=', gms, ' solar masses  = ', totalMass, ' g'
    print, 'Teff=', Teff, ' K'
  endif
  ;
  if (strpos(line,'UVLP    TAU') eq 4) then begin
    ; --- Headline found: atmosphere ---
    ;
    ; --- Save headlines ---
    headatm(0)=line
    ; --- Read table ---
    tmparr=fltarr(n_elements(ratm(0,*)))
    for j=0,jatm-1 do begin
      readf, cunit, format='(F8.3,F9.4,2F8.3,F10.3,F9.3,12X,5F10.5)', $
             tmparr
      print, tmparr
      ratm(j,0:*)=tmparr
    endfor
  endif
  ;
  if (strpos(line,'NR IHV') eq 2) then begin
    ; --- Headline found: outer solution ---
    ;
    ; --- Save headlines ---
    headout(0)=line
    readf, cunit, line
    headout(1)=line
    ; --- Read table ---
    idum1=0
    jind=0
    IHV=0
    tmparr=fltarr(n_elements(rout(0,*)))
    for j=0,jout-1 do begin
;      readin_string=''
;      while ((readin_string eq '') or $
;          (strpos(readin_string,'ITER') ne -1) or $
;          (strpos(readin_string,'ZONE') ne -1)) do begin
;        readf, cunit, format='(A255)', readin_string
;        print, readin_string
;      endwhile
      readf, cunit, $
         format='(I1,2I3,F13.5,F12.5,3F11.5,4F12.4,20X,/,' + $
                  'F8.4,F9.4,2F8.4,F9.4,F11.5,F9.4,F9.5,2E13.4,E14.3,E11.4,20X,/,' + $
                  'F11.5,5F10.5)', $
             idum1, jind, IHV, tmparr
      print, idum1, jind, IHV, tmparr
      rout(j,0:*)=tmparr
    endfor
  endif
  ;
  if (strpos(line,'J     VM') eq 2) then begin
    ; --- Headline found: inner solution ---
    ;
    ; --- Save headlines ---
    head(0)=line
    readf, cunit, line
    head(1)=line
    ; --- Skip 2 lines ---
    readf, cunit, line
    readf, cunit, line
    ; --- Read table ---
    jind=0
    tmparr=fltarr(n_elements(r(0,*)))
    for j=0,jstop-1 do begin
;      readf, cunit, format='(I5,F9.6,4F8.4,2F7.4,2E9.2,3E11.2,2F7.3,F6.3,/,'+ $
;                            '5X,F9.6,4F8.4,2F7.4,2E9.2,3E11.2,2F7.3,F6.3)', $
      readf, cunit, format='(I5,F9.6,4F8.4,2F7.4,2E9.2,3E11.2,2F7.3,F6.3,/,' + $
                            '5X,F9.6,4F8.4,2F7.4,2E9.2,3E11.2,2F7.3,F6.3,/,' + $
                            '10X,8F10.5)', $
             jind, tmparr
;      print, jind, tmparr
      r(j,0:*)=tmparr
    endfor
    endflag=1
  endif
endwhile
;
; --- Some constants ---
Gconst=6.6726E-08   ; cm^3 g^-1 s^-2
Clight=2.9979E+10   ; cm s^-1
;
; --- Select some important quantities from table ---
vm=r(*,0)
P=10.0^r(*,1)
Temp=10.0^r(*,2)
Radiusin=10.0^r(*,3)
RADM=r(*,12)
ADIM=r(*,27)
rho=10.0^r(*,13)
alpha=r(*,20)
delta=r(*,21)
cap=10.0^r(*,28)
beta=r(*,29)
;
; --- Compute some additional internal quantities ---
grav=Gconst*totalMass*vm/Radiusin^2             ; Local gravity [cm/s^2]
Hp=P                                          ; Pressure scale height [cm]
Hp(0:jstop-2)=P(0:jstop-2)/rho(0:jstop-2)/grav(0:jstop-2)
Hp(jstop-1)=Hp(jstop-2)
Gamma1=1.0/(alpha-delta*ADIM)                 ; 1. Adiabatic coefficient [1]
Csound=sqrt(Gamma1*P/rho)                     ; Sound velocity [cm/s]
;
MLTvel=( MLTalpha^2*Clight/cap/rho*grav*beta*(1.0-beta)* $
         ADIM*((RADM-ADIM)>0.0) )^(1.0/3.0)   ; MLT velocity [cm/s]
                                     ; Langer et al. 1985, A&A 145, 179, eq. (2)
;
; --- Compute some additional atmospheric quantities ---
gsurf=10.0^(-0.282)
Radiusatm=fltarr(jatm)
Radiusatm(jatm-1)=10.0^rout(0,2)
Patm   =10.0^ratm(*,0)
betaatm=     ratm(*,3)
rhoatm =10.0^ratm(*,4)
for i=jatm-2,0,-1 do begin
  Radiusatm(i)=Radiusatm(i+1)- (betaatm(i)*Patm(i)-betaatm(i+1)*Patm(i+1)) / $
                               (gsurf*sqrt(rhoatm(i+1)*rhoatm(i)))
endfor
;
; --- Close file ---
close, cunit
free_lun, cunit
;
end ; RDEVOLU
