pro RDF15LIST, listname, uu, vv, pp, tt, dd, ktimearr, time1arr, r, z, $
      ndatamax=ndatamax, $
      no_u=no_u, no_v=no_v, no_p=no_p, no_t=no_t, no_d=no_d, $
      append=append, modelname=modelname, $
      ncuu=ncuu, ncvv=ncvv, ncpp=ncpp, nctt=nctt, ncdd=ncdd, $
      Teff=Teff, glog=glog, $
      firsttime=firsttime, lasttime=lasttime, steptime=steptime, $
      epstime=epstime, reduce=reduce
;
;------------------------------------------------------------------------------
;+
; NAME:
;   RDF15LIST
;
; PURPOSE:
;   Reading a list of f15 data files and transforming into IDL conform 
;   data types
;
; CATEGORY:
;   Input
;
; INPUTS:
;   listname - string: Complete path for input files (wildcards allowed)
;
; OUTPUTS:
;   uu, vv, pp, tt, dd - 3D output arrays
;   ktimearr, time1arr - Time axes
;   r, z - Horizontal and vertical axis
;
; KEYWORDS:
;   ndatamax - Number of timesteps
;   no_u - If set: Array uu will not be loaded
;   no_v - If set: Array vv will not be loaded
;   no_t - If set: Array tt will not be loaded
;   no_p - If set: Array pp will not be loaded
;   no_d - If set: Array dd will not be loaded
;   ncXX - integer: logical channel number for assoc file for variable XX
;   Teff - Effective Temperature (output)
;   glog - log_10(gravity) (output)
;   firsttime - First time
;   lasttime - Last time
;   steptime - time step size
;   epstime - Allowed time deviation
;   reduce - If set the arrays are reduced from ndatamax points to proper size
;
; SUBROUTINES:
;   rdf15 - Read single data set from single F15 file
;
; EXAMPLE (1): 
;   ;  To read in the data from a list of f15 files into 3d arrays for the
;   ;  variables uu, vv, pp, tt:
;   RDF15LIST, 'bak/at75g44n3*.f15', uu,vv,pp,tt,dd,ktimearr,time1arr,r,z,/no_d
;   HELP
;
; EXAMPLE (2): 
;   ;  To read in the data from a list of f15 files into arrays associated
;   ;  with files for the variables uu and vv:
;   get_lun, ncuu
;   get_lun, ncvv
;   openw, ncuu, 'at75g44n3_u_a.dat'
;   openw, ncvv, 'at75g44n3_v_a.dat'
;   RDF15LIST, 'bak/at75g44n3*.f15', uu,vv,pp,tt,dd,ktimearr,time1arr,r,z,$
;              ncuu=ncuu, ncvv=ncvv, /no_p, /no_t, /no_d
;   HELP
;
; MODIFICATION HISTORY:
;   Written: 15.5.93 by Bernd Freytag, Astrophysik Kiel
;            20.7.93 (B.F.) Keyword ndatamax added
;            10.9.93 (B.F.) Keywords no_X added
;            9.10.93 (B.F.) Keywords ncXX added
;            24.1.94 (B.F.) 'aaa.Z' files are decompressed and again compressed
;            4.8.94 (B.F.) Keywords TEFF and GLOG added
;            7.9.94 (B.F.) Keywords ...TIME added
;            7.4.95 (B.F.) Keyword reduce added
;            21.5.95 (B.F.) Fast return if data arrays full
;------------------------------------------------------------------------------
;
; --- channel number of f15 data file ---
get_lun, ncdata
; --- channel number of file name list ---
get_lun, ncfinali
;
filename='dummyfile'
compress_flag=0
;
ndataset=0
;
ifend=0
ibound=0
ibplt=0
mgeom=0
mside=0
ktime=1L
time1=0.0
teff=0.0
glog=0.0
htau0=0.0
hdate='char'
htime='char'
htext='char'
;
no_u=keyword_set(no_u) or keyword_set(ncuu)
no_v=keyword_set(no_v) or keyword_set(ncvv)
no_t=keyword_set(no_t) or keyword_set(nctt)
no_p=keyword_set(no_p) or keyword_set(ncpp)
no_d=keyword_set(no_d) or keyword_set(ncdd)
;
; --- Choose default values for time window and grid ---
if (not keyword_set(firsttime)) then firsttime=-1.0E+30
if (not keyword_set(lasttime)) then lasttime=1.0E+30
if (not keyword_set(steptime)) then steptime=0.0
if (not keyword_set(epstime)) then epstime=steptime*0.08
help, firsttime, lasttime, steptime, epstime
;
;icod15=0
;
if (listname ne 'rdf15listpar.txt') then begin
  fifi=findfile(listname)
  openw, ncfinali, 'rdf15listpar.txt'
  lastindex,fifi,nlistlast
  for nlist=0,nlistlast-1 do printf, ncfinali, fifi(nlist)
  close, ncfinali
endif
;
openr, ncfinali, 'rdf15listpar.txt'
nfile=0
if (not keyword_set(ndatamax)) then ndatamax=401>ndataset
;
repeat begin
  if (EOF(ncfinali)) then begin
    filename=''
  endif else begin
    readf, ncfinali, filename
    filename=strcompress(filename)
  endelse
  if (filename ne '') then begin
    nfile=nfile+1
    print, nfile, ': ',filename
    ;
    suffix=strmid(filename, (strlen(filename)-2)>0, 2)
    if (suffix eq '.Z') then begin 
      spawn, 'uncompress '+filename
      tmpfilename=strmid(filename, 0, (strlen(filename)-2)>0)
    endif else begin
      tmpfilename=filename
    endelse
    ;
    openr, ncdata, tmpfilename
    ; --- Read header ---
    nn=0
    rdf15, nn, ifend, istop, jstop, ibound, ibplt, $
           mgeom, mside, ktime, time1, teff, glog, htau0, $
           r, z, tox, pox, u, v, p, t, d, $
           hdate, htime, htext, ncdata 
    ;
    print, 'Date: ', hdate, ' Time: ', htime
    if (nfile eq 1) then begin
      ; --- Create arrays if first file opened ---
      if (keyword_set(ncuu)) then uu=assoc(ncuu,fltarr(istop,jstop))
      if (keyword_set(ncvv)) then vv=assoc(ncvv,fltarr(istop,jstop))
      if (keyword_set(ncpp)) then pp=assoc(ncpp,fltarr(istop,jstop))
      if (keyword_set(nctt)) then tt=assoc(nctt,fltarr(istop,jstop))
      if (keyword_set(ncdd)) then dd=assoc(ncdd,fltarr(istop,jstop))
      if (not keyword_set(append)) then begin
        if (not no_u) then uu=fltarr(istop,jstop,ndatamax)
        if (not no_v) then vv=fltarr(istop,jstop,ndatamax)
        if (not no_t) then tt=fltarr(istop,jstop,ndatamax)
        if (not no_p) then pp=fltarr(istop,jstop,ndatamax)
        if (not no_d) then dd=fltarr(istop,jstop,ndatamax)
        ktimearr=lonarr(ndatamax)
        time1arr=fltarr(ndatamax)
      endif else begin
        if (not no_u) then begin
           restore, /verbose, modelname+'_uktrz.dat'
           lastindex, uu, istop, jstop, ntime
           uutmp=temporary(uu)
           uu=fltarr(istop,jstop,ndatamax)
           uu(*,*,0:ntime-1)=temporary(uutmp)
        endif
        if (not no_v) then begin
           restore, /verbose, modelname+'_vktrz.dat'
           lastindex, vv, istop, jstop, ntime
           vvtmp=temporary(vv)
           vv=fltarr(istop,jstop,ndatamax)
           vv(*,*,0:ntime-1)=temporary(vvtmp)
        endif
        if (not no_t) then begin
           restore, /verbose, modelname+'_tktrz.dat'
           lastindex, tt, istop, jstop, ntime
           tttmp=temporary(tt)
           tt=fltarr(istop,jstop,ndatamax)
           tt(*,*,0:ntime-1)=temporary(tttmp)
        endif
        if (not no_p) then begin
           restore, /verbose, modelname+'_pktrz.dat'
           lastindex, pp, istop, jstop, ntime
           pptmp=temporary(pp)
           pp=fltarr(istop,jstop,ndatamax)
           pp(*,*,0:ntime-1)=temporary(pptmp)
        endif
        if (not no_d) then begin
           restore, /verbose, modelname+'_dktrz.dat'
           lastindex, dd, istop, jstop, ntime
           ddtmp=temporary(dd)
           dd=fltarr(istop,jstop,ndatamax)
           dd(*,*,0:ntime-1)=temporary(ddtmp)
        endif
        ktimearrtmp=lonarr(ndatamax)
        ktimearrtmp(0:ntime-1)=temporary(ktimearr)
        ktimearr=temporary(ktimearrtmp)
        lastindex, time1arr, ndataset
        time1arrtmp=fltarr(ndatamax)
        time1arrtmp(0:ntime-1)=temporary(time1arr)
        time1arr=temporary(time1arrtmp)
      endelse
    endif
    ;
    nn=1
    repeat begin
      ; --- Read data blocks ---
      if (ndataset lt ndatamax) then begin
        rdf15, nn, ifend, istop, jstop, ibound, ibplt, $
               mgeom, mside, ktime, time1, teff, glog, htau0, $
               r, z, tox, pox, u, v, p, t, d, $
               hdate, htime, htext, ncdata
      endif else begin
        ifend=1
      endelse
      ;
      ; --- Test if data block should be skipped ---
      ;
      ifskip=0
      ; --- Only one timestep further? ---
      if (ndataset gt 0) then begin
        if (ktime eq ktimearr(ndataset-1)+1 ) then ifskip=1
      endif
      ; --- Time slice before selected window? ---
      if (time1 lt firsttime-epstime) then ifskip=1
      ; --- Time slice after selected window? ---
      if (time1 gt lasttime+epstime) then ifskip=1
      ; --- Time slice fits into selected time step scheme? ---
      if (steptime gt 0) then $
        if (abs((time1+epstime) mod steptime) gt 2.0*epstime) then ifskip=1
      ;            
      ; --- Store new data block ---
      if ((ifskip eq 0) and (ifend eq 0) and (ndataset lt ndatamax)) then begin
	print, 'KTIME= ', KTIME, ' time1= ',time1
        if (not no_u) then uu(*,*,ndataset)=u $
          else if keyword_set(ncuu) then uu(ndataset)=u
        if (not no_v) then vv(*,*,ndataset)=v $
          else if keyword_set(ncvv) then vv(ndataset)=v
        if (not no_t) then tt(*,*,ndataset)=t $
          else if keyword_set(nctt) then tt(ndataset)=t
        if (not no_p) then pp(*,*,ndataset)=p $
          else if keyword_set(ncpp) then pp(ndataset)=p
        if (not no_d) then dd(*,*,ndataset)=d $
          else if keyword_set(ncdd) then dd(ndataset)=d
        ;
	ktimearr(ndataset)=ktime
	time1arr(ndataset)=time1
        ndataset=ndataset+1
      endif
    endrep until (ifend eq 1)
    close, ncdata                        ; Close f15 file
    if (suffix eq '.Z') then spawn, 'compress '+tmpfilename
  endif
endrep until ((filename eq '') or (ndataset ge ndatamax))
;
; --- Close files ---
close, ncfinali
free_lun, ncfinali
free_lun, ncdata
;
; --- Reduce arrays to proper size ---
if (keyword_set(reduce)) then begin
  if (not no_u) then uu=uu(*,*,0:ndataset-1) 
  if (not no_v) then vv=vv(*,*,0:ndataset-1) 
  if (not no_P) then PP=PP(*,*,0:ndataset-1) 
  if (not no_T) then TT=TT(*,*,0:ndataset-1) 
  if (not no_D) then DD=DD(*,*,0:ndataset-1)
  ktimearr=ktimearr(0:ndataset-1) 
  time1arr=time1arr(0:ndataset-1) 
endif
;
end ; RDF15LIST
