pro rdf20_para, linearr, str, found, value
;------------------------------------------------------------------------------
;+
; NAME:
;   RDF20_PARA
;
; PURPOSE:
;   Read one parameter out of string with F20-parameter-file data.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   rdf20_para, str, found, value
;
; INPUTS:
;   linearr - String with F20-parameter-file data
;   str - Search string
;
; OUTPUTS:
;   found - Flag: 1 <--> found
;   value - Value corresponding to parameter specified by string
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    3.11.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
pos1=strpos(linearr,str)
len=strlen(str)
if (pos1 ge 0) then begin
  found=1
  pos2=strpos(strmid(linearr,pos1+len,32000),'|')
  valstr=strmid(linearr,pos1+len,pos2)
;  print, str, valstr
  reads, valstr, value
endif else begin
  found=0
  value=0.0
endelse
;
end ; rdf20_para


pro rdf20, filename, istop=istop, jstop=jstop, $
  ibound=ibound, ibotm=ibotm, igas=igas, iparsp=iparsp, $
  ifco=ifco, $
  lmax=lmax, mxbcor=mxbcor, iflcor=iflcor, mside=mside, $
  dr=dr, grav=grav, logg=logg, Teff=Teff, $
  Fimplc=Fimplc, Srgam=Srgam, $
  Hpbot=Hpbot, Hptop=Hptop, Htau0=Htau0, Tsurf=Tsurf, tservo=tservo, $
  abuX=abuX, abuY=abuY, Qmol=Qmol
;
;------------------------------------------------------------------------------
;+
; NAME:
;   RDF20
;
; PURPOSE:
;   Read F20-parameter-file.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   rdf20, filename [, istop=istop, jstop=jstop, Teff=Teff, ...]
;
; INPUTS:
;   filename - Name of F20-parameter-file
;
; OUTPUTS:
;   None (Only keywords)
;
; KEYWORDS:
;   istop - Number of horizontal grid points
;   kstop - Number of vertical grid points
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   rdf20_para: Read one parameter value
;
; SIDE EFFECTS:
;   A file is opened and closed.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   Read input parameters from F06 file:
;     rdf20, 'bak/gt55g44n1aa.f06', istop=istop, jstop=jstop, Teff=Teff
;
; MODIFICATION HISTORY:
;    3.11.95 Written by Bernd Freytag, Astrophysik Kiel
;   15. 5.96 (B.F.) Parameters added
;    9. 9.96 (B.F.) FIMPLC not found -> FIMPLC=0.5
;    9. 5.97 (B.F.) MSIDE read
;-
;------------------------------------------------------------------------------
;
; --- Get number of input channel number ---
get_lun, cunit
;
; --- Does file exist? ---
f=findfile(filename)
if (f(0) eq '') then message, filename + ' is unknown.'
print, 'Loading: ', filename
;
line=''
;linearr=strarr(100)
linearr=''
;
; --- Open file ---
openr, cunit, filename
;
; --- Search part with parameters ---
foundflag=0
while ((not EOF(cunit)) and (not foundflag)) do begin
  readf, cunit, line
  foundflag=(strmid(line,1,9) eq 'ISTOP___=')
endwhile
;
; --- Read parameter file into long string (string array) ---
n=0
;linearr(n)=line
linearr=linearr+line
foundflag=0
while ((not EOF(cunit)) and (not foundflag)) do begin
  n=n+1
  readf, cunit, line
;  linearr(n)=line
  linearr=linearr+line
  foundflag=(strmid(line,1,9) eq 'ABUX____=')
endwhile
;linearr=linearr(0:n)
;
; --- Close channel ---
close, cunit
free_lun, cunit
;
; --- Read selected parameters from parameter string ---
rdf20_para, linearr, 'ISTOP___=', found, istop
rdf20_para, linearr, '_JSTOP__=', found, jstop
;
rdf20_para, linearr, 'IBOUND__=', found, ibound
rdf20_para, linearr, '_IBOTM__=', found, ibotm
rdf20_para, linearr, '_IGAS___=', found, igas
rdf20_para, linearr, '_IPARSP_=', found, iparsp
;
rdf20_para, linearr, 'IFCO____=', found, ifco
;
rdf20_para, linearr, 'LMAX____=', found, lmax
rdf20_para, linearr, '_MXBCOR_=', found, mxbcor
rdf20_para, linearr, '_IFLCOR_=', found, iflcor
rdf20_para, linearr, '_MSIDE__=', found, mside
;
rdf20_para, linearr, 'DR______=', found, dr
rdf20_para, linearr, 'GRAV____=', found, grav  & logg=alog10(grav)
rdf20_para, linearr, '_TEFF___=', found, Teff
;
rdf20_para, linearr, '_FIMPLC_=', found, Fimplc
if (not found) then Fimplc=0.5
rdf20_para, linearr, '_SRGAM__=', found, Srgam
;
rdf20_para, linearr, 'HPBOT___=', found, Hpbot
rdf20_para, linearr, 'HPTOP___=', found, Hptop
rdf20_para, linearr, '_HTAU0__=', found, Htau0
rdf20_para, linearr, '_TSURF__=', found, Tsurf
rdf20_para, linearr, '_TSERVO_=', found, tservo
;
rdf20_para, linearr, 'ABUX____=', found, abuX
rdf20_para, linearr, '_ABUY___=', found, abuY
rdf20_para, linearr, '_QMOL___=', found, Qmol
;
end ; rdf20
