pro rdgmodels, filename, tab, modellist, names=names, units=units
;
;------------------------------------------------------------------------------
;+
; NAME:
;   RDGMODELS (ReaD_Gstar_MODELS)
;
; PURPOSE:
;   Load table with final results of HD simulations (Sun, G stars, and others)
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   rdgmodels, filename, tab, names=names, units=units
;
; INPUTS:
;   filename:  Name of file with data
;
; OUTPUTS:
;   tab:       2D array with data table
;   modellist: Model name array with ntabmax entries
;
; KEYWORDS:
;   names:     Name array with ncolmax entries
;   units:     Unit array with ncolmax entries
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   A file is opened and closed
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   rdgmodels, 'g_models.txt', tab
;
; MODIFICATION HISTORY:
;    3.10.95 Written by Bernd Freytag, Astrophysik Kiel
;   15.11.95 (B.F.) New format of file 'g_models.txt' -> some changes
;   15.03.96 (B.F.) New format of file 'g_models.txt' -> some changes
;-
;------------------------------------------------------------------------------
;
; --- Open file ---
if (n_elements(filename) eq 0) then filename='g_models.txt'
if (filename eq '' or filename eq ' ') then filename='g_models.txt'
print, 'Loading: ', filename
get_lun, cunit 
openr, cunit, filename 
;
; --- Maximum size of table ---
ntabmax=300 ; --- Maximum number of models (lines per sub-table)
ncolmax=1000 ; --- Maximum number of total columns
tab=fltarr(ntabmax,ncolmax) 
modellist=strarr(ntabmax)
names=strarr(ncolmax)
units=strarr(ncolmax)
;
sep=' '
;
; === Read first block ===
; --- Read head ---
head='' 
firstcol=0
ncol=0
readf, cunit, ncol
print, ncol
readf, cunit, head
print, head 
headsep=str_sep(strcompress(strtrim(head,2)),sep)
print,'('+headsep+')'
nheadsep=n_elements(headsep)
if (nheadsep gt 0) then names(firstcol:firstcol+nheadsep-1)=headsep
readf, cunit, head 
print, head
headsep=str_sep(strcompress(strtrim(head,2)),sep)
print,'('+headsep+')'
nheadsep=n_elements(headsep)
if (nheadsep gt 0) then units(firstcol:firstcol+nheadsep-1)=headsep
;
; --- Read data until EOF or blank ---
ncoltemp0=ncol 
tabtemp=fltarr(ncoltemp0 > 1) 
modeltemp=''
line=''
ntab=0
endflag=EOF(cunit)
if (not endflag) then begin
  readf, cunit, line
  endflag=strcompress(line, /rem) eq ''
endif
while (not endflag) do begin 
  if (ncoltemp0 eq 0) then begin
;    reads, line, modeltemp
    modeltemp=strcompress(line, /rem)
  endif else begin
;    reads, line, tabtemp, modeltemp
;    tab(ntab,0:ncoltemp0-1)=tabtemp
    linetemp=str_sep(strtrim(strcompress(line),2), sep)
    tab(ntab,0:ncoltemp0-1)=float(linetemp(0:ncoltemp0-1))
    modeltemp=strcompress(linetemp(ncoltemp0), /rem)
  endelse
  modellist(ntab)=strcompress(modeltemp, /rem)
  ntab=ntab+1
  endflag=EOF(cunit)
  if (not endflag) then begin
    readf, cunit, line
    endflag=strcompress(line, /rem) eq ''
  endif
endwhile 
;
firstcol=firstcol+ncol
;
; --- Reduce array to proper size (number of models) ---
tab=tab(0:ntab-1,*) 
modellist=modellist(0:ntab-1) 
;
; === Read second block and following ===
; --- Read head ---
readf, cunit, ncol
print, firstcol, ncol
while (ncol gt 0) do begin
  readf, cunit, head
  print, head 
  headsep=str_sep(strcompress(strtrim(head,2)),sep)
  nheadsep=n_elements(headsep)
  if (nheadsep gt 0) then names(firstcol:firstcol+nheadsep-1)=headsep
  readf, cunit, head 
  print, head
  headsep=str_sep(strcompress(strtrim(head,2)),sep)
  nheadsep=n_elements(headsep)
  if (nheadsep gt 0) then units(firstcol:firstcol+nheadsep-1)=headsep
  ; --- Read data until EOF or blank ---
  ncoltemp1=ncol 
  tabtemp=fltarr(ncoltemp1) 
  endflag=EOF(cunit)
  if (not endflag) then begin
    readf, cunit, line
    endflag=strcompress(line, /rem) eq ''
  endif
  while (not endflag) do begin 
;    reads, line, tabtemp, modeltemp
;    modeltemp=strcompress(modeltemp, /rem)
    linetemp=str_sep(strcompress(strtrim(line,2)), ' ')
    modeltemp=strcompress(linetemp(ncoltemp1), /rem)
    ntab=where(modellist eq modeltemp)
    ntab=ntab(0)
    if (ntab ge 0) then begin
;      tab(ntab,firstcol:firstcol+ncoltemp1-1)=tabtemp 
      tab(ntab,firstcol:firstcol+ncoltemp1-1)=float(linetemp(0:ncoltemp1-1))
    endif
    endflag=EOF(cunit)
    if (not endflag) then begin
      readf, cunit, line
      endflag=strcompress(line, /rem) eq ''
    endif
  endwhile 
  firstcol=firstcol+ncol
  readf, cunit, ncol
  print, firstcol, ncol
endwhile
;
; --- Close file ---
close, cunit 
free_lun, cunit 
;
; --- Reduce array to proper size (number of columns) ---
tab=tab(*,0:firstcol-1) 
names=names(0:firstcol-1)
units=units(0:firstcol-1)
;
end ; rdgmodels
