pro rdisosal, disk, metid, alphalist, agelist, $
    Teff=Teffarr, Lum=Lumarr, Mass=Massarr, $
    Radius=Radiusarr, grav=gravarr, alpha=alphaarr, suffix=suffix
;------------------------------------------------------------------------------
;+
; NAME:
;   rdisosal ('read_isochrone_salaris')
;
; PURPOSE:
;   Read evolutionary tracks
;
; CATEGORY:
;   Stellar Evolution, I/O
;
; CALLING SEQUENCE:
;   rdisosal, disk, metid, alphalist, agelist, $
;     Teff=Teffarr, Lum=Lumarr, Mass=Massarr, $
;     Radiusarr=Radiusarr, gravarr=gravarr, suffix=suffix
;
; INPUTS:
;   disk:     ('disk') directory where files are located
;   metid:    ('metallicity_identifier') string to identify isochrone file set
;             with proper metallicity:
;               met='mm10': metid='183'
;               met='mm20': metid='184'
;   alphalist:('alpha_list') real 1D vector with alpha values,
;             e.g. alphalist=[1.2, 1.5, 1.8, 2.1]
;   agelist:  ('age_list') real 1D vector with age values [Gyr],
;             e.g. agelist  =[9.0, 10.0, 11.0, 12.0, 13.0, 14.0]
;   suffix:   ('suffix') string, suffix for file names (default: 'norma')
;
; OUTPUTS:
;   Teff:     ('temperature_effective')
;   Lum:      ('luminosity')
;   Mass:     ('mass')
;   Radius:   ('radius')
;   grav:     ('gravity') surface gravity
;   alpha:    ('alpha') mixing-length parameter
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rdarray: Read array
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   22. 5.98 Written by Bernd Freytag, Astrophysik Kiel
;   27. 8.98 (B.F. Copenhagen) agearr, alphaarr -> agelist, alphalist,
;            Keywords alpha, suffix added
;-
;------------------------------------------------------------------------------
;
nalphas=n_elements(alphalist)
nages  =n_elements(agelist  )
nmasses=0
;
if (not keyword_set(metid)) then metid=''
if (not keyword_set(suffix)) then suffix='norma
;
for i=0,nalphas-1 do begin
  alpha_str=strtrim(string(alphalist(i)*10.0, format='(I2)'),2)
  if (strlen(alpha_str) eq 1) then alpha_str='0' + alpha_str
  for j=0,nages-1 do begin
    age_str=strtrim(string(agelist(j), format='(I2)'),2)
    if (strlen(age_str) eq 1) then age_str='0' + age_str
    file='I' + age_str + metid + 'a' + alpha_str + suffix
    print, 'Read file ''' + file + ''''
    rdarray, disk + file, tab
    if (nmasses eq 0) then begin
      nmasses =n_elements(tab(*,0))
      ncolumns=n_elements(tab(0,*))
      Lumarr  =fltarr(nmasses, nages, nalphas)
      Teffarr =fltarr(nmasses, nages, nalphas)
      Massarr =fltarr(nmasses, nages, nalphas)
      alphaarr=fltarr(nmasses, nages, nalphas)
    endif
    Lumarr( *,j,i)=10.0^tab(*,0)
    Teffarr(*,j,i)=10.0^tab(*,1)
    Massarr(*,j,i)=     tab(*,2)
    if (ncolumns gt 3) then begin
      alphaarr(*,j,i)=tab(*,3)
    endif else begin
      alphaarr(*,j,i)=alphalist(i)
    endelse
  endfor ; j
endfor ; i
;
Radiusarr=Lumarr
gravarr  =Lumarr
;
; --- Compute radius and surface gravity ---  
Radiusarr=sqrt(Lumarr*!CON.LSUN / (!CON.SIGMASB*Teffarr^4 * 4.0*!PI))
gravarr  =!CON.G*Massarr*!CON.MSUN/Radiusarr^2
;
end ; rdisosal
