pro RDKOESTER, filename, r, head=head, jstop=jstop
;
;+
; NAME:
;   RDKOESTER
;
; PURPOSE:
;   Load data array from a file produced by the Koester WD atmosphere program.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   RDKOESTER, filename, r [, head=head, jstop=jstop]
;
; INPUTS:
;   filename - Name of the input file
;
; OUTPUTS:
;   r - Output array: r(jstop,18)
;       r(*,0): taus
;       r(*,1): T [K]
;       r(*,2): dT
;       r(*,3): dFlux
;       r(*,4): KJB
;       r(*,5): lg Pg
;       r(*,6): lg rho
;       r(*,7): lg Pe
;       r(*,8): lg kappa
;       r(*,9): T [K]
;      r(*,10): aux1
;      r(*,11): aux2
;      r(*,12): grad
;      r(*,13): gact
;      r(*,14): gl
;      r(*,15): gad
;      r(*,16): Fc/Ftot [sigma Teff^4]
;      r(*,17): Vconv [km/s]
;      r(*,18): z [cm]
;
;
;      r(*,10): hs(km)
;      r(*,11): ad
;      r(*,12): gact
;      r(*,13): dFcm/F
;      r(*,14): dFc/F
;      r(*,15): Fc/F [sigma Teff^4]
;      r(*,16): Vconv [km/s]
;      r(*,17): z [cm]
;
; KEYWORDS:
;   head - (Output) 3 headlines
;   jstop - (Output) Number of depth points
;
; EXAMPLE:
;   RDKOESTER, '../atm/t130g80a20n2.koe', r20, head=r20head
;
; MODIFICATION HISTORY:
;    4.11.93 Written by Bernd Freytag, Astrophysik Kiel
;   12. 8.94 z axis from data file
;    8.12.94 (B.F.) Modification in data format
;-
;
print, 'Loading: ', filename
;
; --- Initializiation ---
jstop=0
jmax=200
r=fltarr(jmax,19)
line=' '
head=strarr(3)
Teff=0.0
logg=0.0
;
; --- Open file on free channel
get_lun, cunit
openr, cunit, filename
;
while ((not EOF(cunit)) and (Teff eq 0.0)) do begin
  readf, cunit, line
  if (strpos(line,'Teff =') gt 0) then $
    if (strpos(line,'log g =') gt 0) then begin
      Teffpos=strpos(line,'Teff =')+6
      reads, strmid(line,Teffpos,7), Teff
      loggpos=strpos(line,'log g =')+7
      reads, strmid(line,loggpos,7), logg
      g=10.0^logg
      print, 'Teff=',Teff, '  logg=',logg
    endif
endwhile
;
nz=18
dumstring='   '
; --- Search through file ---
while (not EOF(cunit)) do begin
  readf, cunit, line
  if (strpos(line,'taus       T') gt 0) then begin
    ; --- Headline of first table found ---
    print, 'strpos: ', strpos(line,'taus       T')
    head(0)=line
    readf, cunit, line
    ;
    ; --- Read first table ---
    jind=0
    tmparr=fltarr(9)
    readf, cunit, line
    while (strtrim(line) ne '') do begin
      reads, line, jind, tmparr, dumstring, ztmp, $
        format='(I4,E12.4,F7.0,F6.0,F7.3,F7.0,F7.3,F8.3,F7.3,F10.4,A3,F10.3)'
      ;reads, line, jind, tmparr
      ;print, jind, tmparr
      r(jind-1,0:8)=tmparr
      r(jind-1,nz)=ztmp
      readf, cunit, line
    endwhile
    jstop=jind
    ;
    ; --- Skip lines, read headlines of next table ---
    readf, cunit, line
    readf, cunit, line
    head(1)=line
    readf, cunit, line
    readf, cunit, line
    head(2)=line
;    readf, cunit, line
    ;
    ; --- Read next table ---
    tmparr=fltarr(9)
    for j=0,jstop-1 do begin
      readf, cunit, jind, tmparr
;      print, jind, tmparr
      r(jind-1,9:17)=tmparr
    endfor
    ;
  endif
endwhile
print,head(1)
;
; --- Reduce array size to actual number of data values ---
r=r(0:jstop-1,*)
;
; --- Create z axis ---
;r(0,nz)=0.0
;for j=1,jstop-1 do $
;  r(j,nz)=r(j-1,nz)-1.0/ (g* (10.0^(0.5*(r(j-1,6)+r(j,6)))) )* $
;                         (10.0^r(j,5)-10.0^r(j-1,5))
; --- Shift z axis ---
;ztau1=interpol(r(*,nz),r(*,0),1.0)
;r(*,nz)=r(*,nz)-ztau1(0)
;print,ztau1(0)
;
close, cunit
free_lun, cunit
;
end ; RDKOESTER
