pro rdphoenix, filename, r, head=head, Teff=Teff, grav=grav
;------------------------------------------------------------------------------
;+
; NAME:
;   rdphoenix ('read_phoenix')
;
; PURPOSE:
;   Load data array from atmosphere file produced by the PHOENIX program.
;
; CATEGORY:
;   Input/Output
;
; INPUT:
;   filename - Name of data file (including path)
;
; OUTPUT:
;   r - Array with output data, r(17,jstop)
;       r( 0,*) - Level
;       r( 1,*) - tau_standard
;       r( 2,*) - Temperature [K]
;       r( 3,*) - Gas       pressure [dyn/cm^2]
;       r( 4,*) - Electron  pressure [dyn/cm^2]
;       r( 5,*) - Density [g/cm^3]
;       r( 6,*) - Molecular weight mu
;       r( 7,*) - Radius [cm]
;       r( 8,*) - Extension (depth) [cm]
;       r( 9,*) - kapq
;       r(10,*) - kappa_Ross [cm^2/g]

;       r(11,*) - RHOX [cm^-2]
;       r(12,*) - tau_Rosseland
;       r(13,*) - Height [cm]
;
; KEYWORDS:
;   head:   ('header') Output: string array with some information
;   Teff:   ('Temperature_effective') float with effective temperature
;   grav:   ('gravity') float with gravity
;
; ROUTINES:
;   rdarray: Read file and put contents into array.
;
; MODIFICATION HISTORY:
;   2007-10-11 (B.F. Lyon) Adapted from rdmarcs
;-
;------------------------------------------------------------------------------
;
print, 'Loading: ', filename
;
; === Read atmosphere ===
;
; --- Get Teff, logg ---
rdarray, filename, a, s, maxlines=1, /notransform
Teff=float(s[0,1])
logg=float(s[0,3])
grav=10.0^logg
help, Teff, logg
;
; --- Get header line ---
rdarray, filename, a, s, skiplines=1, maxlines=2, /notransform
head=strjoin(reform(s), ' ') + ' rhox tau_Rosseland height'
print, head
;
; --- Get bulk data ---
rdarray, filename, a, skiplines=2
; --- Chop of topmost layer ---
a=a[1:*,*]
a=transpose(a)
jstop=n_elements(a[0,*])
ncol =n_elements(a[*,0])
;
; --- Create data array ---
r=fltarr(ncol+3,jstop)
r[0:ncol-1,*]=a
;
; --- Some indices ---
;i_T      = 2
i_P      = 3
i_rho    = 5
i_depth  = 8
i_kapRoss=10
i_rhox   =11
i_tauRoss=12
i_height =13
;
; === Compute additional data ===
;
; --- Mass density per area ---
r[i_rhox,0]=r[i_P,0]/grav
for j=1,jstop-1 do begin
  r[i_rhox,j]=r[i_rhox,j-1] + (r[i_depth,j]-r[i_depth,j-1]) * $
                          0.5*(r[i_rho  ,j]+r[i_rho  ,j-1])
endfor ; j
;
; --- Rosseland optical depth ---
r[i_tauRoss,0]=0.75*r[i_kapRoss,0]*r[i_P,0]/grav
for j=1,jstop-1 do begin
  r[i_tauRoss,j]=r[i_tauRoss,j-1] + (r[i_depth,j]               -r[i_depth,j-1]                 ) * $
                                0.5*(r[i_rho  ,j]*r[i_kapRoss,j]+r[i_rho  ,j-1]*r[i_kapRoss,j-1])
endfor ; j
;
; --- Shifted depth scale ---
height=-reform(r[i_depth,*])
logtauRoss=alog10(reform(r[i_tauRoss,*]))
height_0=interpol(height, logTauRoss, [0.0], /spline)
r[i_height,*]=height-height_0[0]
;
end ; rdphoenix
