pro rdphoenix20, filename, r, head=head, Teff=Teff, grav=grav
;------------------------------------------------------------------------------
;+
; NAME:
;   rdphoenix20 ('read_phoenix_20')
;
; PURPOSE:
;   Load data array from atmosphere file produced by the PHOENIX program.
;
; CATEGORY:
;   Input/Output
;
; INPUT:
;   filename - Name of data file (including path)
;
; OUTPUT:
;   r - Array with output data, r[17,jstop]
;       r[ 0,*] - tau_standard
;       r[ 1,*] - Temperature [K]
;       r[ 2,*] - flxrad
;       r[ 3,*] - Temperature of radiation [K]
;       r[ 4,*] - bkmean
;       r[ 5,*] - jkmean
;       r[ 6,*] - fkmean
;       r[ 7,*] - rkmean
;       r[ 8,*] - Gas       pressure [dyn/cm^2]
;       r[ 9,*] - Electron  pressure [dyn/cm^2]
;       r[10,*] - Density [g/cm^3]
;       r[11,*] - aradtotal
;       r[12,*] - Radius [cm]
;       r[13,*] - vel
;       r[14,*] - RHOX [cm^-2]
;       r[15,*] - Height [cm]
;       r[16,*] - Height1 [cm] (from P, rho, and grav: might be less buggy)
;
; KEYWORDS:
;   head:   ('header') Output: string array with some information
;   Teff:   ('Temperature_effective') float with effective temperature
;   grav:   ('gravity') float with gravity
;
; ROUTINES:
;
; MODIFICATION HISTORY:
;   2007-10-11 (B.F. Lyon) Adapted from rdmarcs
;   2008-04-22 (B.F.) Adapted from rdphoenix
;   2009-03-10 (B.F.) Integrate additional height scale (column 16) from P, rho, and grav
;   2010-01-15 (B.F.) Skip 'ul_*' Entries
;-
;------------------------------------------------------------------------------

print, 'Loading: ', filename
get_lun, nc
openr, nc, filename
line=""

; === Read atmosphere ===

readf, nc, line
print, 'First line: ', line

; --- Get number of vertical points ---
jstop=0L
readf, nc, jstop
help, jstop

; --- Read Teff ---
Teff=0.0D+00
readf, nc, Teff
help, Teff

; --- Determine grav from filename ---
FileWithoutPath=strmid(filename, strpos(filename,'/',/REVERSE_SEARCH)+1)
ItemsBetweenMinus=strsplit(FileWithoutPath, '-', /extract)
if (n_elements(ItemsBetweenMinus) ge 2) then begin
  grav=10.0D+00^double(ItemsBetweenMinus[1])
endif else begin
  print, 'Could not determine gravity.'
  grav=0.0
endelse
help, grav

; === Get bulk data ===

ncol=14L
r=dblarr(ncol+3, jstop)
head=strarr(ncol+3)
help, r

for ic=0L, ncol-1L do begin
  ;print, ic
  ;
  ; --- Guess or read name of variable ---
  if (ic eq 0) then begin
    head[ic]='tau'
  endif else if (ic eq 1) then begin
    head[ic]='T'
  endif else begin
    HeadEntry=''
    readf, nc, HeadEntry
    HeadEntry=strsplit(HeadEntry, /extract)
    head[ic]=HeadEntry[0]
  endelse
  ;help, head[ic]
  ;
  if (head[ic] eq 'ul_db') then begin
    ; --- Skip four entries
    HeadEntry=''
    EntryData=dblarr(jstop)
    readf, nc, EntryData
    readf, nc, HeadEntry
;help, HeadEntry
    readf, nc, EntryData
    readf, nc, HeadEntry
;help, HeadEntry
    readf, nc, EntryData
    readf, nc, HeadEntry
;help, HeadEntry
    readf, nc, EntryData
    readf, nc, HeadEntry
    HeadEntry=strsplit(HeadEntry, /extract)
    head[ic]=HeadEntry[0]
    ;help, head[ic]
  endif
  ;
  ; --- Read data for entry ---
  EntryData=dblarr(jstop)
  readf, nc, EntryData
  r[ic,*]=EntryData
endfor

; --- Chop of topmost layer ---
r=r[*,1:*]
jstop=jstop-1L

; === Compute additional data ===

; --- Some indices ---
i_P      = 8
i_rho    =10
i_tau    = 0
i_radius =12
i_rhox   =14
i_height =15
i_height1=16

; --- Mass density per area ---
r[i_rhox,0]=r[i_P,0]/grav
for j=1,jstop-1 do begin
  r[i_rhox,j]=r[i_rhox,j-1] - (r[i_radius,j]-r[i_radius,j-1]) * $
                          0.5*(r[i_rho   ,j]+r[i_rho   ,j-1])
endfor ; j

; --- Integrated height from P, rho, and grav ---
dh=deriv(reform(r[i_P,*]))/(grav*reform(r[i_rho,*]))
height1=total(-dh, /cumulative)
logtau=alog10(reform(r[i_tau,0:*]))
height_0=interpol(height1, logTau, [0.0], /spline)
r[i_height1,*]=height1-height_0[0]


; --- Shifted depth scale ---
height=reform(r[i_radius,0:*])
logtau=alog10(reform(r[i_tau,0:*]))
height_0=interpol(height, logTau, [0.0], /spline)
r[i_height,*]=r[i_radius,*]-height_0[0]

; --- Add string to head ---
head=strjoin(head, ' ') + 'rhox height height1'

; --- Close file ---
close, nc
; --- Free unit ---
free_lun, nc

end ; rdphoenix20
