pro rdphoenixout, filename, r, head=head, Teff=Teff, grav=grav
;------------------------------------------------------------------------------
;+
; NAME:
;   rdphoenixout ('read_phoenix_out')
;
; PURPOSE:
;   Load data array from atmosphere file produced directly by the PHOENIX program.
;
; CATEGORY:
;   Input/Output
;
; INPUT:
;   filename - Name of data file (including path)
;
; OUTPUT:
;   r - Array with output data, r[17,jstop]
;       r[ 0,*] - Level
;       r[ 1,*] - tau_standard
;       r[ 2,*] - Temperature [K]
;       r[ 3,*] - Gas       pressure [dyn/cm^2]
;       r[ 4,*] - Electron  pressure [dyn/cm^2]
;       r[ 5,*] - Density [g/cm^3]
;       r[ 6,*] - Molecular weight mu
;       r[ 7,*] - Radius [cm]
;       r[ 8,*] - Extension (depth) [cm]
;       r[ 9,*] - kapq
;       r[10,*] - kappa_Ross [cm^2/g]

;       r[11,*] - RHOX [cm^-2]
;       r[12,*] - tau_Rosseland
;       r[13,*] - Height [cm]
;       r[14,*] - suf [10^8 erg/cm^2/s]
;
; KEYWORDS:
;   head:   ('header') Output: string array with some information
;   Teff:   ('Temperature_effective') float with effective temperature
;   grav:   ('gravity') float with gravity
;
; ROUTINES:
;   rdarray: Read file and put contents into array.
;
; EXAMPLE:
;   rdphoenixout, '/Users/bfreytag/dat/atm/phoenix/AMES-Dusty/lte09-5.0-0.0.AMES-dusty.out',r, Teff=Teff, grav=grav
;
; MODIFICATION HISTORY:
;   2007-10-11 (B.F. Lyon) Adapted from rdmarcs
;   2008-10-01 (B.F.) rdphoenix -> rdphoenixout
;   2009-01-21 (B.F.) New column with radiative flux
;-
;------------------------------------------------------------------------------

on_ioerror, label10

; === Read atmosphere ===

; --- Open file ---
print, 'Loading: ', filename
openr, nc, filename, error=ierr, /get_lun
help,ierr
if (ierr gt 0) then stop
nline=0L

; --- Read model parameters from parameter block ---
line=''
readon_flag=1
while (readon_flag) do begin
  readf, nc, line, format='(A)'
  nline=nline+1L
  if (stregex(line, '^ *model: *teff *:', /boolean)) then begin
    print, 'Block with model parameters found'
    print, line
    ; --- Extract Teff ---
    Teff=float((stregex(line, '^ *model: *teff *: *([^ ][^ ]*) .*', /subexpr, /extract))[1])
    help, Teff
    ; --- Skip line ---
    readf, nc, line, format='(A)'
    nline=nline+1L
    ; --- Read and extract gravity ---
    readf, nc, line, format='(A)'
    nline=nline+1L
    print, line
    logg=float((stregex(line, '^ *log\(g\) *: *([^ ][^ ]*) .*', /subexpr, /extract))[1])
    grav=10.0^logg
    help, grav
    ;
    readon_flag=0
  endif
endwhile

; --- Read first (main) table ---
readon_flag=1
while ((readon_flag) and not (EOF(nc))) do begin
  readf, nc, line, format='(A)'
  nline=nline+1L
  if (strmid(line, 0, 28) eq '   l        tstd temperature') then begin
    head=line
    print, head
    rdarray, filename, a, SkipLines=nline+1L, /StopAtEmptyLine
    a=transpose(a)
    help, a
    jstop=n_elements(a[0,*])
    ncol =n_elements(a[*,0])
    ;
    ; --- Create data array ---
    r=fltarr(ncol+5,jstop)
    r[0:ncol-1,*]=a
    ;
    readon_flag=0
  endif
endwhile

; --- Read second table ---
readon_flag=1
while ((readon_flag) and not (EOF(nc))) do begin
  readf, nc, line, format='(A)'
  nline=nline+1L
  if (strmid(line, 0, 29) eq '    i         tau        temp') then begin
    print, line
    rdarray, filename, a, SkipLines=nline+1L, /StopAtEmptyLine
    a=transpose(a)
    help, a
    ;jstop=n_elements(a[0,*])
    ;ncol =n_elements(a[*,0])
    ;
    ; --- Add column to data array ---
    r[10,*]=a[1,*]
    ;
    readon_flag=0
  endif
endwhile

; --- Read third table ---
readon_flag=1
while ((readon_flag) and not (EOF(nc))) do begin
  readf, nc, line, format='(A)'
  nline=nline+1L
  ;if (strmid(line, 0, 29) eq '    i         tau        temp') then begin
  if (strmid(line, 0, 29) eq '   no          tk         sub') then begin
    print, line
    rdarray, filename, a, SkipLines=nline+1L, /StopAtEmptyLine
    help, a
    ind=where(a[*,0] ne 0.0, nind)
    a=a[ind,*]
    a=transpose(a)
    help, a
    ;
    ; --- Add column to data array ---
    r[14,*]=a[6,*]
    ;
    readon_flag=0
  endif
endwhile


; === Compute additional data ===

head=head + ' rhox tau_Rosseland height'
help, head

; --- Some indices ---
;i_T      = 2
i_P      = 3
i_rho    = 5
i_depth  = 8
i_kapRoss=10
i_rhox   =11
i_tauRoss=12
i_height =13

; --- Mass density per area ---
r[i_rhox,0]=r[i_P,0]/grav
for j=1,jstop-1 do begin
  r[i_rhox,j]=r[i_rhox,j-1] + (r[i_depth,j]-r[i_depth,j-1]) * $
                          0.5*(r[i_rho  ,j]+r[i_rho  ,j-1])
endfor ; j

; --- Rosseland optical depth ---
r[i_tauRoss,0]=0.75*r[i_kapRoss,0]*r[i_P,0]/grav
for j=1,jstop-1 do begin
  r[i_tauRoss,j]=r[i_tauRoss,j-1] + (r[i_depth,j]               -r[i_depth,j-1]                 ) * $
                                0.5*(r[i_rho  ,j]*r[i_kapRoss,j]+r[i_rho  ,j-1]*r[i_kapRoss,j-1])
endfor ; j

; --- Shifted depth scale ---
height=-reform(r[i_depth,*])
logtauRoss=alog10(reform(r[i_tauRoss,*]))
height_0=interpol(height, logTauRoss, [0.0], /spline)
r[i_height,*]=height-height_0[0]

; === Error handling ===

goto, label20
label10: 
print, 'An I/O error occured'

label20:
free_lun, nc

end ; rdphoenixout
