function spec_rdflux, infile, NDATASET=ndataset, $
                              VERBOSE=verbose, $
                              OUTSTR=outstr, IERR=ierr
;
;------------------------------------------------------------------------------
;+
; NAME:
;   spec_load_flux()  ('spectrum_synthesis_load_flux_file')
;
; PURPOSE:
;   Load flux data from one file into an array of 
;   anonymous IDL structures.
;
; PROCEDURE:
;   Reads first data set with uio_dataset_rd().
;   Returns an empty structure if there is an I/O error.
; 
;
; CATEGORY:
;   SPEC, I/O
;
; CALLING SEQUENCE:
;   structure = spec_rdflux(infile, ndataset=ndataset, $
;                           outstr=outstr, ierr=ierr)
;
; INPUTS:
;   filestring:  string, input file name (e.g., 'st35gm04n29.flx'),
;                with full path name if not in current directory.
; KEYWORDS:
;   ndataset:    input integer scalar/vector with data set number(s).
; 	         Counting starts at 0 (which is the default).
;   verbose:     input keyword; prints current dataset number if set
;   outstr:      output message string ('done' if no errors)
;   ierr:        output error code (0 if no errors)
;
; OUTPUTS:
;   IDL structure containing flux data
;   ...
;
; COMMON BLOCKS:
;
; ROUTINES:
;   uio_dataset_rd, strn (ASTROLIB)
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; EXAMPLE:
;   filename = 'st35gm04n29_15to18_3l.flx'
;   nsteps   = uio_nidents(filename, 'flux')
;   fluxarr  = spec_rdflux(filename, ndataset=indgen(nsteps), /verbose)
;   help, fluxarr
; 
; MODIFICATION HISTORY:
;   2004-05-03  Written by Michelle Mizuno
;-
;------------------------------------------------------------------------------

  if n_params() LT 1 then begin
    print, 'USAGE:  structure = spec_load_flux(' $
      + 'infile [,NDATASET=ndataset][,OUTSTR=outstr][,IERR=ierr])'
    return, {empty:''}
  endif

; --- Check input parameter NDATASET ---
  if n_elements(ndataset) eq 0 then ndataset=0
  if size(ndataset,/TYPE) ne 2 then begin
    on_error, 2
    message, 'NDATASET must be an integer (scalar or array)'
  endif

; --- Read first data set ---
  iset = 0
  dataset = ndataset[iset]
  if keyword_set(verbose) then print, 'Loading dataset ' + strn(iset)
  tmp = uio_dataset_rd(infile, ndataset=dataset, ierr=ierr)
  if (ierr ne 0) then outstr = 'Error opening dataset '+strn(dataset)

; --- Allocate and fill output array ---
  IF (ierr eq 0) THEN BEGIN
    nsets = n_elements(ndataset)
    output_array = replicate(tmp, nsets)
    WHILE (++iset lt nsets) and (ierr eq 0) DO BEGIN
      dataset = ndataset[iset]
      if keyword_set(verbose) then $
        print, 'Loading dataset ' + strn(iset) + ':'
      tmp = uio_dataset_rd(infile, ndataset=dataset, ierr=ierr)
      if (ierr eq 0) then begin
        output_array[iset] = tmp
        if keyword_set(verbose) then $
          help,output_array[iset],/str
      endif else begin
        errmsg = 'Error  reading dataset ' + strn(dataset)
      endelse
    ENDWHILE
  ENDIF

; --- Return ---
  if (ierr ne 0) then output_array={empty:''} else outstr='done'
  return, output_array

END
