pro lawe_boundaryconditions, bound, StartEnd, VariableSet, boundVec
;
;------------------------------------------------------------------------------
; NAME:
;   lawe_boundaryconditions ('linear_adiabatic_wave_equation_boundary_conditions')
;
; PURPOSE:
;   Set boundary conditions for LAWE.
;
; CATEGORY:
;   Hydrodynamics, differential equations
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   2006-05-09 (B.F.) Put into own routine
;------------------------------------------------------------------------------

if (VariableSet eq 'divv,vz') then begin
  if (StartEnd eq 'start') then begin
    ; --- Lower boundary ---
    ;
    if (bound eq 'vz=0') then begin
      boundVec=[1.0D-200, 0.0D+00]
    endif else if (bound eq 'divv=0') then begin
      boundVec=[0.0D+00, 1.0D-200]
    endif else begin
      message, 'Combination not supported: ' + VariableSet + ' ' + bound + StartEnd
    endelse
    ;
  endif else if (StartEnd eq 'end') then begin
    ; --- Upper boundary ---
    ;
    if (bound eq 'vz=0') then begin
      boundVec=[0.0D+00, 1.0D+00]
    endif else if (bound eq 'divv=0') then begin
      boundVec=[1.0D+00, 0.0D+00]
    endif else begin
      message, 'Combination not supported: ' + VariableSet + ' ' + bound + StartEnd
    endelse
    ;
  endif
  ;
endif else if (VariableSet eq 'vx,vz') then begin
  if (StartEnd eq 'start') then begin
    ; --- Lower boundary ---
    ;
    if (bound eq 'vz=0') then begin
      boundVec=[1.0D-200, 0.0D+00]
    endif else if (bound eq 'vx=0') then begin
      boundVec=[0.0D+00, 1.0D-200]
    endif else begin
      message, 'Combination not supported: ' + VariableSet + ' ' + bound + StartEnd
    endelse
    ;
  endif else if (StartEnd eq 'end') then begin
    ; --- Upper boundary ---
    ;
    if (bound eq 'vz=0') then begin
      boundVec=[0.0D+00, 1.0D+00]
    endif else if (bound eq 'vx=0') then begin
      boundVec=[1.0D+00, 0.0D+00]
    endif else begin
      message, 'Combination not supported: ' + VariableSet + ' ' + bound + StartEnd
    endelse
    ;
  endif
  ;
endif else begin
  message, 'Unknown VariableSet: ' + VariableSet
endelse

end ; lawe_boundaryconditions


pro lawe_qupminimum, nfrequency, frequencylist, qupw, qsign, $
  nmodeforwave, freqllist, freqrlist, qupllist, quprlist
;
;------------------------------------------------------------------------------
; NAME:
;   lawe_qupminimum ('linear_adiabatic_wave_equation_quantity_upper_minimum')
;
; PURPOSE:
;   Find minima that indicate additional zeros.
;
; CATEGORY:
;   Hydrodynamics, differential equations
;
; CALLING SEQUENCE:
;   lawe_qupminimum, nfrequency, frequencylist, qupw, $
;     nmodeforwave, freqllist, freqrlist, qupllist, quprlist
;
; MODIFICATION HISTORY:
;   2006-04-27 (B.F.) First version
;------------------------------------------------------------------------------

indmin=where((qupw[0:nfrequency-3] gt qupw[1:nfrequency-2]) and $
             (qupw[2:nfrequency-1] ge qupw[1:nfrequency-2]) and $
             (0.0D+00              lt qupw[1:nfrequency-2]        )    , nindmin)
if (nindmin gt 0L) then begin
  ; --- Local minimum without zero detected ---
  ;
  indmin=indmin+1L
  ;
  for imin=0L,nindmin-1L do begin
;    extremum, x=frequencylist[indmin[imin]-1L:indmin[imin]+1L]-frequencylist[indmin[imin]], $
;      qupw[indmin[imin]-1L:indmin[imin]+1L], 1, $
;      i0=1L, irange=1, degree=2, $
;      fmin, qupwmin
;    fmin=fmin+frequencylist[indmin[imin]]
    extremum, x=frequencylist, qupw, 1, $
      i0=indmin[imin], irange=1, degree=2, $
      fmin, qupwmin
    ;
    print, '  Local extremum: ', indmin[imin], frequencylist[indmin[imin]], fmin, $
           qupw[indmin[imin]], qupwmin, format='(A,I6,1X,4(E14.6))'
    ;
    ifmin=long(interpol(lindgen(nfrequency),frequencylist,fmin))
    if (qupwmin le 0.0D+00) then begin
      if (nmodeforwave le 0L) then begin
        freqllist=[frequencylist[ifmin], fmin                   ]
        freqrlist=[fmin                , frequencylist[ifmin+1L]]
        qupllist =[qsign*qupw[ifmin]  , qsign*qupwmin ]
        quprlist =[qsign*qupwmin, qsign*qupw[ifmin+1L]]
      endif else begin
        freqllist=[freqllist, frequencylist[ifmin], fmin                   ]
        freqrlist=[freqrlist, fmin                , frequencylist[ifmin+1L]]
        qupllist =[qupllist , qsign*qupw[ifmin]  , qsign*qupwmin ]
        quprlist =[quprlist , qsign*qupwmin, qsign*qupw[ifmin+1L]]
      endelse
      nmodeforwave=nmodeforwave+2L
    endif
    ;
  endfor ; imin
  ;
  if (nmodeforwave gt 0L) then begin
    ; --- Sort frequencies ---
    ind=sort(freqllist)
    freqllist=freqllist[ind]
    freqrlist=freqrlist[ind]
    qupllist =qupllist[ ind]
    quprlist =quprlist[ ind]
  endif
endif

end ; lawe_qupminimum


pro lawe_deqintegrate, VariableSet, solveDEQmode, boundlow, boundup, $
  nz, dz, omega, omegasqrsign, khorizontal, grav, Gamma1, csound, omegaBVsqrograv, $
  q, boundout, totalphase, LastBackward=LastBackward
;
;------------------------------------------------------------------------------
; NAME:
;   lawe_deqintegrate ('linear_adiabatic_wave_equation_differential_equation_integrate')
;
; PURPOSE:
;   Integrate the linear adiabatic wave equation from bottom to top or backward.
;
; CATEGORY:
;   Hydrodynamics, differential equations
;
; CALLING SEQUENCE:
;   lawe_deqintegrate, VariableSet, solveDEQmode, boundlow, boundup, $
;     nz, dz, omega, omegasqrsign, khorizontal, grav, Gamma1, csound, $
;     q, boundout, LastBackward=LastBackward
;
; MODIFICATION HISTORY:
;   2006-04-06 (B.F.) First version
;   2006-04-20 (B.F.) Bugfix in 'pexp' solver
;   2006-04-21 (B.F.) omegasqrsign as input paramter
;   2006-04-25 (B.F.) Keyword LastBackward added: keep expBarr and reuse it
;   2006-04-26 (B.F.) Output of totalphase
;   2006-05-08 (B.F.) Keyword VariableSet added
;   2006-05-09 (B.F.) Keyword omegaBVsqrograv added,
;                     boundary conditions into separate routine
;------------------------------------------------------------------------------

common laweCOMMON, expBarr

; --- Create arrays for independent quantities and DEQ coefficients ---
q   =dblarr(2,  nz  )
dqdz=dblarr(2,2,nz  )
totalphase=0.0D+00

; --- Scale factor ---
;S=1.0D+07

if (not keyword_set(LastBackward)) then begin
  ; --- DEQ coefficients ---
  if (VariableSet eq 'divv,vz') then begin
    dqdz[0,0,*]=grav*Gamma1/csound^2 - omegasqrsign * grav*(khorizontal/omega)^2
;    dqdz[0,1,*]=S          *(omegasqrsign * (-(omega/csound)^2 + (grav*khorizontal/(csound*omega))^2))
;    dqdz[1,0,*]=(1.0D+00/S)*(1.0D+00 - omegasqrsign * (csound*khorizontal/omega)^2)
    dqdz[0,1,*]=            (omegasqrsign * (-(omega/csound)^2 + (grav*khorizontal/(csound*omega))^2))
    dqdz[1,0,*]=            (1.0D+00 - omegasqrsign * (csound*khorizontal/omega)^2)
    dqdz[1,1,*]=grav*omegasqrsign * (khorizontal/omega)^2
  endif else if (VariableSet eq 'vx,vz') then begin
    dqdz[0,0,*]=omegaBVsqrograv
    dqdz[0,1,*]=khorizontal * (omegaBVsqrograv*grav/(omegasqrsign*omega^2)-1.0D+00)
    dqdz[1,0,*]=(omegasqrsign*omega^2)/(khorizontal*csound^2) - khorizontal
    dqdz[1,1,*]=grav/csound^2
  endif else begin
    message, 'VariableSet not supported: ' + VariableSet
  endelse
  ;
  ;print, 'min= ', minmax(abs(dqdz[0,1,*])), minmax(abs(dqdz[1,0,*])), format='(A,2E13.6,2X,2E13.6)'
  ;
  ; --- Set lower start boundary condition ---
  lawe_boundaryconditions, boundlow, 'start', VariableSet, boundVec
  q[*,0]=boundVec
endif else begin
  ; --- Set upper start boundary condition ---
  lawe_boundaryconditions, boundup , 'start', VariableSet, boundVec
  q[*,nz-1]=boundVec
endelse

; --- Integration of the DEQ from bottom to top ---
if (solveDEQmode eq 'euler') then begin
  ; === Simple first-order Euler scheme ===
  ;
  for iz=0L,nz-2L do begin
    q[*,iz+1L]=q[*,iz] + dz[iz]*(dqdz[*,*,iz]#q[*,iz])
    ;print, '    Height ', iz, z[iz], q[*,iz], ' ->', q[*,iz+1L], format='(A,I4,E13.5,2E13.5,A,2E13.5)'
    ;print, dqdz[*,*,iz], dqdz[*,*,iz]#q[*,iz]
  endfor ; iz
  ;
endif else if (solveDEQmode eq 'euler2half') then begin
  ; === Two half first-order Euler steps ===
  ;
  for iz=0L,nz-2L do begin
    q[*,iz+1L]=q[*,iz   ] + 0.5D+00*dz[iz]*(dqdz[*,*,iz   ]#q[*,iz   ])
    q[*,iz+1L]=q[*,iz+1L] + 0.5D+00*dz[iz]*(dqdz[*,*,iz+1L]#q[*,iz+1L])
    ;print, '    Height ', iz, z[iz], q[*,iz], ' ->', q[*,iz+1L], format='(A,I4,E13.5,2E13.5,A,2E13.5)'
    ;print, dqdz[*,*,iz], dqdz[*,*,iz]#q[*,iz]
  endfor ; iz
  ;
endif else if (solveDEQmode eq 'trapezsym') then begin
  ; === Implicit second order scheme: trapezoidal rule ===
  ;
  for iz=0L,nz-2L do begin
    ; --- Explicit half step ---
    q[*,iz+1L]=q[*,iz   ] + 0.5D+00*dz[iz]*(dqdz[*,*,iz   ]#q[*,iz   ])
    ; --- Implicit half step ---
    C=[[1.0D+00 - 0.5D+00*dz[iz]*dqdz[0,0,iz+1L], $
                - 0.5D+00*dz[iz]*dqdz[1,0,iz+1L]], $
       [        - 0.5D+00*dz[iz]*dqdz[0,1,iz+1L], $
        1.0D+00 - 0.5D+00*dz[iz]*dqdz[1,1,iz+1L]]]
    C=invert(C, /double)
    q[*,iz+1L]=C#q[*,iz+1L]
  endfor ; iz
  ;
endif else if (solveDEQmode eq 'pexp') then begin
  ; === Piecewise exponentials ===
  ;
  if (not keyword_set(LastBackward)) then begin
    ; --- Integrate from bottom to top and remember propagation matrices ---
    ;
    expBarr=dblarr(2,2,nz)
    ;
    ; --- Create variable for output of eigenvec function ---
    residual=1
    ;
    ; --- Loop over all height invervals ---
    for iz=0L,nz-2L do begin
      ; --- Averaged system matrix ---
      B=0.5D+00*dz[iz] * (dqdz[*,*,iz  ]+dqdz[*,*,iz+1L])
      ;
      ; --- Reduce to upper Hessenberg format and determine eigenvalues ---
      eigva=HQR(elmhes(B, /double), /double)
      ;
      totalphase=totalphase + max(abs(imaginary(eigva)))
      ;
      if (B[1,0] eq 0.0D+00) then begin
        ; --- First component is easy: no influence of second -> just advance in time ---
        ;q[0,iz+1L]=q[0,iz]*exp(B[0,0])
        ;
        if (abs(B[1,1]-B[0,0]) lt 1.0D-06) then begin
          ; --- Small effective step ---
          ;
          ;q[1,iz+1L]=(q[1,iz] + B[0,1]*q[0,iz])* exp(B[1,1])
          expB=[[exp(B[0,0])       , 0.0         ], $
                [B[0,1]*exp(B[1,1]), exp(B[1,1])]]
        endif else begin
          ; --- Decent step size ---
          ;q[1,iz+1L]=q[1,iz]*exp(B[1,1]) + $
          ;           (B[0,1]*q[0,iz])/(B[0,0]-B[1,1]) * (exp(B[0,0])-exp(B[1,1]))
          expB=[[exp(B[0,0])                                       , 0.0         ], $
                [B[0,1]/(B[0,0]-B[1,1]) * (exp(B[0,0])-exp(B[1,1])), exp(B[1,1])]]
        endelse
        print, '  ************* Special case: B[1,0]=0 ********************'
        ;
      endif else begin
        ; === Regular matrix ===
        ;
        ; --- Compute eigenvectors ---
        evec=eigenvec(B, eigva, /double, residual=residual)
        ;
        ; --- Build unitary matrix and its inverse ---
        U    =transpose(evec)
        Ustar=invert(U, /double)
        ;
        ; --- Transformed system matrix ---
        ; --- (check only: it should have the eigenvalues on the diagonal) ---
        ; Bdash=U#B#Ustar
        ; --- Transform vector of dependent variables ---
        ; qdash=U#q[*,iz]
        ;
        ;q[*,iz+1L]=double(Ustar#(exp(eigva)*(U#q[*,iz])))
        ;expBdash=diag_matrix(exp(eigva))
        ;q[*,iz+1L]=double(Ustar#(expBdash#(U#q[*,iz])))
        ;expB=double(Ustar#expBdash#U)
        expB=double(Ustar#diag_matrix(exp(eigva))#U)
      endelse
      ;
      expBarr[*,*,iz]=expB
      q[*,iz+1L]=expB#q[*,iz]
      ; 
      ;    print, iz, eigva, q[*,iz+1L]
    endfor ; iz
    ;
    ;print, minmax(expBarr)
    ;
    ;     print, '--------------------------------'
    ;    print, U, Ustar#U
    ;    print
    ;    print, B, U#B#Ustar, Ustar#B#U
    ;    print
    ;
  endif else begin
    ; --- Integrate from top to bottom and reuse propagation matrices ---
    ;
    ;print, 'LastBackward: ', minmax(expBarr)
    ;
    ; --- Loop over all height invervals, from top to bottom ---
    for iz=nz-2L,0L,-1L do begin
      q[*,iz]=invert(expBarr[*,*,iz])#q[*,iz+1L]
    endfor ; iz
    ;
  endelse
  ;
endif else begin
  message, 'Unknown mode for the solution of the DEQ.'
endelse

;q[0,*]=q[0,*]/S

if (not keyword_set(LastBackward)) then begin
  ; --- Store deviation from fullfillment of upper boundary condition into output array---
  ;
  lawe_boundaryconditions, boundup, 'end', VariableSet, boundVec
  boundout=total(boundVec*q[*,nz-1])
endif else begin
  ; --- Store deviation from fullfillment of lower boundary condition into output array---
  ;
  lawe_boundaryconditions, boundlow, 'end', VariableSet, boundVec
  boundout=total(boundVec*q[*,0])
endelse

end ; lawe_deqintegrate


pro lawe, dat, $
  model=model0, $
  z=z0, csound=csound0, omegaBVsqrograv=omegaBVsqrograv0, $
  Gamma1=Gamma10, grav=grav0, xtotal=xtotal0, $
  nhwavenumber=nhwavenumber0, hwavenumberlist=hwavenumberlist0, $
  nfrequency=nfrequency0, frequencylist=frequencylist0, $
  omegasqrsign=omegasqrsign0, $
  solveDEQmode=solveDEQmode0, boundlow=boundlow0, boundup=boundup0, $
  VariableSet=VariableSet0, $
  nzinterpol=nzinterpol0, gridopt=gridopt0
;
;-------------------------------------------------------------------------------
;+
; NAME:
;   lawe ('linear_adiabatic_wave_equation')
;
; PURPOSE:
;   Solve the linear adiabatic wave equation as boundary value problem.
;
; CATEGORY:
;   Hydrodynamics, differential equations
;
; CALLING SEQUENCE:
;
; INPUT:
;   None (control via keywords)
;
; OUTPUT:
;   dat:    ('data') structure with computed results and some of the input data
;
; KEYWORDS: (all input)
;   model:           ('model') string with information about base model
;   z:               ('z_axis') vector with vertical axis
;   csound:          ('c_sound') vector or scalar with sound speed
;   omegaBVsqrograv: ('omega_Brunt_Vaeisaelae_square_over_grav') vector or scalar with
;                    omega_BV^2/grav=1/H_rho - 1/(Gamma_1*H_p)
;                                   =delta/c_p * ds/dz
;                                   =dlncsound^2/dz (- dlnGamma_1/dz) + (Gamma_1-1)*Grav/csound^2
;                    
;   Gamma1:          ('Gamma_1') scalar with first adiabatic coefficient
;   grav:            ('gravity') scalar with acceleration due to gravity
;   xtotal:          ('x_total') scalar with total horizontal extent of box
;   nhwavenumber:    ('number_horizontal_wavenumber') number of horizontal wavenumbers
;                    to consider
;   hwavenumberlist: ('horizontal_wavenumber_list') list of horizontal wavenumbers
;                    If this array is specified the nhwavenumber parameter is ignored.
;   nfrequency:      ('number_frequency') number of frequencies to consider
;   frequencylist:   ('frequency_list') real or double array with frequencies.
;                    If this array is specified the nfrequency parameter is ignored.
;   omegasqrsign:    ('omega_square_sign') scalar with sign of omega^2:
;                    +1.0: oscillating modes
;                    -1.0: modes with exponential time behavior
;   solveDEQmode:    ('solve_differential_equation_mode') string with DEQ solver to use:
;                    'Euler'     : very simple first-order scheme
;                    'Euler2half': simple (first or second order?) scheme
;                    'Trapezsym' : second order scheme
;                    'pexp'      : (default) piecewise exponential
;   boundlow:        ('boundary_lower') string with description of lower boundary condition:
;                    'vz=0'   : zero vertical velocities (default)
;                    'divv=0': zero divergence of velocity
;   boundup:         ('boundary_upper') string with description of upper boundary condition
;                    'vz=0'   : zero vertical velocities (default)
;                    'divv=0': zero divergence of velocity
;   VariableSet:     ('Variable_Set') string with names of independent variables:
;                    'divv,vz': (default)
;                    'vx,vz'  :
;   nzinterpol:      ('number_z_interpolate') number of grid points of refined grid
;   gridopt:         ('grid_optimization') string with grid refinement scheme:
;                    'no'      : Just keep the old grid
;                    'linear'  : Interpolate linearly onto new grid.
;                                The old points are possibly not preserved.
;                    'linear_b': Interpolate linearly onto new grid.
;                                The old points are possibly not preserved.
;                                Two "half" grid cells at bottom and top are added.
;                    'opt1'    : Insert new grid points to get step in dlogcsound^2/dz small.
;                                Old grid points are preserved.
;                    'opt1_b'  : Insert new grid points to get step in dlogcsound^2/dz small.
;                                Old grid points are preserved.
;                                Two "half" grid cells at bottom and top are added.
;                    'opt2'    : Insert new grid points to get step in dlogcsound^2/dz small.
;                                Old grid points are preserved.
;                                A minimum refinement everywhere is favored.
;                    'opt2_b'  : Insert new grid points to get step in dlogcsound^2/dz small.
;                                Old grid points are preserved.
;                                A minimum refinement everywhere is favored.
;                                Two "half" grid cells at bottom and top are added.
;;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   lawe_deqintegrate:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   lawe, dat, z=findgen(200)*55.0D+05, csound=1.0D+08, Gamma1=1.5, grav=1.0D+08, xtotal=3.3D+09
;   help, /structure, dat, dat.mode
;   plotfield,/tv,/t_sample,dat.frequency,dat.hwavenumber,transpose(float(dat.qd)),/oldaspect
;   plotlines,dat.mode.frequency,dat.mode.hwavenumber,psym=1,/over,linecolor=!ctv.red
;
;   restore,'lc0dh_seq_10.sav'
;   z=seqavg('xc3',seq,mtime=600)*1e8
;   ;csound=seqavg('c',seq,mtime=600)*1e5
;   csound=reform(sqrt(seq.Gamma1[*,600]*seq.P[*,600]/seq.rho[*,600]))
;   ;Gamma1=avg(seqavg('Gamma1',seq,mtime=600))
;   Gamma1=avg(reform(seq.Gamma1[*,600]))
;   lawe, dat, z=z, csound=csound, Gamma1=Gamma1, grav=10.0^7.695, xtotal=3.3D+09, nhwave=20L
;   ;
;   restore,'lc0gg1_seq_10.sav'
;   z=seqavg('xc3',seq,mtime=1500)*1e8
;   csound=seqavg('c',seq,mtime=1500)*1e5
;   s=seqavg('s',seq,mtime=1500)*1e9
;   dsdz=deriv(z,s)
;   Gamma1=avg(seqavg('Gamma1',seq,mtime=1500))
;   cp=avg(seqavg('cp',seq,mtime=1500))
;   delta=avg(seqavg('delta',seq,mtime=1500))
;   freq=(dindgen(981)+20.0)/2000.0
;   lawe, dat, z=z, csound=csound, Gamma1=Gamma1, grav=10.0^7.695, xtotal=3.3D+09, freq=freq, gridopt='opt1_b', nzinter=400L
;
;   freq=(dindgen(211)+40.0)/20000.0
;   lawe, dat, z=z, csound=csound, Gamma1=Gamma1, grav=10.0^7.695, xtotal=3.3D+09, freq=freq, omegasqrsign=-1.0, nzinter=400, gridopt='opt1_b'
;
;   freq1=1.0/(100.0-dindgen(96))
;   freq2=dindgen(200)/400+0.21
;   freq=[freq1,freq2]
;
; MODIFICATION HISTORY:
;   2006-04-06 (B.F.) First version
;   2006-04-07 (B.F.) More keywords
;   2006-04-20 (B.F.) Interpolation onto new grid possible
;   2006-04-21 (B.F.) omegasqrsign as input paramter
;   2006-04-22 (B.F.) Grid optimization according to gradient of log(sound speed)
;   2006-04-25 (B.F.) Compute "reverse solution" for pexp scheme and merge both solutions
;                     Check for correlation of both solutions
;                     Default scheme: pexp
;   2006-04-26 (B.F.) Used quadratic interpolation to get new sound speed,
;                     Add point at bottom or top if gridopt='*_b'
;   2006-04-27 (B.F.) Try to find also modes that give only a local minimum in qup
;   2006-04-28 (B.F.) Integration of rho, new normalization, check of mode orthogonality 
;   2006-05-02 (B.F.) Use linear interpolation again to get new sound speed,
;   2006-05-08 (B.F.) Keyword VariableSet added
;   2006-05-09 (B.F.) Keyword omegaBVsqrograv added, expand scalar csound to vector,
;                     boundary condition: 'v=0' -> 'vz=0'
;-
;-------------------------------------------------------------------------------

; === Check input parameters ===================================================

; --- Number of points in vertical direction ---
nz=n_elements(z0)
if (nz le 1) then begin
  message, 'z axis has not enough points.'
endif else begin
  z       =double(z0)
  zorg    =z
  ind_zorg=dindgen(n_elements(zorg))
endelse

; --- Gravity ---
if (n_elements(grav0) eq 0) then begin
  grav=0.0D+00
endif else begin
  grav=double(grav0)
endelse
if (n_elements(grav) gt 1) then begin
  message, 'Only scalar values for variable "grav" allowed.'
endif

; --- 1st adiabatic coefficient ---
if (n_elements(Gamma10) eq 0) then begin
  Gamma1=1.666666666666D+00
endif else begin
  Gamma1=double(Gamma10)
endelse
if (n_elements(Gamma1) gt 1) then begin
  message, 'Only scalar values for variable "Gamma1" allowed.'
endif

; --- Sound speed ---
if (n_elements(csound0) eq 0) then begin
  ; --- Default value: 10 km/s ---
  csound=replicate(1.0D+06, nz)
endif else if (n_elements(csound0) eq 1) then begin
  csound=replicate(double(csound0), nz)
endif else if (n_elements(csound0) eq nz) then begin
  csound=double(csound0)
endif else begin
  message, 'Variable "csound" has weird number of elements: ' + $
    trim(string(n_elements(csound0)),2)
endelse

; --- omega_BV^2/grav ---
if (n_elements(omegaBVsqrograv0) eq 0) then begin
  ; --- Compute from sound speed (term -dlnGamma1/dz missing) ---
  omegaBVsqrograv=deriv(z,alog(csound^2)) + (Gamma1-1.0D+00)*grav/csound^2
endif else if (n_elements(omegaBVsqrograv0) eq 1) then begin
  omegaBVsqrograv=replicate(double(omegaBVsqrograv0), nz)
endif else if (n_elements(omegaBVsqrograv0) eq nz) then begin
  omegaBVsqrograv=double(omegaBVsqrograv0)
endif else begin
  message, 'Variable "omegaBVsqrograv" has weird number of elements: ' + $
    trim(string(n_elements(omegaBVsqrograv0)),2)
endelse

; --- Interpolation onto new grid ---
if (n_elements(gridopt0) eq 0) then begin
  ; --- Default: no optimization ---
  if (not (keyword_set(nzinterpol0))) then begin
    gridopt='no'
  endif else begin
    gridopt='linear'
  endelse
endif else begin
  gridopt=strlowcase(string(gridopt0))
endelse
if (not (keyword_set(nzinterpol0))) then begin
  ; --- No change of grid ---
  ;
  nzinterpol=nz
  ;
endif else begin
  if (strmid(gridopt,1,/reverse) eq '_b') then begin
    ; --- Insert point at upper and lower boundary ---
    ;
    zold              =z
    csoundold         =csound
    omegaBVsqrogravold=omegaBVsqrograv
    ;
    z=dblarr(nz+2L)
    z[1:nz]=zold
    z[0]   =zold[0]    - 0.5D+00*(zold[1]   -zold[0]   )
    z[nz+1]=zold[nz-1] + 0.5D+00*(zold[nz-1]-zold[nz-2])
    ;
    if (n_elements(csound) eq nz) then begin
      csound      =dblarr(nz+2L)
      csound[1:nz]=csoundold
      csound[0]   =interpol(csoundold,zold,[z[0]]   )
      csound[nz+1]=interpol(csoundold,zold,[z[nz-1]])
    endif
    if (n_elements(omegaBVsqrograv) eq nz) then begin
      omegaBVsqrograv      =dblarr(nz+2L)
      omegaBVsqrograv[1:nz]=omegaBVsqrogravold
      omegaBVsqrograv[0]   =interpol(omegaBVsqrogravold,zold,[z[0]]   )
      omegaBVsqrograv[nz+1]=interpol(omegaBVsqrogravold,zold,[z[nz-1]])
    endif
    ;
    ind_zorg=ind_zorg+1L
    nz=n_elements(z)
    ;
  endif
  ;
  if ((gridopt eq 'linear') or (gridopt eq 'linear_b')) then begin
    ; --- New grid with constant spacing ---
    ;
    nzinterpol=long(nzinterpol0)
    ;
    zold=z
    z=z[0] + dindgen(nzinterpol)/double(nzinterpol-1L) * (z[nz-1L]-z[0])
    ;
    ind_zorg=interpol(lindgen(nzinterpol),z,zorg)
    nz=nzinterpol
    ;
  endif else if ((gridopt eq 'opt1') or (gridopt eq 'opt1_b') or $
                 (gridopt eq 'opt2') or (gridopt eq 'opt2_b')) then begin
    ; --- New optimized grid ---
    ;
    nzinterpol=long(nzinterpol0) > nz
    ;
    dz       =z[1:nz-1]-z[0:nz-2]
    dlogcs2dz=(alog(csound[1:nz-1]^2)-alog(csound[0:nz-2]^2))/dz
    RefFac   =abs(dlogcs2dz)
    if (avg(RefFac) eq 0.0D+00) then begin
      RefFac=RefFac*0.0D+00 + 1.0D+00
    endif
    RefFac=RefFac/avg(RefFac)/nz*nzinterpol
    if ((gridopt eq 'opt2') or (gridopt eq 'opt2_b')) then begin
      ; --- If there are enough points, force minimum refinement ---
      RefFac=RefFac > smooth(RefFac, 3, /edge_truncate)
      RefFac=RefFac > (double(nzinterpol)/(1.2D+00*double(nz)))
    endif
    RefFac=long(RefFac/avg(RefFac)/nz*nzinterpol + 1.0D-05) > 1L
    nzinterpol=long(total(RefFac)) + 1L
    ;
    zold=z
    z   =dblarr(nzinterpol)
    ind_zorg=lonarr(nz)
    z[0]=zold[0]
    iz  =0L
    for izold=1L,nz-1L do begin
      nitv           =RefFac[izold-1L]
      iznext         =iz+nitv
      ind_zorg[izold]=iznext
      z[iznext]      =zold[izold]
      if (nitv gt 1L) then begin
        ; --- Interval refinement by interpolation ---
        z[iz+1L:iznext-1L]=z[iz] + (dindgen(nitv-1L)+1.0D+00)/double(nitv+1L)*dz[izold-1L]
      endif
      iz=iznext
    endfor
    nz=nzinterpol
    ;
    if (strmid(gridopt,1,/reverse) eq '_b') then begin
      ind_zorg=ind_zorg[1:n_elements(ind_zorg)-2]
    endif
    ;
  endif else begin
    message, 'Grid optimization scheme not known: ' + gridopt
  endelse
endelse
if ((gridopt ne 'no') and (n_elements(csound) gt 1L)) then begin
  csoundold=csound
  csound   =interpol(csoundold, zold, z)
endif
if ((gridopt ne 'no') and (n_elements(omegaBVsqrograv) gt 1L)) then begin
  omegaBVsqrogravold=omegaBVsqrograv
  omegaBVsqrograv   =interpol(omegaBVsqrogravold, zold, z)
endif

; --- Horizontal extent ---
if (n_elements(xtotal0) eq 0) then begin
  xtotal=max(z)-min(z)
endif else begin
  xtotal=double(xtotal0)
endelse
if (n_elements(xtotal) gt 1) then begin
  message, 'Only scalar values for variable "xtotal" allowed.'
endif

; --- List of horizontal wavenumbers to be considered ---
if (n_elements(hwavenumberlist0) eq 0) then begin
  if (n_elements(nhwavenumber0) eq 0) then begin
    nhwavenumber=10L
  endif else begin
    nhwavenumber=long(nhwavenumber0)
  endelse
  hwavenumberlist=dindgen(nhwavenumber)
endif else begin
  hwavenumberlist=double(hwavenumberlist0)
  nhwavenumber=n_elements(hwavenumberlist)
endelse

; --- List of frequencies to be considered ---
if (n_elements(frequencylist0) eq 0) then begin
  if (n_elements(nfrequency0) eq 0) then begin
    nfrequency=1000L
  endif else begin
    nfrequency=long(nfrequency0)
  endelse
  frequencylist=0.005D+00*(max(csound)+min(csound))*0.5D+00/(max(z)-min(z)) * $
                (dindgen(nfrequency)+50.0D+00)
endif else begin
  frequencylist=double(frequencylist0)
  nfrequency=n_elements(frequencylist)
endelse

; --- Sign of omega^2 ---
if (n_elements(omegasqrsign0) eq 0) then begin
  omegasqrsign=1.0D+00
endif else begin
  if (omegasqrsign0 ge 0.0) then begin
    omegasqrsign= 1.0D+00
  endif else begin
    omegasqrsign=-1.0D+00
  endelse
endelse

; --- Boundary conditions ---
if (n_elements(boundlow0) eq 0) then begin
  boundlow='vz=0'
endif else begin
  boundlow=string(boundlow0)
endelse
if (n_elements(boundup0) eq 0) then begin
  boundup='vz=0'
endif else begin
  boundup=string(boundup0)
endelse

; --- Model name ---
if (n_elements(model0) eq 0) then begin
  model=''
endif else begin
  model=model0
endelse

; --- Set of independent variables ---
if (n_elements(VariableSet0) eq 0) then begin
  VariableSet='divv,vz'
endif else begin
  VariableSet=strlowcase(string(VariableSet0))
endelse

; --- Mode for solution of DEQ ---
if (n_elements(solveDEQmode0) eq 0) then begin
  solveDEQmode='pexp'
endif else begin
  solveDEQmode=string(solveDEQmode0)
endelse

; --- Accuracy of frequency iteration ---
;freqeps=max(frequencylist)*1.0D-14
;nitermax=30L
freqeps=max(frequencylist)*1.0D-17
nitermax=40L

help, solveDEQmode, nfrequency, frequencylist, omegasqrsign, nhwavenumber, hwavenumberlist

; === Initialization ===========================================================

dz =z[1:nz-1]-z[0:nz-2]
qup=dblarr(nhwavenumber, nfrequency)

; --- Density (integration from sound speed for constant Gamma1 and grav) ---
rho        =dblarr(nz)
rho[0]     =1.0D+00
oocs       =0.5D+00/csound[0:nz-2]^2 + 0.5D+00/csound[1:nz-1]^2
oocsint    =total(oocs*dz, /cumulative)
rho[1:nz-1]=rho[0] * (csound[0]/csound[1:nz-1])^2 * exp(-Gamma1*grav*oocsint)

; --- Weight ---
dzc        =dblarr(nz)
dzc[1:nz-2]=0.5D+00*(z[2:nz-1]-z[0:nz-3])
dzc[0]     =dz[0]
dzc[nz-1]  =dz[nz-2]
weight     =rho*dzc
weight     =weight/total(weight)

; --- Loop over all wavenumbers ---
for ihwave=0L, nhwavenumber-1L do begin
  khorizontal=hwavenumberlist[ihwave]*2.0D+00*!DPI/xtotal
  print, '  wavenumber ', ihwave, khorizontal
  ;
  ; --- Loop over all frequencies ---
  for ifreq=0,nfrequency-1L do begin
    omega=frequencylist[ifreq]*2.0D+00*!DPI
    ;print, '    omega ', ifreq, omega
    ;
    ; --- Integrate DEQ ---
    lawe_deqintegrate, VariableSet, strlowcase(solveDEQmode), boundlow, boundup, $
      nz, dz, omega, omegasqrsign, khorizontal, grav, Gamma1, csound, omegaBVsqrograv, $
      q, boundout, phase
    ;
    qup[ihwave,ifreq]=boundout
    ;
  endfor ; ifreq
  ;
endfor ; ihwave

; === Array with markers for change in sign: indication for eigenmode ==========
;qd=bytearr[nhwavenumber,nfrequency]
;for ihwave=0L, nhwavenumber-1L do begin
;  for ifreq=0,nfrequency-2L do begin
;    
;  endfor ; ifreq
;endfor ; ihwave
;qd                =(qup*shift(qup,0,-1)) lt 0.0
qd                =(signum(qup) ne shift(signum(qup),0,-1))
qd[*,0           ]=0B
qd[*,nfrequency-1]=0B
nmode             =long(total(float(qd)))
print, '  Estimated number of modes: ', nmode
mode_frequency  =dblarr(2L*nmode+9L)
mode_wavenumber =dblarr(2L*nmode+9L)
mode_q          =dblarr(2,nz,2L*nmode+9L)
;mode_qr         =dblarr(2,nz,2L*nmode+9L)
mode_phase      =dblarr(2L*nmode+9L)

; === Try to find all exact mode frequencies ===================================

imode=0L
iitertotal=0L
; --- Loop over all horizontal wavenumbers to get all modes ---
for ihwave=0L,nhwavenumber-1L do begin
  khorizontal=hwavenumberlist(ihwave)*2.0D+00*!DPI/xtotal
  ;
  ; --- Number of modes from points where qup changes sign ---
  qdwave=reform(qd[ihwave,*])
  nmodeforwave=total(float(qdwave))
  if (nmodeforwave gt 0L) then begin
    ind=where(qdwave eq 1L)
    freqllist=frequencylist[ind   ]
    freqrlist=frequencylist[ind+1L]
    qupllist =[reform(qup[ihwave, ind   ])]
    quprlist =[reform(qup[ihwave, ind+1L])]
  endif
  ;
  ; --- Check for local minima ---
  qupw= reform(qup[ihwave,*])
  lawe_qupminimum, nfrequency, frequencylist, qupw, 1.0D+00, $
    nmodeforwave, freqllist, freqrlist, qupllist, quprlist
  ;
  ; --- Check for local maxima ---
  qupw=-reform(qup[ihwave,*])
  lawe_qupminimum, nfrequency, frequencylist, qupw, -1.0D+00, $
    nmodeforwave, freqllist, freqrlist, qupllist, quprlist
  ;
  print, '  wavenumber ', ihwave, nmodeforwave
  ;
  if (nmodeforwave gt 0) then begin
    ;
    ; --- Loop over all mode candidates for current horizontal wavenumber ---
    for imodeforwave=0L,nmodeforwave-1L do begin
      ; --- Frequency interval that brackets mode (hopefully) ---
      freql=freqllist[imodeforwave]
      freqr=freqrlist[imodeforwave]
      qupl =qupllist[ imodeforwave]
      qupr =quprlist[ imodeforwave]
      ;
      freqc=freql
      ;
      iiter=0L
      while ((iiter lt nitermax) and $
             (abs(freqr-freql) gt freqeps) and $
             ((abs(qupl) > abs(qupr)) gt 0.0D+00)) do begin
        ;
        freqcold=freqc
        if ((abs(freqr-freql)/(abs(freqr)+abs(freql))) gt 1.0D-13) then begin
          ;freqc=freql - (freqr-freql)/(qupr-qupl) * qupl
          ;freqc=0.5D+00*(freql+freqr)
          ;freqc=0.5D+00 * (0.5D+00*(freql+freqr) + freql - (freqr-freql)/(qupr-qupl) * qupl)
          ;freqc=(1.0D+00-mu) * (0.5D+00*(freql+freqr)) + $
          ;               mu  * (freql - (freqr-freql)/(qupr-qupl) * qupl)
          mu=((abs(qupl) < abs(qupr)) / (abs(qupl) > abs(qupr)))^0.00001
          freqc=0.5D+00 * ((freql+freqr) - mu * (freqr-freql)/(qupr-qupl) * (qupl+qupr))
        endif else begin
          freqc=0.5D+00 * (freql+freqr)
        endelse
        ;
        if (freqc eq freqcold) then break
        ;
        omega=freqc*2.0D+00*!DPI
        ;
        ; --- Integrate DEQ ---
        lawe_deqintegrate, VariableSet, strlowcase(solveDEQmode), boundlow, boundup, $
          nz, dz, omega, omegasqrsign, khorizontal, grav, Gamma1, csound, omegaBVsqrograv, $
          q, qupc, phase
        ;
        print, imodeforwave, iiter, freql, freqc, freqr, qupl, qupc, qupr, max(abs(q)), $
           format='(4X,I3,1X,I2,1X,3E24.17,1X,3E12.3,1X,E12.3)'
        ;if (imode eq 10) then stop
        ;
        ;if ((qupc*qupl) ge 0.0D+00) then begin
        if (((qupc ge 0.0D+00) and (qupl ge 0.0D+00)) or $
            ((qupc le 0.0D+00) and (qupl le 0.0D+00))) then begin
          qupl =qupc
          freql=freqc
        endif else begin
          qupr =qupc
          freqr=freqc
        endelse
        iiter=iiter+1L
        ;
      endwhile
      ;
      ;print
      iitertotal=iitertotal+iiter
      ;
      ; --- Compute "reverse" solution starting at the top ---
      lawe_deqintegrate, VariableSet, strlowcase(solveDEQmode), boundlow, boundup, $
        nz, dz, omega, omegasqrsign, khorizontal, grav, Gamma1, csound, omegaBVsqrograv, $
        qr, qupc, phaseback, /LastBackward
      ;
      ; --- Correlate both solutions in interior ---
      nz1=nz*2L/5L
      nz2=nz/2L
      nz3=nz*3L/5L

      ; --- Preliminary normalization ---
print, max(abs(q)), max(abs(q[ *,nz1:nz3])), max(abs(qr)), max(abs(qr[*,nz1:nz3]))
      q =q /max(abs(q[ *,nz1:nz3] ))
      qr=qr/max(abs(qr[*,nz1:nz3]))
      ;
      qfac =sqrt(avg(q[ *,nz1:nz3]^2))
      qrfac=sqrt(avg(qr[*,nz1:nz3]^2))
      qqrcorr=avg(q[*,nz1:nz3]*qr[*,nz1:nz3]) / (qfac*qrfac)
print, qfac, qrfac, qqrcorr
      ;
      ; --- Check if correlation is high ---
      if (abs(qqrcorr) gt 0.999D+00) then begin
        ; --- Normalize normal and reverse solution and merge them ---
        q =q /qfac
        if (qqrcorr gt 0.0D+00) then begin
          qr= qr/qrfac
        endif else begin
          qr=-qr/qrfac
        endelse
        q[*,nz2:*]=qr[*,nz2:*]
print,max(abs(q))
        ;
        ; --- Normalize q: maximum amplitude=1 ---
        qmax=max(q[1,*], min=qmin)
        if (abs(qmax) gt abs(qmin)) then begin
          q=q/qmax
        endif else begin
          q=q/qmin
        endelse
        ;
        ; --- Save result in array ---
        mode_frequency[ imode]=freqc
        mode_wavenumber[imode]=hwavenumberlist[ihwave]
        mode_q[     *,*,imode]=q
        mode_phase[     imode]=phase
        ;
        imode=imode+1L
      endif
      ;
    endfor ; imodeforwave
  endif
endfor ; ihwave

print, '  Total number of iterations: ', iitertotal
print, '  nmode, imode: ', nmode, imode

nmode=imode
if (nmode ge 1) then begin
  ; --- Shrink arrays to include only proper modes ---
  mode_frequency =mode_frequency[ 0:nmode-1]
  mode_wavenumber=mode_wavenumber[0:nmode-1]
  mode_q         =mode_q[     *,*,0:nmode-1]
  ;mode_qr        =mode_qr[   *,*,0:nmode-1]
  mode_phase     =mode_phase[     0:nmode-1]

; --- Derive quantities ---
omegasqr=omegasqrsign * (2.0D+00*!DPI*mode_frequency)^2
k       =mode_wavenumber*2.0D+00*!DPI/xtotal
if (VariableSet eq 'divv,vz') then begin
  divv    =reform(mode_q[0,*,*])
  vz      =reform(mode_q[1,*,*])
  ; --- Horizontal velocity ---
  vx      =dblarr(nz,nmode)
  for imode=0L, nmode-1L do begin
    vx[*,imode]=k[imode]/omegasqr[imode] * (csound^2*divv[*,imode] - grav*vz[*,imode])
  endfor
endif else if (VariableSet eq 'vx,vz') then begin
  vx      =reform(mode_q[0,*,*])
  vz      =reform(mode_q[1,*,*])
  ; --- Divergence ---
  divv    =dblarr(nz,nmode)
  for imode=0L, nmode-1L do begin
    if (k[imode] eq 0.0) then begin
      divv[*,imode]=grav/csound^2*vz[*,imode]
    endif else begin
      divv[*,imode]=omegasqr[imode]/(k[imode]*csound^2)*vx[*,imode] + grav/csound^2*vz[*,imode]
    endelse
  endfor
endif else begin
  message, 'Unknown VariableSet ' + VariableSet
endelse

; --- Normalize ---
for imode=0L, nmode-1L do begin
  w            =sqrt(total(weight*(vx[*,imode]^2+vz[*,imode]^2)))
  vx[*,imode]  =vx[  *,imode]/w
  vz[*,imode]  =vz[  *,imode]/w
  divv[*,imode]=divv[*,imode]/w
endfor

; --- Check orthogonality ---
aoc=dblarr(nmode,nmode)
for imode=0L, nmode-1L do begin
  for jmode=0L, nmode-1L do begin
    if (mode_wavenumber[imode] eq mode_wavenumber[jmode]) then begin
      aoc[imode,jmode]=total(weight*(vx[*,imode]*vx[*,jmode] + vz[*,imode]*vz[*,jmode]))
    endif
  endfor ; jmode
endfor ; imode

; --- Number of nodes ---
nnode=lonarr(nmode)
for imode=0L, nmode-1L do begin
  ind         =where((vz[0:nz-2,imode]*vz[1:nz-2,imode]) lt 0.0D+00, n)
  nnode[imode]=n
endfor ; imode

mode={ $
    nmode:         nmode, $
    frequency:     mode_frequency, $
    omegasqr:      omegasqr, $
    hwavenumber:   mode_wavenumber, $
    k:             k, $
    phase:         mode_phase, $
    nnode:         nnode, $
    q:             mode_q, $
    ;qr:            mode_qr, $
    vx:            vx, $
    vz:            vz, $
    divv:          divv, $
    aoc:           aoc $
     }

endif else begin
  mode={ $
    nmode:         nmode $
       }
endelse


; === Put everything together into output structure ============================
dat={ $
  version:         '2006-06-28_01', $
  model:           model, $
  grav:            grav, $
  xtotal:          xtotal, $
  omegasqrsign:    omegasqrsign, $
  ind_zorg:        ind_zorg, $
  nz:              nz, $
  z:               z, $
  csound:          csound, $
  omegaBVsqrograv: omegaBVsqrograv, $
  rho:             rho, $
  dzc:             dzc, $
  weight:          weight, $
  Gamma1:          Gamma1, $
  boundlow:        boundlow, $
  boundup:         boundup, $
  solveDEQmode:    solveDEQmode, $
  VariableSet:     VariableSet, $
  gridopt:         gridopt, $
  nhwavenumber:    nhwavenumber, $
  hwavenumber:     hwavenumberlist, $
  nfrequency:      nfrequency, $
  frequency:       frequencylist, $
  qup:             qup, $
  qd:              qd, $
  mode:            mode $
    }

end ; lawe
