function cmopta, P, T, iband, opta=opta, grid=grid, clip=clip, cubic=cubic
;------------------------------------------------------------------------------
;+
; NAME:
;   cmopta ('compute_opacity_table')
;
; PURPOSE:
;   Interpolate opacities from data in input structure.
;
; CATEGORY:
;   Opacities, Interpolation
;
; CALLING SEQUENCE:
;   kappa=xkaros(P, T, iband, opta=opta, grid=grid, clip=clip)
;
; INPUTS:
;   P:     ('pressure') real array with pressure data
;   T:     ('Temperature') real array with temperature data
;   iband: ('index_band') integer (always 0)
;
; OUTPUTS:
;   cmopta: ('compute_opacity_table') real array with opacity data (per mass)
;
; KEYWORDS:
;   grid:  ('grid') Optional. If set compute a 2D grid for each combination of input P and T 1D vectors
;   clip:  ('clip') Optional. If not set in case of extrapolation the outermost values are used (0 or n-1)
;                   if set the values 1 gridpoint away from the boundary are used (1 or n-2)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Only linear interpolation is used!
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;            Written by Hans-Guenter Ludwig
;   1999-06-30 (B.F. Copenhagen) keyword clip added
;   1999-10-11 (B.F.) Return data in same format as T
;   2014-06-03 (B.F. Uppsala) Derived from xkaros
;-
;------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr

; --- Determine index coordinates ---
if (keyword_set(cubic)) then begin
  iT=interpol(indgen(opta.nT), opta.tabT, alog10(T), /QUADRATIC)
  iP=interpol(indgen(opta.nP), opta.tabP, alog10(P), /QUADRATIC)
endif else begin
  iT=interpol(indgen(opta.nT), opta.tabT, alog10(T))
  iP=interpol(indgen(opta.nP), opta.tabP, alog10(P))
endelse

; --- Delimit coordinates ---
if (keyword_set(clip)) then begin
  iT=1.0 > iT < float(opta.nT-2)
  iP=1.0 > iP < float(opta.nP-2)
endif

; --- Compute opacities ---
if (not keyword_set(grid)) then begin
  ; --- Compute opacities for all input P/T combinations ---
  xka   =T
  xka[*]=10.0^interpolate(opta.xopta[*,*,iband], iT, iP, CUBIC=cubic)
  ;
endif else begin
  ; --- Compute opacities for a rectangular grid ---
  xka   =fltarr(n_elements(iT), n_elements(iP))
  xka[*]=10.0^interpolate(opta.xopta[*,*,iband], iT, iP, CUBIC=cubic, /GRID)
  ;
endelse

return, xka

end
