pro opac_cluster_CreateOpTa, Tt=Tt, Pt=Pt, rhot=rhot, kappanu=kappanu, $
                             Result=Result, $
                             Opa=Opa
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_cluster_CreateOpTa ('opacity_cluster_Create_Optacity_Table')
;
; PURPOSE:
;   Assemble the standard opacity-table structure from the output of opac_cluster.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   opac_cluster_CreateOpTa, Tt=Tt, Pt=Pt, rhot=rhot, kappanu=kappanu, $
;                            Result=Result, $
;                            Opa=Opa
;
; KEYWORDS (INPUT):
;   --- Raw opacities ---
;   Tt:           ('Temperature_table') 1D real vector (nT elements) with temperature axis
;                 of opacity data block
;   Pt:           ('Pressure_table') 1D real vector (nP elements) with pressure axis
;                 of opacity data block
;   rhot:         ('rho_table') 2D real array (nTxnP elements) with density values
;                 corresponding to T/P axis values
;   kappanu:      ('kappa_nu') 3D real array (nTxnPxnnu) with frequency-dependent opacities (large)
;                 of the form kappa(T,P,nu) as function of temperature, pressure, and wavelength
;   --- Results from opac_cluster ---
;   Result:       ('Result') structure with collected results including the cluster-membership info
;
; KEYWORDS (OUTPUT):
;   Opa:          ('Opacity') structure with opacity data in OPTA format
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   opac_cluster_Mean:     Compute clustered/binned opacity means
;
; SIDE EFFECTS:
;   Global variables controlling plots are changed by opac_cluster_MultiPlot.
;
; RESTRICTIONS:
;   This routine more or less requires a previous run of opac_cluster.
;
; PROCEDURE:
;   Compute the overall grey Rosseland opacities and the cluster/bin opacities with the
;   requested mean.
;   Put everything into the structure in the standard opta format.
;
; MODIFICATION HISTORY:
;   2015-08-24 (B.F. Uppsala) Extracted from opac_cluster.pro and put into own routine
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

; --- Define abbreviations for array sizes ---
nTt =n_elements(Tt )
nPt =n_elements(Pt )

; --- Check for structure with raw-opacity meta data ---
MetaRawTags=tag_names(Result.MetaRaw)

; --- Check for structure with atmosphere meta data ---
MetaAtmTags=tag_names(Result.MetaAtm)


; === Create the structure for the binned-(clustered)-opacity table ================================

nu =Result.All.nu
dnu=Result.All.dnu
nnu=n_elements(nu)

; --- Determine which mean to use for full opacity tables ---
ModeMeanOut=Result.Par.ModeMean
ModeMeanLC =strlowcase(ModeMeanOut)
if ((ModeMeanLC eq strlowcase('IumId')) or $
    (ModeMeanLC eq strlowcase('IupId')) or $
    (ModeMeanLC eq strlowcase('J'    )) or $
    (ModeMeanLC eq strlowcase('IumB' )) or $
    (ModeMeanLC eq strlowcase('BmId' ))     ) then begin
  ; --- The chosen mean depends on the intensity field which is not available for the full tables ---
  ; --- Choose a save alternative                                                                 ---
  ModeMeanOut='RP'
  message, /informational, 'WARNING: Use ModeMean= "' + ModeMeanOut + $
                           '" for table instead of "' + Result.Par.ModeMean + '"'
endif

; --- Compute the full grey Rosseland opacity tables ---
opac_cluster_Mean, ModeMean='R', $
                   Tt=Tt, Pt=Pt, rhot=rhot, nu=nu, dnu=dnu, kappanu=kappanu, $
                   kCluster=1, $
                   kappatCluster=kappatRosseland, BreltCluster=BreltRosseland

; --- Compute the full opacity tables for the clusters ---
opac_cluster_Mean, ModeMean=ModeMeanOut, xLength=Result.Par.xLength, CDeltaT=Result.Par.CDeltaT, $
                   Tt=Tt, Pt=Pt, rhot=rhot, nu=nu, dnu=dnu, kappanu=kappanu, $
                   kCluster=Result.Par.kCluster, IndCluster=Result.Cluster.Ind, $
                   kappatCluster=kappatCluster, BreltCluster=BreltCluster

xOpTa=[ [[float(alog10(kappatRosseland))]], [[float(alog10(kappatCluster))]] ]
WTbbd=[  [float(       BreltRosseland  )] ,  [float(       BreltCluster  )]  ]

; --- Find an integer ID for the output mean ---
IdMeanList=['P', 'Psf', 'R', 'RPsf', 'RP', 'RmRP', 'BmB', 'Mix1', 'Mix2']
Ind=where(ModeMeanLC eq strlowcase(IdMeanList), n)
if (n gt 0) then begin
  ; --- The name of the mean has been found in the list ---
  IdMean=50+Ind[0]
endif else begin
  ; --- The name of the mean has not been found in the list ---
  IdMean=0
endelse

; --- Put together the opacity structure (without header, so far) ---
Opa={ $
  IdVers: 5, $
  OpTaNa: Result.Status.OutputID, $
  DateX:  Result.Status.DateTime, $
  IdMean: IdMean, $
  nT:     nTt, $
  nTBn:   2, $
  nP:     nPt, $
  nPBn:   2, $
  nBand:  Result.Par.kCluster, $
  mStep:  1, $  
  nuLo:   1, $
  nuHi:   nnu, $
  numnu:  nnu, $
  Intmdx: 2, $
  ;
  TabT:   alog10(Tt), $
  TabTBn: alog10([min(Tt), max(Tt)]), $
  IdxTBn: [1, nTt], $
  TabdTB: [float((alog10(max(Tt))-alog10(min(Tt)))/double(nTt-1))], $
  ;
  TabP:   alog10(Pt), $
  TabPBn: alog10([min(Pt), max(Pt)]), $
  IdxPBn: [1, nPt], $
  TabdPB: [float((alog10(max(Pt))-alog10(min(Pt)))/double(nPt-1))], $
  ;
  xOpTa:  xOpTa, $
  ;
  WTbbd:  WTbbd, $
  ;
  InBand: Result.Cluster.Ind+1 $
    }

; --- Handle information about detailed abundances ---
nAbu=n_elements(Result.MetaRaw.ElementAbundances)
if (nAbu gt 2) then begin
  ; --- Construct string array with detailed abundance data ---
  nList=3 + (nAbu-1)/7
  AbuList=strarr(nList)
  ;
  iList=0
  AbuList[iList]='*     Detailed Abundances:'
  ;
  iList=1
  AbuStr='*    '
  for iAbu=0,nAbu-1 do begin
    AbuStr=AbuStr + ' ' $
                  + string(strupcase(Result.MetaRaw.ElementSymbols[iAbu]), format='(A2)') $
                  + string(Result.MetaRaw.ElementAbundances[iAbu], format='(F7.3)')
    if (((iAbu mod 7) eq 6) or (iAbu eq (nAbu -1))) then begin
      AbuList[iList]=AbuStr
      iList=iList+1
      AbuStr='*    '
    endif
  endfor ; iAbu
  ;
  iList=nList-1
  AbuList[iList]='*'
  ;
endif else begin
  AbuList=['*']
  ;
endelse

; --- Construct nice output strings for effective temperature and log(gravity) ---
if ((where(MetaAtmTags eq strupcase('Teffout')))[0] ne -1) then begin
  TeffOutString=strtrim(string(round(Result.MetaAtm.Teffout)              , format='(I8)'), 2)
endif else begin
  TeffOutString=''
endelse

if ((where(MetaAtmTags eq strupcase('grav')))[0] ne -1) then begin
  logg        =alog10(Result.MetaAtm.grav)
  loggwPString=string(logg, format='(F6.3)')
endif else begin
  loggwPString=''
endelse


; --- Assemble the header ---
Header=[ $
;  '************************************************************************', $
  '*', $
  '*     Opacity table name: ' + Opa.OpTaNa + '.opta', $
  '*     Created           : ' + Opa.DateX, $ 
  '*     Contents          : grey Rosseland opacities', $
  '*                            + ' + Result.Par.ModeMean + ' band means', $
  '*', $
  '*     Opacity source data', $
  '*     -------------------', $
  '*     Frequency set  identification: ' + string(Opa.numnu, format='(I7)'), $
;  '*     Frequency ID:' + string(Opa.numnu, format='(I7)'), $
  '*     numnu       =' + string(Opa.numnu, format='(I7)'), $
  '*     nulo        =' + string(Opa.nuLo , format='(I7)'), $
  '*     nuhi        =' + string(Opa.nuHi , format='(I7)'), $
  '*     IFOP        = ' + Result.MetaRaw.Code + ' opacity tables', $
  '*     Code        : ' + Result.MetaRaw.Code, $
  '*     Code version: ' + Result.MetaRaw.CodeVersion, $
  '*     Abundance data identification: ' + Result.MetaRaw.AbundanceID, $
  '*     File           identification: ' + Result.MetaRaw.FileOriginal, $
;  '*     File      ID: ' + Result.MetaRaw.FileOriginal, $
;  '*     Abundance ID: ' + Result.MetaRaw.AbundanceID, $
  '*     x (mass fraction)   = ' + string(Result.MetaRaw.X     , format='(F8.6)'), $
  '*     y (mass fraction)   = ' + string(Result.MetaRaw.Y     , format='(F8.6)'), $
  '*     z (mass fraction)   = ' + string(Result.MetaRaw.Z     , format='(F8.6)'), $
  '*     Corresponding to:', $
  '*     x (number fraction) = ' + string(Result.MetaRaw.abuX  , format='(F8.6)'), $
  '*     y (number fraction) = ' + string(Result.MetaRaw.abuY  , format='(F8.6)'), $
  '*     z (number fraction) = ' + string(Result.MetaRaw.abuZ  , format='(F8.6)'), $
  '*     qmol                = ' + string(Result.MetaRaw.Qmol  , format='(F8.6)'), $
  '*     vmicro              = ' + string(Result.MetaRaw.Vmicro, format='(F8.6)') + ' km/s', $
  '*     Author      : ' + Result.MetaRaw.Author, $
  AbuList, $
  '*     Reference atmosphere', $
  '*     --------------------', $
  '*     Code        : ' + Result.MetaAtm.Code, $
  '*     File      ID: ' + Result.MetaAtm.FileSource, $
  '*     Model     ID: ' + Result.MetaAtm.Model, $
  '*     Teff (out)  : ' + TeffoutString + ' K', $
  '*     logg        : ' + loggwPString, $
  '*     Author      : ' + Result.MetaAtm.Author, $
  '*', $
  '*     Band definition parameters (cgs units)', $
  '*     -------------------------------------', $
  '*     nband       = ' + string(Result.Par.kCluster      , format='(I3)'   ), $
  '*     Code        : ' +        Result.Par.Program       , $
  '*     Code version: ' +        Result.Par.Version       , $
  '*     ModeMean    : ' +        ModeMeanOut              , $
  '*     ModeInit    : ' +        Result.Par.ModeInit      , $
  '*     ModexLength : ' +        Result.Par.ModexLength   , $
  '*     ModeSmall   : ' +        Result.Par.ModeSmall     , $
  '*     xLengthPar  = ' + string(Result.Par.xLengthPar    , format='(E13.6)'), $
  '*     xLength     = ' + string(Result.Par.xLength       , format='(E13.6)') + ' cm', $
  '*     CDeltaT     = ' + string(Result.Par.CDeltaT       , format='(E13.6)'), $
  '*     ExprhoWeight= ' + string(Result.Par.ExprhoWeight  , format='(E13.6)'), $
  '*     mBot        = ' + string(Result.Par.mBot          , format='(I3)'   ), $
  '*     nTop        = ' + string(Result.Par.nTop          , format='(I3)'   ), $
  '*     nnuStepInit = ' + string(Result.Par.nnuStepInitial, format='(I3)'   ), $
  '*     ntauMask    = ' + string(Result.Par.ntauMask      , format='(I3)'   ), $
  '*     CMaskSmall  = ' + string(Result.Par.CMaskSmall    , format='(E13.6)'), $
  '*     mChunk      = ' + string(Result.Par.mChunk        , format='(I3)'   ), $
  '*     nChunk      = ' + string(Result.Par.nChunk        , format='(I3)'   ), $
  '*     Author      : ' +        Result.Par.Author        , $
  '*', $
  '*     P-T domain definition parameters (log cgs units)', $
  '*     ------------------------------------------------', $
  '*     ntbn        =' +         string(Opa.nTBn  , format='(I3)'  ), $
  '*     TABTBN      =' + strjoin(string(Opa.TabTBn, format='(F8.3)')), $
  '*     TABDTB      =' + strjoin(string(opa.TabdTB, format='(F8.3)')), $
  '*     npbn        =' +         string(Opa.nPBn  , format='(I3)'  ), $
  '*     TABPBN      =' + strjoin(string(Opa.TabPBn, format='(F8.3)')), $
  '*     TABDPB      =' + strjoin(string(opa.TabdPB, format='(F8.3)')), $
  '*', $
  '*     Recommended CUBIT interpolation mode', $
  '*     ------------------------------------', $
  '*     intmdx      = ' + string(Opa.Intmdx, format='(I3)'), $
  '*' $
;  '************************************************************************' $
       ]

; --- Merge header and existing structure ---
Opa=create_struct('Header', Header, Opa)

;opa={ $
;  header: header, $
;  idvers: idvers, $
;  optana: optana, $
;  datex:  datex, $
;  idmean: idmean, $
;  nt:     nt, $
;  ntbn:   ntbn, $
;  np:     np, $
;  npbn:   npbn, $
;  nband:  nband, $
;  mstep:  mstep, $  
;  nulo:   nulo, $
;  nuhi:   nuhi, $
;  numnu:  numnu, $
;  intmdx: intmdx, $
;  ;
;  tabt:   topta, $
;  tabtbn: tabtbn, $
;  idxtbn: idxtbn, $
;  tabdtb: tabdtb, $
;  ;
;  tabp:   popta, $
;  tabpbn: tabpbn, $
;  idxpbn: idxpbn, $
;  tabdpb: tabdpb, $
;  ;
;  xopta:  xopta, $
;  ;
;  wtbbd:  wtbbd, $
;  ;
;  inband: inband $
;    }

end ; opac_cluster_CreateOpTa
