pro opac_cluster_Init, Mode=Mode0, Par=Par0, $
                       nnuStep=nnuStep0, $
                       nu=nu, dnu=dnu, IIntuAllTop=IIntuAllTop, tauIntRef=tauIntRef, $
                       Indcluster=Indcluster, nnuCluster=nnuCluster, IndnuCluster=IndnuCluster
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_cluster_Init ('opacity_cluster_Initialize')
;
; PURPOSE:
;   Initialize the cluster membership.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   opac_cluster_Init, Mode=Mode, Par=Par, $
;                      nnuStep=nnuStep, $
;                      nu=nu, dnu=dnu, IIntuAllTop=IIntuAllTop, tauIntRef=tauIntRef, $
;                      Indcluster=Indcluster, nnuCluster=nnuCluster, IndnuCluster=IndnuCluster
;
; INPUT:
;  None (just via keywords)
;
; OUTPUT:
;  None (just via keywords)
;
; KEYWORDS (INPUT):
;   Mode:         ('Mode') string with name of initial clustering pattern
;                   'FrequencySplitEvenBlock' : Split the frequency interval into blocks with
;                                               even number of points
;                   'FrequencySplitRoundRobin': Split the frequency interval in a
;                                               round-robin fashion
;                   'tauSplitEvenBlock'       : Sort the clusters according to their
;                                               reference tau value into blocks with about
;                                               even number of points
;                   'tauSplitBin'             : Sort the clusters according to their
;                                               reference tau value (under construction)
;   Par:          ('Parameter') not used, yet.
;   nnuStep:      ('number_nu_Step') integer with frequency-axis coarsening factor,
;                 typically a power of two.
;   nu:           ('nu') 1D real vector with frequency axis of opacity data block
;   dnu:          ('delta_nu') optional 1D real vector (nnu elements) with frequency intervals.
;                 Not used, yet.
;   IIntuAllTop:  Not used, yet.
;   tauIntRef:    ('tau_Integration_Reference') 1D vector (nnu elements) of reference optical depth
;                 used for sorting in one of the 'tau...' modes.
;
; KEYWORDS (OUTPUT):
;   IndCluster:   ('Index_Cluster') 1D integer vector (nnu) with the cluster
;                 each frequency points belongs to
;   nnuCluster:   ('number_nu_Cluster') 1D integer vector (kCluster) with the number of
;                 frequency points in each cluster
;   IndnuCluster: ('Index_nu_Cluster) 2D integer array (nnu, kCluster) with the
;                 frequency points in each cluster
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2015-07-10 (B.F. Uppsala) First version
;   2015-07-30 (B.F.) Add parameter nnuStep to allow for coarse sampling
;   2015-08-10 (B.F.) Use only frequency points with (IndCluster eq -2)
;   2015-08-22 (B.F.) Create first version of 'tauSplitBin' mode
;   2015-09-06 (B.F.) Remove 'Default' initialization branch
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

; --- Define abbreviations for array sizes ---
nnu     =n_elements(IndCluster)
kCluster=n_elements(nnuCluster)

; --- Get typical length scale ---
if (keyword_set(Mode0)) then begin
  Mode=string(Mode0)
endif else begin
  Mode='tauSplitEvenBlock'
endelse
ModeLC=strlowcase(Mode)

; --- Get step size on frequency axis ---
if (keyword_set(nnuStep0)) then begin
  nnuStep=long(nnuStep0) > 1
endif else begin
  nnuStep=1
endelse

; --- Determine the number of candidates to be put into the active pool ---
IndCand =where(IndCluster eq -2, nIndCand)
IndCand =IndCand[0:*:nnuStep]
nIndCand=n_elements(IndCand)

; === Fill cluster-membership arrays ===============================================================

if (kCluster eq 1) then begin
  ; --- Only one cluster requested: put all frequency points into it ---
  ;
help, IndCluster, nnuCluster, nnu, IndnuCluster
  IndCluster[  *   ]=0
  nnuCluster[     0]=nnu
  IndnuCluster[*, 0]=lindgen(nnu)
  ;
endif else if (kCluster ge nnu) then begin
  ; --- There is at least one cluster per frequency point: distribute them ---
  ;
  IndCluster[  *         ]=lindgen(nnu)
  nnuCluster[           *]=0
  nnuCluster[  0:nnu-1   ]=1
  IndnuCluster[*      , *]=-1
  IndnuCluster[0:nnu-1, 0]=lindgen(nnu)
  ;
endif else if (ModeLC eq strlowcase('FrequencySplitEvenBlock')) then begin
  ; --- Split on frequency axis ---
  ;
  ;IndCluster[*]=long(findgen(nnu)/float(nnu)*float(kCluster))
  ;IndCluster[0:*:nnuStep]=long(findgen(((nnu-1)/nnuStep)+1)/float(((nnu-1)/nnuStep)+1)*float(kCluster))
  ;IndCluster[IndCand[0:*:nnuStep]]=long(findgen(((nIndCand-1)/nnuStep)+1) / $
  ;                                       double(((nIndCand-1)/nnuStep)+1) * double(kCluster))
  IndCluster[IndCand]=long(dindgen(nIndCand) / double(nIndCand) * double(kCluster))
  ;
  ; --- Update the other frequency-index arrays ---
  for iCluster=0,kCluster-1 do begin
    Ind                          =where(IndCluster eq iCluster, n)
    IndnuCluster[0:n-1, iCluster]=Ind
    nnuCluster[         iCluster]=n
  endfor ; iCluster
  ;
endif else if (ModeLC eq strlowcase('FrequencySplitRoundRobin')) then begin
  ; --- Split on frequency axis ---
  ;
  ;IndCluster[0:*:nnuStep]=lindgen(((nnu-1)/nnuStep)+1) mod kCluster
  ;IndCluster[IndCand[0:*:nnuStep]]=lindgen(((nIndCand-1)/nnuStep)+1) mod kCluster
  IndCluster[IndCand]=lindgen(nIndCand) mod kCluster
  ;
  ; --- Update the other frequency-index arrays ---
  for iCluster=0,kCluster-1 do begin
    Ind                          =where(IndCluster eq iCluster, n)
    IndnuCluster[0:n-1, iCluster]=Ind
    nnuCluster[         iCluster]=n
  endfor ; iCluster
  ;
endif else if (ModeLC eq strlowcase('tauSplitEvenBlock' )) then begin
  ; --- Split according to reference optical depth into blocks with same number of points ---
  ;
  ; --- Sort frequency points according to reference optical depth scale ---
  ;Ind=reverse(sort(tauIntRef[0:*:nnuStep]))*nnuStep
  ;Ind=IndCand[reverse(sort(tauIntRef[IndCand[0:*:nnuStep]]))*nnuStep]
  Ind=IndCand[reverse(sort(tauIntRef[IndCand]))]
  ;
  for iCluster=0,kCluster-1 do begin
    ;m                             =(nnu *  iCluster   ) / kCluster
    ;n                             =(nnu * (iCluster+1)) / kCluster - 1
    ;m                              =((((nnu-1)/nnuStep)+1) *  iCluster   ) / kCluster
    ;n                              =((((nnu-1)/nnuStep)+1) * (iCluster+1)) / kCluster - 1
    ;m                              =((((nIndCand-1)/nnuStep)+1) *  iCluster   ) / kCluster
    ;n                              =((((nIndCand-1)/nnuStep)+1) * (iCluster+1)) / kCluster - 1
    m                              =(nIndCand *  iCluster   ) / kCluster
    n                              =(nIndCand * (iCluster+1)) / kCluster - 1
    IndCluster[         Ind[m:n]]=iCluster
    IndnuCluster[0:n-m, iCluster]=Ind[m:n]
    nnuCluster[         iCluster]=n-m+1
  endfor ; iCluster
  ;
endif else if (ModeLC eq strlowcase('tauSplitBin' )) then begin
  ; --- Split according to reference optical depth into prescribed bins ---
  ;
  logtauIntRefCand=alog10(tauIntRef[IndCand])
  logtauRefMin    =min(logtauIntRefCand, max=logtauRefMax)
  help, logtauRefMin, logtauRefMax
  logtauRefMin=logtauRefMin > (-3.0)
  ;
  ; --- Define bins of equal size in log(reference optical depth) ---
  logtauBin    =logtauRefMax - (dindgen(kCluster+1)/double(kCluster)) * (logtauRefMax-logtauRefMin)
  logtauBin[ 0]=logtauRefMax + 50.0
  logtauBin[-1]=logtauRefMin - 50.0
  print, 'logtauBin=', logtauBin
  ;
  ; --- Sort the frequency points into the tau bins ---
  for iCluster=0,kCluster-1 do begin
    Ind=where((logtauIntRefCand le logtauBin[iCluster  ]) and $
              (logtauIntRefCand gt logtauBin[iCluster+1])      , n)
    ;
    if (n gt 0) then begin
      Ind=IndCand[Ind]
      ;
      IndCluster[  Ind            ]=iCluster
      IndnuCluster[0:n-1, iCluster]=Ind
      nnuCluster[         iCluster]=n
    endif else begin
      message, 'Empty bin created'
    endelse
  endfor ; iCluster
  ;
endif else begin
  message, 'Unknown cluster initialization mode'
  ;
endelse

end ; opac_cluster_Init
