pro opac_cluster_Mean, ModeMean=ModeMean0, xLength=xLength, CDeltaT=CDeltaT0, $
                       Tt=Tt, Pt=Pt, rhot=rhot, nu=nu, dnu=dnu, kappanu=kappanu, $
                       kCluster=kCluster, IndCluster=IndCluster, $
                       kappatCluster=kappatCluster, BreltCluster=BreltCluster
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_cluster_Mean ('opacity_cluster_Mean')
;
; PURPOSE:
;   Compute clustered/binned opacity means.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;
; INPUT:
;   None (just via keywords)
;
; OUTPUT:
;   None (just via keywords)
;
; KEYWORDS (INPUT):
;   Tt:           ('Temperature_table') 1D real vector with temperature axis of opacity data block
;   Pt:           ('Pressure_table') 1D real vector with pressure axis of opacity data block
;   rhot:         ('rho_table') 2D real array with density values corresponding to T/P axis values
;   nu:           ('nu') 1D real vector with frequency axis of opacity data block
;   dnu:          ('delta_nu') 1D real vector with intervals for all frequency points
;   kappanu:      ('kappa_nu') 3D real array with frequency-dependent opacities (large)
;                 of the form kappa(T,P,nu) as function of temperature, pressure, and wavelength
;
; KEYWORDS (OUTPUT):
;   kappatCluster: ('kappa_table_Cluster') double-precision 3D array with 2D opacity table for
;                 each cluster
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   BnuPlanck:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2015-08-16 (B.F. Uppsala) First version
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

; --- Define abbreviations for array sizes ---
nTt =n_elements(Tt)
nPt =n_elements(Pt)
nnu =n_elements(nu)
nX1D=nTt

; --- Get mode of opacity mean ---
ModeMean='RP'
if (keyword_set(ModeMean0)) then begin
  ModeMean=string(ModeMean0)
endif
print, 'ModeMean=                 ', ModeMean, format='(A,A)'
ModeMeanLC=strlowcase(ModeMean)

; --- Get temperature fraction ---
CDeltaT=0.2D+00
if (keyword_set(CDeltaT0)) then begin
  CDeltaT=double(CDeltaT0)
endif
print, 'CDeltaT=                  ', CDeltaT, format='(A,E13.6)'

; --- Construct the trivial cluster-membership array for kCluster=1 if necessary ---
if ((not keyword_set(IndCluster)) and (kCluster eq 1)) then begin
  IndCluster=lonarr(nnu)
endif

; --- Construct the other frequency-index arrays ---
nnuCluster  =lonarr(     kCluster)
IndnuCluster=lonarr(nnu, kCluster)
for iCluster=0,kCluster-1 do begin
  Ind                          =where(IndCluster eq iCluster, n)
  IndnuCluster[0:n-1, iCluster]=Ind
  nnuCluster[         iCluster]=n
endfor ; iCluster


; === Compute ======================================================================================

; --- Allocate and create arrays (remember: nx1D=nTt) ---
T1D          =Tt
B1D          =dblarr(nx1D,      nnu     )
dBdT1D       =dblarr(nx1D,      nnu     )
BreltCluster =dblarr(nTt ,      kCluster)
kappatCluster=dblarr(nTt , nPt, kCluster)

; --- Compute Planck function and its temperature derivative ---
for inu=0,nnu-1 do begin
  B1D[   *, inu]=BnuPlanck(   nu[inu], T1D)
  dBdT1D[*, inu]=dBdTnuPlanck(nu[inu], T1D)
  ;
endfor ; inu

; --- Loop over all pressure points to save memory ---
for iPt=0,nPt-1 do begin
  ; --- Use temperature, pressure, and opacity slice ---
  rho1D     =reform(rhot[   *, iPt   ])
  kappa1D   =reform(kappanu[*, iPt, *])
  ;
  ; --- Compute the opacity weights ---
  opac_com_Mean, ModeMean=ModeMean, xLength=xLength, CDeltaT=CDeltaT, $
                 nu=nu, dnu=dnu, $
                 rho1D=rho1D, T1D=T1D, B1D=B1D, dBdT1D=dBdT1D, kappa1D=kappa1D, $
                 wHat1v=wHat1, wHat1kappa=wHat1kappa, wHat2v=wHat2, wHat2kappa=wHat2kappa, $
                 DualwHat_flag=DualwHat_flag
  ;
  for iCluster=0,kCluster-1 do begin
    ; --- Get the elements for the current cluster ---
    nnuCurClus  =nnuCluster[                  iCluster]
    IndnuCurClus=IndnuCluster[0:nnuCurClus-1, iCluster]
    ;
    if (nnuCurClus gt 0) then begin
      ; --- Compute the actual opacities from the opacity weights for each non-empty cluster ---
      wHat1kappaCurClus=total(wHat1kappa[*, IndnuCurClus], 2)
      wHat1CurClus     =total(wHat1[     *, IndnuCurClus], 2)
      ;
      if (DualwHat_flag eq 0) then begin
        kappatCluster[*, iPt, iCluster]=wHat1kappaCurClus/ $
                                        wHat1CurClus
      endif else begin ; (DualwHat_flag)
        wHat2kappaCurClus=total(wHat2kappa[*, IndnuCurClus], 2)
        wHat2CurClus     =total(wHat2[     *, IndnuCurClus], 2)
        ;
        kappatCluster[*, iPt, iCluster]=2.0D+00 / (wHat1CurClus     / $
                                                   wHat1kappaCurClus   + $
                                                   wHat2CurClus     / $
                                                   wHat2kappaCurClus      )
      endelse ; (DualwHat_flag)
      ;
      if (iPt eq 0) then begin
        ; --- Compute the sum of the Planck-function contributions for each cluster ---
        dnuAr        =rebin(reform(dnu[IndnuCurClus], 1, nnuCurClus), nx1D, nnuCurClus)
        BreltCluster[*, iCluster]=total(B1D[*, IndnuCurClus] * dnuAr, 2)
      endif ; (iPt)
      ;
    endif else  begin ; (nnuCurClus)
      ; --- This cluster has no members: set the opacities and Planck-function weights to zero ---
      kappatCluster[*, iPt, iCluster]=0.0D+00
      ;
      if (iPt eq 0) then begin
        BreltCluster[*, iCluster]=0.0D+00
      endif ; (iPt)
    endelse ; (nnuCurClus)
    ;
  endfor ; iCluster
  ;
endfor ; iPt

; --- Normalize the Planck-function weights ---
if (kCluster ge 2) then begin
  BreltCluster=BreltCluster / rebin(reform(total(BreltCluster, 2), nTt, 1), nTt, kCluster)
endif else begin
  BreltCluster[*]=1.0D+00
endelse

end ; opac_cluster_Mean
