pro opac_com, Pt=Pt, Tt=Tt, rhot=rhot, nu=nu, mask=mask, $
              kappaPlan=kappaPlan, kappaRoss=kappaRoss, kappanu=kappanu, $
              x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
              xLength=xLength0, $
              strat=strat
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_com ('opacity_compute')
;
; PURPOSE:
;   Compute intensities and mean opacities along 1D stratification.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   opac_com, Pt=Pt, Tt=Tt, rhot=rhot, nu=nu, mask=mask, $
;             kappaPlan=kappaPlan, kappaRoss=kappaRoss, kappanu=kappanu, $
;             x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
;             xLength=xLength, $
;             strat=strat
;
; INPUT:
;  None (just via keywords)
;
; OUTPUT:
;  None (just via keywords)
;
; KEYWORDS (INPUT):
;   Tt:           ('Temperature_table') 1D real vector with temperature axis of opacity data block
;   Pt:           ('Pressure_table') 1D real vector with pressure axis of opacity data block
;   rhot:         ('rho_table') 2D real array with density values corresponding to T/P axis values
;   nu:           ('nu') 1D real vector with frequency axis of opacity data block
;   mask:         ('mask') optional 1D real vector masking the frequency vector
;   kappaPlan:    ('kappa_Planck') 2D real array with Planck opacities (optional)
;   kappanu:      ('kappa_nu') 3D real array with frequency-dependent opacities (large)
;                 of the form kappa(T,P,nu) as function of temperature, pressure, and wavelength
;   x1D:          ('x_1_Dimension') 1D real vector with x coordinates of points in stratification
;   T1D:          ('Temperature_1_Dimension') 1D real vector with temperature values of stratification
;   P1D:          ('Pressure_1_Dimension') 1D real vector with pressure values of stratification
;   rho1D:        ('rho_1_Dimension') 1D real vector with density values of stratification
;   xLength:      ('x_Length') scalar real with typical length scale
;
; KEYWORDS (OUTPUT):
;   strat:        ('stratification') composed structure with output data
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   BnuPlanck:
;   dBdTnuPlanck:
;   opac_com_Iintegrate: Integrate intensity
;   opac_com_tau:        Integrate optical-depth scale
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2014-05-07 (B.F. Uppsala) First version
;   2014-05-14 (B.F.) Compute dual-bin opacity
;   2014-05-28 (B.F.) Add some more opacity averages
;   2014-06-04 (B.F.) Woitke-Split averages
;   2015-06-30 (B.F.) More means
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===============================================================

; --- Get initial time ---
time0=systime(/SECONDS)

; --- Compute additional axes ---
lambda=!con.clight/nu
dnu   =deriv(double(nu))
if (keyword_set(mask)) then begin
  dnu =dnu*mask
endif
;WaveNumber=1.0/lambda

; --- Define abbreviations for array sizes ---
nTt =n_elements(Tt)
nPt =n_elements(Pt)
nnu =n_elements(nu)
nx1D=n_elements(x1D)
if ((nx1D ne n_elements(P1D)) or $
    (nx1D ne n_elements(T1D))     ) then begin
  message, 'Number of elements of 1D stratification arrays do not agree.'
endif

; --- Get typical length scale ---
if (keyword_set(xLength0)) then begin
  xLength=abs(double(xLength0))
endif else begin
  xLength=0.1D+00*(max(x1D)-min(x1D))
endelse


; === Interpolate data onto input 1D T/P stratification ========================

opac_com_StratInterpolate, $
  time0=time0, $
  Pt=Pt, Tt=Tt, rhot=rhot, nu=nu, $
  kappanu=kappanu, $
  x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
  ilogT1D=ilogT1D, ilogP1D=ilogP1D, $
  kappa1D=kappa1D, B1D=B1D, dBdT1D=dBdT1D

; --- Interpolate input Planck mean ---
if (keyword_set(kappaPlan)) then begin
  print, '% opac_com: ', systime(/SECONDS)-time0, ' s, kappa_Planck', format='(A,F8.2,A)'
  kappaPlanIn=10.0^float(interpolate(double(alog10(kappaPlan)), ilogT1D, ilogP1D, cubic=-0.5))
endif else begin
  ; --- Input Planck mean is not specified: set output to zero ---
  kappaPlanIn=0.0
endelse

; --- Interpolate input Rosseland mean ---
if (not keyword_set(kappaRoss)) then begin
  ; --- Input Rosseland mean is not specified: compute it from frequency-dependent opacities ---
  ;
  print, '% opac_com: ', systime(/SECONDS)-time0, ' s, kappa_Rosseland', format='(A,F8.2,A)'
  kappaRoss=dblarr(nTt, nPt)
  for iT=0,nTt-1 do begin
    dBdTdnu   =rebin(reform(dBdTnuPlanck(nu, double(Tt[iT]))*dnu, 1, nnu), nPt, nnu)
    dBdTdnuTot=total(dBdTdnu, 2)
    kappaRoss[iT, *]=dBdTdnuTot/total(dBdTdnu/reform(kappanu[iT,*,*]), 2)
  endfor ; iT
endif
kappaRossIn=10.0^float(interpolate(double(alog10(kappaRoss)), ilogT1D, ilogP1D, cubic=-0.5))


; === Compute simple opacity means =============================================

print, '% opac_com: ', systime(/SECONDS)-time0, ' s, simple means', format='(A,F8.2,A)'

; --- Expand frequency axis, 1D density vector, 1D temperature vector to 2 dimensions ---
dnuAr=       rebin(reform(dnu,1,nnu), nx1D,nnu)
rhoAr=double(rebin(reform(rho1D, nx1D,1), nx1D,nnu))
TAr  =double(rebin(reform(T1D  , nx1D,1), nx1D,nnu))

; --- Compute Rosseland mean for 1D stratification ---
dBdTTot      =total(dBdT1D*                dnuAr, 2)
dBdTkappaTot =total(dBdT1D/double(kappa1D)*dnuAr, 2)
kappaRoss1D  =float(dBdTTot/dBdTkappaTot)

; --- Compute Planck mean for 1D stratification ---
BTot         =total(B1D*                dnuAr, 2)
BkappaTot    =total(B1D*double(kappa1D)*dnuAr, 2)
kappaPlan1D  =float(BkappaTot/BTot)

; --- Compute small-fluctuation Planck mean for 1D stratification ---
dBdTkappaTot =total(dBdT1D*double(kappa1D)*dnuAr, 2)
kappaPlSm1D  =float(dBdTkappaTot/dBdTTot)

; --- Compute Woitke-Split opacities from normal Rosseland and Planck mean ---
kappaSqrt1D  =sqrt(kappaPlan1D*(kappaPlan1D-kappaRoss1D))
kappaRossWo1D=kappaPlan1D-kappaSqrt1D
kappaPlanWo1D=kappaPlan1D+kappaSqrt1D

; --- Compute mix between Rosseland and Planck_small-fluct means ---
dtauxL      =xLength*rhoAr*double(kappa1D)
expdtauxL   =exp(-dtauxL)
wHat        =dBdT1D * (1.0D+00-expdtauxL)/double(kappa1D)
w           =wHat/rebin(reform(total(wHat*dnuAr, 2), nx1D,1), nx1D,nnu)
kappaRPsf31D=total(w*double(kappa1D)*dnuAr, 2)

; --- Compute mix between Rosseland and Planck means ---
wHat      =B1D*expdtauxL - $
           dBdT1D * 0.2*TAr * ((1.0D+00+dtauxL)*expdtauxL-1.0D+00)/dtauxL
w         =wHat/rebin(reform(total(wHat*dnuAr, 2), nx1D,1), nx1D,nnu)
kappaRP11D=total(w*double(kappa1D)*dnuAr, 2)

; --- Compute opacity from difference of two Planck functions ---
Bp1D      =dblarr(nx1D, nnu)
for inu=0,nnu-1 do begin
  Bp1D[*,inu]=BnuPlanck(nu[inu], reform(T1D * (1.0D+00 + 0.1D+00 * alog10(1.0D+00 + 1.0D+00/dtauxL[*,inu]))))
endfor ; inu
wHat      =Bp1D-B1D
w         =wHat/rebin(reform(total(wHat*dnuAr, 2), nx1D,1), nx1D,nnu)
kappaBmB1D=total(w*double(kappa1D)*dnuAr, 2)

; --- Compute Rosseland average (harmonic mean) between opacities ---
kappaMix1D=2.0/(1.0/kappaRP11D + 1.0/kappaBmB1D)


; === Integrate optical depth scale ============================================

print, '% opac_com: ', systime(/SECONDS)-time0, ' s, tau integration', format='(A,F8.2,A)'

; --- Integrate optical depth scale and compute additional quntities ---
opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappa1D, $
              dxInt=dxInt, dtauInt=dtauInt, tauInt=tauInt, tauTopnu=tauTopnu, HrhoTop=HrhoTop
dx1D=2.0D+00*rebin(dxInt, nx1D)


; === Integrate intensity for all frequency points =============================

; --- Compute corresponding intensities ---
opac_com_Iintegrate, time0=time0, $
                     x1D=x1D, rho1D=rho1D, B1D=B1D, $
                     dtauInt=dtauInt, tauTopnu=tauTopnu, $
                     dIIntu=dIIntu, IIntu=IIntu, dIIntd=dIIntd, IIntd=IIntd, BInt=BInt

; --- Integrate over frequency ---
dnuAr    =rebin(reform(dnu, 1,nnu), 2*nx1D  ,nnu)
dIIntuTot=total(dIIntu*dnuAr, 2)
dIIntdTot=total(dIIntd*dnuAr, 2)
dnuAr   =rebin(reform(dnu, 1,nnu), 2*nx1D+1,nnu)
IIntuTot=total( IIntu*dnuAr, 2)
IIntdTot=total( IIntd*dnuAr, 2)
BIntTot =total( BInt *dnuAr, 2)

; --- Compute "J", "I_up-S", and "I_down-S" means for 1D stratification ---
dnuAr      =rebin(reform(dnu, 1,nnu), nx1D,nnu)
;JInt       =IIntu[1:*:2,*]+IIntd[1:*:2,*]
;JIntTot    =total(JInt*dnuAr, 2)
;kappaIupId =total(kappa1D*JInt *dnuAr, 2)/JIntTot
IwInt      =IIntu[1:*:2,*]+IIntd[1:*:2,*]
kappaIupId =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)
IwInt      =IIntu[1:*:2,*]-B1D
kappaImSu  =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)
IwInt      =IIntd[1:*:2,*]-B1D
kappaImSd  =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)
IwInt      =IIntu[1:*:2,*]
kappaIu    =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)
IwInt      =IIntd[1:*:2,*]
kappaId    =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)
IwInt      =IIntu[1:*:2,*]-IIntd[1:*:2,*]
kappaIumId =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)
IwInt      =IIntu[1:*:2,*]+IIntd[1:*:2,*]+4.0D+00*B1D
kappaJ     =total(kappa1D*IwInt*dnuAr, 2)/total(IwInt*dnuAr, 2)


; === Compute pair of up-/downward opacities ===================================

print, '% opac_com: ', systime(/SECONDS)-time0, ' s, iterate up/down ', format='(A,F8.2,A)'

; --- Iterate to get upward opacity ---
nIter=30
rhoInt   =rebin(rho1D, 2*nx1D)
dtauuIter=dblarr(2*nx1D, nIter)
dtauuthin= dIIntuTot/(-IIntuTot[0:-2]+0.5*(BIntTot[0:-2]+BIntTot[1:-1]))
dtauuIter[*,0]=dtauuthin
for i=0,nIter-2 do begin
  ;  dtauuIter[*,i+1]=-alog((dIIntuTot - $
  ;                      (BIntTot[1:-1]-BIntTot[0:-2]) * (exp(-dtauuIter[*,i])-1.0D+00+dtauuIter[*,i])/dtauuIter[*,i] $
  ;                     )/ $
  ;                     (IIntuTot[0:-2]-BIntTot[0:-2]) + $
  ;                     1.0D+00)
  FIter =-dIIntuTot + $
          (IIntuTot[0:-2]-BIntTot[0:-2]) * (exp(-dtauuIter[*,i])-1.0D+00) + $
          (BIntTot[ 1:-1]-BIntTot[0:-2]) * (exp(-dtauuIter[*,i])-1.0D+00+dtauuIter[*,i])/dtauuIter[*,i]
  dFIter=-(IIntuTot[0:-2]-BIntTot[0:-2]) *                        exp(-dtauuIter[*,i]) + $
          (BIntTot[ 1:-1]-BIntTot[0:-2]) * (-(dtauuIter[*,i]+1.0D+00)*exp(-dtauuIter[*,i])+1.0D+00)/dtauuIter[*,i]^2
  dtauuIter[*,i+1]=dtauuIter[*,i] - FIter/dFIter
  ;print, 'u=', max(abs((dtauuIter[*,i+1]-dtauuIter[*,i])/dtauuIter[*,i]))
endfor ; i
dtauuFin=reform(dtauuIter[*,nIter-1])
dtauu   =rebin(dtauuFin               , nx1D)
kappau  =rebin(dtauuFin/(rhoInt*dxInt), nx1D)

; --- Iterate to get downward opacity ---
dtaudIter=dblarr(2*nx1D, nIter)
dtaudthin= dIIntdTot/(-IIntdTot[1:-1]+0.5*(BIntTot[0:-2]+BIntTot[1:-1]))
dtaudIter[*,0]=dtaudthin
for i=0,nIter-2 do begin
  FIter = -dIIntdTot + $
         (IIntdTot[1:-1]-BIntTot[1:-1]) * (exp(-dtaudIter[*,i])-1.0D+00) + $
         (BIntTot[ 0:-2]-BIntTot[1:-1]) * (exp(-dtaudIter[*,i])-1.0D+00+dtaudIter[*,i])/dtaudIter[*,i]
  dFIter=-(IIntdTot[1:-1]-BIntTot[1:-1]) *                        exp(-dtaudIter[*,i]) + $
         ( BIntTot[ 0:-2]-BIntTot[1:-1]) * (-(dtaudIter[*,i]+1.0D+00)*exp(-dtaudIter[*,i])+1.0D+00)/dtaudIter[*,i]^2
  dtaudIter[*,i+1]=dtaudIter[*,i] - FIter/dFIter
  ;print, 'd=', max(abs((dtaudIter[*,i+1]-dtaudIter[*,i])/dtaudIter[*,i]))
endfor ; i
dtaudFin=reform(dtaudIter[*,nIter-1])
dtaud   =rebin(dtaudFin               , nx1D)
kappad  =rebin(dtaudFin/(rhoInt*dxInt), nx1D)

; --- Compute Rosseland average (harmonic mean) between up and down opacities ---
kappaRossud  =2.0/(1.0/kappau + 1.0/kappad)

; --- Compute Woitke-Split opacities from up and down means ---
kappaSqrt1D  =sqrt(kappad*abs(kappad-kappau))
kappaRossWoud=kappad-kappaSqrt1D
kappaPlanWoud=kappad+kappaSqrt1D


; === Compute dual-bin opacities ===============================================

print, '% opac_com: ', systime(/SECONDS)-time0, ' s, iterate 1/2 ', format='(A,F8.2,A)'

; --- Allocate arrays ---
nBin   =2
nBinIt =10
wBin          =dblarr(nBin)
;kappaBin      =dlbarr(nBin, 2*nx1D)
dtauBin       =dblarr(nBin, 2*nx1D  , nBinIt)
expl2tmdtauBin=dblarr(nBin, 2*nx1D          )
BBin          =dblarr(nBin, 2*nx1D+1        )
dIBinu        =dblarr(nBin, 2*nx1D  , nBinIt)
IBinu         =dblarr(nBin, 2*nx1D+1, nBinIt)
dIBind        =dblarr(nBin, 2*nx1D  , nBinIt)
IBind         =dblarr(nBin, 2*nx1D+1, nBinIt)

; --- Set weights for bins ---
wBin[0]=0.8D+00
wBin[1]=1.0D+00-wBin[0]
wBinAr =rebin(reform(double(wBin), nBin, 1), nBin, 2*nx1D  )
wBinAr1=rebin(reform(double(wBin), nBin, 1), nBin, 2*nx1D+1)

; --- Initialize iteration arrays ---
dtauBin[0,*,0]=dtauuFin
dtauBin[1,*,0]=dtaudFin
BBin[   0,*  ]=BIntTot
BBin[   1,*  ]=BIntTot
;dIBinu[ 0,*,0]=dIIntuTot
;dIBinu[ 1,*,0]=dIIntuTot
IBinu[  0,*,0]=IIntuTot
IBinu[  1,*,0]=IIntuTot
;dIBind[ 0,*,0]=dIIntdTot
;dIBind[ 1,*,0]=dIIntdTot
IBind[  0,*,0]=IIntdTot
IBind[  1,*,0]=IIntdTot

nIterSub =10
dtauuIterSub=dblarr(2*nx1D, nIterSub)
dtaudIterSub=dblarr(2*nx1D, nIterSub)

for iBinIt=0,nBinIt-2 do begin
  ;
  ; --- Compute exp(-dtau)-1+dtau ---
  expl2tmdtauBin=exp(-dtauBin[*,*,iBinIt])-1.0D+00+dtauBin[*,*,iBinIt]
  ;
  ; --- Integrate upward ---
  for i=0,2*nx1D-1 do begin
    dIBinu[*,i  ,iBinIt] =(IBinu[*,i  ,iBinIt]-BBin[*,i]) * (expl2tmdtauBin[*,i]-dtauBin[*,i,iBinIt]) + $
                          (BBin[ *,i+1       ]-BBin[*,i]) * (expl2tmdtauBin[*,i]/dtauBin[*,i,iBinIt])
    IBinu[ *,i+1,iBinIt] =IBinu[ *,i  ,iBinIt]+dIBinu[*,i,iBinIt]
  endfor ; i
  ;
  ; --- Compute new corrected intensities for upward bin ---
  ;FacStep=0.0D+00
  FacStep=1.0D+00
  ;FacStep=1.0/wBin[0]
  dIBinuSum=total(wBinAr *dIBinu[*,*,iBinIt], 1)
  FacStepThis=2.0*FacStep
  repeat begin
    FacStepThis=FacStepThis*0.5
    dIBinuNew=-(dIBinuSum-dIIntuTot)*wBin[0]*FacStepThis + dIBinu[0,*,iBinIt]
  endrep until ((max(dIBinuNew) lt 0.0D+00) or (FacStepThis lt 1.0D-06))
  IBinuSum =total(wBinAr1*IBinu[ *,*,iBinIt], 1)
  ;IBinuNew =-(IBinuSum -IIntuTot )*wBin[0]*FacStepThis + iBinu[ 0,*,iBinIt]
  IBinuNew =-(IBinuSum -IIntuTot )*FacStepThis + iBinu[ 0,*,iBinIt]
  ; --- Set boundary intensity for next iteration ---
  IBinu[0,*,iBinIt+1]=IBinuNew
  IBinu[1,*,iBinIt+1]=(IIntuTot-IBinuNew*wBin[0])/wBin[1]
  ;
  ; --- Compute new upward delta-taus ---
  dtauuIterSub[*,0]=dtauBin[0,*,iBinIt]
  for i=0,nIterSub-2 do begin
    FIter =-dIBinuNew + $
            (IBinuNew[0:-2]-BIntTot[0:-2]) * (exp(-dtauuIterSub[*,i])-1.0D+00) + $
            (BIntTot[ 1:-1]-BIntTot[0:-2]) * (exp(-dtauuIterSub[*,i])-1.0D+00+dtauuIterSub[*,i])/dtauuIterSub[*,i]
    dFIter=-(IBinuNew[0:-2]-BIntTot[0:-2]) *                        exp(-dtauuIterSub[*,i]) + $
            (BIntTot[ 1:-1]-BIntTot[0:-2]) * (-(dtauuIterSub[*,i]+1.0D+00)*exp(-dtauuIterSub[*,i])+1.0D+00)/dtauuIterSub[*,i]^2
    dtauuIterSub[*,i+1]=dtauuIterSub[*,i] - FIter/dFIter
    ;print, 'u=', iBinIt, i, FacStepThis, max(abs((dtauuIterSub[*,i+1]-dtauuIterSub[*,i])/dtauuIterSub[*,i])), $
    ;  format='(A,2I4,E9.2,1X,E13.6)'
  endfor ; i
  dtauBin[0,*,iBinIt+1]=dtauuIterSub[*,nIterSub-1] > (1.0D-01*dtauuFin)
  dtauBin[1,*,iBinIt+1]=dtauBin[1,*,iBinIt]
  ;
  ; --- Compute exp(-dtau)-1+dtau ---
  expl2tmdtauBin=exp(-dtauBin[*,*,iBinIt+1])-1.0D+00+dtauBin[*,*,iBinIt+1]
  ;
  ; --- Integrate downward ---
  for i=2*nx1D,1,-1 do begin
    dIBind[*,i-1,iBinIt] =(IBind[*,i  ,iBinIt]-BBin[*,i]) * (expl2tmdtauBin[*,i-1]-dtauBin[*,i-1,iBinIt+1]) + $
                          (BBin[*, i-1]-BBin[*,i]) * (expl2tmdtauBin[*,i-1]/dtauBin[*,i-1,iBinIt+1])
    IBind[ *,i-1,iBinIt] =IBind[*,i  ,iBinIt]+dIBind[*,i-1,iBinIt]
  endfor ; i
  ;
  ; --- Compute new corrected intensities for downward bin ---
  ;FacStep=0.0D+00
  FacStep=0.2D+00
  dIBindSum=total(wBinAr *dIBind[*,*,iBinIt], 1)
  FacStepThis=2.0*FacStep
  repeat begin
    FacStepThis=FacStepThis*0.5
    dIBindNew=-(dIBindSum-dIIntdTot)*wBin[1]*FacStepThis + dIBind[1,*,iBinIt]
  endrep until ((min(dIBindNew) gt 0.0D+00) or (FacStepThis lt 1.0D-06))
  IBindSum =total(wBinAr1*IBind[ *,*,iBinIt], 1)
  ;IBindNew =-(IBindSum -IIntdTot )*wBin[1]*FacStepThis + IBind[ 1,*,iBinIt]
  IBindNew =-(IBindSum -IIntdTot )*FacStepThis + IBind[ 1,*,iBinIt]
  tauTopBin=dtauBin[*,-1,iBinIt+1]/rebin(dxInt[-1:-1], nBinIt)*HrhoTop
  ; --- Set boundary intensity for next iteration ---
  ;IBind[*,-1,iBinIt+1]=IIntdtot[-1] * (1.0D+00-exp(-tauTopBin)) / total(wBinAr*(1.0D+00-exp(-tauTopBin)))
  IBind[*,-1,iBinIt+1]=IIntdtot[-1]
  ;
  ; --- Compute new downward delta-taus ---
  dtaudIterSub[*,0]=dtauBin[1,*,iBinIt+1]
  for i=0,nIterSub-2 do begin
    FIter = -dIBindNew + $
           (IBindNew[1:-1]-BIntTot[1:-1]) * (exp(-dtaudIterSub[*,i])-1.0D+00) + $
           (BIntTot[ 0:-2]-BIntTot[1:-1]) * (exp(-dtaudIterSub[*,i])-1.0D+00+dtaudIterSub[*,i])/dtaudIterSub[*,i]
    dFIter=-(IBindNew[1:-1]-BIntTot[1:-1]) *                        exp(-dtaudIterSub[*,i]) + $
           ( BIntTot[ 0:-2]-BIntTot[1:-1]) * (-(dtaudIterSub[*,i]+1.0D+00)*exp(-dtaudIterSub[*,i])+1.0D+00)/dtaudIterSub[*,i]^2
    dtaudIterSub[*,i+1]=dtaudIterSub[*,i] - FIter/dFIter
    ;print, 'd=', iBinIt, i, FacStepThis, max(abs((dtaudIterSub[*,i+1]-dtaudIterSub[*,i])/dtaudIterSub[*,i])), $
    ;  format='(A,2I4,E9.2,1X,E13.6)'
  endfor ; i
  dtauBin[1,*,iBinIt+1]=dtaudIterSub[*,nIterSub-1] < (1.0D+01*dtaudFin)
  ;
endfor ; iBinIt

kappaBin0=rebin(reform(dtauBin[0,*,nBinIt-1])/(rhoInt*dxInt), nx1D)
kappaBin1=rebin(reform(dtauBin[1,*,nBinIt-1])/(rhoInt*dxInt), nx1D)


; === Construct/derive some additional arrays ==================================

; --- Compose single array for all kappas ---
namesAll=['Planck', $
          'Planck_input', $
          'Planck_small-fluct', $
          'Planck_Woitke', $
          'Rosseland', $
          'Rosseland_input', $
          'Rosseland_Woitke', $
          'J', $
          'I-S_Up', $
          'I-S_Down', $
          'I_Up', $
          'I_Down', $
          'I_Up-I_Down', $
          'I_Up+I_Down', $
          'Up', $
          'Down', $
          'Rosseland_Up-Down', $
          'Planck_Woitke_Up-Down', $
          'Rosseland_Woitke_Up-Down', $
          'Bin_0', $
          'Bin_1', $
          'RossPlancksf3', $
          'RossPlanck1', $
          'BmB', $
          'Mix']
nAll    =n_elements(namesAll)
kappaAll=dblarr(nx1D, nAll)
i=-1
i=i+1 & kappaAll[*, i]=kappaPlan1D
i=i+1 & kappaAll[*, i]=kappaPlanIn
i=i+1 & kappaAll[*, i]=kappaPlSm1D
i=i+1 & kappaAll[*, i]=kappaPlanWo1D
i=i+1 & kappaAll[*, i]=kappaRoss1D
i=i+1 & kappaAll[*, i]=kappaRossIn
i=i+1 & kappaAll[*, i]=kappaRossWo1D
i=i+1 & kappaAll[*, i]=kappaJ
i=i+1 & kappaAll[*, i]=kappaImSu
i=i+1 & kappaAll[*, i]=kappaImSd
i=i+1 & kappaAll[*, i]=kappaIu
i=i+1 & kappaAll[*, i]=kappaId
i=i+1 & kappaAll[*, i]=kappaIumId
i=i+1 & kappaAll[*, i]=kappaIupId
i=i+1 & kappaAll[*, i]=kappau
i=i+1 & kappaAll[*, i]=kappad
i=i+1 & kappaAll[*, i]=kappaRossud
i=i+1 & kappaAll[*, i]=kappaPlanWoud
i=i+1 & kappaAll[*, i]=kappaRossWoud
i=i+1 & kappaAll[*, i]=kappaBin0
i=i+1 & kappaAll[*, i]=kappaBin1
i=i+1 & kappaAll[*, i]=kappaRPsf31D
i=i+1 & kappaAll[*, i]=kappaRP11D
i=i+1 & kappaAll[*, i]=kappaBmB1D
i=i+1 & kappaAll[*, i]=kappaMix1D

; --- Compute corresponding tau scales ---
opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappaAll, tau1D=tauAll, dtauInt=dtauIntAll, $
              tauTopnu=tauTopAll

; --- Compute corresponding intensities ---
opac_com_Iintegrate, time0=time0, $
                     x1D=x1D, rho1D=rho1D, B1D=BIntTot[1:*:2], $
                     dtauInt=dtauIntAll, tauTopnu=tauTopAll, $
                     dIIntu=dIIntuAll, IIntu=IIntuAll, dIIntd=dIIntdAll, IIntd=IIntdAll

; --- Compress arrays to normal grid ---
dIIntuAll=2.0D+00*rebin(dIIntuAll, nx1D, nAll)
dIIntdAll=2.0D+00*rebin(dIIntdAll, nx1D, nAll)
IIntuAll =IIntuAll[1:*:2,*]
IIntdAll =IIntdAll[1:*:2,*]


; === Compose structure with resulting arrays ==================================

print, '% opac_com: ', systime(/SECONDS)-time0, ' s, create structure ', format='(A,F8.2,A)'

s_in1D=create_struct( $
  'x'          , x1D, $
  'dx'         , dx1D, $
  'T'          , T1D, $
  'P'          , P1D, $
  'rho'        , rho1D)
;s_Rad=create_struct( $
;  'B'          , BTot, $
;  'J'          , JIntTot)
s_Iter=create_struct( $
  'dtauu'      , dtauuIter, $
  'dtaud'      , dtaudIter)
s_kappa=create_struct( $
  'Plan'    , kappaPlan1D, $
  'PlanIn'  , kappaPlanIn, $
  'PlanSm'  , kappaPlSm1D, $
  'PlanWo'  , kappaPlanWo1D, $
  'Ross'    , kappaRoss1D, $
  'RossIn'  , kappaRossIn, $
  'RossWo'  , kappaRossWo1D, $
  'J'       , kappaJ, $
  'ImSup'   , kappaImSu, $
  'ImSdown' , kappaImSd, $
  'IupmIdown', kappaIumId, $
  'IuppIdown', kappaIupId, $
  'up'      , kappau, $
  'down'    , kappad, $
  'Rossud'  , kappaRossud, $
  'PlanWoud', kappaPlanWoud, $
  'RossWoud', kappaRossWoud, $
  'Bin0'    , kappaBin0, $
  'Bin1'    , kappaBin1)
s_All=create_struct( $
  'names'   , namesAll, $
  'kappa'   , temporary(kappaAll), $
  'tau'     , temporary(tauAll  ), $
  'dIIntu'  , temporary(dIIntuAll), $
  'IIntu'   , temporary(IIntuAll), $
  'dIIntd'  , temporary(dIIntdAll), $
  'IIntd'   , temporary(IIntdAll))
s_bin=create_struct( $
  'dIu'     , transpose(dIBinu , [1,0,2]), $
  'Iu'      , transpose(IBinu  , [1,0,2]), $
  'dId'     , transpose(dIBind , [1,0,2]), $
  'Id'      , transpose(IBind  , [1,0,2]), $
  'dtau'    , transpose(dtauBin, [1,0,2]))
;
strat=create_struct( $
  'HrhoTop'    , HrhoTop, $
  'nu'         , nu, $
  'dnu'        , temporary(dnu), $
  'lambda'     , temporary(lambda), $
  'kappanu'    , temporary(kappa1D), $
  'kappaRoss'  , kappaRoss, $
;  'Bnu'        , temporary(B1D), $
;  'dBdTnu'     , temporary(dBdT1D), $
  'dxInt'      , temporary(dxInt), $
;  'dtauIntnu'  , temporary(dtauInt), $
  'tauIntnu'   , temporary(tauInt), $
  'rhoInt'     , temporary(rhoInt), $
;  'BIntnu'     , temporary(BInt), $
;  'BInt'       , temporary(BIntTot), $
;  'dIIntunu'   , temporary(dIIntu), $
;  'IIntunu'    , temporary(IIntu), $
  'dIIntu'     , dIIntuTot, $
  'IIntu'      , IIntuTot, $
;  'dtauu'      , dtauu, $
;  'dIIntdnu'   , dIIntd, $
;  'IIntdnu'    , IIntd, $
  'dIIntd'     , dIIntdTot, $
  'IIntd'      , IIntdTot, $
;  'dtaud'      , dtaud, $
  'in1D'       , temporary(s_in1D), $
;  'rad'        , temporary(s_Rad), $
  'Iter'       , temporary(s_Iter), $
  'kappa'      , temporary(s_kappa), $
  'all'        , temporary(s_All), $
  'bin'        , temporary(s_bin))

;  'in1D'       , s_in1D, $
;  'rad'        , s_Rad, $
;  'Iter'       , s_Iter, $
;  'kappa'      , s_kappa, $
;  'all'        , s_All, $
;  'bin'        , s_bin)


;print, '% opac_com: ', systime(/SECONDS)-time0, ' s, number of P/T points ', nPT, format='(A,F8.2,A,I6)'

print, '% opac_com: ', systime(/SECONDS)-time0, ' s, finished ----------------------------', format='(A,F8.2,A)'

end ; opac_com
