pro opac_com_Iintegrate, time0=time0, $
                         x1D=x1D, rho1D=rho1D, B1D=B1D, $
                         dtauInt=dtauInt, tauTopnu=tauTopnu, $
                         dIIntu=dIIntu, IIntu=IIntu, dIIntd=dIIntd, IIntd=IIntd, BInt=BInt
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_com_Iintegrate ('opacity_compute_Intensity_integrate')
;
; PURPOSE:
;   Integrate upward and downward intensities along 1D ray.
;
; CATEGORY:
;   Opacities, radiation transport
;
; CALLING SEQUENCE:
;   opac_com_Iintegrate, time0=time0, $
;                        x1D=x1D, rho1D=rho1D, B1D=B1D, $
;                        dtauInt=dtauInt, tauTopnu=tauTopnu, $
;                        dIIntu=dIIntu, IIntu=IIntu, dIIntd=dIIntd, IIntd=IIntd, BInt=BInt
;
; INPUT:
;   None (just via keywords)
;
; OUTPUT:
;   None (just via keywords)
;
; KEYWORDS (INPUT):
;   time0:        ('time_0') real scalar with systime at start of program
;   x1D:          ('x_1_Dimension') 1D real vector with x coordinates of points in stratification
;   rho1D:        ('rho_1_Dimension') 1D real vector with density values of stratification
;   B1D:          ('B_1_Dimension') 1D real vector or 2D real array with values of
;                 Planck function on stratification
;   dtauInt:      ('delta_tau_Integration') 2D double-precision array (2*nx1D,nnu)
;                 with delta optical depth
;   tauTopnu:     ('tau_Top_nu') 1D double-precision vector (nnu) with optical-depth estimate at top
;
; KEYWORDS (OUTPUT):
;   dIIntu:       ('delta_Intensity_Integration_up') 2D double-precision array (2*nx1D,nnu)
;                 with upward intensity change 
;   IIntu:        ('Intensity_Integration_up') 2D double-precision array (2*nx1D+1,nnu)
;                 with upward intensity
;   dIIntd:       ('delta_Intensity_Integration_down') 2D double-precision array (2*nx1D,nnu)
;                 with downward intensity change 
;   IIntd:        ('Intensity_Integration_down') 2D double-precision array (2*nx1D+1,nnu)
;                 with downward intensity
;   BInt:         ('B_Integration') 2D double-precision array (2*nx1D+1,nnu)
;                 with Planck function
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2014-05-28 (B.F. Uppsala) Put into separate routine
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr

; === Initialize ===================================================================================

; --- Define abbreviations for array sizes ---
nnu =n_elements(dtauInt[0,*])
nx1D=n_elements(x1D)


; === Compute ======================================================================================

;print, '% opac_com_Iintegrate: ', systime(/SECONDS)-time0, ' s, integration: init', $
;       format='(A,F8.2,A)'

; --- Allocate arrays for finer axis ---
;dIIntu      =dblarr(2*nx1D  , nnu)
dIIntu      =dblarr(nnu     , 2*nx1D)
IIntu       =dblarr(2*nx1D+1, nnu)
;dIIntd      =dblarr(2*nx1D  , nnu)
dIIntd      =dblarr(nnu     ,2*nx1D)
IIntd       =dblarr(2*nx1D+1, nnu)
BInt        =dblarr(2*nx1D+1, nnu)

; --- Interpolate source function ---
if (size(B1D, /N_DIMENSIONS) eq 2) then begin
  B1D0=B1D
endif else begin
  B1D0=rebin(reform(B1D, nx1D, 1), nx1D, nnu)
endelse
BInt[1:-2:2,*]=B1D0
BInt[2:-3:2,*]=0.5D+00*(B1D0[0:-2,*]+B1D0[1:*,*])
BInt[ 0    ,*]=2.0D+00*BInt[ 1,*]-BInt[ 2,*]
BInt[-1    ,*]=BInt[-2,*]
B1D0          =0

; --- Compute additional array for radiation transport ---
expl2tmdtauInt=exp(-dtauInt)-1.0D+00+dtauInt
ind=where(dtauInt lt 1.0D-06, n)
if (n gt 0) then expl2tmdtauInt[ind]=0.5D+00*dtauInt[ind]^2

; --- Set incoming intensity at bottom boundary: use diffusion approximation ---
IIntu[ 0,*]=BInt[0,*] + (BInt[0,*]-BInt[1,*])/dtauInt[0,*]

; --- Set incoming intensity at top boundary:                  ---
; --- Assume constant Bnu but exponentially decreasing density ---
IIntd[-1,*]=B1D[-1,*]*(1.0D+00-exp(-tauTopnu))

; --- Transpose arrays ---
;dIIntu        =transpose(dIIntu )
IIntu         =transpose(IIntu  )
;dIIntd        =transpose(dIIntd )
IIntd         =transpose(IIntd  )
BInt          =transpose(BInt   )
dtauInt       =transpose(dtauInt)
expl2tmdtauInt=transpose(expl2tmdtauInt)

;print, '% opac_com_Iintegrate: ', systime(/SECONDS)-time0, ' s, integration: upward', $
;       format='(A,F8.2,A)'

; --- Integrate upward ---
for i=0,2*nx1D-1 do begin
  dIIntu[*,i  ] =(IIntu[*,i  ]-BInt[*,i]) * (expl2tmdtauInt[*,i]-dtauInt[*,i]) + $
                 (BInt[*, i+1]-BInt[*,i]) * (expl2tmdtauInt[*,i]/dtauInt[*,i])
  IIntu[ *,i+1] =IIntu[*,i  ]+dIIntu[*,i]
endfor ; i

;print, '% opac_com_Iintegrate: ', systime(/SECONDS)-time0, ' s, integration: downward', $
;       format='(A,F8.2,A)'

; --- Integrate downward ---
for i=2*nx1D,1,-1 do begin
  dIIntd[*,i-1] =(IIntd[*,i  ]-BInt[*,i]) * (expl2tmdtauInt[*,i-1]-dtauInt[*,i-1]) + $
                 (BInt[*, i-1]-BInt[*,i]) * (expl2tmdtauInt[*,i-1]/dtauInt[*,i-1])
  IIntd[ *,i-1] =IIntd[*,i  ]+dIIntd[*,i-1]
endfor ; i

;print, '% opac_com_Iintegrate: ', systime(/SECONDS)-time0, ' s, integration: finalize', $
;       format='(A,F8.2,A)'

; --- Transpose arrays ---
dIIntu        =transpose(dIIntu )
IIntu         =transpose(IIntu  )
dIIntd        =transpose(dIIntd )
IIntd         =transpose(IIntd  )
BInt          =transpose(BInt   )
dtauInt       =transpose(dtauInt)

; --- Compress arrays to normal grid ---
;dIIntu=2.0D+00*rebin(dIIntu, nx1D, nnu)
;dIIntd=2.0D+00*rebin(dIIntd, nx1D, nnu)
;IIntu =IIntu[1:*:2,*]
;IIntd =IIntd[1:*:2,*]

end ; opac_com_Iintegrate
