pro opac_com_Mean, ModeMean=ModeMean0, xLength=xLength, CDeltaT=CDeltaT0, $
                   nu=nu, dnu=dnu, $
                   rho1D=rho1D, T1D=T1D, B1D=B1D, dBdT1D=dBdT1D, kappa1D=kappa1D, $
                   IIntu=IIntu, IIntd=IIntd, $
                   wHat1v=wHat1, wHat1kappa=wHat1kappa, wHat2v=wHat2, wHat2kappa=wHat2kappa, $
                   DualwHat_flag=DualwHat_flag
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_com_Mean ('opacity_compute_Mean')
;
; PURPOSE:
;   Compute the weights necessary to get opacity means.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;
; INPUT:
;   None (just via keywords)
;
; OUTPUT:
;   None (just via keywords)
;
; KEYWORDS (INPUT):
;   Mode:         ('Mode') string with type of opacity mean.
;                 The value is used in opac_com_mean.pro where the complete list can be found.
;                 Only 'proper' local means can be used to produce the final opacity table 'opa'.
;                 'Improper' detailed-intensity-dependent means can only be used for the sorting.
;                 Examples:
;                   --- 'Proper' local means ---
;                   'P'   :  Planck mean
;                   'Psf' :  small-fluctuation Planck mean
;                   'R'   :  Rosseland mean
;                   'RP'  :  Blend of Rosseland and Planck mean (default) (xLength, CDeltaT)
;                   'RPsf':  Blend of Rosseland and small-fluctuation Planck mean (xLength)
;                   'BmB' :  Sort-of non-local Rosseland mean with weight B(far)-B(at_point)
;                            (xLength, CDeltaT)
;                   'Mix1':  Mix of RP and BmB mean (xLength, CDeltaT)
;                   'Mix2':  Mix of RP and BmB mean (xLength, CDeltaT)
;                   --- 'Improper' means using the intensity in the reference atmosphere ---
;                   'IumId': Flux-like (Chandrasekhar-type) mean with weight I_up-I_down
;                   'IupId': J-like mean with weight I_up+I_down
;                   'J'    : J-like mean with weight from discrete angles I_up+I_down+4B
;                   'IumB' : Upward-intensity-change mean with weight I_up-B
;                   'BmId' : Downward-intensity-change mean with weight B-I_down
;   xLength:      ('x_Length') scalar real with typical length scale (in cm).
;   CDeltaT:      ('Constant_Delta_Temperature') scalar real with temperature fraction.
;                   0.2: Default
;   nu:           ('nu') 1D real vector (nnu elements) with frequency axis of opacity data block
;   dnu:          ('delta_nu') optional 1D real vector (nnu elements) with frequency intervals
;                 of opacity data block. The default is to compute this vector from nu.
;   rho1D:        ('rho_1_Dimension') 1D real vector with density values of stratification
;   T1D:          ('Temperature_1_Dimension') 1D real vector with temperature values of stratification
;   B1D:          ('B_1_Dimension') 1D real vector or 2D real array with values of
;                 Planck function on stratification
;   dBdT1D:       ('delta_B_delta_Temperature_1_Dimension') 1D real vector or 2D real array with values of
;                 temperature derivative of Planck function on stratification
;   kappa1D:      ('kappa_1_Dimension') 2D double-precision array with
;                 frequency-dependent opacity values (per mass unit) of stratification
;   IIntu:        ('Intensity_Integration_up') 2D double-precision array (nx1D, nnu)
;                 with upward intensity
;   IIntd:        ('Intensity_Integration_down') 2D double-precision array (nx1D, nnu)
;                 with downward intensity
;
; KEYWORDS (OUTPUT):
;   wHat1v:       ('weight_Hat_1_value') 1D vector with weight function for first mean,
;                 e.g. 'B' for the Planck mean
;   wHat1kappa:   ('weight_Hat_1_kappa') 1D vector with weight function times kappa for first mean,
;                 e.g. 'B*kappa' for the Planck mean
;   wHat2v:       ('weight_Hat_2_value') 1D vector with weight function for first mean
;   wHat2kappa:   ('weight_Hat_2_kappa') 1D vector with weight function times kappa for first mean
;   DualwHat_flag: ('Dual_weight_Hat_flag') logical (well: integer) to indicate if the requested mean
;                 uses one (allmost all means) or two weight functions (only 'Mix1', so far)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   BnuPlanck:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Compute the opacity weights from which the mean opacity can be computed by:
;     kappa_mean=total(wHatkappa)/total(wHat)
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2015-08-15 (B.F. Uppsala) Put into separate routine
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

; --- Define abbreviations for array sizes ---
nnu =n_elements(dnu)
nx1D=n_elements(T1D)

; --- Get mode of opacity mean ---
ModeMean='RP'
if (keyword_set(ModeMean0)) then begin
  ModeMean=string(ModeMean0)
endif
;print, 'ModeMean=                 ', ModeMean, format='(A,A)'
ModeMeanLC=strlowcase(ModeMean)

; --- Get temperature fraction ---
CDeltaT=0.2D+00
if (keyword_set(CDeltaT0)) then begin
  CDeltaT=double(CDeltaT0)
endif
;print, 'CDeltaT=                  ', CDeltaT, format='(A,E13.6)'


; === Create and fill arrays to be used for mean-opacity calculations ==============================

; --- Compute further quantities necessary for the opacity means ---
dnuAr        =rebin(reform(dnu, 1, nnu), nx1D, nnu)
DualwHat_flag=0

if (ModeMeanLC eq strlowcase('P')) then begin
  ; --- Use Planck mean ---
  wHat1     =B1D * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('Psf')) then begin
  ; --- Use small-fluctuation Planck mean ---
  wHat1     =dBdT1D * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('P05')) then begin
  ; --- Use modified Planck mean ---
  wHat1     =B1D / sqrt(kappa1D) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('R')) then begin
  ; --- Use Rosseland mean ---
  wHat1kappa=dBdT1D * dnuAr
  wHat1     =wHat1kappa / kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('RPsf')) then begin
  ; --- Use mix between Rosseland and Planck_small-fluct means ---
  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
  dtauxL    =xLength*rhoAr*kappa1D
  expdtauxL =exp(-dtauxL)
  wHat1kappa=dBdT1D * (1.0D+00-expdtauxL) * dnuAr
  wHat1     =wHat1kappa / kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('RP')) then begin
  ; --- Use mix between Rosseland and Planck means ---
  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
  dtauxL    =xLength*rhoAr*kappa1D
  expdtauxL =exp(-dtauxL)
  TAr       =double(rebin(reform(T1D, nx1D, 1), nx1D, nnu))
  wHat1     =(B1D    *                 expdtauxL                                   + $
              dBdT1D * CDeltaT * TAr * (1.0D+00-(1.0D+00+dtauxL)*expdtauxL)/dtauxL    ) * dnuAr
  Ind=where(dtauXL le 1.0D-04, n)
  if (n gt 0) then wHat1[Ind]=B1D[Ind] * dnuAr[Ind]
  wHat1kappa=wHat1 * kappa1D
  ;
;endif else if (ModeMeanLC eq strlowcase('RP05')) then begin
;  ; --- Use mix between Rosseland and Planck means ---
;  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
;  dtauxL    =xLength*rhoAr*kappa1D
;  expdtauxL =exp(-dtauxL)
;  TAr       =double(rebin(reform(T1D, nx1D, 1), nx1D, nnu))
;  wHat1     =(B1D    / kappa1D^0.1   * expdtauxL                                   + $
;              dBdT1D * CDeltaT * TAr * (1.0D+00-(1.0D+00+dtauxL)*expdtauxL)/dtauxL    ) * dnuAr
;  Ind=where(dtauXL le 1.0D-04, n)
;  if (n gt 0) then wHat1[Ind]=B1D[Ind] / kappa1D[Ind]^0.1 * dnuAr[Ind]
;  wHat1kappa=wHat1 * kappa1D
;  ;
endif else if (ModeMeanLC eq strlowcase('RmRP')) then begin
  ; --- Use mix between Rosseland and Planck means ---
  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
  dtauxL    =xLength*rhoAr*kappa1D
  expdtauxL =exp(-dtauxL)
  TAr       =double(rebin(reform(T1D, nx1D, 1), nx1D, nnu))
  ;wHat1     =(B1D    *                 expdtauxL                                   + $
  ;            dBdT1D * CDeltaT * TAr * (2.0D+00-(1.0D+00+dtauxL)*expdtauxL)/dtauxL    ) * dnuAr
  wHat1     =(B1D    *                 expdtauxL                                           + $
              dBdT1D * CDeltaT * TAr * (1.0D+00+1.0D-05-(1.0D+00+dtauxL)*expdtauxL)/dtauxL    ) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('BmB')) then begin
  ; --- Compute opacity from difference of two Planck functions ---
  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
  dtauxL    =xLength*rhoAr*kappa1D
  dB1D      =dblarr(nx1D, nnu)
  for inu=0,nnu-1 do begin
    ;Bp1D[*,inu]=BnuPlanck(nu[inu], reform(T1D * (1.0D+00 + CDeltaT * alog(1.0D+00 + 1.0D+00/dtauxL[*,inu]))))
    dTRel=CDeltaT * alog(1.0D+00 + 1.0D+00/reform(dtauxL[*,inu]))
    Ind=where(dTrel gt 1.0D-04, n)
    if (n gt 0) then dB1D[Ind, inu]=BnuPlanck(nu[inu], T1D[Ind] * (1.0D+00 + dTRel[Ind])) - B1D[Ind, inu]
    Ind=where(dTrel le 1.0D-04, n)
    if (n gt 0) then dB1D[Ind, inu]=dBdT1D[Ind, inu] * (T1D[Ind] * dTRel[Ind])
  endfor ; inu
  ;wHat1     =(dB1D-B1d) * dnuAr
  wHat1     =dB1D * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;print, 'dB1D/B1D:', minmax(dB1D/B1D)
  ;
endif else if (ModeMeanLC eq strlowcase('Mix1')) then begin
  ; --- Compute opacity from mix ---
  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
  dtauxL    =xLength*rhoAr*kappa1D
  expdtauxL =exp(-dtauxL)
  TAr       =double(rebin(reform(T1D, nx1D, 1), nx1D, nnu))
  wHat1     =(B1D    *                 expdtauxL                                   + $
              dBdT1D * CDeltaT * TAr * (1.0D+00-(1.0D+00+dtauxL)*expdtauxL)/dtauxL    ) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
  dB1D      =dblarr(nx1D, nnu)
  for inu=0,nnu-1 do begin
    ;Bp1D[*,inu]=BnuPlanck(nu[inu], reform(T1D * (1.0D+00 + CDeltaT * alog(1.0D+00 + 1.0D+00/dtauxL[*,inu]))))
    dTRel=CDeltaT * alog(1.0D+00 + 1.0D+00/reform(dtauxL[*,inu]))
    Ind=where(dTrel gt 1.0D-04, n)
    if (n gt 0) then dB1D[Ind, inu]=BnuPlanck(nu[inu], T1D[Ind] * (1.0D+00 + dTRel[Ind])) - B1D[Ind, inu]
    Ind=where(dTrel le 1.0D-04, n)
    if (n gt 0) then dB1D[Ind, inu]=dBdT1D[Ind, inu] * (T1D[Ind] * dTRel[Ind])
  endfor ; inu
  wHat2     =dB1D * dnuAr
  wHat2kappa=wHat2 * kappa1D
  DualwHat_flag=1
  ;
endif else if (ModeMeanLC eq strlowcase('Mix2')) then begin
  ; --- Compute opacity from mix ---
  rhoAr     =double(rebin(reform(rho1D, nx1D, 1), nx1D, nnu))
  dtauxL    =xLength*rhoAr*kappa1D
  expdtauxL =exp(-dtauxL)
  TAr       =double(rebin(reform(T1D, nx1D, 1), nx1D, nnu))
  wHat1     =(B1D    *                 expdtauxL                                   + $
              dBdT1D * CDeltaT * TAr * (1.0D+00-(1.0D+00+dtauxL)*expdtauxL)/dtauxL    ) * dnuAr
  ;
  dB1D      =dblarr(nx1D, nnu)
  for inu=0,nnu-1 do begin
    ;Bp1D[*,inu]=BnuPlanck(nu[inu], reform(T1D * (1.0D+00 + CDeltaT * alog(1.0D+00 + 1.0D+00/dtauxL[*,inu]))))
    dTRel=CDeltaT * alog(1.0D+00 + 1.0D+00/reform(dtauxL[*,inu]))
    Ind=where(dTrel gt 1.0D-04, n)
    if (n gt 0) then dB1D[Ind, inu]=BnuPlanck(nu[inu], T1D[Ind] * (1.0D+00 + dTRel[Ind])) - B1D[Ind, inu]
    Ind=where(dTrel le 1.0D-04, n)
    if (n gt 0) then dB1D[Ind, inu]=dBdT1D[Ind, inu] * (T1D[Ind] * dTRel[Ind])
  endfor ; inu
  wHat1     =wHat1 + dB1D * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('IumId')) then begin
  ; --- Use I_up-I_down "Chandrasekhar" mean ---
  wHat1     =(IIntu-IIntd) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('IupId')) then begin
  ; --- Use I_up+I_down mean ---
  wHat1     =(IIntu+IIntd) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('J')) then begin
  ; --- Use I_up+I_down+4B "J" mean ---
  wHat1     =(IIntu+IIntd+4.0D+00*B1D) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('IumB')) then begin
  ; --- Use I_up-B mean ---
  wHat1     =(IIntu-B1D) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else if (ModeMeanLC eq strlowcase('BmId')) then begin
  ; --- Use B-I_down mean ---
  wHat1     =(B1D-IIntd) * dnuAr
  wHat1kappa=wHat1 * kappa1D
  ;
endif else begin
  ; --- Unknown opacity-averaging mode detected: write error message and exit ---
  message, 'Unknown opacity mean'
  ;
endelse

end ; opac_com_Mean
