pro opac_com_StratInterpolate, $
  time0=time0, $
  Pt=Pt, Tt=Tt, rhot=rhot, nu=nu, $
  kappanu=kappanu, $
  x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
  ilogT1D=ilogT1D, ilogP1D=ilogP1D, $
  kappa1D=kappa1D, B1D=B1D, dBdT1D=dBdT1D
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_com_StratInterpolate ('opacity_compute_Stratification_Interpolate')
;
; PURPOSE:
;   Interpolate opacity and source function onto 1D stratification.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   opac_com_StratInterpolate, $
;     Pt=Pt, Tt=Tt, rhot=rhot, nu=nu, $
;     kappanu=kappanu, $
;     x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
;     ilogT1D=ilogT1D, ilogP1D=ilogP1D, $
;     kappa1D=kappa1D, B1D=B1D, dBdT1D=dBdT1D, $
;     time0=time0
;
; INPUT:
;   None (just keywords)
;
; KEYWORDS (INPUT):
;   Tt:           ('Temperature_table') 1D real vector with temperature axis of opacity data block
;   Pt:           ('Pressure_table') 1D real vector with pressure axis of opacity data block
;   rhot:         ('rho_table') 2D real array with density values corresponding to T/P axis values
;   nu:           ('nu') 1D real vector with frequency axis of opacity data block
;   kappanu:      ('kappa_nu') 3D real array with frequency-dependent opacities (large)
;                 of the form kappa(T,P,nu) as function of temperature, pressure, and wavelength
;   x1D:          ('x_1_Dimension') 1D real vector with x coordinates of points in stratification
;   T1D:          ('Temperature_1_Dimension') 1D real vector with temperature values of stratification
;   P1D:          ('Pressure_1_Dimension') 1D real vector with pressure values of stratification
;
; KEYWORDS (INPUT/OUTPUT):
;   rho1D:        ('rho_1_Dimension') 1D real vector with density values of stratification
;
; KEYWORDS (OUTPUT):
;   ilogT1D:      ('index_logarithm_Temperature_1_Dimension') 1D real vector with interpolation indices
;                 on temperature axis
;   ilogP1D:      ('index_logarithm_Pressure_1_Dimension') 1D real vector with interpolation indices
;                 on temperature axis
;   kappa1D:      ('kappa_1_Dimension') 2D double-precision array with
;                 frequency-dependent opacity values (per mass unit) of stratification
;   rhokappa1D:   ('rho_kappa_1_Dimension') 2D double-precision array with
;                 frequency-dependent opacity values (per volume) of stratification
;   B1D:          ('B_1_Dimension') 2D double-precision array with
;                 frequency-dependent values of the Planck function on stratification
;   dBdT1D:       ('delta_B_delta_Temperature_1_Dimension') 2D double-precision array with
;                 frequency-dependent values of the temperature derivative of the Planck function
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   BnuPlanck:
;   dBdTnuPlanck:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2014-05-07 (B.F. Uppsala) First version
;   2015-07-08 (B.F.) Put into own routine
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===============================================================

; --- Define abbreviations for array sizes ---
nTt =n_elements(Tt)
nPt =n_elements(Pt)
nnu =n_elements(nu)
nx1D=n_elements(x1D)
if ((nx1D ne n_elements(P1D)) or $
    (nx1D ne n_elements(T1D))     ) then begin
  message, 'Number of elements of 1D stratification arrays do not agree.'
endif


; === Interpolate data onto input 1D T/P stratification ========================

; --- Allocate arrays ---
kappa1D   =dblarr(nx1D, nnu)
;rhokappa1D=dblarr(nx1D, nnu)
B1D       =dblarr(nx1D, nnu)
dBdT1D    =dblarr(nx1D, nnu)

; --- Compute logarithmic temperature and pressure axes ---
logTt =double(alog10(          Tt           ))
logPt =double(alog10(          Pt           ))
logT1D=double(alog10(min(Tt) > T1D < max(Tt)))
logP1D=double(alog10(min(Pt) > P1D < max(Pt)))

; --- Get T and P interpolation indices ---
ilogT1D=interpol(double(indgen(nTt)), logTt, logT1D, /spline)
ilogP1D=interpol(double(indgen(nPt)), logPt, logP1D, /spline)

; --- Interpolate density ---
if (not keyword_set(rho1D)) then begin
  if (keyword_set(rhot)) then begin
    ; --- Compute density on 1D stratification from input 2D table ---
    ;rho1D=10.0^float(interpolate(double(alog10(rhot)), ilogT1D, ilogP1D, cubic=-0.5))
    rho1D=10.0D+00^interpolate(double(alog10(rhot)), ilogT1D, ilogP1D, cubic=-0.5)
  endif else begin
    message, 'No density array specified.'
  endelse
endif

; --- Interpolate kappanu for all frequency points ---
for inu=0,nnu-1 do begin
  ; --- Print a line every now and then ---
  linu=alog(float(inu+1))/alog(2.0)
  if ((inu gt 1000) and (linu eq long(linu))) then begin
    print, '% opac_com_StratInterpolate: ', systime(/SECONDS)-time0, ' s, frequency: ', inu+1, nu[inu], $
           format='(A,F7.2,A,I7,1X,E15.6)'
  endif
  ;
  ; --- Interpolate kappanu for current frequency point ---
  ;kappa1D[*,inu]=10.0^float(interpolate(double(alog10(kappanu[*,*,inu])), ilogT1D, ilogP1D, cubic=-0.5))
  kappa1D[*,inu]=10.0D+00^interpolate(double(alog10(kappanu[*,*,inu])), ilogT1D, ilogP1D, cubic=-0.5)
  ;
  ; --- Compute opacity per volume ---
  ;rhokappa1D[*,inu]=rho1D*kappa1D[*,inu]
  ;
  ; --- Compute Planck function and its temperature derivative ---
  B1D[   *,inu]=BnuPlanck(   nu[inu], T1D)
  dBdT1D[*,inu]=dBdTnuPlanck(nu[inu], T1D)
  ;
endfor ; inu

end ; opac_com_StratInterpolate
