pro opac_com_tau, $
  x1D=x1D, rho1D=rho1D, kappa1D=kappa1D, $
  tau1D=tau1D, dxInt=dxInt, dtauInt=dtauInt, tauInt=tauInt, $
  tauTopnu=tauTopnu, HrhoTop=HrhoTop
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_com_tau ('opacity_compute_tau')
;
; PURPOSE:
;   Integrate the optical-depth scale of 1D stratification.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   opac_com_tau, $
;     x1D=x1D, rho1D=rho1D, kappa1D=kappa1D, $
;     tau1D=tau1D, dxInt=dxInt, dtauInt=dtauInt, tauInt=tauInt, $
;     tauTopnu=tauTopnu, HrhoTop=HrhoTop
;
; INPUT:
;  None (just via keywords)
;
; OUTPUT:
;  None (just via keywords)
;
; KEYWORDS (INPUT):
;   x1D:          ('x_1_Dimension') 1D real vector with x coordinates of points in stratification,
;                 with nx1D points
;   rho1D:        ('rho_1_Dimension') 1D real vector with density values of stratification,
;                 with nx1D points
;   kappa1D:      ('kappa_1_Dimension') 2D double-precision array with
;                 frequency-dependent opacity values (per mass unit) of stratification
;
; KEYWORDS (OUTPUT):
;   tau1D:        ('tau_1_Dimension') 2D double-precision array (nx1D,nnu) with optical depth
;   dxInt:        ('delta_x_Integration') 1D double-precision vector (2*nx1D)
;                 with grid size of integration grid
;   dtauInt:      ('delta_tau_Integration') 2D double-precision array (2*nx1D,nnu)
;                 with delta optical depth
;   tauInt:       ('tau_Integration') 2D double-precision array (2*nx1D+1,nnu) with optical depth
;   tauTopnu:     ('tau_Top_nu') 1D double-precision vector (nnu) with optical-depth estimate at top
;   HrhoTop:      ('Height_rho_Top') double-precision scalar with density scale height at top
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2014-05-20 (B.F. Uppsala) First version
;   2015-07-09 (B.F.) Put into own file
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===============================================================

; --- Define abbreviations for array sizes ---
nnu =n_elements(kappa1D[0,*])
nx1D=n_elements(x1D)


; === Compute ==================================================================

; --- Estimate tau at upper boundary ---
diTop      =10 < (nx1D/2)
HrhoTop    =-double((x1D[-1-diTop]-x1D[-1])/alog(rho1D[-1-diTop]/rho1D[-1]))
tauTopnu   =reform(rho1D[-1]*kappa1D[-1,*])*HrhoTop

; --- Compute delta x and delta tau on refined grid ---
dxInt        =dblarr(2*nx1D)
dxInt[1:-3:2]=0.5D+00*(x1D[0:-2]-x1D[1:-1])
dxInt[2:-2:2]=0.5D+00*(x1D[0:-2]-x1D[1:-1])
dxInt[ 0    ]=dxInt[ 1]
dxInt[-1    ]=dxInt[-2]
dxInt        =abs(dxInt)
rhoInt       =rebin(rho1D, 2*nx1D)
dtauInt      =double(rebin(       kappa1D                 , 2*nx1D, nnu) * $
                     rebin(reform(rhoInt*dxInt, 2*nx1D, 1), 2*nx1D, nnu))

; --- Integrate tau scale ---
dtauIntTmp       =dblarr(2*nx1D+1, nnu)
dtauIntTmp[0  ,*]=tauTopnu
dtauIntTmp[1:*,*]=reverse(dtauInt, 1)
tauInt           =reverse(total(dtauIntTmp, 1, /CUMULATIVE), 1)

; --- Compute cell-centered tau scale ---
tau1D            =tauInt[1:*:2,*]

end ; opac_com_tau
