pro LABELTABLE, xin, fin, fstr, name=name, unit=unit, $
  xaxis=xaxis, format=format, $
  noticks=noticks, nonumbers=nonumbers, $
  lcolor=lcolor, colorrange=colorrange, minrange=minrange, maxrange=maxrange, $
  width=width, round=round, noinit=noinit
;
;+
; NAME:
;   LABELTABLE
;
; PURPOSE:
;   Put ticks and labels on an irregular axis.
;
; CATEGORY:
;   Plotting
;
; CALLING SEQUENCE:
;   LABELTABLE, xin, fin [, fstr, name=name, unit=unit, $
;     xaxis=xaxis, format=format, $
;     noticks=noticks, nonumbers=nonumbers, $
;     lcolor=lcolor, colorrange=colorrange, $
;     minrange=minrange, maxrange=maxrange, $
;     width=width, round=round, noinit=noinit ]
;
; INPUT:
;   xin - x or y values on specified axis
;   fin - Values of function for labelling
;
; OUTPUT:
;   fstr - String array containing the axis labels (for test purposes)
;
; KEYWORDS:
;   name - Name of the axis (Short string placed on topor at left)
;   unit - Unit (Short string placed at bottom or at right)
;   xaxis - If set: tick marks on top of figure (extra x axis)
;           otherwise: tick marks at right side of figure (extra y axis)
;   format - Format string for numbers, FORTRAN style or '+-'
;   noticks - If set no tick marks are drawn
;   nonumbers - If set no numbers are drawn
;   lcolor - If set the numbers are drawn in the appropriate color
;     colorrange - Two byte array [mincolor,maxcolor] defining the color range
;     minrange - Data value corresponding to the first color
;     maxrange - Data value corresponding to the last color
;   width - Width = number of characters of the axis
;   round - If set to 1 not every value gets a tick mark but only tick marks at
;           rounded values are drawn (useful for monotonic values).
;           If set to >1 this number of tick marks is equally distributed over
;           the axis.
;   noinit - If set the extra axis is placed to the right of the former and
;            not directly at the right side of the figure
;
; SUBROUTINES:
;
; COMMONBLOCKS:
;   LABELTABLECOMMON - Stores the position of the former axis
;
; SIDE EFFECTS:
;   Do plotting into current device. Change COMMON block LABELTABLECOMMON.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  21. 4.94 Written by Bernd Freytag, Astrophysik Kiel
;   8. 8.94 (B.F.) nonumbers -> constant tick length, FORMAT keyword modified
;  13. 8.94 (B.F.) Additional meaning of keyword ROUND
;   9.11.94 (B.F.) print commands removed
;  14. 3.97 (B.F.) finer steps
;-
;
; --- Common block to store position of next axis ---
common labeltablecommon, xrightnext, ytopnext
;
ix=n_elements(xin)
;
; --- Default values for some keywords ---
if (keyword_set(width)) then wi= 1 > byte(width) < 50 else wi=8
if (not keyword_set(nonumbers)) then nonumbers=0
if (not keyword_set(round)) then round=0
;print, 'round=', round
if (!P.CHARSIZE ne 0) then charsize=!P.CHARSIZE*0.75 else charsize=0.75
if (keyword_set(xaxis)) then begin
  dx=!D.X_CH_SIZE * (wi+1.0) * charsize
endif else begin
  dx=!D.Y_CH_SIZE * 1.2 * charsize
endelse
;
if (ix gt 0) then begin
  ;
  x0=xin
  f=fin
  if (keyword_set(round)) then begin
   if (round eq 1) then begin
    ; --- Find round values ---
    minf=min(fin,max=maxf)
    if (minf ne maxf) then begin
;      step=10.0^fix(alog10((maxf-minf)/2.0))
      step=10.0^fix(alog10((maxf-minf)/20.0))
      ;print, 'step=', step
      f =replicate(fin(0),fix(ix*((maxf-minf)/step+1.0)))
      x0=replicate(xin(0),fix(ix*((maxf-minf)/step+1.0)))
      inew=0
      for i=0,ix-2 do begin
        a=fin(i) < fin(i+1)
        b=fin(i) > fin(i+1)
        if (a ne b) then begin
          an=fix(a/step)
          if (a ge 0.0) then an=an+1        
          bn=fix(b/step)
          if (b le 0.0) then bn=bn-1
          ;print, i, a, b, an, bn
          if (fin(i) gt fin(i+1)) then begin
            sn=an
            an=bn
            bn=sn
            sn=-1
          endif else begin
            sn=1
          endelse
          for n=an,bn,sn do begin
            f(inew)=n*step
            x0(inew)=(f(inew)-fin(i))/(fin(i+1)-fin(i))* $
                     (xin(i+1)-xin(i)) + xin(i)
            ;print,inew,x0(inew),f(inew)
            inew=inew+1
          endfor
        endif
      endfor
      ix=inew
      f = f(0:inew-1)
      x0=x0(0:inew-1)
    endif
   endif else begin
     ; --- Compute locations and values of given number of tick marks ---
     minx=min(xin,max=maxx)
     if (round ne -1) then begin
       ; --- Use given number of tick marks ---
       ix=fix(abs(round))
     endif else begin
       ; --- Compute optimum number of tick marks ---
       if (keyword_set(xaxis)) then begin
         ix=fix( (!X.WINDOW(1)-!X.WINDOW(0))*!D.X_VSIZE / dx )
       endif else begin
         ix=fix( (!Y.WINDOW(1)-!Y.WINDOW(0))*!D.Y_VSIZE / dx )
       endelse
     endelse
     ; --- Create equidistant x axis ---
     x0=findgen(ix)/float(ix-1)*(maxx-minx)+minx
     ; --- Find corresponding values by interpolation ---
     f=interpol(fin,xin,x0)
   endelse
  endif
  ;
  ; --- Colors ---
  if (keyword_set(lcolor)) then begin
    if (not keyword_set(colorrange)) then colorrange=[0,!D.TABLE_SIZE]
    if (not keyword_set(minrange)) then minrange=min(f)
    if (not keyword_set(maxrange)) then maxrange=max(f)
    colors=byte( (f-minrange)/(maxrange-minrange)* $
                 (colorrange(1)-colorrange(0)) + colorrange(0) )
  endif else begin
    colors=replicate(!P.COLOR,ix)
  endelse
  ;
  ; --- Transform axis values to device units and choose spacing ---
  xx=convert_coord(x0,x0, /data, /to_device)
  if (keyword_set(xaxis)) then begin
    x=xx(0,*)
  endif else begin
    x=xx(1,*)
  endelse
  ; --- Reduce minimum distance for equally spaced axis ---
  if (round lt 0) then dx=0.0
  ;
  ; --- Select format ---
  if (not keyword_set(format)) then begin
    maxabs=max(abs(f))
    if (((maxabs ge 10.0^(wi-2)) or (maxabs lt 10.0^(5-wi))) and (wi ge 7)) $
    then begin
      format='(E8.1)'
      format='(E' + string(wi) + '.' + string(wi-7) + ')'
    endif else begin
      format='(F' + string(wi) + '.' + $
             string( 0 > ((wi-3-fix(alog10(maxabs))) < (wi-3)) ) + ')'
    endelse
    format=strcompress(format, /remove_all)
  endif
  ;print, 'format=', format
  ;
  ; --- Select only values with minimum distance to neighbors ---
  fstr=strarr(ix)
  xold=x(0)
  if (format eq '+-') then begin
    if (f(0) lt 0.0) then fstr(0)='-' else fstr(0)='+'
    if (f(ix-1) lt 0.0) then fstr(ix-1)='-' else fstr(ix-1)='+'
  endif else begin
    fstr(0)=strcompress(string(f(0),format=format))
    fstr(ix-1)=strcompress(string(f(ix-1),format=format))
  endelse
  for i=1,ix-2 do begin
    if (abs(x(i)-xold) gt dx) and (abs(x(i)-x(ix-1)) gt dx) then begin
      xold=x(i)
      if (format eq '+-') then begin
        if (f(i) lt 0.0) then fstr(i)='-' else fstr(i)='+'
      endif else begin
        fstr(i)=strcompress(string(f(i),format=format))
      endelse
    endif
  endfor
  ;
  ; --- Delete trailing '.' ---
  for i=0,ix-1 do begin
    if (fstr(i) ne '') then begin
      lenfstr=strlen(fstr(i))
      if strmid(fstr(i),lenfstr-1,1) eq '.' then $
        fstr(i)=strmid(fstr(i),0,lenfstr-1)
    endif
  endfor
  ;
  ; --- Find coordinates of frame ---
  px = !X.WINDOW * !D.X_VSIZE  ; Get size of window in device units (frame size)
  py = !Y.WINDOW * !D.Y_VSIZE
  xleft0=px(0)
  ybot0=py(0)
  xright0=px(1)
  ytop0=py(1)
  ;
  if (not keyword_set(noinit)) then begin
    ; --- Put upper right edge coordinates into common block ---
    xrightnext=xright0
    ytopnext=ytop0
  endif
  ;
  ; --- Plot tick marks and write labels ---
  ticklen=!D.X_CH_SIZE*charsize/2.0
  if (keyword_set(xaxis)) then begin      ; --- x axis ---
    for i=0,ix-1 do begin
      if (not keyword_set(noticks)) then $
        plots, [x(i), x(i)], $
               [ytopnext,ytopnext+ $
                ticklen*(1.0+0.7*((fstr(i) ne '') and (nonumbers eq 0)))], $
               /device
      if (not keyword_set(nonumbers)) then $
        xyouts, x(i), ytopnext+!D.X_CH_SIZE*charsize, color=colors(i), $
                /device, fstr(i), alignment=0.5, charsize=charsize
    endfor
    if (keyword_set(name)) then begin
      xyouts, xleft0-(wi+2)/2.0*!D.X_CH_SIZE*charsize, $
              ytopnext+!D.X_CH_SIZE*charsize, /device, $
              name, alignment=1, charsize=charsize
    endif
    if (keyword_set(unit)) then begin
      xyouts, xright0+(wi+2)/2.0*!D.X_CH_SIZE*charsize, $
              ytopnext+!D.X_CH_SIZE*charsize, /device, $
              unit, alignment=0, charsize=charsize
    endif
    ;
    ; --- Put new upper side coordinates into common block ---
    ytopnext=ytopnext+(!D.X_CH_SIZE+!D.Y_CH_SIZE*1.2)*charsize
  endif else begin                         ; --- y axis ---
    for i=0,ix-1 do begin
      if (not keyword_set(noticks)) then $
        plots, [xrightnext,xrightnext+ticklen* $
               (1.0+0.7*((fstr(i) ne '') and (nonumbers eq 0)))], $
               [x(i),x(i)], /device
      if (not keyword_set(nonumbers)) then $
        xyouts, xrightnext+(wi+1)*!D.X_CH_SIZE*charsize, $
                x(i)-!D.Y_CH_SIZE*charsize/2.5, color=colors(i), $
                /device, fstr(i), alignment=1, charsize=charsize
    endfor
    if (keyword_set(name)) then begin
      xyouts, xrightnext+(wi+1)*!D.X_CH_SIZE*charsize, $
              ytop0+!D.Y_CH_SIZE*charsize, /device, $
              name, alignment=1, charsize=charsize
    endif
    if (keyword_set(unit)) then begin
      xyouts, xrightnext+(wi+1)*!D.X_CH_SIZE*charsize, $
              ybot0-1.5*!D.Y_CH_SIZE*charsize, /device, $
              unit, alignment=1, charsize=charsize
    endif
    ;
    ; --- Put new right side coordinates into common block ---
    if (keyword_set(nonumbers)) then begin
      xrightnext=xrightnext+(wi)*!D.X_CH_SIZE*charsize
    endif else begin
      xrightnext=xrightnext+(wi+2)*!D.X_CH_SIZE*charsize
    endelse
  endelse
  ;
endif
;
end ; LABELTABLE
