pro plotfield_arrow, x0, y0, x1, y1, $
      hsize=hsize, color=color, thick=thick, noclip=noclip
;------------------------------------------------------------------------------
;+
; NAME:
;   plotfield_arrow ('plot_field_arrow')
;
; PURPOSE:
;   Draw a vectors with arrow heads
;
; CATEGORY:
;   Graphics
;
; CALLING SEQUENCE:
;   plotfield_arrow, x0, y0, x1, y1, $
;     hsize=hsize, color=color, thick=thick, noclip=noclip
;
; INPUT:
;   x0:     ('x_0') 1D real array with x (data) coordinates of start of vectors.
;   y0:     ('y_0') 1D real array with y (data) coordinates of start of vectors.
;   x1:     ('x_1') 1D real array with x (data) coordinates of end of vectors.
;   y1:     ('y_1') 1D real array with y (data) coordinates of end of vectors.
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   hsize:  ('head_size') size of arrowhead.  Default = 1/64th the width of
;           the device, (!D.X_SIZE / 64.).
;           If the size is positive, it is assumed to be in device
;           coordinate units.  If it is NEGATIVE, then the head length
;           is set to the vector length * abs(hsize), giving heads
;           proportional in size to the bodies.  The size is defined as
;           the length of each of the lines (separated by 60 degrees) 
;           that make the head.
;   color:  ('color') drawing color.  Default = highest color index.
;   thick:  ('thick') thickness of body. Default = 1.0.
;   noclip: ('no_clip') if set to zero, do clipping. Default: clipping
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   This is an extended/restricted version of the standard IDL routine 'arrow'.
;   It is usually called by 'plotfield_streamlines'.
;
; PROCEDURE:
;   Loop over all input coordinate tupels and draw line with arrow head.
;
; EXAMPLE:
;   ; Draw an arrow from (100,150) to (300,350) in DEVICE units.
;   plotfield_arrow, 100, 150,  300, 350
;
; MODIFICATION HISTORY:
;   1994-04-18 (B.F.) Keyword NOCLIP added to standard IDL routine 'arrow'
;   2005-07-07 (B.F.) arrow1 -> plotfield_arrow;
;                     changed default: do clipping; use normalized coordinates
;                     limit relativ arrow size
;   2006-02-07 (B.F.) Back to device coordinates and unlimited arrow size
;-
;------------------------------------------------------------------------------

; --- Keyword handling ---

if (n_elements(thick) eq 0) then thick=1.0

; --- Head size in device units ---
if (n_elements(hsize) eq 0) then begin
  ;arrowsize=!d.x_size/64. * (thick/2. > 1.0)
  arrowsize=10.0*(thick/2. > 1.0)
endif else begin
  arrowsize=float(hsize)
endelse
if (n_elements(color) eq 0) then color=!P.COLOR
if (n_elements(noclip) eq 0) then noclip=0

; --- Head angle: 30 degrees ---
mcost=-0.866
sint = 0.500
msint=-sint

; --- Loop over all vectors ---
for i=0, n_elements(x0)-1 do begin
  ; --- Transform to normalized coordinats ---
  p=convert_coord([x0(i),x1(i)],[y0(i),y1(i)], /data, /to_dev)
  xp0=p(0,0) & xp1=p(0,1)
  yp0=p(1,0) & yp1=p(1,1)
  ;
  ; --- Normalize vector ---
  dx  =float(xp1-xp0)
  dy  =float(yp1-yp0)
  dlen=sqrt(dx^2 + dy^2)
  if (dlen gt 1.0e-10) then begin
    dx  =dx/dlen
    dy  =dy/dlen
  endif else begin
    dx  =1.0
    dy  =0.0
    dlen=0.5
  endelse
  ;
  ; --- Size of arrow ---
  if (arrowsize gt 0.0) then begin
    a=0.0025*arrowsize
  endif else begin
;    a=-dlen *arrowsize < 0.02
    a=-dlen *arrowsize
  endelse
  ;
  ; --- Arrow coordinates ---
  xxp0=xp1 + a * (dx*mcost - dy*msint)
  yyp0=yp1 + a * (dx*msint + dy*mcost)
  xxp1=xp1 + a * (dx*mcost - dy*sint )
  yyp1=yp1 + a * (dx*sint  + dy*mcost)
  ;
  plots, [xp0, xp1]     ,[yp0, yp1]     , /dev, color=color, thick=thick, noclip=noclip
  plots, [xxp0,xp1,xxp1],[yyp0,yp1,yyp1], /dev, color=color, thick=thick, noclip=noclip
endfor

end ; plotfield_arrow
