function plotfield_rectpix, din, x, y, xfactor=xfactor, aspectratio=aspectratio, $
  sample=sample, xnew=xnew, ynew=ynew
;
;+
; NAME:
;   plotfield_rectpix ('plot_field_rectangular_pixels')
;
; PURPOSE:
;   Interpolate from 2D grid with irregular spaced axes to regular grid using
;   bilinear interpolation or nearest neighbor sampling.
;
; CATEGORY:
;   Mathematical functions
;
; INPUT:
;   din:         ('data_in') 2D input array
;   x:           ('x') Vector with old x axis
;   y:           ('y') Vector with old y axis
;
; OUTPUT:
;   plotfield_rectpix: ('plot_field_rectangular_pixels') Rescaled 2D array
;
; KEYWORDS:
;   xfactor:     ('x_factor') Magnification factor for x axis (y axis is scaled automatically)
;   aspectratio: ('aspect_ratio') If set -> aspectratio of output array,
;                 default: aspectratio=(max(x)-min(x))/(max(y)-min(y))
;   sample:      ('sample') If set -> Use nearest neighbor sampling instead of
;                           bilinear interpolation
;   xnew:        ('x_new') Output: new x axis (min(x) = min(xnew), max(x) = max(xnew))
;   ynew:        ('y_new') Output: new y axis (min(y) = min(ynew), max(y)>= max(ynew))
;
; SUBROUTINES:
;   Only standard
;
; SIDE EFFECTS:
;   None
;
; EXAMPLE:
;
; PROCEDURE:
;
; RESTRICTIONS:
;   This routine might need an update, possibly to account for newer IDL routines.
;
; MODIFICATION HISTORY:
;  1994-04-17 (B.F. Kiel) Written
;  1994-04-18 (B.F.)
;  2005-05-15 (B.F. LA) Screen output changed
;  2005-07-07 (B.F.) rectpix -> plotfield_rectpix
;-
;
; --- Analyse input axis: size, minimum, maximum, step ---
xsize=size(x)
ix=xsize(xsize(0)+2)
ysize=size(y)
iy=ysize(ysize(0)+2)
xmax=max(x,min=xmin)
ymax=max(y,min=ymin)
xtotal=xmax-xmin
ytotal=ymax-ymin
dx=xtotal/(ix-1)
dy=ytotal/(iy-1)
;
; --- Compute new equidistant axes ---
if (not keyword_set(xfactor)) then xfactor=1.0
ixnew=long(ix*xfactor)
dxnew=xtotal/((ixnew-1) > 1)
xnew=findgen(ixnew)*dxnew + xmin              ; New x axis
if (not keyword_set(aspectratio)) then begin
  dynew=dxnew
  iynew=long(ytotal/dynew+0.9999)
endif else begin
  iynew=long(float(ixnew)/abs(aspectratio))
  dynew=ytotal/((iynew-1) > 1)
endelse
ynew=findgen(iynew)*dynew + ymin              ; New y axis
;
; --- Output ---
;print, 'Old grid:'
;print, 'Old grid x:', x(0), ' to', x(ix-1), ' step', dx, ' with', ix, ' points', format='(3(A,G13.5),A,I5,A)'
;print, '         y:', y(0), ' to', y(iy-1), ' step', dy, ' with', iy, ' points', format='(3(A,G13.5),A,I5,A)'
;print, 'Grid:', x(0), ' to', x(ix-1), ' d', dx, ' w', ix, ' px', $
;         '  x', y(0), ' to', y(iy-1), ' d', dy, ' w', iy, ' px', format='(2(3(A,G12.5),A,I5,A))'
;
;print, 'New grid:'
;print, 'New grid x:', xnew(0), ' to', xnew(ixnew-1), ' step', dxnew, ' with', ixnew, ' points', format='(3(A,G13.5),A,I5,A)'
;print, '         y:', ynew(0), ' to', ynew(iynew-1), ' step', dynew, ' with', iynew, ' points', format='(3(A,G13.5),A,I5,A)'
;print, '   ->', xnew(0), ' to', xnew(ixnew-1), ' d', dxnew, ' w', ixnew, ' px', $
;         '  x', ynew(0), ' to', ynew(iynew-1), ' d', dynew, ' w', iynew, ' px', format='(2(3(A,G12.5),A,I5,A))'
;
; --- Compute weights for spatial interpolation in x direction ---
xindnew=interpol(findgen(ix),x,xnew)
xindnewminus=long(xindnew) < (ix-1L)
xindnewplus=(xindnewminus+1) < (ix-1L)
xweightplus=xindnew-float(xindnewminus)
if (keyword_set(sample)) then xweightplus=float(xweightplus gt 0.5)
xweightminus=1.0-xweightplus
;
; --- Compute weights for spatial interpolation in y direction ---
yindnew=interpol(findgen(iy),y,ynew)
yindnewminus=long(yindnew) < (iy-1L)
yindnewplus=(yindnewminus+1) < (iy-1L)
yweightplus=yindnew-float(yindnewminus)
if (keyword_set(sample)) then yweightplus=float(yweightplus gt 0.5)
yweightminus=1.0-yweightplus
;
; --- Interpolate x direction ---
dtmp=replicate(din(0,0),ixnew,iy)
for i=0L,ixnew-1L do begin
  dtmp(i,*)=din(xindnewplus( i),*)*xweightplus( i) + $
            din(xindnewminus(i),*)*xweightminus(i)
endfor
;
; --- Interpolate y direction ---
dout=replicate(din(0,0),ixnew,iynew)
for j=0L,iynew-1L do begin
  dout(*,j)=dtmp(*,yindnewplus( j))*yweightplus( j) + $
            dtmp(*,yindnewminus(j))*yweightminus(j)
endfor
;
return, dout
;
end ; plotfield_rectpix
