pro plotfield_streamlines, xvec, yvec, xaxis=xaxis, yaxis=yaxis, $
  overplot=overplot, $
  device=device, psfilename=psfilename, landscape=landscape, noclose=noclose, $
  xtitle=xtitle, ytitle=ytitle, title=title, $
  xthick=xthick, ythick=ythick, $
  head=head, $
  randomstart=randomstart, nparticles=nparticles0, nrow=nrow0, ncolumn=ncolumn0, $
  timestep=timestep, nsteps=nsteps0, norder=norder0, $
  thick=thick, velcolor=velcolor, arrowsize=arrowsize, $
  longtrace=longtrace, naddstep=naddstep0, oldstart=oldstart, $
  sidecond=sidecond, botcond=botcond, $
  xpin=xpin, ypin=ypin, xpout=xpout, ypout=ypout, xpall=xp, ypall=yp
;
;------------------------------------------------------------------------------
;+
; NAME:
;   plotfield_streamlines ('plot_field_stream_lines')
;
; PURPOSE:
;   Plot two 2D arrays as streamlines.
;
; CATEGORY:
;   Plotting
;
; CALLING SEQUENCE:
;   plotfield_streamlines, xvec, yvec, xaxis=xaxis, yaxis=yaxis, $
;     overplot=overplot, $
;     device=device, psfilename=psfilename, landscape=landscape, noclose=noclose, $
;     xtitle=xtitle, ytitle=ytitle, title=title, $
;     xthick=xthick, ythick=ythick, $
;     head=head, $
;     randomstart=randomstart, nparticles=nparticles0, nrow=nrow0, ncolumn=ncolumn0, $
;     timestep=timestep, nsteps=nsteps0, norder=norder0, $
;     thick=thick, velcolor=velcolor, arrowsize=arrowsize, $
;     longtrace=longtrace, naddstep=naddstep0, oldstart=oldstart, $
;     sidecond=sidecond, botcond=botcond, $
;     xpin=xpin, ypin=ypin, xpout=xpout, ypout=ypout, xpall=xp, ypall=yp
;
; INPUT:
;   xvec:       ('x_vector') real 2D array with x-component of vector field
;   yvec:       ('y_vector') real 2D array with y-component of vector field
;
; OUTPUT:
;   None (only optional via keywords)
;
; KEYWORDS:
;   xaxis:      ('x_axis') x axis values
;   yaxis:      ('y_axis') y axis values
;   device:     ('device') string: name of output device:
;               'X':  graphic display or
;               'PS': postscript file
;   psfilename: ('postscript_file_name) string, name of postscript file.
;               If keyword DEVICE is set a postscript file with this name is generated.
;   landscape:  ('landscape') logical to switch from portrait (default) to landscape
;   noclose:    ('no_close') logical (0/1): if set and device='ps' the
;               output postscript file is not closed after plotting
;               (to allow overplotting)
;   xtitle:     ('x_title') string, title of x axis
;   ytitle:     ('y_title') string, title of y axis
;   title:      ('title') string, head line on top of plot
;   xthick:     ('x_thick') x thickness of frame (see !X.THICK)
;   ythick:     ('y_thick') y thickness of frame (see !Y.THICK)
;   head:       ('head') character vector with header (above plot)
;   randomstart:('random_start') real, start with random initial positions
;   nparticles: ('number_particles') (long) integer with number of particles, can also
;               be specified via nrow and ncolumn
;   nrow:       ('number_row') integer, number of tracer particle rows
;   ncolumn:    ('number_column') integer, number of tracer particle columns
;   timestep:   ('time_step') real, timestep
;   nsteps:     ('number_steps') integer, number of points for one streamline (default: 4)
;   norder:     ('number_order') integer, order of integration (1 or 2)
;   thick:      ('thick') integer with thickness of streamlines (and arrows)
;   velcolor:   ('velocity_color') integer with color index of streamlines
;   arrowsize:  ('arrow_size') real with size of arrow head
;
;
; SUBROUTINES:
;   plotfield_arrow: Replacement of standard IDL routine 'arrow'.
;
; SIDE EFFECTS:
;   Plotting into window or postscript file
;
; EXAMPLE:
;   vx=cos(dist(100)/20.0) & vy=sin(dist(100)/20.0)
;   streamlines, vx, vy
;
; MODIFICATION HISTORY:
;  1994-04-17 (B.F. Kiel) Written
;  1994-04-18 (B.F.)
;  1994-05-12 (B.F.) Double precision for timestep determination
;  1994-07-25 (B.F.) long
;  1999-09-16 (B.F. Copenhagen) To find proper cell: ... +0.5
;  2005-06-23 (B.F. LA) More comments
;  2005-07-07 (B.F.) streamlines -> plotfield_streamlines
;-
;------------------------------------------------------------------------------
;
if (n_params() lt 2) then begin
  message, 'Not enough parameters.'
endif else if (n_params() gt 2) then begin
  message, 'Too many parameters.'
endif
;
; --- Check field dimensions: xvec ---
xvecsize=size(xvec)
if (xvecsize(0) eq 0) then begin
  if (xvecsize(1) eq 0) then begin
    message, 'x vector field is undefined.'
  endif else begin
    ix=1
    iy=1
  endelse
endif else if (xvecsize(0) eq 1) then begin
  ix=xvecsize(1)
  iy=1
endif else if (xvecsize(0) eq 2) then begin
  ix=xvecsize(1)
  iy=xvecsize(2)
endif else begin
  message, 'x vector field has more than 2 dimensions.'
endelse  
;print, 'Field dimensions:', ix, ' x', iy
;
; --- Check field dimensions: yvec ---
yvecsize=size(yvec)
if (min(yvecsize eq xvecsize) eq 0) then begin
  message, 'x and y vector fields are incompatible.'
endif
;
; --- Check x axis ---
if (n_elements(xaxis) ne 0) then begin
  xaxissize=size(xaxis)
  if (xaxissize(0) eq 0) then begin
    if (ix ne 1) then begin
      message, 'x axis has too many elements.'
    endif
  endif else if (xaxissize(0) gt 2) then begin
    message, 'x axis has too many dimensions.'
  endif else if (xaxissize(1) ne ix) then begin
    message, 'x axis has an incorrect number of elements.'
  endif
  xax=xaxis
endif else begin
  xax=findgen(ix)
endelse
;
; --- Check y axis ---
if (n_elements(yaxis) ne 0) then begin
  yaxissize=size(yaxis)
  if (yaxissize(0) eq 0) then begin
    if (iy ne 1) then begin
      message, 'y axis has too many elements.'
    endif
  endif else if (yaxissize(0) gt 2) then begin
    message, 'y axis has too many dimensions.'
  endif else if (yaxissize(1) ne iy) then begin
    message, 'y axis has an incorrect number of elements.'
  endif
  yax=yaxis
endif else begin
  yax=findgen(iy)
endelse
;
; --- Chose device ---
if keyword_set(device) then dev=device else dev='X'
if (not keyword_set(overplot)) then begin
  if (strupcase(dev) eq 'X') then begin
    set_plot, 'X'
  endif else begin
    set_plot, dev, /copy               ; --- Select postscript output device ---
    if (n_elements(psfilename) ne 0) then device, filename=psfilename
    if (keyword_set(landscape)) then begin
      device, /landscape, xsize=29.0, ysize=19.3, xoffset=1.0, yoffset=29.7
    endif else begin
      device, /portrait , xsize=19.3, ysize=28.0,  xoffset=1.0, yoffset=1.0
    endelse
    if (n_elements(velcolor) ne 0) then device, /color ; - Enable color output -
    help, /device 
  endelse
endif
;
; --- Title ---
if (not keyword_set(xtitle)) then xtitle=!X.TITLE
if (not keyword_set(ytitle)) then ytitle=!Y.TITLE
if (not keyword_set(title))  then  title=!P.TITLE
;
; --- Thick ---
if (not keyword_set(xthick)) then $
   if (!X.THICK eq 0) then xthick=2 else xthick=!X.THICK
if (not keyword_set(ythick)) then $
   if (!Y.THICK eq 0) then ythick=2 else ythick=!Y.THICK
if (not keyword_set(thick))  then  thick=!P.THICK
;
; --- Linestyle ---
if (not keyword_set(linestyle)) then linestyle=!P.LINESTYLE
;
; --- Style ---
if (not keyword_set(xstyle)) then $
  if (!X.STYLE eq 0) then xstyle=1 else xstyle=!X.STYLE
if (not keyword_set(ystyle)) then $
  if (!Y.STYLE eq 0) then ystyle=1 else ystyle=!Y.STYLE
;
; --- Range ---
xaxmin=min(xax, max=xaxmax)
xtotal=xaxmax-xaxmin
yaxmin=min(yax, max=yaxmax)
ytotal=yaxmax-yaxmin
;xvecmin=min(xvec, max=xvecmax)
;yvecmin=min(yvec, max=yvecmax)
;
; --- Header --
ymargin=!Y.MARGIN
if (keyword_set(head)) then begin
  headsize=size(head)
  headnum=headsize(headsize(0)+2)
  ymargin(1)=!Y.MARGIN(1) > float(3.0+headnum*2.0)
endif
;
; --- Plot frame ---
if (not keyword_set(overplot)) then begin
  plot, xax, yax, /nodata, ymargin=ymargin, $
    xstyle=xstyle, ystyle=xstyle, $
    xtitle=xtitle, ytitle=ytitle, title=title, $
    xthick=xthick, ythick=ythick
endif
;
; --- Header --
if (keyword_set(head)) then begin
  for n=0,headnum-1 do begin
    linestep=1.5*float(!D.Y_CH_SIZE)/float(!D.Y_SIZE)
    xyouts, linestep, 0.98-linestep*(n+1), /norm, head(n)
  endfor
endif
;
; --- Initialize particle positions ---
seed=5                                          ; Initialize seed for RANDOMU
; --- Initialize vectors for particle start positions ---
if keyword_set(norder0) then norder=1>fix(norder0)<2 $
                        else norder=2
if keyword_set(nsteps0) then nsteps=2>fix(nsteps0)<10000 $
                        else nsteps=4
if keyword_set(naddstep0) then naddstep=0>naddstep0<10000 $
                          else naddstep=0
if keyword_set(longtrace) then nsteptotal=nsteps $
                          else nsteptotal=nsteps+naddstep
if (n_elements(arrowsize) eq 0) then arrowsize=-1.0
;
if (keyword_set(xpin) and keyword_set(ypin)) then begin
  xpsize=size(xpin)
  if (xpsize(0) eq 1) then begin
    lastindex, xpin, npart
    xp=fltarr(npart,nsteptotal)
    yp=fltarr(npart,nsteptotal)
    xp(*,0)=xpin
    yp(*,0)=ypin
  endif else begin
    lastindex, xpin, xpi1, xpi2
    npart=xpi1*xpi2
    xp=fltarr(npart,nsteptotal)
    yp=fltarr(npart,nsteptotal)
    xp(*,0)=reform(xpin(*,*,0),npart)
    yp(*,0)=reform(ypin(*,*,0),npart)
  endelse
endif else begin
  if (keyword_set(nparticles0)) then begin
    npart=2 > fix(nparticles0) < 100000L
    if (keyword_set(nrow0)) then begin
      nrow=1 > fix(nrow0) < npart
    endif else if (keyword_set(ncolumn0)) then begin
      nrow=npart/(1 > fix(ncolumn0) < npart)
    endif else begin
      nrow=2
    endelse
  endif else begin
    if (keyword_set(nrow0)) then begin
      nrow=1 > fix(nrow0) < 10000
      if (keyword_set(ncolumn0)) then begin
        npart=nrow * (1 > fix(ncolumn0) < 10000)
      endif else begin
        npart=10*nrow
      endelse
    endif else begin
      if (keyword_set(ncolumn0)) then begin
        nrow=10
        npart=nrow * (1 > fix(ncolumn0) < 10000)
      endif else begin
        nrow=10
        npart=100
      endelse
    endelse
  endelse
  xp=fltarr(npart,nsteptotal)
  yp=fltarr(npart,nsteptotal)
  if keyword_set(randomstart) then begin    ; --- Start positions at random -`
    xp(*,0)=xtotal*randomu(seed,npart) + xaxmin
    if (abs(randomstart) lt 0.01) then begin
      yp(*,0)=ytotal*randomu(seed,npart) + yaxmin
    endif else begin
      yp(*,0)=ytotal*alog((exp(randomstart)-1.0)* $
                           randomu(seed,npart)+1.0)/randomstart + yaxmin
    endelse   
  endif else begin                            ; --- Start positions in rows -`
    xp(*,0)=((nrow*findgen(npart))/npart*xtotal) mod xtotal + $
            xaxmin
    for l=0,nrow-1 do begin
      yp(long(l)*npart/nrow:long(l+1)*npart/nrow-1,0)= $
            float(nrow-l)/(nrow+1)*yaxmin + float(l+1)/(nrow+1)*yaxmax
    endfor
  endelse
endelse
xpstart=xp(*,0)
ypstart=yp(*,0)
if (n_elements(velcolor) ne 0) then begin   ; Color number for particle traces
  vcolor=0 > velcolor < (!D.N_COLORS-1)
endif else begin
  vcolor=!P.COLOR
endelse
;print, 'Color for particle traces:', vcolor
;
;if keyword_set(timestep) then dtime=timestep else dtime=1.0
if keyword_set(timestep) then begin
  dtime=timestep
endif else begin
  xyvecmax=max(sqrt(double(xvec)^2+double(yvec)^2))
  dtime=float((xtotal < ytotal)/xyvecmax/10.0)
endelse
dtimestep=dtime/(nsteps-1)
;
; --- Interpolate velocity field ---
u=xvec
v=yvec
;
; --- Boundary conditions ---
n=0
if (n gt 0) then begin
  if (not keyword_set(longtrace)) then nnew=nnew-naddstep
  if keyword_set(sidecond) then begin          ; Side boundary conditions
    xp(*,nhead)=xaxmin>xp(*,nnew)<xaxmax
  endif else begin
    xp(*,nhead)=(xp(*,nnew)+xtotal-raxmin) mod xtotal + xaxmin
  endelse
  ; yp(*,nhead)=yp(*,nnew)<yaxmax           ; Top boundary condition
  ;
  yp(*,nhead)=yp(*,nnew)                              ; New head = old tail
  ;
  if (not keyword_set(botcond)) then begin      ; Bottom boundary condition
    nlost=where(yp(*,nnew) lt yaxmin, numlost)
    if (numlost gt 0) then begin   ; Some particles are lost through bottom
      print, 'Particles are lost:',numlost
;      yp(nlost,nhead)=    dy*randomu(seed,numlost)+yaxmin
      yp(nlost,nhead)=                             yaxmin
      xp(nlost,nhead)=xtotal*randomu(seed,numlost)+xaxmin
    endif
  endif
endif
;
; --- Solve DEQ ---
for nstep=0,nsteps-2+naddstep do begin
  if keyword_set(longtrace) then nold=(n*framereuse+nf)*(nsteps-1)+nstep $
                            else nold=nstep
  nnew=nold+1
;  if (keyword_set(oldstart)) then begin
;    ip=fix((xpstart-xaxmin)/xtotal*ix)
;    jp=fix((ypstart-yaxmin)/ytotal*iy)
;  endif else begin
;    ip=fix((xp(*,nold)-xaxmin)/xtotal*ix)
;    jp=fix((yp(*,nold)-yaxmin)/ytotal*iy)
;  endelse
  if (keyword_set(oldstart)) then begin
    ip=interpol(findgen(ix),xax,xpstart)+0.5
    jp=interpol(findgen(iy),yax,ypstart)+0.5
  endif else begin
    ip=interpol(findgen(ix),xax,xp(*,nold))+0.5
    jp=interpol(findgen(iy),yax,yp(*,nold))+0.5
  endelse
  up=u(ip,jp)
  vp=v(ip,jp)
  xp(*,nnew)=xp(*,nold)+dtimestep*up
  yp(*,nnew)=yp(*,nold)+dtimestep*vp
  if (norder eq 2) then begin          ; --- Corrector step -> 2. order ---
;    if (keyword_set(oldstart)) then begin
;      ip=fix((xpstart-xaxmin)/xtotal*ix)
;      jp=fix((ypstart-yaxmin)/ytotal*iy)
;    endif else begin
;      ip=fix((xp(*,nnew)-xaxmin)/xtotal*ix)
;      jp=fix((yp(*,nnew)-yaxmin)/ytotal*iy)
;    endelse
    if (keyword_set(oldstart)) then begin
      ip=interpol(findgen(ix),xax,xpstart)+0.5
      jp=interpol(findgen(iy),yax,ypstart)+0.5
    endif else begin
      ip=interpol(findgen(ix),xax,xp(*,nnew))+0.5
      jp=interpol(findgen(iy),yax,yp(*,nnew))+0.5
    endelse
    up=u(ip,jp)
    vp=v(ip,jp)
    xp(*,nnew)=0.5*(xp(*,nold)+xp(*,nnew)+dtimestep*up)
    yp(*,nnew)=0.5*(yp(*,nold)+yp(*,nnew)+dtimestep*vp)
  endif
endfor
;
if keyword_set(longtrace) then nhead=nnew else nhead=0
;
; --- Plot particle traces (and arrows) ---
for k=0,npart-1 do plots, xp(k,0:nnew), yp(k,0:nnew), noclip=0, $
                          color=vcolor, thick=thick
if (arrowsize ne 0) then $
  plotfield_arrow, xp(*,nnew-1),yp(*,nnew-1), xp(*,nnew),yp(*,nnew), $
         color=vcolor, hsize=arrowsize, thick=thick
;
; -----------------------------------------------------------------------------
;
; --- Reset variables ---
if (strupcase(dev) ne 'X' and not keyword_set(noclose)) then begin
  device, /close 
  set_plot, 'X'
endif
;
xpout=xp(*,nnew)
ypout=yp(*,nnew)
;
end ; plotfield_streamlines
