pro PLOTLINES, xaxis, datain, nodata=nodata, $
  xtitle=xtitle0, ytitle=ytitle0, title=title0, $
  device=device, $
    psfilename=psfilename, psformat=psformat, noclose=noclose, psnoinit=psnoinit, $
  overplot=overplot, aspectratio=aspectratio, xrange=xrange0, drange=drange, $
  xthick=xthick0, ythick=ythick0, thick=thick0, psym=psym0, symsize=symsize0, $
  xstyle=xstyle0, ystyle=ystyle0, linecolor=linecolor, linestyle=linestyle0, $
  legend=legend, legitems=legitems, label=label, $
  zeroline=zeroline, meansub=meansub, offset=offset, head=head, date=date
;
;+
; NAME:
;   PLOTLINES
;
; PURPOSE:
;   Plot a 1D or 2D array as set of lines with style.
;
; CATEGORY:
;   Plotting
;
; CALLING SEQUENCE:
;   PLOTLINES, xaxis, datain, nodata=nodata, $
;     xtitle=xtitle, ytitle=ytitle, title=title, $
;     psfilename=psfilename, device=device, psformat=psformat, noclose=noclose, $
;     psnoinit=psnoinit, $
;     overplot=overplot, aspectratio=aspectratio, xrange=xrange, drange=drange, $
;     xthick=xthick, ythick=ythick, thick=thick, psym=psym, symsize=symsize0, $
;     xstyle=xstyle, ystyle=ystyle, linecolor=linecolor, linestyle=linestyle, $
;     legend=legend, legitems=legitems, label=label, $
;     zeroline=zeroline, meansub=meansub, offset=offset, head=head, date=date
;
; INPUT:
;   xaxis:  ('x_axis') x axis values (1D or 2D)
;   datain: ('data_in') input data array (1D or 2D)
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   nodata:  ('no_data') if set not data are plotted (only frame)
;   xtitle:  ('x_title') x axis title string
;   ytitle:  ('y_title') y axis title string
;   title:   ('title') title string
;   psfilename: ('postscript_file_name') name of postscript file.
;            If keyword DEVICE is set to 'ps' a
;            postscript file with this name is generated.
;   device:  ('device') String, name of output device:
;            'X':  graphic display or
;            'PS': postscript file
;   psformat:('postscript_format') list of formattings for postscript:
;            'ldina4':    Landscape, DIN A 4
;            'pdina4':    Portrait, DIN A 4
;            'pdina4_16': Portrait, DIN A 4, small version
;            'nochange':  No change of plot window size and offset
;            default:     Portrait, DIN A 4
;   noclose: ('no_close') logical (0/1): if set and device='ps' the
;            output postscript file is not closed after plotting
;            (to allow overplotting)
;   psnoinit:('postscript_no_init') logical (0/1): if set no new output
;            postscript file is generated.
;            It is assumed that one is opened already.
;   overplot:('overplot') logical (0/1): if set the old coordinate system
;            and labeling is used
;   aspectratio:('aspect_ratio') aspect ratio of plot frame
;   xrange:  ('x_range') 2-element vector with x-axis range 
;            (default: from input data)
;   drange:  ('data_range') 2-element vector with data range
;            (default: from input data)
;   xthick:  ('x_thick') x thickness of frame (see !X.THICK)
;   ythick:  ('y_thick') y thickness of frame (see !Y.THICK)
;   thick:   ('thick') (list of) line thickness value(s) (default !P.THICK)
;            It is possible to specify a thickness for all lines or
;            individual values. The thickness values are repeated cyclically.
;   psym:    ('plot_symbol') (list of) plot symbol value(s) (devault !P.PSYM)
;   symsize: ('symbol_size') (list of) plot symbol sizes (devault !P.SYMSIZE)
;   xstyle:  ('x_style') x style of frame (see !X.STYLE)
;   ystyle:  ('y_style') y style of frame (see !Y.STYLE)
;   linecolor:('line_color') (list of) line color(s)
;   linestyle:('line_style') (list of) line style value(s)
;   legend:  ('legend') logical (0/1): if set draw a legend
;   legitems:('legend_items') list of strings to label
;            items in legend (if keyword legend is set)
;            or plot lines (if keyword label is set)
;   label:   ('label') logical (0/1): if set the lines are labeled with the
;            contents of legitems
;   zeroline:('zero_line') logical (0/1): if set a line with zero values is plotted
;   meansub: ('mean_subtract') (0/1/2): if set the horizontal mean
;            (average over first dimension (1) or all other dimensions (2)) is subtracted
;   offset:  ('offset') scalar value: each plotted line is shifted by this value
;            relative to its predecessor
;   head:    ('head') vector of strings to be plotted on top of plot
;   date:    ('date') logical (0/1): if set the current time and date is plotted
;            in the upper right corner
;
; SUBROUTINES:
;   Replicate1: Extension of REPLICATE: Replicate an array
;   Legend: Draw a legend
;   Psymuser: Draw standard and additional plot symbols
;
; SIDE EFFECTS:
;   Plotting into window or postscript file
;
; EXAMPLE:
;   plotlines, dist(20)
;   plotlines, findgen(20,10), /label
;   plotlines, findgen(20)*10, findgen(20,3), /legend, linestyle=[1,2,4]
;
; MODIFICATION HISTORY:
;    7. 4.94 (B.F. Kiel) Written
;   15. 4.94 (B.F.) Keyword NOCLOSE added
;   20. 4.94 (B.F.) Keyword ASPECTRATIO added
;   25. 4.94 (B.F.) Keyword LANDSCAPE substituted by PSFORMAT
;   12. 8.94 (B.F.) Now allowed: only plot of header
;   26. 8.94 (B.F.) x axis may be 2-dimensional
;    6. 4.95 (B.F.) Routine PSYMUSER added, keywords are not changed anymore
;    7. 4.95 (B.F.) Linecolor: -1 -> !P.COLOR
;    5. 5.95 (B.F.) Header handling from PLOTFIELD
;   11. 5.95 (B.F.) Keyword DRANGE added
;   15. 7.96 (B.F.) OVERPLOT -> no headlines
;   14. 3.97 (B.F.) call set_plot, 'X' only if !D.NAME ne 'X'
;   26. 1.99 (B.F. Copenhagen) More comments, keyword xrange added
;   15. 2.99 (B.F.) date string smaller
;   12. 3.01 (B.F. Uppsala) Use !P.SYMSIZE
;   20. 3.02 (B.F.) Allow linecolor keyword with [*,3] dimension
;   30. 7.04 (B.F. Montpellier) meansub for first or rest of dimensions
;   19. 5.05 (B.F. LA) Keyword nodata added
;   23. 5.09 (B.F. Lyon) Keyword symsize added
;   28. 4.13 (B.F. Uppsala) Print color 0 (and not necessarily !P.COLOR) if (linecolor eq 0)
;-
;
if (n_params() eq 1) then begin
  data=xaxis
endif else if (n_params() eq 0) then begin
  data=0.0
endif else begin
  xax=xaxis
  data=datain
endelse
;
; --- Check field dimensions ---
datasize=size(data)
if (datasize[0] eq 0) then begin
  if (datasize[1] eq 0) then begin
    message, 'Data field is undefined.'
  endif else begin
    num1=1
    num2=1
  endelse
endif else if (datasize[0] eq 1) then begin
  num1=datasize[1]
  num2=1
endif else if (datasize[0] eq 2) then begin
  num1=datasize[1]
  num2=datasize[2]
endif else begin
  ; --- Force 2D array ---
  data=reform(data,datasize[1],datasize(datasize[0]+2)/datasize[1])
  datasize=size(data)
  num1=datasize[1]
  num2=datasize[2]
endelse
;print, 'Field dimensions:', num1, ' x', num2
;
; --- Subtract mean ---
if (keyword_set(meansub)) then begin
  if (meansub eq 1) then begin
    dataavg=avg(data,0)
    for n=0,num2-1 do data[*,n]=data[*,n]-dataavg[n]
  endif else begin
    dataavg=avg(data,1)
    for n=0,num1-1 do data[n,*]=data[n,*]-dataavg(n)
  endelse
endif
;
; --- Add offset ---
if (keyword_set(offset)) then begin
  for n=0,num2-1 do data[*,n]=data[*,n]+float(n)*offset
endif
;
; --- Check x axis ---
if (n_params() eq 2) then begin
  xaxissize=size(xax)
  if (xaxissize[0] eq 0) then begin
    if (num1 ne 1) then begin
      message, 'x axis has too many elements.'
    endif
  endif else if (xaxissize[0] eq 2) then begin
    xdimfac=1
  endif else if (xaxissize[0] gt 2) then begin
;    message, 'x axis has too many dimensions.'
    xdimfac=1
    ; --- Force 2D array ---
    xax=reform(xax,xaxissize[1],xaxissize(xaxissize[0]+2)/xaxissize[1])
    xaxissize=size(xax)
  endif else if (xaxissize[1] ne num1) then begin
    message, 'x axis has an incorrect number of elements.'
  endif else begin
    xdimfac=0
  endelse
endif else begin
  xax=findgen(num1)
  xdimfac=0
endelse
;
; --- Choose device ---
if keyword_set(device) then dev=device else dev='X'
if (not keyword_set(overplot) and not keyword_set(psnoinit)) then begin
  if (strupcase(dev) eq 'X') then begin
    if (!D.NAME ne 'X') then set_plot, 'X'
  endif else begin
    set_plot, 'ps', /copy              ; --- Select postscript output device ---
    if (n_elements(psfilename) eq 0) then psfilename='idl.ps' ; --- Filename ---
    device, filename=psfilename
    if (keyword_set(psformat)) then begin
      if (strlowcase(psformat) eq 'ldina4') then begin
        ; --- Landscape, DIN A 4 ---
        device, /landscape, xsize=29.0, ysize=19.3, xoffset=1.0, yoffset=29.7
      endif else if (strlowcase(psformat) eq 'pdina4') then begin
        ; --- Portrait, DIN A 4 ---
        device, /portrait , xsize=19.3, ysize=28.0,  xoffset=1.0, yoffset=1.0
      endif else if (strlowcase(psformat) eq 'pdina4_16') then begin
        ; --- Portrait, DIN A 4, small version ---
        device, /portrait , xsize=16.0, ysize=28.0,  xoffset=1.0, yoffset=1.0
      endif else if (strlowcase(psformat) eq 'nochange') then begin
        ; --- No change of plot window size and offset ---
        print, 'Old device settings'
      endif else begin
        ; --- Default: Portrait, DIN A 4 ---
        device, /portrait , xsize=19.3, ysize=28.0,  xoffset=1.0, yoffset=1.0
      endelse
    endif
;    if (keyword_set(linecolor)) then if (max(linecolor) ge 0) then $
    if (n_elements(linecolor) ge 1) then if (max(linecolor) ge 0) then $
      device, /color  ; --- Enable color output ---
    help, /device 
  endelse
endif
;
; --- Title ---
if (not keyword_set(xtitle0)) then xtitle=!X.TITLE else xtitle=xtitle0
if (not keyword_set(ytitle0)) then ytitle=!Y.TITLE else ytitle=ytitle0
if (not keyword_set(title0))  then  title=!P.TITLE else  title=title0
;
; --- Thickness ---
if (not keyword_set(xthick0)) then $
   if (!X.THICK eq 0) then xthick=2 else xthick=!X.THICK else xthick=xthick0
if (not keyword_set(ythick0)) then $
   if (!Y.THICK eq 0) then ythick=2 else ythick=!Y.THICK else ythick=ythick0
if (not keyword_set(thick0)) then thick=!P.THICK else thick=thick0
numthick=n_elements(thick)
;
; --- Psym ---
if (not keyword_set(psym0)) then psym=!P.PSYM else psym=psym0
numpsym=n_elements(psym)
;
; --- Symsize ---
if (not keyword_set(symsize0)) then symsize=!P.SYMSIZE else symsize=symsize0
numsymsize=n_elements(symsize)
;
; --- Color ---
;if (not keyword_set(linecolor)) then begin
if (n_elements(linecolor) eq 0) then begin
  lc=!P.COLOR
endif else begin
  linecolor_dim=(size(linecolor))[0]
  if (linecolor_dim le 1) then begin
    lc=linecolor
  endif else begin
    lc=linecolor[*,0] + 256L*linecolor[*,1] + 65536L*linecolor[*,2]
  endelse
  ind=where(lc lt 0, count)
  if (count ge 1) then lc[ind]=!P.COLOR
endelse
numlinecolor=n_elements(lc)
;
; --- Linestyle ---
if (not keyword_set(linestyle0)) then linestyle=!P.LINESTYLE $
                                 else linestyle=linestyle0
numlinestyle=n_elements(linestyle)
;
; --- Style ---
if (not keyword_set(xstyle0)) then $
   if (!X.STYLE eq 0) then xstyle=3 else xstyle=!X.STYLE else xstyle=xstyle0
if (not keyword_set(ystyle0)) then $
   if (!Y.STYLE eq 0) then ystyle=3 else ystyle=!Y.STYLE else ystyle=ystyle0
;
; --- Range ---
if (keyword_set(drange)) then begin
  datamin=min(drange, max=datamax)
  if (datamin eq datamax) then datamin=min(data, max=datamax)
endif else begin
  datamin=min(data, max=datamax)
endelse
xaxmin=min(xax, max=xaxmax)
;print, 'Minimum=', datamin, ' Maximum=', datamax
if (keyword_set(xrange0)) then begin
  xrange=xrange0
endif else begin
  if (!X.RANGE[0] eq !X.RANGE[1]) then begin
    if (xaxmin eq xaxmax) then begin
      xrange=[xaxmin-1.0,abs(2.0*xaxmax)] 
    endif else begin
      xrange=[xaxmin, xaxmax]
    endelse
  endif else begin
    xrange=!X.RANGE
  endelse
endelse
if (!Y.RANGE[0] eq !Y.RANGE[1]) then begin
  if (datamin eq datamax) then begin
    yrange=[datamin-1.0,abs(2.0*datamax)]
  endif else begin
    yrange=[datamin, datamax]
  endelse
endif else begin
  yrange=!Y.RANGE
endelse
;
; --- Header --
ymargin=!Y.MARGIN
if (keyword_set(head)) then begin
  headnum=n_elements(head)
;  ymargin[1]=!Y.MARGIN[1] > float(3.0+headnum*2.0)
  if (!P.CHARSIZE eq 0) then headcharsize=1.0 else headcharsize=!P.CHARSIZE
;  ymargin[1]=!Y.MARGIN[1] > float((2.0+headnum*1.8)*headcharsize)
  ymargin[1]=!Y.MARGIN[1] > float((2.0+headnum*1.8))
endif
;
; --- Plot ---
if (not keyword_set(overplot) and(n_elements(xaxis) gt 0)) then begin
  if (keyword_set(aspectratio)) then begin
    ; --- Make unvisible testplot ---
    plot, xrange, yrange, /nodata, ymargin=ymargin, $
      xstyle=8*(xstyle/8)+4+(xstyle mod 4), ystyle=8*(ystyle/8)+4+(ystyle mod 4)
    ; --- Find coordinates of frame ---
    px = !X.WINDOW * !D.X_VSIZE ;Get size of window in device units (frame size)
    py = !Y.WINDOW * !D.Y_VSIZE
    xleft0 =px[0]
    ybot0  =py[0]
    xright0=px[1]
    ytop0  =py[1]
    frameaspect=(xright0-xleft0)/(ytop0-ybot0)
    if (aspectratio gt frameaspect) then begin
      ytop0=(ytop0-ybot0)*frameaspect/aspectratio+ybot0
    endif else begin
      xright0=(xright0-xleft0)*aspectratio/frameaspect+xleft0
    endelse
    ; --- Fix plotting coordinate system with correct aspect ratio ---
    plot, xrange, yrange, position=[xleft0,ybot0,xright0,ytop0], /device, $
      /nodata, /noerase, $
      xrange=xrange, yrange=yrange, $
      xtitle=xtitle, ytitle=ytitle, title=title, $
      xthick=xthick, ythick=ythick, xstyle=xstyle, ystyle=ystyle
  endif else begin
    plot, xrange, yrange, /nodata, ymargin=ymargin, $
      xrange=xrange, yrange=yrange, $
      xtitle=xtitle, ytitle=ytitle, title=title, $
      xthick=xthick, ythick=ythick, xstyle=xstyle, ystyle=ystyle
  endelse
endif else if (not keyword_set(overplot)) then begin
  erase
endif
;
if (not keyword_set(nodata)) then begin
 if (n_elements(xaxis) ne 0) then begin
  for n=0L,num2-1 do $
    oplot, xax[*,n*xdimfac], data[*,n], $
      symsize=symsize, $
      thick=thick[n mod numthick], psym=psymuser(psym[n mod numpsym]),$
      color=lc[n mod numlinecolor], linestyle=linestyle[n mod numlinestyle]
;      symsize=!P.SYMSIZE, $
 endif
endif
;
if (keyword_set(zeroline)) then oplot, xax, replicate(0.0, num1), linestyle=2
;
;if (keyword_set(date)) then xyouts, 0.99, 0.98, /norm, ali=1.0, charsize=0.7, $
;                                    systime(0)
if (keyword_set(date)) then xyouts, 0.99, 0.985, /norm, ali=1.0, charsize=0.6, $
                                    systime(0)
;
; --- Header --
if (keyword_set(head) and not keyword_set(overplot)) then begin
  for n=0,headnum-1 do begin
;    linestep=1.5*float(!D.Y_CH_SIZE)/float(!D.Y_SIZE)
;    xyouts, linestep, 0.98-linestep*(n+1), /norm, head(n)
    linestep=1.8*float(!D.Y_CH_SIZE)*headcharsize
    xyouts, 3*!D.X_CH_SIZE, !D.Y_SIZE-linestep*(n+0.8), /device, head(n), $
            charsize=headcharsize
  endfor
endif
;
; --- Legend ---
if (not keyword_set(legitems)) then begin
  items=strcompress(string(indgen(num2)))
endif else begin
  items=replicate1(legitems,num2)
endelse
if (keyword_set(label)) then $
  for n=0L,num2-1 do xyouts, xax[num1-1], data[num1-1,n], items[n], $
    charthick=thick[n mod numthick], color=lc[n mod numlinecolor], $
    charsize=!P.CHARSIZE
if (keyword_set(legend)) then begin
  legend, items, psym=replicate1(psym,num2), $
          linestyle=replicate1(linestyle,num2), color=replicate1(lc,num2)
endif
;
; --- Reset variables ---
if (strupcase(dev) ne 'X' and not keyword_set(noclose)) then begin
  device, /close 
  set_plot, 'X'
endif
;
end ; PLOTLINES
