function HaltonSequence, index, base
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   HaltonSequence ('Halton_Sequence')
;
; PURPOSE:
;   Compute the requested entry of the Halton sequence of quasi-random numbers
;
; CATEGORY:
;   Monte-Carlo
;
; CALLING SEQUENCE:
;   result=HaltonSequence(index, base)
;
; INPUT:
;   index:  ('index') integer (>= 0) with number of requested entry in Halton sequence
;   base:   ('base') integer with base of sequence, ususally a small prime: 2, 3, 5,...
;
; OUTPUT:
;   HaltonSequence: ('Halton_Sequence') real with output quasi-number in Halton sequence
;
; LOCAL VARIABLES:
;   f:      ('factor') real with sequence 1./base, 1./base^2, 1./base^3,...
;             e.g., 1./2., 1./4., 1./8.,...
;   i:      ('integer') integer with sequence 1, 1/base, (1/base)/base,...
;             The sequence 1 mod base, 1/base mod base, (1/base)/base mod base,...
;             gives a representation of 'index' in base 'base' (from small to large)
;
; KEYWORDS:
;   Non
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The index in the sequency has to be non-negative.
;   The base should be a (positive) prime number.
;   Quasi-random numbers are not as random as pseudo-random numbers.
;
; PROCEDURE:
;   See pseudo code on
;   https://en.wikipedia.org/wiki/Halton_sequence
;   
; EXAMPLE:
;   IDL> print, haltonsequence(6,2)
;          2       6       0     0.500000      0.00000      0.00000
;          2       3       1     0.250000     0.250000     0.250000
;          2       1       1     0.125000     0.125000     0.375000
;        0.375000
;
; MODIFICATION HISTORY:
;   2016-03-31 (B.F. Uppsala) First version
;-
;---------------------------------------------------------------------------------------------------

result=0.0
f     =1.0
i     =index
while (i gt 0) do begin
  f     =f/base
  result=result + f*(i mod base)
  print, base, i, i mod base, f, f*(i mod base), result
  i     =i/base
endwhile

return, result

end ; HaltonSequence
