pro greyintegrate, $
  Teff=Teff, logg=logg, $
  gas=gas, $
  Ptop=Ptop, Pbot=Pbot, Tmax=Tmax, $
  dlntau=dlntau0, approximation=approximation, $
  tauin=tauin, Tin=Tin, noshift=noshift, $
  tauout=tau, Tout=T, Pout=P, zout=z, $
  rhoout=rho, kapout=kap
;
;------------------------------------------------------------------------------
;+
; NAME:
;   greyintegrate (grey_integrate)
;
; PURPOSE:
;   Integrate a grey atmosphere in radiative equilibrium
;
; CATEGORY:
;   Computations (Special)
;
; CALLING SEQUENCE:
;   greyintegrate
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;   ...
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   tempgreytau:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Slow
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   28.10.94 Written by Bernd Freytag, Astrophysik Kiel
;   29.10.94 (B.F.) Change of integration scheme
;   31. 3.95 (B.F.) Approximation from Mihalas for TEMPGREYTAU
;    9. 4.95 (B.F.) Keywords TAUIN, TIN added
;   10. 4.95 (B.F.) Ptop < 0: Ptop is estimated from tauin(0)
;   16. 3.96 (B.F.) Compute kap0 if tauin is not given
;   18.11.96 (B.F.) low metallicty abundances included
;   28. 4.99 (B.F. Copenhagen) greyin -> greyintegrate,
;            use IDL interpolation for EOS and OPTA
;    7. 5.99 (B.F.) Keyword approximation added
;    5. 5.00 (B.F.) Keyword Tmax added
;-
;------------------------------------------------------------------------------
;
; --- Define default input keyword parameters ---
if (n_elements(Teff) eq 0) then Teff=8500.0
if (n_elements(logg) eq 0) then logg=4.40
g=10.0^logg
if (n_elements(Ptop) eq 0) then Ptop=20.0
if (n_elements(Pbot) eq 0) then begin
  if (n_elements(tauin) ne 0) then Pbot=1.0E+30 else Pbot=1.0E+08
endif
if (n_elements(dlntau0) eq 0) then dlntau0=0.1
;
if (not keyword_set(approximation)) then approximation='mihalas'
;
; --- Generate arrays ---
if (n_elements(tauin) ne 0) then begin
  jmax=n_elements(tauin)
endif else begin
  jmax=2000
endelse
tau=fltarr(jmax)
T=fltarr(jmax)
P=fltarr(jmax)
z=fltarr(jmax)
rho=fltarr(jmax)
kap=fltarr(jmax)
;
; --- Start values ---
z(0)=0.0
if ((n_elements(tauin) ne 0) and (n_elements(Tin) ne 0)) then begin
  T(0)=Tin(jmax-1)
endif else begin
  T(0)=Teff*tempgreytau(0.0,approximation=approximation)
endelse
if (keyword_set(Tmax)) then T(0)=T(0) < Tmax
;
if (n_elements(tauin) ne 0) then begin
  ; --- tau scale prescribed ---
  if (ptop gt 0.0) then begin
    ; --- Given tau(0), P(0) ---
    tau(0)=tauin(jmax-1)
    P(0)=ptop
  endif else begin
    ; --- Compute P(0) from tau(0) ---
    tau(0)=tauin(jmax-1)
    Piterold=0.0
    Piternew=abs(ptop)
    Perr=Piternew
    niter=0
    while ((niter lt 25) and (abs(Piternew-Piterold) ge 0.001*Perr)) do begin
      ; --- Compute opacity kap ---
      kap0=xkaros(Piternew, T(0), 0)
      Piterold=Piternew
      ; --- Compute new pressure ---
      Piternew=0.5*(tau(0)*g/kap0+Piterold)
      print, 'iter:', niter, ':', Piterold, ' ->', Piternew
      niter=niter+1
    endwhile
    P(0)=Piternew
  endelse
endif else begin
  ; --- Compute tau(0) from P(0) ---
  P(0)=abs(ptop)
  ; --- Compute opacity kap ---
  kap0=xkaros(P(0), T(0), 0)
  ; --- tau(0)=0.75*kap*rho*Hp ---
  ;tau(0)=0.75*kap0*P(0)/g ; 10.4.95: factor 0.75 removed
  tau(0)=kap0*P(0)/g
endelse
;
j=0
; --- Do the integration (Heun scheme, cf. Stoer, Burlisch, II., p. 109) ---
while (P(j) le Pbot and j lt (jmax-1)) do begin
;  print,'tau,z,P,T', tau(j),z(j),P(j),T(j)
  if (n_elements(tauin) ne 0) then begin
    tau(j+1)=tauin(jmax-1-(j+1))
  endif else begin
    ; --- Next optical depth point ---
    tau(j+1)=tau(j)*exp(dlntau0)
  endelse
  if ((n_elements(tauin) ne 0) and (n_elements(Tin) ne 0)) then begin
    T(j+1)=Tin(jmax-1-(j+1))
  endif else begin
    ; --- Compute temperature from formula for grey atmosphere ---
    T(j+1)=Teff*tempgreytau(tau(j+1),approximation=approximation)
  endelse
  if (keyword_set(Tmax)) then T(j+1)=T(j+1) < Tmax
  dtau=tau(j+1)-tau(j)
  dlntau=alog(tau(j+1)/tau(j))
;  dlntau=2.0*dtau/(tau(j+1)+tau(j))
  ; --- Compute density rho from EOS with gasinter ---
  gasinter, [P(j)], [T(j)], rho1, eos=gas
  rho1=rho1(0)
  ; --- Compute opacity kap ---
  kappa1=xkaros(P(j), T(j), 0)
  ;
  rho(j)=rho1
  kap(j)=kappa1
  ;
  ; --- Coefficients of DEQ ---
  fP1=g/kappa1
  fz1=-1.0/(kappa1*rho1)
  ; --- Compute predictor values for next point ---
;  P1=P(j)+dtau*fP1
;  z1=z(j)+dtau*fz1
;  print,j,P(j),T(j),dtau,fP1,g,kappa1
  P1=P(j)*exp(dtau*fP1/P(j))
  z1=z(j)+dlntau*fz1*tau(j)
  ; --- Compute density rho from EOS with gasinter ---
  gasinter, [P1], [T(j+1)], rho2, eos=gas
  rho2=rho2(0)
  ; --- Compute opacity kap ---
  kappa2=xkaros(P1, T(j+1), 0)
  ; --- Coefficients of DEQ ---
  fP2=g/kappa2
  fz2=-1.0/(kappa2*rho2)
  ; --- Compute final values for next point ---
;  P(j+1)=P(j) + dtau*0.5 * (fP1+fP2)
;  z(j+1)=z(j) + dtau*0.5 * (fz1+fz2)
  P(j+1)=P(j) * exp(dtau * (fP1+fP2)/(P(j)+P1))
  z(j+1)=z(j) + dlntau * 0.25*(fz1+fz2)*(tau(j)+tau(j+1))
;  z(j+1)=z(j) + dlntau * 0.5*(fz1+fz2)*sqrt(tau(j)*tau(j+1))
;  z(j+1)=z(j) + dlntau * 0.5*(fz1*tau(j)+fz2*tau(j+1))
  ; --- Increment j ---
  j=j+1
endwhile
;
rho(j)=rho2
kap(j)=kappa2
;
; --- Reduce output arrays to proper size and reverse them ---
tau=reverse(tau(0:j))
T=reverse(T(0:j))
P=reverse(P(0:j))
z=reverse(z(0:j))
rho=reverse(rho(0:j))
kap=reverse(kap(0:j))
;
; --- Shift z axis so that tau(z=0)=1 ---
if (not keyword_set(noshift)) then begin
  ztau1=interpol(z,tau,1.0)
  z=z-ztau1(0)
endif
;
end ; greyintegrate
