pro power2d1d, q, qs, $
      WaveDelta=WaveDelta0, ComputeNorm_flag=ComputeNorm_flag, qn=qn, qw=qw, $
      q0=q0, qp=qp, WaveNumber2D=WaveNumber2D, qsn=qsn, qsa=qsa, qsw=qsw, $
      WaveNumber1DDown=WaveNumber1DDown, WaveNumber1DUp=WaveNumber1DUp
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   power2d1d ('power_2_dimensions_1_dimension')
;
; PURPOSE:
;   Compute 1D size distribution from 2D quadratic image.
;
; CATEGORY:
;   Data analysis
;
; CALLING SEQUENCE:
;   power2d1d, q, qs, $
;              WaveDelta=WaveDelta, ComputeNorm_flag=ComputeNorm_flag, qn=qn, qw=qw,
;              q0=q0, qp=qp, WaveNumber2D=WaveNumber2D, qsn=qsn, qsa=qsa, qsw=qsw, $
;              WaveNumber1DDown=WaveNumber1DDown, WaveNumber1DUp=WaveNumber1DUp
;    
; INPUT:
;   q:                ('quantity') real (or double-precision) 2D array
;
; OUTPUT:
;   qs:               ('quantity_size') real vector with size spectrum
;
; KEYWORDS:
;   WaveDelta:        ('WaveNumber_Delta') real scalar with half wavenumber interval:
;                       0.5: most common value
;                       0.3: default
;   ComputeNorm_flag: ('Compute_Norm_flag') if set, the vectores qn and qw are computed
;   qn:               ('quantity_number') real vector (same size as qs) with normalization factors.
;                       As final result, the produce qs*qn should be used (plotted vs. qw)
;   qw:               ('quantity_wavelength') real vector with corrected wavelength axis.
;                     A value of 1 (usually of the first bin) corresponds to a wavelegth of the image width.
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The input array has to be two-dimensional and quadratic.
;
; PROCEDURE:
;   See
;   http://www.exelisvis.com/Support/HelpArticles/TabId/185/ArtMID/800/ArticleID/1293/1293.aspx
;
; TODO:
;
; EXAMPLE:
;   nx=350 & ny=350
;   q=  rebin(reform(sin(findgen(nx)/float(nx)*1.0*2.0*!pi),nx,1),nx,ny)
;   q=q+rebin(reform(sin(findgen(nx)/float(nx)*5.0*2.0*!pi),nx,1),nx,ny)
;
; MODIFICATION HISTORY:
;   2015-12-08 (B.F. Lyon) First version
;   2015-12-09 (B.F.) More comments
;   2015-12-10 (B.F.) Allow non-equal wavenumber bins
;   2015-12-16 (B.F. Uppsala) Allow 1D arrays
;-
;---------------------------------------------------------------------------------------------------

; --- Set common (but non-default) compiler options
compile_opt defint32, strictarr


; ==================================================================================================

; --- Set default value for optional keyword ---
if (keyword_set(WaveDelta0)) then begin
  WaveDelta=abs(WaveDelta0)
endif else begin
  WaveDelta=0.3
endelse

; --- Check size of input array ---
nqdim=size(q, /n_dimension)
qdim =size(q, /dimension)
nx=qdim[0]
if (nqdim eq 1) then begin
  ny=1
endif else begin
  ny=qdim[1]
  if (ny eq 1) then nqdim=1
endelse
if (nqdim eq 1) then begin
  DimFac=1.0
endif else begin
  DimFac=1.42
endelse

if ((nqdim gt 1) and (nx ne ny)) then message, 'Input array is not quadratic.'

; --- Subtract the total mean ---
qavg=mean(q)
q0  =q-qavg

; --- Compute the power spectrum. Use the square of the amount. ---
qp=abs(fft(q0, -1))^2
;;;;; qp[*]=1.0 ; --- Attention: testing only ---

; --- Ensure that the total mean is exactly zero ---
qp[0]=0.0

; --- Create the quadratic wavenumber array. It should have the same dimensionas as q. ---
if (nqdim eq 1) then begin
  WaveNumber2D=findgen(nx)
  WaveNumber2D[nx/2:nx-1]=reverse(findgen(nx-nx/2)+1.0)
endif else begin
  WaveNumber2D=dist(nx)
endelse

; --- Create vector with wavenumber bins ---
if (WaveDelta ge 0.2) then begin
  ; --- Determine number of wavenumber bins ---
  ;nWaveBin=nx/2
  ;nWaveBin=long(nx/1.415)
  nWaveBin=long(nx/DimFac/(2.0*WaveDelta))
  ;help, nWaveBin
  ;
  ; --- Create vector with wavenumber bins ---
  ;WaveNumber1D    =findgen(nWaveBin)*2.0*WaveDelta+1.0
  WaveNumber1D    =findgen(nWaveBin)*2.0*WaveDelta+1.01-WaveDelta
  ;
  WaveNumber1DDown=(WaveNumber1D-WaveDelta) > 0.0
  WaveNumber1DUp  = WaveNumber1D+WaveDelta
  ;
endif else begin
  WaveNumber1D=exp(2.0D+00*WaveDelta*dindgen(1000))
  ;help, WaveNumber1D & print, minmax(WaveNumber1D)
  WaveNumber1D=float(WaveNumber1D[where(WaveNumber1D le (nx/DimFac))])
  ;help, WaveNumber1D & print, minmax(WaveNumber1D)
  ;
  nWaveBin=n_elements(WaveNumber1D)
  ;
  WaveNumber1DDown=fltarr(nWaveBin)
  WaveNumber1DUp  =fltarr(nWaveBin)
  WaveNumber1DDown[   0]= WaveNumber1D[   0]-WaveDelta
  WaveNumber1DDown[1:-1]=(WaveNumber1D[0:-2]+WaveNumber1D[1:-1]) * 0.5
  WaveNumber1DUp[  0:-2]= WaveNumber1DDown[1:-1]
  WaveNumber1DUp[    -1]= WaveNumber1D[  -1]+ 0.5*(WaveNumber1D[-1]-WaveNumber1D[-2])
  ;
endelse

; ==================================================================================================

qs=fltarr(nWaveBin)

for i=0, nWaveBin-1 do begin
  WaveNumberUp  =WaveNumber1DUp[  i]
  WaveNumberDown=WaveNumber1DDown[i]
  ;
  Ind=where((WaveNumber2D ge (WaveNumberDown)) and $
            (WaveNumber2D lt (WaveNumberUp  ))      , nInd)
  if (nInd gt 0) then begin
    qs[i]=total(qp[Ind])
  endif
endfor ; i


; ==================================================================================================

if (keyword_set(ComputeNorm_flag)) then begin
  ;
  help, nqdim, nx, ny, DimFac
  ;
  qp1=qp*0.0+1.0
  qsn=fltarr(nWaveBin)
  qn =fltarr(nWaveBin)
  qsw=fltarr(nWaveBin)
  qw =fltarr(nWaveBin)
  ;
  for i=0, nWaveBin-1 do begin
    WaveNumberUp  =WaveNumber1DUp[  i]
    WaveNumberDown=WaveNumber1DDown[i]
    ;
    Ind=where((WaveNumber2D ge (WaveNumberDown)) and $
              (WaveNumber2D lt (WaveNumberUp  ))      , nInd)
    if (nInd gt 0) then begin
      qsn[i]=total(qp1[         Ind])
      qsw[i]=total(WaveNumber2D[Ind])
    endif
    ;
  endfor ; i
  ;
  if (nqdim eq 1) then begin
    qsa=replicate(1.0, nWaveBin)
  endif else begin
    qsa=!PI*(WaveNumber1DUp^2-WaveNumber1DDown^2)/(WaveNumber1DUp-WaveNumber1DDown)
  endelse
  ;
  Ind=where(qsn gt 0.0)
  ;qsa=qsa[Ind]
  ;qsn=qsn[Ind]
  ;qsw=qsw[Ind]
  qn[Ind]=qsa[Ind]/qsn[Ind]
  qw[Ind]=qsw[Ind]/qsn[Ind]
  ;
endif ; (ComputeNorm_flag)

end ; power2d1d
