function QGREY, tau, approximation=approximation
;
;------------------------------------------------------------------------------
;+
; NAME:
;   QGREY
;
; PURPOSE:
;   Compute the Hopf function q, necessary to compute the
;   temperature stratification of a grey atmosphere. 
;
; CATEGORY:
;   Computation (Special)
;
; CALLING SEQUENCE:
;   q=qgrey(tau [,approximation=approximation])
;
; INPUTS:
;   tau - Optical depth (>=0)
;
; OUTPUTS:
;   gqrey = Approximate Hopf function, e.g. 0.7104-0.1331*exp(-3.4488*tau)
;           or q for realistic atmosphere (e.g. Holweger-Mueller)
;
; KEYWORDS:
;   approximation: Input string: determines the approximation:
;           'unsoeld': (Unsoeld 1968, Eq. (40,3), p. 145), default
;           'eddington': q=2/3
;           'singleray': q=1
;           'chandrasekhar1': Chandrasekhar, n=1, (Mihalas 1970, p. 52)
;           'chandrasekhar2': Chandrasekhar, n=2
;           'chandrasekhar3': Chandrasekhar, n=3
;           'chandrasekhar4': Chandrasekhar, n=4
;           'mihalas': Exact solution interpolated from table
;                      (Mihalas, 'Stellar Atmospheres', Table 3-2)
;           'hdfit':   Fit to RHD model 'zt142g80n1'
;           'morelga': Morel, Formula (3) in Gabriel 1996, A&A 309, 939
;           'holmu':   Holweger-Mueller, Table II in Holweger & Muller (1974)
;                      Solar Physics 39, 19-30
;           'holmuga': Holweger-Mueller, Formula (4) in Gabriel 1996
;           'krishna-swamy': Krishna Swamy, 1966, ApJ 145, 174
;           'hdconv_0': Mean T(tau)-relation derived by averaging RHD radiative
;                       flux profiles, computing T(tau) and fitting q(tau),
;                       T(tau) for tau>1 -> T(tau)_radiative
;           'hdconv_1': Mean T(tau)-relation derived by averaging RHD radiative
;                       flux profiles, computing T(tau) and fitting q(tau),
;                       T(tau) for tau>>1 -> T(tau)_radiative
;           'hdconv_3': T(tau)-relation dervied from t65g44i1a15n5.ttau
;                       LHD-Code
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   monocubic - Monotonic cubic interpolation
;               (If not available use INTERPOL instead)
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The input variable tau should be not less than 0.
;
; PROCEDURE:
;   Straightforward.
;
; EXAMPLE:
;   tau=exp(findgen(100)/8.0-8.0)
;   q=qgrey(tau)
;   plot_oi, tau, q
;
; MODIFICATION HISTORY:
;   27.10.94 Written by Bernd Freytag, Astrophysik Kiel
;   31.10.94 (B.F.) Several different approximations introduced
;    3. 3.95 (B.F.) Tabulated exact solution from Mihalas
;    8. 3.95 (B.F.) Analytical fit to RHD stratification
;    3. 6.96 (B.F.) 'morelga' and 'holmuga' added
;   23.10.96 (B.F.) 'krishna-swamy' added
;   25.10.96 (B.F.) 'hdconv_0,1' added
;   30.10.96 (B.F.) unknown approximation -> q=0.0; 'holmu' added
;   13.11.96 (B.F.) 'hdconv_2' added
;   11. 5.99 (B.F. Copenhagen) 'singleray' added
;    4. 6.14 (B.F. Uppsala) 'holmulimit' added
;-
;------------------------------------------------------------------------------
;
; --- Default approximation ---
if (not keyword_set(approximation)) then app='unsoeld' $
                                    else app=strlowcase(approximation)
;
; --- Compute q_qrey according to the selected approximation ---
if (app eq 'unsoeld') then begin
  q=0.7104 - 0.1331*exp(-3.4488*tau)
  ;
endif else if (app eq 'eddington') then begin
  q=tau*0.0+2.0/3.0
  ;
endif else if (app eq 'singleray') then begin
  q=tau*0.0+1.0
  ;
endif else if (app eq 'chandrasekhar1') then begin
  q=tau*0.0+1.0/sqrt(3.0)
endif else if (app eq 'chandrasekhar2') then begin
  q=0.694025 - 0.116675*exp(-1.97023*tau)
endif else if (app eq 'chandrasekhar3') then begin
  q=0.703899 - 0.101245*exp(-3.20295*tau) - 0.02530*exp(-1.22521*tau)
endif else if (app eq 'chandrasekhar4') then begin
  q=0.70692 - 0.08392*exp(-4.45808*tau) -0.03619*exp(-1.59178*tau) - $
              0.00946*exp(-1.10319*tau)
  ;
endif else if (app eq 'mihalas') then begin
  tau0=[0.00, 0.01, 0.03, 0.05, 0.10, $
        0.20, 0.30, 0.40, 0.50, 0.60, $
        0.80, 1.00, 1.50, 2.00, 2.50, $
        3.00, 3.50, 4.00, 5.00, 50.0, 1e4, 1e8]
  q0=[0.577351, 0.588236, 0.601242, 0.610758, 0.627919, $
      0.649550, 0.663365, 0.673090, 0.680240, 0.685801, $
      0.693534, 0.698540, 0.705130, 0.707916, 0.709191, $
      0.709806, 0.710120, 0.710270, 0.710398, 0.710446, 0.710446, 0.710446]
  q=monocubic(q0,tau0,tau)
;  q=interpol(q0,tau0,tau) ; --- Simple linear interpolation ---
;  q=spline(tau0,q0,tau,500.0) ; --- Spline interpolation (errors) ---
  ;
endif else if (app eq 'hdfit') then begin
  q=1.0/sqrt(3.0) + 0.5*1.13* (-1.0/sqrt(3.0) + $
      0.694025 - 0.116675*exp(-1.97023*tau) ) $
                  + 0.5*0.93* (-1.0/sqrt(3.0) + $
      0.703899 - 0.101245*exp(-3.20295*tau) - 0.02530*exp(-1.22521*tau) )
;
endif else if (app eq 'morelga') then begin
  q=0.8965285 - 0.1111744*exp(-47125.54*tau) - $
                0.1343378*exp(-1249.336*tau) - $
                0.1405484*exp(-50.42235*tau) - $
                0.2342407*exp(-2.006144*tau)
;
endif else if (app eq 'holmu') then begin
  tau0=[0.0000, 2.0e-7, 2.5e-7, 5.0e-7, 1.0e-6, $
        2.0e-6, 5.0e-6, 1.0e-5, 2.0e-5, 5.0e-5, $
        1.0e-4, 2.0e-4, 5.0e-4, 0.001,  0.002,  $
        0.005, 0.01, 0.02, 0.05, 0.10, $
        0.25, 0.40, 0.70, 1.0, 1.5, $
        2.0, 4.0, 6.0, 8.0, 10.0, $
        1e2, 1e4, 1e8]
  q0=[0.27637, 0.27637, 0.28208, 0.29675, 0.31510, $
      0.33103, 0.35776, 0.37527, 0.39712, 0.42377, $
      0.44765, 0.47248, 0.50256, 0.52925, 0.55173, $
      0.58792, 0.62415, 0.66867, 0.74558, 0.81447, $
      0.89163, 0.99911, 1.13453, 1.22581, 1.17659, $
      1.13964, 0.70033, -0.46595, -1.99552, -3.76404, $ 
      -3.76404, -3.76404, -3.76404]
  q=monocubic(q0,tau0,tau)
  ;
endif else if (app eq 'holmulimit') then begin
  tau0=[0.0000, 2.0e-7, 2.5e-7, 5.0e-7, 1.0e-6, $
        2.0e-6, 5.0e-6, 1.0e-5, 2.0e-5, 5.0e-5, $
        1.0e-4, 2.0e-4, 5.0e-4, 0.001,  0.002,  $
        0.005, 0.01, 0.02, 0.05, 0.10, $
        0.25, 0.40, 0.70, 1.0, 1.5, $
        2.0, 4.0, 6.0, 8.0, 10.0, $
        1e2, 1e4, 1e8]
  q0=[0.25, 0.27637, 0.28208, 0.29675, 0.31510, $
      0.33103, 0.35776, 0.37527, 0.39712, 0.42377, $
      0.44765, 0.47248, 0.50256, 0.52925, 0.55173, $
      0.58792, 0.62415, 0.66867, 0.74558, 0.81447, $
      0.89163, 0.99911, 1.13453, 1.22581, 1.17659, $
      1.13964, 0.70033, 2.0/3.0, 2.0/3.0, 2.0/3.0, $ 
      2.0/3.0, 2.0/3.0, 2.0/3.0]
  q=monocubic(q0,tau0,tau)
  ;
endif else if (app eq 'holmuga') then begin
  q=1.467296 - 0.1035854*exp(-103585.4*tau) - $
               0.1202583*exp(-2767.498*tau) - $
               0.1910047*exp(-52.93280*tau) - $
               0.7451550*exp(-1.118261*tau)
;
endif else if (app eq 'krishna-swamy') then begin
  q=1.017     - 0.3   * exp( -2.54*tau)- $
                0.291 * exp(-30.0 *tau)
;
endif else if (app eq 'hdconv_0') then begin
  q=0.738957 - 0.166399*exp(-8.22738*tau) - $
               0.144174*exp(-125.372*tau)
;
endif else if (app eq 'hdconv_1') then begin
  q=0.540158 - 0.171217*exp(-96.8027 *tau) + $
               2.69589 *exp(-1.00388 *tau) - $
               1.87406 *exp(-1.59187 *tau) - $
               0.76060 *exp(-0.543346*tau)
;
endif else if (app eq 'hdconv_2') then begin
  taufac=2216.53
  q= (-0.620963 + (0.956230-0.0697092 * atan(-563.312 + 773652.*tau/taufac)) $
                                  * atan( 6.41178 + 2.67254e+06*tau/taufac)) $
               - 0.447817 * exp(-1995.72 * tau/taufac)
endif else if (app eq 'hdconv_3') then begin
  taufac=7963.50
  q=0.809247 - 0.107884*exp(-844988.    *tau/taufac) - $
               4.19934 *exp(-5156.40    *tau/taufac) - $
               0.123591*exp(-1.35702e+07*tau/taufac) + $
               6.50619 *exp(-3403.49    *tau/taufac) - $
               2.54480 *exp(-2026.98    *tau/taufac) 
endif else begin ; --- Default ---
  q=0.0*tau
endelse
;
return, q
;
end ; QGREY
