pro rhd_seq_v3stat, basDB, seqDB, indModelList, v3DB
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq_v3stat ('rhd_sequence_velocity_3_statistics')
;
; PURPOSE:
;    statistics derived from average of sequence data
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;
; INPUT:
;   basDB:        ('base_Data_Base') UDB data base with basic model data
;   seqDB:        ('sequence_Data_Base') UDB data base with averaged (1D) stratifications
;   indModelList: ('index_Model_List') integer vector with indices of models in seqDB
;
; OUTPUT:
;   v3DB:         ('velocity_3_Data_Base') output UDB data base (vector of structures) 
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; TOOLS:
;   Ho=v3DB[*].H_HV3OHP1 < v3DB[*].H_HV3OHP2
;   Hw=-(alog10(v3DB[*].P_v3rmsmaxwave)-alog10(v3DB[*].P_v3rmsmin))/(alog10(v3DB[*].v3rms_v3rmsmaxwave)-alog10(v3DB[*].v3rms_v3rmsmin))
;
; MODIFICATION HISTORY:
;   2010-11-26 (B.F. Lyon) From rhd_seq_v3statextract
;-
;---------------------------------------------------------------------------------------------------

DateAdded=systime(/julian)

; --- Default smoothing parameter for stratifications
qsmooth=0

; --- Number of models ---
ModelList=udb_ListAll(seqDB)
nMod=n_elements(ModelList[indModelList])

; --- Loop over all selected models ---
for iMod=0,nMod-1 do begin
  seq=*seqDB[indModelList[iMod]]
  ;print, iMod, seq.model, format='(I4,2X,A)'
  ;
  ; --- Extract parameters ---
  Teff      =      (udb_extract('Teffout'  , [seq.model], basDB))[0]
  grav      =      (udb_extract('grav'     , [seq.model], basDB))[0]
  dimension =      (udb_extract('dimension', [seq.model], basDB))[0]
  ;
  ; --- Extract vectors ---
  v1rms =seqavg('v1rms', seq, unitsystem='cgs', uniteflux='cgs', qsmooth=2, /intertoxc)
  v3rms =seqavg('v3rms', seq, unitsystem='cgs', uniteflux='cgs', qsmooth=2, /intertoxc)
  P     =seqavg('P'    , seq, unitsystem='cgs', uniteflux='cgs', qsmooth=qsmooth, /intertoxc)
  s     =seqavg('s'    , seq, unitsystem='cgs', uniteflux='cgs', qsmooth=qsmooth, /intertoxc)
  cs    =seqavg('cs'   , seq, unitsystem='cgs', uniteflux='cgs', qsmooth=qsmooth, /intertoxc)
  tau   =seqavg('tau'  , seq, unitsystem='cgs', uniteflux='cgs', qsmooth=qsmooth, /intertoxc)
  logtau=alog10(tau)
  ;Hv3oHp=(deriv(alog(v3rms),alog(P)))
  HpoHv3=(deriv(alog(P),alog(v3rms)))
  ;
  ;seqavgstat, q2a, q2b, q2c, q2d, P, q, q2aqcmd,q2afcmd,q2apcmd, q2bqcmd,q2bfcmd,q2bpcmd, q2cqcmd,q2cfcmd, q2dqcmd,q2dfcmd,q2dpcmd,q2drcmd
  ;
  ; --- Velocity at fixed optical depths ---
;  seqavgstat, logtau, v1rms, v1rms, logtau, P, v1rms_logtaum0, 'log10tau','q+0.0','zero',  'v1rms','','',  '','',  '','','',''
;  seqavgstat, logtau, v1rms, v1rms, logtau, P, v1rms_logtaum1, 'log10tau','q+1.0','zero',  'v1rms','','',  '','',  '','','',''
  seqavgstat, logtau, P    , P    , logtau, P, P_logtaum0    , 'log10tau','q+0.0','zero',  'P'    ,'','',  '','',  '','','',''
  seqavgstat, logtau, v3rms, v3rms, logtau, P, v3rms_logtaum0, 'log10tau','q+0.0','zero',  'v3rms','','',  '','',  '','','',''
  seqavgstat, logtau, cs   , cs   , logtau, P, cs_logtaum0   , 'log10tau','q+0.0','zero',  'cs'   ,'','',  '','',  '','','',''
  seqavgstat, logtau, P    , P    , logtau, P, P_logtaum1    , 'log10tau','q+1.0','zero',  'P'    ,'','',  '','',  '','','',''
  seqavgstat, logtau, v3rms, v3rms, logtau, P, v3rms_logtaum1, 'log10tau','q+1.0','zero',  'v3rms','','',  '','',  '','','',''
  seqavgstat, logtau, cs   , cs   , logtau, P, cs_logtaum1   , 'log10tau','q+1.0','zero',  'cs'   ,'','',  '','',  '','','',''
  ;
  ; --- Maximum convective velocity ---
  seqavgstat, v3rms, P     , P     , s, P, P_v3rmsmaxconv     , 'v3rms','','maxtop1',  'P'    ,'','',  '','',  's','','min','below'
  seqavgstat, v3rms, v3rms , v3rms , s, P, v3rms_v3rmsmaxconv , 'v3rms','','maxtop1',  'v3rms','','',  '','',  's','','min','below'
  seqavgstat, v3rms, cs    , cs    , s, P, cs_v3rmsmaxconv    , 'v3rms','','maxtop1',  'cs'   ,'','',  '','',  's','','min','below'
  ;seqavgstat, v3rms, P     , P     , s, P, P_v3rmsmaxconvtmp  , 'v3rms','','maxtop1',  'P'    ,'','',  '','',  's','','min','below'
  ;seqavgstat, v3rms, P     , P     , P, P, P_v3rmsmaxconv     , 'v3rms','','max'    ,  'P'    ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmaxconvtmp),2),'zero','below+1'
  ;seqavgstat, v3rms, v3rms , v3rms , P, P, v3rms_v3rmsmaxconv , 'v3rms','','max'    ,  'v3rms','','',  '','',  'P','q-'+strtrim(string(P_v3rmsmaxconvtmp),2),'zero','below+1'
  ;
  ; --- Minimum velocity ---
  seqavgstat, v3rms, P     , P     , P, P, P_v3rmsmin          , 'v3rms','','minbot1',  'P'     ,'','',  '','',  'P', 'q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  seqavgstat, v3rms, v3rms , P     , P, P, v3rms_v3rmsmin      , 'v3rms','','minbot1',  'v3rms' ,'','',  '','',  'P', 'q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  seqavgstat, v3rms, cs    , P     , P, P, cs_v3rmsmin         , 'v3rms','','minbot1',  'cs'    ,'','',  '','',  'P', 'q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  ;
  ; --- Maximum wave velocity ---
  seqavgstat, v3rms, P     , P     , P, P, P_v3rmsmaxwave      , 'v3rms','','max'    ,  'P'     ,'','',  '','',  'P', 'q-'+strtrim(string(P_v3rmsmin    ),2),'zero','above'
  seqavgstat, v3rms, v3rms , P     , P, P, v3rms_v3rmsmaxwave  , 'v3rms','','max'    ,  'v3rms' ,'','',  '','',  'P', 'q-'+strtrim(string(P_v3rmsmin    ),2),'zero','above'
  ;
  ; --- Negative scale height of wave amplitude growth ---
  H_dPodv3rms=-(alog10(P_v3rmsmaxwave)-alog10(P_v3rmsmin))/(alog10(v3rms_v3rmsmaxwave)-alog10(v3rms_v3rmsmin))
  ; --- Velocity difference at maximum convective velocity pressure ---
  rv3rms_v3rmsmaxconv=10.0^(-(alog10(v3rms_v3rmsmaxconv)-(alog10(v3rms_v3rmsmin)-(alog10(P_v3rmsmaxconv)-alog10(P_v3rmsmin))/H_dPodv3rms)))
  ;
  ; --- Turning point in velocity -> overshoot scale height ---
  ;seqavgstat, Hv3oHp, Hv3oHp, Hv3oHp, s, P, H_Hv3oHp    , 'Hv3oHp','','minbot1',  'Hv3oHp','','',  '','',  's','','min','above-20'
  ;seqavgstat, Hv3oHp, P     , P     , s, P, P_Hv3oHp    , 'Hv3oHp','','minbot1',  'P'     ,'','',  '','',  's','','min','above-20'
  ;seqavgstat, Hv3oHp, v3rms , P     , s, P, v3rms_Hv3oHp, 'Hv3oHp','','minbot1',  'v3rms' ,'','',  '','',  's','','min','above-20'
  ;seqavgstat, HpoHv3, HpoHv3, HpoHv3, s, P, H_HpoHv3    , 'HpoHv3','','maxbot1',  'HpoHv3','','',  '','',  's','','min','above-30'
  ;seqavgstat, HpoHv3, P     , P     , s, P, P_Hv3oHp    , 'HpoHv3','','maxbot1',  'P'     ,'','',  '','',  's','','min','above-30'
  ;seqavgstat, HpoHv3, v3rms , P     , s, P, v3rms_Hv3oHp, 'HpoHv3','','maxbot1',  'v3rms' ,'','',  '','',  's','','min','above-30'
  seqavgstat, HpoHv3, P     , P     , P, P, P_Hv3oHp1    , 'HpoHv3','','maxbot1',  'P'     ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  seqavgstat, HpoHv3, v3rms , P     , P, P, v3rms_Hv3oHp1, 'HpoHv3','','maxbot1',  'v3rms' ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  seqavgstat, HpoHv3, HpoHv3, HpoHv3, P, P, H_HpoHv31    , 'HpoHv3','','maxbot1',  'HpoHv3','','',  '','',  'P','q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  seqavgstat, HpoHv3, cs    , P     , P, P, cs_Hv3oHp1   , 'HpoHv3','','maxbot1',  'cs'    ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmaxconv),2),'zero','above'
  H_Hv3oHp1=1.0/H_HpoHv31
  seqavgstat, HpoHv3, P     , P     , P, P, P_Hv3oHp2    , 'HpoHv3','','maxtop1',  'P'     ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmin),2),'zero','below'
  seqavgstat, HpoHv3, v3rms , P     , P, P, v3rms_Hv3oHp2, 'HpoHv3','','maxtop1',  'v3rms' ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmin),2),'zero','below'
  seqavgstat, HpoHv3, HpoHv3, HpoHv3, P, P, H_HpoHv32    , 'HpoHv3','','maxtop1',  'HpoHv3','','',  '','',  'P','q-'+strtrim(string(P_v3rmsmin),2),'zero','below'
  seqavgstat, HpoHv3, cs    , P     , P, P, cs_Hv3oHp2   , 'HpoHv3','','maxtop1',  'cs'    ,'','',  '','',  'P','q-'+strtrim(string(P_v3rmsmin),2),'zero','below'
  H_Hv3oHp2=1.0/H_HpoHv32
  ;
  print, iMod, seq.model, v3rms_v3rmsmaxconv, H_Hv3oHp1, H_Hv3oHp2, H_dPodv3rms, rv3rms_v3rmsmaxconv, format='(I4,2X,A20,5(2X,E12.5))'
  ;
  plotlines, alog10(P), alog10(v3rms > 1e1), title=seq.model
  ;plotlines, alog10(P), -(alog10(P)-alog10(P_v3rmsmin))/H_dPodv3rms+alog10(v3rms_v3rmsmin), /over
  plotlines, alog10(P), -(alog10(P)-alog10(P_v3rmsmaxconv))/H_dPodv3rms+alog10(v3rms_v3rmsmaxconv*rv3rms_v3rmsmaxconv), /over
  plotlines, [alog10(P_v3rmsmaxconv)],[alog10(v3rms_v3rmsmaxconv*rv3rms_v3rmsmaxconv)], psym=6, linecolor=!ctv.red  , /over
  plotlines, [alog10(P_Hv3oHp1     )],[alog10(v3rms_Hv3oHp1                         )], psym=1,                       /over
  plotlines, [alog10(P_Hv3oHp2     )],[alog10(v3rms_Hv3oHp2                         )], psym=6, linecolor=!ctv.gray , /over
  plotlines, [alog10(P_v3rmsmaxconv)],[alog10(v3rms_v3rmsmaxconv                    )], psym=1, linecolor=!ctv.red  , /over
  plotlines, [alog10(P_v3rmsmin    )],[alog10(v3rms_v3rmsmin                        )], psym=1, linecolor=!ctv.blue , /over
  plotlines, [alog10(P_v3rmsmaxwave)],[alog10(v3rms_v3rmsmaxwave                    )], psym=1, linecolor=!ctv.green, /over
  plotlines, [alog10(P_logtaum0    )],[alog10(v3rms_logtaum0                        )], psym=2, linecolor=!ctv.red , /over
  ;wait, 1.5
  Dummy=get_kbrd()
  ;
  ResLine={ $
    model                   : seq.model, $
    Teff                    : Teff, $
    Teff__name              : 'Effective temperature', $
    Teff__unit              : 'K', $
    grav                    : grav, $
    grav__name              : 'Gravity', $
    grav__unit              : 'cm/s^2', $
    dimension               : dimension, $
    dimension__name         : 'Box dimension', $
    DateAdded               : DateAdded, $
    DateAdded__unit         : 'd', $
    ;
    P_Pmax                  : max(P), $
    P_Pmax__name            : 'Maximum pressure', $
    P_Pmax__unit            : 'dyn/cm^2', $
    P_Pmin                  : min(P), $
    P_Pmin__name            : 'Minimum pressure', $
    P_Pmin__unit            : 'dyn/cm^2', $
    ;
    P_logtaum1              : P_logtaum1, $
    P_logtaum1__name        : 'Pressure at log(tau_Ross)=-1', $
    P_logtaum1__unit        : 'dyn/cm^2', $
    v3rms_logtaum1          : v3rms_logtaum1, $
    v3rms_logtaum1__name    : 'RMS vertical velocity at log(tau_Ross)=-1', $
    v3rms_logtaum1__unit    : 'cm/s', $
    cs_logtaum1             : cs_logtaum1, $
    cs_logtaum1__name       : 'Sound speed at log(tau_Ross)=-1', $
    cs_logtaum1__unit       : 'cm/s', $
    P_logtaum0              : P_logtaum0, $
    P_logtaum0__name        : 'Pressure at log(tau_Ross)=0', $
    P_logtaum0__unit        : 'dyn/cm^2', $
    v3rms_logtaum0          : v3rms_logtaum0, $
    v3rms_logtaum0__name    : 'RMS vertical velocity at log(tau_Ross)=0', $
    v3rms_logtaum0__unit    : 'cm/s', $
    cs_logtaum0             : cs_logtaum0, $
    cs_logtaum0__name       : 'Sound speed at log(tau_Ross)=0', $
    cs_logtaum0__unit       : 'cm/s', $
    ;
    P_v3rmsmaxconv          : P_v3rmsmaxconv, $
    P_v3rmsmaxconv__name    : 'Pressure at maximum convective velocity', $
    P_v3rmsmaxconv__unit    : 'dyn/cm^2', $
    v3rms_v3rmsmaxconv      : v3rms_v3rmsmaxconv, $
    v3rms_v3rmsmaxconv__name: 'Maximum convective velocity', $, $
    v3rms_v3rmsmaxconv__unit: 'cm/s', $, $
    cs_v3rmsmaxconv         : cs_v3rmsmaxconv, $
    cs_v3rmsmaxconv__name   : 'Sound speed at maximum convective velocity', $
    cs_v3rmsmaxconv__unit   : 'cm/s', $
    ;
    P_Hv3oHp1               : P_Hv3oHp1, $
    P_Hv3oHp1__name         : 'Pressure at lower turning point', $
    P_Hv3oHp1__unit         : 'dyn/cm^2', $
    v3rms_Hv3oHp1           : v3rms_Hv3oHp1, $
    v3rms_Hv3oHp1__name     : 'Vertical velocity at lower turning point', $
    v3rms_Hv3oHp1__unit     : 'cm/s', $
    cs_Hv3oHp1              : cs_Hv3oHp1, $
    cs_Hv3oHp1__name        : 'Sound speed at lower turning point', $
    cs_Hv3oHp1__unit        : 'cm/s', $
    H_Hv3oHp1               : H_Hv3oHp1, $
    H_Hv3oHp1__name         : 'Vertical velocity scale height at lower turning point', $
    H_Hv3oHp1__unit         : '1', $
    ;
    P_Hv3oHp2               : P_Hv3oHp2, $
    P_Hv3oHp2__name         : 'Pressure at upper turning point', $
    P_Hv3oHp2__unit         : 'dyn/cm^2', $
    v3rms_Hv3oHp2           : v3rms_Hv3oHp2, $
    v3rms_Hv3oHp2__name     : 'Vertical velocity at upper turning point', $
    v3rms_Hv3oHp2__unit     : 'cm/s', $
    cs_Hv3oHp2              : cs_Hv3oHp2, $
    cs_Hv3oHp2__name        : 'Sound speed at upper turning point', $
    cs_Hv3oHp2__unit        : 'cm/s', $
    H_Hv3oHp2               : H_Hv3oHp2, $
    H_Hv3oHp2__name         : 'Vertical velocity scale height at upper turning point', $
    H_Hv3oHp2__unit         : '1', $
    ;
    P_v3rmsmin              : P_v3rmsmin, $
    P_v3rmsmin__name        : 'Pressure at vertical velocity minimum', $
    P_v3rmsmin__unit        : 'dyn/cm^2', $
    v3rms_v3rmsmin          : v3rms_v3rmsmin, $
    v3rms_v3rmsmin__name    : 'Vertical velocity minimum', $
    v3rms_v3rmsmin__unit    : 'cm/s', $
    cs_v3rmsmin             : cs_v3rmsmin, $
    cs_v3rmsmin__name       : 'Sound speed at vertical velocity minimum', $
    ;
    P_v3rmsmaxwave          : P_v3rmsmaxwave, $
    P_v3rmsmaxwave__name    : 'Pressure at atmospheric velocity maximum', $
    P_v3rmsmaxwave__unit    : 'dyn/cm^2', $
    v3rms_v3rmsmaxwave      : v3rms_v3rmsmaxwave, $
    v3rms_v3rmsmaxwave__name: 'Amospheric velocity maximum', $
    v3rms_v3rmsmaxwave__unit: 'cm/s', $
    ;
    H_dPodv3rms             : H_dPodv3rms, $
    H_dPodv3rms__name       : 'Atmospheric velocity scale height', $
    H_dPodv3rms__unit       : '1', $
    rv3rms_v3rmsmaxconv      : rv3rms_v3rmsmaxconv, $
    rv3rms_v3rmsmaxconv__name: 'Velocity ratio at maximum convective velocity', $
    rv3rms_v3rmsmaxconv__unit: '1' $
    }
  if (iMod eq 0) then v3DB=ResLine else v3DB=[v3DB, ResLine]
  ;
endfor

end ; rhd_seq_v3stat
