function seqavg, command, seq, par=par0, $
  mtime=mtime, ntime=ntime, qavg=qavg0, qsmooth=qsmooth, $
  unitsystem=unitsystem0, uniteflux=uniteflux0, $
  verbose=verbose, intertoxc=intertoxc, $
  qname=qname, qunit=qunit, qtitle=qtitle, qraw=qraw, qmeansub=qmeansub
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   seqavg ('sequence_average')
;
; PURPOSE:
;   Compute averages from sequence data.
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   q=seqavg(command, seq, par=par, &
;            mtime=mtime, ntime=ntime, qavg=qavg0, qsmooth=qsmooth, $
;            unitsystem=unitsystem, uniteflux=uniteflux, $
;            verbose=verbose, intertoxc=intertoxc, $
;            qname=qname, qunit=qunit, qtitle=qtitle, qraw=qraw, qmeansub=qmeansub)
;
; INPUT:
;   command: ('command') string with name of quantity to be extracted
;   seq:     ('sequence') structure with data from mean files,
;                         e.g., extracted with rhd_seq_collect or rhd_seqpp_collect
;
; OUTPUT:
;   seqavg: ('sequence_average') extracted (and processed) quantity
;
; KEYWORDS (INPUT):
;   par:      ('parameters') optional structure containing CO5BOLD parameter file,
;             if not part of seq structure
;   mtime:    ('m_time') integer with index of first time step to be considered,
;                default: 0
;   ntime:    ('n_time') integer with index of last time step to be considered,
;                default: last time step in structure
;   qavg:     ('quantity_average') ineger with averaging mode:
;               -1: no averaging, results in output of 2D array
;                0: default: the value most reasonable for selected quantity is chosen automatically
;                1: q=avg(q_array)
;                2: q=sqrt(avg(q_array^2))
;   qsmooth:  ('quantity_smooth') integer, if larger than zero: size of box averaging window
;   qmeansub: ('quantity_mean_sub') subtract temporal mean, output is 2D, qavg gives the type of mean
;   unitsystem: ('unit_system') string to set the unit system:
;               'cgs':          cgs units (useful for further computations)
;               'cgssE9':       cgs units but entropy s in units of 10^9 erg/g/K
;               'km,s':
;               'km,s,cm/s':
;               'Mm,s,km/s':    (default)
;               'Gm,s,km/s':
;               'Rsun,s,km/s':  unit name of length scale='R!DSun!N'
;               'Rsun1,s,km/s': unit name of length scale='R!10!D!Z(2609)!X!N'
;               'Rsun2,s,km/s': unit name of length scale='R' + sunsymbol() + ' '
;   uniteflux: ('unit_energy_flux') string to set the units of fluxes:
;               'cgs':          cgs units (useful for further computations)
;               '1e9cgs':
;               'Teff':         in units of !con.sigma * par.Teff^4 (default)
;               'Frtop':        in units of radiative flux at the top of the model
;
; KEYWORDS (OUTPUT):
;   qname:    ('quantity_name') string with name of quantity
;   qunit:    ('quantity_unit') string with unit of quantity
;   qtitle:   ('quantity_title') string with proper plot title for quantity,
;             made from qname and qunit
;   qraw:     ('quantity_raw') 2D array with with "raw" quantity,
;             after restriction to intervall, before averaging
;
; COMMON BLOCKS:
;   None
;
; GLOBAL VARIABLES:
;   The user defined system variable !con (physical constants) is used.
;
; ROUTINES:
;   seqavg_unitsystem: Get unit factors and names corresponding to choosen unit system
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-09-10 (B.F. Uppsala)
;   2003-12-09 (B.F.) q=alog10(q) -> q=alog10(abs(q))
;   2004-07-21 (B.F.) aarsg_seqavg_data.pro -> seqavg_data.pro
;   2005-06-20 (B.F. LA) RAGE version: keyword unitsystem added
;   2005-09-08 (B.F.) Allow negative mtime
;   2006-01-30 (B.F.) Additional energy fluxes (combinations)
;   2006-06-02 (B.F.) Do averaging in double precision
;   2007-09-03 (B.F. Lyon) Keyword qmeansub added
;   2007-02-25 (B.F.) Arbitrary quc's
;   2008-05-21 (B.F.) seqavg_data -> seqavg; Interpret tag 'BinFactor'
;   2009-10-14 (B.F.) Add compiler options, replace some () by []
;   2009-10-25 (B.F.) Compute xc3 and xb3 once at the beginning
;   2009-11-24 (B.F.) Diffusion coefficients
;   2010-03-15 (B.F. Naples) Bugfix: case intertoxc3 and mavg>navg
;   2010-03-22 (B.F.) Magnetic field quantities
;   2010-03-31 (B.F. Lyon) Average rms magnetic quantities with qavg=2
;   2012-04-25 (B.F., W.S.) New option (r2) for potential
;   2014-02-25 (B.F. Uppsala) Changed definition: 'rhov3': seq.rho*seq.v3 -> seq.rhov3;
;                     Removed some wrong qavgd=2 definitions
;   2015-01-13 (B.F.) fewb
;   2016-04-25 (B.F.) Allow trailing '_rm2'; 'cm/s' -> 'cm s^-1'; for central potential: 'vert' -> 'radial
;   2016-05-16 (B.F.) Change formula for 'v3mass' and 'v3massb'
;   2016-11-01 (B.F.) More comments
;   2016-11-02 (B.F.) Change definition of 'rhov3v3'; add 'frhov3b'
;   2016-11-05 (B.F.) Bugfix: Account for all-capital tag names in axis initialization
;   2016-11-30 (B.F.) Call seperate routine seqavg_unitsystem
;   2016-12-01 (B.F.) rhov3sqrorho
;   2017-11-01 (B.F.) Perform intertoxc interpolation in double precision; compute PradT
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

; --- Take parameter file structure directly from input parameters or from seq structure ---
tags=tag_names(seq)
if (keyword_set(par0)) then begin
  par=par0
endif else begin
  if (total(tags eq 'PAR') gt 0.0) then begin
    par=seq.par
  endif else begin
    par=0
  endelse 
endelse

; --- Get unit names and factors ---
seqavg_unitsystem, unitsystem=unitsystem0, $
  unit_length_value=unit_length_value, $
  unit_length_name =unit_length_name, $
  unit_velocity_value=unit_velocity_value, $
  unit_velocity_name =unit_velocity_name, $
  unit_entropy_value=unit_entropy_value, $
  unit_entropy_name =unit_entropy_name, $
  unit_DiffusionCoefficient_value=unit_DiffusionCoefficient_value, $
  unit_DiffusionCoefficient_name =unit_DiffusionCoefficient_name, $
  unit_MagneticField_value=unit_MagneticField_value, $
  unit_MagneticField_name =unit_MagneticField_name, $
  unit_density_name=unit_density_name

; --- Get unit names and factors if quantity is an energy flux ---
if (keyword_set(uniteflux0)) then begin
  uniteflux=uniteflux0
endif else begin
  uniteflux='Teff'
endelse
if (uniteflux eq 'cgs') then begin
  unit_flux_value=1.0
  unit_flux_name='erg/cm!U2!N/s'
endif else if (uniteflux eq '1e9cgs') then begin
  unit_flux_value=1.0E+09
  unit_flux_name='10!U9!N erg/cm!U2!N/s'
endif else if (uniteflux eq 'Teff') then begin
  unit_flux_value=!con.sigmasb*par.Teff^4
  unit_flux_name='F!D*!N'
endif else if (uniteflux eq 'Frtop') then begin
  fer=seqavg('fer', seq, mtime=mtime, ntime=ntime, uniteflux='cgs')
  unit_flux_value=fer[n_elements(fer)-1]
  unit_flux_name='F!D*,top!N'
endif else begin
  message, 'unitsystem for fluxes not supported'
endelse

; --- Parse command string ---
cm=strlowcase(command)
if (strmid(cm, 0, 6) eq 'log10_') then begin
  cm_log='log10'
  cm=strmid(cm, 6)
endif else if (strmid(cm, 0, 5) eq 'loge_') then begin
  cm_log='loge'
  cm=strmid(cm, 5)
endif else begin
  cm_log=''
endelse
if (strmid(cm, 3, 4, /reverse_offset) eq '_rm2') then begin
  cm_rm2='rm2'
  cm=strmid(cm, 0, strlen(cm)-4)
endif else begin
  cm_rm2=''
endelse
; help, command, cm, cm_rm2

;help, cm, cm_log, (strmid(cm,0,3) eq 'quc') , (long(strmid(cm,3,3)) ne 0L), (strmid(cm,0,4) eq 'nquc'), (long(strmid(cm,4,3)) ne 0L)

; --- Default value ---
qavgd=1

; --- Determine gravity mode ---
if (size(par,/type) eq 8) then begin
  grav_mode=strlowcase(par.grav_mode)
endif else if (total(tags eq 'XC3_RADIAL') gt 0.0) then begin
  grav_mode='central'
endif else begin
  grav_mode='constant'
endelse
if (grav_mode eq 'central') then begin
  if (total(tag_names(par) eq 'R2_GRAV') gt 0.0) then begin
    r2_grav=par.r2_grav
  endif else begin
    r2_grav=0.0
  endelse
endif
;help,  grav_mode, r2_grav

; --- Extract vertical axes (directly or from included mean structure) ---
if (total(tags eq 'XC3') gt 0.0) then begin
  xc3=reform(seq.xc3)
endif else begin
  xc3=reform(seq.mean.z3.xc3)
endelse
if (total(tags eq 'XB3') gt 0.0) then begin
  xb3=reform(seq.xb3)
endif else begin
  xb3=reform(seq.mean.z3.xb3)
endelse

if (cm eq 'am3') then begin
  ; --- Angular momentum 3 ---
  ;
  q=seq.AngularMomentum3
  qname='AngMom3'
  qunit='g/cm/s'
  ;
endif else if (cm eq 'am30') then begin
  ; --- Angular momentum 3 (including contribution from average rotation) ---
  ;
  q=seq.AngularMomentum3 + 2.0*!PI*seq.par.nu_rotation*seq.MomentOfInertia3
  qname='AngMom3'
  qunit='g/cm/s'
  ;
endif else if (cm eq 'omegarot3') then begin
  ; --- Rotational omega 3 ---
  ;
  q       =seq.AngularMomentum3*0.0
  q[1:*,*]=seq.AngularMomentum3[1:*,*]/seq.MomentOfInertia3[1:*,*]
  q[0,*]  =q[1,*]
  qname='omega!Drot,3!N'
  qunit='s!U-1!N'
  ;
endif else if (cm eq 'omegarot30') then begin
  ; --- Rotational omega 3 (including contribution from average rotation) ---
  ;
  q       =seq.AngularMomentum3*0.0
  q[1:*,*]=seq.AngularMomentum3[1:*,*]/seq.MomentOfInertia3[1:*,*] + 2.0*!PI*seq.par.nu_rotation
  q[0,*]  =q[1,*]
  qname='omega!Drot,3!N'
  qunit='s!U-1!N'
  ;
endif else if (cm eq 'bc1') then begin
  ; --- Horizontal magnetic field 1 ---
  ;
  q=seq.bc1/unit_MagneticField_value
  qname='Bc!D1!N'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'bc1rms') then begin
  ; --- Horizontal magnetic field 1 ---
  ;
  q=seq.bc1rms/unit_MagneticField_value
  qname='Bc!D1,rms!N'
  qunit=unit_MagneticField_name
  qavgd=2
  ;
endif else if (cm eq 'bc1drms') then begin
  ; --- Horizontal magnetic field 1, fluctuations ---
  ;
  q=sqrt(abs(seq.bc1rms^2-seq.bc1^2))/unit_MagneticField_value
  qname='Bc!D1,drms!N'
  qunit=unit_MagneticField_name
  qavgd=2
  ;
endif else if (cm eq 'bc1abs') then begin
  ; --- Horizontal magnetic field 1 ---
  ;
  q=seq.bc1abs/unit_MagneticField_value
  qname='Bc!D1,abs!N'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'bc2') then begin
  ; --- Horizontal magnetic field 2 ---
  ;
  q=seq.bc2/unit_MagneticField_value
  qname='Bc!D2!N'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'bc2rms') then begin
  ; --- Horizontal magnetic field 2 ---
  ;
  q=seq.bc2rms/unit_MagneticField_value
  qname='Bc!D2,rms!N'
  qunit=unit_MagneticField_name
  qavgd=2
  ;
endif else if (cm eq 'bc2drms') then begin
  ; --- Horizontal magnetic field 2, fluctuations ---
  ;
  q=sqrt(abs(seq.bc2rms^2-seq.bc2^2))/unit_MagneticField_value
  qname='Bc!D2,drms!N'
  qunit=unit_MagneticField_name
  qavgd=2
  ;
endif else if (cm eq 'bc2abs') then begin
  ; --- Horizontal magnetic field 2 ---
  ;
  q=seq.bc2abs/unit_MagneticField_value
  qname='Bc!D2,abs!N'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'bc3') then begin
  ; --- Vertical magnetic field 3 ---
  ;
  q=seq.bc3/unit_MagneticField_value
  qname='Bc!D3!N'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'bc3rms') then begin
  ; --- Vertical magnetic field 3 ---
  ;
  q=seq.bc3rms/unit_MagneticField_value
  qname='Bc!D3,rms!N'
  qunit=unit_MagneticField_name
  qavgd=2
  ;
endif else if (cm eq 'bc3drms') then begin
  ; --- Vertical magnetic field 3, fluctuations ---
  ;
  q=sqrt(abs(seq.bc3rms^2-seq.bc3^2))/unit_MagneticField_value
  qname='Bc!D3,drms!N'
  qunit=unit_MagneticField_name
  qavgd=2
  ;
endif else if (cm eq 'bc3abs') then begin
  ; --- Vertical magnetic field 3 ---
  ;
  q=seq.bc3abs/unit_MagneticField_value
  qname='Bc!D3,abs!N'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'intrhoem') then begin
  ; --- Height-integrated magnetic energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  ;dq=0.5*(4.0*!PI)*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)*dx
  dq=0.5*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='Int(!4q!Xebdz)'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'em') then begin
  ; --- Magnetic energy per mass ---
  ;
  if (total(tags eq 'BC1RMS') gt 0.0) then begin
    ;q=0.5*(4.0*!PI)*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)/seq.rho
    q=0.5*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)/seq.rho
  endif else begin
    ; --- No magnetic field data present ---
    q=seq.rho*0.0
  endelse
  qname='eb'
  qunit='erg/g'
  ;
endif else if (cm eq 'rhoem') then begin
  ; --- Magnetic energy per mass ---
  ;
  if (total(tags eq 'BC1RMS') gt 0.0) then begin
    ;q=0.5*(4.0*!PI)*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)
    q=0.5*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)
  endif else begin
    ; --- No magnetic field data present ---
    q=seq.rho*0.0
  endelse
  qname='eb'
  qunit='erg/cm!U3!N'
  ;
endif else if (cm eq 'bcrms') then begin
  ; --- Magnetic field ---
  ;
  if (total(tags eq 'BC1RMS') gt 0.0) then begin
    q=sqrt((seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2))/unit_MagneticField_value
  endif else begin
    ; --- No magnetic field data present ---
    q=seq.rho*0.0
  endelse
  qname='b'
  qunit=unit_MagneticField_name
  ;
endif else if (cm eq 'pmag') then begin
  ; --- Magnetic pressure ---
  ;
  if (total(tags eq 'BC1RMS') gt 0.0) then begin
    q=0.5*(seq.Bc1rms^2+seq.Bc2rms^2+seq.Bc3rms^2)
  endif else begin
    ; --- No magnetic field data present ---
    q=seq.rho*0.0
  endelse
  qname='P!Dmag!N'
  qunit='dyn/cm!U2!N'
  ;
endif else if ((cm eq 'c') or (cm eq 'cs') or (cm eq 'csound')) then begin
  ; --- Adiabatic sound speed ---
  ;
  q=sqrt(seq.Gamma1*seq.P/seq.rho)/unit_velocity_value
  qname='c!Dsound!N'
  qunit=unit_velocity_name
  ;
endif else if (cm eq 'cp') then begin
  ; --- Heat capacity ---
  ;
  q=seq.delta*seq.Gamma1*seq.P / ((seq.Gamma3-1.0)*seq.rho*seq.T)
  qname='c!Dp!N'
  qunit='erg/K/g'
  ;
endif else if (cm eq 'cv') then begin
  ; --- Heat capacity ---
  ;
  q=seq.delta*seq.Gamma1*seq.P / ((seq.Gamma3-1.0)*seq.rho*seq.T *(1.0+seq.delta*(seq.Gamma3-1.0)))
  qname='c!Dv!N'
  qunit='erg/K/g'
  ;
endif else if (cm eq 'cvrho') then begin
  ; --- Heat capacity ---
  ;
  q=seq.delta*seq.Gamma1*seq.P / ((seq.Gamma3-1.0)        *seq.T *(1.0+seq.delta*(seq.Gamma3-1.0)))
  qname='c!Dv!N'
  qunit='erg/K/cm!U3!N'
  ;
endif else if (cm eq 'delta') then begin
  ; --- delta ---
  ;
  q=seq.delta
  qname='!4d!X'
  qunit=''
  ;
endif else if (cm eq 'diffv3rmshp') then begin
  ; --- Diffusion coefficient: v * H_p ---
  ;
  q=seq.v3rms * ((seq.P)/(par.grav*seq.rho))
  qname='D'
  qunit=unit_DiffusionCoefficient_name
  ;
endif else if (cm eq 'diffv3rms2timeac') then begin
  ; --- Diffusion coefficient: v^2 * time_ac ---
  ;
  q=seq.v3rms^2 * (2.0*!pi/(par.grav*0.5*sqrt(seq.Gamma1*seq.rho/seq.P)))
  qname='D'
  qunit=unit_DiffusionCoefficient_name
  ;
endif else if (cm eq 'diffv3rms2timeacmach') then begin
  ; --- Diffusion coefficient: v^2 * time_ac * Ma ---
  ;
  q=seq.v3rms^2 * (2.0*!pi/(par.grav*0.5*sqrt(seq.Gamma1*seq.rho/seq.P))) * $
    (seq.v3rms/sqrt(seq.Gamma1*seq.P/seq.rho))
  qname='D'
  qunit=unit_DiffusionCoefficient_name
  ;
endif else if (cm eq 'dtau') then begin
  ; --- Delta optical depth ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  q=seq.kapparho*dx
  qname='!4Ds!X'
  qunit=''
  ;
endif else if (cm eq 'tau') then begin
  ; --- Optical depth ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dtau=seq.kapparho*dx
  q=reverse(total(reverse(dtau, 1), 1, /cumulative), 1)
  qname='!4s!X'
  qunit=''
  ;
endif else if (cm eq 'htau') then begin
  ; --- Scale height of optical depth ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dtau=seq.kapparho*dx
  q=reverse(total(reverse(dtau, 1), 1, /cumulative), 1)
  q=q/seq.kapparho/unit_length_value
  qname='H!D!4s!X!N'
  qunit=unit_length_name
  ;
endif else if (cm eq 'intrhoeg') then begin
  ; --- Height-integrated potential energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dq=seq.rhoeg*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='Int(!4q!Xegdz)'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'intrhoei') then begin
  ; --- Height-integrated internal energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dq=seq.rhoei*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='Int(!4q!Xeidz)'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'intrhoek') then begin
  ; --- Height-integrated kinetic energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dq=seq.rhoek*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='Int(!4q!Xekdz)'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'ei') then begin
  ; --- Internal energy per mass ---
  ;
  q=seq.rhoei/seq.rho
  qname='ei'
  qunit='erg/g'
  ;
endif else if (cm eq 'gamma1') then begin
  ; --- Gamma1 ---
  q=seq.Gamma1
  qname='!4C!X!D1!N'
  qunit=''
  ;
endif else if (cm eq 'gamma3') then begin
  ; --- Gamma3 ---
  q=seq.Gamma3
  qname='!4C!X!D3!N'
  qunit=''
  ;
endif else if ((cm eq 'grav') or (cm eq 'grava')) then begin
  ; --- Gravity (height dependent) ---
  if (grav_mode eq 'central') then begin
    q=spheregrav(reform(seq.xc3_radial), r0=par.r0_grav, r1=par.r1_grav, r2=r2_grav, C=!con.G*par.mass_star)
  endif else begin
    q=xc3*0.0+par.grav
  endelse
  if (cm eq 'grava') then begin
    q=rebin(reform(q, n_elements(xc3), 1), n_elements(xc3), n_elements(seq.time))
  endif
  qavgd=-1
  qname='g'
  qunit='cm/s!U2!N'
  ;
endif else if (cm eq 'hp') then begin
  ; --- Pressure scale height ---
  if (grav_mode eq 'central') then begin
    grav=spheregrav(reform(seq.xc3_radial), r0=par.r0_grav, r1=par.r1_grav, r2=r2_grav, C=!con.G*par.mass_star)
    nt=n_elements(seq.rho[0,*])
    q=seq.rho
    for it=0,nt-1 do begin
      q[*,it]=(seq.P[*,it])/(grav*seq.rho[*,it])/unit_length_value
    endfor
  endif else begin
    ;grav=xc3*0.0+par.grav
    q=(seq.P)/(par.grav*seq.rho)/unit_length_value
  endelse
  qname='H!Dp!N'
  qunit=unit_length_name
  ;
endif else if ((cm eq 'kapparho') or (cm eq 'rhokappa')) then begin
  ; --- Opacity ---
  ;
  q=seq.kapparho
  qname='!4jq!X'
  qunit='1/cm'
  ;
endif else if (cm eq 'kappa') then begin
  ; --- Opacity ---
  ;
  q=seq.kapparho/seq.rho
  qname='!4j!X'
  qunit='cm!U2!N/g'
  ;
endif else if (cm eq 'timeac') or (cm eq 'time_ac') then begin
  ; --- Local acoustic time scale 2 H_p/cs [PhD (3.21)] ---
  ;
  q=2.0*!pi/(par.grav*0.5*sqrt(seq.Gamma1*seq.rho/seq.P))
  qname='t!Dacoustic!N'
  qunit='s'
  ;
endif else if (cm eq 'timeacfromtop') or (cm eq 'time_acfromtop') then begin
  ; --- Local acoustic travel time scale 2*Int(dxc3/cs)_top^local. ---
  ; --- The value at the bottom gives the value once down and up   ---
  ;
  n3=n_elements(xc3)
  dx=fltarr(n3)
  dx[0:n3-2]=xc3[1:n3-1]-xc3[0:n3-2]
  dx=rebin(reform(dx,n3,1), n3, n_elements(seq.time))
  dq=2.0/sqrt(seq.Gamma1*seq.P/seq.rho)*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='t!Dacoustic,travel-fromtop!N'
  qunit='s'
  ;
endif else if (cm eq 'omegaac') then begin
  ; --- Acoustic (circular) frequency [PhD (2.75)] ---
  ;
  q=par.grav*0.5*sqrt(seq.Gamma1*seq.rho/seq.P)
  qname='!4x!X!Dacoustic!N'
  qunit='s!U-1!N'
  ;
endif else if (cm eq 'omegabvsqrograv') then begin
  ; --- Brunt-Vaeisaelae frequency squared over gravity: delta/c_p * ds/dz ---
  ;
  q=1.0 / (seq.Gamma1*seq.P/((seq.Gamma3-1.0)*seq.rho*seq.T))
  for i=0L,n_elements(q[0,*])-1L do begin
    q[*,i]=q[*,i]*deriv(xc3, seq.s[*,i])
  endfor
  qname='!4x!X!U2!N!DBV!N/g'
  qunit='cm!U-1!N'
  ;
endif else if (cm eq 'omegabvsqr') then begin
  ; --- Brunt-Vaeisaelae frequency squared: delta/c_p * ds/dz * grav ---
  ;
  q=1.0 / (seq.Gamma1*seq.P/((seq.Gamma3-1.0)*seq.rho*seq.T))
  for i=0L,n_elements(q[0,*])-1L do begin
    q[*,i]=q[*,i]*deriv(xc3, seq.s[*,i])
  endfor
  q=q*par.grav
  qname='!4x!X!U2!N!DBV!N'
  qunit='s!U-2!N'
  ;
endif else if (cm eq 'timebv') or (cm eq 'time_bv') then begin
  ; --- Brunt-Vaeisaelae time scale: 2*pi/abs(omega_BV) ---
  ;
  q=1.0 / (seq.Gamma1*seq.P/((seq.Gamma3-1.0)*seq.rho*seq.T))
  for i=0L,n_elements(q[0,*])-1L do begin
    q[*,i]=q[*,i]*deriv(xc3, seq.s[*,i])
  endfor
  q=2.0*!pi/sqrt(abs(q*par.grav))
;  q=2.0*!pi/sqrt(abs((q*par.grav)<0.0))
  qname='t!DBV!N'
  qunit='s'
  ;
endif else if (cm eq 'timeff') or (cm eq 'time_ff') then begin
  ; --- Free fall time scale: sqrt(2*H_p/g) [PhD (3.27)] ---
  ;
  q=sqrt((2.0*seq.P)/(seq.rho*par.grav^2))
  qname='t!Dff!N'
  qunit='s'
  ;
endif else if (cm eq 'rhoeg') then begin
  ; --- Gravitational energy per volume ---
  ;
  q=seq.rhoeg
  qname='!4q!Xeg'
  qunit='erg/cm!U3!N'
  ;
endif else if (cm eq 'eg') then begin
  ; --- Gravitational energy per gram ---
  ;
  q=seq.rhoeg/seq.rho
  qname='!4q!Xeg'
  qunit='erg/g'
  ;
endif else if (cm eq 'rhoegint') then begin
  ; --- Integrated gravitational energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dq=seq.rhoeg*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='!4q!Xeg'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'rhoei') then begin
  ; --- Internal energy per volume ---
  ;
  q=seq.rhoei
  qname='!4q!Xei'
  qunit='erg/cm!U3!N'
  ;
endif else if (cm eq 'rhoeiint') then begin
  ; --- Integrated internal energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dq=seq.rhoei*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='!4q!Xei'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'rhoek') then begin
  ; --- Kinetic energy per volume ---
  ;
  q=seq.rhoek
  qname='!4q!Xek'
  qunit='erg/cm!U3!N'
  ;
endif else if (cm eq 'rhoekint') then begin
  ; --- Integrated kinetic energy ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dq=seq.rhoek*dx
  q=reverse(total(reverse(dq, 1), 1, /cumulative), 1)
  qname='!4q!Xek'
  qunit='erg/cm!U2!N'
  ;
endif else if (cm eq 'rhoekoverp') then begin
  ; --- Kinetic energy per volume over pressure ---
  ;
  q=seq.rhoek/seq.P
  qname='!4q!Xek/P'
  qunit=''
  ;
endif else if (cm eq 'p') then begin
  ; --- Pressure ---
  ;
  q=seq.P
  qname='P'
  qunit='dyn/cm!U2!N'
  ;
endif else if (cm eq 'pradt') then begin
  ; --- Radiation pressure from temperature ---
  ;
  q=((4.0*!con.sigmaSB)/(3.0*!con.cLight))*seq.T^4
  qname='P!Drad!n'
  qunit='dyn/cm!U2!N'
  ;
endif else if (cm eq 'pradtoverp') then begin
  ; --- Radiation pressure from temperature over gas pressure---
  ;
  q=((4.0*!con.sigmaSB)/(3.0*!con.cLight))*seq.T^4/seq.P
  qname='P!Drad!n/P'
  qunit=''
  ;
endif else if (cm eq 'prms') then begin
  ; --- rms Pressure ---
  ;
  q=seq.Prms
  qname='P!Drms!N'
  qunit='dyn/cm!U2!N'
  qavgd=2
  ;
endif else if (cm eq 'pdrms') then begin
  ; --- Pressure fluctuations---
  ;
  ;q=seq.Pdrms
  q=sqrt(abs(seq.Prms^2-seq.P^2))
  qname='P!D!4D!Xrms!N'
  qunit='dyn/cm!U2!N'
  qavgd=2
  ;
endif else if (cm eq 'pdrmsop') then begin
  ; --- Relative pressure fluctuations---
  ;
  ;q=seq.Pdrms/seq.P
  q=sqrt(abs(seq.Prms^2-seq.P^2))/seq.P
  qname='P!D!4D!Xrms!N/P'
  qunit=''
  qavgd=2
  ;
endif else if (cm eq 'rhov1') then begin
  ; --- Horizontal mass flux ---
  ;
  q=seq.rho*seq.v1
  qname='!4q!Xv!Dhor!N'
  qunit='g/cm!U2!N/s'
  ;
endif else if (cm eq 'rhov2') then begin
  ; --- Horizontal mass flux ---
  ;
  q=seq.rho*seq.v2
  qname='!4q!Xv!Dhor!N'
  qunit='g/cm!U2!N/s'
  ;
endif else if (cm eq 'rhovb') then begin
  ; --- Vertical mass flux ---
  ;
  q=seq.rhovb
  qname='!4q!Xv!Dvert!N'
  qunit='g/cm!U2!N/s'
  ;
endif else if (cm eq 'rhov3') then begin
  ; --- Vertical mass flux ---
  ;
  ;q=seq.rho*seq.v3
  q=seq.rhov3
  qname='!4q!Xv!Dvert!N'
  qunit='g/cm!U2!N/s'
  ;
endif else if (cm eq 'rhov3sqrorho') then begin
  ; --- Vertical mass flux squared over density ---
  ;
  q=seq.rhov3^2/seq.rho
  qname='P!Ddyn!N'
  qunit='dyn/cm!U2!N'
  ;
endif else if (cm eq 'rhov3v3') then begin
  ; --- Turbulent Pressure ---
  ;
  ;q=seq.rho*seq.v3rms^2
  q=seq.rhov3v3
  qname='P!Ddyn!N'
  qunit='dyn/cm!U2!N'
  ;
endif else if (cm eq 'frhov3b') then begin
  ; --- Flux of momentum x3 in x3 direction (includes gas and dynamical pressure) ---
  ;
  if (total(tags eq 'FRHOV3B') ge 1.0) then begin
    q=seq.frhov3b
  endif else begin
    q=seq.fekb*0.0
  endelse
  qname='F!D!4q!Xv33!N'
  qunit='dyn/cm!U2!N'
  ;
endif else if (cm eq 'rhov3v3op') then begin
  ; --- Turbulent Pressure over gas pressure---
  ;
  ;q=seq.rho*seq.v3rms^2/seq.P
  q=seq.rhov3v3/seq.P
  qname='P!Ddyn!N/P'
  qunit=''
endif else if (cm eq 'rho') then begin
  ; --- Mass density ---
  ;
  q=seq.rho
  qname='!4q!X'
  qunit=unit_density_name
  ;
endif else if (cm eq 'rhodrms') then begin
  ; --- Mass density fluctuation ---
  ;
  q=seq.rhodrms
  qname='!4q!X!D!4D!Xrms!N'
  qunit=unit_density_name
  qavgd=2
  ;
endif else if (cm eq 'rhodrmsorho') then begin
  ; --- Relative mass density fluctuation ---
  ;
  q=seq.rhodrms/seq.rho
  qname='!4q!X!D!4D!Xrms!N/!4q!X'
  qunit=''
  qavgd=2
  ;
endif else if (cm eq 's') then begin
  ; --- Entropy per mass unit ---
  ;
  ;q=seq.s*1.0E-09
  ;qname='s'
  ;qunit='10!U9!N erg/K/g'
  q=seq.s/unit_entropy_value
  qname='s'
  qunit=unit_entropy_name
  ;
endif else if (cm eq 'smass') then begin
  ; --- Entropy per mass unit ---
  ;
  ;q=seq.s*1.0E-09
  ;qname='s'
  ;qunit='10!U9!N erg/K/g'
  q=seq.rhos/seq.rho/unit_entropy_value
  qname='s'
  qunit=unit_entropy_name
  ;
endif else if (cm eq 'sdrms') then begin
  ; --- Entropy flctuations per mass unit ---
  ;
  if (total(tags eq 'SDRMS') ge 1.0) then begin
    q=seq.sdrms/unit_entropy_value
  endif else begin
    q=sqrt(abs(seq.srms^2-seq.s^2))/unit_entropy_value
  endelse
  qname='s!D!4D!Xrms!N'
  qunit=unit_entropy_name
  qavgd=2
  ;
endif else if (cm eq 't') then begin
  ; --- Temperature ---
  ;
  q=seq.T
  qname='T'
  qunit='K'
  ;
endif else if ((cm eq 't_2') or (cm eq 'trms')) then begin
  ; --- Temperature (T^2 average) ---
  ;
  q=seq.Trms
  qname='T!D!4D!Xrms!N'
  qunit='K'
  qavgd=2
  ;
endif else if (cm eq 't_4') then begin
  ; --- Temperature (T^4 average) ---
  ;
  q=seq.T_XMEAN4
  qname='T'
  qunit='K'
  ;
endif else if (cm eq 'tdrms') then begin
  ; --- Temperature fluctuations ---
  ;
  if (total(tags eq 'TDRMS') le 0.0) then begin
    q=sqrt(abs(seq.Trms^2-seq.T^2))
  endif else begin
    q=seq.Tdrms
  endelse
  qname='T!D!4D!Xrms!N'
  qunit='K'
  qavgd=2
  ;
endif else if (cm eq 'tflrhov3fl') then begin
  ; --- Temperature fluctuations times mass flux fluctuations ---
  ;
  q=seq.Tflrhov3fl
  qname='T!Dfluct!N !4q!Xv!Dfluct!N'
  qunit='Kg/cm!U2!N/s'
  ;
endif else if (cm eq 'tflrhov3fl0') then begin
  ; --- Temperature fluctuations times mass flux fluctuations, corrected for mean mass flux ---
  ;
  q=seq.Tflrhov3fl-seq.Tdrms*seq.rhov3
  qname='T!Dfluct!N !4q!Xv!Dfluct!N'
  qunit='Kg/cm!U2!N/s'
  ;
endif else if ((cm eq 'ma1rms') or (cm eq 'mach1rms') or (cm eq 'machv1rms')) then begin
  ; --- rms of horizontal velocity (as Mach number) ---
  ;
  q=seq.v1rms/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Dhor,rms!N'
  qunit=''
  qavgd=2
  ;
endif else if ((cm eq 'ma2rms') or (cm eq 'mach2rms') or (cm eq 'machv2rms')) then begin
  ; --- rms of horizontal velocity (as Mach number) ---
  ;
  q=seq.v2rms/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Dhor,rms!N'
  qunit=''
  qavgd=2
  ;
endif else if ((cm eq 'ma1p2rms') or (cm eq 'mach1p2rms') or (cm eq 'machv1p2rms')) then begin
  ; --- rms of horizontal velocity (as Mach number) ---
  ;
  q=sqrt(seq.v1rms^2+seq.v2rms^2)/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Dhor,rms!N'
  qunit=''
  qavgd=2
  ;
endif else if (cm eq 'ma3drms') then begin
  ; --- drms of vertical velocity (as Mach number) ---
  ;
  q=seq.v3drms/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Dvert,!4D!Xrms!N'
  qunit=''
  qavgd=2
  ;
endif else if ((cm eq 'ma3rms') or (cm eq 'mach3rms') or (cm eq 'machv3rms')) then begin
  ; --- rms of vertical velocity (as Mach number) ---
  ;
  q=seq.v3rms/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Dvert,rms!N'
  qunit=''
  qavgd=2
  ;
endif else if ((cm eq 'ma3') or (cm eq 'mach3') or (cm eq 'machv3')) then begin
  ; --- vertical velocity (as Mach number) ---
  ;
  q=seq.v3/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Dvert!N'
  qunit=''
  ;
endif else if (cm eq 'marms') then begin
  ; --- rms of velocity (as Mach number) ---
  ;
  ;q=sqrt(seq.v1rms^2+seq.v3rms^2)/sqrt(seq.Gamma1*seq.P/seq.rho) --- v2rms included: 2012-02-08
  q=sqrt(seq.v1rms^2+seq.v2rms^2+seq.v3rms^2)/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='Mach!Drms!N'
  qunit=''
  qavgd=2
  ;
endif else if ((cm eq 'massperarea') or (cm eq 'rhoint') or (cm eq 'intrho')) then begin
  ; --- Integrated mass ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  dmass=seq.rho*dx
  q=reverse(total(reverse(dmass, 1), 1, /cumulative), 1)
  qname='m'
  qunit='g/cm!U2!N'
  ;
endif else if ((cm eq 'massintegralb') or (cm eq 'massintupb') or (cm eq 'massintdownb')) then begin
  ; --- Integrated shell mass (on cell boundaries) ---
  ;
  xb3_volume=1.0D+00 * reform(seq.mean.r.xb3_volume)
  n3=n_elements(xb3_volume) -1
  nt=n_elements(seq.time)
  shell_volume=4.0D+00/3.0D+00 * !DPI * (xb3_volume[1:n3]^3-xb3_volume[0:n3-1]^3)
  shell_volume=rebin(reform(shell_volume, n3, 1), n3, nt)
  mass_shell=shell_volume * seq.rho
  q=fltarr(n3+1, nt)
  if (cm eq 'massintdownb') then begin
    mass_integral=reverse(float(total(reverse(mass_shell, 1), 1, /cumulative)), 1)
    q[0:-2,*]=mass_integral
  endif else begin
    mass_integral=float(total(mass_shell, 1, /cumulative))
    q[1:* ,*]=mass_integral
  endelse
  qname='M'
  qunit='g'
  ;
endif else if ((cm eq 'rhoegintupb') or (cm eq 'rhoegintdownb')) then begin
  ; --- Integrated shell internal energy (on cell boundaries) ---
  ;
  xb3_volume=1.0D+00 * reform(seq.mean.r.xb3_volume)
  n3=n_elements(xb3_volume) -1
  nt=n_elements(seq.time)
  shell_volume=4.0D+00/3.0D+00 * !DPI * (xb3_volume[1:n3]^3-xb3_volume[0:n3-1]^3)
  shell_volume=rebin(reform(shell_volume, n3, 1), n3, nt)
  energy_shell=double(shell_volume) * double(seq.rhoeg)
  q=dblarr(n3+1, nt)
  if (cm eq 'rhoegintdownb') then begin
    energy_integral=reverse((total(reverse(energy_shell, 1), 1, /cumulative)), 1)
    q[0:-2,*]=energy_integral
  endif else begin
    energy_integral=(total(energy_shell, 1, /cumulative))
    q[1:* ,*]=energy_integral
  endelse
  qname='E'
  qunit='erg'
  ;
endif else if ((cm eq 'rhoeiintupb') or (cm eq 'rhoeiintdownb')) then begin
  ; --- Integrated shell internal energy (on cell boundaries) ---
  ;
  xb3_volume=1.0D+00 * reform(seq.mean.r.xb3_volume)
  n3=n_elements(xb3_volume) -1
  nt=n_elements(seq.time)
  shell_volume=4.0D+00/3.0D+00 * !DPI * (xb3_volume[1:n3]^3-xb3_volume[0:n3-1]^3)
  shell_volume=rebin(reform(shell_volume, n3, 1), n3, nt)
  energy_shell=double(shell_volume) * double(seq.rhoei)
  q=dblarr(n3+1, nt)
  if (cm eq 'rhoeiintdownb') then begin
    energy_integral=reverse((total(reverse(energy_shell, 1), 1, /cumulative)), 1)
    q[0:-2,*]=energy_integral
  endif else begin
    energy_integral=(total(energy_shell, 1, /cumulative))
    q[1:* ,*]=energy_integral
  endelse
  qname='E'
  qunit='erg'
  ;
endif else if (cm eq 'masslossb') then begin
  ; --- Mass flux at cell boundaries ---
  ;
  n3=n_elements(seq.xb3)
  nt=n_elements(seq.time)
  Surf=rebin(reform( (1.0D+00*seq.xb3_surface)^2*4.0*!pi, n3, 1), n3, nt)
  q=seq.rhovb*Surf
  qname='Mdot'
  qunit='g/s'
  ;
endif else if (cm eq 'masslossc') then begin
  ; --- Mass flux at cell centers ---
  ;
  n3=n_elements(seq.xc3)
  nt=n_elements(seq.time)
  Surf=rebin(reform( (1.0D+00*seq.xc3)^2*4.0*!pi, n3, 1), n3, nt)
  q=seq.rhov3*Surf
  qname='Mdot'
  qunit='g/s'
  ;
endif else if (cm eq 'mu') then begin
  ; --- Mean particle weight ---
  ;
  q=!con.Rgas * (seq.rho*seq.T)/seq.P
  qname='!4l!X'
  qunit='u'
  ;
endif else if (cm eq 'time_hyd') then begin
  ; --- Courant time-scale (l=dx, v=c) ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  q=dx/sqrt(seq.Gamma1*seq.P/seq.rho)
  qname='t!Dhyd!N'
  qunit='s'
  ;
endif else if (cm eq 'time_rad') then begin
  ; --- Radiative time-scale (l=dx) ---
  ;
  n3=n_elements(xb3)
  dx=xb3[1:n3-1]-xb3[0:n3-2]
  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
  cv=seq.delta*seq.Gamma1*seq.P / ((seq.Gamma3-1.0)*seq.rho*seq.T *(1.0+seq.delta*(seq.Gamma3-1.0)))
  q=seq.kapparho*dx
  q=1.0/(16.0*!con.sigmasb)/ $
        (seq.kapparho/(cv*seq.rho)*seq.T^3) * $
        (1.0+1.5*q^2)
  qname='t!Drad!N'
  qunit='s'
  ;
endif else if (cm eq 'time_rad_hp') then begin
  ; --- Radiative time-scale (l=Hp) ---
  ;
  ; --- Pressure scale height ---
  if (grav_mode eq 'central') then begin
    grav=spheregrav(reform(seq.xc3_radial), r0=par.r0_grav, r1=par.r1_grav, r2=r2_grav, C=!con.G*par.mass_star)
    nt=n_elements(seq.rho[0,*])
    Hp=seq.rho
    for it=0,nt-1 do begin
      Hp[*,it]=(seq.P[*,it])/(grav*seq.rho[*,it])
    endfor ; it
  endif else begin
    Hp=(seq.P)/(par.grav*seq.rho)
  endelse
  ;
  cv=seq.delta*seq.Gamma1*seq.P / ((seq.Gamma3-1.0)*seq.rho*seq.T *(1.0+seq.delta*(seq.Gamma3-1.0)))
  q=seq.kapparho*Hp
  q=1.0/(16.0*!con.sigmasb)/ $
        (seq.kapparho/(cv*seq.rho)*seq.T^3) * $
        (1.0+1.5*q^2)
  qname='t!Drad!N'
  qunit='s'
  ;
endif else if (cm eq 'v1') then begin
  ; --- horizontal velocity ---
  ;
  q=seq.v1/unit_velocity_value
  qname='v!Dhor!N'
  qunit=unit_velocity_name
  ;
endif else if (cm eq 'v1mass') then begin
  ; --- horizontal velocity ---
  ;
  q=seq.rhov1/seq.rho/unit_velocity_value
  qname='v!Dhor!N'
  qunit=unit_velocity_name
  ;
endif else if (cm eq 'v1rms') then begin
  ; --- rms of horizontal velocity ---
  ;
  q=seq.v1rms/unit_velocity_value
  qname='v!Dhor,rms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v1drms') then begin
  ; --- horizontal velocity fluctuations---
  ;
  q=seq.v1drms/unit_velocity_value
  qname='v!Dhor,!4D!Xrms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v1p2rms') then begin
  ; --- rms of horizontal velocity ---
  ;
  q=sqrt(seq.v1rms^2+seq.v2rms^2)/unit_velocity_value
  qname='v!Dhor,rms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v1p2drms') then begin
  ; --- horizontal velocity fluctuations---
  ;
  q=sqrt(seq.v1drms^2+seq.v2drms^2)/unit_velocity_value
  qname='v!Dhor,!4D!Xrms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v2') then begin
  ; --- horizontal velocity ---
  ;
  q=seq.v2/unit_velocity_value
  qname='v!Dhor!N'
  qunit=unit_velocity_name
  ;
endif else if (cm eq 'v2mass') then begin
  ; --- horizontal velocity ---
  ;
  q=seq.rhov2/seq.rho/unit_velocity_value
  qname='v!Dhor!N'
  qunit=unit_velocity_name
  ;
endif else if (cm eq 'v2rms') then begin
  ; --- rms of horizontal velocity ---
  ;
  q=seq.v2rms/unit_velocity_value
  qname='v!Dhor,rms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v2drms') then begin
  ; --- horizontal velocity fluctuations---
  ;
  q=seq.v2drms/unit_velocity_value
  qname='v!Dhor,!4D!Xrms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v3') then begin
  ; --- mean of vertical velocity ---
  ;
  q=seq.v3/unit_velocity_value
  qname='v!Dvert!N'
  qunit=unit_velocity_name
  ;
;endif else if ((cm eq 'v3massold') or (cm eq 'v3massbold')) then begin
;  ; --- mean of vertical velocity from cell-boundary mass flux ---
;  ;
;  n3=n_elements(xb3)
;  q=0.5*(seq.rhovb[0:n3-2,*]+seq.rhovb[1:n3-1,*])/seq.rho
;  q=q/unit_velocity_value
;  if (grav_mode eq 'central') then begin
;    qname='v!Dradial!N'
;  endif else begin
;    qname='v!Dvert!N'
;  endelse
;  qunit=unit_velocity_name
;  ;
endif else if ((cm eq 'v3mass') or (cm eq 'v3massb')) then begin
  ; --- mean of vertical velocity from cell-boundary mass flux, located at xb3_surface ---
  ;
  n3=n_elements(xb3)
  nt=n_elements(seq.rho[0,*])
  ;
  rhob=fltarr(n3, nt)
  for it=0,nt-1 do begin
    rhob[*,it]=interpol(alog(reform(seq.rho[*,it])), seq.xc3_radial, seq.xb3_surface)
  endfor ; it
  rhob=exp(rhob)
  ;
  q=seq.rhovb/rhob
  ;
  q=q/unit_velocity_value
  if (grav_mode eq 'central') then begin
    qname='v!Dradial!N'
  endif else begin
    qname='v!Dvert!N'
  endelse
  qunit=unit_velocity_name
  ;
endif else if (cm eq 'v3massc') then begin
  ; --- mean of vertical velocity from cell-center mass flux ---
  ;
  q=seq.rhov3/seq.rho
  q=q/unit_velocity_value
  if (grav_mode eq 'central') then begin
    qname='v!Dradial!N'
  endif else begin
    qname='v!Dvert!N'
  endelse
  qunit=unit_velocity_name
  ;
endif else if ((cm eq 'time_v3mass') or (cm eq 'time_v3massb')) then begin
  ; --- Time scale of vertical or radial mass flow ---
  ;
  ; --- Pressure scale height ---
  if (grav_mode eq 'central') then begin
    grav=spheregrav(reform(seq.xc3_radial), r0=par.r0_grav, r1=par.r1_grav, r2=r2_grav, C=!con.G*par.mass_star)
    nt=n_elements(seq.rho[0,*])
    Hp=seq.rho
    for it=0,nt-1 do begin
      Hp[*,it]=(seq.P[*,it])/(grav*seq.rho[*,it])
    endfor
  endif else begin
    Hp=(seq.P)/(par.grav*seq.rho)
  endelse
  ;
  q=Hp/(0.5*(seq.rhovb[0:-2,*]+seq.rhovb[1:-1,*])/seq.rho)
  qname='t!Dv3mass!N'
  qunit='s'
  ;
endif else if ((cm eq 'ratio_time_rad_hp_v3mass') or (cm eq 'ratio_time_rad_hp_v3massb')) then begin
  ; --- Time scale of vertical or radial mass flow ---
  ;
  ; --- Pressure scale height ---
  if (grav_mode eq 'central') then begin
    grav=spheregrav(reform(seq.xc3_radial), r0=par.r0_grav, r1=par.r1_grav, r2=r2_grav, C=!con.G*par.mass_star)
    nt=n_elements(seq.rho[0,*])
    Hp=seq.rho
    for it=0,nt-1 do begin
      Hp[*,it]=(seq.P[*,it])/(grav*seq.rho[*,it])
    endfor
  endif else begin
    Hp=(seq.P)/(par.grav*seq.rho)
  endelse
  ;
  time_v3mass=Hp/(0.5*(seq.rhovb[0:-2,*]+seq.rhovb[1:-1,*])/seq.rho)
  ;
  cv=seq.delta*seq.Gamma1*seq.P / ((seq.Gamma3-1.0)*seq.rho*seq.T *(1.0+seq.delta*(seq.Gamma3-1.0)))
  time_rad=seq.kapparho*Hp
  time_rad=1.0/(16.0*!con.sigmasb)/ $
               (seq.kapparho/(cv*seq.rho)*seq.T^3) * $
               (1.0+1.5*time_rad^2)
  ;
  q=time_rad/time_v3mass
  ;
  qname='t!Drad!N/t!Dv3mass!N'
  qunit=''
  ;
endif else if (cm eq 'v3drms') then begin
  ; --- drms of vertical velocity ---
  ;
  q=sqrt(abs(seq.v3rms^2-seq.v3^2))/unit_velocity_value
  if (grav_mode eq 'central') then begin
    qname='v!Dradial,!4D!Xrms!N'
  endif else begin
    qname='v!Dvert,!4D!Xrms!N'
  endelse
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'v3rms') then begin
  ; --- rms of vertical velocity ---
  ;
  q=seq.v3rms/unit_velocity_value
  if (grav_mode eq 'central') then begin
    qname='v!Dradial,rms!N'
  endif else begin
    qname='v!Dvert,rms!N'
  endelse
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'vrms') then begin
  ; --- rms of velocity ---
  ;
  ;q=sqrt(seq.v1rms^2+seq.v3rms^2)/unit_velocity_value --- v2rms included: 2012-02-08
  q=sqrt(seq.v1rms^2+seq.v2rms^2+seq.v3rms^2)/unit_velocity_value
  qname='v!Drms!N'
  qunit=unit_velocity_name
  qavgd=2
  ;
endif else if (cm eq 'xc') then begin
  ; --- Radial/vertical axis ---
  ;
  if (grav_mode eq 'central') then begin
    q=reform(seq.xc3_radial)/unit_length_value
    qname='r'
  endif else begin
    q=reform(xc3)/unit_length_value
    qname='z'
    ; qname='y'
  endelse
  qunit=unit_length_name
  qavgd=-1
  ;
endif else if (cm eq 'xc3') then begin
  ; --- xc3 ---
  ;
  q=reform(xc3)/unit_length_value
  qname='z'
;  qname='y'
  qunit=unit_length_name
  qavgd=-1
  ;
endif else if (cm eq 'xc3v') then begin
  ; --- xc3 ---
  ;
  q=xc3/unit_length_value
  qname='z'
;  qname='y'
  qunit=unit_length_name
  qavgd=-1
  ;
endif else if (cm eq 'dxb3') then begin
  ; --- dxb3 ---
  ;
  q=xb3/unit_length_value
  n3=n_elements(q)
  q=q[1:n3-1]-q[0:n3-1]
  qname='dz'
;  qname='dy'
  qunit=unit_length_name
  qavgd=-1
  ;
endif else if (cm eq 'dxb3a') then begin
  ; --- dxb3 array ---
  ;
  q=xb3/unit_length_value
  n3=n_elements(q)
  q=rebin(reform(q[1:n3-1]-q[0:n3-1], n3-1, 1), n3-1, n_elements(seq.time))
  qname='dz'
;  qname='dy'
  qunit=unit_length_name
  ;
endif else if (cm eq 'xb3') then begin
  ; --- xb3 ---
  ;
  q=xb3/unit_length_value
  qname='z'
;  qname='y'
  qunit=unit_length_name
  qavgd=-1
  ;
endif else if (cm eq 'xb3s') then begin
  ; --- xb3 ---
  ;
  q=xb3/unit_length_value
  qname='z'
  qunit=unit_length_name
  qavgd=-1
  ;
endif else if (cm eq 'xm1v3fft') then begin
  ; --- xm1v3fft ---
  ;
  q=reform(seq.xm1v3fft)/unit_length_value
  qname='xm1v3fft'
  qunit=unit_length_name
  ;
endif else if (cm eq 'xp1v3fft') then begin
  ; --- xp1v3fft ---
  ;
  q=reform(seq.xp1v3fft)/unit_length_value
  qname='xp1v3fft'
  qunit=unit_length_name
  ;
endif else if (cm eq 'feg') then begin
  ; --- Potential energy flux ---
  ;
  q=seq.fegb/unit_flux_value
  qname='F!Dg!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feip') then begin
  ; --- Enthalpy flux ---
  ;
  q=seq.feipb/unit_flux_value
  qname='F!Dip!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'fep') then begin
  ; --- Acustic flux ---
  ;
  q=seq.fepb/unit_flux_value
  qname='F!Dp!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipk') then begin
  ; --- Enthalpy + kinetic energy flux ---
  ;
  q=(seq.feipb+seq.fekb)/unit_flux_value
  qname='F!Dipk!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipk0') then begin
  ; --- Enthalpy + kinetic energy flux, with mass flux correction ---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  q=(seq.feipb-feipb_mass + seq.fekb-fekb_mass)/unit_flux_value
  qname='F!Dipk!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkr') then begin
  ; --- Enthalpy + kinetic energy + radiative flux ---
  ;
  q=(seq.feipb + seq.fekb + seq.ferb)/unit_flux_value
  qname='F!Dipkr!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkr0') then begin
  ; --- Enthalpy + kinetic energy + radiative flux, with mass flux correction ---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  q=(seq.feipb-feipb_mass + seq.fekb-fekb_mass + seq.ferb)/unit_flux_value
  qname='F!Dipk!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkv') then begin
  ; --- Enthalpy + kinetic energy + viscous flux (total convective) ---
  ;
  q=(seq.feipb + seq.fekb + seq.fevb)/unit_flux_value
  qname='F!Dipkv!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkv0') then begin
  ; --- Enthalpy + kinetic energy + viscous flux (total convective), with mass flux correction ---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  q=(seq.feipb-feipb_mass + seq.fekb-fekb_mass + seq.fevb)/unit_flux_value
  qname='F!Dipkv!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkvg') then begin
  ; --- Enthalpy + kinetic energy + viscous + gravitational flux ---
  ;
  q=(seq.feipb + seq.fekb + seq.fevb + seq.fegb)/unit_flux_value
  qname='F!Dipkvg!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkvr') then begin
  ; --- Enthalpy + kinetic energy + viscous + radiative flux ---
  ;
  q=(seq.feipb + seq.fekb + seq.fevb + seq.ferb)/unit_flux_value
  qname='F!Dipkvr!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkvr0') then begin
  ; --- Enthalpy + kinetic energy + viscous + radiative flux, with mass flux correction ---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  q=(seq.feipb-feipb_mass + seq.fekb-fekb_mass + seq.fevb + seq.ferb)/unit_flux_value
  qname='F!Dipkvr!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feipkvgr') then begin
  ; --- Enthalpy + kinetic energy + viscous + gravitational + radiative flux ---
  ;
  q=(seq.feipb + seq.fekb + seq.fevb + seq.fegb + seq.ferb)/unit_flux_value
  qname='F!Dipkvbr!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'feip0') then begin
  ; --- Enthalpy flux with mass flux correction ---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  q=(seq.feipb - feipb_mass)/unit_flux_value
  qname='F!Dip!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'fek') then begin
  ; --- Kinetic energy flux ---
  ;
  q=seq.fekb/unit_flux_value
  qname='F!Dk!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'fek0') then begin
  ; --- Kinetic energy flux with mass flux correction ---
  ;
  n3=n_elements(xc3)
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  q=(seq.fekb - fekb_mass)/unit_flux_value
  qname='F!Dk!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'fer') then begin
  ; --- Radiative energy flux ---
  ;
  q=seq.ferb/unit_flux_value
  qname='F!Dr!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'fev') then begin
  ; --- Viscous energy flux ---
  ;
  q=seq.fevb/unit_flux_value
  qname='F!Dv!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'few') then begin
  ; --- Viscous P2P energy flux ---
  ;
  q=seq.fewb/unit_flux_value
  qname='F!Dv!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'fem') then begin
  ; --- Poynting flux of magnetic energy ---
  ;
  q=seq.femb/unit_flux_value
  qname='F!Dm!N'
  qunit=unit_flux_name
  ;
endif else if (cm eq 'lumr') then begin
  ; --- Radiative luminosity ---
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=seq.ferb * SoL
  qname='L!Dr!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumi') then begin
  ; --- Convective luminosity ---
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.feipb-seq.fepb) * SoL
  qname='L!Di!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumip') then begin
  ; --- Convective luminosity ---
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=seq.feipb * SoL
  qname='L!Dip!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumip0') then begin
  ; --- Total reduced convective luminosity---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.feipb-feipb_mass) * SoL
  qname='L!Dip!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumipk') then begin
  ; --- Total convective luminosity ---
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.feipb+seq.fekb) * SoL
  qname='L!Dipk!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumipk0') then begin
  ; --- Total reduced convective luminosity---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.feipb-feipb_mass + seq.fekb-fekb_mass) * SoL
  qname='L!Dipk!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lump') then begin
  ; --- Acoustic luminosity ---
  ;
  SoL=rebin(reform( xb3^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=seq.fepb * SoL
  qname='L!Dp!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumk') then begin
  ; --- Kinetic energy luminosity ---
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=seq.fekb * SoL
  qname='L!Dk!N'
  qunit='L!DSun!N'
  ;
endif else if (cm eq 'lumk0') then begin
  ; --- Reduced kinetic-energy luminosity---
  ;
  n3=n_elements(xc3)
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.fekb-fekb_mass) * SoL
  qname='L!Dk!N'
  qunit='L!DSun!N'
  ;
endif else if ((cm eq 'lumripk') or (cm eq 'lumipkr')) then begin
  ; --- Total luminosity---
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.ferb+seq.feipb + seq.fekb) * SoL
  qname='L!Dipk!N'
  qunit='L!DSun!N'
  ;
endif else if ((cm eq 'lumripk0') or (cm eq 'lumipkr0')) then begin
  ; --- Total reduced luminosity---
  ;
  n3=n_elements(xc3)
  eip=(seq.rhoei+seq.P)/seq.rho
  eipb=seq.feipb*0.0
  eipb[1:n3-1,*]=0.5*(eip[0:n3-2,*]+eip[1:n3-1,*])
  feipb_mass=eipb * seq.rhovb
  ;
  ek=(seq.rhoek)/seq.rho
  ekb=seq.fekb*0.0
  ekb[1:n3-1,*]=0.5*(ek[0:n3-2,*]+ek[1:n3-1,*])
  fekb_mass=ekb * seq.rhovb
  ;
  SoL=rebin(reform( seq.xb3_surface^2*4.0*!pi/!con.Lsun, (size(seq.ferb))[1], 1), $
            (size(seq.ferb))[1], (size(seq.ferb))[2])
  q=(seq.ferb+seq.feipb-feipb_mass + seq.fekb-fekb_mass) * SoL
  qname='L!Dipk!N'
  qunit='L!DSun!N'
  ;
endif else if ((cm eq 'nabla_ad') or (cm eq 'nablaad')) then begin
  ; --- Adiabatic temperature gradient ---
  q=(seq.Gamma3-1.0)/seq.Gamma1
  qname='!MG!X!Dad!N'
  qunit=''
  ;
endif else if (cm eq 'nabla') then begin
  ; --- Temperature gradient ---
  logP=alog(seq.P)
  logT=alog(seq.T)
  q   =logT*0.0
  for i=0L,n_elements(q[0,*])-1L do begin
    q[*,i]=deriv(reform(logP[*,i]), reform(logT[*,i]))
  endfor
  qname='!MG!X!Dad!N'
  qunit=''
;endif else if (cm eq 'quc001') then begin
;  ; --- Additional quantity quc001 ---
;  ;
;  q=seq.quc001
;  qname='quc001'
;  qunit=''
;  ;
;endif else if (cm eq 'quc001int') then begin
;  ; --- Additional quantity quc001, integrated from top to bottom ---
;  ;
;  n3=n_elements(xb3)
;  dx=xb3[1:n3-1]-xb3[0:n3-2]
;  dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
;  q=seq.quc001*dx
;  q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
;  qname='quc001-int'
;  qunit=''
;  ;
endif else if (cm eq 'rdust') then begin
  ; --- Grain radius ---
  ;
  if (par.dustscheme eq 'dust_k3mon_03') then begin
    q=par.ar_dustGrainRadius[1]*(seq.quc002/(seq.quc001+seq.quc002))^(1.0/3.0)
  endif else if (par.dustscheme eq 'dust_bins_01') then begin
    ;indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
    indquctag=where(strmatch(tags, 'QUC[0-9][0-9][0-9]'), nquc)
    qr=0.0*seq.rho
    q=0.0*seq.rho
    for iquc=1,nquc-1 do begin
      qucid='00'+trim(string(iquc+1),2)
;help, qucid
      qucid=strmid(qucid, strlen(qucid)-1, 3, /reverse_offset)
;help, qucid
      dummy=execute('qr=qr+seq.quc' + qucid + '*par.ar_dustGrainRadius[' + string(iquc) + ']')
      dummy=execute('q=q+seq.quc' + qucid)
    endfor ; i
    ;
    ind=where(q ne 0.0, nind)
;help, ind, nind
    if (nind ge 1) then begin
      q[ind]=qr[ind]/q[ind]
    endif
  endif
  qname='radius!DGrain!N'
  qunit='cm'
  ;
endif else if (cm eq 'rdust001') then begin
  ; --- Grain radius ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Grain radius: forsterite ---
    HEH   = 1.0E-01                     ; abundance of He
    SMASS = !con.MProt*(1.0 + HEH*4.0)
    ZNL   = 1.0E+03                     ; number of monomers in seed particle
    RHOD  = 3.27                        ; density of dust monomer, Mg2SiO4 density (Jager et al. 2003)
    AW    = 2.0*24.0 + 28.0 + 4.0*16.0  ; mass of monomer, Mg2SiO4, in proton masses
    ;
    q=(((3.0*AW*!con.MProt)/(4.0*!PI*RHOD)) * $
      (ZNL + (SMASS/par.ar_dustGrainAbu[0]) * seq.quc001/seq.rho))^(1.0/3.0)
    qname='radius!DGrain!N'
    qunit='cm'
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'rdust002') then begin
  ; --- Grain radius ---
  ;
  if (par.dustscheme eq 'dust_Mg2SiO4pAl2O3AGB') then begin
    ; --- Grain radius: corundum ---
    HEH   = 1.0E-01                     ; abundance of He
    SMASS = !con.MProt*(1.0 + HEH*4.0)
    ZNL   = 1.0E+03                     ; number of monomers in seed particle
    RHOD  = 3.99                        ; density of dust monomer, Al2O3 (G&S Ph+Ch_CDS Tab.12.1/CRC-HB)
    AW    = 2.0*27.0 + 3.0*16.0         ; mass of monomer, Al2O3, in proton masses
    ;
    q=(((3.0*AW*!con.MProt)/(4.0*!PI*RHOD)) * $
      (ZNL + (SMASS/par.ar_dustGrainAbu[1]) * seq.quc002/seq.rho))^(1.0/3.0)
    qname='radius!DGrain!N'
    qunit='cm'
    ;
  endif else begin
    message, 'quantity not defined for selected dustscheme'
    ;
  endelse ; (par.dustscheme)
  ;
endif else if (cm eq 'qucsat') then begin
  ; --- Saturation density ---
  ;
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    psatc1, seq.T, Psat
    q=(par.C_dust03*par.C_dust02*!con.muatom)/(!con.kBoltz*seq.T)*Psat
    qname='!4q!X!D!4D!Xsat!N'
    qunit='g/cm!U3!N' 
  endif else begin
    message, 'quantity not defined for selected dustscheme'
  endelse
  ;
endif else if (cm eq 'sat') then begin
  ; --- Saturation ---
  ;
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    psatc1, seq.T, Psat
    q=seq.quc001/((par.C_dust03*par.C_dust02*!con.muatom)/(!con.kBoltz*seq.T)*Psat)
    qname='S'
    qunit='' 
  endif else begin
    message, 'quantity not defined for selected dustscheme'
  endelse
  ;
endif else if (cm eq 'qucall') or (cm eq 'qucallint') or (cm eq 'qucallorho') then begin
  ; --- Additional tracers: add all, integrate or divide by rho ---
  ;
  ;indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
  indquctag=where(strmatch(tags, 'QUC[0-9][0-9][0-9]'), nquc)
  q=0.0*seq.rho
  for iquc=0,nquc-1 do begin
    dummy=execute('q=q+seq.' + tags[indquctag[iquc]])
  endfor
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    qname='!4q!X!Dmomomers+dust!N'
    qunit='g/cm!U3!N'
  endif else begin
    qname=cm
    qunit=''
  endelse
  ;
  if (strmid(cm,6,3) eq 'int') then begin
    n3=n_elements(xb3)
    dx=xb3[1:n3-1]-xb3[0:n3-2]
    dx=rebin(reform(dx, n3-1, 1), n3-1, n_elements(seq.time))
    q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
    qname=qname + '-int'
    if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
      qunit='g/cm!U2!N'
    endif else begin
      qunit=''
    endelse
  endif else if (strmid(cm,6,4) eq 'orho') then begin
    q=q/seq.rho
    qname=qname + '/' + '!4q!X'
    ;qunit='1'
    qunit=''
  endif
  ;
endif else if (cm eq 'qucallm1') or (cm eq 'qucallm1int') or (cm eq 'qucallm1orho') then begin
  ; --- Additional tracers: add all except monomers, integrate or divide by rho ---
  ;
  ;indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
  indquctag=where(strmatch(tags, 'QUC[0-9][0-9][0-9]'), nquc)
  q=0.0*seq.rho
  for iquc=0,nquc-1 do begin
    if (tags[indquctag[iquc]] ne 'QUC001') then begin
      dummy=execute('q=q+seq.' + tags[indquctag[iquc]])
    endif
  endfor
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    qname='!4q!X!Ddust!N'
    qunit='g/cm!U3!N'
  endif else begin
    qname=cm
    qunit=''
  endelse
  ;
  if (strmid(cm,8,3) eq 'int') then begin
    n3=n_elements(xb3)
    dx=xb3[1:n3-1]-xb3[0:n3-2]
    dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
    q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
    qname=qname + '-int'
    if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
      qunit='g/cm!U2!N'
    endif else begin
      qunit=''
    endelse
  endif else if (strmid(cm,8,4) eq 'orho') then begin
    q=q/seq.rho
    qname=qname + '/' + '!4q!X'
    ;qunit='1'
    qunit=''
  endif
  ;
endif else if ((strmid(cm,0,3) eq 'quc') and (strmatch(strmid(cm,3,3), '[0-9][0-9][0-9]'))) then begin
  ; --- Additional quantity ---
  ;
  dummy=execute('q=seq.' + strmid(cm,0,6))
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    if (strmid(cm,0,6) eq 'quc001') then begin
      qname='!4q!X!Dmomomer!N'
    endif else begin
      qname='!4q!X!Ddust!N'
    endelse
    qunit='g/cm!U3!N'
  endif else begin
    qname=strmid(cm,0,6)
    qunit=''
  endelse
  ;
  if (strmid(cm,6,3) eq 'int') then begin
    n3=n_elements(xb3)
    dx=xb3[1:n3-1]-xb3[0:n3-2]
    dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
    q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
    qname=qname + '-int'
    if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
      qunit='g/cm!U2!N'
    endif else begin
      qunit=''
    endelse
  endif else if (strmid(cm,6,4) eq 'orho') then begin
    q=q/seq.rho
    qname=qname + '/' + '!4q!X'
    ;qunit='1'
    qunit=''
  endif
  ;
endif else if ((strmid(cm,0,8) eq 'nmonoquc') and (strmatch(strmid(cm,8,3), '[0-9][0-9][0-9]'))) then begin
  ; --- Number density of additional quantity: monomers ---
  ;
help,cm,strmid(cm,0,8),strmid(cm,8,3),strmid(cm,5,6),strmid(cm,11,3)
  dummy=execute('q=seq.' + strmid(cm,5,6))
help,q,min(q),max(q),(par.C_dust02*!CON.muatom)
  if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
    q=q/(par.C_dust02*!CON.muatom)
    if (strmid(cm,5,6) eq 'quc001') then begin
      qname='Nmono!Dmomomer!N'
    endif else begin
      qname='Nmono!Ddust!N'
    endelse
    qunit='1/cm!U3!N'
  endif else begin
    qname=cm
    qunit=''
  endelse
  ;
  if (strmid(cm,11,3) eq 'int') then begin
    n3=n_elements(xb3)
    dx=xb3[1:n3-1]-xb3[0:n3-2]
    dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
    q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
    qname=qname + '-int'
    if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
      qunit='1/cm!U2!N'
    endif else begin
      qunit=''
    endelse
  endif
  ;
endif else if ((strmid(cm,0,9) eq 'ngrainquc') and (strmatch(strmid(cm,9,3), '[0-9][0-9][0-9]'))) then begin
  ; --- Number density of additional quantity: grains ---
  ;
;help,cm,strmid(cm,0,9),strmid(cm,9,3),strmid(cm,6,6),strmid(cm,12,3)
  dummy=execute('q=seq.' + strmid(cm,6,6))
;help,q,min(q),max(q)
  if (par.dustscheme eq 'dust_k3mon_03') then begin
    if (strmid(cm,6,6) eq 'quc001') then begin
      q=q/(par.C_dust02*!CON.muatom)  ; --- Number density of monomers ---
      qname='N!Dmonomers!N'
    endif else begin
      ;message, 'quantity not defined for selected dustscheme'
      rgrain3=par.ar_dustGrainRadius[1]^3*(seq.quc002/(seq.quc001+seq.quc002))
      mgrain =(4.0/3.0*!pi)*rgrain3*par.C_dust01
      q=q/mgrain
      qname='N!Dgrains!N'
    endelse
    qunit='1/cm!U3!N'
  endif else if (par.dustscheme eq 'dust_bins_01') then begin
    if (strmid(cm,6,6) eq 'quc001') then begin
      message, 'quantity not defined for selected dustscheme'
    endif else begin
      iquc=long(strmid(cm,9,3))
      mgrain=(4.0/3.0*!pi)*par.ar_dustGrainRadius[iquc-1]^3*par.C_dust01
      q=q/mgrain
      qname='N!Dgrain!N'
    endelse
    qunit='1/cm!U3!N'
  endif else begin
    qname=cm
    qunit=''
  endelse
  ;
  if (strmid(cm,12,3) eq 'int') then begin
    n3=n_elements(xb3)
    dx=xb3[1:n3-1]-xb3[0:n3-2]
    dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
    q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
    qname=qname + '-int'
    if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
      qunit='1/cm!U2!N'
    endif else begin
      qunit=''
    endelse
  endif
;help, qname, qunit
  ;
endif else if  (cm eq 'ngrainqucall') or (cm eq 'ngrainqucallint') or (cm eq 'ngrainqucallorho') then begin
  ; --- Number density of additional quantity: grains ---
  ;
help,cm,strmid(cm,0,9),strmid(cm,9,3),strmid(cm,6,6),strmid(cm,12,3)
  if (par.dustscheme eq 'dust_k3mon_03') then begin
    message, 'quantity not defined for selected dustscheme'
    if (strmid(cm,6,6) eq 'quc001') then begin
      qname='Ngrain!Dmomomer!N'
    endif else begin
      qname='Ngrain!Ddust!N'
    endelse
    qunit='1/cm!U3!N'
  endif else if (par.dustscheme eq 'dust_bins_01') then begin
    ;indquctag=where(strmid(tags,0,3) eq 'QUC', nquc)
    indquctag=where(strmatch(tags, 'QUC[0-9][0-9][0-9]'), nquc)
    q=0.0*seq.rho
    for iquc=1,nquc-1 do begin
      mgrain=(4.0/3.0*!pi)*par.ar_dustGrainRadius[iquc]^3*par.C_dust01
help,mgrain
      qucid='00'+trim(string(iquc+1),2)
help, qucid
      qucid=strmid(qucid, strlen(qucid)-1, 3, /reverse_offset)
help, qucid
      dummy=execute('q=q+seq.quc' + qucid + '/mgrain')
    endfor
help,q,min(q),max(q)
    qname='Ngrain!Ddust!N'
    qunit='1/cm!U3!N'
  endif else begin
    qname=cm
    qunit=''
  endelse
  ;
  if (strmid(cm,12,3) eq 'int') then begin
    n3=n_elements(xb3)
    dx=xb3[1:n3-1]-xb3[0:n3-2]
    dx=rebin(reform(dx,n3-1,1),n3-1,n_elements(seq.time))
    q=reverse(total(reverse(q, 1), 1, /cumulative), 1)
    qname=qname + '-int'
    if ((par.dustscheme eq 'dust_k3mon_03') or (par.dustscheme eq 'dust_bins_01')) then begin
      qunit='1/cm!U2!N'
    endif else begin
      qunit=''
    endelse
  endif
;help, qname, qunit
  ;
endif else if ((strmid(cm,0,4) eq 'fquc') and (strmatch(strmid(cm,4,3), '[0-9][0-9][0-9]'))) then begin
  ; --- Flux of additional quantity ---
  ;
  dummy=execute('q=seq.' + strmid(cm,0,7))
  qname=cm
  qunit=''
  ;
endif else begin
  message, 'Unknown quantity: "' + cm + '"'
endelse

; --- Find sub interval ---
if (total(tags eq 'BINFACTOR') gt 0.0) then begin
  BinFactor=long(seq.BinFactor) > 1
endif else begin
  BinFactor=1
endelse
if (keyword_set(ntime)) then begin
  navg=(long(ntime)/BinFactor) < (n_elements(seq.time)-1)
endif else begin
  navg=n_elements(seq.time)-1
endelse
if (keyword_set(mtime)) then begin
  if (mtime ge 0) then begin
    mavg=long(mtime)/BinFactor
  endif else begin
    mavg=(navg+long(mtime)/BinFactor) > 0
  endelse
endif else begin
  mavg=0
endelse

if (keyword_set(verbose)) then help, mavg, navg

; --- Restrict to sub interval ---
if (keyword_set(qavg0)) then begin
  qavg=qavgd < qavg0
endif else begin
  qavg=qavgd
endelse

if ((qavgd ge 1) and (n_elements(seq.time) gt 1)) then begin
  q=q[*,mavg:navg]
endif
; --- Interpolate onto different grid ---
if (keyword_set(intertoxc) and (n_elements(q[*,0]) eq n_elements(xb3))) then begin
  ; help, q, mavg, navg
  q0=double(q)
  q=fltarr(n_elements(q0[*,0])-1, (navg-mavg+1)>1)
  xb3d=double(xb3)
  xc3d=double(xc3)
  for it=0,(navg-mavg)>0 do begin
    q[*,it]=interpol(reform(q0[*,it]), xb3d, xc3d)
  endfor
endif
; --- Save "raw" array ---
qraw=q
; --- Average in time over sub interval ---
if (navg gt mavg) then begin
  if ((qavg eq 1) and (cm_rm2 ne 'rm2')) then begin
    ;q=avg(q,1)
    q=(avg(q,1,/double))
  endif else if ((qavg eq 2) or (cm_rm2 eq 'rm2')) then begin
    ;q=sqrt(avg(q^2,1))
    q=(sqrt(avg(q^2,1,/double)))
  endif
endif

; --- Subtract temporal average ---
if (keyword_set(qmeansub)) then begin
  if (qavgd ge 1) then begin
    q=qraw-rebin(reform(q,n_elements(q),1), n_elements(q),navg-mavg+1)
  endif
endif

; --- Handle logarithm ---
if (cm_log eq 'log10') then begin
  q=alog10(abs(q))
  if (qunit eq '') then begin
    qtitle='log(' + qname +')'
  endif else begin
    qtitle='log(' + qname +'/[' + qunit + '])'
  endelse
endif else if (cm_log eq 'loge') then begin
  q=alog(abs(q))
  if (qunit eq '') then begin
    qtitle='log!De!N(' + qname +')'
  endif else begin
    qtitle='log!De!N(' + qname +'/[' + qunit + '])'
  endelse
endif else begin
  if (qunit eq '') then begin
    qtitle=qname
  endif else begin
    qtitle=qname +' [' + qunit + ']'
  endelse
endelse

; --- Smooth ---
if (keyword_set(qsmooth)) then begin
  q=smooth(q, qsmooth, /edge_truncate)
endif

; --- Modify title for !P.FONT=0 ---
qtitle=transfont(qtitle)

; --- Return the output quantity ---
return, q

end ; seqavg
