pro seqavg_unitsystem, unitsystem=unitsystem0, $
  unit_length_value=unit_length_value, $
  unit_length_name =unit_length_name, $
  unit_velocity_value=unit_velocity_value, $
  unit_velocity_name =unit_velocity_name, $
  unit_entropy_value=unit_entropy_value, $
  unit_entropy_name =unit_entropy_name, $
  unit_DiffusionCoefficient_value=unit_DiffusionCoefficient_value, $
  unit_DiffusionCoefficient_name =unit_DiffusionCoefficient_name, $
  unit_MagneticField_value=unit_MagneticField_value, $
  unit_MagneticField_name =unit_MagneticField_name, $
  unit_density_name=unit_density_name
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   seqavg_unitsystem ('sequence_average_unit_system')
;
; PURPOSE:
;   Get unit factors and names corresponding to choosen unit system.
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   seqavg_unitsystem, unitsystem=unitsystem0, $
;     unit_length_value=unit_length_value, $
;     unit_length_name =unit_length_name, $
;     unit_velocity_value=unit_velocity_value, $
;     unit_velocity_name =unit_velocity_name, $
;     unit_entropy_value=unit_entropy_value, $
;     unit_entropy_name =unit_entropy_name, $
;     unit_DiffusionCoefficient_value=unit_DiffusionCoefficient_value, $
;     unit_DiffusionCoefficient_name =unit_DiffusionCoefficient_name, $
;     unit_MagneticField_value=unit_MagneticField_value, $
;     unit_MagneticField_name =unit_MagneticField_name, $
;     unit_density_name=unit_density_name
;
; INPUT:
;   None (just via optional keywords)
;
; OUTPUT:
;   None (just via optional keywords)
;
; KEYWORDS (INPUT):
;   unitsystem: ('unit_system') string to set the unit system:
;               'cgs':          cgs units (useful for further computations)
;               'cgssE9':       cgs units but entropy s in units of 10^9 erg/g/K
;               'km,s':
;               'km,s,cm/s':
;               'Mm,s,km/s':    (default)
;               'Gm,s,km/s':
;               'Rsun,s,km/s':  unit name of length scale='R!DSun!N'
;               'Rsun1,s,km/s': unit name of length scale='R!10!D!Z(2609)!X!N'
;               'Rsun2,s,km/s': unit name of length scale='R' + sunsymbol() + ' '
;
; KEYWORDS (OUTPUT):
;   unit_q_name:  ('unit_quantity_name') string with name of quantity
;   unit_q_value: ('unit_quantity_value') real with factor of quantity relative to cgs, e.g.:
;                 'cm' -> 'cm', 1.0
;                 'cm' -> 'km', 1.0E+05
;
; COMMON BLOCKS:
;   None
;
; GLOBAL VARIABLES:
;   The user defined system variable !con (physical constants) is used.
;
; ROUTINES:
;   sunsymbol: function to give a string with the sun symbol
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   IDL> seqavg_unitsystem, unitsystem='cgs', unit_length_value=unit_length_value, unit_length_name=unit_length_name  
;   IDL> help, unit_length_value, unit_length_name                                                                  
;   UNIT_LENGTH_VALUE  FLOAT  = 1.00000
;   UNIT_LENGTH_NAME   STRING = 'cm'
;
; MODIFICATION HISTORY:
;   2016-11-30 (B.F. Uppsala) Extract from seqavg.pro
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

; --- Set default unit system ---
if (keyword_set(unitsystem0)) then begin
  unitsystem=unitsystem0
endif else begin
  unitsystem='Mm,s,km/s'
endelse

; --- Set unit type ('-1' to get 'cm s^-1'; '/' to get 'cm/s') ---
UnitType='-1'

if (unitsystem eq 'cgs') then begin
  unit_length_value=1.0
  unit_length_name='cm'
  unit_velocity_value=1.0
  unit_entropy_value=1.0
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='cm/s'
    unit_entropy_name             ='erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='cm s!U-1!N'
    unit_entropy_name             ='erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else if (unitsystem eq 'cgssE9') then begin
  unit_length_value=1.0
  unit_length_name='cm'
  unit_velocity_value=1.0
  unit_entropy_value=1.0E+09
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='cm/s'
    unit_entropy_name             ='10!U9!N erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='cm s!U-1!N'
    unit_entropy_name             ='10!U9!N erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else if (unitsystem eq 'km,s') then begin
  unit_length_value=1.0E+05
  unit_length_name='km'
  unit_velocity_value=1.0E+05
  unit_entropy_value=1.0E+09
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='km/s'
    unit_entropy_name             ='10!U9!N erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='km s!U-1!N'
    unit_entropy_name             ='10!U9!N erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else if (unitsystem eq 'km,s,cm/s') then begin
  unit_length_value=1.0E+05
  unit_length_name='km'
  unit_velocity_value=1.0
  unit_entropy_value=1.0E+09
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='cm/s'
    unit_entropy_name             ='10!U9!N erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='cm s!U-1!N'
    unit_entropy_name             ='10!U9!N erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else if (unitsystem eq 'Mm,s,km/s') then begin
  unit_length_value=1.0E+08
  unit_length_name='Mm'
  unit_velocity_value=1.0E+05
  unit_entropy_value=1.0E+09
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='km/s'
    unit_entropy_name             ='10!U9!N erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='km s!U-1!N'
    unit_entropy_name             ='10!U9!N erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else if (unitsystem eq 'Gm,s,km/s') then begin
  unit_length_value=1.0E+11
  unit_length_name='Gm'
  unit_velocity_value=1.0E+05
  unit_entropy_value=1.0E+09
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='km/s'
    unit_entropy_name             ='10!U9!N erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='km s!U-1!N'
    unit_entropy_name             ='10!U9!N erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else if ((unitsystem eq 'Rsun,s,km/s' ) or $
               (unitsystem eq 'Rsun1,s,km/s') or $
               (unitsystem eq 'Rsun2,s,km/s')     ) then begin
  unit_length_value=!con.Rsun
  unit_length_name='R!DSun!N'
  if (unitsystem eq 'Rsun1,s,km/s') then unit_length_name='R!10!D!Z(2609)!X!N'
  if (unitsystem eq 'Rsun2,s,km/s') then unit_length_name='R' + sunsymbol() + ' '
  unit_velocity_value=1.0E+05
  unit_entropy_value=1.0E+09
  unit_DiffusionCoefficient_value=1.0
  unit_MagneticField_value=1.0/sqrt(4.0*!PI)
  unit_MagneticField_name='G'
  if (UnitType eq '/') then begin
    unit_velocity_name            ='km/s'
    unit_entropy_name             ='10!U9!N erg/K/g'
    unit_DiffusionCoefficient_name='cm!U2!N/s'
  endif else begin
    unit_velocity_name            ='km s!U-1!N'
    unit_entropy_name             ='10!U9!N erg K!U-1!N g!U-1!N'
    unit_DiffusionCoefficient_name='cm!U2!N s!U-1!N'
  endelse
  ;
endif else begin
  message, 'unitsystem not supported'
endelse
if (UnitType eq '/') then begin
  unit_density_name='g/cm!U3!N'
endif else begin
  unit_density_name='g cm!U-3!N'
endelse


end ; pro seqavg_unitsystem

