pro seqavgstat_getposition, q, iq, qextremum, qpcommand
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   seqavgstat_getposition ('sequence_average_statistics_get_position')
;
; PURPOSE:
;   Derive statistics from input vectors
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;
; INPUT:
;   q:         ('quantity') 1D real vector with quantity to analyse
;   qpcommand: ('quantity_position_command') String indicating the type of extremum to find:
;              'zero': zero closest to the minimum of abs(q)
;              'min':
;   
;
; OUTPUT:
;   iq:        ('index_quantity') Real scalar with (interpolated) index of extremum
;   qextremum: ('quantity_extremum') Real scalar with value at extremum
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2008-05-22 (B.F. Lyon) 
;   2008-05-27 (B.F.) Put into subroutine
;-
;---------------------------------------------------------------------------------------------------

; --- Find position of extremum or zero in first vector ---
nq=n_elements(q)
if (qpcommand eq 'max') then begin
  ; --- Global Maximum ---
  ;
  dummy=max(q, iqstart)
  if ((iqstart eq 0) or (iqstart eq (nq-1))) then begin
    iq=float(iqstart)
    qextremum=q[iqstart]
  endif else begin
    extremum, q, 1, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
  endelse
  ;
endif else if (qpcommand eq 'min') then begin
  ; --- Global Minimum ---
  ;
  dummy=min(q, iqstart)
  if ((iqstart eq 0) or (iqstart eq (nq-1))) then begin
    iq=float(iqstart)
    qextremum=q[iqstart]
  endif else begin
    extremum, q, 1, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
  endelse
  ;
endif else if (qpcommand eq 'zero') then begin
  ; --- Zero ---
  ;
  dummy=min(abs(q), iqstart)
  iqstart=iqstart > 1
  extremum, q, 0, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
  ;
endif else if (qpcommand eq 'minbot1') then begin
  ; --- First local minimum from bottom (not boundary element) ---
  ;
  if (nq le 2) then begin
    ; --- Not enough elements to define local minimum ---
    dummy=min(q, iqstart)
    qextremum=q[iqstart]
    iq=float(iqstart)   
  endif else begin
    dqs=signum(q[1:nq-1]-q[0:nq-2])
    ind=where(dqs[1:nq-2] gt dqs[0:nq-3], nind)+1L
    if (nind eq 0) then begin
      ; --- No change in sign found: there is no min, find the element with the smallest value instead ---
      dummy=min(q, iq)
      qextremum=q[iq]
      iq=float(iq)
    endif else begin
      ; --- Determine exact position of min of fitted polynomial around first element of sub vector ---
      iqstart=min(ind)
      iqstart=iqstart > 1
      extremum, q, 1, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
    endelse
  endelse
  ;
endif else if (qpcommand eq 'maxbot1') then begin
  ; --- First local maximum from bottom (not boundary element) ---
  ;
  if (nq le 2) then begin
    ; --- Not enough elements to define local minimum ---
    dummy=max(q, iqstart)
    qextremum=q[iqstart]
    iq=float(iqstart)   
  endif else begin
    dqs=signum(q[1:nq-1]-q[0:nq-2])
    ind=where(dqs[1:nq-2] lt dqs[0:nq-3], nind)+1L
    if (nind eq 0) then begin
      ; --- No change in sign found: there is no max, find the element with the largest value instead ---
      dummy=max(q, iq)
      qextremum=q[iq]
      iq=float(iq)
    endif else begin
      ; --- Determine exact position of max of fitted polynomial around first element of sub vector ---
      iqstart=min(ind)
      iqstart=iqstart > 1
      extremum, q, 1, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
    endelse
  endelse
  ;
endif else if (qpcommand eq 'maxtop1') then begin
  ; --- First local maximum from top (not boundary element) ---
  ;
  if (nq le 2) then begin
    ; --- Not enough elements to define local minimum ---
    dummy=max(q, iqstart)
    qextremum=q[iqstart]
    iq=float(iqstart)   
  endif else begin
    dqs=signum(q[1:nq-1]-q[0:nq-2])
    ind=where(dqs[1:nq-2] lt dqs[0:nq-3], nind)+1L
    if (nind eq 0) then begin
      ; --- No change in sign found: there is no max, find the element with the largest value instead ---
      dummy=max(q, iq)
      qextremum=q[iq]
      iq=float(iq)
    endif else begin
      ; --- Determine exact position of max of fitted polynomial around first element of sub vector ---
      iqstart=max(ind)
      iqstart=iqstart > 1
      extremum, q, 1, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
    endelse
  endelse
  ;
endif else if (qpcommand eq 'mintop1') then begin
  ; --- First local minimum from top (not boundary element) ---
  ;
  if (nq le 2) then begin
    ; --- Not enough elements to define local minimum ---
    dummy=min(q, iqstart)
    qextremum=q[iqstart]
    iq=float(iqstart)   
  endif else begin
    dqs=signum(q[1:nq-1]-q[0:nq-2])
    ind=where(dqs[1:nq-2] gt dqs[0:nq-3], nind)+1L
help,dqs,q,nq,nind
    if (nind eq 0) then begin
      ; --- No change in sign found: there is no min, find the element with the smallest value instead ---
      dummy=min(q, iq)
      qextremum=q[iq]
      iq=float(iq)
    endif else begin
      ; --- Determine exact position of min of fitted polynomial around first element of sub vector ---
      iqstart=max(ind)
      iqstart=iqstart < (nq-2)
      extremum, q, 1, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
    endelse
  endelse
  ;
endif else if (qpcommand eq 'zerobot1') then begin
  ; --- First zero from bottom ---
  ;
  iref=0
  qref=q[iref]
  if (qref eq 0.0) then begin
    ; --- First element is already zero ---
    iq=float(iref)
    qextremum=qref
  endif else begin
    ; --- Find all elements that have a different sign than first element ---
    ind=where(q*signum(qref) lt 0.0, nind)
    if (nind eq 0) then begin
      ; --- No change in sign found: there is no zero, find the element with the smallest amount instead ---
      dummy=min(abs(q), iq)
      qextremum=q[iq]
      iq=float(iq)
    endif else begin
      ; --- Determine exact position of zero of fitted polynomial around first element of sub vector ---
      iqstart=min(ind)
      iqstart=iqstart > 1
      extremum, q, 0, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
    endelse
  endelse
  ;
endif else if (qpcommand eq 'zerotop1') then begin
  ; --- First zero from top ---
  ;
  iref=nq-1
  qref=q[iref]
  if (qref eq 0.0) then begin
    ; --- First element is already zero ---
    iq=float(iref)
    qextremum=qref
  endif else begin
    ; --- Find all elements that have a different sign than first element ---
    ind=where(q*signum(qref) lt 0.0, nind)
    if (nind eq 0) then begin
      ; --- No change in sign found: there is no zero, find the element with the smallest amount instead ---
      dummy=min(abs(q), iq)
      qextremum=q[iq]
      iq=float(iq)
    endif else begin
      ; --- Determine exact position of zero of fitted polynomial around first element of sub vector ---
      iqstart=max(ind)
      iqstart=iqstart > 1
      extremum, q, 0, degree=2, i0=iqstart, irange=2, iq, qextremum, verbose=0
    endelse
  endelse
  ;
endif else begin
  ; --- Unknown command ---
  ;
  iq=0.0
  qextremum=q[0]
  ;
endelse

end ; seqavgstat_getposition


pro seqavgstat, qa0, qb0, qc0, qd0, P, q, $
  qaqcommand, qafcommand, qapcommand, $
  qbqcommand, qbfcommand, qbpcommand, $
  qcqcommand, qcfcommand, $
  qdqcommand, qdfcommand, qdpcommand, qdrcommand
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   seqavgstat ('sequence_average_statistics')
;
; PURPOSE:
;   Derive statistics from input vectors
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;    seqavgstat, qa0, qb0, qc0, qd0, P, q, $
;      qaqcommand, qafcommand, qapcommand, $
;      qbqcommand, qbfcommand, qbpcommand, $
;      qcqcommand, qcfcommand, $
;      qdqcommand, qdfcommand, qdpcommand, qdrcommand
;
; INPUT:
;   qa0:        ('quantity_a_0') 1D real vector in which to find extremum
;   qb0:        ('quantity_b_0') 1D real vector with quantity to take at the extremum (optional: usually "")
;   qc0:        ('quantity_c_0') 1D real vector with quantity to use finally, after interpolation (optional: usually "")
;   qd0:        ('quantity_a_0') 1D real vector to use for range restriction of the other three
;   P:          ('Pressure') 1D real vector with pressure (not used yet)
;   qaqcommand: ('quantity_a_quantity_command') string with name of quantity in qa0
;   qafcommand: ('quantity_a_function_command') string with function to be applied to vector:
;                 dummy=execute('qa0='+qafcommand)
;   qapcommand: ('quantity_a_position_command') string indicating the type of extremum to find:
;                 'max'     : global maximum
;                 'maxbot1' : first local maximum from bottom (not boundary element)
;                 'maxtop1' : first local maximum from top (not boundary element)
;                 'min'     : global minimum
;                 'minbot1' : first local minimum from bottom (not boundary element)
;                 'mintop1' : first local minimum from top (not boundary element)
;                 'zero'    : zero closest to the minimum of abs(q)
;                 'zerobot1': first zero from bottom
;                 'zerotop1': first zero from top
;   qdrcommand: ('quantity_d_range_command') string with range command:
;                 'above'   : use range above value
;                 'above+10': use range above value, skip 10 grid points
;                 'above-10': use range above value, start 10 grid points deeper
;                 'below'   : use range below value
;
; OUTPUT:
;   q:      ('quantity') real scalar with result
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2008-05-22 (B.F. Lyon) First version
;   2008-11-10 (B.F.) Parameter P added, not used yet
;   2010-11-26 (B.F.) Assure, that q is scalar
;-
;---------------------------------------------------------------------------------------------------

if (qdrcommand eq '') then begin
  ; --- No range specified: just copy input vectors ---
  if (n_elements(qa0) ne 0) then qa=qa0
  if (n_elements(qb0) ne 0) then qb=qb0
  if (n_elements(qc0) ne 0) then qc=qc0
  ;
endif else begin
  ; --- Find sub-range ---
  ;
  ; --- Copy range variable into working variable ---
  q=qd0
  ;
  ; --- Apply function to range vector ---
  if (qdfcommand ne '') then begin
    dummy=execute('q='+qdfcommand)
  endif
  ;
  ; --- Find position of extremum or zero in range vector ---
  seqavgstat_getposition, q, iq, qextremum, qdpcommand
  ;
  if (qdrcommand eq 'above') then begin
    iq=long(iq+1.0) < (n_elements(qa0)-1L)
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[iq:*]
    if (n_elements(qb0) ne 0) then qb=qb0[iq:*]
    if (n_elements(qc0) ne 0) then qc=qc0[iq:*]
  endif else if (qdrcommand eq 'above+10') then begin
    iq=long(iq+11.0) < (n_elements(qa0)-1L)
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[iq:*]
    if (n_elements(qb0) ne 0) then qb=qb0[iq:*]
    if (n_elements(qc0) ne 0) then qc=qc0[iq:*]
  endif else if (qdrcommand eq 'above-10') then begin
    iq=long(iq-9.0) < (n_elements(qa0)-1L)
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[iq:*]
    if (n_elements(qb0) ne 0) then qb=qb0[iq:*]
    if (n_elements(qc0) ne 0) then qc=qc0[iq:*]
  endif else if (qdrcommand eq 'above-20') then begin
    iq=long(iq-19.0) < (n_elements(qa0)-1L)
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[iq:*]
    if (n_elements(qb0) ne 0) then qb=qb0[iq:*]
    if (n_elements(qc0) ne 0) then qc=qc0[iq:*]
  endif else if (qdrcommand eq 'above-30') then begin
    iq=long(iq-29.0) < (n_elements(qa0)-1L)
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[iq:*]
    if (n_elements(qb0) ne 0) then qb=qb0[iq:*]
    if (n_elements(qc0) ne 0) then qc=qc0[iq:*]
  endif else if (qdrcommand eq 'below') then begin
    iq=long(iq) > 0L
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[0:iq]
    if (n_elements(qb0) ne 0) then qb=qb0[0:iq]
    if (n_elements(qc0) ne 0) then qc=qc0[0:iq]
  endif else if (qdrcommand eq 'below+1') then begin
    iq=long(iq+1.0) < (n_elements(qa0)-1L)
    ;
    if (n_elements(qa0) ne 0) then qa=qa0[0:iq]
    if (n_elements(qb0) ne 0) then qb=qb0[0:iq]
    if (n_elements(qc0) ne 0) then qc=qc0[0:iq]
  endif else begin
    ; --- Unknown range command: just copy input vectors ---
    if (n_elements(qa0) ne 0) then qa=qa0
    if (n_elements(qb0) ne 0) then qb=qb0
    if (n_elements(qc0) ne 0) then qc=qc0
  endelse
endelse

; --- Start with base input vector ---
q=qa

; --- Apply function to first vector ---
if (qafcommand ne '') then begin
  dummy=execute('q='+qafcommand)
endif

; --- Find position of extremum or zero in first vector ---
seqavgstat_getposition, q, iq, qextremum, qapcommand

; --- Determine value of second vector at extremum or zero ---
if ((qaqcommand eq qbqcommand) and (qafcommand eq qbfcommand)) then begin
  q=qextremum
endif else begin
  q=reform(interpol(qb, findgen(n_elements(qb)), iq, /quadratic))
endelse

if (qbfcommand ne '') then begin
  ; --- Apply function to value from second vector ---
  dummy=execute('q='+qbfcommand)
  ;
  ; --- Determine new position ---
  if (qbpcommand ne '') then begin
    q=reform(interpol(qc, qb, q, /quadratic))
    ;
    if (qcfcommand ne '') then begin
      ; --- Apply function to value from third vector ---
      dummy=execute('q='+qcfcommand)
    endif
  endif
  ;
endif

q=q[0]

end ; seqavgstat
