function seqcomp, command, seq, par=par0, $
  mtime=mtime, ntime=ntime, qavg=qavg0, qsmooth=qsmooth, $
  unitsystem=unitsystem0, uniteflux=uniteflux0, $
  verbose=verbose, intertoxc=intertoxc, $
  qname=qname, qunit=qunit, qtitle=qtitle
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   seqcomp ('sequence_average')
;
; PURPOSE:
;   Compute averages from sequence data.
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   q=seqcomp(command, seq, par=par, &
;            mtime=mtime, ntime=ntime, qavg=qavg0, qsmooth=qsmooth, $
;            unitsystem=unitsystem, uniteflux=uniteflux, $
;            verbose=verbose, intertoxc=intertoxc, $
;            qname=qname, qunit=qunit, qtitle=qtitle)
;
; INPUT:
;   command: ('command') string with name of quantity to be extracted
;   seq:     ('sequence') structure with data from mean files,
;                         e.g., extracted with rhd_seq_collect or rhd_seqpp_collect
;
; OUTPUT:
;   seqcomp: ('sequence_compute') output vector quantity computed from sequence averages
;
; KEYWORDS (INPUT):
;   par:      ('parameters') optional structure containing CO5BOLD parameter file,
;             if not part of seq structure
;   mtime:    ('m_time') integer with index of first time step to be considered,
;                default: 0
;   ntime:    ('n_time') integer with index of last time step to be considered,
;                default: last time step in structure
;   qavg:     ('quantity_average') ineger with averaging mode:
;               -1: no averaging, results in output of 2D array
;                0: default: the value most reasonable for selected quantity is chosen automatically
;                1: q=avg(q_array)
;                2: q=sqrt(avg(q_array^2))
;   qsmooth:  ('quantity_smooth') integer, if larger than zero: size of box averaging window
;   unitsystem: ('unit_system') string to set the unit system:
;               'cgs':          cgs units (useful for further computations)
;               'cgssE9':       cgs units but entropy s in units of 10^9 erg/g/K
;               'km,s':
;               'km,s,cm/s':
;               'Mm,s,km/s':    (default)
;               'Gm,s,km/s':
;               'Rsun,s,km/s':  unit name of length scale='R!DSun!N'
;               'Rsun1,s,km/s': unit name of length scale='R!10!D!Z(2609)!X!N'
;               'Rsun2,s,km/s': unit name of length scale='R' + sunsymbol() + ' '
;   uniteflux: ('unit_energy_flux') string to set the units of fluxes:
;               'cgs':          cgs units (useful for further computations)
;               '1e9cgs':
;               'Teff':         in units of !con.sigma * par.Teff^4 (default)
;               'Frtop':        in units of radiative flux at the top of the model
;
; KEYWORDS (OUTPUT):
;   qname:    ('quantity_name') string with name of quantity
;   qunit:    ('quantity_unit') string with unit of quantity
;   qtitle:   ('quantity_title') string with proper plot title for quantity,
;             made from qname and qunit
;
; COMMON BLOCKS:
;   None
;
; GLOBAL VARIABLES:
;   The user defined system variable !con (physical constants) is used.
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2016-11-30 (B.F. Uppsala) First version
;   2016-12-01 (B.F.) Treat some ratios specially; handle sqrt
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

; --- Take parameter file structure directly from input parameters or from seq structure ---
tags=tag_names(seq)
if (keyword_set(par0)) then begin
  par=par0
endif else begin
  if (total(tags eq 'PAR') gt 0.0) then begin
    par=seq.par
  endif else begin
    par=0
  endelse 
endelse

; --- Get unit names and factors ---
seqavg_unitsystem, unitsystem=unitsystem0, $
  unit_length_value=unit_length_value, $
  unit_length_name =unit_length_name, $
  unit_velocity_value=unit_velocity_value, $
  unit_velocity_name =unit_velocity_name, $
  unit_entropy_value=unit_entropy_value, $
  unit_entropy_name =unit_entropy_name, $
  unit_DiffusionCoefficient_value=unit_DiffusionCoefficient_value, $
  unit_DiffusionCoefficient_name =unit_DiffusionCoefficient_name, $
  unit_MagneticField_value=unit_MagneticField_value, $
  unit_MagneticField_name =unit_MagneticField_name, $
  unit_density_name=unit_density_name

; --- Parse command string ---
cm=strlowcase(command)
;
if (strmid(cm, 0, 6) eq 'log10_') then begin
  cm_log='log10'
  cm=strmid(cm, 6)
endif else begin
  cm_log=''
endelse
;
if (strmid(cm, 0, 5) eq 'sqrt_') then begin
  cm_sqrt='sqrt'
  cm=strmid(cm, 5)
endif else begin
  cm_sqrt=''
endelse
;
cm_posover=strpos(cm, '_over_')
qoverq_flag=(cm_posover ge 0)

; --- Determine gravity mode ---
if (size(par,/type) eq 8) then begin
  grav_mode=strlowcase(par.grav_mode)
endif else if (total(tags eq 'XC3_RADIAL') gt 0.0) then begin
  grav_mode='central'
endif else begin
  grav_mode='constant'
endelse


if (cm eq '') then begin
  ;
  qunit=''
  qname=''
  q=0
  ;
endif else if (qoverq_flag) then begin
  ; --- The ratio of two quantities is requested ---
  ;
  ; --- Determine both quantity names ---
  cm1=strmid(cm, 0, cm_posover)
  cm2=strmid(cm, cm_posover+6)
  ;
  ; --- Compute both quantities ---
  help, cm, cm1
  q1=seqavg(cm1, seq, par=par0, $
     mtime=mtime, ntime=ntime, $
     unitsystem='cgs', uniteflux='cgs', $
     verbose=verbose, intertoxc=intertoxc, $
     qname=qname1, qunit=qunit1, qtitle=qtitle1)
  help, cm2
  q2=seqavg(cm2, seq, par=par0, $
     mtime=mtime, ntime=ntime, $
     unitsystem='cgs', uniteflux='cgs', $
     verbose=verbose, intertoxc=intertoxc, $
     qname=qname2, qunit=qunit2, qtitle=qtitle2)
  ;
  ; --- Compute the ratio ---
  q=q1/q2
  ;
  ; --- Handle quantity name and unit name ---
  if (cm eq 'rhov3v3_over_p') then begin
    ; --- Dynamical pressure over gas pressure ---
    qname='P!Ddyn!N/P'
    qunit=''
    ;
  endif else if (cm eq 'rhov3_rm2_over_rho') then begin
    ; --- Mass-averaged radial velocity ---
    q=q/unit_velocity_value
    ;
    if (grav_mode eq 'central') then begin
      qname='v!Dradial!N'
    endif else begin
      qname='v!Dvert!N'
    endelse
    qunit=unit_velocity_name
    ;
  endif else if ((cm eq 'rhov3sqrorho_over_rho') and (cm_sqrt eq 'sqrt')) then begin
    ; --- Mass-averaged radial velocity ---
    q=sqrt(q)/unit_velocity_value
    cm_sqrt=''
    ;
    if (grav_mode eq 'central') then begin
      qname='v!Dradial!N'
    endif else begin
      qname='v!Dvert!N'
    endelse
    qunit=unit_velocity_name
    ;
  endif else if ((cm eq 'rhov3v3_over_rho') and (cm_sqrt eq 'sqrt')) then begin
    ; --- Mass-averaged rms radial velocity ---
    q=sqrt(q)/unit_velocity_value
    cm_sqrt=''
    ;
    if (grav_mode eq 'central') then begin
      qname='v!Dradial,rms!N'
    endif else begin
      qname='v!Dvert,rms!N'
    endelse
    qunit=unit_velocity_name
    ;
  endif else begin
    ; --- Default: compose the output names from both components
    qname=qname1 + '/' + qname2
    qunit=qunit1 + '/' + qunit2
    ;
  endelse
  ;
endif else begin
  ; --- Default: pass arguments to seqavg() ---
  q=seqavg(cm, seq, par=par0, $
    mtime=mtime, ntime=ntime, $
    unitsystem=unitsystem0, uniteflux=uniteflux0, $
    verbose=verbose, intertoxc=intertoxc, $
    qname=qname, qunit=qunit, qtitle=qtitle)
  ;
endelse

; --- Handle square root ---
if (cm_sqrt eq 'sqrt') then begin
  q=sqrt(q > 0.0)
  qname='sqrt(' + qname +')'
  if (qunit ne '') then begin
    qunit='sqrt(' + qunit + ')'
  endif
endif

; --- Handle logarithm ---
if (cm_log eq 'log10') then begin
  ;q=alog10(q)
  q=alog10(abs(q))
  if (qunit eq '') then begin
    qtitle='log(' + qname +')'
  endif else begin
    qtitle='log(' + qname +'/[' + qunit + '])'
  endelse
endif else begin
  if (qunit eq '') then begin
    qtitle=qname
  endif else begin
    qtitle=qname +' [' + qunit + ']'
  endelse
endelse

; --- Smooth ---
if (keyword_set(qsmooth)) then begin
  q=smooth(q, qsmooth, /edge_truncate)
endif

; --- Modify title for !P.FONT=0 ---
qtitle=transfont(qtitle)

; --- Return the output quantity ---
return, q

end ; function seqcomp
