function seqrebin, seq0, BinFactor0, mtime=mtime, ntime=ntime
;
;------------------------------------------------------------------------------
;+
; NAME:
;   seqrebin ('sequence_rebin')
;
; PURPOSE:
;   Rebin sequence structure in time.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   result=seqrebin(seq, BinFactor, mtime=mtime, ntime=ntime)
;
; INPUT:
;   seq:       ('sequence') input sequence structure (e.g. read with rhd_seqpp_collect.pro)
;   BinFactor: ('binning_factor') integer with binning factor (e.g. 10 or 100)
;
; OUTPUT:
;   result:    ('sequence_rebinned') Same type of structure as input seq, less time points
;
; KEYWORDS:
;   mtime:     ('m_time') integer with index of first time step to be considered,
;                         default: 0
;   ntime:     ('n_time') integer with index of last time step to be considered,
;                         default: last time step in structure
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   rhd_seq_put2struct: Collect data from CO5BOLD mean files
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2008-05-20 (B.F. Lyon) Written
;   2009-07-20 (B.F. Paris) Add v2XXX
;   2009-10-10 (B.F.) Tags JulianDate, Version added
;   2009-10-13 (B.F.) Some rearrangements, T_xmean4 with proper mean
;   2010-03-26 (B.F. Naples) Magnetic field data
;   2014-08-07 (B.F. Uppsala) Angular momentum
;   2016-11-07 (B.F.) Add rhov1, rhov2, rhovXvY, frhovXb
;   2018-06-25 (B.F. Kiel) Add spherical wave amplitudes rhovbl???m????
;   2018-06-25 (B.F.) Add rhotil, rhovbrms
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

nstr='__NAME'

; --- Check parameters ---
if (n_elements(BinFactor0) eq 0) then begin
  message, 'Parameter BinFactor is not specified.'
endif
if (BinFactor0 lt 0) then begin
  message, 'Parameter BinFactor is too small.'
endif
BinFactor=long(BinFactor0)
ind=where(tag_names(seq0) eq 'BINFACTOR', nBinFactorTags)
if (BinFactor eq 1) then begin
  ; --- No binning requested ---
  if (nBinFactorTags eq 1) then begin
    ; --- Just return the input array ---
    return, seq0
  endif else begin
    ; --- Just return the input array with a BinFactor tag added ---
    return, create_struct(seq0, 'BINFACTOR', 1, $
                                'BINFACTOR'  + nstr, 'Binning factor')
  endelse
endif

if (nBinFactorTags eq 1) then begin
  seqBinFactor=long(seq0.BinFactor) > 1
endif else begin
  seqBinFactor=1
endelse
if (BinFactor gt 0) then begin
  if (((BinFactor mod seqBinFactor) ne 0) or $
       (BinFactor lt seqBinFactor)       ) then begin
    message, 'Requested binning factor and file binning factor not compatible'
  endif
endif

if (keyword_set(ntime)) then begin
  navg=(long(ntime)/seqBinFactor) < (n_elements(seq0.time)-1)
endif else begin
  navg=n_elements(seq0.time)-1
endelse

if (keyword_set(mtime)) then begin
  if (mtime ge 0) then begin
    mavg=long(mtime)/seqBinFactor
  endif else begin
    mavg=(navg+long(mtime)/seqBinFactor) > 0
  endelse
endif else begin
  mavg=0
endelse

; --- Number of time steps in new structure  ---
;nTimeNew=n_elements(seq0.time)/(BinFactor)
;if (nTimeNew le 0) then begin
;  message, 'Binfactor is larger than the number of time steps in the input structure.'
;endif
;nTimeOld=nTimeNew*BinFactor-1
if (BinFactor eq 0) then begin
  Binfactor=(navg-mavg+1)*seqBinFactor
endif
nTimeNew=(navg-mavg+1)/(BinFactor/seqBinFactor)
if (nTimeNew le 0) then begin
  message, 'Binfactor is larger than the number of time steps in the input structure.'
endif
mTimeOld=mavg
nTimeOld=mTimeOld+nTimeNew*(BinFactor/seqBinFactor)-1
help, seqBinFactor, BinFactor, mtime, ntime, mavg, navg, mTimeOld, nTimeOld, nTimeNew

dummy=where(tag_names(seq0) eq ('XC3'+nstr), ReadMetaData)
dummy=where(strmatch(tag_names(seq0), 'RHOVBL[0-9][0-9][0-9]M?[0-9][0-9][0-9]'), nwave)
nwave=nwave > 0
help, ReadMetaData

isCentral_flag=(strmatch(seq0.par.grav_mode, 'central' ,/fold))

; --- Create new structure ---
rhd_seq_put2struct, seq0.mean, seq, par=seq0.par, $
  /InitStruct, modelname=seq0.model, modelident=seq0.modelident, n_timestep=nTimeNew, $
  ReadMetaData=ReadMetaData
;if (nBinFactorTags eq 1) then begin
;  seq.BinFactor=BinFactor*seq0.BinFactor
;endif else begin
  seq.BinFactor=BinFactor
;endelse
BinFactor=BinFactor/seqBinFactor
help, Binfactor

; --- Array sizes ---
;ind=where(strmid(tag_names(seq0), 0, 3) eq 'QUC', nquc)
ind=where(strmatch(tag_names(seq0), 'QUC[0-9][0-9][0-9]'), nquc)
ind=where(strmatch(tag_names(seq0), 'FQUC[0-9][0-9][0-9]'), nfquc)
nr =n_elements(seq0.xc3)

; --- Fill new stucture with rebinned data ---
seq.itime    =long(rebin(double(seq0.itime[mTimeOld:nTimeOld]), nTimeNew))
seq.time     =     rebin(       seq0.time[ mTimeOld:nTimeOld] , nTimeNew)
if ((total(tag_names(seq0) eq 'TIME_FIRST') gt 0.0) and (total(tag_names(seq0) eq 'TIME_LAST') gt 0.0)) then begin
  seq.itime_first=seq0.itime_first[mTimeOld            :nTimeOld:BinFactor]
  seq.itime_last =seq0.itime_last[ mTimeOld+BinFactor-1:nTimeOld:BinFactor]
  seq.time_first =seq0.time_first[ mTimeOld            :nTimeOld:BinFactor]
  seq.time_last  =seq0.time_last[  mTimeOld+BinFactor-1:nTimeOld:BinFactor]
endif else begin
  seq.itime_first=seq0.itime[      mTimeOld            :nTimeOld:BinFactor]
  seq.itime_last =seq0.itime[      mTimeOld+BinFactor-1:nTimeOld:BinFactor]
  seq.time_first =seq0.time[       mTimeOld            :nTimeOld:BinFactor]
  seq.time_last  =seq0.time[       mTimeOld+BinFactor-1:nTimeOld:BinFactor]
endelse
if (total(tag_names(seq0) eq 'JULIANDATE') gt 0.0) then begin
  seq.JulianDate=rebin(seq0.JulianDate[mTimeOld:nTimeOld], nTimeNew)
endif
seq.ifile    =seq0.ifile[   mTimeOld:nTimeOld:BinFactor]
seq.file     =seq0.file[    mTimeOld:nTimeOld:BinFactor]
seq.version  =seq0.version[ mTimeOld:nTimeOld:BinFactor]
seq.s_inflow =rebin(seq0.s_inflow[ mTimeOld:nTimeOld], nTimeNew)
seq.c_schange=rebin(seq0.c_schange[mTimeOld:nTimeOld], nTimeNew) 
if (total(tag_names(seq0) eq 'C_HPTOPFACTOR') gt 0.0) then begin
  seq.c_hptopfactor=rebin(seq0.c_hptopfactor[mTimeOld:nTimeOld], nTimeNew)
endif
seq.I3r      =rebin(seq0.I3r[      mTimeOld:nTimeOld], nTimeNew)

seq.v1       =     rebin(seq0.v1[      *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.v1drms   =sqrt(rebin(seq0.v1drms[  *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.v1rms    =sqrt(rebin(seq0.v1rms[   *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.v2       =     rebin(seq0.v2[      *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.v2drms   =sqrt(rebin(seq0.v2drms[  *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.v2rms    =sqrt(rebin(seq0.v2rms[   *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.v3       =     rebin(seq0.v3[      *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.v3drms   =sqrt(rebin(seq0.v3drms[  *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.v3rms    =sqrt(rebin(seq0.v3rms[   *,mTimeOld:nTimeOld]^2, nr, nTimeNew))

seq.rhov3    =     rebin(seq0.rhov3[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)

if (total(tag_names(seq0) eq 'RHOV3V3') gt 0.0) then begin
  seq.rhov1    =     rebin(seq0.rhov1[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.rhov2    =     rebin(seq0.rhov2[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  ;
  seq.rhov1v1  =     rebin(seq0.rhov1v1[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.rhov1v2  =     rebin(seq0.rhov1v2[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.rhov1v3  =     rebin(seq0.rhov1v3[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.rhov2v2  =     rebin(seq0.rhov2v2[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.rhov2v3  =     rebin(seq0.rhov2v3[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.rhov3v3  =     rebin(seq0.rhov3v3[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)
endif

seq.rho      =     rebin(seq0.rho[     *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.rhorms   =sqrt(rebin(seq0.rhorms[  *,mTimeOld:nTimeOld]^2, nr, nTimeNew))

seq.ei       =     rebin(seq0.ei[      *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.rhoei    =     rebin(seq0.rhoei[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.rhoek    =     rebin(seq0.rhoek[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.rhoeg    =     rebin(seq0.rhoeg[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)

seq.T        =     rebin(seq0.T[       *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.Tdrms    =sqrt(rebin(seq0.Tdrms[   *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.Trms     =sqrt(     rebin(seq0.Trms[    *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.T_xmean4 =sqrt(sqrt(rebin(seq0.T_xmean4[*,mTimeOld:nTimeOld]^4, nr, nTimeNew)))     ; --- Attention: correct average?
seq.T_xmeankapparho=rebin(seq0.T_xmeankapparho[*,mTimeOld:nTimeOld], nr, nTimeNew)      ; --- Attention: correct average?

seq.P        =     rebin(seq0.P[       *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.Prms     =sqrt(rebin(seq0.Prms[    *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.s        =     rebin(seq0.s[       *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.srms     =sqrt(rebin(seq0.srms[    *,mTimeOld:nTimeOld]^2, nr, nTimeNew))
seq.rhos     =     rebin(seq0.rhos[    *,mTimeOld:nTimeOld]  , nr, nTimeNew)

seq.Gamma1   =     rebin(seq0.Gamma1[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.Gamma3   =     rebin(seq0.Gamma3[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.delta    =     rebin(seq0.delta[   *,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.kapparho =     rebin(seq0.kapparho[*,mTimeOld:nTimeOld]  , nr, nTimeNew)
seq.gravrad  =     rebin(seq0.gravrad[ *,mTimeOld:nTimeOld]  , nr, nTimeNew)

if (total(tag_names(seq0) eq 'RHOTIL') gt 0.0) then begin
  seq.rhotil   =     rebin(seq0.rhotil[  *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
endif
seq.rhovb    =     rebin(seq0.rhovb[   *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
if (total(tag_names(seq0) eq 'RHOVBRMS') gt 0.0) then begin
  seq.rhovbrms =sqrt(rebin(seq0.rhovbrms[*,mTimeOld:nTimeOld]^2, nr+1, nTimeNew))
endif

if (total(tag_names(seq0) eq 'FRHOV1B') gt 0.0) then begin
  seq.frhov1b  =     rebin(seq0.frhov1b[ *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
  seq.frhov2b  =     rebin(seq0.frhov2b[ *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
  seq.frhov3b  =     rebin(seq0.frhov3b[ *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
endif

seq.feipb    =     rebin(seq0.feipb[   *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
seq.fekb     =     rebin(seq0.fekb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
seq.fegb     =     rebin(seq0.fegb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
seq.fepb     =     rebin(seq0.fepb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
seq.fevb     =     rebin(seq0.fevb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
seq.ferb     =     rebin(seq0.ferb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)

for iquc=0,nquc-1 do begin
  qucsid ='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
  dummy=execute('seq.' + qucsid + '=rebin(seq0.' + qucsid +'[*,mTimeOld:nTimeOld], nr, nTimeNew)')
endfor
for iquc=0,nfquc-1 do begin
  fqucsid ='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
  dummy=execute('seq.' + fqucsid + '=rebin(seq0.' + fqucsid +'[*,mTimeOld:nTimeOld], nr+1, nTimeNew)')
endfor

if (total(tag_names(seq0) eq 'BC1') gt 0.0) then begin
  ; --- Magnetic field data ---
  seq.Bc1      =     rebin(seq0.Bc1[     *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc1abs   =     rebin(seq0.Bc1abs[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc1rms   =     rebin(seq0.Bc1rms[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc2      =     rebin(seq0.Bc2[     *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc2abs   =     rebin(seq0.Bc2abs[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc2rms   =     rebin(seq0.Bc2rms[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc3      =     rebin(seq0.Bc3[     *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc3abs   =     rebin(seq0.Bc3abs[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  seq.Bc3rms   =     rebin(seq0.Bc3rms[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
endif
if (total(tag_names(seq0) eq 'FEMB') gt 0.0) then begin
  ; --- Magnetic field data ---
  seq.femb     =     rebin(seq0.femb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
  seq.feyb     =     rebin(seq0.feyb[    *,mTimeOld:nTimeOld]  , nr+1, nTimeNew)
endif

if (isCentral_flag) then begin
  ; --- Bin additional terms for central potential ---
  seq.luminositypervolume=rebin(seq0.luminositypervolume[ mTimeOld:nTimeOld], nTimeNew)
  seq.rhototal1=rebin(seq0.rhototal1[ mTimeOld:nTimeOld], nTimeNew)
  seq.rhototal2=rebin(seq0.rhototal2[ mTimeOld:nTimeOld], nTimeNew)
  seq.rhototal3=rebin(seq0.rhototal3[ mTimeOld:nTimeOld], nTimeNew)
  seq.x1center=rebin(seq0.x1center[ mTimeOld:nTimeOld], nTimeNew)
  seq.x2center=rebin(seq0.x2center[ mTimeOld:nTimeOld], nTimeNew)
  seq.x3center=rebin(seq0.x3center[ mTimeOld:nTimeOld], nTimeNew)
  seq.v1center=rebin(seq0.v1center[ mTimeOld:nTimeOld], nTimeNew)
  seq.v2center=rebin(seq0.v2center[ mTimeOld:nTimeOld], nTimeNew)
  seq.v3center=rebin(seq0.v3center[ mTimeOld:nTimeOld], nTimeNew)
  seq.I1l=rebin(seq0.I1l[ mTimeOld:nTimeOld], nTimeNew)
  seq.I1r=rebin(seq0.I1r[ mTimeOld:nTimeOld], nTimeNew)
  seq.I2l=rebin(seq0.I2l[ mTimeOld:nTimeOld], nTimeNew)
  seq.I2r=rebin(seq0.I2r[ mTimeOld:nTimeOld], nTimeNew)
  seq.I3l=rebin(seq0.I3l[ mTimeOld:nTimeOld], nTimeNew)
  seq.F1l=rebin(seq0.F1l[ mTimeOld:nTimeOld], nTimeNew)
  seq.F1r=rebin(seq0.F1r[ mTimeOld:nTimeOld], nTimeNew)
  seq.F2l=rebin(seq0.F2l[ mTimeOld:nTimeOld], nTimeNew)
  seq.F2r=rebin(seq0.F2r[ mTimeOld:nTimeOld], nTimeNew)
  seq.F3l=rebin(seq0.F3l[ mTimeOld:nTimeOld], nTimeNew)
  seq.F3r=rebin(seq0.F3r[ mTimeOld:nTimeOld], nTimeNew)
  ;
  if ((total(tag_names(seq0) eq 'ANGULARMOMENTUM3') gt 0.0) and $
      (total(tag_names(seq0) eq 'MOMENTOFINERTIA3') gt 0.0)    )  then begin
    seq.MomentOfInertia3   =     rebin(seq0.MomentOfInertia3[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
    seq.AngularMomentum3   =     rebin(seq0.AngularMomentum3[  *,mTimeOld:nTimeOld]  , nr, nTimeNew)
  endif
  ;
  for iwave=0,nwave-1 do begin
    ; --- Rebin wave amplitudes ---
    lwave=floor(sqrt(float(iwave)+0.1))
    mwave=iwave - lwave^2 - lwave
    ;
    lwaveStr=strmid('000' + strtrim(string(    lwave) , 2), 2, /reverse)
    mwaveStr=strmid('000' + strtrim(string(abs(mwave)), 2), 2, /reverse)
    if (mwave lt 0) then begin
      mwaveStr='m' + mwaveStr
    endif else begin
      mwaveStr='p' + mwaveStr
    endelse
    ;
    waveid ='rhovb' + 'l' + lwaveStr + 'm' + mwaveStr
    ;
    dummy=execute('seq.' + waveid + '=rebin(seq0.' + waveid + '[*,mTimeOld:nTimeOld], nr+1, nTimeNew)')
  endfor ; (iwave)
  ;
endif ; (isCentral_flag)

; --- Return result structure ---
return, seq

end ; seqrebin
