function seqreduce, seq0, mtime=mtime, ntime=ntime
;
;------------------------------------------------------------------------------
;+
; NAME:
;   seqreduce ('sequence_reduce')
;
; PURPOSE:
;   Reduce number of arrays in sequence structure.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   result=seqreduce(seq, mtime=mtime, ntime=ntime)
;
; INPUT:
;   seq:       ('sequence') input sequence structure (e.g. read with rhd_seqpp_collect.pro)
;
; OUTPUT:
;   result:    ('sequence_reduced') Same type of structure as input seq, less entries
;
; KEYWORDS:
;   mtime:     ('m_time') integer with index of first time step to be considered,
;                         default: 0
;   ntime:     ('n_time') integer with index of last time step to be considered,
;                         default: last time step in structure
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   rhd_seq_put2struct: Collect data from CO5BOLD mean files
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2017-11-06 (B.F. Uppsala) Adapted from seqrebin.pro
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

nstr='__NAME'

; --- Check parameters ---
BinFactor   =1
seqBinFactor=1

if (keyword_set(ntime)) then begin
  navg=(long(ntime)/seqBinFactor) < (n_elements(seq0.time)-1)
endif else begin
  navg=n_elements(seq0.time)-1
endelse

if (keyword_set(mtime)) then begin
  if (mtime ge 0) then begin
    mavg=long(mtime)/seqBinFactor
  endif else begin
    mavg=(navg+long(mtime)/seqBinFactor) > 0
  endelse
endif else begin
  mavg=0
endelse

; --- Number of time steps in new structure  ---
nTimeNew=(navg-mavg+1)/(BinFactor/seqBinFactor)
if (nTimeNew le 0) then begin
  message, 'Binfactor is larger than the number of time steps in the input structure.'
endif
mTimeOld=mavg
nTimeOld=mTimeOld+nTimeNew*(BinFactor/seqBinFactor)-1
help, mtime, ntime, mavg, navg, mTimeOld, nTimeOld, nTimeNew

dummy=where(tag_names(seq0) eq ('XC3'+nstr), ReadMetaData)
dummy=where(tag_names(seq0) eq ('BINFACTOR'), BinFactorIsSet_flag)
if (BinFactorIsSet_flag) then begin
  BinFactor=seq0.BinFactor
endif else begin
  BinFactor=1
endelse
help, ReadMetaData, BinFactor

;isCentral_flag=(strmatch(seq0.par.grav_mode, 'central' ,/fold))

; --- Create new structure ---
;rhd_seq_put2struct, seq0.mean, seq, par=seq0.par, $
;  /InitStruct, modelname=seq0.model, modelident=seq0.modelident, n_timestep=nTimeNew, $
;  ReadMetaData=ReadMetaData

seq=create_struct( $
  'model'     , seq0.model, $
  'par'       , seq0.par, $
  'BinFactor' , BinFactor, $
  'itime'     , seq0.itime[     mTimeOld:nTimeOld], $
  'time'      , seq0.time[      mTimeOld:nTimeOld], $
  'JulianDate', seq0.JulianDate[mTimeOld:nTimeOld], $
  'ifile'     , seq0.ifile[     mTimeOld:nTimeOld] $
  )

; --- Return result structure ---
return, seq

end ; seqreduce
