pro sphereintegrate, $
  Teff=Teff, r0_grav=r0_grav, r1_grav=r1_grav, r2_grav=r2_grav, mass_star=mass_star, $
  gas=gas, opta=opta, $
  Ptop=Ptop, Radius=Radius, RSurface=RSurface, Center=Center, smax=smax, facsmax=facsmax, $
  gfac=gfac, $
  dlntau=dlntau0, tauthick=tauthick, approximation=approximation, $
  tauin=tauin, Tin=Tin, noshift=noshift, $
  tauout=tau, Tout=T, Pout=P, zout=z, gout=g, $
  rhoout=rho, kapout=kap
;
;------------------------------------------------------------------------------
;+
; NAME:
;   sphereintegrate (sphere_integrate)
;
; PURPOSE:
;   Produce a stratification from a spherical potential to get an initial model.
;
; CATEGORY:
;   Computations (Special)
;
; CALLING SEQUENCE:
;   sphereintegrate
;
; INPUT:
;   All via keywords
;
; OUTPUT:
;   All via keywords
;
; KEYWORDS:
;   ...
;
; COMMON BLOCKS:
;   None
;
; ROUTINES AND FUNCTIONS:
;   cmopta:      Interpolate opacities from data in input structure
;   gasinter:    Interpolate EOS quantities from data in input structure
;   spheregrav:  Compute spherical (smoothed) gravitational potential
;   tempgreytau: Compute temperature structure from optical depth
;   xkaros:      Interpolate opacities from data in common block
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Slow
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-06-09 (B.F. Copenhagen) Written
;   1999-06-10 (B.F.) Gravity corrected
;   1999-06-11 (B.F.) Interpolate between temperature gradients
;   1999-06-30 (B.F.) Function with keyword 'clip'
;   2000-02-24 (B.F.) Parameter facsmax made optional
;   2000-04-27 (B.F.) Keyword r1_grav added
;   2000-05-26 (B.F.) jmax=4000
;   2003-01-30 (B.F. Uppsala) jmax=40000, dT=dTad for tau > 10^4
;   2007-07-18 (B.F. Lyon) constant entropy in deep interior only for facsmax<=1
;   2014-05-27 (B.F. Uppsala) New keyword r2_grav, ()->[], keyword RSurface added;
;              entropy limiter consistent with sphereintegraterhoei
;   2014-06-03 (B.F.) Add parameter opta und use it in interpolation with cmopta;
;              constant tauthick becomes input parameter
;-
;------------------------------------------------------------------------------
;
; --- Set some compiler options ---
compile_opt defint32, strictarr
;
; --- Define default input keyword parameters ---
if (n_elements(Teff     ) eq 0) then Teff     =5770.0
if (n_elements(r0_grav  ) eq 0) then r0_grav  =0.0
if (n_elements(r1_grav  ) eq 0) then r1_grav  =0.0
if (n_elements(r2_grav  ) eq 0) then r2_grav  =0.0
if (n_elements(mass_star) eq 0) then mass_star=1.98900e+33
if (n_elements(Radius   ) eq 0) then Radius   =6.98000e+10
if (n_elements(Center   ) eq 0) then Center   =0.0
if (n_elements(Ptop     ) eq 0) then Ptop     =20.0
if (n_elements(smax     ) eq 0) then smax     =1.0E+35
if (n_elements(dlntau0  ) eq 0) then dlntau0  =0.1
if (n_elements(gfac     ) eq 0) then gfac     =1.0
if (n_elements(tauthick ) eq 0) then tauthick =2.0
;
luminosity=-1.0
;mugreyfac =0.8
mugreyfac =0.4
if (keyword_set(facsmax)) then begin
  smaxfac=facsmax
endif else begin
  smaxfac   =0.995
endelse
Pmax=1.0E+08
;
if (not keyword_set(approximation)) then approximation='mihalas'
;
; --- Generate arrays ---
if (n_elements(tauin) ne 0) then begin
  jmax=n_elements(tauin)
endif else begin
;  jmax=4000
  jmax=50000
endelse
tau=dblarr(jmax)
T  =dblarr(jmax)
P  =dblarr(jmax)
z  =dblarr(jmax)
g  =dblarr(jmax)
rho=dblarr(jmax)
kap=dblarr(jmax)
;
; --- Start values ---
z[0]=radius
g[0]=gfac*spheregrav(z[0], r0=r0_grav, r1=r1_grav, r2=r2_grav, C=!con.G*mass_star)
;print, g[0]
if ((n_elements(tauin) ne 0) and (n_elements(Tin) ne 0)) then begin
  T[0]=Tin[jmax-1]
endif else begin
  T[0]=Teff*tempgreytau(0.0, approximation=approximation)
  if (keyword_set(RSurface)) then begin
    T[0]=T[0]*sqrt(RSurface/z[0])
  endif
endelse
;
if (n_elements(tauin) ne 0) then begin
  ; --- tau scale prescribed ---
  if (ptop gt 0.0) then begin
    ; --- Given tau[0], P[0] ---
    tau[0]=tauin[jmax-1]
    P[0]=ptop
  endif else begin
    ; --- Compute P[0] from tau[0] ---
    tau[0]=tauin[jmax-1]
    Piterold=0.0
    Piternew=abs(ptop)
    Perr=Piternew
    niter=0
    while ((niter lt 25) and (abs(Piternew-Piterold) ge 0.001*Perr)) do begin
      ; --- Compute opacity kap ---
      if (keyword_set(opta)) then begin
        kap0=cmopta(Piternew, T[0], 0, /clip, opta=opta)
      endif else begin
        kap0=xkaros(Piternew, T[0], 0, /clip)
      endelse
      Piterold=Piternew
      ; --- Compute new pressure ---
      Piternew=0.5*(tau[0]*g[0]/kap0+Piterold)
      print, 'iter:', niter, ':', Piterold, ' ->', Piternew
      niter=niter+1
    endwhile
    P[0]=Piternew
  endelse
endif else begin
  ; --- Compute tau[0] from P[0] ---
  P[0]=abs(ptop)
  ; --- Compute opacity ---
  if (keyword_set(opta)) then begin
    kap0=cmopta(P[0], T[0], 0, /clip, opta=opta)
  endif else begin
    kap0=xkaros(P[0], T[0], 0, /clip)
  endelse
  ; --- tau[0]=0.75*kap*rho*Hp ---
  ;tau[0]=0.75*kap0*P[0]/g[0] ; 10.4.95: factor 0.75 removed
  tau[0]=kap0*P[0]/g[0]
endelse
;
T_const=sqrt(sqrt(0.75D+00)) * 0.25D+00
j=0L
; --- Do the integration (Heun scheme, cf. Stoer, Burlisch, II., p. 109) ---
while ((z[j] gt Center) and (j lt (jmax-1))) do begin
  ;print,'j,jmax,tau,z,P,T', j,jmax,tau[j],z[j],P[j],T[j]
  ; --- Get next tau point ---
  if (n_elements(tauin) ne 0) then begin
    ; --- Use input tau axis ---
    tau[j+1]=tauin(jmax-1-(j+1))
    dlntau  =alog(tau[j+1]/tau[j])
    dtau    =tau[j+1]-tau[j]
  endif else begin
    ; --- Compute next optical depth point (regular spacing in log(tau) ---
    tau[j+1]=tau[j]*exp(dlntau0)
    dlntau  =dlntau0
    dtau    =(exp(dlntau0)-1.0)*tau[j]
  endelse
  ;
  ; --- Temperature ---
  if ((n_elements(tauin) ne 0) and (n_elements(Tin) ne 0)) then begin
    T[j+1]=Tin(jmax-1-(j+1))
  endif else if (tau[j+1] lt tauthick) then begin
    ; --- Compute temperature from formula for grey atmosphere ---
    T[j+1]=Teff*tempgreytau(tau[j+1], approximation=approximation)
    if (keyword_set(RSurface)) then begin
      T[j+1]=T[j+1]*sqrt(RSurface/z[j])
    endif
  endif else begin
    if (luminosity le 0.0) then begin
      ; --- Define luminosity ---
      luminosity=4.0*!pi*(z[j]^2+r0_grav^2) * !con.sigmasb*Teff^4
      print,'Luminosity=', luminosity, luminosity/!con.Lsun, ' at r=', z[j]
    endif
;    T[j+1]=T[j] + dtau * T_const / sqrt(sqrt(tau[j+1]))^3 * $
;                  sqrt(sqrt(luminosity/(4.0D+00*!dpi*(z[j]^2+r0_grav^2) * !con.sigmasb)))
    T[j+1]=T[j] + dtau * T_const * tau[j+1]^(-0.75) * $
                  (luminosity/(4.0*!pi*(z[j]^2+r0_grav^2) * !con.sigmasb))^0.25
;    T[j+1]=T[j] + dtau * T_const / sqrt(sqrt(tau[j+1]))^3 * $
;                  (luminosity/(4.0*!pi*(z[j]^2+r0_grav^2) * !con.sigmasb))^0.25
  endelse
  dTgrey=T[j+1]-T[j]
  ;
  ; --- Compute density rho from EOS with gasinter ---
  if (P[j] le Pmax) then begin
    gasinter, [P[j]], [T[j]], rho1, xcp1, xgam1, cxt1, h1, s1, eos=gas
    rho1   =rho1[0]
    rho1fac=T[j]*rho1/P[j]
    if (smax ge 0.0) then begin
      xcp1   =xcp1[0] > 0.0
      s1     =s1[0]
    endif
  endif else begin
    rho1=rho1fac*P[j]/T[j]
  endelse
  ;
  ; --- Local gravity ---
  ;g1=!con.G*mass_star * (z[j]^2+r0_grav^2)^(-1.5) * abs(z[j])
  g1=gfac*spheregrav(z[j], r0=r0_grav, r1=r1_grav, r2=r2_grav, C=!con.G*mass_star)
  ;
  ; --- Compute opacity kappa ---
  if (keyword_set(opta)) then begin
    kappa1=cmopta(P[j], T[j], 0, /clip, opta=opta)
  endif else begin
    kappa1=xkaros(P[j], T[j], 0, /clip)
  endelse
  ;
  rho[j]=rho1
  kap[j]=kappa1
  g[j]  =g1
  ;
  if (smax ge 0.0) then begin
    dTad=1.0/(rho1*xcp1) * g1/kappa1 * dtau
    ;  if ((tau[j] gt 1.0E+04) and (smaxfac le 1.0)) then begin --- 2014-05-27 (B.F.) Outcommented ---
    ;  if (tau[j] gt 1.0E+04) then begin ; --- 2004-03-10: tau value increased
    if (tau[j] gt 1.0E+08) then begin
      dT1 = dTad
    endif else if ((s0 ge         smax) and (tau[j] gt 1.0)) then begin
      dT1 = dTad
    endif else if ((s1 ge smaxfac*smax) and (tau[j] gt 1.0)) then begin
      ; --- Correct temperature ---
      ;    T[j+1]=T[j] + 1.0/(rho1*xcp1) * g1/kappa1 * dtau
      mugrey=mugreyfac*((smax-s1)/smax) > 0.0
      dT1 = mugrey * dTgrey + (1.0-mugrey) * dTad
    endif else begin
      dT1=dTgrey
    endelse
  endif else begin
    dT1=dTgrey
  endelse
  T[j+1]=T[j] + dT1
;print, j, tau[j], dtau, P[j], T[j],  rho1, xcp1, g1, kappa1, dT1
;print, j, tau[j], dtau, P[j], T[j],  rho1, g1, kappa1, dT1
  ;
  ; --- Coefficients of DEQ ---
  fP1=g1/kappa1
  fz1=-1.0/(kappa1*rho1)
  ; --- Compute predictor values for next point ---
;  P1=P[j]+dtau*fP1
;  z1=z[j]+dtau*fz1
;  print,j,P[j],T[j],dtau,fP1,g[j],kappa1
  P1=P[j]*exp(dtau*fP1/P[j])
  z1=z[j]+dlntau*fz1*tau[j]
  ;
  ; --- Compute density rho from EOS with gasinter ---
  if (P1 le Pmax) then begin
    gasinter, [P1], [T[j+1]], rho2, xcp2, xgam2, cxt2, h2, s2, eos=gas
    rho2   =rho2[0]
    rho2fac=T[j+1]*rho2/P1
    if (smax ge 0.0) then begin
      xcp2   =xcp2[0] > 0.0
      s2     =s2[0]
    endif
  endif else begin
    rho2   =rho2fac*P1/T[j+1]
  endelse
  ;
  ; --- Local gravity ---
;  g2=!con.G*mass_star * (z1^2+r0_grav^2)^(-1.5) * abs(z1)
  g2=gfac*spheregrav(z1, r0=r0_grav, r1=r1_grav, r2=r2_grav, C=!con.G*mass_star)
  ;
  ; --- Compute opacity kappa ---
  if (keyword_set(opta)) then begin
    kappa2=cmopta(P1, T[j+1], 0, /clip, opta=opta)
  endif else begin
    kappa2=xkaros(P1, T[j+1], 0, /clip)
  endelse
  ;
  if (smax ge 0.0) then begin
    dTad=1.0/(rho2*xcp2) * g2/kappa2 * dtau
    if (tau[j] gt 1.0E+08) then begin
      dT2 = dTad
    endif else if ((s0 ge         smax) and (tau[j] gt 1.0)) then begin
      dT2 = dTad
    endif else if ((s1 ge smaxfac*smax) and (tau[j] gt 1.0)) then begin
      ; --- Correct temperature ---
      ;    T[j+1]=T[j] + (0.5/(rho1*xcp1) * g1/kappa1 + $
      ;                   0.5/(rho2*xcp2) * g2/kappa2) * dtau
      mugrey=mugreyfac*((smax-s2)/smax) > 0.0
      dT2 = mugrey * dTgrey + (1.0-mugrey) * dTad
    endif else begin
      dT2=dTgrey
    endelse
  endif else begin
    dT2=dTgrey
  endelse
  T[j+1]=T[j] + 0.5*(dT1+dT2)
  ;
  ; --- Coefficients of DEQ ---
  fP2=g2/kappa2
  fz2=-1.0/(kappa2*rho2)
  ; --- Compute final values for next point ---
;  P[j+1]=P[j] + dtau*0.5 * (fP1+fP2)
;  z[j+1]=z[j] + dtau*0.5 * (fz1+fz2)
  P[j+1]=P[j] * exp(dtau * (fP1+fP2)/(P[j]+P1))
  z[j+1]=z[j] + dlntau * 0.25*(fz1+fz2)*(tau[j]+tau[j+1])
;  z[j+1]=z[j] + dlntau * 0.5*(fz1+fz2)*sqrt(tau[j]*tau[j+1])
;  z[j+1]=z[j] + dlntau * 0.5*(fz1*tau[j]+fz2*tau[j+1])
  ; --- Increment j ---
  j=j+1L
endwhile
;
rho[j]=rho2
kap[j]=kappa2
g[  j]=g2
;
; --- Reduce output arrays to proper size and reverse them ---
tau=reverse(tau[0:j-1])
T  =reverse(T[  0:j-1])
P  =reverse(P[  0:j-1])
z  =reverse(z[  0:j-1])
g  =reverse(g[  0:j-1])
rho=reverse(rho[0:j-1])
kap=reverse(kap[0:j-1])
;
tau=float(tau)
T  =float(T  )
P  =float(P  )
z  =float(z  )
g  =float(g  )
rho=float(rho)
kap=float(kap)
;
; --- Shift z axis so that tau(z=0)=1 ---
;if (not keyword_set(noshift)) then begin
;  ztau1=interpol(z, tau, 1.0)
;  z=z-ztau1[0]
;endif
;
end ; sphereintegrate
