pro sphereintegraterhoei, $
  Teff=Teff, $
  r0_grav=r0_grav, r1_grav=r1_grav, r2_grav=r2_grav, gravCorePotDepth=gravCorePotDepth, mass_star=mass_star, $
  gas=gas, eos=eos, $
  Ptop=Ptop, Radius=Radius, RSurface=RSurface, Center=Center, smax=smax, facsmax=facsmax, $
  dlntau=dlntau0, approximation=approximation, $
  tauin=tauin, Tin=Tin, noshift=noshift, $
  tauout=tau, Tout=T, Pout=P, zout=z, gout=g, $
  rhoout=rho, eiout=ei, kapout=kap
;
;------------------------------------------------------------------------------
;+
; NAME:
;   sphereintegraterhoei ('sphere_integrate_rho_ei')
;
; PURPOSE:
;   Compute spherical stellar structure.
;
; CATEGORY:
;   RHD, stellar atmospheres, stellar interiors
;
; CALLING SEQUENCE:
;
; INPUT:
;   None (all via keywords)
;
; OUTPUT:
;   None (all via keywords)
;
; KEYWORDS:
;   ...
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   tempgreytau:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Slow
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-06-09 (B.F. Copenhagen) Written
;   1999-06-10 (B.F.) Gravity corrected
;   1999-06-11 (B.F.) Interpolate between temperature gradients
;   1999-06-30 (B.F.) Function with keyword 'clip'
;   2000-02-24 (B.F.) Parameter facsmax made optional
;   2000-04-27 (B.F.) Keyword r1_grav added
;   2000-05-26 (B.F.) jmax=4000
;   2003-01-30 (B.F. Uppsala) jmax=40000, dT=dTad for tau > 10^4
;   2003-01-30 (B.F.) sphereintegrate -> sphereintegraterhoei
;   2004-03-10 (B.F. Montpellier) dT=dTad for tau > 10^8
;   2010-12-17 (B.F. Lyon) () -> []
;   2012-05-14 (B.F. & W.S.) Bugfix: ei[0] =h0-P[0]  ->   ei[0] =h0-P[0]/rho[0],
;                   Keyword RSurface
;   2014-05-27 (B.F. Uppsala) New keyword r2_grav, in-file potential functions outcommented
;   2018-04-19 (B.F.) Change comments; add parameter gravDeep
;   2018-04-25 (B.F.) Rename parameter gravDeep -> gravCorePotDepth
;-
;------------------------------------------------------------------------------
;
; --- Define default input keyword parameters ---
if (n_elements(Teff     ) eq 0) then Teff     =5770.0
if (n_elements(r0_grav  ) eq 0) then r0_grav  =0.0
if (n_elements(r1_grav  ) eq 0) then r1_grav  =0.0
if (n_elements(r2_grav  ) eq 0) then r2_grav  =0.0
if (n_elements(mass_star) eq 0) then mass_star=1.98900e+33
if (n_elements(Radius   ) eq 0) then Radius   =6.98000e+10
if (n_elements(Center   ) eq 0) then Center   =0.0
if (n_elements(smax     ) eq 0) then smax     =1.0E+35
if (n_elements(dlntau0  ) eq 0) then dlntau0  =0.1
if (n_elements(Ptop     ) eq 0) then Ptop     =20.0

timeall=0.0D+00

tauthick  =2.0
luminosity=-1.0
;mugreyfac =0.8
mugreyfac =0.4 ; --- Normal value
;mugreyfac =0.01
if (keyword_set(facsmax)) then begin
  smaxfac=facsmax
endif else begin
  smaxfac=0.995
endelse
;
if (not keyword_set(approximation)) then approximation='mihalas'
;
; --- Generate arrays ---
if (n_elements(tauin) ne 0) then begin
  jmax=n_elements(tauin)
endif else begin
  jmax=50000
endelse
tau=dblarr(jmax)
T  =dblarr(jmax)
P  =dblarr(jmax)
z  =dblarr(jmax)
g  =dblarr(jmax)
rho=dblarr(jmax)
ei =dblarr(jmax)
kap=dblarr(jmax)
;
; --- Start values ---
z[0]=radius
g[0]=spheregrav(z[0], r0=r0_grav, r1=r1_grav, r2=r2_grav, C=!con.G*mass_star, gravCorePotDepth=gravCorePotDepth)
print, 'Outermost point: gravity=    ', g[0]
;
if ((n_elements(tauin) ne 0) and (n_elements(Tin) ne 0)) then begin
  T[0]=Tin[jmax-1]
endif else begin
  T[0]=Teff*tempgreytau(0.0, approximation=approximation)
  if (keyword_set(RSurface)) then begin
    T[0]=T[0]*sqrt(RSurface/z[0])
  endif
endelse
print, 'Outermost point: temperature=', T[0]
;
if (n_elements(tauin) ne 0) then begin
  ; --- tau scale prescribed ---
  if (ptop gt 0.0) then begin
    ; --- Given tau[0], P[0] ---
    tau[0]=tauin[jmax-1]
    P[0]=ptop
  endif else begin
    ; --- Compute P[0] from tau[0] ---
    tau[0]=tauin[jmax-1]
    Piterold=0.0
    Piternew=abs(ptop)
    Perr=Piternew
    niter=0
    while ((niter lt 25) and (abs(Piternew-Piterold) ge 0.001*Perr)) do begin
      ; --- Compute opacity kap ---
      kap0=xkaros(Piternew, T[0], 0, /clip)
      Piterold=Piternew
      ; --- Compute new pressure ---
      Piternew=0.5*(tau[0]*g[0]/kap0+Piterold)
      print, 'iter:', niter, ':', Piterold, ' ->', Piternew
      niter=niter+1
    endwhile
    P[0]=Piternew
  endelse
endif else begin
  ; --- Compute tau[0] from P[0] ---
  P[0]=abs(ptop)
  ; --- Compute opacity kap ---
  kap0=xkaros(P[0], T[0], 0, /clip)
  ; --- tau[0]=0.75*kap*rho*Hp ---
  ;tau[0]=0.75*kap0*P[0]/g[0] ; 10.4.95: factor 0.75 removed
  tau[0]=kap0*P[0]/g[0]
endelse
print, 'Outermost point: pressure=   ', P[0]
print, 'Outermost point: tau=        ', tau[0]
;
; --- Density and internal energy at starting point ---
gasinter, [P[0]], [T[0]], rho0, xcp0, xgam0, cxt0, h0, s0, eos=gas
rho[0]=rho0
ei[0] =h0-P[0]/rho[0]
print, 'Outermost point: s=          ', s0
print, 'Outermost point: rho=        ', rho[0]
print, 'Outermost point: ei=         ', ei[0]
;
T_const=sqrt(sqrt(0.75D+00)) * 0.25D+00
j=0L
;print, j, ei[j], rho[j], P[j], T[j], tau[j], z[j], g[j]
; --- Do the integration (Heun scheme, cf. Stoer, Burlisch, II., p. 109) ---
while ((z[j] gt Center) and (j lt (jmax-1))) do begin
  ;print,'j,jmax,tau,z,P,T', j,jmax,tau[j],z[j],P[j],T[j]
  ;
  ; --- Next tau point ---
  if (n_elements(tauin) ne 0) then begin
    tau[j+1]=tauin[jmax-1-(j+1)]
    dlntau=alog(tau[j+1]/tau[j])
    dtau=tau[j+1]-tau[j]
  endif else begin
    ; --- Next optical depth point ---
    tau[j+1]=tau[j]*exp(dlntau0)
    dlntau=dlntau0
    dtau=(exp(dlntau0)-1.0)*tau[j]
  endelse
  ;
  ; --- grey Temperature ---
  if ((n_elements(tauin) ne 0) and (n_elements(Tin) ne 0)) then begin
    T[j+1]=Tin[jmax-1-(j+1)]
  endif else if (tau[j+1] lt tauthick) then begin
    ; --- Compute temperature from formula for grey atmosphere ---
    T[j+1]=Teff*tempgreytau(tau[j+1],approximation=approximation)
    if (keyword_set(RSurface)) then begin
      T[j+1]=T[j+1]*sqrt(RSurface/z[j])
    endif
  endif else begin
    if (luminosity le 0.0) then begin
      ; --- Define luminosity ---
      luminosity=4.0*!pi*(z[j]^2+r0_grav^2) * !con.sigmasb*Teff^4
      print,'Luminosity=', luminosity, luminosity/!con.Lsun, ' at r=', z[j]
    endif
;    T[j+1]=T[j] + dtau * T_const / sqrt(sqrt(tau[j+1]))^3 * $
;                  sqrt(sqrt(luminosity/(4.0D+00*!dpi*(z[j]^2+r0_grav^2) * !con.sigmasb)))
    T[j+1]=T[j] + dtau * T_const * tau[j+1]^(-0.75) * $
                  (luminosity/(4.0*!pi*(z[j]^2+r0_grav^2) * !con.sigmasb))^0.25
;    T[j+1]=T[j] + dtau * T_const / sqrt(sqrt(tau[j+1]))^3 * $
;                  (luminosity/(4.0*!pi*(z[j]^2+r0_grav^2) * !con.sigmasb))^0.25
  endelse
  dTgrey=T[j+1]-T[j]
;help, dTgrey
 ;
  ; === Predictor step ===
  ;
  rho0=rho[j]
  ei0 =ei[j]
  z0  =z[j]
  ;
  ; --- EOS ---
;time0=systime(1)
  eosinter, [rho[j]], [ei[j]], P0, dPdrho0, dPdei0, T0, dTdei0, s0, eos=eos
;timeall=timeall+(systime(1)-time0)
;help, timeall
  P0     =P0[0]
  dPdrho0=dPdrho0[0]
  dPdei0 =dPdei0[0]
  T0     =T0[0]
  dTdei0 =dTdei0[0]
  s0     =s0[0]
  xcp0   =P0/(T0*rho0) + rho0/T0 * dPdrho0/dPdei0
  ;print, j, ei[j], rho[j], P0, dPdrho0, dPdei0, T0, dTdei0, s0, ' <--- After first EOS'
  ;
  ; --- Opacity ---
  kappa0=xkaros(P[j], T[j], 0, /clip)
  ;
  ; --- Local gravity ---
  g0=(spheregrav(z[j], r0=r0_grav, r1=r1_grav, r2=r2_grav, C=!con.G*mass_star, gravCorePotDepth=gravCorePotDepth))[0]
  ;print, 'z[j]=', z[j]
  ;help, r0_grav, r1_grav, r2_grav, gravCorePotDepth, g0
  ;
  P[j]  =P0
  T[j]  =T0
  kap[j]=kappa0
  g[j]  =g0
  ;
  ; --- Temperature step ---
  dTad=1.0/(rho0*xcp0) * g0/kappa0 * dtau
;  if (tau[j] gt 1.0E+04) then begin ; --- 2004-03-10: tau value increased
  if (tau[j] gt 1.0E+08) then begin
    dT1 = dTad
  endif else if ((s0 ge         smax) and (tau[j] gt 1.0)) then begin
    dT1 = dTad
  endif else if ((s0 ge smaxfac*smax) and (tau[j] gt 1.0)) then begin  ; --- Outcommented: BF 2012-05-14 ---
  ;endif else if ((s0 ge smaxfac*smax) and (tau[j] gt 0.1)) then begin
    ; --- Correct temperature ---
    ;mugrey=mugreyfac*((smax-s0)/smax) > 0.0
    mugrey=sqrt(mugreyfac*((smax-s0)/smax)) > 0.0
    dT1 = mugrey * dTgrey + (1.0-mugrey) * dTad
  endif else begin
    dT1=dTgrey
  endelse
  T[j+1]=T[j] + dT1
;print, j, tau[j], dtau, P0, T0,  rho0, xcp0, g0, kappa0, dTad, dT1
  ;
  ; --- Steps ---
  dP1= g0/kappa0         * dtau
  dz1=-1.0/(kappa0*rho0) * dtau
  ;
  dTdrho0=T0/rho0^2 * dPdei0  - P0/rho0^2 * dTdei0
  Deter1 =dTdei0    * dPdrho0 - dPdei0    * dTdrho0
  ;
  drho1 =( dTdei0  * dP1 - dPdei0  * dT1) / Deter1
  dei1  =(-dTdrho0 * dP1 + dPdrho0 * dT1) / Deter1
  ;
  ; --- Compute predictor values for next point ---
;  P1=P[j]*exp(dP1/P[j])
;  z1=z[j]+dlntau*fz1*tau[j]
  ei1 =ei[j]  + dei1
  rho1=rho[j] + drho1
  P1  =P[j]   + dP1
  z1  =z[j]   + dz1
  ;
  ; === Corrector step ===
  ;
  ; --- Fake ---
  dei2 =dei1
  drho2=drho1
  dP2  =dP1
  dz2  =dz1
  ;
  kappa1=kappa0
  g1=g0
  ;
  ; --- Update ---
  ei[j+1] =ei0  + 0.5*(dei1  + dei2 )
  rho[j+1]=rho0 + 0.5*(drho1 + drho2)
  P[j+1]  =P0   + 0.5*(dP1   + dP2  )
  z[j+1]  =z0   + 0.5*(dz1   + dz2  )
  ;
  ; --- Increment j ---
  j=j+1L
;print, j, ei[j], rho[j], P[j], T[j], tau[j], z[j], g[j-1]
endwhile
;
kap[j]=kappa1
g[  j]=g1
;
; --- Reduce output arrays to proper size and reverse them ---
tau=float(tau)
T  =float(T  )
P  =float(P  )
z  =float(z  )
g  =float(g  )
rho=float(rho)
ei =float(ei )
kap=float(kap)
;
tau=reverse(tau[0:j-1])
T  =reverse(T[  0:j-1])
P  =reverse(P[  0:j-1])
z  =reverse(z[  0:j-1])
g  =reverse(g[  0:j-1])
rho=reverse(rho[0:j-1])
ei =reverse(ei[ 0:j-1])
kap=reverse(kap[0:j-1])
;
; --- Shift z axis so that tau(z=0)=1 ---
;if (not keyword_set(noshift)) then begin
;  ztau1=interpol(z,tau,1.0)
;  z=z-ztau1[0]
;endif
;
end ; sphereintegraterhoei
