function TEMPGREYTAU, tau, approximation=approximation
;------------------------------------------------------------------------------
;+
; NAME:
;   TEMPGREYTAU
;
; PURPOSE:
;   Compute the temperature stratification of a (non-grey) atmosphere
;   according to Unsoeld 1968, Eq. (40,3) or an other formula.
;
; CATEGORY:
;   Computation (Special)
;
; CALLING SEQUENCE:
;   T=tempgreytau(tau)
;
; INPUTS:
;   tau - Optical depth (>=0)
;
; OUTPUTS:
;   T = tempgreytau(tau)
;
; KEYWORDS:
;   approximation: Input string: determines the approximation:
;           'unsoeld': (Unsoeld 1968, Eq. (40,3), p. 145), default
;           'eddington': q=2/3
;           'chandrasekhar1': Chandrasekhar, n=1, (Mihalas 1970, p. 52)
;           'chandrasekhar2': Chandrasekhar, n=2
;           'chandrasekhar3': Chandrasekhar, n=3
;           'chandrasekhar4': Chandrasekhar, n=4
;           'mihalas': Exact solution interpolated from table
;                      (Mihalas, 'Stellar Atmospheres', Table 3-2)
;           'hdfit':   Fit to RHD model 'zt142g80n1'
;           'morelga': Morel, Formula (3) in Gabriel 1996, A&A 309, 939
;           'holmu':   Holweger-Mueller, Table II in Holweger & Muller (1974)
;                      Solar Physics 39, 19-30
;           'holmuga': Holweger-Mueller, Formula (4) in Gabriel 1996
;           'krishna-swamy': Krishna Swamy, 1966, ApJ 145, 174
;           'hdconv_0': Mean T(tau)-relation derived by averaging RHD radiative
;                       flux profiles, computing T(tau) and fitting q(tau),
;                       T(tau) for tau>1 -> T(tau)_radiative
;           'hdconv_1': Mean T(tau)-relation derived by averaging RHD radiative
;                       flux profiles, computing T(tau) and fitting q(tau),
;                       T(tau) for tau>>1 -> T(tau)_radiative
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   qgrey : Approximation for the Hopf function
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The input variable tau should be not less then 0.
;
; PROCEDURE:
;   Straightforward: tempgreytau=(0.75*(tau+qgrey(tau)))^0.25
;
; EXAMPLE:
;   ; --- Compute the (radiative) temperature stratification for the Sun ---
;   tau=exp(findgen(100)/8.0-8.0)
;   Teff=5770.0
;   T=Teff*tempgreytau(tau)
;   plot_oi, tau, T
;
; MODIFICATION HISTORY:
;   27.10.94 Written by Bernd Freytag, Astrophysik Kiel
;   31.10.94 Keyword APPROXIMATION added
;-
;------------------------------------------------------------------------------
;
; --- Default approximation ---
if (not keyword_set(approximation)) then app='unsoeld' $
                                    else app=strlowcase(approximation)
;
return, (0.75*(tau+qgrey(tau,approximation=app)))^0.25
;
end ; TEMPGREYTAU
