;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   abu_extract ('abu_extract')
;
; PURPOSE:
;   Read a file with abundances and put the data into a structure.
;
; CATEGORY:
;   Abundances
;
; CALLING SEQUENCE:
;   @abu_extract
;
; EXAMPLE:
;   @abu_extract
;
; MODIFICATION HISTORY:
;   2015-09-02 (B.F. Uppsala) Written
;-
;---------------------------------------------------------------------------------------------------

; --- Define file name and give some meta data ---
AbuDisk=getenv('HOME') + '/dat/abu/Caffau09/'
AbuID  ='Caffau09'
AbuFile=AbuID + '.tab'
AbuWeb ='http://adsabs.harvard.edu/abs/2009MmSAI..80..643C'
;AbuRef ='Caffau2009MmSAI..80..643C'
AbuRef ='Caffau, E., Ludwig, H.-G, & Steffen, M. 2009, MmSAI, 80, 643C'
AbuOut ='Abundances_' + AbuID + '.sav'

; --- Read the file with the element abundances ---
RdArray, AbuDisk + AbuFile, a, s, /notransform
help, s
print, s[0, *]


EleSym=['H', 'He', reform(s[1:-3, 0])]

sAbu=fltarr(n_elements(s[*, 0])+1, n_elements(s[0, *])-2)
sAbu[2:*, *]=float(s[1:*, 2:*])

; Use X = Z/(Z/X) ; Y=1.0-X-Z
Z=  reform(sAbu[-2, 0:*:2])
X=Z/reform(sAbu[-1, 0:*:2])
Y=1.0-X-Z
print, X
print, Y
print, Z
print, Z/X

wX=1.0080 & wY=4.0026
abuX=X/wX
abuY=Y/wY

; --- Add hydrogen and helium abundance ---
sAbu[0, 0:*:2]=12.0
sAbu[1, 0:*:2]=12.0+alog10(abuY/abuX)

; --- Strip Z and Z/X entries from abundance table ---
sAbu=sAbu[0:-3, *]


; --- Set file name for atomic-weight data and load it ---
EleID  ='IUPAC_2013_Atomic_Weights'
EleOut =EleID + '.sav'
restore, EleOut, /verbose

; --- Compute abundances --- 
EleInd=strCrossCmp(strtrim(EleSym, 2), Ele.Symbol, v2=Ele.Weight, Default=0.0, v1=EleWeight, /FOLD_CASE)
;Ind   =sort(EleInd)
;EleSym=EleSym[Ind]
EleZ  =Ele.Z[EleInd]
;for i=0, n_elements(sAbu[0,*])-1 do begin &$
;  sAbu[*, i]=sAbu[Ind, i] &$
;endfor ; i

; --- Put the data into a structure ---
Abu= { $
  ID:             AbuID, $
  File:           AbuFile, $
  Path:           AbuDisk, $
  Webpage:        AbuWeb, $
  Reference:      AbuRef, $
  Date:           strDate(systime(/JULIAN)), $
  TableOriginal:  s, $
  ElementSymbols: EleSym, $
  ElementZ:       EleZ, $
  Ca09: { $
  Ref:               AbuRef, $
  X:                 X[0], $
  Y:                 Y[0], $
  Z:                 Z[0], $
  ZoX:               Z[0]/X[0], $
  ElementAbundances: reform(sAbu[*, 0]), $
  Error:             reform(sAbu[*, 1]) $
        }, $
  AG89: { $
  Ref:               'Anders, E., & Grevesse, N. 1989, Geochim. Cosmochim. Acta, 53, 197', $
  X:                 X[1], $
  Y:                 Y[1], $
  Z:                 Z[1], $
  ZoX:               Z[1]/X[1], $
  ElementAbundances: reform(sAbu[*, 2]), $
  Error:             reform(sAbu[*, 3]) $
        }, $
  GS98: { $
  Ref:               'Grevesse, N., & Sauval, A. J. 1998, Space Science Reviews, 85, 161', $
  X:                 X[2], $
  Y:                 Y[2], $
  Z:                 Z[2], $
  ZoX:               Z[2]/X[2], $
  ElementAbundances: reform(sAbu[*, 4]), $
  Error:             reform(sAbu[*, 5]) $
        }, $
  AGS05: { $
  Ref:               'Asplund, M., Grevesse, N., & Sauval, A. J. 2005, ASP Conf. Ser. 336: Cosmic Abundances as Records of Stellar Evolution and Nucleosynthesis, 336, 25', $
  X:                 X[3], $
  Y:                 Y[3], $
  Z:                 Z[3], $
  ZoX:               Z[3]/X[3], $
  ElementAbundances: reform(sAbu[*, 6]), $
  Error:             reform(sAbu[*, 7]) $
        }, $
  AGSS09: { $
  Ref:               'Asplund, M., Grevesse, N., Sauval, A. J. & Scott, P. 2009, ARA&A, 47, 481', $
  X:                 X[4], $
  Y:                 Y[4], $
  Z:                 Z[4], $
  ZoX:               Z[4]/X[4], $
  ElementAbundances: reform(sAbu[*, 8]), $
  Error:             reform(sAbu[*, 9]) $
        } $
     }

help, /str, Abu

; --- Save the output structure to a file ---
save, file=AbuOut, Abu

;EleAbu=reform(sAbu[*, 0])
;EA =10.0^EleAbu
;EAT=total(EA)
;abuX=EA[0]/EAT
;abuY=EA[1]/EAT
;abuZ=total(EA[2:*])/EAT
;help, abuX, abuY, abuZ, abuX+abuY+abuZ
;;
;EW =EA * EleWeight
;EWT=total(EW)
;X=EW[0]/EWT
;Y=EW[1]/EWT
;Z=total(EW[2:*])/EWT
;Qmol=EWT/EAT
;help, X, Y, Z, X+Y+Z, Qmol
