;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_com_ExtractAtmRing ('opacity_compute_Extract_Atmosphere_Ring')
;
; PURPOSE:
;   Construct a 1D stellar atmosphere from data from multidimensional dynamical model
;
; CATEGORY:
;   Opacities, hydrodynamics
;
; CALLING SEQUENCE:
;   @opac_com_extractatm
;
; PROCEDURE;
;   Load file with 'ring data', likely produced with rhd_ring_collect.pro.
;   Extract mean temperature, pressure, and density.
;   Optionally, add some points at the top.
;   Interpolate to different grid.
;   Put the results into a structure.
;   Put some meta data into another structure.
;   Write both structures into an IDL save file.
;
; MODIFICATION HISTORY:
;   2015-08-19 (B.F. Uppsala) First version
;-
;---------------------------------------------------------------------------------------------------

; --- Set model name and data ---
Model='st28gm06n25' & mx1=20 & nx1=-5 & mr1=0 & nr1=0 & mf1=20 & nf1=-1 & nPertau10=15 & TeffTarget=2800.0

InputFileID='_ring_a08-11.sav'
FileData   =Model + InputFileID

; --- Load the data file
;restore, FileData, /verbose
par=ring.par

; --- Extract a number of 2D arrays depending on height and sample number ---
nx=n_elements(ring.T_mean[-1-nx1:-1-mx1, 0      , 0, 0      ])
nm=n_elements(ring.T_mean[ 0         , mr1:nr1, *, mf1:nf1])
help, nx, nm
;Tm  =reform(ring.T_mean[  -1-nx1:-1-mx1, mr1:nr1, *, mf1:nf1], nx, nm)
Tm  =reform(ring.T_mean4[ -1-nx1:-1-mx1, mr1:nr1, *, mf1:nf1], nx, nm)
Pm  =reform(ring.P_mean[  -1-nx1:-1-mx1, mr1:nr1, *, mf1:nf1], nx, nm)
rhom=reform(ring.rho_mean[-1-nx1:-1-mx1, mr1:nr1, *, mf1:nf1], nx, nm)
rm  =reform(ring.r_mean[  -1-nx1:-1-mx1, mr1:nr1, *, mf1:nf1], nx, nm)
taum=       ring.tau_mean[-1-nx1:-1-mx1]
mum =!con.Rgas * (rhom*Tm)/Pm

help, Tm, Pm, rhom, rm, mum

; --- Filter ---
itau   =60
itauMid=40
itauTop=0
dlogT=deriv(alog10(Tm))
dlogT[ 0,*]=0.0
dlogT[-1,*]=0.0
dlogTmin=min(dlogT, dim=1)
Ind=where(((rhom[itauTop, *] le 3.0E-15) and $
           (rhom[itauMid, *] le 6.0E-11) and $
           (rhom[itau   , *] le 3.0E-09) and $
           (rhom[itau   , *] ge 1.0E-10) and $
           (Tm[  itau   , *] gt 3000.0 ) and $
           (dlogTmin         gt -0.005 )      ), n)
;Ind=where(((rhom[itauTop, *] le 3.0E-15) and $
;           (rhom[itauMid, *] le 6.0E-11) and $
;           (rhom[itau   , *] le 3.0E-09) and $
;           (rhom[itau   , *] ge 1.0E-10) and $
;           (Tm[  itau   , *] gt 3000.0 )      ), n)
;Ind=where(((Tm[  itau, *] gt 2600.0 ) and $
;           (Tm[  itau, *] le 4000.0 ) and $
;           (rhom[itau, *] le 10.0E-10)      ), n)
;Ind=where(((rhom[itau, *] ge  5.0E-10)      ), n)
;Ind=where(((Tm[  itau, *] gt 3000.0 )       ), n)
help, n

; --- Construct 1D stratifications ---
;ftau =1.0 + 100.0*taum/(1.0+100.0*taum)*3.0
;ftau =rebin(ftau, nx, n)
x1D  =          mean(       rm[  *, Ind]  , dim=2)
mu1D =          mean(      mum[  *, Ind]  , dim=2)
;T1D  =     10.0^mean(alog10(Tm[  *, Ind]) , dim=2)
;T1D  =           mean(       Tm[  *, Ind]  , dim=2)
T1D  =sqrt(sqrt(mean(       Tm[  *, Ind]^4, dim=2)))
;T1D  =mean(       Tm[  *, Ind]^ftau, dim=2)^(1.0/ftau)
;
P1D  =     10.0^mean(alog10(Pm[  *, Ind]) , dim=2)
;P1D  =           mean(       Pm[  *, Ind] , dim=2)
;P1D  =       1.0/mean(   1.0/Pm[  *, Ind] , dim=2)
;
rho1D=     10.0^mean(alog10(rhom[*, Ind]) , dim=2)
;rho1D=          mean(       rhom[*, Ind]  , dim=2)
;rho1D=       1.0/mean(   1.0/rhom[*, Ind]  , dim=2)
;rho1D=(mu1D*P1D)/(!con.Rgas*T1D)

;T1D=10.0^smooth(alog10(T1D), 3)

; --- Compute grey opacities on 1D stratification ---
;eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
; --- Read parameter, EOS, and opacity file ---
  ;tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
;    dfopta, opadisk + par.opafile &$
if (not keyword_set(EOS)) then begin &$
  if (par.opafile ne '') then begin &$
    rdopta, opadisk + par.opafile, opadat &$
    opa=1 &$
  endif else begin &$, drange=alog10([min(Tm), max(T1D)])
    opa=0 &$
  endelse &$
endif
kappa1D=xkaros(P1D, T1D, 0)

; --- Integrate the optical-depth scale ---
dx1D        =abs(deriv(x1D))
dtau1D      =rho1D * kappa1D * dx1D
dtau1D[1:-1]=0.5*(dtau1D[0:-2]+dtau1D[1:-1])
tau1D       =total(dtau1d, /CUMULATIVE)

; --- Determine at which optical depth the effective temperature is reached ---
tauTeffTarget=10.0^(interpol(alog10(tau1D), T1D, [TeffTarget]))[0]

; --- Rescale the geometrical scale to make sure that T(tau=2/3)=T_eff ---
ScaleFactor=(2./3.)/tauTeffTarget
help, TeffTarget, tauTeffTarget, ScaleFactor
x1D        =x1D * ScaleFactor

; --- Optionally, extrapolate to lower densities ---
;nplus=60 & mdelta=15 & ndelta=5
nplus=35 & mdelta=15 & ndelta=5
if (nplus gt 0) then begin &$
  n1Do  =n_elements(x1D) &$
  x1Do  =x1D &$
  mu1Do =mu1D &$
  T1Do  =T1D &$
  P1Do  =P1D &$
  rho1Do=rho1D &$
  tau1Do=tau1D &$
  n1D   =nplus+n1Do &$
  x1D   =fltarr(n1D) &$
  mu1D  =fltarr(n1D) &$
  T1D   =fltarr(n1D) &$
  P1D   =fltarr(n1D) &$
  rho1D =fltarr(n1D) &$
  x1D[  nplus:-1]=x1Do &$
  mu1D[ nplus:-1]=mu1Do &$
  P1D[  nplus:-1]=P1Do &$
  T1D[  nplus:-1]=T1Do &$
  rho1D[nplus:-1]=rho1Do &$
  dx=(x1Do[mdelta]-x1Do[ndelta])/float(mdelta-ndelta) &$
  Hp  =abs((x1Do[mdelta]-x1Do[ndelta])/alog(P1Do[  mdelta]/P1Do[  ndelta])) &$
  HT  =abs((x1Do[mdelta]-x1Do[ndelta])/alog(T1Do[  mdelta]/T1Do[  ndelta])) &$
  Hrho=abs((x1Do[mdelta]-x1Do[ndelta])/alog(rho1Do[mdelta]/rho1Do[ndelta])) &$
  print, dx, Hp, HT, Hrho &$
  x1D[  0:nplus-1]=(-float(nplus)+findgen(nplus))*dx + x1Do[0] &$
  mu1D[ 0:nplus-1]=mu1Do[ 0] &$
  P1D[  0:nplus-1]=P1Do[  0]*exp(-(x1d[0:nplus-1]-x1Do[0])/Hp  ) &$
  T1D[  0:nplus-1]=T1Do[  0]*exp(-(x1d[0:nplus-1]-x1Do[0])/HT  ) &$
  rho1D[0:nplus-1]=rho1Do[0]/(P1Do[0]/T1Do[0]) * P1D[0:nplus-1]/T1D[0:nplus-1] &$
endif

; --- Re-integrate the optical-depth scale ---
kappa1D     =xkaros(P1D, T1D, 0)
dx1D        =abs(deriv(x1D))
dtau1D      =rho1D * kappa1D * dx1D
dtau1D[1:-1]=0.5*(dtau1D[0:-2]+dtau1D[1:-1])
;tau1D       =total(dtau1d, /CUMULATIVE) + rho1D[0]*kappa1D[0] * (Hp-0.5*dx1D[0])
tau1D       =total(dtau1d, /CUMULATIVE) + rho1D[0] * kappa1D[0] * (Hp/(exp(1.0)-1.0) - 0.5*dx1D[0])

; --- Interpolate onto regular grid (equidistant in log tau?) ---
x1D0  =x1D
mu1D0 =mu1D
T1D0  =T1D
P1D0  =P1D
rho1D0=rho1D
;
logtau1D=alog10(tau1D)
logtauMin=min(logtau1D, max=logtauMax)
nlogtauRef=long((logtauMax-logtauMin) * float(nPertau10)) + 1
logtauRef=logtauMin + (logtauMax-logtauMin) * (findgen(nlogtauRef)/float(nlogtauRef-1))
help, nlogtauRef, logtauRef
x1D  =     interpol(       x1D0   , logtau1D, logtauRef, /SPLINE)
mu1D =     interpol(       mu1D0  , logtau1D, logtauRef, /SPLINE)
T1D  =10.0^interpol(alog10(T1D0  ), logtau1D, logtauRef, /SPLINE)
P1D  =10.0^interpol(alog10(P1D0  ), logtau1D, logtauRef, /SPLINE)
rho1D=10.0^interpol(alog10(rho1D0), logtau1D, logtauRef, /SPLINE)
tau1D=10.0^logtauRef

; --- Reverse ---
x1D  =reverse(x1D  )
mu1D =reverse(mu1D )
T1D  =reverse(T1D  )
P1D  =reverse(P1D  )
rho1D=reverse(rho1D)
tau1D=reverse(tau1D)

; --- Put atmosphere data into structure ---
Atm= { $
  Description:       '1D reference atmosphere', $
  x:                 x1D, $
  mu:                mu1D, $
  T:                 T1D, $
  P:                 P1D, $
  rho:               rho1D $
     }

; --- Assemble some meta data ---
rhdmodeldata, modDB
;restore, 'allbasdb.sav', /verbose
MetaAtm= { $
  Description: 'Meta data of 1D reference atmosphere', $
  Model:             Model, $
  FileData:          FileData, $
  FileSource:        Model + '_atm_ring_02.sav', $
  Code:              'CO5BOLD', $
  Teff:              (udb_extract('Teff'     , [Model], basDB, Default=0.0))[0], $
  Teffout:           (udb_extract('Teffout'  , [Model], basDB, Default=0.0))[0], $
  TeffTarget:        TeffTarget, $
  grav:              (udb_extract('grav'     , [Model], basDB, Default=0.0))[0], $
  Date:              strDate(systime(/JULIAN)), $
  Author:            (udb_extract('Author'   , [Model], modDB, Default='' ))[0], $
  mx1:               mx1, $
  nx1:               nx1, $
  mr1:               mr1, $
  nr1:               nr1, $
  mf1:               mf1, $
  nf1:               nf1, $
  nplus:             nplus, $
  mdelta:            mdelta, $
  ndelta:            ndelta, $
  nPertau10:         nPertau10, $
  ScaleFactor:       ScaleFactor $
         }

; --- Plot ---
!P.MULTI=[0,2,2,0,0]

plotlines, alog10(taum ), alog10(Tm), xrange=[-12.0,2.5], drange=alog10([min(T1D)*0.9, max(Tm)])
plotlines, alog10(taum ), alog10(Tm[  *,ind]), /over, linecol=!ctv.yellow, thick=1
plotlines, alog10(tau1D), alog10(T1D), /over, linecol=!ctv.red, thick=5

plotlines, alog10(taum ), alog10(mum), xrange=[-12.0,2.5], drange=alog10([min(mum), max(mum)])
plotlines, alog10(taum ), alog10(mum[*,ind]), /over, linecol=!ctv.yellow, thick=1
plotlines, alog10(tau1D), alog10(!con.Rgas*rho1D*T1D/P1D), /over, linecol=!ctv.blue, thick=5
plotlines, alog10(tau1D), alog10(mu1D), /over, linecol=!ctv.red, thick=5

plotlines, alog10(taum ), alog10(rhom), xrange=[-12.0,2.5], drange=alog10([min(rho1D)*0.7, max(rhom)])
plotlines, alog10(taum ), alog10(rhom[*,ind]), /over, linecol=!ctv.yellow, thick=1
plotlines, alog10(tau1D), alog10(rho1D), /over, linecol=!ctv.red, thick=5

plotlines, alog10(taum ), alog10(Pm), xrange=[-12.0,2.5], drange=alog10([min(P1D)*0.7, max(Pm)])
plotlines, alog10(taum ), alog10(Pm[*,ind]), /over, linecol=!ctv.yellow, thick=1
plotlines, alog10(tau1D), alog10(P1D), /over, linecol=!ctv.red, thick=5

!P.MULTI=0

; --- Store into file ---
save, Atm, MetaAtm, file=MetaAtm.FileSource, /verbose
