; pro rhd_isotausurface
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_isotausurface ('radiation_hydrodynamics_iso_tau_surface')
;
; PURPOSE:
;   Load model data for single time step, interpolate temperature on iso-tau surface.
;
; CATEGORY:
;   RHD, analysis
;
; CALLING SEQUENCE:
;   .run rhd_isotausurface
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   The input/output system UIO has to be initialized (once), e.g. with
;     uio_init, progrm='by hand'
;
;   Before first use of rhd_isotausurface (or using a different EOS or opacity table), type
;     delvar, EOS
;
; PROCEDURE:
;   Load snapshot from specified RHD model,
;   load parameter file,
;   load EOS table,
;   load opacity table,
;   compute EOS and opacity data for snapshot,
;   interpolate temperature 
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2017-01-17 (B.F. Uppsala) Derived from rhd_load
;-
;---------------------------------------------------------------------------------------------------

; === Manual setup =====================

modelident=''

; --- Set model name ---
;model='st26gm09n02' & modelident='_07'
;model='st28gm06n01' & modelident='_10'
;model='st28gm06n02' & modelident='_29'
;model='st28gm06n03' & modelident='_08'
;model='st28gm06n04' & modelident='_02'
;model='st28gm06n05' & modelident='_01'
;model='st28gm06n06' & modelident='_23'
;model='st28gm06n07' & modelident='_03d'
;model='st28gm06n11' & modelident='_01'
;model='st28gm06n12' & modelident='_011'
;model='st28gm06n13' & modelident='_080'
;model='st28gm06n14' & modelident='_100'
;model='st28gm06n15' & modelident='_001'
;model='st28gm06n16' & modelident='_010'
;model='st28gm06n18' & modelident='_014'
;model='st28gm06n19' & modelident='_040'
;model='st28gm06n22' & modelident='_002'
;model='st28gm06n23' & modelident='_010'
;model='st28gm06n25' & modelident='_100'
;model='st28gm06n26' & modelident='_080'
;model='st28gm06n27' & modelident='_005'
;model='st28gm06n28' & modelident='_005'
;model='st28gm06n031' & modelident='_080'

;model='st33gm06n02' & modelident='_21'
;model='st33gm06n03' & modelident='_01'
;model='st35gm06n02' & modelident='_01'

;model='st35gm04n01' & modelident='_50'
;model='st35gm04n02' & modelident='_20'
;model='st35gm04n03' & modelident='_20'
;model='st35gm04n04' & modelident='_33'
;model='st35gm04n06' & modelident='_12'
;model='st35gm04n07' & modelident='_29'
;model='st35gm04n08' & modelident='_03'
;model='st35gm04n22' & modelident='_32'
;model='st35gm04n23' & modelident='_08'
;model='st35gm04n24' & modelident='_13'
;model='st35gm04n26' & modelident='_75'
;model='st35gm04n27' & modelident='_08'
model='st35gm04n28' & modelident='_50'
;model='st35gm04n29' & modelident='_26'
;model='st35gm04n29' & modelident='_sum'
;model='st35gm04n30' & modelident='_01'
;model='st35gm04n31' & modelident='_22'
;model='st35gm04n32' & modelident='_10'
;model='st35gm04n34' & modelident='_01'
;model='st35gm04n36' & modelident='_048'
;model='st35gm04n37' & modelident='_137'
;model='st35gm04b0n001' & modelident='_009'
;model='st35gm04b1n001' & modelident='_446'
;model='st35gm04b1n002' & modelident='_850'
;model='st35gm04b1n003' & modelident='_500'
;model='st35gm04b1n004' & modelident='_140'
;model='st35gm04b1n005' & modelident='_184'

;model='st35gm03n01' & modelident='_01'
;model='st35gm03n02' & modelident='_23'
;model='st35gm03n04' & modelident='_01'
;model='st35gm03n06' & modelident='_73t-20'
;model='st35gm03n07' & modelident='_150'
;model='st35gm03n09' & modelident='_80'
;model='st35gm03n10' & modelident='_70'
;model='st35gm03n11' & modelident='_06'
;model='st35gm03n13' & modelident='_008'
;model='st35gm03n14c' & modelident='_001'
;model='st35gm03n15' & modelident='_008'
;model='st35gm03n16' & modelident='_020'
;model='st35gm03n17' & modelident='_200'
;model='st35gm03n19' & modelident='_100'

;model='rhd' & modelident=''

; --- Set some paths (model master directory, EOS data, opacities) ---
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/rhd/tmp/'
EOSdisk=getenv('HOME') + '/for/eos/dat/'
;EOSdisk=getenv('HOME') + '/for/eos/ddat/'
opadisk=getenv('HOME') + '/for/opa/dat/'

; --- Select the snapshot ---
modelfile=modeldisk + model + modelident + '.end'  & n=0
;modelfile=modeldisk + model + modelident + '.full' & n=0
;modelfile=modeldisk + model + modelident + '.sta'  & n=0
;modelfile=modeldisk + model + modelident + '.snap'  & n=0


; === Read files =======================

; --- Read parameter file ---
ierr=0
parfile=modeldisk + model + modelident + '.par'
par=uio_struct_rd(parfile, ierr=ierr)
if (ierr gt 0) then stop

; --- Read EOS and opacity file ---
if (not keyword_set(EOS)) then begin
  print, 'Load EOS data from ' + EOSdisk + par.EOSfile
  tabinter_rdcoeff, EOSdisk + par.EOSfile, EOS
  if (par.opafile ne '') then begin
    print, 'Load opacity data from ' + opadisk + par.opafile
    rdopta, opadisk + par.opafile, opadat
    opa=1
  endif else begin
    opa=0
  endelse
endif else begin
  print, 'Re-use EOS and opacity data'
endelse

; --- Load the model data (skip entries by adding them to the blacklist) ---
ierr=0
print, 'Load ' + modelfile
ful=uio_dataset_rd(modelfile, n=n, ierr=ierr)
;ful=uio_dataset_rd(modelfile, n=n, blacklist=['v1','v2',v3'], ierr=ierr)
if (ierr gt 0) then stop


; === Compute ==========================

; --- Compute EOS and opacities ---
print, 'Compute EOS and opacities (may take a while)'
;eosbox, ful, eos=EOS, opa=opa, out_q=0, ierror=ierror & if (ierror ne 0) then help,ierror
eosbox, ful, eos=EOS, opa=opa, out_q=1, ierror=ierror & if (ierror ne 0) then help,ierror

help, /str, ful
;help, /str, ful.opa
;help, /str, ful.eos

; --- Set a reference log(tau) value (avoid zero) ---
logtauRefValue=1.0E-30
logtaur       =[logtauRefValue]
;logtaur=-findgen(12)/12.0+6.0

; --- Extract 3D temperature and x3 arrays ---
T=combox('T'   , ful)
;x=combox('xc3a', ful)

; --- Compute log tau scale (for all columns of 'left' or 'right' x3-surface) ---
;logtau=alog10(combox('tau3l', ful))
logtau=alog10(combox('tau3r', ful))

; --- Interpolate onto given tau value ---
Ttau0=reform(interbox(T, logtau, x_reg=logtaur, orient=3, bound=0))
;xtau =reform(interbox(x, logtau, x_reg=logtaur, orient=3, bound=0))

; --- Compute mask ---
;taumask=reform(logtau[*,*,-1]) gt logtauRefValue
taumask=reform(logtau[*,*,0]) gt logtauRefValue

; --- Multiply array with mask ---
Ttau=Ttau0*taumask

; --- Show what has been produced ---
help, Ttau0, Ttau, taumask
;surface, taumask
;surface, Ttau

end ; rhd_isotausurface
