; rhd_mlt_alphafromstrat
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_mlt_alphafromstrat ('alpha_mixing-length-theory_alpha_from_stratification')
;
; PURPOSE:
;   Compute mixing-length alpha from stratification of RHD models
;
; CATEGORY:
;   RHD, analysis
;
; CALLING SEQUENCE:
;   @rhd_mlt_alphafromstrat
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2012-04-25 (B.F. Lyon) First version
;   2012-04-26 (B.F.) Compute nabla_el
;-
;------------------------------------------------------------------------------

; --- Set input model name ---
if (not keyword_set(external_control)) then begin &$
  Model='st36g45n04'  & m=75 & n=130 & mcolor=[!ctv.white] &$
;Model='st36g45n04rot' & m=25 & n=118 & mcolor=[!ctv.red] &$
;Model='st36g45n04rot2' & m=45 & n=125 & mcolor=[!ctv.green] &$
endif

; --- Load data with averaged model data ---
SaveFile=Model + '_seq_0.sav'
restore, SaveFile

; --- Compute basic thermodynamic quantities from averaged models ---
cp      =seqavg('cp'      , seq, unitsystem='cgs', uniteflux='cgs')
delta   =seqavg('delta'   , seq, unitsystem='cgs', uniteflux='cgs')
grav    =seqavg('grav'    , seq, unitsystem='cgs', uniteflux='cgs')
Hp      =seqavg('Hp'      , seq, unitsystem='cgs', uniteflux='cgs')
kapparho=seqavg('kapparho', seq, unitsystem='cgs', uniteflux='cgs')
nabla_ad=seqavg('nabla_ad', seq, unitsystem='cgs', uniteflux='cgs')
nabla   =seqavg('nabla'   , seq, unitsystem='cgs', uniteflux='cgs')
P       =seqavg('P'       , seq, unitsystem='cgs', uniteflux='cgs')
rho     =seqavg('rho'     , seq, unitsystem='cgs', uniteflux='cgs')
T       =seqavg('T'       , seq, unitsystem='cgs', uniteflux='cgs')
xc      =seqavg('xc'      , seq, unitsystem='cgs', uniteflux='cgs')
; --- Compute additional thermodynamic quantities from averaged models ---
s       =seqavg('s'       , seq, unitsystem='cgs', uniteflux='cgs')

; --- Compute basic hydrodynamic quantities from averaged models ---
v       =seqavg('v3rms'   , seq, unitsystem='cgs')
Fconv   =seqavg('feip0'   , seq, unitsystem='cgs', uniteflux='cgs', /intertoxc)

; --- Compute nabla_element ---
nabla_el=nabla_ad
nablatic=nabla-nabla_ad
for i=0,n_elements(T)-1 do begin &$
  ; --- Constant ---
  C=((12.0D+00*!con.sigmaSB*T^3*grav)/(1.0D+00*kapparho*rho*cp*v^2))^2 * (1.0D+00*rho*delta)/(2.0D+00*p) &$
  if (nablatic[i] gt 0.0) then begin &$
    ;print, i &$
    ; --- Polynomial coefficients ---
    Coeff=[-nablatic[i]^3, 3.0D+00*nablatic[i]^2, 1.0D+00/C[i]-3.0D+00*nablatic[i], 1.0D+00] &$
    ; --- Derivative ---
    CoeffD=(Coeff*findgen(n_elements(Coeff)))[1:*] &$
    ; --- Iteration ---
    xeps=nablatic[i]*1.0D-06 &$
    iter=0 &$
    xnew=0.0D+00 &$
    repeat begin &$
      iter=iter+1 &$
      xold=xnew &$
      xnew=xold - poly(xold,Coeff)/poly(xold,CoeffD) &$
      ;print, i, iter, xnew, abs(xnew-xold) &$
    endrep until ((iter ge 20) or (abs(xnew-xold) lt xeps)) &$
    nabla_el[i]=xnew+nabla_ad[i] &$
  endif &$
endfor

; --- Approximate delta nabla ---
deltanabla=nabla-nabla_ad
;deltanabla=nabla-nabla_el

; --- Solve Kippenhahn & Weigert (7.7) for flux alpha ---
alphaF=sqrt(8.0*sqrt(2.0)*Fconv/(sqrt(rho*P*delta)*cp*T*deltanabla^1.5))

; --- Solve Kippenhahn & Weigert (7.6) for velocity alpha ---
alphaV=v * sqrt((8.0*rho)/(delta*P*deltanabla))

; --- Estimate Coriolis number Omega*L/V (inverse of Rossby number, U/(L*f))---
if (total(tag_names(seq.par) eq 'NU_ROTATION') gt 0.0) then begin &$
  Co=2.0*!pi*seq.par.nu_rotation*Hp/v &$
endif else begin &$
  ; --- No rotation parameter found: set Coriolis number to zero ---
  Co=v*0.0 &$
endelse
   
; --- Determine proper range within convection zone, far away from center ---
smin=min(s, ntop)
;n=long(ntop*0.86)
;m=long(ntop*0.25)
help,ntop,m,n

if (not keyword_set(external_control)) then begin &$
  ; --- Plot derived alpha values over Coriolis parameter ---
  plotlines, Co[m:n], alphaV[m:n], $
    over=(model ne 'st36g45n04'), $
    xrange=[0.0, 11.0], drange=[0.0,7.0], $
    xtitle='Coriolis number', ytitle='!4a!X', $
    linestyle=1, $
    linecolor=mcolor &$
  plotlines, Co[m:n], alphaF[m:n], $
    /over, $
    linecolor=mcolor &$
  ;
  CoVec=findgen(200)/10.0 &$
  plotlines, CoVec, 3.0*exp(-CoVec/12.0), $
    /over, $
    linecolor=!ctv.blue &$
endif

; end ; rhd_mlt_alphafromstrat
