; pro rhd_power2d1d_start
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_power2d1d_start ('rhd_power_2_dimensions_1_dimension_start')
;
; PURPOSE:
;   Compute 1D size distributions from sequence of 2D intensity images.
;
; CATEGORY:
;   Radiation hydrodynamics, data analysis
;
; CALLING SEQUENCE:
;   .run rhd_power2d1d_start
;    
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   pwer2d1d: Compute 1D size distribution from 2d quadratic image
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; TODO:
;
; EXAMPLE:
;   .run rhd_power2d1d_start
;
; MODIFICATION HISTORY:
;   2015-12-08 (B.F. Lyon) First version
;   2015-12-15 (B.F. Uppsala) First run over model grid
;-
;---------------------------------------------------------------------------------------------------

;Model='d3t57g44b0n001'
;Model='d3t68g43mm00n01'
;Model='mt40g50mm00n04'

;ModelSet='d3t6*mm00n??_intens.sav'
;ModelSet='d3t57g44b0n033_intens.sav'
;ModelSet='mt[1-4]?g50mm00n*_intens.sav'
ModelSet='mt40g50mm00n05_intens.sav'
;ModelSet='{mt28g50mm00n05,mt30g50mm00n12,mt32g50mm00n03,mt33g50mm00n14}_intens.sav'
;ModelSet='mt*g*mm00n*_intens.sav'
OutputID='intpow'

WaveDelta=0.025
Version  ='Power2D1D 2016.02.04'


ModelFileList=file_search(ModelSet, count=nModel)
if (nModel eq 0) then message, 'No model found for set ' + ModelSet

; --- Loop over all models ---
for iModel=0L,nModel-1L do begin
  ;
  ModelFile=ModelFileList[iModel]
  ;
  print
  print, 'Restore ', ModelFile
  intens=0
  restore, ModelFile
  Model=intens.Model
  ;
  dim=size(intens.I3r, /dimension)
  if (dim[0] ne dim[1]) then begin
    print, Model, ' does not have a square surface.'
  endif else begin ; (dim[0] ne dim[1])
    print, Model, ' has a quadratic surface.'
    ;
    ; --- First and second are equal: proceed ---
    ;
    help, /str, intens
    ;
    ntime=n_elements(intens.time)
    ;
    ; --- Loop over all time steps ---
    for itime=0L, (ntime-1L) do begin
      ;
      ; --- Print progress information every now and then ---
      if (((itime/100L)*100L) eq itime) then print, itime, '/', ntime-1L
      ;
      ; --- Pick the quantity to analyse ---
      q =reform(intens.I3r[*,*,itime])
      qd=double(q)
      ;
      if (itime eq 0L) then begin
        ; --- Analyse first snapshot and initialize some arrays ---
        ;
        ; --- Compute 1D power spectrum of 2D snapshot ---
        power2d1d, q, qs, WaveDelta=WaveDelta, $
                   ComputeNorm_flag=1, qn=qn, qw=qw, qsn=qsn, $
                   WaveNumber1DDown=WaveNumber1DDown, WaveNumber1DUp=WaveNumber1DUp
        ;
        ; --- Remove entries with zero weight ---
        IndNonZero=where(qw gt 0.0, nIndNonZero)
        if (nIndNonZero eq n_elements(qw)) then nIndNonZero=0
        if (nIndNonZero gt 0) then begin
          qsn             =qsn[             IndNonZero]
          qw              =qw[              IndNonZero]
          qn              =qn[              IndNonZero]
          WaveNumber1DDown=WaveNumber1DDown[IndNonZero]
          WaveNumber1DUp  =WaveNumber1DUp[  IndNonZero]
        endif
        ;
        ; --- Create some arrays ---
        qsAr=fltarr(n_elements(qw), ntime)
        qmeanAr=fltarr(ntime)
        qdrmsAr=fltarr(ntime)
        ;
      endif else begin
        ; --- Perform follow-up analyse step ---
        ;
        ; --- Compute 1D power spectrum of 2D snapshot ---
        power2d1d, q, qs, WaveDelta=WaveDelta
        ;
      endelse
      ;
      ; --- Put non-zero entries of size spectrum into array ---
      if (nIndNonZero gt 0) then begin
        qs=qs[IndNonZero]
      endif
      qsAr[*, itime]=qs
      ;
      ; --- Compute mean and rms intensity ---
      qmean         =mean(qd)
      qmeanAr[itime]=float(qmean)
      qdrmsAr[itime]=float(sqrt(mean((qd-qmean)^2)))
    endfor ; itime
    ;
    if (dim[1] eq 1) then begin
      qNormAr=rebin(reform(1.0/qsn, n_elements(qw), 1), n_elements(qw), ntime)
    endif else begin
      ;qNormAr=rebin(reform(qn, n_elements(qn), 1), n_elements(qn), ntime)
      ;qNormAr=rebin(reform(qw, n_elements(qw), 1), n_elements(qw), ntime)
      qNormAr=rebin(reform(qw/qsn, n_elements(qw), 1), n_elements(qw), ntime)
    endelse
    ;
    if (total(tag_names(intens) eq 'TIME_DB') eq 0.0) then begin
      time_db=intens.time
    endif else begin
      time_db=intens.time_db
    endelse
    ;
    ;xb1tot=max(intens.mean.z1.xb1)-min(intens.mean.z1.xb1)
    ;xb2tot=max(intens.mean.z2.xb2)-min(intens.mean.z2.xb2)
    xb1tot =max(intens.mean.rad.xb1)-min(intens.mean.rad.xb1)
    xb2tot =max(intens.mean.rad.xb2)-min(intens.mean.rad.xb2)
    nx1    =n_elements(intens.mean.rad.xc1)
    nx2    =n_elements(intens.mean.rad.xc2)
    ;
    SizePow={ $
    Model:             intens.Model, $
    ;
    ModelIdent:        intens.ModelIdent, $
    itime:             intens.itime, $
    time:              intens.time, $
    time_db:           time_db, $
    File:              intens.File, $
    nx1:               nx1, $
    nx2:               nx2, $
    xb1tot:            xb1tot, $
    xb2tot:            xb2tot, $
    ;
    Power2D1DVersion:  Version, $
    quantity:          'bolometric intensity', $
    ;
    WaveDelta:         WaveDelta, $
    WaveNumberDown:    WaveNumber1DDown, $
    WaveNumberUp:      WaveNumber1DUp, $
    WaveNumberCorr:    qw, $
    WaveNumberDensFac: qn, $
    WaveNumberDensity: qsn, $
    Power:             qsAr*qNormAr, $
    ;
    mean:              qmeanAr, $
    drms:              qdrmsAr}
    ;
    SaveFileName=Model + '_' + OutputID + '.sav'
    print, 'Save data into ', SaveFileName
    save, SizePow, file=SaveFileName
    ;
  endelse ; (dim[0] eq dim[1])
  ;
endfor ; iModel

end ; rhd_power2d1d_start
