; pro rhd_power_extract
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_power_extract ('rhd_power_extract')
;
; PURPOSE:
;   Find the typical granule or structure size from a azimuthally averaged 2D power spectrum
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   .run rhd_power2d1d_start
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;   extremum:      Find "exact" position of zero, minimum, maximum,... of function given by vector
;   smoothweight:  Smooth x- and y-axis values of 1D vector accounting for weights
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   .run rhd_power_extract
;
; MODIFICATION HISTORY:
;   2016-02-03 (B.F. Lyon) First version
;   2016-02-04 (B.F.) Further work
;   2016-02-19 (B.F.) Add parameter PowMode
;-
;---------------------------------------------------------------------------------------------------

;ModelSet='mt15g50mm00n17_intpow.sav'
;ModelSet='mt22g50mm00n14_intpow.sav'
;ModelSet='mt40g50mm00n04_intpow.sav'
;ModelSet='mt33g50mm00n*_intpow.sav'
;ModelSet='mt[2-4]*g50mm00n*_intpow.sav'
;ModelSet='mt[1-4]*g50mm00n*_intpow.sav'
;ModelSet='mt*g*mm00n*_intpow.sav'
ModelSet='*_intpow.sav'

;SaveFile='testipsdb.sav'
SaveFile='allipsdb.sav'

;iRange=0
iRange=2
Degree=3
nRMax=5
PowMode=1

!X.STYLE=0
!Y.STYLE=0

ModelFileList=file_search(ModelSet, count=nModel)
if (nModel eq 0) then message, 'No model found for set ' + ModelSet

; --- Load table with model data ---
;.run rhdmodeldata
rhdmodeldata, modDB
;restore, 'allbasdb.sav'

ips={ $
  Model:            '', $
  mtime:            0L, $
  ntime:            0L, $
  nx1:              0L, $
  nx2:              0L, $
  xb1tot:          0.0, $
  xb2tot:          0.0, $
  Power2D1DVersion: '', $
  Quantity:         '', $
  PowMode:         PowMode, $
  extremumiRange:  iRange, $
  extremumDegree:  Degree, $
  x0vec:           fltarr(nRMax+1), $
  yvec:            fltarr(nRMax+1), $
  x0max:           0.0, $
  xmax:            0.0   }
ipsDB=replicate(ips, nModel)

; --- Loop over all models ---
for iModel=0L,nModel-1L do begin
  ;
  ; --- Get name of next data file ---
  ModelFile=ModelFileList[iModel]
  ;
  ; --- Load next data set ---
  print
  print, 'Restore ', ModelFile
  SizePow=0
  restore, ModelFile
  Model=SizePow.Model
  ;
  ; help, /str, SizePow
  ;
  ; print, 'xb=', SizePow.xb1Tot, SizePow.xb2Tot
  ;
  if ((SizePow.xb1Tot eq SizePow.xb2Tot) and $
      (n_elements(SizePow.time) gt 1   )      ) then begin
    ; --- Extract the averaging interval from the database ---
    mtime0=udb_extract('mmeanavg', [Model], modDB)
    ntime0=udb_extract('nmeanavg', [Model], modDB)
    ntime =0L > ntime0 < (n_elements(SizePow.time)-1L)
    mtime =0L > mtime0 < ntime
    print, Model, mtime0, ntime0, mtime, ntime, format='(A20,1X,2I7,2X,2I7)'
    ;
    if ((mtime gt 0L) or (ntime gt mtime)) then begin
      ; --- Compute the mean distribution over the requested interval ---
      x0=SizePow.WaveNumberCorr
      y0=mean(SizePow.Power[*,mtime:ntime], dim=2)
      w0=SizePow.WaveNumberDensity
      ;
      ; --- Put data into structure ---
      ipsDB[iModel].Model           =Model
      ipsDB[iModel].mtime           =mtime
      ipsDB[iModel].ntime           =ntime
      ipsDB[iModel].nx1             =SizePow.nx1
      ipsDB[iModel].nx2             =SizePow.nx2
      ipsDB[iModel].xb1Tot          =SizePow.xb1Tot
      ipsDB[iModel].xb2Tot          =SizePow.xb2Tot
      ipsDB[iModel].Power2D1DVersion=SizePow.Power2D1DVersion
      ipsDB[iModel].Quantity        =SizePow.Quantity
      ;
      for nR=0L,nRmax do begin
        ; --- Smooth the distribution (x and y axis) ---
        smoothweight, x0, y0, x1, y1, w0=w0, nR=nR
print, nR, minmax(x0), minmax(y0), minmax(x1), minmax(y1), format='(I3,8E14.6)'
        ;
        if (PowMode eq 0) then begin
          y1=y1*(SizePow.xb1Tot/7.5E+07)
        endif else begin
          y1=sqrt(y1*x1)
        endelse
        ;
        ; --- Find the maximum ---
        ymax=max(y1, iymax)
        ;
        if ((iRange eq 0) or (degree eq 0) or (iymax eq 0L)) then begin
          logx2=alog10(x1[iymax])
          logy2=alog10(ymax)
        endif else begin
          i0=iymax
          extremum, x=alog10(x1), alog10(y1), 1, logx2, logy2, $
            i0=i0, irange=iRange, degree=Degree, verbose=0
        endelse
        print, nR, iymax, alog10(x1[iymax]), alog10(ymax), logx2, logy2, format='(I3,2X,I3,4E14.6)'
        ;
        ; --- Plot the distribution and the maximum ---
        if (nR eq 0L) then begin
          plotlines, alog10(x1), alog10(y1), title=Model
        endif else begin
          plotlines, alog10(x1), alog10(y1), /over, linecol=!ctv.red
          plotlines, [alog10(x1[iymax])], [alog10(ymax)], /over, linecol=!ctv.red, psym=1
          plotlines, [logx2], [logy2], /over, linecol=!ctv.blue, psym=5
        endelse
        ;
        ; --- Put data into structure ---
        ipsDB[iModel].x0vec[nR]=10.0^(-logx2)
        ipsDB[iModel].yvec[ nR]=ymax
      endfor ; nR
      ;
      ; --- Put data into structure ---
      if (ipsDB[iModel].x0vec[0] ge 1.0) then begin
        ; --- The first point in the unsmoothed spectrum is the maximum: take it ---
        ipsDB[iModel].x0max=ipsDB[iModel].x0vec[0]
      endif else begin
        ; --- Take the maximum from a smoothed spectrum ---
        ipsDB[iModel].x0max=ipsDB[iModel].x0vec[2]
      endelse
      ipsDB[iModel].xmax =ipsDB[iModel].x0max * SizePow.xb1ToT
    endif ;  ((mtime gt 0L) or (ntime gt mtime))
  endif ; (SizePow.xb1Tot eq SizePow.xb2Tot)
  ;
endfor ; iModel

; --- Reduce the list only to models with sensible size data ---
;ModelList=udb_listall(ipsDB)
;ModDimension=udb_extract('dimension', ModelList, basDB)
;ind=where(ModDimension eq 3)
ind=where(ipsDB[*].xmax gt 0.0)
ipsDB=ipsdB[ind]
ModelList=udb_listall(ipsDB)

; --- Save the result data ---
print, 'Save data (ipsDB) into ' + SaveFile
save, filename=SaveFile, ipsDB

end ; pro rhd_power_extract
