function strCrossCmp, s1, s2, $
                      fold_case=fold_case, n_characters=n_characters, $
                      v2=v2, Default=Default, $
                      v1=v1
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   strCrossCmp ('string_Cross_Compare')
;
; PURPOSE:
;   Cross-compare the elements of two string arrays.
;
; CATEGORY:
;   String handling
;
; CALLING SEQUENCE:
;   Result=strCrossCmp(s1, s2, $
;                      fold_case=fold_case, n_characters=n_characters, $
;                      v2=v2, Default=Default, $
;                      v1=v1)
;
; INPUT:
;   s1:           ('string_1') input string vector
;   s2:           ('string_2') input string vector
;
; OUTPUT:
;   Result:       Integer array of size n_elements(s1) with the indices of the elements of s1 in s2
;
; KEYWORDS:
;   fold_case:    ('fold_case') switch passed to strcmp to enable case folding
;   n_characters: ('number_characters') integer with number of characters to compare, counted from the left
;   v2:           ('vector_2') optional input vector of any kind with the same number of elements as s2
;   Default:      ('Default') input scalar that determines the type of v1. Mandatory, if v2 is supplied.
;   v1:           ('vector_1') optional output vector with the type of the Default and the number of
;                 elements as s1
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   For all elements of s1, find the position of the first occurrence in s2.
;   A position of -1 means the element was not found.
;   Optionally, this position can be used to find the corresponding element in s2, which is then
;   put into s1.
;   Symbollically:
;     strCrossCmp=s2[s1]
;     v1         =v2[s2[s1]]
;
; EXAMPLE:
;   print, strcrosscmp(['a', 'b', 'c', 'dd'], ['A', 'c', 'b', 'cc', 'ddd'], v2=[0.5, 1.0, 1.5, 2.0, 5.0], v1=v1)
;          -1           2           1          -1
;   print, v1
;     0.00000      1.50000      1.00000      0.00000
;
; MODIFICATION HISTORY:
;   2015-08-26 (B.F. Uppsala) Written
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr

n1=n_elements(s1)

if (n1 eq 0) then begin
  message, 'First string array is empty'
  ;
endif else begin ; (n1 eq 0)
  IndCross=replicate(-1, n1)
  ;
  n2=n_elements(s2)
  ;
  if (n2 gt 0) then begin
    for i1=0,n1-1 do begin
      if (keyword_set(n_characters)) then begin
        Ind=where(strcmp(s1[i1], s2, n_characters, fold_case=fold_case) gt 0, n)
      endif else begin
        Ind=where(strcmp(s1[i1], s2,               fold_case=fold_case) gt 0, n)
      endelse
      ;
      if (n gt 0) then begin
        IndCross[i1]=Ind[0]
      endif
    endfor ; i2
  endif ; (n2 gt 0)
endelse ; (n1 eq 0)

if (keyword_set(v2) and (n_elements(v2) eq n2) and (n_elements(Default) ne 0)) then begin
  v1 =replicate(Default, n1)
  ;
  Ind=where(IndCross ge 0, n)
  if (n gt 0) then begin
    ; --- At least some elements where found: copy the correspongind vector values ---
    v1[Ind]=v2[IndCross[Ind]]
  endif
endif

return, IndCross

end ; strCrossCmp
