function strDate, JulianDate, date_only=date_only, time_only=time_only, compressed=compressed
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   strDate ('string_Date')
;
; PURPOSE:
;   Produce sensibly formatted date/time string from julian date.
;
; CATEGORY:
;   String handling
;
; CALLING SEQUENCE:
;   Result=strDate(JulianDate, date_only=date_only, time_only=time_only, compressed=compressed)
;
; INPUT:
;   JulianDate:   ('Julian_Date') julian date in double precision, likely produced with
;                 systime(/JULIAN)
;
; OUTPUT:
;   Result:       String with date/time information
;
; KEYWORDS:
;   date_only:    ('date_only') flag to suppress the time string
;   time_only:    ('time_only') flag to suppress the date string
;   compressed:   ('compressed') flag to suppress separators and spaces (useful to get file names)
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   The input julian date is transformed with caldat into strings for years, months,... seconds.
;   These are then concatenated according to the specified keywords.
;
; EXAMPLE:
;   print, strDate(systime(/JULIAN))
;     2015-08-26 18:39:06
;   print, strDate(systime(/JULIAN), /compressed, /date_only)
;     20150826
;
; MODIFICATION HISTORY:
;   2015-08-26 (B.F. Uppsala) Written
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr

; --- Construct date and time string ---
caldat, JulianDate, mm,dd,yyyy, hh,mmin,ss

ss=round(ss)
yyyy=strmid('000'+strtrim(string(yyyy), 2), 3, 4, /REVERSE)
mm  =strmid('0'  +strtrim(string(mm  ), 2), 1, 2, /REVERSE)
dd  =strmid('0'  +strtrim(string(dd  ), 2), 1, 2, /REVERSE)
hh  =strmid('0'  +strtrim(string(hh  ), 2), 1, 2, /REVERSE)
mmin=strmid('0'  +strtrim(string(mmin), 2), 1, 2, /REVERSE)
ss  =strmid('0'  +strtrim(string(ss  ), 2), 1, 2, /REVERSE)

if (keyword_set(compressed)) then begin
  ; --- Produce strings without any separators ---
  DateString    =yyyy       + mm         + dd
  TimeString    =hh         + mmin       + ss
  DateTimeString=DateString + TimeString
  ;
endif else begin ; (keyword_set(compressed))
  ; --- Produce nice strings with separators ---
  DateString      =yyyy + '-' + mm   + '-' + dd
  TimeString      =hh   + ':' + mmin + ':' + ss
  DateTimeString  =DateString + ' ' + TimeString
  ;
endelse ; (keyword_set(compressed))

if (keyword_set(date_only)) then begin
  Result=DateString
  ;
endif else if (keyword_set(time_only)) then begin
  Result=TimeString
  ;
endif else begin
  Result=DateTimeString
  ;
endelse

Return, Result

end ; strDate
