pro rhd_makeeddy, file=file0, form=form0, conv=conv0, $
  units=units0, $
  m1=m10, n1=n10, m2=m20, n2=n20, m3=m30, n3=n30, dx=dx0, $
  en1=en10, en2=en20, en3=en30, $
  dtime=dtime0, $
  ei_in =ei_in0 , rho_in =rho_in0 , v_in =v_in0 , $
  ei_out=ei_out0, rho_out=rho_out0, v_out=v_out0, $
  Radius=Radius0, Omega=Omega0, OmNorm=OmNorm0, EOS=EOS, $
  r0_grav=r0_grav0, r1_grav=r1_grav0, GM_grav=GM_grav0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makeeddy ('RHD_make_eddy')
;
; PURPOSE:
;   Generate a RHD start model file: 2D eddy
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS (ALL INPUT):
;   file:   ('filename') character, filename
;   form:   ('format') character, 'formatted' or 'unformatted'
;   conv:   ('conversion') character, type of conversion procedure
;   units:  ('units') character: "1" (default) or "cgs"
;   m1:     ('m_number_1') integer, 1. dimension, start index
;   n1:     ('number_1')   integer, 1. dimension, end index
;   m2:     ('m_number_2') integer, 2. dimension, start index
;   n2:     ('number_2')   integer, 2. dimension, end index
;   m3:     ('m_number_3') integer, 3. dimension, start index
;   n3:     ('number_3')   integer, 3. dimension, end index
;   dx:     ('delta_x) real with grid spacing
;   en1:    ('eddy_number_1') position of eddy center (x1 direction), default: center
;   en2:    ('eddy_number_2') position of eddy center (x2 direction), default: center
;   en3:    ('eddy_number_3') position of eddy center (x3 direction), default: center
;   dtime:  ('delta_time') real, time step
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-16 Written by Bernd Freytag, Astrophysik Kiel
;   1999-06-03 (B.F. Copenhagen)
;   2011-07-19 (B.F. Lyon) More comments
;   2011-10-18 (B.F.) Transformed to rhd_makeeddy.pro
;   2012-01-26 (B.F.) Gravity
;   2012-02-06 (B.F.) Bugfix in radius vector definition
;-
;------------------------------------------------------------------------------

; === Default values for keywords ==============================================

; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse

; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_v  ='cm/s'
  unit_e   ='erg/g'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_px  ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_px  =''
  unit_e   =''
endelse

; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=0
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=0
endelse
if (keyword_set(m20)) then begin
  m2=m20
endif else begin
  m2=0
endelse
if (keyword_set(n20)) then begin
  n2=n20
endif else begin
  n2=0
endelse
if (keyword_set(m30)) then begin
  m3=m30
endif else begin
  m3=0
endelse
if (keyword_set(n30)) then begin
  n3=n30
endif else begin
  n3=0
endelse
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(dtime0)) then begin
  dtime=dtime0
endif else begin
  dtime=1.0
endelse

; --- Position of eddy center ---
if (keyword_set(en10)) then begin
  en1=en10
endif else begin
  en1=(n1-m1+1)/2
endelse
if (keyword_set(en20)) then begin
  en2=en20
endif else begin
  en2=(n2-m2+1)/2
endelse
if (keyword_set(en30)) then begin
  en3=en30
endif else begin
  en3=(n3-m3+1)/2
endelse

; --- States: inside ---
if (keyword_set(ei_in0)) then begin
  ei_in=ei_in0
endif else begin
  ei_in=100.0
endelse
if (keyword_set(rho_in0)) then begin
  rho_in=rho_in0
endif else begin
  rho_in=1.0
endelse
if (keyword_set(v_in0)) then begin
  v_in=v_in0
endif else begin
  v_in=0.0
endelse
; --- States: outside ---
if (keyword_set(ei_out0)) then begin
  ei_out=ei_out0
endif else begin
  ei_out=1.0
endelse
if (keyword_set(rho_out0)) then begin
  rho_out=rho_out0
endif else begin
  rho_out=1.0
endelse
if (keyword_set(v_out0)) then begin
  v_out=v_out0
endif else begin
  v_out=0.0
endelse

; --- Eddy parameters
if (keyword_set(Radius0)) then begin
  Radius=Radius0
endif else begin
  Radius=0.45*(n1-m1)*dx
endelse
if (keyword_set(OmNorm0)) then begin
  OmNorm=OmNorm0
endif else begin
  OmNorm=0.5*(1.0-cos(2.0*!PI*findgen(501)/500.0))
endelse
if (keyword_set(Omega0)) then begin
  Omega=Omega0
endif else begin
  Omega=1.0
endelse

; --- Gravity ---
if (keyword_set(GM_grav0)) then begin
  GM_grav=GM_grav0
endif else begin
  GM_grav=0.0
endelse

; --- Channel for comment output ---
nc_p=-1

; --- Description and history ---
description=['Eddy problem ', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makeeddy.pro at ' + strtrim(systime(0),2)]
nhis=1

; --- Initial time ---
itime=0
time =0.0

; --- Grid ---
xc1=dx*(findgen(n1-m1+1)+float(m1))
xc2=dx*(findgen(n2-m2+1)+float(m2))
xc3=dx*(findgen(n3-m3+1)+float(m3))
xb1=dx*(findgen(n1-m1+2)+float(m1)-0.5)
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)

; === Radial profile (1D) ======================================================

; --- Initialize radial 1D arrays ---
nOmega      =n_elements(OmNorm)
OmVec       =Omega*OmNorm
Omvec[-2:-1]=0.0
;nRadius=fix(nOmega*sqrt(2.0)+1.0)
;RaVec =Radius*findgen(nRadius)/float(nRadius-1)
;dR    =Radius/float(nRadius-1)
RadiusMax=sqrt(max(xc1^2) > max(xc3^2))
nRadius  =fix(RadiusMax/Radius*nOmega)+1
dR       =RadiusMax/float(nRadius-1)
RaVec    =dR*findgen(nRadius)
rhoVec   =fltarr(nRadius)
eiVec    =fltarr(nRadius)

; --- Central point ---
rhoVec[0]=rho_in
eiVec[ 0]=ei_in
eosinter, [rhoVec[0]], [eiVec[0]], P0, dPdrho0, dPdei0, T0, dTdei0, s0, EOS=EOS
print, 'Center: rho, ei, P, T= ', [rhoVec[0]], [eiVec[0]], P0, T0, format='(A,4E14.6)'

; --- Loop over all radius points ---
print,  'iRad', 'radius', 'rho', 'drho', 'ei', 'dei', 'dP', 'dP_rot', 'dP_grav', 'g_grav', format='(A4,9A14)'
for iRadius=1,nRadius-1 do begin
  rho0=rhoVec[iRadius-1]
  ei0 =eiVec[ iRadius-1]
  ;
  ; --- EOS ---
  eosinter, [rho0], [ei0], P0, dPdrho0, dPdei0, T0, dTdei0, s0, EOS=EOS
  xcp0=P0/(T0*rho0) + rho0/T0 * dPdrho0/dPdei0
  ;
  ; --- Change in pressure for stationary eddy ---
  if (GM_grav gt 0.0) then begin
    g_grav =(spheregrav(RaVec[iRadius-1], r0=r0_grav0, r1=r1_grav0, C=GM_grav))[0]
    dP_grav=-rho0 * g_grav * dR
  endif else begin
    g_grav =0.0
    dP_grav=0.0
  endelse
  if (iRadius lt nOmega) then begin
    dP_rot=rho0*RaVec[iRadius-1]*OmVec[iRadius-1]^2 * dR
  endif else begin
    dP_rot=0.0
  endelse
  dP1   =dP_rot + dP_grav
  ;
  ; --- Temperature step ---
  ;dTad=1.0/(rho0*xcp0) * g0/kappa0 * dtau
  dT1=0.0
  ;
  ; --- Transform: dP, dT -> drho, dei ---
  dTdrho0=T0/rho0^2 * dPdei0  - P0/rho0^2 * dTdei0
  Deter1 =dTdei0    * dPdrho0 - dPdei0    * dTdrho0
  drho1  =( dTdei0  * dP1 - dPdei0  * dT1) / Deter1
  dei1   =(-dTdrho0 * dP1 + dPdrho0 * dT1) / Deter1
  ;
  ; --- Put scalars into vector ---
  rhoVec[iRadius]=rho0+drho1
  eiVec[ iRadius]=ei0 +dei1
  ;
  if (((iRadius mod 50) eq 0) or (iRadius eq 1) or (iRadius eq (nRadius-1))) then begin
    print, iRadius, RaVec[iRadius-1], rho0, drho1, ei0, dei1, dP1, dP_rot, dP_grav, g_grav, format='(I4,9E14.5)'
  endif
endfor ; iRadius
VeVec=OmVec*RaVec[0:nOmega-1]

print, 'Outside: rho, ei, P, T=', rho0, ei0, P0, T0, format='(A,4E14.6)'

; === Interpolation to 2D/3D ===================================================

; --- 3D version of axes ---
xc1Ar=rebin(reform(xc1, n1-m1+1, 1,       1), n1-m1+1, n2-m2+1, n3-m3+1)
xc3Ar=rebin(reform(xc3,       1, 1, n3-m3+1), n1-m1+1, n2-m2+1, n3-m3+1)
Ra0Ar =(1.0E-5*dx) > sqrt(xc1Ar^2+xc3Ar^2)
Ra1Ar =sqrt(xc1Ar^2+xc3Ar^2) < Radius
Ra2Ar =sqrt(xc1Ar^2+xc3Ar^2)

print,minmax(Ra0Ar), minmax(Ra1Ar)

; --- Interpolate from 1D to 2D/3D case ---
rho=interpol(rhoVec, RaVec, Ra2Ar)
ei =interpol( eiVec, RaVec, Ra2Ar)
v1 =interpol( VeVec, RaVec[0:nOmega-1], Ra1Ar) * (-xc3Ar/Ra0Ar)
v3 =interpol( VeVec, RaVec[0:nOmega-1], Ra1Ar) * ( xc1Ar/Ra0Ar)
;rho=   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)
;v1 =   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)
v2 =   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)
;v3 =   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)
;ei =   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)

; --- Make sure all arrays are 3D ---
rho=rebin(reform(rho,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v1 =rebin(reform(v1 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v2 =rebin(reform(v2 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v3 =rebin(reform(v3 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
ei =rebin(reform(ei ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)

; === Output of results ========================================================

; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
                nc_p, ncout, file=file, form=form, conv=conv, $
                ndes=ndes, description=description, $
                nhis=nhis, history=history, $
                unit_time=unit_time, $
                unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
                unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
                m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
                itime=itime, time=time, $
                dtime=dtime, out_time_full_last=-1.0E08, $
                xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
                rho=rho, ei=ei, v1=v1, v2=v2, v3=v3

; --- Read file just written and look for some quantities ---
ful=uio_dataset_rd(file)
eosbox, ful, eos=EOS
print, 'Max(v):        ', max(combox('v'    ,ful))
print, 'Max(cs):       ', max(combox('cs'   ,ful))
print, 'Max(Ma):       ', max(combox('mach' ,ful))
Pmin=min(combox('P',ful),max=Pmax)
print, 'dP/Max(P):     ', (Pmax-Pmin)/Pmax
print, 'Max(P)/Min(P): ', Pmax/Pmin

end ; rhd_makeeddy
