pro rhd_makesedov, file=file0, form=form0, conv=conv0, $
  units=units0, $
  m1=m10, n1=n10, m2=m20, n2=n20, m3=m30, n3=n30, dx=dx0, $
  en1=en10, en2=en20, en3=en30, ren=ren0, q_smooth=q_smooth, q_drop=q_drop, $
  dtime=dtime0, $
  ei_in =ei_in0 , rho_in =rho_in0 , v_in =v_in0 , $
  ei_out=ei_out0, rho_out=rho_out0, v_out=v_out0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makesedov
;
; PURPOSE:
;   Generate a RHD start model file: 1D/2D/3D Sedov expanding blast wave
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS (ALL INPUT):
;   file:   ('filename') character, filename
;   form:   ('format') character, 'formatted' or 'unformatted'
;   conv:   ('conversion') character, type of conversion procedure
;   units:  ('units') character: "1" (default) or "cgs"
;   m1:     ('m_number_1') integer, 1. dimension, start index
;   n1:     ('number_1')   integer, 1. dimension, end index
;   m2:     ('m_number_2') integer, 2. dimension, start index
;   n2:     ('number_2')   integer, 2. dimension, end index
;   m3:     ('m_number_3') integer, 3. dimension, start index
;   n3:     ('number_3')   integer, 3. dimension, end index
;   dx:     ('delta_x) real with grid spacing
;   en1:    ('explosion_number_1') position of explosion center (x1 direction), default: center
;   en2:    ('explosion_number_2') position of explosion center (x2 direction), default: center
;   en3:    ('explosion_number_3') position of explosion center (x3 direction), default: center
;   ren:    ('radius_explosion_number') radius of explosion center, default: 0.5
;   dtime:  ('delta_time') real, time step
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-16 Written by Bernd Freytag, Astrophysik Kiel
;   1999-06-03 (B.F. Copenhagen)
;   2011-07-19 (B.F. Lyon) More comments
;   2012-04-04 (B.F.) Parameters q_smooth and q_drop added
;-
;------------------------------------------------------------------------------
;
; --- Default values for keywords ---
; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse
;
; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_v  ='cm/s'
  unit_e   ='erg/g'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_px  ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_px  =''
  unit_e   =''
endelse
;
; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=0
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=0
endelse
if (keyword_set(m20)) then begin
  m2=m20
endif else begin
  m2=0
endelse
if (keyword_set(n20)) then begin
  n2=n20
endif else begin
  n2=0
endelse
if (keyword_set(m30)) then begin
  m3=m30
endif else begin
  m3=0
endelse
if (keyword_set(n30)) then begin
  n3=n30
endif else begin
  n3=0
endelse
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(dtime0)) then begin
  dtime=dtime0
endif else begin
  dtime=1.0
endelse
;
; --- Position of explosion center ---
if (keyword_set(en10)) then begin
  en1=en10
endif else begin
  en1=(n1-m1+1)/2
endelse
if (keyword_set(en20)) then begin
  en2=en20
endif else begin
  en2=(n2-m2+1)/2
endelse
if (keyword_set(en30)) then begin
  en3=en30
endif else begin
  en3=(n3-m3+1)/2
endelse
; --- Radius of explosion center ---
if (keyword_set(ren0)) then begin
  ren=abs(ren0)
endif else begin
  ren=0.5
endelse
;
; --- States: inside ---
if (keyword_set(ei_in0)) then begin
  ei_in=ei_in0
endif else begin
  ei_in=100.0
endelse
if (keyword_set(rho_in0)) then begin
  rho_in=rho_in0
endif else begin
  rho_in=1.0
endelse
if (keyword_set(v_in0)) then begin
  v_in=v_in0
endif else begin
  v_in=0.0
endelse
; --- States: outside ---
if (keyword_set(ei_out0)) then begin
  ei_out=ei_out0
endif else begin
  ei_out=1.0
endelse
if (keyword_set(rho_out0)) then begin
  rho_out=rho_out0
endif else begin
  rho_out=1.0
endelse
if (keyword_set(v_out0)) then begin
  v_out=v_out0
endif else begin
  v_out=0.0
endelse
;
; --- Channel for comment output ---
nc_p=-1
;
; --- Description and history ---
description=['Sedov problem ', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makesedov.pro at ' + strtrim(systime(0),2)]
nhis=1
;
; --- Initial time ---
itime=0
time =0.0
;
; --- Grid ---
xc1=dx*(findgen(n1-m1+1)+float(m1))
xc2=dx*(findgen(n2-m2+1)+float(m2))
xc3=dx*(findgen(n3-m3+1)+float(m3))
xb1=dx*(findgen(n1-m1+2)+float(m1)-0.5)
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)
;
; --- Outside states on all grid points ---
rho=replicate(rho_out,n1-m1+1,n2-m2+1,n3-m3+1)
v1 =replicate(v_out,  n1-m1+1,n2-m2+1,n3-m3+1)
v2 =   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)
v3 =   fltarr(        n1-m1+1,n2-m2+1,n3-m3+1)
ei =replicate(ei_out, n1-m1+1,n2-m2+1,n3-m3+1)
rho=rebin(reform(rho,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v1 =rebin(reform(v1 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v2 =rebin(reform(v2 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v3 =rebin(reform(v3 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
ei =rebin(reform(ei ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
;
; --- State in explosion center ---
help,en1,ren,m1,n1
for i3=round(en3-ren)>0,round(en3+ren)<(n3-m3) do begin
  for i2=round(en2-ren)>0,round(en2+ren)<(n2-m2) do begin
    for i1=round(en1-ren)>0,round(en1+ren)<(n1-m1) do begin
;      if (((i1-m1+1-en1)^2+(i2-m2+1-en2)^2+(i3-m3+1-en3)^2) lt ren^2) then begin
      if (((i1-en1)^2+(i2-en2)^2+(i3-en3)^2) lt ren^2) then begin
        print, format='(3I4)', i1+m1,i2+m2,i3+m3
        rho(i1,i2,i3)=rho_in
        v1( i1,i2,i3)=v_in
        ei( i1,i2,i3)=ei_in
      endif
    endfor
  endfor
endfor
;
if (keyword_set(q_smooth)) then begin
  if (n2 gt n2) then message, 'Smoothing not implemented for this dimension.'
  ;
  ei=reform(ei[*,m2,*])
  ; --- Smooth ---
  ei=smooth2d(ei, q_smooth=q_smooth, q_drop=q_drop)
  ;
  ei =rebin(reform(ei ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
endif
;
; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
                nc_p, ncout, file=file, form=form, conv=conv, $
                ndes=ndes, description=description, $
                nhis=nhis, history=history, $
                unit_time=unit_time, $
                unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
                unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
                m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
                itime=itime, time=time, $
                dtime=dtime, out_time_full_last=-1.0E08, $
                xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
                rho=rho, ei=ei, v1=v1, v2=v2, v3=v3
;
end ; rhd_makesedov
