function rhd_makespike_gauss, x, beta, z
return, exp(-beta*(x-z)^2)
end ; rhd_makespike_gauss

function rhd_makespike_triangle, x, gamma, g
return, 1.0-abs(gamma*(x-g))
end ; rhd_makespike_triangle

function rhd_makespike_ellipse, x, alpha, a
return, sqrt( (1.0-alpha^2*(x-a)^2)>0.0)
end ; rhd_makespike_ellipse


pro rhd_makespike, file=file0, form=form0, conv=conv0, $
  funct=funct0, modulation=modulation0, units=units0, $
  RepliNumber=RepliNumber0, RepliFactor=RepliFactor0, $
  m1=m10, n1=n10, m2=m20, n2=n20, m3=m30, n3=n30, dx=dx0, $
  dtime=dtime0, $
  rhoei_in=rhoei_in0, rho_in=rho_in0, v_in=v_in0, $
  ampl=ampl0
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makespike
;
; PURPOSE:
;   Generate a RHD start model file: 1D/2D/3D spike wave
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;   rhd_makespike, file=file, form=form, conv=conv, $
;                  funct=funct, modulation=modulation, units=units, $
;                  RepliNumber=RepliNumber, RepliFactor=RepliFactor, $
;                  m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, dx=dx, $
;                  dtime=dtime, $
;                  rhoei_in=rhoei_in, rho_in=rho_in, v_in=v_in, $
;                  ampl=ampl
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS (ALL INPUT):
;   file:         ('filename') character, filename, default: 'rhd.sta'
;   form:         ('format') character, 'formatted' or 'unformatted', default: 'formatted'
;   conv:         ('conversion') character, type of conversion procedure
;   funct:        ('function') character
;                   'spikes'  : gaussian, rectangle, triangle, ellipse (default)
;                   'needles' : small rectangles with increasing width
;                   'needles2': small rectangles with increasing width
;                   'sine'    : sine wave
;   modulation:   ('modulation') character
;                    'rho,1/ei'
;                     'v1'
;   RepliNumber:  ('Replication_Number') integer with number of replications of basis vector
;   RepliFactor:  ('Replication_Factor') factor for each replication
;   units:        ('units') character: "1" (default) or "cgs"
;   m1:           ('m_number_1') integer, 1. dimension, start index
;   n1:           ('number_1')   integer, 1. dimension, end index 
;   m2:           ('m_number_2') integer, 2. dimension, start index
;   n2:           ('number_2')   integer, 2. dimension, end index
;   m3:           ('m_number_3') integer, 3. dimension, start index
;   n3:           ('number_3')   integer, 3. dimension, end index
;   dx:           ('delta_x) real with grid spacing
;   dtime:        ('delta_time') real, time step
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;   Make a plot.
;   Create a file.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-17 (B.F. Astrophysik Kiel)
;   1997-10-19 (B.F.) Keyword funct added, funct='spikes', 'sine'
;   1997-10-23 (B.F.) funct='needles'
;   1997-10-27 (B.F.) funct='needles2'
;   2012-04-19 (B.F. Lyon) New parameters modulation, RepliNumber, RepliFactor
;              compute xnorm differently
;   2018-05-15 (B.F.) Add comments, change brackets () -> []
;-
;---------------------------------------------------------------------------------------------------

; --- Default values for keywords ---
; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse

; --- Type of function ---
if (keyword_set(funct0)) then begin
  funct=strlowcase(funct0)
endif else begin
  funct='spikes'
endelse
if (keyword_set(modulation0)) then begin
  modulation=strlowcase(modulation0)
endif else begin
  modulation=''
endelse

; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_v   ='cm/s'
  unit_e   ='erg/g'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_v   ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_v   =''
  unit_e   =''
endelse

; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=-100
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=99
endelse
if (keyword_set(m20)) then begin
  m2=m20
endif else begin
  m2=0
endelse
if (keyword_set(n20)) then begin
  n2=n20
endif else begin
  n2=0
endelse
if (keyword_set(m30)) then begin
  m3=m30
endif else begin
  m3=0
endelse
if (keyword_set(n30)) then begin
  n3=n30
endif else begin
  n3=0
endelse
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(dtime0)) then begin
  dtime=dtime0
endif else begin
  dtime=1.0
endelse
if (keyword_set(RepliNumber0)) then begin
  RepliNumber=fix(RepliNumber0) > 1
endif else begin
  RepliNumber=1
endelse
if (keyword_set(RepliFactor0)) then begin
  RepliFactor=RepliFactor0
endif else begin
  RepliFactor=1.0
endelse

; --- States: inside ---
if (keyword_set(rhoei_in0)) then begin
  rhoei_in=rhoei_in0
endif else begin
  rhoei_in=100.0
endelse
if (keyword_set(rho_in0)) then begin
  rho_in=rho_in0
endif else begin
  rho_in=1.0
endelse
if (keyword_set(v_in0)) then begin
  v_in=v_in0
endif else begin
  v_in=0.0
endelse

; --- Amplitude ---
if (keyword_set(ampl0)) then begin
  ampl=ampl0
endif else begin
  ampl=0.0
endelse

; --- Channel for comment output ---
nc_p=-1

; --- Description and history ---
;version='MAKESPIKE 000.02.1999.03.25'
version='MAKESPIKE 000.03.2018.05.15'
description=['RHD test data ', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makespike.pro: '+strtrim(systime(0),2)]
nhis=1

; --- Initial time ---
ntime=0
time =0.0

; --- Grid ---
xc1=dx*(findgen(n1-m1+1)+float(m1))
;xnorm=xc1/dx*0.01
xnorm=2.0*findgen(n1-m1+1)/float(n1-m1+1)-1

u=fltarr(n1-m1+1)

if (funct eq 'spikes') then begin
  ; --- Function with spikes ---
  a=0.5
  z=-0.7
  delta=0.005
  alpha=10.0
  beta=alog(2.0)/(36.0*delta^2)
  for i=0,n1-m1 do begin
    x=xnorm[i]
    if ((x ge -0.8) and (x le -0.6)) then begin
      u[i]=1.0/6.0*(    rhd_makespike_gauss(x, beta, z-delta) + $
                        rhd_makespike_gauss(x, beta, z+delta) + $
                    4.0*rhd_makespike_gauss(x, beta, z      )    )
    endif else if ((x ge -0.4) and (x le -0.2)) then begin
      u[i]=1.0
    endif else if ((x ge  0.0) and (x le  0.2)) then begin
      u[i]=rhd_makespike_triangle(x,10.0,0.1)
    endif else if ((x ge  0.4) and (x le  0.6)) then begin
      u[i]=1.0/6.0*(    rhd_makespike_ellipse(x, alpha, a-delta) + $
                        rhd_makespike_ellipse(x, alpha, a+delta) + $
                    4.0*rhd_makespike_ellipse(x, alpha, a      )    )
    endif
  endfor ; i
  u=u*ampl
endif else if (funct eq 'needles') then begin
  period=20
  for i=0,n1-m1-period do begin
    idiv=i/period
    imod = i mod period
    if (imod lt idiv) then u[i]=ampl
  endfor ; i
endif else if (funct eq 'needles2') then begin
  period=26
  for i=0,n1-m1-period do begin
    idiv=2*(i/period)
    imod = i mod period
    if (imod lt idiv) then u[i]=ampl
   endfor ;
endif else if (funct eq 'sine') then begin
  u=ampl*sin(2.0*!PI*xnorm)
endif else begin
  message, 'Function ' + funct + ' not implemented, yet.'
endelse

if (RepliNumber gt 1) then begin
  ; --- Replicate input array applying each time a factor ---
  uOut=u
  for i=1,RepliNumber-1 do begin
    u=u*RepliFactor
    uOut=[uOut, u]
  endfor ; i
  u=uOut
  ;
  n1=n_elements(u)+m1-1
endif

; --- Grid ---
xc1=dx*(findgen(n1-m1+1)+float(m1))
xc2=dx*(findgen(n2-m2+1)+float(m2))
xc3=dx*(findgen(n3-m3+1)+float(m3))
xb1=dx*(findgen(n1-m1+2)+float(m1)-0.5)
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)

;plot,xnorm,u, psym=-1, ystyle=3
plot,reform(xc1),u, psym=-1, ystyle=3

u=rebin(reform(u,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)

; --- Modulate states ---
if (modulation eq 'v1') then begin
  rho=replicate(rho_in  , n1-m1+1,n2-m2+1,n3-m3+1)
  ei =replicate(rhoei_in, n1-m1+1,n2-m2+1,n3-m3+1)/rho
  v1 =replicate(0.0     , n1-m1+1,n2-m2+1,n3-m3+1)
  v1 =u*v_in
endif else begin ; --- default case ---
  rho=replicate(rho_in  , n1-m1+1,n2-m2+1,n3-m3+1)
  rho=rho*(1.0+u)
  ei =replicate(rhoei_in, n1-m1+1,n2-m2+1,n3-m3+1)/rho
  v1 =replicate(v_in    , n1-m1+1,n2-m2+1,n3-m3+1)
endelse

; --- Set the transversal velocities to zero ---
v2 =   fltarr(       n1-m1+1,n2-m2+1,n3-m3+1)
v3 =   fltarr(       n1-m1+1,n2-m2+1,n3-m3+1)

; --- Make sure all bulk arrays are 3D ---
rho=rebin(reform(rho,n1-m1+1,n2-m2+1,n3-m3+1), n1-m1+1,n2-m2+1,n3-m3+1)
v1 =rebin(reform(v1 ,n1-m1+1,n2-m2+1,n3-m3+1), n1-m1+1,n2-m2+1,n3-m3+1)
v2 =rebin(reform(v2 ,n1-m1+1,n2-m2+1,n3-m3+1), n1-m1+1,n2-m2+1,n3-m3+1)
v3 =rebin(reform(v3 ,n1-m1+1,n2-m2+1,n3-m3+1), n1-m1+1,n2-m2+1,n3-m3+1)
ei =rebin(reform(ei ,n1-m1+1,n2-m2+1,n3-m3+1), n1-m1+1,n2-m2+1,n3-m3+1)

; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
                nc_p, ncout, file=file, form=form, conv=conv, $
                ndes=ndes, description=description, $
                nhis=nhis, history=history, $
                unit_time=unit_time, $
                unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
                unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
                m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
                itime=itime, time=time, $
                dtime=dtime, out_time_full_last=0.0, $
                xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
                rho=rho, ei=ei, v1=v1, v2=v2, v3=v3

end ; rhd_makespike
