function udb_Extract, SearchKey, IdList0, uDB0, uDB1, uDB2, Default=Default, FoundVector=FoundVector
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   udb_Extract ('universal_data_base_Extract')
;
; PURPOSE:
;   Extract information from data base.
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_Extract(SearchKey, IdList0, uDB0, uDB1, uDB2, $
;                       Default=Default, FoundVector=FoundVector)
;
; INPUT:
;   SearchKey:    ('Search_Key') string with key to search for
;   IdList0:      ('Identifier_List_0') string array with identifier (model) list
;   uDB0:         ('universal_Data_Base_0') data base 0
;   uDB1:         ('universal_Data_Base_1') data base 1
;   uDB2:         ('universal_Data_Base_2') data base 2
;
; OUTPUT:
;   result:       Vector with n_elements(IdList0) values corresponding to SearchKey
;
; KEYWORDS:
;   Default:      ('Default') optional input scalar with default value if the search key wasn't found
;   FoundVector:  ('Found_Vector') integer output vector indicating in which database
;                                  the key was found for each IdList0 item
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   uDB=[{model: 'm1', data1: 1, data2: 1}, {model: 'm2', data1: 2, data2: 4}]
;   print, udb_extract('data1', ['m1'], uDB)
;       1
;   print, udb_extract('data2', ['m1', 'm2'], uDB)
;       1       4
;   print, udb_extract('data2', ['m1', 'm2', 'm3'], uDB)
;       1       4       0
;   print, udb_extract('data2', ['m1', 'm2', 'm3'], uDB, Default=99)
;       1       4      99
;   print, udb_extract('data3', ['m1', 'm2'], uDB)
;           0           0
;   print, udb_extract('data3', ['m1', 'm2'], uDB, Default='None')
;       None None
;
; TODO:
;   seqrebin: strip mean structure, strip par structure
;
;   Additional routines:
;     udb_IsConsistent(uDB1, uDB2)
;     udb_Convert, optional: permutate/rename entries
;     udb_Merge: Append,Cat,Merge,Replace,Update
;       Cases: append new data sets (same shape, new models)
;                Append anything to pointer list
;                Append to structure vector if consistent
;              append_new/overwrite_old data sets (same shape, old/new models)
;              add_new/overwrite_old tags
;     udb_Test,ShowStats
;     udb_PrintTable: ASCII, LaTeX, HTML, XML, CSV, UIO; only from StrutureOfVectors
;     udb_WritePro
;     uio_udb_rd,wr (via convert to UIO table?)
;
;  Additional databases:
;     averaging interval (in sec), selected snapshots
;     par data (reading routine, update routine)
;     basic: Table with:
;       Radius, luminosity
;       nquc
;       v3rms(tau=2/3), v3rms_max, s_min, s_max(z<z(s_min)), s_top,
;       T_min/max, P_min/max, rho_min/max, s_min/max, Hp
;       Fe_rad_min/max, Fe_conv_min/max
;       DateStart, DateEnd, FileFirst, FileLast (determined with separate skript, from seq file)
;     fine data
;     sizes (FFT: different averages/weights, max; granule counting) 
;     tau-averages (computed by CO5BOLD?)
;     ModelListId, ListOfModels, Description, Date/History, Comment, color, linestyle, psym
;     DirPrefix, Mean/Full/Out/Fine/ParSubDir (determined with separate skript)
;
;  Additional features:
;    filenames instead of databases?
;    Entry__date,description?
;    Allow * in IdList, uDB_entry list, in SearchName
;    Allow 'if not found then look for' in SearchKey (e.g. 'Teffout,Teff')
;
;  Sequence:
;    Update modeltable
;    run rhd_seqpp_collect
;    extract par, seq from seq_0
;    analyse intens.sav: contrast (spatial, temporal),
;                        size (FFT: different averages; counting)
;    derive basic data
;
; MODIFICATION HISTORY:
;   2009-10-06 (B.F. Lyon)
;   2009-10-25 (B.F.) Handle databases with new types 'StructureOfVectors', 'UIOTable'
;   2015-08-13 (B.F. Uppsala) Add keyword Default
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_MasterKey='MODEL'

if (n_params() le 1) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Normalize ID list ---
if (n_elements(IdList0) eq 1) then begin
  ; --- It might be a blank separated list: parse it into an 1D array ---
  IdList=strsplit(IdList0, /extract)
endif else begin
  ; --- It's an array: just remove leading and trailing blanks ---
  IdList=strtrim(IdList0, 2)
endelse

if (n_params() le 2) then begin
  ; --- No database given, just give ID list ---
  return, IdList
endif

; --- Number of databases to inquire ---
nDB=n_params()-2
; --- Number of entry identifiers ---
nId=n_elements(IdList)

; --- Define output arrays (and flag indicating if at least one entry is found) ---
ResultDefined_flag=0
if (n_elements(Default) eq 1) then begin
  ResultVector    =replicate(Default, nId)
endif else begin
  ResultVector    =lonarr(nId)
endelse
FoundVector       =lonarr(nId)

; --- Loop over all list entries ---
for iId=0,nId-1 do begin
  KeyFound_flag=0
  ; --- Loop over all data bases (until end of list or key is found) ---
  for iDB=0,nDB-1 do begin
    ; --- Select current data base (out of 3 so far) ---
    if (iDB eq 0) then begin
      uDB=uDB0
    endif else if (iDB eq 1) then begin
      uDB=uDB1
    endif else if (iDB eq 2) then begin
      uDB=uDB2
    endif
    ;
    ; --- Determine type of data base and number of elements: ---
    ; --- (1D array of structures, structure of 1D arrays, 1D array of pointers to structures, uio table) ---
    uDB_Type=udb_GetType(uDB, nElDB=nElDB)
    ;print, 'Type of data base: ' + uDB_Type + ' with ', nELDB, ' elements'
    ;
    if ((uDB_Type eq 'VectorOfStructures') or $
        (uDB_Type eq 'StructureOfVectors') or $
        (uDB_Type eq 'UIOTable'          )   ) then begin
      ; === uDB is 1D array of (identical) structures or structure of vectors or UIO table =========
      ;
      uDB_Tags=udb_GetTags(uDB)
      uDB_MaskMasterKey=strcmp(uDB_Tags, uDB_MasterKey, /fold_case)
      uDB_MaskSearchKey=strcmp(uDB_Tags,     SearchKey, /fold_case)
      if ((total(uDB_MaskMasterKey) ge 1.0) and (total(uDB_MaskSearchKey) ge 1.0)) then begin
        ; --- Tags exist in structure: search match ---
        ;
        ; --- Number of entry of master key (there can only be one) ---
        iMasterKey=(where(uDB_MaskMasterKey))[0]
        ; --- Array of values corresponding to master key (e.g. the list of the model names) ---
        if (uDB_Type eq 'UIOTable') then begin
          uio_tabr, AllMasterValue, ''+uDB_MasterKey, uDB
        endif else begin
          AllMasterValue=uDB.(iMasterKey)
        endelse
        ; --- The mask indicating where the searched for ID is found ---
        MaskMasterFound=strmatch(AllMasterValue, IdList[iId], /fold_case)
        if (total(MaskMasterFound) ge 1.0) then begin
          ; --- At least one entry matches ---
          ; --- The index of the first found entry ---
          iMasterFound=(where(MaskMasterFound))[0]
          iSearchKey  =(where(uDB_MaskSearchKey))[0]
          if            (uDB_Type eq 'VectorOfStructures') then begin
            SearchValue =uDB[iMasterFound].(iSearchKey)
          endif else if (uDB_Type eq 'StructureOfVectors') then begin
            SearchValue =(uDB.(iSearchKey))[iMasterFound]
          endif else begin
            uio_tabr, SearchValueList, ''+SearchKey, uDB
            SearchValue=SearchValueList[iMasterFound]
          endelse
          ;help, iMasterKey, iSearchKey, SearchKey, SearchValue
          if (ResultDefined_flag eq 0) then begin
            ResultVector=replicate(SearchValue, nId)
            if (nId ge 2) then begin
              if (n_elements(Default) eq 1) then begin
                ResultVector[*]=Default
              endif else begin
                ResultVector[*]=0
              endelse
            endif
            ResultDefined_flag=1
          endif
          ResultVector[iId]=SearchValue
          FoundVector[iId]=iDB+1
        endif
      endif
      ;
    endif else if (uDB_Type eq 'ListOfPointers') then begin
      ; === uDB is 1D array of pointers to structures ========================================
      ;
;help, uDB_Type
      ; --- Loop over all elements of list ---
      for iElDB=0,nElDB-1 do begin
        ;
;help,iELDB,nELDB
        if (ptr_valid(uDB[iElDB])) then begin
          ; --- Pointer is valid: go on ---
          uDB_Tags=tag_names((*uDB[iElDB]))
          uDB_MaskMasterKey=strcmp(uDB_Tags, uDB_MasterKey, /fold_case)
          uDB_MaskSearchKey=strcmp(uDB_Tags,     SearchKey, /fold_case)
          if ((total(uDB_MaskMasterKey) ge 1.0) and (total(uDB_MaskSearchKey) ge 1.0)) then begin
            ; --- Tags exist in structure: search match ---
            ;
            ; --- Number of entry of master key (there can only be one) ---
            iMasterKey=(where(uDB_MaskMasterKey))[0]
            ; --- Value corresponding to master key (e.g. the model name) ---
            MasterValue=(*uDB[iElDB]).(iMasterKey)
            ;help, MasterValue
            ; --- The mask indicating where the searched for ID is found (a 0 or a 1) ---
            MaskMasterFound=strmatch(MasterValue, IdList[iId], /fold_case)
            ; --- The index of the first found entry ---
            if (MaskMasterFound eq 1) then begin
              ; --- Entry matches ---
              ; --- Get the value for the search key ---
              iSearchKey =(where(uDB_MaskSearchKey))[0]
              SearchValue=(*uDB[iELDB]).(iSearchKey)
              ;help, iMasterKey, iSearchKey, SearchKey, SearchValue
              ; --- Check if the result vector is already defined ---
              if (ResultDefined_flag eq 0) then begin
                ; --- The result vector is not defined: create it with proper type and dimension ---
                ResultVector=replicate(SearchValue, nId)
                if (n_elements(Default) eq 1) then begin
                  ResultVector[*]=Default
                endif else begin
                  ResultVector[*]=''
                endelse
                ResultDefined_flag=1
              endif
              ResultVector[iId]=SearchValue
              FoundVector[iId]=iDB+1
            endif
          endif
        endif
      endfor; iElDB
    endif
    ;
    ; --- Exit if all IDs found ---
    if (FoundVector[iId] gt 0) then begin
      ;print, 'Exit search for ' + IdList[iId] + ' after database ', trim(string(iDB),2)
      break
    endif
    ;
  endfor ; iDB
  ;
endfor ; iId

return, ResultVector

end ; udb_Extract
