function udb_Merge, MergeMode, uDB0, uDB1, mask1=mask1
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   udb_Merge ('universal_data_base_Merge')
;
; PURPOSE:
;   Merge two data bases
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_Merge(MergeMode, uDB0, uDB1, mask1=mask1)
;
; INPUT:
;   MergeMode: ('Merge_Mode') String: 'Append', 'Replace', 'Merge'
;   uDB0:   ('universal_Data_Base_0')
;   uDB1:   ('universal_Data_Base_1')
;
; OUTPUT:
;   result:   Merged universal data base
;
; KEYWORDS:
;   mask1:  ('mask_1') Output long-integer vector with 0/1 values indicating which elements
;                      from uDB1 have been put into the output uDB
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   udb_GetTags:
;   udb_GetType:
;   udb_ListAll:
;   udb_MergeStringVectors:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Not all combinations implemented, yet.
;   Append:  from LoP VoS SoV UIO
;       to LoP     *   *   *   *
;          VoS         *   *   *
;          SoV         *   *   *
;          UIO         *   *   *
;   Replace: from LoP VoS SoV UIO
;       to LoP     *   *   *   *
;          VoS         *   *   *
;          SoV         *   *   *
;          UIO         *   *   *
;   -----------------------------
;   Convert: from LoP VoS SoV UIO
;       to LoP     *   *   *   *
;          VoS         *   *   *
;          SoV         *   *   *
;          UIO         *   *   *
;
; PROCEDURE:
;     Append, Replace, Merge, Update
;       Cases: append new data sets (same shape, new models)
;                Append anything to pointer list
;                Append to structure vector if consistent
;              append_new/overwrite_old data sets (same shape, old/new models)
;              add_new/overwrite_old tags
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-10-08 (B.F. Lyon)
;   2009-11-30 (B.F.) More cases
;   2009-12-08 (B.F.) Implement "Replace" for VectorOfStructures
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_MasterKey='MODEL'

if (n_params() lt 3) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif
; --- Determine types of databases ---
uDB_Type0=udb_GetType(uDB0, nElDB=nElDB0)
uDB_Type1=udb_GetType(uDB1, nElDB=nElDB1)

if (strmatch(MergeMode, 'Append', /fold_case)) then begin ; ========================================
  ; === Just append the second database to the first, irrespective of entry Ids ===
  ;
  if (uDB_Type0 eq 'ListOfPointers') then begin ;---------------------------------------------------
    ; --- Everything can be appended to a pointer list ---
    ;
    ; --- Create empty combined pointer array ---
    nElDB2=nElDB0+nElDB1
    uDB2=ptrarr(nElDB2>1, /allocate_heap)
    ;
    ; --- Copy first database ---
    iElDB2=0
    for iElDB0=0,nElDB0-1 do begin
      if (ptr_valid(uDB0[iElDB0])) then begin
        *uDB2[iElDB2]=*uDB0[iElDB0]
        iElDB2=iElDB2+1
      endif
    endfor ; iElDB0
    ;
    ; --- Copy second database ---
    if (uDB_Type1 eq 'ListOfPointers') then begin ; ................................................
      for iElDB1=0,nELDB1-1 do begin
        if (ptr_valid(uDB1[iElDB1])) then begin
          *uDB2[iElDB2]=*uDB1[iElDB1]
          iElDB2=iElDB2+1
        endif
      endfor ; iElDB1
      ;
    endif else if (uDB_Type1 eq 'VectorOfStructures') then begin ; .................................
      for iElDB1=0,nElDB1-1 do begin
        *uDB2[iElDB2]=uDB1[iElDB1]
        iElDB2=iElDB2+1
      endfor ; iElDB1
      ;
    endif else if (uDB_Type1 eq 'StructureOfVectors') then begin ; .................................
      ; --- Get tag names and number of second data base ---
      TagsDB1=udb_GetTags(uDB1, nTags=nTagsDB1)
      ;help, nTagsDB1, nELDB1
      ; --- Create template structure with only one element (but with all tags) ---
      if (nTagsDB1 ge 1) then uDBtemplate=create_struct(TagsDB1[0], (uDB1.(0))[0])
      for iTagsDB1=1,nTagsDB1-1 do begin
        uDBtemplate=create_struct(uDBtemplate, TagsDB1[iTagsDB1], (uDB1.(iTagsDB1))[0])
      endfor ; iTagsDB1
      ;help, /str, uDBtemplate
      ;
      ; --- Copy entries individually ---
      for iElDB1=0,nELDB1-1 do begin
        uDBsingle=uDBtemplate
        for iTagsDB1=0,nTagsDB1-1 do begin
          uDBsingle.(iTagsDB1)=(uDB1.(iTagsDB1))[iElDB1<(n_elements(uDB1.(iTagsDB1))-1)]
        endfor ; iTagsDB1
        ;help, iELDB1
        ;help, /str, uDBsingle
        ;
        *uDB2[iElDB2]=uDBsingle
        iElDB2=iElDB2+1
      endfor ; iElDB1
      ;
    endif else if (uDB_Type1 eq 'UIOTable') then begin ; ...........................................
      ; --- Get tag names and number of second data base ---
      TagsDB1=udb_GetTags(uDB1, nTags=nTagsDB1)
      ;help, nTagsDB1, nELDB1
      nstr='__NAME'
      ustr='__UNIT'
      ; --- Create template structure with only one element (but with all tags) ---
      if (nTagsDB1 ge 1) then begin
        uio_tabr, val, ident, uDB1, incolumn=1, name=valName, unit=valUnit
        uDBtemplate=create_struct(ident, val[0], ident+nstr, valName, ident+ustr, valUnit)
      endif
      for iTagsDB1=1,nTagsDB1-1 do begin
        uio_tabr, val, ident, uDB1, incolumn=iTagsDB1+1, name=valName, unit=valUnit
        uDBtemplate=create_struct(uDBtemplate, ident, val[0], ident+nstr, valName, ident+ustr, valUnit)
      endfor ; iTagsDB1
      ;help, /str, uDBtemplate
      ;
      ; --- Copy entries individually ---
      for iElDB1=0,nELDB1-1 do begin
        uDBsingle=uDBtemplate
        for iTagsDB1=0,nTagsDB1-1 do begin
          uio_tabr, val, ident, uDB1, incolumn=iTagsDB1+1
          uDBsingle.(3*iTagsDB1)=val[iELDB1]
        endfor ; iTagsDB1
        ;help, iELDB1
        ;help, /str, uDBsingle
        ;
        *uDB2[iElDB2]=uDBsingle
        iElDB2=iElDB2+1
      endfor ; iElDB1
      ;
    endif else begin ; .............................................................................
      message, 'Unknown uDB Type.'
      ;
    endelse ; ......................................................................................
    ;
  endif else if (uDB_Type0 eq 'VectorOfStructures') then begin ; -----------------------------------
    if ( (uDB_Type1 eq 'VectorOfStructures') or $
        ((uDB_Type1 eq 'StructureOfVectors') and (nElDB1 eq 1))) then begin ; ......................
      ; --- Create tag lists ---
      TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
      TagsDB1=udb_GetTags(uDB1, nTags=nTagsDB1)
      TagsDB2=udb_MergeStringVectors(TagsDB0, TagsDB1)
      nTagsDB2=n_elements(TagsDB2)
      ;
      ; --- Create template structure with tags from both input structures ---
      for iTagsDB2=0,nTagsDB2-1 do begin
        ; --- Find matching tag in one of the input structures to get a value of proper type ---
        iMatch=(where(strcmp(TagsDB2[iTagsDB2], TagsDB0) eq 1))[0]
        if (iMatch ne -1) then begin
          TagValue=(uDB0.(iMatch))[0]
          ;if (size(uDB0.(iMatch), /tname) eq 'STRING') then begin
          ;  TagValue=''
          ;endif else begin
          ;  TagValue=(uDB0.(iMatch))[0]*0b
          ;endelse
        endif else begin
          iMatch=(where(strcmp(TagsDB2[iTagsDB2], TagsDB1) eq 1))[0]
          if (iMatch ne -1) then begin
            TagValue=(uDB1.(iMatch))[0]
            ;if (size(uDB1.(iMatch), /tname) eq 'STRING') then begin
            ;  TagValue=''
            ;endif else begin
            ;  TagValue=(uDB1.(iMatch))[0]*0b
            ;endelse
          endif else begin
            message, 'Merged tag list contains unknown tag'
          endelse
        endelse
        ; --- Append new tag with value ---
        if (iTagsDB2 eq 0) then begin
          uDB2Template=create_struct(              TagsDB2[iTagsDB2], TagValue)
        endif else begin
          uDB2Template=create_struct(uDB2Template, TagsDB2[iTagsDB2], TagValue)
        endelse
      endfor ; iTagsDB2
      ;
      ; --- Create vector of structures ---
      nElDB2=nElDB0+nElDB1
      uDB2=replicate(uDB2Template, nElDB2)  
      ;
      ; --- Fill the vector of structures ---
      ; --- Copy first database ---
      iElDB2=0
      for iElDB0=0,nELDB0-1 do begin
        uDBtmp=uDB2[iElDB2]
        struct_assign, uDB0[iElDB0], uDBtmp
        uDB2[iElDB2]=uDBtmp
        iElDB2=iElDB2+1
      endfor ; iElDB0
      ; --- Copy second database ---
      for iElDB1=0,nELDB1-1 do begin
        uDBtmp=uDB2[iElDB2]
        struct_assign, uDB1[iElDB1], uDBtmp
        uDB2[iElDB2]=uDBtmp
        iElDB2=iElDB2+1
      endfor ; iElDB0
      ;
    endif else begin ; ..............................................................................
      uDB1tmp=udb_Convert(uDB_Type0, uDB1)
      uDB2=udb_Merge(MergeMode, uDB0, uDB1tmp)
      uDB_Delete, uDB1tmp
      ;
    endelse ; .......................................................................................
    ;
  endif else if (uDB_Type0 eq 'StructureOfVectors') then begin ; -----------------------------------
    ; --- This type is not implemented properly, yet.         ---
    ; --- Transform to other type, merge, and transform back. ---
    uDB0tmp=udb_Convert('VectorOfStructures', uDB0)
    uDB1tmp=udb_Convert('VectorOfStructures', uDB1)
    uDB2tmp=udb_merge(MergeMode, uDB0tmp, uDB1tmp)
    uDB_Delete, uDB0tmp
    uDB_Delete, uDB1tmp
    uDB2=udb_Convert(uDB_Type0, uDB2tmp)
    uDB_Delete, uDB2tmp
    ;
  endif else if (uDB_Type0 eq 'UIOTable') then begin ; ----------------------------------------------
    ; --- This type is not implemented properly, yet.         ---
    ; --- Transform to other type, merge, and transform back. ---
    uDB0tmp=udb_Convert('VectorOfStructures', uDB0)
    uDB1tmp=udb_Convert('VectorOfStructures', uDB1)
    uDB2tmp=udb_merge(MergeMode, uDB0tmp, uDB1tmp)
    uDB_Delete, uDB0tmp
    uDB_Delete, uDB1tmp
    uDB2=udb_Convert(uDB_Type0, uDB2tmp)
    uDB_Delete, uDB2tmp
    ;
  endif else begin ; -------------------------------------------------------------------------------
    ; --- Other uDB types ... ---
    message, 'Combination of types not implemented yet'
    ;
  endelse ; ----------------------------------------------------------------------------------------
  ;
endif else if (strmatch(MergeMode, 'Replace', /fold_case)) then begin ; ============================
  ; === Replace existing entries in first database with entries from second one. ===
  ; === Discard others in second database.                                       ===
  ;
  ; --- Create mask that indicates which elements have been copied from uDB1 into uDB0 ---
  if (nElDB1 ge 1) then mask1=lonarr(nElDB1)
  ;
  if (uDB_Type0 eq 'ListOfPointers') then begin ;---------------------------------------------------
    ; --- Everything can be inserted into a pointer list ---
    ;
    ; --- Create empty combined pointer array ---
    nElDB2=nElDB0
    uDB2=ptrarr(nElDB2>1, /allocate_heap)
    ;
    ; --- Copy first database ---
    iElDB2=0
    for iElDB0=0,nElDB0-1 do begin
      if (ptr_valid(uDB0[iElDB0])) then begin
        *uDB2[iElDB2]=*uDB0[iElDB0]
        iElDB2=iElDB2+1
      endif
    endfor ; iElDB0
    ;
    IdList0=udb_ListAll(uDB0)
    IdList1=udb_ListAll(uDB1)
    ; --- Insert elements from second database ---
    if (uDB_Type1 eq 'ListOfPointers') then begin ; ................................................
      for iElDB1=0,nELDB1-1 do begin
        if (IdList1[iElDB1] ne '') then begin
          iMatch=(where(strcmp(IdList1[iElDB1], IdList0) eq 1))[0]
          if (iMatch gt -1) then begin
            *uDB2[iMatch]=*uDB1[iElDB1]
            mask1[iElDB1]=1
          endif
        endif
      endfor ; iElDB1
      ;
    endif else if (uDB_Type1 eq 'VectorOfStructures') then begin ; .................................
      for iElDB1=0,nElDB1-1 do begin
        if (IdList1[iElDB1] ne '') then begin
          iMatch=(where(strcmp(IdList1[iElDB1], IdList0) eq 1))[0]
          if (iMatch gt -1) then begin
            *uDB2[iMatch]=uDB1[iElDB1]
            mask1[iElDB1]=1
          endif
        endif
      endfor ; iElDB1
      ;
      ;
    endif else if (uDB_Type1 eq 'StructureOfVectors') then begin ; .................................
      ; --- Get tag names and number of second data base ---
      TagsDB1=udb_GetTags(uDB1, nTags=nTagsDB1)
      ;help, nTagsDB1, nELDB1
      ; --- Create template structure with only one element (but with all tags) ---
      if (nTagsDB1 ge 1) then uDBtemplate=create_struct(TagsDB1[0], (uDB1.(0))[0])
      for iTagsDB1=1,nTagsDB1-1 do begin
        uDBtemplate=create_struct(uDBtemplate, TagsDB1[iTagsDB1], (uDB1.(iTagsDB1))[0])
      endfor ; iTagsDB1
      ;help, /str, uDBtemplate
      ;
      for iElDB1=0,nELDB1-1 do begin
        if (IdList1[iElDB1] ne '') then begin
          iMatch=(where(strcmp(IdList1[iElDB1], IdList0) eq 1))[0]
          if (iMatch gt -1) then begin
            uDBsingle=uDBtemplate
            for iTagsDB1=0,nTagsDB1-1 do begin
              uDBsingle.(iTagsDB1)=(uDB1.(iTagsDB1))[iMatch<(n_elements(uDB1.(iMatch))-1)]
            endfor ; iTagsDB1
            ;help, iMatch
            ;help, /str, uDBsingle
            ;
            *uDB2[iMatch]=uDBsingle
            mask1[iElDB1]=1
          endif
        endif
      endfor ; iElDB1
      ;
    endif else if (uDB_Type1 eq 'UIOTable') then begin ; ...........................................
      ; --- Get tag names and number of second data base ---
      TagsDB1=udb_GetTags(uDB1, nTags=nTagsDB1)
      ;help, nTagsDB1, nELDB1
      ;print, TagsDB1
      nstr='__NAME'
      ustr='__UNIT'
      ; --- Create template structure with only one element (but with all tags) ---
      if (nTagsDB1 ge 1) then begin
        uio_tabr, val, ident, uDB1, incolumn=1, name=valName, unit=valUnit
        uDBtemplate=create_struct(ident, val[0], ident+nstr, valName, ident+ustr, valUnit)
      endif
      for iTagsDB1=1,nTagsDB1-1 do begin
        uio_tabr, val, ident, uDB1, incolumn=iTagsDB1+1, name=valName, unit=valUnit
        uDBtemplate=create_struct(uDBtemplate, ident, val[0], ident+nstr, valName, ident+ustr, valUnit)
      endfor ; iTagsDB1
      ;help, /str, uDBtemplate
      ;
      for iElDB1=0,nELDB1-1 do begin
        print, iElDB1, ' ', IdList1[iElDB1]
        if (IdList1[iElDB1] ne '') then begin
          iMatch=(where(strcmp(IdList1[iElDB1], IdList0) eq 1))[0]
          ;help, iMatch
          if (iMatch gt -1) then begin
            uDBsingle=uDBtemplate
            for iTagsDB1=0,nTagsDB1-1 do begin
              uio_tabr, val, ident, uDB1, incolumn=iTagsDB1+1
              ;print, iTagsDB1, nTagsDB1, ident, val
              uDBsingle.(3*iTagsDB1)=val[iMatch]
            endfor ; iTagsDB1
            ;help, iELDB1
            ;help, /str, uDBsingle
            ;
            *uDB2[iMatch]=uDBsingle
            mask1[iElDB1]=1
          endif
        endif
      endfor ; iElDB1
      ;
    endif else begin ; .............................................................................
      message, 'Unknown uDB Type.'
      ;
    endelse ; ......................................................................................
    ;
  endif else if (uDB_Type0 eq 'VectorOfStructures') then begin ; -----------------------------------
    if (uDB_Type1 eq 'VectorOfStructures') then begin ; ............................................
      ; --- Create tag lists ---
      TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
      TagsDB1=udb_GetTags(uDB1, nTags=nTagsDB1)
      TagsDB2=udb_MergeStringVectors(TagsDB0, TagsDB1)
      nTagsDB2=n_elements(TagsDB2)
      ;
      ; --- Create template structure with tags from both input structures ---
      for iTagsDB2=0,nTagsDB2-1 do begin
        ; --- Find matching tag in one of the input structures to get a value of proper type ---
        iMatch=(where(strcmp(TagsDB2[iTagsDB2], TagsDB0) eq 1))[0]
        if (iMatch ne -1) then begin
          TagValue=(uDB0.(iMatch))[0]
          ;if (size(uDB0.(iMatch), /tname) eq 'STRING') then begin
          ;  TagValue=''
          ;endif else begin
          ;  TagValue=(uDB0.(iMatch))[0]*0b
          ;endelse
        endif else begin
          iMatch=(where(strcmp(TagsDB2[iTagsDB2], TagsDB1) eq 1))[0]
          if (iMatch ne -1) then begin
            TagValue=(uDB1.(iMatch))[0]
            ;if (size(uDB1.(iMatch), /tname) eq 'STRING') then begin
            ;  TagValue=''
            ;endif else begin
            ;  TagValue=(uDB1.(iMatch))[0]*0b
            ;endelse
          endif else begin
            message, 'Merged tag list contains unknown tag'
          endelse
        endelse
        ; --- Append new tag with value ---
        if (iTagsDB2 eq 0) then begin
          uDB2Template=create_struct(              TagsDB2[iTagsDB2], TagValue)
        endif else begin
          uDB2Template=create_struct(uDB2Template, TagsDB2[iTagsDB2], TagValue)
        endelse
      endfor ; iTagsDB2
      ;
      ; --- Create vector of structures ---
      nElDB2=nElDB0
      uDB2=replicate(uDB2Template, nElDB2)  
      ;
      ; --- Fill the vector of structures ---
      ; --- Copy first database ---
      for iElDB0=0,nELDB0-1 do begin
        uDBtmp=uDB2[iElDB0]
        struct_assign, uDB0[iElDB0], uDBtmp
        uDB2[iElDB0]=uDBtmp
      endfor ; iElDB0
      ;
      ; --- Copy over elements that match from second database ---
      ;uDB0_MaskMasterkey=strcmp(TagsDB0, uDB_MasterKey, /fold_case)
      ;uDB1_MaskMasterkey=strcmp(TagsDB1, uDB_MasterKey, /fold_case)
      iMasterKeyDB0=(where(strcmp(TagsDB0, uDB_MasterKey)))[0]
      iMasterKeyDB1=(where(strcmp(TagsDB1, uDB_MasterKey)))[0]
      if ((iMasterkeyDB0 ge 0) and (iMasterkeyDB1 ge 0)) then begin
        ; --- Both databases contain the master key ---
        for iElDB1=0,nELDB1-1 do begin
          iMatch=(where(strcmp(uDB0.(iMasterkeyDB0), uDB1[iElDB1].(iMasterKeyDB1))))[0]
          if (iMatch ge 0) then begin
            uDBtmp=uDB2[iMatch]
            struct_assign, uDB1[iElDB1], uDBtmp
            uDB2[iMatch]=uDBtmp
            mask1[iElDB1]=1
          endif
        endfor ; iElDB0
      endif
      ;
    endif else begin ; ..............................................................................
      uDBtmp=udb_Convert(uDB_Type0, uDB1)
      uDB2=udb_merge('Replace', uDB0, uDBtmp, mask1=mask1)
      uDB_Delete, uDBtmp
      ;
    endelse ; .......................................................................................
    ;
 endif else if (uDB_Type0 eq 'StructureOfVectors') then begin ; -----------------------------------
    ; --- This type is not implemented properly, yet.         ---
    ; --- Transform to other type, merge, and transform back. ---
    uDB0tmp=udb_Convert('VectorOfStructures', uDB0)
    uDB1tmp=udb_Convert('VectorOfStructures', uDB1)
    uDB2tmp=udb_merge(MergeMode, uDB0tmp, uDB1tmp)
    uDB_Delete, uDB0tmp
    uDB_Delete, uDB1tmp
    uDB2=udb_Convert(uDB_Type0, uDB2tmp)
    uDB_Delete, uDB2tmp
    ;
 endif else if (uDB_Type0 eq 'UIOTable') then begin ; ----------------------------------------------
    ; --- This type is not implemented properly, yet.         ---
    ; --- Transform to other type, merge, and transform back. ---
    uDB0tmp=udb_Convert('VectorOfStructures', uDB0)
    uDB1tmp=udb_Convert('VectorOfStructures', uDB1)
    uDB2tmp=udb_merge(MergeMode, uDB0tmp, uDB1tmp)
    uDB_Delete, uDB0tmp
    uDB_Delete, uDB1tmp
    uDB2=udb_Convert(uDB_Type0, uDB2tmp)
    uDB_Delete, uDB2tmp
    ;
  endif else begin ; -------------------------------------------------------------------------------
    ; --- Other uDB types ... ---
    message, 'Combination of types not implemented yet'
    ;
  endelse ; ----------------------------------------------------------------------------------------
  ;
endif else if (strmatch(MergeMode, 'Merge', /fold_case)) then begin ; ==============================
  ; === Replace existing entries in first database with entries from second one. ===
  ; === Append the rest from the second database at the end of the first one.    ===
  ;
  ; --- Replace existing values in uDB0 with new ones from uDB1 ---
  uDB2tmp=udb_Merge('Replace', uDB0, uDB1, mask1=Mask)
  Mask=1-Mask
  ind=where(Mask, nMask)
  ;
  if (nMask eq 0) then begin
    ; --- No new entries ---
    uDB2=udb_Copy(uDB2tmp)
    udb_Delete, uDB2tmp
  endif else begin
    ; --- Append the new entries ---
    uDB1tmp=udb_Copy(uDB1, mask=Mask)
    uDB2=udb_Merge('Append', uDB2tmp, uDB1tmp)
    udb_Delete, uDB1tmp
    udb_Delete, uDB2tmp
  endelse
  ;
endif else begin ; =================================================================================
  message, 'Unknown Merge Mode'
endelse ; ==========================================================================================

return, uDB2

end ; udb_Merge
