function udb_MergeStringVectors, v0, v1
;------------------------------------------------------------------------------
;+
; NAME:
;   udb_MergeStringVectors ('universal_data_base_Merge_String_Vectors')
;
; PURPOSE:
;   Merge two vectors of strings maintining the order where possible
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_MergeStringVectors(v0, v1)
;
; INPUT:
;   v0:     ('vector_0') First vector of strings
;   v1:     ('vector_1') Second vector of strings
;
; OUTPUT:
;   result: Result vector of strings (doublettes removed)
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   The entries in the individual vectors should be uniq.
;   However, the vectors don't have to be sorted.
;   And double entries (one in each vector) are allowed.
;
; PROCEDURE:
;
; EXAMPLE:
;   IDL> print, udb_mergestringvectors(['a', 'b', 'c', 'd'],['a', 'c', 'b', 'd'])               
;     a b c d
;   IDL> print, udb_mergestringvectors(['b', 'd'],['a', 'b', 'c', 'e', 'd', 'f'])
;     a b c e d f
;   IDL> print, udb_mergestringvectors(['b', 'd', 'g'],['a', 'b', 'c', 'e', 'd', 'f'])
;     a b c e d f g
;   IDL> print, udb_mergestringvectors(['a', 'b', 'c', 'd', 'e'],['d', 'e', 'a', 'b', 'c', 'g'])
;     a b c d e g
;
; MODIFICATION HISTORY:
;   2009-11-31 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

n0=n_elements(v0)
n1=n_elements(v1)
help,n0,n1

if (n0 eq 0) then begin
  ; --- First vector is empty: return second one ---
  return, v1
  ;
endif else if (n1 eq 0) then begin
  ; --- Second vector is empty: return first one ---
  return, v0
  ;
endif else begin
  ; --- Both vectors are non-empty: merge them properly ---
  ;
  ; --- Index of elements of v0 in v1 (and vice versa) ---
  p0in1=lonarr(n0)
  p1in0=lonarr(n1) & p1in0[*]=-1
  for i0=0,n0-1 do begin
    p0in1[i0]=(where(strcmp(v0[i0], v1) eq 1))[0]
    if (p0in1[i0] ne -1) then begin
      p1in0[p0in1[i0]]=i0
    endif
  endfor ; i0
  ;
  print, p0in1, format='(20I3)'
  print, p1in0, format='(20I3)'
  ;
  ; --- Number of elements in both vectors ---
  ind=where(p1in0 ne -1, nDouble)
  ; --- Number of elements in new vector ---
  n2=n0+n1-nDouble
  v2=strarr(n2)
  ;
  help, nDouble, n2
  ;
  i0=0 & i1=0 & i2=0
  while (i2 lt n2) do begin
    print, i2, i0, i1, p0in1[i0<(n0-1)], p1in0[i1<(n1-1)], format='(I3,1X,2I3,1X,2I3)'
    if            (i0 ge n0) then begin
      c=0
      if (p1in0[i1] ge -1) then begin
        p1in0[i1]=-2
        v2[i2]=v1[i1] & i2=i2+1
      endif
      i1=i1+1
    endif else if (i1 ge n1) then begin
      c=1
      if (p0in1[i0] ge -1) then begin
        p1in0[i1]=-2
        v2[i2]=v0[i0] & i2=i2+1
      endif
      i0=i0+1
    endif else if (p1in0[i1] eq -1) then begin
      c=2
      v2[i2]=v1[i1] & i2=i2+1
      i1=i1+1
    endif else if (p0in1[i0] eq -1) then begin
      c=3
      v2[i2]=v0[i0] & i2=i2+1
      i0=i0+1
    endif else if (p0in1[i0] eq i1) then begin
      c=4
      v2[i2]=v0[i0] & i2=i2+1
      p0in1[i0]=-2
      p1in0[i1]=-2
      i0=i0+1
      i1=i1+1
    endif else if (p0in1[i0] ne -2) then begin
      c=5
      v2[i2]=v0[i0] & i2=i2+1
      p1in0[p0in1[i0]]=-2
      p0in1[i0]=-2
      i0=i0+1
    endif else if (p1in0[i1] ne -2) then begin
      c=6
      v2[i2]=v1[i1] & i2=i2+1
      p0in1[p1in0[i1]]=-2
      p1in0[i1]=-2
      i1=i1+1
    endif else begin
      c=7
      i0=i0+1
      i1=i1+1
    endelse
    print, c, ' -> ', v2[i2-1], format='(16X,I2,A,1X,A)'
  endwhile ; i2
  ;
  return, v2
  ;
endelse

end ; udb_MergeStringVectors
