pro uio_chconv, conv,prefix, nc, ouconv, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_chconv ('uio_channel_conversion')
;
; PURPOSE:
;   Actualize list of conversion types for all channels.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   uio_chconv, conv,prefix, nc, ouconv, outstr,ierr
;
; INPUT:
;   conv:   ('conversion') character, one conversion type of list cvnals in
;           common block uio_cvlist;
;           conv='' -> default conversion type
;   prefix: ('prefix') character:
;           ' ': Search if (conv .eq. cvnals(?))
;           'X': Search if 'X'//cvnals(?) in conv
;   nc:     ('number_channel') integer, number of channel
;
; OUTPUT:
;   ouconv: ('output_conversion'), character, allowed conversion type
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; LOCAL VARIABLES:
;   nconv:  ('number_conversion'), character, number of conversion type
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;
; SIDE EFFECTS:
;   Change parameters in common block uio_chainf in uio_uiocstinc.
;
; RESTRICTIONS:
;   
; PROCEDURE:
;   This routine is used for opening files for reading.
;   Look if requested conversion type is possible for current machine, i.e.
;   conversion type conv is in list cvnals in common block uio_cvlist.
;   If it is possible save it in array cvnum in common block uio_chainf.
;
; EXAMPLE:
;   uio_chunit, 'init',        20,90, outstr,ierr
;   uio_chunit, 'get',         nc, 0, outstr,ierr
;   uio_chunit, 'unformatted', nc, 0, outstr,ierr
;   uio_chconv, 'ieee_4',''  , nc, ouconv, outstr,ierr
;   help, nc, ouconv, outstr,ierr
;     --->
;   NC              INT       =       20
;   OUCONV          STRING    = 'ieee_4'
;   OUTSTR          STRING    = 'known type'
;   IERR            INT       =        0
;
; MODIFICATION HISTORY:
;   18. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   20. 9.96 (B.F.) corrections
;    2.10.96 (B.F.) ncmin may become negative, cvnum(.) -> cvnum(.-ncmin)
;   14.10.96 (B.F.) More comments
;   15.10.96 (B.F.) Default conversion type ncvdef introduced
;   16.10.96 (B.F.) 'ieee' -> 'ieee_4'
;-
;------------------------------------------------------------------------------
;
@uio_uiocstinc
@uio_uiocvlinc
@uio_uiosizinc
;
; --- Look if conversion type 'conv' is in list 'cvnals()' ---
if (n_elements(conv) eq 0) then goto, defaultlabel
if (strtrim(conv) eq '') then goto, defaultlabel
nconv=0
if (strtrim(prefix) eq '') then begin
  loop10label:
    if (conv eq  cvnals(nconv)) then goto, foundlabel
    nconv=nconv+1
  if (nconv lt ncv) then goto, loop10label
endif else begin
  loop20label:
    str=prefix+cvnals(nconv)
    if (strpos(conv,str) ne -1) then goto, foundlabel
    nconv=nconv+1
  if (nconv lt ncv) then goto, loop20label
endelse
;
; --- Unknown conversion type: take first value ---
notfoundlabel:
nconv=0
outstr='unknown type'
ierr=1
goto, endlabel
;
; --- Take default conversion type ---
defaultlabel:
nconv=ncvdef
outstr='default type'
ierr=0
goto, endlabel
;
; --- Conversion type found ---
foundlabel:
outstr='known type'
ierr=0
;
endlabel:
cvnum(nc-ncmin)=nconv
ouconv=cvnals(nconv)
;
end ; uio_chconv
