pro uio_chunit, cmdstr, n1,n2, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_chunit ('uio_channel_unit')
;
; PURPOSE:
;   Initialize, store and actualize a list of free and occupied unit numbers.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   uio_chunit, cmdstr, n1,n2, outstr,ierr
;
; INPUT:
;   cmdstr: ('command_string'), character, supported values:
;           'init': First initialization command: Set total except
;                   selected range (n1-n2) to 'unavailable', selected
;                   range to 'free'
;           'setrange': Set selected range (n1-n2) to 'free'
;           'resetrange': Set selected range (n1-n2) to 'unavailable'
;           'ask': Give out status of selected unit (n1) in outstr
;           'get': Search lowest 'free' unit, set its status to 'occupied',
;                  give out unit number in n1
;           'set': Set selected free unit (n1) to 'occupied'
;           'formatted': Set selected occupied unit (n1) to 'formatted'
;           'unformatted': Set selected occupied unit (n1) to 'unformatted'
;           'return': Reset selected unit (n1) to 'free'
;           'maxrange': Give out minimum (n1) and maximum possible (n2) unit
;                       number
;
; INPUT/OUTPUT:
;   n1,n2:  ('number_1, number_2') integer, number of channel
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;
; SIDE EFFECTS:
;   Change parameters in common block uio_chainf
;
; RESTRICTIONS:
;   Do not open channel units within the range managed by uio_chunit without
;   using this routine. Otherwise the saved state of the units may not
;   represent the current state of open/closed units.
;   
; PROCEDURE:
;   According to command string cmdstr look into character array ncstat or
;   modify it. The array ncstat contains the states of all input/output channels
;   within a specified range.
;   The routine is similar to the IDL routines get_lun and free_lun.
;   But uio_chunit and uio_chconv store additional information.
;
; EXAMPLE:
;   Initialization of chunit, channel number range is 20 to 80:
;     uio_chunit, 'init', 20,80, outstr,ierr
;   Search and reservation of a free channel, answer in nc:
;     uio_chunit, 'get',nc,0, outstr,ierr
;   Open and close file
;     openw, nc,'testfile'
;     close, nc
;   Return unit number
;     uio_chunit, 'return', nc, 0, outstr,ierr
;
; MODIFICATION HISTORY:
;   18. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;    2.10.96 (B.F.) ncmin may become negative, ncstat(.) -> ncstat(.-ncmin)
;   14.10.96 (B.F.) Output channel is (two byte) integer, more comments
;-
;------------------------------------------------------------------------------
;
@uio_uiocstinc
;
ierr=0
;
if (cmdstr eq 'ask') then begin
  ; --- Give out status of selected unit ---
  if (n1 lt ncmin) then begin
    outstr='unknown'
    ierr=1
  endif else if (n1 gt ncmax) then begin
    outstr='unknown'
    ierr=2
  endif else begin
    outstr=ncstat(n1-ncmin)
  endelse
endif else if (cmdstr eq 'init') then begin
  ; --- First initialisation command: Set total except selected range ---
  ; --- to 'unavailable', selected range to 'free'.                   ---
  ncmax=99
  ncmin=-1
  ncnum=ncmax-ncmin+1
  ncstat=strarr(ncnum)
  cvnum=intarr(ncnum)
  ncstat(0:ncmax-ncmin)='unavailable'
  ncstat((ncmin>n1<ncmax)-ncmin:(ncmin>n2<ncmax)-ncmin)='free'
  outstr='done'
endif else if (cmdstr eq 'setrange') then begin
  ; --- Set selected range to 'free' ---
  ncstat((ncmin>n1<ncmax)-ncmin:(ncmin>n2<ncmax)-ncmin)='free'
  outstr='done'
endif else if (cmdstr eq 'resetrange') then begin
  ; --- Set selected range to 'unavailable' ---
  ncstat((ncmin>n1<ncmax)-ncmin:(ncmin>n2<ncmax)-ncmin)='unavailable'
  outstr='done'
endif else if (cmdstr eq 'get') then begin
  ; --- Search lowest 'free' unit, set its status to 'occupied', ---
  ; --- give out unit number.                                    ---
  n1=ncmax
  outstr='nothing free'
  ierr=3
  ind=where(ncstat(0:ncmax-ncmin) eq 'free',count)
  if (count eq 0) then begin
    n1=ncmax
    outstr='nothing free'
    ierr=3
  endif else begin
    n1=ncmin+fix(ind(0))
    outstr=ncstat(n1-ncmin)
    ncstat(n1-ncmin)='occupied'
    ierr=0
  endelse
endif else if (cmdstr eq 'set') then begin
  ; --- Set selected unit to 'occupied' ---
  if (n1 lt ncmin) then begin
    outstr='unknown'
    ierr=1
  endif else if (n1 gt ncmax) then begin
    outstr='unknown'
    ierr=2
  endif else begin
    outstr=ncstat(n1-ncmin)
    if (ncstat(n1-ncmin) eq 'free') then begin
      ncstat(n1-ncmin)='occupied'
    endif else begin
      ierr=4
    endelse
  endelse
endif else if (cmdstr eq 'formatted') then begin
  ; --- Set selected unit to 'formatted' ---
  if (n1 lt ncmin) then begin
    outstr='unknown'
    ierr=1
  endif else if (n1 gt ncmax) then begin
    outstr='unknown'
    ierr=2
  endif else begin
    outstr=ncstat(n1-ncmin)
    if (ncstat(n1-ncmin) eq 'occupied') then begin
      ncstat(n1-ncmin)='formatted'
    endif else begin
      ierr=4
    endelse
  endelse
endif else if (cmdstr eq 'unformatted') then begin
  ; --- Set selected unit to 'unformatted' ---
  if (n1 lt ncmin) then begin
    outstr='unknown'
    ierr=1
  endif else if (n1 gt ncmax) then begin
    outstr='unknown'
    ierr=2
  endif else begin
    outstr=ncstat(n1-ncmin)
    if (ncstat(n1-ncmin) eq 'occupied') then begin
      ncstat(n1-ncmin)='unformatted'
    endif else begin
      ierr=4
    endelse
  endelse
endif else if (cmdstr eq 'return') then begin
  ; --- Reset selected unit to 'free' ---
  if (n1 lt ncmin) then begin
    outstr='unknown'
    ierr=1
  endif else if (n1 gt ncmax) then begin
    outstr='unknown'
    ierr=2
  endif else begin
    outstr=ncstat(n1-ncmin)
    if ((ncstat(n1-ncmin) eq 'occupied'   ) or $
        (ncstat(n1-ncmin) eq 'formatted'  ) or $
        (ncstat(n1-ncmin) eq 'unformatted')) then begin
      ncstat(n1-ncmin)='free'
    endif else begin
      ierr=4
    endelse
  endelse
endif else if (cmdstr eq 'maxrange') then begin
  ;  --- Give out minimum and maximum possible unit number ---
  n1=ncmin
  n2=ncmax
  outstr='range'
endif else if (cmdstr eq 'check') then begin
  ; --- Check wether ncstat() matches opened and closed units ---
  for n=ncmin,ncmax do begin
    if ((ncstat(n-ncmin) eq 'free'       ) or $
        (ncstat(n-ncmin) eq 'occupied'   ) or $
        (ncstat(n-ncmin) eq 'formatted'  ) or $
        (ncstat(n-ncmin) eq 'unformatted')) then begin
      ncfstat=fstat(n)
      od=ncfstat.open
      odnc=((ncstat(n-ncmin) eq 'occupied'   ) or $
            (ncstat(n-ncmin) eq 'formatted'  ) or $
            (ncstat(n-ncmin) eq 'unformatted'))
      if ((      od  and ( not odnc))  or $
          (( not od) and       odnc ) ) then begin
        outstr='not '+ncstat(n-ncmin)
        ierr=6
        n1=n
        goto, errorlabel
      endif
    endif
  endfor
  outstr='match'
  ;
  errorlabel:
endif else begin
  outstr='pardon'
  ierr=5
endelse
;
end ; uio_chunit
