pro uio_data, ident, mode=mode0, $
  filename=filename0, family=family0, ngroup=ngroup, $
  value=val, termt=termt, ntt=ntt, $
  outidentlist=identlist, outposlist=poslist, outfilelist=filelistout, $
  outstr=outstr, ierr=ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_data ('uio_data')
;
; PURPOSE:
;   Handle uio-file(s) in quasi direct access mode.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   uio_data
;
; INPUT:
;   ident:  ('identifier') character, entry name
;
; INPUT/OUTPUT:
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   mode:   ('mode') character:
;           'open':     Open specified file(s)
;           'close':    Close specified file(s)
;           'allclose': Close all files
;           'read':     Give selected entry from specified file (default)
;           'content':  Give content of selected file
;           'files':    Print list of files ('ls')
;           'help':     Print some helpful information
;
; ROUTINES:
;   sort1:      Sort vector
;   uniq1:      Return the subscripts of the unique elements in an array
;   uio_chpos:  Give current file position or jump to specified position
;   uio_closrd: Close file after reading
;   uio_filcon: Determine file content: list of all entries with its positions
;   uio_openrd: Open file for reading, read header
;   uio_pptrmt: Print term table in pretty form
;   uio_rd:     Reading scalar and array data of all types
;   uio_rdhd:   Read header of variable and extract keywords
;   uio_rdtab:  Read table of integer, real, and/or character data from file
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_data_common
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Read files, move file position, change content of common block.
;
; RESTRICTIONS:
;   Read only files in uio file format.
;   The maximum number of open files and of entries per file is limited.
;   
; PROCEDURE:
;   This routine is intended to be an user front end to the uio-routines.
;   On can open (and close again) one (or a list of) file(s).
;   Each file (or a group or 'family' of files) can be referred to by a
;   short 'family' name.
;   
;
; EXAMPLE:
;   ; --- Print list of mode keywords of uio_data ---
;   uio_data, mode='help'
;
;   ; --- Open a list of files with appropriate short 'family' names ---
;   uio_data, mode='open', file='uiotestfile*.txt', family='test'
;   uio_data, mode='open', file='holmu.atm', family='holmu'
;   uio_data, mode='open', file='../vald/vald*.data', family='vald'
;
;   ; --- Show list of open files ---
;   uio_data, mode='files'
;   ; --- Show content of all families ---
;   uio_data, mode='content', family=['test','holmu','vald']
;   ; --- Show content of only one family ---
;   uio_data, mode='content', family=['test']
;   ; --- Show content of only one file ---
;   uio_data, mode='content', file='holmu.atm'
;   ; --- Show content of last file ---
;   uio_data, mode='content'
;   ; --- Show content of table 'atmosphere' of last file ---
;   uio_data, mode='tablecontent', 'atmosphere'
;
;   ; --- Read table 'atmosphere' out of last file ---
;   uio_data, mode='read', 'atmosphere', val=holmu
;   ; --- Read column 'tauross' of table 'atmosphere' of last file ---
;   uio_data, mode='read', 'atmosphere.tauross', val=tau
;   ; --- Read column 'tauross' of table 'atmosphere' of last file ---
;   uio_data, mode='read', 'atmosphere.t', val=t
;   ; --- Read data and plot them in one line with uio_d ---
;   plot,alog10(uio_d('atmosphere.tauross')), uio_d('atmosphere.t')
;
;   ; --- Close file ---
;   uio_data, mode='close',file='holmu.atm'
;   ; --- Close all files ---
;   uio_data, mode='allclose'
;
; MODIFICATION HISTORY:
;   29.10.96 Written by Bernd Freytag, Astrophysik Kiel
;   30.10.96 (B.F.) First working version
;   31.10.96 (B.F.) 'get' -> 'read', 'allclose'
;    1.11.96 (B.F.) 'files', 'help';
;            for 'open' and 'close' filename may be vector
;   11.11.96 (B.F.) Check first if nfile eq 0
;   13.11.96 (B.F.) Keyword ngroup added
;   14.11.96 (B.F.) Modifications
;   16.11.96 (B.F.) lastfilename, lastfamily-handling modified,
;            lasttable_file, lasttable_pos added to common block
;   25.11.96 (B.F.) uio_filcon gives position of eof
;    1. 1.97 (B.F.) 'X' -> '1X'
;   30. 3.98 (B.F.) nentrymax=300 -> 1000
;    8.12.98 (B.F.) Print less
;   30. 3.98 (B.F.) nentrymax=1000 -> 1500
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
common uio_data_common, nfile, nfilemax, nentrymax, $
  nentry, filelist, familylist, nclist, identarr, posarr, $
  lastfilename, lastfamily, lastgroup, $
  lasttable, lasttable_file, lasttable_pos
;
; --- Initialize uio if necessary ---
if (n_elements(intena) eq 0) then begin
  uio_init, outstr,ierr
endif
;
; --- Initialize common block at first call of routine ---
if (n_elements(nfilemax) eq 0) then begin
  nfilemax=70                ; maximum number of open files
  nentrymax=1500             ; maximum number of entries per file
  nfile=0                    ; current number of open files
  nentry=intarr(nfilemax)    ; current number of entries per file
  filelist=strarr(nfilemax)  ; list of names of open files
  familylist=strarr(nfilemax); list of short names for files or groups of files
  nclist=intarr(nfilemax)    ; list of channel unit numbers
  identarr=strarr(3,nentrymax,nfilemax) ; array with type and identifier of
                                        ; every entry for all open files
  posarr=lonarr(nentrymax,nfilemax)     ; array with position of
                                        ; every entry for all open files
  ;
  lastfilename=''
  lastfamily=''
  lastgroup=-1
  ;
  lasttable=''
  lasttable_file=''
  lasttable_pos=-999L
endif
;
; --- Default choices for keywords ---
if (n_elements(mode0) eq 0) then begin
  mode='read'
endif else begin
  mode=mode0
endelse
if (keyword_set(filename0)) then begin
  filename=strtrim(filename0,2)
endif else begin
  filename=''
endelse
if (keyword_set(family0)) then begin
  family=strtrim(family0,2)
endif else begin
  family=''
endelse
;if (keyword_set(filename)) or (keyword_set(family)) then lastgroup=-1
;
; --- Find mode ---
lm=strlen(mode)
if (mode eq strmid('open',0,lm)) then begin
  mode='open'
endif else if (mode eq strmid('content',0,lm)) then begin
  mode='content'
endif else if (mode eq strmid('tablecontent',0,lm)) then begin
  mode='tablecontent'
endif else if (mode eq strmid('files',0,lm)) then begin
  mode='files'
endif else if (mode eq strmid('ls',0,lm)) then begin
  mode='files'
endif else if (mode eq strmid('read',0,lm)) then begin
  mode='read'
endif else if (mode eq strmid('close',0,lm)) then begin
  mode='close'
endif else if (mode eq strmid('allclose',0,lm)) then begin
  mode='allclose'
endif else begin
  mode='help'
endelse
;
;help, mode
;
if ((mode eq 'close') or (mode eq 'allclose') or (mode eq 'read') or $
    (mode eq 'content') or (mode eq 'tablecontent') or (mode eq 'files')) $
  then begin
  ; --- Do nothing if no file is open ---
  if (nfile eq 0) then begin
    print, 'uio_data: no file opened yet'
    outstr='no file opened yet'
    ierr=1
    goto, endlabel
  endif
endif
;
; --- Check filename, expand wildcards ---
ierr=0
if ((mode eq 'open') or (mode eq 'close') or (mode eq 'read') or $
    (mode eq 'content') or (mode eq 'tablecontent')) then begin
  if (not keyword_set(filename)) then begin
    ; --- No new filename selected: look into existing file list ---
    if (not keyword_set(family)) then begin
      if ((mode eq 'close') or (mode eq 'read') or $
        (mode eq 'content') or (mode eq 'tablecontent')) then begin
        if (lastfilename ne '') then begin
          newfilelist=lastfilename
          filecount=1
        endif else if (lastfamily ne '') then begin
          ifile_tmp=where(lastfamily eq familylist, filecount)
          if (filecount gt 0) then begin
            newfilelist=filelist(ifile_tmp)
          endif else begin
            newfilelist=''
          endelse
        endif else begin
;          print, 'uio_data: no file specified'
          outstr='filename not specified'
          ierr=1
        endelse
      endif
      goto, endchecklabel
    endif
    ;
    ifile=intarr(nfilemax)
    filecount=0
    for i=0,n_elements(family)-1 do begin
      ifile_tmp=where(family(i) eq familylist, fcount)
help,filecount,fcount,nfilecount
      nfilecount=filecount+fcount
      if (nfilecount gt nfilemax) then begin
        print, 'uio_data: too many files in input list'
        nfilecount=nfilemax
      endif
      if (nfilecount gt filecount) then begin
        ifile(filecount:nfilecount-1)=ifile_tmp
        filecount=nfilecount
      endif
    endfor
    if (filecount eq 0) then begin
      print, "uio_data: family name '" +family+ "' not found."
      outstr='family name not found'
      ierr=1
      goto, endlabel
    endif
    help,ifile(0:filecount-1)
    print,ifile(0:filecount-1)
    ifile=ifile(uniq1(ifile(0:filecount-1),sort1(ifile(0:filecount-1))))
    help,ifile
    print,ifile
    filecount=n_elements(ifile)
    newfilelist=filelist(ifile)
    newfamilylist=familylist(ifile)
    familycount=filecount
  endif else begin
    ; --- Filename(s) specified ---
    newfilelist=strarr(nfilemax)
    filecount=0
    for i=0,n_elements(filename)-1 do begin
      newfilelist_tmp=findfile(filename(i), count=fcount)
      nfilecount=filecount+fcount
      if (nfilecount gt nfilemax) then begin
        print, 'uio_data: too many files in input list'
        nfilecount=nfilemax
      endif
      if (nfilecount gt filecount) then begin
        newfilelist(filecount:nfilecount-1)=newfilelist_tmp
        filecount=nfilecount
      endif
    endfor
    if (filecount eq 0) then begin
      outstr='no file found'
      ierr=1
      goto, endlabel
    endif
    newfilelist=newfilelist(uniq1(newfilelist(0:filecount-1), $
                            sort1(newfilelist(0:filecount-1))))
    filecount=n_elements(newfilelist)
    ;
    if ((mode eq 'open') and (keyword_set(family))) then begin
      ; --- Try to find familylist corresponding to filelist ---
      if (n_elements(family) eq 1) then begin
        newfamilylist=replicate(family,filecount)
        familycount=filecount
      endif else if (n_elements(family) eq filecount) then begin
        newfamilylist=family
        familycount=filecount
      endif else begin
        newfamilylist=replicate('',filecount)
        familycount=0
        if (n_elements(family) gt 1) then begin
          print,'uio_data: no relation filename/family found'
        endif
      endelse
    endif else begin
      newfamilylist=replicate('',filecount)
      familycount=0
    endelse
  endelse
endif
;
endchecklabel:
;
if ((mode ne 'help') and (mode ne 'files') and (mode ne 'allclose')) then begin
  if (filecount eq 0) then begin
    print, 'uio_data: no file specified'
    goto, endlabel
  endif
endif
;
;print
;help, filecount, newfilelist
;if (n_elements(newfilelist) gt 0) then print,newfilelist
;help, familycount, newfamilylist
;if (n_elements(newfamilylist) gt 0) then print,newfamilylist
;
; --- Action according to selected mode ---------------------------------------
;
if (mode eq 'open') then begin
  ; --- Open (list of) file(s), determine and save its content ----------------
  ;
  ierr=0
  for i=0,filecount-1 do begin
    file=newfilelist(i)
    if (nfile lt nfilemax) then begin
      ; --- Check if file is already opened by uio_data ---
      if (nfile gt 0) then begin
        ifile=where(file eq filelist(0:nfile-1), fcount)
      endif else begin
        fcount=0
      endelse
      if (fcount eq 0) then begin
        ; --- File not open yet: open file ---
        uio_openrd, nc,file, outstr,ierr, termt=termt, ntt=ntt
        if (ierr gt 0) then begin
          ; --- Error: ignore file ---
          print, 'uio_data: error while opening ',file
          uio_closrd, nc, outstr, ierr
        endif else begin
          print, 'File ' +file+ ' opened'
;          uio_pptrmt, termt,ntt
          filelist(nfile)=file
          if (familycount gt 0) then begin
            familylist(nfile)=newfamilylist(i)
          endif else begin
            familylist(nfile)=''
          endelse
          nclist(nfile)=nc
          nentry(nfile)=0
          ; --- Determine content of file ---
          uio_filcon, nc, identlist,poslist, outstr,ierr, /names
          if (n_elements(poslist) le 1) then begin
            print, 'uio_data: file ', file, ' has no entries.'
            uio_closrd, nc, outstr, ierr
            goto, nextopenlabel
          endif else if (n_elements(poslist) gt nentrymax) then begin
            print, 'uio_data: file ', file, ' too long.'
            poslist=poslist(0:nentrymax-1)
            identlist=identlist(*,0:nentrymax-1)
          endif
          nentry(nfile)=n_elements(poslist)-1
          posarr(0:nentry(nfile)-1,nfile)=poslist(0:nentry(nfile)-1)
          identarr(*,0:nentry(nfile)-1,nfile)=identlist(*,0:nentry(nfile)-1)
          if (keyword_set(filename)) then begin
            lastfilename=file
          endif else if (keyword_set(family)) then begin
            lastfilename=''
          endif
          lastfamily=familylist(nfile)
          nfile=nfile+1
          ierr=0
        endelse
        nextopenlabel:
      endif else begin
        ; --- File already open ---
        if (keyword_set(filename)) then begin
          lastfilename=file
        endif else if (keyword_set(family)) then begin
          lastfilename=''
        endif
        if (familycount gt 0) then familylist(ifile(0))=newfamilylist(i)
        lastfamily=familylist(ifile(0))
      endelse
    endif else begin
      outstr='too many files to open'
      ierr=1
    endelse
  endfor
  ;
  lastgroup=-1
  ;
  if (ierr eq 0) then outstr='done'
endif else if (mode eq 'close') then begin
  ; --- Close (list of) file(s), remove them from internal arrays -------------
  ;
  for i=0,filecount-1 do begin
    ; --- Remove files from list, close file ---
    file=newfilelist(i)
    ifile=where(file eq filelist(0:nfile-1), fcount)
    if (fcount gt 0) then begin
      ifile=ifile(0)
      print, 'File ' +file+ ' closed'
      uio_closrd, nclist(ifile), outstr,ierr
;      if (lastfamily eq familylist(ifile)) then lastgroup=-1
      if (lasttable_file eq file) then begin
        lasttable=''
        lasttable_file=''
        lasttable_pos=-999
      endif
      if (lastfilename eq file) then begin
        lastgroup=-1
        lastfilename=''
      endif
      filelist(ifile)=''
      familylist(ifile)=''
    endif
  endfor
  ;
  ifile=where(filelist(0:nfile-1) ne '', filecount)
  if (filecount eq 0) then begin
    ; --- Remove all files from list ---
    nfile=filecount    
    identarr(*,*,*)=''
    ;
    outstr='all files closed'
  endif else if (filecount lt nfile) then begin
    ; --- Remove files from list ---
    filelist(0:filecount-1)=filelist(ifile)
    familylist(0:filecount-1)=familylist(ifile)
    nclist(0:filecount-1)=nclist(ifile)
    nentry(0:filecount-1)=nentry(ifile)
    identarr(*,*,0:filecount-1)=identarr(*,*,ifile)
    posarr(*,0:filecount-1)=posarr(*,ifile)
    nfile=filecount    
    ;
    outstr='file(s) closed'
  endif
  ;
  ; --- Is a member of lastfamily still open? ---
  if ((nfile gt 0) and (lastfamily ne '')) then begin
    ifile=where(lastfamily eq familylist(0:nfile-1), fcount)
    if (fcount eq 0) then begin
      lastfamily=''
      lastgroup=-1
    endif
  endif else begin
    lastfamily=''
  endelse
  ;
  ierr=0
endif else if (mode eq 'allclose') then begin
  ; --- Close all files, remove them from internal arrays -------------
  ;
  for i=0,nfile-1 do begin
    uio_closrd, nclist(i), outstr,ierr
  endfor  
  nfile=0    
  identarr(*,*,*)=''
  ;
  print, 'All files closed'
  outstr='all files closed'
  lastfilename=''
  lastfamily=''
;  lastgroup=-1
  lasttable=''
  lasttable_file=''
  lasttable_pos=-999
  ;
  ierr=0
endif else if (mode eq 'content') then begin
  ; --- Give content of specified file ----------------------------------------
  ;
  for i=0,filecount-1 do begin
    ; --- Check filename ---
    ifile=where(newfilelist(i) eq filelist(0:nfile-1), fcount)
    ifile=ifile(0)
    if (fcount eq 0) then begin
      print, 'uio_data: file ', newfilelist(i), ' not found'
    endif else begin
      if ((lastfilename ne filelist(ifile)) or $
          (lastfamily ne familylist(ifile))) then lastgroup=-1
      if (keyword_set(filename)) then begin
        lastfilename=filelist(ifile)
      endif else if (keyword_set(family)) then begin
        lastfilename=''
      endif
      lastfamily=familylist(ifile)
      poslist=posarr(0:nentry(ifile)-1,ifile)
      identlist=identarr(*,0:nentry(ifile)-1,ifile)
      filelistout=filelist(0:nfile-1)
      ;
      ; --- Print file content ---
      print, 'Content of file ', filelist(ifile)
      tyfo='A9'
      typad=string(' ',format='('+tyfo+')')
      idfo='A'+strtrim(string(8>max(strlen(identarr(1,0:nentry(ifile),ifile))) $
                          <60, format='(I3)'),2)
      idpad=string(' ',format='('+idfo+')')
      nafo='A'+strtrim(string(8>max(strlen(identarr(2,0:nentry(ifile),ifile))) $
                          <60, format='(I3)'),2)
      napad=string(' ',format='('+nafo+')')
      format='(I4,1X,'+ tyfo +',1X,'+ idfo +',1X,'+ nafo +')'
      for n=0,nentry(ifile)-1 do begin
        print, n, identarr(0,n,ifile)+typad,identarr(1,n,ifile)+idpad, $
          identarr(2,n,ifile)+napad, $
          format=format
      endfor
      ;
      outstr=strtrim(string(nentry(ifile),format='(I2)'),2)+ ' entries in ' + $
               lastfilename
    endelse
  endfor
  ierr=0
endif else if (mode eq 'files') then begin
  ; --- List all open files ---------------------------------------------------
  ;
  ; --- Print information ---
  print, 'Open files:'
  finafo='A'+strtrim(string(8>max(strlen(filelist(0:nfile-1)))<50, $
                            format='(I3)'),2)
  fanafo='A'+strtrim(string(6>max(strlen(familylist(0:nfile-1)))<12, $
                            format='(I3)'),2)
  format='(A1,1X,I3,1X,I3,1X,' +fanafo+ ',1X,' +finafo+ ',1X,I4,1X,A1)'
  headfo='(A1,1X,A3,1X,A3,1X,' +fanafo+ ',1X,' +finafo+ ',1X,A4)'
  print, ' ', '#fi', 'nc', 'family', 'filename', 'entries', $
    format=headfo
  for i=0,nfile-1 do begin
    if (filelist(i) eq lastfilename) then begin
      mark='*'
;      if (lastgroup eq -1) then begin
;        prigroup=''
;      endif else begin
;        prigroup=strtrim(string(lastgroup),2)
;      endelse
    endif else if ((familylist(i) eq lastfamily) and (lastfamily ne '')) then $
      begin
      mark='.'
;      if (lastgroup eq -1) then begin
;        prigroup=' '
;      endif else begin
;        prigroup=strtrim(string(lastgroup),2)
;      endelse
    endif else begin
      mark=' '
;      prigroup=' '
    endelse
    print, mark, i, nclist(i), familylist(i), filelist(i), nentry(i), mark, $
      format=format
  endfor
  ;
  outstr=strtrim(string(nfile,format='(I2)'),2)+ ' files open'
  ierr=0
endif else if ((mode eq 'read') or (mode eq 'tablecontent')) then begin
  ; --- Give value of requested variable --------------------------------------
  ;
  val=''
  ierr=0
  ;
  if (not keyword_set(ident)) then begin
    ; --- No entry name specified ---
    outstr='no entry name specified'
    ierr=1
    if (keyword_set(filename)) then begin
      lastfilename=newfilelist(0)
    endif else if (keyword_set(family)) then begin
      lastfilename=''
    endif
    goto, endlabel
  endif
  ; --- Parse identifier: ident=tab.col -> identfirst=tab, identlast=col ---
  ppos=strpos(ident,'.')
  if (ppos eq -1) then begin
    identfirst=ident
    identlast=''
  endif else begin
    identfirst=strmid(ident,0,ppos)
    identlast=strmid(ident,ppos+1,80)
    if (identfirst eq '') then begin
      outstr='no valid entry name specified'
      ierr=1
      if (keyword_set(filename)) then begin
        lastfilename=newfilelist(0)
      endif else if (keyword_set(family)) then begin
        lastfilename=''
      endif
      goto, endlabel
    endif
  endelse 
  ;
  ; --- Look through file(s) ---
  groupcount=0
  if (n_elements(ngroup) eq 0) then begin
;    searchgroup=lastgroup
    searchgroup=0
  endif else begin
    searchgroup=ngroup
  endelse
  searchgroup=searchgroup>0
  for i=0,filecount-1 do begin
    ifile=where(newfilelist(i) eq filelist(0:nfile-1), fcount)
    if (fcount eq 0) then begin    
      outstr='no file found'
      ierr=1
      goto, nextlooklabel
    endif else begin
      ifile=ifile(0)
      file=filelist(ifile)
    endelse
    ;
    ; --- Are there entries in file? ---
    if (nentry(ifile) eq 0) then begin
      outstr='empty file'
      ierr=1
      goto, nextlooklabel
    endif
    ;
    ientry=where(identarr(1,0:nentry(ifile)-1,ifile) eq identfirst, entrycount)
    groupcount=groupcount+entrycount
    if (groupcount gt searchgroup) then goto, endlooklabel
    ;
    nextlooklabel:
  endfor
  endlooklabel:
  ;
  if (searchgroup ge groupcount) then begin
    print, 'uio_data: no entry found'
    outstr='no entry found'
    ierr=1
    if (keyword_set(filename)) then begin
      lastfilename=file
    endif else if (keyword_set(family)) then begin
      lastfilename=''
    endif
    goto, endlabel
  endif else begin
;    help,groupcount,entrycount,searchgroup,ngroup,lastgroup
    ientry=ientry(-groupcount+entrycount+searchgroup)
  endelse
  ;
;  if ((lastfilename eq file) and (searchgroup eq lastgroup) and $
;  help, lastfilename, file
  if ((lasttable_file eq file) and $
      (lasttable_pos  eq posarr(ientry,ifile)) and $
      (n_elements(lasttable) ne 0)) then begin
    use_lasttable=(identfirst eq lasttable.termt(0,1,0))
  endif else begin
    use_lasttable=0
  endelse
;  help,use_lasttable
  ;
  if (use_lasttable) then begin
    termt=lasttable.termt(*,*,0)
    ntt=lasttable.ntta(0)
  endif else begin
    pos=posarr(ientry,ifile)
    uio_chpos,  nclist(ifile), 'goto',pos, outstr,ierr
    uio_rdhd, nclist(ifile), termt,ntt, outstr,ierr
  endelse
  type=termt(0,0)
  if ((type eq intena) or (type eq realna) or (type eq compna) or $
      (type eq charna)) then begin
    ; --- Read scalar or array ---
    uio_rd, nclist(ifile), termt,ntt, val,idval, outstr,ierr
  endif else if (type eq tabna) then begin
    if (not use_lasttable) then begin
      ; --- Read table ---
      uio_rdtab, nclist(ifile), termt,ntt, lasttable,idval, outstr,ierr
      lasttable_file=file
      lasttable_pos=pos
    endif
    if (identlast eq '') then begin
      ; --- Give entire table ---
      val=lasttable
    endif else begin
      ; --- Select one column ---
      uio_tabr, val,identlast, lasttable, outstr,ierr, termt=termt,ntt=ntt
      if (ierr gt 0) then begin
        val=''
        ntt=0
        outstr='table column not found'
        print, "uio_data: table column '"+ identlast +"' not found."
      endif
    endelse
    if (mode eq 'tablecontent') then begin
      ; --- Print table content ---
      print, 'Content of table '+ identfirst +' in file '+ file
      tyfo='A9'
      typad=string(' ',format='('+tyfo+')')
      idfo='A'+strtrim(string(8> $
               max(strlen(lasttable.termt(0,1,0:lasttable.ncol-1))) $
                                <60, format='(I3)'),2)
      idpad=string(' ',format='('+idfo+')')
      nafo='A'+strtrim(string(8> 40 $
                          <60, format='(I3)'),2)
      napad=string(' ',format='('+nafo+')')
      format='(I4,1X,'+ tyfo +',1X,'+ idfo +',1X,'+ nafo +')'
      for n=0,lasttable.ncol-1 do begin
        uio_exkeyw, 0, lasttable.termt(*,*,n), $
          lasttable.ntta(n), namena,name, outstr,ierr
        print, n, lasttable.termt(0,0,n)+typad, $
                  lasttable.termt(0,1,n)+idpad, $
                  name+napad, $
                  format=format
      endfor
      ;
      outstr=strtrim(string(lasttable.ncol,format='(I2)'),2)+ ' entries in ' + $
               'table '+ identfirst +' in file '+ file
    endif
  endif else begin
    val=''
    outstr='type without value'
    ierr=1
    if (keyword_set(filename)) then begin
      lastfilename=file
    endif else if (keyword_set(family)) then begin
      lastfilename=''
    endif
    goto, endlabel
  endelse
  if (keyword_set(filename)) then begin
    lastfilename=file
  endif else if (keyword_set(family)) then begin
    lastfilename=''
  endif
  lastfamily=familylist(ifile)
  lastgroup=searchgroup
  ;
  print, 'Read ' +type+ ' ' +ident+ ' from ' +file+ $
    ', ngroup# ' +strtrim(string(searchgroup),2)+ $
    ', entry(file)# ' + strtrim(string(ientry),2)
  outstr='done'
  ierr=0
endif else if (mode eq 'help') then begin
  ; --- Print some help information -------------------------------------------
  print,"Possible values of mode keyword:"
  print,"open:     Open specified file(s)"
  print,"  uio_data, mode='open', filename='model.dat'"
  print,"  uio_data, mode='open', filename='model*.txt'"
  print,"close:    Close specified file(s)"
  print,"  uio_data, mode='close', filename='model.dat'"
  print,"  uio_data, mode='close', filename='model*.txt'"
  print,"allclose: Close all files:
  print,"  uio_data, mode='allclose'"
  print,"read:     Give selected entry from specified file (default)"
  print,"  uio_data, mode='read', value=rho, 'rho'"
  print,"  uio_data, value=temp, 'temp'"
  print,"  uio_data, value=p, 'p', filename='model.dat'"
  print,"  uio_data, value=p, 'p', family='mod'"
  print,"tablecontent: Give content of selected table"
  print,"content:  Give content of selected file"
  print,"  uio_data, mode='content', identlist=identlist, poslist=poslist"
  print,"files: Print list of files"
  print,"  uio_data, mode='files'"
  print,"help:     Print some helpful information"
  print,"  uio_data, mode='help'"
  ;
  outstr='done'
  ierr=0
endif
;
endlabel:
;print
pristr=''
if (lastfamily ne '') then pristr=pristr+ ': family ' +lastfamily
if (lastfilename ne '') then pristr=pristr+ ': file ' +lastfilename
;if (lastgroup ne -1) then pristr=pristr+ ': group '+strtrim(string(lastgroup),2)
;if (pristr ne '') then print, 'Current choice' +pristr
;
end ; uio_data
