function uio_dataset_rd, filename, $
  Channel=Channel, HeadRead=HeadRead, nDataSet=nDataSet, $
  BlackList=BlackList, ReadMetaData=ReadMetaData, $
  Date=Date, $
  outstr=outstr, ierr=ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_dataset_rd ('uio_data_set_read')
;
; PURPOSE:
;   Read uio file and put data into anonymous structure
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   data_structure=uio_dataset_rd(filename, $
;                                 Channel=Channel, HeadRead=HeadRead, nDataSet=nDataSet, $
;                                 BlackList=BlackList, ReadMetaData=ReadMetaData, $
;                                 Date=Date, $
;                                 outstr=outstr, ierr=ierr)
;
; INPUT:
;   filename:      ('file_name') string, name of input file
;
; OUTPUT:
;
; KEYWORDS:
;   Channel:       ('Channel') integer, channel number (if file is already open
;                           and should stay so)
;   HeadRead:      ('Head_Read') input flag to indicate that the file header
;                  should be read and put into separate structure
;   nDataSet:      ('number_Data_Set') integer (default: 0): dataset to read
;   BlackList:     ('Black_List') vector of strings with identifiers of quantities NOT to be read
;   ReadMetaData:  ('Read_Meta_Data') input integer (0 or 1) to indicate if meta data (unit
;                  and name) is to be read and included as entry__unit and entry__name
;   outstr:        ('output_string') output string
;   ierr:          ('integer_error') output integer
;                                 =0: correct operation
;                                 >0: an error occured while reading (e.g. the
;                                     specified dataset was not found)
;
; ROUTINES:
;   uio_struct_rd:
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;   uio_uiotabinc: empty table
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;   uio_taborg in uio_uiotabinc
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   The file has to be in uio format and has to contain the proper
;   box/endbox and dataset/enddataset labels.
;
; PROCEDURE:
;   An uio file with the entries
;
;     fileform uio
;     headentry1
;     headentry2
;     headentry3
;     ...
;     label dataset
;       datasetentry1
;       datasetentry2
;       datasetentry3
;       label box
;         character box_id=box1
;         boxentry1
;         boxentry2
;         boxentry3
;         ...
;       label endbox
;       label box
;         character box_id=box2
;         boxentry1
;         boxentry2
;         boxentry3
;         ...
;       label endbox
;       ...
;     label enddataset
;
;   is read into a corresponding IDL structure
;
;     data.head.headentry1
;              .headentry2
;              .headentry3
;              ...
;         .datasetentry1
;         .datasetentry2
;         .datasetentry3
;         ...
;         .box1.boxentry1
;              .boxentry2
;              .boxentry3
;              ...
;         .box2.boxentry1
;              .boxentry2
;              .boxentry3
;              ...
;         ...
;
;   It is read only one dataset at a time. There might be more than one box.
;
; EXAMPLE:
;   You can read
;
;     the first dataset by omitting the 'nDataSet' and 'Channel' keywords,
;       e.g.
;         gt45g35n1_bin=uio_dataset_rd('gt45g35n1.bin1')
;
;     the dataset number 'nlabel' (starting with 0) by specifying 'nDataSet' and
;       omitting the 'Channel' keyword (slow for datasets in files with many
;       entries),
;       e.g.
;         zt142g80n1_cut=uio_dataset_rd('zt142g80n1.cut', nDataSet=1)
;
;     read each dataset one by one by opening the file with uio_openrd and
;       specifying the 'Channel' keyword,
;       e.g.
;         uio_openrd, 'zt142g80n1.cut', nc
;         zt142g80n1_cut0=uio_dataset_rd('zt142g80n1.cut', Channel=nc)
;         zt142g80n1_cut1=uio_dataset_rd('zt142g80n1.cut', Channel=nc)
;         ; ...
;         uio_closrd, nc
;
; MODIFICATION HISTORY:
;   1999-01-06 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   1999-01-28 (B.F.) No initialization of uio
;   1999-04-13 (B.F.) Keyword Channel added
;   1999-08-31 (B.F.) More comments
;   2003-07-08 (B.F. Uppsala) Keyword BlackList added
;   2009-10-10 (B.F. Lyon) Keywords Date, ReadMetaData added; some names put into CamelCase
;   2013-11-20 (B.F.) Pass parameter HeadRead
;-
;------------------------------------------------------------------------------
;
data_structure=uio_struct_rd(filename, HeadRead=HeadRead, $
                             StartLabel='dataset', EndLabel='enddataset', $
                             SubStartLabel='box', SubEndLabel='endbox', $
                             nLabel=nDataSet, Channel=Channel, $
                             BlackList=BlackList, ReadMetaData=ReadMetaData, $
                             Date=Date, $
                             outstr=outstr, ierr=ierr)

;
return, data_structure
;
end ; uio_dataset_rd
