function uio_datasetlist_rd, FileName, $
  ListData=ListData, $
  BlackList=BlackList, ReadMetaData=ReadMetaData, HeadRead=HeadRead, $
  Date=Date, $
  outstr=outstr, ierr=ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_datasetlist_rd ('uio_dataset_list_read')
;
; PURPOSE:
;   Read data from list of uio files and put it into anonymous structure.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   data_structure=uio_datasetlist_rd(FileName, ListData=ListData, $
;                    BlackList=BlackList, $
;                    ReadMetaData=ReadMetaData, HeadRead=HeadRead, $
;                    Date=Date, $
;                    outstr=outstr, ierr=ierr)
;
; INPUT:
;   FileName:     ('File_Name') string, name of input file
;
; OUTPUT:
;
; KEYWORDS:
;   ListData:     ('List_Data') structure with temporary data, empty at first call
;   BlackList:    ('Black_List') vector of strings with identifiers of quantities NOT to be read
;   ReadMetaData: ('Read_Meta_Data') input integer (0 or 1) to indicate if meta data (unit
;                 and name) is to be read and included as entry__unit and entry__name
;   HeadRead:     ('Head_Read') input flag to indicate that the file header
;                 should be read and put into separate structure
;   outstr:       ('output_string') output string
;   ierr:         ('integer_error') output integer
;
; ROUTINES:
;   Several uio routines are called.
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   15. 8.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;    8. 7.03 (B.F.) Keyword BlackList added
;    1. 7.05 (B.F. LA) Structure ListData gets 'type' tag.
;   10.10.09 (B.F. Lyon) Keyword Date added; some names put into CamelCase
;   2018-06-07 (B.F. Uppsala) Add and pass keyword HeadRead, change default to not read
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
outstr='done'
ierr=0
;
; --- Initialize uio if necessary ---
if (n_elements(intena) eq 0) then begin
  uio_init, outstr0,ierr0
endif
;
; --- Initialize list data if necessary ---
if (n_elements(ListData) eq 0) then begin
  filelist=findfile(FileName, count=filecount)
  ListData=create_struct('type'        , 'uio', $
                         'file'        , [filelist], $
                         'total_number', fix(filecount), $
                         'open_number' ,  -1, $
                         'channel'     , 0 )
  if (filecount eq 0) then begin
    ; --- Error: no files in list ---
    ierr=3
    outstr='List of files "' + FileName + '" is empty'
    goto, errorendlabel
  endif
endif
;
;help,/str,ListData
;
; --- Open first file ---
if (ListData.open_number lt 0) then begin
  ListData.open_number=0
  file=ListData.file(ListData.open_number)
  print, 'Open first file ' + file
  uio_openrd, channel, file, outstr0, ierr0
  ListData.channel=channel
  print, 'Open first file ' + file + ' on channel ', ListData.channel
  ;help, outstr0, ierr0
  if (ierr0 gt 0) then begin
    ; --- Error ---
    print, 'uio_datasetlist_rd: error while opening ', file
    uio_closrd, ListData.channel, outstr0,ierr0
    outstr='error during opening of first file'
    ierr=1
    goto, errorendlabel
  endif
endif
;
; --- Flags set to zero ---
eol_flag=0   ; --- Becomes true if end of list is reached ---
read_flag=0  ; --- Becomes true if structure is read ---
;
while ((not eol_flag) and (not read_flag)) do begin
  file=ListData.file(ListData.open_number)
  ; --- Try to read data from open file ---
  data_structure=''
  data_structure=uio_dataset_rd(file, $
                                channel=ListData.channel, $
                                BlackList=BlackList, ReadMetaData=ReadMetaData, $
                                Date=Date, HeadRead=HeadRead, $
                                outstr=outstr0, ierr=ierr0)
  read_flag=(ierr0 eq 0)
;help, read_flag
  ;
  if (not read_flag) then begin
    ; --- Data has not been found: try to open next file ---
    ;
    uio_closrd, ListData.channel, outstr0,ierr0
    ; --- Check if end of list is already reached ---
    eol_flag=(ListData.open_number ge (ListData.total_number-1))
    if (not eol_flag) then begin
      ; --- End of list is not yet reached ---
      ListData.open_number=ListData.open_number+1
      file=ListData.file(ListData.open_number)
      print, 'Open next file ' + file
      uio_openrd, channel, ListData.file(ListData.open_number), outstr0, ierr0
      ListData.channel=channel
      if (ierr0 gt 0) then begin
        ; --- Error ---
        print, 'uio_datasetlist_rd: error while opening ', file
        uio_closrd, ListData.channel, outstr0,ierr0
        outstr='error during opening of file'
        ierr=1
        goto, errorendlabel
      endif
    endif else begin
      ; --- End of list has been reached
      outstr='end of file list has been reached'
      ierr=3
      goto, errorendlabel
    endelse
  endif
endwhile
;
goto, endlabel
;
errorendlabel:
data_structure={empty:''}
;
endlabel:
;
return, data_structure
end ; uio_datasetlist_rd
