pro uio_deform, nbytin,nbytre,nbytco,nbytdb, $
                cbytin,cbytre,cbytco,cbytdb, $
                defoin,defore,defoco,defodb
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_deform ('default_format')
;
; PURPOSE:
;   Determine the default output format for
;   integers and real, complex, and double precision numbers.
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_deform, nbytin,nbytre,nbytco,nbytdb, $
;               cbytin,cbytre,cbytco,cbytdb, $
;               defoin,defore,defoco,defodb
;
; INPUT:
;    None
;
; OUTPUT:
;   nbytin: 'number_byte_integer', integer, number of bytes needed to store
;           an integer
;   nbytre: 'number_byte_real', integer, number of bytes needed to store
;           a real value
;   nbytco: 'number_byte_complex', integer, number of bytes needed to store
;           a complex value = 2*nbytre
;   nbytdb: 'number_byte_double', integer, number of bytes needed to store
;           a double precision value
;   cbytin: 'character_byte_integer', character, number of bytes needed to store
;           an integer
;   cbytre: 'character_byte_real', character, number of bytes needed to store
;           a real value
;   cbytco: 'character_byte_complex', character, number of bytes needed to store
;           a complex value = 2*nbytre
;   cbytdb: 'character_byte_double', character, number of bytes needed to store
;           a double precision value
;   defoin: 'default_format_integer', character, default format string for
;           output of an integer
;   defore: 'default_format_real', character, default format string for
;           output of a real value
;   defoco: 'default_format_complex', character, default format string for
;           output of a complex value
;   defodb: 'default_format_double', character, default format string for
;           output of a double precision value
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   machar - Standard ILD 4.0 function, determines machine-specific parameters
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   The routine relies on the results of machar.
;   Assumption: number of bytes to store an integer and a real are the same.
;   The formats are slightly too small to store all bits of information.
;
; PROCEDURE:
;   Determine from information provided by routine machar reasonable default
;   output formats for integers, real, double precision real and complex
;   numbers.
;
; EXAMPLE:
;   uio_deform, nbytin,nbytre,nbytco,nbytdb, $
;               cbytin,cbytre,cbytco,cbytdb, $
;               defoin,defore,defoco,defodb
;   help, nbytin,nbytre,nbytco,nbytdb, $
;         cbytin,cbytre,cbytco,cbytdb, $
;         defoin,defore,defoco,defodb
;     --->
;   NBYTIN          INT       =        4
;   NBYTRE          INT       =        4
;   NBYTCO          INT       =        8
;   NBYTDB          INT       =        8
;   CBYTIN          STRING    = '4'
;   CBYTRE          STRING    = '4'
;   CBYTCO          STRING    = '8'
;   CBYTDB          STRING    = '8'
;   DEFOIN          STRING    = 'I11'
;   DEFORE          STRING    = 'E13.6'
;   DEFOCO          STRING    = ''(',E13.6,',',E13.6,')''
;   DEFODB          STRING    = 'E23.15'
;
; MODIFICATION HISTORY:
;   17. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   24. 9.96 (B.F.) One digit more for real (single, double, complex) variables
;   14.10.96 (B.F.) More comments, complex numbers again with brackets
;-
;------------------------------------------------------------------------------
;
; --- Determine machine dependent parameters ---
r=machar()
d=machar(/double)
;
; --- Real numbers ---
; --- Number of bytes needed to store a real number ---
nbytre=fix((r.it+r.iexp)/8)
cbytre=strtrim(string(nbytre),2)
; --- Default output format for a real number ---
nmant=fix(      -alog10(r.eps ))
nexp =fix(alog10(alog10(r.xmax)))+1
;defore='E' + strtrim(string(nmant+nexp+4),2) + '.' + $
;             strtrim(string(nmant),2)
defore='E' + strtrim(string(nmant+nexp+5),2) + '.' + $
             strtrim(string(nmant),2)
;
; --- Double precision real numbers ---
; --- Number of bytes needed to store a double precision real number ---
nbytdb=fix((d.it+d.iexp)/8)
cbytdb=strtrim(string(nbytdb),2)
; --- Default output format for a precision real real number ---
nmant=fix(      -alog10(d.eps ))
nexp =fix(alog10(alog10(d.xmax)))+1
;defodb='E' + strtrim(string(nmant+nexp+4),2) + '.' + $
;             strtrim(string(nmant),2)
defodb='E' + strtrim(string(nmant+nexp+5),2) + '.' + $
             strtrim(string(nmant),2)
;
; --- Integers ---
; --- Number of bytes needed to store an integer (equal to real numbers?) ---
nbytin=nbytre
cbytin=strtrim(string(nbytin),2)
; --- Default output format for an integer ---
defoin='I'+strtrim(string( ((fix(nbytin*alog10(256.0))+2) > 1) < 78),2)
;
; --- Complex numbers ---
; --- Number of bytes needed to store a complex number (2*real number) ---
nbytco=2*nbytre
cbytco=strtrim(string(nbytco),2)
; --- Default output format for a complex number ---
;defoco='(' + defore + ',' + defore + ')'
defoco="'('," + defore + ",','," + defore + ",')'"
;defoco='X,' + defore + ',' + defore
;
; --- Print results ---
;help, /str, r, d
;help, nbytin,nbytre,nbytco,nbytdb, $
;      cbytin,cbytre,cbytco,cbytdb, $
;      defoin,defore,defoco,defodb
;
end ; uio_deform
