pro uio_filcon, nc, identlist,poslist, outstr,ierr, names=names
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_filcon ('uio_file_content')
;
; PURPOSE:
;   Determine file content: list of all entries with its positions.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   uio_filcon, nc, identlist,poslist, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; OUTPUT:
;   identlist: ('identifier_list') string array with types and identifiers
;           (and names) of all entries
;   poslist:('position_list') integer array with all entry positions
;           corresponding to identlist
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   names:  ('names') if set, put names together with types and identifiers
;           into array identlist
;
; ROUTINES:
;   uio_chpos:  Give current file position or jump to specified position
;   uio_rdhd: Read header of variable and extract keywords
;   uio_skipda: Skip data block
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Read file on unit nc. Return with file position at eof.
;
; RESTRICTIONS:
;   The routine is only useful for uio-files.
;   Before first call of uio_filcon, initialize uio-routines with
;     uio_init
;   
; PROCEDURE:
;   Read all entries one by one and save its type, identifier and position.
;   Start from current position. Return with file position at eof.
;
; EXAMPLE:
;   uio_openrd, nc,'uiotestfile.txt', outstr,ierr
;   uio_filcon, nc, identlist,poslist, outstr,ierr
;   uio_closrd, nc, outstr,ierr
;   print, identlist
;   print, poslist
;
; MODIFICATION HISTORY:
;   29.10.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.11.96 (B.F.) Keyword names added
;   25.11.96 (B.F.) nentry=nentry+1 (position of eof)
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
; --- Make empty arrays ---
nentrymax=1000L
if (keyword_set(names)) then begin
  identlist=strarr(3,nentrymax)
endif else begin
  identlist=strarr(2,nentrymax)
endelse
poslist=lonarr(nentrymax)
nentry=0L
;
nextlabel:
; --- Determine and save current file position ---
uio_chpos, nc,'get',pos, outstr,ierr
poslist(nentry)=pos
; --- Read entry header ---
uio_rdhd, nc, termt,ntt, outstr,ierr
if (outstr eq 'eof') then begin
  ; --- End of file reached ---
  identlist(0,nentry)='eof'
  identlist(1:*,nentry)=''
  nentry=nentry+1
  outstr='done'
  ierr=0
  goto, eoflabel
endif else begin
  ; --- Save entry type and identifier ---
  identlist(0,nentry)=termt(0,0)
  identlist(1,nentry)=termt(0,1)
  if (keyword_set(names)) then begin
    uio_exkeyw, 0, termt,ntt, namena,name, outstr,ierr
    identlist(2,nentry)=name
  endif
  ; --- Skip data block ---
  uio_skipda, nc, termt,ntt, outstr,ierr
  nentry=nentry+1
  if (nentry eq nentrymax) then begin
    ; --- Increase array sizes ---
    nentrymax_old=nentrymax
    identlist_old=temporary(identlist)
    poslist_old=temporary(poslist)
    nentrymax=2L*nentrymax
    if (keyword_set(names)) then begin
      identlist=strarr(3,nentrymax)
    endif else begin
      identlist=strarr(2,nentrymax)
    endelse
    poslist=lonarr(nentrymax)
    identlist(*,0:nentrymax_old-1)=temporary(identlist_old)
    poslist(0:nentrymax_old-1)=temporary(poslist_old)
  endif
  goto, nextlabel
endelse
;
eoflabel:
; --- Shrink lists to proper sizes ---
identlist=identlist(*,0:nentry-1)
poslist=poslist(0:nentry-1)
;
end ; uio_filcon
