pro uio_getenv, system=system, machin=machin, osrels=osrels, osvers=osvers, $
                hrdwar=hrdwar, progrm=progrm, langua=langua, $
                outstr=outstr, ierr=ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_getenv ('uio_get_environment')
;
; PURPOSE:
;   Get information about environment.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_getenv, system,machin,osrels,osvers,hrdwar,progrm, outstr,ierr
;
; INPUT:
;   None
;
; OUTPUT:
;
; KEYWORDS: (output)
;   system: ('system') character, operating system      (e.g. SunOS, craSH    )
;   machin: ('machine') character, empty or machine name(e.g. venus, craSH    )
;   osrels: ('operating_system_relaese') character, release of operating system
;                                                       (e.g. 4.1.3, 8.0.3.0  )
;   osvers: ('operating system version') character, version of operating system
;                                                       (e.g. 1,     awa.5    )
;   hrdwar: ('hardware') character, empty or machine hardware
;                                                       (e.g. sun4m, CRAY Y-MP)
;   progrm: ('program_name') character, name of main program
;   langua: ('language') character, language: 'IDL' or 'WAVE'
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; ROUTINES:
;   None
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   uio_envinf: information about environment
;
; SIDE EFFECTS:
;   During the first call on UNIX systems a job is spawned. It creates
;   a file 'uioenvfile.txt'. It is read by IDL. The result is put into the
;   common block uio_envinf.
;
; RESTRICTIONS:
;   All information is received only on UNIX systems.
;
; PROCEDURE:
;   Start shell script (which executes 'uname') and then
;   look into file 'uioenvfile.txt' to get information about environment.
;
; EXAMPLE:
;   uio_init, prog='interactive', outstr,ierr
;   ; ...
;   uio_getenv, system,machin,osrels,osvers,hrdwar,progrm, outstr,ierr
;   help, system,machin,osrels,osvers,hrdwar,progrm, outstr,ierr
;
; MODIFICATION HISTORY:
;    8. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;   11.10.96 (B.F.) Use system variable !VERSION, spawn only on UNIX systems
;   11. 1.97 (B.F.) Parameter date removed
;    1. 2.97 (B.F.) Parameters made optional, parameter langua added
;    1.10.97 (B.F.) Different test if 'IDL' or 'WAVE'
;-
;------------------------------------------------------------------------------
;
common uio_envinf, syste0, machi0, osrel0, osver0, hrdwa0, progr0, langu0
;
if (n_elements(syste0) eq 0) then begin
  syste0=!VERSION.OS
  machi0=!VERSION.ARCH
  osrel0=''
  osver0=''
  hrdwa0=!VERSION.ARCH
  ;
  if (n_elements(!X.TICKV) ne 150) then begin
    langu0='IDL'
    os_f=!VERSION.OS_FAMILY
  endif else begin
    langu0='WAVE'
    os_f='unix'
  endelse
  ;
  if (os_f eq 'unix') then begin
    ; --- System call: UNIX command 'uname' to get information about environment
    spawn, 'uname -s >  uioenvfile.txt ;'+ $
           'uname -n >> uioenvfile.txt ;'+ $
           'uname -r >> uioenvfile.txt ;'+ $
           'uname -v >> uioenvfile.txt ;'+ $
           'uname -m >> uioenvfile.txt'     , result
  endif
  ; --- Get free unit number and open file ---
  on_ioerror, label_close
  get_lun, nc
  openr, nc, 'uioenvfile.txt'
  readf, nc, syste0
  readf, nc, machi0
  readf, nc, osrel0
  readf, nc, osver0
  readf, nc, hrdwa0
  label_close:
  on_ioerror, null
  close, nc
  free_lun,nc
endif
;
system=syste0
machin=machi0
osrels=osrel0
osvers=osver0
hrdwar=hrdwa0
progrm=progr0
langua=langu0
;
outstr='done'
ierr=0
;
end ; uio_getenv
