pro uio_init, progrm=progrm, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_init ('uio_initialize_universal_input_output')
;
; PURPOSE:
;   Initialization procedure for uio input/output routine package.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_init, progrm, outstr,ierr
;
; INPUT:
;   None
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   progrm: ('program_name') character, name of main program, input
;
; ROUTINES:
;   uio_chconv: Actualize list of conversion types for all channels
;   uio_chunit: Initialize, store and actualize a list of free and occupied unit
;               numbers
;   uio_getenv:
;   uio_mkcvls: Make list with possible conversion types
;   uio_tab0:   Create empty table structure
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiotabinc: empty table
;
; COMMON BLOCKS:
;   uio_envinf:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   The common block envinf is changed. Subroutines change the contents of
;   the common blocks chainf and cvlist.
;
; RESTRICTIONS:
;   Don't use the reserved channel units (10 to 80) without uio_init and
;   uio_chunit.
;
; PROCEDURE:
;   Initialize names of variable types, keywords, ... in common block
;   uio_defnam.
;   Create list of conversion types and default formats by calling uio_mkcvls.
;   Initialize list of channel unit numbers for input/output (here 20-90).
;   Prepare channel -1 and 0 for standard output and input, respectively.
;   Create empty table structure in standard size (slow process).
;
; EXAMPLE:
;   uio_init, progrm='IDL-test', outstr,ierr
;
; MODIFICATION HISTORY:
;    8. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;   28. 9.96 (B.F.) Definition of tab_org
;   30. 9.96 (B.F.) progrm becomes keyword
;    7.10.96 (B.F.) ita -> nlin
;   14.10.96 (B.F.) chconv called with empty strings, more comments
;   11. 1.97 (B.F.) common block uio_envinf without date
;-
;------------------------------------------------------------------------------
;
; --- Include common blocks ---
@uio_uionaminc
@uio_uiotabinc
;
common uio_envinf, syste0, machi0, osrel0, osver0, hrdwa0, progr0, langu0
;
; --- Save program name in common block ---
if (keyword_set(progrm)) then begin
  progr0=progrm
endif else begin
  progr0=''
endelse
;
; --- Initialize information about environment ---
uio_getenv, langua=langua
;
; --- Save program name in common block ---
if (not keyword_set(progrm)) then progr0=langua
;
; --- Make 'universal_input_output_definitions'
versio='0.0.2018.06.07'    ; uio version
fifona='fileform'          ; type name for uio file format
uiona ='uio'               ; identifier for uio file format
formna='form'              ; keyword name for file format
convna='convert'           ; keyword name for conversion type
versna='version'           ; keyword name for uio version
datena='date'              ; keyword name for date (and time)
systna='system'            ; keyword name for system name
machna='machine'           ; keyword name for machine name
osrena='osrelease'         ; keyword name for release of operating system     
osvena='osversion'         ; keyword name for version of operating system
hrdwna='hardware'          ; keyword name for hardware description
langna='language'          ; keyword name for source language
progna='program'           ; keyword name for name of main program
lnkchr='&'                 ; link character ('&')
equchr='='                 ; character between keyword and its value ('=')
qmachr="'"                 ; quotation mark ("'")
realna='real'              ; type name of real variables
intena='integer'           ; type name of integer variables
charna='character'         ; type name of character variables
compna='complex'           ; type name of complex variables
tabna ='table'             ; type name of tables
lablna='label'             ; type name of labels
bytena='b'                 ; keyword name for number of bytes
dimna ='d'                 ; keyword name for dimensions
frmtna='f'                 ; keyword name for format string
namena='n'                 ; keyword name for name string
plinna='p'                 ; keyword name for number of values per line
tranna='t'                 ; keyword name for transformation procedure
unitna='u'                 ; keyword name for units
;
; commna='c0...9'          ; Proposed comment name
; kindna='k'               ; Proposed name for kind of variable
; idlina='idli'            ; Proposed name for IDL identifier expression
; idlnna='idln'            ; Proposed name for IDL name expression
; idluna='idlu'            ; Proposed name for IDL unit expression
; texina='texi'            ; Proposed name for LaTeX identifier expression
; texnna='texn'            ; Proposed name for LaTeX name expression
; texuna='texu'            ; Proposed name for LaTeX unit expression
;
; --- Set default write formats for numbers, create conversion list ---
uio_mkcvls, outstr, ierr
;
; --- Initialize list of free unit numbers: reserve units 20 to 90 ---
uio_chunit, 'init',     20,90, outstr,ierr
;
; --- Prepare channels 0/-1 as formatted standard input/output ---
uio_chunit, 'setrange', -1,  0, outstr,ierr
uio_chunit, 'set',       0,  0, outstr,ierr
uio_chunit, 'formatted', 0,  0, outstr,ierr
uio_chconv, '', ''     , 0, ouconv, outstr,ierr
uio_chunit, 'set',      -1, -1, outstr,ierr 
uio_chunit, 'formatted',-1, -1, outstr,ierr
uio_chconv, '', ''     ,-1, ouconv, outstr,ierr
;
; --- Construct empty table structure ---
uio_tab0, tab_org
;
outstr='done'
ierr=0
;
end ; uio_init
