pro uio_memtrm, termt,ntt, linet,nltls, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_memtrm ('uio_merge_multiple_term')
;
; PURPOSE:
;   Merge a list of terms into a line table.
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_memtrm, termt,ntt, linet,nltls, outstr,ierr
;
; INPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; INPUT/OUTPUT:
;   linet:  ('line_table') character array, nltls (<nltmx) lines
;   nltls:  ('number_line_table_last') integer, number of non-empty lines
;           of linet
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_me1trm: Merge the input term: 'keyword', 'value' -> 'keyword=value'
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Concatenate keywords and their values (if any) in termt 
;   and append them to line table linet.
;
; EXAMPLE:
;   linet:  |real                                                       |  (1)
;     +                  (1)                           (2)
;   termt:  |rho                          |                             |  (1)
;           |name                         |density                      |  (2)
;           |unit                         |g/cm**3                      |  (3)
;           |c1                           |This is a very long comment. |  (4)
;    --->
;   linet:  |real rho name=density unit=g/cm**3 &                       |  (1)
;           |c1='This is a very long comment.'                          |  (2)
;
; MODIFICATION HISTORY:
;   10. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;   26. 9.96 (B.F.) Check if ntt is defined
;   14.10.96 (B.F.) Renaming: linet, more comments
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc.pro
@uio_uiosizinc.pro
;
ierr=0
outstr='done'
lenmax=lel-3
;
if (nltls le 0) then begin
  ; --- Create output line table ---
  nltls=1
  linet=strarr(nltmx)
endif else if (nltls lt nltmx) then begin
  ; --- Add lines to output line table ---
  linet=[linet,replicate('',nltmx-nltls)]
endif
;
n=0
if (n_elements(ntt) eq 0) then begin
  ntt0=0
endif else begin
  ntt0=ntt
endelse
while ((n lt ntt0) and (nltls le nltmx)) do begin
  ;--- Merge next keyword-value-pair in term table ---
  uio_me1trm, termt(*,n), ouline, outstr,ierr
  lenoul=strlen(ouline)
  lentab=strlen(linet(nltls-1))
  ; --- Does new string fit in old line? ---
  if ((lenoul+lentab) gt lenmax) then begin
    ; --- New string does not fit in old line, start new one ---
    if (nltls lt nltmx) then begin
      ; --- Add link character to old line, start new line ---
      linet(nltls-1)=linet(nltls-1)+' '+lnkchr
      nltls=nltls+1
      linet(nltls-1)=ouline
    endif else begin
      ; --- Line table is already full ---
      outstr='table is full'
      ierr=1
    endelse
  endif else if (lentab eq 0) then begin
    ; --- New string at beginning of empty line ---
    linet(nltls-1)=ouline
  endif else begin
    ; --- Append new string on old line with single blank ---
    linet(nltls-1)=linet(nltls-1)+' '+ouline
  endelse
  n=n+1
endwhile
;
linet=linet(0:nltls-1)
;
end ; uio_memtrm
