pro uio_mkcvls, outstr, ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_mkcvls ('uio_make_convert_list')
;
; PURPOSE:
;   Make list with possible conversion types.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   uio_mkcvls, outstr, ierr
;
; INPUT:
;    None
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   deform - Determine the default output format for numbers
;
; INCLUDES:
;   uio_uiocvlinc: all convert type and default format for current machine
;
; COMMON BLOCKS:
;   uio_cflist in uio_uiocvlinc
;
; SIDE EFFECTS:
;   Change parameters in common blocks.
;
; RESTRICTIONS:
;   This routine is machine (and ILD version) dependent. The list of
;   conversion types may vary. Only conversion type 'native' is allowed on all
;   machines. But its data representation may differ from machine to machine.
;   Thus conversion type 'ieee_4' is recommended for unformatted output.
;   Current conversion types:
;   'native'
;   'ieee_4'
;   'ieee'
;   'idl'
;   'xdr'
;   
; PROCEDURE:
;   To determine numbers of bytes to store variable and good formats for
;   current machine (conversion type 'native') call uio_deform.
;   Put results for all types into common block uio_cflist.
;
; EXAMPLE:
;   uio_mkcvls, outstr, ierr
;
; MODIFICATION HISTORY:
;   17. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   29. 9.96 (B.F.) Conversion types: 'idl', 'xdr'
;   15.10.96 (B.F.) Default conversion type ncvdef=3 ('ieee_4')
;   16.10.96 (B.F.) List order changed, ieee -> ieee_4
;    1. 1.97 (B.F.) 'X' -> '1X'
;   10. 1.97 (B.F.) ieee_4, ieee
;   17. 4.01 (B.F.) ieeele_4
;-
;------------------------------------------------------------------------------
;
@uio_uiocvlinc
;
ncvmax=6
cvnals=strarr(ncvmax)
btinls=strarr(ncvmax)
btrels=strarr(ncvmax)
btcols=strarr(ncvmax)
btdbls=strarr(ncvmax)
foinls=strarr(ncvmax)
forels=strarr(ncvmax)
focols=strarr(ncvmax)
fodbls=strarr(ncvmax)
;
; ---------******
n=0
cvnals(n)='native'
uio_deform, nbytin,nbytre,nbytco,nbytdb, $
            cbytin,cbytre,cbytco,cbytdb, $
            defoin,defore,defoco,defodb
btinls(n)=cbytin
btrels(n)=cbytre
btcols(n)=cbytco
btdbls(n)=cbytdb
foinls(n)=defoin
forels(n)=defore
focols(n)=defoco
fodbls(n)=defodb
;
; ---------******
n=n+1
cvnals(n)='ieee_4'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)="'(',E13.6,',',E13.6,')'"
fodbls(n)='E23.15'
;
ncvdef=n ; Default conversion type: 'ieee_4'
;
; ---------********
n=n+1
cvnals(n)='ieeele_4'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)="'(',E13.6,',',E13.6,')'"
fodbls(n)='E23.15'
;
; ---------****
n=n+1
cvnals(n)='ieee'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)="'(',E13.6,',',E13.6,')'"
fodbls(n)='E23.15'
;
; ---------***
n=n+1
cvnals(n)='idl'
btinls(n)=btinls(0)
btrels(n)=btrels(0)
btcols(n)=btcols(0)
btdbls(n)=btdbls(0)
foinls(n)=foinls(0)
forels(n)=forels(0)
focols(n)=focols(0)
fodbls(n)=fodbls(0)
;
; ---------***
n=n+1
cvnals(n)='xdr'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)="'(',E13.6,',',E13.6,')'"
fodbls(n)='E23.15'
;
ncv=n+1
;
print, 'List of conversion types:'
for n=0,ncv-1 do begin
  if (n eq ncvdef) then def=' default' else def =''
  print, n, cvnals(n), btrels(n), forels(n), def, $
         format='(I2,1X,A12,1X,A,1X,A,A)'
endfor
;
outstr='done'
ierr=0
;
end ; uio_mkcvls
