pro uio_openrd, nc,file, outstr,ierr, $
  format=form, conversion=conv, date=date, program=progrm, $
  system=system, machine=machin, osrelease=osrels, osversion=osvers, $
  hardware=hrdwar, language=langua, $
  termt=termt,ntt=ntt
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_openrd ('open_read')
;
; PURPOSE:
;   Open a file for reading, read header.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_openrd, nc,file, outstr,ierr, $
;     format=form, conversion=conv, date=date, program=progrm, $
;     system=system, machine=machin, osrelease=osrels, osversion=osvers, $
;     hardware=hrdwar, language=langua, $
;     termt=termt,ntt=ntt
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   file:   ('file_name') character, filename
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   (Output:)
;   format: ('format') character, 'formatted' or 'unformatted'
;   conversion: ('conversion') character, conversion type
;   date:   ('date') character, empty or date information
;   program:('program_name') character, name of main program
;   system: ('system') character, operating system
;                                                        (e.g. SunOS, craSH    )
;   machine:('machine') character, empty or machine name
;                                                    (e.g. venus, kueste, craSH)
;   osreleases: ('operating_system_release') character, operating system release
;                                                        (e.g. 4.1.3, 8.0.3.0  )
;   osversion: ('operating_system_version') character, operating system version
;                                                        (e.g. 1,     awa.5    )
;   hardware: ('hardware') character, empty or machine hardware
;                                                      (e.g. sun4m, 'CRAY Y-MP')
;   language: ('language') character, language of program that wrote file
;                                                      (e.g. 'FORTRAN77', 'IDL')
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; ROUTINES:
;   uio_chconv: Actualize list of conversion types for all channels
;   uio_chunit: Initialize, store and actualize a list of free and occupied unit
;               numbers
;   uio_rdfifo: Read file header
;   uio_rdhd: Read header of variable and extract keywords
;   uio_uclose: Close file with special handling for conversion type
;   uio_uopen:  Open file with special handling for conversion type
;
; INCLUDES:
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   A file is opened. Its status is stored.
;
; RESTRICTIONS:
;
; PROCEDURE:
;  Try to identify format and conversion type of file from first line.
;   Open file with correct format ('formatted' or 'unformatted'). Save
;   conversion type in common block.
;
; EXAMPLE:
;   ; --- Open file, read first entry and print its header, close file again --
;   filename='uiotestfile.txt'
;   uio_openrd, nc,filename, outstr,ierr
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;   uio_pptrmt, termt,ntt
;   uio_closrd, nc, outstr,ierr
;
; MODIFICATION HISTORY:
;   21. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   22. 9.96 (B.F.) corrections
;   15.10.96 (B.F.) uio_rdfifo: parameters -> keywords
;   16.10.96 (B.F.) Parameters -> keywords
;   31.10.96 (B.F.) Unknown format -> no crash, but ierr=1
;   20. 2.97 (B.F.) A formatted file also has a conversion type
;   28. 7.03 (B.F.) Print statements outcommented
;   2009-03-18 (B.F. Lyon) Bugfix: add argument to call of uio_uopen
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocvlinc
;
recog=0
line=''
;
; --- Get channel unit ---
uio_chunit, 'get',nc,0, outstr,ierr
;
; --- Open file 'formatted' and read first line ---
on_ioerror, label10
uio_uopen, nc,file,'formatted','','old', 'read', outstr,ierr
readf, nc, line, format='(A)'
goto, label20
;
; --- File definitely not 'formatted' ---
label10:
recog=-1
;
label20:
uio_uclose, nc, outstr,ierr
;help, line
;help, recog
;
if (recog ge 0) then begin
  ; --- File possibly 'formatted' ---
  ; --- Test if there is a readable format description in line ---
  if (strpos(line,formna+equchr+'formatted') ne -1) then begin
    ; --- File definitely 'formatted' ---
    recog=1
    form='formatted'
    outstr='known format'
  endif
endif
;help, recog
;
if (recog ne 1) then begin
  ; --- File format not recognized yet ---
  ; --- Loop over all conversion types ---
  for n=0,ncv-1 do begin
;    help, n
    recog=0
    ;  --- Open file 'unformatted' and read first line ---
    on_ioerror, label30
    uio_uopen, nc,file,'unformatted',cvnals(n),'old', 'read', outstr,ierr
    line=string(' ',format='(A'+string(lel)+')')
    readu, nc, line
    goto, label40
    ;
    ; --- File format not recognized yet ---
    label30:
    recog=-1
    ;
    label40:
    uio_uclose, nc, outstr,ierr
;    help,/files
    ; --- Test if there is format description in line ---
    if (recog ge 0) then begin
      ; --- File format possibly 'unformatted' with current conversion type ---
      if (strpos(line,formna+equchr+'unformatted') ne -1) then begin
        ; --- File is indeed 'unformatted' ---
        recog=1
        form='unformatted'
        outstr='known format'
        goto, label90
      endif
    endif
    ; --- Unknown format ---
    form='unknown'
    outstr='unknown format'
    ierr=1
  endfor
endif
;
;help,form,outstr,ierr,recog
if (recog ne 1) then goto, endlabel
;
label90:
;
; --- Find and save conversion type ---
;if (form eq 'unformatted') then begin
  ; --- File is 'unformatted' but conversion type is still unknown ---
  ;print,'File is ' + form + ' but conversion type is still unknown'
  uio_chconv, line,convna+equchr, nc,conv, outstr,ierr
  ;print, 'uio_chconv: ', outstr, ierr
  if (ierr gt 0) then begin
    outstr='Conversion type not found'
    if (form eq 'unformatted') then goto, endlabel
  endif
;endif else begin
;  ; --- A 'formatted' file has no conversion type ---
;  uio_chconv, '','', nc, conv, outstr,ierr
;endelse
;
; --- Open file on given channel with specified format and conversion type ---
uio_uopen, nc,file,form,conv,'old', 'read', outstr,ierr
;
; --- Save form of file ---
uio_chunit, form, nc, 0, outstr,ierr
;
; --- Read file header completely ---
uio_rdhd, nc, termt,ntt, outstr,ierr
;form=''
;conv=''
;date=''
;progrm=''
;system=''
;machin=''
;osrels=''
;osvers=''
;hrdwar=''
;langua=''
uio_rdfifo, nc,termt,ntt, outstr,ierr, $
  format=form, conversion=conv, date=date, program=progrm, $
  system=system, machine=machin, osrelease=osrels, osversion=osvers, $
  hardware=hrdwar, language=langua
;
outstr='done'
ierr=0
;
endlabel:
;
end ; uio_openrd
