pro uio_openwr, nc,file, outstr,ierr, form=form0, conv=conv0, $
    position=position0, $
    termt=termt, ntt=ntt, progrm=progrm
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_openwr ('uio_open_write')
;
; PURPOSE:
;   Open a file for writing, write header.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_openwr, nc,file, outstr,ierr, form=form, conv=conv, position=position, $
;               termt=termt, ntt=ntt, progrm=progrm
;
; INPUT:
;   file:    ('file_name') character, filename
;
; OUTPUT:
;   nc:      ('number_channel') integer, channel unit number
;   outstr:  ('output_string') character
;   ierr:    ('integer_error') integer, contains exit error status
;             0: correct operation
;            >0: occurance of an error
;
; KEYWORDS:
;   form:    ('file_format') character, 'formatted' or 'unformatted',
;            default: 'formatted'
;   conv:    ('convert') character, type of conversion procedure,
;            default: '' ('formatted': no conversion, 'unformatted': take default
;            conversion type)
;   position:('position') character, 'asis' (default) or 'append'
;   termt:   ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:     ('number_term_table') integer, number of non-empty lines of termt
;            (ntt >= 0)
;   progrm:  ('program') character, name of calling program
;
; ROUTINES:
;   uio_chconv: Actualize list of conversion types for all channels
;   uio_chunit: Initialize, store and actualize a list of free and occupied unit
;               numbers
;   uio_uopen:  Open a file with special handling for conversion type
;   uio_wrfifo: Write file header
;
; INCLUDES:
;   uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   A new file is created and opened for writing. The format and
;   conversion type are stored in common blocks. The file position counter is
;   set to zero.
;
; RESTRICTIONS:
;   An old file is overwritten and its contents is deleted.
;
; PROCEDURE:
;   Save form of file and conversion type, open file and write file header.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   30. 9.96 (B.F.) Keyword progrm added
;   13.10.96 (B.F.) Renaming: termt
;   14.10.96 (B.F.) termt, ntt become keywords
;   16.10.96 (B.F.) termt, ntt become keywords in call of uio_wrfifo,
;                   form, conv become keywords
;   24. 7.03 (B.F.) Keyword position added
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
; --- Default keyword values ---
if (not keyword_set(form0)) then begin
  form='formatted'
endif else if (form0 eq 'unformatted') then begin
  form='unformatted'
endif else begin
  form='formatted'
endelse
if (not keyword_set(conv0)) then begin
  conv=''
endif else begin
  conv=conv0
endelse
if (not keyword_set(position0)) then begin
  rdrw='write'
endif else begin
  rdrw=position0
endelse
;
; --- Get channel unit ---
uio_chunit, 'get',nc,0, outstr,ierr
;
; --- Save form of file ---
uio_chunit, form,nc,0, outstr,ierr
;
; --- Save conversion type ---
uio_chconv, conv,'', nc, ouconv, outstr,ierr
;
; --- Open file on given channel with specified format ---
uio_uopen, nc,file,form,ouconv,'unknown',rdrw, outstr,ierr
;
; --- Write file header ---
if (n_elements(progrm) eq 0) then progrm=''
if (n_elements(ntt) eq 0) then ntt=n_elements(termt)/2
uio_wrfifo, nc, outstr,ierr, progrm=progrm, termt=termt,ntt=ntt
;
end ; uio_openwr
