pro uio_qmadel, inline, ouline,remain, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_qmadel ('uio_quotation_mark_delete')
;
; PURPOSE:
;   Parse string 'inline' and transform its first term
;   with or without limiting quotation marks into a string 'ouline' without.
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_qmadel, inline, ouline,remain, outstr,ierr
;
; INPUT:
;   inline: ('input_line') character
;
; OUTPUT:
;   ouline: ('output_line') character
;   remain: ('remainder') character
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Parse string 'inline' and transform its first term
;   with or without limiting quotation marks into a string 'ouline' without.
;   The first term is marked by a limiter:
;     The end of the string.
;     A blank ' ' for a string without leading quotation mark.
;     The second quotation mark for a string with leading quotation mark.
;  The rest of inline (if there is any) is given out in remain.
;
; EXAMPLE:
;    inline:                   ouline            remain
;    'I''m tired.'     --->    I'm tired.
;    'I''m tired.'Oh!  --->    I'm tired.        Oh!
;    I'm               --->    I'm
;    I'm tired.        --->    I'm               tired.
;     I                --->                      I
;
; MODIFICATION HISTORY:
;    9. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;   15.10.96 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
ierr=0
;
if (strtrim(inline) eq '') then begin
  ; --- String is empty ---
  ouline=''
  remain=''
  outstr='empty'
endif else if (strmid(inline,0,1) ne qmachr) then begin
  ; --- String without quotation marks ---
  ind=strpos(inline,' ')
  if (ind le -1) then begin
    ; --- No blanks, copy line ---
    ouline=inline
    remain=''
  endif else begin
    ; --- Extract all characters up to blank ---
    ouline=strmid(inline,0,ind)
    remain=strmid(inline,ind+1,80)
  endelse
  outstr='done'
endif else begin
  ; --- String with quotation marks ---
  if (strlen(inline) eq 1) then begin
    ; --- No following letters ---
    ouline=''
    remain=''
    outstr='only one '+qmachr
  endif else begin
    ; --- Transform string with quotation marks ---
    iin=1
    iout=0
    ouline=''
    leninl=strlen(inline)
    remain=''
    outstr='done'
    while (iin lt leninl) do begin
      if (strmid(inline,iin,1) eq qmachr) then begin
        ; --- Quotation mark found ---
        if (iin lt leninl-1) then begin
          ; --- End of input string not yet reached ---
          iin=iin+1
          if (strmid(inline,iin,1) eq qmachr) then begin
            ; --- Two consecutive quotation marks ---
            ouline=ouline+qmachr
          endif else begin
            ; --- Limiting second quotation mark found ---
            remain=strmid(inline,iin,leninl)
            outstr='done'
            iin=leninl
          endelse
        endif else begin
          ; --- End of input string reached ---
          remain=''
          outstr='done'
        endelse
      endif else begin
	; --- Copy character ---
        ouline=ouline+strmid(inline,iin,1)
      endelse
      ; --- Icrement counters ---
      iin=iin+1
      iout=iout+1
    endwhile
  endelse
endelse
;
end ; uio_qmadel
