pro uio_rd, nc, termt,ntt, val,ident, iarr=iarr, outstr,ierr, $
    mode=mode, nbytes=nbytes, $
    format=format, name=name, unit=unit, tran=tran, date=date
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_rd ('uio_read')
;
; PURPOSE:
;   Read scalar or array data of all types.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_rd, nc, termt,ntt, mode=mode, val,ident, iarr=iarr, outstr,ierr, $
;           nbytes=nbytes, $
;           format=format, name=name, unit=unit, tran=tran, date=date
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; OUTPUT:
;   val:    ('value') real, array to be written
;   ident:  ('identifier') character, identifier (mandatory)
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; KEYWORDS:
;   mode:   ('mode') integer, read store mode, (to be implemented))
;   iarr:   ('index_array') integer array(4,number_of_dimensions), output
;   nbytes: ('number_bytes') integer, number of characters of variable val,
;           output
;   format: ('format') character, FORTRAN style format string, output
;   name:   ('name') character, name of variable, output
;   unit:   ('unit') character, string with units, output
;   tran:   ('transformation') character, transformation, output
;   date:   ('date') character, date string, output
;
; ROUTINES:
;   chunit: Initialize, store and actualize a list of unit numbers
;   exkeyw: Extract value of keyword from table
;   st2dim: Parse dimension string
;   wf2rf:  Produce from write format string corresponding read format string
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data are read from file on unit nc.
;
; RESTRICTIONS:
;   Differences to FORTRAN77 version:
;     iarr is output, data block can't be written into subarray of val.
;     mode is not used
;
; PROCEDURE:
;   Parse header, determine type(, extract dimensions), and read data (array).
;
; EXAMPLE:
;   ; --- Open file, read first entry and print it, close file again --
;   filename='uiotestfile.txt'
;   uio_openrd, nc,filename, outstr,ierr
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;   uio_rd, nc, termt,ntt, val,ident, outstr,ierr
;   print, val
;   uio_closrd, nc, outstr,ierr
;
; MODIFICATION HISTORY:
;   24. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   25. 9.96 (B.F.)
;   29. 9.96 (B.F.) Prepare character array of proper size and length
;    2.10.96 (B.F.) ncmin may become negative, cvnum(.) -> cvnum(.-ncmin)
;    9.10.96 (B.F.) Character: remove following blanks, correct string length
;   10.10.96 (B.F.) Free input format ('' or '*')
;   12.10.96 (B.F.) Routine uio_wf2rf included
;   15.10.96 (B.F.) More comments
;    8.12.98 (B.F., Copenhagen) Force array to have the exact number of
;                   dimensions
;   2010-03-15 (B.F. Naples) Use "reform" with keyword "overwrite"
;   2013-12-09 (B.F. & S.W.) Merged version
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocvlinc
@uio_uiocstinc
;
; --- Find keywords and extract their values ---
type =termt[0,0]
; --- Check type ---
if ((type ne intena) and (type ne realna) and (type ne compna) and $
    (type ne charna)) then begin
  ierr=1
  outstr='No data (array), but ' + type
  goto, endlabel
endif
ident=termt[0,1]
termt[0,0:1]=''
uio_exkeyw, 1,termt,ntt, dimna ,dimstr, outstr,ierr
m=(n_elements(format) ne 0)
uio_exkeyw, m,termt,ntt, frmtna,format, outstr,ierr
uio_exkeyw, 1,termt,ntt, plinna,perlin, outstr,ierr
if ((format eq '') or (format eq '*')) then begin
  rfrmt=''
endif else begin
  rfrmt='(' + perlin + '(' + format + '))'
endelse
uio_wf2rf, rfrmt, rfrmt, outstr, ierr, qmachr=qmachr
m=(n_elements(name) ne 0)
uio_exkeyw, m,termt,ntt, namena,name  , outstr,ierr
m=(n_elements(unit) ne 0)
uio_exkeyw, m,termt,ntt, unitna,unit  , outstr,ierr
m=(n_elements(tran) ne 0)
uio_exkeyw, m,termt,ntt, tranna,tran  , outstr,ierr
m=(n_elements(date) ne 0)
uio_exkeyw, m,termt,ntt, datena,date  , outstr,ierr
uio_exkeyw, 3,termt,ntt, bytena,bytes , outstr,ierr
;reads, bytes, nbytes, format='(I6)'
nbytes=fix(bytes)
;
; --- Get format of input file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
; --- Index range (iarr) ... ---
if (dimstr ne '') then begin
  uio_st2dim, dimstr, iarr
  dim=reform(iarr[1,*]-iarr[0,*])+1
  val_dim=n_elements(dim)
endif else begin
  val_dim=0
endelse
;
; --- Make empty data variable with correct type ---
; --- size(0): dimension
; --- size(size(0)+1): 0:undefined,   1:byte,    2:integer, 3:long, 4:real,
; ---                  5:double real, 6:complex, 7:string,  8:structure,
; ---                  9:double complex
if (type eq intena) then begin
  if (bytes eq btinls(cvnum(nc-ncmin))) then begin
    val_type=3
  endif else if (bytes eq '2') then begin
    val_type=2
  endif else if (bytes eq '1') then begin
    val_type=1
  endif else if (form eq 'formatted') then begin
    val_type=3
  endif else begin
    message, 'Integer data type not known'
  endelse
endif else if (type eq realna) then begin
  if (bytes eq btrels(cvnum(nc-ncmin))) then begin
    val_type=4
  endif else if (bytes eq btdbls(cvnum(nc-ncmin))) then begin
    val_type=5
  endif else if (form eq 'formatted') then begin
    val_type=4
  endif else begin
    message, 'Real data type not known'
  endelse
endif else if (type eq compna) then begin
  if (bytes eq btcols(cvnum(nc-ncmin))) then begin
    val_type=6
  endif else if (bytes eq 2*btcols(cvnum(nc-ncmin))) then begin
    val_type=9
  endif else if (form eq 'formatted') then begin
    val_type=6
  endif else begin
    message, 'Complex data type not known'
  endelse
endif else if (type eq charna) then begin
  val_type=7
endif else begin
  message, 'Unknown data type'
endelse
;
; --- Create scalar (or array) of proper type (and size) ---
if (val_dim gt 0) then begin
  val_size=lonarr(val_dim+3)
  val_size[0]=val_dim
  val_size[1:val_dim]=dim
  val_size[val_dim+1]=val_type
  val=make_array(size=val_size)
  if (val_type eq 7) then val[*]=string(' ',format='(A'+bytes+')')
  if (n_elements(size(val)) lt n_elements(val_size)) then begin
    ; --- Force array to have the exact number of dimensions ---
    val=reform(val, dim, /overwrite)
  endif
endif else begin
       if (val_type eq 1) then val=0B $
  else if (val_type eq 2) then val=0 $
  else if (val_type eq 3) then val=0L $
  else if (val_type eq 4) then val=0.0 $
  else if (val_type eq 5) then val=0.0D0 $
  else if (val_type eq 6) then val=complex(0) $
  else if (val_type eq 7) then val=string(' ',format='(A'+bytes+')') $
  else if (val_type eq 9) then val=dcomplex(0)
endelse
;
; --- Read data from file ---
if (form eq 'unformatted') then begin
  readu, nc, val
endif else begin
  readf, nc, val, format=rfrmt
endelse
;
; --- Remove trailing blanks ---
if (val_type eq 7) then val=strtrim(val)
;
; --- Transformation ---
if (tran eq 'log10') then val=10.0^val
;
outstr='done'
ierr=0
;
endlabel:
;
end ; uio_rd
