pro uio_rdhdln, nc,form,linet,nltls, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_rdhdln ('rd_head_line')
;
; PURPOSE:
;   Read header of variable.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_rdhdln, nc,form,linet,nltls, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   form:   ('format') character, file format: 'formatted' or 'unformatted'
;
; OUTPUT:
;   linet:  ('line_table') character array, nltmx lines
;   nlt:    ('number_line_table') integer, number of non-empty lines of linet
;   outstr: ('output_string') character, EOF -> 'eof'
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt, linet)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data are read from file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Read file line by line until end of header or end of file is reached.
;
; EXAMPLE:
;   ; --- Open file, read first entry and print its header, close file again --
;   filename='uiotestfile.txt'
;   uio_openrd, nc,filename, outstr,ierr
;   uio_rdhdln, nc, linet,nlt, outstr,ierr
;   print, linet(0:nlt-1)
;   uio_closrd, nc, outstr,ierr
;
; MODIFICATION HISTORY:
;   16. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;    9.10.96 (B.F.) Allow empty lines in header (treat them as '&'),
;            don't load lines with only '&'
;   14.10.96 (B.F.) Renaming: linet
;   15.10.96 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
ierr=0
;
nltls=1
linet=strarr(nltmx)
;
; --- Read line according to current file format ---
nextline:
line=string(' ',format='(A'+string(lel)+')')
if (form eq 'unformatted') then begin
  if (eof(nc)) then goto, eoflabel
  readu, nc, line
endif else begin
  if (eof(nc)) then goto, eoflabel
  readf, nc, line, format='(A)'
endelse
; --- Delete leading blanks ---
line=strtrim(line,2)
linet(nltls-1)=line
; --- Is there a following header line? ---
if (line eq '') then begin
  ; --- Empty line ---
  goto, nextline
endif else if (line eq lnkchr) then begin
  ; --- Line with only one link character ---
  goto, nextline
endif else begin
  idxlin=strlen(line)
  if (idxlin eq 1) then begin
    outstr='Only one letter'
  endif else if (strmid(line,idxlin-2,2) eq ' '+lnkchr) then begin
    ; --- Line with link character at end ---
    nltls=nltls+1
    goto, nextline
  endif else begin
    outstr='end'
  endelse
endelse
goto, endlabel
;
; --- End of file reached ---
eoflabel:
ierr=0
outstr='eof'
;
endlabel:
;
end ; uio_rdhdln
