pro uio_rdtab, nc, termt,ntt, tab,ident, outstr,ierr, $
    format=format, name=name, date=date, oldtable=oldtable
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_rdtab ('uio_read_table')
;
; PURPOSE:
;   Read table of integer, real, and/or character data from file.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_rdtab, nc, termt,ntt, tab,ident, outstr,ierr, $
;              format=format, name=name, date=date
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   tab:    ('table') structure, complete table to be read from file
;   ident:  ('identifier') character, identifier
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   format: ('format') character, FORTRAN style format string, output
;   name:   ('name') character, name of variable, output
;   date:   ('date') character, date string, output
;   oldtable: ('old_table') if set use input table if possible
;
; ROUTINES:
;   chunit: Initialize, store and actualize a list of free and occupied unit
;           numbers
;   exkeyw: Extract value of keyword from table
;   getenv: Get information about environment
;   rdhd:   Read next header and extract keywords
;   st2dim: Parse dimension string
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;   uio_uiotabinc: empty table
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;   uio_taborg in uio_uiotabinc
;
; SIDE EFFECTS:
;   Data are read from file on unit nc.
;
; RESTRICTIONS:
;   The table in the file has to be small enough to fit into the default table
;   size.
;
; PROCEDURE:
;   Extract keywords from table header and read headers for all table entries.
;   Skip single table header line. Compose input format from entry keywords.
;   Determine length of input strings (if necessary). Find loop indices for
;   integer, real and character subtables. Read table data from file.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   27. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   28. 9.96 (B.F.) New common block in uio_uiotabinc
;   30. 9.96 (B.F.) Keyword bytes added; pad computed to skip header line
;    7.10.96 (B.F.) ita -> nlin
;    9.10.96 (B.F.) Table header as first table column, read with strtrim(),
;            check type
;   10.10.96 (B.F.) Allow table-format='*'
;   11.10.96 (B.F.) Allow table-format including strings
;   12.10.96 (B.F.) Routine uio_wf2rf included
;   16.10.96 (B.F.) Corrections for empty table
;   18.10.96 (B.F.) Keyword oldtable added, choose size of tab dynamically
;   30.10.96 (B.F.) empty entry format allowed, corrections
;    1. 1.97 (B.F.) 'X' -> '1X'
;    2. 2.97 (B.F.) Version for 'IDL' and 'WAVE'
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocvlinc
@uio_uiocstinc
@uio_uiotabinc
;
; --- Check type ---
if (termt(0,0) ne tabna) then begin
  ierr=1
  outstr='No table, but ' + termt(0,0)
  goto, endlabel
endif
;
; --- Name of language ('IDL' or 'WAVE') ---
uio_getenv, langua=langua
;
time=dblarr(20) & timecom=strarr(20) & ntime=0
time(ntime)=systime(1) & timecom(ntime)='table header' & ntime=ntime+1
; --- Save termt in temporary variable ---
termt_tmp=termt
ntt_tmp=ntt
;
; --- Find keywords and extract their values ---
type =termt(0,0)
ident=termt(0,1)
termt(0,0:1)=''
uio_exkeyw, 1,termt,ntt, dimna ,dimstr, outstr,ierr
uio_st2dim, dimstr, iarr
ncol=iarr(1,0)-iarr(0,0)+2
nlin=iarr(1,1)-iarr(0,1)+1
m=(n_elements(format) ne 0)
uio_exkeyw, m,termt,ntt, frmtna,format, outstr,ierr
uio_exkeyw, 1,termt,ntt, bytena,bytes , outstr,ierr
if (bytes eq '') then bytes='0'
;reads, bytes, nbytes, format='(I6)'
nbytes=fix(bytes)
m=(n_elements(name) ne 0)
uio_exkeyw, m,termt,ntt, namena,name  , outstr,ierr
m=(n_elements(date) ne 0)*3
uio_exkeyw, m,termt,ntt, datena,date  , outstr,ierr
;
time(ntime)=systime(1) & timecom(ntime)='get form' & ntime=ntime+1
; --- Get format of input file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
time(ntime)=systime(1) & timecom(ntime)='tab_tmp' & ntime=ntime+1
; --- Create temporary arrays for all header entries ---
tabtermt_tmp=strarr(2,nttmx,ncol)
tabntta_tmp=lonarr(ncol)
; --- Put table header into header field for first column ---
tabtermt_tmp(*,*,0)=termt_tmp
tabntta_tmp(0)=ntt_tmp
;
time(ntime)=systime(1) & timecom(ntime)='entry header' & ntime=ntime+1
; --- Read header data for all table entries from file ---
for i=1,ncol-1 do begin
  uio_rdhd, nc, termt_tmp,ntt_tmp, outstr,ierr
  tabtermt_tmp(*,0:ntt_tmp-1,i)=termt_tmp(*,0:ntt_tmp-1)
  tabntta_tmp(i)=ntt_tmp
endfor
;
time(ntime)=systime(1) & timecom(ntime)='tfrmt' & ntime=ntime+1
; --- Compose input format ---
ntai=0
ntar=0
ntas=0
if (format eq ',') then begin
  sfrmt=format
endif else if (format eq '*') then begin
  sfrmt=',1X,'
endif else begin
  if (nbytes eq 0) then begin
    sfrmt=','
  endif else begin
    sfrmt=',' + strtrim(string(nbytes,format='(I6)'),2) + 'X,'
  endelse
endelse
for i=1,ncol-1 do begin
  uio_exkeyw, 0, tabtermt_tmp(*,*,i),tabntta_tmp(i), frmtna,efrmt,outstr,ierr
  uio_wf2rf, efrmt, efrmt, outstr,ierr, qmachr=qmachr
  type=tabtermt_tmp(0,0,i)
  if (type eq intena) then begin
    if (efrmt eq '') then begin
      nebytes=2
    endif else begin
      str=string(0L, format='('+efrmt+',''<'')')
      nebytes=strlen(str)-1
    endelse
    ntai=ntai+1
  endif else if (type eq realna) then begin
    if (efrmt eq '') then begin
      nebytes=2
    endif else begin
      str=string(0.0, format='('+efrmt+',''<'')')
      nebytes=strlen(str)-1
    endelse
    ntar=ntar+1
  endif else if (type eq charna) then begin
    if (efrmt eq '') then begin
      nebytes=2
    endif else begin
      str=string(' ', format='('+efrmt+',''<'')')
      nebytes=strlen(str)-1
    endelse
    ntas=ntas+1
  endif
  if (i eq 1) then begin
    bytes2skip=nebytes
    tfrmt='(' + efrmt
  endif else begin
    bytes2skip=bytes2skip+nebytes+nbytes
    tfrmt=tfrmt + sfrmt + efrmt
  endelse
endfor ; i
if (bytes2skip eq 0) then begin
  pad=''
endif else begin
  pad=string(' ',format='(A'+strtrim(string(bytes2skip,format='(I6)'),2)+')')
endelse
tfrmt=tfrmt + ')'
if ((langua eq 'WAVE') and (nlin gt 1)) then begin
  ;tfrmt=tfrmt + ',/)' ; 2.2.97
  ;tfrmt='(28(' +tfrmt+ '/),' +tfrmt+ ')'   ; !!!!!!!!!!!!
  tfrmt='(' + strtrim(string(nlin-1,format='(I11)'),2) + '(' + tfrmt + '/),' + $
         tfrmt +')'
endif
if (format eq '*') then tfrmt=''
;
time(ntime)=systime(1) & timecom(ntime)='skip header' & ntime=ntime+1
; --- Skip single table header line ---
if (form eq 'unformatted') then begin
  readu, nc, pad
endif else begin
  readf, nc, format='(A1)'
endelse
;
time(ntime)=systime(1) & timecom(ntime)='make tab0' & ntime=ntime+1
if (keyword_set(oldtable)) then begin
  ; --- Maximum size of input table ---
  nlinmx=tab.nlinmx
  ntaimx=tab.ntaimx
  ntarmx=tab.ntarmx
  ntasmx=tab.ntasmx
  ncolmx=tab.ncolmx
endif else begin
  ; --- Maximum size of default table ---
  uio_tab0, tab_dum, outstr, ierr, nlinmx=nlinmx, $
    ntaimx=ntaimx, ntarmx=ntarmx, ntasmx=ntasmx, ncolmx=ncolmx, /default
endelse
if ((nlin le nlinmx) and (ntai le ntaimx) and (ntar le ntarmx) and $
    (ntas le ntasmx) and (ncol le ncolmx)) then begin
  if (keyword_set(oldtab)) then begin
    ; --- Delete contents of old table ---
    uio_tabc, tab, outstr,ierr, mode='rearrange', index=[-1]
  endif else begin
    ; --- Copy standard table ---
    tab=tab_org
  endelse
endif else begin
  ; --- Create new table with proper size ---
  uio_tab0, tab, outstr, ierr, nlinmx=(nlinmx>nlin), $
    ntaimx=(ntaimx>ntai), ntarmx=(ntarmx>ntar), ntasmx=(ntasmx>ntas), $
    ncolmx=(ncolmx>ncol)
endelse
;
time(ntime)=systime(1) & timecom(ntime)='tab.=tab_tmp' & ntime=ntime+1
; --- Put temporary arrays into table ---
tab.termt(*,*,0:ncol-1)=temporary(tabtermt_tmp)
tab.ntta(0:ncol-1)=temporary(tabntta_tmp)
tab.nlin=nlin
tab.ntai=ntai
tab.ntar=ntar
tab.ntas=ntas
tab.ncol=ncol
;
if (tab.nlin gt 0) then begin
  time(ntime)=systime(1) & timecom(ntime)='make lineti' & ntime=ntime+1
  ; --- Make empty input structure intab ---
  nts=0
  ;
  if (langua eq 'IDL') then begin
    execute_str="intabline=create_struct("
    for i=1,tab.ncol-1 do begin
      type=tab.termt(0,0,i)
      istr=strtrim(string(i),2)
      if (type eq intena) then begin
        execute_str=execute_str + "'i" + istr + "',0L"
      endif else if (type eq realna) then begin
        execute_str=execute_str + "'r" + istr + "',0.0"
      endif else if (type eq charna) then begin
        uio_exkeyw, 0, tab.termt(*,*,i),tab.ntta(i), bytena,bytes, outstr,ierr
        execute_str=execute_str + "'s" + istr + "','" + $
                                  string(' ',format='(A'+bytes+')') + "'"
        nts=nts+1
      endif
      if (i lt (tab.ncol-1)) then execute_str=execute_str + ","
    endfor
    execute_str=execute_str + ")"
  endif else begin ; --- 'WAVE' ---
    execute_str="intabline={,"
    for i=1,tab.ncol-1 do begin
      type=tab.termt(0,0,i)
      istr=strtrim(string(i),2)
      if (type eq intena) then begin
        execute_str=execute_str + "i" + istr + ":0L"
      endif else if (type eq realna) then begin
        execute_str=execute_str + "r" + istr + ":0.0"
      endif else if (type eq charna) then begin
        uio_exkeyw, 0, tab.termt(*,*,i),tab.ntta(i), bytena,bytes, outstr,ierr
        execute_str=execute_str + "s" + istr + ":'" + $
                                  string(' ',format='(A'+bytes+')') + "'"
        nts=nts+1
      endif
      if (i lt (tab.ncol-1)) then execute_str=execute_str + ","
    endfor
    execute_str=execute_str + "}"
  endelse
  ;
  print,execute_str
  result=execute(execute_str)
  ;
  time(ntime)=systime(1) & timecom(ntime)='make lineti()' & ntime=ntime+1
  intab=make_array(tab.nlin, value=intabline)
  ;
  time(ntime)=systime(1) & timecom(ntime)='read lineti()' & ntime=ntime+1
  ; --- Read table from file ---
  if (form eq 'unformatted') then begin
    readu, nc, intab
  endif else begin
;tfrmt='(28(' +tfrmt+ '/),' +tfrmt+ ')'   ; !!!!!!!!!!!!
    readf, nc, intab, format=tfrmt
  endelse
  ;
  time(ntime)=systime(1) & timecom(ntime)='tab=lineti' & ntime=ntime+1
  ; --- Put data from input/output structure into table ---
  nti=0
  ntr=0
  nts=0
  for i=1,tab.ncol-1 do begin
    type=tab.termt(0,0,i)
    istr=strtrim(string(i),2)
    if (type eq intena) then begin
      tab.i(0:tab.nlin-1,nti)=intab(*).(i-1)
      nti=nti+1
    endif else if (type eq realna) then begin
      tab.r(0:tab.nlin-1,ntr)=intab(*).(i-1)
      ntr=ntr+1
    endif else if (type eq charna) then begin
      tab.s(0:tab.nlin-1,nts)=strtrim(intab(*).(i-1))
      nts=nts+1
    endif
  endfor
endif
;
time(ntime)=systime(1) & timecom(ntime)='' & ntime=ntime+1
outstr='done'
ierr=0
;
for n=0,ntime-2 do begin
  print,time(n+1)-time(n), format='(F6.3,1X,A)', timecom(n)
endfor
print,time(ntime-1)-time(0), format='(F6.3,1X,A)', '--- Total ---'
;
endlabel:
;
end ; uio_rdtab
