pro uio_skipda, nc, termt,ntt, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_skipda ('uio_skip_data')
;
; PURPOSE:
;   Skip data block
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_skipda, nc, termt,ntt, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number (actually not used)
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_chunit: Initialize, store and actualize a list of unit numbers
;   uio_exkeyw: Extract value of keyword from table
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data in file on unit nc are skipped.
;
; RESTRICTIONS:
;   Works for all data types implemented so far, not for every possible.
;
; PROCEDURE:
;   Analyze header and skip following data lines or data block.
;
; EXAMPLE:
;   ; --- Open file, read first entry, print its header, skip it, ... ---
;   filename='uiotestfile.txt'
;   uio_openrd, nc,filename, outstr,ierr
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;   uio_pptrmt, termt,ntt
;   uio_skipda, nc, termt,ntt, outstr,ierr
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;   ; ...   
;   uio_closrd, nc, outstr,ierr
;
; MODIFICATION HISTORY:
;   23. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   25. 9.96 (B.F.) Number of skipped lines for multi-dimensional arrays
;   30. 9.96 (B.F.) Skip table
;    2.10.96 (B.F.) ncmin may become negative, cvnum(.) -> cvnum(.-ncmin)
;   13.10.96 (B.F.) Renaming: linet
;   15.10.96 (B.F.) More comments
;   16.10.96 (B.F.) Corrections for empty table
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocstinc
@uio_uiocvlinc
;
; --- Find keywords and extract their values ---
type=termt(0,0)
ident=termt(0,1)
;termt(0,0:1)=''
uio_exkeyw, 0,termt,ntt,frmtna, format, outstr,ierr
uio_exkeyw, 0,termt,ntt,bytena, cbyt,   outstr,ierr
uio_exkeyw, 0,termt,ntt,dimna , dimstr, outstr,ierr
;
; --- Get format of input file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
; --- Read data from file: skip ---
if (type eq fifona) then begin
  ; --- fileform: no data ---
  ;
endif else if (type eq lablna) then begin
  ; --- label: no data ---
  ;
endif else if (type eq tabna) then begin
  ; --- Table ---
  if ((form eq 'unformatted') and ((cvnals(cvnum(nc-ncmin)) eq 'native') or $
                                   (cvnals(cvnum(nc-ncmin)) eq 'idl') or $
                                   (cvnals(cvnum(nc-ncmin)) eq 'xdr'))) then begin
    ; --- Read table ---
    uio_rdtab, nc, termt,ntt, tab,ident, outstr,ierr
  endif else begin
    ; --- Skip table ---
    uio_st2dim, dimstr, iarr
    ; --- Skip header of entries ---
    for i=iarr(0,0),iarr(1,0) do begin
      uio_rdhdln, nc,form,linet,nlt, outstr,ierr
    endfor ; i
    ; --- Skip single table header line ---
    if (form eq 'unformatted') then begin
      readu, nc
    endif else begin
      readf, nc, format='(A1)'
    endelse
    ; --- Skip table data ---
    if (form eq 'unformatted') then begin
      if (iarr(1,1) ge iarr(0,1)) then readu, nc
    endif else begin
      for i=iarr(0,1),iarr(1,1) do begin
        readf, nc, format='(A1)'
      endfor ; i
    endelse
  endelse
endif else begin
  ; --- Integer, real, double precision real, complex, or character data ---
  if ((form eq 'unformatted') and ((cvnals(cvnum(nc-ncmin)) eq 'native') or $
                                   (cvnals(cvnum(nc-ncmin)) eq 'idl') or $
                                   (cvnals(cvnum(nc-ncmin)) eq 'xdr'))) then begin
    ; --- Read array ---
    uio_rd, nc, termt,ntt, val,ident, outstr,ierr
  endif else begin
    ; --- Skip array ---
    if (dimstr eq '') then begin
      ; --- Scalar data ---
      if (form eq 'unformatted') then begin
        readu, nc
      endif else begin
        readf, nc, format='(A1)'
      endelse
    endif else begin
      ; --- Array data ---
      if (form eq 'unformatted') then begin
        readu, nc
      endif else begin
        uio_st2dim, dimstr, iarr
        uio_exkeyw, 0,termt,ntt, plinna ,perlin, outstr,ierr
        ;reads, perlin, npl, format='(I6)'
        ;npl=fix(npl)
        npl=fix(perlin)
        nskip=1L
        for i=0,n_elements(iarr)/2-1 do nskip=nskip*(iarr(1,i)-iarr(0,i)+1L)
        nskip=(nskip-1)/npl+1
        for i=1,nskip do begin
          readf, nc, format='(A1)'
        endfor ; i
      endelse
    endelse
  endelse
endelse
;
termt(0,0)=type
termt(0,1)=ident
;
ierr=0
outstr='done'
;
end ; uio_skipda
