function uio_struct_rd, FileName, Channel=Channel, $
  HeadRead=HeadRead, $
  StartLabel=StartLabel, EndLabel=EndLabel, nLabel=nLabel, $
  SubStartLabel=SubStartLabel, SubEndLabel=SubEndLabel, $
  BlackList=BlackList, ReadMetaData=ReadMetaData, $
  Date=Date, $
  outstr=outstr, ierr=ierr
;----------------------------------------------------------------------------------------------------
;+
; NAME:
;   uio_struct_rd ('uio_structure_read')
;
; PURPOSE:
;   Read uio file and put data into anonymous structure
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   data_structure=uio_struct_rd(FileName, Channel=Channel, $
;                    HeadRead=HeadRead, $
;                    StartLabel=StartLabel, EndLabel=EndLabel, nLabel=nLabel, $
;                    SubStartLabel=SubStartLabel, SubEndLabel=SubEndLabel, $
;                    BlackList=BlackList, ReadMetaData=ReadMetaData, $
;                    Date=Date, $
;                    outstr=outstr, ierr=ierr)
;
; INPUT:
;   FileName:      ('File_Name') string, name of input file
;
; OUTPUT:
;
; KEYWORDS:
;   Channel:       ('Channel') input integer, if this keyword is set it is
;                  assumed that a file is already opened at this channel unit.
;   HeadRead:      ('Head_Read') input flag to indicate that the file header
;                  should be read and put into separate structure
;   StartLabel:    ('start_label') input string, name of label marking the
;                  beginning of a structure (e.g. 'dataset')
;   EndLabel:      ('End_Label') input string, name of label marking the
;                  end of a structure (e.g. 'enddataset')
;   nLabel:        ('number_Label') input integer, occurrence of structure to be read:
;                    0: read first occurrence of structure marked e.g. by
;                       'label dataset'/'label enddataset' pair
;                    n: read n+1st occurrence of structure
;   SubStartLabel: ('Sub_Start_Label') input string, name of label marking the
;                  beginning of a sub structure (e.g. 'box')
;   SubEndLabel:   ('Sub_End_Label') input string, name of label marking the
;                  end of a sub structure (e.g. 'endbox')
;   BlackList:     ('Black_List') vector of strings with identifiers of quantities NOT to be read
;   ReadMetaData:  ('Read_Meta_Data') input integer (0 or 1) to indicate if meta data (unit
;                  and name) is to be read and included as entry__unit and entry__name
;   outstr:        ('output_string') output string
;   ierr:          ('integer_error') output integer
;
; ROUTINES:
;   Several uio routines are called.
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;   uio_taborg in uio_uiotabinc
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   This routine is made to map a complete unstructured UIO file (as e.g. a
;   CO5BOLD parameter file) or one dataset of a structured UIO file (as e.g. a
;   CO5BOLD data file) inta an IDL structure.

;   One dataset of a structured UIO file with a contents of the form
;
;     fileform uio form=unformatted convert=ieee_4
; 
;     character file_id f=A8 b=8 n='File identification'
;     character description d=(1:1) f=A14 p=2 b=14 n='File description'
;     character history d=(1:20) f=A80 p=1 b=80 n='File history'
;     character version f=A80 b=80 n='Program version'
;
;     label dataset n='RHD model'
;       label box date='25.05.2001 09:41:29.983'
;         character box_id f=A2 b=2 n='Block identification'
;         z1
;         ...
;       label endbox
;       label box date='25.05.2001 09:41:30.078'
;         character box_id f=A2 b=2 n='Block identification'
;         z2
;         ...
;       label endbox
;     label enddataset
;
;     label dataset n='RHD model'
;       ...
;     label enddataset
;     .
;     .
;     .
;
;   into an IDL structure
;
;     dataset.{head,z1,z2}
;
;   where 'head' contains the header entries and
;   'z'1 and 'z2' are sub structures comprising all entries (except for the
;   'box_id' entry) between the corresponding 'box'/'endbox' label pairs.
;
;
; MODIFICATION HISTORY:
;   1999-01-06 (B.F. Copenhagen) Written
;   1999-03-26 (B.F.) Default values for ...label
;   1999-11-08 (B.F.) Look for misshaped identifiers of data entries
;                     (patch for error in the Fortran90 MEAN program)
;   2000-06-03 (B.F.) End of structure may be defined by start label of next one
;   2000-03-28 (B.F.) Special handling for multiple occurrences of header entries
;   2003-07-08 (B.F. Uppsala) Keyword BlackList added
;   2009-10-10 (B.F. Lyon) Keywords Date, ReadMetaData added; some names put into CamelCase
;   2014-09-18 (B.F. Uppsala) Consider each 'startXXX'/'endXXX' pair as marker for substructure
;-
;----------------------------------------------------------------------------------------------------

@uio_uionaminc

outstr='done'
ierr=0

; --- Initialize uio if necessary ---
if (n_elements(intena) eq 0) then begin
  uio_init, outstr0,ierr0
endif

if (n_elements(StartLabel   ) eq 0) then StartLabel   =''
if (n_elements(EndLabel     ) eq 0) then EndLabel     =''
if (n_elements(SubStartLabel) eq 0) then SubStartLabel=''
if (n_elements(SubEndLabel  ) eq 0) then SubEndLabel  =''

; --- Open file ---
if (keyword_set(Channel)) then begin
  nc=Channel
endif else begin
  uio_openrd, nc, FileName, outstr0, ierr0
  if (ierr0 gt 0) then begin
    ; --- Error ---
    print, 'uio_struct_rd: error while opening ', FileName
    if (not keyword_set(Channel)) then uio_closrd, nc, outstr0,ierr0
    data_structure=''
    outstr='error during opening of file'
    ierr=1
    goto, errorEndLabel
  endif
endelse

; --- Read header ---
if (keyword_set(HeadRead)) then begin
  ; --- Flags set to zero ---
  eof_flag=0    ; --- Becomes true if EOF is reached ---
  found_flag=0  ; --- Becomes true if label is found ---
  id_flag=0
  his_flag=0
  des_flag=0
  ver_flag=0
  head_structure=create_struct('type', 'uio')
  ; --- Search until label found or EOF reached ---
  while ((not eof_flag) and (not found_flag)) do begin
    uio_chpos, nc, 'get', position, outstr0,ierr0
    uio_rdhd, nc, termt,ntt, outstr0,ierr0
    if (outstr0 eq 'eof') then begin
      eof_flag=1
    endif else begin
      type =termt[0,0]
      if (type eq lablna) then begin
        found_flag=1
        uio_chpos, nc, 'goto', position, outstr0,ierr0
      endif else if (type eq fifona) then begin
        uio_skipda, nc, termt,ntt, outstr0,ierr0
      endif else begin
        if (type eq tabna) then begin
          uio_rdtab, nc, termt,ntt, data,ident, outstr0,ierr0
        endif else begin
          uio_rd, nc, termt,ntt, data,ident, outstr0,ierr0
        endelse
        ; print, type, ' ', ident
        ;
        if (not (((ident eq 'file_id')     and id_flag ) or $
                 ((ident eq 'description') and des_flag) or $
                 ((ident eq 'history')     and his_flag) or $
                 ((ident eq 'version')     and ver_flag))) then begin
          head_structure=create_struct(head_structure, ident, data)
        endif
        ;
        id_flag =id_flag  or (ident eq 'file_id')
        des_flag=des_flag or (ident eq 'description')
        his_flag=his_flag or (ident eq 'history')
        ver_flag=ver_flag or (ident eq 'version')
      endelse
    endelse
  endwhile
endif

; --- Search until start label with specified number is reached ---
if (keyword_set(StartLabel)) then begin
  ; --- Default value for label number ---
  if (keyword_set(nLabel)) then begin
    nLabel0=nLabel
  endif else begin
    nLabel0=0
  endelse
  ; --- End flags and counter set to zero ---
  eof_flag=0    ; --- Becomes true if EOF is reached ---
  found_flag=0  ; --- Becomes true if start label is found ---
  number_of_labels=0
  ; --- Search until label found or EOF reached ---
  while ((not eof_flag) and (not found_flag)) do begin
    uio_rdhd, nc, termt,ntt, outstr0,ierr0
    if (outstr0 eq 'eof') then begin
      eof_flag=1
    endif else begin
      type =termt(0,0)
      if (type eq lablna) then begin
        uio_rdlabl, nc, termt,ntt, ident, outstr0,ierr0
        if (ident eq StartLabel) then begin
          number_of_labels=number_of_labels+1
          if (number_of_labels gt nLabel0) then begin
            found_flag=1
            uio_exkeyw, '10', termt,ntt, datena, Date
          endif
        endif
      endif else begin
        uio_skipda, nc, termt,ntt, outstr0,ierr0
      endelse
    endelse
  endwhile
  if (eof_flag) then begin
    ; --- Error ---
    print, 'uio_struct_rd: start label not found'
    if (not keyword_set(Channel)) then uio_closrd, nc, outstr0,ierr0
    data_structure=''
    outstr='start label not found'
    ierr=2
    goto, errorEndLabel
  endif
endif

; --- Read all file data entries except labels and fileform entries ---
eof_flag=0    ; --- Becomes true if EOF is reached ---
found_flag=0  ; --- Becomes true if start label is found ---
data_structure=create_struct('type', 'uio')
if (n_elements(head_structure)) then begin
  data_structure=create_struct(data_structure, 'head', head_structure)
endif
while ((not eof_flag) and (not found_flag)) do begin
  uio_chpos, nc, 'get', position, outstr0,ierr0
  uio_rdhd, nc, termt,ntt, outstr0,ierr0
  ;
  if (outstr0 eq 'eof') then begin
    eof_flag=1
  endif else begin
    type =termt[0,0]
    if (type eq fifona) then begin
      ; --- The entry is a file header: skip it ---
      ;
      uio_skipda, nc, termt,ntt, outstr0,ierr0
    endif else if (type eq lablna) then begin
      ; --- The entry is a label: check if it indicates a sub structure ---
      ;
      uio_rdlabl, nc, termt,ntt, ident, outstr0,ierr0
      found_flag=(ident eq EndLabel)
      if (not found_flag) then begin
        ; --- If no end label is found, test if entry is new start label ---
        found_flag=(ident eq StartLabel)
        if (found_flag) then uio_chpos, nc, 'goto', position, outstr0,ierr0
      endif
      ;
      if ((not found_flag) and keyword_set(SubStartLabel)) then begin
        if (ident eq SubStartLabel) then begin
          ; --- Load box ---
          uio_chpos, nc, 'goto', position, outstr0,ierr0
          data=uio_struct_rd(FileName, Channel=nc, $
                             StartLabel=SubStartLabel, EndLabel=SubEndLabel, $
                             BlackList=BlackList, ReadMetaData=ReadMetaData)
          if ( (where(tag_names(data) eq 'BOX_ID'))[0] ne -1) then begin
            ident=data.box_id
          endif else begin
            ident='box'
          endelse
          data_structure=create_struct(data_structure, ident, data)
          ;found_flag=1
        endif
      endif
      ;
      if (not found_flag) then begin
        identPrefix=strmid(ident, 0, 5)
        TmpLabel   =strmid(ident, 5)
        if ((identPrefix eq 'start') and (TmpLabel ne '')) then begin
          ; --- Assume the label is the start of a sub structure and load it ---
          TmpStartLabel=ident
          TmpEndLabel  ='end' + TmpLabel
          uio_chpos, nc, 'goto', position, outstr0,ierr0
          data=uio_struct_rd(FileName, Channel=nc, $
                             StartLabel=TmpStartLabel, EndLabel=TmpEndLabel, $
                             BlackList=BlackList, ReadMetaData=ReadMetaData)
          if ( (where(tag_names(data) eq 'BOX_ID'))[0] ne -1) then begin
            ident=data.box_id
          endif else if ( (where(tag_names(data) eq (strupcase(TmpLabel) + '_ID')))[0] ne -1) then begin
            ident=data.TmpLabel_id
          endif else begin
            ident=TmpLabel
          endelse
          data_structure=create_struct(data_structure, ident, data)
;          found_flag=1
        endif
      endif
      ;
    endif else begin
      readit_flag=1
      if (keyword_set(BlackList)) then begin
        ident=strtrim(termt[0,1],2)
        readit_flag=(total(ident eq [BlackList]) lt 0.5)
      endif
      if (readit_flag) then begin
        ; --- Read entry ---
        if (type eq tabna) then begin
          uio_rdtab, nc, termt,ntt, data,ident, outstr0,ierr0
        endif else begin
          uio_rd, nc, termt,ntt, data,ident, outstr0,ierr0
        endelse
;print, type, ' ', ident
;if (type eq 'character') then print, '  ', data
;help,/str,data_structure
        if (strpos(ident, '-') ne -1) then begin
;        print, 'Misshaped identifier: ', type, ' ', ident
        endif else if (keyword_set(ReadMetaData)) then begin
          ; --- Read data value, unit, and value and append them to structure ---
          uio_exkeyw, '10', termt,ntt, unitna, unit
          uio_exkeyw, '10', termt,ntt, namena, name
          data_structure=create_struct(data_structure, ident           , data, $
                                                       ident + '__unit', unit, $
                                                       ident + '__name', name)
        endif else begin
          ; --- Read just data value and append it to structure ---
          data_structure=create_struct(data_structure, ident, data)
        endelse
      endif else begin
        ; --- Entry in black list: skip it ---
        uio_skipda, nc, termt,ntt, outstr0,ierr0
      endelse
    endelse
  endelse
endwhile

; --- Close file again ---
if (not keyword_set(Channel)) then uio_closrd, nc, outstr0,ierr0

goto, EndLabel

errorEndLabel:
data_structure={empty:''}

EndLabel:

return, data_structure

end ; uio_struct_rd
