pro uio_tab0, tab_org, outstr, ierr, nlinmx=nlinmx, $
  ntaimx=ntaimx, ntarmx=ntarmx, ntasmx=ntasmx, ncolmx=ncolmx, $
  default=default
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_tab0 ('uio_table_0')
;
; PURPOSE:
;   Create empty table structure.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_tab0, tab_org, outstr, ierr, nlinmx=nlinmx, $
;     ntaimx=ntaimx, ntarmx=ntarmx, ntarmx=ntasmx, ncolmx=ncolmx
;
; INPUT:
;   NONE
;
; OUTPUT:
;   tab_org:('table_original') structure, empty table
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   nlinmx: ('number_line_maximum') integer, maximum number of table lines;
;           default=1000
;   ntaimx: ('number_table_integer_maximum') integer, maximum number of columns
;            with integers; default=20
;   ntarmx: ('number_table_real_maximum') integer, maximum number of columns
;           with real values; default=70
;   ntasmx: ('number_table_string_maximum') integer, maximum number of columns
;            with character values; default=10
;   ncolmx: ('number_column_maximum') integer, maximum total column number of
;           complete table (including table header entries);
;           default=ntaimx+ntarmx+ntasmx
;   default: ('default') if set: don't create table but give out default values
;
; ROUTINES:
;   uio_getenv:
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers
;   uio_uiosizinc: length of strings, size of tables (termt, linet)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Create anonymous table structure according to keyword or default values.
;
; EXAMPLE:
;   uio_tab0, tab_org
;   help, /structure, tab_org
;     --->
;   ** Structure <39fdc0>, 16 tags, length=472772, refs=1:
;     TYPE            STRING    'table'
;     NCOLMX          LONG               101
;     NCOL            LONG                 0
;     NLINMX          LONG              1000
;     NLIN            LONG                 0
;     NTAIMX          LONG                20
;     NTAI            LONG                 0
;     NTARMX          LONG                70
;     NTAR            LONG                 0
;     NTASMX          LONG                10
;     NTAS            LONG                 0
;     I               LONG      Array(1000, 20)
;     R               FLOAT     Array(1000, 70)
;     S               STRING    Array(1000, 10)
;     NTTA            LONG      Array(101)
;     TRMTAB          STRING    Array(2, 20, 101)
;   uio_tab0, tab_org, outstr, ierr, nlinmx=nlinmx, $
;     ntaimx=ntaimx, ntarmx=ntarmx, ntasmx=ntasmx, ncolmx=ncolmx, /default
;   help, nlinmx, ntaimx, ntarmx, ntasmx, ncolmx
;     --->
;   NLINMX          LONG      =         1000
;   NTAIMX          LONG      =           20
;   NTARMX          LONG      =           70
;   NTASMX          LONG      =           10
;   NCOLMX          LONG      =          101
;
; MODIFICATION HISTORY:
;   11.10.96 Written by Bernd Freytag, Astrophysik Kiel
;   17.10.96 (B.F.) Keyword default added
;   18.10.96 (B.F.) tab.type added
;    1. 2.97 (B.F.) Structure creation in execute command: 'IDL'/'WAVE' version
;-
;------------------------------------------------------------------------------
;
; --- Include common blocks ---
@uio_uiosizinc
@uio_uionaminc
;
nlindef=1000L
ntaidef=20L
ntardef=70L
ntasdef=10L
ncoldef=ntaidef+ntardef+ntasdef+1
;
; --- Default values for keywords ---
if (keyword_set(nlinmx) and (not keyword_set(default))) then begin
  nlinmx=long(nlinmx)
endif else begin
  nlinmx=nlindef
endelse
if (keyword_set(ntaimx) and (not keyword_set(default))) then begin
  ntaimx=long(ntaimx)
endif else begin
  ntaimx=ntaidef
endelse
if (keyword_set(ntarmx) and (not keyword_set(default))) then begin
  ntarmx=long(ntarmx)
endif else begin
  ntarmx=ntardef
endelse
if (keyword_set(ntasmx) and (not keyword_set(default))) then begin
  ntasmx=long(ntasmx)
endif else begin
  ntasmx=ntasdef
endelse
if (keyword_set(ncolmx) and (not keyword_set(default))) then begin
  ncolmx=long(ncolmx)
endif else begin
  ncolmx=ntaimx+ntarmx+ntasmx+1
endelse
;
if (not keyword_set(default)) then begin
  ; --- Construct empty table structure ---
;  tab_org=create_struct( $
;          'type',tabna, $
;          'ncolmx',ncolmx, 'ncol',0L, $
;          'nlinmx',nlinmx, 'nlin',0L, $
;          'ntaimx',ntaimx, 'ntai',0L, $
;          'ntarmx',ntarmx, 'ntar',0L, $
;          'ntasmx',ntasmx, 'ntas',0L, $
;          'i',lonarr(nlinmx,ntaimx), $
;          'r',fltarr(nlinmx,ntarmx), $
;          's',strarr(nlinmx,ntasmx), $
;          'ntta',lonarr(ncolmx), $
;          'termt',strarr(2,nttmx,ncolmx))
  ;
;    execute_str="tab_org=create_struct(" + $
;      "'type',tabna," + $
;      "'ncolmx'," + string(ncolmx) + "L, 'ncol',0L," + $
;      "'nlinmx'," + string(nlinmx) + "L, 'nlin',0L," + $
;      "'ntaimx'," + string(ntaimx) + "L, 'ntai',0L," + $
;      "'ntarmx'," + string(ntarmx) + "L, 'ntar',0L," + $
;      "'ntasmx'," + string(ntasmx) + "L, 'ntas',0L," + $
;      "'i',lonarr(" + string(nlinmx) + "," + string(ntaimx) + ")," + $
;      "'r',fltarr(" + string(nlinmx) + "," + string(ntarmx) + ")," + $
;      "'s',strarr(" + string(nlinmx) + "," + string(ntasmx) + ")," + $
;      "'ntta',lonarr(" + string(ncolmx) + ")," + $
;      "'termt',strarr(2," + string(nttmx) + "," + string(ncolmx) + "))"
  ; --- Name of language ('IDL' or 'WAVE') ---
  uio_getenv, langua=langua
  ;
  if (langua eq 'IDL') then begin
    execute_str="tab_org=create_struct(" + $
          "'type',tabna," + $
          "'ncolmx',ncolmx, 'ncol',0L," + $
          "'nlinmx',nlinmx, 'nlin',0L," + $
          "'ntaimx',ntaimx, 'ntai',0L," + $
          "'ntarmx',ntarmx, 'ntar',0L," + $
          "'ntasmx',ntasmx, 'ntas',0L," + $
          "'i',lonarr(nlinmx,ntaimx)," + $
          "'r',fltarr(nlinmx,ntarmx)," + $
          "'s',strarr(nlinmx,ntasmx)," + $
          "'ntta',lonarr(ncolmx)," + $
          "'termt',strarr(2,nttmx,ncolmx))"
  endif else begin
    execute_str="tab_org={," + $
          "type:tabna," + $
          "ncolmx:ncolmx, ncol:0L," + $
          "nlinmx:nlinmx, nlin:0L," + $
          "ntaimx:ntaimx, ntai:0L," + $
          "ntarmx:ntarmx, ntar:0L," + $
          "ntasmx:ntasmx, ntas:0L," + $
          "i:lonarr(nlinmx,ntaimx)," + $
          "r:fltarr(nlinmx,ntarmx)," + $
          "s:strarr(nlinmx,ntasmx)," + $
          "ntta:lonarr(ncolmx)," + $
          "termt:strarr(2,nttmx,ncolmx)}"
  endelse
  ;
  ;print,execute_str
  result=execute(execute_str)
  ;
  outstr='table created'
  ierr=0
endif else begin
;  tab_org=''
  ;
  outstr='defaults'
  ierr=0
endelse
;
end ; uio_tab0
