pro uio_tabc, tab, outstr,ierr, mode=mode, incolumn=incolumn, index=index0
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_tabc ('uio_table_change')
;
; PURPOSE:
;   Change and modify table contents: rearrange lines.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_tabc, tab, outstr,ierr, mode=mode, incolumn=incolumn, index=index
;
; INPUT:
;   None
;
; INPUT/OUTPUT:
;   tab:    ('table') structure, complete table
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   mode:   ('mode') character, mode: 'sort', 'sortreverse', 'reverse',
;           'rearrange', 'delete', input
;   incolumn: ('input_column') integer, column for sorting, input,
;           default=last column
;   index:  ('index') integer array, indices for rearranging, input/output
;
; ROUTINES:
;   uio_nc2nt:  From column number or entry name find table entry number
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   The table tab is modified according to the mode keyword:
;   All columns of tab may be sorted according to ascending (mode='sort') or
;   descending (mode='sortreverse') of a specified column.
;   The contents of tab may be reversed (mode='reverse').
;   Selected lines may be rearranged (mode='rearrange') or
;   deleted (mode='delete').
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    2.10.96 Written by Bernd Freytag, Astrophysik Kiel
;    4.10.96 (B.F.) Don't sort table header column
;    6.10.96 (B.F.) Parameter nc removed
;    7.10.96 (B.F.) ita -> nlin
;    9.10.96 (B.F.) Produce empty table if requested
;   10.10.96 (B.F.) Renamed: uio_ct -> uio_tabc
;   15.10.96 (B.F.) More comments
;   16.10.96 (B.F.) Corrections
;   17.10.96 (B.F.) 'Sort', 'Sortreverse', 'Reverse' on index range only
;   16.11.96 (B.F.) Use sort1 instead of sort to preserve order of elements
;            if possible
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
ierr=0
;
; --- Default values for keywords ---
if (not keyword_set(mode)) then mode='rearrange'
if ((mode ne 'rearrange') and (mode ne 'sort') and (mode ne 'sortreverse') and $
    (mode ne 'reverse') and (mode ne 'delete')) then begin
  outstr='unknown table manipulation mode'
  ierr=2
  goto, endlabel
endif
;
if (n_elements(tab) eq 0) then begin
  ; --- Table is undefined ---
  outstr='empty table'
  ierr=1
  goto, endlabel
endif
;
if ((tab.nlin eq 0) or (tab.ncol eq 0)) then begin
  outstr='empty table'
  ierr=0
  goto, endlabel
endif
;
; --- Find table column ---
if (n_elements(incolumn) eq 0) then begin
  newcol=tab.ncol-1
  ntaisl=tab.ntai-1
  ntarsl=tab.ntar-1
  ntassl=tab.ntas-1
  type_tmp=tab.termt(0,0,newcol)
endif else begin
  newcol=0>incolumn<(tab.ncol-1)
  uio_nc2nt, reform(tab.termt(0,0:1,0:tab.ncol-1)), tab.ncol, newcol, $
    ntaisl, ntarsl, ntassl, outstr,ierr
  type_tmp=tab.termt(0,0,newcol)
endelse
;
; --- Basic index range ---
;print,'keyword_set(index0)',keyword_set(index0)
if (keyword_set(index0)) then begin
  subindex=where((index0 ge 0) and (index0 lt tab.nlin), n_subindex)
  if (n_subindex gt 0) then begin
    subindex=long(index0(subindex))
  endif
endif else begin
  n_subindex=tab.nlin
  subindex=lindgen(n_subindex)
endelse
n_index=tab.nlin
index=lindgen(n_index)
subindex0=subindex
;
; --- Compute or rectify index vector for rearranging ---
if (((mode eq 'sort') or (mode eq 'sortreverse')) and (n_subindex gt 0)) then $
  begin
  ; --- Sort requested column ---
  if (type_tmp eq intena) then begin
    subindex=subindex(sort1(tab.i(subindex,ntaisl)))
  endif else if (type_tmp eq realna) then begin
    subindex=subindex(sort1(tab.r(subindex,ntarsl)))
  endif else if (type_tmp eq charna) then begin
    subindex=subindex(sort1(tab.s(subindex,ntassl)))
  endif ; --- Table ---
  ; --- Reverse order if requested ---
  if (mode eq 'sortreverse') then begin
    subindex=reverse(subindex)
  endif
  index(subindex0)=subindex
endif else if ((mode eq 'reverse') and (n_subindex gt 0)) then begin
  ; --- Reverse table order ---
  subindex=reverse(subindex)
  index(subindex0)=subindex
endif else begin
  ; --- Rearrange or delete ---
  if (mode eq 'delete') then begin
    if ((n_subindex gt 0) and (keyword_set(index0))) then begin
      negindex=index
      negindex(subindex)=-1
      subindex=where(negindex ne -1, n_subindex)
      index=subindex
      n_index=n_subindex
    endif
  endif else begin ; Rearrange
    index=subindex
    n_index=n_subindex
  endelse
endelse
;
; --- Rearrange lines of complete table ---
if (n_index gt 0) then begin
  if (n_index gt tab.nlinmx) then begin
    n_index=tab.nlinmx
    index=index(0:nlinmx-1)
    outstr='done: but only '+strtrim(string(nlinmx),2)+' lines produced'
    ierr=1
  endif else begin
    outstr='done'
    ierr=0
  endelse
  if (tab.ntai gt 0) then begin
    tab.i(0:n_index-1,0:tab.ntai-1)=tab.i(index,0:tab.ntai-1)
;    if (n_index lt tab.nlin) then begin
;      tab.i(n_index:tab.nlin-1,0:tab.ntai-1)=0
;    endif
  endif
  if (tab.ntar gt 0) then begin
    tab.r(0:n_index-1,0:tab.ntar-1)=tab.r(index,0:tab.ntar-1)
;    if (n_index lt tab.nlin) then begin
;      tab.r(n_index:tab.nlin-1,0:tab.ntar-1)=0.0
;    endif
  endif
  if (tab.ntas gt 0) then begin
    tab.s(0:n_index-1,0:tab.ntas-1)=tab.s(index,0:tab.ntas-1)
;    if (n_index lt tab.nlin) then begin
;      tab.s(n_index:tab.nlin-1,0:tab.ntas-1)=''
;    endif
  endif
  ;
endif else begin
  n_index=0
  outstr='empty table produced'
  ierr=0
endelse  
;
if (n_index lt tab.nlin) then begin
  ; --- Set free table range to zero ---
  if (tab.ntai gt 0) then begin
    tab.i(n_index:tab.nlin-1,0:tab.ntai-1)=0
  endif
  if (tab.ntar gt 0) then begin
    tab.r(n_index:tab.nlin-1,0:tab.ntar-1)=0.0
  endif
  if (tab.ntas gt 0) then begin
    tab.s(n_index:tab.nlin-1,0:tab.ntas-1)=''
  endif
endif
;
tab.nlin=n_index
;
endlabel:
;
end ; uio_tabc
