pro uio_tabm, tab0, tab1, outstr,ierr, $
    tableheader=tableheader, $
    overwrite=overwrite, lineappend=lineappend, columnappend=columnappend, $
    replaceheader=replaceheader, addheader=addheader, $
    comparedata=comparedata, col0=col0, col1=col1, $
    compareheader=compareheader
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_tabm ('uio_table_merge')
;
; PURPOSE:
;   Merge two tables in different ways.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_tabm, tab0, tab1, outstr,ierr, $
;     tableheader=tableheader, $
;     overwrite=overwrite, lineappend=lineappend, columnappend=columnappend, $
;     replaceheader=replaceheader, addheader=addheader, $
;     comparedata=comparedata, col0=col0, col1=col1, $
;     compareheader=compareheader
;
; INPUT:
;   tab1:    ('table_1') structure, table
;
; INPUT/OUTPUT:
;   tab0:    ('table_0') structure, table to be modified
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   addheader: ('add_header') if set, only new header keywords from tab1 are
;              added to tab0, no keyword from tab1 is overwritten; default=0
;   columnappend: ('column_append') if set, the appending of new columns from
;                 tab1 to tab0 is allowed, otherwise forbitten; default=1
;   compareheader: ('compare_header') if set, the header type and identifier
;                  of tab0 and tab1 are compared, matching columns of tab0
;                  are overwritten (if overwrite=1); default=0
;   comparedata: ('compare_data') if set, the data of col0 in tab0 and col1 in
;                tab1 are compared. If overwrite=1 or (columnappend=1 and
;                lineappend=0) table rows of tab1 with matching entries are
;                written to tab0. If lineappend=1 non-matching lines are
;                appended.
;   col0: ('column_0') integer, column number of tab0 for data comparison
;   col1: ('column_1') integer, column number of tab1 for data comparison
;   lineappend: ('line_append') if set, the appending of new rows from
;               tab1 to tab0 is allowed, otherwise forbitten; default=1
;   overwrite: ('overwrite') if set, it is allowed to overwrite data in tab0,
;              opposite to appending; default=1
;   replaceheader: ('replace_header') if set, header keywords from tab0 are
;                  replaced with keywords from tab1 if they match and
;                  compareheader=1; default=0
;   tableheader: ('table_header') if set, the entry for the table entry or
;                entries of tab1 are copied to tab0, otherwise they are
;                ignored; default=1
;
; ROUTINES:
;   uio_exkeyw: Extract value of keyword from table
;   uio_nc2nt:  From column number or entry name find table entry number
;   uio_tabr:   Read 1d array from 2d table array (all types)
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   tab0 = tab0 '+' tab1
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    4.10.96 Written by Bernd Freytag, Astrophysik Kiel
;    6.10.96 (B.F.) Parameter nc removed
;    7.10.96 (B.F.) First working version
;    8.10.96 (B.F.) tab0 or tab1 may be empty, default values for keywords
;    9.10.96 (B.F.) Keywords replacehead, addheader added
;   10.10.96 (B.F.) Renamed: uio_mt -> uio_tabm
;   15.10.96 (B.F.) More keywords
;   16.10.96 (B.F.) Corrections for empty tables
;   10.12.09 (B.F. Lyon) Replaced some round by square brackets, indgen->lindgen, fix->long
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
; --- Check table contents ---
if ((n_elements(tab0) eq 0) and (n_elements(tab1) eq 0)) then begin
  ; --- Table is undefined ---
  outstr='empty tables'
  ierr=1
  goto, endlabel
endif
if (n_elements(tab0) eq 0) then begin
  tab0=tab1
  outstr='tab0 is empty: tab1 copied'
  ierr=0
  goto, endlabel
endif
if (n_elements(tab1) eq 0) then begin
  outstr='tab1 is empty: nothing done'
  ierr=0
  goto, endlabel
endif
;
; --- Default values for kewords ---
if (n_elements(tableheader) eq 0) then tableheader=1 $
                                  else tableheader=keyword_set(tableheader)
if (n_elements(overwrite) eq 0) then overwrite=1 $
                                else overwrite=keyword_set(overwrite)
if (n_elements(columnappend) eq 0) then columnappend=1 $
                                   else columnappend=keyword_set(columnappend)
if (n_elements(lineappend) eq 0) then lineappend=1 $
                                 else lineappend=keyword_set(lineappend)
;
; ------------------------------------
; --- Compute column index vectors ---
; ------------------------------------
;
ncolold=0
icol0old=intarr(tab0.ncolmx)
icol1old=intarr(tab1.ncolmx)
ncolnew=0
icol0new=intarr(tab0.ncolmx)
icol1new=intarr(tab1.ncolmx)
ntaiold=0
itai0old=intarr(tab0.ntaimx)
itai1old=intarr(tab1.ntaimx)
ntainew=0
itai0new=intarr(tab0.ntaimx)
itai1new=intarr(tab1.ntaimx)
ntarold=0
itar0old=intarr(tab0.ntarmx)
itar1old=intarr(tab1.ntarmx)
ntarnew=0
itar0new=intarr(tab0.ntarmx)
itar1new=intarr(tab1.ntarmx)
ntasold=0
itas0old=intarr(tab0.ntasmx)
itas1old=intarr(tab1.ntasmx)
ntasnew=0
itas0new=intarr(tab0.ntasmx)
itas1new=intarr(tab1.ntasmx)
ntai1=0
ntar1=0
ntas1=0
; --- Compare entry names ---
if (keyword_set(compareheader)) then begin
  ; --- Discriminate between columns to be updated (...old) and columns to
  ;     be added (...new)
  ;
  tab0type=reform(tab0.termt[0,0:1,0:tab0.ncol-1])
  ; --- Loop over all columns of second table (tab1) ---
  for ncol1=0,tab1.ncol-1 do begin
    ; --- Look if current column of tab1 exists in tab0 ---
    type1 =tab1.termt[0,0,ncol1]
    ident1=tab1.termt[0,1,ncol1]
    if ((type1 ne tabna) or tableheader) then begin
      uio_nc2nt, tab0type, tab0.ncol, ncol0, ntai0, ntar0, ntas0, outstr,ierr, $
        type=type1, ident=ident1
      ;
      if (outstr eq 'found') then begin
        ; --- Entry is found in tab0 ---
        icol0old[ncolold]=ncol0
        icol1old[ncolold]=ncol1
        ncolold=ncolold+1
        if (type1 eq intena) then begin
          itai0old[ntaiold]=ntai0
          itai1old[ntaiold]=ntai1
          ntaiold= ntaiold+1
        endif else if (type1 eq realna) then begin
          itar0old[ntarold]=ntar0
          itar1old[ntarold]=ntar1
          ntarold= ntarold+1
        endif else if (type1 eq charna) then begin
          itas0old[ntasold]=ntas0
          itas1old[ntasold]=ntas1
          ntasold= ntasold+1
        endif
      endif else begin
        ; --- No corresponding entry in tab0 ---
        icol1new[ncolnew]=ncol1
        ncolnew=ncolnew+1
        if (type1 eq intena) then begin
          itai1new[ntainew]=ntai1
          ntainew= ntainew+1
        endif else if (type1 eq realna) then begin
          itar1new[ntarnew]=ntar1
          ntarnew= ntarnew+1
        endif else if (type1 eq charna) then begin
          itas1new[ntasnew]=ntas1
          ntasnew= ntasnew+1
        endif
      endelse    
      ;
      if (type1 eq intena) then begin
        ntai1=ntai1+1
      endif else if (type1 eq realna) then begin
        ntar1=ntar1+1
      endif else if (type1 eq charna) then begin
        ntas1=ntas1+1
      endif
    endif
  endfor
  if (ncolnew gt 0) then icol0new=lindgen(ncolnew)+long(tab0.ncol)
  if (ntainew gt 0) then itai0new=lindgen(ntainew)+long(tab0.ntai)
  if (ntarnew gt 0) then itar0new=lindgen(ntarnew)+long(tab0.ntar)
  if (ntasnew gt 0) then itas0new=lindgen(ntasnew)+long(tab0.ntas)
endif else begin
  ; --- Columns of tab1 are only appended to tab0 ---
  ncolold=0
  ncolnew=tab1.ncol
  if (ncolnew gt 0) then begin
    if (tableheader) then begin
      icol0new=lindgen(tab1.ncol)+tab0.ncol
      icol1new=lindgen(tab1.ncol)
    endif else begin
      icol1new=where(tab1.termt[0,0,0:tab1.ncol-1] ne tabna, ncolnew)
      if (ncolnew gt 0) then begin
        icol0new=lindgen(ncolnew)+tab0.ncol
      endif
    endelse
  endif
  ntainew =tab1.ntai
  if (ntainew gt 0) then begin
    itai0new=lindgen(tab1.ntai)+tab0.ntai
    itai1new=lindgen(tab1.ntai)
  endif
  ntarnew =tab1.ntar
  if (ntarnew gt 0) then begin
    itar0new=lindgen(tab1.ntar)+tab0.ntar
    itar1new=lindgen(tab1.ntarmx)
  endif
  ntasnew=tab1.ntas
  if (ntasnew gt 0) then begin
    itas0new=lindgen(tab1.ntas)+tab0.ntas
    itas1new=lindgen(tab1.ntas)
  endif
endelse
;
; ---------------------------------
; --- Compute row index vectors ---
; ---------------------------------
;
if (keyword_set(comparedata)) then begin
  if ((n_elements(col0) ne 0) and (n_elements(col1) ne 0)) then begin
    comparedata_tmp=(col0 ge 0) and (col1 ge 0)
  endif else begin
    comparedata_tmp=0
  endelse
endif else begin
  comparedata_tmp=0
endelse
;
nlinold=0
ilin0old=intarr(tab0.nlinmx)
ilin1old=intarr(tab1.nlinmx)
nlinnew=0
ilin0new=intarr(tab0.nlinmx)
ilin1new=intarr(tab1.nlinmx)
if (comparedata_tmp) then begin
  ; --- Compare data of columns col0 in tab0 and col1 in tab1 ---
  ;
  ident0=''
  type0=''
  uio_tabr, val0, ident0, type=type0, tab0, outstr,ierr, $
    incolumn=ncol0
  if (ierr gt 0) then goto, endlabel
  ident1=''
  type1=''
  uio_tabr, val1, ident1, type=type1, tab1, outstr,ierr, $
    incolumn=ncol1
  if (ierr gt 0) then goto, endlabel
  ;
  ; --- Loop over all lines of second table (tab1) ---
  for i=0,tab1.nlin-1 do begin
    ind=where(val1[i] eq val0, count)
    if (count gt 0) then begin
      ; --- At least one data pair is equal ---
      ilin0old[nlinold]=ind[0]
      ilin1old[nlinold]=i
      nlinold=nlinold+1
    endif else begin
      ; --- No match ---
      ilin1new[nlinnew]=i
      nlinnew=nlinnew+1
    endelse
  endfor  
  if (nlinnew gt 0) then ilin0new=lindgen(nlinnew)+tab0.nlin
endif else begin
  ; --- tab1 is only appended to tab0 ---
  nlinold=0
  nlinnew=tab1.nlin
  ilin0new=lindgen(nlinnew>1)
  ilin1new=lindgen(nlinnew>1)
  if (not columnappend) then begin
    ilin0new=ilin0new+tab0.nlin
  endif
endelse
;
; --------------------------
; --- Check array ranges ---
; --------------------------
;
if (ncolnew gt 0) then begin
  if (icol0new[ncolnew-1] ge tab0.ncolmx) then begin
    ierr=1
    outstr='too many columns in merged table'
    goto, endlabel
  endif
endif
if (ntainew gt 0) then begin
  if (itai0new[ntainew-1] ge tab0.ntaimx) then begin
    ierr=1
    outstr='too many integer entries in merged table'
    goto, endlabel
  endif
endif
if (ntarnew gt 0) then begin
  if (itar0new[ntarnew-1] ge tab0.ntarmx) then begin
    ierr=1
    outstr='too many real entries in merged table'
    goto, endlabel
  endif
endif
if (ntasnew gt 0) then begin
  if (itas0new[ntasnew-1] ge tab0.ntasmx) then begin
    ierr=1
    outstr='too many character entries in merged table'
    goto, endlabel
  endif
endif
if (nlinnew gt 0) then begin
  if (ilin0new[nlinnew-1] ge tab0.nlinmx) then begin
    ierr=1
    outstr='too many lines in merged table'
    goto, endlabel
  endif
endif
;
; -----------------------------
; --- Complete index fields ---
; -----------------------------
;
; --- Number of columns
tab0.ncol=tab0.ncol+ncolnew
;
;help,ncolnew,icol0new,icol1new
;print,icol0new[0:ncolnew-1]
;print,icol1new[0:ncolnew-1]
; --- Header data ---
if (ncolnew gt 0) then begin
  ; --- Copy headers of appended columns ---
  tab0.termt[*,*,icol0new[0:ncolnew-1]]=tab1.termt[*,*,icol1new[0:ncolnew-1]]
  tab0.ntta[icol0new[0:ncolnew-1]]=tab1.ntta[icol1new[0:ncolnew-1]]
endif
if ((ncolold gt 0) and keyword_set(replacehead)) then begin
  ; --- Replace header ---
  tab0.termt[*,*,icol0old[0:ncolold-1]]=tab1.termt[*,*,icol1old[0:ncolold-1]]
  tab0.ntta[icol0old[0:ncolold-1]]=tab1.ntta[icol1old[0:ncolold-1]]
endif else if ((ncolold gt 0) and keyword_set(addheader)) then begin
  ; --- Append only keywords which are new ---
  for icol=0,ncolold-1 do begin
    ncol_0=icol0old[icol]
    ncol_1=icol1old[icol]
    termt0=tab0.termt[*,*,ncol_0]
    ntt0=tab0.ntta[ncol_0]
    termt1=tab1.termt[*,*,ncol_1]
    ntt1=tab1.ntta[ncol_1]
    termt1[0,0:1]=''
    for i=2,ntt0-1 do begin
      uio_exkeyw, 1+2*(i eq (ntt0-1)), termt1,ntt1, termt[0,i],keyval, $
                  outstr,ierr
    endfor
    nttnew=(ntt0+ntt1)<nttmx
    termt0[*,ntt0:nttnew-1]=termt1[0:nttnew-ntt0-1]
    tab0.termt[*,*,ncol_0]=termt0
    tab0.ntta[ncol_0]=nttnew
  endfor
endif
;
;ncoltot=0
;icol0tot=intarr(tab0.ncolmx)
;icol1tot=intarr(tab1.ncolmx)
ntaitot=0
itai0tot=intarr(tab0.ntaimx)
itai1tot=intarr(tab1.ntaimx)
ntartot=0
itar0tot=intarr(tab0.ntarmx)
itar1tot=intarr(tab1.ntarmx)
ntastot=0
itas0tot=intarr(tab0.ntasmx)
itas1tot=intarr(tab1.ntasmx)
;
if (overwrite or (lineappend and not columnappend)) then begin
  if (ntaiold gt 0) then begin
    itai0tot[ntaitot:ntaitot+ntaiold-1]=itai0old[0:ntaiold-1]
    itai1tot[ntaitot:ntaitot+ntaiold-1]=itai1old[0:ntaiold-1]
    ntaitot=ntaitot+ntaiold
  endif   
  if (ntarold gt 0) then begin
    itar0tot[ntartot:ntartot+ntarold-1]=itar0old[0:ntarold-1]
    itar1tot[ntartot:ntartot+ntarold-1]=itar1old[0:ntarold-1]
    ntartot=ntartot+ntarold
  endif   
  if (ntasold gt 0) then begin
    itas0tot[ntastot:ntastot+ntasold-1]=itas0old[0:ntasold-1]
    itas1tot[ntastot:ntastot+ntasold-1]=itas1old[0:ntasold-1]
    ntastot=ntastot+ntasold
  endif   
endif
;
;help,columnappend,ncolnew,ntainew,ntarnew,ntasnew
if (columnappend) then begin
  if (ntainew gt 0) then begin
    itai0tot[ntaitot:ntaitot+ntainew-1]=itai0new[0:ntainew-1]
    itai1tot[ntaitot:ntaitot+ntainew-1]=itai1new[0:ntainew-1]
    ntaitot=ntaitot+ntainew
  endif   
  if (ntarnew gt 0) then begin
    itar0tot[ntartot:ntartot+ntarnew-1]=itar0new[0:ntarnew-1]
    itar1tot[ntartot:ntartot+ntarnew-1]=itar1new[0:ntarnew-1]
    ntartot=ntartot+ntarnew
  endif
  if (ntasnew gt 0) then begin
    itas0tot[ntastot:ntastot+ntasnew-1]=itas0new[0:ntasnew-1]
    itas1tot[ntastot:ntastot+ntasnew-1]=itas1new[0:ntasnew-1]
    ntastot=ntastot+ntasnew
  endif   
endif
;
;help, ntaitot, itai0tot, itai1tot
;print, itai0tot[0:ntaitot-1]
;print, itai1tot[0:ntaitot-1]
;
nlintot=0
ilin0tot=intarr(tab0.nlinmx)
ilin1tot=intarr(tab1.nlinmx)
;
if (overwrite or (columnappend and not lineappend)) then begin
  if (nlinold gt 0) then begin
    ilin0tot[nlintot:nlintot+nlinold-1]=ilin0old[0:nlinold-1]
    ilin1tot[nlintot:nlintot+nlinold-1]=ilin1old[0:nlinold-1]
    nlintot=nlintot+nlinold
  endif
endif
;
if (lineappend) then begin
  if (nlinnew gt 0) then begin
    ilin0tot[nlintot:nlintot+nlinnew-1]=ilin0new[0:nlinnew-1]
    ilin1tot[nlintot:nlintot+nlinnew-1]=ilin1new[0:nlinnew-1]
    nlintot=nlintot+nlinnew
  endif
endif
;
;help, nlintot, ilin0tot, ilin1tot
;print,ilin0tot[0:[nlintot-1)>0]
;print,ilin1tot(0:(nlintot-1)>0]
;
; --------------------
; --- Merge tables ---
; --------------------
;
if (((ntainew+ntarnew+ntasnew) gt 0) and (nlintot gt 0)) then begin
  ilin0tot=ilin0tot[0:nlintot-1]
  ilin1tot=ilin1tot[0:nlintot-1]
  if (ntaitot gt 0) then begin
    itai0tot=itai0tot[0:ntaitot-1]
    itai1tot=itai1tot[0:ntaitot-1]
    indi0=rebin(reform(long(itai0tot)*tab0.nlinmx, 1,ntaitot),nlintot,ntaitot)+$
          rebin(reform(long(ilin0tot)            , nlintot,1),nlintot,ntaitot)
    indi1=rebin(reform(long(itai1tot)*tab1.nlinmx,1,ntaitot),nlintot,ntaitot)+$
          rebin(reform(long(ilin1tot)            ,nlintot,1),nlintot,ntaitot)
    tab0.i[indi0]=tab1.i[indi1]
    tab0.ntai=tab0.ntai+ntainew
  endif
  if (ntartot gt 0) then begin
    itar0tot=itar0tot[0:ntartot-1]
    itar1tot=itar1tot[0:ntartot-1]
    indr0=rebin(reform( long(itar0tot)*tab0.nlinmx,1,ntartot),nlintot,ntartot)+$
          rebin(reform( long(ilin0tot)            ,nlintot,1),nlintot,ntartot)
    indr1=rebin(reform( long(itar1tot)*tab1.nlinmx,1,ntartot),nlintot,ntartot)+$
          rebin(reform( long(ilin1tot)            ,nlintot,1),nlintot,ntartot)
    tab0.r[indr0]=tab1.r[indr1]
    tab0.ntar=tab0.ntar+ntarnew
  endif
  if (ntastot gt 0) then begin
    itas0tot=itas0tot[0:ntastot-1]
    itas1tot=itas1tot[0:ntastot-1]
    inds0=rebin(reform( long(itas0tot)*tab0.nlinmx,1,ntastot),nlintot,ntastot)+$
          rebin(reform( long(ilin0tot)            ,nlintot,1),nlintot,ntastot)
    inds1=rebin(reform( long(itas1tot)*tab1.nlinmx,1,ntastot),nlintot,ntastot)+$
          rebin(reform( long(ilin1tot)            ,nlintot,1),nlintot,ntastot)
    tab0.s[inds0]=tab1.s[inds1]
    tab0.ntar=tab0.ntar+ntarnew
  endif
  tab0.nlin=tab0.nlin>tab1.nlin
  ;
  ierr=0
  outstr='done'
endif else begin
  ierr=0
  outstr='nothing done'
endelse
;
endlabel:
;
end ; uio_tabm
