pro uio_tabw, val,ident, tab, outstr,ierr, $
    nbytes=nbytes, $
    format=format, name=name, unit=unit, tran=tran, date=date, $
    termt=termt,ntt=ntt, $
    mode=mode, incolumn=incolumn, tableheader=tableheader
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_tabw ('uio_table_write')
;
; PURPOSE:
;   Write 1d array data of any type (integer, real, or character) into table.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_tabw, val,ident, tab, outstr,ierr, $
;             nbytes=nbytes, $
;             format=format, name=name, unit=unit, tran=tran, date=date, $
;             termt=termt,ntt=ntt, $
;             mode=mode, incolumn=incolumn, tableheader=tableheader
;
; INPUT:
;   val:    ('value') integer, real, or character; array to be written
;   ident:  ('identifier') character, identifier (mandatory)
;
; INPUT/OUTPUT:
;   tab:    ('table') structure, complete table
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   nbytes: ('number_bytes') integer, number of characters of variable val to
;           be written (only if val is of type character)
;   format: ('format') character, FORTRAN style format string
;   name:   ('name') character, name of variable
;   unit:   ('unit') character, string with units
;   tran:   ('transformation') character, transformation
;   date:   ('date') character, date string, empty or 'now'
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;           (ntt >= 0)
;   mode:   ('mode') character, mode: 'append' (default), 'insert', 'overwrite', 'delete'
;   incolumn: ('input_column') integer, column to be modified (default:tab.ncol)
;   tableheader: ('table_header') indicate, that current entry is new table
;                header.
;
; ROUTINES:
;   uio_adkey1: Add one keyword to term table, keyword-value=' ', no link char.
;   uio_adkey2: Add one keyword to term table with keyword-value
;   uio_dim2st: Make dimension string
;   uio_nc2nt:  From column number  or entry name find table entry number
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;   uio_uiotabinc: empty table
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;   uio_taborg in uio_uiotabinc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Table tab only contains long integers, reals and character strings.
;   Other types are converted to one of theses three types.
;   The table tab has only a finite size in every dimension.
;
; PROCEDURE:
;   Write 1d array val as column into table tab with correct type, identifier
;   and specified keywords. The column may be appended or inserted. An old
;   column may be overwritten or deleted.
;
; EXAMPLE:
;   ; --- Delete table value ---
;   delvar, tab
;   ; --- At first write header (no data value) ---
;   uio_tabw, '', 'testtable', tab, outstr,ierr, $
;     name='Tabelle zum Testen', date=now, /tableheader
;   ; --- linenumber ---
;   nlin=10
;   ; --- Test data: index values ---
;   uio_tabw, indgen(nlin), 'index', tab, outstr,ierr
;   ; --- Test data: index values squared (as reals)  ---
;   uio_tabw, findgen(nlin)^2, 'index_real', tab, outstr,ierr, format='F6.1'
;   ; --- Test data: strings ---
;   uio_tabw, ['zero', 'one', 'two', 'three', 'four', 'five', 'six', 'seven', $
;              'eight','nine'], 'numbers', tab, outstr,ierr, nbytes=8
;   ; --- Show structure ---
;   help, /structure, tab
;   ; --- Write result on screen (unit=-1) ---
;   uio_wrtab, -1, tab
;
; MODIFICATION HISTORY:
;   25. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   26. 9.96 (B.F.) Corrections
;   28. 9.96 (B.F.) New common block in uio_uiotabinc
;   30. 9.96 (B.F.) Keyword incolumn added
;    1.10.96 (B.F.) Keyword mode added
;    2.10.96 (B.F.) ncmin may become negative, cvnum(.) -> cvnum(.-ncmin)
;                   ident may be undefined
;    4.10.96 (B.F.) Column entry may be table header
;    6.10.96 (B.F.) Parameter nc deleted, routine uio_nc2nt called
;    7.10.96 (B.F.) ita -> nlin
;   10.10.96 (B.F.) Allow format='*'; renamed: uio_wt -> uio_tabw
;   13.10.96 (B.F.) Renaming: termt
;   15.10.96 (B.F.) More comments
;   26.10.96 (B.F.) Merging of entry header modified,
;            format handling into uio_wrtab.
;   27.10.96 (B.F.) goto, endlabel
;   30.10.96 (B.F.) Corrections during deletion of last column
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiotabinc
;
; --- Default values for keywords ---
if (not keyword_set(mode)) then mode='append'
if ((mode ne 'append') and (mode ne 'insert') and (mode ne 'delete') and $
    (mode ne 'overwrite')) then begin
  print,   'uio_tabw: mode:'+mode
  message, 'uio_tabw: Unknown storage mode'
endif
;
; --- Determine type of data variable val ---
; --- size(0): dimension
; --- size(size(0)+1): 0:undefined, 1:byte,          2:integer, 3:long, 
; ---                  4:real,      5:double real,   6:complex, 7:string, 
; ---                  8:structure, 9:double complex
if (keyword_set(tableheader)) then begin
  ; --- Type value corresponding to table header ---
  val_type=-1
endif else begin
  val_size=size(val)
  val_type=val_size(val_size(0)+1)
  if ((val_type eq 1) or (val_type eq 2)) then begin
    ; --- Byte, integer ---
    ; --- Assume val is of type long integer ---
    val_type=3
  endif else if (val_type eq 5) then begin
    ; --- Double real ---
    ; --- Assume val is of type single real ---
    val_type=4
  endif else if (val_type eq 6) then begin
    ; --- Single complex ---
    ; --- Assume val is of type single real ---
    val_type=4
  endif else if (val_type eq 9) then begin
    ; --- Double complex ---
    ; --- Assume val is of type single real ---
    val_type=4
  endif
endelse
;
if (keyword_set(format)) then begin
  frmt=format
endif else begin
  frmt=''
endelse
; --- Type dependent settings ---
if (val_type eq -1) then begin
  ; --- Table header ---
  type=tabna
endif else if (val_type eq 3) then begin
  ; --- Long integer ---
  type=intena
endif else if (val_type eq 4) then begin
  ; --- Single real ---
  type=realna
endif else if (val_type eq 7) then begin
  ; --- String ---
  type=charna
  if (keyword_set(nbytes)) then begin
    lenval=strtrim(string(fix(abs(nbytes))),2)
  endif else begin
    lenval=''
  endelse
endif else if (mode eq 'delete') then begin
  val_type=3
endif else begin
  message, 'uio_tabw: unknown data type'
endelse
;
if (n_elements(tab) eq 0) then begin
  ; --- Table is undefined, construct empty structure ---
  tab=tab_org
endif
;
ierr=0
; --- Check table array size ---
if ((mode eq 'append') or (mode eq 'insert') or (mode eq 'overwrite')) then $
  begin
  if (tab.ncol ge tab.ncolmx) then begin
    ; --- Table is already full ---
    outstr='Total table full'
    ierr=1
    goto, endlabel
  endif else if ((tab.ntai ge tab.ntaimx) and (val_type eq 3)) then begin
    ; --- Table with integers is already full ---
    outstr='Integer table full'
    ierr=1
    goto, endlabel
  endif else if ((tab.ntar ge tab.ntarmx) and (val_type eq 4)) then begin
    ; --- Table with reals is already full ---
    outstr='Real table full'
    ierr=1
    goto, endlabel
  endif else if ((tab.ntas ge tab.ntasmx) and (val_type eq 7)) then begin
    ; --- Table with strings is already full ---
    outstr='Character table full'
    ierr=1
    goto, endlabel
  endif
endif
;
; --- Define header ---
ntt_tmp=0
if ((mode eq 'append') or (mode eq 'insert') or (mode eq 'overwrite')) then $
  begin
  termt_tmp=strarr(2,nttmx)
  uio_adkey1, termt_tmp,ntt_tmp,type  ,              outstr,ierr
  if (keyword_set(ident)) then begin
    uio_adkey1, termt_tmp,ntt_tmp,ident,             outstr,ierr
  endif else begin
    uio_adkey1, termt_tmp,ntt_tmp,type,              outstr,ierr
  endelse
  if (val_type eq -1) then begin
    uio_adkey2, termt_tmp,ntt_tmp,dimna,'(1:0,1:0)', outstr,ierr
  endif
  uio_adkey2, termt_tmp,ntt_tmp,frmtna,frmt  ,       outstr,ierr
  if (type eq charna) then begin
    uio_adkey2, termt_tmp,ntt_tmp,bytena,lenval,     outstr,ierr
  endif
  if (n_elements(name) ne 0) then begin
    uio_adkey2, termt_tmp,ntt_tmp,namena,name  ,     outstr,ierr
  endif
  if (n_elements(unit) ne 0) then begin
    uio_adkey2, termt_tmp,ntt_tmp,unitna,unit,       outstr,ierr
  endif
  if (n_elements(tran) eq 0) then tran=''
  if ((tran eq 'log10') and (type eq realna)) then begin
    uio_adkey2, termt_tmp,ntt_tmp,tranna,tran,       outstr,ierr
  endif
  if (n_elements(date) ne 0) then begin
    if (date eq 'now') then begin
      dat=systime(0)
    endif else begin
      dat=date
    endelse
    uio_adkey2, termt_tmp,ntt_tmp,datena,dat   ,     outstr,ierr
  endif
endif
;
; --- Find table column ---
if ((mode eq 'append') or (n_elements(incolumn) eq 0)) then begin
  newcol=tab.ncol
  ntaisl=tab.ntai
  ntarsl=tab.ntar
  ntassl=tab.ntas
endif else begin
  newcol=0>incolumn<tab.ncol
  uio_nc2nt, reform(tab.termt(0,0:1,0:tab.ncol-1)), tab.ncol, newcol, $
    ntaisl, ntarsl, ntassl, outstr,ierr
  if (outstr eq 'found') then begin
    type_tmp=tab.termt(0,0,newcol)
  endif else begin
    type_tmp=''
  endelse
endelse
;
; --- Shift table header array ---
if (mode eq 'delete') then begin
  if (newcol lt (tab.ncol-1)) then begin
    tab.termt(*,*,newcol:tab.ncol-2)=tab.termt(*,*,newcol+1:tab.ncol-1)
    tab.ntta(newcol:tab.ncol-2)=tab.ntta(newcol+1:tab.ncol-1)
  endif
  tab.ncol=tab.ncol-1
endif else if (mode eq 'append') then begin
  tab.ncol=tab.ncol+1
endif else if (mode eq 'insert') then begin
  tab.termt(*,*,newcol+1:tab.ncol)=tab.termt(*,*,newcol:tab.ncol-1)
  tab.ntta(newcol+1:tab.ncol)=tab.ntta(newcol:tab.ncol-1)
  tab.ncol=tab.ncol+1
endif
;
if ((mode eq 'append') or (mode eq 'insert') or (mode eq 'overwrite')) then $
  begin
  ; --- Put term table into table header array ---
  tab.termt(*,0:ntt_tmp-1,newcol)=termt_tmp(*,0:ntt_tmp-1)
  if (n_elements(ntt) eq 0) then ntt=n_elements(termt)/2
  nttn=(ntt_tmp+ntt)<nttmx
  if (nttn gt ntt_tmp) then begin
    tab.termt(*,ntt_tmp:nttn-1,newcol)=termt(*,0:nttn-ntt_tmp-1)
  endif
  tab.ntta(newcol)=nttn
endif
;
; --- Shift table data array ---
if ((mode eq 'delete') or (mode eq 'overwrite')) then begin
  if (type_tmp eq intena) then begin
    ; --- Integer ---
    if (ntaisl le (tab.ntai-2)) then begin
      tab.i(0:tab.nlin-1,ntaisl  :tab.ntai-2)= $
      tab.i(0:tab.nlin-1,ntaisl+1:tab.ntai-1)
    endif
    tab.ntai=tab.ntai-1
  endif else if (type_tmp eq realna) then begin
    ; --- Real ---
    if (ntarsl le (tab.ntar-2)) then begin
      tab.r(0:tab.nlin-1,ntarsl  :tab.ntar-2)= $
      tab.r(0:tab.nlin-1,ntarsl+1:tab.ntar-1)
    endif
    tab.ntar=tab.ntar-1
  endif else if (type_tmp eq charna) then begin
    ; --- Character ---
    if (ntassl le (tab.ntas-2)) then begin
      tab.s(0:tab.nlin-1,ntassl  :tab.ntas-2)= $
      tab.s(0:tab.nlin-1,ntassl+1:tab.ntas-1)
    endif
    tab.ntas=tab.ntas-1
  endif
endif
if ((mode eq 'insert') or (mode eq 'overwrite') or (mode eq 'append')) then $
  begin
  if (val_type eq 3) then begin
    ; --- Integer ---
    if (ntaisl le (tab.ntai-1)) then begin
      tab.i(0:tab.nlin-1,ntaisl+1:tab.ntai  )= $
      tab.i(0:tab.nlin-1,ntaisl  :tab.ntai-1)
    endif
    tab.ntai=tab.ntai+1
  endif else if (val_type eq 4) then begin
    ; --- Real ---
    if (ntarsl le (tab.ntar-1)) then begin
      tab.r(0:tab.nlin-1,ntarsl+1:tab.ntar  )= $
      tab.r(0:tab.nlin-1,ntarsl  :tab.ntar-1)
    endif
    tab.ntar=tab.ntar+1
  endif else if (val_type eq 7) then begin
    ; --- Character ---
    if (ntassl le (tab.ntas-1)) then begin
      tab.s(0:tab.nlin-1,ntassl+1:tab.ntas  )= $
      tab.s(0:tab.nlin-1,ntassl  :tab.ntas-1)
    endif
    tab.ntas=tab.ntas+1
  endif
endif
;
if ((mode eq 'append') or (mode eq 'insert') or (mode eq 'overwrite')) then $
  begin
  ; --- Write 1D input array into table data array ---
  if (val_type eq 3) then begin
    ; --- Integer ---
    nlin_new=n_elements(val)<tab.nlinmx
    tab.i(0:nlin_new-1,ntaisl)=long(val(0:nlin_new-1))
    ; --- Fill up with zeros ---
    if (nlin_new lt tab.nlinmx) then begin
      tab.i(nlin_new:*,ntaisl)=0L
    endif      
  endif else if (val_type eq 4) then begin
    ; --- Real ---
    nlin_new=n_elements(val)<tab.nlinmx
    if (tran eq 'log10') then begin
      ; --- Transformation ---
      tab.r(0:nlin_new-1,ntarsl)=alog10(float(val(0:nlin_new-1)))
    endif else begin
      tab.r(0:nlin_new-1,ntarsl)=float(val(0:nlin_new-1))
    endelse
    ; --- Fill up with zeros ---
    if (nlin_new lt tab.nlinmx) then begin
      tab.r(nlin_new:*,ntarsl)=0.0
    endif      
  endif else if (val_type eq 7) then begin
    ; --- String ---
    nlin_new=n_elements(val)<tab.nlinmx
    tab.s(0:nlin_new-1,ntassl)=val(0:nlin_new-1)
    ; --- Fill up with blanks ---
    if (nlin_new lt tab.nlinmx) then begin
      tab.s(nlin_new:*,ntassl)=''
    endif      
  endif else begin
    nlin_new=tab.nlin
  endelse
  tab.nlin=tab.nlin>nlin_new
endif
;
outstr='done'
ierr=0
;
endlabel:
;
end ; uio_tabw
