pro uio_uopen, nc,file,form,conv,status,rdwr, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_uopen ('uio_universal_open')
;
; PURPOSE:
;   Open file with special handling for conversion type.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_uopen, nc,file,form,conv,status,rdwr, outstr,ierr
;
; INPUT:
;   file:   ('filename') character, filename
;   form:   ('format') character, 'formatted' or 'unformatted'
;   conv:   ('conversion') character, type of conversion procedure
;   status: ('status') character, 'append'
;   rdwr:   ('read_write') character, 'read', 'write', 'append'
;
; OUTPUT:
;   nc:     ('number_channel') integer, channel unit number
;   outstr: ('output_string'), character
;   ierr:   ('integer_error'), integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;   A file is opened.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Just calls openr.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   18. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   29. 9.96 (B.F.) conv: 'native', 'idl', 'xdr', 'ieee_4'
;   15.10.96 (B.F.) open statements with error keyword
;   16.10.96 (B.F.) 'ieee' -> 'ieee_4'
;   17.04.01 (B.F.) 'ieeele_4', 'ieee_4'
;   24.07.03 (B.F.) status='append' allowed
;   2009-03-18 (B.F. Lyon) Open compressed (.gz) files for reading
;-
;------------------------------------------------------------------------------
;
ierr=0
append_flag=(rdwr eq 'append')
;
; --- Find out if compression flag should be set ---
;compress_flag=0
if ((rdwr eq 'write') or (rdwr eq 'append')) then begin
  ; --- No compression during writing, yet ---
  compress_flag=0
  file0=file
endif else if (file_test(file)) then begin
  ; --- Compression if specified file name ends with '.gz' ---
  compress_flag=(strlowcase(strmid(file, strlen(file)-3)) eq '.gz')
  file0=file
endif else begin
  ; --- Compression if a file with additional '.gz' exists ---
  file0=file + '.gz'
  compress_flag=file_test(file0)
endelse
;print, 'file = ' + file
;print, 'file0= ' + file0
;help, form, rdwr, conv, compress_flag
;
; --- Open file on given channel with specified format ---
if (form eq 'formatted') then begin
  if ((rdwr eq 'write') or (rdwr eq 'append')) then begin
    openw, nc, file0, append=append_flag, error=ierr
  endif else begin
    openr, nc, file0, compress=compress_flag, error=ierr
  endelse
endif else begin ; --- 'unformatted' ---
  if ((rdwr eq 'write') or (rdwr eq 'append')) then begin
    if (conv eq 'ieee_4') then begin
      openw, nc, file0, append=append_flag, /f77_unformatted, /swap_if_little_endian, error=ierr
    endif else if (conv eq 'ieeele_4') then begin
      openw, nc, file0, append=append_flag, /f77_unformatted, /swap_if_big_endian, error=ierr
    endif else if (conv eq 'xdr') then begin
      openw, nc, file0, append=append_flag, /xdr, error=ierr
    endif else begin
      openw, nc, file0, append=append_flag, error=ierr
    endelse
  endif else begin
    if (conv eq 'ieee_4') then begin
      openr, nc, file0, /f77_unformatted, /swap_if_little_endian, compress=compress_flag, error=ierr
    endif else if (conv eq 'ieeele_4') then begin
      openr, nc, file0, /f77_unformatted, /swap_if_big_endian, compress=compress_flag, error=ierr
    endif else if (conv eq 'xdr') then begin
      openr, nc, file0, /xdr, compress=compress_flag, error=ierr
    endif else begin
      openr, nc, file0, compress=compress_flag, error=ierr
    endelse
  endelse
endelse
;
if (ierr eq 0) then begin
  outstr='done'
endif else begin
  outstr=!ERR_STRING
endelse
;
end ; uio_uopen
