pro uio_wf2rf, wf, rf, outstr, ierr, qmachr=qmachr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wf2rf ('uio_write_format_to_read_format')
;
; PURPOSE:
;   Produce from write format string corresponding read format string.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wf2rf, wf, rf, outstr, ierr
;
; INPUT:
;   wf:     ('write_format') character, write format string
;
; OUTPUT:
;   rf:     ('read_format') character, read format string
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   qmachr: ('quotation_mark_character') character, default="'" (single)
;
; ROUTINES:
;   None
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Don't use inner (double) quotation marks.
;
; PROCEDURE:
;   IDL behaviour: Strings within a format string are printed while writing
;   and ignored (and NOT skipped) while reading.
;   This routine extracts the strings and replaces them by an appropriate
;   number of skip commands ('X').
;   Allowed call:
;   f='I4'
;   uio_wf2rf, f, f, outstr,ierr
;
; EXAMPLE:
;   wf="'a=',E13.6"
;   uio_wf2rf, wf, rf, outstr,ierr
;   help, wf, rf, outstr,ierr
;     --->
;   WF              STRING    = ''a=',E13.6'
;   RF              STRING    = '2X,E13.6'
;   OUTSTR          STRING    = 'done'
;   IERR            INT       =        0
;
; MODIFICATION HISTORY:
;   12.10.96 Written by Bernd Freytag, Astrophysik Kiel
;   15.10.96 (B.F.) Keyword qmachr added
;-
;------------------------------------------------------------------------------
;
; --- Default value for optional parameter ---
if (keyword_set(qmachr)) then begin
  qma=strmid(qmachr,0,1)
endif else begin
  qma="'"
endelse
;
if (n_elements(wf) eq 0) then begin
  rf=''
  outstr='empty format'
  ierr=0
  goto, endlabel
endif
;
rf=wf
qma_strpos1=strpos(rf,qma)
while (qma_strpos1 ge 0) do begin
  qma_strpos2=strpos(rf,qma,qma_strpos1+1)
  if (qma_strpos2 eq -1) then begin
    ; --- No matching second quotation mark ---
    outstr='No matching second quotation mark'
    ierr=1
    goto, endlabel
  endif else begin
;    print, '---------------------------------------------------------'
;    help,rf
    ; str=strmid(rf,qma_strpos1+1,qma_strpos2-qma_strpos1-1)
    ; --- Assume that there are no inner quotation marks ---
    len=qma_strpos2-qma_strpos1-1
    insert=strtrim(string(len,format='(I6)'),2) +'X'
    ; --- Replace string: str -> insert ---
    rf=strmid(rf,0,qma_strpos1) + insert + $
       strmid(rf,qma_strpos2+1,strlen(rf))
;    help,qma_strpos1,qma_strpos2,len,insert,rf,outstr,ierr
  endelse
  qma_strpos1=strpos(rf,qma)
endwhile
;
outstr='done'
ierr=0
;
endlabel:
;
end ; uio_wf2rf
