pro uio_wr, nc, val,ident, iarr=iarr, outstr,ierr, $
    nbytes=nbytes, $
    format=format, name=name, unit=unit, tran=tran, date=date, $
    termt=termt, ntt=ntt
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wr ('uio_write')
;
; PURPOSE:
;   Write scalar or array data of all types
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wr, nc, val,ident, iarr=iarr, outstr,ierr, $
;           nbytes=nbytes, $
;           format=format, name=name, unit=unit, tran=tran, date=date, $
;           termt=termt,ntt=ntt
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   val:    ('value') real, array to be written
;   ident:  ('identifier') character, identifier (mandatory)
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   iarr:   ('index_array') integer array, element number = number of 
;           dimensions of data variable val 
;   nbytes: ('number_bytes') integer, number of characters of variable val to
;           be written (only if val is of type character)
;   format: ('format') character, FORTRAN style format string
;   name:   ('name') character, name of variable
;   unit:   ('unit') character, string with units
;   tran:   ('transformation') character, transformation
;   date:   ('date') character, date string, empty or 'now'
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;           (ntt >= 0)
;
; ROUTINES:
;   uio_adkey1: Add one keyword to term table, keyword-value=' ', no link char.
;   uio_adkey2: Add one keyword to term table with keyword-value
;   uio_chunit: Initialize, store and actualize a list of unit numbers
;   uio_dim2st: Make dimension string
;   uio_wrhd: Write header of variable, input: term table
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,linet)
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data are written to file on channel nc.
;
; RESTRICTIONS:
;   There is no check if the format string has a proper format: Do not mix
;   e.g. integer data values and real format strings (possible in IDL, but
;   causes an error in FORTRAN).
;   Allowed format strings for character data: '', '*', 'A', 'A#' (#=number).
;   Character data: Use the combination of format and nbytes carefully.
;   The default format strings for numbers are not quite sufficient to preserve
;   all the information of the original data.
;   Don't use format statements with strings included in blanks: IDL understands
;   what's going on only during the writing process - reading is impossible.
;   Surplus header keywords are silently ignored.
;
; PROCEDURE:
;   First, the header is composed and written into file.
;   Then, the data is written in formatted or unformatted form.
;
; EXAMPLE:
;   ; --- Initialization ---
;   uio_init, progrm='interactive'
;   ; --- Write to standard output (-1) ---
;   uio_wr, -1, 42L, 'answer', outstr,ierr, format='I3', date='now'
;     --->
;   integer answer f=I3 b=4 date='Wed Oct 16 15:40:57 1996'
;    42
;
; MODIFICATION HISTORY:
;   24. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   25. 9.96 (B.F.) byte, short integer ('formatted') -> val_type:=3
;                   tran:'' or 'log10' for real
;   29. 9.96 (B.F.) Write character arrays with correct padding
;    2.10.96 (B.F.) ncmin may become negative, cvnum(.) -> cvnum(.-ncmin)
;                   ident may be undefined
;   10.10.96 (B.F.) format='*' allowed: choose appropriate format
;   11.10.96 (B.F.) format='*' and scalar: no 'X' in format statement;
;                   strtrim(format); special handling of format='a'
;   13.10.96 (B.F.) Renaming: termt, linet
;   15.10.96 (B.F.) More comments
;   26.10.96 (B.F.) Use uio_wrhd instead of uio_wrhdln.
;    1. 1.97 (B.F.) 'X' -> '1X'
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocvlinc
@uio_uiocstinc
;
; --- Get format of output file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
; --- Determine type of data variable val ---
; --- size(0): dimension
; --- size(size(0)+1): 0:undefined, 1:byte,    2:integer, 3:long, 4:real,
; ---                  5:double,    6:complex, 7:string
val_size=size(val)
val_type=val_size(val_size(0)+1)
if ((val_type eq 1) or (val_type eq 2)) then begin
  if (form eq 'unformatted') then begin
    ; --- Transform val into long integer ---
    val=long(val)
    val_size=size(val)
    val_type=val_size(val_size(0)+1)
  endif else begin
    ; --- For 'formatted' output: Assume val is of type long integer ---
    val_type=3
  endelse
endif
;
if (keyword_set(format)) then format=strtrim(format,2)
; --- Type dependent settings ---
if (val_type eq 3) then begin
  ; --- Long integer ---
  type=intena
  if (keyword_set(format)) then begin
    if (format eq '*') then begin
      if (val_size(0) eq 0) then begin
        frmt=foinls(cvnum(nc-ncmin))
      endif else begin
        frmt='1X,'+foinls(cvnum(nc-ncmin))
      endelse
    endif else begin
      frmt=format
    endelse
  endif else begin
    frmt=foinls(cvnum(nc-ncmin))
  endelse
  lenval=btinls(cvnum(nc-ncmin))
endif else if (val_type eq 4) then begin
  ; --- Single real ---
  type=realna
  if (keyword_set(format)) then begin
    if (format eq '*') then begin
      if (val_size(0) eq 0) then begin
        frmt=forels(cvnum(nc-ncmin))
      endif else begin
        frmt='1X,'+forels(cvnum(nc-ncmin))
      endelse
    endif else begin
      frmt=format
    endelse
  endif else begin
    frmt=forels(cvnum(nc-ncmin))
  endelse
  lenval=btrels(cvnum(nc-ncmin))
endif else if (val_type eq 5) then begin
  ; --- Double real ---
  type=realna
  if (keyword_set(format)) then begin
    if (format eq '*') then begin
      if (val_size(0) eq 0) then begin
        frmt=fodbls(cvnum(nc-ncmin))
      endif else begin
        frmt='1X,'+fodbls(cvnum(nc-ncmin))
      endelse
    endif else begin
      frmt=format
    endelse
  endif else begin
    frmt=fodbls(cvnum(nc-ncmin))
  endelse
  lenval=btdbls(cvnum(nc-ncmin))
endif else if (val_type eq 6) then begin
  ; --- Complex ---
  type=compna
  if (keyword_set(format)) then begin
    if (format eq '*') then begin
      if (val_size(0) eq 0) then begin
        frmt=focols(cvnum(nc-ncmin))
      endif else begin
        frmt='1X,'+focols(cvnum(nc-ncmin))
      endelse
    endif else begin
      frmt=format
    endelse
  endif else begin
    frmt=focols(cvnum(nc-ncmin))
  endelse
  lenval=btcols(cvnum(nc-ncmin))
endif else if (val_type eq 7) then begin
  ; --- String ---
  type=charna
  nlen=max(strlen(val))
  if (keyword_set(format)) then begin
    if (format eq '*') then begin
      if (keyword_set(nbytes)) then begin
        lenval=strtrim(string(fix(abs(nbytes))),2)
      endif else begin
        lenval=strtrim(string(nlen),2)
      endelse
      frmt='A'+lenval
      ;reads,lenval,nlen,format='(I6)'
      nlen=fix(lenval)
    endif else begin
      frmt=format
      ; ?
      if (keyword_set(nbytes)) then begin
        lenval=strtrim(string(fix(abs(nbytes))),2)
      endif else begin
        lenval=strtrim(string(nlen),2)
      endelse
    endelse
  endif else begin
    if (keyword_set(nbytes)) then begin
      lenval=strtrim(string(fix(abs(nbytes))),2)
    endif else begin
      lenval=strtrim(string(nlen),2)
    endelse
    frmt='A'+lenval
    ;reads,lenval,nlen,format='(I6)'
    nlen=fix(lenval)
  endelse
endif else begin
  ierr=1
  outstr='Unknown type'
  goto, endlabel
endelse
;
if (val_size(0) gt 0) then begin
  ; --- Array data ---
  ;
  ; --- Make dimension string ---
  val_dim=lonarr(2,val_size(0))
  val_dim(1,*)=val_size(1:val_size(0))-1
  if (n_elements(iarr) ne 0) then begin
    ; --- Shift array indices ---
    val_dim(0,*)=val_dim(0,*)+long(iarr)
    val_dim(1,*)=val_dim(1,*)+long(iarr)
  endif
  uio_dim2st, val_dim, dimstr
  ;
  ; --- Data sets per line ---
  if (val_type eq 7) then begin
    ; --- character ---
    if (not keyword_set(format)) then begin
      perlin=strtrim(string((80/(nlen>1))>1),2)
    endif else if ((format eq '*') or (strupcase(format) eq 'A')) then begin
      perlin='1'
    endif else begin
      perlin=strtrim(string((80/(nlen>1))>1),2)
    endelse
  endif else begin
    perlin='4'
  endelse
  wfrmt='(' + perlin + '(' + frmt + '))'
endif else begin
  ; --- Scalar data ---
  ;
  ; --- Empty dimension string ---
  dimstr=''
  ;
  ; --- Data sets per line: empty string ---
  perlin=''
  wfrmt='(' + frmt + ')'
endelse
;
; --- Define header ---
ntt_tmp=0
uio_adkey1, termt_tmp,ntt_tmp,type  ,              outstr,ierr
if (keyword_set(ident)) then begin
  uio_adkey1, termt_tmp,ntt_tmp,ident,             outstr,ierr
endif else begin
  uio_adkey1, termt_tmp,ntt_tmp,type,              outstr,ierr
endelse
uio_adkey2, termt_tmp,ntt_tmp,dimna ,dimstr,       outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,frmtna,frmt  ,       outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,plinna,perlin,       outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,bytena,lenval,       outstr,ierr
if (n_elements(name) ne 0) then begin
  uio_adkey2, termt_tmp,ntt_tmp,namena,name  ,     outstr,ierr
endif
if (n_elements(unit) ne 0) then begin
  uio_adkey2, termt_tmp,ntt_tmp,unitna,unit,       outstr,ierr
endif
if (n_elements(tran) eq 0) then tran=''
if ((tran eq 'log10') and (type eq realna)) then begin
  uio_adkey2, termt_tmp,ntt_tmp,tranna,tran,       outstr,ierr
endif
if (n_elements(date) ne 0) then begin
  if (date eq 'now') then begin
;    dat=systime(0)
    dat=strmid(systime(0),0,24)
  endif else begin
    dat=date
  endelse
  uio_adkey2, termt_tmp,ntt_tmp,datena,dat   ,       outstr,ierr
endif
if (n_elements(ntt) eq 0) then ntt=n_elements(termt)/2
;
; --- Merge header ---
;nlt=0
;uio_memtrm, termt_tmp,ntt_tmp, linet,nlt, outstr,ierr
;uio_memtrm, termt    ,ntt    , linet,nlt, outstr,ierr
;;
;; --- Write header of variable ---
;uio_wrhdln, nc,form, linet,nlt, outstr,ierr
;
; --- Concatenate the two term tables ---
nttn=(ntt_tmp+ntt)<nttmx
if (nttn gt ntt_tmp) then begin
  termt_tmp(*,ntt_tmp:nttn-1)=termt(*,0:nttn-ntt_tmp-1)
endif
ntt_tmp=nttn
;
; --- Write header of variable ---
uio_wrhd, nc,form, termt_tmp,ntt_tmp, outstr,ierr
;
if ((tran eq 'log10') and (type eq realna)) then begin
  if (form eq 'unformatted') then begin
    writeu, nc, alog10(val)
  endif else begin
    printf, nc, alog10(val), format=wfrmt
  endelse
endif else begin
  if (form eq 'unformatted') then begin
    if (val_type eq 7) then begin
      pad=string(' ',format='(A'+lenval+')')
      writeu, nc, strmid(val+pad,0,nlen)
    endif else begin
      writeu, nc, val
    endelse
  endif else begin
    if (val_type eq 7) then begin
      if (strupcase(wfrmt) eq '(1(A))') then begin
        printf, nc, val, format=wfrmt
      endif else begin
        pad=string(' ',format='(A'+lenval+')')
        printf, nc, strmid(val+pad,0,nlen), format=wfrmt
      endelse
    endif else begin
      printf, nc, val, format=wfrmt
    endelse
  endelse
endelse
;
outstr='done'
ierr=0
;
endlabel:
;
end ; uio_wr
