pro uio_wrlabl, nc,ident, name=name,date=date,termt=termt,ntt=ntt, $
    outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wrlabl ('uio_write_label')
;
; PURPOSE:
;   Write label.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wrlabl, nc,ident, name=name,date=date,termt=termt,ntt=ntt, $
;               outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   ident:  ('identifier') character, identifier (mandatory)
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   name:   ('name') character, name of variable, input
;   date:   ('date') character, date string, empty or 'now', input
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
;           input
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;           (ntt >= 0), input
;
; ROUTINES:
;   uio_adkey1: Add one keyword to term table, keyword-value=' ', no link char.
;   uio_adkey2: Add one keyword to term table with keyword-value
;   uio_chunit: Initialize, store and actualize a list of unit numbers
;   uio_wrhd: Write header of variable, input: term table
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data are written into file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Assemble entry header and write it into file without data block.
;
; EXAMPLE:
;   uio_wrlabl, -1, 'mark_1', name='This mark marks nothing', date='now', $
;     outstr,ierr
;     --->
;   label mark_1 n='This mark marks nothing' date='Wed Oct 16 13:31:36 1996'
;
; MODIFICATION HISTORY:
;   22. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   25. 9.96 (B.F.) Keywords
;    2.10.96 (B.F.) ident may be undefined
;   13.10.96 (B.F.) Renaming: termt, linet
;   16.10.96 (B.F.) More comments
;   26.10.96 (B.F.) Use uio_wrhd instead of uio_wrhdln.
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
; --- Get format of output file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
; --- Define header ---
ntt_tmp=0
uio_adkey1, termt_tmp,ntt_tmp,lablna,                  outstr,ierr
if (keyword_set(ident)) then begin
  uio_adkey1, termt_tmp,ntt_tmp,ident,                 outstr,ierr
endif else begin
  uio_adkey1, termt_tmp,ntt_tmp,'mark',                outstr,ierr
endelse
if (n_elements(name) ne 0) then begin
  uio_adkey2, termt_tmp,ntt_tmp,namena,name  ,         outstr,ierr
endif
if (n_elements(date) ne 0) then begin
  if (date eq 'now') then begin
;    dat=systime(0)
    dat=strmid(systime(0),0,24)
  endif else begin
    dat=date
  endelse
  uio_adkey2, termt_tmp,ntt_tmp,datena,dat   ,       outstr,ierr
endif
if (n_elements(ntt) eq 0) then ntt=n_elements(termt)/2
;
; --- Concatenate the two term tables ---
nttn=(ntt_tmp+ntt)<nttmx
if (nttn gt ntt_tmp) then begin
  termt_tmp(*,ntt_tmp:nttn-1)=termt(*,0:nttn-ntt_tmp-1)
endif
ntt_tmp=nttn
;
; --- Write header of variable ---
uio_wrhd, nc,form, termt_tmp,ntt_tmp, outstr,ierr
;
end ; uio_wrlabl
