pro uio_wrtab, nc, tab, outstr,ierr, date=date
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wrtab ('uio_write_table')
;
; PURPOSE:
;   Write table of integer, real, and/or character data to file.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wrtab, nc, tab, outstr,ierr, date=date
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; INPUT/OUTPUT:
;   tab:    ('table') structure, complete table to be written into file
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   date:   ('date') character, date string, empty or 'now', input
;
; ROUTINES:
;   uio_adkey1: Add one keyword to term table, keyword-value=' ', no link char.
;   uio_adkey2: Add one keyword to term table with keyword-value
;   uio_adkey3: Add one keyword to term table with keyword-value or default
;   uio_chunit: Initialize, store and actualize a list of unit numbers
;   uio_dim2st: Make dimension string
;   uio_getenv: Get information about environment
;   uio_wrhd: Write header of variable, input: term table
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;   uio_taborg in uio_uiotabinc
;
; SIDE EFFECTS:
;   Data are written into file on unit nc.
;
; RESTRICTIONS:
;   For table entries other than table headers don't use format statements with
;   strings included in blanks with included blanks:
;
; PROCEDURE:
;   First, the table header is composed and written into file.
;   Then, the header for each entry is written into file.
;   Then, the data is written in formatted or unformatted form row by row.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   26. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   30. 9.96 (B.F.) Keyword bytes added.
;    2.10.96 (B.F.) ident may be undefined
;    4.10.96 (B.F.) Write list of tables
;    6.10.96 (B.F.) Add format strings for integer and real entries, add
;                   number of bytes according to output channel properties
;    7.10.96 (B.F.) ita -> nlin, table format may be empty
;   13.10.96 (B.F.) Renaming: linet
;   16.10.96 (B.F.) More comments, corrections for empty table
;   26.10.96 (B.F.) Format handling from uio_tabw into uio_wrtab.
;   27.10.96 (B.F.) termt_tmp <-> termt
;   28.10.96 (B.F.) Two loops in one
;    1. 1.97 (B.F.) 'X' -> '1X'
;    4. 2.97 (B.F.) Version for 'IDL' and 'WAVE'
;    1. 4.98 (B.F.) nts0 introduced, count nts properly for determination of
;                   maximum string length
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocvlinc
@uio_uiocstinc
;
; --- Name of language ('IDL' or 'WAVE') ---
uio_getenv, langua=langua
;
; --- Get format of output file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
nminitab=0
ncolhead_next=0
nti=0
ntr=0
nts=0
; --- Loop over all mini tables in large table array ---
while (ncolhead_next lt tab.ncol) do begin
  ncolhead_this=ncolhead_next
  ;
  ; --- Search next table header entry ---
  found_flag=0
  while (not found_flag) do begin
    ncolhead_next=ncolhead_next+1
    found_flag=(ncolhead_next eq tab.ncol)
    if (not found_flag) then found_flag=(tab.termt(0,0,ncolhead_next) eq tabna)
  endwhile
  ;
  ; --- Define header ---
  termt=tab.termt(*,*,ncolhead_this)
  ntt=tab.ntta(ncolhead_this)
  nttt=0
  uio_adkey1, termt_tmp,nttt,tabna,               outstr,ierr
  uio_adkey1, termt_tmp,nttt,termt(0,1),         outstr,ierr
  termt(0,0:1)=''
  uio_exkeyw, 1, termt,ntt, dimna,dimstr, outstr,ierr
  uio_dim2st, [[1,ncolhead_next-ncolhead_this-1],[1,tab.nlin]], dimstr
  uio_adkey2, termt_tmp,nttt,dimna, dimstr,       outstr,ierr
  uio_exkeyw, 1, termt,ntt, frmtna,sfrmt, outstr,ierr
  if (sfrmt eq '*') then begin
    sfrmt='1X'
  endif
  uio_adkey2, termt_tmp,nttt,frmtna,sfrmt ,       outstr,ierr
  if (sfrmt eq '') then sfrmt=','
  if (sfrmt ne ',') then sfrmt=',' + sfrmt + ','
  str=string('>', format="(A1" + sfrmt + "'<')")
  lenval=strtrim(string(strlen(str)-2,format='(I6)'),2)
  uio_adkey2, termt_tmp,nttt,bytena,lenval,       outstr,ierr
  uio_exkeyw, 1, termt,ntt, bytena,bytes, outstr,ierr
  uio_exkeyw,13, termt,ntt, datena,date_tmp, outstr,ierr
  if (date_tmp eq '') then begin
    if (n_elements(date) ne 0) then begin
      if (date eq 'now') then begin
;        dat=systime(0)
        dat=strmid(systime(0),0,24)
      endif else begin
        dat=date
      endelse
      uio_adkey2, termt_tmp,nttt,datena,dat   ,       outstr,ierr
    endif
  endif else begin
    uio_adkey2, termt_tmp,nttt,datena,date_tmp,       outstr,ierr
  endelse
  ; --- Concatenate term tables ---
  tab.termt(*,*,ncolhead_this)=termt_tmp
  nttn=(nttt+ntt)<nttmx
  if (nttn gt nttt) then begin
    tab.termt(*,nttt:nttn-1,ncolhead_this)=termt(*,0:nttn-nttt-1)
  endif
  tab.ntta(ncolhead_this)=nttn
  ;
  ; --- Write header of table ---
  uio_wrhd, nc,form, tab.termt(*,*,ncolhead_this),tab.ntta(ncolhead_this), $
    outstr,ierr
  ;
  nts0=nts
  ; --- Write header data for all table entries into file ---
  for i=ncolhead_this+1,ncolhead_next-1 do begin
    termt=tab.termt(*,*,i)
    ntt=tab.ntta(i)
    nttt=0
    type=termt(0,0)
    uio_adkey1, termt_tmp,nttt,type,               outstr,ierr
    uio_adkey1, termt_tmp,nttt,termt(0,1),         outstr,ierr
    termt(0,0:1)=''
    uio_exkeyw, 1, termt,ntt, frmtna,efrmt, outstr,ierr
    uio_exkeyw,13, termt,ntt, bytena,lenval,outstr,ierr
    if (type eq intena) then begin
      ; --- Integer ---
      if ((efrmt eq '') or (efrmt eq '*')) then efrmt=foinls(cvnum(nc-ncmin))
      lenval=btinls(cvnum(nc-ncmin))
    endif else if (type eq realna) then begin
      ; --- Real ---
      if ((efrmt eq '') or (efrmt eq '*')) then efrmt=forels(cvnum(nc-ncmin))
      lenval=btrels(cvnum(nc-ncmin))
    endif else begin
      ; --- String ---
      if (lenval eq '') then begin
        if (tab.nlin gt 0) then begin
          nlen=max(strlen(tab.s(0:tab.nlin-1,nts)))
        endif else begin
          nlen=4
        endelse
        lenval=strtrim(string(nlen),2)
      endif
      if ((efrmt eq '') or (efrmt eq '*')) then begin
        efrmt='A'+lenval
      endif else begin
        ; ?
      endelse
      nts=nts+1
    endelse
    uio_adkey2, termt_tmp,nttt,frmtna,efrmt ,       outstr,ierr
    uio_adkey2, termt_tmp,nttt,bytena,lenval,       outstr,ierr
    ; --- Concatenate term tables ---
    tab.termt(*,*,i)=termt_tmp
    nttn=(nttt+ntt)<nttmx
    if (nttn gt nttt) then begin
      tab.termt(*,nttt:nttn-1,i)=termt(*,0:nttn-nttt-1)
    endif
    tab.ntta(i)=nttn
    ; --- Write header ---
    uio_wrhd, nc,form, tab.termt(*,*,i),tab.ntta(i), outstr,ierr
    ;
    ; --- Compose single header line with entry names and output formt ---
    if (type eq intena) then begin
      str=string(0L , format='('+efrmt+',''<'')')
    endif else if (type eq realna) then begin
      str=string(0.0, format='('+efrmt+',''<'')')
    endif else if (type eq charna) then begin
      str=string(' ', format='('+efrmt+',''<'')')
    endif
    lenval=strtrim(string(strlen(str)-1,format='(I6)'),2)
    if (i eq (ncolhead_this+1)) then begin
      hfrmt='(A' + lenval
      tfrmt='(' + efrmt
      tfrmtarr=strarr(ncolhead_next-ncolhead_this)
      tfrmtarr(i-(ncolhead_this+1))='(' + efrmt + ',$)'
    endif else begin
      hfrmt=hfrmt + sfrmt + 'A' + lenval
      tfrmt=tfrmt + sfrmt + efrmt
      tfrmtarr(i-(ncolhead_this+1))='(' + strmid(sfrmt,1,80) + efrmt + ',$)'
    endelse
  endfor ; i
  nts=nts0
  hfrmt=hfrmt + ')'
  tfrmt=tfrmt + ')'
  ;
  ; --- Write header line to file ---
  if (form eq 'unformatted') then begin
    outun=string(tab.termt(0,1,ncolhead_this+1:ncolhead_next-1), format=hfrmt)
    writeu, nc, outun
  endif else begin
    printf, nc, tab.termt(0,1,ncolhead_this+1:ncolhead_next-1), format=hfrmt
  endelse
  ;
  if (tab.nlin gt 0) then begin
    ; --- Make empty output structure ---
    ;
;    for i=ncolhead_this+1,ncolhead_next-1 do begin
;      type=tab.termt(0,0,i)
;      istr=strtrim(string(i),2)
;      if (i eq (ncolhead_this+1)) then begin
;        if (type eq intena) then begin
;          outabline=create_struct('i'+istr, 0L)
;        endif else if (type eq realna) then begin
;          outabline=create_struct('r'+istr, 0.0)
;        endif else if (type eq charna) then begin
;          outabline=create_struct('s'+istr, '')
;        endif
;      endif else begin
;        if (type eq intena) then begin
;          outabline=create_struct(outabline, 'i'+istr, 0L)
;        endif else if (type eq realna) then begin
;          outabline=create_struct(outabline, 'r'+istr, 0.0)
;        endif else if (type eq charna) then begin
;          outabline=create_struct(outabline, 's'+istr, '')
;        endif
;      endelse
;    endfor
    ;
    if (langua eq 'IDL') then begin
      execute_str="outabline=create_struct("
      for i=ncolhead_this+1,ncolhead_next-1 do begin
        type=tab.termt(0,0,i)
        istr=strtrim(string(i),2)
        if (type eq intena) then begin
          execute_str=execute_str + "'i" + istr + "',0L"
        endif else if (type eq realna) then begin
          execute_str=execute_str + "'r" + istr + "',0.0"
        endif else if (type eq charna) then begin
          execute_str=execute_str + "'s" + istr + "',''"
        endif
        if (i lt (ncolhead_next-1)) then execute_str=execute_str + ","
      endfor
      execute_str=execute_str + ")"
    endif else begin ; --- 'WAVE' ---
      execute_str="outabline={,"
      for i=ncolhead_this+1,ncolhead_next-1 do begin
        type=tab.termt(0,0,i)
        istr=strtrim(string(i),2)
        if (type eq intena) then begin
          execute_str=execute_str + "i" + istr + ":0L"
        endif else if (type eq realna) then begin
          execute_str=execute_str + "r" + istr + ":0.0"
        endif else if (type eq charna) then begin
          execute_str=execute_str + "s" + istr + ":''"
        endif
        if (i lt (ncolhead_next-1)) then execute_str=execute_str + ","
      endfor
      execute_str=execute_str + "}"
    endelse
    ;
;    print,execute_str
    result=execute(execute_str)
    ;
    outab=make_array(tab.nlin, value=outabline)
    ;
    ; --- Fill output structure ---
    for i=ncolhead_this+1,ncolhead_next-1 do begin
      type=tab.termt(0,0,i)
      istr=strtrim(string(i),2)
      if (type eq intena) then begin
        outab(*).(i-(ncolhead_this+1))=tab.i(0:tab.nlin-1,nti)
        nti=nti+1
      endif else if (type eq realna) then begin
        outab(*).(i-(ncolhead_this+1))=tab.r(0:tab.nlin-1,ntr)
        ntr=ntr+1
      endif else if (type eq charna) then begin
        uio_exkeyw, 0, tab.termt(*,*,i),tab.ntta(i), bytena,bytes, outstr,ierr
        pad=string(' ',format='(A'+bytes+')')
        ;reads, bytes, nbytes, format='(I6)'
        nbytes=fix(bytes)
        outab(*).(i-(ncolhead_this+1))= $
                                  strmid(tab.s(0:tab.nlin-1,nts)+pad ,0,nbytes)
        nts=nts+1
      endif
    endfor
    ;
    ; --- Write table into file ---
    if (form eq 'unformatted') then begin
      writeu, nc, outab
    endif else begin
      if (langua eq 'IDL') then begin
        printf, nc, outab, format=tfrmt
      endif else begin ; --- 'WAVE' ---
        for n=0,tab.nlin-1 do begin
          for i=0,ncolhead_next-1-(ncolhead_this+1) do begin
            printf, nc, outab(n).(i), format=tfrmtarr(i)
          endfor
          printf, nc
        endfor
      endelse
    endelse
  endif
endwhile
;
outstr='done'
ierr=0
;
end ; uio_wrtab
