function map2box, command, box
;
;------------------------------------------------------------------------------
;+
; NAME:
;   map2box ('map_2_box')
;
; PURPOSE:
;   Put 2D arrays on the surface sides of a cube
;
; CATEGORY:
;   Array processing
;
; CALLING SEQUENCE:
;   out=map2box(command, box)
;
; INPUT:
;   command: ('command') string with requested quantity (so far only 'int')
;   box:     ('box') data structure with COBOLD mean data (containing axes and intensity maps)
;
; OUTPUT:
;   out:     ('output') 3D array
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   smooth2d: ('smooth_2_dimensions') Smooth image
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   The 2D intensity arrays from the mean structure are copied to the surface
;   layers of a cube (or 'rectangular parallelepiped'). Actually the two outermost
;   layers are filled.
;
; EXAMPLE:
;   ; --- Assume: modelfile is defined; EOS and opacity table are read ---
;
;   ; --- Load data set ---
;   mean=uio_dataset_rd(modelfile)
;
;   ; --- Compute heat capacity ---
;   a=map2box('int', mean)
;
; MODIFICATION HISTORY:
;   27. 5.02 Written by Bernd Freytag, Uppsala
;-
;------------------------------------------------------------------------------
;
;if (not keyword_set(par)) then par=0
;
m1=0
n1=n_elements(box.z1.xc1)-1
m2=0
n2=n_elements(box.z2.xc2)-1
m3=0
n3=n_elements(box.z3.xc3)-1
;
; --- Set number of boundary layers ---
n_bound=20 < ((n1-m1+1)/2) < ((n2-m2+1)/2) < ((n3-m3+1)/2)
;
cm=strlowcase(command)
;
if (cm eq 'int') then begin
  ; --- Map intensity arrays to surface of box ---
  ;
  ; --- Set smoothing parameters ---
;  q_smooth=1
;  q_median=3
  q_smooth=0
  q_median=0
  ;
  ; --- Create empty array ---
  out=fltarr(n1-m1+1, n2-m2+1, n3-m3+1)
  ;
  ; --- Fill boundary layers ---
  for i=n_bound-1,0,-1 do begin
    out (m1+i , m2:n2, m3:n3)=smooth2d(-box.rad.intb1_l, q_smooth=q_smooth, q_median=q_median)
    out (n1-i , m2:n2, m3:n3)=smooth2d( box.rad.intb1_r, q_smooth=q_smooth, q_median=q_median)
    out (m1:n1, m2+i , m3:n3)=smooth2d(-box.rad.intb2_l, q_smooth=q_smooth, q_median=q_median)
    out (m1:n1, n2-i , m3:n3)=smooth2d( box.rad.intb2_r, q_smooth=q_smooth, q_median=q_median)
    out (m1:n1, m2:n2, m3+i )=smooth2d(-box.rad.intb3_l, q_smooth=q_smooth, q_median=q_median)
    out (m1:n1, m2:n2, n3-i )=smooth2d( box.rad.intb3_r, q_smooth=q_smooth, q_median=q_median)
  endfor
  ;
endif else begin
  ; --- Default: zero ---
  ;
  out=fltarr(n1-m1+1, n2-m2+1, n3-m3+1)
  ;
endelse
;
return, out
;
end ; map2box
