;-------------------------------------------------------------------------------
; Vis3D module: Box structure containing input arrays
;-------------------------------------------------------------------------------
; Contains:
;   v3d_box_isvaliddata:     Check the validity of datasets
;   v3d_box_insertisodata:   Insert array as iso dataset in box structure
;   v3d_box_insertcolordata: Insert array as color dataset in box structure, modify color structure
;   v3d_box_destroy:         Free memory used in box structure
;   v3d_box_init:            Initialize box structure
;-------------------------------------------------------------------------------


function v3d_box_isvaliddata, dn
;-------------------------------------------------------------------------------
; NAME:
;   v3d_box_isvaliddata ('v3d_box_isvalid_data')
;
; PURPOSE:
;   Check the validity of datasets. 
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_box_isvaliddata(dn)
;
; INPUT:
;   dn:     ('data_n') 3D dataset array
;
; PROCEDURE:
;   Checks the validity of datasets. 
;   Returns 0 if not valid, 1 otherwise.
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-16 (B.F.) More comments
;   2005-07-26 (B.F.) Put into separate file
;   2005-08-31 (B.F.) Put into v3d_initbox.pro file
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  dns=size(dn, /STRUCTURE)

  ; Check if number of dimensions isn't 3.
  if (dns.n_dimensions ne 3) then begin
    return, 0
  endif
 
  ; The array seems ok
  return, 1

end ; v3d_box_isvaliddata


pro v3d_box_insertisodata, sState, index, d1, NO_COPY=no_copy
;-------------------------------------------------------------------------------
; NAME:
;   v3d_box_insertisodata ('v3d_insert_iso_data')
;
; PURPOSE:
;   Insert input data as iso dataset into box structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_box_insertisodata, sState, index, d1
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-09-11 (B.F.) Put into own routine into box module
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  strindex=string(index+1, format='(I0)')

  if (size(d1, /N_DIMENSIONS) eq 3) then begin
    ; 3D scalar array
    (*sState)[index].box.isodataset=PTR_NEW(d1, NO_COPY=no_copy)
  endif else if (size(d1, /N_DIMENSIONS) ne 4) then begin
    message, 'ERROR: Invalid iso dataset nr. ' + strindex + '.'
  endif else if ((size(d1, /DIMENSIONS))[3] ne 3) then begin
    message, 'ERROR: Invalid iso dataset nr. ' + strindex + '.'
  endif else begin
    (*sState)[index].box.isodataset =PTR_NEW(sqrt(total(d1^2,4)))
    (*sState)[index].box.vecxdataset=PTR_NEW(d1[*,*,*,0])
    (*sState)[index].box.vecydataset=PTR_NEW(d1[*,*,*,1])
    (*sState)[index].box.veczdataset=PTR_NEW(d1[*,*,*,2])
  endelse

  ; Add range of iso-dataset to box structure
  ISO_min=min(*(*sState)[index].box.isodataset, MAX=ISO_max)
  (*sState)[index].box.fullisorange=[ISO_min, ISO_max]

  ; Get dimensions of data
  datadim=size(*(*sState)[index].box.isodataset) 

  ; This procedure can only handle 3D datasets
  if (datadim[0] ne 3) then begin
    message, 'ERROR: Not a 3D array in parameter iso.'
  endif

  ; Put dimension range of array in box structure.
  ; Now the procedure only supports using the array dimensions 
  ; and so all ranges begin with 0 and end with last index value of the array.
  (*sState)[index].box.xrange=double([0L, datadim[1]-1L])
  (*sState)[index].box.yrange=double([0L, datadim[2]-1L])
  (*sState)[index].box.zrange=double([0L, datadim[3]-1L])
  (*sState)[index].box.nx    =datadim[1]
  (*sState)[index].box.ny    =datadim[2]
  (*sState)[index].box.nz    =datadim[3]
  ;(*sState)[index].box.xrange=[-0.5D+00, 0.5D+00]*double(datadim[1]-1L)
  ;(*sState)[index].box.yrange=[-0.5D+00, 0.5D+00]*double(datadim[2]-1L)
  ;(*sState)[index].box.zrange=[-0.5D+00, 0.5D+00]*double(datadim[3]-1L)

end ; v3d_box_insertisodata


pro v3d_box_insertcolordata, sState, index, c1, NO_COPY=no_copy
;-------------------------------------------------------------------------------
; NAME:
;   v3d_box_insertcolordata ('v3d_insert_color_data')
;
; PURPOSE:
;   Insert input data as color dataset into box structure, modify color structure
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_box_insertcolordata, sState, index, d1
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-09-11 (B.F.) Put into own routine into box module
;   2006-06-23 (B.F.) Check if iso and color datasets have same dimensions
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  strindex=string(index+1, format='(I0)')

  if (NOT v3d_box_isvaliddata(c1)) then begin
    if (ARRAY_EQUAL(size(c1, /DIMENSIONS), 3)) then begin
      (*sState)[index].color.solid_color  =c1
      (*sState)[index].color.is_solidcolor=1
    endif else if (size(c1, /N_DIMENSIONS) eq 0) then begin
      if (size(c1, /TYPE) eq size('a', /TYPE))  then begin
        nx=(*sState)[index].box.nx
        ny=(*sState)[index].box.ny
        nz=(*sState)[index].box.nz
        if (c1 eq 'x') then begin
          (*sState)[index].box.colordataset=PTR_NEW(rebin(reform(findgen(nx), nx,1,1), nx,ny,nz))
        endif else if (c1 eq 'y') then begin
          (*sState)[index].box.colordataset=PTR_NEW(rebin(reform(findgen(ny), 1,ny,1), nx,ny,nz))
        endif else begin
          (*sState)[index].box.colordataset=PTR_NEW(rebin(reform(findgen(nz), 1,1,nz), nx,ny,nz))
        endelse
      endif else begin
        (*sState)[index].color.solid_color  =v3dpar_SolidColor_default
        (*sState)[index].color.is_solidcolor=1
      endelse
    endif else begin
      message, 'ERROR: Invalid color dataset nr. ' + strindex + '.'
    endelse
  endif else begin
    ; Put array on the heap, make a copy if keyword NO_COPY is not set
    (*sState)[index].box.colordataset=PTR_NEW(c1, NO_COPY=no_copy) 
  endelse

  if (PTR_VALID((*sState)[index].box.colordataset)) then begin
    ; Get dimensions
    datadim=size(*(*sState)[index].box.colordataset) 
    ;
    ; This procedure can only handle 3D datasets
    if (datadim[0] ne 3) then begin
      message, 'ERROR: Not a 3D array in parameter color.'
    endif
    ;
    if ((datadim[1] ne (*sState)[index].box.nx) or $
        (datadim[2] ne (*sState)[index].box.ny) or $
        (datadim[3] ne (*sState)[index].box.nz)   ) then begin
      message, 'ERROR: Datasets iso and color do not have same dimensions.'
    endif
    ;
    ; Add full range of color-dataset to data3d structure
    COLOR_min=min(*(*sState)[index].box.colordataset, MAX=COLOR_max)
    (*sState)[index].box.fullcolorrange=[COLOR_min, COLOR_max]
  endif

end ; v3d_box_insertcolordata


pro v3d_box_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_box_destroy ('v3d_box_destroy')
;
; PURPOSE:
;   Free memory used in box structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_box_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Free existing heap variables and clean up using a null_pointer.
;
; HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-09-11 (B.F.) Put into own routine
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ptr_free, (*sState)[index].box.isodataset
  (*sState)[index].box.isodataset=ptr_new()  

  ptr_free, (*sState)[index].box.colordataset 
  (*sState)[index].box.colordataset=ptr_new()

  ptr_free, (*sState)[index].box.vecxdataset 
  (*sState)[index].box.vecxdataset=ptr_new()
  ptr_free, (*sState)[index].box.vecydataset 
  (*sState)[index].box.vecydataset=ptr_new()
  ptr_free, (*sState)[index].box.veczdataset 
  (*sState)[index].box.veczdataset=ptr_new()

end ; v3d_box_destroy


function v3d_box_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_box_init ('v3d_box_initialize')
;
; PURPOSE:
;   Initialize box structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   box=v3d_box_init()
;
; MODIFICATION HISTORY:
;   2002-06-12 Written by Sigurdur Finnsson, Uppsala
;   2005-08-15 (B.F.) data3d split into box, color, data3d
;   2005-09-11 (B.F.) v3d_initbox -> v3d_box_init
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  box={$
    isodataset:            PTR_NEW(), $          ; 3D array: data for iso surfaces
    colordataset:          PTR_NEW(), $          ; 3D array: data for colors
    vecxdataset:           PTR_NEW(), $          ; 3D array: x component of vectors
    vecydataset:           PTR_NEW(), $          ; 3D array: y component of vectors
    veczdataset:           PTR_NEW(), $          ; 3D array: z component of vectors
    fullisorange:          [0.0, 0.0], $         ; minmax of isosurface dataset
    fullcolorrange:        [0.0, 0.0], $         ; minmax of color data set
    xrange:                [0.0D+00, 0.0D+00], $ ; range of x axis values
    yrange:                [0.0D+00, 0.0D+00], $ ; range of y axis values
    zrange:                [0.0D+00, 0.0D+00], $ ; range of z axis values
    nx:                    0L, $                 ; Number of elements in x direction
    ny:                    0L, $                 ; Number of elements in y direction
    nz:                    0L $                  ; Number of elements in x direction
      }

  return, box

end ; v3d_box_init
