pro v3d_cbar_updatewidget, oColorbar, dmax, dmin, palette
;-------------------------------------------------------------------------------
; NAME:
;   v3d_cbar_updatewidget ('v3d_colorbar_update_widget')
;
; PURPOSE:
;   Update the colorbar object: palette and tickmarks.
;
; CALLING SEQUENCE:
;   v3d_cbar_updatewidget, oColorbar, dmax, dmin, palette
;
; INPUT:
;   dmax:      ('data_maximum') maximum of physical values which the colorbar should cover
;   dmin:      ('data_minimum') minimum of physical values which the colorbar should cover
;   palette:   ('palette') color palette array or pointer to it
;
; INPUT/OUTPUT:
;   oColorbar: ('object_Colorbar') colorbar object handle
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-13 (B.F.) More comments, keyword 'new' removed, rearrangements
;   2005-07-20 (B.F.) Put into separate file
;   2005-08-02 (B.F.) v3d_update_colorbar -> v3d_updatecolorbar
;   2005-09-13 (B.F.) Put into v3d_cbar module file, v3d_updatecolorbar -> v3d_cbar_updatewidget
;-------------------------------------------------------------------------------

  compile_opt hidden

  ; --- Color palette ---

  ; Palette could be either a pointer or simply an array [3,256]
  if (n_elements(palette) eq 1) then begin
    if (ptr_valid(palette)) then palarr = *palette
  endif else begin
    palarr=palette
  endelse

  ; Update the palette with the given palette 
  oColorbar->SetProperty, RED_VALUES  =palarr[0,*], $
                          GREEN_VALUES=palarr[1,*], $
                          BLUE_VALUES =palarr[2,*]

  ; --- Tickmarks ---

  ; Get handler to current IDLgrText object used for the labels in the colorbar.
  ; n gets the number of major tickmarks used in the object.
  oColorbar->GetProperty, TICKTEXT=oCbarText, MAJOR=n

  np=n_elements(palarr[0,*])

  paletteindex=replicate(0,n)
  values = replicate(double(0.0), n)

  delta_d = dmax - dmin 
 
  ; Convenient constant, the distance between tickmarks (in physical values)
  a = double(delta_d)/double(n-1)

  ; Loop over all major tickmarks
  for i=0, n-1 do begin
    od = a * double(i)  ; The physical value at tickmark i 
    values[i] = od + dmin 
    paletteindex[i] = round(double(np-1) * od/delta_d) ; Palette index for tickm i.
  endfor
  
  ; Update the changed properties of the objects
  oCbarText->SetProperty, STRINGS = string(values, FORMAT='(G0)')
  ;oCbarText->SetProperty, STRINGS = '!D' + string(values, FORMAT='(G0)') +'!N'
  oColorbar->SetProperty, TICKVALUES = paletteindex

end ; v3d_cbar_updatewidget


function v3d_cbar_createwidget, wBase, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_cbar_createwidget ('v3d_colorbar_create_widget')
;
; PURPOSE:
;   Create the colorbar widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_cbar_createwidget(wBase, sState, index)
;
; INPUT:
;   wBase:  ('widget_Base') handle of the base widget
;
; RETURN VALUE:
;   result: handle to the colorbar widget
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-16 (B.F.) More comments
;   2005-07-26 (B.F.) Put into separate file
;   2005-07-29 (B.F.) Keyword renderer added
;   2005-09-13 (B.F.) Put into v3d_cbar module file, create only colorbar widget
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  wColorbar=widget_draw(wBase, $
                        UVALUE         ='COLORBAR', $
                        XSIZE          =v3dpar_colorbar_WindowWidth, $
                        YSIZE          =(*sState)[index].viewysize, $
                        RENDERER       =1, $
                        RETAIN         =0, $
                        GRAPHICS_LEVEL =2, $
                        KEYBOARD_EVENTS=1, $
                        /EXPOSE_EVENTS, $
                        /BUTTON_EVENTS)

  return, wColorbar

end ; v3d_cbar_createwidget
