;-------------------------------------------------------------------------------
; Vis3D module: Color
;-------------------------------------------------------------------------------
; Contains:
;   v3d_color_setpalette:   Recolor all visible objects with new palette
;   v3d_color_loadpalette:  Load palette with specified index
;   v3d_color_set_event:    Handle all color settings events
;   v3d_color_event:        Handle all color events
;   v3d_color_updatewidget: Update the color control widget
;   v3d_color_createwidget: Create the color widget from data in the state variable
;   v3d_color_destroy:      Destroy color structure and free memory used in it.
;   v3d_color_init:         Initialize color structure
;-------------------------------------------------------------------------------


pro v3d_color_displayhelp, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_displayhelp ('v3d_color_display_help')
;
; PURPOSE:
;   Display help text for color-palette widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_color_displayhelp, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-11-16 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Color-palette', $
'=============', $
'', $
'This panel controls the color of iso-surfaces, slices, or volume renders.',$
'', $
'A click on the "Color-palette X" field opens a menu.', $
'The "Settings" entry opens another window where the background color can be set.', $
'The "Palette" entry allows the choice of a new palette (via xloadct).', $
'The "help" entry gives this help text.', $
'', $
'The "Close" button just closes the panel (not the entire window) without', $
'starting anything.', $
'', $
'With the two fields in the second row the minimum and maximum value of the', $
'color dataset array that correspond to the minimum and maximum of the', $
'color palette can be specified. Dataset values outside this range are mapped', $
'to the first or last palette entry.', $
'', $
'In the line below the total minimum and maximum are printed.', $
'The "Full" button uses these values for the current palette range.', $
'', $
'The "Palette" button allows the choice of a new palette (via xloadct).', $
'the "Sync" button exports the current palette to all other views.', $
'', $
'The "Apply" button starts the recoloring of all objects with the new palette.']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' Help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_color_displayhelp


pro v3d_color_setpalette, sState, index, $
                          ALLNEW=allnew
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_color_setpalette ('v3d_color_set_palette')
;
; PURPOSE:
;   Recolor all visible objects with new palette.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_color_setpalette, sState, index, ALLNEW=allnew
;
; INPUT:
;   index:  ('index') integer with view number (index of sState[])
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-17 (B.F.) Call v3d_create_info to update the text
;   2005-07-22 (B.F.) v3d_obj_setpalette -> v3d_setpalette, put into own file
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;   2005-08-16 (B.F.) Call v3d_colorizeiso
;   2005-09-03 (B.F.) Check isCurr???Color, handle dots colors
;   2005-09-22 (B.F.) v3d_setpalette -> v3d_color_setpalette, put into color module file
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  message, 'Check and update colors', /INFORMATIONAL

  if (keyword_set(allnew)) then begin
    ; Invalidate all color data (so they will be recomputed the next time)
    (*sState)[index].isCurrIsoColor  =0
    (*sState)[index].isCurrDotsColor =0
    (*sState)[index].isCurrVolColor  =0
    (*sState)[index].isCurrSliceColor=0
  endif

  (*sState)[index].oPolyHigh->GetProperty, HIDE=hide
  if ((hide eq 0) and ((*sState)[index].isCurrIsoColor eq 0)) then begin
    ; Compute new iso colors
    ;
    v3d_iso_computevertcolors, sState, index
    ;
    (*sState)[index].oPolyhigh->SetProperty, VERT_COLORS=*(*sState)[index].iso.vert_colors
    (*sState)[index].oPoly    ->SetProperty, VERT_COLORS=*(*sState)[index].iso.vert_colors
  endif

  (*sState)[index].oDots->GetProperty, HIDE=hide
  if ((hide eq 0) and ((*sState)[index].isCurrDotsColor eq 0)) then begin
    ; Compute new dots colors
    ;
    v3d_dots_computevertcolors, sState, index
    ;
    (*sState)[index].oDots->SetProperty, VERT_COLORS=*(*sState)[index].dots.vert_colors
  endif

  if (obj_valid((*sState)[index].oVol)) then begin
    (*sState)[index].oVol->GetProperty, HIDE=hide
    if ((hide eq 0) and ((*sState)[index].isCurrVolColor eq 0)) then begin
      ; Compute new vol colors
      ;
      v3d_vol_createobj, sState, index
    endif
  endif

  (*sState)[index].oSlice->GetProperty, HIDE=hide
  if ((hide eq 0) and ((*sState)[index].isCurrSliceColor eq 0)) then begin
    ; Compute new slice colors
    ;
    v3d_slice_createobj, sState, index
  endif

  (*sState)[index].oWindow->draw, (*sState)[index].oView

end ; v3d_color_setpalette


pro v3d_color_loadpalette, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_loadpalette ('v3d_color_load_palette')
;
; PURPOSE:
;   Load palette with specified index.
;
; CALLING SEQUENCE:
;   v3d_color_loadpalette, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;   Loads one of the predefined palettes in IDL and puts into the color data structure.
;
; HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, 'v3d_findtag' checks removed
;   2006-07-19 (B.F.) Check if color table file exists in working directory;
;                     moved check for number of palettes
;   2010-03-23 (B.F.) Explicit default path to color table
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  COMMON COLORS, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr

  @v3d_parameters

  table=(*sState)[index].color.idlpal

  colorfile=(file_search(v3dpar_ColorTablePath_default + '/*colors1.tbl'))[0]
  if (colorfile ne '') then begin
    loadct, table, $
            FILE=colorfile
  endif else begin
    if (table gt 41) then begin 
      message, 'ERROR: This color table doesn''t exist in IDL.'
    endif
    ;
    loadct, table
  endelse
 
  PTR_FREE, (*sState)[index].color.palette
  (*sState)[index].color.palette=PTR_NEW(temporary([transpose(r_curr), $
                                                    transpose(g_curr), $
                                                    transpose(b_curr)]))

  (*sState)[index].isCurrIsoColor  =0
  (*sState)[index].isCurrDotsColor =0
  (*sState)[index].isCurrVolColor  =0
  (*sState)[index].isCurrSliceColor=0

end ; v3d_color_loadpalette


pro v3d_color_set_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_set_event ('v3d_color_settings_event')
;
; PURPOSE:
;   Handle all color settings events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2006-06-27 (B.F.) Adopted from v3d_stream_set_event
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

;  message, '', /INFORMATIONAL
;  help, /str, sEvent
;  help, sState, index, uval

end ; v3d_color_set_event


function v3d_color_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_event ('v3d_color_event')
;
; PURPOSE:
;   Handle all color events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-08-22 (B.F.) Taken from v3d_dataset_event
;   2006-06-27 (B.F.) Handle additional settings widget
;   2006-07-06 (B.F.) Check for custom color palette file
;   2010-03-23 (B.F.) Explicit default path to color table
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ;message, '', /INFORMATIONAL
  ;help, /str, sEvent
  ;help, sState, index, uval

  if (uval eq 'BUTTON_COLOR_FULL') then begin ; --------------------------------
    (*sState)[index].color.palette_top   =(*sState)[index].box.fullcolorrange[1]
    (*sState)[index].color.palette_bottom=(*sState)[index].box.fullcolorrange[0]
    ;
    v3d_color_setpalette, sState, index, /ALLNEW
    ;
    if ((*sState)[index].no_colorbar eq 0) then begin
      v3d_cbar_updatewidget, (*sState)[index].oColorbar, $
                             (*sState)[index].color.palette_top, $
                             (*sState)[index].color.palette_bottom, $
                             (*sState)[index].color.palette
      ;
      (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
    endif
    ;
    ; And update the textfields
    v3d_color_updatewidget, sState, index
  endif else $

  if (uval eq 'BUTTON_COLOR_PALETTE') then begin ; -----------------------------
    ; The Change Colors button which brings up the palette chooser
    ;
    COMMON COLORS, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
    ;
    pal=(*sState)[index].color.palette
    ;
    ; The transposing is necessary, otherwise IDL complains about
    ; mismatching of array types.
    r_orig=transpose((*pal)[0,*])
    g_orig=transpose((*pal)[1,*])
    b_orig=transpose((*pal)[2,*])
    ;
    tvlct, transpose(*pal) 
    ;
    r_curr=transpose((*pal)[0,*])
    g_curr=transpose((*pal)[1,*])
    b_curr=transpose((*pal)[2,*])
    ;
    colorfile=(file_search(v3dpar_ColorTablePath_default + '/*colors1.tbl'))[0]
    if (colorfile ne '') then begin
      xloadct, GROUP=(*sState)[index].wColorControl, $
               /MODAL, $
               FILE=colorfile
    endif else begin
      xloadct, GROUP=(*sState)[index].wColorControl, $
               /MODAL
    endelse
    ;
    PTR_FREE, (*sState)[index].color.palette
    (*sState)[index].color.palette=PTR_NEW([transpose(r_curr), $
                                            transpose(g_curr), $
                                            transpose(b_curr)] )
    ;
    v3d_color_setpalette, sState, index, /ALLNEW
    ;
    if ((*sState)[index].no_colorbar eq 0) then begin
      ; Update the colorbar object
      v3d_cbar_updatewidget, (*sState)[index].oColorbar , $
                             (*sState)[index].color.palette_top, $
                             (*sState)[index].color.palette_bottom, $
                             (*sState)[index].color.palette
      ; And redraw the colorbar
      (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
    endif
  endif else $

  if (uval eq 'BUTTON_COLOR_SYNC') then begin ; --------------------------------
    ; Copy current palette and range to all other views
    ;
    ; Copy palette, palette range to all other datasets, skip the current dataset.
    for i=0, n_elements(*sState)-1 do begin
      if (i ne index) then begin 
        ;
        ; Free the old palette array and copy the current one 
        ptr_free, (*sState)[i].color.palette
        (*sState)[i].color.palette=ptr_new(*((*sState)[index].color.palette))
        ;
        ; Copy the palette ranges 
        (*sState)[i].color.palette_top   =(*sState)[index].color.palette_top
        (*sState)[i].color.palette_bottom=(*sState)[index].color.palette_bottom
        ;
        ; Update the object in the other views
        v3d_color_setpalette, sState, i, /ALLNEW
        ;
        if ((*sState)[index].no_colorbar eq 0) then begin
          v3d_cbar_updatewidget, (*sState)[i].oColorbar, $
                                 (*sState)[i].color.palette_top, $
                                 (*sState)[i].color.palette_bottom, $
                                 (*sState)[i].color.palette
          (*sState)[i].oCbarDrawable->Draw, (*sState)[i].oCbarView
        endif
        ;
        ; Update the text fields in CHCOLORS widget
        v3d_color_updatewidget, sState, i
      endif
    endfor ; i
  endif else $

  if (uval eq 'BUTTON_COLOR_APPLY') then begin ; -------------------------------
    ; User presses button for displaying the new color range on object
    ; 
    widget_control, (*sState)[index].wPalMin, $
                    GET_VALUE=strPalMin
    widget_control, (*sState)[index].wPalMax, $
                    GET_VALUE=strPalMax
    (*sState)[index].color.palette_bottom=float(strPalMin)
    (*sState)[index].color.palette_top   =float(strPalMax)
    ;
    v3d_color_setpalette, sState, index, /ALLNEW
    ;
    if ((*sState)[index].no_colorbar eq 0) then begin
        v3d_cbar_updatewidget, (*sState)[index].oColorbar, $
                               (*sState)[index].color.palette_top, $
                               (*sState)[index].color.palette_bottom, $
                               (*sState)[index].color.palette
      ;
      (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
    endif
  endif else $ ; 'BUTTON_COLOR_APPLY'

  if (uval eq 'BUTTON_COLOR_HELP') then begin ; --------------------------------
    ; Show help
    ;
    v3d_color_displayhelp, group=(*sState)[index].wDraw
    ;
  endif else $ ; 'BUTTON_COLOR_HELP'

  if (uval eq 'BUTTON_COLOR_SETTINGS') then begin ; ----------------------------
    ; Open additional settings panel
    ;
    v3d_color_createsetwidget, sState, index
    ;
  endif else $ ; 'BUTTON_COLOR_SETTINGS'

  if (uval eq 'BUTTON_COLOR_CLOSE') then begin ; -------------------------------
    ; Close widget without any further action
    ;
    (*sState)[index].hideControlColor=1
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif else $ ; 'BUTTON_COLOR_CLOSE'

  if (uval eq 'BUTTON_COLORSET_APPLY') then begin ; ----------------------------
    ; Apply changes from additional settings widget
    ;
    widget_control, (*sState)[index].wColorRed,   GET_VALUE=strred
    widget_control, (*sState)[index].wColorGreen, GET_VALUE=strgreen
    widget_control, (*sState)[index].wColorBlue,  GET_VALUE=strblue
    ;
    view_color=byte(0L > long([strred[0], strgreen[0], strblue[0]]) < 255L)
    (*sState)[index].oView->SetProperty, $
                            COLOR=view_color
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_COLORSET_APPLY'

  if (uval eq 'BUTTON_COLORSET_CLOSE') then begin ; ----------------------------
    ; Close widget without any further action
    ;
    widget_control, (*sState)[index].wColorSet, /DESTROY
    ;
  endif ; 'BUTTON_COLORSET_CLOSE'

return, 1

end ; v3d_color_event


pro v3d_color_createsetwidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_createsetwidget ('v3d_color_create_settings_widget')
;
; PURPOSE:
;   Create the color settings widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_color_createsetwidget(sState, index)
;
; INPUT:
;   index:       ('index') current view index
;
; INPUT/OUTPUT:
;   sState:      ('structure_State') The state variable array
;
; RETURN VALUE:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-09-22 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if (not widget_info((*sState)[index].wColorSet, /VALID_ID)) then begin

    strindex=string(index+1, format='(I0)')
    ;
    wBaseSet=widget_base(TITLE='Vis3D color ' + strindex, $
                         GROUP=(*sState)[0].wBase, $
                         /ROW, $
                         XPAD=0, YPAD=0)
    (*sState)[index].wColorSet=wBaseSet
      ;
      wBaseButtons=widget_base(wBaseSet, $
                                                 UVALUE=index, $
                               EVENT_FUNC='v3d_color_event', $
                               /COLUMN, $
                               XPAD=0, YPAD=0)
        wTopRow=widget_base(wBaseButtons, $
                            /ROW, $
                            XPAD=0, YPAD=0)
          wEntry=widget_label(wTopRow,           VALUE ='Color ' + strindex + $
                              '                                  ')
          wEntry=widget_button(wTopRow,          VALUE ='Close', $
                                                 UVALUE='BUTTON_COLORSET_CLOSE', $
                               TOOLTIP='Close this window without further action')

        ;
        wCenterFrame=widget_base(wBaseButtons, $
                                 /COLUMN, $
                                 XPAD=0, YPAD=2)
          wLine1=widget_base(    wCenterFrame, $
                                 /ROW, $
                                 XPAD=0, YPAD=0)
            (*sState)[index].oView->GetProperty, COLOR=view_color
            str_red  =string(view_color[0], format='(I0)')
            str_green=string(view_color[1], format='(I0)')
            str_blue =string(view_color[2], format='(I0)')
            wEntry=widget_label( wLine1,          VALUE ='Background: RED:')
            wEntry=widget_text(  wLine1,          VALUE=str_red, $
                                                  UVALUE='TEXT_COLORSET_RED', $
                                 /EDITABLE, $
                                 XSIZE=3)
            (*sState)[index].wColorRed=wEntry
            wEntry=widget_label( wLine1,          VALUE ='GREEN:')
            wEntry=widget_text(  wLine1,          VALUE=str_green, $
                                                  UVALUE='TEXT_COLORSET_GREEN', $
                                 /EDITABLE, $
                                 XSIZE=3)
            (*sState)[index].wColorGreen=wEntry
            wEntry=widget_label( wLine1,          VALUE ='BLUE:')
            wEntry=widget_text(  wLine1,          VALUE=str_blue, $
                                                  UVALUE='TEXT_COLORSET_BLUE', $
                                 /EDITABLE, $
                                 XSIZE=3)
            (*sState)[index].wColorBlue=wEntry
        ;
        wBottomRow=widget_base(wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
          wEntry=widget_button(wBottomRow,        VALUE ='Apply', $
                                                  UVALUE='BUTTON_COLORSET_APPLY', $
                               TOOLTIP='Apply changes in settings')
    ;
    geometry1=widget_info((*sState)[index].wBaseControl,   /GEOMETRY)
    geometry2=widget_info((*sState)[index].wBaseFields,    /GEOMETRY)
    geometry3=widget_info((*sState)[index].wColorControl,  /GEOMETRY)
    widget_control, wBaseSet, $
                    XOFFSET=geometry1.xoffset+geometry2.xoffset+geometry3.xoffset, $
                    YOFFSET=geometry1.yoffset+geometry2.yoffset+geometry3.yoffset+geometry3.ysize+30

    ;
    widget_control, wBaseSet, /REALIZE
    widget_control, wBaseSet, SET_UVALUE=sState
    xmanager, 'vis3d', wBaseSet, /NO_BLOCK, EVENT_HANDLER='v3d_color_set_event'

  endif

end ; v3d_color_createsetwidget


pro v3d_color_updatewidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_updatewidget ('v3d_color_update_widget')
;
; PURPOSE:
;   Update the color control widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_color_updatewidget, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2005-08-22 (B.F.) Put into separate file and adapted to new widget structure
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ;widget_control, (*sState)[index].wColorControl, $
  ;                SET_VALUE={TOP:   (*sState)[index].color.palette_top, $
  ;                           BOTTOM:(*sState)[index].color.palette_bottom}

  if ((*sState)[index].hideControlColor eq 0) then begin
    widget_control, (*sState)[index].wPalMin, $
                    SET_VALUE=string((*sState)[index].color.palette_bottom, FORMAT='(G0)')
    widget_control, (*sState)[index].wPalMax, $
                    SET_VALUE=string((*sState)[index].color.palette_top,    FORMAT='(G0)')
  endif

end ; v3d_color_updatewidget


function v3d_color_createwidget, wBaseColor, sState, index
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_color_createwidget ('v3d_color_create_widget')
;
; PURPOSE:
;   Create the color widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_color_createwidget(wBaseColor, sState, index, parent=parent)
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState:  ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Widget handle
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-08-22 (B.F.) Modified menu put into separate routine
;   2005-11-16 (B.F.) Color menu
;-
;-------------------------------------------------------------------------------

  compile_opt hidden

  strindex=string(index+1, format='(I0)')
  ;
  wBaseButtons=widget_base(wBaseColor, $
                           /COLUMN, /FRAME, $
                           UVALUE=index, $
                           EVENT_FUNC='v3d_color_event', $
                           XPAD=2, YPAD=2)
    wTopRow=widget_base(     wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      wMenu=widget_button(     wTopRow,           VALUE ='Color-palette ' + strindex, $
                                                  UVALUE='BUTTON_COLOR_MENU', $
                               /MENU, $
                               TOOLTIP='Show color menu')
        wEntry=widget_button(  wMenu,              VALUE ='Settings', $
                                                   UVALUE='BUTTON_COLOR_SETTINGS')
        wEntry=widget_button(  wMenu,              VALUE ='Palette', $
                                                   UVALUE='BUTTON_COLOR_PALETTE')
        wEntry=widget_button(  wMenu,              VALUE='Help', $
                                                   UVALUE='BUTTON_COLOR_HELP')
      wEntry=widget_label(     wTopRow,           VALUE ='   ')
      wEntry=widget_button(    wTopRow,           VALUE ='Close', $
                               /ALIGN_CENTER,     UVALUE='BUTTON_COLOR_CLOSE', $
                               TOOLTIP='Close this panel without further action')
    ;
    wCenterFrame=widget_base(wBaseButtons, $
                             /COLUMN, $
                             XPAD=0, YPAD=2)
      wLine1=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
;        wEntry=widget_label(     wLine1,          VALUE ='Range:')
        (*sState)[index].wPalMin=widget_text(wLine1,VALUE =string((*sState)[index].color.palette_bottom, $
                                                                  FORMAT='(G0)'), $
                                 /ALIGN_LEFT,     UVALUE='TEXT_COLOR_PALMIN', $
                                 /EDITABLE, $
                                 XSIZE=12)
        (*sState)[index].wPalMax=widget_text(wLine1,VALUE =string((*sState)[index].color.palette_top, $
                                                                  FORMAT='(G0)'), $
                                 /ALIGN_LEFT,     UVALUE='TEXT_COLOR_PALMAX', $
                                 /EDITABLE, $
                                 XSIZE=12)
      ;
      wLine2=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine2,          VALUE =' ', $
                                 /ALIGN_LEFT)
        wEntry=widget_label(     wLine2,          VALUE = $
                                  string((*sState)[index].box.fullcolorrange[0], FORMAT='(G0)') + ' .. ' + $
                                  string((*sState)[index].box.fullcolorrange[1], FORMAT='(G0)'), $
                                 /ALIGN_LEFT)
      ;
      wLine3=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_button(    wLine3,          VALUE ='Full', $
                                                  UVALUE='BUTTON_COLOR_FULL', $
                                 TOOLTIP='Use entire data array range for palette')
        wEntry=widget_button(    wLine3,           VALUE ='Palette', $
                                                   UVALUE='BUTTON_COLOR_PALETTE', $
                                 TOOLTIP='Choose palette with xloadct')
        wEntry=widget_button(    wLine3,           VALUE ='Sync', $
                                                   UVALUE='BUTTON_COLOR_SYNC', $
                                 TOOLTIP='Synchronize colors: export this palette to other views')
    ;
    wBottomRow=widget_base(  wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
                                                  UVALUE='BUTTON_COLOR_APPLY', $
                               TOOLTIP='Apply new color palette (range)')

  return, wBaseButtons

end ; v3d_color_createwidget


pro v3d_color_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_color_destroy ('v3d_color_destroy')
;
; PURPOSE:
;   Destroy color structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_color_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Free existing heap variables and clean up using a null_pointer.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;
; HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-09-11 (B.F.) Put into own routine
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ptr_free, (*sState)[index].color.palette 
  (*sState)[index].color.palette=ptr_new()

end ; v3d_color_destroy


function v3d_color_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_color_init ('v3d_color_initialize')
;
; PURPOSE:
;   Initialize color structure.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   color=v3d_color_init()
;
; MODIFICATION HISTORY:
;   2002-06-12 Written by Sigurdur Finnsson, Uppsala
;   2005-08-15 (B.F.) data3d split into box, color, data3d
;-
;-------------------------------------------------------------------------------

  compile_opt hidden

  @v3d_parameters

  color={$
    idlpal:                v3dpar_Palette_default, $
    palette:               PTR_NEW(), $
    is_solidcolor:         0, $
    solid_color:           v3dpar_SolidColor_default, $
    view_color:            v3dpar_ViewBackgroundColor_default, $
    palette_top:           0.0, $             ; Color dataset value corresponding to palette maximum
    palette_bottom:        0.0 $              ; Color dataset value corresponding to palette minimum
        }

  return, color

end ; v3d_color_init
