pro v3d_createobjects, sState, EXPORT_ON=export_on
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_createobjects ('v3d_create_objects')
;
; PURPOSE:
;   Create object hierarchies for all views.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_createobjects, sState, EXPORT_ON=export_on
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; KEYWORDS:
;   export_on: ('export_on') Activate export mode
;
; PROCEDURE:
;   Object tree (one for each view panel):
;     oView            ; The 3D graphics object
;       oWorld
;         oLight1      ; Light source 1 (optional)
;         oLight2      ; Light source 2 (optional)
;         oModel
;           oPoly      ; Low resolution polygon surface
;           oPolyHigh  ; High resolution polygon surface
;           oDots      ; Dot cloud
;           oStream    ; Streamlines
;           oSlice     ; 2D slices
;           oBox       ; Box edges
;           oUser      ; User provided object (low resolution, optional)
;           oUserHigh  ; User provided object (high resolution, optional)
;           oVol       ; Volume rendering object
;       oTrack         ; Trackball
;       oCBarView      ; Color bar
;         oColorbar
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-08-24 (B.F.) Put into separate routine and file
;   2005-09-01 (B.F.) Object oDots added
;   2005-09-04 (B.F.) oHolder object removed
;   2006-06-07 (B.F.) Object oVol added
;   2007-04-03 (B.F. Lyon) Font and size put into parameter file
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  nrofviews=n_elements(*sState)

  ; Loop over all views
  for i=0, nrofviews-1 do begin
    ; --- Create the object hierarchy to display the graphics ------------------
    ;
    ; --- View object ---
    ;
    ; Create the view object
    (*sState)[i].oView=OBJ_NEW('IDLgrView', $
                               color=(*sState)[i].color.view_color, $
                               projection=(*sState)[i].projection)
    ;
    ; Set the view volume properties right for the dataset
    v3d_getviewvolume, (*sState)[i].box, (*sState)[i].oView, $
                       SCALE=(*sState)[i].scale, $
                       XSIZE=(*sState)[i].viewxsize, YSIZE=(*sState)[i].viewysize
    ;
    ; Create model object which will contain the 3D model and lights
    (*sState)[i].oWorld=OBJ_NEW('IDLgrModel')
    (*sState)[i].oView->Add, (*sState)[i].oWorld
    ;
    ; --- Light sources ---
    v3d_light_createobj, sState, i
    (*sState)[i].oWorld->Add, (*sState)[i].oLight1
    (*sState)[i].oWorld->Add, (*sState)[i].oLight2
    ;
    ; --- Model object ---
    ;
    ; The model object to contain graphics.
    (*sState)[i].oModel=OBJ_NEW('IDLgrModel')
    (*sState)[i].oWorld->Add, (*sState)[i].oModel
    ;
    ; Set the rotation matrix passed as parameter.
    (*sState)[i].oModel->SetProperty, TRANSFORM=(*sState)[i].rotatematrix
    ;
    ; Pre-rotate the model by amount specified by parameters.
    ; Center is needed for translation.
    (*sState)[i].center=[0.5D+00*((*sState)[i].box.xrange[0]+(*sState)[i].box.xrange[1]), $
                         0.5D+00*((*sState)[i].box.yrange[0]+(*sState)[i].box.yrange[1]), $
                         0.5D+00*((*sState)[i].box.zrange[0]+(*sState)[i].box.zrange[1])]
    (*sState)[i].oModel->Translate, -(*sState)[i].center[0], $
                                    -(*sState)[i].center[1], $
                                    -(*sState)[i].center[2]
    (*sState)[i].oModel->Rotate, (*sState)[i].rotateaxis, (*sState)[i].rotateangle
    (*sState)[i].oModel->Translate,  (*sState)[i].center[0], $
                                     (*sState)[i].center[1], $
                                     (*sState)[i].center[2]
    ;
    ; --- Iso surface ---
    ;
    ; Generate 3D mesh
    v3d_iso_generate, sState, i, $
                      EXPORT_ON=export_on
    ;
    ; Create 3D polygon object in full resolution.
    (*sState)[i].oPolyHigh=v3d_iso_createpolyobj(sState, i, HIDE=v3dpar_Iso_hide)
    (*sState)[i].oModel->Add, (*sState)[i].oPolyHigh
    ;
    if (NOT KEYWORD_SET(export_on)) then begin
      ; Create the 3D polygon object (use the low resolution object here) 
      (*sState)[i].oPoly=v3d_iso_createpolyobj(sState, i, /LOW, /HIDE)
      (*sState)[i].oModel->Add, (*sState)[i].oPoly
    endif
    ;
    ; --- Volume rendering with dots ---
    (*sState)[i].dots.nvertices=v3dpar_Dots_Number_default
    v3d_dots_createvertcoords,  sState, i
    v3d_dots_computevertvalues, sState, i
    v3d_dots_computevertcolors, sState, i
    (*sState)[i].oModel->Add, (*sState)[i].oDots
    ;
    ; --- Streamlines ---
    if (PTR_VALID((*sState)[i].box.vecxdataset)) then begin
      (*sState)[i].stream.nlines =v3dpar_Stream_NLines_default
      (*sState)[i].stream.npoints=v3dpar_Stream_NPoints_default
      vecmax=max(abs((*sState)[i].box.fullisorange))
      if (vecmax eq 0.0) then vecmax=1.0
      (*sState)[i].stream.step=1.0/vecmax
      v3d_stream_createvertcoords,  sState, i
      ;v3d_stream_computevertvalues, sState, i
      ;v3d_stream_computevertcolors, sState, i
      (*sState)[i].oModel->Add, (*sState)[i].oStream
    endif
    ;
    ; --- Slices ---
    v3d_slice_createobj, sState, i
    (*sState)[i].oModel->Add, (*sState)[i].oSlice
    ;
    ; --- Box/axes ---
    ;
    ; Create the 3D boundary box around the dataset.
    v3d_axis_createobj, sState, i
    (*sState)[i].oModel->Add, (*sState)[i].oBox
    ; (*sState)[i].oModel->Add, (*sState)[i].oBoxEdges
    ;
    ; --- User-defined objects ---
    ;
    ; Insert user-defined objects, if there are any
    if (OBJ_VALID((*sState)[i].oUserHigh)) then (*sState)[i].oModel->Add, (*sState)[i].oUserHigh
    if (OBJ_VALID((*sState)[i].oUser    )) then (*sState)[i].oModel->Add, (*sState)[i].oUser
    ;
    ; --- Volume ---
    if ((*sState)[i].box.fullisorange[1] gt 0.0) then begin
      (*sState)[i].vol.scale=1.0D+00/(*sState)[i].box.fullisorange[1]
    endif
    v3d_vol_createobj, sState, i
    (*sState)[i].oModel->Add, (*sState)[i].oVol
    ;
    ; --- Trackball ---
    ;
    if (NOT KEYWORD_SET(export_on)) then begin
      ; Trackball is not needed in Export mode
      ;
      ; Data used for trackball object in both datasets
      trackCenterX=round((*sState)[i].viewxsize/2.)
      trackCenterY=round((*sState)[i].viewysize/2.)
      trackRadius=sqrt(trackcenterX^2+trackcenterY^2)
      ;
      ; Create the trackball object to stear the 3D object and add it to holder
      (*sState)[i].oTrack=OBJ_NEW('Trackball', [trackCenterX, trackCenterY], trackRadius)
    endif
    ;
    ; --- Colorbar ---
    ;
    ; Note, coordinates are assuming the view window of the 
    ; colorbar has width=1.0 and height=1.0
    if ((*sState)[i].no_colorbar eq 0) then begin
      (*sState)[i].oCbarView=OBJ_NEW('IDLgrView', $
                                     COLOR=v3dpar_colorbar_BackgroundColor, $
                                     VIEWPLANE_RECT=[0.0, 0.0, 1.0, 1.0])
      ;
      (*sState)[i].oCbarText=OBJ_NEW('IDLgrText', replicate('', v3dpar_colorbar_Major), $
      ;                               /ENABLE_FORMATTING, $
                                     /ONGLASS)
      oCbarFont=OBJ_NEW('IDLgrFont',v3dpar_colorbar_Font,size=v3dpar_colorbar_FontSize)
      ;oCbarFont=OBJ_NEW('IDLgrFont','Helvetica',size=7)
      ;oCbarFont=OBJ_NEW('IDLgrFont','Hershey*3',size=5)
      (*sState)[i].oCbarText->SetProperty, FONT=oCbarFont
      ;
      (*sState)[i].oColorbar=OBJ_NEW('IDLgrColorbar', $
                        DIMENSIONS=  [v3dpar_colorbar_WindowWidth, (*sState)[i].viewysize], $
                        SHOW_AXIS=   2, $
                        /SHOW_OUTLINE, $
                        TICKTEXT=    (*sState)[i].oCbarText, $
                        SUBTICKLEN=  v3dpar_colorbar_SubTickLen, $
                        TICKLEN=     v3dpar_colorbar_TickLen,$
                        MAJOR=       v3dpar_colorbar_Major, $
                        MINOR=       v3dpar_colorbar_Minor, $
                        XCOORD_CONV=[v3dpar_colorbar_LeftMargin, $
                                     v3dpar_colorbar_Width/double(v3dpar_colorbar_WindowWidth)], $
                        YCOORD_CONV=[(1.0-v3dpar_colorbar_Height)/2.0, $
                                     v3dpar_colorbar_Height/double((*sState)[i].viewysize)], $
                        COLOR=       v3dpar_colorbar_Color)
      ;
      ; Put current palette and physical data ranges to colorbar 
;      if ((*sState)[i].color.is_solidcolor eq 0) then begin
        v3d_cbar_updatewidget, (*sState)[i].oColorbar, (*sState)[i].color.palette_top, $
                                                       (*sState)[i].color.palette_bottom, $
                                                       (*sState)[i].color.palette
;      endif else begin
;        v3d_cbar_updatewidget, (*sState)[i].oColorbar, 0.0, 0.0, (*sState)[i].color.palette
;      endelse
      ;
      (*sState)[i].oCbarView->Add, (*sState)[i].oColorbar
      ;
    endif ; End of: if ((*sState)[i].no_colorbar eq 0)
    ;
  endfor ; Loop over all views

end ; v3d_createobjects
