;-------------------------------------------------------------------------------
; Vis3D module: Volume rendering with dots
;-------------------------------------------------------------------------------
; Contains:
;   v3d_dots_displayhelp:       Display help text for dots generation widget
;   v3d_dots_createvertcoords:  Generate random dot coordinates with propability from isodataset
;   v3d_dots_computevertvalues: Interpolate the vertex values from the vertex coordinates
;   v3d_dots_computevertcolors: Set the color values in the dots structure
;   v3d_dots_event:             Handle all dots events
;   v3d_dots_updatewidget:      Update the dots control widget
;   v3d_dots_createwidget:      Create the dots widget
;   v3d_dots_destroy:           Destroy dots structure and free memory used in it
;   v3d_dots_init:              Initialize dots structure
;-------------------------------------------------------------------------------


pro v3d_dots_displayhelp, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_displayhelp ('v3d_dots_display_help')
;
; PURPOSE:
;   Display help text for dots generation widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_dots_displayhelp, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-08-31 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Dot generation', $
'==============', $
'', $
'This panel controls the volume rendering via particles (dots).', $
'It employs kind of a Monte-Carlo radiation transport. It is fast.', $
'However, the output quality is not sufficient for all purposes.', $
'', $
'A click on the "Dots X" field opens a menu with a help entry,', $
'which gives this help text.', $
'', $
'The "Hide" or "Show" button toggles the visibility of the dot cloud.', $
'', $
'The "Close" button just closes the panel (not the entire window) without', $
'starting a generation process.', $
'', $
'The "Min" field sets a lower limit to the iso dataset values.', $
'The "Max" field sets an upper limit to the iso dataset values.', $
'It can be used to avoid wasting too many points in regions that are optically', $
'thick (and invisible) anyway.', $
'', $
'The "Num" field specifies the number of dots to use.', $
'', $
'The "Apply" button starts the dot cloud generation (and keeps the window open).']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_dots_displayhelp


pro v3d_dots_createvertcoords, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_createvertcoords ('v3d_dots_create_vertex_coordinates')
;
; PURPOSE:
;   Generate random dot coordinates with propability proportional to isodataset.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_dots_createvertcoords, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2005-09-01 (B.F. LA) Written
;   2010-06-01 (B.F. Freiburg) Double precision operations
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Save system time at start of routine
  time0=systime(1)

  ; Clean up
  if (ptr_valid((*sState)[index].dots.vertices)) then begin
    ptr_free, (*sState)[index].dots.vertices
  endif

  ; Determine number of dots and elements of dataset
  ndots   =(*sState)[index].dots.nvertices
  ndataset=n_elements(*(*sState)[index].box.isodataset)
  nvec    =size(*(*sState)[index].box.isodataset, /DIMENSION)

  ; Create scalar index positions proportional to isodataset density
;  dataset_cum  =total(reform(*(*sState)[index].box.isodataset > 0.0, ndataset), /CUMULATIVE)
;  dataset_cum  =total(reform(0.0 > *(*sState)[index].box.isodataset < (*sState)[index].dots.CurrIsoMax, $
;                             ndataset), /DOUBLE, /CUMULATIVE)
  if ((*sState)[index].dots.CurrIsoMin le 0.0) then begin
    dataset_cum  =total(reform(0.0 > *(*sState)[index].box.isodataset < (*sState)[index].dots.CurrIsoMax, $
                              ndataset), /DOUBLE, /CUMULATIVE)
  endif else  begin
    isotmp=*(*sState)[index].box.isodataset
    isotmp[where(isotmp le (*sState)[index].dots.CurrIsoMin)]=0.0
    dataset_cum  =total(reform(0.0 > isotmp < (*sState)[index].dots.CurrIsoMax, $
                               ndataset), /DOUBLE, /CUMULATIVE)
    isotmp=0.0
  endelse
  random_ampl  =dataset_cum[ndataset-1L]
  random_values=random_ampl*randomu(seed, ndots)
  random_pos   =interpol(dindgen(ndataset), dataset_cum, random_values)
  random_index =long(random_pos)
print,minmax(random_index)
  ; Create vector index positions proportional to isodataset density
  random_vecind=lonarr(3, ndots)
  factor=1L
  for idim=0, 2 do begin
help, idim, factor, nvec[idim]
    random_vecind[idim, *]=(random_index / factor) mod nvec[idim]
    factor=factor*nvec[idim]
help, factor
  endfor ; idim

  ; Add small random numbers to distribute dots within cells
  random_vecpos=float(random_vecind)
  random_vecpos=random_vecpos + randomu(seed, 3, ndots)-0.5

  ; Put result into state structure
  (*sState)[index].dots.vertices=ptr_new(temporary(random_vecpos))

  ; Modify or create graphics object
  if (obj_valid((*sState)[index].oDots)) then begin
    (*sState)[index].oDots->SetProperty, DATA=*(*sState)[index].dots.vertices
  endif else begin
    (*sState)[index].oDots=obj_new('IDLgrPolygon', $
                                   *(*sState)[index].dots.vertices, $
                                   COLOR=[255B, 255B, 255B], $
                                   STYLE=0, $
                                   HIDE=v3dpar_Dots_hide)
  endelse

  message, 'Routine needed ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL

end ; v3d_dots_createvertcoords


pro v3d_dots_computevertvalues, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_computevertvalues ('v3d_dots_compute_vertex_values')
;
; PURPOSE:
;   Interpolate the vertex values from the vertex coordinates.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_dots_computevertvalues, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; MODIFICATION HISTORY:
;   2002-06-10 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, 'v3d_findtag' checks removed
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;   2005-09-01 (B.F.) Adapted to dots structure
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ; If vertex data in dots is zeroed, then make a valid, empty array of
  ; vertex values and skip interpolating. 
  if ((*sState)[index].dots.nvertices le 0L) then begin
    message, 'WARNING: No dots defined', /INFORMATIONAL 
    vert_values=0.0
  endif else if (PTR_VALID((*sState)[index].box.colordataset)) then begin
    vert_values=interpolate( *(*sState)[index].box.colordataset, $
                            (*(*sState)[index].dots.vertices)[0,*], $
                            (*(*sState)[index].dots.vertices)[1,*], $
                            (*(*sState)[index].dots.vertices)[2,*] )
  endif else begin
    vert_values=(*(*sState)[index].dots.vertices)[0,*]*0.0
  endelse

  ; Put vert_values in dots structure: update the pointer value.
  PTR_FREE, (*sState)[index].dots.vert_values
  (*sState)[index].dots.vert_values=PTR_NEW(temporary(vert_values))

end ; v3d_dots_computevertvalues


pro v3d_dots_computevertcolors, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_computevertcolors ('v3d_dots_compute_vertex_colors')
;
; PURPOSE:
;   Set the color values in the dots structure.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_dots_computevertcolors, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; HISTORY:
;   2002-06-13 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, 'v3d_findtag' checks removed,
;                     'v3d_addtag' calls replaced
;   2005-07-18 (B.F.) Account for zero colorrange
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;   2005-08-16 (B.F.) Keywords bottom, top removed,
;                     faster (vectorized) color interpolation
;   2005-09-01 (B.F.) Adapted to dots structure
;-------------------------------------------------------------------------------

  compile_opt hidden

  ;message, 'Scale the color values', /INFORMATIONAL

  if ((*sState)[index].color.is_solidcolor ne 1) then begin
    ;
    top   =(*sState)[index].color.palette_top
    bottom=(*sState)[index].color.palette_bottom
    ;
    ; The range of physical values which the palette will be mapped to
    delta=top-bottom
    ;
    if (delta eq 0.0) then begin
      ; The color range is zero.
      ;
      ; Create an array which will contain all computed colors
      vert_colors=make_array(3, (*sState)[index].dots.nvertices, /byte)
      ;
      ; Use first color in palette for all vertices
      vert_colors[0,*]=(*(*sState)[index].color.palette)[0,0]
      vert_colors[1,*]=(*(*sState)[index].color.palette)[1,0]
      vert_colors[2,*]=(*(*sState)[index].color.palette)[2,0]
      ;
    endif else begin
      ; There is a non-zero color range
      ;
      ; How many entries are in current palette?
      n_pal=(size(*(*sState)[index].color.palette))[2]
      ;
      time0=systime(1)
      c=0L > $
        (round((float(n_pal-1L)/delta) * (*(*sState)[index].dots.vert_values-bottom))) < $
        (n_pal-1L)
      vert_colors=(*(*sState)[index].color.palette)[*,c]
      ;
      message, 'Vertex coloring took ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL
    endelse
    ;
    (*sState)[index].oDots->SetProperty, VERT_COLORS=vert_colors
    ;
  endif else begin
    ; Put one dummy white color in structure
    vert_colors=(*sState)[index].color.solid_color
    ;
    (*sState)[index].oDots->SetProperty, COLOR=vert_colors
    ;
  endelse

  ; Put vert_colors in dots structure: update the pointer value.
  PTR_FREE, (*sState)[index].dots.vert_colors
  (*sState)[index].dots.vert_colors=PTR_NEW(temporary(vert_colors))

  (*sState)[index].isCurrDotsColor=1

end ; v3d_dots_computevertcolors


function v3d_dots_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_event ('v3d_dots_event')
;
; PURPOSE:
;   Handle all dots events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-08-12 (B.F.) Taken from v3d_dataset_event
;   2005-11-21 (B.F.) Help from menu under tool button
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ;message, '', /INFORMATIONAL
  ;help, /str, sEvent
  ;help, sState, index, uval

  if (uval eq 'BUTTON_DOTS_APPLY') then begin ; --------------------------------
    ; Fetch new value for maximum density
    widget_control, (*sState)[index].wDotsCurrIsoMin, $
                    GET_VALUE=strDotsCurrIsoMin
    (*sState)[index].dots.CurrIsoMin=0.0 > float(strDotsCurrIsoMin)
    widget_control, (*sState)[index].wDotsCurrIsoMax, $
                    GET_VALUE=strDotsCurrIsoMax
    (*sState)[index].dots.CurrIsoMax=0.0 > float(strDotsCurrIsoMax)
    ;
    ; Fetch new value for number of dots
    widget_control, (*sState)[index].wDotsNumber, $
                    GET_VALUE=strDotsNumber
    (*sState)[index].dots.nvertices=1L > long(strDotsNumber) < 100000000L
    ;
    ; Generate new vertex coordinates
    v3d_dots_createvertcoords, sState, index
    ;
    ; Compute vertex values
    v3d_dots_computevertvalues, sState, index
    ;
    ; Compute vertex colors
    v3d_dots_computevertcolors, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_DOTS_APPLY'

  if (uval eq 'BUTTON_DOTS_SHOWHIDEOBJ') then begin ; --------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oDots->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oDots->SetProperty, HIDE=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wDotsShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wDotsShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_DOTS_SHOWHIDEOBJ'

  if (uval eq 'BUTTON_DOTS_HELP') then begin ; ---------------------------------
    ; Show help
    ;
    v3d_dots_displayhelp, group=(*sState)[index].wDraw
    ;
  endif else $ ; 'BUTTON_DOTS_HELP'

  if (uval eq 'BUTTON_DOTS_CLOSE') then begin ; --------------------------------
    ; Close widget without any further action
    ;
    (*sState)[index].hideControlDots=1
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif ; 'BUTTON_DOTS_CLOSE'

  return, 1

end ; v3d_dots_event


pro v3d_dots_updatewidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_updatewidget ('v3d_dots_update_widget')
;
; PURPOSE:
;   Update the dots control widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_dots_updatewidget, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2005-09-01 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if ((*sState)[index].hideControlDots eq 0) then begin
    ;
    ; Display minimum density value
    widget_control, (*sState)[index].wDotsCurrIsoMin  , $
                    SET_VALUE=string((*sState)[index].dots.CurrIsoMin, FORMAT='(G0)')
    ;
    ; Display maximum density value
    widget_control, (*sState)[index].wDotsCurrIsoMax  , $
                    SET_VALUE=string((*sState)[index].dots.CurrIsoMax, FORMAT='(G0)')
    ;
    ; Display number of dots
    widget_control, (*sState)[index].wDotsNumber  , $
                    SET_VALUE=string((*sState)[index].dots.nvertices, FORMAT='(I0)')
    ;
    ; Use either "Show" or "Hide" button
    (*sState)[index].oDots->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wDotsShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wDotsShowHide, $
                      SET_VALUE='Hide'
    endelse
    ;
  endif

end ; v3d_dots_updatewidget


function v3d_dots_createwidget, wBaseDots, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_createwidget ('v3d_dots_create_widget')
;
; PURPOSE:
;   Create the dots widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_dots_createwidget(wBaseDots, sState, index)
;
; INPUT:
;   wBaseDots: ('widget_Base_Dots') parent widget
;   index:     ('index') current view index
;
; INPUT/OUTPUT:
;   sState:    ('structure_State') The state variable array
;
; RETURN VALUE:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-08-31 (B.F.) Written
;   2005-11-21 (B.F.) Help from menu under tool button
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  strindex=string(index+1, FORMAT='(I0)')

  ;wBaseOut=widget_base(wBaseDots, $
  ;                         /ROW, $
  ;                         XPAD=0, YPAD=0)

  wBaseButtons=widget_base(wBaseDots, $
                           /COLUMN, /FRAME, $
                           UVALUE=index, $
                           EVENT_FUNC='v3d_dots_event', $
                           XPAD=2, YPAD=2)
    ;
    wTopRow=widget_base(     wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
;      wEntry=widget_label(     wTopRow,           VALUE ='Dots ' + strindex)
      wMenu=widget_button(     wTopRow,           VALUE ='Dots ' + strindex, $
                                                  UVALUE='BUTTON_DOTS_MENU', $
                               /MENU, $
                               TOOLTIP='Show dots menu')
;        wEntry=widget_button(  wMenu,             VALUE='Settings', $
;                                                  UVALUE='BUTTON_DOTS_SETTINGS')
;        wEntry=widget_button(  wMenu,             VALUE='Control', $
;                                                  UVALUE='BUTTON_DOTS_CONTROL')
        wEntry=widget_button(  wMenu,             VALUE='Help', $
                                                  UVALUE='BUTTON_DOTS_HELP')
      wEntry=widget_button(    wTopRow,  VALUE ='Hide', $
                               /ALIGN_RIGHT,      UVALUE='BUTTON_DOTS_SHOWHIDEOBJ', $
                               TOOLTIP='Show/hide dots object')
      (*sState)[index].wDotsShowHide=wEntry
      wEntry=widget_button(    wTopRow,           VALUE ='Close', $
                               /ALIGN_CENTER,     UVALUE='BUTTON_DOTS_CLOSE', $
                               TOOLTIP='Close this panel without further action')
    ;
    wCenterFrame=widget_base(wBaseButtons, $
                             /COLUMN, $
                             XPAD=0, YPAD=2)
      ;
      wLine0=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine0,          VALUE ='Min:', $
                                 /ALIGN_BOTTOM)
        (*sState)[index].wDotsCurrIsoMin=widget_text(wLine0,  $
                                 VALUE =strtrim(string((*sState)[index].dots.CurrIsoMin, FORMAT='(G0)'),2), $
                                 /ALIGN_LEFT, UVALUE='TEXT_DOTS_CURRISOMIN', $
                                 /EDITABLE, $
                                 XSIZE=12)
      ;
      wLine1=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine1,          VALUE ='Max:', $
                                 /ALIGN_BOTTOM)
        (*sState)[index].wDotsCurrIsoMax=widget_text(wLine1,  $
                                 VALUE =strtrim(string((*sState)[index].dots.CurrIsoMax, FORMAT='(G0)'),2), $
                                 /ALIGN_LEFT, UVALUE='TEXT_DOTS_CURRISOMAX', $
                                 /EDITABLE, $
                                 XSIZE=12)
      ;
      wLine2=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine2,          VALUE ='Num:')
        (*sState)[index].wDotsNumber=widget_text(wLine2,  VALUE =strtrim(string((*sState)[index].dots.nvertices),2), $
                                 /ALIGN_LEFT, UVALUE='TEXT_DOTS_NUMBER', $
                                 /EDITABLE, $
                                 XSIZE=12)
   ;
    wBottomRow=widget_base(  wBaseButtons, $
                             /ROW, $
                             /ALIGN_LEFT, $
                             XPAD=0, YPAD=0)
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
                               /ALIGN_CENTER,     UVALUE='BUTTON_DOTS_APPLY', $
                               TOOLTIP='Render with new number of dots')
  ;

  return, wBaseButtons

end ; v3d_dots_createwidget


pro v3d_dots_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_dots_destroy ('v3d_dots_destroy')
;
; PURPOSE:
;   Destroy dots structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_dots_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Free existing heap variables and clean up using a null_pointer.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;
; HISTORY:
;   2005-08-31 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ptr_free, (*sState)[index].dots.vertices
  (*sState)[index].dots.vertices=ptr_new()

  ptr_free, (*sState)[index].dots.vert_values 
  (*sState)[index].dots.vert_values=ptr_new()

  ptr_free, (*sState)[index].dots.vert_colors 
  (*sState)[index].dots.vert_colors=ptr_new()

end ; v3d_dots_destroy


function v3d_dots_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_dots_init ('v3d_dots_initialize')
;
; PURPOSE:
;   Initialize dots structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   dots=v3d_dots_init()
;
; MODIFICATION HISTORY:
;   2005-08-31 (B.F. LA) Written
;   2010-06-01 (B.F. Freiburg) CurrIsoMin added
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ; Create an anonymous structure containing the basic data for the dots rendering
  dots={$
    CurrIsoMin:            0.0D+00, $              ; Minimum density value
    CurrIsoMax:            1.0D+38, $              ; Maximum density value
    vertices:              PTR_NEW(), $            ; List of vertex coordinates
    vert_values:           PTR_NEW(), $            ; List of vertex values
    vert_colors:           PTR_NEW(), $            ; List of vertex colors
    nvertices:             0L $                    ; Number of active vertices
       }

  return, dots

end ; v3d_dots_init
