pro v3d_getviewvolume, box, oView, $
                       SCALE=scale, $
                       XSIZE=xsize, YSIZE=ysize, $
                       ONLYVIEWPLANE=onlyviewplane
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_getviewvolume ('v3d_get_view_volume')
;
; PURPOSE:
;   Set view so eye looks at dataset.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_getviewvolume, box, oView, SCALE=scale
;
; HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments
;   2005-07-19 (B.F.) Larger eye distance to account for projection:perspective
;   2005-07-29 (B.F.) Keywords xsize and ysize added to preserve aspect ratio
;              of object, use double precision consistently
;   2005-08-09 (B.F.) Keyword onlyviewplane added
;-
;-------------------------------------------------------------------------------

compile_opt hidden

; Constant which enlarges (or decreases) the view box
if (NOT KEYWORD_SET(scale)) then begin
  scalecalc=1.0D+00
endif else begin
  scalecalc=double(scale)
endelse

; Aspect ratio
if (NOT (KEYWORD_SET(xsize) and KEYWORD_SET(ysize))) then begin
  aspect=1.0D+00
endif else begin
  aspect=double(ysize)/double(xsize)
endelse

; limit the scale to be above 0.001, otherwize division by zero can occur.
scalecalc=scalecalc > 0.001D+00

; reverse the scale for the calculations.
; scale is inverse proportional for user in order to
; be more logical (higher values enlarge the object).
scalecalc= 1.0D+00 / scalecalc

; Some variables for more convenient typing
d3xmin=box.xrange[0]
d3xmax=box.xrange[1]
d3ymin=box.yrange[0]
d3ymax=box.yrange[1]
d3zmin=box.zrange[0]
d3zmax=box.zrange[1]
d3_dx=d3xmax-d3xmin
d3_dy=d3ymax-d3ymin
d3_dz=d3zmax-d3zmin

; Coordinate of center (for rotating)
center_x=d3xmin + d3_dx/2.0D+00
center_y=d3ymin + d3_dy/2.0D+00
center_z=d3zmin + d3_dz/2.0D+00

; Size of view rectangle and zclip

; Maximum distance in 3D data cube *0.5.
; because d3_dx is the width of the cube, but I want distance to center 
; so I half each d3_dx etc etc
d3_maxdist=0.5D+00*sqrt(d3_dx^2 + d3_dy^2 + d3_dz^2)

; Lower left corner and maximum width and length of view rectangle
if (aspect lt 1.0) then begin
  vv_xpos=center_x -     (scalecalc*d3_maxdist)/aspect
  vv_w   =           2.0*(scalecalc*d3_maxdist)/aspect
  vv_ypos=center_y -     (scalecalc*d3_maxdist)
  vv_l   =           2.0*(scalecalc*d3_maxdist)
endif else begin
  vv_xpos=center_x -     (scalecalc*d3_maxdist)
  vv_w   =           2.0*(scalecalc*d3_maxdist)
  vv_ypos=center_y -     (scalecalc*d3_maxdist)*aspect
  vv_l   =           2.0*(scalecalc*d3_maxdist)*aspect
endelse

; The near and far clipping planes
; No need to move far and near point with vr_scale
vv_zfar =center_z -         d3_maxdist
vv_znear=vv_zfar  + 2.0D+00*d3_maxdist
;vv_zfar =center_z - 2.0D+00*d3_maxdist
;vv_znear=vv_zfar  + 3.0D+00*d3_maxdist

; Eye distance. The eye is outside the boundaries
vv_eye  =vv_znear + 8.0D+00*d3_maxdist

;print, vv_xpos, vv_ypos, vv_w, vv_l, xsize, ysize, aspect

; Set the properties of the oView object
if (KEYWORD_SET(onlyviewplane)) then begin
  oView->SetProperty, VIEWPLANE_RECT=[vv_xpos, vv_ypos, vv_w, vv_l]
endif else begin
  oView->SetProperty, VIEWPLANE_RECT=[vv_xpos, vv_ypos, vv_w, vv_l], $
                      ZCLIP         =[vv_znear, vv_zfar], $
;                      DEPTH_CUE     =[vv_zfar, vv_znear], $
                      EYE           =vv_eye
endelse

end ; v3d_getviewvolume
