;-------------------------------------------------------------------------------
; Vis3D module: Iso-surface
;-------------------------------------------------------------------------------
; Contains:
;   v3d_iso_displayhelp:        Display help text for iso-surface mesh generation widget
;   v3d_iso_createvertcoords:   Create 3D isosurface (vertex coordinates and polygons) from 3D array
;   v3d_iso_computevertvalues:  Interpolate the vertex values from the vertex coordinates
;   v3d_iso_smoothvertvalues:   Smooth the values of all vertices
;   v3d_iso_smoothmesh:         Smooth the isosurfaces
;   v3d_iso_simplifymesh:       Simplify the 3D mesh by reducing the number of polygons
;   v3d_iso_computevertcolors:  Scale the color values on the mesh
;   v3d_iso_generate:           Generate the complete isosurface data
;   v3d_iso_createpolyobj:      Create a polygon object from the isosurface data
;   v3d_iso_regenerate:         Regenerate the 3D mesh
;   v3d_iso_event:              Handle all iso events
;   v3d_iso_updatewidget:       Update the iso control widget
;   v3d_iso_createwidget:       Create the iso widget from data in the state variable
;   v3d_iso_destroy:            Destroy iso structure and free memory used in it
;   v3d_iso_init:               Initialize iso structure
;-------------------------------------------------------------------------------


pro v3d_iso_displayhelp, GROUP=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_displayhelp ('v3d_iso_display_help')
;
; PURPOSE:
;   Display help text for iso-surface mesh generation widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_displayhelp, GROUP=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-08-17 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Iso-surface generation', $
'======================', $
'', $
'This panel controls the generation of an iso-surface, in particular the', $
'iso-value and the coarseness of the mesh.', $
'', $
'A click on the "Isosurf X" field opens a menu with allows to print some mesh', $
'statistics. Additionally, there is a help entry which gives this help text.', $
'', $
'The "Hide" or "Show" button toggles the visibility of the iso-surface.', $
'', $
'The "Close" button just closes the panel (not the entire window) without', $
'starting a mesh generation process.', $
'', $
'The "Iso" field contains the level of the iso-surface. It''s value should', $
'lie within the minimum and maximum value of the iso data array (shown directly', $
'below the text widget).', $
'With "-" and "+" the iso value can be reduced or increased. The new iso surface', $
'is displayed immediately (and the decimation value is set to 100 %).', $
'', $
'The "Dec" field specifies the ratio of vertices of the simplified', $
'mesh compared to the original one in percent. A value of "100 %" means', $
'no decimation and the use of the original mesh during rotation and static', $
'views. The decimation process itself is very time consuming. However, the', $
'decimated mesh can be displayed very quickly.', $
'', $
'The "Apply" button starts the mesh generation (and keeps the window open).', $
'', $
'The "Apply+Color" button does the same and adjusts the color table to the new', $
'iso-surface.', $
'', $
'The "Color" button does only the color table adjustment.']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_iso_displayhelp


pro v3d_iso_createvertcoords, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_createvertcoords ('v3d_isosurface_create_vertex_coordinates')
;
; PURPOSE:
;   Create 3D isosurface from 3D data.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_iso_createvertcoords, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-08 (B.F.) Some rearrangements
;   2005-07-14 (B.F.) 'v3d_findtag' checks removed, 'v3d_addtag' calls replaced
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;   2005-08-24 (B.F.) Determination of xyzrange moved to another place
;-------------------------------------------------------------------------------

  compile_opt hidden

  time0=systime(1)

  shade_volume, *(*sState)[index].box.isodataset, (*sState)[index].iso.curriso, $
                vertices, polygons, /LOW, /VERBOSE

  message, 'shade_volume needed ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL
   
  ; Check if shade_volume could create mesh-data. If not then arrays will
  ; not be defined, define the arrays with empty values
  if (n_elements(vertices) eq 0) then begin
    vertices=[0,0,0]
    message, 'WARNING: No vertices created.', /INFORMATIONAL
  endif 

  if (n_elements(polygons) eq 0) then begin
    ; No polygons created, terminate list
    polygons=[-1]
    message, 'WARNING: No polygons created.', /INFORMATIONAL
  endif

  ; Replace the older arrays 'polygons' and 'vertices' in the heap
  ; with the new arrays and replace pointers in iso.

  ; Free the old polygon array from the heap
  PTR_FREE, (*sState)[index].iso.polygons
  ; Put the new polygon array in place
  (*sState)[index].iso.polygons=PTR_NEW(temporary(polygons))

  ; Free the old vertices array from the heap
  PTR_FREE, (*sState)[index].iso.vertices
  ; Put the new vertices array in place
  (*sState)[index].iso.vertices=PTR_NEW(temporary(vertices))

end ; v3d_iso_createvertcoords


pro v3d_iso_computevertvalues, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_computevertvalues ('v3d_isosurface_compute_vertex_values')
;
; PURPOSE:
;   Interpolate the vertex values from the vertex coordinates.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_computevertvalues, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; MODIFICATION HISTORY:
;   2002-06-10 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, 'v3d_findtag' checks removed
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ; If polygonal or vertex data in iso is zeroed, then make a valid, empty array of
  ; vertex values and skip interpolating. 
  if (((*(*sState)[index].iso.polygons)[0] eq -1) AND $
      array_equal((*(*sState)[index].iso.vertices)[*,0], [0,0,0])) then begin
    message, 'WARNING: No mesh object is defined', /INFORMATIONAL 
    vert_values=0
  endif else begin
    vert_values=interpolate( *(*sState)[index].box.colordataset, $
                            (*(*sState)[index].iso.vertices)[0,*], $
                            (*(*sState)[index].iso.vertices)[1,*], $
                            (*(*sState)[index].iso.vertices)[2,*] )
  endelse

  ; Find minimum and maximum of the physical values at isosurface
  currcolorrangeMin=min(vert_values, MAX=currcolorrangeMax)

  (*sState)[index].iso.currcolorrange=[currcolorrangeMin, currcolorrangeMax]

  ; Put vert_values in iso structure: update the pointer value.
  PTR_FREE, (*sState)[index].iso.vert_values
  (*sState)[index].iso.vert_values=PTR_NEW(temporary(vert_values))

end ; v3d_iso_computevertvalues


pro v3d_iso_smoothvertvalues, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_smoothvertvalues ('v3d_iso_smooth_vertex_values')
;
; PURPOSE:
;   Smooth the values of all vertices.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;    v3d_iso_smoothvertvalues, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2002-06-15 Written by Sigurdur Finnsson, Uppsala
;   2005-07-19 (B.F.) More comments
;   2005-08-03 (B.F.) v3d_vertcolorsmooth -> v3d_smoothvertcolor
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;-------------------------------------------------------------------------------

  compile_opt hidden

  if (N_ELEMENTS(*(*sState)[index].iso.vert_values) eq 0) then begin
    message, 'ERROR: No vertex values to smooth in iso'
  endif

  vertval_org=reform(*(*sState)[index].iso.vert_values, $
                     1, n_elements(*(*sState)[index].iso.vert_values)) 
  vertvalues=[vertval_org, vertval_org, vertval_org]

  smooth_values=mesh_smooth(vertvalues, *(*sState)[index].iso.polygons, $
                            LAMBDA    =(*sState)[index].iso.colorsmooth_lambda, $
                            ITERATIONS=(*sState)[index].iso.colorsmooth_iterations)

  ptr_free, (*sState)[index].iso.vert_values
  (*sState)[index].iso.vert_values=ptr_new(reform(smooth_values[0,*]))

end ; v3d_iso_smoothvertvalues


pro v3d_iso_smoothmesh, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_smoothmesh ('v3d_iso_smooth_mesh')
;
; PURPOSE:
;   Smooth the isosurfaces.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;    v3d_iso_smoothmesh, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;   Since it only smoothes the vertices in iso, then this
;   affects both the highres 3D mesh and the lowres one because
;   both share a common vertex table, but only the highres polygonal
;   data is used.
;
; MODIFICATION HISTORY:
;   2002-06-11 Written by Sigurdur Finnsson, Uppsala
;   2005-07-19 (B.F.) More comments
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if (NOT ARRAY_EQUAL(size(*(*sState)[index].iso.vertices, /DIMENSIONS), [3])) then begin 
    smooth_vert=mesh_smooth(*(*sState)[index].iso.vertices, $
                            *(*sState)[index].iso.polygons, $
                            LAMBDA    =(*sState)[index].iso.smooth_lambda, $
                            ITERATIONS=(*sState)[index].iso.smooth_iterations)
  endif else begin
    message, 'WARNING: No polygons to smooth', /INFORMATIONAL
    smooth_vert=[0,0,0]
  endelse
  
  ptr_free, (*sState)[index].iso.vertices
  (*sState)[index].iso.vertices=ptr_new(temporary(smooth_vert))

end ; v3d_iso_smoothmesh


pro v3d_iso_simplifymesh, sState, index, LOW=low
;------------------------------------------------------------------------------
; NAME:
;   v3d_iso_simplifymesh ('v3d_iso_simplify_mesh')
;
; PURPOSE:
;   Simplify the 3D mesh by reducing the number of polygons.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_simplifymesh, sState, index, LOW=low
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; KEYWORDS:
;   low:        ('low') If set the simplifed mesh is added in the iso structure
;
; HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) 'v3d_findtag' checks removed, 'v3d_addtag' calls replaced
;   2005-07-28 (B.F.) Print timing statistics
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;------------------------------------------------------------------------------

  compile_opt hidden

  if ((*(*sState)[index].iso.polygons)[0] eq -1 AND $
      array_equal((*(*sState)[index].iso.vertices)[*,0],[0,0,0])) then begin
    message, 'WARNING: Empty 3D mesh object', /INFORMATIONAL
    ; Create an empty polygon object
    new_poly=[-1]
  endif else begin
    ; Decimating mesh. The function returns the number of triangles.
    if ((*sState)[index].iso.decim ne 100) then begin
      message, 'Number of triangles: ' + $
               strtrim(string(mesh_numtriangles(*(*sState)[index].iso.polygons))), $
               /informational
      time0=systime(1)
      n_triangles=mesh_decimate(*(*sState)[index].iso.vertices, $
                                *(*sState)[index].iso.polygons, $
                                new_poly, $
                                percent_vertices=(*sState)[index].iso.decim)
      message, 'Number of triangles: ' + strtrim(string(n_triangles)) + $
               ' after ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL
      ; Terminate the polygon list
      new_poly=[temporary(new_poly),-1]
    endif else begin
      message, 'Decimation level 100%, skip decimating mesh', /INFORMATIONAL
      new_poly=*(*sState)[index].iso.polygons
    endelse
  endelse

  ; Check if the simplified mesh will replace the original or being added to the structure
  if (NOT KEYWORD_SET(low)) then begin
    ; Free the old heap variable
    PTR_FREE, (*sState)[index].iso.polygons
    (*sState)[index].iso.polygons=PTR_NEW(temporary(new_poly))
  endif else begin
    if (PTR_VALID((*sState)[index].iso.polygons_low)) then begin 
      PTR_FREE, (*sState)[index].iso.polygons_low
      (*sState)[index].iso.polygons_low=PTR_NEW(temporary(new_poly))
    endif else begin
      (*sState)[index].iso.polygons_low=PTR_NEW(temporary(new_poly))
    endelse
  endelse
   
end ; v3d_iso_simplifymesh


pro v3d_iso_computevertcolors, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_computevertcolors ('v3d_iso_compute_vertex_colors')
;
; PURPOSE:
;   Scale the color values in the iso structure.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_iso_computevertcolors, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; HISTORY:
;   2002-06-13 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, 'v3d_findtag' checks removed,
;                     'v3d_addtag' calls replaced
;   2005-07-18 (B.F.) Account for zero colorrange
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;   2005-08-16 (B.F.) Keywords bottom, top removed,
;                     faster (vectorized) color interpolation
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ;@v3d_parameters

  ;message, 'Scale the color values', /INFORMATIONAL

  ; Put one dummy white color in structure
  vert_colors=(*sState)[index].color.solid_color

  if ((*sState)[index].color.is_solidcolor ne 1) then begin
   if (n_elements(*(*sState)[index].iso.vert_values) gt 1) then begin
    ;
    top   =(*sState)[index].color.palette_top
    bottom=(*sState)[index].color.palette_bottom
    ;
    ; The range of physical values which the palette will be mapped to
    delta=top-bottom
    ;
    if (delta eq 0.0) then begin
      ; The color range is zero.
      ;
      n_vertvalues=n_elements(*(*sState)[index].iso.vert_values)
      ;
      ; Create an array which will contain all computed colors
      vert_colors=make_array(3, n_vertvalues, /byte)
      ;
      ; Use first color in palette for all vertices
      vert_colors[0,*]=(*(*sState)[index].color.palette)[0,0]
      vert_colors[1,*]=(*(*sState)[index].color.palette)[1,0]
      vert_colors[2,*]=(*(*sState)[index].color.palette)[2,0]
      ;
    endif else begin
      ; There is a non-zero color range
      ;
      ; How many entries are in current palette?
      n_pal=(size(*(*sState)[index].color.palette))[2]
      ;
      time0=systime(1)
      c=0L > $
        (round((float(n_pal-1L)/delta) * (*(*sState)[index].iso.vert_values-bottom))) < $
        (n_pal-1L)
      vert_colors=(*(*sState)[index].color.palette)[*,c]
      ;
      message, 'Vertex coloring took ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL
    endelse
    ;
   endif else begin 
    message, 'WARNING: Only one physical value in array, probably empty 3D mesh.', /INFORMATIONAL
   endelse
  endif

  ; Put vert_colors in iso structure: update the pointer value.
  PTR_FREE, (*sState)[index].iso.vert_colors
  (*sState)[index].iso.vert_colors=PTR_NEW(temporary(vert_colors))

  (*sState)[index].isCurrIsoColor=1

end ; v3d_iso_computevertcolors


pro v3d_iso_generate, sState, index, $
                      EXPORT_ON=export_on
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_generate ('v3d_isosurface_generate')
;
; PURPOSE:
;   Generate the complete isosurface data.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_generate, sState, index, $
;                     EXPORT_ON=export_on
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; KEYWORDS:
;   export_on:    ('export_on')
;
; ROUTINES:
;   v3d_iso_createvertcoords:  Create 3D isosurface (vertex coordinates and polygons) from 3D array
;   v3d_iso_computevertvalues: Interpolate the vertex values from the vertex coordinates
;   v3d_iso_smoothvertvalues:  Smooth the values of all vertices
;   v3d_iso_smoothmesh:        Smooth the surfaces in iso
;   v3d_iso_simplifymesh:      Simplify the 3D mesh by reducing the number of polygons
;   v3d_iso_computevertcolors: Scale the color values in the iso structure
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-13 (B.F.) More comments
;   2005-07-22 (B.F.) Put into separate file
;   2005-08-06 (B.F.) Show hourglass-shaped cursor
;   2005-08-15 (B.F.) Redesign: act on sState directly
;   2005-08-16 (B.F.) Keyword paletterange removed
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  time0=systime(1)

  widget_control, /HOURGLASS

  message, 'Generating 3D isosurface.', /INFORMATIONAL
  ; Create 3D isosurface
  v3d_iso_createvertcoords, sState, index

  message, 'Calculating physical quantities at each vertex.', /INFORMATIONAL
  ; Get physical values of each vertex in mesh3d
  if ((*sState)[index].color.is_solidcolor ne 1) then begin
    ;
    v3d_iso_computevertvalues, sState, index
    ;
    ; Smooth the colors of the surface
    if ((*sState)[index].iso.colorsmooth_on eq 1) then begin
      message, 'Smoothing vertex values', /INFORMATIONAL
      v3d_iso_smoothvertvalues, sState, index
    endif
  endif else begin
    (*sState)[index].iso.currcolorrange=[0.0, 0.0] 
    (*sState)[index].iso.vert_values   =PTR_NEW()
  endelse

  if ((*sState)[index].iso.smooth_on eq 1) then begin 
    message, 'Smoothing the 3D mesh data.', /INFORMATIONAL
    v3d_iso_smoothmesh, sState, index
  endif

  ; Decimating (or simplifying) the 3D mesh data
  if (NOT KEYWORD_SET(export_on)) then begin
    message, 'Decimating 3D mesh data.', /INFORMATIONAL
    v3d_iso_simplifymesh, sState, index, /LOW
  endif

  if ((*sState)[index].color.is_solidcolor ne 1) then begin 
    message, 'Creating color table for each vertex.', /INFORMATIONAL
    ;
    ; If bottom and top both equals zero, then full range should be used
    if (((*sState)[index].color.palette_bottom eq 0) AND $
        ((*sState)[index].color.palette_top    eq 0)    ) then begin
      (*sState)[index].color.palette_bottom=(*sState)[index].iso.currcolorrange[0]
      (*sState)[index].color.palette_top   =(*sState)[index].iso.currcolorrange[1]
    endif 
    ;
    ; Converting the physical values to RGB color values for each vertex,
    v3d_iso_computevertcolors, sState, index
  endif

  message, 'Procedure finished after ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL
  ; DONE, now (*sState)[index].iso contains the generated surface 

end ; v3d_iso_generate


function v3d_iso_createpolyobj, sState, index, SHADING=shading, LOW=low, HIDE=hide
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_createpolyobj ('v3d_iso_create_polygon_object')
;
; PURPOSE:
;   Create a polygon object from the isosurface data.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   oPoly=v3d_iso_createpolyobj(sState, index, SHADING=shading, LOW=low, HIDE=hide)
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; OUTPUT:
;   result: handle to polygon object
;
; KEYWORDS:
;   shading: ('shading') select shading
;   low:     ('low')     select low resolution polygon representation
;   hide:    ('hide')    select hidden object
;
; PROCEDURE:
;   This simple routine puts the vertex coordinates, polygon data and
;   vertex colors into a graphics object.
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-16 (B.F.) More comments
;   2005-07-22 (B.F.) More comments, v3d_create_polyobj -> v3d_createpolyobj
;              Parameters from v3d_parameters
;   2005-07-26 (B.F.) Put into separate file
;   2005-08-15 (B.F.) sState, index as input instead of data3d
;   2006-06-13 (B.F.) Create dummy object if no polygons in stored object
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  if (N_ELEMENTS(shading) eq 0) then shading=v3dpar_PolygonShading_default

  ; By default the object is not hidden
  if (N_ELEMENTS(hide) eq 0) then hide=0

  ; Should the object be created out of the low resolution model?
  polygons= (KEYWORD_SET(low)) ? (*sState)[index].iso.polygons_low $
                               : (*sState)[index].iso.polygons

  ;oPoly->SetProperty, XCOORD_CONV=[(*sState)[i].box.xrange[0], 1.0]
  ;oPoly->SetProperty, YCOORD_CONV=[(*sState)[i].box.yrange[0], 1.0]
  ;oPoly->SetProperty, ZCOORD_CONV=[(*sState)[i].box.zrange[0], 1.0]

  ; Check if a solid color is in use, different keywords must be used each time
  if (n_elements(*polygons) le 1) then begin
    message, 'Create dummy polygon object', /INFORMATIONAL
    mesh_obj, 4, dummy_vertices, dummy_polygons, replicate(2.0, 11, 11) 
      return, OBJ_NEW('IDLgrPolygon', $
                      dummy_vertices, $
                      polygons=dummy_polygons, $
                      color=(*sState)[index].color.solid_color, $
                      shading=shading, $
                      style=v3dpar_PolygonStyle_default, $
                      HIDE=hide)
  endif else begin
    if ((*sState)[index].color.is_solidcolor eq 0) then begin
      return, OBJ_NEW('IDLgrPolygon', $
                      *(*sState)[index].iso.vertices, $
                      polygons=*polygons, $
                      vert_colors=*(*sState)[index].iso.vert_colors, $
                      shading=shading, $
                      style=v3dpar_PolygonStyle_default, $
                      HIDE=hide)
    endif else begin 
      ;print, 'is_solidcolor ne 0'
      return, OBJ_NEW('IDLgrPolygon', $
                      *(*sState)[index].iso.vertices, $
                      polygons=*polygons, $
                      color=(*sState)[index].color.solid_color, $
                      shading=shading, $
                      style=v3dpar_PolygonStyle_default, $
                      HIDE=hide)
    endelse
  endelse

end ; v3d_iso_createpolyobj


pro v3d_iso_regenerate, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_regenerate ('v3d_iso_regenerate')
;
; PURPOSE:
;   Regenerate the 3D mesh.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_regenerate, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;   Destroy the old mesh data.
;   Create new 3D mesh objects.
;   Connect to the object hierarchy.
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, check with v3d_findtag removed
;   2005-07-26 (B.F.) Put into separate file
;   2005-07-27 (B.F.) Transfer polygon properties, do not touch oBox
;   2005-08-05 (B.F.) Call v3d_putdata3dintosstate
;   2005-08-15 (B.F.) Redesign: act on sState and not on tmpmesh
;   2005-08-16 (B.F.) Keyword paletterange removed
;   2005-08-26 (B.F.) Handle alpha_channel
;   2006-06-30 (B.F.) Do not destroy and recreate object
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  time0=systime(1)

  v3d_iso_generate, sState, index

  if (n_elements(*(*sState)[index].iso.polygons) le 1) then begin
    message, 'Put dummy polygons into object', /INFORMATIONAL
    mesh_obj, 4, dummy_vertices, dummy_polygons, replicate(2.0, 11, 11) 
    (*sState)[index].oPolyHigh->SetProperty, data      =dummy_vertices, $
                                             polygons  =dummy_polygons, $
                                             vert_color=(*sState)[index].color.solid_color
    (*sState)[index].oPoly    ->SetProperty, data      =dummy_vertices, $
                                             polygons  =dummy_polygons, $
                                             vert_color=(*sState)[index].color.solid_color
  endif else begin
    if ((*sState)[index].color.is_solidcolor eq 0) then begin
      (*sState)[index].oPolyHigh->SetProperty, data      =*(*sState)[index].iso.vertices, $
                                               polygons  =*(*sState)[index].iso.polygons, $
                                               vert_color=*(*sState)[index].iso.vert_colors
      (*sState)[index].oPoly    ->SetProperty, data      =*(*sState)[index].iso.vertices, $
                                               polygons  =*(*sState)[index].iso.polygons_low, $
                                               vert_color=*(*sState)[index].iso.vert_colors
    endif else begin 
      ;print, 'is_solidcolor ne 0'
      (*sState)[index].oPolyHigh->SetProperty, data      =*(*sState)[index].iso.vertices, $
                                               polygons  =*(*sState)[index].iso.polygons, $
                                               color     =(*sState)[index].color.solid_color
      (*sState)[index].oPoly    ->SetProperty, data      =*(*sState)[index].iso.vertices, $
                                               polygons  =*(*sState)[index].iso.polygons_low, $
                                               color     =(*sState)[index].color.solid_color
    endelse
  endelse
 
  message, 'Complete regeneration took ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL

end ; v3d_iso_regenerate


function v3d_iso_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_event ('v3d_iso_event')
;
; PURPOSE:
;   Handle all iso events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-08-12 (B.F.) Taken from v3d_dataset_event
;   2006-05-31 (B.F.) Print mesh statistics
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ;message, '', /INFORMATIONAL
  ;help, /str, sEvent
  ;help, sState, index, uval

  if (uval eq 'TEXT_ISO_ISO') then begin ; -------------------------------------
    ; User has changed the value of isosurface, remember the new isovalue
    ;
    ;(*sState)[index].iso.curriso=sEvent.value
  endif else $ ; 'ISO'

  if (uval eq 'SLIDER_ISO_DECIM') then begin ; ---------------------------------
    ; User has changed the value of decimation
    ;
    (*sState)[index].iso.decim=sEvent.value
  endif else $ ; 'DECIM'

  if ((uval eq 'BUTTON_ISO_APPLY') or $
      (uval eq 'BUTTON_ISO_MINUS') or $
      (uval eq 'BUTTON_ISO_PLUS') or $
      (uval eq 'BUTTON_ISO_APPLYNEWCOLOR')) then begin ; -----------------------
    ;
    widget_control, (*sState)[index].wIsoVal, $
                    GET_VALUE=strcurriso
    curriso=float(strcurriso)
    (*sState)[index].iso.curriso=(*sState)[index].box.fullisorange[0] > $
                                    curriso                              < $
                                    (*sState)[index].box.fullisorange[1]
    ;
    if (uval eq 'BUTTON_ISO_MINUS') then begin
      (*sState)[index].iso.decim  =100
      (*sState)[index].iso.curriso=((*sState)[index].iso.curriso - $
                                       0.01*((*sState)[index].box.fullisorange[1]- $
                                             (*sState)[index].box.fullisorange[0])) > $
                                      (*sState)[index].box.fullisorange[0]
    endif
    if (uval eq 'BUTTON_ISO_PLUS') then begin
      (*sState)[index].iso.decim  =100
      (*sState)[index].iso.curriso=((*sState)[index].iso.curriso + $
                                       0.01*((*sState)[index].box.fullisorange[1]- $
                                             (*sState)[index].box.fullisorange[0])) < $
                                      (*sState)[index].box.fullisorange[1]
    endif
    ;
    ; Recalculate mesh
    v3d_iso_regenerate, sState, index
    ;
    if (uval eq 'BUTTON_ISO_APPLYNEWCOLOR') then begin
      currcolorrange=(*sState)[index].iso.currcolorrange
      (*sState)[index].color.palette_top   =currcolorrange[1]
      (*sState)[index].color.palette_bottom=currcolorrange[0]
      ;
      if ((*sState)[index].no_colorbar eq 0) then begin
        v3d_cbar_updatewidget, (*sState)[index].oColorbar , $
                               (*sState)[index].color.palette_top, $
                               (*sState)[index].color.palette_bottom, $
                             *((*sState)[index].color.palette)
        ; And redraw the colorbar
        (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
        ;(*sState)[index].oColorbar->GetProperty, tickvalues=tickvalues
        ;print,tickvalues
      endif
      ;
      ; Update the text widgets with current palette range
      v3d_color_updatewidget, sState, index
      ;
      v3d_color_setpalette, sState, index, /ALLNEW
    endif
    ;
    ; and redraw the view  
    ;message, 'Redraw the view', /INFORMATIONAL
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
    widget_control, (*sState)[index].wIsoVal  , $
                    SET_VALUE=string((*sState)[index].iso.curriso, FORMAT='(G0)')
    widget_control, (*sState)[index].wDecimVal, $
                    SET_VALUE=string((*sState)[index].iso.decim)
  endif else $ ;

  if (uval eq 'BUTTON_ISO_NEWCOLOR') then begin ; ------------------------------
    currcolorrange=(*sState)[index].iso.currcolorrange
    (*sState)[index].color.palette_top   =currcolorrange[1]
    (*sState)[index].color.palette_bottom=currcolorrange[0]
    ;
    v3d_color_setpalette, sState, index, /ALLNEW
    ;
    if ((*sState)[index].no_colorbar eq 0) then begin
      v3d_cbar_updatewidget, (*sState)[index].oColorbar , $
                             (*sState)[index].color.palette_top, $
                             (*sState)[index].color.palette_bottom, $
                           *((*sState)[index].color.palette)
      ; And redraw the colorbar
      (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
    endif
    ;
    ; Update the text widgets with current palette range
    v3d_color_updatewidget, sState, index
    ;
    ; and redraw the view  
    ;message, 'Redraw the view', /INFORMATIONAL
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ;

  if (uval eq 'BUTTON_ISO_SHOWHIDEOBJ') then begin ; ---------------------------
    ; Toggle the hide flag of the data object
    ;
    (*sState)[index].oPolyHigh->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oPolyHigh->SetProperty, HIDE=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wShowHideIso, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wShowHideIso, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_ISO_SHOWHIDEOBJ'

  if (uval eq 'BUTTON_ISO_STAT') then begin ; ----------------------------------
    ; Print statistics about mesh
    ;
    q_surfacearea=mesh_surfacearea(*(*sState)[index].iso.vertices, $
                                   *(*sState)[index].iso.polygons)
    c_surfacearea=mesh_surfacearea(*(*sState)[index].iso.vertices, $
                                   *(*sState)[index].iso.polygons, $
                           auxdata=*(*sState)[index].iso.vert_values)
    ;
    print, 'Mesh: ', index+1
    print, '  # Triangles:   ', mesh_numtriangles(*(*sState)[index].iso.polygons)
    print, '  # Vertices:    ', n_elements(*(*sState)[index].iso.vertices)
    print, '  Surface area:  ', q_surfacearea
    if (q_surfacearea ne 0.0) then begin
      print, '  avg(colordata):', c_surfacearea/q_surfacearea
    endif
    print, '  Volume:        ', mesh_volume(*(*sState)[index].iso.vertices, $
                                            *(*sState)[index].iso.polygons)
    ;
  endif else $ ; 'BUTTON_ISO_STAT'

  if (uval eq 'BUTTON_ISO_HELP') then begin ; ----------------------------------
    ; Show help
    ;
    v3d_iso_displayhelp, group=(*sState)[index].wDraw
    ;
  endif else $ ; 'BUTTON_ISO_HELP'

  if (uval eq 'BUTTON_ISO_CLOSE') then begin ; ---------------------------------
    ; Close widget without any further action
    ;
    (*sState)[index].hideControlIso=1
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif ; 'BUTTON_ISO_CLOSE'

  return, 1

end ; v3d_iso_event


pro v3d_iso_updatewidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_updatewidget ('v3d_iso_update_widget')
;
; PURPOSE:
;   Update the iso control widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_updatewidget, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2005-09-02 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if ((*sState)[index].hideControlIso eq 0) then begin
    widget_control, (*sState)[index].wIsoVal  , $
                    SET_VALUE=string((*sState)[index].iso.curriso, FORMAT='(G0)')
    widget_control, (*sState)[index].wDecimVal, $
                    SET_VALUE=(*sState)[index].iso.decim
    (*sState)[index].oPolyHigh->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wShowHideIso, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wShowHideIso, $
                      SET_VALUE='Hide'
    endelse
  endif

end ; v3d_iso_updatewidget


function v3d_iso_createwidget, wBaseMesh, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_createwidget ('v3d_iso_create_widget')
;
; PURPOSE:
;   Create the iso widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_iso_createwidget(wBaseMesh, sState, index)
;
; INPUT:
;   wBaseMesh: ('widget_Base_Mesh') parent widget
;   index:     ('index') current view index
;
; INPUT/OUTPUT:
;   sState:    ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-08-12 (B.F.) Split of
;   2005-08-17 (B.F.) Put into separate routine
;   2005-11-21 (B.F.) Help from menu under tool button
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  strindex=string(index+1, format='(I0)')
  ;
  wBaseButtons=widget_base(wBaseMesh, $
                           /COLUMN, /FRAME, $
                           UVALUE=index, $
                           EVENT_FUNC='v3d_iso_event', $
                           XPAD=2, YPAD=2)
    ;
    wTopRow=widget_base(     wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
;      wEntry=widget_label(     wTopRow,           VALUE ='Iso-surface ' + strindex)
      wMenu=widget_button(     wTopRow,           VALUE ='Isosurf ' + strindex, $
                                                  UVALUE='BUTTON_ISO_MENU', $
                               /MENU, $
                               TOOLTIP='Show isosurface menu')
;        wEntry=widget_button(  wMenu,             VALUE='Settings', $
;                                                  UVALUE='BUTTON_ISO_SETTINGS')
;        wEntry=widget_button(  wMenu,             VALUE='Control', $
;                                                  UVALUE='BUTTON_ISO_CONTROL')
        wEntry=widget_button(  wMenu,             VALUE='Print mesh statistics', $
                                                  UVALUE='BUTTON_ISO_STAT')
        wEntry=widget_button(  wMenu,             VALUE='Help', $
                                                  UVALUE='BUTTON_ISO_HELP')
      wEntry=widget_label(     wTopRow,           VALUE ='')
      wEntry=widget_button(    wTopRow,           VALUE ='Hide', $
                                                  UVALUE='BUTTON_ISO_SHOWHIDEOBJ', $
                               TOOLTIP='Show/hide iso-surface object')
      (*sState)[index].wShowHideIso=wEntry
      wEntry=widget_button(    wTopRow,           VALUE ='Close', $
                                                  UVALUE='BUTTON_ISO_CLOSE', $
                               TOOLTIP='Close this panel without further action')
    ;
    wCenterFrame=widget_base(wBaseButtons, $
                             /COLUMN, $
                             XPAD=0, YPAD=2)
      wLine1=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine1,          VALUE ='Iso:')
        wEntry=widget_text(      wLine1,          VALUE=string((*sState)[index].iso.curriso, $
                                                               FORMAT='(G0)'), $
                                                  UVALUE='TEXT_ISO_ISO', $
                                 /EDITABLE, $
                                 XSIZE=12)
        (*sState)[index].wIsoVal=wEntry
        wEntry=widget_button(    wLine1,           VALUE ='-', $
                                                   UVALUE='BUTTON_ISO_MINUS', $
                                 TOOLTIP='Reduce iso value')
        wEntry=widget_button(    wLine1,           VALUE ='+', $
                                                   UVALUE='BUTTON_ISO_PLUS', $
                                 TOOLTIP='Increase iso value')
      ;
      wLine2=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine2,          VALUE =' ')
        wEntry=widget_label(     wLine2,          VALUE = $
                                  string((*sState)[index].box.fullisorange[0], FORMAT='(G0)') + $
                                  ' .. ' + $
                                  string((*sState)[index].box.fullisorange[1], FORMAT='(G0)'))
      ;
      wLine3=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine3,          VALUE ='Dec:')
        wEntry=widget_slider(    wLine3,          VALUE =(*sState)[index].iso.decim, $
                                                  UVALUE='SLIDER_ISO_DECIM', $
                                 MINIMUM=1, MAXIMUM=100, SCROLL=5)
        (*sState)[index].wDecimVal=wEntry
        wEntry=widget_label(     wLine3,          VALUE ='%')
    ;
    wBottomRow=widget_base(  wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
                                                  UVALUE='BUTTON_ISO_APPLY', $
                               TOOLTIP='Create new mesh')
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply+Color', $
                                                  UVALUE='BUTTON_ISO_APPLYNEWCOLOR', $
                               TOOLTIP='Create new mesh and adjust color palette to it')
      wEntry=widget_button(    wBottomRow,        VALUE ='Color', $
                                                  UVALUE='BUTTON_ISO_NEWCOLOR', $
                               TOOLTIP='Adjust color palette to new mesh')
  ;
  return, wBaseButtons

end ; v3d_iso_createwidget


pro v3d_iso_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_iso_destroy ('v3d_iso_destroy')
;
; PURPOSE:
;   Destroy iso structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_iso_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Free existing heap variables and clean up using a null_pointer.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;
; HISTORY:
;   2005-08-31 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ptr_free, (*sState)[index].iso.polygons
  (*sState)[index].iso.polygons=ptr_new()

  ptr_free, (*sState)[index].iso.polygons_low
  (*sState)[index].iso.polygons_low=ptr_new()

  ptr_free, (*sState)[index].iso.vertices 
  (*sState)[index].iso.vertices=ptr_new()

  ptr_free, (*sState)[index].iso.vert_values 
  (*sState)[index].iso.vert_values=ptr_new()

  ptr_free, (*sState)[index].iso.vert_colors 
  (*sState)[index].iso.vert_colors=ptr_new()

end ; v3d_iso_destroy


function v3d_iso_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_iso_init ('v3d_iso_initialize')
;
; PURPOSE:
;   Initialize iso structure.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   iso=v3d_iso_init()
;
; MODIFICATION HISTORY:
;   2002-06-12 Written by Sigurdur Finnsson, Uppsala
;   2005-07-09 (B.F.) Comments and extra space added, tags re-ordered
;   2005-07-18 (B.F.) Get default decimation from parameter file
;   2005-07-31 (B.F.) xyzrange: long -> double
;   2005-08-15 (B.F.) data3d split into box, color, data3d
;   2005-09-02 (B.F.) Renamed and put module file
;   2006-06-30 (B.F.) Renamed: data3D -> iso
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Create an anonymous structure:
  iso={$
    vertices:              PTR_NEW(), $            ; List of vertex coordinates
    vert_values:           PTR_NEW(), $            ; List of vertex values
    vert_colors:           PTR_NEW(), $            ; List of vertex colors
    polygons:              PTR_NEW(), $            ; List of polygons (full resolution)
    polygons_low:          PTR_NEW(), $            ; List of polygons (low resolution)
    curriso:               0.0D+00, $              ; Choosen isosurface value
    currcolorrange:        [0.0D+00, 0.0D+00], $   ; minmax of color data on current isosurface
    smooth_on:             0, $                    ; Switch to activate coordinate smoothing
    smooth_iterations:     0, $                    ; Number of coordinate smoothing iterations
    smooth_lambda:         0.0, $                  ; Amount of coordinate smoothing
    colorsmooth_on:        0, $                    ; Switch to activate coordinate smoothing
    colorsmooth_iterations:0, $                    ; Number of coordinate smoothing iterations
    colorsmooth_lambda:    0.0, $                  ; Amount of coordinate smoothing
    decim:                 v3dpar_Decimation_default $  ; Decimation in %
       }

  return, iso

end; v3d_iso_init
