;-------------------------------------------------------------------------------
; Vis3D module: Slices through volume
;-------------------------------------------------------------------------------
; Contains:
;   v3d_slice_displayhelp:         Display help text for slice widget
;   v3d_slice_makexim:             Make x image for slice object
;   v3d_slice_makeyim:             Make y image for slice object
;   v3d_slice_makezim:             Make z image for slice object
;   v3d_slice_createobj:           Create slice object
;   v3d_slice_event:               Handle all slice events
;   v3d_slice_createcontrolwidget: Create the slice control widget
;   v3d_slice_updatewidget:        Update the slice widget
;   v3d_slice_createwidget:        Create the slice widget from data in the state variable
;   v3d_slice_destroy:             Destroy slice structure and free memory used in it
;   v3d_slice_init:                Initialize slice structure
;-------------------------------------------------------------------------------


pro v3d_slice_displayhelp, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_displayhelp ('v3d_slice_display_help')
;
; PURPOSE:
;   Display help text for slice widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_slice_displayhelp, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-09-05 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Slices', $
'======', $
'', $
'Up to three orthogonal slices through the 3D color dataset can be displayed at', $
'the same time. They are controlled via this panel.', $
'', $
'A click on the "Slice X" field opens a menu with a control entry that allows the', $
'automatic positioning of the slices to the point with the minimum or maximum', $
'value of the color or the iso dataset', $
'and a help entry, which gives this help text.', $
'', $
'The "Hide" or "Show" button on top of the panel toggles the visibility of', $
'all slices at once.', $
'', $
'The "Close" button just closes the panel (not the entire window) without', $
'starting a generation process.', $
'', $
'There are three sets of sliders and control buttons for the slices in the', $
'three spatial dimensions:', $
'', $
'The "x" slider specifies the x position of the yz slice.', $
'It''s visibility can be toggled with the "Hide" or "Show" button next to it.', $
'The x sliders of all views can be coupled with the "Couple" button.', $
'', $
'The controls for the y and z directions work correspondingly.', $
'Position changes are applied instantenously.', $
'', $
'The "Apply" button updates the used intensity scaling value from the numerical', $
'input field on the left:', $
'A value of 0 means that no scaling/masking is applied.', $
'A value > 0 means that the intensity is scaled with values from the the', $
'iso dataset, divided by the scaling factor.', $
'', $
'The "Color" button adjusts the color palette according to the visible slices.']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_slice_displayhelp


pro v3d_slice_makexim, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_makexim ('v3d_slice_make_x_image')
;
; PURPOSE:
;   Make x image for slice object.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_slice_makexim, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2005-09-05 Written by Bernd Freytag, L.A.
;   2005-09-08 (B.F.) Computation of image put into separate routine
;   2005-11-14 (B.F. Uppsala) Bugfix: swapped indices in reform statement
;   2011-02-21 (B.F. Lyon) Option of masking with values from iso dataset
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  nx=(*sState)[index].box.nx
  ny=(*sState)[index].box.ny
  nz=(*sState)[index].box.nz

  if ((*sState)[index].color.is_solidcolor ne 1) then begin
    ;
    top   =(*sState)[index].color.palette_top
    bottom=(*sState)[index].color.palette_bottom
    ;
    ; The range of physical values which the palette will be mapped to
    delta=top-bottom
    ;
    if (delta eq 0.0) then begin
      ; The color range is zero.
      ;
      ; Use first color in palette for all points
      ximage=rebin(reform((*sState)[index].color.palette[*,0], 3, 1, 1), $
                   3, ny, nz)
      ;
    endif else begin
      ; There is a non-zero color range
      ;
      xslice=reform((*(*sState)[index].box.colordataset)[(*sState)[index].slice.ix, *, *])
      ;xslice=rotate(xslice, 2)
;plotlines, xslice, title=string((*sState)[index].slice.ix)
      xslice=transpose(xslice)
      ;
      ; How many entries are in current palette?
      n_pal=(size(*(*sState)[index].color.palette))[2]
      ;
      c=0L > $
        (round((float(n_pal-1L)/delta) * (xslice-bottom))) < $
        (n_pal-1L)
      ximage=(*(*sState)[index].color.palette)[*,c]
      ximage=reform(ximage, 3, nz, ny)
      ;
      ;help, xslice, c, ximage
    endelse
    ;
  endif else begin
    ; Produce image with single color
    ;
    ximage=rebin(reform((*sState)[index].color.solid_color, 3, 1, 1), $
                 3, nz, ny)
  endelse
  ;
  if ((*sState)[index].slice.Scale gt 0.0) then begin
    xmask=reform((*(*sState)[index].box.isodataset)[(*sState)[index].slice.ix, *, *])
    xmask=transpose(xmask)
    xmask=(xmask/(*sState)[index].slice.Scale) < 1.0
    ximage[0,*,*]=byte(ximage[0,*,*]*xmask)
    ximage[1,*,*]=byte(ximage[1,*,*]*xmask)
    ximage[2,*,*]=byte(ximage[2,*,*]*xmask)
  endif
  ;
  if (obj_valid((*sState)[index].oSlicexIm)) then begin
    (*sState)[index].oSlicexIm->SetProperty, DATA       =ximage, $
                                             ZCOORD_CONV=[-float((*sState)[index].slice.ix), 1.0]
  endif else begin
    oSlicexModel              =obj_new('IDLgrModel')
    oSlicexModel              ->Rotate, [0.0, 1.0, 0.0], -90.0
    (*sState)[index].oSlice   ->Add, oSlicexModel
    (*sState)[index].oSlicexIm=obj_new('IDLgrImage', ximage, $
                                       INTERLEAVE        =0, $
;                                       ALPHA             =0.5, $
;                                       BLEND_FUNCTION    =[3, 4], $
                                       TRANSFORM_MODE    =1, $
                                       DEPTH_TEST_DISABLE=2, $
                                       XCOORD_CONV       =[-0.5, 1.0], $
                                       YCOORD_CONV       =[-0.5, 1.0], $
                                       ZCOORD_CONV       =[-float((*sState)[index].slice.ix), 1.0], $
                                       HIDE              =0, $
                                       /INTERPOLATE)
    oSlicexModel              ->Add, (*sState)[index].oSlicexIm
;    (*sState)[index].oSlicexIm=obj_new('IDLgrImage', ximage, $
;                                       INTERLEAVE        =0, $
;                                       TRANSFORM_MODE    =0, $
;                                       HIDE              =0, $
;                                       /INTERPOLATE)
;    oSlicexSurf               =obj_new('IDLgrSurface', $
;                                       xslice*0.0-float((*sState)[index].slice.ix), $
;                                       TEXTURE_MAP=oSlicexIm, $
;                                       HIDE       =0, $
;                                       LINESTYLE  =0, $
;                                       STYLE      =2, $
;                                       COLOR      =[255B, 255B, 255B])
;    oSlicexModel              ->Add, oSlicexSurf
  endelse

end ; v3d_slice_makexim


pro v3d_slice_makeyim, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_makeyim ('v3d_slice_make_y_image')
;
; PURPOSE:
;   Make y image for slice object.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_slice_makeyim, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2005-09-05 Written by Bernd Freytag, L.A.
;   2005-09-08 (B.F.) Derived from v3d_slice_makexim
;   2011-02-21 (B.F. Lyon) Option of masking with values from iso dataset
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  nx=(*sState)[index].box.nx
  ny=(*sState)[index].box.ny
  nz=(*sState)[index].box.nz

  if ((*sState)[index].color.is_solidcolor ne 1) then begin
    ;
    top   =(*sState)[index].color.palette_top
    bottom=(*sState)[index].color.palette_bottom
    ;
    ; The range of physical values which the palette will be mapped to
    delta=top-bottom
    ;
    if (delta eq 0.0) then begin
      ; The color range is zero.
      ;
      ; Use first color in palette for all points
      yimage=rebin(reform((*sState)[index].color.palette[*,0], 3, 1, 1), $
                   3, nx, nz)
      ;
    endif else begin
      ; There is a non-zero color range
      ;
      yslice=reform((*(*sState)[index].box.colordataset)[*, (*sState)[index].slice.iy, *])
      ;yslice=rotate(yslice, 2)
      ;yslice=transpose(yslice)
      ;
      ; How many entries are in current palette?
      n_pal=(size(*(*sState)[index].color.palette))[2]
      ;
      c=0L > $
        (round((float(n_pal-1L)/delta) * (yslice-bottom))) < $
        (n_pal-1L)
      yimage=(*(*sState)[index].color.palette)[*,c]
      yimage=reform(yimage, 3, nx, nz)
      ;
      ;help, yslice, c, yimage
    endelse
    ;
  endif else begin
    ; Produce image with single color
    ;
    yimage=rebin(reform((*sState)[index].color.solid_color, 3, 1, 1), $
                 3, nx, nz)
  endelse
  ;
  if ((*sState)[index].slice.Scale gt 0.0) then begin
    ymask=reform((*(*sState)[index].box.isodataset)[*,(*sState)[index].slice.iy, *])
    ymask=(ymask/(*sState)[index].slice.Scale) < 1.0
    yimage[0,*,*]=byte(yimage[0,*,*]*ymask)
    yimage[1,*,*]=byte(yimage[1,*,*]*ymask)
    yimage[2,*,*]=byte(yimage[2,*,*]*ymask)
  endif
  ;
  if (obj_valid((*sState)[index].oSliceyIm)) then begin
    (*sState)[index].oSliceyIm->SetProperty, DATA       =yimage, $
                                             ZCOORD_CONV=[-float((*sState)[index].slice.iy), 1.0]
  endif else begin
    oSliceyModel              =obj_new('IDLgrModel')
    oSliceyModel              ->Rotate, [1.0, 0.0, 0.0], 90.0
    (*sState)[index].oSlice   ->Add, oSliceyModel
    (*sState)[index].oSliceyIm=obj_new('IDLgrImage', yimage, $
                                       INTERLEAVE        =0, $
;                                       ALPHA             =0.5, $
;                                       BLEND_FUNCTION    =[3, 4], $
                                       TRANSFORM_MODE    =1, $
                                       DEPTH_TEST_DISABLE=2, $
                                       XCOORD_CONV       =[-0.5, 1.0], $
                                       YCOORD_CONV       =[-0.5, 1.0], $
                                       ZCOORD_CONV       =[-float((*sState)[index].slice.iy), 1.0], $
                                       HIDE              =0, $
                                       /INTERPOLATE)
    oSliceyModel              ->Add, (*sState)[index].oSliceyIm
  endelse

end ; v3d_slice_makeyim


pro v3d_slice_makezim, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_makezim ('v3d_slice_make_z_image')
;
; PURPOSE:
;   Make z image for slice object.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_slice_makezim, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2005-09-05 Written by Bernd Freytag, L.A.
;   2005-09-08 (B.F.) Derived from v3d_slice_makexim
;   2011-02-21 (B.F. Lyon) Option of masking with values from iso dataset
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  nx=(*sState)[index].box.nx
  ny=(*sState)[index].box.ny
  nz=(*sState)[index].box.nz

  if ((*sState)[index].color.is_solidcolor ne 1) then begin
    ;
    top   =(*sState)[index].color.palette_top
    bottom=(*sState)[index].color.palette_bottom
    ;
    ; The range of physical values which the palette will be mapped to
    delta=top-bottom
    ;
    if (delta eq 0.0) then begin
      ; The color range is zero.
      ;
      ; Use first color in palette for all points
      zimage=rebin(reform((*sState)[index].color.palette[*,0], 3, 1, 1), $
                   3, nx, ny)
      ;
    endif else begin
      ; There is a non-zero color range
      ;
      zslice=reform((*(*sState)[index].box.colordataset)[*, *, (*sState)[index].slice.iz])
;surface, zslice
      ;zslice=rotate(zslice, 2)
      ;zslice=transpose(zslice)
      ;
      ; How many entries are in current palette?
      n_pal=(size(*(*sState)[index].color.palette))[2]
      ;
      c=0L > $
        (round((float(n_pal-1L)/delta) * (zslice-bottom))) < $
        (n_pal-1L)
      zimage=(*(*sState)[index].color.palette)[*,c]
      zimage=reform(zimage, 3, nx, ny)
      ;
      ;help, zslice, c, zimage
    endelse
    ;
  endif else begin
    ; Produce image with single color
    ;
    zimage=rebin(reform((*sState)[index].color.solid_color, 3, 1, 1), $
                 3, nx, ny)
  endelse
  ;
  if ((*sState)[index].slice.Scale gt 0.0) then begin
    zmask=reform((*(*sState)[index].box.isodataset)[*, *, (*sState)[index].slice.iz])
    zmask=(zmask/(*sState)[index].slice.Scale) < 1.0
    zimage[0,*,*]=byte(zimage[0,*,*]*zmask)
    zimage[1,*,*]=byte(zimage[1,*,*]*zmask)
    zimage[2,*,*]=byte(zimage[2,*,*]*zmask)
  endif
  ;
  if (obj_valid((*sState)[index].oSlicezIm)) then begin
    (*sState)[index].oSlicezIm->SetProperty, DATA       =zimage, $
                                             ZCOORD_CONV=[float((*sState)[index].slice.iz), 1.0]
  endif else begin
    oSlicezModel              =obj_new('IDLgrModel')
    oSlicezModel              ->Rotate, [1.0, 0.0, 0.0], 0.0
    (*sState)[index].oSlice   ->Add, oSlicezModel
    (*sState)[index].oSlicezIm=obj_new('IDLgrImage', zimage, $
                                       INTERLEAVE        =0, $
;                                       ALPHA             =0.5, $
;                                       BLEND_FUNCTION    =[3, 4], $
                                       TRANSFORM_MODE    =1, $
                                       DEPTH_TEST_DISABLE=2, $
                                       XCOORD_CONV       =[-0.5, 1.0], $
                                       YCOORD_CONV       =[-0.5, 1.0], $
                                       ZCOORD_CONV       =[float((*sState)[index].slice.iz), 1.0], $
                                       HIDE              =0, $
                                       /INTERPOLATE)
    oSlicezModel              ->Add, (*sState)[index].oSlicezIm
  endelse

end ; v3d_slice_makezim


pro v3d_slice_createobj, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_createobj ('v3d_slice_create_object')
;
; PURPOSE:
;   Create slice object.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_slice_createobj, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2005-09-05 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Save system time at start of routine
  time0=systime(1)

  if (not obj_valid((*sState)[index].oSlice)) then begin
    (*sState)[index].oSlice=obj_new('IDLgrModel', HIDE=v3dpar_Slices_hide)
  endif

  v3d_slice_makexim, sState, index
  v3d_slice_makeyim, sState, index
  v3d_slice_makezim, sState, index

  (*sState)[index].isCurrSliceColor=1

  message, 'Routine needed ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL

end ; v3d_slice_createobj


function v3d_slice_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_event ('v3d_slice_event')
;
; PURPOSE:
;   Handle all slice events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-09-05 (B.F.) Taken from v3d_dots_event
;   2006-05-31 (B.F.) Allow coloring according to values on slices
;   2006-06-02 (B.F.) Bugfix in slice coloring scheme
;   2007-04-12 (B.F.) Handle additional control widget
;   2011-02-21 (B.F. Lyon) Option of masking with values from iso dataset
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ;message, '', /INFORMATIONAL
  ;help, /str, sEvent
  ;help, sState, index, uval

  if (uval eq 'SLIDER_SLICE_X') then begin ; -----------------------------------
    ;
    ; Fetch new value for number of slice
    widget_control, (*sState)[index].wSlicexSlider, $
                    GET_VALUE=strix
    ix=long(strix)
    (*sState)[index].slice.ix=0L > ix < ((*sState)[index].box.nx-1L)
    ;
    ; Generate new image
    v3d_slice_makexim, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
    if (keyword_set((*sState)[index].slice.couplex)) then begin
      ; Process the slices of the other views, too
      ;
      nrofviews=n_elements(*sState)
      ;
      for i=0, nrofviews-1 do begin
        if (i ne index) then begin
          (*sState)[i].slice.ix=0L > ix < ((*sState)[i].box.nx-1L)
          ;
          ; Set the slider value
          if ((*sState)[i].hideControlSlice eq 0) then begin
            widget_control, (*sState)[i].wSlicexSlider, $
                            SET_VALUE=(*sState)[i].slice.ix

          endif
          ;
          ; Generate new image (even if slice is hidden)
          v3d_slice_makexim, sState, i
          ;
          ; Redraw the view (if slice is visible)
          (*sState)[i].oSlice   ->GetProperty, HIDE=hide
          (*sState)[i].oSlicexIm->GetProperty, HIDE=hidex
          if ((hide eq 0) and (hidex eq 0)) then begin
            (*sState)[i].oWindow->Draw, (*sState)[i].oView
          endif
          ;
        endif
      endfor ; i
    endif
    ;
  endif else $ ; 'SLIDER_SLICE_X'

  if (uval eq 'BUTTON_SLICE_XSHOWHIDEOBJ') then begin ; ------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oSlicexIm->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oSlicexIm->SetProperty, HIDE=hide
    ;
    (*sState)[index].slice.hidex=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wSlicexShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSlicexShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_SLICE_XSHOWHIDEOBJ'

  if (uval eq 'BUTTON_SLICE_XCOUPLE') then begin ; -----------------------------
    ; Couple/decouple the x slices of all views
    ;
    couple=1-keyword_set((*sState)[index].slice.couplex)
    (*sState)[*].slice.couplex=couple
    ;
    nrofviews=n_elements(*sState)
    ;
    for i=0, nrofviews-1 do begin
      if ((*sState)[i].hideControlSlice eq 0) then begin
        if (KEYWORD_SET(couple)) then begin
          widget_control, (*sState)[i].wSlicexCouple, $
                          SET_VALUE='Decple'
        endif else begin
          widget_control, (*sState)[i].wSlicexCouple, $
                          SET_VALUE='Couple'
        endelse
      endif
    endfor ; i  
    ;
  endif else $ ; 'BUTTON_SLICE_XCOUPLE'

  if (uval eq 'SLIDER_SLICE_Y') then begin ; -----------------------------------
    ;
    ; Fetch new value for number of slice
    widget_control, (*sState)[index].wSliceySlider, $
                    GET_VALUE=striy
    iy=long(striy)
    (*sState)[index].slice.iy=0L > iy < ((*sState)[index].box.ny-1L)
    ;
    ; Generate new image
    v3d_slice_makeyim, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
    if (keyword_set((*sState)[index].slice.coupley)) then begin
      ; Process the slices of the other views, too
      ;
      nrofviews=n_elements(*sState)
      ;
      for i=0, nrofviews-1 do begin
        if (i ne index) then begin
          (*sState)[i].slice.iy=0L > iy < ((*sState)[i].box.ny-1L)
          ;
          ; Set the slider value
          if ((*sState)[i].hideControlSlice eq 0) then begin
            widget_control, (*sState)[i].wSliceySlider, $
                            SET_VALUE=(*sState)[i].slice.iy

          endif
          ;
          ; Generate new image (even if slice is hidden)
          v3d_slice_makeyim, sState, i
          ;
          ; Redraw the view (if slice is visible)
          (*sState)[i].oSlice   ->GetProperty, HIDE=hide
          (*sState)[i].oSliceyIm->GetProperty, HIDE=hidey
          if ((hide eq 0) and (hidey eq 0)) then begin
            (*sState)[i].oWindow->Draw, (*sState)[i].oView
          endif
          ;
        endif
      endfor ; i
    endif
    ;
  endif else $ ; 'SLIDER_SLICE_Y'

  if (uval eq 'BUTTON_SLICE_YSHOWHIDEOBJ') then begin ; ------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oSliceyIm->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oSliceyIm->SetProperty, HIDE=hide
    ;
    (*sState)[index].slice.hidey=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wSliceyShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSliceyShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_SLICE_YSHOWHIDEOBJ'

  if (uval eq 'BUTTON_SLICE_YCOUPLE') then begin ; -----------------------------
    ; Couple/decouple the y slices of all views
    ;
    couple=1-keyword_set((*sState)[index].slice.coupley)
    (*sState)[*].slice.coupley=couple
    ;
    nrofviews=n_elements(*sState)
    ;
    for i=0, nrofviews-1 do begin
      if ((*sState)[i].hideControlSlice eq 0) then begin
        if (KEYWORD_SET(couple)) then begin
          widget_control, (*sState)[i].wSliceyCouple, $
                          SET_VALUE='Decple'
        endif else begin
          widget_control, (*sState)[i].wSliceyCouple, $
                          SET_VALUE='Couple'
        endelse
      endif
    endfor ; i  
    ;
  endif else $ ; 'BUTTON_SLICE_YCOUPLE'

  if (uval eq 'SLIDER_SLICE_Z') then begin ; -----------------------------------
    ;
    ; Fetch new value for number of slice
    widget_control, (*sState)[index].wSlicezSlider, $
                    GET_VALUE=striz
    iz=long(striz)
    (*sState)[index].slice.iz=0L > iz < ((*sState)[index].box.nz-1L)
    ;
    ; Generate new image
    v3d_slice_makezim, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
    if (keyword_set((*sState)[index].slice.couplez)) then begin
      ; Process the slices of the other views, too
      ;
      nrofviews=n_elements(*sState)
      ;
      for i=0, nrofviews-1 do begin
        if (i ne index) then begin
          (*sState)[i].slice.iz=0L > iz < ((*sState)[i].box.nz-1L)
          ;
          ; Set the slider value
          if ((*sState)[i].hideControlSlice eq 0) then begin
            widget_control, (*sState)[i].wSlicezSlider, $
                            SET_VALUE=(*sState)[i].slice.iz

          endif
          ;
          ; Generate new image (even if slice is hidden)
          v3d_slice_makezim, sState, i
          ;
          ; Redraw the view (if slice is visible)
          (*sState)[i].oSlice   ->GetProperty, HIDE=hide
          (*sState)[i].oSlicezIm->GetProperty, HIDE=hidez
          if ((hide eq 0) and (hidez eq 0)) then begin
            (*sState)[i].oWindow->Draw, (*sState)[i].oView
          endif
          ;
        endif
      endfor ; i
    endif
    ;
  endif else $ ; 'SLIDER_SLICE_Z'

  if (uval eq 'BUTTON_SLICE_ZSHOWHIDEOBJ') then begin ; ------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oSlicezIm->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oSlicezIm->SetProperty, HIDE=hide
    ;
    (*sState)[index].slice.hidez=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wSlicezShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSlicezShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_SLICE_ZSHOWHIDEOBJ'

  if (uval eq 'BUTTON_SLICE_ZCOUPLE') then begin ; -----------------------------
    ; Couple/decouple the z slices of all views
    ;
    couple=1-keyword_set((*sState)[index].slice.couplez)
    (*sState)[*].slice.couplez=couple
    ;
    nrofviews=n_elements(*sState)
    ;
    for i=0, nrofviews-1 do begin
      if ((*sState)[i].hideControlSlice eq 0) then begin
        if (KEYWORD_SET(couple)) then begin
          widget_control, (*sState)[i].wSlicezCouple, $
                          SET_VALUE='Decple'
        endif else begin
          widget_control, (*sState)[i].wSlicezCouple, $
                          SET_VALUE='Couple'
        endelse
      endif
    endfor ; i  
    ;
  endif else $ ; 'BUTTON_SLICE_ZCOUPLE'

  if (uval eq 'BUTTON_SLICE_SHOWHIDEOBJ') then begin ; -------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oSlice->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oSlice->SetProperty, HIDE=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wSliceShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSliceShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_SLICE_SHOWHIDEOBJ'

  if (uval eq 'BUTTON_SLICE_NEWCOLOR') then begin ; ----------------------------
    ; Set color palette according to visible slices
    ;
    ; Check hidden/visible slices
    (*sState)[index].oSlicexIm->GetProperty, HIDE=hidex
    (*sState)[index].oSliceyIm->GetProperty, HIDE=hidey
    (*sState)[index].oSlicezIm->GetProperty, HIDE=hidez
    ;
    if ((hidex eq 0) or (hidey eq 0) or (hidez eq 0)) then begin
      ; At least one slice is visible
      ;
      ; Find minimum and maximum colordataset values on visible slices
      fullcolorrange=(*sState)[index].box.fullcolorrange
      palette_top   =fullcolorrange[0]
      palette_bottom=fullcolorrange[1]
      if (hidex eq 0) then begin
        slice=reform((*(*sState)[index].box.colordataset)[(*sState)[index].slice.ix, *, *])
        palette_top   =palette_top    > max(slice)
        palette_bottom=palette_bottom < min(slice)
      endif
      if (hidey eq 0) then begin
        slice=reform((*(*sState)[index].box.colordataset)[*, (*sState)[index].slice.iy, *])
        palette_top   =palette_top    > max(slice)
        palette_bottom=palette_bottom < min(slice)
      endif
      if (hidez eq 0) then begin
        slice=reform((*(*sState)[index].box.colordataset)[*, *, (*sState)[index].slice.iz])
        palette_top   =palette_top    > max(slice)
        palette_bottom=palette_bottom < min(slice)
      endif
      ;
      if (palette_top gt palette_bottom) then begin
        ; Non-zero palette range
        ;
        (*sState)[index].color.palette_top   =palette_top
        (*sState)[index].color.palette_bottom=palette_bottom
        ;
        v3d_color_setpalette, sState, index, /ALLNEW
        ;
        if ((*sState)[index].no_colorbar eq 0) then begin
          v3d_cbar_updatewidget, (*sState)[index].oColorbar , $
                                 (*sState)[index].color.palette_top, $
                                 (*sState)[index].color.palette_bottom, $
                               *((*sState)[index].color.palette)
          ; And redraw the colorbar
          (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
        endif
        ;
        ; Update the text widgets with current palette range
        v3d_color_updatewidget, sState, index
        ;
        ; and redraw the view  
        ;message, 'Redraw the view', /INFORMATIONAL
        (*sState)[index].oWindow->Draw, (*sState)[index].oView
        ;
      endif
    endif
    ;
  endif else $ ; 'BUTTON_SLICE_NEWCOLOR'

  if (uval eq 'BUTTON_SLICE_APPLY') then begin ; -------------------------------
    ; Apply new scaling-value settings
    ;
    widget_control, (*sState)[index].wSliceScale, GET_VALUE=strfield
    valfield=double(strfield)
    (*sState)[index].slice.Scale=valfield
    ;
    ; Generate new images
    v3d_slice_makexim, sState, index
    v3d_slice_makeyim, sState, index
    v3d_slice_makezim, sState, index
    ;
    ; Redraw the view
    ;message, 'Redraw the view', /INFORMATIONAL
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_SLICE_APPLY'

  if (uval eq 'BUTTON_SLICE_HELP') then begin ; --------------------------------
    ; Show help
    ;
    v3d_slice_displayhelp, group=(*sState)[index].wDraw
    ;
  endif else $ ; 'BUTTON_SLICE_HELP'

  if (uval eq 'BUTTON_SLICE_CONTROL') then begin ; ----------------------------
    ; Open additional control panel
    ;
    v3d_slice_createcontrolwidget, sState, index
    ;
  endif else $ ; 'BUTTON_SLICE_CONTROL'

  if (uval eq 'BUTTON_SLICE_CLOSE') then begin ; -------------------------------
    ; Close widget without any further action
    ;
    (*sState)[index].hideControlSlice=1
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif ; 'BUTTON_SLICE_CLOSE'

  if ((uval eq 'BUTTON_SLICECON_MINCOL') or $
      (uval eq 'BUTTON_SLICECON_MAXCOL') or $
      (uval eq 'BUTTON_SLICECON_MINISO') or $
      (uval eq 'BUTTON_SLICECON_MAXISO')) then begin ; -------------------------
    ; Set slices to position of point with minimum or maximum color or iso value 
    ;
    if (uval eq 'BUTTON_SLICECON_MINCOL') then begin
      ; Get scalar index of color minimum
      ColorMin=min(*(*sState)[index].box.colordataset, NewIndex)
    endif else if (uval eq 'BUTTON_SLICECON_MAXCOL') then begin
      ; Get scalar index of color maximum
      ColorMax=max(*(*sState)[index].box.colordataset, NewIndex)
    endif else if (uval eq 'BUTTON_SLICECON_MINISO') then begin
      ; Get scalar index of iso minimum
      IsoMin=min(*(*sState)[index].box.isodataset, NewIndex)
    endif else begin
      ; Get scalar index of iso maximum
      IsoMax=max(*(*sState)[index].box.isodataset, NewIndex)
    endelse
    ;
    ; Transform scalar to vector index and save it into sState structure
    IndVec=array_indices([(*sState)[index].box.nx, $
                          (*sState)[index].box.ny, $
                          (*sState)[index].box.nz], $
                         NewIndex, /DIMENSIONS)
    (*sState)[index].slice.ix=IndVec[0]
    (*sState)[index].slice.iy=IndVec[1]
    (*sState)[index].slice.iz=IndVec[2]
    ;
    ; Reposition sliders of current view
    widget_control, (*sState)[index].wSlicexSlider, $
                    SET_VALUE=(*sState)[index].slice.ix
    widget_control, (*sState)[index].wSliceySlider, $
                    SET_VALUE=(*sState)[index].slice.iy
    widget_control, (*sState)[index].wSlicezSlider, $
                    SET_VALUE=(*sState)[index].slice.iz
    ;
    ; Generate new images
    v3d_slice_makexim, sState, index
    v3d_slice_makeyim, sState, index
    v3d_slice_makezim, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
    if ((keyword_set((*sState)[index].slice.couplex)) or $
        (keyword_set((*sState)[index].slice.coupley)) or $
        (keyword_set((*sState)[index].slice.couplez))) then begin
      ; Process the slices of the other views, too
      ;
      nrofviews=n_elements(*sState)
      ;
      for i=0, nrofviews-1 do begin
        if (i ne index) then begin
          ; x slider
          if (keyword_set((*sState)[index].slice.couplex)) then begin
            (*sState)[i].slice.ix=0L > (*sState)[index].slice.ix < ((*sState)[i].box.nx-1L)
            ;
            ; Set the slider value
            if ((*sState)[i].hideControlSlice eq 0) then begin
              widget_control, (*sState)[i].wSlicexSlider, $
                              SET_VALUE=(*sState)[i].slice.ix
            endif
            ;
            ; Generate new image (even if slice is hidden)
            v3d_slice_makexim, sState, i
          endif
          ;
          ; y slider
          if (keyword_set((*sState)[index].slice.coupley)) then begin
            (*sState)[i].slice.iy=0L > (*sState)[index].slice.iy < ((*sState)[i].box.ny-1L)
            ;
            ; Set the slider value
            if ((*sState)[i].hideControlSlice eq 0) then begin
              widget_control, (*sState)[i].wSliceySlider, $
                              SET_VALUE=(*sState)[i].slice.iy
            endif
            ;
            ; Generate new image (even if slice is hidden)
            v3d_slice_makeyim, sState, i
          endif
          ;
          ; z slider
          if (keyword_set((*sState)[index].slice.couplez)) then begin
            (*sState)[i].slice.iz=0L > (*sState)[index].slice.iz < ((*sState)[i].box.nz-1L)
            ;
            ; Set the slider value
            if ((*sState)[i].hideControlSlice eq 0) then begin
              widget_control, (*sState)[i].wSlicezSlider, $
                              SET_VALUE=(*sState)[i].slice.iz
            endif
            ;
            ; Generate new image (even if slice is hidden)
            v3d_slice_makezim, sState, i
          endif
          ;
          ; Redraw the view (if slice is visible)
          (*sState)[i].oSlice   ->GetProperty, HIDE=hide
          (*sState)[i].oSlicexIm->GetProperty, HIDE=hidex
          (*sState)[i].oSliceyIm->GetProperty, HIDE=hidey
          (*sState)[i].oSlicezIm->GetProperty, HIDE=hidez
          if ((hide eq 0) and ((hidex eq 0) or (hidey eq 0) or (hidez eq 0))) then begin
            (*sState)[i].oWindow->Draw, (*sState)[i].oView
          endif
          ;
        endif
      endfor ; i
    endif
    ;
  endif ; 'BUTTON_SLICECON_MINCOL'

  if (uval eq 'BUTTON_SLICECON_CLOSE') then begin ; ----------------------------
    ; Close widget without any further action
    ;
    widget_control, (*sState)[index].wSliceCon, /DESTROY
    ;
  endif ; 'BUTTON_SLICECON_CLOSE'

  return, 1

end ; v3d_slice_event


pro v3d_slice_createcontrolwidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_createcontrolwidget ('v3d_slice_create_control_widget')
;
; PURPOSE:
;   Create the slice control widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_slice_createcontrolwidget(sState, index)
;
; INPUT:
;   index:       ('index') current view index
;
; INPUT/OUTPUT:
;   sState:      ('structure_State') The state variable array
;
; RETURN VALUE:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2007-04-12 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if (not widget_info((*sState)[index].wSliceCon, /VALID_ID))  then begin

    strindex=string(index+1, format='(I0)')
    ;
    wBaseCon=widget_base(TITLE='Vis3D slice ' + strindex, $
                         GROUP=(*sState)[0].wBase, $
                         /ROW, $
                         XPAD=0, YPAD=0)
    (*sState)[index].wSliceCon=wBaseCon
      ;
      wBaseButtons=widget_base(wBaseCon, $
                                                 UVALUE=index, $
                               EVENT_FUNC='v3d_slice_event', $
                               /COLUMN, $
                               XPAD=0, YPAD=0)
        wTopRow=widget_base(wBaseButtons, $
                            /ROW, $
                            XPAD=0, YPAD=0)
          wEntry=widget_label(wTopRow,           VALUE ='Slice ' + strindex + $
                              '                    ')
          wEntry=widget_button(wTopRow,          VALUE ='Close', $
                                                 UVALUE='BUTTON_SLICECON_CLOSE', $
                               TOOLTIP='Close this window without further action')
        ;
        wCenterFrame=widget_base(wBaseButtons, $
                                 /COLUMN, $
                                 XPAD=0, YPAD=2)
          wLine1=widget_base(    wCenterFrame, $
                                 /ROW, $
                                 XPAD=0, YPAD=0)
          wEntry=widget_button(wLine1,           VALUE ='MinCol', $
                                                 UVALUE='BUTTON_SLICECON_MINCOL', $
                               TOOLTIP='Move slices to point with minimum color value')
          wEntry=widget_button(wLine1,           VALUE ='MaxCol', $
                                                 UVALUE='BUTTON_SLICECON_MAXCOL', $
                               TOOLTIP='Move slices to point with maximum color value')
          wEntry=widget_label(wLine1,           VALUE =' ')
          wEntry=widget_button(wLine1,           VALUE ='MinIso', $
                                                 UVALUE='BUTTON_SLICECON_MINISO', $
                               TOOLTIP='Move slices to point with minimum iso value')
          wEntry=widget_button(wLine1,           VALUE ='MaxIso', $
                                                 UVALUE='BUTTON_SLICECON_MAXISO', $
                               TOOLTIP='Move slices to point with maximum iso value')
    ;
    geometry1=widget_info((*sState)[index].wBaseControl,  /GEOMETRY)
    geometry2=widget_info((*sState)[index].wBaseFields,   /GEOMETRY)
    geometry3=widget_info((*sState)[index].wSliceControl, /GEOMETRY)
    widget_control, wBaseCon, $
                 XOFFSET=geometry1.xoffset+geometry2.xoffset+geometry3.xoffset, $
                 YOFFSET=geometry1.yoffset+geometry2.yoffset+geometry3.yoffset+geometry3.ysize+36

    ;
    widget_control, wBaseCon, /REALIZE
    widget_control, wBaseCon, SET_UVALUE=sState
    xmanager, 'vis3d', wBaseCon, /NO_BLOCK

  endif

end ; v3d_slice_createcontrolwidget


pro v3d_slice_updatewidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_updatewidget ('v3d_slice_update_widget')
;
; PURPOSE:
;   Update the slice control widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_slice_updatewidget, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2005-09-02 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if ((*sState)[index].hideControlSlice eq 0) then begin
    (*sState)[index].oSlice->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wSliceShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSliceShowHide, $
                      SET_VALUE='Hide'
    endelse
    ;
    ; --- x slice ---
    widget_control, (*sState)[index].wSlicexSlider, $
                    SET_VALUE=(*sState)[index].slice.ix
    ;
    (*sState)[index].oSlicexIm->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wSlicexShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSlicexShowHide, $
                      SET_VALUE='Hide'
    endelse
    ;
    if (KEYWORD_SET((*sState)[index].slice.couplex)) then begin
      widget_control, (*sState)[index].wSlicexCouple, $
                      SET_VALUE='Decple'
    endif else begin
      widget_control, (*sState)[index].wSlicexCouple, $
                      SET_VALUE='Couple'
    endelse
    ;
    ; --- y slice ---
    widget_control, (*sState)[index].wSliceySlider, $
                    SET_VALUE=(*sState)[index].slice.iy
    ;
    (*sState)[index].oSliceyIm->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wSliceyShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSliceyShowHide, $
                      SET_VALUE='Hide'
    endelse
    ;
    if (KEYWORD_SET((*sState)[index].slice.coupley)) then begin
      widget_control, (*sState)[index].wSliceyCouple, $
                      SET_VALUE='Decple'
    endif else begin
      widget_control, (*sState)[index].wSliceyCouple, $
                      SET_VALUE='Couple'
    endelse
    ;
    ; --- z slice ---
    widget_control, (*sState)[index].wSlicezSlider, $
                    SET_VALUE=(*sState)[index].slice.iz
    ;
    (*sState)[index].oSlicezIm->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wSlicezShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wSlicezShowHide, $
                      SET_VALUE='Hide'
    endelse
    ;
    if (KEYWORD_SET((*sState)[index].slice.couplez)) then begin
      widget_control, (*sState)[index].wSlicezCouple, $
                      SET_VALUE='Decple'
    endif else begin
      widget_control, (*sState)[index].wSlicezCouple, $
                      SET_VALUE='Couple'
    endelse
    ;
    widget_control, (*sState)[index].wSliceScale  , $
                     SET_VALUE=string((*sState)[index].slice.Scale, FORMAT='(G0)')
  endif

end ; v3d_slice_updatewidget


function v3d_slice_createwidget, wBaseSlice, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_createwidget ('v3d_slice_create_widget')
;
; PURPOSE:
;   Create the slice widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_slice_createwidget(wBaseSlice, sState, index)
;
; INPUT:
;   wBaseSlice: ('widget_Base_Slice') parent widget
;   index:      ('index') current view index
;
; INPUT/OUTPUT:
;   sState:     ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-09-05 (B.F.) Written
;   2005-11-21 (B.F.) Help from menu under tool button
;   2007-04-12 (B.F.) Menu option for control widget
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  strindex=string(index+1, format='(I0)')
  
  wBaseButtons=widget_base(wBaseSlice, $
                           /COLUMN, /FRAME, $
                           UVALUE=index, $
                           EVENT_FUNC='v3d_slice_event', $
                           XPAD=2, YPAD=2)
    ;
    wTopRow=widget_base(     wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      wMenu=widget_button(     wTopRow,           VALUE ='Slice ' + strindex, $
                                                  UVALUE='BUTTON_SLICE_MENU', $
                               /MENU, $
                               TOOLTIP='Show slice menu')
;        wEntry=widget_button(  wMenu,             VALUE='Settings', $
;                                                  UVALUE='BUTTON_SLICE_SETTINGS')
        wEntry=widget_button(  wMenu,             VALUE='Control', $
                                                  UVALUE='BUTTON_SLICE_CONTROL')
        wEntry=widget_button(  wMenu,             VALUE='Help', $
                                                  UVALUE='BUTTON_SLICE_HELP')

      wEntry=widget_label(     wTopRow,           VALUE ='        ')
      wEntry=widget_button(    wTopRow,           VALUE ='Hide', $
                                                  UVALUE='BUTTON_SLICE_SHOWHIDEOBJ', $
                               TOOLTIP='Show/hide entire slice object')
      (*sState)[index].wSliceShowHide=wEntry
      wEntry=widget_button(    wTopRow,           VALUE ='Close', $
                                                  UVALUE='BUTTON_SLICE_CLOSE', $
                               TOOLTIP='Close this slice control panel without further action')
    ;
    wCenterFrame=widget_base(wBaseButtons, $
                             /COLUMN, $
                             XPAD=0, YPAD=2)
      ;
      wLine1=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wLine1,          VALUE ='x:')
        wEntry=widget_slider(    wLine1,          VALUE =(*sState)[index].slice.ix, $
                                                  UVALUE='SLIDER_SLICE_X', $
                                 /DRAG, $
                                 MINIMUM=0, $
                                 MAXIMUM=(*sState)[index].box.nx-1L, $
                                 SCROLL=1)
        (*sState)[index].wSlicexSlider=wEntry
        ;
        wEntry=widget_button(    wLine1,          VALUE ='Hide', $
                                                  UVALUE='BUTTON_SLICE_XSHOWHIDEOBJ', $
                                 TOOLTIP='Show/hide x slice')
        (*sState)[index].wSlicexShowHide=wEntry
        ;
        wEntry=widget_button(    wLine1,          VALUE ='Couple', $
                                                  UVALUE='BUTTON_SLICE_XCOUPLE', $
                                 TOOLTIP='Couple/decouple the x slices of all views')
        (*sState)[index].wSlicexCouple=wEntry
      ;
      wline2=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wline2,          VALUE ='y:')
        wEntry=widget_slider(    wline2,          VALUE =(*sState)[index].slice.iy, $
                                                  UVALUE='SLIDER_SLICE_Y', $
                                 /DRAG, $
                                 MINIMUM=0, $
                                 MAXIMUM=(*sState)[index].box.ny-1L, $
                                 SCROLL=1)
        (*sState)[index].wSliceySlider=wEntry
        ;
        wEntry=widget_button(    wline2,          VALUE ='Hide', $
                                                  UVALUE='BUTTON_SLICE_YSHOWHIDEOBJ', $
                                 TOOLTIP='Show/hide y slice')
        (*sState)[index].wSliceyShowHide=wEntry
        ;
        wEntry=widget_button(    wline2,          VALUE ='Couple', $
                                                  UVALUE='BUTTON_SLICE_YCOUPLE', $
                                 TOOLTIP='Couple/decouple the y slices of all views')
        (*sState)[index].wSliceyCouple=wEntry
      ;
      wline3=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wline3,          VALUE ='z:')
        wEntry=widget_slider(    wline3,          VALUE =(*sState)[index].slice.iz, $
                                                  UVALUE='SLIDER_SLICE_Z', $
                                 /DRAG, $
                                 MINIMUM=0, $
                                 MAXIMUM=(*sState)[index].box.nz-1L, $
                                 SCROLL=1)
        (*sState)[index].wSlicezSlider=wEntry
        ;
        wEntry=widget_button(    wline3,          VALUE ='Hide', $
                                                  UVALUE='BUTTON_SLICE_ZSHOWHIDEOBJ', $
                                 TOOLTIP='Show/hide z slice')
        (*sState)[index].wSlicezShowHide=wEntry
        ;
        wEntry=widget_button(    wline3,          VALUE ='Couple', $
                                                  UVALUE='BUTTON_SLICE_ZCOUPLE', $
                                 TOOLTIP='Couple/decouple the z slices of all views')
        (*sState)[index].wSlicezCouple=wEntry
        ;
    wBottomRow=widget_base(  wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      wEntry=widget_label(     wBottomRow,      VALUE ='Scl:')
      wEntry=widget_text(      wBottomRow,      VALUE=string((*sState)[index].slice.Scale, $
                                                             FORMAT='(G0)'), $
                                                UVALUE='TEXT_SLICE_SCALE', $
                               /EDITABLE, $
                               XSIZE=12)
      (*sState)[index].wSliceScale=wEntry
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
                                                  UVALUE='BUTTON_SLICE_APPLY', $
                               TOOLTIP='Use current scaling value')
      wEntry=widget_button(    wBottomRow,        VALUE ='Color', $
                                                  UVALUE='BUTTON_SLICE_NEWCOLOR', $
                               TOOLTIP='Adjust color palette to visible slices')
      ;
;      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
;                               /ALIGN_CENTER,     UVALUE='BUTTON_SLICE_APPLY', $
;                               TOOLTIP='Apply')
  ;

  return, wBaseButtons

end ; v3d_slice_createwidget


pro v3d_slice_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_slice_destroy ('v3d_slice_destroy')
;
; PURPOSE:
;   Destroy slice structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_slice_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Just don't do anything.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;
; HISTORY:
;   2005-09-05 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ; Actually, there is nothing to do yet.

end ; v3d_slice_destroy


function v3d_slice_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_slice_init ('v3d_slice_initialize')
;
; PURPOSE:
;   Initialize slice structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   slice=v3d_slice_init()
;
; MODIFICATION HISTORY:
;   2005-09-05 Written by Bernd Freytag, L.A.
;   2011-04-01 (B.F. Lyon) Couple Slices by default
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Create an anonymous structure containing the basic data for the slices
  slice={$
    ix:                    0L, $
    iy:                    0L, $
    iz:                    0L, $
    hidex:                 v3dpar_Slices_hide, $
    hidey:                 v3dpar_Slices_hide, $
    hidez:                 v3dpar_Slices_hide, $
    couplex:               1, $
    coupley:               1, $
    couplez:               1, $
    Scale:                 0.0 $
       }

  return, slice

end ; v3d_slice_init
