;-------------------------------------------------------------------------------
; Vis3D module: sState
;-------------------------------------------------------------------------------
; Contains:
;   v3d_sstate_putcommon:      Put state variable into common block to allow external access
;   v3d_sstate_destroystructs: Destroy sub-structures
;   v3d_sstate_freemem:        Free memory by destroying all v3d graphics objects
;   v3d_sstate_init:           Initialize the state variable
;-------------------------------------------------------------------------------


pro v3d_sstate_putcommon, sState, commonblock
;-------------------------------------------------------------------------------
; NAME:
;   v3d_sstate_putcommon ('v3d_sstate_put_commonblock')
;
; PURPOSE:
;   Put state variable into common block to allow external access.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_sstate_putcommon, sState, commonblock
;
; INPUT:
;   commonblock: ('common_block') index of common block (so far: 1 or 2)
;
; INPUT:
;   sState: ('structure_State') array of structures with state information
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-08-17 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

compile_opt hidden, strictarr

  if (commonblock eq 1) then begin
    message, 'Putting variable in common block: ' + $
             '"common v3dcommon' + strtrim(string(commonblock), 2) + ', sState"', /INFORMATIONAL
    common v3dcommon1, sStatetmp1
    sStatetmp1=sState
    ;
  endif else if (commonblock eq 2) then begin
    message, 'Putting variable in common block: ' + $
             '"common v3dcommon' + strtrim(string(commonblock), 2) + ', sState"', /INFORMATIONAL
    common v3dcommon2, sStatetmp2
    sStatetmp2=sState
    ;
  endif

end ; v3d_sstate_putcommon


pro v3d_sstate_destroystructs, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_sstate_destroystructs ('v3d_destroy_structures')
;
; PURPOSE:
;   Destroy sub-structures and free memory used in the structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_sstate_destroystructs, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Free existing heap variables and clean up cleanly using a null_pointer.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;   TODO: Make the procedure itself find all pointers to free?
;
; HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-07-14 (B.F.) More comments, 'v3d_findtag' checks removed,
;                     'v3d_addtag' calls replaced
;   2005-08-31 (B.F.) v3d_destroydata3d -> v3d_destroystructs
;   2005-09-07 (B.F.) v3d_destroystructs -> v3d_sstate_destroystructs
;-------------------------------------------------------------------------------

  compile_opt hidden

  v3d_box_destroy,    sState, index
  v3d_color_destroy,  sState, index
  v3d_iso_destroy,    sState, index
  v3d_dots_destroy,   sState, index
  v3d_vol_destroy,    sState, index
  v3d_slice_destroy,  sState, index
  v3d_stream_destroy, sState, index

  v3d_axis_destroy,  sState, index

end ; v3d_sstate_destroystructs


pro v3d_sstate_freemem, sState
;-------------------------------------------------------------------------------
; NAME:
;   v3d_sstate_freemem ('v3d_sstate_free_memory')
;
; PURPOSE:
;   Free memory by destroying all v3d graphics objects.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_sstate_freemem, sState
;
; INPUT/OUTPUT:
;   sState: ('s_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-13 (B.F.) More comments
;   2005-07-26 (B.F.) Put into separate file
;   2005-08-31 (B.F.) put into v3d_initsstate.pro file
;-------------------------------------------------------------------------------

  compile_opt hidden

  ; ToDo: Check use of heap_free.

  message, 'Destroying sState structure', /INFORMATIONAL

  nrofviews=n_elements(*sState)

  for i=0, nrofviews-1 do begin
    ; Destroy all heap variables in the data structures of each view
    v3d_sstate_destroystructs, sState, i
    ;
    ; Destroy all objects
    OBJ_DESTROY, (*sState)[i].oView
    OBJ_DESTROY, (*sState)[i].oTrack
    ;
    if ((*sState)[i].no_colorbar eq 0) then begin
      OBJ_DESTROY, (*sState)[i].oCbarView
      OBJ_DESTROY, (*sState)[i].oCbarText 
    endif
  endfor ; i

  ptr_free, sState

end ; v3d_sstate_freemem


function v3d_sstate_init, nrofviews, export_on=export_on
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_sstate_init ('v3d_structure_state_init')
;
; PURPOSE:
;   Initialize the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_sstate_init(nrofviews, export_on=export_on)
;
; INPUT:
;   nrofviews:    ('number_of_views') integer with number of view windows
;
; OUTPUT:
;   result:       sState: the state variable array
;
; KEYWORDS:
;   export_on:    ('export_on') If set the eport mode is activated and
;                 a restricted state variable is produced.
;
; MODIFICATION HISTORY:
;   2005-08-05 (B.F.) Written
;   2005-08-06 (B.F.) More entries
;   2005-08-31 (B.F.) v3d_initdots() added
;   2005-09-04 (B.F.) oHolder object removed
;   2005-09-05 (B.F.) v3d_slice_init() added
;   2006-06-07 (B.F.) v3d_vol_init() added
;   2006-06-16 (B.F.) wVolHzBuffer added
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  box   =v3d_box_init()
  color =v3d_color_init()
  iso   =v3d_iso_init()
  dots  =v3d_dots_init()
  vol   =v3d_vol_init()
  slice =v3d_slice_init()
  stream=v3d_stream_init()
  light =v3d_light_init()
  axis  =v3d_axis_init()

  ; Create the state variable for a single view
  if (NOT KEYWORD_SET(export_on)) then begin
    ; Full sState variable
    sState_new={ $
      box:      box, $
      color:    color, $
      iso:      iso, $
      dots:     dots, $
      vol:      vol, $
      slice:    slice, $
      stream:   stream, $
      axis:     axis, $
      ;
      oWindow:       obj_new(), $
      oCbarDrawable: obj_new(), $
      oView:         obj_new(), $
       oWorld:       obj_new(), $
        oModel:      obj_new(), $
         oPoly:      obj_new(), $
         oPolyHigh:  obj_new(), $
         oSlice:     obj_new(), $
          oSlicexIm: obj_new(), $
          oSliceyIm: obj_new(), $
          oSlicezIm: obj_new(), $
         oStream:    obj_new(), $
         oDots:      obj_new(), $
         oBox:       obj_new(), $
;         oBoxEdges:  obj_new(), $
;          oBoxEdgeX: obj_new(), $
;          oBoxEdgeY: obj_new(), $
;          oBoxEdgeZ: obj_new(), $
         oUser:      obj_new(), $
         oUserHigh:  obj_new(), $
         oVol:       obj_new(), $
        oLight1:     obj_new(), $
        oLight2:     obj_new(), $
      oTrack:        obj_new(), $
      oCbarView:     obj_new(), $
       oColorbar:    obj_new(), $
        oCbarText:   obj_new(), $
      ;
      wBase:            0L, $
       wColorbar:       0L, $
       wDraw:           0L, $
      wBaseControl:     0L, $
      wBaseFields:      0L, $
       wMenu:           0L, $
       wShowHideAxis:   0L, $
       wIsoControl:     0L, $
        wDecimVal:      0L, $
        wIsoVal:        0L, $
        wShowHideIso:   0L, $
       wColorControl:   0L, $
        wPalMin:        0L, $
        wPalMax:        0L, $
        wColorSet:      0L, $
        wColorRed:      0L, $
        wColorGreen:    0L, $
        wColorBlue:     0L, $
       wDotsControl:    0L, $
        wDotsCurrIsoMin:0L, $
        wDotsCurrIsoMax:0L, $
        wDotsNumber:    0L, $
        wDotsShowHide:  0L, $
       wVolControl:     0L, $
        wVolShowHide:   0L, $
        wVolHInter:     0L, $
        wVolHzBuffer:   0L, $
        wVolHxStep:     0L, $
        wVolHyStep:     0L, $
        wVolHzStep:     0L, $
        wVolLInter:     0L, $
        wVolLzBuffer:   0L, $
        wVolLxStep:     0L, $
        wVolLyStep:     0L, $
        wVolLzStep:     0L, $
        wVolScale:      0L, $
       wSliceControl:   0L, $
        wSliceShowHide: 0L, $
        wSlicexSlider:  0L, $
        wSlicexShowHide:0L, $
        wSlicexCouple:  0L, $
        wSliceySlider:  0L, $
        wSliceyShowHide:0L, $
        wSliceyCouple:  0L, $
        wSlicezSlider:  0L, $
        wSlicezShowHide:0L, $
        wSlicezCouple:  0L, $
        wSliceScale:    0L, $
        wSliceCon:      0L, $
       wStreamControl:  0L, $
        wStreamShowHide:0L, $
        wStreamNLines:  0L, $
        wStreamNPoints: 0L, $
        wStreamStep:    0L, $
        wStreamSet:     0L, $
        wStreamRed:     0L, $
        wStreamGreen:   0L, $
        wStreamBlue:    0L, $
        wStreamCon:     0L, $
        wStreamSlNPts:  0L, $
      ;
      hideLights:       1, $
      hideAxis:         1, $
      ;
      hideControlColor: 0, $
      hideControlIso:   0, $
      hideControlDots:  1, $
      hideControlVol:   0, $
      hideControlSlice: 0, $
      hideControlStream:1, $
      ;
      isCurrIsoColor:   0, $
      isCurrDotsColor:  0, $
      isCurrVolColor:   0, $
      isCurrSliceColor: 0, $
      ;
      renderer:       v3dpar_Renderer_default, $
      projection:     v3dpar_Projection_default, $
      rotateaxis:     v3dpar_RotateAxis_default, $
      rotateangle:    v3dpar_RotateAngle_default, $
      rotatematrix:   diag_matrix(replicate(1.0, 4)), $
      center:         dblarr(3), $
      scale:          v3dpar_Scale_default, $
      viewxsize:      v3dpar_xsize_default, $
      viewysize:      v3dpar_ysize_default, $
      ;
      title:          'Vis3D View', $
      no_controls:    0, $
      no_menu:        0, $
      no_colorbar:    0, $
      ;
      rotzoompan:     0, $
      key:            0b, $
      ;
      movie_index:            0, $
      movie_dimensions:       [v3dpar_xsize_default, v3dpar_ysize_default], $
      movie_nframes:          v3dpar_Movie_nFrames_default, $
      movie_mode:             v3dpar_Movie_Mode_default, $
      movie_oscillationangle: v3dpar_Movie_OscillationAngle_default, $
      movie_rotateaxis:       v3dpar_Movie_RotateAxis_default, $
      movie_transsystem:      v3dpar_Movie_TransSystem_default, $
      movie_framerate:        v3dpar_Movie_FrameRate_default, $
      movie_quality:          v3dpar_Movie_Quality_default, $
      movie_antialiasfactor:  v3dpar_Movie_AntiAliasFactor_default, $
      movie_format:           v3dpar_Movie_Format_default, $
      movie_imageformat:      v3dpar_Movie_ImageFormat_default, $
      movie_filename:         (file_search('.',  /FULLY_QUALIFY_PATH))[0] + '/' + $
                                v3dpar_Movie_FileName_default $
               }
    ;
  endif else begin
    ; Simplifed sState for Export mode
    sState_new={ $
      box:      box, $
      color:    color, $
      iso:      iso, $
      dots:     dots, $
      vol:      vol, $
      slice:    slice, $
      stream:   stream, $
      axis:     axis, $
      ;
      oView:         obj_new(), $
       oWorld:       obj_new(), $
        oModel:      obj_new(), $
         oPolyHigh:  obj_new(), $
         oSlice:     obj_new(), $
          oSlicexIm: obj_new(), $
          oSliceyIm: obj_new(), $
          oSlicezIm: obj_new(), $
         oStream:    obj_new(), $
         oDots:      obj_new(), $
         oBox:       obj_new(), $
;         oBoxEdges:  obj_new(), $
;          oBoxEdgeX: obj_new(), $
;          oBoxEdgeY: obj_new(), $
;          oBoxEdgeZ: obj_new(), $
         oUser:      obj_new(), $
         oUserHigh:  obj_new(), $
         oVol:       obj_new(), $
        oLight1:     obj_new(), $
        oLight2:     obj_new(), $
      oTrack:        obj_new(), $
      oCbarView:     obj_new(), $
       oColorbar:    obj_new(), $
        oCbarText:   obj_new(), $
      ;
      hideLights:       1, $
      hideAxis:         1, $
      ;
      isCurrIsoColor:   0, $
      isCurrDotsColor:  0, $
      isCurrVolColor:   0, $
      isCurrSliceColor: 0, $
      ; 
      renderer:       v3dpar_Renderer_default, $
      projection:     v3dpar_Projection_default, $
      rotateaxis:     v3dpar_RotateAxis_default, $
      rotateangle:    v3dpar_RotateAngle_default, $
      rotatematrix:   diag_matrix(replicate(1.0, 4)), $
      center:         dblarr(3), $
      scale:          v3dpar_Scale_default, $
      viewxsize:      v3dpar_xsize_default, $
      viewysize:      v3dpar_ysize_default, $
      ;
      title:          'Visualizing 3D arrays', $
      no_controls:    0, $
      no_menu:        0, $
      no_colorbar:    0 $
               }
    ;
  endelse

  ; Make a pointer to the array of state variables
  sState=ptr_new(temporary(replicate(sState_new, nrofviews)))

  return, sState

end ; v3d_sstate_init
