;-------------------------------------------------------------------------------
; Vis3D module: Streamlines
;-------------------------------------------------------------------------------
; Contains:
;   v3d_stream_displayhelp:         Display help text for streamline generation widget
;   v3d_stream_createvertcoords:    Integrate streamlines
;   v3d_stream_set_event:           Handle all streamline settings events
;   v3d_stream_event:               Handle all streamline events
;   v3d_stream_createsetwidget:     Create the stream settings widget
;   v3d_stream_createcontrolwidget: Create the stream control widget
;   v3d_stream_updatewidget:        Update the streamline control widget
;   v3d_stream_createwidget:        Create the stream widget
;   v3d_stream_destroy:             Destroy stream structure and free memory used in it
;   v3d_stream_init:                Initialize stream structure
;-------------------------------------------------------------------------------


pro v3d_stream_displayhelp, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_displayhelp ('v3d_stream_display_help')
;
; PURPOSE:
;   Display help text for streamline generation widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_stream_displayhelp, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-09-14 (B.F.) Written
;   2005-10-25 (B.F.) Updated info
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Streamline generation', $
'=====================', $
'', $
'This panel controls the generation of pseudo-streamlines.', $
'', $
'A click on the "Stream X" field opens a menu with entries "Settings",', $
'"Control", and "Help". These open panels that allow to change additional', $
'settings, to directly control some parameters, or to display help,', $
'respectively.', $
'', $
'The "Hide" or "Show" button toggles the visibility of the streamlines.', $
'', $
'The "Close" button just closes the panel (not the entire window) without', $
'starting a generation process.', $
'', $
'The "NLin" field specifies the number of streamlines to use.', $
'The "NPts" field specifies the number of points per streamline.', $
'The "Step" field specifies the integration step size.', $
'', $
'The "Apply" button starts the streamline generation (and keeps the window open).']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' Help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_stream_displayhelp


pro v3d_stream_createvertcoords, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_createvertcoords ('v3d_stream_create_vertex_coordinates')
;
; PURPOSE:
;   Integrate streamlines.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_stream_createvertcoords, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2005-09-14 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ; Save system time at start of routine
  time0=systime(1)

  ; Clean up
  if (ptr_valid((*sState)[index].stream.vertices)) then begin
    ptr_free, (*sState)[index].stream.vertices
  endif

  ; Get number of streamlines and points on streamline, integration step size
  nlines =(*sState)[index].stream.nlines
  npoints=(*sState)[index].stream.npoints
  step   =(*sState)[index].stream.step

  ; Compute "proper" number of streamlines
  ;xc     =findgen((*sState)[index].box.nx)
  x0     =(*sState)[index].box.xrange[0]
  y0     =(*sState)[index].box.yrange[0]
  z0     =(*sState)[index].box.zrange[0]
  dx     =(*sState)[index].box.xrange[1]-(*sState)[index].box.xrange[0]
  dy     =(*sState)[index].box.yrange[1]-(*sState)[index].box.yrange[0]
  dz     =(*sState)[index].box.zrange[1]-(*sState)[index].box.zrange[0]
  dVol   =dx*dy*dz/float(nlines)
  dLen   =dVol^(1.0/3.0)
  nsx    =round(((*sState)[index].box.xrange[1]-(*sState)[index].box.xrange[0])/dLen) > 1L
  nsy    =round(((*sState)[index].box.yrange[1]-(*sState)[index].box.yrange[0])/dLen) > 1L
  nsz    =round(((*sState)[index].box.zrange[1]-(*sState)[index].box.zrange[0])/dLen) > 1L
  nlines =nsx*nsy*nsz

  ; Create new vertex coordinate array
  (*sState)[index].stream.vertices=ptr_new(fltarr(3, npoints, nlines))

  ; Set initial positions
  x=reform(rebin(reform((findgen(nsx)+0.5)/float(nsx)*dx+x0, nsx,1,1), nsx,nsy,nsz), nlines)
  y=reform(rebin(reform((findgen(nsy)+0.5)/float(nsy)*dy+y0, 1,nsy,1), nsx,nsy,nsz), nlines)
  z=reform(rebin(reform((findgen(nsz)+0.5)/float(nsz)*dz+z0, 1,1,nsz), nsx,nsy,nsz), nlines)
  j=0L
  (*(*sState)[index].stream.vertices)[0, j, *]=x
  (*(*sState)[index].stream.vertices)[1, j, *]=y
  (*(*sState)[index].stream.vertices)[2, j, *]=z

  ; Integrate
  for j=1,npoints-1 do begin
    ; Interpolate change vectors (velocities) at current position
    vx=interpolate(*(*sState)[index].box.vecxdataset, x, y, z)
    vy=interpolate(*(*sState)[index].box.vecydataset, x, y, z)
    vz=interpolate(*(*sState)[index].box.veczdataset, x, y, z)
    ;
    ; Add change
    x=x + step*vx
    y=y + step*vy
    z=z + step*vz
    ;
    ; Put into vertex pointer array
    (*(*sState)[index].stream.vertices)[0, j, *]=x
    (*(*sState)[index].stream.vertices)[1, j, *]=y
    (*(*sState)[index].stream.vertices)[2, j, *]=z
  endfor ; j

  ; Modify or create graphics object
  if (obj_valid((*sState)[index].oStream)) then begin
    (*sState)[index].oStream->SetProperty, $
                        DATA=reform(*(*sState)[index].stream.vertices, 3, nlines*npoints), $
                        COLOR=(*sState)[index].stream.solid_color
  endif else begin
    (*sState)[index].oStream=obj_new('IDLgrPolygon', $
                             reform(*(*sState)[index].stream.vertices, 3, nlines*npoints), $
                                     COLOR=(*sState)[index].stream.solid_color, $
                                     STYLE=0, $
                                     HIDE=1)
  endelse

  message, 'Routine needed ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL

end ; v3d_stream_createvertcoords


pro v3d_stream_set_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_set_event ('v3d_stream_settings_event')
;
; PURPOSE:
;   Handle all streamline settings events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-09-21 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

;  message, '', /INFORMATIONAL
;  help, /str, sEvent
;  help, sState, index, uval

end ; v3d_stream_set_event


function v3d_stream_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_event ('v3d_stream_event')
;
; PURPOSE:
;   Handle all streamline events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-09-14 (B.F.) Derived from v3d_dots_event
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

;  message, '', /INFORMATIONAL
;  help, /str, sEvent
;  help, sState, index, uval

  if (uval eq 'BUTTON_STREAM_APPLY') then begin ; ------------------------------
    ; Generate new streamlines
    ;
    ; Fetch new value for number of streamlines
    widget_control, (*sState)[index].wStreamNLines, $
                    GET_VALUE=strStreamNLines
    (*sState)[index].stream.nlines=1L > long(strStreamNLines) < 1000000L
    ;
    ; Fetch new value for number of points on streamline
    widget_control, (*sState)[index].wStreamNPoints, $
                    GET_VALUE=strStreamNPoints
    (*sState)[index].stream.npoints=2L > $
                                    (long(strStreamNPoints) < $
                                     (5000000L/(*sState)[index].stream.nlines))
    ;
    ; Fetch new value for integration step size
    widget_control, (*sState)[index].wStreamStep, $
                    GET_VALUE=strStreamStep
    (*sState)[index].stream.step=float(strStreamStep)
    ;
    ; Generate new vertex coordinates
    v3d_stream_createvertcoords, sState, index
    ;
    ; Compute vertex values
    ;v3d_stream_computevertvalues, sState, index
    ;
    ; Compute vertex colors
    ;v3d_stream_computevertcolors, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_STREAM_APPLY'

  if (uval eq 'BUTTON_STREAM_SHOWHIDEOBJ') then begin ; ------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oStream->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oStream->SetProperty, HIDE=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wStreamShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wStreamShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_STREAM_SHOWHIDEOBJ'

  if (uval eq 'BUTTON_STREAM_HELP') then begin ; -------------------------------
    ; Show help
    ;
    v3d_stream_displayhelp, group=(*sState)[index].wDraw
    ;
  endif else $ ; 'BUTTON_STREAM_HELP'

  if (uval eq 'BUTTON_STREAM_SETTINGS') then begin ; ---------------------------
    ; Open additional settings panel
    ;
    v3d_stream_createsetwidget, sState, index
    ;
  endif else $ ; 'BUTTON_STREAM_SETTINGS'

  if (uval eq 'BUTTON_STREAM_CONTROL') then begin ; ----------------------------
    ; Open additional control panel
    ;
    v3d_stream_createcontrolwidget, sState, index
    ;
  endif else $ ; 'BUTTON_STREAM_CONTROL'

  if (uval eq 'BUTTON_STREAM_CLOSE') then begin ; ------------------------------
    ; Close widget without any further action
    ;
    (*sState)[index].hideControlStream=1
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif else $ ; 'BUTTON_STREAM_CLOSE'

  if (uval eq 'BUTTON_STREAMSET_APPLY') then begin ; ---------------------------
    ; Apply changes from additional settings widget
    ;
    widget_control, (*sState)[index].wStreamRed,   GET_VALUE=strred
    widget_control, (*sState)[index].wStreamGreen, GET_VALUE=strgreen
    widget_control, (*sState)[index].wStreamBlue,  GET_VALUE=strblue
    ;
    (*sState)[index].stream.solid_color=byte(long([strred[0], strgreen[0], strblue[0]]))
    ;
    ; Generate new vertex coordinates
    v3d_stream_createvertcoords, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_STREAMSET_APPLY'

  if (uval eq 'BUTTON_STREAMSET_CLOSE') then begin ; ---------------------------
    ; Close widget without any further action
    ;
    widget_control, (*sState)[index].wStreamSet, /DESTROY
    ;
  endif else $ ; 'BUTTON_STREAMSET_CLOSE'

  if (uval eq 'SLIDER_STREAMCON_NPOINTS') then begin ; -------------------------
    ; Slider
    ;
    (*sState)[index].stream.npoints=sEvent.value
    if (widget_info((*sState)[index].wStreamNPoints, /VALID_ID)) then begin
      widget_control, (*sState)[index].wStreamNPoints, $
                      SET_VALUE=string((*sState)[index].stream.npoints, format='(I0)')
    endif
    ;
    ; Generate new vertex coordinates
    v3d_stream_createvertcoords, sState, index
    ;
    ; Compute vertex values
    ;v3d_stream_computevertvalues, sState, index
    ;
    ; Compute vertex colors
    ;v3d_stream_computevertcolors, sState, index
    ;
    ; Redraw the view
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'SLIDER_STREAMCON_NPOINTS'

  if (uval eq 'BUTTON_STREAMCON_CLOSE') then begin ; ---------------------------
    ; Close widget without any further action
    ;
    widget_control, (*sState)[index].wStreamCon, /DESTROY
    ;
  endif ; 'BUTTON_STREAMCON_CLOSE'

  return, 1

end ; v3d_stream_event


pro v3d_stream_createsetwidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_createsetwidget ('v3d_stream_create_settings_widget')
;
; PURPOSE:
;   Create the stream settings widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_stream_createsetwidget(sState, index)
;
; INPUT:
;   index:       ('index') current view index
;
; INPUT/OUTPUT:
;   sState:      ('structure_State') The state variable array
;
; RETURN VALUE:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-09-22 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if (not widget_info((*sState)[index].wStreamSet, /VALID_ID)) then begin

    strindex=string(index+1, format='(I0)')
    ;
    wBaseSet=widget_base(TITLE='Vis3D stream ' + strindex, $
                         GROUP=(*sState)[0].wBase, $
                         /ROW, $
                         XPAD=0, YPAD=0)
    (*sState)[index].wStreamSet=wBaseSet
      ;
      wBaseButtons=widget_base(wBaseSet, $
                                                 UVALUE=index, $
                               EVENT_FUNC='v3d_stream_event', $
                               /COLUMN, $
                               XPAD=0, YPAD=0)
        wTopRow=widget_base(wBaseButtons, $
                            /ROW, $
                            XPAD=0, YPAD=0)
          wEntry=widget_label(wTopRow,           VALUE ='Stream ' + strindex + $
                              '                     ')
          wEntry=widget_button(wTopRow,          VALUE ='Close', $
                                                 UVALUE='BUTTON_STREAMSET_CLOSE', $
                               TOOLTIP='Close this window without further action')

        ;
        wCenterFrame=widget_base(wBaseButtons, $
                                 /COLUMN, $
                                 XPAD=0, YPAD=2)
          wLine1=widget_base(    wCenterFrame, $
                                 /ROW, $
                                 XPAD=0, YPAD=0)
            str_red  =string((*sState)[index].stream.solid_color[0], format='(I0)')
            str_green=string((*sState)[index].stream.solid_color[1], format='(I0)')
            str_blue =string((*sState)[index].stream.solid_color[2], format='(I0)')
            wEntry=widget_label( wLine1,          VALUE ='RED:')
            wEntry=widget_text(  wLine1,          VALUE=str_red, $
                                                  UVALUE='TEXT_STREAMSET_RED', $
                                 /EDITABLE, $
                                 XSIZE=3)
            (*sState)[index].wStreamRed=wEntry
            wEntry=widget_label( wLine1,          VALUE ='GREEN:')
            wEntry=widget_text(  wLine1,          VALUE=str_green, $
                                                  UVALUE='TEXT_STREAMSET_GREEN', $
                                 /EDITABLE, $
                                 XSIZE=3)
            (*sState)[index].wStreamGreen=wEntry
            wEntry=widget_label( wLine1,          VALUE ='BLUE:')
            wEntry=widget_text(  wLine1,          VALUE=str_blue, $
                                                  UVALUE='TEXT_STREAMSET_BLUE', $
                                 /EDITABLE, $
                                 XSIZE=3)
            (*sState)[index].wStreamBlue=wEntry
        ;
        wBottomRow=widget_base(wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
          wEntry=widget_button(wBottomRow,        VALUE ='Apply', $
                                                  UVALUE='BUTTON_STREAMSET_APPLY', $
                               TOOLTIP='Apply changes in settings')
    ;
    geometry1=widget_info((*sState)[index].wBaseControl,   /GEOMETRY)
    geometry2=widget_info((*sState)[index].wBaseFields,    /GEOMETRY)
    geometry3=widget_info((*sState)[index].wStreamControl, /GEOMETRY)
    widget_control, wBaseSet, $
                 XOFFSET=geometry1.xoffset+geometry2.xoffset+geometry3.xoffset, $
                 YOFFSET=geometry1.yoffset+geometry2.yoffset+geometry3.yoffset+geometry3.ysize+30

    ;
    widget_control, wBaseSet, /REALIZE
    widget_control, wBaseSet, SET_UVALUE=sState
    xmanager, 'vis3d', wBaseSet, /NO_BLOCK, EVENT_HANDLER='v3d_stream_set_event'

  endif

end ; v3d_stream_createsetwidget


pro v3d_stream_createcontrolwidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_createcontrolwidget ('v3d_stream_create_control_widget')
;
; PURPOSE:
;   Create the stream control widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_stream_createcontrolwidget(sState, index)
;
; INPUT:
;   index:       ('index') current view index
;
; INPUT/OUTPUT:
;   sState:      ('structure_State') The state variable array
;
; RETURN VALUE:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2005-09-22 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if (not widget_info((*sState)[index].wStreamCon, /VALID_ID))  then begin

    strindex=string(index+1, format='(I0)')
    ;
    wBaseCon=widget_base(TITLE='Vis3D stream ' + strindex, $
                         GROUP=(*sState)[0].wBase, $
                         /ROW, $
                         XPAD=0, YPAD=0)
    (*sState)[index].wStreamCon=wBaseCon
      ;
      wBaseButtons=widget_base(wBaseCon, $
                                                 UVALUE=index, $
                               EVENT_FUNC='v3d_stream_event', $
                               /COLUMN, $
                               XPAD=0, YPAD=0)
        wTopRow=widget_base(wBaseButtons, $
                            /ROW, $
                            XPAD=0, YPAD=0)
          wEntry=widget_label(wTopRow,           VALUE ='Stream ' + strindex + $
                              '                     ')
          wEntry=widget_button(wTopRow,          VALUE ='Close', $
                                                 UVALUE='BUTTON_STREAMCON_CLOSE', $
                               TOOLTIP='Close this window without further action')

        ;
        wCenterFrame=widget_base(wBaseButtons, $
                                 /COLUMN, $
                                 XPAD=0, YPAD=2)
          wLine1=widget_base(    wCenterFrame, $
                                 /ROW, $
                                 XPAD=0, YPAD=0)
            wEntry=widget_label(   wLine1,       VALUE ='Npts:')
            wEntry=widget_slider(  wLine1,       VALUE =(*sState)[index].stream.npoints, $
                                                 UVALUE='SLIDER_STREAMCON_NPOINTS', $
                                   /DRAG, $
                                   XSIZE=200, $
                                   MINIMUM=2, $
                                   MAXIMUM=((*sState)[index].stream.npoints*5 ) > 100, $
                                   SCROLL =((*sState)[index].stream.npoints/20) > 2)
        (*sState)[index].wStreamSlNPts=wEntry
    ;
    geometry1=widget_info((*sState)[index].wBaseControl,   /GEOMETRY)
    geometry2=widget_info((*sState)[index].wBaseFields,    /GEOMETRY)
    geometry3=widget_info((*sState)[index].wStreamControl, /GEOMETRY)
    widget_control, wBaseCon, $
                 XOFFSET=geometry1.xoffset+geometry2.xoffset+geometry3.xoffset, $
                 YOFFSET=geometry1.yoffset+geometry2.yoffset+geometry3.yoffset+geometry3.ysize+30

    ;
    widget_control, wBaseCon, /REALIZE
    widget_control, wBaseCon, SET_UVALUE=sState
    xmanager, 'vis3d', wBaseCon, /NO_BLOCK

  endif

end ; v3d_stream_createcontrolwidget


pro v3d_stream_updatewidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_updatewidget ('v3d_stream_update_widget')
;
; PURPOSE:
;   Update the streamline control widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_stream_updatewidget, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2005-09-14 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if ((*sState)[index].hideControlStream eq 0) then begin
    ;
    ; Display number of streamlines
    widget_control, (*sState)[index].wStreamNLines  , $
                    SET_VALUE=string((*sState)[index].stream.nlines, FORMAT='(I0)')
    ;
    ; Display number of points on each streamline
    widget_control, (*sState)[index].wStreamNPoints  , $
                    SET_VALUE=string((*sState)[index].stream.npoints, FORMAT='(I0)')
    ;
    ; Display integration step size
    widget_control, (*sState)[index].wStreamStep  , $
                    SET_VALUE=string((*sState)[index].stream.step, FORMAT='(G0)')
    ;
    ; Use either "Show" or "Hide" button
    (*sState)[index].oStream->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wStreamShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wStreamShowHide, $
                      SET_VALUE='Hide'
    endelse
    ;
  endif

end ; v3d_stream_updatewidget


function v3d_stream_createwidget, wBaseStream, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_createwidget ('v3d_stream_create_widget')
;
; PURPOSE:
;   Create the stream widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_stream_createwidget(wBaseStream, sState, index)
;
; INPUT:
;   wBaseStream: ('widget_Base_Stream') parent widget
;   index:       ('index') current view index
;
; INPUT/OUTPUT:
;   sState:      ('structure_State') The state variable array
;
; RETURN VALUE:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; TODO:
;   polygons, vert_values, vert_colors
;   Settings:
;     single color (which) or palette
;     "box" of starting points: size and position (mouse-controlled)
;     box struct: boundary conditions (xyz): open, closed, periodic, periodic with remap
;
; MODIFICATION HISTORY:
;   2005-09-14 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  strindex=string(index+1, format='(I0)')

  wBaseButtons=widget_base(wBaseStream, $
                                                  UVALUE=index, $
                           /COLUMN, /FRAME, $
                           EVENT_FUNC='v3d_stream_event', $
                           XPAD=2, YPAD=2)
    ;
    wTopRow=widget_base(     wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
;      wEntry=widget_label(     wTopRow,           VALUE ='Stream ' + strindex)
      wMenu=widget_button(     wTopRow,           VALUE ='Stream ' + strindex, $
                                                  UVALUE='BUTTON_STREAM_MENU', $
                               /MENU, $
                               TOOLTIP='Show streamlines menu')
        wEntry=widget_button(  wMenu,             VALUE='Settings', $
                                                  UVALUE='BUTTON_STREAM_SETTINGS')
        wEntry=widget_button(  wMenu,             VALUE='Control', $
                                                  UVALUE='BUTTON_STREAM_CONTROL')
        wEntry=widget_button(  wMenu,             VALUE='Help', $
                                                  UVALUE='BUTTON_STREAM_HELP')
      wEntry=widget_label(     wTopRow,           VALUE =' ')
      ;
      wEntry=widget_button(    wTopRow,           VALUE ='Hide', $
                                                  UVALUE='BUTTON_STREAM_SHOWHIDEOBJ', $
                               TOOLTIP='Show/hide streamlines')
      (*sState)[index].wStreamShowHide=wEntry
      wEntry=widget_button(    wTopRow,           VALUE ='Close', $
                                                  UVALUE='BUTTON_STREAM_CLOSE', $
                               TOOLTIP='Close streamline panel without further action')
    ;
    wCenterFrame=widget_base(wBaseButtons, $
                             /COLUMN, $
                             XPAD=0, YPAD=2)
      ;
      wLine1=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine1,          VALUE ='NLin:')
        wEntry=widget_text(wLine1,           VALUE=strtrim(string((*sState)[index].stream.nlines),2), $
                                                  UVALUE='TEXT_STREAM_NLINES', $
                                 /EDITABLE, $
                                 XSIZE=5)
        (*sState)[index].wStreamNLines=wEntry
      ;
;      wLine2=widget_base(      wCenterFrame, $
;                               /ROW, $
;                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wLine1,          VALUE ='NPts:', $
                                 /ALIGN_BOTTOM)
        wEntry=widget_text(wLine1,          VALUE=strtrim(string((*sState)[index].stream.npoints),2), $
                                                  UVALUE='TEXT_STREAM_NPOINTS', $
                                 /EDITABLE, $
                                 XSIZE=4)
        (*sState)[index].wStreamNPoints=wEntry
      ;
      wLine3=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        wEntry=widget_label(     wLine3,          VALUE ='Step:')
        wEntry=widget_text(wLine3,             VALUE=strtrim(string((*sState)[index].stream.step),2), $
                                                  UVALUE='TEXT_STREAM_STEP', $
                                 /EDITABLE, $
                                 XSIZE=12)
        (*sState)[index].wStreamStep=wEntry
    ;
    wBottomRow=widget_base(  wBaseButtons, $
                             /ROW, $
                             /ALIGN_LEFT, $
                             XPAD=0, YPAD=0)
;      wEntry=widget_label(     wBottomRow,        VALUE ='    ')
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
                                                  UVALUE='BUTTON_STREAM_APPLY', $
                               TOOLTIP='Apply new streamline settings')

  return, wBaseButtons

end ; v3d_stream_createwidget


pro v3d_stream_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_stream_destroy ('v3d_stream_destroy')
;
; PURPOSE:
;   Destroy stream structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_stream_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Free existing heap variables and clean up using a null_pointer.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;
; HISTORY:
;   2005-09-14 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ptr_free, (*sState)[index].stream.vertices
  (*sState)[index].stream.vertices=ptr_new()

  ptr_free, (*sState)[index].stream.vert_values 
  (*sState)[index].stream.vert_values=ptr_new()

  ptr_free, (*sState)[index].stream.vert_colors 
  (*sState)[index].stream.vert_colors=ptr_new()

end ; v3d_stream_destroy


function v3d_stream_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_stream_init ('v3d_stream_initialize')
;
; PURPOSE:
;   Initialize stream structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   stream=v3d_stream_init()
;
; MODIFICATION HISTORY:
;   2005-09-14 Written by Bernd Freytag, L.A.
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Create an anonymous structure containing the basic data for the streamlines
  stream={$
    vertices:              PTR_NEW(), $            ; List of vertex coordinates
    vert_values:           PTR_NEW(), $            ; List of vertex values
    vert_colors:           PTR_NEW(), $            ; List of vertex colors
    is_solidcolor:         1, $
    solid_color:           [255B,255B,255B], $
    nlines:                0L, $                   ; Number of streamlines
    npoints:               0L, $                   ; Number of points on streamline
    step:                  0.0 $                   ; Step size    
         }

  return, stream

end ; v3d_stream_init
