;-------------------------------------------------------------------------------
; Vis3D module: Volume rendering
;-------------------------------------------------------------------------------
; Contains:
;   v3d_vol_displayhelp:  Display help text for vol widget
;   v3d_vol_createobj:    Create vol object
;   v3d_vol_event:        Handle all vol events
;   v3d_vol_updatewidget: Update the vol control widget
;   v3d_vol_createwidget: Create the vol widget from data in the state variable
;   v3d_vol_destroy:      Destroy vol structure and free memory used in it
;   v3d_vol_init:         Initialize vol structure
;-------------------------------------------------------------------------------


pro v3d_vol_displayhelp, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_vol_displayhelp ('v3d_vol_display_help')
;
; PURPOSE:
;   Display help text for vol widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_vol_displayhelp, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2006-06-07 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Volume rendering', $
'================', $
'', $
'This panel controls the proper (slow) volume rendering of objects.', $
'', $
'A click on the "Vol X" field opens a menu with a help entry,', $
'which gives this help text.', $
'', $
'The "Hide" or "Show" button on top of the panel toggles the visibility of', $
'the volume rendering.', $
'', $
'The "Close" button just closes the panel (not the entire window) without', $
'starting a volume rendering process.', $
'', $
'The "HiZxyz" and "LiZxyz" fields control the rendering accuracy (and the time', $
'needed for it) for the high-resolution static display and the low-resolution', $
'moving display mode.', $
'The first entry ("i") in both lines stands for interpolation:', $
'  0: nearest neighbor, 1: trilinear (slow).', $
'The second entry ("Z") contols the update of the zbuffer, where:', $
'  0: no update (fast),', $
'  1: update (slow, necessary when other objects, e.g. the box, should be', $
'     displayed properly.)', $
'The next three entries ("xyz") stand for the step in x, y, and z direction,', $
'respectively:', $
'If all steps are 1, all data points are considered during the volume rendering.', $
'That is accurate but slow.', $
'A larger step size means that points are skipped, which saves considerable time.', $
'The values for the high-res and low-res fields have to be adjusted to fit', $
'processor speed, data size, and quality requirements', $
'(see the IDL documentation about IDLgrVolume).', $
'', $
'The "Scale" factor is applied to the data set to get opacities.', $
'', $
'The "Apply" botton starts the volume rendering with the new values.']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_vol_displayhelp


pro v3d_vol_createobj, sState, index, LOW=LOW
;-------------------------------------------------------------------------------
; NAME:
;   v3d_vol_createobj ('v3d_vol_create_object')
;
; PURPOSE:
;   Create vol object.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   v3d_vol_createobj, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; MODIFICATION HISTORY:
;   2006-06-07 Written by Bernd Freytag, L.A.
;   2006-06-27 (B.F.) Bugfixes
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  if (not obj_valid((*sState)[index].oVol)) then begin
    ; Create new object
    ;
    (*sState)[index].oVol=obj_new('IDLgrVolume', $
                                  HINTS=2, $
                                  LIGHTING_MODEL=1, $
                                  VOLUME_SELECT=2, $
                                  HIDE=v3dpar_Vol_hide)
  endif

  if (keyword_set(LOW)) then begin
    (*sState)[index].oVol->SetProperty, RENDER_STEP=[(*sState)[index].Vol.LxStep, $
                                                     (*sState)[index].Vol.LyStep, $
                                                     (*sState)[index].Vol.LzStep], $
                                        INTERPOLATE= (*sState)[index].Vol.LInter, $
                                        ZBUFFER    = (*sState)[index].Vol.LzBuffer, $
                                        OPACITY_TABLE0=byte((lindgen(256)*(*sState)[index].Vol.LzStep) < 255L)
  endif else begin
    (*sState)[index].oVol->SetProperty, RENDER_STEP=[(*sState)[index].Vol.HxStep, $
                                                     (*sState)[index].Vol.HyStep, $
                                                     (*sState)[index].Vol.HzStep], $
                                        INTERPOLATE= (*sState)[index].Vol.HInter, $
                                        ZBUFFER    = (*sState)[index].Vol.HzBuffer, $
                                        OPACITY_TABLE0=byte((lindgen(256)*(*sState)[index].Vol.HzStep) < 255L)
  endelse

  if ((*sState)[index].isCurrVolColor ne 1) then begin
    ;
    ; Save system time
    time0=systime(1)
    if ((*sState)[index].color.is_solidcolor ne 1) then begin
      ;
      ;
      top   =(*sState)[index].color.palette_top
      bottom=(*sState)[index].color.palette_bottom
      ;
      ; The range of physical values which the palette will be mapped to
      delta=top-bottom
      ;
      if (delta eq 0.0) then begin
        ; The color range is zero.
        ;
        n_pal=256
        ;
        RGB_Table0=make_array(n_pal, 3, /byte)
        ;
        ; Use first color in palette
        RGB_Table0[*,0]=(*(*sState)[index].color.palette)[0,0]
        RGB_Table0[*,1]=(*(*sState)[index].color.palette)[1,0]
        RGB_Table0[*,2]=(*(*sState)[index].color.palette)[2,0]
        ;
        Data0=make_array((*sState)[index].box.nx, $
                         (*sState)[index].box.ny, $
                         (*sState)[index].box.nz, /BYTE)
        ;
      endif else begin
        ; There is a non-zero color range
        ;
        ; How many entries are in current palette?
        n_pal=(size(*(*sState)[index].color.palette))[2]
        ;
        RGB_Table0=transpose(*(*sState)[index].color.palette)
        ;
        Data0=byte(0.0 > $
                   ((float(n_pal-1L)/delta) * (*(*sState)[index].box.colordataset - bottom)) < $
                   float(n_pal-1L))
        ;
      endelse
    endif else begin
      ; Put one color in structure
      ;
      n_pal=256
      ;
      RGB_Table0=rebin(reform((*sState)[index].color.solid_color, 1,3), n_pal,3)
      ;
      Data0=make_array((*sState)[index].box.nx, $
                       (*sState)[index].box.ny, $
                       (*sState)[index].box.nz, /BYTE)
      ;
    endelse
    ;
    Data1=byte(0.0 > $
               ((*sState)[index].Vol.Scale*float(n_pal)) * (*(*sState)[index].box.isodataset) < $
               float(n_pal-1L))
    ;
    ;
;    RGB_Table1    =replicate(255B, n_pal, 3)
;    Opacity_Table0=replicate(255B, n_pal)
;    (*sState)[index].oVol->SetProperty, OPACITY_TABLE0=Opacity_Table0, $
;                                        RGB_TABLE0    =RGB_Table0, $
;                                        RGB_TABLE1    =RGB_Table1, $
;                                        DATA0=Data0, $
;                                        DATA1=Data1
    (*sState)[index].oVol->SetProperty, RGB_TABLE0    =RGB_Table0, $
                                        DATA0=Data0, $
                                        DATA1=Data0, $
                                        DATA2=Data0, $
                                        DATA3=Data1
    ;
;    (*sState)[index].oVol->SetProperty, HINT=0
    ;
;    (*sState)[index].oVol->ComputeBounds, OPACITY=100B, $
;                                          VOLUMES=[0,1]
;    (*sState)[index].oVol->GetProperty, BOUNDS=bounds
;    print, '% v3d_vol_createobj: ', bounds, format='(A,6I4)'
    ;
    (*sState)[index].isCurrVolColor=1
    ;
    message, 'Routine needed ' + strtrim(string(systime(1)-time0)) + ' sec', /INFORMATIONAL
    ;
  endif

end ; v3d_vol_createobj


function v3d_vol_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_vol_event ('v3d_vol_event')
;
; PURPOSE:
;   Handle all vol events.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2006-06-07 (B.F.) Taken from v3d_slice_event
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ;message, '', /INFORMATIONAL
  ;help, /str, sEvent
  ;help, sState, index, uval

  if (uval eq 'BUTTON_VOL_APPLY') then begin ; ---------------------------------
    ; Read fields, apply new values, render the volume, and draw it
    ;
    flag_HHasChanged=0
    flag_LHasChanged=0
    flag_SHasChanged=0
    ;
    widget_control, (*sState)[index].wVolHInter, GET_VALUE=strfield
    valfield=0L > long(strfield) < 1L
    flag_HHasChanged=flag_HHasChanged or (valfield ne (*sState)[index].Vol.HInter)
    (*sState)[index].Vol.HInter=valfield
    ;
    widget_control, (*sState)[index].wVolHzBuffer, GET_VALUE=strfield
    valfield=0L > long(strfield) < 1L
    flag_HHasChanged=flag_HHasChanged or (valfield ne (*sState)[index].Vol.HzBuffer)
    (*sState)[index].Vol.HzBuffer=valfield
    ;
    widget_control, (*sState)[index].wVolHxStep, GET_VALUE=strfield
    valfield=1L > long(strfield) < 99L
    flag_HHasChanged=flag_HHasChanged or (valfield ne (*sState)[index].Vol.HxStep)
    (*sState)[index].Vol.HxStep=valfield
    ;
    widget_control, (*sState)[index].wVolHyStep, GET_VALUE=strfield
    valfield=1L > long(strfield) < 99L
    flag_HHasChanged=flag_HHasChanged or (valfield ne (*sState)[index].Vol.HyStep)
    (*sState)[index].Vol.HyStep=valfield
    ;
    widget_control, (*sState)[index].wVolHzStep, GET_VALUE=strfield
    valfield=1L > long(strfield) < 99L
    flag_HHasChanged=flag_HHasChanged or (valfield ne (*sState)[index].Vol.HzStep)
    (*sState)[index].Vol.HzStep=valfield
    ;
    widget_control, (*sState)[index].wVolLInter, GET_VALUE=strfield
    valfield=0L > long(strfield) < 1L
    flag_LHasChanged=flag_LHasChanged or (valfield ne (*sState)[index].Vol.LInter)
    (*sState)[index].Vol.LInter=valfield
    ;
    widget_control, (*sState)[index].wVolLzBuffer, GET_VALUE=strfield
    valfield=0L > long(strfield) < 1L
    flag_LHasChanged=flag_LHasChanged or (valfield ne (*sState)[index].Vol.LzBuffer)
    (*sState)[index].Vol.LzBuffer=valfield
    ;
    widget_control, (*sState)[index].wVolLxStep, GET_VALUE=strfield
    valfield=1L > long(strfield) < 99L
    flag_LHasChanged=flag_LHasChanged or (valfield ne (*sState)[index].Vol.LxStep)
    (*sState)[index].Vol.LxStep=valfield
    ;
    widget_control, (*sState)[index].wVolLyStep, GET_VALUE=strfield
    valfield=1L > long(strfield) < 99L
    flag_LHasChanged=flag_LHasChanged or (valfield ne (*sState)[index].Vol.LyStep)
    (*sState)[index].Vol.LyStep=valfield
    ;
    widget_control, (*sState)[index].wVolLzStep, GET_VALUE=strfield
    valfield=1L > long(strfield) < 99L
    flag_LHasChanged=flag_LHasChanged or (valfield ne (*sState)[index].Vol.LzStep)
    (*sState)[index].Vol.LzStep=valfield
    ;
    widget_control, (*sState)[index].wVolScale, GET_VALUE=strfield
    valfield=double(strfield)
    flag_HHasChanged=flag_HHasChanged or (valfield ne (*sState)[index].Vol.Scale)
    flag_LHasChanged=flag_LHasChanged or (valfield ne (*sState)[index].Vol.Scale)
    flag_SHasChanged=flag_SHasChanged or (valfield ne (*sState)[index].Vol.Scale)
    (*sState)[index].Vol.Scale=valfield
    ;
    if (flag_SHasChanged) then begin
      message, 'Recreate Vol datasets', /INFORMATIONAL
      (*sState)[index].isCurrVolColor=0
      v3d_vol_createobj, sState, index
    endif
    ;
    if (flag_HHasChanged) then begin
      message, 'Render object', /INFORMATIONAL
      v3d_vol_createobj, sState, index
      (*sState)[index].oWindow->Draw, (*sState)[index].oView
    endif
    ;
  endif else $ ; 'BUTTON_VOL_APPLY'

  if (uval eq 'BUTTON_VOL_SHOWHIDEOBJ') then begin ; ---------------------------
    ; Toggle the hide flag of the object
    ;
    (*sState)[index].oVol->GetProperty, HIDE=hide
    hide=1-keyword_set(hide)
    (*sState)[index].oVol->SetProperty, HIDE=hide
    ;
    if (KEYWORD_SET(hide)) then begin
      widget_control, (*sState)[index].wVolShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wVolShowHide, $
                      SET_VALUE='Hide'
      v3d_color_setpalette, sState, index
    endelse
    ;
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
    ;
  endif else $ ; 'BUTTON_VOL_SHOWHIDEOBJ'

  if (uval eq 'BUTTON_VOL_HELP') then begin ; ----------------------------------
    ; Show help
    ;
    v3d_vol_displayhelp, group=(*sState)[index].wDraw
    ;
  endif else $ ; 'BUTTON_VOL_HELP'

  if (uval eq 'BUTTON_VOL_CLOSE') then begin ; ---------------------------------
    ; Close widget without any further action
    ;
    (*sState)[index].hideControlVol=1
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif ; 'BUTTON_VOL_CLOSE'

  return, 1

end ; v3d_vol_event


pro v3d_vol_updatewidget, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_vol_updatewidget ('v3d_vol_update_widget')
;
; PURPOSE:
;   Update the vol control widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_vol_updatewidget, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2006-06-07 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if ((*sState)[index].hideControlVol eq 0) then begin
    widget_control, (*sState)[index].wVolHInter  , $
                    SET_VALUE=string((*sState)[index].Vol.HInter, FORMAT='(I1)')
    widget_control, (*sState)[index].wVolHzBuffer  , $
                    SET_VALUE=string((*sState)[index].Vol.HzBuffer, FORMAT='(I1)')
    widget_control, (*sState)[index].wVolHxStep  , $
                    SET_VALUE=string((*sState)[index].Vol.HxStep, FORMAT='(I2)')
    widget_control, (*sState)[index].wVolHyStep  , $
                    SET_VALUE=string((*sState)[index].Vol.HyStep, FORMAT='(I2)')
    widget_control, (*sState)[index].wVolHzStep  , $
                    SET_VALUE=string((*sState)[index].Vol.HzStep, FORMAT='(I2)')
    ;
    widget_control, (*sState)[index].wVolLInter  , $
                    SET_VALUE=string((*sState)[index].Vol.LInter, FORMAT='(I1)')
    widget_control, (*sState)[index].wVolLzBuffer  , $
                    SET_VALUE=string((*sState)[index].Vol.LzBuffer, FORMAT='(I1)')
    widget_control, (*sState)[index].wVolLxStep  , $
                    SET_VALUE=string((*sState)[index].Vol.LxStep, FORMAT='(I2)')
    widget_control, (*sState)[index].wVolLyStep  , $
                    SET_VALUE=string((*sState)[index].Vol.LyStep, FORMAT='(I2)')
    widget_control, (*sState)[index].wVolLzStep  , $
                    SET_VALUE=string((*sState)[index].Vol.LzStep, FORMAT='(I2)')
    ;
    widget_control, (*sState)[index].wVolScale  , $
                    SET_VALUE=string((*sState)[index].Vol.Scale, FORMAT='(G0)')
    ;
    (*sState)[index].oVol->GetProperty, HIDE=hide
    if (hide) then begin
      widget_control, (*sState)[index].wVolShowHide, $
                      SET_VALUE='Show'
    endif else begin
      widget_control, (*sState)[index].wVolShowHide, $
                      SET_VALUE='Hide'
    endelse
  endif

end ; v3d_vol_updatewidget


function v3d_vol_createwidget, wBaseVol, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_vol_createwidget ('v3d_vol_create_widget')
;
; PURPOSE:
;   Create the vol widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_vol_createwidget(wBaseVol, sState, index)
;
; INPUT:
;   wBaseVol: ('widget_Base_Vol') parent widget
;   index:      ('index') current view index
;
; INPUT/OUTPUT:
;   sState:     ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Widget
;
; KEYWORDS:
;   None
;
; MODIFICATION HISTORY:
;   2006-06-07 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  strindex=string(index+1, format='(I0)')
  
  wBaseButtons=widget_base(wBaseVol, $
                           /COLUMN, /FRAME, $
                           UVALUE=index, $
                           EVENT_FUNC='v3d_vol_event', $
                           XPAD=2, YPAD=2)
    ;
    wTopRow=widget_base(     wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      wMenu=widget_button(     wTopRow,           VALUE ='Vol ' + strindex, $
                                                  UVALUE='BUTTON_VOL_MENU', $
                               /MENU, $
                               TOOLTIP='Show volume-render menu')
;        wEntry=widget_button(  wMenu,             VALUE='Settings', $
;                                                  UVALUE='BUTTON_VOL_SETTINGS')
;        wEntry=widget_button(  wMenu,             VALUE='Control', $
;                                                  UVALUE='BUTTON_VOL_CONTROL')
        wEntry=widget_button(  wMenu,             VALUE='Help', $
                                                  UVALUE='BUTTON_VOL_HELP')
        ;
      wEntry=widget_label(     wTopRow,           VALUE ='        ')
      wEntry=widget_button(    wTopRow,           VALUE ='Hide', $
                                                  UVALUE='BUTTON_VOL_SHOWHIDEOBJ', $
                               TOOLTIP='Show/hide volume object')
      (*sState)[index].wVolShowHide=wEntry
      wEntry=widget_button(    wTopRow,           VALUE ='Close', $
                                                  UVALUE='BUTTON_VOL_CLOSE', $
                               TOOLTIP='Close this volume control panel without further action')
    ;
    wCenterFrame=widget_base(wBaseButtons, $
                             /COLUMN, $
                             XPAD=0, YPAD=2)
      ;
      wLine1=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wLine1,          VALUE ='HiZxyz:')
        wEntry=widget_text(      wLine1,          VALUE=string((*sState)[index].Vol.HInter, $
                                                               FORMAT='(I1)'), $
                                                  UVALUE='TEXT_VOL_HINTER', $
                                 /EDITABLE, $
                                 XSIZE=1)
        (*sState)[index].wVolHInter=wEntry
        wEntry=widget_text(      wLine1,          VALUE=string((*sState)[index].Vol.HzBuffer, $
                                                               FORMAT='(I1)'), $
                                                  UVALUE='TEXT_VOL_HZBUFFER', $
                                 /EDITABLE, $
                                 XSIZE=1)
        (*sState)[index].wVolHzBuffer=wEntry
        wEntry=widget_text(      wLine1,          VALUE=string((*sState)[index].Vol.HxStep, $
                                                               FORMAT='(I2)'), $
                                                  UVALUE='TEXT_VOL_HXSTEP', $
                                 /EDITABLE, $
                                 XSIZE=2)
        (*sState)[index].wVolHxStep=wEntry
        wEntry=widget_text(      wLine1,          VALUE=string((*sState)[index].Vol.HyStep, $
                                                               FORMAT='(I2)'), $
                                                  UVALUE='TEXT_VOL_HYSTEP', $
                                 /EDITABLE, $
                                 XSIZE=2)
        (*sState)[index].wVolHyStep=wEntry
        wEntry=widget_text(      wLine1,          VALUE=string((*sState)[index].Vol.HzStep, $
                                                               FORMAT='(I2)'), $
                                                  UVALUE='TEXT_VOL_HZSTEP', $
                                 /EDITABLE, $
                                 XSIZE=2)
        (*sState)[index].wVolHzStep=wEntry
      ;
      wLine2=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wLine2,          VALUE ='LiZxyz:')
        wEntry=widget_text(      wLine2,          VALUE=string((*sState)[index].Vol.LInter, $
                                                               FORMAT='(I1)'), $
                                                  UVALUE='TEXT_VOL_LINTER', $
                                 /EDITABLE, $
                                 XSIZE=1)
        (*sState)[index].wVolLInter=wEntry
        wEntry=widget_text(      wLine2,          VALUE=string((*sState)[index].Vol.LzBuffer, $
                                                               FORMAT='(I1)'), $
                                                  UVALUE='TEXT_VOL_LZBUFFER', $
                                 /EDITABLE, $
                                 XSIZE=1)
        (*sState)[index].wVolLzBuffer=wEntry
        wEntry=widget_text(      wLine2,          VALUE=string((*sState)[index].Vol.LxStep, $
                                                               FORMAT='(I2)'), $
                                                  UVALUE='TEXT_VOL_LXSTEP', $
                                 /EDITABLE, $
                                 XSIZE=2)
        (*sState)[index].wVolLxStep=wEntry
        wEntry=widget_text(      wLine2,          VALUE=string((*sState)[index].Vol.LyStep, $
                                                               FORMAT='(I2)'), $
                                                  UVALUE='TEXT_VOL_LYSTEP', $
                                 /EDITABLE, $
                                 XSIZE=2)
        (*sState)[index].wVolLyStep=wEntry
        wEntry=widget_text(      wLine2,          VALUE=string((*sState)[index].Vol.LzStep, $
                                                               FORMAT='(I2)'), $
                                                  UVALUE='TEXT_VOL_LZSTEP', $
                                 /EDITABLE, $
                                 XSIZE=2)
        (*sState)[index].wVolLzStep=wEntry
        ;
      wLine3=widget_base(      wCenterFrame, $
                               /ROW, $
                               XPAD=0, YPAD=0)
        ;
        wEntry=widget_label(     wLine3,          VALUE ='Scale:')
        wEntry=widget_text(      wLine3,          VALUE=string((*sState)[index].Vol.Scale, $
                                                               FORMAT='(G0)'), $
                                                  UVALUE='TEXT_VOL_SCALE', $
                                 /EDITABLE, $
                                 XSIZE=12)
        (*sState)[index].wVolScale=wEntry
        ;
    wBottomRow=widget_base(  wBaseButtons, $
                             /ROW, $
                             XPAD=0, YPAD=0)
      ;
      wEntry=widget_button(    wBottomRow,        VALUE ='Apply', $
                               /ALIGN_CENTER,     UVALUE='BUTTON_VOL_APPLY', $
                               TOOLTIP='Apply new volume-render settings')
  ;

  return, wBaseButtons

end ; v3d_vol_createwidget


pro v3d_vol_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_vol_destroy ('v3d_vol_destroy')
;
; PURPOSE:
;   Destroy vol structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_vol_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Just don't do anything.
;
; RESTRICTIONS:
;   Must be kept updated when new things are added to the structure
;
; HISTORY:
;   2006-06-07 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  ; Actually, there is nothing to do yet.

end ; v3d_vol_destroy


function v3d_vol_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_vol_init ('v3d_vol_initialize')
;
; PURPOSE:
;   Initialize vol structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   vol=v3d_vol_init()
;
; MODIFICATION HISTORY:
;   2006-06-07 Written by Bernd Freytag, L.A.
;   2011-04-01 (B.F. Lyon) LxStep=LyStep=10->6
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Create an anonymous structure containing the basic data for the volume render
  vol={$
    HInter:                0L, $
    HzBuffer:              0L, $
    HxStep:                2L, $
    HyStep:                2L, $
    HzStep:                1L, $
    LInter:                0L, $
    LzBuffer:              0L, $
    LxStep:                6L, $
    LyStep:                6L, $
    LzStep:                1L, $
    Scale:                 1.0D+00 $
       }

  return, vol

end ; v3d_vol_init


