function v3dgr_surface, d1, c1, DIR=dir0, INDEX=index0, $
  image=image, oIm=oIm, $
  hidden_lines=hidden_lines, hide=hide, $
  linestyle=linestyle, style=style, thick=thick

;-------------------------------------------------------------------------------
;+
; NAME:
;   v3dgr_surface ('v3d_graphics_surface')
;
; PURPOSE:
;   Create a surface object from 3D array optionally with texture map
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3dgr_surface(d1, c1, DIR=dir0, INDEX=index0)
;
; INPUT:
;   d1:         ('data_1')  3D array used to define the surface
;   c1:         ('color_1') 3D array for color (texture map)
;
; OUTPUT:
;   result:     handle to the surface object
;
; KEYWORDS:
;   dir:        ('direction') direction index (0, 1, 2)
;   index:      ('index') index of slice in specified direction
;
; MODIFICATION HISTORY:
;   2005-08-10 Written by B.F.
;-
;-------------------------------------------------------------------------------

; --- Check keywords ---

if (not keyword_set(dir0)) then begin
  dir=0
endif else begin
  dir=0 > fix(dir0) < 2
endelse

if (not keyword_set(index0)) then begin
  index=0L
endif else begin
  index=0 > long(index0)
endelse

; --- Check input arrays ---

exist_d1=0
exist_c1=0

if ((size(d1, /n_dim) ne 3) and (size(c1, /n_dim) ne 3)) then begin
  message, 'ERROR: No 3D input arrays supplied.'
endif

if (size(d1, /n_dim) eq 3) then begin
  exist_d1=1
  d1_size=size(d1, /dim)
  n1=d1_size[0]-1L
  n2=d1_size[1]-1L
  n3=d1_size[2]-1L
endif

if (size(c1, /n_dim) eq 3) then begin
  exist_c1=1
  c1_size=size(c1, /dim)
  if (exist_d1) then begin
    if (not array_equal(d1_size, c1_size)) then begin
      message, 'ERROR: Input arrays have incompatible dimensions.'
    endif
  endif
  n1=c1_size[0]-1L
  n2=c1_size[1]-1L
  n3=c1_size[2]-1L
endif

help, exist_d1, exist_c1, n1,n2,n3

; --- Produce 2D slice(s) ---

if (exist_d1) then begin
  if (dir eq 0) then begin
    d1_slice=reform(d1(index < n1, *, *))
  endif else if (dir eq 1) then begin
    d1_slice=reform(d1(*, index < n2, *))
  endif else begin
    d1_slice=reform(d1(*, *, index < n3))
  endelse
  ;
  d1s_min=min(d1_slice, max=d1s_max)
  if (d1s_min eq d1s_max) then begin
    d1_slice=d1_slice*0.0
  endif else begin
    d1_slice=(d1_slice-d1s_min)/(d1s_max-d1s_min)*float(n1)
  endelse
endif

if (exist_c1) then begin
  if (dir eq 0) then begin
    c1_slice=reform(c1(index < n1, *, *))
  endif else if (dir eq 1) then begin
    c1_slice=reform(c1(*, index < n2, *))
  endif else begin
    c1_slice=reform(c1(*, *, index < n3))
  endelse
  ;
  if (not exist_d1) then begin
    d1_slice=c1_slice*0.0 + float(index)
   endif
endif

; --- Generate surface object ---

oSurf=obj_new('IDLgrSurface', d1_slice, $
              hidden_lines=hidden_lines, $
              hide=hide, $
              linestyle=linestyle, $
              style=style, $
              thick=thick)

; --- Make texture map from color array slice ---

if (exist_c1) then begin
  c1_min=min(c1, max=c1_max)
  ;
  tvlct, ctred, ctgreen, ctblue, /GET
  color_fi=0
  color_ra=n_elements(ctred)-1
  ;
  image=(c1_slice-c1_min)/(c1_max-c1_min)*float(color_ra)
  image=byte(( 0 > image < color_ra) + color_fi)
  image=[[[ctred(image)]], [[ctgreen(image)]], [[ctblue( image)]]]
  ;
  help, image, min(image), max(image)
  ;
  oIm=obj_new('IDLgrImage', image, interleave=2, /interpolate)
  oSurf->SetProperty, style=2
  oSurf->SetProperty, texture_map=oIm, color=[255b,255b,255b]
endif

return, oSurf

end ; v3dgr_surface
