!------**************--------------------------------------------------------------------------------
module bicubic_module
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_module ('bicubic')
!
! PURPOSE:
!   Routines for bicubic interpolation
!
! ROUTINES: (contained)
!   bicubic_coeff_1:       Compute bicubic interpolation coefficients from tabulated values in a cell
!   bicubic_coeff:         Compute bicubic interpolation coefficients from tabulated values
!   bicubic_interpol_1:    Do bicubic interpolation in a single cell 
!   bicubic_interpol :     Do bicubic interpolation on a regular grid
!   bicubic_interpol_gas:  Do bicubic interpolation on a regular grid, special GAS routine version
!
! MODIFICATION HISTORY:
!   18. 3.96 Written by Bernd Freytag, Astrophysik Kiel
!   22. 1.97 (B.F.) !  module procedure ...
!   18.11.03 (B.F. Uppsala) single and double precision version
!----------------------------------------------------------------------------------------------------
interface bicubic_coeff_1
!  module procedure bicubic_coeff_1_srk, bicubic_coeff_1_drk
  module procedure bicubic_coeff_1_srk
end interface
!
interface bicubic_coeff
!  module procedure bicubic_coeff_srk, bicubic_coeff_drk
  module procedure bicubic_coeff_srk
end interface
!
contains

!----------*******************-----------------------------------------------------------------------
subroutine bicubic_coeff_1_srk(x1,x2,y,y1,y2,y12, C, reuse)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_coeff_1_srk ('bicubic_coefficients_1cell_single_real_kind')
!
! PURPOSE:
!   Derive coefficients for bicubic interpolation in a single cell.
!
! INPUT:
!   x1:  Independent vector of first coordinate
!   x2:  Independent vector of second coordinate
!   y:   y=y(x1,x2), function value at (x1,x2)
!   y1:  y1=dy/dx1(x1,x2), gradient at (x1,x2)
!   y2:  y2=dy/dx2(x1,x2), gradient at (x1,x2)
!   y12: y12=ddy/dx1dx2(x1,x2), cross derivative at (x1,x2)
!   reuse: Set to true, to reuse old G(x1) and G(x2) matrices in case of
!          constant grid spacing
!
! OUTPUT:
!   C:   Coefficients for bicubic interpolation
!
! PROCEDURE:
!   Cell geometry:
!     x2(2)  +-------+     / y(x1(2),x2(2))
!            |       | \   |
!            |       |  \ <  y1(x1(2),x2(2))
!            |       |     | y2(x1(2),x2(2))
!     x2(1)  +-------+     \ y12(x1(2),x2(2))
!          x1(1) x1(2)
!
!   Algorithm from: Helmut Spaeth, 1986, 'Spline-Algorithmen zur Konstruktion
!   glatter Kurven und Flaechen', Oldenbourg, 4. edition, p. 111-112
!   (See also: Press et al., 1992, 'Numerical recipies in FORTRAN', Cambridge,
!    2. edition, p. 116-120)
!
!   Parameter correspondence:
!     Here   Spaeth
!      x1    x
!      x2    y
!      y     u
!      y1    p
!      y2    q
!      y12   r
!      Gx1   G(x_i)
!      Gx2   G(y_j)
!      Sy    S
!      C     A
!
! MODIFICATION HISTORY:
!   17-Mar-1996 18:02:11 Written by Bernd Freytag, Astrophysik Kiel
!   18-Mar-1996 14:50:31 (B.F.) Completely different method
!   20-Mar-1996 18:37:02 (B.F.) No write statements, keyword reuse added
!------------------------------------------------------------------------------
use matinv_gj_module                      ! --- Module for matrix inversion ---
!
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0)
! --- Input/output parameter
real(kind=rk), dimension(1:2),      intent(in) :: x1, x2
real(kind=rk), dimension(1:2,1:2),  intent(in) :: y, y1, y2, y12
real(kind=rk), dimension(1:4,1:4), intent(out) :: C
logical, optional                              :: reuse
!
!integer i1,i2
real(kind=rk), dimension(1:4,1:4), save        :: Gx1, Gx2
real(kind=rk), dimension(1:4,1:4)              :: Sy
logical                                        :: reuse_local
!------------------------------------------------------------------------------
!
! --- Set up matrices ---
if (present(reuse)) then
  reuse_local=reuse
else
  reuse_local=.false.
endif
!
if (.not. reuse_local) then
  ! --- Set up matrices describing cell geometry ---
  Gx1=reshape((/ 1._rk   , 0._rk         , 1._rk   , 0._rk         , &
                 x1(1)   , 1._rk         , x1(2)   , 1._rk         , &
                 x1(1)**2, 2._rk*x1(1)   , x1(2)**2, 2._rk*x1(2)   , &
                 x1(1)**3, 3._rk*x1(1)**2, x1(2)**3, 3._rk*x1(2)**2 /), (/4,4/))
  Gx2=reshape((/ 1._rk   , 0._rk         , 1._rk   , 0._rk         , &
                 x2(1)   , 1._rk         , x2(2)   , 1._rk         , &
                 x2(1)**2, 2._rk*x2(1)   , x2(2)**2, 2._rk*x2(2)   , &
                 x2(1)**3, 3._rk*x2(1)**2, x2(2)**3, 3._rk*x2(2)**2 /), (/4,4/))
  !
  ! --- Invert (and transpose) matrices ---
  call matinv_gj_singlereal(Gx1)
  Gx2=transpose(Gx2)
  call matinv_gj_singlereal(Gx2)
endif
!
Sy =reshape((/  y(1,1),  y1(1,1),  y(2,1),  y1(2,1), &
               y2(1,1), y12(1,1), y2(2,1), y12(2,1), &
                y(1,2),  y1(1,2),  y(2,2),  y1(2,2), &
               y2(1,2), y12(1,2), y2(2,2), y12(2,2) /), (/4,4/))
!
!write(6,*)
!write(6,*) 'Gx1:'
!do i1=1,4
!  write(6,'(4(F11.7))') (Gx1(i1,i2),i2=1,4)
!end do ! i1
!write(6,*)
!write(6,*) 'Gx2:'
!do i1=1,4
!  write(6,'(4(F11.7))') (Gx2(i1,i2),i2=1,4)
!end do ! i1
!write(6,*)
!write(6,*) 'Sy:'
!do i1=1,4
!  write(6,'(4(F11.7))') (Sy(i1,i2),i2=1,4)
!end do ! i1
!
! --- Matrix multiplication gives coefficients ---
C=matmul(Gx1,matmul(Sy,Gx2))
!
!write(6,*)
!write(6,*) 'C:'
!do i1=1,4
!  write(6,'(4(F11.7))') (C(i1,i2),i2=1,4)
!end do ! i1
!
end subroutine bicubic_coeff_1_srk


!----------*******************-----------------------------------------------------------------------
subroutine bicubic_coeff_1_drk(x1,x2,y,y1,y2,y12, C, reuse)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_coeff_1_drk ('bicubic_coefficients_1cell_double_real_kind')
!
! PURPOSE:
!   Derive coefficients for bicubic interpolation in a single cell.
!
! INPUT:
!   x1:  Independent vector of first coordinate
!   x2:  Independent vector of second coordinate
!   y:   y=y(x1,x2), function value at (x1,x2)
!   y1:  y1=dy/dx1(x1,x2), gradient at (x1,x2)
!   y2:  y2=dy/dx2(x1,x2), gradient at (x1,x2)
!   y12: y12=ddy/dx1dx2(x1,x2), cross derivative at (x1,x2)
!   reuse: Set to true, to reuse old G(x1) and G(x2) matrices in case of
!          constant grid spacing
!
! OUTPUT:
!   C:   Coefficients for bicubic interpolation
!
! PROCEDURE:
!   Cell geometry:
!     x2(2)  +-------+     / y(x1(2),x2(2))
!            |       | \   |
!            |       |  \ <  y1(x1(2),x2(2))
!            |       |     | y2(x1(2),x2(2))
!     x2(1)  +-------+     \ y12(x1(2),x2(2))
!          x1(1) x1(2)
!
!   Algorithm from: Helmut Spaeth, 1986, 'Spline-Algorithmen zur Konstruktion
!   glatter Kurven und Flaechen', Oldenbourg, 4. edition, p. 111-112
!   (See also: Press et al., 1992, 'Numerical recipies in FORTRAN', Cambridge,
!    2. edition, p. 116-120)
!
!   Parameter correspondence:
!     Here   Spaeth
!      x1    x
!      x2    y
!      y     u
!      y1    p
!      y2    q
!      y12   r
!      Gx1   G(x_i)
!      Gx2   G(y_j)
!      Sy    S
!      C     A
!
! MODIFICATION HISTORY:
!   17-Mar-1996 18:02:11 Written by Bernd Freytag, Astrophysik Kiel
!   18-Mar-1996 14:50:31 (B.F.) Completely different method
!   20-Mar-1996 18:37:02 (B.F.) No write statements, keyword reuse added
!------------------------------------------------------------------------------
use matinv_gj_module                      ! --- Module for matrix inversion ---
!
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0D+00)
! --- Input/output parameter
real(kind=rk), dimension(1:2),      intent(in) :: x1, x2
real(kind=rk), dimension(1:2,1:2),  intent(in) :: y, y1, y2, y12
real(kind=rk), dimension(1:4,1:4), intent(out) :: C
logical, optional                              :: reuse
!
!integer i1,i2
real(kind=rk), dimension(1:4,1:4), save        :: Gx1, Gx2
real(kind=rk), dimension(1:4,1:4)              :: Sy
logical                                        :: reuse_local
!------------------------------------------------------------------------------
!
! --- Set up matrices ---
if (present(reuse)) then
  reuse_local=reuse
else
  reuse_local=.false.
endif
!
if (.not. reuse_local) then
  ! --- Set up matrices describing cell geometry ---
  Gx1=reshape((/ 1._rk   , 0._rk         , 1._rk   , 0._rk         , &
                 x1(1)   , 1._rk         , x1(2)   , 1._rk         , &
                 x1(1)**2, 2._rk*x1(1)   , x1(2)**2, 2._rk*x1(2)   , &
                 x1(1)**3, 3._rk*x1(1)**2, x1(2)**3, 3._rk*x1(2)**2 /), (/4,4/))
  Gx2=reshape((/ 1._rk   , 0._rk         , 1._rk   , 0._rk         , &
                 x2(1)   , 1._rk         , x2(2)   , 1._rk         , &
                 x2(1)**2, 2._rk*x2(1)   , x2(2)**2, 2._rk*x2(2)   , &
                 x2(1)**3, 3._rk*x2(1)**2, x2(2)**3, 3._rk*x2(2)**2 /), (/4,4/))
  !
  ! --- Invert (and transpose) matrices ---
  call matinv_gj_doublereal(Gx1)
  Gx2=transpose(Gx2)
  call matinv_gj_doublereal(Gx2)
endif
!
Sy =reshape((/  y(1,1),  y1(1,1),  y(2,1),  y1(2,1), &
               y2(1,1), y12(1,1), y2(2,1), y12(2,1), &
                y(1,2),  y1(1,2),  y(2,2),  y1(2,2), &
               y2(1,2), y12(1,2), y2(2,2), y12(2,2) /), (/4,4/))
!
!write(6,*)
!write(6,*) 'Gx1:'
!do i1=1,4
!  write(6,'(4(F11.7))') (Gx1(i1,i2),i2=1,4)
!end do ! i1
!write(6,*)
!write(6,*) 'Gx2:'
!do i1=1,4
!  write(6,'(4(F11.7))') (Gx2(i1,i2),i2=1,4)
!end do ! i1
!write(6,*)
!write(6,*) 'Sy:'
!do i1=1,4
!  write(6,'(4(F11.7))') (Sy(i1,i2),i2=1,4)
!end do ! i1
!
! --- Matrix multiplication gives coefficients ---
C=matmul(Gx1,matmul(Sy,Gx2))
!
!write(6,*)
!write(6,*) 'C:'
!do i1=1,4
!  write(6,'(4(F11.7))') (C(i1,i2),i2=1,4)
!end do ! i1
!
end subroutine bicubic_coeff_1_drk


!----------*****************-------------------------------------------------------------------------
subroutine bicubic_coeff_srk(x1,x2,y,y1,y2,y12, C, constant_spacing)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_coeff_srk ('bicubic_coefficients_single_real_kind')
!
! PURPOSE:
!   Derive coefficients for bicubic interpolation.
!
! INPUT:
!   x1:  Independent vector of first coordinate
!   x2:  Independent vector of second coordinate
!   y:   y=y(x1,x2), function value at (x1,x2)
!   y1:  y1=dy/dx1(x1,x2), gradient at (x1,x2)
!   y2:  y2=dy/dx2(x1,x2), gradient at (x1,x2)
!   y12: y12=ddy/dx1dx2(x1,x2), cross derivative at (x1,x2)
!   constant_spacing: (optional) logical, set to true if grid has constant
!                     spacing
!
! OUTPUT:
!   C:   Coefficients for bicubic interpolation
!
! PROCEDURE:
!   Call bicubic_coeff_1 for each cell in array.
!
! MODIFICATION HISTORY:
!   19-Mar-1996 18:39:31 Written by Bernd Freytag, Astrophysik Kiel
!   20-Mar-1996 18:53:53 (B.F.) Parameter constant_spacing added
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0)
! --- Input/output parameter
real(kind=rk), dimension(1:),       intent(in) :: x1, x2
real(kind=rk), dimension(1: ,1: ),  intent(in) :: y, y1, y2, y12
real(kind=rk), dimension(1:4,1:4,1:size(x1,1)-1,1:size(x2,1)-1), &
                                   intent(out) :: C
logical, optional                              :: constant_spacing
!
integer                                        :: n1, n2, i1, i2
logical                                        :: reuse, reuse1
!------------------------------------------------------------------------------
!
! --- Array size ---
n1=size(x1,1)-1
n2=size(x2,1)-1
!
! --- Has grid constant spacing, can matrices be reused? ---
if (present(constant_spacing)) then
  reuse1=constant_spacing
else
  reuse1=.false.
endif
reuse=.false.
!
do i2=1,n2
  do i1=1,n1
    call bicubic_coeff_1_srk(x1(i1:i1+1)-x1(i1) ,  x2(i2:i2+1)-x2(i2) , &
                              y(i1:i1+1,i2:i2+1),  y1(i1:i1+1,i2:i2+1), &
                             y2(i1:i1+1,i2:i2+1), y12(i1:i1+1,i2:i2+1), &
                              C(:,:,i1,i2), &
                             reuse=reuse)
    reuse=reuse1
  end do ! i1
end do ! i2
!
end subroutine bicubic_coeff_srk


!----------*****************-------------------------------------------------------------------------
subroutine bicubic_coeff_drk(x1,x2,y,y1,y2,y12, C, constant_spacing)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_coeff_drk ('bicubic_coefficients_double_real_kind')
!
! PURPOSE:
!   Derive coefficients for bicubic interpolation.
!
! INPUT:
!   x1:  Independent vector of first coordinate
!   x2:  Independent vector of second coordinate
!   y:   y=y(x1,x2), function value at (x1,x2)
!   y1:  y1=dy/dx1(x1,x2), gradient at (x1,x2)
!   y2:  y2=dy/dx2(x1,x2), gradient at (x1,x2)
!   y12: y12=ddy/dx1dx2(x1,x2), cross derivative at (x1,x2)
!   constant_spacing: (optional) logical, set to true if grid has constant
!                     spacing
!
! OUTPUT:
!   C:   Coefficients for bicubic interpolation
!
! PROCEDURE:
!   Call bicubic_coeff_1 for each cell in array.
!
! MODIFICATION HISTORY:
!   19-Mar-1996 18:39:31 Written by Bernd Freytag, Astrophysik Kiel
!   20-Mar-1996 18:53:53 (B.F.) Parameter constant_spacing added
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0D+00)
! --- Input/output parameter
real(kind=rk), dimension(1:),       intent(in) :: x1, x2
real(kind=rk), dimension(1: ,1: ),  intent(in) :: y, y1, y2, y12
real(kind=rk), dimension(1:4,1:4,1:size(x1,1)-1,1:size(x2,1)-1), &
                                   intent(out) :: C
logical, optional                              :: constant_spacing
!
integer                                        :: n1, n2, i1, i2
logical                                        :: reuse, reuse1
!------------------------------------------------------------------------------
!
! --- Array size ---
n1=size(x1,1)-1
n2=size(x2,1)-1
!
! --- Has grid constant spacing, can matrices be reused? ---
if (present(constant_spacing)) then
  reuse1=constant_spacing
else
  reuse1=.false.
endif
reuse=.false.
!
do i2=1,n2
  do i1=1,n1
    call bicubic_coeff_1_drk(x1(i1:i1+1)-x1(i1) ,  x2(i2:i2+1)-x2(i2) , &
                              y(i1:i1+1,i2:i2+1),  y1(i1:i1+1,i2:i2+1), &
                             y2(i1:i1+1,i2:i2+1), y12(i1:i1+1,i2:i2+1), &
                              C(:,:,i1,i2), &
                             reuse=reuse)
    reuse=reuse1
  end do ! i1
end do ! i2
!
end subroutine bicubic_coeff_drk


!----------******************------------------------------------------------------------------------
subroutine bicubic_interpol_1(x1,x2,C, y,y1,y2)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_interpol_1 ('bicubic_interpolation_1cell')
!
! PURPOSE:
!   Do bicubic interpolation with coefficients derived e.g. by bicubic_coeff_1.
!
! INPUT:
!   x1:  Independent value of first coordinate
!   x2:  Independent value of second coordinate
!   C:   Coefficients for bicubic interpolation
!
! OUTPUT:
!   y:   y=y(x1,x2), function value at (x1,x2)
!   y1:  y1=dy/dx1(x1,x2), gradient at (x1,x2)
!   y2:  y2=dy/dx2(x1,x2), gradient at (x1,x2)
!
! PROCEDURE:
!   Cell geometry:
!     x2(2)  +-------+     / y(x1(2),x2(2))
!            |       | \   |
!            |       |  \ <  y1(x1(2),x2(2))
!            |       |     | y2(x1(2),x2(2))
!     x2(1)  +-------+     \ y12(x1(2),x2(2))
!          x1(1) x1(2)
!
!   Algorithm: 2 dimensional Horner scheme.
!
! MODIFICATION HISTORY:
!   18-Mar-1996 15:46:13 Written by Bernd Freytag, Astrophysik Kiel
!   22-Mar-1996 15:34:52 (B.F.) y, y1, y2 optional
!   24-Mar-1996 20:54:23 (B.F.) Ordering of 2d Horner scheme changed
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0)
! --- Input/output parameter
real(kind=rk),                      intent(in) :: x1, x2
real(kind=rk), dimension(1:4,1:4),  intent(in) :: C
real(kind=rk), optional,           intent(out) :: y, y1, y2
! --- Locals ---
integer                                        :: i
!------------------------------------------------------------------------------
!
! --- !DIR$ ALIGN
if (present(y)) then
!!  y =sum( C* matmul(reshape( (/1._rk,x1,x1*x1,x1*x1*x1/), (/4,1/) ), &
!!                    reshape( (/1._rk,x2,x2*x2,x2*x2*x2/), (/1,4/) )) )
  y=0.0
  do i=4,1,-1
    y=x2*y + C(1,i) + x1*(C(2,i) + x1*(C(3,i) + x1*C(4,i)))
  end do ! i
!  y=      C(1,1) + x1*(C(2,1) + x1*(C(3,1) + x1*C(4,1))) + &
!     x2*( C(1,2) + x1*(C(2,2) + x1*(C(3,2) + x1*C(4,2))) + &
!     x2*( C(1,3) + x1*(C(2,3) + x1*(C(3,3) + x1*C(4,3))) + &
!     x2*( C(1,4) + x1*(C(2,4) + x1*(C(3,4) + x1*C(4,4)))  )))
!!  y=  ((( ((C(4,4)*x1 +C(3,4))*x1 +C(2,4))*x1 +C(1,4) )*x2+ &
!!          ((C(4,3)*x1 +C(3,3))*x1 +C(2,3))*x1 +C(1,3) )*x2+ &
!!          ((C(4,2)*x1 +C(3,2))*x1 +C(2,2))*x1 +C(1,2) )*x2+ &
!!          ((C(4,1)*x1 +C(3,1))*x1 +C(2,1))*x1 +C(1,1)
endif
if (present(y1)) then
  y1=     C(2,1) + x1*(2._rk*C(3,1) + x1*3._rk*C(4,1)) + &
     x2*( C(2,2) + x1*(2._rk*C(3,2) + x1*3._rk*C(4,2)) + &
     x2*( C(2,3) + x1*(2._rk*C(3,3) + x1*3._rk*C(4,3)) + &
     x2*( C(2,4) + x1*(2._rk*C(3,4) + x1*3._rk*C(4,4))  )))
!  y1= ((( (3._rk*C(4,4)*x1 +2._rk*C(3,4))*x1 +C(2,4) )*x2+ &
!          (3._rk*C(4,3)*x1 +2._rk*C(3,3))*x1 +C(2,3) )*x2+ &
!          (3._rk*C(4,2)*x1 +2._rk*C(3,2))*x1 +C(2,2) )*x2+ &
!          (3._rk*C(4,1)*x1 +2._rk*C(3,1))*x1 +C(2,1)
endif
if (present(y2)) then
  y2=            C(1,2) + x1*(C(2,2) + x1*(C(3,2) + x1*C(4,2))) + &
      2._rk*x2*( C(1,3) + x1*(C(2,3) + x1*(C(3,3) + x1*C(4,3))) + &
     1.5_rk*x2*( C(1,4) + x1*(C(2,4) + x1*(C(3,4) + x1*C(4,4)))  ))
!  y2= (( 3._rk*(((C(4,4)*x1 +C(3,4))*x1 +C(2,4))*x1 +C(1,4)) )*x2+ &
!         2._rk*(((C(4,3)*x1 +C(3,3))*x1 +C(2,3))*x1 +C(1,3)) )*x2+ &
!                ((C(4,2)*x1 +C(3,2))*x1 +C(2,2))*x1 +C(1,2)
endif
!
end subroutine bicubic_interpol_1


!----------****************--------------------------------------------------------------------------
subroutine bicubic_interpol(x1,x2,C,x1in,x2in, x1off,x1fac,x2off,x2fac, y,y1,y2)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_interpol ('bicubic_interpolation')
!
! PURPOSE:
!   Do bicubic interpolation with coefficients derived e.g. by bicubic_coeff.
!   Only for regular grid.
!
! INPUT:
!   x1:   Independent value of first coordinate
!   x2:   Independent value of second coordinate
!   C:    Coefficients for bicubic interpolation
!   x1in: Independent vector of first coordinate
!   x2in: Independent vector of second coordinate
!   x1off,x1fac: Transformation parameters: x1_index=(x1-x1off)/x1fac+1
!   x2off,x2fac: Transformation parameters: x2_index=(x2-x2off)/x2fac+1
!
! OUTPUT:
!   y:   y=y(x1,x2), function value at (x1,x2)
!   y1:  y1=dy/dx1(x1,x2), gradient at (x1,x2)
!   y2:  y2=dy/dx2(x1,x2), gradient at (x1,x2)
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   20-Mar-1996 20:47:49 Written by Bernd Freytag, Astrophysik Kiel
!   22-Mar-1996 15:32:03 (B.F.) y, y1, y2 at end
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0)
! --- Input/output parameter ---
real(kind=rk),                      intent(in) :: x1, x2
real(kind=rk), dimension(1:,1:,1:,1:), &
                                    intent(in) :: C
real(kind=rk), dimension(1:),       intent(in) :: x1in        
real(kind=rk), dimension(1:),       intent(in) :: x2in        
real(kind=rk),                      intent(in) :: x1off, x1fac, x2off, x2fac
real(kind=rk), optional,           intent(out) :: y, y1, y2
!
! --- Local variables ---
integer                                        :: nx1, nx2, i1, i2
!------------------------------------------------------------------------------
!
! --- Size of coefficient array ---
nx1=size(C,3)
nx2=size(C,4)
!
! --- Indices of coefficient matrix ---
i1=min(max(int((x1-x1off)/x1fac),0)+1,nx1)
i2=min(max(int((x2-x2off)/x2fac),0)+1,nx2)
!
!write(6,*) 1,i1,nx1,x1in(i1),x1,x1in(i1+1)
!write(6,*) 1,i2,nx2,x2in(i2),x2,x2in(i2+2)
!
call bicubic_interpol_1(x1-x1in(i1),x2-x2in(i2),C(:,:,i1,i2), y,y1,y2)
!
!write(6,*) y,y1,y2
!
end subroutine bicubic_interpol


!----------********************----------------------------------------------------------------------
subroutine bicubic_interpol_gas(x1,x2,C,x1in,x2in, x1off,x1fac,x2off,x2fac, &
                                ya,ya1,ya2, yb,yb2, yc)
!----------------------------------------------------------------------------------------------------
! NAME:
!   bicubic_interpol_gas ('bicubic_interpolation_gas')
!
! PURPOSE:
!   Do bicubic interpolation with coefficients derived e.g. by bicubic_coeff.
!   Only for regular grid. Special version for GAS routine.
!
! INPUT:
!   x1:   Independent value of first coordinate
!   x2:   Independent value of second coordinate
!   C:    Coefficients for bicubic interpolation
!   x1in: Independent vector of first coordinate
!   x2in: Independent vector of second coordinate
!   x1off,x1fac: Transformation parameters: x1_index=(x1-x1off)/x1fac+1
!   x2off,x2fac: Transformation parameters: x2_index=(x2-x2off)/x2fac+1
!
! OUTPUT:
!   ya:   ya=ya(x1,x2), function value at (x1,x2)
!   ya1:  ya1=dya/dx1(x1,x2), gradient at (x1,x2)
!   ya2:  ya2=dya/dx2(x1,x2), gradient at (x1,x2)
!   yb, yb2: Same as ya for second quantity
!   yc:   Same as ya for third quantity
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   22-Mar-1996 16:00:00 Written by Bernd Freytag, Astrophysik Kiel
!   23-Mar-1996 21:25:58 (B.F.) nx1, nx2 from correct dimension of C
!   24-Mar-1996 20:20:16 (B.F.) yb1 removed
!   29-Mar-1996 23:25:48 (B.F.) Lots of changes
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0)
! --- Input/output parameter ---
real(kind=rk),                      intent(in) :: x1, x2
real(kind=rk), dimension(1:,1:,1:,1:,1:), &
                            target, intent(in) :: C
real(kind=rk), dimension(1:),       intent(in) :: x1in        
real(kind=rk), dimension(1:),       intent(in) :: x2in        
real(kind=rk),                      intent(in) :: x1off, x1fac, x2off, x2fac
real(kind=rk), optional,           intent(out) :: ya, ya1, ya2, yb, yb2, yc
!
! --- Local variables ---
integer                                        :: nx1, nx2, i1, i2
real(kind=rk)                                  :: x1t, x2t
!real(kind=rk), dimension(1:4,1:12)             :: Ct
real(kind=rk), dimension(:,:), pointer         :: Ct
!real(kind=rk), dimension(1:12)                 :: yt
!------------------------------------------------------------------------------
!
! --- Size of coefficient array ---
nx1=size(C,4)
nx2=size(C,5)
!
! --- Indices of coefficient matrix ---
i1=min(max(int((x1-x1off)/x1fac),0)+1,nx1)
i2=min(max(int((x2-x2off)/x2fac),0)+1,nx2)
!
!write(6,'(3I4,3E14.6)') 1,i1,nx1,x1in(i1),x1,x1in(i1+1)
!write(6,'(3I4,3E14.6)') 1,i2,nx2,x2in(i2),x2,x2in(i2+1)
!
x1t=x1-x1in(i1)
x2t=x2-x2in(i2)
!
!Ct=reshape(C(:,:,:,i1,i2),(/4,12/))
Ct=>C(:,:,1,i1,i2)
ya=      Ct(1, 1) + x1t*(Ct(2, 1) + x1t*(Ct(3, 1) + x1t*Ct(4, 1))) + &
   x2t*( Ct(1, 2) + x1t*(Ct(2, 2) + x1t*(Ct(3, 2) + x1t*Ct(4, 2))) + &
   x2t*( Ct(1, 3) + x1t*(Ct(2, 3) + x1t*(Ct(3, 3) + x1t*Ct(4, 3))) + &
   x2t*( Ct(1, 4) + x1t*(Ct(2, 4) + x1t*(Ct(3, 4) + x1t*Ct(4, 4)))  )))
ya1=       Ct(2,1) + x1t*(2._rk*Ct(3,1) + x1t*3._rk*Ct(4,1)) + &
     x2t*( Ct(2,2) + x1t*(2._rk*Ct(3,2) + x1t*3._rk*Ct(4,2)) + &
     x2t*( Ct(2,3) + x1t*(2._rk*Ct(3,3) + x1t*3._rk*Ct(4,3)) + &
     x2t*( Ct(2,4) + x1t*(2._rk*Ct(3,4) + x1t*3._rk*Ct(4,4))  )))
ya2=             Ct(1,2) + x1t*(Ct(2,2) + x1t*(Ct(3,2) + x1t*Ct(4,2))) + &
     2._rk*x2t*( Ct(1,3) + x1t*(Ct(2,3) + x1t*(Ct(3,3) + x1t*Ct(4,3))) + &
    1.5_rk*x2t*( Ct(1,4) + x1t*(Ct(2,4) + x1t*(Ct(3,4) + x1t*Ct(4,4)))  ))
Ct=>C(:,:,2,i1,i2)
yb=      Ct(1, 1) + x1t*(Ct(2, 1) + x1t*(Ct(3, 1) + x1t*Ct(4, 1))) + &
   x2t*( Ct(1, 2) + x1t*(Ct(2, 2) + x1t*(Ct(3, 2) + x1t*Ct(4, 2))) + &
   x2t*( Ct(1, 3) + x1t*(Ct(2, 3) + x1t*(Ct(3, 3) + x1t*Ct(4, 3))) + &
   x2t*( Ct(1, 4) + x1t*(Ct(2, 4) + x1t*(Ct(3, 4) + x1t*Ct(4, 4)))  )))
yb2=             Ct(1,2) + x1t*(Ct(2,2) + x1t*(Ct(3,2) + x1t*Ct(4,2))) + &
     2._rk*x2t*( Ct(1,3) + x1t*(Ct(2,3) + x1t*(Ct(3,3) + x1t*Ct(4,3))) + &
    1.5_rk*x2t*( Ct(1,4) + x1t*(Ct(2,4) + x1t*(Ct(3,4) + x1t*Ct(4,4)))  ))
Ct=>C(:,:,3,i1,i2)
yc=      Ct(1, 1) + x1t*(Ct(2, 1) + x1t*(Ct(3, 1) + x1t*Ct(4, 1))) + &
   x2t*( Ct(1, 2) + x1t*(Ct(2, 2) + x1t*(Ct(3, 2) + x1t*Ct(4, 2))) + &
   x2t*( Ct(1, 3) + x1t*(Ct(2, 3) + x1t*(Ct(3, 3) + x1t*Ct(4, 3))) + &
   x2t*( Ct(1, 4) + x1t*(Ct(2, 4) + x1t*(Ct(3, 4) + x1t*Ct(4, 4)))  )))
!
!do i=1,12
!  yt(i)=Ct(1,i) + x1t*(Ct(2,i) + x1t*(Ct(3,i) + x1t*Ct(4,i)))
!end do ! i
!ya =yt(1) + x2t*(yt( 2) + x2t*(yt( 3) + x2t*yt( 4)))
!yb =yt(5) + x2t*(yt( 6) + x2t*(yt( 7) + x2t*yt( 8)))
!yc =yt(9) + x2t*(yt(10) + x2t*(yt(11) + x2t*yt(12)))
!ya2=yt(2) + 2._rk*x2t*(yt(3) + 1.5_rk*x2t*yt(4))
!yb2=yt(6) + 2._rk*x2t*(yt(7) + 1.5_rk*x2t*yt(8))
!
!Ct=>C(:,:,1,i1,i2)
!call bicubic_interpol_1(x1t,x2t,Ct, y=ya,y1=ya1,y2=ya2)
!Ct=>C(:,:,2,i1,i2)
!call bicubic_interpol_1(x1t,x2t,Ct, y=yb,       y2=yb2)
!Ct=>C(:,:,3,i1,i2)
!call bicubic_interpol_1(x1t,x2t,Ct, y=yc)
!
end subroutine bicubic_interpol_gas

end module bicubic_module
