!------**************--------------------------------------------------------------------------------
module broyden_module
!----------------------------------------------------------------------------------------------------
! NAME:
!   broyden_module ('broyden')
!
! PURPOSE:
!   Search for zero of n dimensional function.
!
! MODIFICATION HISTORY:
!   18-Mar-1996 20:25:50 Written by Bernd Freytag
!----------------------------------------------------------------------------------------------------
contains

!----------*******-----------------------------------------------------------------------------------
subroutine broyden(xin,testfunction,eps,kmax, xout, reuse)
!----------------------------------------------------------------------------------------------------
! NAME:
!   broyden ('broyden')
!
! PURPOSE:
!   Find zero of function
!
! INPUT:
!
! OUTPUT:
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   18-Mar-1996 20:28:46 Written by Bernd Freytag, Astrophysik Kiel
!----------------------------------------------------------------------------------------------------
use matinv_gj_module                      ! --- Module for matrix inversion ---
!
implicit none
!
! --- kind of reals ---
integer, parameter                                 :: rk=kind(1.0)
! --- Input/output parameter ---
real(kind=rk), dimension(1:),          intent(in)  :: xin
real(kind=rk),                         intent(in)  :: eps
integer,                               intent(in)  :: kmax
logical, optional,                     intent(in)  :: reuse
real(kind=rk), dimension(1:size(xin,1)), intent(out) :: xout
interface
  function testfunction(x)
  integer, parameter                               :: rk=kind(1.0)
  real(kind=rk), dimension(1:),        intent(in)  :: x
  real(kind=rk),dimension(1:size(x,1))             :: testfunction
  end function testfunction
end interface
!
! --- Local variables ---
integer                                            :: k, ndim, n
real(kind=rk)                                      :: eps2
real(kind=rk), dimension(1:size(xin,1),1)          :: xold, xnew, yold, ynew, &
                                                      dx, dy, Ainvdy
real(kind=rk), dimension(1:1,1:1)                  :: nominator
!real(kind=rk), dimension(1:size(xin,1),1:size(xin,1)), save :: Ainv
real(kind=rk), dimension(:,:), allocatable, save :: Ainv
integer i,j
!----------------------------------------------------------------------------------------------------
!
! --- Number of parameters to be fitted ---
ndim=size(xin,1)
allocate(Ainv(1:ndim,1:ndim))
!
! --- Square of eps, used for convergence testing --- 
eps2=eps*eps
!
! --- Start guess for vector ---
xold(:,1)=xin
! --- Function value at start point ---
yold(:,1)=testfunction(xold(:,1))
!
if (.not. present(reuse)) then
  ! --- Make start broyden matrix from approximate Jacobian ---
  !
  ! --- Find vector with reasonable (small) step size ---
  dx=0.001_rk*abs(xold) + 100.0_rk*tiny(xold)
  ! --- Compute approximate Jacobian Ainv from finite differences ---
  do n=1,ndim
    xnew=xold
    xnew(n,1)=xnew(n,1)+dx(n,1)
    ynew(:,1)=testfunction(xnew(:,1))
    Ainv(n,:)=(ynew(:,1)-yold(:,1))/dx(n,1)
  end do ! n
  write(6,*) 'A='
  write(6,'(3(F10.5))') ((Ainv(i,j),i=1,ndim),j=1,ndim)
  !
  ! --- Invert Jacobian ---
  call matinv_gj(Ainv)
endif
!
! --- Iteration loop ---
do k=1,kmax
write(6,*) 'k=',k
write(6,'(A,3(F10.5))') 'xold=', (xold(i,1),i=1,ndim)
write(6,'(A,3(F10.5))') 'yold=', (yold(i,1),i=1,ndim)
write(6,*) 'Ainv='
write(6,'(3(F10.5))') ((Ainv(i,j),i=1,ndim),j=1,ndim)
  ! --- New x ---
  dx=-matmul(Ainv,yold)
  xnew=xold+dx
write(6,'(A,3(F10.5))') 'dx  =', (dx(i,1),i=1,ndim)
write(6,'(A,3(F10.5))') 'xnew=', (xnew(i,1),i=1,ndim)
  ! --- y values for new x ---
  ynew(:,1)=testfunction(xnew(:,1))
  dy=ynew-yold
write(6,'(A,3(F10.5))') 'dy  =', (dy(i,1),i=1,ndim)
write(6,'(A,3(F10.5))') 'ynew=', (ynew(i,1),i=1,ndim)
  !
  ! --- Convergence test: last step dx smaller than limit eps? ---
  if (sum(dx*dx) < eps2) exit
  !
  ! --- Update inverse Broyden matrix ---
  Ainvdy=matmul(Ainv,dy)
write(6,'(A,3(F10.5))') 'Aidy=', (Ainvdy(i,1),i=1,ndim)
write(6,*) matmul(dx-Ainvdy,matmul(transpose(dx),Ainv))
write(6,*) matmul(transpose(dx),Ainvdy)
write(6,*) size(matmul(transpose(dx),Ainvdy))
  nominator=matmul(transpose(dx),Ainvdy)
  Ainv=Ainv + matmul(dx-Ainvdy,matmul(transpose(dx),Ainv))/nominator(1,1)
  !
  ! --- New becomes old ---
  xold=xnew
  yold=ynew
end do
!
! --- Output of last x value ---
xout=xnew(:,1)
!
end subroutine broyden

end module broyden_module
