!------****************------------------------------------------------------------------------------
module matinv_gj_module
!------------------------------------------------------------------------------
! NAME:
!   matinv_gj_module ('matrix_invert_Gauss_Jordan')
!
! PURPOSE:
!   Invert matrix by Gauss-Jordan elimination.
!
! MODIFICATION HISTORY:
!   12-Mar-1996 15:58:14 Converted to Fortran90 by Bernd Freytag
!------------------------------------------------------------------------------
!
interface matinv_gj
!  module procedure matinv_gj_singlereal, matinv_gj_doublereal
  module procedure matinv_gj_singlereal
end interface
!
contains

!----------********************----------------------------------------------------------------------
subroutine matinv_gj_singlereal(A)
!----------------------------------------------------------------------------------------------------
! NAME:
!   matinv_gj_singlereal ('matrix_invert_Gauss_Jordan_single_real')
!
! PURPOSE:
!   Invert matrix by Gauss-Jordan elimination.
!
! INPUT/OUTPUT:
!   A: Single precision real array, serves as input and output.
!
! MODIFICATION HISTORY:
!   Converted to standard FORTRAN 77 by HGL 31-MAR-1992 15:26:33
!   Converted to double precision by HGL 4-APR-1992 15:58:29
!   12-Mar-1996 15:58:14 Converted to Fortran90 by Bernd Freytag
!   17-Mar-1996 17:12:50 (B.F.) inv -> matinv
!   18-Mar-1996 15:30:09 (B.F.) Amax -> abs(Amax)
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0)
! --- I/O parameter ---
real(kind=rk), dimension(1:,1:), intent(inout) :: A
!
! --- Local variables ---
integer                                        :: n
real(kind=rk)                                  :: Amax
real(kind=rk), dimension(size(A,1))            :: save
integer, dimension(size(A,1))                  :: ik, jk
integer                                        :: i, j, k, l
integer, dimension(2)                          :: ijmax
!------------------------------------------------------------------------------
!
n=size(A,1)
!
! --- Check if A is square matrix ---
if (n /= size(A,2)) stop 'matinv_gj: matrix is not square.'
!
do k=1,n
  ! --- Search maximum matrix element ---
  ijmax=maxloc(abs(A(k:n,k:n)))+k-1
  ik(k)=ijmax(1)
  jk(k)=ijmax(2)
  Amax=A(ik(k),jk(k))
  !
  ! --- stop in case of singularity (pivot element = 0) ---
  if (abs(Amax) < 5.0_rk*tiny(Amax)) stop 'matinv_gj: singularity discovered.'
  !
  ! --- Swap lines i and k, change sign ---
  i=ik(k)
  if (i /= k) then
    save  =A(k,:)
    A(k,:)=A(i,:)
    A(i,:)=-save
  endif
  !
  ! --- Swap columns j and k, change sign ---
  j=jk(k)
  if (j /= k) then
    save  =A(:,k)
    A(:,k)=A(:,j)
    A(:,j)=-save
  endif
  !
  ! --- Divide column k by -Amax ---
  A(:,k)=A(:,k)*(-1.0_rk/Amax)
  A(k,k)=0.0_rk
  !
  ! --- Transform lines and columns ---
  ! --- Basic transformation: Line i = Line i + A(i,k) * Line k
  do i=1,n
    if (i /= k) A(i,:)=A(i,:) + A(i,k)*A(k,:)
  end do ! i
  !
  ! --- Column k: division by Amax ---
  A(k,:)=A(k,:)/Amax
  !
  ! --- Diagonal element ---
  A(k,k)=1.0_rk/Amax
  !
end do ! k
!
! --- Back swapping of lines and columns ---
do l=1,n
  k=n-l+1
  j=ik(k)
  if (j > k) then
    ! --- Swap columns j and k, change back sign ---
    save  =A(:,k)
    A(:,k)=-A(:,j)
    A(:,j)=save
  endif
  i=jk(k)
  if(i > k) then
    ! --- Swap lines i and k, change back sign ---
    save=A(k,:)
    A(k,:)=-a(i,:)
    A(i,:)=save
  endif
end do ! l
!
end subroutine matinv_gj_singlereal


!----------********************----------------------------------------------------------------------
subroutine matinv_gj_doublereal(A)
!----------------------------------------------------------------------------------------------------
! NAME:
!   matinv_gj_doublereal ('matrix_invert_Gauss_Jordan_double_real')
!
! PURPOSE:
!   Invert matrix by Gauss-Jordan elimination.
!
! INPUT/OUTPUT:
!   A: Double precision real array, serves as input and output.
!
! MODIFICATION HISTORY:
!   Converted to standard FORTRAN 77 by HGL 31-MAR-1992 15:26:33
!   Converted to double precision by HGL 4-APR-1992 15:58:29
!   12-Mar-1996 15:58:14 Converted to Fortran90 by Bernd Freytag
!   12-Mar-1996 17:46:51 (B.F.) single and double precision version
!   17-Mar-1996 17:14:36 (B.F.) inv -> matinv
!   18-Mar-1996 15:30:09 (B.F.) Amax -> abs(Amax)
!------------------------------------------------------------------------------
implicit none
!
! --- kind of reals ---
integer, parameter                             :: rk=kind(1.0D0)
! --- Input/output ---
real(kind=rk), dimension(1:,1:), intent(inout) :: A
!
integer                                        :: n
real(kind=rk)                                  :: Amax
real(kind=rk), dimension(size(A,1))            :: save
integer, dimension(size(A,1))                  :: ik, jk
integer                                        :: i, j, k, l
integer, dimension(2)                          :: ijmax
!------------------------------------------------------------------------------
!
n=size(A,1)
!
! --- Check if A is square matrix ---
if (n /= size(A,2)) stop 'matinv_gj: matrix is not square.'
!
do k=1,n
  ! --- Search maximum matrix element ---
  ijmax=maxloc(abs(A(k:n,k:n)))+k-1
  ik(k)=ijmax(1)
  jk(k)=ijmax(2)
  Amax=A(ik(k),jk(k))
  !
  ! --- stop in case of singularity (pivot element = 0) ---
  if (abs(Amax) < 5.0_rk*tiny(Amax)) stop 'matinv_gj: singularity discovered.'
  !
  ! --- Swap lines i and k, change sign ---
  i=ik(k)
  if (i /= k) then
    save  =A(k,:)
    A(k,:)=A(i,:)
    A(i,:)=-save
  endif
  !
  ! --- Swap columns j and k, change sign ---
  j=jk(k)
  if (j /= k) then
    save  =A(:,k)
    A(:,k)=A(:,j)
    A(:,j)=-save
  endif
  !
  ! --- Divide column k by -Amax ---
  A(:,k)=A(:,k)*(-1.0_rk/Amax)
  A(k,k)=0.0_rk
  !
  ! --- Transform lines and columns ---
  ! --- Basic transformation: Line i = Line i + A(i,k) * Line k
  do i=1,n
    if (i /= k) A(i,:)=A(i,:) + A(i,k)*A(k,:)
  end do ! i
  !
  ! --- Column k: division by Amax ---
  A(k,:)=A(k,:)/Amax
  !
  ! --- Diagonal element ---
  A(k,k)=1.0_rk/Amax
  !
end do ! k
!
! --- Back swapping of lines and columns ---
do l=1,n
  k=n-l+1
  j=ik(k)
  if (j > k) then
    ! --- Swap columns j and k, change back sign ---
    save  =A(:,k)
    A(:,k)=-A(:,j)
    A(:,j)=save
  endif
  i=jk(k)
  if(i > k) then
    ! --- Swap lines i and k, change back sign ---
    save=A(k,:)
    A(k,:)=-a(i,:)
    A(i,:)=save
  endif
end do ! l
!
end subroutine matinv_gj_doublereal

end module matinv_gj_module
