!***************************************************************************************************
!
!                       #     # #######
!    ####  ##### #####  #     #    #
!   #        #   #    # #     #    #
!    ####    #   #    # #     #    #
!        #   #   #####  #     #    #
!   #    #   #   #   #  #     #    #
!    ####    #   #    #  #####     #  _module
!
!   Unit-test routines for string-handling routines
!
!***************************************************************************************************
!   Fortran 90
!   2017-08-26
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!***************************************************************************************************
! MODULES:
!   strUT_module:          Unit-test routines for string-handling routines
!
!***************************************************************************************************


!------****************-----------------------------------------------------------------------------
module strUT_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   strUT_module ('string_Unit_Test_module')
!
! PURPOSE:
!   Provide unit-test routines for string-handling routines.
!
! CATEGORY:
!   Testing, str handling
!
! CALLING SEQUENCE:
!   use strUT_module
!
! TYPES:
!   None
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   str_lowercaseUT:       Perform unit tests for str_lowercase
!   str_uppercaseUT:       Perform unit tests for str_uppercase
!   str_pathandfileUT:     Perform unit tests for str_pathandfile
!   strf_IsEqualCIUT:      Perform unit tests for strf_IsEqualCI
!   strUT_TestAll:         Perform all unit tests of string-handling routines
!   strUT_TestSome:        Perform some unit tests of string-handling routines
!
! MODIFICATION HISTORY:
!   2016-04-29 (B.F. Uppsala) First version
!   2016-04-30 (B.F.) Test str_lowercase, str_uppercase
!   2016-05-01 (B.F.) Test str_pathandfile
!   2016-05-02 (B.F.) Test strf_IsEqualCI
!   2017-08-26 (B.F. Lyon) Add routine for partial unit tests
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------***************--------------------------------------------------------------------------
subroutine str_lowercaseUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_lowercaseUT ('string_lower_case_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for str_lowercase.
!
! CATEGORY:
!   String manipulation, testing
!
! CALLING SEQUENCE:
!   call str_lowercaseUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   str_lowercase:         Transform all uppercase letters into lowercase form
!   str_uppercase:         Transform all lowercase letters into uppercase form
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   str_module:            String handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call str_lowercaseUT('str')
!
! MODIFICATION HISTORY:
!   2016-04-30 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use str_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer, parameter                      :: nA=15
character(len=128)                      :: StringName, TestName
character(len=12)                       :: s, sin, sout, strue
character(len=12), dimension(nA)        :: sAin, sAout0, sAout1, sAout2, sAtrue
integer                                 :: iString, i
!---------------------------------------------------------------------------------------------------
!
! === Create test case =============================================================================
StringName='Blank'
sin       =''
strue     =sin
! --- Transform the input string ---
call str_lowercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='a'
sin       ='a'
strue     =sin
! --- Transform the input string ---
call str_lowercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='A'
sin       ='A'
strue     ='a'
! --- Transform the input string ---
call str_lowercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='Z'
sin       ='Z'
strue     ='z'
! --- Transform the input string ---
call str_lowercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='a-i'
sin       ='abcdefghi'
strue     =sin
! --- Transform the input string ---
call str_lowercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
!
! === Create test cases ============================================================================
do iString=1,8
  ! --- Choose one out of 8 base strings ---
  if (iString == 1) then
    StringName='a-gPlus'
    s         ='abcdefg'
    strue     =s
  else if (iString == 2) then
    StringName='h-nPlus'
    s         ='hijklmn'
    strue     =s
  else if (iString == 3) then
    StringName='o-tPlus'
    s         ='opqrst'
    strue     =s
  else if (iString == 4) then
    StringName='u-zPlus'
    s         ='uvwxyz'
    strue     =s
  else if (iString == 5) then
    StringName='A-GPlus'
    s         ='ABCDEFG'
    strue     ='abcdefg'
  else if (iString == 6) then
    StringName='H-NPlus'
    s         ='HIJKLMN'
    strue     ='hijklmn'
  else if (iString == 7) then
    StringName='O-TPlus'
    s         ='OPQRST'
    strue     ='opqrst'
  else if (iString == 8) then
    StringName='U-ZPlus'
    s         ='UVWXYZ'
    strue     ='uvwxyz'
  endif
  !
  ! --- Combine the base string with others and give the correct result ---
  sAin      =(/ s, &
                ' '      // s, &
                'axgt'   // s, &
                trim(s)  // 'axgt', &
                'QcKt'   // s, &
                trim(s)  // 'QcKt', &
                'Ha'     // trim(s) // 'xH', &
                '01234'  // s, &
                '56789'  // s, &
                '`~!@#'  // s, &
                '$%^&*'  // s, &
                '()-_+'  // s, &
                '=[]{}'  // s, &
                '\|;:''' // s, &
                ',.<>/'  // s &
              /)
  sAtrue    =(/ strue, &
                ' '         // strue, &
                'axgt'      // strue, &
                trim(strue) // 'axgt', &
                'qckt'      // strue, &
                trim(strue) // 'qckt', &
                'ha'        // trim(strue) // 'xh', &
                '01234'     // strue, &
                '56789'     // strue, &
                '`~!@#'     // strue, &
                '$%^&*'     // strue, &
                '()-_+'     // strue, &
                '=[]{}'     // strue, &
                '\|;:'''    // strue, &
                ',.<>/'     // strue &
              /)
  !
  ! --- Transform the input strings ---
  do i=1,nA
    call str_lowercase(sAin(  i), sAout0(i))
    call str_uppercase(sAout0(i), sAout1(i))
    call str_lowercase(sAout1(i), sAout2(i))
  end do ! i
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
           'TransformationIsCorrect'
  call UT_Assert('==', TestName, sAout0, sAtrue)
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_lowercase_' // 'TestStr=' // trim(StringName) // '_' // &
           'TransUpperLowerIsCorrect'
  call UT_Assert('==', TestName, sAout2, sAout0)
  !
end do ! iString
!
end subroutine str_lowercaseUT


!----------***************--------------------------------------------------------------------------
subroutine str_uppercaseUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_uppercaseUT ('string_upper_case_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for str_uppercase.
!
! CATEGORY:
!   String manipulation, testing
!
! CALLING SEQUENCE:
!   call str_uppercaseUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   str_lowercase:         Transform all uppercase letters into lowercase form
!   str_uppercase:         Transform all lowercase letters into uppercase form
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   str_module:            String handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call str_uppercaseUT('str')
!
! MODIFICATION HISTORY:
!   2016-04-30 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use str_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
integer, parameter                      :: nA=15
character(len=128)                      :: StringName, TestName
character(len=12)                       :: s, sin, sout, strue
character(len=12), dimension(nA)        :: sAin, sAout0, sAout1, sAout2, sAtrue
integer                                 :: iString, i
!---------------------------------------------------------------------------------------------------
!
! === Create test case =============================================================================
StringName='Blank'
sin       =''
strue     =sin
! --- Transform the input string ---
call str_uppercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='A'
sin       ='A'
strue     =sin
! --- Transform the input string ---
call str_uppercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='a'
sin       ='a'
strue     ='A'
! --- Transform the input string ---
call str_uppercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='z'
sin       ='z'
strue     ='Z'
! --- Transform the input string ---
call str_uppercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
! === Create test case =============================================================================
StringName='A-I'
sin       ='ABCDEFGHI'
strue     =sin
! --- Transform the input string ---
call str_uppercase(sin, sout)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
         'TransformationIsCorrect'
call UT_Assert('==', TestName, (/sout/), (/strue/))
!
!
! === Create test cases ============================================================================
do iString=1,8
  ! --- Choose one out of 8 base strings ---
  if (iString == 1) then
    StringName='a-gPlus'
    s         ='abcdefg'
    strue     ='ABCDEFG'
  else if (iString == 2) then
    StringName='h-nPlus'
    s         ='hijklmn'
    strue     ='HIJKLMN'
  else if (iString == 3) then
    StringName='o-tPlus'
    s         ='opqrst'
    strue     ='OPQRST'
  else if (iString == 4) then
    StringName='u-zPlus'
    s         ='uvwxyz'
    strue     ='UVWXYZ'
  else if (iString == 5) then
    StringName='A-GPlus'
    s         ='ABCDEFG'
    strue     =s
  else if (iString == 6) then
    StringName='H-NPlus'
    s         ='HIJKLMN'
    strue     =s
  else if (iString == 7) then
    StringName='O-TPlus'
    s         ='OPQRST'
    strue     =s
  else if (iString == 8) then
    StringName='U-ZPlus'
    s         ='UVWXYZ'
    strue     =s
  endif
  !
  ! --- Combine the base string with others and give the correct result ---
  sAin      =(/ s, &
                ' '      // s, &
                'axgt'   // s, &
                trim(s)  // 'axgt', &
                'QcKt'   // s, &
                trim(s)  // 'QcKt', &
                'Ha'     // trim(s) // 'xH', &
                '01234'  // s, &
                '56789'  // s, &
                '`~!@#'  // s, &
                '$%^&*'  // s, &
                '()-_+'  // s, &
                '=[]{}'  // s, &
                '\|;:''' // s, &
                ',.<>/'  // s &
              /)
  sAtrue    =(/ strue, &
                ' '         // strue, &
                'AXGT'      // strue, &
                trim(strue) // 'AXGT', &
                'QCKT'      // strue, &
                trim(strue) // 'QCKT', &
                'HA'        // trim(strue) // 'XH', &
                '01234'     // strue, &
                '56789'     // strue, &
                '`~!@#'     // strue, &
                '$%^&*'     // strue, &
                '()-_+'     // strue, &
                '=[]{}'     // strue, &
                '\|;:'''    // strue, &
                ',.<>/'     // strue &
              /)
  !
  ! --- Transform the input strings ---
  do i=1,nA
    call str_uppercase(sAin(  i), sAout0(i))
    call str_lowercase(sAout0(i), sAout1(i))
    call str_uppercase(sAout1(i), sAout2(i))
  end do ! i
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
           'TransformationIsCorrect'
  call UT_Assert('==', TestName, sAout0, sAtrue)
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_uppercase_' // 'TestStr=' // trim(StringName) // '_' // &
           'TransLowerUpperIsCorrect'
  call UT_Assert('==', TestName, sAout2, sAout0)
  !
end do ! iString
!
end subroutine str_uppercaseUT


!----------*****************------------------------------------------------------------------------
subroutine str_pathandfileUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_pathandfileUT ('string_path_and_file_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for str_pathandfile.
!
! CATEGORY:
!   String manipulation, testing
!
! CALLING SEQUENCE:
!   call str_pathandfileUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   str_pathandfile:       Merge path name and file name
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   str_module:            String handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call str_pathandfileUT('str')
!
! MODIFICATION HISTORY:
!   2016-05-01 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
use str_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
character(len=128)                      :: StringName, TestName
character(len=80)                       :: path, file, pathandfilecomp, pathandfiletrue
!---------------------------------------------------------------------------------------------------
!
! === Create test case =============================================================================
path           =' '
file           =' '
pathandfiletrue=' '
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // 'Blank' // '#1_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
! === Create test case =============================================================================
path           ='/'
file           =' '
pathandfiletrue=' '
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // 'Blank' // '#2_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
! === Create test case =============================================================================
path           ='bak'
file           =' '
pathandfiletrue=' '
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // 'Blank' // '#3_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
! === Create test case =============================================================================
path           ='bak/'
file           =' '
pathandfiletrue=' '
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // 'Blank' // '#4_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
!
! === Create test case =============================================================================
path           =' '
file           ='file.dat'
pathandfiletrue='file.dat'
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // trim(pathandfiletrue) // '_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
! === Create test case =============================================================================
path           ='bak'
file           ='file.dat'
pathandfiletrue='bak/file.dat'
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // trim(pathandfiletrue) // '#1_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
! === Create test case =============================================================================
path           ='bak/'
file           ='file.dat'
pathandfiletrue='bak/file.dat'
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // trim(pathandfiletrue) // '#2_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
! === Create test case =============================================================================
path           ='[bak]'
file           ='file.dat'
pathandfiletrue='[bak]file.dat'
!
! --- Merge the input strings ---
call str_pathandfile(path,file, pathandfilecomp)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_pathandfile_' // 'TestStr=' // trim(pathandfiletrue) // '_' // &
         'MergingIsCorrect'
call UT_Assert('==', TestName, (/pathandfilecomp/), (/pathandfiletrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data
  call UT_PrintArray('Via=', (/path/))
  call UT_PrintArray('Vib=', (/file/))
endif
!
end subroutine str_pathandfileUT


!----------*****************------------------------------------------------------------------------
subroutine strf_IsEqualCIUT(TestPrefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   strf_IsEqualCIUT ('string_function_Is_Equal_Case_Insensitive_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for strf_IsEqualCI.
!
! CATEGORY:
!   String manipulation, testing
!
! CALLING SEQUENCE:
!   call strf_IsEqualCIUT(TestPrefix)
!
! INPUT:
!   TestPrefix:   ('Test_Prefix') character with prefix of testname
!
! OUTPUT:
!   None
!
! ROUTINES:
!   strf_IsEqualCI:        Function: test if two strings are equal ignoring case
!   UT_Assert:             Check array or relation between two arrays
!
! MODULES:
!   str_module:            String handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call strf_IsEqualCIUT('str')
!
! MODIFICATION HISTORY:
!   2016-05-02 (B.F. Uppsala) First version
!   2017-05-17 (B.F.) Ensure that all strings have the same length in array constructor
!---------------------------------------------------------------------------------------------------
use str_module
use UT_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: TestPrefix
!
! --- Local variables ---
character(len=128)                      :: StringName, TestName
character(len=12)                       :: sin1, sin2
logical                                 :: IsEqualComp, IsEqualTrue
integer, parameter                      :: nTest=8
character(len=12), dimension(nTest)     :: s1Test, s2Test
logical, dimension(nTest)               :: IsEqualTestComp, IsEqualTestTrue
integer                                 :: iTest, jTest
!---------------------------------------------------------------------------------------------------
!
! === Create test case =============================================================================
sin1       =' '
sin2       =' '
IsEqualTrue=.true.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // 'TwoBlanks' // '_' // &
         'LowerCaseIsEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
sin1       =' '
sin2       ='a'
IsEqualTrue=.false.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // 'FirstBlank' // '_' // &
         'LowerCaseIsNotEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
sin1       ='a'
sin2       =' '
IsEqualTrue=.false.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // 'SecondBlank' // '_' // &
         'LowerCaseIsNotEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
sin1       ='a'
sin2       ='a'
IsEqualTrue=.true.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // trim(sin1) // '+' // trim(sin2) // '_' // &
         'LowerCaseIsEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
sin1       ='a'
sin2       ='A'
IsEqualTrue=.true.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // trim(sin1) // '+' // trim(sin2) // '_' // &
         'LowerCaseIsEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
sin1       ='A'
sin2       ='a'
IsEqualTrue=.true.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // trim(sin1) // '+' // trim(sin2) // '_' // &
         'LowerCaseIsEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
sin1       ='A'
sin2       ='A'
IsEqualTrue=.true.
!
! --- Compare the input strings ---
IsEqualComp=strf_IsEqualCI(sin1, sin2)
!
! --- Compose the test name and perform the test ---
TestName=trim(TestPrefix) // '_IsEqualCI_' // &
         'TestStr=' // trim(sin1) // '+' // trim(sin2) // '_' // &
         'LowerCaseIsEqual'
call UT_Assert('==', TestName, (/IsEqualComp/), (/IsEqualTrue/))
!
if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
  ! --- Print the input data ---
  call UT_PrintArray('Via=', (/sin1/))
  call UT_PrintArray('Vib=', (/sin2/))
endif
!
! === Create test case =============================================================================
s1Test=(/'aa', 'aA', 'Aa', 'AA', &
         'bb', 'bB', 'Bb', 'BB'   /)
do iTest=1,nTest
  s2Test                            =s1Test(iTest)
  IsEqualTestTrue(        1:nTest/2)=(iTest <= nTest/2)
  IsEqualTestTrue(nTest/2+1:nTest  )=(iTest >  nTest/2)
  !
  do jTest=1,nTest
    ! --- Compare the input strings ---
    IsEqualTestComp(jTest)=strf_IsEqualCI(s1Test(jTest), s2Test(jTest))
  end do ! jTest
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_IsEqualCI_' // &
           'TestStr=' // 'List' // '+' // trim(s2Test(1)) // '_' // &
           'CompareLowerCase'
  call UT_Assert('==', TestName, IsEqualTestComp, IsEqualTestTrue)
  !
  if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
    ! --- Print the input data ---
    call UT_PrintArray('Via=', s1Test)
    call UT_PrintArray('Vib=', s2Test)
  endif
  !
end do ! iTest
!
! === Create test case =============================================================================
s1Test=(/'xaa ', 'xaA ', 'xAa ', 'xAA ', &
         'xaaa', 'xaAa', 'xAaa', 'xAAa'  /)
do iTest=1,nTest
  s2Test                            =s1Test(iTest)
  IsEqualTestTrue(        1:nTest/2)=(iTest <= nTest/2)
  IsEqualTestTrue(nTest/2+1:nTest  )=(iTest >  nTest/2)
  !
  do jTest=1,nTest
    ! --- Compare the input strings ---
    IsEqualTestComp(jTest)=strf_IsEqualCI(s1Test(jTest), s2Test(jTest))
  end do ! jTest
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_IsEqualCI_' // &
           'TestStr=' // 'List' // '+' // trim(s2Test(1)) // '_' // &
           'CompareLowerCase'
  call UT_Assert('==', TestName, IsEqualTestComp, IsEqualTestTrue)
  !
  if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
    ! --- Print the input data ---
    call UT_PrintArray('Via=', s1Test)
    call UT_PrintArray('Vib=', s2Test)
  endif
  !
end do ! iTest
!
! === Create test case =============================================================================
s1Test=(/'hello', 'Hello', 'HellO', 'HELLO', &
         'hallo', 'Hallo', 'HallO', 'HALLO'  /)
do iTest=1,nTest
  s2Test                            =s1Test(iTest)
  IsEqualTestTrue(        1:nTest/2)=(iTest <= nTest/2)
  IsEqualTestTrue(nTest/2+1:nTest  )=(iTest >  nTest/2)
  !
  do jTest=1,nTest
    ! --- Compare the input strings ---
    IsEqualTestComp(jTest)=strf_IsEqualCI(s1Test(jTest), s2Test(jTest))
  end do ! jTest
  !
  ! --- Compose the test name and perform the test ---
  TestName=trim(TestPrefix) // '_IsEqualCI_' // &
           'TestStr=' // 'List' // '+' // trim(s2Test(1)) // '_' // &
           'CompareLowerCase'
  call UT_Assert('==', TestName, IsEqualTestComp, IsEqualTestTrue)
  !
  if (UT%AllTestPrint_flag .and. UT%AllVecInPrint_flag) then
    ! --- Print the input data ---
    call UT_PrintArray('Via=', s1Test)
    call UT_PrintArray('Vib=', s2Test)
  endif
  !
end do ! iTest
!
end subroutine strf_IsEqualCIUT


!----------*************----------------------------------------------------------------------------
subroutine strUT_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   strUT_TestAll ('strint_Unit_Test_Test_All')
!
! PURPOSE:
!   Perform all unit tests of string-handling routines.
!
! CATEGORY:
!   Testing, string handling
!
! CALLING SEQUENCE:
!   call strUT_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   str_lowercaseUT:       Perform unit tests for str_lowercase
!   str_uppercaseUT:       Perform unit tests for str_uppercase
!   str_pathandfileUT:     Perform unit tests for str_pathandfile
!   strf_IsEqualCIUT:      Perform unit tests for strf_IsEqualCI
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in the test subroutines.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call strUT_TestAll()
!
! MODIFICATION HISTORY:
!   2016-04-29 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='str'
!
call str_lowercaseUT(TestPrefix)
call str_uppercaseUT(TestPrefix)
call str_pathandfileUT(TestPrefix)
call strf_IsEqualCIUT(TestPrefix)
!
end subroutine strUT_TestAll


!----------**************---------------------------------------------------------------------------
subroutine strUT_TestSome()
!---------------------------------------------------------------------------------------------------
! NAME:
!   strUT_TestSome ('strint_Unit_Test_Test_Some')
!
! PURPOSE:
!   Perform some unit tests of string-handling routines.
!
! CATEGORY:
!   Testing, string handling
!
! CALLING SEQUENCE:
!   call strUT_TestSome()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   str_lowercaseUT:       Perform unit tests for str_lowercase
!   str_uppercaseUT:       Perform unit tests for str_uppercase
!   str_pathandfileUT:     Perform unit tests for str_pathandfile
!   strf_IsEqualCIUT:      Perform unit tests for strf_IsEqualCI
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Global variables are changed and output is printed via calls to UT_Assert
!   in the test subroutines.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call strUT_TestSome()
!
! MODIFICATION HISTORY:
!   2016-04-29 (B.F. Uppsala) First version
!   2017-08-26 (B.F. Lyon) Adapted from strUT_TestAll
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- Local variables ---
character(len=80)                       :: TestPrefix
!---------------------------------------------------------------------------------------------------
!
TestPrefix='str'
!
!call str_lowercaseUT(TestPrefix)
!call str_uppercaseUT(TestPrefix)
!call str_pathandfileUT(TestPrefix)
call strf_IsEqualCIUT(TestPrefix)
!
end subroutine strUT_TestSome

end module strUT_module
