!***************************************************************************************************
!
!    #### ##### #####  
!   #       #   #    # 
!    ####   #   #    # 
!        #  #   #####  
!   #    #  #   #   #  
!    ####   #   #    #_module
!
!   String Manipulation Routines
!
!***************************************************************************************************
!   This module provides string manipulation routines.
!***************************************************************************************************
!   Fortran90
!   2016-05-02
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Lyon, Uppsala
!***************************************************************************************************
! MODULES:
!   str_module:            String handling routines
!
!***************************************************************************************************

!------**********-----------------------------------------------------------------------------------
module str_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_module ('string_module')
!
! PURPOSE:
!   Provide string manipulation routines.
!
! CATEGORY:
!   String manipulation
!
! CALLING SEQUENCE:
!   use str_module
!
! ROUTINES: (CONTAINED)
! > str_form:              Remove all characters after the first blank, substitute some substrings
! > str_cut:               Cut input string into two parts at first blank
! > str_count:             Increment string counter
!   str_str2int:           Transform a string into a positive integer
!   str_int2str:           Transform a positive integer into a string
! > str_sub:               Substitute every occurrence of substring in string by another
! > str_lowercase:         Transform all uppercase letters into lowercase form
! > str_uppercase:         Transform all lowercase letters into uppercase form
! > str_pathandfile:       Merge path name and file name
! > str_identcompose:      Compose variable identifier in standard form
! > str_identparse:        Parse variable identifier
! > strf_IsEqualCI:        Function: test if two strings are equal ignoring case
!
! MODULES:
!   None
!
! MODIFICATION HISTORY:
!   1992-03    (B.F. Kiel) Written
!   1998-02-04 (B.F.) Routines put into module
!   1998-02-10 (B.F.) Add routine str_lowercase
!   1998-03-13 (B.F.) Add routine str_pathandfile
!   1998-04-14 (B.F.) Further work
!   2013-08-24 (B.F. Lyon) Change layout of comments
!   2016-04-29 (B.F. Uppsala) Beautify
!   2016-04-30 (B.F.) Add routine str_uppercase
!   2016-05-02 (B.F.) Add function strf_IsEqualCI
!---------------------------------------------------------------------------------------------------
!
private str_str2int, str_int2str
!
contains

!----------********---------------------------------------------------------------------------------
subroutine str_form(stin,lsbstmx,substr, stout)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_form ('string_form')
!
! PURPOSE:
!   Remove all characters after the first blank of input string and substitute some substrings.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_form(stin,lsbstmx,substr, stout)
!
! INPUT:
!   stin:         ('string_input') character, input string
!   lsbstmx:      ('list_substitute_string_maximum') integer, number of strings to substitute
!   substr:       ('substitute_string') character array
!
! OUTPUT:
!   stout:        ('string_output') character, output string
!
! ROUTINES:
!   str_cut:               Cut input string into two parts at first blank
!   str_sub:               Substitute every occurrence of substring in string by another
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   This routine removes all characters after the first blank of stin and
!   substitutes substr(1,l) by substr(3,l) for all l=1..lsbstmx.
!   Suggestion: substr(1,l) should be of the form
!     '(XXX)'
!   as in
!     '(ABCD)',  '(disk)' oder  '(model)'
!   and must NOT contain any blanks.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1992-03    (B.F. Kiel) Written
!   1998-02-06 (B.F.) Fortran90 format
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: lsbstmx
character(len=*),          intent(in)   :: stin
character(len=*), dimension(3,lsbstmx), &
                           intent(in)   :: substr
character(len=*),          intent(out)  :: stout
!
! --- Local variables ---
integer                                 :: l
character*(80)                          :: sdum
!---------------------------------------------------------------------------------------------------
call str_cut(stin, stout,sdum)
!
do l=1,lsbstmx
  if (substr(3,l) /= '') call str_sub(stout,substr(1,l),substr(3,l), stout)
end do ! l
!
end subroutine str_form


!----------*******----------------------------------------------------------------------------------
subroutine str_cut(s, sfirst,slast, separator)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_cut ('string_cut')
!
! PURPOSE:
!   Cut input string into two parts at first blank.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_cut(s, sfirst,slast)
!
! INPUT:
!   s:            ('string') character, input string
!   separator:    ('string') character, optional, separator character (default: ' ')
!
! OUTPUT:
!   sfirst:       ('string_first') character, first part of input string
!   slast:        ('string_last') character, rest of input string
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   len(sfirst) >= len(s)
!
! PROCEDURE:
!   This routine cuts the string s into two parts, sfirst and slast.
!   If s is empty, then sfirst and slast are empty.
!   Otherwise, sfirst contains the first group of characters (without leading
!   blanks) up to the next blank. And slast contains the rest:
!     s =     ' ...'     // trim(sfirst) // ' '  // slast
!                ^               ^           ^        ^
!         leading blanks    first part     blank     rest
!
! EXAMPLE:
!   s0='Halli   Hallo'
!   call str_cut(s0, s1,s2)
!   write(6,'(A)') '('// trim(s0) //') => ('// trim(s1) //')|('// trim(s2) //')'
!     ==>
!   (Halli   Hallo) => (Halli)|(  Hallo)
!
! MODIFICATION HISTORY:
!   1992-03-27 (B.F. Kiel) 27-MAR-1992 14:33:55: Written
!   1998-02-05 (B.F.) Fortran90 format
!   1998-02-06 (B.F.) Revised "algorithm"
!   1998-02-10 (B.F.) Optional parameter separator introduced
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: s
character(len=1), optional,intent(in)   :: separator
character(len=*),          intent(out)  :: sfirst, slast
!
! --- Local variables ---
integer                                 :: i
character(len=1)                        :: separator0
!---------------------------------------------------------------------------------------------------
!
sfirst=adjustl(s)
!
if (present(separator)) then
  separator0=separator
else
  separator0=' '
endif
!
if (sfirst == '') then
  ! --- Only blanks ---
  slast =''
else
  i=index(sfirst,separator0)
  if (i == 0) then
    ! --- No separator found ---
    slast =''
  else
    ! --- Cut at separator position ---
    slast =sfirst(i+1:   )
    sfirst=sfirst(   :i-1)
  endif
endif
!
end subroutine str_cut


!----------*********--------------------------------------------------------------------------------
subroutine str_count(stin, stout,stf,step,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_count ('string_count')
!
! PURPOSE:
!   Increment string counter.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_count(stin, stout,stf,step=step,ierr=ierr)
!     allowed:
!   call str_count(st  , st   ,stf,step=step,ierr=ierr)
!
! INPUT:
!   stin:         ('string_input') character, input string with counter
!   step:         ('step') integer, optional, step size (+1 (default) or -1)
!
! OUTPUT:
!   stout:        ('string_output') character, next counter value or empty
!   stf:          ('string_file') character, file name derived from current counter value
!   ierr:         ('integer_error) integer, optional: 0,-1 => no error occurred
!                                                     > 0  => error
!
! ROUTINES:
!   str_str2int:           Transform a string into a positive integer.
!   str_int2str:           Transform a positive integer into a string.
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that the letters 'a..z' are in a strict sequence,
!   as well as 'A..Z' and '0..9'.
!
! PROCEDURE:
!   This routine is intended to abbreviate lists of file names.
!   If stin is a file name without counter, it is output in stf and
!   stout is empty.
!   If stin contains a string including a counter of the form
!     stin  = 'XXX (# startvalue - endvalue #) YYY'
!   as shown in the examples, then the output is
!     stout = 'XXX (# startvalue+1 - endvalue #) YYY'
!   if startvalue<endvalue and
!     stout = '',
!   otherwise and
!     stf   = 'XXX startvalue YYY'.
!   Also allowed are single valued counters of the form
!     stin  = 'XXX (# value #) YYY'
!   with the output
!     stout = ''
!     stf   = 'XXX value YYY'.
!   In addition, a list of counters (single valued or not) is possible, e.g.
!     stin  = 'XXX (# value1, startvalue2 - endvalue2, value3 #) YYY'
!   In this case the counters are calculated from left to right, with the
!   output of e.g.
!     stout = 'XXX (# startvalue2 - endvalue2, value3 #) YYY'
!     stf   = 'XXX value1 YYY'
!   As startvalue and endvalue are allowed:
!     small letter strings (a...z)
!   or
!     large letter strings (A...Z)
!   or
!     number with digits (0...9).
!   Determining the counter character range is the last (least significant)
!   character. If the other characters in the counters are not conformal,
!   they are transformed into the "zero" of the range, e.g.
!     (#5b-zz#)   ->   (#ab-zz#)   ->   (#ac-zz#)   
!   if stout is empty it means, that the file list is finished.
!   If stf is empty it means, that no file name could be generated.
!
! LOCAL VARIABLES:
!    test(#000001#).dat
!        ^       ^
!        i_open  i_to=i_close=i_comma
!
!    test(#000001-000010#).dat
!        ^       ^      ^
!        i_open  i_to   i_comma=i_close
!
!    test(#000001-000010,finalmodel#).dat
!        ^       ^      ^          ^
!        i_open  i_to   i_comma    i_close
!
! EXAMPLES:
!   stin:                 stout:              stf:             ierr
!   test_(#09-20#).dat    test_(#10-20#).dat  test_09.dat      -1
!   TEST_(#a-h#).txt      TEST_(#b-h#).txt    TEST_a.txt       -1
!   File.dat;(#01-30#)    file.dat;(#02-30#)  file.dat;01      -1
!   Modell_d1.f15         ''                  Modell_d1.f15     0
!   ''                    ''                  ''                0
!   test.(#strt,0-2,end#) test.(#0-2,end#)    test.strt         0
!   test.(#0-2,end#)      test.(#1-2,end#)    test.0           -1
!   test.(#1-2,end#)      test.(#2-2,end#)    test.1           -1
!   test.(#2-2,end#)      test.(#end#)        test.2            0
!   test.(#end#)          ''                  test.end          0
!   ''                    ''                  ''                0
!   test_(#08.dat         test_(#08.dat       ''                1
!   test_(#8-10#).dat     test_(#8-10#).dat   ''                2
!   test_(#+-*#).dat      test_(#+-*#).dat    ''                3
!   test_(#12-10#).dat    ''                  ''                5
!
! MODIFICATION HISTORY:
!   1992-04-04 (B.F. Kiel) 4-APR-1992 19:32:50: Written
!   1998-02-05 (B.F.) Fortran90 version,
!              no counter => stf=stin, stout='';
!              comma separated list of counters allowed;
!              sign of ierr reversed: ierr > 0 => error, ierr <= 0 => o.k.
!   1998-02-06 (B.F.) ierr made optional
!   1998-02-19 (B.F.) Optional parameter step added
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: stin
character(len=*),          intent(out)  :: stout, stf
integer,         optional, intent(in)   :: step
integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
character(len=80)                       :: scfirst, sclast
integer                                 :: inumin, inumax, ilsmin, ilsmax, &
                                           ilBmin, ilBmax
integer                                 :: i_open, i_to, i_comma, i_close, &
                                           ic, imin, imax, &
                                           nfirst, nlast, ie, ierr0, step0
!---------------------------------------------------------------------------------------------------
!
! --- The three possible "number" ranges ---
inumin=ichar('0')
inumax=ichar('9')
ilsmin=ichar('a')
ilsmax=ichar('z')
ilBmin=ichar('A')
ilBmax=ichar('Z')
!
! --- Default value for optional parameter ---
if (present(step)) then
  step0=step
else
  step0=1
endif
!
! --- Initialization ---
ierr0=0
stout=stin
stf  =''
!
! --- Check if all counter information is present ---
i_open=index(stin,'(#')
if (i_open == 0) then
  ! --- No counter information present (at least no counter opening bracket ) --
  ierr0=0
  stf  =stin
  stout=''
  goto 100
endif
!
i_close=index(stin(i_open+2:),'#)')
if (i_close == 0) then
  ! --- No counter closing bracket present ---
  ierr0=1
  goto 100
endif
i_close=i_close+i_open+1
!
i_comma=index(stin(i_open+2:), ',', back=(step0 < 0))
if (i_comma == 0) then
  ! --- No comma present ---
  if (step0 < 0) then
    i_comma=i_open+1
  else
    i_comma=i_close
  endif
else
  i_comma=i_comma+i_open+1
endif
!
if (step0 < 0) then
  ! --- Count backward ---
  i_to=index(stin(i_comma+1:i_close-1),'-')
  if (i_to == 0) then
    ! --- No minus sign present ---
    i_to=i_close
    ! --- Extract counter start and end value: both are the same ---
    scfirst=stin(i_comma+1:i_close-1)
    sclast =scfirst
    !
    !--- Output (file) name ---
    stf=stin(:i_open-1) // trim(scfirst) // stin(i_close+2:)
    !
    ! --- Output string with new counter information ---
    if (i_comma == i_open+1) then
      ! --- No other counter present ---
      stout=''
    else
      ! --- Other counter present ---
      stout=stin(:i_comma-1) // stin(i_close:)
    endif
  else
    i_to=i_to+i_comma
    ! --- Extract counter start and end value ---
    scfirst=stin(i_comma+1:i_to   -1)
    sclast =stin(i_to   +1:i_close-1)
    !
    if (len_trim(scfirst) /= len_trim(sclast)) then
      ! --- Length of counter start and end value strings are not equal ---
      ierr0=2
      goto 100
    endif
    !
    ! --- Last character of counter ---
    ic=ichar(scfirst(len_trim(scfirst):len_trim(scfirst)))
    !
    !--- Counter in small capitals, large capitals or numbers? ---
    if ((ic >= inumin) .and. (ic <= inumax)) then
      imin=inumin
      imax=inumax
    else if ((ic >= ilsmin) .and. (ic <= ilsmax)) then
      imin=ilsmin
      imax=ilsmax
    else if ((ic >= ilBmin) .and. (ic <= ilBmax)) then
      imin=ilBmin
      imax=ilBmax
    else
      ! --- Unknown counter range ---
      ierr0=3
      goto 100
    endif
    !
    ! --- Transform start value of counter into integer ---
    call str_str2int(scfirst,imin,imax, nfirst)
    ! --- Normalize start value of counter ---
    call str_int2str(nfirst,imin,imax,len_trim(scfirst), scfirst,ie)
    !
    !--- Output (file) name ---
    stf=stin(:i_open-1) // trim(sclast) // stin(i_close+2:)
    !
    ! --- Transform end value of counter into integer ---
    call str_str2int(sclast,imin,imax, nlast)
    !
    if (nfirst < nlast) then
      ! --- Decrement counter and transform into string ---
      call str_int2str(nlast-1,imin,imax,len_trim(scfirst),sclast,ie)
      if (ie /= 0) then
        ierr0=4
      else
        ierr0=-1
      endif
      ! --- Output string with new counter information ---
      stout=stin(:i_to) // trim(sclast) // stin(i_close:)
    else if (nfirst > nlast) then
      stf  =''
      stout=''
      ierr0=5
    else
      ! --- Output string with new counter information ---
      if (i_comma == i_open+1) then
        ! --- No other counter present ---
        stout=''
      else
        ! --- Other counter present ---
        stout=stin(:i_comma-1) // stin(i_close:)
      endif
    endif
  endif
else
  ! --- Count forward ---
  i_to=index(stin(i_open+2:i_comma-1),'-')
  if (i_to == 0) then
    ! --- No minus sign present ---
    i_to=i_comma
    ! --- Extract counter start and end value: both are the same ---
    scfirst=stin(i_open+2:i_comma-1)
    sclast =scfirst
    !
    !--- Output (file) name ---
    stf=stin(:i_open-1) // trim(scfirst) // stin(i_close+2:)
    !
    ! --- Output string with new counter information ---
    if (i_comma == i_close) then
      ! --- No other counter present ---
      stout=''
    else
      ! --- Other counter present ---
      stout=stin(:i_open+1) // stin(i_comma+1:)
    endif
  else
    i_to=i_to+i_open+1
    ! --- Extract counter start and end value ---
    scfirst=stin(i_open+2:i_to   -1)
    sclast =stin(i_to  +1:i_comma-1)
    !
    if (len_trim(scfirst) /= len_trim(sclast)) then
      ! --- Length of counter start and end value strings are not equal ---
      ierr0=2
      goto 100
    endif
    !
    ! --- Last character of counter ---
    ic=ichar(scfirst(len_trim(scfirst):len_trim(scfirst)))
    !
    !--- Counter in small capitals, large capitals or numbers? ---
    if ((ic >= inumin) .and. (ic <= inumax)) then
      imin=inumin
      imax=inumax
    else if ((ic >= ilsmin) .and. (ic <= ilsmax)) then
      imin=ilsmin
      imax=ilsmax
    else if ((ic >= ilBmin) .and. (ic <= ilBmax)) then
      imin=ilBmin
      imax=ilBmax
    else
      ! --- Unknown counter range ---
      ierr0=3
      goto 100
    endif
    !
    ! --- Transform start value of counter into integers ---
    call str_str2int(scfirst,imin,imax, nfirst)
    ! --- Normalize start value of counter ---
    call str_int2str(nfirst,imin,imax,len_trim(scfirst), scfirst,ie)
    !
    !--- Output (file) name ---
    stf=stin(:i_open-1) // trim(scfirst) // stin(i_close+2:)
    !
    ! --- Transform end value of counter into integers ---
    call str_str2int(sclast,imin,imax, nlast)
    !
    if (nfirst < nlast) then
      ! --- Increment counter and transform into string ---
      call str_int2str(nfirst+1,imin,imax,len_trim(scfirst),scfirst,ie)
      if (ie /= 0) then
        ierr0=4
      else
        ierr0=-1
      endif
      ! --- Output string with new counter information ---
      stout=stin(:i_open+1) // trim(scfirst) // stin(i_to:)
    else if (nfirst > nlast) then
      stf  =''
      stout=''
      ierr0=5
    else
      ! --- Output string with new counter information ---
      if (i_comma == i_close) then
        ! --- No other counter present ---
        stout=''
      else
        ! --- Other counter present ---
        stout=stin(:i_open+1) // stin(i_comma+1:)
      endif
    endif
  endif
endif
! 
100 continue
!
if (present(ierr)) ierr=ierr0
!
end subroutine str_count


!----------***********------------------------------------------------------------------------------
subroutine str_str2int(st,imin,imax, n)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_str2int ('string_string_to_integer')
!
! PURPOSE:
!   Transform a string into a positive integer.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_str2int(st,imin,imax, n)
!
! INPUT:
!   st:           ('string') character, input string
!   imin:         ('index_minimum') integer, lower index range
!   imax:         ('index_maximum') integer, upper index range
!
! OUTPUT:
!   n:            ('number') integer, output number
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   This routine transforms a string st into a positive integer n.
!   In imin and imax the possible 'digit' range has to be specified.
!   The reverse routine is
!     str_int2str.
!
! EXAMPLES:
!   call str_str2int('42',ichar('0'),ichar('9'), n)     => n=42
!   call str_str2int('ab',ichar('a'),ichar('z'), n)     => n=1
!   call str_str2int('BA',ichar('A'),ichar('Z'), n)     => n=26
!
! MODIFICATION HISTORY:
!   1992-03    (B.F. Kiel) Written
!   1998-02-06 (B.F.) Fortran90 format
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: st
integer,                   intent(in)   :: imin, imax
integer,                   intent(out)  :: n
!
! --- Local variables ---
integer                                 :: j, in
!---------------------------------------------------------------------------------------------------
!
n=0
do j=1,len_trim(st)
  in=ichar(st(j:j))
  if ((in < imin) .or. (in > imax)) then
    in=0
  else
    in=in-imin
  endif
  n=n*(imax-imin+1) + in
end do ! j
!
end subroutine str_str2int
	  

!----------***********------------------------------------------------------------------------------
subroutine str_int2str(n,imin,imax,nchar, str,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_int2str ('string_integer_to_string')
!
! PURPOSE:
!   Transform a positive integer into a string.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_int2str(n,imin,imax,nchar, str,ierr=ierr)
!
! INPUT:
!   n:            ('number') integer, input number
!   imin:         ('index_minimum') integer, lower index range
!   imax:         ('index_maximum') integer, upper index range
!   nchar:        ('number_character') integer, number of digits of output string
!
! OUTPUT:
!   str:          ('string') character, output string
!   ierr:         ('integer_error) integer, optional:  0 => no error occurred
!                                                     >0 => error
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   This routine transforms a positive integer n into a string str.
!   In imin and imax the possible 'digit' range has to be specified.
!   The reverse routine is
!     str_str2int.
!
! EXAMPLES:
!   call str_str2int(42,ichar('0'),ichar('9'),2, str)     => str='42'
!   call str_str2int(42,ichar('0'),ichar('9'),3, str)     => str='042'
!   call str_str2int( 1,ichar('a'),ichar('z'),2, str)     => str='ab'
!   call str_str2int(26,ichar('A'),ichar('Z'),2, str)     => str='BA'
!
! MODIFICATION HISTORY:
!   1992-03    (B.F. Kiel) Written
!   1998-02-06 (B.F.) Fortran90 format
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: n, imin, imax, nchar
character(len=*),          intent(out)  :: str
integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
integer                                 :: idel, m, mmod, j, ierr0
!---------------------------------------------------------------------------------------------------
! --- Empty output string ---
str=''
! --- Number range ---
idel=imax-imin+1
!
if (idel <= 1) then
  ! --- No reasonable output range ---
  ierr0=1
else
  ierr0=0
  m=abs(n)
  ! --- Transform integer ---
  do
    mmod=mod(m,idel)
    m   =int(m/idel)
    str =char(mmod+imin) // str
    if (m <= 0) exit
  end do
  !
  ! --- Add leading "zeros" ---
  str=repeat(char(imin),max(nchar-len_trim(str),0)) // str
endif
!
if (present(ierr)) ierr=ierr0
!
end subroutine str_int2str
		  

!----------*******----------------------------------------------------------------------------------
subroutine str_sub(stin,stsub1,stsub2, stout)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_sub ('string_substitute')
!
! PURPOSE:
!   Substitute every occurrence of substring in string by another.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_sub(stin,stsub1,stsub2, stout)
!
! INPUT:
!   stin:         ('string_input') character, input string
!   stsub1:       ('string_substitute_1') character
!   stsub2:       ('string_substitute_2') character
!
! OUTPUT:
!   stout:        ('string_output') character, output string
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   stsub1 must not be empty.
!
! PROCEDURE:
!   This routine substitutes every occurrence of substring stsub1 in stin by the string stsub2.
!   Output in stout.
!
! EXAMPLES:
!     stin   = 'TEST60_(*)_mean.dat'
!     stsub1 = '(*)'
!     stsub2 = 'Velocity'
!   call str_sub(stin,stsub1,stsub2, stout)
!   --->
!     stout  = 'TEST60_Velocity_mean.dat'
!
! MODIFICATION HISTORY:
!   1992-03    (B.F. Kiel) Written
!   1998-02-06 (B.F.) Fortran90 format
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: stin, stsub1, stsub2
character(len=*),          intent(out)  :: stout
!
! --- Local variables ---
integer                                 :: i, i0, idx1, idx2
!---------------------------------------------------------------------------------------------------
!
! --- Initialization ---
idx1=len_trim(stsub1)
idx2=len_trim(stsub2)
stout=stin
i0=1
!
! --- Loop over every occurrence of stsub1 in stout ---
do
  ! --- Position of substring ---
  i=index(stout(i0:),stsub1(:idx1))
  if (i /= 0) then
    ! --- Substring found => replace it ---
    stout=stout(1:i0+i-2) // stsub2(:idx2) // stout(i0+i+idx1-1:)
    i0=i0+i+idx2-idx1
  else
    ! --- No more substrings to replace ---
    exit
  endif
end do
!
end subroutine str_sub


!----------*************----------------------------------------------------------------------------
subroutine str_lowercase(stin, stout)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_lowercase ('string_lower_case')
!
! PURPOSE:
!   Transform all uppercase letters into lowercase form.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_lowercase(stin, stout)
!
! INPUT:
!   stin:         ('string_input') character, input string
!
! OUTPUT:
!   stout:        ('string_output') character, output string
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   s0='Hello. What''s going on?'
!   call str_lowercase(s0, s1)
!   write(6,'(A)') '('// trim(s0) //') => ('// trim(s1) //')'
!     ==>
!   (Hello. What's going on?) => (hello. what's going on?)
!
! MODIFICATION HISTORY:
!   1998-02-10 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: stin
character(len=*),          intent(out)  :: stout
!
! --- Local variables ---
integer                                 :: i, ic, delta
integer                                 :: ilsmin, ilsmax, ilBmin, ilBmax
!---------------------------------------------------------------------------------------------------
!
ilsmin=ichar('a')
ilsmax=ichar('z')
ilBmin=ichar('A')
ilBmax=ichar('Z')
delta=ilsmin-ilBmin
!
stout=stin
!
do i=1,len_trim(stout)
  ic=ichar(stout(i:i))
  if ((ic >= ilBmin) .and. (ic <= ilBmax)) stout(i:i)=char(ic+delta)
end do ! i
!
end subroutine str_lowercase


!----------*************----------------------------------------------------------------------------
subroutine str_uppercase(stin, stout)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_uppercase ('string_upper_case')
!
! PURPOSE:
!   Transform all lowercase letters into uppercase form.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_uppercase(stin, stout)
!
! INPUT:
!   stin:         ('string_input') character, input string
!
! OUTPUT:
!   stout:        ('string_output') character, output string
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   s0='Hello. What''s going on?'
!   call str_uppercase(s0, s1)
!   write(6,'(A)') '('// trim(s0) //') => ('// trim(s1) //')'
!     ==>
!   (Hello. What's going on?) => (HELLO. WHAT'S GOING ON?)
!
! MODIFICATION HISTORY:
!   2016-04-30 (B.F. Kiel) Derived from str_lowercase
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: stin
character(len=*),          intent(out)  :: stout
!
! --- Local variables ---
integer                                 :: i, ic, delta
integer                                 :: ilsmin, ilsmax, ilBmin, ilBmax
!---------------------------------------------------------------------------------------------------
!
ilsmin=ichar('a')
ilsmax=ichar('z')
ilBmin=ichar('A')
ilBmax=ichar('Z')
delta=ilsmin-ilBmin
!
stout=stin
!
do i=1,len_trim(stout)
  ic=ichar(stout(i:i))
  if ((ic >= ilsmin) .and. (ic <= ilsmax)) stout(i:i)=char(ic-delta)
end do ! i
!
end subroutine str_uppercase


!----------***************--------------------------------------------------------------------------
subroutine str_pathandfile(path,file, pathandfile)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_pathandfile ('string_path_and_file')
!
! PURPOSE:
!   Merge path name and file name.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_pathandfile(path,file, pathandfile)
!
! INPUT:
!   path:         ('path') character, path name
!   file:         ('file') character, file name
!
! OUTPUT:
!   pathandfile:  ('path_and_file') character, merged path and file name
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Append file name to path name. Include '/' if necessary.
!
! EXAMPLE:
!   ' '     + ' '        => ' '
!   '/'     + ' '        => ' '
!   'bak'   + ' '        => ' '
!   'bak/'  + ' '        => ' '
!   ' '     + 'file.dat' => 'file.dat'
!   'bak'   + 'file.dat' => 'bak/file.dat'
!   'bak/'  + 'file.dat' => 'bak/file.dat'
!   '[bak]' + 'file.dat' => '[bak]file.dat'
!
! MODIFICATION HISTORY:
!   1998-03-13 (B.F. Kiel) Written
!   2016-05-01 (B.F. Uppsala) Output empty string if file is empty
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: path, file
character(len=*),          intent(out)  :: pathandfile
!
! --- Local variables ---
character(len=1)                        :: right
!---------------------------------------------------------------------------------------------------
!
if (path == '') then
  pathandfile=file
else if (file == '') then
  pathandfile=''
else
  right=path(len_trim(path):len_trim(path))
  if ((right == '/') .or. (right == ']')) then
    pathandfile=trim(path) // file
  else
    pathandfile=trim(path) // '/' // file
  endif
endif
!
end subroutine str_pathandfile


!----------****************-------------------------------------------------------------------------
subroutine str_identcompose(quant, ident, &
                            iso, defiso, xmean, tmean, &
                            source, defsource, &
                            add1, add2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_identcompose ('string_ident_compose')
!
! PURPOSE:
!   Compose variable identifier from input strings.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_identcompose(quant, ident, &
!                         iso=iso, defiso=defiso, xmean=xmean, tmean=tmean, &
!                         source=source, defsource=defsource, &
!                         add1=add1, add2=add2)
!
! INPUT:
!   quant:        ('quantity') character, quantity name (e.g. 'p', 'rho')
!   iso:          ('iso') character, optional (e.g. 'z', 'tau', 'mass')
!   defiso:       ('default_iso') character, optional, default iso-type
!                 (e.g. 'z', 'tau', 'mass') if iso==defiso then iso is ignored
!   xmean:        ('x_mean') character, optional, horizontal averaging type
!                 (e.g. 'mean', 'min', 'max', 'rms')
!   tmean:        ('t_mean') character, optional, time averaging type
!                 (e.g. 'mean', 'min', 'max', 'rms')
!   source:       ('source') character, optional, source (e.g. 'f15', 'f06')
!   defsource:    ('default_source') character, optional, default source
!                 (e.g. 'f15', 'f06') if source==defsource then source is ignored
!   add1:         ('additional_1) character, optional, additional suffix
!   add2:         ('additional_2) character, optional, additional suffix
!
! OUTPUT:
!   ident:        ('identifier') character, composed identifier
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Be careful in the choice of add1 and add2.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-03-23 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: quant
character(len=*), optional,intent(in)   :: source, defsource, xmean, &
                                           tmean, iso, defiso, add1, add2
character(len=*),          intent(out)  :: ident
!
! --- Local variables ---
character(len=40)                       :: source0, defsource0, xmean0, &
                                           tmean0, iso0, defiso0, add10, add20, &
                                           suffix
!---------------------------------------------------------------------------------------------------
!
! --- Some defaults ---
if (present(source)) then
  source0=source
else
  source0=''
endif
if (present(defsource)) then
  defsource0=defsource
else
  defsource0=''
endif
if (present(xmean)) then
  xmean0=xmean
else
  xmean0=''
endif
if (present(tmean)) then
  tmean0=tmean
else
  tmean0=''
endif
if (present(iso)) then
  iso0=iso
else
  iso0=''
endif
if (present(defiso)) then
  defiso0=defiso
else
  defiso0=''
endif
if (present(add1)) then
  add10=add1
else
  add10=''
endif
if (present(add2)) then
  add20=add2
else
  add20=''
endif
!
! --- Ignore suffix if it is equal to default ---
if (source0 == defsource0) source0=''
if (iso0    == defiso0   ) iso0=''
!
! --- Construct suffix ---
suffix=''
if (iso0    /= '') suffix=trim(suffix) // '_i' // trim(iso0   )
if (xmean0  /= '') suffix=trim(suffix) // '_x' // trim(xmean0 )
if (tmean0  /= '') suffix=trim(suffix) // '_t' // trim(tmean0 )
if (source0 /= '') suffix=trim(suffix) // '_s' // trim(source0)
if (add10   /= '') suffix=trim(suffix) // '_'  // trim(add10  )
if (add20   /= '') suffix=trim(suffix) // '_'  // trim(add20  )
!
! --- Construct variable identifier ---
if (suffix /= '') then
  if (index(quant,'_') == 0) then
    ident=trim(quant) //        suffix 
  else
    ident=trim(quant) // '_' // suffix 
  endif
else
  ident=quant
endif
!
end subroutine str_identcompose


!----------**************---------------------------------------------------------------------------
subroutine str_identparse(ident, quant, &
                          iso, defiso, xmean, tmean, &
                          source, defsource, &
                          add1, add2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_identparse ('string_ident_parse')
!
! PURPOSE:
!   Parse variable identifier.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_identparse(ident, quant, &
!                       iso=iso, defiso=defiso, xmean=xmean, tmean=tmean, &
!                       source=source, defsource=defsource, &
!                       add1=add1, add2=add2)
!
! INPUT:
!   ident:        ('identifier') character, composed identifier
!   defiso:       ('default_iso') character, optional, default iso-type
!                 (e.g. 'z', 'tau', 'mass') if iso=='' then iso=defiso
!   defsource:    ('default_source') character, optional, default source
!                 (e.g. 'f15', 'f06') if source=='' then source=defsource
!
! OUTPUT:
!   quant:        ('quantity') character, quantity name (e.g. 'p', 'rho')
!   iso:          ('iso') character, optional (e.g. 'z', 'tau', 'mass')
!   xmean:        ('x_mean') character, optional, horizontal averaging type
!                 (e.g. 'mean', 'min', 'max', 'rms')
!   tmean:        ('t_mean') character, optional, time averaging type
!                 (e.g. 'mean', 'min', 'max', 'rms')
!   source:       ('source') character, optional, source (e.g. 'f15', 'f06')
!   add1:         ('additional_1) character, optional, additional suffix
!   add2:         ('additional_2) character, optional, additional suffix
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Be careful in the interpretation of add1 and add2.
!
! PROCEDURE:
!
! EXAMPLE:
!   ident ='rho_iz_xmean_trms_sf06_000'
!     =>
!   quant ='rho'
!   iso   ='z'
!   xmean ='mean'
!   tmean ='rms'
!   source='f06'
!   add1  ='000'
!   add2  =''
!
! MODIFICATION HISTORY:
!   1998-03-23 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: ident
character(len=*),          intent(out)  :: quant
character(len=*), optional,intent(in)   :: defsource, defiso
character(len=*), optional,intent(out)  :: source, xmean, &
                                           tmean, iso, add1, add2
!
! --- Local variables ---
character(len=80)                       :: ident0, suffix
character(len=40)                       :: source0, defsource0, xmean0, &
                                           tmean0, iso0, defiso0, add10, add20, &
                                           entry
character(len=2)                        :: entry_id
integer                                 :: isep
!---------------------------------------------------------------------------------------------------
!
! --- Some defaults ---
if (present(defsource)) then
  defsource0=defsource
else
  defsource0=''
endif
if (present(defiso)) then
  defiso0=defiso
else
  defiso0=''
endif
!
ident0=ident
!
! --- Find quantity name ---
isep=index(ident0,'__')
if (isep /= 0) then
  quant =ident0(:isep-1)
  ident0=ident0(isep+1:)
else
  isep=index(ident0,'_')
  if (isep /= 0) then
    quant =ident0(:isep-1)
    ident0=ident0(isep:)
  else
    quant=ident0
    ident0=''
  endif
endif
!
! --- Delete substring variables ---
iso0   =''
xmean0 =''
tmean0 =''
source0=''
add10  =''
add20  =''
!
do
  ! --- Loop over all substrings ---
  ! --- Exit condition ---
  if (ident0 == '') exit
  ! --- Find next substring ---
  entry_id=ident0(1:2)
  isep=index(ident0(3:),'_')
  if (isep == 0) then
    isep=len_trim(ident0)+1
  else
    isep=isep+2
  endif
  entry=ident0(3:isep-1)
  ! --- Rest of identifier ---
  ident0=ident0(isep:)
  ! --- Classify entry ---
  select case(entry_id)
    case('_i')
      iso0   =entry
    case('_x')
      xmean0 =entry
    case('_t')
      tmean0 =entry
    case('_s')
      source0=entry
    case default
      if (add10 == '') then
        add10 =entry_id(2:2) // entry
      else
        add20 =entry_id(2:2) // trim(entry) // ident0
        ident0=''
      endif
  end select ! entry_id
end do
!
! --- Empty suffix -> default ---
if (source0 == '') source0=defsource0
if (iso0    == '') iso0   =defiso0
!
! --- Put results into optional output parameters ---
if (present(iso   )) iso   =iso0
if (present(xmean )) xmean =xmean0
if (present(tmean )) tmean =tmean0
if (present(source)) source=source0
if (present(add1  )) add1  =add10
if (present(add2  )) add2  =add20
!
end subroutine str_identparse


!----------***************--------------------------------------------------------------------------
subroutine str_rightnumber(stin, stout, number)
!---------------------------------------------------------------------------------------------------
! NAME:
!   str_rightnumber ('string_right_number')
!
! PURPOSE:
!   Parse string into string and number.
!
! CATEGORY:
!   String Manipulation
!
! CALLING SEQUENCE:
!   call str_rightnumber(stin, stout, number)
!
! INPUT:
!   stin:         ('string_input') character, input string
!
! OUTPUT:
!   stout:        ('string_output') character, output string
!   number:       ('number') integer, output
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   Allowed call:
!     call str_rightnumber(st, st, number)
!     'stringnumber' -> 'string', number
!   e.g.
!     'mean4'        -> 'mean'  , 4
!
! MODIFICATION HISTORY:
!   1998-04-14 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: stin
character(len=*),          intent(out)  :: stout
integer,                   intent(out)  :: number
!
! --- Local variables ---
integer                                 :: i
character(len=40)                       :: stnum
!---------------------------------------------------------------------------------------------------
!
! --- Number range ---
!
i=len_trim(stin)
if (i > 0) then
  ! --- Input string is non-empty ---
  !
  ! --- Determine index of first digit of suffix number ---
  do
    if ((stin(i:i) < '0') .or. (stin(i:i) > '9')) then
      i=i+1
      exit
    endif
    if (i <= 1) exit
    i=i-1
  end do ! i
  !
  ! --- Parse input string into string with number (possibly empty) and rest ---
  stnum=stin(i:)
  stout=stin(1:i-1)
  !
  if (stnum == '') then
    ! --- No suffix number: output=-1 ---
    number=-1
  else
    read (stnum,*) number
  endif
else
  stout=stin
  number=-1
endif
!
end subroutine str_rightnumber


!--------**************-----------------------------------------------------------------------------
function strf_IsEqualCI(sin1, sin2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   strf_IsEqualCI ('string_function_Is_Equal_Case_Insensitive')
!
! PURPOSE:
!   Test if two strings are equal ignoring case.
!
! CATEGORY:
!   String Handling
!
! CALLING SEQUENCE:
!   Result=strf_IsEqualCI(sin1, sin2)
!
! INPUT:
!   sin1:         ('string_input_1') character, first input string
!   sin2:         ('string_input_2') character, second input string
!
! ROUTINES:
!   str_lowercase:         Transform all uppercase letters into lowercase form
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Transform both input strings to lowercase and compare.
!
! EXAMPLE:
!   write(6,*) strf_IsEqualCI('Hello', 'hello')
!
! MODIFICATION HISTORY:
!   2016-05-02 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character(len=*), intent(in)            :: sin1, sin2
logical                                 :: strf_IsEqualCI
!
! --- Local variables ---
character(len(sin1))                    :: slow1
character(len(sin2))                    :: slow2
!---------------------------------------------------------------------------------------------------
!
! --- Transform both input strings to lowercase ---
call str_lowercase(sin1, slow1)
call str_lowercase(sin2, slow2)
!
! --- Compare the lower case version of both strings ---
strf_IsEqualCI=(slow1 == slow2)
!
end function strf_IsEqualCI

end module str_module
