C------------************------------------------------------------------------
      module cubit_module
C------------------------------------------------------------------------------
C NAME:
C   cubit_module ('cubic_interpolation_module')
C
C PURPOSE:
C   Contains CUBIT routine for 1D cubic interpolation
C
C MODIFICATION HISTORY:
C    6. 2.92 cubit version written by Hans-Guenter Ludwig
C   20. 3.98 (Bernd Freytag) cubit put into module
C   26.11.00 (B.F. Uppsala) Separate subroutines for cubit and cubiti
C------------------------------------------------------------------------------
C
      contains
C
C----------------*****---------------------------------------------------------
      SUBROUTINE CUBIT( XX, YY, NXY, INTMOD, X, Y, I)
********************************************************************************
*
* Version: 6-MAR-1992 15:37:39 
*
* Authors: HGL, MS
*
* Modification: 26.11.00 (B.F.) Separate subroutines for cubit and cubiti
*                               ENTRY statements outcommented
*
* Description: CUBIT performs an 1D-interpolation using cubic polynomials.
*   The following scheme shows the meanings of the different variables:
*      
*     YY
*     ^
*     |
*     |
*     |                                                            +
*     |
*     |                        +                  +
*     |                             +
*   Y *--------------/.../-------------*    +
*     |        +                       |
*     |    +                           |
*     |                                |
*     |----|---|-----/.../-----|----|--*----|-----|-----/.../------|----> XX
*          1   2              I-1   I      I+1   I+2              NXY
*                                   |  X    |
*
*   NXY (NXY >= 3!) data points Y(X) are given by the arrays XX and YY. 
*   The XX-values must be ordered monotonically ( For all i: X(i+1)>X(i), 
*   or X(i+1) < X(i)). CUBIT works in the interval specified by i 
*   (1 <= i <= NXY-1). The function Y(X) described by XX and YY is
*   approximated in interval i by a third order polynomial of the form
*
*   Y_i(X) = A_i*(X-X_i)**3 + B_i*(X-X_i)**2 + C_i*(X-X_i) + D_i .
*
*   The coefficients are determined by the values Y_i and Y_(i+1) at the 
*   interval boundaries and the derivatives YS_i and YS_(i+1). These
*   derivatives are computed in various ways for different values of the
*   parameter INTMOD from the specified XX and YY values. Y gives the 
*   value of the interpolating polynomial at point X. 
*
*   X may lie outside the XX-range. But take care! In this case the polynomial 
*   is computed for the nearest boundary interval in XX. The extrapolated value 
*   may be quite different from what you expect because the interpolating 
*   polynamial may be non-monotonic outside the interval for which it was 
*   constructed. The same is true for entries CUBITI and EUBITI because here it 
*   is possible to specify the interval for which the polynomial is computed 
*   regardless what X-value is actually requested.
*
*   For every value of INTMOD the slope at the boundaries of the XX-range
*   is set to the slope of the secant through the nearest neighbour points.
*
*   There are eight more ENTRY points in this subroutine: 
*
*   ENTRY EUBIT( XX, INCX, YY, INCY, NXY, INTMOD, X, Y, I)
*   EUBIT is analogous to CUBIT but offers additionally the possiblity
*   to specify an increment for the array XX and YY so that not consecutive
*   elements can be regarded. This is especially useful if the data arrays
*   in the calling routine are two or more dimensinal and one wants to 
*   interpolate in the second (ore further) array index.
*
*   ENTRY CUBIB( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2, X, Y, I)
*   as CUBIT but with additional parameters to specify
*   boundary conditions at left (MBND1, SBND1) and right (MBND2, SBND2) edges
*   of data array
*
*   ENTRY cubiti( XX, YY, NXY, INTMOD, I, X, Y)
*   CUBITI is used when the interval index I is already known. In this case it 
*   has to be specified and is not determined or modified in this subroutine. 
*
*   ENTRY EUBITI( XX, INCX, YY, INCY, NXY, INTMOD, I, X, Y)
*   EUBITI is analogous to CUBITI but offers additionally the feature of array
*   increments.
*
*   ENTRY CUBIBI( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2, I, X, Y)
*   as CUBITI but with additional parameters to specify
*   boundary conditions at left (MBND1, SBND1) and right (MBND2, SBND2) edges
*   of data array
*
*   The remaining three entries can be used if a polynomial has been computed 
*   during a previous call of CUBIT, EUBIT, CUBITI or EUBITI. Since the
*   polynomial has been already computed these entries need no specification
*   of data points. Consequently there is only one version for each purpose
*   and not two like in the previous cases.
*
*   ENTRY CUBITP( X, Y)
*   CUBITP gives the value Y of the current polynomial at location X.
*
*   ENTRY CUBITS( X, YS)
*   CUBITS gives the derivative YS of the current polynomial at location X. 
*
*   ENTRY CUBITQ( XA, XB, YQ)
*   CUBITQ gives the integral of the current polynomial between XA and XB.
*
*
*   For a detailed treatment of the problem see
*   M. Steffen: "A simple method for monotonic interpolation in one dimension"
*                Astron. & Astrophys., vol. 239, pp. 443-450, 1990
*              
*   In this paper a somewhat more elaborated treatment of the slopes at the
*   interval boundaries is presented. This is not adopted here since it would
*   require some more if-then-else constructs.
*
*
* Arguments, access, type, dimension, description:  
*   NOTE: Not every described variable is used in the different entries. See
*   the description of the various entries which are in use.
*   XX     R R1: X-values of data points, ordered! 
*   INCX   R I : Array incrment of XX array, INCX<>0!
*   YY     R R1: Y-values of data points         
*   INCY   R I : Array incrment of YY array, INCY<>0!      
*   NXY    R I : Number of data points which will be regarded, NXY>=3!    
*   INTMOD R I : Mode of interpolation:
*                INTMOD = 1: Derivatives at the interval boundaries are given
*                            by the slope of the secant through the 
*                            neighbouring points.
*                INTMOD = 2: Derivatives at the interval boundaries are given
*                            by a the slope of a parabola through the boundary
*                            point and its neighbours. If the XX-values are
*                            equidistant INTMOD=1 and INTMOD=2 work identical.
*                INTMOD = 3: The derivatives of INTMOD = 2 are modified to
*                            assure monotonic behaviour of the interpolating
*                            polynomial in interval i.
*   MBND1  R I : Specifies boundray condition at leftmost  data piont (I=1)
*   SBND1  R R : Specifies slope at leftmost data piont (I=1)
*                in combination with MBND1 = 4, dummy otherwise
*   MBND2  R I : Specifies boundray condition at rightmost data piont (I=NXY)
*   SBND2  R R : Specifies slope at rightmost data piont (I=NXY)
*                in combination with MBND2 = 4, dummy otherwise
*                MBNDx = 0:  Periodic boundary conditions
*                            (for reasonable results YY(1) should be YY(NXY) )
*                MBNDx = 1:  Slope at boundaries is zero
*                MBNDx = 2:  Slope at boundaries is computed from one-sided
*                            differences
*                MBNDx = 3:  Slope at boundaries is computed such that
*                            second derivative is zero at boundaries
*                MBNDx = 4:  Slope at boundaries is taken as SBND1 (left)
*                            and SBND2 (left), respectively
*                
*   X      R R : X-value for which the interpolating polynomial is evaluated
*   Y      W R : Value of interpolating polynomial at point X
*   I      W I : Index of interval for which the polynomial was computed.
*                In CUBITI and EUBITI it has to be specified and is a readonly 
*                variable!
*   YS     W R : Derivative of the interpolating polynomial at point X.
*   XA     R R : Start values for which the integral of the polynomial is
*                computed.
*   XB     R R : End values for which the integral of the polynomial is
*                computed.
*   YQ     W R : Integral of the interpolating polynomial between XA and XB
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: In order to minimize the number of if-then-else blocks a kind of
*   spaghetti coding is adopted. So some piece of code may appear multiple
*   times. For the same reason this version is restricted to NXY >=3. 
*   Otherwise it would be necessary to regard separately the case NXY=2.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (R)eal, (S)tring, (*)label
*
********************************************************************************
      REAL XX(*), YY(*), X, XA, XB, Y, YS, YQ
      REAL FF(0:3,0:4)
      INTEGER I, INTMOD, NXY, INCX, INCY
      EQUIVALENCE ( CI, CSI, YSI), ( DI, YYI)
      common /cubit_common/ AI, BI, CI, DI, XXI
      save /cubit_common/
C      DATA AI , BI , CI , DI , XXI / 0.0, 0.0, 0.0, 0.0, 0.0 /
      DATA O2, O3, O4 / 0.5, 0.3333333333333333, 0.25 /
      DATA (FF(I,0),I=0,3) / 1.0, 0.0, 0.0, 0.0 /
      DATA (FF(I,1),I=0,3) / 0.0, 0.0, 0.0, 0.0 /
      DATA (FF(I,2),I=0,3) / 0.0, 1.0, 0.0, 0.0 /
      DATA (FF(I,3),I=0,3) / 0.0, 1.5,-0.5, 0.0 /
      DATA (FF(I,4),I=0,3) / 0.0, 0.0, 0.0, 1.0 /
***   Functions that transform user indices to linear indices and vice versa:
      ILINA( I, INC) = ( I - 1) * INC + 1
      IUSER( I, INC) = ( I - 1) / INC + 1

***   Determine interval index
      IF ( XX( 1) .LT. XX( NXY)) THEN
***     XX ordered low - up
        DO 10 I = NXY-1, 2, -1
          IF ( X .GE. XX( I)) GOTO 30
10      CONTINUE
***     If I is not 2 one has to use I=1 anyway. The DO-loop is left with
***     I=1 if the GOTO statement was not executed and the compiler works
***     really according the FORTRAN 77 standard! But to guarantee this in any
***     case ...
        I = 1
      ELSE
***     XX ordered up - low
        DO 20 I = NXY-1, 2, -1
          IF ( X .LE. XX( I)) GOTO 30
20      CONTINUE
***     See comment in THEN-block
        I = 1
      ENDIF

***   The interval index I is known.
C      ENTRY cubiti( XX, YY, NXY, INTMOD, I, X, Y)

***   Set auxiliary indices
30    IM1 = I - 1
      IP1 = I + 1
      IP2 = I + 2
***   Set parameters for all cases
      XXI   = XX( I)
      YYI   = YY( I)
      XXIP1 = XX( IP1)
      HI    = XXIP1 - XXI
      SI    = ( YY( IP1) - YYI) / HI

      IF ( I .EQ. 1) THEN
***     Interpolation in first interval
        HIM1 = HI
        SIM1 = SI
        HIP1 = XX( IP2) - XXIP1
        SIP1 = ( YY( IP2) - YY( IP1)) / HIP1
      ELSEIF ( I .EQ. NXY-1) THEN
***     Interpolation in last interval
        HIM1 = XXI - XX( IM1)
        SIM1 = ( YYI - YY( IM1)) / HIM1
        HIP1 = HI
        SIP1 = SI
      ELSE
***     Interpolation in inner interval
        HIM1 = XXI - XX( IM1)
        SIM1 = ( YYI - YY( IM1)) / HIM1
        HIP1 = XX( IP2) - XXIP1
        SIP1 = ( YY( IP2) - YY( IP1)) / HIP1
      ENDIF

40    IF ( INTMOD .EQ. 3) THEN
***     Monotonic interpolation
***
        YSI   = ( SIM1 * HI   + SI   * HIM1) / ( HIM1 + HI  )
        YSIP1 = ( SI   * HIP1 + SIP1 * HI  ) / ( HI   + HIP1)
***     So far the same like in the case INTMOD = 2, but now the magic
***     statements come ...
        YSI   = ( SIGN( 1.0, SIM1) + SIGN( 1.0, SI  )) *
     >          MIN( ABS( SIM1), ABS( SI  ), 0.5*ABS( YSI  ))
        YSIP1 = ( SIGN( 1.0, SI  ) + SIGN( 1.0, SIP1)) *
     >          MIN( ABS( SI  ), ABS( SIP1), 0.5*ABS( YSIP1))
***-----------------------------------------------------------------------------
      ELSEIF ( INTMOD .EQ. 2) THEN
***     Derivatives given by parabolas
***
        YSI   = ( SIM1 * HI   + SI   * HIM1) / ( HIM1 + HI  )
        YSIP1 = ( SI   * HIP1 + SIP1 * HI  ) / ( HI   + HIP1)
***-----------------------------------------------------------------------------
      ELSE
***     Derivatives given by slopes of secants through neighbouring points
***
        YSI   = ( SI * HI + SIM1 * HIM1) / ( HI + HIM1)
        YSIP1 = ( SI * HI + SIP1 * HIP1) / ( HI + HIP1)
***-----------------------------------------------------------------------------
      ENDIF

***   Compute interpolation coefficients for the polynomial:
   50 AI = ( YSI + YSIP1 - 2.0 * SI) / ( HI * HI)
      BI = ( 3.0 * SI - 2.0 * YSI - YSIP1) / HI
***   The assignments CI = YSI and DI = YYI are missing since they are done 
***   by an EQUIVALENCE statement.

***   Evaluate current polynomial at location X:
C      ENTRY CUBITP( X, Y)
      DX = X - XXI
      Y  = DX * ( DX * ( DX * AI + BI) + CI) + DI
      RETURN

***   Evaluate derivative of the current polynomial at location X:
C      ENTRY CUBITS( X, YS)
***   Compute interpolation coefficients for the derivative of the polynomial:
      ASI = 3.0 * AI
      BSI = BI + BI
***   The assignment CSI = CI is done by EQUIVALENCE.
      DX = X - XXI
      YS = DX * ( DX * ASI + BSI) + CSI
      RETURN

***   Integrate current polynomial between XA and XB:
C      ENTRY CUBITQ( XA, XB, YQ)
      DXA = XA - XXI
      DXB = XB - XXI
      YQ = DXB * ( DI + DXB * ( O2*CI + DXB * ( O3*BI + DXB * O4*AI)))
     >   - DXA * ( DI + DXA * ( O2*CI + DXA * ( O3*BI + DXA * O4*AI)))
      RETURN

***=============================================================================

C      ENTRY EUBIT( XX, INCX, YY, INCY, NXY, INTMOD, X, Y, I)
***   The remaining program section is analogous to the CUBIT/CUBITI part but
***   extended to include array increments.

***   Determine linear interval index IX in XX array in which X is situated
      NXYLX = ILINA( NXY, INCX)
      IF ( XX( 1) .LT. XX( NXYLX)) THEN
***     XX ordered low - up
        DO 100 IX = NXYLX-INCX, 1+INCX, -INCX
          IF ( X .GE. XX( IX)) GOTO 300
100     CONTINUE
        IX = 1
      ELSE
***     XX ordered up - low
        DO 200 IX = NXYLX-INCX, 1+INCX, -INCX
          IF ( X .LE. XX( IX)) GOTO 300
200     CONTINUE
        IX = 1
      ENDIF
***   Transform linear index to user index
300   I = IUSER( IX, INCX)
      GOTO 305

***   The user interval index I is known.
C      ENTRY EUBITI( XX, INCX, YY, INCY, NXY, INTMOD, I, X, Y)

***   Transform user index to linear indices
      IX = ILINA( I, INCX)
305   IY = ILINA( I, INCY)

***   Set further auxiliary indices
      IXM1 = IX - INCX
      IXP1 = IX + INCX
      IXP2 = IX + INCX + INCX
      IYM1 = IY - INCY
      IYP1 = IY + INCY
      IYP2 = IY + INCY + INCY
***   Set parameters for all cases
      XXI   = XX( IX)
      YYI   = YY( IY)
      XXIP1 = XX( IXP1)
      HI    = XXIP1 - XXI
      SI    = ( YY( IYP1) - YYI) / HI

      IF ( I .EQ. 1) THEN
***     Interpolation in first interval
        HIM1 = HI
        SIM1 = SI
        HIP1 = XX( IXP2) - XXIP1
        SIP1 = ( YY( IYP2) - YY( IYP1)) / HIP1
      ELSEIF ( I .EQ. NXY-1) THEN
***     Interpolation in last interval
        HIM1 = XXI - XX( IXM1)
        SIM1 = ( YYI - YY( IYM1)) / HIM1
        HIP1 = HI
        SIP1 = SI
      ELSE
***     Interpolation in inner interval
        HIM1 = XXI - XX( IXM1)
        SIM1 = ( YYI - YY( IYM1)) / HIM1
        HIP1 = XX( IXP2) - XXIP1
        SIP1 = ( YY( IYP2) - YY( IYP1)) / HIP1
      ENDIF

***   Until now the whole procedure is independend from XX and YY. So one
***   can switch back to main stream CUBIT ...
      GOTO 40
C
C      ENTRY CUBIB( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2,
C     >             X, Y, I)
***   Determine interval index
      IF ( XX( 1) .LT. XX( NXY)) THEN
***     XX ordered low - up
        DO 110 I = NXY-1, 2, -1
          IF ( X .GE. XX( I)) GOTO 130
110     CONTINUE
***     If I is not 2 one has to use I=1 anyway. The DO-loop is left with
***     I=1 if the GOTO statement was not executed and the compiler works
***     really according the FORTRAN 77 standard! But to guarantee this in any
***     case ...
        I = 1
      ELSE
***     XX ordered up - low
        DO 120 I = NXY-1, 2, -1
          IF ( X .LE. XX( I)) GOTO 130
120     CONTINUE
***     See comment in THEN-block
        I = 1
      ENDIF

***   The interval index I is known.
C      ENTRY CUBIBI( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2,
C     >              I, X, Y)

***   Set auxiliary indices
130   IM1 = I - 1
      IP1 = I + 1
      IP2 = I + 2
***   Set parameters for all cases
      XXI   = XX( I)
      YYI   = YY( I)
      XXIP1 = XX( IP1)
      HI    = XXIP1 - XXI
      SI    = ( YY( IP1) - YYI) / HI

      IF ( I .EQ. 1) THEN
***     Interpolation in first interval
        HIM1 =   XX(NXY) - XX(NXY-1)
        SIM1 = ( YY(NXY) - YY(NXY-1)) / HIM1
        HIP1 =   XX( IP2) - XXIP1
        SIP1 = ( YY( IP2) - YY( IP1)) / HIP1
        MB1  = MBND1
        MB2  = 0
      ELSEIF ( I .EQ. NXY-1) THEN
***     Interpolation in last interval
        HIM1 =   XXI - XX( IM1)
        SIM1 = ( YYI - YY( IM1)) / HIM1
        HIP1 =   XX(2) - XX(1)
        SIP1 = ( YY(2) - YY(1)) / HIP1
        MB1  = 0
        MB2  = MBND2
      ELSE
***     Interpolation in inner interval
        HIM1 = XXI - XX( IM1)
        SIM1 = ( YYI - YY( IM1)) / HIM1
        HIP1 = XX( IP2) - XXIP1
        SIP1 = ( YY( IP2) - YY( IP1)) / HIP1
        MB1  = 0
        MB2  = 0
      ENDIF

      IF ( INTMOD .EQ. 3) THEN
***     Monotonic interpolation
***
        YSI   = ( SIM1 * HI   + SI   * HIM1) / ( HIM1 + HI  )
        YSIP1 = ( SI   * HIP1 + SIP1 * HI  ) / ( HI   + HIP1)
***     So far the same like in the case INTMOD = 2, but now the magic
***     statements come ...
        YSI   = ( SIGN( 1.0, SIM1) + SIGN( 1.0, SI  )) *
     >          MIN( ABS( SIM1), ABS( SI  ), 0.5*ABS( YSI  ))
        YSIP1 = ( SIGN( 1.0, SI  ) + SIGN( 1.0, SIP1)) *
     >          MIN( ABS( SI  ), ABS( SIP1), 0.5*ABS( YSIP1))
***-----------------------------------------------------------------------------
      ELSEIF ( INTMOD .EQ. 2) THEN
***     Derivatives given by parabolas
***
        YSI   = ( SIM1 * HI   + SI   * HIM1) / ( HIM1 + HI  )
        YSIP1 = ( SI   * HIP1 + SIP1 * HI  ) / ( HI   + HIP1)
***-----------------------------------------------------------------------------
      ELSE
***     Derivatives given by slopes of secants through neighbouring points
***
        YSI   = ( SI * HI + SIM1 * HIM1) / ( HI + HIM1)
        YSIP1 = ( SI * HI + SIP1 * HIP1) / ( HI + HIP1)
***-----------------------------------------------------------------------------
      ENDIF
C
      YSI   = FF(0,MB1)*YSI   + FF(1,MB1)*SI + FF(2,MB1)*YSIP1 +
     >        FF(3,MB1)*SBND1
      YSIP1 = FF(0,MB2)*YSIP1 + FF(1,MB2)*SI + FF(2,MB2)*YSI   +
     >        FF(3,MB2)*SBND2
      GOTO 50
C
      END subroutine cubit


C----------------******--------------------------------------------------------
      SUBROUTINE cubiti( XX, YY, NXY, INTMOD, I, X, Y)
********************************************************************************
*
* Version: 6-MAR-1992 15:37:39 
*
* Authors: HGL, MS
*
* Modification: 26.11.00 (B.F.) Separate subroutines for cubit and cubiti
*                               ENTRY statements outcommented
*
* Description: CUBIT performs an 1D-interpolation using cubic polynomials.
*   The following scheme shows the meanings of the different variables:
*      
*     YY
*     ^
*     |
*     |
*     |                                                            +
*     |
*     |                        +                  +
*     |                             +
*   Y *--------------/.../-------------*    +
*     |        +                       |
*     |    +                           |
*     |                                |
*     |----|---|-----/.../-----|----|--*----|-----|-----/.../------|----> XX
*          1   2              I-1   I      I+1   I+2              NXY
*                                   |  X    |
*
*   NXY (NXY >= 3!) data points Y(X) are given by the arrays XX and YY. 
*   The XX-values must be ordered monotonically ( For all i: X(i+1)>X(i), 
*   or X(i+1) < X(i)). CUBIT works in the interval specified by i 
*   (1 <= i <= NXY-1). The function Y(X) described by XX and YY is
*   approximated in interval i by a third order polynomial of the form
*
*   Y_i(X) = A_i*(X-X_i)**3 + B_i*(X-X_i)**2 + C_i*(X-X_i) + D_i .
*
*   The coefficients are determined by the values Y_i and Y_(i+1) at the 
*   interval boundaries and the derivatives YS_i and YS_(i+1). These
*   derivatives are computed in various ways for different values of the
*   parameter INTMOD from the specified XX and YY values. Y gives the 
*   value of the interpolating polynomial at point X. 
*
*   X may lie outside the XX-range. But take care! In this case the polynomial 
*   is computed for the nearest boundary interval in XX. The extrapolated value 
*   may be quite different from what you expect because the interpolating 
*   polynamial may be non-monotonic outside the interval for which it was 
*   constructed. The same is true for entries CUBITI and EUBITI because here it 
*   is possible to specify the interval for which the polynomial is computed 
*   regardless what X-value is actually requested.
*
*   For every value of INTMOD the slope at the boundaries of the XX-range
*   is set to the slope of the secant through the nearest neighbour points.
*
*   There are eight more ENTRY points in this subroutine: 
*
*   ENTRY EUBIT( XX, INCX, YY, INCY, NXY, INTMOD, X, Y, I)
*   EUBIT is analogous to CUBIT but offers additionally the possiblity
*   to specify an increment for the array XX and YY so that not consecutive
*   elements can be regarded. This is especially useful if the data arrays
*   in the calling routine are two or more dimensinal and one wants to 
*   interpolate in the second (ore further) array index.
*
*   ENTRY CUBIB( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2, X, Y, I)
*   as CUBIT but with additional parameters to specify
*   boundary conditions at left (MBND1, SBND1) and right (MBND2, SBND2) edges
*   of data array
*
*   ENTRY CUBITI( XX, YY, NXY, INTMOD, I, X, Y)
*   CUBITI is used when the interval index I is already known. In this case it 
*   has to be specified and is not determined or modified in this subroutine. 
*
*   ENTRY EUBITI( XX, INCX, YY, INCY, NXY, INTMOD, I, X, Y)
*   EUBITI is analogous to CUBITI but offers additionally the feature of array
*   increments.
*
*   ENTRY CUBIBI( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2, I, X, Y)
*   as CUBITI but with additional parameters to specify
*   boundary conditions at left (MBND1, SBND1) and right (MBND2, SBND2) edges
*   of data array
*
*   The remaining three entries can be used if a polynomial has been computed 
*   during a previous call of CUBIT, EUBIT, CUBITI or EUBITI. Since the
*   polynomial has been already computed these entries need no specification
*   of data points. Consequently there is only one version for each purpose
*   and not two like in the previous cases.
*
*   ENTRY CUBITP( X, Y)
*   CUBITP gives the value Y of the current polynomial at location X.
*
*   ENTRY CUBITS( X, YS)
*   CUBITS gives the derivative YS of the current polynomial at location X. 
*
*   ENTRY CUBITQ( XA, XB, YQ)
*   CUBITQ gives the integral of the current polynomial between XA and XB.
*
*
*   For a detailed treatment of the problem see
*   M. Steffen: "A simple method for monotonic interpolation in one dimension"
*                Astron. & Astrophys., vol. 239, pp. 443-450, 1990
*              
*   In this paper a somewhat more elaborated treatment of the slopes at the
*   interval boundaries is presented. This is not adopted here since it would
*   require some more if-then-else constructs.
*
*
* Arguments, access, type, dimension, description:  
*   NOTE: Not every described variable is used in the different entries. See
*   the description of the various entries which are in use.
*   XX     R R1: X-values of data points, ordered! 
*   INCX   R I : Array incrment of XX array, INCX<>0!
*   YY     R R1: Y-values of data points         
*   INCY   R I : Array incrment of YY array, INCY<>0!      
*   NXY    R I : Number of data points which will be regarded, NXY>=3!    
*   INTMOD R I : Mode of interpolation:
*                INTMOD = 1: Derivatives at the interval boundaries are given
*                            by the slope of the secant through the 
*                            neighbouring points.
*                INTMOD = 2: Derivatives at the interval boundaries are given
*                            by a the slope of a parabola through the boundary
*                            point and its neighbours. If the XX-values are
*                            equidistant INTMOD=1 and INTMOD=2 work identical.
*                INTMOD = 3: The derivatives of INTMOD = 2 are modified to
*                            assure monotonic behaviour of the interpolating
*                            polynomial in interval i.
*   MBND1  R I : Specifies boundray condition at leftmost  data piont (I=1)
*   SBND1  R R : Specifies slope at leftmost data piont (I=1)
*                in combination with MBND1 = 4, dummy otherwise
*   MBND2  R I : Specifies boundray condition at rightmost data piont (I=NXY)
*   SBND2  R R : Specifies slope at rightmost data piont (I=NXY)
*                in combination with MBND2 = 4, dummy otherwise
*                MBNDx = 0:  Periodic boundary conditions
*                            (for reasonable results YY(1) should be YY(NXY) )
*                MBNDx = 1:  Slope at boundaries is zero
*                MBNDx = 2:  Slope at boundaries is computed from one-sided
*                            differences
*                MBNDx = 3:  Slope at boundaries is computed such that
*                            second derivative is zero at boundaries
*                MBNDx = 4:  Slope at boundaries is taken as SBND1 (left)
*                            and SBND2 (left), respectively
*                
*   X      R R : X-value for which the interpolating polynomial is evaluated
*   Y      W R : Value of interpolating polynomial at point X
*   I      W I : Index of interval for which the polynomial was computed.
*                In CUBITI and EUBITI it has to be specified and is a readonly 
*                variable!
*   YS     W R : Derivative of the interpolating polynomial at point X.
*   XA     R R : Start values for which the integral of the polynomial is
*                computed.
*   XB     R R : End values for which the integral of the polynomial is
*                computed.
*   YQ     W R : Integral of the interpolating polynomial between XA and XB
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: In order to minimize the number of if-then-else blocks a kind of
*   spaghetti coding is adopted. So some piece of code may appear multiple
*   times. For the same reason this version is restricted to NXY >=3. 
*   Otherwise it would be necessary to regard separately the case NXY=2.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (R)eal, (S)tring, (*)label
*
********************************************************************************
      REAL XX(*), YY(*), X, XA, XB, Y, YS, YQ
      REAL FF(0:3,0:4)
      INTEGER I, INTMOD, NXY, INCX, INCY
      EQUIVALENCE ( CI, CSI, YSI), ( DI, YYI)
      common /cubit_common/ AI, BI, CI, DI, XXI
      save /cubit_common/
C      DATA AI , BI , CI , DI , XXI / 0.0, 0.0, 0.0, 0.0, 0.0 /
      DATA O2, O3, O4 / 0.5, 0.3333333333333333, 0.25 /
      DATA (FF(I,0),I=0,3) / 1.0, 0.0, 0.0, 0.0 /
      DATA (FF(I,1),I=0,3) / 0.0, 0.0, 0.0, 0.0 /
      DATA (FF(I,2),I=0,3) / 0.0, 1.0, 0.0, 0.0 /
      DATA (FF(I,3),I=0,3) / 0.0, 1.5,-0.5, 0.0 /
      DATA (FF(I,4),I=0,3) / 0.0, 0.0, 0.0, 1.0 /
***   Functions that transform user indices to linear indices and vice versa:
      ILINA( I, INC) = ( I - 1) * INC + 1
      IUSER( I, INC) = ( I - 1) / INC + 1

***   The interval index I is known.
C      ENTRY CUBITI( XX, YY, NXY, INTMOD, I, X, Y)

***   Set auxiliary indices
30    IM1 = I - 1
      IP1 = I + 1
      IP2 = I + 2
***   Set parameters for all cases
      XXI   = XX( I)
      YYI   = YY( I)
      XXIP1 = XX( IP1)
      HI    = XXIP1 - XXI
      SI    = ( YY( IP1) - YYI) / HI

      IF ( I .EQ. 1) THEN
***     Interpolation in first interval
        HIM1 = HI
        SIM1 = SI
        HIP1 = XX( IP2) - XXIP1
        SIP1 = ( YY( IP2) - YY( IP1)) / HIP1
      ELSEIF ( I .EQ. NXY-1) THEN
***     Interpolation in last interval
        HIM1 = XXI - XX( IM1)
        SIM1 = ( YYI - YY( IM1)) / HIM1
        HIP1 = HI
        SIP1 = SI
      ELSE
***     Interpolation in inner interval
        HIM1 = XXI - XX( IM1)
        SIM1 = ( YYI - YY( IM1)) / HIM1
        HIP1 = XX( IP2) - XXIP1
        SIP1 = ( YY( IP2) - YY( IP1)) / HIP1
      ENDIF

40    IF ( INTMOD .EQ. 3) THEN
***     Monotonic interpolation
***
        YSI   = ( SIM1 * HI   + SI   * HIM1) / ( HIM1 + HI  )
        YSIP1 = ( SI   * HIP1 + SIP1 * HI  ) / ( HI   + HIP1)
***     So far the same like in the case INTMOD = 2, but now the magic
***     statements come ...
        YSI   = ( SIGN( 1.0, SIM1) + SIGN( 1.0, SI  )) *
     >          MIN( ABS( SIM1), ABS( SI  ), 0.5*ABS( YSI  ))
        YSIP1 = ( SIGN( 1.0, SI  ) + SIGN( 1.0, SIP1)) *
     >          MIN( ABS( SI  ), ABS( SIP1), 0.5*ABS( YSIP1))
***-----------------------------------------------------------------------------
      ELSEIF ( INTMOD .EQ. 2) THEN
***     Derivatives given by parabolas
***
        YSI   = ( SIM1 * HI   + SI   * HIM1) / ( HIM1 + HI  )
        YSIP1 = ( SI   * HIP1 + SIP1 * HI  ) / ( HI   + HIP1)
***-----------------------------------------------------------------------------
      ELSE
***     Derivatives given by slopes of secants through neighbouring points
***
        YSI   = ( SI * HI + SIM1 * HIM1) / ( HI + HIM1)
        YSIP1 = ( SI * HI + SIP1 * HIP1) / ( HI + HIP1)
***-----------------------------------------------------------------------------
      ENDIF

***   Compute interpolation coefficients for the polynomial:
   50 AI = ( YSI + YSIP1 - 2.0 * SI) / ( HI * HI)
      BI = ( 3.0 * SI - 2.0 * YSI - YSIP1) / HI
***   The assignments CI = YSI and DI = YYI are missing since they are done 
***   by an EQUIVALENCE statement.

***   Evaluate current polynomial at location X:
C      ENTRY CUBITP( X, Y)
      DX = X - XXI
      Y  = DX * ( DX * ( DX * AI + BI) + CI) + DI
C
      end subroutine cubiti


C----------------******--------------------------------------------------------
      SUBROUTINE CUBITQ(XA, XB, YQ)
********************************************************************************
*
* Version: 6-MAR-1992 15:37:39 
*
* Authors: HGL, MS
*
* Modification: 26.11.00 (B.F.) Separate subroutines for cubit and cubiti
*                               ENTRY statements outcommented
*                               Own routine for cubitq
*
* Description: CUBIT performs an 1D-interpolation using cubic polynomials.
*   The following scheme shows the meanings of the different variables:
*      
*     YY
*     ^
*     |
*     |
*     |                                                            +
*     |
*     |                        +                  +
*     |                             +
*   Y *--------------/.../-------------*    +
*     |        +                       |
*     |    +                           |
*     |                                |
*     |----|---|-----/.../-----|----|--*----|-----|-----/.../------|----> XX
*          1   2              I-1   I      I+1   I+2              NXY
*                                   |  X    |
*
*   NXY (NXY >= 3!) data points Y(X) are given by the arrays XX and YY. 
*   The XX-values must be ordered monotonically ( For all i: X(i+1)>X(i), 
*   or X(i+1) < X(i)). CUBIT works in the interval specified by i 
*   (1 <= i <= NXY-1). The function Y(X) described by XX and YY is
*   approximated in interval i by a third order polynomial of the form
*
*   Y_i(X) = A_i*(X-X_i)**3 + B_i*(X-X_i)**2 + C_i*(X-X_i) + D_i .
*
*   The coefficients are determined by the values Y_i and Y_(i+1) at the 
*   interval boundaries and the derivatives YS_i and YS_(i+1). These
*   derivatives are computed in various ways for different values of the
*   parameter INTMOD from the specified XX and YY values. Y gives the 
*   value of the interpolating polynomial at point X. 
*
*   X may lie outside the XX-range. But take care! In this case the polynomial 
*   is computed for the nearest boundary interval in XX. The extrapolated value 
*   may be quite different from what you expect because the interpolating 
*   polynamial may be non-monotonic outside the interval for which it was 
*   constructed. The same is true for entries CUBITI and EUBITI because here it 
*   is possible to specify the interval for which the polynomial is computed 
*   regardless what X-value is actually requested.
*
*   For every value of INTMOD the slope at the boundaries of the XX-range
*   is set to the slope of the secant through the nearest neighbour points.
*
*   There are eight more ENTRY points in this subroutine: 
*
*   ENTRY EUBIT( XX, INCX, YY, INCY, NXY, INTMOD, X, Y, I)
*   EUBIT is analogous to CUBIT but offers additionally the possiblity
*   to specify an increment for the array XX and YY so that not consecutive
*   elements can be regarded. This is especially useful if the data arrays
*   in the calling routine are two or more dimensinal and one wants to 
*   interpolate in the second (ore further) array index.
*
*   ENTRY CUBIB( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2, X, Y, I)
*   as CUBIT but with additional parameters to specify
*   boundary conditions at left (MBND1, SBND1) and right (MBND2, SBND2) edges
*   of data array
*
*   ENTRY CUBITI( XX, YY, NXY, INTMOD, I, X, Y)
*   CUBITI is used when the interval index I is already known. In this case it 
*   has to be specified and is not determined or modified in this subroutine. 
*
*   ENTRY EUBITI( XX, INCX, YY, INCY, NXY, INTMOD, I, X, Y)
*   EUBITI is analogous to CUBITI but offers additionally the feature of array
*   increments.
*
*   ENTRY CUBIBI( XX, YY, NXY, INTMOD, MBND1, MBND2, SBND1, SBND2, I, X, Y)
*   as CUBITI but with additional parameters to specify
*   boundary conditions at left (MBND1, SBND1) and right (MBND2, SBND2) edges
*   of data array
*
*   The remaining three entries can be used if a polynomial has been computed 
*   during a previous call of CUBIT, EUBIT, CUBITI or EUBITI. Since the
*   polynomial has been already computed these entries need no specification
*   of data points. Consequently there is only one version for each purpose
*   and not two like in the previous cases.
*
*   ENTRY CUBITP( X, Y)
*   CUBITP gives the value Y of the current polynomial at location X.
*
*   ENTRY CUBITS( X, YS)
*   CUBITS gives the derivative YS of the current polynomial at location X. 
*
*   ENTRY CUBITQ( XA, XB, YQ)
*   CUBITQ gives the integral of the current polynomial between XA and XB.
*
*
*   For a detailed treatment of the problem see
*   M. Steffen: "A simple method for monotonic interpolation in one dimension"
*                Astron. & Astrophys., vol. 239, pp. 443-450, 1990
*              
*   In this paper a somewhat more elaborated treatment of the slopes at the
*   interval boundaries is presented. This is not adopted here since it would
*   require some more if-then-else constructs.
*
*
* Arguments, access, type, dimension, description:  
*   NOTE: Not every described variable is used in the different entries. See
*   the description of the various entries which are in use.
*   XX     R R1: X-values of data points, ordered! 
*   INCX   R I : Array incrment of XX array, INCX<>0!
*   YY     R R1: Y-values of data points         
*   INCY   R I : Array incrment of YY array, INCY<>0!      
*   NXY    R I : Number of data points which will be regarded, NXY>=3!    
*   INTMOD R I : Mode of interpolation:
*                INTMOD = 1: Derivatives at the interval boundaries are given
*                            by the slope of the secant through the 
*                            neighbouring points.
*                INTMOD = 2: Derivatives at the interval boundaries are given
*                            by a the slope of a parabola through the boundary
*                            point and its neighbours. If the XX-values are
*                            equidistant INTMOD=1 and INTMOD=2 work identical.
*                INTMOD = 3: The derivatives of INTMOD = 2 are modified to
*                            assure monotonic behaviour of the interpolating
*                            polynomial in interval i.
*   MBND1  R I : Specifies boundray condition at leftmost  data piont (I=1)
*   SBND1  R R : Specifies slope at leftmost data piont (I=1)
*                in combination with MBND1 = 4, dummy otherwise
*   MBND2  R I : Specifies boundray condition at rightmost data piont (I=NXY)
*   SBND2  R R : Specifies slope at rightmost data piont (I=NXY)
*                in combination with MBND2 = 4, dummy otherwise
*                MBNDx = 0:  Periodic boundary conditions
*                            (for reasonable results YY(1) should be YY(NXY) )
*                MBNDx = 1:  Slope at boundaries is zero
*                MBNDx = 2:  Slope at boundaries is computed from one-sided
*                            differences
*                MBNDx = 3:  Slope at boundaries is computed such that
*                            second derivative is zero at boundaries
*                MBNDx = 4:  Slope at boundaries is taken as SBND1 (left)
*                            and SBND2 (left), respectively
*                
*   X      R R : X-value for which the interpolating polynomial is evaluated
*   Y      W R : Value of interpolating polynomial at point X
*   I      W I : Index of interval for which the polynomial was computed.
*                In CUBITI and EUBITI it has to be specified and is a readonly 
*                variable!
*   YS     W R : Derivative of the interpolating polynomial at point X.
*   XA     R R : Start values for which the integral of the polynomial is
*                computed.
*   XB     R R : End values for which the integral of the polynomial is
*                computed.
*   YQ     W R : Integral of the interpolating polynomial between XA and XB
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: In order to minimize the number of if-then-else blocks a kind of
*   spaghetti coding is adopted. So some piece of code may appear multiple
*   times. For the same reason this version is restricted to NXY >=3. 
*   Otherwise it would be necessary to regard separately the case NXY=2.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (R)eal, (S)tring, (*)label
*
********************************************************************************
      REAL X, XA, XB, Y, YS, YQ
      REAL FF(0:3,0:4)
      INTEGER I, INTMOD, NXY, INCX, INCY
      EQUIVALENCE ( CI, CSI, YSI), ( DI, YYI)
      common /cubit_common/ AI, BI, CI, DI, XXI
      save /cubit_common/
C      DATA AI , BI , CI , DI , XXI / 0.0, 0.0, 0.0, 0.0, 0.0 /
      DATA O2, O3, O4 / 0.5, 0.3333333333333333, 0.25 /
      DATA (FF(I,0),I=0,3) / 1.0, 0.0, 0.0, 0.0 /
      DATA (FF(I,1),I=0,3) / 0.0, 0.0, 0.0, 0.0 /
      DATA (FF(I,2),I=0,3) / 0.0, 1.0, 0.0, 0.0 /
      DATA (FF(I,3),I=0,3) / 0.0, 1.5,-0.5, 0.0 /
      DATA (FF(I,4),I=0,3) / 0.0, 0.0, 0.0, 1.0 /
***   Functions that transform user indices to linear indices and vice versa:
      ILINA( I, INC) = ( I - 1) * INC + 1
      IUSER( I, INC) = ( I - 1) / INC + 1

***   Integrate current polynomial between XA and XB:
C      ENTRY CUBITQ( XA, XB, YQ)
      DXA = XA - XXI
      DXB = XB - XXI
      YQ = DXB * ( DI + DXB * ( O2*CI + DXB * ( O3*BI + DXB * O4*AI)))
     >   - DXA * ( DI + DXA * ( O2*CI + DXA * ( O3*BI + DXA * O4*AI)))
      END subroutine cubitq

      end module cubit_module
