!***************************************************************************************************
!
!    ####  #####  #####   ##         #####  #    #  ####  ##### 
!   #    # #    #   #    #  #        #    # #    # #        #   
!   #    # #    #   #   #    #       #    # #    #  ####    #   
!   #    # #####    #   ######       #    # #    #      #   #   
!   #    # #        #   #    #       #    # #    # #    #   #   
!    ####  #        #   #    # ##### #####   ####   ####    #   
!
!   Dust opacity routines
!
!***************************************************************************************************
!   Bernd Freytag:        Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Susanne Hoefner:      Uppsala
!   Matthias Steffen:     Kiel, Potsdam
!   2017-07-10
!***************************************************************************************************
! MODULES:
!   op_dust_module:        Dust opacity routines
!
!***************************************************************************************************

!------**************-------------------------------------------------------------------------------
module op_dust_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   op_dust_module ('opacity_dust_module')
!
! PURPOSE:
!   Contains dust opacity routines.
!
! CATEGORY:
!   Radiation transport, opacities
!
! CALLING SEQUENCE:
!   use op_dust_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   kappadust:             Compute Rosseland dust opacity for amorph. carbon in Small-Particle Limit
!   op_dust1:              Compute dust opacity for various dust models for one vector (old version)
!   op_dust2:              Compute dust opacity for various dust models for one vector
!   op_dust:               Compute dust opacity for various dust models for 3D arrays with OpenMP
!
! MODIFICATION HISTORY:
!   2013-02-21 (B.F. Lyon) Change layout, more comments; merged with kappadust_mdule
!   2013-03-12 (B.F.) Create routine op_dust1 from op_dust
!   2013-08-26 (B.F.) Create routine op_dust2 from op_dust1
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------*********--------------------------------------------------------------------------------
subroutine kappadust(n1, TRAD, DK3, OPADUS)
!---------------------------------------------------------------------------------------------------
! NAME:
!   kappadust ('kappa_dust')
!
! PURPOSE:
!   Compute Rosseland dust opacity in Small Particle Limit (SPL) for N-dim vector
!
! CATEGORY:
!   Opacities
!
! CALLING SEQUENCE:
!   call kappadust(n1, TRAD, DK3, R0D, OPADUS)
!
! INPUT:
!   n1:     ('number_1') integer with total number of (1D) grid points
!   TRAD:   ('Temperature_RADiation') vector with radiation temperature
!   DK3     ('D_K_3') vector with 3rd dust moment: K3
!
! OUTPUT:
!   OPADUS: ('OPAcity_DUSt') vector with dust opacity (SPL) in 1/cm
!
! LOCAL VARIABLES
!   PI   =3.1415926535898  : pi
!   PMASS=1.6726E-24       : proton mass
!   AGW  =12.01115         : atomic weight of dust forming material (carbon)
!   RHOD =2.25             : density of condensed material (amorphous carbon)
!   R0D3                   : 3rd power of monomer radius, R0D3=R0D**3
!
! ROUTINES:
!   None
!
! COMMENTS:
!   TXKAPD:  Identifikationstext 'KAPPAD_SPL   : Staubkappa im SPL, Q/a nach JMW94PC '
!
! MODIFICATION HISTORY:
!   1992-08-31 (S.H.) Written, original file: KAPPAD_SPL44.FOR
!   1994-06-30 (S.H.) Wert fuer frequenzgemitteltes Q'(= Q/a) nach Winters (1994, priv.comm.)
!   2004-06-04 (B.F.) Put into module, Fortran90fied,
!              output of derivatives removed, output of TXKAPD removed,
!              Input parameter: SENE -> TRAD,
!              INIT removed, R0D3 instead of R0D
!   2008-06-17 (B.F.) Put into module again
!   2013-02-21 (B.F. Lyon) Remove argument RHO; rename N -> n1;
!              change output unit (remove division by rho): [cm^2/g] -> [cm^-1]
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: rk=kind(0.0E+00)
integer,                   intent(in)   :: n1
real (kind=rk),            intent(in)   :: TRAD(n1), DK3(n1)
real (kind=rk),            intent(out)  :: OPADUS(n1)
!
! --- Local variables ---
integer                                 :: m1, i1
real (kind=rk)                          :: QEXTR, XKAPR
real (kind=rk), parameter               :: PI   = 3.1415926535898_rk, &
                                           PMASS=1.6726E-24_rk, &
                                           AGW  =12.01115_rk, &
                                           RHOD =2.25_rk, &
                                           R0D3 = 3.0_rk * AGW * PMASS / (4.0_rk * PI * RHOD), &
                                           H3V04= PI * R0D3
!                                           PIST = PI / 5.66961E-05_rk
!---------------------------------------------------------------------------------------------------
!
m1=1
!
! === Compute dust kappa (Rosseland mean) ===
!
do i1=m1,n1
  ! --- Q' (Rosseland mean) according to Winters (1994, priv. comm.) ---
  QEXTR     =4.4_rk * TRAD(i1)
  !
  ! --- Dust kappa (Rossel mean) [1/cm] ---
  XKAPR     =H3V04 * max(DK3(i1), 0.0) * QEXTR
  OPADUS(i1)=XKAPR
end do ! i1
!
end subroutine kappadust


!----------********---------------------------------------------------------------------------------
subroutine op_dust2(T, rho, quc, &
                    nu, m1,n1, m2,n2, m3,n3, nquc, i2,i3, opclam)
!---------------------------------------------------------------------------------------------------
! NAME:
!   op_dust2 ('opacity_dust_2')
!
! PURPOSE:
!   Compute dust opacity for various dust models.
!   
! CATEGORY:
!   Radiative transfer, opacities
!
! CALLING SEQUENCE:
!   call op_dust2(T, rho, quc, &
!                 nu, m1,n1, m2,n2, m3,n3, nquc, i2,i3, opclam)
!
! INPUT:
!   T:           ('Temperature') real 3D array with temperature
!   rho:         ('rho') real 3D array with density, attention: one buffer cell in all directions
!   quc:         ('quantity_center') real 4D array with list of centered variables
!   nu:          ('nu') integer with frequency index
!   m1:          ('m_number_1') integer, 1. dimension, start index
!   n1:          ('number_1')   integer, 1. dimension, end index
!   m2:          ('m_number_2') integer, 2. dimension, start index
!   n2:          ('number_2')   integer, 2. dimension, end index
!   m3:          ('m_number_3') integer, 3. dimension, start index
!   n3:          ('number_3')   integer, 3. dimension, end index
!   nquc:        ('number_quantities_center') integer, number of additional density arrays
!
! OUTPUT:
!   opclam:      ('opacity_c_lambda') real 3D array with opacities
!
! RESTRICTIONS:
!
! MODIFICATION HISTORY:
!   2013-02-21 (B.F. Lyon) Change layout, add some more comments
!   2013-03-12 (B.F.) Remove argument Plog; derived from op_dust
!   2013-08-26 (B.F.) Derived from op_dust1, changed array declarations because of
!              performance problems, added SIMD directives
!   2014-11-05 (B.F. Uppsala) quc array gets buffer cells
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nu, m1,n1, m2,n2, m3,n3, nquc, i2,i3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(in)   :: T, rho
!!$real, dimension(m1  :n1  , &
!!$                m2  :n2  , &
!!$                m3  :n3  , &
!!$                1   :nquc   ), &
!!$                           intent(in)   :: quc
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1, &
                1   :nquc   ), &
                           intent(in)   :: quc
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(inout):: opclam
!
! --- Local variables ---
!
real, parameter                         :: PI=3.1415926536, SIGMA=5.6705E-5, C5=SIGMA/PI
integer                                 :: i1, iquc
real, dimension(m1-1:n1+1)              :: addata
!---------------------------------------------------------------------------------------------------
!
! write(6,'(A)') 'prop%quc_prop=' // trim(prop%quc_prop) // &
!                '  par%dustopamode=' // par%dustopamode
!
if (prop%quc_prop == 'dust_bins_01') then ! ========================================================
  ! --- Grey dust opacities from added dual-slope fits ---
  !
  ! --- (Grey dust opacities from added geometrical cross section) ---
  !
  ! --- Reset addata array  ---
  addata=0.0
  !
  ! --- Add bin opacities (use constant mass density per monomer) ---
      do iquc=2,prop%nquc
        !DIR$ SIMD
        do i1=m1-1,n1+1
          addata(i1)=addata(i1) + &
                           1.5/(par%C_dust01*par%ar_dustGrainRadius(iquc)) * &
                            min(par%ar_dustGrainRadius(iquc)*1.0E+04, 1.0) * &
                           min(max( par%ar_dustGrainRadius(iquc)/6.0E-03, &
                                   (par%ar_dustGrainRadius(iquc)/6.0E-05)**3), 1.0) * &
                           max(quc(i1,i2,i3,iquc), 0.0)
        end do ! i1
      end do ! iquc
  !
else if (prop%quc_prop == 'dust_k3mon_03') then ! ==================================================
  ! --- Dust opacity [1/cm] (bin independent) ---
  !
  if (par%dustopamode == 'largeparticlelimit') then
    ! --- Large particle limit ---
    ! --- Opacity of single grain is 2*pi*r_grain^2 (2*geometrical cross section) ---
    !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          addata(i1)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             ( quc(i1,i2,i3,2)**2 * &
                              (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
        end do ! i1
    !
  else if (par%dustopamode == 'dualslope01') then
    ! --- Dual-slope method ---
    !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Particle radius ---
          addata(i1)=par%ar_dustGrainRadius(2) * &
                           ( quc(i1,i2,i3,2)         / &
                            (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
          ! --- Limiter function (fit by eye to Rosseland means of Mie opacities ---
          ! ---                   of forsterite; fit parameters in [cm])         ---
          addata(i1)=min(max( addata(i1)/6.0E-03, &
                                   (addata(i1)/6.0E-05)**3), 1.0)
          ! --- Opacity ---
          addata(i1)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             addata(i1) * &
                             ( quc(i1,i2,i3,2)**2 * &
                              (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
        end do ! i1
    !
  else if (par%dustopamode == '') then
    ! --- Old default (before 2008-10-31) ---
    !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          addata(i1)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             min(par%ar_dustGrainRadius(2)*1.0E+04, 1.0) * &
                             ( quc(i1,i2,i3,2)**2 * &
                              (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
        end do ! i1
    !
  !else ! (par%dustopamode)
  !  stop 'Unknown dustopamode'
  else ! (par%dustopamode)
    ! --- No additional opacities ---
    !
    addata=0.0
    !
  endif ! (par%dustopamode)
  !
else if ((prop%quc_prop == 'none') .or. (prop%quc_prop == 'None')) then ! ==========================
  ! --- No additional opacities ---
  !
  addata=0.0
  !
else ! (prop%quc_prop) =============================================================================
  ! --- dust opacity [1/cm] (bin independent) ---
  !
  !DIR$ SIMD
  do i1=m1-1,n1+1
    addata(i1) = (quc(i1,i2,i3,1)*par%C_dust02)/ &
                     (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)+rho(i1,i2,i3))
    addata(i1) = par%C_dust05*addata(i1)**(2.0/3.0) * & 
                     ((quc(i1,i2,i3,1)+quc(i1,i2,i3,2))/par%C_dust02)
  end do ! i1
  !
!!$      do i1=1,1
!!$        write(6,'(A,3I4,6(1X, 1PE11.4))') 'i123,quc1,quc2,dustopac=', &
!!$          i1, quc(i1,i2,i3,1), quc(i1,i2,i3,2), addata(i1)
!!$      end do ! i1
endif ! (prop%quc_prop) ============================================================================
!
!DIR$ SIMD
do i1=m1-1,n1+1
  opclam(i1,i2,i3) = opclam(i1,i2,i3) + addata(i1)
end do ! i1
!
end subroutine op_dust2


!----------********---------------------------------------------------------------------------------
subroutine op_dust1(T, rho, quc, &
                    nu, m1,n1, nquc, opclam)
!---------------------------------------------------------------------------------------------------
! NAME:
!   op_dust1 ('opacity_dust_1')
!
! PURPOSE:
!   Compute dust opacity for various dust models.
!   
! CATEGORY:
!   Radiative transfer, opacities
!
! CALLING SEQUENCE:
!   call op_dust1(T, rho, quc, &
!                 nu, m1,n1, nquc, opclam)
!
! INPUT:
!
! OUTPUT:
!
! RESTRICTIONS:
!
! MODIFICATION HISTORY:
!   2013-02-21 (B.F. Lyon) Change layout, add some more comments
!   2013-03-12 (B.F.) Remove argument Plog; derived from op_dust
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nu, m1,n1, nquc
real, dimension(m1:n1), &
                           intent(in)   :: T, rho
real, dimension(m1:n1,1:nquc), &
                           intent(in)   :: quc
real, dimension(m1:n1), &
                           intent(inout):: opclam
!
! --- Local variables ---
!
real, parameter                         :: PI=3.1415926536, SIGMA=5.6705E-5, C5=SIGMA/PI
integer                                 :: i1, iquc
real, dimension(m1:n1)                  :: addata
!---------------------------------------------------------------------------------------------------
!
! write(6,'(A)') 'prop%quc_prop=' // trim(prop%quc_prop) // &
!                '  par%dustopamode=' // par%dustopamode
!
if (prop%quc_prop == 'dust_bins_01') then ! ========================================================
  ! --- Grey dust opacities from added dual-slope fits ---
  !
  ! --- (Grey dust opacities from added geometrical cross section) ---
  !
  ! --- Reset addata array  ---
  addata=0.0
  !
  ! --- Add bin opacities (use constant mass density per monomer) ---
      do iquc=2,prop%nquc
        !DIR$ SIMD
        do i1=m1,n1
          addata(i1)=addata(i1) + &
                           1.5/(par%C_dust01*par%ar_dustGrainRadius(iquc)) * &
                            min(par%ar_dustGrainRadius(iquc)*1.0E+04, 1.0) * &
                           min(max( par%ar_dustGrainRadius(iquc)/6.0E-03, &
                                   (par%ar_dustGrainRadius(iquc)/6.0E-05)**3), 1.0) * &
                           max(quc(i1,iquc), 0.0)
        end do ! i1
      end do ! iquc
  !
else if (prop%quc_prop == 'dust_k3mon_03') then ! ==================================================
  ! --- Dust opacity [1/cm] (bin independent) ---
  !
  if (par%dustopamode == 'largeparticlelimit') then
    ! --- Large particle limit ---
    ! --- Opacity of single grain is 2*pi*r_grain^2 (2*geometrical cross section) ---
    !
        !DIR$ SIMD
        do i1=m1,n1
          addata(i1)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             ( quc(i1,2)**2 * &
                              (quc(i1,1)+quc(i1,2)) )**(1.0/3.0)
        end do ! i1
    !
  else if (par%dustopamode == 'dualslope01') then
    ! --- Dual-slope method ---
    !
        !DIR$ SIMD
        do i1=m1,n1
          ! --- Particle radius ---
          addata(i1)=par%ar_dustGrainRadius(2) * &
                           ( quc(i1,2)         / &
                            (quc(i1,1)+quc(i1,2)) )**(1.0/3.0)
          ! --- Limiter function (fit by eye to Rosseland means of Mie opacities ---
          ! ---                   of forsterite; fit parameters in [cm])         ---
          addata(i1)=min(max( addata(i1)/6.0E-03, &
                                   (addata(i1)/6.0E-05)**3), 1.0)
          ! --- Opacity ---
          addata(i1)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             addata(i1) * &
                             ( quc(i1,2)**2 * &
                              (quc(i1,1)+quc(i1,2)) )**(1.0/3.0)
        end do ! i1
    !
  else if (par%dustopamode == '') then
    ! --- Old default (before 2008-10-31) ---
    !
        !DIR$ SIMD
        do i1=m1,n1
          addata(i1)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             min(par%ar_dustGrainRadius(2)*1.0E+04, 1.0) * &
                             ( quc(i1,2)**2 * &
                              (quc(i1,1)+quc(i1,2)) )**(1.0/3.0)
        end do ! i1
    !
  !else ! (par%dustopamode)
  !  stop 'Unknown dustopamode'
  else ! (par%dustopamode)
    ! --- No additional opacities ---
    !
    addata=0.0
    !
  endif ! (par%dustopamode)
  !
else if ((prop%quc_prop == 'none') .or. (prop%quc_prop == 'None')) then ! ==========================
  ! --- No additional opacities ---
  !
  addata=0.0
  !
else ! (prop%quc_prop) =============================================================================
  ! --- dust opacity [1/cm] (bin independent) ---
  !
  !DIR$ SIMD
  do i1=m1,n1
    addata(i1) = (quc(i1,1)*par%C_dust02)/ &
                     (quc(i1,1)+quc(i1,2)+rho(i1))
    addata(i1) = par%C_dust05*addata(i1)**(2.0/3.0) * & 
                     ((quc(i1,1)+quc(i1,2))/par%C_dust02)
  end do ! i1
  !
!!$      do i1=1,1
!!$        write(6,'(A,3I4,6(1X, 1PE11.4))') 'i123,quc1,quc2,dustopac=', &
!!$          i1, quc(i1,1), quc(i1,2), addata(i1)
!!$      end do ! i1
endif ! (prop%quc_prop) ============================================================================
!
!DIR$ SIMD
do i1=m1,n1
  opclam(i1) = opclam(i1) + addata(i1)
end do ! i1
!
end subroutine op_dust1


!----------*******----------------------------------------------------------------------------------
subroutine op_dust(T, rho, quc, &
                   nu, m1,n1, m2,n2, m3,n3, nquc, opclam)
!---------------------------------------------------------------------------------------------------
! NAME:
!   op_dust ('opacity_dust')
!
! PURPOSE:
!   Compute dust opacity for various dust models.
!   
! CATEGORY:
!   Radiative transfer, opacities
!
! CALLING SEQUENCE:
!   call op_dust(T, rho, quc, &
!                nu, m1,n1, m2,n2, m3,n3, nquc, opclam)
!
! INPUT:
!
! OUTPUT:
!
! RESTRICTIONS:
!   This routine must be called in a scalar environment.
!
! MODIFICATION HISTORY:
!   2013-02-21 (B.F. Lyon) Change layout, add some more comments
!   2013-03-12 (B.F.) Remove argument Plog
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nu, m1,n1, m2,n2, m3,n3, nquc
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: T, rho
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(inout):: opclam
real, dimension(m1:,m2:,m3:,1:), &
                           intent(in)   :: quc
!
! --- Local variables ---
!
real, parameter                         :: PI=3.1415926536, SIGMA=5.6705E-5, C5=SIGMA/PI
integer                                 :: i1, i2, i3, iquc
real, dimension(:,:,:), allocatable     :: addata
!---------------------------------------------------------------------------------------------------
!
! write(6,'(A)') 'prop%quc_prop=' // trim(prop%quc_prop) // &
!                '  par%dustopamode=' // par%dustopamode
!
allocate(addata(m1:n1,m2:n2,m3:n3))
!
if (prop%quc_prop == 'dust_bins_01') then ! ========================================================
  ! --- Grey dust opacities from added dual-slope fits ---
  !
  ! --- (Grey dust opacities from added geometrical cross section) ---
  !
  !$OMP PARALLEL DEFAULT(NONE) &
  !$OMP SHARED(m1,n1, m2,n2, m3,n3,par,prop,quc,addata)
  !
  ! --- Reset addata array  ---
  !$OMP DO PRIVATE(i1,i2,i3)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        addata(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  ! --- Add bin opacities (use constant mass density per monomer) ---
  !$OMP DO PRIVATE(i1,i2,i3,iquc)
  do i3=m3,n3
    do i2=m2,n2
      do iquc=2,prop%nquc
        do i1=m1,n1
          addata(i1,i2,i3)=addata(i1,i2,i3) + &
                           1.5/(par%C_dust01*par%ar_dustGrainRadius(iquc)) * &
                            min(par%ar_dustGrainRadius(iquc)*1.0E+04, 1.0) * &
                           min(max( par%ar_dustGrainRadius(iquc)/6.0E-03, &
                                   (par%ar_dustGrainRadius(iquc)/6.0E-05)**3), 1.0) * &
                           max(quc(i1,i2,i3,iquc), 0.0)
        end do ! i1
      end do ! iquc
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  !$OMP END PARALLEL
  !
else if (prop%quc_prop == 'dust_k3mon_03') then ! ==================================================
  ! --- Dust opacity [1/cm] (bin independent) ---
  !
  if (par%dustopamode == 'largeparticlelimit') then
    ! --- Large particle limit ---
    ! --- Opacity of single grain is 2*pi*r_grain^2 (2*geometrical cross section) ---
    !
    !$OMP PARALLEL DEFAULT(NONE) &
    !$OMP SHARED(m1,n1, m2,n2, m3,n3,par,quc,addata)
    !
    !$OMP DO PRIVATE(i1,i2,i3)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          addata(i1,i2,i3)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             ( quc(i1,i2,i3,2)**2 * &
                              (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  else if (par%dustopamode == 'dualslope01') then
    ! --- Dual-slope method ---
    !
    !$OMP PARALLEL DEFAULT(NONE) &
    !$OMP SHARED(m1,n1, m2,n2, m3,n3,par,quc,addata)
    !
    !$OMP DO PRIVATE(i1,i2,i3)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          ! --- Particle radius ---
          addata(i1,i2,i3)=par%ar_dustGrainRadius(2) * &
                           ( quc(i1,i2,i3,2)         / &
                            (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
          ! --- Limiter function (fit by eye to Rosseland means of Mie opacities ---
          ! ---                   of forsterite; fit parameters in [cm])          ---
          addata(i1,i2,i3)=min(max( addata(i1,i2,i3)/6.0E-03, &
                                   (addata(i1,i2,i3)/6.0E-05)**3), 1.0)
          ! --- Opacity ---
          addata(i1,i2,i3)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             addata(i1,i2,i3) * &
                             ( quc(i1,i2,i3,2)**2 * &
                              (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  else if (par%dustopamode == '') then
    ! --- Old default (before 2008-10-31) ---
    !
    !$OMP PARALLEL DEFAULT(NONE) &
    !$OMP SHARED(m1,n1, m2,n2, m3,n3,par,quc,addata)
    !
    !$OMP DO PRIVATE(i1,i2,i3)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          addata(i1,i2,i3)=1.5/(par%C_dust01*par%ar_dustGrainRadius(2)) * &
                             min(par%ar_dustGrainRadius(2)*1.0E+04, 1.0) * &
                             ( quc(i1,i2,i3,2)**2 * &
                              (quc(i1,i2,i3,1)+quc(i1,i2,i3,2)) )**(1.0/3.0)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  !else ! (par%dustopamode)
  !  stop 'Unknown dustopamode'
  else ! (par%dustopamode)
    ! --- No additional opacities ---
    !
    addata=0.0
    !
  endif ! (par%dustopamode)
  !
else if ((prop%quc_prop == 'none') .or. (prop%quc_prop == 'None')) then ! ==========================
  ! --- No additional opacities ---
  !
  addata=0.0
  !
else ! (prop%quc_prop) =============================================================================
  ! --- dust opacity [1/cm] (bin independent) ---
  !
  !$OMP PARALLEL DEFAULT(NONE) &
  !$OMP SHARED(m3,n3,par,rho,quc,addata)
  !
  !$OMP DO PRIVATE(i3)
  do i3=m3,n3
    addata(:,:,i3) = (quc(:,:,i3,1)*par%C_dust02)/ &
                     (quc(:,:,i3,1)+quc(:,:,i3,2)+rho(:,:,i3))
    addata(:,:,i3) = par%C_dust05*addata(:,:,i3)**(2.0/3.0) * & 
                     ((quc(:,:,i3,1)+quc(:,:,i3,2))/par%C_dust02)
  end do ! i3
  !$OMP END DO
  !
  !$OMP END PARALLEL
  !
!!$  do i3=m3,n3
!!$    do i2=1,1
!!$      do i1=1,1
!!$        write(6,'(A,3I4,6(1X, 1PE11.4))') 'i123,quc1,quc2,dustopac=', &
!!$          i1,i2,i3, quc(i1,i2,i3,1), quc(i1,i2,i3,2), addata(i1,i2,i3)
!!$      end do ! i1
!!$    end do ! i2
!!$  end do ! i3
endif ! (prop%quc_prop) ============================================================================
!
!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(m1,n1, m2,n2, m3,n3,opclam,addata)
!
!$OMP DO PRIVATE(i1,i2,i3)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      opclam(i1,i2,i3) = opclam(i1,i2,i3) + addata(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
deallocate(addata)
!
end subroutine op_dust

end module op_dust_module
!---------------------------------------------------------------------------------------------------
