!***************************************************************************************************
!  ____  _   _ ____       __  __ ____   ____ ____      _    ____  
! |  _ \| | | |  _ \     |  \/  / ___| / ___|  _ \    / \  |  _ \
! | |_) | |_| | | | |    | |\/| \___ \| |   | |_) |  / _ \ | | | |
! |  _ <|  _  | |_| |    | |  | |___) | |___|  _ <  / ___ \| |_| |
! |_| \_\_| |_|____/ ___ |_|  |_|____/ \____|_| \_\/_/   \_\____/ _module
!                                                      
!   Radiation-Hydrodynamics Code: local multi-short-characteristics radiation-transport module
!
!***************************************************************************************************
!   Fortran90
!   Bernd Freytag; Uppsala
!   Matthias Steffen; Potsdam
!   2018-05-07
!***************************************************************************************************
! MODULES:
!   mscrad_gl_module:  Global control parameters for rhd_mscrad routines
!   rhd_mscrad_module: Radiative transfer routines for rhd
!***************************************************************************************************

!------****************-----------------------------------------------------------------------------
MODULE mscrad_gl_module
!
IMPLICIT NONE
SAVE
!
INTEGER :: ncrad
INTEGER :: ifco, ifdust, nbin, nu1, nu2, nnu
!
END MODULE mscrad_gl_module
!----------****************-------------------------------------------------------------------------
!
!------*****************----------------------------------------------------------------------------
module rhd_mscrad_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mscrad_module ('rhd_multi_short_characteristics_radiation_module')
!
! PURPOSE:
!   Radiative transfer routines for rhd.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   use rhd_mscrad_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_mscrad_init:       Initialize MSCrad radiative transfer equations by calling mscrad_ini
!   rhd_mscrad_output:     Derive some averages as additional output quantities
!   rhd_mscrad_step:       Perform 3d radiative energy transfer step
!   rhd_mscrad_cls:        Deallocate arrays used by MSCrad by calling mscrad_cls
!   rhd_mscrad_TemperatureLimiter: 
!                          Correct internal energy to prevent temperature from dropping below limit
!
! MODIFICATION HISTORY:
!   15. 4.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
!   13. 4.00 (B.F. Potsdam) New version
!   16. 2.05 (SWB) revised treatment of CO number densities for radiative cooling
!   2008-08-08 (M.S.) Incident radiation
!   2008-10-10 (B.F.) Merging
!   2009-01-03 (M.S.) Allocatable array addata introduced for storing 
!                     additional external opacities (CO, dust)
!---------------------------------------------------------------------------------------------------
!
contains
!
!----------***************--------------------------------------------------------------------------
subroutine rhd_mscrad_init(model, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mscrad_init ('rhd_multi_short_characteristics_radiation_initialize')
!
! PURPOSE:
!   Initialize radiative transfer solver by calling mscrad_ini.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   model:        ('model') derived type, model data: rho, pv, e, x, time, ...
!   action:       ('action') derived type with collection of control parameters
!
! OUTPUT:
!
! ROUTINES:
!
! MODULES:
!   opta_module:           Opacity table routines
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   15. 4.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
!   13. 4.00 (B.F. Potsdam) New version
!   24.03.04 (M.S.) Open output file 'rhd.fine'
!   16.02.05 (SWB)  initialisation for CO rad. cooling
!   03.01.09 (MS.)  Allocation of addata array
!   20.04.11 (B.F.) Switch off boundary flux also for bound_inoutflow2
!---------------------------------------------------------------------------------------------------
use uio_bulk_module
use opta_module
use mscrad_module
use rhd_gl_module
use mscrad_gl_module
use rhd_action_module
use rhd_box_module
!
# ifdef rhd_chem01
use chem_rn_module, only: chem_rn_init_co_indices
# endif 
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
integer                                 :: mradbc, mthint
real                                    :: dr, Dinci, Tinci, Tsurf
real, dimension(model%m3:model%n3)      :: z
!
LOGICAL, PARAMETER                      :: D15_RT=.false. ! 1.5D RT if .true.
!
!---------------------------------------------------------------------------------------------------
!
! === Geometry ===
!
! --- Horizontal grid spacing from first axis ---
dr=model%xb1(model%m1+1)-model%xb1(model%m1)
!
! --- Vertical axis and (variable) spacing ---
z=model%xc3(model%m3:model%n3)
!
! --- Temperature at tau --> 0 ---
Tsurf = par%Teff*par%C_Tsurf
!
! === Boundary conditions ===
!
! --- Upper and lower boundary condition ---
if ((action%bound_l3 == bound_inoutflow) .OR. (action%bound_l3 == bound_inoutflow2)) then
  mradbc = 0
else
  if (trim(par%heat_mode) == '' .OR. trim(par%heat_mode) == 'None') then
    mradbc = 1
  else
    mradbc =-1
  endif
endif
!
Tinci=par%C_radTinci
Dinci=par%C_radDinci
!
! --- Method to be used for theta integration ---
if (    (trim(action%radraybase) == 'lobatto')) then
   mthint=1
elseif ((trim(action%radraybase) == 'dblgaus')) then
   mthint=2
elseif ((trim(action%radraybase) == 'dblrdau')) then
   mthint=3
else
   write(gl%nc_p,*)
   write(gl%nc_p,*) 'Unknown radraybase:', action%radraybase
   write(gl%nc_p,*) 'Aborting program !'
   stop
endif
!
if ((Tinci > 0.0) .AND. (Dinci > 0.0) .AND. (mthint == 2)) then
   write(gl%nc_p,*) 'Warning ...'
   write(gl%nc_p,*) 'No incident radiation possible with' // &
                    ' selected quadrature scheme:', trim(action%radraybase)
   write(gl%nc_p,*) 'Continuing program ...'
endif
!
! --- Open channel for full MSCrad output ---
if ((trim(par%OutFile_fine) /= '')) then
   !--- Get channel number for 'rhd.fine'
   call uio_chunit('get', nc1out=ncrad)
   !--- Open output file 
   write(gl%nc_p,*)
   write(gl%nc_p,'(A,I3)') 'mscrad_init: file ''' // trim(par%OutFile_fine) // &
                           ''' opened on channel', ncrad
   open(ncrad, file=trim(par%OutFile_fine), form='formatted', status='unknown')
endif
!
! --- Start initialization ---
call mscrad_ini(model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                par%N_radtheta, par%N_radThickPoint, mradbc, mthint, dr, z, &
                par%Teff, par%grav, Tsurf, par%C_radHtautop, Tinci, Dinci, &
                D15_RT)
!
if ((par%N_radband >= 1) .and. (par%N_radband <= 4)) then  
  if (    par%N_radband == 1) then
    nbin  =0
    nu1   =0
    nu2   =0
    ifco  =0
  elseif (par%N_radband == 2) then
    call opta_Info(nbin=nbin)
    nu1   =1
    nu2   =nbin
    ifco  =0
  elseif (par%N_radband == 3) then
    nbin  =2
    nu1   =1
    nu2   =2
    ifco  =1
  elseif (par%N_radband == 4) then
    nbin  =2
    nu1   =1
    nu2   =2
    ifco  =2
  endif
  nnu = nu2 - nu1 + 1
else 
  ! --- Error! Invalid value for N_radband ---
  write(6,'(A,I7)') 'ERROR> Invalid value for N_radband: ', par%N_radband
  stop
end if 
!
if (par%dustopamode == '') then  
  ifdust=0
else
  ifdust=1
endif
!
! --- init CO radiative cooling ---
if (par%N_radband == 4) then 
# ifdef rhd_chem01
  call chem_rn_init_co_indices()
# else 
  write(6,'(A)') 'ERROR> Chemistry routines are not available but are required for N_radband=4!'
  stop
# endif 
end if 
!
end subroutine rhd_mscrad_init
!--------------***************----------------------------------------------------------------------

!----------*****************------------------------------------------------------------------------
subroutine rhd_mscrad_output(model, dtimerel, kapparho, T, Ferb3_xmean, Intsurf, &
                             dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mscrad_output ('rhd_multi_short_characteristics_radiation_output')
!
! PURPOSE:
!   Derive some averages as additional output quantities.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_mscrad_output(model, dtimerel, kapparho, T, Ferb3_xmean, Intsuf, &
!                         dat)
!
! INPUT:
!   model:       ('model') derived type, model data: rho, e, v, x, time, ...
!   dtimerel:    ('delta_time_relative') real, fraction of current sub time step from full time step
!   kapparho:    ('kappa_rho') real 3D array with opacities (per volume)
!   T:           ('Temperature') real 3D array with temperature
!   Ferb3_xmean: ('Flux_energy_radiative_boundary_3_x_mean') real 1D array with mean radiative flux
!                in x3 direction
!   Intsurf:     ('Intensity_surface') real 3D array with frequency-integrated surface intensity
!                                      as well as surface intensity in all frequency bins
!
! INPUT/OUTPUT:
!   dat:         ('data') derived type with additional output data
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!
! MODULES:
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_glrad_module:      Global parameters for MSCrad
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute some averages for requested quantities
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-05-22 (B.F.) Put into own routine
!   2001-07-02 (B.F.) Temperature averages added
!   2008-02-26 (B.F. Lyon) T^2 average added
!   2017-08-21 (B.F.) Change some indentations and modify or add comments
!   2017-08-23 (B.F.) Adapt averaging calls to new datx type; add AreaFractions
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!---------------------------------------------------------------------------------------------------
use mscrad_gl_module
use rhd_dat_module
use rhd_box_module
use rhd_prop_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),           intent(in)    :: model
real,                     intent(in)    :: dtimerel
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3), &
                          intent(in)    :: kapparho, T
real, dimension(model%m3:model%n3+1), &
                          intent(in)    :: Ferb3_xmean
real, dimension(model%m1:model%n1,  &
                model%m2:model%n2,  &
                       0:     nnu), &
                          intent(in)    :: Intsurf
type(dat_type),           intent(inout) :: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, nu, di3
real                                    :: AreaFraction12, AreaFraction13, AreaFraction23
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: x12weight
real, dimension(model%m1:model%n1, &
                model%m3:model%n3)      :: x13weight
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: x23weight
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Update surface intensities ---
!if (associated(dat%int1_l)) then
!  dat%int1_l=0.0
!endif
!if (associated(dat%int1_r)) then
!  dat%int1_r=0.0
!endif
!if (associated(dat%int2_l)) then
!  dat%int2_l=0.0
!endif
!if (associated(dat%int2_r)) then
!  dat%int2_r=0.0
!endif
!if (associated(dat%int3_l)) then
!  dat%int3_l=0.0
!endif
if (associated(dat%int3_r)) then
  dat%int3_r=dat%int3_r + dtimerel*Intsurf(m1:n1,m2:n2,0)
endif
if (associated(dat%intxx3_r)) then
  do nu=nu1,nu2
    dat%intxx3_r(m1:n1,m2:n2,nu)=dat%intxx3_r(m1:n1,m2:n2,nu) + &
                                 dtimerel*Intsurf(m1:n1,m2:n2,nu)
  end do
endif
!
! --- Update surface fluxes ---
!if (associated(dat%fer1_l)) then
!  dat%fer1_l=0.0
!endif
!if (associated(dat%fer1_r)) then
!  dat%fer1_r=0.0
!endif
!if (associated(dat%fer2_l)) then
!  dat%fer2_l=0.0
!endif
!if (associated(dat%fer2_r)) then
!  dat%fer2_r=0.0
!endif
!if (associated(dat%fer3_l)) then
!  dat%fer3_l=0.0
!endif
!if (associated(dat%fer3_r)) then
!  dat%fer3_r=0.0
!endif
!
if ((wg%rad(1)%n(1) > wg%rad(1)%m(1)) .and. (dat%x23%na > 0)) then
  ! --- Compute weight ---
  AreaFraction23=((model%xb2(n2+1)-model%xb2(m2)) / (wg%rad(1)%xb_r(2)-wg%rad(1)%xb_l(2))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%rad(1)%xb_r(3)-wg%rad(1)%xb_l(3)))
  call rhd_mean_aweight(m2,n2, m3,n3, model%xb2, model%xb3, AreaFraction23*dtimerel, &
                        x23weight)
  !
  ! --- Average ---
  call rhd_mean_x23mean(kapparho     , dat%x23%d(m1:n1,dat%x23%i_kapparho), x23weight)
  call rhd_mean_x23mean(T            , dat%x23%d(m1:n1,dat%x23%i_T)       , x23weight)
  call rhd_mean_x23mean(T**2         , dat%x23%d(m1:n1,dat%x23%i_T2)      , x23weight)
  call rhd_mean_x23mean(T**4         , dat%x23%d(m1:n1,dat%x23%i_T4)      , x23weight)
  call rhd_mean_x23mean(T**4*kapparho, dat%x23%d(m1:n1,dat%x23%i_T4kaprho), x23weight)
  !
endif
!
if ((wg%rad(1)%n(2) > wg%rad(1)%m(2)) .and. (dat%x13%na > 0)) then
  ! --- Compute weight ---
  AreaFraction13=((model%xb1(n1+1)-model%xb1(m1)) / (wg%rad(1)%xb_r(1)-wg%rad(1)%xb_l(1))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%rad(1)%xb_r(3)-wg%rad(1)%xb_l(3)))
  call rhd_mean_aweight(m1,n1, m3,n3, model%xb1, model%xb3, AreaFraction13*dtimerel, &
                        x13weight)
  !
  ! --- Average ---
  call rhd_mean_x13mean(kapparho     , dat%x13%d(m2:n2,dat%x13%i_kapparho), x13weight)
  call rhd_mean_x13mean(T            , dat%x13%d(m2:n2,dat%x13%i_T)       , x13weight)
  call rhd_mean_x13mean(T**2         , dat%x13%d(m2:n2,dat%x13%i_T2)      , x13weight)
  call rhd_mean_x13mean(T**4         , dat%x13%d(m2:n2,dat%x13%i_T4)      , x13weight)
  call rhd_mean_x13mean(T**4*kapparho, dat%x13%d(m2:n2,dat%x13%i_T4kaprho), x13weight)
  !
endif
!
if ((wg%rad(1)%n(3) > wg%rad(1)%m(3)) .and. (dat%x12%na > 0)) then
  ! --- Compute weight ---
  AreaFraction12=((model%xb1(n1+1)-model%xb1(m1)) / (wg%rad(1)%xb_r(1)-wg%rad(1)%xb_l(1))) * &
                 ((model%xb2(n2+1)-model%xb2(m2)) / (wg%rad(1)%xb_r(2)-wg%rad(1)%xb_l(2)))
  call rhd_mean_aweight(m1,n1, m2,n2, model%xb1, model%xb2, AreaFraction12*dtimerel, &
                        x12weight)
  !
  ! --- Average ---
  call rhd_mean_x12mean(kapparho     , dat%x12%d(m3:n3,dat%x12%i_kapparho), x12weight)
  call rhd_mean_x12mean(T            , dat%x12%d(m3:n3,dat%x12%i_T)       , x12weight)
  call rhd_mean_x12mean(T**2         , dat%x12%d(m3:n3,dat%x12%i_T2)      , x12weight)
  call rhd_mean_x12mean(T**4         , dat%x12%d(m3:n3,dat%x12%i_T4)      , x12weight)
  call rhd_mean_x12mean(T**4*kapparho, dat%x12%d(m3:n3,dat%x12%i_T4kaprho), x12weight)
  !
  if (n3 < wg%rad(1)%n(3)) then
    di3=0
  else
    di3=1
  endif
  !  original statement:
  !  dat%x12%d(m3:n3+di3,dat%x12%i_fer)=AreaFraction12*Ferb3_xmean(m3:n3+di3)
  !  [10-2017, mst] replaced by:
  dat%x12%d(m3:n3+di3,dat%x12%i_fer)=dat%x12%d(m3:n3+di3,dat%x12%i_fer) + &
                            dtimerel*AreaFraction12*Ferb3_xmean(m3:n3+di3)
  !
endif
!
end subroutine rhd_mscrad_output
!--------------*****************--------------------------------------------------------------------


!----------***************--------------------------------------------------------------------------
subroutine rhd_mscrad_step(model, action, dtime, dtime_limit, &
                           update, output, outintensity, dat, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mscrad_step ('rhd_multi_short_characteristics_radiation_step')
!
! PURPOSE:
!   Perform 3d radiative energy transfer step.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_mscrad_step(model, action, dtime, dtime_limit, &
!                        update=update, output=output, &
!                        outintensity=outintensity, &
!                        dat=dat, outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   update:       ('update') logical, if true the internal energy is updated, default=.true.
!   output:       ('output') logical, if true the output fluxes are computed, default=.false.
!   outintensity: ('out_intensity') logical, presently not used.
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, pv, e, x, time, ...
!!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, recommended maximum radiative time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                   =0: normal run
!                 1-99:  "small" error (e.g. time step limit violation)
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   eosinter_logPTdTOMP:   Evaluate EOS (via interpolation in table) rho, e -> P, T, ...
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines for rhd to handle the parameter structure 'action'
!   rhd_box_module:        Box handling routines for rhd
!   rhd_gl_module:         Global type definitions and parameters for rhd
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   13. 4.00 Bernd Freytag, Matthias Steffen, AIP Potsdam
!   20. 6.00 (B.F. Kiel) Index correction opclam
!   23. 3.01 (M.S. Potsdam) OMP Version
!   22. 5.01 (B.F. Uppsala)
!   20.12.01 (B.F.) Test if minimum temperature is below absolute limit
!   28.11.02 (M.S.) opclam, blclam now 3D instead of 4D
!   24.03.04 (M.S.) write intensity output to file 'rhd.fine'
!   16.02.05 (SWB)  indices for chemical species for non-equilibrium CO, output of Qrad
!   02.08.07 (M.S.) OpenMP: DEFAULT(NONE)
!   09.07.07 (B.F. Lyon) Dust opacities
!   13.12.07 (B.F.) Dust opacity formulas for 'dust_bins_01' and 'dust_k3mon_03' changed
!   14.01.08 (B.F.) Reduced opacities for small grains for 'dust_bins_01' and 'dust_k3mon_03'
!   02.01.08 (B.F.) Correction in small-grain opacity
!   04.07.08 (B.F.) New formula for grey dust opacities for dustScheme dust_bins_01
!   31.10.08 (B.F.) dustopamode
!   03.01.09 (M.S.) Use addata array instead of abusing dhoeidtime array
!   12.04.11 (S.W.) Calling artificial temperature limiter
!   15.02.12 (S.W.) Calling artificial temperature limiter with parameter C_TminLimit
!   17.02.13 (B.F.) Rename eosinter routine
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use const_module, only: sigma, twopi
use timing_module
use rhd_gl_module
use mscrad_gl_module
use rhd_action_module
use rhd_box_module
use rhd_dat_module
use opta_chem
use mscrad_module
use mscrad_dat3, ONLY: nmu, xmu
!
! --- needed for output of Qrad ---
use uio_bulk_module
use uio_filedef_module
!
! --- needed for chemical species ---
# ifdef rhd_chem01
use chem_rn_module, only: isp_n, isp_co,  isp_c, nsp_cmult
# endif 
!
implicit none
!
! --- I/O parameters ---
type(box_type),             intent(inout) :: model
type(action_type),          intent(in)    :: action
real,                       intent(in)    :: dtime
real,                       intent(out)   :: dtime_limit
logical,          optional, intent(in)    :: update, output, outintensity
type(dat_type),   optional, intent(inout) :: dat
integer,          optional, intent(out)   :: ierr
character(LEN=*), optional, intent(out)   :: outstr
!
! --- Local variables ---
integer                               :: icount, nc
integer                               :: i1Tmin, i2Tmin, i3Tmin
integer                               :: i1_thread, i2_thread, i3_thread
real, parameter                       :: T_min=100.0
real, dimension(5)                    :: Frbot
real, dimension(4)                    :: Frtop
real, dimension(:,:,:), ALLOCATABLE   :: T, logP, time_rad, dTdei, &
                                         drhoeidtime, blclam, opclam
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                       0:     nnu)    :: Intsurf
real, dimension(model%m3:model%n3)    :: Ferc3_xmean, divFr, dxb3, dxc3
real, dimension(model%m3:model%n3+1)  :: Ferb3_xmean

real                                  :: dtime_allsteps, dtime_thisstep, &
                                         dtime_limit_thisstep, dtime_limit_recommend, &
                                         dtime_rad, dtime_drhoei, dtimerel, T_minval, &
                                         time_rad_min, dlneidt_max, &
                                         Hptop, Tsurf, dcool, scool, tcool
real                                  :: dum1_thread, dum2_thread, dum3_thread
real                                  :: eitable_min
!
integer                               :: mu, nu
integer                               :: i1, i2, i3, iquc, &
                                         n,m1,n1,m2,n2,m3,n3,n_subdtime
logical                               :: exit_flag, update_flag, output_flag, &
                                         fine1_flag, periodic_flag
integer                               :: ierr0
character(LEN=80)                     :: outstr0
REAL, DIMENSION(:,:),   ALLOCATABLE   :: y3avg, y3rms
!
!---------------------------------------------------------------------------------------------------
!
write(6,*)
write(6,'(A)') '=== Start of rhd_mscrad_step ==='
!
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Default values for keywords ---
if (.not. present(update)) then
  update_flag=.true.
else
  update_flag=update
endif
if (.not. present(output)) then
  output_flag=.false.
else
  output_flag=output
endif
!
! --- fine output flag ---
if ((trim(par%OutFile_fine) /= '') .AND. (par%dtimestep_out_fine >= 0)) then
  fine1_flag=.true.
else
  fine1_flag=.false.
endif
!
! --- Boundary conditions periodic? ---
periodic_flag=((action%bound_l1 == bound_periodic) .or. (action%bound_r1 == bound_periodic) .or. &
               (action%bound_l2 == bound_periodic) .or. (action%bound_r2 == bound_periodic))
if (.not. periodic_flag) then
  ierr0=101
  outstr0='RAD: Boundary conditions not periodic'
  goto 999
endif
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
ALLOCATE(y3avg(1:nmu,0:nbin),y3rms(1:nmu,0:nbin)) 
!
ALLOCATE(       T(m1:n1,m2:n2,m3:n3),   logP(m1:n1,m2:n2,m3:n3), &
         time_rad(m1:n1,m2:n2,m3:n3),  dTdei(m1:n1,m2:n2,m3:n3), &
      drhoeidtime(m1:n1,m2:n2,m3:n3), blclam(m1:n1,m2:n2,m3:n3), &
           opclam(m1:n1,m2:n2,m3:n3) )
!
dtime_limit=huge(dtime_limit)
dtime_allsteps=0.0
n_subdtime=0
!
! --- Loop over all sub time steps ---
DTIME_LOOP: do
  !
  n_subdtime=n_subdtime+1
  write(6,'(A,I4)') 'n_subdtime: ', n_subdtime
  !
  ! --- Test if number of time steps is above absolute limit ---
  if (par%N_radmaxiter > 0) then
    if (n_subdtime > par%N_radmaxiter) then
      ierr0  =1
      outstr0='RAD: Number of sub time step above absolute limit'
      dtime_limit=dtime * real(max(par%N_raditer,1)) / &
                          real(max(par%N_raditer+1,par%N_radmaxiter,2))
      exit DTIME_LOOP
    endif
  endif
  !
  ! -- get minimum internal energy of EOS table ---
  call tabinter_getinfo(x2min=eitable_min)
  !
  call timing_start('RHD_MSCRAD EOS')
  !
  !$OMP PARALLEL DEFAULT(NONE) &
  !$OMP SHARED(model, logP, T, dTdei, T_minval, m1, m2, m3, n1, n2, n3, &
  !$OMP        i1Tmin, i2Tmin, i3Tmin), &
  !$OMP PRIVATE(i1_thread, i2_thread, i3_thread, dum1_thread)
  !
  ! --- Solve EOS ---
  call eosinter_logPTdTOMP(model%rho, model%ei, logP, T, dTdei, m1,n1, m2,n2, m3,n3)
  !
  ! --- Minimum temperature
  !$OMP MASTER
  T_minval = T(m1,m2,m3)
  i1Tmin   = m1  
  i2Tmin   = m2  
  i3Tmin   = m3  
  !$OMP END MASTER
  !
  dum1_thread = T(m1,m2,m3)
  i1_thread   = m1
  i2_thread   = m2
  i3_thread   = m3
  !
  !$OMP DO PRIVATE(i1,i2,i3)
  do i3=m3, n3
    do i2=m2,n2
      do i1=m1,n1
        if (T(i1,i2,i3) < dum1_thread) THEN
          dum1_thread = T(i1,i2,i3)
          i1_thread=i1
          i2_thread=i2
          i3_thread=i3
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  !$OMP CRITICAL
   if (dum1_thread < T_minval) THEN
     T_minval = dum1_thread
     i1Tmin   = i1_thread
     i2Tmin   = i2_thread
     i3Tmin   = i3_thread
   endif
  !$OMP END CRITICAL
  !
  !$OMP END PARALLEL
  !
  call timing_stop('RHD_MSCRAD EOS')
  !
  WRITE(6,'(A,1P,E12.4,A,I4,A,I4,A,I4,A)') &
            "Minimum Temperature         after call of    EOS:", &
             T_minval, ' (', i1Tmin, ',', i2Tmin, ',', i3Tmin, ')'
  !
  ! --------------------------------------------------------------------------------
  ! --- Prevent the gas temperature dropping below a specified limit --- 
  !
  if (par%C_TminLimit > T_minval) then 
     call rhd_mscrad_TemperatureLimiter(gl%nc_p, model, T, dTdei, &
                                        par%C_TminLimit)
     T_minval=par%C_TminLimit
  endif
  ! --------------------------------------------------------------------------------
  ! --- Test if minimum temperature is below absolute limit ---
  if (T_minval < T_min) then
    ierr0  =103
    outstr0='RAD: Minimum temperature below absolute limit'    
    dtime_limit=dtime_limit_thisstep
    exit_flag=.true.
    exit DTIME_LOOP
  endif
  !
  IF (par%N_radband > 0) THEN
  !
  ! --- Solve radiative transfer equations, give div(Frad) ---
  ! --- mscrad3 provides kappa_Rosseland [cm^-1] in opclam    ---
  !
    call timing_start('MSCrad')
  !
# ifdef rhd_chem01
    if (isp_n.ge.1) then
       ispn    = isp_n
       ispco   = isp_co
       ALLOCATE(ispc(ispn), nspcmult(ispn))
       ispc    = isp_c
       nspcmult= nsp_cmult
    endif
# endif
  !
    IF (par%N_radrsyslevel < 0) THEN
       icount=model%itime
    ELSE
       icount=par%N_radrsyslevel
    ENDIF
!
    IF (model%nquc == 0) THEN
      call mscrad3(nu1, nu2, logP, T, model%rho, drhoeidtime, time_rad, &
                   blclam, opclam, Intsurf, Ferc3_xmean, Ferb3_xmean, &
                   Frbot, Frtop, y3avg, y3rms, ifco, ifdust, &
                   model%nquc, output_flag=output_flag)
    ELSE
      call mscrad3(nu1, nu2, logP, T, model%rho, drhoeidtime, time_rad, &
                   blclam, opclam, Intsurf, Ferc3_xmean, Ferb3_xmean, &
                   Frbot, Frtop, y3avg, y3rms, ifco, ifdust, &
                   model%nquc, quc=model%quc, output_flag=output_flag)
    ENDIF
  !
    call timing_stop('MSCrad')
  !
    time_rad_min = huge(time_rad_min)
    i1Tmin = m1
    i2Tmin = m2
    i3Tmin = m3
    dlneidt_max = 1.0E-06/dtime

    !$OMP PARALLEL DEFAULT(NONE) &
    !$OMP SHARED(model, logP, T, dTdei, drhoeidtime, eitable_min, &
    !$OMP        time_rad, time_rad_min, dlneidt_max, dtime, &
    !$OMP        m1, m2, m3, n1, n2, n3, i1Tmin, i2Tmin, i3Tmin) &
    !$OMP PRIVATE(i1_thread, i2_thread, i3_thread, &
    !$OMP         dum2_thread, dum3_thread)
    !
    dum2_thread = huge(dum2_thread)
    i1_thread   = m1
    i2_thread   = m2
    i3_thread   = m3
    dum3_thread = 1.0E-06/dtime
    !$OMP DO PRIVATE(i1,i2,i3)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
    ! --- (non-grey) Eddington approximation ---
          time_rad(i1,i2,i3)=model%rho(i1,i2,i3)/(16.0*sigma) / &
                            (dTdei(i1,i2,i3)*T(i1,i2,i3)**3*time_rad(i1,i2,i3))
          if (time_rad(i1,i2,i3) < dum2_thread) then
             dum2_thread = time_rad(i1,i2,i3)
             i1_thread   = i1
             i2_thread   = i2
             i3_thread   = i3
          endif
          dum3_thread=max(dum3_thread, &
               ABS(drhoeidtime(i1,i2,i3) / &
                   (model%rho(i1,i2,i3)*(model%ei(i1,i2,i3)-eitable_min))))
        end do
      end do
    end do
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (MINMAX_REDUCTION)
    if (dum2_thread < time_rad_min) then
      time_rad_min = dum2_thread
      i1Tmin = i1_thread
      i2Tmin = i2_thread
      i3Tmin = i3_thread
    endif
      dlneidt_max = max(  dlneidt_max, dum3_thread)
    !$OMP END CRITICAL (MINMAX_REDUCTION)
    !
    !$OMP END PARALLEL
    !
    WRITE(6,'(A,1P,E12.4,A,I4,A,I4,A,I4,A)') &
              "Minimum radiative time step after call of mscrad3:", &
               time_rad_min, ' (', i1Tmin, ',', i2Tmin, ',', i3Tmin, ')'
    !
  ENDIF
  !
  IF (par%N_radThickPoint > 0) THEN
  write(6,'(A,1P,3E13.5)') 'FDbot0,   FDbot1,   FDbot2:', &
                            Frbot(1), Frbot(2), Frbot(3)
  ENDIF
  write(6,'(A,1P,2E13.5)') 'Frbot0,   Frbot1:          ', &
                            Frbot(4), Frbot(5)
  write(6,'(A,1P,3E13.5)') 'Frtop0,   Frtop1,   Frtop2:', &
                            Frtop(1), Frtop(2), Frtop(3)
  write(6,'(A,1P,1E13.5)') 'Frtau0:                    ', &
                            Frtop(4)
  !
  if (fine1_flag .AND. (MOD(model%itime, MAX(1,par%dtimestep_out_fine)) == 0) &
                 .AND. (n_subdtime == 1)) then
     write(ncrad,'(A)')
     write(ncrad,'(A,I8)')        'itime:', model%itime+1
     write(ncrad,'(A,1P, E14.7)') ' time:', model%time+dtime
     write(ncrad,'(A,1P,2E13.5)') 'Frbot0,   Frbot1:          ', &
                                   Frbot(4), Frbot(5)
     write(ncrad,'(A,1P,3E13.5)') 'Frtop0,   Frtop1,   Frtop2:', &
                                   Frtop(1), Frtop(2), Frtop(3)
     write(ncrad,'(A)')
     write(ncrad,'(A)') ' nu  mu      Mean Intensity   RMS contrast'
     write(ncrad,'(A)') '=========================================='
     do nu=nu1,nu2
       do mu=1,nmu
         write(ncrad,'(I2,F8.3,2X,1P,E12.5,5X,E12.5)') &
               nu, xmu(mu), y3avg(mu,nu), y3rms(mu,nu)
       enddo
     enddo
     write(ncrad,'(A)') '=========================================='
  endif
  !
  ! --- Maximum time step ---
  dtime_rad =par%C_radCourantMax*time_rad_min
!
  ! --- Compute change in internal energy ---
  !
  ! --- Check change in internal energy: derive characteristic time scale ---
  !  dtime_drhoei=1.0 / &
  !  max(maxval(abs(drhoeidtime/exp(logP))), 100.0*tiny(dtime_drhoei))
  dtime_drhoei=1.0D0 / dlneidt_max
  !
  ! --- Radiative time scale ---
  dtime_limit_thisstep=dtime_rad
  ! --- Minimum iteration number ---
  if (par%N_radminiter > 0) then
    dtime_limit_thisstep=min(dtime_limit_thisstep, dtime/(real(par%N_radminiter)-0.9))
  endif
  ! --- Time step due to maximum change in internal energy ---
  if (par%C_radmaxeiChange > 0.0) then
    dtime_limit_thisstep=min(dtime_limit_thisstep, par%C_radmaxeiChange*dtime_drhoei)
  endif
  !
  ! --- Test if time step is below absolute limit ---
  if (dtime_limit_thisstep < par%dtime_min_stop) then
    ierr0  =100
    outstr0='RAD: Time step below absolute limit'    
    dtime_limit=dtime_limit_thisstep
    exit_flag=.true.
    if (update_flag) exit DTIME_LOOP
  endif
  !
  dtime_rad =par%C_radCourant*time_rad_min
  ! --- Radiative time scale ---
  dtime_limit_recommend=dtime_rad
  ! --- Minimum iteration number ---
  if (par%N_radminiter > 0) then
    dtime_limit_recommend=min(dtime_limit_recommend, dtime/(real(par%N_radminiter)-0.9))
  endif
  ! --- Time step due to maximum change in internal energy ---
  if (par%C_radmaxeiChange > 0.0) then
    dtime_limit_recommend=min(dtime_limit_recommend, par%C_radmaxeiChange*dtime_drhoei)
  endif
  !
  ! --- Check if current sub time step is large enough to be the last one ---
  if (dtime_limit_thisstep >= (dtime-dtime_allsteps)) then
    dtime_thisstep=dtime-dtime_allsteps
    exit_flag=.true.
  else
    dtime_thisstep=dtime_limit_recommend
    exit_flag=.false.
  endif
  !
  write(6,'(A,1P,4E11.4)') 'dtime_:rad,drhoei,limit_thisstep,allsteps: ', &
                            dtime_rad, dtime_drhoei, dtime_limit_thisstep, dtime_allsteps
  !
  if (update_flag) then
    dtimerel=dtime_thisstep/dtime
  else
    dtimerel=1.0
  endif
  !
  ! --- Update output fluxes (for equidistant grid) ---
  if ((output_flag) .and. present(dat)) then
    call rhd_mscrad_output(model, dtimerel, opclam, T, Ferb3_xmean, &
                           Intsurf, dat)
  endif
  !
  if (update_flag) then
  !$OMP PARALLEL DEFAULT(NONE) &
  !$OMP SHARED(model, drhoeidtime, dtime_thisstep, &
  !$OMP        m1, m2, m3, n1, n2, n3)
  !$OMP DO PRIVATE(i1,i2,i3)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
  ! --- Update internal energy ---
        model%ei(i1,i2,i3) = model%ei(i1,i2,i3) + &
           dtime_thisstep*drhoeidtime(i1,i2,i3)/model%rho(i1,i2,i3)
      end do
    end do
  end do
  !$OMP END DO
  !$OMP END PARALLEL
  else
    exit_flag=.true.
  endif
  !
  ! --- Update sub time step ---
  dtime_allsteps=dtime_allsteps+dtime_thisstep
  !
  ! --- Check for exit ---
  if (exit_flag) exit DTIME_LOOP
  !
end do DTIME_LOOP
!
DEALLOCATE(y3avg,y3rms) 
!
! --- Suggest new time step for radiation transport ---
if (ierr0 == 0) then
  if (par%N_raditer >= max(1, par%N_radminiter)) then
    dtime_limit=dtime_limit_recommend*real(par%N_raditer)
  else
    dtime_limit=dtime_rad
    ! --- Time step due to maximum change in internal energy ---
    if (par%C_radmaxeiChange > 0.0) then
      dtime_limit=min(dtime_limit, par%C_radmaxeiChange*dtime_drhoei)
    endif
  endif
endif
!
!!$! --- Additional test output ---
!!$write(6,'(A)') 'Write into rhd_mscrad_step_test.out'
!!$include 'rhd_mscrad_step_testout.f90'
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!
write(6,'(A)') '=== End of rhd_mscrad_step ==='
write(6,*)
!
!===================================================================================================
! --- OUTPUT QRAD ---
!
! --- Output every dtime_out_full seconds ---
!if ((model%time-time_out_full_last) > par%dtime_out_full) then
!
if (par%N_radoutput .ge. 1) then 
  write(6,*) '> Open file rhd.qrad.'
  call uio_openwr(nc, 'rhd.qrad', form='unformatted', conv='ieee_4', &
                  position='append')
  call uio_wrlabl(nc, dataset_ident, date='now' )
  !
  call uio_wrlabl(nc, box_ident                 )
  call uio_wr(nc, 'z'                  , box_id_ident   )
  call uio_wr(nc, model%xc1            , 'xc1'          )
  call uio_wr(nc, model%xb1            , 'xb1'          )
  call uio_wr(nc, model%xc2            , 'xc2'          )
  call uio_wr(nc, model%xb2            , 'xb2'          )
  call uio_wr(nc, model%xc3            , 'xc3'          )
  call uio_wr(nc, model%xb3            , 'xb3'          )
  call uio_wr(nc, model%itime          , 'itime'        )
  call uio_wr(nc, model%time           , 'time'         )
  call uio_wr(nc, dtime_thisstep       , 'dtime_thisstep')
  call uio_wr(nc, drhoeidtime/model%rho, 'qrad'         )
  if (par%N_radoutput .ge. 2) then 
    call uio_wr(nc, model%ei           , 'ei'           )
    call uio_wr(nc, model%rho          , 'rho'          )
    if (par%N_radoutput .ge. 3) then     
      call uio_wr(nc, model%quc(:,:,:,5) , 'nco'          )
    end if
  end if
  call uio_wrlabl(nc, endbox_ident)
  !
  call uio_wrlabl(nc, box_ident                 )
  call uio_wr(nc, 'par'          , box_id_ident   )
  call uio_wr(nc, par%n_radband         , 'n_radband'          )
  call uio_wrlabl(nc, endbox_ident)
  !
  call uio_wrlabl(nc, enddataset_ident)
  call uio_closrd(nc)
  !
  write(6,*) '> Wrote additional output to file rhd.qrad.'
end if
!
DEALLOCATE(T, logP, time_rad, dTdei, drhoeidtime, blclam, opclam )
!
end subroutine rhd_mscrad_step
!--------------***************----------------------------------------------------------------------


!----------**************---------------------------------------------------------------------------
subroutine rhd_mscrad_cls()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mscrad_cls ('rhd_multi_short_characteristics_radiation_close')
!
! PURPOSE:
!   Deallocate arrays used by MSCrad by calling mscrad_cls.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_mscrad_cls()
!
! INPUT:
!   none
!
! OUTPUT:
!   none
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   16. 3.01 Written by Matthias Steffen, AIP
!   24.03.04 (M.S.) Close output file 'rhd.fine'
!   03.01.09 (M.S.) Deallocate addata array
!---------------------------------------------------------------------------------------------------
!
use uio_bulk_module
use rhd_gl_module
use mscrad_gl_module
use opta_chem
use mscrad_module
!
implicit none
!
! --- Start deallocation ---
call mscrad_cls()
!
 if(ALLOCATED(ispc))     DEALLOCATE(ispc)
 if(ALLOCATED(nspcmult)) DEALLOCATE(nspcmult)
!
if ((trim(par%OutFile_fine) /= '')) then
   !--- Close file 'rhd.fine'
   write(6,'(A)') 'rad_cls: file ''' // trim(par%OutFile_fine) // ''' closed'
   close(ncrad)
   !--- Return channel number for MSCrad output file
   call uio_chunit('return', nc1in=ncrad)
endif

end subroutine rhd_mscrad_cls
!--------------**************-----------------------------------------------------------------------

!----------****************************-------------------------------------------------------------
subroutine rhd_mscrad_TemperatureLimiter(nc_p, model, T, dTdei, TminLimit)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mscrad_TemperatureLimiter ('rhd_radiation_Temperature_Limiter')
!
! PURPOSE:
!   Corrects the internal energy to prevent the gas temperature dropping 
!   below a specified limit, TminLimit. 
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   nc_p:         ('number_channel_print') integer, channel for text output, typically 6
!   T:            ('Temperature') real 3D array with temperature
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array with inverse cv
!   TminLimit:    ('Temperature_minimum_Limit') real scalar with requested minimum temperature
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, pv, e, x, time, ...
!
! OUTPUT:
!   none
!
! ROUTINES:
!   none
!
! MODULES:
!   rhd_box_module:           Box handling routines for rhd
!
! SIDE EFFECTS:
!
! RESTRICTIONS: 
!   Violates conservation laws
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   12. 4.11 Written by Sven Wedemeyer (UiO)
!   12.12.11 (S.W.) use parameter C_TminLimit
!   15.02.12 (M.S.) Routine simplified
!   24.08.12 (M.S.) Array index range corrected, formal argument nc_p added.
!
!---------------------------------------------------------------------------------------------------
!
use rhd_box_module
!
implicit none
!
integer,                 intent(in)   :: nc_p
type(box_type),          intent(inout):: model
real,                    intent(in)   :: TminLimit
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3   ), &
                         intent(in)   :: T, dTdei
!
! --- Local variables ---
integer                               :: i1, i2, i3, m1,n1, m2,n2, m3,n3
integer                               :: ncnt_tfix
!-------------------------------------------------------------------------------
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
ncnt_tfix=0
write(nc_p,'(A,1P,E12.4)') "Limiting minimum temperature to               :", &
      TminLimit

!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(model, T, dTdei, m1, m2, m3, n1, n2, n3, ncnt_tfix, TminLimit) 
!$OMP DO PRIVATE(i1,i2,i3) REDUCTION(+: ncnt_tfix)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      if ( T(i1,i2,i3) < TminLimit) then 
        model%ei(i1,i2,i3) = model%ei(i1,i2,i3) + &
                            (TminLimit-T(i1,i2,i3)) / dTdei(i1,i2,i3)
        ncnt_tfix = ncnt_tfix + 1
      endif
    end do
  end do
end do
!$OMP END DO
!$OMP END PARALLEL

write(nc_p,'(A,1P,I12)') "Activated temperature fix, # of affected cells:", &
      ncnt_tfix  

end subroutine rhd_mscrad_TemperatureLimiter
!--------------*****************************--------------------------------------------------------

end module rhd_mscrad_module
