!***************************************************************************************************
!
!   #####  #  #    #  #  #    #   ####
!     #    #  ##  ##  #  ##   #  #    #
!     #    #  # ## #  #  # #  #  #
!     #    #  #    #  #  #  # #  #  ###
!     #    #  #    #  #  #   ##  #    #
!     #    #  #    #  #  #    #   #### _module
!
!   Timing routines for performance measurements and benchmarks
!
!***************************************************************************************************
!   This module provides a set of routines that allow to measure and record the time spend between
!   calls to start and stop named timers. Strategically placed pairs of calls allow to benchmark
!   individual routine calls or code blocks.
!***************************************************************************************************
!   Fortran90
!   2017-08-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!   Werner Schaffenberger: Potsdam, Freiburg, East Lansing, Exeter
!***************************************************************************************************
! MODULES:
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- ('timing_count_factor') count factor: \label{f90:timing_c_factor1} --- */
# ifndef timing_c_factor
#   define timing_c_factor 1
# endif
!#
!# /* --- ('timing_count_range') counter range: \label{f90:timing_c_range1} --- */
!# /* ---    9: single-precision integer                                    --- */
!# /* ---   15: double-precision integer                                    --- */
# ifndef timing_c_range
#   define timing_c_range 9
# endif
!#
!# /* --- ('timing_rate_type') type of timer to use: \label{f90:timing_r_type1} --- */
!# /* ---   1: call system_clock() (default)                                    --- */
!# /* ---   2: call date_and_time()                                             --- */
!# /* ---   3: t=etime() (Sun version, useful in most cases)                    --- */
!# /* ---   4: call etime() (gfortran version)                                  --- */
!# /* ---   5: call tremain() (Cray version)                                    --- */
!# /* ---   6: call clock() (Hitachi version)                                   --- */
!# /* ---   7: omp_get_wtime()                                                  --- */
# ifndef timing_r_type
#   define timing_r_type 1
# endif
!#
!# /* --- MPI activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------*************--------------------------------------------------------------------------------
module timing_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_module ('timing_module')
!
! PURPOSE:
!   Timing routines for performance measurements.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   use timing_module
!
! VARIABLES:
!   n_time_max:   ('number_time_maximum') integer parameter, maximum number of blocks in list
!   n_thread_max: ('number_thread_maximum') integer parameter, maximum number of threads to be timed
!   n_time_last:  ('number_time_last') integer array, number of blocks in list
!   count_name:   ('counter_name') character array with names of all blocks (32 characters maximum)
!   count_total:  ('counter_total') integer array with total number of counts for each block
!   count_start:  ('counter_start') integer array with start values of system_clock counter for
!                 each block
!   count_number: ('count_number') integer array with number of start-stop cycles for each block
!   count_flag:   ('counter_flag') logical array, indicating if counting process is active for
!                 each block
!
! ROUTINES: (contained)
!   timing_SwitchInfo:     Print information about used compiler switches.
!   timing_Init:           Initialize arrays for timing routines
!   timing_GetHandle:      Get list number from block name
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_SelfTest:       Measure time needed for call of timing routines
!   timing_Write:          Write results of counting process into table
!   timing_Job:            Return the remaining CPU or wall-clock time
!
! EXAMPLE:
!   ! --- Typical example ---
!   call timing_Init()
!   call timing_SelfTest()
!   call timing_start('Code')
!   LOOP:
!     call timing_start('Hydro')
!     call rhd_Hydro(...)
!     call timing_stop('Hydro')
!     !
!     call timing_start('Rad')
!     call rhd_Rad(...)
!     call timing_stop('Rad')
!     ...
!   END LOOP
!   call timing_stop('Code')
!   call timing_SelfTest()
!   call timing_Write(6)
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2000-01-27 (B.F.) Some extensions
!   2002-05-27 (B.F. Uppsala) c_factor introduced
!   2010-04-23 (B.F. Lyon) rhd_tremain: moved from rhd_mac_module.F90 to timing_module.F90
!                     (timing_Job)
!   2013-09-07 (B.F.) Thread-save routines, first version
!   2015-10-28 (B.F. Uppsala) Added option for printout of results
!   2017-08-10 (B.F.) Make printout of statistics work under MPI
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- Global parameters ---
integer, parameter                      :: n_time_max=120, n_thread_max=32
                                           ! --- \label{f90:timing_c_range2} ---
integer(kind=selected_int_kind(timing_c_range)), &
         parameter                      :: c_factor=timing_c_factor
integer, dimension(0:n_thread_max-1)    :: n_time_last
!
character(len=32), &
  dimension(n_time_max,0:n_thread_max-1):: count_name
!
integer(kind=selected_int_kind(timing_c_range)), &
  dimension(n_time_max,0:n_thread_max-1):: count_total, count_start, count_number
logical, &
  dimension(n_time_max,0:n_thread_max-1):: count_flag
real, &
  dimension(n_time_max,0:n_thread_max-1):: count_factor
!
private n_time_max, n_thread_max, &
        c_factor, n_time_last, &
        count_total, count_start, count_number, count_name, count_flag, count_factor
!
interface timing_start
  module procedure timing_startn, timing_starts
end interface ! timing_start
!
interface timing_stop
  module procedure timing_stopn, timing_stops
end interface ! timing_stop
!
contains

!----------*****************------------------------------------------------------------------------
subroutine timing_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_SwitchInfo ('timing_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call timing_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2010-03-31 (B.F. Lyon) Written
!   2017-08-10 (B.F. Uppsala) Add MPI switch
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: timing_module ..........................'
!
write(ncp,'(A,I9)') '  timing' // '_c_factor:         '   , timing_c_factor
write(ncp,'(A,I6)') '  timing' // '_c_range:             ', timing_c_range
write(ncp,'(A,I6)') '  timing' // '_r_type:              ', timing_r_type
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
!
end subroutine timing_SwitchInfo


!----------***********------------------------------------------------------------------------------
subroutine timing_Init(n, all)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_Init ('timing_Initialize')
!
! PURPOSE:
!   Initialize arrays for timing routines.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_Init(n=n, all=all)
!
! INPUT:
!   n:            ('number') optional integer, routine number in timing information list
!   all:          ('all') optional logical, if true (or omitted) all fields are initialized
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! RESTRICTIONS:
!   Should be called only once (e.g., from master thread) to reset data for all threads.
!   Only data for up to n_thread_max threads is collected.
!
! PROCEDURE:
!   Set all counter arrays to zero and all flags to .false.
!
! EXAMPLE:
!   ! --- Initialize all fields (standard initialization) ---
!   call timing_Init(all=.true.)
!   ! --- or: ---
!   call timing_Init()
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2013-09-07 (B.F. Lyon) Thread-save version: reset quantities for all threads
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,         optional, intent(in)   :: n
logical,         optional, intent(in)   :: all
!
! --- Local parameters ---
integer                                 :: ifirst, ilast
!---------------------------------------------------------------------------------------------------
!
! --- Determine range of indices for arrays to be initialized ---
ifirst=0
ilast =0
!
if (present(n)) then
  if ((n>=1) .and. (n<=n_time_max)) then
    ifirst=n
    ilast =n
  endif
endif
!
if (present(all)) then
  if (all) then
    n_time_last(:)=0
    ifirst=1
    ilast =n_time_max
  endif
else
  n_time_last(:)=0
  ifirst=1
  ilast =n_time_max
endif
!
! --- Initialize arrays of specified range ---
count_total( ifirst:ilast,:)=0
count_start( ifirst:ilast,:)=0
count_number(ifirst:ilast,:)=0
count_flag(  ifirst:ilast,:)=.false.
count_name(  ifirst:ilast,:)=''
count_factor(ifirst:ilast,:)=1.0
!
end subroutine timing_Init


!----------****************-------------------------------------------------------------------------
subroutine timing_GetHandle(s, n)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_GetHandle ('timing_Get_Handle')
!
! PURPOSE:
!   Get list number from block name.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_GetHandle(s, n)
!
! INPUT:
!   s:            ('string') character, routine name
!
! OUTPUT:
!   n:            ('number') integer, routine number in timing information list
!
! ROUTINES:
!   None
!
! MODULES:
!   omp_lib:               Library of system OpenMP definitions and routines
!
! RESTRICTIONS:
!   Only the first 32 characters of input string are relevant.
!
! PROCEDURE:
!   Search for index of specified block in stored array.
!
! EXAMPLE:
!   call timing_GetHandle('Hydro', n)
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2013-09-07 (B.F. Lyon) Thread-save version
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: s
integer,                   intent(out)  :: n
!
! --- Local parameters ---
integer                                 :: i, omp_iThread
character(len=32)                       :: s0
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef _OPENMP
!#
omp_iThread=omp_get_thread_num()
!#
# else
!#     /* _OPENMP */
omp_iThread=0
!#
# endif
!#      /* _OPENMP */
!
if (omp_iThread < n_thread_max) then
  n =0
  s0=s
  !
  ! --- Search routine name in list ---
  do i=1,n_time_last(omp_iThread)
    if (s0 == count_name(i,omp_iThread)) then
      n=i
      exit
    endif
  end do ! i
  !
  if (n == 0) then
    ! --- Routine not in list so far ---
    if (n_time_last(omp_iThread) < n_time_max) then
      ! --- Add name to list ---
      n_time_last(omp_iThread)=n_time_last(omp_iThread)+1
      count_name(n_time_last(omp_iThread),omp_iThread)=s0
    else
      ! --- Give handle of last list entry ---
    endif
    n=n_time_last(omp_iThread)
  endif
else
  !write(6,*) 'timing_GetHandle: too many threads: ', omp_iThread
  n=n_time_last(omp_iThread)
endif
!
end subroutine timing_GetHandle


!----------*************----------------------------------------------------------------------------
subroutine timing_startn(n)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_startn ('timing_start_number')
!
! PURPOSE:
!   Start timer for block specified by its array index.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_startn(n)
!
! INPUT:
!   n:            ('number') integer, routine number in timing information list
!
! OUTPUT:
!   None
!
! ROUTINES:
!   system_clock:          Standard Fortran routine to access the system clock
!
! MODULES:
!   omp_lib:               Library of system OpenMP definitions and routines
!
! RESTRICTIONS:
!   The input parameter n has to be within the allowed range:
!     1 <= n <= n_time_max
!   No check is performed!
!
! PROCEDURE:
!   Save current values of system_clock in array.
!
! EXAMPLE:
!   call timing_GetHandle('Hydro', n)
!   LOOP:
!     call timing_start(n)
!     ...
!     call timing_stop(n)
!     ...
!   END LOOP:
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2013-09-07 (B.F. Lyon) Thread-save version
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: n
!
! --- Local parameters ---
integer                                 :: omp_iThread
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef _OPENMP
!#
omp_iThread=omp_get_thread_num()
!#
# else
!#     /* _OPENMP */
omp_iThread=0
!#
# endif
!#      /* _OPENMP */
!
if (omp_iThread < n_thread_max) then
  count_flag(n,omp_iThread)=.true.
  call system_clock(count=count_start(n,omp_iThread))
endif
!
end subroutine timing_startn


!----------*************----------------------------------------------------------------------------
subroutine timing_starts(s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_starts ('timing_start_string')
!
! PURPOSE:
!   Start timer for block specified by a string.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_starts(s)
!
! INPUT:
!   s:            ('string') character, routine name in timing information list
!
! OUTPUT:
!   None
!
! ROUTINES:
!   system_clock:          Standard Fortran routine to access the system clock
!   timing_GetHandle:      Get list number from block name
!
! MODULES:
!   omp_lib:               Library of system OpenMP definitions and routines
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Save current values of system_clock in array.
!
! EXAMPLE:
!   call timing_start('Hydro')
!   call timing_stop('Hydro')
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2013-09-07 (B.F. Lyon) Thread-save version
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: s
!
! --- Local parameters ---
integer                                 :: n, omp_iThread
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef _OPENMP
!#
omp_iThread=omp_get_thread_num()
!#
# else
!#     /* _OPENMP */
omp_iThread=0
!#
# endif
!#      /* _OPENMP */
!
if (omp_iThread < n_thread_max) then
  !write(6,'(A)') 'Timing start: '//s
  call timing_GetHandle(s, n)
  !
  count_flag(n,omp_iThread)=.true.
  call system_clock(count=count_start(n,omp_iThread))
endif
!
end subroutine timing_starts


!----------************-----------------------------------------------------------------------------
subroutine timing_stopn(n, cycles, factor)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_stopn ('timing_stop_number')
!
! PURPOSE:
!   Stop timer for block specified by its array index.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_stopn(n, cycles=cycles, factor=factor)
!
! INPUT:
!   n:            ('number') integer, routine number in timing information list
!   cycles:       ('cycles') optional integer, number of timing cycles to be put into list
!   factor:       ('factor') optional real with factor the final mean time is divided by
!
! OUTPUT:
!   None
!
! ROUTINES:
!   system_clock:          Standard Fortran routine to access the system clock
!
! MODULES:
!   omp_lib:               Library of system OpenMP definitions and routines
!
! RESTRICTIONS:
!   The input parameter n has to be within the allowed range:
!     1 <= n <= n_time_max
!   No check is performed!
!   Care is taken if there is an overflow during counting:
!   System counter exceeds count_max and starts at 0 again.
!   But only one overflow per counting cycle is recognized.
!   In general, if the total number of counts per job comes close to count_max, there will be
!   errors in the results of the routine.
!
! PROCEDURE:
!  Read current value of system_clock and update total number of counts with difference of
!  counter between start of timing and current value.
!
! EXAMPLE:
!   call timing_GetHandle('Hydro', n)
!   LOOP:
!     call timing_start(n)
!     ...
!     call timing_stop(n)
!     ...
!   END LOOP:
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2000-01-27 (B.F.) Parameter cycles added
!   2013-09-07 (B.F. Lyon) Thread-save version
!   2015-10-28 (B.F. Uppsala) Parameter factor added
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: n
integer,         optional, intent(in)   :: cycles
real,            optional, intent(in)   :: factor
!
! --- Local parameters ---
integer                                 :: omp_iThread
integer(kind=selected_int_kind(timing_c_range)) &
                                        :: c_stop, count_max
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef _OPENMP
!#
omp_iThread=omp_get_thread_num()
!#
# else
!#     /* _OPENMP */
omp_iThread=0
!#
# endif
!#      /* _OPENMP */
!
if (omp_iThread < n_thread_max) then
  if (count_flag(n,omp_iThread)) then
    call system_clock(count=c_stop)
    if (c_stop >= count_start(n,omp_iThread)) then
      ! --- Add time interval\label{f90:timing_c_factor2} ---
      count_total(n,omp_iThread)=count_total(n,omp_iThread) + &
                                 (c_stop-count_start(n,omp_iThread))/c_factor
    else
      ! --- There has probably been an overflow ---
      call system_clock(count_max=count_max)
      count_total(n,omp_iThread)=count_total(n,omp_iThread) + &
                                 (count_max-count_start(n,omp_iThread))/c_factor + c_stop/c_factor
    endif
    if (present(cycles)) then
      count_number(n,omp_iThread)=count_number(n,omp_iThread)+cycles
    else
      count_number(n,omp_iThread)=count_number(n,omp_iThread)+1
    endif
    if (present(factor)) then
      count_factor(n,omp_iThread)=factor
    endif
    count_flag(n,omp_iThread)=.false.
  endif
endif
!
end subroutine timing_stopn


!----------************-----------------------------------------------------------------------------
subroutine timing_stops(s, cycles, factor)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_stops ('timing_stop_string')
!
! PURPOSE:
!   Stop timer for block specified by a string.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_stops(s, cycles=cycles, factor=factor)
!
! INPUT:
!   s:            ('string') character, routine name in timing information list
!   cycles:       ('cycles') optional integer, number of timing cycles to be put into list
!   factor:       ('factor') optional real with factor the final mean time is divided by
!
! OUTPUT:
!   None
!
! ROUTINES:
!   system_clock:          Standard Fortran routine to access the system clock
!   timing_GetHandle:      Get list number from block name
!   timing_stopn:          Stop timer for block specified by its array index
!
! MODULES:
!   omp_lib:               Library of system OpenMP definitions and routines
!
! RESTRICTIONS:
!   Some care is taken if there is an overflow during counting:
!   System counter exceeds count_max and starts at 0 again.
!   But only one overflow per counting cycle is recognized.
!   In general, if the total number of counts per job comes close to count_max, there will be
!   errors in the results of the routine.
!
! PROCEDURE:
!  Read current value of system_clock and update total number of counts with difference of
!  counter between start of timing and current value.
!
! EXAMPLE:
!   call timing_start('Hydro')
!   call timing_stop('Hydro')
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2000-01-27 (B.F.) Parameter cycles added
!   2013-09-07 (B.F. Lyon) Thread-save version
!   2015-10-28 (B.F. Uppsala) Insert call to timing_stopn; parameter factor added
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: s
integer,         optional, intent(in)   :: cycles
real,            optional, intent(in)   :: factor
!
! --- Local parameters ---
integer                                 :: n, omp_iThread
integer(kind=selected_int_kind(timing_c_range)) &
                                        :: c_stop, count_max
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef _OPENMP
!#
omp_iThread=omp_get_thread_num()
!#
# else
!#     /* _OPENMP */
omp_iThread=0
!#
# endif
!#      /* _OPENMP */
!
if (omp_iThread < n_thread_max) then
  !
  call timing_GetHandle(s, n)
  !
  call timing_stopn(n, cycles=cycles, factor=factor)
  !
endif
!
end subroutine timing_stops


!----------***************--------------------------------------------------------------------------
subroutine timing_SelfTest(cycles)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_SelfTest ('timing_Self_Test')
!
! PURPOSE:
!   Measure time needed for call of timing routines.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_SelfTest(cycles=cycles)
!
! INPUT:
!   cycles:       ('cycles') integer, number of timing routine calls
!
! OUTPUT:
!   None
!
! ROUTINES:
!   timing_GetHandle:      Get list number from block name
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   None
!
! RESTRICTIONS:
!   The number of cycles has to be reasonable (not too large). No check is performed.
!
! PROCEDURE:
!   Do a number of start/stop cycles of timing measurements and measure their duration.
!
! EXAMPLE:
!   call timing_SelfTest(cycles=20000)
!
! MODIFICATION HISTORY:
!   2000-01-27 (B.F. Copenhagen) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,         optional, intent(in)   :: cycles
!
! --- Local parameters ---
integer                                 :: i, cycles0, handle_timingn, handle_emptyn
integer      , parameter                :: cycles_default=10000
character(len=*), parameter             :: name_timings='Timing cycle (call by name)', &
                                           name_emptys ='Timing empty (call by name)', &
                                           name_timingn='Timing cycle (call by number)', &
                                           name_emptyn ='Timing empty (call by number)'
!---------------------------------------------------------------------------------------------------
!
if (present(cycles)) then
  cycles0=abs(cycles)
else
  cycles0=cycles_default
endif
!
! --- Start timing measurement for call with routine names ---
call timing_start(name_timings)
!
do i=1,cycles0
  ! --- Make dummy calls of start/stop timing cycles ---
  call timing_start(name_emptys)
  call timing_stop( name_emptys)
end do ! i
!
! --- Stop timing measurement ---
call timing_stop( name_timings, cycles=cycles0)
!
! --- Get handles ---
!
call timing_GetHandle(name_timingn, handle_timingn)
call timing_GetHandle(name_emptyn , handle_emptyn )
!
! --- Start timing measurement for call with routine handles ---
call timing_start(handle_timingn)
!
do i=1,cycles0
  ! --- Make dummy calls of start/stop timing cycles ---
  call timing_start(handle_emptyn)
  call timing_stop( handle_emptyn)
end do ! i
!
! --- Stop timing measurement
call timing_stop( handle_timingn, cycles=cycles0)
!
end subroutine timing_SelfTest


!----------************-----------------------------------------------------------------------------
subroutine timing_Write(nc, ScaleExponent, nThread, nProcess)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_Write ('timing_Write')
!
! PURPOSE:
!   Write results of counting process into table.
!
! CATEGORY:
!   Timing
!
! CALLING SEQUENCE:
!   call timing_Write(nc, ScaleExponent=ScaleExponent, nThread=nThread)
!
! INPUT:
!   nc:           ('number_channel') integer, channel number for output
!   ScaleExponent: ('Scale_Exponent') integer with exponent of scale factor for output:
!                   0: output in sec (default)
!                  -3: output in msec (example)
!                   3: output in ksec (example)
!   nThread:      ('number_Threads') integer with number of OpenMP threads for which data should
!                 be printed
!   nProcess:     ('number_Processes') integer with number of MPI processes for which data should
!                 be printed
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   omp_lib:               Library of system OpenMP definitions and routines
!
! RESTRICTIONS:
!   In general, if the total number of counts per job comes close to count_max, there will be
!   errors in the results of the routine.
!
! PROCEDURE:
!   Analyze counter entries and print results in table form.
!
! EXAMPLE:
!   ! --- Write counting results onto screen ---
!   call timing_Write(6)
!
! TODO:
!   Allow choice of output formats.
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2013-09-07 (B.F. Lyon) Thread-save version: print information for all threads
!   2015-10-28 (B.F. Uppsala) Keyword ScaleExponent added
!   2017-08-10 (B.F.) Reshuffle and rename parameters;
!                     add option to print data from several MPI processes
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc
integer,         optional, intent(in)   ::  ScaleExponent, nThread, nProcess
!
! --- Local parameters ---
integer                                 :: n, i_thread, n_threads
integer(kind=selected_int_kind(timing_c_range)) &
                                        :: count_rate
real, dimension(n_time_max)             :: total_time, mean_time
real                                    :: max_total_time, max_mean_time, count_scale
character(len=12)                       :: total_time_format, mean_time_format, mean_time_unit
character(len=80)                       :: head_format, data_format
! --- MPI variables ---
integer                                 :: MyProcWo, nProcsWo, ierrMPI, &
                                           i_process, n_processes
!---------------------------------------------------------------------------------------------------
!
call system_clock(count_rate=count_rate)
!write(nc,*) 'count_rate: ', count_rate
!
! --- Determine/set some global MPI parameters ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
call MPI_Comm_Rank(MPI_COMM_WORLD, MyProcWo, ierrMPI)
call MPI_Comm_Size(MPI_COMM_WORLD, NProcsWO, ierrMPI)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
MyProcWo=0
NProcsWo=1
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
! --- Handle optional parameters ---
if (present(ScaleExponent)) then
  if (ScaleExponent == 0) then
    count_scale=1.0
    mean_time_unit='sec'
  else
    count_scale=10.0**ScaleExponent
    write(mean_time_unit, '(I3)'), ScaleExponent
    mean_time_unit='10^' // trim(adjustl(mean_time_unit)) // ' sec'
  endif
else
  count_scale=1.0
  mean_time_unit='sec'
endif
!
if (present(nThread)) then
  n_threads=max(min(n_thread_max, nThread), 1)
else
  ! n_threads=n_thread_max
  n_threads=1
endif
!
if (present(nProcess)) then
  n_processes=max(min(NProcsWO, nProcess), 1)
else
  n_processes=1
endif
!
! --- Loop over all MPI processes and all OpenMP threads that are requested ---
do i_process=0,n_processes-1
  if (MyProcWo == i_process) then
    do i_thread=0,n_threads-1
      if ((count_rate > 0) .and. (any(count_total(:,i_thread) > 0))) then
        ! --- Write header of timing statistics\label{f90:timing_c_factor3} ---
        write(nc,*)
        write(nc,'(A,I10,A,I10,A,I3,A,I3)') 'Timing statistics (rate=', count_rate, &
                                            ' , factor=', c_factor, &
                                            ') process #', i_process, &
                                            ', thread ', i_thread
        write(nc,'(A)') &
                  '================================================================================'
        !
        where (count_number(:,i_thread) > 0)
          total_time=real(count_total(:,i_thread))/real(count_rate)*real(c_factor)
        end where
        where ((count_number(:,i_thread) > 0) .and. (count_factor(:,i_thread) > 0.0))
          mean_time =total_time / &
                     (real(count_number(:,i_thread))*count_factor(:,i_thread)*count_scale)
        end where
        where ((count_number(:,i_thread) > 0) .and. (count_factor(:,i_thread) == 0.0))
          mean_time =0.0
        end where
        !
        max_total_time=maxval(total_time, mask=(count_total(:,i_thread) > 0))
        max_mean_time =maxval(mean_time , mask=(count_total(:,i_thread) > 0))
        !
        if (max_total_time >= 1.0E+07) then
          total_time_format='1PE14.7'
        else
          total_time_format='F14.6'
        endif
        if (max_mean_time >= 1.0E+07) then
          mean_time_format ='1PE14.7'
        else
          mean_time_format ='F14.6'
        endif
        head_format      ='(A32,1X,A8,1X,A14,1X,A14)'
        data_format      ='(A32,1X,I8,1X,' // trim(total_time_format) // ',1X,' // &
                          trim(mean_time_format) // ')'
        !
        !write(nc,*)
        write(nc,head_format) 'Block                           ', &
                                'Samples', 'Total time', 'Mean time'
        write(nc,head_format) '                                ', &
                                '       ',      '[sec]', '[' // trim(mean_time_unit) // ']'
        write(nc,'(A)') repeat('-',80)
        !
        do n=1,n_time_last(i_thread)
          if (count_number(n,i_thread) > 0) then
            write(nc,data_format) count_name(n,i_thread), &
                                  count_number(n,i_thread), total_time(n), mean_time(n)
          endif
        end do ! n
        !
        !write(nc,*)
        call flush(nc)
        !
      else
        if (i_thread == 0) then
          write(nc,'(A)') 'No timing data available.'
          !write(nc,*)
          call flush(nc)
        endif
      endif
      !
    end do ! i_thread
  endif
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
  call flush(nc)
  call MPI_Barrier(MPI_COMM_WORLD, ierrMPI)
  call flush(nc)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
end do ! i_process
!
end subroutine timing_Write


!----------**********-------------------------------------------------------------------------------
subroutine timing_Job(trem, tela, CPUtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   timing_Job ('rhd_time_remain')
!
! PURPOSE:
!   Return the remaining CPU time.
!   
! CATEGORY:
!   Timing, machine-dependent routines
!
! CALLING SEQUENCE:
!   call timing_Job(trem=trem, tela=tela, CPUtime=CPUtime)
!
! INPUT:
!   CPUtime:      ('CPU_time') real, maximum allowed CPU time for the job, optional
!
! OUTPUT:
!   trem:         ('time_remain') real, remaining CPU seconds for current job, optional
!   tela:         ('time_elapse') real, elapsed CPU seconds for current job, optional
!
! ROUTINES:
!   clock:                 Hitachi: return the remaining CPU time
!   date_and_time:         Standard Fortran routine to get current date and time
!   etime:                 Many Unix systems: return the elapsed CPU time
!   system_clock:          Standard Fortran routine to access the system clock
!   tremain:               CRAY: return the remaining CPU time
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   No outside effects. However, some values are saved.
!
! RESTRICTIONS:
!   The maximum CPU time is set to 1 day. It may be changed with input parameter CPUtime.
!   The various timing routines are not available on all systems.
!   Even the standard Fortran timing routine are not guaranteed to give non-zero results.
!   The standard Fortran routine 'system_clock' can have different count rates, depending on
!   the machine.
!   The various timing routines might count different things (wallclock time, CPU time, ...).
!
! PROCEDURE:
!   Call system or standard Fortran routine to get the current, elapsed, or remaining time.
!   Store initial clock values, if necessary.
!   Compute requested time from current and stored values.
!
! EXAMPLE:
!   call timing_Job()
!   ! ... Some code ...
!   call timing_Job(trem)
!   write(6,*) trem
!
! MODIFICATION HISTORY:
!   1997-05-29 (B.F. Kiel) First version
!   1997-10-14 (B.F.) Sun version written
!   2010-04-21 (B.F. Lyon) Universal version with compiler switch
!   2010-04-22 (B.F.) Some reordering
!   2010-04-23 (B.F.) Moved from rhd_mac_module.F90 (rhd_tremain) to timing_module.F90 (timing_Job),
!                     gfortran branch
!   2011-11-15 (W.S.) Variables in gfortran branch defined as real(kind=4)
!   2013-10-11 (B.F.) omp_get_wtime version
!---------------------------------------------------------------------------------------------------
!$ use omp_lib
!
implicit none
!
! --- I/O parameters ---
real,            optional, intent(in)   :: CPUtime
real,            optional, intent(out)  :: trem, tela
!
! --- Local variables ---
real, save                              :: CPUtime_max=24.0*3600.0
real(kind=kind(0.0D+00)), save          :: CPUtime_maxDB=24.0D+00*3600.0D+00
!---------------------------------------------------------------------------------------------------
!
!#
# if (timing_r_type == 1)
!#                                 /* --- System_Clock version \label{f90:timing_r_type2} ------- */
integer(kind=selected_int_kind(9))      :: count, count_rate, count_max
real(kind=kind(0.0D+00))                :: time_current
real(kind=kind(0.0D+00)), save          :: time_start
!
call system_clock(count=count, count_rate=count_rate, count_max=count_max)
time_current=(real(count     ,kind=kind(0.0D+00))+real(count_max,kind=kind(0.0D+00)))/ &
              real(count_rate,kind=kind(0.0D+00))
!
if ((.not. present(CPUtime)) .and. (.not. present(trem)) .and. (.not. present(tela))) then
  time_start=time_current
endif
if (present(CPUtime)) CPUtime_maxDB=real(CPUtime, kind=kind(0.0D+00))+time_current
!
if (present(trem)) trem=real(CPUtime_maxDB-time_current, kind=kind(0.0E+00))
if (present(tela)) tela=real(time_current -time_start  , kind=kind(0.0E+00))
!
!#
# elif (timing_r_type == 2)
!#                                 /* --- Date_and_Time version --------------------------------- */
integer                                 :: nYear, nLeapDays
integer, dimension(8)                   :: DTv
real(kind=kind(0.0D+00))                :: time_current
real(kind=kind(0.0D+00)), save          :: time_start
integer,dimension(12),parameter         :: &
  days=(/ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /)
!
call date_and_time(values=DTv)
nYear       =max(DTv(1)-2000, 0)
time_current=nYear*365 + days(DTv(2)) + (DTv(3)-1)
nLeapDays   =nYear/4 - nYear/100 + nYear/400 + 1
if ((DTv(2) <= 2) .and. &
    ((mod(nYear,4) == 0) .and. (.not.(mod(nYear,100) == 0))) .or. &
     (mod(nYear,400) == 0)) nLeapDays=nLeapDays-1
time_current=(time_current+nLeapDays)*(24.0D+00*3600.0D+00) + &
             (DTv(5)*60.0D+00 + DTv(6))*60.0D+00 + DTv(7) + DTv(8)*1.0D-03
!
if ((.not. present(CPUtime)) .and. (.not. present(trem)) .and. (.not. present(tela))) then
  time_start=time_current
endif
if (present(CPUtime)) CPUtime_maxDB=real(CPUtime, kind=kind(0.0D+00))+time_current
!
if (present(trem)) trem=real(CPUtime_maxDB-time_current, kind=kind(0.0E+00))
if (present(tela)) tela=real(time_current -time_start  , kind=kind(0.0E+00))
!
# elif (timing_r_type == 3)
!#                                 /* --- Sun version ------------------------------------------- */
real, dimension(2)                      :: tarray
real                                    :: time_elapsed
real, external                          :: etime
!
time_elapsed=etime(tarray)
!
if (present(CPUtime)) CPUtime_max=CPUtime
!
if (present(trem)) trem=CPUtime_max-time_elapsed
if (present(tela)) tela=time_elapsed
!
!#
# elif (timing_r_type == 4)
!#                                 /* --- gfortran version -------------------------------------- */
real(kind=4), dimension(2)              :: tarray
real(kind=4)                            :: time_elapsed
!
call etime(tarray, time_elapsed)
!
if (present(CPUtime)) CPUtime_max=CPUtime
!
if (present(trem)) trem=CPUtime_max-time_elapsed
if (present(tela)) tela=time_elapsed
!
!#
# elif (timing_r_type == 5)
!#                                 /* --- Cray version ------------------------------------------ */
real                                    :: time_remain
external                                :: tremain
!
call tremain(time_remain)
!
if (present(CPUtime)) CPUtime_max=time_remain
!
if (present(trem)) trem=time_remain
if (present(tela)) tela=CPUtime_max-time_remain
!
!#
# elif (timing_r_type == 6)
!#                                 /* --- Hitachi version --------------------------------------- */
real                                    :: time_remain
external                                :: clock
!
call clock(time_remain, 6)
!
if (present(CPUtime)) CPUtime_max=time_remain
!
if (present(trem)) trem=time_remain
if (present(tela)) tela=CPUtime_max-time_remain
!
!#
# elif (timing_r_type == 7)
!#                                 /* --- omp_get_wtime version --------------------------------- */
real(kind=kind(0.0D+00))                :: time_current
real(kind=kind(0.0D+00)), save          :: time_start
!
time_current=omp_get_wtime()
!
if ((.not. present(CPUtime)) .and. (.not. present(trem)) .and. (.not. present(tela))) then
  time_start=time_current
endif
if (present(CPUtime)) CPUtime_maxDB=real(CPUtime, kind=kind(0.0D+00))+time_current
!
if (present(trem)) trem=real(CPUtime_maxDB-time_current, kind=kind(0.0E+00))
if (present(tela)) tela=real(time_current -time_start  , kind=kind(0.0E+00))
!
!#
# else
!#                                 /* --- No counter -------------------------------------------- */
real                                    :: time_elapsed
!
time_elapsed=0.0
!
if (present(CPUtime)) CPUtime_max=CPUtime
!
if (present(trem)) trem=CPUtime_max-time_elapsed
if (present(tela)) tela=time_elapsed
!
!#
# endif
!#      /* timing_r_type -------------------------------------------------------------------------*/
!
end subroutine timing_Job

end module timing_module
