!***************************************************************************************************
!
!   #    # #  ####        #####    ##    ####  ###### 
!   #    # # #    #       #    #  #  #  #      #      
!   #    # # #    #       #####  #    #  ####  #####  
!   #    # # #    #       #    # ######      # #      
!   #    # # #    #       #    # #    # #    # #      
!    ####  #  #### ###### #####  #    #  ####  ######_module
!
!   Universal Input/Output Routines: base part
!
!***************************************************************************************************
!   This set of input and output routines allows the reading and writing of scalar, array, and
!   table data in header-data units in formatted or unformatted Fortran file format.
!***************************************************************************************************
!   Fortran90
!   2017-08-05
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Lyon, Uppsala
!***************************************************************************************************
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_inf_module:        Information about environment
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!   uio_base_module:       Basic set of UIO routines: string processing, header, I/O channels
!
!***************************************************************************************************

!------**************-------------------------------------------------------------------------------
module uio_siz_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_siz_module ('uio_size_module')
!
! PURPOSE:
!   Contain parameters for string length, line and term table size.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_siz_module
!
! VARIABLES:
!   let:    ('length_table') integer, string length for keywords, term tables..
!   lel:    ('length_line') integer, string length of input/output line
!   nttmx:  ('number_term_table_maximum') integer, maximum term table size
!   nltmx:  ('number_line_table_maximum') integer, maximum header line number
!
! MODIFICATION HISTORY:
!   1997-01-10 (B.F. Kiel) Put into module
!---------------------------------------------------------------------------------------------------
implicit none
!
integer, parameter                      :: let=80,   lel=80, &
                                           nttmx=20, nltmx=20
!
end module uio_siz_module


!------**************-------------------------------------------------------------------------------
module uio_inf_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_inf_module ('uio_information_module')
!
! PURPOSE:
!   Contain variables with information about environment.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_inf_module
!
! VARIABLES:
!   syste0: ('system_0') character, operating system (e.g. SunOS, craSH    )
!   machi0: ('machine_0') character, empty or machine name (e.g. venus, craSH  )
!   osrel0: ('operating_system_release_0') character, release of operating system
!           (e.g. 4.1.3, 8.0.3.0)
!   osver0: ('operating system version_0') character, version of operating system
!           (e.g. 1, awa.5)
!   hrdwa0: ('hardware_0') character, empty or machine hardware (e.g. sun4m, CRAY Y-MP)
!   progr0: ('program_name_0') character, name of main program
!
! MODIFICATION HISTORY:
!   1996       (B.F. Kiel) Old: include file with common block: envinf
!   1997-01-11 (B.F.) Put into module
!   1997-02-02 (B.F.) Saved parameter langu0 added
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
character*(let), save                   :: syste0='', machi0, osrel0, &
                                           osver0, hrdwa0, progr0, langu0
!
end module uio_inf_module


!------**************-------------------------------------------------------------------------------
module uio_cst_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_cst_module ('uio_channel_status_information_module')
!
! PURPOSE:
!   Contain variables with channel status information.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_cst_module
!
! VARIABLES:
!   ncmin:  ('number_channel_minimum') integer, first possible channel (e.g. 0)
!   ncmax:  ('number_channel_maximum') integer, last possible channel (e.g. 99)
!   ncnum:  ('number_channel_number') integer, number of channels (ncnum=ncmax-ncmin+1)
!   cvnum:  ('convert_type_number') integer array, number of convert type for each (opened) channel
!   ncpos:  ('number_channel_position') integer array, file pointer position
!   ncstat: ('number_channel_status') character array, status of each channel,
!            actualized by uio_chunit
!
! MODIFICATION HISTORY:
!   1996       (B.F. Kiel) Old: include file with common block: chainf
!   1997-01-11 (B.F.) Put into module
!---------------------------------------------------------------------------------------------------
implicit none
!
integer, parameter                      :: ncmin=0, ncmax=99, &
                                           ncnum=ncmax-ncmin+1
integer, save                           :: cvnum(ncmin:ncmax), &
                                           ncpos(ncmin:ncmax)
character*(20), save                    :: ncstat(ncmin:ncmax)='uninitialized'
!
end module uio_cst_module


!------**************-------------------------------------------------------------------------------
module uio_cvl_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_cvl_module ('uio_conversion_list_module')
!
! PURPOSE:
!   Contain convert type list of current machine.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_cvl_module
!
! VARIABLES:
!   ncvmax: ('number_convert_types_maximum') integer (e.g. 10)
!   ncv:    ('number_convert_types') integer, actual number of conversion types
!   ncvdef: ('number_convert_types_default') integer, default conversion type
!   cvnals: ('convert_types_name_list') character array, names of possible
!           convert types of current machine,
!           set by uio_mkcvls during call of uio_init
!   btinls: ('byte_integer_list') character array, number of bytes for integer
!   btrels: ('byte_real_list') character array, number of bytes for real
!   btcols: ('byte_complex_list') character array, number of bytes for complex
!   btdbls: ('byte_double_precision_list') character array, number of bytes for double-precision real
!   foinls: ('format_integer_list') character, default format for integer
!   forels: ('format_real_list') character, default format for real
!   focols: ('format_complex_list') character, default format for complex
!   fodbls: ('format_double_precision_list') character, default format for double-precision real
!
! MODIFICATION HISTORY:
!   1996       (B.F. Kiel) Old: include file with common block: cvlist
!   1997-01-12 (B.F.) Put into module
!---------------------------------------------------------------------------------------------------
implicit none
!
integer, parameter                      :: ncvmx=10
integer, save                           :: ncv, ncvdef
character*(20), save                    :: cvnals(ncvmx)
character*(2), save                     :: btinls(ncvmx),    btrels(ncvmx), &
                                           btcols(ncvmx),    btdbls(ncvmx)
character, save                         :: foinls(ncvmx)*3,  forels(ncvmx)*8, &
                                           focols(ncvmx)*29, fodbls(ncvmx)*8
!
end module uio_cvl_module


!------**************-------------------------------------------------------------------------------
module uio_nam_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_nam_module ('uio_name_module')
!
! PURPOSE:
!   Contain definition of names.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_nam_module
!
! VARIABLES:
!   fifona: ('file_form_name') character, type name for uio file format
!   uiona:  ('universal_input_output_name') character, identifier for uio file format
!   formna: ('form_name') character, keyword name for file format
!   convna: ('convert_type_name') character, keyword name for conversion type
!   versna: ('version_name') character, keyword name for uio version
!   datena: ('date_name') character, keyword name for date (and time)
!   systna: ('system_name') character, keyword name for system name
!   machna: ('machine_name') character, keyword name for machine name
!   osrena: ('operating_system_release_name') character, keyword name for
!           release of operating system
!   osvena: ('operating_system_version_name') character, keyword name for
!           version of operating system
!   hrdwna: ('hardware_name') character, keyword name for hardware description
!   langna: ('language_name') character, keyword name for source language
!   progna: ('program_name') character, keyword name for name of main program
!   lnkchr: ('link_character') character*1, link character ('&')
!   equchr: ('equivalence_character') character*1, character between keyword and its value ('=')
!   qmachr: ('quotation_mark_character') character*1, quotation mark (''')
!   realna: ('real_name') character, type name of real variables
!   intena: ('integer_name') character, type name of integer variables
!   charna: ('character_name') character, type name of character variables
!   compna: ('complex_name') character, type name of complex variables
!   tabna:  ('table_name') character, type name of tables
!   lablna: ('label_name') character, type name of labels
!   bytena: ('byte_name') character, keyword name for number of bytes
!   dimna:  ('dimension_name') character, keyword name for dimensions
!   frmtna: ('format_name') character, keyword name for format string
!   namena: ('name_name') character, keyword name for name string
!   plinna: ('per_line_name') character, keyword name for number of values per line
!           (needed for arrays in 'formatted' file format)
!   tranna: ('transformation_name') character, keyword name for transformation procedure
!   unitna: ('unit_name') character, keyword name for units
!
! MODIFICATION HISTORY:
!   1996       (B.F. Kiel) Old: include with common block: defnam
!   1997-01-13 (B.F.) Put into module
!---------------------------------------------------------------------------------------------------
implicit none
!
character, parameter                    :: versio*(*)='0.1.2017.08.05', &
                                           fifona*(*)='fileform', &
                                           uiona*(*) ='uio', &
                                           formna*(*)='form', &
                                           convna*(*)='convert', &
                                           versna*(*)='version', &
                                           datena*(*)='date', &
                                           systna*(*)='system', &
                                           machna*(*)='machine', &
                                           osrena*(*)='osrelease', &
                                           osvena*(*)='osversion', &
                                           hrdwna*(*)='hardware', &
                                           langna*(*)='language', &
                                           progna*(*)='program'
character, parameter                    :: lnkchr*(*)='&', &
                                           equchr*(*)='=', &
                                           qmachr*(*)='''', &
                                           realna*(*)='real', &
                                           intena*(*)='integer', &
                                           charna*(*)='character', &
                                           compna*(*)='complex', &
                                           tabna*(*) ='table', &
                                           lablna*(*)='label', &
                                           dimna*(*) ='d', &
                                           frmtna*(*)='f', &
                                           bytena*(*)='b', &
                                           namena*(*)='n', &
                                           unitna*(*)='u', &
                                           tranna*(*)='t', &
                                           plinna*(*)='p'
!
end module uio_nam_module


!------***************------------------------------------------------------------------------------
module uio_base_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_base_module ('uio_base_module')
!
! PURPOSE:
!   Basic set of uio-routines: string processing, header handling,
!   I/O channel management
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_base_module
!
! ROUTINES: (CONTAINED)
!   --- Initialization ---
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_chconv:            Actualize list of conversion types for all channels
!   --- Useful high level routines ---
!   uio_ppkeyw:            Print keywords with values in pretty form
!   uio_pptrmt:            Print term table in pretty form
!   uio_err:               Handle UIO errors
!   --- Keyword handling ---
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_exkeyw:            Extract value of keyword from table
!   --- Lower level routines ---
!   uio_dattim:            Get string with current date and time
!   uio_deform:            Determine the default output format for numbers
!   uio_dim2st:            Compose dimension string
!   uio_st2dim:            Parse dimension string
!   uio_memtrm:            Merge a list of terms (keywords and their values) into a line table
!   uio_me1trm:            Merge the input term: 'keyword', 'value' -> 'keyword=value'
!   uio_qmaadd:            Add quotation marks to string if necessary
!   uio_exmtrm:            Transform a list of items into its components
!   uio_ex1trm:            Extract one term from the input line
!   uio_qmadel:            Parse string "inline" and remove quotation marks if necessary
!   uio_vnanrm:            Transform a string into one that contains a correct name of a variable
!   uio_wf2rf:             Produce from write format string read format string
!
! MODIFICATION HISTORY:
!   1997-01-13 (B.F. Kiel) Routines put into module
!   2013-08-24 (B.F.) Change layout of comments
!---------------------------------------------------------------------------------------------------
private                                 :: uio_me1trm, uio_qmaadd, &
                                           uio_ex1trm, uio_qmadel, uio_vnanrm
!
contains

!----------**********-------------------------------------------------------------------------------
subroutine uio_chunit(cmdstr, nc1in,nc2in, nc1out,nc2out, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_chunit ('uio_channel_unit')
!
! PURPOSE:
!   Initialize, store and actualize a list of free and occupied unit numbers.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_chunit(cmdstr, nc1in=nc1in,nc2in=nc2in, &
!                   nc1out=nc1out,nc2out=nc2out, outstr=outstr,ierr=ierr)
!
! INPUT:
!   cmdstr: ('command_string') character, supported values:
!           'init': First initialization command: Set total except
!                   selected range (nc1in-nc2in) to 'unavailable', selected
!                   range to 'free'
!           'setrange': Set selected range (nc1in-nc2in) to 'free'
!           'resetrange': Set selected range (nc1in-nc2in) to 'unavailable'
!           'ask': Give out status of selected unit (nc1in) in outstr
!           'get': Search lowest 'free' unit, set its status to 'occupied',
!                  give out unit number in nc1out
!           'set': Set selected free unit (nc1in) to 'occupied'
!           'formatted': Set selected occupied unit (nc1in) to 'formatted'
!           'unformatted': Set selected occupied unit (nc1in) to 'unformatted'
!           'return': Reset selected unit (nc1in) to 'free'
!           'maxrange': Give out minimum (nc1out) and maximum possible (nc2out)
!                       unit number
!   nc1in:  ('number_channel_1_input') integer, optional, number of channel
!   nc2in:  ('number_channel_2_input') integer, optional, number of channel
!
! OUTPUT:
!   nc1out: ('number_channel_1_output') integer, optional, number of channel
!   nc2out: ('number_channel_2_output') integer, optional, number of channel
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_err:               Handle UIO errors
!
! MODULES:
!   uio_cst_module:        Channel status information
!
! SIDE EFFECTS:
!   Change parameters in module uio_cst_module.
!
! RESTRICTIONS:
!   Do not open channel units within the range managed by uio_chunit without
!   using this routine. Otherwise the saved state of the units may not
!   represent the current state of open/closed units.
!
! PROCEDURE:
!   According to command string cmdstr look into character array ncstat or
!   modify it. The array ncstat contains the states of all input/output channels
!   within a specified range.
!
! EXAMPLE:
!   Initialization of uio_chunit, channel number range is 10 to 80:
!     nc1=10 ; nc2=80
!     call uio_chunit('init', nc1,nc2, outstr,ierr)
!   Search and reservation of a free channel, answer in nc:
!     call uio_chunit('get',nc,nc2, outstr,ierr)
!   Open and close file
!     open(nc,'testfile')
!     close(nc)
!   Return unit number
!     call uio_chunit('return', nc, nc2, outstr,ierr)
!
! MODIFICATION HISTORY:
!   1993-02-28 (B.F. Kiel) Written
!   1993-03-08 (B.F.)
!   1995-11-11 (B.F.) More comments
!   1995-11-16 (B.F.) More comments
!   1995-11-19 (B.F.) Different use of od (change for CRAY)
!   1995-12-25 (B.F.) save -> common
!   1996-01-29 (B.F.) 'ask' first item
!   1997-01-02 (B.F.) continue -> end do
!   1997-01-20 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!---------------------------------------------------------------------------------------------------
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
character*(*),           intent(in)     :: cmdstr
integer,       optional, intent(in)     :: nc1in, nc2in
integer,       optional, intent(out)    :: nc1out, nc2out
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
logical                                 :: od, odnc
integer                                 :: n, ierr0
character*(80)                          :: outstr0
!---------------------------------------------------------------------------------------------------
ierr0=0
!
if (cmdstr == 'ask') then
  ! --- Give out status of selected unit ---
  if (nc1in < ncmin) then
    outstr0='unknown'
    ierr0=1
  else if (nc1in > ncmax) then
    outstr0='unknown'
    ierr0=2
  else
    outstr0=ncstat(nc1in)
  endif
else if (cmdstr == 'init') then
  ! --- First initialization command: Set total except selected range ---
  ! --- to 'unavailable', selected range to 'free'.                   ---
  do n=ncmin,ncmax
    ncstat(n)='unavailable'
  end do ! n
  do n=min(max(nc1in,ncmin),ncmax),min(max(nc2in,ncmin),ncmax)
    ncstat(n)='free'
  end do ! n
  outstr0='done'
else if (cmdstr == 'setrange') then
  ! --- Set selected range to 'free' ---
  do n=min(max(nc1in,ncmin),ncmax),min(max(nc2in,ncmin),ncmax)
    ncstat(n)='free'
  end do ! n
  outstr0='done'
else if (cmdstr == 'resetrange') then
  ! --- Set selected range to 'unavailable' ---
  do n=min(max(nc1in,ncmin),ncmax),min(max(nc2in,ncmin),ncmax)
    ncstat(n)='unavailable'
  end do ! n
  outstr0='done'
else if (cmdstr == 'get') then
  ! --- Search lowest 'free' unit, set its status to 'occupied', ---
  ! --- give out unit number.                                    ---
  nc1out=ncmax
  outstr0='nothing free'
  ierr0=3
  do n=ncmin,ncmax
    if (ncstat(n) == 'free') then
      ierr0=0
      nc1out=n
      outstr0=ncstat(nc1out)
      ncstat(nc1out)='occupied'
      exit
    endif
  end do ! n
else if (cmdstr == 'set') then
  ! --- Set selected unit to 'occupied' ---
  if (nc1in < ncmin) then
    outstr0='unknown'
    ierr0=1
  else if (nc1in > ncmax) then
    outstr0='unknown'
    ierr0=2
  else
    outstr0=ncstat(nc1in)
    if (ncstat(nc1in) == 'free') then
      ncstat(nc1in)='occupied'
    else
      ierr0=4
    endif
  endif
else if (cmdstr == 'formatted') then
  ! --- Set selected unit to 'formatted' ---
  if (nc1in < ncmin) then
    outstr0='unknown'
    ierr0=1
  else if (nc1in > ncmax) then
    outstr0='unknown'
    ierr0=2
  else
    outstr0=ncstat(nc1in)
    if (ncstat(nc1in) == 'occupied') then
      ncstat(nc1in)='formatted'
    else
      ierr0=4
    endif
  endif
else if (cmdstr == 'unformatted') then
  ! --- Set selected unit to 'unformatted' ---
  if (nc1in < ncmin) then
    outstr0='unknown'
    ierr0=1
  else if (nc1in > ncmax) then
    outstr0='unknown'
    ierr0=2
  else
    outstr0=ncstat(nc1in)
    if (ncstat(nc1in) == 'occupied') then
      ncstat(nc1in)='unformatted'
    else
      ierr0=4
    endif
  endif
else if (cmdstr == 'return') then
  ! --- Reset selected unit to 'free' ---
  if (nc1in < ncmin) then
    outstr0='unknown'
    ierr0=1
  else if (nc1in > ncmax) then
    outstr0='unknown'
    ierr0=2
  else
    outstr0=ncstat(nc1in)
    if ((ncstat(nc1in) == 'occupied' ) .or. &
        (ncstat(nc1in) == 'formatted') .or. &
        (ncstat(nc1in) == 'unformatted')) then
      ncstat(nc1in)='free'
    else
      ierr0=4
    endif
  endif
else if (cmdstr == 'maxrange') then
  ! --- Give out minimum and maximum possible unit number ---
  nc1out=ncmin
  nc2out=ncmax
  outstr0='range'
else if (cmdstr == 'check') then
  ! --- Check whether ncstat() matches opened and closed units ---
  do n=ncmin,ncmax
    if ((ncstat(n) == 'free'       ).or. (ncstat(n) == 'occupied'   ).or. &
        (ncstat(n) == 'formatted'  ).or. (ncstat(n) == 'unformatted')) then
      inquire(n,opened=od)
      odnc=((ncstat(n) == 'occupied'   ) .or. (ncstat(n) == 'formatted'  ) .or. &
            (ncstat(n) == 'unformatted'))
      if ((      od .and.(.not.odnc)) .or. &
          ((.not.od).and.      odnc ) ) then
        outstr0='not '//ncstat(n)
        ierr0=6
        nc1out=n
        exit
      endif
    endif
  end do ! n
  if (ierr0 == 0) outstr0='match'
else
  ! --- Command not known ---
  outstr0='pardon? I do not understand ''' // trim(cmdstr) // ''''
  ierr0=5
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) then
  if (present(nc1in)) then
    call uio_err(outstr=outstr0, ierr=ierr0, nc=nc1in, routine='uio_chunit', &
                 errlevel=3)
  else
    call uio_err(outstr=outstr0, ierr=ierr0, routine='uio_chunit', errlevel=3)
  endif
endif
!
end subroutine uio_chunit


!----------**********-------------------------------------------------------------------------------
subroutine uio_chconv(nc, conv,prefix, ouconv, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_chconv ('uio_channel_conversion')
!
! PURPOSE:
!   Actualize list of conversion types for all channels.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_chconv(nc, conv=conv,prefix=prefix, ouconv=ouconv, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, number of channel
!   conv:   ('conversion') character, optional, conversion type:
!           standard (all machines):
!           'native'  -> standard machine type
!           'default' -> default (may be 'native')
!           ''        -> default (may be 'native')
!           additional (machine dependent):
!           'ieee_4'
!           'ieee_8'
!           'crayxmp_8'
!           'crayxmp_16'
!           'idl'
!           'xdr'
!           ...
!   prefix: ('prefix') character, optional:
!           '' : Search if (conv == cvnals(?))
!           'X': Search if 'X'//cvnals(?) in conv
!
! OUTPUT:
!   ouconv: ('output_conversion') character, optional, allowed conversion type
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! LOCAL VARIABLES:
!   nconv:  ('number_conversion') character, number of conversion type
!
! ROUTINES:
!   uio_err:               Handle UIO errors
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Change parameters in module uio_cst_module.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   This routine is used while opening files for reading.
!   Look if requested conversion type is possible for current machine.
!   If it is possible, save it in array cvnum.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-27 (B.F. Kiel) Written
!   1995-12-28 (B.F.) n->nconv: output
!   1996-02-21 (B.F.) if conversion type is ' ' -> nconv=1
!   1996-10-19 (B.F.) Default conversion type ncvdef introduced:
!                   if conversion type is ' ' -> nconv=ncvdef
!   1997-01-03 (B.F.) gotos removed
!   1997-01-20 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-19 (B.F.) conv may be 'default', more comments
!   1999-05-19 (B.F.) call of uio_err removed
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*), optional, intent(in)     :: conv, prefix
character*(*), optional, intent(out)    :: ouconv, outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nconv, ierr0
logical                                 :: found
character*(lel)                         :: str
character*(80)                          :: conv0, prefix0, outstr0
!---------------------------------------------------------------------------------------------------
! --- Default values for optional parameters ---
if (present(conv)) then
  conv0=conv
else
  conv0=''
endif
if (present(prefix)) then
  prefix0=prefix
else
  prefix0=''
endif
!
! --- Look if conversion type 'conv0' is in list 'cvnals()' ---
nconv=1
if ((conv0 == '') .or. (conv0 == 'default')) then
  ! --- Take default conversion type ---
  nconv=ncvdef
  outstr0='default type'
  ierr0=0
else
  ! --- Determine conversion type ---
  if (prefix0 == '') then
    do
      found=conv0 == cvnals(nconv)
      if (found) exit
      nconv=nconv+1
      if (nconv > ncv) exit
    end do
  else
    do
      str=trim(prefix0)//cvnals(nconv)
      found=index(conv0,trim(str)) /= 0
      if (found) exit
      nconv=nconv+1
      if (nconv > ncv) exit
    end do
  endif
  !
  if (found) then
    ! --- Conversion type found ---
    outstr0='known type'
    ierr0=0
  else
    ! --- Unknown conversion type ---
    nconv=1
    outstr0='unknown type'
    ierr0=1
  endif
endif
!
cvnum(nc)=nconv
if (present(ouconv)) ouconv=cvnals(nconv)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
!!$if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
!!$                            routine='uio_chconv', errlevel=3)
!
end subroutine uio_chconv


!----------*******----------------------------------------------------------------------------------
subroutine uio_err(outstr,ierr, nc, routine, errlevel, outlevel, mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_err ('uio_error')
!
! PURPOSE:
!   Error handling.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_err(outstr=outstr,ierr=ierr, nc=nc, routine=routine, &
!                errlevel=errlevel, outlevel=outlevel, mode=mode)
!
! INPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!   nc:     ('number_channel') integer, optional, number of channel
!   routine:('routine') character, optional, name of routine where the error
!           occurred.
!   errlevel:('error_level') integer, optional
!   outlevel:('output_level') integer, optional
!   mode:   ('mode') integer, optional
!
! OUTPUT:
!   None
!
! ROUTINES:
!   uio_dattim:            Get string with current date and time
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!
! SIDE EFFECTS:
!   Output is written to channel ncerr.
!   Program may be stopped.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-01-25 (B.F. Kiel) Written
!   1997-02-05 (B.F.) More comments
!   1999-12-08 (B.F. Copenhagen) errlevel -> errlevel0, Set default ncerr=6
!   2007-02-02 (B.F. Lyon) Printing of error conversion type removed: might not be defined yet
!---------------------------------------------------------------------------------------------------
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,       optional, intent(in)     :: ierr, nc, errlevel, outlevel, mode
character*(*), optional, intent(in)     :: outstr, routine
!
! --- Local variables ---
integer, save                           :: ncerr=6, error_level=5, output_level=5
integer                                 :: errlevel0, outlevel0, mode0, ierr0, iostat
logical                                 :: opened, named
character*(80)                          :: outstr0, routine0, date0, &
                                           file, form, access, position, action
!---------------------------------------------------------------------------------------------------
! --- Default values for optional parameters ---
if (present(outstr)) then
  outstr0=outstr
else
  outstr0=''
endif
if (present(routine)) then
  routine0=routine
else
  routine0=''
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
if (present(outlevel)) then
  outlevel0=outlevel
else
  outlevel0=output_level
endif
if (present(errlevel)) then
  errlevel0=errlevel
else
  errlevel0=error_level
endif
!
if (mode0 == 1) then
  ! --- Initialization ---
  !
  ! --- Channel for output of error messages ---
  if (present(nc)) then
    ncerr=nc
  else
    ncerr=6
  endif
  ! --- Severeness level for errors to cause an interrupt ---
  error_level=errlevel0
  ! --- Output quantity ---
  output_level=outlevel0
  !
else
  ! --- Error handling ---
  !
  ! --- Error message ---
  if (outlevel0 >= 1) then
    if (errlevel0 <= 0) then
      write(ncerr,'(A)',advance='no') 'I''m'
    else
      write(ncerr,'(A)',advance='no') 'Error'
    endif
    if (routine0 /= '') write(ncerr,'(A)',advance='no') ' in routine ' // &
                             trim(routine0)
    if (present(nc)) then
      write(ncerr,'(A,I3)',advance='no') ' on channel ', nc
    endif
    write(ncerr,'(A)') ':'
    if (present(ierr)) write(ncerr,'(A,I6)') 'Error code (ierr): ', ierr
    if (outstr0 /= '') write(ncerr,'(A)') trim(outstr0)
    if (present(nc) .and. (outlevel0 >= 5)) then
      write(ncerr,'(A,I11)') 'record no=', ncpos(nc)
      call uio_chunit('ask',nc1in=nc,outstr=form,ierr=ierr0)
      if (ierr0 == 0) then
        write(ncerr,'(A)') 'Channel status (uio_chunit):  ' // trim(form)
        !if ((nc >= ncmin) .and. (nc <= ncmax)) write(ncerr,'(A)') &
        !  'Conversion type (uio_chconv): ' // trim(cvnals(cvnum(nc)))
      endif
      ! --- Inquire information about unit ---
      inquire(nc, iostat=iostat, named=named, name=file, opened=opened, &
                  form=form, access=access, position=position, action=action)
      if ((iostat == 0) .and. (opened) .and. (named)) then
        write(ncerr,'(A,I3,A)') 'inquire(nc=', nc, '):'
        write(ncerr,'(A)') 'filename = ' // trim(file)
        write(ncerr,'(A)') 'form     = ' // trim(form)
        write(ncerr,'(A)') 'access   = ' // trim(access)
        write(ncerr,'(A)') 'position = ' // trim(position)
        write(ncerr,'(A)') 'action   = ' // trim(action)
      endif
    endif
    if (outlevel0 >= 7) then
      call uio_dattim(date0)
      write(ncerr,'(A)') 'Date: ' // trim(date0)
    endif
  endif
  !
  ! --- Interrupt ---
  if (errlevel0 >= error_level) stop
endif
!
end subroutine uio_err


!----------**********-------------------------------------------------------------------------------
subroutine uio_adkeyw(termt,ntt,keywrd,keyval,keydef, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_adkeyw ('uio_add_keyword')
!
! PURPOSE:
!   Add one keyword to term table, keyword-value=keyval or default:keydef
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_adkeyw(termt,ntt,keywrd, keyval=keyval,keydef=keydef, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   keywrd: ('keyword') character, single keyword to be added
!   keyval: ('keyword_value') character, optional, value associated with keyword
!   keydef: ('keyword_default') character, optional, default value if keyval is
!           empty
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Add scalar keyword with value to term table:
!   If no keyword value is specified, add only keyword name.
!   If keyword value is specified and non-empty, add keyword name and value.
!   If keyword value is specified and empty, look if default value is specified:
!     If default value is specified (empty or non-empty), add keyword name and
!       default value.
!     If default value is not specified, ignore keyword and don't alter table.
!
! EXAMPLE:
!                        (1)                           (2)
!   termt:  |real                         |                            |
!   keywrd: |temp                         |
!    --->
!   termt:  |real                         |                            |
!           |temp                         |                            |
!                        (1)                           (2)
!   termt:  |real                         |                            |
!           |temp                         |                            |
!   keywrd: |name                         |
!   keyval: |temperature                  |
!     --->
!   termt:  |real                         |                            |
!           |temp                         |                            |
!           |name                         |temperature                 |
!   ,
!   keywrd: |unit                         |
!   keyval: |                             |
!     --->
!   termt:  |real                         |                            |
!           |temp                         |                            |
!           |name                         |temperature                 |
!                        (1)                           (2)
!   termt:  |real                         |                            |
!           |temp                         |                            |
!           |name                         |temperature                 |
!   keywrd: |format                       |
!   keyval: |                             |
!   keydef: |F7.4                         |
!     --->
!   termt:  |real                         |                            |
!           |temp                         |                            |
!           |name                         |temperature                 |
!           |format                       |F7.4                        |
!
! MODIFICATION HISTORY:
!   1995-12-07 (B.F. Kiel) Written
!   1996-01-14 (B.F.) adkey3 -> uio_adkeyw; parameters keyval,keydef, outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(inout)  :: ntt
character*(*),           intent(inout)  :: termt(2,nttmx)
character*(*),           intent(in)     :: keywrd
character*(*), optional, intent(in)     :: keyval, keydef
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
if (ntt < nttmx) then
  ! --- There is still space in table left ---
  if (present(keyval)) then
    ! --- Value for keyword is specified ---
    if (present(keydef)) then
      ! --- Default value for keyword is specified: add keyword anyway ---
      ntt=ntt+1
      termt(1,ntt)=keywrd
      if (keyval /= '') then
        ! --- Take keyword value ---
        termt(2,ntt)=keyval
        outstr0='done'
      else
        ! --- Take default ---
        termt(2,ntt)=keydef
        outstr0='default is taken'
      endif
    else
      ! --- No default value for keyword is specified ---
      if (keyval /= '') then
        ! --- Keyword value is non-empty, add keyword+value to table --- 
        ntt=ntt+1
        termt(1,ntt)=keywrd
        termt(2,ntt)=keyval
        outstr0='done'
      else
        ! --- Keyword value is empty, ignore keyword --- 
        outstr0='keyword value is empty'
      endif
    endif
  else
    ! --- No value for keyword is specified, add keyword anyway ---
    ntt=ntt+1
    termt(1,ntt)=keywrd
    termt(2,ntt)=''
    outstr0='no value for keyword'
  endif
  ierr0=0
else
  ! --- Table is already full ---
  outstr0='table is full'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_adkeyw


!----------**********-------------------------------------------------------------------------------
subroutine uio_exkeyw(termt,ntt, keywrd,keyval, mode, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_exkeyw ('uio_extract_keyword')
!
! PURPOSE:
!   Extract value of keyword from table.
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_exkeyw(termt,ntt, keywrd,keyval, mode=mode, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   keywrd: ('keyword') character, keyword to be searched for
!   mode:   ('mode') integer, optional, search mode:
!           0X: start search form 3. element
!           1X: start search from 1. element
!            0: search only (default)
!            1: search and delete (leave blank)
!            2: search, delete, and shift
!            3: search, delete, and reconstruct
!            4: reconstruct only
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   keyval: ('keyword_value') character, value of found keyword, otherwise empty
!   outstr: ('output_string') character, optional, '' or 'found' or 'not found'
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Search for keyword keywrd in term table termt. If found, put the
!   corresponding value in keyval. Remove keyword from table according to mode.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-06 (B.F. Kiel) Written
!   1995-12-08 (B.F.) Parameter mode added
!   1996-01-08 (B.F.) mode=3, reconstruction also if keyword not found
!   1996-02-01 (B.F.) Start point of search
!   1996-10-14 (B.F.) More comments
!   1997-01-06 (B.F.) gotos removed
!   1997-01-14 (B.F.) mode, outstr,ierr made optional
!   1997-01-17 (B.F.) mode=4 added
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(inout)  :: ntt
character*(*),           intent(in)     :: keywrd
character*(*),           intent(inout)  :: termt(2,nttmx)
character*(*),           intent(out)    :: keyval
integer,       optional, intent(in)     :: mode
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: n, n1, n2, nttstt, mode0, mode1
integer                                 :: ierr0
character*(80)                          :: outstr0
!---------------------------------------------------------------------------------------------------
! --- Default values for optional parameter ---
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif  
!
ierr0=0
!
! --- Set start point of search ---
if (mode0 >= 10) then
  nttstt=1
  mode1=mode0-10
else
  nttstt=3
  mode1=mode0
endif
!
outstr0=''
! --- Search in table until end or keyword found ---
if (mode1 <= 3) then
 do n=nttstt,ntt
  if (termt(1,n) == keywrd) then
    ! --- Keyword found ---
    keyval=termt(2,n)
    outstr0='found'
    ! --- Delete keyword from table ---
    if (mode1 >= 1) termt(1,n)=''
    if (mode1 == 2) then
      ! --- Shift following keywords ---
      do n1=n,ntt-1
        termt(1,n1)=termt(1,n1+1)
        termt(2,n1)=termt(2,n1+1)
      end do ! n1
      ntt=ntt-1
    endif
    exit
  endif
 end do ! n
 !
 ! --- Keyword not found ---
 if (outstr0 /= 'found') then
   keyval=''
   outstr0='not found'
  endif
endif
!
! --- Reconstruct complete table ---
if (mode1 >= 3) then
  n1=1
  do n2=1,ntt
    if (termt(1,n2) /= '') then
      termt(1,n1)=termt(1,n2)
      termt(2,n1)=termt(2,n2)
      n1=n1+1
    else
      ntt=ntt-1
    endif
  end do ! n2
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_exkeyw


!----------**********-------------------------------------------------------------------------------
subroutine uio_dim2st(ilow, iup, dimstr, ilbound, ishift, mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_dim2st ('uio_dimensions_to_string')
!
! PURPOSE:
!   Make dimension string from integer dimension specifications.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_dim2st(ilow, iup, dimstr, ilbound=ilbound, ishift=ishift, &
!                   mode=mode)
!
! INPUT:
!   ilow:   ('index_low') integer vector, lower indices
!   iup:    ('index_up') integer vector, upper indices
!   ilbound:('index_lower_boundary') integer vector, optional
!   ishift: ('index_shift') integer vector, optional
!   mode:   ('mode') integer, optional, shift mode:
!               imin           imax
!           0:  ilow           iup               (default: no shift)
!           1:  0              iup-ilow
!           2:  1              iup-ilow+1
!           3:  0              iup-ilow
!           4:  1              iup-ilow+1
!           5:  ilow-ilbound   iup-ilbound
!           6:  ilow-ilbound+1 iup-ilbound+1
!           7:  ilow-ishift    iup-ishift
!           8:  ishift         ishift+iup-ilow
!
! OUTPUT:
!   dimstr: ('dimension_string') character, string with dimensions '(imin:imax)'
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Put integer vectors with dimension specifications into single string
!   '(imin1:imax1,imin2:imax2,...)'.
!   Indices are shifted/modified according to mode:
!     mode=0 uses the precise indices.
!     mode=1,3 use only the index range, start=0.
!     mode=2,4 use only the index range, start=1.
!     mode=5,6,7,8 are for special purposes only.
!   The inverse routine is uio_st2dim.
!
! EXAMPLE:
!   ilow=    (/  4,  6,  8 /)
!   iup=     (/ 14, 16, 18 /)
!   ilbound= (/  2,  2, -2 /)
!   ishift=  (/  9, 11, 13 /)
!   mode=0   ->   dimstr='(4:14,6:16,8:18)'
!   mode=1   ->   dimstr='(0:10,0:10,0:10)'
!   mode=2   ->   dimstr='(1:11,1:11,1:11)'
!   mode=3   ->   dimstr='(0:10,0:10,0:10)'         same as mode=1
!   mode=4   ->   dimstr='(1:11,1:11,1:11)'         same as mode=2
!   mode=5   ->   dimstr='(2:12,4:14,10:20)'
!   mode=6   ->   dimstr='(2:12,2:12,-2:8)'
!   mode=7   ->   dimstr='(-5:5,-5:5,-5:5)'
!   mode=8   ->   dimstr='(9:19,11:21,13:23)'
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1997-01-14 (B.F.) Slightly shorter
!   1997-01-21 (B.F.) vector arithmetic
!   1997-01-22 (B.F.) Mode modes
!   1997-11-27 (B.F.) dimension(1:) -> dimension(:)
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
integer, dimension(:), intent(in)       :: ilow, iup
character*(*),          intent(out)     :: dimstr
integer, dimension(:), &
              optional, intent(in)      :: ilbound, ishift
integer,      optional, intent(in)      :: mode
!
! --- Local variables ---
integer                                 :: ndim, n, mode0
integer, dimension(1:size(ilow,1))      :: imin, imax
character                               :: dimmin*(11), dimmax*(11), sep*(1)
!---------------------------------------------------------------------------------------------------
ndim=size(ilow,1)
!
! --- Default values for optional parameters ---
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
!
! --- Shift index range according to mode ---
select case(mode0)
  case(0)                      ! --- No shift ---
    imin=ilow
    imax=iup
  case(1,3)                    ! --- Shift: imin -> 0 ---
    imin=0
    imax=iup-ilow
  case(2,4)                    ! --- Shift: imax -> 1 ---
    imin=1
    imax=iup-ilow+1
  case(5)                      ! --- Shift: use distance to lower boundary ---
    if (present(ilbound)) then
      imin=ilow-ilbound
      imax=iup-ilbound
    else
      imin=ilow
      imax=iup
    endif
  case(6)                      ! --- Shift: use distance to lower boundary ---
    if (present(ilbound)) then
      imin=ilow-ilbound+1
      imax=iup-ilbound+1
    else
      imin=ilow
      imax=iup
    endif
  case(7)                      ! --- Shift by ishift ---
    if (present(ishift)) then
      imin=ilow-ishift
      imax=iup-ishift
    else
      imin=ilow
      imax=iup
    endif
  case(8)                      ! --- Shift to ishift ---
    if (present(ishift)) then
      imin=ishift
      imax=ishift+iup-ilow
    else
      imin=ilow
      imax=iup
    endif
  case default                 ! --- Default: no shift ---
    imin=ilow
    imax=iup
end select ! mode0
!
sep='('
dimstr=''
!
! --- Merge indices into single string ---
do n=1,ndim
  write(dimmin,'(I11)') imin(n)
  write(dimmax,'(I11)') imax(n)
  dimstr=trim(dimstr) // sep // trim(adjustl(dimmin)) // ':' // adjustl(dimmax)
  sep=','
end do ! n
!
dimstr=trim(dimstr) // ')'
!
end subroutine uio_dim2st


!----------**********-------------------------------------------------------------------------------
subroutine uio_st2dim(dimstr, ilow, iup, ilbound, iubound, ishift, mode, ndim)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_st2dim ('uio_string_to_dimensions')
!
! PURPOSE:
!   Analyze and parse dimension string.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_st2dim(dimstr, ilow, iup, ilbound=ilbound, iubound=iubound, &
!                   ishift=ishift, mode=mode, ndim=ndim)
!
! INPUT:
!   dimstr: ('dimension_string') character, string with dimensions '(imin:imax)'
!   ilbound:('index_lower_boundary') integer vector, optional
!   iubound:('index_upper_boundary') integer vector, optional
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!               ilow              iup
!           0:  imin              imax               (default: no shift)
!           1:  ilbound           ilbound+imax-imin
!           2:  ilbound           ilbound+imax-imin
!           3:  iubound-imax+imin iubound
!           4:  iubound-imax+imin iubound
!           5:  ilbound+imin      ilbound+imax
!           6:  ilbound+imin-1    ilbound+imax-1
!           7:  imin-ishift       imax-ishift
!           8:  ishift            ishift+imax-imin
!
! OUTPUT:
!   ilow:   ('index_low') integer vector, lower indices
!   iup:    ('index_up') integer vector, upper indices
!   ndim:   ('number_dimension') integer, optional, number of dimensions
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   A string with dimensional information
!     '(AA:BB,CC:DD,EE:FF,GG:HH)'
!   is parsed into its components
!     (AA, CC, EE, GG) and
!     (BB, DD, FF, HH).
!   The start point of the dimension values may be shifted/modified with the
!   mode parameter.
!   uio_st2dim is the inverse of uio_dim2st.
!
! EXAMPLES:
!   dimstr= '(4:14,6:16,8:18)'
!   ilbound= (/  2,  2, -2 /)
!   iubound= (/ 50, 60, 70 /)
!   ishift=  (/  9, 11, 13 /)
!   mode=0   ->   ilow=(/  4,  6,  8 /)   iup=(/ 14, 16, 18 /)
!   mode=1   ->   ilow=(/  2,  2, -2 /)   iup=(/ 12, 12,  8 /)
!   mode=2   ->   ilow=(/  2,  2, -2 /)   iup=(/ 12, 12,  8 /)   same as mode=1
!   mode=3   ->   ilow=(/ 40, 50, 60 /)   iup=(/ 50, 60, 70 /)
!   mode=4   ->   ilow=(/ 40, 50, 60 /)   iup=(/ 50, 60, 70 /)   same as mode=3
!   mode=5   ->   ilow=(/  6,  8,  6 /)   iup=(/ 16, 18, 16 /)
!   mode=6   ->   ilow=(/  5,  7,  5 /)   iup=(/ 15, 17, 15 /)
!   mode=7   ->   ilow=(/ 13, 17, 21 /)   iup=(/ 23, 27, 31 /)
!   mode=8   ->   ilow=(/  9, 11, 13 /)   iup=(/ 19, 21, 23 /)
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1997-01-06 (B.F.) goto removed
!   1997-01-21 (B.F.) Indices as vectors
!   1997-01-22 (B.F.) Parameter ishift added, more modes
!   1997-01-25 (B.F.) number of dimensions of dimstr may be lower than that of ilow
!   1997-11-27 (B.F.) dimension(1:) -> dimension(:)
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
character*(*),          intent(in)      :: dimstr
integer, dimension(:), intent(out)      :: ilow, iup
integer, dimension(:), &
              optional, intent(in)      :: ilbound, iubound, ishift
integer,      optional, intent(in)      :: mode
integer,      optional, intent(out)     :: ndim
!
! --- Local variables ---
integer                                 :: mode0, lasti, comma, colon, n, &
                                           ndim0, ndimmax
character*(let)                         :: dimhlp, term, ind1, ind2
integer, dimension(1:7)                 :: imin, imax
!---------------------------------------------------------------------------------------------------
ndimmax=size(ilow,1)
!
! --- Default values for optional parameters ---
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
!
dimhlp=dimstr
! --- Remove brackets ---
if (dimhlp(1:1) == '(') dimhlp=dimhlp(2:)
lasti=len_trim(dimhlp)
if (dimhlp(lasti:lasti) == ')') dimhlp=dimhlp(:lasti-1)
!
! --- Extract indices ---
ndim0=0
do
  ndim0=ndim0+1
  ! --- Search next comma ---
  comma=index(dimhlp,',')
  if (comma > 0) then
    term=dimhlp(:comma-1)
    dimhlp=dimhlp(comma+1:)
  else
    term=dimhlp
  endif
  ! --- Search next colon ---
  colon=index(term,':')
  if (colon == 0) then
    ind1='1'
    ind2=term
  else
    ind1=term(:colon-1)
    ind2=term(colon+1:)
  endif
  read(ind1,'(I11)') imin(ndim0)
  read(ind2,'(I11)') imax(ndim0)
  if (comma <= 0) exit
end do
!
if (ndimmax < ndim0) then
  ! --- Reduce number of dimensions and blow up last dimension ---
  do n=ndimmax+1,ndim0
    imax(ndimmax)=imin(ndimmax)-1 + (imax(ndimmax)-imin(ndimmax)+1) * &
                                    (imax(n)-imin(n)+1)
  end do ! n
  ndim0=ndimmax
else if (ndimmax < ndim0) then
  ! --- Fill up non-specified dimensions with 1s ---
  imin(ndim0+1:ndimmax)=1
  imax(ndim0+1:ndimmax)=1
endif
!
! --- Shift index range according to mode ---
select case(mode0)
  case(0)                      ! --- No shift ---
    ilow=imin(1:ndimmax)
    iup =imax(1:ndimmax)
  case(1,2)                    ! --- Shift to lower boundary ---
    if (present(ilbound)) then
      ilow=ilbound
      iup =ilbound+imax(1:ndimmax)-imin(1:ndimmax)
    else
      ilow=imin(1:ndimmax)
      iup =imax(1:ndimmax)
    endif
  case(3,4)                    ! --- Shift to upper boundary ---
    if (present(iubound)) then
      ilow=iubound+imin(1:ndimmax)-imax(1:ndimmax)
      iup= iubound
    else
      ilow=imin(1:ndimmax)
      iup =imax(1:ndimmax)
    endif
  case(5)                      ! --- Shift relative to actual lower boundary --
    if (present(ilbound)) then
      ilow=ilbound+imin(1:ndimmax)
      iup =ilbound+imax(1:ndimmax)
    else
      ilow=imin(1:ndimmax)
      iup =imax(1:ndimmax)
    endif
  case(6)                      ! --- Shift relative to actual lower boundary --
    if (present(ilbound)) then
      ilow=ilbound+imin(1:ndimmax)-1
      iup =ilbound+imax(1:ndimmax)-1
    else
      ilow=imin(1:ndimmax)
      iup =imax(1:ndimmax)
    endif
  case(7)                      ! --- Shift according to ishift ---
    if (present(ishift)) then
      ilow=ishift+imin(1:ndimmax)
      iup =ishift+imax(1:ndimmax)
    else
      ilow=imin(1:ndimmax)
      iup =imax(1:ndimmax)
    endif
  case(8)                      ! --- Shift to ishift ---
    if (present(ishift)) then
      ilow=ishift
      iup =ishift+imax(1:ndimmax)-imin(1:ndimmax)
    else
      ilow=imin(1:ndimmax)
      iup =imax(1:ndimmax)
    endif
  case default                 ! --- Default: no shift ---
    ilow=imin(1:ndimmax)
    iup =imax(1:ndimmax)
end select ! mode0
!
if (present(ndim)) ndim=ndim0
!
end subroutine uio_st2dim


!----------**********-------------------------------------------------------------------------------
subroutine uio_memtrm(termt,ntt, linet,nlt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_memtrm ('uio_merge_multiple_terms')
!
! PURPOSE:
!   Merge a list of terms into a line table.
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_memtrm(termt,ntt, linet,nlt, outstr,ierr)
!
! INPUT:
!   None
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   linet:  ('line_table') string array (nltmx), nltmx lines
!   nlt:    ('number_line_table') integer, number of non-empty lines of linet
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_me1trm:            Merge the input term: 'keyword', 'value' -> 'keyword=value'
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   File pointer has to be positioned properly on beginning of entry header.

! PROCEDURE:
!   Concatenate keywords and their values (if any) in termt
!   and append them to line table linet.
!
! EXAMPLE:
!   linet:  |real                                                       |  (1)
!     +                  (1)                           (2)
!   termt:  |rho                          |                             |  (1)
!           |name                         |density                      |  (2)
!           |unit                         |g/cm**3                      |  (3)
!           |c1                           |This is a very long comment. |  (4)
!    --->
!   linet:  |real rho name=density unit=g/cm**3 &                       |  (1)
!           |c1='This is a very long comment.'                          |  (2)
!
! MODIFICATION HISTORY:
!   1993-03    (B.F. Kiel) Started writing
!   1995-11-10 (B.F.) More comments
!   1995-11-11 (B.F.) Written
!   1995-11-16 (B.F.) Only one blank in front of lnkchr
!   1995-11-22 (B.F.) nlt starts with 1
!   1995-11-23 (B.F.) idx0 instead of idx
!   1995-12-06 (B.F.) 'uiosiz.inc' included
!   1995-12-07 (B.F.) details, allowed call: (linet...linet)
!   1995-12-27 (B.F.) lenmax=lel-3 (not -2)
!   1996-02-10 (B.F.) More comments
!   1996-10-14 (B.F.) linet is input/output variable, more comments
!   1996-10-28 (B.F.) Renaming
!   1997-01-03 (B.F.) goto removed
!   1997-02-10 (B.F.) nlt set to 1 at start
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(in)               :: ntt
integer,       intent(out)              :: nlt, ierr
character*(*), intent(in)               :: termt(2,nttmx)
character*(*), intent(out)              :: linet(nltmx), outstr
!
! --- Local variables ---
integer                                 :: lenmax, lenoul, lentab, n
character                               :: ouline*(lel)
!---------------------------------------------------------------------------------------------------
ierr=0
outstr='done'
lenmax=lel-3
!
n=1
nlt=1
linet(nlt)=''
!
! --- Begin loop ---
do while ((n <= ntt) .and. (nlt <= nltmx))
  ! --- Merge next keyword-value-pair in term table ---
  call uio_me1trm(termt(1:2,n), ouline, outstr,ierr)
  lenoul=len_trim(ouline)
  lentab=len_trim(linet(nlt))
  ! --- Does new string fit in old line? ---
  if ((lenoul+lentab) > lenmax) then
    ! --- New string does not fit in old line, start new one ---
    if (nlt < nltmx) then
      ! --- Add link character to old line, start new line ---
      linet(nlt)(lentab+2:)=lnkchr
      nlt=nlt+1
      linet(nlt)=ouline
    else
      ! --- Line table is already full ---
      outstr='table is full'
      ierr=1
    endif
  else if (lentab == 0) then
    ! --- New string at beginning of empty line ---
    linet(nlt)=ouline
  else
    ! --- Append new string on old line with single blank ---
    linet(nlt)(lentab+2:)=ouline
  endif
  n=n+1
end do
!
end subroutine uio_memtrm


!----------**********-------------------------------------------------------------------------------
subroutine uio_me1trm(interm, ouline, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_me1trm ('uio_merge_1_term')
!
! PURPOSE:
!   Merge the input term: 'keyword', 'value' -> 'keyword=value'
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_me1trm(interm, ouline,outstr,ierr)
!
! INPUT:
!   interm: ('input_term') character*(*)(2) containing keyword (and value)
!
! OUTPUT:
!   ouline: ('output_line') character, merged keyword, '=', and value
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_qmaadd:            Add quotation marks to string if necessary
!   uio_vnanrm:            Transform a string into one that contains a correct name of a variable
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The first element of interm gives the keyword name, the second element its
!   value. They are merged into a single string ouline.
!   Special handling for blanks and quotations marks.
!
! EXAMPLES:
!   interm(1)    interm(2)   ->    ouline
!   name         density           name=density
!   name                           name
!                density
!   greetings    hello hello       greetings='hello hello'
!   state        I'm tired         state='I''m tired'
!
! MODIFICATION HISTORY:
!   1993-03-13 (B.F. Kiel) Written
!   1995-12-06 (B.F.) 'uiosiz.inc' included
!   1996-02-10 (B.F.) Local variable lenoul removed; (interm(1) eq ' ') -> ouline=' '
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(out)              :: ierr
character*(*), intent(in)               :: interm(2)
character*(*), intent(out)              :: ouline, outstr
!
! --- Local variables ---
integer                                 :: lenou2, idxoul
character*(lel)                         :: remain
!---------------------------------------------------------------------------------------------------
ierr=0
!
lenou2=lel-2
if (interm(1) == '') then
    ! if (interm(2) == '') then
    ! --- Everything empty ---
    ouline=''
    outstr='empty'
else
  ouline=''
  ! --- Make correct name of variable ---
  call uio_vnanrm(interm(1), ouline(:lenou2),remain,outstr,ierr)
  if (interm(2) == '') then
    outstr='just name'
  else
    idxoul=len_trim(ouline)
    if (idxoul >= lenou2) then
      outstr='2. term too long'
      ierr=1
    else
      idxoul=idxoul+1
      ouline(idxoul:idxoul)=equchr
      if (idxoul >= lenou2) then
        outstr='2. term too long'
        ierr=1
      else
        ! --- Add quotation marks to keyword value if necessary ---
        call uio_qmaadd(interm(2), ouline(idxoul+1:lenou2), remain, outstr,ierr)
        if (remain == '') then
          outstr='done'
        else
          outstr='2. term too long remain:' // remain
          ierr=1
        endif
      endif
    endif
  endif
endif
!
end subroutine uio_me1trm


!----------**********-------------------------------------------------------------------------------
subroutine uio_qmaadd(inline, ouline,remain, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_qmaadd ('uio_quotation_mark_add')
!
! PURPOSE:
!   Transform a string "inline" into a string "ouline" with quotation marks if necessary.
!
! CATEGORY:
!   String processing
!
! CALLING SEQUENCE:
!   call uio_qmaadd(inline, ouline,remain,outstr,ierr)
!
! INPUT:
!   inline: ('input_line') character
!
! OUTPUT:
!   ouline: ('output_line') character
!   remain: ('remainder') character
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Leading and trailing blanks are removed. If the remaining string contains
!   any blanks, the string is included in quotation marks and
!   inner quotation marks are doubled.
!
! EXAMPLE:
!    inline:                   ouline            remain
!    tired             --->    tired
!    very tired        --->    'very tired'
!    I'm tired.        --->    'I''m tired.'
!    I'm               --->    I'm           ? 'I''m'
!   The rest of 'inline' which does not fit in 'ouline' is given out in
!   'remain'.
!
! MODIFICATION HISTORY:
!   1993-03-13 (B.F. Kiel) Written
!   1995-11-10 (B.F.) More comments
!   1997-01-06 (B.F.) goto removed
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(out)              :: ierr
character*(*), intent(in)               :: inline
character*(*), intent(out)              :: ouline, remain
character*(*), intent(out)              :: outstr
!
! --- Local variables ---
integer                                 :: iin, iout, lenoul, idxinl
!---------------------------------------------------------------------------------------------------
ierr=0
!
if (inline == '') then
  ! --- String is empty ---
  ouline=inline
  remain=''
  outstr='empty'
else
  ! --- Non-empty string ---
  idxinl=len_trim(inline)
  lenoul=len(ouline)
  if ((index(inline(:idxinl),' ') == 0) .and. (index(inline,qmachr) == 0)) then
    ! --- String without included blanks and without quotation marks ---
    ouline=inline
    if (lenoul >= idxinl) then
      remain=''
      outstr='just copied'
    else
      remain=inline(lenoul+1:)
      outstr='rest cut'
    endif
  else if (lenoul <= 2) then
    ouline=''
    remain=inline
    outstr='output string only one or two chars'
  else
    ! --- String needs quotation marks ---
    ! --- Transform to string with quotation marks ---
    iin=0
    iout=1
    ouline=qmachr
    do
      ! --- Increment counters, analyze next character ---
      iin=iin+1
      iout=iout+1
      if (iin > idxinl) then
        ! --- End of input string reached ---
        ouline(iout:iout)=qmachr
        remain=''
        outstr='done'
        exit
      else if (iout == lenoul-1) then
        ! --- End of output string nearly reached ---
        if (inline(iin:iin) == qmachr) then
          ! --- Input string too long ---
          ouline(iout:iout)=qmachr
          remain=inline(iin:)
          outstr='too long, end with '//qmachr
        else
          ouline(iout:iout+1)=inline(iin:iin)//qmachr
          if (iin < idxinl) then
            remain=inline(iin+1:)
            outstr='too long'
          else
            remain=''
            outstr='done'
          endif
        endif
        exit
      else if (inline(iin:iin) == qmachr) then
        ! --- Quotation mark found ---
        ouline(iout:iout+1)=qmachr // qmachr
        iout=iout+1
        if (iout >= lenoul-1) then
          ! --- End of ouline nearly reached ---
          ouline(iout+1:iout+1)=qmachr
          if (iin > idxinl-1) then
            remain=''
            outstr='done'
          else
            remain=inline(iin+1:)
            outstr='cut'
          endif
          exit
        endif
      else
        ! --- Just ordinary character: copy it ---
        ouline(iout:iout)=inline(iin:iin)
      endif
    end do
  endif
endif
!
end subroutine uio_qmaadd


!----------**********-------------------------------------------------------------------------------
subroutine uio_exmtrm(linet,nlt, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_exmtrm ('uio_extract_multiple_terms')
!
! PURPOSE:
!   Transform a list of items into its components.
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_exmtrm(linet,nlt, termt,ntt, outstr,ierr)
!
! INPUT:
!   linet:  ('line_table') string array, nltmx lines
!   nlt:    ('number_line_table') integer, number of non-empty lines of linet
!
! OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Parse all lines in linet, extract all terms (keyword (+value)), result in termt.
!
! EXAMPLE:
!                         (1)
!   linet:  |real rho &                                                 |  (1)
!           | name=density unit='g/cm**3'                               |  (2)
!    --->
!                         (1)                          (2)
!   termt:  |real                         |                             |  (1)
!           |rho                          |                             |  (2)
!           |name                         |density                      |  (3)
!           |unit                         |g/cm**3                      |  (4)
!
! MODIFICATION HISTORY:
!   1993-03-09 (B.F. Kiel) Written
!   1995-11-10 (B.F.) More comments
!   1995-12-06 (B.F.) 'uiosiz.inc' included; dimension of linet changed; max -> min
!   1996-03-15 (B.F.) line of uio_ex1trm ouline 1 string
!   1996-10-28 (B.F.) Renaming
!   1996-12-05 (B.F.) Don't destroy linetab
!   1997-01-06 (B.F.) gotos removed
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(in)               :: nlt
integer,       intent(out)              :: ntt,ierr
character*(*), intent(in)               :: linet(nltmx)
character*(*), intent(out)              :: termt(2,nttmx), outstr
!
! --- Local variables ---
integer                                 :: n, nltt
character*(lel)                         :: linett(nltmx)
character*(let)                         :: line
!---------------------------------------------------------------------------------------------------
ierr=0
!
! --- Copy input table into scratch table ---
nltt=min(nlt,nltmx)
do n=1,nlt
  linett(n)=linet(n)
end do ! n
!
ntt=1
! --- Loop over all lines in line table ---
nloop: do n=1,nltt
  ! --- Loop over all entries of current line ---
  loop: do
    line=linett(n)
    call uio_ex1trm(line, termt(1:2,ntt),linett(n), outstr,ierr)
    ierr=0
    if (termt(1,ntt) /= '') then
      if (ntt == nttmx) then
        outstr='output list full'
        ierr=1
        exit nloop
      else
        ntt=ntt+1
      endif
    else
      exit loop
    endif
  end do loop
end do nloop ! n
!
if (ierr == 0) then
  ntt=ntt-1
  outstr='done'
endif
!
end subroutine uio_exmtrm


!----------**********-------------------------------------------------------------------------------
subroutine uio_ex1trm(inline, outerm,ouline, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_ex1trm ('uio_extract_1_term')
!
! PURPOSE:
!   Extract one term from the input line.
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_ex1trm(inline, outerm,ouline, outstr,ierr)
!
! INPUT:
!   inline: ('input_line') character*(*), line to be parsed
!
! OUTPUT:
!   outerm: ('output_term') character*(*)(2), first term: keyword(+value)
!   ouline: ('output_line') character*(*), rest of inline
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_qmadel:            Parse string "inline" and remove quotation marks if necessary
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Search first term in inline, remove it from inline
!   and put it in separated form (keyword (+value)) in outerm.
!
! EXAMPLE:
!                         (1)
!   inline: | name=density  unit=g/cm**3                                |
!    --->
!                         (1)                         (2)
!   outerm: |name                         |density                      |
!                         (1)
!   ouline: |  unit=g/cm**3                                             |
!
! MODIFICATION HISTORY:
!   1995-03-09 (B.F. Kiel) Written
!   1996-03-15 (B.F.) inline, ouline scalar strings
!   1996-10-14 (B.F.) More comments
!   1997-01-06 (B.F.) Some gotos removed
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(out)              :: ierr
character*(*), intent(in)               :: inline
character*(*), intent(out)              :: outerm(2), ouline, outstr
!
! --- Local variables ---
integer                                 :: i1, i2, lenlin
!---------------------------------------------------------------------------------------------------
ierr=0
!
if (inline == '') then
  ! --- Input line is empty ---
  outerm(1)=''
  outerm(2)=''
  ouline=''
  outstr='empty'
else
  ! --- Start extraction ---
  lenlin=len(inline)
  ! --- Skip blanks and link-characters ---
  i1=verify(inline,' '//lnkchr)
  if (i1 == 0) then
    outerm(1)=''
    outerm(2)=''
    ouline=''
    outstr='only '//lnkchr
    goto 900
  endif
  !
  ! --- Search separator equchr ---
  i2=scan(inline(i1:lenlin),' '//equchr)
  if (i2 == 0) then
    outerm(1)=inline(i1:lenlin)
    outerm(2)=''
    ouline=''
    outstr='just name'
    goto 900
  else
    i2=i2+i1-1
  endif
  !
  ! --- End of lhs ---
  if (i1 == i2) then
    ! --- No lhs ---
    outerm(1)=''
    outerm(2)=''
    if (i2 < lenlin) then
      ouline=inline(i2+1:)
    else
      ouline=''
    endif
    outstr='start with '//equchr
    ierr=1
  else if (inline(i2:i2) == ' ') then
    ! --- No link character ---
    outerm(1)=inline(i1:i2-1)
    outerm(2)=''
    ouline=inline(i2:)
    outstr='just name'
    goto 900
  else
    ! --- Find rhs ---
    outerm(1)=inline(i1:i2-1)
    call uio_qmadel(inline(i2+1:),outerm(2),ouline, outstr,ierr)
    if (ierr == 0) then
      outstr='done'
    else
      outstr='ex1trm:qmadel: ' // outstr
      ierr=2
    endif
  endif
endif
!
900 continue
!
end subroutine uio_ex1trm


!----------**********-------------------------------------------------------------------------------
subroutine uio_qmadel(inline, ouline,remain, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_qmadel ('uio_quotation_mark_delete')
!
! PURPOSE:
!   Parse string 'inline' and transform its first term
!   with or without limiting quotation marks into a string 'ouline' without.
!
! CATEGORY:
!   String/array processing
!
! CALLING SEQUENCE:
!   call uio_qmadel(inline, ouline,remain, outstr,ierr)
!
! INPUT:
!   inline: ('input_line') character
!
! OUTPUT:
!   ouline: ('output_line') character
!   remain: ('remainder') character
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Parse string 'inline' and transform its first term
!   with or without limiting quotation marks into a string 'ouline' without.
!   The first term is marked by a limiter:
!     The end of the string.
!     A blank ' ' for a string without leading quotation mark.
!     The second quotation mark for a string with leading quotation mark.
!  The rest of inline (if there is any) is given out in remain.
!
! EXAMPLE:
!    inline:                   ouline            remain
!    'I''m tired.'     --->    I'm tired.
!    'I''m tired.'Oh!  --->    I'm tired.        Oh!
!    I'm               --->    I'm
!    I'm tired.        --->    I'm               tired.
!
! MODIFICATION HISTORY:
!   1993-03-08 (B.F. Kiel) Written
!   1993-03-09 (B.F.)
!   1995-11-10 (B.F.) More comments
!   1996-10-15 (B.F.) More comments
!   1997-01-03 (B.F.) gotos removed
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(out)              :: ierr
character*(*), intent(in)               :: inline
character*(*), intent(out)              :: ouline, remain, outstr
!
integer                                 :: ind, iin, iout, leninl,lenoul
!---------------------------------------------------------------------------------------------------
ierr=0
!
if (inline == '') then
  ! --- String is empty ---
  ouline=inline
  remain=''
  outstr='empty'
else if (inline(1:1) /= qmachr) then
  ! --- String without quotation marks ---
  ind=index(inline,' ')
  if (ind == 0) then
    ! --- No blanks, copy line ---
    ouline=inline
    remain=''
  else
    ! --- Extract all characters up to blank ---
    ouline=inline(1:max(ind-1,1))
    remain=inline(min(ind+1,len(inline)):)
  endif
  outstr='done'
else
  ! --- String with quotation marks ---
  if (len(inline) == 1) then
    ! --- No following letters ---
    ouline=''
    remain=''
    outstr='only one ' // qmachr
  else
    ! --- Transform string with quotation marks ---
    iin=1
    iout=0
    ouline=''
    leninl=len(inline)
    lenoul=len(ouline)
    do
      ! --- Increment counters, analyze next character ---
      iin=iin+1
      iout=iout+1
      if (iin > leninl) then
        ! --- End of input string reached ---
        remain=''
        outstr='done'
        exit
      else if (iout > lenoul) then
        ! --- End of output string reached ---
        remain=inline(iin:)
        outstr='outstring too short'
        ierr=1
        exit
      else if (inline(iin:iin) == qmachr) then
        ! --- Quotation mark found ---
        if (iin < leninl) then
          ! --- End of input string not yet reached ---
          iin=iin+1
          if (inline(iin:iin) == qmachr) then
            ! --- Two consecutive quotation marks ---
            ouline(iout:iout)=qmachr
          else
            ! --- Limiting second quotation mark found ---
            remain=inline(iin:)
            outstr='done'
            exit
          endif
        else
          ! --- End of input string reached ---
          remain=''
          outstr='done'
          exit
        endif
      else
        ! --- Copy character, goto next character ---
        ouline(iout:iout)=inline(iin:iin)
      endif
    end do
  endif
endif
!
end subroutine uio_qmadel


!----------**********-------------------------------------------------------------------------------
subroutine uio_vnanrm(inline, ouline,remain, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_vnanrm ('uio_variable_name_norm')
!
! PURPOSE:
!   Transforms a string "inline"
!   into a string "ouline", which now contains a correct name of a variable.
!
! CATEGORY:
!   String processing
!
! CALLING SEQUENCE:
!   call uio_vnanrm(inline, ouline,remain, outstr,ierr)
!
! INPUT:
!   inline: ('input_line') character
!
! OUTPUT:
!   ouline: ('output_line') character
!   remain: ('remainder') character
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!  The input string is tested if it is a correct variable/keyword name.
!  The end of the name of a variable is determined by a following blank ' ' or
!  a "equchr" '='.
!  The rest of "inline" which does not fit in "ouline" is given out in "remain".
!
! EXAMPLES:
!    inline:                ouline       remain        ierr
!    'rho'                  'rho'        ' '           0
!    '  rho'                'rho'        ' '           0
!    'rho  rho'             'rho'        '  rho'       1
!    'rho=10.0'             'rho'        '=10.0'       1
!    '=10.0'                ' '          '=10.0'       1
!    ' '                    ' '          ' '           1
!
! MODIFICATION HISTORY:
!   1993-03-13 (B.F. Kiel) Written
!   1995-11-10 (B.F.) More comments
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,       intent(out)              :: ierr
character*(*), intent(in)               :: inline
character*(*), intent(out)              :: ouline, remain, outstr
!
! --- Local variables ---
integer                                 :: idxinl, ind, indbl, indequ
!---------------------------------------------------------------------------------------------------
ierr=0
!
if (inline == '') then
  ! --- Input string empty ---
  ouline=''
  remain=inline
  outstr='empty'
  ierr=1
else
  ! --- Input string non-empty ---
  ! --- Delete leading blanks ---
  ouline=adjustl(inline)
  idxinl=len_trim(ouline)
  indbl =index(ouline(:idxinl),' '   )
  indequ=index(ouline(:idxinl),equchr)
  ! --- Cut at end of first regular variable name ---
  if (indequ == 1) then
    remain=ouline
    ouline=''
    outstr='no name'
    ierr=1
  else if (indbl == 0) then
    if (indequ == 0) then
      remain=''
      outstr='done'
    else
      remain=ouline(indequ:)
      ouline=ouline(:indequ-1)
      outstr='name not correct'
    endif
  else
    if (indequ == 0) then
      ind=indbl
    else
      ind=min(indbl,indequ)
    endif
    remain=ouline(ind:)
    ouline=ouline(:ind-1)
    outstr='more than one name'
  endif
endif
!
end subroutine uio_vnanrm


!----------**********-------------------------------------------------------------------------------
subroutine uio_ppkeyw(k1,v1, k2,v2, k3,v3, mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_ppkeyw ('uio_pretty_print_keywords')
!
! PURPOSE:
!   Print keywords with values in pretty form.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_ppkeyw(k1,v1, k2,v2, k3,v3, mode=mode)
!
! INPUT:
!   mode:   ('mode') integer, optional, determines interpretation of
!           first two keywords in term table:
!           0: first two keywords give variable type and identifier
!           1: all entries are keywords
!   k1,2,3: ('keyword_1,2,3') character, names of keywords
!   v1,2,3: ('value_1,2,3') character, values of keywords
!
! ROUTINES:
!   uio_pptrmt:            Print term table in pretty form.
!
! OUTPUT:
!   None
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data to standard output.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Put 3 keywords with their values into term table and call uio_pptrmt.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-02-19 (B.F. Kiel) Written
!   1996-02-22 (B.F.) Parameter mode added
!   1997-01-20 (B.F.) Parameter mode made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
character*(*),           intent(in)     :: k1,v1, k2,v2, k3,v3
integer,       optional, intent(in)     :: mode
!
! --- Local variables ---
character*(let)                         :: termt(2,nttmx)
integer                                 :: ntt
!---------------------------------------------------------------------------------------------------
!
! --- Print scalar keywords and values ---
termt(1,1)=k1
termt(2,1)=v1
termt(1,2)=k2
termt(2,2)=v2
termt(1,3)=k3
termt(2,3)=v3
ntt=3
!
! --- Print term table ---
call uio_pptrmt(termt,ntt, mode=mode)
!
end subroutine uio_ppkeyw


!----------**********-------------------------------------------------------------------------------
subroutine uio_pptrmt(termt,ntt, mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_pptrmt ('uio_pretty_print_term_table')
!
! PURPOSE:
!   Print term table in pretty form.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_pptrmt(termt,ntt, mode=mode)
!
! INPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!   mode:   ('mode') integer, optional,
!           determines interpretation of first two keywords
!           0: first two keywords give variable type and identifier (default)
!           1: all entries are keywords
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data to standard output.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Straightforward: Just print strings in term table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-05 (B.F. Kiel) Written
!   1996-02-19 (B.F.) No printing of scalar keywords, A12 -> A12
!   1996-02-22 (B.F.) Parameter mode added
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-14 (B.F.) Parameter mode made optional
!   1997-01-20 (B.F.) termt has assumed shape
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: ntt
character*(*),           intent(in)     :: termt(1:,1:)
integer,       optional, intent(in)     :: mode
!
! --- Local variables ---
integer, parameter                      :: nc=6
integer                                 :: n, n1, mode0
character*(*), parameter                :: fo='(1X,A12,A,1X,A64)'
character*(1), parameter                :: sh=' '
!---------------------------------------------------------------------------------------------------
! --- Default values for optional parameter ---
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
!
! --- Print variable type and identifier ---
n1=1
if ((mode0 == 0) .and. (ntt >= 2)) then
  write(nc,'(A)') trim(termt(1,1))//' '//trim(termt(1,2))
  n1=3
endif
!
! --- Print term table ---
do n=n1,ntt
  if (termt(1,n) /= '') then
    if (termt(2,n) /= '') then
      write(nc,fo) trim(termt(1,n))//sh, equchr, termt(2,n)
    else
      write(nc,fo) trim(termt(1,n))//sh
    endif
  endif
end do ! n
!
end subroutine uio_pptrmt


!----------**********-------------------------------------------------------------------------------
subroutine uio_dattim(date)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_dattim ('uio_date_time')
!
! PURPOSE:
!   Get string with current date and time.
!
! CATEGORY:
!   System routines
!
! CALLING SEQUENCE:
!   call uio_dattim(date)
!
! INPUT:
!   None
!
! OUTPUT:
!   date: ('date') character, date and time information
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Uses Fortran90 routine.
!
! PROCEDURE:
!   Call date_and_time
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-14 (B.F. Kiel) Written
!   1996-12-30 (B.F.) Fortran90 version
!   1997-01-02 (B.F.) Nicer date/time format
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameter ---
character*(*), intent(out)              :: date
!
! --- Local variables ---
character                               :: date0*8, time0*10
!---------------------------------------------------------------------------------------------------
!
call date_and_time(date=date0, time=time0)
! date=date0//' '//time0
date=date0(7:8)//'.'//date0(5:6)//'.'//date0(1:4)//' '// &
     time0(1:2)//':'//time0(3:4)//':'//time0(5:10)
!
end subroutine uio_dattim


!----------**********-------------------------------------------------------------------------------
subroutine uio_deform(nbytin,nbytre,nbytco,nbytdb, &
                      cbytin,cbytre,cbytco,cbytdb, &
                      defoin,defore,defoco,defodb)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_deform ('uio_default_format')
!
! PURPOSE:
!   Determine the default output format for
!   integers and real, complex, and double precision numbers.
!
! CALLING SEQUENCE:
!   call deform(nbytin,nbytre,nbytco,nbytdb, &
!               cbytin,cbytre,cbytco,cbytdb, &
!               defoin,defore,defoco,defodb)
!
! INPUT:
!   None
!
! OUTPUT:
!   nbytin: ('character_byte_integer'), integer, number of bytes needed to store
!           an integer
!   nbytre: ('character_byte_real'), integer, number of bytes needed to store
!           a real value
!   nbytco: ('character_byte_complex'), integer, number of bytes needed to store
!           a complex value
!   nbytdb: ('character_byte_double'), integer, number of bytes needed to store
!           a double precision value
!   nbytin: ('character_byte_integer'), integer, number of bytes needed to store
!           an integer
!   nbytre: ('character_byte_real'), integer, number of bytes needed to store
!           a real value
!   nbytco: ('character_byte_complex'), integer, number of bytes needed to store
!           a complex value
!   nbytdb: ('character_byte_double'), integer, number of bytes needed to store
!           a double precision value
!   defoin: ('default_format_integer'), character*3, default format string for
!           output of an integer
!   defore: ('default_format_real'), character*6, default format string for
!           output of a real value
!   defoco: ('default_format_complex'), character*25, default format string for
!           output of a complex value
!   defodb: ('default_format_double'), character*6, default format string for
!           output of a double precision value
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The number of bytes needed to store a variable is determined by
!   equivalencing this variable and a long string, writing something into
!   the variable and looking how many characters of the string have changed.
!
! MODIFICATION HISTORY:
!   1993-03-21 (B.F. Kiel) Written
!   1993-03-22 (B.F.)
!   1995-11-13 (B.F.)
!   1996-09-24 (B.F.) One digit more for real (single, double, complex) variables,
!                     necessary for compatibility with IDL
!   1996-12-23 (B.F.) Fortran90 version
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,       intent(out)              :: nbytin, nbytre, nbytco, nbytdb
character*(*), intent(out)              :: cbytin, cbytre, cbytco, cbytdb, &
                                           defoin, defore, defoco, defodb
!
! --- Local variables ---
integer                                 :: i=0
real                                    :: r=0.0
double precision                        :: d=0.0D0
complex                                 :: c=(0.0,0.0)
character*(2)                           :: sthlp
!---------------------------------------------------------------------------------------------------
!
! --- Determine number of bytes to store an integer ---
inquire(iolength=nbytin) i
write(cbytin,'(I2)') nbytin
cbytin=adjustl(cbytin)
! --- Derive default output format for an integer ---
write(defoin,'(I2)') range(i)+1                           ! --- +1 for sign ---
defoin='I'//adjustl(defoin)
!
! --- Determine number of bytes needed to store a real number ---
inquire(iolength=nbytre) r
write(cbytre,'(I2)') nbytre
cbytre=adjustl(cbytre)
write(defore,'(I2)') precision(r)
write(sthlp,'(I2)') int(alog10(real(range(r)))+6.0)+precision(r)
defore='E'//trim(adjustl(sthlp))//'.'//adjustl(defore)
!
! --- Determine number of bytes needed to store a complex number ---
inquire(iolength=nbytco) c
write(cbytco,'(I2)') nbytco
cbytco=adjustl(cbytco)
write(defoco,'(I2)') precision(c)
write(sthlp,'(I2)') int(alog10(real(range(c)))+6.0)+precision(c)
defoco='E'//trim(adjustl(sthlp))//'.'//adjustl(defoco)
defoco='''('','//trim(adjustl(defoco))//','','','// &
                 trim(adjustl(defoco))//','')'''
!
! --- Determine number of bytes needed to store a double precision number ---
inquire(iolength=nbytdb) d
write(cbytdb,'(I2)') nbytdb
cbytdb=adjustl(cbytdb)
write(defodb,'(I2)') precision(d)
write(sthlp,'(I2)') int(alog10(real(range(d)))+6.0)+precision(d)
defodb='E'//trim(adjustl(sthlp))//'.'//adjustl(defodb)
!
end subroutine uio_deform


!----------*********--------------------------------------------------------------------------------
subroutine uio_wf2rf(wf,rf, qmachr, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wf2rf ('uio_write_format_to_read_format')
!
! PURPOSE:
!   Produce from write format string corresponding read format string.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wf2rf(wf,rf, qmachr=qmachr, outstr=outstr,ierr=ierr)
!
! INPUT:
!   wf:     ('write_format') character, write format string
!   qmachr: ('quotation_mark_character') character, optional,
!           default="'" (single)
!
! OUTPUT:
!   rf:     ('read_format') character, read format string
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Extract literal string in format specification and replace it by an
!   appropriate number of skip commands ('X').
!
! EXAMPLE:
!   wf="'a=',E13.6"
!   call uio_wf2rf(wf, rf, outstr,ierr)
!     ->
!   WF              STRING    = ''a=',E13.6'
!   RF              STRING    = '2X,E13.6'
!   OUTSTR          STRING    = 'done'
!   IERR            INT       =        0
!
! MODIFICATION HISTORY:
!   1997-02-13 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
character*(*),           intent(in)     :: wf
character*(*),           intent(out)    :: rf
character*(1), optional, intent(in)     :: qmachr
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: ierr0, qma_strpos1, qma_strpos2
character*(1)                           :: qma
character*(8)                           :: rfin
character*(80)                          :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- Default value for optional parameter ---
if (present(qmachr)) then
  if (qmachr == '') then
    qma=''''
  else
    qma=qmachr
  endif
else
  qma=''''
endif
!
if (len_trim(wf) == 0) then
  rf=''
  outstr0='empty format'
  ierr0=0
  goto 900
endif
!
rf=wf
do
  qma_strpos1=index(rf,qma)
  if (qma_strpos1 == 0) then
    outstr0='done'
    ierr0=0
    exit
  endif
  qma_strpos2=index(rf(qma_strpos1+1:),qma)
  if (qma_strpos2 == 0) then
    outstr0='No matching second quotation mark'
    ierr0=1
    exit
  endif
  write(rfin,'(I8)') qma_strpos2-1
  rfin=adjustl(rfin // 'X')
  rf=rf(1:qma_strpos1-1) // trim(rfin) // rf(qma_strpos1+qma_strpos2+1:)
!  write(6,*) rf
end do
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_wf2rf

end module uio_base_module
