!***************************************************************************************************
!
!   #    # #  ####        #    #   ##    ####  
!   #    # # #    #       ##  ##  #  #  #    # 
!   #    # # #    #       # ## # #    # #      
!   #    # # #    #       #    # ###### #      
!   #    # # #    #       #    # #    # #    # 
!    ####  #  #### ###### #    # #    #  #### _module
!
!   Universal Input/Output Routines: Machine-dependent part (all machines)
!
!***************************************************************************************************
!   This set of input and output routines allows the reading and writing of scalar, array, and
!   table data in header-data units in formatted or unformatted Fortran file format.
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Lyon
!***************************************************************************************************
!   Fortran90, non-standard Fortran: depending on machine
!   2017-08-05
!***************************************************************************************************
! MODULES:
!   uio_mac_module:        Set of (possibly) machine-dependent uio-routines
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Type of call to get system information:                                             --- */
!# /* ---   0: Read information from file 'uioenvfile.txt', don't write or delete it          --- */
!# /* ---   1: Call external function 'system()' to produce 'uioenvfile.txt', read and delete it  */
!# /* ---   2: Call routine 'system()' to produce 'uioenvfile.txt', read and delete it        --- */
!# /* ---   3: Call external routine 'HF_SH()' to produce 'uioenvfile.txt', read and delete it -- */
!# /* ---   4: Call external function 'uname()' to get system information                     --- */
!# /* ---   5: Call external routine 'pxfuname()' to get system information                   --- */
# ifndef uio_switch_system_l01
#   define uio_switch_system_l01 1
# endif
!#
!# /* --- 'Native' conversion (i.e. none)                             --- */
!# /* ---   1: Get information about word lengths from 'uio_deform()' --- */
!# /* ---   0,2: Set word lengths explicitely                         --- */
# ifndef uio_switch_native_l01
#   define uio_switch_native_l01 1
# endif
!#
!# /* --- 'Big Endian' conversion (the default for binary files for CO5BOLD) --- */
!# /* ---   0: Don't include this conversion type                            --- */
!# /* ---   1: Get information about word lengths from 'uio_deform()'        --- */
!# /* ---   2: Set word lengths explicitely                                  --- */
# ifndef uio_switch_ieeebe_l01
#   define uio_switch_ieeebe_l01 0
# endif
!#
!# /* --- 'Little Endian' conversion                                  --- */
!# /* ---   0: Don't include this conversion type                     --- */
!# /* ---   1: Get information about word lengths from 'uio_deform()' --- */
!# /* ---   2: Set word lengths explicitely                           --- */
# ifndef uio_switch_ieeele_l01
#   define uio_switch_ieeele_l01 0
# endif
!#
!# /* --- 'Default Endian' conversion                                 --- */
!# /* ---   0: Don't include this conversion type                     --- */
!# /* ---   1: Get information about word lengths from 'uio_deform()' --- */
!# /* ---   2: Set word lengths explicitely (4-byte words)            --- */
!# /* ---   3: Set word lengths explicitely (8-byte words)            --- */
# ifndef uio_switch_ieee_l01
#   define uio_switch_ieee_l01 0
# endif
!#
!# /* --- 'Cray X-MP' format                                          --- */
!# /* ---   0: Don't include this conversion type                     --- */
!# /* ---   1: Get information about word lengths from 'uio_deform()' --- */
# ifndef uio_switch_crayxmp_l01
#   define uio_switch_crayxmp_l01 0
# endif
!#
!# /* --- Activate conversion while opening a file             --- */
!# /* ---   0: No conversion                                   --- */
!# /* ---   1: Use 'convert' keyword in the 'open()' statement --- */
!# /* ---   2: Use 'asnunit()' Cray X-MP style                 --- */
!# /* ---   3: Use 'asnunit()' Cray TS style                   --- */
# ifndef uio_switch_open_l01
#   define uio_switch_open_l01 0
# endif
!#
!# /* --- De-activate conversion while closing a file               --- */
!# /* ---   0: Nothing to de-activate                               --- */
!# /* ---   1: Use 'asnunit()' to remove any Cray-style assignments --- */
!# ifndef uio_switch_close_l01
!#   define uio_switch_close_l01 0
!# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------**************-------------------------------------------------------------------------------
module uio_mac_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_mac_module ('uio_machine_module')
!
! PURPOSE:
!   Contain (possibly) machine dependent routines for UIO.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_mac_module
!
! ROUTINES: (CONTAINED)
!   uio_SwitchInfo:        Print information about used compiler switches.
!   uio_getenv:            Get information about environment
!   uio_mkcvls:            Make list with possible conversion types (e.g.: 'native' or 'ieee_4')
!   uio_uopen:             Open a file with special handling for conversion type
!   uio_uclose:            Close file with special handling for conversion type
!
! MODULES:
!   uio_base_module:       Basic set of UIO routines: string processing, header, I/O channels
!
! MODIFICATION HISTORY:
!   1997-01-15 (B.F. Kiel) Routines put into module
!   1997-02-21 (B.F.) Standard uio_getenv, IEEE uio_mkcvls (uio_mac_ieee_module)
!   1999-03-04 (B.F.) DECalpha version: special open keywords
!   1999-05-12 (B.F. Copenhagen) Keyword position added
!   2002-05-16 (B.F. Uppsala) NEC SX-5 version
!   2010-05-04 (B.F. Lyon) Merge versions, use compiler preprocessor switches:
!                          uio_mac_sun_module.f90      (1999-05-12),
!                          uio_mac_ieee_module.f90     (2000-03-07),
!                          uio_mac_aix_module.f90      (2009-10-15),
!                          uio_mac_module.f90          (2000-03-07),
!                          uio_mac_intel_module.f90    (2002-01-29),
!                          uio_mac_nec_module.f90      (2002-05-16),
!                          uio_mac_hitachi_module.f90  (2001-05-30),
!                          uio_mac_decalpha_module.f90 (2000-03-07),
!                          uio_mac_crayxmp_module.f90  (2000-03-07),
!                          uio_mac_crayts_module.f90   (2000-03-07)
!   2010-05-07 (B.F.)
!   2013-08-24 (B.F.) Change layout of comments
!   2017-08-05 (B.F. Uppsala) Change layout of comments and indentation;
!                             modernize character declarations
!---------------------------------------------------------------------------------------------------
use uio_base_module
!
contains

!----------**************---------------------------------------------------------------------------
subroutine uio_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_SwitchInfo ('uio_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   I/O
!
! CALLING SEQUENCE:
!   call uio_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2010-05-05 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: uio_mac_module .........................'
!
write(ncp,'(A,I6)') '  uio' // '_switch_system_l01:      ', uio_switch_system_l01
write(ncp,'(A,I6)') '  uio' // '_switch_ieeebe_l01:      ', uio_switch_ieeebe_l01
write(ncp,'(A,I6)') '  uio' // '_switch_ieeele_l01:      ', uio_switch_ieeele_l01
write(ncp,'(A,I6)') '  uio' // '_switch_ieee_l01:        ', uio_switch_ieee_l01
write(ncp,'(A,I6)') '  uio' // '_switch_crayxmp_l01:     ', uio_switch_crayxmp_l01
write(ncp,'(A,I6)') '  uio' // '_switch_open_l01:        ', uio_switch_open_l01
!write(ncp,'(A,I6)') '  uio' // '_switch_close_l01:       ', uio_switch_close_l01
!
end subroutine uio_SwitchInfo


!----------**********-------------------------------------------------------------------------------
subroutine uio_getenv(MyProc, system,machin,osrels,osvers,hrdwar,progrm,langua, &
                      outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_getenv ('uio_get_environment')
!
! PURPOSE:
!   Get information about environment.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_getenv(system=system, machin=machin, osrels=osrels, &
!                   osvers=osvers, hrdwar=hrdwar, progrm=progrm, &
!                   langua=langua, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   MyProc:       ('My_Process') optional integer with MPI process ID:
!                   0: master process -> the environment parameters are determined (default)
!                  >0: other process -> the environment parameters are not determined if a file
!                      has to be written
!
! OUTPUT:
!   system:       ('system') character, operating system,
!                   e.g., SunOS, craSH
!   machin:       ('machine') character, empty or machine name,
!                   e.g., venus, craSH
!   osrels:       ('operating_system_release') character, release of operating system,
!                   e.g., 4.1.3, 8.0.3.0
!   osvers:       ('operating system version') character, version of operating system,
!                   e.g., 1,     awa.5
!   hrdwar:       ('hardware') character, empty or machine hardware,
!                   e.g., sun4m, CRAY Y-MP
!   progrm:       ('program_name') character, name of main program
!   langua:       ('language') character, language 'Fortran90'
!   outstr:       ('output_string') character, optional
!   ierr:         ('integer_error') integer, optional, contains exit error status
!                   0: correct operation
!                  >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   system:                Spawn UNIX script (system call on e.g. Suns)
!   uname:                 Get information about environment and machine (system call on Cray XMP)
!   pxfstructcreate:       Auxiliary PXF routine (system call on Cray TS)
!   pxfuname:              Get information about environment and machine (system call on Cray TS)
!   pxfstrget:             Auxiliary PXF routine (system call on Cray TS)
!   pxfstructfree:         Auxiliary PXF routine (system call on Cray TS)
!
! MODULES:
!   uio_inf_module:        Information about environment
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Depending on machine:
!   The file 'uioenvfile.txt' may be produced and read.
!
! RESTRICTIONS:
!   Depending on machine:
!   Only useful on UNIX systems. The FORTRAN compiler has to know the system call.
!   A file 'uioenvfile.txt' is created and deleted.
!
!   The file 'uioenvfile.txt' has to be prepared properly before execution.
!   If it is non-existent or empty, the output strings are empty.
!
! PROCEDURE:
!   Start shell script (which executes 'uname') and then
!   look into file 'uioenvfile.txt' to get information about environment.
!   When the MPI process ID is passed with the MyProc parameter, only the master process
!   writes and reads the file. The other processes have no environment information.
!
!   Look into file 'uioenvfile.txt' to get information about environment.
!   The file has to be created before program execution e.g. with
!     uname -s >  uioenvfile.txt
!     uname -n >> uioenvfile.txt
!     uname -r >> uioenvfile.txt
!     uname -v >> uioenvfile.txt
!     uname -m >> uioenvfile.txt
!
!   Call uname().
!
!   Call pxfuname().
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-22 (B.F. Kiel) Written
!   1995-11-24 (B.F.) Save instead of common block
!   1995-12-06 (B.F.) 'uiosiz.inc' included
!   1995-12-14 (B.F.) Routine dattim used
!   1995-12-15 (B.F.) 'uioenvfile.txt', some rearrangement, call system or uname
!   1996-01-30 (B.F.) Common block instead of save
!   1996-02-01 (B.F.) Parameter progrm added
!   1996-10-23 (B.F.) Delete environment file with closing it
!   1996-12-20 (B.F.) Include 'uioinf.inc'
!   1996-12-30 (B.F.) Call uio_dattim
!   1997-01-11 (B.F.) Parameter date removed
!   1997-01-20 (B.F.) outstr,ierr made optional
!   1997-02-02 (B.F.) Parameter langua added, parameters made optional
!   1997-02-13 (B.F.) Derived from uio_mac_crayxmp_module.f90 (uio_mac_crayts_module.f90)
!   1997-09-10 (B.F.) syste1 -> system
!   2017-07-15 (B.F. Uppsala) Add optional input parameter MyProc to make routine MPI safe
!---------------------------------------------------------------------------------------------------
use uio_inf_module
!
implicit none
!
! --- I/O parameters ---
integer,         optional, intent(in)   :: MyProc
character(len=*),optional, intent(out)  :: system, machin, osrels, osvers, &
                                           hrdwar, progrm, langua
character(len=*),optional, intent(out)  :: outstr
integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
integer                                 :: nc, idum, ierr0, ierrt, ie, junam, ilen
character*(80)                          :: outstr0
!#
# if (uio_switch_system_l01 == 4)
!#
! --- Externals ---
integer uname
external uname
!#
# elif (uio_switch_system_l01 == 5)
!#
! --- Externals ---
external pxfstructcreate, pxfuname, pxfstrget, pxfstructfree
!#
# endif
!#      /* uio_switch_system_l01 */
!
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Some information about environment from file 'uioenvfile.txt' ---
if (syste0 == '') then
  langu0='Fortran90'
  !
!#
# if (uio_switch_system_l01 == 4)
!#
  ! --- Call non-standard Cray Fortran routine 'uname' to get ---
  ! --- some information about environment                    ---
  ie=uname(syste0,machi0,osrel0,osver0,hrdwa0)
  !
  if (ie >= 5) then
    outstr0='done'
    ierr0=0
  else
    syste0=''
    machi0=''
    osrel0=''
    osver0=''
    hrdwa0='Computer'
    outstr0='error in call of uname'
    ierr0=10-ie
  endif
!#
# elif (uio_switch_system_l01 == 5)
!#
  ! --- Call non-standard Cray Fortran routines to get ---
  ! --- some information about environment             ---
  call pxfstructcreate('utsname', junam, ierr0)
  call pxfuname(junam, ierr0)
  !
  if (ierr0 == 0) then
    ilen=0
    call pxfstrget(junam, 'sysname', syste0, ilen, ierr0)
    ilen=0
    call pxfstrget(junam, 'nodename',machi0, ilen, ierr0)
    ilen=0
    call pxfstrget(junam, 'release', osrel0, ilen, ierr0)
    ilen=0
    call pxfstrget(junam, 'version', osver0, ilen, ierr0)
    ilen=0
    call pxfstrget(junam, 'machine', hrdwa0, ilen, ierr0)
    !
    call pxfstructfree(junam, ierr0)
    outstr0='done'
    ierr0=0
  else
    syste0=''
    machi0=''
    osrel0=''
    osver0=''
    hrdwa0='Computer'
    outstr0='error in call of pxfuname'
  endif
!#
# else
!#     /* uio_switch_system_l01 */
!#
# if (uio_switch_system_l01 == 1) || (uio_switch_system_l01 == 2) || (uio_switch_system_l01 == 3)
!#
  if (merge(MyProc, 0, present(MyProc)) == 0) then
    ! --- System call: issue UNIX command 'uname' to get information about environment ---
    ierrt=call_system('uname -s >  uioenvfile.txt ;'// &
                      'uname -n >> uioenvfile.txt ;'// &
                      'uname -r >> uioenvfile.txt ;'// &
                      'uname -v >> uioenvfile.txt ;'// &
                      'uname -m >> uioenvfile.txt'    )
  else
    ! --- Skip calling 'uname' and reading the resulting file ---
    ! --- Take fixed default values ---
    syste0=''
    machi0=''
    osrel0=''
    osver0=''
    hrdwa0='Computer'
    outstr0='No environment information determined.'
    goto 300
  endif
!#
# endif
!#      /* uio_switch_system_l01 */
  !
  ! --- Get free unit number and open file ---
  call uio_chunit('get', nc1out=nc)
  open(nc,file='uioenvfile.txt',status='old',err=100)
  read(nc,'(A)',err=110) syste0
  read(nc,'(A)',err=120) machi0
  read(nc,'(A)',err=130) osrel0
  read(nc,'(A)',err=140) osver0
  read(nc,'(A)',err=150) hrdwa0
  outstr0='Information read'
  goto 200
  !
  100 continue
  ! --- Take fixed default values ---
  110 syste0=''
  120 machi0=''
  130 osrel0=''
  140 osver0=''
  150 hrdwa0='Computer'
  ierr0=1
  outstr0='File ''uioenvfile.txt'' with environment data not correct.'
  !
  200 continue
  !
  ! --- Close file and return unit number ---
!#
# if (uio_switch_system_l01 == 1)
!#
  close(nc,status='delete')
!#
# else
!#     /* uio_switch_system_l01 */
  close(nc)
!#
# endif
!#      /* uio_switch_system_l01 */
  call uio_chunit('return', nc1in=nc)
!#
# endif
!#      /* uio_switch_system_l01 */
  !
  300 continue
  !
endif
!
if (present(system)) system=syste0
if (present(machin)) machin=machi0
if (present(osrels)) osrels=osrel0
if (present(osvers)) osvers=osver0
if (present(hrdwar)) hrdwar=hrdwa0
if (present(progrm)) progrm=progr0
if (present(langua)) langua=langu0
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
!#
# if (uio_switch_system_l01 == 1)
!#
contains
!
function call_system(in_string)
!
implicit none
!
! --- I/O parameters ---
integer                                 :: call_system
character(len=*)                        :: in_string
!
! --- Local variables ---
!
! --- Externals ---
integer system
external system
!
call_system=system(in_string)
!
end function call_system
!#
# elif (uio_switch_system_l01 == 3)
!#
contains
!
function call_system(in_string)
!
implicit none
!
! --- I/O parameters ---
integer                                 :: call_system
character(len=*)                        :: in_string
!
! --- Local variables ---
!
! --- Externals ---
integer system
external system
!
call HF_SH(in_string,call_system)
!
end function call_system
!#
# endif
!#      /* uio_switch_system_l01 */
!
end subroutine uio_getenv


!#
# if (uio_switch_system_l01 == 2)
!#
function call_system(in_string)
!
implicit none
!
! --- I/O parameters ---
integer                                 :: call_system
character(len=*)                        :: in_string
!
call system(in_string,call_system)
!
end function call_system
!#
# endif
!#      /* uio_switch_system_l01 */


!----------**********-------------------------------------------------------------------------------
subroutine uio_mkcvls(outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_mkcvls ('uio_make_convert_list')
!
! PURPOSE:
!   Make list with possible conversion types.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_mkcvls(outstr=outstr,ierr=ierr)
!
! INPUT:
!   None
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('integer_error') integer, optional, contains exit error status
!                   0: correct operation
!                  >0: occurrence of an error
!
! ROUTINES:
!   uio_deform:            Determine the default output format for numbers
!
! MODULES:
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   Change variables in module uio_cvl_module.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-27 (B.F. Kiel) Written
!   1995-12-28 (B.F.) Sun version, Cray version
!   1996-10-16 (B.F.) ieee -> ieee_4
!   1996-10-19 (B.F.) Default conversion type ncvdef defined: ieee_4
!   1996-12-30 (B.F.) Use uio_deform
!   1997-01-06 (B.F.) ieee_4, ieee
!   1997-01-11 (B.F.) cbytin... are now local variables
!   1997-01-19 (B.F.) Parameters made optional
!   1999-03-04 (B.F.) Additional conversion types: ieeele_4, ieeele_8 (uio_mac_decalpha_module.f90)
!   2001-05-30 (M.S.) Parameters set 'by hand' (uio_mac_hitachi_module.f90)
!   2002-01-29 (B.F.) Big Endian -> Little Endian (uio_mac_intel_module.f90)
!   1998-02-27 (B.F.) cvnals(n)='ieee_4_limit' added (Cray modules)
!   2002-05-16 (B.F.) Ignore uio_deform information and use ieee_4 values (uio_mac_nec_module.f90)
!---------------------------------------------------------------------------------------------------
use uio_cvl_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: n
integer                                 :: nbytin, nbytre, nbytco, nbytdb
character(len=2)                        :: cbytin, cbytre, cbytco, cbytdb
character(len=3)                        :: defoin
character(len=6)                        :: defore
character(len=25)                       :: defoco
character(len=6)                        :: defodb
!---------------------------------------------------------------------------------------------------
! -------- ******
n=1
cvnals(n)='native'
!#
# if (uio_switch_native_l01 == 1)
!#                                /* ------------------------------------------------------------ */
call uio_deform(nbytin,nbytre,nbytco,nbytdb, &
                cbytin,cbytre,cbytco,cbytdb, &
                defoin,defore,defoco,defodb)
btinls(n)=cbytin
btrels(n)=cbytre
btcols(n)=cbytco
btdbls(n)=cbytdb
foinls(n)=defoin
forels(n)=defore
focols(n)=defoco
fodbls(n)=defodb
!#
# else
!#     /* uio_switch_native_l01 */
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)='''('',E13.6,'','',E13.6,'')'''
fodbls(n)='E23.15'
!#
# endif
!#      /* uio_switch_native_l01 */
!
! --- Default conversion type ---
ncvdef=n
!
!#
# if (uio_switch_ieeebe_l01 == 1)
!#                                /* ------------------------------------------------------------ */
! -------- ******
n=n+1
cvnals(n)='ieee_4'
if (btrels(1) == '8') cvnals(n)='ieee_8'
btinls(n)=btinls(1)
btrels(n)=btrels(1)
btcols(n)=btcols(1)
btdbls(n)=btdbls(1)
foinls(n)=foinls(1)
forels(n)=forels(1)
focols(n)=focols(1)
fodbls(n)=fodbls(1)
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# elif (uio_switch_ieeebe_l01 == 2)
!#
! -------- ******
n=n+1
cvnals(n)='ieee_4'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)='''('',E13.6,'','',E13.6,'')'''
fodbls(n)='E23.15'
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# endif
!#      /* uio_switch_ieeebe_l01 */
!
!#
# if (uio_switch_ieeele_l01 == 1)
!#                                /* ------------------------------------------------------------ */
! -------- ******
n=n+1
cvnals(n)='ieeele_4'
if (btrels(1) == '8') cvnals(n)='ieeele_8'
btinls(n)=btinls(1)
btrels(n)=btrels(1)
btcols(n)=btcols(1)
btdbls(n)=btdbls(1)
foinls(n)=foinls(1)
forels(n)=forels(1)
focols(n)=focols(1)
fodbls(n)=fodbls(1)
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# elif (uio_switch_ieeele_l01 == 2)
!#
! -------- ********
n=n+1
cvnals(n)='ieeele_4'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)='''('',E13.6,'','',E13.6,'')'''
fodbls(n)='E23.15'
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# endif
!#      /* uio_switch_ieeele_l01 */
!
!#
# if (uio_switch_ieee_l01 == 1)
!#                              /* -------------------------------------------------------------- */
! -------- ****
n=n+1
cvnals(n)='ieee'
btinls(n)=btinls(1)
btrels(n)=btrels(1)
btcols(n)=btcols(1)
btdbls(n)=btdbls(1)
foinls(n)=foinls(1)
forels(n)=forels(1)
focols(n)=focols(1)
fodbls(n)=fodbls(1)
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# elif (uio_switch_ieee_l01 == 2)
!#
! -------- ******
n=n+1
cvnals(n)='ieee'
btinls(n)='4'
btrels(n)='4'
btcols(n)='8'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E13.6'
focols(n)='''('',E13.6,'','',E13.6,'')'''
fodbls(n)='E23.15'
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# elif (uio_switch_ieee_l01 == 3)
!#
! -------- ******
n=n+1
cvnals(n)='ieee'
btinls(n)='4'
btrels(n)='8'
btcols(n)='16'
btdbls(n)='8'
foinls(n)='I11'
forels(n)='E23.15'
focols(n)='''('',E23.15,'','',E23.15,'')'''
fodbls(n)='E23.15'
!
! --- Default conversion type ---
if (ncvdef == 1) ncvdef=n
!#
# endif
!#      /* uio_switch_ieee_l01 */
!
!#
# if (uio_switch_crayxmp_l01 == 1)
!#                                 /* ----------------------------------------------------------- */
! -------- *********
n=n+1
cvnals(n)='crayxmp_8'
if (btrels(1) == '16') cvnals(n)='crayxmp_16'
btinls(n)=btinls(1)
btrels(n)=btrels(1)
btcols(n)=btcols(1)
btdbls(n)=btdbls(1)
foinls(n)=foinls(1)
forels(n)=forels(1)
focols(n)=focols(1)
fodbls(n)=fodbls(1)
!#
# endif
!#      /* uio_switch_crayxmp_l01 */
!
ncv=n
!
! write(6,*) 'List of conversion types:'
!   do n=1,ncv
!     write(6,*) n, ' '//cvnals(n)//' '//btrels(n)//' '//forels(n)
!   end do ! n
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_mkcvls


!----------*********--------------------------------------------------------------------------------
subroutine uio_uopen(nc,file,form,conv,status,position, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_uopen ('uio_universal_open')
!
! PURPOSE:
!   Open file with special handling for conversion type.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_uopen(nc,file, form=form, conv=conv, status=status, position=position, &
!                  outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!   file:         ('filename') character, filename
!   form:         ('format') character, optional, 'formatted' (def.) or 'unformatted'
!   conv:         ('conversion') character, optional, type of conversion procedure
!   status:       ('status') character, optional, 'old', 'new', 'unknown' (def.)
!   position:     ('position') character, optional, 'rewind', 'append', 'asis'
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('integer_error') integer, optional, contains exit error status
!                   0: correct operation
!                  >0: occurrence of an error
!
! ROUTINES:
!   asnunit:               Assign information to channel unit (Crays only)
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   A file is opened.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Call open.
!   Account for type conversion by 'convert=' keyword (Dec Alphas; Intel, Gnu compilers) or
!   information assignment to channel unit according to selected conversion type (Crays)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-28 (B.F. Kiel) Written
!   1996-10-16 (B.F.) Conversion types crayxmp_16 and ieee_8 added, ieee -> ieee_4 (Cray)
!   1997-01-06 (B.F.) ieee_4, ieee (Cray)
!   1997-01-15 (B.F.) conv=conv0 introduced
!                   form,conv,status, outstr,ierr made optional
!   1997-02-19 (B.F.) Not: call asnunit(nc, '-I -F cos',ierrt) (Cray XMP)
!                   but: call asnunit(nc, '-R', ierrt)       (Cray TS)
!   1999-03-04 (B.F.) Additional conversion types: ieeele_4, ieeele_8 ->
!                   use keyword 'convert' in open statement (DecAlpha version)
!   1999-05-12 (B.F. Copenhagen) Keyword position added
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc
character(len=*),          intent(in)   :: file
character(len=*),optional, intent(in)   :: form, conv, status, position
character(len=*),optional, intent(out)  :: outstr
integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
integer                                 :: ierrt
character(len=80)                       :: form0, conv0, status0, position0
!---------------------------------------------------------------------------------------------------
! --- Default values for optional parameters ---
if (present(form)) then
  form0=form
else
  form0='formatted'
endif
! --- Put parameter into variable to prevent compiler from complaining about --
! --- 'Unused dummy variable CONV' ---
if (present(conv)) then
  conv0=conv
else
  conv0=''
endif
if (present(status)) then
  status0=status
else
  status0='unknown'
endif
if (present(position)) then
  position0=position
else
  position0='asis'
endif
!
! --- Open file on given channel with specified format ---
!#
# if (uio_switch_open_l01 == 1)
!#
if (form0 == 'unformatted') then
  if (conv0 == 'native') then
    open(nc, file=file, status=status0, form=form0, position=position0, convert='native')
  else if ((conv0 == 'ieeele_4') .or. (conv0 == 'ieeele_8')) then
    open(nc, file=file, status=status0, form=form0, position=position0, convert='little_endian')
  else if ((conv0 == 'ieee_4') .or. (conv0 == 'ieee_8') .or. (conv0 == '')) then
    open(nc, file=file, status=status0, form=form0, position=position0, convert='big_endian')
  endif
else
  open(nc, file=file, status=status0, form=form0, position=position0)
endif
!#
# elif (uio_switch_open_l01 == 2)
!#
! --- Cray XMP only ---
if (form0 == 'unformatted') then
  if ((conv0 == 'native') .or. (conv0 == '')) then
    call asnunit(nc, '-I -F cos'        ,    ierrt)
  else if (conv0 == 'crayxmp_8') then
    call asnunit(nc, '-I -F cos'        ,    ierrt)
  else if (conv0 == 'crayxmp_16') then
    call asnunit(nc, '-I -F cos'        ,    ierrt)
!  else if (conv0 == 'ieee_4_limit') then
!    call asnunit(nc, '-I -F f77 -N ieee_dp',    ierrt)
  else if (conv0 == 'ieee_4') then
    call asnunit(nc, '-I -F f77 -N ieee_32',    ierrt)
!    call asnunit(nc, '-I -F f77 -N ieee',    ierrt)
  else if (conv0 == 'ieee_8') then
    call asnunit(nc, '-I -F f77 -N ieee_64', ierrt)
!    call asnunit(nc, '-I -F f77 -N ieee_dp', ierrt)
  else if (conv0 == 'ieee') then
    call asnunit(nc, '-I -F f77 -N ieee',    ierrt)
  endif
  ! write(ncchr,'(I3)') nc
  ! ncchr=adjustl(ncchr)
  ! call assign('assign -F f77 -N ieee u:'//ncchr)
  ! call assign('assign -R u:'//ncchr)
endif
!
open(nc, file=file, status=status0, form=form0, position=position0)
!#
# elif (uio_switch_open_l01 == 3)
!#
! --- Cray TS only ---
if (form0 == 'unformatted') then
  if ((conv0 == 'native') .or. (conv0 == '')) then
    call asnunit(nc, '-R', ierrt)
!    call asnunit(nc, '-I -F cos'        ,    ierrt)
  else if (conv0 == 'crayxmp_8') then
    call asnunit(nc, '-R', ierrt)
!    call asnunit(nc, '-I -F cos'        ,    ierrt)
  else if (conv0 == 'crayxmp_16') then
    call asnunit(nc, '-R', ierrt)
!    call asnunit(nc, '-I -F cos'        ,    ierrt)
!  else if (conv0 == 'ieee_4_limit') then
!    call asnunit(nc, '-I -F f77 -N ieee_dp',    ierrt)
  else if (conv0 == 'ieee_4') then
    call asnunit(nc, '-I -F f77 -N ieee_32',    ierrt)
!    call asnunit(nc, '-I -F f77 -N ieee',    ierrt)
  else if (conv0 == 'ieee_8') then
    call asnunit(nc, '-I -F f77 -N ieee_64', ierrt)
!    call asnunit(nc, '-I -F f77 -N ieee_dp', ierrt)
  else if (conv0 == 'ieee') then
    call asnunit(nc, '-I -F f77 -N ieee_32',    ierrt)
!    call asnunit(nc, '-I -F f77 -N ieee',    ierrt)
  endif
  ! write(ncchr,'(I3)') nc
  ! ncchr=adjustl(ncchr)
  ! call assign('assign -F f77 -N ieee u:'//ncchr)
  ! call assign('assign -R u:'//ncchr)
endif
!
open(nc, file=file, status=status0, form=form0, position=position0)
!#
# else
!#     /* uio_switch_open_l01 */
open(nc, file=file, status=status0, form=form0, position=position0)
!#
# endif
!#      /* uio_switch_open_l01 */
!
if (present(outstr)) outstr=''
if (present(ierr)) ierr=0
!
end subroutine uio_uopen


!----------**********-------------------------------------------------------------------------------
subroutine uio_uclose(nc, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_uclose ('uio_universal_close')
!
! PURPOSE:
!   Close file with special handling for conversion type.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_uclose(nc, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:           ('number_channel') integer, channel unit number
!
! OUTPUT:
!   outstr:       ('output_string') character, optional, always='done'
!   ierr:         ('integer_error') integer, optional, always=0, contains exit error status
!                   0: correct operation
!                  >0: occurrence of an error
!
! ROUTINES:
!   asnunit:               Assign information to channel unit (Crays only)
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   A file is closed.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Just calls close.
!   On Crays: remove all assignments to channel unit.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-28 (B.F. Kiel) Written
!   1997-01-15 (B.F.) outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
!---------------------------------------------------------------------------------------------------
! --- Close channel ---
close(nc)
!
!#
# if (uio_switch_open_l01 == 2) || (uio_switch_open_l01 == 3)
!#
! --- Cray only ---
! if (form == 'unformatted') then
    call asnunit(nc, '-R', ierr)
!   write(ncchr,'(I3)') nc
!   call assign('assign -R u:'//adjustl(ncchr))
! endif
!#
# endif
!#      /* uio_switch_open_l01 */
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_uclose

end module uio_mac_module
