!***************************************************************************************************
!
!   #    # #  ####      #####  ##   #####  #      ###### 
!   #    # # #    #       #   #  #  #    # #      #      
!   #    # # #    #       #  #    # #####  #      #####  
!   #    # # #    #       #  ###### #    # #      #      
!   #    # # #    #       #  #    # #    # #      #      
!    ####  #  #### ###### #  #    # #####  ###### ######_module
!
!   Universal Input/Output Routines: Table part
!
!***************************************************************************************************
!   This set of input and output routines allows the reading and writing of scalar, array, and
!   table data in header-data units in formatted or unformatted Fortran file format.
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:          Kiel, Copenhagen, Uppsala, Lyon
!***************************************************************************************************
!   Fortran90
!   2016-03-16
!***************************************************************************************************
! MODULES:
!   uio_table_module:       Table input/output/manipulation
!
!***************************************************************************************************


!------****************-----------------------------------------------------------------------------
module uio_table_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_table_module ('universal_input_output_table_module')
!
! PURPOSE:
!   Extension set of uio-routines: table input/output/manipulation
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_table_module
!
! ROUTINES:
!   --- Initialization/creation/deleting ---
!   uio_tabnull:           Initialize uio table (nullify pointers...)
!   uio_tabdelete:         Delete uio table (deallocate pointers...)
!   uio_tabmake:           Create or enlarge uio table
!   uio_tabcopy:           Copy uio table (operator: =)
!   uio_tabput:            Put table header/1d integer/real/character array into table
!   uio_tabget:            Get table header/1d integer/real/character array from table
!   uio_tabaskhead:        Get data from table entry header or general data about table
!   --- Writing ---
!   uio_wrtab:             Write table of integer, real, and/or character data to file
!   --- Reading ---
!   uio_rdtab:             Read table of integer, real, and/or character data from file
!   --- Useful high level routines ---
!   uio_cpentr:            Copy entry (header + data) from one file to another.
!
! TYPES:
!   uio_table_type:
!     nlinmx: ('number_line_maximum') integer, maximum line number of table
!     nlin:   ('number_line') integer, current line number of table
!     ncolmx: ('number_column_maximum') integer, maximum number of table entries
!     ncol:   ('number_column') integer, current number of table entries
!     i:      ('integer') integer array, table with all columns with integer values
!     ntaimx: ('number_table_integer_maximum') integer, maximum column number of integer table
!     ntai:   ('number_table_integer') integer, current column number of integer table
!     r:      ('real') real array, table with all columns with real values
!     ntarmx: ('number_table_real_maximum') integer, maximum column number of real table
!     ntar:   ('number_table_real') integer, current column number of real table
!     s:      ('string') character array, table with all columns with character values
!     ntasmx: ('number_table_string_maximum') integer, maximum column number of string table
!     ntas:   ('number_table_string') integer, current column number of string table
!     trtb:   ('term_table') character array, headers for all table entries
!     ntta:   ('number_term_table_array') integer array, number of headlines for each entry
!
! PARAMETERS:
!   uio_tab_charlen: ('uio_table_character_length') integer, parameter,
!                    maximum length of characters in uio table (=80)
!
! USE:
!   uio_siz_module:        String length, table size for UIO
!   uio_bulk_module:       Main set of UIO routines
!
! MODIFICATION HISTORY:
!   1997-01-13 (B.F. Kiel) Routines put into module
!   1998-01-29 (B.F.)
!   2003-06-12 (B.F. Uppsala)
!   2013-08-24 (B.F.) Change layout of comments
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_bulk_module
!
integer, parameter                      :: uio_tab_charlen=80
!
type uio_table_type
  private
  integer                               :: nlinmx,nlin, ncolmx,ncol, &
                                           ntaimx,ntai, ntarmx,ntar, &
                                           ntasmx,ntas
  integer,         pointer, dimension(:):: ntta
  character*(let), pointer, &
                       dimension(:,:,:) :: trtb
  integer,         pointer, &
                       dimension(:,:)   :: i
  real,            pointer, &
                       dimension(:,:)   :: r
  character*(uio_tab_charlen), pointer, &
                       dimension(:,:)   :: s
end type uio_table_type
!
interface uio_tabput
  module procedure uio_tabputi, uio_tabputr, uio_tabputs, uio_tabputt
end interface ! uio_tabput
!
private uio_tabputi, uio_tabputr, uio_tabputs, uio_tabputt
!
interface uio_tabget
  module procedure uio_tabgeti, uio_tabgetr, uio_tabgets, uio_tabgett
end interface ! uio_tabget
!
private uio_tabgeti, uio_tabgetr, uio_tabgets, uio_tabgett
!
interface assignment (=)
  module procedure uio_tabcopy
end interface ! assignment (=)
!
contains

!----------***********------------------------------------------------------------------------------
subroutine uio_tabnull(table, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabnull ('uio_table_null')
!
! PURPOSE:
!   Initialize uio table (nullify pointers...)
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabnull(table, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be initialized
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Some pointers are nullified. Thus, this routine should not be used with
!   a filled table (with allocated pointers).
!
! PROCEDURE:
!   Set counters to zero, nullify pointers.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-01-22 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Size of all arrays is 0 ---
table%nlinmx=0
table%nlin  =0
table%ncolmx=0
table%ncol  =0
table%ntaimx=0
table%ntai  =0
table%ntarmx=0
table%ntar  =0
table%ntasmx=0
table%ntas  =0
!
! --- Nullify pointers ---
nullify(table%ntta, table%trtb)
nullify(table%i, table%r, table%s)
!
if (present(outstr)) outstr='done'
if (present(ierr  )) ierr  =0
!
end subroutine uio_tabnull


!----------*************----------------------------------------------------------------------------
subroutine uio_tabdelete(table, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabdelete ('uio_table_delete')
!
! PURPOSE:
!   Delete uio table (deallocate pointers...)
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabdelete(table, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be deleted
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Some pointers are deallocated if they are associated.
!   Thus, they have to be in a defined status.
!
! PROCEDURE:
!   Set counters to zero, nullify pointers.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-01-22 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Size of all arrays is set to 0 ---
table%nlinmx=0
table%nlin  =0
table%ncolmx=0
table%ncol  =0
table%ntaimx=0
table%ntai  =0
table%ntarmx=0
table%ntar  =0
table%ntasmx=0
table%ntas  =0
!
! --- Deallocate pointers ---
if (associated(table%ntta)) deallocate(table%ntta)
if (associated(table%trtb)) deallocate(table%trtb)
if (associated(table%i   )) deallocate(table%i   )
if (associated(table%r   )) deallocate(table%r   )
if (associated(table%s   )) deallocate(table%s   )
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_tabdelete


!----------***********------------------------------------------------------------------------------
subroutine uio_tabmake(table, plus_nlin, plus_ncol, &
                       plus_nint, plus_nreal, plus_nchar, plus_ntab, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabmake ('uio_table_make')
!
! PURPOSE:
!   Create or enlarge uio table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabmake(table, plus_nlin=plus_nlin, plus_ncol=plus_ncol, &
!                    plus_nint=plus_nint, plus_nreal=plus_nreal, &
!                    plus_nchar=plus_nchar, plus_ntab=plus_ntab, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   plus_nlin: ('plus_number_lines') integer, optional, additional number of lines
!   plus_ncol: ('plus_number_columns') integer, optional, additional number of header entries
!   plus_nint: ('plus_number_integers') integer, optional, additional number of integer entries
!   plus_nreal:('plus_number_reals') integer, optional, additional number of real entries
!   plus_nchar:('plus_number_characterss') integer, optional, additional number of character entries
!   plus_ntab: ('plus_number_table_headers') integer, optional, additional number of table headers
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be created/enlarged
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The uio table has to be in a defined status.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-01-23 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
integer,       optional, intent(in)     :: plus_nlin, plus_ncol, &
                                           plus_nint, plus_nreal, &
                                           plus_nchar, plus_ntab
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nlin_new, ncol_min, ncol_new, &
                                           nint_new, nreal_new, nchar_new
integer,         pointer, dimension(:)  :: ntta_tmp
character*(let), pointer, &
                       dimension(:,:,:) :: trtb_tmp
integer,         pointer, dimension(:,:):: i_tmp
real,            pointer, dimension(:,:):: r_tmp
character*(uio_tab_charlen), pointer, &
                          dimension(:,:):: s_tmp
!---------------------------------------------------------------------------------------------------
!
! --- New (possibly increased) number of lines ---
nlin_new=table%nlinmx
if (present(plus_nlin)) nlin_new=nlin_new+max(plus_nlin,0)
!
! --- New minimum number of columns ---
ncol_min=table%ncol
ncol_new=table%ncolmx
if (present(plus_ncol)) ncol_new=ncol_new+max(plus_ncol,0)
!
! --- Entry numbers of subarrays ---
nint_new=table%ntaimx
nreal_new=table%ntarmx
nchar_new=table%ntasmx
if (present(plus_nint)) then
  nint_new=nint_new+max(plus_nint,0)
  ncol_min=ncol_min+max(plus_nint,0)
endif
if (present(plus_nreal)) then
  nreal_new=nreal_new+max(plus_nreal,0)
  ncol_min =ncol_min +max(plus_nreal,0)
endif
if (present(plus_nchar)) then
  nchar_new=nchar_new+max(plus_nchar,0)
  ncol_min =ncol_min+max( plus_nchar,0)
endif
if (present(plus_ntab)) then
  ncol_min=ncol_min+max(plus_ntab,0)
endif
!
! --- New number of columns ---
ncol_new=max(ncol_new,ncol_min)
!
if (ncol_new > table%ncolmx) then
  ! --- Enlarge header entry table ---
  !
  if (table%ncolmx > 0) then
    ! --- Save existing table ---
    ntta_tmp=>table%ntta
    trtb_tmp=>table%trtb
  endif
  ! --- Allocate and fill new table ---
  allocate(table%ntta(ncol_new), table%trtb(2,nttmx,ncol_new))
  table%ntta=0
  table%trtb=''
  if (table%ncolmx > 0) then
    ! --- Copy existing table into new one ---
    table%ntta(1:table%ncol)=ntta_tmp(1:table%ncol)
    table%trtb(:,:,1:table%ncol)=trtb_tmp(:,:,1:table%ncol)
    ! --- Deallocate temporary tables ---
    deallocate(ntta_tmp, trtb_tmp)
  endif
  table%ncolmx=ncol_new
endif
!
if ((nlin_new > table%nlin) .or. (nint_new > table%ntaimx)) then
  ! -- Enlarge integer array ---
  !
  if (associated(table%i)) then
    i_tmp=>table%i
  endif
  allocate(table%i(nlin_new,nint_new))
  table%i=0
  if (associated(i_tmp)) then
    table%i(1:table%nlin,1:table%ntai)=i_tmp(1:table%nlin,1:table%ntai)
    deallocate(i_tmp)
  endif
  table%ntaimx=nint_new
endif
!
if ((nlin_new > table%nlin) .or. (nreal_new > table%ntarmx)) then
  ! -- Enlarge real array ---
  !
  if (associated(table%r)) then
    r_tmp=>table%r
  endif
  allocate(table%r(nlin_new,nreal_new))
  table%r=0.0
  if (associated(r_tmp)) then
    table%r(1:table%nlin,1:table%ntar)=r_tmp(1:table%nlin,1:table%ntar)
    deallocate(r_tmp)
  endif
  table%ntarmx=nreal_new
endif
!
if ((nlin_new > table%nlin) .or. (nchar_new > table%ntasmx)) then
  ! -- Enlarge character array ---
  !
  if (associated(table%s)) then
    s_tmp=>table%s
  endif
  allocate(table%s(nlin_new,nchar_new))
  table%s=''
  if (associated(s_tmp)) then
    table%s(1:table%nlin,1:table%ntas)=s_tmp(1:table%nlin,1:table%ntas)
    deallocate(s_tmp)
  endif
  table%ntasmx=nchar_new
endif
!
if (nlin_new > table%nlin) then
  ! --- Account for increased number of lines ---
  table%nlinmx=nlin_new
endif
!
if (present(outstr)) outstr=''
if (present(ierr  )) ierr  =0
!
end subroutine uio_tabmake


!----------***********------------------------------------------------------------------------------
subroutine uio_tabcopy(table, tableright)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabcopy ('uio_table_copy')
!
! PURPOSE:
!   Make copy of table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabcopy(table, tableright)
!
! INPUT:
!   tableright:  ('table_right') type(uio_table_type), table to be copied
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), copy of table
!
! OUTPUT:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The uio table has to be in a defined status.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-01-26 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(in)     :: tableright
type(uio_table_type),    intent(inout)  :: table
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Delete table ---
call uio_tabdelete(table)
!
! --- Copy header ---
!table%nlinmx=tableright%nlinmx
!table%nlin  =tableright%nlin
!table%ncolmx=tableright%ncolmx
!table%ncol  =tableright%ncol
!table%ntaimx=tableright%ntaimx
!table%ntai  =tableright%ntai
!table%ntarmx=tableright%ntarmx
!table%ntar  =tableright%ntar
!table%ntasmx=tableright%ntasmx
!table%ntas  =tableright%ntas
!
! --- Produce table of same size as old one ---
call uio_tabmake(table, &
                 plus_nlin =tableright%nlinmx, plus_ncol =tableright%ncolmx, &
                 plus_nint =tableright%ntaimx, plus_nreal=tableright%ntarmx, &
                 plus_nchar=tableright%ntasmx)
!
! --- Copy table contents ---
table%ncol=tableright%ncol
if (tableright%ncol > 0) then
  table%ntta=tableright%ntta
  table%trtb=tableright%trtb
endif
table%nlin=tableright%nlin
table%ntai=tableright%ntai
if (tableright%ntai > 0) table%i=tableright%i
table%ntar=tableright%ntar
if (tableright%ntar > 0) table%r=tableright%r
table%ntas=tableright%ntas
if (tableright%ntas > 0) table%s=tableright%s
!
end subroutine uio_tabcopy


!----------***********------------------------------------------------------------------------------
subroutine uio_tabputt(table, ident,format,name, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabputt ('uio_table_put_tableheader')
!
! PURPOSE:
!   Put table header entry into entry table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabputt(table, ident, &
!                    format=format,name=name, termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   ident:  ('identifier') character, identifier
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional,  name of variable
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be modified
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input table has to be in a defined status.
!
! PROCEDURE:
!   Write entry for table header in entry header table (tatrtb).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-10-28 (B.F. Kiel) Written
!   1998-01-23 (B.F.) uio_tabwt -> uio_tabputt
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(in)     :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, n
!---------------------------------------------------------------------------------------------------
!
! --- Check table size ---
if (table%ncol >= table%ncolmx) then
  ! --- Table is full -> enlarge table ---
  call uio_tabmake(table, plus_ncol=10)
endif
!
table%ncol=table%ncol+1
!
! --- Define header ---
nttt=0
call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, tabna)
if (ident /= '') then
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, ident)
else
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, tabna)
endif
!
if (present(format)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                     frmtna,keyval=format)
if (present(name)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   namena,keyval=name)
!
! --- Concatenate term tables ---
if (present(termt) .and. present(ntt)) then
  nttn=min(nttt+ntt,nttmx)
  table%trtb(:,nttt+1:nttn,table%ncol)=termt(:,1:nttn-nttt)
else
  nttn=nttt
endif
table%ntta(table%ncol)=nttn
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_tabputt


!----------***********------------------------------------------------------------------------------
subroutine uio_tabputi(table, val, ident, format,name,unit, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabputi ('uio_table_put_integer')
!
! PURPOSE:
!   Put integer array into table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabputi(table, val, ident, &
!                    format=format,name=name,unit=unit, termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   val:    ('value') integer, array to be written
!   ident:  ('identifier') character, identifier
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional,  name of variable
!   unit:   ('unit') character, optional, string with units
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be modified
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input table has to be in a defined status.
!
! PROCEDURE:
!   Put integer column into table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-26 (B.F. Kiel) Written
!   1996-02-02 (B.F.) Parameter nlin added, nlinimx-> nlinmx
!   1996-10-27 (B.F.) Table concatenating, ita -> nlin, parameter nc deleted
!   1998-01-23 (B.F.) uio_tabwi1 -> uio_tabputi
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
integer, dimension(:),   intent(in)     :: val
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name, unit
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(in)     :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, n, nlin_new, plus_nint
!---------------------------------------------------------------------------------------------------
!
! --- Check table size ---
if (table%ncol >= table%ncolmx) then
  ! --- Not enough columns -> enlarge table ---
  call uio_tabmake(table, plus_ncol=10)
endif
!
table%ncol=table%ncol+1
!
! --- Define header ---
nttt=0
call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, intena)
if (ident /= '') then
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, ident)
else
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, intena)
endif
!
if (present(format)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                     frmtna,keyval=format)
if (present(name)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   namena,keyval=name)
if (present(unit)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   unitna,keyval=unit)
!
! --- Concatenate term tables ---
if (present(termt) .and. present(ntt)) then
  nttn=min(nttt+ntt,nttmx)
  table%trtb(:,nttt+1:nttn,table%ncol)=termt(:,1:nttn-nttt)
else
  nttn=nttt
endif
table%ntta(table%ncol)=nttn
!
nlin_new=size(val,1)
!
! --- Check table size ---
if ((nlin_new > table%nlinmx) .or. (table%ntai >= table%ntaimx)) then
  ! --- Not enough lines or space in integer array -> enlarge table ---
  if (table%ntai >= table%ntaimx) then
    plus_nint=10
  else
    plus_nint=0
  endif
  call uio_tabmake(table, plus_nlin=nlin_new-table%nlinmx+100, &
                          plus_nint=plus_nint)
endif
!
table%nlin=max(table%nlin,nlin_new)
table%ntai=table%ntai+1
!
! --- Write 1D input array into table array ---
table%i(1:nlin_new,table%ntai)=val
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_tabputi


!----------***********------------------------------------------------------------------------------
subroutine uio_tabputr(table, val, ident, format,name,unit,tran, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabputr ('uio_table_put_real')
!
! PURPOSE:
!   Put real array into table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabputr(table, val, ident, &
!                    format=format,name=name,unit=unit,tran=tran, &
!                    termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   val:    ('value') real, array to be written
!   ident:  ('identifier') character, identifier
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional,  name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, '' or 'log10'
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be modified
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contarns exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input table has to be in a defined status.
!
! PROCEDURE:
!   Put real column into table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-26 (B.F. Kiel) Written
!   1996-02-02 (B.F.) Parameter nlin added, nlinimx-> nlinmx
!   1996-10-27 (B.F.) Table concatenating, ita -> nlin, parameter nc deleted
!   1998-01-23 (B.F.) uio_tabwi1 -> uio_tabputr
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
real(kind(0.0E0)), dimension(:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name, unit, tran
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(in)     :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, n, nlin_new, plus_nreal
character*(let)                         :: tran0
!---------------------------------------------------------------------------------------------------
!
! --- Check table size ---
if (table%ncol >= table%ncolmx) then
  ! --- Not enough columns -> enlarge table ---
  call uio_tabmake(table, plus_ncol=10)
endif
!
table%ncol=table%ncol+1
!
! --- Define header ---
nttt=0
call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, realna)
if (ident /= '') then
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, ident)
else
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, realna)
endif
!
if (present(format)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                     frmtna,keyval=format)
if (present(name)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   namena,keyval=name)
if (present(unit)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
!
! --- Concatenate term tables ---
if (present(termt) .and. present(ntt)) then
  nttn=min(nttt+ntt,nttmx)
  table%trtb(:,nttt+1:nttn,table%ncol)=termt(:,1:nttn-nttt)
else
  nttn=nttt
endif
table%ntta(table%ncol)=nttn
!
nlin_new=size(val,1)
!
! --- Check table size ---
if ((nlin_new > table%nlinmx) .or. (table%ntar >= table%ntarmx)) then
  ! --- Not enough lines or space in real array -> enlarge table ---
  if (table%ntar >= table%ntarmx) then
    plus_nreal=10
  else
    plus_nreal=0
  endif
  call uio_tabmake(table, plus_nlin=nlin_new-table%nlinmx+100, &
                          plus_nreal=plus_nreal)
endif
!
table%nlin=max(table%nlin,nlin_new)
table%ntar=table%ntar+1
!
! --- Write 1D input array into table array ---
if (tran0 == 'log10') then
  table%r(1:nlin_new,table%ntar)=alog10(val)
else
  table%r(1:nlin_new,table%ntar)=val
endif
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_tabputr


!----------***********------------------------------------------------------------------------------
subroutine uio_tabputs(table, val, ident, nbytes, format,name,unit, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabputs ('uio_table_put_string')
!
! PURPOSE:
!   Put character array into table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabputs(table, val, ident, nbytes=nbytes, &
!                    format=format,name=name,unit=unit, termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   val:    ('value') character, array to be written
!   ident:  ('identifier') character, identifier
!   nbytes: ('number_bytes') integer, optional, number of characters of variable val to be written
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional,  name of variable
!   unit:   ('unit') character, optional, string with units
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!
! INPUT/OUTPUT:
!   table:  ('table') type(uio_table_type), table to be modified
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input table has to be in a defined status.
!
! PROCEDURE:
!   Put character column into table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-26 (B.F. Kiel) Written
!   1996-02-02 (B.F.) Parameter nlin added, nlinimx-> nlinmx
!   1996-10-27 (B.F.) Table concatenating, ita -> nlin, parameter nc deleted
!   1998-01-23 (B.F.) uio_tabputi -> uio_tabputs
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),    intent(inout)  :: table
character*(*), dimension(:), intent(in) :: val
character*(*),           intent(in)     :: ident
integer,       optional, intent(in)     :: nbytes
character*(*), optional, intent(in)     :: format, name, unit
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(in)     :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, n, nlin_new, &
                                           plus_nchar, nlast
character*(let)                         :: lenval
!---------------------------------------------------------------------------------------------------
!
! --- Check table size ---
if (table%ncol >= table%ncolmx) then
  ! --- Not enough columns -> enlarge table ---
  call uio_tabmake(table, plus_ncol=10)
endif
!
table%ncol=table%ncol+1
!
! --- Define header ---
nttt=0
call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, charna)
if (ident /= '') then
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, ident)
else
  call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, charna)
endif
!
if (present(format)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                     frmtna,keyval=format)
if (present(nbytes)) then
  if (nbytes <= 0) then
    lenval=''
    nlast=len(val)
  else
    write(lenval,'(I6)') nbytes
    nlast=min(len(val),nbytes)
    lenval=adjustl(lenval)
  endif
else
  lenval=''
  nlast=len(val)
endif
call uio_adkeyw(table%trtb(:,:,table%ncol),nttt,bytena,keyval=lenval)
if (present(name)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   namena,keyval=name)
if (present(unit)) call uio_adkeyw(table%trtb(:,:,table%ncol),nttt, &
                                   unitna,keyval=unit)
!
! --- Concatenate term tables ---
if (present(termt) .and. present(ntt)) then
  nttn=min(nttt+ntt,nttmx)
  table%trtb(:,nttt+1:nttn,table%ncol)=termt(:,1:nttn-nttt)
else
  nttn=nttt
endif
table%ntta(table%ncol)=nttn
!
nlin_new=size(val,1)
!
! --- Check table size ---
if ((nlin_new > table%nlinmx) .or. (table%ntas >= table%ntasmx)) then
  ! --- Not enough lines or space in character array -> enlarge table ---
  if (table%ntas >= table%ntasmx) then
    plus_nchar=10
  else
    plus_nchar=0
  endif
  call uio_tabmake(table, plus_nlin=nlin_new-table%nlinmx+100, &
                          plus_nchar=plus_nchar)
endif
!
table%nlin=max(table%nlin,nlin_new)
table%ntas=table%ntas+1
!
! --- Write 1D input array into table array ---
table%s(1:nlin_new,table%ntas)=val(:nlast)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_tabputs


!----------*********--------------------------------------------------------------------------------
subroutine uio_wrtab(nc, table, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrtab ('uio_write_table')
!
! PURPOSE:
!   Write table of integer, real, and/or character data to file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrtab(nc, table, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   table:  ('table') type(uio_table_type), table to be written into file
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_exkeyw:            Extract value of keyword from table
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!   Maximum number of character columns is 256.
!   Maximum number of integer/real/character-blocks is 4.
!   Maximum line length is 1024 (but there is no test).
!
! PROCEDURE:
!   First, the table header is composed and written into file.
!   Then, the header for each entry is written into file.
!   Then, the data is written in formatted or unformatted form row by row.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-22 (B.F. Kiel) Written
!   1996-01-25 (B.F.)
!   1996-01-26 (B.F.) First working version, integer columns added.
!   1996-01-29 (B.F.) More comments
!   1996-02-02 (B.F.) Parameters nlinimx,nlinrmx,nlinsmx deleted
!                   parameters nlinmx,nlin added
!   1996-09-26 (B.F.) line:nti=1 added
!   1996-10-21 (B.F.) File position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd, format handling from tabw into wrtab. mini tables
!   1996-10-28 (B.F.)
!   1996-12-04 (B.F.) Error in index handling removed
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-03 (B.F.) gotos removed
!   1997-02-14 (B.F.) dim2st -> uio_dim2st
!   1998-01-27 (B.F.) table is pure input
!---------------------------------------------------------------------------------------------------
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
type(uio_table_type),    intent(in)     :: table
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
logical                                 :: foundf
integer                                 :: ntt, nttn, i, idxsfr
integer                                 :: a, b, c, d, n
integer                                 :: nti,ntr,nts, nti0,ntr0,nts0
integer                                 :: last, next, block
integer                                 :: ncolht, ncolhn, nlen
integer                                 :: l(3,0:4), r(3,0:4), z(256)
character*(let)                         :: dimstr, sfrmt, efrmt, lenval, &
                                           form, type, str, bytes
character*(1024)                        :: outun, hfrmt, tfrmt
character*(let)                         :: termt(2,nttmx)
integer,         dimension(:), &
                            allocatable :: ntta
character*(let), dimension(:,:,:), &
                            allocatable :: trtb
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
ncolhn=1
nti=1
ntr=1
nts=1
!
! --- Allocate temporary header entry arrays ---
allocate(ntta(table%ncol), trtb(2,nttmx,table%ncol))
!
! --- Loop over all mini tables in large table array ---
do while (ncolhn <= table%ncol)
  ncolht=ncolhn
  !
  ! --- Search next table header entry ---
  foundf=.false.
  do while (.not. foundf)
    ncolhn=ncolhn+1
    if (ncolhn <= table%ncol) then
      foundf=(table%trtb(1,1,ncolhn) == tabna)
    else
      exit
    endif
  end do
  !
  ! --- Define header ---
  !
  ntt=table%ntta(ncolht)
  termt(:,1:ntt)=table%trtb(:,1:ntt,ncolht)
  ntta(ncolht)=0

  call uio_adkeyw(trtb(:,:,ncolht),ntta(ncolht),tabna)
  call uio_adkeyw(trtb(:,:,ncolht),ntta(ncolht),termt(1,2))
  termt(1,1)=''
  termt(1,2)=''
  call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
  call uio_dim2st((/1,1/), (/ncolhn-ncolht-1,table%nlin/), dimstr, mode=0)
  call uio_adkeyw(trtb(:,:,ncolht),ntta(ncolht),dimna ,keyval=dimstr)
  call uio_exkeyw(termt, ntt, frmtna,sfrmt , mode=1)
  call uio_exkeyw(termt, ntt, bytena,bytes , mode=1)
  if (sfrmt == '*') sfrmt='1X'
  call uio_adkeyw(trtb(:,:,ncolht),ntta(ncolht),frmtna,keyval=sfrmt)
  if (sfrmt == '') sfrmt=','
  if (sfrmt /= ',') sfrmt=',' // trim(sfrmt) // ','
  idxsfr=len_trim(sfrmt)
  write(str,'(A1' // sfrmt(:idxsfr) // '''<'',)') '>'
  write(lenval,'(I6)') len_trim(str)-2
  lenval=adjustl(lenval)
  call uio_adkeyw(trtb(:,:,ncolht),ntta(ncolht),bytena,keyval=lenval)
  !
  ! --- Concatenate term tables ---
  nttn=min(ntta(ncolht)+ntt,nttmx)
  trtb(:,ntta(ncolht)+1:nttn,ncolht)=termt(:,1:nttn-ntta(ncolht))
  ntta(ncolht)=nttn
  !
  ! --- Put temporary term table into entry header array ---
!  table%ntta(ncolht)=nttt
!  table%trtb(:,1:nttt,ncolht)=termtt(:,1:nttt)
  !
  ! --- Write header into file ---
  call uio_wrhd(nc,form, trtb(:,:,ncolht),ntta(ncolht))
  !
  ! --- Write header data for all table entries into file ---
  nti0=nti
  ntr0=ntr
  nts0=nts
  do i=ncolht+1,ncolhn-1
    ntt=table%ntta(i)
    termt(:,1:ntt)=table%trtb(:,1:ntt,i)
    ntta(i)=0
    type=termt(1,1)
    call uio_adkeyw(trtb(:,:,i),ntta(i),type)
    call uio_adkeyw(trtb(:,:,i),ntta(i),termt(1,2))
    termt(1,1)=''
    termt(1,2)=''
    call uio_exkeyw(termt,ntt, frmtna,efrmt , mode=1)
    call uio_exkeyw(termt,ntt, bytena,lenval, mode=1)
    select case(type)
    case(intena)
      ! --- Integer ---
      if ((efrmt == '') .or. (efrmt == '*')) then
        efrmt=foinls(cvnum(nc))
      endif
      lenval=btinls(cvnum(nc))
    case(realna)
      ! --- Real ---
      if ((efrmt == '') .or. (efrmt == '*')) then
        efrmt=forels(cvnum(nc))
      endif
      lenval=btrels(cvnum(nc))
    case default
      ! --- String ---
      if (lenval == '') then
        nlen=0
        do n=1,table%nlin
          nlen=max(nlen,len_trim(table%s(n,nts)))
        end do ! n
        write(lenval,'(I6)') nlen
        lenval=adjustl(lenval)
      endif
      if ((efrmt == '') .or. (efrmt == '*')) then
        efrmt='A'//lenval
      else
      !               ?
      endif
    end select ! type
    call uio_adkeyw(trtb(:,:,i),ntta(i),frmtna,keyval=efrmt)
    call uio_adkeyw(trtb(:,:,i),ntta(i),bytena,keyval=lenval)
    !
    ! --- Concatenate term tables ---
    nttn=min(ntta(i)+ntt,nttmx)
    trtb(:,ntta(i)+1:nttn,i)=termt(:,1:nttn-ntta(i))
    ntta(i)=nttn
    !
    ! --- Put temporary term table into entry header array ---
!    table%ntta(i)=nttt
!    table%trtb(:,1:nttt,i)=termtt(:,1:nttt)
    !
    ! --- Write header into file ---
    call uio_wrhd(nc,form,trtb(:,:,i),ntta(i))
    !
    ! --- Compose single header line with entry names ---
    select case(type)
    case(intena)
      write(str,'(' // trim(efrmt) // ',''<'',)') 0
      nti=nti+1
    case(realna)
      write(str,'(' // trim(efrmt) // ',''<'',)') 0.0
      ntr=ntr+1
    case default
      write(str,'(' // trim(efrmt) // ',''<'',)') ''
      nts=nts+1
    end select ! type
    write(lenval,'(I6)') len_trim(str)-1
    lenval=adjustl(lenval)
    if (i == (ncolht+1)) then
      hfrmt='(A' // lenval
    else
      hfrmt=trim(hfrmt) // sfrmt(:idxsfr) // 'A' // lenval
    endif
  end do ! i
  hfrmt=trim(hfrmt) // ')'
  !
  ! --- Write header line to file ---
  if (form == 'unformatted') then
    write(outun,hfrmt) (trim(trtb(1,2,i)),i=ncolht+1,ncolhn-1)
    write(nc) trim(outun)
  else
    write(nc,hfrmt) (trim(trtb(1,2,i)), i=ncolht+1,ncolhn-1)
  endif
  !
  ! --- Increment file position counter ---
  ncpos(nc)=ncpos(nc)+1
  !
  ! --- Compose output format ---
  if (form == 'formatted') then
    do i=ncolht+1,ncolhn-1
      call uio_exkeyw(trtb(:,:,i),ntta(i), frmtna,efrmt)
      if (i == (ncolht+1)) then
        tfrmt='('//efrmt
      else
        tfrmt=trim(tfrmt) // sfrmt(:idxsfr) // efrmt
      endif
    end do ! i
    tfrmt=trim(tfrmt) // ')'
  endif
  !
  ! --- Determine length of output strings ---
  b=1
  do i=ncolht+1,ncolhn-1
    if (trtb(1,1,i) == charna) then
      call uio_exkeyw(trtb(:,:,i),ntta(i), bytena,lenval)
      read(lenval,'(I6)') z(b)
      b=b+1
    endif
  end do ! i
  !
  ! --- Find loop indices ---
  do i=0,4
    l(1,i)=nti0
    r(1,i)=nti0-1
    l(2,i)=ntr0
    r(2,i)=ntr0-1
    l(3,i)=nts0
    r(3,i)=nts0-1
  end do ! i
  next=10
  block=0
  !
  do i=ncolht+1,ncolhn-1
    last=next
    select case(trtb(1,1,i))
      case(intena)
        next=1
      case(realna)
        next=2
      case default
        next=3
    end select ! trtb(1,1,i)
    if (next == last) then
      r(next,block)=r(next,block)+1
    else
      if (next > last) then
        do n=last+1,next-1
          l(n,block)=r(n,block-1)+1
          r(n,block)=r(n,block-1)
        end do ! n
      else
        do n=last+1,3
          l(n,block)=r(n,block-1)+1
          r(n,block)=r(n,block-1)
        end do ! n
        block=block+1
        if (block > 4) stop 'Too many table columns'
        do n=1,next-1
          l(n,block)=r(n,block-1)+1
          r(n,block)=r(n,block-1)
        end do ! n
      endif
      l(next,block)=r(next,block-1)+1
      r(next,block)=l(next,block)
    endif
  end do ! i
!  do i=0,4
!    do n=1,3
!      write(6,*) 'n,i,l(n,i),r(n,i)', n,i,l(n,i),r(n,i)
!    end do ! n
!  end do ! i
  !
  ! --- Write table into file ---
  if (form == 'unformatted') then
    write(nc) ( &
     table%i(i,l(1,1):r(1,1)), table%r(i,l(2,1):r(2,1)), &
       (table%s(i,a)(:z(a)),a=l(3,1),r(3,1)), &
     table%i(i,l(1,2):r(1,2)), table%r(i,l(2,2):r(2,2)), &
       (table%s(i,b)(:z(b)),b=l(3,2),r(3,2)), &
     table%i(i,l(1,3):r(1,3)), table%r(i,l(2,3):r(2,3)), &
       (table%s(i,c)(:z(c)),c=l(3,3),r(3,3)), &
     table%i(i,l(1,4):r(1,4)), table%r(i,l(2,4):r(2,4)), &
       (table%s(i,d)(:z(d)),d=l(3,4),r(3,4)), &
                    i=1,table%nlin)
    !
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+1
  else
    write(nc,tfrmt) ( &
     table%i(i,l(1,1):r(1,1)), table%r(i,l(2,1):r(2,1)), &
       (table%s(i,a)(:z(a)),a=l(3,1),r(3,1)), &
     table%i(i,l(1,2):r(1,2)), table%r(i,l(2,2):r(2,2)), &
       (table%s(i,b)(:z(b)),b=l(3,2),r(3,2)), &
     table%i(i,l(1,3):r(1,3)), table%r(i,l(2,3):r(2,3)), &
       (table%s(i,c)(:z(c)),c=l(3,3),r(3,3)), &
     table%i(i,l(1,4):r(1,4)), table%r(i,l(2,4):r(2,4)), &
       (table%s(i,d)(:z(d)),d=l(3,4),r(3,4)), &
                    i=1,table%nlin)
    !
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+table%nlin
  endif
  !
  ! --- Next mini table ---
end do
!
deallocate(ntta, trtb)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_wrtab


!----------**************---------------------------------------------------------------------------
subroutine uio_tabaskhead(table, icol, type,ident,format,name,unit, termt,ntt, &
                          ncol, nlin, &
                          outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabaskhead ('uio_table_ask_header')
!
! PURPOSE:
!   Get data from table entry header or general data about table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabaskhead(table, icol=icol, &
!                       type=type, ident=ident, &
!                       format=format,name=name,unit=unit, &
!                       termt=termt,ntt=ntt, &
!                       outstr=outstr,ierr=ierr)
!
! INPUT:
!   table:  ('table') type(uio_table_type), table to be searched for entry
!   icol:   ('index_column) integer, index of requested table column
!
! OUTPUT:
!   type:   ('type') character, type of entry
!   ident:  ('identifier') character, identifier
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!   ncol:   ('number_columns') integer, number of total table entries (including header entries)
!   nlin:   ('number_lines') integer, number of lines in table
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Find variable identifier in entry table tatrtb.
!   Read corresponding keywords and extract their values.
!   Read requested 1D array from table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-01-27 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),       intent(in)  :: table
integer,          optional, intent(in)  :: icol
character*(*),    optional, intent(out) :: type,ident, format, name, unit
integer,          optional, intent(out) :: ntt
character*(*),    optional, intent(out) :: termt(2,nttmx)
integer,          optional, intent(out) :: ncol, nlin
character*(*),    optional, intent(out) :: outstr
integer,          optional, intent(out) :: ierr
!
! --- Local variables ---
integer                                 :: ncolsl
character*(let)                         :: bytes, format0
integer                                 :: nttt
character*(let)                         :: termtt(2,nttmx)
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
!
ierr0=0
outstr0='done'
!
! --- General data ---
if (present(ncol)) ncol=table%ncol
if (present(nlin)) nlin=table%nlin
!
! --- Data about entry with number icol ---
if (present(icol)) then
  if (icol >= 1) then
    if (icol <= table%ncol) then
      ncolsl=icol
    else
      ierr0=1
      outstr0='column number to small'
    endif
  else
    ierr0=2
    outstr0='column number to large'
  endif
endif
!
if (present(icol) .and. (ierr0 == 0)) then
  ! --- Put header into temporary variable ---
  nttt=table%ntta(ncolsl)
  termtt(:,1:nttt)=table%trtb(:,1:nttt,ncolsl)
  !
  ! --- Find keywords and extract their values ---
  if (present(type )) type =termtt(1,1)
  if (present(ident)) ident=termtt(1,2)
  termtt(1,1)=''
  termtt(1,2)=''
  call uio_exkeyw(termtt,nttt, frmtna,format0, mode=1)
  if (present(format)) format=format0
  if (present(unit)) call uio_exkeyw(termtt,nttt, unitna,unit  , mode=1)
  if (present(name)) call uio_exkeyw(termtt,nttt, namena,name  , mode=1)
  call uio_exkeyw(termtt,nttt, bytena,bytes  , mode=3)
  if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
  if (present(ntt)) ntt=nttt
  !
  ierr0=0
  outstr0='done'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_tabaskhead


!----------***********------------------------------------------------------------------------------
subroutine uio_tabgett(table, ident, format,name, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabgett ('uio_table_get_tableheader')
!
! PURPOSE:
!   Read table header from table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabgett(table, ident, &
!                    format=format,name=name, &
!                    termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   table:  ('table') type(uio_table_type), table to be searched for entry
!   ident:  ('identifier') character, identifier (mandatory)
!
! OUTPUT:
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Find variable identifier in entry table tatrtb.
!   Read corresponding keywords and extract their values.
!   Read requested 1D array from table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1998-01-26 (B.F. Kiel) Written
!             uio_tabgeti -> uio_tabgett
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),       intent(in)  :: table
character*(*),              intent(in)  :: ident
character*(*),    optional, intent(out) :: format, name
integer,          optional, intent(out) :: ntt
character*(*),    optional, intent(out) :: termt(2,nttmx)
character*(*),    optional, intent(out) :: outstr
integer,          optional, intent(out) :: ierr
!
! --- Local variables ---
integer                                 :: i, ncolsl, ilast
character*(let)                         :: bytes, format0
integer                                 :: nttt
character*(let)                         :: termtt(2,nttmx)
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
!
! --- Find table column ---
ncolsl=1
ierr0=0
do
  if (table%trtb(1,1,ncolsl) == tabna) then
    if (table%trtb(1,2,ncolsl) == ident) exit
  endif
  ncolsl=ncolsl+1
  if (ncolsl > table%ncol) then
    ierr0=1
    outstr0='not found'
    exit
  endif
end do
!
if (ierr0 == 0) then
  ! --- Put header into temporary variable ---
  nttt=table%ntta(ncolsl)
  termtt(:,1:nttt)=table%trtb(:,1:nttt,ncolsl)
  !
  ! --- Find keywords and extract their values ---
  termtt(1,1)=''
  termtt(1,2)=''
  call uio_exkeyw(termtt,nttt, frmtna,format0, mode=1)
  if (present(format)) format=format0
  if (present(name)) call uio_exkeyw(termtt,nttt, namena,name  , mode=1)
  call uio_exkeyw(termtt,nttt, bytena,bytes  , mode=3)
  if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
  if (present(ntt)) ntt=nttt
  !
  ierr0=0
  outstr0='done'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_tabgett


!----------***********------------------------------------------------------------------------------
subroutine uio_tabgeti(table, ident, val, format,name,unit, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabgeti ('uio_table_get_integer')
!
! PURPOSE:
!   Read 1d integer array from table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabgeti(table, ident, val, &
!                    format=format,name=name,unit=unit, &
!                    termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   table:  ('table') type(uio_table_type), table to be searched for entry
!   ident:  ('identifier') character, identifier (mandatory)
!
! OUTPUT:
!   val:    ('value') integer, 1d (column) array to be read
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Find variable identifier in entry table tatrtb.
!   Read corresponding keywords and extract their values.
!   Read requested 1D array from table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-02-04 (B.F. Kiel) Written
!   1996-10-28 (B.F.) Parameter nc removed
!   1998-01-24 (B.F.) uio_tabri1 -> uio_tabgeti
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),       intent(in)  :: table
character*(*),              intent(in)  :: ident
integer, dimension(:),      intent(out) :: val
character*(*),    optional, intent(out) :: format, name, unit
integer,          optional, intent(out) :: ntt
character*(*),    optional, intent(out) :: termt(2,nttmx)
character*(*),    optional, intent(out) :: outstr
integer,          optional, intent(out) :: ierr
!
! --- Local variables ---
integer                                 :: i, ncolsl, ntaisl, ilast
character*(let)                         :: bytes, format0
integer                                 :: nttt
character*(let)                         :: termtt(2,nttmx)
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
!
! --- Find table column ---
ntaisl=0
ncolsl=1
ierr0=0
do
  if (table%trtb(1,1,ncolsl) == intena) then
    ntaisl=ntaisl+1
    if (table%trtb(1,2,ncolsl) == ident) exit
  endif
  ncolsl=ncolsl+1
  if (ncolsl > table%ncol) then
    ierr0=1
    outstr0='not found'
    exit
  endif
end do
!
if (ierr0 == 0) then
  ! --- Put header into temporary variable ---
  nttt=table%ntta(ncolsl)
  termtt(:,1:nttt)=table%trtb(:,1:nttt,ncolsl)
  !
  ! --- Find keywords and extract their values ---
  termtt(1,1)=''
  termtt(1,2)=''
  call uio_exkeyw(termtt,nttt, frmtna,format0, mode=1)
  if (present(format)) format=format0
  if (present(name)) call uio_exkeyw(termtt,nttt, namena,name  , mode=1)
  if (present(unit)) call uio_exkeyw(termtt,nttt, unitna,unit  , mode=1)
  call uio_exkeyw(termtt,nttt, bytena,bytes  , mode=3)
  if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
  if (present(ntt)) ntt=nttt
  !
  ! --- Read 1D array from table array ---
  ilast=min(size(val,1), table%nlin)
  val(1:ilast)=table%i(1:ilast,ntaisl)
  !
  ierr0=0
  outstr0='done'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_tabgeti


!----------***********------------------------------------------------------------------------------
subroutine uio_tabgetr(table, ident, val, format,name,unit,tran, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabgetr ('uio_table_get_real')
!
! PURPOSE:
!   Read 1d real array from table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabgetr(table, ident, val, &
!                    format=format, name=name, unit=unit, tran=tran, &
!                    termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   table:  ('table') type(uio_table_type), table to be searched for entry
!   ident:  ('identifier') character, identifier (mandatory)
!
! OUTPUT:
!   val:    ('value') real, 1d (column) array to be read
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, '' or 'log10'
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Find variable identifier in entry table tatrtb.
!   Read corresponding keywords and extract their values.
!   Read requested 1D array from table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-02-04 (B.F. Kiel) Written
!   1996-10-28 (B.F.) Parameter nc removed
!   1998-01-24 (B.F.) uio_tabri1 -> uio_tabgeti -> uio_tabgetr
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),       intent(in)  :: table
character*(*),              intent(in)  :: ident
real(kind(0.0E0)), dimension(:), &
                            intent(out) :: val
character*(*),    optional, intent(out) :: format, name, unit, tran
integer,          optional, intent(out) :: ntt
character*(*),    optional, intent(out) :: termt(2,nttmx)
character*(*),    optional, intent(out) :: outstr
integer,          optional, intent(out) :: ierr
!
! --- Local variables ---
integer                                 :: i, ncolsl, ntarsl, ilast
character*(let)                         :: bytes, format0, tran0
integer                                 :: nttt
character*(let)                         :: termtt(2,nttmx)
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
!
! --- Find table column ---
ntarsl=0
ncolsl=1
ierr0=0
do
  if (table%trtb(1,1,ncolsl) == realna) then
    ntarsl=ntarsl+1
    if (table%trtb(1,2,ncolsl) == ident) exit
  endif
  ncolsl=ncolsl+1
  if (ncolsl > table%ncol) then
    ierr0=1
    outstr0='not found'
    exit
  endif
end do
!
if (ierr0 == 0) then
  ! --- Put header into temporary variable ---
  nttt=table%ntta(ncolsl)
  termtt(:,1:nttt)=table%trtb(:,1:nttt,ncolsl)
  !
  ! --- Find keywords and extract their values ---
  termtt(1,1)=''
  termtt(1,2)=''
  call uio_exkeyw(termtt,nttt, frmtna,format0, mode=1)
  if (present(format)) format=format0
  call uio_exkeyw(termtt,nttt, tranna,tran0  , mode=1)
  if (present(tran)) tran=tran0
  if (present(name)) call uio_exkeyw(termtt,nttt, namena,name  , mode=1)
  if (present(unit)) call uio_exkeyw(termtt,nttt, unitna,unit  , mode=3)
  call uio_exkeyw(termtt,nttt, bytena,bytes  , mode=3)
  if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
  if (present(ntt)) ntt=nttt
  !
  ! --- Read 1D array from table array ---
  ilast=min(size(val,1), table%nlin)
  val(1:ilast)=table%r(1:ilast,ntarsl)
  !
  ! --- Transform ---
  if (tran0 == 'log10') val(1:ilast)=10.0E0**val(1:ilast)
  !
  ierr0=0
  outstr0='done'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_tabgetr


!----------***********------------------------------------------------------------------------------
subroutine uio_tabgets(table, ident, val, nbytes, format,name, termt,ntt, &
                       outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_tabgets ('uio_table_get_string')
!
! PURPOSE:
!   Read 1d character array from table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_tabgets(table, ident, val, nbytes=nbytes, &
!                    format=format, name=name, &
!                    termt=termt,ntt=ntt, &
!                    outstr=outstr,ierr=ierr)
!
! INPUT:
!   table:  ('table') type(uio_table_type), table to be searched for entry
!   ident:  ('identifier') character, identifier (mandatory)
!
! OUTPUT:
!   val:    ('value') real, 1d (column) array to be read
!   nbytes: ('number_bytes') integer, optional, number of characters of variable val
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   termt:  ('term_table') character array, optional, 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines of termt
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Find variable identifier in entry table tatrtb.
!   Read corresponding keywords and extract their values.
!   Read requested 1D array from table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-02-04 (B.F. Kiel) Written
!   1996-10-28 (B.F.) Parameter nc removed
!   1998-01-24 (B.F.) uio_tabri1 -> uio_tabgeti -> uio_tabgetr -> uio_tabgets
!---------------------------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_table_type),       intent(in)  :: table
character*(*),              intent(in)  :: ident
character*(*), dimension(:),intent(out) :: val
integer,          optional, intent(out) :: nbytes
character*(*),    optional, intent(out) :: format, name
integer,          optional, intent(out) :: ntt
character*(*),    optional, intent(out) :: termt(2,nttmx)
character*(*),    optional, intent(out) :: outstr
integer,          optional, intent(out) :: ierr
!
! --- Local variables ---
integer                                 :: i, ncolsl, ntassl, ilast
character*(let)                         :: bytes, format0
integer                                 :: nttt
character*(let)                         :: termtt(2,nttmx)
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
!
! --- Find table column ---
ntassl=0
ncolsl=1
ierr0=0
do
  if (table%trtb(1,1,ncolsl) == charna) then
    ntassl=ntassl+1
    if (table%trtb(1,2,ncolsl) == ident) exit
  endif
  ncolsl=ncolsl+1
  if (ncolsl > table%ncol) then
    ierr0=1
    outstr0='not found'
    exit
  endif
end do
!
if (ierr0 == 0) then
  ! --- Put header into temporary variable ---
  nttt=table%ntta(ncolsl)
  termtt(:,1:nttt)=table%trtb(:,1:nttt,ncolsl)
  !
  ! --- Find keywords and extract their values ---
  termtt(1,1)=''
  termtt(1,2)=''
  call uio_exkeyw(termtt,nttt, frmtna,format0, mode=1)
  if (present(format)) format=format0
  if (present(name)) call uio_exkeyw(termtt,nttt, namena,name  , mode=1)
  call uio_exkeyw(termtt,nttt, bytena,bytes  , mode=3)
  if (present(nbytes)) read(bytes,'(I6)') nbytes
  if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
  if (present(ntt)) ntt=nttt
  !
  ! --- Read 1D array from table array ---
  ilast=min(size(val,1), table%nlin)
  val(1:ilast)=table%s(1:ilast,ntassl)
  !
  ierr0=0
  outstr0='done'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_tabgets


!----------*********--------------------------------------------------------------------------------
subroutine uio_rdtab(nc, termt,ntt, table, ident, &
                     format,name, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdtab ('uio_read_table')
!
! PURPOSE:
!   Read table of integer, real, and/or character data from file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdtab(nc, termt,ntt, table, ident, &
!                  format=format,name=name, &
!                  outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!   table:  ('table') type(uio_table_type), table to be read from file
!
! OUTPUT:
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_exkeyw:            Extract value of keyword from table
!   uio_rdhd:              Read header of variable and extract keywords
!   uio_wf2rf:             Produce from write format string read format string
!   uio_st2dim:            Parse dimension string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Extract keywords from table header and read headers for all table entries.
!   Skip single table header line. Compose input format from entry keywords.
!   Determine length of input strings (if necessary). Find loop indices for
!   integer, real and character subtables. Read table data from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-29 (B.F. Kiel) Written
!   1996-02-01 (B.F.) More comments
!   1996-02-02 (B.F.) Parameters nlinimx,nlinrmx,nlinsmx deleted; parameters nlinmx,nlin added
!   1996-10-21 (B.F.) File position counter
!   1996-10-28 (B.F.) Allow format='*', account for table header entry
!   1996-12-04 (B.F.) Error in index handling removed
!   1996-12-05 (B.F.) Double definition of ntt removed
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-02-14 (B.F.) Use uio_wf2rf; st2dim -> uio_st2dim
!   1998-01-23 (B.F.) uio_table_type
!   2000-05-03 (B.F. Copenhagen) Bug fix, add: table%ncol=1
!---------------------------------------------------------------------------------------------------
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
type(uio_table_type),    intent(inout)  :: table
character*(*),           intent(out)    :: ident
character*(*), optional, intent(out)    :: format,name
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i, j, idxsfr, nti, ntr, nts, &
                                           ncol_new, nlin_new
integer, dimension(2)                   :: ilow, iup
integer                                 :: a, b, c, d, n
integer                                 :: last, next, block
integer                                 :: l(3,0:4), r(3,0:4), z(256)
character*(let)                         :: dimstr, sfrmt, efrmt, lenval, &
                                           form, bytes, format0
character*(1024)                        :: tfrmt
integer                                 :: ierr0
character*(80)                          :: outstr0
!---------------------------------------------------------------------------------------------------
!
ierr0=0
outstr0=''
!
! --- Make minimum table ---
call uio_tabdelete(table)
call uio_tabmake(table, plus_ncol=20)
!
! --- Put term table as first entry into table header array ---
table%ncol=1
table%trtb(:,1:ntt,1)=termt(:,1:ntt)
table%ntta(1)=ntt
!
! --- Find keywords and extract their values ---
! --- Identifier ---
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Check size of table ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
call uio_st2dim(dimstr, ilow, iup, ilbound=(/1,1/), mode=1)
ncol_new=iup(1)+1
nlin_new=iup(2)
if (ncol_new > table%ncolmx) then
  ! --- Increase maximum number of table header entries ---
  call uio_tabmake(table, plus_ncol=ncol_new-table%ncolmx+10)
endif
!
call uio_exkeyw(termt,ntt, frmtna,format0, mode=1)
if (present(format)) format=format0
if (present(name  )) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
call uio_exkeyw(termt,ntt, bytena,bytes , mode=3)
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read header data for all table entries from file ---
table%ncol=ncol_new
do i=2,table%ncol
  call uio_rdhd(nc, table%trtb(:,:,i),table%ntta(i))
end do ! i
!
! --- Count number of integer, real, and character entries ---
nti=count(table%trtb(1,1,1:table%ncol) == intena)
ntr=count(table%trtb(1,1,1:table%ncol) == realna)
nts=count(table%trtb(1,1,1:table%ncol) == charna)
!
! --- Create table of proper size ---
call uio_tabmake(table, plus_nlin=nlin_new+100, &
                 plus_nint=nti+5, plus_nreal=ntr+5, plus_nchar=nts+1)
table%nlin=nlin_new
table%ntai=nti
table%ntar=ntr
table%ntas=nts
!
! --- Skip single table header line ---
if (form == 'unformatted') then
  read(nc)
else
  read(nc,'(A1)')
endif
! --- Increment file position counter ---
ncpos(nc)=ncpos(nc)+1
!
! --- Compose input format ---
if (format0 == ',') then
  sfrmt=format0
else if (format0 == '*') then
  sfrmt=',1X,'
else
  call uio_wf2rf(format0,sfrmt, qmachr=qmachr)
  sfrmt=',' // trim(sfrmt) // ','
endif
idxsfr=len_trim(sfrmt)
tfrmt=''
if ((form == 'formatted') .and. (format0 /= '*')) then
  do i=2,table%ncol
    call uio_exkeyw(table%trtb(:,:,i),table%ntta(i), frmtna,efrmt)
    call uio_wf2rf(efrmt,efrmt, qmachr=qmachr)
    if (i == 2) then
      tfrmt='(' // efrmt
    else
      tfrmt=trim(tfrmt) // sfrmt(:idxsfr) // efrmt
    endif
  end do ! i
  tfrmt=trim(tfrmt) // ')'
endif
!
! --- Determine length of input strings ---
b=1
do i=2,table%ncol
  if (table%trtb(1,1,i) == charna) then
    call uio_exkeyw(table%trtb(:,:,i),table%ntta(i), bytena,lenval)
    read(lenval,'(I6)') z(b)
    b=b+1
  endif
end do ! i
!
! --- Find loop indices ---
do i=0,4
  do j=1,3
    l(j,i)=1
    r(j,i)=0
  end do ! j
end do ! i
next=10
block=0
!
do i=2,table%ncol
  last=next
  select case(table%trtb(1,1,i))
    case(intena)
      next=1
    case(realna)
      next=2
    case default
      next=3
  end select ! table%trtb(1,1,i)
  if (next == last) then
    r(next,block)=r(next,block)+1
  else
    if (next > last) then
      do n=last+1,next-1
        l(n,block)=r(n,block-1)+1
        r(n,block)=r(n,block-1)
      end do ! n
    else
      do n=last+1,3
        l(n,block)=r(n,block-1)+1
        r(n,block)=r(n,block-1)
      end do ! n
      block=block+1
      if (block > 4) stop 'Too many table columns'
      do n=1,next-1
        l(n,block)=r(n,block-1)+1
        r(n,block)=r(n,block-1)
      end do ! n
    endif
    l(next,block)=r(next,block-1)+1
    r(next,block)=l(next,block)
  endif
end do ! i
! do i=0,4
!   do j=1,3
!     write(6,*) 'j,i,l(j,i),r(j,i)', j,i,l(j,i),r(j,i)
!   end do ! j
! end do ! i
!
! --- Read table from file ---
if (form == 'unformatted') then
  read(nc) ( &
     table%i(i,l(1,1):r(1,1)), table%r(i,l(2,1):r(2,1)), &
       (table%s(i,a)(:z(a)),a=l(3,1),r(3,1)), &
     table%i(i,l(1,2):r(1,2)), table%r(i,l(2,2):r(2,2)), &
       (table%s(i,b)(:z(b)),b=l(3,2),r(3,2)), &
     table%i(i,l(1,3):r(1,3)), table%r(i,l(2,3):r(2,3)), &
       (table%s(i,c)(:z(c)),c=l(3,3),r(3,3)), &
     table%i(i,l(1,4):r(1,4)), table%r(i,l(2,4):r(2,4)), &
       (table%s(i,d)(:z(d)),d=l(3,4),r(3,4)), &
                   i=1,table%nlin)
  !
  ! --- Increment file position counter ---
  ncpos(nc)=ncpos(nc)+1
else
 if (format0 == '*') then
   read(nc,*) ( &
     table%i(i,l(1,1):r(1,1)), table%r(i,l(2,1):r(2,1)), &
       (table%s(i,a)(:z(a)),a=l(3,1),r(3,1)), &
     table%i(i,l(1,2):r(1,2)), table%r(i,l(2,2):r(2,2)), &
       (table%s(i,b)(:z(b)),b=l(3,2),r(3,2)), &
     table%i(i,l(1,3):r(1,3)), table%r(i,l(2,3):r(2,3)), &
       (table%s(i,c)(:z(c)),c=l(3,3),r(3,3)), &
     table%i(i,l(1,4):r(1,4)), table%r(i,l(2,4):r(2,4)), &
       (table%s(i,d)(:z(d)),d=l(3,4),r(3,4)), &
                    i=1,table%nlin)
 else
   read(nc,tfrmt) ( &
     table%i(i,l(1,1):r(1,1)), table%r(i,l(2,1):r(2,1)), &
       (table%s(i,a)(:z(a)),a=l(3,1),r(3,1)), &
     table%i(i,l(1,2):r(1,2)), table%r(i,l(2,2):r(2,2)), &
       (table%s(i,b)(:z(b)),b=l(3,2),r(3,2)), &
     table%i(i,l(1,3):r(1,3)), table%r(i,l(2,3):r(2,3)), &
       (table%s(i,c)(:z(c)),c=l(3,3),r(3,3)), &
     table%i(i,l(1,4):r(1,4)), table%r(i,l(2,4):r(2,4)), &
       (table%s(i,d)(:z(d)),d=l(3,4),r(3,4)), &
                    i=1,table%nlin)
 endif
 !
 ! --- Increment file position counter ---
 ncpos(nc)=ncpos(nc)+table%nlin
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_rdtab


!----------**********-------------------------------------------------------------------------------
subroutine uio_cpentr(nc, termt,ntt, ncout, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   cpentr ('copy_entry')
!
! PURPOSE:
!   Copy entry (header + data) from one file to another.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_cpentr(nc, termt,ntt, ncout, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number for input
!   ncout:  ('number_channel_out') integer, channel unit number for output
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_rdfifo:            Read file header
!   uio_rdlabl:            Read label
!   uio_rdtab:             Read table of integer, real, and/or character data from file
!   uio_skipda:            Skip data block
!   uio_st2dim:            Parse dimension string
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrfifo:            Write file header
!   uio_wrhd:              Write header of variable, input: term table
!   uio_wrlabl:            Write label
!   wrtab:                 Write table of integer, real, and/or character data to file
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_table_module:      Table input/output/manipulation
!
! SIDE EFFECTS:
!   Data are read from file on unit nc and written into file on unit ncout.
!
! RESTRICTIONS:
!   The lengths of strings is limited to 256 characters.
!   The lengths of strings inside tables is limited to 80 characters.
!
! PROCEDURE:
!   Extract variable type from header, read data from one file (unit nc), write
!   header and data to another file (unit ncout).
!
! EXAMPLE:
!   call uio_rdhd(ncin, termt,ntt, outstr=outstr,ierr=ierr)
!   if (outstr /= 'eof') then
!     ! --- Copy entry ---
!     call uio_cpentr(ncin, termt,ntt, ncout, outstr=outstr,ierr=ierr)
!   endif
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-10-19 (B.F.) cpentry extracted from program uiocop, correction: rdd->rdc
!   1996-10-28 (B.F.) Copy table
!   1997-02-17 (B.F.) Fortran90 version
!   1997-02-20 (B.F.) Multi-dimensional arrays
!   1998-01-22 (B.F.) Minor modifications
!   1998-01-23 (B.F.) From uio_copy_module into uio_table_module
!   1998-01-24 (B.F.) Introduce uio_table_type
!---------------------------------------------------------------------------------------------------
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc, ncout
integer,                  intent(inout) :: ntt
character*(let),          intent(inout) :: termt(2,nttmx)
integer,        optional, intent(out)   :: ierr
character*(80), optional, intent(out)   :: outstr
!
! --- Local variables ---
integer                                 :: nbytes, nbtrels, ndim, nperlin, ierr0
character*(80)                          :: outstr0
character*(let)                         :: dimstr, type, ident, format, name, &
                                           unit, tran, date, bytes
integer, dimension(7)                   :: ilow, iup
! --- All possible variables ---
! --- Integer ---
integer                                 :: i
integer,              allocatable       :: i1(:),i2(:,:),i3(:,:,:),i4(:,:,:,:)
! --- Real ---
real(kind(0.0E0))                       :: r
real(kind(0.0E0)),    allocatable       :: r1(:),r2(:,:),r3(:,:,:),r4(:,:,:,:)
! --- Double precision real ---
real(kind(0.0D0))                       :: d
real(kind(0.0D0)),    allocatable       :: d1(:),d2(:,:),d3(:,:,:),d4(:,:,:,:)
! --- Complex ---
complex(kind(0.0E0))                    :: c
complex(kind(0.0E0)), allocatable       :: c1(:),c2(:,:),c3(:,:,:),c4(:,:,:,:)
! --- String ---
character*(256)                         :: s
character*(256),      allocatable       :: s1(:),s2(:,:),s3(:,:,:),s4(:,:,:,:)
!
! --- Table ---
type(uio_table_type)                    :: table
!integer                                 :: nlinmx,nlin, &
!                                           ntaimx,ntai, ntarmx,ntar, &
!                                           ntasmx,ntas, ncolmx,ncol
!integer,         allocatable            :: ntta(:)
!character*(let), allocatable            :: tatrtb(:,:,:)
!integer,         allocatable            :: tai(:,:)
!real,            allocatable            :: tar(:,:)
!character*(80),  allocatable            :: tas(:,:)
!parameter(nlinmx=2000, &
!          ntaimx=20,  ntarmx=50,  ntasmx=10, &
!          ncolmx=ntaimx+ntarmx+ntasmx)
!integer         ntta(ncolmx)
!character*(let) tatrtb(2,nttmx,ncolmx)
!integer         tai(nlinmx,ntaimx)
!real            tar(nlinmx,ntarmx)
!character*(80)  tas(nlinmx,ntasmx)
!---------------------------------------------------------------------------------------------------
!
! --- Determine type and dimension (if any) ---
type =termt(1,1)
ident=termt(1,2)
call uio_exkeyw(termt,ntt, bytena, bytes)
if (bytes == '') then
  nbytes=0
else
  read(bytes,'(I6)') nbytes
endif
call uio_exkeyw(termt,ntt,  dimna,dimstr)
if (dimstr /= '') then
  call uio_st2dim(dimstr, ilow, iup, ndim=ndim)
else
  ndim=0
endif
!
read(btrels(cvnum(nc)),'(I6)') nbtrels
!read(btdbls(cvnum(nc)),'(I6)') nbtdbls
!
! --- Check type of data block ---
if (type == intena) then
  ! --- Integer ---
  if (ndim == 0) then
    ! --- Scalar ---
    call uio_rd(nc,termt,ntt, i, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, i, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
  else if (ndim == 1) then
    ! --- 1D array ---
    allocate(i1(ilow(1):iup(1)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, i1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, i1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(i1)
  else if (ndim == 2) then
    ! --- 2D array ---
    allocate(i2(ilow(1):iup(1),ilow(2):iup(2)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, i2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, i2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(i2)
  else if (ndim == 3) then
    ! --- 3D array ---
    allocate(i3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, i3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, i3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(i3)
  else if (ndim == 4) then
    ! --- 4D array ---
    allocate(i4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3),ilow(4):iup(4)), &
             stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, i4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, i4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(i4)
  else
    outstr0=trim(type) // ' ' // trim(ident) // ' with too many dimensions'
    ierr0=1
    call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                 errlevel=1)
    ! --- Skip data set ---
    call uio_skipda(nc,termt,ntt)
  endif
!
else if ((type == realna) .and. (nbytes <= nbtrels)) then
  ! --- Real ---
  if (ndim == 0) then
    ! --- Scalar ---
    call uio_rd(nc,termt,ntt, r, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, r, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
  else if (ndim == 1) then
    ! --- 1D array ---
    allocate(r1(ilow(1):iup(1)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, r1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, r1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(r1)
  else if (ndim == 2) then
    ! --- 2D array ---
    allocate(r2(ilow(1):iup(1),ilow(2):iup(2)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, r2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, r2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(r2)
  else if (ndim == 3) then
    ! --- 3D array ---
    allocate(r3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, r3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, r3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(r3)
  else if (ndim == 4) then
    ! --- 4D array ---
    allocate(r4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3),ilow(4):iup(4)), &
             stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, r4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, r4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(r4)
  else
    outstr0=trim(type) // ' ' // trim(ident) // ' with too many dimensions'
    ierr0=1
    call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                 errlevel=1)
    ! --- Skip data set ---
    call uio_skipda(nc,termt,ntt)
  endif
!
else if ((type == realna) .and. (nbytes > nbtrels)) then
  ! --- Double precision real ---
  if (ndim == 0) then
    ! --- Scalar ---
    call uio_rd(nc,termt,ntt, d, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, d, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
  else if (ndim == 1) then
    ! --- 1D array ---
    allocate(d1(ilow(1):iup(1)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, d1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, d1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(d1)
  else if (ndim == 2) then
    ! --- 2D array ---
    allocate(d2(ilow(1):iup(1),ilow(2):iup(2)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, d2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, d2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(d2)
  else if (ndim == 3) then
    ! --- 3D array ---
    allocate(d3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, d3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, d3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(d3)
  else if (ndim == 4) then
    ! --- 4D array ---
    allocate(d4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3),ilow(4):iup(4)), &
             stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, d4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, d4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(d4)
  else
    outstr0=trim(type) // ' ' // trim(ident) // ' with too many dimensions'
    ierr0=1
    call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                 errlevel=1)
    ! --- Skip data set ---
    call uio_skipda(nc,termt,ntt)
  endif
!
else if (type == compna) then
  ! --- Complex ---
  if (ndim == 0) then
    ! --- Scalar ---
    call uio_rd(nc,termt,ntt, c, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, c, ident, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
  else if (ndim == 1) then
    ! --- 1D array ---
    allocate(c1(ilow(1):iup(1)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, c1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, c1, ident, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(c1)
  else if (ndim == 2) then
    ! --- 2D array ---
    allocate(c2(ilow(1):iup(1),ilow(2):iup(2)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, c2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, c2, ident, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(c2)
  else if (ndim == 3) then
    ! --- 3D array ---
    allocate(c3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, c3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, c3, ident, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(c3)
  else if (ndim == 4) then
    ! --- 4D array ---
    allocate(c4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3),ilow(4):iup(4)), &
             stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, c4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, c4, ident, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(c4)
  else
    outstr0=trim(type) // ' ' // trim(ident) // ' with too many dimensions'
    ierr0=1
    call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                 errlevel=1)
    ! --- Skip data set ---
    call uio_skipda(nc,termt,ntt)
  endif
!
else if (type == charna) then
  ! --- Character ---
  if (ndim == 0) then
    ! --- Scalar ---
    call uio_rd(nc,termt,ntt, s, ident, nbytes=nbytes, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, s, ident, nbytes=nbytes, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
  else if (ndim == 1) then
    ! --- 1D array ---
    allocate(s1(ilow(1):iup(1)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, s1, ident, nbytes=nbytes, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, s1, ident, nbytes=nbytes, &
                ilb=ilow(1:1), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(s1)
  else if (ndim == 2) then
    ! --- 2D array ---
    allocate(s2(ilow(1):iup(1),ilow(2):iup(2)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, s2, ident, nbytes=nbytes, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, s2, ident, nbytes=nbytes, &
                ilb=ilow(1:2), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(s2)
  else if (ndim == 3) then
    ! --- 3D array ---
    allocate(s3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)),stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, s3, ident, nbytes=nbytes, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, s3, ident, nbytes=nbytes, &
                ilb=ilow(1:3), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(s3)
  else if (ndim == 4) then
    ! --- 4D array ---
    allocate(s4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3),ilow(4):iup(4)), &
             stat=ierr0)
    if (ierr0 /= 0) then
      outstr0='error in allocation of ' // trim(type) // ' ' // ident
      call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                   errlevel=3)
    endif
    call uio_rd(nc,termt,ntt, s4, ident, nbytes=nbytes, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date)
    call uio_wr(ncout, s4, ident, nbytes=nbytes, &
                ilb=ilow(1:4), nperlin=nperlin, &
                format=format, name=name, unit=unit, tran=tran, date=date, &
                termt=termt,ntt=ntt)
    deallocate(s4)
  else
    outstr0=trim(type) // ' ' // trim(ident) // ' with too many dimensions'
    ierr0=1
    call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
                 errlevel=1)
    ! --- Skip data set ---
    call uio_skipda(nc,termt,ntt)
  endif
!
else if (type == fifona) then
  ! --- fileform entry ---
  call uio_rdfifo(nc, termt,ntt)
  !call uio_wrfifo(ncout, termt=termt,ntt=ntt)
  call uio_wrfifo(ncout)
!
else if (type == lablna) then
  ! --- Label ---
  call uio_rdlabl(nc, termt,ntt, ident, name=name,date=date)
  call uio_wrlabl(ncout,ident, name=name,date=date, termt=termt,ntt=ntt)
!
else if (type == tabna) then
  ! --- Table ---
!  ncolmx=iup(1)-ilow(1)+2
!  nlinmx=iup(2)-ilow(2)+1
!  ntaimx=ncolmx
!  ntarmx=ncolmx
!  ntasmx=ncolmx
!  allocate(ntta(ncolmx), tatrtb(2,nttmx,ncolmx), &
!           tai(nlinmx,ntaimx), tar(nlinmx,ntarmx), tas(nlinmx,ntasmx), &
!           stat=ierr0)
!  if (ierr0 /= 0) then
!    outstr0='error in allocation of ' // trim(type) // ' ' // ident
!    call uio_err(outstr=outstr0,ierr=ierr0,nc=nc,routine='uio_cpentr', &
!                   errlevel=3)
!  endif
  call uio_tabnull(table)
  call uio_rdtab(nc, termt,ntt, table, ident)
  call uio_wrtab(ncout, table)
  call uio_tabdelete(table)
!
!  call uio_rdtab(nc, termt,ntt, nlinmx,nlin, &
!                 tai,ntaimx,ntai, tar,ntarmx,ntar, tas,ntasmx,ntas, &
!                 tatrtb,ntta,ncolmx,ncol, ident,format,name, &
!                 outstr=outstr0,ierr=ierr0)
!  call uio_wrtab(ncout, nlinmx,nlin, &
!                 tai,ntaimx,ntai, tar,ntarmx,ntar, tas,ntasmx,ntas, &
!                 tatrtb,ntta,ncolmx,ncol, outstr=outstr0,ierr=ierr0)
!  deallocate(ntta, tatrtb, tai,tar,tas)
else
  outstr0='unknown type: ' // trim(type) // ' ' // trim(ident)
  ierr0=1
  call uio_err(outstr=outstr0,ierr=ierr, routine='uio_cpentr', errlevel=3)
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_cpentr

end module uio_table_module
