!******************************************************************************
!       Universal Input/Output Routines
!       uio flexible variable file type
!       27-Mar-1998 13:25:03
!******************************************************************************
! This set of routines allows the basic handling of internal files of
! uio flexible variables.
!******************************************************************************
! Non-standard FORTRAN:
!   Hopefully none!
!******************************************************************************


!------******************------------------------------------------------------
module uio_varfile_module
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_module ('uio_variable_file_module')
!
! PURPOSE:
!   Contain uio-routines for the basic handling of internal files of
!   uio flexible variables.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_varfile_module
!
! ROUTINES: (CONTAINED)
!   uio_varfile_askhead: Get some information about uio varfile
!   uio_varfile_search:  Search entry in uio varfile
!   uio_varfile_null:    Initialize uio varfile. Nullify pointers
!   uio_varfile_make:    Make uio varfile of requested size or enlarge it
!   uio_varfile_delete:  Delete uio varfile. Deallocate pointers
!   uio_varfile_copy:    Copy uio varfile
!   uio_varfile_move:    Move uio varfile
!   uio_varfile_insert:  Insert uio varfile into another one
!   uio_varfile_goto:    Change position in uio varfile
!   uio_varfile_pointput:Put uio flexible variable into uio varfile via pointing
!   uio_varfile_put:     Put uio flexible variable into uio varfile via copying
!   uio_varfile_rd:      Read uio varfile from external file
!   uio_varfile_wr:      Write uio varfile into external file
!   uio_varfile_get:     Get uio flexible variable from uio varfile via copying
!   uio_varfile_pointget:Get uio flexible variable from uio varfile via pointing
!
! PARAMETERS:
!
! TYPES:
!   uio_varpointer_type: pointer to uio flexible variable
!   uio_varfile_type:    file of uio flexible variables
!     ident:    ('identifier') character, file identifier (not used)
!     type:     ('type') character, file type: 'file' (not used)
!     status:   ('status') integer, file status:
!               0: completely empty
!               1: at least entry array present
!     nentry:   ('number_entry') integer, entry number (0...nmaxentry)
!     nmaxentry:('number_maximum_entry') integer, maximum entry number (size of
!               file array)
!     position: ('position') integer, file position counter (0...nentry)
!     file:     ('file') array of uio_varpointer_type, list of pointers to
!               all entries
!
! MODULES:
!   uio_var_module: uio-routines for uio flexible variable type
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   29. 1.98 (B.F.)
!   18. 3.98 (B.F.)
!   24. 3.98 (B.F.) uio_varfile_search improved
!------------------------------------------------------------------------------
use uio_var_module
!
implicit none
!
type uio_varpointer_type
  private
  type(uio_var_type), pointer           :: var
end type uio_varpointer_type
!
type uio_varfile_type
  private
  character*(80)                        :: ident, type
  integer                               :: status, nentry, nmaxentry, position
  type(uio_varpointer_type), pointer, &
                           dimension(:) :: file
end type uio_varfile_type
!
interface assignment (=)
  module procedure uio_varfile_copy
end interface ! assignment (=)
!
contains
!
!----------*******************-------------------------------------------------
subroutine uio_varfile_askhead(varfile, nentry,position, &
                               type,ident,nkind,ndim,name, &
                               outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_askhead ('uio_variable_file_askhead')
!
! PURPOSE:
!   Get some information about uio varfile and about current entry.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_askhead(varfile, nentry=nentry, position=position, &
!                            type=type, ident=ident, nkind=nkind, ndim=ndim, &
!                            name=name, &
!                            outstr=outstr,ierr=ierr)
!
! INPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be checked
!
! OUTPUT:
!   nentry: ('number_entry') integer, optional, number of file entries
!   position:('position') integer, optional, current file position
!   ident:  ('identifier') character, optional, identifier
!   type:   ('type') character, optional, type
!   nkind:  ('number_kind') integer, optional, kind of variable
!   ndim:   ('number_dimension') integer, optional, number of dimensions
!   name:   ('name') character, optional, name of variable
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_siz_module:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   29. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   27. 3.98 (B.F.) Proper check of EOF
!------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(in)     :: varfile
integer,       optional, intent(out)    :: nentry, position
integer,       optional, intent(out)    :: nkind, ndim
character*(*), optional, intent(out)    :: type, ident, name
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(let)                         :: type0, ident0, name0
integer                                 :: status0, ndim0, nkind0
character*(80)                          :: outstr0
integer                                 :: ierr0
!------------------------------------------------------------------------------
!
! --- General data ---
if (present(nentry  )) nentry  =varfile%nentry
if (present(position)) position=varfile%position
!
outstr0='done'
ierr0=0
!
if (varfile%position < varfile%nentry) then
  if ( present(type) .or. present(ident) .or. present(nkind) .or. &
       present(ndim) .or. present(name )                           ) then
    ! --- File is non-empty and file position counter shows to existing entry --
    call uio_var_askhead(varfile%file(varfile%position+1)%var, status=status0, &
                         ident=ident0, type=type0, nkind=nkind0, ndim=ndim0, &
                         name=name0)
    if (status0 > 0) then
      ! --- Entry is a filled uio flexible variable ---
      if (present(ident)) ident=ident0
      if (present(type )) type =type0
      if (present(nkind)) nkind=nkind0
      if (present(ndim )) ndim =ndim0
      if (present(name )) name =name0
    else
      outstr0='entry is undefined'
      ierr0=1
    endif
  endif
else
  outstr0='eof'
  ierr0=0
endif
!
if (present(ierr)) ierr=ierr0
if (present(outstr)) outstr=outstr0
!
end subroutine uio_varfile_askhead


!----------******************--------------------------------------------------
subroutine uio_varfile_search(varfile, &
                              type,ident, srchls,nsrc, position, &
                              outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_search ('uio_variable_file_search')
!
! PURPOSE:
!   Search entry in uio varfile.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_search(varfile, &
!                           type=type,ident=ident, srchls=srchls,nsrc=nsrc, &
!                           position=position, &
!                           outstr=outstr,ierr=ierr)
!
! INPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be searched in
!   ident:  ('identifier') character, optional, identifier
!   type:   ('type') character, optional, type
!   srchls: ('search_list') character array (2,nsrcmx), optional,list with
!           names of types and/or identifiers of variables to be searched for
!   nsrc:   ('number_search') integer, optional, number of variables to be
!           searched for
!   position: ('position') integer, optional, end position
!
! OUTPUT:
!   outstr: ('output_string') character, optional:
!           'found':        Entry found
!           'eof':          End of file reached, entry not found
!           'end position': End position reached, entry not found
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_varfile_askhead:
!   uio_varfile_goto:
!
! MODULES:
!   uio_siz_module:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!    5. 3.98 Written by Bernd Freytag, Astrophysik Kiel
!   24. 3.98 (B.F.) Parameter position added
!------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
character*(*), optional, intent(in)     :: type, ident
integer,       optional, intent(in)     :: nsrc
character*(*), optional, intent(in)     :: srchls(:,:)
integer,       optional, intent(in)     :: position
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nsrc0, nsrc1, n, pos
integer, parameter                      :: nsrcmx0=20
character*(let)                         :: srchls0(2,nsrcmx0)
character*(let)                         :: type0, ident0
character*(80)                          :: outstr0
integer                                 :: ierr0
logical                                 :: found
!------------------------------------------------------------------------------
!
! --- Complete search list ---
nsrc0=0
if (present(type) .or. present(ident)) then
  ! --- Put type and identifier into search list ---
  nsrc0=nsrc0+1
  srchls0(:,nsrc0)=''
  if (present(type)) srchls0(1,nsrc0)=type
  if (present(ident)) srchls0(2,nsrc0)=ident
endif
if (present(srchls)) then
  ! --- Put input search list into actual search list ---
  if (present(nsrc)) then
    nsrc1=nsrc
  else
    nsrc1=size(srchls,2)
  endif
  nsrc1=min(nsrc0+nsrc1,nsrcmx0)
  srchls0(:,nsrc0+1:nsrc0+nsrc1)=srchls(:,1:nsrc1)
  nsrc0=nsrc1
endif
!
! --- Search in varfile ---
! --- Loop over search list until variable found or end of list ---
ENTRYLOOP: do
  ! --- Read next header ---
  call uio_varfile_askhead(varfile, type=type0, ident=ident0, position=pos, &
                           outstr=outstr0)
  ! --- Check EOF ---
  if (outstr0 == 'eof') then
    found=.false.
    ierr0=1
    outstr0='eof'
    exit ENTRYLOOP
  endif
  ! --- Check end position ---
  if (present(position)) then
    if (pos >= position) then
      found=.false.
      ierr0=2
      outstr0='end position'
      exit ENTRYLOOP
    endif
  endif
  !
  found=.true.
  NSRCLOOP2: do n=1,nsrc0
    if (srchls0(1,n) == '') then
      found=srchls0(2,n) == ident0
    else if (srchls0(1,n) /= type0) then
      found=.false.
    else if (srchls0(2,n) == '') then
      found=.true.
    else
      found=srchls0(2,n) == ident0
    endif
    if (found) exit NSRCLOOP2
  end do NSRCLOOP2 ! n
  !
  if (found) then
    exit ENTRYLOOP
  endif
  !
  ! --- Skip entry ---
  call uio_varfile_goto(varfile, 'next')
end do ENTRYLOOP
!
if (found) then
  ! --- Entry found in list ---
  ierr0=0
  outstr0='found'
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_varfile_search


!----------****************----------------------------------------------------
subroutine uio_varfile_null(varfile, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_null ('uio_variable_file_null')
!
! PURPOSE:
!   Initialize uio varfile. Nullify pointers.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_null(varfile, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be initialized
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   A pointer is nullified. The routine should only be used for the
!   initialization of a variable of type uio_varfile_type.
!
! PROCEDURE:
!   Delete header entries (Set counter to zero, description string to '').
!   Nullify pointer an file array.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   28. 1.98 (B.F.) uio_varfile_init -> uio_varfile_null
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(out)    :: varfile
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i
!------------------------------------------------------------------------------
!
varfile%ident    =''
varfile%type     ='file'
varfile%status   =0
varfile%nmaxentry=0
varfile%nentry   =0
varfile%position =0
!
nullify(varfile%file)
!
if (present(ierr)) ierr=0
if (present(outstr)) outstr='done'
!
end subroutine uio_varfile_null


!----------****************----------------------------------------------------
subroutine uio_varfile_make(varfile, plus_nentry, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_make ('uio_variable_file_make')
!
! PURPOSE:
!   Make uio varfile of requested size or enlarge file array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_make(varfile, plus_nentry=plus_nentry, &
!                         outstr=outstr,ierr=ierr)
!
! INPUT:
!   plus_nentry: ('plus_number_variable') integer, optional, number of
!
! INPUT/OUTPUT:
!   varfile: ('variable_file') uio_varfile_type, uio varfile to be initialized
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   A pointer is nullified. The routine should only be used for the
!   initialization of a variable of type uio_varfile_type.
!
! PROCEDURE:
!   Delete header entries (Set counter to zero, description string to '').
!   Nullify pointer an file array.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   28. 1.98 (B.F.) uio_varfile_init -> uio_varfile_make
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
integer,       optional, intent(in)     :: plus_nentry
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i, nentry_new
type(uio_varpointer_type), pointer, &
                           dimension(:) :: file_tmp
character*(80)                          :: outstr0
!------------------------------------------------------------------------------
!
if (present(plus_nentry)) then
  ! --- Compute new number of entries ---
  nentry_new=varfile%nmaxentry+max(plus_nentry,0)
endif
!
if (nentry_new > varfile%nmaxentry) then
  ! --- Enlarge file entry array ---
  !
  if (varfile%nmaxentry > 0) then
    ! --- Save existing file entry array into temporary array ---
    file_tmp=>varfile%file
  endif
  !
  ! --- Allocate and nullify new array ---
  allocate(varfile%file(nentry_new))  
  do i=1,nentry_new
    nullify(varfile%file(i)%var)
  end do ! i
  !
  if (varfile%nmaxentry > 0) then
    ! --- Copy existing array into new one ---
    do i=1,varfile%nmaxentry
      varfile%file(i)%var=>file_tmp(i)%var
    end do ! i
    ! --- Deallocate temporary array ---
    deallocate(file_tmp)
  endif
  !
  ! --- Update entry number ---
  varfile%nmaxentry=nentry_new
  !
  outstr0='file enlarged'
else
  outstr0='file created'
endif
!
if (varfile%nmaxentry > 0) varfile%status=1
!
if (present(ierr)) ierr=0
if (present(outstr)) outstr=outstr0
!
end subroutine uio_varfile_make


!----------****************----------------------------------------------------
subroutine uio_varfile_copy(varfile, varfileright)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_copy ('uio_variable_file_copy')
!
! PURPOSE:
!   Copy uio varfile.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_copy(varfile, varfileright)
!
! INPUT:
!   varfileright: ('variable_file_right') uio_varfile_type, uio varfile to be
!                 copied
!
! OUTPUT:
!   varfile: ('variable_file') uio_varfile_type, copy of uio varfile
!
! ROUTINES:
!   uio_varfile_delete: Delete uio varfile. Deallocate pointers
!   uio_varfile_make:   Make uio varfile of requested size or enlarge file array
!   uio_var_null::      Set uio flexible variable to zero (initialization)
!   var=var:            Copy uio flexible variable (uio_var_copy)
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The resulting uio_varfile has to be in a defined status.
!
! PROCEDURE:
!   The header of the varfile is copied.
!   An empty file entry array of proper size is produced.
!   All files are copied properly. Effect:
!     varfile = varfileright
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   28. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
type(uio_varfile_type),  intent(in)     :: varfileright
!
! --- Local variables ---
integer                                 :: i
!------------------------------------------------------------------------------
!
! --- Delete target varfile ---
call uio_varfile_delete(varfile)
!
if (varfileright%status > 0) then
  ! --- Produce empty table of proper size ---
  call uio_varfile_make(varfile, plus_nentry=varfileright%nmaxentry)
  !
  varfile%type     =varfileright%type
  varfile%ident    =varfileright%ident
  varfile%status   =varfileright%status
  varfile%nentry   =varfileright%nentry
  varfile%position =varfileright%position
  !
  ! --- Copy existing array into new one ---
  do i=1,varfileright%nentry
    allocate(varfile%file(i)%var)
    call uio_var_null(varfile%file(i)%var)
    varfile%file(i)%var=varfileright%file(i)%var
  end do ! i
endif
!
end subroutine uio_varfile_copy


!----------****************----------------------------------------------------
subroutine uio_varfile_move(varfilesource, varfiletarget)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_move ('uio_variable_file_move')
!
! PURPOSE:
!   Move uio varfile.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_move(varfiletarget, varfilesource)
!
! INPUT:
!   varfilesource: ('variable_file_source') uio_varfile_type, uio varfile to be
!                  moved
!
! OUTPUT:
!   varfiletarget: ('variable_file_target') uio_varfile_type, new uio varfile
!
! ROUTINES:
!   uio_varfile_delete: Delete uio varfile. Deallocate pointers
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The resulting uio_varfile has to be in a defined status.
!
! PROCEDURE:
!   The header of the varfile is copied.
!   The file entry array is copied via pointing.
!   The old varfile is deleted.
!   Effect: mv varfilesource varfiletarget
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!    9. 3.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfilesource, varfiletarget
!
! --- Local variables ---
!------------------------------------------------------------------------------
!
! --- Delete target varfile ---
call uio_varfile_delete(varfiletarget)
!
if (varfilesource%status > 0) then
  ! --- Copy file header information ---
  varfiletarget%type     =varfilesource%type
  varfiletarget%ident    =varfilesource%ident
  varfiletarget%status   =varfilesource%status
  varfiletarget%nentry   =varfilesource%nentry
  varfiletarget%nmaxentry=varfilesource%nmaxentry
  varfiletarget%position =varfilesource%position
  !
  ! --- Move existing file into new varfile ---
  varfiletarget%file=>varfilesource%file
  !
  ! --- Delete target varfile ---
  nullify(varfilesource%file)
  varfilesource%type     =''
  varfilesource%ident    =''
  varfilesource%status   =0
  varfilesource%nentry   =0
  varfilesource%nmaxentry=0
  varfilesource%position =0
endif
!
end subroutine uio_varfile_move


!----------******************--------------------------------------------------
subroutine uio_varfile_insert(varfile, varfileright)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_insert ('uio_variable_file_insert')
!
! PURPOSE:
!   Insert uio varfile into another one
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_insert(varfile, varfileright)
!
! INPUT/OUTPUT:
!   varfileright: ('variable_file_right') uio_varfile_type, uio varfile to be
!                 inserted
!
! OUTPUT:
!   varfile: ('variable_file') uio_varfile_type, uio varfile in which
!            varfileright is inserted
!
! ROUTINES:
!   uio_varfile_delete:  Delete uio varfile. Deallocate pointers
!   uio_varfile_move:    Move uio varfile
!   uio_varfile_pointput:Put uio flexible variable into uio varfile via pointing
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The resulting uio_varfile has to be in a defined status.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!    4.11.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile, varfileright
!
! --- Local variables ---
integer                                 :: i
!------------------------------------------------------------------------------
!
if (varfileright%status > 0) then
  if (varfile%status == 0) then
    ! --- Just move varfileright ---
    !
    call uio_varfile_move(varfileright, varfile)
  else
    ! --- Insert all entries of varfileright into varfile ---
    !
    ! --- Move existing entries into new varfile ---
    do i=1,varfileright%nentry
      call uio_varfile_pointput(varfile, varfileright%file(i)%var)
      nullify(varfileright%file(i)%var)
    end do ! i
    !
    ! --- All entries are moved, set number of entries to zero ---
    varfileright%nentry=0
    !
    ! --- Delete varfileright ---
    call uio_varfile_delete(varfileright)
  endif
endif
!
end subroutine uio_varfile_insert


!----------******************--------------------------------------------------
subroutine uio_varfile_delete(varfile, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_delete ('uio_variable_file_delete')
!
! PURPOSE:
!   Delete uio varfile. Deallocate pointers.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_delete(varfile, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be deleted
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The uio_varfile_type variable has to be in a defined status.
!
! PROCEDURE:
!   Delete header entries (Set counter to zero, description string to '').
!   Delete all entries in file array. Deallocate file array.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   28. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!    6. 3.98 (B.F.) nentry -> nmaxentry
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i
!------------------------------------------------------------------------------
!
if (varfile%status > 0) then
  ! --- Delete file array ---
  do i=1,varfile%nentry
    call uio_var_delete(varfile%file(i)%var)
    deallocate(varfile%file(i)%var)
  end do ! i
  !
  deallocate(varfile%file)
endif
!
! --- Delete header ---
varfile%ident    =''
varfile%status   =0
varfile%nmaxentry=0
varfile%nentry   =0
varfile%position =0
!
if (present(ierr)) ierr=0
if (present(outstr)) outstr='done'
!
end subroutine uio_varfile_delete


!----------****************----------------------------------------------------
subroutine uio_varfile_goto(varfile, command, position, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_goto ('uio_variable_file_goto')
!
! PURPOSE:
!   Change position in uio varfile.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_goto(varfile, command, position=position, &
!                         outstr=outstr,ierr=ierr)
!
! INPUT:
!   command: ('command') character, command string
!            'rewind':            rewind file: goto first entry
!            'end', 'eof':        goto end of file (behind last entry)
!            'back', 'backspace': go one entry backward or stay at top of file
!            'next', 'skip':      go one entry forward or stay at EOF
!            'goto':              got to specified file position
!
!   position:('position') integer, file position, default=0
!
! INPUT/OUTPUT:
!   varfile: ('variable_file') uio_varfile_type, uio varfile to be initialized
!
! OUTPUT:
!   outstr:  ('output_string') character, optional
!   ierr:    ('integer_error') integer, optional, contains exit error status
!             0: correct operation
!            >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Change value of file position counter according to command.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   28. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   19. 2.98 (B.F.) max -> min
!   17. 3.98 (B.F.) Parameter position added
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
character*(*),           intent(in)     :: command
integer,       optional, intent(in)     :: position
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: position0
character*(80)                          :: outstr0
integer                                 :: ierr0
!------------------------------------------------------------------------------
!
ierr0=0
outstr0='done'
!
! --- Change position of file pointer in dependence of command ---
select case (command)
  case('rewind')
    varfile%position=0
  case('end', 'eof')
    varfile%position=varfile%nentry
  case('back', 'backspace')
    varfile%position=max(varfile%position-1,0)
  case('next', 'skip')
    varfile%position=min(varfile%position+1,varfile%nmaxentry)
  case('goto')
    if (present(position)) then
      position0=position
    else
      position0=0
    endif
    varfile%position=min(max(position0,0),varfile%nmaxentry)
  case default
    ierr0=1
    outstr0='action not found'
end select ! command
!
if (present(ierr)) ierr=ierr0
if (present(outstr)) outstr=outstr0
!
end subroutine uio_varfile_goto


!----------********************------------------------------------------------
subroutine uio_varfile_pointput(varfile, var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_pointput ('uio_variable_file_point_put')
!
! PURPOSE:
!   Put uio flexible variable into uio varfile via pointing.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_pointput(varfile, var, outstr=outstr,ierr=ierr)
!
! INPUT:
!   var:    ('variable') uio_var_type, pointer, uio flexible variable to be
!           put into file
!
! INPUT/OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be extended
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_varfile_make: Make uio varfile of requested size or enlarge file array
!   uio_var_null:     Set uio flexible variable to zero
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Enlarge varfile if necessary.
!   Shift entries in varfile if necessary.
!   Point from varfile to input uio flexible variable.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   29. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
type(uio_var_type)     , pointer        :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i
!------------------------------------------------------------------------------
!
if (varfile%nentry == varfile%nmaxentry) then
  ! --- Enlarge file array ---
  call uio_varfile_make(varfile, plus_nentry=max(varfile%nmaxentry,100))
endif
!
if (varfile%position < varfile%nentry) then
  ! --- File position counter does not show behind last entry ---
  ! --- Shift file array entries ---
  do i=varfile%nentry+1,varfile%position+2,-1
    varfile%file(i)%var=>varfile%file(i-1)%var
  end do ! i
endif
!
! --- Increment entry number and file position counter ---
varfile%nentry  =varfile%nentry  +1
varfile%position=varfile%position+1
!
! --- Put uio flexible variable into varfile via pointing ---
varfile%file(varfile%position)%var=>var
!
if (present(ierr)) ierr=0
if (present(outstr)) outstr='done'
!
end subroutine uio_varfile_pointput


!----------***************-----------------------------------------------------
subroutine uio_varfile_put(varfile, var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_put ('uio_variable_file_put')
!
! PURPOSE:
!   Put uio flexible variable into uio varfile via copying.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_put(varfile, var, outstr=outstr,ierr=ierr)
!
! INPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be put into file
!
! INPUT/OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be extended
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_varfile_make: Make uio varfile of requested size or enlarge file array
!   uio_var_null:     Set uio flexible variable to zero
!   var=var:          Copy uio flexible variable (uio_var_copy)
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Enlarge varfile if necessary.
!   Shift entries in varfile if necessary.
!   Copy uio flexible variable into varfile.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
type(uio_var_type)     , intent(in)     :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i
!------------------------------------------------------------------------------
!
if (varfile%nentry == varfile%nmaxentry) then
  ! --- Enlarge file array ---
  call uio_varfile_make(varfile, plus_nentry=max(varfile%nmaxentry,100))
endif
!
if (varfile%position < varfile%nentry) then
  ! --- File position counter does not show behind last entry ---
  ! --- Shift file array entries ---
  do i=varfile%nentry+1,varfile%position+2,-1
    varfile%file(i)%var=>varfile%file(i-1)%var
  end do ! i
endif
!
! --- Increment entry number and file position counter ---
varfile%nentry  =varfile%nentry  +1
varfile%position=varfile%position+1
!
! --- Put uio flexible variable into varfile ---
allocate(         varfile%file(varfile%position)%var)
call uio_var_null(varfile%file(varfile%position)%var)
varfile%file(varfile%position)%var=var
!
if (present(ierr)) ierr=0
if (present(outstr)) outstr='done'
!
end subroutine uio_varfile_put


!----------***************-----------------------------------------------------
subroutine uio_varfile_get(varfile, var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_get ('uio_variable_file_get')
!
! PURPOSE:
!   Get uio flexible variable from uio varfile via copying.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_get(varfile, var, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! INPUT/OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be extended
!
! OUTPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be put into file
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_delete:   Delete uio flexible variable
!   var=var:          Copy uio flexible variable (uio_var_copy)
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Input variable var has to be in a defined status.
!
! PROCEDURE:
!   Copy uio flexible variable from varfile into variable.
!   Go to next entry.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   29. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
type(uio_var_type)     , intent(out)    :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
!------------------------------------------------------------------------------
!
! --- Delete input variable ---
call uio_var_delete(var)
!
if (varfile%position < varfile%nentry) then
  ! --- File position counter shows on uio flexible variable ---
  ! --- Get uio flexible variable from varfile ---
  var=varfile%file(varfile%position+1)%var
  !
  ! --- Increment file position counter ---
  varfile%position=varfile%position+1
endif
!
if (present(ierr)) ierr=0
if (present(outstr)) outstr='done'
!
end subroutine uio_varfile_get


!----------********************------------------------------------------------
subroutine uio_varfile_pointget(varfile, var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_pointget ('uio_variable_file_point_get')
!
! PURPOSE:
!   Get uio flexible variable from uio varfile via pointing.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_pointget(varfile, var, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! INPUT/OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be extended
!
! OUTPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be put into file
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_delete:   Delete uio flexible variable
!   var=var:          Copy uio flexible variable (uio_var_copy)
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Input variable var has to be in a defined status.
!
! PROCEDURE:
!   Point to uio flexible variable in varfile.
!   Go to next entry.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   29. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!    5. 3.98 (B.F.) outstr='eof' if EOF
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_varfile_type),  intent(inout)  :: varfile
type(uio_var_type)     , pointer        :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(80)                          :: outstr0
integer                                 :: ierr0
!------------------------------------------------------------------------------
!
if (varfile%position < varfile%nentry) then
  ! --- File position counter shows on uio flexible variable ---
  ! --- Get uio flexible variable from varfile ---
  var=>varfile%file(varfile%position+1)%var
  !
  ! --- Increment file position counter ---
  varfile%position=varfile%position+1
  !
  ierr0=0
  outstr0='done'
else
  ! --- Nullify pointer ---
  nullify(var)
  ierr0=1
  outstr0='eof'
endif
!
if (present(ierr)) ierr=ierr0
if (present(outstr)) outstr=outstr0
!
end subroutine uio_varfile_pointget


!----------**************------------------------------------------------------
subroutine uio_varfile_rd(nc, varfile, type, ident, offset, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_rd ('uio_variable_file_read')
!
! PURPOSE:
!   Read uio varfile from file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_rd(nc, varfile, type=type,ident=ident,offset=offset, &
!                       outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   type:   ('type') character, optional, type of end mark
!   ident:  ('identifier') character, optional, identifier of end mark
!   offset: ('offset') integer, offset: 0 -> read end mark
!                                      -1 -> do not read end mark
!
! INPUT/OUTPUT:
!   varfile:('variable_file') uio_varfile_type, uio varfile to be read from file
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_rdhd:            Read header of variable and extract keywords
!   uio_var_null:        Set uio flexible variable to zero (initialization)
!   uio_var_rd:          Read uio file entry into uio flexible variable
!   uio_varfile_pointput:Put uio flexible variable into uio varfile via pointing
!
! MODULES:
!   uio_siz_module:  String length, table size
!   uio_bulk_module: Main set of uio-routines
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   29. 1.98 (B.F.) uio_varfile_put -> uio_varfile_pointput
!   16. 3.98 (B.F.) Parameters type, ident added
!   18. 3.98 (B.F.) Parameter offset added
!------------------------------------------------------------------------------
use uio_siz_module
use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
type(uio_varfile_type),  intent(inout)  :: varfile
character*(*), optional, intent(in)     :: type, ident
integer,       optional, intent(in)     :: offset
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
logical                                 :: found
character*(let)                         :: termt(2,nttmx)
integer                                 :: ntt
type(uio_var_type), pointer             :: var
character*(let)                         :: type0, ident0
integer                                 :: offset0, position
character*(80)                          :: outstr0
integer                                 :: ierr0
!------------------------------------------------------------------------------
!
! --- Defaults ---
if (present(type)) then
  type0=type
else
  type0=''
endif
if (present(ident)) then
  ident0=ident
else
  ident0=''
endif
if (present(offset)) then
  offset0=offset
else
  offset0=0
endif
!
do
  ! --- Save position in file ---
  call uio_chpos(nc, mode='get', posout=position)
  ! --- Read header ---
  call uio_rdhd(nc, termt,ntt, outstr=outstr0,ierr=ierr0)
  if (outstr0 == 'eof') then
    exit
  endif
  if (type0 == '') then
    found=ident0 == termt(1,2)
  else if (type0 /= termt(1,1)) then
    found=.false.
  else if (ident0 == '') then
    found=.true.
  else
    found=ident0 == termt(1,2)
  endif
  !
  if (.not. (found .and. (offset0 < 0))) then
    ! --- Read file entry into uio flexible variable ---
    allocate(var)
    call uio_var_null(var)
    call uio_var_rd(nc, termt,ntt, var, outstr0,ierr0)
    !
    ! --- Put uio flexible variable into varfile ---
    call uio_varfile_pointput(varfile, var, outstr0,ierr0)
  else
    ! --- Go back to old position ---
    call uio_chpos(nc, mode='goto', posin=position)
  endif
  !
  if (found) then
    ierr0=0
    outstr0='end mark found'
    exit
  endif
end do
!
if (present(ierr)) ierr=ierr0
if (present(outstr)) outstr=outstr0
!
end subroutine uio_varfile_rd


!----------**************------------------------------------------------------
subroutine uio_varfile_wr(nc, varfile, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_wr ('uio_variable_file_write')
!
! PURPOSE:
!   Write uio varfile into file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_varfile_wr(nc, varfile, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   varfile:('variable_file') uio_varfile_type, uio varfile to be written into
!           file
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_wr: Write uio flexible variable into uio file
!
! MODULES:
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Write all entries beginning from position of file counter into file
!   via repeated call of uio_var_wr.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
type(uio_varfile_type),  intent(in)     :: varfile
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: p
character*(80)                          :: outstr0
integer                                 :: ierr0
!------------------------------------------------------------------------------
ierr0=0
outstr0=''
!
do p=varfile%position+1,varfile%nentry
  call uio_var_wr(nc, varfile%file(p)%var, outstr=outstr0,ierr=ierr0)
end do ! p
!
if (present(ierr)) ierr=ierr0
if (present(outstr)) outstr=outstr0
!
end subroutine uio_varfile_wr
!
end module uio_varfile_module
