!***************************************************************************************************
!
!    ####     ##     ####    #   #    #  #####  ######  #####
!   #    #   #  #   #        #   ##   #    #    #       #    #
!   #       #    #   ####    #   # #  #    #    #####   #    #
!   #  ###  ######       #   #   #  # #    #    #       #####
!   #    #  #    #  #    #   #   #   ##    #    #       #   #
!    ####   #    #   ####    #   #    #    #    ######  #    # _routines
!
!   EOS Interpolation Routines
!
!***************************************************************************************************
!   The provided set of routines allows to read, write, handle, and interpolate the table data
!   representing the CO5BOLD equation of state. Various different interpolation routines compute
!   only selected sub sets of the thermodynamic data -- for performance reasons. Compiler switches
!   give the choice between different code branches -- for performance reasons, too.
!***************************************************************************************************
!   Fortran90
!   2018-11-24
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Sven Wedemeyer-Boehm:  Freiburg, Oslo
!   Matthias Steffen:      Potsdam
!***************************************************************************************************
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Handling of arrays and loops to improve performance                                 --- */
!# /*     0: Temporary coefficient arrays are copied                                              */
!# /*     1: Temporary coefficient arrays with pointers (needs gasinter_l02=0)                    */
!# /*     2: Avoiding temporary coefficient arrays                                                */
!# /*     3: Experimental (actually still identical to 0 in most cases)                           */
!# /*     4: Inner loop split in 3 (default, better vectorization; identical to 2 in some cases)  */
# ifndef gasinter_l01
#   define gasinter_l01 4
# endif
!#
!# /* --- Handling of coefficient arrays to improve performance                               --- */
!# /*     0: Pointer arrays (old version)                                                         */
!# /*     1: Allocatable arrays (default)                                                         */
# ifndef gasinter_l02
#   define gasinter_l02 1
# endif
!#
!# /* --- Arrays to use for interpolation                                                     --- */
!# /*     0: s, P, T arrays (old version, default)                                                */
!# /*     1: s array (new, experimental)                                                          */
# ifndef gasinter_l03
#   define gasinter_l03 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*********************------------------------------------------------------------------------
module tabinter_coeff_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_coeff_module ('table_interpolation_coefficients_module')
!
! PURPOSE:
!   Save coefficients for later bicubic interpolation of GAS quantities.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   use tabinter_coeff_module
!
! VARIABLES:
!   ndesmax:      ('number_description_maximum') integer parameter, maximum number
!                 of description lines
!   nhismax:      ('number_history_maximum') integer parameter, maximum number
!                 of history lines
!   nEOSlistmax:  ('number_Equation_Of_State_list_maximum') integer parameter,
!                 maximum number of interpolation tables
!   EOS:          ('Equation_Of_State') type(EOS_type), structure:
!     file_id:    ('file_idenfication') character
!     ndes:       ('number_description') integer, number of lines of description
!     description: ('description') character, dimension(1:ndesmax)
!     nhis:       ('number_history') integer, number of lines of history
!     history:    ('history') character, dimension(1:nhismax)
!     file:       ('file') character, name of data file
!     form:       ('format') character, name of data file
!     conv:       ('conversion') character, conversion type of data file
!     number_format: ('number_format') character, number format specification
!     variables:  ('variables') character with names of independent and dependent variables:
!                 'lnrho,lne->s,lnP,lnT': The default table format for CO5BOLD
!                 'lnP,lnT->lnrho,h,s':   The original format (inverted relative to the default)
!     interpolation: ('interpolation') character, e.g.,
!                 'bicubic': So far the only choice
!     source:     ('source') character with prescription how to compute the EOS quantities
!                 'gasgas': Use GASGAS or rather superGASGAS
!                 'gamma':  Use a simple gamma law with specified gamma
!     derivative_mode: ('derivative_mode') character with mode how to compute derivatives
!     abuX:       ('abundance_X') real, hydrogen abundance (number fraction)
!     abuY:       ('abundance_Y') real, helium abundance (number fraction)
!     abuZ:       ('abundance_Z') real, metal abundance (number fraction)
!     Qmol:       ('Q_molecular') real, mean molecular weight [u]
!     ifion:      ('i_flag_ionization') integer, determines how to treat ionization processes:
!                 0...16
!     gamma:      ('gamma') real with gamma value for source='gamma'
!     x_name:     ('x_name') character(len=80), dimension(nx), names of axes
!     x_unit:     ('x_unit') character(len=80), dimension(nx), units of axes
!     nx:         ('number_x') integer, number of axes of independent variables,
!                 exactly=2
!     ny:         ('number') integer, number of variables tabulated in array,
!                 typically=3
!     nC:         ('number_Coefficients') integer, number of interpolation coefficients per cell,
!                 typically=16
!     nx1:        ('number_x_1') integer, vector length of first independent variable
!     x1min:      ('x_1_minimum') real, minimum value of x1 axis
!     x1max:      ('x_1_maximum') real, maximum value of x1 axis
!     lnx1off:    ('logarithm_x_1_offset') real
!     lnx1fac:    ('logarithm_x_1_factor') real
!     x1_shift:   ('x_1_shift') real, shift value for x1 axis
!     nx2, x2min, x2max, lnx2off, lnx2fac, x2_shift: ...
!     C_shift:    ('Coefficients_shift') real, dimension(nymax), shift values for interpolated
!                 values
!     lnx11d:     ('logarithm_x_1_1_dimensional') real, dimension(1:nx1+1)
!     lnx21d:     ('logarithm_x_2_1_dimensional') real, dimension(1:nx2+1)
!     C_name:     ('Coefficients_name') character(len=80), dimension(nx), names of
!                 tabulated variables
!     C_unit:     ('Coefficients_unit') character(len=80), dimension(nx), units of 
!                 tabulated variables
!     C_shift:    ('Coefficients_shift') real, shift values for tabulated variables
!     C:          ('Coefficients') real, dimension(nC,ny,nx1,nx2),
!                 coefficients for bicubic interpolation
!   EOS_list:     ('Equation_Of_State_list') array of parameter structures
!   nEOSlist:     ('number_Equation_Of_State_list') number of currently active parameter set
!   EOS_list_init: ('Equation_Of_State_list_initialization') integer,
!                 0: parameter list is not initialized (pointers are undefined)
!                 1: parameter list is initialized (pointers are defined)
!
! MODIFICATION HISTORY:
!   1996-03-19 (B.F. Kiel) Written
!   1996-03-20 (B.F.) lnP1d, T1d, lnPoff, lnPfac, Toff, Tfac added
!   1996-03-30 (B.F.) lnPfac -> 1/lnPfac
!   1997-04-11 (B.F.) type EOS_type in module
!   1997-04-28 (B.F.) Everything in variable par, C(4,4,...) -> C(16,...)
!   1997-04-29 (B.F.) More comments
!   1997-04-30 (B.F.) ifion, number_format, variables, interpolation, source in par
!   1997-05-06 (B.F.) x1_shift, x2_shift, C_shift
!   1997-05-13 (B.F.) Arrays in par converted to pointers
!   1997-05-16 (B.F.) nx, ny, nc added
!   1997-07-25 (B.F.) parlist_init added
!   1997-10-15 (B.F.) gamma added
!   2003-11-14 (B.F. Uppsala) x1_offset, x2_offset, C_offset added
!   2014-08-06 (B.F.) Structure par renamed to EOS
!   2014-09-10 (B.F.) derivative_mode added
!   2017-08-10 (B.F.) Change some indentations and comments
!---------------------------------------------------------------------------------------------------
implicit none
!
integer, parameter                      :: ndesmax=20, nhismax=20
!
type EOS_type
  character(len=80)                     :: file_id
  integer                               :: ndes
  character(len=80), dimension(ndesmax) :: description
  integer                               :: nhis
  character(len=80), dimension(nhismax) :: history
  character(len=80)                     :: file, form, conv, number_format, &
                                           variables, interpolation, source, derivative_mode
  real                                  :: abuX, abuY, abuZ, Qmol, gamma
  integer                               :: ifion
  integer                               :: nx, ny, nc, nx1, nx2
  real                                  :: x1min,x1max,     x2min,x2max, &
                                           lnx1off,lnx1fac, lnx2off,lnx2fac, &
                                           x1_shift,        x2_shift, &
                                           x1_offset,       x2_offset
!#
# if (gasinter_l02 == 1)
!#                       /* allocatable arrays */
  character(len=80), &
        allocatable, dimension(:      ) :: x_name, x_unit
  real, allocatable, dimension(:      ) :: lnx11d
  real, allocatable, dimension(:      ) :: lnx21d
  character(len=80), &
        allocatable, dimension(:      ) :: C_name, C_unit
  real, allocatable, dimension(:      ) :: C_shift, C_offset
  real, allocatable, dimension(:,:,:,:) :: C
  real, allocatable, dimension(:      ) :: Cv
!#
# else
!#     /* pointer arrays */
  character(len=80), &
            pointer, dimension(:      ) :: x_name, x_unit
  real,     pointer, dimension(:      ) :: lnx11d
  real,     pointer, dimension(:      ) :: lnx21d
  character(len=80), &
            pointer, dimension(:      ) :: C_name, C_unit
  real,     pointer, dimension(:      ) :: C_shift, C_offset
  real,     pointer, dimension(:,:,:,:) :: C
  real,     pointer, dimension(:      ) :: Cv
!#
# endif
!#      /* (gasinter_l02) */
end type EOS_type
!
type EOSlist_type
  type(EOS_type), pointer               :: EOS
end type EOSlist_type
!
integer, parameter                      :: nEOSlistmax=16
type(EOSlist_type), &
  dimension(1:nEOSlistmax), save        :: EOSlist
integer, save                           :: nEOSlist
integer, save                           :: EOSlist_init=0
!
type(EOS_type), pointer                 :: EOS
!
end module tabinter_coeff_module


!------***************------------------------------------------------------------------------------
module tabinter_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_module ('table_interpolation_module')
!
! PURPOSE:
!   Provide routines for handling of interpolation coefficients.
!
! CATEGORY:
!   Interpolation
!
! CALLING SEQUENCE:
!   use tabinter_module
!
! VARIABLES:
!
! ROUTINES: (contained)
!   tabinter_delete:       Deallocate EOS table completely
!   tabinter_info:         Print information about interpolation table
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   tabinter_reset:        Set all quantities in EOS table to zero
!   tabinter_select:       Select table
!   tabinter_wrcoeff:      Write coefficients for bicubic interpolation of GAS quantities into file
!   tabinter_rdcoeff:      Read coefficients for bicubic interpolation of EOS quantities from file
!
! MODIFICATION HISTORY:
!   1997-03-11 (B.F. Kiel) Put into module
!   1995-05-06 (B.F.) tabinter_wrcoeff from gasinter_setup_module
!   1997-05-16 (B.F.) gasinter -> tabinter, gasinter
!   2002-10-29 (B.F.) tabinter_getinfo added
!---------------------------------------------------------------------------------------------------
contains
!
!----------***************--------------------------------------------------------------------------
subroutine tabinter_select(nEOS, ncpri, silent)
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_select ('table_interpolation_select')
!
! PURPOSE:
!   Select table.
!
! CATEGORY:
!   Interpolation
!
! CALLING SEQUENCE:
!   call tabinter_select(nEOS, ncpri, silent=silent)
!
! INPUT:
!   nEOS:         ('number_Equation_Of_State') integer, number of EOS parameter set to be used
!   ncpri:        ('number_channel_print') integer, logical output channel number
!   silent:       ('silent') logical, optional, if present and true do not print output
!
! OUTPUT:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!
! SIDE EFFECTS:
!   Change global variables EOS and nEOSlist. Create new table if necessary.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Select one table from table list. If table is non-existent, allocate it.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-16 (B.F. Kiel) Written
!   1997-07-27 (B.F.) Nullify EOSlist and entries
!   2002-08-11 (B.F.) Keyword silent added
!   2012-07-19 (B.F.) Check for ncpri >= 0
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nEOS, ncpri
logical,         optional, intent(in)   :: silent
!
! --- Local variables ---
integer                                 :: n
logical                                 :: silent0
!---------------------------------------------------------------------------------------------------
!
if (EOSlist_init == 0) then
  ! --- Initialization: nullify pointer array ---
  do n=1,nEOSlistmax
    nullify(EOSlist(n)%EOS)
  end do ! n
  !
  ! --- Pointer array is initialized ---
  EOSlist_init=1
endif
!
! --- Choose parameter set ---
nEOSlist=nEOS
if (.not. associated(EOSlist(nEOSlist)%EOS)) then
  allocate(EOSlist(nEOSlist)%EOS)
  EOS=>EOSlist(nEOSlist)%EOS
!#
# if (gasinter_l02 == 0)
!#                       /* pointer arrays */
  nullify(EOS%lnx11d, EOS%lnx21d )
  nullify(EOS%x_name, EOS%x_unit )
  nullify(EOS%C )
  nullify(EOS%Cv)
  nullify(EOS%C_name, EOS%C_unit, EOS%C_shift, EOS%C_offset)
!#
# endif
!#      /* (gasinter_l02) */
else
  EOS=>EOSlist(nEOSlist)%EOS
endif
!
if (present(silent)) then
  silent0=silent
else
  silent0=.false.
endif
!
if ((.not. silent0) .and. (ncpri >= 0)) write(ncpri,'(A,I3)') 'Parameter set # ', nEOSlist
!
end subroutine tabinter_select


!----------**************---------------------------------------------------------------------------
subroutine tabinter_reset(nEOS, ncpri, silent)
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_reset ('table_interpolation_reset')
!
! PURPOSE:
!   Set all quantities in EOS table to zero.
!
! CATEGORY:
!   Interpolation, EOS
!
! CALLING SEQUENCE:
!   call tabinter_reset(nEOS, ncpri, silent=silent)
!
! INPUT:
!   nEOS:         ('number_Equation_Of_State') integer, number of EOS parameter set to be used
!   ncpri:        ('number_channel_print') integer, logical output channel number
!   silent:       ('silent') logical, optional, if present and true do not print output
!
! OUTPUT:
!   None
!
! ROUTINES:
!   tabinter_select:       Select table
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!
! SIDE EFFECTS:
!   Set all quantities in selected table to zero.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-04-29 (B.F. Kiel) Written
!   1997-04-30 (B.F.) Reset ifion, number_format
!   1997-05-02 (B.F.) Additional parameters reset
!   1997-05-06 (B.F.) x1_shift, x2_shift, C_shift included
!   1997-05-14 (B.F.) Some arrays in EOS are pointers. They become deallocated. nEOS as input.
!   1997-05-16 (B.F.) nx, ny, nc added
!   1997-05-19 (B.F.) Parameter ncpri added
!   2003-11-14 (B.F.) x1_offset, x2_offset, C_offset added
!   2017-08-10 (B.F.) Keyword silent added
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nEOS, ncpri
logical,         optional, intent(in)   :: silent
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Choose parameter set ---
call tabinter_select(nEOS, ncpri, silent=silent)
!
! --- Set coefficients to zero (or '') ---
EOS%file_id=''
EOS%ndes=0
EOS%description=''
EOS%nhis=0
EOS%history=''
EOS%variables=''
EOS%interpolation=''
EOS%source=''
EOS%derivative_mode=''
EOS%file=''
EOS%form=''
EOS%conv=''
EOS%number_format=''
EOS%abuX =0.0
EOS%abuY =0.0
EOS%abuZ =0.0
EOS%Qmol =0.0
EOS%ifion=0
EOS%gamma=0.0
EOS%nx=0
EOS%ny=0
EOS%nC=0
EOS%nx1=0
EOS%nx2=0
EOS%x1min=0.0
EOS%x1max=0.0
EOS%x2min=0.0
EOS%x2max=0.0
EOS%lnx1off=0.0
EOS%lnx1fac=0.0
EOS%lnx2off=0.0
EOS%lnx2fac=0.0
EOS%x1_shift=0.0
EOS%x2_shift=0.0
EOS%x1_offset=0.0
EOS%x2_offset=0.0
!#
# if (gasinter_l02 == 1)
!#                       /* allocatable arrays */
if (allocated(EOS%lnx11d  )) deallocate(EOS%lnx11d  )
if (allocated(EOS%lnx21d  )) deallocate(EOS%lnx21d  )
if (allocated(EOS%x_name  )) deallocate(EOS%x_name  )
if (allocated(EOS%x_unit  )) deallocate(EOS%x_unit  )
if (allocated(EOS%C       )) deallocate(EOS%C       )
if (allocated(EOS%Cv      )) deallocate(EOS%Cv      )
if (allocated(EOS%C_name  )) deallocate(EOS%C_name  )
if (allocated(EOS%C_unit  )) deallocate(EOS%C_unit  )
if (allocated(EOS%C_shift )) deallocate(EOS%C_shift )
if (allocated(EOS%C_offset)) deallocate(EOS%C_offset)
!#
# else
!#     /* pointer arrays */
if (associated(EOS%lnx11d  )) deallocate(EOS%lnx11d  )
if (associated(EOS%lnx21d  )) deallocate(EOS%lnx21d  )
if (associated(EOS%x_name  )) deallocate(EOS%x_name  )
if (associated(EOS%x_unit  )) deallocate(EOS%x_unit  )
if (associated(EOS%C       )) deallocate(EOS%C       )
if (associated(EOS%Cv      )) deallocate(EOS%Cv      )
if (associated(EOS%C_name  )) deallocate(EOS%C_name  )
if (associated(EOS%C_unit  )) deallocate(EOS%C_unit  )
if (associated(EOS%C_shift )) deallocate(EOS%C_shift )
if (associated(EOS%C_offset)) deallocate(EOS%C_offset)
!#
# endif
!#      /* (gasinter_l02) */
!
end subroutine tabinter_reset


!----------***************--------------------------------------------------------------------------
subroutine tabinter_delete(nEOS, ncpri)
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_delete ('table_interpolation_delete')
!
! PURPOSE:
!   Delete (deallocate) EOS table.
!
! CATEGORY:
!   Interpolation, EOS
!
! CALLING SEQUENCE:
!   call tabinter_delete(nEOS)
!
! INPUT:
!   nEOS:         ('number_Equation_Of_State') integer, number of EOS parameter set to be used
!   ncpri:        ('number_channel_print') integer, logical output channel number
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!
! SIDE EFFECTS:
!   Deallocate selected table.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-16 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nEOS, ncpri
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Deallocate parameter set ---
nEOSlist=nEOS
if (associated(EOSlist(nEOSlist)%EOS)) then
  EOS=>EOSlist(nEOSlist)%EOS
  deallocate(EOSlist(nEOSlist)%EOS)
endif
!
end subroutine tabinter_delete


!----------*************----------------------------------------------------------------------------
subroutine tabinter_info(ncpri)
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_info ('table_interpolation_information')
!
! PURPOSE:
!   Write information about current interpolation table.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call tabinter_info(ncpri)
!
! INPUT:
!   ncpri:        ('number_channel_print') integer, logical output channel number
!
! OUTPUT:
!   None
!
! ROUTINES:
!
! MODULES:
!   tabinter_coeff_module
!
! SIDE EFFECTS:
!   Write text to selected channel.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-16 (B.F. Kiel) Written
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncpri
!
! --- Local variables ---
integer                                 :: i
character(len=1)                        :: i_str
!---------------------------------------------------------------------------------------------------
!
write(ncpri,*)
write(ncpri,'(A,I3)') 'Parameter set ', nEOSlist
write(ncpri,'(A)')    '================='
!
! --- Description ---
write(ncpri,'(A)') &
  'Description: --------------------------------------------------------------'
write(ncpri,'(A)') EOS%description(:EOS%ndes)
!
! --- History ---
write(ncpri,'(A)') &
  'History: ------------------------------------------------------------------'
write(ncpri,'(A)') EOS%history(:EOS%nhis)
write(ncpri,'(A)') &
  '---------------------------------------------------------------------------'
!
! --- File type info ---
write(ncpri,'(A)') 'Input filename:   ' // trim(EOS%file)
write(ncpri,'(A)') 'Input format:     ' // trim(EOS%form)
write(ncpri,'(A)') 'Input conversion: ' // trim(EOS%conv)
!
! --- Block dimension ---
write(ncpri,'(A,2I5)') 'Dimension:         ', EOS%nx1, EOS%nx2
write(ncpri,'(A,I15)') 'Coefficients:      ', EOS%nC
!
! --- Scalars ---
!
! --- Number of tabulated variables ---
write(ncpri,'(A,I15)') 'Variables:         ', EOS%ny
!
! --- Type: ---
write(ncpri,'(A)') 'Variable names:    ' // trim(EOS%variables)
!
! --- Type: bicubic ---
write(ncpri,'(A)') 'Interpolation:     ' // trim(EOS%interpolation)
!
! --- Source ---
write(ncpri,'(A)') 'Source:            ' // trim(EOS%source)
!
! --- Derivative mode ---
write(ncpri,'(A)') 'Derivative mode:   ' // trim(EOS%derivative_mode)
!
! --- Hydrogen, helium, metal abundance ---
write(ncpri,'(A,F15.12)') 'abuX: ', EOS%abuX
write(ncpri,'(A,F15.12)') 'abuY: ', EOS%abuY
write(ncpri,'(A,F15.12)') 'abuZ: ', EOS%abuZ
!
! --- Mean molecular weight ---
write(ncpri,'(A,E15.8)') 'Qmol: ', EOS%Qmol
!
! --- ifion: handling of ionization ---
write(ncpri,'(A,I15)') 'ifion:', EOS%ifion
!
! --- Adiabatic coefficient ---
write(ncpri,'(A,F15.12)') 'gamma:', EOS%gamma
!
! --- Shifts ---
write(ncpri,'(A,E15.8)') 'x1_shift   =', EOS%x1_shift
write(ncpri,'(A,E15.8)') 'x2_shift   =', EOS%x2_shift
!
do i=1,EOS%ny
  write(i_str,'(I1)') i
  write(ncpri,'(A,E15.8)') 'C_shift(' // i_str // ') =', EOS%C_shift(i)
end do ! i
!
! --- Offsets ---
write(ncpri,'(A,E15.8)') 'x1_offset  =', EOS%x1_offset
write(ncpri,'(A,E15.8)') 'x2_offset  =', EOS%x2_offset
do i=1,EOS%ny
  write(i_str,'(I1)') i
  write(ncpri,'(A,E15.8)') 'C_offset(' // i_str // ')=', EOS%C_offset(i)
end do ! i
!
! --- Arrays ---
!
! --- 1D vectors ---
write(ncpri,'(A,E15.8,1X,E15.8)') 'x1min,x1max=', EOS%x1min, EOS%x1max
write(ncpri,'(A,E15.8,1X,E15.8)') 'x2min,x2max=', EOS%x2min, EOS%x2max
!
! --- Coefficients ---
!
end subroutine tabinter_info


!----------****************-------------------------------------------------------------------------
subroutine tabinter_getinfo(abuX, abuY, abuZ, x1min, x1max, x2min, x2max)
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_getinfo ('table_interpolation_get_information')
!
! PURPOSE:
!   Give requested information about current interpolation table.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call tabinter_getinfo(abuX=abuX, abuY=abuY, abuZ=abuZ, &
!                         x1min=x1min, x1max=x1max, x2min=x2min, x2max=x2max)
!
! INPUT:
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   abuX:         ('abundance_X') real, hydrogen abundance (number fraction)
!   abuY:         ('abundance_Y') real, helium abundance (number fraction)
!   abuZ:         ('abundance_Z') real, metal abundance (number fraction)
!   x1min:        ('x_1_minimum') real, minimum value of x1 axis
!   x1max:        ('x_1_maximum') real, maximum value of x1 axis
!   x2min:        ('x_2_minimum') real, minimum value of x2 axis
!   x2max:        ('x_2_maximum') real, maximum value of x2 axis
!
! ROUTINES:
!
! MODULES:
!   tabinter_coeff_module
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call tabinter_getinfo(x1min=x1min, x1max=x1max, x2min=x2min, x2max=x2max)
!   write(6,*) 'Table range x1: ', x1min, ' to ', x1max
!   write(6,*) 'Table range x2: ', x2min, ' to ', x2max
!
! MODIFICATION HISTORY:
!   2002-10-29 (B.F.) Written
!   2002-12-16 (B.F.) x1min, x1max, x2min, x2max
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, optional,            intent(out)  :: abuX, abuY, abuZ, &
                                           x1min, x1max, x2min, x2max
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (present(abuX )) abuX =EOS%abuX
if (present(abuY )) abuY =EOS%abuY
if (present(abuZ )) abuZ =EOS%abuZ
!
if (present(x1min)) x1min=EOS%x1min
if (present(x1max)) x1max=EOS%x1max
if (present(x2min)) x2min=EOS%x2min
if (present(x2max)) x2max=EOS%x2max
!
end subroutine tabinter_getinfo


subroutine tabinter_wrcoeff()
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_wrcoeff ('table_interpolation_write_coefficients')
!
! PURPOSE:
!   Write coefficients for later bicubic interpolation of GAS quantities into
!   file.
!
! CATEGORY:
!   Interpolation, EOS, I/O
!
! CALLING SEQUENCE:
!   call tabinter_wrcoeff()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_closwr:            Close file after writing
!   uio_dim2st:            Compose dimension string
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   uio_bulk_module:       Main set of UIO routines
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Open file. Write data into it.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Open file in UIO format.
!   Write interpolation-coefficient table data.
!   Close file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-04-09 (B.F. Kiel) Written
!   1997-04-11 (B.F.)
!   1997-04-30 (B.F.) EOS%ifion added
!   1997-05-06 (B.F.) x1_shift, x2_shift, C_shift included
!   1997-05-20 (B.F.) Axes are shifted ('ds=...')
!   1997-05-26 (B.F.) Correction: ds=..., number_format also for abuX,...
!   1997-05-28 (B.F.) ds=... typically positiv
!   2003-11-14 (B.F.) x1_offset, x2_offset, C_offset added
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_bulk_module
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
!
! --- Local variables ---
integer                                 :: ncout, i
character                               :: i_str*(1)
integer                                 :: ntt
character(len=80)                       :: shistr
character(len=80)                       :: termt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- uio routines have to be initialized ---
!
! --- Open file in uio-form ---
call uio_openwr(ncout, EOS%file, form=EOS%form, conv=EOS%conv)
!
! --- Write file_id ---
call uio_wr(ncout, 'eos-tables: bicubic', 'file_id', name='File identification')
!
! --- File description ---
call uio_wr(ncout, EOS%description(1:EOS%ndes), 'description', &
            name='File description')
!
! --- File history ---
call uio_wr(ncout, EOS%history(1:EOS%nhis), 'history', name='File history')
!
! --- Label ---
call uio_wrlabl(ncout, 'block', date='now')
!
! --- Block dimension ---
call uio_wr(ncout, reshape((/ 1,EOS%nC, 1,EOS%nx1, 1,EOS%nx2 /), &
            (/2,3/)), 'dimension', format='I6', nperlin=6)
call uio_wr(ncout, EOS%ny, 'number_variables', format='I3', &
            name='Number of variables in table')
!
! --- Scalars ---
!
! --- Type ---
call uio_wr(ncout, EOS%variables, 'variables', name='Variables in table')
call uio_wr(ncout, EOS%interpolation, 'interpolation', name='Type of interpolation')
!
! --- Source: GASGAS routine or gamma law ---
call uio_wr(ncout, EOS%source, 'source', name='EOS source')
!
! --- Derivative mode ---
call uio_wr(ncout, EOS%derivative_mode, 'derivative_mode', name='Derivative mode')
!
! --- Write hydrogen and helium abundance, mean molecular weight ---
call uio_wr(ncout, EOS%abuX,  'abux',  name='Hydrogen abundance',    unit='1', &
            format=EOS%number_format)
call uio_wr(ncout, EOS%abuY,  'abuy',  name='Helium abundance',      unit='1', &
            format=EOS%number_format)
call uio_wr(ncout, EOS%abuZ,  'abuz',  name='Metal abundance',       unit='1', &
            format=EOS%number_format)
call uio_wr(ncout, EOS%Qmol,  'qmol',  name='Mean molecular weight', unit='u', &
            format=EOS%number_format)
call uio_wr(ncout, EOS%ifion, 'ifion', name='Ionization handling')
call uio_wr(ncout, EOS%gamma, 'gamma', name='Adiabatic coefficient', unit='1', &
            format=EOS%number_format)
!
! --- Shift of axes and tabulated values ---
call uio_wr(ncout, EOS%x1_shift, 'x1shift', name='Shift: axis 1', &
            format=EOS%number_format)
call uio_wr(ncout, EOS%x2_shift, 'x2shift', name='Shift: axis 2', &
            format=EOS%number_format)
do i=1,EOS%ny
  write(i_str,'(I1)') i
  call uio_wr(ncout, EOS%C_shift(i), 'c' // i_str // 'shift', &
              name='Shift: coefficient ' // i_str, format=EOS%number_format)
end do ! i
!
! --- Offset of axes and tabulated values ---
ntt=0
call uio_adkeyw(termt,ntt,'c0','q_GAS=q_current+offset')
call uio_wr(ncout, EOS%x1_offset, 'x1offset', name='Offset: axis 1', &
            format=EOS%number_format, ntt=ntt, termt=termt)
call uio_wr(ncout, EOS%x2_offset, 'x2offset', name='Offset: axis 2', &
            format=EOS%number_format, ntt=ntt, termt=termt)
do i=1,EOS%ny
  write(i_str,'(I1)') i
  call uio_wr(ncout, EOS%C_offset(i), 'c' // i_str // 'offset', &
              name='Offset: coefficient ' // i_str, format=EOS%number_format, ntt=ntt, termt=termt)
end do ! i
!
! --- Arrays: ---
!
! --- 1D vectors: spread -> 3D ---
call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
ntt=0
call uio_adkeyw(termt,ntt,'ds',shistr)
call uio_wr(ncout, reshape(exp(EOS%lnx11d(1:EOS%nx1+1))-EOS%x1_shift, &
            (/1,EOS%nx1+1,1/)), 'x1', mode=8, ishift=(/0,1,0/), &
            name=EOS%x_name(1), unit=EOS%x_unit(1), format=EOS%number_format, &
            ntt=ntt, termt=termt)
call uio_wr(ncout, reshape(exp(EOS%lnx21d(1:EOS%nx2+1))-EOS%x2_shift, &
            (/1,1,EOS%nx2+1/)), 'x2', mode=8, ishift=(/0,0,1/), &
            name=EOS%x_name(2), unit=EOS%x_unit(2), format=EOS%number_format, &
            ntt=ntt, termt=termt)
!
! --- Coefficients ---
do i=1,EOS%ny
  write(i_str,'(I1)') i
  call uio_wr(ncout, EOS%C(:,i,1:EOS%nx1,1:EOS%nx2), 'c' // i_str, &
              name=EOS%C_name(i), unit=EOS%C_unit(i), format=EOS%number_format)
end do ! i
!
! --- End block label ---
call uio_wrlabl(ncout, 'endblock', date='now')
!
! --- Close file ---
call uio_closwr(ncout)
!
end subroutine tabinter_wrcoeff


!----------****************-------------------------------------------------------------------------
subroutine tabinter_rdcoeff(file, ncpri, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   tabinter_rdcoeff ('table_interpolation_read_coefficients')
!
! PURPOSE:
!   Read coefficients for bicubic interpolation of EOS quantities from file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call tabinter_rdcoeff(file, ncpri, outstr,ierr)
!
! INPUT:
!   file:         ('file') character, name of file
!   ncpri:        ('number_channel_print') integer, logical output channel number
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('integer_error') integer, optional, contains exit error status
!                  0: correct operation
!                 >0: occurrence of an error
!
! ROUTINES:
!   uio_chpos:             Give current file position or jump to specified position
!   uio_closrd:            Close file after reading
!   uio_err:               Handle UIO errors
!   uio_filcon:            Determine file contents: list of all entries with its positions
!   uio_openrd:            Open file for reading, read header
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_srhd:              Search entry header in file or in entry list
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   uio_bulk_module:       Main set of UIO routines
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   A file is opened, read, and closed. Parameters are stored in saved global variable EOS.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Open requested file (in UIO format).
!   Determine file contents.
!   Search each data item, read it and put it into saved variable.
!   Close file again.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-04-16 (B.F. Kiel) Written
!   1997-04-17 (B.F.) Corrections: Pmax, Tmax
!   1997-04-30 (B.F.) Read ifion, number_format
!   1997-05-02 (B.F.) Additional parameters read; dimvec 2d
!   1997-05-06 (B.F.) x1_shift, x2_shift, C_shift included
!   1997-05-20 (B.F.) Read vectors and arrays with mode=1
!   1997-05-21 (B.F.) Don't check number of coefficients per cell; EOS%x_name, EOS%x_unit allocated
!   1997-06-05 (B.F.) Exit with error status if error during opening of file.
!   2002-08-09 (B.F.) Extra 'rewind' command to account for bug in Intel compiler
!   2003-11-14 (B.F.) x1_offset, x2_offset, C_offset added
!   2014-03-13 (B.F.) Intialize coefficient array with OpenMP loop
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: file
integer,                   intent(in)   :: ncpri
character(len=*),optional, intent(out)  :: outstr
integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
integer                                 :: ncin, ierr0, ntt, i, iC, iy, ix1, ix2
character(len=80)                       :: outstr0
character(len=1)                        :: i_str
character(len=let)                      :: termt(2,nttmx)
character(len=let)                      :: ident
integer, parameter                      :: nentmx=200
integer                                 :: nentry
integer                                 :: poslist(nentmx)
character(len=let)                      :: identlist(3,nentmx)
integer                                 :: iup(2), dimvec(2,3)
!---------------------------------------------------------------------------------------------------
!
! --- Open parameter file for reading ---
call uio_openrd(ncin, file, form=EOS%form, conv=EOS%conv, &
                outstr=outstr0,ierr=ierr0)
if (ierr0 > 0) then
  call uio_closrd(ncin)
  goto 900
endif
EOS%file=file
!
if (ncpri >= 0) then
  write(ncpri,'(A,I3)') 'Data file ''' // trim(file) // &
                        ''' opened on channel ', ncin
  !
  ! --- Print file type info ---
  write(ncpri,'(A)') 'Input filename:   ' // trim(EOS%file)
  write(ncpri,'(A)') 'Input format:     ' // trim(EOS%form)
  write(ncpri,'(A)') 'Input conversion: ' // trim(EOS%conv)
endif
!
! --- Determine data file contents ---
call uio_filcon(ncin, identlist,poslist,nentry)
!
!if (ncpri >= 0) then
!  write(ncpri,*) 'nentry=', nentry
!  write(ncpri,'(A)') identlist(:,:nentry)
!  write(ncpri,'(I7)') poslist(:nentry)
!endif
!
! --- Patch for Intel compiler ---
call uio_chpos(ncin, mode='rewind')
!
! --- Read file_id ---
call uio_srhd(ncin, termt,ntt, type=charna, ident='file_id', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%file_id,ident)
  if (EOS%file_id /= 'eos-tables: bicubic') then
    outstr0='''' // trim(file) // ''' is not EOS data file'
    ierr0=1
    call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
                 errlevel=3)
  endif
else
  outstr0='Type of ''' // trim(file) // ''' can not be determined'
  ierr0=2
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
!
! --- Read description ---
call uio_srhd(ncin, termt,ntt, type=charna, ident='description', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%description,ident,mode=2,iup=iup(1:1))
  EOS%ndes=iup(1)
else
  EOS%ndes=0
endif
if (ncpri >= 0) then
  write(ncpri,'(A)') EOS%description(:EOS%ndes)
endif
!
! --- Read history ---
call uio_srhd(ncin, termt,ntt, type=charna, ident='history', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%history,ident,mode=2,iup=iup(1:1))
  EOS%nhis=iup(1)
else
  EOS%nhis=0
endif
if (ncpri >= 0) then
  write(ncpri,'(A)') EOS%history(:EOS%nhis)
endif
!
! --- Read block dimension ---
call uio_srhd(ncin, termt,ntt, type=intena, ident='dimension', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, dimvec,ident,mode=2)
  EOS%nx =2
  EOS%nC =dimvec(2,1)
  EOS%nx1=dimvec(2,2)
  EOS%nx2=dimvec(2,3)
else
  outstr0='Dimension information missing'
  ierr0=4
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
if (ncpri >= 0) then
  write(ncpri,'(A,2I5)') 'Dimension:         ', EOS%nx1, EOS%nx2
  write(ncpri,'(A,I15)') 'Coefficients:      ', EOS%nC
endif
!
! --- Scalars ---
!
! --- Read number of tabulated variables ---
call uio_srhd(ncin, termt,ntt, type=intena, ident='number_variables', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%ny,ident)
else
  EOS%ny=3
  outstr0='number of variables is not specified'
  ierr0=3
endif
if (ncpri >= 0) then
  write(ncpri,'(A,I15)') 'Number of variables', EOS%ny
endif
!
! --- Allocate memory for arrays ---
allocate(EOS%x_name(EOS%nx), EOS%x_unit(EOS%nx))
allocate(EOS%lnx11d(EOS%nx1+1), EOS%lnx21d(EOS%nx2+1))
allocate(EOS%C_name(EOS%ny), EOS%C_unit(EOS%ny), EOS%C_shift(EOS%ny), EOS%C_offset(EOS%ny))
allocate(EOS%C( EOS%nC, EOS%ny, EOS%nx1, EOS%nx2))
allocate(EOS%Cv(EOS%nC* EOS%ny* EOS%nx1* EOS%nx2))
! --- Set array to zero within OpenMP loop ---
!$OMP PARALLEL DEFAULT(SHARED) &
!$OMP   PRIVATE(ix2, ix1, iy, iC)
!$OMP DO SCHEDULE(STATIC)
do ix2=1,EOS%nx2
  do ix1=1,EOS%nx1
    do iy=1,EOS%ny
      do iC=1,EOS%nC
        EOS%C(iC, iy, ix1, ix2)=0.0
      end do ! iC
    end do ! iy
  end do ! ix1
end do ! ix2
!$OMP END DO
!$OMP END PARALLEL
!
! --- Type: P,T -> ... or rho,e -> ... ---
EOS%variables=''
call uio_srhd(ncin, termt,ntt, type=charna, ident='variables', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%variables,ident)
else
  EOS%variables='lnP,lnT->lnrho,h,s'
endif
if (ncpri >= 0) then
  write(ncpri,'(A)') 'Variables:         ' // trim(EOS%variables)
endif
!
! --- Type: bicubic ---
EOS%interpolation=''
call uio_srhd(ncin, termt,ntt, type=charna, ident='interpolation', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%interpolation,ident)
else
  EOS%interpolation='bicubic'
endif
if (ncpri >= 0) then
  write(ncpri,'(A)') 'Interpolation:     ' // trim(EOS%interpolation)
endif
!
! --- Source ---
call uio_srhd(ncin, termt,ntt, type=charna, ident='source', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%source,ident)
else
  EOS%source=''
endif
if (ncpri >= 0) then
  write(ncpri,'(A)') 'Source:            ' // trim(EOS%source)
endif
!
! --- Derivative mode ---
call uio_srhd(ncin, termt,ntt, type=charna, ident='derivative_mode', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%derivative_mode,ident)
else
  EOS%derivative_mode=''
endif
if (ncpri >= 0) then
  write(ncpri,'(A)') 'Derivative_mode:   ' // trim(EOS%derivative_mode)
endif
!
! --- Read hydrogen abundance ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='abux', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%abuX,ident)
else
  outstr0='hydrogen abundance is not specified'
  ierr0=3
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
if (ncpri >= 0) then
  write(ncpri,'(A,F15.12)') 'abuX: ', EOS%abuX
endif
!
! --- Read helium abundance ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='abuy', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%abuY,ident)
else
  outstr0='helium abundance is not specified'
  ierr0=4
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
if (ncpri >= 0) then
  write(ncpri,'(A,F15.12)') 'abuY: ', EOS%abuY
endif
!
! --- Compute metal abundance ---
EOS%abuZ=1.0-EOS%abuX-EOS%abuY
if (ncpri >= 0) then
  write(ncpri,'(A,F15.12)') 'abuZ: ', EOS%abuZ
endif
!
! --- Read mean molecular weight ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='qmol', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%Qmol,ident)
else
  outstr0='mean molecular weight is not specified'
  ierr0=5
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8)') 'Qmol: ', EOS%Qmol
endif
!
! --- Read ifion: handling of ionization ---
call uio_srhd(ncin, termt,ntt, type=intena, ident='ifion', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%ifion,ident)
else
  EOS%ifion=1
  outstr0='ionization handling is not specified'
  ierr0=6
endif
if (ncpri >= 0) then
  write(ncpri,'(A,I15)') 'ifion:', EOS%ifion
endif
!
! --- Read adiabatic coefficient ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='gamma', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%gamma,ident)
else
  EOS%gamma=0.0
endif
if (ncpri >= 0) then
  write(ncpri,'(A,F15.12)') 'gamma:', EOS%gamma
endif
!
! --- Shifts ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='x1shift', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%x1_shift,ident)
else
  EOS%x1_shift=0.0
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8)') 'x1_shift   =', EOS%x1_shift
endif
!
call uio_srhd(ncin, termt,ntt, type=realna, ident='x2shift', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%x2_shift,ident)
else
  EOS%x2_shift=0.0
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8)') 'x2_shift   =', EOS%x2_shift
endif
!
do i=1,EOS%ny
  write(i_str,'(I1)') i
  call uio_srhd(ncin, termt,ntt, type=realna, ident='c' // i_str // 'shift', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncin, termt,ntt, EOS%C_shift(i),ident)
  else
    EOS%C_shift(i)=0.0
  endif
  if (ncpri >= 0) then
    write(ncpri,'(A,E15.8)') 'C_shift(' // i_str // ') =', EOS%C_shift(i)
  endif
end do ! i
!
! --- Offsets ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='x1offset', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%x1_offset,ident)
else
  EOS%x1_offset=0.0
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8)') 'x1_offset  =', EOS%x1_offset
endif
!
call uio_srhd(ncin, termt,ntt, type=realna, ident='x2offset', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%x2_offset,ident)
else
  EOS%x2_offset=0.0
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8)') 'x2_offset  =', EOS%x2_offset
endif
!
do i=1,EOS%ny
  write(i_str,'(I1)') i
  call uio_srhd(ncin, termt,ntt, type=realna, ident='c' // i_str // 'offset', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncin, termt,ntt, EOS%C_offset(i),ident)
  else
    EOS%C_offset(i)=0.0
  endif
  if (ncpri >= 0) then
    write(ncpri,'(A,E15.8)') 'C_offset(' // i_str // ')=', EOS%C_offset(i)
  endif
end do ! i
!
! --- Arrays: ---
!
! --- 1D vectors: spread: 3D ---
call uio_srhd(ncin, termt,ntt, type=realna, ident='x1', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%lnx11d,ident,format=EOS%number_format, &
              name=EOS%x_name(1), unit=EOS%x_unit(1), mode=1)
  EOS%x1min=EOS%lnx11d(1)
  EOS%x1max=EOS%lnx11d(EOS%nx1+1)
  EOS%lnx11d(1:EOS%nx1+1)=alog(EOS%lnx11d(1:EOS%nx1+1)+EOS%x1_shift)
  EOS%lnx1off=EOS%lnx11d(1)
  EOS%lnx1fac=real(EOS%nx1)/(EOS%lnx11d(EOS%nx1+1)-EOS%lnx11d(1))
else
  outstr0='First axis is not specified'
  ierr0=7
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8,1X,E15.8)') 'x1min,x1max=', EOS%x1min, EOS%x1max
endif
!
call uio_srhd(ncin, termt,ntt, type=realna, ident='x2', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncin, termt,ntt, EOS%lnx21d,ident, &
              name=EOS%x_name(2), unit=EOS%x_unit(2), mode=1)
  EOS%x2min=EOS%lnx21d(1)
  EOS%x2max=EOS%lnx21d(EOS%nx2+1)
  EOS%lnx21d(1:EOS%nx2+1)=alog(EOS%lnx21d(1:EOS%nx2+1)+EOS%x2_shift)
  EOS%lnx2off=EOS%lnx21d(1)
  EOS%lnx2fac=real(EOS%nx2)/(EOS%lnx21d(EOS%nx2+1)-EOS%lnx21d(1))
else
  outstr0='second axis is not specified'
  ierr0=8
  call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
               errlevel=3)
endif
if (ncpri >= 0) then
  write(ncpri,'(A,E15.8,1X,E15.8)') 'x2min,x2max=', EOS%x2min, EOS%x2max
endif
!
! --- Coefficients ---
do i=1,EOS%ny
  write(i_str,'(I1)') i
  call uio_srhd(ncin, termt,ntt, type=realna, ident='c' // i_str, &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncin, termt,ntt, EOS%C(:,i,:,:),ident, &
                name=EOS%C_name(i), unit=EOS%C_unit(i), mode=1)
  else
    outstr0='Coefficients for variable ' // i_str // 'not found'
    ierr0=9
    call uio_err(nc=ncin,outstr=outstr0,ierr=ierr0,routine='tabinter_rdcoeff', &
                 errlevel=3)
  endif
end do ! i
!
! --- Attention: copy all coefficients into 1D vector (testing) ---
!EOS%Cv(1:EOS%nC*EOS%ny*EOS%nx1*EOS%nx2)=reshape(EOS%C, (/ EOS%nC*EOS%ny*EOS%nx1*EOS%nx2 /))
!
! --- Close file after reading ---
call uio_closrd(ncin)
!
900 continue
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine tabinter_rdcoeff

end module tabinter_module


!------***************------------------------------------------------------------------------------
module gasinter_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter_module ('gas_interpolation_module')
!
! PURPOSE:
!   Provide routines for interpolation of GAS or EOS quantities.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   use gasinter_module
!
! VARIABLES:
!
! ROUTINES: (contained)
!   eosinter_SwitchInfo:   Print information about used compiler switches
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_allOMP:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_Ts:           Solve EOS by bicubic interpolation: rho, e -> T, s
!   eosinter_TOMP:         Solve EOS by bicubic interpolation: rho, e -> T
!   eosinter_TdT:          Solve EOS by bicubic interpolation: rho, e -> T, dTde
!   eosinter_TdTOMP:       Solve EOS by bicubic interpolation: rho, e -> T, dTde
!   eosinter_PT:           Solve EOS by bicubic interpolation: rho, e -> P, T
!   eosinter_PTdT:         Solve EOS by bicubic interpolation: rho, e -> P, T, dTde
!   eosinter_logPTdT:      Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinter_logPTdTOMP:   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   gasinter:              Solve EOS by bicubic interpolation: P  , T -> rho, h, ...
!
! PROCEDURE:
!   Additional quantities:
!     cpd:    ('cp_cash') Cp' = (xcp) = d(S)/d(ln rho) | P [erg/g/K]
!     Gamma1: ('Gamma_1') 1. adiabatic exponent: Gamma1 = (xgam) = Cp'/Cv'
!                                                       = d(ln P)/d(ln rho) | s
!     delta:  ('delta') expansion coefficient: delta = (cxt) = alpha*T
!                                                    =-d(ln rho)/d(ln T) | P
!     h:      ('h') enthalpy per mass unit [erg/g]
!     cs:     ('c_sound') adiabatic sound velocity = (csound) [cm/s]
!     cpd   =P/(rho*T) + rho*dPdrho/(T*dPde)
!     Gamma1=T*cpd*dPde/P
!     delta =1.0/(cpd*dTde-(Gamma1*P)/(T*rho*cpd))
!     cs    =sqrt(Gamma1*P/rho)
!     h     =e+P/rho
!
! MODIFICATION HISTORY:
!   1997-03-11 (B.F. Kiel) Put into module
!   1998-05-06 (B.F.) tabinter_wrcoeff from gasinter_setup_module
!   2000-12-30 (B.F. Uppsala) 2D version for eosinter
!   2001-02-08 (B.F.) Special versions for eosinter_XXX
!   2002-04-19 (B.F.) Special version for eosinter_TdT
!   2002-05-14 (B.F.) Version with compiler switches
!   2002-08-11 (B.F.) eosinter_switchinfo, eosinter_T
!   2007-08-02 (M.S.) unnecessary "use tabinter_module" statements removed
!   2008-08-12 (B.F. Lyon) Merge with version from M.S.:
!              eosinter_logPTdT and eosinter_TdT get additional arguments (m1,n1,...)
!   2010-03-17 (B.F. Naples) Add option to use allocatable arrays instead of pointer arrays
!   2010-03-19 (B.F.) Swap meanings of new switch options
!   2011-12-05 (B.F. Lyon) New variety: eosinter_PdPTdT
!   2012-04-03 (B.F.) New version eosinter_PdPs
!   2013-02-09 (B.F.) Experimental versions of eosinter routines to improve vectorization
!   2013-02-13 (B.F.) Experimental version with Cv(:) coefficients
!   2013-02-14 (B.F.) Some rearrangements
!   2013-02-15 (B.F.) Rename index variables; new routine eosinter3a_logPTdT; remove eosinter_PdPs
!   2013-02-17 (B.F.) Append "OMP" to some subroutine names; remove eosinter_PTdT
!   2014-02-24 (B.F. Uppsala) Change default value of compiler switch
!   2018-11-24 (B.F.) Add eosinter1_TdT
!---------------------------------------------------------------------------------------------------
!
interface gasinter
  module procedure gasinter0, gasinter1
end interface ! gasinter
!
interface eosinter
  module procedure eosinter0, eosinter1, eosinter2, eosinter3
end interface ! eosinter
!
interface eosinter_all
  module procedure eosinter0, eosinter1, eosinter2_all, eosinter3_all
end interface ! eosinter_all
!
interface eosinter_allOMP
  module procedure eosinter3_allOMP
end interface ! eosinter_allOMP
!
interface eosinter_Ts
  module procedure eosinter2_Ts, eosinter3_Ts
end interface ! eosinter_Ts
!
interface eosinter_TOMP
  module procedure eosinter3_TOMP
end interface ! eosinter_TOMP
!
interface eosinter_TdT
  module procedure eosinter1_TdT
end interface ! eosinter_TdT
!
interface eosinter_TdTOMP
  module procedure eosinter3_TdTOMP
end interface ! eosinter_TdTOMP
!
interface eosinter_PT
  module procedure eosinter1_PT
end interface ! eosinter_PT
!
interface eosinter_PdPTdT
  module procedure eosinter2_PdPTdT, eosinter3_PdPTdT
end interface ! eosinter_PdPTdT
!
interface eosinter_logPTdT
  module procedure eosinter1_logPTdT, eosinter2_logPTdT, eosinter3_logPTdT, eosinter3ii_logPTdT
end interface ! eosinter_logPTdT
!
interface eosinter_logPTdTOMP
  module procedure eosinter3_logPTdTOMP, eosinter3i_logPTdTOMP, eosinter3ii_logPTdTOMP
end interface ! eosinter_logPTdTOMP
!
interface eosinter_PdP
  module procedure eosinter1_PdP, eosinter3_PdP
end interface ! eosinter_PdP
!
interface eosinter_PGamma13
  module procedure eosinter3_PGamma13
end interface ! eosinter_PGamma13
!
contains

!----------*******************----------------------------------------------------------------------
subroutine eosinter_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter_SwitchInfo ('equation_of_state_interpolation_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   I/O
!
! CALLING SEQUENCE:
!   call eosinter_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call eosinter_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2002-11-08 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
!character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: eosinter_module ........................'
!
write(ncp,'(A,I6)') '  gasinter' // '_l01:               ', gasinter_l01
write(ncp,'(A,I6)') '  gasinter' // '_l02:               ', gasinter_l02
!
end subroutine eosinter_SwitchInfo


!----------*********--------------------------------------------------------------------------------
subroutine gasinter0(P, T, rho, xcp, xgam, cxt, h, s, csound, &
                     ierror, ifent)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter0 ('gas_interpolation_0')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation. Behave like the original GAS routine.
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter(P, T, rho, xcp, xgam, cxt, h, s, csound, &
!                 ierror, ifent)
!
! INPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   T:            ('Temperature') temperature [K]
!   ifent:        ('i_flag_entropy') entropy is computed if ifent=1 (0...1)
!
! OUTPUT:
!   rho:          ('rho') density [g/cm^3]
!   xcp:          ('x_cp') Cp' = d(S)/d(ln rho) | P [erg/g/K]
!   xgam:         ('x_gamma') 1. adiabatic exponent: xgam = Gamma1 = Cp'/Cv'
!                                                         = d(ln P)/d(ln rho) | s
!   cxt:          ('cxt') expansion coefficient: cxt = delta = alpha*T
!                                                    =-d(ln rho)/d(ln T) | P
!   h:            ('h') enthalpy per mass unit [erg/g]
!   s:            ('s') entropy per mass unit [erg/g/K]
!   cs:           ('c_sound') adiabatic sound velocity [cm/s]
!   ierror:       ('index_error') integer, error number
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Parameters x1_shift, x2_shift and C_shift are ignored (assumed to be zero).
!
! PROCEDURE:
!   For P/T pair determine indices of interpolation coefficients and relative position in cell.
!   Do bicubic interpolation for desired P/T pair and some derivatives.
!   Compute other thermodynamic quantities by some thermodynamic relations.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-03-20 (B.F. Kiel) Written
!   1996-03-22 (B.F.) Order of parameters in call of bicubic_interpol changed.
!   1996-03-23 (B.F.) T -> lnT
!   1996-03-24 (B.F.) dhdlnp removed
!   1996-03-30 (B.F.) Interpolation inlined, xgam_rho; lnPfac -> 1/lnPfac
!   1997-05-20 (B.F.) lnrho eliminated
!   1997-05-21 (B.F.) ierror>0: point outside table boundaries
!   1997-05-27 (B.F.) Entropy is computed if ifent=1
!   1998-03-17 (B.F.) Parameters ifion, kenter, abuX, abuY, Qmol removed
!   2004-03-08 (S.W.-B.) New alternative version without temporary coefficient arrays
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ifent
integer,                   intent(out)  :: ierror
real,                      intent(in)   :: P, T
real,                      intent(out)  :: rho, xcp, xgam, cxt, h, s, csound
!
! --- Local variables ---
integer                                 :: j1, j2
real                                    :: lnP, lnT, &
                                           dlnrhodlnP, dlnrhodlnT, dhdlnT, &
                                           x1t, x2t, xgam_rho
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
ierror=0
!
lnP=alog(P)
lnT=alog(T)
!
! --- Determine indices of interpolation coefficients ---
j1=int((lnP-EOS%lnx1off)*EOS%lnx1fac)
j2=int((lnT-EOS%lnx2off)*EOS%lnx2fac)
if (j1 < 0      ) ierror=ierror+1
if (j1 > EOS%nx1) ierror=ierror+1
if (j2 < 0      ) ierror=ierror+1
if (j2 > EOS%nx2) ierror=ierror+1
j1=min(max(j1+1,1),EOS%nx1)
j2=min(max(j2+1,1),EOS%nx2)
!
! --- Get relative position in cell ---
x1t=lnP-EOS%lnx11d(j1)
x2t=lnT-EOS%lnx21d(j2)
!
!#
# if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                              /* avoid temporary array Ct */
  ! --- Do bicubic interpolation for desired P/T pair ---
  rho=exp(        EOS%C( 1,1,j1,j2) + &
             x1t*(EOS%C( 2,1,j1,j2) + &
             x1t*(EOS%C( 3,1,j1,j2) + &
             x1t* EOS%C( 4,1,j1,j2))) + &
        x2t*(     EOS%C( 5,1,j1,j2) + &
             x1t*(EOS%C( 6,1,j1,j2) + &
             x1t*(EOS%C( 7,1,j1,j2) + &
             x1t* EOS%C( 8,1,j1,j2))) + &
        x2t*(     EOS%C( 9,1,j1,j2) + &
             x1t*(EOS%C(10,1,j1,j2) + &
             x1t*(EOS%C(11,1,j1,j2) + &
             x1t* EOS%C(12,1,j1,j2))) + &
        x2t*(     EOS%C(13,1,j1,j2) + &
             x1t*(EOS%C(14,1,j1,j2) + &
             x1t*(EOS%C(15,1,j1,j2) + &
             x1t* EOS%C(16,1,j1,j2)))  ))) )
  dlnrhodlnP=     EOS%C( 2,1,j1,j2) + &
          x1t*(2.*EOS%C( 3,1,j1,j2) + &
          x1t* 3.*EOS%C( 4,1,j1,j2)) + &
     x2t*(        EOS%C( 6,1,j1,j2) + &
          x1t*(2.*EOS%C( 7,1,j1,j2) + &
          x1t* 3.*EOS%C( 8,1,j1,j2)) + &
     x2t*(        EOS%C(10,1,j1,j2) + &
          x1t*(2.*EOS%C(11,1,j1,j2) + &
          x1t* 3.*EOS%C(12,1,j1,j2)) + &
     x2t*(        EOS%C(14,1,j1,j2) + &
          x1t*(2.*EOS%C(15,1,j1,j2) + &
          x1t* 3.*EOS%C(16,1,j1,j2))  )))
  dlnrhodlnT=     EOS%C( 5,1,j1,j2) + &
             x1t*(EOS%C( 6,1,j1,j2) + & 
             x1t*(EOS%C( 7,1,j1,j2) + &
             x1t* EOS%C( 8,1,j1,j2))) + &
    2.0*x2t*(     EOS%C( 9,1,j1,j2) + &
             x1t*(EOS%C(10,1,j1,j2) + & 
             x1t*(EOS%C(11,1,j1,j2) + &
             x1t* EOS%C(12,1,j1,j2))) + &
    1.5*x2t*(     EOS%C(13,1,j1,j2) + &
             x1t*(EOS%C(14,1,j1,j2) + & 
             x1t*(EOS%C(15,1,j1,j2) + &
             x1t* EOS%C(16,1,j1,j2)))  ))
  h=              EOS%C( 1,2,j1,j2) + &
             x1t*(EOS%C( 2,2,j1,j2) + &
             x1t*(EOS%C( 3,2,j1,j2) + &
             x1t* EOS%C( 4,2,j1,j2))) + &
        x2t*(     EOS%C( 5,2,j1,j2) + &
             x1t*(EOS%C( 6,2,j1,j2) + &
             x1t*(EOS%C( 7,2,j1,j2) + &
             x1t* EOS%C( 8,2,j1,j2))) + &
        x2t*(     EOS%C( 9,2,j1,j2) + &
             x1t*(EOS%C(10,2,j1,j2) + &
             x1t*(EOS%C(11,2,j1,j2) + &
             x1t* EOS%C(12,2,j1,j2))) + &
        x2t*(     EOS%C(13,2,j1,j2) + &
             x1t*(EOS%C(14,2,j1,j2) + &
             x1t*(EOS%C(15,2,j1,j2) + &
             x1t* EOS%C(16,2,j1,j2)))  )))
  dhdlnT=         EOS%C( 5,2,j1,j2) + &
             x1t*(EOS%C( 6,2,j1,j2) + &
             x1t*(EOS%C( 7,2,j1,j2) + &
             x1t* EOS%C( 8,2,j1,j2))) + &
    2.0*x2t*(     EOS%C( 9,2,j1,j2) + &
             x1t*(EOS%C(10,2,j1,j2) + &
             x1t*(EOS%C(11,2,j1,j2) + &
             x1t* EOS%C(12,2,j1,j2))) + &
    1.5*x2t*(     EOS%C(13,2,j1,j2) + &
             x1t*(EOS%C(14,2,j1,j2) + &
             x1t*(EOS%C(15,2,j1,j2) + &
             x1t* EOS%C(16,2,j1,j2)))  ))
  !
  if (ifent == 1) then
    s=            EOS%C( 1,3,j1,j2) + &
             x1t*(EOS%C( 2,3,j1,j2) + & 
             x1t*(EOS%C( 3,3,j1,j2) + &
             x1t* EOS%C( 4,3,j1,j2))) + &
        x2t*(     EOS%C( 5,3,j1,j2) + &
             x1t*(EOS%C( 6,3,j1,j2) + &
             x1t*(EOS%C( 7,3,j1,j2) + &
             x1t* EOS%C( 8,3,j1,j2))) + &
        x2t*(     EOS%C( 9,3,j1,j2) + &
             x1t*(EOS%C(10,3,j1,j2) + &
             x1t*(EOS%C(11,3,j1,j2) + &
             x1t* EOS%C(12,3,j1,j2))) + &
        x2t*(     EOS%C(13,3,j1,j2) + &
             x1t*(EOS%C(14,3,j1,j2) + &
             x1t*(EOS%C(15,3,j1,j2) + &
             x1t* EOS%C(16,3,j1,j2)))  )))
  endif
!#
# else
!#                        /* using temporary array Ct */
! --- Do bicubic interpolation for desired P/T pair ---
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,1,j1,j2)
!#
# endif
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
  Ct=>EOS%C(:,1,j1,j2)
!#
# endif
!#
  rho=exp( &
                 Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
            x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
            x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
            x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
  dlnrhodlnP=                  Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
            x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
            x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
            x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  )))
  dlnrhodlnT=    Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
        2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
        1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,2,j1,j2)
!#
# endif
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
  Ct=>EOS%C(:,2,j1,j2)
!#
# endif
!#
  h=             Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
            x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
            x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
            x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )))
  dhdlnT=        Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
        2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
        1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))
!
  if (ifent == 1) then
!#
#   if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                /* temporary array is copied */
    Ct=EOS%C(:,3,j1,j2)
!#
#   endif
!#
#   if (gasinter_l01 == 1)
!#                         /* temporary array with pointer */
    Ct=>EOS%C(:,3,j1,j2)
!#
#   endif
!#
    s=           Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
            x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
            x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
            x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )))
  endif
!#
# endif
!#
!
! --- Compute all desired quantities from tabulated values ---
cxt     =-dlnrhodlnT
xcp     = dhdlnT/(cxt*T)
xgam_rho= dhdlnT/((rho*dhdlnT)*dlnrhodlnP-cxt*cxt*P)
csound  = sqrt(xgam_rho*P)
xgam    = xgam_rho*rho
!
end subroutine gasinter0


!----------*********--------------------------------------------------------------------------------
subroutine gasinter1(P, T, rho, xcp, xgam, cxt, h, s, csound, &
                     ierror, ifent)
!---------------------------------------------------------------------------------------------------
! NAME:
!   gasinter1 ('gas_interpolation_1')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation. Behave like the original GAS routine,
!   but: accept 1D vectors
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call gasinter(P, T, rho, xcp, xgam, cxt, h, s, csound, &
!                 ierror, ifent)
!
! INPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   T:            ('Temperature') temperature [K]
!   ifent:        ('i_flag_entropy') entropy is always computed
!
! OUTPUT:
!   rho:          ('rho') density [g/cm^3]
!   xcp:          ('x_cp') Cp' = d(S)/d(ln rho) | P [erg/g/K]
!   xgam:         ('x_gamma') 1. adiabatic exponent: xgam = Gamma1 = Cp'/Cv'
!                                                         = d(ln P)/d(ln rho) | s
!   cxt:          ('cxt') expansion coefficient: cxt = delta = alpha*T
!                                                    =-d(ln rho)/d(ln T) | P
!   h:            ('h') enthalpy per mass unit [erg/g]
!   s:            ('s') entropy per mass unit [erg/g/K]
!   cs:           ('c_sound') adiabatic sound velocity [cm/s]
!   ierror:       ('index_error') error number
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Parameters x1_shift, x2_shift and C_shift are ignored (assumed to be zero).
!
! PROCEDURE:
!   For P/T pair determine indices of interpolation coefficients and relative position in cell.
!   Do bicubic interpolation for desired P/T pair and some derivatives.
!   Compute other thermodynamic quantities by some thermodynamic relations.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-05-20 (B.F. Kiel) Written
!   1997-05-21 (B.F.) ierror>0: point outside table boundaries
!   1997-05-26 (B.F.) Directive COPY_ASSUMED_SHAPE included
!   1998-03-17 (B.F.) Parameters ifion, kenter, abuX, abuY, Qmol removed
!   2004-03-08 (S.W.-B.) New alternative version without temporary coefficient arrays
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ifent
integer,                   intent(out)  :: ierror
real, dimension(:),        intent(in)   :: P, T
real, dimension(:),        intent(out)  :: rho, xcp, xgam, cxt, h, s, csound
!
! --- Local variables ---
integer                                 :: n1, i1, j1, j2
integer, dimension(1:size(P,1))         :: j1a, j2a
real,    dimension(1:size(P,1))         :: lnP, lnT, x1ta, x2ta, &
                                           dlnrhodlnP, dlnrhodlnT, dhdlnT, &
                                           xgam_rho
real                                    :: x1t, x2t
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(P,1)
!
ierror=0
!
lnP=alog(P)
lnT=alog(T)
!
! --- Determine indices of interpolation coefficients ---
j1a=int((lnP-EOS%lnx1off)*EOS%lnx1fac)
j2a=int((lnT-EOS%lnx2off)*EOS%lnx2fac)
ierror=ierror+count(j1a < 0)+count(j1a > EOS%nx1)+ &
              count(j2a < 0)+count(j2a > EOS%nx2)
j1a=min(max(j1a+1,1),EOS%nx1)
j2a=min(max(j2a+1,1),EOS%nx2)
!
! --- Get relative position in cell ---
x1ta=lnP-EOS%lnx11d(j1a)
x2ta=lnT-EOS%lnx21d(j2a)
!
do i1=1,n1
  x1t=x1ta(i1)
  x2t=x2ta(i1)
  j1 =j1a(i1)
  j2 =j2a(i1)
  !
!#
# if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                              /* avoid temporary array Ct */
  ! --- Do bicubic interpolation for desired P/T pair ---
  rho(i1)=exp(    EOS%C( 1,1,j1,j2) + &
             x1t*(EOS%C( 2,1,j1,j2) + &
             x1t*(EOS%C( 3,1,j1,j2) + &
             x1t* EOS%C( 4,1,j1,j2))) + &
        x2t*(     EOS%C( 5,1,j1,j2) + &
             x1t*(EOS%C( 6,1,j1,j2) + &
             x1t*(EOS%C( 7,1,j1,j2) + &
             x1t* EOS%C( 8,1,j1,j2))) + &
        x2t*(     EOS%C( 9,1,j1,j2) + &
             x1t*(EOS%C(10,1,j1,j2) + &
             x1t*(EOS%C(11,1,j1,j2) + &
             x1t* EOS%C(12,1,j1,j2))) + &
        x2t*(     EOS%C(13,1,j1,j2) + &
             x1t*(EOS%C(14,1,j1,j2) + &
             x1t*(EOS%C(15,1,j1,j2) + &
             x1t* EOS%C(16,1,j1,j2)))  ))) )
  dlnrhodlnP(i1)= EOS%C( 2,1,j1,j2) + &
          x1t*(2.*EOS%C( 3,1,j1,j2) + &
          x1t* 3.*EOS%C( 4,1,j1,j2)) + &
     x2t*(        EOS%C( 6,1,j1,j2) + &
          x1t*(2.*EOS%C( 7,1,j1,j2) + &
          x1t* 3.*EOS%C( 8,1,j1,j2)) + &
     x2t*(        EOS%C(10,1,j1,j2) + &
          x1t*(2.*EOS%C(11,1,j1,j2) + &
          x1t* 3.*EOS%C(12,1,j1,j2)) + &
     x2t*(        EOS%C(14,1,j1,j2) + &
          x1t*(2.*EOS%C(15,1,j1,j2) + &
          x1t* 3.*EOS%C(16,1,j1,j2))  )))
  dlnrhodlnT(i1)= EOS%C( 5,1,j1,j2) + &
             x1t*(EOS%C( 6,1,j1,j2) + & 
             x1t*(EOS%C( 7,1,j1,j2) + &
             x1t* EOS%C( 8,1,j1,j2))) + &
    2.0*x2t*(     EOS%C( 9,1,j1,j2) + &
             x1t*(EOS%C(10,1,j1,j2) + & 
             x1t*(EOS%C(11,1,j1,j2) + &
             x1t* EOS%C(12,1,j1,j2))) + &
    1.5*x2t*(     EOS%C(13,1,j1,j2) + &
             x1t*(EOS%C(14,1,j1,j2) + & 
             x1t*(EOS%C(15,1,j1,j2) + &
             x1t* EOS%C(16,1,j1,j2)))  ))
  h(i1)=          EOS%C( 1,2,j1,j2) + &
             x1t*(EOS%C( 2,2,j1,j2) + &
             x1t*(EOS%C( 3,2,j1,j2) + &
             x1t* EOS%C( 4,2,j1,j2))) + &
        x2t*(     EOS%C( 5,2,j1,j2) + &
             x1t*(EOS%C( 6,2,j1,j2) + &
             x1t*(EOS%C( 7,2,j1,j2) + &
             x1t* EOS%C( 8,2,j1,j2))) + &
        x2t*(     EOS%C( 9,2,j1,j2) + &
             x1t*(EOS%C(10,2,j1,j2) + &
             x1t*(EOS%C(11,2,j1,j2) + &
             x1t* EOS%C(12,2,j1,j2))) + &
        x2t*(     EOS%C(13,2,j1,j2) + &
             x1t*(EOS%C(14,2,j1,j2) + &
             x1t*(EOS%C(15,2,j1,j2) + &
             x1t* EOS%C(16,2,j1,j2)))  )))
  dhdlnT(i1)=     EOS%C( 5,2,j1,j2) + &
             x1t*(EOS%C( 6,2,j1,j2) + &
             x1t*(EOS%C( 7,2,j1,j2) + &
             x1t* EOS%C( 8,2,j1,j2))) + &
    2.0*x2t*(     EOS%C( 9,2,j1,j2) + &
             x1t*(EOS%C(10,2,j1,j2) + &
             x1t*(EOS%C(11,2,j1,j2) + &
             x1t* EOS%C(12,2,j1,j2))) + &
    1.5*x2t*(     EOS%C(13,2,j1,j2) + &
             x1t*(EOS%C(14,2,j1,j2) + &
             x1t*(EOS%C(15,2,j1,j2) + &
             x1t* EOS%C(16,2,j1,j2)))  ))
  s(i1)=          EOS%C( 1,3,j1,j2) + &
             x1t*(EOS%C( 2,3,j1,j2) + & 
             x1t*(EOS%C( 3,3,j1,j2) + &
             x1t* EOS%C( 4,3,j1,j2))) + &
        x2t*(     EOS%C( 5,3,j1,j2) + &
             x1t*(EOS%C( 6,3,j1,j2) + &
             x1t*(EOS%C( 7,3,j1,j2) + &
             x1t* EOS%C( 8,3,j1,j2))) + &
        x2t*(     EOS%C( 9,3,j1,j2) + &
             x1t*(EOS%C(10,3,j1,j2) + &
             x1t*(EOS%C(11,3,j1,j2) + &
             x1t* EOS%C(12,3,j1,j2))) + &
        x2t*(     EOS%C(13,3,j1,j2) + &
             x1t*(EOS%C(14,3,j1,j2) + &
             x1t*(EOS%C(15,3,j1,j2) + &
             x1t* EOS%C(16,3,j1,j2)))  )))
!#
# else
!#                        /* using temporary array Ct */
  ! --- Do bicubic interpolation for desired P/T pair ---
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,1,j1,j2)
!#
# endif
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
  Ct=>EOS%C(:,1,j1,j2)
!#
# endif
!#
  rho(i1)=exp( &
                 Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
            x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
            x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
            x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
  dlnrhodlnP(i1)=              Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
            x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
            x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
            x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  )))
  dlnrhodlnT(i1)=Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
        2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
        1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,2,j1,j2)
!#
# endif
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
!#
  Ct=>EOS%C(:,2,j1,j2)
!#
# endif
!#
  h(i1)=         Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
            x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
            x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
            x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )))
  dhdlnT(i1)=    Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
        2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
        1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,3,j1,j2)
!#
# endif
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
  Ct=>EOS%C(:,3,j1,j2)
!#
# endif
!#
  s(i1)=         Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
            x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
            x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
            x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )))
!#
# endif
!#
end do ! i1
!
! --- Compute all desired quantities from tabulated values ---
cxt     =-dlnrhodlnT
xcp     = dhdlnT/(cxt*T)
xgam_rho= dhdlnT/((rho*dhdlnT)*dlnrhodlnP-cxt*cxt*P)
csound  = sqrt(xgam_rho*P)
xgam    = xgam_rho*rho
!
end subroutine gasinter1


!----------*********--------------------------------------------------------------------------------
subroutine eosinter0(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter0 ('equation_of_state_interpolation_0')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter(rho, e, P=P, dPdrho=dPdrho, dPde=dPde, T=T, dTde=dTde, s=s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT: (optional)
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1997-10-01 (B.F.) ierr0=0 instead of ierr=0
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-15 (B.F. Lyon) Removed ierr parameter
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: rho, e
real,            optional, intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
! --- Local variables ---
integer                                 :: j1, j2
real                                    :: lnx1, lnx2, &
                                           x1t, x2t
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
lnx1=alog(rho             )
lnx2=alog(max(e  +EOS%x2_shift, EOS%lnx2off))
!
! --- Determine indices of interpolation coefficients ---
j1=int((lnx1-EOS%lnx1off)*EOS%lnx1fac)
j2=int((lnx2-EOS%lnx2off)*EOS%lnx2fac)
j1=min(max(j1+1,1),EOS%nx1)
j2=min(max(j2+1,1),EOS%nx2)
!
! --- Get relative position in cell ---
x1t=lnx1-EOS%lnx11d(j1)
x2t=lnx2-EOS%lnx21d(j2)
!
!#
# if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                              /* avoid temporary array Ct */
if (present(s)) then
  ! --- Do bicubic interpolation for desired rho/e pair: s ---
    s= &
                          EOS%C( 1,1,j1,j2) + &
                     x1t*(EOS%C( 2,1,j1,j2) + &
                     x1t*(EOS%C( 3,1,j1,j2) + &
                     x1t* EOS%C( 4,1,j1,j2))) + &
                x2t*(     EOS%C( 5,1,j1,j2) + &
                     x1t*(EOS%C( 6,1,j1,j2) + &
                     x1t*(EOS%C( 7,1,j1,j2) + &
                     x1t* EOS%C( 8,1,j1,j2))) + &
                x2t*(     EOS%C( 9,1,j1,j2) + &
                     x1t*(EOS%C(10,1,j1,j2) + &
                     x1t*(EOS%C(11,1,j1,j2) + &
                     x1t* EOS%C(12,1,j1,j2))) + &
                x2t*(     EOS%C(13,1,j1,j2) + &
                     x1t*(EOS%C(14,1,j1,j2) + &
                     x1t*(EOS%C(15,1,j1,j2) + &
                     x1t* EOS%C(16,1,j1,j2)))  )))
endif
!
if (present(P)) then
  ! --- Do bicubic interpolation for desired rho/e pair: P ---
    P=exp( &
                          EOS%C( 1,2,j1,j2) + &
                     x1t*(EOS%C( 2,2,j1,j2) + &
                     x1t*(EOS%C( 3,2,j1,j2) + &
                     x1t* EOS%C( 4,2,j1,j2))) + &
                x2t*(     EOS%C( 5,2,j1,j2) + &
                     x1t*(EOS%C( 6,2,j1,j2) + &
                     x1t*(EOS%C( 7,2,j1,j2) + &
                     x1t* EOS%C( 8,2,j1,j2))) + &
                x2t*(     EOS%C( 9,2,j1,j2) + &
                     x1t*(EOS%C(10,2,j1,j2) + &
                     x1t*(EOS%C(11,2,j1,j2) + &
                     x1t* EOS%C(12,2,j1,j2))) + &
                x2t*(     EOS%C(13,2,j1,j2) + &
                     x1t*(EOS%C(14,2,j1,j2) + &
                     x1t*(EOS%C(15,2,j1,j2) + &
                     x1t* EOS%C(16,2,j1,j2)))  ))) )
    dPdrho=P/rho* ( &
                          EOS%C( 2,2,j1,j2) + &
                  x1t*(2.*EOS%C( 3,2,j1,j2) + &
                  x1t* 3.*EOS%C( 4,2,j1,j2)) + &
             x2t*(        EOS%C( 6,2,j1,j2) + &
                  x1t*(2.*EOS%C( 7,2,j1,j2) + &
                  x1t* 3.*EOS%C( 8,2,j1,j2)) + &
             x2t*(        EOS%C(10,2,j1,j2) + &
                  x1t*(2.*EOS%C(11,2,j1,j2) + & 
                  x1t* 3.*EOS%C(12,2,j1,j2)) + &
             x2t*(        EOS%C(14,2,j1,j2)  + &
                  x1t*(2.*EOS%C(15,2,j1,j2) + &
                  x1t* 3.*EOS%C(16,2,j1,j2))  ))) )
    dPde=P/(e+EOS%x2_shift)* ( &
                          EOS%C( 5,2,j1,j2) + &
                     x1t*(EOS%C( 6,2,j1,j2) + &
                     x1t*(EOS%C( 7,2,j1,j2) + &
                     x1t* EOS%C( 8,2,j1,j2))) + &
            2.0*x2t*(     EOS%C( 9,2,j1,j2) + &
                     x1t*(EOS%C(10,2,j1,j2) + &
                     x1t*(EOS%C(11,2,j1,j2) + &
                     x1t* EOS%C(12,2,j1,j2))) + &
            1.5*x2t*(     EOS%C(13,2,j1,j2) + &
                     x1t*(EOS%C(14,2,j1,j2)  + &
                     x1t*(EOS%C(15,2,j1,j2) + &
                     x1t* EOS%C(16,2,j1,j2)))  )) )
endif
!
if (present(T)) then
  ! --- Do bicubic interpolation for desired rho/e pair: T ---
    T=exp( &
                          EOS%C( 1,3,j1,j2) + &
                     x1t*(EOS%C( 2,3,j1,j2) + &
                     x1t*(EOS%C( 3,3,j1,j2) + &
                     x1t* EOS%C( 4,3,j1,j2))) + &
                x2t*(     EOS%C( 5,3,j1,j2) + &
                     x1t*(EOS%C( 6,3,j1,j2) + &
                     x1t*(EOS%C( 7,3,j1,j2) + &
                     x1t* EOS%C( 8,3,j1,j2))) + &
                x2t*(     EOS%C( 9,3,j1,j2) + &
                     x1t*(EOS%C(10,3,j1,j2) + &
                     x1t*(EOS%C(11,3,j1,j2) + &
                     x1t* EOS%C(12,3,j1,j2))) + &
                x2t*(     EOS%C(13,3,j1,j2) + &
                     x1t*(EOS%C(14,3,j1,j2) + &
                     x1t*(EOS%C(15,3,j1,j2) + &
                     x1t* EOS%C(16,3,j1,j2)))  ))) )
    dTde=T/(e+EOS%x2_shift)* ( &
                          EOS%C( 5,3,j1,j2) + &
                     x1t*(EOS%C( 6,3,j1,j2) + &
                     x1t*(EOS%C( 7,3,j1,j2) + &
                     x1t* EOS%C( 8,3,j1,j2))) + &
            2.0*x2t*(     EOS%C( 9,3,j1,j2) + &
                     x1t*(EOS%C(10,3,j1,j2) + &
                     x1t*(EOS%C(11,3,j1,j2) + &
                     x1t* EOS%C(12,3,j1,j2))) + &
            1.5*x2t*(     EOS%C(13,3,j1,j2) + &
                     x1t*(EOS%C(14,3,j1,j2) + &
                     x1t*(EOS%C(15,3,j1,j2) + &
                     x1t* EOS%C(16,3,j1,j2)))  )) )
endif
!#
# else
!#                        /* using temporary array Ct */
!
if (present(s)) then
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,1,j1,j2)
!#
# else
!#                    /* temporary array with pointer */
  Ct=>EOS%C(:,1,j1,j2)
!#
# endif
!#
  s= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
endif
!
if (present(P)) then
  ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,2,j1,j2)
!#
# else
!#                        /* temporary array with pointer */
  Ct=>EOS%C(:,2,j1,j2)
!#
# endif
!#
  P=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
  dPdrho=P/rho* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
              x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
              x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
              x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
  dPde=P/(e+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
endif
!
if (present(T)) then
  ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
  Ct=EOS%C(:,3,j1,j2)
!#
# else
!#                        /* temporary array with pointer */
  Ct=>EOS%C(:,3,j1,j2)
!#
# endif
!#
  T=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
  dTde=T/(e+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
endif
!#
# endif
!#
!
end subroutine eosinter0


!----------*********--------------------------------------------------------------------------------
subroutine eosinter1(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter1 ('equation_of_state_interpolation_1')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter(rho, e, P=P, dPdrho=dPdrho, dPde=dPde, T=T, dTde=dTde, s=s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT: (optional)
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-15 (B.F. Lyon) Removed ierr parameter
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),        intent(in)   :: rho, e
real, dimension(:), &
                 optional, intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
! --- Local variables ---
integer                                 :: i1, n1, j1, j2
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x1t, x2t
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
!
lnx1=alog(rho)
lnx2=alog(max(e  +EOS%x2_shift, EOS%lnx2off))
!
! --- Determine indices of interpolation coefficients ---
j1a=int((lnx1-EOS%lnx1off)*EOS%lnx1fac)
j2a=int((lnx2-EOS%lnx2off)*EOS%lnx2fac)
j1a=min(max(j1a+1,1),EOS%nx1)
j2a=min(max(j2a+1,1),EOS%nx2)
!
! --- Get relative position in cell ---
x1ta=lnx1-EOS%lnx11d(j1a)
x2ta=lnx2-EOS%lnx21d(j2a)
!
!#
# if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                              /* avoid temporary array Ct */
if (present(s)) then
  ! --- Do bicubic interpolation for desired rho/e pair: s ---
  do i1=1,n1
    x1t=x1ta(i1)
    x2t=x2ta(i1)
    j1 =j1a(i1)
    j2 =j2a(i1)
    !
    s(i1)= &
                          EOS%C( 1,1,j1,j2) + &
                     x1t*(EOS%C( 2,1,j1,j2) + &
                     x1t*(EOS%C( 3,1,j1,j2) + &
                     x1t* EOS%C( 4,1,j1,j2))) + &
                x2t*(     EOS%C( 5,1,j1,j2) + &
                     x1t*(EOS%C( 6,1,j1,j2) + &
                     x1t*(EOS%C( 7,1,j1,j2) + &
                     x1t* EOS%C( 8,1,j1,j2))) + &
                x2t*(     EOS%C( 9,1,j1,j2) + &
                     x1t*(EOS%C(10,1,j1,j2) + &
                     x1t*(EOS%C(11,1,j1,j2) + &
                     x1t* EOS%C(12,1,j1,j2))) + &
                x2t*(     EOS%C(13,1,j1,j2) + &
                     x1t*(EOS%C(14,1,j1,j2) + &
                     x1t*(EOS%C(15,1,j1,j2) + &
                     x1t* EOS%C(16,1,j1,j2)))  )))
  end do ! i
endif
!
if (present(P)) then
  ! --- Do bicubic interpolation for desired rho/e pair: P ---
  do i1=1,n1
    x1t=x1ta(i1)
    x2t=x2ta(i1)
    j1 =j1a(i1)
    j2 =j2a(i1)
    !
    P(i1)=exp( &
                          EOS%C( 1,2,j1,j2) + &
                     x1t*(EOS%C( 2,2,j1,j2) + &
                     x1t*(EOS%C( 3,2,j1,j2) + &
                     x1t* EOS%C( 4,2,j1,j2))) + &
                x2t*(     EOS%C( 5,2,j1,j2) + &
                     x1t*(EOS%C( 6,2,j1,j2) + &
                     x1t*(EOS%C( 7,2,j1,j2) + &
                     x1t* EOS%C( 8,2,j1,j2))) + &
                x2t*(     EOS%C( 9,2,j1,j2) + &
                     x1t*(EOS%C(10,2,j1,j2) + &
                     x1t*(EOS%C(11,2,j1,j2) + &
                     x1t* EOS%C(12,2,j1,j2))) + &
                x2t*(     EOS%C(13,2,j1,j2) + &
                     x1t*(EOS%C(14,2,j1,j2) + &
                     x1t*(EOS%C(15,2,j1,j2) + &
                     x1t* EOS%C(16,2,j1,j2)))  ))) )
    dPdrho(i1)=P(i1)/rho(i1)* ( &
                          EOS%C( 2,2,j1,j2) + &
                  x1t*(2.*EOS%C( 3,2,j1,j2) + &
                  x1t* 3.*EOS%C( 4,2,j1,j2)) + &
             x2t*(        EOS%C( 6,2,j1,j2) + &
                  x1t*(2.*EOS%C( 7,2,j1,j2) + &
                  x1t* 3.*EOS%C( 8,2,j1,j2)) + &
             x2t*(        EOS%C(10,2,j1,j2) + &
                  x1t*(2.*EOS%C(11,2,j1,j2) + & 
                  x1t* 3.*EOS%C(12,2,j1,j2)) + &
             x2t*(        EOS%C(14,2,j1,j2)  + &
                  x1t*(2.*EOS%C(15,2,j1,j2) + &
                  x1t* 3.*EOS%C(16,2,j1,j2))  ))) )
    dPde(i1)=P(i1)/(e(i1)+EOS%x2_shift)* ( &
                          EOS%C( 5,2,j1,j2) + &
                     x1t*(EOS%C( 6,2,j1,j2) + &
                     x1t*(EOS%C( 7,2,j1,j2) + &
                     x1t* EOS%C( 8,2,j1,j2))) + &
            2.0*x2t*(     EOS%C( 9,2,j1,j2) + &
                     x1t*(EOS%C(10,2,j1,j2) + &
                     x1t*(EOS%C(11,2,j1,j2) + &
                     x1t* EOS%C(12,2,j1,j2))) + &
            1.5*x2t*(     EOS%C(13,2,j1,j2) + &
                     x1t*(EOS%C(14,2,j1,j2)  + &
                     x1t*(EOS%C(15,2,j1,j2) + &
                     x1t* EOS%C(16,2,j1,j2)))  )) )
  end do ! i
endif
!
if (present(T)) then
  ! --- Do bicubic interpolation for desired rho/e pair: T ---
  do i1=1,n1
    x1t=x1ta(i1)
    x2t=x2ta(i1)
    j1 =j1a(i1)
    j2 =j2a(i1)
    !
    T(i1)=exp( &
                          EOS%C( 1,3,j1,j2) + &
                     x1t*(EOS%C( 2,3,j1,j2) + &
                     x1t*(EOS%C( 3,3,j1,j2) + &
                     x1t* EOS%C( 4,3,j1,j2))) + &
                x2t*(     EOS%C( 5,3,j1,j2) + &
                     x1t*(EOS%C( 6,3,j1,j2) + &
                     x1t*(EOS%C( 7,3,j1,j2) + &
                     x1t* EOS%C( 8,3,j1,j2))) + &
                x2t*(     EOS%C( 9,3,j1,j2) + &
                     x1t*(EOS%C(10,3,j1,j2) + &
                     x1t*(EOS%C(11,3,j1,j2) + &
                     x1t* EOS%C(12,3,j1,j2))) + &
                x2t*(     EOS%C(13,3,j1,j2) + &
                     x1t*(EOS%C(14,3,j1,j2) + &
                     x1t*(EOS%C(15,3,j1,j2) + &
                     x1t* EOS%C(16,3,j1,j2)))  ))) )
    dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* ( &
                          EOS%C( 5,3,j1,j2) + &
                     x1t*(EOS%C( 6,3,j1,j2) + &
                     x1t*(EOS%C( 7,3,j1,j2) + &
                     x1t* EOS%C( 8,3,j1,j2))) + &
            2.0*x2t*(     EOS%C( 9,3,j1,j2) + &
                     x1t*(EOS%C(10,3,j1,j2) + &
                     x1t*(EOS%C(11,3,j1,j2) + &
                     x1t* EOS%C(12,3,j1,j2))) + &
            1.5*x2t*(     EOS%C(13,3,j1,j2) + &
                     x1t*(EOS%C(14,3,j1,j2) + &
                     x1t*(EOS%C(15,3,j1,j2) + &
                     x1t* EOS%C(16,3,j1,j2)))  )) )
  end do ! i
endif
!#
# else
!#                        /* using temporary array Ct */
if (present(s)) then
  ! --- Do bicubic interpolation for desired rho/e pair: s ---
  do i1=1,n1
    x1t=x1ta(i1)
    x2t=x2ta(i1)
    j1 =j1a(i1)
    j2 =j2a(i1)
!#
#   if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                /* temporary array is copied */
    Ct=EOS%C(:,1,j1,j2)
!#
#   else
!#                          /* temporary array with pointer */
    Ct=>EOS%C(:,1,j1,j2)
!#
#   endif
!#
    s(i1)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
  end do ! i
endif
!
if (present(P)) then
  ! --- Do bicubic interpolation for desired rho/e pair: P ---
  do i1=1,n1
    x1t=x1ta(i1)
    x2t=x2ta(i1)
    j1 =j1a(i1)
    j2 =j2a(i1)
!#
#   if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                /* temporary array is copied */
    Ct=EOS%C(:,2,j1,j2)
!#
#   else
!#                          /* temporary array with pointer */
    Ct=>EOS%C(:,2,j1,j2)
!#
#   endif
!#
    P(i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
    dPdrho(i1)=P(i1)/rho(i1)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
              x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
              x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
              x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
    dPde(i1)=P(i1)/(e(i1)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
  end do ! i
endif
!
if (present(T)) then
  ! --- Do bicubic interpolation for desired rho/e pair: T ---
  do i1=1,n1
    x1t=x1ta(i1)
    x2t=x2ta(i1)
    j1 =j1a(i1)
    j2 =j2a(i1)
!#
#   if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                /* temporary array is copied */
    Ct=EOS%C(:,3,j1,j2)
!#
#   else
!#                          /* temporary array with pointer */
    Ct=>EOS%C(:,3,j1,j2)
!#
#   endif
!#
    T(i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
    dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
  end do ! i
endif
!#
# endif
!#
!
end subroutine eosinter1


!----------*************----------------------------------------------------------------------------
subroutine eosinternewa1(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinternewa1 ('equation_of_state_interpolation_new_a1')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinternewa1(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-09 (B.F.) Experimental version; remove ierr count; use array of coeffs Ct(:,16,2:3)
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real,    dimension(:),     intent(in)   :: rho, e
real,    dimension(:),     intent(out)  :: logP, T, dTde
!
! --- Local variables ---
integer                                 :: i, j, n
integer, dimension(1:size(rho,1))       :: j1, j2
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x1t, x2t
!real,    dimension(16,1:size(rho,1),2:3):: Ctt
real,    dimension(1:size(rho,1),16,2:3):: Ct
!---------------------------------------------------------------------------------------------------
!
n=size(rho,1)
!
! --- Determine indices of interpolation coefficients ---
lnx1=alog(rho)
j1  =int((lnx1-EOS%lnx1off)*EOS%lnx1fac)
j1  =min(max(j1+1,1),EOS%nx1)
lnx2=alog(max(e  +EOS%x2_shift, EOS%lnx2off))
j2  =int((lnx2-EOS%lnx2off)*EOS%lnx2fac)
j2  =min(max(j2+1,1),EOS%nx2)
!
! --- Get relative position in cell ---
!DIR$ IVDEP
do i=1,n
  x1ta(i)=lnx1(i)-EOS%lnx11d(j1(i))
  x2ta(i)=lnx2(i)-EOS%lnx21d(j2(i))
end do ! i
!
! --- Put coefficients into temporary array ---
!DIR$ IVDEP
do i=1,n
  !DIR$ IVDEP
  Ct(i,:,2)=EOS%C(:,2,j1(i),j2(i))
  !DIR$ IVDEP
  Ct(i,:,3)=EOS%C(:,3,j1(i),j2(i))
end do ! i
!
!!DIR$ IVDEP
!do i=1,n
!  !DIR$ IVDEP
!  Ctt(:,i,2)=EOS%C(:,2,j1(i),j2(i))
!  !DIR$ IVDEP
!  Ctt(:,i,3)=EOS%C(:,3,j1(i),j2(i))
!end do ! i
!Ct(:,:,2)=transpose(Ctt(:,:,2))
!Ct(:,:,3)=transpose(Ctt(:,:,3))
!Ct=reshape(Ctt, (/ n,16,2 /), order=(/ 2,1,3 /))
!DIR$ IVDEP
!do j=1,16
!  !DIR$ IVDEP
!  Ct(:,j,2)=Ctt(j,:,2)
!  !DIR$ IVDEP
!  Ct(:,j,3)=Ctt(j,:,3)
!end do ! f
!DIR$ IVDEP
!do i=1,n
!  !DIR$ IVDEP
!  Ct(i,:,2)=Ctt(:,i,2)
!  !DIR$ IVDEP
!  Ct(i,:,3)=Ctt(:,i,3)
!end do ! f
!
! --- Do bicubic interpolation for desired rho/e pair: P, T ---
do i=1,n
  x1t=x1ta(i)
  x2t=x2ta(i)
  logP(i)= &
           Ct(i, 1,2) + x1t*(Ct(i, 2,2) + x1t*(Ct(i, 3,2) + x1t*Ct(i, 4,2))) + &
     x2t*( Ct(i, 5,2) + x1t*(Ct(i, 6,2) + x1t*(Ct(i, 7,2) + x1t*Ct(i, 8,2))) + &
     x2t*( Ct(i, 9,2) + x1t*(Ct(i,10,2) + x1t*(Ct(i,11,2) + x1t*Ct(i,12,2))) + &
     x2t*( Ct(i,13,2) + x1t*(Ct(i,14,2) + x1t*(Ct(i,15,2) + x1t*Ct(i,16,2)))  )))
end do ! i
do i=1,n
  x1t=x1ta(i)
  x2t=x2ta(i)
  T(i)=exp( &
           Ct(i, 1,3) + x1t*(Ct(i, 2,3) + x1t*(Ct(i, 3,3) + x1t*Ct(i, 4,3))) + &
     x2t*( Ct(i, 5,3) + x1t*(Ct(i, 6,3) + x1t*(Ct(i, 7,3) + x1t*Ct(i, 8,3))) + &
     x2t*( Ct(i, 9,3) + x1t*(Ct(i,10,3) + x1t*(Ct(i,11,3) + x1t*Ct(i,12,3))) + &
     x2t*( Ct(i,13,3) + x1t*(Ct(i,14,3) + x1t*(Ct(i,15,3) + x1t*Ct(i,16,3)))  ))) )
end do ! i
do i=1,n
  x1t=x1ta(i)
  x2t=x2ta(i)
  dTde(i)=T(i)/(e(i)+EOS%x2_shift)* ( &
                Ct(i, 5,3) + x1t*(Ct(i, 6,3) + x1t*(Ct(i, 7,3) + x1t*Ct(i, 8,3))) + &
      2.0*x2t*( Ct(i, 9,3) + x1t*(Ct(i,10,3) + x1t*(Ct(i,11,3) + x1t*Ct(i,12,3))) + &
      1.5*x2t*( Ct(i,13,3) + x1t*(Ct(i,14,3) + x1t*(Ct(i,15,3) + x1t*Ct(i,16,3)))  )) )
end do ! i
!
end subroutine eosinternewa1


!----------*************----------------------------------------------------------------------------
subroutine eosinternewb1(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinternewb1 ('equation_of_state_interpolation_new_b1')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinternewb1(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-09 (B.F.) Experimental version; remove ierr count; allow better vectorization
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real,    dimension(:),     intent(in)   :: rho, e
real,    dimension(:),     intent(out)  :: logP, T, dTde
!
! --- Local variables ---
integer                                 :: i, j, n
integer, dimension(1:size(rho,1))       :: j1, j2
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x1t, x2t
!real,    dimension(16,1:size(rho,1),2:3):: Ctt
!real,    dimension(1:size(rho,1),16,2:3):: Ct
real,    dimension(8,4)                 :: Ct
real,    dimension(8)                   :: poly1
!---------------------------------------------------------------------------------------------------
!
n=size(rho,1)
!
! --- Determine indices of interpolation coefficients and relative position in cell ---
lnx1=alog(rho)
j1  =int((lnx1-EOS%lnx1off)*EOS%lnx1fac)
!r1  =(lnx1-EOS%lnx1off)*EOS%lnx1fac
!j1  =int(r1)
j1  =min(max(j1+1,1),EOS%nx1)
!x1ta=(r1-real(j1-1))/EOS%lnx1fac+EOS%lnx1off
!DIR$ IVDEP
x1ta=lnx1-EOS%lnx11d(j1)
lnx2=alog(max(e  +EOS%x2_shift, EOS%lnx2off))
j2  =int((lnx2-EOS%lnx2off)*EOS%lnx2fac)
!r2  =(lnx2-EOS%lnx2off)*EOS%lnx2fac
!j2  =int(r2)
j2  =min(max(j2+1,1),EOS%nx2)
!x2ta=(r2-real(j2-1))/EOS%lnx2fac+EOS%lnx2off
!DIR$ IVDEP
x2ta=lnx2-EOS%lnx21d(j2)
!
! --- Get relative position in cell ---
! !DIR$ IVDEP
!do i=1,n
!  x1ta(i)=lnx1(i)-EOS%lnx11d(j1(i))
!  x2ta(i)=lnx2(i)-EOS%lnx21d(j2(i))
!end do ! i
!
do i=1,n
  ! --- Put coefficients into temporary array ---
  !Ct(1:4,:)=reshape(EOS%C(:,2,j1(i),j2(i)), (/ 4,4 /), order=(/ 2,1 /))
  !Ct(5:8,:)=reshape(EOS%C(:,3,j1(i),j2(i)), (/ 4,4 /), order=(/ 2,1 /))
  !Ct(1:4,:)=transpose(reshape(EOS%C(:,2,j1(i),j2(i)), (/ 4,4 /)))
  !Ct(5:8,:)=transpose(reshape(EOS%C(:,3,j1(i),j2(i)), (/ 4,4 /)))
  Ct(1,1:4)=EOS%C( 1: 4,2,j1(i),j2(i))
  Ct(2,1:4)=EOS%C( 5: 8,2,j1(i),j2(i))
  Ct(3,1:4)=EOS%C( 9:12,2,j1(i),j2(i))
  Ct(4,1:4)=EOS%C(13:16,2,j1(i),j2(i))
  Ct(5,1:4)=EOS%C( 1: 4,3,j1(i),j2(i))
  Ct(6,1:4)=EOS%C( 5: 8,3,j1(i),j2(i))
  Ct(7,1:4)=EOS%C( 9:12,3,j1(i),j2(i))
  Ct(8,1:4)=EOS%C(13:16,3,j1(i),j2(i))
  !
  ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
  x1t=x1ta(i)
  do j=1,8
    poly1(j)=Ct(j, 1) + x1t*(Ct(j, 2) + x1t*(Ct(j, 3) + x1t*Ct(j, 4)))
  end do ! j
  !
  ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
  x2t=x2ta(i)
  logP(i)=         poly1(1) + &
              x2t*(poly1(2) + &
              x2t*(poly1(3) + &
              x2t*(poly1(4)  )))
  !T(i)   =exp( &
  !                 poly1(5) + &
  !            x2t*(poly1(6) + &
  !            x2t*(poly1(7) + &
  !            x2t*(poly1(8)  ))) )
  !dTde(i)=T(i)/(e(i)+EOS%x2_shift)* ( &
  !                 poly1(6) + &
  !        2.0*x2t*(poly1(7) + &
  !        1.5*x2t*(poly1(8)  )) )
  T(i)   =         poly1(5) + &
              x2t*(poly1(6) + &
              x2t*(poly1(7) + &
              x2t*(poly1(8)  )))
  dTde(i)=         poly1(6) + &
          2.0*x2t*(poly1(7) + &
          1.5*x2t*(poly1(8)  ))
end do ! i
!
do i=1,n
  ! --- Perform final exponentiation and scaling ---
  T(i)   =exp(T(i))
  dTde(i)=T(i)/(e(i)+EOS%x2_shift)* dTde(i)
end do ! i
!
end subroutine eosinternewb1


!----------*************----------------------------------------------------------------------------
subroutine eosinternewc1(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinternewc1 ('equation_of_state_interpolation_new_b1')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinternewc1(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-09 (B.F.) Experimental version; remove ierr count; allow better vectorization
!   2013-02-11 (B.F.) Unrolling by 2
!   2013-02-15 (B.F.) Variable renaming
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real,    dimension(:),     intent(in)   :: rho, e
real,    dimension(:),     intent(out)  :: logP, T, dTde
!
! --- Local variables ---
integer                                 :: i1, ib, k, l, n1, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(2*8,4)               :: Ct
real,    dimension(2*8)                 :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
n1    =size(rho,1)
n1part=n1/2*2
!n1part=n/4*4
!
! --- Determine indices of interpolation coefficients and relative position in cell ---
lnx1=alog(rho)
j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
!DIR$ IVDEP
x1ta=lnx1-EOS%lnx11d(j1a)
lnx2=alog(max(e  +EOS%x2_shift, EOS%lnx2off))
j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
!DIR$ IVDEP
x2ta=lnx2-EOS%lnx21d(j2a)
!
! --- Unroll the first part ---
do i1=1,n1part,2
!!$  Ct( 1,1:4)=EOS%C( 1: 4,2,j1a(i1  ),j2a(i1  ))
!!$  Ct( 2,1:4)=EOS%C( 5: 8,2,j1a(i1  ),j2a(i1  ))
!!$  Ct( 3,1:4)=EOS%C( 9:12,2,j1a(i1  ),j2a(i1  ))
!!$  Ct( 4,1:4)=EOS%C(13:16,2,j1a(i1  ),j2a(i1  ))
!!$  Ct( 5,1:4)=EOS%C( 1: 4,3,j1a(i1  ),j2a(i1  ))
!!$  Ct( 6,1:4)=EOS%C( 5: 8,3,j1a(i1  ),j2a(i1  ))
!!$  Ct( 7,1:4)=EOS%C( 9:12,3,j1a(i1  ),j2a(i1  ))
!!$  Ct( 8,1:4)=EOS%C(13:16,3,j1a(i1  ),j2a(i1  ))
!!$  Ct( 9,1:4)=EOS%C( 1: 4,2,j1a(i1+1),j2a(i1+1))
!!$  Ct(10,1:4)=EOS%C( 5: 8,2,j1a(i1+1),j2a(i1+1))
!!$  Ct(11,1:4)=EOS%C( 9:12,2,j1a(i1+1),j2a(i1+1))
!!$  Ct(12,1:4)=EOS%C(13:16,2,j1a(i1+1),j2a(i1+1))
!!$  Ct(13,1:4)=EOS%C( 1: 4,3,j1a(i1+1),j2a(i1+1))
!!$  Ct(14,1:4)=EOS%C( 5: 8,3,j1a(i1+1),j2a(i1+1))
!!$  Ct(15,1:4)=EOS%C( 9:12,3,j1a(i1+1),j2a(i1+1))
!!$  Ct(16,1:4)=EOS%C(13:16,3,j1a(i1+1),j2a(i1+1))
  !
!!$  !DIR$ UNROLL
!!$  do ib=0,1
!!$    ! --- Put coefficients into temporary array ---
!!$    Ct( 1+8*ib,1:4)=EOS%C( 1: 4,2,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 2+8*ib,1:4)=EOS%C( 5: 8,2,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 3+8*ib,1:4)=EOS%C( 9:12,2,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 4+8*ib,1:4)=EOS%C(13:16,2,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 5+8*ib,1:4)=EOS%C( 1: 4,3,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 6+8*ib,1:4)=EOS%C( 5: 8,3,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 7+8*ib,1:4)=EOS%C( 9:12,3,j1a(i1+ib),j2a(i1+ib))
!!$    Ct( 8+8*ib,1:4)=EOS%C(13:16,3,j1a(i1+ib),j2a(i1+ib))
!!$  end do ! ib
  !
!!$  !DIR$ UNROLL
!!$  do ib=0,1
!!$    !DIR$ UNROLL
!!$    do k=0,1
!!$      !DIR$ UNROLL
!!$      do l=0,3
!!$        Ct(8*j+4*k+l+1, 1:4)=EOS%C(4*l+1: 4*l+4, 2+k, j1a(i1+ib), j2a(i1+ib))
!!$      end do ! l
!!$    end do ! k
!!$  end do ! ib
  !
  ! --- Put coefficients into temporary array ---
  !DIR$ UNROLL
  do ib=0,1
    !DIR$ UNROLL
    do k=0,1
      Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
      Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
      Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
      Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
    end do ! k
  end do ! ib
  !
  ! --- Put delta rho values into array ---
!!$  !DIR$ UNROLL
!!$  do ib=0,1
!!$    x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
!!$  end do ! ib
  x1t( 1: 8)=x1ta(i1  )
  x1t( 9:16)=x1ta(i1+1)
  !x1t(17:24)=x1ta(i1+2)
  !x1t(25:32)=x1ta(i1+3)
  !
  ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
  do ib=1,2*8
    poly1(ib)=Ct(ib, 1) + x1t(ib)*(Ct(ib, 2) + x1t(ib)*(Ct(ib, 3) + x1t(ib)*Ct(ib, 4)))
  end do ! ib
  !
!!$  x2t      =x2ta(i1)
!!$  logP(i1  )=        poly1( 1) + &
!!$                x2t*(poly1( 2) + &
!!$                x2t*(poly1( 3) + &
!!$                x2t*(poly1( 4)  )))
!!$  T(   i1  )=        poly1( 5) + &
!!$                x2t*(poly1( 6) + &
!!$                x2t*(poly1( 7) + &
!!$                x2t*(poly1( 8)  )))
!!$  dTde(i1  )=        poly1( 6) + &
!!$            2.0*x2t*(poly1( 7) + &
!!$            1.5*x2t*(poly1( 8)  ))
!!$  x2t      =x2ta(i1+1)
!!$  logP(i1+1)=        poly1( 9) + &
!!$                x2t*(poly1(10) + &
!!$                x2t*(poly1(11) + &
!!$                x2t*(poly1(12)  )))
!!$  T(   i1+1)=        poly1(13) + &
!!$                x2t*(poly1(14) + &
!!$                x2t*(poly1(15) + &
!!$                x2t*(poly1(16)  )))
!!$  dTde(i1+1)=        poly1(14) + &
!!$            2.0*x2t*(poly1(15) + &
!!$            1.5*x2t*(poly1(16)  ))
  !
  ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
  !DIR$ UNROLL
  do ib=0,1
    x2t      =x2ta(i1+ib)
    logP(i1+ib)=       poly1(1+8*ib) + &
                  x2t*(poly1(2+8*ib) + &
                  x2t*(poly1(3+8*ib) + &
                  x2t*(poly1(4+8*ib)  )))
    T(   i1+ib)=       poly1(5+8*ib) + &
                  x2t*(poly1(6+8*ib) + &
                  x2t*(poly1(7+8*ib) + &
                  x2t*(poly1(8+8*ib)  )))
    dTde(i1+ib)=       poly1(6+8*ib) + &
              2.0*x2t*(poly1(7+8*ib) + &
              1.5*x2t*(poly1(8+8*ib)  ))
  end do ! ib
end do ! i1
!
! --- Do the rest ---
do i1=n1part+1,n1
  ! --- Put coefficients into temporary array ---
!!$  Ct(1,1:4)=EOS%C( 1: 4,2,j1a(i1),j2a(i1))
!!$  Ct(2,1:4)=EOS%C( 5: 8,2,j1a(i1),j2a(i1))
!!$  Ct(3,1:4)=EOS%C( 9:12,2,j1a(i1),j2a(i1))
!!$  Ct(4,1:4)=EOS%C(13:16,2,j1a(i1),j2a(i1))
!!$  Ct(5,1:4)=EOS%C( 1: 4,3,j1a(i1),j2a(i1))
!!$  Ct(6,1:4)=EOS%C( 5: 8,3,j1a(i1),j2a(i1))
!!$  Ct(7,1:4)=EOS%C( 9:12,3,j1a(i1),j2a(i1))
!!$  Ct(8,1:4)=EOS%C(13:16,3,j1a(i1),j2a(i1))
  !
  !DIR$ UNROLL
  do k=0,1
    Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
    Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
    Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
    Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
  end do ! k
  !
  ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
  !x1t(1: 8)=x1ta(i1)
  do ib=1,8
    poly1(ib)=Ct(ib, 1) + x1ta(i1)*(Ct(ib, 2) + x1ta(i1)*(Ct(ib, 3) + x1ta(i1)*Ct(ib, 4)))
  end do ! ib
  !
  ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
  x2t    =x2ta(i1)
  logP(i1)=        poly1(1) + &
              x2t*(poly1(2) + &
              x2t*(poly1(3) + &
              x2t*(poly1(4)  )))
  T(i1   )=        poly1(5) + &
              x2t*(poly1(6) + &
              x2t*(poly1(7) + &
              x2t*(poly1(8)  )))
  dTde(i1)=        poly1(6) + &
          2.0*x2t*(poly1(7) + &
          1.5*x2t*(poly1(8)  ))
end do ! i1
!
! --- Perform final exponentiation and scaling ---
do i1=1,n1
  T(i1)   =exp(T(i1))
  dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* dTde(i1)
end do ! i1
!
end subroutine eosinternewc1


!----------*************----------------------------------------------------------------------------
subroutine eosinternewd1(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinternewd1 ('equation_of_state_interpolation_new_b1')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinternewd1(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-09 (B.F.) Experimental version; remove ierr count; allow better vectorization
!   2013-02-11 (B.F.) Rearrangements in index computation
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real,    dimension(:),     intent(in)   :: rho, e
real,    dimension(:),     intent(out)  :: logP, T, dTde
!
! --- Local variables ---
integer                                 :: i, j, n
integer, dimension(1:size(rho,1))       :: j1, j2
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x1t, x2t
real,    dimension(8,4)                 :: Ct
real,    dimension(8)                   :: poly1
!---------------------------------------------------------------------------------------------------
!
n=size(rho,1)
!
! --- Determine indices of interpolation coefficients and relative position in cell ---
lnx1=alog(rho)
!j1  =int((lnx1-EOS%lnx1off)*EOS%lnx1fac)
!j1  =min(max(j1+1,1),EOS%nx1)
j1  =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
!DIR$ IVDEP
x1ta=lnx1-EOS%lnx11d(j1)
lnx2=alog(max(e  +EOS%x2_shift, EOS%lnx2off))
!j2  =int((lnx2-EOS%lnx2off)*EOS%lnx2fac)
!j2  =min(max(j2+1,1),EOS%nx2)
j2  =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
!DIR$ IVDEP
x2ta=lnx2-EOS%lnx21d(j2)
!
!!$!DIR$ IVDEP
!!$do i=1,n
!!$  lnx1(i)=alog(rho(i))
!!$  j1(i)  =int((lnx1(i)-EOS%lnx1off)*EOS%lnx1fac)
!!$  j1(i)  =min(max(j1(i)+1,1),EOS%nx1)
!!$  x1ta(i)=lnx1(i)-EOS%lnx11d(j1(i))
!!$  lnx2(i)=alog(max(e(i)  +EOS%x2_shift, EOS%lnx2off))
!!$  j2(i)  =int((lnx2(i)-EOS%lnx2off)*EOS%lnx2fac)
!!$  j2(i)  =min(max(j2(i)+1,1),EOS%nx2)
!!$  x2ta(i)=lnx2(i)-EOS%lnx21d(j2(i))
!!$end do ! i
!
do i=1,n
  ! --- Put coefficients into temporary array ---
  Ct(1,1:4)=EOS%C( 1: 4,2,j1(i),j2(i))
  Ct(2,1:4)=EOS%C( 5: 8,2,j1(i),j2(i))
  Ct(3,1:4)=EOS%C( 9:12,2,j1(i),j2(i))
  Ct(4,1:4)=EOS%C(13:16,2,j1(i),j2(i))
  Ct(5,1:4)=EOS%C( 1: 4,3,j1(i),j2(i))
  Ct(6,1:4)=EOS%C( 5: 8,3,j1(i),j2(i))
  Ct(7,1:4)=EOS%C( 9:12,3,j1(i),j2(i))
  Ct(8,1:4)=EOS%C(13:16,3,j1(i),j2(i))
  !
  ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
  x1t=x1ta(i)
  do j=1,8
    poly1(j)=Ct(j, 1) + x1t*(Ct(j, 2) + x1t*(Ct(j, 3) + x1t*Ct(j, 4)))
  end do ! j
  !
  ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
  x2t=x2ta(i)
  logP(i)=         poly1(1) + &
              x2t*(poly1(2) + &
              x2t*(poly1(3) + &
              x2t*(poly1(4)  )))
  T(i)   =         poly1(5) + &
              x2t*(poly1(6) + &
              x2t*(poly1(7) + &
              x2t*(poly1(8)  )))
  dTde(i)=         poly1(6) + &
          2.0*x2t*(poly1(7) + &
          1.5*x2t*(poly1(8)  ))
end do ! i
!
do i=1,n
  ! --- Perform final exponentiation and scaling ---
  T(i)   =exp(T(i))
  dTde(i)=T(i)/(e(i)+EOS%x2_shift)* dTde(i)
end do ! i
!
end subroutine eosinternewd1


!----------*********--------------------------------------------------------------------------------
subroutine eosinter2(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter2 ('equation_of_state_interpolation_2')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, s, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter(rho, e, P=P, dPdrho=dPdrho, dPde=dPde, T=T, dTde=dTde, s=s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT: (optional)
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 2D version
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-15 (B.F. Lyon) Removed ierr parameter. Temporary arrays only 1D.
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:),      intent(in)   :: rho, e
real, dimension(:,:), &
                 optional, intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
! --- Local variables ---
integer                                 :: i1, i2, n1, n2
integer, dimension(1:size(rho,1))       :: j1, j2
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x1t, x2t
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                               /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
n2=size(rho,2)
!
do i2=1,n2
  ! --- Determine indices of interpolation coefficients and relative position in cell ---
  lnx1=alog(rho(:,i2))
  j1  =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1,1),EOS%nx1)
  !DIR$ IVDEP
  x1ta=lnx1-EOS%lnx11d(j1)
  lnx2=alog(max(e(:,i2)  +EOS%x2_shift, EOS%lnx2off))
  j2  =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1,1),EOS%nx2)
  !DIR$ IVDEP
  x2ta=lnx2-EOS%lnx21d(j2)
  !
!#
# if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                              /* avoid temporary array Ct */
  if (present(s)) then
    ! --- Do bicubic interpolation for desired rho/e pair: s ---
    do i1=1,n1
      x1t=x1ta(i1)
      x2t=x2ta(i1)
      !
      s(i1,i2)= &
                          EOS%C( 1,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 2,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 3,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 4,1,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 5,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,1,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 9,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,1,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C(13,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,1,j1(i1),j2(i1))))  )))
    end do ! i1
  endif
  !
  if (present(P)) then
    ! --- Do bicubic interpolation for desired rho/e pair: P ---
    do i1=1,n1
      x1t=x1ta(i1)
      x2t=x2ta(i1)
      !
      P(i1,i2)=exp( &
                          EOS%C( 1,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 2,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 3,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 4,2,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 5,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,2,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 9,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,2,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C(13,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,2,j1(i1),j2(i1))))  ))) )
      dPdrho(i1,i2)=P(i1,i2)/rho(i1,i2)* ( &
                          EOS%C( 2,2,j1(i1),j2(i1)) + &
                  x1t*(2.*EOS%C( 3,2,j1(i1),j2(i1)) + &
                  x1t* 3.*EOS%C( 4,2,j1(i1),j2(i1))) + &
             x2t*(        EOS%C( 6,2,j1(i1),j2(i1)) + &
                  x1t*(2.*EOS%C( 7,2,j1(i1),j2(i1)) + &
                  x1t* 3.*EOS%C( 8,2,j1(i1),j2(i1))) + &
             x2t*(        EOS%C(10,2,j1(i1),j2(i1)) + &
                  x1t*(2.*EOS%C(11,2,j1(i1),j2(i1)) + & 
                  x1t* 3.*EOS%C(12,2,j1(i1),j2(i1))) + &
             x2t*(        EOS%C(14,2,j1(i1),j2(i1))  + &
                  x1t*(2.*EOS%C(15,2,j1(i1),j2(i1)) + &
                  x1t* 3.*EOS%C(16,2,j1(i1),j2(i1)))  ))) )
      dPde(i1,i2)=P(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                          EOS%C( 5,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,2,j1(i1),j2(i1)))) + &
            2.0*x2t*(     EOS%C( 9,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,2,j1(i1),j2(i1)))) + &
            1.5*x2t*(     EOS%C(13,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,2,j1(i1),j2(i1))  + &
                     x1t*(EOS%C(15,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,2,j1(i1),j2(i1))))  )) )
    end do ! i1
  endif
  !
  if (present(T)) then
    ! --- Do bicubic interpolation for desired rho/e pair: T ---
    do i1=1,n1
      x1t=x1ta(i1)
      x2t=x2ta(i1)
      !
      T(i1,i2)=exp( &
                          EOS%C( 1,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 2,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 3,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 4,3,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 5,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,3,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 9,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,3,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C(13,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,3,j1(i1),j2(i1))))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                          EOS%C( 5,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,3,j1(i1),j2(i1)))) + &
            2.0*x2t*(     EOS%C( 9,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,3,j1(i1),j2(i1)))) + &
            1.5*x2t*(     EOS%C(13,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,3,j1(i1),j2(i1))))  )) )
    end do ! i1
  endif
!#
# else
!#                        /* using temporary array Ct */
  if (present(s)) then
    ! --- Do bicubic interpolation for desired rho/e pair: s ---
    do i1=1,n1
      x1t=x1ta(i1)
      x2t=x2ta(i1)
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,1,j1(i1),j2(i1))
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,1,j1(i1),j2(i1))
!#
#     endif
!#
      s(i1,i2)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
    end do ! i1
  endif
  !
  if (present(P)) then
    ! --- Do bicubic interpolation for desired rho/e pair: P ---
    do i1=1,n1
      x1t=x1ta(i1)
      x2t=x2ta(i1)
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1(i1),j2(i1))
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1(i1),j2(i1))
!#
#     endif
!#
      P(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2)=P(i1,i2)/rho(i1,i2)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
              x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
              x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
              x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2)=P(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
    end do ! i1
  endif
  !
  if (present(T)) then
    ! --- Do bicubic interpolation for desired rho/e pair: T ---
    do i1=1,n1
      x1t=x1ta(i1)
      x2t=x2ta(i1)
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1(i1),j2(i1))
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1(i1),j2(i1))
!#
#     endif
!#
      T(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
    end do ! i1
  endif
!#
# endif
!#
end do ! i2
!
end subroutine eosinter2


!----------*********--------------------------------------------------------------------------------
subroutine eosinter3(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3 ('equation_of_state_interpolation_3')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, s, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter(rho, e, P=P, dPdrho=dPdrho, dPde=dPde, T=T, dTde=dTde, s=s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT: (optional)
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-15 (B.F. Lyon) Removed ierr parameter. Temporary arrays only 1D.
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:), &
                 optional, intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
! --- Local variables ---
integer                                 :: i1, i2, i3, n1, n2, n3
integer, dimension(1:size(rho,1))       :: j1, j2
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x1t, x2t
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
do i3=1,n3
  do i2=1,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(rho(:,i2,i3))
    j1  =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1,1),EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1)
    lnx2=alog(max(e(:,i2,i3)  +EOS%x2_shift, EOS%lnx2off))
    j2  =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1,1),EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2)
    !
!#
#   if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                /* avoid temporary array Ct */
    if (present(s)) then
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
      do i1=1,n1
        x1t=x1ta(i1)
        x2t=x2ta(i1)
        !
        s(i1,i2,i3)= &
                          EOS%C( 1,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 2,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 3,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 4,1,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 5,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,1,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 9,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,1,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C(13,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,1,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,1,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,1,j1(i1),j2(i1))))  )))
      end do ! i1
    endif
    !
    if (present(P)) then
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
      do i1=1,n1
        x1t=x1ta(i1)
        x2t=x2ta(i1)
        !
        P(i1,i2,i3)=exp( &
                          EOS%C( 1,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 2,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 3,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 4,2,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 5,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,2,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 9,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,2,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C(13,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,2,j1(i1),j2(i1))))  ))) )
        dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                          EOS%C( 2,2,j1(i1),j2(i1)) + &
                  x1t*(2.*EOS%C( 3,2,j1(i1),j2(i1)) + &
                  x1t* 3.*EOS%C( 4,2,j1(i1),j2(i1))) + &
             x2t*(        EOS%C( 6,2,j1(i1),j2(i1)) + &
                  x1t*(2.*EOS%C( 7,2,j1(i1),j2(i1)) + &
                  x1t* 3.*EOS%C( 8,2,j1(i1),j2(i1))) + &
             x2t*(        EOS%C(10,2,j1(i1),j2(i1)) + &
                  x1t*(2.*EOS%C(11,2,j1(i1),j2(i1)) + & 
                  x1t* 3.*EOS%C(12,2,j1(i1),j2(i1))) + &
             x2t*(        EOS%C(14,2,j1(i1),j2(i1))  + &
                  x1t*(2.*EOS%C(15,2,j1(i1),j2(i1)) + &
                  x1t* 3.*EOS%C(16,2,j1(i1),j2(i1)))  ))) )
        dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                          EOS%C( 5,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,2,j1(i1),j2(i1)))) + &
            2.0*x2t*(     EOS%C( 9,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,2,j1(i1),j2(i1)))) + &
            1.5*x2t*(     EOS%C(13,2,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,2,j1(i1),j2(i1))  + &
                     x1t*(EOS%C(15,2,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,2,j1(i1),j2(i1))))  )) )
      end do ! i1
    endif
    !
    if (present(T)) then
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
      do i1=1,n1
        x1t=x1ta(i1)
        x2t=x2ta(i1)
        !
        T(i1,i2,i3)=exp( &
                          EOS%C( 1,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 2,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 3,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 4,3,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 5,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,3,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C( 9,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,3,j1(i1),j2(i1)))) + &
                x2t*(     EOS%C(13,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,3,j1(i1),j2(i1))))  ))) )
        dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                          EOS%C( 5,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 6,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C( 7,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C( 8,3,j1(i1),j2(i1)))) + &
            2.0*x2t*(     EOS%C( 9,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(10,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(11,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(12,3,j1(i1),j2(i1)))) + &
            1.5*x2t*(     EOS%C(13,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(14,3,j1(i1),j2(i1)) + &
                     x1t*(EOS%C(15,3,j1(i1),j2(i1)) + &
                     x1t* EOS%C(16,3,j1(i1),j2(i1))))  )) )
      end do ! i1
    endif
!#
#   else
!#                          /* using temporary array Ct */
    if (present(s)) then
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
      do i1=1,n1
        x1t=x1ta(i1)
        x2t=x2ta(i1)
!#
#       if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                    /* temporary array is copied */
        Ct=EOS%C(:,1,j1(i1),j2(i1))
!#
#       else
!#                              /* temporary array with pointer */
        Ct=>EOS%C(:,1,j1(i1),j2(i1))
!#
#       endif
!#
        s(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      end do ! i1
    endif
    !
    if (present(P)) then
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
      do i1=1,n1
        x1t=x1ta(i1)
        x2t=x2ta(i1)
!#
#       if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                    /* temporary array is copied */
        Ct=EOS%C(:,2,j1(i1),j2(i1))
!#
#       else
!#                              /* temporary array with pointer */
        Ct=>EOS%C(:,2,j1(i1),j2(i1))
!#
#       endif
!#
        P(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
        dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
              x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
              x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
              x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
        dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
      end do ! i1
    endif
    !
    if (present(T)) then
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
      do i1=1,n1
        x1t=x1ta(i1)
        x2t=x2ta(i1)
!#
#       if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                    /* temporary array is copied */
        Ct=EOS%C(:,3,j1(i1),j2(i1))
!#
#       else
!#                              /* temporary array with pointer */
        Ct=>EOS%C(:,3,j1(i1),j2(i1))
!#
#       endif
!#
        T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
        dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
      end do ! i1
    endif
!#
#   endif
!#
  end do ! i2
end do ! i3
!
end subroutine eosinter3


!----------*************----------------------------------------------------------------------------
subroutine eosinter2_all(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter2_all ('equation_of_state_interpolation_2_all')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, s, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_all(rho, e, P, dPdrho, dPde, T, dTde, s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   All output quantities (P, dPdrho, dPde, T, dTde, s) have to be present
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine, keyword ierr removed
!   2004-03-05 (S.W.-B.) New alternative version without temporary coefficient arrays
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:),      intent(in)   :: rho, e
real, dimension(:,:),      intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
! --- Local variables ---
integer                                 :: i1, i2, n1, n2
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
n2=size(rho,2)
!
  do i2=1,n2
    do i1=1,n1
      lnx1_s=alog(rho(  i1,i2))
      lnx2_s=alog(max(e(i1,i2) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: s, P, T ---
      s(i1,i2)= &
                        EOS%C( 1,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,1,j1_s,j2_s) + &
                   x1t* EOS%C( 4,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,1,j1_s,j2_s) + &
                   x1t* EOS%C( 8,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,1,j1_s,j2_s) + &
                   x1t*(EOS%C(10,1,j1_s,j2_s) + &
                   x1t*(EOS%C(11,1,j1_s,j2_s) + &
                   x1t* EOS%C(12,1,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,1,j1_s,j2_s) + &
                   x1t*(EOS%C(14,1,j1_s,j2_s) + &
                   x1t*(EOS%C(15,1,j1_s,j2_s) + &
                   x1t* EOS%C(16,1,j1_s,j2_s)))  )))
      P(i1,i2)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1,i2)=P(i1,i2)/rho(i1,i2)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1,i2)=P(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
      T(i1,i2)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
      s(i1,i2)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2)=P(i1,i2)/rho(i1,i2)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
             x2t*(               Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
             x2t*(               Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
             x2t*(               Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2)=P(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
!
end subroutine eosinter2_all


!----------*************----------------------------------------------------------------------------
subroutine eosinter3_all(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_all ('equation_of_state_interpolation_3_all')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, s, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_all(rho, e, P, dPdrho, dPde, T, dTde, s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-16 (B.F. Lyon) Extra branch with new vectorization scheme
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
!#
# if (gasinter_l03 == 0)
!#                       /* Use s, P, T arrays ================================================== */
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=1
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*12,4)         :: Ct
real,    dimension(nblock*12)           :: poly1, x1t
real,    dimension(nblock*4)            :: poly2
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
! !$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,2
          Ct( 1+12*ib+4*k, 1:4)=EOS%C( 1: 4, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+12*ib+4*k, 1:4)=EOS%C( 5: 8, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+12*ib+4*k, 1:4)=EOS%C( 9:12, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+12*ib+4*k, 1:4)=EOS%C(13:16, 1+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+12*ib:12+12*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1:12)=x1ta(i1  )
      !x1t(13:24)=x1ta(i1+1)
      !x1t(25:36)=x1ta(i1+2)
      !x1t(37:48)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*12
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      do ib=0,nblock-1
        do ip=5,8
          poly2(ip-4+4*ib)=                   Ct(ip+12*ib, 2) + &
                           x1t(ip+12*ib)*(2.0*Ct(ip+12*ib, 3) + &
                           x1t(ip+12*ib)* 3.0*Ct(ip+12*ib, 4)  )
        end do ! ip
      end do ! ib
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t                =x2ta(i1+ib)
        !
        s(     i1+ib,i2,i3)=     poly1( 1+12*ib) + &
                            x2t*(poly1( 2+12*ib) + &
                            x2t*(poly1( 3+12*ib) + &
                            x2t*(poly1( 4+12*ib)  )))
        !
        P(     i1+ib,i2,i3)=     poly1( 5+12*ib) + &
                            x2t*(poly1( 6+12*ib) + &
                            x2t*(poly1( 7+12*ib) + &
                            x2t*(poly1( 8+12*ib)  )))
        dPde(  i1+ib,i2,i3)=     poly1( 6+12*ib) + &
                        2.0*x2t*(poly1( 7+12*ib) + &
                        1.5*x2t*(poly1( 8+12*ib)  ))
        !
        T(     i1+ib,i2,i3)=     poly1( 9+12*ib) + &
                            x2t*(poly1(10+12*ib) + &
                            x2t*(poly1(11+12*ib) + &
                            x2t*(poly1(12+12*ib)  )))
        dTde(  i1+ib,i2,i3)=     poly1(10+12*ib) + &
                        2.0*x2t*(poly1(11+12*ib) + &
                        1.5*x2t*(poly1(12+12*ib)  ))
        !
        dPdrho(i1+ib,i2,i3)=     poly2( 1+ 4*ib) + &
                            x2t*(poly2( 2+ 4*ib) + &
                            x2t*(poly2( 3+ 4*ib) + &
                            x2t*(poly2( 4+ 4*ib)  )))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,2
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 1+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 1+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 1+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 1+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,12
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      do ip=5,8
        poly2(ip-4)=Ct(ip, 2) + x1ta(i1)*(2.0*Ct(ip, 3) + x1ta(i1)*3.0*Ct(ip, 4))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t           =x2ta(i1)
      !
      s(     i1,i2,i3)=        poly1( 1) + &
                          x2t*(poly1( 2) + &
                          x2t*(poly1( 3) + &
                          x2t*(poly1( 4)  )))
      !
      P(     i1,i2,i3)=        poly1( 5) + &
                          x2t*(poly1( 6) + &
                          x2t*(poly1( 7) + &
                          x2t*(poly1( 8)  )))
      dPde(  i1,i2,i3)=        poly1( 6) + &
                      2.0*x2t*(poly1( 7) + &
                      1.5*x2t*(poly1( 8)  ))
      !
      T(     i1,i2,i3)=        poly1( 9) + &
                          x2t*(poly1(10) + &
                          x2t*(poly1(11) + &
                          x2t*(poly1(12)  )))
      dTde(  i1,i2,i3)=        poly1(10) + &
                      2.0*x2t*(poly1(11) + &
                      1.5*x2t*(poly1(12)  ))
      !
      dPdrho(i1,i2,i3)=        poly2( 1) + &
                          x2t*(poly2( 2) + &
                          x2t*(poly2( 3) + &
                          x2t*(poly2( 4)  )))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      P(     i1,i2,i3)=exp(P(i1,i2,i3))
      dPde(  i1,i2,i3)=P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dPde(  i1,i2,i3)
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/(rho(i1,i2,i3)             )* dPdrho(i1,i2,i3)
      T(     i1,i2,i3)=exp(T(i1,i2,i3))
      dTde(  i1,i2,i3)=T(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dTde(  i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
! !$OMP END DO
!#
# else
!#     /* gasinter_l01: standard version -------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0)
!#                       /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
# if (gasinter_l01 == 3)
!#                       /* temporary array is copied */
real, dimension(size(rho,1),16,3)       :: Ctv
real, dimension(size(rho,1))            :: x1tv, x2tv
!#
# endif 
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
!#
# if (gasinter_l01 == 3)
!#                       /* temporary arrays are copied into one large array */
do i3=m3,n3
  do i2=m2,n2
    !
    ! --- First inner loop: collect coefficients ---
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
      ! --- Put some values into arrays ---
      x1tv(i1) =x1t
      x2tv(i1) =x2t
      Ctv( i1,:,:)=EOS%C(:,:,j1_s,j2_s)
    end do ! i1
    !
    ! --- Second inner loop: interpolate ---
    do i1=m1,n1
      x1t=x1tv(i1)
      x2t=x2tv(i1)
      !
      ! --- Do bicubic interpolation for desired rho/e pair: s, P, T ---
      s(i1,i2,i3)= &
                 Ctv(i1, 1,1) + x1t*(Ctv(i1, 2,1) + x1t*(Ctv(i1, 3,1) + x1t*Ctv(i1, 4,1))) + &
            x2t*(Ctv(i1, 5,1) + x1t*(Ctv(i1, 6,1) + x1t*(Ctv(i1, 7,1) + x1t*Ctv(i1, 8,1))) + &
            x2t*(Ctv(i1, 9,1) + x1t*(Ctv(i1,10,1) + x1t*(Ctv(i1,11,1) + x1t*Ctv(i1,12,1))) + &
            x2t*(Ctv(i1,13,1) + x1t*(Ctv(i1,14,1) + x1t*(Ctv(i1,15,1) + x1t*Ctv(i1,16,1)))  )))
      P(i1,i2,i3)=exp( &
                 Ctv(i1, 1,2) + x1t*(Ctv(i1, 2,2) + x1t*(Ctv(i1, 3,2) + x1t*Ctv(i1, 4,2))) + &
            x2t*(Ctv(i1, 5,2) + x1t*(Ctv(i1, 6,2) + x1t*(Ctv(i1, 7,2) + x1t*Ctv(i1, 8,2))) + &
            x2t*(Ctv(i1, 9,2) + x1t*(Ctv(i1,10,2) + x1t*(Ctv(i1,11,2) + x1t*Ctv(i1,12,2))) + &
            x2t*(Ctv(i1,13,2) + x1t*(Ctv(i1,14,2) + x1t*(Ctv(i1,15,2) + x1t*Ctv(i1,16,2)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                 Ctv(i1, 2,2) + x1t*(2.*Ctv(i1, 3,2) + x1t*3.*Ctv(i1, 4,2)) + &
            x2t*(Ctv(i1, 6,2) + x1t*(2.*Ctv(i1, 7,2) + x1t*3.*Ctv(i1, 8,2)) + &
            x2t*(Ctv(i1,10,2) + x1t*(2.*Ctv(i1,11,2) + x1t*3.*Ctv(i1,12,2)) + &
            x2t*(Ctv(i1,14,2) + x1t*(2.*Ctv(i1,15,2) + x1t*3.*Ctv(i1,16,2))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                 Ctv(i1, 5,2) + x1t*(Ctv(i1, 6,2) + x1t*(Ctv(i1, 7,2) + x1t*Ctv(i1, 8,2))) + &
        2.0*x2t*(Ctv(i1, 9,2) + x1t*(Ctv(i1,10,2) + x1t*(Ctv(i1,11,2) + x1t*Ctv(i1,12,2))) + &
        1.5*x2t*(Ctv(i1,13,2) + x1t*(Ctv(i1,14,2) + x1t*(Ctv(i1,15,2) + x1t*Ctv(i1,16,2))) )))
      T(i1,i2,i3)=exp( &
                 Ctv(i1, 1,3) + x1t*(Ctv(i1, 2,3) + x1t*(Ctv(i1, 3,3) + x1t*Ctv(i1, 4,3))) + &
            x2t*(Ctv(i1, 5,3) + x1t*(Ctv(i1, 6,3) + x1t*(Ctv(i1, 7,3) + x1t*Ctv(i1, 8,3))) + &
            x2t*(Ctv(i1, 9,3) + x1t*(Ctv(i1,10,3) + x1t*(Ctv(i1,11,3) + x1t*Ctv(i1,12,3))) + &
            x2t*(Ctv(i1,13,3) + x1t*(Ctv(i1,14,3) + x1t*(Ctv(i1,15,3) + x1t*Ctv(i1,16,3)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                 Ctv(i1, 5,3) + x1t*(Ctv(i1, 6,3) + x1t*(Ctv(i1, 7,3) + x1t*Ctv(i1, 8,3))) + &
        2.0*x2t*(Ctv(i1, 9,3) + x1t*(Ctv(i1,10,3) + x1t*(Ctv(i1,11,3) + x1t*Ctv(i1,12,3))) + &
        1.5*x2t*(Ctv(i1,13,3) + x1t*(Ctv(i1,14,3) + x1t*(Ctv(i1,15,3) + x1t*Ctv(i1,16,3))) )))
    end do ! i1
  end do ! i2
end do ! i3
!#
# else
!#     /* gasinter_l01: get coefficient set for each rho/e pair separately */
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: s, P, T ---
      s(i1,i2,i3)= &
                        EOS%C( 1,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,1,j1_s,j2_s) + &
                   x1t* EOS%C( 4,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,1,j1_s,j2_s) + &
                   x1t* EOS%C( 8,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,1,j1_s,j2_s) + &
                   x1t*(EOS%C(10,1,j1_s,j2_s) + &
                   x1t*(EOS%C(11,1,j1_s,j2_s) + &
                   x1t* EOS%C(12,1,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,1,j1_s,j2_s) + &
                   x1t*(EOS%C(14,1,j1_s,j2_s) + &
                   x1t*(EOS%C(15,1,j1_s,j2_s) + &
                   x1t* EOS%C(16,1,j1_s,j2_s)))  )))
      P(i1,i2,i3)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* gasinter_l01: use temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
!#
#     if (gasinter_l01 == 0)
!#                           /* temporary array is copied */
      Ct=EOS%C(:,1,j1_s,j2_s)
!#
#     else
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
      s(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0)
!#                           /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     else
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
             x2t*(               Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
             x2t*(               Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
             x2t*(               Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0)
!#                           /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     else
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!#
# endif
!#      /* gasinter_l01 */
# endif
!#      /* gasinter_l01: loop-split/standard version -------------------------------------------- */
# else
!#     /* gasinter_l03 ========================================================================== */
! --- Local variables ---
integer, parameter                      :: nblock=1
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta, &
                                           dsdlnrho, dsdlne, &
                                           dsdlnrhodlnrho, ddsdlnedlne, ddsdlnrhodlne
                                           
real                                    :: x2t, &
                                           dlnPdlnrho, dlnPdlne, dlnTdlnrho, dlnTdlne
real,    dimension(nblock*4,4)          :: Ct
real,    dimension(nblock*4)            :: poly1, x1t
real,    dimension(nblock*4)            :: poly2, poly3
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
! !$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        k=0
          Ct( 1+4*ib+4*k, 1:4)=EOS%C( 1: 4, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+4*ib+4*k, 1:4)=EOS%C( 5: 8, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+4*ib+4*k, 1:4)=EOS%C( 9:12, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+4*ib+4*k, 1:4)=EOS%C(13:16, 1+k, j1a(i1+ib), j2a(i1+ib))
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+4*ib:4+4*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 4)=x1ta(i1  )
      !x1t( 5: 8)=x1ta(i1+1)
      !x1t( 9:12)=x1ta(i1+2)
      !x1t(13:16)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*4
        poly1(ip)=             Ct(ip, 1) + &
                  x1t(ip)*(    Ct(ip, 2) + &
                  x1t(ip)*(    Ct(ip, 3) + &
                  x1t(ip)*     Ct(ip, 4)  ))
        poly2(ip)=             Ct(ip, 2) + &
                  x1t(ip)*(2.0*Ct(ip, 3) + &
                  x1t(ip)* 3.0*Ct(ip, 4)  )
        poly3(ip)=         2.0*Ct(ip, 3) + &
                  x1t(ip)* 6.0*Ct(ip, 4)
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t                   =         x2ta(i1+ib)
        !
        s(        i1+ib,i2,i3)=      poly1( 1+4*ib) + &
                            x2t*(    poly1( 2+4*ib) + &
                            x2t*(    poly1( 3+4*ib) + &
                            x2t*(    poly1( 4+4*ib)  )))
        dsdlnrho(       i1+ib)=      poly2( 1+4*ib) + &
                            x2t*(    poly2( 2+4*ib) + &
                            x2t*(    poly2( 3+4*ib) + &
                            x2t*(    poly2( 4+4*ib)  )))
        ddsdlnrhodlnrho(i1+ib)=&
                                     poly3( 1+4*ib) + &
                            x2t*(    poly3( 2+4*ib) + &
                            x2t*(    poly3( 3+4*ib) + &
                            x2t*(    poly3( 4+4*ib)  )))
        !
        dsdlne(         i1+ib)=      poly1( 2+4*ib) + &
                            x2t*(2.0*poly1( 3+4*ib) + &
                            x2t*(3.0*poly1( 4+4*ib)  ))
        ddsdlnrholne(   i1+ib)=      poly2( 2+4*ib) + &
                            x2t*(2.0*poly2( 3+4*ib) + &
                            x2t*(3.0*poly2( 4+4*ib)  ))
        !
        ddsdlnelne(     i1+ib)=  2.0*poly1( 3+4*ib) + &
                            x2t*(6.0*poly1( 4+4*ib)  )
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      k=0
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 1+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 1+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 1+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 1+k, j1a(i1), j2a(i1))
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,4
        poly1(ip)=             Ct(ip, 1) + &
                  x1t(ip)*(    Ct(ip, 2) + &
                  x1t(ip)*(    Ct(ip, 3) + &
                  x1t(ip)*     Ct(ip, 4)  ))
        poly2(ip)=             Ct(ip, 2) + &
                  x1t(ip)*(2.0*Ct(ip, 3) + &
                  x1t(ip)* 3.0*Ct(ip, 4)  )
        poly3(ip)=         2.0*Ct(ip, 3) + &
                  x1t(ip)* 6.0*Ct(ip, 4)
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t           =x2ta(i1)
      !
      ib=0
        x2t                =x2ta(i1+ib)
        !
        s(     i1+ib,i2,i3)=         poly1( 1+4*ib) + &
                            x2t*(    poly1( 2+4*ib) + &
                            x2t*(    poly1( 3+4*ib) + &
                            x2t*(    poly1( 4+4*ib)  )))
        dsdlnrho(       i1+ib)=      poly2( 1+4*ib) + &
                            x2t*(    poly2( 2+4*ib) + &
                            x2t*(    poly2( 3+4*ib) + &
                            x2t*(    poly2( 4+4*ib)  )))
        ddsdlnrhodlnrho(i1+ib)=&
                                     poly3( 1+4*ib) + &
                            x2t*(    poly3( 2+4*ib) + &
                            x2t*(    poly3( 3+4*ib) + &
                            x2t*(    poly3( 4+4*ib)  )))
        !
        dsdlne(         i1+ib)=      poly1( 2+4*ib) + &
                            x2t*(2.0*poly1( 3+4*ib) + &
                            x2t*(3.0*poly1( 4+4*ib)  ))
        ddsdlnrholne(   i1+ib)=      poly2( 2+4*ib) + &
                            x2t*(2.0*poly2( 3+4*ib) + &
                            x2t*(3.0*poly2( 4+4*ib)  ))
        !
        ddsdlnelne(     i1+ib)=  2.0*poly1( 3+4*ib) + &
                            x2t*(6.0*poly1( 4+4*ib)  )
    end do ! i1
    !
    ! --- Perform final scaling ---
    do i1=m1,n1
      dlnTdlne  =1.0-ddsdlnedlne(i1)/dsdlne(i1)
      dlnTdlnrho=-ddsdlnrhodlne(i1)/dsdlne(i1)
      dlnPdlne  =dlnTdlne         + ddsdlnrhodlne(i1)/dsdlnrho(i1)
      dlnPdlnrho=dlnTdlnrho + 1.0 + ddsdlnrhodlnrho(i1)/dsdlnrho(i1)
      !
      T(     i1,i2,i3)=(e(i1,i2,i3)+x2_shift)/dsdlne(i1)
      P(     i1,i2,i3)=-T(i1,i2,i3)*rho(i1,i2,i3)*dsdlnrho(i1)
      !
      dPde(  i1,i2,i3)=P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dlnPdlne
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/(rho(i1,i2,i3)             )* dlnPdlnrho
      dTde(  i1,i2,i3)=T(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dlnTdlne
      !
    end do ! i1
  end do ! i2
end do ! i3
! !$OMP END DO
!#
# endif
!#      /* gasinter_l03 ========================================================================= */
!
end subroutine eosinter3_all


!----------****************-------------------------------------------------------------------------
subroutine eosinter3_allOMP(rho, e, P, dPdrho, dPde, T, dTde, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_allOMP ('equation_of_state_interpolation_3_all_OpenMP')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, s, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_allOMP(rho, e, P, dPdrho, dPde, T, dTde, s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-16 (B.F. Lyon) Extra branch with new vectorization scheme
!   2013-02-17 (B.F.) OpenMP version
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: P, dPdrho, dPde, T, dTde, s
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=1
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*12,4)         :: Ct
real,    dimension(nblock*12)           :: poly1, x1t
real,    dimension(nblock*4)            :: poly2
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,2
          Ct( 1+12*ib+4*k, 1:4)=EOS%C( 1: 4, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+12*ib+4*k, 1:4)=EOS%C( 5: 8, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+12*ib+4*k, 1:4)=EOS%C( 9:12, 1+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+12*ib+4*k, 1:4)=EOS%C(13:16, 1+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+12*ib:12+12*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1:12)=x1ta(i1  )
      !x1t(13:24)=x1ta(i1+1)
      !x1t(25:36)=x1ta(i1+2)
      !x1t(37:48)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*12
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      do ib=0,nblock-1
        do ip=5,8
          poly2(ip-4+4*ib)=                   Ct(ip+12*ib, 2) + &
                           x1t(ip+12*ib)*(2.0*Ct(ip+12*ib, 3) + &
                           x1t(ip+12*ib)* 3.0*Ct(ip+12*ib, 4)  )
        end do ! ip
      end do ! ib
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t                =x2ta(i1+ib)
        !
        s(     i1+ib,i2,i3)=     poly1( 1+12*ib) + &
                            x2t*(poly1( 2+12*ib) + &
                            x2t*(poly1( 3+12*ib) + &
                            x2t*(poly1( 4+12*ib)  )))
        !
        P(     i1+ib,i2,i3)=     poly1( 5+12*ib) + &
                            x2t*(poly1( 6+12*ib) + &
                            x2t*(poly1( 7+12*ib) + &
                            x2t*(poly1( 8+12*ib)  )))
        dPde(  i1+ib,i2,i3)=     poly1( 6+12*ib) + &
                        2.0*x2t*(poly1( 7+12*ib) + &
                        1.5*x2t*(poly1( 8+12*ib)  ))
        !
        T(     i1+ib,i2,i3)=     poly1( 9+12*ib) + &
                            x2t*(poly1(10+12*ib) + &
                            x2t*(poly1(11+12*ib) + &
                            x2t*(poly1(12+12*ib)  )))
        dTde(  i1+ib,i2,i3)=     poly1(10+12*ib) + &
                        2.0*x2t*(poly1(11+12*ib) + &
                        1.5*x2t*(poly1(12+12*ib)  ))
        !
        dPdrho(i1+ib,i2,i3)=     poly2( 1+ 4*ib) + &
                            x2t*(poly2( 2+ 4*ib) + &
                            x2t*(poly2( 3+ 4*ib) + &
                            x2t*(poly2( 4+ 4*ib)  )))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,2
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 1+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 1+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 1+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 1+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,12
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      do ip=5,8
        poly2(ip-4)=Ct(ip, 2) + x1ta(i1)*(2.0*Ct(ip, 3) + x1ta(i1)*3.0*Ct(ip, 4))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t           =x2ta(i1)
      !
      s(     i1,i2,i3)=        poly1( 1) + &
                          x2t*(poly1( 2) + &
                          x2t*(poly1( 3) + &
                          x2t*(poly1( 4)  )))
      !
      P(     i1,i2,i3)=        poly1( 5) + &
                          x2t*(poly1( 6) + &
                          x2t*(poly1( 7) + &
                          x2t*(poly1( 8)  )))
      dPde(  i1,i2,i3)=        poly1( 6) + &
                      2.0*x2t*(poly1( 7) + &
                      1.5*x2t*(poly1( 8)  ))
      !
      T(     i1,i2,i3)=        poly1( 9) + &
                          x2t*(poly1(10) + &
                          x2t*(poly1(11) + &
                          x2t*(poly1(12)  )))
      dTde(  i1,i2,i3)=        poly1(10) + &
                      2.0*x2t*(poly1(11) + &
                      1.5*x2t*(poly1(12)  ))
      !
      dPdrho(i1,i2,i3)=        poly2( 1) + &
                          x2t*(poly2( 2) + &
                          x2t*(poly2( 3) + &
                          x2t*(poly2( 4)  )))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      P(     i1,i2,i3)=exp(P(i1,i2,i3))
      dPde(  i1,i2,i3)=P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dPde(  i1,i2,i3)
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/(rho(i1,i2,i3)             )* dPdrho(i1,i2,i3)
      T(     i1,i2,i3)=exp(T(i1,i2,i3))
      dTde(  i1,i2,i3)=T(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dTde(  i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0)
!#                       /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
# if (gasinter_l01 == 3)
!#                       /* temporary array is copied */
real, dimension(size(rho,1),16,3)       :: Ctv
real, dimension(size(rho,1))            :: x1tv, x2tv
!#
# endif 
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
!#
# if (gasinter_l01 == 3)
!#                       /* temporary arrays are copied into one large array */
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    !
    ! --- First inner loop: collect coefficients ---
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
      ! --- Put some values into arrays ---
      x1tv(i1) =x1t
      x2tv(i1) =x2t
      Ctv( i1,:,:)=EOS%C(:,:,j1_s,j2_s)
    end do ! i1
    !
    ! --- Second inner loop: interpolate ---
    do i1=m1,n1
      x1t=x1tv(i1)
      x2t=x2tv(i1)
      !
      ! --- Do bicubic interpolation for desired rho/e pair: s, P, T ---
      s(i1,i2,i3)= &
                 Ctv(i1, 1,1) + x1t*(Ctv(i1, 2,1) + x1t*(Ctv(i1, 3,1) + x1t*Ctv(i1, 4,1))) + &
            x2t*(Ctv(i1, 5,1) + x1t*(Ctv(i1, 6,1) + x1t*(Ctv(i1, 7,1) + x1t*Ctv(i1, 8,1))) + &
            x2t*(Ctv(i1, 9,1) + x1t*(Ctv(i1,10,1) + x1t*(Ctv(i1,11,1) + x1t*Ctv(i1,12,1))) + &
            x2t*(Ctv(i1,13,1) + x1t*(Ctv(i1,14,1) + x1t*(Ctv(i1,15,1) + x1t*Ctv(i1,16,1)))  )))
      P(i1,i2,i3)=exp( &
                 Ctv(i1, 1,2) + x1t*(Ctv(i1, 2,2) + x1t*(Ctv(i1, 3,2) + x1t*Ctv(i1, 4,2))) + &
            x2t*(Ctv(i1, 5,2) + x1t*(Ctv(i1, 6,2) + x1t*(Ctv(i1, 7,2) + x1t*Ctv(i1, 8,2))) + &
            x2t*(Ctv(i1, 9,2) + x1t*(Ctv(i1,10,2) + x1t*(Ctv(i1,11,2) + x1t*Ctv(i1,12,2))) + &
            x2t*(Ctv(i1,13,2) + x1t*(Ctv(i1,14,2) + x1t*(Ctv(i1,15,2) + x1t*Ctv(i1,16,2)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                 Ctv(i1, 2,2) + x1t*(2.*Ctv(i1, 3,2) + x1t*3.*Ctv(i1, 4,2)) + &
            x2t*(Ctv(i1, 6,2) + x1t*(2.*Ctv(i1, 7,2) + x1t*3.*Ctv(i1, 8,2)) + &
            x2t*(Ctv(i1,10,2) + x1t*(2.*Ctv(i1,11,2) + x1t*3.*Ctv(i1,12,2)) + &
            x2t*(Ctv(i1,14,2) + x1t*(2.*Ctv(i1,15,2) + x1t*3.*Ctv(i1,16,2))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                 Ctv(i1, 5,2) + x1t*(Ctv(i1, 6,2) + x1t*(Ctv(i1, 7,2) + x1t*Ctv(i1, 8,2))) + &
        2.0*x2t*(Ctv(i1, 9,2) + x1t*(Ctv(i1,10,2) + x1t*(Ctv(i1,11,2) + x1t*Ctv(i1,12,2))) + &
        1.5*x2t*(Ctv(i1,13,2) + x1t*(Ctv(i1,14,2) + x1t*(Ctv(i1,15,2) + x1t*Ctv(i1,16,2))) )))
      T(i1,i2,i3)=exp( &
                 Ctv(i1, 1,3) + x1t*(Ctv(i1, 2,3) + x1t*(Ctv(i1, 3,3) + x1t*Ctv(i1, 4,3))) + &
            x2t*(Ctv(i1, 5,3) + x1t*(Ctv(i1, 6,3) + x1t*(Ctv(i1, 7,3) + x1t*Ctv(i1, 8,3))) + &
            x2t*(Ctv(i1, 9,3) + x1t*(Ctv(i1,10,3) + x1t*(Ctv(i1,11,3) + x1t*Ctv(i1,12,3))) + &
            x2t*(Ctv(i1,13,3) + x1t*(Ctv(i1,14,3) + x1t*(Ctv(i1,15,3) + x1t*Ctv(i1,16,3)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                 Ctv(i1, 5,3) + x1t*(Ctv(i1, 6,3) + x1t*(Ctv(i1, 7,3) + x1t*Ctv(i1, 8,3))) + &
        2.0*x2t*(Ctv(i1, 9,3) + x1t*(Ctv(i1,10,3) + x1t*(Ctv(i1,11,3) + x1t*Ctv(i1,12,3))) + &
        1.5*x2t*(Ctv(i1,13,3) + x1t*(Ctv(i1,14,3) + x1t*(Ctv(i1,15,3) + x1t*Ctv(i1,16,3))) )))
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# else
!#     /* Get coefficient set for each rho/e pair separately */
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: s, P, T ---
      s(i1,i2,i3)= &
                        EOS%C( 1,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,1,j1_s,j2_s) + &
                   x1t* EOS%C( 4,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,1,j1_s,j2_s) + &
                   x1t* EOS%C( 8,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,1,j1_s,j2_s) + &
                   x1t*(EOS%C(10,1,j1_s,j2_s) + &
                   x1t*(EOS%C(11,1,j1_s,j2_s) + &
                   x1t* EOS%C(12,1,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,1,j1_s,j2_s) + &
                   x1t*(EOS%C(14,1,j1_s,j2_s) + &
                   x1t*(EOS%C(15,1,j1_s,j2_s) + &
                   x1t* EOS%C(16,1,j1_s,j2_s)))  )))
      P(i1,i2,i3)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
!#
#     if (gasinter_l01 == 0)
!#                           /* temporary array is copied */
      Ct=EOS%C(:,1,j1_s,j2_s)
!#
#     else
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
      s(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0)
!#                           /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     else
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
             x2t*(               Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
             x2t*(               Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
             x2t*(               Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0)
!#                           /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     else
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# endif 
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
!
end subroutine eosinter3_allOMP


!----------************-----------------------------------------------------------------------------
subroutine eosinter2_Ts(rho, e, T, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter2_Ts ('equation_of_state_interpolation_2_Temperature_s')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> T, s
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_Ts(rho, e, T, s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   T:            ('Temperature') temperature [K]
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Own routine
!   2004-03-08 (S.W.-B.) New alternative version without temporary coefficient arrays
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:),      intent(in)   :: rho, e
real, dimension(:,:),      intent(out)  :: T, s
!
! --- Local variables ---
integer                                 :: i1, i2, n1, n2
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
n2=size(rho,2)
!
  do i2=1,n2
    do i1=1,n1
      lnx1_s=alog(rho(  i1,i2))
      lnx2_s=alog(max(e(i1,i2) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: s, T ---
      s(i1,i2)= &
                        EOS%C( 1,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,1,j1_s,j2_s) + &
                   x1t* EOS%C( 4,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,1,j1_s,j2_s) + &
                   x1t* EOS%C( 8,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,1,j1_s,j2_s) + &
                   x1t*(EOS%C(10,1,j1_s,j2_s) + &
                   x1t*(EOS%C(11,1,j1_s,j2_s) + &
                   x1t* EOS%C(12,1,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,1,j1_s,j2_s) + &
                   x1t*(EOS%C(14,1,j1_s,j2_s) + &
                   x1t*(EOS%C(15,1,j1_s,j2_s) + &
                   x1t* EOS%C(16,1,j1_s,j2_s)))  )))
      T(i1,i2)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
      s(i1,i2)= &
               Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
         x2t*( Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
         x2t*( Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
         x2t*( Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2)=exp( &
               Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
         x2t*( Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
         x2t*( Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
         x2t*( Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
!
end subroutine eosinter2_Ts


!----------************-----------------------------------------------------------------------------
subroutine eosinter3_Ts(rho, e, T, s)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_Ts ('equation_of_state_interpolation_3_Temperature_s')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> T, s
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_Ts(rho, e, T, s)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   T:            ('Temperature') temperature [K]
!   s:            ('s') entropy per mass unit [erg/g/K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Own routine
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: T, s
!
! --- Local variables ---
integer                                 :: i1, i2, i3, n1, n2, n3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
do i3=1,n3
  do i2=1,n2
    do i1=1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: s, T ---
      s(i1,i2,i3)= &
                        EOS%C( 1,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,1,j1_s,j2_s) + &
                   x1t* EOS%C( 4,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,1,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,1,j1_s,j2_s) + &
                   x1t* EOS%C( 8,1,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,1,j1_s,j2_s) + &
                   x1t*(EOS%C(10,1,j1_s,j2_s) + &
                   x1t*(EOS%C(11,1,j1_s,j2_s) + &
                   x1t* EOS%C(12,1,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,1,j1_s,j2_s) + &
                   x1t*(EOS%C(14,1,j1_s,j2_s) + &
                   x1t*(EOS%C(15,1,j1_s,j2_s) + &
                   x1t* EOS%C(16,1,j1_s,j2_s)))  )))
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: s ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,1,j1_s,j2_s)
!#
#     endif
!#
      s(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine eosinter3_Ts


!----------************-----------------------------------------------------------------------------
subroutine eosinter1_PT(rho, e, P, T)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter1_PT ('equation_of_state_interpolation_1_Pressure_Temperature')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_PT(rho, e, P, T)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   T:            ('Temperature') temperature [K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2004-03-08 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2007-12-03 (B.F. Lyon) Version with P,T output only
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),        intent(in)   :: rho, e
real, dimension(:),        intent(out)  :: P, T
!
! --- Local variables ---
integer                                 :: i1, n1
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
!
    do i1=1,n1
      lnx1_s=alog(rho(  i1))
      lnx2_s=alog(max(e(i1) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: P, T ---
      P(i1)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      T(i1)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
!#
#     endif
!#
end do ! i1
!
end subroutine eosinter1_PT


!----------****************-------------------------------------------------------------------------
subroutine eosinter2_PdPTdT(rho, e, P, dPdrho, dPde, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter2_PdPTdT ('eos_interpolation_2_Pressure_delta_Pressure_Temperature_delta_Temperature')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_PdPTdT(rho, e, P, dPdrho, dPde, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2011-12-08 (B.F.) Derived from eosinter2_PTdT and eosinter3_PdP
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:),      intent(in)   :: rho, e
real, dimension(:,:),      intent(out)  :: P, dPdrho, dPde, T, dTde
!
! --- Local variables ---
integer                                 :: i1, i2, n1, n2
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
n2=size(rho,2)
!
  do i2=1,n2
    do i1=1,n1
      lnx1_s=alog(rho(  i1,i2))
      lnx2_s=alog(max(e(i1,i2) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: P, T ---
      P(i1,i2)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1,i2)=P(i1,i2)/rho(i1,i2)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1,i2)=P(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
      T(i1,i2)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2)=P(i1,i2)/rho(i1,i2)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
              x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
              x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
              x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2)=P(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
!
end subroutine eosinter2_PdPTdT


!----------****************-------------------------------------------------------------------------
subroutine eosinter3_PdPTdT(rho, e, P, dPdrho, dPde, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_PdPTdT ('eos_interpolation_3_Pressure_delta_Pressure_Temperature_delta_Temperature')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, T, ...
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_PdPTdT(rho, e, P, dPdrho, dPde, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2004-04-13 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2011-12-05 (B.F.) Derived from eosinter3_PTdT and eosinter3_PdP
!   2013-02-18 (B.F. Lyon) Extra branch with new vectorization scheme
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: P, dPdrho, dPde, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
real,    dimension(nblock*4)            :: poly2
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
!
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:36)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      do ib=0,nblock-1
        do ip=1,4
          poly2(ip+4*ib)=                    Ct(ip+8*ib, 2) + &
                           x1t(ip+8*ib)*(2.0*Ct(ip+8*ib, 3) + &
                           x1t(ip+8*ib)* 3.0*Ct(ip+8*ib, 4)  )
        end do ! ip
      end do ! ib
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t                =x2ta(i1+ib)
        !
        P(     i1+ib,i2,i3)=     poly1( 1+8*ib) + &
                            x2t*(poly1( 2+8*ib) + &
                            x2t*(poly1( 3+8*ib) + &
                            x2t*(poly1( 4+8*ib)  )))
        dPde(  i1+ib,i2,i3)=     poly1( 2+8*ib) + &
                        2.0*x2t*(poly1( 3+8*ib) + &
                        1.5*x2t*(poly1( 4+8*ib)  ))
        !
        T(     i1+ib,i2,i3)=     poly1( 5+8*ib) + &
                            x2t*(poly1( 6+8*ib) + &
                            x2t*(poly1( 7+8*ib) + &
                            x2t*(poly1( 8+8*ib)  )))
        dTde(  i1+ib,i2,i3)=     poly1( 6+8*ib) + &
                        2.0*x2t*(poly1( 7+8*ib) + &
                        1.5*x2t*(poly1( 8+8*ib)  ))
        !
        dPdrho(i1+ib,i2,i3)=     poly2( 1+ 4*ib) + &
                            x2t*(poly2( 2+ 4*ib) + &
                            x2t*(poly2( 3+ 4*ib) + &
                            x2t*(poly2( 4+ 4*ib)  )))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      do ip=1,4
        poly2(ip)=Ct(ip, 2) + x1ta(i1)*(2.0*Ct(ip, 3) + x1ta(i1)*3.0*Ct(ip, 4))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t           =x2ta(i1)
      !
      P(     i1,i2,i3)=        poly1( 1) + &
                          x2t*(poly1( 2) + &
                          x2t*(poly1( 3) + &
                          x2t*(poly1( 4)  )))
      dPde(  i1,i2,i3)=        poly1( 2) + &
                      2.0*x2t*(poly1( 3) + &
                      1.5*x2t*(poly1( 4)  ))
      !
      T(     i1,i2,i3)=        poly1( 5) + &
                          x2t*(poly1( 6) + &
                          x2t*(poly1( 7) + &
                          x2t*(poly1( 8)  )))
      dTde(  i1,i2,i3)=        poly1( 6) + &
                      2.0*x2t*(poly1( 7) + &
                      1.5*x2t*(poly1( 8)  ))
      !
      dPdrho(i1,i2,i3)=        poly2( 1) + &
                          x2t*(poly2( 2) + &
                          x2t*(poly2( 3) + &
                          x2t*(poly2( 4)  )))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      P(     i1,i2,i3)=exp(P(i1,i2,i3))
      dPde(  i1,i2,i3)=P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dPde(  i1,i2,i3)
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/(rho(i1,i2,i3)             )* dPdrho(i1,i2,i3)
      T(     i1,i2,i3)=exp(T(i1,i2,i3))
      dTde(  i1,i2,i3)=T(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dTde(  i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: P, T ---
      P(i1,i2,i3)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
              x2t*(              Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
              x2t*(              Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
              x2t*(              Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     else
!#                            /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
!
end subroutine eosinter3_PdPTdT


!----------**************---------------------------------------------------------------------------
subroutine eosinter3_TOMP(rho, e, T, m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_TOMP ('equation_of_state_interpolation_3_Temperature_OpenMP')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> T
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_TOMP(rho, e, T, m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells of rho, e
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells of rho, e
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells of rho, e
!
! OUTPUT:
!   T:            ('Temperature') temperature [K]
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-11-08 (B.F.) Special routine for T only (out of eosinter3_TdT)
!   2004-04-14 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-17 (B.F. Lyon) Rename routine name: append OMP
!   2017-09-17 (B.F Uppsala) Add parameters for number of ghost cells
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3
real, dimension(m1-nghost1:n1+nghost1, &
                m2-nghost2:n2+nghost2, &
                m3-nghost3:n3+nghost3   ), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: T
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
!#
#     else
!#                            /* using temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
end subroutine eosinter3_TOMP


!----------*************----------------------------------------------------------------------------
subroutine eosinter1_TdT(rho, e, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter1_TdT ('eos_interpolation_1_Temperature_delta_Temp')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdT(rho, e, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version without input indices. Extra branch with new vectorization scheme
!   2013-02-18 (B.F.) 1D version
!   2018-11-24 (B.F. Uppsala) Derived from eosinter1_logPTdT
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),        intent(in)   :: rho, e
real, dimension(:),        intent(out)  :: T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=4
integer                                 :: m1,n1
integer                                 :: i1, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*4,4)          :: Ct
real,    dimension(nblock*4)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
m1=1
n1=size(rho,1)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        k=0
          Ct( 1+4*ib+4*k, 1:4)=EOS%C( 1: 4, 3+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+4*ib+4*k, 1:4)=EOS%C( 5: 8, 3+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+4*ib+4*k, 1:4)=EOS%C( 9:12, 3+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+4*ib+4*k, 1:4)=EOS%C(13:16, 3+k, j1a(i1+ib), j2a(i1+ib))
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+4*ib:4+4*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 4)=x1ta(i1  )
      x1t( 5: 8)=x1ta(i1+1)
      x1t( 9:12)=x1ta(i1+2)
      x1t(13:16)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*4
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t        =x2ta(i1+ib)
        !
        T(   i1+ib)=             poly1(1+4*ib) + &
                            x2t*(poly1(2+4*ib) + &
                            x2t*(poly1(3+4*ib) + &
                            x2t*(poly1(4+4*ib)  )))
        dTde(i1+ib)=             poly1(2+4*ib) + &
                        2.0*x2t*(poly1(3+4*ib) + &
                        1.5*x2t*(poly1(4+4*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      k=0
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 3+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 3+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 3+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 3+k, j1a(i1), j2a(i1))
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,4
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t     =x2ta(i1)
      !
      T(   i1)=             poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      dTde(i1)=             poly1(2) + &
                   2.0*x2t*(poly1(3) + &
                   1.5*x2t*(poly1(4)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      T(   i1)=exp(T(i1))
      dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* dTde(i1)
    end do ! i1
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: m1,n1
integer                                 :: i1
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
n1=size(rho,1)
!
    do i1=m1,n1
      lnx1_s=alog(    rho(i1)                             )
      lnx2_s=alog(max(e(  i1) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: T---
      T(   i1)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(   i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter1_TdT


!----------****************-------------------------------------------------------------------------
subroutine eosinter3_TdTOMP(rho, e, T, dTde, m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_TdTOMP ('equation_of_state_interpolation_3_Temperature_delta_Temperature_OpenMP')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> T, dT
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_TdTOMP(rho, e, T, dTde, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2002-04-19 (B.F.) Special routine for T,dT pair
!   2004-03-08 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-17 (B.F.) Rename routine name: append OMP
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: T, dTde
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
end subroutine eosinter3_TdTOMP


!----------*****************------------------------------------------------------------------------
subroutine eosinter1_logPTdT(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter1_logPTdT ('eos_interpolation_1_logarithm_Pressure_Temperature_delta_Temp')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdT(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version without input indices. Extra branch with new vectorization scheme
!   2013-02-18 (B.F.) 1D version
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),        intent(in)   :: rho, e
real, dimension(:),        intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: m1,n1
integer                                 :: i1, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
m1=1
n1=size(rho,1)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t        =x2ta(i1+ib)
        logP(i1+ib)=             poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib)=             poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib)=             poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t     =x2ta(i1)
      logP(i1)=             poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1)=             poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1)=             poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      T(   i1)=exp(T(i1))
      dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* dTde(i1)
    end do ! i1
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: m1,n1
integer                                 :: i1
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
n1=size(rho,1)
!
    do i1=m1,n1
      lnx1_s=alog(    rho(i1)                             )
      lnx2_s=alog(max(e(  i1) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(   i1)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(   i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1)=T(i1)/(e(i1)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter1_logPTdT


!----------*****************------------------------------------------------------------------------
subroutine eosinter2_logPTdT(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter2_logPTdT ('eos_interpolation_2_logarithm_Pressure_Temperature_delta_Temp')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdT(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version without input indices. Extra branch with new vectorization scheme
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:),      intent(in)   :: rho, e
real, dimension(:,:),      intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: m1,n1, m2,n2
integer                                 :: i1, i2, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
n1=size(rho,1)
n2=size(rho,2)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t           =x2ta(i1+ib)
        logP(i1+ib,i2)=          poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib,i2)=          poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib,i2)=          poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t        =x2ta(i1)
      logP(i1,i2)=          poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1,i2)=          poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1,i2)=          poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      T(   i1,i2)=exp(T(i1,i2))
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* dTde(i1,i2)
    end do ! i1
  end do ! i2
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: m1,n1, m2,n2
integer                                 :: i1, i2
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
n1=size(rho,1)
n2=size(rho,2)
!
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(    rho(i1,i2)                             )
      lnx2_s=alog(max(e(  i1,i2) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1,i2)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(   i1,i2)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1,i2)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2)=T(i1,i2)/(e(i1,i2)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter2_logPTdT


!----------*****************------------------------------------------------------------------------
subroutine eosinter3_logPTdT(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_logPTdT ('eos_interpolation_3_logarithm_Pressure_Temperature_delta_Temp')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdT(rho, e, logP, T, dTde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version without input indices. Extra branch with new vectorization scheme
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t              =x2ta(i1+ib)
        logP(i1+ib,i2,i3)=       poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib,i2,i3)=       poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib,i2,i3)=       poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t           =x2ta(i1)
      logP(i1,i2,i3)=       poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1,i2,i3)=       poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1,i2,i3)=       poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      T(i1,i2,i3)   =exp(T(i1,i2,i3))
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* dTde(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(    rho(i1,i2,i3)                             )
      lnx2_s=alog(max(e(  i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1,i2,i3)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter3_logPTdT


!----------*******************----------------------------------------------------------------------
subroutine eosinter3ii_logPTdT(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3, m1c,n1c, m2c,n2c, m3c,n3c)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3ii_logPTdT ('eos_interpolation_3ii_logarithm_Pressure_Temperature_delta_Temperature')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdT(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3, m1c,n1c, m2c,n2c, m3c,n3c)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!   m1:           ('m_1') integer, lower index of array, 1st dimension
!   n1:           ('n_1') integer, upper index of array, 1st dimension
!   m2:           ('m_2') integer, lower index of array, 2nd dimension
!   n2:           ('n_2') integer, upper index of array, 2nd dimension
!   m3:           ('m_3') integer, lower index of array, 3rd dimension
!   n3:           ('n_3') integer, upper index of array, 3rd dimension
!   m1c:          ('m_1_chunk') integer, lower index of chunk to treat, 1st dimension
!   n1c:          ('n_1_chunk') integer, upper index of chunk to treat, 1st dimension
!   m2c:          ('m_2_chunk') integer, lower index of chunk to treat, 2nd dimension
!   n2c:          ('n_2_chunk') integer, upper index of chunk to treat, 2nd dimension
!   m3c:          ('m_3_chunk') integer, lower index of chunk to treat, 3rd dimension
!   n3c:          ('n_3_chunk') integer, upper index of chunk to treat, 3rd dimension
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version with chunk indices. Extra branch with new vectorization scheme
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, m1c,n1c, m2c,n2c, m3c,n3c
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(m1c:n1c)             :: j1a, j2a
real,    dimension(m1c:n1c)             :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
!n1part=(n1c-(m1c-1))/2*2 + (m1c-1)
!n1part=(n1c-(m1c-1))/4*4 + (m1c-1)
n1part=(n1c-(m1c-1))/nblock*nblock + (m1c-1)
!
do i3=m3c,n3c
  do i2=m2c,n2c
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1c:n1c,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1c:n1c,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1c,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t              =x2ta(i1+ib)
        logP(i1+ib,i2,i3)=       poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib,i2,i3)=       poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib,i2,i3)=       poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1c
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t           =x2ta(i1)
      logP(i1,i2,i3)=       poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1,i2,i3)=       poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1,i2,i3)=       poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1c,n1c
      T(i1,i2,i3)   =exp(T(i1,i2,i3))
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* dTde(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
do i3=m3c,n3c
  do i2=m2c,n2c
    do i1=m1c,n1c
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1,i2,i3)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter3ii_logPTdT


!----------********************---------------------------------------------------------------------
subroutine eosinter3_logPTdTOMP(rho, e, logP, T, dTde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_logPTdTOMP ('eos_interpolation_3_logarithm_Pressure_Temperature_delta_Temp_OpenMP')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdTOMP(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version without input indices. Extra branch with new vectorization scheme
!   2013-02-17 (B.F.) Rename routine name: append OMP
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t              =x2ta(i1+ib)
        logP(i1+ib,i2,i3)=       poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib,i2,i3)=       poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib,i2,i3)=       poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t           =x2ta(i1)
      logP(i1,i2,i3)=       poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1,i2,i3)=       poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1,i2,i3)=       poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      T(i1,i2,i3)   =exp(T(i1,i2,i3))
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* dTde(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1,i2,i3)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter3_logPTdTOMP


!----------*********************--------------------------------------------------------------------
subroutine eosinter3i_logPTdTOMP(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3i_logPTdTOMP ('eos_interpolation_3i_logarithm_Pressure_Temp_delta_Temp_OpenMP')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdTOMP(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Extra branch with new vectorization scheme
!   2013-02-17 (B.F.) Rename routine name: append OMP
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(m1:n1)               :: j1a, j2a
real,    dimension(m1:n1)               :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t              =x2ta(i1+ib)
        logP(i1+ib,i2,i3)=       poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib,i2,i3)=       poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib,i2,i3)=       poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t           =x2ta(i1)
      logP(i1,i2,i3)=       poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1,i2,i3)=       poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1,i2,i3)=       poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      T(i1,i2,i3)   =exp(T(i1,i2,i3))
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* dTde(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1,i2,i3)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter3i_logPTdTOMP


!----------**********************-------------------------------------------------------------------
subroutine eosinter3ii_logPTdTOMP(rho, e, logP, T, dTde, &
                                  m1,n1, m2,n2, m3,n3, m1c,n1c, m2c,n2c, m3c,n3c)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3ii_logPTdTOMP ('eos_interpolation_3ii_logarithm_Pressure_Temp_delta_Temp_OpenMP')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_logPTdTOpenMP(rho, e, logP, T, dTde, &
!                               m1,n1, m2,n2, m3,n3, m1c,n1c, m2c,n2c, m3c,n3c)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!   m1:           ('m_1') integer, lower index of array, 1st dimension
!   n1:           ('n_1') integer, upper index of array, 1st dimension
!   m2:           ('m_2') integer, lower index of array, 2nd dimension
!   n2:           ('n_2') integer, upper index of array, 2nd dimension
!   m3:           ('m_3') integer, lower index of array, 3rd dimension
!   n3:           ('n_3') integer, upper index of array, 3rd dimension
!   m1c:          ('m_1_chunk') integer, lower index of chunk to treat, 1st dimension
!   n1c:          ('n_1_chunk') integer, upper index of chunk to treat, 1st dimension
!   m2c:          ('m_2_chunk') integer, lower index of chunk to treat, 2nd dimension
!   n2c:          ('n_2_chunk') integer, upper index of chunk to treat, 2nd dimension
!   m3c:          ('m_3_chunk') integer, lower index of chunk to treat, 3rd dimension
!   n3c:          ('n_3_chunk') integer, upper index of chunk to treat, 3rd dimension
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!   2013-02-15 (B.F. Lyon) Version with chunk indices. Extra branch with new vectorization scheme
!   2013-02-17 (B.F.) Rename routine name: append OMP
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, m1c,n1c, m2c,n2c, m3c,n3c
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: logP, T, dTde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=2
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(m1c:n1c)             :: j1a, j2a
real,    dimension(m1c:n1c)             :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*8,4)          :: Ct
real,    dimension(nblock*8)            :: poly1, x1t
!---------------------------------------------------------------------------------------------------
!
!n1part=(n1c-(m1c-1))/2*2 + (m1c-1)
!n1part=(n1c-(m1c-1))/4*4 + (m1c-1)
n1part=(n1c-(m1c-1))/nblock*nblock + (m1c-1)
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3c,n3c
  do i2=m2c,n2c
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1c:n1c,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1c:n1c,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1c,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        !DIR$ UNROLL
        do k=0,1
          Ct( 1+8*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+8*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+8*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+8*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
        end do ! k
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+8*ib:8+8*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 8)=x1ta(i1  )
      x1t( 9:16)=x1ta(i1+1)
      !x1t(17:24)=x1ta(i1+2)
      !x1t(25:32)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,nblock*8
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t              =x2ta(i1+ib)
        logP(i1+ib,i2,i3)=       poly1(1+8*ib) + &
                            x2t*(poly1(2+8*ib) + &
                            x2t*(poly1(3+8*ib) + &
                            x2t*(poly1(4+8*ib)  )))
        T(   i1+ib,i2,i3)=       poly1(5+8*ib) + &
                            x2t*(poly1(6+8*ib) + &
                            x2t*(poly1(7+8*ib) + &
                            x2t*(poly1(8+8*ib)  )))
        dTde(i1+ib,i2,i3)=       poly1(6+8*ib) + &
                        2.0*x2t*(poly1(7+8*ib) + &
                        1.5*x2t*(poly1(8+8*ib)  ))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1c
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do k=0,1
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      end do ! k
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction: P, T ---
      do ip=1,8
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction: P, T ---
      x2t           =x2ta(i1)
      logP(i1,i2,i3)=       poly1(1) + &
                       x2t*(poly1(2) + &
                       x2t*(poly1(3) + &
                       x2t*(poly1(4)  )))
      T(   i1,i2,i3)=       poly1(5) + &
                       x2t*(poly1(6) + &
                       x2t*(poly1(7) + &
                       x2t*(poly1(8)  )))
      dTde(i1,i2,i3)=       poly1(6) + &
                   2.0*x2t*(poly1(7) + &
                   1.5*x2t*(poly1(8)  ))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1c,n1c
      T(i1,i2,i3)   =exp(T(i1,i2,i3))
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* dTde(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif 
!#
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3c,n3c
  do i2=m2c,n2c
    do i1=m1c,n1c
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      ! !DIR$ IVDEP
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      ! !DIR$ IVDEP
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2)
!#                           /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
      logP(i1,i2,i3)= &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )))
      T(i1,i2,i3)=exp( &
                        EOS%C( 1,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,3,j1_s,j2_s) + &
                   x1t* EOS%C( 4,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,3,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,3,j1_s,j2_s) + &
                   x1t* EOS%C( 8,3,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
                   x1t*(EOS%C(10,3,j1_s,j2_s) + &
                   x1t*(EOS%C(11,3,j1_s,j2_s) + &
                   x1t* EOS%C(12,3,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
                   x1t*(EOS%C(14,3,j1_s,j2_s) + &
                   x1t*(EOS%C(15,3,j1_s,j2_s) + &
                   x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary arrays */
      ! --- Do bicubic interpolation for desired rho/e pair: logP ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      logP(i1,i2,i3)= &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )))
      !
      ! --- Do bicubic interpolation for desired rho/e pair: T ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,3,j1_s,j2_s)
!#
#     endif
!#
      T(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(Ct( 3) + x1t*Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(Ct( 7) + x1t*Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(Ct(11) + x1t*Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(Ct(15) + x1t*Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
end subroutine eosinter3ii_logPTdTOMP


!----------***********************------------------------------------------------------------------
subroutine eosinter3_logPTdTnewCv1(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_logPTdT ('eos_interpolation_3_logarithm_Pressure_Temperature_delta_Temperature')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter3_logPTdT(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: logP, T, dTde
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!---------------------------------------------------------------------------------------------------
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      ! --- Determine indices of interpolation coefficients and relative position in cell ---
      lnx1_s=alog(rho(  i1,i2,i3))
      j1_s  =min(max(int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)+1,1),EOS%nx1)
      ! !DIR$ IVDEP
      x1t   =lnx1_s-EOS%lnx11d(j1_s)
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      j2_s  =min(max(int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)+1,1),EOS%nx2)
      ! !DIR$ IVDEP
      x2t   =lnx2_s-EOS%lnx21d(j2_s)
      !
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
!!$      logP(i1,i2,i3)= &
!!$                      EOS%C( 1,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 2,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 3,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 4,2,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 6,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 7,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 8,2,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(10,2,j1_s,j2_s) + & 
!!$                 x1t*(EOS%C(11,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C(12,2,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C(13,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(14,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(15,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C(16,2,j1_s,j2_s)))  )))
!!$      T(i1,i2,i3)=exp( &
!!$                      EOS%C( 1,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 2,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 3,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 4,3,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 6,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 7,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 8,3,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(10,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(11,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(12,3,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C(13,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(14,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(15,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
!!$      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
!!$                      EOS%C( 5,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 6,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 7,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 8,3,j1_s,j2_s))) + &
!!$        2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(10,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(11,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(12,3,j1_s,j2_s))) + &
!!$        1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(14,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(15,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
      logP(i1,i2,i3)= &
                      EOS%Cv( 1 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 2 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 3 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 4 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 5 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 6 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 7 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 8 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 9 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(10 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + & 
                 x1t*(EOS%Cv(11 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(12 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv(13 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(14 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(15 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(16 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1))))))  )))
      T(i1,i2,i3)=exp( &
                      EOS%Cv( 1 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 2 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 3 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 4 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 5 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 6 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 7 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 8 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 9 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(10 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(11 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(12 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv(13 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(14 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(15 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(16 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1))))))  ))) )
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                      EOS%Cv( 5 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 6 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 7 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 8 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
        2.0*x2t*(     EOS%Cv( 9 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(10 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(11 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(12 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
        1.5*x2t*(     EOS%Cv(13 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(14 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(15 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(16 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1))))))  )) )
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
end subroutine eosinter3_logPTdTnewCv1


!----------***********************------------------------------------------------------------------
subroutine eosinter3_logPTdTnewCv2(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_logPTdT ('eos_interpolation_3_logarithm_Pressure_Temperature_delta_Temperature')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter3_logPTdT(rho, e, logP, T, dTde, m1,n1, m2,n2, m3,n3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   logP:         ('logarithm_Pressure') natural logarithm of pressure [log(dyn/cm^2)]
!   T:            ('Temperature') temperature [K]
!   dTde:         ('delta_Temperature_delta_energy') dT/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine
!   2001-02-20 (B.F.) Version for output of logP
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2008-05-30 (M.S.) Array dimensions as formal arguments to avoid problems
!              with size() encountered using OMP under linux / intel.
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(in)   :: rho, e
real, dimension(m1:n1,m2:n2,m3:n3), &
                           intent(out)  :: logP, T, dTde
!
! --- Local variables ---
integer                                 :: i1, i2, i3
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
real                                    :: x1t, x2t
integer, dimension(m1:n1)               :: j1_sa, j2_sa
real,    dimension(m1:n1)               :: x1ta, x2ta
!---------------------------------------------------------------------------------------------------
!
!$OMP DO SCHEDULE(RUNTIME) &
!$OMP    PRIVATE(i1,i2,i3, j1_s,j2_s, j1_sa,j2_sa, lnx1_s,lnx2_s, x1t,x2t, x1ta,x2ta)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      ! --- Determine indices of interpolation coefficients and relative position in cell ---
      lnx1_s   =alog(rho(  i1,i2,i3))
      j1_sa(i1)=min(max(int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)+1,1),EOS%nx1)
      ! !DIR$ IVDEP
      x1ta(i1)=lnx1_s-EOS%lnx11d(j1_sa(i1))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      j2_sa(i1)=min(max(int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)+1,1),EOS%nx2)
      ! !DIR$ IVDEP
      x2ta(i1)=lnx2_s-EOS%lnx21d(j2_sa(i1))
    end do ! i1
    !
    do i1=m1,n1
      x1t =x1ta( i1)
      x2t =x2ta( i1)
      j1_s=j1_sa(i1)
      j2_s=j2_sa(i1)
      !
      ! --- Do bicubic interpolation for desired rho/e pair: logP and T---
!!$      logP(i1,i2,i3)= &
!!$                      EOS%C( 1,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 2,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 3,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 4,2,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 6,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 7,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 8,2,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(10,2,j1_s,j2_s) + & 
!!$                 x1t*(EOS%C(11,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C(12,2,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C(13,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(14,2,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(15,2,j1_s,j2_s) + &
!!$                 x1t* EOS%C(16,2,j1_s,j2_s)))  )))
!!$      T(i1,i2,i3)=exp( &
!!$                      EOS%C( 1,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 2,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 3,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 4,3,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 5,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 6,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 7,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 8,3,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(10,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(11,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(12,3,j1_s,j2_s))) + &
!!$            x2t*(     EOS%C(13,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(14,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(15,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(16,3,j1_s,j2_s)))  ))) )
!!$      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
!!$                      EOS%C( 5,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 6,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C( 7,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C( 8,3,j1_s,j2_s))) + &
!!$        2.0*x2t*(     EOS%C( 9,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(10,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(11,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(12,3,j1_s,j2_s))) + &
!!$        1.5*x2t*(     EOS%C(13,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(14,3,j1_s,j2_s) + &
!!$                 x1t*(EOS%C(15,3,j1_s,j2_s) + &
!!$                 x1t* EOS%C(16,3,j1_s,j2_s)))  )) )
      logP(i1,i2,i3)= &
                      EOS%Cv( 1 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 2 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 3 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 4 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 5 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 6 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 7 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 8 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 9 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(10 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + & 
                 x1t*(EOS%Cv(11 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(12 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv(13 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(14 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(15 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(16 + EOS%nC*(2-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1))))))  )))
      T(i1,i2,i3)=( &
                      EOS%Cv( 1 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 2 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 3 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 4 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 5 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 6 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 7 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 8 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv( 9 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(10 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(11 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(12 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
            x2t*(     EOS%Cv(13 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(14 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(15 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(16 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1))))))  ))) )
      dTde(i1,i2,i3)=( &
                      EOS%Cv( 5 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 6 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv( 7 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv( 8 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
        2.0*x2t*(     EOS%Cv( 9 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(10 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(11 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(12 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))))) + &
        1.5*x2t*(     EOS%Cv(13 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(14 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t*(EOS%Cv(15 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1)))) + &
                 x1t* EOS%Cv(16 + EOS%nC*(3-1 + EOS%ny*(j1_s-1 + EOS%nx1*(j2_s-1))))))  )) )
    end do ! i1
    !
    do i1=m1,n1
      ! --- Perform final exponentiation and scaling ---
      T(   i1,i2,i3)=exp(T(i1,i2,i3))
      dTde(i1,i2,i3)=T(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift) * dTde(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
end subroutine eosinter3_logPTdTnewCv2


!----------*************----------------------------------------------------------------------------
subroutine eosinter1_PdP(rho, e, P, dPdrho, dPde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter1_PdP ('equation_of_state_interpolation_1_Pressure_delta_Pressure')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_PdP(rho, e, P, dPdrho, dPde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine, keyword ierr removed
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2007-12-26 (B.F.) Derived from eosinter3_PdP
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),        intent(in)   :: rho, e
real, dimension(:),        intent(out)  :: P, dPdrho, dPde
!
! --- Local variables ---
integer                                 :: i1, n1
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
n1=size(rho,1)
!
    do i1=1,n1
      lnx1_s=alog(rho(  i1))
      lnx2_s=alog(max(e(i1) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
      P(i1)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1)=P(i1)/rho(i1)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1)=P(i1)/(e(i1)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary array */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1)=P(i1)/rho(i1)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
             x2t*(               Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
             x2t*(               Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
             x2t*(               Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1)=P(i1)/(e(i1)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
!
end subroutine eosinter1_PdP


!----------*************----------------------------------------------------------------------------
subroutine eosinter3_PdP(rho, e, P, dPdrho, dPde)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_PdP ('equation_of_state_interpolation_3_Pressure_delta_Pressure')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_PdP(rho, e, P, dPdrho, dPde)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   dPdrho:       ('delta_Pressure_delta_rho') dP/drho|e [dyn cm/g]
!   dPde:         ('delta_Pressure_delta_energy') dP/de|rho
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine, keyword ierr removed
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-18 (B.F. Lyon) Extra branch with new vectorization scheme
!   2013-03-02 (B.F.) Bugfix: 8*ib -> 4*ib
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: P, dPdrho, dPde
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=4
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t
real,    dimension(nblock*4,4)          :: Ct
real,    dimension(nblock*4)            :: poly1, x1t
real,    dimension(nblock*4)            :: poly2
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
! !$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        k=0
          Ct( 1+4*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+4*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+4*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+4*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+4*ib:4+4*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 4)=x1ta(i1  )
      x1t( 5: 8)=x1ta(i1+1)
      x1t( 9:12)=x1ta(i1+2)
      x1t(13:16)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*4
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      do ib=0,nblock-1
        do ip=1,4
          poly2(ip+4*ib)=                    Ct(ip+4*ib, 2) + &
                           x1t(ip+4*ib)*(2.0*Ct(ip+4*ib, 3) + &
                           x1t(ip+4*ib)* 3.0*Ct(ip+4*ib, 4)  )
        end do ! ip
      end do ! ib
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t                =x2ta(i1+ib)
        !
        P(     i1+ib,i2,i3)=     poly1( 1+4*ib) + &
                            x2t*(poly1( 2+4*ib) + &
                            x2t*(poly1( 3+4*ib) + &
                            x2t*(poly1( 4+4*ib)  )))
        dPde(  i1+ib,i2,i3)=     poly1( 2+4*ib) + &
                        2.0*x2t*(poly1( 3+4*ib) + &
                        1.5*x2t*(poly1( 4+4*ib)  ))
        !
        dPdrho(i1+ib,i2,i3)=     poly2( 1+ 4*ib) + &
                            x2t*(poly2( 2+ 4*ib) + &
                            x2t*(poly2( 3+ 4*ib) + &
                            x2t*(poly2( 4+ 4*ib)  )))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      k=0
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,4
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      do ip=1,4
        poly2(ip)=Ct(ip, 2) + x1ta(i1)*(2.0*Ct(ip, 3) + x1ta(i1)*3.0*Ct(ip, 4))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t           =x2ta(i1)
      !
      P(     i1,i2,i3)=        poly1( 1) + &
                          x2t*(poly1( 2) + &
                          x2t*(poly1( 3) + &
                          x2t*(poly1( 4)  )))
      dPde(  i1,i2,i3)=        poly1( 2) + &
                      2.0*x2t*(poly1( 3) + &
                      1.5*x2t*(poly1( 4)  ))
      !
      dPdrho(i1,i2,i3)=        poly2( 1) + &
                          x2t*(poly2( 2) + &
                          x2t*(poly2( 3) + &
                          x2t*(poly2( 4)  )))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      P(     i1,i2,i3)=exp(P(i1,i2,i3))
      dPde(  i1,i2,i3)=P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dPde(  i1,i2,i3)
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/(rho(i1,i2,i3)             )* dPdrho(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: x1t, x2t
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
      P(i1,i2,i3)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary array */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dPdrho(i1,i2,i3)=P(i1,i2,i3)/rho(i1,i2,i3)* ( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
             x2t*(               Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
             x2t*(               Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
             x2t*(               Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde(i1,i2,i3)=P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
!#
#     endif
!#
    end do ! i1
  end do ! i2
end do ! i3
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
!
end subroutine eosinter3_PdP


!----------******************-----------------------------------------------------------------------
subroutine eosinter3_PGamma13(rho, e, P, Gamma_1, Gamma_3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   eosinter3_PGamma13 ('equation_of_state_interpolation_3_Pressure_Gamma_1_3')
!
! PURPOSE:
!   Solve EOS by bicubic interpolation: rho, e -> P, Gamma_1, Gamma_3
!
! CATEGORY:
!   EOS, Interpolation
!
! CALLING SEQUENCE:
!   call eosinter_PdP(rho, e, P, Gamma_1, Gamma_3)
!
! INPUT:
!   rho:          ('rho') density [g/cm^3]
!   e:            ('energy') internal energy per mass unit [erg/g]
!
! OUTPUT:
!   P:            ('Pressure') pressure [dyn/cm^2]
!   Gamma_1:      ('Gamma_1') 1st adiabatic coefficient
!   Gamma_3:      ('Gamma_3') 3rd adiabatic coefficient
!
! ROUTINES:
!   None
!
! MODULES:
!   tabinter_coeff_module: Interpolation coefficients
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   For rho/e pair determine indices of interpolation coefficients and relative
!   position in cell.
!   Do bicubic interpolation for desired rho/e pair and some derivatives.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-05-05 (B.F. Kiel) Written
!   1997-05-06 (B.F.) Correction of dP/drho|e (not complete)
!   1997-05-16 (B.F.) Vectorized version
!   1997-05-21 (B.F.) ierr>0: point outside table boundaries, only shifts in e are allowed
!   1997-07-25 (B.F.) Number of input parameters reduced, output quantities changed
!   1997-09-11 (B.F.) "kenter=0" removed
!   1999-05-31 (B.F. Copenhagen) Restrict minimum internal energy
!   2000-12-30 (B.F. Kiel) 3D version
!   2001-01-21 (B.F. Uppsala) Special case if all quantities are requested
!   2001-01-25 (B.F.) Special case if only s and T are requested
!   2001-02-06 (B.F.) Ct=> replaced with Ct=
!   2001-02-08 (B.F.) Put into own routine, keyword ierr removed
!   2004-03-02 (S.W.-B.) New alternative version without temporary coefficient arrays
!   2013-02-18 (B.F. Lyon) Extra branch with new vectorization scheme
!   2013-03-02 (B.F.) Bugfix: 8*ib -> 4*ib
!   2013-05-14 (B.F.) eosinter3_PdP -> eosinter3_PGamma13
!---------------------------------------------------------------------------------------------------
use tabinter_coeff_module
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:,:),    intent(in)   :: rho, e
real, dimension(:,:,:),    intent(out)  :: P, Gamma_1, Gamma_3
!
!#
# if (gasinter_l01 == 4)
!#                       /* Loop-split vectorized version --------------------------------------- */
! --- Local variables ---
integer, parameter                      :: nblock=4
integer                                 :: m1,n1, m2,n2, m3,n3
integer                                 :: i1, i2, i3, ib, ip, k, l, j1, j2, n1part
integer, dimension(1:size(rho,1))       :: j1a, j2a
real,    dimension(1:size(rho,1))       :: lnx1, lnx2, x1ta, x2ta
real                                    :: x2t, dlnPdlnrho, dPde
real,    dimension(nblock*4,4)          :: Ct
real,    dimension(nblock*4)            :: poly1, x1t
real,    dimension(nblock*4)            :: poly2
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
n1part=(n1-(m1-1))/nblock*nblock + (m1-1)
!
! !$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Determine indices of interpolation coefficients and relative position in cell ---
    lnx1=alog(    rho(m1:n1,i2,i3)                           )
    j1a =min(max(int((lnx1-EOS%lnx1off)*EOS%lnx1fac)+1, 1), EOS%nx1)
    !DIR$ IVDEP
    x1ta=lnx1-EOS%lnx11d(j1a)
    lnx2=alog(max(e(  m1:n1,i2,i3)+EOS%x2_shift, EOS%lnx2off))
    j2a =min(max(int((lnx2-EOS%lnx2off)*EOS%lnx2fac)+1, 1), EOS%nx2)
    !DIR$ IVDEP
    x2ta=lnx2-EOS%lnx21d(j2a)
    !
    ! --- Block and partly unroll the first part ---
    do i1=m1,n1part,nblock
      !
      ! --- Put coefficients into temporary array ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        k=0
          Ct( 1+4*ib+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 2+4*ib+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 3+4*ib+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1+ib), j2a(i1+ib))
          Ct( 4+4*ib+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1+ib), j2a(i1+ib))
      end do ! ib
      !
      ! --- Put delta rho values into array (do different unrolling counts manually!) ---
      ! !DIR$ UNROLL
      ! do ib=0,nblock-1
      !   x1t(1+4*ib:4+4*ib)=x1ta(i1+ib)
      ! end do ! ib
      x1t( 1: 4)=x1ta(i1  )
      x1t( 5: 8)=x1ta(i1+1)
      x1t( 9:12)=x1ta(i1+2)
      x1t(13:16)=x1ta(i1+3)
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,nblock*4
        poly1(ip)=Ct(ip, 1) + x1t(ip)*(Ct(ip, 2) + x1t(ip)*(Ct(ip, 3) + x1t(ip)*Ct(ip, 4)))
      end do ! ip
      do ib=0,nblock-1
        do ip=1,4
          poly2(ip+4*ib)=                    Ct(ip+4*ib, 2) + &
                           x1t(ip+4*ib)*(2.0*Ct(ip+4*ib, 3) + &
                           x1t(ip+4*ib)* 3.0*Ct(ip+4*ib, 4)  )
        end do ! ip
      end do ! ib
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      !DIR$ UNROLL
      do ib=0,nblock-1
        x2t                =x2ta(i1+ib)
        !
        P(     i1+ib,i2,i3)=     poly1( 1+4*ib) + &
                            x2t*(poly1( 2+4*ib) + &
                            x2t*(poly1( 3+4*ib) + &
                            x2t*(poly1( 4+4*ib)  )))
        !dPde(  i1+ib,i2,i3)=    poly1( 2+4*ib) + &
        Gamma_3(  i1+ib,i2,i3)=  poly1( 2+4*ib) + &
                        2.0*x2t*(poly1( 3+4*ib) + &
                        1.5*x2t*(poly1( 4+4*ib)  ))
        !
        !dPdrho(i1+ib,i2,i3)=    poly2( 1+ 4*ib) + &
        Gamma_1(i1+ib,i2,i3)=    poly2( 1+ 4*ib) + &
                            x2t*(poly2( 2+ 4*ib) + &
                            x2t*(poly2( 3+ 4*ib) + &
                            x2t*(poly2( 4+ 4*ib)  )))
      end do ! ib
    end do ! i1
    !
    ! --- Do the rest ---
    do i1=n1part+1,n1
      ! --- Put coefficients into temporary array ---
      k=0
        Ct( 1+4*k, 1:4)=EOS%C( 1: 4, 2+k, j1a(i1), j2a(i1))
        Ct( 2+4*k, 1:4)=EOS%C( 5: 8, 2+k, j1a(i1), j2a(i1))
        Ct( 3+4*k, 1:4)=EOS%C( 9:12, 2+k, j1a(i1), j2a(i1))
        Ct( 4+4*k, 1:4)=EOS%C(13:16, 2+k, j1a(i1), j2a(i1))
      !
      ! --- Do cubic interpolation for desired rho/e pair in x1 direction ---
      do ip=1,4
        poly1(ip)=Ct(ip, 1) + x1ta(i1)*(Ct(ip, 2) + x1ta(i1)*(Ct(ip, 3) + x1ta(i1)*Ct(ip, 4)))
      end do ! ip
      do ip=1,4
        poly2(ip)=Ct(ip, 2) + x1ta(i1)*(2.0*Ct(ip, 3) + x1ta(i1)*3.0*Ct(ip, 4))
      end do ! ip
      !
      ! --- Do cubic interpolation for desired rho/e pair in x2 direction ---
      x2t           =x2ta(i1)
      !
      P(     i1,i2,i3)=        poly1( 1) + &
                          x2t*(poly1( 2) + &
                          x2t*(poly1( 3) + &
                          x2t*(poly1( 4)  )))
      !dPde(  i1,i2,i3)=       poly1( 2) + &
      Gamma_3(  i1,i2,i3)=     poly1( 2) + &
                      2.0*x2t*(poly1( 3) + &
                      1.5*x2t*(poly1( 4)  ))
      !
      !dPdrho(i1,i2,i3)=       poly2( 1) + &
      Gamma_1(i1,i2,i3)=       poly2( 1) + &
                          x2t*(poly2( 2) + &
                          x2t*(poly2( 3) + &
                          x2t*(poly2( 4)  )))
    end do ! i1
    !
    ! --- Perform final exponentiation and scaling ---
    do i1=m1,n1
      P(     i1,i2,i3) =exp(P(i1,i2,i3))
      !dPde(  i1,i2,i3)=P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* dPde(  i1,i2,i3)
      !dPdrho(i1,i2,i3)=P(i1,i2,i3)/(rho(i1,i2,i3)             )* dPdrho(i1,i2,i3)
      dPde             =P(i1,i2,i3)/(e(  i1,i2,i3)+EOS%x2_shift)* Gamma_3(i1,i2,i3)
      dlnPdlnrho       =                                          Gamma_1(i1,i2,i3)
      Gamma_1(i1,i2,i3)=dlnPdlnrho + dPde/rho(i1,i2,i3)
      Gamma_3(i1,i2,i3)=1.0        + dPde/rho(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# else
!#     /* standard version ---------------------------------------------------------------------- */
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: x1t, x2t, dlnPdlnrho, dPde
integer                                 :: j1_s, j2_s
real                                    :: lnx1_s, lnx2_s
!#
# if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                              /* temporary array is copied */
real, dimension(16)                     :: Ct
!#
# endif
!# 
# if (gasinter_l01 == 1)
!#                       /* temporary array with pointer */
real, dimension(:), pointer             :: Ct
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
m1=1
m2=1
m3=1
n1=size(rho,1)
n2=size(rho,2)
n3=size(rho,3)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      lnx1_s=alog(rho(  i1,i2,i3))
      lnx2_s=alog(max(e(i1,i2,i3) + EOS%x2_shift, EOS%lnx2off))
      !
      ! --- Determine indices of interpolation coefficients ---
      j1_s=int((lnx1_s-EOS%lnx1off)*EOS%lnx1fac)
      j2_s=int((lnx2_s-EOS%lnx2off)*EOS%lnx2fac)
      j1_s=min(max(j1_s+1,1),EOS%nx1)
      j2_s=min(max(j2_s+1,1),EOS%nx2)
      !
      ! --- Get relative position in cell ---
      x1t=lnx1_s-EOS%lnx11d(j1_s)
      x2t=lnx2_s-EOS%lnx21d(j2_s)
      !
!#
#     if (gasinter_l01 == 2) || (gasinter_l01 == 4)
!#                                                  /* avoid temporary array Ct */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
      P(i1,i2,i3)=exp( &
                        EOS%C( 1,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 2,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 3,2,j1_s,j2_s) + &
                   x1t* EOS%C( 4,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
              x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + & 
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
              x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  ))) )
      dlnPdlnrho =( &
                        EOS%C( 2,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 3,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 4,2,j1_s,j2_s)) + &
           x2t*(        EOS%C( 6,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C( 7,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C( 8,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(10,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(11,2,j1_s,j2_s) + & 
                x1t* 3.*EOS%C(12,2,j1_s,j2_s)) + &
           x2t*(        EOS%C(14,2,j1_s,j2_s) + &
                x1t*(2.*EOS%C(15,2,j1_s,j2_s) + &
                x1t* 3.*EOS%C(16,2,j1_s,j2_s))  ))) )
      dPde       =P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                        EOS%C( 5,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 6,2,j1_s,j2_s) + &
                   x1t*(EOS%C( 7,2,j1_s,j2_s) + &
                   x1t* EOS%C( 8,2,j1_s,j2_s))) + &
          2.0*x2t*(     EOS%C( 9,2,j1_s,j2_s) + &
                   x1t*(EOS%C(10,2,j1_s,j2_s) + &
                   x1t*(EOS%C(11,2,j1_s,j2_s) + &
                   x1t* EOS%C(12,2,j1_s,j2_s))) + &
          1.5*x2t*(     EOS%C(13,2,j1_s,j2_s) + &
                   x1t*(EOS%C(14,2,j1_s,j2_s) + &
                   x1t*(EOS%C(15,2,j1_s,j2_s) + &
                   x1t* EOS%C(16,2,j1_s,j2_s)))  )) )
!#
#     else
!#                            /* using temporary array */
      ! --- Do bicubic interpolation for desired rho/e pair: P ---
!#
#     if (gasinter_l01 == 0) || (gasinter_l01 == 3)
!#                                                  /* temporary array is copied */
      Ct=EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
#     if (gasinter_l01 == 1)
!#                           /* temporary array with pointer */
      Ct=>EOS%C(:,2,j1_s,j2_s)
!#
#     endif
!#
      P(i1,i2,i3)=exp( &
                   Ct( 1) + x1t*(Ct( 2) + x1t*(   Ct( 3) + x1t*   Ct( 4))) + &
              x2t*(Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
              x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
              x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  ))) )
      dlnPdlnrho =( &
                                 Ct( 2) + x1t*(2.*Ct( 3) + x1t*3.*Ct( 4)) + &
             x2t*(               Ct( 6) + x1t*(2.*Ct( 7) + x1t*3.*Ct( 8)) + &
             x2t*(               Ct(10) + x1t*(2.*Ct(11) + x1t*3.*Ct(12)) + &
             x2t*(               Ct(14) + x1t*(2.*Ct(15) + x1t*3.*Ct(16))  ))) )
      dPde       =P(i1,i2,i3)/(e(i1,i2,i3)+EOS%x2_shift)* ( &
                   Ct( 5) + x1t*(Ct( 6) + x1t*(   Ct( 7) + x1t*   Ct( 8))) + &
          2.0*x2t*(Ct( 9) + x1t*(Ct(10) + x1t*(   Ct(11) + x1t*   Ct(12))) + &
          1.5*x2t*(Ct(13) + x1t*(Ct(14) + x1t*(   Ct(15) + x1t*   Ct(16)))  )) )
!#
#     endif
!#
      Gamma_1(i1,i2,i3)=dlnPdlnrho + dPde/rho(i1,i2,i3)
      Gamma_3(i1,i2,i3)=1.0        + dPde/rho(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!#
# endif
!#      /* Loop-split/standard version ---------------------------------------------------------- */
!
end subroutine eosinter3_PGamma13

end module gasinter_module
