c***************************************************************************
c    Grain diffusion routine by Tristan Guillot <guillot@obs-nice.fr> 
c      with updates by F. Allard and B. Freytag 2006/2007
c               original version: Fri Apr 21 17:21:33 2000
c      This routine computes new abundances of condensates and their 
c      averaged size. It returns right away when the initial abundance
c      of condensate is <=0.
c      Characteristic mixing time (tmix) must be between:
c                    1.d3 sec (nothing settles)
c               and  1.d4 sec (practically all grains settle)
c***************************************************************************


      subroutine rossow_grains(p,t,rho,grav,hp,tmix,qcond,rog,
     &                         qleft,amoy)
c     ------------------------------------------------------------
c     Version: 18/04/00
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Code de calcul de temps caracteristiques associes a la formation c
c     de nuages & grains dans les atmospheres planetaires et stellairesc
c     D'apres Rossow 1978                                              c
c     Note: On tient compte uniquement d'un equilibre entre            c
c      melange turbulent, condensation et sedimentation                c
c      (tps de coagulation & coalescence negliges)                     c
ccccccccccccccccccccccccsccccccccccccccccccccccccccccccccccccccccccccccc
c     (c) T. Guillot, Laboratoire Cassini, OCA, Nice
c         guillot@obs-nice.fr
c Entrees:
c     t,p,rho,grav,hp: temperature, pression, densite, gravite, 
c                  echelle de hauteur de pression; tout en cgs
c     tmix: temps de melange turbulent (en secondes)
c     qcond: fraction molaire initiale totale des elements condenses
c     rog: densite des grains condenses (moyenne) (en g/cm3)
c Sorties:
c     qleft: fraction molaire des grains tel que 
c            tmix <= (tfall=tcondensation)
c     amoy: taille moyenne des grains (tel que tcond=tmix) (en cm)

      implicit none
      real*8 t,p,rho,grav,hp,tmix,qcond,rog,qleft,amoy

c Variables locales
      integer i,itype,j,k,ier
      integer*4 leng
      real*8 minx,maxx,tcond,tfall,tcoag,tcoal,tgrow
      data minx/-8.d0/,maxx/0.d0/

c Fonctions externes
      real*8 ffall_mix,fgrow_mix,f
      external ffall_mix,fgrow_mix,f

c Common: pour appel de fonction uniquement
      real*8 rp,rt,rro,rgrav,rqleft,rrog,rtmix
      common /rossow/rp,rt,rro,rgrav,rqleft,rrog,rtmix

c----------------------------------------------------------------------

      if (qcond.le.0.d0) then
       qleft=0.d0
       amoy=0.d0
       return
      endif

      qleft=qcond

      rp=p
      rt=t
      rro=rho
      rgrav=grav
      rqleft=qleft
      rrog=rog
      rtmix=tmix

c On calcule la taille d'equilibre <a> melange/sedimentation
c Valeurs maximales de log10(size): minx(-8) et maxx (0) (en cm)
      call secante(ffall_mix,minx,maxx,amoy,1d-4,ier)
c      write(*,*)'Fall/Mix: amoy,ier',amoy,ier
      if ((ier.eq.1).and.(amoy.le.minx)) then
       qleft=0.d0
       amoy=0.d0
       return
      endif
c Tps de croissance pour cette taille de particules
      amoy=10.d0**amoy
      call rossow78_mod(p,t,rho,grav,qleft,rog,
     &      amoy,tcond,tfall,tcoag,tcoal)
      tgrow=1/(1.d0/tcond+1.d0/tcoag+1.d0/tcoal)
c      write(*,*)'tgrow,tcond:',tgrow,tcond,tcoag,tcoal,tfall
c On compare le tps de melange au temps de condensation pour la taille <a>
c Si la condensation est plus rapide, on recalcule la fraction de 
c condenses pour obtenir tcon=tmix
      if (tgrow.lt.tmix) then
       qleft=qleft*tgrow/tmix
       do while (abs(qleft-rqleft).gt.1d-10)
        call rossow78_mod(p,t,rho,grav,qleft,rog,
     &       amoy,tcond,tfall,tcoag,tcoal)
        tgrow=1.d0/(1.d0/tcond+1.d0/tcoag+1.d0/tcoal)
c        write(*,*)qleft,rqleft,tcond,tgrow
        rqleft=qleft
        qleft=qleft*tgrow/tmix
       enddo
      else
c Si le melange est rapide, la fraction de condenses n'est pas affectee
c mais on recalcule la taille moyenne <a> d'un equilibre entre le
c melange et la condensation
       call secante(fgrow_mix,minx,maxx,amoy,1d-4,ier)
       amoy=10.d0**amoy
c       write(*,*)'Grow/Mix: amoy,ier',amoy,ier
      endif

      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function f(x)
      real*8 f,x
      f=x*x*x+1
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function ffall_mix(x)
      implicit none
      real*8 ffall_mix,x

      real*8 size,tcond,tfall,tcoag,tcoal
      real*8 rp,rt,rro,rgrav,rqleft,rrog,rtmix
      common /rossow/rp,rt,rro,rgrav,rqleft,rrog,rtmix

      size=10.d0**x
      call rossow78_mod(rp,rt,rro,rgrav,rqleft,rrog,
     &     size,tcond,tfall,tcoag,tcoal)

      ffall_mix=log(tfall/rtmix)
      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fgrow_mix(x)
      implicit none
      real*8 fgrow_mix,x

      real*8 size,tcond,tfall,tcoag,tcoal
      real*8 rp,rt,rro,rgrav,rqleft,rrog,rtmix
      common /rossow/rp,rt,rro,rgrav,rqleft,rrog,rtmix

      size=10.d0**x
      call rossow78_mod(rp,rt,rro,rgrav,rqleft,rrog,
     &     size,tcond,tfall,tcoag,tcoal)

      fgrow_mix=log((tcond)/rtmix)
      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine secante(f,a,b,x,dx,ier)
      implicit none
      real*8 f,a,b,x,dx
      integer ier
      real*8 f1,f2,fx,x1,x2
      real*8 rp,rt,rro,rgrav,rqleft,rrog,rtmix
      common /rossow/rp,rt,rro,rgrav,rqleft,rrog,rtmix

      x1=a
      x2=b
      f1=f(x1)
      f2=f(x2)
      if (f1*f2.gt.0.) then
       if (abs(f1).gt.abs(f2)) then
        x=x2 
       else 
        x=x1
       endif
       ier=1
       return
      endif
c On alterne dichotomie et secantes
      do while (abs(x2-x1).gt.dx)
       x=x1+(x2-x1)*f1/(f1-f2)
       fx=f(x)
       if (fx*f1.gt.0.d0) then
        x1=x
        f1=fx
       else
        x2=x
        f2=fx
       endif
       x=(x1+x2)/2
       fx=f(x)
       if (fx*f1.gt.0.d0) then
        x1=x
        f1=fx
       else
        x2=x
        f2=fx
       endif
      enddo

      ier=0
      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
      subroutine rossow78_mod(p,t,ro,grav,xcond,rop,
     &     size,tcond,tfall,tcoag,tcoal)
c     Version: 16/04/00
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Calcul des temps caracteristiques de formation de nuages de      c
c     condensation (d'apres Rossow (1978), Icarus 30, 1-50)            c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Auteur: T. Guillot revised by F. Allard and B. Freytag 2006
c new entries: introducing grain type dependent rossow instead of mixed grains
c     eps : normalised abundance by number of limiting constituting element
c     stoich : stoichiometric coefficient of limiting element (Mg in Mg2SiO4)
c     ms :  monomere's mass 
c Entrees:
c     c_mol: type de molecule (ex: 'H2O')
c     p,t,ro: pression, temperature, densite du milieu (cgs)
c     grav: acceleration gravifique (en cm/s2)
c     xcond: concentration totale des condenses
c Sorties:
c     lat: chaleur latente de transition
c     nb: dimension des tableaux suivants
c     size(nb): taille des "gouttes"
c     t...(nb): temps de condensation, sedimentation, coagulation,coalescence
c NOTE: les temps de coagulation et coalescence ne sont pas calcules

      implicit none

      character*8 c_mol
      real*8 p,t,ro,grav,xcond,lat,size,tcond,tfall,
     &     tcoag,tcoal

      integer i
      real*8 eta,f,smax,alpha,sig,mu,rn,lambda,a70,ros,rop,rov,rovs,
     &     logamin,logamax,kn,re,nn,rgas
      logical phase

      real*8 pi,kbol,amu
      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/

c     van der Waals and covalent radii  for H and He:
      real*8 rvdw_h, rcov_h, rvdw_he
      data rvdw_h/1.2d-8/, rcov_h/0.37d-8/, rvdw_he/1.4d-8/

c     drag coefficient 
      real*8,parameter :: Cd=0.2d0  !for a sphere or a good car lol     fa07 

c      condensation curve of Mg2SiO4
c      real*8 a_ps, b_ps, c_ps, d_ps
c      data a_ps/42.179466259637d00/,b_ps/-2.94064111396886d04/
c      data c_ps/-4.52457333892826d07/,d_ps/2.00362363225697d10/

c-----------------------------------------------------------------------
 2000 format(7(1x,1pd10.3))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Parametres fixes dans le programme:                              c
c     -Correction de masse finie [f]				       c
c     -Supersaturation maximale [smax]				       c
c     -Efficacite du collage [alpha]				       c
c     -Section efficace du gaz [sig]				       c
c     -Masse molaire moyenne du gaz [mu]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     sig=pi*(1.d-8)**2
c     sig=pi*(2.d-8)**2    !CFA07 should be H2 not HI !!!
c     so take the vdW radius, for the "long side" cross-section add 
c     2*rvdw*covalent length for 2 out of 3 aspects       dh 12 Dec 2007
      sig = 4.d0 * (pi*rvdw_h**2 + 2.d0/3.d0*4.d0*rvdw_h*rcov_h)
      f=5.d0
      alpha=1.d0
      rgas=kbol/amu
      mu=ro*rgas*t/p

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Autres quantites:                                                c
c     -Densite atmospherique en nombre de particules/cm3 [rn]	       c
c     -Libre parcours moyen du gaz [lambda]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rn=ro/mu/amu
cdh07 corrected to canonical value:
      lambda = 1.d0 / (sqrt(2.d0)*rn*sig) ! == kbol*t / (sqrt(2.)*p*sig)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Viscosite dynamique [eta]					       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cfa07 eta=ro/sqrt(t)  !dh   eta = mu*amu/kbol * p/t / sqrt(t)  
cfa07 use Bernd's viscosity!
c      eta=ro*lambda*sqrt(8.d0*rgas*t/(pi*mu))
c      mu = rgas * ro*t/p
c      ro = mu*amu/kbol * p/t
c      lambda=sqrt(3.d0*pi)/(8.d0*rn*sig)  ! the old mean free path value 
c      eta=sqrt(3.d0/8.d0*kbol*t*mu*amu) / sig 
c
c     now using 14.8.33 from Reif (1965)                  dh 15 Dec 2007
      eta = sqrt(pi*kbol*t*mu*amu) * 5.d0/(16.d0*sig)
c        == ro * sqrt(2.d0*pi*kbol*t/(mu*amu)) * 5.d0/16.d0 * lambda
c units:   sqrt([g**2 * cm**2 / s**2]) / cm**2 == [g/cm/s] == [P] 
c

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Densite des condenses [ros]                                      c
c     Densite de vapeur equivalente[rov]                               c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      ros = xcond*ro

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Densite de vapeur equivalente[rov]                               c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      rov = (eps/stoich)*(ro/mu/amu)*ms-ros
      rov = ros  ! bad approximation

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     From Curtis S. Cooper vapor.                                     c
c     See http://www.lpl.arizona.edu/~curtis/research/clouds/          c
c                           doxygen/vapor_8c-source.html               c
c     00146         a =  42.179466259637;                              c
c     00147         b = -2.94064111396886e4;                           c
c     00148         c = -4.52457333892826e7;                           c
c     00149         d = 2.00362363225697e10;                           c
c     00151         return exp( a + b/x + c/pow(x,2.0) + d/pow(x,3.0) )c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      rovs = 0.5 * eps * ms / (kbol*t)
c     .     *exp(a_ps + b_ps/t + c_ps/t**2 + d_ps/t**3 )

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Supersaturation           				       c
c         s>0: growth ; s<0: evaporation                               c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      if (rovs.gt.0.d0) then
c         smax = max(rov/rovs - 1.d0, 1.d-2)
c      else
c         smax = 1.d-2             
c      endif
      smax=1.d-2
      rovs = rov/(1.d0+smax)  ! arbitrary approx due to use of smax=1.d-2

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Taille des particules pour lesquelles Re=70                      c
c     Corrigee, d'apres Carlson et al. 1988                            c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      a70=(9.d0*270d0*eta*eta/(4.d0*rop*ro*grav))**(1.d0/3.d0)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Nombre de Knudsen [kn]                                          c
c     -Nombre de Reynolds [re]                                         c
c     -Densite de particules condensees pour une masse de condenses    c
c      supposee egale a la masse de vapeur saturante [nn]              c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       kn=lambda/size
cdh07  re=70.d0*(size/a70)**3 ! FA07: Re depends on size**3, better 
c      re=2*ro*size*v/eta ! v is a function of size depending on re...
c      first, calculate for Stokes velocity: v=2*rop*grav*size**2/9/eta
       re=4.d0*ro*rop*grav*size*size*size/(9.d0*eta*eta)
c      at larger Re we have to switch to turbulent flow: 
c      v=sqrt(8*rop*grav*size/(3*Cd*ro)) ! Cd=4/3 in this case
       if (re.gt.18.d0) re=sqrt(8.d0*ro*rop*grav*size*size*size/eta/eta)
c
       nn=ros/(4.d0/3.d0*pi*rop*size*size*size)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Calcul des temps de condensation, sedimentation, coagulation     c
c     et coalescence.						       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       tcond = 1.d70            !cfa06 default long time value (sec).
       if (kn.lt.alpha) then
c       laminar viscous flow (Stokes)     ! Eq. 16 
cfa06   tcond=ro*size*size/(2.d0*f*eta*ros*smax/rop)
          if (rovs.gt.0.d0) 
     &         tcond=ro*size*size/(2.d0*f*eta*rovs*smax/rop)
          if (re.gt.1.d0.and.ros.gt.0) then
c            turbulent flow (Newton) 
             tcond=tcond/(1.d0+2.d-1*sqrt(re))
          endif
       else
c       subsonic free molecular flow        ! Eq. 17 
        if (rovs.gt.0.d0) tcond=size/(3.d0*alpha*f*rovs*smax
     &       /(2.d0*rop))/sqrt(2.d0*kbol*t/(pi*amu*mu))
       endif

cfa06 verified on 6/11/2006                 ! Eqs. 19
       if (kn.gt.1.d0) then
c        subsonic free molecular flow
          tfall=27d0*pi*ro*(2.d0*kbol*t/(pi*amu*mu))**(3.d0/2.d0)
     &         /(16.d0*rop*grav*grav*size)
       elseif (re.gt.70.d0) then
c         turbulent flow (Newton)
cfa06  4/3 pi a^3 * rop is the mass! F = 1/2 Cd * a^2 * vfall**2 !  dh07
cfa07  40 is correct, being 4/(0.5*Cd) with the drag coefficient Cd ~ 0.2
          tfall=sqrt(3.d0*0.5d0*Cd*ro*(kbol*t)**2/(4.d0*rop*(amu*mu)**2*
     &         grav*grav*grav*size))
       else
c         laminar viscous flow (stokes)
          tfall=9.d0*eta*kbol*t/(2.d0*rop*amu*mu*grav*grav*size*size)
       endif

       tcoag = 1.d70            ! cfa06 set to large value by default
       if (kn.lt.1.d0) then
c         laminar viscous flow (Stokes)     ! Eq. 32 
          if(ros.gt.0.d0) tcoag=1.d0/((4.d0*kbol*t/(3.d0*eta))*nn)
       else
c         subsonic free molecular flow      ! Eq. 33 
          if(ros.gt.0.d0) 
     &         tcoag=1.d0/(4.d0*sqrt(3.d0*size*kbol*t/rop)*nn)
       endif

       tcoal = 1.d70            ! cfa06 set to large value by default
       if (kn.gt.1.d0) then                 ! Eq. 37 
          if (ros.gt.0.d0) tcoal=1.d0/((4.d0*pi*rop*grav/(27.d0*ro))*
     &         sqrt(pi*amu*mu/(2.d0*kbol*t))*size**3*nn)
       elseif (re.gt.70.d0) then            ! Eq. 36 
          if(ros.gt.0.d0) tcoal=1.d0/(sqrt(10.d0*pi*pi*rop*grav*size**5
     &         /(3.d0*ro))*nn)
       else
          if(ros.gt.0.d0)                   ! Eq. 35 
     &         tcoal=1.d0/((pi*rop*grav/(9.d0*eta))*size**4*nn)
       endif

c       write(*,'(1p,8d10.3)')p,t,size,tcond,tfall,tcoag,tcoal
      end
