!***************************************************************************************************
!
!   #####   #    #  #####          #    #  #    #  #####
!   #    #  #    #  #    #         ##  ##  #    #  #    #
!   #    #  ######  #    #         # ## #  ######  #    #
!   #####   #    #  #    #         #    #  #    #  #    #
!   #   #   #    #  #    #         #    #  #    #  #    #
!   #    #  #    #  #####  ####### #    #  #    #  ##### _module
!
!   Radiation-Hydrodynamics-Code: MHD module
!
!***************************************************************************************************
!   Fortran90
!   2018-11-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Uppsala, Lyon, Naples
!   Werner Schaffenberger: Freiburg, Exeter
!   Oskar Steiner:         Freiburg
!***************************************************************************************************
! MODULES:
!   rhd_mhd_module:        Magneto-hydrodynamics routines
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Gravity correction: 0:'off0', 1:'default0', 2:'default2', 3:'default3', 4, 5 --- */
# ifndef rhd_hyd_gravcorr_p01
#   define rhd_hyd_gravcorr_p01 5
# endif
!#
!# /* --- Method of entropy fix calculation: 0, 1, 2 --- */
# ifndef rhd_hyd_entropyfix_p01
#   define rhd_hyd_entropyfix_p01 0
# endif
!#
!# /* --- Method of upwind calculation: 0 or 1 --- */
# ifndef rhd_hyd_upwind_p01
#   define rhd_hyd_upwind_p01 0
# endif
!#
!# /* --- Additional slope reduction 0 (off), 1 (expansion wave), or 2 (density contrast) --- */
# ifndef rhd_roe1d_slope_l01
#   define rhd_roe1d_slope_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------**************-------------------------------------------------------------------------------
module rhd_mhd_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_module ('rhd_magneto_hydrodynamics_module')
!
! PURPOSE:
!   Provide MHD routines
!
! CATEGORY:
!   Hydrodynamics, Magnetic fields
!
! CALLING SEQUENCE:
!   use rhd_mhd_module
!
! VARIABLES:
!   dtime_limit_last: ('delta_time_limit_last') real, a hidden state variable with previous
!                     time step limit
!
! ROUTINES: (contained)
! > rhd_mhd_SwitchInfo:    Print information about used compiler switches
!   rhd_mhd_Roe1Doutput:   Extract some averaged quantities
!   rhd_mhd_Res3Doutput:   Extract some averaged quantities of resistivity step
!   rhd_mhd_Bboutput:      Extract averages of the boundary-centered magnetic field
!   rhd_mhd_Bb2Bc:         Compute cell-centered magnetic field from boundary values
!   rhd_bound_3DInOutFlow_mhd: Correct values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DTransmit_mhd: Correct values in cells of all sides for transmitting boundary cond.
!   rhd_bound_1DTransmit_mhd: Correct values in top or bottom cells for transmitting boundary cond.
!   rhd_mhd_Roe1Dflux:     Compute 1D MHD Roe fluxes
!   rhd_mhd_Roe1Dstep:     Perform 1D magnetohydrodynamics step in x1 direction (Roe solver)
!   rhd_mhd_PPvec:         Reconstruct with piecewise-parabolic scheme and compute boundary values
!   rhd_mhd_PPeqvec:       Reconstruct with piecewise-parabolic scheme and compute boundary values
!   rhd_mhd_FReqvec:       Reconstruct with Frankenstein scheme and compute boundary values
!   rhd_mhd_PLvec:         Piecewise linear reconstruction using different slope limiters
!   rhd_mhd_PLeqvec:       Piecewise linear reconstruction using different slope limiters
!   rhd_mhd_Reconstruct:   Reconstruct quantities in cells and compute boundary values
!   rhd_mhd_Hancock:       Perform Hancock predictor step in 1 dimension
!   rhd_mhd_Hancock2D:     Perform Hancock predictor step in 2 dimensions
!   rhd_mhd_Hancock3D:     Perform Hancock predictor step in 3 dimensions
!   rhd_mhd_CheckFlux:     Check if fluxes are save and replace by 1st order HLL fluxes if necessary
!   rhd_mhd_HLL1Dflux:     Compute 1D MHD HLL fluxes
!   rhd_mhd_HLL1Dstep:     Perform 1D magnetohydrodynamics step in x1 direction (HLL solver)
!   rhd_mhd_HLL2Dstep:     Perform 2D magnetohydrodynamics step (HLL solver)
!   rhd_mhd_HLL3Dstep:     Perform 3D magnetohydrodynamics step (HLL solver)
!   rhd_mhd_Res2DStep:     Perform 2D resistivity step
!   rhd_mhd_Res3DStep:     Perform 3D resistivity step
!   rhd_mhd_Constrain:     Perform constrained-transport step for magnetic fields
!   rhd_mhd_SingleStep:    Perform single magnetohydrodynamic step
!   rhd_mhd_UnsplitStep:   Perform single unsplit magnetohydrodynamic step
! > rhd_mhd_Step:          Perform entire 2D or 3D magneto-hydrodynamics step, employ HLLE scheme
!
! CALLING TREE:
!   rhd_mhd_SwitchInfo
!   |
!   rhd_bound_3DCenter
!   | eosinter
!   |
!   rhd_bound_3DHeatBottom
!   | eosinter
!   |
!   rhd_mhd_Step
!     rhd_box_AppendBc
!     rhd_mhd_SingleStep
!       * rhd_mhd_Bb2Bc
!       * rhd_bound_3DTransmit_mhd
!       * rhd_bound_3DInOutFlow_mhd
!       * rhd_mhd_Roe1Dstep
!       | + tabinter_getinfo
!       | + rhd_bound_1DGhost
!       | + rhd_grav
!       | + rhd_bound_1DTransmit_mhd
!       | + rhd_bound_1DGhost
!       | + eosinter_all
!       | + eosinter_PdP
!       | + rhd_mhd_Roe1Dflux
!       | | * rhd_hyd_Fluxalpha
!       | | * rhd_hyd_FluxalphaEq
!       | + rhd_mhd_Roe1Doutput
!       | + rhd_hyd_Dust
!       * rhd_mhd_HLL1Dstep
!       | + tabinter_getinfo
!       | + rhd_bound_1DGhost
!       | + rhd_grav
!       | + rhd_bound_1DTransmit_mhd
!       | + rhd_bound_1DGhost
!       | + eosinter_all
!       | + eosinter_PdP
!       | + rhd_mhd_Reconstruct
!       | | * tabinter_getinfo
!       | | * rhd_mhd_PPvec
!       | | * rhd_mhd_PPeqvec
!       | | * rhd_mhd_FReqvec
!       | | * rhd_mhd_PLvec
!       | | * rhd_mhd_PLeqvec
!       | + rhd_mhd_Hancock
!       | + rhd_mhd_HLL1Dflux
!       | | * tabinter_getinfo
!       | | * rhd_mhd_CheckFlux
!       | + rhd_mhd_Roe1Doutput
!       | + rhd_hyd_Dust
!       * rhd_mhd_Res2DStep
!       | + rhd_mhd_Res3Doutput
!       * rhd_mhd_Res3DStep
!       | + rhd_mhd_Res3Doutput
!       * rhd_mhd_Constrain
!     rhd_mhd_UnsplitStep
!       * rhd_mhd_Bb2Bc
!       * rhd_bound_3DTransmit_mhd
!       * rhd_bound_3DInOutFlow_mhd
!       * rhd_mhd_HLL2Dstep
!       | + tabinter_getinfo
!       | + rhd_grav
!       | + eosinter_all
!       | + eosinter_PdP
!       | + rhd_mhd_Reconstruct
!       | | * tabinter_getinfo
!       | | * rhd_mhd_PPvec
!       | | * rhd_mhd_PPeqvec
!       | | * rhd_mhd_FReqvec
!       | | * rhd_mhd_PLvec
!       | | * rhd_mhd_PLeqvec
!       | + rhd_mhd_Hancock2D
!       | + rhd_mhd_HLL1Dflux
!       | | * tabinter_getinfo
!       | | * rhd_mhd_CheckFlux
!       | + rhd_mhd_Roe1Doutput
!       | + rhd_hyd_Dust
!       * rhd_mhd_HLL3Dstep
!       | + tabinter_getinfo
!       | + rhd_grav
!       | + eosinter_all
!       | + eosinter_PdP
!       | + rhd_mhd_Reconstruct
!       | | * tabinter_getinfo
!       | | * rhd_mhd_PPvec
!       | | * rhd_mhd_PPeqvec
!       | | * rhd_mhd_FReqvec
!       | | * rhd_mhd_PLvec
!       | | * rhd_mhd_PLeqvec
!       | + rhd_mhd_Hancock3D
!       | + rhd_mhd_HLL1Dflux
!       | | * tabinter_getinfo
!       | | * rhd_mhd_CheckFlux
!       | + rhd_mhd_Roe1Doutput
!       | + rhd_hyd_Dust
!       * rhd_mhd_Res2DStep
!       | + rhd_mhd_Res3Doutput
!       * rhd_mhd_Res3DStep
!       | + rhd_mhd_Res3Doutput
!       * rhd_mhd_Constrain
!
! MODIFICATION HISTORY:
!   2002-11-04 (B.F. Uppsala) First version
!   2002-11-12 (B.F.) 3D
!   2003-12-15 (W.S.) Begin of implementation of MHD-Roe solver
!   2004-05-06 (W.S.) Routine rhd_mhd_HLL1Dflux added
!   2004-05-17 (W.S.) Routine rhd_mhd_SwitchInfo added
!   2004-05-21 (W.S.) Compiler switches for testing
!   2006-07-18 (W.S.) Cleanup (Test switches and some routines and code parts removed)
!   2008-03-24 (W.S.) Dual energy method
!   2010-02-16 (W.S.) Routine rhd_grav replaced by corresponding routine in rhd_hyd_module.F90
!   2010-04-12 (W.S.) Various bugfixes
!   2010-05-21 (B.F. Lyon) Identical copies of routines from rhd_hyd_module removed
!   2010-06-03 (W.S.) New routine rhd_mhd_SingleStep
!   2010-12-08 (W.S. Exeter) rhd_hyd_Roe1Doutput -> rhd_mhd_Roe1Doutput
!   2010-12-16 (W.S.) New routines rhd_mhd_Res_2Dstep, rhd_mhd_Res3D_step, rhd_mhd_Res3Doutput
!   2011-06-03 (W.S.) Unsplit Hancock scheme, new routines rhd_mhd_HLL1Dstep, rhd_mhd_HLL2Dstep,
!                     rhd_mhd_HLL3Dstep, rhd_mhd_Hancock2D, rhd_mhd_Hancock3D, rhd_mhd_UnsplitStep,
!                     rhd_bound_3DTransmit_mhd
!   2013-03-06 (B.F.) Performance improvements, bugfixes in mean output with Hancock/unsplit
!   2014-02-26 (B.F. Uppsala) Changes in OpenMP scheduling
!   2014-07-15 (B.F. Lyon) Allow explicit specification of gravity time centering
!   2014-08-17 (B.F. Uppsala) Use global boxes for working copies
!   2017-08-18 (B.F. Lyon) Change indentations and add comments
!   2017-08-19 (B.F.) Account for new datx type in averaging of quantities
!   2017-09-18 (B.F Uppsala) Account for move of boundary-condition routines into separate module
!   2018-05-22 (B.F.) Use the standard reconstruction module
!   2018-10-25 (B.F.) Reorder routines in module
!---------------------------------------------------------------------------------------------------
implicit none
!
private
!
public rhd_mhd_SwitchInfo, rhd_mhd_Step
!
real, save                              :: dtime_limit_last
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_mhd_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_SwitchInfo ('rhd_magneto-hydrodynamics_switch_information')
!
! PURPOSE:
!   Print information about used compiler switches.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!   The routine is abused to initialize dtime_limit_last.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_mhd_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2004-05-17 (W.S.) Written
!   2004-05-21 (W.S.) Compiler switches for testing
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_mhd_module .........................'
!
write(ncp,'(A,I6)') '  rhd' // '_hyd_gravcorr_p01:       ', rhd_hyd_gravcorr_p01
write(ncp,'(A,I6)') '  rhd' // '_hyd_entropyfix_p01:     ', rhd_hyd_entropyfix_p01
write(ncp,'(A,I6)') '  rhd' // '_hyd_upwind_p01:         ', rhd_hyd_upwind_p01
write(ncp,'(A,I6)') '  rhd' // '_roe1d_slope_l01:        ', rhd_roe1d_slope_l01
!
!#
# ifdef rhd_bound_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_bound_t01:           ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_bound_t01:           ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_roe1d_flux_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_flux_t01:      ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_flux_t01:      ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_roe1d_step_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_step_t01:      ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_step_t01:      ', undefstr
!#
# endif
!#
!
! --- Attention: initialize variable (a hidden state variable) ---
dtime_limit_last=huge(dtime_limit_last)
!
end subroutine rhd_mhd_SwitchInfo


!----------*******************----------------------------------------------------------------------
subroutine rhd_mhd_Res3Doutput(model, &
                               AreaFraction1, AreaFraction2, AreaFraction3, &
                               Fey1, Fey2, Fey3, dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Res3Doutput ('rhd_magneto-hydrodynamics_resistivity_3_dimension_output')
!
! PURPOSE:
!   Put some additional quantities into dat structure.
!
! CATEGORY:
!   Magnetohydrodynamics, Resistivity
!
! CALLING SEQUENCE:
!   call rhd_mhd_Res3Doutput(model, &
!                            AreaFraction1, AreaFraction2, AreaFraction3, &
!                            Fey1, Fey2, Fey3, dat)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   AreaFraction1: ('Area_Fraction_1') real, optional, weighting factor for chunk in x23 plane
!   AreaFraction2: ('Area_Fraction_2') real, optional, weighting factor for chunk in x13 plane
!   AreaFraction3: ('Area_Fraction_3') real, optional, weighting factor for chunk in x12 plane
!   Fey1:         ('Flux_energy_poynting_1) real 3D array,
!                     resistivity part of Poynting flux in x1 direction
!   Fey2:         ('Flux_energy_poynting_2) real 3D array,
!                     resistivity part of Poynting flux in x2 direction
!   Fey3:         ('Flux_energy_poynting_3) real 3D array,
!                     resistivity part of Poynting flux in x3 direction
!
! INPUT/OUTPUT:
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   weight:       ('weight') real 2D array with column weights
!
! ROUTINES:
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TODO:
!   Spherical mean.
!
! MODIFICATION HISTORY:
!   2010-12-16 (W.S.) First version
!   2017-08-19 (B.F. Lyon) Bugfix: Use averaging routine appropriate for each direction;
!                     account for new datx type in averaging of quantities
!   2017-08-23 (B.F.) Compute AreaFractions from local and global quantities
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_dat_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real,                      intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2, &
                model%m3  :model%n3), &
                           intent(in)   :: Fey1
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2+1, &
                model%m3  :model%n3), &
                           intent(in)   :: Fey2
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3+1), &
                           intent(in)   :: Fey3
type(dat_type),            intent(inout):: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, di1, di2, di3
real                                    :: AreaFraction12, AreaFraction13, AreaFraction23
real   , dimension(model%m1:model%n1, &
                   model%m2:model%n2 )  :: weight12
real   , dimension(model%m1:model%n1, &
                   model%m3:model%n3 )  :: weight13
real   , dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: weight23
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Compute averages ---
!
if ((wg%hyd%n(1) > wg%hyd%m(1)) .and. (dat%x23%na > 0) .and. (dat%x23%i_fey > 0)) then
  AreaFraction23=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m2, n2, m3, n3, model%xb2(m2:n2+1), model%xb3(m3:n3+1), AreaFraction23, &
                        weight23)
  !
  if (n1 < wg%hyd%n(1)) then
    di1=0
  else
    di1=1
  endif
  call rhd_mean_x23mean(Fey1(m1:n1+di1,:,:), dat%x23%d(m1:n1+di1,dat%x23%i_fey), weight23)
endif
!
if ((wg%hyd%n(2) > wg%hyd%m(2)) .and. (dat%x13%na > 0) .and. (dat%x13%i_fey > 0)) then
  AreaFraction13=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m1, n1, m3, n3, model%xb1(m1:n1+1), model%xb3(m3:n3+1), AreaFraction13, &
                        weight13)
  !
  if (n2 < wg%hyd%n(2)) then
    di2=0
  else
    di2=1
  endif
  call rhd_mean_x13mean(Fey2(:,m2:n2+di2,:), dat%x13%d(m2:n2+di2,dat%x13%i_fey), weight13)
endif
!
if ((wg%hyd%n(3) > wg%hyd%m(3)) .and. (dat%x12%na > 0) .and. (dat%x12%i_fey > 0)) then
  AreaFraction12=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
  call rhd_mean_aweight(m1, n1, m2, n2, model%xb1(m1:n1+1), model%xb2(m2:n2+1), AreaFraction12, &
                        weight12)
  !
  if (n3 < wg%hyd%n(3)) then
    di3=0
  else
    di3=1
  endif
  call rhd_mean_x12mean(Fey3(:,:,m3:n3+di3), dat%x12%d(m3:n3+di3,dat%x12%i_fey), weight12)
endif
!
end subroutine rhd_mhd_Res3Doutput


!----------*******************----------------------------------------------------------------------
subroutine rhd_mhd_Roe1Doutput(model, dim, AreaFraction, OutRadialC_flag, &
                               P, T, s, Gamma_1, Gamma_3, dTdei, dPdei, &
                               phic_grav, phib_grav, &
                               flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                               flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                               flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                               datx, datr, datc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Roe1Doutput ('rhd_magneto-hydrodynamics_Roe_1D_output')
!
! PURPOSE:
!   Extract some averaged quantities.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Roe1Doutput(model, dim, AreaFraction, OutRadialC_flag, &
!                            P, T, s, Gamma_1, Gamma_3, dTdei, dPdei, &
!                            phic_grav, phib_grav, &
!                            flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
!                            flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
!                            flux_rhov1B, flux_rhov2B, flux_rhov3B, &
!                            datx=datx, datr=datr, datc=datc)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!   dim:          ('dimension') integer, actual dimension the first index of 3D arrays refers to
!   AreaFraction: ('Area_Fraction') real, weighting factor for the current model slice
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, flag that determines if
!                 volume averaging of cell centered quantities is performed
!                 (it has only to be done at most once in a hydro/mhd substep)
!   P:            ('Pressure') real 3D array, pressure on model grid
!   T:            ('Temperature') real 3D array, temperature on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   Gamma_3:      ('Gamma_3') real 3D array, 3rd adiabatic coefficient on model grid
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array, temperature derivative
!                 at constant density on model grid
!   dPdei:        ('delta_Pressure_delta_energy_internal') real 3D array, pressure derivative
!                 at constant density on model grid
!   phic_grav:    ('phi_central_gravity) real 3D array, gravitational potential, cell centers
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!
! INPUT/OUTPUT:
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for add. quantities
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   weight:       ('weight') 2D real array with weights
!   x123weight:   ('x_123_weight') 3D real array with weights
!   nbindex:      ('number_boundary_index') 2D integer array
!   bindex:       ('boundary_index') 3D integer array
!   cindex:       ('center_index') 3D integer array
!
! ROUTINES:
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_c1Index:      Produce index array for azimuthal averaging over cylinders
!   rhd_mean_c1VecProj:    Project vector on radial vector perpendicular to x1 axis
!   rhd_mean_ccamean:      Average cell-centered quantity over cylinders (azimuthal mean)
!   rhd_mean_rb23mean:     Average boundary-centered vector quantity over spheres in asymmetric box
!   rhd_mean_rb23absmean:  Average boundary-centered scalar quantity over spheres in asymmetric box
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_VecProj:      Project vector on radial ray
!   rhd_mean_weight:       Produce weight array for horizontal averaging
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The input 3D real arrays and some derived quantities are averaged over planes
!   and spheres (if requested) and put into datx and datr, respectively.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-18 (B.F. Uppsala) Put into own module
!   2001-01-26 (B.F.) Bugfix for computation of feg: term 1/dxc1 removed
!   2001-03-18 (B.F.) Radial case: rhov3, frhov3
!   2010-03-11 (B.F.) Entries p2_xmean, rho2_xmean, s2_xmean added
!   2010-12-17 (W.S. Exeter) Bh_xmean, Bhrms_xmean added
!   2012-07-27 (B.F. Lyon) Move scale%length into prop%length
!   2012-09-19 (B.F.) Compute v1v1v1_xmean and v1v1v1v1_xmean
!   2012-11-21 (W.S.) Bugfix: Indices in call for calculation of mean values of fluxes corrected
!   2013-03-06 (B.F.) Bugfix: Don't count outside-boundary fluxes twice; new parameter dim
!   2013-07-08 (B.F.) Add means of moment of inertia and angular momentum over spheres
!   2013-08-20 (B.F.) Add means of several momentum fluxes
!   2017-08-21 (B.F.) Compute temperature averages if no radiation transport is done;
!                     make routine more similar to HD counterpart;
!                     account for new datx format
!   2017-08-23 (B.F.) Compute AreaFraction from local and global quantities
!   2017-09-10 (B.F.) Change to averaging routines for asymmetric boxes for new datx type
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!   2018-06-04 (B.F. Uppsala) Add routines for azimuthal means
!   2018-06-13 (B.F.) Copy additional averaging calls from rhd_hyd_Roe1Doutput; compute rho_til
!   2018-06-21 (B.F. Kiel) Add mean of frho**2
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_mean_module
use rhd_dat_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
integer,                   intent(in)   :: dim
real,                      intent(in)   :: AreaFraction
logical,                   intent(in)   :: OutRadialC_flag
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3), &
                           intent(in)   :: P, T, s, &
                                           Gamma_1, Gamma_3, dTdei, dPdei, &
                                           phic_grav
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3), &
                           intent(in)   :: phib_grav
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2,&
                model%m3:model%n3), &
                           intent(in)   :: flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                                           flux_rhov1B, flux_rhov2B, flux_rhov3B
type(datx_type), optional, intent(inout):: datx, datr
type(datc_type), optional, intent(inout):: datc
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, iquc, i1, i2, i3, m, n, di1
real                                    :: AreaFraction0
real   , dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: weight
real   , dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: x123weight, vr, va, qr
real   , dimension(model%m1:model%n1+1, &
                   model%m2:model%n2  , &
                   model%m3:model%n3   ):: qrb, rho_til
integer, dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: nbindex
integer, dimension(6, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: nb6index
integer, dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: cindex
integer, dimension(:,:)  , allocatable  :: ccindex
integer, dimension(:,:,:), allocatable  :: bindex
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Compute boundary-centered density in a Roe-style ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Roe average of density (use two sqrt because rho**2 might become small) ---
      rho_til( i1,i2,i3)=sqrt(model%rho(i1-1,i2,i3)) * sqrt(model%rho(i1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Compute averages ---
!
if (merge((datx%na > 0), .false., present(datx))) then
  ! === Average over horizontal planes =============================================================
  !
  if (dim == 1) then
    AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                  ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  else if (dim == 2) then
    AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                  ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  else ! (dim == 3)
    AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                  ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
  endif
  !
  ! --- Compute weight arrays ---
  call rhd_mean_weight(model, AreaFraction0, weight)
  !
  ! --- Compute individual averages: volume quantities ---
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rho)     , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_rho2)    , weight)
  call rhd_mean_x23mean(model%v1( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_v1)      , weight)
  call rhd_mean_x23mean(model%v2( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_v2)      , weight)
  call rhd_mean_x23mean(model%v3( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_v3)      , weight)
  call rhd_mean_x23mean(model%v1( m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_v1v1)    , weight)
  call rhd_mean_x23mean(model%v2( m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_v2v2)    , weight)
  call rhd_mean_x23mean(model%v3( m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_v3v3)    , weight)
  call rhd_mean_x23mean(model%v1( m1:n1,m2:n2,m3:n3)**3   , datx%d(m1:n1,datx%i_v1v1v1)  , weight)
  call rhd_mean_x23mean(model%v1( m1:n1,m2:n2,m3:n3)**4   , datx%d(m1:n1,datx%i_v1v1v1v1), weight)
  call rhd_mean_x23mean(sign(0.5,model%v1(m1:n1,m2:n2,m3:n3))+0.5 &
                                                          , datx%d(m1:n1,datx%i_v1upfrac), weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v1( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhov1)   , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v2( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhov2)   , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v3( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhov3)   , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v1( m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_rhov1v1) , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v2( m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_rhov2v2) , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v3( m1:n1,m2:n2,m3:n3)**2   , datx%d(m1:n1,datx%i_rhov3v3) , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v1( m1:n1,m2:n2,m3:n3)* &
                        model%v2( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhov1v2) , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v1( m1:n1,m2:n2,m3:n3)* &
                        model%v3( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhov1v3) , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%v2( m1:n1,m2:n2,m3:n3)* &
                        model%v3( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhov2v3) , weight)
  call rhd_mean_x23mean(model%ei( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_ei)      , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        model%ei( m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhoei)   , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)*0.5* &
                        (model%v1(m1:n1,m2:n2,m3:n3)**2 + &
                         model%v2(m1:n1,m2:n2,m3:n3)**2 + &
                         model%v3(m1:n1,m2:n2,m3:n3)**2 ) , datx%d(m1:n1,datx%i_rhoek)   , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        phic_grav(m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhoeg)   , weight)
  call rhd_mean_x23mean(P(m1:n1,m2:n2,m3:n3)              , datx%d(m1:n1,datx%i_p)       , weight)
  call rhd_mean_x23mean(P(m1:n1,m2:n2,m3:n3)**2           , datx%d(m1:n1,datx%i_p2)      , weight)
  ! --- Temperature averages are usually computed by the radiation-transport solver. If not... ---
  if ((par%OpaFile == '') .or. (par%A%radScheme == radScheme_None)) then
    call rhd_mean_x23mean(T(m1:n1,m2:n2,m3:n3)            , datx%d(m1:n1,datx%i_T)       , weight)
    call rhd_mean_x23mean(T(m1:n1,m2:n2,m3:n3)**2         , datx%d(m1:n1,datx%i_T2)      , weight)
    call rhd_mean_x23mean(T(m1:n1,m2:n2,m3:n3)**4         , datx%d(m1:n1,datx%i_T4)      , weight)
  endif
  call rhd_mean_x23mean(s(m1:n1,m2:n2,m3:n3)              , datx%d(m1:n1,datx%i_s)       , weight)
  call rhd_mean_x23mean(s(m1:n1,m2:n2,m3:n3)**2           , datx%d(m1:n1,datx%i_s2)      , weight)
  call rhd_mean_x23mean(model%rho(m1:n1,m2:n2,m3:n3)* &
                        s(        m1:n1,m2:n2,m3:n3)      , datx%d(m1:n1,datx%i_rhos)    , weight)
  call rhd_mean_x23mean(Gamma_1(m1:n1,m2:n2,m3:n3)        , datx%d(m1:n1,datx%i_Gamma1)  , weight)
  call rhd_mean_x23mean(Gamma_3(m1:n1,m2:n2,m3:n3)        , datx%d(m1:n1,datx%i_Gamma3)  , weight)
  call rhd_mean_x23mean(1.0/((Gamma_1(m1:n1,m2:n2,m3:n3)* &
                              P(m1:n1,m2:n2,m3:n3)*dTdei(m1:n1,m2:n2,m3:n3))/ &
                             (T(m1:n1,m2:n2,m3:n3)*dPdei(m1:n1,m2:n2,m3:n3))- &
                             (Gamma_3(m1:n1,m2:n2,m3:n3)-1.0)) &
                                                          , datx%d(m1:n1,datx%i_delta)   , weight)
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Additional densities ---
  if ((model%nquc > 0) .and. (datx%i_quc >0)) then
    do iquc=1,prop%nquc
      call rhd_mean_x23mean(model%quc(m1:n1,m2:n2,m3:n3,iquc) , &
                                                        datx%d(m1:n1,datx%i_quc+(iquc-1)), weight)
    end do ! iquc
  endif
!#
#   endif
!#        /* rhd_box_quc01 */
  if ((model%nBc > 0) .and. (datx%i_Bc1 > 0)) then
    call rhd_mean_x23mean(model%Bc1(m1:n1,m2:n2,m3:n3)    , datx%d(m1:n1,datx%i_Bc1)     , weight)
    call rhd_mean_x23mean(model%Bc2(m1:n1,m2:n2,m3:n3)    , datx%d(m1:n1,datx%i_Bc2)     , weight)
    call rhd_mean_x23mean(model%Bc3(m1:n1,m2:n2,m3:n3)    , datx%d(m1:n1,datx%i_Bc3)     , weight)
    call rhd_mean_x23mean(abs(model%Bc1(m1:n1,m2:n2,m3:n3)),datx%d(m1:n1,datx%i_Bc1abs)  , weight)
    call rhd_mean_x23mean(abs(model%Bc2(m1:n1,m2:n2,m3:n3)),datx%d(m1:n1,datx%i_Bc2abs)  , weight)
    call rhd_mean_x23mean(abs(model%Bc3(m1:n1,m2:n2,m3:n3)),datx%d(m1:n1,datx%i_Bc3abs)  , weight)
    call rhd_mean_x23mean(model%Bc1(m1:n1,m2:n2,m3:n3)**2 , datx%d(m1:n1,datx%i_Bc12)    , weight)
    call rhd_mean_x23mean(model%Bc2(m1:n1,m2:n2,m3:n3)**2 , datx%d(m1:n1,datx%i_Bc22)    , weight)
    call rhd_mean_x23mean(model%Bc3(m1:n1,m2:n2,m3:n3)**2 , datx%d(m1:n1,datx%i_Bc32)    , weight)
    call rhd_mean_x23mean(sqrt(model%Bc2(m1:n1,m2:n2,m3:n3)**2 + &
                               model%Bc3(m1:n1,m2:n2,m3:n3)**2  ) &
                                                          , datx%d(m1:n1,datx%i_Bh)      , weight)
    call rhd_mean_x23mean(model%Bc2(m1:n1,m2:n2,m3:n3)**2 + &
                          model%Bc3(m1:n1,m2:n2,m3:n3)**2 , datx%d(m1:n1,datx%i_Bhrms)   , weight)
  endif
  !
  ! --- Check if fluxes at right boundary should be considered ---
  if (n1 < wg%hyd%n(dim)) then
    di1=0
  else
    di1=1
  endif
  !
  ! --- Compute individual averages: fluxes ---
  call rhd_mean_x23mean(rho_til(    m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_rhotil), weight)
  call rhd_mean_x23mean(flux_rho(   m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_frho)  , weight)
  call rhd_mean_x23mean(flux_rho(   m1:n1+di1,m2:n2,m3:n3)**2, &
                                                          datx%d(m1:n1+di1,datx%i_frhofrho), weight)
  call rhd_mean_x23mean(flux_rhov1( m1:n1+di1,m2:n2,m3:n3) + &
                        flux_rhov1B(m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_frhov1), weight)
  call rhd_mean_x23mean(flux_rhov2( m1:n1+di1,m2:n2,m3:n3) + &
                        flux_rhov2B(m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_frhov2), weight)
  call rhd_mean_x23mean(flux_rhov3( m1:n1+di1,m2:n2,m3:n3) + &
                        flux_rhov3B(m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_frhov3), weight)
  call rhd_mean_x23mean(flux_rhoei( m1:n1+di1,m2:n2,m3:n3) + &
                        fep(        m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_feip)  , weight)
  call rhd_mean_x23mean(flux_rhoek( m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_fek)   , weight)
  call rhd_mean_x23mean(flux_rhoem( m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_fem)   , weight)
  call rhd_mean_x23mean(-phib_grav( m1:n1+di1,m2:n2,m3:n3)* &
                        flux_rho(   m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_feg)   , weight)
  call rhd_mean_x23mean(fep(        m1:n1+di1,m2:n2,m3:n3), datx%d(m1:n1+di1,datx%i_fep)   , weight)
endif ! (merge(datx%na > 0), .false., present(datx)))
!
! \label{f90:grav_mode__useMHD1}
if (merge(((datr%na > 0) .and. (datr%i_rhoei > 0)), .false., present(datr)) .and. &
    (par%grav_mode == 'central')                                                   ) then
  ! === Average over spheres =======================================================================
  !
  ! --- Make abbreviations ---
  m=datr%m
  n=datr%n
  !
  ! --- Allocate boundary-index array ---
  allocate(bindex(-(n-m+2):(n-m+2),m2:n2,m3:n3))
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(model, prop%length, weight, x123weight)
  !
  ! --- Compute index arrays ---
!  call rhd_mean_rindex(model, datr%d1(:,datr%i_xb), nbindex, bindex, cindex)
  call rhd_mean_a2rindex(m1,n1, m2,n2, m3,n3, &
                         model%nghost1, wg%hyd%m(dim), wg%hyd%n(dim), &
                         model%xc1, model%xc2(m2:n2), model%xc3(m3:n3), &
                         datr%d1(:,datr%i_xb), nb6index, bindex, cindex)
  !
  if (OutRadialC_flag) then
    ! --- Compute individual averages: volume quantities ---
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         1.0+0.0*model%rho(m1:n1,m2:n2,m3:n3)      , datr%d(m:n,datr%i_relvol), &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,m2:n2,m3:n3)              , datr%d(m:n,datr%i_rho)   , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,m2:n2,m3:n3)**2           , datr%d(m:n,datr%i_rho2)  , &
                         cindex, x123weight)
    !
    call rhd_mean_VecProj(model, model%v1(m1:n1,m2:n2,m3:n3), &
                                 model%v2(m1:n1,m2:n2,m3:n3), &
                                 model%v3(m1:n1,m2:n2,m3:n3), vr)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         vr                                        , datr%d(m:n,datr%i_v3)    , &
                         cindex, x123weight)
    !
    qr=vr*model%rho(m1:n1,m2:n2,m3:n3)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_rhov3) , &
                         cindex, x123weight)
    !
    qr=vr**2
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v3v3)  , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr*model%rho(m1:n1,m2:n2,m3:n3)           , datr%d(m:n,datr%i_rhov3v3), &
                         cindex, x123weight)
    !
    qr=max(model%v1(m1:n1,m2:n2,m3:n3)**2 + &
           model%v2(m1:n1,m2:n2,m3:n3)**2 + &
           model%v3(m1:n1,m2:n2,m3:n3)**2 - qr, 0.0)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v1v1)  , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr*model%rho(m1:n1,m2:n2,m3:n3)           , datr%d(m:n,datr%i_rhov1v1)  , &
                         cindex, x123weight)
    !
    qr=vr**3
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v1v1v1)  , &
                         cindex, x123weight)
    qr=vr**4
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v1v1v1v1), &
                         cindex, x123weight)
    !
    qr=sign(0.5,vr)+0.5
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v1upfrac), &
                         cindex, x123weight)
    !
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%ei(m1:n1,m2:n2,m3:n3)               , datr%d(m:n,datr%i_ei)    , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,m2:n2,m3:n3)* &
                         model%ei(m1:n1,m2:n2,m3:n3)               , datr%d(m:n,datr%i_rhoei) , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,m2:n2,m3:n3)*0.5* &
                        (model%v1( m1:n1,m2:n2,m3:n3)**2+ &
                         model%v2( m1:n1,m2:n2,m3:n3)**2+ &
                         model%v3( m1:n1,m2:n2,m3:n3)**2)           , datr%d(m:n,datr%i_rhoek) , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,m2:n2,m3:n3)* &
                         phic_grav(m1:n1,m2:n2,m3:n3)              , datr%d(m:n,datr%i_rhoeg) , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         P(m1:n1,m2:n2,m3:n3)                      , datr%d(m:n,datr%i_p)     , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         P(m1:n1,m2:n2,m3:n3)**2                   , datr%d(m:n,datr%i_p2)    , &
                         cindex, x123weight)
    ! --- Temperature averages are usually computed by the radiation-transport solver. If not... -
    if ((par%OpaFile == '') .or. (par%A%radScheme == radScheme_None)) then
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         T(m1:n1,m2:n2,m3:n3)                      , datr%d(m:n,datr%i_T)     , &
                         cindex, x123weight)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         T(m1:n1,m2:n2,m3:n3)**2                   , datr%d(m:n,datr%i_T2)    , &
                         cindex, x123weight)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         T(m1:n1,m2:n2,m3:n3)**4                   , datr%d(m:n,datr%i_T4)    , &
                         cindex, x123weight)
    endif
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         s(m1:n1,m2:n2,m3:n3)                      , datr%d(m:n,datr%i_s)     , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         s(m1:n1,m2:n2,m3:n3)**2                   , datr%d(m:n,datr%i_s2)    , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,m2:n2,m3:n3)* &
                         s(        m1:n1,m2:n2,m3:n3)              , datr%d(m:n,datr%i_rhos)  , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         Gamma_1(m1:n1,m2:n2,m3:n3)                , datr%d(m:n,datr%i_Gamma1), &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         Gamma_3(m1:n1,m2:n2,m3:n3)                , datr%d(m:n,datr%i_Gamma3), &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         1.0/((Gamma_1(m1:n1,m2:n2,m3:n3)*P(m1:n1,m2:n2,m3:n3)* &
                               dTdei(m1:n1,m2:n2,m3:n3))/ &
                              (T(m1:n1,m2:n2,m3:n3)*dPdei(m1:n1,m2:n2,m3:n3))- &
                              (Gamma_3(m1:n1,m2:n2,m3:n3)-1.0))    , datr%d(m:n,datr%i_delta) , &
                         cindex, x123weight)
    !
    if (dim == 1) then
      ! --- Compute and average moment of inertia ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            qr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc1(i1)**2+model%xc2(i2)**2)
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_MomIne3), &
                           cindex, x123weight)
      !
      ! --- Compute and average angular momentum ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            qr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc1(i1)*model%v2(i1,i2,i3)- &
                                                model%xc2(i2)*model%v1(i1,i2,i3)   )
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_AngMom3), &
                           cindex, x123weight)
    else if (dim == 2) then
      ! --- Compute and average moment of inertia (transpose 2 -> 1) ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            qr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc1(i1)**2+model%xc2(i2)**2)
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_MomIne3), &
                           cindex, x123weight)
      !
      ! --- Compute and average angular momentum ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            qr(i1,i2,i3)=-model%rho(i1,i2,i3) * (model%xc1(i1)*model%v2(i1,i2,i3)- &
                                                 model%xc2(i2)*model%v1(i1,i2,i3)   )
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_AngMom3), &
                           cindex, x123weight)
    else if (dim == 3) then
      ! --- Compute and average moment of inertia (transpose 3 -> 1) ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            qr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc3(i3)**2+model%xc2(i2)**2)
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_MomIne3), &
                           cindex, x123weight)
      !
      ! --- Compute and average angular momentum ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            qr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc3(i3)*model%v2(i1,i2,i3)- &
                                                model%xc2(i2)*model%v3(i1,i2,i3)   )
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_AngMom3), &
                           cindex, x123weight)
    endif ! (dim)
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    if ((model%nquc > 0) .and. (datr%i_quc > 0)) then
      do iquc=1,prop%nquc
        call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                             model%quc(m1:n1,m2:n2,m3:n3,iquc)  , datr%d(m:n,datr%i_quc+(iquc-1)), &
                             cindex, x123weight)
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
    !
    if ((model%nBc > 0) .and. (datr%i_Bc1 > 0)) then
      call rhd_mean_VecProj(model, model%Bc1(m1:n1,m2:n2,m3:n3), &
                                   model%Bc2(m1:n1,m2:n2,m3:n3), &
                                   model%Bc3(m1:n1,m2:n2,m3:n3), qr)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc3)    , &
                           cindex, x123weight)
      !
      qr=abs(qr)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc3abs) , &
                           cindex, x123weight)
      !
      qr=qr**2
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc32)   , &
                           cindex, x123weight)
      !
      qr=max(model%Bc1(m1:n1,m2:n2,m3:n3)**2 + &
             model%Bc2(m1:n1,m2:n2,m3:n3)**2 + &
             model%Bc3(m1:n1,m2:n2,m3:n3)**2 - qr, &
             0.0)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc12)   , &
                           cindex, x123weight)
    endif ! ((model%nBc > 0) .and. (datr%i_Bc1 >))
  endif ! (OutRadialC_flag)
  !
  ! --- Compute individual averages: boundary-centered density, fluxes, waves ---
  !
  ! --- Compute surface areas ---
  call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       0.0*flux_rho(m1:n1+1,m2:n2,m3:n3)           , datr%d(:,datr%i_relsurf), &
                       nb6index, bindex, weight, unity=.true.)
  !
  if (datr%i_runiflux > 0) then
    call rhd_mean_VecbProjUnity(model, qrb)
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       qrb                                         , datr%d(:,datr%i_runiflux), &
                       nb6index, bindex, weight)
  endif
  !
  ! --- Average boundary-centered density ---
  call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       rho_til(m1:n1+1,m2:n2,m3:n3)                , datr%d(:,datr%i_rhotil)  , &
                       nb6index, bindex, weight)
  !
  ! --- Average fluxes ---
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       flux_rho(m1:n1+1,m2:n2,m3:n3)               , datr%d(:,datr%i_frho)   , &
                       nb6index, bindex, weight)
  call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       flux_rho(m1:n1+1,m2:n2,m3:n3)**2            , datr%d(:,datr%i_frhofrho), &
                       nb6index, bindex, weight)
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       flux_rhov1(m1:n1+1,m2:n2,m3:n3) + &
                       flux_rhov1B(m1:n1+1,m2:n2,m3:n3)            , datr%d(:,datr%i_frhov3) , &
                       nb6index, bindex, weight)
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       flux_rhoei(m1:n1+1,m2:n2,m3:n3) + &
                       fep(m1:n1+1,m2:n2,m3:n3)                    , datr%d(:,datr%i_feip)   , &
                       nb6index, bindex, weight)
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       flux_rhoek(m1:n1+1,m2:n2,m3:n3)             , datr%d(:,datr%i_fek)    , &
                       nb6index, bindex, weight)
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       flux_rhoem(m1:n1+1,m2:n2,m3:n3)             , datr%d(:,datr%i_fem)    , &
                       nb6index, bindex, weight)
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       -phib_grav(m1:n1+1,m2:n2,m3:n3)* &
                       flux_rho(  m1:n1+1,m2:n2,m3:n3)             , datr%d(:,datr%i_feg)    , &
                       nb6index, bindex, weight)
  call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                       fep(m1:n1+1,m2:n2,m3:n3)                    , datr%d(:,datr%i_fep)    , &
                       nb6index, bindex, weight)
  !
  ! --- Compute wave amplitudes ---
  if ((datr%nWave > 0) .and. (datr%i_frhowave > 0)) then
    call rhd_mean_rb23shmean(model, m1,n1, m2,n2, m3,n3, &
                             datr%m,datr%n, datr%na, datr%nWave, datr%i_frhowave, dim, &
                             flux_rho(m1:n1+1,m2:n2,m3:n3), datr%d, &
                             nb6index, bindex, weight)
  endif
  !
  ! --- Deallocate boundary-index array ---
  deallocate(bindex)
  !
endif ! ((present(datr)) .and. ...)
!
if (merge(associated(datc%d), .false., present(datc)) .and. &
    (par%grav_mode == 'central') .and. (dim == 3)            ) then
  ! === Average over cylinders (azimuthally) =======================================================
  !
  ! --- Allocate column-index array ---
  allocate(ccindex(m2:n2,m3:n3))
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(model, prop%length, weight, x123weight)
  !
  ! --- Compute index array ---
  call rhd_mean_c1Index(model, datc%d1(:,datc%i_xb), ccindex)
  !
  ! --- Compute individual averages: volume quantities ---
  !
  ! --- Relative volume (to get a proper weight for each radius bin) ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_relvol, &
         1.0+0.0*model%rho(m1:n1,m2:n2,m3:n3))
  !
  ! --- Basic thermodynamic quantities ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rho, &
         model%rho(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_ei, &
         model%ei(m1:n1,m2:n2,m3:n3))
  !
  ! --- Additional thermodynamic quantities ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhoei, &
         model%rho(m1:n1,m2:n2,m3:n3)*model%ei(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_T, &
         T(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_P, &
         P(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_Gamma1, &
         Gamma_1(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_Gamma3, &
         Gamma_3(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_delta, &
         1.0/((Gamma_1(m1:n1,m2:n2,m3:n3)*P(m1:n1,m2:n2,m3:n3)*dTdei(m1:n1,m2:n2,m3:n3))/ &
              (T(m1:n1,m2:n2,m3:n3)*dPdei(m1:n1,m2:n2,m3:n3))-(Gamma_3(m1:n1,m2:n2,m3:n3)-1.0)))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_s, &
         s(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_ss, &
         s(m1:n1,m2:n2,m3:n3)**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhos, &
         model%rho(m1:n1,m2:n2,m3:n3)*s(m1:n1,m2:n2,m3:n3))
  !
  ! --- Velocities ---
  call rhd_mean_c1VecProj(model, model%v2(m1:n1,m2:n2,m3:n3),  model%v3(m1:n1,m2:n2,m3:n3), vr, va)
  !
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v1, &
         vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v1v1, &
         vr**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1, &
         model%rho(m1:n1,m2:n2,m3:n3)*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1v1, &
         model%rho(m1:n1,m2:n2,m3:n3)*vr**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1v2, &
         model%rho(m1:n1,m2:n2,m3:n3)*vr*va)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1v3, &
         model%rho(m1:n1,m2:n2,m3:n3)*vr*model%v1(m1:n1,m2:n2,m3:n3))
  !
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v2, &
         va)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v2v2, &
         va**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov2, &
         model%rho(m1:n1,m2:n2,m3:n3)*va)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov2v2, &
         model%rho(m1:n1,m2:n2,m3:n3)*va**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov2v3, &
         model%rho(m1:n1,m2:n2,m3:n3)*va*model%v1(m1:n1,m2:n2,m3:n3))
  !
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v3, &
         model%v1(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v3v3, &
         model%v1(m1:n1,m2:n2,m3:n3)**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov3, &
         model%rho(m1:n1,m2:n2,m3:n3)*model%v1(m1:n1,m2:n2,m3:n3))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov3v3, &
         model%rho(m1:n1,m2:n2,m3:n3)*model%v1(m1:n1,m2:n2,m3:n3)**2)
  !
  ! --- Energy fluxes ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fep1, &
         P(m1:n1,m2:n2,m3:n3)*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fep3, &
         P(m1:n1,m2:n2,m3:n3)*model%v1(m1:n1,m2:n2,m3:n3))
  qr=model%rho(m1:n1,m2:n2,m3:n3)*model%ei(m1:n1,m2:n2,m3:n3)+P(m1:n1,m2:n2,m3:n3)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_feip1, &
         qr*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_feip3, &
         qr*model%v1(m1:n1,m2:n2,m3:n3))
  qr=0.5*model%rho(m1:n1,m2:n2,m3:n3) * (model%v1(m1:n1,m2:n2,m3:n3)**2 + &
                                         model%v2(m1:n1,m2:n2,m3:n3)**2 + &
                                         model%v3(m1:n1,m2:n2,m3:n3)**2    )
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fek1, &
         qr*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fek3, &
         qr*model%v1(m1:n1,m2:n2,m3:n3))
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Additional densities and corresponding fluxes (only computed with quc*v) ---
  if ((model%nquc > 0) .and. (datc%i_quc > 0)) then
    do iquc=1,prop%nquc
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_quc-1+iquc, &
             model%quc(m1:n1,:,:,iquc))
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_fquc1-1+iquc, &
             model%quc(m1:n1,:,:,iquc)*vr)
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_fquc2-1+iquc, &
             model%quc(m1:n1,:,:,iquc)*va)
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_fquc3-1+iquc, &
             model%quc(m1:n1,:,:,iquc)*model%v1(m1:n1,m2:n2,m3:n3))
    end do ! iquc
  endif
!#
# endif
!#      /* rhd_box_quc01 */
  !
  if ((model%nBc > 0) .and. (datr%i_Bc1 > 0)) then
    call rhd_mean_c1VecProj(model, model%Bc2(m1:n1,m2:n2,m3:n3),model%Bc3(m1:n1,m2:n2,m3:n3), vr,va)
    !
    call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
           ccindex, x123weight, datc%d, datc%i_Bc1, &
           vr)
    call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
           ccindex, x123weight, datc%d, datc%i_Bc2, &
           va)
    call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
           ccindex, x123weight, datc%d, datc%i_Bc3, &
           model%Bc1(m1:n1,m2:n2,m3:n3))
  endif ! ((model%nBc > 0) .and. (datr%i_Bc1 >))
  !
  ! --- Deallocate column-index array ---
  deallocate(ccindex)
  !
endif ! ((present(datc)) .and. ...)
!
end subroutine rhd_mhd_Roe1Doutput


!----------*******************----------------------------------------------------------------------
subroutine rhd_mhd_BboutputOld(model, action, dtime, &
                               dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Bboutput ('rhd_magneto-hydrodynamics_B_boundary_output')
!
! PURPOSE:
!   Extract averages of the boundary-centered magnetic field.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Bboutput(model, action, dtime, &
!                         dat)
!
! INPUT:
!   model:        ('model') derived type, model data
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step
!
! INPUT/OUTPUT:
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The structure dat has to be present and initialized.
!
! PROCEDURE:
!
! EXAMPLE:
!
! TODO:
!   OpenMP-parallelize mean-data generation
!
! MODIFICATION HISTORY:
!   2002-11-05 (B.F.) First version
!   2010-12-17 (W.S.) Output of some averages to meanfile: new optional parameters output,
!                     outradialc, AreaFraction1, AreaFraction2, AreaFraction3, dat
!   2011-01-05 (W.S.) Quadratic interpolation of fluxes for the computation of the electric field
!   2012-11-28 (W.S.) Bugfix: Error in computation of averages for meanfile fixed
!   2013-03-05 (B.F.) Parameter outradialc removed, more timing markers
!   2017-08-19 (B.F.) Account for new datx type in averaging of quantities
!   2017-08-23 (B.F.) Compute AreaFractions from local and global quantities;
!                     Bugfix (for old averaging scheme): actually process AreaFraction20
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!   2018-10-25 (B.F.) Remove optional input parameters AreaFraction1, AreaFraction2, AreaFraction3;
!                     Extract average calculations from rhd_mhd_Constrain
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_dat_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
type(dat_type),  optional, intent(inout):: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, di1, di2, di3
real                                    :: AreaFraction12, AreaFraction13, AreaFraction23
real   , dimension(model%m1:model%n1, &
                   model%m2:model%n2 )  :: weight12
real   , dimension(model%m1:model%n1, &
                   model%m3:model%n3 )  :: weight13
real   , dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: weight23
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Output of additional data ---
!
if ((wg%hyd%n(1) > wg%hyd%m(1)) .and. (dat%x23%i_Bb > 0)) then
  AreaFraction23=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m2,n2, m3,n3, model%xb2(m2:n2+1), model%xb3(m3:n3+1), AreaFraction23, &
                        weight23)
  !
  if (n1 < wg%hyd%n(1)) then
    di1=0
  else
    di1=1
  endif
  call rhd_mean_x23mean(    model%Bb1(m1:n1+di1,:,:)   , dat%x23%d(m1:n1+di1,dat%x23%i_Bb   ), &
                        weight23)
  call rhd_mean_x23mean(abs(model%Bb1(m1:n1+di1,:,:))  , dat%x23%d(m1:n1+di1,dat%x23%i_Bbabs), &
                        weight23)
  call rhd_mean_x23mean(    model%Bb1(m1:n1+di1,:,:)**2, dat%x23%d(m1:n1+di1,dat%x23%i_Bb2  ), &
                        weight23)
endif
!
if ((wg%hyd%n(2) > wg%hyd%m(2)) .and. (dat%x13%i_Bb > 0)) then
  AreaFraction13=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m1,n1, m3,n3, model%xb1(m1:n1+1), model%xb3(m3:n3+1), AreaFraction13, &
                        weight13)
  !
  if (n2 < wg%hyd%n(2)) then
    di2=0
  else
    di2=1
  endif
  call rhd_mean_x13mean(    model%Bb2(:,m2:n2+di2,:)   , dat%x13%d(m2:n2+di2,dat%x13%i_Bb   ), &
                        weight13)
  call rhd_mean_x13mean(abs(model%Bb2(:,m2:n2+di2,:))  , dat%x13%d(m2:n2+di2,dat%x13%i_Bbabs), &
                        weight13)
  call rhd_mean_x13mean(    model%Bb2(:,m2:n2+di2,:)**2, dat%x13%d(m2:n2+di2,dat%x13%i_Bb2  ), &
                        weight13)
endif
!
if ((wg%hyd%n(3) > wg%hyd%m(3)) .and. (dat%x12%i_Bb > 0)) then
  AreaFraction12=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
  call rhd_mean_aweight(m1,n1, m2,n2, model%xb1(m1:n1+1), model%xb2(m2:n2+1), AreaFraction12, &
                        weight12)
  !
  if (n3 < wg%hyd%n(3)) then
    di3=0
  else
    di3=1
  endif
  call rhd_mean_x12mean(    model%Bb3(:,:,m3:n3+di3)   , dat%x12%d(m3:n3+di3,dat%x12%i_Bb   ), &
                        weight12)
  call rhd_mean_x12mean(abs(model%Bb3(:,:,m3:n3+di3))  , dat%x12%d(m3:n3+di3,dat%x12%i_Bbabs), &
                        weight12)
  call rhd_mean_x12mean(    model%Bb3(:,:,m3:n3+di3)**2, dat%x12%d(m3:n3+di3,dat%x12%i_Bb2  ), &
                        weight12)
endif
!
end subroutine rhd_mhd_BboutputOld


!----------****************-------------------------------------------------------------------------
subroutine rhd_mhd_Bboutput(model, action, dtime, &
                            dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Bboutput ('rhd_magneto-hydrodynamics_B_boundary_output')
!
! PURPOSE:
!   Extract averages of the boundary-centered magnetic field.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Bboutput(model, action, dtime, &
!                         dat)
!
! INPUT:
!   model:        ('model') derived type, model data
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step
!
! INPUT/OUTPUT:
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The structure dat has to be present and initialized.
!
! PROCEDURE:
!
! EXAMPLE:
!
! TODO:
!   OpenMP-parallelize mean-data generation
!
! MODIFICATION HISTORY:
!   2002-11-05 (B.F.) First version
!   2010-12-17 (W.S.) Output of some averages to meanfile: new optional parameters output,
!                     outradialc, AreaFraction1, AreaFraction2, AreaFraction3, dat
!   2011-01-05 (W.S.) Quadratic interpolation of fluxes for the computation of the electric field
!   2012-11-28 (W.S.) Bugfix: Error in computation of averages for meanfile fixed
!   2013-03-05 (B.F.) Parameter outradialc removed, more timing markers
!   2017-08-19 (B.F.) Account for new datx type in averaging of quantities
!   2017-08-23 (B.F.) Compute AreaFractions from local and global quantities;
!                     Bugfix (for old averaging scheme): actually process AreaFraction20
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!   2018-10-25 (B.F.) Remove optional input parameters AreaFraction1, AreaFraction2, AreaFraction3;
!                     Extract average calculations from rhd_mhd_Constrain
!   2018-10-26 (B.F.) Compute (non-)radial wave amplitudes of magnetic field
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_dat_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
type(dat_type),  optional, intent(inout):: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, i, m, n, di1, di2, di3
real                                    :: AreaFraction, dtimerel
real   , dimension(  :,:), pointer      :: weight
real   , dimension(:,:,:), pointer      :: x123weight, Bb_tmp
integer, dimension(  :,:), pointer      :: nbindex, ccindex
integer, dimension(:,:,:), pointer      :: bindex, cindex, nb6index
type(box_type)                          :: slice
type(datx_type)                         :: datx_thread, datr_thread
!---------------------------------------------------------------------------------------------------
!
dtimerel=1.0
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
  ! --- Make abbreviations ---
  m=dat%r%m
  n=dat%r%n
endif
!
  if (dat%x23%na > 0) then
    ! === Average over x2-x3 planes and spherical surfaces =========================================
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(datx_thread, datr_thread, weight, slice, AreaFraction, i, &
    !$OMP           x123weight, cindex, bindex, nb6index)
    !
    call rhd_datx_Make(datx_thread, dat%x23%m, dat%x23%n, dat%x23%nWave)
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Make(datr_thread, dat%r%m, dat%r%n, dat%r%nWave, radial_flag=.true.)
    endif
    !
    !$OMP DO
    do i2=m2,n2
      ! --- Allocate temporary arrays ---
      allocate(weight(           i2:i2, m3:n3))
      !
      ! --- Create temporary box (only with axes filled) ---
      call rhd_box_Make(slice, m1=m1, n1=n1, m2=i2, n2=i2, m3=m3, n3=n3)
      call rhd_box_Copy(model, m1, n1, i2, i2, m3, n3, &
                        slice, m1,     i2,     m3, &
                        trans_mode=1, bulk_mode=0)
      !
      ! --- Compute weights for horizontal averaging ---
      AreaFraction=dtimerel * &
                   ((model%xb2(i2+1)-model%xb2(i2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                   ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
      call rhd_mean_weight(slice, AreaFraction, weight)
      !
      ! --- Check if right boundary values should be ignored or included ---
      if (n1 < wg%hyd%n(1)) then
        di1=0
      else
        di1=1
      endif
      !
      ! --- Average horizontally ---
      i=dat%x23%i_Bb
      if (i > 0) &
        call rhd_mean_x23mean(    model%Bb1(m1:n1+di1,i2:i2,m3:n3)   , datx_thread%d(m1:n1+di1,i), &
                              weight)
      i=dat%x23%i_Bbabs
      if (i > 0) &
        call rhd_mean_x23mean(abs(model%Bb1(m1:n1+di1,i2:i2,m3:n3))  , datx_thread%d(m1:n1+di1,i), &
                              weight)
      i=dat%x23%i_Bb2
      if (i > 0) &
        call rhd_mean_x23mean(    model%Bb1(m1:n1+di1,i2:i2,m3:n3)**2, datx_thread%d(m1:n1+di1,i), &
                              weight)
      !
      if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
        ! --- Average over spheres ---
        !
        ! --- Allocate index and gravity array ---
        allocate(x123weight(  m1:n1     , i2:i2, m3:n3), &
                 cindex(      m1:n1     , i2:i2, m3:n3), &
                 bindex(-(n-m+2):(n-m+2), i2:i2, m3:n3), &
                 nb6index(        6     , i2:i2, m3:n3)   )
        !
        ! --- Compute weight arrays ---
        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtimerel)
        !
        ! --- Generate index arrays ---
        call rhd_mean_a2rindex(m1,n1, i2,i2, m3,n3, &
                               model%nghost1, wg%hyd%m(1), wg%hyd%n(1), &
                               model%xc1, model%xc2(i2:i2), model%xc3(m3:n3), &
                               dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
        !
        ! --- Compute averages of Bb over spheres ---
        i=dat%r%i_Bb
        if (i > 0) &
          call rhd_mean_rb23mean(   m1,n1, i2,i2, m3,n3, (n-m+2), &
                                        model%Bb1(m1:n1+1,i2:i2,m3:n3)   , datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        i=dat%r%i_Bbabs
        if (i > 0) &
          call rhd_mean_rb23absmean(m1,n1, i2,i2, m3,n3, (n-m+2), &
                                    abs(model%Bb1(m1:n1+1,i2:i2,m3:n3))  , datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        i=dat%r%i_Bb2
        if (i > 0) &
          call rhd_mean_rb23absmean(m1,n1, i2,i2, m3,n3, (n-m+2), &
                                        model%Bb1(m1:n1+1,i2:i2,m3:n3)**2, datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        !
        ! --- Compute wave amplitudes ---
        if ((dat%r%nWave > 0) .and. (dat%r%i_Bbwave > 0)) then
          call rhd_mean_rb23shmean(model, m1,n1, m2,n2, m3,n3, &
                                   dat%r%m,dat%r%n, dat%r%na, dat%r%nWave, dat%r%i_Bbwave, 1, &
                                   model%Bb1(m1:n1+1,m2:n2,m3:n3), dat%r%d, &
                                   nb6index, bindex, weight)
        endif
        !
        ! --- Deallocate temporary arrays ---
        deallocate(x123weight, cindex, bindex, nb6index)
      endif ! ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0))
      !
      ! --- Delete temporary box ---
      call rhd_box_Delete(slice)
      !
      ! --- Deallocate temporary arrays ---
      deallocate(weight)
    end do ! i2
    !$OMP END DO
    !
    !$OMP CRITICAL (X23MEAN)
    call rhd_datx_Add(dat%x23, datx_thread)
    call rhd_datx_Delete(datx_thread)
    !
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Add(dat%r, datr_thread)
      call rhd_datx_Delete(datr_thread)
    endif
    !$OMP END CRITICAL (X23MEAN)
    !
    !$OMP END PARALLEL
    !
  endif ! (datx23%na > 0)
  !
  if (dat%x13%na > 0) then
    ! === Average over x1-x3 planes and over spherical surfaces ====================================
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(datx_thread, datr_thread, weight, slice, AreaFraction, i, &
    !$OMP           x123weight, cindex, bindex, nb6index, &
    !$OMP           Bb_tmp)
    !
    call rhd_datx_Make(datx_thread, dat%x13%m, dat%x13%n, dat%x13%nWave)
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Make(datr_thread, dat%r%m, dat%r%n, dat%r%nWave, radial_flag=.true.)
    endif
    !
    !$OMP DO
    do i1=m1,n1
      ! --- Allocate temporary arrays ---
      allocate(weight(         i1:i1, m3:n3  ), &
               Bb_tmp(n2-m2+2, i1:i1, n3-m3+1)   )
      !
      ! --- Create temporary box (only with axes filled) ---
      call rhd_box_Make(slice, m1=m2, n1=n2, m2=i1, n2=i1, m3=m3, n3=n3)
      call rhd_box_Copy(model, i1, i1, m2, n2, m3, n3, &
                        slice, m2,     i1,     m3, &
                        trans_mode=2, bulk_mode=0)
      !
      ! --- Compute weights for horizontal averaging ---
      AreaFraction=dtimerel * &
                   ((model%xb1(i1+1)-model%xb1(i1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                   ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
      call rhd_mean_weight(slice, AreaFraction, weight)
      !
      ! --- Reshape magnetic-field array ---
      Bb_tmp=reshape(model%Bb2(i1,m2:n2+1,m3:n3), (/n2-m2+2,1,n3-m3+1/))
      !
      ! --- Check if right boundary values should be ignored or included ---
      if (n2 < wg%hyd%n(2)) then
        di2=0
      else
        di2=1
      endif
      !
      ! --- Average horizontally ---
      i=dat%x13%i_Bb
      if (i > 0) &
        call rhd_mean_x23mean(    Bb_tmp(m2:n2+di2,i1:i1,m3:m3)   , datx_thread%d(m2:n2+di2,i), &
                                 weight)
      i=dat%x13%i_Bbabs
      if (i > 0) &
        call rhd_mean_x23mean(abs(Bb_tmp(m2:n2+di2,i1:i1,m3:m3))  , datx_thread%d(m2:n2+di2,i), &
                                 weight)
      i=dat%x13%i_Bb2
      if (i > 0) &
        call rhd_mean_x23mean(    Bb_tmp(m2:n2+di2,i1:i1,m3:m3)**2, datx_thread%d(m2:n2+di2,i), &
                                 weight)
      !
      if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
        ! --- Average over spheres ---
        !
        ! --- Allocate index array ---
        allocate(x123weight(  m2:n2     , i1:i1, m3:n3), &
                 cindex(      m2:n2     , i1:i1, m3:n3), &
                 bindex(-(n-m+2):(n-m+2), i1:i1, m3:n3), &
                 nb6index(       6      , i1:i1, m3:n3)   )
        !
        ! --- Compute weight arrays ---
        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtimerel)
        !
        ! --- Generate index arrays ---
        call rhd_mean_a2rindex(m2,n2, i1,i1, m3,n3, &
                               1, wg%hyd%m(2), wg%hyd%n(2), &
                               model%xc2(m2-1:n2+1), model%xc1(i1:i1), model%xc3(m3:n3), &
                               dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
        !
        ! --- Compute averages of Bb over spheres ---
        i=dat%r%i_Bb
        if (i > 0) &
          call rhd_mean_rb23mean(   m2,n2,i1,i1,m3,n3, (n-m+2), &
                                        Bb_tmp   , datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        i=dat%r%i_Bbabs
        if (i > 0) &
          call rhd_mean_rb23absmean(m2,n2,i1,i1,m3,n3, (n-m+2), &
                                    abs(Bb_tmp)   , datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        i=dat%r%i_Bb2
        if (i > 0) &
          call rhd_mean_rb23absmean(m2,n2,i1,i1,m3,n3, (n-m+2), &
                                         Bb_tmp**2, datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        !
        ! --- Compute wave amplitudes ---
        if ((dat%r%nWave > 0) .and. (dat%r%i_Bbwave > 0)) then
          call rhd_mean_rb23shmean(model, m2,n2, i1,i1, m3,n3, &
                                   dat%r%m,dat%r%n, dat%r%na, dat%r%nWave, dat%r%i_Bbwave, 2, &
                                   Bb_tmp, dat%r%d, &
                                   nb6index, bindex, weight)
        endif
        !
        ! --- Deallocate temporary array ---
        deallocate(x123weight, cindex, bindex, nb6index)
      endif ! ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0))
      !
      ! --- Delete temporary box ---
      call rhd_box_Delete(slice)
      !
      ! --- Deallocate temporary arrays ---
      deallocate(weight, Bb_tmp)
    end do ! i1
    !$OMP END DO
    !
    !$OMP CRITICAL (X13MEAN)
    call rhd_datx_Add(dat%x13, datx_thread)
    call rhd_datx_Delete(datx_thread)
    !
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Add(dat%r, datr_thread)
      call rhd_datx_Delete(datr_thread)
    endif
    !$OMP END CRITICAL (X13MEAN)
    !
    !$OMP END PARALLEL
    !
  endif ! (dat%x13%na > 0)
  !
  if (dat%x12%na > 0) then
    ! === Average over x1-x2 planes and over spherical surfaces ====================================
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(datx_thread, datr_thread, weight, slice, AreaFraction, i, &
    !$OMP           x123weight, cindex, bindex, nb6index, &
    !$OMP           Bb_tmp)
    !
    call rhd_datx_Make(datx_thread, dat%x12%m, dat%x12%n, dat%x12%nWave)
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Make(datr_thread, dat%r%m, dat%r%n, dat%r%nWave, radial_flag=.true.)
    endif
    !
    !$OMP DO
    do i2=m2,n2
      ! --- Allocate temporary arrays ---
      allocate(weight(            i2:i2, m1:n1), &
               Bb_tmp(   m3:n3+1, i2:i2, m1:n1)   )
      !
      ! --- Create temporary box (only with axes filled) ---
      call rhd_box_Make(slice, m1=m3, n1=n3, m2=i2, n2=i2, m3=m1, n3=n1)
      call rhd_box_Copy(model, m1, n1, i2, i2, m3, n3, &
                        slice, m3,     i2,     m1, &
                        trans_mode=3, bulk_mode=0)
      !
      ! --- Compute weights for horizontal averaging ---
      AreaFraction=dtimerel * &
                   ((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                   ((model%xb2(i2+1)-model%xb2(i2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
      call rhd_mean_weight(slice, AreaFraction, weight)
      !
      ! --- Transpose array so that averaging can be done over 2nd and 3rd dimension ---
      Bb_tmp(:,i2,:)=transpose(model%Bb3(m1:n1,i2,m3:n3+1))
      !
      ! --- Check if right boundary values should be ignored or included ---
      if (n3 < wg%hyd%n(3)) then
        di3=0
      else
        di3=1
      endif
      !
      ! --- Average horizontally ---
      i=dat%x12%i_Bb
      if (i > 0) &
        call rhd_mean_x23mean(    Bb_tmp(m3:n3+di3,i2:i2,m1:n1)   , datx_thread%d(m3:n3+di3,i), &
                              weight)
      i=dat%x12%i_Bbabs
      if (i > 0) &
        call rhd_mean_x23mean(abs(Bb_tmp(m3:n3+di3,i2:i2,m1:n1))  , datx_thread%d(m3:n3+di3,i), &
                              weight)
      i=dat%x12%i_Bb2
      if (i > 0) &
        call rhd_mean_x23mean(    Bb_tmp(m3:n3+di3,i2:i2,m1:n1)**2, datx_thread%d(m3:n3+di3,i), &
                              weight)
      !
      if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
        ! --- Average over spheres ---
        !
        ! --- Allocate index array ---
        allocate(x123weight(  m3:n3     , i2:i2, m1:n1), &
                 cindex(      m3:n3     , i2:i2, m1:n1), &
                 bindex(-(n-m+2):(n-m+2), i2:i2, m1:n1), &
                 nb6index(        6     , i2:i2, m1:n1)   )
        !
        ! --- Compute weight arrays ---
        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtimerel)
        !
        ! --- Generate index arrays ---
        call rhd_mean_a2rindex(m3,n3, i2,i2, m1,n1, &
                               1, wg%hyd%m(3), wg%hyd%n(3), &
                               model%xc3(m3-1:n3+1), model%xc2(i2:i2), model%xc1(m1:n1), &
                               dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
        !
        ! --- Compute averages of Bb over spheres ---
        i=dat%r%i_Bb
        if (i > 0) &
          call rhd_mean_rb23mean(   m3,n3,i2,i2,m1,n1, (n-m+2), &
                                        Bb_tmp   , datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        i=dat%r%i_Bbabs
        if (i > 0) &
          call rhd_mean_rb23absmean(m3,n3,i2,i2,m1,n1, (n-m+2), &
                                    abs(Bb_tmp)  , datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        i=dat%r%i_Bb2
        if (i > 0) &
          call rhd_mean_rb23absmean(m3,n3,i2,i2,m1,n1, (n-m+2), &
                                        Bb_tmp**2, datr_thread%d(:,i), &
                                    nb6index, bindex, weight)
        !
        ! --- Compute wave amplitudes ---
        if ((dat%r%nWave > 0) .and. (dat%r%i_Bbwave > 0)) then
          call rhd_mean_rb23shmean(model, m3,n3, i2,i2, m1,n1, &
                                   dat%r%m,dat%r%n, dat%r%na, dat%r%nWave, dat%r%i_Bbwave, 3, &
                                   Bb_tmp, dat%r%d, &
                                   nb6index, bindex, weight)
        endif
        !
        ! --- Deallocate temporary array ---
        deallocate(x123weight, cindex, bindex, nb6index)
      endif ! ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0))
      !
      ! --- Delete temporary box ---
      call rhd_box_Delete(slice)
      !
      ! --- Deallocate temporary arrays ---
      deallocate(weight, Bb_tmp)
    end do ! i2
    !$OMP END DO
    !
    !$OMP CRITICAL (X12MEAN)
    call rhd_datx_Add(dat%x12, datx_thread)
    call rhd_datx_Delete(datx_thread)
    !
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Add(dat%r, datr_thread)
      call rhd_datx_Delete(datr_thread)
    endif
    !$OMP END CRITICAL (X12MEAN)
    !
    !$OMP END PARALLEL
    !
  endif ! (dat%x12%na > 0)
!
end subroutine rhd_mhd_Bboutput


!----------*************----------------------------------------------------------------------------
subroutine rhd_mhd_Bb2Bc(model)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Bb2Bc ('rhd_magneto-hydrodynamics_B_boundary_to_B_center')
!
! PURPOSE:
!   Compute cell centered magnetic field from boundary values.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Bb2Bc(model)
!
! INPUT:
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, contains all model data (e.g. model%rho)
!
! OUTPUT:
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_box_module:        Box handling routines for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   It is assumed that cell centered magnetic fields have been added to the model
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-11-04 (B.F.) First version
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3
!---------------------------------------------------------------------------------------------------
!
call timing_start('MHD: Bb2Bc')
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Typical loop ---
!$OMP PARALLEL DEFAULT(SHARED), &
!$OMP   PRIVATE(i1, i2)
!
!$OMP DO
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      !
      model%Bc1(i1,i2,i3)=0.5 * (model%Bb1(i1,i2,i3) + model%Bb1(i1+1,i2  ,i3  ))
      model%Bc2(i1,i2,i3)=0.5 * (model%Bb2(i1,i2,i3) + model%Bb2(i1  ,i2+1,i3  ))
      model%Bc3(i1,i2,i3)=0.5 * (model%Bb3(i1,i2,i3) + model%Bb3(i1  ,i2  ,i3+1))
      !
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
call timing_stop('MHD: Bb2Bc')
!
end subroutine rhd_mhd_Bb2Bc


!----------*************************----------------------------------------------------------------
subroutine rhd_bound_3DInOutFlow_mhd(model, action, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DInOutFlow_mhd ('rhd_boundary_conditions_3D_In_Out_Flow_magneto-hydrodynamics')
!
! PURPOSE:
!   Correct values in bottom cells to fullfill the open lower boundary condition.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DInOutFlow_mhd(model, action, dtime)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   This routine realizes the open lower boundary condition for models of the surface layers
!   deep solar-type convection zones.
!   The values of density, internal energy and velocity in the deepest layer of the model
!   are modified.
!   The extrapolation into the ghost cells at the bottom (later by rhd_bound_1DGhost) should be
!   'constant'.
!   The internal energies in the upward streaming cells are modified to push their entropy closer
!   to the value prescribed by s_InFlow.
!   The densities are adjusted to give a pressure closer to the mean value with no net mass change.
!   The velocities are modified to give zero mass flux.
!
!   New corrections:
!     drho  =  drho/dP|s       * dP  +  drho/ds|P                        * ds
!     dei   =   dei/dP|s       * dP  +   dei/ds|P                        * ds
!   =>
!     drho  =  rho/(Gamma_1*P) * dP  +  (-rho^2*T/P*(Gamma_3-1)/Gamma_1) * ds
!     dei   =    1/(Gamma_1*P) * dP  +  T(1-(Gamma_3-1)/Gamma_1))        * ds
!
! EXAMPLE:
!   call rhd_bound_3DInOutFlow_mhd(model, action, 0.5)
!
! MODIFICATION HISTORY:
!   1999-05-11 Written by Bernd Freytag, Astronomical Observatory Copenhagen
!   1999-05-17 (B.F.) Mass flux correction at end
!   1999-05-19 (B.F.) Correct effect of pressure correction for conserving the mass in bottom layer
!   2000-03-22 (B.F.) Density and energy corrections depend both on pressure and entropy
!   2000-12-14 (B.F.) Pressure correction: ei: time_s_corr => time_P_corr
!   2006-09-12 (W.S. Freiburg) New boundary condition inoutflow2
!   2006-11-07 (B.F.) C_test1 -> B1_inflow
!   2010-03-25 (B.F. Naples) Bugfix against loss/gain of total mass: new brackets in update formula
!                     normalize area in a separate step; some local variables double precision
!   2010-07-16 (B.F. Naples) More comments; bugfix in computation of Bc2
!   2015-04-01 (B.F., O.S.) Compute dd with max
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_bound_t01
!#
use timing_module
!#
# endif
!#
use gasinter_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2
real(kind=kind(0.0D+00))                :: rhov3_mean, rho_mean, rho_mean_new, v3_correction, &
                                           P_mean, caq, dd, B_sq_tan, &
                                           time_char, time_s_corr, time_P_corr, area_sum
real(kind=kind(0.0D+00)), &
      dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: area
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: P_gas, P_mag, P_tot, dPdei, dPdrho, T, dTdei, s, &
                                           Gamma_1, Gamma_3, cs, va, vf
logical, dimension(model%m1:model%n1, &
                   model%m2:model%n2)   :: mask_inflow
real, parameter                         :: epsilon=0.000001D+00
!---------------------------------------------------------------------------------------------------
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Relative cell (column) areas ---
do i1=m1, n1
  do i2=m2, n2
!    area(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2)) / &
!                ((model%xb1(n1+1)-model%xb1(m1))*(model%xb2(n2+1)-model%xb2(m2)))
    area(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2))
  end do ! i2
end do ! i1
area_sum=sum(area)
do i2=m2,n2
  do i1=m1,n1
    area(i1,i2)=area(i1,i2)/area_sum
  end do ! i1
end do ! i2
!
! --- Solve EOS ---
!#
# ifdef rhd_bound_t01
!#
call timing_start( 'MHD: bound3d: eosinter')
!#
# endif
!#
!
call eosinter_all(model%rho(:,:,m3), model%ei(:,:,m3), &
                  P=P_gas, dPdrho=dPdrho, dPde=dPdei, &
                  T=T,                dTde=dTdei, &
                  s=s)
!#
# ifdef rhd_bound_t01
!#
call timing_stop( 'MHD: bound3d: eosinter')
!#
# endif
!#
!
! --- Derived quantities ---
P_mag=0.5*(model%Bc1(:,:,m3)**2+model%Bc2(:,:,m3)**2+model%Bc3(:,:,m3)**2)
P_tot=P_gas+P_mag
Gamma_1=dPdrho*model%rho(:,:,m3)/P_gas + dPdei/model%rho(:,:,m3)
Gamma_3=dPdei/model%rho(:,:,m3) + 1.0
cs     =sqrt(P_gas/model%rho(:,:,m3)*Gamma_1)
va     =abs(model%Bc1(:,:,m3))/sqrt(model%rho(:,:,m3))
do i1=m1,n1
  do i2=m2,n2
    B_sq_tan=model%Bc2(i1,i2,m3)**2+model%Bc3(i1,i2,m3)**2
    !
    if (B_sq_tan > epsilon * P_tot(i1,i2)) then
      caq      =cs(i1,i2)**2+2.0*P_mag(i1,i2)/model%rho(i1,i2,m3)
      dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2)*va(i1,i2)/caq)**2))
      vf(i1,i2)=sqrt(0.5*(caq + dd))
    else
      if (cs(i1,i2) - va(i1,i2) > epsilon * cs(i1,i2)) then
        vf(i1,i2) = cs(i1,i2)
      else if (va(i1,i2) - cs(i1,i2) > epsilon * cs(i1,i2)) then
        vf(i1,i2) = va(i1,i2)
      else
        vf(i1,i2) = cs(i1,i2)
      endif
    endif
  end do ! i2
end do ! i1
!
! --- Area averaged density ---
rho_mean=sum(area*model%rho(m1:n1,m2:n2,m3))
! --- Mean pressure ---
P_mean=sum(area*P_tot)
!
! \label{f90:c_schange__useMHD1}\label{f90:c_pchange__useMHD1}
! --- Compute characteristic time scales ---
time_char=(model%xb3(m3+1)-model%xb3(m3)) / sum(area*(vf + abs(model%v3(:,:,m3))))
time_s_corr=time_char/par%C_sChange
time_P_corr=time_char/par%C_PChange
!
! === Entropy of material streaming into the computational volume ===
!
! --- Find inflow columns ---
mask_inflow=(model%v3(m1:n1,m2:n2,m3) > 0.0)
!
if (action%bound_mag_l3 == bound_mag_inoutflow) then
  !
  ! \label{f90:b1_inflow__useMHD1}
  ! --- Set horizontal magnetic field in inflow cells ---
  where (mask_inflow)
    model%Bb1(m1  :n1  ,m2  :n2  ,m3) = 0.282094792 * par%B1_inflow
    model%Bb1(m1+1:n1+1,m2  :n2  ,m3) = 0.282094792 * par%B1_inflow
    ! --- Set at the same time the 2-component zero  OST: 1. 9. 2009 ---
    model%Bb2(m1  :n1  ,m2  :n2  ,m3) = 0.0
    model%Bb2(m1  :n1  ,m2+1:n2+1,m3) = 0.0
  end where
  !
  ! --- Keep field divergence free ---
  do i1=m1,n1
    do i2=m2,n2
      model%Bb3(i1,i2,m3) = model%Bb3(i1,i2,m3+1) + (model%xb3(m3+1) - model%xb3(m3)) * &
        ((model%Bb1(i1+1,i2  ,m3) - model%Bb1(i1,i2,m3)) / (model%xb1(i1+1) - model%xb1(i1)) + &
         (model%Bb2(i1  ,i2+1,m3) - model%Bb2(i1,i2,m3)) / (model%xb2(i2+1) - model%xb1(i2)))
    end do
  end do
  !
  ! --- Recompute cell-centered magnetic field from cell-boundary values ---
  model%Bc1(m1:n1,m2:n2,m3) = 0.5 * (model%Bb1(m1+1:n1+1,m2  :n2  ,m3  ) + model%Bb1(m1:n1,m2:n2,m3))
  ! --- Next line in connection with setting 2-component zero ---
  !model%Bc2(m1:n1,m2:n2,m3) = 0.5 * (model%Bb1(m1  :n1  ,m2+1:n2+1,m3  ) + model%Bb1(m1:n1,m2:n2,m3))
! Bug? B.F. 2010-07-16: there should be Bb2 on the right side ---
  model%Bc2(m1:n1,m2:n2,m3) = 0.5 * (model%Bb2(m1  :n1  ,m2+1:n2+1,m3  ) + model%Bb2(m1:n1,m2:n2,m3))
  model%Bc3(m1:n1,m2:n2,m3) = 0.5 * (model%Bb3(m1  :n1  ,m2  :n2  ,m3+1) + model%Bb3(m1:n1,m2:n2,m3))
endif
!
! \label{f90:s_inflow__useMHD1}
! --- Modify internal energy to come closer to specified inflow entropy ---
where (mask_inflow)
  model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + dtime/time_s_corr * &
                (-(model%rho(m1:n1,m2:n2,m3)**2)*T/P_gas*(Gamma_3-1.0)/Gamma_1) * (par%s_InFlow-s)
  model%ei( m1:n1,m2:n2,m3)=model%ei( m1:n1,m2:n2,m3) + dtime/time_s_corr * &
                      T*(1.0-(Gamma_3-1.0)/Gamma_1)                             * (par%s_InFlow-s)
end where
!
! === Pressure ===
!
! --- Modify density to shift local pressure closer to average ---
model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + dtime/time_P_corr * &
                                       1.0/cs**2                                * (P_mean-P_tot)
model%ei( m1:n1,m2:n2,m3)=model%ei( m1:n1,m2:n2,m3) + dtime/time_P_corr * &
                                       1.0/(Gamma_1*model%rho(m1:n1,m2:n2,m3))  * (P_mean-P_tot)
!
! --- New area averaged density after corrections ---
rho_mean_new=sum(area*model%rho(m1:n1,m2:n2,m3))
! --- Correct density to give same overall mass in bottom layer ---
!model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + rho_mean-rho_mean_new
model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + (rho_mean-rho_mean_new)
!
!write(gl%nc_p,'(A,F9.7,1X,3E15.7)') '3DInOutFlow: area; rho0,n,e: ', &
!                     sum(area), rho_mean, rho_mean_new, sum(area*model%rho(m1:n1,m2:n2,m3))
!
! === Mass flux correction: Modify vertical velocity to give overall mass flux = 0 ===
!
! --- Average mass flux through bottom cell layer ---
! --- Correction velocity ---
rhov3_mean  =sum(area*model%rho(m1:n1,m2:n2,m3)*model%v3(m1:n1,m2:n2,m3))
v3_correction=rhov3_mean/rho_mean
if (action%bound_l3 == bound_inoutflow .and. par%A%reconstruction_order > 0 .and. &
    par%hdTimeIntegrationScheme == 'Hancock') then
  v3_correction=v3_correction - 0.125 * dtime * par%grav
endif
! --- Apply correction ---
model%v3(m1:n1,m2:n2,m3)=model%v3(m1:n1,m2:n2,m3)-v3_correction
!
end subroutine rhd_bound_3DInOutFlow_mhd


!----------************************-----------------------------------------------------------------
subroutine rhd_bound_3DTransmit_mhd(model, action, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DTransmit_mhd ('rhd_boundary_conditions_3D_Transmit_magneto-hydrodynamics')
!
! PURPOSE:
!   Correct values in cells for all sides to perform the open transmitting boundary condition.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DTransmit_mhd(model, action, dtime)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!
! ROUTINES:
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_bound_3DTransmit_mhd(model, action, 0.5)
!
! MODIFICATION HISTORY:
!   2010-04-05 (B.F.) First Version
!   2010-04-28 (B.F.) bound_transmitting3
!   2011-06-01 (W.S.) Derived from rhd_bound_3DTransmit and rhd_bound_1DTransmit_mhd for unsplit
!                     version of Hancock scheme, density Correction for all 6 sides
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_prop_module
!#
# ifdef rhd_bound_t01
!#
use timing_module
!#
# endif
!#
use gasinter_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3
real                                    :: T_inflow, dd, caq, B_sq, B_sq_tan, eiTable_min
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: rho1, ei1, v11, Bc11, Bc21, Bc31, &
                                           P1, dPdei1, dPdrho1, T1, T21, dTdei1, s1, cs1, va1, &
                                           vf1, v1org1, time_char1, time_T_corr1, &
                                           drho_l1, drho_r1, dei_l1, dei_r1
real, dimension(model%m1:model%n1, &
                model%m3:model%n3)      :: rho2, ei2, v22, Bc12, Bc22, Bc32, &
                                           P2, dPdei2, dPdrho2, T2, T22, dTdei2, s2, cs2, va2, &
                                           vf2, v2org2, time_char2, time_T_corr2, &
                                           drho_l2, drho_r2, dei_l2, dei_r2
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: rho3, ei3, v33, Bc13, Bc23, Bc33, &
                                           P3, dPdei3, dPdrho3, T3, T23, dTdei3, s3, cs3, va3, &
                                           vf3, v3org3, time_char3, time_T_corr3, &
                                           drho_l3, drho_r3, dei_l3, dei_r3
logical, dimension(model%m2:model%n2, &
                   model%m3:model%n3)   :: mask_inflow1
logical, dimension(model%m1:model%n1, &
                   model%m3:model%n3)   :: mask_inflow2
logical, dimension(model%m1:model%n1, &
                   model%m2:model%n2)   :: mask_inflow3
real, parameter                         :: epsilon=0.000001
real(kind=kind(0.0D+00))                :: rho_mean, ei_mean, &
                                           time_char_mean, area_sum, cs_mean
real                                    :: P_mean, dPdrho_mean, dPdei_mean, rho_min, rho_max
real(kind=kind(0.0D+00)), &
      dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: area12
real(kind=kind(0.0D+00)), &
      dimension(model%m1:model%n1, &
                model%m3:model%n3)      :: area13
real(kind=kind(0.0D+00)), &
      dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: area23
!---------------------------------------------------------------------------------------------------
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set energy corrections to zero ---
dei_l1=0.0
dei_r1=0.0
dei_l2=0.0
dei_r2=0.0
dei_l3=0.0
dei_r3=0.0
!
! \label{f90:teff__useMHD1}\label{f90:c_tsurf__useMHD1}
! --- Compute target temperature of inflowing material from effective temperature ---
! --- and reduction factor                                                        ---
T_inflow=par%Teff*par%C_Tsurf
!
! \label{f90:c_rhochangetop__useMHD1}
if (par%C_rhoChangeTop > 0.0) then
  !
  drho_l1=0.0
  drho_r1=0.0
  drho_l2=0.0
  drho_r2=0.0
  drho_l3=0.0
  drho_r3=0.0
  !
  if (n1 > m1) then
    !
    if ((action%bound_l1 == bound_transmitting ) .or. &
        (action%bound_l1 == bound_transmitting2) .or. &
        (action%bound_l1 == bound_transmitting3)       ) then
      !
      !
      i1=m1
      !
      ! --- Relative cell (column) areas ---
      do i3=m3,n3
        do i2=m2,n2
          area23(i2,i3)=(model%xb2(i2+1)-model%xb2(i2)) * (model%xb3(i3+1)-model%xb3(i3))
        end do ! i2
      end do ! i3
      area_sum=sum(area23)
      do i2=m2,n2
        do i3=m3,n3
          area23(i2,i3)=area23(i2,i3)/area_sum
        end do ! i3
      end do ! i2
      !
      ! --- Old minimum density ---
      rho_min=minval(model%rho(i1,:,:))
      rho_max=maxval(model%rho(i1,:,:))
      !
      ! --- Area-averaged density and energy ---
      rho_mean=sum(area12*model%rho(i1,:,:))
      ei_mean =sum(area12*model%ei( i1,:,:))
      !
      ! --- Solve EOS for average and compute sound speed ---
      call eosinter(real(rho_mean, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                    P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
      cs_mean=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
              real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean**2)
      !
      ! --- Characteristic time scale ---
      time_char_mean=(model%xb1(i1+1)-model%xb1(i1)) / &
                     (0.01*(cs_mean+sum(area23*abs(model%v1(i1,:,:)))))
      !
      ! --- Adjust density ---
      rho1=rho_mean + (model%rho(i1,:,:)-rho_mean) * &
                                   exp(-dtime/time_char_mean*log(rho_mean/model%rho(i1,:,:)+1.0))
      !
      ! --- Correct density to keep overall mass in layer ---
      drho_l1=rho1*(rho_mean/sum(area23*rho1)) - model%rho(i1,:,:)
      !
      ! --- Print result ---
      !write(gl%nc_p, '(2(A,1PE13.6,1X,1PE13.6))') &
      !               '3DTransmit: min/max rho: ', &
      !               rho_min, rho_max, ' => ', &
      !               minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    endif
    !
    if ((action%bound_r1 == bound_transmitting ) .or. &
        (action%bound_r1 == bound_transmitting2) .or. &
        (action%bound_r1 == bound_transmitting3)       ) then
      !
      !
      i1=n1
      !
      ! --- Relative cell (column) areas ---
      do i3=m3,n3
        do i2=m2,n2
          area23(i2,i3)=(model%xb2(i2+1)-model%xb2(i2)) * (model%xb3(i3+1)-model%xb3(i3))
        end do ! i2
      end do ! i3
      area_sum=sum(area23)
      do i2=m2,n2
        do i3=m3,n3
          area23(i2,i3)=area23(i2,i3)/area_sum
        end do ! i3
      end do ! i2
      !
      ! --- Old minimum density ---
      rho_min=minval(model%rho(i1,:,:))
      rho_max=maxval(model%rho(i1,:,:))
      !
      ! --- Area-averaged density and energy ---
      rho_mean=sum(area23*model%rho(i1,:,:))
      ei_mean =sum(area23*model%ei( i1,:,:))
      !
      ! --- Solve EOS for average and compute sound speed ---
      call eosinter(real(rho_mean, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                    P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
      cs_mean=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
              real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean**2)
      !
      ! --- Characteristic time scale ---
      time_char_mean=(model%xb1(i1+1)-model%xb1(i1)) / &
                     (0.01*(cs_mean+sum(area23*abs(model%v1(i1,:,:)))))
      !
      ! --- Adjust density ---
      rho1=rho_mean + (model%rho(i1,:,:)-rho_mean) * &
                                   exp(-dtime/time_char_mean*log(rho_mean/model%rho(i1,:,:)+1.0))
      !
      ! --- Correct density to keep overall mass in layer ---
      drho_r1=rho1*(rho_mean/sum(area23*rho1)) - model%rho(i1,:,:)
      !
      ! --- Print result ---
      !write(gl%nc_p, '(2(A,1PE13.6,1X,1PE13.6))') &
      !               '3DTransmit: min/max rho: ', &
      !               rho_min, rho_max, ' => ', &
      !               minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    endif
    !
  endif
  !
  if (n2 > m2) then
    !
    if ((action%bound_l2 == bound_transmitting ) .or. &
        (action%bound_l2 == bound_transmitting2) .or. &
        (action%bound_l2 == bound_transmitting2)       ) then
      !
      !
      i2=m2
      !
      ! --- Relative cell (column) areas ---
      do i1=m1,n1
        do i3=m3,n3
          area13(i1,i3)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb3(i3+1)-model%xb3(i3))
        end do ! i3
      end do ! i1
      area_sum=sum(area13)
      do i3=m3,n3
        do i1=m1,n1
          area12(i1,i3)=area13(i1,i3)/area_sum
        end do ! i1
      end do ! i3
      !
      ! --- Old minimum density ---
      rho_min=minval(model%rho(:,i2,:))
      rho_max=maxval(model%rho(:,i2,:))
      !
      ! --- Area-averaged density and energy ---
      rho_mean=sum(area13*model%rho(:,i2,:))
      ei_mean =sum(area13*model%ei( :,i2,:))
      !
      ! --- Solve EOS for average and compute sound speed ---
      call eosinter(real(rho_mean, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                    P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
      cs_mean=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
              real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean**2)
      !
      ! --- Characteristic time scale ---
      time_char_mean=(model%xb2(i2+1)-model%xb2(i2)) / &
                     (0.01*(cs_mean+sum(area13*abs(model%v2(:,i2,:)))))
      !
      ! --- Adjust density ---
      rho2=rho_mean + (model%rho(:,i2,:)-rho_mean) * &
                                   exp(-dtime/time_char_mean*log(rho_mean/model%rho(:,i2,:)+1.0))
      !
      ! --- Correct density to keep overall mass in layer ---
      drho_l2=rho2*(rho_mean/sum(area13*rho2)) - model%rho(:,i2,:)
      !
      ! --- Print result ---
      !write(gl%nc_p, '(2(A,1PE13.6,1X,1PE13.6))') &
      !               '3DTransmit: min/max rho: ', &
      !               rho_min, rho_max, ' => ', &
      !               minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    endif
    !
    if ((action%bound_r2 == bound_transmitting ) .or. &
        (action%bound_r2 == bound_transmitting2) .or. &
        (action%bound_r2 == bound_transmitting3)       ) then
      !
      !
      i2=n2
      !
      ! --- Relative cell (column) areas ---
      do i1=m1,n1
        do i3=m3,n3
          area13(i1,i3)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb3(i3+1)-model%xb3(i3))
        end do ! i3
      end do ! i1
      area_sum=sum(area13)
      do i3=m3,n3
        do i1=m1,n1
          area13(i1,i3)=area13(i1,i3)/area_sum
        end do ! i1
      end do ! i3
      !
      ! --- Old minimum density ---
      rho_min=minval(model%rho(:,i2,:))
      rho_max=maxval(model%rho(:,i2,:))
      !
      ! --- Area-averaged density and energy ---
      rho_mean=sum(area12*model%rho(:,i2,:))
      ei_mean =sum(area12*model%ei( :,i2,:))
      !
      ! --- Solve EOS for average and compute sound speed ---
      call eosinter(real(rho_mean, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                    P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
      cs_mean=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
              real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean**2)
      !
      ! --- Characteristic time scale ---
      time_char_mean=(model%xb2(i2+1)-model%xb2(i2)) / &
                     (0.01*(cs_mean+sum(area13*abs(model%v2(:,i2,:)))))
      !
      ! --- Adjust density ---
      rho2=rho_mean + (model%rho(:,i2,:)-rho_mean) * &
                                   exp(-dtime/time_char_mean*log(rho_mean/model%rho(:,i2,:)+1.0))
      !
      ! --- Correct density to keep overall mass in layer ---
      drho_r2=rho2*(rho_mean/sum(area13*rho2)) - model%rho(:,i2,:)
      !
      ! --- Print result ---
      !write(gl%nc_p, '(2(A,1PE13.6,1X,1PE13.6))') &
      !               '3DTransmit: min/max rho: ', &
      !               rho_min, rho_max, ' => ', &
      !               minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    endif
    !
  endif
  !
  if (n3 > m3) then
    !
    if ((action%bound_l3 == bound_transmitting ) .or. &
        (action%bound_l3 == bound_transmitting2) .or. &
        (action%bound_l3 == bound_transmitting3)       ) then
      !
      !
      i3=m3
      !
      ! --- Relative cell (column) areas ---
      do i1=m1,n1
        do i2=m2,n2
          area12(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2))
        end do ! i2
      end do ! i1
      area_sum=sum(area12)
      do i2=m2,n2
        do i1=m1,n1
          area12(i1,i2)=area12(i1,i2)/area_sum
        end do ! i1
      end do ! i2
      !
      ! --- Old minimum density ---
      rho_min=minval(model%rho(:,:,i3))
      rho_max=maxval(model%rho(:,:,i3))
      !
      ! --- Area-averaged density and energy ---
      rho_mean=sum(area12*model%rho(:,:,i3))
      ei_mean =sum(area12*model%ei( :,:,i3))
      !
      ! --- Solve EOS for average and compute sound speed ---
      call eosinter(real(rho_mean, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                    P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
      cs_mean=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
              real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean**2)
      !
      ! --- Characteristic time scale ---
      time_char_mean=(model%xb3(i3+1)-model%xb3(i3)) / &
                     (0.01*(cs_mean+sum(area12*abs(model%v3(:,:,i3)))))
      !
      ! --- Adjust density ---
      rho3=rho_mean + (model%rho(:,:,i3)-rho_mean) * &
                                   exp(-dtime/time_char_mean*log(rho_mean/model%rho(:,:,i3)+1.0))
      !
      ! --- Correct density to keep overall mass in layer ---
      drho_l3=rho3*(rho_mean/sum(area12*rho3)) - model%rho(:,:,i3)
      !
      ! --- Print result ---
      !write(gl%nc_p, '(2(A,1PE13.6,1X,1PE13.6))') &
      !               '3DTransmit: min/max rho: ', &
      !               rho_min, rho_max, ' => ', &
      !               minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    endif
    !
    if ((action%bound_r3 == bound_transmitting ) .or. &
        (action%bound_r3 == bound_transmitting2) .or. &
        (action%bound_r3 == bound_transmitting3)       ) then
      !
      !
      i3=n3
      !
      ! --- Relative cell (column) areas ---
      do i1=m1,n1
        do i2=m2,n2
          area12(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2))
        end do ! i2
      end do ! i1
      area_sum=sum(area12)
      do i2=m2,n2
        do i1=m1,n1
          area12(i1,i2)=area12(i1,i2)/area_sum
        end do ! i1
      end do ! i2
      !
      ! --- Old minimum density ---
      rho_min=minval(model%rho(:,:,i3))
      rho_max=maxval(model%rho(:,:,i3))
      !
      ! --- Area-averaged density and energy ---
      rho_mean=sum(area12*model%rho(:,:,i3))
      ei_mean =sum(area12*model%ei( :,:,i3))
      !
      ! --- Solve EOS for average and compute sound speed ---
      call eosinter(real(rho_mean, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                    P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
      cs_mean=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
              real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean**2)
      !
      ! --- Characteristic time scale ---
      time_char_mean=(model%xb3(i3+1)-model%xb3(i3)) / &
                     (0.01*(cs_mean+sum(area12*abs(model%v3(:,:,i3)))))
      !
      ! --- Adjust density ---
      rho3=rho_mean + (model%rho(:,:,i3)-rho_mean) * &
                                   exp(-dtime/time_char_mean*log(rho_mean/model%rho(:,:,i3)+1.0))
      !
      ! --- Correct density to keep overall mass in layer ---
      drho_r3=rho3*(rho_mean/sum(area12*rho3)) - model%rho(:,:,i3)
      !
      ! --- Print result ---
      !write(gl%nc_p, '(2(A,1PE13.6,1X,1PE13.6))') &
      !               '3DTransmit: min/max rho: ', &
      !               rho_min, rho_max, ' => ', &
      !               minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    endif
    !
  endif
  !
  model%rho(m1,:,:)=model%rho(m1,:,:)+drho_l1
  model%rho(n1,:,:)=model%rho(n1,:,:)+drho_r1
  model%rho(:,m2,:)=model%rho(:,m2,:)+drho_l2
  model%rho(:,n2,:)=model%rho(:,n2,:)+drho_r2
  model%rho(:,:,m3)=model%rho(:,:,m3)+drho_l3
  model%rho(:,:,n3)=model%rho(:,:,n3)+drho_r3
  !
endif
!
if (par%hdsplit == 'unsplit' .and. par%hdTimeIntegrationScheme == 'Hancock') then
  !
  if (n1 > m1) then
    !
    if ((action%bound_l1 == bound_transmitting ) .or. &
        (action%bound_l1 == bound_transmitting2) .or. &
        (action%bound_l1 == bound_transmitting3)       ) then
      !
      i1=m1
      !
      rho1=model%rho(i1,:,:)
      ei1= model%ei( i1,:,:)
      v11= model%v1( i1,:,:)
      Bc11=model%Bc1(i1,:,:)
      Bc21=model%Bc2(i1,:,:)
      Bc31=model%Bc3(i1,:,:)
      !
      ! --- Solve EOS ---
!#
#     ifdef rhd_bound_t01
!#
      call timing_start('MHD: bound3d: eosinter')
!#
#     endif
!#
      call eosinter_all(rho1, ei1, &
                        P=P1, dPdrho=dPdrho1, dPde=dPdei1, &
                        T=T1,                 dTde=dTdei1, &
                        s=s1)
!#
#     ifdef rhd_bound_t01
!#
      call timing_stop( 'MHD: bound3d: eosinter')
!#
#     endif
!#
      cs1=sqrt(P1*(dPdei1/rho1)/rho1 + dPdrho1)
      va1=abs(Bc11)/sqrt(rho1)
      !
      do i3=m3,n3
        do i2=m2,n2
          B_sq     = Bc11(i2,i3)**2+Bc21(i2,i3)**2+Bc31(i2,i3)**2
          B_sq_tan = Bc21(i2,i3)**2+Bc31(i2,i3)**2

          if (B_sq_tan > epsilon * (P1(i1,i2)+0.5*B_sq)) then
            caq      =cs1(i2,i3)**2+B_sq/rho1(i2,i3)
            dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs1(i2,i3)*va1(i2,i3)/caq)**2))
            vf1(i2,i3)=sqrt(0.5*(caq + dd))
          else
            if (cs1(i2,i3) - va1(i2,i3) > epsilon * cs1(i2,i3)) then
              vf1(i2,i3) = cs1(i2,i3)
            else if (va1(i2,i3) - cs1(i2,i3) > epsilon * cs1(i2,i3)) then
              vf1(i2,i3) = va1(i2,i3)
            else
              vf1(i2,i3) = cs1(i2,i3)
            endif
          endif
        end do ! i2
      end do ! i3
      !
      ! \label{f90:c_tchange__useMHD1}
      ! --- Compute characteristic time scales ---
      time_char1=(model%xb1(i1+1)-model%xb1(i1)) / (0.01*vf1 + abs(v11))
      time_T_corr1=time_char1/par%C_TChange
!!$    write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                    minval(time_T_corr), maxval(time_T_corr)
      !
      ! --- Find inflow columns ---
      mask_inflow1=(v11 > 0.0)
      !
      ! --- Modify internal energy to come closer to specified inflow temperature ---
      where (mask_inflow1)
        dei_l1=dei_l1 + dtime/time_T_corr1 * (T_inflow-T1)/dTdei1
      end where
      !
      if (par%C_visBound > 0.0) then
        ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
        v1org1=v11
        where (mask_inflow1)
          v11=v11 * max( (1.0-par%C_visBound*dtime/(model%xb1(i1+1)-model%xb1(i1))*cs1 * &
                         (1.0+(abs(v11)/cs1)**2)) &
                       ,0.0)
          dei_l1=dei_l1 + 0.5 * (v1org1**2 - v11**2)
        end where
      endif
      !
      model%v1( i1,:,:)=v11
      !
    endif
    !
    if ((action%bound_r1 == bound_transmitting ) .or. &
        (action%bound_r1 == bound_transmitting2) .or. &
        (action%bound_r1 == bound_transmitting3)       ) then
      !
      i1=n1
      !
      rho1=model%rho(i1,:,:)
      ei1= model%ei( i1,:,:)
      v11= model%v1( i1,:,:)
      Bc11=model%Bc1(i1,:,:)
      Bc21=model%Bc2(i1,:,:)
      Bc31=model%Bc3(i1,:,:)
      !
      ! --- Solve EOS ---
!#
#     ifdef rhd_bound_t01
!#
      call timing_start('MHD: bound3d: eosinter')
!#
#     endif
!#
      call eosinter_all(rho1, ei1, &
                        P=P1, dPdrho=dPdrho1, dPde=dPdei1, &
                        T=T1,                 dTde=dTdei1, &
                        s=s1)
!#
#     ifdef rhd_bound_t01
!#
      call timing_stop( 'MHD: bound3d: eosinter')
!#
#     endif
!#
      cs1=sqrt(P1*(dPdei1/rho1)/rho1 + dPdrho1)
      va1=abs(Bc11)/sqrt(rho1)
      !
      do i3=m3,n3
        do i2=m2,n2
          B_sq     = Bc11(i2,i3)**2+Bc21(i2,i3)**2+Bc31(i2,i3)**2
          B_sq_tan = Bc21(i2,i3)**2+Bc31(i2,i3)**2

          if (B_sq_tan > epsilon * (P1(i2,i3)+0.5*B_sq)) then
            caq      =cs1(i2,i3)**2+B_sq/rho1(i2,i3)
            dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs1(i2,i3)*va1(i2,i3)/caq)**2))
            vf1(i2,i3)=sqrt(0.5*(caq + dd))
          else
            if (cs1(i2,i3) - va1(i2,i3) > epsilon * cs1(i2,i3)) then
              vf1(i2,i3) = cs1(i2,i3)
            else if (va1(i2,i3) - cs1(i2,i3) > epsilon * cs1(i2,i3)) then
              vf1(i2,i3) = va1(i2,i3)
            else
              vf1(i2,i3) = cs1(i2,i3)
            endif
          endif
        end do ! i2
      end do ! i3
      !
      ! --- Compute characteristic time scales ---
      time_char1=(model%xb1(i1+1)-model%xb1(i1)) / (0.01*vf1 + abs(v11))
      time_T_corr1=time_char1/par%C_TChange
!!$    write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                    minval(time_T_corr), maxval(time_T_corr)
      !
      ! --- Find inflow columns ---
      mask_inflow1=(v11 < 0.0)
      !
      ! --- Modify internal energy to come closer to specified inflow temperature ---
      where (mask_inflow1)
        dei_r1=dei_r1 + dtime/time_T_corr1 * (T_inflow-T1)/dTdei1
      end where
      !
      if (par%C_visBound > 0.0) then
        ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
        v1org1=v11
        where (mask_inflow1)
          v11=v11 * max( (1.0-par%C_visBound*dtime/(model%xb1(i1+1)-model%xb1(i1))*cs1 * &
                         (1.0+(abs(v11)/cs1)**2)) &
                       ,0.0)
          dei_r1=dei_r1 + 0.5 * (v1org1**2 - v11**2)
        end where
      endif
      !
      model%v1( i1,:,:)=v11
      !
    endif
    !
  endif
  !
  if (n2 > m2) then
    !
    if ((action%bound_l2 == bound_transmitting ) .or. &
        (action%bound_l2 == bound_transmitting2) .or. &
        (action%bound_l2 == bound_transmitting3)       ) then
      !
      i2=m2
      !
      rho2=model%rho(:,i2,:)
      ei2= model%ei( :,i2,:)
      v22= model%v2( :,i2,:)
      Bc12=model%Bc1(:,i2,:)
      Bc22=model%Bc2(:,i2,:)
      Bc32=model%Bc3(:,i2,:)
      !
      ! --- Solve EOS ---
!#
#     ifdef rhd_bound_t01
!#
      call timing_start('MHD: bound3d: eosinter')
!#
#     endif
!#
      call eosinter_all(rho2, ei2, &
                        P=P2, dPdrho=dPdrho2, dPde=dPdei2, &
                        T=T2,                 dTde=dTdei2, &
                        s=s2)
!#
#     ifdef rhd_bound_t01
!#
      call timing_stop( 'MHD: bound3d: eosinter')
!#
#     endif
!#
      cs2=sqrt(P2*(dPdei2/rho2)/rho2 + dPdrho2)
      va2=abs(Bc22)/sqrt(rho2)
      !
      do i1=m1,n1
        do i3=m3,n3
          B_sq     = Bc12(i1,i3)**2+Bc22(i1,i3)**2+Bc32(i1,i3)**2
          B_sq_tan = Bc12(i1,i3)**2+Bc32(i1,i3)**2

          if (B_sq_tan > epsilon * (P2(i1,i3)+0.5*B_sq)) then
            caq      =cs2(i1,i3)**2+B_sq/rho2(i1,i3)
            dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs2(i1,i3)*va2(i1,i3)/caq)**2))
            vf2(i1,i3)=sqrt(0.5*(caq + dd))
          else
            if (cs2(i1,i3) - va2(i1,i3) > epsilon * cs2(i1,i3)) then
              vf2(i1,i3) = cs2(i1,i3)
            else if (va2(i1,i3) - cs2(i1,i3) > epsilon * cs2(i1,i3)) then
              vf2(i1,i3) = va2(i1,i3)
            else
              vf2(i1,i3) = cs2(i1,i3)
            endif
          endif
        end do ! i3
      end do ! i1
      !
      ! --- Compute characteristic time scales ---
      time_char2=(model%xb2(i2+1)-model%xb2(i2)) / (0.01*vf2 + abs(v22))
      time_T_corr2=time_char2/par%C_TChange
!!$    write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                    minval(time_T_corr), maxval(time_T_corr)
      !
      ! --- Find inflow columns ---
      mask_inflow2=(v22 > 0.0)
      !
      ! --- Modify internal energy to come closer to specified inflow temperature ---
      where (mask_inflow2)
        dei_l2=dei_l2 + dtime/time_T_corr2 * (T_inflow-T2)/dTdei2
      end where
      !
      if (par%C_visBound > 0.0) then
        ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
        v2org2=v22
        where (mask_inflow2)
          v22=v22 * max( (1.0-par%C_visBound*dtime/(model%xb2(i2+1)-model%xb2(i2))*cs2 * &
                         (1.0+(abs(v22)/cs2)**2)) &
                       ,0.0)
          dei_l2=dei_l2 + 0.5 * (v2org2**2 - v22**2)
        end where
      endif
      !
      model%v2( :,i2,:)=v22
      !
    endif
    !
    if ((action%bound_r2 == bound_transmitting ) .or. &
        (action%bound_r2 == bound_transmitting2) .or. &
        (action%bound_r2 == bound_transmitting3)       ) then
      !
      i2=n2
      !
      rho2=model%rho(:,i2,:)
      ei2= model%ei( :,i2,:)
      v22= model%v2( :,i2,:)
      Bc12=model%Bc1(:,i2,:)
      Bc22=model%Bc2(:,i2,:)
      Bc32=model%Bc3(:,i2,:)
      !
      ! --- Solve EOS ---
!#
#     ifdef rhd_bound_t01
!#
      call timing_start('MHD: bound3d: eosinter')
!#
#     endif
!#
      call eosinter_all(rho2, ei2, &
                        P=P2, dPdrho=dPdrho2, dPde=dPdei2, &
                        T=T2,                 dTde=dTdei2, &
                        s=s2)
!#
#     ifdef rhd_bound_t01
!#
      call timing_stop( 'MHD: bound3d: eosinter')
!#
#     endif
!#
      cs2=sqrt(P2*(dPdei2/rho2)/rho2 + dPdrho2)
      va2=abs(Bc22)/sqrt(rho2)
      !
      do i1=m1,n1
        do i3=m3,n3
          B_sq     = Bc12(i1,i3)**2+Bc22(i1,i3)**2+Bc32(i1,i3)**2
          B_sq_tan = Bc12(i1,i3)**2+Bc32(i1,i3)**2

          if (B_sq_tan > epsilon * (P2(i1,i3)+0.5*B_sq)) then
            caq      =cs2(i1,i3)**2+B_sq/rho2(i1,i3)
            dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs2(i1,i3)*va2(i1,i3)/caq)**2))
            vf2(i1,i3)=sqrt(0.5*(caq + dd))
          else
            if (cs2(i1,i3) - va2(i1,i3) > epsilon * cs2(i1,i3)) then
              vf2(i1,i3) = cs2(i1,i3)
            else if (va2(i1,i3) - cs2(i1,i3) > epsilon * cs2(i1,i3)) then
              vf2(i1,i3) = va2(i1,i3)
            else
              vf2(i1,i3) = cs2(i1,i3)
            endif
          endif
        end do ! i3
      end do ! i1
      !
      ! --- Compute characteristic time scales ---
      time_char2=(model%xb2(i2+1)-model%xb2(i2)) / (0.01*vf2 + abs(v22))
      time_T_corr2=time_char2/par%C_TChange
!!$    write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                    minval(time_T_corr), maxval(time_T_corr)
      !
      ! --- Find inflow columns ---
      mask_inflow2=(v22 < 0.0)
      !
      ! --- Modify internal energy to come closer to specified inflow temperature ---
      where (mask_inflow2)
        dei_r2=dei_r2 + dtime/time_T_corr2 * (T_inflow-T2)/dTdei2
      end where
      !
      if (par%C_visBound > 0.0) then
        ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
        v2org2=v22
        where (mask_inflow2)
          v22=v22 * max( (1.0-par%C_visBound*dtime/(model%xb2(i2+1)-model%xb2(i2))*cs2 * &
                         (1.0+(abs(v22)/cs2)**2)) &
                       ,0.0)
          dei_r2=dei_r2 + 0.5 * (v2org2**2 - v22**2)
        end where
      endif
      !
      model%v2( :,i2,:)=v22
      !
    endif
    !
  endif
  !
  if (n3 > m3) then
    !
    if ((action%bound_l3 == bound_transmitting ) .or. &
        (action%bound_l3 == bound_transmitting2) .or. &
        (action%bound_l3 == bound_transmitting3)       ) then
      !
      i3=m3
      !
      rho3=model%rho(:,:,i3)
      ei3= model%ei( :,:,i3)
      v33= model%v3( :,:,i3)
      Bc13=model%Bc1(:,:,i3)
      Bc23=model%Bc2(:,:,i3)
      Bc33=model%Bc3(:,:,i3)
      !
      ! --- Solve EOS ---
!#
#     ifdef rhd_bound_t01
!#
      call timing_start('MHD: bound3d: eosinter')
!#
#     endif
!#
      call eosinter_all(rho3, ei3, &
                        P=P3, dPdrho=dPdrho3, dPde=dPdei3, &
                        T=T3,                 dTde=dTdei3, &
                        s=s3)
!#
#     ifdef rhd_bound_t01
!#
      call timing_stop( 'MHD: bound3d: eosinter')
!#
#     endif
!#
      cs3=sqrt(P3*(dPdei3/rho3)/rho3 + dPdrho3)
      va3=abs(Bc33)/sqrt(rho3)
      !
      do i1=m1,n1
        do i2=m2,n2
          B_sq     = Bc13(i1,i2)**2+Bc23(i1,i2)**2+Bc33(i1,i2)**2
          B_sq_tan = Bc13(i1,i2)**2+Bc23(i1,i2)**2

          if (B_sq_tan > epsilon * (P3(i1,i2)+0.5*B_sq)) then
            caq      =cs3(i1,i2)**2+B_sq/rho3(i1,i2)
            dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs3(i1,i2)*va3(i1,i2)/caq)**2))
            vf3(i1,i2)=sqrt(0.5*(caq + dd))
          else
            if (cs3(i1,i2) - va3(i1,i2) > epsilon * cs3(i1,i2)) then
              vf3(i1,i2) = cs3(i1,i2)
            else if (va3(i1,i2) - cs3(i1,i2) > epsilon * cs3(i1,i2)) then
              vf3(i1,i2) = va3(i1,i2)
            else
              vf3(i1,i2) = cs3(i1,i2)
            endif
          endif
        end do ! i2
      end do ! i1
      !
      ! --- Compute characteristic time scales ---
      time_char3=(model%xb3(i3+1)-model%xb3(i3)) / (0.01*vf3 + abs(v33))
      time_T_corr3=time_char3/par%C_TChange
!!$    write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                    minval(time_T_corr), maxval(time_T_corr)
      !
      ! --- Find inflow columns ---
      mask_inflow3=(v33 > 0.0)
      !
      ! --- Modify internal energy to come closer to specified inflow temperature ---
      where (mask_inflow3)
        dei_l3=dei_l3 + dtime/time_T_corr3 * (T_inflow-T3)/dTdei3
      end where
      !
      if (par%C_visBound > 0.0) then
        ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
        v3org3=v33
        where (mask_inflow3)
          v33=v33 * max( (1.0-par%C_visBound*dtime/(model%xb3(i3+1)-model%xb3(i3))*cs3 * &
                         (1.0+(abs(v33)/cs3)**2)) &
                       ,0.0)
          dei_l3=dei_l3 + 0.5 * (v3org3**2 - v33**2)
        end where
      endif
      !
      model%v3( :,:,i3)=v33
      !
    endif
    !
    if ((action%bound_r3 == bound_transmitting ) .or. &
        (action%bound_r3 == bound_transmitting2) .or. &
        (action%bound_r3 == bound_transmitting3)       ) then
      !
      i3=n3
      !
      rho3=model%rho(:,:,i3)
      ei3= model%ei( :,:,i3)
      v33= model%v3( :,:,i3)
      Bc13=model%Bc1(:,:,i3)
      Bc23=model%Bc2(:,:,i3)
      Bc33=model%Bc3(:,:,i3)
      !
      ! --- Solve EOS ---
!#
#     ifdef rhd_bound_t01
!#
      call timing_start('MHD: bound3d: eosinter')
!#
#     endif
!#
      call eosinter_all(rho3, ei3, &
                        P=P3, dPdrho=dPdrho3, dPde=dPdei3, &
                        T=T3,                 dTde=dTdei3, &
                        s=s3)
!#
#     ifdef rhd_bound_t01
!#
      call timing_stop( 'MHD: bound3d: eosinter')
!#
#     endif
!#
      cs3=sqrt(P3*(dPdei3/rho3)/rho3 + dPdrho3)
      va3=abs(Bc33)/sqrt(rho3)
      !
      do i1=m1,n1
        do i2=m2,n2
          B_sq     = Bc13(i1,i2)**2+Bc23(i1,i2)**2+Bc33(i1,i2)**2
          B_sq_tan = Bc13(i1,i2)**2+Bc23(i1,i2)**2

          if (B_sq_tan > epsilon * (P3(i1,i2)+0.5*B_sq)) then
            caq      =cs3(i1,i2)**2+B_sq/rho3(i1,i2)
            dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs3(i1,i2)*va3(i1,i2)/caq)**2))
            vf3(i1,i2)=sqrt(0.5*(caq + dd))
          else
            if (cs3(i1,i2) - va3(i1,i2) > epsilon * cs3(i1,i2)) then
              vf3(i1,i2) = cs3(i1,i2)
            else if (va3(i1,i2) - cs3(i1,i2) > epsilon * cs3(i1,i2)) then
              vf3(i1,i2) = va3(i1,i2)
            else
              vf3(i1,i2) = cs3(i1,i2)
            endif
          endif
        end do ! i2
      end do ! i1
      !
      ! --- Compute characteristic time scales ---
      time_char3=(model%xb3(i3+1)-model%xb3(i3)) / (0.01*vf3 + abs(v33))
      time_T_corr3=time_char3/par%C_TChange
!!$    write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                    minval(time_T_corr), maxval(time_T_corr)
      !
      ! --- Find inflow columns ---
      mask_inflow3=(v33 < 0.0)
      !
      ! --- Modify internal energy to come closer to specified inflow temperature ---
      where (mask_inflow3)
        dei_r3=dei_r3 + dtime/time_T_corr3 * (T_inflow-T3)/dTdei3
      end where
      !
      if (par%C_visBound > 0.0) then
        ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
        v3org3=v33
        where (mask_inflow3)
          v33=v33 * max( (1.0-par%C_visBound*dtime/(model%xb3(i3+1)-model%xb3(i3))*cs3 * &
                         (1.0+(abs(v33)/cs3)**2)) &
                       ,0.0)
          dei_r3=dei_r3 + 0.5 * (v3org3**2 - v33**2)
        end where
      endif
      !
      model%v3( :,:,i3)=v33
      !
    endif
    !
  endif
  !
  ! --- Apply energy corrections ---
  model%ei(m1,:,:)=model%ei(m1,:,:)+dei_l1
  model%ei(n1,:,:)=model%ei(n1,:,:)+dei_r1
  model%ei(:,m2,:)=model%ei(:,m2,:)+dei_l2
  model%ei(:,n2,:)=model%ei(:,n2,:)+dei_r2
  model%ei(:,:,m3)=model%ei(:,:,m3)+dei_l3
  model%ei(:,:,n3)=model%ei(:,:,n3)+dei_r3
  !
endif
!
end subroutine rhd_bound_3DTransmit_mhd


!----------************************-----------------------------------------------------------------
subroutine rhd_bound_1DTransmit_mhd(model, action, dtime, grav_flag, phib_grav, Hp_l, Hp_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_1DTransmit_mhd ('rhd_boundary_conditions_1D_Transmitting_magneto-hydrodynamics')
!
! PURPOSE:
!   Correct values in top or bottom cells to fullfill the open transmitting boundary condition.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_1DTransmit_mhd(model, action, dtime, grav_flag, phib_grav, Hp_l=Hp_l, Hp_r=Hp_r)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   Hp_l:         ('Height_pressure_left') real, averaged pressure scale height on left (m1) side
!   Hp_r:         ('Height_pressure_right') real, averaged pressure scale height on right (n1) side
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Parameters from par are used. The EOS is solved with the standard table.
!
! RESTRICTIONS:
!   There are no parallelization directives: the routine is intended to be used inside
!   a parallel block.
!
! PROCEDURE:
!   This routine realizes part of the open upper or outer boundary condition.
!   Each side is handled separately.
!   First, the EOS is solved for a boundary layers.
!   The hydrostatic pressure scale is computed for further use (extrapolation of the density)
!   by rhd_bound_1DGhost.
!   In inflow cells in a boundary layer the internal energy is modified to come closer towards
!   a specified temperature.
!   Additionally, in inflow cells the inflow velocity may be reduced to increase stability (usually
!   not used).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-09-28 Written by Bernd Freytag, Astronomical Observatory Copenhagen
!   1999-09-29 (B.F.) Correct temperature for BOTH boundaries
!   1999-10-01 (B.F.) Characteristic time local
!   2000-03-22 (B.F.) Reduce inflow velocity, parameters Hp_l, Hp_r added
!   2000-03-25 (B.F.) Parameters Hp_l, Hp_r become arrays
!   2000-03-27 (B.F.) Corrections in computation of Hp_l, Hp_r
!   2001-02-06 (B.F.) Put rho, ei, v1 into temporary arrays; call 2D version of eosinter
!                     compute area not with spread and matmul anymore
!   2001-02-13 (B.F.) Hp_l and Hp_r are computed from local P and rho (and phi_grav) only
!   2001-04-09 (B.F.) Formula for cs changed (possible overflow)
!   2010-04-28 (B.F. Lyon) bound_transmitting3
!   2015-04-01 (B.F., O.S.) Compute dd with max
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_bound_t01
!#
use timing_module
!#
# endif
!#
use gasinter_module
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: grav_flag
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: phib_grav
real, dimension(model%m2:model%n2, &
                model%m3:model%n3), &
                 optional, intent(out)  :: Hp_l, Hp_r
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3
integer                                 :: bound_l, bound_r
real                                    :: T_inflow, dd, caq, B_sq, B_sq_tan, eiTable_min
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: rho, ei, v1, Bc1, Bc2, Bc3, &
                                           P, dPdei, dPdrho, T, T2, dTdei, s, cs, va, vf, v1org, &
                                           time_char, time_T_corr
logical, dimension(model%m2:model%n2, &
                   model%m3:model%n3)   :: mask_inflow
real, parameter                         :: epsilon=0.000001
!---------------------------------------------------------------------------------------------------
!
call tabinter_getinfo(x2min=eiTable_min)

! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! \label{f90:teff__useMHD2}\label{f90:c_tsurf__useMHD2}
! --- Compute target temperature of inflowing material from effective temperature ---
! --- and reduction factor                                                        ---
T_inflow=par%Teff*par%C_Tsurf
!
if ((bound_l == bound_transmitting ) .or. &
    (bound_l == bound_transmitting2) .or. &
    (bound_l == bound_transmitting3)       ) then
  ! === Lower boundary ===
  !
  i1=m1
  !
  rho=model%rho(i1,:,:)
  ei= model%ei( i1,:,:)
  v1= model%v1( i1,:,:)
  Bc1=model%Bc1(i1,:,:)
  Bc2=model%Bc2(i1,:,:)
  Bc3=model%Bc3(i1,:,:)
  !
  ! --- Solve EOS ---
!#
# ifdef rhd_bound_t01
!#
  call timing_start('MHD: bound1d: eosinter')
!#
# endif
!#
  call eosinter_all(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
!#
# ifdef rhd_bound_t01
!#
  call timing_stop( 'MHD: bound1d: eosinter')
!#
# endif
!#
!!$  cs=sqrt(P*dPdei/rho**2 + dPdrho)
  cs=sqrt(P*(dPdei/rho)/rho + dPdrho)
  va=abs(Bc1)/sqrt(rho)
  !
  do i3=m3,n3
    do i2=m2,n2
      B_sq     = Bc1(i2,i3)**2+Bc2(i2,i3)**2+Bc3(i2,i3)**2
      B_sq_tan = Bc2(i2,i3)**2+Bc3(i2,i3)**2

      if (B_sq_tan > epsilon * (P(i2,i3)+0.5*B_sq)) then
        caq      =cs(i2,i3)**2+B_sq/rho(i2,i3)
        dd       =caq*sqrt(max(0.0, 1.0 - 4.0*(cs(i2,i3)*va(i2,i3)/caq)**2))
        vf(i2,i3)=sqrt(0.5*(caq + dd))
      else
        if (cs(i2,i3) - va(i2,i3) > epsilon * cs(i2,i3)) then
          vf(i2,i3) = cs(i2,i3)
        else if (va(i2,i3) - cs(i2,i3) > epsilon * cs(i2,i3)) then
          vf(i2,i3) = va(i2,i3)
        else
          vf(i2,i3) = cs(i2,i3)
        endif
      endif
    end do ! i2
  end do ! i3
  !
  ! --- Pressure scale height ---
  if (present(Hp_l)) then
    if ((grav_flag) .and. (bound_l /= bound_transmitting3)) then
      Hp_l=P / (abs(phib_grav(i1+1,:,:)-phib_grav(i1,:,:))/(model%xb1(i1+1)-model%xb1(i1)) * rho)
!      Hp_l=dPdrho / (abs(phib_grav(i1+1,:,:)-phib_grav(i1,:,:))/(model%xb1(i1+1)-model%xb1(i1)))
    else
      Hp_l=0.0
    endif
  endif
  !
  ! --- Compute characteristic time scales ---
  time_char=(model%xb1(i1+1)-model%xb1(i1)) / (0.01*vf + abs(v1))
  time_T_corr=time_char/par%C_TChange
!!$  write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                  minval(time_T_corr), maxval(time_T_corr)
  !
  ! --- Find inflow columns ---
  mask_inflow=(v1 > 0.0)
  !
  ! --- Modify internal energy to come closer to specified inflow temperature ---
  where (mask_inflow)
    ei=ei + dtime/time_T_corr * (T_inflow-T)/dTdei
  end where
  !
  ! \label{f90:c_visbound__useMHD1}
  if (par%C_visBound > 0.0) then
    ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
    v1org=v1
    where (mask_inflow)
      v1=v1 * max( (1.0-par%C_visBound*dtime/(model%xb1(i1+1)-model%xb1(i1))*cs * &
                      (1.0+(abs(v1)/cs)**2)) &
                   ,0.0)
      ei=ei + 0.5 * (v1org**2 - v1**2)
    end where
  endif
  !
!  model%rho(i1,:,:)=rho
  model%ei( i1,:,:)=ei
  model%v1( i1,:,:)=v1
  !
endif
!
if ((bound_r == bound_transmitting ) .or. &
    (bound_r == bound_transmitting2) .or. &
    (bound_r == bound_transmitting3)       ) then
  ! === Upper boundary ===
  !
  i1=n1
  !
  rho=model%rho(i1,:,:)
  ei= model%ei( i1,:,:)
  v1= model%v1( i1,:,:)
  Bc1=model%Bc1(i1,:,:)
  Bc2=model%Bc2(i1,:,:)
  Bc3=model%Bc3(i1,:,:)
  !
  ! --- Solve EOS ---
!#
# ifdef rhd_bound_t01
!#
  call timing_start('MHD: bound1d: eosinter')
!#
# endif
!#
  call eosinter_all(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
!#
# ifdef rhd_bound_t01
!#
  call timing_stop( 'MHD: bound1d: eosinter')
!#
# endif
!#
!!$  cs=sqrt(P*dPdei/rho**2 + dPdrho)
  cs=sqrt(P*(dPdei/rho)/rho + dPdrho)
  va=abs(Bc1)/sqrt(rho)
  !
  do i3=m3,n3
    do i2=m2,n2
      B_sq     = Bc1(i2,i3)**2+Bc2(i2,i3)**2+Bc3(i2,i3)**2
      B_sq_tan = Bc2(i2,i3)**2+Bc3(i2,i3)**2

      if (B_sq_tan > epsilon * (P(i2,i3)+0.5*B_sq)) then
        caq      =cs(i2,i3)**2+B_sq/rho(i2,i3)
        dd       =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i2,i3)*va(i2,i3)/caq)**2))
        vf(i2,i3)=sqrt(0.5*(caq + dd))
      else
        if (cs(i2,i3) - va(i2,i3) > epsilon * cs(i2,i3)) then
          vf(i2,i3) = cs(i2,i3)
        else if (va(i2,i3) - cs(i2,i3) > epsilon * cs(i2,i3)) then
          vf(i2,i3) = va(i2,i3)
        else
          vf(i2,i3) = cs(i2,i3)
        endif
      endif
    end do ! i2
  end do ! i3
  !
  ! --- Pressure scale height ---
  if (present(Hp_r)) then
    if ((grav_flag) .and. (bound_r /= bound_transmitting3)) then
      Hp_r=P / (abs(phib_grav(i1+1,:,:)-phib_grav(i1,:,:))/(model%xb1(i1+1)-model%xb1(i1)) * rho)
!      Hp_r=dPdrho / (abs(phib_grav(i1+1,:,:)-phib_grav(i1,:,:))/(model%xb1(i1+1)-model%xb1(i1)))
    else
      Hp_r=0.0
    endif
  endif
  !
  ! \label{f90:c_tchange__useMHD2}
  ! --- Compute characteristic time scales ---
  time_char=(model%xb1(i1+1)-model%xb1(i1)) / (0.01*vf + abs(v1))
  time_T_corr=time_char/par%C_TChange
!!$  write(6,'(A,4E15.8)') 'time_char, time_T_corr', minval(time_char), maxval(time_char), &
!!$                                                  minval(time_T_corr), maxval(time_T_corr)
!!$  write(6,'(A,3E15.8)') 'time_char, time_T_corr', time_char, time_T_corr
  !
  ! --- Find inflow columns ---
  mask_inflow=(v1 < 0.0)
  !
  ! --- Modify internal energy to come closer to specified inflow temperature ---
  where (mask_inflow)
    ei=ei + dtime/time_T_corr * (T_inflow-T)/dTdei
  end where
  !
  call eosinter_all(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T2,                dTde=dTdei, &
                    s=s)

  ! \label{f90:c_visbound__useMHD2}
  if (par%C_visBound > 0.0) then
    ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
    v1org=v1
    where (mask_inflow)
      v1=v1 * max( (1.0-par%C_visBound*dtime/(model%xb1(i1+1)-model%xb1(i1))*cs * &
                     (1.0+(abs(v1)/cs)**2)) &
                   ,0.0)
      ei=ei + 0.5 * (v1org**2 - v1**2)
    end where
  endif
  !
!  model%rho(i1,:,:)=rho
  model%ei( i1,:,:)=ei
  model%v1( i1,:,:)=v1
  !
endif
!
end subroutine rhd_bound_1DTransmit_mhd


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Roe1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
                             bound_l, bound_r, reconstruction_order, reconstruction_type, dim, &
                             C_SlopeRed, &
                             dtime, xb1, wl, &
                             rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, dpde, dpdrho, &
                             cs, va, vf, vs, sqrt_rho, &
                             phic_grav, phib_grav, &
                             v1_til, v2_til, v3_til, &
                             flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                             flux_Bc1, flux_Bc2, flux_Bc3, &
                             flux_rhoeikm, fep)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Roe1Dflux ('rhd_magneto-hydrodynamics_Roe_1D_flux')
!
! PURPOSE:
!   Compute 1D MHD Roe fluxes.
!
! CATEGORY:
!   Magnetohydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Roe1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
!                          bound_l, bound_r, reconstruction_order, reconstruction_type, dim, &
!                          C_SlopeRed, &
!                          dtime, xb1, wl, &
!                          rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, dpde, dpdrho, &
!                          cs, va, vf, vs, sqrt_rho, &
!                          phic_grav, phib_grav, &
!                          v1_til, v2_til, v3_til, &
!                          flux_rho, flux_rhov1, flux_rhov2, flux_rhov3,
!                          flux_Bc1, flux_Bc2, flux_Bc3, &
!                          flux_rhoeikm, fep)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   bound_l:      ('boundary_left_1') integer, left boundary type for dimension 1
!                   values (constants defined in rhd_action_module):
!                     bound_None, bound_reflective, bound_periodic, bound_constant,
!                     bound_closed, bound_inoutflow, bound_closedbottom,
!                     bound_transmitting, bound_transmitting2, bound_transmitting3
!   bound_r:      ('boundary_right_1') integer, right boundary type for dimension 1
!                   values (constants defined in rhd_action_module):
!                     bound_None, bound_reflective, bound_periodic, bound_constant,
!                     bound_closed, bound_inoutflow, bound_closedbottom,
!                     bound_transmitting, bound_transmitting2, bound_transmitting3
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   C_SlopeRed:   ('Constant_Slope_Reduction') real, parameter for slope reduction in case
!                 of strong density contrast (default: 0.0: off)
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   dpde:         ('delta_pressure_delta_energy') real 3D array, pressure derivative with respect to
!                 thermal energy on model grid
!   dpdrho:       ('delta_pressure_delta_rho') real 3D array, pressure derivative with respect to
!                 density on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   va:           ('velocity_alfven') real 3D array, Alfven velocity on model grid
!   vf:           ('velocity_fast') real 3D array, fast magnetosonic speed on model grid
!   vs:           ('velocity_slow') real 3D array, slow magnetosonic speed on model grid
!   sqrt_rho:     ('square_root_rho') real 3D array, square root of density on model grid
!   phic_grav:    ('phi_center_gravitation') real, dimension(:,:,:), cell centered grav. potential
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!
! OUTPUT:
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe averaged velocity 3 at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_Bc1:     ('flux_B_center_1') real 3D array
!   flux_Bc2:     ('flux_B_center_2') real 3D array
!   flux_Bc3:     ('flux_B_center_3') real 3D array
!   flux_rhoeikm: ('flux_rho_energy_internal_kinetik_magnetic') real 3D array,
!                 total energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!
! LOCAL VARIABLES:
!   P_red:        ('pressure_reduced') real 4D array, hydrostatic pressure correction
!
! ROUTINES:
!   rhd_hyd_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   rhd_hyd_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   It is assumed that there are no ghost zones present in "orthogonal directions":
!   model%nghost2=0, model%nghost3=0
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-03-23 (W.S.) First version
!   2008-11-02 (W.S.) Major revision:
!                     MHD Roe averages due to Cargo & Gallice
!                     Vinokur variables kappa and chi for handling of non-polytropic EOS
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_rec_module
use gasinter_module
use rhd_phys_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           bound_l, bound_r, &
                                           reconstruction_order, reconstruction_type, dim
real,                      intent(in)   :: C_SlopeRed, dtime
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m1-nghost1+1: &
                n1+nghost1), &
                           intent(in)   :: wl
real, dimension(m1-nghost1:n1+nghost1,&
                m2        :n2        ,&
                m3        :n3      ), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, dpde, dpdrho, &
                                           cs, va, vf, vs, sqrt_rho, &
                                           phic_grav
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2        :n2        ,&
                m3        :n3      ), &
                           intent(in)   :: phib_grav
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3    ),intent(out)  :: v1_til, v2_til, v3_til, &
                                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_Bc1, flux_Bc2, flux_Bc3, flux_rhoeikm, fep
!
! --- Local variables ---
integer                                 :: i1, i2, i3, j, nb1, nt1, nb2, nt2, nb3, nt3
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3    )              :: rho_til, P_til, ei_til, eip_til, sqrt_rho_til, &
                                           Bc1_til, Bc2_til, Bc3_til, &
                                           cs_til, va_til, vf_til, vs_til, x, kappa_til, chi_til, &
                                           alpha_f, alpha_s, beta_y, beta_z, &
                                           lambda_til_fast_mi, lambda_til_alfven_mi, &
                                           lambda_til_slow_mi, lambda_til_slow_pl, &
                                           lambda_til_alfven_pl, lambda_til_fast_pl
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3   )               :: P_avg
integer, dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3  )             :: i_up_fast_mi, i_up_alfven_mi, i_up_slow_mi, i_up_nu, &
                                           i_up_slow_pl, i_up_alfven_pl, i_up_fast_pl
real, dimension(m1:n1+1, &
                -reconstruction_order: &
                  reconstruction_order, &
                m2:n2  , &
                m3:n3     )             :: alpha_til_1, alpha_til_2, &
                                           alpha_til_3, alpha_til_4, &
                                           alpha_til_5, alpha_til_6, &
                                           alpha_til_7
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3     )             :: flux_alpha_til_1, flux_alpha_til_2, &
                                           flux_alpha_til_3, flux_alpha_til_4, &
                                           flux_alpha_til_5, flux_alpha_til_6, &
                                           flux_alpha_til_7, flux_alpha_til_c, sigma

!#
# if (rhd_roe1d_slope_l01 != 0)
!#
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3     )             :: flux_alpha_til_mm, Cmm

!#
# endif /* ---  rhd_roe1d_slope_l01 --- */
!#
real, dimension(m1:n1+1, &
                -reconstruction_order-1: &
                  reconstruction_order, &
                m2:n2  , &
                m3:n3     )             :: P_red
real, dimension(m1:n1+1)            :: mask_bound_l, mask_bound_r
real, parameter                         :: C_dxb1_maxvar=0.01, C_max_Predfac=0.99, &
                                           epsilon=0.000001
real                                    :: dxb1_avg, dxb1_var, &
                                           v_sq_til, B_sq_til, caq, dd, B_sq_tan, &
                                           sgnBc1, cs_sq, Bt, vt, vc, sqrt_2, &
                                           rho_avg, v1_avg, v2_avg, v3_avg, &
                                           Bc1_avg, Bc2_avg, Bc3_avg, &
                                           rho_mean_m2, rho_mean_m1, &
                                           rho_mean_p0, rho_mean_p1, rho_mean_p2, &
                                           wrho, delta, &
                                           mask_up_fast_mi, mask_up_alfven_mi, mask_up_slow_mi, &
                                           mask_up_slow_pl, mask_up_alfven_pl, mask_up_fast_pl, &
                                           mask_up_nu
!---------------------------------------------------------------------------------------------------
sqrt_2=sqrt(0.5)
!
! === Boundary conditions III ===
!
mask_bound_l=0.0
mask_bound_r=0.0
! --- Constant pressure at boundary ---
if ((bound_l == bound_closed) .or. (bound_l == bound_closedbottom)) then
  mask_bound_l(m1)=1.0
endif
if ((bound_r == bound_closed) .or. (bound_r == bound_closedbottom)) then
  mask_bound_r(n1+1)=1.0
endif
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- State averages (linear) ---
      P_avg(   i1,i2,i3)=     wl(i1) * P(    i1-1,i2,i3) + &
                         (1.0-wl(i1))* P(    i1  ,i2,i3)
      !
      ! --- Roe averages: ---
      rho_til( i1,i2,i3)=sqrt_rho(i1-1,i2,i3)*sqrt_rho(i1,i2,i3)
      wrho              =sqrt_rho(i1-1,i2,i3)/(sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3))
      v1_til(  i1,i2,i3)=wrho*(v1(i1-1,i2,i3) - v1(  i1,i2,i3)) + v1(  i1,i2,i3)
      v2_til(  i1,i2,i3)=wrho*(v2(i1-1,i2,i3) - v2(  i1,i2,i3)) + v2(  i1,i2,i3)
      v3_til(  i1,i2,i3)=wrho*(v3(i1-1,i2,i3) - v3(  i1,i2,i3)) + v3(  i1,i2,i3)
      Bc1_til( i1,i2,i3)=wrho*(Bc1(  i1,i2,i3) - Bc1(i1-1,i2,i3)) + Bc1(i1-1,i2,i3)
      Bc2_til( i1,i2,i3)=wrho*(Bc2(  i1,i2,i3) - Bc2(i1-1,i2,i3)) + Bc2(i1-1,i2,i3)
      Bc3_til( i1,i2,i3)=wrho*(Bc3(  i1,i2,i3) - Bc3(i1-1,i2,i3)) + Bc3(i1-1,i2,i3)
      ei_til(  i1,i2,i3)=wrho*(ei(i1-1,i2,i3) - ei(  i1,i2,i3)) + ei(  i1,i2,i3)
      !
      ! --- Derived Roe averages: ---
      x(i1,i2,i3)       =0.5*((Bc2(i1-1,i2,i3) - Bc2(i1,i2,i3))**2 + &
                              (Bc3(i1-1,i2,i3) - Bc3(i1,i2,i3))**2) / &
                         (sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3))**2
      P_til(i1,i2,i3)   =wrho*(P(  i1,i2,i3) - P(i1-1,i2,i3)) + P(i1-1,i2,i3) + &
                         (0.5 * rho_til(i1,i2,i3) * ((v1(i1-1,i2,i3) - v1(i1,i2,i3))**2 + &
                                                     (v2(i1-1,i2,i3) - v2(i1,i2,i3))**2 + &
                                                     (v3(i1-1,i2,i3) - v3(i1,i2,i3))**2) / &
                         (sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3))**2 + &
                           2.0 * x(i1,i2,i3)) * rho_til(i1,i2,i3)
      eip_til(i1,i2,i3) =ei_til(i1,i2,i3) + P_til(i1,i2,i3) / rho_til(i1,i2,i3) + &
                         0.5 * (v1_til(i1,i2,i3)**2+v2_til(i1,i2,i3)**2+v3_til(i1,i2,i3)**2)
      sqrt_rho_til(i1,i2,i3)=sqrt(rho_til(i1,i2,i3))
      !
      ! === Boundary conditions III ===
      P_til(i1,i2,i3)=(1.0-mask_bound_l(i1)-mask_bound_r(i1)) * P_til(i1  ,i2,i3) + &
                                            mask_bound_l(i1)  * P(    i1  ,i2,i3) + &
                                            mask_bound_r(i1)  * P(    i1-1,i2,i3)
      !
      ! --- Averages of kappa, chi and sound speed ---
      kappa_til(i1,i2,i3)=0.5 * (dpde(i1-1,i2,i3) / rho(i1-1,i2,i3) + &
                                 dpde(  i1,i2,i3) / rho(  i1,i2,i3))
      chi_til(i1,i2,i3)  =0.5 * (dpdrho(i1-1,i2,i3) - &
                                 dpde(i1-1,i2,i3) * ei(i1-1,i2,i3) / rho(i1-1,i2,i3) + &
                                 dpdrho(  i1,i2,i3) - &
                                 dpde(  i1,i2,i3) * ei(  i1,i2,i3) / rho(  i1,i2,i3)) + &
                                (1.0 - kappa_til(i1,i2,i3)) * x(i1,i2,i3)
      cs_til(    i1,i2,i3)=sqrt(kappa_til(i1,i2,i3) * &
                                (ei_til(i1,i2,i3) + P_til(i1,i2,i3)/rho_til(i1,i2,i3)) + &
                                 chi_til(i1,i2,i3))
      va_til(    i1,i2,i3)=abs(Bc1_til(i1,i2,i3))/sqrt_rho_til(i1,i2,i3)
      B_sq_tan=Bc2_til(i1,i2,i3)**2 + Bc3_til(i1,i2,i3)**2
      B_sq_til=B_sq_tan + Bc1_til(i1,i2,i3)**2
      !
      ! --- Calculate normalization factors alpha_f, alpha_s, beta_y, beta_z ---
      ! --- to handle certain degeneracies                                   ---
      if (B_sq_tan > epsilon * (P_til(i1,i2,i3) + 0.5 * B_sq_til)) then
        ! --- No degeneracies ---
        caq               =cs_til(i1,i2,i3)**2+B_sq_til/rho_til(i1,i2,i3)
        dd                =caq * sqrt(max(0.0, 1.0 - 4.0*(cs_til(i1,i2,i3)*va_til(i1,i2,i3)/caq)**2))
        !
        vf_til(    i1,i2,i3)=sqrt(0.5*(caq+dd))
        vs_til(    i1,i2,i3)=cs_til(i1,i2,i3)*va_til(i1,i2,i3)/vf_til(i1,i2,i3)
        !
        beta_y(i1,i2,i3)=Bc2_til(i1,i2,i3)/sqrt(B_sq_tan)
        beta_z(i1,i2,i3)=Bc3_til(i1,i2,i3)/sqrt(B_sq_tan)
        !
        alpha_f(i1,i2,i3)=sqrt((cs_til(i1,i2,i3)**2-vs_til(i1,i2,i3)**2)/dd)
        alpha_s(i1,i2,i3)=sqrt((vf_til(i1,i2,i3)**2-cs_til(i1,i2,i3)**2)/dd)
        !
      else
        ! --- Weak transversal magnetic field, eiter fast or slow magnetosonic ---
        ! --- wave coincides with Alfven wave                                  ---
        beta_y(i1,i2,i3)=sqrt_2
        beta_z(i1,i2,i3)=sqrt_2
        !
        if (cs_til(i1,i2,i3) - va_til(i1,i2,i3) > epsilon * cs_til(i1,i2,i3)) then
          ! --- Slow wave coincides with Alfven wave ---
          alpha_f(i1,i2,i3)=1.0
          alpha_s(i1,i2,i3)=0.0
          vf_til(i1,i2,i3)=cs_til(i1,i2,i3)
          vs_til(i1,i2,i3)=va_til(i1,i2,i3)
        else if (cs_til(i1,i2,i3) - va_til(i1,i2,i3) < epsilon * cs_til(i1,i2,i3)) then
          ! --- Fast wave coincides with Alfven wave ---
          alpha_f(i1,i2,i3)=0.0
          alpha_s(i1,i2,i3)=1.0
          vf_til(i1,i2,i3)=va_til(i1,i2,i3)
          vs_til(i1,i2,i3)=cs_til(i1,i2,i3)
        else
          ! --- Fast, slow and Alfven wave coincide, triple umbilic point ---
          alpha_f(i1,i2,i3)=sqrt_2
          alpha_s(i1,i2,i3)=sqrt_2
          vf_til(i1,i2,i3)=cs_til(i1,i2,i3)
          vs_til(i1,i2,i3)=cs_til(i1,i2,i3)
        endif
        !
      endif
      !
      ! === Characteristic velocities at cell boundaries ===
      !
      ! --- Characteristic velocities ---
      lambda_til_fast_mi(  i1,i2,i3)=v1_til(i1,i2,i3)-vf_til(i1,i2,i3)
      lambda_til_alfven_mi(i1,i2,i3)=v1_til(i1,i2,i3)-va_til(i1,i2,i3)
      lambda_til_slow_mi(  i1,i2,i3)=v1_til(i1,i2,i3)-vs_til(i1,i2,i3)
      lambda_til_slow_pl(  i1,i2,i3)=v1_til(i1,i2,i3)+vs_til(i1,i2,i3)
      lambda_til_alfven_pl(i1,i2,i3)=v1_til(i1,i2,i3)+va_til(i1,i2,i3)
      lambda_til_fast_pl(  i1,i2,i3)=v1_til(i1,i2,i3)+vf_til(i1,i2,i3)
      ! --- lambda_til_nu( i1,i2,i3)=v1_til(i1,i2,i3) --- removed because of optimization ---
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Entropy fix: modified LeVeque formula (1991) ---
      if (((v1(i1  ,i2,i3) - vf(i1  ,i2,i3)) > 0.0) .and. &
          ((v1(i1-1,i2,i3) - vf(i1-1,i2,i3)) < 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#
        if (lambda_til_fast_mi(i1,i2,i3) > 0.0) then
          lambda_til_fast_mi(i1,i2,i3)=v1(i1  ,i2,i3) - vf(i1  ,i2,i3)
        else
          lambda_til_fast_mi(i1,i2,i3)=v1(i1-1,i2,i3) - vf(i1-1,i2,i3)
        endif
!#
# else /* rhd_hyd_entropyfix_p01 */
!#
        mask_up_fast_mi=sign(0.5,lambda_til_fast_mi(i1,i2,i3))+0.5
        lambda_til_fast_mi(i1,i2,i3)=(1.0-mask_up_fast_mi) * (v1(i1-1,i2,i3) - vf(i1-1,i2,i3)) + &
                                     mask_up_fast_mi  * (v1(i1  ,i2,i3) - vf(i1  ,i2,i3))
!#
# endif /* rhd_hyd_entropyfix_p01 */
!#
      endif
      !
      if (((v1(i1  ,i2,i3) - va(i1  ,i2,i3)) > 0.0) .and. &
          ((v1(i1-1,i2,i3) - va(i1-1,i2,i3)) < 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#
        if (lambda_til_alfven_mi(i1,i2,i3) > 0.0) then
          lambda_til_alfven_mi(i1,i2,i3)=v1(i1  ,i2,i3) - va(i1  ,i2,i3)
        else
          lambda_til_alfven_mi(i1,i2,i3)=v1(i1-1,i2,i3) - va(i1-1,i2,i3)
        endif
!#
# else /* rhd_hyd_entropyfix_p01 */
!#
        mask_up_alfven_mi=sign(0.5,lambda_til_alfven_mi(i1,i2,i3))+0.5
        lambda_til_alfven_mi(i1,i2,i3)=(1.0-mask_up_alfven_mi) * (v1(i1-1,i2,i3) - va(i1-1,i2,i3))&
                                   + mask_up_alfven_mi  * (v1(i1  ,i2,i3) - va(i1  ,i2,i3))
!#
# endif /* rhd_hyd_entropyfix_p01 */
!#
      endif
      !
      if (((v1(i1  ,i2,i3) - vs(i1  ,i2,i3)) > 0.0) .and. &
          ((v1(i1-1,i2,i3) - vs(i1-1,i2,i3)) < 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#
        if (lambda_til_slow_mi(i1,i2,i3) > 0.0) then
          lambda_til_slow_mi(i1,i2,i3)=v1(i1  ,i2,i3) - vs(i1  ,i2,i3)
        else
          lambda_til_slow_mi(i1,i2,i3)=v1(i1-1,i2,i3) - vs(i1-1,i2,i3)
        endif
!#
# else /* rhd_hyd_entropyfix_p01 */
!#
        mask_up_slow_mi=sign(0.5,lambda_til_slow_mi(i1,i2,i3))+0.5
        lambda_til_slow_mi(i1,i2,i3)=(1.0-mask_up_slow_mi) * (v1(i1-1,i2,i3) - vs(i1-1,i2,i3)) + &
                                     mask_up_slow_mi  * (v1(i1  ,i2,i3) - vs(i1  ,i2,i3))
!#
# endif /* rhd_hyd_entropyfix_p01 */
!#
      endif
      !
      if (((v1(i1-1,i2,i3) + vs(i1-1,i2,i3)) < 0.0) .and. &
          ((v1(i1  ,i2,i3) + vs(i1  ,i2,i3)) > 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#
        if (lambda_til_slow_pl(i1,i2,i3) >= 0.0) then
          lambda_til_slow_pl(i1,i2,i3)=v1(i1  ,i2,i3) + vs(i1  ,i2,i3)
        else
          lambda_til_slow_pl(i1,i2,i3)=v1(i1-1,i2,i3) + vs(i1-1,i2,i3)
        endif
!#
# else /* rhd_hyd_entropyfix_p01 */
!#
        mask_up_slow_pl=sign(0.5,lambda_til_slow_pl(i1,i2,i3))+0.5
        lambda_til_slow_pl(i1,i2,i3)=(1.0-mask_up_slow_pl) * (v1(i1-1,i2,i3) + vs(i1-1,i2,i3)) + &
                                     mask_up_slow_pl  * (v1(i1  ,i2,i3) + vs(i1  ,i2,i3))
!#
# endif /* rhd_hyd_entropyfix_p01 */
!#
      endif
      if (((v1(i1  ,i2,i3) + va(i1  ,i2,i3)) > 0.0) .and. &
          ((v1(i1-1,i2,i3) + va(i1-1,i2,i3)) < 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#
        if (lambda_til_alfven_pl(i1,i2,i3) > 0.0) then
          lambda_til_alfven_pl(i1,i2,i3)=v1(i1  ,i2,i3) + va(i1  ,i2,i3)
        else
          lambda_til_alfven_pl(i1,i2,i3)=v1(i1-1,i2,i3) + va(i1-1,i2,i3)
        endif
!#
# else /* rhd_hyd_entropyfix_p01 */
!#
        mask_up_alfven_pl=sign(0.5,lambda_til_alfven_pl(i1,i2,i3))+0.5
        lambda_til_alfven_pl(i1,i2,i3)=(1.0-mask_up_alfven_pl) * (v1(i1-1,i2,i3) + va(i1-1,i2,i3))&
                                   + mask_up_alfven_pl  * (v1(i1  ,i2,i3) + va(i1  ,i2,i3))
!#
# endif /* rhd_hyd_entropyfix_p01 */
!#
      endif
      !
      !
      if (((v1(i1-1,i2,i3) + vf(i1-1,i2,i3)) < 0.0) .and. &
          ((v1(i1  ,i2,i3) + vf(i1  ,i2,i3)) > 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#
        if (lambda_til_fast_pl(i1,i2,i3) >= 0.0) then
          lambda_til_fast_pl(i1,i2,i3)=v1(i1  ,i2,i3) + vf(i1  ,i2,i3)
        else
          lambda_til_fast_pl(i1,i2,i3)=v1(i1-1,i2,i3) + vf(i1-1,i2,i3)
        endif
!#
# else /* rhd_hyd_entropyfix_p01 */
!#
        mask_up_fast_pl=sign(0.5,lambda_til_fast_pl(i1,i2,i3))+0.5
        lambda_til_fast_pl(i1,i2,i3)=(1.0-mask_up_fast_pl) * (v1(i1-1,i2,i3) + vf(i1-1,i2,i3)) + &
                                     mask_up_fast_pl  * (v1(i1  ,i2,i3) + vf(i1  ,i2,i3))
!#
# endif /* rhd_hyd_entropyfix_p01 */
!#
      endif
      !
      !
      ! --- Find Upwind direction ---
      !
!#
# if (rhd_hyd_upwind_p01 == 0)
!#
      if (lambda_til_fast_mi(i1,i2,i3) > 0.0) then
        i_up_fast_mi(i1,i2,i3)=i1-1
      else
        i_up_fast_mi(i1,i2,i3)=i1
      endif
      if (lambda_til_alfven_mi(i1,i2,i3) > 0.0) then
        i_up_alfven_mi(i1,i2,i3)=i1-1
      else
        i_up_alfven_mi(i1,i2,i3)=i1
      endif
      if (lambda_til_slow_mi(i1,i2,i3) > 0.0) then
        i_up_slow_mi(i1,i2,i3)=i1-1
      else
        i_up_slow_mi(i1,i2,i3)=i1
      endif
      if (v1_til(i1,i2,i3) > 0.0) then
        i_up_nu(i1,i2,i3)=i1-1
      else
        i_up_nu(i1,i2,i3)=i1
      endif
      if (lambda_til_slow_pl(i1,i2,i3) > 0.0) then
        i_up_slow_pl(i1,i2,i3)=i1-1
      else
        i_up_slow_pl(i1,i2,i3)=i1
      endif
      if (lambda_til_alfven_pl(i1,i2,i3) > 0.0) then
        i_up_alfven_pl(i1,i2,i3)=i1-1
      else
        i_up_alfven_pl(i1,i2,i3)=i1
      endif
      if (lambda_til_fast_pl(i1,i2,i3) > 0.0) then
        i_up_fast_pl(i1,i2,i3)=i1-1
      else
        i_up_fast_pl(i1,i2,i3)=i1
      endif
!#
# else /* rhd_hyd_upwind_p01 */
!#
      i_up_fast_mi(  i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_fast_mi(  i1,i2,i3))-0.5)
      i_up_alfven_mi(i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_alfven_mi(i1,i2,i3))-0.5)
      i_up_slow_mi(  i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_slow_mi(  i1,i2,i3))-0.5)
      i_up_nu(       i1,i2,i3)=i1 + int(sign(0.6,-v1_til(              i1,i2,i3))-0.5)
      i_up_slow_pl(  i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_slow_pl(  i1,i2,i3))-0.5)
      i_up_alfven_pl(i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_alfven_pl(i1,i2,i3))-0.5)
      i_up_fast_pl(  i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_fast_pl(  i1,i2,i3))-0.5)
!#
# endif /* rhd_hyd_upwind_p01 */
!#
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Reduced gravitational potential: Int((rho*dphi/dz)*dz) ---
!#
# if (rhd_hyd_gravcorr_p01 == 0) /* 'off0' */
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      P_red(i1  , 0,i2,i3)=0.0
      P_red(i1  ,-1,i2,i3)=0.0
    end do ! i1
  end do ! i2
end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 1) /* 'default' */
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      rho_mean_p0=(1.0-wl(i1  ))*rho(i1-1,i2,i3) + &
                       wl(i1  ) *rho(i1  ,i2,i3)
      P_red(i1  , 0,i2,i3)=rho_mean_p0 * &
                         (phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
      P_red(i1  ,-1,i2,i3)=rho_mean_p0 * &
                         (phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 3) /* 'default3' */
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
      P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
      !
      P_red(i1  , 0,i2,i3)=min(max(P_red(i1  , 0,i2,i3), &
                                   min(P(i1  ,i2,i3)-P(i1-1,i2,i3), 0.0)), &
                               0.0) + &
                           max(min(P_red(i1  , 0,i2,i3), &
                                   max(P(i1  ,i2,i3)-P(i1-1,i2,i3), 0.0)), &
                               0.0)
      P_red(i1  ,-1,i2,i3)=min(max(P_red(i1  ,-1,i2,i3), &
                                   min(P(i1-1,i2,i3)-P(i1  ,i2,i3), 0.0)), &
                               0.0) + &
                           max(min(P_red(i1  ,-1,i2,i3), &
                                   max(P(i1-1,i2,i3)-P(i1  ,i2,i3), 0.0)), &
                               0.0)
      !
      P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                      (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 4) /* 'default4' */
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
      P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
      !
      !
      P_avg(i1,i2,i3)=(     wl(i1) /rho(i1-1,i2,i3) * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                       (1.0-wl(i1))/rho(i1  ,i2,i3) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))  > / &
                      (wl(i1) /rho(i1-1,i2,i3) +  (1.0-wl(i1))/rho(i1  ,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 5) /* 'default5' */
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
      P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
      !
      P_red(i1  , 0,i2,i3)=min(P_red(i1  , 0,i2,i3), C_max_Predfac*P(i1  ,i2,i3))
      P_red(i1  ,-1,i2,i3)=min(P_red(i1  ,-1,i2,i3), C_max_Predfac*P(i1-1,i2,i3))
      !
      P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                      (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!#
# else /* rhd_hyd_gravcorr_p01 == 2 'default2' */
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
      P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
      !
      P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                      (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!#
# endif /* rhd_hyd_gravcorr_p01 */
!#
if (reconstruction_order >= 1) then
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        rho_mean_p1=(1.0-wl(i1+1))*rho(i1  ,i2,i3) + &
                         wl(i1+1) *rho(i1+1,i2,i3)
        rho_mean_m1=(1.0-wl(i1-1))*rho(i1-2,i2,i3) + &
                         wl(i1-1) *rho(i1-1,i2,i3)
        P_red(i1  , 1,i2,i3)=P_red(i1  , 0,i2,i3) + rho_mean_p1 * &
                                                  (phic_grav(i1+1,i2,i3)-phic_grav(i1  ,i2,i3))
        P_red(i1  ,-2,i2,i3)=P_red(i1  ,-1,i2,i3) + rho_mean_m1 * &
                                                  (phic_grav(i1-2,i2,i3)-phic_grav(i1-1,i2,i3))
      end do ! i1
    end do ! i2
  end do ! i3
endif
if (reconstruction_order >= 2) then
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        rho_mean_p2=(1.0-wl(i1+2))*rho(i1+1,i2,i3) + &
                         wl(i1+2) *rho(i1+2,i2,i3)
        rho_mean_m2=(1.0-wl(i1-2))*rho(i1-3,i2,i3) + &
                         wl(i1-2) *rho(i1-2,i2,i3)
        P_red(i1  , 2,i2,i3)=P_red(i1  , 1,i2,i3) + rho_mean_p2 * &
                                                  (phic_grav(i1+2,i2,i3)-phic_grav(i1+1,i2,i3))
        P_red(i1  ,-3,i2,i3)=P_red(i1  ,-2,i2,i3) + rho_mean_m2 * &
                                                  (phic_grav(i1-3,i2,i3)-phic_grav(i1-2,i2,i3))
      end do ! i1
    end do ! i2
  end do ! i3
endif!
! === Projection ===
!
! --- Projection coefficients ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('MHD: Roe1Dflux: alpha_til')
!#
# endif
!#
!
select case (reconstruction_order)
case(0)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                (1.0-wl(i1))* rho(i1  ,i2,i3)
      v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                (1.0-wl(i1))* v1( i1  ,i2,i3)
      v2_avg   =     wl(i1) * v2( i1-1,i2,i3) + &
                (1.0-wl(i1))* v2( i1  ,i2,i3)
      v3_avg   =     wl(i1) * v3( i1-1,i2,i3) + &
                (1.0-wl(i1))* v3( i1  ,i2,i3)
      Bc2_avg  =     wl(i1) * Bc2(i1-1,i2,i3) + &
                (1.0-wl(i1))* Bc2(i1  ,i2,i3)
      Bc3_avg  =     wl(i1) * Bc3(i1-1,i2,i3) + &
                (1.0-wl(i1))* Bc3(i1  ,i2,i3)
      cs_sq = cs_til(i1,i2,i3)**2
      sgnBc1=sign(1.0, Bc1_til(i1,i2,i3))

      j=0

        alpha_til_1(i1,j,i2,i3) = -0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_2(i1,j,i2,i3) = sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_3(i1,j,i2,i3) = -0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_4(i1,j,i2,i3) = rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                  1.0/cs_til(i1,i2,i3)**2 * &
                                          (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_5(i1,j,i2,i3) = 0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - rho_avg))


        alpha_til_6(i1,j,i2,i3) = -sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_7(i1,j,i2,i3) = 0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - rho_avg))

    end do ! i1
  end do ! i2
end do ! i3
!
case(1)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                (1.0-wl(i1))* rho(i1  ,i2,i3)
      v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                (1.0-wl(i1))* v1( i1  ,i2,i3)
      v2_avg   =     wl(i1) * v2( i1-1,i2,i3) + &
                (1.0-wl(i1))* v2( i1  ,i2,i3)
      v3_avg   =     wl(i1) * v3( i1-1,i2,i3) + &
                (1.0-wl(i1))* v3( i1  ,i2,i3)
      Bc2_avg  =     wl(i1) * Bc2(i1-1,i2,i3) + &
                (1.0-wl(i1))* Bc2(i1  ,i2,i3)
      Bc3_avg  =     wl(i1) * Bc3(i1-1,i2,i3) + &
                (1.0-wl(i1))* Bc3(i1  ,i2,i3)
      cs_sq = cs_til(i1,i2,i3)**2
      sgnBc1=sign(1.0, Bc1_til(i1,i2,i3))
!DIR$ UNROLL
      do j=-1,1
        alpha_til_1(i1,j,i2,i3) = -0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_2(i1,j,i2,i3) = sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_3(i1,j,i2,i3) = -0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_4(i1,j,i2,i3) = rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                  1.0/cs_til(i1,i2,i3)**2 * &
                                          (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_5(i1,j,i2,i3) = 0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_6(i1,j,i2,i3) = -sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_7(i1,j,i2,i3) = 0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - rho_avg))

      end do ! j
    end do ! i1
  end do ! i2
end do ! i3
!
case(2)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                (1.0-wl(i1))* rho(i1  ,i2,i3)
      v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                (1.0-wl(i1))* v1( i1  ,i2,i3)
      v2_avg   =     wl(i1) * v2( i1-1,i2,i3) + &
                (1.0-wl(i1))* v2( i1  ,i2,i3)
      v3_avg   =     wl(i1) * v3( i1-1,i2,i3) + &
                (1.0-wl(i1))* v3( i1  ,i2,i3)
      Bc2_avg  =     wl(i1) * Bc2(i1-1,i2,i3) + &
                (1.0-wl(i1))* Bc2(i1  ,i2,i3)
      Bc3_avg  =     wl(i1) * Bc3(i1-1,i2,i3) + &
                (1.0-wl(i1))* Bc3(i1  ,i2,i3)
      cs_sq = cs_til(i1,i2,i3)**2
      sgnBc1=sign(1.0, Bc1_til(i1,i2,i3))
!DIR$ UNROLL
      do j=-2,2
        alpha_til_1(i1,j,i2,i3) = -0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_2(i1,j,i2,i3) = sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_3(i1,j,i2,i3) = -0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_4(i1,j,i2,i3) = rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                  1.0/cs_til(i1,i2,i3)**2 * &
                                          (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_5(i1,j,i2,i3) = 0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_6(i1,j,i2,i3) = -sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_7(i1,j,i2,i3) = 0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - rho_avg))

      end do ! j
    end do ! i1
  end do ! i2
end do ! i3
!
case default
do i3=m3,n3
  do i2=m2,n2
    do j=-reconstruction_order,reconstruction_order
      do i1=m1,n1+1
        rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                  (1.0-wl(i1))* rho(i1  ,i2,i3)
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        v2_avg   =     wl(i1) * v2( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v2( i1  ,i2,i3)
        v3_avg   =     wl(i1) * v3( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v3( i1  ,i2,i3)
        Bc2_avg  =     wl(i1) * Bc2(i1-1,i2,i3) + &
                  (1.0-wl(i1))* Bc2(i1  ,i2,i3)
        Bc3_avg  =     wl(i1) * Bc3(i1-1,i2,i3) + &
                  (1.0-wl(i1))* Bc3(i1  ,i2,i3)
        cs_sq = cs_til(i1,i2,i3)**2
        sgnBc1=sign(1.0, Bc1_til(i1,i2,i3))

        alpha_til_1(i1,j,i2,i3) = -0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_2(i1,j,i2,i3) = sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_3(i1,j,i2,i3) = -0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_mi(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_4(i1,j,i2,i3) = rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                  1.0/cs_til(i1,i2,i3)**2 * &
                                          (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_5(i1,j,i2,i3) = 0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v1_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - v3_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  0.5 * alpha_f(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_slow_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_slow_pl(i1,i2,i3)+j,i2,i3) - rho_avg))

        alpha_til_6(i1,j,i2,i3) = -sqrt_2 * beta_z(i1,i2,i3) * sgnBc1 * &
                                          (v2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v2_avg) + &
                                  sqrt_2 * beta_y(i1,i2,i3) * sgnBc1 * &
                                          (v3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  sqrt_2 * beta_z(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc2(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) - &
                                  sqrt_2 * beta_y(i1,i2,i3) / sqrt_rho_til(i1,i2,i3) * &
                                          (Bc3(i_up_alfven_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg)

        alpha_til_7(i1,j,i2,i3) = 0.5 * alpha_f(i1,i2,i3) * vf_til(i1,i2,i3) / cs_sq * &
                                          (v1(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v1_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_y(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v2_avg) - &
                                  0.5 * alpha_s(i1,i2,i3) * vs_til(i1,i2,i3) * &
                                           beta_z(i1,i2,i3) * sgnBc1 / cs_sq * &
                                          (v3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - v3_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_y(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc2(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc2_avg) + &
                                  0.5 * alpha_s(i1,i2,i3) * beta_z(i1,i2,i3) / &
                                          (sqrt_rho_til(i1,i2,i3) * cs_til(i1,i2,i3)) * &
                                          (Bc3(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - Bc3_avg) + &
                                  0.5 * alpha_f(i1,i2,i3) / (rho_til(i1,i2,i3) * cs_sq) * &
                                          (P(i_up_fast_pl(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) - &
                                           P_red(i1,i_up_fast_pl(i1,i2,i3)-i1+j,i2,i3) + &
                            x(i1,i2,i3) * (rho(i_up_fast_mi(i1,i2,i3)+j,i2,i3) - rho_avg))

      end do ! i1
    end do ! j
  end do ! i2
end do ! i3
!
end select
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('MHD: Roe1Dflux: alpha_til')
!#
# endif
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('MHD: Roe1Dflux: fluxalpha')
!#
# endif
!#
!
! === Reconstruction (constant, slope limiter, PP), fluxes of partial waves ===
!
!
dxb1_avg=(xb1(n1+1+reconstruction_order+1)-xb1(m1-reconstruction_order-1))/ &
         float(n1-m1+3+2*reconstruction_order)
dxb1_var=(maxval(abs( xb1(m1-reconstruction_order  : n1+1+reconstruction_order+1)- &
                      xb1(m1-reconstruction_order-1: n1+1+reconstruction_order  )-dxb1_avg) ))/ &
         dxb1_avg
!
if (dxb1_var > C_dxb1_maxvar) then
  ! --- Spatially varying grid ---
  !
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_1, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_fast_mi, lambda_til_fast_mi, &
!!$                         flux_alpha_til_1)
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_2, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_alfven_mi, lambda_til_alfven_mi, &
!!$                         flux_alpha_til_2)
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_3, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_slow_mi, lambda_til_slow_mi, &
!!$                         flux_alpha_til_3)
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_4, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_nu, v1_til, &
!!$                         flux_alpha_til_4)
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_5, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_slow_pl, lambda_til_slow_pl, &
!!$                         flux_alpha_til_5)
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_6, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_alfven_pl, lambda_til_alfven_pl, &
!!$                         flux_alpha_til_6)
!!$  call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                         reconstruction_order, reconstruction_type, &
!!$                         dtime, &
!!$                         alpha_til_7, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                         i_up_fast_pl, lambda_til_fast_pl, &
!!$                         flux_alpha_til_7)
  !
!#
# if (rhd_roe1d_slope_l01 == 1)
!#
  ! --- Slope reduction in case of rarefaction wave ---
  if ((C_SlopeRed > 0.0) .and. &
      ((reconstruction_type == reconstruction_type_vanLeer) .or. &
       (reconstruction_type == reconstruction_type_SuperBee) .or. &
       (reconstruction_type == reconstruction_type_PP))) then
    ! write(6,*) 'Additional slope reduction'
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_1, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_fast_mi, lambda_til_fast_mi, &
!!$                           flux_alpha_til_mm)
!!$    do i3=m3,n3
!!$      do i2=m2,n2
!!$        Cmm(:,i2,i3)=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,i2,i3)-vf(m1:n1+1,i2,i3))-(v1(m1-1:n1,i2,i3)-vf(m1-1:n1,i2,i3)),0.0)* &
!!$              2.0*dtime/(xb1(m1+1:n1+2)-xb1(m1-1:n1)), &
!!$            1.0)
!!$      end do ! i2
!!$    end do ! i3
!!$    flux_alpha_til_1=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_1
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_3, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_slow_mi, lambda_til_slow_mi, &
!!$                           flux_alpha_til_mm)
!!$    do i3=m3,n3
!!$      do i2=m2,n2
!!$        Cmm(:,i2,i3)=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,i2,i3)-vs(m1:n1+1,i2,i3))-(v1(m1-1:n1,i2,i3)-vs(m1-1:n1,i2,i3)),0.0)* &
!!$              2.0*dtime/(xb1(m1+1:n1+2)-xb1(m1-1:n1)), &
!!$            1.0)
!!$      end do ! i2
!!$    end do ! i3
!!$    flux_alpha_til_3=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_3
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_5, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_slow_pl, lambda_til_slow_pl, &
!!$                           flux_alpha_til_mm)
!!$    do i3=m3,n3
!!$      do i2=m2,n2
!!$        Cmm(:,i2,i3)=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,i2,i3)+vs(m1:n1+1,i2,i3))-(v1(m1-1:n1,i2,i3)+vs(m1-1:n1,i2,i3)),0.0)* &
!!$              2.0*dtime/(xb1(m1+1:n1+2)-xb1(m1-1:n1)), &
!!$            1.0)
!!$      end do ! i2
!!$    end do ! i3
!!$    flux_alpha_til_5=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_5
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_7, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_fast_pl, lambda_til_fast_pl, &
!!$                           flux_alpha_til_mm)
!!$    do i3=m3,n3
!!$      do i2=m2,n2
!!$        Cmm(:,i2,i3)=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,i2,i3)+vf(m1:n1+1,i2,i3))-(v1(m1-1:n1,i2,i3)+vf(m1-1:n1,i2,i3)),0.0)* &
!!$              2.0*dtime/(xb1(m1+1:n1+2)-xb1(m1-1:n1)), &
!!$            1.0)
!!$      end do ! i2
!!$    end do ! i3
!!$    flux_alpha_til_7=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_7
!!$  else
!!$    ! write(6,*) 'No extra slope reduction'
!!$  endif
!!$!#
!!$# elif (rhd_roe1d_slope_l01 == 2)
!!$!#
!!$  ! --- Slope reduction in case of strong density gradient ---
!!$  if ((C_SlopeRed > 0.0) .and. &
!!$      ((reconstruction_type == reconstruction_type_vanLeer) .or. &
!!$       (reconstruction_type == reconstruction_type_SuperBee) .or. &
!!$       (reconstruction_type == reconstruction_type_PP))) then
!!$    Cmm=min(C_SlopeRed * &
!!$            max(rho(m1:n1+1,:,:)/rho(m1-1:n1,:,:)+rho(m1-1:n1,:,:)/rho(m1:n1+1,:,:)-2.5,0.0)**2, &
!!$            1.0)
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_1, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_fast_mi, lambda_til_fast_mi, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_1=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_1
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_2, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_alfven_mi, lambda_til_alfven_mi, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_2=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_2
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_3, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_slow_mi, lambda_til_slow_mi, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_3=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_3
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_4, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_nu, v1_til, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_4=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_4
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_5, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_slow_pl, lambda_til_slow_pl, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_5=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_5
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_6, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_alfven_pl, lambda_til_alfven_pl, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_6=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_6
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Minmod, &
!!$                           dtime, &
!!$                         alpha_til_7, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_fast_pl, lambda_til_fast_pl, &
!!$                           flux_alpha_til_mm)
!!$    flux_alpha_til_7=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_7
    !
  endif
!#
# endif /* ---  rhd_roe1d_slope_l01 --- */
!#
  ! \label{f90:n_magdiff__useMHD1}
  ! --- Slope reduction near boundaries ---
  if (par%N_magDiffSide > 0 .or. par%N_magDiffTop > 0 .or. par%N_magDiffBottom > 0) then
    !
    if (dim == 3) then
      nb1 = par%N_magDiffBottom
      nt1 = par%N_magDiffTop
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffSide
      nt3 = par%N_magDiffSide
    else
      nb1 = par%N_magDiffSide
      nt1 = par%N_magDiffSide
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffBottom
      nt3 = par%N_magDiffTop
    endif
    !
    do i1 = m1, n1 + 1
      do i2 = m2, n2
        do i3 = m3, n3
          !
          if (n2 > m2) then
            delta = min((i1 - m1     + epsilon) / (nb1 + epsilon), &
                        (n1 + 1 - i1 + epsilon) / (nt1 + epsilon), &
                        (i2 - m2     + epsilon) / (nb2 + epsilon), &
                        (n2 - i2     + epsilon) / (nt2 + epsilon), &
                        (i3 - m3     + epsilon) / (nb3 + epsilon), &
                        (n3 - i3     + epsilon) / (nt3 + epsilon))
          else
            delta = min((i1 - m1     + epsilon) / (nb1 + epsilon), &
                        (n1 + 1 - i1 + epsilon) / (nt1 + epsilon), &
                        (i3 - m3     + epsilon) / (nb3 + epsilon), &
                        (n3 - i3     + epsilon) / (nt3 + epsilon))
          endif
          !
          if (delta < 1.0) then
            sigma(i1,i2,i3) = 1.0 / (1.0 + exp(10.0 * (0.5 - delta)))
          else
            sigma(i1,i2,i3) = 1.0
          endif
          !
        end do
      end do
    end do
    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_1, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_fast_mi, lambda_til_fast_mi, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_1=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_1
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_2, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_alfven_mi, lambda_til_alfven_mi, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_2=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_2
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_3, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_slow_mi, lambda_til_slow_mi, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_3=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_3
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_4, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_nu, v1_til, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_4=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_4
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_5, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_slow_pl, lambda_til_slow_pl, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_5=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_5
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_6, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_alfven_pl, lambda_til_alfven_pl, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_6=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_6
!!$    !
!!$    call rhd_hyd_Fluxalpha(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type_Constant, &
!!$                           dtime, &
!!$                         alpha_til_7, xb1(m1-reconstruction_order-1: n1+1+reconstruction_order+1),&
!!$                           i_up_fast_pl, lambda_til_fast_pl, &
!!$                           flux_alpha_til_c)
!!$    flux_alpha_til_7=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_7
    !
  endif
  !
else
  ! --- Equidistant grid ---
  !
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_1, dxb1_avg, &
!!$                           lambda_til_fast_mi, &
!!$                           flux_alpha_til_1)
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_2, dxb1_avg, &
!!$                           lambda_til_alfven_mi, &
!!$                           flux_alpha_til_2)
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_3, dxb1_avg, &
!!$                           lambda_til_slow_mi, &
!!$                           flux_alpha_til_3)
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_4, dxb1_avg, &
!!$                           v1_til, &
!!$                           flux_alpha_til_4)
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_5, dxb1_avg, &
!!$                           lambda_til_slow_pl, &
!!$                           flux_alpha_til_5)
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_6, dxb1_avg, &
!!$                           lambda_til_alfven_pl, &
!!$                           flux_alpha_til_6)
!!$  call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                           reconstruction_order, reconstruction_type, &
!!$                           dtime, &
!!$                           alpha_til_7, dxb1_avg, &
!!$                           lambda_til_fast_pl, &
!!$                           flux_alpha_til_7)
  !
!#
# if (rhd_roe1d_slope_l01 == 1)
!#
  ! --- Slope reduction in case of rarefaction wave ---
!!$  if ((C_SlopeRed > 0.0) .and. &
!!$      ((reconstruction_type == reconstruction_type_vanLeer) .or. &
!!$       (reconstruction_type == reconstruction_type_SuperBee) .or. &
!!$       (reconstruction_type == reconstruction_type_PP))) then
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_1, dxb1_avg, &
!!$                             lambda_til_fast_mi, &
!!$                             flux_alpha_til_mm)
!!$    Cmm=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,:,:)-vf(m1:n1+1,:,:))-(v1(m1-1:n1,:,:)-vf(m1-1:n1,:,:)),0.0)* &
!!$              dtime/dxb1_avg, &
!!$            1.0)
!!$    flux_alpha_til_1=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_1
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_3, dxb1_avg, &
!!$                             lambda_til_slow_mi, &
!!$                             flux_alpha_til_mm)
!!$    Cmm=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,:,:)-vs(m1:n1+1,:,:))-(v1(m1-1:n1,:,:)-vs(m1-1:n1,:,:)),0.0)* &
!!$              dtime/dxb1_avg, &
!!$            1.0)
!!$    flux_alpha_til_3=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_3
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_5, dxb1_avg, &
!!$                             lambda_til_slow_pl, &
!!$                             flux_alpha_til_mm)
!!$    Cmm=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,:,:)+vs(m1:n1+1,:,:))-(v1(m1-1:n1,:,:)+vs(m1-1:n1,:,:)),0.0)* &
!!$              dtime/dxb1_avg, &
!!$            1.0)
!!$    flux_alpha_til_5=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_5
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_7, dxb1_avg, &
!!$                             lambda_til_fast_mi, &
!!$                             flux_alpha_til_mm)
!!$    Cmm=min(C_SlopeRed* &
!!$            max((v1(m1:n1+1,:,:)+vf(m1:n1+1,:,:))-(v1(m1-1:n1,:,:)+vf(m1-1:n1,:,:)),0.0)* &
!!$              dtime/dxb1_avg, &
!!$            1.0)
!!$    flux_alpha_til_7=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_7
  endif
!#
# elif (rhd_roe1d_slope_l01 == 2)
!#
  ! --- Slope reduction in case of strong density gradient ---
  if ((C_SlopeRed > 0.0) .and. &
      ((reconstruction_type == reconstruction_type_vanLeer) .or. &
       (reconstruction_type == reconstruction_type_SuperBee) .or. &
       (reconstruction_type == reconstruction_type_PP))) then
!!$    Cmm=min(C_SlopeRed * &
!!$            max(rho(m1:n1+1,:,:)/rho(m1-1:n1,:,:)+rho(m1-1:n1,:,:)/rho(m1:n1+1,:,:)-2.5,0.0)**2, &
!!$            1.0)
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_1, dxb1_avg, &
!!$                             lambda_til_fast_mi, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_1=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_1
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_2, dxb1_avg, &
!!$                             lambda_til_alfven_mi, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_2=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_2
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_3, dxb1_avg, &
!!$                             lambda_til_slow_mi, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_3=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_3
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_4, dxb1_avg, &
!!$                             v1_til, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_4=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_4
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_5, dxb1_avg, &
!!$                             lambda_til_slow_pl, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_5=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_5
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_6, dxb1_avg, &
!!$                             lambda_til_alfven_pl, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_6=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_6
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Minmod, &
!!$                             dtime, &
!!$                             alpha_til_7, dxb1_avg, &
!!$                             lambda_til_fast_pl, &
!!$                             flux_alpha_til_mm)
!!$    flux_alpha_til_7=Cmm*flux_alpha_til_mm + (1.0-Cmm)*flux_alpha_til_7
    !
  endif
!#
# endif /* ---  rhd_roe1d_slope_l01 --- */
!#
  ! \label{f90:n_magdiff__useMHD2}
  ! --- Slope reduction near boundaries ---
  if (par%N_magDiffSide > 0.0 .or. par%N_magDiffTop > 0 .or. par%N_magDiffBottom > 0) then
    !
    if (dim == 3) then
      nb1 = par%N_magDiffBottom
      nt1 = par%N_magDiffTop
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffSide
      nt3 = par%N_magDiffSide
    else
      nb1 = par%N_magDiffSide
      nt1 = par%N_magDiffSide
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffBottom
      nt3 = par%N_magDiffTop
    endif
    !
    do i1 = m1, n1 + 1
      do i2 = m2, n2
        do i3 = m3, n3
          !
          if (n2 > m2) then
            delta = min((i1 - m1     + epsilon) / (nb1 + epsilon), &
                        (n1 + 1 - i1 + epsilon) / (nt1 + epsilon), &
                        (i2 - m2     + epsilon) / (nb2 + epsilon), &
                        (n2 - i2     + epsilon) / (nt2 + epsilon), &
                        (i3 - m3     + epsilon) / (nb3 + epsilon), &
                        (n3 - i3     + epsilon) / (nt3 + epsilon))
          else
            delta = min((i1 - m1     + epsilon) / (nb1 + epsilon), &
                        (n1 + 1 - i1 + epsilon) / (nt1 + epsilon), &
                        (i3 - m3     + epsilon) / (nb3 + epsilon), &
                        (n3 - i3     + epsilon) / (nt3 + epsilon))
          endif
          !
          if (delta < 1.0) then
            sigma(i1, i2, i3) = 1.0 / (1.0 + exp(10.0 * (0.5 - delta)))
          else
            sigma(i1, i2, i3) = 1.0
          endif
          !
        end do
      end do
    end do
    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_1, dxb1_avg, &
!!$                             lambda_til_fast_mi, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_1=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_1
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_2, dxb1_avg, &
!!$                             lambda_til_alfven_mi, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_2=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_2
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_3, dxb1_avg, &
!!$                             lambda_til_slow_mi, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_3=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_3
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_4, dxb1_avg, &
!!$                             v1_til, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_4=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_4
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_5, dxb1_avg, &
!!$                             lambda_til_slow_pl, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_5=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_5
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_6, dxb1_avg, &
!!$                             lambda_til_alfven_pl, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_6=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_6
!!$    !
!!$    call rhd_hyd_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!!$                             reconstruction_order, reconstruction_type_Constant, &
!!$                             dtime, &
!!$                             alpha_til_7, dxb1_avg, &
!!$                             lambda_til_fast_pl, &
!!$                             flux_alpha_til_c)
!!$    flux_alpha_til_7=(1.0 - sigma)*flux_alpha_til_c + sigma*flux_alpha_til_7
    !
  endif
  !
endif
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop( 'MHD: Roe1Dflux: fluxalpha', cycles=9)
!#
# endif
!#
!
!
! === Boundary conditions IV ===
!
! --- Set fluxes explicitly to zero ---
if ((bound_l == bound_closed) .or. (bound_l == bound_closedbottom)) then
  flux_alpha_til_1(m1,:,:)=0.0
  flux_alpha_til_3(m1,:,:)=0.0
  flux_alpha_til_5(m1,:,:)=0.0
  flux_alpha_til_7(m1,:,:)=0.0
endif
if ((bound_r == bound_closed) .or. (bound_r == bound_closedbottom)) then
  flux_alpha_til_1(n1+1,:,:)=0.0
  flux_alpha_til_3(n1+1,:,:)=0.0
  flux_alpha_til_5(n1+1,:,:)=0.0
  flux_alpha_til_7(n1+1,:,:)=0.0
endif
!
!
! === Back projection => fluxes ===
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('MHD: Roe1Dflux: fluxes')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1

      bt=beta_y(i1,i2,i3)*Bc2_til(i1,i2,i3)+beta_z(i1,i2,i3)*Bc3_til(i1,i2,i3)
      vt=beta_y(i1,i2,i3)* v2_til(i1,i2,i3)+beta_z(i1,i2,i3)* v3_til(i1,i2,i3)
      vc=beta_z(i1,i2,i3)* v2_til(i1,i2,i3)-beta_y(i1,i2,i3)* v3_til(i1,i2,i3)
      sgnBc1=sign(1.0,Bc1_til(i1,i2,i3))
      B_sq_til=Bc1_til(i1,i2,i3)**2+Bc2_til(i1,i2,i3)**2+Bc3_til(i1,i2,i3)**2

      flux_rho(i1,i2,i3)=   rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*flux_alpha_til_1(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*flux_alpha_til_3(i1,i2,i3) + &
                            flux_alpha_til_4(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*flux_alpha_til_5(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*flux_alpha_til_7(i1,i2,i3) + &
                              wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3) + &
                         (1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)

      flux_rhov1(i1,i2,i3)= rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*(v1_til(i1,i2,i3) - &
                            vf_til(i1,i2,i3))*flux_alpha_til_1(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*(v1_til(i1,i2,i3) - &
                            vs_til(i1,i2,i3))*flux_alpha_til_3(i1,i2,i3) + &
                            v1_til(i1,i2,i3) *flux_alpha_til_4(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*(v1_til(i1,i2,i3) + &
                            vs_til(i1,i2,i3))*flux_alpha_til_5(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*(v1_til(i1,i2,i3) + &
                            vf_til(i1,i2,i3))*flux_alpha_til_7(i1,i2,i3) + &
                            wl(i1) *(rho(i1-1,i2,i3)*v1(i1-1,i2,i3)*v1(i1-1,i2,i3) + &
                            0.5*(Bc2(i1-1,i2,i3)**2+Bc3(i1-1,i2,i3)**2)) + &
                       (1.0-wl(i1))*(rho(i1  ,i2,i3)*v1(i1  ,i2,i3)*v1(i1  ,i2,i3) + &
                            0.5*(Bc2(i1  ,i2,i3)**2+Bc3(i1  ,i2,i3)**2)) + &
                             P_avg(i1,i2,i3) - 0.5 * Bc1_til(i1,i2,i3)**2

      flux_rhov2(i1,i2,i3)= rho_til(i1,i2,i3)*(alpha_f(i1,i2,i3)*v2_til(i1,i2,i3) + &
       alpha_s(i1,i2,i3)*beta_y(i1,i2,i3)*vs_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_1(i1,i2,i3) + &
                    sqrt_2*rho_til(i1,i2,i3)*beta_z(i1,i2,i3)*sgnBc1*flux_alpha_til_2(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_s(i1,i2,i3)*v2_til(i1,i2,i3) - &
       alpha_f(i1,i2,i3)*beta_y(i1,i2,i3)*vf_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_3(i1,i2,i3) + &
                            v2_til(i1,i2,i3)*flux_alpha_til_4(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_s(i1,i2,i3)*v2_til(i1,i2,i3) + &
       alpha_f(i1,i2,i3)*beta_y(i1,i2,i3)*vf_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_5(i1,i2,i3) - &
                    sqrt_2*rho_til(i1,i2,i3)*beta_z(i1,i2,i3)*sgnBc1*flux_alpha_til_6(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_f(i1,i2,i3)*v2_til(i1,i2,i3) - &
       alpha_s(i1,i2,i3)*beta_y(i1,i2,i3)*vs_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_7(i1,i2,i3) + &
        wl(i1) *(v2(i1-1,i2,i3)*rho(i1-1,i2,i3)*v1(i1-1,i2,i3)-Bc1_til(i1,i2,i3)*Bc2(i1-1,i2,i3)) + &
   (1.0-wl(i1))*(v2(i1  ,i2,i3)*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)-Bc1_til(i1,i2,i3)*Bc2(i1  ,i2,i3))

      flux_rhov3(i1,i2,i3)= rho_til(i1,i2,i3)*(alpha_f(i1,i2,i3)*v3_til(i1,i2,i3) + &
       alpha_s(i1,i2,i3)*beta_z(i1,i2,i3)*vs_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_1(i1,i2,i3) - &
                    sqrt_2*rho_til(i1,i2,i3)*beta_y(i1,i2,i3)*sgnBc1*flux_alpha_til_2(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_s(i1,i2,i3)*v3_til(i1,i2,i3) - &
       alpha_f(i1,i2,i3)*beta_z(i1,i2,i3)*vf_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_3(i1,i2,i3) + &
                            v3_til(i1,i2,i3)*flux_alpha_til_4(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_s(i1,i2,i3)*v3_til(i1,i2,i3) + &
       alpha_f(i1,i2,i3)*beta_z(i1,i2,i3)*vf_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_5(i1,i2,i3) + &
                    sqrt_2*rho_til(i1,i2,i3)*beta_y(i1,i2,i3)*sgnBc1*flux_alpha_til_6(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_f(i1,i2,i3)*v3_til(i1,i2,i3) - &
       alpha_s(i1,i2,i3)*beta_z(i1,i2,i3)*vs_til(i1,i2,i3)*sgnBc1) * flux_alpha_til_7(i1,i2,i3) + &
        wl(i1) *(v3(i1-1,i2,i3)*rho(i1-1,i2,i3)*v1(i1-1,i2,i3)-Bc1_til(i1,i2,i3)*Bc3(i1-1,i2,i3)) + &
   (1.0-wl(i1))*(v3(i1  ,i2,i3)*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)-Bc1_til(i1,i2,i3)*Bc3(i1  ,i2,i3))

      flux_Bc1(i1,i2,i3)= 0.0

      flux_Bc2(i1,i2,i3)=   sqrt_rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*beta_y(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_1(i1,i2,i3) + &
                      sqrt_2*sqrt_rho_til(i1,i2,i3)*beta_z(i1,i2,i3)*flux_alpha_til_2(i1,i2,i3) - &
                            sqrt_rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*beta_y(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_3(i1,i2,i3) - &
                            sqrt_rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*beta_y(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_5(i1,i2,i3) + &
                      sqrt_2*sqrt_rho_til(i1,i2,i3)*beta_z(i1,i2,i3)*flux_alpha_til_6(i1,i2,i3) + &
                            sqrt_rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*beta_y(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_7(i1,i2,i3) + &
                        wl(i1) *(Bc2(i1-1,i2,i3)*v1(i1-1,i2,i3)-Bc1_til(i1,i2,i3)*v2(i1-1,i2,i3)) + &
                   (1.0-wl(i1))*(Bc2(i1  ,i2,i3)*v1(i1  ,i2,i3)-Bc1_til(i1,i2,i3)*v2(i1  ,i2,i3))

      flux_Bc3(i1,i2,i3)=   sqrt_rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*beta_z(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_1(i1,i2,i3) - &
                      sqrt_2*sqrt_rho_til(i1,i2,i3)*beta_y(i1,i2,i3)*flux_alpha_til_2(i1,i2,i3) - &
                            sqrt_rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*beta_z(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_3(i1,i2,i3) - &
                            sqrt_rho_til(i1,i2,i3)*alpha_f(i1,i2,i3)*beta_z(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_5(i1,i2,i3) - &
                      sqrt_2*sqrt_rho_til(i1,i2,i3)*beta_y(i1,i2,i3)*flux_alpha_til_6(i1,i2,i3) + &
                            sqrt_rho_til(i1,i2,i3)*alpha_s(i1,i2,i3)*beta_z(i1,i2,i3) * &
                            cs_til(i1,i2,i3)*flux_alpha_til_7(i1,i2,i3) + &
                        wl(i1) *(Bc3(i1-1,i2,i3)*v1(i1-1,i2,i3)-Bc1_til(i1,i2,i3)*v3(i1-1,i2,i3)) + &
                   (1.0-wl(i1))*(Bc3(i1  ,i2,i3)*v1(i1  ,i2,i3)-Bc1_til(i1,i2,i3)*v3(i1  ,i2,i3))

      ! --- flux_rhoei=flux_rhoeik - ek_til*flux_rho ---

      flux_rhoeikm(i1,i2,i3)= rho_til(i1,i2,i3)*(alpha_f(i1,i2,i3)*(eip_til(i1,i2,i3) - &
                            v1_til(i1,i2,i3)*vf_til(i1,i2,i3)) + &
                            alpha_s(i1,i2,i3)*(cs_til(i1,i2,i3)*bt/sqrt_rho_til(i1,i2,i3) + &
                            vs_til(i1,i2,i3)*vt*sgnBc1))*flux_alpha_til_1(i1,i2,i3) + &
                            sqrt_2*rho_til(i1,i2,i3)*vc*sgnBc1*flux_alpha_til_2(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_s(i1,i2,i3)*(eip_til(i1,i2,i3) - &
                            v1_til(i1,i2,i3)*vs_til(i1,i2,i3)) - &
                            alpha_f(i1,i2,i3)*(cs_til(i1,i2,i3)*bt/sqrt_rho_til(i1,i2,i3) + &
                            vf_til(i1,i2,i3)*vt*sgnBc1))*flux_alpha_til_3(i1,i2,i3) + &
                            (0.5*(v1_til(i1,i2,i3)**2+v2_til(i1,i2,i3)**2+v3_til(i1,i2,i3)**2) - &
                            chi_til(i1,i2,i3)/kappa_til(i1,i2,i3)) * &
                            flux_alpha_til_4(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_s(i1,i2,i3)*(eip_til(i1,i2,i3) + &
                            v1_til(i1,i2,i3)*vs_til(i1,i2,i3)) - &
                            alpha_f(i1,i2,i3)*(cs_til(i1,i2,i3)*bt/sqrt_rho_til(i1,i2,i3) - &
                            vf_til(i1,i2,i3)*vt*sgnBc1))*flux_alpha_til_5(i1,i2,i3) - &
                            sqrt_2*rho_til(i1,i2,i3)*vc*sgnBc1*flux_alpha_til_6(i1,i2,i3) + &
                            rho_til(i1,i2,i3)*(alpha_f(i1,i2,i3)*(eip_til(i1,i2,i3) + &
                            v1_til(i1,i2,i3)*vf_til(i1,i2,i3)) + &
                            alpha_s(i1,i2,i3)*(cs_til(i1,i2,i3)*bt/sqrt_rho_til(i1,i2,i3) - &
                            vs_til(i1,i2,i3)*vt*sgnBc1))*flux_alpha_til_7(i1,i2,i3) + &
         wl(i1) *((rho(i1-1,i2,i3)*(0.5*(v1(i1-1,i2,i3)**2+v2(i1-1,i2,i3)**2+v3(i1-1,i2,i3)**2) + &
                            ei(i1-1,i2,i3)) + P(i1-1,i2,i3)+ &
                       0.5*(Bc1(i1-1,i2,i3) - Bc1_til(i1,i2,i3)) * &
                           (Bc1(i1-1,i2,i3) + Bc1_til(i1,i2,i3)) + &
                            Bc2(i1-1,i2,i3)**2+Bc3(i1-1,i2,i3)**2)*v1(i1-1,i2,i3) - &
                           (Bc2(i1-1,i2,i3)*v2(i1-1,i2,i3) + &
                            Bc3(i1-1,i2,i3)*v3(i1-1,i2,i3))*Bc1_til(i1,i2,i3)) + &
    (1.0-wl(i1))*((rho(i1  ,i2,i3)*(0.5*(v1(i1  ,i2,i3)**2+v2(i1  ,i2,i3)**2+v3(i1  ,i2,i3)**2) + &
                            ei(i1  ,i2,i3)) + P(i1  ,i2,i3)+ &
                       0.5*(Bc1(i1  ,i2,i3) - Bc1_til(i1,i2,i3)) * &
                           (Bc1(i1  ,i2,i3) + Bc1_til(i1,i2,i3)) + &
                            Bc2(i1  ,i2,i3)**2+Bc3(i1  ,i2,i3)**2)*v1(i1  ,i2,i3) - &
                           (Bc2(i1  ,i2,i3)*v2(i1  ,i2,i3) + &
                            Bc3(i1  ,i2,i3)*v3(i1  ,i2,i3))*Bc1_til(i1,i2,i3))
      !
      fep(i1,i2,i3)=P_til(i1,i2,i3)/rho_til(i1,i2,i3) * (flux_alpha_til_1(i1,i2,i3)+ &
                                                         flux_alpha_til_2(i1,i2,i3)) + &
               wl(i1) * P(i1-1,i2,i3) * v1(i1-1,i2,i3) + &
          (1.0-wl(i1))* P(i1  ,i2,i3) * v1(i1  ,i2,i3)

    end do ! i1
  end do ! i2
end do ! i3

!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('MHD: Roe1Dflux: fluxes')
!#
# endif
!#
end subroutine rhd_mhd_Roe1Dflux


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Roe1Dstep(model, action, dtime, dtime_limit, &
                             output, OutRadialC_flag, AreaFraction, datx, datr, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Roe1Dstep ('rhd_magneto-hydrodynamics_Roe_1D_step')
!
! PURPOSE:
!   Perform 1D magnetohydrodynamics step in x1 direction, employ Roe scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Roe1Dstep(model, action, dtime, dtime_limit, &
!                   output=output, OutRadialC_flag=OutRadialC_flag, AreaFraction=AreaFraction, &
!                   datx=datx, datr=datr, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                 volume averaging of cell centered quantities is performed
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!                   100:   Pressure becomes non-positive
!                   101:   Pressure derivative dPdrho|ei becomes non-positive
!                   102:   Pressure derivative dPdei|rho becomes non-positive
!                   103:   Density becomes non-positive
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   rho:          ('rho') real 3D array, mass density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('velocity_1') real 3D array, velocity, 1st coordinate
!   v2:           ('velocity_2') real 3D array, velocity, 2nd coordinate
!   v3:           ('velocity_3') real 3D array, velocity, 3rd coordinate
!   Bc1:          ('B_center_1') real 3D array, cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2') real 3D array, cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3') real 3D array, cell centered magnetic field, 3. component
!   eip:          ('energy_internal_pressure') real 3D array, internal + pressure energy density
!   rhoei:        ('rho_energy_internal') real 3D array, internal energy density per volume, rho*ei
!   rhov1:        ('rho_velocity_1') real 3D array, momentum, 1st dimension
!   P:            ('Pressure') real 3D array, gas pressure on model grid
!   P_mag:        ('Pressure_magnetic') real 3D array, magnetic pressure on model grid
!   P_tot:        ('Pressure_total') real 3D array, total pressure on model grid
!   dPdei:        ('delta_pressure_delta_energy_internal') real 3D array, pressure derivative
!                 (constant density) on model grid
!   dPdrho:       ('delta_pressure_delta_rho') real 3D array, pressure derivative
!                 (constant internal energy)
!   T:            ('Temperature') real 3D array, temperature on model grid
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array, temperature derivative
!                 (constant density) on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   va:           ('velocity_alfven') real 3D array, Alfven velocity on model grid
!   vf:           ('velocity_fast') real 3D array, fast magnetosonic speed on model grid
!   vs:           ('velocity_slow') real 3D array, slow magnetosonic speed on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   drhoeidpconsts: ('delta_rho_energy_internal_delta_pressure_constant_s') real 3D array,
!                   thermodynamic quantity: drhoei/dP|_s
!   sqrt_rho:     ('square_root_rho') real 3D array, square root of density on model grid
!   phic_grav:    ('phi_central_gravity) real 3D array, gravitational potential, cell centers
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   Hp_l:         ('Height_pressure_left') real 2D array, averaged pressure scale height
!                 on left (m1) side
!   Hp_r:         ('Height_pressure_right') real 2D array, averaged pressure scale height
!                 on right (n1) side
!   wl:           ('weight_left') real 3D array, geometrical weights due to non-equidistant grid
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe averaged velocity 3 at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe averaged sound speed at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_Bc1:     ('flux_B_center_2') real 3D array
!   flux_Bc2:     ('flux_B_center_2') real 3D array
!   flux_Bc3:     ('flux_B_center_3') real 3D array
!   flux_rhoeikm: ('flux_rho_energy_internal_kinetic_magnetic') total energy flux through cell bnd.
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!   sourceBc1_l:  ('source_B_center_1_left') real 3D array, Janhunen source term
!   sourceBc2_l:  ('source_B_center_2_left') real 3D array, Janhunen source term
!   sourceBc3_l:  ('source_B_center_3_left') real 3D array, Janhunen source term
!   sourceBc1_r:  ('source_B_center_1_right') real 3D array, Janhunen source term
!   sourceBc2_r:  ('source_B_center_2_right') real 3D array, Janhunen source term
!   sourceBc3_r:  ('source_B_center_3_right') real 3D array, Janhunen source term
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_bound_1DTransmit_mhd: Correct values in top or bottom cells for transmitting boundary cond
!   rhd_grav:              Compute gravitational potential
!   rhd_hyd_Coriolis:      Apply Coriolis rotation
!   rhd_mhd_Roe1Dflux:     Compute 1D MHD Roe fluxes
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   model%nghost2=0, model%nghost3=0
!
! KEYWORDS:
!   equations of 3D compressible hydrodynamics
!   directional splitting: 1D substep
!   Roe solver
!   higher order by flux-limiters, reconstruction schemes: constant/Minmod,vanLeer,Superbee/PP
!   general EOS (tabulated values)
!   non-equidistant grid possible
!   source terms due to gravity: no operator-splitting here (constant gravity or central potential)
!   see: Roe, Leveque, Strang
!
! PROCEDURE:
!   1. Preliminaries: compute gravitational potential, fill ghost cells, apply boundary conditions
!   2. Solve EOS and compute further quantities from old values on extended grid
!   3. Compute averages and fluxes located at cell boundaries
!   4. Update basic quantities with fluxes and source terms
!
!   Based on similar Routine 'rhd_hyd_Roe1Dstep'
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-03-23 (W.S.) First version
!   2004-10-26 (W.S.) Update of internal energy modified
!   2008-03-24 (W.S. East Lansing) Dual energy calculation
!   2010-02-16 (W.S. Freiburg) c_test2 -> va_max
!   2010-04-10 (W.S. Freiburg) Alfven speed reduction not only for beta_inv = 0.0
!   2010-06-11 (B.F. Lyon) Higher-order terms (in time) in source term for rhov1 removed
!   2011-06-17 (W.S.) Roe solver only version
!   2013-06-10 (B.F.) Request only time-step reduction (no force stop) for small Gamma_1 or P
!   2014-07-19 (B.F. Uppsala) C_visNeu1, C_visNeu2, C_visDrag removed
!   2015-04-01 (B.F., O.S.) Bugfix: compute dd with max
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_bound_module
use rhd_phys_module
!#
# ifdef rhd_roe1d_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output, OutRadialC_flag
real,            optional, intent(in)   :: AreaFraction
type(datx_type), optional, intent(inout):: datx, datr
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: grav_flag, Output_flag, T_flag
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1
integer                                 :: i1, i2, i3
integer                                 :: bound_l, bound_r
real                                    :: dei_min, &
                                           rhonew_min, rhoout, eiout, v1out, v2out, v3out, &
                                           Bc1out, Bc2out, Bc3out, w0, w1, &
                                           einew_min, eiTable_min, Pg, Pm, ei_min, &
                                           source_rhov1, source_eikm, source_ei, &
                                           caq, dd, B_sq_tan, dt
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, dimension(3)                   :: error_loc
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1) :: xc1
real, dimension( &
            model%m1-model%nghost1:&
            model%n1+model%nghost1+1)   :: xb1
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, P_mag, P_tot, cs, va, vf, vs, Gamma_1, &
                                           sqrt_rho, phic_grav
logical, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: energy_switch
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: phib_grav
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: Hp_l, Hp_r
real, dimension( &
            model%m1-model%nghost1+1:&
            model%n1+model%nghost1) :: wl
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3)      :: flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_Bc1, flux_Bc2, flux_Bc3, &
                                           flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                                           v1_til, v2_til, v3_til
real, dimension(model%m1:model%n1)      :: dtimeoverdx
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: dPdei, dPdrho, T, dTdei, s
real, parameter                         :: epsilon=0.000001
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D:')
!#
# endif
!#
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1     =model%m1
n1     =model%n1
m2     =model%m2
n2     =model%n2
m3     =model%m3
n3     =model%n3
nghost1=model%nghost1
!
! --- Output of certain averages ---
if (present(output)) then
  Output_flag=output
else
  Output_flag=.false.
endif
!
call tabinter_getinfo(x2min=eiTable_min)
!
! \label{f90:nu_rotation__useMHD1}
! === Coriolis terms ===============================================================================
!
if ((action%dim == 1) .and. (par%nu_rotation /= 0.0)) then
  call rhd_hyd_Coriolis(model, dtime)
endif
!
! === Boundary conditions I ========================================================================
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
! --- Boundary conditions by filling of ghost cells: axes only ---
call rhd_bound_1DGhost(model, action, grid_mode=1, bulk_mode=0)
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D: grav')
!#
# endif
!#
! === Gravitational potential ======================================================================
call rhd_grav(model, action, grav_flag, phic_grav, phib_grav)
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D: grav')
call timing_start('MHD: Roe1D: boundaries')
call timing_start('MHD: Roe1D: boundaries: phi')
!#
# endif
!#
!
!
! === Boundary conditions II =======================================================================
!
! --- Boundary conditions for gravity: potential set constant in ghost zones ---
if (grav_flag) then
  if (.not. ((bound_l == bound_periodic        ) .or. &
             (bound_l == bound_transmitting    ) .or. &
             (bound_l == bound_transmitting2   ) .or. &
             (bound_l == bound_inoutflow2      ) .or. &
             (bound_l == bound_inoutflow3      ) .or. &
             (bound_l == bound_closedbottom2   ) .or. &
             (bound_l == bound_InternalNext    ) .or. &
             (bound_l == bound_InternalPeriodic))    )   then
    do i1=m1-nghost1,m1-1
      phic_grav(i1,:,:)=phic_grav(m1,:,:)
    end do ! i1
    do i1=m1-nghost1,m1
      phib_grav(i1,:,:)=phic_grav(m1,:,:)
    end do ! i1
  endif
  if (.not. ((bound_r == bound_periodic        ) .or. &
             (bound_r == bound_transmitting    ) .or. &
             (bound_r == bound_transmitting2   ) .or. &
             (bound_r == bound_InternalNext    ) .or. &
             (bound_r == bound_InternalPeriodic))    )   then
    do i1=n1+1,n1+nghost1
      phic_grav(i1,:,:)=phic_grav(n1,:,:)
    end do ! i1
    do i1=n1+1,n1+nghost1+1
      phib_grav(i1,:,:)=phic_grav(n1,:,:)
    end do ! i1
  endif
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D: boundaries: phi')
!#
# endif
!#
if ((bound_l == bound_transmitting ) .or. (bound_r == bound_transmitting ) .or. &
    (bound_l == bound_transmitting2) .or. (bound_r == bound_transmitting2) .or. &
    (bound_l == bound_transmitting3) .or. (bound_r == bound_transmitting3)) then
  ! --- Transmitting lower and upper boundary ---
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('MHD: Roe1D: boundaries: transmit')
!#
# endif
!#
  call rhd_bound_1DTransmit_mhd(model, action, dtime, grav_flag, phib_grav, Hp_l=Hp_l, Hp_r=Hp_r)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: Roe1D: boundaries: transmit')
!#
# endif
!#
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D: boundaries: ghost')
!#
# endif
!#
!
! --- Boundary conditions by filling of ghost cells: bulk data only ---
call rhd_bound_1DGhost(model, action, grid_mode=0, bulk_mode=1, Hp_l=Hp_l, Hp_r=Hp_r)
!if (action%dim == 3) then
!  write(6,'(A, 8(1PE14.6))') 'Roe1Dstep; minmax(rho), minmax(ei):', &
!    minval(model%rho(m1        :n1        , m2:n2, m3:n3)), &
!    maxval(model%rho(m1        :n1        , m2:n2, m3:n3)), &
!    minval(model%rho(m1-nghost1:n1+nghost1, m2:n2, m3:n3)), &
!    maxval(model%rho(m1-nghost1:n1+nghost1, m2:n2, m3:n3)), &
!    minval(model%ei( m1        :n1        , m2:n2, m3:n3)), &
!    maxval(model%ei( m1        :n1        , m2:n2, m3:n3)), &
!    minval(model%ei( m1-nghost1:n1+nghost1, m2:n2, m3:n3)), &
!    maxval(model%ei( m1-nghost1:n1+nghost1, m2:n2, m3:n3))
!endif

!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D: boundaries: ghost')
call timing_stop('MHD: Roe1D: boundaries')
!#
# endif
!#
! --- Save original values into local copies ---
xc1=model%xc1
xb1=model%xb1
rho=model%rho
ei =model%ei
v1 =model%v1
v2 =model%v2
v3 =model%v3
Bc1=model%Bc1
Bc2=model%Bc2
Bc3=model%Bc3
energy_switch=model%energy_switch
!
!
!
! === EOS, further quantities, check Courant condition =============================================
!
! --- Solve EOS ---
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D: eosinter')
!#
# endif
!#
if (model%nquc > 0) then
  T_flag=(Output_flag .or. any(prop%quc_advect == 2))
else
  T_flag=Output_flag
endif
if (T_flag) then
  call eosinter_all(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
else
  call eosinter_PdP(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei)
endif
Gamma_1=dPdrho*rho/P + dPdei/rho
P_mag=0.5*(Bc1**2+Bc2**2+Bc3**2)
P_tot=P+P_mag
!
if (action%dim == 1 .or. par%hdsplit =='unsplit') then
  !
  if (par%beta_inv < 0.0 .or. par%A%hdScheme == hdScheme_RoeMHD) then
    energy_switch = .true.
  else
    energy_switch=(P_mag/P < par%beta_inv)
    !where(P_mag / P < par%beta_inv)
    !  energy_switch = .true.
    !elsewhere
    !  energy_switch = .false.
    !endwhere
  endif
  !
  model%energy_switch=energy_switch
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop( 'MHD: Roe1D: eosinter')
!#
# endif
!#
!
! --- Check positivity of quantities ---
if (any(P <= 0.0)) then
  !if (action%dim == 1) then
  !  ierr0=100
  !else
    ierr0=4
  !endif
  error_loc=minloc(P)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+ m3
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') P(  error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Pressure non-positive (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif
if (any(Gamma_1 < 1.0)) then
  !if (action%dim == 1) then
  !  ierr0=101
  !else
    ierr0=5
  !endif
  error_loc=minloc(Gamma_1)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+ m3
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Gamma_1 smaller than 1 (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D: derived q')
!#
# endif
!#
dtime_limit=0.1*huge(dtime_limit)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1-nghost1,n1+nghost1
      ! --- Derived quantities: sound speed, square root of density, maximum allowed time step ---
      cs(      i1,i2,i3)=sqrt(P(i1,i2,i3)/rho(i1,i2,i3)*Gamma_1(i1,i2,i3))
      sqrt_rho(i1,i2,i3)=sqrt(rho(i1,i2,i3))
      va(      i1,i2,i3)=abs(Bc1(i1,i2,i3))/sqrt_rho(i1,i2,i3)
      B_sq_tan          =Bc2(i1,i2,i3)**2+Bc3(i1,i2,i3)**2
      !
      if (B_sq_tan > epsilon * P_tot(i1,i2,i3)) then
        caq               =cs(i1,i2,i3)**2+2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd                =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va(i1,i2,i3)/caq)**2))
        vf(      i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs(      i1,i2,i3)=cs(i1,i2,i3)*va(i1,i2,i3)*sqrt(2.0 / (caq + dd))
      else
        if (cs(i1,i2,i3) - va(i1,i2,i3) > epsilon * cs(i1,i2,i3)) then
          vf(i1,i2,i3) = cs(i1,i2,i3)
          vs(i1,i2,i3) = va(i1,i2,i3)
        else if (va(i1,i2,i3) - cs(i1,i2,i3) > epsilon * cs(i1,i2,i3)) then
          vf(i1,i2,i3) = va(i1,i2,i3)
          vs(i1,i2,i3) = cs(i1,i2,i3)
        else
          vf(i1,i2,i3) = cs(i1,i2,i3)
          vs(i1,i2,i3) = cs(i1,i2,i3)
        endif
      endif
      !
      if (i1 >= m1 - 1 .and. i1 <= n1 + 1) then
        dtime_limit       =min(dtime_limit, (xb1(i1+1)-xb1(i1)) / (abs(v1(i1,i2,i3))+vf(i1,i2,i3)))
      endif
      !
    end do ! i1
  end do ! i2
end do ! i3

!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D: derived q')
!#
# endif
!#
!
! --- Check Courant condition ---
!
if ((par%C_CourantMax > 0.0) .and. (dtime > par%C_CourantMax * dtime_limit)) then
  ! --- Time step too large: return without further computation ---
  ierr0=1
  outstr0='Time step too large for Courant condition'
  goto 999
endif
!
! --- Geometrical weights due to non-equidistant grid ---
wl(m1-nghost1+1:n1+nghost1  )=(xc1(m1-nghost1+1:n1+nghost1  )- &
                               xb1(m1-nghost1+1:n1+nghost1  )) / &
                              (xc1(m1-nghost1+1:n1+nghost1  )- &
                               xc1(m1-nghost1  :n1+nghost1-1))
!
! === Main step: compute boundary located (flux-like) averages and fluxes ==========================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D: fluxes')
!#
# endif
!#
!
call rhd_mhd_Roe1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
                       bound_l, bound_r, par%A%reconstruction_order, par%A%reconstruction_type, &
                       action%dim, par%C_SlopeRed, &
                       dtime, xb1, wl, &
                       rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, dpdei, dpdrho, &
                       cs, va, vf, vs, sqrt_rho, &
                       phic_grav, phib_grav, &
                       v1_til, v2_til, v3_til, &
                       flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                       flux_Bc1, flux_Bc2, flux_Bc3, &
                       flux_rhoeikm, fep)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D: fluxes')
!#
# endif
!#
if (Output_flag) then
  ! === Put additional output data into datx and datr ==============================================
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('MHD: Roe1D: (output)')
!#
# endif
!#
!  call rhd_mhd_Roe1Doutput(model, action%dim, AreaFraction, OutRadialC_flag, &
!                           P, T, s, Gamma_1, dPdei/rho + 1.0, dTdei, dPdei, &
!                           phic_grav, phib_grav, &
!                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
!                           flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
!                           flux_rhov1B, flux_rhov2B, flux_rhov3B, &
!                           datx, datr)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: Roe1D: (output)')
!#
# endif
!#
endif
!
!
! === Update with computed fluxes and source terms =================================================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: Roe1D: update')
!#
# endif
!#
! --- dtimeoverdx=dt/dx ---
dtimeoverdx=dtime/(xb1(m1+1:n1+1)-xb1(m1:n1))
!
! --- Check if density is positive everywhere ---
!#
# if (rhd_mhd_test_split == 1)
!#
if (action%dim == 3) then
!#
# endif
!#
rhonew_min=huge(rhonew_min)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      rhonew_min=min(rhonew_min, rho(i1,i2,i3) - &
                                 dtimeoverdx(i1) * (flux_rho(i1+1,i2,i3)-flux_rho(i1,i2,i3)))

    end do ! i1
  end do ! i2
end do ! i3
if (rhonew_min <= 0.0) then
  ierr0=6
  dtime_limit=dtime
  outstr0='Density becomes non-positive'
  goto 999
endif
!
!#
# if (rhd_mhd_test_split == 1)
!#
endif
!#
# endif
!#
!if (model%nquc > 0) then
!  ! --- Handle additional passive tracer fields ---
!  call rhd_hyd_Dust(model, T, phic_grav, &
!                    par%A%reconstruction_order, par%A%reconstruction_quctype, &
!                    dtime, flux_rho)
!endif
!
!
! --- Update all quantities ---
!
do i3=m3,n3
  do i2=m2,n2
!DIR$ IVDEP
    do i1=m1,n1
      ! --- Source terms due to gravity ---
      w0=(phic_grav(i1  ,i2,i3)-phib_grav(i1,i2,i3)) / &
         (xb1(      i1+1      )-xb1(      i1      ))
      w1=(phib_grav(i1+1,i2,i3)-phic_grav(i1,i2,i3)) / &
         (xb1(      i1+1      )-xb1(      i1      ))
      source_ei   = -P(i1,i2,i3) * (v1(i1+1,i2,i3) - v1(i1-1,i2,i3)) / (xc1(i1+1) - xc1(i1-1))
      source_rhov1=rho(i1,i2,i3) * (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) / &
                                   (xb1(      i1+1      )-xb1(      i1      ))
      !
      ! --- Update ---
      rhoout=rho(i1,i2,i3) - dtimeoverdx(i1) * (flux_rho(  i1+1,i2,i3)-flux_rho(  i1,i2,i3))
      v1out =(rho(i1,i2,i3) * v1(i1,i2,i3) - &
                             dtimeoverdx(i1) * ((flux_rhov1(i1+1,i2,i3)-flux_rhov1(i1,i2,i3))) + &
                             dtime           * source_rhov1 &
             )/rhoout
      v2out =(rho(i1,i2,i3) * v2(i1,i2,i3) - &
                             dtimeoverdx(i1) * ((flux_rhov2(i1+1,i2,i3)-flux_rhov2(i1,i2,i3))) &
             )/rhoout
      v3out =(rho(i1,i2,i3) * v3(i1,i2,i3) - &
                             dtimeoverdx(i1) * ((flux_rhov3(i1+1,i2,i3)-flux_rhov3(i1,i2,i3))) &
             )/rhoout
      Bc2out=Bc2(i1,i2,i3) - dtimeoverdx(i1) * (flux_Bc2(  i1+1,i2,i3)-flux_Bc2(  i1,i2,i3))
      Bc3out=Bc3(i1,i2,i3) - dtimeoverdx(i1) * (flux_Bc3(  i1+1,i2,i3)-flux_Bc3(  i1,i2,i3))
      !
      eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) + &
              0.5 * rho(i1,i2,i3) * (v1(i1,i2,i3)**2 + v2(i1,i2,i3)**2 + v3(i1,i2,i3)**2) - &
              dtimeoverdx(i1) * (flux_rhoeikm(i1+1,i2,i3)-flux_rhoeikm(i1,i2,i3)) + &
              0.5 * (Bc1(i1,i2,i3) + Bc1out) * &
                            (dtimeoverdx(i1) * (flux_Bc1(  i1+1,i2,i3)-flux_Bc1(  i1,i2,i3))) + &
              0.5 * (Bc2(i1,i2,i3) + Bc2out) * &
                            (dtimeoverdx(i1) * (flux_Bc2(  i1+1,i2,i3)-flux_Bc2(  i1,i2,i3))) + &
              0.5 * (Bc3(i1,i2,i3) + Bc3out) * &
                            (dtimeoverdx(i1) * (flux_Bc3(  i1+1,i2,i3)-flux_Bc3(  i1,i2,i3))) + &
              dtime           * source_eikm &
             )/rhoout - 0.5 * (v1out**2 + v2out**2 + v3out**2)
      !
      model%rho(i1,i2,i3)=rhoout
      model%v1( i1,i2,i3)=v1out
      model%v2( i1,i2,i3)=v2out
      model%v3( i1,i2,i3)=v3out
      model%Bc2(i1,i2,i3)=Bc2out
      model%Bc3(i1,i2,i3)=Bc3out
      model%ei( i1,i2,i3)=eiout
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Save fluxes for flux-interpolated transport step ---
do i3 = m3, n3
  do i2 = m2, n2
    do i1 = m1, n1 + 1
      model%flux_B21(i1, i2, i3) = flux_Bc2(i1, i2, i3)
      model%flux_B31(i1, i2, i3) = flux_Bc3(i1, i2, i3)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Check if internal energy is below minimum table value anywhere ---
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*dtime
  outstr0='Internal energy below minimum table value'
  goto 998
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D: update')
!#
# endif
!#
!
! --- Check internal energy ---
if (par%C_maxeiChange > 0.0) then
  dei_min=min(minval((model%ei(m1:n1,:,:)-ei(m1:n1,:,:))*rho(m1:n1,:,:)/P(m1:n1,:,:)),0.0)
  dtime_limit=min(dtime_limit, &
                  dtime*par%C_maxeiChange/(-dei_min+10.*tiny(dei_min))*0.999)
  if (dei_min < -par%C_maxeiChange) then
    ierr0=2
    outstr0='Drop in internal energy too large'
    goto 999
  endif
endif
!
! === Boundary conditions VI =======================================================================
if (bound_l == bound_transmitting2) then
  model%v1(m1,m2:n2,m3:n3)=v1(m1,m2:n2,m3:n3) + dtime*cs(m1,m2:n2,m3:n3) * &
                                         (       v1(m1+1,m2:n2,m3:n3)-       v1(m1,m2:n2,m3:n3))/ &
                                         (model%xc1(m1+1            )-model%xc1(m1            ))
endif
if (bound_r == bound_transmitting2) then
  model%v1(n1,m2:n2,m3:n3)=v1(n1,m2:n2,m3:n3) - dtime*cs(n1,m2:n2,m3:n3) * &
                                         (       v1(n1-1,m2:n2,m3:n3)-       v1(n1,m2:n2,m3:n3))/ &
                                         (model%xc1(n1-1            )-model%xc1(n1            ))
endif
!
!
999 dtime_limit=par%C_Courant*dtime_limit
!
998 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: Roe1D:')
!#
# endif
!#
!
end subroutine rhd_mhd_Roe1Dstep


!----------*************----------------------------------------------------------------------------
subroutine rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, av, dx, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_PPvec ('rhd_magneto-hydrodynamics_Piecewise_Parabolic_vector')
!
! PURPOSE:
!   Reconstruct input vector with piecewise-parabolic scheme and compute boundary values.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, av, dx, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!   dx:           ('delta_x') real vector with grid-cell sizes
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds.
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Woodward & Colella PPM reconstruction without extra steepening
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F. Lyon) Adapted from rhd_mhd_PP
!   2011-06-24 (W.S. Exeter) New parameters nghost2, nghost3, d2, d3 for use in unsplit
!                     Hancock scheme
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
real, dimension(m1-nghost1:n1+nghost1),&
                           intent(in)   :: dx
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1:n1+1,m2-d2:n2+d2,&
                          m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, dxm1, dxp0, dxp1, dxp2, &
                                           dam1, dap0, dap1, ap, am, aL, aR
real, dimension(m1-2:n1+2,m2-d2:n2+d2, &
                          m3-d2:n3+d3)  :: dapv
real, dimension(m1-2:n1+1,m2-d2:n2+d2, &
                          m3-d3:n3+d3)  :: apv
!---------------------------------------------------------------------------------------------------
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-2,n1+2
      !
      ! --- Grid size ---
      dxm1=dx(i1-1)
      dxp0=dx(i1  )
      dxp1=dx(i1+1)
      ! --- Cell values ---
      am1 =av(i1-1,i2,i3)
      ap0 =av(i1  ,i2,i3)
      ap1 =av(i1+1,i2,i3)
      !
      ! --- Average slope in zones (1.7) ---
      dap0=  dxp0/(dxm1+dxp0+dxp1) * &
           ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
             (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
      !
      ! --- Steepening, monotonizing (1.8) ---
      dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
           min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
      !
      dapv(i1,i2,i3)=dap0
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-2,n1+1
      !
      ! --- Grid size ---
      dxm1=dx(i1-1)
      dxp0=dx(i1  )
      dxp1=dx(i1+1)
      dxp2=dx(i1+2)
      ! --- Cell values ---
      ap0 =av(i1  ,i2,i3)
      ap1 =av(i1+1,i2,i3)
      ! --- Slopes ---
      dap0=dapv(i1  ,i2,i3)
      dap1=dapv(i1+1,i2,i3)
      !
      ! --- Preliminary boundary values (1.6) ---
      ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
         1.0/(dxm1+dxp0+dxp1+dxp2) * &
         ( 2.0*dxp1*dxp0/(dxp0+dxp1) * ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
             (ap1-ap0) - &
           dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1)*dap1 + &
           dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1)*dap0  )
      !
      apv(i1,i2,i3)=ap
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-1,n1+1
      !
      ! --- Cell values ---
      ap0 =av(i1  ,i2,i3)
      ! --- Preliminary boundary values ---
      am  =apv(i1-1,i2,i3)
      ap  =apv(i1  ,i2,i3)
      !
      ! --- Corrected boundary values (1.10) ---
      if ((ap-ap0)*(ap0-am) <= 0.0) then
        ! --- Local extremum in input values ---
        aL=ap0
        aR=ap0
      else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
        !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
        ! --- Inner extremum in parabola ---
        aL=3.0*ap0-2.0*ap
        aR=ap
      else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
        !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
        ! --- Inner extremum in parabola ---
        aL=am
        aR=3.0*ap0-2.0*am
      else
        ! --- Standard case ---
        aL=am
        aR=ap
      endif
      !
      aLv(i1,i2,i3)=aL
      aRv(i1,i2,i3)=aR
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_PPvec


!----------***************--------------------------------------------------------------------------
subroutine rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_PPeqvec ('rhd_magneto-hydrodynamics_Piecewise_Parabolic_equidistant_vector')
!
! PURPOSE:
!   Reconstruct input array with piecewise-parabolic scheme and compute boundary values.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds.
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Woodward & Colella PPM reconstruction without extra steepening
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F. Lyon) Adapted from rhd_mhd_PP
!   2010-06-09 (B.F. Lyon) Adapted from rhd_mhd_PPeq and rhd_hyd_PPeq
!   2011-06-24 (W.S. Exeter) New parameters nghost2, nghost3, d2, d3 for use in unsplit
!                     Hancock scheme
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   ), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, &
                                           dam1, dap0, dap1, ap, am, aL, aR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: dapv
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: apv
!---------------------------------------------------------------------------------------------------
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-2,n1+2
      !
      ! --- Cell values ---
      am1 =av(i1-1,i2,i3)
      ap0 =av(i1  ,i2,i3)
      ap1 =av(i1+1,i2,i3)
      !
      ! --- Average slope in zones (1.7) ---
      dap0=0.5*(ap1-am1)
      !
      ! --- Steepening, monotonizing (1.8) ---
      dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
           min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
      !
      dapv(i1,i2,i3)=dap0
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-2,n1+1
      !
      ! --- Cell values ---
      ap0 =av(i1  ,i2,i3)
      ap1 =av(i1+1,i2,i3)
      ! --- Slopes ---
      dap0=dapv(i1  ,i2,i3)
      dap1=dapv(i1+1,i2,i3)
      !
      ! --- Preliminary boundary values (1.6) ---
      ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * ( -dap1 + dap0 )
      !
      apv(i1,i2,i3)=ap
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-1,n1+1
      !
      ! --- Cell values ---
      ap0 =av(i1  ,i2,i3)
      ! --- Preliminary boundary values ---
      am  =apv(i1-1,i2,i3)
      ap  =apv(i1  ,i2,i3)
      !
      ! --- Corrected boundary values (1.10) ---
      if ((ap-ap0)*(ap0-am) <= 0.0) then
        ! --- Local extremum in input values ---
        aL=ap0
        aR=ap0
      else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
        !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
        ! --- Inner extremum in parabola ---
        aL=3.0*ap0-2.0*ap
        aR=ap
      else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
        !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
        ! --- Inner extremum in parabola ---
        aL=am
        aR=3.0*ap0-2.0*am
      else
        ! --- Standard case ---
        aL=am
        aR=ap
      endif
      !
      aLv(i1,i2,i3)=aL
      aRv(i1,i2,i3)=aR
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_PPeqvec


!----------***************--------------------------------------------------------------------------
subroutine rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           reconstruction_type, av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_FReqvec ('rhd_magneto-hydrodynamics_Frankenstein_equidistant_vector')
!
! PURPOSE:
!   Reconstruct input array with Frankenstein scheme and compute boundary values.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                        reconstruction_type, av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds.
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-03-02 (B.F. Lyon) Loop rearrangement
!   2013-03-10 (B.F. Lyon) Further loop rearrangement: put if-then-else construct into separate loop
!   2013-03-12 (B.F. Lyon) Put if-then-else construct into separate loop also for FRmono
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           d2, d3, reconstruction_type
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   ), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, dimension(m1-1 :n1+1)             :: ptilLv, ptilRv, ptilRnv, ptilLnv, fLv, fRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    !DIR$ SIMD
    do i1=m1-2,n1+2
      !
      ! --- Cell values ---
      am1 =av(i1-1,i2,i3)
      ap0 =av(i1  ,i2,i3)
      ap1 =av(i1+1,i2,i3)
      !
      atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
      atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
      atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
      atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
      !
      ! --- Get coefficients of polynomials ---
      pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
      !
      ! --- Get weights from quadratic coefficients ---
      we01=0.5/(abs(pc_20) + eps_tiny)
      we02=0.5/((pc_20)**2 + eps_tiny)
      !
      atil0v(i1,i2,i3)=atil0
      atil1v(i1,i2,i3)=atil1
      atil2v(i1,i2,i3)=atil2
      atil3v(i1,i2,i3)=atil3
      !
      we01v(i1,i2,i3)=we01
      we02v(i1,i2,i3)=we02
      !
    end do ! i1
!  end do ! i2
!end do ! i3
!!
!do i3=m3-d3,n3+d3
!  do i2=m2-d2,n2+d2
    !DIR$ SIMD
    do i1=m1-2,n1+1
      !
      ! --- Cell values ---
      ap0 =av(i1  ,i2,i3)
      ap1 =av(i1+1,i2,i3)
      !
      c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
      cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
      !
      ! --- Compute weighted averages of boundary values in a WENO style ---
      atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
      atilv(i1,i2,i3)=atil
      !
    end do ! i1
    !
    if (reconstruction_type == reconstruction_type_FRmono) then
      !DIR$ SIMD
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil=atilv(i1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
    endif
    !
    if (reconstruction_type == reconstruction_type_FRcont) then
      do i1=m1-1,n1+1
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        ptilL = atilL
        ptilR = atilR
        !
        aLv(i1,i2,i3)=ptilL
        aRv(i1,i2,i3)=ptilR
        !
      end do ! i1
    else if (reconstruction_type == reconstruction_type_FRmono) then
      !DIR$ SIMD
      do i1=m1-1,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        !
        atil0L=atil0v(i1+1,i2,i3)
        atil1L=atil1v(i1  ,i2,i3)
        atil1R=atil1v(i1+1,i2,i3)
        atil2L=atil2v(i1-1,i2,i3)
        atil2R=atil2v(i1  ,i2,i3)
        atil3R=atil3v(i1-1,i2,i3)
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        ! === Construct preliminary boundary values of reconstruction polynomial =======
        !
        ! --- Compute normalized weights of polynomials from quadratic coefficients ---
        wem  =0.5*we02v(i1-1,i2,i3)
        we0  =    we02v(i1  ,i2,i3)
        wep  =0.5*we02v(i1+1,i2,i3)
        wesum=wem+we0+wep
        wem  =wem/wesum
        we0  =we0/wesum
        wep  =wep/wesum
        !
        ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style ---
        ptilL=wem*atil2L + we0*atil1L + wep*atil0L
        ptilR=wem*atil3R + we0*atil2R + wep*atil1R
        !

        ! --- Testing: new loop split ---
        ptilLv(i1)=ptilL
        ptilRv(i1)=ptilR
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
        fLv(i1)=f
        ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        !
        f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
        fRv(i1)=f
        ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1-1,n1+1
        ptilL=ptilLv(i1)
        ptilR=ptilRv(i1)
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !

        ! === Monotonize boundary values ===============================================
        if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
            ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
            ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
          ! --- Local extremum in preliminary boundary values or               ---
          ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
          ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
          ! --- Reconstruction constant                                        ---
          ptilL=ap0
          ptilR=ap0
          !
        else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
          ! --- Left side has smaller slope ---
          !
          !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          f=fLv(i1)
          if (f > 1.0) then
            ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides ---
            ptilL=atilL
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            ptilR=ptilRnv(i1)
          endif
          !
          if (abs(ptilR-ap0) > abs(atilR-ap0)) then
            ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
            ptilR=atilR
          endif
          !
          !if ((reconstruction_type == reconstruction_type_FRmono) .and. &
          if  (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            ptilR=3.0*ap0-2.0*ptilL
          endif
          !
        else
          ! --- Right side has smaller slope ---
          !
          !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          f=fRv(i1)
          if (f > 1.0) then
            ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides ---
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            ptilL=ptilLnv(i1)
            ptilR=atilR
          endif
          !
          if (abs(ptilL-ap0) > abs(atilL-ap0)) then
            ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
            ptilL=atilL
          endif
          !
          !if ((reconstruction_type == reconstruction_type_FRmono) .and. &
          if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            ptilL=3.0*ap0-2.0*ptilR
          endif
          !
        endif
        !
        aLv(i1,i2,i3)=ptilL
        aRv(i1,i2,i3)=ptilR
        !
      end do ! i1
    else ! (FRweno)
      !DIR$ SIMD
      do i1=m1-1,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        !
        atil0L=atil0v(i1+1,i2,i3)
        atil1L=atil1v(i1  ,i2,i3)
        atil1R=atil1v(i1+1,i2,i3)
        atil2L=atil2v(i1-1,i2,i3)
        atil2R=atil2v(i1  ,i2,i3)
        atil3R=atil3v(i1-1,i2,i3)
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        ! === Construct preliminary boundary values of reconstruction polynomial =======
        !
        ! --- Compute normalized weights of polynomials from quadratic coefficients ---
        wem  =0.5*we02v(i1-1,i2,i3)
        we0  =    we02v(i1  ,i2,i3)
        wep  =0.5*we02v(i1+1,i2,i3)
        wesum=wem+we0+wep
        wem  =wem/wesum
        we0  =we0/wesum
        wep  =wep/wesum
        !
        ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style ---
        ptilL=wem*atil2L + we0*atil1L + wep*atil0L
        ptilR=wem*atil3R + we0*atil2R + wep*atil1R
        !

        ! --- Testing: new loop split ---
        ptilLv(i1)=ptilL
        ptilRv(i1)=ptilR
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
        fLv(i1)=f
        ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        !
        f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
        fRv(i1)=f
        ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1-1,n1+1
        ptilL=ptilLv(i1)
        ptilR=ptilRv(i1)
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !

        !
        ! === Monotonize boundary values ===============================================
        if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
            ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
            ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
          ! --- Local extremum in preliminary boundary values or               ---
          ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
          ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
          ! --- Reconstruction constant                                        ---
          ptilL=ap0
          ptilR=ap0
          !
        else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
          ! --- Left side has smaller slope ---
          !
          !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          f=fLv(i1)
          if (f > 1.0) then
            ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides ---
            ptilL=atilL
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            ptilR=ptilRnv(i1)
          endif
          !
          if (abs(ptilR-ap0) > abs(atilR-ap0)) then
            ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
            ptilR=atilR
          endif
          !
          !if ((reconstruction_type == reconstruction_type_FRmono) .and. &
          !    ((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
          !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
          !  ptilR=3.0*ap0-2.0*ptilL
          !endif
          !
        else
          ! --- Right side has smaller slope ---
          !
          !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          f=fRv(i1)
          if (f > 1.0) then
            ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides --
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            ptilL=ptilLnv(i1)
            ptilR=atilR
          endif
          !
          if (abs(ptilL-ap0) > abs(atilL-ap0)) then
            ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
            ptilL=atilL
          endif
          !
          !if ((reconstruction_type == reconstruction_type_FRmono) .and. &
          !    ((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
          !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
          !  ptilL=3.0*ap0-2.0*ptilR
          !endif
          !
        endif
        !
        aLv(i1,i2,i3)=ptilL
        aRv(i1,i2,i3)=ptilR
        !
      end do ! i1
    endif
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_FReqvec


!----------*************----------------------------------------------------------------------------
subroutine rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                         reconstruction_type, av, dx, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_PLvec ('rhd_magneto-hydrodynamics_Piecewise_Linear_equidistant_vector')
!
! PURPOSE:
!   Reconstruct input vector with piecewise-linear scheme and compute boundary values.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                      reconstruction_type, av, dx, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!   dx:           ('delta_x') real vector with grid-cell sizes
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds.
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!  Linear Reconstruction (MinMod, vanLeer, Superbee)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F. Lyon) Adapted from rhd_mhd_PP
!   2011-01-04 (W.S. Exeter) Adapted from rhd_mhd_PPvec
!   2011-06-24 (W.S. Exeter) New parameters nghost2, nghost3, d2, d3 for use in unsplit
!                     Hancock scheme
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           d2, d3, reconstruction_type
real, dimension(m1-nghost1:n1+nghost1),&
                           intent(in)   :: dx
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1:n1+1,m2-d2:n2+d2,&
                          m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: delta, d1d2
real, dimension(m1-2:n1+1,m2-d2:n2+d2, &
                          m3-d3:n3+d3)  :: d, dd
!---------------------------------------------------------------------------------------------------
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-2,n1+1
      !
      d(i1,i2,i3)=2.0*(av(i1+1,i2,i3)-av(i1  ,i2,i3))/(dx(i1+1)+dx(i1  ))
      dd(i1,i2,i3)=(av(i1+1,i2,i3)-av(i1  ,i2,i3))/dx(i1  )
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-1,n1+1
      !
      select case(reconstruction_type)
        !
        case(reconstruction_type_minmod)
          ! --- Minmod limiter ---
          delta=dx(i1)* &
                    (min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                     max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
        case(reconstruction_type_vanLeer)
          ! --- vanLeer limiter
          d1d2=2.0*d(i1-1,i2,i3)*d(i1  ,i2,i3)
          if (d1d2 > 0.0) then
            delta=dx(i1)*d1d2/(d(i1-1,i2,i3)+d(i1  ,i2,i3))
          else
            delta=0.0
          endif
          !
        case(reconstruction_type_superbee)
          ! --- Superbee limiter ---
          delta=dx(i1)*(sign(1.0,d(i1-1,i2,i3))+sign(1.0,d(i1  ,i2,i3)))* &
                    (min(abs(dd(i1-1,i2,i3)),abs(dd(i1  ,i2,i3)), &
                     0.5*max(abs(d(i1-1,i2,i3)),abs(d(i1  ,i2,i3)))))
          !
      end select
      !
      aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
      aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_PLvec


!----------***************--------------------------------------------------------------------------
subroutine rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           reconstruction_type, av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_PLeqvec ('rhd_magneto-hydrodynamics_Piecewise_Linear_vector')
!
! PURPOSE:
!   Reconstruct input vector with piecewise-linear scheme and compute boundary values.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                      reconstruction_type, av, dx, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds.
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!  Linear Reconstruction (MinMod, vanLeer, Superbee)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F. Lyon) Adapted from rhd_mhd_PP
!   2011-01-04 (W.S. Exeter) Adapted from rhd_mhd_PPvec
!   2011-06-24 (W.S. Exeter) New parameters nghost2, nghost3, d2, d3 for use in unsplit
!                     Hancock scheme
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           d2, d3, reconstruction_type
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1:n1+1,m2-d2:n2+d2,&
                          m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: delta, d1d2
real, dimension(m1-2:n1+1,m2-d2:n2+d2, &
                          m3-d3:n3+d3)  :: d
!---------------------------------------------------------------------------------------------------
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-2,n1+1
      !
      d(i1,i2,i3)=av(i1+1,i2,i3)-av(i1  ,i2,i3)
      !
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-d3,n3+d3
  do i2=m2-d2,n2+d2
    do i1=m1-1,n1+1
      !
      select case(reconstruction_type)
        !
        case(reconstruction_type_minmod)
          ! --- Minmod limiter ---
          delta=    (min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                     max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
        case(reconstruction_type_vanLeer)
          ! --- vanLeer limiter
          d1d2=2.0*d(i1-1,i2,i3)*d(i1  ,i2,i3)
          if (d1d2 > 0.0) then
            delta=d1d2/(d(i1-1,i2,i3)+d(i1  ,i2,i3))
          else
            delta=0.0
          endif
          !
        case(reconstruction_type_superbee)
          ! --- Superbee limiter ---
          delta=(sign(1.0,d(i1-1,i2,i3))+sign(1.0,d(i1  ,i2,i3)))* &
                    (min(abs(d(i1-1,i2,i3)),abs(d(i1  ,i2,i3)), &
                     0.5*max(abs(d(i1-1,i2,i3)),abs(d(i1  ,i2,i3)))))
          !
      end select
      !
      aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
      aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_PLeqvec


!----------*******************----------------------------------------------------------------------
subroutine rhd_mhd_Reconstruct_old(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               reconstruction_order, reconstruction_type, dim, &
                               dtime, xb1, f, &
                               rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                               Gamma_1, &
                               rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                               rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Reconstruct ('rhd_magneto-hydrodynamics_Reconstruction')
!
! PURPOSE:
!   Perform data reconstruction within cells.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Reconstruct(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            reconstruction_order, reconstruction_type, dim, &
!                            dtime, xb1, f, &
!                            rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
!                            Gamma_1, &
!                            rho_l, v1_l, v2_l, v3_l, ei_l, Bc1_l, Bc2_l, Bc3_l, P_l, &
!                            rhov1_l, rhov2_l, rhov3_l, rhoeikm_l, rhoek_l, rhoem_l, &
!                            rho_r, v1_r, v2_r, v3_r, ei_r, Bc1_r, Bc2_r, Bc3_r, P_r, &
!                            rhov1_r, rhov2_r, rhov3_r, rhoeikm_r, rhoek_r, rhoem_r)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!
! OUTPUT:
!   rho_l:        ('rho_left') real 3D array, density at the left edge of the cell
!   v1_l:         ('velocity_1_left') real 3D array, velocity 1 at the left edge of the cell
!   v2_l:         ('velocity_2_left') real 3D array, velocity 2 at the left edge of the cell
!   v3_l:         ('velocity_3_left') real 3D array, velocity 3 at the left edge of the cell
!   ei_l:         ('energy_internal_left') real 3D array, internal energy at left edge of the cell
!   Bc1_l:        ('Bc1_left') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l:        ('Bc2_left') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l:        ('Bc3_left') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l:          ('Pressure_left') real 3D array, pressure at the left edge of the cell
!   rhov1_l:      ('rho_velocity_1_left') real 3D array, momentum 1 at the left edge of the cell
!   rhov2_l:      ('rho_velocity_2_left') real 3D array, momentum 2 at the left edge of the cell
!   rhov3_l:      ('rho_velocity_3_left') real 3D array, momentum 3 at the left edge of the cell
!   rhoeikm_l:    ('rhoeikm_left') real 3D array, total energy at the left edge of the cell
!   rho_r:        ('rho_right') real 3D array, density at the right edge of the cell
!   v1_r:         ('velocity_1_right') real 3D array, velocity 1 at the right edge of the cell
!   v2_r:         ('velocity_2_right') real 3D array, velocity 2 at the right edge of the cell
!   v3_r:         ('velocity_3_right') real 3D array, velocity 3 at the right edge of the cell
!   ei_r:         ('energy_internal_right') real 3D array, internal energy at right edge of the cell
!   Bc1_r:        ('Bc1_right') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r:        ('Bc2_right') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r:        ('Bc3_right') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r:          ('Pressure_right') real 3D array, pressure at the right edge of the cell
!   rhov1_r:      ('rho_velocity_1_right') real 3D array, momentum 1 at the right edge of the cell
!   rhov2_r:      ('rho_velocity_2_right') real 3D array, momentum 2 at the right edge of the cell
!   rhov3_r:      ('rho_velocity_3_right') real 3D array, momentum 3 at the right edge of the cell
!   rhoeikm_r:    ('rhoeikm_right') real 3D array, total energy at the right edge of the cell
!
! VARIABLES:
!
! ROUTINES:
!   rhd_mhd_FReqvec:       Reconstruct with Frankenstein scheme and compute boundary values
!   rhd_mhd_PLeqvec:       Piecewise linear reconstruction using different slope limiters
!   rhd_mhd_PLvec:         Piecewise linear reconstruction using different slope limiters
!   rhd_mhd_PPeqvec:       Reconstruct with piecewise-parabolic scheme and compute boundary values
!   rhd_mhd_PPvec:         Reconstruct with piecewise-parabolic scheme and compute boundary values
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   rhoei_l and rhoei_r have the minimum internal energy of the EOS table (eiTable_min) subtracted.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2005-11-11 (W.S.) Put into extra subroutine
!   2008-02-29 (W.S. East Lansing) Gravity terms removed to avoid massloss in lower open boundary
!   2010-06-05 (W.S.) In case of too small boundary values: set ei_l,r to ei instead of rho*ei,
!                     Hancock terms can be switched off
!   2010-06-08 (B.F. Lyon) PP reconstruction
!   2010-06-09 (B.F.) Add two boundary cells to dtimeoverdx,
!              use en_l,r as temporary arrays for rhoei
!   2010-06-11 (B.F.) Slope reduction in PP case not for rho, P, rhoei
!   2010-06-16 (B.F.) Slope reduction in PP case small for rho, P, rhoei
!   2011-01-04 (W.S. Exeter) subroutine renamed to rhd_mhd_Reconstruct, Hancock predictor step
!                     separated from data reconstruction and put into separate subroutine
!   2011-02-02 (W.S.) Reconstruction of rhoei modified
!   2011-06-24 (W.S.) New parameters nghost2, nghost3, d2, d3 for use in unsplit Hancock scheme
!   2013-03-10 (B.F.) Set positive lower limit for positive-definite quantities
!   2018-05-08 (B.F.) Move previously doubled code for near-boundary slope reduction to end;
!                     Bugfix: Subtract eiTable_min from ei for constant reconstruction
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           d2, d3, reconstruction_order, reconstruction_type, dim
real,                      intent(in)   :: dtime
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, Gamma_1, f
real, dimension(m1-1 :n1+1 ,&
                m2-d2:n2+d2,&
                m3-d3:n3+d3), &
                           intent(out)  :: rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, &
                                           P_l, rhoei_l, &
                                           rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, &
                                           P_r, rhoei_r
!
! --- Local variables ---
real, dimension(m1-nghost1:n1+nghost1)  :: dx
integer                                 :: i1, i2, i3, i0, nb1, nt1, nb2, nt2, nb3, nt3, &
                                           m1_0, n1_0, m2_0, n2_0, m3_0, n3_0
real                                    :: eiTable_min, delta, sigma, &
                                           dxmin, dxmax, &
                                           rho_min, P_min, rhoei_min
real, parameter                         :: epsilon = 0.0000001, C_dxb1_maxvar=0.01
!---------------------------------------------------------------------------------------------------
!
call tabinter_getinfo(x2min=eiTable_min)
!
! --- Construct grid spacing vector ---
dx=xb1(m1-nghost1+1:n1+nghost1+1)-xb1(m1-nghost1:n1+nghost1)
dxmin=minval(dx)
dxmax=maxval(dx)
!
! \label{f90:reconstruction__useMHD1}
select case(reconstruction_order)
  !
  case(2)
  ! === 2nd-order (e.g., piecewise-parabolic) case ===
  !
  ! --- Reconstruction for all quantities needed ---
  if ((dxmax-dxmin) < (C_dxb1_maxvar*(dxmin+dxmax))) then
    ! --- Equidistant grid ---
    if (reconstruction_type == reconstruction_type_FRmono .or. &
        reconstruction_type == reconstruction_type_FRweno .or. &
        reconstruction_type == reconstruction_type_FRcont) then
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, rho   , rho_l, rho_r)
      if (reconstruction_type == reconstruction_type_FRweno .or. &
          reconstruction_type == reconstruction_type_FRcont) then
        ! --- Set positive lower limit for positive-definite quantities ---
        do i3=m3-d3,n3+d3
          do i2=m2-d2,n2+d2
            do i1=m1-1,n1+1
              rho_min        =0.5*min(rho(i1,i2,i3), rho(i1+1,i2,i3))
              rho_l(i1,i2,i3)=max(rho_min, rho_l(i1,i2,i3))
              rho_r(i1,i2,i3)=max(rho_min, rho_r(i1,i2,i3))
            end do ! i1
          end do ! i2
        end do ! i3
      endif
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, v1    , v1_l , v1_r )
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, v2    , v2_l , v2_r )
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, v3    , v3_l , v3_r )
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, Bc1   , Bc1_l, Bc1_r)
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, Bc2   , Bc2_l, Bc2_r)
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, Bc3   , Bc3_l, Bc3_r)
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                             reconstruction_type, P     , P_l  , P_r  )
      if (reconstruction_type == reconstruction_type_FRweno .or. &
          reconstruction_type == reconstruction_type_FRcont) then
        ! --- Set positive lower limit for positive-definite quantities ---
        do i3=m3-d3,n3+d3
          do i2=m2-d2,n2+d2
            do i1=m1-1,n1+1
              P_min        =0.5*min(P(i1,i2,i3), P(i1+1,i2,i3))
              P_l(i1,i2,i3)=max(P_min, P_l(i1,i2,i3))
              P_r(i1,i2,i3)=max(P_min, P_r(i1,i2,i3))
            end do ! i1
          end do ! i2
        end do ! i3
      endif
      call rhd_mhd_FReqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           reconstruction_type, rho*(ei-eiTable_min), rhoei_l , rhoei_r )
      if (reconstruction_type == reconstruction_type_FRweno .or. &
          reconstruction_type == reconstruction_type_FRcont) then
        ! --- Set positive lower limit for positive-definite quantities ---
        do i3=m3-d3,n3+d3
          do i2=m2-d2,n2+d2
            do i1=m1-1,n1+1
              rhoei_min        =0.5*min(rho(i1  ,i2,i3)*(ei(i1  ,i2,i3)-eiTable_min), &
                                        rho(i1+1,i2,i3)*(ei(i1+1,i2,i3)-eiTable_min)   )
              rhoei_l(i1,i2,i3)=max(rhoei_min, rhoei_l(i1,i2,i3))
              rhoei_r(i1,i2,i3)=max(rhoei_min, rhoei_r(i1,i2,i3))
            end do ! i1
          end do ! i2
        end do ! i3
      endif
    else
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  rho   , rho_l, rho_r)
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  v1    , v1_l , v1_r )
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  v2    , v2_l , v2_r )
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  v3    , v3_l , v3_r )
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  Bc1   , Bc1_l, Bc1_r)
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  Bc2   , Bc2_l, Bc2_r)
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  Bc3   , Bc3_l, Bc3_r)
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                                  P     , P_l  , P_r  )
      call rhd_mhd_PPeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                rho*(ei-eiTable_min), rhoei_l , rhoei_r )
    endif
  else
    ! --- Non-equidistant grid ---
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              rho   , dx, rho_l, rho_r)
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              v1    , dx, v1_l , v1_r )
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              v2    , dx, v2_l , v2_r )
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              v3    , dx, v3_l , v3_r )
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              Bc1   , dx, Bc1_l, Bc1_r)
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              Bc2   , dx, Bc2_l, Bc2_r)
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              Bc3   , dx, Bc3_l, Bc3_r)
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                                              P     , dx, P_l  , P_r  )
    call rhd_mhd_PPvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            rho*(ei-eiTable_min), dx, rhoei_l , rhoei_r )
  endif
  !
  case(1)
  ! === Linear case ===
  !
  ! --- Reconstruction for all quantities needed ---
  if ((dxmax-dxmin) < (C_dxb1_maxvar*(dxmin+dxmax))) then
    ! --- Equidistant grid ---
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, rho   , rho_l, rho_r)
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, v1    , v1_l , v1_r )
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, v2    , v2_l , v2_r )
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, v3    , v3_l , v3_r )
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, Bc1   , Bc1_l, Bc1_r)
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, Bc2   , Bc2_l, Bc2_r)
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, Bc3   , Bc3_l, Bc3_r)
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                              reconstruction_type, P     , P_l  , P_r  )
    call rhd_mhd_PLeqvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                         reconstruction_type, rho*(ei-eiTable_min), rhoei_l , rhoei_r )
  else
    ! --- Non-equidistant grid ---
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, rho   , dx, rho_l, rho_r)
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, v1    , dx, v1_l , v1_r )
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, v2    , dx, v2_l , v2_r )
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, v3    , dx, v3_l , v3_r )
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, Bc1   , dx, Bc1_l, Bc1_r)
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, Bc2   , dx, Bc2_l, Bc2_r)
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, Bc3   , dx, Bc3_l, Bc3_r)
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                            reconstruction_type, P     , dx, P_l  , P_r  )
    call rhd_mhd_PLvec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                       reconstruction_type, rho*(ei-eiTable_min), dx, rhoei_l , rhoei_r )
  endif
  !
  case(0)
  ! === Constant reconstruction ===
  !
  rho_l(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=rho(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  v1_l( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=v1( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  v2_l( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=v2( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  v3_l( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=v3( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  Bc1_l(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=Bc1(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  Bc2_l(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=Bc2(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  Bc3_l(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=Bc3(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  P_l(  m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=P(  m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  rhoei_l(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=rho(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)* &
                                             (ei(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)-eiTable_min)
  ! --- Bugfix 2018-05-08 B.F.: Subtract eiTable_min from ei ---
  !
  rho_r(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=rho(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  v1_r( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=v1( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  v2_r( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=v2( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  v3_r( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=v3( m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  Bc1_r(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=Bc1(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  Bc2_r(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=Bc2(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  Bc3_r(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=Bc3(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  P_r(  m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=P(  m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)
  rhoei_r(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)=rho(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)* &
                                             (ei(m1-1:n1+1,m2-d2:n2+d2,m3-d3:n3+d3)-eiTable_min)
  ! --- Bugfix 2018-05-08 B.F.: Subtract eiTable_min from ei ---
  !
end select ! (reconstruction_order)
!
if (reconstruction_order >= 1) then
  if ((par%N_magDiffSide > 0) .or. (par%N_magDiffTop > 0) .or. (par%N_magDiffBottom > 0)) then
    ! === Perform near-boundary slope reduction ===
    !
    ! \label{f90:n_magdiff__useMHD4}
    ! --- Set variables for diffusive boundary layers ---
    if (dim == 1) then
      ! --- Set dimensions for x1 sweep ---
      !
      m1_0 = prop%m1
      n1_0 = prop%n1
      m2_0 = prop%m2
      n2_0 = prop%n2
      m3_0 = prop%m3
      n3_0 = prop%n3
      nb1 = par%N_magDiffSide
      nt1 = par%N_magDiffSide
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffBottom
      nt3 = par%N_magDiffTop
      !
    else if (dim == 2) then
      ! --- Set dimensions for x2 sweep ---
      !
      m1_0 = prop%m2
      n1_0 = prop%n2
      m2_0 = prop%m1
      n2_0 = prop%n1
      m3_0 = prop%m3
      n3_0 = prop%n3
      nb1 = par%N_magDiffSide
      nt1 = par%N_magDiffSide
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffBottom
      nt3 = par%N_magDiffTop
      !
   else ! (dim == 3)
      ! --- Set dimensions for x3 sweep ---
      !
      m1_0 = prop%m3
      n1_0 = prop%n3
      m2_0 = prop%m2
      n2_0 = prop%n2
      m3_0 = prop%m1
      n3_0 = prop%n1
      nb1 = par%N_magDiffBottom
      nt1 = par%N_magDiffTop
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffSide
      nt3 = par%N_magDiffSide
      !
    endif ! (dim)
    !
    ! --- Reduce slope near the boundaries ---
    do i3=m3-d3,n3+d3
      do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          if (prop%n2 > prop%m2) then
            delta=min((i1   - m1_0 + 1 + epsilon) / (nb1 + epsilon), &
                      (n1_0 - i1   + 1 + epsilon) / (nt1 + epsilon), &
                      (i2   - m2_0     + epsilon) / (nb2 + epsilon), &
                      (n2_0 - i2       + epsilon) / (nt2 + epsilon), &
                      (i3   - m3_0     + epsilon) / (nb3 + epsilon), &
                      (n3_0 - i3       + epsilon) / (nt3 + epsilon))
          else
            delta=min((i1   - m1_0 + 1 + epsilon) / (nb1 + epsilon), &
                      (n1_0 - i1   + 1 + epsilon) / (nt1 + epsilon), &
                      (i3   - m3_0     + epsilon) / (nb3 + epsilon), &
                      (n3_0 - i3       + epsilon) / (nt3 + epsilon))
          endif
          !
          if (delta < 1.0) then
            sigma = 1.0 / (1.0 + exp(10.0 * (0.5 - delta)))
            !
            rho_l(i1,i2,i3)=sigma*rho_l(i1,i2,i3) + (1.0-sigma)* rho(i1,i2,i3)
            rho_r(i1,i2,i3)=sigma*rho_r(i1,i2,i3) + (1.0-sigma)* rho(i1,i2,i3)
            v1_l( i1,i2,i3)=sigma*v1_l( i1,i2,i3) + (1.0-sigma)* v1( i1,i2,i3)
            v1_r( i1,i2,i3)=sigma*v1_r( i1,i2,i3) + (1.0-sigma)* v1( i1,i2,i3)
            v2_l( i1,i2,i3)=sigma*v2_l( i1,i2,i3) + (1.0-sigma)* v2( i1,i2,i3)
            v2_r( i1,i2,i3)=sigma*v2_r( i1,i2,i3) + (1.0-sigma)* v2( i1,i2,i3)
            v3_l( i1,i2,i3)=sigma*v3_l( i1,i2,i3) + (1.0-sigma)* v3( i1,i2,i3)
            v3_r( i1,i2,i3)=sigma*v3_r( i1,i2,i3) + (1.0-sigma)* v3( i1,i2,i3)
            Bc1_l(i1,i2,i3)=sigma*Bc1_l(i1,i2,i3) + (1.0-sigma)* Bc1(i1,i2,i3)
            Bc1_r(i1,i2,i3)=sigma*Bc1_r(i1,i2,i3) + (1.0-sigma)* Bc1(i1,i2,i3)
            Bc2_l(i1,i2,i3)=sigma*Bc2_l(i1,i2,i3) + (1.0-sigma)* Bc2(i1,i2,i3)
            Bc2_r(i1,i2,i3)=sigma*Bc2_r(i1,i2,i3) + (1.0-sigma)* Bc2(i1,i2,i3)
            Bc3_l(i1,i2,i3)=sigma*Bc3_l(i1,i2,i3) + (1.0-sigma)* Bc3(i1,i2,i3)
            Bc3_r(i1,i2,i3)=sigma*Bc3_r(i1,i2,i3) + (1.0-sigma)* Bc3(i1,i2,i3)
            P_l(  i1,i2,i3)=sigma*P_l(  i1,i2,i3) + (1.0-sigma)* P(  i1,i2,i3)
            P_r(  i1,i2,i3)=sigma*P_r(  i1,i2,i3) + (1.0-sigma)* P(  i1,i2,i3)
            rhoei_l( i1,i2,i3)=sigma*rhoei_l( i1,i2,i3) + &
                               (1.0-sigma)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
            rhoei_r( i1,i2,i3)=sigma*rhoei_r( i1,i2,i3) + &
                               (1.0-sigma)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
            !
          endif
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !
  endif ! ((par%N_magDiffSide > 0) .or. (par%N_magDiffTop > 0) .or. (par%N_magDiffBottom > 0))
endif ! (reconstruction_order)
!
end subroutine rhd_mhd_Reconstruct_old


!----------*******************----------------------------------------------------------------------
subroutine rhd_mhd_Reconstruct(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               reconstruction_order, reconstruction_type, dim, &
                               dtime, xb1, f, &
                               rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, Gamma_1, &
                               rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                               rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Reconstruct ('rhd_magneto-hydrodynamics_Reconstruction')
!
! PURPOSE:
!   Perform data reconstruction within cells.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Reconstruct(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            reconstruction_order, reconstruction_type, dim, &
!                            dtime, xb1, f, &
!                            rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, Gamma_1, &
!                            rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
!                            rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!
! OUTPUT:
!   rho_l:        ('rho_left') real 3D array, density at the left edge of the cell
!   v1_l:         ('velocity_1_left') real 3D array, velocity 1 at the left edge of the cell
!   v2_l:         ('velocity_2_left') real 3D array, velocity 2 at the left edge of the cell
!   v3_l:         ('velocity_3_left') real 3D array, velocity 3 at the left edge of the cell
!   ei_l:         ('energy_internal_left') real 3D array, internal energy at left edge of the cell
!   Bc1_l:        ('Bc1_left') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l:        ('Bc2_left') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l:        ('Bc3_left') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l:          ('Pressure_left') real 3D array, pressure at the left edge of the cell
!   rhoei_l:      ('rhoei_left') real 3D array, internal energy at the left edge of the cell,
!                                with rho*eiTable_min subtracted
!   rho_r:        ('rho_right') real 3D array, density at the right edge of the cell
!   v1_r:         ('velocity_1_right') real 3D array, velocity 1 at the right edge of the cell
!   v2_r:         ('velocity_2_right') real 3D array, velocity 2 at the right edge of the cell
!   v3_r:         ('velocity_3_right') real 3D array, velocity 3 at the right edge of the cell
!   ei_r:         ('energy_internal_right') real 3D array, internal energy at right edge of the cell
!   Bc1_r:        ('Bc1_right') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r:        ('Bc2_right') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r:        ('Bc3_right') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r:          ('Pressure_right') real 3D array, pressure at the right edge of the cell
!   rhoei_r:      ('rhoei_right') real 3D array, internal energy at the right edge of the cell,
!                                 with rho*eiTable_min subtracted
!
! VARIABLES:
!
! ROUTINES:
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_rec_module:        Reconstruction routines
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   rhoei_l and rhoei_r have the minimum internal energy of the EOS table (eiTable_min) subtracted.
!
! PROCEDURE:
!   Reconstruct all quantities by calling general vector reconstruction routines.
!   Set positive lower limit for positive-definite quantities if the reconstruction
!     is (possibly) not monotonic.
!   Perform optional near-boundary slope reduction.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2005-11-11 (W.S.) Put into extra subroutine
!   2008-02-29 (W.S. East Lansing) Gravity terms removed to avoid massloss in lower open boundary
!   2010-06-05 (W.S.) In case of too small boundary values: set ei_l,r to ei instead of rho*ei,
!                     Hancock terms can be switched off
!   2010-06-08 (B.F. Lyon) PP reconstruction
!   2010-06-09 (B.F.) Add two boundary cells to dtimeoverdx,
!                     use en_l,r as temporary arrays for rhoei
!   2010-06-11 (B.F.) Slope reduction in PP case not for rho, P, rhoei
!   2010-06-16 (B.F.) Slope reduction in PP case small for rho, P, rhoei
!   2011-01-04 (W.S. Exeter) subroutine renamed to rhd_mhd_Reconstruct, Hancock predictor step
!                     separated from data reconstruction and put into separate subroutine
!   2011-02-02 (W.S.) Reconstruction of rhoei modified
!   2011-06-24 (W.S.) New parameters nghost2, nghost3, d2, d3 for use in unsplit Hancock scheme
!   2013-03-10 (B.F.) Set positive lower limit for positive-definite quantities
!   2018-05-08 (B.F.) Move previously doubled code for near-boundary slope reduction to end;
!                     Bugfix: Subtract eiTable_min from ei for constant reconstruction
!   2018-05-10 (B.F.) Reorder blocks and call routines from reconstruction module
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_rec_module
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           d2, d3, reconstruction_order, reconstruction_type, dim
real,                      intent(in)   :: dtime
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, Gamma_1, f
real, dimension(m1-1 :n1+1 ,&
                m2-d2:n2+d2,&
                m3-d3:n3+d3), &
                           intent(out)  :: rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, &
                                           P_l, rhoei_l, &
                                           rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, &
                                           P_r, rhoei_r
!
! --- Local variables ---
integer                                 :: i1, i2, i3, &
                                           nb1, nt1, nb2, nt2, nb3, nt3, &
                                           m1_0,n1_0, m2_0,n2_0, m3_0,n3_0
real                                    :: eiTable_min, delta, sigma, &
                                           dxmin, dxmax, &
                                           rho_min, P_min, rhoei_min
logical                                 :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 )           :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1)  :: dx
real, parameter                         :: epsilon=0.0000001, C_dxb1_maxvar=0.01
!---------------------------------------------------------------------------------------------------
!
! --- Get minimum internal energy from EOS table ---
call tabinter_getinfo(x2min=eiTable_min)
!
! --- Deactivate slope reduction and steepening (for now) ---
SlopeRed_flag=.false.
SlopeCon_flag=.false.
!
! --- Construct grid spacing vector ---
dx   =xb1(m1-nghost1+1:n1+nghost1+1) - &
      xb1(m1-nghost1  :n1+nghost1  )
dxmin=minval(dx)
dxmax=maxval(dx)
!
! === Perform reconstruction of all quantities ===
if ((dxmax-dxmin) < (C_dxb1_maxvar*(dxmin+dxmax))) then
  ! --- Handle equidistant grid ---
  !
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     rho, rho_l, rho_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     v1 , v1_l , v1_r )
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     v2 , v2_l , v2_r )
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     v3 , v3_l , v3_r )
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     Bc1, Bc1_l, Bc1_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     Bc2, Bc2_l, Bc2_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     Bc3, Bc3_l, Bc3_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     P  , P_l  , P_r  )
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     rho*(ei-eiTable_min), rhoei_l, rhoei_r)
  !
else ! ((dxmax-dxmin) >= (C_dxb1_maxvar*(dxmin+dxmax)))
  ! --- Handle non-equidistant grid ---
  !
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   rho, dx, rho_l, rho_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   v1 , dx, v1_l , v1_r )
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   v2 , dx, v2_l , v2_r )
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   v3 , dx, v3_l , v3_r )
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   Bc1, dx, Bc1_l, Bc1_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   Bc2, dx, Bc2_l, Bc2_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   Bc3, dx, Bc3_l, Bc3_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   P  , dx, P_l  , P_r  )
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   rho*(ei-eiTable_min), dx, rhoei_l, rhoei_r)
  !
endif ! ((dxmax-dxmin) < (C_dxb1_maxvar*(dxmin+dxmax)))
!
!\label{f90:reconstruction__useMHD1__new}
! === Check if reconstruction is not monotonic and limit positive-definite quantities if not ===
if ((reconstruction_type == reconstruction_type_FRweno ) .or. &
    (reconstruction_type == reconstruction_type_FRweno2) .or. &
    (reconstruction_type == reconstruction_type_FRcont ) .or. &
    (reconstruction_type == reconstruction_type_VAweno ) .or. &
    (reconstruction_type == reconstruction_type_HBweno )       ) then
  ! --- Set positive lower limit for positive-definite quantities ---
  do i3=m3-d3,n3+d3
    do i2=m2-d2,n2+d2
      do i1=m1-1,n1+1
        ! --- Limit density ---
        rho_min          =0.5*min(rho(i1,i2,i3), rho(i1+1,i2,i3))
        rho_l(i1,i2,i3)  =max(rho_min, rho_l(i1,i2,i3))
        rho_r(i1,i2,i3)  =max(rho_min, rho_r(i1,i2,i3))
        !
        ! --- Limit pressure ---
        P_min            =0.5*min(P(i1,i2,i3), P(i1+1,i2,i3))
        P_l(i1,i2,i3)    =max(P_min, P_l(i1,i2,i3))
        P_r(i1,i2,i3)    =max(P_min, P_r(i1,i2,i3))
        !
        ! --- Limit internal energy, take non-zero lower limit into account ---
        rhoei_min        =0.5*min(rho(i1  ,i2,i3)*(ei(i1  ,i2,i3)-eiTable_min), &
                                  rho(i1+1,i2,i3)*(ei(i1+1,i2,i3)-eiTable_min)   )
        rhoei_l(i1,i2,i3)=max(rhoei_min, rhoei_l(i1,i2,i3))
        rhoei_r(i1,i2,i3)=max(rhoei_min, rhoei_r(i1,i2,i3))
        !
      end do ! i1
    end do ! i2
  end do ! i3
endif ! (reconstruction_type == ...)
!
! === Perform near-boundary slope reduction, if requested ===
if (reconstruction_order >= 1) then
  if ((par%N_magDiffSide > 0) .or. (par%N_magDiffTop > 0) .or. (par%N_magDiffBottom > 0)) then
    !
    ! \label{f90:n_magdiff__useMHD4__new}
    ! --- Set parameters for diffusive boundary layers ---
    if (dim == 1) then
      ! --- Set dimensions for x1 sweep ---
      !
      m1_0 = prop%m1
      n1_0 = prop%n1
      m2_0 = prop%m2
      n2_0 = prop%n2
      m3_0 = prop%m3
      n3_0 = prop%n3
      nb1 = par%N_magDiffSide
      nt1 = par%N_magDiffSide
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffBottom
      nt3 = par%N_magDiffTop
      !
    else if (dim == 2) then
      ! --- Set dimensions for x2 sweep ---
      !
      m1_0 = prop%m2
      n1_0 = prop%n2
      m2_0 = prop%m1
      n2_0 = prop%n1
      m3_0 = prop%m3
      n3_0 = prop%n3
      nb1 = par%N_magDiffSide
      nt1 = par%N_magDiffSide
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffBottom
      nt3 = par%N_magDiffTop
      !
   else ! (dim == 3)
      ! --- Set dimensions for x3 sweep ---
      !
      m1_0 = prop%m3
      n1_0 = prop%n3
      m2_0 = prop%m2
      n2_0 = prop%n2
      m3_0 = prop%m1
      n3_0 = prop%n1
      nb1 = par%N_magDiffBottom
      nt1 = par%N_magDiffTop
      nb2 = par%N_magDiffSide
      nt2 = par%N_magDiffSide
      nb3 = par%N_magDiffSide
      nt3 = par%N_magDiffSide
      !
    endif ! (dim)
    !
    ! --- Reduce the slopes near the boundaries ---
    do i3=m3-d3,n3+d3
      do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Compute relative distance to boundary ---
          if (prop%n2 > prop%m2) then
            ! --- Handle 3D case ---
            delta=min((i1   - m1_0 + 1 + epsilon) / (nb1 + epsilon), &
                      (n1_0 - i1   + 1 + epsilon) / (nt1 + epsilon), &
                      (i2   - m2_0     + epsilon) / (nb2 + epsilon), &
                      (n2_0 - i2       + epsilon) / (nt2 + epsilon), &
                      (i3   - m3_0     + epsilon) / (nb3 + epsilon), &
                      (n3_0 - i3       + epsilon) / (nt3 + epsilon)   )
          else
            ! --- Handle 2D case ---
            delta=min((i1   - m1_0 + 1 + epsilon) / (nb1 + epsilon), &
                      (n1_0 - i1   + 1 + epsilon) / (nt1 + epsilon), &
                      (i3   - m3_0     + epsilon) / (nb3 + epsilon), &
                      (n3_0 - i3       + epsilon) / (nt3 + epsilon)   )
          endif
          !
          ! --- Check if grid point is close to the boundary and reduce the slopes, if yes ---
          if (delta < 1.0) then
            !
            sigma = 1.0 / (1.0 + exp(10.0 * (0.5 - delta)))
            !
            ! --- Reduce the slope ---
            rho_l(i1,i2,i3)=sigma*rho_l(i1,i2,i3) + (1.0-sigma)* rho(i1,i2,i3)
            rho_r(i1,i2,i3)=sigma*rho_r(i1,i2,i3) + (1.0-sigma)* rho(i1,i2,i3)
            v1_l( i1,i2,i3)=sigma*v1_l( i1,i2,i3) + (1.0-sigma)* v1( i1,i2,i3)
            v1_r( i1,i2,i3)=sigma*v1_r( i1,i2,i3) + (1.0-sigma)* v1( i1,i2,i3)
            v2_l( i1,i2,i3)=sigma*v2_l( i1,i2,i3) + (1.0-sigma)* v2( i1,i2,i3)
            v2_r( i1,i2,i3)=sigma*v2_r( i1,i2,i3) + (1.0-sigma)* v2( i1,i2,i3)
            v3_l( i1,i2,i3)=sigma*v3_l( i1,i2,i3) + (1.0-sigma)* v3( i1,i2,i3)
            v3_r( i1,i2,i3)=sigma*v3_r( i1,i2,i3) + (1.0-sigma)* v3( i1,i2,i3)
            Bc1_l(i1,i2,i3)=sigma*Bc1_l(i1,i2,i3) + (1.0-sigma)* Bc1(i1,i2,i3)
            Bc1_r(i1,i2,i3)=sigma*Bc1_r(i1,i2,i3) + (1.0-sigma)* Bc1(i1,i2,i3)
            Bc2_l(i1,i2,i3)=sigma*Bc2_l(i1,i2,i3) + (1.0-sigma)* Bc2(i1,i2,i3)
            Bc2_r(i1,i2,i3)=sigma*Bc2_r(i1,i2,i3) + (1.0-sigma)* Bc2(i1,i2,i3)
            Bc3_l(i1,i2,i3)=sigma*Bc3_l(i1,i2,i3) + (1.0-sigma)* Bc3(i1,i2,i3)
            Bc3_r(i1,i2,i3)=sigma*Bc3_r(i1,i2,i3) + (1.0-sigma)* Bc3(i1,i2,i3)
            P_l(  i1,i2,i3)=sigma*P_l(  i1,i2,i3) + (1.0-sigma)* P(  i1,i2,i3)
            P_r(  i1,i2,i3)=sigma*P_r(  i1,i2,i3) + (1.0-sigma)* P(  i1,i2,i3)
            rhoei_l( i1,i2,i3)=sigma*rhoei_l( i1,i2,i3) + &
                               (1.0-sigma)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
            rhoei_r( i1,i2,i3)=sigma*rhoei_r( i1,i2,i3) + &
                               (1.0-sigma)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
            !
          endif
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !
  endif ! ((par%N_magDiffSide > 0) .or. (par%N_magDiffTop > 0) .or. (par%N_magDiffBottom > 0))
endif ! (reconstruction_order)
!
end subroutine rhd_mhd_Reconstruct


!----------***************--------------------------------------------------------------------------
subroutine rhd_mhd_Hancock(m1,n1, m2,n2, m3,n3, nghost1, &
                           dtime, xb1, &
                           rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                           Gamma_1, f, phib_grav, &
                           rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                           rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Hancock ('rhd_magneto-hydrodynamics_Hancock')
!
! PURPOSE:
!   Perform (1D) Hancock predictor step.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Hancock(m1,n1, m2,n2, m3,n3, nghost1, &
!                        dtime, xb1, &
!                        rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
!                        Gamma_1, f, phib_grav, &
!                        rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
!                        rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('v_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('v_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('v_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   f:
!   phib_grav:
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   rho_l:        ('rho_left') real 3D array, density at the left edge of the cell
!   v1_l:         ('v1_left') real 3D array, velocity 1 at the left edge of the cell
!   v2_l:         ('v2_left') real 3D array, velocity 2 at the left edge of the cell
!   v3_l:         ('v3_left') real 3D array, velocity 3 at the left edge of the cell
!   Bc1_l:        ('Bc1_left') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l:        ('Bc2_left') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l:        ('Bc3_left') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l:          ('P_left') real 3D array, pressure at the left edge of the cell
!   rhoei_l:      ('rhoei_left') real 3D array, internal energy per volume at left edge of the cell
!                 with eiTable_min subtracted
!   rho_r:        ('rho_right') real 3D array, density at the right edge of the cell
!   v1_r:         ('v1_right') real 3D array, velocity 1 at the right edge of the cell
!   v2_r:         ('v2_right') real 3D array, velocity 2 at the right edge of the cell
!   v3_r:         ('v3_right') real 3D array, velocity 3 at the right edge of the cell
!   Bc1_r:        ('Bc1_right') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r:        ('Bc2_right') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r:        ('Bc3_right') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r:          ('P_right') real 3D array, pressure at the right edge of the cell
!   rhoei_r:      ('rhoei_right') real 3D array, internal energy per volume at right edge of cell
!                 with eiTable_min subtracted
!
! VARIABLES:
!
! ROUTINES:
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that there are no ghost zones present in "orthogonal directions":
!     model%nghost2=0, model%nghost3=0
!   No gravity corrections.
!   rhoei_l and rhoei_r have the minimum internal energy of the EOS table (eiTable_min) subtracted.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-01-04 (W.S. Exeter) put into separate subroutine
!   2011-01-07 (W.S.) Positivity of Hancock step by additional limiting
!   2012-01-17 (W.S,) Gravity terms in Hancock predictor step
!   2013-03-01 (B.F. Lyon) Added some abs() functions in additional limiting step
!   2013-03-06 (B.F.) Outcommented the abs() functions in additional limiting step
!   2013-06-21 (B.F.) Compute the change for left and right side only once
!   2018-10-18 (B.F.) Bugfix: subtract eiTable_min from ei, where necessary
!---------------------------------------------------------------------------------------------------
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1
real,                      intent(in)   :: dtime
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m1-nghost1:n1+nghost1,&
                m2        :n2        ,&
                m3        :n3      ), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, Gamma_1, f
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2        :n2         ,&
                m3        :n3       ), &
                           intent(in)   :: phib_grav
real, dimension(m1-1:n1+1,&
                m2  :n2        ,&
                m3  :n3      ), &
                           intent(inout):: rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, &
                                           P_l, rhoei_l, &
                                           rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, &
                                           P_r, rhoei_r
!
! --- Local variables ---
real, dimension(m1-1:n1+1)              :: dtimeoverdx
integer                                 :: i1, i2, i3
real                                    :: delta_rho, delta_v1, delta_v2, delta_v3, &
                                           delta_Bc1, delta_Bc2, delta_Bc3, &
                                           delta_P, delta_rhoei, delta_phi, &
                                           rho_min, P_min, rhoei_min, eiTable_min, &
                                           lambda_v1, lambda_rho, lambda_P, lambda_rhoei, lambda, &
                                           delta
!---------------------------------------------------------------------------------------------------
!
dtimeoverdx = dtime / (xb1(m1:n1+2) - xb1(m1-1:n1+1))
!
call tabinter_getinfo(x2min=eiTable_min)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1-1,n1+1
      !
      delta_rho  = rho_r(  i1,i2,i3) - rho_l(  i1,i2,i3)
      delta_v1   = v1_r(   i1,i2,i3) - v1_l(   i1,i2,i3)
      delta_v2   = v2_r(   i1,i2,i3) - v2_l(   i1,i2,i3)
      delta_v3   = v3_r(   i1,i2,i3) - v3_l(   i1,i2,i3)
      delta_Bc1  = Bc1_r(  i1,i2,i3) - Bc1_l(  i1,i2,i3)
      delta_Bc2  = Bc2_r(  i1,i2,i3) - Bc2_l(  i1,i2,i3)
      delta_Bc3  = Bc3_r(  i1,i2,i3) - Bc3_l(  i1,i2,i3)
      delta_P    = P_r(    i1,i2,i3) - P_l(    i1,i2,i3)
      delta_rhoei= rhoei_r(i1,i2,i3) - rhoei_l(i1,i2,i3)
      !
      delta_phi  = phib_grav(i1+1,i2,i3) - phib_grav(i1,i2,i3)
      !
      ! --- Limit reconstruction to ensure positivity of Hancock step ---
      !
      rho_min  = min(rho_l(  i1,i2,i3), rho_r(  i1,i2,i3))
      p_min    = min(P_l(    i1,i2,i3), P_r(    i1,i2,i3))
      rhoei_min= min(rhoei_l(i1,i2,i3), rhoei_r(i1,i2,i3))
      !
      lambda_v1   = 1.8 * min(rho_min  / rho(i1,i2,i3), &
                              P_min    / P(  i1,i2,i3), &
                              rhoei_min/(rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) &
                             )    / dtimeoverdx(i1)
      lambda_rho  = 1.8 * rho_min / dtimeoverdx(i1) - &
                    0.9 * rho(i1,i2,i3) * delta_v1
      lambda_P    = 1.8 * P_min   / dtimeoverdx(i1) - &
                    0.9 * Gamma_1(i1,i2,i3) * P(i1,i2,i3) * delta_v1
      lambda_rhoei= 1.8 * rhoei_min / dtimeoverdx(i1) - &
                    0.9 * (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) * delta_v1
      !
      if (delta_v1 > lambda_v1) then
!!$      if (abs(delta_v1) > lambda_v1) then
        !
        lambda = lambda_v1 / delta_v1
!!$        lambda = lambda_v1 / abs(delta_v1)
        !
        ! --- Constant reconstruction for rho, P, rhoei ---
        delta_rho  = 0.0
        delta_P    = 0.0
        delta_rhoei= 0.0
        !
        ! --- Limit v1 reconstruction ---
        delta_v1 = lambda_v1
!!$        delta_v1 = delta_v1 * lambda
        !
        ! --- Recompute left and right states ---
        rho_l(  i1,i2,i3) = rho(i1,i2,i3)
        rho_r(  i1,i2,i3) = rho(i1,i2,i3)
        P_l(    i1,i2,i3) = P(  i1,i2,i3)
        P_r(    i1,i2,i3) = P(  i1,i2,i3)
        ! --- Bugfix: 2018-10-18: ei -> ei-eiTable_min ---
        rhoei_l(i1,i2,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r(i1,i2,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        v1_l(   i1,i2,i3) = lambda*v1_l(i1,i2,i3) + (1.0-lambda)*v1(i1,i2,i3)
        v1_r(   i1,i2,i3) = lambda*v1_r(i1,i2,i3) + (1.0-lambda)*v1(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta_rho > lambda_rho) then
!!$      else if (abs(v1(i1,i2,i3)*delta_rho) > lambda_rho) then
        !
        ! --- Limit rho reconstruction ---
        lambda = lambda_rho / abs(v1(i1,i2,i3)*delta_rho)
        !
        delta_rho = delta_rho * lambda
        rho_l(i1,i2,i3) = lambda*rho_l(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)
        rho_r(i1,i2,i3) = lambda*rho_r(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta_P > lambda_P) then
!!$      else if (abs(v1(i1,i2,i3)*delta_P) > lambda_P) then
        !
        ! --- Limit P reconstruction ---
        lambda = lambda_P / abs(v1(i1,i2,i3)*delta_P)
        !
        delta_P = delta_P * lambda
        P_l(i1,i2,i3) = lambda*P_l(i1,i2,i3)+(1.0-lambda)*P(i1,i2,i3)
        P_r(i1,i2,i3) = lambda*P_r(i1,i2,i3)+(1.0-lambda)*P(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta_rhoei > lambda_rhoei) then
!!$      else if (abs(v1(i1,i2,i3)*delta_rhoei) > lambda_rhoei) then
        !
        ! --- Limit rhoei reconstruction ---
        lambda = lambda_rhoei / abs(v1(i1,i2,i3)*delta_rhoei)
        !
        delta_rhoei = delta_rhoei * lambda
        rhoei_l(i1,i2,i3) =     lambda *rhoei_l(i1,i2,i3)+ &
                           (1.0-lambda)*    rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r(i1,i2,i3) =     lambda *rhoei_r(i1,i2,i3)+ &
                           (1.0-lambda)*    rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        !
      endif
      !
      ! === Hancock predictor step ========================================================
      !
      ! --- Note that the left state of the Riemann problem is the state at the right ---
      ! --- side of a cell and vice versa
      !
      !
      ! --- Left and right side of the cell ---
      delta=-0.5*dtimeoverdx(i1) * &
                           (             v1(i1,i2,i3)             * delta_rho + &
                                        rho(i1,i2,i3)             * delta_v1     )
      rho_l(i1,i2,i3)=rho_l(i1,i2,i3) + delta
      rho_r(i1,i2,i3)=rho_r(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (             v1(i1,i2,i3)             * delta_v1                - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta_Bc1/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta_Bc2/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta_Bc3/rho(i1,i2,i3) + &
                                                                    delta_P  /rho(i1,i2,i3) - &
                                                                    delta_phi                  )
      v1_l( i1,i2,i3)=v1_l( i1,i2,i3) + delta
      v1_r( i1,i2,i3)=v1_r( i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (             v1(i1,i2,i3)             * delta_v2                - &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta_Bc1/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta_Bc2/rho(i1,i2,i3)    )
      v2_l( i1,i2,i3)=v2_l( i1,i2,i3) + delta
      v2_r( i1,i2,i3)=v2_r( i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (             v1(i1,i2,i3)             * delta_v3                - &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta_Bc1/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta_Bc3/rho(i1,i2,i3)    )
      v3_l( i1,i2,i3)=v3_l( i1,i2,i3) + delta
      v3_r( i1,i2,i3)=v3_r( i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (             v1(i1,i2,i3)             * delta_Bc1   )
      Bc1_l(i1,i2,i3)=Bc1_l(i1,i2,i3) + delta
      Bc1_r(i1,i2,i3)=Bc1_r(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (            Bc2(i1,i2,i3)             * delta_v1 - &
                                        Bc1(i1,i2,i3)             * delta_v2 + &
                                         v1(i1,i2,i3)             * delta_Bc2   )
      Bc2_l(i1,i2,i3)=Bc2_l(i1,i2,i3) + delta
      Bc2_r(i1,i2,i3)=Bc2_r(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (            Bc3(i1,i2,i3)             * delta_v1 - &
                                        Bc1(i1,i2,i3)             * delta_v3 + &
                                         v1(i1,i2,i3)             * delta_Bc3   )
      Bc3_l(i1,i2,i3)=Bc3_l(i1,i2,i3) + delta
      Bc3_r(i1,i2,i3)=Bc3_r(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                           (        Gamma_1(i1,i2,i3)*P(i1,i2,i3) * delta_v1 + &
                                         v1(i1,i2,i3)             * delta_P     )
      P_l(  i1,i2,i3)=P_l(  i1,i2,i3) + delta
      P_r(  i1,i2,i3)=P_r(  i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx(i1) * &
                          ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3)) * delta_v1 + &
                             v1(i1,i2,i3)                                         * delta_rhoei )
      rhoei_l( i1,i2,i3)=rhoei_l(i1,i2,i3) + delta
      rhoei_r( i1,i2,i3)=rhoei_r(i1,i2,i3) + delta
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_Hancock


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Hancock2D(m1,n1, m2,n2, m3,n3, nghost1, nghost3, &
                             dtime, xb1, xb3, &
                             rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                             Gamma_1,  f, phib_grav, phib_grav3, &
                             rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                             rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
                             rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, P_l3, rhoei_l3, &
                             rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, P_r3, rhoei_r3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Hancock2D ('rhd_magneto-hydrodynamics_Hancock_2D')
!
! PURPOSE:
!   Perform 2D Hancock predictor step.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Hancock2D(m1,n1, m2,n2, m3,n3, nghost1, nghost3, &
!                          dtime, xb1, xb3, &
!                          rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
!                          Gamma_1, f, phib_grav, phib_grav3, &
!                          rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
!                          rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('v_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('v_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('v_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('Pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   f:
!   phib_grav:
!   phib_grav3:
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   rho_l1:       ('rho_left_1') real 3D array, density at the left edge of the cell
!   v1_l1:        ('v1_left_1') real 3D array, velocity 1 at the left edge of the cell
!   v2_l1:        ('v2_left_1') real 3D array, velocity 2 at the left edge of the cell
!   v3_l1:        ('v3_left_1') real 3D array, velocity 3 at the left edge of the cell
!   Bc1_l1:       ('Bc1_left_1') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l1:       ('Bc2_left_1') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l1:       ('Bc3_left_1') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l1:         ('P_left_1') real 3D array, pressure at the left edge of the cell
!   rhoei_l1:     ('rhoei_left_1') real 3D array, internal energy per volume at left edge of cell
!   rho_r1:       ('rho_right_1') real 3D array, density at the right edge of the cell
!   v1_r1:        ('v1_right_1') real 3D array, velocity 1 at the right edge of the cell
!   v2_r1:        ('v2_right_1') real 3D array, velocity 2 at the right edge of the cell
!   v3_r1:        ('v3_right_1') real 3D array, velocity 3 at the right edge of the cell
!   Bc1_r1:       ('Bc1_right_1') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r1:       ('Bc2_right_1') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r1:       ('Bc3_right_1') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r1:         ('P_right_1') real 3D array, pressure at the right edge of the cell
!   rhoei_r1:     ('rhoei_right_1') real 3D array, internal energy per volume at right edge of cell
!   rho_l3:       ('rho_left_3') real 3D array, density at the left edge of the cell
!   v1_l3:        ('v1_left_3') real 3D array, velocity 1 at the left edge of the cell
!   v2_l3:        ('v2_left_3') real 3D array, velocity 2 at the left edge of the cell
!   v3_l3:        ('v3_left_3') real 3D array, velocity 3 at the left edge of the cell
!   Bc1_l3:       ('Bc1_left_3') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l3:       ('Bc2_left_3') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l3:       ('Bc3_left_3') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l3:         ('P_left_3') real 3D array, pressure at the left edge of the cell
!   rhoei_l3:     ('rhoei_left_3') real 3D array, internal energy per volume at left edge of cell
!   rho_r3:       ('rho_right_3') real 3D array, density at the right edge of the cell
!   v1_r3:        ('v1_right_3') real 3D array, velocity 1 at the right edge of the cell
!   v2_r3:        ('v2_right_3') real 3D array, velocity 2 at the right edge of the cell
!   v3_r3:        ('v3_right_3') real 3D array, velocity 3 at the right edge of the cell
!   Bc1_r3:       ('Bc1_right_3') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r3:       ('Bc2_right_3') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r3:       ('Bc3_right_3') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r3:         ('P_right_3') real 3D array, pressure at the right edge of the cell
!   rhoei_r3:     ('rhoei_right_3') real 3D array, internal energy per volume at right edge of cell
!
! VARIABLES:
!
! ROUTINES:
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   It is assumed that there are no ghost zones present in x2-direction:
!   model%nghost2=0
!   No gravity corrections
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-01-04 (W.S. Exeter) put into separate subroutine
!   2011-01-07 (W.S.) Positivity of Hancock step by additional limiting
!   2011-06-10 (W.S.) Derived from routine rhd_mhd_Hancock
!   2012-01-17 (W.S.) Gravity terms in Hancock predictor step
!   2013-03-01 (B.F. Lyon) Added some abs() functions in additional limiting step
!   2013-03-06 (B.F.) Outcommented the abs() functions in additional limiting step
!   2018-10-18 (B.F.) Bugfix: subtract eiTable_min from ei, where necessary
!---------------------------------------------------------------------------------------------------
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost3
real,                      intent(in)   :: dtime
real, dimension(m1-nghost1  :  &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m3-nghost3  :  &
                n3+nghost3+1), &
                           intent(in)   :: xb3
real, dimension(m1-nghost1:n1+nghost1 , &
                m2        :n2         , &
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, Gamma_1, f
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2        :n2         , &
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: phib_grav
real, dimension(m3-nghost3:n3+nghost3+1,&
                m2        :n2         , &
                m1-nghost1:n1+nghost1), &
                           intent(in)   :: phib_grav3
real, dimension(m1-1      :n1+1       , &
                m2        :n2         , &
                m3-1      :n3+1), &
                           intent(inout):: rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, &
                                           P_l1, rhoei_l1, &
                                           rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, &
                                           P_r1, rhoei_r1
real, dimension(m3-1      :n3+1       , &
                m2        :n2         , &
                m1-1      :n1+1      ), &
                           intent(inout):: rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, &
                                           P_l3, rhoei_l3, &
                                           rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, &
                                           P_r3, rhoei_r3
!
! --- Local variables ---
real, dimension(m1-1:n1+1)              :: dtimeoverdx1
real, dimension(m3-1:n3+1)              :: dtimeoverdx3
integer                                 :: i1, i2, i3
real                                    :: delta1_rho, delta1_v1, delta1_v2, delta1_v3, &
                                           delta1_Bc1, delta1_Bc2, delta1_Bc3, &
                                           delta1_P, delta1_rhoei, delta1_phi, &
                                           delta3_rho, delta3_v1, delta3_v2, delta3_v3, &
                                           delta3_Bc1, delta3_Bc2, delta3_Bc3, &
                                           delta3_P, delta3_rhoei, delta3_phi, &
                                           rho_min, P_min, rhoei_min, eiTable_min, &
                                           lambda_v1, lambda_v3, &
                                           lambda_rho, lambda_P, lambda_rhoei, lambda
!---------------------------------------------------------------------------------------------------
!
dtimeoverdx1 = dtime / (xb1(m1:n1+2) - xb1(m1-1:n1+1))
dtimeoverdx3 = dtime / (xb3(m3:n3+2) - xb3(m3-1:n3+1))
!
call tabinter_getinfo(x2min=eiTable_min)
!
do i3=m3-1,n3+1
  do i2=m2,n2
    do i1=m1-1,n1+1
      !
      delta1_rho = rho_r1(i1,i2,i3) - rho_l1(i1,i2,i3)
      delta1_v1  = v1_r1( i1,i2,i3) - v1_l1( i1,i2,i3)
      delta1_v2  = v2_r1( i1,i2,i3) - v2_l1( i1,i2,i3)
      delta1_v3  = v3_r1( i1,i2,i3) - v3_l1( i1,i2,i3)
      delta1_Bc1 = Bc1_r1(i1,i2,i3) - Bc1_l1(i1,i2,i3)
      delta1_Bc2 = Bc2_r1(i1,i2,i3) - Bc2_l1(i1,i2,i3)
      delta1_Bc3 = Bc3_r1(i1,i2,i3) - Bc3_l1(i1,i2,i3)
      delta1_P   = P_r1(  i1,i2,i3) - P_l1(  i1,i2,i3)
      delta1_rhoei = rhoei_r1(i1,i2,i3) - rhoei_l1(i1,i2,i3)
      delta1_phi = phib_grav(i1+1,i2,i3) - phib_grav(i1,i2,i3)
      !
      delta3_rho = rho_r3(i3,i2,i1) - rho_l3(i3,i2,i1)
      delta3_v1  = v1_r3( i3,i2,i1) - v1_l3( i3,i2,i1)
      delta3_v2  = v2_r3( i3,i2,i1) - v2_l3( i3,i2,i1)
      delta3_v3  = v3_r3( i3,i2,i1) - v3_l3( i3,i2,i1)
      delta3_Bc1 = Bc1_r3(i3,i2,i1) - Bc1_l3(i3,i2,i1)
      delta3_Bc2 = Bc2_r3(i3,i2,i1) - Bc2_l3(i3,i2,i1)
      delta3_Bc3 = Bc3_r3(i3,i2,i1) - Bc3_l3(i3,i2,i1)
      delta3_P   = P_r3(  i3,i2,i1) - P_l3(  i3,i2,i1)
      delta3_rhoei = rhoei_r3(i3,i2,i1) - rhoei_l3(i3,i2,i1)
      delta3_phi = phib_grav3(i3+1,i2,i1) - phib_grav3(i3,i2,i1)
      !
      ! --- Limit reconstruction to ensure positivity of Hancock step---
      !
      rho_min = min(rho_l1(i1,i2,i3), rho_r1(i1,i2,i3), rho_l3(i3,i2,i1), rho_r3(i3,i2,i1))
      p_min   = min(P_l1(  i1,i2,i3), P_r1(  i1,i2,i3), P_l3(  i3,i2,i1), P_r3(  i3,i2,i1))
      rhoei_min = min(rhoei_l1(i1,i2,i3), rhoei_r1(i1,i2,i3),rhoei_l3(i3,i2,i1), rhoei_r3(i3,i2,i1))
      !
      lambda_v1 = 0.9 * min(rho_min / rho(i1,i2,i3), P_min / P(i1,i2,i3), &
                          rhoei_min / (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3))) / &
                        dtimeoverdx1(i1)
      !
      lambda_rho = 0.9 * rho_min / dtimeoverdx1(i1) - 0.45 * rho(i1,i2,i3) * delta1_v1
      lambda_P   = 0.9 * P_min / dtimeoverdx1(i1) - 0.45 * Gamma_1(i1,i2,i3) * P(i1,i2,i3) * delta1_v1
      lambda_rhoei = 0.9 * rhoei_min / dtimeoverdx1(i1) - &
                     0.45 * (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) * delta1_v1
      !
      if (delta1_v1 > lambda_v1) then
!!$      if (abs(delta1_v1) > lambda_v1) then
        !
        lambda = lambda_v1 / delta1_v1
!!$        lambda = lambda_v1 / abs(delta1_v1)
        !
        ! --- Constant reconstruction for rho, P, rhoei ---
        delta1_rho = 0.0
        delta1_P = 0.0
        delta1_rhoei = 0.0
        !
        ! --- Limit v1 reconstruction ---
        delta1_v1 = lambda_v1
!!$        delta1_v1 = delta1_v1 * lambda
        !
        ! --- Recompute left and right states ---
        rho_l1(i1,i2,i3) = rho(i1,i2,i3)
        rho_r1(i1,i2,i3) = rho(i1,i2,i3)
        P_l1(i1,i2,i3) = P(i1,i2,i3)
        P_r1(i1,i2,i3) = P(i1,i2,i3)
        ! --- Bugfix: 2018-10-18: ei -> ei-eiTable_min ---
        rhoei_l1(i1,i2,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r1(i1,i2,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        v1_l1(i1,i2,i3) = lambda*v1_l1(i1,i2,i3) + (1.0-lambda)*v1(i1,i2,i3)
        v1_r1(i1,i2,i3) = lambda*v1_r1(i1,i2,i3) + (1.0-lambda)*v1(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta1_rho > lambda_rho) then
!!$      else if (abs(v1(i1,i2,i3)*delta1_rho) > lambda_rho) then
        !
        ! --- Limit rho reconstruction ---
        lambda = lambda_rho / abs(v1(i1,i2,i3)*delta1_rho)
        !
        delta1_rho = delta1_rho * lambda
        rho_l1(i1,i2,i3) = lambda*rho_l1(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)
        rho_r1(i1,i2,i3) = lambda*rho_r1(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta1_P > lambda_P) then
!!$      else if (abs(v1(i1,i2,i3)*delta1_P) > lambda_P) then
        !
        ! --- Limit P reconstruction ---
        lambda = lambda_P / abs(v1(i1,i2,i3)*delta1_P)
        !
        delta1_P = delta1_P * lambda
        P_l1(i1,i2,i3) = lambda*P_l1(i1,i2,i3)+(1.0-lambda)*P(i1,i2,i3)
        P_r1(i1,i2,i3) = lambda*P_r1(i1,i2,i3)+(1.0-lambda)*P(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta1_rhoei > lambda_rhoei) then
!!$      else if (abs(v1(i1,i2,i3)*delta1_rhoei) > lambda_rhoei) then
        !
        ! --- Limit rhoei reconstruction ---
        lambda = lambda_rhoei / abs(v1(i1,i2,i3)*delta1_rhoei)
        !
        delta1_rhoei = delta1_rhoei * lambda
        rhoei_l1(i1,i2,i3) = lambda*rhoei_l1(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r1(i1,i2,i3) = lambda*rhoei_r1(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        !
      endif
      !
      lambda_v3 = 0.9 * min(rho_min / rho(i1,i2,i3), P_min / P(i1,i2,i3), &
                          rhoei_min / (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3))) / &
                        dtimeoverdx3(i3)
      !
      lambda_rho = 0.9 * rho_min / dtimeoverdx3(i3) - 0.45 * rho(i1,i2,i3) * delta3_v3
      lambda_P   = 0.9 * P_min / dtimeoverdx3(i3) - 0.45 * Gamma_1(i1,i2,i3) * P(i1,i2,i3) * delta3_v3
      lambda_rhoei = 0.9 * rhoei_min / dtimeoverdx3(i3) - &
                     0.45 * (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) * delta3_v3
      !
      if (delta3_v3 > lambda_v3) then
!!$      if (abs(delta3_v3) > lambda_v3) then
        !
        lambda = lambda_v3 / delta3_v3
!!$        lambda = lambda_v3 / abs(delta3_v3)
        !
        ! --- Constant reconstruction for rho, P, rhoei ---
        delta3_rho = 0.0
        delta3_P = 0.0
        delta3_rhoei = 0.0
        !
        ! --- Limit v3 reconstruction ---
        delta3_v3 = lambda_v3
!!$        delta3_v3 = delta3_v3 * lambda
        !
        ! --- Recompute left and right states ---
        rho_l3(i3,i2,i1) = rho(i1,i2,i3)
        rho_r3(i3,i2,i1) = rho(i1,i2,i3)
        P_l3(i3,i2,i1) = P(i1,i2,i3)
        P_r3(i3,i2,i1) = P(i1,i2,i3)
        ! --- Bugfix: 2018-10-18: ei -> ei-eiTable_min ---
        rhoei_l3(i3,i2,i1) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r3(i3,i2,i1) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        v3_l3(i3,i2,i1) = lambda*v3_l3(i3,i2,i1) + (1.0-lambda)*v3(i1,i2,i3)
        v3_r3(i3,i2,i1) = lambda*v3_r3(i3,i2,i1) + (1.0-lambda)*v3(i1,i2,i3)
        !
      else if (v3(i1,i2,i3)*delta3_rho > lambda_rho) then
!!$      else if (abs(v3(i1,i2,i3)*delta3_rho) > lambda_rho) then
        !
        ! --- Limit rho reconstruction ---
        lambda = lambda_rho / abs(v3(i1,i2,i3)*delta3_rho)
        !
        delta3_rho = delta3_rho * lambda
        rho_l3(i3,i2,i1) = lambda*rho_l3(i3,i2,i1)+(1.0-lambda)*rho(i1,i2,i3)
        rho_r3(i3,i2,i1) = lambda*rho_r3(i3,i2,i1)+(1.0-lambda)*rho(i1,i2,i3)
        !
      else if (v3(i1,i2,i3)*delta3_P > lambda_P) then
!!$      else if (abs(v3(i1,i2,i3)*delta3_P) > lambda_P) then
        !
        ! --- Limit P reconstruction ---
        lambda = lambda_P / abs(v3(i1,i2,i3)*delta3_P)
        !
        delta3_P = delta3_P * lambda
        P_l3(i3,i2,i1) = lambda*P_l3(i3,i2,i1)+(1.0-lambda)*P(i1,i2,i3)
        P_r3(i3,i2,i1) = lambda*P_r3(i3,i2,i1)+(1.0-lambda)*P(i1,i2,i3)
        !
      else if (v3(i1,i2,i3)*delta3_rhoei > lambda_rhoei) then
!!$      else if (abs(v3(i1,i2,i3)*delta3_rhoei) > lambda_rhoei) then
        !
        ! --- Limit rhoei reconstruction ---
        lambda = lambda_rhoei / abs(v3(i1,i2,i3)*delta3_rhoei)
        !
        delta3_rhoei = delta3_rhoei * lambda
        rhoei_l3(i3,i2,i1) = lambda*rhoei_l3(i3,i2,i1)+(1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r3(i3,i2,i1) = lambda*rhoei_r3(i3,i2,i1)+(1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        !
      endif
      !
      ! === Hancock predictor step ========================================================
      !
      ! --- Note that the left state of the Riemann problem is the state at the right ---
      ! --- side of a cell and vice versa
      !
      ! --- x1 direction ---
      !
      ! --- Left side of the cell ---
      rho_l1(i1,i2,i3)=rho_l1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_rho + rho(i1,i2,i3)*delta1_v1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_rho + rho(i1,i2,i3)*delta3_v3)
      v1_l1( i1,i2,i3)=v1_l1( i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3) + delta1_P/rho(i1,i2,i3) - &
                       delta1_phi) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3))
      v2_l1( i1,i2,i3)=v2_l1( i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3))
      v3_l1( i1,i2,i3)=v3_l1( i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3) + delta3_P/rho(i1,i2,i3) - &
                       delta3_phi)
      Bc1_l1(i1,i2,i3)=Bc1_l1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_Bc1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc1(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v1 + &
                       v3(i1,i2,i3)*delta3_Bc1)
      Bc2_l1(i1,i2,i3)=Bc2_l1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (Bc2(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v2 + &
                       v1(i1,i2,i3)*delta1_Bc2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc2(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v2 + &
                       v3(i1,i2,i3)*delta3_Bc2)
      Bc3_l1(i1,i2,i3)=Bc3_l1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (Bc3(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v3 + &
                       v1(i1,i2,i3)*delta1_Bc3) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_Bc3)
      P_l1(  i1,i2,i3)=P_l1(  i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_P) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_P)
      rhoei_l1( i1,i2,i3)=rhoei_l1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_rhoei) - &
                                               0.5*dtimeoverdx3(i3) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_rhoei)
      !
      ! --- Right side of cell ---
      rho_r1(i1,i2,i3)=rho_r1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_rho + rho(i1,i2,i3)*delta1_v1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_rho + rho(i1,i2,i3)*delta3_v3)
      v1_r1( i1,i2,i3)=v1_r1( i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3) + delta1_P/rho(i1,i2,i3) - &
                       delta1_phi) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3))
      v2_r1( i1,i2,i3)=v2_r1( i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3))
      v3_r1( i1,i2,i3)=v3_r1( i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3) + delta3_P/rho(i1,i2,i3) - &
                       delta3_phi)
      Bc1_r1(i1,i2,i3)=Bc1_r1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_Bc1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc1(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v1 + &
                       v3(i1,i2,i3)*delta3_Bc1)
      Bc2_r1(i1,i2,i3)=Bc2_r1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (Bc2(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v2 + &
                       v1(i1,i2,i3)*delta1_Bc2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc2(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v2 + &
                       v3(i1,i2,i3)*delta3_Bc2)
      Bc3_r1(i1,i2,i3)=Bc3_r1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (Bc3(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v3 + &
                       v1(i1,i2,i3)*delta1_Bc3) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_Bc3)
      P_r1(  i1,i2,i3)=P_r1(  i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_P) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_P)
      rhoei_r1( i1,i2,i3)=rhoei_r1(i1,i2,i3) - 0.5*dtimeoverdx1(i1) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_rhoei) - &
                                               0.5*dtimeoverdx3(i3) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_rhoei)
      !
      ! --- x3 direction ---
      !
      ! --- Left side of the cell ---
      rho_l3(i3,i2,i1)=rho_l3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_rho + rho(i1,i2,i3)*delta1_v1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_rho + rho(i1,i2,i3)*delta3_v3)
      v1_l3( i3,i2,i1)=v1_l3( i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3) + delta1_P/rho(i1,i2,i3) - &
                       delta1_phi) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3))
      v2_l3( i3,i2,i1)=v2_l3( i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3))
      v3_l3( i3,i2,i1)=v3_l3( i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3) + delta3_P/rho(i1,i2,i3) - &
                       delta3_phi)
      Bc1_l3(i3,i2,i1)=Bc1_l3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_Bc1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc1(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v1 + &
                       v3(i1,i2,i3)*delta3_Bc1)
      Bc2_l3(i3,i2,i1)=Bc2_l3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (Bc2(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v2 + &
                       v1(i1,i2,i3)*delta1_Bc2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc2(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v2 + &
                       v3(i1,i2,i3)*delta3_Bc2)
      Bc3_l3(i3,i2,i1)=Bc3_l3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (Bc3(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v3 + &
                       v1(i1,i2,i3)*delta1_Bc3) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_Bc3)
      P_l3(  i3,i2,i1)=P_l3(  i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_P) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_P)
      rhoei_l3( i3,i2,i1)=rhoei_l3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_rhoei) - &
                                               0.5*dtimeoverdx3(i3) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_rhoei)
      !
      ! --- Right side of cell ---
      rho_r3(i3,i2,i1)=rho_r3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_rho + rho(i1,i2,i3)*delta1_v1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_rho + rho(i1,i2,i3)*delta3_v3)
      v1_r3( i3,i2,i1)=v1_r3( i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3) + delta1_P/rho(i1,i2,i3) - &
                       delta1_phi) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3))
      v2_r3( i3,i2,i1)=v2_r3( i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3))
      v3_r3( i3,i2,i1)=v3_r3( i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3) + delta3_P/rho(i1,i2,i3) - &
                       delta3_phi)
      Bc1_r3(i3,i2,i1)=Bc1_r3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_Bc1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc1(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v1 + &
                       v3(i1,i2,i3)*delta3_Bc1)
      Bc2_r3(i3,i2,i1)=Bc2_r3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (Bc2(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v2 + &
                       v1(i1,i2,i3)*delta1_Bc2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc2(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v2 + &
                       v3(i1,i2,i3)*delta3_Bc2)
      Bc3_r3(i3,i2,i1)=Bc3_r3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (Bc3(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v3 + &
                       v1(i1,i2,i3)*delta1_Bc3) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_Bc3)
      P_r3(  i3,i2,i1)=P_r3(  i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_P) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_P)
      rhoei_r3( i3,i2,i1)=rhoei_r3(i3,i2,i1) - 0.5*dtimeoverdx1(i1) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_rhoei) - &
                                               0.5*dtimeoverdx3(i3) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_rhoei)
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_Hancock2D


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Hancock3D(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                             dtime, xb1, xb2, xb3, &
                             rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                             Gamma_1,  f, phib_grav, phib_grav2, phib_grav3, &
                             rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                             rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
                             rho_l2, v1_l2, v2_l2, v3_l2, Bc1_l2, Bc2_l2, Bc3_l2, P_l2, rhoei_l2, &
                             rho_r2, v1_r2, v2_r2, v3_r2, Bc1_r2, Bc2_r2, Bc3_r2, P_r2, rhoei_r2, &
                             rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, P_l3, rhoei_l3, &
                             rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, P_r3, rhoei_r3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Hancock3D ('rhd_magneto-hydrodynamics_Hancock_3D')
!
! PURPOSE:
!   Perform 3D Hancock predictor step.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Hancock3D(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
!                         dtime, xb1, xb2, xb3, &
!                         rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
!                         Gamma_1, f, phib_grav, phib_grav2, phib_grav3, &
!                         rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
!                         rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
!                         rho_l2, v1_l2, v2_l2, v3_l2, Bc1_l2, Bc2_l2, Bc3_l2, P_l2, rhoei_l2, &
!                         rho_r2, v1_r2, v2_r2, v3_r2, Bc1_r2, Bc2_r2, Bc3_r2, P_r2, rhoei_r2, &
!                         rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, P_l3, rhoei_l3, &
!                         rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, P_r3, rhoei_r3)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('v_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('v_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('v_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   f:
!   phib_grav:
!   phib_grav2:
!   phib_grav3:
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   rho_l1:       ('rho_left_1') real 3D array, density at the left edge of the cell
!   v1_l1:        ('v1_left_1') real 3D array, velocity 1 at the left edge of the cell
!   v2_l1:        ('v2_left_1') real 3D array, velocity 2 at the left edge of the cell
!   v3_l1:        ('v3_left_1') real 3D array, velocity 3 at the left edge of the cell
!   Bc1_l1:       ('Bc1_left_1') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l1:       ('Bc2_left_1') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l1:       ('Bc3_left_1') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l1:         ('P_left_1') real 3D array, pressure at the left edge of the cell
!   rhoei_l1:     ('rhoei_left_1') real 3D array, internal energy per volume at the left edge
!                 of the cell
!   rho_r1:       ('rho_right_1') real 3D array, density at the right edge of the cell
!   v1_r1:        ('v1_right_1') real 3D array, velocity 1 at the right edge of the cell
!   v2_r1:        ('v2_right_1') real 3D array, velocity 2 at the right edge of the cell
!   v3_r1:        ('v3_right_1') real 3D array, velocity 3 at the right edge of the cell
!   Bc1_r1:       ('Bc1_right_1') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r1:       ('Bc2_right_1') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r1:       ('Bc3_right_1') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r1:         ('P_right_1') real 3D array, pressure at the right edge of the cell
!   rhoei_r1:     ('rhoei_right_1') real 3D array, internal energy per volume at the right edge
!                 of the cell
!   rho_l2:       ('rho_left_2') real 3D array, density at the left edge of the cell
!   v1_l2:        ('v1_left_2') real 3D array, velocity 1 at the left edge of the cell
!   v2_l2:        ('v2_left_2') real 3D array, velocity 2 at the left edge of the cell
!   v3_l2:        ('v3_left_2') real 3D array, velocity 3 at the left edge of the cell
!   Bc1_l2:       ('Bc1_left_2') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l2:       ('Bc2_left_2') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l2:       ('Bc3_left_2') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l2:         ('P_left_2') real 3D array, pressure at the left edge of the cell
!   rhoei_l2:     ('rhoei_left_2') real 3D array, internal energy per volume at the left edge
!                 of the cell
!   rho_r2:       ('rho_right_2') real 3D array, density at the right edge of the cell
!   v1_r2:        ('v1_right_2') real 3D array, velocity 1 at the right edge of the cell
!   v2_r2:        ('v2_right_2') real 3D array, velocity 2 at the right edge of the cell
!   v3_r2:        ('v3_right_2') real 3D array, velocity 3 at the right edge of the cell
!   Bc1_r2:       ('Bc1_right_2') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r2:       ('Bc2_right_2') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r2:       ('Bc3_right_2') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r2:         ('P_right_2') real 3D array, pressure at the right edge of the cell
!   rhoei_r2:     ('rhoei_right_2') real 3D array, internal energy per volume at the right edge
!                 of the cell
!   rho_l3:       ('rho_left_3') real 3D array, density at the left edge of the cell
!   v1_l3:        ('v1_left_3') real 3D array, velocity 1 at the left edge of the cell
!   v2_l3:        ('v2_left_3') real 3D array, velocity 2 at the left edge of the cell
!   v3_l3:        ('v3_left_3') real 3D array, velocity 3 at the left edge of the cell
!   Bc1_l3:       ('Bc1_left_3') real 3D array, magnetic field 1 at the left edge of the cell
!   Bc2_l3:       ('Bc2_left_3') real 3D array, magnetic field 2 at the left edge of the cell
!   Bc3_l3:       ('Bc3_left_3') real 3D array, magnetic field 3 at the left edge of the cell
!   P_l3:         ('P_left_3') real 3D array, pressure at the left edge of the cell
!   rhoei_l3:     ('rhoei_left_3') real 3D array, internal energy per volume at the left edge
!                 of the cell
!   rho_r3:       ('rho_right_3') real 3D array, density at the right edge of the cell
!   v1_r3:        ('v1_right_3') real 3D array, velocity 1 at the right edge of the cell
!   v2_r3:        ('v2_right_3') real 3D array, velocity 2 at the right edge of the cell
!   v3_r3:        ('v3_right_3') real 3D array, velocity 3 at the right edge of the cell
!   Bc1_r3:       ('Bc1_right_3') real 3D array, magnetic field 1 at the right edge of the cell
!   Bc2_r3:       ('Bc2_right_3') real 3D array, magnetic field 2 at the right edge of the cell
!   Bc3_r3:       ('Bc3_right_3') real 3D array, magnetic field 3 at the right edge of the cell
!   P_r3:         ('P_right_3') real 3D array, pressure at the right edge of the cell
!   rhoei_r3:     ('rhoei_right_3') real 3D array, internal energy per volume at the right edge
!                 of the cell
!
! VARIABLES:
!
! ROUTINES:
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   No gravity corrections
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-01-04 (W.S. Exeter) put into separate subroutine
!   2011-01-07 (W.S.) Positivity of Hancock step by additional limiting
!   2011-06-10 (W.S.) Derived from routine rhd_mhd_Hancock
!   2012-01-17 (W.S.) Gravity terms in Hancock predictor step
!   2013-03-01 (B.F. Lyon) Added some abs() functions in additional limiting step
!   2013-03-06 (B.F.) Outcommented the abs() functions in additional limiting step,
!                     compute the change for left and right side only once
!   2013-03-08 (B.F.) Reformat Hancock equations in x1 direction
!   2018-10-18 (B.F.) Bugfix: subtract eiTable_min from ei, where necessary
!---------------------------------------------------------------------------------------------------
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3
real,                      intent(in)   :: dtime
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m2-nghost2  : &
                n2+nghost2+1), &
                           intent(in)   :: xb2
real, dimension(m3-nghost3  : &
                n3+nghost3+1), &
                           intent(in)   :: xb3
real, dimension(m1-nghost1:n1+nghost1 , &
                m2-nghost2:n2+nghost2 , &
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, Gamma_1, f
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: phib_grav
real, dimension(m2-nghost2:n2+nghost2+1,&
                m1-nghost1:n1+nghost1,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: phib_grav2
real, dimension(m3-nghost3:n3+nghost3+1,&
                m2-nghost2:n2+nghost2,&
                m1-nghost1:n1+nghost1), &
                           intent(in)   :: phib_grav3
real, dimension(m1-1      :n1+1       , &
                m2-1      :n2+1       , &
                m3-1      :n3+1      ), &
                           intent(inout):: rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, &
                                           P_l1, rhoei_l1, &
                                           rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, &
                                           P_r1, rhoei_r1
real, dimension(m2-1      :n2+1       , &
                m1-1      :n1+1       , &
                m3-1      :n3+1      ), &
                           intent(inout):: rho_l2, v1_l2, v2_l2, v3_l2, Bc1_l2, Bc2_l2, Bc3_l2, &
                                           P_l2, rhoei_l2, &
                                           rho_r2, v1_r2, v2_r2, v3_r2, Bc1_r2, Bc2_r2, Bc3_r2, &
                                           P_r2, rhoei_r2
real, dimension(m3-1      :n3+1       , &
                m2-1      :n2+1       , &
                m1-1      :n1+1      ), &
                           intent(inout):: rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, &
                                           P_l3, rhoei_l3, &
                                           rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, &
                                           P_r3, rhoei_r3
!
! --- Local variables ---
real, dimension(m1-1:n1+1)              :: dtimeoverdx1
real, dimension(m2-1:n2+1)              :: dtimeoverdx2
real, dimension(m3-1:n3+1)              :: dtimeoverdx3
integer                                 :: i1, i2, i3
real                                    :: delta1_rho, delta1_v1, delta1_v2, delta1_v3, &
                                           delta1_Bc1, delta1_Bc2, delta1_Bc3, &
                                           delta1_P, delta1_rhoei, delta1_phi, &
                                           delta2_rho, delta2_v1, delta2_v2, delta2_v3, &
                                           delta2_Bc1, delta2_Bc2, delta2_Bc3, &
                                           delta2_P, delta2_rhoei, delta2_phi, &
                                           delta3_rho, delta3_v1, delta3_v2, delta3_v3, &
                                           delta3_Bc1, delta3_Bc2, delta3_Bc3, &
                                           delta3_P, delta3_rhoei, delta3_phi, &
                                           rho_min, P_min, rhoei_min, eiTable_min, &
                                           lambda_v1, lambda_v2, lambda_v3, &
                                           lambda_rho, lambda_P, lambda_rhoei, lambda, &
                                           delta
!---------------------------------------------------------------------------------------------------
!
dtimeoverdx1 = dtime / (xb1(m1:n1+2) - xb1(m1-1:n1+1))
dtimeoverdx2 = dtime / (xb2(m2:n2+2) - xb2(m2-1:n2+1))
dtimeoverdx3 = dtime / (xb3(m3:n3+2) - xb3(m3-1:n3+1))
!
call tabinter_getinfo(x2min=eiTable_min)
!
do i3=m3-1,n3+1
  do i2=m2-1,n2+1
    do i1=m1-1,n1+1
      !
      ! === Compute preliminary slopes within cells ================================================
      !
      delta1_rho  = rho_r1(  i1,i2,i3) - rho_l1(  i1,i2,i3)
      delta1_v1   = v1_r1(   i1,i2,i3) - v1_l1(   i1,i2,i3)
      delta1_v2   = v2_r1(   i1,i2,i3) - v2_l1(   i1,i2,i3)
      delta1_v3   = v3_r1(   i1,i2,i3) - v3_l1(   i1,i2,i3)
      delta1_Bc1  = Bc1_r1(  i1,i2,i3) - Bc1_l1(  i1,i2,i3)
      delta1_Bc2  = Bc2_r1(  i1,i2,i3) - Bc2_l1(  i1,i2,i3)
      delta1_Bc3  = Bc3_r1(  i1,i2,i3) - Bc3_l1(  i1,i2,i3)
      delta1_P    = P_r1(    i1,i2,i3) - P_l1(    i1,i2,i3)
      delta1_rhoei= rhoei_r1(i1,i2,i3) - rhoei_l1(i1,i2,i3)
      delta1_phi  = phib_grav(i1+1,i2,i3) - phib_grav(i1,i2,i3)
      !
      delta2_rho  = rho_r2(  i2,i1,i3) - rho_l2(  i2,i1,i3)
      delta2_v1   = v1_r2(   i2,i1,i3) - v1_l2(   i2,i1,i3)
      delta2_v2   = v2_r2(   i2,i1,i3) - v2_l2(   i2,i1,i3)
      delta2_v3   = v3_r2(   i2,i1,i3) - v3_l2(   i2,i1,i3)
      delta2_Bc1  = Bc1_r2(  i2,i1,i3) - Bc1_l2(  i2,i1,i3)
      delta2_Bc2  = Bc2_r2(  i2,i1,i3) - Bc2_l2(  i2,i1,i3)
      delta2_Bc3  = Bc3_r2(  i2,i1,i3) - Bc3_l2(  i2,i1,i3)
      delta2_P    = P_r2(    i2,i1,i3) - P_l2(    i2,i1,i3)
      delta2_rhoei= rhoei_r2(i2,i1,i3) - rhoei_l2(i2,i1,i3)
      delta2_phi  = phib_grav2(i2+1,i1,i3) - phib_grav2(i2,i1,i3)
      !
      delta3_rho  = rho_r3(  i3,i2,i1) - rho_l3(  i3,i2,i1)
      delta3_v1   = v1_r3(   i3,i2,i1) - v1_l3(   i3,i2,i1)
      delta3_v2   = v2_r3(   i3,i2,i1) - v2_l3(   i3,i2,i1)
      delta3_v3   = v3_r3(   i3,i2,i1) - v3_l3(   i3,i2,i1)
      delta3_Bc1  = Bc1_r3(  i3,i2,i1) - Bc1_l3(  i3,i2,i1)
      delta3_Bc2  = Bc2_r3(  i3,i2,i1) - Bc2_l3(  i3,i2,i1)
      delta3_Bc3  = Bc3_r3(  i3,i2,i1) - Bc3_l3(  i3,i2,i1)
      delta3_P    = P_r3(    i3,i2,i1) - P_l3(    i3,i2,i1)
      delta3_rhoei= rhoei_r3(i3,i2,i1) - rhoei_l3(i3,i2,i1)
      delta3_phi  = phib_grav3(i3+1,i2,i1) - phib_grav3(i3,i2,i1)
      !


! # ifdef rhd_mhd_test01

      ! === Limit reconstruction to ensure positivity of Hancock step ==============================
      !
      rho_min  = min(rho_l1(  i1,i2,i3), rho_r1(  i1,i2,i3), &
                     rho_l2(  i2,i1,i3), rho_r2(  i2,i1,i3), &
                     rho_l3(  i3,i2,i1), rho_r3(  i3,i2,i1))
      P_min    = min(P_l1(    i1,i2,i3), P_r1(    i1,i2,i3), &
                     P_l2(    i2,i1,i3), P_r2(    i2,i1,i3), &
                     P_l3(    i3,i2,i1), P_r3(    i3,i2,i1))
      rhoei_min= min(rhoei_l1(i1,i2,i3), rhoei_r1(i1,i2,i3), &
                     rhoei_l2(i2,i1,i3), rhoei_r2(i2,i1,i3), &
                     rhoei_l3(i3,i2,i1), rhoei_r3(i3,i2,i1))
      !
      lambda_v1 = 0.6 * min(rho_min   / rho(i1,i2,i3), &
                            P_min     / P(i1,i2,i3), &
                            rhoei_min / (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3))) / &
                        dtimeoverdx1(i1)
      !
      lambda_rho = 0.6 * rho_min / dtimeoverdx1(i1) - 0.3 * rho(i1,i2,i3) * delta1_v1
      lambda_P   = 0.6 * P_min / dtimeoverdx1(i1) - 0.3 * Gamma_1(i1,i2,i3) * P(i1,i2,i3) * delta1_v1
      lambda_rhoei = 0.6 * rhoei_min / dtimeoverdx1(i1) - &
                     0.3 * (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) * delta1_v1
      !
      if (delta1_v1 > lambda_v1) then
!!$      if (abs(delta1_v1) > lambda_v1) then
        !
        lambda = lambda_v1 / delta1_v1
!!$        lambda = lambda_v1 / abs(delta1_v1)
        !
        ! --- Constant reconstruction for rho, P, rhoei ---
        delta1_rho  = 0.0
        delta1_P    = 0.0
        delta1_rhoei= 0.0
        !
        ! --- Limit v1 reconstruction ---
        delta1_v1 = lambda_v1
!!$        delta1_v1 = delta1_v1 * lambda
        !
        ! --- Recompute left and right states ---
        rho_l1(  i1,i2,i3) = rho(i1,i2,i3)
        rho_r1(  i1,i2,i3) = rho(i1,i2,i3)
        P_l1(    i1,i2,i3) = P(  i1,i2,i3)
        P_r1(    i1,i2,i3) = P(  i1,i2,i3)
        ! --- Bugfix: 2018-10-18: ei -> ei-eiTable_min ---
        rhoei_l1(i1,i2,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r1(i1,i2,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        v1_l1(   i1,i2,i3) = lambda*v1_l1(i1,i2,i3) + (1.0-lambda)*v1(i1,i2,i3)
        v1_r1(   i1,i2,i3) = lambda*v1_r1(i1,i2,i3) + (1.0-lambda)*v1(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta1_rho > lambda_rho) then
!!$      else if (abs(v1(i1,i2,i3)*delta1_rho) > lambda_rho) then
        !
        ! --- Limit rho reconstruction ---
        lambda = lambda_rho / abs(v1(i1,i2,i3)*delta1_rho)
        !
        delta1_rho = delta1_rho * lambda
        rho_l1(i1,i2,i3) = lambda*rho_l1(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)
        rho_r1(i1,i2,i3) = lambda*rho_r1(i1,i2,i3)+(1.0-lambda)*rho(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta1_P > lambda_P) then
!!$      else if (abs(v1(i1,i2,i3)*delta1_P) > lambda_P) then
        !
        ! --- Limit P reconstruction ---
        lambda = lambda_P / abs(v1(i1,i2,i3)*delta1_P)
        !
        delta1_P = delta1_P * lambda
        P_l1(i1,i2,i3) = lambda*P_l1(i1,i2,i3)+(1.0-lambda)*P(i1,i2,i3)
        P_r1(i1,i2,i3) = lambda*P_r1(i1,i2,i3)+(1.0-lambda)*P(i1,i2,i3)
        !
      else if (v1(i1,i2,i3)*delta1_rhoei > lambda_rhoei) then
!!$      else if (abs(v1(i1,i2,i3)*delta1_rhoei) > lambda_rhoei) then
        !
        ! --- Limit rhoei reconstruction ---
        lambda = lambda_rhoei / abs(v1(i1,i2,i3)*delta1_rhoei)
        !
        delta1_rhoei = delta1_rhoei * lambda
        rhoei_l1(i1,i2,i3) =     lambda *rhoei_l1(i1,i2,i3)+ &
                            (1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r1(i1,i2,i3) =     lambda *rhoei_r1(i1,i2,i3)+ &
                            (1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        !
      endif
      !
      lambda_v2 = 0.6 * min(rho_min   / rho(i1,i2,i3), &
                            P_min     / P(i1,i2,i3), &
                            rhoei_min / (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3))) / &
                        dtimeoverdx2(i2)
      !
      lambda_rho = 0.6 * rho_min / dtimeoverdx2(i2) - 0.3 * rho(i1,i2,i3) * delta2_v2
      lambda_P   = 0.6 * P_min / dtimeoverdx2(i2) - 0.3 * Gamma_1(i1,i2,i3) * P(i1,i2,i3) * delta2_v2
      lambda_rhoei = 0.6 * rhoei_min / dtimeoverdx2(i2) - &
                     0.3 * (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) * delta2_v2
      !
      if (delta2_v2 > lambda_v2) then
!!$      if (abs(delta2_v2) > lambda_v2) then
        !
        lambda = lambda_v2 / delta2_v2
!!$        lambda = lambda_v2 / abs(delta2_v2)
        !
        ! --- Constant reconstruction for rho, P, rhoei ---
        delta2_rho   = 0.0
        delta2_P     = 0.0
        delta2_rhoei = 0.0
        !
        ! --- Limit v2 reconstruction ---
        delta2_v2 = lambda_v2
!!$        delta2_v2 = delta2_v2 * lambda
        !
        ! --- Recompute left and right states ---
        rho_l2(  i2,i1,i3) = rho(i1,i2,i3)
        rho_r2(  i2,i1,i3) = rho(i1,i2,i3)
        P_l2(    i2,i1,i3) = P(  i1,i2,i3)
        P_r2(    i2,i1,i3) = P(  i1,i2,i3)
        ! --- Bugfix: 2018-10-18: ei -> ei-eiTable_min ---
        rhoei_l2(i2,i1,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r2(i2,i1,i3) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        v2_l2(   i2,i1,i3) = lambda*v2_l2(i2,i1,i3) + (1.0-lambda)*v2(i1,i2,i3)
        v2_r2(   i2,i1,i3) = lambda*v2_r2(i2,i1,i3) + (1.0-lambda)*v2(i1,i2,i3)
        !
      else if (v2(i1,i2,i3)*delta2_rho > lambda_rho) then
!!$      else if (abs(v2(i1,i2,i3)*delta2_rho) > lambda_rho) then
        !
        ! --- Limit rho reconstruction ---
        lambda = lambda_rho / abs(v2(i1,i2,i3)*delta2_rho)
        !
        delta2_rho = delta2_rho * lambda
        rho_l2(i2,i1,i3) = lambda*rho_l2(i2,i1,i3)+(1.0-lambda)*rho(i1,i2,i3)
        rho_r2(i2,i1,i3) = lambda*rho_r2(i2,i1,i3)+(1.0-lambda)*rho(i1,i2,i3)
        !
      else if (v2(i1,i2,i3)*delta2_P > lambda_P) then
!!$      else if (abs(v2(i1,i2,i3)*delta2_P) > lambda_P) then
        !
        ! --- Limit P reconstruction ---
        lambda = lambda_P / abs(v2(i1,i2,i3)*delta2_P)
        !
        delta2_P = delta2_P * lambda
        P_l2(i2,i1,i3) = lambda*P_l2(i2,i1,i3)+(1.0-lambda)*P(i1,i2,i3)
        P_r2(i2,i1,i3) = lambda*P_r2(i2,i1,i3)+(1.0-lambda)*P(i1,i2,i3)
        !
      else if (v2(i1,i2,i3)*delta2_rhoei > lambda_rhoei) then
!!$      else if (abs(v2(i1,i2,i3)*delta2_rhoei) > lambda_rhoei) then
        !
        ! --- Limit rhoei reconstruction ---
        lambda = lambda_rhoei / abs(v2(i1,i2,i3)*delta2_rhoei)
        !
        delta2_rhoei = delta2_rhoei * lambda
        rhoei_l2(i2,i1,i3) =     lambda *rhoei_l2(i2,i1,i3)+ &
                            (1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r2(i2,i1,i3) =     lambda *rhoei_r2(i2,i1,i3)+ &
                            (1.0-lambda)*rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        !
      endif
      !
      lambda_v3 = 0.6 * min(rho_min   / rho(i1,i2,i3), &
                            P_min     / P(i1,i2,i3), &
                            rhoei_min / (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3))) / &
                        dtimeoverdx3(i3)
      !
      lambda_rho = 0.6 * rho_min / dtimeoverdx3(i3) - 0.3 * rho(i1,i2,i3) * delta3_v3
      lambda_P   = 0.6 * P_min / dtimeoverdx3(i3) - 0.3 * Gamma_1(i1,i2,i3) * P(i1,i2,i3) * delta3_v3
      lambda_rhoei = 0.6 * rhoei_min / dtimeoverdx3(i3) - &
                     0.3 * (rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min) + P(i1,i2,i3)) * delta3_v3
      !
      if (delta3_v3 > lambda_v3) then
!!$      if (abs(delta3_v3) > lambda_v3) then
        !
        lambda = lambda_v3 / delta3_v3
!!$        lambda = lambda_v3 / abs(delta3_v3)
        !
        ! --- Constant reconstruction for rho, P, rhoei ---
        delta3_rho   = 0.0
        delta3_P     = 0.0
        delta3_rhoei = 0.0
        !
        ! --- Limit v3 reconstruction ---
        delta3_v3 = lambda_v3
!!$        delta3_v3 = delta3_v3 * lambda
        !
        ! --- Recompute left and right states ---
        rho_l3(  i3,i2,i1) = rho(i1,i2,i3)
        rho_r3(  i3,i2,i1) = rho(i1,i2,i3)
        P_l3(    i3,i2,i1) = P(  i1,i2,i3)
        P_r3(    i3,i2,i1) = P(  i1,i2,i3)
        ! --- Bugfix: 2018-10-18: ei -> ei-eiTable_min ---
        rhoei_l3(i3,i2,i1) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r3(i3,i2,i1) = rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        v3_l3(   i3,i2,i1) = lambda*v3_l3(i3,i2,i1) + (1.0-lambda)*v3(i1,i2,i3)
        v3_r3(   i3,i2,i1) = lambda*v3_r3(i3,i2,i1) + (1.0-lambda)*v3(i1,i2,i3)
        !
      else if (v3(i1,i2,i3)*delta3_rho > lambda_rho) then
!!$      else if (abs(v3(i1,i2,i3)*delta3_rho) > lambda_rho) then
        !
        ! --- Limit rho reconstruction ---
        lambda = lambda_rho / abs(v3(i1,i2,i3)*delta3_rho)
        !
        delta3_rho = delta3_rho * lambda
        rho_l3(i3,i2,i1) = lambda*rho_l3(i3,i2,i1)+(1.0-lambda)*rho(i1,i2,i3)
        rho_r3(i3,i2,i1) = lambda*rho_r3(i3,i2,i1)+(1.0-lambda)*rho(i1,i2,i3)
        !
      else if (v3(i1,i2,i3)*delta1_P > lambda_P) then
!!$      else if (abs(v3(i1,i2,i3)*delta1_P) > lambda_P) then
        !
        ! --- Limit P reconstruction ---
        lambda = lambda_P / abs(v3(i1,i2,i3)*delta3_P)
        !
        delta3_P = delta3_P * lambda
        P_l3(i3,i2,i1) = lambda*P_l3(i3,i2,i1)+(1.0-lambda)*P(i1,i2,i3)
        P_r3(i3,i2,i1) = lambda*P_r3(i3,i2,i1)+(1.0-lambda)*P(i1,i2,i3)
        !
      else if (v3(i1,i2,i3)*delta3_rhoei > lambda_rhoei) then
!!$      else if (abs(v3(i1,i2,i3)*delta3_rhoei) > lambda_rhoei) then
        !
        ! --- Limit rhoei reconstruction ---
        lambda = lambda_rhoei / abs(v3(i1,i2,i3)*delta3_rhoei)
        !
        delta3_rhoei = delta3_rhoei * lambda
        rhoei_l3(i3,i2,i1) =     lambda *rhoei_l3(i3,i2,i1)+ &
                            (1.0-lambda)*     rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        rhoei_r3(i3,i2,i1) =     lambda *rhoei_r3(i3,i2,i1)+ &
                            (1.0-lambda)*     rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)
        !
      endif

! # endif /* rhd_mhd_test01 */


      !
      ! === Hancock predictor step =================================================================
      !
      ! --- Note that the left state of the Riemann problem is the state at the right ---
      ! --- side of a cell and vice versa
      !
      ! --- x1 direction ---------------------------------------------------------------------------
      !
      ! --- Left and right side of the cell ---
      delta=-0.5*dtimeoverdx1(i1) * &
                           (             v1(i1,i2,i3)             * delta1_rho + &
                                        rho(i1,i2,i3)             * delta1_v1     ) - &
             0.5*dtimeoverdx2(i2) * &
                           (             v2(i1,i2,i3)             * delta2_rho + &
                                        rho(i1,i2,i3)             * delta2_v2     ) - &
             0.5*dtimeoverdx3(i3) * &
                           (             v3(i1,i2,i3)             * delta3_rho + &
                                        rho(i1,i2,i3)             * delta3_v3     )
      rho_l1(i1,i2,i3)=rho_l1(i1,i2,i3) + delta
      rho_r1(i1,i2,i3)=rho_r1(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (             v1(i1,i2,i3)             * delta1_v1                - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta1_Bc1/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta1_Bc2/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta1_Bc3/rho(i1,i2,i3) + &
                                                                    delta1_P  /rho(i1,i2,i3) - &
                                                                    delta1_phi                  )- &
             0.5*dtimeoverdx2(i2) * &
                           (             v2(i1,i2,i3)             * delta2_v1                - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta2_Bc2/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta2_Bc1/rho(i1,i2,i3)    )- &
             0.5*dtimeoverdx3(i3) * &
                           (             v3(i1,i2,i3)             * delta3_v1                - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta3_Bc3/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta3_Bc1/rho(i1,i2,i3)    )
      v1_l1( i1,i2,i3)=v1_l1( i1,i2,i3) + delta
      v1_r1( i1,i2,i3)=v1_r1( i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (             v1(i1,i2,i3)             * delta1_v2                - &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta1_Bc1/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta1_Bc2/rho(i1,i2,i3)    )- &
             0.5*dtimeoverdx2(i2) * &
                           (             v2(i1,i2,i3)             * delta2_v2                - &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta2_Bc2/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta2_Bc1/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta2_Bc3/rho(i1,i2,i3) + &
                                                                    delta2_P  /rho(i1,i2,i3) - &
                                                                    delta2_phi                  )- &
             0.5*dtimeoverdx3(i3) * &
                           (             v3(i1,i2,i3)             * delta3_v2                - &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta3_Bc1/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta3_Bc2/rho(i1,i2,i3)    )
      v2_l1( i1,i2,i3)=v2_l1( i1,i2,i3) + delta
      v2_r1( i1,i2,i3)=v2_r1( i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (             v1(i1,i2,i3)             * delta1_v3                - &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta1_Bc1/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta1_Bc3/rho(i1,i2,i3)    )- &
             0.5*dtimeoverdx2(i2) * &
                           (             v2(i1,i2,i3)             * delta2_v3                - &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta2_Bc2/rho(i1,i2,i3) - &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta2_Bc3/rho(i1,i2,i3)    )- &
             0.5*dtimeoverdx3(i3) * &
                           (             v3(i1,i2,i3)             * delta3_v3                - &
                            f(i1,i2,i3)*Bc3(i1,i2,i3)             * delta3_Bc3/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc1(i1,i2,i3)             * delta3_Bc1/rho(i1,i2,i3) + &
                            f(i1,i2,i3)*Bc2(i1,i2,i3)             * delta3_Bc2/rho(i1,i2,i3) + &
                                                                    delta3_P  /rho(i1,i2,i3) - &
                                                                    delta3_phi                  )
      v3_l1( i1,i2,i3)=v3_l1( i1,i2,i3) + delta
      v3_r1( i1,i2,i3)=v3_r1( i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (             v1(i1,i2,i3)             * delta1_Bc1   ) - &
             0.5*dtimeoverdx2(i2) * &
                           (            Bc1(i1,i2,i3)             * delta2_v2 - &
                                        Bc2(i1,i2,i3)             * delta2_v1 + &
                                         v2(i1,i2,i3)             * delta2_Bc1   ) - &
             0.5*dtimeoverdx3(i3) * &
                           (            Bc1(i1,i2,i3)             * delta3_v3 - &
                                        Bc3(i1,i2,i3)             * delta3_v1 + &
                                         v3(i1,i2,i3)             * delta3_Bc1   )
      Bc1_l1(i1,i2,i3)=Bc1_l1(i1,i2,i3) + delta
      Bc1_r1(i1,i2,i3)=Bc1_r1(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (            Bc2(i1,i2,i3)             * delta1_v1 - &
                                        Bc1(i1,i2,i3)             * delta1_v2 + &
                                         v1(i1,i2,i3)             * delta1_Bc2   ) - &
             0.5*dtimeoverdx2(i2) * &
                           (             v2(i1,i2,i3)             * delta2_Bc2   ) - &
             0.5*dtimeoverdx3(i3) * &
                           (            Bc2(i1,i2,i3)             * delta3_v3 - &
                                        Bc3(i1,i2,i3)             * delta3_v2 + &
                                         v3(i1,i2,i3)             * delta3_Bc2)
      Bc2_l1(i1,i2,i3)=Bc2_l1(i1,i2,i3) + delta
      Bc2_r1(i1,i2,i3)=Bc2_r1(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (            Bc3(i1,i2,i3)             * delta1_v1 - &
                                        Bc1(i1,i2,i3)             * delta1_v3 + &
                                         v1(i1,i2,i3)             * delta1_Bc3   ) - &
             0.5*dtimeoverdx2(i2) * &
                           (            Bc3(i1,i2,i3)             * delta2_v2 - &
                                        Bc2(i1,i2,i3)             * delta2_v3 + &
                                         v2(i1,i2,i3)             * delta2_Bc3   ) - &
             0.5*dtimeoverdx3(i3) * &
                           (             v3(i1,i2,i3)             * delta3_Bc3   )
      Bc3_l1(i1,i2,i3)=Bc3_l1(i1,i2,i3) + delta
      Bc3_r1(i1,i2,i3)=Bc3_r1(i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                           (        Gamma_1(i1,i2,i3)*P(i1,i2,i3) * delta1_v1 + &
                                         v1(i1,i2,i3)             * delta1_P     ) - &
             0.5*dtimeoverdx2(i2) * &
                           (        Gamma_1(i1,i2,i3)*P(i1,i2,i3) * delta2_v2 + &
                                         v2(i1,i2,i3)             * delta2_P     ) - &
             0.5*dtimeoverdx3(i3) * &
                           (        Gamma_1(i1,i2,i3)*P(i1,i2,i3) * delta3_v3 + &
                                         v3(i1,i2,i3)             * delta3_P     )
      P_l1(  i1,i2,i3)=P_l1(  i1,i2,i3) + delta
      P_r1(  i1,i2,i3)=P_r1(  i1,i2,i3) + delta
      !
      delta=-0.5*dtimeoverdx1(i1) * &
                         ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3)) * delta1_v1 + &
                           v1( i1,i2,i3)                                         * delta1_rhoei )- &
             0.5*dtimeoverdx2(i2) * &
                         ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3)) * delta2_v2 + &
                           v2( i1,i2,i3)                                         * delta2_rhoei )- &
             0.5*dtimeoverdx3(i3) * &
                         ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3)) * delta3_v3 + &
                           v3( i1,i2,i3)                                         * delta3_rhoei )
      rhoei_l1( i1,i2,i3)=rhoei_l1(i1,i2,i3) + delta
      rhoei_r1( i1,i2,i3)=rhoei_r1(i1,i2,i3) + delta
      !
      ! --- x2 direction ---------------------------------------------------------------------------
      !
      ! --- Left and right side of the cell ---
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_rho + rho(i1,i2,i3)*delta1_v1) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_rho + rho(i1,i2,i3)*delta2_v2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_rho + rho(i1,i2,i3)*delta3_v3)
      rho_l2(i2,i1,i3)=rho_l2(i2,i1,i3) + delta
      rho_r2(i2,i1,i3)=rho_r2(i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3) + delta1_P/rho(i1,i2,i3) - &
                       delta1_phi) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta2_Bc2/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta2_Bc1/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3))
      v1_l2( i2,i1,i3)=v1_l2( i2,i1,i3) + delta
      v1_r2( i2,i1,i3)=v1_r2( i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta2_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta2_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta2_Bc3/rho(i1,i2,i3) + delta2_P/rho(i1,i2,i3) - &
                       delta2_phi) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3))
      v2_l2( i2,i1,i3)=v2_l2( i2,i1,i3) + delta
      v2_r2( i2,i1,i3)=v2_r2( i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta2_Bc2/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta2_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3) + delta3_P/rho(i1,i2,i3) - &
                       delta3_phi)
      v3_l2( i2,i1,i3)=v3_l2( i2,i1,i3) + delta
      v3_r2( i2,i1,i3)=v3_r2( i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_Bc1) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (Bc1(i1,i2,i3)*delta2_v2 - Bc2(i1,i2,i3)*delta2_v1 + &
                       v2(i1,i2,i3)*delta2_Bc1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc1(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v1 + &
                       v3(i1,i2,i3)*delta3_Bc1)
      Bc1_l2(i2,i1,i3)=Bc1_l2(i2,i1,i3) + delta
      Bc1_r2(i2,i1,i3)=Bc1_r2(i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (Bc2(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v2 + &
                       v1(i1,i2,i3)*delta1_Bc2) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_Bc2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc2(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v2 + &
                       v3(i1,i2,i3)*delta3_Bc2)
      Bc2_l2(i2,i1,i3)=Bc2_l2(i2,i1,i3) + delta
      Bc2_r2(i2,i1,i3)=Bc2_r2(i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (Bc3(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v3 + &
                       v1(i1,i2,i3)*delta1_Bc3) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (Bc3(i1,i2,i3)*delta2_v2 - Bc2(i1,i2,i3)*delta2_v3 + &
                       v2(i1,i2,i3)*delta2_Bc3) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_Bc3)
      Bc3_l2(i2,i1,i3)=Bc3_l2(i2,i1,i3) + delta
      Bc3_r2(i2,i1,i3)=Bc3_r2(i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_P) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta2_v2 + &
                       v2(i1,i2,i3)*delta2_P) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_P)
      P_l2(  i2,i1,i3)=P_l2(  i2,i1,i3) + delta
      P_r2(  i2,i1,i3)=P_r2(  i2,i1,i3) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_rhoei) - &
                                               0.5*dtimeoverdx2(i2) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta2_v2 + &
                       v2(i1,i2,i3)*delta2_rhoei) - &
                                               0.5*dtimeoverdx3(i3) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_rhoei)
      rhoei_l2(i2,i1,i3)=rhoei_l2(i2,i1,i3) + delta
      rhoei_r2(i2,i1,i3)=rhoei_r2(i2,i1,i3) + delta
      !
      ! --- x3 direction ---------------------------------------------------------------------------
      !
      ! --- Left side and right of the cell ---
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_rho + rho(i1,i2,i3)*delta1_v1) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_rho + rho(i1,i2,i3)*delta2_v2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_rho + rho(i1,i2,i3)*delta3_v3)
      rho_l3(i3,i2,i1)=rho_l3(i3,i2,i1) + delta
      rho_r3(i3,i2,i1)=rho_r3(i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3) + delta1_P/rho(i1,i2,i3) - &
                       delta1_phi) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta2_Bc2/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta2_Bc1/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v1 - f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3))
      v1_l3( i3,i2,i1)=v1_l3( i3,i2,i1) + delta
      v1_r3( i3,i2,i1)=v1_r3( i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc2/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta2_Bc2/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta2_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta2_Bc3/rho(i1,i2,i3) + delta2_P/rho(i1,i2,i3) - &
                       delta2_phi) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v2 - f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3))
      v2_l3( i3,i2,i1)=v2_l3( i3,i2,i1) + delta
      v2_r3( i3,i2,i1)=v2_r3( i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta1_Bc1/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta1_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta2_Bc2/rho(i1,i2,i3) - &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta2_Bc3/rho(i1,i2,i3)) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_v3 - f(i1,i2,i3)*Bc3(i1,i2,i3)*delta3_Bc3/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc1(i1,i2,i3)*delta3_Bc1/rho(i1,i2,i3) + &
                       f(i1,i2,i3)*Bc2(i1,i2,i3)*delta3_Bc2/rho(i1,i2,i3) + delta3_P/rho(i1,i2,i3) - &
                       delta3_phi)
      v3_l3( i3,i2,i1)=v3_l3( i3,i2,i1) + delta
      v3_r3( i3,i2,i1)=v3_r3( i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (v1(i1,i2,i3)*delta1_Bc1) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (Bc1(i1,i2,i3)*delta2_v2 - Bc2(i1,i2,i3)*delta2_v1 + &
                       v2(i1,i2,i3)*delta2_Bc1) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc1(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v1 + &
                       v3(i1,i2,i3)*delta3_Bc1)
      Bc1_l3(i3,i2,i1)=Bc1_l3(i3,i2,i1) + delta
      Bc1_r3(i3,i2,i1)=Bc1_r3(i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (Bc2(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v2 + &
                       v1(i1,i2,i3)*delta1_Bc2) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (v2(i1,i2,i3)*delta2_Bc2) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Bc2(i1,i2,i3)*delta3_v3 - Bc3(i1,i2,i3)*delta3_v2 + &
                       v3(i1,i2,i3)*delta3_Bc2)
      Bc2_l3(i3,i2,i1)=Bc2_l3(i3,i2,i1) + delta
      Bc2_r3(i3,i2,i1)=Bc2_r3(i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (Bc3(i1,i2,i3)*delta1_v1 - Bc1(i1,i2,i3)*delta1_v3 + &
                       v1(i1,i2,i3)*delta1_Bc3) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (Bc3(i1,i2,i3)*delta2_v2 - Bc2(i1,i2,i3)*delta2_v3 + &
                       v2(i1,i2,i3)*delta2_Bc3) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (v3(i1,i2,i3)*delta3_Bc3)
      Bc3_l3(i3,i2,i1)=Bc3_l3(i3,i2,i1) + delta
      Bc3_r3(i3,i2,i1)=Bc3_r3(i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_P) - &
                                          0.5*dtimeoverdx2(i2) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta2_v2 + &
                       v2(i1,i2,i3)*delta2_P) - &
                                          0.5*dtimeoverdx3(i3) * &
                      (Gamma_1(i1,i2,i3)*P(i1,i2,i3)*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_P)
      P_l3(  i3,i2,i1)=P_l3(  i3,i2,i1) + delta
      P_r3(  i3,i2,i1)=P_r3(  i3,i2,i1) + delta
      !
      delta           = - 0.5*dtimeoverdx1(i1) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta1_v1 + &
                       v1(i1,i2,i3)*delta1_rhoei) - &
                                               0.5*dtimeoverdx2(i2) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta2_v2 + &
                       v2(i1,i2,i3)*delta2_rhoei) - &
                                               0.5*dtimeoverdx3(i3) * &
                       ((rho(i1,i2,i3)*(ei(i1,i2,i3)-eiTable_min)+P(i1,i2,i3))*delta3_v3 + &
                       v3(i1,i2,i3)*delta3_rhoei)
      rhoei_l3( i3,i2,i1)=rhoei_l3(i3,i2,i1) + delta
      rhoei_r3( i3,i2,i1)=rhoei_r3(i3,i2,i1) + delta
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_Hancock3D


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_CheckFlux(m1,n1, m2,n2, m3,n3, m2i, n2i, m3i, n3i, &
                             nghost1, nghost2, nghost3, &
                             hdScheme, &
                             bound_l, bound_r, &
                             dtime, xb1, eiTable_min, &
                             rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                             rhov1, rhov2, rhov3, rhoeikm, &
                             Gamma_1, energy_switch, f, &
                             phic_grav, phib_grav, lambda_til_min, lambda_til_max, &
                             flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                             flux_Bc1, flux_Bc2, flux_Bc3, &
                             flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                             flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                             sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                             sourceBc1_r, sourceBc2_r, sourceBc3_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_CheckFlux ('rhd_magneto-hydrodynamics_Check_Flux')
!
! PURPOSE:
!   Check if fluxes are save and replace by 1st order HLL-fluxes if necessary.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_CheckFlux(m1,n1, m2,n2, m3,n3, m2i, n2i, m3i, n3i, &
!                          nghost1, nghost2, nghost3, &
!                          hdScheme, &
!                          bound_l, bound_r, &
!                          dtime, xb1, eiTable_min, &
!                          rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
!                          rhov1, rhov2, rhov3, rhoeikm, &
!                          Gamma_1, energy_switch, f, &
!                          phic_grav, phib_grav, lambda_til_min, lambda_til_max, &
!                          flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
!                          flux_Bc1, flux_Bc2, flux_Bc3, &
!                          flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
!                          flux_rhov1B, flux_rhov2B, flux_rhov3B, &
!                          sourceBc1_l, sourceBc2_l, sourceBc3_l, &
!                          sourceBc1_r, sourceBc2_r, sourceBc3_r)
!
! INPUT:
!   m1:           ('m_1') integer, lower array index, 1st dimension
!   n1:           ('n_1') integer, upper array index, 1st dimension
!   m2:           ('m_2') integer, lower array index, 2nd dimension
!   n2:           ('n_2') integer, upper array index, 2nd dimension
!   m3:           ('m_3') integer, lower array index, 3rd dimension
!   n3:           ('n_3') integer, upper array index, 3rd dimension
!   m2i:          ('m_2') integer, lower loop index, 2nd dimension
!   n2i:          ('n_2') integer, upper loop index, 2nd dimension
!   m3i:          ('m_3') integer, lower loop index, 3rd dimension
!   n3i:          ('n_3') integer, upper loop index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   hdScheme:     ('hydrodynamics_Scheme') character, hydrodynamics scheme
!   bound_l:      ('boundary_left_1') integer, left boundary type for dimension 1
!                   values (constants defined in rhd_action_module):
!                     bound_None, bound_reflective, bound_periodic, bound_constant,
!                     bound_closed, bound_inoutflow, bound_closedbottom,
!                     bound_transmitting, bound_transmitting2, bound_transmitting3
!   bound_r:      ('boundary_right_1') integer, right boundary type for dimension 1
!                   values (constants defined in rhd_action_module):
!                     bound_None, bound_reflective, bound_periodic, bound_constant,
!                     bound_closed, bound_inoutflow, bound_closedbottom,
!                     bound_transmitting, bound_transmitting2, bound_transmitting3
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   eiTable_min:  ('energy_internal_Table_minimum') real with minimum allowed internal-energy value
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   va:           ('velocity_alfven') real 3D array, Alfven velocity on model grid
!   vf:           ('velocity_fast') real 3D array, fast magnetosonic speed on model grid
!   vs:           ('velocity_slow') real 3D array, slow magnetosonic speed on model grid
!   phic_grav:    ('phi_center_gravitation') real, dimension(:,:,:), cell centered grav. potential
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!
! INPUT/OUTPUT:
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_Bc1:     ('flux_B_center_1') real 3D array
!   flux_Bc2:     ('flux_B_center_2') real 3D array
!   flux_Bc3:     ('flux_B_center_3') real 3D array
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   sourceBc1_l:  ('source_B_center_1_left') real 3D array, Janhunen source term
!   sourceBc2_l:  ('source_B_center_2_left') real 3D array, Janhunen source term
!   sourceBc3_l:  ('source_B_center_3_left') real 3D array, Janhunen source term
!   sourceBc1_r:  ('source_B_center_1_right') real 3D array, Janhunen source term
!   sourceBc2_r:  ('source_B_center_2_right') real 3D array, Janhunen source term
!   sourceBc3_r:  ('source_B_center_3_right') real 3D array, Janhunen source term
!
! VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   No gravity corrections
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2005-11-08 (W.S.) Put into extra subroutine
!   2008-12-06 (W.S. East Lansing) Parameter energy_switch added
!   2010-04-10 (W.S. Freiburg) Several bugfixes
!   2010-06-11 (B.F. Lyon) Ignore mask_bound_l,r (consistent with usage in rhd_mhd_HLL1Dflux)
!   2011-01-31 (W.S. Exeter) New parameter hdScheme
!   2011-06-24 (W.S. Exeter) New parameters nghost2, nghost3 for use in unsplit Hancock scheme
!   2013-03-05 (B.F.) m2i, n2i, m3i, n3i, eiTable_min become input parameter
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, m2i, n2i, m3i, n3i, &
                                           nghost1, nghost2, nghost3, &
                                           hdScheme, &
                                           bound_l, bound_r
real,                      intent(in)   :: dtime, eiTable_min
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, rhov1, rhov2, rhov3, rhoeikm, &
                                           Gamma_1, f, &
                                           phic_grav
logical, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: energy_switch
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: phib_grav
real, dimension(m1        :n1+1      ,&
                m2        :n2        ,&
                m3        :n3      ), &
                           intent(in)   :: lambda_til_min, lambda_til_max
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3    ),intent(inout)::flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                          flux_Bc1, flux_Bc2, flux_Bc3, &
                                          flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                                          flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                                          sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                                          sourceBc1_r, sourceBc2_r, sourceBc3_r
!
! --- Local variables ---
integer                                 :: i1, i2, i3, n_HLL
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3    )              :: rho_til, P_til, &
                                           Bc1_sq_til, B_sq_til, &
                                           vf_til, &
                                           Gamma_1_til, &
                                           lambda_til_mi, lambda_til_pl
real                                    :: c1, c2, c3, ei_til, f_til, &
                                           v_sq_til, B_sq_tan, sqrt_2, cs_sq, &
                                           dpdrho, dpde, T, dTde, smin_l, smin_r, &
                                           csq, vaq, caq, dd, dx_l, dx_r, dx1, &
                                           source_rhov1, source_eikm, lambda0, &
                                           delta_Bc1, delta_Bc2, delta_Bc3, &
                                           flux_rho_l, flux_rhov1_l, flux_rhov2_l, flux_rhov3_l, &
                                           flux_Bc2_l, flux_Bc3_l, flux_rhoeikm_l, flux_rhoei_l, &
                                           flux_rhoek_l, flux_rhoem_l, &
                                           flux_rho_r, flux_rhov1_r, flux_rhov2_r, flux_rhov3_r, &
                                           flux_Bc2_r, flux_Bc3_r, flux_rhoeikm_r, flux_rhoei_r, &
                                           flux_rhoek_r, flux_rhoem_r, &
                                           flux_rhov1B_l, flux_rhov2B_l, flux_rhov3B_l, &
                                           flux_rhov1B_r, flux_rhov2B_r, flux_rhov3B_r, &
                                           rho_m, rhov1_m, rhov2_m, rhov3_m, v1_m, v2_m, v3_m, &
                                           rhoout_l, rhoout_r, v1out, v2out, v3out, &
                                           Bc1out, Bc2out, Bc3out, eiout_l, eiout_r
real, parameter                         :: epsilon=0.000001, beta0=0.001
!---------------------------------------------------------------------------------------------------
!
!call tabinter_getinfo(x2min=eiTable_min)
!n_HLL=0
!
do i3=m3i,n3i
  do i2=m2i,n2i
    !DIR$ SIMD
    do i1=m1,n1+1
      !
      ! --- flux for left state ---
      flux_rho_l   = rhov1(i1-1,i2,i3)
      flux_rhov1_l = rhov1(i1-1,i2,i3)*v1(i1-1,i2,i3) + P(i1-1,i2,i3)
      flux_rhov2_l = rhov2(i1-1,i2,i3)*v1(i1-1,i2,i3)
      flux_rhov3_l = rhov3(i1-1,i2,i3)*v1(i1-1,i2,i3)
      flux_rhov1B_l = 0.5*(Bc2(i1-1,i2,i3)**2+Bc3(i1-1,i2,i3)**2-Bc1(i1-1,i2,i3)**2)
      flux_rhov2B_l = -Bc1(i1-1,i2,i3)*Bc2(i1-1,i2,i3)
      flux_rhov3B_l = -Bc1(i1-1,i2,i3)*Bc3(i1-1,i2,i3)
      flux_Bc2_l   = Bc2(i1-1,i2,i3)*v1(i1-1,i2,i3)-Bc1(i1-1,i2,i3)*v2(i1-1,i2,i3)
      flux_Bc3_l   = Bc3(i1-1,i2,i3)*v1(i1-1,i2,i3)-Bc1(i1-1,i2,i3)*v3(i1-1,i2,i3)
      flux_rhoeikm_l = (rhoeikm(i1-1,i2,i3) + P(i1-1,i2,i3) + 0.5*(Bc1(i1-1,i2,i3)**2 + &
                            Bc2(i1-1,i2,i3)**2+Bc3(i1-1,i2,i3)**2))*v1(i1-1,i2,i3) - &
                           (Bc1(i1-1,i2,i3)*v1(i1-1,i2,i3)+Bc2(i1-1,i2,i3)*v2(i1-1,i2,i3) + &
                            Bc3(i1-1,i2,i3)*v3(i1-1,i2,i3))*Bc1(i1-1,i2,i3)
      flux_rhoei_l = rhov1(i1-1,i2,i3)*ei(i1-1,i2,i3)
      flux_rhoek_l = rhov1(i1-1,i2,i3)*0.5*(v1(i1-1,i2,i3)**2+v2(i1-1,i2,i3)**2+v3(i1-1,i2,i3)**2)
      flux_rhoem_l = v1(i1-1,i2,i3)*(Bc1(i1-1,i2,i3)**2+Bc2(i1-1,i2,i3)**2+Bc3(i1-1,i2,i3)**2) - &
                     Bc1(i1-1,i2,i3)*(Bc1(i1-1,i2,i3)*v1(i1-1,i2,i3)+ &
                     Bc2(i1-1,i2,i3)*v2(i1-1,i2,i3)+Bc3(i1-1,i2,i3)*v3(i1-1,i2,i3))
      !
      ! --- flux for right state ---
      flux_rho_r   = rhov1(i1  ,i2,i3)
      flux_rhov1_r = rhov1(i1  ,i2,i3)*v1(i1  ,i2,i3) + P(i1  ,i2,i3)
      flux_rhov2_r = rhov2(i1  ,i2,i3)*v1(i1  ,i2,i3)
      flux_rhov3_r = rhov3(i1  ,i2,i3)*v1(i1  ,i2,i3)
      flux_rhov1B_r = 0.5*(Bc2(i1  ,i2,i3)**2+Bc3(i1  ,i2,i3)**2-Bc1(i1  ,i2,i3)**2)
      flux_rhov2B_r = -Bc1(i1  ,i2,i3)*Bc2(i1  ,i2,i3)
      flux_rhov3B_r = -Bc1(i1  ,i2,i3)*Bc3(i1  ,i2,i3)
      flux_Bc2_r   = Bc2(i1  ,i2,i3)*v1(i1  ,i2,i3)-Bc1(i1  ,i2,i3)*v2(i1  ,i2,i3)
      flux_Bc3_r   = Bc3(i1  ,i2,i3)*v1(i1  ,i2,i3)-Bc1(i1  ,i2,i3)*v3(i1  ,i2,i3)
      flux_rhoeikm_r = (rhoeikm(i1  ,i2,i3) + P(i1  ,i2,i3) + 0.5*(Bc1(i1  ,i2,i3)**2 + &
                            Bc2(i1  ,i2,i3)**2+Bc3(i1  ,i2,i3)**2))*v1(i1  ,i2,i3) - &
                           (Bc1(i1  ,i2,i3)*v1(i1  ,i2,i3)+Bc2(i1  ,i2,i3)*v2(i1  ,i2,i3) + &
                            Bc3(i1  ,i2,i3)*v3(i1  ,i2,i3))*Bc1(i1  ,i2,i3)
      flux_rhoei_r = rhov1(i1  ,i2,i3)*ei(i1  ,i2,i3)
      flux_rhoek_r = rhov1(i1  ,i2,i3)*0.5*(v1(i1  ,i2,i3)**2+v2(i1  ,i2,i3)**2+v3(i1  ,i2,i3)**2)
      flux_rhoem_r = v1(i1  ,i2,i3)*(Bc1(i1  ,i2,i3)**2+Bc2(i1  ,i2,i3)**2+Bc3(i1  ,i2,i3)**2) - &
                     Bc1(i1  ,i2,i3)*(Bc1(i1  ,i2,i3)*v1(i1  ,i2,i3)+ &
                     Bc2(i1  ,i2,i3)*v2(i1  ,i2,i3)+Bc3(i1  ,i2,i3)*v3(i1  ,i2,i3))
      !
      ! --- Size of left virtual cell ---
      dx1 = max(0.1*(xb1(i1)-xb1(i1-1)), -dtime*lambda_til_min(i1,i2,i3))
      dx1 = min(dx1, 0.4*(xb1(i1)-xb1(i1-1)))
      dx_l = dtime * (min(lambda_til_max(i1,i2,i3), 0.0) - min(lambda_til_min(i1,i2,i3), 0.0))
      source_rhov1=rho(i1-1,i2,i3)*(phib_grav(i1,i2,i3)-phic_grav(i1-1,i2,i3)) / dx1
      source_eikm=flux_rho(i1,i2,i3)*(phib_grav(i1,i2,i3)-phic_grav(i1-1,i2,i3)) / dx1
      !
      ! --- Update of left virtual cell ---
      rhoout_l=rho(i1-1,i2,i3) - dtime / dx1 * (flux_rho(  i1,i2,i3)-flux_rho_l)
      v1out =(rho(i1-1,i2,i3) * v1(i1-1,i2,i3) - &
                             dtime / dx1 * ((flux_rhov1(i1,i2,i3)-flux_rhov1_l) + &
                             f(i1,i2,i3) * (flux_rhov1B(i1,i2,i3)-flux_rhov1B_l)) + &
                             dtime * source_rhov1 &
             )/rhoout_l
      v2out =(rho(i1-1,i2,i3) * v2(i1-1,i2,i3) - &
                             dtime / dx1 * ((flux_rhov2(i1,i2,i3)-flux_rhov2_l) + &
                             f(i1,i2,i3) * (flux_rhov2B(i1,i2,i3)-flux_rhov2B_l)) &
             )/rhoout_l
      v3out =(rho(i1-1,i2,i3) * v3(i1-1,i2,i3) - &
                             dtime / dx1 * ((flux_rhov3(i1,i2,i3)-flux_rhov3_l) + &
                             f(i1,i2,i3) * (flux_rhov3B(i1,i2,i3)-flux_rhov3B_l)) &
             )/rhoout_l
      Bc1out=Bc1(i1-1,i2,i3) - dtime / dx1 *  flux_Bc2(  i1,i2,i3)             + &
             sourceBc1_l(i1,i2,i3) * (xb1(i1) - xb1(i1-1)) / dx1
      Bc2out=Bc2(i1-1,i2,i3) - dtime / dx1 * (flux_Bc2(  i1,i2,i3)-flux_Bc2_l) + &
             sourceBc2_l(i1,i2,i3) * (xb1(i1) - xb1(i1-1)) / dx1
      Bc3out=Bc3(i1-1,i2,i3) - dtime / dx1 * (flux_Bc3(  i1,i2,i3)-flux_Bc3_l) + &
             sourceBc3_l(i1,i2,i3) * (xb1(i1) - xb1(i1-1)) / dx1

      eiout_l =(rho(i1-1,i2,i3) * ei(i1-1,i2,i3) + 0.5 * rho(i1-1,i2,i3) * (v1(i1-1,i2,i3)**2 + &
                             v2(i1-1,i2,i3)**2 + v3(i1-1,i2,i3)**2) - &
                             dtime / dx1 * (flux_rhoeikm(i1,i2,i3)-flux_rhoeikm_l) + &
                                             0.5 * (Bc1(i1-1,i2,i3) + Bc1out) * &
                            (dtime / dx1 *  flux_Bc1(  i1,i2,i3)             - &
                                             sourceBc1_l(i1,i2,i3) * dx_l / dx1) + &
                                             0.5 * (Bc2(i1-1,i2,i3) + Bc2out) * &
                            (dtime / dx1 * (flux_Bc2(  i1,i2,i3)-flux_Bc2_l) - &
                                             sourceBc2_l(i1,i2,i3) * dx_l / dx1) + &
                                             0.5 * (Bc3(i1  ,i2,i3) + Bc3out) * &
                            (dtime / dx1 * (flux_Bc3(  i1,i2,i3)-flux_Bc3_l) - &
                                             sourceBc3_l(i1,i2,i3) * dx_l / dx1) + &
                             dtime * source_eikm &
                                             )/rhoout_l - 0.5 * (v1out**2 + v2out**2 + v3out**2)
      !
      ! --- Size of right virtual cell ---
      dx1 = max(0.1*(xb1(i1+1)-xb1(i1)), dtime*lambda_til_max(i1,i2,i3))
      dx1 = min(dx1, 0.4*(xb1(i1+1)-xb1(i1)))
      dx_r = dtime * (max(lambda_til_max(i1,i2,i3), 0.0) - max(lambda_til_min(i1,i2,i3), 0.0))
      source_rhov1=rho(i1  ,i2,i3)*(phic_grav(i1,i2,i3)-phib_grav(i1  ,i2,i3)) / dx1
      source_eikm=flux_rho(i1,i2,i3)*(phic_grav(i1,i2,i3)-phib_grav(i1  ,i2,i3)) / dx1
      !
      ! --- Update of right virtual cell ---
      rhoout_r=rho(i1  ,i2,i3) + dtime / dx1 * (flux_rho(  i1,i2,i3)-flux_rho_r)
      v1out =(rho(i1  ,i2,i3) * v1(i1  ,i2,i3) + &
                             dtime / dx1 * ((flux_rhov1(i1,i2,i3)-flux_rhov1_r) + &
                             f(i1,i2,i3) * (flux_rhov1B(i1,i2,i3)-flux_rhov1B_r)) + &
                             dtime * source_rhov1 &
             )/rhoout_r
      v2out =(rho(i1  ,i2,i3) * v2(i1  ,i2,i3) + &
                             dtime / dx1 * ((flux_rhov2(i1,i2,i3)-flux_rhov2_r) + &
                             f(i1,i2,i3) * (flux_rhov2B(i1,i2,i3)-flux_rhov2B_r)) &
             )/rhoout_r
      v3out =(rho(i1  ,i2,i3) * v3(i1  ,i2,i3) + &
                             dtime / dx1 * ((flux_rhov3(i1,i2,i3)-flux_rhov3_r) + &
                             f(i1,i2,i3) * (flux_rhov3B(i1,i2,i3)-flux_rhov3B_r)) &
             )/rhoout_r
      Bc1out=Bc1(i1-1,i2,i3) + dtime / dx1 *  flux_Bc1(  i1,i2,i3)             + &
             sourceBc1_r(i1,i2,i3) * (xb1(i1+1) - xb1(i1)) / dx1
      Bc2out=Bc2(i1  ,i2,i3) + dtime / dx1 * (flux_Bc2(  i1,i2,i3)-flux_Bc2_r) + &
             sourceBc2_r(i1,i2,i3) * (xb1(i1+1) - xb1(i1)) / dx1
      Bc3out=Bc3(i1  ,i2,i3) + dtime / dx1 * (flux_Bc3(  i1,i2,i3)-flux_Bc3_r) + &
             sourceBc3_r(i1,i2,i3) * (xb1(i1+1) - xb1(i1)) / dx1
      !
      eiout_r =(rho(i1  ,i2,i3) * ei(i1  ,i2,i3) + 0.5 * rho(i1  ,i2,i3) * (v1(i1  ,i2,i3)**2 + &
                             v2(i1  ,i2,i3)**2 + v3(i1  ,i2,i3)**2) + &
                             dtime / dx1 * (flux_rhoeikm(i1,i2,i3)-flux_rhoeikm_r) - &
                                             0.5 * (Bc1(i1  ,i2,i3) + Bc1out) * &
                            (dtime / dx1 *  flux_Bc1(  i1,i2,i3)             + &
                                             sourceBc1_r(i1,i2,i3) * dx_r / dx1) - &
                                             0.5 * (Bc2(i1  ,i2,i3) + Bc2out) * &
                            (dtime / dx1 * (flux_Bc2(  i1,i2,i3)-flux_Bc2_r) + &
                                             sourceBc2_r(i1,i2,i3) * dx_r / dx1) - &
                                             0.5 * (Bc3(i1  ,i2,i3) + Bc3out) * &
                            (dtime / dx1 * (flux_Bc3(  i1,i2,i3)-flux_Bc3_r) + &
                                             sourceBc3_r(i1,i2,i3) * dx_r / dx1) + &
                             dtime * source_eikm &
                                             )/rhoout_r - 0.5 * (v1out**2 + v2out**2 + v3out**2)
      !
      if ((rhoout_l < 0.0 .or. rhoout_r < 0.0                              ) .or. &
          ((eiout_l < eiTable_min     .or. eiout_r  < eiTable_min ) .and. &
           (energy_switch(i1-1,i2,i3) .or. energy_switch(i1,i2,i3))        )       ) then
        !
        ! --- if states of virtual cells are unphysical,    ---
        ! --- replace original flux by first order HLL flux ---
        !
        !n_HLL = n_HLL + 1
        !
        rho_til( i1,i2,i3)=min(rho(i1-1,i2,i3),rho(i1,i2,i3))
        Bc1_sq_til(i1,i2,i3)=max(Bc1(i1-1,i2,i3)**2,Bc1(i1,i2,i3)**2)
        B_sq_til(i1,i2,i3)=max(Bc1(i1-1,i2,i3)**2+Bc2(i1-1,i2,i3)**2+Bc3(i1-1,i2,i3)**2, &
                               Bc1(i1  ,i2,i3)**2+Bc2(i1  ,i2,i3)**2+Bc3(i1  ,i2,i3)**2)
        !
        P_til(  i1,i2,i3)=max(P(i1-1,i2,i3),P(i1,i2,i3))
        Gamma_1_til(i1,i2,i3)=max(Gamma_1(i1-1,i2,i3),Gamma_1(i1,i2,i3))
        f_til=0.5*(f(i1-1,i2,i3)+f(i1,i2,i3))
        !
        ! === Boundary conditions III ===
        !P_til(i1,i2,i3)=(1.0-mask_bound_l(i1)-mask_bound_r(i1)) * P_til(i1  ,i2,i3) + &
        !                                      mask_bound_l(i1)  * P(    i1  ,i2,i3) + &
        !                                      mask_bound_r(i1)  * P(    i1-1,i2,i3)
        !
        ! --- Sound speed, Alfven speed, fast magnetosonic speed ---
        csq=P_til(i1,i2,i3)/rho_til(i1,i2,i3)*Gamma_1_til(i1,i2,i3)
        vaq=f_til*Bc1_sq_til(i1,i2,i3)/rho_til(i1,i2,i3)
        caq=csq+f_til*B_sq_til(i1,i2,i3)/rho_til(i1,i2,i3)
        dd =caq*sqrt(max(0.0,1.0 - 4.0*(csq/caq)*(vaq/caq)))
        vf_til(    i1,i2,i3)=sqrt(0.5*(caq+dd))
        !
        ! === Characteristic velocities at cell boundaries ===
        !
        ! --- Characteristic velocities ---
        lambda_til_mi(  i1,i2,i3)=min(v1(i1-1,i2,i3),v1(i1,i2,i3))-vf_til(i1,i2,i3)
        lambda_til_pl(  i1,i2,i3)=max(v1(i1-1,i2,i3),v1(i1,i2,i3))+vf_til(i1,i2,i3)
        !
        if (hdScheme == hdScheme_CenMHD) then
          !
          ! --- For Central scheme, use only maximum characteristic velocity ---
          lambda0                =max(abs(lambda_til_pl(i1,i2,i3)),abs(lambda_til_mi(i1,i2,i3)))
          lambda_til_mi(i1,i2,i3)=-lambda0
          lambda_til_pl(i1,i2,i3)= lambda0
          !
        endif
        !
        ! --- Construct Janhunen source terms ---
        ! --- Eq. (7) for the HLL middle state ---
        rho_m   = (lambda_til_pl(i1,i2,i3)*rho(i1  ,i2,i3) - &
                   lambda_til_mi(i1,i2,i3)*rho(i1-1,i2,i3) + &
                   flux_rho_l - flux_rho_r) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        rhov1_m = (lambda_til_pl(i1,i2,i3)*rhov1(i1  ,i2,i3) - &
                   lambda_til_mi(i1,i2,i3)*rhov1(i1-1,i2,i3) + &
                   flux_rhov1_l + f(i1  ,i2,i3)*flux_rhov1B_l - &
                   flux_rhov1_r - f(i1-1,i2,i3)*flux_rhov1B_r) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        rhov2_m = (lambda_til_pl(i1,i2,i3)*rhov2(i1  ,i2,i3) - &
                   lambda_til_mi(i1,i2,i3)*rhov2(i1-1,i2,i3) + &
                   flux_rhov2_l + f(i1  ,i2,i3)*flux_rhov2B_l - &
                   flux_rhov2_r - f(i1-1,i2,i3)*flux_rhov2B_r) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        rhov3_m = (lambda_til_pl(i1,i2,i3)*rhov3(i1  ,i2,i3) - &
                   lambda_til_mi(i1,i2,i3)*rhov3(i1-1,i2,i3) + &
                   flux_rhov3_l + f(i1  ,i2,i3)*flux_rhov3B_l - &
                   flux_rhov3_r - f(i1-1,i2,i3)*flux_rhov3B_r) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        ! --- Text after Eq. (26) ---
        v1_m = rhov1_m / rho_m
        v2_m = rhov2_m / rho_m
        v3_m = rhov3_m / rho_m
        ! --- Eq. (26) ---
        delta_Bc1 = -v1_m * (Bc1(i1,i2,i3) - Bc1(i1-1,i2,i3)) / &
                    (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        delta_Bc2 = -v2_m * (Bc1(i1,i2,i3) - Bc1(i1-1,i2,i3)) / &
                    (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        delta_Bc3 = -v3_m * (Bc1(i1,i2,i3) - Bc1(i1-1,i2,i3)) / &
                    (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        ! --- Text after Eq. (14) ---
        dx_l = dtime * (min(lambda_til_pl(i1,i2,i3), 0.0) - min(lambda_til_mi(i1,i2,i3), 0.0))
        ! --- Text after Eq. (15) ---
        dx_r = dtime * (max(lambda_til_pl(i1,i2,i3), 0.0) - max(lambda_til_mi(i1,i2,i3), 0.0))
        ! --- Eq. (14) ---
        sourceBc1_l(i1,i2,i3)=delta_Bc1 * dx_l / (xb1(i1  ) - xb1(i1-1))
        sourceBc2_l(i1,i2,i3)=delta_Bc2 * dx_l / (xb1(i1  ) - xb1(i1-1))
        sourceBc3_l(i1,i2,i3)=delta_Bc3 * dx_l / (xb1(i1  ) - xb1(i1-1))
        ! --- Eq. (15) ---
        sourceBc1_r(i1,i2,i3)=delta_Bc1 * dx_r / (xb1(i1+1) - xb1(i1  ))
        sourceBc2_r(i1,i2,i3)=delta_Bc2 * dx_r / (xb1(i1+1) - xb1(i1  ))
        sourceBc3_r(i1,i2,i3)=delta_Bc3 * dx_r / (xb1(i1+1) - xb1(i1  ))
        !
        lambda_til_mi(i1,i2,i3)=min(lambda_til_mi(i1,i2,i3), 0.0)
        lambda_til_pl(i1,i2,i3)=max(lambda_til_pl(i1,i2,i3), 0.0)
        !
        c1=lambda_til_pl(i1,i2,i3)/(lambda_til_pl(i1,i2,i3)-lambda_til_mi(i1,i2,i3))
        c2=lambda_til_mi(i1,i2,i3)/(lambda_til_pl(i1,i2,i3)-lambda_til_mi(i1,i2,i3))
        c3=lambda_til_pl(i1,i2,i3)*lambda_til_mi(i1,i2,i3)/(lambda_til_pl(i1,i2,i3) - &
           lambda_til_mi(i1,i2,i3))
        !
        ! --- Replace original flux by HLL-flux ---

        flux_rho(i1,i2,i3)=   c1 * flux_rho_l - c2 * flux_rho_r - &
                              c3*(rho(i1-1,i2,i3)-rho(i1  ,i2,i3))

        flux_rhov1(i1,i2,i3)= c1 * flux_rhov1_l - c2 * flux_rhov1_r - &
                              c3*(rhov1(i1-1,i2,i3) - rhov1(i1  ,i2,i3))

        flux_rhov2(i1,i2,i3)= c1 * flux_rhov2_l - c2 * flux_rhov2_r - &
                              c3*(rhov2(i1-1,i2,i3) - rhov2(i1  ,i2,i3))

        flux_rhov3(i1,i2,i3)= c1 * flux_rhov3_l - c2 * flux_rhov3_r - &
                              c3*(rhov3(i1-1,i2,i3) - rhov3(i1  ,i2,i3))

        flux_Bc1(i1,i2,i3)=  -c3*(Bc1(i1-1,i2,i3) - Bc1(i1  ,i2,i3))

        flux_Bc2(i1,i2,i3)=   c1 * flux_Bc2_l - c2 * flux_Bc2_r - &
                              c3*(Bc2(i1-1,i2,i3) - Bc2(i1  ,i2,i3))

        flux_Bc3(i1,i2,i3)=   c1 * flux_Bc3_l - c2 * flux_Bc3_r - &
                              c3*(Bc3(i1-1,i2,i3) - Bc3(i1  ,i2,i3))

        flux_rhoeikm(i1,i2,i3)= c1 * flux_rhoeikm_l - c2 * flux_rhoeikm_r - &
                              c3*(rhoeikm(i1-1,i2,i3) - rhoeikm(i1  ,i2,i3))

        flux_rhoei(i1,i2,i3) = c1 * flux_rhoei_l - c2 * flux_rhoei_r - &
                              c3 * (rho(i1-1,i2,i3) * ei(i1-1,i2,i3) - &
                                    rho(i1  ,i2,i3) * ei(i1  ,i2,i3))

        flux_rhoek(i1,i2,i3) = c1 * flux_rhoek_l - c2 * flux_rhoek_r - &
                              c3 * (rho(i1-1,i2,i3) * 0.5 * (v1(i1-1,i2,i3)**2+v2(i1-1,i2,i3)**2+ &
                                                             v3(i1-1,i2,i3)**2) - &
                                    rho(i1  ,i2,i3) * 0.5 * (v1(i1  ,i2,i3)**2+v2(i1  ,i2,i3)**2+ &
                                                             v3(i1  ,i2,i3)**2))

        flux_rhoem(i1,i2,i3) = c1 * flux_rhoem_l - c2 * flux_rhoem_r - &
                              c3 * (0.5 * (Bc1(i1-1,i2,i3)**2+Bc2(i1-1,i2,i3)**2+ &
                                           Bc3(i1-1,i2,i3)**2) - &
                                    0.5 * (Bc1(i1  ,i2,i3)**2+Bc2(i1  ,i2,i3)**2+ &
                                           Bc3(i1  ,i2,i3)**2))

        flux_rhov1B(i1,i2,i3)= c1 * flux_rhov1B_l - c2 * flux_rhov1B_r
        flux_rhov2B(i1,i2,i3)= c1 * flux_rhov2B_l - c2 * flux_rhov2B_r
        flux_rhov3B(i1,i2,i3)= c1 * flux_rhov3B_l - c2 * flux_rhov3B_r
        !
      endif
      !
     end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_mhd_CheckFlux


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_HLL1Dflux(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             hdScheme, &
                             bound_l, bound_r, reconstruction_order, reconstruction_type, dim, &
                             dtime, xb1, f, &
                             rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                             rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                             rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r, &
                             Gamma_1, energy_switch, &
                             phic_grav, phib_grav, &
                             flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                             flux_Bc1, flux_Bc2, flux_Bc3, &
                             flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                             flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                             sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                             sourceBc1_r, sourceBc2_r, sourceBc3_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_HLL1Dflux ('rhd_magneto-hydrodynamics_HLL_1D_flux')
!
! PURPOSE:
!   Compute 1D HLL fluxes.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_HLL1Dflux(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          hdScheme, &
!                          bound_l, bound_r, reconstruction_order, reconstruction_type, dim, &
!                          dtime, xb1, wl, f, &
!                          rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
!                          rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
!                          rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r, &
!                          Gamma_1, energy_switch, &
!                          phic_grav, phib_grav, &
!                          flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
!                          flux_Bc1, flux_Bc2, flux_Bc3, &
!                          flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
!                          flux_rhov1B, flux_rhov2B, flux_rhov3B, &
!                          sourceBc1_l, sourceBc2_l, sourceBc3_l, &
!                          sourceBc1_r, sourceBc2_r, sourceBc3_r)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   hdScheme:
!   bound_l:      ('boundary_left') integer, left boundary type for dimension 1
!                   values (constants defined in rhd_action_module):
!                     bound_None, bound_reflective, bound_periodic, bound_constant,
!                     bound_closed, bound_inoutflow, bound_closedbottom,
!                     bound_transmitting, bound_transmitting2, bound_transmitting3
!   bound_r:      ('boundary_right') integer, right boundary type for dimension 1
!                   values (constants defined in rhd_action_module):
!                     bound_None, bound_reflective, bound_periodic, bound_constant,
!                     bound_closed, bound_inoutflow, bound_closedbottom,
!                     bound_transmitting, bound_transmitting2, bound_transmitting3
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme;
!                    values:
!                      0: scheme, e.g., 'Constant'
!                      1: schemes, e.g., 'MinMod', 'VanLeer', 'SuperBee'
!                      2: scheme, e.g., 'PP'
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dim:          ('dimension') integer, actual dimension the first index of 3D arrays refers to
!   dtime:        ('delta_time') real, time step to be used
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   f:            ('factor') real, dimension(:,:,:), reduction factor in case of limited Alven speed
!   rho:          ('rho') real, dimension(:,:,:), density
!   ei:           ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:           ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:           ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:           ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   Bc1:          ('B_center_1'), dimension(:,:,:), cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2'), dimension(:,:,:), cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3'), dimension(:,:,:), cell centered magnetic field, 3. component
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   energy_switch: ('energy_switch') logical 3D array, switch for dual energy method
!   phic_grav:    ('phi_center_gravitation') real, dimension(:,:,:), cell centered grav. potential
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!
! OUTPUT:
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_Bc1:     ('flux_B_center_1') real 3D array, magnetic field 1 flux through celll boundary
!   flux_Bc2:     ('flux_B_center_2') real 3D array, magnetic field 2 flux through celll boundary
!   flux_Bc3:     ('flux_B_center_3') real 3D array, magnetic field 3 flux through celll boundary
!   flux_rhoeikm: ('flux_rho_energy_internal_kinetik_magnetic') real 3D array,
!                 total energy flux through cell bnd.
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, thermal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!   flux_rhov1B:  ('flux_rho_velocity_1_B') real 3D array, momentum 1 flux due to magnetic fields
!   flux_rhov2B:  ('flux_rho_velocity_2_B') real 3D array, momentum 2 flux due to magnetic fields
!   flux_rhov3B:  ('flux_rho_velocity_3_B') real 3D array, momentum 3 flux due to magnetic fields
!   sourceBc1_l:  ('source_B_center_1_left') real 3D array, Janhunen source term
!   sourceBc2_l:  ('source_B_center_2_left') real 3D array, Janhunen source term
!   sourceBc3_l:  ('source_B_center_3_left') real 3D array, Janhunen source term
!   sourceBc1_r:  ('source_B_center_1_right') real 3D array, Janhunen source term
!   sourceBc2_r:  ('source_B_center_2_right') real 3D array, Janhunen source term
!   sourceBc3_r:  ('source_B_center_3_right') real 3D array, Janhunen source term
!
! VARIABLES:
!
! ROUTINES:
!   rhd_mhd_CheckFlux:     Check if fluxes are save and replace by 1st order HLL fluxes if necessary
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   No gravity corrections
!
! PROCEDURE:
!   For Janhunen source terms, see \href{http://adsabs.harvard.edu/abs/2000JCoPh.160..649J}.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-05-06 (W.S.) First version
!   2004-11-03 (W.S.) Definitions of unused variables deleted
!   2005-04-29 (W.S. Freiburg) Extended to 2nd order
!   2006-09-05 (W.S. Freiburg) Boundary condition IV for closed boundaries
!   2008-03-24 (W.S. East Lansing) flux_rhoe for dual energy calculation
!   2008-12-06 (W.S. East Lansing) parameter energy_switch added
!   2010-06-09 (B.F. Lyon) Call of rhd_mhd_CheckFlux can be switched off
!   2010-06-14 (B.F.) Switch off Janhunen terms for unsplit update
!   2010-06-16 (B.F.) Switch on Janhunen terms again, correct inoutflow2
!   2010-06-30 (B.F.) hdScheme_CenMHD
!   2010-07-22 (B.F.) Switch on Janhunen terms again (now for real)
!   2011-02-25 (W.S. Exeter) Janhunen terms also for unsplit case
!   2011-06-17 (W.S.) Interface changed, Reconstruction and Hancock predictor step removed
!   2011-06-24 (W.S.) New parameters nghost2, nghost3, d2, d2 for use in unsplit Hancock scheme
!   2013-03-02 (B.F.) Rearrange loops in bndvelo step,
!                     remove Janhunen_flag, use closedbound_l/r_flag
!   2013-03-05 (B.F.) Integrate "Boundary conditions V" and call to CheckFlux into large loop
!   2013-03-06 (B.F.) Added compiler directive
!   2018-05-14 (B.F.) Add or change comments, remove ei_l and ei_r
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use tabinter_module
use gasinter_module
!#
# ifdef rhd_roe1d_flux_t01
!#
!use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                                           d2, d3, &
                                           hdScheme, &
                                           bound_l, bound_r, &
                                           reconstruction_order, reconstruction_type, dim
real,                      intent(in)   :: dtime
real, dimension(m1-nghost1  : &
                n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m1-1      :n1+1      ,&
                m2-d2     :n2+d2     ,&
                m3-d3     :n3+d3)    ,&
                           intent(inout):: rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, &
                                           P_l, rhoei_l, &
                                           rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, &
                                           P_r, rhoei_r
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                                           Gamma_1, phic_grav, f
logical, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: energy_switch
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: phib_grav
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3    ),intent(out)  :: flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_Bc1, flux_Bc2, flux_Bc3, &
                                           flux_rhoeikm, flux_rhoei, &
                                           flux_rhoek, flux_rhoem, fep, &
                                           flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                                           sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                                           sourceBc1_r, sourceBc2_r, sourceBc3_r
!
! --- Local variables ---
integer                                 :: i1, i2, i3, nb1, nt1, nb2, nt2, nb3, nt3, &
                                           m1_0, n1_0, m2_0, n2_0, m3_0, n3_0
logical                                 :: outerbound_l_flag , outerbound_r_flag, &
                                           closedbound_l_flag, closedbound_r_flag
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3)  :: rhov1, rhov2, rhov3, rhoeikm
real, dimension(m1-1 :n1+1 ,&
                m2-d2:n2+d2,&
                m3-d3:n3+d3)            :: rhov1_l, rhov2_l, rhov3_l, &
                                           rhoeikm_l, rhoek_l, rhoem_l, &
                                           rhov1_r, rhov2_r, rhov3_r, &
                                           rhoeikm_r, rhoek_r, rhoem_r
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3    )              :: rho_til, P_til, &
                                           Bc1_sq_til, B_sq_til, &
                                           v1_til, v2_til, v3_til, vf_til, &
                                           lambda_til_mi, lambda_til_pl
real                                    :: c1, c2, c3, d, eiTable_min, &
                                           csq, vaq, caq, dd, dx_l, dx_r, dx1, &
                                           Gamma_1_til, f_til, &
                                           flux_rho_l, flux_rhov1_l, flux_rhov2_l, flux_rhov3_l, &
                                           flux_Bc2_l, flux_Bc3_l, flux_rhoeikm_l, flux_rhoei_l, &
                                           flux_rhoek_l, flux_rhoem_l, &
                                           flux_rho_r, flux_rhov1_r, flux_rhov2_r, flux_rhov3_r, &
                                           flux_Bc2_r, flux_Bc3_r, flux_rhoeikm_r, flux_rhoei_r, &
                                           flux_rhoek_r, flux_rhoem_r, &
                                           flux_rhov1B_l, flux_rhov2B_l, flux_rhov3B_l, &
                                           flux_rhov1B_r, flux_rhov2B_r, flux_rhov3B_r, &
                                           delta_Bc1, delta_Bc2, delta_Bc3,  lambda0, &
                                           rho_m, rhov1_m, rhov2_m, rhov3_m, maxflux, delta, sigma
real, parameter                         :: epsilon = 0.0000001
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_roe1d_flux_t01
!#
!call timing_start('MHD: HLL1Dflux: fluxes')
!#
# endif
!#
call tabinter_getinfo(x2min=eiTable_min)
!
! \label{f90:n_magdiff__useMHD3}
! --- Set variables for diffusive boundary layers ---
if (dim == 1) then
  !
  ! --- Set dimensions for x1 sweep ---
  m1_0 = prop%m1
  n1_0 = prop%n1
  m2_0 = prop%m2
  n2_0 = prop%n2
  m3_0 = prop%m3
  n3_0 = prop%n3
  nb1 = par%N_magDiffSide
  nt1 = par%N_magDiffSide
  nb2 = par%N_magDiffSide
  nt2 = par%N_magDiffSide
  nb3 = par%N_magDiffBottom
  nt3 = par%N_magDiffTop
  !
else if (dim == 2) then
  !
  ! --- Set dimensions for x2 sweep ---
  m1_0 = prop%m2
  n1_0 = prop%n2
  m2_0 = prop%m1
  n2_0 = prop%n1
  m3_0 = prop%m3
  n3_0 = prop%n3
  nb1 = par%N_magDiffSide
  nt1 = par%N_magDiffSide
  nb2 = par%N_magDiffSide
  nt2 = par%N_magDiffSide
  nb3 = par%N_magDiffBottom
  nt3 = par%N_magDiffTop
  !
else
  !
  ! --- Set dimensions for x3 sweep ---
  m1_0 = prop%m3
  n1_0 = prop%n3
  m2_0 = prop%m2
  n2_0 = prop%n2
  m3_0 = prop%m1
  n3_0 = prop%n1
  nb1 = par%N_magDiffBottom
  nt1 = par%N_magDiffTop
  nb2 = par%N_magDiffSide
  nt2 = par%N_magDiffSide
  nb3 = par%N_magDiffSide
  nt3 = par%N_magDiffSide
  !
endif
!
! === Boundary conditions III ===
outerbound_l_flag =((dim == 1 .and. m1 == prop%m1) .or. &
                    (dim == 2 .and. m1 == prop%m2) .or. &
                    (dim == 3 .and. m1 == prop%m3)       )
closedbound_l_flag=.false.
if (outerbound_l_flag) then
  closedbound_l_flag=((bound_l == bound_closed       ) .or. &
                      (bound_l == bound_closedbottom ) .or. &
                      (bound_l == bound_closedbottom2)      )
endif
!
outerbound_r_flag =((dim == 1 .and. n1 == prop%n1) .or. &
                    (dim == 2 .and. n1 == prop%n2) .or. &
                    (dim == 3 .and. n1 == prop%n3)       )
closedbound_r_flag=.false.
if (outerbound_r_flag) then
  closedbound_r_flag=((bound_r == bound_closed       ) .or. &
                      (bound_r == bound_closedbottom ) .or. &
                      (bound_r == bound_closedbottom2)       )
endif
!
do i3=m3,n3
  !DIR$ FORCEINLINE
  do i2=m2,n2
    ! --- Compute conservative variables (cell centered) ---
    !
    do i1=m1-nghost1,n1+nghost1
      !
      rhov1(  i1,i2,i3)=rho(i1,i2,i3)*v1(i1,i2,i3)
      rhov2(  i1,i2,i3)=rho(i1,i2,i3)*v2(i1,i2,i3)
      rhov3(  i1,i2,i3)=rho(i1,i2,i3)*v3(i1,i2,i3)
      rhoeikm(i1,i2,i3)=rho(i1,i2,i3)*(0.5*(v1(i1,i2,i3)**2+v2(i1,i2,i3)**2+v3(i1,i2,i3)**2) + &
                                       ei(i1,i2,i3)) + &
                        0.5*(Bc1(i1,i2,i3)**2+Bc2(i1,i2,i3)**2+Bc3(i1,i2,i3)**2)
      !
    end do ! i1
    !
    ! --- Compute further variables (boundary centered) ---
    !
    do i1=m1-1,n1+1
      !
      rhov1_l(i1,i2,i3)=v1_l(i1,i2,i3) * rho_l(i1,i2,i3)
      rhov2_l(i1,i2,i3)=v2_l(i1,i2,i3) * rho_l(i1,i2,i3)
      rhov3_l(i1,i2,i3)=v3_l(i1,i2,i3) * rho_l(i1,i2,i3)
      !
      rhoek_l(i1,i2,i3)=0.5*(v1_l( i1,i2,i3)**2+v2_l( i1,i2,i3)**2+v3_l( i1,i2,i3)**2) * &
                        rho_l(i1,i2,i3)
      rhoem_l(i1,i2,i3)=0.5*(Bc1_l(i1,i2,i3)**2+Bc2_l(i1,i2,i3)**2+Bc3_l(i1,i2,i3)**2)
      !
      rhoeikm_l(i1,i2,i3)=rhoei_l(i1,i2,i3) + rhoek_l(i1,i2,i3) + rhoem_l(i1,i2,i3)
      !
      rhov1_r(i1,i2,i3)=v1_r(i1,i2,i3) * rho_r(i1,i2,i3)
      rhov2_r(i1,i2,i3)=v2_r(i1,i2,i3) * rho_r(i1,i2,i3)
      rhov3_r(i1,i2,i3)=v3_r(i1,i2,i3) * rho_r(i1,i2,i3)
      !
      rhoek_r(i1,i2,i3)=0.5*(v1_r(i1,i2,i3)**2+v2_r(i1,i2,i3)**2+v3_r(i1,i2,i3)**2)*rho_r(i1,i2,i3)
      rhoem_r(i1,i2,i3)=0.5*(Bc1_r(i1,i2,i3)**2+Bc2_r(i1,i2,i3)**2+Bc3_r(i1,i2,i3)**2)
      !
      rhoeikm_r(i1,i2,i3)=rhoei_r(i1,i2,i3) + rhoek_r(i1,i2,i3) + rhoem_r(i1,i2,i3)
      !
    end do ! i1
    !
    do i1=m1,n1+1
      !
      rho_til(   i1,i2,i3)=min(rho_r(  i1-1,i2,i3) ,rho_l(  i1,i2,i3)   )
      Bc1_sq_til(i1,i2,i3)=max(Bc1_r(  i1-1,i2,i3)**2,Bc1_l(i1,i2,i3)**2)
      B_sq_til(  i1,i2,i3)=max(Bc1_r(  i1-1,i2,i3)**2+Bc2_r(i1-1,i2,i3)**2+Bc3_r(i1-1,i2,i3)**2, &
                               Bc1_l(  i1  ,i2,i3)**2+Bc2_l(i1  ,i2,i3)**2+Bc3_l(i1  ,i2,i3)**2)
      !
      P_til(     i1,i2,i3)=max(P_r(    i1-1,i2,i3) ,P_l(    i1,i2,i3)   )
      Gamma_1_til         =max(Gamma_1(i1-1,i2,i3) ,Gamma_1(i1,i2,i3)   )
      f_til=0.5*(f(i1-1,i2,i3)+f(i1,i2,i3))
      !
      ! --- Compute sound speed, Alfven speed, fast magnetosonic speed ---
      csq=P_til(i1,i2,i3)/rho_til(i1,i2,i3)*Gamma_1_til
      vaq=f_til*Bc1_sq_til(i1,i2,i3)/rho_til(i1,i2,i3)
      caq=csq+f_til*B_sq_til(i1,i2,i3)/rho_til(i1,i2,i3)
      dd =caq*sqrt(max(0.0,1.0 - 4.0*(csq/caq)*(vaq/caq)))
      vf_til(    i1,i2,i3)=sqrt(0.5*(caq+dd))
      !
      ! --- Compute characteristic velocities at cell boundaries ---
      lambda_til_mi(i1,i2,i3)=min(v1_r(i1-1,i2,i3),v1_l(i1,i2,i3))-vf_til(i1,i2,i3)
      lambda_til_pl(i1,i2,i3)=max(v1_r(i1-1,i2,i3),v1_l(i1,i2,i3))+vf_til(i1,i2,i3)
    end do ! i1
    !
    ! --- Modify characteristic velocities at cell boundaries, if necessary ---
    if (hdScheme == hdScheme_CenMHD) then
      do i1=m1,n1+1
        lambda0=max(abs(lambda_til_pl(i1,i2,i3)),abs(lambda_til_mi(i1,i2,i3)))
        !
        ! --- For Central scheme, use only maximum characteristic velocity ---
        lambda_til_mi(i1,i2,i3)=-lambda0
        lambda_til_pl(i1,i2,i3)= lambda0
        !
      end do ! i1
    else if (par%N_magDiffSide > 0 .or. par%N_magDiffTop > 0 .or. par%N_magDiffBottom > 0) then
      if (prop%n2 > prop%m2) then
        !DIR$ SIMD
        do i1=m1,n1+1
          ! --- Reduce to Rusanov flux near the boundaries: 3D case ---
          !
          delta = min((i1       - m1_0 + epsilon) / (nb1 + epsilon), &
                      (n1_0 + 1 - i1   + epsilon) / (nt1 + epsilon), &
                      (i2       - m2_0 + epsilon) / (nb2 + epsilon), &
                      (n2_0     - i2   + epsilon) / (nt2 + epsilon), &
                      (i3       - m3_0 + epsilon) / (nb3 + epsilon), &
                      (n3_0     - i3   + epsilon) / (nt3 + epsilon))
          if (delta < 1.0) then
            sigma = 1.0 / (1.0 + exp(10.0 * (0.5 - delta)))
          else
            sigma = 1.0
          endif
          !
          lambda0                =max(abs(lambda_til_pl(i1,i2,i3)),abs(lambda_til_mi(i1,i2,i3)))
          lambda_til_mi(i1,i2,i3)=sigma*lambda_til_mi(i1,i2,i3) - (1.0 - sigma)*lambda0
          lambda_til_pl(i1,i2,i3)=sigma*lambda_til_pl(i1,i2,i3) + (1.0 - sigma)*lambda0
          !
        end do ! i1
      else ! (prop%n2 <= prop%m2)
        !DIR$ SIMD
        do i1=m1,n1+1
          ! --- Reduce to Rusanov flux near the boundaries: 2D case ---
          !
          delta = min((i1       - m1_0 + epsilon) / (nb1 + epsilon), &
                      (n1_0 + 1 - i1   + epsilon) / (nt1 + epsilon), &
                      (i3       - m3_0 + epsilon) / (nb3 + epsilon), &
                      (n3_0     - i3   + epsilon) / (nt3 + epsilon))
          if (delta < 1.0) then
            sigma = 1.0 / (1.0 + exp(10.0 * (0.5 - delta)))
          else
            sigma = 1.0
          endif
          !
          lambda0                =max(abs(lambda_til_pl(i1,i2,i3)),abs(lambda_til_mi(i1,i2,i3)))
          lambda_til_mi(i1,i2,i3)=sigma*lambda_til_mi(i1,i2,i3) - (1.0 - sigma)*lambda0
          lambda_til_pl(i1,i2,i3)=sigma*lambda_til_pl(i1,i2,i3) + (1.0 - sigma)*lambda0
          !
        end do ! i1
      endif ! (prop%n2 > prop%m2)
      !
    endif
    !
    ! === Boundary conditions III ===
    if (closedbound_l_flag) then
      P_r(m1-1,i2,i3)=P_l(m1,i2,i3)
    endif
    !
    if (closedbound_r_flag) then
      P_l(n1+1,i2,i3)=P_r(n1,i2,i3)
    endif
    !
    do i1=m1,n1+1
      !
      ! --- Compute flux for left state ---
      flux_rho_l   = rhov1_r(i1-1,i2,i3)
      flux_rhov1_l = rhov1_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3) + P_r(i1-1,i2,i3)
      flux_rhov2_l = rhov2_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)
      flux_rhov3_l = rhov3_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)
      flux_Bc2_l   = Bc2_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)-Bc1_r(i1-1,i2,i3)*v2_r(i1-1,i2,i3)
      flux_Bc3_l   = Bc3_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)-Bc1_r(i1-1,i2,i3)*v3_r(i1-1,i2,i3)
      flux_rhoeikm_l = (rhoeikm_r(i1-1,i2,i3) + P_r(i1-1,i2,i3) + &
                      0.5*(Bc1_r(i1-1,i2,i3)**2 + &
                           Bc2_r(i1-1,i2,i3)**2 + &
                           Bc3_r(i1-1,i2,i3)**2) &
                     )*v1_r(i1-1,i2,i3) - &
                     ( Bc1_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3) + &
                       Bc2_r(i1-1,i2,i3)*v2_r(i1-1,i2,i3) + &
                       Bc3_r(i1-1,i2,i3)*v3_r(i1-1,i2,i3) &
                     )*Bc1_r(i1-1,i2,i3)
      flux_rhoei_l  = rhoei_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)
      flux_rhoek_l  = rhoek_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)
      flux_rhoem_l  = rhoem_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)
      flux_rhov1B_l= 0.5*(Bc2_r(i1-1,i2,i3)**2+Bc3_r(i1-1,i2,i3)**2-Bc1_r(i1-1,i2,i3)**2)
      flux_rhov2B_l= -Bc1_r(i1-1,i2,i3)*Bc2_r(i1-1,i2,i3)
      flux_rhov3B_l= -Bc1_r(i1-1,i2,i3)*Bc3_r(i1-1,i2,i3)
      !
      ! --- Compute flux for right state ---
      flux_rho_r   = rhov1_l(i1  ,i2,i3)
      flux_rhov1_r = rhov1_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3) + P_l(i1  ,i2,i3)
      flux_rhov2_r = rhov2_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)
      flux_rhov3_r = rhov3_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)
      flux_Bc2_r   = Bc2_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)-Bc1_l(i1  ,i2,i3)*v2_l(i1  ,i2,i3)
      flux_Bc3_r   = Bc3_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)-Bc1_l(i1  ,i2,i3)*v3_l(i1  ,i2,i3)
      flux_rhoeikm_r = (rhoeikm_l(i1  ,i2,i3) + P_l(i1  ,i2,i3) + &
                      0.5*(Bc1_l(i1  ,i2,i3)**2 + &
                           Bc2_l(i1  ,i2,i3)**2 + &
                           Bc3_l(i1  ,i2,i3)**2) &
                     )*v1_l(i1  ,i2,i3) - &
                     ( Bc1_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3) + &
                       Bc2_l(i1  ,i2,i3)*v2_l(i1  ,i2,i3) + &
                       Bc3_l(i1  ,i2,i3)*v3_l(i1  ,i2,i3) &
                     )*Bc1_l(i1  ,i2,i3)
      flux_rhoei_r  = rhoei_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)
      flux_rhoek_r  = rhoek_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)
      flux_rhoem_r  = rhoem_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)
      flux_rhov1B_r= 0.5*(Bc2_l(i1  ,i2,i3)**2+Bc3_l(i1  ,i2,i3)**2-Bc1_l(i1  ,i2,i3)**2)
      flux_rhov2B_r= -Bc1_l(i1  ,i2,i3)*Bc2_l(i1  ,i2,i3)
      flux_rhov3B_r= -Bc1_l(i1  ,i2,i3)*Bc3_l(i1  ,i2,i3)
      !
      ! --- Construct Janhunen source terms ---
      ! --- Eq. (7) for the HLL middle state ---
      rho_m   = (lambda_til_pl(i1,i2,i3)*rho_l(i1  ,i2,i3) - &
                 lambda_til_mi(i1,i2,i3)*rho_r(i1-1,i2,i3) + &
                 flux_rho_l - flux_rho_r) / &
                (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      rhov1_m = (lambda_til_pl(i1,i2,i3)*rhov1_l(i1  ,i2,i3) - &
                 lambda_til_mi(i1,i2,i3)*rhov1_r(i1-1,i2,i3) + &
                 flux_rhov1_l + f(i1  ,i2,i3)*flux_rhov1B_l - &
                 flux_rhov1_r - f(i1-1,i2,i3)*flux_rhov1B_r) / &
                (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      rhov2_m = (lambda_til_pl(i1,i2,i3)*rhov2_l(i1  ,i2,i3) - &
                 lambda_til_mi(i1,i2,i3)*rhov2_r(i1-1,i2,i3) + &
                 flux_rhov2_l + f(i1  ,i2,i3)*flux_rhov2B_l - &
                 flux_rhov2_r - f(i1-1,i2,i3)*flux_rhov2B_r) / &
                (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      rhov3_m = (lambda_til_pl(i1,i2,i3)*rhov3_l(i1  ,i2,i3) - &
                 lambda_til_mi(i1,i2,i3)*rhov3_r(i1-1,i2,i3) + &
                 flux_rhov3_l + f(i1  ,i2,i3)*flux_rhov3B_l - &
                 flux_rhov3_r - f(i1-1,i2,i3)*flux_rhov3B_r) / &
                (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      ! --- Text after Eq. (26) ---
      v1_til(i1,i2,i3) = rhov1_m / rho_m
      v2_til(i1,i2,i3) = rhov2_m / rho_m
      v3_til(i1,i2,i3) = rhov3_m / rho_m
      ! --- Eq. (26) ---
      delta_Bc1 = -v1_til(i1,i2,i3) * (Bc1_l(i1  ,i2,i3) - Bc1_r(i1-1,i2,i3)) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      delta_Bc2 = -v2_til(i1,i2,i3) * (Bc1_l(i1  ,i2,i3) - Bc1_r(i1-1,i2,i3)) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      delta_Bc3 = -v3_til(i1,i2,i3) * (Bc1_l(i1  ,i2,i3) - Bc1_r(i1-1,i2,i3)) / &
                  (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      ! --- Text after Eq. (14) ---
      dx_l = dtime * (min(lambda_til_pl(i1,i2,i3), 0.0) - min(lambda_til_mi(i1,i2,i3), 0.0))
      ! --- Text after Eq. (15) ---
      dx_r = dtime * (max(lambda_til_pl(i1,i2,i3), 0.0) - max(lambda_til_mi(i1,i2,i3), 0.0))
      ! --- Eq. (14) ---
      sourceBc1_l(i1,i2,i3) = delta_Bc1 * dx_l / (xb1(i1  ) - xb1(i1-1))
      sourceBc2_l(i1,i2,i3) = delta_Bc2 * dx_l / (xb1(i1  ) - xb1(i1-1))
      sourceBc3_l(i1,i2,i3) = delta_Bc3 * dx_l / (xb1(i1  ) - xb1(i1-1))
      ! --- Eq. (15) ---
      sourceBc1_r(i1,i2,i3) = delta_Bc1 * dx_r / (xb1(i1+1) - xb1(i1  ))
      sourceBc2_r(i1,i2,i3) = delta_Bc2 * dx_r / (xb1(i1+1) - xb1(i1  ))
      sourceBc3_r(i1,i2,i3) = delta_Bc3 * dx_r / (xb1(i1+1) - xb1(i1  ))
      !
      lambda_til_pl(i1,i2,i3) = max(0.0,lambda_til_pl(i1,i2,i3))
      lambda_til_mi(i1,i2,i3) = min(0.0,lambda_til_mi(i1,i2,i3))
      !
      ! --- Compute weights of fluxes from left and right state, diffusive velocity ---
      c1 = lambda_til_pl(i1,i2,i3)/(lambda_til_pl(i1,i2,i3)-lambda_til_mi(i1,i2,i3))
      c2 = lambda_til_mi(i1,i2,i3)/(lambda_til_pl(i1,i2,i3)-lambda_til_mi(i1,i2,i3))
      c3 = lambda_til_pl(i1,i2,i3)*lambda_til_mi(i1,i2,i3)/ &
                                   (lambda_til_pl(i1,i2,i3)-lambda_til_mi(i1,i2,i3))
      !
      ! --- Compose net HLL fluxes from fluxes and states of left and right side ---
      flux_rho(    i1,i2,i3) = c1 * flux_rho_l - c2 * flux_rho_r - &
                               c3*(rho_r(i1-1,i2,i3)-rho_l(i1  ,i2,i3))

      flux_rhov1(  i1,i2,i3) = c1 * flux_rhov1_l - c2 * flux_rhov1_r - &
                               c3*(rhov1_r(i1-1,i2,i3) - rhov1_l(i1  ,i2,i3))

      flux_rhov2(  i1,i2,i3) = c1 * flux_rhov2_l - c2 * flux_rhov2_r - &
                               c3*(rhov2_r(i1-1,i2,i3) - rhov2_l(i1  ,i2,i3))

      flux_rhov3(  i1,i2,i3) = c1 * flux_rhov3_l - c2 * flux_rhov3_r - &
                               c3*(rhov3_r(i1-1,i2,i3) - rhov3_l(i1  ,i2,i3))

      flux_Bc1(    i1,i2,i3) =-c3*(Bc1_r(i1-1,i2,i3) - Bc1_l(i1  ,i2,i3))

      flux_Bc2(    i1,i2,i3) = c1 * flux_Bc2_l - c2 * flux_Bc2_r - &
                               c3*(Bc2_r(i1-1,i2,i3) - Bc2_l(i1  ,i2,i3))

      flux_Bc3(    i1,i2,i3) = c1 * flux_Bc3_l - c2 * flux_Bc3_r - &
                               c3*(Bc3_r(i1-1,i2,i3) - Bc3_l(i1  ,i2,i3))

      flux_rhoeikm(i1,i2,i3) = c1 * flux_rhoeikm_l - c2 * flux_rhoeikm_r - &
                               c3*(rhoeikm_r(i1-1,i2,i3) - rhoeikm_l(i1  ,i2,i3))

      flux_rhoei(  i1,i2,i3) = c1 * flux_rhoei_l  - c2 * flux_rhoei_r - &
                               c3*(rhoei_r(i1-1,i2,i3) - rhoei_l(i1  ,i2,i3))

      flux_rhoek(  i1,i2,i3) = c1 * flux_rhoek_l  - c2 * flux_rhoek_r - &
                               c3*(rhoek_r(i1-1,i2,i3) - rhoek_l(i1  ,i2,i3))

      flux_rhoem(  i1,i2,i3) = c1 * flux_rhoem_l  - c2 * flux_rhoem_r - &
                               c3*(rhoem_r(i1-1,i2,i3) - rhoem_l(i1  ,i2,i3))

      fep(         i1,i2,i3) = c1 * P_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3) - &
                               c2 * P_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)

      flux_rhov1B( i1,i2,i3) = c1 * flux_rhov1B_l - c2 * flux_rhov1B_r
      flux_rhov2B( i1,i2,i3) = c1 * flux_rhov2B_l - c2 * flux_rhov2B_r
      flux_rhov3B( i1,i2,i3) = c1 * flux_rhov3B_l - c2 * flux_rhov3B_r
      !
    end do ! i1
    !
    ! === Boundary conditions V ===
    !
    if (outerbound_l_flag) then
      if (closedbound_l_flag) then
        ! --- Set mass flux to zero ---
        i1=m1
        flux_rho(i1,i2,i3)=0.0
      endif
      if (bound_l == bound_closedbottom2) then
        ! --- Set advection fluxes explicitly to zero ---
        i1=m1
        flux_rho(    i1,i2,i3)=0.0
        flux_rhov1(  i1,i2,i3)=(lambda_til_pl(i1,i2,i3) * P_r(i1-1,i2,i3) - &
                                lambda_til_mi(i1,i2,i3) * P_l(i1  ,i2,i3)) / &
                               (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        flux_rhov2(  i1,i2,i3)=0.0
        flux_rhov3(  i1,i2,i3)=0.0
        flux_Bc1(    i1,i2,i3)=0.0
        flux_Bc2(    i1,i2,i3)=0.0
        flux_Bc3(    i1,i2,i3)=0.0
        flux_rhoeikm(i1,i2,i3)=0.0
        flux_rhoei(  i1,i2,i3)=0.0
        flux_rhoek(  i1,i2,i3)=0.0
        flux_rhoem(  i1,i2,i3)=0.0
        fep(         i1,i2,i3)=0.0
        flux_rhov1B( i1,i2,i3)=0.0
        flux_rhov2B( i1,i2,i3)=0.0
        flux_rhov3B( i1,i2,i3)=0.0
      endif
      if (bound_l == bound_inoutflow2) then
        ! --- Set fluxes explicitly to fluxes inside box, ---
        ! --- except for some pressure and magnetic field terms --
        i1=m1
        flux_rho(    i1,i2,i3)=rho(i1,i2,i3)*v1(i1,i2,i3)
        flux_rhov1(  i1,i2,i3)=rho(i1,i2,i3)*v1(i1,i2,i3)*v1(i1,i2,i3) + &
                                 (lambda_til_pl(i1,i2,i3) * P_r(i1-1,i2,i3) - &
                                  lambda_til_mi(i1,i2,i3) * P_l(i1  ,i2,i3)) / &
                                 (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
        flux_rhov2(  i1,i2,i3)=rho(i1,i2,i3)*v1(i1,i2,i3)*v2(i1,i2,i3)
        flux_rhov3(  i1,i2,i3)=rho(i1,i2,i3)*v1(i1,i2,i3)*v3(i1,i2,i3)
        flux_rhoeikm(i1,i2,i3)=(rho(i1,i2,i3)*(ei( i1,i2,i3) + &
                                       0.5*(v1( i1,i2,i3)**2 + v2( i1,i2,i3)**2 + &
                                            v3( i1,i2,i3)**2)) + &
                                       0.5*(Bc1(i1,i2,i3)**2 + Bc2(i1,i2,i3)**2 + &
                                            Bc3(i1,i2,i3)**2) + &
                                      (lambda_til_pl(i1,i2,i3) * P_r(i1-1,i2,i3) - &
                                       lambda_til_mi(i1,i2,i3) * P_l(i1  ,i2,i3)) / &
                                      (lambda_til_pl(i1,i2,i3) - &
                                       lambda_til_mi(i1,i2,i3)) + &
                                       0.5*(Bc1(i1,i2,i3)**2 + Bc2(i1,i2,i3)**2 + &
                                            Bc3(i1,i2,i3)**2))*v1(i1,i2,i3) - &
                                      ( Bc1(i1,i2,i3)*v1(i1,i2,i3) + &
                                        Bc2(i1,i2,i3)*v2(i1,i2,i3) + &
                                        Bc3(i1,i2,i3)*v3(i1,i2,i3))*Bc1(i1,i2,i3)
        flux_rhoei(  i1,i2,i3)=rho(i1,i2,i3)*v1(i1,i2,i3)*ei(i1,i2,i3)
        flux_rhoek(  i1,i2,i3)=0.5*rho(i1,i2,i3)*v1(i1,i2,i3)* &
                                        (v1(i1,i2,i3)**2+ &
                                         v2(i1,i2,i3)**2+ &
                                         v3(i1,i2,i3)**2)
        flux_rhoem(  i1,i2,i3)=v1(i1,i2,i3)*(Bc1(i1,i2,i3)**2+ &
                                                        Bc2(i1,i2,i3)**2+ &
                                                        Bc3(i1,i2,i3)**2) - &
                                    Bc1(i1,i2,i3)*(v1(i1,i2,i3)*Bc1(i1,i2,i3)+ &
                                                         v2(i1,i2,i3)*Bc2(i1,i2,i3)+ &
                                                         v3(i1,i2,i3)*Bc3(i1,i2,i3))
        fep(         i1,i2,i3)=(lambda_til_pl(i1,i2,i3) * P_r(i1-1,i2,i3) - &
                                lambda_til_mi(i1,i2,i3) * P_l(i1  ,i2,i3)) * &
                                v1(i1,i2,i3) / &
                                (lambda_til_pl(i1,i2,i3) - lambda_til_mi(i1,i2,i3))
      endif
      !
    endif
    !
    if (closedbound_r_flag) then
      ! --- Set mass flux to zero ---
      i1=n1+1
      flux_rho(i1,i2,i3)=0.0
    endif
    !
    ! \label{f90:hdcheckflux__useMHD1}
    ! === Check if fluxes are save =================================================================
    if (par%hdCheckFlux /= 'Off') then
      call rhd_mhd_CheckFlux(m1,n1, m2,n2, m3,n3, i2,i2, i3,i3, &
                             nghost1, nghost2, nghost3, &
                             hdScheme, &
                             bound_l, bound_r, &
                             dtime, xb1, eiTable_min, &
                             rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                             rhov1, rhov2, rhov3, rhoeikm, &
                             Gamma_1, energy_switch, f, &
                             phic_grav, phib_grav, lambda_til_mi, lambda_til_pl, &
                             flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                             flux_Bc1, flux_Bc2, flux_Bc3, &
                             flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                             flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                             sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                             sourceBc1_r, sourceBc2_r, sourceBc3_r)
    endif
    !
  end do ! i2
end do ! i3
!
!#
# ifdef rhd_roe1d_flux_t01
!#
!call timing_stop('MHD: HLL1Dflux: fluxes')
!#
# endif
!#
!
end subroutine rhd_mhd_HLL1Dflux


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_HLL1Dstep(model, action, dtime, dtime_limit, &
                             output, OutRadialC_flag, AreaFraction, datx, datr, datc, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_HLL1Dstep ('rhd_magneto-hydrodynamics_HLL_1D_step')
!
! PURPOSE:
!   Perform 1D magnetohydrodynamics step in x1 direction, employ HLL scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_HLL1Dstep(model, action, dtime, dtime_limit, &
!                          output=output, OutRadialC_flag=OutRadialC_flag, &
!                          AreaFraction=AreaFraction, &
!                          datx=datx, datr=datr, datc=datc, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                 volume averaging of cell centered quantities is performed
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for
!                 additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!                   100:   Pressure becomes non-positive
!                   101:   Pressure derivative dPdrho|ei becomes non-positive
!                   102:   Pressure derivative dPdei|rho becomes non-positive
!                   103:   Density becomes non-positive
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   rho:          ('rho') real 3D array, mass density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('v_1') real 3D array, velocity, 1st coordinate
!   v2:           ('v_2') real 3D array, velocity, 2nd coordinate
!   v3:           ('v_3') real 3D array, velocity, 3rd coordinate
!   Bc1:          ('B_center_1') real 3D array, cell centered magnetic field, 1. component
!   Bc2:          ('B_center_2') real 3D array, cell centered magnetic field, 2. component
!   Bc3:          ('B_center_3') real 3D array, cell centered magnetic field, 3. component
!   eip:          ('energy_internal_pressure') real 3D array, internal + pressure energy density
!   rhoei:        ('rho_energy_internal') real 3D array, internal energy density per volume, rho*ei
!   rhov1:        ('rho_velocity_1') real 3D array, momentum, 1st dimension
!   P:            ('pressure') real 3D array, gas pressure on model grid
!   P_mag:        ('pressure') real 3D array, magnetic pressure on model grid
!   P_tot:        ('pressure') real 3D array, total pressure on model grid
!   dPdei:        ('delta_pressure_delta_energy_internal') real 3D array, pressure derivative
!                 (constant density) on model grid
!   dPdrho:       ('delta_pressure_delta_rho') real 3D array, pressure derivative
!                 (constant internal energy)
!   T:            ('temperature') real 3D array, temperature on model grid
!   dTdei:        ('dT_dei') real 3D array, temperature derivative (constant density) on model grid
!                 on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   va:           ('v_alfven') real 3D array, Alfven velocity on model grid
!   vf:           ('v_fast') real 3D array, fast magnetosonic speed on model grid
!   vs:           ('v_slow') real 3D array, slow magnetosonic speed on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   drhoeidpconsts: ('delta_rho_energy_internal_delta_pressure_constant_s') real 3D array,
!                   thermodynamic quantity: drhoei/dP|_s
!   sqrt_rho:     ('square_root_rho') real scalar, square root of density on model grid
!   phic_grav:    ('phi_central_gravity) real 3D array, gravitational potential, cell centers
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   Hp_l:         ('height_pressure_left') real 2D array, averaged pressure scale height
!                 on left (m1) side
!   Hp_r:         ('height_pressure_right') real 2D array, averaged pressure scale height
!                 on right (n1) side
!   wl:           ('weight_left') real 3D array, geometrical weights due to non-equidistant grid
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe averaged velocity 3 at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe averaged sound speed at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_Bc1:     ('flux_B_center_2') real 3D array
!   flux_Bc2:     ('flux_B_center_2') real 3D array
!   flux_Bc3:     ('flux_B_center_3') real 3D array
!   flux_rhoeikm: ('flux_rho_energy_internal_kinetic_magnetic') total energy flux through cell bnd.
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!   sourceBc1_l:  ('source_B_center_1_left') real 3D array, Janhunen source term
!   sourceBc2_l:  ('source_B_center_2_left') real 3D array, Janhunen source term
!   sourceBc3_l:  ('source_B_center_3_left') real 3D array, Janhunen source term
!   sourceBc1_r:  ('source_B_center_1_right') real 3D array, Janhunen source term
!   sourceBc2_r:  ('source_B_center_2_right') real 3D array, Janhunen source term
!   sourceBc3_r:  ('source_B_center_3_right') real 3D array, Janhunen source term
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_bound_1DTransmit_mhd: Correct values in top or bottom cells for transmitting boundary cond
!   rhd_grav:              Compute gravitational potential
!   rhd_mhd_Hancock:       Perform Hancock predictor step in 1 dimension
!   rhd_mhd_HLL1Dflux:     Compute 1D MHD HLL fluxes
!   rhd_mhd_Reconstruct:   Reconstruct quantities in cells and compute boundary values
!   rhd_mhd_Roe1Doutput:   Extract some averaged quantities
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   model%nghost2=0, model%nghost3=0
!
! KEYWORDS:
!   equations of 3D compressible hydrodynamics
!   directional splitting: 1D substep
!   HLL solver
!   higher order reconstruction schemes: constant/Minmod,vanLeer,Superbee/PP
!   optional Hancock predictor step
!   general EOS (tabulated values)
!   non-equidistant grid possible
!   source terms due to gravity: no operator-splitting here (constant gravity or central potential)
!   see: HLL, Leveque, Strang
!
! PROCEDURE:
!   1. Preliminaries: compute gravitational potential, fill ghost cells, apply boundary conditions
!   2. Solve EOS and compute further quantities from old values on extended grid
!   3. Reconstruct quantities within the cells
!   4. Perform Hancock predictor step
!   5. Compute averages and fluxes located at cell boundaries
!   6. Update basic quantities with fluxes and source terms
!
! EXAMPLE:
!
! TODO:
!   Substitution: par -> action
!
! Based on similar Routine 'rhd_hyd_Roe1Dstep'
!
! MODIFICATION HISTORY:
!   2004-03-23 (W.S.) First version
!   2004-10-26 (W.S.) Update of internal energy modified
!   2008-03-24 (W.S. East Lansing) Dual energy calculation
!   2010-02-16 (W.S. Freiburg) c_test2 -> va_max
!   2010-04-10 (W.S. Freiburg) Alfven speed reduction not only for beta_inv = 0.0
!   2010-06-11 (B.F. Lyon) Higher-order terms (in time) in source term for rhov1 removed
!   2011-06-17 (W.S.) Derived from routine rhd_mhd_Roe1Dstep
!   2013-03-05 (B.F.) sqrt_rho becomes scalar
!   2013-06-10 (B.F.) Request only time-step reduction (no force stop) for small Gamma_1 or P
!   2014-07-15 (B.F.) grav_factor -> wgrav_new, account for parameter C_gravmu
!   2015-04-01 (B.F., O.S.) Bugfix: compute dd with max
!   2018-10-18 (B.F.) Some cleanup. Add eiTable_min for all reconstruction orders
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_bound_module
use rhd_box_module
use rhd_phys_module
!#
# ifdef rhd_roe1d_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output, OutRadialC_flag
real,            optional, intent(in)   :: AreaFraction
type(datx_type), optional, intent(inout):: datx, datr
type(datc_type), optional, intent(inout):: datc
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: grav_flag, Output_flag, T_flag
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1
integer                                 :: i1, i2, i3
integer                                 :: bound_l, bound_r
real                                    :: dei_min, &
                                           rhonew_min, rhoout, eiout, v1out, v2out, v3out, &
                                           Bc1out, Bc2out, Bc3out, w0, w1, &
                                           einew_min, eiTable_min, Pg, Pm, ei_min, &
                                           source_rhov1, source_eikm, source_ei, &
                                           sqrt_rho, &
                                           caq, dd, B_sq_tan, dt, drho, wgrav_new
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, dimension(3)                   :: error_loc
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1) :: xc1
real, dimension( &
            model%m1-model%nghost1:&
            model%n1+model%nghost1+1)   :: xb1
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, P_mag, P_tot, cs, va, vf, vs, Gamma_1, &
                                           phic_grav, f, rhov1, rhov2, rhov3, rhoeikm, &
                                           rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, &
                                           P_L, rhoei_l, &
                                           rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, &
                                           P_r, rhoei_r
logical, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: energy_switch
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: phib_grav
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: Hp_l, Hp_r
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3)      :: flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_Bc1, flux_Bc2, flux_Bc3, &
                                           flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                                           flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                                           v1_til, v2_til, v3_til, &
                                           sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                                           sourceBc1_r, sourceBc2_r, sourceBc3_r
real, dimension(model%m1:model%n1)      :: dtimeoverdx
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: dPdei, dPdrho, T, dTdei, s
real, parameter                         :: epsilon=0.000001
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D:')
!#
# endif
!#
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1     =model%m1
n1     =model%n1
m2     =model%m2
n2     =model%n2
m3     =model%m3
n3     =model%n3
nghost1=model%nghost1
!
! --- Output of certain averages ---
if (present(output)) then
  Output_flag=output
else
  Output_flag=.false.
endif
!
call tabinter_getinfo(x2min=eiTable_min)
!
! === Boundary conditions I ========================================================================
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
! --- Boundary conditions by filling of ghost cells: axes only ---
call rhd_bound_1DGhost(model, action, grid_mode=1, bulk_mode=0)
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D: grav')
!#
# endif
!#
! === Gravitational potential ======================================================================
call rhd_grav(model, action, grav_flag, phic_grav, phib_grav)
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D: grav')
call timing_start('MHD: HLL1D: boundaries')
call timing_start('MHD: HLL1D: boundaries: phi')
!#
# endif
!#
!
!
! === Boundary conditions II =======================================================================
!
! --- Boundary conditions for gravity: potential set constant in ghost zones ---
if (grav_flag) then
  if (.not. ((bound_l == bound_periodic        ) .or. &
             (bound_l == bound_transmitting    ) .or. &
             (bound_l == bound_transmitting2   ) .or. &
             (bound_l == bound_inoutflow2      ) .or. &
             (bound_l == bound_inoutflow3      ) .or. &
             (bound_l == bound_closedbottom2   ) .or. &
             (bound_l == bound_InternalNext    ) .or. &
             (bound_l == bound_InternalPeriodic))    )   then
    do i1=m1-nghost1,m1-1
      phic_grav(i1,:,:)=phic_grav(m1,:,:)
    end do ! i1
    do i1=m1-nghost1,m1
      phib_grav(i1,:,:)=phic_grav(m1,:,:)
    end do ! i1
  endif
  if (.not. ((bound_r == bound_periodic        ) .or. &
             (bound_r == bound_transmitting    ) .or. &
             (bound_r == bound_transmitting2   ) .or. &
             (bound_r == bound_InternalNext    ) .or. &
             (bound_r == bound_InternalPeriodic))    )   then
    do i1=n1+1,n1+nghost1
      phic_grav(i1,:,:)=phic_grav(n1,:,:)
    end do ! i1
    do i1=n1+1,n1+nghost1+1
      phib_grav(i1,:,:)=phic_grav(n1,:,:)
    end do ! i1
  endif
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D: boundaries: phi')
!#
# endif
!#
if ((bound_l == bound_transmitting ) .or. (bound_r == bound_transmitting ) .or. &
    (bound_l == bound_transmitting2) .or. (bound_r == bound_transmitting2) .or. &
    (bound_l == bound_transmitting3) .or. (bound_r == bound_transmitting3)     ) then
  ! --- Transmitting lower and upper boundary ---
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('MHD: HLL1D: boundaries: transmit')
!#
# endif
!#
  call rhd_bound_1DTransmit_mhd(model, action, dtime, grav_flag, phib_grav, Hp_l=Hp_l, Hp_r=Hp_r)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: HLL1D: boundaries: transmit')
!#
# endif
!#
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D: boundaries: ghost')
!#
# endif
!#
!
! --- Boundary conditions by filling of ghost cells: bulk data only ---
call rhd_bound_1DGhost(model, action, grid_mode=0, bulk_mode=1, Hp_l=Hp_l, Hp_r=Hp_r)
!if (action%dim == 3) then
!  write(6,'(A, 8(1PE14.6))') 'Roe1Dstep; minmax(rho), minmax(ei):', &
!    minval(model%rho(m1        :n1        , m2:n2, m3:n3)), &
!    maxval(model%rho(m1        :n1        , m2:n2, m3:n3)), &
!    minval(model%rho(m1-nghost1:n1+nghost1, m2:n2, m3:n3)), &
!    maxval(model%rho(m1-nghost1:n1+nghost1, m2:n2, m3:n3)), &
!    minval(model%ei( m1        :n1        , m2:n2, m3:n3)), &
!    maxval(model%ei( m1        :n1        , m2:n2, m3:n3)), &
!    minval(model%ei( m1-nghost1:n1+nghost1, m2:n2, m3:n3)), &
!    maxval(model%ei( m1-nghost1:n1+nghost1, m2:n2, m3:n3))
!endif

!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D: boundaries: ghost')
call timing_stop('MHD: HLL1D: boundaries')
!#
# endif
!#
! --- Save original values into local copies ---
xc1=model%xc1
xb1=model%xb1
rho=model%rho
ei =model%ei
v1 =model%v1
v2 =model%v2
v3 =model%v3
Bc1=model%Bc1
Bc2=model%Bc2
Bc3=model%Bc3
energy_switch=model%energy_switch
!
!
! === EOS, further quantities, check Courant condition =============================================
!
! --- Solve EOS ---
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D: eosinter')
!#
# endif
!#
if (model%nquc > 0) then
  T_flag=(Output_flag .or. any(prop%quc_advect == 2))
else
  T_flag=Output_flag
endif
if (T_flag) then
  call eosinter_all(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
else
  call eosinter_PdP(rho, ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei)
endif
Gamma_1=dPdrho*rho/P + dPdei/rho
P_mag=0.5*(Bc1**2+Bc2**2+Bc3**2)
P_tot=P+P_mag
!
if (action%dim == 1 .or. par%hdsplit =='unsplit') then
  !
  if (par%beta_inv < 0.0 .or. par%A%hdScheme == hdScheme_RoeMHD) then
    energy_switch = .true.
  else
    energy_switch=(P_mag/P < par%beta_inv)
  endif
  !
  model%energy_switch=energy_switch
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop( 'MHD: HLL1D: eosinter')
!#
# endif
!#
!
! --- Check positivity of quantities ---
if (any(P <= 0.0)) then
  ierr0=4
  error_loc=minloc(P)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+ m3
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') P(  error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Pressure non-positive (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif ! (any(P <= 0.0))
if (any(Gamma_1 < 1.0)) then
  ierr0=5
  error_loc=minloc(Gamma_1)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+ m3
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Gamma_1 smaller than 1 (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif ! (any(Gamma_1 < 1.0))
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D: derived q')
!#
# endif
!#
dtime_limit=0.1*huge(dtime_limit)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1-nghost1,n1+nghost1
      ! --- Compute derived quantities: sound speed,            ---
      ! --- speeds of magnetic waves, maximum allowed time step ---
      !
      cs(     i1,i2,i3)=sqrt(P(i1,i2,i3)/rho(i1,i2,i3)*Gamma_1(i1,i2,i3))
      sqrt_rho         =sqrt(rho(i1,i2,i3))
      va(     i1,i2,i3)=abs(Bc1(i1,i2,i3))/sqrt_rho
      !
      ! \label{f90:va_max__useMHD1}
      if ((par%va_max > 0.0) .and. &
          ((par%A%hdScheme == hdScheme_HLLMHD) .or. (par%A%hdScheme == hdScheme_CenMHD))) then
        f(    i1,i2,i3)=par%va_max**2/(par%va_max**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3))
        va(   i1,i2,i3)=sqrt(f(i1,i2,i3))*va(i1,i2,i3)
        P_mag(i1,i2,i3)=f(i1,i2,i3)*P_mag(i1,i2,i3)
        P_tot(i1,i2,i3)=P(i1,i2,i3)+P_mag(i1,i2,i3)
      else
        f(i1,i2,i3)=1.0
      endif
      !
      B_sq_tan         =Bc2(i1,i2,i3)**2+Bc3(i1,i2,i3)**2
      !
      if ((B_sq_tan * f(i1,i2,i3)) > (epsilon * P_tot(i1,i2,i3))) then
        caq            =cs(i1,i2,i3)**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd             =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va(i1,i2,i3)/caq)**2))
        vf(   i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs(   i1,i2,i3)=cs(i1,i2,i3)*va(i1,i2,i3) * sqrt(2.0 / (caq+dd))
      else
        if      ((cs(i1,i2,i3) - va(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf(i1,i2,i3) = cs(i1,i2,i3)
          vs(i1,i2,i3) = va(i1,i2,i3)
        else if ((va(i1,i2,i3) - cs(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf(i1,i2,i3) = va(i1,i2,i3)
          vs(i1,i2,i3) = cs(i1,i2,i3)
        else
          vf(i1,i2,i3) = cs(i1,i2,i3)
          vs(i1,i2,i3) = cs(i1,i2,i3)
        endif
      endif
      !
      if ((i1 >= (m1-1)) .and. (i1 <= (n1+1))) then
        dtime_limit    =min(dtime_limit, (xb1(i1+1)-xb1(i1)) / (abs(v1(i1,i2,i3))+vf(i1,i2,i3)))
      endif
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D: derived q')
!#
# endif
!#
!
! --- Check Courant condition ---
!
if ((par%C_CourantMax > 0.0) .and. (dtime > par%C_CourantMax * dtime_limit)) then
  ! --- Time step too large: return without further computation ---
  ierr0=1
  outstr0='Time step too large for Courant condition'
  goto 999
endif
!
! --- Perform reconstruction and (optional) Hancock predictor step ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('MHD: HLL1D: Reconstruction')
!#
# endif
!#
call rhd_mhd_Reconstruct(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                         par%A%reconstruction_order, par%A%reconstruction_type, action%dim, &
                         dtime, xb1, f, &
                         rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, Gamma_1, &
                         rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                         rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('MHD: HLL1D: Reconstruction')
!#
# endif
!#
!
if (par%A%reconstruction_order > 0) then
  if (par%hdTimeIntegrationScheme == 'Hancock') then
    ! --- Perform Hancock predictor step ---
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('MHD: HLL1D: Hancock')
!#
#   endif
!#
    call rhd_mhd_Hancock(m1,n1, m2,n2, m3,n3, nghost1, &
                         dtime, xb1, &
                         rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                         Gamma_1, f, phib_grav, &
                         rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                         rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('MHD: HLL1D: Hancock')
!#
#   endif
!#
  endif ! (par%hdTimeIntegrationScheme == 'Hancock')
endif ! (par%A%reconstruction_order > 0)
!
! --- Add minimum internal energy. It was subtracted in rhd_mhd_Reconstruct. ---
rhoei_l=rhoei_l + rho_l*eiTable_min
rhoei_r=rhoei_r + rho_r*eiTable_min
!
!
! === Main step: compute boundary located (flux-like) averages and fluxes ==========================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D: fluxes')
!#
# endif
!#
!
call rhd_mhd_HLL1Dflux(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                       par%A%hdScheme, &
                       bound_l, bound_r, par%A%reconstruction_order, par%A%reconstruction_type, &
                       action%dim, dtime, xb1, f, &
                       rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                       rho_l, v1_l, v2_l, v3_l, Bc1_l, Bc2_l, Bc3_l, P_l, rhoei_l, &
                       rho_r, v1_r, v2_r, v3_r, Bc1_r, Bc2_r, Bc3_r, P_r, rhoei_r, &
                       Gamma_1, energy_switch, &
                       phic_grav, phib_grav, &
                       flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_Bc1, flux_Bc2, flux_Bc3, &
                       flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                       flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                       sourceBc1_l, sourceBc2_l, sourceBc3_l, &
                       sourceBc1_r, sourceBc2_r, sourceBc3_r)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D: fluxes')
!#
# endif
!#
if (Output_flag) then
  ! === Put additional output data into datx, datr, and datc =======================================
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('MHD: HLL1D: (output)')
!#
# endif
!#
  call rhd_mhd_Roe1Doutput(model, action%dim, AreaFraction, OutRadialC_flag, &
                           P, T, s, Gamma_1, dPdei/rho + 1.0, dTdei, dPdei, &
                           phic_grav, phib_grav, &
                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                           flux_rhoeikm, flux_rhoei, flux_rhoek, flux_rhoem, fep, &
                           flux_rhov1B, flux_rhov2B, flux_rhov3B, &
                           datx=datx, datr=datr, datc=datc)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: HLL1D: (output)')
!#
# endif
!#
endif
!
!
! === Update with computed fluxes and source terms =================================================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL1D: update')
!#
# endif
!#
! --- dtimeoverdx=dt/dx ---
dtimeoverdx=dtime/(xb1(m1+1:n1+1)-xb1(m1:n1))
!
! --- Check if density is positive everywhere ---
!#
# if (rhd_mhd_test_split == 1)
!#
if (action%dim == 3) then
!#
# endif
!#
rhonew_min=huge(rhonew_min)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      rhonew_min=min(rhonew_min, rho(i1,i2,i3) - &
                                 dtimeoverdx(i1) * (flux_rho(i1+1,i2,i3)-flux_rho(i1,i2,i3)))

    end do ! i1
  end do ! i2
end do ! i3
if (rhonew_min <= 0.0) then
  ierr0=6
  dtime_limit=dtime
  outstr0='Density becomes non-positive'
  goto 999
endif
!
!#
# if (rhd_mhd_test_split == 1)
!#
endif
!#
# endif
!#
!
! \label{f90:c_gravmu__useMHD1}
! --- Choose time-centering of gravity source terms ---
if (par%C_gravmu < 0.0) then
  ! --- Use old defaults ---
  if (par%hdTimeIntegrationScheme == 'Hancock' .and. par%A%reconstruction_order > 0) then
    ! --- Use time-centered density ---
    wgrav_new=0.5
  else
    ! --- E.g., for Runge-Kutta use only first-order terms (density at old time step) ---
    wgrav_new=0.0
  endif
else
  ! --- Use input parameter as weight ---
  wgrav_new=par%C_gravmu
endif
!
! --- Update all quantities ---
!
do i3=m3,n3
  do i2=m2,n2
    !DIR$ IVDEP
    do i1=m1,n1
      ! --- Compute change in mass density and update it ---
      drho  =-dtimeoverdx(i1) * (flux_rho(  i1+1,i2,i3)-flux_rho(  i1,i2,i3))
      rhoout=rho(i1,i2,i3) + drho
      !
      ! --- Compute source terms due to gravity ---
      w0=(phic_grav(i1  ,i2,i3)-phib_grav(i1,i2,i3)) / &
         (xb1(      i1+1      )-xb1(      i1      ))
      w1=(phib_grav(i1+1,i2,i3)-phic_grav(i1,i2,i3)) / &
         (xb1(      i1+1      )-xb1(      i1      ))
      source_eikm =w0*flux_rho(i1,i2,i3) + w1*flux_rho(i1+1,i2,i3) + (1.0 - f(i1,i2,i3)) * &
                   (v1(i1,i2,i3) * (flux_rhov1B(i1+1,i2,i3) - flux_rhov1B(i1  ,i2,i3)) / &
                                   (xb1(        i1+1      ) - xb1(        i1        )) + &
                    v2(i1,i2,i3) * (flux_rhov2B(i1+1,i2,i3) - flux_rhov2B(i1  ,i2,i3)) / &
                                   (xb1(        i1+1      ) - xb1(        i1        )) + &
                    v3(i1,i2,i3) * (flux_rhov3B(i1+1,i2,i3) - flux_rhov3B(i1  ,i2,i3)) / &
                                   (xb1(        i1+1      ) - xb1(        i1        )))
      source_ei   = -P(i1,i2,i3) * (v1(i1+1,i2,i3) - v1(i1-1,i2,i3)) / (xc1(i1+1) - xc1(i1-1))
      source_rhov1=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) / &
                                                    (xb1(      i1+1      )-xb1(      i1      ))
      !
      ! --- Update ---
      !
      v1out =(rho(i1,i2,i3) * v1(i1,i2,i3) - &
                             dtimeoverdx(i1) * ((flux_rhov1(i1+1,i2,i3)-flux_rhov1(i1,i2,i3)) + &
                             f(i1,i2,i3)     *  (flux_rhov1B(i1+1,i2,i3)-flux_rhov1B(i1,i2,i3))) + &
                             dtime           * source_rhov1 &
             )/rhoout
      v2out =(rho(i1,i2,i3) * v2(i1,i2,i3) - &
                             dtimeoverdx(i1) * ((flux_rhov2(i1+1,i2,i3)-flux_rhov2(i1,i2,i3)) + &
                             f(i1,i2,i3)     *  (flux_rhov2B(i1+1,i2,i3)-flux_rhov2B(i1,i2,i3))) &
             )/rhoout
      v3out =(rho(i1,i2,i3) * v3(i1,i2,i3) - &
                             dtimeoverdx(i1) * ((flux_rhov3(i1+1,i2,i3)-flux_rhov3(i1,i2,i3)) + &
                             f(i1,i2,i3)     *  (flux_rhov3B(i1+1,i2,i3)-flux_rhov3B(i1,i2,i3))) &
             )/rhoout
      Bc1out=Bc1(i1,i2,i3) - dtimeoverdx(i1) * (flux_Bc1(  i1+1,i2,i3)-flux_Bc1(  i1,i2,i3)) + &
                             sourceBc1_l(i1+1,i2,i3) + sourceBc1_r(i1,i2,i3)
      Bc2out=Bc2(i1,i2,i3) - dtimeoverdx(i1) * (flux_Bc2(  i1+1,i2,i3)-flux_Bc2(  i1,i2,i3)) + &
                             sourceBc2_l(i1+1,i2,i3) + sourceBc2_r(i1,i2,i3)
      Bc3out=Bc3(i1,i2,i3) - dtimeoverdx(i1) * (flux_Bc3(  i1+1,i2,i3)-flux_Bc3(  i1,i2,i3)) + &
                             sourceBc3_l(i1+1,i2,i3) + sourceBc3_r(i1,i2,i3)
      !
      if (model%energy_switch(i1,i2,i3)) then
        !
        ! --- Equation for total energy ---
        eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) + &
                0.5 * rho(i1,i2,i3) * (v1(i1,i2,i3)**2 + v2(i1,i2,i3)**2 + v3(i1,i2,i3)**2) - &
                dtimeoverdx(i1) * (flux_rhoeikm(i1+1,i2,i3)-flux_rhoeikm(i1,i2,i3)) + &
                0.5 * (Bc1(i1,i2,i3) + Bc1out) * &
                              (dtimeoverdx(i1) * (flux_Bc1(  i1+1,i2,i3)-flux_Bc1(  i1,i2,i3)) - &
                               sourceBc1_l(i1+1,i2,i3) - sourceBc1_r(i1,i2,i3)) + &
                0.5 * (Bc2(i1,i2,i3) + Bc2out) * &
                              (dtimeoverdx(i1) * (flux_Bc2(  i1+1,i2,i3)-flux_Bc2(  i1,i2,i3)) - &
                               sourceBc2_l(i1+1,i2,i3) - sourceBc2_r(i1,i2,i3)) + &
                0.5 * (Bc3(i1,i2,i3) + Bc3out) * &
                              (dtimeoverdx(i1) * (flux_Bc3(  i1+1,i2,i3)-flux_Bc3(  i1,i2,i3)) - &
                               sourceBc3_l(i1+1,i2,i3) - sourceBc3_r(i1,i2,i3)) + &
                dtime           * source_eikm &
               )/rhoout - 0.5 * (v1out**2 + v2out**2 + v3out**2)
      else
        !
        ! --- Equation for thermal energy (without any dissipation terms) ---
        eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) - &
                              dtimeoverdx(i1) * (flux_rhoei(i1+1,i2,i3) -flux_rhoei(i1,i2,i3)) + &
                              dtime           * source_ei) / rhoout
      endif
      !
      model%rho(i1,i2,i3)=rhoout
      model%v1( i1,i2,i3)=v1out
      model%v2( i1,i2,i3)=v2out
      model%v3( i1,i2,i3)=v3out
      model%Bc1(i1,i2,i3)=Bc1out
      model%Bc2(i1,i2,i3)=Bc2out
      model%Bc3(i1,i2,i3)=Bc3out
      model%ei( i1,i2,i3)=eiout
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Save fluxes for flux-interpolated transport step ---
do i3 = m3, n3
  do i2 = m2, n2
    do i1 = m1, n1 + 1
      model%flux_B21(i1, i2, i3) = flux_Bc2(i1, i2, i3)
      model%flux_B31(i1, i2, i3) = flux_Bc3(i1, i2, i3)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Check if internal energy is below minimum table value anywhere ---
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*dtime
  outstr0='Internal energy below minimum table value'
  goto 998
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D: update')
!#
# endif
!#
!
! --- Check internal energy ---
if (par%C_maxeiChange > 0.0) then
  dei_min=min(minval((model%ei(m1:n1,:,:)-ei(m1:n1,:,:))*rho(m1:n1,:,:)/P(m1:n1,:,:)),0.0)
  dtime_limit=min(dtime_limit, &
                  dtime*par%C_maxeiChange/(-dei_min+10.*tiny(dei_min))*0.999)
  if (dei_min < -par%C_maxeiChange) then
    ierr0=2
    outstr0='Drop in internal energy too large'
    goto 999
  endif
endif
!
! === Boundary conditions VI =======================================================================
if (bound_l == bound_transmitting2) then
  model%v1(m1,m2:n2,m3:n3)=v1(m1,m2:n2,m3:n3) + dtime*cs(m1,m2:n2,m3:n3) * &
                                         (       v1(m1+1,m2:n2,m3:n3)-       v1(m1,m2:n2,m3:n3))/ &
                                         (model%xc1(m1+1            )-model%xc1(m1            ))
endif
if (bound_r == bound_transmitting2) then
  model%v1(n1,m2:n2,m3:n3)=v1(n1,m2:n2,m3:n3) - dtime*cs(n1,m2:n2,m3:n3) * &
                                         (       v1(n1-1,m2:n2,m3:n3)-       v1(n1,m2:n2,m3:n3))/ &
                                         (model%xc1(n1-1            )-model%xc1(n1            ))
endif
!
!
999 dtime_limit=par%C_Courant*dtime_limit
!
998 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL1D:')
!#
# endif
!#
!
end subroutine rhd_mhd_HLL1Dstep


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_HLL2Dstep(model, action, dtime, dtime_limit, &
                             output, AreaFraction1, AreaFraction3, &
                             datx1, datx3, datr, datc, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_HLL2Dstep ('rhd_magneto-hydrodynamics_HLL_2D_step')
!
! PURPOSE:
!   Perform unsplit 2D magnetohydrodynamics step in, employ HLL-Hancock scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_HLL2Dstep(model, action, dtime, dtime_limit, &
!                          output=output, AreaFraction1=AreaFraction1, &
!                          AreaFraction3=AreaFraction3, &
!                          datx1=datx1, datx3=datx3, datr=datr, datc=datc, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!   AreaFraction1: ('Area_Fraction1') real, optional, weighting factor for the current model slice
!   AreaFraction3: ('Area_Fraction3') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   datx1:        ('data_x1') type(datx_type), optional, structure for additional quantities
!   datx3:        ('data_x3') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for
!                 additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!                   100:   Pressure becomes non-positive
!                   101:   Pressure derivative dPdrho|ei becomes non-positive
!                   102:   Pressure derivative dPdei|rho becomes non-positive
!                   103:   Density becomes non-positive
!
! LOCAL VARIABLES:
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                    volume averaging of cell centered quantities is performed
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_grav:              Compute gravitational potential
!   rhd_mhd_Hancock2D:     Perform Hancock predictor step in 2 dimensions
!   rhd_mhd_HLL1Dflux:     Compute 1D MHD HLL fluxes
!   rhd_mhd_Reconstruct:   Reconstruct quantities in cells and compute boundary values
!   rhd_mhd_Roe1Doutput:   Extract some averaged quantities
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   model%nghost2=0
!
! KEYWORDS:
!   equations of 3D compressible hydrodynamics
!   unsplit scheme: 3D substep
!   HLL solver, Hancock scheme
!   higher order by slope-limiters, reconstruction schemes: constant/Minmod,vanLeer,Superbee/PP
!   general EOS (tabulated values)
!   non-equidistant grid possible
!   source terms due to gravity: no operator-splitting here (constant gravity or central potential)
!
! PROCEDURE:
!   1. Preliminaries: compute gravitational potential, fill ghost cells, apply boundary conditions
!   2. Solve EOS and compute further quantities from old values on extended grid
!   3. Reconstruct quantities within the cells
!   4. Perform Hancock predictor step
!   5. Compute averages and fluxes located at cell boundaries
!   6. Update basic quantities with fluxes and source terms
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-03-23 (W.S.) First version
!   2004-10-26 (W.S.) Update of internal energy modified
!   2008-03-24 (W.S. East Lansing) Dual energy calculation
!   2010-02-16 (W.S. Freiburg) c_test2 -> va_max
!   2010-04-10 (W.S. Freiburg) Alfven speed reduction not only for beta_inv = 0.0
!   2010-06-11 (B.F. Lyon) Higher-order terms (in time) in source term for rhov1 removed
!   2011-06-10 (W.S.) Derived from routine rhd_mhd_HLL1Dstep
!   2013-03-05 (B.F.) Parameter outradialc becomes local variable; sqrt_rho becomes scalar
!   2013-03-07 (B.F.) Account for "internal" MPI boundary conditions for gravitatioal potential
!   2013-06-10 (B.F.) Request only time-step reduction (no force stop) for small Gamma_1 or P
!   2014-07-15 (B.F.) grav_factor -> wgrav_new, account for parameter C_gravmu
!   2014-08-17 (B.F.) Rename: model3 -> model_3
!   2018-10-18 (B.F.) Some cleanup. Add eiTable_min for all reconstruction orders.
!                     Change loop order for transmitting left boundaries.
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_phys_module
!#
# ifdef rhd_roe1d_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output
real,            optional, intent(in)   :: AreaFraction1, AreaFraction3
type(datx_type), optional, intent(inout):: datx1, datx3, datr
type(datc_type), optional, intent(inout):: datc
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
type(box_type)                          :: model_3
logical                                 :: grav_flag, Output_flag, T_flag, OutRadialC_flag
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, nghost3
integer                                 :: i1, i2, i3, m10, n10, m30, n30
integer                                 :: bound_l1, bound_r1, bound_l3, bound_r3
real                                    :: dei_min, &
                                           rhonew_min, rhoout, eiout, v1out, v2out, v3out, &
                                           Bc1out, Bc2out, Bc3out, w01, w11, w03, w13, &
                                           einew_min, eiTable_min, Pg, Pm, ei_min, &
                                           source_rhov1, source_rhov3, source_eikm, source_ei, &
                                           sqrt_rho, &
                                           caq, dd, B_sq_tan, dt, drho, wgrav_new
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, dimension(3)                   :: error_loc
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1) :: xc1
real, dimension( &
            model%m1-model%nghost1:&
            model%n1+model%nghost1+1)   :: xb1
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3) :: xc3
real, dimension( &
            model%m3-model%nghost3:&
            model%n3+model%nghost3+1)   :: xb3
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2     ,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, P_mag, P_tot, cs, va1, va3, vf1, vf3, vs1, vs3, &
                                           Gamma_1, phic_grav, f
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3,&
                model%m2:model%n2     ,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: rho3, ei3, v13, v23, v33, Bc13, Bc23, Bc33, &
                                           P3, Gamma_13, phic_grav3, f3
real, dimension(model%m1-1:model%n1+1 ,&
                model%m2  :model%n2   ,&
                model%m3-1:model%n3+1)  :: rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, &
                                           P_l1, rhoei_l1, &
                                           rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, &
                                           P_r1, rhoei_r1
real, dimension(model%m3-1:model%n3+1 ,&
                model%m2  :model%n2   ,&
                model%m1-1:model%n1+1)  :: rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, &
                                           P_l3, rhoei_l3, &
                                           rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, &
                                           P_r3, rhoei_r3
logical, dimension( &
                model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: energy_switch
logical, dimension( &
                model%m3-model%nghost3:&
                model%n3+model%nghost3,&
                model%m2:model%n2, &
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: energy_switch3
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: phib_grav
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3+1,&
                model%m2:model%n2, &
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: phib_grav3
real, dimension(model%m2:model%n2, &
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: Hp1, rhoFac1
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2)      :: Hp3, rhoFac3
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2     ,&
                model%m3:model%n3)      :: flux1_rho, flux1_rhov1, flux1_rhov2, flux1_rhov3, &
                                           flux1_Bc1, flux1_Bc2, flux1_Bc3, &
                                           flux1_rhoeikm, flux1_rhoei, flux1_rhoek, flux1_rhoem, fep1, &
                                           flux1_rhov1B, flux1_rhov2B, flux1_rhov3B, &
                                           source1Bc1_l, source1Bc2_l, source1Bc3_l, &
                                           source1Bc1_r, source1Bc2_r, source1Bc3_r
real, dimension(model%m3:model%n3+1, &
                model%m2:model%n2  , &
                model%m1:model%n1)      :: flux3_rho, flux3_rhov1, flux3_rhov2, flux3_rhov3, &
                                           flux3_Bc1, flux3_Bc2, flux3_Bc3, &
                                           flux3_rhoeikm, flux3_rhoei, flux3_rhoek, flux3_rhoem, fep3, &
                                           flux3_rhov1B, flux3_rhov2B, flux3_rhov3B, &
                                           source3Bc1_l, source3Bc2_l, source3Bc3_l, &
                                           source3Bc1_r, source3Bc2_r, source3Bc3_r
real, dimension(model%m1:model%n1)      :: dtimeoverdx1
real, dimension(model%m3:model%n3)      :: dtimeoverdx3
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2,     &
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: dPdei, dPdrho, T, dTdei, s
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3,&
                model%m2:model%n2     ,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: dPdei3, dPdrho3, T3, dTdei3, s3
real, parameter                         :: epsilon=0.000001
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL2D:')
!#
# endif
!#
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1     =model%m1
n1     =model%n1
m2     =model%m2
n2     =model%n2
m3     =model%m3
n3     =model%n3
nghost1=model%nghost1
nghost3=model%nghost3
!
! --- Output of certain averages ---
if (present(output)) then
  Output_flag=output
else
  Output_flag=.false.
endif
OutRadialC_flag=.true.
!
call tabinter_getinfo(x2min=eiTable_min)
!
! --- Create box for sweep in x3 direction ---
call rhd_box_Make(model_3, m3, n3, m2, n2, m1, n1, nghost3, 0, nghost1, Bc_flag=.true.)
!
! --- Copy model (axes only) ---
call rhd_box_Copy(model  , m1-nghost1, n1+nghost1, m2, n2, m3-nghost3, n3+nghost3, &
                  model_3, m3-nghost3, m2, m1-nghost1, &
                  trans_mode=3, grid_mode=1, bulk_mode=0, flux_mode=0)
!
! === Boundary conditions I ========================================================================
!
! --- Look for which dimension boundary conditions have to be applied ---
bound_l1=action%bound_l1
bound_r1=action%bound_r1
bound_l3=action%bound_l3
bound_r3=action%bound_r3
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL2D: grav')
!#
# endif
!#
! === Gravitational potential ======================================================================
call rhd_grav(model  , rhd_action_New(action, dim=1), grav_flag, phic_grav , phib_grav )
call rhd_grav(model_3, rhd_action_New(action, dim=3, dimV=((/3, 2, 1/))), &
                                                      grav_flag, phic_grav3, phib_grav3)
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL2D: grav')
call timing_start('MHD: HLL2D: boundaries: phi')
!#
# endif
!#
!
!
! === Boundary conditions II =======================================================================
!
! --- Boundary conditions for gravity: potential set constant in ghost zones ---
if (grav_flag) then
  !
  ! --- x1 direction ---
  if (.not. ((bound_l1 == bound_periodic        ) .or. &
             (bound_l1 == bound_transmitting    ) .or. &
             (bound_l1 == bound_transmitting2   ) .or. &
             (bound_l1 == bound_inoutflow2      ) .or. &
             (bound_l1 == bound_inoutflow3      ) .or. &
             (bound_l1 == bound_closedbottom2   ) .or. &
             (bound_l1 == bound_InternalNext    ) .or. &
             (bound_l1 == bound_InternalPeriodic)) .and. m1 == prop%m1) then
    do i1=m1-nghost1,m1-1
      phic_grav( i1,:,:)=phic_grav( m1,:,:)
      phic_grav3(:,:,i1)=phic_grav3(:,:,m1)
      phib_grav3(:,:,i1)=phib_grav3(:,:,m1)
    end do ! i1
    do i1=m1-nghost1,m1
      phib_grav(i1,:,:)=phic_grav(m1,:,:)
    end do ! i1
  endif
  if (.not. ((bound_r1 == bound_periodic        ) .or. &
             (bound_r1 == bound_transmitting    ) .or. &
             (bound_r1 == bound_transmitting2   ) .or. &
             (bound_r1 == bound_InternalNext    ) .or. &
             (bound_r1 == bound_InternalPeriodic)) .and. n1 == prop%n1) then
    do i1=n1+1,n1+nghost1
      phic_grav( i1,:,:)=phic_grav( n1,:,:)
      phic_grav3(:,:,i1)=phic_grav3(:,:,n1)
      phib_grav3(:,:,i1)=phib_grav3(:,:,n1)
    end do ! i1
    do i1=n1+1,n1+nghost1+1
      phib_grav(i1,:,:)=phic_grav(n1,:,:)
    end do ! i1
  endif
  !
  ! --- x3 direction ---
  if (.not. ((bound_l3 == bound_periodic        ) .or. &
             (bound_l3 == bound_transmitting    ) .or. &
             (bound_l3 == bound_transmitting2   ) .or. &
             (bound_l3 == bound_inoutflow2      ) .or. &
             (bound_l3 == bound_inoutflow3      ) .or. &
             (bound_l3 == bound_closedbottom2   ) .or. &
             (bound_l3 == bound_InternalNext    ) .or. &
             (bound_l3 == bound_InternalPeriodic)) .and. m3 == prop%m3) then
    do i3=m3-nghost3,m3-1
      phic_grav3(i3,:,:)=phic_grav3(m3,:,:)
      phic_grav( :,:,i3)=phic_grav( :,:,m3)
      phib_grav( :,:,i3)=phib_grav( :,:,m3)
    end do ! i3
    do i3=m3-nghost3,m3
      phib_grav3(i3,:,:)=phic_grav3(m3,:,:)
    end do ! i3
  endif
  if (.not. ((bound_r3 == bound_periodic        ) .or. &
             (bound_r3 == bound_transmitting    ) .or. &
             (bound_r3 == bound_transmitting2   ) .or. &
             (bound_r3 == bound_InternalNext    ) .or. &
             (bound_r3 == bound_InternalPeriodic)) .and. n3 == prop%n3) then
    do i3=n3+1,n3+nghost3
      phic_grav3(i3,:,:)=phic_grav3(n3,:,:)
      phic_grav( :,:,i3)=phic_grav( :,:,n3)
      phib_grav( :,:,i3)=phib_grav( :,:,n3)
    end do ! i3
    do i3=n3+1,n3+nghost3+1
      phib_grav3(i3,:,:)=phic_grav3(n3,:,:)
    end do ! i3
  endif
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL2D: boundaries: phi')
!#
# endif
!#
!
!
! === EOS, further quantities, check Courant condition =============================================
!
! --- Solve EOS ---
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL2D: eosinter')
!#
# endif
!#
if (model%nquc > 0) then
  T_flag=(Output_flag .or. any(prop%quc_advect == 2))
else
  T_flag=Output_flag
endif
if (T_flag) then
  call eosinter_all(model%rho, model%ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
else
  call eosinter_PdP(model%rho, model%ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei)
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop( 'MHD: HLL2D: eosinter')
call timing_start( 'MHD: HLL2D: boundaries: transmitting')
!#
# endif
!#
! === Boundary conditions III ======================================================================
!
! --- transmitting boundary conditions ---
if (grav_flag) then
  !
  if ((bound_l1 == bound_transmitting .or. bound_l1 == bound_transmitting2) .and. &
      (m1-nghost1 < prop%m1)) then
    !
    m10 = prop%m1
    !
    do i3=m3-nghost3,n3+nghost3
      Hp1(m2,i3)=P(m10,m2,i3) / (abs(phib_grav(m10+1,m2,i3)-phib_grav(m10,m2,i3)) / &
                    (model%xb1(m10+1)-model%xb1(m10)) * model%rho(m10,m2,i3))
    end do ! i3
    !
    ! \label{f90:c_hptopfactor__useMHD1}
    if (par%C_Hptopfactor < 0.0) then
      rhoFac1=exp(-abs(                  (model%xc1(m10+1)-model%xc1(m10))/Hp1))
    else
      rhoFac1=exp(-abs(par%C_Hptopfactor*(model%xc1(m10+1)-model%xc1(m10))/Hp1))
    endif
    !
!!$    do i1 = m1-nghost1, m10-1
    do i1 =  m10-1, m1-nghost1, -1
      model%rho(i1,:,:) = model%rho(i1+1,:,:) * rhoFac1
    end do ! i1
    !
    ! \label{f90:rho_min__useMHD1}
    if (par%rho_min > 0.0) then
     do i1=m1-nghost1, m10-1
        model%rho(i1,:,:)=max(model%rho(i1,:,:), par%rho_min)
      end do ! i1
    endif
    !
    call eosinter_PdP(model%rho(m1-nghost1:m10-1,:,:), model%ei(m1-nghost1:m10-1,:,:), &
                      P=P(m1-nghost1:m10-1,:,:), dPdrho=dPdrho(m1-nghost1:m10-1,:,:), &
                                                 dPde  =dPdei( m1-nghost1:m10-1,:,:))
    !
  endif
  !
  if ((bound_r1 == bound_transmitting .or. bound_r1 == bound_transmitting2) .and. &
      (n1+nghost1 > prop%n1)) then
    !
    n10 = prop%n1
    !
    do i3=m3-nghost3,n3+nghost3
      Hp1(m2,i3)=P(n10,m2,i3) / (abs(phib_grav(n10+1,m2,i3)-phib_grav(n10,m2,i3)) / &
                    (model%xb1(n10+1)-model%xb1(n10)) * model%rho(n10,m2,i3))
    end do ! i3
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac1=exp(-abs(                  (model%xc1(n10+1)-model%xc1(n10))/Hp1))
    else
      rhoFac1=exp(-abs(par%C_Hptopfactor*(model%xc1(n10+1)-model%xc1(n10))/Hp1))
    endif
    !
    do i1 = n10+1, n1+nghost1
      model%rho(i1,:,:) = model%rho(i1-1,:,:) * rhoFac1
    end do ! i1
    !
    if (par%rho_min > 0.0) then
     do i1=n10+1, n1+nghost1
        model%rho(i1,:,:)=max(model%rho(i1,:,:), par%rho_min)
      end do ! i1
    endif
    !
    call eosinter_PdP(model%rho(n10+1:n1+nghost1,:,:), model%ei(n10+1:n1+nghost1,:,:), &
                      P=P(n10+1:n1+nghost1,:,:), dPdrho=dPdrho(n10+1:n1+nghost1,:,:), &
                                                 dPde  =dPdei( n10+1:n1+nghost1,:,:))
    !
  endif
  !
  if ((bound_l3 == bound_transmitting .or. bound_l3 == bound_transmitting2) .and. &
      (m3-nghost3 < prop%m3)) then
    !
    m30 = prop%m3
    !
    do i1=m1-nghost1,n1+nghost1
      Hp3(i1,m2)=P(i1,m2,m30) / (abs(phib_grav3(m30+1,m2,i1)-phib_grav3(m30,m2,i1)) / &
                    (model%xb3(m30+1)-model%xb3(m30)) * model%rho(i1,m2,m30))
    end do ! i1
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac3=exp(-abs(                  (model%xc3(m30+1)-model%xc3(m30))/Hp3))
    else
      rhoFac3=exp(-abs(par%C_Hptopfactor*(model%xc3(m30+1)-model%xc3(m30))/Hp3))
    endif
    !
!!$    do i3 = m3-nghost3, m30-1
    do i3 = m30-1, m3-nghost3, -1
      model%rho(:,:,i3) = model%rho(:,:,i3+1) * rhoFac3
    end do ! i3
    !
    if (par%rho_min > 0.0) then
     do i3=m3-nghost3, m30-1
        model%rho(:,:,i3)=max(model%rho(:,:,i3), par%rho_min)
      end do ! i3
    endif
    !
    call eosinter_PdP(model%rho(:,:,m3-nghost3:m30-1), model%ei(:,:,m3-nghost3:m30-1), &
                      P=P(:,:,m3-nghost3:m30-1), dPdrho=dPdrho(:,:,m3-nghost3:m30-1), &
                                                 dPde  =dPdei( :,:,m3-nghost3:m30-1))
    !
  endif
  !
  if ((bound_r3 == bound_transmitting .or. bound_r3 == bound_transmitting2) .and. &
      (n3+nghost3 > prop%n3)) then
    !
    n30 = prop%n3
    !
    do i1=m1-nghost1,n1+nghost1
      Hp3(i1,m2)=P(i1,m2,n30) / (abs(phib_grav3(n30+1,m2,i1)-phib_grav3(n30,m2,i1)) / &
                    (model%xb3(n30+1)-model%xb3(n30)) * model%rho(i1,m2,n30))
    end do ! i1
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac3=exp(-abs(                  (model%xc3(n30+1)-model%xc3(n30))/Hp3))
    else
      rhoFac3=exp(-abs(par%C_Hptopfactor*(model%xc3(n30+1)-model%xc3(n30))/Hp3))
    endif
    !
    do i3 = n30+1, n3+nghost3
      model%rho(:,:,i3) = model%rho(:,:,i3-1) * rhoFac3
    end do ! i3
    !
    if (par%rho_min > 0.0) then
     do i3=n30+1, n3+nghost3
        model%rho(:,:,i3)=max(model%rho(:,:,i3), par%rho_min)
      end do ! i3
    endif
    !
    call eosinter_PdP(model%rho(:,:,n30+1:n3+nghost3), model%ei(:,:,n30+1:n3+nghost3), &
                      P=P(:,:,n30+1:n3+nghost3), dPdrho=dPdrho(:,:,n30+1:n3+nghost3), &
                                                 dPde  =dPdei( :,:,n30+1:n3+nghost3))
    !
  endif
  !
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop( 'MHD: HLL2D: boundaries: transmitting')
!#
# endif
!#
!
Gamma_1=dPdrho*model%rho/P + dPdei/model%rho
P_mag=0.5*(model%Bc1**2+model%Bc2**2+model%Bc3**2)
P_tot=P+P_mag
!
if (par%beta_inv < 0.0 .or. par%A%hdScheme == hdScheme_RoeMHD) then
  energy_switch = .true.
else
  energy_switch=(P_mag/P < par%beta_inv)
endif
!
P3(:,m2,:)=transpose(P(:,m2,:))
Gamma_13(:,m2,:)=transpose(Gamma_1(:,m2,:))
energy_switch3(:,m2,:)=transpose(energy_switch(:,m2,:))
!
if (Output_flag) then
  dPdrho3(:,m2,:)=transpose(dPdrho(:,m2,:))
  dPdei3(:,m2,:)=transpose(dPdei(:,m2,:))
  T3(:,m2,:)=transpose(T(:,m2,:))
  dTdei3(:,m2,:)=transpose(dTdei(:,m2,:))
  s3(:,m2,:)=transpose(s(:,m2,:))
endif
!
call rhd_box_Copy(model  , m1-nghost1, n1+nghost1, m2, n2, m3-nghost3, n3+nghost3, &
                  model_3, m3-nghost3, m2, m1-nghost1, &
                  trans_mode=3, grid_mode=0, bulk_mode=1, flux_mode=0)
!
! --- Save original values into local copies ---
xc1=model%xc1
xb1=model%xb1
xc3=model%xc3
xb3=model%xb3
!
rho=model%rho
ei =model%ei
v1 =model%v1
v2 =model%v2
v3 =model%v3
Bc1=model%Bc1
Bc2=model%Bc2
Bc3=model%Bc3
!
rho3=model_3%rho
ei3 =model_3%ei
v13 =model_3%v3
v23 =model_3%v2
v33 =model_3%v1
Bc13=model_3%Bc3
Bc23=model_3%Bc2
Bc33=model_3%Bc1
!
model%energy_switch=energy_switch
!
! --- Check positivity of quantities ---
if (any(P <= 0.0)) then
  ierr0=4
  error_loc=minloc(P)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+(m3-nghost3)
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') P(  error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Pressure non-positive (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif ! (any(P <= 0.0))
if (any(Gamma_1 < 1.0)) then
  ierr0=5
  error_loc=minloc(Gamma_1)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+(m3-nghost3)
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Gamma_1 smaller than 1 (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif ! (any(Gamma_1 < 1.0))
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL2D: derived q')
!#
# endif
!#
dtime_limit=0.1*huge(dtime_limit)
!
do i3=m3-nghost3,n3+nghost3
  do i2=m2,n2
    do i1=m1-nghost1,n1+nghost1
      ! --- Compute derived quantities: sound speed,            ---
      ! --- speeds of magnetic waves, maximum allowed time step ---
      !
      cs(      i1,i2,i3)=sqrt(P(i1,i2,i3)/rho(i1,i2,i3)*Gamma_1(i1,i2,i3))
      sqrt_rho          =sqrt(rho(i1,i2,i3))
      va1(     i1,i2,i3)=abs(Bc1(i1,i2,i3))/sqrt_rho
      va3(     i1,i2,i3)=abs(Bc3(i1,i2,i3))/sqrt_rho
      !
      ! \label{f90:va_max__useMHD2}
      if ((par%va_max > 0.0) .and. &
          ((par%A%hdScheme == hdScheme_HLLMHD) .or. (par%A%hdScheme == hdScheme_CenMHD))) then
        f(    i1,i2,i3) =par%va_max**2/(par%va_max**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3))
        va1(  i1,i2,i3) =sqrt(f(i1,i2,i3))*va1(i1,i2,i3)
        va3(  i1,i2,i3) =sqrt(f(i1,i2,i3))*va3(i1,i2,i3)
        P_mag(i1,i2,i3) =f(i1,i2,i3)*P_mag(i1,i2,i3)
        P_tot(i1,i2,i3) =P(i1,i2,i3)+P_mag(i1,i2,i3)
      else
        f(i1,i2,i3)=1.0
      endif
      !
      B_sq_tan          =Bc2(i1,i2,i3)**2+Bc3(i1,i2,i3)**2
      !
      if ((B_sq_tan * f(i1,i2,i3)) > (epsilon * P_tot(i1,i2,i3))) then
        caq             =cs(i1,i2,i3)**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd              =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va1(i1,i2,i3)/caq)**2))
        vf1(   i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs1(   i1,i2,i3)=cs(i1,i2,i3)*va1(i1,i2,i3) * sqrt(2.0 / (caq+dd))
      else
        if      ((cs(i1,i2,i3) - va1(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf1(i1,i2,i3) = cs( i1,i2,i3)
          vs1(i1,i2,i3) = va1(i1,i2,i3)
        else if ((va1(i1,i2,i3) - cs(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf1(i1,i2,i3) = va1(i1,i2,i3)
          vs1(i1,i2,i3) = cs( i1,i2,i3)
        else
          vf1(i1,i2,i3) = cs( i1,i2,i3)
          vs1(i1,i2,i3) = cs( i1,i2,i3)
        endif
      endif
      !
      B_sq_tan          =Bc2(i1,i2,i3)**2+Bc1(i1,i2,i3)**2
      !
      if ((B_sq_tan * f(i1,i2,i3)) > (epsilon * P_tot(i1,i2,i3))) then
        caq             =cs(i1,i2,i3)**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd              =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va3(i1,i2,i3)/caq)**2))
        vf3(   i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs3(   i1,i2,i3)=cs(i1,i2,i3)*va3(i1,i2,i3) * sqrt(2.0 / (caq+dd))
      else
        if      ((cs(i1,i2,i3) - va3(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf3(i1,i2,i3) = cs( i1,i2,i3)
          vs3(i1,i2,i3) = va3(i1,i2,i3)
        else if ((va3(i1,i2,i3) - cs(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf3(i1,i2,i3) = va3(i1,i2,i3)
          vs3(i1,i2,i3) = cs( i1,i2,i3)
        else
          vf3(i1,i2,i3) = cs( i1,i2,i3)
          vs3(i1,i2,i3) = cs( i1,i2,i3)
        endif
      endif
      !
      dtime_limit       =min(dtime_limit, &
                             1.0 / ((abs(v1(i1,i2,i3))+vf1(i1,i2,i3)) / (xb1(i1+1)-xb1(i1)) + &
                                    (abs(v3(i1,i2,i3))+vf3(i1,i2,i3)) / (xb3(i3+1)-xb3(i3))    ))
      !
    end do ! i1
  end do ! i2
end do ! i3
!
f3(:,m2,:)=transpose(f(:,m2,:))
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL2D: derived q')
!#
# endif
!#
!
! --- Check Courant condition ---
!
if ((par%C_CourantMax > 0.0) .and. (dtime > par%C_CourantMax * dtime_limit)) then
  ! --- Time step too large: return without further computation ---
  ierr0=1
  outstr0='Time step too large for Courant condition'
  goto 999
endif
!
!
! --- Perform reconstruction and (optional) Hancock predictor step ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('MHD: HLL2D: Reconstruction')
!#
# endif
!#
call rhd_mhd_Reconstruct(m1,n1, m2,n2, m3,n3, nghost1, 0, nghost3, 0, 1, &
                         par%A%reconstruction_order, par%A%reconstruction_type, 1, &
                         dtime, xb1, f, &
                         rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, Gamma_1, &
                         rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                         rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1)
!
call rhd_mhd_Reconstruct(m3,n3, m2,n2, m1,n1, nghost3, 0, nghost1, 0, 1, &
                         par%A%reconstruction_order, par%A%reconstruction_type, 3, &
                         dtime, xb3, f3, &
                         rho3, ei3, v33, v23, v13, Bc33, Bc23, Bc13, P3, Gamma_13, &
                         rho_l3, v3_l3, v2_l3, v1_l3, Bc3_l3, Bc2_l3, Bc1_l3, P_l3, rhoei_l3, &
                         rho_r3, v3_r3, v2_r3, v1_r3, Bc3_r3, Bc2_r3, Bc1_r3, P_r3, rhoei_r3)
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('MHD: HLL2D: Reconstruction')
!#
# endif
!#
!
if (par%A%reconstruction_order > 0) then
  if (par%hdTimeIntegrationScheme == 'Hancock') then
    ! --- Perform unsplit 2D Hancock predictor step ---
!#
#   ifdef rhd_roe1d_step_t01
!#
    call timing_start('MHD: HLL2D: Hancock')
!#
#   endif
!#
    call rhd_mhd_Hancock2D(m1,n1, m2,n2, m3,n3, nghost1, nghost3, &
                           dtime, xb1, xb3, &
                           rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                           Gamma_1, f, phib_grav, phib_grav3, &
                           rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                           rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
                           rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, P_l3, rhoei_l3, &
                           rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, P_r3, rhoei_r3)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('MHD: HLL2D: Hancock')
!#
#   endif
!#
  endif ! (par%hdTimeIntegrationScheme == 'Hancock')
endif ! (par%A%reconstruction_order > 0)
!
! --- Add minimum internal energy. It was subtracted in rhd_mhd_Reconstruct. ---
rhoei_l1=rhoei_l1 + rho_l1*eiTable_min
rhoei_r1=rhoei_r1 + rho_r1*eiTable_min
rhoei_l3=rhoei_l3 + rho_l3*eiTable_min
rhoei_r3=rhoei_r3 + rho_r3*eiTable_min
!
!
! === Main step: compute boundary located (flux-like) averages and fluxes ==========================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL2D: fluxes')
!#
# endif
!#
!
call rhd_mhd_HLL1Dflux(m1,n1, m2,n2, m3,n3, nghost1, 0, nghost3, 0, 1, &
                       par%A%hdScheme, &
                       bound_l1, bound_r1, par%A%reconstruction_order, par%A%reconstruction_type, &
                       1, dtime, xb1, f, &
                       rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                       rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                       rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
                       Gamma_1, energy_switch, &
                       phic_grav, phib_grav, &
                       flux1_rho, flux1_rhov1, flux1_rhov2, flux1_rhov3, &
                                  flux1_Bc1  , flux1_Bc2  , flux1_Bc3  , &
                       flux1_rhoeikm, flux1_rhoei, flux1_rhoek, flux1_rhoem, fep1, &
                       flux1_rhov1B, flux1_rhov2B, flux1_rhov3B, &
                       source1Bc1_l, source1Bc2_l, source1Bc3_l, &
                       source1Bc1_r, source1Bc2_r, source1Bc3_r)
!
call rhd_mhd_HLL1Dflux(m3, n3, m2, n2, m1, n1, nghost3, 0, nghost1, 0, 1, &
                       par%A%hdScheme, &
                       bound_l3, bound_r3, par%A%reconstruction_order, par%A%reconstruction_type, &
                       3, dtime, xb3, f3, &
                       rho3, ei3, v33, v23, v13, Bc33, Bc23, Bc13, P3, &
                       rho_l3, v3_l3, v2_l3, v1_l3, Bc3_l3, Bc2_l3, Bc1_l3, P_l3, rhoei_l3, &
                       rho_r3, v3_r3, v2_r3, v1_r3, Bc3_r3, Bc2_r3, Bc1_r3, P_r3, rhoei_r3, &
                       Gamma_13, energy_switch3, &
                       phic_grav3, phib_grav3, &
                       flux3_rho, flux3_rhov3, flux3_rhov2, flux3_rhov1, &
                                  flux3_Bc3  , flux3_Bc2  , flux3_Bc1  , &
                       flux3_rhoeikm, flux3_rhoei, flux3_rhoek, flux3_rhoem, fep3, &
                       flux3_rhov3B, flux3_rhov2B, flux3_rhov1B, &
                       source3Bc3_l, source3Bc2_l, source3Bc1_l, &
                       source3Bc3_r, source3Bc2_r, source3Bc1_r)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL2D: fluxes')
!#
# endif
!#
if (Output_flag) then
  ! === Put additional output data into datx, datr, and datc =======================================
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('MHD: HLL2D: (output)')
!#
# endif
!#
  call rhd_mhd_Roe1Doutput(model, 1, AreaFraction1, OutRadialC_flag, &
                           P, T, s, Gamma_1, dPdei/rho + 1.0, dTdei, dPdei, &
                           phic_grav, phib_grav, &
                           flux1_rho, flux1_rhov1, flux1_rhov2, flux1_rhov3, &
                           flux1_rhoeikm, flux1_rhoei, flux1_rhoek, flux1_rhoem, fep1, &
                           flux1_rhov1B, flux1_rhov2B, flux1_rhov3B, &
                           datx=datx1, datr=datr, datc=datc)
  OutRadialC_flag=.false.
  !
  call rhd_mhd_Roe1Doutput(model_3, 3, AreaFraction3, OutRadialC_flag, &
                           P3, T3, s3, Gamma_13, dPdei3/rho3 + 1.0, dTdei3, dPdei3, &
                           phic_grav3, phib_grav3, &
                           flux3_rho, flux3_rhov3, flux3_rhov2, flux3_rhov1, &
                           flux3_rhoeikm, flux3_rhoei, flux3_rhoek, flux3_rhoem, fep3, &
                           flux3_rhov3B, flux3_rhov2B, flux3_rhov1B, &
                           datx=datx3, datr=datr)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: HLL2D: (output)')
!#
# endif
!#
endif
!
!
! === Update with computed fluxes and source terms =================================================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL2D: update')
!#
# endif
!#
! --- dtimeoverdx=dt/dx ---
dtimeoverdx1=dtime/(xb1(m1+1:n1+1)-xb1(m1:n1))
dtimeoverdx3=dtime/(xb3(m3+1:n3+1)-xb3(m3:n3))
!
! --- Check if density is positive everywhere ---
rhonew_min=huge(rhonew_min)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      rhonew_min=min(rhonew_min, rho(i1,i2,i3) - &
                                 dtimeoverdx1(i1) * (flux1_rho(i1+1,i2,i3)-flux1_rho(i1,i2,i3)) - &
                                 dtimeoverdx3(i3) * (flux3_rho(i3+1,i2,i1)-flux3_rho(i3,i2,i1)))

    end do ! i1
  end do ! i2
end do ! i3
if (rhonew_min <= 0.0) then
  ierr0=6
  dtime_limit=dtime
  outstr0='Density becomes non-positive'
  goto 999
endif
!
!if (model%nquc > 0) then
!  ! --- Handle additional passive tracer fields ---
!  call rhd_hyd_Dust(model, T, phic_grav, &
!                    par%A%reconstruction_order, par%A%reconstruction_quctype, &
!                    dtime, flux1_rho)
!endif
!
!
! \label{f90:c_gravmu__useMHD2}
! --- Choose time-centering of gravity source terms ---
if (par%C_gravmu < 0.0) then
  ! --- Use old defaults ---
  if (par%hdTimeIntegrationScheme == 'Hancock' .and. par%A%reconstruction_order > 0) then
    ! --- Use time-centered density ---
    wgrav_new=0.5
  else
    ! --- E.g., for Runge-Kutta use only first-order terms (density at old time step) ---
    wgrav_new=0.0
  endif
else
  ! --- Use input parameter as weight ---
  wgrav_new=par%C_gravmu
endif
!
! --- Update all quantities ---
!
do i3=m3,n3
  do i2=m2,n2
    !DIR$ IVDEP
    do i1=m1,n1
      ! --- Compute change in mass density and update it ---
      drho  =-dtimeoverdx1(i1) * (flux1_rho(  i1+1,i2,i3)-flux1_rho(  i1,i2,i3)) - &
              dtimeoverdx3(i3) * (flux3_rho(  i3+1,i2,i1)-flux3_rho(  i3,i2,i1))
      rhoout=rho(i1,i2,i3) + drho
      !
      ! --- Source terms due to gravity ---
      w01=(phic_grav(i1  ,i2,i3)-phib_grav(i1,i2,i3)) / &
          (xb1(      i1+1      )-xb1(      i1      ))
      w11=(phib_grav(i1+1,i2,i3)-phic_grav(i1,i2,i3)) / &
          (xb1(      i1+1      )-xb1(      i1      ))
      w03=(phic_grav3(i3  ,i2,i1)-phib_grav3(i3,i2,i1)) / &
          (xb3(       i3+1      )-xb3(       i3      ))
      w13=(phib_grav3(i3+1,i2,i1)-phic_grav3(i3,i2,i1)) / &
          (xb3(       i3+1      )-xb3(       i3      ))
      source_eikm =w01*flux1_rho(i1,i2,i3) + w11*flux1_rho(i1+1,i2,i3) + &
                   w03*flux3_rho(i3,i2,i1) + w13*flux3_rho(i3+1,i2,i1) + (1.0 - f(i1,i2,i3)) * &
                   (v1(i1,i2,i3) * (flux1_rhov1B(i1+1,i2,i3) - flux1_rhov1B(i1  ,i2,i3)) / &
                                   (xb1(        i1+1      ) - xb1(        i1        )) + &
                    v2(i1,i2,i3) * (flux1_rhov2B(i1+1,i2,i3) - flux1_rhov2B(i1  ,i2,i3)) / &
                                   (xb1(        i1+1      ) - xb1(        i1        )) + &
                    v3(i1,i2,i3) * (flux1_rhov3B(i1+1,i2,i3) - flux1_rhov3B(i1  ,i2,i3)) / &
                                   (xb1(        i1+1      ) - xb1(        i1        )) + &
                    v1(i1,i2,i3) * (flux3_rhov1B(i3+1,i2,i1) - flux3_rhov1B(i3  ,i2,i1)) / &
                                   (xb3(        i3+1      ) - xb3(        i3        )) + &
                    v2(i1,i2,i3) * (flux3_rhov2B(i3+1,i2,i1) - flux3_rhov2B(i3  ,i2,i1)) / &
                                   (xb3(        i3+1      ) - xb3(        i3        )) + &
                    v3(i1,i2,i3) * (flux3_rhov3B(i3+1,i2,i1) - flux3_rhov3B(i3  ,i2,i1)) / &
                                   (xb3(        i3+1      ) - xb3(        i3        )))
      source_ei   = -P(i1,i2,i3) * ((v1(i1+1,i2,i3) - v1(i1-1,i2,i3)) / (xc1(i1+1) - xc1(i1-1)) + &
                                    (v3(i1,i2,i3+1) - v3(i1,i2,i3-1)) / (xc3(i3+1) - xc3(i3-1)))
      source_rhov1=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) / &
                                                    (xb1(      i1+1      )-xb1(      i1      ))
      source_rhov3=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav3(i3+1,i2,i1)-phib_grav3(i3,i2,i1)) / &
                                                    (xb3(       i3+1      )-xb3(       i3      ))
      !
      ! --- Update ---
      !
      v1out =(rho(i1,i2,i3) * v1(i1,i2,i3) - &
                             dtimeoverdx1(i1) * ((flux1_rhov1(i1+1,i2,i3)-flux1_rhov1(i1,i2,i3)) + &
                             f(i1,i2,i3)      *  (flux1_rhov1B(i1+1,i2,i3)-flux1_rhov1B(i1,i2,i3))) - &
                             dtimeoverdx3(i3) * ((flux3_rhov1(i3+1,i2,i1)-flux3_rhov1(i3,i2,i1)) + &
                             f(i1,i2,i3)      *  (flux3_rhov1B(i3+1,i2,i1)-flux3_rhov1B(i3,i2,i1))) + &
                             dtime            * source_rhov1 &
             )/rhoout
      v2out =(rho(i1,i2,i3) * v2(i1,i2,i3) - &
                             dtimeoverdx1(i1) * ((flux1_rhov2(i1+1,i2,i3)-flux1_rhov2(i1,i2,i3)) + &
                             f(i1,i2,i3)      *  (flux1_rhov2B(i1+1,i2,i3)-flux1_rhov2B(i1,i2,i3))) - &
                             dtimeoverdx3(i3) * ((flux3_rhov2(i3+1,i2,i1)-flux3_rhov2(i3,i2,i1)) + &
                             f(i1,i2,i3)      *  (flux3_rhov2B(i3+1,i2,i1)-flux3_rhov2B(i3,i2,i1))) &
             )/rhoout
      v3out =(rho(i1,i2,i3) * v3(i1,i2,i3) - &
                             dtimeoverdx1(i1) * ((flux1_rhov3(i1+1,i2,i3)-flux1_rhov3(i1,i2,i3)) + &
                             f(i1,i2,i3)      *  (flux1_rhov3B(i1+1,i2,i3)-flux1_rhov3B(i1,i2,i3))) - &
                             dtimeoverdx3(i3) * ((flux3_rhov3(i3+1,i2,i1)-flux3_rhov3(i3,i2,i1)) + &
                             f(i1,i2,i3)      *  (flux3_rhov3B(i3+1,i2,i1)-flux3_rhov3B(i3,i2,i1))) + &
                             dtime            * source_rhov3 &
             )/rhoout
      Bc1out=Bc1(i1,i2,i3) - dtimeoverdx1(i1) * (flux1_Bc1(  i1+1,i2,i3)-flux1_Bc1(  i1,i2,i3)) + &
                             source1Bc1_l(i1+1,i2,i3) + source1Bc1_r(i1,i2,i3) - &
                             dtimeoverdx3(i3) * (flux3_Bc1(  i3+1,i2,i1)-flux3_Bc1(  i3,i2,i1)) + &
                             source3Bc1_l(i3+1,i2,i1) + source3Bc1_r(i3,i2,i1)
      Bc2out=Bc2(i1,i2,i3) - dtimeoverdx1(i1) * (flux1_Bc2(  i1+1,i2,i3)-flux1_Bc2(  i1,i2,i3)) + &
                             source1Bc2_l(i1+1,i2,i3) + source1Bc2_r(i1,i2,i3) - &
                             dtimeoverdx3(i3) * (flux3_Bc2(  i3+1,i2,i1)-flux3_Bc2(  i3,i2,i1)) + &
                             source3Bc2_l(i3+1,i2,i1) + source3Bc2_r(i3,i2,i1)
      Bc3out=Bc3(i1,i2,i3) - dtimeoverdx1(i1) * (flux1_Bc3(  i1+1,i2,i3)-flux1_Bc3(  i1,i2,i3)) + &
                             source1Bc3_l(i1+1,i2,i3) + source1Bc3_r(i1,i2,i3) - &
                             dtimeoverdx3(i3) * (flux3_Bc3(  i3+1,i2,i1)-flux3_Bc3(  i3,i2,i1)) + &
                             source3Bc3_l(i3+1,i2,i1) + source3Bc3_r(i3,i2,i1)
      !
      if (model%energy_switch(i1,i2,i3)) then
        !
        ! --- Equation for total energy ---
        eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) + &
                0.5 * rho(i1,i2,i3) * (v1(i1,i2,i3)**2 + v2(i1,i2,i3)**2 + v3(i1,i2,i3)**2) - &
                dtimeoverdx1(i1) * (flux1_rhoeikm(i1+1,i2,i3)-flux1_rhoeikm(i1,i2,i3)) - &
                dtimeoverdx3(i3) * (flux3_rhoeikm(i3+1,i2,i1)-flux3_rhoeikm(i3,i2,i1)) + &
                0.5 * (Bc1(i1,i2,i3) + Bc1out) * &
                              (dtimeoverdx1(i1) * (flux1_Bc1(  i1+1,i2,i3)-flux1_Bc1(  i1,i2,i3)) - &
                               source1Bc1_l(i1+1,i2,i3) - source1Bc1_r(i1,i2,i3) + &
                               dtimeoverdx3(i3) * (flux3_Bc1(  i3+1,i2,i1)-flux3_Bc1(  i3,i2,i1)) - &
                               source3Bc1_l(i3+1,i2,i1) - source3Bc1_r(i3,i2,i1)) + &
                0.5 * (Bc2(i1,i2,i3) + Bc2out) * &
                              (dtimeoverdx1(i1) * (flux1_Bc2(  i1+1,i2,i3)-flux1_Bc2(  i1,i2,i3)) - &
                               source1Bc2_l(i1+1,i2,i3) - source1Bc2_r(i1,i2,i3) + &
                               dtimeoverdx3(i3) * (flux3_Bc2(  i3+1,i2,i1)-flux3_Bc2(  i3,i2,i1)) - &
                               source3Bc2_l(i3+1,i2,i1) - source3Bc2_r(i3,i2,i1)) + &
                0.5 * (Bc3(i1,i2,i3) + Bc3out) * &
                              (dtimeoverdx1(i1) * (flux1_Bc3(  i1+1,i2,i3)-flux1_Bc3(  i1,i2,i3)) - &
                               source1Bc3_l(i1+1,i2,i3) - source1Bc3_r(i1,i2,i3) + &
                               dtimeoverdx3(i3) * (flux3_Bc3(  i3+1,i2,i1)-flux3_Bc3(  i3,i2,i1)) - &
                               source3Bc3_l(i3+1,i2,i1) - source3Bc3_r(i3,i2,i1)) + &
                dtime           * source_eikm &
               )/rhoout - 0.5 * (v1out**2 + v2out**2 + v3out**2)
      else
        !
        ! --- Equation for thermal energy (without any dissipation terms) ---
        eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) - &
                              dtimeoverdx1(i1) * (flux1_rhoei(i1+1,i2,i3) -flux1_rhoei(i1,i2,i3)) - &
                              dtimeoverdx3(i3) * (flux3_rhoei(i3+1,i2,i1) -flux3_rhoei(i3,i2,i1)) + &
                              dtime           * source_ei) / rhoout
      endif
      !
      model%rho(i1,i2,i3)=rhoout
      model%v1( i1,i2,i3)=v1out
      model%v2( i1,i2,i3)=v2out
      model%v3( i1,i2,i3)=v3out
      model%Bc1(i1,i2,i3)=Bc1out
      model%Bc2(i1,i2,i3)=Bc2out
      model%Bc3(i1,i2,i3)=Bc3out
      model%ei( i1,i2,i3)=eiout
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Save fluxes for flux-interpolated transport step ---
do i3 = m3, n3
  do i2 = m2, n2
    do i1 = m1, n1 + 1
      model%flux_B31(i1, i2, i3) = flux1_Bc3(i1, i2, i3)
    end do ! i1
  end do ! i2
end do ! i3
!
do i3 = m3, n3 + 1
  do i2 = m2, n2
    do i1 = m1, n1
      model%flux_B13(i1, i2, i3) = flux3_Bc1(i3, i2, i1)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Check if internal energy is below minimum table value anywhere ---
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*dtime
  outstr0='Internal energy below minimum table value'
  goto 998
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL2D: update')
!#
# endif
!#
!
! --- Check internal energy ---
if (par%C_maxeiChange > 0.0) then
  dei_min=min(minval((model%ei(m1:n1,:,:)-ei(m1:n1,:,:))*rho(m1:n1,:,:)/P(m1:n1,:,:)),0.0)
  dtime_limit=min(dtime_limit, &
                  dtime*par%C_maxeiChange/(-dei_min+10.*tiny(dei_min))*0.999)
  if (dei_min < -par%C_maxeiChange) then
    ierr0=2
    outstr0='Drop in internal energy too large'
    goto 999
  endif
endif
!
! === Boundary conditions VI =======================================================================
if (bound_l1 == bound_transmitting2 .and. m1 == prop%m1) then
  model%v1(m1,m2:n2,m3:n3)=v1(m1,m2:n2,m3:n3) + dtime*cs(m1,m2:n2,m3:n3) * &
                                         (       v1(m1+1,m2:n2,m3:n3)-       v1(m1,m2:n2,m3:n3))/ &
                                         (model%xc1(m1+1            )-model%xc1(m1            ))
endif
if (bound_r1 == bound_transmitting2 .and. n1 == prop%n1) then
  model%v1(n1,m2:n2,m3:n3)=v1(n1,m2:n2,m3:n3) - dtime*cs(n1,m2:n2,m3:n3) * &
                                         (       v1(n1-1,m2:n2,m3:n3)-       v1(n1,m2:n2,m3:n3))/ &
                                         (model%xc1(n1-1            )-model%xc1(n1            ))
endif
if (bound_l3 == bound_transmitting2 .and. m3 == prop%m3) then
  model%v3(m1:n1,m2:n2,m3)=v3(m1:n1,m2:n2,m3) + dtime*cs(m1:n1,m2:n2,m3) * &
                                         (       v3(m1:n1,m2:n2,m3+1)-       v3(m1:n1,m2:n2,m3))/ &
                                         (model%xc3(            m3+1)-model%xc3(            m3))
endif
if (bound_r3 == bound_transmitting2 .and. n3 == prop%n3) then
  model%v3(m1:n1,m2:n2,n3)=v3(m1:n1,m2:n2,n3) - dtime*cs(m1:n1,m2:n2,n3) * &
                                         (       v3(m1:n1,m2:n2,n3-1)-       v3(m1:n1,m2:n2,n3))/ &
                                         (model%xc3(            n3-1)-model%xc3(            n3))
endif
!
!
999 dtime_limit=par%C_Courant*dtime_limit
!
998 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL2D:')
!#
# endif
!#
!
! --- Delete box for x3 sweep ---
call rhd_box_Delete(model_3)
!
end subroutine rhd_mhd_HLL2Dstep


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_HLL3Dstep(model, action, dtime, dtime_limit, &
                             output, AreaFraction1, AreaFraction2, AreaFraction3, &
                             datx1, datx2, datx3, datr, datc, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_HLL3Dstep ('rhd_magneto-hydrodynamics_HLL_3D_step')
!
! PURPOSE:
!   Perform 3D unsplit magnetohydrodynamics step, employ HLL-Hancock scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_HLL3Dstep(model, action, dtime, dtime_limit, &
!                          output=output, AreaFraction1=AreaFraction1, &
!                          AreaFraction2=AreaFraction2, AreaFraction3=AreaFraction3, &
!                          datx1=datx1, datx2=datx2, datx3=datx3, datr=datr, datc=datc, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!   AreaFraction1: ('Area_Fraction1') real, optional, weighting factor for the current model slice
!   AreaFraction2: ('Area_Fraction2') real, optional, weighting factor for the current model slice
!   AreaFraction3: ('Area_Fraction3') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   datx1:        ('data_x1') type(datx_type), optional, structure for additional quantities
!   datx2:        ('data_x2') type(datx_type), optional, structure for additional quantities
!   datx3:        ('data_x3') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for
!                 additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!                   100:   Pressure becomes non-positive
!                   101:   Pressure derivative dPdrho|ei becomes non-positive
!                   102:   Pressure derivative dPdei|rho becomes non-positive
!                   103:   Density becomes non-positive
!
! LOCAL VARIABLES:
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                    volume averaging of cell centered quantities is performed
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_grav:              Compute gravitational potential
!   rhd_mhd_Hancock3D:     Perform Hancock predictor step in 3 dimensions
!   rhd_mhd_HLL1Dflux:     Compute 1D MHD HLL fluxes
!   rhd_mhd_Reconstruct:   Reconstruct quantities in cells and compute boundary values
!   rhd_mhd_Roe1Doutput:   Extract some averaged quantities
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! KEYWORDS:
!   equations of 3D compressible hydrodynamics
!   unsplit scheme: 3D substep
!   HLL solver, Hancock scheme
!   higher order by slope-limiters, reconstruction schemes: constant/Minmod,vanLeer,Superbee/PP
!   general EOS (tabulated values)
!   non-equidistant grid possible
!   source terms due to gravity: no operator-splitting here (constant gravity or central potential)
!
! PROCEDURE:
!   1. Preliminaries: compute gravitational potential, fill ghost cells, apply boundary conditions
!   2. Solve EOS and compute further quantities from old values on extended grid
!   3. Reconstruct quantities within the cells
!   4. Perform Hancock predictor step
!   5. Compute averages and fluxes located at cell boundaries
!   6. Update basic quantities with fluxes and source terms
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-03-23 (W.S.) First version
!   2004-10-26 (W.S.) Update of internal energy modified
!   2008-03-24 (W.S. East Lansing) Dual energy calculation
!   2010-02-16 (W.S. Freiburg) c_test2 -> va_max
!   2010-04-10 (W.S. Freiburg) Alfven speed reduction not only for beta_inv = 0.0
!   2010-06-11 (B.F. Lyon) Higher-order terms (in time) in source term for rhov1 removed
!   2011-06-10 (W.S.) Derived from routine rhd_mhd_HLL1D
!   2013-03-02 (B.F.) Change loop order for transmitting boundaries;
!                     subdivide long 'derived q' loop;
!                     update loop: use variable grav_factorweight
!   2013-03-05 (B.F.) Parameter outradialc becomes local variable; sqrt_rho becomes scalar
!   2013-03-07 (B.F.) Account for "internal" MPI boundary conditions for gravitatioal potential
!   2013-03-09 (B.F.) Some rearrangements
!   2013-06-10 (B.F.) Request only time-step reduction (no force stop) for small Gamma_1 or P
!   2014-07-15 (B.F.) grav_factor -> wgrav_new, account for parameter C_gravmu
!   2014-08-17 (B.F.) Rename: model2 -> model_2, model3 -> model_3
!   2018-10-18 (B.F.) Some cleanup. Add eiTable_min for all reconstruction orders
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_phys_module
!#
# ifdef rhd_roe1d_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(datx_type), optional, intent(inout):: datx1, datx2, datx3, datr
type(datc_type), optional, intent(inout):: datc
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
type(box_type)                          :: model_2, model_3
logical                                 :: grav_flag, Output_flag, T_flag, OutRadialC_flag
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3
integer                                 :: i1, i2, i3, m10, n10, m20, n20, m30, n30
integer                                 :: bound_l1, bound_r1, bound_l2, bound_r2, bound_l3, bound_r3
real                                    :: dei_min, &
                                           rhonew_min, rhoout, eiout, v1out, v2out, v3out, &
                                           Bc1out, Bc2out, Bc3out, w01, w11, w02, w12, w03, w13, &
                                           einew_min, eiTable_min, Pg, Pm, ei_min, &
                                           source_rhov1, source_rhov2, source_rhov3, &
                                           source_eikm, source_ei, &
                                           sqrt_rho, &
                                           caq, dd, B_sq_tan, dt, drho, wgrav_new
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, dimension(3)                   :: error_loc
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1) :: xc1
real, dimension( &
            model%m1-model%nghost1:&
            model%n1+model%nghost1+1)   :: xb1
real, dimension(model%m2-model%nghost2:&
                model%n2+model%nghost2) :: xc2
real, dimension( &
            model%m2-model%nghost2:&
            model%n2+model%nghost2+1)   :: xb2
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3) :: xc3
real, dimension( &
            model%m3-model%nghost3:&
            model%n3+model%nghost3+1)   :: xb3
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, &
                                           P, P_mag, P_tot, &
                                           cs, va1, va2, va3, vf1, vf2, vf3, vs1, vs2, vs3, &
                                           Gamma_1, phic_grav, f
real, dimension(model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: rho2, ei2, v12, v22, v32, Bc12, Bc22, Bc32, &
                                           P2, Gamma_12, phic_grav2, f2
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: rho3, ei3, v13, v23, v33, Bc13, Bc23, Bc33, &
                                           P3, Gamma_13, phic_grav3, f3
real, dimension(model%m1-1:model%n1+1 ,&
                model%m2-1:model%n2+1, &
                model%m3-1:model%n3+1)  :: rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, &
                                           P_l1, rhoei_l1, &
                                           rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, &
                                           P_r1, rhoei_r1
real, dimension(model%m2-1:model%n2+1 ,&
                model%m1-1:model%n1+1 ,&
                model%m3-1:model%n3+1)  :: rho_l2, v1_l2, v2_l2, v3_l2, Bc1_l2, Bc2_l2, Bc3_l2, &
                                           P_l2, rhoei_l2, &
                                           rho_r2, v1_r2, v2_r2, v3_r2, Bc1_r2, Bc2_r2, Bc3_r2, &
                                           P_r2, rhoei_r2
real, dimension(model%m3-1:model%n3+1 ,&
                model%m2-1:model%n2+1, &
                model%m1-1:model%n1+1)  :: rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, &
                                           P_l3, rhoei_l3, &
                                           rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, &
                                           P_r3, rhoei_r3
logical, dimension( &
                model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: energy_switch
logical, dimension( &
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: energy_switch2
logical, dimension( &
                model%m3-model%nghost3:&
                model%n3+model%nghost3,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: energy_switch3
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: phib_grav
real, dimension(model%m2-model%nghost2:&
                model%n2+model%nghost2+1,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: phib_grav2
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3+1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: phib_grav3
real, dimension(model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: Hp1, rhoFac1
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: Hp2, rhoFac2
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2) :: Hp3, rhoFac3
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2,&
                model%m3:model%n3)      :: flux1_rho, flux1_rhov1, flux1_rhov2, flux1_rhov3, &
                                           flux1_Bc1, flux1_Bc2, flux1_Bc3, &
                                           flux1_rhoeikm, flux1_rhoei, flux1_rhoek, flux1_rhoem, &
                                           fep1, &
                                           flux1_rhov1B, flux1_rhov2B, flux1_rhov3B, &
                                           source1Bc1_l, source1Bc2_l, source1Bc3_l, &
                                           source1Bc1_r, source1Bc2_r, source1Bc3_r
real, dimension(model%m2:model%n2+1, &
                model%m1:model%n1,&
                model%m3:model%n3)      :: flux2_rho, flux2_rhov1, flux2_rhov2, flux2_rhov3, &
                                           flux2_Bc1, flux2_Bc2, flux2_Bc3, &
                                           flux2_rhoeikm, flux2_rhoei, flux2_rhoek, flux2_rhoem, &
                                           fep2, &
                                           flux2_rhov1B, flux2_rhov2B, flux2_rhov3B, &
                                           source2Bc1_l, source2Bc2_l, source2Bc3_l, &
                                           source2Bc1_r, source2Bc2_r, source2Bc3_r
real, dimension(model%m3:model%n3+1, &
                model%m2:model%n2,&
                model%m1:model%n1) ::      flux3_rho, flux3_rhov1, flux3_rhov2, flux3_rhov3, &
                                           flux3_Bc1, flux3_Bc2, flux3_Bc3, &
                                           flux3_rhoeikm, flux3_rhoei, flux3_rhoek, flux3_rhoem, &
                                           fep3, &
                                           flux3_rhov1B, flux3_rhov2B, flux3_rhov3B, &
                                           source3Bc1_l, source3Bc2_l, source3Bc3_l, &
                                           source3Bc1_r, source3Bc2_r, source3Bc3_r
real, dimension(model%m1:model%n1)      :: dtimeoverdx1
real, dimension(model%m2:model%n2)      :: dtimeoverdx2
real, dimension(model%m3:model%n3)      :: dtimeoverdx3
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: dPdei, dPdrho, T, dTdei, s
real, dimension(model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m3-model%nghost3:&
                model%n3+model%nghost3) :: dPdei2, dPdrho2, T2, dTdei2, s2
real, dimension(model%m3-model%nghost3:&
                model%n3+model%nghost3,&
                model%m2-model%nghost2:&
                model%n2+model%nghost2,&
                model%m1-model%nghost1:&
                model%n1+model%nghost1) :: dPdei3, dPdrho3, T3, dTdei3, s3
real, parameter                         :: epsilon=0.000001
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL3D:')
!#
# endif
!#
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1     =model%m1
n1     =model%n1
m2     =model%m2
n2     =model%n2
m3     =model%m3
n3     =model%n3
nghost1=model%nghost1
nghost2=model%nghost2
nghost3=model%nghost3
!
! --- Output of certain averages ---
if (present(output)) then
  Output_flag=output
else
  Output_flag=.false.
endif
OutRadialC_flag=.true.
!
call tabinter_getinfo(x2min=eiTable_min)
!
! --- Create boxes for sweeps in x2 and x3 directions ---
call rhd_box_Make(model_2, m2, n2, m1, n1, m3, n3, nghost2, nghost1, nghost3, Bc_flag=.true.)
call rhd_box_Make(model_3, m3, n3, m2, n2, m1, n1, nghost3, nghost2, nghost1, Bc_flag=.true.)
!
! --- Copy model (axes only) ---
call rhd_box_Copy(model  , m1-nghost1, n1+nghost1, m2-nghost2, n2+nghost2, m3-nghost3, n3+nghost3, &
                  model_2, m2-nghost2, m1-nghost1, m3-nghost3, &
                  trans_mode=2, grid_mode=1, bulk_mode=0, flux_mode=0)
call rhd_box_Copy(model  , m1-nghost1, n1+nghost1, m2-nghost2, n2+nghost2, m3-nghost3, n3+nghost3, &
                  model_3, m3-nghost3, m2-nghost2, m1-nghost1, &
                  trans_mode=3, grid_mode=1, bulk_mode=0, flux_mode=0)
!
! === Boundary conditions I ========================================================================
!
! --- Look for which dimension boundary conditions have to be applied ---
bound_l1=action%bound_l1
bound_r1=action%bound_r1
bound_l2=action%bound_l2
bound_r2=action%bound_r2
bound_l3=action%bound_l3
bound_r3=action%bound_r3
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL3D: grav')
!#
# endif
!#
! === Gravitational potential ======================================================================
call rhd_grav(model,   rhd_action_New(action, dim=1), grav_flag, phic_grav , phib_grav )
call rhd_grav(model_2, rhd_action_New(action, dim=2, dimV=((/2, 1, 3/))), &
                                                      grav_flag, phic_grav2, phib_grav2)
call rhd_grav(model_3, rhd_action_New(action, dim=3, dimV=((/3, 2, 1/))), &
                                                      grav_flag, phic_grav3, phib_grav3)
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: grav')
call timing_start('MHD: HLL3D: boundaries: phi')
!#
# endif
!#
!
!
! === Boundary conditions II =======================================================================
!
! --- Boundary conditions for gravity: potential set constant in ghost zones ---
if (grav_flag) then
  !
  ! --- x1 direction ---
  if (.not. ((bound_l1 == bound_periodic        ) .or. &
             (bound_l1 == bound_transmitting    ) .or. &
             (bound_l1 == bound_transmitting2   ) .or. &
             (bound_l1 == bound_inoutflow2      ) .or. &
             (bound_l1 == bound_inoutflow3      ) .or. &
             (bound_l1 == bound_closedbottom2   ) .or. &
             (bound_l1 == bound_InternalNext    ) .or. &
             (bound_l1 == bound_InternalPeriodic)) .and. m1 == prop%m1) then
    do i1=m1-nghost1,m1-1
      phic_grav( i1,:,:)=phic_grav( m1,:,:)
      phic_grav2(:,i1,:)=phic_grav2(:,m1,:)
      phib_grav2(:,i1,:)=phib_grav2(:,m1,:)
      phic_grav3(:,:,i1)=phic_grav3(:,:,m1)
      phib_grav3(:,:,i1)=phib_grav3(:,:,m1)
    end do ! i1
    do i1=m1-nghost1,m1
      phib_grav(i1,:,:)=phic_grav(m1,:,:)
    end do ! i1
  endif
  if (.not. ((bound_r1 == bound_periodic        ) .or. &
             (bound_r1 == bound_transmitting    ) .or. &
             (bound_r1 == bound_transmitting2   ) .or. &
             (bound_r1 == bound_InternalNext    ) .or. &
             (bound_r1 == bound_InternalPeriodic)) .and. n1 == prop%n1) then
    do i1=n1+1,n1+nghost1
      phic_grav( i1,:,:)=phic_grav( n1,:,:)
      phic_grav2(:,i1,:)=phic_grav2(:,n1,:)
      phib_grav2(:,i1,:)=phib_grav2(:,n1,:)
      phic_grav3(:,:,i1)=phic_grav3(:,:,n1)
      phib_grav3(:,:,i1)=phib_grav3(:,:,n1)
    end do ! i1
    do i1=n1+1,n1+nghost1+1
      phib_grav(i1,:,:)=phic_grav(n1,:,:)
    end do ! i1
  endif
  !
  ! --- x2 direction ---
  if (.not. ((bound_l2 == bound_periodic        ) .or. &
             (bound_l2 == bound_transmitting    ) .or. &
             (bound_l2 == bound_transmitting2   ) .or. &
             (bound_l2 == bound_inoutflow2      ) .or. &
             (bound_l2 == bound_inoutflow3      ) .or. &
             (bound_l2 == bound_closedbottom2   ) .or. &
             (bound_l2 == bound_InternalNext    ) .or. &
             (bound_l2 == bound_InternalPeriodic)) .and. m2 == prop%m2) then
    do i2=m2-nghost2,m2-1
      phic_grav2(i2,:,:)=phic_grav2(m2,:,:)
      phic_grav( :,i2,:)=phic_grav( :,m2,:)
      phib_grav( :,i2,:)=phib_grav( :,m2,:)
      phic_grav3(:,i2,:)=phic_grav3(:,m2,:)
      phib_grav3(:,i2,:)=phib_grav3(:,m2,:)
    end do ! i2
    do i2=m2-nghost2,m2
      phib_grav2(i2,:,:)=phic_grav2(m2,:,:)
    end do ! i2
  endif
  if (.not. ((bound_r2 == bound_periodic        ) .or. &
             (bound_r2 == bound_transmitting    ) .or. &
             (bound_r2 == bound_transmitting2   ) .or. &
             (bound_r2 == bound_InternalNext    ) .or. &
             (bound_r2 == bound_InternalPeriodic)) .and. n2 == prop%n2) then
    do i2=n2+1,n2+nghost2
      phic_grav2(i2,:,:)=phic_grav2(n2,:,:)
      phic_grav( :,i2,:)=phic_grav( :,n2,:)
      phib_grav( :,i2,:)=phib_grav( :,n2,:)
      phic_grav3(:,i2,:)=phic_grav3(:,n2,:)
      phib_grav3(:,i2,:)=phib_grav3(:,n2,:)
    end do ! i2
    do i2=n2+1,n2+nghost2+1
      phib_grav2(i2,:,:)=phic_grav2(n2,:,:)
    end do ! i2
  endif
  !
  ! --- x3 direction ---
  if (.not. ((bound_l3 == bound_periodic        ) .or. &
             (bound_l3 == bound_transmitting    ) .or. &
             (bound_l3 == bound_transmitting2   ) .or. &
             (bound_l3 == bound_inoutflow2      ) .or. &
             (bound_l3 == bound_inoutflow3      ) .or. &
             (bound_l3 == bound_closedbottom2   ) .or. &
             (bound_l3 == bound_InternalNext    ) .or. &
             (bound_l3 == bound_InternalPeriodic)) .and. m3 == prop%m3) then
    do i3=m3-nghost3,m3-1
      phic_grav3(i3,:,:)=phic_grav3(m3,:,:)
      phic_grav( :,:,i3)=phic_grav( :,:,m3)
      phib_grav( :,:,i3)=phib_grav( :,:,m3)
      phic_grav2(:,:,i3)=phic_grav2(:,:,m3)
      phib_grav2(:,:,i3)=phib_grav2(:,:,m3)
    end do ! i3
    do i3=m3-nghost3,m3
      phib_grav3(i3,:,:)=phic_grav3(m3,:,:)
    end do ! i3
  endif
  if (.not. ((bound_r3 == bound_periodic        ) .or. &
             (bound_r3 == bound_transmitting    ) .or. &
             (bound_r3 == bound_transmitting2   ) .or. &
             (bound_r3 == bound_InternalNext    ) .or. &
             (bound_r3 == bound_InternalPeriodic)) .and. n3 == prop%n3) then
    do i3=n3+1,n3+nghost3
      phic_grav3(i3,:,:)=phic_grav3(n3,:,:)
      phic_grav( :,:,i3)=phic_grav( :,:,n3)
      phib_grav( :,:,i3)=phib_grav( :,:,n3)
      phic_grav2(:,:,i3)=phic_grav2(:,:,n3)
      phib_grav2(:,:,i3)=phib_grav2(:,:,n3)
    end do ! i3
    do i3=n3+1,n3+nghost3+1
      phib_grav3(i3,:,:)=phic_grav3(n3,:,:)
    end do ! i3
  endif
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: boundaries: phi')
!#
# endif
!#
!
!
! === EOS, further quantities, check Courant condition =============================================
!
! --- Solve EOS ---
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL3D: eosinter')
!#
# endif
!#
if (model%nquc > 0) then
  T_flag=(Output_flag .or. any(prop%quc_advect == 2))
else
  T_flag=Output_flag
endif
if (T_flag) then
  call eosinter_all(model%rho, model%ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
else
  call eosinter_PdP(model%rho, model%ei, &
                    P=P, dPdrho=dPdrho, dPde=dPdei)
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop( 'MHD: HLL3D: eosinter')
call timing_start('MHD: HLL3D: boundaries: transmitting')
!#
# endif
!#
! === Boundary conditions III ======================================================================
!
! --- transmitting boundary conditions ---
if (grav_flag) then
  !
  if ((bound_l1 == bound_transmitting .or. bound_l1 == bound_transmitting2) .and. &
      (m1-nghost1 < prop%m1)) then
    !
    m10 = prop%m1
    !
    do i3=m3-nghost3,n3+nghost3
      do i2=m2-nghost2,n2+nghost2
        Hp1(i2,i3)=P(m10,i2,i3) / (abs(phib_grav(m10+1,i2,i3)-phib_grav(m10,i2,i3)) / &
                    (model%xb1(m10+1)-model%xb1(m10)) * model%rho(m10,i2,i3))
      end do ! i2
    end do ! i3
    !
    ! \label{f90:c_hptopfactor__useMHD2}
    if (par%C_Hptopfactor < 0.0) then
      rhoFac1=exp(-abs(                  (model%xc1(m10+1)-model%xc1(m10))/Hp1))
    else
      rhoFac1=exp(-abs(par%C_Hptopfactor*(model%xc1(m10+1)-model%xc1(m10))/Hp1))
    endif
    !
!!$    do i1 = m1-nghost1, m10-1
    do i1 =  m10-1, m1-nghost1, -1
      model%rho(i1,:,:) = model%rho(i1+1,:,:) * rhoFac1
    end do ! i1
    !
    ! \label{f90:rho_min__useMHD2}
    if (par%rho_min > 0.0) then
     do i1=m1-nghost1, m10-1
        model%rho(i1,:,:)=max(model%rho(i1,:,:), par%rho_min)
      end do ! i1
    endif
    !
    call eosinter_PdP(model%rho(m1-nghost1:m10-1,:,:), model%ei(m1-nghost1:m10-1,:,:), &
                      P=P(m1-nghost1:m10-1,:,:), dPdrho=dPdrho(m1-nghost1:m10-1,:,:), &
                                                 dPde  =dPdei( m1-nghost1:m10-1,:,:))
    !
  endif
  !
  if ((bound_r1 == bound_transmitting .or. bound_r1 == bound_transmitting2) .and. &
      (n1+nghost1 > prop%n1)) then
    !
    n10 = prop%n1
    !
    do i3=m3-nghost3,n3+nghost3
      do i2=m2-nghost2, n2+nghost2
        Hp1(i2,i3)=P(n10,i2,i3) / (abs(phib_grav(n10+1,i2,i3)-phib_grav(n10,i2,i3)) / &
                    (model%xb1(n10+1)-model%xb1(n10)) * model%rho(n10,i2,i3))
      end do ! i2
    end do ! i3
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac1=exp(-abs(                  (model%xc1(n10+1)-model%xc1(n10))/Hp1))
    else
      rhoFac1=exp(-abs(par%C_Hptopfactor*(model%xc1(n10+1)-model%xc1(n10))/Hp1))
    endif
    !
    do i1 = n10+1, n1+nghost1
      model%rho(i1,:,:) = model%rho(i1-1,:,:) * rhoFac1
    end do ! i1
    !
    if (par%rho_min > 0.0) then
     do i1=n10+1, n1+nghost1
        model%rho(i1,:,:)=max(model%rho(i1,:,:), par%rho_min)
      end do ! i1
    endif
    !
    call eosinter_PdP(model%rho(n10+1:n1+nghost1,:,:), model%ei(n10+1:n1+nghost1,:,:), &
                      P=P(n10+1:n1+nghost1,:,:), dPdrho=dPdrho(n10+1:n1+nghost1,:,:), &
                                                 dPde  =dPdei( n10+1:n1+nghost1,:,:))
    !
  endif
  !
  if ((bound_l2 == bound_transmitting .or. bound_l2 == bound_transmitting2) .and. &
      (m2-nghost2 < prop%m2)) then
    !
    m20 = prop%m2
    !
    do i3=m3-nghost3,n3+nghost3
      do i1=m1-nghost1,n1+nghost1
        Hp2(i1,i3)=P(i1,m20,i3) / (abs(phib_grav2(m20+1,i1,i3)-phib_grav2(m20,i1,i3)) / &
                    (model%xb2(m20+1)-model%xb2(m20)) * model%rho(i1,m20,i3))
      end do ! i1
    end do ! i3
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac2=exp(-abs(                  (model%xc2(m20+1)-model%xc2(m20))/Hp2))
    else
      rhoFac2=exp(-abs(par%C_Hptopfactor*(model%xc2(m20+1)-model%xc2(m20))/Hp2))
    endif
    !
!!$    do i2 = m2-nghost2, m20-1
    do i2 = m20-1, m2-nghost2, -1
      model%rho(:,i2,:) = model%rho(:,i2+1,:) * rhoFac2
    end do ! i2
    !
    if (par%rho_min > 0.0) then
     do i2=m2-nghost2, m20-1
        model%rho(:,i2,:)=max(model%rho(:,i2,:), par%rho_min)
      end do ! i2
    endif
    !
    call eosinter_PdP(model%rho(:,m2-nghost2:m20-1,:), model%ei(:,m2-nghost2:m20-1,:), &
                      P=P(:,m2-nghost2:m20-1,:), dPdrho=dPdrho(:,m2-nghost2:m20-1,:), &
                                                 dPde  =dPdei( :,m2-nghost2:m20-1,:))
    !
  endif
  !
  if ((bound_r2 == bound_transmitting .or. bound_r2 == bound_transmitting2) .and. &
      (n2+nghost2 > prop%n2)) then
    !
    n20 = prop%n2
    !
    do i3=m3-nghost3,n3+nghost3
      do i1=m1-nghost1,n1+nghost1
        Hp2(i1,i3)=P(i1,n20,i3) / (abs(phib_grav2(n20+1,i1,i3)-phib_grav2(n20,i1,i3)) / &
                    (model%xb2(n20+1)-model%xb2(n20)) * model%rho(i1,n20,i3))
      end do ! i1
    end do ! i3
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac2=exp(-abs(                  (model%xc2(n20+1)-model%xc2(n20))/Hp2))
    else
      rhoFac2=exp(-abs(par%C_Hptopfactor*(model%xc2(n20+1)-model%xc2(n20))/Hp2))
    endif
    !
    do i2 = n20+1, n2+nghost2
      model%rho(:,i2,:) = model%rho(:,i2-1,:) * rhoFac2
    end do ! i2
    !
    if (par%rho_min > 0.0) then
     do i2=n20+1, n2+nghost2
        model%rho(:,i2,:)=max(model%rho(:,i2,:), par%rho_min)
      end do ! i2
    endif
    !
    call eosinter_PdP(model%rho(:,n20+1:n2+nghost2,:), model%ei(:,n20+1:n2+nghost2,:), &
                      P=P(:,n20+1:n2+nghost2,:), dPdrho=dPdrho(:,n20+1:n2+nghost2,:), &
                                                 dPde  =dPdei( :,n20+1:n2+nghost2,:))
    !
  endif
  !
  if ((bound_l3 == bound_transmitting .or. bound_l3 == bound_transmitting2) .and. &
      (m3-nghost3 < prop%m3)) then
    !
    m30 = prop%m3
    !
    do i1=m1-nghost1,n1+nghost1
      do i2=m2-nghost2,n2+nghost2
        Hp3(i1,i2)=P(i1,i2,m30) / (abs(phib_grav3(m30+1,i2,i1)-phib_grav3(m30,i2,i1)) / &
                    (model%xb3(m30+1)-model%xb3(m30)) * model%rho(i1,i2,m30))
      end do ! i2
    end do ! i1
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac3=exp(-abs(                  (model%xc3(m30+1)-model%xc3(m30))/Hp3))
    else
      rhoFac3=exp(-abs(par%C_Hptopfactor*(model%xc3(m30+1)-model%xc3(m30))/Hp3))
    endif
    !
!!$    do i3 = m3-nghost3, m30-1
    do i3 = m30-1, m3-nghost3, -1
      model%rho(:,:,i3) = model%rho(:,:,i3+1) * rhoFac3
    end do ! i3
    !
    if (par%rho_min > 0.0) then
     do i3=m3-nghost3, m30-1
        model%rho(:,:,i3)=max(model%rho(:,:,i3), par%rho_min)
      end do ! i3
    endif
    !
    call eosinter_PdP(model%rho(:,:,m3-nghost3:m30-1), model%ei(:,:,m3-nghost3:m30-1), &
                      P=P(:,:,m3-nghost3:m30-1), dPdrho=dPdrho(:,:,m3-nghost3:m30-1), &
                                                 dPde  =dPdei( :,:,m3-nghost3:m30-1))
    !
  endif
  !
  if ((bound_r3 == bound_transmitting .or. bound_r3 == bound_transmitting2) .and. &
      (n3+nghost3 > prop%n3)) then
    !
    n30 = prop%n3
    !
    do i1=m1-nghost1,n1+nghost1
      do i2=m2-nghost2,n2+nghost2
        Hp3(i1,i2)=P(i1,i2,n30) / (abs(phib_grav3(n30+1,i2,i1)-phib_grav3(n30,i2,i1)) / &
                    (model%xb3(n30+1)-model%xb3(n30)) * model%rho(i1,i2,n30))
      end do ! i2
    end do ! i1
    !
    if (par%C_Hptopfactor < 0.0) then
      rhoFac3=exp(-abs(                  (model%xc3(n30+1)-model%xc3(n30))/Hp3))
    else
      rhoFac3=exp(-abs(par%C_Hptopfactor*(model%xc3(n30+1)-model%xc3(n30))/Hp3))
    endif
    !
    do i3 = n30+1, n3+nghost3
      model%rho(:,:,i3) = model%rho(:,:,i3-1) * rhoFac3
    end do ! i3
    !
    if (par%rho_min > 0.0) then
     do i3=n30+1, n3+nghost3
        model%rho(:,:,i3)=max(model%rho(:,:,i3), par%rho_min)
      end do ! i3
    endif
    !
    call eosinter_PdP(model%rho(:,:,n30+1:n3+nghost3), model%ei(:,:,n30+1:n3+nghost3), &
                      P=P(:,:,n30+1:n3+nghost3), dPdrho=dPdrho(:,:,n30+1:n3+nghost3), &
                                                 dPde  =dPdei( :,:,n30+1:n3+nghost3))
    !
  endif
  !
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: boundaries: transmitting')
call timing_start('MHD: HLL3D: copy and transpose')
!#
# endif
!#
!
Gamma_1=dPdrho*model%rho/P + dPdei/model%rho
P_mag  =0.5*(model%Bc1**2+model%Bc2**2+model%Bc3**2)
P_tot  =P+P_mag
!
if (par%beta_inv < 0.0 .or. par%A%hdScheme == hdScheme_RoeMHD) then
  energy_switch=.true.
else
  energy_switch=(P_mag/P < par%beta_inv)
endif
model%energy_switch=energy_switch
!
do i3=m3-nghost3,n3+nghost3
  !
  P2(            :,:,i3)=transpose(P(            :,:,i3))
  Gamma_12(      :,:,i3)=transpose(Gamma_1(      :,:,i3))
  energy_switch2(:,:,i3)=transpose(energy_switch(:,:,i3))
  !
  if (Output_flag) then
    dPdrho2(:,:,i3)=transpose(dPdrho(:,:,i3))
    dPdei2( :,:,i3)=transpose(dPdei( :,:,i3))
    T2(     :,:,i3)=transpose(T(     :,:,i3))
    dTdei2( :,:,i3)=transpose(dTdei( :,:,i3))
    s2(     :,:,i3)=transpose(s(     :,:,i3))
  endif
  !
end do ! i3
!
do i2=m2-nghost2,n2+nghost2
  !
  P3(            :,i2,:)=transpose(P(            :,i2,:))
  Gamma_13(      :,i2,:)=transpose(Gamma_1(      :,i2,:))
  energy_switch3(:,i2,:)=transpose(energy_switch(:,i2,:))
  !
  if (Output_flag) then
    dPdrho3(:,i2,:)=transpose(dPdrho(:,i2,:))
    dPdei3( :,i2,:)=transpose(dPdei( :,i2,:))
    T3(     :,i2,:)=transpose(T(     :,i2,:))
    dTdei3( :,i2,:)=transpose(dTdei( :,i2,:))
    s3(     :,i2,:)=transpose(s(     :,i2,:))
  endif
  !
end do ! i2
!
! --- Save original values into local copies ---
xc1=model%xc1
xb1=model%xb1
xc2=model%xc2
xb2=model%xb2
xc3=model%xc3
xb3=model%xb3
!
rho=model%rho
ei =model%ei
v1 =model%v1
v2 =model%v2
v3 =model%v3
Bc1=model%Bc1
Bc2=model%Bc2
Bc3=model%Bc3
!
! --- Copy model to boxes for x2 sweep, transpose arrays ---
!call timing_start('MHD: HLL3D: copy and transpose 2')
call rhd_box_Copy(model  , m1-nghost1, n1+nghost1, m2-nghost2, n2+nghost2, m3-nghost3, n3+nghost3, &
                  model_2, m2-nghost2, m1-nghost1, m3-nghost3, &
                  trans_mode=2, grid_mode=0, bulk_mode=1, flux_mode=0)
!call timing_stop('MHD: HLL3D: copy and transpose 2')
!
! --- Save original values into local copies ---
rho2=model_2%rho
ei2 =model_2%ei
v12 =model_2%v2
v22 =model_2%v1
v32 =model_2%v3
Bc12=model_2%Bc2
Bc22=model_2%Bc1
Bc32=model_2%Bc3
!
! --- Copy model to boxes for x3 sweeps, transpose arrays ---
!call timing_start('MHD: HLL3D: copy and transpose 3')
call rhd_box_Copy(model  , m1-nghost1, n1+nghost1, m2-nghost2, n2+nghost2, m3-nghost3, n3+nghost3, &
                  model_3, m3-nghost3, m2-nghost2, m1-nghost1, &
                  trans_mode=3, grid_mode=0, bulk_mode=1, flux_mode=0)
!call timing_stop('MHD: HLL3D: copy and transpose 3')
!
! --- Save original values into local copies ---
rho3=model_3%rho
ei3 =model_3%ei
v13 =model_3%v3
v23 =model_3%v2
v33 =model_3%v1
Bc13=model_3%Bc3
Bc23=model_3%Bc2
Bc33=model_3%Bc1
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: copy and transpose')
!#
# endif
!#
! --- Check positivity of quantities ---
if (any(P <= 0.0)) then
  ierr0=4
  error_loc=minloc(P)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+(m2-nghost2)
  error_loc(3)=error_loc(3)-1+(m3-nghost3)
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') P(  error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Pressure non-positive (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif ! (any(P <= 0.0))
if (any(Gamma_1 < 1.0)) then
  ierr0=5
  error_loc=minloc(Gamma_1)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+(m2-nghost2)
  error_loc(3)=error_loc(3)-1+(m3-nghost3)
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Gamma_1 smaller than 1 (' // trim(outstr0) // ')'
  !
  dtime_limit=dtime
  goto 999
endif ! (any(Gamma_1 < 1.0))
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL3D: derived q')
!#
# endif
!#
dtime_limit=0.1*huge(dtime_limit)
!
do i3=m3-nghost3,n3+nghost3
  do i2=m2-nghost2,n2+nghost2
    !DIR$ SIMD
    do i1=m1-nghost1,n1+nghost1
      ! --- Compute derived quantities: sound speed,            ---
      ! --- speeds of magnetic waves, maximum allowed time step ---
      !
      cs(      i1,i2,i3)=sqrt(P(i1,i2,i3)/rho(i1,i2,i3)*Gamma_1(i1,i2,i3))
      sqrt_rho          =sqrt(rho(i1,i2,i3))
      va1(     i1,i2,i3)=abs(Bc1(i1,i2,i3))/sqrt_rho
      va2(     i1,i2,i3)=abs(Bc2(i1,i2,i3))/sqrt_rho
      va3(     i1,i2,i3)=abs(Bc3(i1,i2,i3))/sqrt_rho
    end do ! i1
    !
    ! \label{f90:va_max__useMHD3}
    if ((par%va_max > 0.0) .and. &
        ((par%A%hdScheme == hdScheme_HLLMHD) .or. (par%A%hdScheme == hdScheme_CenMHD))) then
      !DIR$ SIMD
      do i1=m1-nghost1,n1+nghost1
        f(    i1,i2,i3)=par%va_max**2/(par%va_max**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3))
        va1(  i1,i2,i3)=sqrt(f(i1,i2,i3))*va1(i1,i2,i3)
        va2(  i1,i2,i3)=sqrt(f(i1,i2,i3))*va2(i1,i2,i3)
        va3(  i1,i2,i3)=sqrt(f(i1,i2,i3))*va3(i1,i2,i3)
        P_mag(i1,i2,i3)=f(i1,i2,i3)*P_mag(i1,i2,i3)
        P_tot(i1,i2,i3)=P(i1,i2,i3)+P_mag(i1,i2,i3)
      end do ! i1
    else
      do i1=m1-nghost1,n1+nghost1
        f(i1,i2,i3)=1.0
      end do ! i1
    endif
    !
    !DIR$ SIMD
    do i1=m1-nghost1,n1+nghost1
      B_sq_tan          =Bc2(i1,i2,i3)**2+Bc3(i1,i2,i3)**2
      !
      if ((B_sq_tan * f(i1,i2,i3)) > (epsilon * P_tot(i1,i2,i3))) then
        caq             =cs(i1,i2,i3)**2 + 2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd              =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va1(i1,i2,i3)/caq)**2))
        vf1(   i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs1(   i1,i2,i3)=cs(i1,i2,i3)*va1(i1,i2,i3) * sqrt(2.0 / (caq+dd))
      else
        if      ((cs(i1,i2,i3) - va1(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf1(i1,i2,i3) = cs( i1,i2,i3)
          vs1(i1,i2,i3) = va1( i1,i2,i3)
        else if ((va1(i1,i2,i3) - cs(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf1(i1,i2,i3) = va1(i1,i2,i3)
          vs1(i1,i2,i3) = cs( i1,i2,i3)
        else
          vf1(i1,i2,i3) = cs( i1,i2,i3)
          vs1(i1,i2,i3) = cs( i1,i2,i3)
        endif
      endif
      !
    end do ! i1
    !
    !DIR$ SIMD
    do i1=m1-nghost1,n1+nghost1
      B_sq_tan          =Bc3(i1,i2,i3)**2+Bc1(i1,i2,i3)**2
      !
      if ((B_sq_tan * f(i1,i2,i3)) > (epsilon * P_tot(i1,i2,i3))) then
        caq             =cs(i1,i2,i3)**2+2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd              =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va2(i1,i2,i3)/caq)**2))
        vf2(   i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs2(   i1,i2,i3)=cs(i1,i2,i3)*va2(i1,i2,i3) * sqrt(2.0 / (caq+dd))
      else
        if      ((cs(i1,i2,i3) - va3(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf2(i1,i2,i3) = cs( i1,i2,i3)
          vs2(i1,i2,i3) = va2( i1,i2,i3)
        else if ((va3(i1,i2,i3) - cs(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf2(i1,i2,i3) = va2(i1,i2,i3)
          vs2(i1,i2,i3) = cs( i1,i2,i3)
        else
          vf2(i1,i2,i3) = cs( i1,i2,i3)
          vs2(i1,i2,i3) = cs( i1,i2,i3)
        endif
      endif
    end do ! i1
    !
    !DIR$ SIMD
    do i1=m1-nghost1,n1+nghost1
      !
      B_sq_tan          =Bc2(i1,i2,i3)**2+Bc1(i1,i2,i3)**2
      !
      if ((B_sq_tan * f(i1,i2,i3)) > (epsilon * P_tot(i1,i2,i3))) then
        caq             =cs(i1,i2,i3)**2+2.0*P_mag(i1,i2,i3)/rho(i1,i2,i3)
        dd              =caq * sqrt(max(0.0, 1.0 - 4.0*(cs(i1,i2,i3)*va3(i1,i2,i3)/caq)**2))
        vf3(   i1,i2,i3)=sqrt(0.5*(caq + dd))
        vs3(   i1,i2,i3)=cs(i1,i2,i3)*va3(i1,i2,i3) * sqrt(2.0 / (caq+dd))
      else
        if ((cs(i1,i2,i3) - va3(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf3(i1,i2,i3) = cs( i1,i2,i3)
          vs3(i1,i2,i3) = va3(i1,i2,i3)
        else if ((va3(i1,i2,i3) - cs(i1,i2,i3)) > (epsilon * cs(i1,i2,i3))) then
          vf3(i1,i2,i3) = va3(i1,i2,i3)
          vs3(i1,i2,i3) = cs( i1,i2,i3)
        else
          vf3(i1,i2,i3) = cs( i1,i2,i3)
          vs3(i1,i2,i3) = cs( i1,i2,i3)
        endif
      endif
    end do ! i1
    !
    do i1=m1-nghost1,n1+nghost1
      dtime_limit       =min(dtime_limit, &
                             1.0 / ((abs(v1(i1,i2,i3))+vf1(i1,i2,i3)) / (xb1(i1+1)-xb1(i1)) + &
                                    (abs(v2(i1,i2,i3))+vf2(i1,i2,i3)) / (xb2(i2+1)-xb2(i2)) + &
                                    (abs(v3(i1,i2,i3))+vf3(i1,i2,i3)) / (xb3(i3+1)-xb3(i3))    ))
    end do ! i1
  end do ! i2
end do ! i3
!
do i3=m3-nghost3,n3+nghost3
  f2(:,:,i3)=transpose(f(:,:,i3))
end do
do i2=m2-nghost2,n2+nghost2
  f3(:,i2,:)=transpose(f(:,i2,:))
end do
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: derived q')
!#
# endif
!#
!
! --- Check Courant condition ---
!
if ((par%C_CourantMax > 0.0) .and. (dtime > par%C_CourantMax * dtime_limit)) then
  ! --- Time step too large: return without further computation ---
  ierr0=1
  outstr0='Time step too large for Courant condition'
  goto 999
endif
!
!
! --- Perform reconstruction and (optional) Hancock predictor step ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('MHD: HLL3D: Reconstruction')
!#
# endif
!#
call rhd_mhd_Reconstruct(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, 1, 1, &
                         par%A%reconstruction_order, par%A%reconstruction_type, 1, &
                         dtime, xb1, f, &
                         rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, Gamma_1, &
                         rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                         rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1)
!
call rhd_mhd_Reconstruct(m2,n2, m1,n1, m3,n3, nghost2, nghost1, nghost3, 1, 1, &
                         par%A%reconstruction_order, par%A%reconstruction_type, 2, &
                         dtime, xb2, f2, &
                         rho2, ei2, v22, v12, v32, Bc22, Bc12, Bc32, P2, Gamma_12, &
                         rho_l2, v2_l2, v1_l2, v3_l2, Bc2_l2, Bc1_l2, Bc3_l2, P_l2, rhoei_l2, &
                         rho_r2, v2_r2, v1_r2, v3_r2, Bc2_r2, Bc1_r2, Bc3_r2, P_r2, rhoei_r2)
!
call rhd_mhd_Reconstruct(m3,n3, m2,n2, m1,n1, nghost3, nghost2, nghost1, 1, 1, &
                         par%A%reconstruction_order, par%A%reconstruction_type, 3, &
                         dtime, xb3, f3, &
                         rho3, ei3, v33, v23, v13, Bc33, Bc23, Bc13, P3, Gamma_13, &
                         rho_l3, v3_l3, v2_l3, v1_l3, Bc3_l3, Bc2_l3, Bc1_l3, P_l3, rhoei_l3, &
                         rho_r3, v3_r3, v2_r3, v1_r3, Bc3_r3, Bc2_r3, Bc1_r3, P_r3, rhoei_r3)
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('MHD: HLL3D: Reconstruction')
!#
# endif
!#
!
if (par%A%reconstruction_order > 0) then
  if (par%hdTimeIntegrationScheme == 'Hancock') then
    ! --- Perform unsplit 3D Hancock predictor step ---
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('MHD: HLL3D: Hancock')
!#
#   endif
!#
    call rhd_mhd_Hancock3D(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                           dtime, xb1, xb2, xb3, &
                           rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                           Gamma_1, f, phib_grav, phib_grav2, phib_grav3, &
                           rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                           rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
                           rho_l2, v1_l2, v2_l2, v3_l2, Bc1_l2, Bc2_l2, Bc3_l2, P_l2, rhoei_l2, &
                           rho_r2, v1_r2, v2_r2, v3_r2, Bc1_r2, Bc2_r2, Bc3_r2, P_r2, rhoei_r2, &
                           rho_l3, v1_l3, v2_l3, v3_l3, Bc1_l3, Bc2_l3, Bc3_l3, P_l3, rhoei_l3, &
                           rho_r3, v1_r3, v2_r3, v3_r3, Bc1_r3, Bc2_r3, Bc3_r3, P_r3, rhoei_r3)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('MHD: HLL3D: Hancock')
!#
#   endif
!#
  endif ! (par%hdTimeIntegrationScheme == 'Hancock')
endif ! (par%A%reconstruction_order > 0)
!
! --- Add minimum internal energy. It was subtracted in rhd_mhd_Reconstruct. ---
rhoei_l1=rhoei_l1 + rho_l1*eiTable_min
rhoei_r1=rhoei_r1 + rho_r1*eiTable_min
rhoei_l2=rhoei_l2 + rho_l2*eiTable_min
rhoei_r2=rhoei_r2 + rho_r2*eiTable_min
rhoei_l3=rhoei_l3 + rho_l3*eiTable_min
rhoei_r3=rhoei_r3 + rho_r3*eiTable_min
!
!
! === Main step: compute boundary located (flux-like) averages and fluxes ==========================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL3D: fluxes')
call timing_start('MHD: HLL3D: flux 1')
!#
# endif
!#
!
call rhd_mhd_HLL1Dflux(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, 1, 1, &
                       par%A%hdScheme, &
                       bound_l1, bound_r1, par%A%reconstruction_order, par%A%reconstruction_type, &
                       1, dtime, xb1, f, &
                       rho, ei, v1, v2, v3, Bc1, Bc2, Bc3, P, &
                       rho_l1, v1_l1, v2_l1, v3_l1, Bc1_l1, Bc2_l1, Bc3_l1, P_l1, rhoei_l1, &
                       rho_r1, v1_r1, v2_r1, v3_r1, Bc1_r1, Bc2_r1, Bc3_r1, P_r1, rhoei_r1, &
                       Gamma_1, energy_switch, &
                       phic_grav, phib_grav, &
                       flux1_rho, flux1_rhov1, flux1_rhov2, flux1_rhov3, &
                                  flux1_Bc1  , flux1_Bc2  , flux1_Bc3  , &
                       flux1_rhoeikm, flux1_rhoei, flux1_rhoek, flux1_rhoem, fep1, &
                       flux1_rhov1B, flux1_rhov2B, flux1_rhov3B, &
                       source1Bc1_l, source1Bc2_l, source1Bc3_l, &
                       source1Bc1_r, source1Bc2_r, source1Bc3_r)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: flux 1')
call timing_start('MHD: HLL3D: flux 2')
!#
# endif
!#
call rhd_mhd_HLL1Dflux(m2, n2, m1, n1, m3, n3, nghost2, nghost1, nghost3, 1, 1, &
                       par%A%hdScheme, &
                       bound_l2, bound_r2, par%A%reconstruction_order, par%A%reconstruction_type, &
                       2, dtime, xb2, f2, &
                       rho2, ei2, v22, v12, v32, Bc22, Bc12, Bc32, P2, &
                       rho_l2, v2_l2, v1_l2, v3_l2, Bc2_l2, Bc1_l2, Bc3_l2, P_l2, rhoei_l2, &
                       rho_r2, v2_r2, v1_r2, v3_r2, Bc2_r2, Bc1_r2, Bc3_r2, P_r2, rhoei_r2, &
                       Gamma_12, energy_switch2, &
                       phic_grav2, phib_grav2, &
                       flux2_rho, flux2_rhov2, flux2_rhov1, flux2_rhov3, &
                                  flux2_Bc2  , flux2_Bc1  , flux2_Bc3  , &
                       flux2_rhoeikm, flux2_rhoei, flux2_rhoek, flux2_rhoem, fep2, &
                       flux2_rhov2B, flux2_rhov1B, flux2_rhov3B, &
                       source2Bc2_l, source2Bc1_l, source2Bc3_l, &
                       source2Bc2_r, source2Bc1_r, source2Bc3_r)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: flux 2')
call timing_start('MHD: HLL3D: flux 3')
!#
# endif
!#
call rhd_mhd_HLL1Dflux(m3, n3, m2, n2, m1, n1, nghost3, nghost2, nghost1, 1, 1, &
                       par%A%hdScheme, &
                       bound_l3, bound_r3, par%A%reconstruction_order, par%A%reconstruction_type, &
                       3, dtime, xb3, f3, &
                       rho3, ei3, v33, v23, v13, Bc33, Bc23, Bc13, P3, &
                       rho_l3, v3_l3, v2_l3, v1_l3, Bc3_l3, Bc2_l3, Bc1_l3, P_l3, rhoei_l3, &
                       rho_r3, v3_r3, v2_r3, v1_r3, Bc3_r3, Bc2_r3, Bc1_r3, P_r3, rhoei_r3, &
                       Gamma_13, energy_switch3, &
                       phic_grav3, phib_grav3, &
                       flux3_rho, flux3_rhov3, flux3_rhov2, flux3_rhov1, &
                                  flux3_Bc3  , flux3_Bc2  , flux3_Bc1  , &
                       flux3_rhoeikm, flux3_rhoei, flux3_rhoek, flux3_rhoem, fep3, &
                       flux3_rhov3B, flux3_rhov2B, flux3_rhov1B, &
                       source3Bc3_l, source3Bc2_l, source3Bc1_l, &
                       source3Bc3_r, source3Bc2_r, source3Bc1_r)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: flux 3')
call timing_stop('MHD: HLL3D: fluxes')
!#
# endif
!#
if (Output_flag) then
  ! === Put additional output data into datx, datr, and datc =======================================
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('MHD: HLL3D: (output)')
!#
# endif
!#
  call rhd_mhd_Roe1Doutput(model, 1, AreaFraction1, OutRadialC_flag, &
                           P, T, s, Gamma_1, dPdei/rho + 1.0, dTdei, dPdei, &
                           phic_grav, phib_grav, &
                           flux1_rho, flux1_rhov1, flux1_rhov2, flux1_rhov3, &
                           flux1_rhoeikm, flux1_rhoei, flux1_rhoek, flux1_rhoem, fep1, &
                           flux1_rhov1B, flux1_rhov2B, flux1_rhov3B, &
                           datx=datx1, datr=datr, datc=datc)
  OutRadialC_flag=.false.
   !
  call rhd_mhd_Roe1Doutput(model_2, 2, AreaFraction2, OutRadialC_flag, &
                           P2, T2, s2, Gamma_12, dPdei2/rho2 + 1.0, dTdei2, dPdei2, &
                           phic_grav2, phib_grav2, &
                           flux2_rho, flux2_rhov2, flux2_rhov1, flux2_rhov3, &
                           flux2_rhoeikm, flux2_rhoei, flux2_rhoek, flux2_rhoem, fep2, &
                           flux2_rhov2B, flux2_rhov1B, flux2_rhov3B, &
                           datx2, datr)
  !
  call rhd_mhd_Roe1Doutput(model_3, 3, AreaFraction3, OutRadialC_flag, &
                           P3, T3, s3, Gamma_13, dPdei3/rho3 + 1.0, dTdei3, dPdei3, &
                           phic_grav3, phib_grav3, &
                           flux3_rho, flux3_rhov3, flux3_rhov2, flux3_rhov1, &
                           flux3_rhoeikm, flux3_rhoei, flux3_rhoek, flux3_rhoem, fep3, &
                           flux3_rhov3B, flux3_rhov2B, flux3_rhov1B, &
                           datx3, datr)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: HLL3D: (output)')
!#
# endif
!#
endif
!
!
! === Update with computed fluxes and source terms =================================================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('MHD: HLL3D: update')
!#
# endif
!#
! --- dtimeoverdx=dt/dx ---
dtimeoverdx1=dtime/(xb1(m1+1:n1+1)-xb1(m1:n1))
dtimeoverdx2=dtime/(xb2(m2+1:n2+1)-xb2(m2:n2))
dtimeoverdx3=dtime/(xb3(m3+1:n3+1)-xb3(m3:n3))
!
! --- Check if density is positive everywhere ---
rhonew_min=huge(rhonew_min)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      rhonew_min=min(rhonew_min, rho(i1,i2,i3) - &
                                 dtimeoverdx1(i1) * (flux1_rho(i1+1,i2,i3)-flux1_rho(i1,i2,i3)) - &
                                 dtimeoverdx2(i2) * (flux2_rho(i2+1,i1,i3)-flux2_rho(i2,i1,i3)) - &
                                 dtimeoverdx3(i3) * (flux3_rho(i3+1,i2,i1)-flux3_rho(i3,i2,i1)))

    end do ! i1
  end do ! i2
end do ! i3
if (rhonew_min <= 0.0) then
  ierr0=6
  dtime_limit=dtime
  outstr0='Density becomes non-positive'
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('MHD: HLL3D: update')
!#
# endif
!#
  goto 999
endif
!
!if (model%nquc > 0) then
!  ! --- Handle additional passive tracer fields ---
!  call rhd_hyd_Dust(model, T, phic_grav, &
!                    par%A%reconstruction_order, par%A%reconstruction_quctype, &
!                    dtime, flux1_rho)
!endif
!
!
! \label{f90:c_gravmu__useMHD3}
! --- Choose time-centering of gravity source terms ---
if (par%C_gravmu < 0.0) then
  ! --- Use old defaults ---
  if (par%hdTimeIntegrationScheme == 'Hancock' .and. par%A%reconstruction_order > 0) then
    ! --- Use time-centered density ---
    wgrav_new=0.5
  else
    ! --- E.g., for Runge-Kutta use only first-order terms (density at old time step) ---
    wgrav_new=0.0
  endif
else
  ! --- Use input parameter as weight ---
  wgrav_new=par%C_gravmu
endif
!
! --- Update all quantities ---
!
do i3=m3,n3
  do i2=m2,n2
    !DIR$ SIMD
    do i1=m1,n1
      ! --- Compute change in mass density and update it ---
      drho  =-dtimeoverdx1(i1) * (flux1_rho(  i1+1,i2,i3)-flux1_rho(  i1,i2,i3)) - &
              dtimeoverdx2(i2) * (flux2_rho(  i2+1,i1,i3)-flux2_rho(  i2,i1,i3)) - &
              dtimeoverdx3(i3) * (flux3_rho(  i3+1,i2,i1)-flux3_rho(  i3,i2,i1))
      rhoout=rho(i1,i2,i3) + drho
      !
      ! --- Compute source terms due to gravity ---
      w01=(phic_grav( i1  ,i2,i3)-phib_grav( i1,i2,i3)) / &
          (xb1(       i1+1      )-xb1(       i1      ))
      w11=(phib_grav( i1+1,i2,i3)-phic_grav( i1,i2,i3)) / &
          (xb1(       i1+1      )-xb1(       i1      ))
      w02=(phic_grav2(i2  ,i1,i3)-phib_grav2(i2,i1,i3)) / &
          (xb2(       i2+1      )-xb2(       i2      ))
      w12=(phib_grav2(i2+1,i1,i3)-phic_grav2(i2,i1,i3)) / &
          (xb2(       i2+1      )-xb2(       i2      ))
      w03=(phic_grav3(i3  ,i2,i1)-phib_grav3(i3,i2,i1)) / &
          (xb3(       i3+1      )-xb3(       i3      ))
      w13=(phib_grav3(i3+1,i2,i1)-phic_grav3(i3,i2,i1)) / &
          (xb3(       i3+1      )-xb3(       i3      ))
      source_eikm =w01*flux1_rho(i1,i2,i3) + w11*flux1_rho(i1+1,i2,i3) + &
                   w02*flux2_rho(i2,i1,i3) + w12*flux2_rho(i2+1,i1,i3) + &
                   w03*flux3_rho(i3,i2,i1) + w13*flux3_rho(i3+1,i2,i1) + (1.0 - f(i1,i2,i3)) * &
                   (v1(i1,i2,i3) * (flux1_rhov1B(i1+1,i2,i3) - flux1_rhov1B(i1  ,i2,i3)) / &
                                   (xb1(         i1+1      ) - xb1(         i1        )) + &
                    v2(i1,i2,i3) * (flux1_rhov2B(i1+1,i2,i3) - flux1_rhov2B(i1  ,i2,i3)) / &
                                   (xb1(         i1+1      ) - xb1(         i1        )) + &
                    v3(i1,i2,i3) * (flux1_rhov3B(i1+1,i2,i3) - flux1_rhov3B(i1  ,i2,i3)) / &
                                   (xb1(         i1+1      ) - xb1(         i1        )) + &
                    v1(i1,i2,i3) * (flux2_rhov1B(i2+1,i1,i3) - flux2_rhov1B(i2  ,i1,i3)) / &
                                   (xb2(         i2+1      ) - xb2(         i2        )) + &
                    v2(i1,i2,i3) * (flux2_rhov2B(i2+1,i1,i3) - flux2_rhov2B(i2  ,i1,i3)) / &
                                   (xb2(         i2+1      ) - xb2(         i2        )) + &
                    v3(i1,i2,i3) * (flux2_rhov3B(i2+1,i1,i3) - flux2_rhov3B(i2  ,i1,i3)) / &
                                   (xb2(         i2+1      ) - xb2(         i2        )) + &
                    v1(i1,i2,i3) * (flux3_rhov1B(i3+1,i2,i1) - flux3_rhov1B(i3  ,i2,i1)) / &
                                   (xb3(         i3+1      ) - xb3(         i3        )) + &
                    v2(i1,i2,i3) * (flux3_rhov2B(i3+1,i2,i1) - flux3_rhov2B(i3  ,i2,i1)) / &
                                   (xb3(         i3+1      ) - xb3(         i3        )) + &
                    v3(i1,i2,i3) * (flux3_rhov3B(i3+1,i2,i1) - flux3_rhov3B(i3  ,i2,i1)) / &
                                   (xb3(         i3+1      ) - xb3(         i3        )))
      source_ei   = -P(i1,i2,i3) * ((v1(i1+1,i2,i3) - v1(i1-1,i2,i3)) / (xc1(i1+1) - xc1(i1-1)) + &
                                    (v2(i1,i2+1,i3) - v2(i1,i2-1,i3)) / (Xc2(I2+1) - xc2(i2-1)) + &
                                    (v3(i1,i2,i3+1) - v3(i1,i2,i3-1)) / (xc3(i3+1) - xc3(i3-1)))
      source_rhov1=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav( i1+1,i2,i3)-phib_grav( i1,i2,i3)) / &
                                                    (xb1(       i1+1      )-xb1(       i1      ))
      source_rhov2=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav2(i2+1,i1,i3)-phib_grav2(i2,i1,i3)) / &
                                                    (xb2(       i2+1      )-xb2(       i2      ))
      source_rhov3=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav3(i3+1,i2,i1)-phib_grav3(i3,i2,i1)) / &
                                                    (xb3(       i3+1      )-xb3(       i3      ))
      !
      ! --- Update ---
      !
      v1out =(rho(i1,i2,i3) * v1(i1,i2,i3) - &
                             dtimeoverdx1(i1) * ((flux1_rhov1( i1+1,i2,i3)-flux1_rhov1( i1,i2,i3)) + &
                             f(i1,i2,i3)      *  (flux1_rhov1B(i1+1,i2,i3)-flux1_rhov1B(i1,i2,i3))) - &
                             dtimeoverdx2(i2) * ((flux2_rhov1( i2+1,i1,i3)-flux2_rhov1( i2,i1,i3)) + &
                             f(i1,i2,i3)      *  (flux2_rhov1B(i2+1,i1,i3)-flux2_rhov1B(i2,i1,i3))) - &
                             dtimeoverdx3(i3) * ((flux3_rhov1( i3+1,i2,i1)-flux3_rhov1( i3,i2,i1)) + &
                             f(i1,i2,i3)      *  (flux3_rhov1B(i3+1,i2,i1)-flux3_rhov1B(i3,i2,i1))) + &
                             dtime            * source_rhov1 &
             )/rhoout
      v2out =(rho(i1,i2,i3) * v2(i1,i2,i3) - &
                             dtimeoverdx1(i1) * ((flux1_rhov2( i1+1,i2,i3)-flux1_rhov2( i1,i2,i3)) + &
                             f(i1,i2,i3)      *  (flux1_rhov2B(i1+1,i2,i3)-flux1_rhov2B(i1,i2,i3))) - &
                             dtimeoverdx2(i2) * ((flux2_rhov2( i2+1,i1,i3)-flux2_rhov2( i2,i1,i3)) + &
                             f(i1,i2,i3)      *  (flux2_rhov2B(i2+1,i1,i3)-flux2_rhov2B(i2,i1,i3))) - &
                             dtimeoverdx3(i3) * ((flux3_rhov2( i3+1,i2,i1)-flux3_rhov2( i3,i2,i1)) + &
                             f(i1,i2,i3)      *  (flux3_rhov2B(i3+1,i2,i1)-flux3_rhov2B(i3,i2,i1))) + &
                             dtime            * source_rhov2 &
             )/rhoout
      v3out =(rho(i1,i2,i3) * v3(i1,i2,i3) - &
                             dtimeoverdx1(i1) * ((flux1_rhov3( i1+1,i2,i3)-flux1_rhov3( i1,i2,i3)) + &
                             f(i1,i2,i3)      *  (flux1_rhov3B(i1+1,i2,i3)-flux1_rhov3B(i1,i2,i3))) - &
                             dtimeoverdx2(i2) * ((flux2_rhov3( i2+1,i1,i3)-flux2_rhov3( i2,i1,i3)) + &
                             f(i1,i2,i3)      *  (flux2_rhov3B(i2+1,i1,i3)-flux2_rhov3B(i2,i1,i3))) - &
                             dtimeoverdx3(i3) * ((flux3_rhov3( i3+1,i2,i1)-flux3_rhov3( i3,i2,i1)) + &
                             f(i1,i2,i3)      *  (flux3_rhov3B(i3+1,i2,i1)-flux3_rhov3B(i3,i2,i1))) + &
                             dtime            * source_rhov3 &
             )/rhoout
      Bc1out=Bc1(i1,i2,i3) - dtimeoverdx1(i1) * (flux1_Bc1(  i1+1,i2,i3)-flux1_Bc1(  i1,i2,i3)) + &
                             source1Bc1_l(i1+1,i2,i3) + source1Bc1_r(i1,i2,i3) - &
                             dtimeoverdx2(i2) * (flux2_Bc1(  i2+1,i1,i3)-flux2_Bc1(  i2,i1,i3)) + &
                             source2Bc1_l(i2+1,i1,i3) + source2Bc1_r(i2,i1,i3) - &
                             dtimeoverdx3(i3) * (flux3_Bc1(  i3+1,i2,i1)-flux3_Bc1(  i3,i2,i1)) + &
                             source3Bc1_l(i3+1,i2,i1) + source3Bc1_r(i3,i2,i1)
      Bc2out=Bc2(i1,i2,i3) - dtimeoverdx1(i1) * (flux1_Bc2(  i1+1,i2,i3)-flux1_Bc2(  i1,i2,i3)) + &
                             source1Bc2_l(i1+1,i2,i3) + source1Bc2_r(i1,i2,i3) - &
                             dtimeoverdx2(i2) * (flux2_Bc2(  i2+1,i1,i3)-flux2_Bc2(  i2,i1,i3)) + &
                             source2Bc2_l(i2+1,i1,i3) + source2Bc2_r(i2,i1,i3) - &
                             dtimeoverdx3(i3) * (flux3_Bc2(  i3+1,i2,i1)-flux3_Bc2(  i3,i2,i1)) + &
                             source3Bc2_l(i3+1,i2,i1) + source3Bc2_r(i3,i2,i1)
      Bc3out=Bc3(i1,i2,i3) - dtimeoverdx1(i1) * (flux1_Bc3(  i1+1,i2,i3)-flux1_Bc3(  i1,i2,i3)) + &
                             source1Bc3_l(i1+1,i2,i3) + source1Bc3_r(i1,i2,i3) - &
                             dtimeoverdx2(i2) * (flux2_Bc3(  i2+1,i1,i3)-flux2_Bc3(  i2,i1,i3)) + &
                             source2Bc3_l(i2+1,i1,i3) + source2Bc3_r(i2,i1,i3) - &
                             dtimeoverdx3(i3) * (flux3_Bc3(  i3+1,i2,i1)-flux3_Bc3(  i3,i2,i1)) + &
                             source3Bc3_l(i3+1,i2,i1) + source3Bc3_r(i3,i2,i1)
      !
      if (model%energy_switch(i1,i2,i3)) then
        !
        ! --- Equation for total energy ---
        eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) + &
                0.5 * rho(i1,i2,i3) * (v1(i1,i2,i3)**2 + v2(i1,i2,i3)**2 + v3(i1,i2,i3)**2) - &
                dtimeoverdx1(i1) * (flux1_rhoeikm(i1+1,i2,i3)-flux1_rhoeikm(i1,i2,i3)) - &
                dtimeoverdx2(i2) * (flux2_rhoeikm(i2+1,i1,i3)-flux2_rhoeikm(i2,i1,i3)) - &
                dtimeoverdx3(i3) * (flux3_rhoeikm(i3+1,i2,i1)-flux3_rhoeikm(i3,i2,i1)) + &
                0.5 * (Bc1(i1,i2,i3) + Bc1out) * &
                              (dtimeoverdx1(i1) * (flux1_Bc1(  i1+1,i2,i3)-flux1_Bc1(  i1,i2,i3)) - &
                               source1Bc1_l(i1+1,i2,i3) - source1Bc1_r(i1,i2,i3) + &
                               dtimeoverdx2(i2) * (flux2_Bc1(  i2+1,i1,i3)-flux2_Bc1(  i2,i1,i3)) - &
                               source2Bc1_l(i2+1,i1,i3) - source2Bc1_r(i2,i1,i3) + &
                               dtimeoverdx3(i3) * (flux3_Bc1(  i3+1,i2,i1)-flux3_Bc1(  i3,i2,i1)) - &
                               source3Bc1_l(i3+1,i2,i1) - source3Bc1_r(i3,i2,i1)) + &
                0.5 * (Bc2(i1,i2,i3) + Bc2out) * &
                              (dtimeoverdx1(i1) * (flux1_Bc2(  i1+1,i2,i3)-flux1_Bc2(  i1,i2,i3)) - &
                               source1Bc2_l(i1+1,i2,i3) - source1Bc2_r(i1,i2,i3) + &
                               dtimeoverdx2(i2) * (flux2_Bc2(  i2+1,i1,i3)-flux2_Bc2(  i2,i1,i3)) - &
                               source2Bc2_l(i2+1,i1,i3) - source2Bc2_r(i2,i1,i3) + &
                               dtimeoverdx3(i3) * (flux3_Bc2(  i3+1,i2,i1)-flux3_Bc2(  i3,i2,i1)) - &
                               source3Bc2_l(i3+1,i2,i1) - source3Bc2_r(i3,i2,i1)) + &
                0.5 * (Bc3(i1,i2,i3) + Bc3out) * &
                              (dtimeoverdx1(i1) * (flux1_Bc3(  i1+1,i2,i3)-flux1_Bc3(  i1,i2,i3)) - &
                               source1Bc3_l(i1+1,i2,i3) - source1Bc3_r(i1,i2,i3) + &
                               dtimeoverdx2(i2) * (flux2_Bc3(  i2+1,i1,i3)-flux2_Bc3(  i2,i1,i3)) - &
                               source2Bc3_l(i2+1,i1,i3) - source2Bc3_r(i2,i1,i3) + &
                               dtimeoverdx3(i3) * (flux3_Bc3(  i3+1,i2,i1)-flux3_Bc3(  i3,i2,i1)) - &
                               source3Bc3_l(i3+1,i2,i1) - source3Bc3_r(i3,i2,i1)) + &
                dtime           * source_eikm &
               )/rhoout - 0.5 * (v1out**2 + v2out**2 + v3out**2)
      else
        !
        ! --- Equation for thermal energy (without any dissipation terms) ---
        eiout =(rho(i1,i2,i3) * ei(i1,i2,i3) - &
                              dtimeoverdx1(i1) * (flux1_rhoei(i1+1,i2,i3)-flux1_rhoei(i1,i2,i3)) - &
                              dtimeoverdx2(i2) * (flux2_rhoei(i2+1,i1,i3)-flux2_rhoei(i2,i1,i3)) - &
                              dtimeoverdx3(i3) * (flux3_rhoei(i3+1,i2,i1)-flux3_rhoei(i3,i2,i1)) + &
                              dtime           * source_ei) / rhoout
      endif
      !
      model%rho(i1,i2,i3)=rhoout
      model%v1( i1,i2,i3)=v1out
      model%v2( i1,i2,i3)=v2out
      model%v3( i1,i2,i3)=v3out
      model%Bc1(i1,i2,i3)=Bc1out
      model%Bc2(i1,i2,i3)=Bc2out
      model%Bc3(i1,i2,i3)=Bc3out
      model%ei( i1,i2,i3)=eiout
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Save fluxes for flux-interpolated transport step ---
do i3 = m3, n3
  do i2 = m2, n2
    do i1 = m1, n1 + 1
      model%flux_B21(i1, i2, i3) = flux1_Bc2(i1, i2, i3)
      model%flux_B31(i1, i2, i3) = flux1_Bc3(i1, i2, i3)
    end do ! i1
  end do ! i2
end do ! i3
!
do i3 = m3, n3
  do i2 = m2, n2 + 1
    do i1 = m1, n1
      model%flux_B12(i1, i2, i3) = flux2_Bc1(i2, i1, i3)
      model%flux_B32(i1, i2, i3) = flux2_Bc3(i2, i1, i3)
    end do ! i1
  end do ! i2
end do ! i3
!
do i3 = m3, n3 + 1
  do i2 = m2, n2
    do i1 = m1, n1
      model%flux_B23(i1, i2, i3) = flux3_Bc2(i3, i2, i1)
      model%flux_B13(i1, i2, i3) = flux3_Bc1(i3, i2, i1)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Check if internal energy is below minimum table value anywhere ---
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D: update')
!#
# endif
!#
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*dtime
  outstr0='Internal energy below minimum table value'
  goto 998
endif
!
! --- Check internal energy ---
if (par%C_maxeiChange > 0.0) then
  dei_min=min(minval((model%ei(m1:n1,:,:)-ei(m1:n1,:,:))*rho(m1:n1,:,:)/P(m1:n1,:,:)),0.0)
  dtime_limit=min(dtime_limit, &
                  dtime*par%C_maxeiChange/(-dei_min+10.*tiny(dei_min))*0.999)
  if (dei_min < -par%C_maxeiChange) then
    ierr0=2
    outstr0='Drop in internal energy too large'
    goto 999
  endif
endif
!
! === Boundary conditions VI =======================================================================
if (bound_l1 == bound_transmitting2 .and. m1 == prop%m1) then
  model%v1(m1,m2:n2,m3:n3)=v1(m1,m2:n2,m3:n3) + dtime*cs(m1,m2:n2,m3:n3) * &
                                         (       v1(m1+1,m2:n2,m3:n3)-       v1(m1,m2:n2,m3:n3))/ &
                                         (model%xc1(m1+1            )-model%xc1(m1            ))
endif
if (bound_r1 == bound_transmitting2 .and. n1 == prop%n1) then
  model%v1(n1,m2:n2,m3:n3)=v1(n1,m2:n2,m3:n3) - dtime*cs(n1,m2:n2,m3:n3) * &
                                         (       v1(n1-1,m2:n2,m3:n3)-       v1(n1,m2:n2,m3:n3))/ &
                                         (model%xc1(n1-1            )-model%xc1(n1            ))
endif
if (bound_l2 == bound_transmitting2 .and. m2 == prop%m2) then
  model%v2(m1:n1,m2,m3:n3)=v2(m1:n1,m2,m3:n3) + dtime*cs(m1:n1,m2,m3:n3) * &
                                         (       v2(m1:n1,m2+1,m3:n3)-       v2(m1:n1,m2,m3:n3))/ &
                                         (model%xc2(      m2+1      )-model%xc2(      m2      ))
endif
if (bound_r2 == bound_transmitting2 .and. n2 == prop%n2) then
  model%v2(m1:n1,n2,m3:n3)=v2(m1:n1,n2,m3:n3) - dtime*cs(m1:n1,n2,m3:n3) * &
                                         (       v2(m1:n1,n2-1,m3:n3)-       v2(m1:n1,n2,m3:n3))/ &
                                         (model%xc2(      n2-1      )-model%xc2(      n2      ))
endif
if (bound_l3 == bound_transmitting2 .and. m3 == prop%m3) then
  model%v3(m1:n1,m2:n2,m3)=v3(m1:n1,m2:n2,m3) + dtime*cs(m1:n1,m2:n2,m3) * &
                                         (       v3(m1:n1,m2:n2,m3+1)-       v3(m1:n1,m2:n2,m3))/ &
                                         (model%xc3(            m3+1)-model%xc3(            m3))
endif
if (bound_r3 == bound_transmitting2 .and. n3 == prop%n3) then
  model%v3(m1:n1,m2:n2,n3)=v3(m1:n1,m2:n2,n3) - dtime*cs(m1:n1,m2:n2,n3) * &
                                         (       v3(m1:n1,m2:n2,n3-1)-       v3(m1:n1,m2:n2,n3))/ &
                                         (model%xc3(            n3-1)-model%xc3(            n3))
endif
!
!
999 dtime_limit=par%C_Courant*dtime_limit
!
998 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('MHD: HLL3D:')
!#
# endif
!#
!
! --- Delete boxes for x2 and x3 sweeps ---
call rhd_box_Delete(model_2)
call rhd_box_Delete(model_3)
!
end subroutine rhd_mhd_HLL3Dstep


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Res2DStep(model, action, dtime, dtime_limit, &
                             output, AreaFraction1, AreaFraction2, &
                             AreaFraction3, dat, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Res2DStep ('rhd_magneto-hydrodynamics_Resistivity_2D_Step')
!
! PURPOSE:
!   Perform 2D Resistivity step.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Res2DStep(model, action, dtime, dtime_limit, &
!                          output=output, AreaFraction1=AreaFraction1, &
!                          AreaFraction2=AreaFraction2, AreaFraction3=AreaFraction3, dat=dat, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!                   100:   Pressure becomes non-positive
!                   101:   Pressure derivative dPdrho|ei becomes non-positive
!                   102:   Pressure derivative dPdei|rho becomes non-positive
!                   103:   Density becomes non-positive
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_mhd_Res3Doutput:   Extract some averaged quantities of resistivity step
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2010-12-16 (W.S.) First version
!   2013-03-05 (B.F.) Parameter outradialc removed
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
use tabinter_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag
real, dimension(model%m1-model%nghost1: &
                model%n1+model%nghost1, &
                model%m2:model%n2, &
                model%m3-model%nghost3: &
                model%n3+model%nghost3) :: P, dPdrho, dPdei, T, dTdei, s
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2,   &
                model%m3:model%n3)      :: flux_B21, flux_B31, flux_rhoeikm1
real, dimension(model%m1:model%n1,   &
                model%m2:model%n2+1, &
                model%m3:model%n3)      :: flux_rhoeikm2
real, dimension(model%m1:model%n1  , &
                model%m2:model%n2,   &
                model%m3:model%n3+1)    :: flux_B13, flux_B23, flux_rhoeikm3
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, n
real                                    :: ei_new, einew_min, eiTable_min, &
                                           B1_out, B2_out, B3_out, &
                                           j1, j2, j3, nu_B, nu_E, dx1, dx2, dx3, &
                                           dtoverdx1, dtoverdx3, &
                                           inv_dt1, inv_dt3, inv_dt, &
                                           AreaFraction10, AreaFraction30
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Default values for keywords ---
if (.not. present(output)) then
  Output_flag=.false.
else
  Output_flag=output
endif
if (.not. present(AreaFraction1)) then
  AreaFraction10=1.0
else
  AreaFraction10=AreaFraction1
endif
if (.not. present(AreaFraction3)) then
  AreaFraction30=1.0
else
  AreaFraction30=AreaFraction3
endif
!
inv_dt1=0.0
inv_dt3=0.0
!
dtime_limit=huge(dtime_limit)
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
i2=m2
!
flux_rhoeikm2=0.0
!
! --- Fluxes in x1 direction ---
!
do i1=m1,n1+1
  do i3=m3,n3
    !
    ! --- Spatial steps ---
    dx1 = 0.5*(model%xb1(i1+1) - model%xb1(i1-1))
    dx2 =      model%xb2(i2+1) - model%xb2(i2  )
    dx3 =      model%xb3(i3+1) - model%xb3(i3  )
    !
    ! --- Current densities ---
    j1=-0.25*(model%Bc2(i1  ,i2,i3+1) - model%Bc2(i1  ,i2,i3-1) + &
              model%Bc2(i1-1,i2,i3+1) - model%Bc2(i1-1,i2,i3-1)) / dx3
    j2= 0.25*(model%Bc1(i1  ,i2,i3+1) - model%Bc1(i1  ,i2,i3-1) + &
              model%Bc1(i1-1,i2,i3+1) - model%Bc1(i1-1,i2,i3-1)) / dx3 &
            -(model%Bc3(i1  ,i2,i3  ) - model%Bc3(i1-1,i2,i3  )) / dx1
    j3=      (model%Bc2(i1  ,i2,i3  ) - model%Bc2(i1-1,i2,i3  )) / dx1
    !
    ! \label{f90:c_resb__useMHD1}
    ! --- Diffusion coefficients ---
    nu_B=par%C_resBconstant + par%C_resB*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
                    sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1  ,i2,i3),model%rho(i1-1,i2,i3)))
    nu_E=par%C_resEpsilon*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
                    sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1  ,i2,i3),model%rho(i1-1,i2,i3)))
    !
    ! --- Diffusive fluxes for magnetic fields ---
    flux_B21(i1,i2,i3)=-nu_B*j3
    flux_B31(i1,i2,i3)= nu_B*j2
    !
    ! --- Add diffusive fluxes to nondiffusive fluxes ---
    model%flux_B31(i1,i2,i3)=model%flux_B31(i1,i2,i3)+nu_B*j2
    !
    ! --- Energy flux x1 direction ---
    flux_rhoeikm1(i1,i2,i3)= 0.5*(model%Bc2(i1 ,i2,i3)+model%Bc2(i1-1,i2,i3)) * &
                            flux_B21(i1,i2,i3) + &
                             0.5*(model%Bc3(i1 ,i2,i3)+model%Bc3(i1-1,i2,i3)) * &
                            flux_B31(i1,i2,i3) - &
          nu_E*(model%rho(i1  ,i2,i3)*model%ei(i1  ,i2,i3) - &
                model%rho(i1-1,i2,i3)*model%ei(i1-1,i2,i3)) / dx1
    !
    ! --- Maximum timestep ---
    inv_dt1=max(inv_dt1,2.0*max(nu_B,nu_E)/dx1**2)
    !
  end do ! i3
end do ! i1
!
! --- Fluxes in x3 direction ---
!
do i1=m1,n1
  do i3=m3,n3+1
    !
    ! --- Spatial steps ---
    dx1 =      model%xb1(i1+1) - model%xb1(i1  )
    dx2 =      model%xb2(i2+1) - model%xb2(i2  )
    dx3 = 0.5*(model%xb3(i3+1) - model%xb3(i3-1))
    !
    ! --- Current densities ---
    j1=     -(model%Bc2(i1  ,i2,i3  ) - model%Bc2(i1  ,i2,i3-1)) / dx3
    j2=      (model%Bc1(i1  ,i2,i3  ) - model%Bc1(i1  ,i2,i3-1)) / dx3 &
       -0.25*(model%Bc3(i1+1,i2,i3  ) - model%Bc3(i1-1,i2,i3  ) + &
              model%Bc3(i1+1,i2,i3-1) - model%Bc3(i1-1,i2,i3-1)) / dx1
    j3= 0.25*(model%Bc2(i1+1,i2,i3  ) - model%Bc2(i1-1,i2,i3  ) + &
              model%Bc2(i1+1,i2,i3-1) - model%Bc2(i1-1,i2,i3-1)) / dx1
    !
    ! --- Diffusion coefficients ---
    nu_B=par%C_resBconstant + par%C_resB*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
                    sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1,i2,i3  ),model%rho(i1,i2,i3-1)))
    nu_E=par%C_resEpsilon*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
                    sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1,i2,i3  ),model%rho(i1,i2,i3-1)))

    !
    ! --- Diffusive fluxes for magnetic fields ---
    flux_B13(i1,i2,i3)=-nu_B*j2
    flux_B23(i1,i2,i3)= nu_B*j1
    !
    ! --- Add diffusive fluxes to nondiffusive fluxes ---
    model%flux_B13(i1,i2,i3)=model%flux_B13(i1,i2,i3)-nu_B*j2
    !
    ! --- Energy flux x3 direction ---
    flux_rhoeikm3(i1,i2,i3)= 0.5*(model%Bc1(i1,i2,i3  )+model%Bc1(i1,i2,i3-1)) * &
                            flux_B13(i1,i2,i3) + &
                             0.5*(model%Bc2(i1,i2,i3  )+model%Bc2(i1,i2,i3-1)) * &
                            flux_B23(i1,i2,i3) - &
          nu_E*(model%rho(i1,i2,i3  )*model%ei(i1,i2,i3  ) - &
                model%rho(i1,i2,i3-1)*model%ei(i1,i2,i3-1)) / dx3
    !
    ! --- Maximum timestep ---
    inv_dt3=max(inv_dt3,2.0*max(nu_B,nu_E)/dx3**2)
    !
  end do ! i3
end do ! i1
!
! --- Update cell centered magnetic field and internal energy ---
!
do i1=m1,n1
  do i3=m3,n3
    !
    dtoverdx1 = dtime / (model%xb1(i1+1) - model%xb1(i1  ))
    dtoverdx3 = dtime / (model%xb3(i3+1) - model%xb3(i3  ))
    !
    ! --- Update cell centered magnetic field ---
    B1_out=model%Bc1(i1,i2,i3) - dtoverdx3 * (flux_B13(i1  ,i2,i3+1) - flux_B13(i1  ,i2,i3  ))
    B2_out=model%Bc2(i1,i2,i3) - dtoverdx1 * (flux_B21(i1+1,i2,i3  ) - flux_B21(i1  ,i2,i3  )) - &
                                 dtoverdx3 * (flux_B23(i1  ,i2,i3+1) - flux_B23(i1  ,i2,i3  ))
    B3_out=model%Bc3(i1,i2,i3) - dtoverdx1 * (flux_B31(i1+1,i2,i3  ) - flux_B31(i1  ,i2,i3  ))
    !
    ! --- Update internal energy ---
    if (par%A%hdScheme == hdScheme_RoeMHD .or. par%A%hdScheme == hdScheme_HLLMHD) then
      !
      model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + ( &
                  0.5 * (model%Bc1(i1,i2,i3)**2+ &
                         model%Bc2(i1,i2,i3)**2+model%Bc3(i1,i2,i3)**2) - &
                  0.5 * (B1_out**2 + B2_out**2 + B3_out**2) - &
                        dtoverdx1 * (flux_rhoeikm1(i1+1,i2,i3  ) - flux_rhoeikm1(i1  ,i2,i3  )) - &
                        dtoverdx3 * (flux_rhoeikm3(i1  ,i2,i3+1) - flux_rhoeikm3(i1  ,i2,i3  )) &
                                              ) / model%rho(i1  ,i2  ,i3  )
    endif
    !
    model%Bc1(i1,i2,i3) = B1_out
    model%Bc2(i1,i2,i3) = B2_out
    model%Bc3(i1,i2,i3) = B3_out
    !
  end do ! i3
end do ! i1
!
! \label{f90:c_rescourant__useMHD1}
! --- Check for time step limit violation ---
inv_dt=inv_dt1+inv_dt3
if (inv_dt>0.0) then
  dtime_limit=par%C_resCourant/inv_dt
  if (dtime>dtime_limit*par%C_resCourantMax/par%C_resCourant) then
    ierr0=1
    outstr0='Time step exceeds resistive time scale'
    goto 999
  endif
endif
!
! --- Check if internal energy is below minimum table value anywhere ---
call tabinter_getinfo(x2min=eiTable_min)
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
!
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*min(dtime, dtime_limit)
  outstr0='Internal energy below minimum table value'
endif
!
! --- Output of additional data ---
if ((Output_flag) .and. present(dat)) then
  call rhd_mhd_Res3Doutput(model, &
                           AreaFraction10, 1.0, AreaFraction30, &
                           flux_rhoeikm1, flux_rhoeikm2, flux_rhoeikm3, dat)
endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_mhd_Res2DStep


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Res3DStep(model, action, dtime, dtime_limit, &
                             output, AreaFraction1, AreaFraction2, &
                             AreaFraction3, dat, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Res3DStep ('rhd_magneto-hydrodynamics_Resistivity_3D_Step')
!
! PURPOSE:
!   Perform 3D magnetohydrodynamics resistivity step.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Res3DStep(model, action, dtime, dtime_limit, &
!                          output=output, AreaFraction=AreaFraction, &
!                          datx=datx, datr=datr, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!                   100:   Pressure becomes non-positive
!                   101:   Pressure derivative dPdrho|ei becomes non-positive
!                   102:   Pressure derivative dPdei|rho becomes non-positive
!                   103:   Density becomes non-positive
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_mhd_Res3Doutput:   Extract some averaged quantities of resistivity step
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   tabinter_module:       Routines for handling of interpolation coefficients
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2010-12-16 (W.S.) First version
!   2013-03-05 (B.F. Lyon) Parameter outradialc removed
!   2017-08-23 (B.F.) Bugfix: pass AreaFraction20 to rhd_mhd_Res3Doutput
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
use tabinter_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2,   &
                model%m3:model%n3)      :: flux_B21, flux_B31, flux_rhoeikm1
real, dimension(model%m1:model%n1  , &
                model%m2:model%n2+1, &
                model%m3:model%n3)      :: flux_B12, flux_B32, flux_rhoeikm2
real, dimension(model%m1:model%n1  , &
                model%m2:model%n2,   &
                model%m3:model%n3+1)    :: flux_B13, flux_B23, flux_rhoeikm3
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, &
                                           i1p, i1m, i2p, i2m, i3p, i3m, n
real                                    :: ei_new, einew_min, eiTable_min, &
                                           B1_out, B2_out, B3_out, &
                                           j1, j2, j3, nu_B, nu_E, dx1, dx2, dx3, &
                                           dtoverdx1, dtoverdx2, dtoverdx3, &
                                           inv_dt1, inv_dt2, inv_dt3,inv_dt, &
                                           AreaFraction10, AreaFraction20, AreaFraction30
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- No error occurred so far ---
ierr0=0
outstr0=''
!
! --- Default values for keywords ---
if (.not. present(output)) then
  Output_flag=.false.
else
  Output_flag=output
endif
!
if (.not. present(AreaFraction1)) then
  AreaFraction10=1.0
else
  AreaFraction10=AreaFraction1
endif
if (.not. present(AreaFraction2)) then
  AreaFraction20=1.0
else
  AreaFraction20=AreaFraction2
endif
if (.not. present(AreaFraction3)) then
  AreaFraction30=1.0
else
  AreaFraction30=AreaFraction3
endif
!
inv_dt1=0.0
inv_dt2=0.0
inv_dt3=0.0
!
dtime_limit=huge(dtime_limit)
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Fluxes in x1 direction ---
!
do i1=m1,n1+1
  do i2=m2,n2
    do i3=m3,n3
      !
      ! --- Spatial steps ---
      dx1 = 0.5*(model%xb1(i1+1) - model%xb1(i1-1))
      dx2 =      model%xb2(i2+1) - model%xb2(i2  )
      dx3 =      model%xb3(i3+1) - model%xb3(i3  )
      !
      ! --- Current densities ---
      j1= 0.25*(model%Bc3(i1  ,i2+1,i3  ) - model%Bc3(i1  ,i2-1,i3  ) + &
                model%Bc3(i1-1,i2+1,i3  ) - model%Bc3(i1-1,i2-1,i3  )) / dx2 &
         -0.25*(model%Bc2(i1  ,i2  ,i3+1) - model%Bc2(i1  ,i2  ,i3-1) + &
                model%Bc2(i1-1,i2  ,i3+1) - model%Bc2(i1-1,i2  ,i3-1)) / dx3
      j2= 0.25*(model%Bc1(i1  ,i2  ,i3+1) - model%Bc1(i1  ,i2  ,i3-1) + &
                model%Bc1(i1-1,i2  ,i3+1) - model%Bc1(i1-1,i2  ,i3-1)) / dx3 &
              -(model%Bc3(i1  ,i2  ,i3  ) - model%Bc3(i1-1,i2  ,i3  )) / dx1
      j3=      (model%Bc2(i1  ,i2  ,i3  ) - model%Bc2(i1-1,i2  ,i3  )) / dx1 &
         -0.25*(model%Bc1(i1  ,i2+1,i3  ) - model%Bc1(i1  ,i2-1,i3  ) + &
                model%Bc1(i1-1,i2+1,i3  ) - model%Bc1(i1-1,i2-1,i3  )) / dx2
      !
      ! \label{f90:c_resb__useMHD2}
      ! --- Diffusion coefficients ---
      nu_B=par%C_resBconstant + par%C_resB*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
              sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1  ,i2,i3),model%rho(i1-1,i2,i3)))
      nu_E=par%C_resEpsilon*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
              sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1  ,i2,i3),model%rho(i1-1,i2,i3)))
      !
      ! --- diffusive fluxes for magnetic fields ---
      flux_B21(i1,i2,i3)=-nu_B*j3
      flux_B31(i1,i2,i3)= nu_B*j2
      !
      ! --- add diffusive fluxes to nondiffusive fluxes ---
      model%flux_B21(i1,i2,i3)=model%flux_B21(i1,i2,i3)-nu_B*j3
      model%flux_B31(i1,i2,i3)=model%flux_B31(i1,i2,i3)+nu_B*j2
      !
      !
      ! --- Energy flux x1 direction ---
      flux_rhoeikm1(i1,i2,i3)= 0.5*(model%Bc2(i1  ,i2,i3)+model%Bc2(i1-1,i2,i3)) * &
                                 flux_B21(i1,i2,i3) + &
                               0.5*(model%Bc3(i1  ,i2,i3)+model%Bc3(i1-1,i2,i3)) * &
                                 flux_B31(i1,i2,i3) - &
          nu_E * (model%rho(i1  ,i2,i3)*model%ei(i1  ,i2,i3) - &
                  model%rho(i1-1,i2,i3)*model%ei(i1-1,i2,i3)) / dx1
      !
      ! --- Maximum timestep ---
      inv_dt1=max(inv_dt1,2.0*max(nu_B,nu_E)/dx1**2)
      !
    end do ! i3
  end do ! i2
end do ! i1
!
! --- Fluxes in x2 direction ---
!
do i1=m1,n1
  do i2=m2,n2+1
    do i3=m3,n3
      !
      ! --- Spatial steps ---
      dx1 =      model%xb1(i1+1) - model%xb1(i1  )
      dx2 = 0.5*(model%xb2(i2+1) - model%xb2(i2-1))
      dx3 =      model%xb3(i3+1) - model%xb3(i3  )
      !
      ! --- Current densities ---
      j1=      (model%Bc3(i1  ,i2  ,i3  ) - model%Bc3(i1  ,i2-1,i3  )) / dx2 &
         -0.25*(model%Bc2(i1  ,i2  ,i3+1) - model%Bc2(i1  ,i2  ,i3-1) + &
                model%Bc2(i1  ,i2-1,i3+1) - model%Bc2(i1  ,i2-1,i3-1)) / dx3
      j2= 0.25*(model%Bc1(i1  ,i2  ,i3+1) - model%Bc1(i1  ,i2  ,i3-1) + &
                model%Bc1(i1  ,i2-1,i3+1) - model%Bc1(i1  ,i2-1,i3-1)) / dx3 &
         -0.25*(model%Bc3(i1+1,i2  ,i3  ) - model%Bc3(i1-1,i2  ,i3  ) + &
                model%Bc3(i1+1,i2-1,i3  ) - model%Bc3(i1-1,i2-1,i3  )) / dx1
      j3= 0.25*(model%Bc2(i1+1,i2  ,i3  ) - model%Bc2(i1-1,i2  ,i3  ) + &
                model%Bc2(i1+1,i2-1,i3  ) - model%Bc2(i1-1,i2-1,i3  )) / dx1 &
              -(model%Bc1(i1  ,i2  ,i3  ) - model%Bc1(i1  ,i2-1,i3  )) / dx2
      !
      ! --- Diffusion coefficients ---
      nu_B=par%C_resBconstant + par%C_resB*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
              sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1,i2  ,i3),model%rho(i1,i2-1,i3)))
      nu_E=par%C_resEpsilon*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
              sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1,i2  ,i3),model%rho(i1,i2-1,i3)))
      !
      ! --- Diffusive fluxes for magnetic fields ---
      flux_B12(i1,i2,i3)= nu_B*j3
      flux_B32(i1,i2,i3)=-nu_B*j1
      !
      ! --- Add diffusive fluxes to nondiffusive fluxes ---
      model%flux_B12(i1,i2,i3)=model%flux_B12(i1,i2,i3)+nu_B*j3
      model%flux_B32(i1,i2,i3)=model%flux_B32(i1,i2,i3)-nu_B*j1
      !
      !
      ! --- Energy flux x2 direction ---
      flux_rhoeikm2(i1,i2,i3)= 0.5*(model%Bc1(i1,i2  ,i3)+model%Bc1(i1,i2-1,i3)) * &
                                 flux_B12(i1,i2,i3) + &
                               0.5*(model%Bc3(i1,i2  ,i3)+model%Bc3(i1,i2-1,i3)) * &
                                 flux_B32(i1,i2,i3) - &
          nu_E * (model%rho(i1,i2  ,i3)*model%ei(i1,i2  ,i3) - &
                  model%rho(i1,i2-1,i3)*model%ei(i1,i2-1,i3)) / dx2
      !
      ! --- Maximum timestep ---
      inv_dt2=max(inv_dt2,2.0*max(nu_B,nu_E)/dx2**2)
      !
    end do ! i3
  end do ! i2
end do ! i1
!
! --- Fluxes in x3 direction ---
!
do i1=m1,n1
  do i2=m2,n2
    do i3=m3,n3+1
      !
      ! --- Spatial steps ---
      dx1 =      model%xb1(i1+1) - model%xb1(i1  )
      dx2 =      model%xb2(i2+1) - model%xb2(i2  )
      dx3 = 0.5*(model%xb3(i3+1) - model%xb3(i3-1))
      !
      ! --- Current densities ---
      j1= 0.25*(model%Bc3(i1  ,i2+1,i3  ) - model%Bc3(i1  ,i2-1,i3  ) + &
                model%Bc3(i1  ,i2+1,i3-1) - model%Bc3(i1  ,i2-1,i3-1)) / dx2 &
              -(model%Bc2(i1  ,i2  ,i3  ) - model%Bc2(i1  ,i2  ,i3-1)) / dx3
      j2=      (model%Bc1(i1  ,i2  ,i3  ) - model%Bc1(i1  ,i2  ,i3-1)) / dx3 &
         -0.25*(model%Bc3(i1+1,i2  ,i3  ) - model%Bc3(i1-1,i2  ,i3  ) + &
                model%Bc3(i1+1,i2  ,i3-1) - model%Bc3(i1-1,i2  ,i3-1)) / dx1
      j3= 0.25*(model%Bc2(i1+1,i2  ,i3  ) - model%Bc2(i1-1,i2  ,i3  ) + &
                model%Bc2(i1+1,i2  ,i3-1) - model%Bc3(i1-1,i2  ,i3-1)) / dx1 &
         -0.25*(model%Bc1(i1  ,i2+1,i3  ) - model%Bc1(i1  ,i2-1,i3  ) + &
                model%Bc1(i1  ,i2+1,i3-1) - model%Bc1(i1  ,i2-1,i3-1)) / dx2
      !
      ! --- Diffusion coefficients ---
      nu_B=par%C_resBconstant + par%C_resB*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
              sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1,i2,i3  ),model%rho(i1,i2,i3-1)))
      nu_E=par%C_resEpsilon*min(dx1,dx2,dx3)*max(dx1,dx2,dx3) * &
              sqrt((j1**2+j2**2+j3**2)/min(model%rho(i1,i2,i3  ),model%rho(i1,i2,i3-1)))
      !
      ! --- Fluxes for magnetic fields ---
      flux_B13(i1,i2,i3)=-nu_B*j2
      flux_B23(i1,i2,i3)= nu_B*j1
      !
      ! --- Add diffusive fluxes to nondiffusive fluxes ---
      model%flux_B13(i1,i2,i3)=model%flux_B13(i1,i2,i3)-nu_B*j2
      model%flux_B23(i1,i2,i3)=model%flux_B23(i1,i2,i3)+nu_B*j1
      !
      ! --- Energy flux x1 direction ---
      flux_rhoeikm3(i1,i2,i3)= 0.5*(model%Bc1(i1,i2,i3  )+model%Bc1(i1,i2,i3-1)) * &
                                 flux_B13(i1,i2,i3) + &
                               0.5*(model%Bc2(i1,i2,i3  )+model%Bc2(i1,i2,i3-1)) * &
                                 flux_B23(i1,i2,i3) - &
          nu_E * (model%rho(i1,i2,i3  )*model%ei(i1,i2,i3  ) - &
                  model%rho(i1,i2,i3-1)*model%ei(i1,i2,i3-1)) / dx3
      !
      ! --- Maximum timestep ---
      inv_dt3=max(inv_dt3,2.0*max(nu_B,nu_E)/dx3**2)
      !
    end do ! i3
  end do ! i2
end do ! i1
!
! --- Update of cell centered magnetic field and internal energy ---
!
do i1=m1,n1
  do i2=m2,n2
    do i3=m3,n3
      !
      dtoverdx1 = dtime / (model%xb1(i1+1) - model%xb1(i1  ))
      dtoverdx2 = dtime / (model%xb2(i2+1) - model%xb2(i2  ))
      dtoverdx3 = dtime / (model%xb3(i3+1) - model%xb3(i3  ))
      !
      ! --- Update cell centered magnetic field ---
      B1_out=model%Bc1(i1,i2,i3) - &
                       dtoverdx2 * (flux_B12(i1  ,i2+1,i3  ) - flux_B12(i1  ,i2  ,i3  )) - &
                       dtoverdx3 * (flux_B13(i1  ,i2  ,i3+1) - flux_B13(i1  ,i2  ,i3  ))
      B2_out=model%Bc2(i1,i2,i3) - &
                       dtoverdx1 * (flux_B21(i1+1,i2  ,i3  ) - flux_B21(i1  ,i2  ,i3  )) - &
                       dtoverdx3 * (flux_B23(i1  ,i2  ,i3+1) - flux_B23(i1  ,i2  ,i3  ))
      B3_out=model%Bc3(i1,i2,i3) - &
                       dtoverdx1 * (flux_B31(i1+1,i2  ,i3  ) - flux_B31(i1  ,i2  ,i3  )) - &
                       dtoverdx2 * (flux_B32(i1  ,i2+1,i3  ) - flux_B32(i1  ,i2  ,i3  ))
      !
      ! --- Update of internal energy ---
      model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + ( &
              0.5 * (model%Bc1(i1,i2,i3)**2+ &
                     model%Bc2(i1,i2,i3)**2+model%Bc3(i1,i2,i3)**2) - &
              0.5 * (B1_out**2+B2_out**2+B3_out**2) - &
                     dtoverdx1 * (flux_rhoeikm1(i1+1,i2  ,i3  ) - flux_rhoeikm1(i1  ,i2  ,i3  )) - &
                     dtoverdx2 * (flux_rhoeikm2(i1  ,i2+1,i3  ) - flux_rhoeikm2(i1  ,i2  ,i3  )) - &
                     dtoverdx3 * (flux_rhoeikm3(i1  ,i2  ,i3+1) - flux_rhoeikm3(i1  ,i2  ,i3  )) &
                                              ) / model%rho(i1,i2,i3)
      !
      model%Bc1(i1,i2,i3) = B1_out
      model%Bc2(i1,i2,i3) = B2_out
      model%Bc3(i1,i2,i3) = B3_out
      !
    end do ! i3
  end do ! i2
end do ! i1
!
! \label{f90:c_rescourant__useMHD2}
! --- Check for time step limit violation ---
inv_dt=inv_dt1+inv_dt2+inv_dt3
if (inv_dt>0.0) then
  dtime_limit=par%C_resCourant/inv_dt
  if (dtime>dtime_limit*par%C_resCourantMax/par%C_resCourant) then
    ierr0=1
    outstr0='Time step exceeds resistive time scale'
    goto 999
  endif
endif
!
! --- Check if internal energy is below minimum table value anywhere ---
call tabinter_getinfo(x2min=eiTable_min)
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
!
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*min(dtime, dtime_limit)
  outstr0='Internal energy below minimum table value'
endif
!
! --- Output of additional data ---
if ((Output_flag) .and. present(dat)) then
  call rhd_mhd_Res3Doutput(model, &
                           AreaFraction10, AreaFraction20, AreaFraction30, &
                           flux_rhoeikm1, flux_rhoeikm2, flux_rhoeikm3, dat)
endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_mhd_Res3DStep


!----------*****************------------------------------------------------------------------------
subroutine rhd_mhd_Constrain(model, action, dtime, &
                             output, dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Constrain ('rhd_magneto-hydrodynamics_Constrained_transport')
!
! PURPOSE:
!   Perform constrained transport step for magnetic fields.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Constrain(model, action, dtime, &
!                          output=output, dat=dat)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data after predictor mhd step (from HLL solver)
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   xb1:          ('x_boundary_1') real 1D array, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real 1D array, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real 1D array, 3. coordinate of cell boundaries
!   E1_edge:      ('Electric_1_edge') real 2D array, edge located electric field, 1. component
!   E2_edge:      ('Electric_2_edge') real 2D array, edge located electric field, 2. component
!   E3_edge:      ('Electric_3_edge') real 2D array, edge located electric field, 3. component
!
! ROUTINES:
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Calculates edge located electric fields and updates the face centered magnetic field
!
! EXAMPLE:
!
! TODO:
!
! MODIFICATION HISTORY:
!   2002-11-05 (B.F.) First version
!   2002-11-06 (B.F.) Use upwind magnetic field
!   2002-11-12 (B.F.) 3D
!   2003-12-15\
!              (W.S.) Temporal averaging of magnetic fields instead of upwind magnetic fields
!              (W.S.) Special treatment for 1D included
!              (W.S.) Treatment of boundary conditions by extrapolating magnetic fields and
!                     velocities
!              (W.S.) Energy correction step included
!              (W.S.) Check for minimum internal energy included
!              (W.S.) Parameters dtime_limit, ierr, outstr added
!   2004-03-23/
!   2004-03-26 (W.S.) Energy correction modified
!   2004-03-31 (W.S.) Energy correction only for hdScheme==hdScheme_RoeMHD
!   2004-07-20 (W.S.) Artificial resistivity and additional energy diffusion added
!   2004-07-23 (W.S.) Time step restriction due to resistivity and energy diffusion
!   2004-10-26 (W.S.) Treatment of boundary conditions put into new subroutine
!   2004-10-28 (W.S.) 3D version
!   2006-02-23 (W.S.) Electric field now 2D array (less memory consuming), magnetic diffusivity
!                   temporarily removed
!   2006-09-08 (W.S.) Magnetic diffusivity and energy diffusion reintroduced
!   2008-03-09 (W.S.) OpenMP directives
!   2010-04-10 (W.S. Freiburg) Resistivity fixed
!   2010-06-08 (B.F. Lyon) Bugfixes: some indices in resistivity calculation
!   2010-12-15 (W.S. Exeter) Resistivity part removed, parameters model0, dtime_limit, outstr, ierr
!                     removed, boundary handling modified,
!                     rigorous splitting of routine into 1D, 2D and 3D case
!   2010-12-17 (W.S.) Output of some averages to meanfile: new optional parameters output,
!                     outradialc, AreaFraction1, AreaFraction2, AreaFraction3, dat
!   2011-01-05 (W.S.) Quadratic interpolation of fluxes for the computation of the electric field
!   2012-11-28 (W.S.) Bugfix: Error in computation of averages for meanfile fixed
!   2013-03-05 (B.F.) Parameter outradialc removed, more timing markers
!   2013-03-06 (B.F.) Make E1_edge a 3D array again and reorder loops
!   2015-04-09 (B.F.) Bugfix in 2D case: constant boundary-cell extrapolation
!   2017-08-19 (B.F.) Account for new datx type in averaging of quantities
!   2017-08-23 (B.F.) Compute AreaFractions from local and global quantities;
!                     Bugfix (for old averaging scheme): actually process AreaFraction20
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!   2018-10-25 (B.F.) Remove optional input parameters AreaFraction1, AreaFraction2, AreaFraction3
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use tabinter_module
use rhd_dat_module
use rhd_mean_module
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
logical,         optional, intent(in)   :: output
type(dat_type),  optional, intent(inout):: dat
!
! --- Local variables ---
logical                                 :: Output_flag
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, i, n, di1, di2, di3
real                                    :: ei_new, einew_min, eiTable_min, &
                                           AreaFraction12, AreaFraction13, AreaFraction23
real, dimension(model%m1-2:model%n1+3)  :: xb1
real, dimension(model%m2-2:model%n2+3)  :: xb2
real, dimension(model%m3-2:model%n3+3)  :: xb3
real, dimension(model%m1-2:model%n1+2)  :: xc1
real, dimension(model%m2-2:model%n2+2)  :: xc2
real, dimension(model%m3-2:model%n3+2)  :: xc3
real, dimension(model%m1:model%n1+1, &
                               -2:1)    :: c1
real, dimension(model%m2:model%n2+1, &
                               -2:1)    :: c2
real, dimension(model%m3:model%n3+1, &
                               -2:1)    :: c3
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2+1 )   :: E3_edge
real, dimension(model%m1:model%n1+1, &
                model%m3:model%n3+1 )   :: E2_edge
real, dimension(model%m1:model%n1  , &
                model%m2:model%n2+1, &
                model%m3:model%n3+1 )   :: E1_edge
real   , dimension(model%m1:model%n1, &
                   model%m2:model%n2 )  :: weight12
real   , dimension(model%m1:model%n1, &
                   model%m3:model%n3 )  :: weight13
real   , dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: weight23
!$ integer                              :: omp_get_thread_num
!---------------------------------------------------------------------------------------------------
!
call timing_start('MHD: constrain')
!
! --- Set default values for undefined keywords ---
if (.not. present(output)) then
  Output_flag=.false.
else
  Output_flag=output
endif
!
n=0
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
xb1(m1:n1+1)=model%xb1(m1:n1+1)
xb2(m2:n2+1)=model%xb2(m2:n2+1)
xb3(m3:n3+1)=model%xb3(m3:n3+1)
!
call tabinter_getinfo(x2min=eiTable_min)
!
if ((m2 == n2) .and. (m3 == n3)) then
  ! === 1D ========================================================================================
  !
  ! --- No constrained transport step needed                                                  ---
  ! --- Copy components 2 and 3 of cell centered magnetic fields to boundary located magnetic ---
  ! --- fields                                                                                ---
  !
  model%Bb2(:,m2  ,:)=model%Bc2(:,m2,:)
  model%Bb2(:,m2+1,:)=model%Bc2(:,m2,:)
  model%Bb3(:,:,m3  )=model%Bc3(:,:,m3)
  model%Bb3(:,:,m3+1)=model%Bc3(:,:,m3)
  ! --- 1 component of magnetic field remains unchanged in 1D ---
  !
else if (m2 == n2) then
  ! === 2D ========================================================================================
  !
  i2=m2
  !
  !
  ! --- Boundary conditions ---
  !
  ! --- Axes ---
  ! --- 'Left' boundary ---
  if (action%bound_mag_l1 == bound_mag_periodic) then
    ! --- periodic ---
    xb1(m1-1) = xb1(n1  ) + xb1(m1  ) - xb1(n1+1)
    xb1(m1-2) = xb1(n1-1) + xb1(m1  ) - xb1(n1+1)
  else
    ! --- constant ---
    xb1(m1-1) = xb1(m1  ) + xb1(m1  ) - xb1(m1+1)
    xb1(m1-2) = xb1(m1  ) + xb1(m1  ) - xb1(m1+2)
  endif
  if (action%bound_mag_l3 == bound_mag_periodic) then
    ! --- periodic ---
    xb3(m3-1) = xb3(n3  ) + xb3(m3  ) - xb3(n3+1)
    xb3(m3-2) = xb3(n3-1) + xb3(m3  ) - xb3(n3+1)
  else
    ! --- constant ---
    xb3(m3-1) = xb3(m3  ) + xb3(m3  ) - xb3(m3+1)
    xb3(m3-2) = xb3(m3  ) + xb3(m3  ) - xb3(m3+2)
  endif
  !
  ! --- 'Right' boundary ---
  if (action%bound_mag_r1 == bound_mag_periodic) then
    ! --- periodic ---
    xb1(n1+2) = xb1(m1+1) + xb1(n1+1) - xb1(m1  )
    xb1(n1+3) = xb1(m1+2) + xb1(n1+1) - xb1(m1  )
  else
    ! --- constant ---
    xb1(n1+2) = xb1(n1+1) + xb1(n1+1) - xb1(n1  )
    xb1(n1+3) = xb1(n1+1) + xb1(n1+1) - xb1(n1-1)
  endif
  if (action%bound_mag_r3 == bound_mag_periodic) then
    ! --- periodic ---
    xb3(n3+2) = xb3(m3+1) + xb3(n3+1) - xb3(m3  )
    xb3(n3+3) = xb3(m3+2) + xb3(n3+1) - xb3(m3  )
  else
    ! --- constant ---
    xb3(n3+2) = xb3(n3+1) + xb3(n3+1) - xb3(n3  )
    xb3(n3+3) = xb3(n3+1) + xb3(n3+1) - xb3(n3-1)
  endif
  !
  xc1(m1-2:n1+2) = 0.5 * (xb1(m1-2:n1+2) + xb1(m1-1:n1+3))
  xc3(m3-2:n3+2) = 0.5 * (xb3(m3-2:n3+2) + xb3(m3-1:n3+3))
  !
  ! --- Coefficients for quadratic interpolation of fluxes ---
  do i1=m1,n1+1
    !
    c1(i1,-2) = 0.25 * (xc1(i1-1)-xb1(i1  )) * (xc1(i1  )-xb1(i1  )) / &
                      ((xc1(i1-1)-xc1(i1-2)) * (xc1(i1  )-xc1(i1-2)))
    c1(i1,-1) = 0.25 * (xb1(i1  )-xc1(i1-2)) * (xc1(i1  )-xb1(i1  )) / &
                      ((xc1(i1-1)-xc1(i1-2)) * (xc1(i1  )-xc1(i1-1))) + &
                0.25 * (xc1(i1  )-xb1(i1  )) * (xc1(i1+1)-xb1(i1  )) / &
                      ((xc1(i1  )-xc1(i1-1)) * (xc1(i1+1)-xc1(i1-1)))
    c1(i1, 0) = 0.25 * (xb1(i1  )-xc1(i1-1)) * (xb1(i1  )-xc1(i1-2)) / &
                      ((xc1(i1  )-xc1(i1-1)) * (xc1(i1  )-xc1(i1-2))) + &
                0.25 * (xb1(i1  )-xc1(i1-1)) * (xc1(i1+1)-xb1(i1  )) / &
                      ((xc1(i1  )-xc1(i1-1)) * (xc1(i1+1)-xc1(i1  )))
    c1(i1, 1) = 0.25 * (xb1(i1  )-xc1(i1  )) * (xb1(i1  )-xc1(i1-1)) / &
                      ((xc1(i1+1)-xc1(i1  )) * (xc1(i1+1)-xc1(i1-1)))
    !
  end do
  !
  do i3=m3,n3+1
    !
    c3(i3,-2) = 0.25 * (xc3(i3-1)-xb3(i3  )) * (xc3(i3  )-xb3(i3  )) / &
                      ((xc3(i3-1)-xc3(i3-2)) * (xc3(i3  )-xc3(i3-2)))
    c3(i3,-1) = 0.25 * (xb3(i3  )-xc3(i3-2)) * (xc3(i3  )-xb3(i3  )) / &
                      ((xc3(i3-1)-xc3(i3-2)) * (xc3(i3  )-xc3(i3-1))) + &
                0.25 * (xc3(i3  )-xb3(i3  )) * (xc3(i3+1)-xb3(i3  )) / &
                      ((xc3(i3  )-xc3(i3-1)) * (xc3(i3+1)-xc3(i3-1)))
    c3(i3, 0) = 0.25 * (xb3(i3  )-xc3(i3-1)) * (xb3(i3  )-xc3(i3-2)) / &
                      ((xc3(i3  )-xc3(i3-1)) * (xc3(i3  )-xc3(i3-2))) + &
                0.25 * (xb3(i3  )-xc3(i3-1)) * (xc3(i3+1)-xb3(i3  )) / &
                      ((xc3(i3  )-xc3(i3-1)) * (xc3(i3+1)-xc3(i3  )))
    c3(i3, 1) = 0.25 * (xb3(i3  )-xc3(i3  )) * (xb3(i3  )-xc3(i3-1)) / &
                      ((xc3(i3+1)-xc3(i3  )) * (xc3(i3+1)-xc3(i3-1)))
    !
  end do
  !
  ! --- Fluxes ---
  !
  if (action%bound_mag_l1 == bound_mag_periodic) then
    ! --- periodic --
    do i3=m3,n3+1
      !
      model%flux_B13(m1-1,i2,i3) = model%flux_B13(n1,i2,i3)
      model%flux_B13(m1-2,i2,i3) = model%flux_B13(n1-1,i2,i3)
      !
    end do ! i3
  else
    ! --- constant --
    do i3=m3,n3+1
      !
      model%flux_B13(m1-1,i2,i3) = model%flux_B13(m1,i2,i3)
      model%flux_B13(m1-2,i2,i3) = model%flux_B13(m1+1,i2,i3)
      !
    end do ! i3
  endif
  !
  if (action%bound_mag_r1 == bound_mag_periodic) then
    ! --- periodic --
    do i3=m3,n3+1
      !
      model%flux_B13(n1+1,i2,i3) = model%flux_B13(m1,i2,i3)
      model%flux_B13(n1+2,i2,i3) = model%flux_B13(m1+1,i2,i3)
      !
    end do ! i3
  else
    ! --- constant --
    do i3=m3,n3+1
      !
      model%flux_B13(n1+1,i2,i3) = model%flux_B13(n1,i2,i3)
      model%flux_B13(n1+2,i2,i3) = model%flux_B13(n1-1,i2,i3)
      !
    end do ! i3
  endif
  !
  if (action%bound_mag_l3 == bound_mag_periodic) then
    ! --- periodic ---
    do i1=m1,n1+1
      !
      model%flux_B31(i1,i2,m3-1) = model%flux_B31(i1,i2,n3)
      model%flux_B31(i1,i2,m3-2) = model%flux_B31(i1,i2,n3-1)
      !
    end do ! i1
  else
    ! --- constant ---
    do i1=m1,n1+1
      !
      model%flux_B31(i1,i2,m3-1) = model%flux_B31(i1,i2,m3)
      model%flux_B31(i1,i2,m3-2) = model%flux_B31(i1,i2,m3+1)
      !
    end do ! i1
  endif
  !
  if (action%bound_mag_r3 == bound_mag_periodic) then
    ! --- periodic ---
    do i1=m1,n1+1
      !
      model%flux_B31(i1,i2,n3+1) = model%flux_B31(i1,i2,m3)
      model%flux_B31(i1,i2,n3+2) = model%flux_B31(i1,i2,m3+1)
      !
    end do ! i1
  else
    ! --- constant ---
    do i1=m1,n1+1
      !
      model%flux_B31(i1,i2,n3+1) = model%flux_B31(i1,i2,n3)
      model%flux_B31(i1,i2,n3+2) = model%flux_B31(i1,i2,n3-1)
      !
    end do ! i1
  endif
  !
  ! --- Compute electric fields at cell edges ---
  !
  !$OMP PARALLEL PRIVATE(i1, i)
  !
  ! --- Electric field in x2 direction ---
  !
  !$OMP DO
  do i3=m3,n3+1
    do i1=m1,n1+1
      !
      ! \label{f90:n_orderconstrainedtransport__useMHD1}
      if (par%N_orderConstrainedTransport == 2) then
        !
        ! --- quadratic interpolation of fluxes ---
        E2_edge(i1,i3)=0.0
        !
        do i=-2,1
          !
          E2_edge(i1,i3)=E2_edge(i1,i3) - c1(i1,i)*model%flux_B13(i1+i,i2,i3) + &
                                          c3(i3,i)*model%flux_B31(i1,i2,i3+i)
        end do
        !
      else
        !
        ! --- simple averaging ---
        E2_edge(i1,i3)= -0.25 * (model%flux_B13(i1,i2,i3) + model%flux_B13(i1-1,i2,i3  )) + &
                         0.25 * (model%flux_B31(i1,i2,i3) + model%flux_B31(i1  ,i2,i3-1))
      endif
      !
    end do ! i1
  end do ! i3
  !
  ! --- Fixed boundary conditions ---
  !$OMP SINGLE
  if (action%bound_mag_l1 == bound_mag_fixed) then
    E2_edge(m1  ,:   )=0.0
  endif
  if (action%bound_mag_r1 == bound_mag_fixed) then
    E2_edge(n1+1,:   )=0.0
  endif
  if (action%bound_mag_l3 == bound_mag_fixed) then
    E2_edge(:   ,m3  )=0.0
  endif
  if (action%bound_mag_r3 == bound_mag_fixed) then
    E2_edge(:   ,n3+1)=0.0
  endif
  !$OMP END SINGLE
  !
  ! --- Update Bb1 ---
  !
  !$OMP DO
  do i3=m3,n3
    do i1=m1,n1+1

      model%Bb1(i1,i2,i3)=model%Bb1(i1,i2,i3) + &
                            dtime/(model%xb3(     i3+1) - model%xb3(     i3  )) * &
                                  (  E2_edge(i1  ,i3+1) -   E2_edge(i1  ,i3  ))
    end do ! i1
  end do ! i3
  !$OMP END DO NOWAIT
  !
  ! --- Update Bb3 ---
  !
  !$OMP DO
  do i3=m3,n3+1
    do i1=m1,n1

      model%Bb3(i1,i2,i3)=model%Bb3(i1,i2,i3) - &
                            dtime/(model%xb1(i1+1     ) - model%xb1(i1       )) * &
                                  (  E2_edge(i1+1,i3  ) -   E2_edge(i1  ,i3  ))
    end do ! i1
  end do ! i3
  !
  !$OMP END PARALLEL
  !
  ! --- 2 component of magnetic field not included in constrained transport step ---
  ! --- Copy 2 component of cell centered magnetic field to boundary located magnetic field ---
  model%Bb2(:,m2  ,:)=model%Bc2(:,m2,:)
  model%Bb2(:,m2+1,:)=model%Bc2(:,m2,:)
  !
  if (par%A%hdScheme == hdScheme_RoeMHD .or. par%A%hdScheme == hdScheme_HLLMHD) then
    !
    ! --- Energy correction ---
    !
    ! --- Add old magnetic energy to internal energy        ---
    ! --- Subtract new magnetic energy from internal energy ---
    !
    n = 0
    !$OMP PARALLEL
    !$OMP DO PRIVATE(i1, ei_new) REDUCTION(+: n)
    do i3=m3, n3
      do i1 = m1, n1
        !
        if (model%energy_switch(i1,i2,i3)) then
          !
          ei_new=model%ei(i1,i2,i3) + 0.5 * &
                  ((model%Bc1(i1,i2,i3)+0.5*(model%Bb1(i1,i2,i3)+model%Bb1(i1+1,i2,i3))) * &
                   (model%Bc1(i1,i2,i3)-0.5*(model%Bb1(i1,i2,i3)+model%Bb1(i1+1,i2,i3))) + &
                   (model%Bc3(i1,i2,i3)+0.5*(model%Bb3(i1,i2,i3)+model%Bb3(i1,i2,i3+1))) * &
                   (model%Bc3(i1,i2,i3)-0.5*(model%Bb3(i1,i2,i3)+model%Bb3(i1,i2,i3+1))) &
                             ) / model%rho(i1,i2,i3)
          !
          if (ei_new > eiTable_min) then
            model%ei(i1,i2,i3)=ei_new
          endif
          !
          n=n+1
        endif
        !
      end do ! i1
    end do ! i3
    !$OMP END PARALLEL
    !
    write(gl%nc_p, '(A,I8)') "Cells updated with total energy equation: ", n
  endif
  !
else
  ! === 3D ========================================================================================
  !
!#
# ifdef rhd_bound_t01
!#
  call timing_start('MHD: constrain: boundaries')
!#
# endif
!#
  !
  ! --- Boundary conditions ---
  !
  ! --- Axes ---
  ! --- 'Left' boundary ---
  if (action%bound_mag_l1 == bound_mag_periodic) then
    ! --- periodic ---
    xb1(m1-1) = xb1(n1  ) + xb1(m1  ) - xb1(n1+1)
    xb1(m1-2) = xb1(n1-1) + xb1(m1  ) - xb1(n1+1)
  else
    ! --- constant ---
    xb1(m1-1) = xb1(m1  ) + xb1(m1  ) - xb1(m1+1)
    xb1(m1-2) = xb1(m1  ) + xb1(m1  ) - xb1(m1+2)
  endif
  if (action%bound_mag_l2 == bound_mag_periodic) then
    ! --- periodic ---
    xb2(m2-1) = xb2(n2  ) + xb2(m2  ) - xb2(n2+1)
    xb2(m2-2) = xb2(n2-1) + xb2(m2  ) - xb2(n2+1)
  else
    ! --- constant ---
    xb2(m2-1) = xb2(m2  ) + xb2(m2  ) - xb2(m2+1)
    xb2(m2-2) = xb2(m2  ) + xb2(m2  ) - xb2(m2+2)
  endif
  if (action%bound_mag_l3 == bound_mag_periodic) then
    ! --- periodic ---
    xb3(m3-1) = xb3(n3  ) + xb3(m3  ) - xb3(n3+1)
    xb3(m3-2) = xb3(n3-1) + xb3(m3  ) - xb3(n3+1)
  else
    ! --- constant ---
    xb3(m3-1) = xb3(m3  ) + xb3(m3  ) - xb3(m3+1)
    xb3(m3-2) = xb3(m3  ) + xb3(m3  ) - xb3(m3+2)
  endif
  !
  ! --- 'Right' boundary ---
  if (action%bound_mag_r1 == bound_mag_periodic) then
    ! --- periodic ---
    xb1(n1+2) = xb1(m1+1) + xb1(n1+1) - xb1(m1  )
    xb1(n1+3) = xb1(m1+2) + xb1(n1+1) - xb1(m1  )
  else
    ! --- constant ---
    xb1(n1+2) = xb1(n1+1) + xb1(n1+1) - xb1(n1  )
    xb1(n1+3) = xb1(n1+1) + xb1(n1+1) - xb1(n1-1)
  endif
  if (action%bound_mag_r2 == bound_mag_periodic) then
    ! --- periodic ---
    xb2(n2+2) = xb2(m2+1) + xb2(n2+1) - xb2(m2  )
    xb2(n2+3) = xb2(m2+2) + xb2(n2+1) - xb2(m2  )
  else
    ! --- constant ---
    xb2(n2+2) = xb2(n2+1) + xb2(n2+1) - xb2(n2  )
    xb2(n2+3) = xb2(n2+1) + xb2(n2+1) - xb2(n2-1)
  endif
  if (action%bound_mag_r3 == bound_mag_periodic) then
    ! --- periodic ---
    xb3(n3+2) = xb3(m3+1) + xb3(n3+1) - xb3(m3  )
    xb3(n3+3) = xb3(m3+2) + xb3(n3+1) - xb3(m3  )
  else
    ! --- constant ---
    xb3(n3+2) = xb3(n3+1) + xb3(n3+1) - xb3(n3  )
    xb3(n3+3) = xb3(n3+1) + xb3(n3+1) - xb3(n3-1)
  endif
  !
  xc1(m1-2:n1+2) = 0.5 * (xb1(m1-2:n1+2) + xb1(m1-1:n1+3))
  xc2(m1-2:n1+2) = 0.5 * (xb2(m1-2:n2+2) + xb2(m2-1:n2+3))
  xc3(m3-2:n3+2) = 0.5 * (xb3(m3-2:n3+2) + xb3(m3-1:n3+3))
  !
  ! --- Coefficients for quadratic interpolation of fluxes ---
  do i1=m1,n1+1
    !
    c1(i1,-2) = 0.25 * (xc1(i1-1)-xb1(i1  )) * (xc1(i1  )-xb1(i1  )) / &
                      ((xc1(i1-1)-xc1(i1-2)) * (xc1(i1  )-xc1(i1-2)))
    c1(i1,-1) = 0.25 * (xb1(i1  )-xc1(i1-2)) * (xc1(i1  )-xb1(i1  )) / &
                      ((xc1(i1-1)-xc1(i1-2)) * (xc1(i1  )-xc1(i1-1))) + &
                0.25 * (xc1(i1  )-xb1(i1  )) * (xc1(i1+1)-xb1(i1  )) / &
                      ((xc1(i1  )-xc1(i1-1)) * (xc1(i1+1)-xc1(i1-1)))
    c1(i1, 0) = 0.25 * (xb1(i1  )-xc1(i1-1)) * (xb1(i1  )-xc1(i1-2)) / &
                      ((xc1(i1  )-xc1(i1-1)) * (xc1(i1  )-xc1(i1-2))) + &
                0.25 * (xb1(i1  )-xc1(i1-1)) * (xc1(i1+1)-xb1(i1  )) / &
                      ((xc1(i1  )-xc1(i1-1)) * (xc1(i1+1)-xc1(i1  )))
    c1(i1, 1) = 0.25 * (xb1(i1  )-xc1(i1  )) * (xb1(i1  )-xc1(i1-1)) / &
                      ((xc1(i1+1)-xc1(i1  )) * (xc1(i1+1)-xc1(i1-1)))
    !
  end do
  !
  do i2=m2,n2+1
    !
    c2(i2,-2) = 0.25 * (xc2(i2-1)-xb2(i2  )) * (xc2(i2  )-xb2(i2  )) / &
                      ((xc2(i2-1)-xc2(i2-2)) * (xc2(i2  )-xc2(i2-2)))
    c2(i2,-1) = 0.25 * (xb2(i2  )-xc2(i2-2)) * (xc2(i2  )-xb2(i2  )) / &
                      ((xc2(i2-1)-xc2(i2-2)) * (xc2(i2  )-xc2(i2-1))) + &
                0.25 * (xc2(i2  )-xb2(i2  )) * (xc2(i2+1)-xb2(i2  )) / &
                      ((xc2(i2  )-xc2(i2-1)) * (xc2(i2+1)-xc2(i2-1)))
    c2(i2, 0) = 0.25 * (xb2(i2  )-xc2(i2-1)) * (xb2(i2  )-xc2(i2-2)) / &
                      ((xc2(i2  )-xc2(i2-1)) * (xc2(i2  )-xc2(i2-2))) + &
                0.25 * (xb2(i2  )-xc2(i2-1)) * (xc2(i2+1)-xb2(i2  )) / &
                      ((xc2(i2  )-xc2(i2-1)) * (xc2(i2+1)-xc2(i2  )))
    c2(i2, 1) = 0.25 * (xb2(i2  )-xc2(i2  )) * (xb2(i2  )-xc2(i2-1)) / &
                      ((xc2(i2+1)-xc2(i2  )) * (xc2(i2+1)-xc2(i2-1)))
    !
  end do
  !
  do i3=m3,n3+1
    !
    c3(i3,-2) = 0.25 * (xc3(i3-1)-xb3(i3  )) * (xc3(i3  )-xb3(i3  )) / &
                      ((xc3(i3-1)-xc3(i3-2)) * (xc3(i3  )-xc3(i3-2)))
    c3(i3,-1) = 0.25 * (xb3(i3  )-xc3(i3-2)) * (xc3(i3  )-xb3(i3  )) / &
                      ((xc3(i3-1)-xc3(i3-2)) * (xc3(i3  )-xc3(i3-1))) + &
                0.25 * (xc3(i3  )-xb3(i3  )) * (xc3(i3+1)-xb3(i3  )) / &
                      ((xc3(i3  )-xc3(i3-1)) * (xc3(i3+1)-xc3(i3-1)))
    c3(i3, 0) = 0.25 * (xb3(i3  )-xc3(i3-1)) * (xb3(i3  )-xc3(i3-2)) / &
                      ((xc3(i3  )-xc3(i3-1)) * (xc3(i3  )-xc3(i3-2))) + &
                0.25 * (xb3(i3  )-xc3(i3-1)) * (xc3(i3+1)-xb3(i3  )) / &
                      ((xc3(i3  )-xc3(i3-1)) * (xc3(i3+1)-xc3(i3  )))
    c3(i3, 1) = 0.25 * (xb3(i3  )-xc3(i3  )) * (xb3(i3  )-xc3(i3-1)) / &
                      ((xc3(i3+1)-xc3(i3  )) * (xc3(i3+1)-xc3(i3-1)))
    !
  end do
  !
  ! --- Fluxes ---
  !
  !$OMP PARALLEL PRIVATE(i2, i3)
  if (action%bound_mag_l1 == bound_mag_periodic) then
    !
    ! --- periodic --
    !$OMP DO
    do i3=m3,n3
      do i2=m2,n2+1
        !
        model%flux_B12(m1-1,i2,i3) = model%flux_B12(n1  ,i2,i3)
        model%flux_B12(m1-2,i2,i3) = model%flux_B12(n1-1,i2,i3)
        model%flux_B32(m1-1,i2,i3) = model%flux_B32(n1  ,i2,i3)
        model%flux_B32(m1-2,i2,i3) = model%flux_B32(n1-1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i2=m2,n2
        !
        model%flux_B13(m1-1,i2,i3) = model%flux_B13(n1  ,i2,i3)
        model%flux_B13(m1-2,i2,i3) = model%flux_B13(n1-1,i2,i3)
        model%flux_B23(m1-1,i2,i3) = model%flux_B23(n1  ,i2,i3)
        model%flux_B23(m1-2,i2,i3) = model%flux_B23(n1-1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  else
    !
    ! --- constant --
    !$OMP DO
    do i3=m3,n3
      do i2=m2,n2+1
        !
        model%flux_B12(m1-1,i2,i3) = model%flux_B12(m1  ,i2,i3)
        model%flux_B12(m1-2,i2,i3) = model%flux_B12(m1+1,i2,i3)
        model%flux_B32(m1-1,i2,i3) = model%flux_B32(m1  ,i2,i3)
        model%flux_B32(m1-2,i2,i3) = model%flux_B32(m1+1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i2=m2,n2
        !
        model%flux_B13(m1-1,i2,i3) = model%flux_B13(m1  ,i2,i3)
        model%flux_B13(m1-2,i2,i3) = model%flux_B13(m1+1,i2,i3)
        model%flux_B23(m1-1,i2,i3) = model%flux_B23(m1  ,i2,i3)
        model%flux_B23(m1-2,i2,i3) = model%flux_B23(m1+1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  endif
  !
  if (action%bound_mag_r1 == bound_mag_periodic) then
    !
    ! --- periodic --
    !$OMP DO
    do i3=m3,n3
      do i2=m2,n2+1
        !
        model%flux_B12(n1+1,i2,i3) = model%flux_B12(m1  ,i2,i3)
        model%flux_B12(n1+2,i2,i3) = model%flux_B12(m1+1,i2,i3)
        model%flux_B32(n1+1,i2,i3) = model%flux_B32(m1  ,i2,i3)
        model%flux_B32(n1+2,i2,i3) = model%flux_B32(m1+1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i2=m2,n2
        !
        model%flux_B13(n1+1,i2,i3) = model%flux_B13(m1  ,i2,i3)
        model%flux_B13(n1+2,i2,i3) = model%flux_B13(m1+1,i2,i3)
        model%flux_B23(n1+1,i2,i3) = model%flux_B23(m1  ,i2,i3)
        model%flux_B23(n1+2,i2,i3) = model%flux_B23(m1+1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  else
    !
    ! --- constant --
    !$OMP DO
    do i3=m3,n3
      do i2=m2,n2+1
        !
        model%flux_B12(n1+1,i2,i3) = model%flux_B12(n1  ,i2,i3)
        model%flux_B12(n1+2,i2,i3) = model%flux_B12(n1-1,i2,i3)
        model%flux_B32(n1+1,i2,i3) = model%flux_B32(n1  ,i2,i3)
        model%flux_B32(n1+2,i2,i3) = model%flux_B32(n1-1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i2=m2,n2
        !
        model%flux_B13(n1+1,i2,i3) = model%flux_B13(n1  ,i2,i3)
        model%flux_B13(n1+2,i2,i3) = model%flux_B13(n1-1,i2,i3)
        model%flux_B23(n1+1,i2,i3) = model%flux_B23(n1  ,i2,i3)
        model%flux_B23(n1+2,i2,i3) = model%flux_B23(n1-1,i2,i3)
        !
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  endif
  !$OMP END PARALLEL
  !
  !$OMP PARALLEL PRIVATE(i1, i3)
  if (action%bound_mag_l2 == bound_mag_periodic) then
    !
    ! --- periodic ---
    !$OMP DO
    do i3=m3,n3
      do i1=m1,n1+1
        !
        model%flux_B21(i1,m2-1,i3) = model%flux_B21(i1,n2  ,i3)
        model%flux_B21(i1,m2-2,i3) = model%flux_B21(i1,n2-1,i3)
        model%flux_B31(i1,m2-1,i3) = model%flux_B31(i1,n2  ,i3)
        model%flux_B31(i1,m2-2,i3) = model%flux_B31(i1,n2-1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i1=m1,n1
        !
        model%flux_B13(i1,m2-1,i3) = model%flux_B13(i1,n2  ,i3)
        model%flux_B13(i1,m2-2,i3) = model%flux_B13(i1,n2-1,i3)
        model%flux_B23(i1,m2-1,i3) = model%flux_B23(i1,n2  ,i3)
        model%flux_B23(i1,m2-2,i3) = model%flux_B23(i1,n2-1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
  else
    !
    ! --- constant ---
    !$OMP DO
    do i3=m3,n3
      do i1=m1,n1+1
        !
        model%flux_B21(i1,m2-1,i3) = model%flux_B21(i1,m2  ,i3)
        model%flux_B21(i1,m2-2,i3) = model%flux_B21(i1,m2+1,i3)
        model%flux_B31(i1,m2-1,i3) = model%flux_B31(i1,m2  ,i3)
        model%flux_B31(i1,m2-2,i3) = model%flux_B31(i1,m2+1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i1=m1,n1
        !
        model%flux_B13(i1,m2-1,i3) = model%flux_B13(i1,m2  ,i3)
        model%flux_B13(i1,m2-2,i3) = model%flux_B13(i1,m2+1,i3)
        model%flux_B23(i1,m2-1,i3) = model%flux_B23(i1,m2  ,i3)
        model%flux_B23(i1,m2-2,i3) = model%flux_B23(i1,m2+1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
  endif
  !
  if (action%bound_mag_r2 == bound_mag_periodic) then
    !
    ! --- periodic ---
    !$OMP DO
    do i3=m3,n3
      do i1=m1,n1+1
        !
        model%flux_B21(i1,n2+1,i3) = model%flux_B21(i1,m2  ,i3)
        model%flux_B21(i1,n2+2,i3) = model%flux_B21(i1,m2+1,i3)
        model%flux_B31(i1,n2+1,i3) = model%flux_B31(i1,m2  ,i3)
        model%flux_B31(i1,n2+2,i3) = model%flux_B31(i1,m2+1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i1=m1,n1
        !
        model%flux_B13(i1,n2+1,i3) = model%flux_B13(i1,m2  ,i3)
        model%flux_B13(i1,n2+2,i3) = model%flux_B13(i1,m2+1,i3)
        model%flux_B23(i1,n2+1,i3) = model%flux_B23(i1,m2  ,i3)
        model%flux_B23(i1,n2+2,i3) = model%flux_B23(i1,m2+1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
  else
    !
    ! --- constant ---
    !$OMP DO
    do i3=m3,n3
      do i1=m1,n1+1
        !
        model%flux_B21(i1,n2+1,i3) = model%flux_B21(i1,n2  ,i3)
        model%flux_B21(i1,n2+2,i3) = model%flux_B21(i1,n2-1,i3)
        model%flux_B31(i1,n2+1,i3) = model%flux_B31(i1,n2  ,i3)
        model%flux_B31(i1,n2+2,i3) = model%flux_B31(i1,n2-1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i3=m3,n3+1
      do i1=m1,n1
        !
        model%flux_B13(i1,n2+1,i3) = model%flux_B13(i1,n2  ,i3)
        model%flux_B13(i1,n2+2,i3) = model%flux_B13(i1,n2-1,i3)
        model%flux_B23(i1,n2+1,i3) = model%flux_B23(i1,n2  ,i3)
        model%flux_B23(i1,n2+2,i3) = model%flux_B23(i1,n2-1,i3)
        !
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
  endif
  !$OMP END PARALLEL
  !
  !$OMP PARALLEL PRIVATE(i1, i2)
  if (action%bound_mag_l3 == bound_mag_periodic) then
    !
    ! --- periodic ---
    !$OMP DO
    do i2=m2,n2
      do i1=m1,n1+1
        !
        model%flux_B21(i1,i2,m3-1) = model%flux_B21(i1,i2,n3  )
        model%flux_B21(i1,i2,m3-2) = model%flux_B21(i1,i2,n3-1)
        model%flux_B31(i1,i2,m3-1) = model%flux_B31(i1,i2,n3  )
        model%flux_B31(i1,i2,m3-2) = model%flux_B31(i1,i2,n3-1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i2=m2,n2+1
      do i1=m1,n1
        !
        model%flux_B12(i1,i2,m3-1) = model%flux_B12(i1,i2,n3  )
        model%flux_B12(i1,i2,m3-2) = model%flux_B12(i1,i2,n3-1)
        model%flux_B32(i1,i2,m3-1) = model%flux_B32(i1,i2,n3  )
        model%flux_B32(i1,i2,m3-2) = model%flux_B32(i1,i2,n3-1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
  else
    !
    ! --- constant ---
    !$OMP DO
    do i2=m2,n2
      do i1=m1,n1+1
        !
        model%flux_B21(i1,i2,m3-1) = model%flux_B21(i1,i2,m3  )
        model%flux_B21(i1,i2,m3-2) = model%flux_B21(i1,i2,m3+1)
        model%flux_B31(i1,i2,m3-1) = model%flux_B31(i1,i2,m3  )
        model%flux_B31(i1,i2,m3-2) = model%flux_B31(i1,i2,m3+1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i2=m2,n2+1
      do i1=m1,n1
        !
        model%flux_B12(i1,i2,m3-1) = model%flux_B12(i1,i2,m3  )
        model%flux_B12(i1,i2,m3-2) = model%flux_B12(i1,i2,m3+1)
        model%flux_B32(i1,i2,m3-1) = model%flux_B32(i1,i2,m3  )
        model%flux_B32(i1,i2,m3-2) = model%flux_B32(i1,i2,m3+1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
  endif
  !
  if (action%bound_mag_r3 == bound_mag_periodic) then
    !
    ! --- periodic ---
    !$OMP DO
    do i2=m2,n2
      do i1=m1,n1+1
        !
        model%flux_B21(i1,i2,n3+1) = model%flux_B21(i1,i2,m3  )
        model%flux_B21(i1,i2,n3+2) = model%flux_B21(i1,i2,m3+1)
        model%flux_B31(i1,i2,n3+1) = model%flux_B31(i1,i2,m3  )
        model%flux_B31(i1,i2,n3+2) = model%flux_B31(i1,i2,m3+1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i2=m2,n2+1
      do i1=m1,n1
        !
        model%flux_B12(i1,i2,n3+1) = model%flux_B12(i1,i2,m3  )
        model%flux_B12(i1,i2,n3+2) = model%flux_B12(i1,i2,m3+1)
        model%flux_B32(i1,i2,n3+1) = model%flux_B32(i1,i2,m3  )
        model%flux_B32(i1,i2,n3+2) = model%flux_B32(i1,i2,m3+1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
  else
    !
    ! --- constant ---
    !$OMP DO
    do i2=m2,n2
      do i1=m1,n1+1
        !
        model%flux_B21(i1,i2,n3+1) = model%flux_B21(i1,i2,n3  )
        model%flux_B21(i1,i2,n3+2) = model%flux_B21(i1,i2,n3-1)
        model%flux_B31(i1,i2,n3+1) = model%flux_B31(i1,i2,n3  )
        model%flux_B31(i1,i2,n3+2) = model%flux_B31(i1,i2,n3-1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
    !
    !$OMP DO
    do i2=m2,n2+1
      do i1=m1,n1
        !
        model%flux_B12(i1,i2,n3+1) = model%flux_B12(i1,i2,n3  )
        model%flux_B12(i1,i2,n3+2) = model%flux_B12(i1,i2,n3-1)
        model%flux_B32(i1,i2,n3+1) = model%flux_B32(i1,i2,n3  )
        model%flux_B32(i1,i2,n3+2) = model%flux_B32(i1,i2,n3-1)
        !
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
  endif
  !$OMP END PARALLEL
  !
!#
# ifdef rhd_bound_t01
!#
  call timing_stop('MHD: constrain: boundaries')
!#
# endif
!#
  call timing_start('MHD: constrain: E1_edge')
  ! === Compute electric fields at cell edges ===
  !
  ! --- Electric field in x1 direction ---
  !
  ! \label{f90:n_orderconstrainedtransport__useMHD2}
  !$OMP PARALLEL PRIVATE(i1, i2, i3)
  if (par%N_orderConstrainedTransport == 2) then
    ! --- quadratic interpolation ---
    !
    !$OMP DO
    do i3=m3,n3+1
      do i2=m2,n2+1
        do i1=m1,n1
          !
          E1_edge(i1,i2,i3)=0.0
          !
          do i=-2,1
            E1_edge(i1,i2,i3)=E1_edge(i1,i2,i3) + c2(i2,i)*model%flux_B23(i1,i2+i,i3  ) - &
                                                  c3(i3,i)*model%flux_B32(i1,i2  ,i3+i)
          end do ! i
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
  else
    ! --- simple average ---
    !
    !$OMP DO
    do i3=m3,n3+1
      do i2=m2,n2+1
        do i1=m1,n1
          E1_edge(i1,i2,i3)= 0.25 * (model%flux_B23(i1,i2,i3) + model%flux_B23(i1,i2-1,i3  )) - &
                             0.25 * (model%flux_B32(i1,i2,i3) + model%flux_B32(i1,i2  ,i3-1))
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
  endif
  !
  ! --- Fixed boundary conditions ---
  !$OMP SINGLE
  if (action%bound_mag_l2 == bound_mag_fixed) then
    E1_edge(:,m2  ,:   )=0.0
  endif
  if (action%bound_mag_r2 == bound_mag_fixed) then
    E1_edge(:,n2+1,:   )=0.0
  endif
  if (action%bound_mag_l3 == bound_mag_fixed) then
    E1_edge(:,:   ,m3  )=0.0
  endif
  if (action%bound_mag_r3 == bound_mag_fixed) then
    E1_edge(:,:   ,n3+1)=0.0
  endif
  !$OMP END SINGLE
  !
  ! --- Update Bb2 ----------------------------------------------------
  !
  !$OMP DO
  do i3=m3,n3
    do i2=m2,n2+1
      do i1=m1,n1
        model%Bb2(i1,i2,i3)=model%Bb2(i1,i2,i3) &
                            - dtime/(model%xb3(        i3+1) - model%xb3(        i3  )) * &
                                    (  E1_edge(i1,i2  ,i3+1) -   E1_edge(i1,i2  ,i3  ))
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !
  ! --- Update Bb3 ----------------------------------------------------
  !
  !$OMP DO
  do i3=m3,n3+1
    do i2=m2,n2
      do i1=m1,n1
        model%Bb3(i1,i2,i3)=model%Bb3(i1,i2,i3) &
                            + dtime/(model%xb2(   i2+1     ) - model%xb2(   i2       )) * &
                                    (  E1_edge(i1,i2+1,i3  ) -   E1_edge(i1,i2  ,i3  ))
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
  !
  call timing_stop('MHD: constrain: E1_edge')
  call timing_start('MHD: constrain: E2_edge')
  ! --- Electric field in x2 direction ---
  !
  !$OMP PARALLEL PRIVATE(i1, i3, E2_edge)
  !$OMP DO
  do i2=m2,n2
    !
    if (par%N_orderConstrainedTransport == 2) then
      !
      ! --- quadratic interpolation ---
      do i3=m3,n3+1
        do i1=m1,n1+1
          !
          E2_edge(i1,i3)=0.0
          !
          do i=-2,1
            !
            E2_edge(i1,i3)=E2_edge(i1,i3) - c1(i1,i)*model%flux_B13(i1+i,i2,i3) + &
                                            c3(i3,i)*model%flux_B31(i1,i2,i3+i)
          end do
          !
        end do ! i1
      end do ! i3
    else
      !
      ! --- simple average ---
      do i3=m3,n3+1
        do i1=m1,n1+1
          E2_edge(i1,i3)= -0.25 * (model%flux_B13(i1,i2,i3) + model%flux_B13(i1-1,i2,i3  )) + &
                           0.25 * (model%flux_B31(i1,i2,i3) + model%flux_B31(i1  ,i2,i3-1))
        end do ! i1
      end do ! i3
    endif
    !
    ! --- Fixed boundary conditions ---
    if (action%bound_mag_l1 == bound_mag_fixed) then
      E2_edge(m1  ,:   )=0.0
    endif
    if (action%bound_mag_r1 == bound_mag_fixed) then
      E2_edge(n1+1,:   )=0.0
    endif
    if (action%bound_mag_l3 == bound_mag_fixed) then
      E2_edge(:   ,m3  )=0.0
    endif
    if (action%bound_mag_r3 == bound_mag_fixed) then
      E2_edge(:   ,n3+1)=0.0
    endif
    !
    ! --- Update Bb1 ----------------------------------------------------
    !
    do i3=m3,n3
      do i1=m1,n1+1
        model%Bb1(i1,i2,i3)=model%Bb1(i1,i2,i3) &
                            + dtime/(model%xb3(     i3+1) - model%xb3(     i3  )) * &
                                    (  E2_edge(i1  ,i3+1) -   E2_edge(i1  ,i3  ))
      end do ! i1
    end do ! i3
    !
    ! --- Update Bb3 ----------------------------------------------------
    !
    do i3=m3,n3+1
      do i1=m1,n1
        model%Bb3(i1,i2,i3)=model%Bb3(i1,i2,i3) &
                            - dtime/(model%xb1(i1+1     ) - model%xb1(i1       )) * &
                                    (  E2_edge(i1+1,i3  ) -   E2_edge(i1  ,i3  ))
      end do ! i1
    end do ! i3
    !
  end do ! i2
  !$OMP END DO
  !$OMP END PARALLEL
  !
  call timing_stop('MHD: constrain: E2_edge')
  call timing_start('MHD: constrain: E3_edge')
  ! --- Electric field in x3 direction ---
  !
  !$OMP PARALLEL PRIVATE(i1, i2, E3_edge)
  !$OMP DO
  do i3=m3,n3
    !
    if (par%N_orderConstrainedTransport == 2) then
      !
      ! --- quadratic interpolation ---
      do i2=m2,n2+1
        do i1=m1,n1+1
          !
          E3_edge(i1,i2)=0.0
          !
          do i=-2,1
            !
            E3_edge(i1,i2)=E3_edge(i1,i2) + c1(i1,i)*model%flux_B12(i1+i,i2,i3) - &
                                            c2(i2,i)*model%flux_B21(i1,i2+i,i3)
          end do
          !
        end do ! i1
      end do ! i2
    else
      !
      ! --- simple average ---
      do i2=m2,n2+1
        do i1=m1,n1+1
          E3_edge(i1,i2)= 0.25 * (model%flux_B12(i1,i2,i3) + model%flux_B12(i1-1,i2  ,i3)) - &
                          0.25 * (model%flux_B21(i1,i2,i3) + model%flux_B21(i1  ,i2-1,i3))
        end do ! i1
      end do ! i2
    endif
    !
    ! --- Fixed boundary condition ---
    if (action%bound_mag_l1 == bound_mag_fixed) then
      E3_edge(m1  ,:   )=0.0
    endif
    if (action%bound_mag_r1 == bound_mag_fixed) then
      E3_edge(n1+1,:   )=0.0
    endif
    if (action%bound_mag_l2 == bound_mag_fixed) then
      E3_edge(:   ,m2  )=0.0
    endif
    if (action%bound_mag_r2 == bound_mag_fixed) then
      E3_edge(:   ,n2+1)=0.0
    endif
    !
    ! --- Update Bb1 ----------------------------------------------------
    !
    do i2=m2,n2
      do i1=m1,n1+1
        model%Bb1(i1,i2,i3)=model%Bb1(i1,i2,i3) &
                            - dtime/(model%xb2(     i2+1) - model%xb2(     i2  )) * &
                                    (  E3_edge(i1  ,i2+1) -   E3_edge(i1,  i2  ))
      end do ! i1
    end do ! i2
    !
    ! --- Update Bb2 ----------------------------------------------------
    !
    do i2=m2,n2+1
      do i1=m1,n1
        model%Bb2(i1,i2,i3)=model%Bb2(i1,i2,i3) &
                            + dtime/(model%xb1(i1+1     ) - model%xb1(i1       )) * &
                                    (  E3_edge(i1+1,i2  ) -   E3_edge(i1,  i2  ))
      end do ! i1
    end do ! i2
    !
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
  !
  call timing_stop('MHD: constrain: E3_edge')
  if (par%A%hdScheme == hdScheme_RoeMHD .or. par%A%hdScheme == hdScheme_HLLMHD) then
    !
    call timing_start('MHD: constrain: energy')
    ! --- Energy correction ---
    !
    ! --- Add old magnetic energy to internal energy        ---
    ! --- Subtract new magnetic energy from internal energy ---
    !
    n = 0
    !$OMP PARALLEL
    !$OMP DO PRIVATE(i1, i2, ei_new) REDUCTION(+: n)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          !
          if (model%energy_switch(i1,i2,i3)) then
            !
            ei_new=model%ei(i1,i2,i3) + 0.5 * &
                  ((model%Bc1(i1,i2,i3)+0.5*(model%Bb1(i1,i2,i3)+model%Bb1(i1+1,i2,i3))) * &
                   (model%Bc1(i1,i2,i3)-0.5*(model%Bb1(i1,i2,i3)+model%Bb1(i1+1,i2,i3))) + &
                   (model%Bc2(i1,i2,i3)+0.5*(model%Bb2(i1,i2,i3)+model%Bb2(i1,i2+1,i3))) * &
                   (model%Bc2(i1,i2,i3)-0.5*(model%Bb2(i1,i2,i3)+model%Bb2(i1,i2+1,i3))) + &
                   (model%Bc3(i1,i2,i3)+0.5*(model%Bb3(i1,i2,i3)+model%Bb3(i1,i2,i3+1))) * &
                   (model%Bc3(i1,i2,i3)-0.5*(model%Bb3(i1,i2,i3)+model%Bb3(i1,i2,i3+1))) &
                             ) / model%rho(i1,i2,i3)
            !
            if (ei_new > eiTable_min) then
              model%ei(i1,i2,i3)=ei_new
            endif
            !
            n=n+1
          endif
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END PARALLEL
    !
    call timing_stop('MHD: constrain: energy')
    !
    write(gl%nc_p, '(A,I9,A,I9,A)') 'Cells updated (not) with total energy equation: ', n, &
                                    ' (', (n1-m1+1)*(n2-m2+1)*(n3-m3+1) - n, ')'
  endif
  !
endif
!
if ((Output_flag) .and. present(dat)) then
  ! --- Output of additional data ---
  !
  if ((wg%hyd%n(1) > wg%hyd%m(1)) .and. (dat%x23%i_Bb > 0)) then
    AreaFraction23=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                   ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
    call rhd_mean_aweight(m2,n2, m3,n3, model%xb2(m2:n2+1), model%xb3(m3:n3+1), AreaFraction23, &
                          weight23)
    !
    if (n1 < wg%hyd%n(1)) then
      di1=0
    else
      di1=1
    endif
    call rhd_mean_x23mean(    model%Bb1(m1:n1+di1,:,:)   , dat%x23%d(m1:n1+di1,dat%x23%i_Bb   ), &
                          weight23)
    call rhd_mean_x23mean(abs(model%Bb1(m1:n1+di1,:,:))  , dat%x23%d(m1:n1+di1,dat%x23%i_Bbabs), &
                          weight23)
    call rhd_mean_x23mean(    model%Bb1(m1:n1+di1,:,:)**2, dat%x23%d(m1:n1+di1,dat%x23%i_Bb2  ), &
                          weight23)
  endif
  !
  if ((wg%hyd%n(2) > wg%hyd%m(2)) .and. (dat%x13%i_Bb > 0)) then
    AreaFraction13=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                   ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
    call rhd_mean_aweight(m1,n1, m3,n3, model%xb1(m1:n1+1), model%xb3(m3:n3+1), AreaFraction13, &
                          weight13)
    !
    if (n2 < wg%hyd%n(2)) then
      di2=0
    else
      di2=1
    endif
    call rhd_mean_x13mean(    model%Bb2(:,m2:n2+di2,:)   , dat%x13%d(m2:n2+di2,dat%x13%i_Bb   ), &
                          weight13)
    call rhd_mean_x13mean(abs(model%Bb2(:,m2:n2+di2,:))  , dat%x13%d(m2:n2+di2,dat%x13%i_Bbabs), &
                          weight13)
    call rhd_mean_x13mean(    model%Bb2(:,m2:n2+di2,:)**2, dat%x13%d(m2:n2+di2,dat%x13%i_Bb2  ), &
                          weight13)
  endif
  !
  if ((wg%hyd%n(3) > wg%hyd%m(3)) .and. (dat%x12%i_Bb > 0)) then
    AreaFraction12=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                   ((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
    call rhd_mean_aweight(m1,n1, m2,n2, model%xb1(m1:n1+1), model%xb2(m2:n2+1), AreaFraction12, &
                          weight12)
    !
    if (n3 < wg%hyd%n(3)) then
      di3=0
    else
      di3=1
    endif
    call rhd_mean_x12mean(    model%Bb3(:,:,m3:n3+di3)   , dat%x12%d(m3:n3+di3,dat%x12%i_Bb   ), &
                          weight12)
    call rhd_mean_x12mean(abs(model%Bb3(:,:,m3:n3+di3))  , dat%x12%d(m3:n3+di3,dat%x12%i_Bbabs), &
                          weight12)
    call rhd_mean_x12mean(    model%Bb3(:,:,m3:n3+di3)**2, dat%x12%d(m3:n3+di3,dat%x12%i_Bb2  ), &
                          weight12)
  endif
  !
endif ! ((Output_flag) .and. present(dat))
!
call timing_stop('MHD: constrain')
!
end subroutine rhd_mhd_Constrain


!----------******************-----------------------------------------------------------------------
subroutine rhd_mhd_SingleStep(model, action, dtime, dtime_limit, &
                              doBb2Bc_flag, doConstrain_flag, &
                              output, dat, &
                              outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_SingleStep ('rhd_magneto-hydrodynamics_Single_Step')
!
! PURPOSE:
!   Perform single 2D or 3D magneto-hydrodynamics step, employ HLLE scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_SingleStep(model, action, dtime, dtime_limit, &
!                           output=output, &
!                           dat=dat, &
!                           outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') type(dat_type), optional, structure for additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!
! LOCAL VARIABLES:
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                    volume averaging of cell centered quantities is performed
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_bound_3DInOutFlow_mhd: Correct values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DTransmit_mhd: Correct values in cells of all sides for transmitting boundary cond
!   rhd_bound_ExtractBox:  Extract part of box, fill boundary ghost cells
!   rhd_box_AddChange:     Add change of (part of) box into another. Transpose, if requested
!   rhd_box_AllCopyOMP:    Copy entire box, in parallel if box is 3D
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_datx_Add:          Add the fields in the structure for additional data
!   rhd_hyd_CoriolisOMP:   Apply Coriolis rotation (with local OpenMP directives)
!   rhd_mhd_Bb2Bc:         Compute cell-centered magnetic field from boundary values
!   rhd_mhd_Constrain:     Perform constrained-transport step for magnetic fields
!   rhd_mhd_HLL1Dstep:     Perform 1D magnetohydrodynamics step in x1 direction (HLL solver)
!   rhd_mhd_Res2DStep:     Perform 2D resistivity step
!   rhd_mhd_Res3DStep:     Perform 3D resistivity step
!   rhd_mhd_Roe1Dstep:     Perform 1D magnetohydrodynamics step in x1 direction (Roe solver)
!   rhd_wrk_module:        Provide persistent work copies of model data
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify the global variable modelB.
!
! RESTRICTIONS:
!
! KEYWORDS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2010-06-03 (W.S.) First version
!   2010-06-14 (B.F.) Branches for 'unsplit' update
!   2010-06-16 (B.F.) In 'unsplit' case do open boundary steps at beginning
!   2010-07-16 (B.F.) doBb2Bc_flag, doConstrain_flag
!   2011-01-05 (W.S.) Parameter model0 removed
!   2013-03-05 (B.F.) Parameter outradialc becomes local variable
!   2013-08-09 (B.F.) Change call to rhd_dat_make
!   2014-08-17 (B.F.) Bugfix: actually allocate Bc components in 3D resistivity step;
!                     replace local modelout by global modelB
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_wrk_module
use rhd_bound_module
use rhd_phys_module
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,                   intent(in)   :: doBb2Bc_flag, doConstrain_flag
logical,         optional, intent(in)   :: output
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
real                                    :: AreaFraction
real                                    :: dtime_limit_chunk, dtime_limit_thread, dtime_limit_mhd
integer                                 :: i_chunk, n_chunks, m1c,n1c, m2c,n2c, m3c,n3c, i3
integer                                 :: ierr0, ierr_chunk, ierr_thread
logical                                 :: StepDone_flag, OutRadialC_flag
character(len=80)                       :: outstr0, outstr_chunk, outstr_thread
type(box_type)                          :: slice, modelres
type(dat_type)                          :: dat_thread
type(action_type)                       :: action_mhd
!integer                                 :: omp_get_thread_num
!logical                                 :: omp_in_parallel
!---------------------------------------------------------------------------------------------------
!
call timing_start('MHD: single step')
!
dtime_limit=huge(dtime_limit)
!
if (doBb2Bc_flag) then
  ! --- Calculate cell-centered magnetic field ---
  call rhd_mhd_Bb2Bc(model)
endif
!
! \label{f90:nu_rotation__useMHD2}
! === Coriolis terms ===============================================================================
!
if (par%nu_rotation /= 0.0) then
  call rhd_hyd_CoriolisOMP(model, dtime)
endif
!
! --- No MHD step performed yet ---
StepDone_flag=.false.
OutRadialC_flag=.true.
!
if (par%hdsplit == 'unsplit') then
  ! \label{f90:c_rhochangetop__useMHD2}
  ! --- Open upper boundary condition ---
  if (((action%bound_r3 == bound_transmitting ) .or. &
       (action%bound_r3 == bound_transmitting2) .or. &
       (action%bound_r3 == bound_transmitting3)       ) .and. &
      ( par%C_rhoChangeTop > 0.0)                              ) then
    call timing_start('MHD: bound_3DTransmit')
    call rhd_bound_3DTransmit_mhd(model, action, dtime)
    call timing_stop('MHD: bound_3DTransmit')
  endif
  !
  ! --- Open lower boundary condition ---
  if ((action%bound_l3 == bound_inoutflow ) .or. &
      (action%bound_l3 == bound_inoutflow2)     ) then
    call timing_start('MHD: bound_3DInOutFlow')
  !  write(6,'(A, 4(1PE14.6))') 'inoutflow: minmax(rho), minmax(ei)', &
  !    minval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    minval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3))
    call rhd_bound_3DInOutFlow_mhd(model, action, dtime)
  !  write(6,'(A, 4(1PE14.6))') 'inoutflow: minmax(rho), minmax(ei)', &
  !    minval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    minval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3))
    call timing_stop('MHD: bound_3DInOutFlow')
  endif
  !
  ! --- Create empty temporary 3D model, if necessary ---
  if (.not. rhd_box_Exist(modelB)) then
    call rhd_box_Make(modelB, &
                      m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3, &
                      quc_flag=.true., Bb_flag=.false., Bc_flag=.true.)
  endif
  ! --- Copy only axes ---
  call rhd_box_Copy(model,  model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                    modelB, model%m1,           model%m2,           model%m3, &
                    grid_mode=1, time_mode=1, bulk_mode=0)
endif
!
if (model%n1 > model%m1) then
  !
  ! --- Magneto-Hydrodynamics substep in x1 direction ---
  !
  action_mhd=rhd_action_New(action, dim=1)
  !
  ! --- Determine number of chunks ---
  call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                         model%m1,model%n1, model%m3,model%n3, model%m2,model%n2, 1, &
                         m3c,n3c, m2c,n2c, &
                         n_chunks=n_chunks)
  write(gl%nc_p,'(A,I10,I6)') &
    'MHD 1: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
  call flush(gl%nc_p)
  !
  ierr0           =0
  outstr0         =''
  !
  call timing_start('MHD: 1')
  !
  ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(dat_thread, slice, &
  !$OMP                  m2c, n2c, m3c, n3c, &
  !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
  !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  !$OMP CRITICAL (HD1_INIT)
  outstr_thread     =''
  ierr_thread       =0
  dtime_limit_thread=huge(dtime_limit_thread)
  !
  if (output) then
    ! --- Thread-local data: set data fields to zero, make radial axis ---
    call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
  endif
  !$OMP END CRITICAL (HD1_INIT)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  CHUNK_LOOP1: do i_chunk=1,n_chunks
    ! --- Current chunk indices ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m3,model%n3, model%m2,model%n2, i_chunk, &
                           m3c,n3c, m2c,n2c)
    !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
    !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
    !
    ! --- Create empty slice and fill it with data ---
    !call timing_start('MHD: make_box(slice 1)')
    call rhd_box_Make(slice, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                             nghost1=par%A%reconstruction_order+1, &
                             quc_flag=.true., Bc_flag=.true.)
    !call timing_stop('MHD: make_box(slice 1)')
    !call timing_start('MHD: copy_box(slice 1)')
    call rhd_box_Copy(model, model%m1, model%n1, m2c     , n2c, m3c     , n3c, &
                      slice, slice%m1,           slice%m2,      slice%m3, &
                      trans_mode=1, flux_mode=0)
    !call timing_stop('MHD: copy_box(slice 1)')
    !
    ! --- Actual hydro step ---
    select case (par%A%hdScheme)
      case (hdScheme_RoeMHD)
        if (output) then
          call rhd_mhd_Roe1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                                (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   datx=dat_thread%x23, datr=dat_thread%r, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_Roe1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case (hdScheme_HLLMHD, hdScheme_CenMHD)
        if (output) then
          call rhd_mhd_HLL1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                                (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   datx=dat_thread%x23, datr=dat_thread%r, datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_HLL1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case default
        dtime_limit_chunk=huge(dtime_limit_chunk)
        ierr_chunk       =0
        outstr_chunk     =''
    end select
    !
    ! --- Copy data back into full model and delete temporary slice ---
    if (par%hdsplit == 'unsplit') then
      if (StepDone_flag) then
        ! --- Add change ---
        !call timing_start('MHD: addchange_box(slice 1)')
        call rhd_box_AddChange(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                               model, &
                               modelB, model%m1,          m2c,                m3c, &
                               grid_mode=0, trans_mode=1)
        !call timing_stop('MHD: addchange_box(slice 1)')
      else
        ! --- Just copy slice ---
        !call timing_start('MHD: copy_box(slice 1)')
        call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelB, model%m1,          m2c,                m3c, &
                          grid_mode=0, trans_mode=1)
        !call timing_stop('MHD: copy_box(slice 1)')
      endif
    else
      !call timing_start('MHD: copy_box(slice 1)')
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        model, model%m1,           m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
      !call timing_stop('MHD: copy_box(slice 1)')
    endif
    !call timing_start('MHD: delete_box(slice 1)')
    call rhd_box_Delete(slice)
    !call timing_stop('MHD: delete_box(slice 1)')
    !
    ! --- Handle error conditions and time step ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
  end do CHUNK_LOOP1 ! i_chunk
  !$OMP END DO NOWAIT
  !
  !$OMP CRITICAL (HD1_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  dtime_limit =min(dtime_limit, dtime_limit_thread)
  !
  if (output) then
    call rhd_datx_Add(dat%x23, dat_thread%x23)
    call rhd_datx_Add(dat%r  , dat_thread%r  )
    call rhd_dat_Delete(dat_thread)
  endif
  !$OMP END CRITICAL (HD1_REDUCTION)
  !
  !$OMP END PARALLEL
  ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  !
  call timing_stop('MHD: 1')
  !
  ! --- Handle error conditions and time step ---
  if (ierr0 > 0) then
    ! --- Error ---
    goto 999
  endif
  !
  OutRadialC_flag=.false.
  !
  ! --- At least one MHD step performed, already ---
  StepDone_flag=.true.
  !
endif
!
if (model%n2 > model%m2) then
  !
  ! --- Magneto-Hydrodynamics substep in x2 direction ---
  !
  action_mhd=rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)))
  !
  ! --- Determine number of chunks ---
  call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                         model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, 1, &
                         m1c,n1c, m3c,n3c, &
                         n_chunks=n_chunks)
  write(gl%nc_p,'(A,I10,I6)') &
    'MHD 2: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
  call flush(gl%nc_p)
  !
  call timing_start('MHD: 2')
  !
  ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(dat_thread, slice, &
  !$OMP                  m1c, n1c, m3c, n3c, &
  !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
  !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  !$OMP CRITICAL (HD2_INIT)
  outstr_thread     =''
  ierr_thread       =0
  dtime_limit_thread=huge(dtime_limit_thread)
  !
  if (output) then
    ! --- Thread-local data: set data fields to zero, make radial axis ---
    call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
  endif
  !$OMP END CRITICAL (HD2_INIT)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  CHUNK_LOOP2: do i_chunk=1,n_chunks
    ! --- Current chunk indices ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, i_chunk, &
                           m1c,n1c, m3c,n3c)
    !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
    !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m3c, n3c
    !
    ! --- Create empty slice and fill it with data ---
    !call timing_start('MHD: make_box(slice 2)')
    call rhd_box_Make(slice, m1=model%m2, n1=model%n2, m2=m1c, n2=n1c, m3=m3c, n3=n3c, &
                             nghost1=par%A%reconstruction_order+1, &
                             quc_flag=.true., Bc_flag=.true.)
    !call timing_stop('MHD: make_box(slice 2)')
    !call timing_start('MHD: copy_box(slice 2)')
    call rhd_box_Copy(model, m1c     , n1c, model%m2, model%n2, m3c     , n3c, &
                      slice, slice%m1,      slice%m2,           slice%m3, &
                      trans_mode=2, flux_mode=0)
    !call timing_stop('MHD: copy_box(slice 2)')
    !
    ! --- Actual hydro step ---
    select case (par%A%hdScheme)
      case (hdScheme_RoeMHD)
        if (output) then
          call rhd_mhd_Roe1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                                (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                                (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   datx=dat_thread%x13, datr=dat_thread%r, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_Roe1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case (hdScheme_HLLMHD, hdScheme_CenMHD)
        if (output) then
          call rhd_mhd_HLL1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                                (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                                (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   datx=dat_thread%x13, datr=dat_thread%r, datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_HLL1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case default
        dtime_limit_chunk=huge(dtime_limit_chunk)
        ierr_chunk       =0
        outstr_chunk     =''
    end select
    !
    ! --- Copy data back into full model and delete temporary slice ---
    if (par%hdsplit == 'unsplit') then
      if (StepDone_flag) then
        ! --- Add change ---
        !call timing_start('MHD: addchange_box(slice 2)')
        call rhd_box_AddChange(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                               model, &
                               modelB, m1c,               model%m2,           m3c, &
                               grid_mode=0, trans_mode=2)
        !call timing_stop('MHD: addchange_box(slice 2)')
      else
        ! --- Just copy slice ---
        !call timing_start('MHD: copy_box(slice 2)')
        call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelB, m1c,               model%m2,           m3c, &
                          grid_mode=0, trans_mode=2)
        !call timing_stop('MHD: copy_box(slice 2)')
      endif
    else
      !call timing_start('MHD: copy_box(slice 2)')
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        model, m1c,                model%m2,           m3c, &
                        grid_mode=0, trans_mode=2)
      !call timing_stop('MHD: copy_box(slice 2)')
    endif
    !call timing_start('MHD: delete_box(slice 2)')
    call rhd_box_Delete(slice)
    !call timing_stop('MHD: delete_box(slice 2)')
    !
    ! --- Handle error conditions and time step ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
  end do CHUNK_LOOP2 ! i_chunk
  !$OMP END DO NOWAIT
  !
  !$OMP CRITICAL (HD2_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  dtime_limit =min(dtime_limit, dtime_limit_thread)
  !
  if (output) then
    call rhd_datx_Add(dat%x13, dat_thread%x13)
    call rhd_datx_Add(dat%r  , dat_thread%r  )
    call rhd_dat_Delete(dat_thread)
  endif
  !$OMP END CRITICAL (HD2_REDUCTION)
  !
  !$OMP END PARALLEL
  ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  !
  call timing_stop('MHD: 2')
  !
  if (ierr0 > 0) then
    ! --- Error ---
    !
    goto 999
  endif
  !
  OutRadialC_flag=.false.
  !
  ! --- At least one hydro step performed, already ---
  StepDone_flag=.true.
  !
endif
!
if (par%hdsplit /= 'unsplit') then
  ! \label{f90:c_rhochangetop__useMHD3}
  ! --- Open upper boundary condition ---
  if (((action%bound_r3 == bound_transmitting ) .or. &
       (action%bound_r3 == bound_transmitting2) .or. &
       (action%bound_r3 == bound_transmitting3)       ) .and. &
      ( par%C_rhoChangeTop > 0.0)                              ) then
    call timing_start('MHD: bound_3DTransmit')
    call rhd_bound_3DTransmit_mhd(model, action, dtime)
    call timing_stop('MHD: bound_3DTransmit')
  endif
  !
  ! --- Open lower boundary condition ---
  if ((action%bound_l3 == bound_inoutflow ) .or. &
      (action%bound_l3 == bound_inoutflow2)     ) then
    call timing_start('MHD: bound_3DInOutFlow')
  !  write(6,'(A, 4(1PE14.6))') 'inoutflow: minmax(rho), minmax(ei)', &
  !    minval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    minval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3))
    call rhd_bound_3DInOutFlow_mhd(model, action, dtime)
  !  write(6,'(A, 4(1PE14.6))') 'inoutflow: minmax(rho), minmax(ei)', &
  !    minval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    minval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
  !    maxval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3))
    call timing_stop('MHD: bound_3DInOutFlow')
  endif
endif
!
if (model%n3 > model%m3) then
  !
  ! --- Magneto-Hydrodynamics substep in x3 direction ---
  !
  action_mhd=rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)))
  !
  ! --- Determine number of chunks ---
  call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                         model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                         m1c,n1c, m2c,n2c, &
                         n_chunks=n_chunks)
  write(gl%nc_p,'(A,I10,I6)') &
    'MHD 3: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
  call flush(gl%nc_p)
  !
  ierr0           =0
  outstr0         =''
  !
  call timing_start('MHD: 3')
  !
  ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(dat_thread, slice, &
  !$OMP                  m1c, n1c, m2c, n2c, &
  !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
  !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  !$OMP CRITICAL (HD3_INIT)
  outstr_thread     =''
  ierr_thread       =0
  dtime_limit_thread=huge(dtime_limit_thread)
  !
  if (output) then
    ! --- Thread-local data: set data fields to zero, make radial axis ---
    call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
  endif
  !$OMP END CRITICAL (HD3_INIT)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  CHUNK_LOOP3: do i_chunk=1,n_chunks
    ! --- Current chunk indices ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk, &
                           m1c,n1c, m2c,n2c)
    !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
    !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
    !
    ! --- Create empty slice and fill it with data ---
    !call timing_start('MHD: make_box(slice 3)')
    call rhd_box_Make(slice, m1=model%m3, n1=model%n3, m2=m2c, n2=n2c, m3=m1c, n3=n1c, &
                             nghost1=par%A%reconstruction_order+1, &
                             quc_flag=.true., Bc_flag=.true.)
    !call timing_stop('MHD: make_box(slice 3)')
    !call timing_start('MHD: copy_box(slice 3)')
    call rhd_box_Copy(model, m1c     , n1c, m2c     , n2c, model%m3, model%n3, &
                      slice, slice%m1,      slice%m2,      slice%m3, &
                      trans_mode=3, flux_mode=0)
    !call timing_stop('MHD: copy_box(slice 3)')
    !
    ! --- Actual hydro step ---
    select case (par%A%hdScheme)
      case (hdScheme_RoeMHD)
        if (output) then
          call rhd_mhd_Roe1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                                (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                                (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                                   datx=dat_thread%x12, datr=dat_thread%r, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_Roe1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case (hdScheme_HLLMHD, hdScheme_CenMHD)
        if (output) then
          call rhd_mhd_HLL1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                                (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                                (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                                   datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_HLL1Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case default
        dtime_limit_chunk=huge(dtime_limit_chunk)
        ierr_chunk       =0
        outstr_thread    =''
    end select
    !
    ! --- Copy data back into full model and delete temporary slice ---
    if (par%hdsplit == 'unsplit') then
      if (StepDone_flag) then
        ! --- Add change ---
        !call timing_start('MHD: addchange_box(slice 3)')
        call rhd_box_AddChange(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                               model, &
                               modelB, m1c,               m2c,                model%m3, &
                               grid_mode=0, trans_mode=3)
        !call timing_stop('MHD: addchange_box(slice 3)')
      else
        ! --- Just copy slice ---
        !call timing_start('MHD: copy_box(slice 3)')
        call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelB, m1c,               m2c,                model%m3, &
                          grid_mode=0, trans_mode=3)
        !call timing_stop('MHD: copy_box(slice 3)')
      endif
    else
      !call timing_start('MHD: copy_box(slice 3)')
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        model, m1c,                m2c,                model%m3, &
                        grid_mode=0, trans_mode=3)
      !call timing_stop('MHD: copy_box(slice 3)')
    endif
    !call timing_start('MHD: delete_box(slice 3)')
    call rhd_box_Delete(slice)
    !call timing_stop('MHD: delete_box(slice 3)')
    !
    ! --- Handle error conditions and time step ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
  end do CHUNK_LOOP3 ! i_chunk
  !$OMP END DO NOWAIT
  !
  !$OMP CRITICAL (HD3_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  dtime_limit =min(dtime_limit, dtime_limit_thread)
  !
  if (output) then
    call rhd_datx_Add(dat%x12, dat_thread%x12)
    call rhd_datx_Add(dat%r  , dat_thread%r  )
    call rhd_dat_Delete(dat_thread)
  endif
  !$OMP END CRITICAL (HD3_REDUCTION)
  !
  !$OMP END PARALLEL
  ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  !
  call timing_stop('MHD: 3')
  !
  ! --- Handle error conditions and time step ---
  if (ierr0 > 0) then
    ! --- Error ---
    !
    goto 999
  endif
  !
  OutRadialC_flag=.false.
  !
  ! --- At least one hydro step performed, already ---
  StepDone_flag=.true.
  !
endif
!
if (par%hdsplit == 'unsplit') then
  call rhd_box_AllCopyOMP(modelB, model, grid_mode=0)
  if (par%hdTmpStructKeepMode == 'Delete') then
    call timing_start('MHD: delete_box')
    call rhd_box_Delete(modelB)
    call timing_stop('MHD: delete_box')
  endif
endif
!
! === Resistivity step ===
!
! \label{f90:c_resb__useMHD3}
if ((par%C_resB         > 0.0) .or. &
    (par%C_resBconstant > 0.0) .or. &
    (par%C_resEpsilon   > 0.0)     ) then
  if (model%n2 == model%m2) then
    ! === 2D ===
    !
    call timing_start('MHD: Resistivity routines 2D')
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_resCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    write(gl%nc_p,'(A,I10,I6)') &
      'RES2D: N_CellsPerChunk, n_chunks: ', par%N_resCellsPerChunk, n_chunks
    call flush(gl%nc_p)
    !
    if (n_chunks > 1) then
      ! --- Prepare box as backup for current model ---
      call rhd_box_Make(modelres, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                  m3=model%m3, n3=model%n3, Bb_flag=.false., Bc_flag=.true.)
    endif
    !
    ierr0           =0
    outstr0         =''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (RES2D_INIT)
    outstr_thread         =''
    ierr_thread           =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (output) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (RES2D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP4: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_resCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
                             m1c,n1c, m2c,n2c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=model%m3, n3=model%n3, &
                               nghost1=1, nghost2=0, nghost3=1, Bb_flag=.false., Bc_flag=.true.)
      call rhd_bound_ExtractBox(model, action, &
                                       m1c     , n1c, m2c     , n2c, model%m3, model%n3, 1, &
                                slice, slice%m1,      slice%m2,      slice%m3              )
      !
      ! --- Actual 2D resistivity step ---
      if (output) then
        call rhd_mhd_Res2DStep(slice, action, dtime, dtime_limit_chunk, &
                               output=output, &
                               AreaFraction1=1.0, &
                               AreaFraction2=1.0, &
                               AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               dat=dat_thread, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_mhd_Res2DStep(slice, action, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelres, m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP4 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (RES2D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (output) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (RES2D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from resistivity step into standard box and delete the former ---
      call rhd_box_Copy(modelres, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                        model   , model%m1,           model%m2,           model%m3, &
                        trans_mode=1)
      call rhd_box_Delete(modelres)
    endif
    !
    call timing_stop('MHD: Resistivity routines 2D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  else
    ! === 3D ===
    !
    call timing_start('MHD: Resistivity routines 3D')
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(1, par%N_resCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    write(gl%nc_p,'(A,I10,I6)') &
      'RES3D: N_CellsPerChunk, n_chunks: ', par%N_resCellsPerChunk, n_chunks
    call flush(gl%nc_p)
    !
    if (n_chunks > 1) then
      ! --- Prepare box as backup for current model ---
      ! --- Bugfix 2014-08-17 B.F.: actually allocate Bc components (before: no mag. fields at all)
      call rhd_box_Make(modelres, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                  m3=model%m3, n3=model%n3, Bb_flag=.false., Bc_flag=.true.)
    endif
    !
    ierr0           =0
    outstr0         =''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (RES3D_INIT)
    outstr_thread         =''
    ierr_thread           =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (output) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (RES3D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP5: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
      call rhd_box_ChunkSize(1, par%N_resCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
                             m1c,n1c, m2c,n2c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=model%m3, n3=model%n3, &
                               nghost1=1, nghost2=1, nghost3=1, Bb_flag=.false., Bc_flag=.true.)
      call rhd_bound_ExtractBox(model, action, &
                                       m1c     , n1c, m2c     , n2c, model%m3, model%n3, 1, &
                                slice, slice%m1,      slice%m2,      slice%m3              )
      !
      ! --- Actual 3D resistivity step ---
      if (output) then
        call rhd_mhd_Res3DStep(slice, action, dtime, dtime_limit_chunk, &
                               output=output, &
                               AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               AreaFraction2=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)), &
                               AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               dat=dat_thread, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_mhd_Res3DStep(slice, action, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelres, m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           model%m3,&
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP5 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (RES3D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (output) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (RES3D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from resistivity step into standard box and delete the former ---
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=model%m3,model%n3
        call rhd_box_Copy(modelres, model%m1, model%n1, model%m2, model%n2, i3, i3, &
                          model   , model%m1,           model%m2,           i3,     &
                          grid_mode=0, time_mode=0, trans_mode=1)
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      call rhd_box_Delete(modelres)
    endif
    !
    call timing_stop('MHD: Resistivity routines 3D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  endif
endif
!
! --- Constrained transport step ---
if (doConstrain_flag) then
  if (output) then
    call rhd_mhd_Constrain(model, action, dtime, &
                           output=output, dat=dat)
  else
    call rhd_mhd_Constrain(model, action, dtime)
  endif
endif
!
999 continue
!
if (par%hdsplit == 'unsplit') then
  if (par%hdTmpStructKeepMode == 'Delete') then
    call timing_start('MHD: delete_box')
    if (rhd_box_Exist(modelB)) call rhd_box_Delete(modelB)
    call timing_stop('MHD: delete_box')
  endif
endif
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
call timing_stop('MHD: single step')
!
end subroutine rhd_mhd_SingleStep


!----------*******************----------------------------------------------------------------------
subroutine rhd_mhd_UnsplitStep(model, action, dtime, dtime_limit, &
                               doBb2Bc_flag, doConstrain_flag, &
                               output, dat, &
                               outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_UnsplitStep ('rhd_magneto-hydrodynamics_Unsplit_Step')
!
! PURPOSE:
!   Perform single (unsplit) 2D or 3D magneto-hydrodynamics step, employ HLLE scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_UnsplitStep(model, action, dtime, dtime_limit, &
!                            output=output, &
!                            dat=dat, &
!                            outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') type(dat_type), optional, structure for additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!
! LOCAL VARIABLES:
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_bound_3DInOutFlow_mhd: Correct values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DTransmit_mhd: Correct values in cells of all sides for transmitting boundary cond
!   rhd_bound_ExtractBox:  Extract part of box, fill boundary ghost cells
!   rhd_box_AllCopyOMP:    Copy entire box, in parallel if box is 3D
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_datx_Add:          Add the fields in the structure for additional data
!   rhd_hyd_CoriolisOMP:   Apply Coriolis rotation (with local OpenMP directives)
!   rhd_mhd_Bb2Bc:         Compute cell-centered magnetic field from boundary values
!   rhd_mhd_Constrain:     Perform constrained-transport step for magnetic fields
!   rhd_mhd_HLL2Dstep:     Perform 2D magnetohydrodynamics step (HLL solver)
!   rhd_mhd_HLL3Dstep:     Perform 3D magnetohydrodynamics step (HLL solver)
!   rhd_mhd_Res2DStep:     Perform 2D resistivity step
!   rhd_mhd_Res3DStep:     Perform 3D resistivity step
!   rhd_wrk_module:        Provide persistent work copies of model data
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify the global variable modelB.
!
! RESTRICTIONS:
!
! KEYWORDS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2010-06-03 (W.S.) First version
!   2010-06-14 (B.F.) Branches for 'unsplit' update
!   2010-06-16 (B.F.) In 'unsplit' case do open boundary steps at beginning
!   2010-07-16 (B.F.) doBb2Bc_flag, doConstrain_flag
!   2011-01-05 (W.S.) Parameter model0 removed
!   2011-06-03 (W.S.) Derived from routine rhd_mhd_SingleStep
!   2013-03-05 (B.F.) Parameter outradialc removed
!   2013-08-09 (B.F.) Change call to rhd_dat_make
!   2014-08-17 (B.F.) Replace local modelout by global modelB
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_wrk_module
use rhd_bound_module
use rhd_phys_module
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,                   intent(in)   :: doBb2Bc_flag, doConstrain_flag
logical,         optional, intent(in)   :: output
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
real                                    :: AreaFraction1, AreaFraction2, AreaFraction3
real                                    :: dtime_limit_chunk,  dtime_limit_thread, dtime_limit_mhd
integer                                 :: i_chunk, n_chunks, m1c,n1c, m2c,n2c, m3c,n3c, i1, i2, i3
integer                                 :: ierr0, ierr_chunk, ierr_thread
logical                                 :: StepDone_flag
character(len=80)                       :: outstr0, outstr_chunk, outstr_thread
type(box_type)                          :: slice, modelres
type(dat_type)                          :: dat_thread
type(action_type)                       :: action_mhd
!integer                                 :: omp_get_thread_num
!logical                                 :: omp_in_parallel
!---------------------------------------------------------------------------------------------------
!
call timing_start('MHD: unsplit step')
!
dtime_limit=huge(dtime_limit)
!
if (doBb2Bc_flag) then
  ! --- Calculate cell-centered magnetic field ---
  call rhd_mhd_Bb2Bc(model)
endif
!
! \label{f90:nu_rotation__useMHD3}
! === Coriolis terms ===============================================================================
!
if (par%nu_rotation /= 0.0) then
  call rhd_hyd_CoriolisOMP(model, dtime)
endif
!
! --- No MHD step performed yet ---
StepDone_flag=.false.
!
! --- Transmitting boundary condition ---
call timing_start('MHD: bound_3DTransmit')
call rhd_bound_3DTransmit_mhd(model, action, dtime)
call timing_stop('MHD: bound_3DTransmit')
!
! --- Open lower boundary condition ---
if ((action%bound_l3 == bound_inoutflow ) .or. &
    (action%bound_l3 == bound_inoutflow2)     ) then
  call timing_start('MHD: bound_3DInOutFlow')
!  write(6,'(A, 4(1PE14.6))') 'inoutflow: minmax(rho), minmax(ei)', &
!    minval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
!    maxval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
!    minval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
!    maxval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3))
  call rhd_bound_3DInOutFlow_mhd(model, action, dtime)
!  write(6,'(A, 4(1PE14.6))') 'inoutflow: minmax(rho), minmax(ei)', &
!    minval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
!    maxval(model%rho(model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
!    minval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3)), &
!    maxval(model%ei( model%m1:model%n1, model%m2:model%n2, model%m3:model%n3))
  call timing_stop('MHD: bound_3DInOutFlow')
endif
!
if (model%n2 == model%m2) then
  !
  ! --- 2D Unsplit Magneto-Hydrodynamics substep ---
  !
  action_mhd=rhd_action_New(action, dim=1)
  !
  ! --- Determine number of chunks ---
  call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                         model%m1,model%n1, model%m3,model%n3, model%m2,model%n2, 1, &
                         m3c,n3c, m2c,n2c, &
                         n_chunks=n_chunks)
  write(gl%nc_p,'(A,I10,I6)') &
    'MHD2D: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
  call flush(gl%nc_p)
  !
  if (n_chunks > 1) then
    ! --- Prepare box as backup for current model ---
    if (.not. rhd_box_Exist(modelB)) then
      call rhd_box_Make(modelB, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                m3=model%m3, n3=model%n3, Bb_flag=.false., Bc_flag=.true.)
    endif
    ! --- Copy only axes ---
    call rhd_box_Copy(model,  model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                      modelB, model%m1,           model%m2,           model%m3, &
                      grid_mode=1, time_mode=1, bulk_mode=0)
    endif
    !
  ierr0           =0
  outstr0         =''
  !
  call timing_start('MHD: 2D')
  !
  ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(dat_thread, slice, &
  !$OMP                  m2c, n2c, m3c, n3c, &
  !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
  !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  !$OMP CRITICAL (HD2D_INIT)
  outstr_thread     =''
  ierr_thread       =0
  dtime_limit_thread=huge(dtime_limit_thread)
  !
  if (output) then
    ! --- Thread-local data: set data fields to zero, make radial axis ---
    call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
  endif
  !$OMP END CRITICAL (HD2D_INIT)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  CHUNK_LOOP2D: do i_chunk=1,n_chunks
    ! --- Current chunk indices ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m3,model%n3, model%m2,model%n2, i_chunk, &
                           m3c,n3c, m2c,n2c)
    !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
    !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
    !
    ! --- Create empty slice and fill it with data ---
    !call timing_start('MHD: make_box(slice 1)')
    call rhd_box_Make(slice, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                             nghost1=par%A%reconstruction_order+1, &
                             nghost3=par%A%reconstruction_order+1, &
                             quc_flag=.true., Bc_flag=.true.)
    !call timing_stop('MHD: make_box(slice 1)')
    !call timing_start('MHD: copy_box(slice 1)')
    call rhd_bound_ExtractBox(model, action, &
                                     model%m1, model%n1, m2c     , n2c, m3c     , n3c, &
                                                                     par%A%reconstruction_order+1, &
                              slice, slice%m1,           slice%m2,      slice%m3)
    !call timing_stop('MHD: copy_box(slice 1)')
    !
    ! --- Actual hydro step ---
    select case (par%A%hdScheme)
      case (hdScheme_HLLMHD, hdScheme_CenMHD)
        if (output) then
          call rhd_mhd_HLL2Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, &
                                   AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                 (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                                 (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                 (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   AreaFraction3=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                 (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                                   datx1=dat_thread%x23, datx3=dat_thread%x12, datr=dat_thread%r, &
                                   datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_HLL2Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case default
        dtime_limit_chunk=huge(dtime_limit_chunk)
        ierr_chunk       =0
        outstr_chunk     =''
    end select
    !
    ! --- Copy data back into full model and delete temporary slice ---
    ! --- Just copy slice ---
    !call timing_start('MHD: copy_box(slice 1)')
    if (n_chunks > 1) then
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        modelB, model%m1,          m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
    else
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        model   , model%m1,        m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
    endif
    !call timing_stop('MHD: copy_box(slice 1)')
    !call timing_start('MHD: delete_box(slice 1)')
    call rhd_box_Delete(slice)
    !call timing_stop('MHD: delete_box(slice 1)')
    !
    ! --- Handle error conditions and time step ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
  end do CHUNK_LOOP2D ! i_chunk
  !$OMP END DO NOWAIT
  !
  !$OMP CRITICAL (HD2D_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  dtime_limit =min(dtime_limit, dtime_limit_thread)
  !
  if (output) then
    call rhd_datx_Add(dat%x23, dat_thread%x23)
    call rhd_datx_Add(dat%x12, dat_thread%x12)
    call rhd_datx_Add(dat%r  , dat_thread%r  )
    call rhd_dat_Delete(dat_thread)
  endif
  !$OMP END CRITICAL (HD2D_REDUCTION)
  !
  !$OMP END PARALLEL
  ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  !
  if (n_chunks > 1) then
    ! --- Copy output box from MHD2D step into standard box and delete the former ---
    call rhd_box_Copy(modelB, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                      model , model%m1,           model%m2,           model%m3, &
                      grid_mode=0, trans_mode=1)
    if (par%hdTmpStructKeepMode == 'Delete') then
      call timing_start('MHD: delete_box')
      call rhd_box_Delete(modelB)
      call timing_stop('MHD: delete_box')
    endif
    !
  endif
  !
  call timing_stop('MHD: 2D')
  !
  ! --- Handle error conditions and time step ---
  if (ierr0 > 0) then
    ! --- Error ---
    goto 999
  endif
  !
else
  !
  ! --- 3D Unsplit Magneto-Hydrodynamics substep ---
  !
  action_mhd=rhd_action_New(action, dim=1)
  !
  ! --- Determine number of chunks ---
  call rhd_box_ChunkSize(1, par%N_hydCellsPerChunk, &
                         model%m1,model%n1, model%m3,model%n3, model%m2,model%n2, 1, &
                         m3c,n3c, m2c,n2c, &
                         n_chunks=n_chunks)
  write(gl%nc_p,'(A,I10,I6)') &
    'MHD3D: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
  call flush(gl%nc_p)
  !
  if (n_chunks > 1) then
    ! --- Prepare box as backup for current model ---
    call rhd_box_Make(modelB, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                              m3=model%m3, n3=model%n3, Bb_flag=.false., Bc_flag=.true.)
    ! --- Copy only axes ---
    call rhd_box_Copy(model,  model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                      modelB, model%m1,           model%m2,           model%m3, &
                      grid_mode=1, time_mode=1, bulk_mode=0)
  endif
  !
  ierr0           =0
  outstr0         =''
  !
  call timing_start('MHD: 3D')
  !
  ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(dat_thread, slice, &
  !$OMP                  m2c, n2c, m3c, n3c, &
  !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
  !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  !$OMP CRITICAL (HD3D_INIT)
  outstr_thread     =''
  ierr_thread       =0

  dtime_limit_thread=huge(dtime_limit_thread)
  !
  if (output) then
    ! --- Thread-local data: set data fields to zero, make radial axis ---
    call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
  endif
  !$OMP END CRITICAL (HD3D_INIT)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  CHUNK_LOOP3D: do i_chunk=1,n_chunks
    ! --- Current chunk indices ---
    call rhd_box_ChunkSize(1, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m3,model%n3, model%m2,model%n2, i_chunk, &
                           m3c,n3c, m2c,n2c)
    !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
    !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
    !
    ! --- Create empty slice and fill it with data ---
    call rhd_box_Make(slice, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                             nghost1=par%A%reconstruction_order+1, &
                             nghost2=par%A%reconstruction_order+1, &
                             nghost3=par%A%reconstruction_order+1, &
                             quc_flag=.true., Bc_flag=.true.)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('MHD: extract_chunk')
!#
#   endif
!#
    call rhd_bound_ExtractBox(model, action, &
                                     model%m1, model%n1, m2c     , n2c, m3c     , n3c, &
                                                                     par%A%reconstruction_order+1, &
                              slice, slice%m1,           slice%m2,      slice%m3)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('MHD: extract_chunk')
!#
#   endif
!#
    !
    ! --- Actual hydro step ---
    select case (par%A%hdScheme)
      case (hdScheme_HLLMHD, hdScheme_CenMHD)
        if (output) then
          call rhd_mhd_HLL3Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 output=output, &
                                   AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                 (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                                 (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                 (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   AreaFraction2=(model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                                 (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                   AreaFraction3=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                                 (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                                   datx1=dat_thread%x23, datx2=dat_thread%x13, &
                                   datx3=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else
          call rhd_mhd_HLL3Dstep(slice, action_mhd, dtime, dtime_limit_chunk, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif
      case default
        dtime_limit_chunk=huge(dtime_limit_chunk)
        ierr_chunk       =0
        outstr_chunk     =''
    end select
    !
    ! --- Copy data back into full model and delete temporary slice ---
    ! --- Just copy slice ---
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('MHD: copy_chunk')
!#
#   endif
!#
    if (n_chunks > 1) then
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        modelB, model%m1,          m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
    else
      call rhd_box_Copy(slice, slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                        model   , model%m1,        m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
    endif
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('MHD: copy_chunk')
!#
#   endif
!#
    call rhd_box_Delete(slice)
    !
    ! --- Handle error conditions and time step ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
  end do CHUNK_LOOP3D ! i_chunk
  !$OMP END DO NOWAIT
  !
  !$OMP CRITICAL (HD3D_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  dtime_limit =min(dtime_limit, dtime_limit_thread)
  !
  if (output) then
    call rhd_datx_Add(dat%x12, dat_thread%x12)
    call rhd_datx_Add(dat%x13, dat_thread%x13)
    call rhd_datx_Add(dat%x23, dat_thread%x23)
    call rhd_datx_Add(dat%r  , dat_thread%r  )
    call rhd_dat_Delete(dat_thread)
  endif
  !$OMP END CRITICAL (HD3D_REDUCTION)
  !
  !$OMP END PARALLEL
  ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  !
  if (n_chunks > 1) then
    ! --- Copy output box from MHD3D step into standard box and delete the former ---
    call rhd_box_AllCopyOMP(modelB, model, grid_mode=0)
    if (par%hdTmpStructKeepMode == 'Delete') then
      call timing_start('MHD: delete_box')
      call rhd_box_Delete(modelB)
      call timing_stop('MHD: delete_box')
    endif
  endif
  !
  call timing_stop('MHD: 3D')
  !
  ! --- Handle error conditions and time step ---
  if (ierr0 > 0) then
    ! --- Error ---
    goto 999
  endif
  !
endif
!
! === Resistivity step ===
!
! \label{f90:c_resb__useMHD4}
if ((par%C_resB         > 0.0) .or. &
    (par%C_resBconstant > 0.0) .or. &
    (par%C_resEpsilon   > 0.0)     ) then
  if (model%n2 == model%m2) then
    ! === 2D ===
    !
    call timing_start('MHD: Resistivity routines 2D')
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_resCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    write(gl%nc_p,'(A,I10,I6)') &
      'RES2D: N_CellsPerChunk, n_chunks: ', par%N_resCellsPerChunk, n_chunks
    call flush(gl%nc_p)
    !
    if (n_chunks > 1) then
      ! --- Prepare box as backup for current model ---
      call rhd_box_Make(modelres, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                  m3=model%m3, n3=model%n3, Bb_flag=.false., Bc_flag=.true.)
    endif
    !
    ierr0           =0
    outstr0         =''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (RES2D_INIT)
    outstr_thread         =''
    ierr_thread           =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (output) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (RES2D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP4: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_resCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
                             m1c,n1c, m2c,n2c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=model%m3, n3=model%n3, &
                               nghost1=1, nghost2=0, nghost3=1, Bb_flag=.false., Bc_flag=.true.)
      call rhd_bound_ExtractBox(model, action, &
                                       m1c     , n1c, m2c     , n2c, model%m3, model%n3, 1, &
                                slice, slice%m1,      slice%m2,      slice%m3              )
      !
      ! --- Actual 2D resistivity step ---
      if (output) then
        call rhd_mhd_Res2DStep(slice, action, dtime, dtime_limit_chunk, &
                               output=output, &
                               AreaFraction1=1.0, &
                               AreaFraction2=1.0, &
                               AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               dat=dat_thread, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_mhd_Res2DStep(slice, action, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelres, m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP4 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (RES2D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (output) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (RES2D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from resistivity step into standard box and delete the former ---
      call rhd_box_Copy(modelres, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                        model   , model%m1,           model%m2,           model%m3, &
                        trans_mode=1)
      call rhd_box_Delete(modelres)
    endif
    !
    call timing_stop('MHD: Resistivity routines 2D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  else
    ! === 3D ===
    !
    call timing_start('MHD: Resistivity routines 3D')
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(1, par%N_resCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    write(gl%nc_p,'(A,I10,I6)') &
      'RES3D: N_CellsPerChunk, n_chunks: ', par%N_resCellsPerChunk, n_chunks
    call flush(gl%nc_p)
    !
    if (n_chunks > 1) then
      ! --- Prepare box as backup for current model ---
      call rhd_box_Make(modelres, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                  m3=model%m3, n3=model%n3)
    endif
    !
    ierr0           =0
    outstr0         =''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (RES3D_INIT)
    outstr_thread         =''
    ierr_thread           =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (output) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (RES3D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP5: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
      call rhd_box_ChunkSize(1, par%N_resCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
                             m1c,n1c, m2c,n2c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=model%m3, n3=model%n3, &
                               nghost1=1, nghost2=1, nghost3=1, Bb_flag=.false., Bc_flag=.true.)
      call rhd_bound_ExtractBox(model, action, &
                                       m1c     , n1c, m2c     , n2c, model%m3, model%n3, 1, &
                                slice, slice%m1,      slice%m2,      slice%m3              )
      !
      ! --- Actual 3D resistivity step ---
      if (output) then
        call rhd_mhd_Res3DStep(slice, action, dtime, dtime_limit_chunk, &
                               output=output, &
                               AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               AreaFraction2=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)), &
                               AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               dat=dat_thread, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_mhd_Res3DStep(slice, action, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelres, m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           model%m3, &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP5 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (RES3D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (output) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (RES3D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from resistivity step into standard box and delete the former ---
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=model%m3,model%n3
        call rhd_box_Copy(modelres, model%m1, model%n1, model%m2, model%n2, i3, i3, &
                          model   , model%m1,           model%m2,           i3,     &
                          grid_mode=0, time_mode=0, trans_mode=1)
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      call rhd_box_Delete(modelres)
    endif
    !
    call timing_stop('MHD: Resistivity routines 3D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  endif
endif
!
! --- Constrained transport step ---
if (doConstrain_flag) then
  if (output) then
    call rhd_mhd_Constrain(model, action, dtime, &
                           output=output, dat=dat)
  else
    call rhd_mhd_Constrain(model, action, dtime)
  endif
endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!
call timing_stop('MHD: unsplit step')
!
end subroutine rhd_mhd_UnsplitStep


!----------************-----------------------------------------------------------------------------
subroutine rhd_mhd_Step(model, action, dtime, dtime_limit, &
                        output, dat, &
                        outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_mhd_Step ('rhd_magneto-hydrodynamics_Step')
!
! PURPOSE:
!   Perform entire 2D or 3D magneto-hydrodynamics step, employ HLLE scheme.
!
! CATEGORY:
!   Magneto-Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_mhd_Step(model, action, dtime, dtime_limit, &
!                     output=output, &
!                     dat=dat, &
!                     outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   output:       ('output') logical, optional, flag that determines if additional averages are
!                 computed and put into datx and datr, default=.false.
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') type(dat_type), optional, structure for additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_box_AllAvgOMP:     Average two boxes, in parallel if n3>m3
!   rhd_box_AllCopyOMP:    Copy entire box, in parallel if box is 3D
!   rhd_box_AppendBc:      Add cell-centered magnetic-field arrays to box
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_RemoveBc:      Remove cell-centered magnetic-field arrays from box
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_mhd_SingleStep:    Perform single magnetohydrodynamic step
!   rhd_mhd_UnsplitStep:   Perform single unsplit magnetohydrodynamic step
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_wrk_module:        Provide persistent work copies of model data
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify the global variable modelA.
!
! RESTRICTIONS:
!
! KEYWORDS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2006-11-27 (B.F. Lyon) Taken from rhd.F90, put into own routine,
!                    Take dtime_limit_mhd into account in computing dtime_limit
!   2008-03-09 (W.S. East Lansing) action_mhd introduced
!   2008-03-24 (W.S.) Multiple MHD substeps possible
!   2010-02-04 (B.F.) Factor removed from AreaFraction: * dtime_substep / dtime
!   2010-04-07 (B.F.) Call external routine rhd_bound_3DTransmit to adjust density in top layers
!   2010-06-08 (B.F. Lyon) RungeKutta2: Output into dat structure only from first step
!   2011-02-25 (W.S. Exeter) Use of parameter flux to avoid unnecessary copying of flux arrays,
!                     use of OMP version of copy routine for copying the entire model
!   2011-06-10 (W.S.) Unsplit Hancock scheme
!   2013-03-05 (B.F.) Parameter outradialc removed
!   2013-06-21 (B.F.) Backup data model0 without Bc fields
!   2014-08-17 (B.F. Uppsala) Replace local model0 with global modelA
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_wrk_module
use rhd_phys_module
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: output
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
real                                    :: dtime_substep, dtime_limit_substep, &
                                           time_substep
integer                                 :: ierr0, n_iter
character(len=80)                       :: outstr0
logical                                 :: new_substep, last_substep, output_substep
!---------------------------------------------------------------------------------------------------
!
call timing_start('Magneto-Hydrodynamics routines')
!
! --- Set time step limits to maximum value ---
dtime_limit=huge(dtime_limit)
!
! --- Create empty model, if necessary ---
if (.not. rhd_box_Exist(modelA)) then
  call rhd_box_Make(modelA, &
                    m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3, &
                    quc_flag=.true., Bb_flag=.true., Bc_flag=.false.)
endif
!
! --- Add arrays for cell-centered magnetic field to box structure ---
call rhd_box_AppendBc(model)
!
! --- Initialize time step for substeps ---
if (par%N_hydIter < 1) then
  dtime_substep = dtime
else
  dtime_substep = min(dtime, dtime_limit_last)
endif
!
! --- Initialize multistep variables ---
time_substep=0.0
n_iter=1
new_substep=.true.
last_substep=.false.
!
! ### Magneto-Hydrodynamics step ###################################################################
!
! --- Loop over sub time steps ---
DTIME_LOOP: do
  !
  ! --- Check if maximum number of iterations reached ---
  if (par%N_hydMaxIter > par%N_hydIter .and. n_iter > par%N_hydMaxIter) then
    ierr0=1
    outstr0='MHD: Number of sub time steps above absolute limit'
    dtime_limit=par%N_hydIter*time_substep
    dtime_limit_last=dtime_limit_substep
    exit DTIME_LOOP
  endif
  !
  ! \label{f90:dtime_min_stop__useMHD1}
  ! --- Check if time step is below absolute limit ---
  if (dtime_substep < par%dtime_min_stop) then
    write(gl%nc_p,'(A,1PE13.6,A,1PE13.6)') &
          'Time step ', dtime_substep, ' below absolute limit ', &
          par%dtime_min_stop
    ierr0  =200
    outstr0='Time step below absolute limit'
    exit DTIME_LOOP
  endif
  !
  ! --- Check if this substep is the last ---
  if (time_substep + dtime_substep * &
     (0.1*max((par%C_CourantMax-par%C_Courant)/par%C_Courant, 0.0)+1.0) > dtime) then
      !
      ! --- Limit time step to exactly reach total time step ---
      dtime_substep = dtime - time_substep
      last_substep=.true.
  endif
  !
  dtime_limit_substep=huge(dtime_limit_substep)
  !
  if (par%N_hydIter > 0) then
    write(gl%nc_p,'(I4,A,1PE10.4)') n_Iter, '. MHD-substep, dtimesub=', dtime_substep
  endif
  !
  if (new_substep) then
    !
    ! --- Save old model ---
    call timing_start('MHD: copy_box')
    call rhd_box_AllCopyOMP(model, modelA, grid_mode=1, flux_mode=0)
    call timing_stop('MHD: copy_box')
    !
    !!$write(gl%nc_p, '(A,2E16.7)') 'rhd_mhd_Step: new_substep=T; model%ei_min,max =', &
    !!$                             minval(model%ei), maxval(model%ei)
    !!$write(gl%nc_p, '(A,2E16.7)') 'rhd_mhd_Step: new_substep=T; modelA%ei_min,max=', &
    !!$                             minval(modelA%ei), maxval(modelA%ei)
    !
  else
    !
    !!$write(gl%nc_p, '(A,2E16.7)') 'rhd_mhd_Step: new_substep=F; modelA%ei_min,max=', &
    !!$                             minval(modelA%ei), maxval(modelA%ei)
    !
    ! --- When substep is repeated copy old model back ---
    call timing_start('MHD: copy_box')
    call rhd_box_AllCopyOMP(modelA, model, grid_mode=1, flux_mode=0)
    call timing_stop('MHD: copy_box')
    !
  endif
  !
  if (output .and. n_iter == 1) then
    ! --- If first step repeated, reset data ---
    call rhd_dat_Reset(dat)
    output_substep=output
  else
    output_substep=.false.
  endif
  !
  ! --- Perform MHD substep ---
  !
  ! \label{f90:hdtimeintegrationscheme__useMHD1}
  select case(par%hdTimeIntegrationScheme)
    !
    case ('RungeKutta3')
      ! --- 3rd order TVD Runge-Kutta ---
      !
      ! --- First Euler step ---
      call rhd_mhd_SingleStep(model, action, dtime_substep, dtime_limit_substep, &
                              doBb2Bc_flag=.true., doConstrain_flag=.true., &
                              output=output_substep, &
                              dat=dat, &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        if (ierr0 > 99) then
          !
          ! --- Severe error, stop magnetohydrodynamics step ---
          exit DTIME_LOOP
        else
          !
          ! --- Time step violation, do substep again
          if (par%N_hydIter < 1) then
            !
            ! --- Single step mode: exit magnetohydrodynamic step ---
            dtime_limit=dtime_limit_substep
            !
            exit DTIME_LOOP
          else
            ! --- Multi step mode: do substep again ---
            write(gl%nc_p,'(A/1PE13.6,A,1PE13.6)') &
                  'MHD:  Recomputation with reduced time step: ' // trim(outstr0) // ' ', &
                  dtime_substep, ' -> ', dtime_limit_substep
            call flush(gl%nc_p)
            dtime_substep = dtime_limit_substep
            new_substep=.false.
            last_substep=.false.
            cycle DTIME_LOOP
          endif
        endif
      endif
      !
      ! --- Second Euler step ---
      call rhd_mhd_SingleStep(model, action, dtime_substep, dtime_limit_substep, &
                              doBb2Bc_flag=.true., doConstrain_flag=.true., &
                              output=.false., &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        if (ierr0 > 99) then
          !
          ! --- Severe error, stop magnetohydrodynamics step ---
          exit DTIME_LOOP
        else
          !
          ! --- Time step violation, do substep again
          if (par%N_hydIter < 1) then
            !
            ! --- Single step mode: exit magnetohydrodynamic step ---
            dtime_limit=dtime_limit_substep
            !
            exit DTIME_LOOP
          else
            ! --- Multi step mode: do substep again ---
            write(gl%nc_p,'(A/1PE13.6,A,1PE13.6)') &
                  'MHD:  Recomputation with reduced time step: ' // trim(outstr0) // ' ', &
                  dtime_substep, ' -> ', dtime_limit_substep
            call flush(gl%nc_p)
            dtime_substep = dtime_limit_substep
            new_substep=.false.
            last_substep=.false.
            cycle DTIME_LOOP
          endif
        endif
      endif
      !
      call timing_start('MHD: avg_box')
      call rhd_box_AllAvgOMP(modelA, model, 0.25)
      call timing_stop('MHD: avg_box')
      !
      ! --- Third Euler step ---
      call rhd_mhd_SingleStep(model, action, dtime_substep, dtime_limit_substep, &
                              doBb2Bc_flag=.true., doConstrain_flag=.true., &
                              output=.false., &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        if (ierr0 > 99) then
          !
          ! --- Severe error, stop magnetohydrodynamics step ---
          exit DTIME_LOOP
        else
          !
          ! --- Time step violation, do substep again
          if (par%N_hydIter < 1) then
            !
            ! --- Single step mode: exit magnetohydrodynamic step ---
            dtime_limit=dtime_limit_substep
            !
            exit DTIME_LOOP
          else
            ! --- Multi step mode: do substep again ---
            write(gl%nc_p,'(A/1PE13.6,A,1PE13.6)') &
                  'MHD:  Recomputation with reduced time step: ' // trim(outstr0) // ' ', &
                  dtime_substep, ' -> ', dtime_limit_substep
            call flush(gl%nc_p)
            dtime_substep = dtime_limit_substep
            new_substep=.false.
            last_substep=.false.
            cycle DTIME_LOOP
          endif
        endif
      endif
      !
      call timing_start('MHD: avg_box')
      call rhd_box_AllAvgOMP(modelA, model, 0.666666666666666666666666)
      call timing_stop('MHD: avg_box')
      !
    case ('RungeKutta2')
      ! --- 2nd order TVD Runge-Kutta ---
      !
      ! --- First Euler step ---
      call rhd_mhd_SingleStep(model, action, dtime_substep, dtime_limit_substep, &
                              doBb2Bc_flag=.true., doConstrain_flag=.true., &
                              output=output_substep, &
                              dat=dat, &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        if (ierr0 > 99) then
          !
          ! --- Severe error, stop magnetohydrodynamics step ---
          exit DTIME_LOOP
        else
          !
          ! --- Time step violation, do substep again
          if (par%N_hydIter < 1) then
            !
            ! --- Single step mode: exit magnetohydrodynamic step ---
            dtime_limit=dtime_limit_substep
            !
            exit DTIME_LOOP
          else
            ! --- Multi step mode: do substep again ---
            write(gl%nc_p,'(A/1PE13.6,A,1PE13.6)') &
                  'MHD:  Recomputation with reduced time step: ' // trim(outstr0) // ' ', &
                  dtime_substep, ' -> ', dtime_limit_substep
            call flush(gl%nc_p)
            dtime_substep = dtime_limit_substep
            new_substep=.false.
            last_substep=.false.
            cycle DTIME_LOOP
          endif
        endif
      endif
      !
      ! --- Second Euler step ---
      call rhd_mhd_SingleStep(model, action, dtime_substep, dtime_limit_substep, &
                              doBb2Bc_flag=.true., doConstrain_flag=.true., &
                              output=.false., &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        if (ierr0 > 99) then
          !
          ! --- Severe error, stop magnetohydrodynamics step ---
          exit DTIME_LOOP
        else
          !
          ! --- Time step violation, do substep again
          if (par%N_hydIter < 1) then
            !
            ! --- Single step mode: exit magnetohydrodynamic step ---
            dtime_limit=dtime_limit_substep
            !
            exit DTIME_LOOP
          else
            ! --- Multi step mode: do substep again ---
            write(gl%nc_p,'(A/1PE13.6,A,1PE13.6)') &
                  'MHD:  Recomputation with reduced time step: ' // trim(outstr0) // ' ', &
                  dtime_substep, ' -> ', dtime_limit_substep
            call flush(gl%nc_p)
            dtime_substep = dtime_limit_substep
            new_substep=.false.
            last_substep=.false.
            cycle DTIME_LOOP
          endif
        endif
      endif
      !
      call timing_start('MHD: avg_box')
      call rhd_box_AllAvgOMP(modelA, model, 0.5)
      call timing_stop('MHD: avg_box')
      !
    case default
      ! --- Simple Euler step ---
      !
      if (par%A%hdScheme              == hdScheme_RoeMHD .or. &
          par%hdsplit                 == '123'           .or. &
          par%hdTimeIntegrationScheme == 'Euler1'              ) then
        call rhd_mhd_SingleStep(model, action, dtime_substep, dtime_limit_substep, &
                                doBb2Bc_flag=.true., doConstrain_flag=.true., &
                                output=output_substep, &
                                dat=dat, &
                                outstr=outstr0,ierr=ierr0)
      else
        ! --- Unsplit Hancock scheme ---
        call rhd_mhd_UnsplitStep(model, action, dtime_substep, dtime_limit_substep, &
                                 doBb2Bc_flag=.true., doConstrain_flag=.true., &
                                 output=output_substep, &
                                 dat=dat, &
                                 outstr=outstr0,ierr=ierr0)
      endif
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        if (ierr0 > 99) then
          !
          ! --- Severe error, stop magnetohydrodynamics step ---
          exit DTIME_LOOP
        else
          !
          ! --- Time step violation, do substep again ---
          if (par%N_hydIter < 1) then
            !
            ! --- Single step mode: exit magnetohydrodynamic step ---
            dtime_limit=dtime_limit_substep
            !
            exit DTIME_LOOP
          else
            ! --- Multi step mode: do substep again ---
            write(gl%nc_p,'(A/1PE13.6,A,1PE13.6)') &
                  'MHD:  Recomputation with reduced time step: ' // trim(outstr0) // ' ', &
                  dtime_substep, ' -> ', dtime_limit_substep
            call flush(gl%nc_p)
            dtime_substep = dtime_limit_substep
            new_substep=.false.
            last_substep=.false.
            cycle DTIME_LOOP
          endif
        endif
      endif
      !
  end select
  !
  if (par%N_hydIter < 1) then
    !
    ! --- Single step mode: calculate time step for next magnetohydrodynamic step ---
    dtime_limit=dtime_limit_substep
    exit DTIME_LOOP
  else
    !
    time_substep = time_substep + dtime_substep
    dtime_substep = dtime_limit_substep
    n_iter = n_iter + 1
    new_substep = .true.
    !
    if (last_substep) then
      !
      ! --- Multistep mode: calculate time step for next magnetohydrodynamic step ---
      dtime_limit=par%N_hydIter*dtime_limit_substep
      dtime_limit_last=dtime_limit_substep
      exit DTIME_LOOP
    endif
  endif
  !
end do DTIME_LOOP
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
! --- Remove cell-centered magnetic field
call rhd_box_RemoveBc(model)
if (par%hdTmpStructKeepMode == 'Delete') then
  call timing_start('MHD: delete_box')
  call rhd_box_Delete(modelA)
  call timing_stop('MHD: delete_box')
endif
!
call timing_stop('Magneto-Hydrodynamics routines')
!
end subroutine rhd_mhd_Step

end module rhd_mhd_module
