!***************************************************************************************************
!
!   #####   #    #  #####          #    #  #   #  #####
!   #    #  #    #  #    #         #    #   # #   #    #
!   #    #  ######  #    #         ######    #    #    #
!   #####   #    #  #    #         #    #    #    #    #
!   #   #   #    #  #    #         #    #    #    #    #
!   #    #  #    #  #####  ####### #    #    #    ##### _module
!
!   Radiation-Hydrodynamics Code: hydrodynamics module
!
!***************************************************************************************************
!   Fortran90
!   2019-01-24
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Werner Schaffenberger: Exeter
!   Sven Wedemeyer-Boehm:  Freiburg
!***************************************************************************************************
! MODULES:
!   rhd_hyd_module:        Hydrodynamics routines
!
!***************************************************************************************************

!# /* --- Set default values for some preprocessor macros --------------------------------------- */
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Gravity correction: 0:'off0', 1:'default0', 2,3,4,5,6,11:'defaultn' --- */
# ifndef rhd_hyd_gravcorr_p01
#   define rhd_hyd_gravcorr_p01 5
# endif
!#
!# /* --- Method of entropy-fix calculation: 0, 1, 2 --- */
# ifndef rhd_hyd_entropyfix_p01
#   define rhd_hyd_entropyfix_p01 0
# endif
!#
!# /* --- Method of upwind calculation: 0 or 1 --- */
# ifndef rhd_hyd_upwind_p01
#   define rhd_hyd_upwind_p01 0
# endif
!#
!# /* --- Loop type for alpha value (amplitude) calculation: 0 (default), 1, 2, 3 --- */
# ifndef rhd_roe1d_flux_l01
#   define rhd_roe1d_flux_l01 1
# endif
!#
!# /* --- MPI activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------**************-------------------------------------------------------------------------------
module rhd_hyd_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_module ('rhd_hydrodynamics_module')
!
! PURPOSE:
!   Provide hydrodynamics routines for rhd.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_hyd_module
!
! VARIABLES:
!   None
!
! ROUTINES: (all contained routines; public ones are marked with '>')
! > rhd_hyd_SwitchInfo:    Print information about used compiler switches
!   rhd_hyd_Roe1Doutput:   Extract some averaged quantities
!   rhd_hyd_sDiff:         Compute 1D energy flux down an entropy step with state reconstruction
!   rhd_hyd_FluxState:     Compute 1D flux from state vector for single quantity
!   rhd_hyd_Pred:          Compute reduced pressure as gravity correction for Roe solver
!   rhd_hyd_RoeWavealpha12: Compute 1D Roe sound-wave fluxes
!   rhd_hyd_RoeWavealpha36: Compute 1D Roe entropy-wave and ionization-energy fluxes
!   rhd_hyd_RoeWavealpha45: Compute 1D Roe transversal-momentum fluxes
!   rhd_hyd_Fluxrhoei:     Compute 1D internal energy flux from the fluxes of individual waves
!   rhd_hyd_RoeWave1Dflux: Compute 1D Roe fluxes with wave reconstructions
!   rhd_hyd_RoeState1Dflux: Compute 1D Roe fluxes with state reconstructions
!   rhd_hyd_Dust:          Advect passive tracers
!   rhd_hyd_mhd:           Advect magnetic fields
!   rhd_hyd_Roe1DStep:     Perform 1D hydrodynamics step in x1 direction with Roe scheme
!   rhd_hyd_Roe1DMincedStep: Perform 1D hydro step in x1 direction, call Roe solver repeatedly
!   rhd_hyd_SingleStep:    Perform entire (single) 2D or 3D hydrodynamics step, employ Roe scheme
! > rhd_hyd_Step:          Perform entire 3D hydro step, possibly composed of multiple time-steps
!
! CALLING TREE:
!   rhd_hyd_SwitchInfo
!   |
!   rhd_bound_3DCenter
!   | * eosinter
!   |
!   rhd_bound_3DHeatBottom
!   | * eosinter
!   |
!   rhd_hyd_Step
!     * rhd_hyd_SingleStep
!       + rhd_hyd_CoriolisOMP
!       + rhd_bound_3DTransmit
!       | * eosinter
!       + rhd_bound_3DInOutFlow
!       | * eosinter_all
!       + rhd_hyd_Roe1DMincedStep
!       | * rhd_hyd_Roe1DStep
!       |   ...
!       + rhd_hyd_Roe1DStep
!         * rhd_bound_1DGhost
!         * rhd_grav
!         * rhd_bound_1DTransmit
!         * + eosinter_PdPTdT
!         * rhd_bound_1DGhost
!         * eosinter_all
!         * eosinter_PdPTdT
!         * eosinter_PdP
!         * rhd_hyd_RoeState1Dflux
!         | + rhd_rec_VecEq
!         | + rhd_rec_Vec
!         * rhd_hyd_RoeWave1Dflux
!         | + rhd_hyd_Pred
!         | + rhd_hyd_RoeWavealpha12
!         |   * rhd_rec_Fluxalpha
!         |   * rhd_rec_FluxalphaEq
!         | + rhd_hyd_RoeWavealpha36
!         |   * rhd_rec_Fluxalpha
!         |   * rhd_rec_FluxalphaEq
!         |   * rhd_hyd_FluxState
!         |     . rhd_rec_VecEq
!         |     . rhd_rec_Vec
!         |   * rhd_hyd_sDiff
!         |     . rhd_rec_VecEq
!         |     . rhd_rec_Vec
!         | + rhd_hyd_RoeWavealpha45
!         |   * rhd_hyd_FluxState
!         |     . rhd_rec_VecEq
!         |     . rhd_rec_Vec
!         | + rhd_hyd_Fluxrhoei
!         * rhd_hyd_Roe1Doutput
!         | + rhd_mean_weight
!         | + rhd_mean_x23mean
!         | + rhd_mean_x23weight
!         | + rhd_mean_a2rindex
!         | + rhd_mean_rc23mean
!         | + rhd_mean_VecProj
!         | + rhd_mean_rb23mean
!         | + rhd_mean_c1Index
!         | + rhd_mean_c1VecProj
!         | + rhd_mean_ccamean
!         * rhd_hyd_Dust
!         | + rhd_hyd_FluxState
!         |    * rhd_rec_VecEq
!         |    * rhd_rec_Vec
!         | + rhd_hyd_Rossow78tfall
!         | + rhd_rec_VecEq
!         | + rhd_rec_Vec
!         | + rhd_mean_weight
!         | + rhd_mean_x23mean
!         | + rhd_mean_a2rindex
!         | + rhd_mean_rb23mean
!         * rhd_hyd_mhd
!           + rhd_rec_Fluxalpha
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   1999-05-11 (B.F. Copenhagen) Routine bound3d_inoutflow added
!   1999-05-26 (B.F.) Routine extract_box added
!   1999-05-27 (B.F.) Routine visctensor_step added
!   1999-05-28 (B.F.) Routines extract_box and visctensor_step put into own module
!   1999-06-10 (B.F.) Routine bound3d_center added
!   1999-09-28 (B.F.) Routine bound1d_transmit added
!   2001-01    (B.F. Uppsala) .f90 -> .F90, preprocessor macros added (for timing routine calls and
!                     routine versions)
!   2001-01    (B.F.) "Monster" routine Roe1D_step split into parts (because of HP compiler)
!   2001-02-11 (B.F.) New Version of Roe1D_flux (inlining highly recommended)
!   2002-10-22 (B.F.) Dust source terms (rhd_hyd_Dust)
!   2002-10-22 (B.F.) Slope reduction inside expansion wave, only some routines made public
!   2002-10-24 (B.F.) Routine vanLeer1d_step put into graveyard, renaming of some routines
!   2002-10-25 (B.F.) Bugfix in computation of dxb1_avg and dxb1_var
!   2002-10-27 (B.F.) Slope reduction removed
!   2002-11-05 (B.F.) Transport of magnetic field (rhd_hyd_mhd)
!   2002-12-29 (B.F.) Slope reduction in case of strong density contrast
!   2002-12-22 (B.F.) Slope reduction in case of strong density contrast controlled with C_SlopeRed
!   2005-06-03 (SWB ) Merged versions
!   2006-12-12 (B.F. Lyon) Iteration of hydro steps possible
!   2007-02-08 (B.F.) Unsplit hydro operator possible
!   2011-08-03 (B.F.) Branch for Colella's CTU scheme
!   2011-09-08 (B.F.) Some reconstruction routines moved into rhd_rec_module.f90
!   2011-11-14 (B.F.) Switch rhd_roe1d_slope_l01 removed
!   2012-02-21 (B.F.) Some routines moved into rhd_phys_module
!   2012-03-22 (B.F.) Created new routine rhd_hyd_RoeState1Dflux from rhd_hyd_Roe1Dflux
!   2012-04-03 (B.F.) New routine rhd_hyd_sDiff
!   2012-10-11 (B.F.) Steps towards MPI
!   2013-02-24 (B.F.) Extract and put rhd_hyd_RoeWavealpha12/36/45 into own routines
!   2013-02-25 (B.F.) Extract and put rhd_hyd_Pred and rhd_hyd_Fluxrhoei into own routines
!   2013-02-26 (B.F.) Modify meaning of rhd_roe1d_flux_l01 switch
!   2014-02-24 (B.F. Uppsala) Change default value of compiler switch
!   2014-02-26 (B.F.) Changes in OpenMP scheduling
!   2014-08-17 (B.F.) Use global boxes for working copies
!   2014-12-02 (B.F.) Remove "rhd_roe1d_step_l01=0" option
!   2016-04-15 (B.F.) Reorganize global world-geometry variables for MPI
!   2017-08-16 (B.F. Uppsala) Account for new datx type
!   2017-09-18 (B.F.) Move boundary-condition routines to separate module rhd_bound_module
!   2018-05-07 (B.F.) Remove ancient work-in-progress routine rhd_shortrad_StepOutput_new
!   2018-11-29 (B.F.) Change criterion for non-equidistant grid: (dxb1_avg<=0.0) -> (dxb1_avg<0.0)
!---------------------------------------------------------------------------------------------------
implicit none
!
private
!
public rhd_hyd_SwitchInfo, rhd_hyd_Step
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_hyd_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_SwitchInfo ('rhd_hydrodynamics_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to requested output channel.
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_hyd_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2002-08-12 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_hyd_module .........................'
!
write(ncp,'(A,I6)') '  rhd' // '_hyd_gravcorr_p01:       ', rhd_hyd_gravcorr_p01
write(ncp,'(A,I6)') '  rhd' // '_box_quc01:              ', rhd_box_quc01
write(ncp,'(A,I6)') '  rhd' // '_hyd_entropyfix_p01:     ', rhd_hyd_entropyfix_p01
write(ncp,'(A,I6)') '  rhd' // '_hyd_upwind_p01:         ', rhd_hyd_upwind_p01
write(ncp,'(A,I6)') '  rhd' // '_roe1d_flux_l01:         ', rhd_roe1d_flux_l01
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
!
!#
# ifdef rhd_roe1d_flux_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_flux_t01:      ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_flux_t01:      ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_roe1d_step_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_step_t01:      ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_roe1d_step_t01:      ', undefstr
!#
# endif
!#
!
end subroutine rhd_hyd_SwitchInfo


!----------*******************----------------------------------------------------------------------
subroutine rhd_hyd_Roe1Doutput(model, dim, AreaFraction, &
                               OutVar_flag, OutFlx_flag, OutRadialC_flag, &
                               P, T, s, Gamma_1, Gamma_3, dTdei, dPdei, &
                               phic_grav, phib_grav, &
                               rho_til, v1_til, v2_til, v3_til, &
                               flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep, &
                               datx, datr, datc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Roe1Doutput ('Roe_1D_output')
!
! PURPOSE:
!   Extract some averaged quantities.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Roe1Doutput(model, dim, AreaFraction, &
!                            OutVar_flag, OutFlx_flag, OutRadialC_flag, &
!                            P, T, s, Gamma_1, Gamma_3, dTdei, dPdei, &
!                            phic_grav, phib_grav, &
!                            rho_til, v1_til, v2_til, v3_til, &
!                            flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep, &
!                            datx=datx, datr=datr, datc=datc)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!   dim:          ('dimension') integer with dimension that is mapped to dimension 1
!   AreaFraction: ('Area_Fraction') real, weighting factor for the current model chunk
!   OutVar_flag:  ('Output_Variable_flag') logical, compute averages of bulk quantities
!   OutFlx_flag:  ('Output_Flux_flag') logical, compute averages of fluxes
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                    volume averaging of cell centered quantities is performed
!   P:            ('Pressure') real 3D array, pressure on model grid
!   T:            ('Temperature') real 3D array, temperature on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   Gamma_3:      ('Gamma_3') real 3D array, 3rd adiabatic coefficient on model grid
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array, temperature derivative
!                 at constant density on model grid
!   dPdei:        ('delta_Presure_delta_energy_internal') real 3D array, pressure derivative
!                 at constant density on model grid
!   phic_grav:    ('phi_central_gravity) real 3D array, gravitational potential, cell centers
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe-averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!
! INPUT/OUTPUT:
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for add. quantities
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   weight:       ('weight') 2D real array with weights
!   x123weight:   ('x_123_weight') 3D real array with weights
!   nbindex:      ('number_boundary_index') 2D integer array
!   bindex:       ('boundary_index') 3D integer array
!   cindex:       ('center_index') 3D integer array
!
! ROUTINES:
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_c1Index:      Produce index array for azimuthal averaging over cylinders
!   rhd_mean_c1VecProj:    Project vector on radial vector perpendicular to x1 axis
!   rhd_mean_ccamean:      Average cell-centered quantity over cylinders (azimuthal mean)
!   rhd_mean_rb23mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_rb23shmean:   Compute spherical-harmonic amplitude of boundary-centered quantity
!   rhd_mean_VecbProjUnity: Project radial unity vector on x1 axis direction
!   rhd_mean_VecProj:      Project vector on radial ray
!   rhd_mean_weight:       Produce weight array for horizontal averaging
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The input 3D real arrays and some derived quantities are averaged over planes
!   and spheres (if requested) and put into datx and datr, respectively.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-18 (B.F. Uppsala) Put into own module
!   2001-01-26 (B.F.) Bugfix for computation of feg: term 1/dxc1 removed
!   2001-03-18 (B.F.) Radial case: rhov3, frhov3
!   2008-02-29 (B.F. Lyon) Entries p2_xmean, rho2_xmean, s2_xmean added
!   2011-07-18 (B.F.) New output control parameters
!   2012-07-27 (B.F.) Move scale%length to prop%length
!   2012-09-19 (B.F.) Compute v1v1v1_xmean and v1v1v1v1_xmean
!   2013-07-08 (B.F.) New parameter dim;
!                     compute means of moment of inertia and angular momentum over spheres
!   2013-07-30 (B.F.) New local variable qr
!   2013-08-15 (B.F.) New optional argument datc
!   2013-08-20 (B.F.) Means of several momentum fluxes; lots of new azimuthal means
!   2017-08-16 (B.F. Uppsala) Account for new datx type
!   2017-08-20 (B.F. Lyon) Change if statements and indentations
!   2017-08-21 (B.F.) Compute temperature averages if no radiation transport is done;
!                     don't count outside-boundary fluxes twice for horizontal means;
!                     make routine more similar to HD counterpart;
!   2017-08-24 (B.F.) Compute AreaFraction from local and global quantities
!   2017-09-05 (B.F.) Change to averaging routines for asymmetric boxes for new datx type
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!   2018-06-02 (B.F.) Add parameter rho_til and compute and save averages of it
!   2018-06-04 (B.F.) Reinsert routines for azimuthal means
!   2018-06-12 (B.F. Uppsala) Call routines to compute spherical wave amplitudes of mass flux
!   2018-06-21 (B.F. Kiel) Add mean of frho**2
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_mean_module
use rhd_dat_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
integer,                   intent(in)   :: dim
real,                      intent(in)   :: AreaFraction
logical,                   intent(in)   :: OutVar_flag, OutFlx_flag, OutRadialC_flag
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: P, T, s, &
                                           Gamma_1, Gamma_3, dTdei, dPdei, &
                                           phic_grav
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: phib_grav
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3), &
                           intent(in)   :: rho_til, v1_til, v2_til, v3_til, &
                                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_rhoei, fep
type(datx_type), optional, intent(inout):: datx, datr
type(datc_type), optional, intent(inout):: datc
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, iquc, m, n, di1
real                                    :: AreaFraction0
real   , dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: weight
real   , dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: x123weight, vr, va, qr
real   , dimension(model%m1:model%n1+1, &
                   model%m2:model%n2  , &
                   model%m3:model%n3   ):: qrb
integer, dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: nbindex
integer, dimension(6, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: nb6index
integer, dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: cindex
integer, dimension(:,:)  , allocatable  :: ccindex
integer, dimension(:,:,:), allocatable  :: bindex
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Compute averages ---
!
if (merge((datx%na > 0), .false., present(datx))) then
  ! === Average over horizontal planes =============================================================
  !
  if (dim == 1) then
    AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                  ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  else if (dim == 2) then
    AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                  ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  else ! (dim == 3)
    AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                  ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
  endif
  !
  ! --- Compute weight array ---
  call rhd_mean_weight(model, AreaFraction0, weight)
  !
  if (OutVar_flag) then
    ! --- Compute individual averages: volume quantities ---
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)            , datx%d(m1:n1,datx%i_rho)     , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_rho2)    , weight)
    call rhd_mean_x23mean(model%v1( m1:n1,:,:)            , datx%d(m1:n1,datx%i_v1)      , weight)
    call rhd_mean_x23mean(model%v2( m1:n1,:,:)            , datx%d(m1:n1,datx%i_v2)      , weight)
    call rhd_mean_x23mean(model%v3( m1:n1,:,:)            , datx%d(m1:n1,datx%i_v3)      , weight)
    call rhd_mean_x23mean(model%v1( m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_v1v1)    , weight)
    call rhd_mean_x23mean(model%v2( m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_v2v2)    , weight)
    call rhd_mean_x23mean(model%v3( m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_v3v3)    , weight)
    call rhd_mean_x23mean(model%v1( m1:n1,:,:)**3         , datx%d(m1:n1,datx%i_v1v1v1)  , weight)
    call rhd_mean_x23mean(model%v1( m1:n1,:,:)**4         , datx%d(m1:n1,datx%i_v1v1v1v1), weight)
    call rhd_mean_x23mean(sign(0.5,model%v1(m1:n1,:,:))+0.5 &
                                                          , datx%d(m1:n1,datx%i_v1upfrac), weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v1( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhov1)   , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v2( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhov2)   , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v3( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhov3)   , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v1( m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_rhov1v1) , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v2( m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_rhov2v2) , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v3( m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_rhov3v3) , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v1( m1:n1,:,:)* &
                          model%v2( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhov1v2) , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v1( m1:n1,:,:)* &
                          model%v3( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhov1v3) , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%v2( m1:n1,:,:)* &
                          model%v3( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhov2v3) , weight)
    call rhd_mean_x23mean(model%ei( m1:n1,:,:)            , datx%d(m1:n1,datx%i_ei)      , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          model%ei( m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhoei)   , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)*0.5* &
                          (model%v1(m1:n1,:,:)**2 + &
                           model%v2(m1:n1,:,:)**2 + &
                           model%v3(m1:n1,:,:)**2  )      , datx%d(m1:n1,datx%i_rhoek)   , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          phic_grav(m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhoeg)   , weight)
    call rhd_mean_x23mean(P(m1:n1,:,:)                    , datx%d(m1:n1,datx%i_p)       , weight)
    call rhd_mean_x23mean(P(m1:n1,:,:)**2                 , datx%d(m1:n1,datx%i_p2)      , weight)
    ! --- Temperature averages are usually computed by the radiation-transport solver. If not... ---
    if ((par%OpaFile == '') .or. (par%A%radScheme == radScheme_None)) then
      call rhd_mean_x23mean(T(m1:n1,:,:)                  , datx%d(m1:n1,datx%i_T)       , weight)
      call rhd_mean_x23mean(T(m1:n1,:,:)**2               , datx%d(m1:n1,datx%i_T2)      , weight)
      call rhd_mean_x23mean(T(m1:n1,:,:)**4               , datx%d(m1:n1,datx%i_T4)      , weight)
    endif
    call rhd_mean_x23mean(s(m1:n1,:,:)                    , datx%d(m1:n1,datx%i_s)       , weight)
    call rhd_mean_x23mean(s(m1:n1,:,:)**2                 , datx%d(m1:n1,datx%i_s2)      , weight)
    call rhd_mean_x23mean(model%rho(m1:n1,:,:)* &
                          s(        m1:n1,:,:)            , datx%d(m1:n1,datx%i_rhos)    , weight)
    call rhd_mean_x23mean(Gamma_1(m1:n1,:,:)              , datx%d(m1:n1,datx%i_Gamma1)  , weight)
    call rhd_mean_x23mean(Gamma_3(m1:n1,:,:)              , datx%d(m1:n1,datx%i_Gamma3)  , weight)
    call rhd_mean_x23mean(1.0/((Gamma_1(m1:n1,:,:)* &
                                P(m1:n1,:,:)*dTdei(m1:n1,:,:))/ &
                               (T(m1:n1,:,:)*dPdei(m1:n1,:,:))- &
                               (Gamma_3(m1:n1,:,:)-1.0)) &
                                                          , datx%d(m1:n1,datx%i_delta)   , weight)
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional densities ---
    if ((model%nquc > 0) .and. (datx%i_quc > 0)) then
      do iquc=1,prop%nquc
        call rhd_mean_x23mean(model%quc(m1:n1,:,:,iquc)   , &
                                                        datx%d(m1:n1,datx%i_quc+(iquc-1)), weight)
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
    if ((model%nBc > 0) .and. (datx%i_Bc1 > 0)) then
      call rhd_mean_x23mean(model%Bc1(m1:n1,:,:)          , datx%d(m1:n1,datx%i_Bc1)     , weight)
      call rhd_mean_x23mean(model%Bc2(m1:n1,:,:)          , datx%d(m1:n1,datx%i_Bc2)     , weight)
      call rhd_mean_x23mean(model%Bc3(m1:n1,:,:)          , datx%d(m1:n1,datx%i_Bc3)     , weight)
      call rhd_mean_x23mean(abs(model%Bc1(m1:n1,:,:))     , datx%d(m1:n1,datx%i_Bc1abs)  , weight)
      call rhd_mean_x23mean(abs(model%Bc2(m1:n1,:,:))     , datx%d(m1:n1,datx%i_Bc2abs)  , weight)
      call rhd_mean_x23mean(abs(model%Bc3(m1:n1,:,:))     , datx%d(m1:n1,datx%i_Bc3abs)  , weight)
      call rhd_mean_x23mean(model%Bc1(m1:n1,:,:)**2       , datx%d(m1:n1,datx%i_Bc12)    , weight)
      call rhd_mean_x23mean(model%Bc2(m1:n1,:,:)**2       , datx%d(m1:n1,datx%i_Bc22)    , weight)
      call rhd_mean_x23mean(model%Bc3(m1:n1,:,:)**2       , datx%d(m1:n1,datx%i_Bc32)    , weight)
    call rhd_mean_x23mean(sqrt(model%Bc2(m1:n1,:,:)**2 + &
                               model%Bc3(m1:n1,:,:)**2  ) , datx%d(m1:n1,datx%i_Bh)      , weight)
    call rhd_mean_x23mean(model%Bc2(m1:n1,:,:)**2 + &
                          model%Bc3(m1:n1,:,:)**2         , datx%d(m1:n1,datx%i_Bhrms)   , weight)
    endif
  endif ! (OutVar_flag)
  !
  if (OutFlx_flag) then
    ! --- Check if fluxes at right boundary should be considered ---
    if (n1 < wg%hyd%n(dim)) then
      di1=0
    else
      di1=1
    endif
    !
    ! --- Compute individual averages: fluxes ---
    call rhd_mean_x23mean(rho_til(    m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_rhotil), weight)
    call rhd_mean_x23mean(flux_rho(   m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_frho)  , weight)
    call rhd_mean_x23mean(flux_rho(   m1:n1+di1,:,:)**2   , datx%d(m1:n1+di1,datx%i_frhofrho), &
                                                                                             weight)
    call rhd_mean_x23mean(flux_rhov1( m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_frhov1), weight)
    call rhd_mean_x23mean(flux_rhov2( m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_frhov2), weight)
    call rhd_mean_x23mean(flux_rhov3( m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_frhov3), weight)
    call rhd_mean_x23mean(flux_rhoei( m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_feip)  , weight)
    call rhd_mean_x23mean(0.5*(v1_til(m1:n1+di1,:,:)**2 + &
                               v2_til(m1:n1+di1,:,:)**2 + &
                               v3_til(m1:n1+di1,:,:)**2  ) * &
                          flux_rho(   m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_fek)   , weight)
    call rhd_mean_x23mean(-phib_grav( m1:n1+di1,:,:) * &
                          flux_rho(   m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_feg)   , weight)
    call rhd_mean_x23mean(fep(        m1:n1+di1,:,:)      , datx%d(m1:n1+di1,datx%i_fep)   , weight)
  endif ! (OutFlx_flag)
endif ! (merge((datx%na > 0), .false., present(datx)))
!
if (merge(((datr%na > 0) .and. (datr%i_rhoei > 0)), .false., present(datr)) .and. &
    (par%grav_mode == 'central')                                                   ) then
  ! === Average over spheres =======================================================================
  !
  ! --- Make abbreviations ---
  m=datr%m
  n=datr%n
  !
  ! --- Allocate boundary-index array ---
  allocate(bindex(-(n-m+2):(n-m+2),m2:n2,m3:n3))
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(model, prop%length, weight, x123weight)
  !
  ! --- Compute index arrays ---
  ! call rhd_mean_rindex(model, datr%d1(:,datr%i_xb), nbindex, bindex, cindex)
  call rhd_mean_a2rindex(m1,n1, m2,n2, m3,n3, &
                         model%nghost1, wg%hyd%m(dim), wg%hyd%n(dim), &
                         model%xc1, model%xc2(m2:n2), model%xc3(m3:n3), &
                         datr%d1(:,datr%i_xb), nb6index, bindex, cindex)
  !
  if (OutVar_flag .and. OutRadialC_flag) then
    ! --- Compute individual averages: volume quantities ---
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         1.0+0.0*model%rho(m1:n1,:,:)              , datr%d(m:n,datr%i_relvol), &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,:,:)                      , datr%d(m:n,datr%i_rho)   , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,:,:)**2                   , datr%d(m:n,datr%i_rho2)  , &
                         cindex, x123weight)
    !
    call rhd_mean_VecProj(model, model%v1(m1:n1,:,:), model%v2(m1:n1,:,:), model%v3(m1:n1,:,:), &
                          vr)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         vr                                        , datr%d(m:n,datr%i_v3)    , &
                         cindex, x123weight)
    !
    qr=vr*model%rho(m1:n1,:,:)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_rhov3) , &
                         cindex, x123weight)
    !
    qr=vr**2
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v3v3)   , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr*model%rho(m1:n1,:,:)                   , datr%d(m:n,datr%i_rhov3v3), &
                         cindex, x123weight)
    !
    qr=max(model%v1(m1:n1,:,:)**2 + model%v2(m1:n1,:,:)**2 + model%v3(m1:n1,:,:)**2 - qr, 0.0)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v1v1)   , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr*model%rho(m1:n1,:,:)                   , datr%d(m:n,datr%i_rhov1v1), &
                         cindex, x123weight)
    !
    qr=vr**3
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                        , datr%d(m:n,datr%i_v1v1v1) , &
                         cindex, x123weight)
    qr=vr**4
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                       , datr%d(m:n,datr%i_v1v1v1v1), &
                         cindex, x123weight)
    !
    qr=sign(0.5,vr)+0.5
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qr                                       , datr%d(m:n,datr%i_v1upfrac), &
                         cindex, x123weight)
    !
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%ei(m1:n1,:,:)                       , datr%d(m:n,datr%i_ei)    , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,:,:)* &
                         model%ei(m1:n1,:,:)                       , datr%d(m:n,datr%i_rhoei) , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,:,:)*0.5* &
                         (model%v1(m1:n1,:,:)**2+ &
                          model%v2(m1:n1,:,:)**2+ &
                          model%v3(m1:n1,:,:)**2)                  , datr%d(m:n,datr%i_rhoek) , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,:,:)*phic_grav(m1:n1,:,:) , datr%d(m:n,datr%i_rhoeg) , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         P(m1:n1,:,:)                              , datr%d(m:n,datr%i_p)     , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         P(m1:n1,:,:)**2                           , datr%d(m:n,datr%i_p2)    , &
                         cindex, x123weight)
    ! --- Temperature averages are usually computed by the radiation-transport solver. If not... ---
    if ((par%OpaFile == '') .or. (par%A%radScheme == radScheme_None)) then
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         T(m1:n1,:,:)                              , datr%d(m:n,datr%i_T)     , &
                         cindex, x123weight)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         T(m1:n1,:,:)**2                           , datr%d(m:n,datr%i_T2)    , &
                         cindex, x123weight)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         T(m1:n1,:,:)**4                           , datr%d(m:n,datr%i_T4)    , &
                         cindex, x123weight)
    endif
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         s(m1:n1,:,:)                              , datr%d(m:n,datr%i_s)     , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         s(m1:n1,:,:)**2                           , datr%d(m:n,datr%i_s2)    , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         model%rho(m1:n1,:,:)* &
                         s(        m1:n1,:,:)                      , datr%d(m:n,datr%i_rhos)  , &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         Gamma_1(m1:n1,:,:)                        , datr%d(m:n,datr%i_Gamma1), &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         Gamma_3(m1:n1,:,:)                        , datr%d(m:n,datr%i_Gamma3), &
                         cindex, x123weight)
    call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         1.0/((Gamma_1(m1:n1,:,:)*P(m1:n1,:,:)* &
                               dTdei(m1:n1,:,:))/ &
                              (T(m1:n1,:,:)*dPdei(m1:n1,:,:))- &
                              (Gamma_3(m1:n1,:,:)-1.0))            , datr%d(m:n,datr%i_delta) , &
                         cindex, x123weight)
    !
    if (dim == 1) then
      ! --- Compute and average moment of inertia ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            vr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc1(i1)**2+model%xc2(i2)**2)
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           vr                                      , datr%d(m:n,datr%i_momIne3), &
                           cindex, x123weight)
      !
      ! --- Compute and average angular momentum ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            vr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc1(i1)*model%v2(i1,i2,i3)- &
                                                model%xc2(i2)*model%v1(i1,i2,i3)   )
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           vr                                      , datr%d(m:n,datr%i_AngMom3), &
                           cindex, x123weight)
    else if (dim == 2) then
      ! --- Compute and average moment of inertia (transpose 2 -> 1) ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            vr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc1(i1)**2+model%xc2(i2)**2)
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           vr                                      , datr%d(m:n,datr%i_momIne3), &
                           cindex, x123weight)
      !
      ! --- Compute and average angular momentum ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            vr(i1,i2,i3)=-model%rho(i1,i2,i3) * (model%xc1(i1)*model%v2(i1,i2,i3)- &
                                                 model%xc2(i2)*model%v1(i1,i2,i3)   )
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           vr                                      , datr%d(m:n,datr%i_AngMom3), &
                           cindex, x123weight)
    else if (dim == 3) then
      ! --- Compute and average moment of inertia (transpose 3 -> 1) ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            vr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc3(i3)**2+model%xc2(i2)**2)
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           vr                                      , datr%d(m:n,datr%i_MomIne3), &
                           cindex, x123weight)
      !
      ! --- Compute and average angular momentum ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            vr(i1,i2,i3)=model%rho(i1,i2,i3) * (model%xc3(i3)*model%v2(i1,i2,i3)- &
                                                model%xc2(i2)*model%v3(i1,i2,i3)   )
          end do ! i1
        end do ! i2
      end do ! i3
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           vr                                      , datr%d(m:n,datr%i_AngMom3), &
                           cindex, x123weight)
    endif ! (dim)
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional densities ---
    if ((model%nquc > 0) .and. (datr%i_quc > 0)) then
      do iquc=1,prop%nquc
        call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                             model%quc(m1:n1,:,:,iquc)        , datr%d(m:n,datr%i_quc+(iquc-1)), &
                             cindex, x123weight)
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
    !
    if ((model%nBc > 0) .and. (datr%i_Bc1 > 0)) then
      call rhd_mean_VecProj(model, model%Bc1(m1:n1,:,:), &
                                   model%Bc2(m1:n1,:,:), &
                                   model%Bc3(m1:n1,:,:), qr)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc3)    , &
                           cindex, x123weight)
      !
      qr=abs(qr)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc3abs) , &
                           cindex, x123weight)
      !
      qr=qr**2
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc32)   , &
                           cindex, x123weight)
      !
      qr=max(model%Bc1(m1:n1,:,:)**2 + &
             model%Bc2(m1:n1,:,:)**2 + &
             model%Bc3(m1:n1,:,:)**2 - qr, &
             0.0)
      call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                           qr                                      , datr%d(m:n,datr%i_Bc12)   , &
                           cindex, x123weight)
    endif ! ((model%nBc > 0) .and. (datr%i_Bc1 > 0))
  endif ! (OutVar_flag .and. OutRadialC_flag)
  !
  if (OutFlx_flag) then
    ! --- Compute individual averages: boundary-centered density, fluxes, waves ---
    !
    ! --- Compute surface areas ---
    call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         0.0*flux_rho(m1:n1+1,:,:)                   , datr%d(:,datr%i_relsurf), &
                         nb6index, bindex, weight, unity=.true.)
    !
    if (datr%i_runiflux > 0) then
      call rhd_mean_VecbProjUnity(model, qrb)
      call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         qrb                                         , datr%d(:,datr%i_runiflux), &
                         nb6index, bindex, weight)
    endif
    !
    ! --- Average boundary-centered density ---
    call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         rho_til(m1:n1+1,:,:)                        , datr%d(:,datr%i_rhotil) , &
                         nb6index, bindex, weight)
    !
    ! --- Average fluxes ---
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         flux_rho(m1:n1+1,:,:)                       , datr%d(:,datr%i_frho)   , &
                         nb6index, bindex, weight)
    call rhd_mean_rb23absmean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         flux_rho(m1:n1+1,:,:)**2                    , datr%d(:,datr%i_frhofrho), &
                         nb6index, bindex, weight)
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         flux_rhov1(m1:n1+1,:,:)                     , datr%d(:,datr%i_frhov3) , &
                         nb6index, bindex, weight)
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         flux_rhoei(m1:n1+1,:,:)                     , datr%d(:,datr%i_feip)   , &
                         nb6index, bindex, weight)
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         0.5*(v1_til(m1:n1+1,:,:)**2+ &
                              v2_til(m1:n1+1,:,:)**2+ &
                              v3_til(m1:n1+1,:,:)**2)*flux_rho(m1:n1+1,:,:) &
                                                                     , datr%d(:,datr%i_fek)    , &
                         nb6index, bindex, weight)
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         -phib_grav(m1:n1+1,:,:)* &
                         flux_rho(m1:n1+1,:,:)                       , datr%d(:,datr%i_feg)    , &
                         nb6index, bindex, weight)
    call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         fep(m1:n1+1,:,:)                            , datr%d(:,datr%i_fep)    , &
                         nb6index, bindex, weight)
    !
    ! --- Compute wave amplitudes ---
    if ((datr%nWave > 0) .and. (datr%i_frhowave > 0)) then
      call rhd_mean_rb23shmean(model, m1,n1, m2,n2, m3,n3, &
                               m,n, datr%na, datr%nWave, datr%i_frhowave, dim, &
                               flux_rho(m1:n1+1,:,:), datr%d, &
                               nb6index, bindex, weight)
    endif
    !
  endif ! (OutFlx_flag)
  !
  ! --- Deallocate boundary-index array ---
  deallocate(bindex)
  !
endif ! ((present(datr) .and. ...)
!
if (merge(associated(datc%d), .false., present(datc)) .and. &
    (par%grav_mode == 'central') .and. (dim == 3) .and. (OutVar_flag)) then
  ! === Average over cylinders (azimuthally) =======================================================
  !
  ! --- Allocate column-index array ---
  allocate(ccindex(m2:n2,m3:n3))
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(model, prop%length, weight, x123weight)
  !
  ! --- Compute index array ---
  call rhd_mean_c1Index(model, datc%d1(:,datc%i_xb), ccindex)
  !
  ! --- Compute individual averages: volume quantities ---
  !
  ! --- Relative volume (to get a proper weight for each radius bin) ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_relvol, &
         1.0+0.0*model%rho(m1:n1,:,:))
  !
  ! --- Basic thermodynamic quantities ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rho, &
         model%rho(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_ei, &
         model%ei(m1:n1,:,:))
  !
  ! --- Additional thermodynamic quantities ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhoei, &
         model%rho(m1:n1,:,:)*model%ei(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_T, &
         T(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_P, &
         P(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_Gamma1, &
         Gamma_1(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_Gamma3, &
         Gamma_3(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_delta, &
         1.0/((Gamma_1(m1:n1,:,:)*P(m1:n1,:,:)*dTdei(m1:n1,:,:))/ &
              (T(m1:n1,:,:)*dPdei(m1:n1,:,:))-(Gamma_3(m1:n1,:,:)-1.0)))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_s, &
         s(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_ss, &
         s(m1:n1,:,:)**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhos, &
         model%rho(m1:n1,:,:)*s(m1:n1,:,:))
  !
  ! --- Velocities ---
  call rhd_mean_c1VecProj(model, model%v2(m1:n1,:,:),  model%v3(m1:n1,:,:), vr, va)
  !
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v1, &
         vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v1v1, &
         vr**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1, &
         model%rho(m1:n1,:,:)*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1v1, &
         model%rho(m1:n1,:,:)*vr**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1v2, &
         model%rho(m1:n1,:,:)*vr*va)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov1v3, &
         model%rho(m1:n1,:,:)*vr*model%v1(m1:n1,:,:))
  !
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v2, &
         va)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v2v2, &
         va**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov2, &
         model%rho(m1:n1,:,:)*va)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov2v2, &
         model%rho(m1:n1,:,:)*va**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov2v3, &
         model%rho(m1:n1,:,:)*va*model%v1(m1:n1,:,:))
  !
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v3, &
         model%v1(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_v3v3, &
         model%v1(m1:n1,:,:)**2)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov3, &
         model%rho(m1:n1,:,:)*model%v1(m1:n1,:,:))
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_rhov3v3, &
         model%rho(m1:n1,:,:)*model%v1(m1:n1,:,:)**2)
  !
  ! --- Energy fluxes ---
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fep1, &
         P(m1:n1,:,:)*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fep3, &
         P(m1:n1,:,:)*model%v1(m1:n1,:,:))
  qr=model%rho(m1:n1,:,:)*model%ei(m1:n1,:,:)+P(m1:n1,:,:)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_feip1, &
         qr*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_feip3, &
         qr*model%v1(m1:n1,:,:))
  qr=0.5*model%rho(m1:n1,:,:) * (model%v1(m1:n1,:,:)**2 + &
                                 model%v2(m1:n1,:,:)**2 + &
                                 model%v3(m1:n1,:,:)**2    )
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fek1, &
         qr*vr)
  call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
         ccindex, x123weight, datc%d, datc%i_fek3, &
         qr*model%v1(m1:n1,:,:))
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Additional densities and corresponding fluxes (only computed with quc*v) ---
  if ((model%nquc > 0) .and. (datc%i_quc > 0)) then
    do iquc=1,prop%nquc
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_quc-1+iquc, &
             model%quc(m1:n1,:,:,iquc))
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_fquc1-1+iquc, &
             model%quc(m1:n1,:,:,iquc)*vr)
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_fquc2-1+iquc, &
             model%quc(m1:n1,:,:,iquc)*va)
      call rhd_mean_ccamean(m1,n1, m2,n2, m3,n3, datc%m1,datc%n1,datc%m2,datc%n2,datc%na, &
             ccindex, x123weight, datc%d, datc%i_fquc3-1+iquc, &
             model%quc(m1:n1,:,:,iquc)*model%v1(m1:n1,:,:))
    end do ! iquc
  endif
!#
# endif
!#      /* rhd_box_quc01 */
  !
  ! --- Deallocate column-index array ---
  deallocate(ccindex)
  !
endif ! ((present(datc)) .and. ...)
!
end subroutine rhd_hyd_Roe1Doutput


!----------*************----------------------------------------------------------------------------
subroutine rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         dxb1_avg, dxb1, &
                         v1, v2, v3, s, T, &
                         v1_til, rho_til, cs_til, wrho, &
                         flux_rhoei_sDiff)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_sDiff ('rhd_hydrodynamics_s_Diffusion')
!
! PURPOSE:
!   Compute 1D energy flux down an entropy gradient with state reconstruction.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
!                      reconstruction_order, reconstruction_type, &
!                      SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                      dxb1_avg, dxb1, &
!                      v1, v2, v3, s, T, &
!                      v1_til, rho_til, cs_til, wrho, &
!                      flux_rhoei_sDiff)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be done
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing,
!                   value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real vector, cell-boundary step size, 1. coordinate
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   s:            ('s') real 3D array, entropy on model grid
!   T:            ('Temperature') real 3D array, temperature on model grid
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe-averaged velocity 1 at cell boundary
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe-averaged sound speed at cell boundary
!   wrho:         ('weight_rho') real 3D array, Roe weight from square root of density
!
! OUTPUT:
!   flux_rhoei_sDiff: ('flux_rho_energy_internal_s_Diffusion') real 3D array, diffusive
!                     internal energy flux
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The entropy is reconstructed (from the entropy-state vector).
!   From the entropy boundary values, the diffusive energy flux is calculated.
!
! MODIFICATION HISTORY:
!   2012-04-03 (B.F. Lyon) Written
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-10-05 (B.F.) Limiting term for vD added
!   2012-10-08 (B.F.) Bugfix in limiting term for vD (now for vD_til)
!   2012-10-15 (B.F.) Bugfix for non-equidistant-grid case: corrected index range for dxb1
!   2012-10-16 (B.F.) C_hydsDiffVelo -> C_hydsDiffVelo**2
!   2013-02-22 (B.F.) Replace sqrt_rho with wrho
!   2014-12-03 (B.F.) Add parameter dxb1; remove parameter xb1
!   2018-05-09 (B.F.) Add dimension specifications in recontruction-routine calls
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_rec_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1      :n1+2, &
                m2        :n2  , &
                m3        :n3   ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2        :n2        ,&
                m3        :n3         ), &
                           intent(in)   :: v1, v2, v3, s, T
real,                      intent(in)   :: dxb1_avg
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(m1        :n1+1,&
                m2        :n2        ,&
                m3        :n3         ), &
                           intent(in)   :: v1_til, rho_til, cs_til, wrho
real, dimension(m1        :n1+1,&
                m2        :n2        ,&
                m3        :n3         ), &
                           intent(out)  :: flux_rhoei_sDiff
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: T_til, vD_til, vDm, vDp
real, dimension(m1-1      :n1+1, &
                m2        :n2  , &
                m3        :n3     )     :: vD, s_l, s_r
!---------------------------------------------------------------------------------------------------
!
! --- Reconstruct entropy (state) and get boundary values ---
if (dxb1_avg < 0.0) then
  ! --- Spatially varying grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecVec')
!#
# endif
!#
  ! --- Reconstruct ---
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   s, dxb1, s_l, s_r)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecVec')
!#
# endif
!#
else
  ! --- Equidistant grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecVecEq')
!#
# endif
!#
  ! --- Reconstruct ---
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     s, s_l, s_r)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecVecEq')
!#
# endif
!#
endif
!
! \label{f90:c_hydsdiffvelo__use1}
! --- Compute diffusive velocity at cell centers ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1-1,n1+1
      vD(i1,i2,i3)=sqrt(max(par%C_hydsDiffVelo**2,1.0)* v1(i1,i2,i3)**2 + &
                            par%C_hydsDiffVelo**2     *(v2(i1,i2,i3)**2 + &
                                                        v3(i1,i2,i3)**2) )
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Compute diffusive energy flux ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Compute Roe averages by weighting with sqrt(rho) ---
      T_til =wrho(i1,i2,i3)*(T( i1-1,i2,i3)-T( i1,i2,i3)) + T( i1,i2,i3)
      vD_til=wrho(i1,i2,i3)*(vD(i1-1,i2,i3)-vD(i1,i2,i3)) + vD(i1,i2,i3)
      vD_til=cs_til(i1,i2,i3) * (1.0-0.25*min(vD_til/cs_til(i1,i2,i3)-2.0,0.0)**2)
      vD_til=max(vD_til-abs(v1_til(i1,i2,i3)), 0.0)
      !
      ! --- Compute energy flux ---
      flux_rhoei_sDiff(i1,i2,i3)=(s_r(i1-1,i2,i3)-s_l(i1,i2,i3)) * &
                                 T_til*rho_til(i1,i2,i3)*vD_til
      !
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_hyd_sDiff


!----------*****************------------------------------------------------------------------------
subroutine rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                             reconstruction_order, reconstruction_type, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             dtime, dxb1_avg, dxb1, i_up, &
                             lambda_til, Diff_flag, vD_til, &
                             a, &
                             flux_a)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_FluxState ('rhd_hydrodynamics_Flux_State')
!
! PURPOSE:
!   Compute 1D flux from state vector for single quantity.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
!                          reconstruction_order, reconstruction_type, &
!                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                          dtime, dxb1_avg, dxb1, i_up, &
!                          lambda_til, Diff_flag, vD_til, &
!                          a, &
!                          flux_a)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   dtime:        ('delta_time') real, time step
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real, cell-boundary step size, 1. coordinate
!   i_up:         ('index_upwind') 3D real array with indices of upwind direction
!   lambda_til:   ('lambda_tilde') 3D real array with characteristic velocities at cell boundaries
!   Diff_flag:    ('Diffusion_flag') logical indicating that extra diffusion should be added
!   vD_til:       ('velocity_Diffusion_tilde') 3D array with additional diffusion velocity
!   a:            ('alpha') 3D real array with state to reconstruct (in first dimension)
!
! OUTPUT:
!   flux_a:       ('flux_alpha') 3D real array with fluxes, units: [flux_a] = [lambda] * [a]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   dxp0:         ('delta_x_plus_0') real, delta x value in cell
!   ap0:          ('alpha_plus_0') real, state value in cell
!   aL:           ('alpha_Left') real, boundary value on left side
!   aR:           ('alpha_Right') real, boundary value on right side
!   a_0:          ('alpha_0') real, 0th coefficient of parabola
!   delta_a:      ('alpha_0') real, 1st coefficient of parabola (slope)
!   a_2:          ('alpha_2') real, 2nd coefficient of parabola
!   y:            ('y') real, lambda*dtime/dx
!   a_l:          ('alpha_left') real 3D array with reconstructed left boundary values
!   a_r:          ('alpha_right') real 3D array with reconstructed right boundary values
!
! ROUTINES:
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   2012-04-12 (B.F. Lyon) Written
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2013-02-24 (B.F. Lyon) Use new type of loops for upwind values: if-then-else instead of (i_up())
!   2014-12-02 (B.F.) Add parameter dxb1; remove parameter xb1
!   2018-05-09 (B.F.) Add dimension specifications in recontruction-routine calls
!---------------------------------------------------------------------------------------------------
use rhd_rec_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime, dxb1_avg
logical,                   intent(in)   :: Diff_flag, SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1      :n1+2,&
                m2        :n2  ,&
                m3        :n3    ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2        :n2        ,&
                m3        :n3          ), &
                           intent(in)   :: a
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
integer, &
      dimension(m1        :n1+1, &
                m2        :n2  , &
                m3        :n3     ), &
                           intent(in)   :: i_up
real, dimension(m1        :n1+1, &
                m2        :n2  , &
                m3        :n3     ), &
                           intent(in)   :: lambda_til, vD_til
real, dimension(m1        :n1+1,&
                m2        :n2        ,&
                m3        :n3         ), &
                           intent(out)  :: flux_a
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: dxp0, ap0, aL, aR, a_0, delta_a, a_2, y
real, dimension(m1-1      :n1+1, &
                m2        :n2  , &
                m3        :n3     )     :: a_l, a_r
!---------------------------------------------------------------------------------------------------
!
! --- Reconstruct state and get boundary values ---
if (dxb1_avg < 0.0) then
  ! --- Spatially varying grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecVec')
!#
# endif
!#
  ! --- Reconstruct ---
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   a, dxb1, a_l, a_r)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecVec')
!#
# endif
!#
else
  ! --- Equidistant grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecVecEq')
!#
# endif
!#
  ! --- Reconstruct ---
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     a, a_l, a_r)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecVecEq')
!#
# endif
!#
endif
!
! --- Compute upwind flux from cell integral (state) and boundary values ---
do i3=m3,n3
  do i2=m2,n2
    !DIR$ SIMD
    do i1=m1,n1+1
      ! --- Take values from upwind cell ---
!!$      dxp0=dxb1(i_up(i1,i2,i3))
!!$      ap0 =a(   i_up(i1,i2,i3),i2,i3)
!!$      aL  =a_l( i_up(i1,i2,i3),i2,i3)
!!$      aR  =a_r( i_up(i1,i2,i3),i2,i3)
      if (i_up(i1,i2,i3) < i1) then
        dxp0=dxb1(i1-1)
        ap0 =a(   i1-1,i2,i3)
        aL  =a_l( i1-1,i2,i3)
        aR  =a_r( i1-1,i2,i3)
      else
        dxp0=dxb1(i1  )
        ap0 =a(   i1  ,i2,i3)
        aL  =a_l( i1  ,i2,i3)
        aR  =a_r( i1  ,i2,i3)
      endif
      !
      ! --- Compute parabola coefficients ---
      a_0    =1.5*ap0-0.25*(aL+aR)
      delta_a=aR-aL
      a_2    =3.0*(aL+aR-2.0*ap0)
      !
      ! --- Compute flux (PP formula) ---
      y=lambda_til(i1,i2,i3)*dtime/dxp0
      flux_a(i1,i2,i3)=lambda_til(i1,i2,i3) * &
                       (a_0 + &
                        delta_a * sign(0.5,lambda_til(i1,i2,i3)) * (1.0-abs(y)) + &
                        a_2 * (0.25-0.5*abs(y)+y*y*(1.0/3.0)) )
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Add diffusive flux if requested ---
if (Diff_flag) then
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        flux_a(i1,i2,i3)=flux_a(i1,i2,i3) + (a_r(i1-1,i2,i3)-a_l(i1,i2,i3)) * vD_til(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
end subroutine rhd_hyd_FluxState


!----------************-----------------------------------------------------------------------------
subroutine rhd_hyd_Pred(m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order, &
                        wl, rho, P, grav_flag, phic_grav, phib_grav, &
                        P_red, P_avg)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Pred ('rhd_hydrodynamics_Pressure_reduced')
!
! PURPOSE:
!   Compute reduced pressure as gravity correction for Roe solver.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Pred(m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order, &
!                     wl, rho, P, grav_flag, phic_grav, phib_grav, &
!                     P_red, P_avg)
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   rho:          ('rho') real 3D array, density
!   P:            ('pressure') real 3D array, pressure on model grid
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!   phic_grav:    ('phi_central_gravity) real 3D array, gravitational potential, cell centers
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!
! OUTPUT:
!   P_red:        ('Pressure_reduced') real 4D array, hydrostatic pressure correction
!   P_avg:        ('Pressure_average') real 3D array, simple average of pressure at cell boundary
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   rho_mean_m2:  ('rho_mean_minus_2') real, average density at relative boundary index -2
!   rho_mean_m1:  ('rho_mean_minus_1') real, average density at relative boundary index -1
!   rho_mean_p0:  ('rho_mean_plus') real, average density at relative boundary index 0
!   rho_mean_p1:  ('rho_mean_plus_1') real, average density at relative boundary index +1
!   rho_mean_p2:  ('rho_mean_plus_2') real, average density at relative boundary index +2
!   C_max_Predfac: ('Constant_maximum_Pressure_reduction_factor') real parameter (0.99),
!                  1-C_max_Predfac gives the minimum remaing pressure in a cell after
!                  P_red subtraction
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute reduced gravitational potential: Int((rho*dphi/dz)*dz)
!   and linearly averaged cell-boundary pressure.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2007-02-01 (B.F. Lyon) Removal of P_red from alpha_til_3 and alpha_til_6,
!                     New pressure reduction scheme: default6
!   2007-02-12 (B.F.) P_red put back into alpha_til_3 and alpha_til_6
!   2013-02-25 (B.F.) Extracted from rhd_hyd_RoeWave1Dflux and put into own routine
!   2014-10-31 (B.F. Uppsala) P_red and P_avg with one element more at each i1 boundary
!   2015-01-14 (B.F.) Re-arrange cases
!   2015-01-15 (B.F.) New case '11'
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order
logical,                   intent(in)   :: grav_flag
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1  :n1+nghost1,&
                m2          :n2        ,&
                m3          :n3          ), &
                           intent(in)   :: rho, P, phic_grav
real, dimension(m1-nghost1  :n1+nghost1+1,&
                m2          :n2          ,&
                m3          :n3            ), &
                           intent(in)   :: phib_grav
real, dimension(m1-1:n1+2, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: P_red
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: P_avg
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: rho_mean_m2, rho_mean_m1, &
                                           rho_mean_p0, rho_mean_p1, rho_mean_p2, &
                                           P_redm1, P_redp0
!real, dimension(m1-nghost1:n1+nghost1,&
!                m2        :n2        ,&
!                m3        :n3      )    :: vsqr
real, parameter                         :: C_max_Predfac=0.99, &
                                           eps_v2=1.0E-10
!---------------------------------------------------------------------------------------------------
!
if (.not. grav_flag) then
  ! --- No gravity: simply set reduced pressure to zero ---
  P_red=0.0
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1-1,n1+2
        P_avg(i1,i2,i3)=     wl(i1)  * P(i1-1,i2,i3) + &
                        (1.0-wl(i1)) * P(i1  ,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
else ! (grav_flag)
  !if (par%grav_mode == 'central') then
  if (.false.) then
!!$    !write(6,*) 'gc_grav: ', minval(gc_grav), maxval(gc_grav)
!!$    !
!!$    ! --- Compute square of velocity at cell centers ---
!!$    do i3=m3,n3
!!$      do i2=m2,n2
!!$        do i1=m1-nghost1,n1+nghost1
!!$          vsqr=v1(i1,i2,i3)**2 + v2(i1,i2,i3)**2 + v3(i1,i2,i3)**2
!!$        end do ! i1
!!$      end do ! i2
!!$    end do ! i3
!!$    !
!!$    ! --- Compute pressure reduction in cell adjacent to boundary ---
!!$    do i3=m3,n3
!!$      do i2=m2,n2
!!$        do i1=m1-1,n1+2
!!$          P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3) * &
!!$                               (phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3)) * &
!!$                               (Radius(i1  ,i2,i3)*gc_grav(i1  ,i2,i3)) / &
!!$                               (Radius(i1  ,i2,i3)*gc_grav(i1  ,i2,i3) + &
!!$                                vsqr(i1  ,i2,i3) + &
!!$                                eps_v2)
!!$          P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3) * &
!!$                               (phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3)) * &
!!$                               (Radius(i1-1,i2,i3)*gc_grav(i1-1,i2,i3)) / &
!!$                               (Radius(i1-1,i2,i3)*gc_grav(i1-1,i2,i3) + &
!!$                                vsqr(  i1-1,i2,i3) + &
!!$                                eps_v2)
!!$          !
!!$  !        !P_red(i1  , 0,i2,i3)=min(P_red(i1  , 0,i2,i3), C_max_Predfac*P(i1  ,i2,i3))
!!$  !        P_red(i1  , 0,i2,i3)=max(P_red(i1  , 0,i2,i3), -C_max_Predfac*P(i1  ,i2,i3))
!!$  !        P_red(i1  ,-1,i2,i3)=min(P_red(i1  ,-1,i2,i3),  C_max_Predfac*P(i1-1,i2,i3))
!!$          !
!!$          P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) + &
!!$                          (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1, 0,i2,i3))
!!$        end do ! i1
!!$      end do ! i2
!!$    end do ! i3
!!$    !
!!$    ! --- Compute pressure reduction in cells further away ---
!!$    if (reconstruction_order >= 1) then
!!$      do i3=m3,n3
!!$        do i2=m2,n2
!!$          do i1=m1,n1+1
!!$            rho_mean_p1=(1.0-wl(i1+1))*rho(i1  ,i2,i3) + &
!!$                             wl(i1+1) *rho(i1+1,i2,i3)
!!$            rho_mean_m1=(1.0-wl(i1-1))*rho(i1-2,i2,i3) + &
!!$                             wl(i1-1) *rho(i1-1,i2,i3)
!!$            P_red(i1  , 1,i2,i3)=P_red(i1  , 0,i2,i3) + &
!!$                                   rho_mean_p1 * &
!!$                                   (phic_grav(i1+1,i2,i3)-phic_grav(i1  ,i2,i3)) * &
!!$                                   (Radius(i1  ,i2,i3)*gc_grav(i1  ,i2,i3) + &
!!$                                    Radius(i1+1,i2,i3)*gc_grav(i1+1,i2,i3)  ) / &
!!$                                   (Radius(i1  ,i2,i3)*gc_grav(i1  ,i2,i3) + &
!!$                                    Radius(i1+1,i2,i3)*gc_grav(i1+1,i2,i3) + &
!!$                                    vsqr(  i1  ,i2,i3) + &
!!$                                    vsqr(  i1+1,i2,i3) + &
!!$                                    eps_v2)
!!$            P_red(i1  ,-2,i2,i3)=P_red(i1  ,-1,i2,i3) + &
!!$                                   rho_mean_m1 * &
!!$                                   (phic_grav(i1-2,i2,i3)-phic_grav(i1-1,i2,i3)) * &
!!$                                   (Radius(i1-2,i2,i3)*gc_grav(i1-2,i2,i3) + &
!!$                                    Radius(i1-1,i2,i3)*gc_grav(i1-1,i2,i3)  ) / &
!!$                                   (Radius(i1-2,i2,i3)*gc_grav(i1-2,i2,i3) + &
!!$                                    Radius(i1-1,i2,i3)*gc_grav(i1-1,i2,i3) + &
!!$                                    vsqr(  i1-2,i2,i3) + &
!!$                                    vsqr(  i1-1,i2,i3) + &
!!$                                    eps_v2)
!!$          end do ! i1
!!$        end do ! i2
!!$      end do ! i3
!!$    endif
!!$    !
!!$    ! --- Compute pressure reduction in cells even further away ---
!!$    if (reconstruction_order >= 2) then
!!$      do i3=m3,n3
!!$        do i2=m2,n2
!!$          do i1=m1,n1+1
!!$            rho_mean_p2=(1.0-wl(i1+2))*rho(i1+1,i2,i3) + &
!!$                             wl(i1+2) *rho(i1+2,i2,i3)
!!$            rho_mean_m2=(1.0-wl(i1-2))*rho(i1-3,i2,i3) + &
!!$                             wl(i1-2) *rho(i1-2,i2,i3)
!!$            P_red(i1  , 2,i2,i3)=P_red(i1  , 1,i2,i3) + &
!!$                                   rho_mean_p2 * &
!!$                                   (phic_grav(i1+2,i2,i3)-phic_grav(i1+1,i2,i3)) * &
!!$                                   (Radius(i1+1,i2,i3)*gc_grav(i1+1,i2,i3) + &
!!$                                    Radius(i1+2,i2,i3)*gc_grav(i1+2,i2,i3)  ) / &
!!$                                   (Radius(i1+1,i2,i3)*gc_grav(i1+1,i2,i3) + &
!!$                                    Radius(i1+2,i2,i3)*gc_grav(i1+2,i2,i3) + &
!!$                                    vsqr(  i1+1,i2,i3) + &
!!$                                    vsqr(  i1+2,i2,i3) + &
!!$                                    eps_v2)
!!$
!!$            P_red(i1  ,-3,i2,i3)=P_red(i1  ,-2,i2,i3) + &
!!$                                   rho_mean_m2 * &
!!$                                   (phic_grav(i1-3,i2,i3)-phic_grav(i1-2,i2,i3)) * &
!!$                                   (Radius(i1-3,i2,i3)*gc_grav(i1-3,i2,i3) + &
!!$                                    Radius(i1-2,i2,i3)*gc_grav(i1-2,i2,i3)  ) / &
!!$                                   (Radius(i1-3,i2,i3)*gc_grav(i1-3,i2,i3) + &
!!$                                    Radius(i1-2,i2,i3)*gc_grav(i1-2,i2,i3) + &
!!$                                    vsqr(  i1-3,i2,i3) + &
!!$                                    vsqr(  i1-2,i2,i3) + &
!!$                                    eps_v2)
!!$          end do ! i1
!!$        end do ! i2
!!$      end do ! i3
!!$    endif
  else
!#
# if (rhd_hyd_gravcorr_p01 == 0)
!#                               /* 'off0' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1  , 0,i2,i3)=0.0
          P_red(i1  ,-1,i2,i3)=0.0
          !
          P_avg(i1,i2,i3)=     wl(i1)  * P(i1-1,i2,i3) + &
                          (1.0-wl(i1)) * P(i1  ,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 1)
!#                                 /* 'default' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          rho_mean_p0=(1.0-wl(i1  ))*rho(i1-1,i2,i3) + &
                           wl(i1  ) *rho(i1  ,i2,i3)
          P_red(i1  , 0,i2,i3)=rho_mean_p0 * &
                             (phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=rho_mean_p0 * &
                             (phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
          !
          P_avg(i1,i2,i3)=     wl(i1)  * P(i1-1,i2,i3) + &
                          (1.0-wl(i1)) * P(i1  ,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 2)
!#                                 /* 'default2' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
          !
          P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                          (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 3)
!#                                 /* 'default3' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
          !
          P_red(i1  , 0,i2,i3)=min(max(P_red(i1  , 0,i2,i3), &
                                       min(P(i1  ,i2,i3)-P(i1-1,i2,i3), 0.0)), &
                                   0.0) + &
                               max(min(P_red(i1  , 0,i2,i3), &
                                       max(P(i1  ,i2,i3)-P(i1-1,i2,i3), 0.0)), &
                                   0.0)
          P_red(i1  ,-1,i2,i3)=min(max(P_red(i1  ,-1,i2,i3), &
                                       min(P(i1-1,i2,i3)-P(i1  ,i2,i3), 0.0)), &
                                   0.0) + &
                               max(min(P_red(i1  ,-1,i2,i3), &
                                       max(P(i1-1,i2,i3)-P(i1  ,i2,i3), 0.0)), &
                                   0.0)
          !
          P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                          (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 4)
!#                                 /* 'default4' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
          !
          P_avg(i1,i2,i3)=(     wl(i1) /rho(i1-1,i2,i3) * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                           (1.0-wl(i1))/rho(i1  ,i2,i3) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))  > / &
                          (wl(i1) /rho(i1-1,i2,i3) +  (1.0-wl(i1))/rho(i1  ,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 5)
!#                                 /* 'default5' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
          !
          P_red(i1  , 0,i2,i3)=min(P_red(i1  , 0,i2,i3), C_max_Predfac*P(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=min(P_red(i1  ,-1,i2,i3), C_max_Predfac*P(i1-1,i2,i3))
          !
          P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1, 0,i2,i3)) + &
                          (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1,-1,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
!#
# elif (rhd_hyd_gravcorr_p01 == 6)
!#                                 /* 'default6' */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1  , 0,i2,i3)=rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))
          !
          P_red(i1  , 0,i2,i3)=min(P_red(i1  , 0,i2,i3), C_max_Predfac*P(i1  ,i2,i3))
          P_red(i1  ,-1,i2,i3)=min(P_red(i1  ,-1,i2,i3), C_max_Predfac*P(i1-1,i2,i3))
          !
          P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) + &
                          (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1, 0,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
!#
# endif
!#      /* rhd_hyd_gravcorr_p01 */
!#
# if (rhd_hyd_gravcorr_p01 == 1) || (rhd_hyd_gravcorr_p01 == 2) || (rhd_hyd_gravcorr_p01 == 3) || \
     (rhd_hyd_gravcorr_p01 == 4) || (rhd_hyd_gravcorr_p01 == 5) || (rhd_hyd_gravcorr_p01 == 6)
!#
    if (reconstruction_order >= 1) then
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1+1
            rho_mean_p1=(1.0-wl(i1+1))*rho(i1  ,i2,i3) + &
                             wl(i1+1) *rho(i1+1,i2,i3)
            rho_mean_m1=(1.0-wl(i1-1))*rho(i1-2,i2,i3) + &
                             wl(i1-1) *rho(i1-1,i2,i3)
            P_red(i1  , 1,i2,i3)=P_red(i1  , 0,i2,i3) + rho_mean_p1 * &
                                                      (phic_grav(i1+1,i2,i3)-phic_grav(i1  ,i2,i3))
            P_red(i1  ,-2,i2,i3)=P_red(i1  ,-1,i2,i3) + rho_mean_m1 * &
                                                      (phic_grav(i1-2,i2,i3)-phic_grav(i1-1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
    endif ! (reconstruction_order >= 1)
    !
    if (reconstruction_order >= 2) then
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1+1
            rho_mean_p2=(1.0-wl(i1+2))*rho(i1+1,i2,i3) + &
                             wl(i1+2) *rho(i1+2,i2,i3)
            rho_mean_m2=(1.0-wl(i1-2))*rho(i1-3,i2,i3) + &
                             wl(i1-2) *rho(i1-2,i2,i3)
            P_red(i1  , 2,i2,i3)=P_red(i1  , 1,i2,i3) + rho_mean_p2 * &
                                                      (phic_grav(i1+2,i2,i3)-phic_grav(i1+1,i2,i3))
            P_red(i1  ,-3,i2,i3)=P_red(i1  ,-2,i2,i3) + rho_mean_m2 * &
                                                      (phic_grav(i1-3,i2,i3)-phic_grav(i1-2,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
    endif ! (reconstruction_order >= 2)
!#
# endif
!#      /* rhd_hyd_gravcorr_p01 */
!#
# if (rhd_hyd_gravcorr_p01 == 11)
!#                               /* isothermal hydrostatic stratification */
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+2
          P_red(i1, 0,i2,i3)=P(i1  ,i2,i3) * &
                             (exp(rho(i1  ,i2,i3)/P(i1  ,i2,i3) * &
                                  (phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                              1.0)
          P_red(i1,-1,i2,i3)=P(i1-1,i2,i3) * &
                             (exp(rho(i1-1,i2,i3)/P(i1-1,i2,i3) * &
                                  (phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                              1.0)
          !
          P_avg(i1,i2,i3)=     wl(i1)  * (P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) + &
                          (1.0-wl(i1)) * (P(i1  ,i2,i3)-P_red(i1, 0,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
    !
    if (reconstruction_order >= 1) then
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1+1
            P_red(i1  , 1,i2,i3)=P_red(i1  , 0,i2,i3) - P_red(i1+1,-1,i2,i3) + P_red(i1+1, 0,i2,i3)
            P_red(i1  ,-2,i2,i3)=P_red(i1  ,-1,i2,i3) + P_red(i1-1,-1,i2,i3) - P_red(i1-1, 0,i2,i3)
          end do ! i1
        end do ! i2
      end do ! i3
    endif ! (reconstruction_order >= 1)
    !
    if (reconstruction_order >= 2) then
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1+1,n1
            P_red(i1  , 2,i2,i3)=P_red(i1  , 1,i2,i3) - P_red(i1+2,-1,i2,i3) + P_red(i1+2, 0,i2,i3)
            P_red(i1  ,-3,i2,i3)=P_red(i1  ,-2,i2,i3) + P_red(i1-2,-1,i2,i3) - P_red(i1-2, 0,i2,i3)
          end do ! i1
          !
          i1=m1-2
          P_redp0           =P(i1  ,i2,i3) * &
                             (exp(rho(i1  ,i2,i3)/P(i1  ,i2,i3) * &
                                  (phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                              1.0)
          P_redm1           =P(i1-1,i2,i3) * &
                             (exp(rho(i1-1,i2,i3)/P(i1-1,i2,i3) * &
                                  (phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                              1.0)
          i1=m1
            P_red(i1  , 2,i2,i3)=P_red(i1  , 1,i2,i3) - P_red(i1+2,-1,i2,i3) + P_red(i1+2, 0,i2,i3)
            P_red(i1  ,-3,i2,i3)=P_red(i1  ,-2,i2,i3) + P_redm1              - P_redp0
          !
          i1=n1+3
          P_redp0           =P(i1  ,i2,i3) * &
                             (exp(rho(i1  ,i2,i3)/P(i1  ,i2,i3) * &
                                  (phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                              1.0)
          P_redm1           =P(i1-1,i2,i3) * &
                             (exp(rho(i1-1,i2,i3)/P(i1-1,i2,i3) * &
                                  (phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                              1.0)
          i1=n1+1
            P_red(i1  , 2,i2,i3)=P_red(i1  , 1,i2,i3) - P_redm1              + P_redp0
            P_red(i1  ,-3,i2,i3)=P_red(i1  ,-2,i2,i3) + P_red(i1-2,-1,i2,i3) - P_red(i1-2, 0,i2,i3)
        end do ! i2
      end do ! i3
    endif ! (reconstruction_order >= 2)
!#
# endif
!#      /* rhd_hyd_gravcorr_p01 */
  endif
endif ! (grav_flag)
!
end subroutine rhd_hyd_Pred


!----------**********************-------------------------------------------------------------------
subroutine rhd_hyd_RoeWavealpha12(m1,n1, m2,n2, m3,n3, nghost1, &
                                  reconstruction_order, reconstruction_type, &
                                  dtimeHO, dxb1_avg, dxb1, wl, &
                                  rho, v1, v2, v3, P, &
                                  P_red, P_avg, &
                                  rho_til, cs_til, csRed_til, &
                                  SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                  i_up_mi, i_up_pl, lambda_til_mi, lambda_til_pl, &
                                  flux_alpha_til_1, flux_alpha_til_2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_RoeWavealpha12 ('rhd_hydrodynamics_Roe_Wave_alpha_1_2')
!
! PURPOSE:
!   Compute 1D Roe sound-wave fluxes.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_RoeWavealpha12(m1,n1, m2,n2, m3,n3, nghost1, &
!                               reconstruction_order, reconstruction_type, &
!                               dtimeHO, dxb1_avg, dxb1, wl, &
!                               rho, v1, v2, v3, P, &
!                               P_red, P_avg, &
!                               rho_til, cs_til, csRed_til, &
!                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                               i_up_mi, i_up_pl, lambda_til_mi, lambda_til_pl, &
!                               flux_alpha_til_1, flux_alpha_til_2)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtimeHO:      ('delta_time_HighOrder') real, high-order time step (might be zero)
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real, cell-boundary step size, 1. coordinate
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   rho:          ('rho') real 3D array, density
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   P:            ('Pressure') real 3D array, pressure on model grid
!   P_red:        ('Pressure_reduced') real 4D array, hydrostatic pressure correction
!   P_avg:        ('Pressure_average') real 3D array, simple average of pressure at cell boundary
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe-averaged sound speed at cell boundary
!   csRed_til:    ('constant_sound_Reduced_tilde') real 3D array, Roe-averaged sound speed
!                 reduced for small Mach numbers at cell boundary
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   i_up_mi:      ('index_upwind_minus') 3D real array: indices of upwind direction,
!                 sound- characteristic
!   i_up_pl:      ('index_upwind_plus') 3D real array: indices of upwind direction,
!                 sound+ characteristic
!   lambda_til_mi: ('lambda_tilde_minus') 3D real array with sound- characteristic velocities
!                  at cell boundaries
!   lambda_til_pl: ('lambda_tilde_plus') 3D real array with sound+ characteristic velocities
!                  at cell boundaries
!
! OUTPUT:
!   flux_alpha_til_1: ('flux_alpha_tilde_1') 3D real array with flux of sound- characteristic
!   flux_alpha_til_2: ('flux_alpha_tilde_2') 3D real array with flux of sound+ characteristic
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   j:            ('j') integer, loop index for stencil points
!   v1_avg:       ('velocity_1_average') real 3D array, simple average of velocity 1 at cell bound.
!   alpha0_til_1: ('alpha_0_tilde_1') real 3D array with amplitude of sound- characteristic
!   alpha0_til_2: ('alpha_0_tilde_2') real 3D array with amplitude of sound+ characteristic
!   alpha_til_1:  ('alpha_tilde_1') real 3D array with amplitude of sound- characteristic,
!                 upwind shifted
!   alpha_til_2:  ('alpha_tilde_2') real 3D array with amplitude of sound+ characteristic,
!                 upwind shifted
!
! ROUTINES:
!   rhd_rec_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   rhd_rec_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute projected wave amplitudes, reconstruct them, and compute upwind Roe correction flux
!   for both sound-wave characteristics.
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2001-01-17 (B.F. Uppsala) Put into own module
!   2013-02-22 (B.F. Lyon) Use new type of loops for upwind values: if-then-else instead of (i_up())
!   2013-02-23 (B.F.) Extracted from rhd_hyd_RoeWave1Dflux and put into own routine
!   2013-02-26 (B.F.) Select branch (i_up vs. if-then-else/indirect vs. direct) with compiler switch
!   2014-12-02 (B.F.) Add parameter dxb1
!   2014-12-03 (B.F.) Remove parameter xb1
!---------------------------------------------------------------------------------------------------
use rhd_rec_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtimeHO, dxb1_avg
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1  :n1+nghost1,&
                m2          :n2        ,&
                m3          :n3          ), &
                           intent(in)   :: rho, v1, v2, v3, P
real, dimension(m1-1:n1+2, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: P_red
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: rho_til, cs_til, csRed_til, &
                                           lambda_til_mi, lambda_til_pl
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: P_avg, SlopeRed, SlopeCon
integer, &
      dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ),&
                           intent(in)   :: i_up_mi, i_up_pl
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: flux_alpha_til_1, flux_alpha_til_2
!
! --- Local variables ---
integer                                 :: i1, i2, i3, j, nc
real                                    :: v1_avg
!integer, dimension(m1  :n1+1)           :: i_up
!#
# if ((rhd_roe1d_flux_l01 == 2) || (rhd_roe1d_flux_l01 == 3))
!#
real, dimension(m1  :n1+1, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     )           :: alpha0_til_1, alpha0_til_2
!#
# endif
!#      /* rhd_roe1d_flux_l01 */
real, dimension(m1  :n1+1, &
                -reconstruction_order: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     )           :: alpha_til_1, alpha_til_2
!---------------------------------------------------------------------------------------------------
! --- Compute projection coefficients ---
!#
# if ((rhd_roe1d_flux_l01 == 2) || (rhd_roe1d_flux_l01 == 3))
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: alpha_til: (12)a')
!#
# endif
!#
! --- First, compute all alpha values in neighborhood of boundary (actually, one too much) ---
select case (reconstruction_order) ! ---------------------------------------------------------------
case(1) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
          alpha0_til_1(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
          alpha0_til_2(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
      end do ! i1
    end do ! i2
  end do ! i3
  !
case(2) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
          alpha0_til_1(i1,-3,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-3  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-3  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-3  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
          alpha0_til_1(i1, 2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 2  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+2  ,i2,i3) - v1_avg) )
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
          alpha0_til_2(i1,-3,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-3  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-3  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-3  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
          alpha0_til_2(i1, 2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 2  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+2  ,i2,i3) - v1_avg) )
      end do ! i1
    end do ! i2
  end do ! i3
  !
!!$  do i3=m3,n3
!!$    do i2=m2,n2
!!$      do i1=m1,n1+1
!!$        v1_avg  =    wl(i1) * v1( i1-1,i2,i3) + &
!!$                (1.0-wl(i1))* v1( i1  ,i2,i3)
!!$        !DIR$ UNROLL
!!$        do j=-3,2
!!$          alpha0_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
!!$                                  ( P(    i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3) &
!!$                                    - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
!!$                                      (v1(i1+j,i2,i3) - v1_avg) )
!!$        end do ! j
!!$        !DIR$ UNROLL
!!$        do j=-3,2
!!$          alpha0_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
!!$                                  ( P(    i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3) &
!!$                                    + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
!!$                                      (v1(i1+j,i2,i3) - v1_avg) )
!!$        end do ! j
!!$      end do ! i1
!!$    end do ! i2
!!$  end do ! i3
  !
case default ! (reconstruction_order) ....................................................
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order-1,reconstruction_order
        do i1=m1,n1+1
!          rho_avg =    wl(i1) * rho(i1-1,i2,i3) + &
!                  (1.0-wl(i1))* rho(i1  ,i2,i3)
!          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
!                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          v1_avg  =    wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
!          v2_avg  =    wl(i1) * v2( i1-1,i2,i3) + &
!                  (1.0-wl(i1))* v2( i1  ,i2,i3)
!          v3_avg  =    wl(i1) * v3( i1-1,i2,i3) + &
!                  (1.0-wl(i1))* v3( i1  ,i2,i3)
          alpha0_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                  ( P(    i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3) &
                                    - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                      (v1(i1+j,i2,i3) - v1_avg) )
          alpha0_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                  ( P(    i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3) &
                                    + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                      (v1(i1+j,i2,i3) - v1_avg) )
!          alpha0_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3) * &
!                                  ( (P(i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3)) / &
!                                    csRed_til(i1,i2,i3) &
!                                   - rho_til(i1,i2,i3) * &
!                                      (v1(i1+j,i2,i3) - v1_avg) )
!          alpha0_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3) * &
!                                  ( (P(i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3)) / &
!                                    csRed_til(i1,i2,i3) &
!                                    + rho_til(i1,i2,i3) * &
!                                     (v1(i1+j,i2,i3) - v1_avg) )
!          alpha0_til_1(i1,j,i2,i3)=0.5*(csRed_til(i1,i2,i3)/cs_til(i1,i2,i3))/cs_til(i1,i2,i3)**2*&
!                                  ( (P(i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3)) &
!                                    - rho_til(i1,i2,i3) * cs_til(i1,i2,i3) * &
!                                      (v1(i1+j,i2,i3) - v1_avg) )
!          alpha0_til_2(i1,j,i2,i3)=0.5*(csRed_til(i1,i2,i3)/cs_til(i1,i2,i3))/cs_til(i1,i2,i3)**2*&
!                                  ( (P(i1+j,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j,i2,i3)) &
!                                    + rho_til(i1,i2,i3) * cs_til(i1,i2,i3) * &
!                                      (v1(i1+j,i2,i3) - v1_avg) )
!          alpha0_til_3(i1,j,i2,i3)=rho(i1+j,i2,i3) - rho_avg - &
!                                   1.0/cs_til(i1,i2,i3)**2 * &
!                                     ( P(i1+j,i2,i3) - P_avg(i1,i2,i3) &
!                                       - par%C_hydPredfactor*P_red(i1,j,i2,i3) &
!                                     )
!          alpha0_til_4(i1,j,i2,i3)=v2(i1+j,i2,i3) - v2_avg
!          alpha0_til_5(i1,j,i2,i3)=v3(i1+j,i2,i3) - v3_avg
!          alpha0_til_6(i1,j,i2,i3)=rho(i1+j,i2,i3)*ei(i1+j,i2,i3) - rhoei_avg - &
!                                   DrhoeiDPConsts_til(i1,i2,i3) * &
!                                     ( P(i1+j,i2,i3) - P_avg(i1,i2,i3) &
!                                       - par%C_hydPredfactor*P_red(i1,j,i2,i3) &
!                                     )
        end do ! i1
      end do ! j
    end do ! i2
  end do ! i3
end select ! (reconstruction_order) ----------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: alpha_til: (12)a')
call timing_start('HYD: Roe1Dflux: alpha_til: (12)b')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# if (rhd_roe1d_flux_l01 == 2)
!#
! --- Do the upwind shift (with i_up) ---
select case (reconstruction_order) ! ---------------------------------------------------------------
case(1) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !DIR$ UNROLL
        do j=-1,1
          alpha_til_1(i1,j,i2,i3)=alpha0_til_1(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3)
        end do ! j
      end do ! i1
      !
      do i1=m1,n1+1
        !DIR$ UNROLL
        do j=-1,1
          alpha_til_2(i1,j,i2,i3)=alpha0_til_2(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3)
        end do ! j
      end do ! i1
    end do ! i2
  end do ! i3
  !
case(2) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !DIR$ UNROLL
        do j=-2,2
          alpha_til_1(i1,j,i2,i3)=alpha0_til_1(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3)
        end do ! j
      end do ! i1
      !
      do i1=m1,n1+1
        !DIR$ UNROLL
        do j=-2,2
          alpha_til_2(i1,j,i2,i3)=alpha0_til_2(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3)
        end do ! j
      end do ! i1
    end do ! i2
  end do ! i3
  !
case default ! (reconstruction_order) ....................................................
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order,reconstruction_order
        do i1=m1,n1+1
          alpha_til_1(i1,j,i2,i3)=alpha0_til_1(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3)
!          alpha_til_2(i1,j,i2,i3)=alpha0_til_2(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3)
!          alpha_til_3(i1,j,i2,i3)=alpha0_til_3(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
!          alpha_til_4(i1,j,i2,i3)=alpha0_til_4(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
!          alpha_til_5(i1,j,i2,i3)=alpha0_til_5(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
!          alpha_til_6(i1,j,i2,i3)=alpha0_til_6(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
        end do ! i1
        !
        do i1=m1,n1+1
           alpha_til_2(i1,j,i2,i3)=alpha0_til_2(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3)
        end do ! i1
      end do ! j
    end do ! i2
  end do ! i3
  !
end select ! (reconstruction_order) ----------------------------------------------------------------
!#
# else
!#     /* (rhd_roe1d_flux_l01 == 3) */
! --- Do the upwind shift (with if-then-else) ---
select case (reconstruction_order) ! ---------------------------------------------------------------
case(1) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1,n1+1
        if (i_up_mi(i1,i2,i3) < i1) then
          alpha_til_1(i1,-1,i2,i3)=alpha0_til_1(i1,-1-1,i2,i3)
          alpha_til_1(i1, 0,i2,i3)=alpha0_til_1(i1, 0-1,i2,i3)
          alpha_til_1(i1, 1,i2,i3)=alpha0_til_1(i1, 1-1,i2,i3)
        else
          alpha_til_1(i1,-1,i2,i3)=alpha0_til_1(i1,-1  ,i2,i3)
          alpha_til_1(i1, 0,i2,i3)=alpha0_til_1(i1, 0  ,i2,i3)
          alpha_til_1(i1, 1,i2,i3)=alpha0_til_1(i1, 1  ,i2,i3)
        endif
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        if (i_up_pl(i1,i2,i3) < i1) then
          alpha_til_2(i1,-1,i2,i3)=alpha0_til_2(i1,-1-1,i2,i3)
          alpha_til_2(i1, 0,i2,i3)=alpha0_til_2(i1, 0-1,i2,i3)
          alpha_til_2(i1, 1,i2,i3)=alpha0_til_2(i1, 1-1,i2,i3)
        else
          alpha_til_2(i1,-1,i2,i3)=alpha0_til_2(i1,-1  ,i2,i3)
          alpha_til_2(i1, 0,i2,i3)=alpha0_til_2(i1, 0  ,i2,i3)
          alpha_til_2(i1, 1,i2,i3)=alpha0_til_2(i1, 1  ,i2,i3)
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !
case(2) ! (reconstruction_order) .........................................................
  !
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1,n1+1
        if (i_up_mi(i1,i2,i3) < i1) then
          alpha_til_1(i1,-2,i2,i3)=alpha0_til_1(i1,-2-1,i2,i3)
          alpha_til_1(i1,-1,i2,i3)=alpha0_til_1(i1,-1-1,i2,i3)
          alpha_til_1(i1, 0,i2,i3)=alpha0_til_1(i1, 0-1,i2,i3)
          alpha_til_1(i1, 1,i2,i3)=alpha0_til_1(i1, 1-1,i2,i3)
          alpha_til_1(i1, 2,i2,i3)=alpha0_til_1(i1, 2-1,i2,i3)
        else
          alpha_til_1(i1,-2,i2,i3)=alpha0_til_1(i1,-2  ,i2,i3)
          alpha_til_1(i1,-1,i2,i3)=alpha0_til_1(i1,-1  ,i2,i3)
          alpha_til_1(i1, 0,i2,i3)=alpha0_til_1(i1, 0  ,i2,i3)
          alpha_til_1(i1, 1,i2,i3)=alpha0_til_1(i1, 1  ,i2,i3)
          alpha_til_1(i1, 2,i2,i3)=alpha0_til_1(i1, 2  ,i2,i3)
        endif
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        if (i_up_pl(i1,i2,i3) < i1) then
          alpha_til_2(i1,-2,i2,i3)=alpha0_til_2(i1,-2-1,i2,i3)
          alpha_til_2(i1,-1,i2,i3)=alpha0_til_2(i1,-1-1,i2,i3)
          alpha_til_2(i1, 0,i2,i3)=alpha0_til_2(i1, 0-1,i2,i3)
          alpha_til_2(i1, 1,i2,i3)=alpha0_til_2(i1, 1-1,i2,i3)
          alpha_til_2(i1, 2,i2,i3)=alpha0_til_2(i1, 2-1,i2,i3)
        else
          alpha_til_2(i1,-2,i2,i3)=alpha0_til_2(i1,-2  ,i2,i3)
          alpha_til_2(i1,-1,i2,i3)=alpha0_til_2(i1,-1  ,i2,i3)
          alpha_til_2(i1, 0,i2,i3)=alpha0_til_2(i1, 0  ,i2,i3)
          alpha_til_2(i1, 1,i2,i3)=alpha0_til_2(i1, 1  ,i2,i3)
          alpha_til_2(i1, 2,i2,i3)=alpha0_til_2(i1, 2  ,i2,i3)
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !
case default ! (reconstruction_order) ....................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        if (i_up_mi(i1,i2,i3) < i1) then
          do j=-reconstruction_order,reconstruction_order
            alpha_til_1(i1,j,i2,i3)=alpha0_til_1(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3)
          end do ! j
        endif
        !
        if (i_up_pl(i1,i2,i3) < i1) then
          do j=-reconstruction_order,reconstruction_order
            alpha_til_2(i1,j,i2,i3)=alpha0_til_2(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3)
          end do ! j
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !
end select ! (reconstruction_order) ----------------------------------------------------------------
!#
# endif
!#      /* rhd_roe1d_flux_l01 */
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: alpha_til: (12)b')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# elif (rhd_roe1d_flux_l01 == 0)
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: alpha_til (12)')
!#
# endif
!#
!
! --- Compute alpha values (amplitude) in one go (with i_up) ---
select case (reconstruction_order) ! ---------------------------------------------------------------
case(0) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
!        rho_avg =      wl(i1) * rho(i1-1,i2,i3) + &
!                  (1.0-wl(i1))* rho(i1  ,i2,i3)
!        rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
!                  (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
        v1_avg  =      wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
!        v2_avg  =      wl(i1) * v2( i1-1,i2,i3) + &
!                  (1.0-wl(i1))* v2( i1  ,i2,i3)
!        v3_avg  =      wl(i1) * v3( i1-1,i2,i3) + &
!                  (1.0-wl(i1))* v3( i1  ,i2,i3)
      j=0
        alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                 ( P(i_up_mi(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                   - P_red(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3) &
                                   - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                     (v1(i_up_mi(i1,i2,i3)+j,i2,i3) - v1_avg) )
        alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                 ( P(i_up_pl(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                   - P_red(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3) &
                                   + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                     (v1(i_up_pl(i1,i2,i3)+j,i2,i3) - v1_avg) )
!        alpha_til_1(i1,j,i2,i3)=0.5*(csRed_til(i1,i2,i3)/cs_til(i1,i2,i3))/cs_til(i1,i2,i3)**2 * &
!                                 ( P(i_up_mi(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
!                                   - P_red(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3) &
!                                   - rho_til(i1,i2,i3) * cs_til(i1,i2,i3) * &
!                                     (v1(i_up_mi(i1,i2,i3)+j,i2,i3) - v1_avg) )
!        alpha_til_2(i1,j,i2,i3)=0.5*(csRed_til(i1,i2,i3)/cs_til(i1,i2,i3))/cs_til(i1,i2,i3)**2 * &
!                                 ( P(i_up_pl(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
!                                   - P_red(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3) &
!                                   + rho_til(i1,i2,i3) * cs_til(i1,i2,i3) * &
!                                     (v1(i_up_pl(i1,i2,i3)+j,i2,i3) - v1_avg) )
!        alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3) * &
!                                 ( (P(i_up_mi(i1,i2,i3)+j,i2,i3) - &
!                                    P_avg(i1,i2,i3) - &
!                                    P_red(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3)) / &
!                                   csRed_til(i1,i2,i3) &
!                                   - rho_til(i1,i2,i3) * &
!                                     (v1(i_up_mi(i1,i2,i3)+j,i2,i3) - v1_avg) )
!        alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3) * &
!                                 ( (P(i_up_pl(i1,i2,i3)+j,i2,i3) - &
!                                    P_avg(i1,i2,i3) - &
!                                    P_red(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3)) / &
!                                   csRed_til(i1,i2,i3) &
!                                   + rho_til(i1,i2,i3) * &
!                                     (v1(i_up_pl(i1,i2,i3)+j,i2,i3) - v1_avg) )
!        alpha_til_3(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
!                                  1.0/cs_til(i1,i2,i3)**2 * &
!                                    ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
!                                      - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) &
!                                    )
!        alpha_til_4(i1,j,i2,i3)=v2(i_up_nu(i1,i2,i3)+j,i2,i3) - v2_avg
!        alpha_til_5(i1,j,i2,i3)=v3(i_up_nu(i1,i2,i3)+j,i2,i3) - v3_avg
!        alpha_til_6(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3)*ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
!                                  rhoei_avg - &
!                                  DrhoeiDPConsts_til(i1,i2,i3) * &
!                                    ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
!                                      - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) &
!                                    )
      end do ! i1
    end do ! i2
  end do ! i3
  !
case(1) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        !DIR$ UNROLL
        do j=-1,1
          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i_up_mi(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                     - P_red(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i_up_mi(i1,i2,i3)+j,i2,i3) - v1_avg) )
        end do ! j
        !
        !DIR$ UNROLL
        do j=-1,1
          alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i_up_pl(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                     - P_red(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i_up_pl(i1,i2,i3)+j,i2,i3) - v1_avg) )
        end do ! j
      end do ! i1
    end do ! i2
  end do ! i3
  !
case(2) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        !DIR$ UNROLL
        do j=-2,2
          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i_up_mi(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                     - P_red(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i_up_mi(i1,i2,i3)+j,i2,i3) - v1_avg) )
        end do ! j
        !
        !DIR$ UNROLL
        do j=-2,2
          alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i_up_pl(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                     - P_red(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i_up_pl(i1,i2,i3)+j,i2,i3) - v1_avg) )
        end do ! j
      end do ! i1
    end do ! i2
  end do ! i3
!
case default ! (reconstruction_order) ....................................................
  do i3=m3,n3
    do i2=m2,n2
      do j=-reconstruction_order,reconstruction_order
        do i1=m1,n1+1
          v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                    (1.0-wl(i1))* v1( i1  ,i2,i3)
          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i_up_mi(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                     - P_red(i1,i_up_mi(i1,i2,i3)-i1+j,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i_up_mi(i1,i2,i3)+j,i2,i3) - v1_avg) )
          alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i_up_pl(i1,i2,i3)+j,i2,i3)     - P_avg(i1,i2,i3) &
                                     - P_red(i1,i_up_pl(i1,i2,i3)-i1+j,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i_up_pl(i1,i2,i3)+j,i2,i3) - v1_avg) )
        end do ! i1
      end do ! j
    end do ! i2
  end do ! i3
  !
end select ! (reconstruction_order) ----------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: alpha_til (12)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# else
!#     /* rhd_roe1d_flux_l01 == 1 */
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: alpha_til (12)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!
! --- Compute alpha values (amplitude) with if-then-else ---
select case (reconstruction_order) ! ---------------------------------------------------------------
case(1) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        if (i_up_mi(i1,i2,i3) < i1) then
          alpha_til_1(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1-1,i2,i3) - v1_avg) )
          alpha_til_1(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0-1,i2,i3) - v1_avg) )
          alpha_til_1(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1-1,i2,i3) - v1_avg) )
        else
          alpha_til_1(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha_til_1(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha_til_1(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
        endif
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        if (i_up_pl(i1,i2,i3) < i1) then
          alpha_til_2(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1-1,i2,i3) - v1_avg) )
          alpha_til_2(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0-1,i2,i3) - v1_avg) )
          alpha_til_2(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1-1,i2,i3) - v1_avg) )
        else
          alpha_til_2(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha_til_2(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha_til_2(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !
!!$case(2)
!!$  do i3=m3,n3
!!$    do i2=m2,n2
!!$      !DIR$ VECTOR ALWAYS
!!$      !DIR$ IVDEP
!!$      do i1=m1,n1+1
!!$        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
!!$                  (1.0-wl(i1))* v1( i1  ,i2,i3)
!!$        if (i_up_mi(i1,i2,i3) < i1) then
!!$         !DIR$ UNROLL
!!$         do j=-2,2
!!$          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
!!$                                   ( P(i1+j-1,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j-1,i2,i3) &
!!$                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
!!$                                       (v1(i1+j-1,i2,i3) - v1_avg) )
!!$         end do ! j
!!$        else
!!$         !DIR$ UNROLL
!!$         do j=-2,2
!!$          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
!!$                                   ( P(i1+j  ,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j  ,i2,i3) &
!!$                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
!!$                                       (v1(i1+j  ,i2,i3) - v1_avg) )
!!$         end do ! j
!!$        endif
!!$        if (i_up_pl(i1,i2,i3) < i1) then
!!$         !DIR$ UNROLL
!!$         do j=-2,2
!!$          alpha_til_2(i1,j,i2,i3) = 0.5/cs_til(i1,i2,i3)**2 * &
!!$                                   ( P(i1+j-1,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j-1,i2,i3) &
!!$                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
!!$                                       (v1(i1+j-1,i2,i3) - v1_avg) )
!!$         end do ! j
!!$        else
!!$         !DIR$ UNROLL
!!$         do j=-2,2
!!$          alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
!!$                                   ( P(i1+j  ,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j  ,i2,i3) &
!!$                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
!!$                                       (v1(i1+j  ,i2,i3) - v1_avg) )
!!$         end do ! j
!!$        endif
!!$      end do ! i1
!!$    end do ! i2
!!$  end do ! i3
!
case(2) ! (reconstruction_order) .........................................................
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        if (i_up_mi(i1,i2,i3) < i1) then
          alpha_til_1(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2-1,i2,i3) - v1_avg) )
          alpha_til_1(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1-1,i2,i3) - v1_avg) )
          alpha_til_1(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0-1,i2,i3) - v1_avg) )
          alpha_til_1(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1-1,i2,i3) - v1_avg) )
          alpha_til_1(i1, 2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+2-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 2-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+2-1,i2,i3) - v1_avg) )
        else
          alpha_til_1(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2  ,i2,i3) - v1_avg) )
          alpha_til_1(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha_til_1(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha_til_1(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
          alpha_til_1(i1, 2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 2  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+2  ,i2,i3) - v1_avg) )
        endif
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        if (i_up_pl(i1,i2,i3) < i1) then
          alpha_til_2(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2-1,i2,i3) - v1_avg) )
          alpha_til_2(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1-1,i2,i3) - v1_avg) )
          alpha_til_2(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0-1,i2,i3) - v1_avg) )
          alpha_til_2(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1-1,i2,i3) - v1_avg) )
          alpha_til_2(i1, 2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+2-1,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 2-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+2-1,i2,i3) - v1_avg) )
        else ! (i_up_pl(i1,i2,i3) >= i1)
          alpha_til_2(i1,-2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-2  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-2  ,i2,i3) - v1_avg) )
          alpha_til_2(i1,-1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1-1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1,-1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1-1  ,i2,i3) - v1_avg) )
          alpha_til_2(i1, 0,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+0  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 0  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+0  ,i2,i3) - v1_avg) )
          alpha_til_2(i1, 1,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+1  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 1  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+1  ,i2,i3) - v1_avg) )
          alpha_til_2(i1, 2,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(    i1+2  ,i2,i3) - P_avg(i1,i2,i3) - P_red(i1, 2  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+2  ,i2,i3) - v1_avg) )
        endif ! (i_up_pl(i1,i2,i3))
      end do ! i1
    end do ! i2
  end do ! i3
  !
case default ! (reconstruction_order) ....................................................
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        v1_avg   =     wl(i1) * v1( i1-1,i2,i3) + &
                  (1.0-wl(i1))* v1( i1  ,i2,i3)
        !
        if (i_up_mi(i1,i2,i3) < i1) then
         do j=-reconstruction_order,reconstruction_order
          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i1+j-1,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j-1,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+j-1,i2,i3) - v1_avg) )
         end do ! j
        else
         do j=-reconstruction_order,reconstruction_order
          alpha_til_1(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i1+j  ,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j  ,i2,i3) &
                                     - rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+j  ,i2,i3) - v1_avg) )
         end do ! j
        endif
        if (i_up_pl(i1,i2,i3) < i1) then
         do j=-reconstruction_order,reconstruction_order
          alpha_til_2(i1,j,i2,i3) = 0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i1+j-1,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j-1,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+j-1,i2,i3) - v1_avg) )
         end do ! j
        else
         do j=-reconstruction_order,reconstruction_order
          alpha_til_2(i1,j,i2,i3)=0.5/cs_til(i1,i2,i3)**2 * &
                                   ( P(i1+j  ,i2,i3)     - P_avg(i1,i2,i3) - P_red(i1,j  ,i2,i3) &
                                     + rho_til(i1,i2,i3) * csRed_til(i1,i2,i3) * &
                                       (v1(i1+j  ,i2,i3) - v1_avg) )
         end do ! j
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !
end select ! (reconstruction_order) ----------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: alpha_til (12)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# endif
!#      /* rhd_roe1d_flux_l01 */
!
! --- Perform reconstruction (constant, slope limiter, PP), compute fluxes of partial waves ---
!
if (dxb1_avg < 0.0) then
  ! --- Reconstruct for spatially varying grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
  call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         dtimeHO, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         alpha_til_1, dxb1, &
                         i_up_mi, lambda_til_mi, &
                         flux_alpha_til_1)
  call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         dtimeHO, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         alpha_til_2, dxb1, &
                         i_up_pl, lambda_til_pl, &
                         flux_alpha_til_2)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop( 'HYD: Roe1Dflux: Fluxalpha', cycles=2)
!#
# endif
!#
!
  !
else ! (dxb1_avg > 0.0)
  ! --- Reconstruct for equidistant grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: FluxalphaEq')
!#
# endif
!#
  call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           alpha_til_1, dxb1_avg, &
                           lambda_til_mi, &
                           flux_alpha_til_1)
  call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           alpha_til_2, dxb1_avg, &
                           lambda_til_pl, &
                           flux_alpha_til_2)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: FluxalphaEq', cycles=2)
!#
# endif
!#
  !
endif ! (dxb1_avg < 0.0)
!
!!$call uio_openwr(nc, 'rhd_roe1dalpha12_test.dat', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now' )
!!$call uio_wrlabl(nc, box_ident                 )
!!$call uio_wr(nc, 'z'               , box_id_ident        )
!!$call uio_wr(nc, dxb1              , 'dxb1'              )
!!$call uio_wr(nc, rho               , 'rho'               )
!!$call uio_wr(nc, v1                , 'v1'                )
!!$call uio_wr(nc, v2                , 'v2'                )
!!$call uio_wr(nc, v3                , 'v3'                )
!!$call uio_wr(nc, p                 , 'p'                 )
!!$call uio_wr(nc, P_avg             , 'p_avg'             )
!!$call uio_wr(nc, P_red             , 'p_red'             )
!!$call uio_wr(nc, rho_til           , 'rho_til'           )
!!$call uio_wr(nc, cs_til            , 'cs_til'            )
!!$call uio_wr(nc, csRed_til         , 'csred_til'         )
!!$call uio_wr(nc, lambda_til_pl     , 'lambda_til_pl'     )
!!$call uio_wr(nc, lambda_til_mi     , 'lambda_til_mi'     )
!!$call uio_wr(nc, i_up_pl           , 'i_up_pl'           )
!!$call uio_wr(nc, i_up_mi           , 'i_up_mi'           )
!!$call uio_wr(nc, alpha_til_1       , 'alpha_til_1'       )
!!$call uio_wr(nc, alpha_til_2       , 'alpha_til_2'       )
!!$call uio_wr(nc, flux_alpha_til_1  , 'flux_alpha_til_1'  )
!!$call uio_wr(nc, flux_alpha_til_2  , 'flux_alpha_til_2'  )
!!$call uio_wrlabl(nc, endbox_ident)
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call flush(nc)
!!$call uio_closrd(nc)
!
end subroutine rhd_hyd_RoeWavealpha12


!----------**********************-------------------------------------------------------------------
subroutine rhd_hyd_RoeWavealpha36(m1,n1, m2,n2, m3,n3, nghost1, &
                                  reconstruction_order, reconstruction_type, &
                                  dtimeHO, dxb1_avg, dxb1, wl, &
                                  rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, s, T, dTdei, &
                                  P_red, P_avg, &
                                  sqrt_rho, wrho, wLgFluct_til, &
                                  rho_til, cs_til, v1abs_til, DrhoeiDPConsts_til, &
                                  SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                  i_up_nu, lambda_til_nu, &
                                  flux_alpha_til_3, flux_alpha_til_6, &
                                  flux_rhoei_sDiff)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_RoeWavealpha36 ('rhd_hydrodynamics_Roe_Wave_alpha_3_6')
!
! PURPOSE:
!   Compute 1D entropy-wave and ionization-energy fluxes.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_RoeWavealpha36(m1,n1, m2,n2, m3,n3, nghost1, &
!                               reconstruction_order, reconstruction_type, &
!                               dtimeHO, dxb1_avg, dxb1, wl, &
!                               rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, s, T, dTdei, &
!                               P_red, P_avg, &
!                               sqrt_rho, wrho, wLgFluct_til, &
!                               rho_til, cs_til, v1abs_til, DrhoeiDPConsts_til, &
!                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                               i_up_nu, lambda_til_nu, &
!                               flux_alpha_til_3, flux_alpha_til_6, &
!                               flux_rhoei_sDiff)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtimeHO:      ('delta_time_HighOrder') real, high-order time step (might be zero)
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real vector, cell-boundary step size, 1. coordinate
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   rho:          ('rho') real 3D array, density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   Gamma_3:      ('Gamma_3') real 3D array, 3rd adiabatic coefficient on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   T:            ('temperature') real 3D array, temperature on model grid
!   dTdei:        ('dT_dei') real 3D array, temperature derivative (constant density) on model grid
!   P_red:        ('Pressure_reduced') real 4D array, hydrostatic pressure correction
!   P_avg:        ('Pressure_average') real 3D array, simple average of pressure at cell boundary
!   wrho:         ('weight_rho') real 3D array, Roe weight from square root of density
!   wLgFluct_til: ('weight_Large_Fluctuations_tilde') real 3D array with weight from
!                 fluctuation amplitude
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe-averaged sound speed at cell boundary
!   v1abs_til:    ('velocity_1_absolute_tilde') real 3D array, Roe-averaged absolute velocity 1
!   DrhoeiDPConsts_til: ('Delta_rho_energy_internal_Delta_Pressure_Constant_s_tilde') real 3D array,
!                       thermodynamic quantity (drhoei/dP|_s) at cell boundary
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   i_up_nu:      ('index_upwind_null') 3D real array: indices of upwind direction,
!                 entropy-wave characteristic
!   lambda_til_nu: ('lambda_tilde_null') 3D real array with entropy-wave velocities
!                  at cell boundaries
!
! OUTPUT:
!   flux_alpha_til_3: ('flux_alpha_tilde_3') 3D real array with flux of entropy-wave characteristic
!   flux_alpha_til_6: ('flux_alpha_tilde_6') 3D real array with flux of ionization-energy charact.
!   flux_rhoei_sDiff: ('flux_rho_energy_internal_s_Diffusion') real 3D array, diffusive
!                     internal energy flux
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   rho_avg:      ('rho_average') real, simple average of density at cell boundary
!   rhoei_avg:    ('rho_energy_internal_average') real, simple average of internal energy
!                 at cell boundary
!   s_avg:        ('s_average') real, simple average of entropy at cell boundary
!   oOcpd_l:      ('one_Over_cp_dash_left') real, 1/cp' in left cell
!   oOcpd_r:      ('one_Over_cp_dash_right') real, 1/cp' in right cell
!   DrhoDsConstP_til: ('Delta_rho_Delta_s_Constant_Pressure_tilde') real, drho/ds|P~, Roe average
!                     of thermodynamic quantity at cell boundary
!   DrhoeiDsConstpOrhoT_l: ('Delta_rho_energy_internal_Delta_s_Const_pressure_Over_rho_Temp_left')
!                          real, (drhoei/ds|p)/(rho*T) in left cell
!   DrhoeiDsConstpOrhoT_r: ('Delta_rho_energy_internal_Delta_s_Const_pressure_Over_rho_Temp_right')
!                          real, (drhoei/ds|p)/(rho*T) in right cell
!   DrhoeiDsConstpOrhoT_til: ('Delta_rho_energy_internal_Delta_s_Const_pressure_Over_rho_Temp_tilde')
!                            real, Roe average of (drhoei/ds|p)/(rho*T) at cell boundary
!   rhoT_l:       ('rho_Temperature_left') real, rho*T in left cell
!   rhoT_r:       ('rho_Temperature_rightt') real, rho*T in right cell
!   rhoT_til      ('rho_Temperature_tilde') real, Roe average of rho*T at cell boundary
!   ds:           ('delta_entropy') real, weight derived from entropy step between adjacent
!                 grid cells
!   dP:           ('delta_pressure') real, weight derived from pressure step between grid cells
!   dPg:          ('delta_pressure_gravity') real, weight derived from gravity-corrected pressure
!                 step between grid adjacent cells
!   wMach:        ('weight_Mach') real, derived from Mach number
!   wTot:         ('weight_Total') real, total weight
!   Diff_flag:    ('Diffusion_flag') logical indicating that extra diffusion should be added
!   vD:           ('velocity_Diffusion') 3D array with additional diffusion velocity in cells
!   vD_til:       ('velocity_Diffusion_tilde') 3D array with add. diffusion velocity at cell bnd.
!   Gamma_1_til:  ('Gamma_1_tilde') Roe-averaged 1st adiabatic coefficient
!   cssim_til:    ('constant_sound_simple_tilde') Roe-averaged pure sound speed at bd.
!   flux_s:       ('flux_s') real 3D array with entropy flux through cell boundary
!   alpha0_til_3: ('alpha_0_tilde_3') real 4D array with amplitude of entropy-wave characteristic
!   alpha0_til_6: ('alpha_0_tilde_6') real 4D array with amplitude of ionization-energy
!                 characteristic
!   alpha_til_3:  ('alpha_tilde_3') real 4D array with amplitude of entropy-wave characteristic,
!                 upwind shifted
!   alpha_til_6:  ('alpha_tilde_6') real 4D array with amplitude of ionization-energy
!                 characteristic, upwind shifted
!   eps_TinyPressure: ('epsilon_Tiny_Pressure') real parameter (1.0E-06) with tiny relative pressure
!                     to prevent division by zero
!
! ROUTINES:
!   rhd_rec_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   rhd_rec_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   rhd_hyd_FluxState:     Compute 1D flux from state vector for single quantity
!   rhd_hyd_sDiff:         Compute 1D energy flux down an entropy step with state reconstruction
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2001-01-17 (B.F. Uppsala) Put into own module
!   2013-02-23 (B.F. Lyon) Split from rhd_hyd_RoeWave1Dflux and put into own routine
!   2013-02-24 (B.F.) Use new type of loops for upwind values: if-then-else instead of (i_up())
!   2013-02-26 (B.F.) Remove some case(0) branches; select
!                     branch (i_up vs. if-then-else/indirect vs. direct) with compiler switch;
!                     new input parameter wLgFluct_til
!   2013-02-27 (B.F.) Precompute some thermodynamics quantities in separate loop
!   2014-08-26 (B.F. Uppsala) Bugfix in averaging of tilde thermodynamic variables
!   2014-12-02 (B.F.) Add parameter dxb1
!   2014-12-03 (B.F.) Remove parameter xb1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_rec_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtimeHO, dxb1_avg
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1  :n1+nghost1,&
                m2          :n2        ,&
                m3          :n3          ), &
                           intent(in)   :: rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, s, T, dTdei, &
                                           sqrt_rho
real, dimension(m1-1:n1+2, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: P_red
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3    ), &
                           intent(in)   :: wrho, &
                                           rho_til, cs_til, v1abs_til, &
                                           DrhoeiDPConsts_til, lambda_til_nu
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: P_avg, wLgFluct_til, SlopeRed, SlopeCon
integer, &
      dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: i_up_nu
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: flux_alpha_til_3, flux_alpha_til_6, &
                                           flux_rhoei_sDiff
!
! --- Local variables ---
integer                                 :: i1, i2, i3, j
real                                    :: rho_avg, rhoei_avg, s_avg, &
                                           DrhoDsConstP_til, &
                                           DrhoeiDsConstPOrhoT_til, &
                                           rhoT_til, ds, dP, dPg, wMach, wTot, &
                                           cssim_til, Psim_til, Gamma_1_til
logical                                 :: Diff_flag
real, dimension(m1-1:n1+1   )           :: oOcpd_v, DrhoeiDsConstpOrhoT_v, rhoT_v
real, dimension(m1-1:n1+1, &
                m2  :n2  , &
                m3  :n3     )           :: vD
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3    )            :: vD_til, flux_s
!#
# if ((rhd_roe1d_flux_l01 == 2) || (rhd_roe1d_flux_l01 == 3))
!#
real, dimension(m1  :n1+1, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     )           :: alpha0_til_3, alpha0_til_6
!#
# endif
!#      /* rhd_roe1d_flux_l01 */
real, dimension(m1  :n1+1, &
                -reconstruction_order: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     )           :: alpha_til_3, alpha_til_6
real, parameter                         :: eps_TinyPressure=1.0E-06
!---------------------------------------------------------------------------------------------------
!
! \label{f90:hdentropywavemode__use1}
if (par%hdEntropyWaveMode == 'Normal') then ! ======================================================
  ! --- Compute entropy-wave contribution(s) in the standard way from dP and drho ---
  !
  ! --- Compute projection coefficients ---
!#
# if ((rhd_roe1d_flux_l01 == 2) || (rhd_roe1d_flux_l01 == 3))
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til: (36)a')
!#
# endif
!#
  ! --- First, compute all alpha values in neighborhood of boundary (actually, one too much) ---
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            !
            ! \label{f90:c_hydpredfactor__use1}
            alpha0_til_3(i1,-3,i2,i3)=rho(  i1-3,i2,i3) - rho_avg - &
                                        1.0/cs_til(i1,i2,i3)**2 * &
                                        ( P(i1-3,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1,-3,i2,i3) &
                                        )
            alpha0_til_3(i1,-2,i2,i3)=rho(  i1-2,i2,i3) - rho_avg - &
                                        1.0/cs_til(i1,i2,i3)**2 * &
                                        ( P(i1-2,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1,-2,i2,i3) &
                                        )
            alpha0_til_3(i1,-1,i2,i3)=rho(  i1-1,i2,i3) - rho_avg - &
                                        1.0/cs_til(i1,i2,i3)**2 * &
                                        ( P(i1-1,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1,-1,i2,i3) &
                                        )
            alpha0_til_3(i1, 0,i2,i3)=rho(  i1+0,i2,i3) - rho_avg - &
                                        1.0/cs_til(i1,i2,i3)**2 * &
                                        ( P(i1+0,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1, 0,i2,i3) &
                                        )
            alpha0_til_3(i1, 1,i2,i3)=rho(  i1+1,i2,i3) - rho_avg - &
                                        1.0/cs_til(i1,i2,i3)**2 * &
                                        ( P(i1+1,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1, 1,i2,i3) &
                                        )
            alpha0_til_3(i1, 2,i2,i3)=rho(  i1+2,i2,i3) - rho_avg - &
                                        1.0/cs_til(i1,i2,i3)**2 * &
                                        ( P(i1+2,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1, 2,i2,i3) &
                                        )
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            !
            alpha0_til_6(i1,-3,i2,i3)=rho(  i1-3,i2,i3)*ei(i1-3,i2,i3) - rhoei_avg - &
                                        DrhoeiDPConsts_til(i1,i2,i3) * &
                                        ( P(i1-3,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1,-3,i2,i3) &
                                        )
            alpha0_til_6(i1,-2,i2,i3)=rho(  i1-2,i2,i3)*ei(i1-2,i2,i3) - rhoei_avg - &
                                        DrhoeiDPConsts_til(i1,i2,i3) * &
                                        ( P(i1-2,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1,-2,i2,i3) &
                                        )
            alpha0_til_6(i1,-1,i2,i3)=rho(  i1-1,i2,i3)*ei(i1-1,i2,i3) - rhoei_avg - &
                                        DrhoeiDPConsts_til(i1,i2,i3) * &
                                        ( P(i1-1,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1,-1,i2,i3) &
                                        )
            alpha0_til_6(i1, 0,i2,i3)=rho(  i1+0,i2,i3)*ei(i1+0,i2,i3) - rhoei_avg - &
                                        DrhoeiDPConsts_til(i1,i2,i3) * &
                                        ( P(i1+0,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1, 0,i2,i3) &
                                        )
            alpha0_til_6(i1, 1,i2,i3)=rho(  i1+1,i2,i3)*ei(i1+1,i2,i3) - rhoei_avg - &
                                        DrhoeiDPConsts_til(i1,i2,i3) * &
                                        ( P(i1+1,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1, 1,i2,i3) &
                                        )
            alpha0_til_6(i1, 2,i2,i3)=rho(  i1+2,i2,i3)*ei(i1+2,i2,i3) - rhoei_avg - &
                                        DrhoeiDPConsts_til(i1,i2,i3) * &
                                        ( P(i1+2,i2,i3) - P_avg(i1,i2,i3) &
                                          - par%C_hydPredfactor*P_red(i1, 2,i2,i3) &
                                        )
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        do j=-reconstruction_order-1,reconstruction_order
          do i1=m1,n1+1
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            alpha0_til_3(i1,j,i2,i3)=rho(  i1+j,i2,i3) - rho_avg - &
                                       1.0/cs_til(i1,i2,i3)**2 * &
                                       ( P(i1+j,i2,i3) - P_avg(i1,i2,i3) &
                                         - par%C_hydPredfactor*P_red(i1,j,i2,i3) &
                                       )
            alpha0_til_6(i1,j,i2,i3)=rho(  i1+j,i2,i3)*ei(i1+j,i2,i3) - rhoei_avg - &
                                       DrhoeiDPConsts_til(i1,i2,i3) * &
                                       ( P(i1+j,i2,i3) - P_avg(i1,i2,i3) &
                                         - par%C_hydPredfactor*P_red(i1,j,i2,i3) &
                                       )
          end do ! i1
        end do ! j
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til: (36)a')
  call timing_start('HYD: Roe1Dflux: alpha_til: (36)b')
!#
# endif
!#
!#
# if (rhd_roe1d_flux_l01 == 2)
!#
  ! --- Do the upwind shift (with i_up) ---
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !DIR$ UNROLL
          do j=-2,2
            alpha_til_3(i1,j,i2,i3)=alpha0_til_3(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
          end do ! j
        end do ! i1
        !
        do i1=m1,n1+1
          !DIR$ UNROLL
          do j=-2,2
            alpha_til_6(i1,j,i2,i3)=alpha0_til_6(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        do j=-reconstruction_order,reconstruction_order
          do i1=m1,n1+1
            alpha_til_3(i1,j,i2,i3)=alpha0_til_3(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
          end do ! i1
          !
          do i1=m1,n1+1
            alpha_til_6(i1,j,i2,i3)=alpha0_til_6(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)
          end do ! i1
        end do ! j
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# else
!#     /* (rhd_roe1d_flux_l01 == 3) */
  ! --- Do the upwind shift (with if-then-else) ---
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          if (i_up_nu(i1,i2,i3) < i1) then
            alpha_til_3(i1,-2,i2,i3)=alpha0_til_3(i1,-2-1,i2,i3)
            alpha_til_3(i1,-1,i2,i3)=alpha0_til_3(i1,-1-1,i2,i3)
            alpha_til_3(i1, 0,i2,i3)=alpha0_til_3(i1, 0-1,i2,i3)
            alpha_til_3(i1, 1,i2,i3)=alpha0_til_3(i1, 1-1,i2,i3)
            alpha_til_3(i1, 2,i2,i3)=alpha0_til_3(i1, 2-1,i2,i3)
            alpha_til_6(i1,-2,i2,i3)=alpha0_til_6(i1,-2-1,i2,i3)
            alpha_til_6(i1,-1,i2,i3)=alpha0_til_6(i1,-1-1,i2,i3)
            alpha_til_6(i1, 0,i2,i3)=alpha0_til_6(i1, 0-1,i2,i3)
            alpha_til_6(i1, 1,i2,i3)=alpha0_til_6(i1, 1-1,i2,i3)
            alpha_til_6(i1, 2,i2,i3)=alpha0_til_6(i1, 2-1,i2,i3)
          else
            alpha_til_3(i1,-2,i2,i3)=alpha0_til_3(i1,-2  ,i2,i3)
            alpha_til_3(i1,-1,i2,i3)=alpha0_til_3(i1,-1  ,i2,i3)
            alpha_til_3(i1, 0,i2,i3)=alpha0_til_3(i1, 0  ,i2,i3)
            alpha_til_3(i1, 1,i2,i3)=alpha0_til_3(i1, 1  ,i2,i3)
            alpha_til_3(i1, 2,i2,i3)=alpha0_til_3(i1, 2  ,i2,i3)
            alpha_til_6(i1,-2,i2,i3)=alpha0_til_6(i1,-2  ,i2,i3)
            alpha_til_6(i1,-1,i2,i3)=alpha0_til_6(i1,-1  ,i2,i3)
            alpha_til_6(i1, 0,i2,i3)=alpha0_til_6(i1, 0  ,i2,i3)
            alpha_til_6(i1, 1,i2,i3)=alpha0_til_6(i1, 1  ,i2,i3)
            alpha_til_6(i1, 2,i2,i3)=alpha0_til_6(i1, 2  ,i2,i3)
          endif
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          if (i_up_nu(i1,i2,i3) < i1) then
            do j=-reconstruction_order,reconstruction_order
              alpha_til_3(i1,j,i2,i3)=alpha0_til_3(i1,j-1,i2,i3)
              alpha_til_6(i1,j,i2,i3)=alpha0_til_6(i1,j-1,i2,i3)
            end do ! j
          else
            do j=-reconstruction_order,reconstruction_order
              alpha_til_3(i1,j,i2,i3)=alpha0_til_3(i1,j-1,i2,i3)
              alpha_til_6(i1,j,i2,i3)=alpha0_til_6(i1,j-1,i2,i3)
            end do ! j
          endif
        end do ! i1
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# endif
!#      /* rhd_roe1d_flux_l01 */
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til: (36)b')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# elif (rhd_roe1d_flux_l01 == 0)
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
  !
  ! --- Compute alpha values (amplitude) in one go (with i_up) ---
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(1) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          !DIR$ UNROLL
          do j=-1,1
            alpha_til_3(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)&
                                     )
            alpha_til_6(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3)*ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)&
                                     )
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          !DIR$ UNROLL
          do j=-2,2
            alpha_til_3(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                    1.0/cs_til(i1,i2,i3)**2 * &
                                    ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
                                      - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) &
                                    )
            alpha_til_6(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3)*ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3)&
                                     )
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        do j=-reconstruction_order,reconstruction_order
          do i1=m1,n1+1
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            alpha_til_3(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                    1.0/cs_til(i1,i2,i3)**2 * &
                                    ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
                                      - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) &
                                    )
            alpha_til_6(i1,j,i2,i3)=rho(i_up_nu(i1,i2,i3)+j,i2,i3)*ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                    rhoei_avg - &
                                    DrhoeiDPConsts_til(i1,i2,i3) * &
                                    ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3) &
                                      - par%C_hydPredfactor*P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) &
                                    )
          end do ! i1
        end do ! j
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# else
!#     /* rhd_roe1d_flux_l01 == 1 */
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
! --- Compute alpha values (amplitude) with if-then-else ---
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          !
          if (i_up_nu(i1,i2,i3) < i1) then
            alpha_til_3(i1,-2,i2,i3)=rho(i1-2-1,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1-2-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-2-1,i2,i3) )
            alpha_til_3(i1,-1,i2,i3)=rho(i1-1-1,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1-1-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-1-1,i2,i3) )
            alpha_til_3(i1, 0,i2,i3)=rho(i1+0-1,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1+0-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 0-1,i2,i3) )
            alpha_til_3(i1, 1,i2,i3)=rho(i1+1-1,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1+1-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 1-1,i2,i3) )
            alpha_til_3(i1, 2,i2,i3)=rho(i1+2-1,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1+2-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 2-1,i2,i3) )
            !
            alpha_til_6(i1,-2,i2,i3)=rho(i1-2-1,i2,i3)*ei(i1-2-1,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1-2-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-2-1,i2,i3) )
            alpha_til_6(i1,-1,i2,i3)=rho(i1-1-1,i2,i3)*ei(i1-1-1,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1-1-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-1-1,i2,i3) )
            alpha_til_6(i1, 0,i2,i3)=rho(i1+0-1,i2,i3)*ei(i1+0-1,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1+0-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 0-1,i2,i3) )
            alpha_til_6(i1, 1,i2,i3)=rho(i1+1-1,i2,i3)*ei(i1+1-1,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1+1-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 1-1,i2,i3) )
            alpha_til_6(i1, 2,i2,i3)=rho(i1+2-1,i2,i3)*ei(i1+2-1,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1+2-1,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 2-1,i2,i3) )
          else ! (i_up_nu(i1,i2,i3) >= i1)
            alpha_til_3(i1,-2,i2,i3)=rho(i1-2  ,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1-2  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-2  ,i2,i3) )
            alpha_til_3(i1,-1,i2,i3)=rho(i1-1  ,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1-1  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-1  ,i2,i3) )
            alpha_til_3(i1, 0,i2,i3)=rho(i1+0  ,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1+0  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 0  ,i2,i3) )
            alpha_til_3(i1, 1,i2,i3)=rho(i1+1  ,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1+1  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 1  ,i2,i3) )
            alpha_til_3(i1, 2,i2,i3)=rho(i1+2  ,i2,i3) - rho_avg - &
                                     1.0/cs_til(i1,i2,i3)**2 * &
                                     ( P(i1+2  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 2  ,i2,i3) )
            !
            alpha_til_6(i1,-2,i2,i3)=rho(i1-2  ,i2,i3)*ei(i1-2  ,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1-2  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-2  ,i2,i3) )
            alpha_til_6(i1,-1,i2,i3)=rho(i1-1  ,i2,i3)*ei(i1-1  ,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1-1  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1,-1  ,i2,i3) )
            alpha_til_6(i1, 0,i2,i3)=rho(i1+0  ,i2,i3)*ei(i1+0  ,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1+0  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 0  ,i2,i3) )
            alpha_til_6(i1, 1,i2,i3)=rho(i1+1  ,i2,i3)*ei(i1+1  ,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1+1  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 1  ,i2,i3) )
            alpha_til_6(i1, 2,i2,i3)=rho(i1+2  ,i2,i3)*ei(i1+2  ,i2,i3) - &
                                     rhoei_avg - &
                                     DrhoeiDPConsts_til(i1,i2,i3) * &
                                     ( P(i1+2  ,i2,i3) - P_avg(i1,i2,i3) &
                                       - par%C_hydPredfactor*P_red(i1, 2  ,i2,i3) )
          endif ! (i_up_nu(i1,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          !
          if (i_up_nu(i1,i2,i3) < i1) then
            do j=-reconstruction_order,reconstruction_order
              alpha_til_3(i1,j,i2,i3)=rho(i1+j-1,i2,i3) - rho_avg - &
                                      1.0/cs_til(i1,i2,i3)**2 * &
                                      ( P(i1+j-1,i2,i3) - P_avg(i1,i2,i3) &
                                        - par%C_hydPredfactor*P_red(i1,j-1,i2,i3) )
              alpha_til_6(i1,j,i2,i3)=rho(i1+j-1,i2,i3)*ei(i1+j-1,i2,i3) - &
                                      rhoei_avg - &
                                      DrhoeiDPConsts_til(i1,i2,i3) * &
                                      ( P(i1+j-1,i2,i3) - P_avg(i1,i2,i3) &
                                        - par%C_hydPredfactor*P_red(i1,j-1,i2,i3) )
            end do ! j
          else ! (i_up_nu(i1,i2,i3) >= i1)
            do j=-reconstruction_order,reconstruction_order
              alpha_til_3(i1,j,i2,i3)=rho(i1+j  ,i2,i3) - rho_avg - &
                                      1.0/cs_til(i1,i2,i3)**2 * &
                                      ( P(i1+j  ,i2,i3) - P_avg(i1,i2,i3) &
                                        - par%C_hydPredfactor*P_red(i1,j  ,i2,i3) )
              alpha_til_6(i1,j,i2,i3)=rho(i1+j  ,i2,i3)*ei(i1+j  ,i2,i3) - &
                                      rhoei_avg - &
                                      DrhoeiDPConsts_til(i1,i2,i3) * &
                                      ( P(i1+j  ,i2,i3) - P_avg(i1,i2,i3) &
                                        - par%C_hydPredfactor*P_red(i1,j  ,i2,i3) )
            end do ! j
          endif ! (i_up_nu(i1,i2,i3))
        end do ! i1
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#      /* rhd_roe1d_flux_t01 */
!#
# endif
!#      /* rhd_roe1d_flux_l01 */
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
  !
  ! --- Perform reconstruction (constant, slope limiter, PP), compute fluxes of partial waves ---
  !
  if (dxb1_avg < 0.0) then
    ! --- Reconstruct for spatially varying grid ---
    !
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_3, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_3)
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_6, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_6)
    !
  else
    ! --- Reconstruct for equidistant grid ---
    !
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_3, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_3)
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_6, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_6)
    !
  endif
  !
  ! --- Compute additional energy flux down an entropy step ---
  if (par%C_hydsDiffVelo > 0.0) then
    call rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                       dxb1_avg, dxb1, &
                       v1, v2, v3, s, T, &
                       lambda_til_nu, rho_til, cs_til, wrho, &
                       flux_rhoei_sDiff)
  else
    flux_rhoei_sDiff=0.0
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop( 'HYD: Roe1Dflux: Fluxalpha', cycles=2)
!#
# endif
!#
  !
else if (par%hdEntropyWaveMode == 'Entropy1') then ! ===============================================
  ! --- Compute entropy-wave contribution from entropy alone ---
  !
  Diff_flag=(par%C_hydsDiffVelo > 0.0)
  !
  if (Diff_flag) then
    ! --- Compute diffusive velocity at cell centers ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+1
          vD(i1,i2,i3)=sqrt(max(par%C_hydsDiffVelo**2,1.0)* v1(i1,i2,i3)**2 + &
                                par%C_hydsDiffVelo**2     *(v2(i1,i2,i3)**2 + &
                                                            v3(i1,i2,i3)**2) )
        end do ! i1
      end do ! i2
    end do ! i3
    !
    ! --- Compute diffusive velocity at cell boundaries ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute Roe averages by weighting with sqrt(rho) ---
          vD_til(i1,i2,i3)=wrho(i1,i2,i3)*(vD(i1-1,i2,i3)-vD(i1,i2,i3)) + vD(i1,i2,i3)
          vD_til(i1,i2,i3)=cs_til(i1,i2,i3) * &
                           (1.0-0.25*min(vD_til(i1,i2,i3)/cs_til(i1,i2,i3)-2.0,0.0)**2)
          vD_til(i1,i2,i3)=max(vD_til(i1,i2,i3)-abs(lambda_til_nu(i1,i2,i3)), 0.0)
        end do ! i1
      end do ! i2
    end do ! i3
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: FluxState')
!#
# endif
!#
  !
  ! --- Reconstruct entropy and compute entropy flux (including extra diffusion if requested) ---
  call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                         dtimeHO, dxb1_avg, dxb1, &
                         i_up_nu, lambda_til_nu, Diff_flag, vD_til, &
                         s, &
                         flux_s)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: FluxState')
!#
# endif
  !
  ! --- Set additional energy flux to zero ---
  flux_rhoei_sDiff=0.0
  !
  ! --- Compute contribution to mass and energy flux ---
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1-1,n1+1
        ! --- Compute additional thermodynamic quantities ---
        oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                      (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
        DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                  (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                  ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
        rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        !
        ! --- Compute the Roe average of the thermodynamic quantities ---
        DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                         (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                          oOcpd_v(i1  )   ) + &
                                          oOcpd_v(i1  )        )
        DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                DrhoeiDsConstPOrhoT_v(i1  )
        rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                 rhoT_v(i1  )   ) + &
                                 rhoT_v(i1  )
        !
        ! --- Remove flux due to average entropy ---
        s_avg=     wl(i1) * s(i1-1,i2,i3) + &
              (1.0-wl(i1))* s(i1  ,i2,i3)
        flux_s(i1,i2,i3)=flux_s(i1,i2,i3) - lambda_til_nu(i1,i2,i3)*s_avg
        !
        ! --- Compute contribution to mass flux ---
        flux_alpha_til_3(i1,i2,i3)=           DrhoDsConstP_til        * flux_s(i1,i2,i3)
        !
        ! --- Compute contribution to energy flux ---
        flux_alpha_til_6(i1,i2,i3)=rhoT_til * DrhoeiDsConstPOrhoT_til * flux_s(i1,i2,i3)
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEntropyWaveMode == 'Entropy2') then ! ===============================================
  ! --- Compute entropy-wave contribution from entropy and pressure ---
  !
  ! --- Compute projection coefficients ---
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1-1,n1+1
        ! --- Compute additional thermodynamic quantities ---
        oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                      (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
      end do ! i1
      !
      do j=-reconstruction_order,reconstruction_order
        !DIR$ SIMD
        do i1=m1,n1+1
          ! --- Compute "simple" Roe sound speed ---
          Psim_til   =rho_til(i1  ,i2,i3)/(sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3)) * &
                           (P(i1-1,i2,i3)/ sqrt_rho(i1-1,i2,i3) + &
                            P(i1  ,i2,i3)/ sqrt_rho(i1  ,i2,i3) )
          Gamma_1_til=wrho(i1,i2,i3)*(Gamma_1(       i1-1,i2,i3)- &
                                      Gamma_1(       i1  ,i2,i3)) + &
                                      Gamma_1(       i1  ,i2,i3)
          cssim_til  =sqrt(Psim_til/rho_til(i1,i2,i3)*Gamma_1_til)
          !
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          !
          ! --- Derive some thermodynamic averages ---
          s_avg  =     wl(i1) * s(i1-1,i2,i3) + &
                  (1.0-wl(i1))* s(i1  ,i2,i3)
          !
          ! --- Compute projection coefficient ---
          alpha_til_3(i1,j,i2,i3)=s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg - &
                                   1.0/DrhoDsConstP_til * &
                                   (1.0/cs_til(i1,i2,i3)**2 - 1.0/cssim_til**2)* &
                                     ( P(    i_up_nu(   i1,i2,i3)   +j,i2,i3) - &
                                       P_avg(           i1,i2,i3) - &
                                       P_red(i1,i_up_nu(i1,i2,i3)-i1+j,i2,i3) &
                                     )
        end do ! i1
      end do ! j
    end do ! i2
  end do ! i3
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
  !
  ! --- Perform reconstruction (constant, slope limiter, PP), compute fluxes of partial waves ---
  !
  if (dxb1_avg < 0.0) then
    ! --- Reconstruct for spatially varying grid ---
    !
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_3, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_s)
    !
  else
    ! --- Reconstruct for equidistant grid ---
    !
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_3, dxb1_avg, &
                             lambda_til_nu, &
                             flux_s)
    !
  endif
  !
  ! --- Compute additional energy flux down an entropy step ---
  if (par%C_hydsDiffVelo > 0.0) then
    call rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                       dxb1_avg, dxb1, &
                       v1, v2, v3, s, T, &
                       lambda_til_nu, rho_til, cs_til, wrho, &
                       flux_rhoei_sDiff)
  else
    flux_rhoei_sDiff=0.0
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop( 'HYD: Roe1Dflux: Fluxalpha')
!#
# endif
  !
  ! --- Compute contribution to mass and energy flux ---
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ SIMD
      do i1=m1-1,n1+1
        ! --- Compute additional thermodynamic quantities ---
        oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                      (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
        DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                  (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                  ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
        rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1,n1+1
        ! --- Compute the Roe average of the thermodynamic quantities ---
        DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                         (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                          oOcpd_v(i1  )   ) + &
                                          oOcpd_v(i1  )        )
        DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                DrhoeiDsConstPOrhoT_v(i1  )
        rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                 rhoT_v(i1  )   ) + &
                                 rhoT_v(i1  )
        !
        ! --- Compute contribution to mass flux ---
        flux_alpha_til_3(i1,i2,i3)=           DrhoDsConstP_til        * flux_s(i1,i2,i3)
        !
        ! --- Compute contribution to energy flux ---
        flux_alpha_til_6(i1,i2,i3)=rhoT_til * DrhoeiDsConstPOrhoT_til * flux_s(i1,i2,i3)
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if ((par%hdEntropyWaveMode == 'Entropy3' ) .or. &
         (par%hdEntropyWaveMode == 'Entropy3M')     ) then ! =======================================
  ! --- Compute entropy-wave contribution from entropy, density, and pressure ---
  !
  ! --- Compute projection coefficients ---
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(1) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do i1=m1,n1+1
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                  DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                  DrhoeiDsConstPOrhoT_v(i1  )
          rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                   rhoT_v(i1  )   ) + &
                                   rhoT_v(i1  )
          !
          ! --- Compute Mach-number weight ---
          wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
          wMach=wMach*(2.0-wMach)
          wTot =wMach
          !
          ! --- Derive some thermodynamic averages ---
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                    (1.0-wl(i1))* s(  i1  ,i2,i3)
          !
          !DIR$ UNROLL
          do j=-1,1
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do i1=m1,n1+1
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                  DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                  DrhoeiDsConstPOrhoT_v(i1  )
          rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                   rhoT_v(i1  )   ) + &
                                   rhoT_v(i1  )
          !
          ! --- Compute Mach-number weight ---
          wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
          wMach=wMach*(2.0-wMach)
          wTot =wMach
          !
          ! --- Derive some thermodynamic averages ---
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                    (1.0-wl(i1))* s(  i1  ,i2,i3)
          !
          !DIR$ UNROLL
          do j=-2,2
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do j=-reconstruction_order,reconstruction_order
          !DIR$ SIMD
          do i1=m1,n1+1
            ! --- Compute the Roe average of the thermodynamic quantities ---
            DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                             (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                              oOcpd_v(i1  )   ) + &
                                              oOcpd_v(i1  )        )
            DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                    DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                    DrhoeiDsConstPOrhoT_v(i1  )
            rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                     rhoT_v(i1  )   ) + &
                                     rhoT_v(i1  )
            !
            ! --- Compute Mach-number weight ---
            wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
            wMach=wMach*(2.0-wMach)
            wTot =wMach
            !
            ! --- Derive some thermodynamic averages ---
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                      (1.0-wl(i1))* s(  i1  ,i2,i3)
            !
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                              ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
                                  (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! i1
        end do ! j
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
  !
  ! --- Perform reconstruction (constant, slope limiter, PP), compute fluxes of partial waves ---
  !
  if (dxb1_avg < 0.0) then
    ! --- Reconstruct for spatially varying grid ---
    !
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_3, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_3)
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_6, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_6)
    !
  else
    ! --- Reconstruct for equidistant grid ---
    !
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_3, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_3)
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_6, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_6)
    !
  endif
  !
  ! --- Compute additional energy flux down an entropy step ---
  if (par%C_hydsDiffVelo > 0.0) then
    call rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                       dxb1_avg, dxb1, &
                       v1, v2, v3, s, T, &
                       lambda_til_nu, rho_til, cs_til, wrho, &
                       flux_rhoei_sDiff)
  else
    flux_rhoei_sDiff=0.0
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop( 'HYD: Roe1Dflux: Fluxalpha', cycles=2)
!#
# endif
!#
  !
else if (par%hdEntropyWaveMode == 'Entropy3Ms') then ! =============================================
  ! --- Compute entropy-wave contribution from entropy, density, and pressure ---
  !
  ! --- Compute projection coefficients ---
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(1) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do i1=m1,n1+1
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                  DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                  DrhoeiDsConstPOrhoT_v(i1  )
          rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                   rhoT_v(i1  )   ) + &
                                   rhoT_v(i1  )
          !
          ! --- Compute entropy step, normalize and limit it and smooth the jump ---
          ds=min(abs(s(i1,i2,i3)-s(i1-1,i2,i3))*(5.0*(dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))), 2.0)
          ds=(-0.25*ds+1.0)*ds
          !
          ! --- Compute Mach-number weight ---
          wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
          wMach=wMach*(2.0-wMach)
          !
          ! --- Compose total weight ---
          wTot =ds + wMach - ds*wMach
          !
          ! --- Derive some thermodynamic averages ---
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                    (1.0-wl(i1))* s(  i1  ,i2,i3)
          !
          !DIR$ UNROLL
          do j=-1,1
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
                                 (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do i1=m1,n1+1
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                  DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                  DrhoeiDsConstPOrhoT_v(i1  )
          rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                   rhoT_v(i1  )   ) + &
                                   rhoT_v(i1  )
          !
          ! --- Compute entropy step, normalize and limit it and smooth the jump ---
          ds=min(abs(s(i1,i2,i3)-s(i1-1,i2,i3))*(5.0*(dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))), 2.0)
          ds=(-0.25*ds+1.0)*ds
          !
          ! --- Compute Mach-number weight ---
          wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
          wMach=wMach*(2.0-wMach)
          !
          ! --- Compose total weight ---
          wTot =ds + wMach - ds*wMach
          !
          ! --- Derive some thermodynamic averages ---
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                    (1.0-wl(i1))* s(  i1  ,i2,i3)
          !
          !DIR$ UNROLL
          do j=-2,2
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
                                 (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! j
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do j=-reconstruction_order,reconstruction_order
          !DIR$ SIMD
          do i1=m1,n1+1
            ! --- Compute the Roe average of the thermodynamic quantities ---
            DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                             (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                              oOcpd_v(i1  )   ) + &
                                              oOcpd_v(i1  )        )
            DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                    DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                    DrhoeiDsConstPOrhoT_v(i1  )
            rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                     rhoT_v(i1  )   ) + &
                                     rhoT_v(i1  )
            !
            ! --- Compute entropy step, normalize and limit it and smooth the jump ---
            ds=min(abs(s(i1,i2,i3)-s(i1-1,i2,i3))*(5.0*(dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))), 2.0)
            ds=(-0.25*ds+1.0)*ds
            !
            ! --- Compute Mach-number weight ---
            wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
            wMach=wMach*(2.0-wMach)
            !
            ! --- Compose total weight ---
            wTot =ds + wMach - ds*wMach
            !
            ! --- Derive some thermodynamic averages ---
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                      (1.0-wl(i1))* s(  i1  ,i2,i3)
            !
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                               (P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                               (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! i1
        end do ! j
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
  !
  ! --- Perform reconstruction (constant, slope limiter, PP), compute fluxes of partial waves ---
  !
  if (dxb1_avg < 0.0) then
    ! --- Reconstruct for spatially varying grid ---
    !
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_3, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_3)
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_6, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_6)
    !
  else
    ! --- Reconstruct for equidistant grid ---
    !
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_3, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_3)
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_6, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_6)
    !
  endif
  !
  ! --- Compute additional energy flux down an entropy step ---
  if (par%C_hydsDiffVelo > 0.0) then
    call rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                       dxb1_avg, dxb1, &
                       v1, v2, v3, s, T, &
                       lambda_til_nu, rho_til, cs_til, wrho, &
                       flux_rhoei_sDiff)
  else
    flux_rhoei_sDiff=0.0
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop( 'HYD: Roe1Dflux: Fluxalpha', cycles=2)
!#
# endif
!#
  !
  ! \label{f90:hdentropywavemode__use2}
else ! === (par%hdEntropyWaveMode == 'Entropy3gMps') ===============================================
  ! --- Compute entropy-wave contribution from entropy, density, and pressure ---
  !
  ! --- Compute projection coefficients ---
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
  select case (reconstruction_order) ! -------------------------------------------------------------
  case(1) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                  DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                  DrhoeiDsConstPOrhoT_v(i1  )
          rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                   rhoT_v(i1  )   ) + &
                                   rhoT_v(i1  )
          !
          ! --- Set total weight ---
          wTot=wLgFluct_til(i1,i2,i3)
          !
          ! --- Derive some thermodynamic averages ---
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                    (1.0-wl(i1))* s(  i1  ,i2,i3)
          !
          ! --- Compute projection coefficients ---
!#
#         if ((rhd_roe1d_flux_l01 == 0) || (rhd_roe1d_flux_l01 == 2))
!#
!!$            !DIR$ UNROLL
!!$            do j=-1,1
!!$              alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
!!$                                                 1.0/cs_til(i1,i2,i3)**2 * &
!!$                                                ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
!!$                                   (1.0-wTot )*(DrhoDsConstP_til * &
!!$                                                ( s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
!!$              alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
!!$                                                 ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
!!$                                                rhoei_avg - &
!!$                                                DrhoeiDPConsts_til(i1,i2,i3) * &
!!$                                                ( P(i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
!!$                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
!!$                                                ( s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
!!$            end do ! j
              alpha_til_3(i1,-1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)-1,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)-1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)-1,i2,i3) - s_avg))
              alpha_til_3(i1, 0,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+0,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)+0,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)+0,i2,i3) - s_avg))
              alpha_til_3(i1, 1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+1,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)+1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)+1,i2,i3) - s_avg))
              !
              alpha_til_6(i1,-1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)-1,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)-1,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)-1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)-1,i2,i3) - s_avg))
              alpha_til_6(i1, 0,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+0,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)+0,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)+0,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)+0,i2,i3) - s_avg))
              alpha_til_6(i1, 1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+1,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)+1,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)+1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)+1,i2,i3) - s_avg))
!#
#         else
!#             /* (rhd_roe1d_flux_l01 == 1 or 3) */
          ! --- Compute projection coefficients ---
          if (i_up_nu(i1,i2,i3) < i1) then
              alpha_til_3(i1,-1,i2,i3)=  wTot  *(rho(i1-1-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1-1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1-1-1,i2,i3) - s_avg))
              alpha_til_3(i1, 0,i2,i3)=  wTot  *(rho(i1+0-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+0-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+0-1,i2,i3) - s_avg))
              alpha_til_3(i1, 1,i2,i3)=  wTot  *(rho(i1+1-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+1-1,i2,i3) - s_avg))
              !
              alpha_til_6(i1,-1,i2,i3)=  wTot  *(rho(i1-1-1,i2,i3)* &
                                                  ei(i1-1-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1-1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1-1-1,i2,i3) - s_avg))
              alpha_til_6(i1, 0,i2,i3)=  wTot  *(rho(i1+0-1,i2,i3)* &
                                                  ei(i1+0-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+0-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+0-1,i2,i3) - s_avg))
              alpha_til_6(i1, 1,i2,i3)=  wTot  *(rho(i1+1-1,i2,i3)* &
                                                  ei(i1+1-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+1-1,i2,i3) - s_avg))
          else
              alpha_til_3(i1,-1,i2,i3)=  wTot  *(rho(i1-1  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1-1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1-1  ,i2,i3) - s_avg))
              alpha_til_3(i1, 0,i2,i3)=  wTot  *(rho(i1+0  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+0  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+0  ,i2,i3) - s_avg))
              alpha_til_3(i1, 1,i2,i3)=  wTot  *(rho(i1+1  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+1  ,i2,i3) - s_avg))
              !
              alpha_til_6(i1,-1,i2,i3)=  wTot  *(rho(i1-1  ,i2,i3)* &
                                                  ei(i1-1  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1-1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1-1  ,i2,i3) - s_avg))
              alpha_til_6(i1, 0,i2,i3)=  wTot  *(rho(i1+0  ,i2,i3)* &
                                                  ei(i1+0  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+0  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+0  ,i2,i3) - s_avg))
              alpha_til_6(i1, 1,i2,i3)=  wTot  *(rho(i1+1  ,i2,i3)* &
                                                  ei(i1+1  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+1  ,i2,i3) - s_avg))
          endif
!#
#         endif
!#              /* rhd_roe1d_flux_l01 */
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case(2) ! (reconstruction_order) .......................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
!!$            ! --- Compute additional thermodynamic quantities ---
!!$            oOcpd_l=(rho(i1-1,i2,i3)*T(i1-1,i2,i3)*(Gamma_3(i1-1,i2,i3)-1.0))/ &
!!$                      (P(i1-1,i2,i3)*Gamma_1(i1-1,i2,i3))
!!$            oOcpd_r=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
!!$                      (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
!!$            DrhoDsConstP_til=-rho_til(i1,i2,i3) * (wrho(i1,i2,i3)*(oOcpd_l-oOcpd_r) + oOcpd_r)
!!$            DrhoeiDsConstpOrhoT_l  =1.0 - &
!!$                                    (rho(i1-1,i2,i3)*ei(i1-1,i2,i3)/P(i1-1,i2,i3)+1.0) * &
!!$                                    ((Gamma_3(i1-1,i2,i3)-1.0)/Gamma_1(i1-1,i2,i3))
!!$            DrhoeiDsConstpOrhoT_r  =1.0 - &
!!$                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
!!$                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
!!$            DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_l-DrhoeiDsConstPOrhoT_r) + &
!!$                                                    DrhoeiDsConstPOrhoT_r
!!$            rhoT_l  =rho(i1-1,i2,i3)*T(i1-1,i2,i3)
!!$            rhoT_r  =rho(i1  ,i2,i3)*T(i1  ,i2,i3)
!!$            rhoT_til=wrho(i1,i2,i3)*(rhoT_l-rhoT_r) + rhoT_r
          !
          ! --- Compute the Roe average of the thermodynamic quantities ---
          DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                           (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                            oOcpd_v(i1  )   ) + &
                                            oOcpd_v(i1  )        )
          DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                  DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                  DrhoeiDsConstPOrhoT_v(i1  )
          rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                   rhoT_v(i1  )   ) + &
                                   rhoT_v(i1  )
          !
          ! --- Set total weight ---
          wTot=wLgFluct_til(i1,i2,i3)
          !
          ! --- Derive some thermodynamic averages ---
          rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)
          rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                    (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
          s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                    (1.0-wl(i1))* s(  i1  ,i2,i3)
          !
          ! --- Compute projection coefficients ---
!#
#         if ((rhd_roe1d_flux_l01 == 0) || (rhd_roe1d_flux_l01 == 2))
!#
!!$            !DIR$ UNROLL
!!$            do j=-2,2
!!$              alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
!!$                                                 1.0/cs_til(i1,i2,i3)**2 * &
!!$                                                (P( i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
!!$                                   (1.0-wTot )*(DrhoDsConstP_til * &
!!$                                                 (s(i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
!!$              alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
!!$                                                 ei(i_up_nu(i1,i2,i3)+j,i2,i3) - rhoei_avg - &
!!$                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
!!$                                                (P( i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) +&
!!$                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
!!$                                                (s( i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
!!$            end do ! j
              alpha_til_3(i1,-2,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)-2,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)-2,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)-2,i2,i3) - s_avg))
              alpha_til_3(i1,-1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)-1,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)-1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)-1,i2,i3) - s_avg))
              alpha_til_3(i1, 0,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+0,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)+0,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)+0,i2,i3) - s_avg))
              alpha_til_3(i1, 1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+1,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)+1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)+1,i2,i3) - s_avg))
              alpha_til_3(i1, 2,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+2,i2,i3) - rho_avg - &
                                                 1.0/cs_til(i1,i2,i3)**2 * &
                                                (P( i_up_nu(i1,i2,i3)+2,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(DrhoDsConstP_til * &
                                                 (s(i_up_nu(i1,i2,i3)+2,i2,i3) - s_avg))
              !
              alpha_til_6(i1,-2,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)-2,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)-2,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)-2,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)-2,i2,i3) - s_avg))
              alpha_til_6(i1,-1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)-1,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)-1,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)-1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)-1,i2,i3) - s_avg))
              alpha_til_6(i1, 0,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+0,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)+0,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)+0,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)+0,i2,i3) - s_avg))
              alpha_til_6(i1, 1,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+1,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)+1,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)+1,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)+1,i2,i3) - s_avg))
              alpha_til_6(i1, 2,i2,i3)= wTot  *(rho(i_up_nu(i1,i2,i3)+2,i2,i3)* &
                                                 ei(i_up_nu(i1,i2,i3)+2,i2,i3) - rhoei_avg - &
                                                 DrhoeiDPConsts_til(i1,i2,i3) * &
                                                (P( i_up_nu(i1,i2,i3)+2,i2,i3) - P_avg(i1,i2,i3)))+&
                                   (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                (s( i_up_nu(i1,i2,i3)+2,i2,i3) - s_avg))
!#
#         else
!#             /* (rhd_roe1d_flux_l01 == 1 or 3) */
          if (i_up_nu(i1,i2,i3) < i1) then
              alpha_til_3(i1,-2,i2,i3)=  wTot  *(rho(i1-2-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1-2-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1-2-1,i2,i3) - s_avg))
              alpha_til_3(i1,-1,i2,i3)=  wTot  *(rho(i1-1-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1-1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1-1-1,i2,i3) - s_avg))
              alpha_til_3(i1, 0,i2,i3)=  wTot  *(rho(i1+0-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+0-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+0-1,i2,i3) - s_avg))
              alpha_til_3(i1, 1,i2,i3)=  wTot  *(rho(i1+1-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+1-1,i2,i3) - s_avg))
              alpha_til_3(i1, 2,i2,i3)=  wTot  *(rho(i1+2-1,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+2-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+2-1,i2,i3) - s_avg))
              !
              alpha_til_6(i1,-2,i2,i3)=  wTot  *(rho(i1-2-1,i2,i3)* &
                                                  ei(i1-2-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1-2-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1-2-1,i2,i3) - s_avg))
              alpha_til_6(i1,-1,i2,i3)=  wTot  *(rho(i1-1-1,i2,i3)* &
                                                  ei(i1-1-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1-1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1-1-1,i2,i3) - s_avg))
              alpha_til_6(i1, 0,i2,i3)=  wTot  *(rho(i1+0-1,i2,i3)* &
                                                  ei(i1+0-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+0-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+0-1,i2,i3) - s_avg))
              alpha_til_6(i1, 1,i2,i3)=  wTot  *(rho(i1+1-1,i2,i3)* &
                                                  ei(i1+1-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+1-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+1-1,i2,i3) - s_avg))
              alpha_til_6(i1, 2,i2,i3)=  wTot  *(rho(i1+2-1,i2,i3)* &
                                                  ei(i1+2-1,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+2-1,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+2-1,i2,i3) - s_avg))
          else
              alpha_til_3(i1,-2,i2,i3)=  wTot  *(rho(i1-2  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1-2  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1-2  ,i2,i3) - s_avg))
              alpha_til_3(i1,-1,i2,i3)=  wTot  *(rho(i1-1  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1-1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1-1  ,i2,i3) - s_avg))
              alpha_til_3(i1, 0,i2,i3)=  wTot  *(rho(i1+0  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+0  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+0  ,i2,i3) - s_avg))
              alpha_til_3(i1, 1,i2,i3)=  wTot  *(rho(i1+1  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+1  ,i2,i3) - s_avg))
              alpha_til_3(i1, 2,i2,i3)=  wTot  *(rho(i1+2  ,i2,i3) - rho_avg - &
                                                  1.0/cs_til(i1,i2,i3)**2 * &
                                                 ( P(i1+2  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(DrhoDsConstP_til * &
                                                  (s(i1+2  ,i2,i3) - s_avg))
              !
              alpha_til_6(i1,-2,i2,i3)=  wTot  *(rho(i1-2  ,i2,i3)* &
                                                  ei(i1-2  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1-2  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1-2  ,i2,i3) - s_avg))
              alpha_til_6(i1,-1,i2,i3)=  wTot  *(rho(i1-1  ,i2,i3)* &
                                                  ei(i1-1  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1-1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1-1  ,i2,i3) - s_avg))
              alpha_til_6(i1, 0,i2,i3)=  wTot  *(rho(i1+0  ,i2,i3)* &
                                                  ei(i1+0  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+0  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+0  ,i2,i3) - s_avg))
              alpha_til_6(i1, 1,i2,i3)=  wTot  *(rho(i1+1  ,i2,i3)* &
                                                  ei(i1+1  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+1  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+1  ,i2,i3) - s_avg))
              alpha_til_6(i1, 2,i2,i3)=  wTot  *(rho(i1+2  ,i2,i3)* &
                                                  ei(i1+2  ,i2,i3) - rhoei_avg - &
                                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                                 (P( i1+2  ,i2,i3) - P_avg(i1,i2,i3))) +&
                                    (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                                 (s( i1+2  ,i2,i3) - s_avg))
          endif
!#
#         endif
!#              /* rhd_roe1d_flux_l01 */
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case default ! (reconstruction_order) ..................................................
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ! --- Compute additional thermodynamic quantities ---
          oOcpd_v(i1)=(rho(i1  ,i2,i3)*T(i1  ,i2,i3)*(Gamma_3(i1  ,i2,i3)-1.0))/ &
                        (P(i1  ,i2,i3)*Gamma_1(i1  ,i2,i3))
          DrhoeiDsConstpOrhoT_v(i1)=1.0 - &
                                    (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3)+1.0) * &
                                    ((Gamma_3(i1  ,i2,i3)-1.0)/Gamma_1(i1  ,i2,i3))
          rhoT_v(i1)=rho(i1  ,i2,i3)*T(i1  ,i2,i3)
        end do ! i1
        !
        do j=-reconstruction_order,reconstruction_order
          !DIR$ SIMD
          do i1=m1,n1+1
            ! --- Compute the Roe average of the thermodynamic quantities ---
            DrhoDsConstP_til=-rho_til(i1,i2,i3) * &
                             (wrho(i1,i2,i3)*(oOcpd_v(i1-1)- &
                                              oOcpd_v(i1  )   ) + &
                                              oOcpd_v(i1  )        )
            DrhoeiDsConstPOrhoT_til=wrho(i1,i2,i3)*(DrhoeiDsConstPOrhoT_v(i1-1)- &
                                                    DrhoeiDsConstPOrhoT_v(i1  )   ) + &
                                                    DrhoeiDsConstPOrhoT_v(i1  )
            rhoT_til=wrho(i1,i2,i3)*(rhoT_v(i1-1)- &
                                     rhoT_v(i1  )   ) + &
                                     rhoT_v(i1  )
            !
            ! --- Set total weight ---
            wTot=wLgFluct_til(i1,i2,i3)
            !
            ! --- Derive some thermodynamic averages ---
            rho_avg  =     wl(i1) * rho(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)
            rhoei_avg=     wl(i1) * rho(i1-1,i2,i3)*ei(i1-1,i2,i3) + &
                      (1.0-wl(i1))* rho(i1  ,i2,i3)*ei(i1  ,i2,i3)
            s_avg    =     wl(i1) * s(  i1-1,i2,i3) + &
                      (1.0-wl(i1))* s(  i1  ,i2,i3)
            !
            ! --- Compute projection coefficients ---
            alpha_til_3(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3) - rho_avg - &
                                               1.0/cs_til(i1,i2,i3)**2 * &
                                             ( P( i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(DrhoDsConstP_til * &
                                              (s( i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
            alpha_til_6(i1,j,i2,i3)=  wTot  *(rho(i_up_nu(i1,i2,i3)+j,i2,i3)* &
                                               ei(i_up_nu(i1,i2,i3)+j,i2,i3) - &
                                              rhoei_avg - &
                                              DrhoeiDPConsts_til(i1,i2,i3) * &
                                             ( P( i_up_nu(i1,i2,i3)+j,i2,i3) - P_avg(i1,i2,i3))) + &
                                 (1.0-wTot )*(rhoT_til * DrhoeiDsConstPOrhoT_til * &
                                              (s( i_up_nu(i1,i2,i3)+j,i2,i3) - s_avg))
          end do ! i1
        end do ! j
      end do ! i2
    end do ! i3
    !
  end select ! (reconstruction_order) --------------------------------------------------------------
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: alpha_til (36)')
!#
# endif
!#
  !
  ! --- Perform reconstruction (constant, slope limiter, PP), compute fluxes of partial waves ---
  !
  if (dxb1_avg < 0.0) then
    ! --- Reconstruct for spatially varying grid ---
    !
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
#   endif
!#
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_3, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_3)
    call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                           reconstruction_order, reconstruction_type, &
                           dtimeHO, &
                           SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                           alpha_til_6, dxb1, &
                           i_up_nu, lambda_til_nu, &
                           flux_alpha_til_6)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop( 'HYD: Roe1Dflux: Fluxalpha', cycles=2)
!#
#   endif
!#
    !
  else
    ! --- Reconstruct for equidistant grid ---
    !
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('HYD: Roe1Dflux: FluxalphaEq')
!#
#   endif
!#
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_3, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_3)
    call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                             reconstruction_order, reconstruction_type, &
                             dtimeHO, &
                             SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                             alpha_til_6, dxb1_avg, &
                             lambda_til_nu, &
                             flux_alpha_til_6)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('HYD: Roe1Dflux: FluxalphaEq', cycles=2)
!#
#   endif
!#
    !
  endif
  !
  ! --- Compute additional energy flux down an entropy step ---
  if (par%C_hydsDiffVelo > 0.0) then
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start( 'HYD: Roe1Dflux: sDiff')
!#
#   endif
!#
    call rhd_hyd_sDiff(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       SlopeRed_flag, .false., SlopeRed, SlopeCon, &
                       dxb1_avg, dxb1, &
                       v1, v2, v3, s, T, &
                       lambda_til_nu, rho_til, cs_til, wrho, &
                       flux_rhoei_sDiff)
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop( 'HYD: Roe1Dflux: sDiff')
!#
#   endif
!#
  !
  else
    flux_rhoei_sDiff=0.0
  endif
  !
endif ! === (par%hdEntropyWaveMode) ================================================================
!
end subroutine rhd_hyd_RoeWavealpha36


!----------**********************-------------------------------------------------------------------
subroutine rhd_hyd_RoeWavealpha45(m1,n1, m2,n2, m3,n3, nghost1, &
                                  reconstruction_order, reconstruction_type, &
                                  dtimeHO, dxb1_avg, dxb1, wl, &
                                  rho, v1, v2, v3, &
                                  wrho, rho_til, v2_til, v3_til, cs_til, &
                                  SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                  i_up_nu, lambda_til_nu, &
                                  flux_rho, &
                                  flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
                                  flux_alpha_til_4, flux_alpha_til_5, &
                                  flux_rhov2, flux_rhov3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_RoeWavealpha45 ('rhd_hydrodynamics_Roe_Wave_alpha_4_5')
!
! PURPOSE:
!   Compute 1D transversal-momentum fluxes.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_RoeWavealpha45(m1,n1, m2,n2, m3,n3, nghost1, &
!                               reconstruction_order, reconstruction_type, &
!                               dtimeHO, dxb1_avg, dxb1, wl, &
!                               rho, v1, v2, v3, &
!                               wrho, rho_til, v2_til, v3_til, cs_til, &
!                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                               i_up_nu, lambda_til_nu, &
!                               flux_rho, &
!                               flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
!                               flux_alpha_til_4, flux_alpha_til_5, &
!                               flux_rhov2, flux_rhov3)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtimeHO:      ('delta_time_HighOrder') real, high-order time step (might be zero)
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real vector, cell-boundary step size, 1. coordinate
!   wl:           ('weight_left') real vector, geometrical weights due to non-equidistant grid
!   rho:          ('rho') real 3D array, density
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   wrho:         ('weight_rho') real 3D array, Roe weight from square root of density
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe-averaged sound speed at cell boundary
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   i_up_nu:      ('index_upwind_null') 3D real array: indices of upwind direction,
!                 entropy-wave characteristic
!   lambda_til_nu: ('lambda_tilde_null') 3D real array with entropy-wave velocities
!                  at cell boundaries
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_alpha_til_1: ('flux_alpha_tilde_1') 3D real array with flux of sound- characteristic
!   flux_alpha_til_2: ('flux_alpha_tilde_2') 3D real array with flux of sound+ characteristic
!   flux_alpha_til_3: ('flux_alpha_tilde_3') 3D real array with flux of entropy-wave characteristic
!
! INPUT/OUTPUT:
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   i_up_nu:      ('index_upwind_null') 3D real array: indices of upwind direction,
!                 entropy-wave characteristic
!   lambda_til_nu: ('lambda_tilde_null') 3D real array with entropy-wave velocities
!                  at cell boundaries
!
! OUTPUT:
!   flux_alpha_til_4: ('flux_alpha_tilde_4') 3D real array with flux of transversal momentum 2
!   flux_alpha_til_5: ('flux_alpha_tilde_5') 3D real array with flux of transversal momentum 3
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   v2_avg:       ('velocity_2_average') real 3D array, simple average of velocity 2 at cell bnd.
!   v3_avg:       ('velocity_3_average') real 3D array, simple average of velocity 3 at cell bnd.
!   flux_alpha_til_123: ('flux_alpha_tilde_1_2_3') real with sum of flux_alpha_1,2,3 resulting in
!                       the total Roe mass flux correction
!   Diff_flag:    ('Diffusion_flag') logical indicating that extra diffusion should be added
!   vD:           ('velocity_Diffusion') 3D array with additional diffusion velocity in cells
!   vD_til:       ('velocity_Diffusion_tilde') 3D array with add. diffusion velocity at cell bnd.
!
! ROUTINES:
!   rhd_hyd_FluxState:     Compute 1D flux from state vector for single quantity
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!   Some arrays are recomputed.
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2001-01-17 (B.F. Uppsala) Put into own module
!   2013-02-23 (B.F. Lyon) Extract from rhd_hyd_RoeWave1Dflux and put into own routine
!   2014-12-02 (B.F.) Add parameter dxb1; remove parameter xb1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_rec_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtimeHO, dxb1_avg
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1  :n1+nghost1,&
                m2          :n2        ,&
                m3          :n3          ), &
                           intent(in)   :: rho, v1, v2, v3
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: wrho, &
                                           rho_til, cs_til
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(inout):: v2_til, v3_til, lambda_til_nu
integer, &
      dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(inout):: i_up_nu
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: flux_rho, &
                                           flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: flux_alpha_til_4, flux_alpha_til_5, &
                                           flux_rhov2, flux_rhov3
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: v2_avg, v3_avg, flux_alpha_til_123
logical                                 :: Diff_flag
real, dimension(m1-1:n1+1, &
                m2  :n2  , &
                m3  :n3     )           :: vD
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3    )            :: vD_til
!---------------------------------------------------------------------------------------------------
!
! \label{f90:hdtransvelomode__use1}
if (par%hdTransVeloMode == 'Normal') then
  ! --- Old standard method of computing flux_alpha_til_4,5 with new routine -----------------------
  !
  Diff_flag=(par%C_hydvDiffVelo > 0.0)
  !
  if (Diff_flag) then
    ! --- Compute diffusive velocity at cell centers ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+1
          vD(i1,i2,i3)=sqrt(max(par%C_hydvDiffVelo**2,1.0)* v1(i1,i2,i3)**2 + &
                                par%C_hydvDiffVelo**2     *(v2(i1,i2,i3)**2 + &
                                                            v3(i1,i2,i3)**2) )
        end do ! i1
      end do ! i2
    end do ! i3
    !
    ! --- Compute diffusive velocity at cell boundaries ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute Roe averages by weighting with sqrt(rho) ---
          vD_til(i1,i2,i3)=wrho(i1,i2,i3)*(vD(i1-1,i2,i3)-vD(i1,i2,i3)) + vD(i1,i2,i3)
          vD_til(i1,i2,i3)=cs_til(i1,i2,i3) * &
                           (1.0-0.25*min(vD_til(i1,i2,i3)/cs_til(i1,i2,i3)-2.0,0.0)**2)
          vD_til(i1,i2,i3)=max(vD_til(i1,i2,i3)-abs(lambda_til_nu(i1,i2,i3)), 0.0)
        end do ! i1
      end do ! i2
    end do ! i3
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: FluxState')
!#
# endif
!#
  !
  ! --- Compute fluxes (including extra diffusion if requested) ---
  call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         dtimeHO, dxb1_avg, dxb1, &
                         i_up_nu, lambda_til_nu, Diff_flag, vD_til, &
                         v2, &
                         flux_alpha_til_4)
  call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         dtimeHO, dxb1_avg, dxb1, &
                         i_up_nu, lambda_til_nu, Diff_flag, vD_til, &
                         v3, &
                         flux_alpha_til_5)
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: FluxState', cycles=2)
  call timing_start('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
  !
  ! === Back and combination projection => fluxes ===
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        ! --- Subtract average velocity contribution from fluxes ---
        v2_avg=     wl(i1) * v2( i1-1,i2,i3) + &
               (1.0-wl(i1))* v2( i1  ,i2,i3)
        v3_avg=     wl(i1) * v3( i1-1,i2,i3) + &
               (1.0-wl(i1))* v3( i1  ,i2,i3)
        flux_alpha_til_4(i1,i2,i3)=flux_alpha_til_4(i1,i2,i3) - lambda_til_nu(i1,i2,i3)*v2_avg
        flux_alpha_til_5(i1,i2,i3)=flux_alpha_til_5(i1,i2,i3) - lambda_til_nu(i1,i2,i3)*v3_avg
        !
        ! --- Compose fluxes of conserved quantities from wave fluxes ---
        flux_alpha_til_123=flux_alpha_til_1(i1,i2,i3) + &
                           flux_alpha_til_2(i1,i2,i3) + &
                           flux_alpha_til_3(i1,i2,i3)
        flux_rhov2(i1,i2,i3)= v2_til(i1,i2,i3) *flux_alpha_til_123 + &
                             rho_til(i1,i2,i3) *flux_alpha_til_4(i1,i2,i3) + &
                             (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * v2(i1-1,i2,i3) + &
                             ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * v2(i1  ,i2,i3)
        flux_rhov3(i1,i2,i3)= v3_til(i1,i2,i3) *flux_alpha_til_123 + &
                             rho_til(i1,i2,i3) *flux_alpha_til_5(i1,i2,i3) + &
                             (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * v3(i1-1,i2,i3) + &
                             ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * v3(i1  ,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
!
else ! (par%hdTransVeloMode /= 'Normal')
  ! --- Alternative way to compute fluxes of transversal velocities --------------------------------
  !
  ! --- Determine upwind direction and flow speed from mass flux ---
  if ((par%hdTransVeloMode == 'CA1') .or. &
      (par%hdTransVeloMode == 'CA3')     ) then
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
!#
#         if (rhd_hyd_upwind_p01 == 0)
!#
          if (flux_rho(i1,i2,i3) > 0.0) then
            i_up_nu(i1,i2,i3)=i1-1
          else
            i_up_nu(i1,i2,i3)=i1
          endif
!#
#         else
!#             /* rhd_hyd_upwind_p01 */
          i_up_nu(i1,i2,i3)=i1 + int(sign(0.6,-flux_rho(i1,i2,i3))-0.5)
!#
#         endif
!#              /* rhd_hyd_upwind_p01 */
          !
          lambda_til_nu(i1,i2,i3)=flux_rho(i1,i2,i3)/rho(i_up_nu(i1,i2,i3),i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
  else
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
!#
#         if (rhd_hyd_upwind_p01 == 0)
!#
          if (flux_rho(i1,i2,i3) > 0.0) then
            i_up_nu(i1,i2,i3)=i1-1
          else
            i_up_nu(i1,i2,i3)=i1
          endif
!#
#         else
!#             /* rhd_hyd_upwind_p01 */
          i_up_nu(i1,i2,i3)=i1 + int(sign(0.6,-flux_rho(i1,i2,i3))-0.5)
!#
#         endif
!#              /* rhd_hyd_upwind_p01 */
          !
          lambda_til_nu(i1,i2,i3)=flux_rho(i1,i2,i3)/rho_til(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
  endif
  !
  ! \label{f90:c_hydvdiffvelo__use1}
  Diff_flag=(par%C_hydvDiffVelo > 0.0)
  !
  if (Diff_flag) then
    ! --- Compute diffusive velocity at cell centers ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-1,n1+1
          vD(i1,i2,i3)=sqrt(max(par%C_hydvDiffVelo**2,1.0)* v1(i1,i2,i3)**2 + &
                                par%C_hydvDiffVelo**2     *(v2(i1,i2,i3)**2 + &
                                                            v3(i1,i2,i3)**2) )
        end do ! i1
      end do ! i2
    end do ! i3
    !
    ! --- Compute diffusive velocity at cell boundaries ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute Roe averages by weighting with sqrt(rho) ---
          vD_til(i1,i2,i3)=wrho(i1,i2,i3)*(vD(i1-1,i2,i3)-vD(i1,i2,i3)) + vD(i1,i2,i3)
          vD_til(i1,i2,i3)=cs_til(i1,i2,i3) * &
                           (1.0-0.25*min(vD_til(i1,i2,i3)/cs_til(i1,i2,i3)-2.0,0.0)**2)
          vD_til(i1,i2,i3)=max(vD_til(i1,i2,i3)-abs(lambda_til_nu(i1,i2,i3)), 0.0)
        end do ! i1
      end do ! i2
    end do ! i3
  endif
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: fluxes')
  call timing_start('HYD: Roe1Dflux: FluxState')
!#
# endif
!#
  !
  ! --- Compute fluxes (including extra diffusion if requested) ---
  call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         dtimeHO, dxb1_avg, dxb1, &
                         i_up_nu, lambda_til_nu, Diff_flag, vD_til, &
                         v2, &
                         flux_alpha_til_4)
  call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         dtimeHO, dxb1_avg, dxb1, &
                         i_up_nu, lambda_til_nu, Diff_flag, vD_til, &
                         v3, &
                         flux_alpha_til_5)
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: FluxState', cycles=2)
  call timing_start('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
  !
  ! === Back projection and combination => fluxes ===
  !
  if ((par%hdTransVeloMode == 'CA1') .or. &
      (par%hdTransVeloMode == 'CA2')     ) then
    ! --- Compute transversal momentum fluxes from mass flux      ---
    ! --- and recomputed transversal velocities v2_til and v3_til ---
    !
    do i3=m3,n3
      do i2=m2,n2
!       --- Due to a bug in ifort 12.1.1.256 Build 20111011, the following loop should ---
!       --- not be vectorized, if OpenMP parallelization is not activated.             ---
!       --- DIR$ NOVECTOR
        do i1=m1,n1+1
          ! --- Recompute transversal velocities at boundary ---
          if (abs(lambda_til_nu(i1,i2,i3)/cs_til(i1,i2,i3)) < 1.0E-7) then
            v2_til(i1,i2,i3)=0.0
            v3_til(i1,i2,i3)=0.0
          else
            !rho_til(i1,i2,i3)=flux_rho(        i1,i2,i3)/lambda_til_nu(i1,i2,i3)
            v2_til(i1,i2,i3)=flux_alpha_til_4(i1,i2,i3)/lambda_til_nu(i1,i2,i3)
            v3_til(i1,i2,i3)=flux_alpha_til_5(i1,i2,i3)/lambda_til_nu(i1,i2,i3)
          endif
          !
          ! --- Subtract contribution of mean velocities from fluxes ---
          v2_avg   =     wl(i1) * v2( i1-1,i2,i3) + &
                    (1.0-wl(i1))* v2( i1  ,i2,i3)
          v3_avg   =     wl(i1) * v3( i1-1,i2,i3) + &
                    (1.0-wl(i1))* v3( i1  ,i2,i3)
          flux_alpha_til_4(i1,i2,i3)=flux_alpha_til_4(i1,i2,i3)-lambda_til_nu(i1,i2,i3)*v2_avg
          flux_alpha_til_5(i1,i2,i3)=flux_alpha_til_5(i1,i2,i3)-lambda_til_nu(i1,i2,i3)*v3_avg
          !
          ! --- Compute transversal momentum fluxes ---
          flux_rhov2(i1,i2,i3)=v2_til(i1,i2,i3) *flux_rho(i1,i2,i3)
          flux_rhov3(i1,i2,i3)=v3_til(i1,i2,i3) *flux_rho(i1,i2,i3)
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !
  else ! (par%hdTransVeloMode)
    ! --- Compute transversal momentum fluxes from mass flux and diffusive momentum flux ---
    !
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- flux_rhov1(i1,i2,i3)=flux_rhov1(i1,i2,i3) + &
          ! ---                      rho_til(i1,i2,i3) * flux_v1_vDiff(   i1,i2,i3)
          ! ---  rho_til(i1,i2,i3)*  v1_til(i1,i2,i3)  * flux_v1_vDiff(   i1,i2,i3) + &
          ! --- Subtract contribution of mean velocities from fluxes ---
          v2_avg   =     wl(i1) * v2( i1-1,i2,i3) + &
                    (1.0-wl(i1))* v2( i1  ,i2,i3)
          v3_avg   =     wl(i1) * v3( i1-1,i2,i3) + &
                    (1.0-wl(i1))* v3( i1  ,i2,i3)
          flux_alpha_til_4(i1,i2,i3)=flux_alpha_til_4(i1,i2,i3)-lambda_til_nu(i1,i2,i3)*v2_avg
          flux_alpha_til_5(i1,i2,i3)=flux_alpha_til_5(i1,i2,i3)-lambda_til_nu(i1,i2,i3)*v3_avg
          !
          ! --- Compute transversal momentum fluxes ---
          flux_rhov2(i1,i2,i3)=v2_til( i1,i2,i3) *flux_rho(        i1,i2,i3) + &
                               rho_til(i1,i2,i3) *flux_alpha_til_4(i1,i2,i3)
          flux_rhov3(i1,i2,i3)=v3_til( i1,i2,i3) *flux_rho(        i1,i2,i3) + &
                               rho_til(i1,i2,i3) *flux_alpha_til_5(i1,i2,i3)
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !
  endif ! (par%hdTransVeloMode)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
  !
endif ! (par%hdTransVeloMode)
!
end subroutine rhd_hyd_RoeWavealpha45


!----------*****************------------------------------------------------------------------------
subroutine rhd_hyd_Fluxrhoei(m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order, &
                             wl, rho, ei, v1, v2, v3, P, Gamma_1, s, dTdei, P_red, P_avg, &
                             wrho, wLgFluct_til, &
                             rho_til, eip_til, v1_til, v2_til, v3_til, cs_til, v1abs_til, &
                             flux_rho, flux_rhoei_sDiff, &
                             flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
                             flux_alpha_til_4, flux_alpha_til_5, flux_alpha_til_6, &
                             flux_rhoei)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Fluxrhoei ('rhd_hydrodynamics_Flux_rho_energy_internal')
!
! PURPOSE:
!   Compute 1D internal energy flux from the fluxes of individual waves.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Fluxrhoei(m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order, &
!                          wl, rho, ei, v1, v2, v3, P, Gamma_1, s, dTdei, &
!                          P_red, P_avg, &
!                          wrho, wLgFluct_til, &
!                          rho_til, eip_til, v1_til, v2_til, v3_til, cs_til, v1abs_til, &
!                          flux_rho, flux_rhoei_sDiff, &
!                          flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
!                          flux_alpha_til_4, flux_alpha_til_5, flux_alpha_til_6, &
!                          flux_rhoei)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   rho:          ('rho') real 3D array, density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   dTdei:        ('dT_dei') real 3D array, temperature derivative (constant density) on model grid
!   P_red:        ('Pressure_reduced') real 4D array, hydrostatic pressure correction
!   P_avg:        ('Pressure_average') real 3D array, simple average of pressure at cell boundary
!   wrho:         ('weight_rho') real 3D array, Roe weight from square root of density
!   wLgFluct_til: ('weight_Large_Fluctuations_tilde') real 3D array with weight from
!                 fluctuation amplitude
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   eip_til:      ('energy_internal_pressure_tilde') real 3D array with Roe-averaged enthalpy
!                 at cell boundaries
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe-averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe-averaged sound speed at cell boundary
!   v1abs_til:    ('velocity_1_absolute_tilde') real 3D array, Roe-averaged absolute velocity 1
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhoei_sDiff: ('flux_rho_energy_internal_s_Diffusion') real 3D array, diffusive
!                     internal energy flux
!   flux_alpha_til_1: ('flux_alpha_tilde_1') 3D real array with flux of sound- characteristic
!   flux_alpha_til_2: ('flux_alpha_tilde_2') 3D real array with flux of sound+ characteristic
!   flux_alpha_til_3: ('flux_alpha_tilde_3') 3D real array with flux of entropy-wave characteristic
!   flux_alpha_til_4: ('flux_alpha_tilde_4') 3D real array with flux of transversal momentum 2
!   flux_alpha_til_5: ('flux_alpha_tilde_5') 3D real array with flux of transversal momentum 3
!   flux_alpha_til_6: ('flux_alpha_tilde_6') 3D real array with flux of ionization-energy charact.
!
! OUTPUT:
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   eip_l:        ('energy_internal_pressure_left') real, enthalpy in left cell
!   eip_r:        ('energy_internal_pressure_right') real, enthalpy in left cell
!   eikp_l:       ('energy_internal_kinetic_pressure_left') real, enthalpy+kinetic energy
!                 in left cell
!   eikp_r:       ('energy_internal_kinetic_pressure_right') real, enthalpy+kinetic energy
!                 in right cell
!   rhov1_l:      ('rho_velocity_1_left') real, 1-momentum change in left cell
!   rhov1_r:      ('rho_velocity_1_right') real, 1-momentum change in right cell
!   ds:           ('delta_entropy') real, weight derived from entropy step between adjacent
!                 grid cells
!   dP:           ('delta_pressure') real, weight derived from pressure step between grid cells
!   dPg:          ('delta_pressure_gravity') real, weight derived from gravity-corrected
!                 pressure step between grid adjacent cells
!   wMach:        ('weight_Mach') real, derived from Mach number
!   wTot:         ('weight_Total') real, total weight
!   eps_TinyPressure: ('epsilon_Tiny_Pressure') real parameter (1.0E-06) with tiny relative pressure
!                     to prevent division by zero
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute total - kinetic energy flux:
!     flux_rhoei=flux_rhoeik - ek_til*flux_rho
!     fek       =0.5 * (v1_til**2 + v2_til**2 + v3_til**2) * flux_rho
!   The kinetic energy flux has to be added explicitly in the energy update step.
!   There are various ways to mix the usual Roe-average update ("Normal") and
!   an update that uses consistent advection base fluxes (e.g., "CA1").
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2012-06-06 (B.F. Lyon) Branches for hdEnthalpyAvgMode='CA1M', 'CA2M'
!   2012-06-07 (B.F.) Regroup computation of wave fluxes 1,2,3,6. New input parameter Gamma_3.
!                     New branch for hdEntropyWaveMode='Entropy1'
!   2012-06-12 (B.F.) Branches for hdEnthalpyAvgMode='CA1s', 'CA2s';
!                     compute vD_til correctly for each case
!   2012-06-14 (B.F.) New parameter dTdei, use it for CA1s and CA2s.
!                     New branches for hdEnthalpyAvgMode='CA1g', 'CA2g'
!   2012-06-15 (B.F.) New branches for hdEnthalpyAvgMode='CA1p', 'CA2p'
!   2012-06-22 (B.F.) New branch for hdEntropyWaveMode='Entropy2'
!   2012-07-11 (B.F.) New branch for hdEntropyWaveMode='Entropy3'
!   2012-10-22 (B.F.) Branch for hdEnthalpyAvgMode='CA1Ms'; modify 'CA1p' and 'CA2p'
!   2012-10-31 (B.F.) Branch for hdEnthalpyAvgMode='CA1gMs';
!                     change definition of dP in 'CA1g' and similar
!   2012-11-05 (B.F.) 'CA1gMs' -> 'CA1gMps', 'Entropy3gMs' -> 'Entropy3gMps'
!   2013-02-25 (B.F.) Extracted from rhd_hyd_RoeWave1Dflux and put into own routine
!   2013-02-26 (B.F.) New input parameter wLgFluct_til
!   2014-08-28 (B.F. Uppsala) Branch for hdEnthalpyAvgMode='CA5gMps'
!   2014-08-30 (B.F. Uppsala) Branch for hdEnthalpyAvgMode='CA1gMps_Pred'
!   2014-09-03 (B.F. Uppsala) Branch for hdEnthalpyAvgMode='CA6gMps'
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1  :n1+nghost1,&
                m2          :n2        ,&
                m3          :n3          ), &
                           intent(in)   :: rho, ei, v1, v2, v3, P, Gamma_1, s, dTdei
real, dimension(m1-1:n1+2, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: P_red
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3    ), &
                           intent(in)   :: P_avg, wLgFluct_til
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3    ), &
                           intent(in)   :: wrho, &
                                           rho_til, eip_til, v1_til, v2_til, v3_til, cs_til, &
                                           v1abs_til
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: flux_rho, flux_rhoei_sDiff, &
                                           flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
                                           flux_alpha_til_4, flux_alpha_til_5, flux_alpha_til_6
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: flux_rhoei
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: eip_ll, eip_l, eip_r, eip_rr, &
                                           eikp_l, eikp_r, rhov1_l, rhov1_r, &
                                           P_l, P_r, logP_l, logP_r, &
                                           ds, dP, dPg, wMach, wTot, P_avg0, logP_avg0, &
                                           Gamma_1_avg, Gamma_1_ratio, eip_0, eipScal_l, eipScal_r,&
                                           eip_avg
real, dimension(m1-1:n1+1)              :: logP
real, parameter                         :: eps_TinyPressure=1.0E-06, eps_TinylogPressure=1.0E-6
!---------------------------------------------------------------------------------------------------
!
! \label{f90:hdenthalpyavgmode__use1}
if (par%hdEnthalpyAvgMode == 'Normal') then ! ------------------------------------------------------
  ! --- Compute energy flux with standard Roe averaging of enthalpy flux ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * &
            (ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
             0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * &
            (ei(i1,i2,i3) + P( i1,i2,i3)/rho(i1,i2,i3) + &
             0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
!!$      fek(i1,i2,i3)=0.5 * (v1_til(i1,i2,i3)**2 + v2_til(i1,i2,i3)**2 + v3_til(i1,i2,i3)**2) * &
!!$                    flux_rho(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1') then ! ----------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            (     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
            (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1M') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux ---
  ! --- Turn back to standard Roe averaging for large Mach numbers   ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute Mach-number weight ---
        wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
        wMach=wMach*(2.0-wMach)
        !
        ! --- Compose total weight ---
        wTot=wMach
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
               wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1Ms') then ! --------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux                ---
  ! --- Turn back to standard Roe averaging for large entropy jumps or Mach numbers ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute entropy step, normalize and limit it and smooth the jump ---
        ds=min(abs(s(i1,i2,i3)-s(i1-1,i2,i3))*(5.0*(dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))), 2.0)
        ds=(-0.25*ds+1.0)*ds
        !
        ! --- Compute Mach-number weight ---
        wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
        wMach=wMach*(2.0-wMach)
        !
        ! --- Compose total weight ---
        wTot =ds + wMach - ds*wMach
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1s') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux ---
  ! --- Turn back to standard Roe averaging for large entropy jumps  ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute entropy step, normalize and limit it and smooth the jump ---
        ds=min(abs(s(i1,i2,i3)-s(i1-1,i2,i3))*(5.0*(dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))), 2.0)
        ds=(-0.25*ds+1.0)*ds
        !
        ! --- Compose total weight ---
        wTot=ds
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1p') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux       ---
  ! --- Turn back to standard Roe averaging for large pressure differences ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute non-corrected pressure step, normalize and limit it and smooth the jump ---
        dP   =abs(P(i1-1,i2,i3)-P(i1,i2,i3))/ &
              (   P(i1-1,i2,i3)+P(i1,i2,i3))
        dP   =min(4.0*max(dP-0.5,0.0), 1.0)
        ! --- Possibly: dP   =dP**2 * (3.0-2.0*dP)
        dP   =dP**2 * (2.0-dP)
        !
        ! --- Compose total weight ---
        wTot=dP
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1g') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux                              ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure corrected P difference ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute corrected pressure step, normalize and limit it and smooth the jump ---
        dPg  =eps_TinyPressure*(P(i1,i2,i3)+P(i1-1,i2,i3))
        dPg  =(abs((P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) - (P(i1,i2,i3)-P_red(i1,0,i2,i3)))+dPg)/ &
              (   ( P(i1-1,i2,i3)                     +  P(i1,i2,i3)                   )+dPg)
        dPg  =min(25.0*dPg, 1.0)
        dPg  =dPg * (2.0-dPg)
        !
        ! --- Compose total weight ---
        wTot=dPg
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
  ! \label{f90:hdenthalpyavgmode__use2}
else if (par%hdEnthalpyAvgMode == 'CA1gMps') then ! ------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux                             ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure corrected P difference --
  ! --- and large Mach number, pressure difference, and entropy difference                       ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Set total weight ---
        wTot=wLgFluct_til(i1,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA1gMps_Pred') then ! -------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux                             ---
  ! --- Turn back to standard Roe averaging for large hydrostatic-pressure corrected P difference --
  ! --- and large Mach number, pressure difference, and entropy difference                       ---
  ! --- Used hydrostatic-pressure corrected P for the average                                    ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_l  =ei(i1-1,i2,i3) + (P( i1-1,i2,i3)-P_red(i1,-1,i2,i3))/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + (P( i1  ,i2,i3)-P_red(i1, 0,i2,i3))/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Set total weight ---
        wTot=wLgFluct_til(i1,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eip_l-eip_r)*(rhov1_l-rhov1_r)) + &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA2') then ! ----------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy+kinetic_energy flux ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy+kinetic_energy and mass flux in left and right neighbor cell ---
        eikp_l =ei(i1-1,i2,i3) + &
                P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
                (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        eikp_r =ei(i1  ,i2,i3) + &
                P( i1  ,i2,i3)/rho(i1  ,i2,i3) + &
                (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            (     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
            (     wl(i1) * eikp_l  + (1.0-wl(i1)) * eikp_r )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA2M') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy+kinetic_energy flux ---
  ! --- Turn back to standard Roe averaging for large Mach numbers                  ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy+kinetic_energy and mass flux in left and right neighbor cell ---
        eikp_l =ei(i1-1,i2,i3) + &
                P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
                (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        eikp_r =ei(i1  ,i2,i3) + &
                P( i1  ,i2,i3)/rho(i1  ,i2,i3) + &
                (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute Mach-number weight ---
        wMach=min(v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3), 1.0)
        wMach=wMach*(2.0-wMach)
        !
        ! --- Compose total weight ---
        wTot=wMach
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eikp_l  + (1.0-wl(i1)) * eikp_r ) + &
             wTot * &
               wl(i1)*(1.0-wl(i1)) * (eikp_l-eikp_r)*(rhov1_l-rhov1_r))
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA2s') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy+kinetic_energy flux ---
  ! --- Turn back to standard Roe averaging for large entropy jumps                 ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy+kinetic_energy and mass flux in left and right neighbor cell ---
        eikp_l =ei(i1-1,i2,i3) + &
                P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
                (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        eikp_r =ei(i1  ,i2,i3) + &
                P( i1  ,i2,i3)/rho(i1  ,i2,i3) + &
                (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute entropy step, normalize and limit it and smooth the jump ---
        ds=min(abs(s(i1,i2,i3)-s(i1-1,i2,i3))*(5.0*(dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))), 2.0)
        ds=(-0.25*ds+1.0)*ds
        !
        ! --- Compose total weight ---
        wTot=ds
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eikp_l  + (1.0-wl(i1)) * eikp_r ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eikp_l-eikp_r)*(rhov1_l-rhov1_r))
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA2p') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy+kinetic_energy flux   ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure correction ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy+kinetic_energy and mass flux in left and right neighbor cell ---
        eikp_l =ei(i1-1,i2,i3) + &
                P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
                (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        eikp_r =ei(i1  ,i2,i3) + &
                P( i1  ,i2,i3)/rho(i1  ,i2,i3) + &
                (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute non-corrected pressure step, normalize and limit it and smooth the jump ---
        dP=abs(P(i1-1,i2,i3)-P(i1,i2,i3))/ &
           (   P(i1-1,i2,i3)+P(i1,i2,i3))
        dP=min(10.0*dP, 2.0)
        dP=(-0.25*dP+1.0)*dP
        !
        ! --- Compose total weight ---
        wTot=dP
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eikp_l  + (1.0-wl(i1)) * eikp_r ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eikp_l-eikp_r)*(rhov1_l-rhov1_r))
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA2g') then ! ---------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy+kinetic_energy flux   ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure correction ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy+kinetic_energy and mass flux in left and right neighbor cell ---
        eikp_l =ei(i1-1,i2,i3) + &
                P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
                (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        eikp_r =ei(i1  ,i2,i3) + &
                P( i1  ,i2,i3)/rho(i1  ,i2,i3) + &
                (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                         (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                         (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Compute non-corrected pressure step, normalize and limit it and smooth the jump ---
        dP=eps_TinyPressure*(P(i1,i2,i3)+P(i1-1,i2,i3))
        dP=(abs((P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) - (P(i1,i2,i3)-P_red(i1,0,i2,i3)))+dP)/ &
           (abs((P(i1-1,i2,i3)                   ) - (P(i1,i2,i3)                  ))+dP)
        dP=min(10.0*dP, 2.0)
        dP=(-0.25*dP+1.0)*dP
        !
        ! --- Compose total weight ---
        wTot=dP
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
             (     wl(i1) * eikp_l  + (1.0-wl(i1)) * eikp_r ) + &
             wTot * &
             wl(i1)*(1.0-wl(i1)) * (eikp_l-eikp_r)*(rhov1_l-rhov1_r))
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA3') then ! ----------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux ---
  ! --- Use eip_til (and ek_til) for all averaged energies           ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
              eip_til(i1,i2,i3)                                   * flux_rho(        i1,i2,i3) + &
            (- v1_til(i1,i2,i3)*cs_til(i1,i2,i3))                 * flux_alpha_til_1(i1,i2,i3) + &
            (  v1_til(i1,i2,i3)*cs_til(i1,i2,i3))                 * flux_alpha_til_2(i1,i2,i3) - &
              eip_til(i1,i2,i3)                                   * flux_alpha_til_3(i1,i2,i3) + &
              rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                * flux_alpha_til_4(i1,i2,i3) + &
                                  v3_til(i1,i2,i3)                * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3)
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA4') then ! ----------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            ((     wl(i1)  * (ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3))) + &
             ((1.0-wl(i1)) * (ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3))) ) * &
                                                                    flux_rho(        i1,i2,i3) + &
            (- v1_til(i1,i2,i3)*cs_til(i1,i2,i3))                 * flux_alpha_til_1(i1,i2,i3) + &
            (  v1_til(i1,i2,i3)*cs_til(i1,i2,i3))                 * flux_alpha_til_2(i1,i2,i3) - &
              eip_til(i1,i2,i3)                                   * flux_alpha_til_3(i1,i2,i3) + &
              rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                * flux_alpha_til_4(i1,i2,i3) + &
                                  v3_til(i1,i2,i3)                * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
            (      wl(i1)  *  rho(i1-1,i2,i3)*v1(i1-1,i2,i3)  + &
              (1.0-wl(i1)) *  rho(i1  ,i2,i3)*v1(i1  ,i2,i3)   ) * &
            (      wl(i1)  * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                               (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                               (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) )  + &
              (1.0-wl(i1)) * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                               (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                               (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA5gMps') then ! ------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux                             ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure corrected P difference --
  ! --- and large Mach number, pressure difference, and entropy difference                       ---
  ! --- Use 3rd order interpolation for eip (only for equidistant grid, so far)                  ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        eip_ll =ei(i1-2,i2,i3) + P( i1-2,i2,i3)/rho(i1-2,i2,i3)
        eip_l  =ei(i1-1,i2,i3) + P( i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P( i1  ,i2,i3)/rho(i1  ,i2,i3)
        eip_rr =ei(i1+1,i2,i3) + P( i1+1,i2,i3)/rho(i1+1,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        !
        ! --- Set total weight ---
        wTot=wLgFluct_til(i1,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
             (1.0-wTot) * &
               ((     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * &
               (-(1.0/12.0)*(eip_ll+eip_rr) + (7.0/12.0)*(eip_l+eip_r)) ) + &
             (     wTot) * &
               (     wl(i1)  * eip_l*rhov1_l + &
                (1.0-wl(i1)) * eip_r*rhov1_r    ) +  &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        ! --- Older ---
        !     (     wl(i1) * eip_l   + (1.0-wl(i1)) * eip_r  )  +  &
        !     (-(1.0/12.0)*eip_ll + (7.0/12.0)*eip_l + (7.0/12.0)*eip_r - (1.0/12.0)*eip_rr) ) + &
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA6gMps') then ! ------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux.                            ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure corrected P difference --
  ! --- and large Mach number, pressure difference, and entropy difference.                      ---
  ! --- Interpolate linearly in ((eip-eip_0)/P^a)(log(P))                                        ---
  ! --- to get small-fluctuation cell-boundary enthalpy.                                         ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1-1,n1+1
        ! --- Compute the logarithm of the pressure ---
        logP(i1)=alog(P(i1,i2,i3))
      end do ! i1
      !
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        Gamma_1_avg  =0.5*(Gamma_1(i1-1,i2,i3)+ Gamma_1(i1,i2,i3))
        Gamma_1_ratio=(Gamma_1_avg-1.0)/Gamma_1_avg
        P_l    =P(i1-1,i2,i3)
        P_r    =P(i1  ,i2,i3)
        logP_l =logP(i1-1)
        logP_r =logP(i1  )
        eip_l  =ei(i1-1,i2,i3) + P_l/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P_r/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        eip_0  =0.5 * (eip_l+eip_r - (1.0/Gamma_1_ratio*(P_l/rho(i1-1,i2,i3) + &
                                                         P_r/rho(i1  ,i2,i3)    )))
        P_avg0   =min(max(P_avg(i1,i2,i3), min(P_l,P_r)), max(P_l,P_r))
        logP_avg0=alog(P_avg0)
        ! --- Compute: eipScal_l=(eip_l-eip_0)/(P_l/P_avg0)**Gamma_1_ratio ---
        ! ---          eipScal_r=(eip_r-eip_0)/(P_r/P_avg0)**Gamma_1_ratio ---
        eipScal_l=(eip_l-eip_0)/exp(Gamma_1_ratio*(logP_l-logP_avg0))
        eipScal_r=(eip_r-eip_0)/exp(Gamma_1_ratio*(logP_r-logP_avg0))
        !
        ! --- Compute average enthalpy ---
        eip_avg=((abs(logP_avg0-logP_l)+    eps_TinylogPressure) / &
                 (abs(logP_r   -logP_l)+2.0*eps_TinylogPressure)    ) * &
                (eipScal_r-eipScal_l) + eipScal_l + eip_0
        !
        ! --- Set total weight ---
        wTot=wLgFluct_til(i1,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
             (1.0-wTot) * &
               (     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * eip_avg + &
             (    wTot) * &
               (     wl(i1)  * eip_l*rhov1_l + &
                (1.0-wl(i1)) * eip_r*rhov1_r    ) +  &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
else if (par%hdEnthalpyAvgMode == 'CA7gMps') then ! ------------------------------------------------
  ! --- Use consistent-advection centering for average enthalpy flux                             ---
  ! --- Turn back to standard Roe averaging for large hydrostatic pressure corrected P difference --
  ! --- and large Mach number, pressure difference, and entropy difference                       ---
  ! --- Interpolate linearly in eip(log(P)) to get small-fluctuation cell-boundary enthalpy      ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1-1,n1+1
        ! --- Compute the logarithm of the pressure ---
        logP(i1)=alog(P(i1,i2,i3))
      end do ! i1
      !
      do i1=m1,n1+1
        !
        ! --- Compute enthalpy and mass flux in left and right neighbor cell ---
        logP_l =logP(i1-1)
        logP_r =logP(i1  )
        eip_l  =ei(i1-1,i2,i3) + P(i1-1,i2,i3)/rho(i1-1,i2,i3)
        eip_r  =ei(i1  ,i2,i3) + P(i1  ,i2,i3)/rho(i1  ,i2,i3)
        rhov1_l=rho(i1-1,i2,i3)*v1(i1-1,i2,i3)
        rhov1_r=rho(i1  ,i2,i3)*v1(i1  ,i2,i3)
        logP_avg0=min(max(alog(P_avg(i1,i2,i3)), min(logP_l,logP_r)), max(logP_l,logP_r))
        !
        ! --- Compute average enthalpy ---
        eip_avg=((abs(logP_avg0-logP_l)+    eps_TinylogPressure) / &
                 (abs(logP_r   -logP_l)+2.0*eps_TinylogPressure)    ) * &
                 (eip_r-eip_l) + &
                eip_l
        !
        ! --- Set total weight ---
        wTot=wLgFluct_til(i1,i2,i3)
        !
        ! --- Compute total - kinetic energy flux: flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
                                                                    flux_rhoei_sDiff(i1,i2,i3) + &
             (1.0-wTot) * &
               (     wl(i1) * rhov1_l + (1.0-wl(i1)) * rhov1_r) * eip_avg + &
             (    wTot) * &
               (     wl(i1)  * eip_l*rhov1_l + &
                (1.0-wl(i1)) * eip_r*rhov1_r    ) +  &
            (     wl(i1) *rhov1_l) * &
            (0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            ((1.0-wl(i1))*rhov1_r) * &
            (0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !
!else ! (par%hdEnthalpyAvgMode) --------------------------------------------------------------------
!  stop 'Unknown hdEnthalpyAvgMode'
endif ! (par%hdEnthalpyAvgMode) --------------------------------------------------------------------
!
end subroutine rhd_hyd_Fluxrhoei


!----------*********************--------------------------------------------------------------------
subroutine rhd_hyd_RoeWave1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
                                 bound_l, bound_r, reconstruction_order, reconstruction_type, &
                                 dtime, dxb1_avg, dxb1, wl, &
                                 rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, &
                                 cs, s, T, dTdei, sqrt_rho, &
                                 OutFlx_flag, grav_flag, phic_grav, phib_grav, &
                                 rho_til, v1_til, v2_til, v3_til, &
                                 flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_RoeWave1Dflux ('rhd_hydrodynamics_Roe_Wave_1D_flux')
!
! PURPOSE:
!   Compute 1D Roe fluxes with wave reconstructions.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_RoeWave1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
!                              bound_l, bound_r, reconstruction_order, reconstruction_type, &
!                              dtime, dxb1_avg, dxb1, wl, &
!                              rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, cs, s, T, dTdei, sqrt_rho,&
!                              OutFlx_flag, grav_flag, phic_grav, phib_grav, &
!                              rho_til, v1_til, v2_til, v3_til, &
!                              flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   bound_l:      ('boundary_left_1') integer, left boundary type for dimension 1,
!                 values (constants defined in rhd_action_module):
!                    bound_None, bound_reflective, bound_periodic, bound_constant,
!                    bound_closed, bound_closed2, &
!                    bound_inoutflow, bound_inoutflow2, bound_inoutflow3, &
!                    bound_closedbottom, bound_closedbottom2,
!                    bound_transmitting, bound_transmitting2, bound_transmitting3
!   bound_r:      ('boundary_right_1') integer, right boundary type for dimension 1,
!                 values (constants defined in rhd_action_module):
!                    bound_None, bound_reflective, bound_periodic, bound_constant,
!                    bound_closed, bound_closed2,
!                    bound_transmitting, bound_transmitting2, bound_transmitting3
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtime:        ('delta_time') real, time step to be used
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real vector, cell-boundary step size, 1. coordinate
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   Radius:       ('Radius') real 3D array with radius (for central potential case)
!   rho:          ('rho') real 3D array, density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   Gamma_3:      ('Gamma_3') real 3D array, 3rd adiabatic coefficient on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   T:            ('temperature') real 3D array, temperature on model grid
!   dTdei:        ('dT_dei') real 3D array, temperature derivative (constant density) on model grid
!   sqrt_rho:     ('square_root_rho') real 3D array, square root of density on model grid
!   OutFlx_flag:  ('Output_Flux_flag') logical, compute acoustic flux fep
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!   phic_grav:    ('phi_center_gravitation') real, dimension(:,:,:), cell centered grav. potential
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   gc_grav:      ('gravity_center_gravity') real 3D array, gravitational acceleration at
!                 cell centers
!
! OUTPUT:
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe-averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!
! LOCAL VARIABLES:
!   cs_til:       ('constant_sound_tilde') real 3D array, Roe-averaged sound speed at cell boundary
!   P_red:        ('Pressure_reduced') real 4D array, hydrostatic pressure correction
!   P_avg:        ('Pressure_average') real 3D array, simple average of pressure at cell boundary
!   rho_avg:      ('rho_average') real 3D array, simple average of density at cell boundary
!   rhoei_avg:    ('rho_energy_internal_average') real 3D array, simple average of internal energy
!                 at cell boundary
!
! ROUTINES:
!   rhd_hyd_Fluxrhoei:     Compute 1D internal energy flux from the fluxes of individual waves
!   rhd_hyd_Pred:          Compute reduced pressure as gravity correction for Roe solver
!   rhd_hyd_RoeWavealpha12: Compute 1D Roe sound-wave fluxes
!   rhd_hyd_RoeWavealpha36: Compute 1D Roe entropy-wave and ionization-energy fluxes
!   rhd_hyd_RoeWavealpha45: Compute 1D Roe transversal-momentum fluxes
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!   It is assumed that there are no ghost zones present in "orthogonal directions":
!   model%nghost2=0, model%nghost3=0
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2001-01-17 (B.F. Uppsala) Put into own module
!   2001-01-21 (B.F.) Check if grid is equidistant and call simplified routines if true
!   2001-01-25 (B.F.) fep -> fep(i1,i2,i3), case(reconstruction_order)
!   2001-02-15 (B.F.) lambda_til_nu removed in input list (assumed: lambda_til_nu=v1_til)
!   2001-02-16 (B.F.) P_red now computed in routine
!   2001-02-17 (B.F.) Entropy fix now done in routine
!   2001-02-18 (B.F.) Compute q_avg and q_til locally
!   2001-03-20 (B.F.) Test: apply entropy fix to "0"-characteristic, too
!   2002-04-11 (B.F.) lambda_til -> v1_til in all cases
!   2002-10-25 (B.F.) Bugfix in computation of dxb1_avg and dxb1_var
!   2002-10-27 (B.F.) Slope reduction removed
!   2002-11-13 (B.F.) No entropy fix for  "0"-characteristic
!   2002-12-29 (B.F.) Slope reduction in case of strong density contrast
!   2002-12-22 (B.F.) Slope reduction in case of strong density contrast controlled with C_SlopeRed
!   2003-05-06 (B.F.) Slope reduction also for Minmod (towards Constant)
!   2007-02-01 (B.F. Lyon) Removal of P_red from alpha_til_3 and alpha_til_6,
!                     New pressure reduction scheme: default6
!   2007-02-12 (B.F.) P_red put back into alpha_til_3 and alpha_til_6
!   2008-03-17 (B.F.) Parameters C_ReconstructiondTimeFactor, C_hydPredfactor added
!   2008-06-12 (B.F.) Branch without entropy fix
!   2008-08-19 (B.F.) Parameter C_ReconstructiondTimeFactor outcommented
!   2010-06-09 (B.F.) Parameter C_dxb1_maxvar=0.001 -> 0.01
!   2011-07-12 (B.F.) Set dtimeHO to zero for RungeKutta2/3
!   2011-07-17 (B.F.) dxb1_avg als parameter
!   2011-07-20 (B.F.) Rearrangement of terms in flux formulae; flux_alpha_til_123 becomes scalar
!   2011-08-08 (B.F.) Test branch for alternative way to compute fluxes of transversal velocities
!   2011-10-27 (B.F.) Bugfix: Set all fluxes except flux_rhov1 explicitly to 0 for closed bnds (V),
!                             Don't set Roe fluxes 1 and 2 to 0 for closed bnds (IV)
!   2011-11-14 (B.F.) Remove expansion-wave slope-reduction branch
!   2011-11-18 (B.F.) Reconstruction more continuous with C_RecContShift and C_RecContSteep
!   2012-02-17 (B.F.) Centrifugal correction for gravity correction
!   2012-03-23 (B.F.) New options for hdtransvelomode: 'CA3' and 'CA4'
!   2012-03-28 (B.F.) rho_til becomes output parameter,
!                     rename routine rhd_hyd_Roe1Dflux -> rhd_hyd_RoeWave1Dflux
!   2012-04-03 (B.F.) New input arrays: s, T. Compute energy flux down an entropy gradient
!   2012-04-13 (B.F.) New branch that uses rhd_hyd_FluxState for 'Normal'
!   2012-04-16 (B.F.) New branch that uses rhd_hyd_FluxState also for 'CA' cases
!                     Renamed options: 'CA?' -> 'CA?old', 'CA?New' -> 'CA?'
!   2012-04-17 (B.F.) Scalar Mach -> array MachEst_til (with factor 0.5),
!                     put back lambda_til_nu again, compute P_red earlier,
!                     new input parameter grav_flag,
!                     Increased array size of SlopeRed, SlopeCon
!   2012-04-20 (B.F.) Some experiments with flux_alpha_til_1,2
!   2012-05-04 (B.F.) csRed_til
!   2012-05-07 (B.F.) Different formulas for MachEst_til, csRed_til not used in wave speeds
!   2012-05-09 (B.F.) csRed_til only reduces velocity fluctuation contribution
!   2012-05-18 (B.F.) Treatment of Roe fluxes 1 and 2 the same for both closed bnds (IV)
!   2012-05-21 (B.F.) Special pressure treatment (mask_bound) at closed boundaries (III) removed
!   2012-05-22 (B.F.) Compute fep in all cases (hdtransvelomode)
!   2012-06-05 (B.F.) Old branches 'NormalOld', 'CA1,2,3,4Old' removed;
!                     added branches for hdEnthalpyAvgMode instead
!   2012-06-06 (B.F.) Branches for hdEnthalpyAvgMode='CA1M', 'CA2M'
!   2012-06-07 (B.F.) Regroup computation of wave fluxes 1,2,3,6. New input parameter Gamma_3.
!                     New branch for hdEntropyWaveMode='Entropy1'
!   2012-06-11 (B.F.) Remove C_SlopeRed from parameter list
!   2012-06-12 (B.F.) Branches for hdEnthalpyAvgMode='CA1s', 'CA2s';
!                     compute vD_til correctly for each case
!   2012-06-14 (B.F.) New parameter dTdei, use it for CA1s and CA2s.
!                     New branches for hdEnthalpyAvgMode='CA1g', 'CA2g'
!   2012-06-15 (B.F.) New branches for hdEnthalpyAvgMode='CA1p', 'CA2p'
!   2012-06-22 (B.F.) New branch for hdEntropyWaveMode='Entropy2'
!   2012-07-11 (B.F.) New branch for hdEntropyWaveMode='Entropy3'
!   2012-10-05 (B.F.) Limiting term for vD added
!   2012-10-08 (B.F.) Bugfix in limiting term for vD (now for vD_til)
!   2012-10-16 (B.F.) C_hydsDiffVelo, C_hydvDiffVelo -> C_hydsDiffVelo**2, C_hydvDiffVelo**2
!   2012-10-22 (B.F.) Branch for hdEnthalpyAvgMode='CA1Ms'; modify 'CA1p' and 'CA2p'
!                     wMach: linear -> quadratic;
!                     branch for hdEntropyWaveMode='Entropy3Ms'
!   2012-10-31 (B.F.) Branch for hdEnthalpyAvgMode='CA1gMs';
!                     branch for hdEntropyWaveMode='Entropy3gMs';
!                     change definition of dP in 'CA1g' and similar
!   2012-11-05 (B.F.) 'CA1gMs' -> 'CA1gMps', 'Entropy3gMs' -> 'Entropy3gMps'
!   2013-02-22 (B.F.) More timing statistics markers; wrho becomes array; rearranged Roe averages
!   2013-02-23 (B.F.) Split and put rhd_hyd_RoeWavealpha12 into own routine,
!                     split and put rhd_hyd_RoeWavealpha45 into own routine
!   2013-02-24 (B.F.) Split and put rhd_hyd_RoeWavealpha36 into own routine.
!   2013-02-25 (B.F.) Split and put rhd_hyd_Pred into own routine,
!                     split and put rhd_hyd_Fluxrhoei into own routine.
!   2013-02-26 (B.F.) Compute wLgFluct_til in this routine
!   2013-02-27 (B.F.) New parameter OutFlx_flag
!   2013-04-26 (B.F.) MachEst_til becomes scalar, remove computation of cssim_til
!   2014-09-21 (B.F. Uppsala) Set flux_rhoei_sDiff explicitly to zero for closed boundaries
!   2014-10-31 (B.F.) Modify definition of wMach for wLgFluct_til; set SlopeCon=wLgFluct_til
!   2014-11-20 (B.F.) New formula with C_RecContShift and C_RecContSteep
!   2014-12-02 (B.F.) Add parameter dxb1
!   2014-12-03 (B.F.) Remove parameter xb1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_rec_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           bound_l, bound_r, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime, dxb1_avg
logical,                   intent(in)   :: OutFlx_flag, grav_flag
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1  :n1+nghost1,&
                m2          :n2        ,&
                m3          :n3          ), &
                           intent(in)   :: rho, ei, v1, v2, v3, &
                                           P, Gamma_1, Gamma_3, cs, s, T, dTdei, sqrt_rho, &
                                           phic_grav
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2        :n2          ,&
                m3        :n3            ), &
                           intent(in)   :: phib_grav
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: rho_til, v1_til, v2_til, v3_til, &
                                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_rhoei, fep
!
! --- Local variables ---
integer                                 :: i1, i2, i3, j, nc
real                                    :: dtimeHO, &
                                           ei_til, Gamma_1_til, Psim_til, &
                                           mask_up_mi, mask_up_pl, mask_up_nu, &
                                           flux_alpha_til_123, vm1, vp0, &
                                           ds, dP, dPg, wMach, MachEst_til
logical                                 :: SlopeRed_flag, SlopeCon_flag, Diff_flag
!
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3    )            :: P_til, eip_til, v1abs_til, wrho, &
                                           cs_til, DrhoeiDPConsts_til, &
                                           lambda_til_mi, lambda_til_pl, lambda_til_nu, &
                                           flux_rhoei_sDiff, &
                                           csRed_til, &
                                           flux_alpha_til_1, flux_alpha_til_2, &
                                           flux_alpha_til_3, flux_alpha_til_4, &
                                           flux_alpha_til_5, flux_alpha_til_6
integer, &
      dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3  )              :: i_up_mi, i_up_pl, i_up_nu
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     )           :: P_avg, wLgFluct_til, SlopeRed, SlopeCon
!
real, dimension(m1-1:n1+2, &
                -reconstruction_order-1: &
                 reconstruction_order, &
                m2  :n2  , &
                m3  :n3     )           :: P_red
!
real, parameter                         :: eps_TinyPressure=1.0E-06
!---------------------------------------------------------------------------------------------------
!
! === Compute gravity correction for pressure ======================================================
!
! --- Compute reduced gravitational potential: Int((rho*dphi/dz)*dz) ---
! --- and linearly averaged cell-boundary pressure                   ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: P_red')
!#
# endif
!#
call rhd_hyd_Pred(m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order, &
                  wl, rho, P, grav_flag, phic_grav, phib_grav, &
                  P_red, P_avg)
!
! === Compute various (Roe) averages ===============================================================
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: P_red')
call timing_start('HYD: Roe1Dflux: RoeAvgs')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Roe averages: weighting with sqrt(rho) ---
      rho_til(  i1,i2,i3)=sqrt_rho(i1-1,i2,i3)*sqrt_rho(i1,i2,i3)
      wrho(     i1,i2,i3)=sqrt_rho(i1-1,i2,i3)/(sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3))
      v1_til(   i1,i2,i3)=wrho(i1,i2,i3)*(v1(i1-1,i2,i3)-v1(i1,i2,i3)) + v1(i1,i2,i3)
      v1abs_til(i1,i2,i3)=wrho(i1,i2,i3)*(abs(v1(i1-1,i2,i3))-abs(v1(i1,i2,i3))) + abs(v1(i1,i2,i3))
      !
    end do ! i1
    do i1=m1,n1+1
      ! --- Derived Roe averages ---
      ! --- Originally: P_til=rho_til * (eikp_til - eik_til) ---
      Psim_til       =rho_til(i1  ,i2,i3)/(sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3)) * &
                           (P(i1-1,i2,i3)/ sqrt_rho(i1-1,i2,i3) + &
                            P(i1  ,i2,i3)/ sqrt_rho(i1  ,i2,i3) )
      P_til(i1,i2,i3)=Psim_til + &
                        (rho_til(i1,i2,i3)/(sqrt_rho(i1-1,i2,i3) + sqrt_rho(i1,i2,i3)))**2 * &
                        0.5 * &
                        ( (v1(i1,i2,i3)-v1(i1-1,i2,i3))**2 + &
                          (v2(i1,i2,i3)-v2(i1-1,i2,i3))**2 + &
                          (v3(i1,i2,i3)-v3(i1-1,i2,i3))**2 )
      ! --- Tried: 2012-03-11 (B.F.): hardly any effect, still: ToDo... ---
      !                 ( (v1(i1,i2,i3)-v1(i1-1,i2,i3))**2 )
      !
      ! --- eip_til=eikp_til-ek_til ---
      ei_til           =wrho(i1,i2,i3)*(ei(i1-1,i2,i3)-ei(i1,i2,i3)) + ei(i1,i2,i3)
      eip_til(i1,i2,i3)=ei_til + P_til(i1,i2,i3)/rho_til(i1,i2,i3)
      !
      ! --- Averages of drhoei/dp_s, Gamma_1 and sound speed ---
      DrhoeiDPConsts_til(i1,i2,i3)= &
        wrho(i1,i2,i3)*((rho(i1-1,i2,i3)*ei(i1-1,i2,i3)/P(i1-1,i2,i3) + 1.0)/Gamma_1(i1-1,i2,i3)- &
                        (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3) + 1.0)/Gamma_1(i1  ,i2,i3)) +&
                        (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3) + 1.0)/Gamma_1(i1  ,i2,i3)
      Gamma_1_til=wrho(i1,i2,i3)*(Gamma_1(  i1-1,i2,i3)- &
                                  Gamma_1(  i1  ,i2,i3)) + &
                                  Gamma_1(  i1  ,i2,i3)
      cs_til(    i1,i2,i3)=sqrt(P_til(i1,i2,i3)/rho_til(i1,i2,i3)*Gamma_1_til)
      !
    end do ! i1
    do i1=m1,n1+1
      ! === Characteristic velocities at cell boundaries ===
      !
      ! --- Characteristic velocities ---
      lambda_til_mi(i1,i2,i3)=v1_til(i1,i2,i3)-cs_til(i1,i2,i3)
      lambda_til_pl(i1,i2,i3)=v1_til(i1,i2,i3)+cs_til(i1,i2,i3)
      lambda_til_nu(i1,i2,i3)=v1_til(i1,i2,i3)
      !
      ! --- Set "reduced" sound speed = normal sound speed ---
      csRed_til(i1,i2,i3)=cs_til(i1,i2,i3)
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! \label{f90:c_hydlowmachcsfactor__use1}
if (par%C_hydLowMachcsFactor > 0.0) then
  ! --- Perform low-Mach-number sound-speed reduction ---
  !
  ! --- Compute reduced sound speed and characteristic velocities at cell boundaries ---
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        ! --- Compute Roe averages by weighting with sqrt(rho)   ---
        ! --- Old settings: csRed_til(i1,i2,i3)=cs_til(i1,i2,i3) ---
        csRed_til(i1,i2,i3)=max(min(par%C_hydLowMachcsFactor * &
                                    (v1abs_til(i1,i2,i3)/csRed_til(i1,i2,i3)), &
                                    1.0), &
                                par%C_hydLowMachcsBase) * &
                            csRed_til(i1,i2,i3)
        !
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
! \label{f90:hdentropywavemode__use3}\label{f90:hdenthalpyavgmode__use3}
! \label{f90:c_hydsmallfluctfactor__use1}
if ((par%hdEntropyWaveMode == 'Entropy3gMps') .or. &
    (par%hdEnthalpyAvgMode == 'CA1gMps'     ) .or. &
    (par%hdEnthalpyAvgMode == 'CA1gMps_Pred') .or. &
    (par%hdEnthalpyAvgMode == 'CA5gMps'     ) .or. &
    (par%hdEnthalpyAvgMode == 'CA6gMps'     ) .or. &
    ((par%C_RecContShift >= 0.0) .and. (par%C_RecContSteep > 0.0))) then
  ! --- Compute special large-fluctuation weight used for entropy wave and enthalpy averaging ---
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1-1,n1+2
        !
        ! --- Compute corrected pressure step, normalize and limit it and smooth the jump ---
        dPg  =eps_TinyPressure*(P(i1,i2,i3)+P(i1-1,i2,i3))
!#
#       if (rhd_hyd_gravcorr_p01 == 0)
!#                                     /* 'off0' */
        dPg  =(abs((P(i1-1,i2,i3)-rho(i1-1,i2,i3)*(phic_grav(i1-1,i2,i3)-phib_grav(i1  ,i2,i3))) - &
                   (P(i1  ,i2,i3)-rho(i1  ,i2,i3)*(phic_grav(i1  ,i2,i3)-phib_grav(i1  ,i2,i3)))) +&
               dPg) / &
              (   ( P(i1-1,i2,i3)                     +  P(i1,i2,i3)                   )+dPg)
!#
#       else
!#           /* rhd_hyd_gravcorr_p01 */
        dPg  =(abs((P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) - (P(i1,i2,i3)-P_red(i1,0,i2,i3)))+dPg)/ &
              (   ( P(i1-1,i2,i3)                     +  P(i1,i2,i3)                   )+dPg)
!#
#       endif
!#            /* rhd_hyd_gravcorr_p01 */
        dPg  =min(25.0*par%C_hydSmallFluctFactor*dPg, 1.0)
        dPg  =dPg * (2.0-dPg)
        !
        ! --- Compute non-corrected pressure step, normalize and limit it and smooth the jump ---
        ! --- Interpolate for jumps between factor 3 and 7 ---
        dP   =abs(P(i1-1,i2,i3)-P(i1,i2,i3))/ &
              (   P(i1-1,i2,i3)+P(i1,i2,i3))
        dP   =min(4.0*par%C_hydSmallFluctFactor*max(dP-0.5,0.0), 1.0)
        ! --- Old: dP   =dP**2 * (2.0-dP)
        dP   =dP**2 * (3.0-2.0*dP)
        !
        ! --- Compute Mach-number weight ---
        !wMach=v1abs_til(i1,i2,i3)/cs_til(i1,i2,i3)
        wMach=0.5*(abs(v1(i1-1,i2,i3)/cs(i1-1,i2,i3)) + abs(v1(i1,i2,i3)/cs(i1,i2,i3)))
        wMach=min(2.0*par%C_hydSmallFluctFactor*wMach, 1.0)
        ! --- Old: wMach=wMach**2 * (2.0-wMach)
        wMach=wMach**2 * (3.0-2.0*wMach)
        !
        ! --- Compute entropy step, normalize and limit it and smooth the jump ---
        ds   =abs(s(i1,i2,i3)-s(i1-1,i2,i3)) * (dTdei(i1,i2,i3)+dTdei(i1-1,i2,i3))
        ds   =min(10.0*par%C_hydSmallFluctFactor*ds, 1.0)
        ds   =ds * (2.0-ds)
        !
        ! --- Compose total weight ---
        wLgFluct_til(i1,i2,i3)=1.0 - (1.0-dPg)*(1.0-dP)*(1.0-wMach)*(1.0-ds)
        !
      end do ! i1
    end do ! i2
  end do ! i3
endif
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: RoeAvgs')
!#
# endif
!#
!
! \label{f90:c_slopered__use1}
! --- Perform slope reduction in case of strong density gradient ---
if ((par%C_SlopeRed > 0.0) .and. &
    (reconstruction_type /= reconstruction_type_Constant)) then
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: SlopeRed')
!#
# endif
!#
  SlopeRed=1.0-min(par%C_SlopeRed * &
                     max(rho(m1-1:n1+2,:,:)/rho(m1-2:n1+1,:,:)+ &
                         rho(m1-2:n1+1,:,:)/rho(m1-1:n1+2,:,:)-2.5, &
                         0.0)**2, &
                   1.0)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: SlopeRed')
!#
# endif
!#
  SlopeRed_flag=.true.
  !
else
  SlopeRed_flag=.false.
endif
!
! \label{f90:c_reccontshift__use1}
! --- Choose a more continuous reconstruction in case of small Mach numbers ---
if ((par%C_RecContShift >= 0.0) .and. &
    (par%C_RecContSteep >  0.0) .and. &
    (reconstruction_type /= reconstruction_type_Constant)) then
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecCont')
!#
# endif
!#
!!$  do i3=m3,n3
!!$    do i2=m2,n2
!!$      do i1=m1,n1+1
!!$        ! --- Compute estimate for Mach number at cell boundary (account also for pressure gradient)
!!$        vm1        =sqrt(v1(i1-1,i2,i3)**2 + v2(i1-1,i2,i3)**2 + v3(i1-1,i2,i3)**2)
!!$        vp0        =sqrt(v1(i1  ,i2,i3)**2 + v2(i1  ,i2,i3)**2 + v3(i1  ,i2,i3)**2)
!!$        MachEst_til=sqrt(abs(((P(i1-1,i2,i3)-P_red(i1,-1,i2,i3)) - &
!!$                              (P(i1  ,i2,i3)-P_red(i1, 0,i2,i3))) / &
!!$                             rho_til(i1,i2,i3)) + &
!!$                         (wrho(i1,i2,i3)*(vm1-vp0) + vp0)**2) / &
!!$                    cs_til(i1,i2,i3)
!!$        !
!!$        ! --- Compute the slope-continuation weight ---
!!$        SlopeCon(i1,i2,i3)=MachEst_til / &
!!$                      (par%C_RecContShift/ &
!!$                         (1.0+MachEst_til/((par%C_RecContShift*par%C_RecContSteep)))**2 + &
!!$                       MachEst_til)
!!$      end do ! i1
!!$    end do ! i2
!!$  end do ! i3
!!$  !
!!$  ! --- Set extra boundary values to "normal" boundary values or periodic values ---
!!$  if ((bound_l == bound_periodic) .and. (bound_r == bound_periodic)) then
!!$    SlopeCon(m1-1,:,:)=SlopeCon(n1  ,:,:)
!!$    SlopeCon(n1+2,:,:)=SlopeCon(m1+1,:,:)
!!$  else
!!$    SlopeCon(m1-1,:,:)=SlopeCon(m1  ,:,:)
!!$    SlopeCon(n1+2,:,:)=SlopeCon(n1+1,:,:)
!!$  endif
  !
  ! --- Use the large-fluctuation-indicator array for the slope-continutation control parameter ---
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1-1,n1+2
        !SlopeCon(i1,i2,i3)=wLgFluct_til(i1,i2,i3)
        !SlopeCon(i1,i2,i3)=par%C_RecContShift*wLgFluct_til(i1,i2,i3)*(1.0-wLgFluct_til(i1,i2,i3))+ &
        !                   (1.0-(1.0-wLgFluct_til(i1,i2,i3))**2)    *(    wLgFluct_til(i1,i2,i3))
        !SlopeCon(i1,i2,i3)=min(par%C_RecContShift       * wLgFluct_til(i1,i2,i3) + &
        !                       (1.0-par%C_RecContShift) * wLgFluct_til(i1,i2,i3)**2, &
        !                       1.0)
        !SlopeCon(i1,i2,i3)=min(par%C_RecContShift       * wLgFluct_til(i1,i2,i3) + &
        !                       (1.0-par%C_RecContShift) * wLgFluct_til(i1,i2,i3)**3, &
        !                       1.0)
        SlopeCon(i1,i2,i3)=(    par%C_RecContShift) + &
                           (1.0-par%C_RecContShift)* &
                             min((    par%C_RecContSteep) * wLgFluct_til(i1,i2,i3) + &
                                 (1.0-par%C_RecContSteep) * wLgFluct_til(i1,i2,i3)**3, &
                                 1.0)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  if (.not. SlopeRed_flag) then
    SlopeRed=1.0
  endif
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecCont')
!#
# endif
!#
  !
  SlopeCon_flag=.true.
else
  SlopeCon_flag=.false.
endif
!
! === Set high-order time step according to time-integration scheme ================================
!
if ((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
    (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) then
  dtimeHO=0.0
else
  dtimeHO=dtime
endif
!
! === Fix entropy in case of transsonic rarefaction wave ===========================================
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: EntropyFix')
!#
# endif
!#
! --- Entropy fix: use modified LeVeque formula (1991) ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Check sound wave moving left (v-c) ---
      if (((v1(i1  ,i2,i3) - cs(i1  ,i2,i3)) > 0.0) .and. &
          ((v1(i1-1,i2,i3) - cs(i1-1,i2,i3)) < 0.0)      ) then
        ! --- Transsonic rarefaction wave detected ---
!#
#       if (rhd_hyd_entropyfix_p01 == 0)
!#                                       /* Use if-statement */
        if (lambda_til_mi(i1,i2,i3) > 0.0) then
          lambda_til_mi(i1,i2,i3)=v1(i1  ,i2,i3) - cs(i1  ,i2,i3)
        else
          lambda_til_mi(i1,i2,i3)=v1(i1-1,i2,i3) - cs(i1-1,i2,i3)
        endif
!#
#       elif (rhd_hyd_entropyfix_p01 == 2)
!#                                         /* No entropy fix */
        !write(6,*) i1, i2, i3, 'No entropy fix'
!#
#       else
!#           /* rhd_hyd_entropyfix_p01: use sign() */
        mask_up_mi=sign(0.5,lambda_til_mi(i1,i2,i3))+0.5
        lambda_til_mi(i1,i2,i3)=(1.0-mask_up_mi) * (v1(i1-1,i2,i3) - cs(i1-1,i2,i3)) + &
                                     mask_up_mi  * (v1(i1  ,i2,i3) - cs(i1  ,i2,i3))
!#
#       endif
!#            /* rhd_hyd_entropyfix_p01 */
      endif ! (v1-cs)
      !
      ! --- Check sound wave moving right (v+c) ---
      if (((v1(i1-1,i2,i3) + cs(i1-1,i2,i3)) < 0.0) .and. &
          ((v1(i1  ,i2,i3) + cs(i1  ,i2,i3)) > 0.0)      ) then
        ! --- Transsonic rarefaction wave detected ---
!#
#       if (rhd_hyd_entropyfix_p01 == 0)
!#                                       /* Use if-statement */
        if (lambda_til_pl(i1,i2,i3) >= 0.0) then
          lambda_til_pl(i1,i2,i3)=v1(i1  ,i2,i3) + cs(i1  ,i2,i3)
        else
          lambda_til_pl(i1,i2,i3)=v1(i1-1,i2,i3) + cs(i1-1,i2,i3)
        endif
!#
#       elif (rhd_hyd_entropyfix_p01 == 2)
!#                                         /* No entropy fix */
        !write(6,*) i1, i2, i3, 'No entropy fix'
!#
#       else
!#           /* rhd_hyd_entropyfix_p01: use sign() */
        mask_up_pl=sign(0.5,lambda_til_pl(i1,i2,i3))+0.5
        lambda_til_pl(i1,i2,i3)=(1.0-mask_up_pl) * (v1(i1-1,i2,i3) + cs(i1-1,i2,i3)) + &
                                     mask_up_pl  * (v1(i1  ,i2,i3) + cs(i1  ,i2,i3))
!#
#       endif
!#            /* rhd_hyd_entropyfix_p01 */
      endif ! (v1+cs)
      !
!!$      ! --- Check entropy wave (v) ---
!!$      if ((v1(i1-1,i2,i3) < 0.0) .and. &
!!$          (v1(i1  ,i2,i3) > 0.0)      ) then
!!$        ! --- Expansion point detected ---
!!$#
!!$#       if (rhd_hyd_entropyfix_p01 == 0)
!!$#
!!$        if (lambda_til_nu(i1,i2,i3) >= 0.0) then
!!$          lambda_til_nu(i1,i2,i3)=v1(i1  ,i2,i3)
!!$        else
!!$          lambda_til_nu(i1,i2,i3)=v1(i1-1,i2,i3)
!!$        endif
!!$#
!!$#       else
!!$#            /* rhd_hyd_entropyfix_p01 */
!!$        mask_up_nu=sign(0.5,lambda_til_nu(i1,i2,i3))+0.5
!!$        lambda_til_nu(i1,i2,i3)=(1.0-mask_up_nu) * v1(i1-1,i2,i3) + &
!!$                              mask_up_nu  * v1(i1  ,i2,i3)
!!$#
!!$#       endif
!!$#             /* rhd_hyd_entropyfix_p01 */
!!$      endif ! (v1)
      !
    end do ! i1
  end do ! i2
end do ! i3
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: EntropyFix')
call timing_start('HYD: Roe1Dflux: Upwind')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Find Upwind direction ---
      !
!#
#     if (rhd_hyd_upwind_p01 == 0)
!#
      if (lambda_til_mi(i1,i2,i3) > 0.0) then
        i_up_mi(i1,i2,i3)=i1-1
      else
        i_up_mi(i1,i2,i3)=i1
      endif
      if (lambda_til_nu(i1,i2,i3) > 0.0) then
        i_up_nu(i1,i2,i3)=i1-1
      else
        i_up_nu(i1,i2,i3)=i1
      endif
      if (lambda_til_pl(i1,i2,i3) > 0.0) then
        i_up_pl(i1,i2,i3)=i1-1
      else
        i_up_pl(i1,i2,i3)=i1
      endif
!#
#     else
!#         /* rhd_hyd_upwind_p01 */
      i_up_mi(i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_mi(i1,i2,i3))-0.5)
      i_up_nu(i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_nu(i1,i2,i3))-0.5)
      i_up_pl(i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_pl(i1,i2,i3))-0.5)
!#
#     endif
!#          /* rhd_hyd_upwind_p01 */
      !
    end do ! i1
  end do ! i2
end do ! i3
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: Upwind')
!#
# endif
!#
!
! === Compute sound-wave fluxes (waves 1 and 2) ====================================================
!
call rhd_hyd_RoeWavealpha12(m1,n1, m2,n2, m3,n3, nghost1, &
                            reconstruction_order, reconstruction_type, &
                            dtimeHO, dxb1_avg, dxb1, wl, &
                            rho, v1, v2, v3, P, &
                            P_red, P_avg, &
                            rho_til, cs_til, csRed_til, &
                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                            i_up_mi, i_up_pl, lambda_til_mi, lambda_til_pl, &
                            flux_alpha_til_1, flux_alpha_til_2)
!
! === Compute entropy-wave fluxes (waves 3 and 6) ==================================================
!
call rhd_hyd_RoeWavealpha36(m1,n1, m2,n2, m3,n3, nghost1, &
                            reconstruction_order, reconstruction_type, &
                            dtimeHO, dxb1_avg, dxb1, wl, &
                            rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, s, T, dTdei, &
                            P_red, P_avg, &
                            sqrt_rho, wrho, wLgFluct_til, &
                            rho_til, cs_til, v1abs_til, DrhoeiDPConsts_til, &
                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                            i_up_nu, lambda_til_nu, &
                            flux_alpha_til_3, flux_alpha_til_6, &
                            flux_rhoei_sDiff)
!
! === Boundary conditions IV =======================================================================
!
! --- Set fluxes explicitly to zero ---
if ((bound_l == bound_closed       ) .or. &
    (bound_l == bound_closed2      ) .or. &
    (bound_l == bound_closedbottom ) .or. &
    (bound_l == bound_closedbottom2)       ) then
!  flux_alpha_til_1(m1,:,:)=0.0
!  flux_alpha_til_2(m1,:,:)=0.0
! --- Fix: 2011-10-27 (B.F.) ---
!  flux_alpha_til_1(m1,:,:)=0.5*(flux_alpha_til_1(m1,:,:)-flux_alpha_til_2(m1,:,:))
!  flux_alpha_til_2(m1,:,:)=    -flux_alpha_til_1(m1,:,:)
  flux_rhoei_sDiff(m1,:,:)=0.0
endif
! --- Fix: 2014-09-22 (B.F.) ---
if ((bound_r == bound_closed       ) .or. &
    (bound_r == bound_closed2      ) .or. &
    (bound_r == bound_closedbottom ) .or. &
    (bound_r == bound_closedbottom2)       ) then
!  flux_alpha_til_1(n1+1,:,:)=0.0
!  flux_alpha_til_2(n1+1,:,:)=0.0
! --- Fix: 2011-10-27 (B.F.) ---
!  flux_alpha_til_1(n1+1,:,:)=0.5*(flux_alpha_til_1(n1+1,:,:)-flux_alpha_til_2(n1+1,:,:))
!  flux_alpha_til_2(n1+1,:,:)=    -flux_alpha_til_1(n1+1,:,:)
  flux_rhoei_sDiff(n1+1,:,:)=0.0
endif
!
! === Back projection and combination => fluxes ===
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
!
! === Compute mass flux, longitudinal momentum, and acoustic flux ==================================
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      flux_alpha_til_123=flux_alpha_til_1(i1,i2,i3) + &
                         flux_alpha_til_2(i1,i2,i3) + &
                         flux_alpha_til_3(i1,i2,i3)
      flux_rho(i1,i2,i3)=      flux_alpha_til_123 + &
                         (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) + &
                         ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3))
      flux_rhov1(i1,i2,i3)= v1_til(i1,i2,i3) *  flux_alpha_til_123 - &
                            cs_til(i1,i2,i3) * (flux_alpha_til_1(i1,i2,i3) - &
                                                flux_alpha_til_2(i1,i2,i3)) + &
                           (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * v1(i1-1,i2,i3) + &
                           ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * v1(i1  ,i2,i3) + &
                             P_avg(i1,i2,i3)
!      flux_rhov1(i1,i2,i3)= v1_til(i1,i2,i3)                   * flux_alpha_til_3(i1,i2,i3) + &
!                           (v1_til(i1,i2,i3)-cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
!                           (v1_til(i1,i2,i3)+cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
!                           (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * v1(i1-1,i2,i3) + &
!                           ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * v1(i1  ,i2,i3) + &
!                             P_avg(i1,i2,i3)
!      flux_rhov1(i1,i2,i3)= v1_til(i1,i2,i3) * flux_rho(i1,i2,i3) - &
!                            cs_til(i1,i2,i3) * (flux_alpha_til_1(i1,i2,i3) - &
!                                                flux_alpha_til_2(i1,i2,i3)) + &
!                             P_avg(i1,i2,i3)
!      flux_rhov2(i1,i2,i3)= v2_til(i1,i2,i3) *flux_alpha_til_123 + &
!                           rho_til(i1,i2,i3) *flux_alpha_til_4(i1,i2,i3) + &
!                           (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * v2(i1-1,i2,i3) + &
!                           ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * v2(i1  ,i2,i3)
!      flux_rhov3(i1,i2,i3)= v3_til(i1,i2,i3) *flux_alpha_til_123 + &
!                           rho_til(i1,i2,i3) *flux_alpha_til_5(i1,i2,i3) + &
!                           (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * v3(i1-1,i2,i3)+ &
!                           ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * v3(i1  ,i2,i3)
!      ! --- flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
!      flux_rhoei(i1,i2,i3)= &
!          (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
!          (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
!           rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
!                               v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
!                                                                  flux_alpha_til_6(i1,i2,i3) + &
!          (     wl(i1) *rho(i1-1,i2,i3)*v1(i1-1,i2,i3)) * &
!          (ei(i1-1,i2,i3) + &
!           P( i1-1,i2,i3)/rho(i1-1,i2,i3) + &
!           0.5 * ( (v1(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
!                   (v2(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
!                   (v3(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
!          ((1.0-wl(i1))*rho(i1  ,i2,i3)*v1(i1  ,i2,i3)) * &
!          (ei(i1,i2,i3) + &
!           P( i1,i2,i3)/rho(i1,i2,i3) + &
!           0.5 * ( (v1(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
!                   (v2(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
!                   (v3(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
!      !
!!!$      fek(i1,i2,i3)=0.5 * (v1_til(i1,i2,i3)**2 + v2_til(i1,i2,i3)**2 + v3_til(i1,i2,i3)**2) * &
!!!$                    flux_rho(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
if (OutFlx_flag) then
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        fep(i1,i2,i3)=P_til(i1,i2,i3)/rho_til(i1,i2,i3) * (flux_alpha_til_1(i1,i2,i3)+ &
                                                           flux_alpha_til_2(i1,i2,i3)) + &
                 wl(i1) * P(i1-1,i2,i3) * v1(i1-1,i2,i3) + &
            (1.0-wl(i1))* P(i1  ,i2,i3) * v1(i1  ,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
!
! === Compute transversal momentum fluxes (waves 4 and 5) ==========================================
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Roe averages: weighting with sqrt(rho) ---
      v2_til(   i1,i2,i3)=wrho(i1,i2,i3)*(v2(i1-1,i2,i3)-v2(i1,i2,i3)) + v2(i1,i2,i3)
      v3_til(   i1,i2,i3)=wrho(i1,i2,i3)*(v3(i1-1,i2,i3)-v3(i1,i2,i3)) + v3(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
call rhd_hyd_RoeWavealpha45(m1,n1, m2,n2, m3,n3, nghost1, &
                            reconstruction_order, reconstruction_type, &
                            dtimeHO, dxb1_avg, dxb1, wl, &
                            rho, v1, v2, v3, &
                            wrho, rho_til, v2_til, v3_til, cs_til, &
                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                            i_up_nu, lambda_til_nu, &
                            flux_rho, &
                            flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
                            flux_alpha_til_4, flux_alpha_til_5, &
                            flux_rhov2, flux_rhov3)
!
! === Compute energy flux ==========================================================================
!
call rhd_hyd_Fluxrhoei(m1,n1, m2,n2, m3,n3, nghost1, reconstruction_order, &
                       wl, rho, ei, v1, v2, v3, P, Gamma_1, s, dTdei, &
                       P_red, P_avg, &
                       wrho, wLgFluct_til, &
                       rho_til, eip_til, v1_til, v2_til, v3_til, cs_til, v1abs_til, &
                       flux_rho, flux_rhoei_sDiff, &
                       flux_alpha_til_1, flux_alpha_til_2, flux_alpha_til_3, &
                       flux_alpha_til_4, flux_alpha_til_5, flux_alpha_til_6, &
                       flux_rhoei)
!
! === Boundary conditions V ========================================================================
!
! --- Set advection fluxes explicitly to zero ---
!if ((bound_l == bound_closedbottom) .or. &
!    (bound_l == bound_closedbottom2)) then
if ((bound_l == bound_closed       ) .or. &
    (bound_l == bound_closed2      ) .or. &
    (bound_l == bound_closedbottom ) .or. &
    (bound_l == bound_closedbottom2)       )    then
  i1=m1
  v1_til(    i1,:,:)=0.0
  flux_rho(  i1,:,:)=0.0
  flux_rhov2(i1,:,:)=0.0
  flux_rhov3(i1,:,:)=0.0
  flux_rhoei(i1,:,:)=0.0
  fep(       i1,:,:)=0.0
endif
if ((bound_r == bound_closed       ) .or. &
    (bound_r == bound_closed2      ) .or. &
    (bound_r == bound_closedbottom ) .or. &
    (bound_r == bound_closedbottom2)       ) then
  i1=n1+1
  v1_til(    i1,:,:)=0.0
  flux_rho(  i1,:,:)=0.0
  flux_rhov2(i1,:,:)=0.0
  flux_rhov3(i1,:,:)=0.0
  flux_rhoei(i1,:,:)=0.0
  fep(       i1,:,:)=0.0
endif
! --- Set boundary fluxes explicitly to fluxes inside box (except for some pressure terms) ---
if (bound_l == bound_inoutflow2) then
  i1=m1
  flux_rho(  i1,:,:)=rho(i1,:,:)*v1(i1,:,:)
  flux_rhov1(i1,:,:)=rho(i1,:,:)*v1(i1,:,:)**2 + P_avg(i1,:,:)
  flux_rhov2(i1,:,:)=v2(i1,:,:)*rho(i1,:,:)*v1(i1,:,:)
  flux_rhov3(i1,:,:)=v3(i1,:,:)*rho(i1,:,:)*v1(i1,:,:)
  flux_rhoei(i1,:,:)=(ei(i1,:,:)+P_avg(i1,:,:)/rho(i1,:,:) + &
                      0.5 * ( (v1(i1,:,:)**2 - v1_til(i1,:,:)**2) + &
                              (v2(i1,:,:)**2 - v2_til(i1,:,:)**2) + &
                              (v3(i1,:,:)**2 - v3_til(i1,:,:)**2) ) &
                     ) * rho(i1,:,:)*v1(i1,:,:)
  fep(       i1,:,:)=P_avg(i1,:,:)*v1(i1,:,:)
else if (bound_l == bound_inoutflow3) then
  i1=m1
  flux_rho(  i1,:,:)=rho(i1,:,:)*v1(i1,:,:)
endif
!
!!$if ((m1 <=0) .and. (n1 >=0) .and. (m2 <=0) .and. (n2 >=0) .and. (m3 <=0) .and. (n3 >=0)) then
!!$ ! if ((m1 == 1) .and. (m2 == 1) .and. (m3 == 1)) then
!!$call uio_openwr(nc, 'rhd_roe1dflux_test.dat', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now' )
!!$call uio_wrlabl(nc, box_ident                 )
!!$call uio_wr(nc, 'z'               , box_id_ident        )
!!$call uio_wr(nc, dxb1              , 'dxb1'              )
!!$call uio_wr(nc, ei                , 'ei'                )
!!$call uio_wr(nc, rho               , 'rho'               )
!!$call uio_wr(nc, v1                , 'v1'                )
!!$call uio_wr(nc, v2                , 'v2'                )
!!$call uio_wr(nc, v3                , 'v3'                )
!!$call uio_wr(nc, P                 , 'p'                 )
!!$call uio_wr(nc, s                 , 's'                 )
!!$call uio_wr(nc, T                 , 't'                 )
!!$call uio_wr(nc, Gamma_1           , 'gamma_1'           )
!!$call uio_wr(nc, Gamma_3           , 'gamma_3'           )
!!$call uio_wr(nc, dTdei             , 'dtdei'             )
!!$call uio_wr(nc, i_up_nu           , 'i_up_nu'           )
!!$call uio_wr(nc, wrho              , 'wrho'              )
!!$call uio_wr(nc, v1_til            , 'v1_til'            )
!!$call uio_wr(nc, v2_til            , 'v2_til'            )
!!$call uio_wr(nc, v3_til            , 'v3_til'            )
!!$call uio_wr(nc, rho_til           , 'rho_til'           )
!!$call uio_wr(nc, P_til             , 'p_til'             )
!!$call uio_wr(nc, P_avg             , 'p_avg'             )
!!$call uio_wr(nc, P_red             , 'p_red'             )
!!$call uio_wr(nc, eip_til           , 'eip_til'           )
!!$call uio_wr(nc, cs_til            , 'cs_til'            )
!!$call uio_wr(nc, csRed_til         , 'csred_til'         )
!!$call uio_wr(nc, SlopeRed          , 'slopered'          )
!!$call uio_wr(nc, lambda_til_pl     , 'lambda_til_pl'     )
!!$call uio_wr(nc, lambda_til_mi     , 'lambda_til_mi'     )
!!$call uio_wr(nc, lambda_til_nu     , 'lambda_til_nu'     )
!!$call uio_wr(nc, wLgFluct_til      , 'wlgfluct_til'      )
!!$call uio_wr(nc, v1abs_til         , 'v1abs_til'         )
!!$call uio_wr(nc, DrhoeiDPConsts_til, 'drhoeidpconsts_til')
!!$call uio_wr(nc, flux_alpha_til_1  , 'flux_alpha_til_1'  )
!!$call uio_wr(nc, flux_alpha_til_2  , 'flux_alpha_til_2'  )
!!$call uio_wr(nc, flux_alpha_til_3  , 'flux_alpha_til_3'  )
!!$call uio_wr(nc, flux_alpha_til_4  , 'flux_alpha_til_4'  )
!!$call uio_wr(nc, flux_alpha_til_5  , 'flux_alpha_til_5'  )
!!$call uio_wr(nc, flux_alpha_til_6  , 'flux_alpha_til_6'  )
!!$call uio_wr(nc, flux_rho          , 'flux_rho'          )
!!$call uio_wr(nc, flux_rhoei        , 'flux_rhoei'        )
!!$call uio_wr(nc, flux_rhoei_sDiff  , 'flux_rhoei_sdiff'  )
!!$call uio_wr(nc, flux_rhov1        , 'flux_rhov1'        )
!!$call uio_wr(nc, flux_rhov2        , 'flux_rhov2'        )
!!$call uio_wr(nc, flux_rhov3        , 'flux_rhov3'        )
!!$call uio_wr(nc, fep               , 'fep'               )
!!$call uio_wrlabl(nc, endbox_ident)
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call flush(nc)
!!$call uio_closrd(nc)
!!$endif
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: fluxes')
!#
# endif
!#
end subroutine rhd_hyd_RoeWave1Dflux


!----------**********************-------------------------------------------------------------------
subroutine rhd_hyd_RoeState1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
                                  bound_l, bound_r, reconstruction_order, reconstruction_type, &
                                  dtime, dxb1_avg, dxb1, wl, &
                                  rho, ei, v1, v2, v3, P, Gamma_1, cs, sqrt_rho, &
                                  phic_grav, phib_grav, &
                                  rho_til, v1_til, v2_til, v3_til, &
                                  flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_RoeState1Dflux ('rhd_hydrodynamics_Roe_State_1D_flux')
!
! PURPOSE:
!   Compute 1D Roe fluxes with state reconstructions.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_RoeState1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
!                               bound_l, bound_r, reconstruction_order, reconstruction_type, &
!                               dtime, dxb1_avg, dxb1, wl, &
!                               rho, ei, v1, v2, v3, P, Gamma_1, cs, sqrt_rho, &
!                               phic_grav, phib_grav, &
!                               rho_til, v1_til, v2_til, v3_til, &
!                               flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   bound_l:      ('boundary_left_1') integer, left boundary type for dimension 1
!                 values (constants defined in rhd_action_module):
!                    bound_None, bound_reflective, bound_periodic, bound_constant,
!                    bound_closed, bound_closed2,
!                    bound_inoutflow, bound_inoutflow2,  bound_inoutflow3, &
!                    bound_closedbottom, bound_closedbottom2,
!                    bound_transmitting, bound_transmitting2, bound_transmitting3
!   bound_r:      ('boundary_right_1') integer, right boundary type for dimension 1
!                 values (constants defined in rhd_action_module):
!                    bound_None, bound_reflective, bound_periodic, bound_constant,
!                    bound_closed, bound_closed2,
!                    bound_transmitting, bound_transmitting2, bound_transmitting3
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtime:        ('delta_time') real, time step to be used
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real, cell-boundary step size, 1. coordinate
!   wl:           ('weight_left') real, geometrical weights due to non-equidistant grid
!   Radius:       ('Radius') real 3D array with radius (for central potential case)
!   rho:          ('rho') real 3D array, density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('v_1') real 3D array, velocity, x1 coordinate
!   v2:           ('v_2') real 3D array, velocity, x2 coordinate
!   v3:           ('v_3') real 3D array, velocity, x3 coordinate
!   P:            ('pressure') real 3D array, pressure on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   sqrt_rho:     ('square_root_rho') real 3D array, square root of density on model grid
!   phic_grav:    ('phi_center_gravitation') real 3D array, cell centered grav. potential
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   gc_grav:      ('gravity_center_gravity') real 3D array, gravitational acceleration
!                 at cell centers
!
! OUTPUT:
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe-averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!
! LOCAL VARIABLES:
!   DrhoeiDPConsts_til: ('Delta_rho_energy_internal_Delta_Pressure_Constant_s_tilde') real 3D array,
!                       thermodynamic quantity (drhoei/dP|_s) at cell boundary
!
! ROUTINES:
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!   It is assumed that there are no ghost zones present in "orthogonal directions":
!     model%nghost2=0, model%nghost3=0
!   The routine computes only fluxes first-order in time. It needs a Runge-Kutta wrapper
!   to deliver stable results. It won't work with directional splitting.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   2001-01-17 (B.F. Uppsala) Put into own module
!   2001-01-21 (B.F.) Check if grid is equidistant and call simplified routines if true
!   2001-01-25 (B.F.) fep -> fep(i1,i2,i3), case(reconstruction_order)
!   2001-02-15 (B.F.) lambda_til_nu removed in input list (assumed: lambda_til_nu=v1_til)
!   2001-02-16 (B.F.) P_red now computed in routine
!   2001-02-17 (B.F.) Entropy fix now done in routine
!   2001-02-18 (B.F.) Compute q_avg and q_til locally
!   2001-03-20 (B.F.) Test: apply entropy fix to "0"-characteristic, too
!   2002-04-11 (B.F.) lambda_til -> v1_til in all cases
!   2002-10-25 (B.F.) Bugfix in computation of dxb1_avg and dxb1_var
!   2002-10-27 (B.F.) Slope reduction removed
!   2002-11-13 (B.F.) No entropy fix for  "0"-characteristic
!   2002-12-29 (B.F.) Slope reduction in case of strong density contrast
!   2002-12-22 (B.F.) Slope reduction in case of strong density contrast controlled with C_SlopeRed
!   2003-05-06 (B.F.) Slope reduction also for Minmod (towards Constant)
!   2007-02-01 (B.F. Lyon) Removal of P_red from alpha_til_3 and alpha_til_6,
!                     New pressure reduction scheme: default6
!   2007-02-12 (B.F.) P_red put back into alpha_til_3 and alpha_til_6
!   2008-03-17 (B.F.) Parameters C_ReconstructiondTimeFactor, C_hydPredfactor added
!   2008-06-12 (B.F.) Branch without entropy fix
!   2008-08-19 (B.F.) Parameter C_ReconstructiondTimeFactor outcommented
!   2010-06-09 (B.F.) Parameter C_dxb1_maxvar=0.001 -> 0.01
!   2011-07-12 (B.F.) Set dtimeHO to zero for RungeKutta2/3
!   2011-07-17 (B.F.) dxb1_avg als parameter
!   2011-07-20 (B.F.) Rearrangement of terms in flux formulae;
!                     flux_alpha_til_123 becomes scalar
!   2011-08-08 (B.F.) Test branch for alternative way to compute fluxes of transversal velocities
!   2011-10-27 (B.F.) Bugfix: Set all fluxes except flux_rhov1 explicitly to 0 for closed bnds (V),
!                             Don't set Roe fluxes 1 and 2 to 0 for closed bnds (IV)
!   2011-11-14 (B.F.) Remove expansion-wave slope-reduction branch
!   2011-11-18 (B.F.) Reconstruction more continuous with C_RecContShift and C_RecContSteep
!   2012-02-17 (B.F.) Centrifugal correction for gravity correction
!   2012-03-22 (B.F.) Create new routine rhd_hyd_RoeState1Dflux from rhd_hyd_Roe1Dflux
!   2012-03-23 (B.F.) New options for hdtransvelomode: 'CA3' and 'CA4'
!   2012-03-26 (B.F.) New options for hdtransvelomode: 'CA5' and 'CA6'
!   2012-03-27 (B.F.) SlopeRed, SlopeCon, dxb1
!   2012-03-28 (B.F.) Call rhd_rec_Vec for spatially varying grid,
!                     rho_til becomes output parameter
!   2012-04-04 (B.F.) Account for parameter C_hydsDiffVelo
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-05-21 (B.F.) Special pressure treatment (mask_bound) at closed boundaries (III) removed
!   2012-06-11 (B.F.) Remove C_SlopeRed from parameter list
!   2012-10-05 (B.F.) Limiting term for vD added
!   2012-10-08 (B.F.) Limiting term for vD removed
!   2012-10-16 (B.F.) C_hydsDiffVelo -> C_hydsDiffVelo**2
!   2013-02-22 (B.F.) wrho becomes array
!   2013-04-26 (B.F.) MachEst_til becomes scalar
!   2014-12-03 (B.F.) Remove parameter xb1
!   2018-05-09 (B.F.) Add dimension specifications in recontruction-routine calls
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_rec_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           bound_l, bound_r, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime, dxb1_avg
real, dimension(m1-nghost1  :n1+nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(m1-nghost1+1:n1+nghost1  ), &
                           intent(in)   :: wl
real, dimension(m1-nghost1:n1+nghost1,&
                m2        :n2        ,&
                m3        :n3          ), &
                           intent(in)   :: rho, ei, v1, v2, v3, &
                                           P, Gamma_1, cs, sqrt_rho, &
                                           phic_grav
real, dimension(m1-nghost1:n1+nghost1+1,&
                m2        :n2          ,&
                m3        :n3            ), &
                           intent(in)   :: phib_grav
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(out)  :: rho_til, v1_til, v2_til, v3_til, &
                                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_rhoei, fep
!
! --- Local variables ---
integer                                 :: i1, i2, i3, j, nc
real                                    :: ei_til, Gamma_1_til, &
                                           mask_up_mi, mask_up_pl, mask_up_nu, &
                                           flux_alpha_til_123, Mach, &
                                           sqrt_rho_l, sqrt_rho_r, MachEst_til
real, dimension(m1-1:n1+1, &
                m2  :n2  , &
                m3  :n3   )             :: rho_l  , rho_r, &
                                           rhoei_l, rhoei_r, &
                                           v1_l   , v1_r, &
                                           v2_l   , v2_r, &
                                           v3_l   , v3_r, &
                                           P_l    , P_r, &
                                           vD
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     )           :: P_til, eip_til, wrho, &
                                           cs_til, DrhoeiDPConsts_til, &
                                           lambda_til_mi, lambda_til_pl, lambda_til_nu
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     )           :: alpha_til_1, alpha_til_2, &
                                           alpha_til_3, alpha_til_4, &
                                           alpha_til_5, alpha_til_6
real, dimension(m1  :n1+1, &
                m2  :n2  , &
                m3  :n3     )           :: flux_alpha_til_1, flux_alpha_til_2, &
                                           flux_alpha_til_3, flux_alpha_til_4, &
                                           flux_alpha_til_5, &
                                           flux_alpha_til_6
logical                                 :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     )           :: SlopeRed, SlopeCon
!---------------------------------------------------------------------------------------------------
!
! === Reconstruction (constant, slope limiter, PP, FR) of state variables ===
!
! \label{f90:c_slopered__use2}
! --- Perform slope reduction in case of strong density gradient ---
if ((par%C_SlopeRed > 0.0) .and. &
    (reconstruction_type /= reconstruction_type_Constant)) then
  SlopeRed=1.0-min(par%C_SlopeRed * &
                     max(rho(m1-1:n1+2,:,:)/rho(m1-2:n1+1,:,:)+ &
                         rho(m1-2:n1+1,:,:)/rho(m1-1:n1+2,:,:)-2.5, &
                         0.0)**2, &
                   1.0)
  SlopeRed_flag=.true.
  !
  !write(6,'(A,3E14.5)') 'C_SlopeRed, minmax(SlopeRed):', &
  !  par%C_SlopeRed, minval(SlopeRed), maxval(SlopeRed)
else
  SlopeRed_flag=.false.
endif
!
! \label{f90:c_reccontshift__use2}
! --- Choose a more continuous reconstruction in case of small Mach numbers ---
if ((par%C_RecContShift > 0.0) .and. (par%C_RecContSteep > 0.0) .and. &
    (reconstruction_type /= reconstruction_type_Constant)) then
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        Mach=sqrt( &
                  !(((P(i1-1,i2,i3)) - (P(i1,i2,i3)))/ &
                  ! (0.5*(rho(i1-1,i2,i3)*cs(i1-1,i2,i3)+rho(i1,i2,i3)*cs(i1,i2,i3)))
                  !)**2 + &
                  v1(i1-1,i2,i3)**2 + v2(i1-1,i2,i3)**2 + v3(i1-1,i2,i3)**2 + &
                  v1(i1  ,i2,i3)**2 + v2(i1  ,i2,i3)**2 + v3(i1  ,i2,i3)**2) / &
             (0.5*(cs(i1-1,i2,i3)+cs(i1,i2,i3)))
        SlopeCon(i1,i2,i3)=Mach / (par%C_RecContShift/ &
                                     (1.0+Mach/((par%C_RecContShift*par%C_RecContSteep)))**2 + &
                                   Mach)
        !
        !SlopeCon(i1,i2,i3)=MachEst_til(i1,i2,i3) / &
        !              (par%C_RecContShift/ &
        !                (1.0+MachEst_til(i1,i2,i3)/((par%C_RecContShift*par%C_RecContSteep)))**2 + &
        !               MachEst_til(i1,i2,i3))
      end do ! i1
    end do ! i2
  end do ! i3
  !
  ! --- Set extra boundary values to "normal" boundary values or periodic values ---
  if ((bound_l == bound_periodic) .and. (bound_r == bound_periodic)) then
    SlopeCon(m1-1,:,:)=SlopeCon(n1  ,:,:)
    SlopeCon(n1+2,:,:)=SlopeCon(m1+1,:,:)
  else
    SlopeCon(m1-1,:,:)=SlopeCon(m1  ,:,:)
    SlopeCon(n1+2,:,:)=SlopeCon(n1+1,:,:)
  endif
  !
  if (.not. SlopeRed_flag) then
    SlopeRed=1.0
  endif
  !
  SlopeCon_flag=.true.
else
  SlopeCon_flag=.false.
endif
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: RoeState1Dflux: recon')
!#
# endif
!#
!
! --- Reconstruct states and get boundary values ---
if (dxb1_avg < 0.0) then
  ! --- Spatially varying grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecVec')
!#
# endif
!#
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   rho, dxb1, rho_l, rho_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   rho*ei, dxb1, rhoei_l, rhoei_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   v1, dxb1, v1_l, v1_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   v2, dxb1, v2_l, v2_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   v3, dxb1, v3_l, v3_r)
  call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                   reconstruction_order, reconstruction_type, &
                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                   P, dxb1, P_l, P_r)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecVec', cycles=6)
!#
# endif
!#
else
  ! --- Equidistant grid ---
  !
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_start('HYD: Roe1Dflux: RecVecEq')
!#
# endif
!#
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     rho, rho_l, rho_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     rho*ei, rhoei_l, rhoei_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     v1, v1_l, v1_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     v2, v2_l, v2_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     v3, v3_l, v3_r)
  call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                     reconstruction_order, reconstruction_type, &
                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                     P, P_l, P_r)
!#
# ifdef rhd_roe1d_flux_t01
!#
  call timing_stop('HYD: Roe1Dflux: RecVecEq', cycles=6)
!#
# endif
!#
endif
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: RoeState1Dflux: recon')
!#
# endif
!#
!
! --- Compute velocity at cell centers ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1-1,n1+1
      vD(i1,i2,i3)=sqrt(max(par%C_hydsDiffVelo**2,1.0)* v1(i1,i2,i3)**2 + &
                            par%C_hydsDiffVelo**2     *(v2(i1,i2,i3)**2 + &
                                                        v3(i1,i2,i3)**2) )
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Compute various averages ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      !
      ! --- Roe averages: weighting with sqrt(rho) ---
      sqrt_rho_r=sqrt(rho_r(i1-1,i2,i3))
      sqrt_rho_l=sqrt(rho_l(i1  ,i2,i3))
      rho_til( i1,i2,i3)=sqrt_rho_r*sqrt_rho_l
      wrho(    i1,i2,i3)=sqrt_rho_r/(sqrt_rho_r + sqrt_rho_l)
      v1_til(  i1,i2,i3)=wrho(i1,i2,i3)*(v1_r(   i1-1,i2,i3)-v1_l( i1,i2,i3)) + v1_l(  i1,i2,i3)
      v2_til(  i1,i2,i3)=wrho(i1,i2,i3)*(v2_r(   i1-1,i2,i3)-v2_l( i1,i2,i3)) + v2_l(  i1,i2,i3)
      v3_til(  i1,i2,i3)=wrho(i1,i2,i3)*(v3_r(   i1-1,i2,i3)-v3_l( i1,i2,i3)) + v3_l(  i1,i2,i3)
      ei_til            =wrho(i1,i2,i3)*(rhoei_r(i1-1,i2,i3)/rho_r(i1-1,i2,i3) - &
                                         rhoei_l(i1  ,i2,i3)/rho_l(i1  ,i2,i3)) + &
                                         rhoei_l(i1  ,i2,i3)/rho_l(i1  ,i2,i3)
      !
      ! --- Derived Roe averages ---
      ! --- Originally: P_til=rho_til * (eikp_til - eik_til) ---
      P_til(i1,i2,i3) = rho_til(i1,i2,i3)/(sqrt_rho_r + sqrt_rho_l) * &
                            (P_r(i1-1,i2,i3)/sqrt_rho_r + &
                             P_l(i1  ,i2,i3)/sqrt_rho_l ) + &
                        (rho_til(i1,i2,i3)/(sqrt_rho_r + sqrt_rho_l))**2 *0.5*&
                        ( (v1_l(i1,i2,i3)-v1_r(i1-1,i2,i3))**2 + &
                          (v2_l(i1,i2,i3)-v2_r(i1-1,i2,i3))**2 + &
                          (v3_l(i1,i2,i3)-v3_r(i1-1,i2,i3))**2 )
      ! --- Tried: 2012-03-11 (B.F.): hardly any effect ---
      !                  ( (v1_l(i1,i2,i3)-v1_r(i1-1,i2,i3))**2 )
      ! --- Tried: 2012-03-23 (B.F.): hardly any effect ---
      !P_til(  i1,i2,i3)=wrho(i1,i2,i3)*(P_r(   i1-1,i2,i3)-P_l( i1,i2,i3)) + P_l(  i1,i2,i3)
      !
      ! --- eip_til=eikp_til-ek_til ---
      eip_til(i1,i2,i3)=ei_til + P_til(i1,i2,i3)/rho_til(i1,i2,i3)
      !
      ! --- Averages of drhoei/dp_s, Gamma_1 and sound speed ---
      DrhoeiDPConsts_til(i1,i2,i3)= &
        wrho(i1,i2,i3)*((rho(i1-1,i2,i3)*ei(i1-1,i2,i3)/P(i1-1,i2,i3) + 1.0)/Gamma_1(i1-1,i2,i3)- &
                        (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3) + 1.0)/Gamma_1(i1  ,i2,i3)) +&
                        (rho(i1  ,i2,i3)*ei(i1  ,i2,i3)/P(i1  ,i2,i3) + 1.0)/Gamma_1(i1  ,i2,i3)
      Gamma_1_til=wrho(i1,i2,i3)*(Gamma_1(       i1-1,i2,i3)- &
                                  Gamma_1(       i1  ,i2,i3)) + &
                                  Gamma_1(       i1  ,i2,i3)
      cs_til(    i1,i2,i3)=sqrt(P_til(i1,i2,i3)/rho_til(i1,i2,i3)*Gamma_1_til)
      !
      ! === Characteristic velocities at cell boundaries ===
      !
      ! --- Characteristic velocities ---
      !lambda_til_nu(i1,i2,i3)=v1_til(i1,i2,i3)
      lambda_til_mi(i1,i2,i3)=v1_til(i1,i2,i3)-cs_til(i1,i2,i3)
      lambda_til_pl(i1,i2,i3)=v1_til(i1,i2,i3)+cs_til(i1,i2,i3)
      !lambda_til_nu(i1,i2,i3)=min(max(abs(v1_til(i1,i2,i3)), &
      !                                abs(v2_til(i1,i2,i3)), &
      !                                abs(v3_til(i1,i2,i3))), &
      !                            max(abs(lambda_til_mi(i1,i2,i3)), &
      !                                abs(lambda_til_pl(i1,i2,i3))))
      !lambda_til_nu(i1,i2,i3)=min(sqrt(v1_til(i1,i2,i3)**2 + &
      !                                 v2_til(i1,i2,i3)**2 + &
      !                                 v3_til(i1,i2,i3)**2), &
      !                            max(abs(lambda_til_mi(i1,i2,i3)), &
      !                                abs(lambda_til_pl(i1,i2,i3))))
      !lambda_til_nu(i1,i2,i3)=min(sqrt(v1_til(i1,i2,i3)**2 + &
      !                                 v2_til(i1,i2,i3)**2 + &
      !                                 v3_til(i1,i2,i3)**2), &
      !                            abs(v1_til(i1,i2,i3)) + &
      !                                cs_til(i1,i2,i3)     )
      lambda_til_nu(i1,i2,i3)=min(wrho(i1,i2,i3)*(vD(i1-1,i2,i3)-vD(i1,i2,i3)) + vD(i1,i2,i3), &
                                  abs(v1_til(i1,i2,i3)) + &
                                      cs_til(i1,i2,i3)     )
      !lambda_til_mi(i1,i2,i3)=lambda_til_nu(i1,i2,i3)-cs_til(i1,i2,i3)
      !lambda_til_pl(i1,i2,i3)=lambda_til_nu(i1,i2,i3)+cs_til(i1,i2,i3)
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Fix entropy in case of transsonic rarefaction wave ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Entropy fix: modified LeVeque formula (1991) ---
      if (((v1_l(i1  ,i2,i3) - cs(i1  ,i2,i3)) > 0.0) .and. &
          ((v1_r(i1-1,i2,i3) - cs(i1-1,i2,i3)) < 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#                                 /* Use if-statement */
        if (lambda_til_mi(i1,i2,i3) > 0.0) then
          lambda_til_mi(i1,i2,i3)=v1_l(i1  ,i2,i3) - cs(i1  ,i2,i3)
        else
          lambda_til_mi(i1,i2,i3)=v1_r(i1-1,i2,i3) - cs(i1-1,i2,i3)
        endif
!#
# elif (rhd_hyd_entropyfix_p01 == 2)
!#                                   /* No entropy fix */
!write(6,*) i1, i2, i3, 'No entropy fix'
!#
# else
!#     /* rhd_hyd_entropyfix_p01: use sign() */
        mask_up_mi=sign(0.5,lambda_til_mi(i1,i2,i3))+0.5
        lambda_til_mi(i1,i2,i3)=(1.0-mask_up_mi) * (v1_r(i1-1,i2,i3) - cs(i1-1,i2,i3)) + &
                                     mask_up_mi  * (v1_l(i1  ,i2,i3) - cs(i1  ,i2,i3))
!#
# endif
!#      /* rhd_hyd_entropyfix_p01 */
      endif
      !
      if (((v1_r(i1-1,i2,i3) + cs(i1-1,i2,i3)) < 0.0) .and. &
          ((v1_l(i1  ,i2,i3) + cs(i1  ,i2,i3)) > 0.0)) then
        ! --- Transsonic rarefaction wave detected ---
!#
# if (rhd_hyd_entropyfix_p01 == 0)
!#                                 /* Use if-statement */
        if (lambda_til_pl(i1,i2,i3) >= 0.0) then
          lambda_til_pl(i1,i2,i3)=v1_l(i1  ,i2,i3) + cs(i1  ,i2,i3)
        else
          lambda_til_pl(i1,i2,i3)=v1_r(i1-1,i2,i3) + cs(i1-1,i2,i3)
        endif
!#
# elif (rhd_hyd_entropyfix_p01 == 2)
!#                                   /* No entropy fix */
!write(6,*) i1, i2, i3, 'No entropy fix'
!#
# else
!#     /* rhd_hyd_entropyfix_p01: use sign() */
        mask_up_pl=sign(0.5,lambda_til_pl(i1,i2,i3))+0.5
        lambda_til_pl(i1,i2,i3)=(1.0-mask_up_pl) * (v1_r(i1-1,i2,i3) + cs(i1-1,i2,i3)) + &
                                     mask_up_pl  * (v1_l(i1  ,i2,i3) + cs(i1  ,i2,i3))
!#
# endif
!#      /* rhd_hyd_entropyfix_p01 */
      endif
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! === Projection ===
!
! --- Compute projection coefficients and fluxes of partial waves ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: RoeState1Dflux: alpha_til')
!#
# endif
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      alpha_til_1(i1,i2,i3) = 0.5/cs_til(i1,i2,i3)**2 * &
                              ( P_l(i1,i2,i3) - P_r(i1-1,i2,i3) &
                                 - rho_til(i1,i2,i3) * cs_til(i1,i2,i3) * &
                                   (v1_l(i1,i2,i3) - v1_r(i1-1,i2,i3)) )
      alpha_til_2(i1,i2,i3) = 0.5/cs_til(i1,i2,i3)**2 * &
                              ( P_l(i1,i2,i3) - P_r(i1-1,i2,i3) &
                                 + rho_til(i1,i2,i3) * cs_til(i1,i2,i3) * &
                                   (v1_l(i1,i2,i3) - v1_r(i1-1,i2,i3)) )
      alpha_til_3(i1,i2,i3) = rho_l(i1,i2,i3) - rho_r(i1-1,i2,i3) - &
                                  1.0/cs_til(i1,i2,i3)**2 * &
                                    ( P_l(i1,i2,i3) - P_r(i1-1,i2,i3) )
      alpha_til_6(i1,i2,i3) = rhoei_l(i1,i2,i3) -rhoei_r(i1-1,i2,i3) - &
                                  DrhoeiDPConsts_til(i1,i2,i3) * &
                                    ( P_l(i1,i2,i3) - P_r(i1-1,i2,i3) )
      !
      flux_alpha_til_1(i1,i2,i3)=-0.5*abs(lambda_til_mi(i1,i2,i3))*alpha_til_1(i1,i2,i3)
      flux_alpha_til_2(i1,i2,i3)=-0.5*abs(lambda_til_pl(i1,i2,i3))*alpha_til_2(i1,i2,i3)
      flux_alpha_til_3(i1,i2,i3)=-0.5*abs(lambda_til_nu(i1,i2,i3))*alpha_til_3(i1,i2,i3)
      flux_alpha_til_6(i1,i2,i3)=-0.5*abs(lambda_til_nu(i1,i2,i3))*alpha_til_6(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: RoeState1Dflux: alpha_til')
!#
# endif
!#
!
!
! === Boundary conditions IV ===
!
! --- Set fluxes explicitly to zero ---
if ((bound_l == bound_closed) .or. (bound_l == bound_closedbottom)) then
  flux_alpha_til_1(m1,:,:)=0.5*(flux_alpha_til_1(m1,:,:)-flux_alpha_til_2(m1,:,:))
  flux_alpha_til_2(m1,:,:)=    -flux_alpha_til_1(m1,:,:)
endif
if ((bound_r == bound_closed) .or. (bound_r == bound_closedbottom)) then
  flux_alpha_til_1(n1+1,:,:)=0.5*(flux_alpha_til_1(n1+1,:,:)-flux_alpha_til_2(n1+1,:,:))
  flux_alpha_til_2(n1+1,:,:)=    -flux_alpha_til_1(n1+1,:,:)
endif
!
!
! === Back projection and combination => fluxes ===
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: RoeState1Dflux: fluxes')
!#
# endif
!#
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      flux_alpha_til_123=flux_alpha_til_1(i1,i2,i3) + &
                         flux_alpha_til_2(i1,i2,i3) + &
                         flux_alpha_til_3(i1,i2,i3)
      flux_rho(i1,i2,i3)= flux_alpha_til_123 + &
                          0.5*(rho_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3) + &
                               rho_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)  )
      flux_rhov1(i1,i2,i3)= v1_til(i1,i2,i3) *  flux_alpha_til_123 - &
                            cs_til(i1,i2,i3) * (flux_alpha_til_1(i1,i2,i3) - &
                                                flux_alpha_til_2(i1,i2,i3)) + &
                     0.5*((rho_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3))*v1_r(i1-1,i2,i3) + P_r(i1-1,i2,i3) +&
                          (rho_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3))*v1_l(i1  ,i2,i3) + P_l(i1  ,i2,i3)  )
      fep(i1,i2,i3)=P_til(i1,i2,i3)/rho_til(i1,i2,i3) * (flux_alpha_til_1(i1,i2,i3)+ &
                                                           flux_alpha_til_2(i1,i2,i3)) + &
                    0.5 * P_r(i1-1,i2,i3) * v1_r(i1-1,i2,i3) + &
                    0.5 * P_l(i1  ,i2,i3) * v1_l(i1  ,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
! \label{f90:hdtransvelomode__use2}
if (par%hdTransVeloMode == 'Normal') then
  ! === Old standard method of computing flux_alpha_til_4,5 ===
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        alpha_til_4(i1,i2,i3) = v2_l(i1,i2,i3) - v2_r(i1-1,i2,i3)
        alpha_til_5(i1,i2,i3) = v3_l(i1,i2,i3) - v3_r(i1-1,i2,i3)
        !
        flux_alpha_til_4(i1,i2,i3)=-0.5*abs(lambda_til_nu(i1,i2,i3))*alpha_til_4(i1,i2,i3)
        flux_alpha_til_5(i1,i2,i3)=-0.5*abs(lambda_til_nu(i1,i2,i3))*alpha_til_5(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  !
  ! === Back projection and combination => fluxes ===
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        flux_alpha_til_123=flux_alpha_til_1(i1,i2,i3) + &
                           flux_alpha_til_2(i1,i2,i3) + &
                           flux_alpha_til_3(i1,i2,i3)
        flux_rhov2(i1,i2,i3)= v2_til(i1,i2,i3) *flux_alpha_til_123 + &
                             rho_til(i1,i2,i3) *flux_alpha_til_4(i1,i2,i3) + &
                             0.5*((rho_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)) * v2_r(i1-1,i2,i3) + &
                                  (rho_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)) * v2_l(i1  ,i2,i3)  )
        flux_rhov3(i1,i2,i3)= v3_til(i1,i2,i3) *flux_alpha_til_123 + &
                             rho_til(i1,i2,i3) *flux_alpha_til_5(i1,i2,i3) + &
                             0.5*((rho_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)) * v3_r(i1-1,i2,i3) + &
                                  (rho_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)) * v3_l(i1  ,i2,i3)  )
        ! --- flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
            (     0.5 *rho_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)) * &
            ((rhoei_r(i1-1,i2,i3)+P_r( i1-1,i2,i3))/rho_r(i1-1,i2,i3) + &
             0.5 * ( (v1_r(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2_r(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3_r(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            (     0.5 *rho_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)) * &
            ((rhoei_l(i1  ,i2,i3)+P_l( i1  ,i2,i3))/rho_l(i1,i2,i3) + &
             0.5 * ( (v1_l(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2_l(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3_l(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
!!$      fek(i1,i2,i3)=0.5 * (v1_til(i1,i2,i3)**2 + v2_til(i1,i2,i3)**2 + v3_til(i1,i2,i3)**2) * &
!!$                    flux_rho(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
else
  ! == Alternative way to compute fluxes of transversal velocities (under construction) ===
  !
  ! --- Determine (upwind direction and) flow speed from mass flux ---
  if ((par%hdTransVeloMode == 'CA1') .or. &
      (par%hdTransVeloMode == 'CA3')     ) then
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
          if (flux_rho(i1,i2,i3) > 0.0) then
            lambda_til_nu(i1,i2,i3)=flux_rho(i1,i2,i3)/rho_r(i1-1,i2,i3)
          else
            lambda_til_nu(i1,i2,i3)=flux_rho(i1,i2,i3)/rho_l(i1  ,i2,i3)
          endif
          !
        end do ! i1
      end do ! i2
    end do ! i3
  else if ((par%hdTransVeloMode == 'CA2') .or. &
           (par%hdTransVeloMode == 'CA4')     ) then
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
          lambda_til_nu(i1,i2,i3)=flux_rho(i1,i2,i3)/rho_til(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
  else
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
          lambda_til_nu(i1,i2,i3)=sqrt(v1_til(i1,i2,i3)**2 + &
                                       v2_til(i1,i2,i3)**2 + &
                                       v3_til(i1,i2,i3)**2  )
        end do ! i1
      end do ! i2
    end do ! i3
  endif
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        alpha_til_4(i1,i2,i3) = v2_l(i1,i2,i3) - v2_r(i1-1,i2,i3)
        alpha_til_5(i1,i2,i3) = v3_l(i1,i2,i3) - v3_r(i1-1,i2,i3)
        !
        flux_alpha_til_4(i1,i2,i3)=-0.5*abs(lambda_til_nu(i1,i2,i3))*alpha_til_4(i1,i2,i3)
        flux_alpha_til_5(i1,i2,i3)=-0.5*abs(lambda_til_nu(i1,i2,i3))*alpha_til_5(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  !
  ! === Back projection and combination => fluxes ===
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1+1
        ! --- Recompute some fluxes ---
        flux_rhov2(i1,i2,i3)=v2_til( i1,i2,i3) *flux_rho(        i1,i2,i3) + &
                             rho_til(i1,i2,i3) *flux_alpha_til_4(i1,i2,i3)
        flux_rhov3(i1,i2,i3)=v3_til( i1,i2,i3) *flux_rho(        i1,i2,i3) + &
                             rho_til(i1,i2,i3) *flux_alpha_til_5(i1,i2,i3)
        ! --- flux_rhoei=flux_rhoeik - ek_til*flux_rho ---
        flux_rhoei(i1,i2,i3)= &
            (eip_til(i1,i2,i3)-v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_1(i1,i2,i3) + &
            (eip_til(i1,i2,i3)+v1_til(i1,i2,i3)*cs_til(i1,i2,i3)) * flux_alpha_til_2(i1,i2,i3) + &
             rho_til(i1,i2,i3)* (v2_til(i1,i2,i3)                 * flux_alpha_til_4(i1,i2,i3) + &
                                 v3_til(i1,i2,i3)                 * flux_alpha_til_5(i1,i2,i3)) + &
                                                                    flux_alpha_til_6(i1,i2,i3) + &
            (     0.5 *rho_r(i1-1,i2,i3)*v1_r(i1-1,i2,i3)) * &
            ((rhoei_r(i1-1,i2,i3)+P_r( i1-1,i2,i3))/rho_r(i1-1,i2,i3) + &
             0.5 * ( (v1_r(i1-1,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2_r(i1-1,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3_r(i1-1,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) ) + &
            (     0.5 *rho_l(i1  ,i2,i3)*v1_l(i1  ,i2,i3)) * &
            ((rhoei_l(i1  ,i2,i3)+P_l( i1  ,i2,i3))/rho_l(i1,i2,i3) + &
             0.5 * ( (v1_l(i1  ,i2,i3)**2 - v1_til(i1,i2,i3)**2) + &
                     (v2_l(i1  ,i2,i3)**2 - v2_til(i1,i2,i3)**2) + &
                     (v3_l(i1  ,i2,i3)**2 - v3_til(i1,i2,i3)**2) ) )
        !
!!$      fek(i1,i2,i3)=0.5 * (v1_til(i1,i2,i3)**2 + v2_til(i1,i2,i3)**2 + v3_til(i1,i2,i3)**2) * &
!!$                    flux_rho(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
! === Boundary conditions V ===
!
! --- Set advection fluxes explicitly to zero ---
if ((bound_l == bound_closedbottom) .or. (bound_l == bound_closedbottom2)) then
  i1=m1
  v1_til(    i1,:,:)=0.0
  flux_rho(  i1,:,:)=0.0
  flux_rhov2(i1,:,:)=0.0
  flux_rhov3(i1,:,:)=0.0
  flux_rhoei(i1,:,:)=0.0
  fep(       i1,:,:)=0.0
endif
! --- Set fluxes explicitly to fluxes inside box (except for some some pressure terms) ---
if (bound_l == bound_inoutflow2) then
  i1=m1
  flux_rho(  i1,:,:)=rho(i1,:,:)*v1(i1,:,:)
  flux_rhov1(i1,:,:)=rho(i1,:,:)*v1(i1,:,:)**2 + 0.5*(P_r(i1-1,:,:)+P_l(i1  ,:,:))
  flux_rhov2(i1,:,:)=v2(i1,:,:)*rho(i1,:,:)*v1(i1,:,:)
  flux_rhov3(i1,:,:)=v3(i1,:,:)*rho(i1,:,:)*v1(i1,:,:)
  flux_rhoei(i1,:,:)=(ei(i1,:,:)+0.5*(P_r(i1-1,:,:)+P_l(i1  ,:,:))/rho(i1,:,:) + &
                      0.5 * ( (v1(i1,:,:)**2 - v1_til(i1,:,:)**2) + &
                              (v2(i1,:,:)**2 - v2_til(i1,:,:)**2) + &
                              (v3(i1,:,:)**2 - v3_til(i1,:,:)**2) ) &
                     ) * rho(i1,:,:)*v1(i1,:,:)
  fep(       i1,:,:)=0.5*(P_r(i1-1,:,:)+P_l(i1  ,:,:))*v1(i1,:,:)
else if (bound_l == bound_inoutflow3) then
  i1=m1
  flux_rho(  i1,:,:)=rho(i1,:,:)*v1(i1,:,:)
endif
!
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: RoeState1Dflux: fluxes')
!#
# endif
!#
!
!!$call uio_openwr(nc, 'rhd_roe1dflux_test.dat', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now' )
!!$call uio_wrlabl(nc, box_ident                 )
!!$call uio_wr(nc, 'z'               , box_id_ident        )
!!$call uio_wr(nc, dxb1              , 'dxb1'              )
!!$call uio_wr(nc, ei                , 'ei'                )
!!$call uio_wr(nc, rho               , 'rho'               )
!!$call uio_wr(nc, v1                , 'v1'                )
!!$call uio_wr(nc, v2                , 'v2'                )
!!$call uio_wr(nc, v3                , 'v3'                )
!!$call uio_wr(nc, rhoei_l           , 'rhoei_l'           )
!!$call uio_wr(nc, rho_l             , 'rho_l'             )
!!$call uio_wr(nc, v1_l              , 'v1_l'              )
!!$call uio_wr(nc, v2_l              , 'v2_l'              )
!!$call uio_wr(nc, v3_l              , 'v3_l'              )
!!$call uio_wr(nc, P_l               , 'p_l'               )
!!$call uio_wr(nc, rhoei_r           , 'rhoei_r'           )
!!$call uio_wr(nc, rho_r             , 'rho_r'             )
!!$call uio_wr(nc, v1_r              , 'v1_r'              )
!!$call uio_wr(nc, v2_r              , 'v2_r'              )
!!$call uio_wr(nc, v3_r              , 'v3_r'              )
!!$call uio_wr(nc, P_r               , 'p_r'               )
!!$call uio_wr(nc, v1_til            , 'v1_til'            )
!!$call uio_wr(nc, v2_til            , 'v2_til'            )
!!$call uio_wr(nc, v3_til            , 'v3_til'            )
!!$call uio_wr(nc, rho_til           , 'rho_til'           )
!!$call uio_wr(nc, P_til             , 'p_til'             )
!!$call uio_wr(nc, eip_til           , 'eip_til'           )
!!$call uio_wr(nc, cs_til            , 'cs_til'            )
!!$call uio_wr(nc, lambda_til_mi     , 'lambda_til_mi'     )
!!$call uio_wr(nc, lambda_til_pl     , 'lambda_til_pl'     )
!!$call uio_wr(nc, lambda_til_nu     , 'lambda_til_nu'     )
!!$call uio_wr(nc, DrhoeiDPConsts_til, 'drhoeidpconsts_til')
!!$call uio_wr(nc, alpha_til_1       , 'alpha_til_1'       )
!!$call uio_wr(nc, alpha_til_2       , 'alpha_til_2'       )
!!$call uio_wr(nc, alpha_til_3       , 'alpha_til_3'       )
!!$call uio_wr(nc, alpha_til_4       , 'alpha_til_4'       )
!!$call uio_wr(nc, alpha_til_5       , 'alpha_til_5'       )
!!$call uio_wr(nc, alpha_til_6       , 'alpha_til_6'       )
!!$call uio_wr(nc, flux_alpha_til_1  , 'flux_alpha_til_1'  )
!!$call uio_wr(nc, flux_alpha_til_2  , 'flux_alpha_til_2'  )
!!$call uio_wr(nc, flux_alpha_til_3  , 'flux_alpha_til_3'  )
!!$call uio_wr(nc, flux_alpha_til_4  , 'flux_alpha_til_4'  )
!!$call uio_wr(nc, flux_alpha_til_5  , 'flux_alpha_til_5'  )
!!$call uio_wr(nc, flux_alpha_til_6  , 'flux_alpha_til_6'  )
!!$call uio_wr(nc, flux_rho          , 'flux_rho'          )
!!$call uio_wr(nc, flux_rhoei        , 'flux_rhoei'        )
!!$call uio_wr(nc, flux_rhov1        , 'flux_rhov1'        )
!!$call uio_wr(nc, flux_rhov2        , 'flux_rhov2'        )
!!$call uio_wr(nc, flux_rhov3        , 'flux_rhov3'        )
!!$call uio_wr(nc, fep               , 'fep'               )
!!$call uio_wrlabl(nc, endbox_ident)
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call flush(nc)
!!$call uio_closrd(nc)
!
end subroutine rhd_hyd_RoeState1Dflux


!----------************-----------------------------------------------------------------------------
subroutine rhd_hyd_Dust(model, action, P, T, cs, rho_til, flux_rho, &
                        grav_flag, ComputeDelta_flag, phib_grav, &
                        bound_l, bound_r, reconstruction_order, reconstruction_type, &
                        dtime, dxb1_avg, dxb1, &
                        Output_flag, datx, datr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Dust ('rhd_hydrodynamics_Dust')
!
! PURPOSE:
!   Advect passive tracers allowing for gravitational settling.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Dust(model, action, P, T, cs, rho_til, flux_rho, &
!                     grav_flag, ComputeDelta_flag, phib_grav, &
!                     bound_l, bound_r, reconstruction_order, reconstruction_type, &
!                     dtime, dxb1_avg, dxb1, &
!                     Output_flag, datx=datx, datr=datr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   P:            ('Pressure') 3D array with pressure
!   T:            ('Temperature') 3D array with temperature
!   cs:           ('c_sound') 3D array with sound speed
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!   ComputeDelta_flag: ('Compute_Delta_flag') logical to control if new state or deltas are
!                      computed:
!                        .false.: compute new state (default),
!                        .true. : compute only deltas; changes in the conserved quantities
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   bound_l:      ('boundary_left_1') integer, left boundary type for dimension 1
!   bound_r:      ('boundary_right_1') integer, right boundary type for dimension 1
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtime:        ('delta_time') real, time step to be used
!   dxb1_avg:     ('delta_x_boundary_1_average') real, average grid spacing:
!                 value <=0: varying grid
!   dxb1:         ('delta_x_boundary_1') real, cell-boundary step size, 1. coordinate
!   Output_flag:  ('Output_flag') logical, flag that determines if additional averages are
!                 computed and put into datx
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, quc, x, time, ...
!
! OUTPUT:
!   None (output through modification of model and datx)
!
! LOCAL VARIABLES:
!   AreaFraction0: ('Area_Fraction_0') real, weighting factor for the current model chunk
!
! ROUTINES:
!   rhd_hyd_FluxState:     Compute 1D flux from state vector for single quantity
!   rhd_hyd_Rossow78tfall: Calculate dust sedimentation velocity
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_rb23mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_weight:       Produce weight array for horizontal averaging
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!   The scheme is currently restricted to plane-parallel models.
!
! PROCEDURE:
!   Compute the gas/dust transport velocity from gas mass transport flux and mass density.
!   Compute the cell-centered settling velocities from dust density and radius.
!   Limit the settling velocity to avoid supersonic settling velocities.
!   Recompute the flow speed for each dust bin.
!   Apply boundary conditions.
!   Find the upwind direction.
!   Compute the fluxes.
!   Update the cell contents with the fluxes.
!   Optional: Average the fluxes horizontally.
!
! TODO:
!   Azimuthal averages
!
! MODIFICATION HISTORY:
!   2002-10-22 (B.F.) First version
!   2002-10-29 (B.F.) High-order advection
!   2002-11-20 (B.F.) Gravitational settling term added
!   2003-08-27 (B.F.) par%C_dust01 -> par%C_dust02, par%C_dust03 -> par%C_dust04
!   2006-12-24 (B.F. Kiel) Parameter grav_flag added; reordering; phic_grav->phib_grav;
!                     recompute i_up_nu
!   2007-11-27 (B.F. Lyon) lambda_til_nu0 (bugfix for many components that should settle)
!   2007-12-11 (B.F.) Numbering for quc_advect changed
!   2007-12-12 (B.F.) rho_Dust=par%C_dust01 (from parameter file)
!   2007-12-15 (B.F.) Output of fluxes
!   2007-12-18 (B.F.) Re-arrangement of parameters
!   2008-01-14 (B.F.) Input parameter cs added; recompute lambda_til_nu only once;
!                     limit settling velocity with 1/2 of sound speed
!   2008-01-21 (B.F.) Handling of boundary conditions for v_settle
!   2008-02-26 (B.F.) Reset lambda_til_nu for each quc
!   2011-07-17 (B.F.) Input parameter dxb1_avg: switch version of flux routine depending on grid;
!                     set dtimeHO according to hdTimeIntegrationScheme
!   2011-11-14 (B.F.) New treatment of slope reduction
!   2011-11-24 (B.F.) New parameter ComputeDelta_flag, timing of reconstruction routine
!   2011-11-29 (B.F.) Insert IVDEP directives
!   2012-03-28 (B.F.) New parameters action, rho_til. Use rho_til as reference
!   2012-04-16 (B.F.) Use routine rhd_hyd_FluxState for RoeWave solver
!   2015-02-01 (B.F. Uppsala) Limit settling velocity by of sound speed: 50% -> 10%
!   2017-08-16 (B.F.) Account for new datx type in averaging of fluxes
!   2017-08-24 (B.F.) Compute AreaFraction from local and global quantities
!   2017-09-14 (B.F.) Pull calculation of averaging weights out of iquc loop; add parameter datr;
!                     average fluxes spherically
!   2018-05-07 (B.F.) Remove branch with old averaging routines; remove areafraction parameter
!   2018-05-09 (B.F.) Add dimension specifications in recontruction-routine calls
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_dat_module
use rhd_mean_module
use rhd_rec_module
use rhd_phys_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: P, T, cs
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2 , &
                model%m3:model%n3), &
                           intent(in)   :: rho_til, flux_rho
logical,                   intent(in)   :: grav_flag, ComputeDelta_flag
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: phib_grav
integer,                   intent(in)   :: bound_l, bound_r, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime, dxb1_avg
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1),&
                           intent(in)   :: dxb1
logical,                   intent(in)   :: Output_flag
type(datx_type), optional, intent(inout):: datx, datr
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, i1, i2, i3, m, n, &
                                           iquc, j, di1
real                                    :: grav, ngrain, Rgrain, eps_AlmostNoDust, dtimeHO, &
                                           AreaFraction0
real(kind=kind(0.0D+00))                :: rho_Dust
real, dimension(model%m1-1:model%n1+2, &
                model%m2  :model%n2  , &
                model%m3  :model%n3    ):: SlopeRed
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3    )  :: lambda_til_nu0, lambda_til_nu
!real, dimension(model%m1:model%n1+1, &
!                -reconstruction_order: &
!                  reconstruction_order, &
!                model%m2:model%n2  , &
!                model%m3:model%n3     ) :: quc_til
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2, &
                model%m3:model%n3)      :: qucOrho
real, dimension(model%m1-1:model%n1+1, &
                model%m2  :model%n2  , &
                model%m3  :model%n3  )  :: qucOrho_l, qucOrho_r
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3  )    :: flux_quc
real, dimension(model%m1-1:model%n1+1, &
                model%m2  :model%n2  , &
                model%m3  :model%n3   ) :: v_settle
integer, dimension(model%m1:model%n1+1, &
                   model%m2:model%n2  , &
                   model%m3:model%n3  ) :: i_up_nu
real   , dimension(model%m2:model%n2, &
                   model%m3:model%n3 )  :: weight
integer, dimension(6, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: nb6index
integer, dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: cindex
integer, dimension(:,:,:), allocatable  :: bindex
real, parameter                         :: T_limit_AbsolutelyNoDust=2500.0
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
nghost1=model%nghost1
!
! --- Input parameters from parameter file ---
! --- For prop%quc_advect(iquc) == 3 or 4  ---
rho_Dust        =par%C_dust01
eps_AlmostNoDust=par%C_dust04
!
if (Output_flag) then
  ! --- Compute quantities used for averaging ---
  !
  if ((datx%na > 0) .and. (datx%i_fquc > 0)) then
    if (action%dim == 1) then
      AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                    ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
    else if (action%dim == 2) then
      AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                    ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
    else ! (action%dim == 3)
      AreaFraction0=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                    ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
    endif
    !
    ! --- Compute weight array ---
    call rhd_mean_weight(model, AreaFraction0, weight)
    !
    ! --- Check if fluxes at right boundary should be considered ---
    if (n1 < wg%hyd%n(action%dim)) then
      di1=0
    else
      di1=1
    endif
    !
    if (merge(((datr%na > 0) .and. (datr%i_rhoei > 0)), .false., present(datr)) .and. &
        (par%grav_mode == 'central')                                                   ) then
      ! --- Compute weights and indices for spherical averaging ---
      !
      ! --- Make abbreviations ---
      m=datr%m
      n=datr%n
      !
      ! --- Allocate boundary-index array ---
      allocate(bindex(-(n-m+2):(n-m+2),m2:n2,m3:n3))
      !
      ! --- Compute weight arrays ---
      !call rhd_mean_x23weight(model, prop%length, weight, x123weight)
      !
      ! --- Compute index arrays ---
      call rhd_mean_a2rindex(m1,n1, m2,n2, m3,n3, &
                             model%nghost1, wg%hyd%m(action%dim), wg%hyd%n(action%dim), &
                             model%xc1, model%xc2(m2:n2), model%xc3(m3:n3), &
                             datr%d1(:,datr%i_xb), nb6index, bindex, cindex)
    endif
  endif ! ((datx%na > 0) .and. (datx%i_fquc > 0))
endif ! (Output_flag)
!
! --- Find upwind direction, compute flow speed ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      !
!#
#     if (rhd_hyd_upwind_p01 == 0)
!#
      if (flux_rho(i1,i2,i3) > 0.0) then
        i_up_nu(i1,i2,i3)=i1-1
      else
        i_up_nu(i1,i2,i3)=i1
      endif
!#
#     else
!#         /* rhd_hyd_upwind_p01 */
      i_up_nu(i1,i2,i3)=i1 + int(sign(0.6,-flux_rho(i1,i2,i3))-0.5)
!#
#     endif
!#          /* rhd_hyd_upwind_p01 */
      !
      !lambda_til_nu0(i1,i2,i3)=flux_rho(i1,i2,i3)/model%rho(i_up_nu(i1,i2,i3),i2,i3)
      lambda_til_nu0(i1,i2,i3)=flux_rho(i1,i2,i3)/rho_til(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Flux and update for additional field ---
do iquc=1,model%nquc
  !
  if ((prop%quc_advect(iquc) >= 1) .and. (prop%quc_advect(iquc) <= 4)) then
    !
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_start('HYD: Roe1D: dust: vsettle')
!#
#   endif
!#
    ! --- Set flow speed to original value ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          lambda_til_nu(i1,i2,i3)=lambda_til_nu0(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !
    if ((prop%quc_advect(iquc) == 2) .and. grav_flag) then
      ! --- Compute settling velocity in gravity field ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1-1,n1+1
            grav=  (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) / &
                   (dxb1(     i1        )                    )
            ngrain=(model%quc(i1,i2,i3,1)*par%C_dust02)/ &
                   (model%quc(i1,i2,i3,1)+model%quc(i1,i2,i3,2)+model%rho(i1,i2,i3))
            v_settle(i1,i2,i3)=par%C_dust04 * ngrain**(2.0/3.0) * grav / &
                               sqrt(T(i1,i2,i3))
!!$            if (i2 == 1) then
!!$              write(6,'(A,3I4,6(1X, 1PE11.4))') 'i123,v_settle,quc1,quc2,n,g,T=', &
!!$              i1,i2,i3, v_settle(i1,i2,i3), model%quc(i1,i2,i3,1), model%quc(i1,i2,i3,2), &
!!$              ngrain, grav, T(i1,i2,i3)
!!$            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !
    else if ((prop%quc_advect(iquc) == 3) .and. grav_flag) then
      ! --- Compute settling velocity in gravity field ---
      do i3=m3,n3
        do i2=m2,n2
          !DIR$ IVDEP
          do i1=m1-1,n1+1
            if (T(i1,i2,i3) < T_limit_AbsolutelyNoDust) then
              grav=  (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) / &
                     (dxb1(     i1        )                    )
              Rgrain=par%ar_dustGrainRadius(2)* &
                     (model%quc(i1,i2,i3,2)/ &
                      (model%quc(i1,i2,i3,1)+model%quc(i1,i2,i3,2)+ &
                       eps_AlmostNoDust*model%rho(i1,i2,i3)))**(1.0/3.0)
              !DIR$ FORCEINLINE
              call rhd_hyd_Rossow78tfall(p(i1,i2,i3), T(i1,i2,i3), model%rho(i1,i2,i3), &
                                         grav, Rgrain, rho_Dust, &
                                         v_settle(i1,i2,i3))
            else
              v_settle(i1,i2,i3)=0.0
            endif
            !if ((i2 == 1) .and. (i3 == 1)) then
            !  write(6,'(A,3I4,2X,6(1X, 1PE10.3))') 'i123,v_settle,quc1,quc2,R,g,T=', &
            !    i1,i2,i3, v_settle(i1,i2,i3), model%quc(i1,i2,i3,1), model%quc(i1,i2,i3,2), &
            !    Rgrain, grav, T(i1,i2,i3)
            !endif
          end do ! i1
        end do ! i2
      end do ! i3
      !
    else if ((prop%quc_advect(iquc) == 4) .and. grav_flag) then
      ! --- Compute settling velocity in gravity field (only for downward gravity field) ---
      do i3=m3,n3
        do i2=m2,n2
          !DIR$ IVDEP
          do i1=m1-1,n1+1
            if (T(i1,i2,i3) < T_limit_AbsolutelyNoDust) then
              grav=(phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3))/ &
                   (dxb1(     i1        )                    )
              !DIR$ FORCEINLINE
              call rhd_hyd_Rossow78tfall(p(i1,i2,i3), T(i1,i2,i3), model%rho(i1,i2,i3), &
                                         grav, par%ar_dustGrainRadius(iquc), rho_Dust, &
                                         v_settle(i1,i2,i3))
            else
              v_settle(i1,i2,i3)=0.0
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !
    endif ! End of computation of settling velocity
!#
#   ifdef rhd_roe1d_flux_t01
!#
    call timing_stop('HYD: Roe1D: dust: vsettle')
!#
#   endif
!#
    !
    if ((prop%quc_advect(iquc) >= 2) .and. (prop%quc_advect(iquc) <= 4) .and. grav_flag) then
      ! --- Limit settling velocity by 10% of sound speed ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1-1,n1+1
            v_settle(i1,i2,i3)=min(max(v_settle(i1,i2,i3), -0.1*cs(i1,i2,i3)), 0.1*cs(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !
      ! --- Recompute flow speed by adding settling velocity ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1+1
            lambda_til_nu(i1,i2,i3)=lambda_til_nu0(i1,i2,i3) + &
                                    0.5*(v_settle(i1-1,i2,i3)+v_settle(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !
      ! --- Boundary conditions: set transport velocity explicitly to zero ---
      if ((bound_l == bound_closed       ) .or. &
          (bound_l == bound_closed2      ) .or. &
          (bound_l == bound_closedbottom ) .or. &
          (bound_l == bound_closedbottom2)       )    then
        lambda_til_nu(m1  ,:,:)=0.0
      endif
      if ((bound_r == bound_closed       ) .or. &
          (bound_r == bound_closed2      ) .or. &
          (bound_r == bound_closedbottom ) .or. &
          (bound_r == bound_closedbottom2)       ) then
        lambda_til_nu(n1+1,:,:)=0.0
      endif
      !
      if (par%A%hdScheme == hdScheme_RoeState) then
        ! --- Use RoeState solver ---
        !
        ! --- Skip upwind calculation ---
      else
        ! --- Use RoeWave solver ---
        !
        ! --- Find new upwind direction ---
        do i3=m3,n3
         do i2=m2,n2
          do i1=m1,n1+1
            !
!#
#           if (rhd_hyd_upwind_p01 == 0)
!#
            if (lambda_til_nu(i1,i2,i3) > 0.0) then
              i_up_nu(i1,i2,i3)=i1-1
            else
              i_up_nu(i1,i2,i3)=i1
            endif
!#
#           else
!#               /* rhd_hyd_upwind_p01 */
            i_up_nu(i1,i2,i3)=i1 + int(sign(0.6,-lambda_til_nu(i1,i2,i3))-0.5)
!#
#           endif
!#                /* rhd_hyd_upwind_p01 */
            !
          end do ! i1
         end do ! i2
        end do ! i3
        !
      endif
    endif
    !
    ! --- Compute concentration (for RoeState *and* RoeWave) ---
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1-nghost1,n1+nghost1
          qucOrho(i1,i2,i3)=model%quc(i1,i2,i3,iquc) / &
                            model%rho(i1,i2,i3     )
        end do ! i1
      end do ! i2
    end do ! i3
    !
    if (par%A%hdScheme == hdScheme_RoeState) then
      ! --- Use RoeState solver ---
      !
      ! --- Reconstruct additional field and get left and right boundary values ---
      if (dxb1_avg < 0.0) then
        ! --- Spatially varying grid ---
!#
#       ifdef rhd_roe1d_flux_t01
!#
        call timing_start('HYD: Roe1Dflux: RecVec')
!#
#       endif
!#
        call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                         reconstruction_order, reconstruction_type,&
                         .false., .false., SlopeRed, SlopeRed, &
                         qucOrho, dxb1, qucOrho_l, qucOrho_r)
!#
#       ifdef rhd_roe1d_flux_t01
!#
        call timing_stop('HYD: Roe1Dflux: RecVec')
!#
#       endif
!#
      else ! (dxb1_avg > 0.0)
        ! --- Equidistant grid ---
!#
#       ifdef rhd_roe1d_flux_t01
!#
        call timing_start('HYD: Roe1Dflux: RecVecEq')
!#
#       endif
!#
        call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, 0, 0, 0, 0, &
                           reconstruction_order, reconstruction_type, &
                           .false., .false., SlopeRed, SlopeRed, &
                           qucOrho, qucOrho_l, qucOrho_r)
!#
#       ifdef rhd_roe1d_flux_t01
!#
        call timing_stop('HYD: Roe1Dflux: RecVecEq')
!#
#       endif
!#
      endif ! (dxb1_avg < 0.0)
      !
      ! --- Compute flux for additional field ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1+1
            !flux_quc(i1,i2,i3)=0.5*((qucOrho_r(i1-1,i2,i3)*rho(i1-1,i2,i3) + &
            !                         qucOrho_l(i1  ,i2,i3)*rho(i1  ,i2,i3)  ) * &
            !                        lambda_til_nu(i1,i2,i3) &
            !                          - &
            !                        (qucOrho_r(i1-1,i2,i3) - &
            !                         qucOrho_l(i1  ,i2,i3)  ) * &
            !                        (rho(i1-1,i2,i3) + rho(i1  ,i2,i3)) * &
            !                        abs(lambda_til_nu(i1,i2,i3)))
            flux_quc(i1,i2,i3)=0.5*rho_til(i1,i2,i3) * &
                               ((qucOrho_r(i1-1,i2,i3)+ &
                                 qucOrho_l(i1  ,i2,i3)  ) * &
                                lambda_til_nu(i1,i2,i3) &
                                  - &
                                (qucOrho_r(i1-1,i2,i3) - &
                                 qucOrho_l(i1  ,i2,i3)  ) * &
                                abs(lambda_til_nu(i1,i2,i3)))
          end do ! i1
        end do ! i2
      end do ! i3
      !
    else ! (par%A%hdScheme /= hdScheme_RoeState)
      ! --- Use RoeWave solver ---
      !
      ! --- High-order time step ---
      if ((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
          (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) then
        dtimeHO=0.0
      else
        dtimeHO=dtime
      endif
      !
      ! --- Compute flux (of concentration) with slope modification or extra diffusion ---
      ! --- Use SlopeRed as dummy array in all cases: it is actually not used, so far ---
      call rhd_hyd_FluxState(m1,n1, m2,n2, m3,n3, nghost1, &
                             reconstruction_order, reconstruction_type, &
                             .false., .false., SlopeRed, SlopeRed, &
                             dtimeHO, dxb1_avg, dxb1, &
                             i_up_nu, lambda_til_nu, .false., SlopeRed, &
                             qucOrho, &
                             flux_quc)
      !
      ! --- Compute flux for additional field ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1+1
            flux_quc(i1,i2,i3)=rho_til(i1,i2,i3) * flux_quc(i1,i2,i3)
          end do ! i1
        end do ! i2
      end do ! i3
      !
    endif
    !
    if (.not. ComputeDelta_flag) then
      ! --- Update additional field ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            model%quc(i1,i2,i3,iquc)=model%quc(i1,i2,i3,iquc) - &
                                       dtime/dxb1(i1) * &
                                       (flux_quc(i1+1,i2,i3)-flux_quc(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !
    else
      ! --- Compute only changes ---
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            model%quc(i1,i2,i3,iquc)=-dtime/dxb1(i1) * &
                                      (flux_quc(i1+1,i2,i3)-flux_quc(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !
    endif
    !
    if (Output_flag) then
      if ((datx%na > 0) .and. (datx%i_fquc > 0)) then
        ! --- Average fluxes horizontally ---
        call rhd_mean_x23mean(flux_quc(m1:n1+di1,:,:), &
                              datx%d(  m1:n1+di1,datx%i_fquc+(iquc-1)), weight)
      endif
      !
      if (merge(((datr%na > 0) .and. (datr%i_fquc > 0)), .false., present(datr)) .and. &
          (par%grav_mode == 'central')                                                   ) then
        ! --- Average fluxes spherically ---
        call rhd_mean_rb23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                               flux_quc(m1:n1+1,:,:), datr%d(:,datr%i_fquc+(iquc-1)), &
                               nb6index, bindex, weight)
      endif
    endif ! (Output_flag)
    !
  endif ! ((prop%quc_advect(iquc) >= 1) .and. (prop%quc_advect(iquc) <= 4))
end do ! iquc
!
if (Output_flag) then
  if (allocated(bindex)) deallocate(bindex)
endif
!
end subroutine rhd_hyd_Dust


!----------***********------------------------------------------------------------------------------
subroutine rhd_hyd_mhd(model, reconstruction_order, reconstruction_type, dtime, dxb1, &
                       flux_rho)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_mhd ('rhd_hydrodynamics_magneto-hydrodynamics')
!
! PURPOSE:
!   Advect transversal magnetic field components
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_mhd(model, reconstruction_order, reconstruction_type, dtime, dxb1, flux_rho)
!
! INPUT:
!   dtime:        ('delta_time') real, time step to be used
!   dxb1:         ('delta_x_boundary_1') real, cell-boundary step size, 1. coordinate
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, quc, B, x, time, ...
!
! OUTPUT:
!   None (output through modification of model)
!
! ROUTINES:
!   rhd_rec_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_rec_module:        Reconstruction routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-11-05 (B.F.) First version
!   2002-11-06 (B.F.) Second term in flux expression added
!   2011-07-17 (B.F.) Set dtimeHO according to hdTimeIntegrationScheme
!   2011-11-14 (B.F.) New treatment of slope reduction
!   2014-12-03 (B.F.) Add parameter dxb1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use rhd_rec_module
!#
# ifdef rhd_roe1d_flux_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
integer,                   intent(in)   :: reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime
real, dimension(model%m1-model%nghost1: &
                model%n1+model%nghost1  ), &
                           intent(in)   :: dxb1
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2 , &
                model%m3:model%n3), &
                           intent(in)   :: flux_rho
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1, i1, i2, i3
integer                                 :: j
real                                    :: dtimeHO
logical                                 :: SlopeRed_flag
real, dimension(model%m1-1:model%n1+2, &
                model%m2  :model%n2  , &
                model%m3  :model%n3    ):: SlopeRed
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3    )  :: lambda_til_nu
real, dimension(model%m1:model%n1+1, &
                -reconstruction_order: &
                  reconstruction_order, &
                model%m2:model%n2  , &
                model%m3:model%n3     ) :: B_til
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3  )    :: flux_B
integer, dimension(model%m1:model%n1+1, &
                   model%m2:model%n2  , &
                   model%m3:model%n3  ) :: i_up_nu
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
nghost1=model%nghost1
!
! \label{f90:c_slopered__use3}
! --- Perform slope reduction in case of strong density gradient ---
if ((par%C_SlopeRed > 0.0) .and. &
    (reconstruction_type /= reconstruction_type_Constant)) then
  SlopeRed=1.0-min(par%C_SlopeRed * &
                     max(model%rho(m1  :n1+1,:,:)/model%rho(m1-1:n1  ,:,:)+ &
                         model%rho(m1-1:n1  ,:,:)/model%rho(m1  :n1+1,:,:)-2.5, &
                         0.0)**2, &
                   1.0)
  SlopeRed_flag=.true.
else
  SlopeRed_flag=.false.
endif
!
! --- High-order time step ---
if ((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
    (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) then
  dtimeHO=0.0
else
  dtimeHO=dtime
endif
!
! --- Find Upwind direction, compute flow speed ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      !
!#
#     if (rhd_hyd_upwind_p01 == 0)
!#
      if (flux_rho(i1,i2,i3) > 0.0) then
        i_up_nu(i1,i2,i3)=i1-1
      else
        i_up_nu(i1,i2,i3)=i1
      endif
!#
#     else
!#         /* rhd_hyd_upwind_p01 */
      i_up_nu(i1,i2,i3)=i1 + int(sign(0.6,-flux_rho(i1,i2,i3))-0.5)
!#
#     endif
!#          /* rhd_hyd_upwind_p01 */
      !
      lambda_til_nu(i1,i2,i3)=flux_rho(i1,i2,i3)/model%rho(i_up_nu(i1,i2,i3),i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!
! === Flux and update transversal magnetic field components 2 ===
!
! --- Magnetic field in neighbor cells ---
do i3=m3,n3
  do i2=m2,n2
    do j=-reconstruction_order,reconstruction_order
      do i1=m1,n1+1
        B_til(i1,j,i2,i3)=model%Bc2(i_up_nu(i1,i2,i3)+j,i2,i3)
      end do ! i1
    end do ! j
  end do ! i2
end do ! i3
!
! --- Advected flux for transversal field ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       dtimeHO, &
                       SlopeRed_flag, .false., SlopeRed, SlopeRed, &
                       B_til, dxb1, &
                       i_up_nu, lambda_til_nu, &
                       flux_B)
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      flux_B(i1,i2,i3)=flux_B(i1,i2,i3) - 0.25 * &
                         ( model%v2(i1,i2,i3) +  model%v2(i1+1,i2,i3)) * &
                         (model%Bc1(i1,i2,i3) + model%Bc1(i1+1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Update magnetic field ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      model%Bc2(i1,i2,i3)=model%Bc2(i1,i2,i3) - &
                            dtime/(model%xb1(i1+1      )-model%xb1(i1      )) * &
                                  (   flux_B(i1+1,i2,i3)-   flux_B(i1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!
! === Flux and update transversal magnetic field components 3 ===
!
! --- Magnetic field in neighbor cells ---
do i3=m3,n3
  do i2=m2,n2
    do j=-reconstruction_order,reconstruction_order
      do i1=m1,n1+1
        B_til(i1,j,i2,i3)=model%Bc3(i_up_nu(i1,i2,i3)+j,i2,i3)
      end do ! i1
    end do ! j
  end do ! i2
end do ! i3
!
! --- Advected flux for transversal field ---
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_start('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                       reconstruction_order, reconstruction_type, &
                       dtimeHO, &
                       SlopeRed_flag, .false., SlopeRed, SlopeRed, &
                       B_til, dxb1, &
                       i_up_nu, lambda_til_nu, &
                       flux_B)
!#
# ifdef rhd_roe1d_flux_t01
!#
call timing_stop('HYD: Roe1Dflux: Fluxalpha')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      flux_B(i1,i2,i3)=flux_B(i1,i2,i3) - 0.25 * &
                         ( model%v3(i1,i2,i3) +  model%v3(i1+1,i2,i3)) * &
                         (model%Bc1(i1,i2,i3) + model%Bc1(i1+1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Update magnetic field ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      model%Bc3(i1,i2,i3)=model%Bc3(i1,i2,i3) - &
                            dtime/(model%xb1(i1+1      )-model%xb1(i1      )) * &
                                  (   flux_B(i1+1,i2,i3)-flux_B(   i1,i2,i3))
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_hyd_mhd


!----------*****************------------------------------------------------------------------------
subroutine rhd_hyd_Roe1DStep(model, action, dtime, dtime_limit, &
                             ComputeDelta_flag, CTUPredStep_flag, &
                               OutVar_flag, OutFlx_flag, OutRadialC_flag, &
                             AreaFraction, datx, datr, datc, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Roe1DStep ('rhd_hydrodynamics_Roe_1D_Step')
!
! PURPOSE:
!   Perform 1D hydrodynamics step in x1 direction, employ Roe scheme.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Roe1DStep(model, action, dtime, dtime_limit, &
!                   ComputeDelta_flag=ComputeDelta_flag, CTUPredStep_flag=CTUPredStep_flag, &
!                   OutVar_flag=OutVar_flag, OutFlx_flag=OutFlx_flag, &
!                   OutRadialC_flag=OutRadialC_flag, AreaFraction=AreaFraction, &
!                   datx=datx, datr=datr, datc=datc, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   ComputeDelta_flag: ('Compute_Delta_flag') logical to control if new state or deltas are
!                      computed:
!                        .false.: compute new state (default),
!                        .true. : compute only deltas; changes in the conserved quantities
!   CTUPredStep_flag: ('CTU_Predictor_Step_flag') logical to indicate CTU predictor step
!   OutVar_flag:  ('Output_Variable_flag') logical, compute averages of bulk quantities
!   OutFlx_flag:  ('Output_Flux_flag') logical, compute averages of fluxes
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                    volume averaging of cell centered quantities is performed
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model chunk
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for
!                 additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                       0: normal run, hydrodynamics step successfully completed
!                    1-99: "small" error (e.g. time step limit violation)
!                   >=100: "severe" error: (currently none)
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   rho:          ('rho') real 3D array, mass density
!   ei:           ('energy_internal') real 3D array, internal energy density per mass
!   v1:           ('v_1') real 3D array, velocity, 1st coordinate
!   v2:           ('v_2') real 3D array, velocity, 2nd coordinate
!   v3:           ('v_3') real 3D array, velocity, 3rd coordinate
!   eip:          ('energy_internal_pressure') real 3D array, internal + pressure energy density
!   rhoei:        ('rho_energy_internal') real 3D array, internal energy density per volume, rho*ei
!   rhov1:        ('rho_velocity_1') real 3D array, momentum, 1st dimension
!   P:            ('pressure') real 3D array, pressure on model grid
!   dPdei:        ('delta_pressure_delta_energy_internal') real 3D array, pressure derivative
!                 (constant density) on model grid
!   dPdrho:       ('delta_pressure_delta_rho') real 3D array, pressure derivative
!                 (constant internal energy)
!   T:            ('Temperature') real 3D array, temperature on model grid
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array, temperature derivative
!                 (at constant density) on model grid
!   s:            ('s') real 3D array, entropy on model grid
!   cs:           ('c_sound') real 3D array, sound velocity on model grid
!   Gamma_1:      ('Gamma_1') real 3D array, 1st adiabatic coefficient on model grid
!   sqrt_rho:     ('square_root_rho') real 3D array, square root of density on model grid
!   phic_grav:    ('phi_central_gravity) real 3D array, gravitational potential, cell centers
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!   Hp_l:         ('height_pressure_left') real 2D array, averaged pressure scale height
!                 on left (m1) side
!   Hp_r:         ('height_pressure_right') real 2D array, averaged pressure scale height
!                 on right (n1) side
!   drhov1_l:     ('delta_rho_velocity_1_left') real 2D array with 1-momentum change at left side
!   drhoeik_l:    ('delta_rho_energy_internal_kinetic_left) 2D array with energy change at left side
!   drhov1_r:     ('delta_rho_velocity_1_right') real 2D array with 1-momentum change at right side
!   drhoeik_r:    ('delta_rho_energy_internal_kinetic_right) 2D array with energy change
!                 at right side
!   wl:           ('weight_left') real 3D array, geometrical weights due to non-equidistant grid
!   rho_mean:     ('rho_mean') real 3D array, averaged density on cell boundary
!   rho_til:      ('rho_tilde') real 3D array, Roe-averaged density at cell boundary
!   ei_til:       ('energy_internal_tilde') real, Roe-averaged internal energy
!                 at cell boundary
!   v1_til:       ('velocity_1_tilde') real 3D array, Roe-averaged velocity 1 at cell boundary
!   v2_til:       ('velocity_2_tilde') real 3D array, Roe-averaged velocity 2 at cell boundary
!   v3_til:       ('velocity_3_tilde') real 3D array, Roe-averaged velocity 3 at cell boundary
!   flux_rho:     ('flux_rho') real 3D array, mass flux through cell boundary
!   flux_rhov1:   ('flux_rho_velocity_1') real 3D array, momentum 1 flux through cell boundary
!   flux_rhov2:   ('flux_rho_velocity_2') real 3D array, momentum 2 flux through cell boundary
!   flux_rhov3:   ('flux_rho_velocity_3') real 3D array, momentum 3 flux through cell boundary
!   flux_rhoei:   ('flux_rho_energy_internal') real 3D array, internal energy flux through cell bnd.
!   fep:          ('flux_energy_pressure') real 3D array, acoustic flux
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   eosinter_PdPTdT:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_bound_1DTransmit:  Adjust values in top or bottom cells for transmitting boundary condition
!   rhd_grav:              Compute gravitational potential
!   rhd_hyd_Dust:          Advect passive tracers
!   rhd_hyd_mhd:           Advect magnetic fields
!   rhd_hyd_RoeWave1Dflux: Compute 1D Roe fluxes with wave reconstructions
!   rhd_hyd_RoeState1Dflux: Compute 1D Roe fluxes with state reconstructions
!   rhd_hyd_Roe1Doutput:   Extract some averaged quantities
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box-handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!   model%nghost2=0, model%nghost3=0
!
! KEYWORDS:
!   equations of 3D compressible hydrodynamics
!   directional splitting: 1D substep
!   Roe solver
!   higher order by flux-limiters, reconstruction schemes: constant/Minmod,vanLeer,Superbee/PP
!   general EOS (tabulated values)
!   non-equidistant grid possible
!   source terms due to gravity: no operator-splitting here (constant gravity or central potential)
!   see: Roe, Leveque, Strang
!
! PROCEDURE:
!   1. Preliminaries: compute gravitational potential, fill ghost cells, apply boundary conditions
!   2. Solve EOS and compute further quantities from old values on extended grid
!   3. Compute averages and fluxes located at cell boundaries
!   4. Update basic quantities with fluxes and source terms
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   1997-07-23 (B.F.) Next
!   1997-10-03 (B.F.) Proper call of eosinter, corrections
!   1997-10-05 (B.F.) Correction: dei_rel, i_up
!   1997-10-06 (B.F.) Correction: mu
!   1997-10-08 (B.F.) Interpolate: Gamma_1, Gamma_3
!   1997-10-19 (B.F.) Glaister averages removed
!   1997-11-13 (B.F.) lambda_nu, lambda_til_nu; entropy fix for lambda_til_nu;
!                     use original lambda for back projection;
!                     alpha_til_4,5 = alpha_til_4,5/rho_til
!   1997-11-20 (B.F.) Harten/Yee additional dissipation possible
!   1999-03-30 (B.F. Copenhagen) Computation of P_til changed
!   1999-04-07 (B.F.) Spatial varying potential
!   1999-04-20 (B.F. Kiel) Drag force
!   1999-05-14 (B.F. Copenhagen) wave interaction instructions removed
!   1999-05-20 (B.F.) Correction in formula for P_til, simplification in computation of P_red
!   1999-05-31 (B.F.) Check Courant condition only if C_CourantMax > 0.0
!   1999-06-08 (B.F.) Check change in internal energy
!   1999-06-10 (B.F.) Formula for phib_grav corrected
!   1999-07-02 (B.F.) P_red-correction also in entropy wave, gravcorr_terms='default0'
!   1999-07-06 (B.F.) No P_red-correction in entropy wave
!   1999-08-19 (B.F.) Use P_avg in computation of flux_rhov1
!   1999-09-13 (B.F.) flux_alpha_til_6, ionization ignored for flux_rhoei
!   1999-09-14 (B.F.) Modifications in flux back projection
!   1999-09-15 (B.F.) drhoeidrho_til
!   1999-09-27 (B.F.) drhoeidrho_eff_til
!   2000-01-24 (B.F.) Computation of gravitational potential put into own routine
!   2000-02-01 (B.F.) Output of some averages
!   2000-02-09 (B.F.) timing included, drhoeidrho=0 outcommented
!   2000-03-17 (B.F.) drhoeidrho: new formulation
!   2000-03-18 (B.F.) DrhoeiDPConsts, flux_alpha_til_6
!   2000-06-07 (B.F.) gravcorr_terms='default3'
!   2000-12-01 (B.F. Uppsala) drhoeidrho terms removed, ifdef preprocessor commands added
!   2000-12-30 (B.F. Kiel) Call 3D version of eosinter
!   2001-01-14 (B.F. Uppsala) Use local copies of rho, ei, v1,2,3,
!                     Lines for v1 and ei swapped in computation of drag terms,
!                     Alternative computation for alpha_til (via alpha0_til)
!   2001-01-17 (B.F.) Projection, flux computation, and back projection into Roe1d_flux,
!                     Names of preprocessor macros changed,
!                     Removed: rhoeik, rhoek, ek, eik, eikp, Gamma3_til, eik_til, eikp_til
!   2001-01-22 (B.F.) Removed: rhov2, rhov3, Gamma3
!                     Computation moved: sqrt_rho, dtime_limit
!   2001-01-23 (B.F.) Definition of wlb changed
!   2001-02-01 (B.F.) Changes in formula for entropy fix
!   2001-02-04 (B.F.) wrho made scalar; cs_til_sqr, lambda_nu removed
!   2001-02-13 (B.F.) Blocks for computation of q_til, cs_til, and q_avg merged,
!                     lambda_til_nu removed; wl: 3D -> 1D
!   2001-02-14 (B.F.) Preprocessor macro IDF introduced
!   2001-02-15 (B.F.) Removed: lambda_mi, lambda_pl, DrhoeiDPConsts
!                     Turned from array into scalar: Gamma_1_til, ei_til
!                     call eosinter_PGamma1; variables v1out... in update introduced
!                     case gravcorr_terms == "default3":
!                       order of P_red limitation and computation of P_avg changed
!                     PvisNeu from rho_til instead of avg_til
!   2001-02-16 (B.F.) Computation of P_red put into Roe1d_flux, wlb -> wl, rho_mean -> scalars,
!                     Computation of source within update step,
!                     gravcorr_terms='off1' no longer supported
!   2001-02-17 (B.F.) Computation of i_up, entropy fix put into Roe1D_flux
!   2001-02-18 (B.F.) Computation of q_avg, q_til put into Roe1D_flux
!   2001-02-25 (B.F.) Use local copies of xc1 and xb1
!   2002-05-08 (B.F.) Non-positive density produces only small error (time step reduction)
!   2002-05-27 (B.F.) dtime_limit -> C_Courant * dtime_limit: apply Courant factor within routine
!   2002-08-07 (B.F.) Compiler switch 0/1 instead of undef/def
!   2002-10-21 (B.F.) Dust advection
!   2002-11-05 (B.F.) Advection of magnetic fields
!   2002-11-13 (B.F.) Call of eosinter_PGamma1 replaced by call of eosinter_PdP,
!                     bound_transmitting2
!   2002-12-16 (B.F.) Check minimum internal energy
!   2003-04-21 (B.F.) Limit decrease in time-step
!   2007-01-18 (B.F. Lyon) Dust transport with reconstruction_type -> reconstruction_quctype
!   2007-02-01 (B.F.) Optional Coriolis terms
!   2007-02-12 (B.F.) Coriolis terms put at the beginning of routine
!   2008-03-11 (B.F.) Variable dtime_limitb introduced: check for too large expansion
!   2008-03-13 (B.F.) Control parameters C_hydExpCourant, C_hydExpCourantMax added
!   2008-07-10 (B.F.) Update scheme changed
!   2011-07-17 (B.F.) C_visNeu1, C_visNeu2, C_visDrag removed; temporary variables for changes;
!                     determine dxb1_avg in Roe1D; OutRadialC_flag got inout attribute
!   2011-08-03 (B.F.) Output only changes in conserved quantities,
!                     move of call timing_start/stop('HYD: Roe1D: update')
!   2011-08-04 (B.F.) ComputeDelta_flag becomes parameter
!   2011-08-06 (B.F.) Coriolis routine removed (should be called before this one),
!                     OutRadialC_flag got only input attribute again,
!                     Parameter output replaced by OutVar_flag and OutFlx_flag
!   2011-08-26 (B.F.) New options for hdTransVeloMode
!   2011-09-01 (B.F.) Temperature diffusion: C_hydTdiff1, C_hydTdiff2
!   2011-11-14 (B.F.) Preprocessor macro IDF removed
!   2011-12-02 (B.F.) Set dtime_limitb=dtime in early error checks
!   2011-12-05 (B.F.) Account for state change by 1DTransmit also if ComputeDelta_flag==.true.
!   2011-12-12 (B.F.) Call eosinter_PdPTdT if appropriate
!   2011-12-13 (B.F.) Smaller upper limit for temperature diffusion: 1.0->1.0/6.0; sqrt->sqrt**3
!   2011-12-19 (B.F.) Compute dtime_limitb also in (rhd_roe1d_step_l01 == 1) branch
!   2011-12-24 (B.F.) Compute dtime_limitP with parameters C_hydSoundCourant(max)
!   2011-12-28 (B.F.) New parameter CTUPredStep_flag
!   2012-02-09 (B.F.) Instead of rho_old use 1/2*(rho_old+rho_new) in gravity term
!   2012-03-27 (B.F.) Compute dxb1
!   2012-06-07 (B.F.) Compute Gamma_3
!   2013-02-28 (B.F.) Change loop order for resetting phi in boundary layers; some cleanup
!   2013-08-15 (B.F.) New optional argument datc
!   2014-08-16 (B.F. Uppsala) Replace some access of quantities from model with local copies
!   2014-12-06 (B.F.) Modify formula for dtime_limitP
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_bound_module
use rhd_box_module
use rhd_phys_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!#
# ifdef rhd_roe1d_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: ComputeDelta_flag, CTUPredStep_flag, &
                                           OutVar_flag, OutFlx_flag, OutRadialC_flag
real,            optional, intent(in)   :: AreaFraction
type(datx_type), optional, intent(inout):: datx, datr
type(datc_type), optional, intent(inout):: datc
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: grav_flag, OutVar_flag0, OutFlx_flag0, T_flag, &
                                           ComputeDelta_flag0, CTUPredStep_flag0
integer                                 :: m1,n1, m2,n2, m3,n3, nghost1
integer                                 :: i1, i2, i3, di1
integer                                 :: bound_l, bound_r, nc
real                                    :: dei_min, v_factor_drag, e_factor_drag, &
                                           rhonew_min, drho, drhoei, drhov1, drhov2, drhov3, &
                                           rhoout, eiout, v1out, v2out, v3out, &
                                           w0, w1, wgrav_old, wgrav_new, &
                                           einew_min, eiTable_min, &
                                           source_rhov1, source_eik, &
                                           dtime_limitb, dtime_limitP, oodxbharm, &
                                           d1fac, d2fac, d3fac, dsum, &
                                           dxb1_avg, dxb1_var, drhoeiDiffFlux
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, dimension(3)                   :: error_loc
!
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: Hp_l, Hp_r, drhov1_l, drhoeik_l, drhov1_r, drhoeik_r
!
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1) :: xc1, dxb1
real, dimension( &
              model%m1-model%nghost1:&
              model%n1+model%nghost1+1) :: xb1
real, dimension( &
              model%m1-model%nghost1+1:&
              model%n1+model%nghost1)   :: wl
real, dimension(model%m1:model%n1)      :: dtimeoverdx
!
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1,&
                model%m2:model%n2,     &
                model%m3:model%n3)      :: rho, ei, v1, v2, v3, &
                                           P, cs, Gamma_1, Gamma_3, sqrt_rho, &
                                           phic_grav, &
                                           dPdei, dPdrho, T, dTdei, s
real, dimension(model%m1-model%nghost1:  &
                model%n1+model%nghost1+1,&
                model%m2:model%n2,       &
                model%m3:model%n3)      :: phib_grav
real, dimension(model%m1:model%n1+1, &
                model%m2:model%n2  , &
                model%m3:model%n3)      :: flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, &
                                           flux_rhoei, fep, &
                                           rho_til, v1_til, v2_til, v3_til
!
real, parameter                         :: C_dxb1_maxvar=0.01, &
                                           eps_TinyVelocity=1.0E-06
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D:')
!#
# endif
!#
!
! --- Indicate that no error occurred so far ---
ierr0=0
outstr0=''
!
! --- Make abbreviations ---
m1     =model%m1
n1     =model%n1
m2     =model%m2
n2     =model%n2
m3     =model%m3
n3     =model%n3
nghost1=model%nghost1
!
! --- Check keywords for output of certain averages ---
if (present(OutVar_flag)) then
  OutVar_flag0=OutVar_flag
else
  OutVar_flag0=.false.
endif
if (present(OutFlx_flag)) then
  OutFlx_flag0=OutFlx_flag
else
  OutFlx_flag0=.false.
endif
if (present(ComputeDelta_flag)) then
  ComputeDelta_flag0=ComputeDelta_flag
else
  ComputeDelta_flag0=.false.
endif
if (present(CTUPredStep_flag)) then
  CTUPredStep_flag0=CTUPredStep_flag
else
  CTUPredStep_flag0=.false.
endif
!
!
! === Boundary conditions I ========================================================================
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
! --- Boundary conditions by filling of ghost cells: axes only ---
call rhd_bound_1DGhost(model, action, grid_mode=1, bulk_mode=0)
!
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D: grav')
!#
# endif
!#
! === Gravitational potential ======================================================================
!call rhd_grav(model, action, grav_flag, phic_grav, phib_grav, gc_grav=gc_grav)
call rhd_grav(model, action, grav_flag, phic_grav, phib_grav)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D: grav')
call timing_start('HYD: Roe1D: boundaries')
call timing_start('HYD: Roe1D: boundaries: phi')
!#
# endif
!#
!
!
! === Boundary conditions II =======================================================================
!
! --- Boundary conditions for gravity: potential set constant in ghost zones ---
if (grav_flag) then
  if (.not. ((bound_l == bound_periodic        ) .or. &
             (bound_l == bound_transmitting    ) .or. &
             (bound_l == bound_transmitting2   ) .or. &
             (bound_l == bound_inoutflow2      ) .or. &
             (bound_l == bound_inoutflow3      ) .or. &
             (bound_l == bound_closed2         ) .or. &
             (bound_l == bound_closedbottom2   ) .or. &
             (bound_l == bound_InternalNext    ) .or. &
             (bound_l == bound_InternalPeriodic)       )) then
    do i3=m3,n3
      do i2=m2,n2
        phic_grav(m1-nghost1:m1-1,i2,i3)=phic_grav(m1,i2,i3)
        phib_grav(m1-nghost1:m1  ,i2,i3)=phic_grav(m1,i2,i3)
      end do ! i2
    end do ! i3
  endif
  if (.not. ((bound_r == bound_periodic        ) .or. &
             (bound_r == bound_transmitting    ) .or. &
             (bound_r == bound_transmitting2   ) .or. &
             (bound_r == bound_closed2         ) .or. &
             (bound_r == bound_InternalNext    ) .or. &
             (bound_r == bound_InternalPeriodic)       )) then
    do i3=m3,n3
      do i2=m2,n2
        phic_grav(n1+1:n1+nghost1  ,i2,i3)=phic_grav(n1,i2,i3)
        phib_grav(n1+1:n1+nghost1+1,i2,i3)=phic_grav(n1,i2,i3)
      end do ! i2
    end do ! i3
  endif
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D: boundaries: phi')
!#
# endif
!#
if ((bound_l == bound_transmitting ) .or. (bound_r == bound_transmitting ) .or. &
    (bound_l == bound_transmitting2) .or. (bound_r == bound_transmitting2) .or. &
    (bound_l == bound_transmitting3) .or. (bound_r == bound_transmitting3)     ) then
  ! --- Transmitting lower and upper boundary ---
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('HYD: Roe1D: boundaries: transmit')
!#
# endif
!#
  call rhd_bound_1DTransmit(model, action, dtime, ComputeDelta_flag0, grav_flag, phib_grav, &
                            drhov1_l, drhoeik_l, drhov1_r, drhoeik_r, &
                            Hp_l=Hp_l, Hp_r=Hp_r)
  !
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('HYD: Roe1D: boundaries: transmit')
!#
# endif
!#
endif
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D: boundaries: ghost')
!#
# endif
!#
!
! --- Boundary conditions by filling of ghost cells: bulk data only ---
call rhd_bound_1DGhost(model, action, grid_mode=0, bulk_mode=1, Hp_l=Hp_l, Hp_r=Hp_r)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D: boundaries: ghost')
call timing_stop('HYD: Roe1D: boundaries')
!#
# endif
!#
! --- Save original values into local copies ---
xc1=model%xc1
xb1=model%xb1
rho=model%rho
ei =model%ei
v1 =model%v1
v2 =model%v2
v3 =model%v3
!
! --- Compute radius for central potential ---
!if (par%grav_mode == 'central') then
!  do i3=m3,n3
!    do i2=m2,n2
!      do i1=m1-nghost1,n1+nghost1
!        Radius(i1,i2,i3)=sqrt(xc1(i1)**2 + xc2(i2)**2 + model%xc3(i3)**2)
!      end do ! i1
!    end do ! i2
!  end do ! i3
!endif
!
!
! === EOS, further quantities, check Courant condition =============================================
!
! --- Solve EOS ---
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D: eosinter')
!#
# endif
!#
T_flag=(OutVar_flag0 .or. &
       (((par%C_hydTDiffLin > 0.0) .or. (par%C_hydTDiffMach > 0.0)) .and. .not. CTUPredStep_flag0))
if (model%nquc > 0) then
  T_flag=(T_flag .or. any(prop%quc_advect > 1))
endif
! \label{f90:hdentropywavemode__use4}\label{f90:hdenthalpyavgmode__use4}
if ((OutVar_flag0                           ) .or. &
    (par%C_hydsDiffVelo    >   0.0          ) .or. &
    (par%hdEntropyWaveMode /= 'Normal'      ) .or. &
    (par%hdEnthalpyAvgMode == 'CA1gMps'     ) .or. &
    (par%hdEnthalpyAvgMode == 'CA1gMps_Pred') .or. &
    (par%hdEnthalpyAvgMode == 'CA5gMps'     ) .or. &
    (par%hdEnthalpyAvgMode == 'CA6gMps'     ) .or. &
    (par%hdEnthalpyAvgMode == 'CA1Ms'       ) .or. &
    (par%hdEnthalpyAvgMode == 'CA1s'        ) .or. &
    (par%hdEnthalpyAvgMode == 'CA2s'        )       ) then
  call eosinter_all(   rho, ei, &
                       P=P, dPdrho=dPdrho, dPde=dPdei, &
                       T=T,                dTde=dTdei, &
                       s=s)
else if (T_flag) then
  call eosinter_PdPTdT(rho, ei, &
                       P=P, dPdrho=dPdrho, dPde=dPdei, &
                       T=T,                dTde=dTdei)
else
  call eosinter_PdP(   rho, ei, &
                       P=P, dPdrho=dPdrho, dPde=dPdei)
endif
Gamma_1=dPdrho*rho/P + dPdei/rho
Gamma_3=1.0+dPdei/rho
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop( 'HYD: Roe1D: eosinter')
!#
# endif
!#
!
! --- Check quantities: P > 0, Gamma_1 >= 1 ---
if (any(P <= 0.0)) then
  ierr0=4
  error_loc=minloc(P)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+ m3
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  write(error_locstr,'(1PE12.4)') P(  error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Pressure non-positive (' // trim(outstr0) // ')'
  !
  dtime_limit=0.7*dtime
  goto 998
endif ! (any(P <= 0.0))
if (any(Gamma_1 < 1.0)) then
  ierr0=5
  error_loc=minloc(Gamma_1)
  error_loc(1)=error_loc(1)-1+(m1-nghost1)
  error_loc(2)=error_loc(2)-1+ m2
  error_loc(3)=error_loc(3)-1+ m3
  write(error_locstr,'(I8)') error_loc(1)
  outstr0=trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(2)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(I8)') error_loc(3)
  outstr0=trim(outstr0) // ',' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') rho(error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // '; ' // trim(adjustl(error_locstr))
  write(error_locstr,'(E13.6)') ei( error_loc(1), error_loc(2), error_loc(3))
  outstr0=trim(outstr0) // ', ' // trim(adjustl(error_locstr))
  outstr0='Gamma_1 smaller than 1 (' // trim(outstr0) // ')'
  !
  dtime_limit=0.7*dtime
  goto 998
endif ! (any(Gamma_1 < 1.0))
!
! --- Compute grid step and determine its variation ---
dxb1=xb1(m1-nghost1+1: n1+nghost1+1) - &
     xb1(m1-nghost1  : n1+nghost1  )
dxb1_avg=(xb1(n1+nghost1+1) - &
          xb1(m1-nghost1  )) / &
         float(n1-m1+1+2*nghost1)
dxb1_var=maxval(abs(dxb1-dxb1_avg)) / dxb1_avg
if (dxb1_var > C_dxb1_maxvar) dxb1_avg=-1.0
!
! --- Compute geometrical weights due to non-equidistant grid ---
wl(m1-nghost1+1:n1+nghost1  )=(xc1(m1-nghost1+1:n1+nghost1  )- &
                               xb1(m1-nghost1+1:n1+nghost1  )) / &
                              (xc1(m1-nghost1+1:n1+nghost1  )- &
                               xc1(m1-nghost1  :n1+nghost1-1))
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D: derived q')
!#
# endif
!#
dtime_limit =0.1*huge(dtime_limit )
dtime_limitP=0.1*huge(dtime_limitP)
dtime_limitb=0.1*huge(dtime_limitb)
!
do i3=m3,n3
  do i2=m2,n2
    do i1=m1-nghost1,n1+nghost1
      ! --- Compute derived quantities: sound speed, square root of density, ---
      ! --- maximum allowed time step                                        ---
      cs(      i1,i2,i3)=sqrt(P(i1,i2,i3)/rho(i1,i2,i3)*Gamma_1(i1,i2,i3))
      sqrt_rho(i1,i2,i3)=sqrt(rho(i1,i2,i3))
      dtime_limit       =min(dtime_limit, &
                             dxb1(i1) / (abs(v1(i1,i2,i3))+cs(i1,i2,i3)))
    end do ! i1
    do i1=m1-nghost1,n1+nghost1-1
      ! --- Compute "Courant" time step based on relative expansion (positive velocity difference) -
      ! --- Avoid division by zero by getting lower limit from sound speed                       ---
      dtime_limitb      =min(dtime_limitb, &
                             (xc1(i1+1)-xc1(i1)) / &
                             (max(v1(i1+1,i2,i3)-v1(i1,i2,i3), &
                                  eps_TinyVelocity*(cs(i1+1,i2,i3)+cs(i1,i2,i3)))))
    end do ! i1
  end do ! i2
end do ! i3
!
if (wg%wld%n(action%dimV(2)) > wg%wld%m(action%dimV(2))) then
  d2fac=1.0
else
  d2fac=0.0
endif
if (wg%wld%n(action%dimV(3)) > wg%wld%m(action%dimV(3))) then
  d3fac=1.0
else
  d3fac=0.0
endif
if (par%hdsplit == 'CTU') then
  ! d1fac=0.0
  dsum=d2fac+d3fac
  if (dsum == 0.0) then
    d2fac=0.5
    d3fac=0.5
  else
    d2fac=d2fac/dsum
    d3fac=d3fac/dsum
  endif
else if (par%hdsplit == 'unsplit') then
  dsum =1.0+d2fac+d3fac
  d1fac=1.0  /dsum
  d2fac=d2fac/dsum
  d3fac=d3fac/dsum
! else
! d1fac=1.0
! d2fac=0.0
! d3fac=0.0
endif
!
if (par%hdsplit == 'CTU') then
  do i3=m3,n3
    do i2=m2,n2
      !oodxbharm=0.5*(1.0/(model%xb2(i2+1)-model%xb2(i2))+1.0/(model%xb3(i3+1)-model%xb3(i3)))
      oodxbharm=d2fac/(model%xb2(i2+1)-model%xb2(i2)) + d3fac/(model%xb3(i3+1)-model%xb3(i3))
      do i1=m1-nghost1,n1+nghost1
        dtime_limitP    =min(dtime_limitP, &
                             1.0/(                oodxbharm *cs(i1,i2,i3)) )
      end do ! i1
    end do ! i2
  end do ! i3
else if (par%hdsplit == 'unsplit') then
  do i3=m3,n3
    do i2=m2,n2
      oodxbharm=d2fac/(model%xb2(i2+1)-model%xb2(i2)) + d3fac/(model%xb3(i3+1)-model%xb3(i3))
      do i1=m1-nghost1,n1+nghost1
        dtime_limitP    =min(dtime_limitP, &
                             1.0/((d1fac/dxb1(i1)+oodxbharm)*cs(i1,i2,i3)) )
      end do ! i1
    end do ! i2
  end do ! i3
else ! (par%hdsplit == '123')
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1-nghost1,n1+nghost1
        dtime_limitP    =min(dtime_limitP, &
                             dxb1(i1)/                       cs(i1,i2,i3)  )
      end do ! i1
    end do ! i2
  end do ! i3
endif ! (par%hdsplit)
!
!!$! --- Derive further quantities: sound speed, square root of density, maximum allowed time step --
!!$cs         =sqrt(P/rho*Gamma_1)
!!$sqrt_rho   =sqrt(rho)
!!$dtime_limit=minval( ( xb1(m1-nghost1+1: n1+nghost1+1) - &
!!$                      xb1(m1-nghost1  : n1+nghost1  )  ) / &
!!$                    maxval(maxval(abs(v1)+cs,2),2) )
!!$! --- Compute "Courant" time step based on relative expansion (positive velocity difference) ---
!!$! --- Avoid division by zero by getting lower limit from sound speed                         ---
!!$dtime_limitb=minval( (xc1(m1-nghost1+1: n1+nghost1  ) - &
!!$                      xc1(m1-nghost1  : n1+nghost1-1)  ) / &
!!$                  maxval(maxval(max(                  v1(m1-nghost1+1: n1+nghost1  ,:,:)- &
!!$                                                      v1(m1-nghost1  : n1+nghost1-1,:,:) , &
!!$                                    eps_TinyVelocity*(cs(m1-nghost1+1: n1+nghost1  ,:,:)+ &
!!$                                                      cs(m1-nghost1  : n1+nghost1-1,:,:))),2),2) )
!!$dtime_limitP=0.1*huge(dtime_limitP)
!!$do i3=m3,n3
!!$  do i2=m2,n2
!!$    dxbharm=2.0/(1.0/(model%xb2(i2+1)-model%xb2(i2))+1.0/(model%xb3(i3+1)-model%xb3(i3)))
!!$    dtime_limitP      =min(dtime_limitP, &
!!$                           dxbharm / maxval(cs(:,i2,i3)))
!!$  end do ! i2
!!$end do ! i3
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D: derived q')
!#
# endif
!#
!
! \label{f90:c_courantmax__use1}\label{f90:c_courant__use1}
! --- Check Courant condition; standard: abs(v1)+cs, cs, and expansion: max(v1(i1+1)-v1(i1),0) ---
!
if ((par%C_CourantMax > 0.0) .and. (dtime > par%C_CourantMax * dtime_limit)) then
  ! --- Time step too large: return without further computation ---
  !
  ! --- Limit decrease in time-step ---
  dtime_limit=max(par%C_Courant*dtime_limit, 0.1*dtime)
  ierr0=1
  outstr0='Time step too large for Courant condition'
  goto 998
endif
!
! \label{f90:c_hydsoundcourantmax__use1}\label{f90:c_hydsoundcourant__use1}
if ((par%C_hydSoundCourantMax > 0.0) .and. (dtime > par%C_hydSoundCourantMax * dtime_limitP)) then
  ! --- Time step too large: return without further computation ---
  !
  ! --- Limit decrease in time-step ---
  dtime_limit=max(par%C_hydSoundCourant*dtime_limitP, 0.1*dtime)
  ierr0=9
  outstr0='Time step too large for (pure-sound) Courant condition'
  goto 998
endif
!
! \label{f90:c_hydexpcourantmax__use1}\label{f90:c_hydexpcourant__use1}
if ((par%C_hydExpCourant > 0.0) .and. (par%C_hydExpCourantMax > 0.0) .and. &
    (dtime > par%C_hydExpCourantMax * dtime_limitb)) then
  ! --- Time step too large: return without further computation ---
  !
  ! --- Limit decrease in time-step ---
  dtime_limit=max(par%C_hydExpCourant*dtime_limitb, 0.1*dtime)
  ierr0=8
  outstr0='Time step too large for expansion Courant condition'
  goto 998
endif
!
! === Main step: compute boundary located (flux-like) averages and fluxes ==========================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D: fluxes')
!#
# endif
!#
!
if (par%A%hdScheme == hdScheme_RoeState) then
  ! --- Call Roe solver based on state reconstruction (experimental) ---
  call rhd_hyd_RoeState1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
                         bound_l, bound_r, par%A%reconstruction_order, par%A%reconstruction_type, &
                         dtime, dxb1_avg, dxb1, wl, &
                         rho, ei, v1, v2, v3, P, Gamma_1, cs, sqrt_rho, &
                         phic_grav, phib_grav, &
                         rho_til, v1_til, v2_til, v3_til, &
                         flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep)
else
  ! --- Call Roe solver based on wave reconstruction (tried and tested) ---
  call rhd_hyd_RoeWave1Dflux(m1,n1, m2,n2, m3,n3, nghost1, &
                         bound_l, bound_r, par%A%reconstruction_order, par%A%reconstruction_type, &
                         dtime, dxb1_avg, dxb1, wl, &
                         rho, ei, v1, v2, v3, P, Gamma_1, Gamma_3, cs, s, T, dTdei, sqrt_rho, &
                         OutFlx_flag0, grav_flag, phic_grav, phib_grav, &
                         rho_til, v1_til, v2_til, v3_til, &
                         flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep)
endif
!
! \label{f90:c_hydtdifflin__use1}
if (((par%C_hydTDiffLin > 0.0) .or. (par%C_hydTDiffMach > 0.0)) .and. .not. CTUPredStep_flag0) then
  ! --- Additional energy diffusion in case of local temperature minimum ---
  ! --- ToDo: output of flux ---
  !
  if (bound_l == bound_periodic) then
    di1=0
  else
    di1=1
  endif
  !
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1+di1,n1-di1
        if ((T(i1,i2,i3) < T(i1-1,i2,i3)) .and. &
            (T(i1,i2,i3) < T(i1+1,i2,i3))) then
          ! --- Local temperature minimum detected: compute energy flux into it ---
          drhoeiDiffFlux=min(T(i1-1,i2,i3)-T(i1,i2,i3), &
                             T(i1+1,i2,i3)-T(i1,i2,i3)) * &
                         rho(i1,i2,i3)/dTdei(i1,i2,i3)*cs(i1,i2,i3)
          flux_rhoei(i1  ,i2,i3)=flux_rhoei(i1  ,i2,i3) + &
                               min((  0.1*par%C_hydTDiffLin) + &
                                   (100.0*par%C_hydTDiffMach) * &
                                                     sqrt(((v1(i1,i2,i3)-v1(i1-1,i2,i3))**2 + &
                                                           (v2(i1,i2,i3)-v2(i1-1,i2,i3))**2 + &
                                                           (v3(i1,i2,i3)-v3(i1-1,i2,i3))**2  ) / &
                                                          ( cs(i1,i2,i3)*cs(i1-1,i2,i3)      ))**3,&
                                   1.0/6.0 &
                                  ) * &
                               drhoeiDiffFlux
        endif
      end do ! i1
      do i1=m1+di1,n1-di1
        if ((T(i1,i2,i3) < T(i1-1,i2,i3)) .and. &
            (T(i1,i2,i3) < T(i1+1,i2,i3))) then
          ! --- Local temperature minimum detected: compute energy flux into it ---
          drhoeiDiffFlux=min(T(i1-1,i2,i3)-T(i1,i2,i3), &
                             T(i1+1,i2,i3)-T(i1,i2,i3)) * &
                         rho(i1,i2,i3)/dTdei(i1,i2,i3)*cs(i1,i2,i3)
          flux_rhoei(i1+1,i2,i3)=flux_rhoei(i1+1,i2,i3) - &
                               min((  0.1*par%C_hydTDiffLin) + &
                                   (100.0*par%C_hydTDiffMach) * &
                                                     sqrt(((v1(i1,i2,i3)-v1(i1+1,i2,i3))**2 + &
                                                           (v2(i1,i2,i3)-v2(i1+1,i2,i3))**2 + &
                                                           (v3(i1,i2,i3)-v3(i1+1,i2,i3))**2  ) / &
                                                          ( cs(i1,i2,i3)*cs(i1+1,i2,i3)      ))**3,&
                                   1.0/6.0 &
                                  ) * &
                               drhoeiDiffFlux
        endif
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D: fluxes')
!#
# endif
!#
!
!
if (OutVar_flag0 .or. OutFlx_flag0) then
  ! === Put additional output data into datx, datr, and datc =======================================
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('HYD: Roe1D: (output)')
!#
# endif
!#
  call rhd_hyd_Roe1Doutput(model, action%dim, AreaFraction, &
                           OutVar_flag0, OutFlx_flag0, OutRadialC_flag, &
                           P, T, s, Gamma_1, Gamma_3, dTdei, dPdei, &
                           phic_grav, phib_grav, &
                           rho_til, v1_til, v2_til, v3_til, &
                           flux_rho, flux_rhov1, flux_rhov2, flux_rhov3, flux_rhoei, fep, &
                           datx=datx, datr=datr, datc=datc)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('HYD: Roe1D: (output)')
!#
# endif
!#
endif
!
! === Check new density ============================================================================
!
! --- dtimeoverdx=dt/dx ---
dtimeoverdx=dtime/dxb1(m1:n1)
!
if (.not. ComputeDelta_flag0) then
  ! --- Check if density is positive everywhere ---
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('HYD: Roe1D: checkrho')
!#
# endif
!#
  rhonew_min=huge(rhonew_min)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        rhonew_min=min(rhonew_min, &
                       rho(i1,i2,i3) - dtimeoverdx(i1)*(flux_rho(i1+1,i2,i3)-flux_rho(i1,i2,i3)))
      end do ! i1
    end do ! i2
  end do ! i3
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('HYD: Roe1D: checkrho')
!#
# endif
!#
  if (rhonew_min <= 0.0) then
    dtime_limit=0.7*dtime
    ierr0  =6
    outstr0='Density becomes non-positive'
    goto 998
  endif
endif
!
! === Handle additional advected quantities and magnetic fields ====================================
!
if (model%nquc > 0) then
  ! --- Handle additional passive tracer fields ---
  !
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('HYD: Roe1D: dust')
!#
# endif
!#
  !
  call rhd_hyd_Dust(model, action, P, T, cs, rho_til, flux_rho, &
                    grav_flag, ComputeDelta_flag0, phib_grav, &
                    bound_l, bound_r, par%A%reconstruction_order, par%A%reconstruction_quctype, &
                    dtime, dxb1_avg, dxb1, &
                    OutFlx_flag0, datx=datx, datr=datr)
  !
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('HYD: Roe1D: dust')
!#
# endif
!#
  !
endif
!
if ((model%nBc > 0) .and. (par%A%hdScheme == hdScheme_RoeMagKin)) then
  ! --- Advect magnetic fields ---
  call rhd_hyd_mhd(model, par%A%reconstruction_order, par%A%reconstruction_type, &
                   dtime, dxb1, &
                   flux_rho)
endif
!
! === Update with computed fluxes and source terms =================================================
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_start('HYD: Roe1D: update')
!#
# endif
!#
!
! \label{f90:c_gravmu__use1}
! --- Choose time-centering of gravity source terms ---
if (par%C_gravmu < 0.0) then
  ! --- Use old defaults ---
  if ((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
      (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) then
    ! --- For Runge-Kutta use only first-order terms (density at old time step) ---
    wgrav_new=0.0
  else
    ! --- Use time-centered density ---
    wgrav_new=0.5
  endif
else
  ! --- Use input parameter as weight ---
  wgrav_new=par%C_gravmu
endif
!wgrav_old=1.0-wgrav_new
!
if (.not. ComputeDelta_flag0) then
  ! --- Update all quantities ---
  !
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        ! --- Compute change in mass density and update it ---
        drho  =-dtimeoverdx(i1) * (flux_rho(  i1+1,i2,i3)-flux_rho(  i1,i2,i3))
        rhoout=rho(i1,i2,i3) + drho
        !
        ! --- Compute source terms due to gravity ---
        w0          =(phic_grav(i1  ,i2,i3)-phib_grav(i1,i2,i3)) / &
                     (dxb1(     i1        )                    )
        w1          =(phib_grav(i1+1,i2,i3)-phic_grav(i1,i2,i3)) / &
                     (dxb1(     i1        )                    )
        source_eik  =w0*flux_rho(i1,i2,i3) + w1*flux_rho(i1+1,i2,i3)
        source_rhov1=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) /&
                                                      (dxb1(     i1        )                    )
        !
        ! --- Compute changes and update ---
        drhov1=-dtimeoverdx(i1) * (flux_rhov1(i1+1,i2,i3)-flux_rhov1(i1,i2,i3)) + &
                dtime           * source_rhov1
        v1out =v1( i1,i2,i3) + (drhov1 - drho*v1(i1,i2,i3))/rhoout
        drhov2=-dtimeoverdx(i1) * (flux_rhov2(i1+1,i2,i3)-flux_rhov2(i1,i2,i3))
        v2out =v2( i1,i2,i3) + (drhov2 - drho*v2(i1,i2,i3))/rhoout
        drhov3=-dtimeoverdx(i1) * (flux_rhov3(i1+1,i2,i3)-flux_rhov3(i1,i2,i3))
        v3out =v3( i1,i2,i3) + (drhov3 - drho*v3(i1,i2,i3))/rhoout
        drhoei=-dtimeoverdx(i1) * (flux_rhoei(i1+1,i2,i3)-flux_rhoei(i1,i2,i3)) + &
                dtime           * source_eik - &
               ( dtimeoverdx(i1) * 0.5 * &
                 ( ( (v1_til(i1+1,i2,i3)**2-v1out**2) + &
                     (v2_til(i1+1,i2,i3)**2-v2out**2) + &
                     (v3_til(i1+1,i2,i3)**2-v3out**2)  &
                   )                                     * flux_rho(i1+1,i2,i3) - &
                   ( (v1_til(i1  ,i2,i3)**2-v1out**2) + &
                     (v2_til(i1  ,i2,i3)**2-v2out**2) + &
                     (v3_til(i1  ,i2,i3)**2-v3out**2)  &
                   )                                     * flux_rho(i1  ,i2,i3) &
                 ) - &
                  0.5 * rho(i1,i2,i3) * &
                       ( (v1(i1,i2,i3)**2-v1out**2) + &
                         (v2(i1,i2,i3)**2-v2out**2) + &
                         (v3(i1,i2,i3)**2-v3out**2)  &
                       ) &
               )
        eiout =ei(i1,i2,i3)  + (drhoei - drho*ei(i1,i2,i3))/rhoout
        !
        model%rho(i1,i2,i3)=rhoout
        model%v1( i1,i2,i3)=v1out
        model%v2( i1,i2,i3)=v2out
        model%v3( i1,i2,i3)=v3out
        model%ei( i1,i2,i3)=eiout
      end do ! i1
    end do ! i2
  end do ! i3
  !
else ! (ComputeDelta_flag0)
  ! --- Compute only changes of conserved quantities ---
  !
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        ! --- Compute change in mass density ---
        drho  =-dtimeoverdx(i1) * (flux_rho(  i1+1,i2,i3)-flux_rho(  i1,i2,i3))
        !
        ! --- Compute source terms due to gravity ---
        w0          =(phic_grav(i1  ,i2,i3)-phib_grav(i1,i2,i3)) / &
                     (dxb1(     i1        )                    )
        w1          =(phib_grav(i1+1,i2,i3)-phic_grav(i1,i2,i3)) / &
                     (dxb1(     i1        )                    )
        source_eik  =w0*flux_rho(i1,i2,i3) + w1*flux_rho(i1+1,i2,i3)
        source_rhov1=(rho(i1,i2,i3)+wgrav_new*drho) * (phib_grav(i1+1,i2,i3)-phib_grav(i1,i2,i3)) /&
                                                      (dxb1(     i1        )                    )
        !
        ! --- Compute changes ---
        drhov1=-dtimeoverdx(i1) * (flux_rhov1(i1+1,i2,i3)-flux_rhov1(i1,i2,i3)) + &
                dtime           * source_rhov1
        drhov2=-dtimeoverdx(i1) * (flux_rhov2(i1+1,i2,i3)-flux_rhov2(i1,i2,i3))
        drhov3=-dtimeoverdx(i1) * (flux_rhov3(i1+1,i2,i3)-flux_rhov3(i1,i2,i3))
        drhoei=-dtimeoverdx(i1) * (flux_rhoei(i1+1,i2,i3)-flux_rhoei(i1,i2,i3)) + &
                dtime           * source_eik - &
               ( dtimeoverdx(i1) * 0.5 * &
                 ( ( (v1_til(i1+1,i2,i3)**2) + &
                     (v2_til(i1+1,i2,i3)**2) + &
                     (v3_til(i1+1,i2,i3)**2)  &
                   )                                     * flux_rho(i1+1,i2,i3) - &
                   ( (v1_til(i1  ,i2,i3)**2) + &
                     (v2_til(i1  ,i2,i3)**2) + &
                     (v3_til(i1  ,i2,i3)**2)  &
                   )                                     * flux_rho(i1  ,i2,i3) &
                 ) &
               )
        !
        ! --- Put only changes in conserved quantities into output model ---
        model%rho(i1,i2,i3)=drho
        model%v1( i1,i2,i3)=drhov1
        model%v2( i1,i2,i3)=drhov2
        model%v3( i1,i2,i3)=drhov3
        model%ei( i1,i2,i3)=drhoei
      end do ! i1
    end do ! i2
  end do ! i3
  !
  if ((bound_l == bound_transmitting ) .or. &
      (bound_l == bound_transmitting2) .or. &
      (bound_l == bound_transmitting3)     ) then
    ! --- Add changes in left boundary layer cells due to transmitting boundary condition ---
    model%v1(m1,m2:n2,m3:n3)=model%v1(m1,m2:n2,m3:n3)+drhov1_l
    model%ei(m1,m2:n2,m3:n3)=model%ei(m1,m2:n2,m3:n3)+drhoeik_l
  endif
  if ((bound_r == bound_transmitting ) .or. &
      (bound_r == bound_transmitting2) .or. &
      (bound_r == bound_transmitting3)     ) then
    ! --- Add changes in right boundary layer cells due to transmitting boundary condition ---
    model%v1(n1,m2:n2,m3:n3)=model%v1(n1,m2:n2,m3:n3)+drhov1_r
    model%ei(n1,m2:n2,m3:n3)=model%ei(n1,m2:n2,m3:n3)+drhoeik_r
  endif
  !
endif ! (ComputeDelta_flag0)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D: update')
!#
# endif
!#
!
if (.not. ComputeDelta_flag0) then
  ! --- Check if internal energy is above minimum table value everywhere ---
  call tabinter_getinfo(x2min=eiTable_min)
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_start('HYD: Roe1D: checkei')
!#
# endif
!#
  einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
!#
# ifdef rhd_roe1d_step_t01
!#
  call timing_stop('HYD: Roe1D: checkei')
!#
# endif
!#
  if (einew_min < eiTable_min) then
    dtime_limit=0.7*dtime
    ierr0  =7
    outstr0='Internal energy below minimum table value'
    goto 998
  endif
  !
  ! \label{f90:c_maxeichange__use1}
  ! --- Check internal energy ---
  if (par%C_maxeiChange > 0.0) then
!#
#   ifdef rhd_roe1d_step_t01
!#
    call timing_start('HYD: Roe1D: checkdei')
!#
#   endif
!#
    dei_min=min(minval((model%ei(m1:n1,m2:n2,m3:n3)-ei(m1:n1,m2:n2,m3:n3))* &
                        rho(m1:n1,m2:n2,m3:n3)/P(m1:n1,m2:n2,m3:n3)),0.0)
    dtime_limit=0.7*min(dtime_limit, &
                        dtime*par%C_maxeiChange/(-dei_min+10.*tiny(dei_min))*0.999)
!#
#   ifdef rhd_roe1d_step_t01
!#
    call timing_stop('HYD: Roe1D: checkdei')
!#
#   endif
!#
    if (dei_min < -par%C_maxeiChange) then
      ierr0  =2
      outstr0='Drop in internal energy too large'
      goto 998
    endif
  endif
  !
  ! === Boundary conditions VI =====================================================================
  if (bound_l == bound_transmitting2) then
    model%v1(m1,m2:n2,m3:n3)=v1(m1,m2:n2,m3:n3) + dtime*cs(m1,m2:n2,m3:n3) * &
                                                    ( v1(m1+1,m2:n2,m3:n3)- v1(m1,m2:n2,m3:n3))/ &
                                                    (xc1(m1+1            )-xc1(m1            ))
  endif
  if (bound_r == bound_transmitting2) then
    model%v1(n1,m2:n2,m3:n3)=v1(n1,m2:n2,m3:n3) - dtime*cs(n1,m2:n2,m3:n3) * &
                                                    ( v1(n1-1,m2:n2,m3:n3)- v1(n1,m2:n2,m3:n3))/ &
                                                    (xc1(n1-1            )- xc1(n1            ))
  endif
  !
endif
!
! \label{f90:c_courant__use2}\label{f90:c_hydexpcourant__use2}\label{f90:c_hydsoundcourant__use2}
dtime_limit=par%C_Courant*dtime_limit
if (par%C_hydExpCourant   > 0.0) dtime_limit=min(dtime_limit, par%C_hydExpCourant  *dtime_limitb)
if (par%C_hydSoundCourant > 0.0) dtime_limit=min(dtime_limit, par%C_hydSoundCourant*dtime_limitP)
!
998 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
!!$call uio_openwr(nc, 'rhd_roe1dstep_test.dat', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now')
!!$call uio_wrlabl(nc, box_ident                )
!!$call uio_wr(nc, 'z'             , box_id_ident   )
!!$call uio_wr(nc, model%m1        , 'm1'           )
!!$call uio_wr(nc, model%n1        , 'n1'           )
!!$call uio_wr(nc, model%m2        , 'm2'           )
!!$call uio_wr(nc, model%n2        , 'n2'           )
!!$call uio_wr(nc, model%m3        , 'm3'           )
!!$call uio_wr(nc, model%n3        , 'n3'           )
!!$call uio_wr(nc, model%nghost1   , 'nghost1'      )
!!$call uio_wr(nc, model%nghost2   , 'nghost2'      )
!!$call uio_wr(nc, model%nghost3   , 'nghost3'      )
!!$call uio_wr(nc, model%xb1       , 'xb1'          )
!!$call uio_wr(nc, model%xb2       , 'xb2'          )
!!$call uio_wr(nc, model%xb3       , 'xb3'          )
!!$call uio_wr(nc, model%xc1       , 'xc1'          )
!!$call uio_wr(nc, model%xc2       , 'xc2'          )
!!$call uio_wr(nc, model%xc3       , 'xc3'          )
!!$call uio_wr(nc, ei              , 'ei'           )
!!$call uio_wr(nc, rho             , 'rho'          )
!!$call uio_wr(nc, v1              , 'v1'           )
!!$call uio_wr(nc, v2              , 'v2'           )
!!$call uio_wr(nc, v3              , 'v3'           )
!!$call uio_wr(nc, P               , 'p'            )
!!$call uio_wr(nc, Gamma_1         , 'gamma_1'      )
!!$call uio_wr(nc, Gamma_3         , 'gamma_3'      )
!!$call uio_wr(nc, T               , 't'            )
!!$call uio_wr(nc, cs              , 'cs'           )
!!$call uio_wr(nc, phic_grav       , 'phic_grav'    )
!!$call uio_wr(nc, phib_grav       , 'phib_grav'    )
!!$call uio_wr(nc, v1_til          , 'v1_til'       )
!!$call uio_wr(nc, v2_til          , 'v2_til'       )
!!$call uio_wr(nc, v3_til          , 'v3_til'       )
!!$call uio_wr(nc, flux_rho        , 'flux_rho'     )
!!$call uio_wr(nc, flux_rhoei      , 'flux_rhoei'   )
!!$call uio_wr(nc, flux_rhov1      , 'flux_rhov1'   )
!!$call uio_wr(nc, flux_rhov2      , 'flux_rhov2'   )
!!$call uio_wr(nc, flux_rhov3      , 'flux_rhov3'   )
!!$call uio_wr(nc, fep             , 'fep'          )
!!$call uio_wrlabl(nc, endbox_ident    )
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call flush(nc)
!!$call uio_closrd(nc)
!
!#
# ifdef rhd_roe1d_step_t01
!#
call timing_stop('HYD: Roe1D:')
!#
# endif
!#
!
end subroutine rhd_hyd_Roe1DStep


!----------***********************------------------------------------------------------------------
subroutine rhd_hyd_Roe1DMincedStep(model, action, dtime, dtime_limit, &
                                   nghost1, mgh1, ngh1, &
                                   ComputeDelta_flag, CTUPredStep_flag, &
                                   OutVar_flag, OutFlx_flag, OutRadialC_flag, &
                                   AreaFraction, datx, datr, datc, &
                                   outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Roe1DMincedStep ('rhd_hydrodynamics_Roe_1D_Minced_Step')
!
! PURPOSE:
!   Perform 1D hydrodynamics step in x1 direction, call Roe solver repeatedly.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Roe1DMincedStep(model, action, dtime, dtime_limit, &
!                   nghost1, mgh1, ngh1, &
!                   ComputeDelta_flag=ComputeDelta_flag, CTUPredStep_flag=CTUPredStep_flag, &
!                   OutVar_flag=OutVar_flag, OutFlx_flag=OutFlx_flag, &
!                   OutRadialC_flag=OutRadialC_flag, &
!                   AreaFraction=AreaFraction, &
!                   datx=datx, datr=datr, datc=datc, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   mgh1:         ('m_ghost_1') integer, number of ghost cells to copy at left side of x1 direction
!   ngh1:         ('m_ghost_1') integer, number of ghost cells to copy at right side of x1 direction
!   ComputeDelta_flag: ('Compute_Delta_flag') logical to control if new state or deltas are
!                      computed:
!                        .false.: compute new state (default),
!                        .true. : compute only deltas; changes in the conserved quantities
!   CTUPredStep_flag: ('CTU_Predictor_Step_flag') logical to indicate CTU predictor step
!   OutVar_flag:  ('Output_Variable_flag') logical, compute averages of bulk quantities
!   OutFlx_flag:  ('Output_Flux_flag') logical, compute averages of fluxes
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!                    volume averaging of cell centered quantities is performed
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model chunk
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   datx:         ('data_x') type(datx_type), optional, structure for additional quantities
!   datr:         ('data_radial') type(datx_type), optional, structure for additional quantities
!   datc:         ('data_cylindrical') type(datc_type), optional, structure for
!                 additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                       0: normal run, hydrodynamics step successfully completed
!                    1-99: "small" error (e.g. time step limit violation)
!                   >=100: "severe" error: (currently none)
!
! LOCAL VARIABLES:
!   Output_flag:  ('Output_flag') logical, flag that determines if additional output should be
!                 generated
!   m2c:          ('m_2_chunk') integer, lower index, 2nd dimension of chunk
!   n2c:          ('n_2_chunk') integer, upper index, 2nd dimension of chunk
!   m3c:          ('m_3_chunk') integer, lower index, 3rd dimension of chunk
!   n3c:          ('n_3_chunk') integer, upper index, 3rd dimension of chunk
!   dtime_limit_chunk: ('delta_time_limit_chunk') real, time step limit for chunk
!   chunk:        ('chunk') box structure with data of current chunk
!   ierr0:        ('integer_error_0') Integer with error status
!   ierr_chunk:   ('integer_error_chunk') integer with error status of chunk
!   outstr0:      ('output_string_0') Character with error report
!   outstr_chunk: ('output_string_chunk') Character with error report for chunk
!
! ROUTINES:
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_hyd_Roe1DStep:     Perform 1D hydrodynamics step in x1 direction with Roe scheme
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   No OpenMP work sharing is done in this routine (or below). All OpenMP directives
!   should be given further up in the calling sequence.
!
! PROCEDURE:
!   Split the input model into small chunks and call rhd_Roe1DStep for each of them.
!
! MODIFICATION HISTORY:
!   2014-08-20 (B.F. Uppsala) Written
!   2014-08-23 (B.F.) Changes in interface
!   2014-10-27 (B.F.) New parameter: nghost1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
integer,                   intent(in)   :: nghost1, mgh1, ngh1
logical,         optional, intent(in)   :: ComputeDelta_flag, CTUPredStep_flag, &
                                           OutVar_flag, OutFlx_flag, OutRadialC_flag
real,            optional, intent(in)   :: AreaFraction
type(datx_type), optional, intent(inout):: datx, datr
type(datc_type), optional, intent(inout):: datc
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag
integer                                 :: m2c,n2c, m3c,n3c, n_chunks, i_chunk
real                                    :: dtime_limit_chunk
type(box_type)                          :: chunk
logical, parameter                      :: Bc_flag=.false.
integer                                 :: ierr0, ierr_chunk
character(len=80)                       :: outstr0, outstr_chunk
!---------------------------------------------------------------------------------------------------
!
! --- Check keywords for output of certain averages ---
if (present(OutVar_flag)) then
  Output_flag=OutVar_flag
else
  Output_flag=.false.
endif
if (present(OutFlx_flag)) then
  Output_flag=Output_flag .or. OutFlx_flag
endif
!
! --- Determine number of chunks ---
call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                       model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                       m2c,n2c, m3c,n3c, &
                       n_chunks=n_chunks)
!if (gl%nc_m >= 0) then
!  write(gl%nc_m,'(A,I10,I8,A)') &
!    'HYD 3: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks, ' minced'
!  call flush(gl%nc_m)
!endif
!
ierr0  =0
outstr0=''
!
! --- Set initial time step estimate ---
dtime_limit=huge(dtime_limit)
!
! --- Loop over all chunks ---
CHUNK_LOOP1: do i_chunk=1,n_chunks
  !
  ! --- Get current chunk indices ---
  call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                         model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk, &
                         m2c,n2c, m3c,n3c)
  !write(gl%nc_p,'(I3,I6,I6,2(A,2I10),A)') &
  !  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c, ' minced'
  !
  ! --- Create empty chunk and fill it with data ---
  call rhd_box_Make(chunk, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                           nghost1=nghost1, &
                           quc_flag=.true., Bc_flag=Bc_flag)
  call rhd_box_Copy(model, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                    chunk, chunk%m1-mgh1,                chunk%m2,      chunk%m3, &
                    trans_mode=1)
  !
  ! --- Perform actual hydro step ---
  !select case (par%A%hdScheme)
  !  case (hdScheme_RoeWave, hdScheme_RoeState, hdScheme_RoeMagKin)
      if (Output_flag) then
        call rhd_hyd_Roe1DStep(chunk, action, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=ComputeDelta_flag, &
                               CTUPredStep_flag=CTUPredStep_flag, &
                               OutVar_flag=OutVar_flag, OutFlx_flag=OutFlx_flag, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction * &
                                              (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                              (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                              (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                              (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                                 datx=datx, datr=datr, datc=datc, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag)
        call rhd_hyd_Roe1DStep(chunk, action, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=ComputeDelta_flag, &
                               CTUPredStep_flag=CTUPredStep_flag, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag)
  !  case default
  !    dtime_limit_chunk=huge(dtime_limit_chunk)
  !    ierr_chunk          =0
  !    outstr_chunk        =''
  !end select
  !
  ! --- Copy data back into full model and delete temporary chunk ---
  call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                    model, model%m1,           m2c,                m3c, &
                    grid_mode=0, trans_mode=1)
  ! --- Delete chunk ---
  call rhd_box_Delete(chunk)
  !
  ! --- Handle error conditions and time step ---
  if (ierr_chunk > ierr0) then
    ierr0  =ierr_chunk
    outstr0=outstr_chunk
  endif
  dtime_limit=min(dtime_limit, dtime_limit_chunk)
end do CHUNK_LOOP1 ! i_chunk
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine rhd_hyd_Roe1DMincedStep


!----------******************-----------------------------------------------------------------------
subroutine rhd_hyd_SingleStep(model, action, dtime, dtime_limit, &
                              Output_flag, dat, &
                              outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_SingleStep ('rhd_hydrodynamics_Single_Step')
!
! PURPOSE:
!   Perform entire (single) 2D or 3D hydrodynamics step, employ Roe scheme.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_SingleStep(model, action, dtime, dtime_limit, &
!                           Output_flag=Output_flag, &
!                           dat=dat, &
!                           outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Output_flag:  ('Output_flag') logical, optional, flag that determines if additional averages are
!                 computed and put into datx, datc, and datr, default=.false.
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') type(dat_type), optional, structure for additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional with error string
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successfully completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error: (currently none)
!
! LOCAL VARIABLES:
!   dtime_limit_chunk: ('delta_time_limit_chunk') real, time step limit for chunk
!   dtime_limit_thread: ('delta_time_limit_thread') real, time step limit for thread
!   eiTable_min:  ('energy_internal_Table_minimum') real, minimum energy allowed by EOS table
!   AreaFraction: ('Area_Fraction') real, weighting factor for the current model chunk
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   m1c:          ('m_1_chunk') integer, lower index, 1st dimension of chunk
!   n1c:          ('n_1_chunk') integer, upper index, 1st dimension of chunk
!   m2c:          ('m_2_chunk') integer, lower index, 2nd dimension of chunk
!   n2c:          ('n_2_chunk') integer, upper index, 2nd dimension of chunk
!   m3c:          ('m_3_chunk') integer, lower index, 3rd dimension of chunk
!   n3c:          ('n_3_chunk') integer, upper index, 3rd dimension of chunk
!   mgh1:         ('m_ghost_1') integer, number of ghost cells to copy at left side of x1 direction
!   ngh1:         ('m_ghost_1') integer, number of ghost cells to copy at right side of x1 direction
!   mgh2:         ('m_ghost_2') integer, number of ghost cells to copy at left side of x2 direction
!   ngh2:         ('m_ghost_2') integer, number of ghost cells to copy at right side of x2 direction
!   mgh3:         ('m_ghost_3') integer, number of ghost cells to copy at left side of x3 direction
!   ngh3:         ('m_ghost_3') integer, number of ghost cells to copy at right side of x3 direction
!   i_chunk:      ('index_chunk') integer with index for loop over chunks
!   n_chunks:     ('number_chunks') integer with total number of chunks
!   N_hydCellsPerChunk: ('Number_hydrodynamics_Cells_Per_Chunk') integer, number of cells per chunk
!   ierr0:        ('integer_error_0') Integer with error status
!   ierr_chunk:   ('integer_error_chunk') integer with error status of chunk
!   ierr_thread:  ('integer_error_thread') integer with error status of thread
!   StepDone_flag: ('Step_Done_flag') logical that is true if already a hydro step has been
!                  performed
!   Output_flag0: ('Output_flag_0') logical, flag that determines if additional averages are
!                 computed and put into dat
!   OutRadialC_flag: ('Output_Radial_Center_flag') logical, optional, flag that determines if
!   Chunks1Level_flag: ('Chunks_1_Level_flag') logical to indicate if there is one level
!                 of chunks (.true.) or there are two levels (.false.)
!   outstr0:      ('output_string_0') Character with error report
!   outstr_chunk: ('output_string_chunk') Character with error report for chunk
!   outstr_thread: ('output_string_chunk') Character with error report for thread
!   chunk:        ('chunk') box structure with data of current chunk
!   chuTr:        ('chunk_Transpose') box structure with data of current chunk
!   delta:        ('delta') box structure with changes
!   dat_thread:   ('dat_thread') data structure with averaged data for thread
!   action_current: ('action_current') derived type with collection of control parameters
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   m0x1_rhdMPI_boxGhostTypes: ('model_0_x_1_rhd_MPI_box_Ghost_Types') integer array with
!                 MPI handles
!                 for MPI types for variables in ghost layers for model (0) and direction x1
!   m0x1_RequestsMPI: ('model_0_x_1_Request_MPI') integer array with MPI handles
!                 for MPI send/receiv requests for model (0) and direction x1
!   m1x2_rhdMPI_boxGhostTypes: ('model_1_x_2_rhd_MPI_box_Ghost_Types') integer array with
!                 MPI handles
!                 for MPI types for variables in ghost layers for model 1 and direction x2
!   m1x2_RequestsMPI: ('model_1_x_2_Request_MPI') integer array with MPI handles
!                 for MPI send/receiv requests for model 1 and direction x2
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_bound_3DInOutFlow: Adjust values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DTransmit:  Adjust density values in top cells for the open boundary condition
!   rhd_box_Add:           Add (part of) box to another. Transpose, if requested
!   rhd_box_AddDelta:      Add delta of (part of) box into another. Transpose, if requested
!   rhd_box_AllAddDelta2OMP: Add change to entire box, in parallel if box is 3D
!   rhd_box_AllAddDeltaOMP: Add change to entire box, in parallel if box is 3D
!   rhd_box_AllAddOMP:     Add box to another, in parallel if n3>m3
!   rhd_box_AllFactorOMP:  Multiply contents of box with constant factor, in parallel if n3>m3
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Factor:        Multiply contents of box with constant factor
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_MPIFillGhostEnd: Finalize filling ghost cells
!   rhd_box_MPIFillGhostStart: Start filling "internal" ghost cells
!   rhd_datc_Add:          Add the fields in the structure for cylindrical mean data
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_datx_Add:          Add the fields in the structure for additional data
!   rhd_hyd_CoriolisOMP:   Apply Coriolis rotation (with local OpenMP directives)
!   rhd_hyd_Roe1DMincedStep: Perform 1D hydro step in x1 direction, call Roe solver repeatedly
!   rhd_hyd_Roe1DStep:     Perform 1D hydrodynamics step in x1 direction with Roe scheme
!   rhd_wrk_module:        Provide persistent work copies of model data
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   omp_lib:               Library of system OpenMP definitions and routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_MPI_module:        MPI routines
!   rhd_phys_module:       Physics routines (source terms: gravity, Coriolis, dust)
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Modify global timing statistics counters.
!   Use global temporary computational boxes model1, model2, model3, modelB.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Solve 2D/3D problem by cutting (and transposing) appropriate chunks out of the entire volume
!   and feeding them into the 1D Roe solver.
!
! TODO:
!   Spherical means of quc fluxes
!
! MODIFICATION HISTORY:
!   2006-11-27 (B.F. Uppsala) Taken from rhd.F90, put into own routine
!   2006-12-12 (B.F. Lyon) Renamed: rhd_hyd_Step -> rhd_hyd_SingleStep
!   2007-02-08 (B.F.) Open lower boundary directly before x3 step, only if hdScheme/=hdScheme_None.
!                     Parameter hdsplit added: allow 'unsplit' scheme
!   2007-12-26 (B.F.) Avoid multiple calls of rhd_action_New, introduce action_current
!   2010-06-12 (B.F.) bound_inoutflow2, bound_closedbottom2
!   2011-07-17 (B.F.) Initialize ierr0 and outstr0 also in x2 branch;
!                     OutRadialC_flag no longer I/O parameter
!   2011-08-03 (B.F.) Branch for Colella's CTU scheme
!   2011-08-06 (B.F.) Call Coriolis routine from here (and not in rhd_hyd_Roe1DStep)
!   2011-08-08 (B.F.) CTU scheme
!   2011-12-12 (B.F.) Declare loop counter as OpenMP private
!   2012-11-07 (B.F.) Added arguments for MPI
!   2012-11-12 (B.F.) MPI parameters put into prop structure
!   2012-11-13 (B.F.) First calls to MPI ghost-layer-exchange routines
!   2012-11-14 (B.F.) Separate branches for unsplit and directional splitting;
!                     Perform 3D-boundary steps at very beginning of unsplit step
!   2012-11-16 (B.F.) Account for handling of MPI ghost cells in more cases;
!                     Change order of CTU predictor steps: 123->231
!   2013-05-09 (B.F.) 'unsplit': reordering (1,2,3->3,2,1), add changes simpler
!   2013-05-12 (B.F.) Timing: 'Hydrodynamics routines' -> 'HYD: SingleStep'
!   2013-08-09 (B.F.) Change call to rhd_dat_Make
!   2014-08-15 (B.F. Uppsala) Account for different size of x3 chunks
!   2014-08-17 (B.F.) Use global model1, model2, model3 boxes; replace modelout by global modelB
!   2014-08-18 (B.F.) Change order of secondary indices in i1 loops
!   2014-08-21 (B.F.) New 2-levels-of-chunks option
!   2014-08-23 (B.F.) New 2-levels-of-chunks option also for CTU
!   2014-09-21 (B.F.) Bugfix concerning incorrect transpose for 1D unsplit runs
!   2017-09-23 (B.F.) Add timing call around MPI ghost-cell-filling calls
!---------------------------------------------------------------------------------------------------
use tabinter_module
use rhd_gl_module
use rhd_dat_module
use rhd_action_module
use rhd_prop_module
use rhd_bound_module
use rhd_box_module
use rhd_wrk_module
use rhd_phys_module
use timing_module
!$ use omp_lib
use rhd_MPI_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: Output_flag
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
real                                    :: dtime_limit_chunk, dtime_limit_thread, eiTable_min, &
                                           AreaFraction
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           m1c,n1c, m2c,n2c, m3c,n3c, &
                                           mgh1,ngh1, mgh2,ngh2, mgh3,ngh3, nghost, &
                                           i_chunk, n_chunks, &
                                           N_hydCellsPerChunk
integer                                 :: ierr0, ierr_chunk, ierr_thread
logical                                 :: StepDone_flag, Output_flag0, OutRadialC_flag, &
                                           Chunks1Level_flag
character(len=80)                       :: outstr0, outstr_chunk, outstr_thread
type(box_type)                          :: chunk, chuTr, delta
type(dat_type)                          :: dat_thread
type(action_type)                       :: action_current
logical, parameter                      :: Bc_flag=.false.
! --- OpenMP variables ---
integer                                 :: omp_MaxThreads
! --- MPI variables ---
integer                                 :: ierrMPI
integer, dimension(1:4)                 :: m0x1_rhdMPI_boxGhostTypes, m0x1_RequestsMPI, &
                                           m0x2_rhdMPI_boxGhostTypes, m0x2_RequestsMPI, &
                                           m0x3_rhdMPI_boxGhostTypes, m0x3_RequestsMPI, &
                                           m1x2_rhdMPI_boxGhostTypes, m1x2_RequestsMPI, &
                                           m1x3_rhdMPI_boxGhostTypes, m1x3_RequestsMPI, &
                                           m2x1_rhdMPI_boxGhostTypes, m2x1_RequestsMPI, &
                                           m2x3_rhdMPI_boxGhostTypes, m2x3_RequestsMPI, &
                                           m3x1_rhdMPI_boxGhostTypes, m3x1_RequestsMPI, &
                                           m3x2_rhdMPI_boxGhostTypes, m3x2_RequestsMPI
!---------------------------------------------------------------------------------------------------
!
call timing_start('HYD: SingleStep')
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set initial time step estimate ---
dtime_limit=huge(dtime_limit)
!
! --- Indicate that no hydro step has been performed yet ---
StepDone_flag  =.false.
OutRadialC_flag=.true.
!
! --- Set a default value for the output flag if it is not present ---
if (present(Output_flag)) then
  Output_flag0=Output_flag
else
  Output_flag0=.false.
endif
!
! --- Get some information about OpenMP environment ---
!#
# ifdef _OPENMP
!#
omp_MaxThreads=omp_get_max_threads()
!#
# else
!#     /* _OPENMP */
omp_MaxThreads=1
!#
# endif
!#      /* _OPENMP */
!
! --- Calculate number of ghost cells ---
nghost=max(par%A%reconstruction_order+1, 2)
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Calculate number of ghost cells that need to be copied (none for non-MPI runs) ---
if ((action%bound_l1 == bound_InternalPeriodic) .or. &
    (action%bound_l1 == bound_InternalNext    )     ) then
  mgh1=nghost
else
  mgh1=0
endif
if ((action%bound_r1 == bound_InternalPeriodic) .or. &
    (action%bound_r1 == bound_InternalNext    )     ) then
  ngh1=nghost
else
  ngh1=0
endif
!
! --- Calculate number of ghost cells that need to be copied (none for non-MPI runs) ---
if ((action%bound_l2 == bound_InternalPeriodic) .or. &
    (action%bound_l2 == bound_InternalNext    )     ) then
  mgh2=nghost
else
  mgh2=0
endif
if ((action%bound_r2 == bound_InternalPeriodic) .or. &
    (action%bound_r2 == bound_InternalNext    )     ) then
  ngh2=nghost
else
  ngh2=0
endif
!
! --- Calculate number of ghost cells that need to be copied (none for non-MPI runs) ---
if ((action%bound_l3 == bound_InternalPeriodic) .or. &
    (action%bound_l3 == bound_InternalNext    )     ) then
  mgh3=nghost
else
  mgh3=0
endif
if ((action%bound_r3 == bound_InternalPeriodic) .or. &
    (action%bound_r3 == bound_InternalNext    )     ) then
  ngh3=nghost
else
  ngh3=0
endif
!#
#  else
!#      /* MPI not activated ------------------------------ */
! --- Initialize number of internal chunk ghost cell layers ---
mgh1=0
ngh1=0
mgh2=0
ngh2=0
mgh3=0
ngh3=0
!#
#  endif
!#       /* rhd_mpi_l01 ----------------------------------- */
!
! === Coriolis terms ===============================================================================
!
if ((action%dim == 1) .and. (par%nu_rotation /= 0.0)) then
  call timing_start('HYD: Coriolis')
  call rhd_hyd_CoriolisOMP(model, dtime)
  call timing_stop('HYD: Coriolis')
endif
!
! \label{f90:hdsplit__use1}
if ((par%A%hdScheme /= hdScheme_None) .and. &
    (par%hdsplit == 'CTU')            .and. &
    (((model%n1 > model%m1) .and. (model%n2 > model%m2)) .or. &
     ((model%n1 > model%m1) .and. (model%n3 > model%m3)) .or. &
     ((model%n2 > model%m2) .and. (model%n3 > model%m3))       )) then
  ! ### Colella's Corner Transport Upwind (CTU) scheme #############################################
  !
  ! --- Get minimum internal energy value of EOS table ---
  call tabinter_getinfo(x2min=eiTable_min)
  !
  !write(6,'(A,I3,A,8(1PE13.5))') '#', wg%hyd%MyProc, ' Model  :', &
  !                               minval(model%v1 ), maxval(model%v1 ), &
  !                               minval(model%v3 ), maxval(model%v3 ), &
  !                               minval(model%rho), maxval(model%rho), &
  !                               minval(model%ei ), maxval(model%ei )
  !
  ! --- Create empty temporary 3D model, if necessary ---
  if (.not. rhd_box_Exist(modelB)) then
    call rhd_box_Make(modelB, &
                    m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3, &
                    quc_flag=.true., Bb_flag=.true.)
  endif
  ! --- Copy only axes ---
  call rhd_box_Copy(model,  model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                    modelB, model%m1,           model%m2,           model%m3, &
                    grid_mode=1, time_mode=1, bulk_mode=0)
  !
  ! --- Initialize predictor-step models ---
  !call rhd_box_Init(model1)
  !call rhd_box_Init(model2)
  !call rhd_box_Init(model3)
  !
  ! \label{f90:c_rhochangetop__use2}
  ! === Open upper boundary condition ==============================================================
  if (((wg%hyd%bound_r(3) == bound_transmitting ) .or. &
       (wg%hyd%bound_r(3) == bound_transmitting2) .or. &
       (wg%hyd%bound_r(3) == bound_transmitting3)       ) .and. &
      ( par%grav_mode /= 'central')                       .and. &
      ( par%C_rhoChangeTop > 0.0)                                ) then
    call timing_start('HYD: bound_3DTransmit')
    call rhd_bound_3DTransmit(model, action, dtime)
    call timing_stop('HYD: bound_3DTransmit')
  endif
  !
  ! === Open lower boundary condition ==============================================================
  if ((wg%hyd%bound_l(3) == bound_inoutflow ) .or. &
      (wg%hyd%bound_l(3) == bound_inoutflow2) .or. &
      (wg%hyd%bound_l(3) == bound_inoutflow3)       ) then
    call timing_start('HYD: bound_3DInOutFlow')
    call rhd_bound_3DInOutFlow(model, action, dtime)
    call timing_stop('HYD: bound_3DInOutFlow')
  endif
  !
  ! === Predictor step =============================================================================
  !
  ! === Predictor substep in x2 direction ==========================================================
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Fill ghost cells with data from neighbor processes ---
  !
  if (model%n2 > model%m2) then
    ! --- Start filling of ghost layers of model in x2 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 2, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x2_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x2_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
    !
    ! --- Finalize filling of ghost layers of model in x2 direction ---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x2_RequestsMPI, m0x2_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
  endif
  !
  if (model%n3 > model%m3) then
    ! --- Start filling of ghost layers of model in x3 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 3, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x3_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x3_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
  if (model%n1 > model%m1) then
    ! --- Start filling of ghost layers of model in x1 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 1, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x1_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  if (model%n2 > model%m2) then
    !
    action_current=rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)))
    !
    ! --- Create empty temporary 3D model, if necessary ---
    if (.not. rhd_box_Exist(model2)) then
      call rhd_box_Make(model2, &
                      m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3,&
                      nghost1=model%nghost1, nghost2=model%nghost2, nghost3=model%nghost3, &
                      quc_flag=.true., Bb_flag=.true.)
    endif
    ! --- Copy only axes ---
    call rhd_box_Copy(model,  model%m1-mgh1, model%n1+ngh1, model%m2-mgh2, model%n2+ngh2, &
                              model%m3-mgh3, model%n3+ngh3, &
                      model2, model%m1-mgh1,                model%m2-mgh2, &
                              model%m3-mgh3, &
                      grid_mode=1, time_mode=1, bulk_mode=0)
    !
    ! \label{f90:n_hydcellsperchunk__use1}
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, 1, &
                           m1c,n1c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HDP 2: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HDP: 2')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m1c, n1c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HDP2_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HDP2_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPP2: do i_chunk=1,n_chunks
      !
      ! \label{f90:n_hydcellsperchunk__use2}
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, i_chunk, &
                             m1c,n1c, m3c,n3c)
      !write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m3c, n3c
      !call flush(gl%nc_p)
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m2, n1=model%n2, m2=m1c, n2=n1c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                        chunk, chunk%m1-mgh2, chunk%m2,                     chunk%m3, &
                        trans_mode=2)
      !
      ! --- Perform predictor step ---
      if (Output_flag0) then
        AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                     (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                     (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                     (model%xb3(model%n3+1)-model%xb3(model%m3))
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               OutVar_flag=Output_flag0, OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x13, datr=dat_thread%r, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag0)
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag0)
      !
      ! --- Copy resulting chunk into model 2 ---
      call rhd_box_Copy(chunk,  chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                        model2, m1c,                model%m2,           m3c, &
                        grid_mode=0, trans_mode=2)
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPP2 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HDP2_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HDP2_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HDP: 2')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of predictor substep in x2 direction ---
  endif ! (model%n2 > model%m2)
  !
  ! === Predictor substep in x3 direction ==========================================================
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Fill ghost cells with data from neighbor processes ---
  !
  if (model%n3 > model%m3) then
    ! --- Finalize filling of ghost layers of model in x3 direction ---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x3_RequestsMPI, m0x3_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
  endif
  !
  if ((model%n2 > model%m2) .and. (model%n1 > model%m1)) then
    ! --- Start filling of ghost layers of model2 in x1 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model2, 1, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m2x1_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m2x1_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
  if ((model%n2 > model%m2) .and. (model%n3 > model%m3)) then
    ! --- Start filling of ghost layers of model2 in x3 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model2, 3, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m2x3_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m2x3_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  if (model%n3 > model%m3) then
    !
    action_current=rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)))
    !
    ! --- Create empty temporary 3D model, if necessary ---
    if (.not. rhd_box_Exist(model3)) then
      call rhd_box_Make(model3, &
                      m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3,&
                      nghost1=model%nghost1, nghost2=model%nghost2, nghost3=model%nghost3, &
                      quc_flag=.true., Bb_flag=.true.)
    endif
    ! --- Copy only axes ---
    call rhd_box_Copy(model,  model%m1-mgh1, model%n1+ngh1, model%m2-mgh2, model%n2+ngh2, &
                              model%m3-mgh3, model%n3+ngh3, &
                      model3, model%m1-mgh1,                model%m2-mgh2, &
                              model%m3-mgh3, &
                      grid_mode=1, time_mode=1, bulk_mode=0)
    !
    ! \label{f90:n_hydcellsperchunk3__use1}
    ! --- Determine number of chunks ---
    if ((2*par%N_hydCellsPerChunk3 >= par%N_hydCellsPerChunk) .or. (model%n2 == model%m2)) then
      ! --- Perform normal 1-level-of-chunks partitioning ---
      Chunks1Level_flag=.true.
      N_hydCellsPerChunk=par%N_hydCellsPerChunk3
    else
      ! --- Perform 2-levels-of-chunks partitioning ---
      Chunks1Level_flag=.false.
      N_hydCellsPerChunk=max(min(par%N_hydCellsPerChunk3, &
                                 ceiling(real(model%n2-model%m2+1)/real(omp_MaxThreads))), &
                             1) * &
                         (model%n1-model%m1+1)*(model%n3-model%m3+1)
    endif
    call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8,A,I8)') &
        'HDP 3: N_CellsPerChunk, n_chunks: ', N_hydCellsPerChunk, n_chunks, &
        '/', omp_MaxThreads
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HDP: 3')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m1c, n1c, m2c, n2c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HDP3_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HDP3_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPP3: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk, &
                             m1c,n1c, m2c,n2c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m3, n1=model%n3, m2=m2c, n2=n2c, m3=m1c, n3=n1c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                        chunk, chunk%m1-mgh3, chunk%m2,      chunk%m3, &
                        trans_mode=3)
      !
      ! --- Perform predictor step ---
      if (Output_flag0) then
        AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                     (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                     (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                     (model%xb2(model%n2+1)-model%xb2(model%m2))
        if (Chunks1Level_flag) then
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               OutVar_flag=Output_flag0, OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Chunks1Level_flag)
          call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               nghost, mgh3, ngh3, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               OutVar_flag=Output_flag0, OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Chunks1Level_flag)
        !
      else ! (.not. Output_flag0)
        if (Chunks1Level_flag) then
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Chunks1Level_flag)
          call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               nghost, mgh3, ngh3, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Chunks1Level_flag)
      endif ! (Output_flag0)
      !
      ! --- Copy resulting chunk into model 3 ---
      call rhd_box_Copy(chunk , chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                        model3, m1c,                m2c,                model%m3, &
                        grid_mode=0, trans_mode=3)
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPP3 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HDP3_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_datc_Add(dat%c  , dat_thread%c  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HDP3_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HDP: 3')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    !write(6,'(A,8(1PE13.5))') 'Model 3:', minval(model3%v1 ), maxval(model3%v1 ), &
    !                                      minval(model3%v3 ), maxval(model3%v3 ), &
    !                                      minval(model3%rho), maxval(model3%rho), &
    !                                      minval(model3%ei ), maxval(model3%ei )
    !
    ! --- End of predictor substep in x3 direction ---
  endif ! (model%n3 > model%m3)
  !
  ! === Predictor substep in x1 direction ==========================================================
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Fill ghost cells with data from neighbor processes ---
  !
  if (model%n1 > model%m1) then
    ! --- Finalize filling of ghost layers of model in x1 direction ---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
  endif
  !
  if ((model%n3 > model%m3) .and. (model%n1 > model%m1)) then
    ! --- Start filling of ghost layers of model3 in x1 direction---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model3, 1, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m3x1_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m3x1_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
  if ((model%n3 > model%m3) .and. (model%n2 > model%m2)) then
    ! --- Start filling of ghost layers of model3 in x2 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model3, 2, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m3x2_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m3x2_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  if (model%n1 > model%m1) then
    !
    action_current=rhd_action_New(action, dim=1)
    !
    ! --- Create empty temporary 3D model, if necessary ---
    if (.not. rhd_box_Exist(model1)) then
      call rhd_box_Make(model1, &
                      m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3,&
                      nghost1=model%nghost1, nghost2=model%nghost2, nghost3=model%nghost3, &
                      quc_flag=.true., Bb_flag=.true.)
    endif
    ! --- Copy only axes ---
    call rhd_box_Copy(model,  model%m1-mgh1, model%n1+ngh1, model%m2-mgh2, model%n2+ngh2, &
                              model%m3-mgh3, model%n3+ngh3, &
                      model1, model%m1-mgh1,                model%m2-mgh2, &
                              model%m3-mgh3, &
                      grid_mode=1, time_mode=1, bulk_mode=0)
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                           m2c,n2c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HDP 1: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HDP: 1')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m2c, n2c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HDP1_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HDP1_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPP1: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk, &
                             m2c,n2c, m3c,n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                        chunk, chunk%m1-mgh1,                chunk%m2,      chunk%m3, &
                        trans_mode=1)
      !
      ! --- Perform predictor step ---
      if (Output_flag0) then
        AreaFraction=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                     (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                     (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                     (model%xb3(model%n3+1)-model%xb3(model%m3))
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               OutVar_flag=Output_flag0, OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x23, datr=dat_thread%r, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag0)
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., CTUPredStep_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag0)
      !
      ! --- Copy resulting chunk into model 1 ---
      call rhd_box_Copy(chunk,  chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                        model1, model%m1,        m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPP1 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HDP1_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HDP1_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HDP: 1')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !write(6,'(A,8(1PE13.5))') 'Model 1:', minval(model1%v1 ), maxval(model1%v1 ), &
    !                                      minval(model1%v3 ), maxval(model1%v3 ), &
    !                                      minval(model1%rho), maxval(model1%rho), &
    !                                      minval(model1%ei ), maxval(model1%ei )
    !
    ! --- End of predictor substep in x1 direction ---
  endif ! (model%n1 > model%m1)
  !
  ! --- Now, model1,2,3 contain the deltas from the predictor steps in 1,2,3 direction ---
  !
  ! === Corrector step =============================================================================
  !
  ! --- Indicate that no hydro step has been performed yet ---
  OutRadialC_flag=.false.
  StepDone_flag=.false.
  !
  if (model%n1 > model%m1) then
    ! === Corrector substep in x1 direction ========================================================
    !
    action_current=rhd_action_New(action, dim=1)
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    if (model%n2 > model%m2) then
      ! --- Finalize filling of ghost layers of model2 in x1 direction  ---
      call timing_start('HYD:   FillGhostEnd')
      call rhd_box_MPIFillGhostEnd(m2x1_RequestsMPI, m2x1_rhdMPI_boxGhostTypes)
      call timing_stop('HYD:   FillGhostEnd')
    endif
    if (model%n3 > model%m3) then
      ! --- Finalize filling of ghost layers of model3 in x1 direction ---
      call timing_start('HYD:   FillGhostEnd')
      call rhd_box_MPIFillGhostEnd(m3x1_RequestsMPI, m3x1_rhdMPI_boxGhostTypes)
      call timing_stop('HYD:   FillGhostEnd')
    endif
    !
    if (model%n2 > model%m2) then
      ! --- Start filling of ghost layers of model1 in x2 direction ---
      call timing_start('HYD:   FillGhostStart')
      call rhd_box_MPIFillGhostStart(wg%hyd, model1, 2, nghost, &
                                     m1,n1, m2,n2, m3,n3, &
                                     RequestsMPI=m1x2_RequestsMPI, &
                                     rhdMPI_boxGhostTypes=m1x2_rhdMPI_boxGhostTypes, &
                                     grid_mode=0, bulk_mode=2)
      call timing_stop('HYD:   FillGhostStart')
    endif
    if (model%n3 > model%m3) then
      ! --- Start filling of ghost layers of model1 in x3 direction ---
      call timing_start('HYD:   FillGhostStart')
      call rhd_box_MPIFillGhostStart(wg%hyd, model1, 3, nghost, &
                                     m1,n1, m2,n2, m3,n3, &
                                     RequestsMPI=m1x3_RequestsMPI, &
                                     rhdMPI_boxGhostTypes=m1x3_rhdMPI_boxGhostTypes, &
                                     grid_mode=0, bulk_mode=2)
      call timing_stop('HYD:   FillGhostStart')
    endif
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                           m2c,n2c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HDC 1: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HDC: 1')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, delta, &
    !$OMP                  m2c, n2c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HDC1_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HDC1_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPC1: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk, &
                             m2c,n2c, m3c,n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
      !
      ! --- Create empty chunks:                                   ---
      ! --- delta contains the changes of the conserved quantities ---
      ! --- chunk contains the new state                           ---
      call rhd_box_Make(chunk, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Make(delta, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      !
      ! --- Sum predictor step results for orthogonal directions ---
      if (model%n2 > model%m2) then
        call rhd_box_Copy( model2, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                           delta , delta%m1-mgh1,                delta%m2,      delta%m3, &
                           trans_mode=1)
        if (model%n3 > model%m3) then
          call rhd_box_Add(model3, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                           delta , delta%m1-mgh1,                delta%m2,      delta%m3, &
                           trans_mode=1)
        endif
      else if (model%n3 > model%m3) then
        call rhd_box_Copy( model3, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                           delta , delta%m1-mgh1,                delta%m2,      delta%m3, &
                           trans_mode=1)
      endif
      ! --- Use only half of the changes ---
      call rhd_box_Factor(delta, &
                          model%m1-mgh1, model%n1+ngh1, delta%m2, delta%n2, delta%m3, delta%n3, 0.5)
      !
      ! --- Add predictor step results for orthogonal directions ---
      call rhd_box_Copy(model, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                        chunk, chunk%m1-mgh1,                chunk%m2,      chunk%m3, &
                        trans_mode=1)
      call rhd_box_AddDelta( &
                   delta, delta%m1-mgh1, delta%n1+ngh1, delta%m2, delta%n2, delta%m3, delta%n3, &
                   chunk, chunk%m1-mgh1,                chunk%m2,           chunk%m3, &
                          eiTable_min, outstr=outstr_chunk, ierr=ierr_chunk)
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(delta)
      !
      ! --- Check for error ---
      if (ierr_chunk > 0) then
        ! --- Time step reduction ---
        dtime_limit_chunk=0.7*dtime
        !
      else
        ! --- Perform corrector step ---
        if (Output_flag0) then
          AreaFraction=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                       (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                       (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                       (model%xb3(model%n3+1)-model%xb3(model%m3))
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 ComputeDelta_flag=.true., &
                                 OutFlx_flag=Output_flag0, &
                                   OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=AreaFraction, &
                                   datx=dat_thread%x23, datr=dat_thread%r, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Output_flag0)
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 ComputeDelta_flag=.true., &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Output_flag0)
        !
        if (StepDone_flag) then
          ! --- Add change ---
          call rhd_box_Add( chunk , chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                            modelB, model%m1,           m2c,                m3c, &
                            grid_mode=0, trans_mode=1)
        else
          ! --- Just copy chunk ---
          call rhd_box_Copy(chunk , chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                            modelB, model%m1,           m2c,                m3c, &
                            grid_mode=0, trans_mode=1)
        endif
      endif
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPC1 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HDC1_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HDC1_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HDC: 1')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    StepDone_flag=.true.
    !
    !write(6,'(A,8(1PE13.5))') 'Modelo1:', minval(modelB%v1 ), maxval(modelB%v1 ), &
    !                                      minval(modelB%v3 ), maxval(modelB%v3 ), &
    !                                      minval(modelB%rho), maxval(modelB%rho), &
    !                                      minval(modelB%ei ), maxval(modelB%ei )
    !
    ! --- End of corrector substep in x1 direction ---
  endif ! (model%n1 > model%m1)
  !
  if (model%n2 > model%m2) then
    ! === Corrector substep in x2 direction ========================================================
    !
    action_current=rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)))
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    if (model%n1 > model%m1) then
      ! --- Finalize filling of ghost layers of model1 in x2 direction ---
      call timing_start('HYD:   FillGhostEnd')
      call rhd_box_MPIFillGhostEnd(m1x2_RequestsMPI, m1x2_rhdMPI_boxGhostTypes)
      call timing_stop('HYD:   FillGhostEnd')
    endif
    if (model%n3 > model%m3) then
      ! --- Finalize filling of ghost layers of model3 in x2 direction ---
      call timing_start('HYD:   FillGhostEnd')
      call rhd_box_MPIFillGhostEnd(m3x2_RequestsMPI, m3x2_rhdMPI_boxGhostTypes)
      call timing_stop('HYD:   FillGhostEnd')
    endif
    !
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! \label{f90:n_hydcellsperchunk2__use1}
    ! --- Determine number of chunks ---
    if ((2*par%N_hydCellsPerChunk2 >= par%N_hydCellsPerChunk) .or. (model%n3 == model%m3)) then
      ! --- Perform normal 1-level-of-chunks partitioning ---
      Chunks1Level_flag=.true.
      N_hydCellsPerChunk=par%N_hydCellsPerChunk2
    else
      ! --- Perform 2-levels-of-chunks partitioning ---
      Chunks1Level_flag=.false.
      N_hydCellsPerChunk=max(min(par%N_hydCellsPerChunk2, &
                                 ceiling(real(model%n3-model%m3+1)/real(omp_MaxThreads))), &
                             1) * &
                         (model%n1-model%m1+1)*(model%n2-model%m2+1)
    endif
    call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                           model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, 1, &
                           m1c,n1c, m3c,n3c, &
                           n_chunks=n_chunks)
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8,A,I8)') &
        'HDC 2: N_CellsPerChunk, n_chunks: ', N_hydCellsPerChunk, n_chunks, &
        '/', omp_MaxThreads
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HDC: 2')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, chuTr, delta, &
    !$OMP                  m1c, n1c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HDC2_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HDC2_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPC2: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                             model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, i_chunk, &
                             m1c,n1c, m3c,n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m3c, n3c
      !
      ! --- Create empty chunks:                                   ---
      ! --- delta contains the changes of the conserved quantities ---
      ! --- chunk contains the new state                           ---
      call rhd_box_Make(chunk, m1=model%m2, n1=model%n2, m2=m1c, n2=n1c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)



      ! --------------------------------------------------------------------------------------------
!!$      call rhd_box_Make(delta, m1=model%m2, n1=model%n2, m2=m1c, n2=n1c, m3=m3c, n3=n3c, &
!!$                               nghost1=nghost, &
!!$                               quc_flag=.true., Bc_flag=Bc_flag)
!!$      !
!!$      ! --- Sum predictor step results for orthogonal directions ---
!!$      if (model%n1 > model%m1) then
!!$        call rhd_box_Copy( model1, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
!!$                           delta , delta%m1-mgh2, delta%m2,                     delta%m3, &
!!$                           trans_mode=2)
!!$        if (model%n3 > model%m3) then
!!$          call rhd_box_Add(model3, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
!!$                           delta , delta%m1-mgh2, delta%m2,                     delta%m3, &
!!$                           trans_mode=2)
!!$        endif
!!$      else if (model%n3 > model%m3) then
!!$        call rhd_box_Copy( model3, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
!!$                           delta , delta%m1-mgh2, delta%m2,                    delta%m3, &
!!$                           trans_mode=2)
!!$      endif
!!$      ! --- Use only half of the changes ---
!!$      call rhd_box_Factor(delta, &
!!$                          delta%m1-mgh2, delta%n1+ngh2, delta%m2, delta%n2, delta%m3, delta%n3, 0.5)
!!$      !
!!$      ! --- Add predictor step results for orthogonal directions ---
!!$      call rhd_box_Copy(model, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
!!$                        chunk, chunk%m1-mgh2, chunk%m2,                     chunk%m3, &
!!$                        trans_mode=2)
!!$      call rhd_box_AddDelta( &
!!$                   delta, delta%m1-mgh2, delta%n1+ngh2, delta%m2, delta%n2, delta%m3, delta%n3, &
!!$                   chunk, chunk%m1-mgh2,                chunk%m2,           chunk%m3, &
!!$                          eiTable_min, outstr=outstr_chunk, ierr=ierr_chunk)
!!$      !
!!$      ! --- Delete chunk ---
!!$      call rhd_box_Delete(delta)
      ! --------------------------------------------------------------------------------------------


      ! --------------------------------------------------------------------------------------------
      call rhd_box_Make(delta, m1=m1c, n1=n1c, m2=model%m2, n2=model%n2, m3=m3c, n3=n3c, &
                               nghost2=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Make(chuTr, m1=m1c, n1=n1c, m2=model%m2, n2=model%n2, m3=m3c, n3=n3c, &
                               nghost2=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      !
      ! --- Sum predictor step results for orthogonal directions ---
      if (model%n1 > model%m1) then
        call rhd_box_Copy( model1, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                           delta , delta%m1,      delta%m2-mgh2,                delta%m3, &
                           trans_mode=1)
        if (model%n3 > model%m3) then
          call rhd_box_Add(model3, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                           delta , delta%m1,      delta%m2-mgh2,                delta%m3, &
                           trans_mode=1)
        endif
      else if (model%n3 > model%m3) then
        call rhd_box_Copy( model3, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                           delta , delta%m1,      delta%m2-mgh2,                delta%m3, &
                           trans_mode=1)
      endif
      ! --- Use only half of the changes ---
      call rhd_box_Factor( &
                   delta, delta%m1, delta%n1, delta%m2-mgh2, delta%n2+ngh2, delta%m3, delta%n3, 0.5)
      !
      ! --- Add predictor step results for orthogonal directions ---
      call rhd_box_Copy(model, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                        chuTr, chuTr%m1,      chuTr%m2-mgh2,                chuTr%m3, &
                        trans_mode=1)
      call rhd_box_AddDelta( &
                   delta, delta%m1, delta%n1, delta%m2-mgh2, delta%n2+ngh2, delta%m3, delta%n3, &
                   chuTr, chuTr%m1,           chuTr%m2-mgh2,                chuTr%m3, &
                          eiTable_min, outstr=outstr_chunk, ierr=ierr_chunk)
      !
      ! --- Copy and transpose the chunk ---
      call rhd_box_Copy( &
                   chuTr, chuTr%m1, chuTr%n1, chuTr%m2-mgh2, chuTr%n2+ngh2, chuTr%m3, chuTr%m3, &
                   chunk, chunk%m1-mgh2,      chunk%m2,                     chunk%m3, &
                   trans_mode=2)
      !
      ! --- Delete chunks ---
      call rhd_box_Delete(chuTr)
      call rhd_box_Delete(delta)
      ! --------------------------------------------------------------------------------------------


      !
      ! --- Check for error ---
      if (ierr_chunk > 0) then
        ! --- Time step reduction ---
        dtime_limit_chunk=0.7*dtime
        !
      else
        ! --- Perform corrector step ---
        if (Output_flag0) then
          AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                       (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                       (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                       (model%xb3(model%n3+1)-model%xb3(model%m3))
          if (Chunks1Level_flag) then
            call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 ComputeDelta_flag=.true., &
                                 OutFlx_flag=Output_flag0, &
                                   OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=AreaFraction, &
                                   datx=dat_thread%x13, datr=dat_thread%r, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          else ! (.not. Chunks1Level_flag)
            call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 nghost, mgh2, ngh2, &
                                 ComputeDelta_flag=.true., &
                                 OutFlx_flag=Output_flag0, &
                                   OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=AreaFraction, &
                                   datx=dat_thread%x13, datr=dat_thread%r, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          endif ! (Chunks1Level_flag)
        else ! (.not. Output_flag0)
          if (Chunks1Level_flag) then
            call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 ComputeDelta_flag=.true., &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          else ! (.not. Chunks1Level_flag)
            call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 nghost, mgh2, ngh2, &
                                 ComputeDelta_flag=.true., &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          endif ! (Chunks1Level_flag)
        endif ! (Output_flag0)
        !
        if (StepDone_flag) then
          ! --- Add change ---
          call rhd_box_Add( chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                            modelB, m1c,               model%m2,           m3c, &
                            grid_mode=0, trans_mode=2)
        else
          ! --- Just copy chunk ---
          call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                            modelB, m1c,               model%m2,           m3c, &
                            grid_mode=0, trans_mode=2)
        endif
      endif
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPC2 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HDC2_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HDC2_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HDC: 2')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    StepDone_flag=.true.
    !
    ! --- End of corrector substep in x2 direction ---
  endif ! (model%n2 > model%m2)
  !
  if (model%n3 > model%m3) then
    ! === Corrector substep in x3 direction ========================================================
    !
    action_current=rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)))
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    if (model%n1 > model%m1) then
      ! --- Finalize filling of ghost layers of model1 in x3 direction ---
      call timing_start('HYD:   FillGhostEnd')
      call rhd_box_MPIFillGhostEnd(m1x3_RequestsMPI, m1x3_rhdMPI_boxGhostTypes)
      call timing_stop('HYD:   FillGhostEnd')
    endif
    if (model%n2 > model%m2) then
      ! --- Finalize filling of ghost layers of model2 in x3 direction ---
      call timing_start('HYD:   FillGhostEnd')
      call rhd_box_MPIFillGhostEnd(m2x3_RequestsMPI, m2x3_rhdMPI_boxGhostTypes)
      call timing_stop('HYD:   FillGhostEnd')
    endif
    !
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! \label{f90:n_hydcellsperchunk3__use2}
    ! --- Determine number of chunks ---
    if ((2*par%N_hydCellsPerChunk3 >= par%N_hydCellsPerChunk) .or. (model%n2 == model%m2)) then
      ! --- Perform normal 1-level-of-chunks partitioning ---
      Chunks1Level_flag=.true.
      N_hydCellsPerChunk=par%N_hydCellsPerChunk3
    else
      ! --- Perform 2-levels-of-chunks partitioning ---
      Chunks1Level_flag=.false.
      N_hydCellsPerChunk=max(min(par%N_hydCellsPerChunk3, &
                                 ceiling(real(model%n2-model%m2+1)/real(omp_MaxThreads))), &
                             1) * &
                         (model%n1-model%m1+1)*(model%n3-model%m3+1)
    endif
    call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8,A,I8)') &
        'HDC 3: N_CellsPerChunk, n_chunks: ', N_hydCellsPerChunk, n_chunks, &
        '/', omp_MaxThreads
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HDC: 3')
    !


    ! --- Preparation step done with full 3D models, not chunks ------------------------------------
!!$    if (model%n2 > model%m2) then
!!$      ! --- Sum predictor step results for orthogonal directions ---
!!$      if (model%n1 > model%m1) then
!!$        call rhd_box_AllAddOMP(    model1, m1, n1, m2, n2, m3-mgh3, n3+ngh3, &
!!$                                   model2, bulk_mode=1)
!!$      endif
!!$      !
!!$      ! --- Use only half of the changes ---
!!$      call rhd_box_AllFactorOMP(   model2, m1, n1, m2, n2, m3-mgh3, n3+ngh3, 0.5)
!!$      !
!!$      ! --- Add predictor step results for orthogonal direction(s) ---
!!$      call rhd_box_AllAddDelta2OMP(model , m1, n1, m2, n2, m3-mgh3, n3+ngh3, &
!!$                                   model2, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      !
!!$      ! --- Compose model in the rest of the MPI boundary layers ---
!!$      if (mgh1 > 0) then
!!$        ! --- Compose model in left x1 direction ghost cells ---
!!$        if (model%n1 > model%m1) then
!!$          call rhd_box_AllAddOMP(    model1, m1-mgh1, m1-1, m2, n2, m3, n3, &
!!$                                     model2, bulk_mode=1)
!!$        endif
!!$        call rhd_box_AllFactorOMP(   model2, m1-mgh1, m1-1, m2, n2, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , m1-mgh1, m1-1, m2, n2, m3, n3, &
!!$                                     model2, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$      if (ngh1 > 0) then
!!$        ! --- Compose model in right x1 direction ghost cells ---
!!$        if (model%n1 > model%m1) then
!!$          call rhd_box_AllAddOMP(    model1, n1+1, n1+ngh1, m2, n2, m3, n3, &
!!$                                     model2, bulk_mode=1)
!!$        endif
!!$        call rhd_box_AllFactorOMP(   model2, n1+1, n1+ngh1, m2, n2, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , n1+1, n1+ngh1, m2, n2, m3, n3, &
!!$                                     model2, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$      if (mgh2 > 0) then
!!$        ! --- Compose model in left x2 direction ghost cells ---
!!$        if (model%n1 > model%m1) then
!!$          call rhd_box_AllAddOMP(    model1, m1, m1, m2-mgh2, m2-1, m3, n3, &
!!$                                     model2, bulk_mode=1)
!!$        endif
!!$        call rhd_box_AllFactorOMP(   model2, m1, m1, m2-mgh2, m2-1, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , m1, m1, m2-mgh2, m2-1, m3, n3, &
!!$                                     model2, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$      if (ngh2 > 0) then
!!$        ! --- Compose model in right x2 direction ghost cells ---
!!$        if (model%n1 > model%m1) then
!!$          call rhd_box_AllAddOMP(    model1, m1, m1, n2+1, n2+ngh2, m3, n3, &
!!$                                     model2, bulk_mode=1)
!!$        endif
!!$        call rhd_box_AllFactorOMP(   model2, m1, m1, n2+1, n2+ngh2, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , m1, m1, n2+1, n2+ngh2, m3, n3, &
!!$                                     model2, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$    else
!!$      ! --- Use only half of the changes ---
!!$      call rhd_box_AllFactorOMP(model1, m1, n1, m2, n2, m3-mgh3, n3+ngh3, 0.5)
!!$      !
!!$      ! --- Add predictor step results for orthogonal direction ---
!!$      call rhd_box_AllAddDelta2OMP(model, m1, n1, m2, n2, m3-mgh3, n3+ngh3, &
!!$                                   model1, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      !
!!$      ! --- Compose model in the rest of the MPI boundary layers ---
!!$      if (mgh1 > 0) then
!!$        ! --- Compose model in left x1 direction ghost cells ---
!!$        call rhd_box_AllFactorOMP(   model1, m1-mgh1, m1-1, m2, n2, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , m1-mgh1, m1-1, m2, n2, m3, n3, &
!!$                                     model1, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$      if (ngh1 > 0) then
!!$        ! --- Compose model in right x1 direction ghost cells ---
!!$        call rhd_box_AllFactorOMP(   model1, n1+1, n1+ngh1, m2, n2, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , n1+1, n1+ngh1, m2, n2, m3, n3, &
!!$                                     model1, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$      if (mgh2 > 0) then
!!$        ! --- Compose model in left x2 direction ghost cells ---
!!$        call rhd_box_AllFactorOMP(   model1, m1, m1, m2-mgh2, m2-1, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , m1, m1, m2-mgh2, m2-1, m3, n3, &
!!$                                     model1, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$      if (ngh2 > 0) then
!!$        ! --- Compose model in right x2 direction ghost cells ---
!!$        call rhd_box_AllFactorOMP(   model1, m1, m1, n2+1, n2+ngh2, m3, n3, 0.5)
!!$        call rhd_box_AllAddDelta2OMP(model , m1, m1, n2+1, n2+ngh2, m3, n3, &
!!$                                     model1, eiTable_min, outstr=outstr0,ierr=ierr0)
!!$      endif
!!$    endif
!!$    if (ierr0 > 0) then
!!$      dtime_limit=0.7*dtime
!!$      goto 999
!!$    endif
    ! ----------------------------------------------------------------------------------------------


    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, chuTr, delta, &
    !$OMP                  m1c, n1c, m2c, n2c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HDC3_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HDC3_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPC3: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk, &
                             m1c,n1c, m2c,n2c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty chunks:                                   ---
      ! --- delta contains the changes of the conserved quantities ---
      ! --- chunk contains the new state                           ---
      call rhd_box_Make(chunk, m1=model%m3, n1=model%n3, m2=m2c, n2=n2c, m3=m1c, n3=n1c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)



      ! --------------------------------------------------------------------------------------------
!!$      call rhd_box_Make(delta, m1=model%m3, n1=model%n3, m2=m2c, n2=n2c, m3=m1c, n3=n1c, &
!!$                               nghost1=nghost, &
!!$                               quc_flag=.true., Bc_flag=Bc_flag)
!!$      !
!!$      ! --- Sum predictor step results for orthogonal directions ---
!!$      if (model%n1 > model%m1) then
!!$        call rhd_box_Copy( model1, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
!!$                           delta , delta%m1-mgh3, delta%m2,      delta%m3, &
!!$                           trans_mode=3)
!!$        if (model%n2 > model%m2) then
!!$          call rhd_box_Add(model2, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
!!$                           delta , delta%m1-mgh3, delta%m2,      delta%m3, &
!!$                           trans_mode=3)
!!$        endif
!!$      else if (model%n2 > model%m2) then
!!$        call rhd_box_Copy( model2, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
!!$                           delta , delta%m1-mgh3, delta%m2,      delta%m3, &
!!$                           trans_mode=3)
!!$
!!$      endif
!!$      ! --- Use only half of the changes ---
!!$      call rhd_box_Factor(delta, &
!!$                          delta%m1-mgh3, delta%n1+ngh3, delta%m2, delta%n2, delta%m3, delta%n3, 0.5)
!!$      !
!!$      ! --- Add predictor step results for orthogonal directions ---
!!$      call rhd_box_Copy(model, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
!!$                        chunk, chunk%m1-mgh3, chunk%m2,      chunk%m3, &
!!$                        trans_mode=3)
!!$      call rhd_box_AddDelta( &
!!$                   delta, delta%m1-mgh3, delta%n1+ngh3, delta%m2, delta%n2, delta%m3, delta%n3, &
!!$                   chunk, chunk%m1-mgh3,                chunk%m2,           chunk%m3, &
!!$                          eiTable_min, outstr=outstr_chunk, ierr=ierr_chunk)
!!$      !
!!$      ! --- Delete chunk ---
!!$      call rhd_box_Delete(delta)
      ! --------------------------------------------------------------------------------------------


      ! --------------------------------------------------------------------------------------------
      call rhd_box_Make(delta, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=model%m3, n3=model%n3, &
                               nghost3=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Make(chuTr, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=model%m3, n3=model%n3, &
                               nghost3=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      !
      ! --- Sum predictor step results for orthogonal directions ---
      if (model%n1 > model%m1) then
        call rhd_box_Copy( model1, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                           delta , delta%m1,      delta%m2,      delta%m3-mgh3, &
                           trans_mode=1)
        if (model%n2 > model%m2) then
          call rhd_box_Add(model2, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                           delta , delta%m1,      delta%m2,      delta%m3-mgh3, &
                           trans_mode=1)
        endif
      else if (model%n2 > model%m2) then
        call rhd_box_Copy( model2, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                           delta , delta%m1,      delta%m2,      delta%m3-mgh3, &
                           trans_mode=1)

      endif
      ! --- Use only half of the changes ---
      call rhd_box_Factor(delta, &
                          delta%m1, delta%n1, delta%m2, delta%n2, delta%m3-mgh3, delta%n3+ngh3, 0.5)
      !
      ! --- Add predictor step results for orthogonal directions ---
      call rhd_box_Copy(model, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                        chuTr, chuTr%m1,      chuTr%m2,      chuTr%m3-mgh3, &
                        trans_mode=1)
      call rhd_box_AddDelta( &
                   delta, delta%m1, delta%n1, delta%m2, delta%n2, delta%m3-mgh3, delta%n3+ngh3, &
                   chuTr, chuTr%m1,           chuTr%m2,           chuTr%m3-mgh3, &
                          eiTable_min, outstr=outstr_chunk, ierr=ierr_chunk)
      !
      ! --- Copy and transpose the chunk ---
      call rhd_box_Copy( &
                   chuTr, chuTr%m1, chuTr%n1, chuTr%m2, chuTr%n2, chuTr%m3-mgh3, chuTr%n3+ngh3, &
                   chunk, chunk%m1-mgh3,      chunk%m2,           chunk%m3, &
                   trans_mode=3)

      !
      ! --- Delete chunks ---
      call rhd_box_Delete(chuTr)
      call rhd_box_Delete(delta)
      ! --------------------------------------------------------------------------------------------



      ! --------------------------------------------------------------------------------------------
!!$      if (model%n2 > model%m2) then
!!$        call rhd_box_Copy(model2, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
!!$                          chunk , chunk%m1-mgh3, chunk%m2,      chunk%m3, &
!!$                          trans_mode=3)
!!$      else
!!$        call rhd_box_Copy(model1, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
!!$                          chunk , chunk%m1-mgh3, chunk%m2,      chunk%m3, &
!$$                          trans_mode=3)
!!$      endif
!!$      ierr_chunk=0
      ! --------------------------------------------------------------------------------------------


      !
      ! --- Check for error ---
      if (ierr_chunk > 0) then
        ! --- Time step reduction ---
        dtime_limit_chunk=0.7*dtime
        !
      else
        ! --- Perform corrector step ---
        if (Output_flag0) then
          AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                       (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                       (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                       (model%xb2(model%n2+1)-model%xb2(model%m2))
          if (Chunks1Level_flag) then
            call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 ComputeDelta_flag=.true., &
                                 OutFlx_flag=Output_flag0, &
                                   OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=AreaFraction, &
                                   datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          else ! (.not. Chunks1Level_flag)
            call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 nghost, mgh3, ngh3, &
                                 ComputeDelta_flag=.true., &
                                 OutFlx_flag=Output_flag0, &
                                   OutRadialC_flag=OutRadialC_flag, &
                                   AreaFraction=AreaFraction, &
                                   datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          endif ! (Chunks1Level_flag)
          !
        else ! (.not. Output_flag0)
          if (Chunks1Level_flag) then
            call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 ComputeDelta_flag=.true., &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          else ! (.not. Chunks1Level_flag)
            call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                                 nghost, mgh3, ngh3, &
                                 ComputeDelta_flag=.true., &
                                 outstr=outstr_chunk, ierr=ierr_chunk)
          endif ! (Chunks1Level_flag)
        endif ! (Output_flag0)
        !
        if (StepDone_flag) then
          ! --- Add change ---
          call rhd_box_Add( chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                            modelB,   m1c,             m2c,                model%m3, &
                            grid_mode=0, trans_mode=3)
        else
          ! --- Just copy chunk ---
          call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                            modelB,   m1c,             m2c,                model%m3, &
                            grid_mode=0, trans_mode=3)
        endif
      endif
      !
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPC3 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HDC3_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_datc_Add(dat%c  , dat_thread%c  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HDC3_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HDC: 3')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    StepDone_flag=.true.
    !
    !write(6,'(A,8(1PE13.5))') 'Modelo3:', minval(modelB%v1 ), maxval(modelB%v1 ), &
    !                                      minval(modelB%v3 ), maxval(modelB%v3 ), &
    !                                      minval(modelB%rho), maxval(modelB%rho), &
    !                                      minval(modelB%ei ), maxval(modelB%ei )
    !
    ! --- End of corrector substep in x3 direction ---
  endif ! (model%n3 > model%m3)
  !
  ! --- Copy back output model and delete temporary 3D model ---
  call timing_start('HYD: AddDelta')
  call rhd_box_AllAddDeltaOMP(modelB, model, action, dtime, .true., eiTable_min, &
                              outstr=outstr0, ierr=ierr0)
  call timing_stop('HYD: AddDelta')
  ! \label{f90:hdtmpstructkeepmode__useHYD1}
  if (par%hdTmpStructKeepMode == 'Delete') then
    call timing_start('HYD: delete_box')
    call rhd_box_Delete(modelB)
    call timing_stop('HYD: delete_box')
  endif
  if (ierr0 > 0) then
    ! --- An error occurred: reduce time step ---
    dtime_limit=0.7*dtime
  endif
  !
  !write(6,'(A,8(1PE13.5))') 'Model o:', minval(model%v1 ), maxval(model%v1 ), &
  !                                      minval(model%v3 ), maxval(model%v3 ), &
  !                                      minval(model%rho), maxval(model%rho), &
  !                                      minval(model%ei ), maxval(model%ei )
  !
  ! \label{f90:hdsplit__use2}
else if ((par%A%hdScheme /= hdScheme_None) .and. (par%hdsplit == 'unsplit')) then
  ! ### Direct unsplit #############################################################################
  !
  ! --- Indicate that no hydro step has been performed yet ---
  StepDone_flag=.false.
  !
  ! --- Get minimum internal energy value of EOS table ---
  call tabinter_getinfo(x2min=eiTable_min)
  !
  ! --- Create empty temporary 3D model, if necessary ---
  if (.not. rhd_box_Exist(modelB)) then
    call rhd_box_Make(modelB, &
                    m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3, &
                    quc_flag=.true., Bb_flag=.true.)
  endif
  ! --- Copy only axes ---
  call rhd_box_Copy(model,  model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                    modelB, model%m1,           model%m2,           model%m3, &
                    grid_mode=1, time_mode=1, bulk_mode=0)
  !
  ! \label{f90:c_rhochangetop__use3}
  ! === Open upper boundary condition ==============================================================
  if (((wg%hyd%bound_r(3) == bound_transmitting ) .or. &
       (wg%hyd%bound_r(3) == bound_transmitting2) .or. &
       (wg%hyd%bound_r(3) == bound_transmitting3)       ) .and. &
      ( par%grav_mode /= 'central')                       .and. &
      ( par%C_rhoChangeTop > 0.0)                                ) then
    call timing_start('HYD: bound_3DTransmit')
    call rhd_bound_3DTransmit(model, action, dtime)
    call timing_stop('HYD: bound_3DTransmit')
  endif
  !
  ! === Open lower boundary condition ==============================================================
  if ((wg%hyd%bound_l(3) == bound_inoutflow ) .or. &
      (wg%hyd%bound_l(3) == bound_inoutflow2) .or. &
      (wg%hyd%bound_l(3) == bound_inoutflow3)     ) then
    call timing_start('HYD: bound_3DInOutFlow')
    call rhd_bound_3DInOutFlow(model, action, dtime)
    call timing_stop('HYD: bound_3DInOutFlow')
  endif
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Fill ghost cells with data from neighbor processes ---
  !
  if (model%n3 > model%m3) then
    ! --- Start filling of ghost layers of model in x3 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 3, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x3_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x3_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
    !
    ! --- Finalize filling of ghost layers of model in x3 direction---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x3_RequestsMPI, m0x3_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
  endif
  !
  if (model%n2 > model%m2) then
    ! --- Start filling of ghost layers of model in x2 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 2, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x2_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x2_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
  !
  if (model%n1 > model%m1) then
    ! --- Start filling of ghost layers of model in x1 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 1, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x1_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
  endif
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  if (model%n3 > model%m3) then
    ! === Hydrodynamics substep in x3 direction ====================================================
    !
    action_current=rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)))
    !
    ! \label{f90:n_hydcellsperchunk3__use3}
    ! --- Determine number of chunks ---
    if ((2*par%N_hydCellsPerChunk3 >= par%N_hydCellsPerChunk) .or. (model%n2 == model%m2)) then
      ! --- Perform normal 1-level-of-chunks partitioning ---
      Chunks1Level_flag=.true.
      N_hydCellsPerChunk=par%N_hydCellsPerChunk3
    else
      ! --- Perform 2-levels-of-chunks partitioning ---
      Chunks1Level_flag=.false.
      N_hydCellsPerChunk=max(min(par%N_hydCellsPerChunk3, &
                                 ceiling(real(model%n2-model%m2+1)/real(omp_MaxThreads))), &
                             1) * &
                         (model%n1-model%m1+1)*(model%n3-model%m3+1)
    endif
    call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8,A,I8)') &
        'HYD 3: N_CellsPerChunk, n_chunks: ', N_hydCellsPerChunk, n_chunks, &
        '/', omp_MaxThreads
      call flush(gl%nc_m)
    endif
    !
    !write(6,*) 'rhd_hyd_SingleStep 3: ', minval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
    !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
    !                                     minval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc)), &
    !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc))
    ierr0  =0
    outstr0=''
    !
    call timing_start('HYD: 3')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m1c, n1c, m2c, n2c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HD3_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HD3_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPu3: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk, &
                             m1c,n1c, m2c,n2c)
      !write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m3, n1=model%n3, m2=m2c, n2=n2c, m3=m1c, n3=n1c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                        chunk, chunk%m1-mgh3, chunk%m2,      chunk%m3, &
                        trans_mode=3)
      !
      ! --- Perform actual hydro step ---
      if (Output_flag0) then
        AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                     (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                     (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                     (model%xb2(model%n2+1)-model%xb2(model%m2))
        if (Chunks1Level_flag) then
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Chunks1Level_flag)
          call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               nghost, mgh3, ngh3, &
                               ComputeDelta_flag=.true., &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Chunks1Level_flag)
        !
      else ! (.not. Output_flag0)
        if (Chunks1Level_flag) then
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Chunks1Level_flag)
          call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               nghost, mgh3, ngh3, &
                               ComputeDelta_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Chunks1Level_flag)
      endif ! (Output_flag0)
      !
      ! --- Copy data back into full model and delete temporary chunk ---
      if (StepDone_flag) then
        ! --- Add change ---
        call rhd_box_Add(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                         modelB, m1c,               m2c,                model%m3, &
                         grid_mode=0, trans_mode=3)
      else
        ! --- Just copy chunk ---
        call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                          modelB, m1c,               m2c,                model%m3, &
                          grid_mode=0, trans_mode=3)
      endif
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      if ((model%n1 <= model%m1) .and. &
          (model%n2 <= model%m2)        ) then
        ! --- Handle error conditions and time step ---
        if (ierr_chunk > ierr_thread) then
          ierr_thread  =ierr_chunk
          outstr_thread=outstr_chunk
        endif
        !
        ! --- Add total changes to output model ---
        call rhd_box_AddDelta(modelB, m1c, n1c, m2c, n2c, model%m3, model%n3, &
                              model,  m1c,      m2c,      model%m3,           &
                              eiTable_min, grid_mode=0, trans_mode=1, &
                              outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPu3 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HD3_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_datc_Add(dat%c  , dat_thread%c  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HD3_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HYD: 3')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of hydrodynamics substep in x3 direction ---
  endif ! (model%n3 > model%m3)
  !
  if (model%n2 > model%m2) then
    ! === Hydrodynamics substep in x2 direction ====================================================
    !
    action_current=rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)))
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    ! --- Finalize filling of ghost layers of model in x2 direction---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x2_RequestsMPI, m0x2_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, 1, &
                           m1c,n1c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HYD 2: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    !write(6,*) 'rhd_hyd_SingleStep 2: ', minval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
    !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
    !                                     minval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc)), &
    !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc))
    ierr0  =0
    outstr0=''
    !
    call timing_start('HYD: 2')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m1c, n1c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HD2_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HD2_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPu2: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, i_chunk, &
                             m1c,n1c, m3c,n3c)
      !write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m3c, n3c
      !call flush(gl%nc_p)
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m2, n1=model%n2, m2=m1c, n2=n1c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                        chunk, chunk%m1-mgh2, chunk%m2,                     chunk%m3, &
                        trans_mode=2)
      !
      ! --- Perform actual hydro step ---
      if (Output_flag0) then
        AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                     (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                     (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                     (model%xb3(model%n3+1)-model%xb3(model%m3))
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x13, datr=dat_thread%r, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag0)
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag0)
      !
      ! --- Copy data back into full model and delete temporary chunk ---
      if (StepDone_flag) then
        ! --- Add change ---
        call rhd_box_Add(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                         modelB, m1c,               model%m2,           m3c, &
                         grid_mode=0, trans_mode=2)
      else
        ! --- Just copy chunk ---
        call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                          modelB, m1c,               model%m2,           m3c, &
                          grid_mode=0, trans_mode=2)
      endif
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      if (model%n1 <= model%m1) then
        ! --- Handle error conditions and time step ---
        if (ierr_chunk > ierr_thread) then
          ierr_thread  =ierr_chunk
          outstr_thread=outstr_chunk
        endif
        !
        ! --- Add total changes to output model ---
        call rhd_box_AddDelta(modelB, m1c     , n1c, model%m2, model%n2, m3c     , n3c, &
                              model,  m1c,           model%m2,           m3c,           &
                              eiTable_min, grid_mode=0, trans_mode=1, &
                              outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPu2 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HD2_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HD2_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HYD: 2')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of hydrodynamics substep in x2 direction ---
  endif ! (model%n2 > model%m2)
  !
  if (model%n1 > model%m1) then
    ! === Hydrodynamics substep in x1 direction ====================================================
    !
    action_current=rhd_action_New(action, dim=1)
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    ! --- Finalize filling of ghost layers of model in x1 direction---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                           m2c,n2c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HYD 1: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    !write(6,*) 'rhd_hyd_SingleStep 1: ', minval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
    !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
    !                                     minval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc)), &
    !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc))
    ierr0  =0
    outstr0=''
    !
    call timing_start('HYD: 1')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m2c, n2c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HD1_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HD1_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPu1: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk, &
                             m2c,n2c, m3c,n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                        chunk, chunk%m1-mgh1,                chunk%m2,      chunk%m3, &
                        trans_mode=1)
      !
      ! --- Perform actual hydro step ---
      if (Output_flag0) then
        AreaFraction=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                     (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                     (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                     (model%xb3(model%n3+1)-model%xb3(model%m3))
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x23, datr=dat_thread%r, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag0)
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               ComputeDelta_flag=.true., &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag0)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      !
      ! --- Copy data back into full model and delete temporary chunk ---
      if (StepDone_flag) then
        ! --- Add change ---
        call rhd_box_Add(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                         modelB, model%m1,          m2c,                m3c, &
                         grid_mode=0, trans_mode=1)
      else
        ! --- Just copy chunk ---
        call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                          modelB, model%m1,          m2c,                m3c, &
                          grid_mode=0, trans_mode=1)
      endif
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      ! --- Add total changes to output model ---
      call rhd_box_AddDelta(modelB, modelB%m1, modelB%n1, m2c, n2c,  m3c, n3c,  &
                            model,  model%m1,                  m2c,       m3c,       &
                            eiTable_min, grid_mode=0, trans_mode=1, &
                            outstr=outstr_chunk, ierr=ierr_chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPu1 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HD1_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HD1_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HYD: 1')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of hydrodynamics substep in x1 direction ---
  endif ! (model%n1 > model%m1)
  !
  !write(6,*) 'rhd_hyd_SingleStep F: ', minval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
  !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2+1,dat%c%i_xb)), &
  !                                     minval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc)), &
  !                                     maxval(dat%c%d1(dat%c%m2:dat%c%n2  ,dat%c%i_xc))
  ! --- Delete temporary model ---
  ! \label{f90:hdtmpstructkeepmode__useHYD2}
  if (par%hdTmpStructKeepMode == 'Delete') then
    call timing_start('HYD: delete_box')
    call rhd_box_Delete(modelB)
    call timing_stop('HYD: delete_box')
  endif
  !
else if (par%A%hdScheme /= hdScheme_None) then
  ! ### Directional splitting (123) ################################################################
  !
  ! --- Indicate that no hydro step has been performed yet ---
  StepDone_flag=.false.
  !
  if (model%n1 > model%m1) then
    ! === Hydrodynamics substep in x1 direction ====================================================
    !
    action_current=rhd_action_New(action, dim=1)
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    ! --- Start filling of ghost layers of model in x1 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 1, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x1_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
    !
    ! --- Finalize filling of ghost layers of model in x1 direction---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                           m2c,n2c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HYD 1: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HYD: 1')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m2c, n2c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HD1_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HD1_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP1: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk, &
                             m2c,n2c, m3c,n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m2c, n2c, ' : ', m3c, n3c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m1, n1=model%n1, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, model%m1-mgh1, model%n1+ngh1, m2c     , n2c, m3c     , n3c, &
                        chunk, chunk%m1-mgh1,                chunk%m2,      chunk%m3, &
                        trans_mode=1)
      !
      ! --- Perform actual hydro step ---
      if (Output_flag0) then
        AreaFraction=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                     (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                     (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                     (model%xb3(model%n3+1)-model%xb3(model%m3))
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x23, datr=dat_thread%r, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag0)
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag0)
      !
      ! --- Copy data back into full model and delete temporary chunk ---
      call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                        model, model%m1,           m2c,                m3c, &
                        grid_mode=0, trans_mode=1)
      ! --- Delete chunk ---
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP1 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HD1_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HD1_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HYD: 1')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of hydrodynamics substep in x1 direction ---
  endif ! (model%n1 > model%m1)
  !
  if (model%n2 > model%m2) then
    ! === Hydrodynamics substep in x2 direction ====================================================
    !
    action_current=rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)))
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    ! --- Start filling of ghost layers of model in x2 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 2, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x2_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x2_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
    !
    ! --- Finalize filling of ghost layers of model in x2 direction---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x2_RequestsMPI, m0x2_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                           model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, 1, &
                           m1c,n1c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'HYD 2: N_CellsPerChunk, n_chunks: ', par%N_hydCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HYD: 2')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m1c, n1c, m3c, n3c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HD2_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HD2_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP2: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_hydCellsPerChunk, &
                             model%m2,model%n2, model%m1,model%n1, model%m3,model%n3, i_chunk, &
                             m1c,n1c, m3c,n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m3c, n3c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m2, n1=model%n2, m2=m1c, n2=n1c, m3=m3c, n3=n3c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, m1c     , n1c, model%m2-mgh2, model%n2+ngh2, m3c     , n3c, &
                        chunk, chunk%m1-mgh2, chunk%m2,                     chunk%m3, &
                        trans_mode=2)
      !
      ! --- Perform actual hydro step ---
      if (Output_flag0) then
        AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                     (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                     (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                     (model%xb3(model%n3+1)-model%xb3(model%m3))
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x13, datr=dat_thread%r, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else ! (.not. Output_flag0)
        call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif ! (Output_flag0)
      !
      ! --- Copy data back into full model and delete temporary chunk ---
      call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                        model, m1c,                model%m2,           m3c, &
                        grid_mode=0, trans_mode=2)
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP2 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HD2_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HD2_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HYD: 2')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of hydrodynamics substep in x2 direction ---
  endif ! (model%n2 > model%m2)
  !
  ! \label{f90:c_rhochangetop__use4}
  ! === Open upper boundary condition ==============================================================
  if (((wg%hyd%bound_r(3) == bound_transmitting ) .or. &
       (wg%hyd%bound_r(3) == bound_transmitting2) .or. &
       (wg%hyd%bound_r(3) == bound_transmitting3)       ) .and. &
      ( par%grav_mode /= 'central')                       .and. &
      ( par%C_rhoChangeTop > 0.0)                                ) then
    call timing_start('HYD: bound_3DTransmit')
    call rhd_bound_3DTransmit(model, action, dtime)
    call timing_stop('HYD: bound_3DTransmit')
  endif
  !
  ! === Open lower boundary condition ==============================================================
  if ((wg%hyd%bound_l(3) == bound_inoutflow ) .or. &
      (wg%hyd%bound_l(3) == bound_inoutflow2) .or. &
      (wg%hyd%bound_l(3) == bound_inoutflow3)     ) then
    call timing_start('HYD: bound_3DInOutFlow')
    call rhd_bound_3DInOutFlow(model, action, dtime)
    call timing_stop('HYD: bound_3DInOutFlow')
  endif
  !
  if (model%n3 > model%m3) then
    ! === Hydrodynamics substep in x3 direction ====================================================
    !
    action_current=rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)))
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    ! --- Start filling of ghost layers of model in x3 direction ---
    call timing_start('HYD:   FillGhostStart')
    call rhd_box_MPIFillGhostStart(wg%hyd, model, 3, nghost, &
                                   m1,n1, m2,n2, m3,n3, &
                                   RequestsMPI=m0x3_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x3_rhdMPI_boxGhostTypes, &
                                   grid_mode=0, bulk_mode=2)
    call timing_stop('HYD:   FillGhostStart')
    !
    ! --- Finalize filling of ghost layers of model in x3 direction---
    call timing_start('HYD:   FillGhostEnd')
    call rhd_box_MPIFillGhostEnd(m0x3_RequestsMPI, m0x3_rhdMPI_boxGhostTypes)
    call timing_stop('HYD:   FillGhostEnd')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! \label{f90:n_hydcellsperchunk3__use4}
    ! --- Determine number of chunks ---
    if ((2*par%N_hydCellsPerChunk3 >= par%N_hydCellsPerChunk) .or. (model%n2 == model%m2)) then
      ! --- Perform normal 1-level-of-chunks partitioning ---
      Chunks1Level_flag=.true.
      N_hydCellsPerChunk=par%N_hydCellsPerChunk3
    else
      ! --- Perform 2-levels-of-chunks partitioning ---
      Chunks1Level_flag=.false.
      N_hydCellsPerChunk=max(min(par%N_hydCellsPerChunk3, &
                                 ceiling(real(model%n2-model%m2+1)/real(omp_MaxThreads))), &
                             1) * &
                         (model%n1-model%m1+1)*(model%n3-model%m3+1)
    endif
    call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8,A,I8)') &
        'HYD 3: N_CellsPerChunk, n_chunks: ', N_hydCellsPerChunk, n_chunks, &
        '/', omp_MaxThreads
      call flush(gl%nc_m)
    endif
    !
    ierr0  =0
    outstr0=''
    !
    call timing_start('HYD: 3')
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, chunk, &
    !$OMP                  m1c, n1c, m2c, n2c, AreaFraction, &
    !$OMP                  i_chunk, dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (HD3_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag0) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (HD3_INIT)
    !
    ! --- Loop over all chunks ---
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOP3: do i_chunk=1,n_chunks
      !
      ! --- Get current chunk indices ---
      call rhd_box_ChunkSize(0, N_hydCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk, &
                             m1c,n1c, m2c,n2c)
      !write(gl%nc_p,'(I3,I6,I6,2(A,2I10))') &
      !  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c, n1c, ' : ', m2c, n2c
      !
      ! --- Create empty chunk and fill it with data ---
      call rhd_box_Make(chunk, m1=model%m3, n1=model%n3, m2=m2c, n2=n2c, m3=m1c, n3=n1c, &
                               nghost1=nghost, &
                               quc_flag=.true., Bc_flag=Bc_flag)
      call rhd_box_Copy(model, m1c     , n1c, m2c     , n2c, model%m3-mgh3, model%n3+ngh3, &
                        chunk, chunk%m1-mgh3, chunk%m2,      chunk%m3, &
                        trans_mode=3)
      !
      ! --- Perform actual hydro step ---
      if (Output_flag0) then
        AreaFraction=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                     (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                     (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                     (model%xb2(model%n2+1)-model%xb2(model%m2))
        if (Chunks1Level_flag) then
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Chunks1Level_flag)
          call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               nghost, mgh3, ngh3, &
                               OutVar_flag=Output_flag0, OutFlx_flag=Output_flag0, &
                                 OutRadialC_flag=OutRadialC_flag, &
                                 AreaFraction=AreaFraction, &
                                 datx=dat_thread%x12, datr=dat_thread%r, datc=dat_thread%c, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Chunks1Level_flag)
        !
      else ! (.not. Output_flag0)
        if (Chunks1Level_flag) then
          call rhd_hyd_Roe1DStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        else ! (.not. Chunks1Level_flag)
          call rhd_hyd_Roe1DMincedStep(chunk, action_current, dtime, dtime_limit_chunk, &
                               nghost, mgh3, ngh3, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
        endif ! (Chunks1Level_flag)
      endif ! (Output_flag0)
      !
      ! --- Copy data back into full model and delete temporary chunk ---
      call rhd_box_Copy(chunk, chunk%m1, chunk%n1, chunk%m2, chunk%n2, chunk%m3, chunk%n3, &
                        model, m1c,                m2c,                model%m3, &
                        grid_mode=0, trans_mode=3)
      call rhd_box_Delete(chunk)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOP3 ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (HD3_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit =min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag0) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_datc_Add(dat%c  , dat_thread%c  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (HD3_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    call timing_stop('HYD: 3')
    !
    ! --- Handle error conditions ---
    if (ierr0 > 0) goto 999
    !
    ! --- Indicate that at least one hydro step has been performed, already ---
    OutRadialC_flag=.false.
    StepDone_flag=.true.
    !
    ! --- End of hydrodynamics substep in x3 direction ---
  endif ! (model%n3 > model%m3)
  !
else ! (par%A%hdScheme == ..., par%hdsplit == ...)
  ! --- Skip hydrodynamics step entirely ---
  !
  ierr0  =0
  outstr0=''
endif ! (par%A%hdScheme == ..., par%hdsplit == ...)
!
999 continue
! \label{f90:hdsplit__use3}\label{f90:hdtmpstructkeepmode__useHYD3}
if ((par%A%hdScheme /= hdScheme_None) .and. (par%hdTmpStructKeepMode == 'Delete')) then
  if (par%hdsplit == 'unsplit') then
    call timing_start('HYD: delete_box')
    if (rhd_box_Exist(modelB)) call rhd_box_Delete(modelB)
    call timing_stop('HYD: delete_box')
  else if (par%hdsplit == 'CTU') then
    call timing_start('HYD: delete_box')
    if (rhd_box_Exist(modelB)) call rhd_box_Delete(modelB)
    if (rhd_box_Exist(model1)) call rhd_box_Delete(model1)
    if (rhd_box_Exist(model2)) call rhd_box_Delete(model2)
    if (rhd_box_Exist(model3)) call rhd_box_Delete(model3)
    call timing_stop('HYD: delete_box')
  endif
endif
!
call timing_stop('HYD: SingleStep')
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine rhd_hyd_SingleStep


!----------************-----------------------------------------------------------------------------
subroutine rhd_hyd_Step(model, action, dtime, dtime_limit, &
                        Output_flag, dat, &
                        outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_hyd_Step ('rhd_hydrodynamics_Step')
!
! PURPOSE:
!   Perform entire 2D or 3D hydrodynamics step, employ Roe scheme.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_hyd_Step(model, action, dtime, dtime_limit, &
!                     Output_flag=Output_flag, &
!                     dat=dat, &
!                     outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Output_flag:  ('Output_flag') logical, optional, flag that determines if additional averages are
!                 computed and put into dat
!
! INPUT/OUTPUT:
!   model:        ('model') type(box_type), model data: rho, ei, v, x, time, ...
!   dat:          ('data') type(dat_type), optional, structure for additional quantities
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, Courant time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successfully completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error: (currently none)
!
! LOCAL VARIABLES:
!   LastStep_flag: ('Last_Step_flag') logical to indicate that current step is the last one
!   time_start:   ('time_start') real with local time (0.0) at beginning of hydro iteration
!   time:         ('time') real with local time, beginning at 0.0, advanced after each iteration
!   dtime_step:   ('delta_time_step') real, with current time step of iteration.
!                 Each single hydro iteration is advanced by this amount.
!   dtime_limit_step: ('delta_time_limit_step') real, current time-step limit given by
!                     rhd_hyd_SingleStep
!   dtime_limit_sum: ('delta_time_limit_su,') real, sum of the time-step limits of all
!                     individual steps
!   i_Iter:       ('integer_Iteration') integer with iteration count
!   i_RKIter:     ('integer_Runge-Kutta_Iteration') integer with count of Runge-Kutta sub steps
!   ierr0:        ('integer_error_0') integer with error status
!   outstr0:      ('output_string_0') character with error report
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   ProcErr:      ('Process_Error') integer with the rank of the process with the largest error
!
! ROUTINES:
!   rhd_box_AllAvgOMP:     Average two boxes, in parallel if n3>m3
!   rhd_box_AllCopyOMP:    Copy entire box, in parallel if box is 3D
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_hyd_SingleStep:    Perform entire (single) 2D or 3D hydrodynamics step, employ Roe scheme
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_MPI_module:        MPI routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   rhd_vis_module:        Artificial-tensor-viscosity routines
!   rhd_wrk_module:        Provide persistent work copies of model data
!   timing_module:         Timing routines for performance measurements
!
! SIDE EFFECTS:
!   The global variable modelA is modified.
!
! RESTRICTIONS:
!   Additional (mean) output is only computed during the first sub time step.
!   This routine should only be called for a process that is member of rhdMPI_COMM_HD -- and for
!   all of them.
!
! PROCEDURE:
!   If hydro iterations are requested, call rhd_hyd_SingleStep repeatedly,
!   otherwise only once (or two or three times for Runge-Kutta steps).
!
! MODIFICATION HISTORY:
!   2006-12-07 (B.F. Lyon) First version of hydro-iteration routine started
!   2006-12-12 (B.F.) First version
!   2007-02-10 (B.F.) New version of formula for new dtime_limit
!   2011-07-12 (B.F.) Go also for (par%N_hydIter <= 0) through iteration branch
!   2011-07-17 (B.F.) Rearrange nested branches for error checking;
!                     OutRadialC_flag no longer handled
!   2012-04-05 (B.F., W.S.) Bugfix: for par%N_hydIter=0 dtime_limit could become zero
!   2012-11-07 (B.F.) Added arguments for MPI
!   2012-11-08 (B.F.) Call rhd_MPI_DtimeErrorDist
!   2012-11-12 (B.F.) MPI parameters put into prop structure
!   2012-11-16 (B.F.) Suppress error output to screen (that is done in calling routine)
!   2014-08-14 (B.F. Uppsala) Put RungeKutta2 sub steps into loop controlled by par%N_hydRKIter
!   2014-08-17 (B.F.) Replace local model0 with global modelA
!   2014-12-17 (B.F.) Optionally call tensor-viscosity step during hydrodynamics iteration
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
use rhd_wrk_module
use rhd_MPI_module
use rhd_vis_module
use timing_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: Output_flag
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: LastStep_flag
integer                                 :: i_Iter, i_RKIter, i_IterVis, i_IterHDVis
real                                    :: time_start, time, &
                                           dtime_step, dtime_limit_step, dtime_limit_sum, &
                                           dtime_stepVis, dtime_limit_stepVis, dtime_limit_Vis
integer                                 :: ierr0
character(len=80)                       :: outstr0
logical, parameter                      :: Bc_flag=.false.
! --- MPI variables ---
integer                                 :: ierrMPI, ProcErr
!---------------------------------------------------------------------------------------------------
!
! === Perform the iteration of a single hydrodynamics step ===
!
ierr0  =0
outstr0=''
!
! \label{f90:n_hyditer__use1}
! --- Save global time step and estimate sub-timestep ---
time_start     =0.0
time           =time_start
dtime_limit_sum=0.0
dtime_step     =dtime/max(par%N_hydIter, 1)
i_Iter         =1
i_IterHDVis    =0
i_IterVis      =0
dtime_stepVis  =0.0
!
if ((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
    (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) then
  ! --- Create empty temporary 3D model, if necessary ---
  if (.not. rhd_box_Exist(modelA)) then
    call rhd_box_Make(modelA, &
                    m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, m3=model%m3, n3=model%n3, &
                    quc_flag=.true., Bb_flag=.true., Bc_flag=Bc_flag)
  endif
endif
!
! --- Loop over all sub-timesteps ------------------------------------------------------------------
TIME_LOOP: do
  !
  if (par%N_hydIter <= 0) then
    ! --- Perform no iteration ---
    !
    LastStep_flag=.true.
  else
    ! --- Iterate ---
    !
    ! \label{f90:n_hydmaxiter__use1}\label{f90:n_hyditer__use2}
    ! --- Check if maximum number of iterations is exceeded (identical on all processes) ---
    if (par%N_hydMaxIter > 0) then
      if (i_Iter > par%N_hydMaxIter) then
        ierr0  =1
        outstr0='HD: Number of sub time steps above absolute limit'
        dtime_limit=dtime * real(    par%N_hydIter                    )/ &
                            real(max(par%N_hydIter+1, par%N_hydMaxIter))
        exit TIME_LOOP
      endif
    endif
    !
    ! --- Check absolute lower time step limit (identical on all processes) ---
    if (dtime_step < (par%dtime_min_stop/par%N_hydIter)) then
      ierr0  =200
      outstr0='Time step below absolute limit'
      exit TIME_LOOP
    endif
    !
    ! \label{f90:c_courantmax__use2}\label{f90:c_courant__use3}
    ! --- Check if this step is last one (identical on all processes) ---
    ! --- Allow a tiny increase in time-step above recommended value  ---
    if ((dtime_step * (0.1*max((par%C_CourantMax-par%C_Courant)/par%C_Courant, 0.0)+1.0)) >= &
        (dtime-(time-time_start))                                                           ) then
      ! --- Estimate of optimum next time step ---
      !dtime_limit=(0.5*(dtime+(time-time_start)+dtime_step)) * real(par%N_hydIter)/real(i_Iter)
      !
      ! --- Limit current step to reach exactly the overall dtime ---
      dtime_step=dtime-(time-time_start)
      LastStep_flag=.true.
    else
      LastStep_flag=.false.
    endif
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Broadcast flag from master process to the rest of the HD world, just in case ---
    call MPI_BCast(LastStep_flag, 1, MPI_LOGICAL, wg%hyd%MaProcWoInCO, wg%hyd%Comm, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(I4,A,1PE10.4)') i_Iter, ' Hyd-substep, dtimesub=', dtime_step
    endif
  endif
  !
  ! --- Perform HD substep ---
  !
  ! \label{f90:hdtimeintegrationscheme__use1}
  select case(par%hdTimeIntegrationScheme)
    case ('RungeKutta3')
      ! === Perform 3rd order TVD Runge-Kutta step =================================================
      !
      ! --- Save old model ---
      call timing_start('HYD: copy_box')
      call rhd_box_AllCopyOMP(model, modelA, grid_mode=1, flux_mode=0)
      call timing_stop('HYD: copy_box')
      !
      ! --- Perform first Euler step ---
      call rhd_hyd_SingleStep(model, action, dtime_step, dtime_limit_step, &
                              Output_flag=(Output_flag .and. (i_Iter == 1)), dat=dat, &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      call rhd_MPI_DtimeErrorDist(wg%hyd%Comm, wg%hyd%MyProc, wg%hyd%MaProcWoInCO, &
                                  dtime_limit_step, ierr0, outstr0, ProcErr)
      if (ierr0 > 0) then
        ! --- Handle the error ---
        !
        if (ierr0 < 100) then
          ! --- Minor error: reduce time step and do HD substep again ---
          ! --- Otherwise: severe error: exit program gracefully      ---
          !
          ! \label{f90:n_hyditer__use3}
          dtime_limit=min(max(par%N_hydIter,1)*dtime_limit_step, 0.7*dtime)
        endif
        !
        exit TIME_LOOP
      endif
      !
      ! --- Perform second Euler step ---
      call rhd_hyd_SingleStep(model, action, dtime_step, dtime_limit_step, &
                              Output_flag=.false., dat=dat, &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      call rhd_MPI_DtimeErrorDist(wg%hyd%Comm, wg%hyd%MyProc, wg%hyd%MaProcWoInCO, &
                                  dtime_limit_step, ierr0, outstr0, ProcErr)
      if (ierr0 > 0) then
        ! --- Handle the error ---
        !
        if (ierr0 < 100) then
          ! --- Minor error: reduce time step and do HD substep again ---
          ! --- Otherwise: severe error: exit program gracefully      ---
          !
          dtime_limit=min(max(par%N_hydIter,1)*dtime_limit_step, 0.7*dtime)
        endif
        !
        exit TIME_LOOP
      endif
      !
      ! --- Combine models ---
      call timing_start('HYD: avg_box')
      call rhd_box_AllAvgOMP(modelA, model, 0.25)
      call timing_stop('HYD: avg_box')
      !
      ! --- Perform third Euler step ---
      call rhd_hyd_SingleStep(model, action, dtime_step, dtime_limit_step, &
                              Output_flag=.false., dat=dat, &
                              outstr=outstr0,ierr=ierr0)
      !
      ! --- Handle error conditions and time step ---
      call rhd_MPI_DtimeErrorDist(wg%hyd%Comm, wg%hyd%MyProc, wg%hyd%MaProcWoInCO, &
                                  dtime_limit_step, ierr0, outstr0, ProcErr)
      if (ierr0 > 0) then
        ! --- Handle the error ---
        !
        if (ierr0 < 100) then
          ! --- Minor error: reduce time step and do HD substep again ---
          ! --- Otherwise: severe error: exit program gracefully      ---
          !
          dtime_limit=min(max(par%N_hydIter,1)*dtime_limit_step, 0.7*dtime)
        endif
        !
        exit TIME_LOOP
      endif
      !
      ! --- Combine models ---
      call timing_start('HYD: avg_box')
      call rhd_box_AllAvgOMP(modelA, model, 2.0/3.0)
      call timing_stop('HYD: avg_box')
      !
    case ('RungeKutta2')
      ! === Perform 2nd order TVD Runge-Kutta step =================================================
      !
      ! --- Save old model ---
      call timing_start('HYD: copy_box')
      call rhd_box_AllCopyOMP(model, modelA, grid_mode=1, flux_mode=0)
      call timing_stop('HYD: copy_box')
      !
      ! \label{f90:n_hydrkiter__use1}
      ! --- Loop over Runge-Kutta steps ---
      do i_RKIter=1,par%N_hydRKIter
        !
        ! --- Perform Euler step ---
        call rhd_hyd_SingleStep(model, action, &
                     dtime_step/real(par%N_hydRKIter-1), dtime_limit_step, &
                     Output_flag=(Output_flag .and. (i_Iter == 1) .and. (i_RKIter == 1)), dat=dat, &
                     outstr=outstr0,ierr=ierr0)
        dtime_limit_step=dtime_limit_step*real(par%N_hydRKIter-1)
        !
        ! --- Handle error conditions and time step ---
        call rhd_MPI_DtimeErrorDist(wg%hyd%Comm, wg%hyd%MyProc, wg%hyd%MaProcWoInCO, &
                                    dtime_limit_step, ierr0, outstr0, ProcErr)
        if (ierr0 > 0) then
          ! --- Handle the error ---
          !
          if (ierr0 < 100) then
            ! --- Minor error: reduce time step and do HD substep again ---
            ! --- Otherwise: severe error: exit program gracefully      ---
            !
            dtime_limit=min(max(par%N_hydIter,1)*dtime_limit_step, 0.7*dtime)
          endif
          !
          exit TIME_LOOP
        endif
        !
      end do ! i_RKIter
      !
      ! --- Combine models ---
      call timing_start('HYD: avg_box')
      call rhd_box_AllAvgOMP(modelA, model, real(par%N_hydRKIter-1)/real(par%N_hydRKIter))
      call timing_stop('HYD: avg_box')
      !
    case default
      ! === Perform single step ====================================================================
      !
      ! --- Single step (with higher-order temporal terms computed internally) ---
      call rhd_hyd_SingleStep(model, action, dtime_step, dtime_limit_step, &
                              Output_flag=(Output_flag .and. (i_Iter == 1)), dat=dat, &
                              outstr=outstr0,ierr=ierr0)
      !
      call rhd_MPI_DtimeErrorDist(wg%hyd%Comm, wg%hyd%MyProc, wg%hyd%MaProcWoInCO, &
                                  dtime_limit_step, ierr0, outstr0, ProcErr)
      !
  end select ! (par%hdTimeIntegrationScheme)
  !
  if (par%N_hydIter <= 0) then
    ! --- Perform no iteration, output error state from last call to rhd_hyd_SingleStep ---
    !
    dtime_limit=dtime_limit_step
    !
    exit TIME_LOOP
  else ! (par%N_hydIter > 0)
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) then
      ! --- An error occurred ---
      !
      if (ierr0 < 100) then
        ! --- Minor error: reduce dtime_limit and start time step again  ---
        ! --- Otherwise: severe error: exit the program gracefully       ---
        !
        dtime_limit=min(par%N_hydIter*dtime_limit_step, 0.7*dtime)
      endif
      !
      exit TIME_LOOP
    endif
    !
    ! --- Update time variables and counter ---
    time           =time+dtime_step
    dtime_limit_sum=dtime_limit_sum+dtime_limit_step
    i_IterHDVis    =i_IterHDVis+1
    dtime_stepVis  =dtime_stepVis+dtime_step
    !
    ! \label{f90:c_visp2pcoeff__use5}\label{f90:c_visp2phypartificial__use5}\label{f90:n_visiter__use2}
    if (((par%N_visIter < 0 ) .and. &
         ((i_IterHDVis >= (-par%N_visIter)) .or. (LastStep_flag))                        ) .and. &
        ((par%C_visLinear           > 0.0) .or. (par%C_visSmagorinsky       > 0.0) .or. &
         (par%C_visArtificial       > 0.0) .or. (par%C_visExpansion         > 0.0) .or. &
         (par%C_visP2Pcoeff         > 0.0) .or. (par%C_visP2PhypSmagorinsky > 0.0) .or. &
         (par%C_visP2PhypArtificial > 0.0) .or. (par%C_visP2PhypExpansion   > 0.0) .or. &
         (par%C_visP2Pdivrhov       > 0.0)                                               )    ) then
      !
      ! --- Perform tensor-viscosity step ----------------------------------------------------------
      !
      ! --- Increment the counter of total tensor-viscosity substeps ---
      i_IterVis  =i_IterVis+1
      !
      ! --- Perform one tensor-viscosity substep ---
      call rhd_vis_Step(model, action, dtime_stepVis, dtime_limit_Vis, &
                        (Output_flag .and. (i_IterVis <= 1)), dat, &
                        outstr,ierr)
      !
      if (gl%nc_m >= 0) then
        write(gl%nc_m,'(3(A,1PE10.4))') ' dtimesub_VIS='     , dtime_stepVis, &
                                       ', dtimesub_limitVIS=', dtime_limit_Vis, &
                                       ', dtimesub_limitHD=' , dtime_limit_step
      endif
      !
      ! --- Reset the counter of hydro steps since last viscosity substep ---
      i_IterHDVis  =0
      dtime_stepVis=0.0
      !
      ! --- Handle error conditions and time step ---
      if (ierr0 > 0) then
        ! --- An error occurred ---
        !
        if (ierr0 < 100) then
          ! --- Minor error: reduce dtime_limit and start time step again  ---
          ! --- Otherwise: severe error: exit the program gracefully       ---
          !
          dtime_limit=min(float(i_Iter)/float(i_IterVis) * dtime_limit_Vis, &
                          0.7*dtime)
        endif
        !
        exit TIME_LOOP
      endif
    endif
    !
    ! --- Determine new time step ---
    if (par%dtime_incmax > 1.0) then
      dtime_step=min(dtime_limit_step, dtime_step*par%dtime_incmax)
    else
      dtime_step=dtime_limit_step
    endif
    !
    if (LastStep_flag) then
      ! \label{f90:n_hyditer__use4}
      dtime_limit=(0.5*(dtime+dtime_limit_sum)) * real(par%N_hydIter)/real(i_Iter)
      if (i_IterVis > 0) then
        ! --- At least one tensor-viscosity substep has been performed: ---
        ! --- Account for additional limit                              ---
        dtime_limit=min(float(par%N_hydIter)/float(min(-par%N_visIter, par%N_hydIter)) * &
                          dtime_limit_Vis, &
                        dtime_limit)
      endif
      exit TIME_LOOP
    endif
    !
    i_Iter=i_Iter+1
  endif ! (par%N_hydIter <= 0)
end do TIME_LOOP
! --- End of main loop -----------------------------------------------------------------------------
!
! \label{f90:hdtmpstructkeepmode__useHYD4}
if (((par%hdTimeIntegrationScheme == 'RungeKutta2') .or. &
     (par%hdTimeIntegrationScheme == 'RungeKutta3')     ) .and. &
    (par%hdTmpStructKeepMode == 'Delete')                      ) then
  call timing_start('HYD: delete_box')
  if (rhd_box_Exist(modelA)) call rhd_box_Delete(modelA)
  call timing_stop('HYD: delete_box')
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine rhd_hyd_Step

end module rhd_hyd_module
