!**************************************************************************************************
!
!   #####   #    #  #####          #       #    #  #####   #####     ##    #####
!   #    #  #    #  #    #         #       #    #  #    #  #    #   #  #   #    #
!   #    #  ######  #    #         #       ######  #    #  #    #  #    #  #    #
!   #####   #    #  #    #         #       #    #  #    #  #####   ######  #    #
!   #   #   #    #  #    #         #       #    #  #    #  #   #   #    #  #    #
!   #    #  #    #  #####  ####### ######  #    #  #####   #    #  #    #  ##### _module
!
!   Radiation-Hydrodynamics Code: global long-characteristics radiation-transport module
!
!**************************************************************************************************
!   Fortran90
!   2019-02-04
!**************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Copenhagen, Montpellier, Lyon, Uppsala
!   Hans-Guenter Ludwig:   Kiel
!**************************************************************************************************
! MODULES:
!   rhd_lhdrad_module:     Long-characteristics radiative transfer routines (global models)
!
!**************************************************************************************************


!------*****************---------------------------------------------------------------------------
module rhd_lhdrad_module
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_lhdrad_module ('rhd_lhdradiation_module')
!
! PURPOSE:
!   Radiative transfer routines for rhd.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   use rhd_lhdrad_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_rad3d_SwitchInfo:  Print information about used compiler switches
!   rhd_rad3d_raylhd:      Solve the radiative transfer equation for one ray by Feautrier scheme
!   rhd_rad3d_smooth:      Smooth 3D array
!   rhd_rad3d_projectonplane: Project point along given direction on plane
!   rhd_rad3d_projectonplane1: Project point along given direction on plane
!   rhd_rad3d_projectarea_sort: Sort real vector y in ascending order
!   rhd_rad3d_projectarea_llw: Compute length of interval [xp0,xp1] that lies within [xr0,xr1]
!   rhd_rad3d_projectarea: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_projectarea1: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!   rhd_rad3d_BaseAxisSystem: Generate random or unity basic orthonormal axis system
!   rhd_rad3d_RayDirections: Create list of basic direction vectors
!   rhd_rad3d_toray:       Interpolate rho, P, and T onto ray
!   rhd_rad3d_fromray:     Put fluxes and energy change from ray back to hydro grid
!   rhd_rad3d_solveeq:     Perform radiative energy transfer for single ray on equidistant grid
!   rhd_rad3d_solve:       Perform radiative energy transfer step for single ray
!   rhd_rad3d_solveadapt:  Perform radiative energy transfer for single ray on refined grid
!   rhd_rad3d_dir:         Compute radiative energy flux for one direction field
!   rhd_rad3d_dirper:      Compute radiative energy flux for one direction field, periodic bnds.
!   rhd_rad3d_diffdeep:    Compute fluxes with diffusion approximation in deeper layers
!   rhd_rad3d_diffthick:   Compute fluxes with diffusion approximation in optically thick part
!   rhd_rad3d_Tvis:        Compute additional "viscous" radiative fluxes in optically thin p.
!   rhd_rad3d_formal:      Compute formal solution: 3D radiative energy flux
!   rhd_rad3d_lambda:      Do Lambda* iteration
!   rhd_rad3d_StepOutput:  Derive some averages as additional output quantities
!   rhd_rad3d_Step:        Perform 3d radiative energy transfer step
!
! MODIFICATION HISTORY:
!   1999-04-27 (B.F. Copenhagen) Written
!   1999-12-29 (B.F.) Working version with inclined rays
!   2000-02-25 (B.F.) Version for periodic boundary conditions
!   2000-11-16 (B.F. Uppsala)
!   2000-12-25 (B.F. Kiel) .f90 => .F90
!   2001-02-10 (B.F. Uppsala) Restructuring: splitting into more routines
!   2001-02-20 (B.F.) Several optimization steps finished
!   2001-03-09 (B.F.) OpenMP directives included
!   2001-03-22 (B.F.) "radiative viscosity" in optically thin region added
!   2001-10-17 (B.F.) New version of rhd_rad3d_raylhd: A -> A-1
!--------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************--------------------------------------------------------------------
subroutine rhd_rad3d_SwitchInfo(ncp)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_SwitchInfo ('rhd_radiation_3_dimensions_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_SwitchInfo(ncp)
!
! INPUT:
!   ncp:   ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-08-12 (B.F.) Written
!--------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!--------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_lhdrad_module ......................'
!
!#
# ifdef rhd_r01_new
!#
write(ncp,'(A,A)' ) '  rhd' // '_r01_new:             ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_r01_new:             ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_rad3d_r02
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_r02:           ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_r02:           ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_rad3d_toray_l01
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_toray_l01:     ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_toray_l01:     ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_rad3d_fromray_l01
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_fromray_l01:   ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_fromray_l01:   ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_rad3d_solve_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_solve_t01:     ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_solve_t01:     ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_rad3d_dir_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_dir_t01:       ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_dir_t01:       ', undefstr
!#
# endif
!#
!
!#
# ifdef rhd_rad3d_step_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_step_t01:      ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_rad3d_step_t01:      ', undefstr
!#
# endif
!#
!
end subroutine rhd_rad3d_SwitchInfo


!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_raylhdold(MU, WFR, NCELL, &
                            ADTAU, DSDTAU, IBDTOP, VIMMST, IBDBOT, VIPMSB, &
                            FR, diffFR)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_raylhd ('rhd_radiation_3_dimension_ray_lagrange_hydrodynamics')
!
! PURPOSE:
!   Solve the radiative transfer equation for single ray adopting the Feautrier scheme.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   call RHD_RAD3D_RAYLHD(MU, WFR, NCELL, ADTAU, DSDTAU, IBDTOP, VIMMST, &
!                         IBDBOT, VIPMSB, &
!                         FR, diffFR)
!
! INPUT:
!   MU:     ('mu') real, cosine of inclination angle
!   WFR:    ('weight_flux_radiative') real, weight factor for flux integration
!   NCELL:  ('number_cell') integer, number of grid points:
!            ADTAU(NCELL), DSDTAU(0:NCELL)!
!   ADTAU:  ('absolute_delta_tau') 1D real, delta taus along the ray or
!           if MU<>0 in vertical direction.
!           Take care! A peculiarity:
!           ADTAU(K) := ABS(TAU(K)-TAU(K-1)), so that ADTAU > 0
!   DSDTAU: ('delta_source_delta_tau) 1D real, derivative of the
!           source function with respect to tau,
!           DSDTAU(0:NCELL), if MU<>0 derivative in vertical direction
!   IBDTOP: ('identifier_boundary_top') integer, identifier of adopted
!           boundary condition at top:
!             = 0 : (I(-) - S)(NCELL) is specified in VIMMST
!             = 1 : v(NCELL) is specified in VIMMST
!             else: v(NCELL)=v(NCELL-1)
!   VIMMST: ('v_intensity_minus_minus_source_top') real, top boundary value, see IBDTOP
!   IBDBOT: ('identifier_boundary_bottom') integer, identifier of adopted
!           boundary condition at bottom:
!             = 0 : (I(+) - S)(0) is specified in VIPMSB
!             = 1 : v(0) is specified in VIPMSB
!             else: v(0)=v(1)
!   VIPMSB: ('v_intensity_plus_minus_source_bottom') real, bottom boundary value, see IBDBOT
!
! INPUT/OUTPUT:
!   FR:     ('flux_radiative') 1D real, (integrated) flux v, FR(0:NCELL)
!   diffFR: ('difference_flux_radiative') 1D real, "div" flux, diffFR(1:NCELL)
!
! OUTPUT:
!
! LOCAL VARIABLES:
!   A:      ('A') 1D real, work array, A(NCELL)
!   B:      ('B') real, coefficient B of tridiagonal matrix
!   C:      ('C') real, coefficient C of tridiagonal matrix
!   W:      ('work') 1D real, work array, W(NCELL)
!   OTK:    ('oneover_tau_k') real
!   OTKM05: ('oneover_tau_k_minus_05') real
!   OTKP05: ('oneover_tau_k_plus_05') real
!   ZWOMU:  ('zwo_mu') real, 2*mu
!   DUM:    ('dummy') real, abbreviation: 2*OTKM05 or 2*OTKP05
!   FACD1:  ('factor_d1') real, 1 / (B_n - A_n+1 * C_n)
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   No common blocks, no file access
!
! RESTRICTIONS:
!   The radiative flux FR is updated only. It has to be set to zero properly at the start.
!
! PROCEDURE:
!   RHD_RAD3D_RAYLHD solves the radiative transfer equation adopting the
!   Feautrier scheme. The equation is solved for the flux-like variable v.
!   The following convention numbering indices is employed:
!
!
!   "bottom"                                                "top"
!   ---|----...----------|---------|-----------|------...-----|--> Geom. Scale
!      0                k-1        k          k+1           NCELL
!
!   <--|-----...---------|----v----|-----v-----|------...-----|--- Tau Scale
!                             k         k+1
!
!   The routine can also handle inclined rays in plan-parallel layers
!   according to an inclination angle given by MU. It is done by stretching
!   delta tau and d S/d tau accordingly. Boundary conditions are not
!   affected!!! Here the user has to set up things appropriately.
!   Forward elimination starts at k=NCELL that is at the "top" if one is
!   regarding a stellar atmosphere.
!   The result is returned in FR. The routine is performing a flux
!   integration controlled by the integration weights WFR in consecutive
!   calls. Initialize FR to zero if you just want v.
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1995-02-01 (HGL) Written: Wed Feb  1 08:56:55 CET 1995
!   1999-04-27 (B.F. Copenhagen) Fortran90 version
!   1999-08-17 (HGL, B.F.) Bug in lower boundary condition (0) removed
!   2000-02-24 (B.F.) Boundary handling: (IBDBOT == 1) : V = VIPMSB  -->  V = MU*VIPMSB
!   2000-03-27 (B.F.) rk -> selected_real_kind(10)
!   2001-10-18 (B.F. Uppsala) diffFR added
!--------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: rk=selected_real_kind(10)
integer,                  intent(in)    :: IBDBOT, IBDTOP, NCELL
real(kind=rk),            intent(in)    :: MU, WFR, VIMMST, VIPMSB
real(kind=rk), dimension(1:ncell), &
                          intent(in)    :: ADTAU
real(kind=rk), dimension(0:ncell), &
                          intent(in)    :: DSDTAU
real(kind=rk), dimension(0:ncell), &
                          intent(inout) :: FR
real(kind=rk), dimension(1:ncell), &
                          intent(inout) :: diffFR
!
! --- Local variables ---
real(kind=rk), dimension(1:ncell)       :: A, W
integer                                 :: K, KP1
real(kind=rk)                           :: B, C, DUM, FACD1, OTK, OTKM05, OTKP05, V, ZWOMU
!--------------------------------------------------------------------------------------------------
!
! --- Due to the convention of grid indices the forward-elimination  ---
! --- starts at the last cell edge which is the upper boundary here. ---
! --- This section is written in a very concise way.                 ---
! --- Be sure what yoy do if you change anything!                    ---
!
ZWOMU  = MU + MU
!
! --- Upper boundary, K = NCELL ---
! --- Specify A(K), W(K)        ---
OTKM05 = MU / ADTAU(NCELL)
if (IBDTOP == 0) then
  ! --- I(-) - S specified ---
  !
  DUM = OTKM05 + OTKM05
  ! --- A = 2.0_rk * OTKM05 * OTKM05
  ! --- B = 1.0_rk + 2.0_rk * OTKM05 * ( 1.0_rk + OTKM05)
  ! --- W = MU * DSDTAU(NCELL) - 2.0_rk * OTKM05 * VIMMST
  ! --- FACD1 = 1.0_rk / B
  FACD1 = 1.0_rk / (1.0_rk + DUM * ( 1.0_rk + OTKM05))
  ! --- A(NCELL) = FACD1 * A
  A(NCELL) = FACD1 * DUM * OTKM05
  ! ---   W(NCELL) = FACD1 * W
  W(NCELL) = FACD1 * (MU * DSDTAU(NCELL) -  DUM * VIMMST)
else if (IBDTOP == 1) then
  ! --- Flux specified ---
  !
  A(NCELL) = 0.0_rk
  W(NCELL) = VIMMST
else
  ! --- A  = 1.0_rk
  ! --- B  = 1.0_rk
  ! --- W  = 0.0_rk
  A(NCELL) = 1.0_rk
  W(NCELL) = 0.0_rk
endif
!
!!$write(6,'(A,I3,4E13.6)') 'K, FACD1, A, B, W', NCELL, FACD1, A(NCELL), 1.0_rk/FACD1, W(NCELL)
!
! --- Forward-elimination: Inner cell edges ---
do K = NCELL-1, 1, -1
  KP1    = K + 1
  ! --- Current OTKP05 is last OTKM05 since loop works from higher to lower index ---
  OTKP05 = OTKM05
  OTK    = ZWOMU / ( ADTAU(K) + ADTAU(KP1))
  OTKM05 = MU / ADTAU(K)
  ! --- A and W are not computed explicidly but replaced in formulas (1) and (2) instead ---
  ! ---   A      = OTKM05 * OTK
  B      = 1.0_rk + 2.0_rk * OTKP05 * OTKM05
  C      = OTKP05 * OTK
  ! ---   W      = MU * DSDTAU(K)
  FACD1  = 1.0_rk / (B - A(KP1) * C)
  ! ---   A(K)   = FACD1 * A                              (1)
  A(K)   = FACD1 * OTKM05 * OTK
  ! ---   W(K)   = FACD1 * ( W + W(K+1) * C)              (2)
  W(K)   = FACD1 * ( MU * DSDTAU(K) + W(KP1) * C)
  !!$write(6,'(A,I3,4E13.6)') 'K, FACD1, A, B, W', K, FACD1, A(K), B, W(K)
end do ! K
!
! --- Lower boundary, K = 0 ---
if (IBDBOT == 0) then
  ! --- I(+) - S specified ---
  !
  OTKP05= OTKM05
  DUM   = OTKP05 +  OTKP05
  ! ---   A     = 0.0_rk
  ! ---   B     = 1.0_rk + 2.0_rk * OTKP05 * ( 1.0_rk + OTKP05)
  B     = 1.0_rk + DUM * ( 1.0_rk + OTKP05)
  ! ---   C     = 2.0_rk * OTKP05 * OTKP05
  C     = DUM * OTKP05
  ! ---   W     = MU * DSDTAU(0) + 2.0_rk * OTKP05 * VIPMSB
         FACD1 = 1.0_rk / (B - A(1) * C)
  ! ---   W(K)  = FACD1 * ( W + W(K+1) * C)
  ! ---   V     = W(K)
          V     = FACD1 * ( MU * DSDTAU(0) + DUM * VIPMSB + W(1) * C)
else if (IBDBOT == 1) then
  ! --- Flux specified ---
  !
!!$  V     = VIPMSB
  V     = MU*VIPMSB
else
  ! ---   A     = 0.0_rk
  ! ---   B     = 1.0_rk
  ! ---   C     = 1.0_rk
  ! ---   W     = 0.0_rk
  V     = W(1) / ( 1.0_rk - A(1))
endif
!
! --- Back-substitution ---
! --- Perform simultaneously flux integration ... ---
FR(0) = FR(0) + WFR * V
do K = 1, NCELL
  V         = W(K) + A(K) * V
  FR(K)     = FR(K) + WFR * V
  diffFR(K) = FR(K) - FR(K-1)
  !!$write(6,'(A,I3,4E13.6)') 'K, A, W, V, FR', K, A(K), W(K), V, FR(K)
end do ! K
!
end subroutine rhd_rad3d_raylhdold


!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_raylhd(MU, WFR, NCELL, &
                            ADTAU, DSDTAU, IBDTOP, VIMMST, IBDBOT, VIPMSB, &
                            FR, diffFR)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_raylhd ('rhd_radiation_3_dimension_ray_lagrange_hydrodynamics')
!
! PURPOSE:
!   Solve the radiative transfer equation for single ray adopting the Feautrier scheme.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   call RHD_RAD3D_RAYLHD(MU, WFR, NCELL, ADTAU, DSDTAU, IBDTOP, VIMMST, &
!                         IBDBOT, VIPMSB, &
!                         FR, diffFR)
!
! INPUT:
!   MU:     ('mu') real, cosine of inclination angle
!   WFR:    ('weight_flux_radiative') real, weight factor for flux integration
!   NCELL:  ('number_cell') integer, number of grid points:
!            ADTAU(NCELL), DSDTAU(0:NCELL)!
!   ADTAU:  ('absolute_delta_tau') 1D real, delta taus along the ray or
!           if MU<>0 in vertical direction.
!           Take care! A peculiarity:
!           ADTAU(K) := ABS(TAU(K)-TAU(K-1)), so that ADTAU > 0
!   DSDTAU: ('delta_source_delta_tau) 1D real, derivative of the
!           source function with respect to tau,
!           DSDTAU(0:NCELL), if MU<>0 derivative in vertical direction
!   IBDTOP: ('identifier_boundary_top') integer, identifier of adopted
!           boundary condition at top:
!             = 0 : (I(-) - S)(NCELL) is specified in VIMMST
!             = 1 : v(NCELL) is specified in VIMMST
!             else: v(NCELL)=v(NCELL-1)
!   VIMMST: ('v_intensity_minus_minus_source_top') real, top boundary value, see IBDTOP
!   IBDBOT: ('identifier_boundary_bottom') integer, identifier of adopted
!           boundary condition at bottom:
!             = 0 : (I(+) - S)(0) is specified in VIPMSB
!             = 1 : v(0) is specified in VIPMSB
!             else: v(0)=v(1)
!   VIPMSB: ('v_intensity_plus_minus_source_bottom') real, bottom boundary value, see IBDBOT
!
! INPUT/OUTPUT:
!   FR:     ('flux_radiative') 1D real, (integrated) flux v, FR(0:NCELL)
!   diffFR: ('difference_flux_radiative') 1D real, "div" flux, diffFR(1:NCELL)
!
! OUTPUT:
!
! LOCAL VARIABLES:
!   Ab:     ('A_bar') 1D real, work array, Ab(NCELL)
!   W:      ('work') 1D real, work array, W(NCELL)
!   TKM05:  ('tau_k_minus_05') real
!   TKP05:  ('tau_k_plus_05') real
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   No common blocks, no file access
!
! RESTRICTIONS:
!   The radiative flux FR is updated only. It has to be set to zero properly at the start.
!
! PROCEDURE:
!   RHD_RAD3D_RAYLHD solves the radiative transfer equation adopting the
!   Feautrier scheme. The equation is solved for the flux-like variable v.
!   The following convention numbering indices is employed:
!
!
!   "bottom"                                                "top"
!   ---|----...----------|---------|-----------|------...-----|--> Geom. Scale
!      0                k-1        k          k+1           NCELL
!
!   <--|-----...---------|----v----|-----v-----|------...-----|--- Tau Scale
!                             k         k+1
!
!   The routine can also handle inclined rays in plan-parallel layers
!   according to an inclination angle given by MU. It is done by stretching
!   delta tau and d S/d tau accordingly. Boundary conditions are not
!   affected!!! Here the user has to set up things appropriately.
!   Forward elimination starts at k=NCELL that is at the "top" if one is
!   regarding a stellar atmosphere.
!   The result is returned in FR. The routine is performing a flux
!   integration controlled by the integration weights WFR in consecutive
!   calls. Initialize FR to zero if you just want v.
!
!   The original coefficient A is replaced by Ab=A-1 to allow for very small
!   optical depths.
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1995-02-01 (HGL) Written: Wed Feb  1 08:56:55 CET 1995
!   1999-04-27 (B.F. Copenhagen) Fortran90 version
!   1999-08-17 (HGL, B.F.) Bug in lower boundary condition (0) removed
!   2000-02-24 (B.F.) Boundary handling: (IBDBOT == 1) : V = VIPMSB  -->  V = MU*VIPMSB
!   2000-03-27 (B.F.) rk -> selected_real_kind(10)
!   2001-10-17 (B.F. Uppsala) A -> A-1 =: Ab
!   2001-10-18 (B.F. Uppsala) diffFR added
!--------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: rk=selected_real_kind(10)
integer,                  intent(in)    :: IBDBOT, IBDTOP, NCELL
real(kind=rk),            intent(in)    :: MU, WFR, VIMMST, VIPMSB
real(kind=rk), dimension(1:ncell), &
                          intent(in)    :: ADTAU
real(kind=rk), dimension(0:ncell), &
                          intent(in)    :: DSDTAU
real(kind=rk), dimension(0:ncell), &
                          intent(inout) :: FR
real(kind=rk), dimension(1:ncell), &
                          intent(inout) :: diffFR
!
! --- Local variables ---
!!$real(kind=rk), dimension(1:ncell)       :: A, W
!!$integer                                 :: K, KP1
!!$real(kind=rk)                           :: B, C, DUM, FACD1, OTK, OTKM05, OTKP05, V, ZWOMU
real(kind=rk), dimension(1:ncell)       :: Ab, W
integer                                 :: K
real(kind=rk)                           :: TKM05, TKP05, V
!--------------------------------------------------------------------------------------------------
!
! --- Due to the convention of grid indices the forward-elimination  ---
! --- starts at the last cell edge which is the upper boundary here. ---
! --- This section is written in a very concise way.                 ---
! --- Be sure what yoy do if you change anything!                    ---
!
!ZWOMU  = MU + MU
!
! --- Upper boundary, K = NCELL ---
! --- Specify A(K), W(K)        ---
!!$OTKM05 = MU / ADTAU(NCELL)
TKM05 = ADTAU(NCELL) / MU
!
if (IBDTOP == 0) then
  ! --- I(-) - S specified ---
  !
  ! --- A = 2.0_rk * OTKM05 * OTKM05
  ! --- B = 1.0_rk + 2.0_rk * OTKM05 * ( 1.0_rk + OTKM05)
  ! --- W = MU * DSDTAU(NCELL) - 2.0_rk * OTKM05 * VIMMST
  ! --- FACD1 = 1.0_rk / B
  ! --- A(NCELL) = FACD1 * A
  ! ---   W(NCELL) = FACD1 * W
  !
!!$  DUM = OTKM05 + OTKM05
!!$  FACD1 = 1.0_rk / (1.0_rk + DUM * ( 1.0_rk + OTKM05))
!!$  A(NCELL) = FACD1 * DUM * OTKM05
!!$  W(NCELL) = FACD1 * (MU * DSDTAU(NCELL) -  DUM * VIMMST)
  !
  Ab(NCELL)= - ( (TKM05 + 2.0_rk) * TKM05         ) / &
               ( (TKM05 + 2.0_rk) * TKM05 + 2.0_rk) 
  W(NCELL) = ( TKM05 * (TKM05 * MU * DSDTAU(NCELL) - 2.0_rk * VIMMST) ) / &
             ( (TKM05 + 2.0_rk) * TKM05 + 2.0_rk                      )
  !
else if (IBDTOP == 1) then
  ! --- Flux specified ---
  !
  Ab(NCELL) =-1.0_rk
  W( NCELL) = VIMMST
else
  ! --- A  = 1.0_rk
  ! --- B  = 1.0_rk
  ! --- W  = 0.0_rk
  Ab(NCELL) = 0.0_rk
  W( NCELL) = 0.0_rk
endif
!
!!$write(6,'(A,I3,2E13.6)') 'K, Ab, W', NCELL, Ab(NCELL), W(NCELL)
!
! --- Forward-elimination: Inner cell edges ---
do K = NCELL-1, 1, -1
  ! --- Current OTKP05 is last OTKM05 since loop works from higher to lower index ---
!!$  OTKP05 = OTKM05
!!$  OTK    = ZWOMU / ( ADTAU(K) + ADTAU(KP1))
!!$  OTKM05 = MU / ADTAU(K)
  TKP05 = TKM05
  TKM05 = ADTAU(K) / MU
  !
  ! --- A and W are not computed explicitly but replaced in formulas (1) and (2) instead ---
  ! ---   A      = OTKM05 * OTK
  ! ---   W      = MU * DSDTAU(K)
  ! ---   A(K)   = FACD1 * A                              (1)
  ! ---   W(K)   = FACD1 * ( W + W(K+1) * C)              (2)
  !
!!$  B      = 1.0_rk + 2.0_rk * OTKP05 * OTKM05
!!$  C      = OTKP05 * OTK
!!$  FACD1  = 1.0_rk / (B - A(KP1) * C)
!!$  A(K)   = FACD1 * OTKM05 * OTK
!!$  W(K)   = FACD1 * ( MU * DSDTAU(K) + W(KP1) * C)
  !
  Ab(K) = (  Ab(K+1) - 0.5_rk * TKP05 * (TKP05 + TKM05) ) / &
          ( -Ab(K+1) + 0.5_rk * TKP05 * (TKP05 + TKM05) + TKP05/TKM05 )
  W(K)  = ( 0.5_rk* TKP05 * (TKP05 + TKM05) * MU * DSDTAU(K) + W(K+1) ) / &
          ( - Ab(K+1) + 0.5_rk * TKP05 * (TKP05 + TKM05) + TKP05/TKM05     )
  !
  !!$write(6,'(A,I3,2E13.6)') 'K, Ab, W', K, Ab(K), W(K)
end do ! K
!
! --- Lower boundary, K = 0 ---
if (IBDBOT == 0) then
  ! --- I(+) - S specified ---
  !
!!$  OTKP05= OTKM05
!!$  DUM   = OTKP05 +  OTKP05
  TKP05= TKM05
  ! ---   A     = 0.0_rk
  ! ---   B     = 1.0_rk + 2.0_rk * OTKP05 * ( 1.0_rk + OTKP05)
  ! ---   C     = 2.0_rk * OTKP05 * OTKP05
  ! ---   W     = MU * DSDTAU(0) + 2.0_rk * OTKP05 * VIPMSB
  ! ---   W(K)  = FACD1 * ( W + W(K+1) * C)
  ! ---   V     = W(K)
  !
!!$  B     = 1.0_rk + DUM * ( 1.0_rk + OTKP05)
!!$  C     = DUM * OTKP05
!!$  FACD1 = 1.0_rk / (B - A(1) * C)
!!$  V     = FACD1 * ( MU * DSDTAU(0) + DUM * VIPMSB + W(1) * C)
  !
  V = ( 0.5_rk * TKP05 * TKP05 * MU * DSDTAU(0) + TKP05 * VIPMSB + W(1) ) / &
      ( - Ab(1) + (0.5_rk * TKP05 + 1.0_rk) * TKP05                   )
else if (IBDBOT == 1) then
  ! --- Flux specified ---
  !
!!$  V     = VIPMSB
  V     = MU*VIPMSB
else
  ! ---   A     = 0.0_rk
  ! ---   B     = 1.0_rk
  ! ---   C     = 1.0_rk
  ! ---   W     = 0.0_rk
  !
!!$  V     = W(1) / ( 1.0_rk - A(1))
  !
  V     = W(1) / (-Ab(1))
endif
!
! --- Back-substitution ---
! --- Perform simultaneously flux integration ... ---
FR(0) = FR(0) + WFR * V
do K = 1, NCELL
  diffFR(K) = diffFR(K) + WFR * (W(K) + Ab(K) * V)
  V         = W(K) + (Ab(K) + 1.0_rk) * V
  FR(K)     = FR(K) + WFR * V
  !!$write(6,'(A,I3,5E13.6)') 'K, Ab, W, V, FR, diffFR(K)', K, Ab(K), W(K), V, FR(K), diffFR(K)
end do ! K
!
end subroutine rhd_rad3d_raylhd


!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_raylhdout(MU, WFR, NCELL, &
                            ADTAU, DSDTAU, IBDTOP, VIMMST, IBDBOT, VIPMSB, &
                            FR)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_raylhd ('rhd_radiation_3_dimension_ray_lagrange_hydrodynamics')
!
! PURPOSE:
!   Solve the radiative transfer equation for single ray adopting the Feautrier scheme.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   call RHD_RAD3D_RAYLHD(MU, WFR, NCELL, ADTAU, DSDTAU, IBDTOP, VIMMST, &
!                         IBDBOT, VIPMSB, &
!                         FR)
!
! INPUT:
!   MU:     ('mu') real, cosine of inclination angle
!   WFR:    ('weight_flux_radiative') real, weight factor for flux integration
!   NCELL:  ('number_cell') integer, number of grid points:
!            ADTAU(NCELL), DSDTAU(0:NCELL)!
!   ADTAU:  ('absolute_delta_tau') 1D real, delta taus along the ray or
!           if MU<>0 in vertical direction.
!           Take care! A peculiarity:
!           ADTAU(K) := ABS(TAU(K)-TAU(K-1)), so that ADTAU > 0
!   DSDTAU: ('delta_source_delta_tau) 1D real, derivative of the
!           source function with respect to tau,
!           DSDTAU(0:NCELL), if MU<>0 derivative in vertical direction
!   IBDTOP: ('identifier_boundary_top') integer, identifier of adopted
!           boundary condition at top:
!             = 0 : (I(-) - S)(NCELL) is specified in VIMMST
!             = 1 : v(NCELL) is specified in VIMMST
!             else: v(NCELL)=v(NCELL-1)
!   VIMMST: ('v_intensity_minus_minus_source_top') real, top boundary value, see IBDTOP
!   IBDBOT: ('identifier_boundary_bottom') integer, identifier of adopted
!           boundary condition at bottom:
!             = 0 : (I(+) - S)(0) is specified in VIPMSB
!             = 1 : v(0) is specified in VIPMSB
!             else: v(0)=v(1)
!   VIPMSB: ('v_intensity_plus_minus_source_bottom') real, bottom boundary value, see IBDBOT
!
! INPUT/OUTPUT:
!   FR:     ('flux_radiative') 1D real, (integrated) flux v, FR(0:NCELL)
!
! OUTPUT:
!
! LOCAL VARIABLES:
!   A:      ('A') 1D real, work array, A(NCELL)
!   B:      ('B') real, coefficient B of tridiagonal matrix
!   C:      ('C') real, coefficient C of tridiagonal matrix
!   W:      ('work') 1D real, work array, W(NCELL)
!   OTK:    ('oneover_tau_k') real
!   OTKM05: ('oneover_tau_k_minus_05') real
!   OTKP05: ('oneover_tau_k_plus_05') real
!   ZWOMU:  ('zwo_mu') real, 2*mu
!   DUM:    ('dummy') real, abbreviation: 2*OTKM05 or 2*OTKP05
!   FACD1:  ('factor_d1') real, 1 / (B_n - A_n+1 * C_n)
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   No common blocks, no file access
!
! RESTRICTIONS:
!   The radiative flux FR is updated only. It has to be set to zero properly at the start.
!
! PROCEDURE:
!   RHD_RAD3D_RAYLHD solves the radiative transfer equation adopting the
!   Feautrier scheme. The equation is solved for the flux-like variable v.
!   The following convention numbering indices is employed:
!
!
!   "bottom"                                                "top"
!   ---|----...----------|---------|-----------|------...-----|--> Geom. Scale
!      0                k-1        k          k+1           NCELL
!
!   <--|-----...---------|----v----|-----v-----|------...-----|--- Tau Scale
!                             k         k+1
!
!   The routine can also handle inclined rays in plan-parallel layers
!   according to an inclination angle given by MU. It is done by stretching
!   delta tau and d S/d tau accordingly. Boundary conditions are not
!   affected!!! Here the user has to set up things appropriately.
!   Forward elimination starts at k=NCELL that is at the "top" if one is
!   regarding a stellar atmosphere.
!   The result is returned in FR. The routine is performing a flux
!   integration controlled by the integration weights WFR in consecutive
!   calls. Initialize FR to zero if you just want v.
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1995-02-01 (HGL) Written: Wed Feb  1 08:56:55 CET 1995
!   1999-04-27 (B.F. Copenhagen) Fortran90 version
!   1999-08-17 (HGL, B.F.) Bug in lower boundary condition (0) removed
!   2000-02-24 (B.F.) Boundary handling: (IBDBOT == 1) :
!                   V = VIPMSB  -->  V = MU*VIPMSB
!   2000-03-27 (B.F.) rk -> selected_real_kind(10)
!--------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: rk=selected_real_kind(10)
integer,                  intent(in)    :: IBDBOT, IBDTOP, NCELL
real(kind=rk),            intent(in)    :: MU, WFR, VIMMST, VIPMSB
real(kind=rk), dimension(1:ncell), &
                          intent(in)    :: ADTAU
real(kind=rk), dimension(0:ncell), &
                          intent(in)    :: DSDTAU
real(kind=rk), dimension(0:ncell), &
                          intent(inout) :: FR
!
! --- Local variables ---
real(kind=rk), dimension(1:ncell)       :: A, W
integer                                 :: K, KP1
real(kind=rk)                           :: B, C, DUM, FACD1, OTK, OTKM05, OTKP05, V, ZWOMU
!--------------------------------------------------------------------------------------------------
!
! --- Due to the convention of grid indices the forward-elimination  ---
! --- starts at the last cell edge which is the upper boundary here. ---
! --- This section is written in a very concise way.                 ---
! --- Be sure what yoy do if you change anything!                    ---
!
ZWOMU  = MU + MU
!
! --- Upper boundary, K = NCELL ---
! --- Specify A(K), W(K)        ---
OTKM05 = MU / ADTAU(NCELL)
if (IBDTOP == 0) then
  ! --- I(-) - S specified ---
  !
  DUM = OTKM05 + OTKM05
  ! --- A = 2.0_rk * OTKM05 * OTKM05
  ! --- B = 1.0_rk + 2.0_rk * OTKM05 * ( 1.0_rk + OTKM05)
  ! --- W = MU * DSDTAU(NCELL) - 2.0_rk * OTKM05 * VIMMST
  ! --- FACD1 = 1.0_rk / B
  FACD1 = 1.0_rk / (1.0_rk + DUM * ( 1.0_rk + OTKM05))
  ! --- A(NCELL) = FACD1 * A
  A(NCELL) = FACD1 * DUM * OTKM05
  ! ---   W(NCELL) = FACD1 * W
  W(NCELL) = FACD1 * (MU * DSDTAU(NCELL) -  DUM * VIMMST)
else if (IBDTOP == 1) then
  ! --- Flux specified ---
  !
  A(NCELL) = 0.0_rk
  W(NCELL) = VIMMST
else
  ! --- A  = 1.0_rk
  ! --- B  = 1.0_rk
  ! --- W  = 0.0_rk
  A(NCELL) = 1.0_rk
  W(NCELL) = 0.0_rk
endif
!
write(6,'(A,I3,4E13.6)') 'K, FACD1, A, B, W', NCELL, FACD1, A(NCELL), 1.0_rk/FACD1, W(NCELL)
!
! --- Forward-elimination: Inner cell edges ---
do K = NCELL-1, 1, -1
  KP1    = K + 1
  ! --- Current OTKP05 is last OTKM05 since loop works from higher to lower index ---
  OTKP05 = OTKM05
  OTK    = ZWOMU / ( ADTAU(K) + ADTAU(KP1))
  OTKM05 = MU / ADTAU(K)
  ! --- A and W are not computed explicidly but replaced in formulas (1) and (2) instead ---
  ! ---   A      = OTKM05 * OTK
  B      = 1.0_rk + 2.0_rk * OTKP05 * OTKM05
  C      = OTKP05 * OTK
  ! ---   W      = MU * DSDTAU(K)
  FACD1  = 1.0_rk / (B - A(KP1) * C)
  ! ---   A(K)   = FACD1 * A                              (1)
  A(K)   = FACD1 * OTKM05 * OTK
  ! ---   W(K)   = FACD1 * ( W + W(K+1) * C)              (2)
  W(K)   = FACD1 * ( MU * DSDTAU(K) + W(KP1) * C)
  write(6,'(A,I3,4E13.6)') 'K, FACD1, A, B, W', K, FACD1, A(K), B, W(K)
end do ! K
!
! --- Lower boundary, K = 0 ---
if (IBDBOT == 0) then
  ! --- I(+) - S specified ---
  !
  OTKP05= OTKM05
  DUM   = OTKP05 +  OTKP05
  ! ---   A     = 0.0_rk
  ! ---   B     = 1.0_rk + 2.0_rk * OTKP05 * ( 1.0_rk + OTKP05)
  B     = 1.0_rk + DUM * ( 1.0_rk + OTKP05)
  ! ---   C     = 2.0_rk * OTKP05 * OTKP05
  C     = DUM * OTKP05
  ! ---   W     = MU * DSDTAU(0) + 2.0_rk * OTKP05 * VIPMSB
         FACD1 = 1.0_rk / (B - A(1) * C)
  ! ---   W(K)  = FACD1 * ( W + W(K+1) * C)
  ! ---   V     = W(K)
          V     = FACD1 * ( MU * DSDTAU(0) + DUM * VIPMSB + W(1) * C)
else if (IBDBOT == 1) then
  ! --- Flux specified ---
  !
!!$  V     = VIPMSB
  V     = MU*VIPMSB
else
  ! ---   A     = 0.0_rk
  ! ---   B     = 1.0_rk
  ! ---   C     = 1.0_rk
  ! ---   W     = 0.0_rk
  V     = W(1) / ( 1.0_rk - A(1))
endif
!
! --- Back-substitution ---
! --- Perform simultaneously flux integration ... ---
FR(0) = FR(0) + WFR * V
do K = 1, NCELL
  V     = W(K) + A(K) * V
  FR(K) = FR(K) + WFR * V
  write(6,'(A,I3,4E13.6)') 'K, A, W, V, FR', K, A(K), W(K), V, FR(K)
end do ! K
!
end subroutine rhd_rad3d_raylhdout


!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_smooth(amount, m1,n1, m2,n2, m3,n3, a, s)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_smooth ('rhd_radiation_3_dimension_smooth')
!
! PURPOSE:
!   Smooth 3D array.
!   
! CATEGORY:
!   Array processing
!
! CALLING SEQUENCE:
!   call rhd_rad3d_smooth(amount, m1,n1, m2,n2, m3,n3, a, s)
!
! INPUT:
!   a:      ('array') real 3D array
!
! OUTPUT:
!   s:      ('smooth') real 3D array
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-02-06 (B.F. Uppsala) Written
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                     intent(in)    :: amount
integer,                  intent(in)    :: m1,n1, m2,n2, m3,n3
real, dimension(m1:n1,m2:n2,m3:n3), &
                          intent(in)    :: a
real, dimension(m1:n1,m2:n2,m3:n3), &
                          intent(out)   :: s
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real, dimension(m1:n1,m2:n2,m3:n3)      :: b
!--------------------------------------------------------------------------------------------------
!
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1)
!$OMP DO SCHEDULE(STATIC)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      b(i1,i2,i3)=64.0*a(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP DO SCHEDULE(STATIC)
do i3=m3,n3
  if ((i3 > m3) .and. (i3 < n3)) then
    do i2=m2+1,n2-1
      do i1=m1+1,n1-1
        b(i1,i2,i3)=     a(i1-1,i2-1,i3-1) +  2.0*a(i1  ,i2-1,i3-1) +      a(i1+1,i2-1,i3-1) + &
                     2.0*a(i1-1,i2  ,i3-1) +  4.0*a(i1  ,i2  ,i3-1) +  2.0*a(i1+1,i2  ,i3-1) + &
                         a(i1-1,i2+1,i3-1) +  2.0*a(i1  ,i2+1,i3-1) +      a(i1+1,i2+1,i3-1) + &
                     2.0*a(i1-1,i2-1,i3  ) +  4.0*a(i1  ,i2-1,i3  ) +  2.0*a(i1+1,i2-1,i3  ) + &
                     4.0*a(i1-1,i2  ,i3  ) +  8.0*a(i1  ,i2  ,i3  ) +  4.0*a(i1+1,i2  ,i3  ) + &
                     2.0*a(i1-1,i2+1,i3  ) +  4.0*a(i1  ,i2+1,i3  ) +  2.0*a(i1+1,i2+1,i3  ) + &
                         a(i1-1,i2-1,i3+1) +  2.0*a(i1  ,i2-1,i3+1) +      a(i1+1,i2-1,i3+1) + &
                     2.0*a(i1-1,i2  ,i3+1) +  4.0*a(i1  ,i2  ,i3+1) +  2.0*a(i1+1,i2  ,i3+1) + &
                         a(i1-1,i2+1,i3+1) +  2.0*a(i1  ,i2+1,i3+1) +      a(i1+1,i2+1,i3+1)
      end do ! i1
    end do ! i2
  endif
end do ! i3
!$OMP END DO
!
!$OMP DO SCHEDULE(STATIC)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      s(i1,i2,i3)=(amount/64.0)*b(i1,i2,i3) + (1.0-amount)*a(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
!
end subroutine rhd_rad3d_smooth


!----------************************----------------------------------------------------------------
subroutine rhd_rad3d_projectonplane(xin, d, a, ipl, xout)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_projectonplane ('rhd_radiation_3_dimension_projection_on_plane')
!
! PURPOSE:
!   Project point along given direction on plane.
!   
! CATEGORY:
!   Vector algebra
!
! CALLING SEQUENCE:
!   call rhd_rad3d_projectonplane(xin, d, a, ipl, xout)
!
! INPUT:
!   xin:    ('x_input') real vector, input point to be projected
!   d:      ('direction') real vector, projection direction
!   a:      ('a') real, distance of plane from origin
!   ipl:    ('index_plane') integer, index of plane:
!           1: x23-plane
!           2: x13-plane
!           3: x23-plane
!
! OUTPUT:
!   xout:   ('x_output') real vector, projection of xin on plane(ipl,a) along d
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   d(ipl) /= 0.0: The direction of projection must not lay parallel to the plane
!                  on which to project.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-17 (B.F. Copenhagen) Written
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real, dimension(3),      intent(in)     :: xin, d
real,                    intent(in)     :: a
integer,                 intent(in)     :: ipl
real, dimension(3),      intent(out)    :: xout
!
! --- Local variables ---
!--------------------------------------------------------------------------------------------------
!
xout = xin + (a-xin(ipl))/d(ipl) * d
!
end subroutine rhd_rad3d_projectonplane


!----------************************----------------------------------------------------------------
subroutine rhd_rad3d_projectonplane1(xin, d, a, ipl, xout)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_projectonplane ('rhd_radiation_3_dimension_projection_on_plane')
!
! PURPOSE:
!   Project point along given direction on plane.
!   
! CATEGORY:
!   Vector algebra
!
! CALLING SEQUENCE:
!   call rhd_rad3d_projectonplane(xin, d, a, ipl, xout)
!
! INPUT:
!   xin:    ('x_input') real vector, input point to be projected
!   d:      ('direction') real vector, projection direction
!   a:      ('a') real, distance of plane from origin
!   ipl:    ('index_plane') integer, index of plane:
!           1: x23-plane
!           2: x13-plane
!           3: x23-plane
!
! OUTPUT:
!   xout:   ('x_output') real vector, projection of xin on plane(ipl,a) along d
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   d(ipl) /= 0.0: The direction of projection must not lay parallel to the plane
!                  on which to project.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-17 (B.F. Copenhagen) Written
!   2000-03-20 (B.F.) Vector version
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real, dimension(:,:),    intent(in)     :: xin
real, dimension(3),      intent(in)     :: d
real, dimension(:),      intent(in)     :: a
integer,                 intent(in)     :: ipl
real, dimension(:,:),    intent(out)    :: xout
!
! --- Local variables ---
!--------------------------------------------------------------------------------------------------
!
xout = xin + spread(a-xin(:,ipl), 2,3)/d(ipl) * spread(d, 1,size(a))
!
end subroutine rhd_rad3d_projectonplane1


!----------**************************--------------------------------------------------------------
subroutine rhd_rad3d_projectarea_sort(y)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_projectarea_sort ('rhd_radiation_3_dimension_project_area_sort')
!
! PURPOSE:
!   Sort real vector y in ascending order.
!
! CATEGORY:
!   Sorting
!
! CALLING SEQUENCE:
!   call rhd_rad3d_projectarea_sort(y)
!
! INPUT/OUTPUT:
!   y:      ('y') real vector to be sorted
!
! PROCEDURE:
!   Very simple: Search minimum element and swap...
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-23 (B.F. Copenhagen) Written
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),      intent(inout)  :: y
!
! --- Local variables ---
integer                                 :: n, i
integer, dimension(1)                   :: i_min
real                                    :: tmp
!--------------------------------------------------------------------------------------------------
!
n=size(y)
!
do i=1,n-1
  i_min=minloc(y(i:n))+i-1
  tmp        =y(i_min(1))
  y(i_min(1))=y(i)
  y(i)       =tmp
end do ! i
!
end subroutine rhd_rad3d_projectarea_sort


!----------*************************---------------------------------------------------------------
subroutine rhd_rad3d_projectarea_llw(xr0, xr1, xp0, xp1, length)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_projectarea_llw ('rhd_radiation_3_dimension_project_area_line_length_within')
!
! PURPOSE:
!   Compute length of interval [xp0,xp1] that lies within [xr0,xr1]
!
! CATEGORY:
!   Geometry
!
! CALLING SEQUENCE:
!   call rhd_rad3d_projectarea_llw(xr0_in, xr1_in, xp0_in, xp1_in, length)
!
! INPUTS:
!   xr0:    ('x_rectangle_0') real
!   xr1:    ('x_rectangle_1') real
!   xp0:    ('x_parallelogram_0') real
!   xp1:    ('x_parallelogram_1') real
!
! OUTPUTS:
!   length: ('length') length of interval (>=0)
!
! PROCEDURE:
!     xr0       xr1
!      +---------+
!          +-----------+
!         xp0         xp1
!   --->
!          |     |
!          +-----+ => length
!
! EXAMPLE:
!   call rhd_rad3d_projectarea_llw( 0.0, 1.0, 0.9, -0.3, length)
!     length = 0.900000
!
! MODIFICATION HISTORY:
!   1999-11-23 (B.F. Copenhagen) Fortran90 version
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                    intent(in)     :: xr0, xr1, xp0, xp1
real,                    intent(out)    :: length
!
! --- Local variables ---
!--------------------------------------------------------------------------------------------------
!
if (xr1 > xr0) then
  length=abs(max(xr0,min(xp1,xr1)) - max(xr0,min(xp0,xr1)))
else
  length=abs(max(xr1,min(xp0,xr0)) - max(xr1,min(xp1,xr0)))
endif
!
end subroutine rhd_rad3d_projectarea_llw


!----------*********************-------------------------------------------------------------------
subroutine rhd_rad3d_projectarea(xr00_in, yr00_in, xr10_in,          yr01_in, &
                                 xp00_in, yp00_in, xp10_in, xp01_in, yp01_in, &
                                 areafraction)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_projectarea ('rhd_radiation_3_dimension_project_area')
!
! PURPOSE:
!   Compute fraction of area of parallelogram that lies within rectangle.
!
! CATEGORY:
!   Geometry
!
! CALLING SEQUENCE:
!   call projectarea(xr00, yr00, xr10,       yr01, &
!                    xp00, yp00, xp10, xp01, yp01, &
!                    areafraction)
!
! INPUTS:
!   xr00:   ('x_rectangle_00') x coordinate of rectangle (see below)
!   yr00:   ('y_rectangle_00') y coordinate of rectangle (see below)
!   xr10:   ('x_rectangle_10') x coordinate of rectangle (see below)
!   yr01:   ('y_rectangle_01') y coordinate of rectangle (see below)
!   xp00:   ('x_parallelogram_00') x coordinate of parallelogram (see below)
!   yp00:   ('y_parallelogram_00') y coordinate of parallelogram (see below)
!   xp10:   ('x_parallelogram_10') x coordinate of parallelogram (see below)
!   xp01:   ('x_parallelogram_01') x coordinate of parallelogram (see below)
!   yp01:   ('y_parallelogram_01') y coordinate of parallelogram (see below)
!
! OUTPUTS:
!   areafraction: ('area_fraction') real (>=0.0), fraction of area of
!                 parallelogram (given by xp00, yp00, xp10, xp01, yp01)
!                 that lies within rectangle (given by xr00, yr00, xr10, yr01)
!
! KEYWORDS:
!   None
!
! COMMON BLOCKS:
!   None
!
! ROUTINES:
!   rhd_rad3d_projectarea_llw: Compute length of interval [xp0,xp1] that lies within [xr0,xr1]
!   rhd_rad3d_projectarea_sort: Sort real vector y in ascending order
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The rectangle is specified by 4 coordinates,
!   the parallelogram is specified by 5 coordinates.
!
!              xp01,yp01
!                  +-------+
!    yr01         /       /
!      +--------------+  /
!      |        /%%%%%| /
!      |       /%%%%%%|/
!      |      /%%%%%%%|
!      |     /%%%%%%%/|
!      |    +-------+ |
!      |xp00,yp00 xp10|
!      |              |
!      +--------------+
!  xr00,yr00        xr10
!
! EXAMPLE:
!   projectarea(0.0, -1.0, 1.0, 0.0,  -1.2, -0.9, 0.8, 1.4,  0.1,  af) 
!   projectarea(0.0, -1.0, 1.0, 0.0,   0.2, -0.9, 0.9, 0.4,  0.2,  af)
!   projectarea(0.0, -1.0, 1.0, 0.0,   0.2, -0.9, 0.8, 0.3, -0.1,  af)  -> 1.0
!   projectarea(0.0, -1.0, 1.0, 0.0,   1.2, -0.9, 1.8, 1.3, -0.1,  af)  -> 0.0
!   projectarea(0.0, -1.0, 1.0, 0.0,  -0.5, -0.9, 0.5, 0.5, -0.1,  af)  -> 0.75
!
! MODIFICATION HISTORY:
!   1999-11-19 (B.F. Copenhagen) Written
!   1999-11-22 (B.F.) Some modifications
!   1999-11-23 (B.F.) Fortran90 version
!   1999-12-02 (B.F.) Check if xp?? or yp?? are in wrong order
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                    intent(in)     :: xr00_in, yr00_in, xr10_in,          yr01_in, &
                                           xp00_in, yp00_in, xp10_in, xp01_in, yp01_in
real,                    intent(out)    :: areafraction
!
! --- Local variables ---
real                                    :: xr00, yr00, xr10,       yr01, &
                                           xp00, yp00, xp10, xp01, yp01, &
                                           width, dxoverdy, areatotal, area, xtmp, &
                                           xsect0, xsect1, &
                                           lengthnew, lengthold
real, dimension(6)                      :: ysect
integer                                 :: i, i_ysect
!--------------------------------------------------------------------------------------------------
!
! --- Sort coordinates ---
if (xr10_in > xr00_in) then
  xr00=xr00_in
  xr10=xr10_in
else
  xr00=xr10_in
  xr10=xr00_in
endif
if (yr01_in > yr00_in) then
  yr00=yr00_in
  yr01=yr01_in
else
  yr00=yr01_in
  yr01=yr00_in
endif
if (xp10_in > xp00_in) then
  xp00=xp00_in
  xp10=xp10_in
  xp01=xp01_in
else
  xp00=xp10_in
  xp10=xp00_in
  xp01=xp01_in+xp10_in-xp00_in
endif
! --- Width of parallelogram ---
width=xp10-xp00
if (yp01_in > yp00_in) then
  yp00=yp00_in
  yp01=yp01_in
else
  yp00=yp01_in
  yp01=yp00_in
  xtmp=xp00
  xp00=xp01
  xp01=xtmp
  xp10=xp00+width
endif
!
! --- Slope, area of parallelogram ---
dxoverdy =(xp01-xp00)/(yp01-yp00)
areatotal= width     *(yp01-yp00)
!
! --- Chop of (possibly) protruding region at top and bottom ---
yp01=max(yr00,min(yp01,yr01))
xp01=xp00+dxoverdy*(yp01-yp00)
yp00=max(yr00,min(yp00,yr01))
xp00=xp01+dxoverdy*(yp00-yp01)
xp10=xp00+width
!
! --- Split area into simple regions ---
i_ysect       =1
ysect(i_ysect)=yp00
i_ysect       =i_ysect+1
if (abs(dxoverdy) >= 1.0E-05) then
  ! --- Search all intersections within parallelogram (max. 4) ---
  !
  ysect(i_ysect)=yp00+(xr00-xp00)/dxoverdy
  if ((ysect(i_ysect) > yp00) .and. (ysect(i_ysect) < yp01)) i_ysect=i_ysect+1
  ysect(i_ysect)=yp00+(xr10-xp00)/dxoverdy
  if ((ysect(i_ysect) > yp00) .and. (ysect(i_ysect) < yp01)) i_ysect=i_ysect+1
  ysect(i_ysect)=yp00+(xr00-xp10)/dxoverdy
  if ((ysect(i_ysect) > yp00) .and. (ysect(i_ysect) < yp01)) i_ysect=i_ysect+1
  ysect(i_ysect)=yp00+(xr10-xp10)/dxoverdy
  if ((ysect(i_ysect) > yp00) .and. (ysect(i_ysect) < yp01)) i_ysect=i_ysect+1
  !
  if (i_ysect > 3) then
    ! --- Sort ---
    call rhd_rad3d_projectarea_sort(ysect(2:i_ysect-1))
  endif
endif
ysect(i_ysect)=yp01
!
! --- Loop over all subregions ---
area=0.0
do i=1,i_ysect
  xsect0=xp00+ (ysect(i)-yp00)*dxoverdy
  xsect1=xp10+ (ysect(i)-yp00)*dxoverdy
  call rhd_rad3d_projectarea_llw(xr00, xr10, xsect0, xsect1, lengthnew)
  if (i > 1) then
    area=area + (ysect(i)-ysect(i-1)) * 0.5 * (lengthnew+lengthold)
  endif
  lengthold=lengthnew
end do
!
areafraction=area/areatotal
!
end subroutine rhd_rad3d_projectarea


!----------**********************------------------------------------------------------------------
subroutine rhd_rad3d_projectarea1(xr00_in, yr00_in, xr10_in,          yr01_in, &
                                 xp00_in, yp00_in, xp10_in, xp01_in, yp01_in, &
                                 areafraction)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_projectarea ('rhd_radiation_3_dimension_project_area')
!
! PURPOSE:
!   Compute fraction of area of parallelogram that lies within rectangle.
!
! CATEGORY:
!   Geometry
!
! CALLING SEQUENCE:
!   call projectarea(xr00, yr00, xr10,       yr01, &
!                    xp00, yp00, xp10, xp01, yp01, &
!                    areafraction)
!
! INPUTS:
!   xr00:   ('x_rectangle_00') x coordinate of rectangle (see below)
!   yr00:   ('y_rectangle_00') y coordinate of rectangle (see below)
!   xr10:   ('x_rectangle_10') x coordinate of rectangle (see below)
!   yr01:   ('y_rectangle_01') y coordinate of rectangle (see below)
!   xp00:   ('x_parallelogram_00') x coordinate of parallelogram (see below)
!   yp00:   ('y_parallelogram_00') y coordinate of parallelogram (see below)
!   xp10:   ('x_parallelogram_10') x coordinate of parallelogram (see below)
!   xp01:   ('x_parallelogram_01') x coordinate of parallelogram (see below)
!   yp01:   ('y_parallelogram_01') y coordinate of parallelogram (see below)
!
! OUTPUTS:
!   areafraction: ('area_fraction') real (>=0.0), fraction of area of
!                 parallelogram (given by xp00, yp00, xp10, xp01, yp01)
!                 that lies within rectangle (given by xr00, yr00, xr10, yr01)
!
! KEYWORDS:
!   None
!
! COMMON BLOCKS:
!   None
!
! ROUTINES:
!   rhd_rad3d_projectarea_sort: Sort real vector y in ascending order
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The rectangle is specified by 4 coordinates,
!   the parallelogram is specified by 5 coordinates.
!
!              xp01,yp01
!                  +-------+
!    yr01         /       /
!      +--------------+  /
!      |        /%%%%%| /
!      |       /%%%%%%|/
!      |      /%%%%%%%|
!      |     /%%%%%%%/|
!      |    +-------+ |
!      |xp00,yp00 xp10|
!      |              |
!      +--------------+
!  xr00,yr00        xr10
!
! EXAMPLE:
!   projectarea(0.0, -1.0, 1.0, 0.0,  -1.2, -0.9, 0.8, 1.4,  0.1,  af) 
!   projectarea(0.0, -1.0, 1.0, 0.0,   0.2, -0.9, 0.9, 0.4,  0.2,  af)
!   projectarea(0.0, -1.0, 1.0, 0.0,   0.2, -0.9, 0.8, 0.3, -0.1,  af)  -> 1.0
!   projectarea(0.0, -1.0, 1.0, 0.0,   1.2, -0.9, 1.8, 1.3, -0.1,  af)  -> 0.0
!   projectarea(0.0, -1.0, 1.0, 0.0,  -0.5, -0.9, 0.5, 0.5, -0.1,  af)  -> 0.75
!
! MODIFICATION HISTORY:
!   1999-11-19 (B.F. Copenhagen) Written
!   1999-11-22 (B.F.) Some modifications
!   1999-11-23 (B.F.) Fortran90 version
!   1999-12-02 (B.F.) Check if xp?? or yp?? are in wrong order
!   2000-03-18 (B.F.) "Inlining" of rhd_rad3d_projectarea_llw
!   2000-03-19 (B.F.) Handle whole vectors
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real, dimension(:),      intent(in)     :: xr00_in, yr00_in, xr10_in,          yr01_in, &
                                           xp00_in, yp00_in, xp10_in, xp01_in, yp01_in
real, dimension(:),      intent(out)    :: areafraction
!
! --- Local variables ---
real, dimension(size(xp00_in))          :: xr00, yr00, xr10,       yr01, &
                                           xp00, yp00, xp10, xp01, yp01, &
                                           width, dxoverdy, areatotal, area, &
                                           xsect0, xsect1, &
                                           xtmp, &
                                           lengthnew, lengthold
real, dimension(size(xp00_in),6)        :: ysect
logical, dimension(size(xp00_in))       :: imask, xrmask
integer, dimension(size(xp00_in))       :: i_ysect
integer                                 :: i
!--------------------------------------------------------------------------------------------------
!
! --- Sort coordinates ---
if (xr10_in(1) > xr00_in(1)) then
  xr00=xr00_in
  xr10=xr10_in
else
  xr00=xr10_in
  xr10=xr00_in
endif
if (yr01_in(1) > yr00_in(1)) then
  yr00=yr00_in
  yr01=yr01_in
else
  yr00=yr01_in
  yr01=yr00_in
endif
if (xp10_in(1) > xp00_in(1)) then
  xp00=xp00_in
  xp10=xp10_in
  xp01=xp01_in
else
  xp00=xp10_in
  xp10=xp00_in
  xp01=xp01_in+xp10_in-xp00_in
endif
! --- Width of parallelogram ---
width=xp10-xp00
if (yp01_in(1) > yp00_in(1)) then
  yp00=yp00_in
  yp01=yp01_in
else
  yp00=yp01_in
  yp01=yp00_in
  xtmp=xp00
  xp00=xp01
  xp01=xtmp
  xp10=xp00+width
endif
!
! --- Slope, area of parallelogram ---
dxoverdy =(xp01-xp00)/(yp01-yp00)
areatotal= width     *(yp01-yp00)
!
! --- Chop of (possibly) protruding region at top and bottom ---
yp01=max(yr00,min(yp01,yr01))
xp01=xp00+dxoverdy*(yp01-yp00)
yp00=max(yr00,min(yp00,yr01))
xp00=xp01+dxoverdy*(yp00-yp01)
xp10=xp00+width
!
! --- Split area into simple regions ---
i_ysect   =1
ysect(:,1)=yp00
i_ysect   =i_ysect+1
if (abs(dxoverdy(1)) >= 1.0E-05) then
  ! --- Search all intersections within parallelogram (max. 4) ---
  !
  do i=1,size(xp00_in)
    ysect(i,i_ysect(i))=yp00(i)+(xr00(i)-xp00(i))/dxoverdy(i)
    if ((ysect(i,i_ysect(i)) > yp00(i)) .and. (ysect(i,i_ysect(i)) < yp01(i))) then
      i_ysect(i)=i_ysect(i)+1
    endif
  end do ! i
  do i=1,size(xp00_in)
    ysect(i,i_ysect)=yp00(i)+(xr10(i)-xp00(i))/dxoverdy(i)
    if ((ysect(i,i_ysect(i)) > yp00(i)) .and. (ysect(i,i_ysect(i)) < yp01(i))) then
      i_ysect(i)=i_ysect(i)+1
    endif
  end do ! i
  do i=1,size(xp00_in)
    ysect(i,i_ysect(i))=yp00(i)+(xr00(i)-xp10(i))/dxoverdy(i)
    if ((ysect(i,i_ysect(i)) > yp00(i)) .and. (ysect(i,i_ysect(i)) < yp01(i))) then
      i_ysect(i)=i_ysect(i)+1
    endif
  end do ! i
  do i=1,size(xp00_in)
    ysect(i,i_ysect(i))=yp00(i)+(xr10(i)-xp10(i))/dxoverdy(i)
    if ((ysect(i,i_ysect(i)) > yp00(i)) .and. (ysect(i,i_ysect(i)) < yp01(i))) then
      i_ysect(i)=i_ysect(i)+1
    endif
  end do ! i
  !
  ! --- Sort ---
  do i=1,size(xp00_in)
    if (i_ysect(i) > 3) then
      call rhd_rad3d_projectarea_sort(ysect(i,2:i_ysect(i)-1))
    endif
  end do ! i
endif
do i=1,size(xp00_in)
  ysect(i,i_ysect(i))=yp01(i)
end do ! i
!
! --- Loop over all subregions ---
area=0.0

!!$do i=1,maxval(i_ysect)
!!$  where (i_ysect <= i)
!!$    xsect0=xp00+ (ysect(:,i)-yp00)*dxoverdy
!!$    xsect1=xp10+ (ysect(:,i)-yp00)*dxoverdy
!!$    where (xr10 > xr00)
!!$      lengthnew=abs(max(xr00,min(xsect1,xr10)) - max(xr00,min(xsect0,xr10)))
!!$    else where
!!$      lengthnew=abs(max(xr10,min(xsect0,xr00)) - max(xr10,min(xsect1,xr00)))
!!$    end where
!!$    if (i > 1) then
!!$      area=area + (ysect(:,i)-ysect(:,i-1)) * 0.5 * (lengthnew+lengthold)
!!$    endif
!!$    lengthold=lengthnew
!!$  end where
!!$end do

!
xrmask=(xr10 > xr00)
do i=1,maxval(i_ysect)
  imask=(i <= i_ysect)
  where (imask)
    xsect0=xp00+ (ysect(:,i)-yp00)*dxoverdy
    xsect1=xp10+ (ysect(:,i)-yp00)*dxoverdy
  end where
  where (imask .and. xrmask)
    lengthnew=abs(max(xr00,min(xsect1,xr10)) - max(xr00,min(xsect0,xr10)))
  end where
  where (imask .and. .not. xrmask)
    lengthnew=abs(max(xr10,min(xsect0,xr00)) - max(xr10,min(xsect1,xr00)))
  end where
  if (i > 1) then
    where (imask)
      area=area + (ysect(:,i)-ysect(:,i-1)) * 0.5 * (lengthnew+lengthold)
    end where
  endif
  where (imask)
    lengthold=lengthnew
  end where
end do
!
areafraction=area/areatotal
!
end subroutine rhd_rad3d_projectarea1


subroutine rhd_rad3d_RotationMatrix(theta, phi, psi, M)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_RotationMatrix ('rhd_radiation_3_dimension_Rotation_Matrix')
!
! PURPOSE:
!   Create matrix for rotation in 3D from 3 angles.
!   
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_rad3d_RotationMatrix(theta, phi, psi, M)
!
! INPUT:
!   theta:
!   phi:
!   psi:
!
! OUTPUT:
!   M:      ('matrix')
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   See Herbert Goldstein, "Klassische Mechanik", p. 121
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Copenhagen) Written
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                    intent(in)     :: theta, phi, psi
real, dimension(3,3),    intent(out)    :: M
!
! --- Local variables ---
!--------------------------------------------------------------------------------------------------
!
M(1,1)= cos(phi)*cos(psi) - cos(theta)*sin(phi)*sin(psi)
M(1,2)= sin(phi)*cos(psi) + cos(theta)*cos(phi)*sin(psi)
M(1,3)=                     sin(theta)*         sin(psi)
M(2,1)=-cos(phi)*sin(psi) - cos(theta)*sin(phi)*cos(psi)
M(2,2)=-sin(phi)*sin(psi) + cos(theta)*cos(phi)*cos(psi)
M(2,3)=                     sin(theta)*         cos(psi)
M(3,1)=                     sin(theta)*sin(phi)
M(3,2)=                    -sin(theta)*cos(phi)
M(3,3)=                     cos(theta)
!
end subroutine rhd_rad3d_RotationMatrix


!----------************************----------------------------------------------------------------
subroutine rhd_rad3d_BaseAxisSystem(radraybase, M_axis) 
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_BaseAxisSystem ('rhd_radiation_3_dimension_Base_Axis_System')
!
! PURPOSE:
!   Generate random or unity basic orthonormal axis system.
!   
! CATEGORY:
!   Vector algebra
!
! CALLING SEQUENCE:
!   call rhd_rad3d_BaseAxisSystem(radraybase, M_axis)
!
! INPUT:
!   radraybase: ('radiation_ray_base') character
!
! OUTPUT:
!   M_axis:     ('Matrix_axis') 3x3 real array, specifying the main axes of the ray system
!
! ROUTINES:
!   rhd_rad3d_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Random number generator has to be initialized.
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!   given theta, phi, psi
!   save theta, phi, psi
!   given delta-(theta, phi, psi)
!
! MODIFICATION HISTORY:
!   1999-11-22 (B.F. Copenhagen) Put into subroutine
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!--------------------------------------------------------------------------------------------------
!use const_module, only: pi_srk => pi
use const_module, only: pi
!
implicit none
!
! --- I/O parameters ---
character*(*),           intent(in)     :: radraybase
real, dimension(3,3),    intent(out)    :: M_axis
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
real (kind=srk)                         :: theta, phi, psi
!--------------------------------------------------------------------------------------------------
!
! --- Create base axis system ---
select case (radraybase)
  case('unity')
    ! --- Unity matrix ---
    M_axis     =0.0_srk
    M_axis(1,1)=1.0_srk
    M_axis(2,2)=1.0_srk
    M_axis(3,3)=1.0_srk
  case('random', 'randomgroup')
    ! --- Matrix with random orientation (but equally distributed over all spatial angles) ---
    call random_number(theta)
    call random_number(phi)
    call random_number(psi)
    theta=acos(2.0_srk*theta-1.0_srk)
    phi  =2.0_srk*pi*phi
    psi  =2.0_srk*pi*psi
    call rhd_rad3d_RotationMatrix(theta, phi, psi, M_axis)
  case default
    ! --- Unity matrix ---
    M_axis     =0.0_srk
    M_axis(1,1)=1.0_srk
    M_axis(2,2)=1.0_srk
    M_axis(3,3)=1.0_srk
end select
!
end subroutine rhd_rad3d_BaseAxisSystem


!----------***********************-----------------------------------------------------------------
subroutine rhd_rad3d_RayDirections(radraystar, x)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_RayDirections ('rhd_radiation_3_dimension_Ray_Directions')
!
! PURPOSE:
!   Create list of basic direction vectors.
!   
! CATEGORY:
!   Matrix algebra
!
! CALLING SEQUENCE:
!   call rhd_rad3d_RayDirections(radraystar, x)
!
! INPUT:
!
! OUTPUT:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The array x is allocated. It has to be deallocated properly by the user.
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Copenhagen) Written
!   1999-11-22 (B.F.) Some more ray systems
!   1999-12-22 (B.F.) oktaeder210, 410
!   2000-11-24 (B.F. Uppsala) oktaeder110
!   2000-11-27 (B.F.) oktaederXXX -> oktaeder-XXX
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
character*(*),           intent(in)     :: radraystar
real, dimension(:,:),    pointer        :: x
!
! --- Local variables ---
integer                                 :: n_v, i
real                                    :: gold
!--------------------------------------------------------------------------------------------------
!
select case (radraystar)
  case ('x1')
    ! --- Only rays along x1 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0    ; x(3,1) = 0.0
  case ('x2')
    ! --- Only rays along x2 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0    ; x(3,1) = 0.0
  case ('x3')
    ! --- Only rays along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v))
    x(1,1) = 0.0            ; x(2,1) = 0.0    ; x(3,1) = 1.0
  case ('x3-012')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v))
    x(1,1) = 0.0            ; x(2,1) = 1.0    ; x(3,1) = 2.0
  case ('x3-124')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v))
    x(1,1) = 1.0            ; x(2,1) = 2.0    ; x(3,1) = 4.0
  case ('x3-011')
    ! --- Only rays mainly along x3 axis (1 direction) ---
    n_v=1
    allocate(x(3,n_v))
    x(1,1) = 0.0            ; x(2,1) = 0.999999    ; x(3,1) = 1.000001
  case ('oktaeder')
    ! --- Oktaeder: rays along x1, x2, and x3 axis (3 directions) ---
    n_v=3
    allocate(x(3,n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0    ; x(3,1) = 0.0
    x(1,2) = 0.0            ; x(2,2) = 1.0    ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0    ; x(3,3) = 1.0
  case ('oktaeder-110')
    ! --- Oktaeder ---
    n_v=3
    allocate(x(3,n_v))
    x(1,1) = 1.0            ; x(2,1) =-1.0    ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 1.0    ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0    ; x(3,3) = 1.0
  case ('oktaeder-210')
    ! --- Oktaeder ---
    n_v=3
    allocate(x(3,n_v))
    x(1,1) = 2.0            ; x(2,1) =-1.0    ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 2.0    ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0    ; x(3,3) = 1.0
  case ('oktaeder-410')
    ! --- Oktaeder ---
    n_v=3
    allocate(x(3,n_v))
    x(1,1) = 4.0            ; x(2,1) =-1.0    ; x(3,1) = 0.0
    x(1,2) = 1.0            ; x(2,2) = 4.0    ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0    ; x(3,3) = 1.0
  case ('tetraeder')
    ! --- Tetraeder: rays in 4 directions ---
    n_v=4
    allocate(x(3,n_v))
    x(1,1) = sqrt(2.0)      ; x(2,1) = 1.0        ; x(3,1) = sqrt(6.0)
    x(1,2) =-2.0*sqrt(2.0)  ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = sqrt(2.0)      ; x(2,3) = 1.0        ; x(3,3) =-sqrt(6.0)
    x(1,4) = 0.0            ; x(2,4) =-3.0        ; x(3,4) = 0.0
  case ('cube')
    ! --- Cube: rays in 4 directions ---
    n_v=4
    allocate(x(3,n_v))
    x(1,1) = 1.0            ; x(2,1) = 1.0        ; x(3,1) = 1.0
    x(1,2) =-1.0            ; x(2,2) = 1.0        ; x(3,2) = 1.0
    x(1,3) = 1.0            ; x(2,3) =-1.0        ; x(3,3) = 1.0
    x(1,4) =-1.0            ; x(2,4) =-1.0        ; x(3,4) = 1.0
  case ('ikosaeder')
    ! --- Ikosaeder: rays in 6 directions ---
    n_v=6
    allocate(x(3,n_v))
    gold = (1.0+sqrt(5.0))/2.0
    x(1,1)  = 0.0           ; x(2,1)  = gold      ; x(3,1)  = 1.0
    x(1,2)  = 0.0           ; x(2,2)  = gold      ; x(3,2)  =-1.0
    x(1,3)  = 1.0           ; x(2,3)  = 0.0       ; x(3,3)  = gold 
    x(1,4)  = 1.0           ; x(2,4)  = 0.0       ; x(3,4)  =-gold
    x(1,5)  = gold          ; x(2,5)  = 1.0       ; x(3,5)  = 0.0
    x(1,6)  = gold          ; x(2,6)  =-1.0       ; x(3,6)  = 0.0
  case ('dodekaeder')
    ! --- Dodekaeder: rays in 10 directions ---
    n_v=10
    allocate(x(3,n_v))
    gold = (1.0+sqrt(5.0))/2.0
    x(1,1)  = 0.0           ; x(2,1)  = 1.0/gold  ; x(3,1)  = gold
    x(1,2)  = 0.0           ; x(2,2)  = 1.0/gold  ; x(3,2)  =-gold
    x(1,3)  = gold          ; x(2,3)  = 0.0       ; x(3,3)  = 1.0/gold 
    x(1,4)  = gold          ; x(2,4)  = 0.0       ; x(3,4)  =-1.0/gold
    x(1,5)  = 1.0/gold      ; x(2,5)  = gold      ; x(3,5)  = 0.0
    x(1,6)  = 1.0/gold      ; x(2,6)  =-gold      ; x(3,6)  = 0.0
    x(1,7)  = 1.0           ; x(2,7)  = 1.0       ; x(3,7)  = 1.0 
    x(1,8)  = 1.0           ; x(2,8)  = 1.0       ; x(3,8)  =-1.0
    x(1,9)  = 1.0           ; x(2,9)  =-1.0       ; x(3,9)  = 1.0
    x(1,10) = 1.0           ; x(2,10) =-1.0       ; x(3,10) =-1.0
  case default
    n_v=3
    allocate(x(3,n_v))
    x(1,1) = 1.0            ; x(2,1) = 0.0        ; x(3,1) = 0.0
    x(1,2) = 0.0            ; x(2,2) = 1.0        ; x(3,2) = 0.0
    x(1,3) = 0.0            ; x(2,3) = 0.0        ; x(3,3) = 1.0
end select
!
! --- Normalize vectors ---
do i=1,n_v
  x(:,i)=x(:,i)/sqrt(sum(x(:,i)**2))
end do ! i
!
end subroutine rhd_rad3d_RayDirections


!----------***************-------------------------------------------------------------------------
subroutine rhd_rad3d_toray(action, &
                           m1,n1, m2,n2, m3,n3, &
                           m1org,n1org, m2org,n2org, m3org,n3org, &
                           m3rad, n3rad, &
                           ic1_0_ray, ic2_0_ray, ic1_1_ray, ic2_1_ray, &
                           wc_00_ray0, wc_01_ray0, wc_10_ray0, wc_11_ray0, &
                           rho, T4, logP, &
                           rho_ray, T4_ray, logP_ray)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_toray ('rhd_radiation_3_dimension_to_ray')
!
! PURPOSE:
!   Interpolate rho, logP, and T^4 onto ray.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!
! INPUT/OUTPUT:
!
! OUTPUT:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-19 (B.F.) Put into own routine
!--------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),          intent(in)  :: action
integer,                    intent(in)  :: m1,n1, m2,n2, m3,n3, &
                                           m1org,n1org, m2org,n2org, m3org,n3org, &
                                           m3rad, n3rad
integer, dimension(0:n3-m3), &
                            intent(in)  :: ic1_0_ray, ic2_0_ray, ic1_1_ray, ic2_1_ray
real,    dimension(m3   :n3   ), &
                            intent(in)  :: wc_00_ray0, wc_01_ray0, wc_10_ray0, wc_11_ray0
real,    dimension(m1org:n1org, &
                   m2org:n2org, &
                   m3org:n3org ), &
                            intent(in)  :: rho, T4, logP
real,    dimension(0:n3-m3), &
                            intent(out) :: rho_ray, T4_ray, logP_ray
!
! --- Local variables ---
integer                                 :: i3, i3o
real                                    :: T4_minimum
logical                                 :: T4inter_bilinear_flag
!--------------------------------------------------------------------------------------------------
!
T4inter_bilinear_flag=(par%C_radTintminfac <= 1.0)
!
! --- Interpolate rho, logP, and T^4 on ray ---
select case (action%dim)
  case(1)
    do i3=m3rad,n3rad
      i3o=i3+m3
      rho_ray(i3)=wc_00_ray0(i3o) * rho(i3o, ic1_0_ray(i3), ic2_0_ray(i3)) + &
                  wc_10_ray0(i3o) * rho(i3o, ic1_1_ray(i3), ic2_0_ray(i3)) + &
                  wc_01_ray0(i3o) * rho(i3o, ic1_0_ray(i3), ic2_1_ray(i3)) + &
                  wc_11_ray0(i3o) * rho(i3o, ic1_1_ray(i3), ic2_1_ray(i3))
!#
# ifdef rhd_rad3d_toray_l01
!#
    end do ! i3
    do i3=m3rad,n3rad
      i3o=i3+m3
!#
# endif
!#
      logP_ray(i3)=wc_00_ray0(i3o) * logP(  i3o, ic1_0_ray(i3), ic2_0_ray(i3)) + &
                   wc_10_ray0(i3o) * logP(  i3o, ic1_1_ray(i3), ic2_0_ray(i3)) + &
                   wc_01_ray0(i3o) * logP(  i3o, ic1_0_ray(i3), ic2_1_ray(i3)) + &
                   wc_11_ray0(i3o) * logP(  i3o, ic1_1_ray(i3), ic2_1_ray(i3))
!#
# ifdef rhd_rad3d_toray_l01
!#
    end do ! i3
    do i3=m3rad,n3rad
      i3o=i3+m3
!#
# endif
!#
      T4_ray(i3)=wc_00_ray0(i3o) * T4(i3o, ic1_0_ray(i3), ic2_0_ray(i3)) + &
                 wc_10_ray0(i3o) * T4(i3o, ic1_1_ray(i3), ic2_0_ray(i3)) + &
                 wc_01_ray0(i3o) * T4(i3o, ic1_0_ray(i3), ic2_1_ray(i3)) + &
                 wc_11_ray0(i3o) * T4(i3o, ic1_1_ray(i3), ic2_1_ray(i3))
    end do ! i3
    if (.not. T4inter_bilinear_flag) then
      do i3=m3rad,n3rad
        i3o=i3+m3
        T4_minimum=min(T4(i3o, ic1_0_ray(i3), ic2_0_ray(i3)), &
                       T4(i3o, ic1_1_ray(i3), ic2_0_ray(i3)), &
                       T4(i3o, ic1_0_ray(i3), ic2_1_ray(i3)), &
                       T4(i3o, ic1_1_ray(i3), ic2_1_ray(i3)) )
        if (T4_ray(i3) > (par%C_radTintminfac*T4_minimum)) then
          T4_ray(i3)=(T4_ray(i3)-par%C_radTintminfac*T4_minimum)/ &
                    (1.0+(T4_ray(i3)/T4_minimum-par%C_radTintminfac)/ &
                              (2.0*(par%C_radTintminfac-1.0))) + &
                       par%C_radTintminfac*T4_minimum
        endif
      end do ! i3
    endif
  case(2)
    do i3=m3rad,n3rad
      i3o=i3+m3
      rho_ray(i3)=wc_00_ray0(i3o) * rho(ic2_0_ray(i3), i3o, ic1_0_ray(i3)) + &
                  wc_10_ray0(i3o) * rho(ic2_0_ray(i3), i3o, ic1_1_ray(i3)) + &
                  wc_01_ray0(i3o) * rho(ic2_1_ray(i3), i3o, ic1_0_ray(i3)) + &
                  wc_11_ray0(i3o) * rho(ic2_1_ray(i3), i3o, ic1_1_ray(i3))
!#
# ifdef rhd_rad3d_toray_l01
!#
    end do ! i3
    do i3=m3rad,n3rad
      i3o=i3+m3
!#
# endif
!#
      logP_ray(i3)=wc_00_ray0(i3o) * logP(  ic2_0_ray(i3), i3o, ic1_0_ray(i3)) + &
                   wc_10_ray0(i3o) * logP(  ic2_0_ray(i3), i3o, ic1_1_ray(i3)) + &
                   wc_01_ray0(i3o) * logP(  ic2_1_ray(i3), i3o, ic1_0_ray(i3)) + &
                   wc_11_ray0(i3o) * logP(  ic2_1_ray(i3), i3o, ic1_1_ray(i3))
!#
# ifdef rhd_rad3d_toray_l01
!#
    end do ! i3
    do i3=m3rad,n3rad
      i3o=i3+m3
!#
# endif
!#
      T4_ray(i3)=wc_00_ray0(i3o) * T4(ic2_0_ray(i3), i3o, ic1_0_ray(i3)) + &
                 wc_10_ray0(i3o) * T4(ic2_0_ray(i3), i3o, ic1_1_ray(i3)) + &
                 wc_01_ray0(i3o) * T4(ic2_1_ray(i3), i3o, ic1_0_ray(i3)) + &
                 wc_11_ray0(i3o) * T4(ic2_1_ray(i3), i3o, ic1_1_ray(i3))
    end do ! i3
    if (.not. T4inter_bilinear_flag) then
      do i3=m3rad,n3rad
        i3o=i3+m3
        T4_minimum=min(T4(ic2_0_ray(i3), i3o, ic1_0_ray(i3)), &
                       T4(ic2_0_ray(i3), i3o, ic1_1_ray(i3)), &
                       T4(ic2_1_ray(i3), i3o, ic1_0_ray(i3)), &
                       T4(ic2_1_ray(i3), i3o, ic1_1_ray(i3)) )
        if (T4_ray(i3) > (par%C_radTintminfac*T4_minimum)) then
          T4_ray(i3)=(T4_ray(i3)-par%C_radTintminfac*T4_minimum)/ &
                         (1.0+(T4_ray(i3)/T4_minimum-par%C_radTintminfac)/ &
                              (2.0*(par%C_radTintminfac-1.0))) + &
                       par%C_radTintminfac*T4_minimum
        endif
      end do ! i3
    endif
  case(3)
    do i3=m3rad,n3rad
      i3o=i3+m3
      rho_ray(i3)=wc_00_ray0(i3o) * rho(ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
                  wc_10_ray0(i3o) * rho(ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
                  wc_01_ray0(i3o) * rho(ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
                  wc_11_ray0(i3o) * rho(ic1_1_ray(i3), ic2_1_ray(i3), i3o)
!#
# ifdef rhd_rad3d_toray_l01
!#
    end do ! i3
    do i3=m3rad,n3rad
      i3o=i3+m3
!#
# endif
!#
      logP_ray(i3)=wc_00_ray0(i3o) * logP(  ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
                   wc_10_ray0(i3o) * logP(  ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
                   wc_01_ray0(i3o) * logP(  ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
                   wc_11_ray0(i3o) * logP(  ic1_1_ray(i3), ic2_1_ray(i3), i3o)
!#
# ifdef rhd_rad3d_toray_l01
!#
    end do ! i3
    do i3=m3rad,n3rad
      i3o=i3+m3
!#
# endif
!#
      T4_ray(i3)=wc_00_ray0(i3o) * T4(ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
                 wc_10_ray0(i3o) * T4(ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
                 wc_01_ray0(i3o) * T4(ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
                 wc_11_ray0(i3o) * T4(ic1_1_ray(i3), ic2_1_ray(i3), i3o)
    end do ! i3
    if (.not. T4inter_bilinear_flag) then
      do i3=m3rad,n3rad
        i3o=i3+m3
        T4_minimum=min(T4(ic1_0_ray(i3), ic2_0_ray(i3), i3o), &
                       T4(ic1_1_ray(i3), ic2_0_ray(i3), i3o), &
                       T4(ic1_0_ray(i3), ic2_1_ray(i3), i3o), &
                       T4(ic1_1_ray(i3), ic2_1_ray(i3), i3o) )
        if (T4_ray(i3) > (par%C_radTintminfac*T4_minimum)) then
          T4_ray(i3)=(T4_ray(i3)-par%C_radTintminfac*T4_minimum)/ &
                         (1.0+(T4_ray(i3)/T4_minimum-par%C_radTintminfac)/ &
                              (2.0*(par%C_radTintminfac-1.0))) + &
                       par%C_radTintminfac*T4_minimum
        endif
      end do ! i3
    endif
end select
!
end subroutine rhd_rad3d_toray


!----------*****************-----------------------------------------------------------------------
subroutine rhd_rad3d_fromray(action, flux_flag, &
                             m1,n1, m2,n2, m3,n3, &
                             m1org,n1org, m2org,n2org, m3org,n3org, &
                             m3rad, n3rad, &
                             dx3, Fradb, diffFradb, &
                             i1_shift, i2_shift, ib1_0_ray, ib2_0_ray, if21_0_ray, if12_0_ray, &
                             fluxfraction10, fluxfraction20, fluxfraction30, sumfluxfraction, &
                             emission, &
                             Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_fromray ('rhd_radiation_3_dimension_from_ray')
!
! PURPOSE:
!   Put fluxes and energy change from ray back to hydro grid.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_fromray(action, flux_flag, &
!                          m1,n1, m2,n2, m3,n3, &
!                          m1org,n1org, m2org,n2org, m3org,n3org, &
!                          m3rad, n3rad, &
!                          dx3, Fradb, diffFradb, &
!                          i1_shift, i2_shift, ib1_0_ray, ib2_0_ray, if21_0_ray, if12_0_ray, &
!                          fluxfraction10, fluxfraction20, fluxfraction30, sumfluxfraction, &
!                          emission, &
!                          Frad1, Frad2, Frad3, drhoeidtime)
!
! INPUT:
!
! INPUT/OUTPUT:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-20 (B.F.) Put into own routine
!   2001-10-18 (B.F.) diffFradb added
!--------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
type(action_type),          intent(in)  :: action
logical,                    intent(in)  :: flux_flag
integer,                    intent(in)  :: m1,n1, m2,n2, m3,n3, &
                                           m1org,n1org, m2org,n2org, m3org,n3org, &
                                           m3rad, n3rad, i1_shift, i2_shift
real (kind=srk),            intent(in)  :: dx3
real (kind=drk), dimension(0:n3-m3+1), &
                            intent(in)  :: Fradb
real (kind=drk), dimension(0:n3-m3), &
                            intent(in)  :: diffFradb
integer, dimension(0:n3-m3+1), &
                            intent(in)  :: ib1_0_ray, ib2_0_ray
integer, dimension(0:n3-m3  ), &
                            intent(in)  :: if21_0_ray, if12_0_ray
real (kind=srk), dimension(m3:n3  ,2,3), &
                            intent(in)  :: fluxfraction10
real (kind=srk), dimension(m3:n3  ,3,2), &
                            intent(in)  :: fluxfraction20
real (kind=srk), dimension(m3:n3+1,2,2), &
                            intent(in)  :: fluxfraction30
real (kind=srk), dimension(m3:n3  ,3,3), &
                            intent(in)  :: sumfluxfraction
real, dimension(m1org-3:n1org+3, &
                m2org-3:n2org+3, &
                m3org-3:n3org+3), &
                          intent(in)    :: emission
real, dimension(m1org-3:n1org+3+1, &
                m2org-3:n2org+3  , &
                m3org-3:n3org+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1org-3:n1org+3  , &
                m2org-3:n2org+3+1, &
                m3org-3:n3org+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1org-3:n1org+3  , &
                m2org-3:n2org+3  , &
                m3org-3:n3org+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1org-3:n1org+3, &
                m2org-3:n2org+3, &
                m3org-3:n3org+3), &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
integer                                 :: i1_0, i1_1, i1_2, i2_0, i2_1, i2_2, i3, i3o
real (kind=srk), dimension(0:n3-m3)     :: Fradc_ray, divFrad_ray
real (kind=srk), dimension(0:n3-m3+1)   :: Fradb_ray
real (kind=srk)                         :: avgemission, invavgemission
!--------------------------------------------------------------------------------------------------
!
Fradb_ray(m3rad:n3rad+1)=real(Fradb(m3rad:n3rad+1), kind=srk)
!
! --- Flux divergence ---
!!$divFrad_ray(m3rad:n3rad)=-real(Fradb(m3rad+1:n3rad+1) - &
!!$                               Fradb(m3rad  :n3rad  ) , kind=srk)/dx3
divFrad_ray(m3rad:n3rad)=-real(diffFradb(m3rad:n3rad), kind=srk)/dx3
!
! --- Flux inside cell ---
Fradc_ray(m3rad:n3rad)=0.5*(Fradb_ray(m3rad  :n3rad  ) + &
                            Fradb_ray(m3rad+1:n3rad+1) )
! 
! --- Project radiative flux along ray onto HD grid ---
select case (action%dim)
  case(1)
    ! --- Old x1 direction is main (new x3) direction ---
    !
    if (flux_flag) then
      ! --- Flux: x3 direction ---
      do i3=m3rad,n3rad+1
        i3o=i3+m3
        !
        i1_0=ib1_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i2_0=ib2_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        !
        Frad1(  i3o, i1_0, i2_0) = &
          Frad1(i3o, i1_0, i2_0) + fluxfraction30(i3o,1,1)*Fradb_ray(i3)
        Frad1(  i3o, i1_1, i2_0) = &
          Frad1(i3o, i1_1, i2_0) + fluxfraction30(i3o,2,1)*Fradb_ray(i3)
        Frad1(  i3o, i1_0, i2_1) = &
          Frad1(i3o, i1_0, i2_1) + fluxfraction30(i3o,1,2)*Fradb_ray(i3)
        Frad1(  i3o, i1_1, i2_1) = &
          Frad1(i3o, i1_1, i2_1) + fluxfraction30(i3o,2,2)*Fradb_ray(i3)
      end do ! i3
      ! --- Flux: x1 direction ---
      do i3=m3rad,n3rad
        i3o=i3+m3
        !
        i1_0=if21_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i1_2=i1_1+1
        i2_0=if12_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        i2_2=i2_1+1
        !
        Frad2(  i3o, i1_1, i2_0) = &
          Frad2(i3o, i1_1, i2_0) + fluxfraction10(i3o,1,1)*Fradc_ray(i3)
        Frad2(  i3o, i1_2, i2_0) = &
          Frad2(i3o, i1_2, i2_0) + fluxfraction10(i3o,2,1)*Fradc_ray(i3)
        Frad2(  i3o, i1_1, i2_1) = &
          Frad2(i3o, i1_1, i2_1) + fluxfraction10(i3o,1,2)*Fradc_ray(i3)
        Frad2(  i3o, i1_2, i2_1) = &
          Frad2(i3o, i1_2, i2_1) + fluxfraction10(i3o,2,2)*Fradc_ray(i3)
        Frad2(  i3o, i1_1, i2_2) = &
          Frad2(i3o, i1_1, i2_2) + fluxfraction10(i3o,1,3)*Fradc_ray(i3)
        Frad2(  i3o, i1_2, i2_2) = &
          Frad2(i3o, i1_2, i2_2) + fluxfraction10(i3o,2,3)*Fradc_ray(i3)
!#
# ifdef rhd_rad3d_fromray_l01
!#
      end do ! i3
      ! --- Flux: x2 direction ---
      do i3=m3rad,n3rad
        i3o=i3+m3
        !
        i1_0=if21_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i1_2=i1_1+1
        i2_0=if12_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        i2_2=i2_1+1
!#
# endif
!#
        !
        Frad3(  i3o, i1_0, i2_1) = &
          Frad3(i3o, i1_0, i2_1) + fluxfraction20(i3o,1,1)*Fradc_ray(i3)
        Frad3(  i3o, i1_1, i2_1) = &
          Frad3(i3o, i1_1, i2_1) + fluxfraction20(i3o,2,1)*Fradc_ray(i3)
        Frad3(  i3o, i1_2, i2_1) = &
          Frad3(i3o, i1_2, i2_1) + fluxfraction20(i3o,3,1)*Fradc_ray(i3)
        Frad3(  i3o, i1_0, i2_2) = &
          Frad3(i3o, i1_0, i2_2) + fluxfraction20(i3o,1,2)*Fradc_ray(i3)
        Frad3(  i3o, i1_1, i2_2) = &
          Frad3(i3o, i1_1, i2_2) + fluxfraction20(i3o,2,2)*Fradc_ray(i3)
        Frad3(  i3o, i1_2, i2_2) = &
          Frad3(i3o, i1_2, i2_2) + fluxfraction20(i3o,3,2)*Fradc_ray(i3)
      end do ! i3
    endif
    !
    ! --- drhoeidtime (back distribution to hd cells according to local emission ---
    do i3=m3rad,n3rad
      i3o=i3+m3
      !
      !
      i1_0=if21_0_ray(i3)+i1_shift
      i1_1=i1_0+1
      i1_2=i1_1+1
      i2_0=if12_0_ray(i3)+i2_shift
      i2_1=i2_0+1
      i2_2=i2_1+1
      !
      ! --- Compute average rho*kappa*T^4 ---
      avgemission=sumfluxfraction(i3o, 1, 1) * emission(i3o, i1_0, i2_0) + &
                  sumfluxfraction(i3o, 1, 2) * emission(i3o, i1_0, i2_1) + &
                  sumfluxfraction(i3o, 1, 3) * emission(i3o, i1_0, i2_2) + &
                  sumfluxfraction(i3o, 2, 1) * emission(i3o, i1_1, i2_0) + &
                  sumfluxfraction(i3o, 2, 2) * emission(i3o, i1_1, i2_1) + &
                  sumfluxfraction(i3o, 2, 3) * emission(i3o, i1_1, i2_2) + &
                  sumfluxfraction(i3o, 3, 1) * emission(i3o, i1_2, i2_0) + &
                  sumfluxfraction(i3o, 3, 2) * emission(i3o, i1_2, i2_1) + &
                  sumfluxfraction(i3o, 3, 3) * emission(i3o, i1_2, i2_2)
      invavgemission=1.0/avgemission
      !
      drhoeidtime(  i3o, i1_0, i2_0) = &
        drhoeidtime(i3o, i1_0, i2_0) + sumfluxfraction(i3o,1,1) * &
           emission(i3o, i1_0, i2_0) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_1, i2_0) = &
        drhoeidtime(i3o, i1_1, i2_0) + sumfluxfraction(i3o,2,1) * &
           emission(i3o, i1_1, i2_0) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_2, i2_0) = &
        drhoeidtime(i3o, i1_2, i2_0) + sumfluxfraction(i3o,3,1) * &
           emission(i3o, i1_2, i2_0) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_0, i2_1) = &
        drhoeidtime(i3o, i1_0, i2_1) + sumfluxfraction(i3o,1,2) * &
           emission(i3o, i1_0, i2_1) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_1, i2_1) = &
        drhoeidtime(i3o, i1_1, i2_1) + sumfluxfraction(i3o,2,2) * &
           emission(i3o, i1_1, i2_1) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_2, i2_1) = &
        drhoeidtime(i3o, i1_2, i2_1) + sumfluxfraction(i3o,3,2) * &
           emission(i3o, i1_2, i2_1) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_0, i2_2) = &
        drhoeidtime(i3o, i1_0, i2_2) + sumfluxfraction(i3o,1,3) * &
           emission(i3o, i1_0, i2_2) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_1, i2_2) = &
        drhoeidtime(i3o, i1_1, i2_2) + sumfluxfraction(i3o,2,3) * &
           emission(i3o, i1_1, i2_2) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i3o, i1_2, i2_2) = &
        drhoeidtime(i3o, i1_2, i2_2) + sumfluxfraction(i3o,3,3) * &
           emission(i3o, i1_2, i2_2) * invavgemission * divFrad_ray(i3)

    end do ! i3
    !
  case(2)
    ! --- Old x2 direction is main (new x3) direction ---
    !
    if (flux_flag) then
      ! --- Flux: x3 direction ---
      do i3=m3rad,n3rad+1
        i3o=i3+m3
        !
        i1_0=ib1_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i2_0=ib2_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        !
        Frad2(  i2_0, i3o, i1_0) = &
          Frad2(i2_0, i3o, i1_0) + fluxfraction30(i3o,1,1)*Fradb_ray(i3)
        Frad2(  i2_0, i3o, i1_1) = &
          Frad2(i2_0, i3o, i1_1) + fluxfraction30(i3o,2,1)*Fradb_ray(i3)
        Frad2(  i2_1, i3o, i1_0) = &
          Frad2(i2_1, i3o, i1_0) + fluxfraction30(i3o,1,2)*Fradb_ray(i3)
        Frad2(  i2_1, i3o, i1_1) = &
          Frad2(i2_1, i3o, i1_1) + fluxfraction30(i3o,2,2)*Fradb_ray(i3)
      end do ! i3
      ! --- Flux: x1 direction ---
      do i3=m3rad,n3rad
        i3o=i3+m3
        !
        i1_0=if21_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i1_2=i1_1+1
        i2_0=if12_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        i2_2=i2_1+1
        !
        Frad3(  i2_0, i3o, i1_1) = &
          Frad3(i2_0, i3o, i1_1) + fluxfraction10(i3o,1,1)*Fradc_ray(i3)
        Frad3(  i2_0, i3o, i1_2) = &
          Frad3(i2_0, i3o, i1_2) + fluxfraction10(i3o,2,1)*Fradc_ray(i3)
        Frad3(  i2_1, i3o, i1_1) = &
          Frad3(i2_1, i3o, i1_1) + fluxfraction10(i3o,1,2)*Fradc_ray(i3)
        Frad3(  i2_1, i3o, i1_2) = &
          Frad3(i2_1, i3o, i1_2) + fluxfraction10(i3o,2,2)*Fradc_ray(i3)
        Frad3(  i2_2, i3o, i1_1) = &
          Frad3(i2_2, i3o, i1_1) + fluxfraction10(i3o,1,3)*Fradc_ray(i3)
        Frad3(  i2_2, i3o, i1_2) = &
          Frad3(i2_2, i3o, i1_2) + fluxfraction10(i3o,2,3)*Fradc_ray(i3)
!#
# ifdef rhd_rad3d_fromray_l01
!#
      end do ! i3
      ! --- Flux: x2 direction ---
      do i3=m3rad,n3rad
        i3o=i3+m3
        !
        i1_0=if21_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i1_2=i1_1+1
        i2_0=if12_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        i2_2=i2_1+1
!#
# endif
!#
        !
        Frad1(  i2_1, i3o, i1_0) = &
          Frad1(i2_1, i3o, i1_0) + fluxfraction20(i3o,1,1)*Fradc_ray(i3)
        Frad1(  i2_1, i3o, i1_1) = &
          Frad1(i2_1, i3o, i1_1) + fluxfraction20(i3o,2,1)*Fradc_ray(i3)
        Frad1(  i2_1, i3o, i1_2) = &
          Frad1(i2_1, i3o, i1_2) + fluxfraction20(i3o,3,1)*Fradc_ray(i3)
        Frad1(  i2_2, i3o, i1_0) = &
          Frad1(i2_2, i3o, i1_0) + fluxfraction20(i3o,1,2)*Fradc_ray(i3)
        Frad1(  i2_2, i3o, i1_1) = &
          Frad1(i2_2, i3o, i1_1) + fluxfraction20(i3o,2,2)*Fradc_ray(i3)
        Frad1(  i2_2, i3o, i1_2) = &
          Frad1(i2_2, i3o, i1_2) + fluxfraction20(i3o,3,2)*Fradc_ray(i3)
      end do ! i3
    endif
    !
    ! --- drhoeidtime ---
    do i3=m3rad,n3rad
      i3o=i3+m3
      !
      i1_0=if21_0_ray(i3)+i1_shift
      i1_1=i1_0+1
      i1_2=i1_1+1
      i2_0=if12_0_ray(i3)+i2_shift
      i2_1=i2_0+1
      i2_2=i2_1+1
      !
!!$      write(6,'(2I4,2X,3I4,2X,3I4)') i3, i3o, &
!!$                i2_0, i2_1, i2_2, &
!!$                i1_0, i1_1, i1_2
      ! --- Compute average rho*kappa*T^4 ---
      avgemission=sumfluxfraction(i3o, 1, 1) * emission(i2_0, i3o, i1_0) + &
                  sumfluxfraction(i3o, 1, 2) * emission(i2_1, i3o, i1_0) + &
                  sumfluxfraction(i3o, 1, 3) * emission(i2_2, i3o, i1_0) + &
                  sumfluxfraction(i3o, 2, 1) * emission(i2_0, i3o, i1_1) + &
                  sumfluxfraction(i3o, 2, 2) * emission(i2_1, i3o, i1_1) + &
                  sumfluxfraction(i3o, 2, 3) * emission(i2_2, i3o, i1_1) + &
                  sumfluxfraction(i3o, 3, 1) * emission(i2_0, i3o, i1_2) + &
                  sumfluxfraction(i3o, 3, 2) * emission(i2_1, i3o, i1_2) + &
                  sumfluxfraction(i3o, 3, 3) * emission(i2_2, i3o, i1_2)
      invavgemission=1.0/avgemission
      !
      drhoeidtime(  i2_0, i3o, i1_0) = &
        drhoeidtime(i2_0, i3o, i1_0) + sumfluxfraction(i3o,1,1) * &
           emission(i2_0, i3o, i1_0) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_0, i3o, i1_1) = &
        drhoeidtime(i2_0, i3o, i1_1) + sumfluxfraction(i3o,2,1) * &
           emission(i2_0, i3o, i1_1) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_0, i3o, i1_2) = &
        drhoeidtime(i2_0, i3o, i1_2) + sumfluxfraction(i3o,3,1) * &
           emission(i2_0, i3o, i1_2) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_1, i3o, i1_0) = &
        drhoeidtime(i2_1, i3o, i1_0) + sumfluxfraction(i3o,1,2) * &
           emission(i2_1, i3o, i1_0) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_1, i3o, i1_1) = &
        drhoeidtime(i2_1, i3o, i1_1) + sumfluxfraction(i3o,2,2) * &
           emission(i2_1, i3o, i1_1) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_1, i3o, i1_2) = &
        drhoeidtime(i2_1, i3o, i1_2) + sumfluxfraction(i3o,3,2) * &
           emission(i2_1, i3o, i1_2) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_2, i3o, i1_0) = &
        drhoeidtime(i2_2, i3o, i1_0) + sumfluxfraction(i3o,1,3) * &
           emission(i2_2, i3o, i1_0) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_2, i3o, i1_1) = &
        drhoeidtime(i2_2, i3o, i1_1) + sumfluxfraction(i3o,2,3) * &
           emission(i2_2, i3o, i1_1) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i2_2, i3o, i1_2) = &
        drhoeidtime(i2_2, i3o, i1_2) + sumfluxfraction(i3o,3,3) * &
           emission(i2_2, i3o, i1_2) * invavgemission * divFrad_ray(i3)
    end do ! i3
    !
  case(3)
    ! --- Regular case: current=global indices ---
    !
    if (flux_flag) then
      ! --- Flux: x3 direction ---
      do i3=m3rad,n3rad+1
        i3o=i3+m3
        !
        i1_0=ib1_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i2_0=ib2_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        !
        Frad3(  i1_0, i2_0, i3o) = &
          Frad3(i1_0, i2_0, i3o) + fluxfraction30(i3o,1,1)*Fradb_ray(i3)
        Frad3(  i1_1, i2_0, i3o) = &
          Frad3(i1_1, i2_0, i3o) + fluxfraction30(i3o,2,1)*Fradb_ray(i3)
        Frad3(  i1_0, i2_1, i3o) = &
          Frad3(i1_0, i2_1, i3o) + fluxfraction30(i3o,1,2)*Fradb_ray(i3)
        Frad3(  i1_1, i2_1, i3o) = &
          Frad3(i1_1, i2_1, i3o) + fluxfraction30(i3o,2,2)*Fradb_ray(i3)
      end do ! i3
      ! --- Flux: x1 direction ---
      do i3=m3rad,n3rad
        i3o=i3+m3
        !
        i1_0=if21_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i1_2=i1_1+1
        i2_0=if12_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        i2_2=i2_1+1
        !
        Frad1(  i1_1, i2_0, i3o) = &
          Frad1(i1_1, i2_0, i3o) + fluxfraction10(i3o,1,1)*Fradc_ray(i3)
        Frad1(  i1_2, i2_0, i3o) = &
          Frad1(i1_2, i2_0, i3o) + fluxfraction10(i3o,2,1)*Fradc_ray(i3)
        Frad1(  i1_1, i2_1, i3o) = &
          Frad1(i1_1, i2_1, i3o) + fluxfraction10(i3o,1,2)*Fradc_ray(i3)
        Frad1(  i1_2, i2_1, i3o) = &
          Frad1(i1_2, i2_1, i3o) + fluxfraction10(i3o,2,2)*Fradc_ray(i3)
        Frad1(  i1_1, i2_2, i3o) = &
          Frad1(i1_1, i2_2, i3o) + fluxfraction10(i3o,1,3)*Fradc_ray(i3)
        Frad1(  i1_2, i2_2, i3o) = &
          Frad1(i1_2, i2_2, i3o) + fluxfraction10(i3o,2,3)*Fradc_ray(i3)
!#
# ifdef rhd_rad3d_fromray_l01
!#
      end do ! i3
      ! --- Flux: x2 direction ---
      do i3=m3rad,n3rad
        i3o=i3+m3
        !
        i1_0=if21_0_ray(i3)+i1_shift
        i1_1=i1_0+1
        i1_2=i1_1+1
        i2_0=if12_0_ray(i3)+i2_shift
        i2_1=i2_0+1
        i2_2=i2_1+1
!#
# endif
!#
        !
        Frad2(  i1_0, i2_1, i3o) = &
          Frad2(i1_0, i2_1, i3o) + fluxfraction20(i3o,1,1)*Fradc_ray(i3)
        Frad2(  i1_1, i2_1, i3o) = &
          Frad2(i1_1, i2_1, i3o) + fluxfraction20(i3o,2,1)*Fradc_ray(i3)
        Frad2(  i1_2, i2_1, i3o) = &
          Frad2(i1_2, i2_1, i3o) + fluxfraction20(i3o,3,1)*Fradc_ray(i3)
        Frad2(  i1_0, i2_2, i3o) = &
          Frad2(i1_0, i2_2, i3o) + fluxfraction20(i3o,1,2)*Fradc_ray(i3)
        Frad2(  i1_1, i2_2, i3o) = &
          Frad2(i1_1, i2_2, i3o) + fluxfraction20(i3o,2,2)*Fradc_ray(i3)
        Frad2(  i1_2, i2_2, i3o) = &
          Frad2(i1_2, i2_2, i3o) + fluxfraction20(i3o,3,2)*Fradc_ray(i3)
      end do ! i3
    endif
    !
    ! --- drhoeidtime ---
    do i3=m3rad,n3rad
      i3o=i3+m3
      !
      i1_0=if21_0_ray(i3)+i1_shift
      i1_1=i1_0+1
      i1_2=i1_1+1
      i2_0=if12_0_ray(i3)+i2_shift
      i2_1=i2_0+1
      i2_2=i2_1+1
      !
      ! --- Compute average rho*kappa*T^4 ---
      avgemission=sumfluxfraction(i3o, 1, 1) * emission(i1_0, i2_0, i3o) + &
                  sumfluxfraction(i3o, 1, 2) * emission(i1_0, i2_1, i3o) + &
                  sumfluxfraction(i3o, 1, 3) * emission(i1_0, i2_2, i3o) + &
                  sumfluxfraction(i3o, 2, 1) * emission(i1_1, i2_0, i3o) + &
                  sumfluxfraction(i3o, 2, 2) * emission(i1_1, i2_1, i3o) + &
                  sumfluxfraction(i3o, 2, 3) * emission(i1_1, i2_2, i3o) + &
                  sumfluxfraction(i3o, 3, 1) * emission(i1_2, i2_0, i3o) + &
                  sumfluxfraction(i3o, 3, 2) * emission(i1_2, i2_1, i3o) + &
                  sumfluxfraction(i3o, 3, 3) * emission(i1_2, i2_2, i3o)
      invavgemission=1.0/avgemission
      !
      drhoeidtime(  i1_0, i2_0, i3o) = &
        drhoeidtime(i1_0, i2_0, i3o) + sumfluxfraction(i3o,1,1) * &
           emission(i1_0, i2_0, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_1, i2_0, i3o) = &
        drhoeidtime(i1_1, i2_0, i3o) + sumfluxfraction(i3o,2,1) * &
           emission(i1_1, i2_0, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_2, i2_0, i3o) = &
        drhoeidtime(i1_2, i2_0, i3o) + sumfluxfraction(i3o,3,1) * &
           emission(i1_2, i2_0, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_0, i2_1, i3o) = &
        drhoeidtime(i1_0, i2_1, i3o) + sumfluxfraction(i3o,1,2) * &
           emission(i1_0, i2_1, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_1, i2_1, i3o) = &
        drhoeidtime(i1_1, i2_1, i3o) + sumfluxfraction(i3o,2,2) * &
           emission(i1_1, i2_1, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_2, i2_1, i3o) = &
        drhoeidtime(i1_2, i2_1, i3o) + sumfluxfraction(i3o,3,2) * &
           emission(i1_2, i2_1, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_0, i2_2, i3o) = &
        drhoeidtime(i1_0, i2_2, i3o) + sumfluxfraction(i3o,1,3) * &
           emission(i1_0, i2_2, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_1, i2_2, i3o) = &
        drhoeidtime(i1_1, i2_2, i3o) + sumfluxfraction(i3o,2,3) * &
           emission(i1_1, i2_2, i3o) * invavgemission * divFrad_ray(i3)
      drhoeidtime(  i1_2, i2_2, i3o) = &
        drhoeidtime(i1_2, i2_2, i3o) + sumfluxfraction(i3o,3,3) * &
           emission(i1_2, i2_2, i3o) * invavgemission * divFrad_ray(i3)
    end do ! i3
    !
end select
!
end subroutine rhd_rad3d_fromray


!----------*****************-----------------------------------------------------------------------
subroutine rhd_rad3d_solveeq(n, m3, n3, rhoin, T4sin, logPin, action, dx, mu_in, &
                             Fradout, diffFradout)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_solve ('rhd_radiation_3_dimension_solve')
!
! PURPOSE:
!   Perform radiative energy transfer step for single ray on equidistant grid.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_solveeq(n, m3, n3, rhoin, T4sin, logPin, action, dx, mu_in, Fradout, diffFrad)
!
! INPUT:
!   n:      ('number') integer specifying the total array size of rhoin, T4sin, and logPin
!   m3:     ('m_3') integer
!   n3:     ('n_3') integer
!   rhoin:  ('rho_input') real array, density
!   T4sin:  ('temperature_4_single_input') real array, temperature^4
!   logPin: ('logarithm_pressure_input') real array, pressure
!   action: ('action') derived type with collection of control parameters
!   dx:     ('delta_x') real, grid spacing (e.g. xb3(m3+1)-xb3(m3))
!   mu_in:  ('mu_input') real, cos(theta)
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   Fradout:    ('Flux_radiative_output') real array, radiative flux
!   diffFradout:('difference_Flux_radiative_output') real array, difference of radiative flux
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_rad3d_raylhd:      Solve the radiative transfer equation for one ray by Feautrier scheme
!   xkaros (function):     Compute opacities
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-22 (B.F. Copenhagen) Written
!   1999-12-03 (B.F.) ei -> T, P
!   2000-01-20 (B.F.) dxarray added
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2000-11-16 (B.F. Uppsala) Frad double precision
!   2001-02-20 (B.F.) Change in input: P,T -> logP, T^4; parameter dx_l removed
!   2001-10-18 (B.F.) diffFradout added
!--------------------------------------------------------------------------------------------------
use opta_module
use const_module, only: fourpi, sigopi
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_rad3d_solve_t01 
!#
use timing_module
!#
# endif
!#
!use uio_bulk_module
!use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
integer,                     intent(in) :: n, m3, n3
real (kind=srk), &
             dimension(n),   intent(in) :: rhoin, T4sin, logPin
type(action_type),           intent(in) :: action
real (kind=srk),             intent(in) :: dx
real,                        intent(in) :: mu_in
real (kind=drk), &
             dimension(0:n), intent(out):: Fradout
real (kind=drk), &
             dimension(n)  , intent(out):: diffFradout
!!$integer,           optional, intent(in) :: printamount
!real, dimension(:), optional,intent(in) :: dxarray
!
! --- Local variables ---
integer                                 :: ncell, ibdtop, ibdbot, ierror, bound_l, bound_r
real (kind=srk), dimension(1:n3-m3+1)   :: kappa, rho, T4s, logP
real (kind=drk), dimension(1:n3-m3+1)   :: adtau, S, diffFrad
real (kind=drk), dimension(0:n3-m3+1)   :: dSdtau, Frad
real (kind=drk)                         :: mu, wFrad, vimmst, vipmsb
!!$logical                                 :: print_flag
!--------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_rad3d_solve_t01
!#
call timing_start('RAD: solve')
!#
# endif
!#
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
mu   =real(mu_in, kind=drk)
wFrad=fourpi/3.0_drk
ncell=n3-m3+1
!
rho  =rhoin( m3+1:n3+1)
T4s  =T4sin( m3+1:n3+1)
logP =logPin(m3+1:n3+1)
!
! --- Initialize flux, compute double precision source (negative temperatures -> no source) ---
diffFrad=0.0_drk
Frad    =0.0_drk
S       =real(sigopi * T4s, kind=drk)
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_start('RAD: solve: xkaros')
!#
# endif
!#
! --- Opacity ---
!!$write(6,'(A)') 'solveeq: compute kappa'
!!$call flush(6)
kappa=xkaros_logPT42kappa(n3-m3+1, logP, T4s, 0)
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_stop('RAD: solve: xkaros')
!#
# endif
!#
! --- Delta (optical depth) ---
adtau=kappa * rho * dx
!
! --- Smoothing of opacity (here adtau) in internal cells ---
if (par%C_radkappasmooth > 0.0) then
  adtau(2:ncell-1)=(1.0-par%C_radkappasmooth)* adtau(2:ncell-1) + &
                    0.5*par%C_radkappasmooth *(adtau(1:ncell-2)+adtau(3:ncell))
endif
!
!
! --- dSdtau inside ---
!!$write(6,'(A)') 'solveeq: compute dSdtau'
!!$call flush(6)
dSdtau(1:ncell-1)=2.0_drk * (S(1:ncell-1)-S(2:ncell)) / (adtau(1:ncell-1)+adtau(2:ncell))
!
! --- Boundary conditions ---
!
! --- Bottom ---
select case (bound_l)
  case(bound_periodic)
    ! --- periodic ---
    ibdbot   =1
    dSdtau(0)=0.0_drk
    vipmsb   =dSdtau(0)
  case(bound_constant, bound_closedbottom, bound_closedbottom2)
    ! --- constant or closed ---
    ! --- Stellar atmosphere: diffusion approximation at bottom with prescribed flux ---
    ibdbot   =1
    dSdtau(0)=0.75_drk * sigopi * (par%Teff**2)**2
    vipmsb   =dSdtau(0)
  case(bound_inoutflow, bound_inoutflow2)
    ! --- constant ---
    ! --- Stellar atmosphere: diffusion approximation at bottom with constant dSdtau ---
    ibdbot   =1
    dSdtau(0)=dSdtau(1)
    vipmsb   =0
  case default
    ! --- Open boundary ---
    ibdbot   =0
    dSdtau(0)=0.0_drk
    vipmsb   =-S(1)
end select
!
! --- Top ---
select case (bound_r)
  case(bound_periodic)
    !--- periodic: zero flux (wild approximation!) ---
    ibdtop       =1
    dSdtau(ncell)=0.0_drk
    vimmst       =dSdtau(ncell)
  case default
    ! --- always open boundary ---
    ibdtop       =0
    dSdtau(ncell)=0.0
    vimmst       =-S(ncell)
end select
!
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_start('RAD: solve: raylhd')
!#
# endif
!#
! --- Solve radiative transfer along ray ---
!!$write(6,'(A)') 'solveeq: call rhd_rad3d_raylhd'
!!$call flush(6)
!!$print_flag=.false.
!!$if (present(printamount)) then
!!$  if (printamount > 0) then
!!$    write(6,*) 'kappa:  ', kappa    
!!$    write(6,*) 'adtau:  ', adtau
!!$    write(6,*) 'dSdtau: ', dSdtau
!!$    write(6,*) 'S:      ', S
!!$    print_flag=.true.
!!$  endif
!!$endif
!
!!$if (print_flag) then
!!$  call rhd_rad3d_raylhdout(mu, wFrad, ncell, &
!!$                        adtau, dSdtau, ibdtop, vimmst, ibdbot, vipmsb, &
!!$                        Frad)
!!$else
  call rhd_rad3d_raylhd(mu, wFrad, ncell, &
                        adtau, dSdtau, ibdtop, vimmst, ibdbot, vipmsb, &
                        Frad, diffFrad)
!!$endif
!!$write(6,'(A)') 'solveeq: rhd_rad3d_raylhd done'
!!$call flush(6)
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_stop('RAD: solve: raylhd')
!#
# endif
!#
!
Fradout(    m3  :n3+1)=Frad
diffFradout(m3+1:n3+1)=diffFrad
!
! --- Additional test output ---
!!$write(6,'(A)') 'Write into rhd_rad3d_solve_test.out'
!!$include 'rhd_rad3d_solve_testout.f90'
!
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_stop('RAD: solve')
!#
# endif
!#
end subroutine rhd_rad3d_solveeq


!----------***************-------------------------------------------------------------------------
subroutine rhd_rad3d_solve(n, m3, n3, rhoin, T4sin, logPin, action, dxarray, mu_in, &
                           Fradout, diffFradout)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_solve ('rhd_radiation_3_dimension_solve')
!
! PURPOSE:
!   Perform radiative energy transfer step for single ray.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_solve(n, m3, n3, rho, T4s, logP, action, dxarray, mu_in, Fradout, diffFradout)
!
! INPUT:
!   rho:    ('rho') real array, density
!   T4s:    ('temperature_4_single') real array, temperature^4
!   logP:   ('pressure') real array, pressure
!   action: ('action') derived type with collection of control parameters
!   dxarray:('delta_x_array') real array (same size as T4s, logP) with (possibly) spatially varying
!           grid size
!   mu_in:  ('mu_input') real, cos(theta)
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   Fradout:    ('Flux_radiative_output') real array, radiative flux
!   diffFradout:('difference_Flux_radiative_output') real array, difference of radiative flux
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_rad3d_raylhd:      Solve the radiative transfer equation for one ray by Feautrier scheme
!   xkaros (function):     Compute opacities
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-22 (B.F. Copenhagen) Written
!   1999-12-03 (B.F.) ei -> T, P
!   2000-01-20 (B.F.) dxarray added
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2000-11-16 (B.F. Uppsala) Frad double precision
!   2001-02-20 (B.F.) Change in input: P,T -> logP, T^4; parameter dx_l removed
!   2001-10-18 (B.F.) diffFradout added
!--------------------------------------------------------------------------------------------------
use opta_module
use const_module, only: fourpi, sigopi
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_rad3d_solve_t01 
!#
use timing_module
!#
# endif
!#
!use uio_bulk_module
!use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
integer,                     intent(in) :: n, m3, n3
real (kind=srk), &
             dimension(n),   intent(in) :: rhoin, T4sin, logPin
type(action_type),           intent(in) :: action
real, dimension(n),          intent(in) :: dxarray
real,                        intent(in) :: mu_in
real (kind=drk), &
             dimension(0:n), intent(out):: Fradout
real (kind=drk), &
             dimension(n)  , intent(out):: diffFradout
!
! --- Local variables ---
integer                                 :: ncell, ibdtop, ibdbot, ierror, bound_l, bound_r
real (kind=srk), dimension(1:n3-m3+1)   :: kappa, kapparho, rho, T4s, logP
real (kind=drk), dimension(1:n3-m3+1)   :: adtau, S, diffFrad
real (kind=drk), dimension(0:n3-m3+1)   :: dSdtau, Frad
real (kind=drk)                         :: mu, wFrad, vimmst, vipmsb
!--------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_rad3d_solve_t01
!#
call timing_start('RAD: solve')
!#
# endif
!#
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
mu   =real(mu_in, kind=drk)
wFrad=fourpi/3.0_drk
ncell=n3-m3+1
!
rho  =rhoin( m3+1:n3+1)
T4s  =T4sin( m3+1:n3+1)
logP =logPin(m3+1:n3+1)
!
! --- Initialize flux, compute double precision source (negative temperatures -> no source) ---
diffFrad=0.0_drk
Frad    =0.0_drk
S       =real(sigopi * T4s, kind=drk)
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_start('RAD: solve: xkaros')
!#
# endif
!#
! --- Opacity ---
kappa=xkaros_logPT42kappa(n3-m3+1, logP, T4s, 0)
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_stop('RAD: solve: xkaros')
!#
# endif
!#
kapparho=kappa*rho
!
! --- Smoothing of opacity in internal cells ---
if (par%C_radkappasmooth > 0.0) then
  kapparho(2:ncell-1)=(1.0-par%C_radkappasmooth)* kapparho(2:ncell-1) + &
                       0.5*par%C_radkappasmooth *(kapparho(1:ncell-2)+kapparho(3:ncell))
endif
!
! --- Delta (optical depth) ---
adtau=kapparho * dxarray
!
! --- dSdtau inside ---
dSdtau(1:ncell-1)=2.0_drk * (S(1:ncell-1)-S(2:ncell)) / (adtau(1:ncell-1)+adtau(2:ncell))
!
! --- Boundary conditions ---
!
! --- Bottom ---
select case (bound_l)
  case(bound_periodic)
    ! --- periodic ---
    ibdbot   =1
    dSdtau(0)=0.0_drk
    vipmsb   =dSdtau(0)
  case(bound_constant, bound_closedbottom, bound_closedbottom2)
    ! --- constant or closed ---
    ! --- Stellar atmosphere: diffusion approximation at bottom with prescribed flux ---
    ibdbot   =1
    dSdtau(0)=0.75_drk * sigopi * (par%Teff**2)**2
    vipmsb   =dSdtau(0)
  case(bound_inoutflow, bound_inoutflow2)
    ! --- constant ---
    ! --- Stellar atmosphere: diffusion approximation at bottom with constant dSdtau ---
    ibdbot   =1
    dSdtau(0)=dSdtau(1)
    vipmsb   =0
  case default
    ! --- Open boundary ---
    ibdbot   =0
    dSdtau(0)=0.0_drk
    vipmsb   =-S(1)
end select
!
! --- Top ---
select case (bound_r)
  case(bound_periodic)
    !--- periodic: zero flux (wild approximation!) ---
    ibdtop       =1
    dSdtau(ncell)=0.0_drk
    vimmst       =dSdtau(ncell)
  case default
    ! --- always open boundary ---
    ibdtop       =0
    dSdtau(ncell)=0.0
    vimmst       =-S(ncell)
end select
!
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_start('RAD: solve: raylhd')
!#
# endif
!#
! --- Solve radiative transfer along ray ---
call rhd_rad3d_raylhd(mu, wFrad, ncell, &
                      adtau, dSdtau, ibdtop, vimmst, ibdbot, vipmsb, &
                      Frad, diffFrad)
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_stop('RAD: solve: raylhd')
!#
# endif
!#
!
Fradout(    m3  :n3+1)=Frad
diffFradout(m3+1:n3+1)=diffFrad
!
! --- Additional test output ---
!!$write(6,'(A)') 'Write into rhd_rad3d_solve_test.out'
!!$include 'rhd_rad3d_solve_testout.f90'
!
!#
# ifdef rhd_rad3d_solve_t01 
!#
call timing_stop('RAD: solve')
!#
# endif
!#
end subroutine rhd_rad3d_solve


!----------********************--------------------------------------------------------------------
subroutine rhd_rad3d_solveadapt(rho, T4, logP, action, dx, mu, Frad, diffFrad)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_solveadapt ('rhd_radiation_3_dimension_solve_adaptive')
!
! PURPOSE:
!   Perform radiative energy transfer step for single ray on locally refined (adapted) grid.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   rho:    ('rho') real array, density
!   T4:     ('temperature_4') real array, temperature^4
!   logP:   ('logarithm_pressure') real array, pressure
!   action: ('action') derived type with collection of control parameters
!   dx:     ('delta_x') real, grid spacing (e.g. xb3(m3+1)-xb3(m3))
!   mu:     ('mu') real, cos(theta)
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   Frad:   ('Flux_radiative_left') real array, radiative flux
!
! LOCAL VARIABLES:
!
! ROUTINES:
!
! MODULES:
!   cubit_module:          Cubic (monotonic) interpolation
!   rhd_action_module:     Routines to handle the control structure 'action'
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   If refinement really takes place, the flux difference is taken from refined flux
!   instead of from the refined flux difference!
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-18 (B.F. Copenhagen) Written
!   2000-11-16 (B.F. Uppsala) Frad double precision
!   2001-02-10 (B.F.) Now allocation of temporary arrays anymore
!   2001-10-18 (B.F.) diffFradout added
!--------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_gl_module
use cubit_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
real, dimension(:),      intent(in)     :: rho, T4, logP
type(action_type),       intent(in)     :: action
real,                    intent(in)     :: dx, mu
real (kind=drk), &
      dimension(0:size(rho)),intent(out):: Frad
real (kind=drk), &
      dimension(1:size(rho)),intent(out):: diffFrad
!
! --- Local variables ---
logical, dimension(1:size(rho)-1)       :: T4step_mask
logical, dimension(1:size(rho)  )       :: T4_mask
integer, dimension(1:size(rho)-1)       :: T4step_size
integer, dimension(1:size(rho)  )       :: Fradstep_size
integer, dimension(1:size(rho)  )       :: oldT4infine_index
integer, dimension(0:size(rho)  )       :: oldFradinfine_index
real, &
  dimension(1:size(rho)*par%N_radtaurefine) &
                                        :: rhofine, T4fine, logPfine, dxfine
real (kind=drk), &
  dimension(0:size(rho)*par%N_radtaurefine) &
                                        :: Fradfine
real (kind=drk), &
  dimension(1:size(rho)*par%N_radtaurefine) &
                                        :: diffFradfine
integer, dimension(1:size(rho))         :: T4stepinold_index, T4stepinfine_index
integer                                 :: ncell, icell, &
                                           nrefine, irefine, &
                                           ncellfine, icellfine, &
                                           isubgrid, ifirst, iinter, Ntauonesiderefine
real                                    :: xpoint
real, dimension(1:4)                    :: xgrid
!--------------------------------------------------------------------------------------------------
!
! --- Number of cells on original ray ---
ncell=size(rho)
!
! --- Find where relative temperature is larger than prescribed value ---
T4step_mask=(par%C_radTstep* abs((T4(2:ncell)-T4(1:ncell-1))/min(T4(2:ncell),T4(1:ncell-1)))) &
            > 0.5
T4step_mask(1      :2      )=.false.
T4step_mask(ncell-2:ncell-1)=.false.
T4_mask(2:ncell-1)=T4step_mask(1:ncell-2) .or. T4step_mask(2:ncell-1)
T4_mask(1    )    =.false.
T4_mask(ncell)    =.false.
!
! --- Number of temperature steps ---
nrefine=count(T4_mask)
!
if ((nrefine > 0) .and. (par%N_radtaurefine > 1)) then
  Ntauonesiderefine=(par%N_radtaurefine-1)/2
  !
  ! --- Number of cells on refined ray ---
  ncellfine=ncell + (par%N_radtaurefine-1)*nrefine
  !
  ! --- Find index of grid points in new grid corresponding to old grid points ---
  T4step_size=1
  where (T4_mask(1:ncell-1))
    T4step_size           =T4step_size            + Ntauonesiderefine
  end where
  where (T4_mask(2:ncell  ))
    T4step_size(1:ncell-2)=T4step_size(1:ncell-2) + Ntauonesiderefine
  end where
  oldT4infine_index(1)=1
  do icell=2,ncell
    oldT4infine_index(icell)=oldT4infine_index(icell-1)+T4step_size(icell-1)
  end do ! icell
  !
  ! --- All indices of points on old grid to be refined ---
  T4stepinold_index(1:nrefine)=pack((/(icell, icell=1,ncell)/), T4_mask)
  !
  ! --- All indices of points on new grid with environment to be refined ---
  T4stepinfine_index(1:nrefine)=oldT4infine_index(T4stepinold_index(1:nrefine))
  !
  ! --- Indices of boundaries with radiative flux corresponding to old grid ---
  where (T4_mask)
    Fradstep_size=par%N_radtaurefine
  elsewhere
    Fradstep_size=1
  end where
  !
  oldFradinfine_index(0)=0
  do icell=1,ncell
    oldFradinfine_index(icell)=oldFradinfine_index(icell-1)+Fradstep_size(icell)
  end do ! icell
  !
  ! --- Put old grid values on new grid ---
  rhofine( oldT4infine_index)=rho
  T4fine(  oldT4infine_index)=T4
  logPfine(oldT4infine_index)=logP
  !
  ! --- dx array ---
  dxfine(1:ncellfine)=dx
  !
  ! --- Interpolate onto finer grid ---
  xgrid=(/-1.0, 0.0, 1.0, 2.0/) * real(par%N_radtaurefine)
  do irefine=1,nrefine
    icell=T4stepinold_index(irefine)-1
    ifirst=min(max(icell-1,1),ncell-3)
    iinter=icell-ifirst+1
    do isubgrid=1,Ntauonesiderefine
      icellfine=T4stepinfine_index(irefine)-isubgrid
      dxfine(icellfine)=dx/real(par%N_radtaurefine)
      xpoint=real(par%N_radtaurefine-isubgrid)
      call cubiti(xgrid, rho( ifirst:ifirst+3), 4, 3, iinter, xpoint, rhofine( icellfine))
      call cubiti(xgrid, T4(  ifirst:ifirst+3), 4, 3, iinter, xpoint, T4fine(  icellfine))
      call cubiti(xgrid, logP(ifirst:ifirst+3), 4, 3, iinter, xpoint, logPfine(icellfine))
    end do ! isubgrid
    !
    icellfine=T4stepinfine_index(irefine)
    dxfine(icellfine)=dx/real(par%N_radtaurefine)
    !
    icell=T4stepinold_index(irefine)
    ifirst=min(max(icell-1,1),ncell-3)
    iinter=icell-ifirst+1
    do isubgrid=1,Ntauonesiderefine
      icellfine=T4stepinfine_index(irefine)+isubgrid
      dxfine(icellfine)=dx/real(par%N_radtaurefine)
      xpoint=real(isubgrid)
      call cubiti(xgrid, rho( ifirst:ifirst+3), 4, 3, iinter, xpoint, rhofine( icellfine))
      call cubiti(xgrid, T4(  ifirst:ifirst+3), 4, 3, iinter, xpoint, T4fine(  icellfine))
      call cubiti(xgrid, logP(ifirst:ifirst+3), 4, 3, iinter, xpoint, logPfine(icellfine))
    end do ! isubgrid
  end do ! irefine
  !
  ! --- Solve radiative transfer equation for single ray on refined grid ---
  call rhd_rad3d_solve(ncell,1,ncellfine, rhofine,T4fine,logPfine, &
                       action, dxfine, mu, &
                       Fradfine, diffFradfine)
  !
  ! --- Put flux on original coarse grid ---
  Frad=Fradfine(oldFradinfine_index)
  !
  ! --- Cheat: take diffFrad from Fradfine instead of from diffFradfine ---
  diffFrad=Frad(1:ncell) - Frad(0:ncell-1)
  !
else
  ! --- Solve radiative transfer equation for single ray on original grid ---
  call rhd_rad3d_solveeq(ncell,1,ncell, rho, T4, logP, &
                         action, dx, mu, &
                         Frad, diffFrad)
endif
!
end subroutine rhd_rad3d_solveadapt


!#
# ifdef rhd_r01_new
!#

!----------*************---------------------------------------------------------------------------
subroutine rhd_rad3d_dir(action, flux_flag, r_dir, n_dir, &
                         m1,n1, m2,n2, m3,n3, m1org,n1org, m2org,n2org, m3org,n3org, &
                         xc1,xc2,xc3, xb1,xb2,xb3, &
                         rho, T4, logP, kappa, emission, &
                         Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_dir ('rhd_radiation_3_dimension_direction')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   action: ('action') derived type with collection of control parameters
!
! INPUT/OUTPUT:
!
! OUTPUT:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_rad3d_projectarea: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_projectonplane: Project point along given direction on plane
!   rhd_rad3d_solve:       Perform radiative energy transfer step for single ray
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1999-12-03 (B.F. Copenhagen) Put into own routine
!   1999-12-10 (B.F.) Definition of ib1,2_ray0 modified
!   1999-12-14 (B.F.) Corrected sign for fluxfraction10,20
!   2000-02-22 (B.F.) Modifications for wc12_ray0
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2000-05-29 (B.F.) Limit T_minimum with T_bilinear
!   2000-05-30 (B.F.) Interpolate between T_bilinear and T_minimum
!   2000-11-21 (B.F. Uppsala) Interpolate drhoeidtime from ray to HD grid
!   2000-11-22 (B.F.) Flux divergence correction
!   2000-11-27 (B.F.) Correction in formula for sumfluxfraction
!   2000-12-28 (B.F.) Allocation of ray vectors outside of loops
!   2001-01-09 (B.F.) Pointers => standard arrays
!   2001-03-09 (B.F.) OpenMP directives
!   2001-03-18 (B.F.) New version with chunks started writing
!--------------------------------------------------------------------------------------------------
use opta_module
use rhd_action_module
!#
# ifdef rhd_rad3d_dir_t01 
!#
use timing_module
!#
# endif
!#
!use uio_filedef_module
!use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),        intent(in)    :: action
logical,                  intent(in)    :: flux_flag
real, dimension(3),       intent(in)    :: r_dir
integer,                  intent(in)    :: n_dir, m1,n1, m2,n2, m3,n3, &
                                           m1org,n1org, m2org,n2org, m3org,n3org
real, dimension(m1:n1  ), intent(in)    :: xc1
real, dimension(m2:n2  ), intent(in)    :: xc2
real, dimension(m3:n3  ), intent(in)    :: xc3
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1org  :n1org, &
                m2org  :n2org, &
                m3org  :n3org ), &
                          intent(in)    :: rho, T4, logP, kappa
real, dimension(m1org-3:n1org+3, &
                m2org-3:n2org+3, &
                m3org-3:n3org+3), &
                          intent(in)    :: emission
real, dimension(m1org-3:n1org+3+1, &
                m2org-3:n2org+3  , &
                m3org-3:n3org+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1org-3:n1org+3  , &
                m2org-3:n2org+3+1, &
                m3org-3:n3org+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1org-3:n1org+3  , &
                m2org-3:n2org+3  , &
                m3org-3:n3org+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1org-3:n1org+3, &
                m2org-3:n2org+3, &
                m3org-3:n3org+3), &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
real (kind=srk)                         :: dx1, dx2, dx3, dx1sub, dx2sub, &
                                           x1, x1min, x1max, x2, x2min, x2max, x3, &
                                           dxeps, slope, mu
real, dimension(3)                      :: r_dir0
real (kind=srk), dimension(3)           :: c000, c100, c010, c110, c001, c101, c011, c111, &
                                           cp001, cp101, cp011, cp111, &
                                           p00, p01, p10, p11
!!$                                           cp000, cp100, cp010, cp110, &
real (kind=srk), dimension(m3:n3)       :: xc1_ray0, xc2_ray0, &
                                           xc3_ray, &
                                           wc1_ray0, wc2_ray0, &
                                           wc_00_ray0, wc_01_ray0, &
                                           wc_10_ray0, wc_11_ray0
real (kind=srk), dimension(m3:n3+1)     :: xb1_ray0, xb2_ray0
real (kind=srk), dimension(m3:n3  ,2,3) :: fluxfraction10
real (kind=srk), dimension(m3:n3  ,3,2) :: fluxfraction20
real (kind=srk), dimension(m3:n3+1,2,2) :: fluxfraction30
real (kind=srk), dimension(m3:n3  ,3,3) :: sumfluxfraction
real (kind=srk)                         :: ffdown, ffup
logical,         dimension(m3:n3+1)     :: flag_ray
integer,         dimension(m3:n3  )     :: ic1_ray0 , ic2_ray0, &
                                           if1_ray0 , if2_ray0
integer,         dimension(m3:n3+1)     :: ib1_ray0 , ib2_ray0, ib3_ray0
real (kind=srk), dimension(0:n3-m3)     :: rho_ray, T4_ray, logP_ray
real (kind=drk), dimension(0:n3-m3  )   :: diffFradb
real (kind=drk), dimension(0:n3-m3+1)   :: Fradb
integer,         dimension(0:n3-m3)     :: ic1_0_ray, ic1_1_ray, &
                                           ic2_0_ray, ic2_1_ray
integer                                 :: i, &
                                           m1rad, n1rad, m2rad, n2rad, m3rad, n3rad, &
                                           i1, i2, i3, i1s, i2s, i1_base, i1_offset, &
                                           i1min, i2min, &
                                           m3orad, n3orad
integer                                 :: m3rad_min, m3rad_max, n3rad_min, n3rad_max, &
                                           ib1_min, ib1_max, ib2_min, ib2_max
integer, parameter                      :: N_radraymult=1, &
                                           N_radminpointsonray=5
integer                                 :: omp_get_thread_num
!--------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_rad3d_dir_t01 
!#
call timing_start('RAD: dir')
call timing_start('RAD: dir: init')
!#
# endif
!#
!
! --- Typical grid size: grid assumed to be equidistant ---
dx1   =(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
dx2   =(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
dx3   =(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
!
dx1sub=dx1/real(N_radraymult)
dx2sub=dx2/real(N_radraymult)
!
dxeps=1.0E-03 * min(dx1,dx2,dx3)
!
! --- Corners ---
c000=(/ xb1(m1  ), xb2(m2  ), xb3(m3  ) /)
c001=(/ xb1(m1  ), xb2(m2  ), xb3(n3+1) /)
c010=(/ xb1(m1  ), xb2(n2+1), xb3(m3  ) /)
c011=(/ xb1(m1  ), xb2(n2+1), xb3(n3+1) /)
c100=(/ xb1(n1+1), xb2(m2  ), xb3(m3  ) /)
c101=(/ xb1(n1+1), xb2(m2  ), xb3(n3+1) /)
c110=(/ xb1(n1+1), xb2(n2+1), xb3(m3  ) /)
c111=(/ xb1(n1+1), xb2(n2+1), xb3(n3+1) /)
!
! --- Normalization ---
r_dir0=sign(1.0,r_dir(3))*r_dir
mu=r_dir0(3)
!
! --- Relative coordinates of ray ---
do i3=m3,n3+1
  xb1_ray0(i3)=(xb3(i3)-xb3(m3)) * (r_dir0(1)/r_dir0(3))
  xb2_ray0(i3)=(xb3(i3)-xb3(m3)) * (r_dir0(2)/r_dir0(3))
!!$  xb3_ray( i3)= xb3(i3)
end do ! i3
do i3=m3,n3
  xc1_ray0(i3)=(xc3(i3)-xb3(m3)) * (r_dir0(1)/r_dir0(3))
  xc2_ray0(i3)=(xc3(i3)-xb3(m3)) * (r_dir0(2)/r_dir0(3))
  xc3_ray( i3)= xc3(i3)
end do ! i3
!
! --- Integer indices and weights of 4 neighbors ---          
ic1_ray0 =floor(   xc1_ray0/dx1-0.5)
wc1_ray0 =1.0_srk-(xc1_ray0/dx1-0.5-real(ic1_ray0))
ic2_ray0 =floor(   xc2_ray0/dx2-0.5)
wc2_ray0 =1.0_srk-(xc2_ray0/dx2-0.5-real(ic2_ray0))
wc_00_ray0=         wc1_ray0 *         wc2_ray0
wc_10_ray0=(1.0_srk-wc1_ray0)*         wc2_ray0
wc_01_ray0=         wc1_ray0 *(1.0_srk-wc2_ray0)
wc_11_ray0=(1.0_srk-wc1_ray0)*(1.0_srk-wc2_ray0)
!
ib1_ray0 =floor((xb1_ray0-0.5*dx1sub)/dx1)
ib2_ray0 =floor((xb2_ray0-0.5*dx2sub)/dx2)
ib3_ray0 =(/(i3,i3=0,n3-m3+1)/)
do i3=m3,n3+1
  do i1s=1,2
    do i2s=1,2
      call rhd_rad3d_projectarea( &
             (ib1_ray0(i3)+i1s-1) *dx1, (ib2_ray0(i3)+i2s-1) *dx2, &
             (ib1_ray0(i3)+i1s  ) *dx1, (ib2_ray0(i3)+i2s  ) *dx2, &
              xb1_ray0(i3)-0.5*dx1sub ,  xb2_ray0(i3)-0.5*dx2sub, &
              xb1_ray0(i3)+0.5*dx1sub ,  xb1_ray0(i3)-0.5*dx1sub, &
                                         xb2_ray0(i3)+0.5*dx2sub, &
              fluxfraction30(i3,i1s,i2s))
    end do ! i2s
  end do ! i1s
end do ! i3
!
do i3=m3,n3
  i1min=min(ib1_ray0(i3),ib1_ray0(i3+1))
  i2min=min(ib2_ray0(i3),ib2_ray0(i3+1))
  if1_ray0(i3)=i1min
  if2_ray0(i3)=i2min
  if (abs(r_dir0(1)) > 1.0E-05) then
    do i1s=1,2
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p01)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p00)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p11)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p10)
      do i2s=1,3
        call rhd_rad3d_projectarea( &
               (i2min+i2s-1)*dx2, xb3(i3), (i2min+i2s)*dx2,            xb3(i3+1), &
                p00(2),           p00(3),   p10(2),           p01(2),  p01(3), &
                fluxfraction10(i3,i1s,i2s))
      end do ! i2s
    end do ! i1s
    if (r_dir0(1) < 0.0) fluxfraction10(i3,:,:)=-fluxfraction10(i3,:,:)
  else
    fluxfraction10(i3,:,:)=0.0
  endif
  if (abs(r_dir0(2)) > 1.0E-05) then
    do i2s=1,2
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p11)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p01)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p10)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p00)

!!$    if (i3 < (m3+5)) then
!!$      write(6,'(A,4I4)') 'i3, i1min, i2min, i2s: ', i3, i1min, i2min, i2s
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p00: ', (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p00/dx1
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p01: ', (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p01/dx1
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p10: ', (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p10/dx1
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p11: ', (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p11/dx1
!!$    endif

      do i1s=1,3
        call rhd_rad3d_projectarea( &
               (i1min+i1s-1)*dx1, xb3(i3), (i1min+i1s)*dx1,            xb3(i3+1), &
               p00(1),           p00(3),   p10(1),           p01(1),  p01(3), &
                fluxfraction20(i3,i1s,i2s))
      end do ! i1s
    end do ! i2s
    if (r_dir0(2) < 0.0) fluxfraction20(i3,:,:)=-fluxfraction20(i3,:,:)
  else
    fluxfraction20(i3,:,:)=0.0
  endif
end do ! i3
!

! --- Compute measure for relative volume fraction ---
do i1s=1,3
  do i2s=1,3
    do i3=m3,n3
      i1=i1s-(ib1_ray0(i3)-if1_ray0(i3))
      if ((i1 < 1) .or. (i1 > 2)) then
        ffdown=0.0
      else
        i2=i2s-(ib2_ray0(i3)-if2_ray0(i3))
        if ((i2 < 1) .or. (i2 > 2)) then
          ffdown=0.0
        else
          ffdown=fluxfraction30(i3,i1,i2)
        endif
      endif
      !
      i1=i1s-(ib1_ray0(i3+1)-if1_ray0(i3))
      if ((i1 < 1) .or. (i1 > 2)) then
        ffup=0.0
      else
        i2=i2s-(ib2_ray0(i3+1)-if2_ray0(i3))
        if ((i2 < 1) .or. (i2 > 2)) then
          ffup=0.0
        else
          ffup=fluxfraction30(i3+1,i1,i2)
        endif
      endif
      !
      sumfluxfraction(i3,i1s,i2s)=0.5*(ffup+ffdown)
      ! write(6,'(I3,2X,4I4,2X,3F8.4)') &
      !   i3, i1s, i2s, i1, i2, ffdown, ffup, sumfluxfraction(i3,i1s,i2s)
    end do ! i3
  end do ! i2s
end do ! i1s
!
!!$do i3=m3,n3
!!$  write(6,'(I3,2X,2I4,2X,2I4)')     i3, ib1_ray0(i3), if1_ray0(i3), ib2_ray0(i3), if2_ray0(i3)
!!$  write(6,'(5X,9F8.4,2X,F8.4)')     sumfluxfraction(i3,:,:), sum(sumfluxfraction(i3,:,:))
!!$  write(6,'(5X,6F8.4,24X,2X,F8.4)') fluxfraction10(i3,:,:) , sum(fluxfraction10(i3,:,:))
!!$  write(6,'(5X,6F8.4,24X,2X,F8.4)') fluxfraction20(i3,:,:) , sum(fluxfraction20(i3,:,:))
!!$  write(6,'(5X,4F8.4,40X,2X,F8.4)') fluxfraction30(i3,:,:) , sum(fluxfraction30(i3,:,:))
!!$end do ! i3
!
! --- Projection of corners onto x12 plane ---
call rhd_rad3d_projectonplane(c001, r_dir0, c000(3), 3, cp001)
call rhd_rad3d_projectonplane(c011, r_dir0, c000(3), 3, cp011)
call rhd_rad3d_projectonplane(c101, r_dir0, c000(3), 3, cp101)
call rhd_rad3d_projectonplane(c111, r_dir0, c000(3), 3, cp111)
!!$call rhd_rad3d_projectonplane(c000, r_dir0, c000(3), 3, cp000)
!!$call rhd_rad3d_projectonplane(c010, r_dir0, c000(3), 3, cp010)
!!$call rhd_rad3d_projectonplane(c100, r_dir0, c000(3), 3, cp100)
!!$call rhd_rad3d_projectonplane(c110, r_dir0, c000(3), 3, cp110)
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c000:', c000, ' ->', cp000
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c100:', c100, ' ->', cp100
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c010:', c010, ' ->', cp010
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c110:', c110, ' ->', cp110
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c001:', c001, ' ->', cp001
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c101:', c101, ' ->', cp101
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c011:', c011, ' ->', cp011
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c111:', c111, ' ->', cp111
!
! --- Range in x1 direction of foot points and foot point indices for rays ---
x1min=min(c000(1),cp001(1))
x1max=max(c100(1),cp101(1))
m1rad=int((x1min-c000(1))/dx1sub)
n1rad=int((x1max-c000(1))/dx1sub)
!write(6,'(A,2E12.5,A,2I5)') 'x1min, x1max: ', x1min, x1max, '   m1rad, n1rad:', m1rad, n1rad
!
if (abs(cp001(1)-c000(1)) > dxeps) then
  slope=(cp001(2)-c000(2))/(cp001(1)-c000(1))
endif
!
!!$ib1_min=n3-m3+100
!!$ib1_max=-100
!!$ib2_min=n3-m3+100
!!$ib2_max=-100
!!$m3rad_min=n3-m3+100
!!$m3rad_max=-100
!!$n3rad_min=n3-m3+100
!!$n3rad_max=-100
!
! --- Range in x2 direction of foot points and foot point indices for rays ---
! --- First guess ---
x2min=min(c000(2), cp001(2))
x2max=max(c010(2), cp011(2))
m2rad=int((x2min-c000(2))/dx2sub)
n2rad=int((x2max-c000(2))/dx2sub)
!


!
call rhd_box_ChunkSize(1, n_elements_per_chunk, m3rad,n3rad, m1rad,n1rad, m2rad,n2rad, , &
                       m1c,n1c, m2c,n2c, &
                       n_chunks=n_chunks, n_chunks_per_slice=n_chunks_per_slice)
!#
# ifdef rhd_rad3d_dir_t01 
!#
call timing_stop('RAD: dir: init')
!#
# endif
!#
do i2_offset=0,1
 do i1_offset=0,1
  n1_chunks=(n_chunks_per_slice          + 1-i1_offset)/2
  n2_chunks=(n_chunks/n_chunks_per_slice + 1-i2_offset)/2
  n_chunks_now=n1_chunks*n2_chunks
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP   PRIVATE(i1, i2, x1, x2min, x2max, m2rad, n2rad, flag_ray, m3rad, n3rad, m3orad, n3orad, &
  !$OMP           ic1_0_ray, ic1_1_ray, ic2_0_ray, ic2_1_ray, rho_ray, T4_ray, logP_ray, &
  !$OMP           Fradb, diffFradb), &
  !$OMP  	FIRSTPRIVATE(dx3, mu, &
  !$OMP                ic1_ray0, ic2_ray0, ib1_ray0, ib2_ray0, &
  !$OMP                wc_00_ray0, wc_01_ray0, wc_10_ray0, wc_11_ray0, &
  !$OMP                fluxfraction10, fluxfraction20, fluxfraction30, sumfluxfraction)
  !$OMP DO
  do i_chunk_now=1,n_chunks_now
    i_chunk=(((  i_chunk_now-1)/n1_chunks)*2+i2_offset)*n_chunks_per_slice + &
             mod(i_chunk_now-1, n1_chunks)*2+i1_offset                     + 1
    !
    call rhd_box_ChunkSize(1, n_elements_per_chunk, m3rad,n3rad,m1rad,n1rad,m2rad,n2rad, i_chunk, &
                           m1c,n1c, m2c,n2c)
    !
    flag_ray=(((ib1_ray0+m1c) >= -1) .and. ((ib1_ray0+m1c) <= (n1-m1)) .and. &
              ((ib2_ray0+m2c) >= -1) .and. ((ib2_ray0+m2c) <= (n2-m2))      ) .or. &
             (((ib1_ray0+m1c) >= -1) .and. ((ib1_ray0+m1c) <= (n1-m1)) .and. &
              ((ib2_ray0+n2c) >= -1) .and. ((ib2_ray0+n2c) <= (n2-m2))      ) .or. &
             (((ib1_ray0+n1c) >= -1) .and. ((ib1_ray0+n1c) <= (n1-m1)) .and. &
              ((ib2_ray0+m2c) >= -1) .and. ((ib2_ray0+m2c) <= (n2-m2))      ) .or. &
             (((ib1_ray0+n1c) >= -1) .and. ((ib1_ray0+n1c) <= (n1-m1)) .and. &
              ((ib2_ray0+n2c) >= -1) .and. ((ib2_ray0+n2c) <= (n2-m2))      )
    !
    if (count(flag_ray) >= N_radminpointsonray) then
      write(6,'(A,I2,A)') ' There are ', count(flag_ray), ' point(s) along ray in chunk'
      call flush(6)
      ! --- Ray has enough points ---


      m3rad=0
      do while (.not. flag_ray(m3rad+m3))
        m3rad=m3rad+1
      end do
      m3rad=max(m3rad-1,0)
!!$      m3rad=max(minval(ib3_ray0, mask=flag_ray)-1,0      )
      write(6,'(I3,A,2I4)') omp_get_thread_num(), ' m3, m3rad: ', m3, m3rad
      call flush(6)
      if (((ib1_ray0(m3rad+m3)+i1  ) < (     -2)) .or. &
          ((ib2_ray0(m3rad+m3)+i2  ) < (     -2)) .or. &
          ((ib1_ray0(m3rad+m3)+i1  ) > (n1-m1+2)) .or. &
          ((ib2_ray0(m3rad+m3)+i2  ) > (n2-m2+2))) m3rad=m3rad+1
      n3rad=n3-m3+1
      do while (.not. flag_ray(n3rad+m3))
        n3rad=n3rad-1
      end do
      n3rad=min(n3rad+1,n3-m3+1)
!!$      n3rad=min(maxval(ib3_ray0, mask=flag_ray)+1,n3-m3+1)
!!$      write(6,'(I3,A,4I4)') omp_get_thread_num(), ' m3, m3rad, n3, n3rad: ', m3, m3rad, n3, n3rad
!!$      call flush(6)
      if (((ib1_ray0(n3rad+m3)+i1  ) < (     -2)) .or. &
          ((ib2_ray0(n3rad+m3)+i2  ) < (     -2)) .or. &
          ((ib1_ray0(n3rad+m3)+i1  ) > (n1-m1+2)) .or. &
          ((ib2_ray0(n3rad+m3)+i2  ) > (n2-m2+2))) n3rad=n3rad-1
      n3rad=n3rad-1
!!$      write(6,'(I3,A,4I4)') omp_get_thread_num(), ' m3, m3rad, n3, n3rad: ', m3, m3rad, n3, n3rad
!!$      call flush(6)
      !
!!$      m3rad_min=min(m3rad_min, m3rad)
!!$      m3rad_max=max(m3rad_max, m3rad)
!!$      n3rad_min=min(n3rad_min, n3rad)
!!$      n3rad_max=max(n3rad_max, n3rad)
      !
      m3orad=m3rad+m3
      n3orad=n3rad+m3
      !
      ic1_0_ray(m3rad:n3rad)=min(max(ic1_ray0(m3orad:n3orad)+m1+i1  ,m1),n1)
      ic1_1_ray(m3rad:n3rad)=min(max(ic1_ray0(m3orad:n3orad)+m1+i1+1,m1),n1)
      ic2_0_ray(m3rad:n3rad)=min(max(ic2_ray0(m3orad:n3orad)+m2+i2  ,m2),n2)
      ic2_1_ray(m3rad:n3rad)=min(max(ic2_ray0(m3orad:n3orad)+m2+i2+1,m2),n2)
      !
!!$      ib1_min=min(ib1_min, minval(ib1_0_ray))
!!$      ib1_max=max(ib1_max, maxval(ib1_1_ray))
!!$      ib2_min=min(ib2_min, minval(ib2_0_ray))
!!$      ib2_max=max(ib2_max, maxval(ib2_1_ray))
      !
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: indices')
      call timing_start('RAD: dir: onto ray')
!#
#     endif
!#
      !
      write(6,'(I3,A,4I5)') omp_get_thread_num(), ' toray ', i1, i2, m3rad, n3rad
      call flush(6)
      ! --- Interpolate rho, T4, and logP on ray ---
      call rhd_rad3d_toray(action, &
                           m1,n1, m2,n2, m3,n3, &
                           m1org,n1org, m2org,n2org, m3org,n3org, &
                           m3rad, n3rad, &
                           ic1_0_ray, ic2_0_ray, ic1_1_ray, ic2_1_ray, &
                           wc_00_ray0, wc_01_ray0, wc_10_ray0, wc_11_ray0, &
                           rho, T4, logP, &
                           rho_ray, T4_ray, logP_ray)
      !
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: onto ray')
      call timing_start('RAD: dir: solve')
!#
#     endif
!#
      ! --- Solve radiative transfer equation for single ray ---
      if (par%N_radtaurefine <= 1) then
      write(6,'(I3,A,4I5)') omp_get_thread_num(), ' solveeq ', i1, i2, m3rad, n3rad
      call flush(6)
        call rhd_rad3d_solveeq(n3-m3+1, m3rad, n3rad, rho_ray, T4_ray, logP_ray, &
                               action, &
                               dx3, mu, &
                               Fradb, diffFradb)
      else
        call rhd_rad3d_solveadapt(rho_ray(m3rad:n3rad), T4_ray(m3rad:n3rad),logP_ray(m3rad:n3rad),&
                                  action, &
                                  dx3, mu, &
                                  Fradb(m3rad:n3rad+1), diffFradb(m3rad:n3rad))
      endif
      !
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: solve')
      call timing_start('RAD: dir: back to HD')
!#
#     endif
!#
      ! --- Flux normalization ---
      Fradb(    m3rad:n3rad+1)=Fradb(    m3rad:n3rad+1) * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
      diffFradb(m3rad:n3rad  )=diffFradb(m3rad:n3rad  ) * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
      !
      ! --- Project radiative flux along ray onto HD grid ---
      write(6,'(I3,A,2E13.5)') omp_get_thread_num(), ' minmax(Fradb): ', &
        minval(Fradb(m3rad:n3rad+1)), maxval(Fradb(m3rad:n3rad+1))     
      write(6,'(I3,A,4I5)') omp_get_thread_num(), ' fromray ', i1, i2, m3rad, n3rad
      call flush(6)
      call rhd_rad3d_fromray(action, flux_flag, &
                             m1,n1, m2,n2, m3,n3, &
                             m1org,n1org, m2org,n2org, m3org,n3org, &
                             m3rad, n3rad, &
                             dx3, Fradb, diffFradb, &
                             m1+i1, m2+i2, ib1_ray0, ib2_ray0, if1_ray0, if2_ray0, &
                             fluxfraction10, fluxfraction20, fluxfraction30, sumfluxfraction, &
                             emission, &
                             Frad1, Frad2, Frad3, drhoeidtime)
      !
      write(6,'(I3,A)') omp_get_thread_num(), ' fromray done'
      call flush(6)
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: back to HD')
!#
#     endif
!#
    else
!!$      write(6,'(A,I2,A)') ' Only ', count(flag_ray), ' point(s) on ray'
!!$      call flush(6)
    endif
  end do ! i2
  !
  endif
  !
 end do ! i1_base
 !$OMP END DO
!!$      write(6,'(I3,A)') omp_get_thread_num(), ' END DO done'
!!$      call flush(6)
!$OMP END PARALLEL
!!$      write(6,'(A)') 'END PARALLEL done'
!!$      call flush(6)
end do ! i1_offset
!!$      write(6,'(A)') 'end do ! i1_offset  done'
!!$      call flush(6)
!
!
!!$write(6,'(A,2I5)') &
!!$        'ib1_min/max:    ', ib1_min, ib1_max
!!$write(6,'(A,2I5)') &
!!$        'ib2_min/max:    ', ib2_min, ib2_max
!!$write(6,'(A,4I5)') &
!!$        'm/n3rad_min/max:', m3rad_min, m3rad_max, n3rad_min, n3rad_max
!
!!$! --- Additional test output ---
!!$write(6,'(A)') 'Write into rhd_rad3d_dir_test.out'
!!$include 'rhd_rad3d_dir_testout.f90'
!
!#
# ifdef rhd_rad3d_dir_t01 
!#
call timing_stop('RAD: dir')
!#
# endif
!#
end subroutine rhd_rad3d_dir

!#
# else /* rhd_r01_new */
!#

!----------*************---------------------------------------------------------------------------
subroutine rhd_rad3d_dir(action, flux_flag, r_dir, n_dir, &
                         m1,n1, m2,n2, m3,n3, m1org,n1org, m2org,n2org, m3org,n3org, &
                         xc1,xc2,xc3, xb1,xb2,xb3, &
                         rho, T4, logP, kappa, emission, &
                         Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_dir ('rhd_radiation_3_dimension_direction')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   action: ('action') derived type with collection of control parameters
!
! INPUT/OUTPUT:
!
! OUTPUT:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_rad3d_projectarea: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_projectonplane: Project point along given direction on plane
!   rhd_rad3d_solve:       Perform radiative energy transfer step for single ray
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1999-12-03 (B.F. Copenhagen) Put into own routine
!   1999-12-10 (B.F.) Definition of ib1,2_ray0 modified
!   1999-12-14 (B.F.) Corrected sign for fluxfraction10,20
!   2000-02-22 (B.F.) Modifications for wc12_ray0
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2000-05-29 (B.F.) Limit T_minimum with T_bilinear
!   2000-05-30 (B.F.) Interpolate between T_bilinear and T_minimum
!   2000-11-21 (B.F. Uppsala) Interpolate drhoeidtime from ray to HD grid
!   2000-11-22 (B.F.) Flux divergence correction
!   2000-11-27 (B.F.) Correction in formula for sumfluxfraction
!   2000-12-28 (B.F.) Allocation of ray vectors outside of loops
!   2001-01-09 (B.F.) Pointers => standard arrays
!   2001-03-09 (B.F.) OpenMP directives
!--------------------------------------------------------------------------------------------------
use opta_module
use rhd_action_module
use rhd_gl_module
!#
# ifdef rhd_rad3d_dir_t01 
!#
use timing_module
!#
# endif
!#
!use uio_filedef_module
!use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),        intent(in)    :: action
logical,                  intent(in)    :: flux_flag
real, dimension(3),       intent(in)    :: r_dir
integer,                  intent(in)    :: n_dir, m1,n1, m2,n2, m3,n3, &
                                           m1org,n1org, m2org,n2org, m3org,n3org
real, dimension(m1:n1  ), intent(in)    :: xc1
real, dimension(m2:n2  ), intent(in)    :: xc2
real, dimension(m3:n3  ), intent(in)    :: xc3
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1org  :n1org, &
                m2org  :n2org, &
                m3org  :n3org ), &
                          intent(in)    :: rho, T4, logP, kappa
real, dimension(m1org-3:n1org+3, &
                m2org-3:n2org+3, &
                m3org-3:n3org+3), &
                          intent(in)    :: emission
real, dimension(m1org-3:n1org+3+1, &
                m2org-3:n2org+3  , &
                m3org-3:n3org+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1org-3:n1org+3  , &
                m2org-3:n2org+3+1, &
                m3org-3:n3org+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1org-3:n1org+3  , &
                m2org-3:n2org+3  , &
                m3org-3:n3org+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1org-3:n1org+3, &
                m2org-3:n2org+3, &
                m3org-3:n3org+3), &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
real (kind=srk)                         :: dx1, dx2, dx3, dx1sub, dx2sub, &
                                           x1, x1min, x1max, x2, x2min, x2max, x3, &
                                           dxeps, slope, mu
real, dimension(3)                      :: r_dir0
real (kind=srk), dimension(3)           :: c000, c100, c010, c110, c001, c101, c011, c111, &
                                           cp001, cp101, cp011, cp111, &
                                           p00, p01, p10, p11
!!$                                           cp000, cp100, cp010, cp110, &
real (kind=srk), dimension(m3:n3)       :: xc1_ray0, xc2_ray0, &
                                           xc3_ray, &
                                           wc1_ray0, wc2_ray0, &
                                           wc_00_ray0, wc_01_ray0, &
                                           wc_10_ray0, wc_11_ray0
real (kind=srk), dimension(m3:n3+1)     :: xb1_ray0, xb2_ray0
real (kind=srk), dimension(m3:n3  ,2,3) :: fluxfraction10
real (kind=srk), dimension(m3:n3  ,3,2) :: fluxfraction20
real (kind=srk), dimension(m3:n3+1,2,2) :: fluxfraction30
real (kind=srk), dimension(m3:n3  ,3,3) :: sumfluxfraction
real (kind=srk)                         :: ffdown, ffup
logical,         dimension(m3:n3+1)     :: flag_ray
integer,         dimension(m3:n3  )     :: ic1_ray0 , ic2_ray0, &
                                           if1_ray0 , if2_ray0
integer,         dimension(m3:n3+1)     :: ib1_ray0 , ib2_ray0, ib3_ray0
real (kind=srk), dimension(0:n3-m3)     :: rho_ray, T4_ray, logP_ray
real (kind=drk), dimension(0:n3-m3  )   :: diffFradb
real (kind=drk), dimension(0:n3-m3+1)   :: Fradb
integer,         dimension(0:n3-m3)     :: ic1_0_ray, ic1_1_ray, &
                                           ic2_0_ray, ic2_1_ray
integer                                 :: i, &
                                           m1rad, n1rad, m2rad, n2rad, m3rad, n3rad, &
                                           i1, i2, i3, i1s, i2s, i1_base, i1_offset, &
                                           i1min, i2min, &
                                           m3orad, n3orad
integer                                 :: m3rad_min, m3rad_max, n3rad_min, n3rad_max, &
                                           ib1_min, ib1_max, ib2_min, ib2_max
integer, parameter                      :: N_radraymult=1, &
                                           N_radminpointsonray=5
integer                                 :: omp_get_thread_num
!--------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_rad3d_dir_t01 
!#
call timing_start('RAD: dir')
call timing_start('RAD: dir: init')
!#
# endif
!#
!
! --- Typical grid size: grid assumed to be equidistant ---
dx1   =(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
dx2   =(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
dx3   =(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
!
dx1sub=dx1/real(N_radraymult)
dx2sub=dx2/real(N_radraymult)
!
dxeps=1.0E-03 * min(dx1,dx2,dx3)
!
! --- Corners ---
c000=(/ xb1(m1  ), xb2(m2  ), xb3(m3  ) /)
c001=(/ xb1(m1  ), xb2(m2  ), xb3(n3+1) /)
c010=(/ xb1(m1  ), xb2(n2+1), xb3(m3  ) /)
c011=(/ xb1(m1  ), xb2(n2+1), xb3(n3+1) /)
c100=(/ xb1(n1+1), xb2(m2  ), xb3(m3  ) /)
c101=(/ xb1(n1+1), xb2(m2  ), xb3(n3+1) /)
c110=(/ xb1(n1+1), xb2(n2+1), xb3(m3  ) /)
c111=(/ xb1(n1+1), xb2(n2+1), xb3(n3+1) /)
!
! --- Normalization ---
r_dir0=sign(1.0,r_dir(3))*r_dir
mu=r_dir0(3)
!
! --- Relative coordinates of ray ---
do i3=m3,n3+1
  xb1_ray0(i3)=(xb3(i3)-xb3(m3)) * (r_dir0(1)/r_dir0(3))
  xb2_ray0(i3)=(xb3(i3)-xb3(m3)) * (r_dir0(2)/r_dir0(3))
!!$  xb3_ray( i3)= xb3(i3)
end do ! i3
do i3=m3,n3
  xc1_ray0(i3)=(xc3(i3)-xb3(m3)) * (r_dir0(1)/r_dir0(3))
  xc2_ray0(i3)=(xc3(i3)-xb3(m3)) * (r_dir0(2)/r_dir0(3))
  xc3_ray( i3)= xc3(i3)
end do ! i3
!
! --- Integer indices and weights of 4 neighbors ---          
ic1_ray0 =floor(   xc1_ray0/dx1-0.5)
wc1_ray0 =1.0_srk-(xc1_ray0/dx1-0.5-real(ic1_ray0))
ic2_ray0 =floor(   xc2_ray0/dx2-0.5)
wc2_ray0 =1.0_srk-(xc2_ray0/dx2-0.5-real(ic2_ray0))
wc_00_ray0=         wc1_ray0 *         wc2_ray0
wc_10_ray0=(1.0_srk-wc1_ray0)*         wc2_ray0
wc_01_ray0=         wc1_ray0 *(1.0_srk-wc2_ray0)
wc_11_ray0=(1.0_srk-wc1_ray0)*(1.0_srk-wc2_ray0)
!
ib1_ray0 =floor((xb1_ray0-0.5*dx1sub)/dx1)
ib2_ray0 =floor((xb2_ray0-0.5*dx2sub)/dx2)
ib3_ray0 =(/(i3,i3=0,n3-m3+1)/)
do i3=m3,n3+1
  do i1s=1,2
    do i2s=1,2
      call rhd_rad3d_projectarea( &
             (ib1_ray0(i3)+i1s-1) *dx1, (ib2_ray0(i3)+i2s-1) *dx2, &
             (ib1_ray0(i3)+i1s  ) *dx1, (ib2_ray0(i3)+i2s  ) *dx2, &
              xb1_ray0(i3)-0.5*dx1sub ,  xb2_ray0(i3)-0.5*dx2sub, &
              xb1_ray0(i3)+0.5*dx1sub ,  xb1_ray0(i3)-0.5*dx1sub, &
                                         xb2_ray0(i3)+0.5*dx2sub, &
              fluxfraction30(i3,i1s,i2s))
    end do ! i2s
  end do ! i1s
end do ! i3
!
do i3=m3,n3
  i1min=min(ib1_ray0(i3),ib1_ray0(i3+1))
  i2min=min(ib2_ray0(i3),ib2_ray0(i3+1))
  if1_ray0(i3)=i1min
  if2_ray0(i3)=i2min
  if (abs(r_dir0(1)) > 1.0E-05) then
    do i1s=1,2
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p01)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p00)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p11)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i1min+i1s  )*dx1, 1, p10)
      do i2s=1,3
        call rhd_rad3d_projectarea( &
               (i2min+i2s-1)*dx2, xb3(i3), (i2min+i2s)*dx2,            xb3(i3+1), &
                p00(2),           p00(3),   p10(2),           p01(2),  p01(3), &
                fluxfraction10(i3,i1s,i2s))
      end do ! i2s
    end do ! i1s
    if (r_dir0(1) < 0.0) fluxfraction10(i3,:,:)=-fluxfraction10(i3,:,:)
  else
    fluxfraction10(i3,:,:)=0.0
  endif
  if (abs(r_dir0(2)) > 1.0E-05) then
    do i2s=1,2
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p11)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p01)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p10)
      call rhd_rad3d_projectonplane( &
             (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/), &
             r_dir0, (i2min+i2s  )*dx2, 2, p00)

!!$    if (i3 < (m3+5)) then
!!$      write(6,'(A,4I4)') 'i3, i1min, i2min, i2s: ', i3, i1min, i2min, i2s
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p00: ', (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p00/dx1
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p01: ', (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)-0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p01/dx1
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p10: ', (/xb1_ray0(i3)-0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p10/dx1
!!$      write(6,'(A,3E12.5,A,3E12.5)') &
!!$        'p11: ', (/xb1_ray0(i3)+0.5*dx1sub, xb2_ray0(i3)+0.5*dx2sub, xb3(i3)/)/dx1, &
!!$        ' -> ', p11/dx1
!!$    endif

      do i1s=1,3
        call rhd_rad3d_projectarea( &
               (i1min+i1s-1)*dx1, xb3(i3), (i1min+i1s)*dx1,            xb3(i3+1), &
               p00(1),           p00(3),   p10(1),           p01(1),  p01(3), &
                fluxfraction20(i3,i1s,i2s))
      end do ! i1s
    end do ! i2s
    if (r_dir0(2) < 0.0) fluxfraction20(i3,:,:)=-fluxfraction20(i3,:,:)
  else
    fluxfraction20(i3,:,:)=0.0
  endif
end do ! i3
!

! --- Compute measure for relative volume fraction ---
do i1s=1,3
  do i2s=1,3
    do i3=m3,n3
      i1=i1s-(ib1_ray0(i3)-if1_ray0(i3))
      if ((i1 < 1) .or. (i1 > 2)) then
        ffdown=0.0
      else
        i2=i2s-(ib2_ray0(i3)-if2_ray0(i3))
        if ((i2 < 1) .or. (i2 > 2)) then
          ffdown=0.0
        else
          ffdown=fluxfraction30(i3,i1,i2)
        endif
      endif
      !
      i1=i1s-(ib1_ray0(i3+1)-if1_ray0(i3))
      if ((i1 < 1) .or. (i1 > 2)) then
        ffup=0.0
      else
        i2=i2s-(ib2_ray0(i3+1)-if2_ray0(i3))
        if ((i2 < 1) .or. (i2 > 2)) then
          ffup=0.0
        else
          ffup=fluxfraction30(i3+1,i1,i2)
        endif
      endif
      !
      sumfluxfraction(i3,i1s,i2s)=0.5*(ffup+ffdown)
      ! write(6,'(I3,2X,4I4,2X,3F8.4)') &
      !   i3, i1s, i2s, i1, i2, ffdown, ffup, sumfluxfraction(i3,i1s,i2s)
    end do ! i3
  end do ! i2s
end do ! i1s
!
!!$do i3=m3,n3
!!$  write(6,'(I3,2X,2I4,2X,2I4)')     i3, ib1_ray0(i3), if1_ray0(i3), ib2_ray0(i3), if2_ray0(i3)
!!$  write(6,'(5X,9F8.4,2X,F8.4)')     sumfluxfraction(i3,:,:), sum(sumfluxfraction(i3,:,:))
!!$  write(6,'(5X,6F8.4,24X,2X,F8.4)') fluxfraction10(i3,:,:) , sum(fluxfraction10(i3,:,:))
!!$  write(6,'(5X,6F8.4,24X,2X,F8.4)') fluxfraction20(i3,:,:) , sum(fluxfraction20(i3,:,:))
!!$  write(6,'(5X,4F8.4,40X,2X,F8.4)') fluxfraction30(i3,:,:) , sum(fluxfraction30(i3,:,:))
!!$end do ! i3
!
! --- Projection of corners onto x12 plane ---
call rhd_rad3d_projectonplane(c001, r_dir0, c000(3), 3, cp001)
call rhd_rad3d_projectonplane(c011, r_dir0, c000(3), 3, cp011)
call rhd_rad3d_projectonplane(c101, r_dir0, c000(3), 3, cp101)
call rhd_rad3d_projectonplane(c111, r_dir0, c000(3), 3, cp111)
!!$call rhd_rad3d_projectonplane(c000, r_dir0, c000(3), 3, cp000)
!!$call rhd_rad3d_projectonplane(c010, r_dir0, c000(3), 3, cp010)
!!$call rhd_rad3d_projectonplane(c100, r_dir0, c000(3), 3, cp100)
!!$call rhd_rad3d_projectonplane(c110, r_dir0, c000(3), 3, cp110)
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c000:', c000, ' ->', cp000
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c100:', c100, ' ->', cp100
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c010:', c010, ' ->', cp010
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c110:', c110, ' ->', cp110
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c001:', c001, ' ->', cp001
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c101:', c101, ' ->', cp101
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c011:', c011, ' ->', cp011
!!$write(6,'(A,3E12.4,A,3E12.4)') 'c111:', c111, ' ->', cp111
!
! --- Range in x1 direction of foot points and foot point indices for rays ---
x1min=min(c000(1),cp001(1))
x1max=max(c100(1),cp101(1))
m1rad=int((x1min-c000(1))/dx1sub)
n1rad=int((x1max-c000(1))/dx1sub)
!write(6,'(A,2E12.5,A,2I5)') 'x1min, x1max: ', x1min, x1max, '   m1rad, n1rad:', m1rad, n1rad
!
if (abs(cp001(1)-c000(1)) > dxeps) then
  slope=(cp001(2)-c000(2))/(cp001(1)-c000(1))
endif
!
!!$ib1_min=n3-m3+100
!!$ib1_max=-100
!!$ib2_min=n3-m3+100
!!$ib2_max=-100
!!$m3rad_min=n3-m3+100
!!$m3rad_max=-100
!!$n3rad_min=n3-m3+100
!!$n3rad_max=-100
!
!
!#
# ifdef rhd_rad3d_dir_t01 
!#
call timing_stop('RAD: dir: init')
!#
# endif
!#
do i1_offset=0,2
!$OMP PARALLEL DEFAULT(SHARED), &
!$OMP   PRIVATE(i1, i2, x1, x2min, x2max, m2rad, n2rad, flag_ray, m3rad, n3rad, m3orad, n3orad, &
!$OMP           ic1_0_ray, ic1_1_ray, ic2_0_ray, ic2_1_ray, rho_ray, T4_ray, logP_ray, &
!$OMP           Fradb, diffFradb), &
!$OMP  	FIRSTPRIVATE(dx3, mu, &
!$OMP                ic1_ray0, ic2_ray0, ib1_ray0, ib2_ray0, &
!$OMP                wc_00_ray0, wc_01_ray0, wc_10_ray0, wc_11_ray0, &
!$OMP                fluxfraction10, fluxfraction20, fluxfraction30, sumfluxfraction)
 !$OMP DO
 do i1_base=m1rad,n1rad,3
  i1=i1_base+i1_offset
  !
  if (i1 <= n1rad) then

!!$  write(6,'(I3,A,3I5)') omp_get_thread_num(), ' i1_base, i1_offset, n1rad: ', &
!!$    i1_base, i1_offset, n1rad
!!$  call flush(6)
  !
  x1        =i1*dx1sub + c000(1)
  !
  ! --- Range in x2 direction of foot points and foot point indices for rays ---
  ! --- First guess ---
  x2min=min(c000(2), cp001(2))
  x2max=max(c010(2), cp011(2))
  if (abs(cp001(1)-c000(1)) > dxeps) then
    ! --- Correction: projection of vertical edge ---
    if (slope > 0.0) then
      x2min=max(x2min, c100(2) + slope*(x1-c100(1)))
      x2max=min(x2max, c010(2) + slope*(x1-c010(1)))
    else
      x2min=max(x2min, c000(2) + slope*(x1-c000(1)))
      x2max=min(x2max, c110(2) + slope*(x1-c110(1)))
    endif
  endif
  m2rad=int((x2min-c000(2))/dx2sub)
  n2rad=int((x2max-c000(2))/dx2sub)
!!$  write(6,'(I3,A,2E12.5,A,2I5)') omp_get_thread_num(), ' x2min, x2max: ', &
!!$    x2min, x2max, '   m2rad, n2rad:', m2rad, n2rad
!!$  call flush(6)
  !
  do i2=m2rad,n2rad
    !
!!$    write(6,'(A,4I5)') 'i1,i2,m2rad,n2rad: ', i1,i2,m2rad,n2rad
!!$    call flush(6)
    flag_ray=(((ib1_ray0+i1  ) >= (     -1)) .and. &
              ((ib1_ray0+i1  ) <= (n1-m1  )) .and. &
              ((ib2_ray0+i2  ) >= (     -1)) .and. &
              ((ib2_ray0+i2  ) <= (n2-m2  )))
    !
    if (count(flag_ray) >= N_radminpointsonray) then
!!$      write(6,'(A,I2,A)') ' EXACTLY ', count(flag_ray), ' point(s) on ray'
!!$      call flush(6)
      ! --- Ray has enough points ---
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_start('RAD: dir: indices')
!#
#     endif
!#
      m3rad=0
      do while (.not. flag_ray(m3rad+m3))
        m3rad=m3rad+1
      end do
      m3rad=max(m3rad-1,0)
!!$      m3rad=max(minval(ib3_ray0, mask=flag_ray)-1,0      )
!!$      write(6,'(I3,A,2I4)') omp_get_thread_num(), ' m3, m3rad: ', m3, m3rad
!!$      call flush(6)
      if (((ib1_ray0(m3rad+m3)+i1  ) < (     -2)) .or. &
          ((ib2_ray0(m3rad+m3)+i2  ) < (     -2)) .or. &
          ((ib1_ray0(m3rad+m3)+i1  ) > (n1-m1+2)) .or. &
          ((ib2_ray0(m3rad+m3)+i2  ) > (n2-m2+2))) m3rad=m3rad+1
      n3rad=n3-m3+1
      do while (.not. flag_ray(n3rad+m3))
        n3rad=n3rad-1
      end do
      n3rad=min(n3rad+1,n3-m3+1)
!!$      n3rad=min(maxval(ib3_ray0, mask=flag_ray)+1,n3-m3+1)
!!$      write(6,'(I3,A,4I4)') omp_get_thread_num(), ' m3, m3rad, n3, n3rad: ', m3, m3rad, n3, n3rad
!!$      call flush(6)
      if (((ib1_ray0(n3rad+m3)+i1  ) < (     -2)) .or. &
          ((ib2_ray0(n3rad+m3)+i2  ) < (     -2)) .or. &
          ((ib1_ray0(n3rad+m3)+i1  ) > (n1-m1+2)) .or. &
          ((ib2_ray0(n3rad+m3)+i2  ) > (n2-m2+2))) n3rad=n3rad-1
      n3rad=n3rad-1
!!$      write(6,'(I3,A,4I4)') omp_get_thread_num(), ' m3, m3rad, n3, n3rad: ', m3, m3rad, n3, n3rad
!!$      call flush(6)
      !
!!$      m3rad_min=min(m3rad_min, m3rad)
!!$      m3rad_max=max(m3rad_max, m3rad)
!!$      n3rad_min=min(n3rad_min, n3rad)
!!$      n3rad_max=max(n3rad_max, n3rad)
      !
      m3orad=m3rad+m3
      n3orad=n3rad+m3
      !
      ic1_0_ray(m3rad:n3rad)=min(max(ic1_ray0(m3orad:n3orad)+m1+i1  ,m1),n1)
      ic1_1_ray(m3rad:n3rad)=min(max(ic1_ray0(m3orad:n3orad)+m1+i1+1,m1),n1)
      ic2_0_ray(m3rad:n3rad)=min(max(ic2_ray0(m3orad:n3orad)+m2+i2  ,m2),n2)
      ic2_1_ray(m3rad:n3rad)=min(max(ic2_ray0(m3orad:n3orad)+m2+i2+1,m2),n2)
      !
!!$      ib1_min=min(ib1_min, minval(ib1_0_ray))
!!$      ib1_max=max(ib1_max, maxval(ib1_1_ray))
!!$      ib2_min=min(ib2_min, minval(ib2_0_ray))
!!$      ib2_max=max(ib2_max, maxval(ib2_1_ray))
      !
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: indices')
      call timing_start('RAD: dir: onto ray')
!#
#     endif
!#
      !
!!$        write(6,'(I3,A,4I5)') omp_get_thread_num(), ' toray ', i1, i2, m3rad, n3rad
!!$        call flush(6)
      ! --- Interpolate rho, T4, and logP on ray ---
      call rhd_rad3d_toray(action, &
                           m1,n1, m2,n2, m3,n3, &
                           m1org,n1org, m2org,n2org, m3org,n3org, &
                           m3rad, n3rad, &
                           ic1_0_ray, ic2_0_ray, ic1_1_ray, ic2_1_ray, &
                           wc_00_ray0, wc_01_ray0, wc_10_ray0, wc_11_ray0, &
                           rho, T4, logP, &
                           rho_ray, T4_ray, logP_ray)
      !
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: onto ray')
      call timing_start('RAD: dir: solve')
!#
#     endif
!#
      ! --- Solve radiative transfer equation for single ray ---
      if (par%N_radtaurefine <= 1) then
!!$        if ((i1 == 69) .and. (i2 == 133) .and. (m3rad == 36) .and. (n3rad == 126)) then
!!$          write(6,'(I3,A,4I5)') omp_get_thread_num(), ' solveeq ', i1, i2, m3rad, n3rad
!!$          write(6,*) 'rho_ray:  ', rho_ray(m3rad:n3rad)
!!$          write(6,*) 'T4_ray:   ', T4_ray(m3rad:n3rad)
!!$          write(6,*) 'logP_ray: ', logP_ray(m3rad:n3rad)
!!$          call flush(6)
!!$          printamount=1
!!$        else
!!$          printamount=0
!!$        endif
        call rhd_rad3d_solveeq(n3-m3+1, m3rad, n3rad, rho_ray, T4_ray, logP_ray, &
                               action, &
                               dx3, mu, &
                               Fradb, diffFradb)
      else
        call rhd_rad3d_solveadapt(rho_ray(m3rad:n3rad), T4_ray(m3rad:n3rad),logP_ray(m3rad:n3rad),&
                                  action, &
                                  dx3, mu, &
                                  Fradb(m3rad:n3rad+1), diffFradb(m3rad:n3rad))
      endif
      !
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: solve')
      call timing_start('RAD: dir: back to HD')
!#
#     endif
!#
      ! --- Flux normalization ---
      Fradb(    m3rad:n3rad+1)=Fradb(    m3rad:n3rad+1) * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
      diffFradb(m3rad:n3rad  )=diffFradb(m3rad:n3rad  ) * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
      !
      ! --- Project radiative flux along ray onto HD grid ---
!!$      if ((i1 == 69) .and. (i2 == 133) .and. (m3rad == 0) .and. (n3rad == 145)) then
!!$        write(6,'(I3,A,2E13.5)') omp_get_thread_num(), ' minmax(Fradb): ', &
!!$          minval(Fradb(m3rad:n3rad+1)), maxval(Fradb(m3rad:n3rad+1))     
!!$        write(6,'(I3,A,4I5)') omp_get_thread_num(), ' fromray ', i1, i2, m3rad, n3rad
!!$        call flush(6)
!!$      endif
      call rhd_rad3d_fromray(action, flux_flag, &
                             m1,n1, m2,n2, m3,n3, &
                             m1org,n1org, m2org,n2org, m3org,n3org, &
                             m3rad, n3rad, &
                             dx3, Fradb, diffFradb, &
                             m1+i1, m2+i2, ib1_ray0, ib2_ray0, if1_ray0, if2_ray0, &
                             fluxfraction10, fluxfraction20, fluxfraction30, sumfluxfraction, &
                             emission, &
                             Frad1, Frad2, Frad3, drhoeidtime)
      !
!!$      write(6,'(I3,A)') omp_get_thread_num(), ' fromray done'
!!$      call flush(6)
!#
#     ifdef rhd_rad3d_dir_t01 
!#
      call timing_stop('RAD: dir: back to HD')
!#
#     endif
!#
    else
!!$      write(6,'(A,I2,A)') ' Only ', count(flag_ray), ' point(s) on ray'
!!$      call flush(6)
    endif
  end do ! i2
  !
  endif
  !
 end do ! i1_base
 !$OMP END DO
!!$      write(6,'(I3,A)') omp_get_thread_num(), ' END DO done'
!!$      call flush(6)
!$OMP END PARALLEL
!!$      write(6,'(A)') 'END PARALLEL done'
!!$      call flush(6)
end do ! i1_offset
!!$      write(6,'(A)') 'end do ! i1_offset  done'
!!$      call flush(6)
!
!
!!$write(6,'(A,2I5)') &
!!$        'ib1_min/max:    ', ib1_min, ib1_max
!!$write(6,'(A,2I5)') &
!!$        'ib2_min/max:    ', ib2_min, ib2_max
!!$write(6,'(A,4I5)') &
!!$        'm/n3rad_min/max:', m3rad_min, m3rad_max, n3rad_min, n3rad_max
!
!!$! --- Additional test output ---
!!$write(6,'(A)') 'Write into rhd_rad3d_dir_test.out'
!!$include 'rhd_rad3d_dir_testout.f90'
!
!#
# ifdef rhd_rad3d_dir_t01 
!#
call timing_stop('RAD: dir')
!#
# endif
!#
end subroutine rhd_rad3d_dir

!#
# endif /* rhd_r01_new */
!#

!#
# ifdef rhd_rad3d_r02
!#
!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_dirper(action, r_dir, w_dir, &
                            n_dir, m1,n1, m2,n2, m3,n3, m3a,n3a, &
                            xc1,xc2,xc3, xb1,xb2,xb3, &
                            rho, T, logP, kappa, emission, &
                            Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_dirper ('rhd_radiation_3_dimension_direction_periodic')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field for periodic boundary conditions.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   action: ('action') derived type with collection of control parameters
!
! INPUT/OUTPUT:
!
! OUTPUT:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_rad3d_projectarea: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_projectonplane: Project point along given direction on plane
!   rhd_rad3d_solve:       Perform radiative energy transfer step for single ray
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   2000-02-23 (B.F. Copenhagen) Derived from rhd_rad3d_dir
!   2000-02-25 (B.F.) Extra case for 1D
!   2000-02-29 (B.F. Uppsala) Parameter w_dir added
!   2000-03-19 (B.F. Copenhagen) Vectorization of projection
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2000-04-01 (B.F.) Parameters m3a,n3a added
!   2000-07-12 (B.F. Uppsala) Computation of if1_ray0 changed
!   2000-07-13 (B.F.) "Radiative shear fluxes" to improve Greens function
!   2000-11-27 (B.F.) Correction in formula for sumfluxfraction
!--------------------------------------------------------------------------------------------------
use opta_module
use rhd_action_module
use uio_filedef_module
use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),        intent(in)    :: action
real, dimension(3),       intent(in)    :: r_dir
real,                     intent(in)    :: w_dir
integer,                  intent(in)    :: n_dir, m1,n1, m2,n2, m3,n3, m3a,n3a
real, dimension(m1:n1  ), intent(in)    :: xc1
real, dimension(m2:n2  ), intent(in)    :: xc2
real, dimension(m3:n3  ), intent(in)    :: xc3
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1:n1,m2:n2,m3:n3), &
                          intent(in)    :: rho, T, logP, kappa
real, dimension(m1-3:n1+3, &
                m2-3:n2+3, &
                m3-3:n3+3), &
                          intent(in)    :: emission
real, dimension(m1-3:n1+3+1, &
                m2-3:n2+3  , &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3+1, &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3) , &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
real (kind=srk)                         :: dx1, dx2, dx3, dx1sub, dx2sub, &
                                           x1min, x1max, x2min, x2max, &
                                           dxeps, &
                                           T_bilinear, T_minimum
real, dimension(3)                      :: r_dir0
real (kind=srk), dimension(3)           :: c000, c100, c010
real (kind=srk), dimension(m3a:n3a,3)   :: p00, p01, p10, xin
real (kind=srk), dimension(m3a:n3a)     :: xc1_ray0, xc2_ray0, &
                                           xc3_ray, &
                                           wc1_ray0, wc2_ray0, &
                                           wc_00_ray0, wc_01_ray0, &
                                           wc_10_ray0, wc_11_ray0
real (kind=srk), dimension(m3a:n3a+1)   :: xb1_ray0, xb2_ray0
real (kind=srk), dimension(m3a:n3a  ,2,3) :: fluxfraction10
real (kind=srk), dimension(m3a:n3a  ,3,2) :: fluxfraction20
real (kind=srk), dimension(m3a:n3a+1,2,2) :: fluxfraction30
real (kind=srk), dimension(m3a:n3a  ,3,3) :: sumfluxfraction
real (kind=srk)                         :: ffdown, ffup, avgemission
real (kind=srk)                         :: shiftamount, shiftdiv, sumshiftdiv, &
                                           shiftflux11, shiftflux13, shiftflux31, shiftflux33
integer                                 :: i1o, i2o


integer,         dimension(m3a:n3a  )   :: ic1_ray0 , ic2_ray0, &
                                           if1_ray0 , if2_ray0
integer,         dimension(m3a:n3a+1)   :: ib1_ray0 , ib2_ray0, ib3_ray0
real (kind=srk), pointer, dimension(:)  :: rho_ray, T4_ray, logP_ray, &
                                           Fradb_ray, Fradc_ray, &
                                           diffFrad_ray, divFrad_ray
real (kind=drk), pointer, dimension(:)  :: Fradb
integer, pointer, dimension(:)          :: ic1_0_ray, ic1_1_ray, &
                                           ic2_0_ray, ic2_1_ray, &
                                           ib1_0_ray, ib1_1_ray, &
                                           ib2_0_ray, ib2_1_ray, &
                                           if11_0_ray, if11_1_ray, &
                                           if12_0_ray, if12_1_ray, if12_2_ray, &
                                           if21_0_ray, if21_1_ray, if21_2_ray, &
                                           if22_0_ray, if22_1_ray
integer                                 :: i, &
                                           m1rad, n1rad, m2rad, n2rad, m3rad, n3rad, &
                                           num1rad, num2rad, &
                                           i1, i2, i3, i1s, i2s, &
                                           i1min, i2min, &
                                           i3o, m3orad, n3orad, &
                                           nc
logical                                 :: only1D_flag, Tinter_bilinear_flag
integer, parameter                      :: N_radraymult=1
!--------------------------------------------------------------------------------------------------
!
Tinter_bilinear_flag=(par%C_radTintminfac <= 1.0)
!
! --- Test if model is one dimensional ---
only1D_flag=((m1 == n1) .and. (m2 == n2))
!
! --- Typical grid size: grid assumed to be equidistant ---
dx1   =(xb1(n1 +1)-xb1(m1 ))/real(n1 +1-m1 )
dx2   =(xb2(n2 +1)-xb2(m2 ))/real(n2 +1-m2 )
dx3   =(xb3(n3a+1)-xb3(m3a))/real(n3a+1-m3a)
!
dx1sub=dx1/real(N_radraymult)
dx2sub=dx2/real(N_radraymult)
!
dxeps=1.0E-03 * min(dx1,dx2,dx3)
!
! --- Corners ---
c000=(/ xb1(m1  ), xb2(m2  ), xb3(m3a ) /)
c010=(/ xb1(m1  ), xb2(n2+1), xb3(m3a ) /)
c100=(/ xb1(n1+1), xb2(m2  ), xb3(m3a ) /)
!
! --- Normalization ---
r_dir0=sign(1.0,r_dir(3))*r_dir
!
if (.not. only1D_flag) then
  ! --- Relative coordinates of ray ---
  do i3=m3a,n3a+1
    xb1_ray0(i3)=(xb3(i3)-xb3(m3a)) * (r_dir0(1)/r_dir0(3))
    xb2_ray0(i3)=(xb3(i3)-xb3(m3a)) * (r_dir0(2)/r_dir0(3))
  !!$  xb3_ray( i3)= xb3(i3)
  end do ! i3
  do i3=m3a,n3a
    xc1_ray0(i3)=(xc3(i3)-xb3(m3a)) * (r_dir0(1)/r_dir0(3))
    xc2_ray0(i3)=(xc3(i3)-xb3(m3a)) * (r_dir0(2)/r_dir0(3))
    xc3_ray( i3)= xc3(i3)
  end do ! i3
  !
  ! --- Integer indices and weights of 4 neighbors ---          
  ic1_ray0 =floor(   xc1_ray0/dx1-0.5)
  wc1_ray0 =1.0_srk-(xc1_ray0/dx1-0.5-real(ic1_ray0))
  ic2_ray0 =floor(   xc2_ray0/dx2-0.5)
  wc2_ray0 =1.0_srk-(xc2_ray0/dx2-0.5-real(ic2_ray0))
  wc_00_ray0=         wc1_ray0 *         wc2_ray0
  wc_10_ray0=(1.0_srk-wc1_ray0)*         wc2_ray0
  wc_01_ray0=         wc1_ray0 *(1.0_srk-wc2_ray0)
  wc_11_ray0=(1.0_srk-wc1_ray0)*(1.0_srk-wc2_ray0)
  !
  ib1_ray0 =floor((xb1_ray0-0.5*dx1sub)/dx1)
  ib2_ray0 =floor((xb2_ray0-0.5*dx2sub)/dx2)
  ib3_ray0 =(/(i3,i3=0,n3a-m3a+1)/)
  do i1s=1,2
    do i2s=1,2
      call rhd_rad3d_projectarea1( &
               (ib1_ray0+i1s-1) *dx1, (ib2_ray0+i2s-1) *dx2, &
               (ib1_ray0+i1s  ) *dx1, (ib2_ray0+i2s  ) *dx2, &
                xb1_ray0-0.5*dx1sub ,  xb2_ray0-0.5*dx2sub, &
                xb1_ray0+0.5*dx1sub ,  xb1_ray0-0.5*dx1sub, &
                                          xb2_ray0+0.5*dx2sub, &
                fluxfraction30(:,i1s,i2s))
    end do ! i2s
  end do ! i1s
  !

  if (ib1_ray0(n3a+1) > ib1_ray0(m3a)) then
    if1_ray0(m3a:n3a)=ib1_ray0(m3a  :n3a  )
  else
    if1_ray0(m3a:n3a)=ib1_ray0(m3a+1:n3a+1)
  endif
  if (ib2_ray0(n3a+1) > ib2_ray0(m3a)) then
    if2_ray0(m3a:n3a)=ib2_ray0(m3a  :n3a  )
  else
    if2_ray0(m3a:n3a)=ib2_ray0(m3a+1:n3a+1)
  endif
  !
  ! --- Compute measure for relative volume fraction ---
  do i1s=1,3
    do i2s=1,3
      do i3=m3a,n3a
        i1=i1s-(ib1_ray0(i3)-if1_ray0(i3))
        if ((i1 < 1) .or. (i1 > 2)) then
          ffdown=0.0
        else
          i2=i2s-(ib2_ray0(i3)-if2_ray0(i3))
          if ((i2 < 1) .or. (i2 > 2)) then
            ffdown=0.0
          else
            ffdown=fluxfraction30(i3,i1,i2)
          endif
        endif
        !
        i1=i1s-(ib1_ray0(i3+1)-if1_ray0(i3))
        if ((i1 < 1) .or. (i1 > 2)) then
          ffup=0.0
        else
          i2=i2s-(ib2_ray0(i3+1)-if2_ray0(i3))
          if ((i2 < 1) .or. (i2 > 2)) then
            ffup=0.0
          else
            ffup=fluxfraction30(i3+1,i1,i2)
          endif
        endif
        !
        sumfluxfraction(i3,i1s,i2s)=0.5*(ffup+ffdown)
      end do ! i3
    end do ! i2s
  end do ! i1s
  !
  if (n1 == m1) then
    ! --- Only 2D problem ---
    do i1s=2,3
      sumfluxfraction(:,1,:)=sumfluxfraction(:,1,:)+sumfluxfraction(:,i1s,:)
    end do ! i1s
    do i1s=2,3
      sumfluxfraction(:,i1s,:)=sumfluxfraction(:,1,:)
    end do ! i1s
  endif
  if (n2 == m2) then
    ! --- Only 2D problem ---
    do i2s=2,3
      sumfluxfraction(:,:,1)=sumfluxfraction(:,:,1)+sumfluxfraction(:,:,i2s)
    end do ! i2s
    do i2s=2,3
      sumfluxfraction(:,:,i2s)=sumfluxfraction(:,:,1)
    end do ! i2s
  endif

  !
  if (abs(r_dir0(1)) > 1.0E-05) then
    do i1s=1,2
      xin(:,1)=xb1_ray0(m3a:n3a)-0.5*dx1sub
      xin(:,2)=xb2_ray0(m3a:n3a)-0.5*dx2sub
      xin(:,3)=xb3(        m3a:n3a)
      call rhd_rad3d_projectonplane1( &
               xin, &
               r_dir0, (if1_ray0(m3a:n3a)+i1s  )*dx1, 1, p01(m3a:n3a,:))
      xin(:,1)=xb1_ray0(m3a:n3a)+0.5*dx1sub
      call rhd_rad3d_projectonplane1( &
               xin, &
               r_dir0, (if1_ray0(m3a:n3a)+i1s  )*dx1, 1, p00(m3a:n3a,:))
      xin(:,2)=xb2_ray0(m3a:n3a)+0.5*dx2sub
      call rhd_rad3d_projectonplane1( &
               xin, &
               r_dir0, (if1_ray0(m3a:n3a)+i1s  )*dx1, 1, p10(m3a:n3a,:))
      do i2s=1,3
        call rhd_rad3d_projectarea1( &
                 (if2_ray0+i2s-1)*dx2, xb3(m3a:n3a), (if2_ray0+i2s)*dx2,  xb3(m3a+1:n3a+1), &
                  p00(:,2),           p00(:,3),   p10(:,2),          p01(:,2),  p01(:,3), &
                  fluxfraction10(:,i1s,i2s))
      end do ! i2s
    end do ! i1s
    if (r_dir0(1) < 0.0) fluxfraction10=-fluxfraction10
  else
    fluxfraction10=0.0
  endif
  !
  if (abs(r_dir0(2)) > 1.0E-05) then
    do i2s=1,2
      xin(:,1)=xb1_ray0(m3a:n3a)+0.5*dx1sub
      xin(:,2)=xb2_ray0(m3a:n3a)-0.5*dx2sub
      xin(:,3)=xb3(        m3a:n3a)
      call rhd_rad3d_projectonplane1( &
               xin, &
               r_dir0, (if2_ray0(m3a:n3a)+i2s  )*dx2, 2, p01(m3a:n3a,:))
      xin(:,2)=xb2_ray0(m3a:n3a)+0.5*dx2sub
      call rhd_rad3d_projectonplane1( &
               xin, &
               r_dir0, (if2_ray0(m3a:n3a)+i2s  )*dx2, 2, p00(m3a:n3a,:))
      xin(:,1)=xb1_ray0(m3a:n3a)-0.5*dx1sub
      call rhd_rad3d_projectonplane1( &
               xin, &
               r_dir0, (if2_ray0(m3a:n3a)+i2s  )*dx2, 2, p10(m3a:n3a,:))
      do i1s=1,3
        call rhd_rad3d_projectarea1( &
                 (if1_ray0+i1s-1)*dx1, xb3(m3a:n3a), (if1_ray0+i1s)*dx1, xb3(m3a+1:n3a+1), &
                 p00(:,1),           p00(:,3),   p10(:,1),           p01(:,1), p01(:,3), &
                 fluxfraction20(:,i1s,i2s))
      end do ! i1s
    end do ! i2s
    if (r_dir0(2) < 0.0) fluxfraction20=-fluxfraction20
  else
    fluxfraction20=0.0
  endif
endif
!
! --- Range in x1 direction of foot points and foot point indices for rays ---
! --- Trivial for periodic boundaries: simply take whole grid           ---
x1min=c000(1)
x1max=c100(1)
m1rad=0
n1rad=int((x1max-c000(1))/dx1sub)-1
num1rad=n1rad-m1rad+1
!!$write(6,'(A,2E12.5,A,2I5)') 'x1min, x1max: ', x1min, x1max, '   m1rad, n1rad:', m1rad, n1rad
!
! --- Range in x2 direction of foot points and foot point indices for rays ---
! --- Trivial for periodic boundaries ---
x2min=c000(2)
x2max=c010(2)
m2rad=0
n2rad=int((x2max-c000(2))/dx2sub)-1
num2rad=n2rad-m2rad+1
!!$write(6,'(A,2E12.5,A,2I5)') 'x2min, x2max: ', x2min, x2max, '   m2rad, n2rad:', m2rad, n2rad
!
! --- Range in x2 direction of foot points and foot point indices for rays ---
! --- Trivial for periodic boundaries ---
m3rad=0
n3rad=n3a-m3a
m3orad=m3rad+m3a
n3orad=n3rad+m3a
!
! --- Integer indices and weights of 4 neighbors ---
allocate(ic1_0_ray(   m3rad:n3rad  ), ic2_0_ray(  m3rad:n3rad  ), &
         ic1_1_ray(   m3rad:n3rad  ), ic2_1_ray(  m3rad:n3rad  ), &
         ib1_0_ray(   m3rad:n3rad+1), ib2_0_ray(  m3rad:n3rad+1), &
         ib1_1_ray(   m3rad:n3rad+1), ib2_1_ray(  m3rad:n3rad+1), &
         if11_0_ray(  m3rad:n3rad  ), if11_1_ray( m3rad:n3rad  ), &
         if12_0_ray(  m3rad:n3rad  ), if12_1_ray( m3rad:n3rad  ), &
         if12_2_ray(  m3rad:n3rad  ), &
         if21_0_ray(  m3rad:n3rad  ), &
         rho_ray(     m3rad:n3rad  ), T4_ray(      m3rad:n3rad  ), &
         logP_ray(       m3rad:n3rad  ), &
         Fradb_ray(   m3rad:n3rad+1), Fradc_ray(  m3rad:n3rad  ), &
         diffFrad_ray(m3rad:n3rad  ), divFrad_ray(m3rad:n3rad  ), &
         Fradb(       m3rad:n3rad+1))
if21_1_ray=> if11_0_ray
if21_2_ray=> if11_1_ray
if22_0_ray=> if12_1_ray
if22_1_ray=> if12_2_ray
!
do i1=m1rad,n1rad
  do i2=m2rad,n2rad
    ! --- Compute actual indices, obey periodic boundary conditions ---
    !
    if (only1D_flag) then
      rho_ray=rho(m1,m2,m3a:n3a)
      T4_ray  =T(   m1,m2,m3a:n3a)
      logP_ray=logP(m1,m2,m3a:n3a)
    else
      ic1_0_ray= modulo((ic1_ray0+i1  ), num1rad) + m1
      ic1_1_ray= modulo((ic1_ray0+i1+1), num1rad) + m1
      ic2_0_ray= modulo((ic2_ray0+i2  ), num2rad) + m2
      ic2_1_ray= modulo((ic2_ray0+i2+1), num2rad) + m2
      !
      ib1_0_ray= modulo((ib1_ray0+i1  ), num1rad) + m1
      ib1_1_ray= modulo((ib1_ray0+i1+1), num1rad) + m1
      ib2_0_ray= modulo((ib2_ray0+i2  ), num2rad) + m2
      ib2_1_ray= modulo((ib2_ray0+i2+1), num2rad) + m2
      !
      if11_0_ray=modulo((if1_ray0+i1+1), num1rad) + m1
      if11_1_ray=modulo((if1_ray0+i1+2), num1rad) + m1
      if12_0_ray=modulo((if2_ray0+i2  ), num2rad) + m2
      if12_1_ray=modulo((if2_ray0+i2+1), num2rad) + m2
      if12_2_ray=modulo((if2_ray0+i2+2), num2rad) + m2
      !
      if21_0_ray=modulo((if1_ray0+i1), num1rad) + m1
      !
      ! --- Interpolate rho, T, and logP on ray; only action%dim=3 ---
      do i3=m3rad,n3rad
        i3o=i3+m3a
        rho_ray(i3)= &
          wc_00_ray0(i3o) * rho(ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
          wc_10_ray0(i3o) * rho(ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
          wc_01_ray0(i3o) * rho(ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
          wc_11_ray0(i3o) * rho(ic1_1_ray(i3), ic2_1_ray(i3), i3o)
      end do ! i3
      do i3=m3rad,n3rad
        i3o=i3+m3a
        logP_ray(i3)= &
          wc_00_ray0(i3o) * logP(  ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
          wc_10_ray0(i3o) * logP(  ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
          wc_01_ray0(i3o) * logP(  ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
          wc_11_ray0(i3o) * logP(  ic1_1_ray(i3), ic2_1_ray(i3), i3o)
      end do ! i3
      if (Tinter_bilinear_flag) then
        do i3=m3rad,n3rad
          i3o=i3+m3a
          T4_ray(i3)= &
            wc_00_ray0(i3o) * T(  ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
            wc_10_ray0(i3o) * T(  ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
            wc_01_ray0(I3o) * T(  ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
            wc_11_ray0(i3o) * T(  ic1_1_ray(i3), ic2_1_ray(i3), i3o)
        end do ! i3
      else
        do i3=m3rad,n3rad
          i3o=i3+m3a
          T_bilinear=wc_00_ray0(i3o) * T(  ic1_0_ray(i3), ic2_0_ray(i3), i3o) + &
                     wc_10_ray0(i3o) * T(  ic1_1_ray(i3), ic2_0_ray(i3), i3o) + &
                     wc_01_ray0(I3o) * T(  ic1_0_ray(i3), ic2_1_ray(i3), i3o) + &
                     wc_11_ray0(i3o) * T(  ic1_1_ray(i3), ic2_1_ray(i3), i3o)
          T_minimum=min(T(ic1_0_ray(i3), ic2_0_ray(i3), i3o), &
                        T(ic1_1_ray(i3), ic2_0_ray(i3), i3o), &
                        T(ic1_0_ray(i3), ic2_1_ray(i3), i3o), &
                        T(ic1_1_ray(i3), ic2_1_ray(i3), i3o) )
          if (T_bilinear <= (par%C_radTintminfac*T_minimum)) then
            T4_ray(i3)=T_bilinear
          else
            T4_ray(i3)=(T_bilinear-par%C_radTintminfac*T_minimum)/ &
                           (1.0+(T_bilinear/T_minimum-par%C_radTintminfac)/ &
                                (2.0*(par%C_radTintminfac-1.0))) + &
                         par%C_radTintminfac*T_minimum
          endif
        end do ! i3
      endif
    endif
    !
    ! --- Solve radiative transfer equation for single ray ---
    if (par%N_radtaurefine <= 1) then
      call rhd_rad3d_solveeq(n3-m3+1, m3rad, n3rad, rho_ray, T4_ray, logP_ray, &
                             action, &
                             dx3, mu, &
                             Fradb)
    else
      call rhd_rad3d_solveadapt(rho_ray(m3rad:n3rad), T4_ray(m3rad:n3rad),logP_ray(m3rad:n3rad),&
                                action, &
                                dx3, r_dir0(3), &
                                Fradb(m3rad:n3rad+1))
    endif
    !
    ! --- Flux normalization ---
    Fradb=Fradb * (w_dir*3.0_drk/real(max(n_dir,3), kind=drk)*r_dir0(3))
    !
    Fradb_ray=real(Fradb, kind=srk)
    !
    ! --- Flux divergence * (-dx)
    diffFrad_ray=real(Fradb(m3rad+1:n3rad+1) - &
                      Fradb(m3rad  :n3rad  ) , kind=srk)
    divFrad_ray=-diffFrad_ray/dx3
    !
    ! --- Project radiative flux along ray onto HD grid; only action%dim=3 ---
    ! --- Regular case: current=global indices ---
    !
    if (only1D_flag) then
      Frad3(m1,m2,m3a:n3a+1)=Frad3(m1,m2,m3a:n3a+1)+Fradb_ray
      drhoeidtime(m1,m2,m3a:n3a)=drhoeidtime(m1,m2,m3a:n3a)+divFrad_ray
    else
      ! --- Flux inside cell ---
      Fradc_ray=0.5*(Fradb_ray(m3rad+1:n3rad+1) + &
                     Fradb_ray(m3rad  :n3rad  ) )
      !
      ! --- Flux: x3 direction ---
      do i3=m3rad,n3rad+1
        i3o=i3+m3a
        Frad3(  ib1_0_ray(i3), ib2_0_ray(i3), i3o) = &
          Frad3(ib1_0_ray(i3), ib2_0_ray(i3), i3o) + &
          fluxfraction30(i3o,1,1)*Fradb_ray(i3)
        Frad3(  ib1_1_ray(i3), ib2_0_ray(i3), i3o) = &
          Frad3(ib1_1_ray(i3), ib2_0_ray(i3), i3o) + &
          fluxfraction30(i3o,2,1)*Fradb_ray(i3)
        Frad3(  ib1_0_ray(i3), ib2_1_ray(i3), i3o) = &
          Frad3(ib1_0_ray(i3), ib2_1_ray(i3), i3o) + &
          fluxfraction30(i3o,1,2)*Fradb_ray(i3)
        Frad3(  ib1_1_ray(i3), ib2_1_ray(i3), i3o) = &
          Frad3(ib1_1_ray(i3), ib2_1_ray(i3), i3o) + &
          fluxfraction30(i3o,2,2)*Fradb_ray(i3)
      end do ! i3
      ! --- Flux: x1 direction ---
      if (n1 > m1) then
        do i3=m3rad,n3rad
          i3o=i3+m3a
          Frad1(  if11_0_ray(i3), if12_0_ray(i3), i3o) = &
            Frad1(if11_0_ray(i3), if12_0_ray(i3), i3o) + &
            fluxfraction10(i3o,1,1)*Fradc_ray(i3)
          Frad1(  if11_1_ray(i3), if12_0_ray(i3), i3o) = &
            Frad1(if11_1_ray(i3), if12_0_ray(i3), i3o) + &
            fluxfraction10(i3o,2,1)*Fradc_ray(i3)
          Frad1(  if11_0_ray(i3), if12_1_ray(i3), i3o) = &
            Frad1(if11_0_ray(i3), if12_1_ray(i3), i3o) + &
            fluxfraction10(i3o,1,2)*Fradc_ray(i3)
          Frad1(  if11_1_ray(i3), if12_1_ray(i3), i3o) = &
            Frad1(if11_1_ray(i3), if12_1_ray(i3), i3o) + &
            fluxfraction10(i3o,2,2)*Fradc_ray(i3)
          Frad1(  if11_0_ray(i3), if12_2_ray(i3), i3o) = &
            Frad1(if11_0_ray(i3), if12_2_ray(i3), i3o) + &
            fluxfraction10(i3o,1,3)*Fradc_ray(i3)
          Frad1(  if11_1_ray(i3), if12_2_ray(i3), i3o) = &
            Frad1(if11_1_ray(i3), if12_2_ray(i3), i3o) + &
            fluxfraction10(i3o,2,3)*Fradc_ray(i3)
        end do ! i3
      endif
      if (n2 > m2) then
        ! --- Flux: x2 direction ---
        do i3=m3rad,n3rad
          i3o=i3+m3a
          Frad2(  if21_0_ray(i3), if22_0_ray(i3), i3o) = &
            Frad2(if21_0_ray(i3), if22_0_ray(i3), i3o) + &
            fluxfraction20(i3o,1,1)*Fradc_ray(i3)
          Frad2(  if21_1_ray(i3), if22_0_ray(i3), i3o) = &
            Frad2(if21_1_ray(i3), if22_0_ray(i3), i3o) + &
            fluxfraction20(i3o,2,1)*Fradc_ray(i3)
          Frad2(  if21_2_ray(i3), if22_0_ray(i3), i3o) = &
            Frad2(if21_2_ray(i3), if22_0_ray(i3), i3o) + &
            fluxfraction20(i3o,3,1)*Fradc_ray(i3)
          Frad2(  if21_0_ray(i3), if22_1_ray(i3), i3o) = &
            Frad2(if21_0_ray(i3), if22_1_ray(i3), i3o) + &
            fluxfraction20(i3o,1,2)*Fradc_ray(i3)
          Frad2(  if21_1_ray(i3), if22_1_ray(i3), i3o) = &
            Frad2(if21_1_ray(i3), if22_1_ray(i3), i3o) + &
            fluxfraction20(i3o,2,2)*Fradc_ray(i3)
          Frad2(  if21_2_ray(i3), if22_1_ray(i3), i3o) = &
            Frad2(if21_2_ray(i3), if22_1_ray(i3), i3o) + &
            fluxfraction20(i3o,3,2)*Fradc_ray(i3)
        end do ! i3
      endif
      !
      ! --- drhoeidtime ---
      if (n1 == m1) then
        do i3=m3rad,n3rad
          i3o=i3+m3a
          drhoeidtime(  if21_1_ray(i3), if12_0_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_0_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,1)*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,2)*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_2_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_2_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,3)*divFrad_ray(i3)
        end do ! i3
      else if (n2 == m2) then
        do i3=m3rad,n3rad
          i3o=i3+m3a
          drhoeidtime(  if21_0_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_0_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,1,2)*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,2)*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_2_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,3,2)*divFrad_ray(i3)
        end do ! i3
      else
        do i3=m3rad,n3rad
          i3o=i3+m3a
          drhoeidtime(  if21_0_ray(i3), if12_0_ray(i3), i3o) = &
            drhoeidtime(if21_0_ray(i3), if12_0_ray(i3), i3o) + &
              sumfluxfraction(i3o,1,1)*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_0_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_0_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,1)*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_0_ray(i3), i3o) = &
            drhoeidtime(if21_2_ray(i3), if12_0_ray(i3), i3o) + &
              sumfluxfraction(i3o,3,1)*divFrad_ray(i3)
          drhoeidtime(  if21_0_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_0_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,1,2)*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,2)*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_1_ray(i3), i3o) = &
            drhoeidtime(if21_2_ray(i3), if12_1_ray(i3), i3o) + &
              sumfluxfraction(i3o,3,2)*divFrad_ray(i3)
          drhoeidtime(  if21_0_ray(i3), if12_2_ray(i3), i3o) = &
            drhoeidtime(if21_0_ray(i3), if12_2_ray(i3), i3o) + &
              sumfluxfraction(i3o,1,3)*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_2_ray(i3), i3o) = &
            drhoeidtime(if21_1_ray(i3), if12_2_ray(i3), i3o) + &
              sumfluxfraction(i3o,2,3)*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_2_ray(i3), i3o) = &
            drhoeidtime(if21_2_ray(i3), if12_2_ray(i3), i3o) + &
              sumfluxfraction(i3o,3,3)*divFrad_ray(i3)
        end do ! i3
      endif
      !
      ! --- Flux divergence correction ---
      !
      if (n2 == m2) then
        ! --- Flux: only x1 direction ---
        !
        do i3=m3rad,n3rad
          sumshiftdiv=0.0
          !
          i3o=i3+m3a
          i2o=if12_1_ray(i3)
          !
          ! --- Compute average rho*kappa*T^4 ---
          avgemission=sumfluxfraction(i3o, 1, 2) * emission(if21_0_ray(i3), i2o, i3o) + &
                      sumfluxfraction(i3o, 2, 2) * emission(if21_1_ray(i3), i2o, i3o) + &
                      sumfluxfraction(i3o, 3, 2) * emission(if21_2_ray(i3), i2o, i3o)
          !
          shiftamount=sumfluxfraction(i3o, 1, 2) * &
                      (emission(if21_0_ray(i3), i2o, i3o) - avgemission) / avgemission
          Frad1(        if21_1_ray(i3), i2o, i3o)= &
            Frad1(      if21_1_ray(i3), i2o, i3o) + shiftamount*diffFrad_ray(i3)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_0_ray(i3), i2o, i3o)= &
            drhoeidtime(if21_0_ray(i3), i2o, i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 3, 2) * &
                      (emission(if21_2_ray(i3), i2o, i3o) - avgemission) / avgemission
          Frad1(        if21_2_ray(i3), i2o, i3o)= &
            Frad1(      if21_2_ray(i3), i2o, i3o) - shiftamount*diffFrad_ray(i3)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), i2o, i3o)= &
            drhoeidtime(if21_2_ray(i3), i2o, i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          drhoeidtime(  if21_1_ray(i3), i2o, i3o)= &
            drhoeidtime(if21_1_ray(i3), i2o, i3o)-sumshiftdiv
        end do ! i3
      else if (n1 == m1) then
        ! --- Flux: only x2 direction ---
        !
        do i3=m3rad,n3rad
          sumshiftdiv=0.0
          !
          i3o=i3+m3a
          i1o=if21_1_ray(i3)
          !
          ! --- Compute average rho*kappa*T^4 ---
          avgemission=sumfluxfraction(i3o, 2, 1) * emission(i1o, if12_0_ray(i3), i3o) + &
                      sumfluxfraction(i3o, 2, 2) * emission(i1o, if12_1_ray(i3), i3o) + &
                      sumfluxfraction(i3o, 2, 3) * emission(i1o, if12_2_ray(i3), i3o)
          !
          shiftamount=sumfluxfraction(i3o, 2, 1) * &
                      (emission(i1o, if12_0_ray(i3), i3o) - avgemission) / avgemission
          Frad2(        i1o, if12_1_ray(i3), i3o)= &
            Frad2(      i1o, if12_1_ray(i3), i3o) + shiftamount*diffFrad_ray(i3)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  i1o, if12_0_ray(i3), i3o)= &
            drhoeidtime(i1o, if12_0_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 2, 3) * &
                      (emission(i1o, if12_2_ray(i3), i3o) - avgemission) / avgemission
          Frad2(        i1o, if12_2_ray(i3), i3o)= &
            Frad2(      i1o, if12_2_ray(i3), i3o) - shiftamount*diffFrad_ray(i3)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  i1o, if12_2_ray(i3), i3o)= &
            drhoeidtime(i1o, if12_2_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          drhoeidtime(  i1o, if12_1_ray(i3), i3o)= &
            drhoeidtime(i1o, if12_1_ray(i3), i3o)-sumshiftdiv
        end do ! i3
      else
        ! --- Flux: truely 3D ---
        !
        do i3=m3rad,n3rad
          sumshiftdiv=0.0
          !
          i3o=i3+m3a
          !
          ! --- Compute average rho*kappa*T^4 ---
          avgemission= &
            sumfluxfraction(i3o, 1, 1) * emission(if21_0_ray(i3), if12_0_ray(i3), i3o) + &
            sumfluxfraction(i3o, 1, 2) * emission(if21_0_ray(i3), if12_1_ray(i3), i3o) + &
            sumfluxfraction(i3o, 1, 3) * emission(if21_0_ray(i3), if12_2_ray(i3), i3o) + &
            sumfluxfraction(i3o, 2, 1) * emission(if21_1_ray(i3), if12_0_ray(i3), i3o) + &
            sumfluxfraction(i3o, 2, 2) * emission(if21_1_ray(i3), if12_1_ray(i3), i3o) + &
            sumfluxfraction(i3o, 2, 3) * emission(if21_1_ray(i3), if12_2_ray(i3), i3o) + &
            sumfluxfraction(i3o, 3, 1) * emission(if21_2_ray(i3), if12_0_ray(i3), i3o) + &
            sumfluxfraction(i3o, 3, 2) * emission(if21_2_ray(i3), if12_1_ray(i3), i3o) + &
            sumfluxfraction(i3o, 3, 3) * emission(if21_2_ray(i3), if12_2_ray(i3), i3o)
          !
          shiftamount=sumfluxfraction(i3o, 1, 1) * &
                 (emission(if21_0_ray(i3), if12_0_ray(i3), i3o) - avgemission) / avgemission
          shiftflux11=0.5*shiftamount*diffFrad_ray(i3)
          Frad1(        if21_1_ray(i3), if12_0_ray(i3), i3o)= &
            Frad1(      if21_1_ray(i3), if12_0_ray(i3), i3o) + shiftflux11
          Frad2(        if21_0_ray(i3), if12_1_ray(i3), i3o)= &
            Frad2(      if21_0_ray(i3), if12_1_ray(i3), i3o) + shiftflux11
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_0_ray(i3), if12_0_ray(i3), i3o)= &
            drhoeidtime(if21_0_ray(i3), if12_0_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 3, 1) * &
                 (emission(if21_2_ray(i3), if12_0_ray(i3), i3o) - avgemission) / avgemission
          shiftflux31=0.5*shiftamount*diffFrad_ray(i3)
          Frad1(        if21_2_ray(i3), if12_0_ray(i3), i3o)= &
            Frad1(      if21_2_ray(i3), if12_0_ray(i3), i3o) - shiftflux31
          Frad2(        if21_2_ray(i3), if12_1_ray(i3), i3o)= &
            Frad2(      if21_2_ray(i3), if12_1_ray(i3), i3o) + shiftflux31
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_0_ray(i3), i3o)= &
            drhoeidtime(if21_2_ray(i3), if12_0_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 1, 3) * &
                 (emission(if21_0_ray(i3), if12_2_ray(i3), i3o) - avgemission) / avgemission
          shiftflux13=0.5*shiftamount*diffFrad_ray(i3)
          Frad1(        if21_1_ray(i3), if12_2_ray(i3), i3o)= &
            Frad1(      if21_1_ray(i3), if12_2_ray(i3), i3o) + shiftflux13
          Frad2(        if21_0_ray(i3), if12_2_ray(i3), i3o)= &
            Frad2(      if21_0_ray(i3), if12_2_ray(i3), i3o) - shiftflux13
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_0_ray(i3), if12_2_ray(i3), i3o)= &
            drhoeidtime(if21_0_ray(i3), if12_2_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 3, 3) * &
                 (emission(if21_2_ray(i3), if12_2_ray(i3), i3o) - avgemission) / avgemission
          shiftflux33=0.5*shiftamount*diffFrad_ray(i3)
          Frad1(        if21_2_ray(i3), if12_2_ray(i3), i3o)= &
            Frad1(      if21_2_ray(i3), if12_2_ray(i3), i3o) - shiftflux33
          Frad2(        if21_2_ray(i3), if12_2_ray(i3), i3o)= &
            Frad2(      if21_2_ray(i3), if12_2_ray(i3), i3o) - shiftflux33
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_2_ray(i3), i3o)= &
            drhoeidtime(if21_2_ray(i3), if12_2_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          !
          shiftamount=sumfluxfraction(i3o, 1, 2) * &
                 (emission(if21_0_ray(i3), if12_1_ray(i3), i3o) - avgemission) / avgemission
          Frad1(        if21_1_ray(i3), if12_1_ray(i3), i3o)= &
            Frad1(      if21_1_ray(i3), if12_1_ray(i3), i3o) + &
                 (shiftamount*diffFrad_ray(i3) + shiftflux11 + shiftflux13)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_0_ray(i3), if12_1_ray(i3), i3o)= &
            drhoeidtime(if21_0_ray(i3), if12_1_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 3, 2) * &
                 (emission(if21_2_ray(i3), if12_1_ray(i3), i3o) - avgemission) / avgemission
          Frad1(        if21_2_ray(i3), if12_1_ray(i3), i3o)= &
            Frad1(      if21_2_ray(i3), if12_1_ray(i3), i3o) - &
                 (shiftamount*diffFrad_ray(i3) + shiftflux31 + shiftflux33)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_2_ray(i3), if12_1_ray(i3), i3o)= &
            drhoeidtime(if21_2_ray(i3), if12_1_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          !
          shiftamount=sumfluxfraction(i3o, 2, 1) * &
                  (emission(if21_1_ray(i3), if12_0_ray(i3), i3o) - avgemission) / avgemission
          Frad2(        if21_1_ray(i3), if12_1_ray(i3), i3o)= &
            Frad2(      if21_1_ray(i3), if12_1_ray(i3), i3o) + &
                 (shiftamount*diffFrad_ray(i3) + shiftflux11 + shiftflux31)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_0_ray(i3), i3o)= &
            drhoeidtime(if21_1_ray(i3), if12_0_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          shiftamount=sumfluxfraction(i3o, 2, 3) * &
                  (emission(if21_1_ray(i3), if12_2_ray(i3), i3o) - avgemission) / avgemission
          Frad2(        if21_1_ray(i3), if12_2_ray(i3), i3o)= &
            Frad2(      if21_1_ray(i3), if12_2_ray(i3), i3o) - &
                 (shiftamount*diffFrad_ray(i3) + shiftflux13 + shiftflux33)
          shiftdiv=shiftamount*divFrad_ray(i3)
          drhoeidtime(  if21_1_ray(i3), if12_2_ray(i3), i3o)= &
            drhoeidtime(if21_1_ray(i3), if12_2_ray(i3), i3o) + shiftdiv
          sumshiftdiv=sumshiftdiv+shiftdiv
          !
          !
          drhoeidtime(  if21_1_ray(i3), if12_1_ray(i3), i3o)= &
            drhoeidtime(if21_1_ray(i3), if12_1_ray(i3), i3o)-sumshiftdiv
        end do ! i3
      endif
    endif
  end do ! i2
end do ! i1
!
! --- Deallocate arrays for individual ray ---
deallocate(ic1_0_ray , ic2_0_ray, &
           ic1_1_ray , ic2_1_ray, &
           ib1_0_ray , ib2_0_ray, &
           ib1_1_ray , ib2_1_ray, &
           if11_0_ray, if11_1_ray, &
           if12_0_ray, if12_1_ray, if12_2_ray, &
           if21_0_ray, &
           rho_ray   , T4_ray     , logP_ray, &
           Fradb_ray , Fradc_ray, diffFrad_ray, divFrad_ray, &
           Fradb)
!
!!$! --- Additional test output ---
!!$write(6,'(A)') 'Write into rhd_rad3d_dirper_test.out'
!!$include 'rhd_rad3d_dirper_testout.f90'
!
end subroutine rhd_rad3d_dirper
!#
# endif /* rhd_rad3d_r02 */
!#

!----------******************----------------------------------------------------------------------
subroutine rhd_rad3d_diffdeep(m1,n1, m2,n2, m3,n3, m3ray, bound_l3, &
                              xb1, xb2, xb3, rho, kappa, T4, &
                              Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_diffdeep ('rhd_radiation_3_dimension_diffusion_deep')
!
! PURPOSE:
!   Compute fluxes with diffusion approximation in deeper layers.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_diffdeep(m1,n1, m2,n2, m3,n3, m3ray, bound_l3, &
!                           xb1, xb2, xb3, rho, kappa, T4, &
!                           Frad1, Frad2, Frad3, drhoeidtime)
!
! INPUT:
!   m1:      ('m_number_1') integer, 1. dimension, start index
!   n1:      ('number_1')   integer, 1. dimension, end index
!   m2:      ('m_number_2') integer, 2. dimension, start index
!   n2:      ('number_2')   integer, 2. dimension, end index
!   m3:      ('m_number_3') integer, 3. dimension, start index
!   n3:      ('number_3')   integer, 3. dimension, end index
!   m3ray:
!   bound_l3:
!   xb1:     ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:     ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:     ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:     ('rho') real 3D array, density
!   kappa:   ('kappa') real 3D array with opacities
!   T4:      ('temperature_4') real 3D array, temperature^4
!
! INPUT/OUTPUT:
!   Frad1:       ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:       ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:       ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime: ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!
! LOCAL VARIABLES:
!   dxb1,2,3:       ('delta_x_boundary_1,2,3') real 1D array, grid size in respective direction
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute radiative energy flux in diffusion approximation at points from m3 to m3ray-1
!   Compute updated values for drhoeidtime.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-18 (B.F.) Put into own routine
!   2000-02-06 (B.F.) x**4 -> (x**2)**2
!--------------------------------------------------------------------------------------------------
use const_module, only: sigma
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: m1,n1, m2,n2, m3,n3, m3ray, bound_l3
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1:n1, m2:n2, m3:n3), &
                          intent(in)    :: rho, kappa, T4
real, dimension(m1-3:n1+3+1, &
                m2-3:n2+3  , &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3+1, &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3) , &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
real,    dimension(m1:n1)               :: dxb1
real,    dimension(m2:n2)               :: dxb2
real,    dimension(m3:n3)               :: dxb3
integer                                 :: i1, i2, i3
!--------------------------------------------------------------------------------------------------
!
! --- Diffusion approximation in lower part ---
!
! --- Fluxes in x1 direction ---
if (n1 > m1) then
  dxb1=xb1(m1+1:n1+1)-xb1(m1:n1)
  !
  do i3=m3,m3ray-1
    do i2=m2,n2
      Frad1(m1+1:n1  ,i2,i3)=8.0/3.0*sigma * &
                             (T4(m1  :n1-1,i2,i3) - T4(m1+1:n1  ,i2,i3)) / &
                             (kappa(m1  :n1-1,i2,i3)*rho(m1  :n1-1,i2,i3)*dxb1(m1  :n1-1) + &
                              kappa(m1+1:n1  ,i2,i3)*rho(m1+1:n1  ,i2,i3)*dxb1(m1+1:n1  ))
    end do ! i2
  end do ! i3
  !
  Frad1(m1,m2:n2,m3:m3ray-1)=8.0/3.0*sigma * &
                             (T4(n1,i2,i3) - T4(m1,i2,i3)) / &
                             (kappa(n1,m2:n2,m3:m3ray-1)*rho(n1,m2:n2,m3:m3ray-1)*dxb1(n1) + &
                              kappa(m1,m2:n2,m3:m3ray-1)*rho(m1,m2:n2,m3:m3ray-1)*dxb1(m1))
endif
!
! --- Fluxes in x2 direction ---
if (n2 > m2) then
  dxb2=xb2(m2+1:n2+1)-xb2(m2:n2)
  !
  do i3=m3,m3ray-1
    do i2=m2+1,n2
      Frad2(m1:n1,i2  ,i3)=8.0/3.0*sigma * &
                           (T4(m1:n1,i2-1,i3) - T4(m1:n1,i2  ,i3)) / &
                           (kappa(m1:n1,i2-1,i3)*rho(m1:n1,i2-1,i3)*dxb2(i2-1) + &
                           kappa(m1:n1,i2  ,i3)*rho(m1:n1,i2  ,i3)*dxb2(i2  ))
    end do ! i2
  end do ! i3
  !
  Frad2(m1:n1,i2  ,m3:m3ray-1)=8.0/3.0*sigma * &
                              (T4(m1:n1,n2,m3:m3ray-1) - T4(m1:n1,m2,m3:m3ray-1)) / &
                              (kappa(m1:n1,n2,m3:m3ray-1)*rho(m1:n1,n2,m3:m3ray-1)*dxb2(n2) + &
                               kappa(m1:n1,m2,m3:m3ray-1)*rho(m1:n1,m2,m3:m3ray-1)*dxb2(m2))
endif
!
! --- Fluxes in x3 direction ---
dxb3=xb3(m3+1:n3+1)-xb3(m3:n3)
!
do i3=m3+1,m3ray
  do i2=m2,n2
    Frad3(m1:n1,i2,i3)=8.0/3.0*sigma * &
                       (T4(m1:n1,i2,i3-1) - T4(m1:n1,i2,i3  )) / &
                       (kappa(m1:n1,i2,i3-1)*rho(m1:n1,i2,i3-1)*dxb3(i3-1) + &
                        kappa(m1:n1,i2,i3  )*rho(m1:n1,i2,i3  )*dxb3(i3  ))
  end do ! i2
end do ! i3
!
! --- Lower boundary condition ---
select case (bound_l3)
  case(bound_inoutflow, bound_inoutflow2)
    ! --- constant ---
    ! --- Stellar atmosphere: diffusion approximation at bottom with constant dSdtau ---
    Frad3(m1:n1,m2:n2,m3)=Frad3(m1:n1,m2:n2,m3+1)
  case default
    ! --- constant or closed ---
    ! --- Stellar atmosphere: diffusion approximation at bottom with prescribed flux ---
    Frad3(m1:n1,m2:n2,m3)=sigma*par%Teff*par%Teff*par%Teff*par%Teff
end select
!
! --- Compute change in internal energy for optical thick points ---
do i3=m3,m3ray
  do i2=m2,n2
    do i1=m1,n1
      drhoeidtime(i1,i2,i3)=-( (Frad1(    i1+1,i2  ,i3  )-Frad1(    i1,i2,i3))/dxb1(i1) + & 
                               (Frad2(    i1  ,i2+1,i3  )-Frad2(    i1,i2,i3))/dxb2(i2) + &
                               (Frad3(    i1  ,i2  ,i3+1)-Frad3(    i1,i2,i3))/dxb3(i3)  )
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_rad3d_diffdeep


!----------*******************---------------------------------------------------------------------
subroutine rhd_rad3d_diffthick(m1,n1, m2,n2, m3,n3, C_raddiffthicklimit, C_raddifffraction, &
                               xb1, xb2, xb3, rho, kappa, T4, &
                               Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_diffthick ('rhd_radiation_3_dimension_diffusion_thick')
!
! PURPOSE:
!   Compute fluxes with diffusion approximation in optical thick part.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_diffthick(m1,n1, m2,n2, m3,n3, C_raddiffthicklimit, C_raddifffraction, &
!                            xb1, xb2, xb3, rho, kappa, T4, &
!                            Frad1, Frad2, Frad3, drhoeidtime)
!
! INPUT:
!   m1:      ('m_number_1') integer, 1. dimension, start index
!   n1:      ('number_1')   integer, 1. dimension, end index
!   m2:      ('m_number_2') integer, 2. dimension, start index
!   n2:      ('number_2')   integer, 2. dimension, end index
!   m3:      ('m_number_3') integer, 3. dimension, start index
!   n3:      ('number_3')   integer, 3. dimension, end index
!   C_raddiffthicklimit: ('Constant_radiation_diffusion_thick_limit') real parameter for
!                        optical depth above which the matter is considered to be optically thick,
!                        typically 1.0
!   C_raddifffraction:   ('Constant_radiation_diffusion_fraction') real parameter, fraction of
!                        newly computed flux:
!                          Frad=(1-C_raddifffraction) * Frad_old + &
!                                  C_raddifffraction  * Frad_thick
!   xb1:     ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:     ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:     ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:     ('rho') real 3D array, density
!   kappa:   ('kappa') real 3D array with opacities
!   T4 :     ('temperature_4') real 3D array, temperature^4
!
! INPUT/OUTPUT:
!   Frad1:       ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:       ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:       ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime: ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!
! LOCAL VARIABLES:
!   diffthick_mask: ('diffusion_thick_mask') logical 3D array, set to .true. if point is
!                   optically thick
!   dxb1,2,3:       ('delta_x_boundary_1,2,3') real 1D array, grid size in respective direction
!   dtau_l:         ('delta_tau_left') real, optical depth of point left to cell boundary
!   dtau_r:         ('delta_tau_right') real, optical depth of point right to cell boundary
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Only 'internal' boundaries are considered. Therefore not useful yet for periodic boundaries.
!
! PROCEDURE:
!   Mark optical thick points and compute radiative flux in diffusion approximation there.
!   Compute updated values for drhoeidtime.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-08 (B.F.) Diffusion approximation at optical thick points
!   2001-01-18 (B.F.) Put into own routine
!--------------------------------------------------------------------------------------------------
use const_module, only: sigma
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: m1,n1, m2,n2, m3,n3
real,                     intent(in)    :: C_raddiffthicklimit, C_raddifffraction
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1:n1, m2:n2, m3:n3), &
                          intent(in)    :: rho, kappa, T4
real, dimension(m1-3:n1+3+1, &
                m2-3:n2+3  , &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3+1, &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3) , &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
logical, dimension(m1:n1, m2:n2, m3:n3) :: diffthick_mask
real,    dimension(m1:n1)               :: dxb1
real,    dimension(m2:n2)               :: dxb2
real,    dimension(m3:n3)               :: dxb3
integer                                 :: i1, i2, i3
real                                    :: dtau_l, dtau_r
!--------------------------------------------------------------------------------------------------
!
! --- Diffusion approximation at optical thick points ---
!
diffthick_mask=.false.
!  
dxb1=xb1(m1+1:n1+1)-xb1(m1:n1)
dxb2=xb2(m2+1:n2+1)-xb2(m2:n2)
dxb3=xb3(m3+1:n3+1)-xb3(m3:n3)
!
! --- Fluxes in x1 direction ---
if (n1 > m1) then
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1-1
        dtau_l=kappa(i1  ,i2,i3)*rho(i1  ,i2,i3)*dxb1(i1  )
        dtau_r=kappa(i1+1,i2,i3)*rho(i1+1,i2,i3)*dxb1(i1+1)
        if ((dtau_l >= C_raddiffthicklimit) .and. (dtau_r >= C_raddiffthicklimit)) then
           Frad1(i1+1,i2,i3)=(1.0-C_raddifffraction)*Frad1(i1+1,i2,i3) + &
                                  C_raddifffraction *8.0/3.0*sigma * &
                                                 (T4(i1  ,i2,i3) - T4(i1+1,i2,i3)) / &
                                                 (dtau_l + dtau_r)
           diffthick_mask(i1  ,i2,i3)=.true.
           diffthick_mask(i1+1,i2,i3)=.true.
        endif
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
! --- Fluxes in x2 direction ---
if (n2 > m2) then
  do i3=m3,n3
    do i2=m2+1,n2
      do i1=m1,n1
        dtau_l=kappa(i1,i2-1,i3)*rho(i1,i2-1,i3)*dxb2(i2-1)
        dtau_r=kappa(i1,i2  ,i3)*rho(i1,i2  ,i3)*dxb2(i2  )
        if ((dtau_l >= C_raddiffthicklimit) .and. (dtau_r >= C_raddiffthicklimit)) then
          Frad2(i1,i2  ,i3)=(1.0-C_raddifffraction)*Frad2(i1,i2  ,i3) + &
                                 C_raddifffraction *8.0/3.0*sigma * &
                                                 (T4(i1,i2-1,i3) - T4(i1,i2  ,i3)) / &
                                                  (dtau_l + dtau_r)
          diffthick_mask(i1,i2-1,i3)=.true.
          diffthick_mask(i1,i2  ,i3)=.true.
        endif
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
! --- Fluxes in x3 direction ---
if (n3 > m3) then
  do i3=m3+1,n3
    do i2=m2,n2
      do i1=m1,n1
        dtau_l=kappa(i1,i2,i3-1)*rho(i1,i2,i3-1)*dxb3(i3-1)
        dtau_r=kappa(i1,i2,i3  )*rho(i1,i2,i3  )*dxb3(i3  )
        if ((dtau_l >= C_raddiffthicklimit) .and. (dtau_r >= C_raddiffthicklimit)) then
          Frad3(i1,i2,i3  )=(1.0-C_raddifffraction)*Frad3(i1,i2,i3  ) + &
                                 C_raddifffraction *8.0/3.0*sigma * &
                                                (T4(i1,i2,i3-1) - T4(i1,i2,i3  )) / &
                                                (dtau_l + dtau_r)
          diffthick_mask(i1,i2,i3-1)=.true.
          diffthick_mask(i1,i2,i3  )=.true.
        endif
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
! --- Compute change in internal energy for optical thick points ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      if (diffthick_mask(i1,i2,i3)) then
        drhoeidtime(i1,i2,i3)=-( (Frad1(    i1+1,i2  ,i3  )-Frad1(    i1,i2,i3))/dxb1(i1) + & 
                                 (Frad2(    i1  ,i2+1,i3  )-Frad2(    i1,i2,i3))/dxb2(i2) + &
                                 (Frad3(    i1  ,i2  ,i3+1)-Frad3(    i1,i2,i3))/dxb3(i3)  )
      endif
    end do ! i1
  end do ! i2
end do ! i3
!
end subroutine rhd_rad3d_diffthick


!----------***************-------------------------------------------------------------------------
subroutine rhd_rad3d_Tvis(m1,n1, m2,n2, m3,n3, C_radTvisdtau, C_radTvis, &
                          xb1, xb2, xb3, rho, kappa, T, dTdei, time_rad, &
                          Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_Tvis ('rhd_radiation_3_temperature_viscosity')
!
! PURPOSE:
!   Compute additional "viscous" radiative fluxes in optically thin part.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call 
!
! INPUT:
!   m1:      ('m_number_1') integer, 1. dimension, start index
!   n1:      ('number_1')   integer, 1. dimension, end index
!   m2:      ('m_number_2') integer, 2. dimension, start index
!   n2:      ('number_2')   integer, 2. dimension, end index
!   m3:      ('m_number_3') integer, 3. dimension, start index
!   n3:      ('number_3')   integer, 3. dimension, end index
!   C_radTvisdtau: ('Constant_radiation_temperature_viscosity_delta_tau') real parameter for
!                        optical depth below which the matter is considered to be optically thin,
!                        typically 0.1
!   C_radTvis:   ('Constant_radiation_viscosity') real parameter, strength of viscosity
!   xb1:     ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:     ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:     ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:     ('rho') real 3D array, density
!   kappa:   ('kappa') real 3D array with opacities
!   T:       ('temperature') real 3D array, temperature
!
! INPUT/OUTPUT:
!   Frad1:       ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:       ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:       ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime: ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!
! LOCAL VARIABLES:
!   diffthick_mask: ('diffusion_thick_mask') logical 3D array, set to .true. if point is
!                   optically thick
!   dxb1,2,3:       ('delta_x_boundary_1,2,3') real 1D array, grid size in respective direction
!   dtau_l:         ('delta_tau_left') real, optical depth of point left to cell boundary
!   dtau_r:         ('delta_tau_right') real, optical depth of point right to cell boundary
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Only 'internal' boundaries are considered. Therefore not useful yet for periodic boundaries.
!
! PROCEDURE:
!   Mark optical thin points and compute additional 'diffusive' radiative flux
!   to smooth strong temperature fluctuations.
!   Compute updated values for drhoeidtime.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-03-22 (B.F.) Written
!   2001-03-23 (B.F.) Corrections
!   2001-06-11 (B.F.) if ((dtau_l <= C_radTvisdtau) .and. (dtau_r <= C_radTvisdtau)) then  => .or.
!   2002-01-29 (B.F.) Shift outcommented lines because of Intel compiler
!--------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: m1,n1, m2,n2, m3,n3
real,                     intent(in)    :: C_radTvisdtau, C_radTvis
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1:n1, m2:n2, m3:n3), &
                          intent(in)    :: rho, kappa, T, dTdei, time_rad
real, dimension(m1-3:n1+3+1, &
                m2-3:n2+3  , &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad1
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3+1, &
                m3-3:n3+3  ), &
                          intent(inout) :: Frad2
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3+1), &
                          intent(inout) :: Frad3
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3) , &
                          intent(inout) :: drhoeidtime
!
! --- Local variables ---
real,    dimension(m1:n1)               :: dxb1
real,    dimension(m2:n2)               :: dxb2
real,    dimension(m3:n3)               :: dxb3
integer                                 :: i1, i2, i3
real                                    :: dtau_l, dtau_r
real, parameter                         :: Teps=1.0E-03
!--------------------------------------------------------------------------------------------------
!
write(6,*) 'Compute dxb'
call flush(6)
dxb1=xb1(m1+1:n1+1)-xb1(m1:n1)
dxb2=xb2(m2+1:n2+1)-xb2(m2:n2)
dxb3=xb3(m3+1:n3+1)-xb3(m3:n3)
write(6,*) 'Computation of dxb done'
call flush(6)
!
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1, dtau_l, dtau_r)
! --- Fluxes in x1 direction ---
if (n1 > m1) then
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1+1,n1
        dtau_l=kappa(i1-1,i2,i3)*rho(i1-1,i2,i3)*dxb1(i1-1)
        dtau_r=kappa(i1  ,i2,i3)*rho(i1  ,i2,i3)*dxb1(i1  )
        if ((dtau_l <= C_radTvisdtau) .or. (dtau_r <= C_radTvisdtau)) then
           Frad1(i1  ,i2,i3)= &
           Frad1(i1  ,i2,i3) - &
             C_radTvis * 0.5 * (dxb1(i1-1)+dxb1(i1)) * &
            (T(  i1      ,i2,i3) - T(i1-1,i2,i3)) * &
             min(rho(i1-1,i2,i3)/(time_rad(i1-1,i2,i3)*dTdei(i1-1,i2,i3)), &
                 rho(i1  ,i2,i3)/(time_rad(i1  ,i2,i3)*dTdei(i1  ,i2,i3)) &
                ) / &
             (1.0 + (dtau_l+dtau_r)/C_radTvisdtau)
!!$    (  abs(T(    i1      ,i2,i3) - T(i1-1,i2,i3)  -    (T(i1-1,i2,i3) - T(max(i1-2,m1),i2,i3))) / &
!!$     ( abs(T(    i1      ,i2,i3) - T(i1-1,i2,i3)) + abs(T(i1-1,i2,i3) - T(max(i1-2,m1),i2,i3)) + &
!!$                              Teps*T(i1-1,i2,i3) ) + &
!!$       abs(T(min(i1+1,n1),i2,i3) - T(i1  ,i2,i3)  -    (T(i1  ,i2,i3) - T(    i1-1    ,i2,i3))) / &
!!$     ( abs(T(min(i1+1,n1),i2,i3) - T(i1  ,i2,i3)) + abs(T(i1  ,i2,i3) - T(    i1-1    ,i2,i3)) + &
!!$                              Teps*T(i1  ,i2,i3) ) &
!!$    ) * 
!
!!$             (0.5*(rho(i1-1,i2,i3)/(time_rad(i1-1,i2,i3)*dTdei(i1-1,i2,i3))+ &
!!$                   rho(i1  ,i2,i3)/(time_rad(i1  ,i2,i3)*dTdei(i1  ,i2,i3)) &
!!$                  )) / 
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
! --- Fluxes in x2 direction ---
if (n2 > m2) then
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3,n3
    do i2=m2+1,n2
      do i1=m1,n1
        dtau_l=kappa(i1,i2-1,i3)*rho(i1,i2-1,i3)*dxb2(i2-1)
        dtau_r=kappa(i1,i2  ,i3)*rho(i1,i2  ,i3)*dxb2(i2  )
        if ((dtau_l <= C_radTvisdtau) .or. (dtau_r <= C_radTvisdtau)) then
           Frad2(i1,i2,  i3)= &
           Frad2(i1,i2,  i3) - &
             C_radTvis * 0.5 * (dxb2(i2-1)+dxb2(i2)) * &
             (T(i1,i2  ,i3) - T(i1,i2-1,i3)) * &
             min(rho(i1,i2-1,i3)/(time_rad(i1,i2-1,i3)*dTdei(i1,i2-1,i3)), &
                 rho(i1,i2  ,i3)/(time_rad(i1,i2  ,i3)*dTdei(i1,i2  ,i3)) &
                ) / &
             (1.0 + (dtau_l+dtau_r)/C_radTvisdtau)
!!$    (  abs(T(i1,    i2      ,i3) - T(i1,i2-1,i3)  -    (T(i1,i2-1,i3) - T(i1,max(i2-2,m2),i3))) / &
!!$     ( abs(T(i1,    i2      ,i3) - T(i1,i2-1,i3)) + abs(T(i1,i2-1,i3) - T(i1,max(i2-2,m2),i3)) + &
!!$                              Teps*T(i1,i2-1,i3) ) + &
!!$       abs(T(i1,min(i2+1,n2),i3) - T(i1,i2  ,i3)  -    (T(i1,i2  ,i3) - T(i1,    i2-1    ,i3))) / &
!!$     ( abs(T(i1,min(i2+1,n2),i3) - T(i1,i2  ,i3)) + abs(T(i1,i2  ,i3) - T(i1,    i2-1    ,i3)) + &
!!$                              Teps*T(i1,i2  ,i3) ) &
!!$    ) * 
!
!!$             (0.5*(rho(i1,i2-1,i3)/(time_rad(i1,i2-1,i3)*dTdei(i1,i2-1,i3))+ &
!!$                   rho(i1,i2  ,i3)/(time_rad(i1,i2  ,i3)*dTdei(i1,i2  ,i3)) &
!!$                  )) / 
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
! --- Fluxes in x3 direction ---
if (n3 > m3) then
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3,n3
   if (i3 >= (m3+1)) then
    do i2=m2,n2
      do i1=m1,n1
        dtau_l=kappa(i1,i2,i3-1)*rho(i1,i2,i3-1)*dxb3(i3-1)
        dtau_r=kappa(i1,i2,i3  )*rho(i1,i2,i3  )*dxb3(i3  )
        if ((dtau_l <= C_radTvisdtau) .or. (dtau_r <= C_radTvisdtau)) then
           Frad3(i1,i2,i3  )= &
           Frad3(i1,i2,i3  ) - &
             C_radTvis * 0.5 * (dxb3(i3-1)+dxb3(i3)) * &
             (T(i1,i2,i3  ) - T(i1,i2,i3-1)) * &
             min(rho(i1,i2,i3-1)/(time_rad(i1,i2,i3-1)*dTdei(i1,i2,i3-1)), &
                 rho(i1,i2,i3  )/(time_rad(i1,i2,i3  )*dTdei(i1,i2,i3  )) &
                ) / &
             (1.0 + (dtau_l+dtau_r)/C_radTvisdtau)
!!$    (  abs(T(i1,i2,    i3      ) - T(i1,i2,i3-1)  - (   T(i1,i2,i3-1) - T(i1,i2,max(i3-2,m3)))) / &
!!$     ( abs(T(i1,i2,    i3      ) - T(i1,i2,i3-1)) + abs(T(i1,i2,i3-1) - T(i1,i2,max(i3-2,m3))) + &
!!$                              Teps*T(i1,i2,i3-1) ) + &
!!$       abs(T(i1,i2,min(i3+1,n3)) - T(i1,i2,i3  )  - (   T(i1,i2,i3  ) - T(i1,i2,    i3-1   ))) / &
!!$     ( abs(T(i1,i2,min(i3+1,n3)) - T(i1,i2,i3  )) + abs(T(i1,i2,i3  ) - T(i1,i2,    i3-1   )) + &
!!$                              Teps*T(i1,i2,i3  ) ) &
!!$    ) * 
!
!!$             (0.5*(rho(i1,i2,i3-1)/(time_rad(i1,i2,i3-1)*dTdei(i1,i2,i3-1))+ &
!!$                   rho(i1,i2,i3  )/(time_rad(i1,i2,i3  )*dTdei(i1,i2,i3  )) &
!!$                  )) /
        endif
      end do ! i1
    end do ! i2
   endif
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
!$OMP BARRIER
!
! --- Compute change in internal energy for optical thick points ---
!$OMP DO SCHEDULE(STATIC)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      drhoeidtime(i1,i2,i3)=-( (Frad1(    i1+1,i2  ,i3  )-Frad1(    i1,i2,i3))/dxb1(i1) + & 
                               (Frad2(    i1  ,i2+1,i3  )-Frad2(    i1,i2,i3))/dxb2(i2) + &
                               (Frad3(    i1  ,i2  ,i3+1)-Frad3(    i1,i2,i3))/dxb3(i3)  )
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
!
end subroutine rhd_rad3d_Tvis


!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_formal(action, flux_flag, periodic_flag, &
                            n_dir, dx_dir, ray_dir, w_dir, &
                            m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
                            xc1,xc2,xc3, xb1,xb2,xb3, &
                            rho, T, logP, dTdei, kappa, time_rad, emission, &
                            Frad1, Frad2, Frad3, drhoeidtime)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_formal ('rhd_radiation_3_dimension_formal')
!
! PURPOSE:
!   Compute formal solution: 3D radiative energy flux.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_formal(action, periodic_flag, &
!                         n_dir, dx_dir, ray_dir, w_dir, &
!                         m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
!                         xc1,xc2,xc3, xb1,xb2,xb3, &
!                         rho, T, logP, dTdei, kappa, time_rad, emission, &
!                         Frad1, Frad2, Frad3, drhoeidtime)
!
! INPUT:
!   action: ('action') derived type with collection of control parameters
!
! INPUT/OUTPUT:
!
! OUTPUT:
!
! LOCAL VARIABLES:
!
! ROUTINES:
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   2001-01-19 (B.F.) Put into own routine
!   2001-03-26 (B.F.) Bugfix: Initialize flux arrays in all cases
!--------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),        intent(in)    :: action
logical,                  intent(in)    :: flux_flag, periodic_flag
integer,                  intent(in)    :: n_dir
real, dimension(3),       intent(in)    :: dx_dir
real, dimension(3,n_dir), intent(in)    :: ray_dir
real,                     intent(in)    :: w_dir
integer,                  intent(in)    :: m1,n1, m2,n2, m3,n3, m3ray, n3ray
real, dimension(m1:n1  ), intent(in)    :: xc1
real, dimension(m2:n2  ), intent(in)    :: xc2
real, dimension(m3:n3  ), intent(in)    :: xc3
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1  :n1, &
                m2  :n2, &
                m3  :n3 ), &
                          intent(in)    :: rho, T, dTdei, logP, kappa, time_rad
real, dimension(m1-3:n1+3, &
                m2-3:n2+3, &
                m3-3:n3+3), &
                          intent(in)    :: emission
real, dimension(m1-3:n1+3+1, &
                m2-3:n2+3  , &
                m3-3:n3+3  ), &
                          intent(out)   :: Frad1
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3+1, &
                m3-3:n3+3  ), &
                          intent(out)   :: Frad2
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3+1), &
                          intent(out)   :: Frad3
real, dimension(m1-3:n1+3, &
                m2-3:n2+3, &
                m3-3:n3+3), &
                          intent(out)   :: drhoeidtime
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
integer                                 :: i_dir, i1,i2,i3
real, dimension(3)                      :: r_dir
real, dimension(m1  :n1, &
                m2  :n2, &
                m3  :n3 )               :: T4
!real                                    :: C_raddiffthicklimit, C_raddifffraction
real, parameter                         :: C_raddiffthicklimit=0.0, C_raddifffraction=0.0
!real, parameter                         :: C_raddiffthicklimit=1.0, C_raddifffraction=0.5
!--------------------------------------------------------------------------------------------------
!
!!$write(6,*) 'In rhd_rad3d_formal'
!!$write(6,*) 'Initialize fluxes'
!!$call flush(6)
!
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1)
!
! --- Set fluxes to zero ---
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3-3,n3+3
    do i2=m2-3,n2+3
      do i1=m1-3,n1+3+1
        Frad1(i1,i2,i3)=0.0_srk
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3-3,n3+3
    do i2=m2-3,n2+3+1
      do i1=m1-3,n1+3
        Frad2(i1,i2,i3)=0.0_srk
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3-3,n3+3+1
    do i2=m2-3,n2+3
      do i1=m1-3,n1+3
        Frad3(i1,i2,i3)=0.0_srk
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
!
!$OMP DO SCHEDULE(STATIC)
do i3=m3-3,n3+3
  do i2=m2-3,n2+3
    do i1=m1-3,n1+3
      drhoeidtime(i1,i2,i3)=0.0_srk
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO NOWAIT
!
! --- T^4 ---
!$OMP DO SCHEDULE(STATIC)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
!!$      T4(i1,i2,i3)=(max(T(i1,i2,i3),0.0)**2)**2
! --- ATTENTION ---
      T4(i1,i2,i3)=(max(3300.0,0.0)**2)**2
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
if ((par%C_radTvisdtau > 0.0) .and. (par%C_radTvis > 0.0)) then

!!$write(6,*) 'Call rhd_rad3d_Tvis'
!!$call flush(6)

!!$write(6,'(A,2E13.5)') 'minmax(drhoeidtime): ', minval(drhoeidtime(m1:n1,m2:n2,m3:n3)), &
!!$                                               maxval(drhoeidtime(m1:n1,m2:n2,m3:n3))
!!$write(6,'(A,2E13.5)') 'minmax(Frad1)      : ', minval(Frad1(m1:n1+1,m2:n2,m3:n3)), &
!!$                                               maxval(Frad1(m1:n1+1,m2:n2,m3:n3))
!!$write(6,'(A,2E13.5)') 'minmax(Frad2)      : ', minval(Frad2(m1:n1,m2:n2+1,m3:n3)), &
!!$                                               maxval(Frad2(m1:n1,m2:n2+1,m3:n3))
!!$write(6,'(A,2E13.5)') 'minmax(Frad3)      : ', minval(Frad3(m1:n1,m2:n2,m3:n3+1)), &
!!$                                               maxval(Frad3(m1:n1,m2:n2,m3:n3+1))
!!$call flush(6)

  call rhd_rad3d_Tvis(m1,n1, m2,n2, m3,n3, par%C_radTvisdtau, par%C_radTvis, &
                      xb1, xb2, xb3, rho, kappa, T, dTdei, time_rad, &
                      Frad1, Frad2, Frad3, drhoeidtime)

write(6,'(A,2E13.5,2X,3I5,2X,3I5)') 'minmax(drhoeidtime): ', &
                                    minval(drhoeidtime(m1:n1,m2:n2,m3:n3)), &
                                    maxval(drhoeidtime(m1:n1,m2:n2,m3:n3)), &
                                    minloc(drhoeidtime(m1:n1,m2:n2,m3:n3)), &
                                    maxloc(drhoeidtime(m1:n1,m2:n2,m3:n3))
write(6,'(A,2E13.5,2X,3I5,2X,3I5)') 'minmax(Frad1)      : ', &
                                    minval(Frad1(m1:n1+1,m2:n2,m3:n3)), &
                                    maxval(Frad1(m1:n1+1,m2:n2,m3:n3)), &
                                    minloc(Frad1(m1:n1+1,m2:n2,m3:n3)), &
                                    maxloc(Frad1(m1:n1+1,m2:n2,m3:n3))
write(6,'(A,2E13.5,2X,3I5,2X,3I5)') 'minmax(Frad2)      : ', &
                                    minval(Frad2(m1:n1,m2:n2+1,m3:n3)), &
                                    maxval(Frad2(m1:n1,m2:n2+1,m3:n3)), &
                                    minloc(Frad2(m1:n1,m2:n2+1,m3:n3)), &
                                    maxloc(Frad2(m1:n1,m2:n2+1,m3:n3))
write(6,'(A,2E13.5,2X,3I5,2X,3I5)') 'minmax(Frad3)      : ', &
                                    minval(Frad3(m1:n1,m2:n2,m3:n3+1)), &
                                    maxval(Frad3(m1:n1,m2:n2,m3:n3+1)), &
                                    minloc(Frad3(m1:n1,m2:n2,m3:n3+1)), &
                                    maxloc(Frad3(m1:n1,m2:n2,m3:n3+1))
call flush(6)
endif
!
! --- Loop over all directions ---
do i_dir=1,n_dir
  ! write(6,*) 'i_dir, n_dir: ', i_dir, n_dir
  r_dir=ray_dir(:,i_dir)
  if (periodic_flag) then
!#
#   ifdef rhd_rad3d_r02
!#
    ! write(6,*)
    ! write(6,'(A,3F9.5,A)') 'Main ray direction 3: ', r_dir, ' -------------------------------'
    !
    ! --- Update fluxes for given direction ---
    call rhd_rad3d_dirper(rhd_action_new(action, dim=3), r_dir, w_dir, &
                          n_dir, m1,n1, m2,n2, m3,n3, m3ray,n3ray, &
                          xc1, xc2, xc3, xb1, xb2, xb3, &
                          rho, T4, logP, kappa, emission, &
                          Frad1, Frad2, Frad3, drhoeidtime)
!#
#   else /* rhd_rad3d_r02 */
!#
    write(gl%nc_p,*) 'Routine rhd_rad3d_dirper not installed !!!'
!#
#   endif /* rhd_rad3d_r02 */
!#
  else
    ! --- Open boundaries ---
    !
    ! --- Determine main ray direction ---
    if (((abs(r_dir(1)/dx_dir(1))) >= (abs(r_dir(2)/dx_dir(2)))) .and. &
        ((abs(r_dir(1)/dx_dir(1))) >= (abs(r_dir(3)/dx_dir(3))))) then
      write(6,'(A,3F9.5,A)') 'Main ray direction 1: ', r_dir, ' -----------------------------'
      call flush(6)
      !
      ! --- Update fluxes for given direction ---
      call rhd_rad3d_dir(rhd_action_new(action, dim=1), flux_flag, (/r_dir(2),r_dir(3),r_dir(1)/), &
                         n_dir, m2,n2, m3,n3, m1,n1,  m1,n1, m2,n2, m3,n3, &
                         xc2, xc3, xc1, xb2, xb3, xb1, &
                         rho, T4, logP, kappa, emission, &
                         Frad1, Frad2, Frad3, drhoeidtime)
    else if (((abs(r_dir(2)/dx_dir(2))) >= (abs(r_dir(1)/dx_dir(1)))) .and. &
             ((abs(r_dir(2)/dx_dir(2))) >= (abs(r_dir(3)/dx_dir(3))))) then
      write(6,'(A,3F9.5,A)') 'Main ray direction 2: ', r_dir, ' -----------------------------'
      call flush(6)
      !
      ! --- Update fluxes for given direction ---
      call rhd_rad3d_dir(rhd_action_new(action, dim=2), flux_flag, (/r_dir(3),r_dir(1),r_dir(2)/), &
                         n_dir, m3,n3, m1,n1, m2,n2,  m1,n1, m2,n2, m3,n3, &
                         xc3, xc1, xc2, xb3, xb1, xb2, &
                         rho, T4, logP, kappa, emission, &
                         Frad1, Frad2, Frad3, drhoeidtime)
    else
      write(6,'(A,3F9.5,A)') 'Main ray direction 3: ', r_dir, ' -----------------------------'
      call flush(6)
      !
      ! --- Update fluxes for given direction ---
      call rhd_rad3d_dir(rhd_action_new(action, dim=3), flux_flag, r_dir, &
                         n_dir, m1,n1, m2,n2, m3,n3,  m1,n1, m2,n2, m3,n3, &
                         xc1, xc2, xc3, xb1, xb2, xb3, &
                         rho, T4, logP, kappa, emission, &
                         Frad1, Frad2, Frad3, drhoeidtime)
    endif
    !
  endif
end do ! i_dir
!
if (periodic_flag) then
  if (m3ray > m3) then
    ! --- Diffusion approximation in lower part ---
    !
    call rhd_rad3d_diffdeep(m1,n1, m2,n2, m3,n3, m3ray, action%bound_l3, &
                            xb1, xb2, xb3, rho, kappa, T4, &
                            Frad1, Frad2, Frad3, drhoeidtime)
  endif
  !
  ! --- Account for periodic boundaries in Frad1 and Frad2 ---
  Frad1(n1+1   ,m2  :n2,m3:n3)=Frad1(m1   ,m2:n2,m3:n3)
  Frad2(m1  :n1,n2+1   ,m3:n3)=Frad2(m1:n1,m2   ,m3:n3)
else
  if ((C_raddiffthicklimit > 0.0) .and. (C_raddifffraction > 0.0)) then
    ! --- Diffusion approximation at optical thick points ---
    !
    call rhd_rad3d_diffthick(m1,n1, m2,n2, m3,n3, C_raddiffthicklimit, C_raddifffraction, &
                             xb1, xb2, xb3, rho, kappa, T4, &
                             Frad1, Frad2, Frad3, drhoeidtime)
  endif
endif
!
end subroutine rhd_rad3d_formal


!----------****************------------------------------------------------------------------------
subroutine rhd_rad3d_lambda(action, dtime, flux_flag, periodic_flag, &
                            n_dir, dx_dir, ray_dir, w_dir, &
                            m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
                            xc1,xc2,xc3, xb1,xb2,xb3, &
                            rho, T, logP, dTdei, kappa, time_rad, emission, &
                            dtime_limit, &
                            Frad1, Frad2, Frad3, drhoeidtime, &
                            outstr, ierr)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_lambda ('rhd_radiation_3_dimension_lambda')
!
! PURPOSE:
!   Do Lambda* iteration.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_lambda(action, dtime, flux_flag, periodic_flag, &
!                         n_dir, dx_dir, ray_dir, w_dir, &
!                         m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
!                         xc1,xc2,xc3, xb1,xb2,xb3, &
!                         rho, T, logP, dTdei, kappa, time_rad, emission, &
!                         dtime_limit, &
!                         Frad1, Frad2, Frad3, drhoeidtime, &
!                         outstr=outstr, ierr=ierr)
! INPUT:
!   action: ('action') derived type with collection of control parameters
!   dtime:  ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!
! OUTPUT:
!   dtime_limit:   ('delta_time_limit') real, recommended maximum radiative time step
!   outstr: ('output_string') character, optional
!   ierr:   ('index_error') integer, optional error number:
!           =0: normal run
!           1-99:  "small" error (e.g. time step limit violation)
!
! LOCAL VARIABLES:
!
! ROUTINES:
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Copenhagen) Writing started
!   2000-05-09 (B.F.) Lambda* iteration
!   2001-02-10 (B.F.) Put into own routine
!   2001-10-03 (B.F.) New start value for dT (switched sign)
!   2001-10-04 (B.F.) New start value for dT (take C_radimplicitmu into account)
!                   No interpolation of final drhoeidtime anymore
!--------------------------------------------------------------------------------------------------
!use const_module, only: sigma_srk => sigma
use const_module, only: sigma
use rhd_gl_module
use rhd_action_module
!#
# ifdef rhd_rad3d_step_t01 
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(action_type),        intent(in)    :: action
real,                     intent(in)    :: dtime
logical,                  intent(in)    :: flux_flag, periodic_flag
integer,                  intent(in)    :: n_dir
real, dimension(3),       intent(in)    :: dx_dir
real, dimension(3,n_dir), intent(in)    :: ray_dir
real,                     intent(in)    :: w_dir
integer,                  intent(in)    :: m1,n1, m2,n2, m3,n3, m3ray, n3ray
real, dimension(m1:n1  ), intent(in)    :: xc1
real, dimension(m2:n2  ), intent(in)    :: xc2
real, dimension(m3:n3  ), intent(in)    :: xc3
real, dimension(m1:n1+1), intent(in)    :: xb1
real, dimension(m2:n2+1), intent(in)    :: xb2
real, dimension(m3:n3+1), intent(in)    :: xb3
real, dimension(m1  :n1, &
                m2  :n2, &
                m3  :n3 ), &
                          intent(in)    :: rho, logP, dTdei, kappa, time_rad
real, dimension(m1  :n1, &
                m2  :n2, &
                m3  :n3 ), &
                          intent(inout) :: T
real, dimension(m1-3:n1+3, &
                m2-3:n2+3, &
                m3-3:n3+3), &
                          intent(in)    :: emission
real,                     intent(out)   :: dtime_limit
real, dimension(m1-3:n1+3+1, &
                m2-3:n2+3  , &
                m3-3:n3+3  ), &
                          intent(out)   :: Frad1
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3+1, &
                m3-3:n3+3  ), &
                          intent(out)   :: Frad2
real, dimension(m1-3:n1+3  , &
                m2-3:n2+3  , &
                m3-3:n3+3+1), &
                          intent(out)   :: Frad3
real, dimension(m1-3:n1+3, &
                m2-3:n2+3, &
                m3-3:n3+3), &
                          intent(out)   :: drhoeidtime
integer,        optional, intent(out)   :: ierr
character*(*),  optional, intent(out)   :: outstr
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
integer                                 :: n_lambdaiter
real (kind=srk), &
  dimension(m1  :n1, &
            m2  :n2, &
            m3  :n3)                    :: T0, T_ref, dT, dT0, dT_iter, dT_update, dT_update_last
real (kind=srk), &
  dimension(m1-3:n1+3  , &
            m2-3:n2+3  , &
            m3-3:n3+3  )                :: drhoeidtime0
real (kind=srk), &
  dimension(m1-3:n1+3+1, &
            m2-3:n2+3  , &
            m3-3:n3+3  )                :: Frad10
real (kind=srk), &
  dimension(m1-3:n1+3  , &
            m2-3:n2+3+1, &
            m3-3:n3+3  )                :: Frad20
real (kind=srk), &
  dimension(m1-3:n1+3  , &
            m2-3:n2+3  , &
            m3-3:n3+3+1)                :: Frad30
integer, dimension(3)                   :: iml
real                                    :: C_raditerstep_now, max_dT_update, max_dT_update_last, &
                                           min_diffT, max_diffT, dx
real (kind=srk)                         :: dtime_limit_recommend, dtime_drhoei
integer                                 :: ierr0
character*(80)                          :: outstr0
real, parameter                         :: C_raditerdTfactor=5.0
real, parameter                         :: C_raditerTsmalllimit=0.1, C_raditerTsmallfactor=10.0
real, parameter                         :: C_radtimeestlimit=1.2, C_radtimeestfactor=0.3
!--------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_rad3d_step_t01 
!#
call timing_start('RAD: lambda')
call timing_start('RAD: lambda: init')
!#
# endif
!#
! --- No error occurred ---
ierr0=0
outstr0=''
!
C_raditerstep_now=par%C_raditerstep
!
dx=sqrt(( ((xb1(n1+1)-xb1(m1   )) / real(n1+1-m1   ))**2 + &
          ((xb2(n2+1)-xb2(m2   )) / real(n2+1-m2   ))**2 + &
          ((xb3(n3+1)-xb3(m3ray)) / real(n3+1-m3ray))**2     )/3.0)
!
dtime_limit=huge(dtime_limit)
!
! --- Reference temperature ---
T_ref=T
T_ref(   m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)=T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1) + &
      C_raditerdTfactor * &
        (abs(T(m1+2:n1  , m2+1:n2-1, m3+1:n3-1)-T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)) + &
         abs(T(m1  :n1-2, m2+1:n2-1, m3+1:n3-1)-T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)) + &
         abs(T(m1+1:n1-1, m2+2:n2  , m3+1:n3-1)-T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)) + &
         abs(T(m1+1:n1-1, m2  :n2-2, m3+1:n3-1)-T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)) + &
         abs(T(m1+1:n1-1, m2+1:n2-1, m3+2:n3  )-T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)) + &
         abs(T(m1+1:n1-1, m2+1:n2-1, m3  :n3-2)-T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1))  )
      T_ref(   m1  :n1  , m2  :n2  , m3       )=T(m1  :n1  , m2  :n2  , m3       ) + &
        C_raditerdTfactor * 6.0 * &
        (abs(T(m1  :n1  , m2  :n2  , m3+1     )-T(m1  :n1  , m2  :n2  , m3       ))  )
      T_ref(   m1  :n1  , m2  :n2  , n3       )=T(m1  :n1  , m2  :n2  , n3       ) + &
        C_raditerdTfactor * 6.0 * &
        (abs(T(m1  :n1  , m2  :n2  , n3-1     )-T(m1  :n1  , m2  :n2  , n3       ))  )
      T_ref(   m1  :n1  , m2       , m3  :n3  )=T(m1  :n1  , m2       , m3  :n3  ) + &
        C_raditerdTfactor * 6.0 * &
        (abs(T(m1  :n1  , m2+1     , m3  :n3  )-T(m1  :n1  , m2       , m3  :n3  ))  )
      T_ref(   m1  :n1  , n2       , m3  :n3  )=T(m1  :n1  , n2       , m3  :n3  ) + &
        C_raditerdTfactor * 6.0 * &
        (abs(T(m1  :n1  , n2-1     , m3  :n3  )-T(m1  :n1  , n2       , m3  :n3  ))  )
      T_ref(   m1       , m2  :n2  , m3  :n3  )=T(m1       , m2  :n2  , m3  :n3  ) + &
        C_raditerdTfactor * 6.0 * &
        (abs(T(m1+1     , m2  :n2  , m3  :n3  )-T(m1       , m2  :n2  , m3  :n3  ))  )
      T_ref(   n1       , m2  :n2  , m3  :n3  )=T(n1       , m2  :n2  , m3  :n3  ) + &
        C_raditerdTfactor * 6.0 * &
        (abs(T(n1-1     , m2  :n2  , m3  :n3  )-T(n1       , m2  :n2  , m3  :n3  ))  )
!
dT=(epsilon(1.0) * sigma * (par%Teff**2)**2 * dtime/dx) * &
   dTdei/(T*rho * (1.0+dx*kappa*rho))
!!$write(6,'(A,I8)') 'Number of points with very low density: ', &
!!$  count( dT > (par%C_raditereps * C_raditerTsmalllimit) )
!
where( dT > (par%C_raditereps * C_raditerTsmalllimit) )
  T_ref= C_raditerTsmallfactor * T_ref
end where
!#
#   ifdef rhd_rad3d_step_t01 
!#
call timing_stop('RAD: lambda: init')
call timing_start('RAD: lambda: iter')
!#
#   endif
!#
n_lambdaiter=1
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
LAMBDAITERATION_LOOP: do
  ! --- Perform Lambda iteration ---
  !
  if (n_lambdaiter > 1) then
     T=T0 +      par%C_radimplicitmu  * dT_iter + &
            (1.0-par%C_radimplicitmu) * dT0
  endif
  !
  ! --- Compute 3D radiative flux: formal solution
  call rhd_rad3d_formal(action, flux_flag, periodic_flag, &
                        n_dir, dx_dir, ray_dir, w_dir, &
                        m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
                        xc1,xc2,xc3, xb1,xb2,xb3, &
                        rho, T, logP, dTdei, kappa, time_rad, emission, &
                        Frad1, Frad2, Frad3, drhoeidtime)
  !
  dT=dtime*drhoeidtime(m1:n1,m2:n2,m3:n3)*dTdei/rho
  !
  if (n_lambdaiter <= 1) then
    ! --- Start values for iteration ---
    T0          =T
    dT0         =dT
!!$    dT_iter     =dT
    dT_iter     =- (1.0-par%C_radimplicitmu)/par%C_radimplicitmu * dT  
    dT_update   =dT
    drhoeidtime0=drhoeidtime
    Frad10      =Frad1
    Frad20      =Frad2
    Frad30      =Frad3
  endif
  !
  ! --- Iteration update ---
  dT_update_last=dT_update
  dT_update     =C_raditerstep_now*(dT-dT_iter) / &
                 (1.0+par%C_radimplicitmu*dtime/time_rad)
  !
  iml=maxloc(abs(dT_update)/T_ref)+lbound(T_ref)-1
  max_dT_update_last=dT_update_last(iml(1),iml(2),iml(3))
  max_dT_update     =dT_update     (iml(1),iml(2),iml(3))
  min_diffT=minval((T-T0)/T0)
  max_diffT=maxval((T-T0)/T0)
  !
  write(6,*)
  write(6,'(A,I4,2X,3I4,2X,3(1PE14.6))') &
          'LAMBDA* Iteration: ', &
          n_lambdaiter, &
          iml, &
          max_dT_update_last, &
          max_dT_update_last/T_ref(iml(1),iml(2),iml(3)), &
          max_dT_update_last/T_ref(iml(1),iml(2),iml(3))/par%C_raditereps
  write(6,'(2X,2F6.3,A,4X,2X,12X,2X,3(1PE14.6))') &
          C_raditerstep_now, par%C_raditerstep, &
          ' --> ', &
          max_dT_update, &
          max_dT_update/T_ref(iml(1),iml(2),iml(3)), &
          max_dT_update/T_ref(iml(1),iml(2),iml(3))/par%C_raditereps
  write(6,'(A, 5(I8,1X))') &
          '  Points: dT/T > 100/10/1/.1/.01 eps:  ', &
          count(abs(dT_update)/T_ref > (100.0 *par%C_raditereps)), &
          count(abs(dT_update)/T_ref > ( 10.0 *par%C_raditereps)), &
          count(abs(dT_update)/T_ref > (  1.0 *par%C_raditereps)), &
          count(abs(dT_update)/T_ref > (  0.1 *par%C_raditereps)), &
          count(abs(dT_update)/T_ref > (  0.01*par%C_raditereps))
  write(6,'(A, 5(I8,1X))') &
          '  Points: (T-T0)/T0 > 1/.3/.1/.03/.01: ', &
          count(abs(T-T0)/T0 > 1.0 ), &
          count(abs(T-T0)/T0 > 0.3 ), &
          count(abs(T-T0)/T0 > 0.1 ), &
          count(abs(T-T0)/T0 > 0.03), &
          count(abs(T-T0)/T0 > 0.01)
  write(6,'(A, 2(1PE14.6))') &
          '  Statistics: minmax((T-T0)/T0): ', &
          min_diffT, max_diffT
  write(6,'(A, 3(1PE14.6))') &
          '              sum(dabs), drms:   ', &
          sum( abs(dT_update)/T_ref) / real(size(T_ref) ), &
          sqrt(sum( (abs(dT_update)/T_ref)**2) / real(size(T_ref) )), &
          sum( sign(1.0,dT_update_last*dT_update) * &
            abs(dT_update)/(abs(dT_update_last)+par%C_raditereps*T_ref) )/real(size(T_ref))
  call flush(6)
  !
  !!$    if ((n_lambdaiter > 3) .and. (C_radtimeestfactor > 0.0)) then
  !!$      time_rad=time_rad / &
  !!$               max(min(C_radtimeestlimit, &
  !!$                       1.0 - sign(C_radtimeestfactor,dT_update_last*dT_update) * &
  !!$                             abs(dT_update)/(abs(dT_update_last)+par%C_raditereps*T_ref)), &
  !!$                   1.0/C_radtimeestlimit)
  !!$    endif
  !
  if ((n_lambdaiter > 3) .and. (C_radtimeestfactor > 0.0)) then
    C_raditerstep_now=min(par%C_raditerstep, C_raditerstep_now / &
           max(min(C_radtimeestlimit, &
                   1.0 - sign(C_radtimeestfactor,max_dT_update_last*max_dT_update) * &
                         abs(max_dT_update)/(abs(max_dT_update_last) + &
                                           par%C_raditereps*T_ref(iml(1),iml(2),iml(3)))), &
               1.0/sqrt(C_radtimeestlimit))      )
  endif
  !
  ! --- Check if iteration has converged ---
  if (n_lambdaiter > 1) then
    if (maxval(abs(dT_update)/T_ref) <= par%C_raditereps) then
      exit LAMBDAITERATION_LOOP
    endif
  endif
  !
  ! --- Increment iteration count ---
  n_lambdaiter=n_lambdaiter+1
  !
  ! --- Check if maximum number of iterations has been reached ---
  if ((par%N_radmaxiter > 0) .and. (n_lambdaiter >= par%N_radmaxiter)) then
    ! --- Too many iterations: decrease time step ---
    ierr0=4
    outstr0='Too many iterations for radiative transfer'
    exit  LAMBDAITERATION_LOOP
  endif
  !
  dT_iter=dT_iter + dT_update
  !
end do LAMBDAITERATION_LOOP
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!#
#   ifdef rhd_rad3d_step_t01 
!#
call timing_stop('RAD: lambda: iter')
call timing_start('RAD: lambda: final')
!#
#   endif
!#
!
if (ierr0 /= 4) then
!!$  if (n_lambdaiter > 1) then
!!$    ! --- Compute final fluxes ---
!!$    drhoeidtime=(par%C_radimplicitmu*drhoeidtime + (1.0-par%C_radimplicitmu)*drhoeidtime0)
!!$    Frad1      =(par%C_radimplicitmu*Frad1       + (1.0-par%C_radimplicitmu)*Frad10      )
!!$    Frad2      =(par%C_radimplicitmu*Frad2       + (1.0-par%C_radimplicitmu)*Frad20      )
!!$    Frad3      =(par%C_radimplicitmu*Frad3       + (1.0-par%C_radimplicitmu)*Frad30      )
    !
!!$      ! --- ... and change in internal energy and temperature ---
!!$      do i3=m3,n3
!!$        do i2=m2,n2
!!$          drhoeidtime(m1:n1,i2,i3)=-( &
!!$                                     (Frad1(    m1+1:n1+1,i2  ,i3  )-Frad1(    m1:n1,i2,i3))/ &
!!$                                     (xb1(m1+1:n1+1          )-xb1(m1:n1      )) + &
!!$                                     (Frad2(    m1  :n1  ,i2+1,i3  )-Frad2(    m1:n1,i2,i3))/ &
!!$                                     (xb2(          i2+1     )-xb2(      i2   )) + &
!!$                                     (Frad3(    m1  :n1  ,i2  ,i3+1)-Frad3(    m1:n1,i2,i3))/ &
!!$                                     (xb3(               i3+1)-xb3(         i3))    )
!!$        end do ! i2
!!$      end do ! i3
    !  
!!$    dT=dtime*drhoeidtime(m1:n1,m2:n2,m3:n3)*dTdei/rho
!!$  endif
  !
  ! --- Check change in internal energy: derive characteristic time scale ---
  dtime_drhoei=dtime / max(maxval(abs(dT/T)), 1.0E-06)
  !
  ! --- Time step restrictions in implicit case ---
  !
  !!$  dtime_limit_recommend=huge(dtime_limit_recommend)
  !
  ! --- Check maximum number of iterations ---
  if ((par%N_radminiter > 0) .and. (par%N_raditer >= par%N_radminiter) .and. &
      (n_lambdaiter < par%N_radminiter)) then
    dtime_limit_recommend=dtime*par%dtime_incmax
  else if ((par%N_raditer > 0) .and. (n_lambdaiter > par%N_raditer)) then
    dtime_limit_recommend=dtime/par%dtime_incmax
  else
    dtime_limit_recommend=dtime
  endif
  !
  ! --- Time step due to maximum change in internal energy ---
  if (par%C_radmaxeiChange > 0.0) then
    dtime_limit_recommend=min(dtime_limit_recommend, par%C_radmaxeiChange*dtime_drhoei)
  endif
else
  dtime_limit_recommend=dtime/(par%dtime_incmax**2)
endif
!
dtime_limit=dtime_limit_recommend
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
!#
# ifdef rhd_rad3d_step_t01 
!#
call timing_stop('RAD: lambda: final')
call timing_stop('RAD: lambda')
!#
# endif
!#
end subroutine rhd_rad3d_lambda


!----------********************--------------------------------------------------------------------
subroutine rhd_rad3d_StepOutput(model, OutIntensity_flag, dtrel, radpressure, &
                                kappa, T, Frad1, Frad2, Frad3, &
                                dat, &
                                g1, g2, g3)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_StepOutput ('rhd_radiation_3_dimension_Step_Output')
!
! PURPOSE:
!   Derive some averages as additional output quantities.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_rad3d_StepOutput(model, OutIntensity_flag, dtrel, radpressure, &
!                             kappa, T, Frad1, Frad2, Frad3, &
!                             dat, &
!                             g1=g1, g2=g2, g3=g3)
!
! INPUT:
!   model:             ('model') derived type, model data: rho, pv, e, x, time, ...
!   OutIntensity_flag: ('Output_Intensity_flag') logical, if true the output intensities are
!                      computed, otherwise output fluxes etc., default=.false.
!   dtrel:  ('delta_time_relative') real, fraction of current sub time step from full time step
!   radpressure: ('radiation_pressure') character for radiation pressure mode:
!                'off', 'on'
!   kappa:  ('kappa') real 3D array with opacities
!   T:      ('Temperature') real 3D array with temperature
!   g1:     ('g_1') real 3D array with radiative acceleration, x1 direction (if radpressure='on')
!   g2:     ('g_2') real 3D array with radiative acceleration, x2 direction (if radpressure='on')
!   g3:     ('g_3') real 3D array with radiative acceleration, x3 direction (if radpressure='on')
!   Frad1:  ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:  ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:  ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!
! INPUT/OUTPUT:
!   dat:    ('data') derived type with additional output data
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute some averages for requested quantities
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-18 (B.F.) Put into own routine
!   2001-07-02 (B.F.) Temperature averages included
!   2012-07-27 (B.F.) Move scale%length to prop%length
!   2018-05-07 (B.F.) Outcomment branch with old averaging routines (there is no new one)
!--------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_dat_module
use rhd_box_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),           intent(in)    :: model
logical,                  intent(in)    :: OutIntensity_flag
real,                     intent(in)    :: dtrel
character*(*),            intent(in)    :: radpressure
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3), &
                          intent(in)    :: kappa, T
real, dimension(model%m1-3:model%n1+3+1, &
                model%m2-3:model%n2+3  , &
                model%m3-3:model%n3+3  ), &
                          intent(in)    :: Frad1
real, dimension(model%m1-3:model%n1+3  , &
                model%m2-3:model%n2+3+1, &
                model%m3-3:model%n3+3  ), &
                          intent(in)    :: Frad2
real, dimension(model%m1-3:model%n1+3  , &
                model%m2-3:model%n2+3  , &
                model%m3-3:model%n3+3+1), &
                          intent(in)    :: Frad3
type(dat_type),           intent(inout) :: dat
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3), &
                optional, intent(in)    :: g1, g2, g3
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3
real                                    :: areafraction
real   , dimension(  :,:), pointer      :: weight
real   , dimension(:,:,:), pointer      :: x123weight, kapparho_trans, T_trans, Frad3_trans, &
                                           g3_trans, gr
integer, dimension(  :,:), pointer      :: nbindex
integer, dimension(:,:,:), pointer      :: bindex, cindex
type (box_type)                         :: slice
!--------------------------------------------------------------------------------------------------
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Update output fluxes (for equidistant grid) ---
!
! --- Surface intensities/fluxes ---
if (OutIntensity_flag) then
  if (associated(dat%int1_l)) then
    dat%int1_l=dat%int1_l + dtrel*Frad1(m1     , m2:n2  , m3:n3  )
  endif
  if (associated(dat%int1_r)) then
    dat%int1_r=dat%int1_r + dtrel*Frad1(   n1+1, m2:n2  , m3:n3  )
  endif
  if (associated(dat%int2_l)) then
    dat%int2_l=dat%int2_l + dtrel*Frad2(m1:n1  , m2     , m3:n3  )
  endif
  if (associated(dat%int2_r)) then
    dat%int2_r=dat%int2_r + dtrel*Frad2(m1:n1  ,    n2+1, m3:n3  )
  endif
  if (associated(dat%int3_l)) then
    dat%int3_l=dat%int3_l + dtrel*Frad3(m1:n1  , m2:n2  , m3     )
  endif
  if (associated(dat%int3_r)) then
    dat%int3_r=dat%int3_r + dtrel*Frad3(m1:n1  , m2:n2  ,    n3+1)
  endif
else ! (.not. OutIntensity_flag)
  if (associated(dat%fer1_l)) then
    dat%fer1_l=dat%fer1_l + dtrel*Frad1(m1     , m2:n2  , m3:n3  )
  endif
  if (associated(dat%fer1_r)) then
    dat%fer1_r=dat%fer1_r + dtrel*Frad1(   n1+1, m2:n2  , m3:n3  )
  endif
  if (associated(dat%fer2_l)) then
    dat%fer2_l=dat%fer2_l + dtrel*Frad2(m1:n1  , m2     , m3:n3  )
  endif
  if (associated(dat%fer2_r)) then
    dat%fer2_r=dat%fer2_r + dtrel*Frad2(m1:n1  ,    n2+1, m3:n3  )
  endif
  if (associated(dat%fer3_l)) then
    dat%fer3_l=dat%fer3_l + dtrel*Frad3(m1:n1  , m2:n2  , m3     )
  endif
  if (associated(dat%fer3_r)) then
    dat%fer3_r=dat%fer3_r + dtrel*Frad3(m1:n1  , m2:n2  ,    n3+1) 
  endif
  !
!!$  if (n1 > m1) then
!!$    do i2=m2,n2
!!$      areafraction=dtrel*(model%xb2(i2+1)-model%xb2(i2)) / (model%xb2(n2+1)-model%xb2(m2))
!!$      !
!!$      allocate(weight( i2:i2, m3:n3), x123weight(m1:n1, i2:i2, m3:n3), &
!!$               nbindex(i2:i2, m3:n3), cindex(    m1:n1, i2:i2, m3:n3))
!!$      call rhd_box_Make(slice, m1=m1, n1=n1, m2=i2, n2=i2, m3=m3, n3=n3)
!!$      call rhd_box_Copy(model, m1, n1, i2, i2, m3, n3, &
!!$                        slice, m1,     i2,     m3,    trans_mode=1, bulk_mode=0)
!!$      !
!!$      call rhd_mean_weight(slice, areafraction, weight)
!!$      !
!!$      call rhd_mean_x23mean(kappa(m1:n1  ,i2:i2,m3:n3)*model%rho(m1:n1,i2:i2,m3:n3) &
!!$                                                      , dat%x23%kapparho_xmean, weight)
!!$      call rhd_mean_x23mean(T(m1:n1  ,i2:i2,m3:n3)    , dat%x23%T_xmean       , weight)
!!$      call rhd_mean_x23mean(T(m1:n1  ,i2:i2,m3:n3)**4 , dat%x23%T4_xmean      , weight)
!!$      call rhd_mean_x23mean(T(m1:n1  ,i2:i2,m3:n3)**4 * &
!!$                            kappa(m1:n1  ,i2:i2,m3:n3)*model%rho(m1:n1,i2:i2,m3:n3) &
!!$                                                      , dat%x23%T4kaprho_xmean, weight)
!!$      call rhd_mean_x23mean(Frad1(m1:n1+1,i2:i2,m3:n3), dat%x23%fer_xmean     , weight)
!!$      !
!!$      if ((radpressure == 'on') .and. (present(g1))) then
!!$        call rhd_mean_x23mean(g1(m1:n1  ,i2:i2,m3:n3), dat%x23%grad_xmean, weight)
!!$      endif
!!$      !
!!$      if (par%grav_mode == 'central') then
!!$        ! --- Averages over spheres (only mass and energy) ---
!!$        !
!!$        allocate(bindex(-size(dat%r%xb):size(dat%r%xb),i2:i2,m3:n3), &
!!$                 gr(m1:n1,i2:i2,m3:n3))
!!$        !
!!$        ! --- Compute weight arrays ---
!!$        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtrel)
!!$        !
!!$        ! --- Index arrays ---
!!$        call rhd_mean_rindex(slice, dat%r%xb, nbindex, bindex, cindex)
!!$        !
!!$        call rhd_mean_rcmean(m1,n1,i2,i2,m3,n3, size(dat%r%xb), &
!!$                             kappa(:,i2:i2,:)*model%rho(m1:n1,i2:i2,m3:n3), &
!!$                                                                            dat%r%kapparho_xmean, &
!!$                             nbindex, bindex, cindex, x123weight)
!!$        call rhd_mean_rcmean(m1,n1,i2,i2,m3,n3, size(dat%r%xb), &
!!$                             T(:,i2:i2,:),                                  dat%r%T_xmean, &
!!$                             nbindex, bindex, cindex, x123weight)
!!$        call rhd_mean_rcmean(m1,n1,i2,i2,m3,n3, size(dat%r%xb), &
!!$                             T(:,i2:i2,:)**4,                               dat%r%T4_xmean, &
!!$                             nbindex, bindex, cindex, x123weight)
!!$        call rhd_mean_rcmean(m1,n1,i2,i2,m3,n3, size(dat%r%xb), &
!!$                             T(:,i2:i2,:)**4*kappa(:,i2:i2,:)*model%rho(m1:n1,i2:i2,m3:n3), &
!!$                                                                            dat%r%T4kaprho_xmean, &
!!$                             nbindex, bindex, cindex, x123weight)
!!$        call rhd_mean_rbmean(m1,n1,i2,i2,m3,n3, size(dat%r%xb), &
!!$                             Frad1(m1:n1+1,i2:i2,m3:n3)               , dat%r%fer_xmean, &
!!$                             nbindex, bindex, weight)
!!$        if ((radpressure == 'on') .and. (present(g1)) .and. (present(g2))  .and. (present(g3))) then
!!$          call rhd_mean_VecProj(slice, g1(:,i2:i2,:), g2(:,i2:i2,:), g3(:,i2:i2,:), gr)
!!$          call rhd_mean_rcmean(m1,n1,i2,i2,m3,n3, size(dat%r%xb), &
!!$                               gr, dat%r%grad_xmean, &
!!$                               nbindex, bindex, cindex, x123weight)
!!$        endif
!!$        !
!!$        deallocate(bindex, gr)
!!$      endif
!!$      !
!!$      call rhd_box_Delete(slice)
!!$      !
!!$      ! --- Deallocate temporary arrays ---
!!$      deallocate(weight, x123weight, nbindex, cindex)
!!$    end do ! i2
!!$  endif
!!$  !
!!$  if (n2 > m2) then
!!$    do i1=m1,n1
!!$      areafraction=dtrel*(model%xb1(i1+1)-model%xb1(i1)) / (model%xb1(n1+1)-model%xb1(m1))
!!$      !
!!$      allocate(weight( i1:i1, m3:n3), x123weight(m2:n2, i1:i1, m3:n3), &
!!$               nbindex(i1:i1, m3:n3), cindex(    m2:n2, i1:i1, m3:n3))
!!$      call rhd_box_Make(slice, m1=m2, n1=n2, m2=i1, n2=i1, m3=m3, n3=n3)
!!$      call rhd_box_Copy(model, i1, i1, m2, n2, m3, n3, &
!!$                        slice, m2,     i1,     m3,    trans_mode=2, bulk_mode=0)
!!$      !
!!$      call rhd_mean_weight(slice, areafraction, weight)
!!$      !
!!$      call rhd_mean_x23mean(reshape(kappa(i1:i1,m2:n2,m3:n3)*model%rho(i1:i1,m2:n2,m3:n3), &
!!$                                    (/n2-m2+1,1,n3-m3+1/)), &
!!$                            dat%x13%kapparho_xmean, weight)
!!$      call rhd_mean_x23mean(reshape(T(i1:i1,m2:n2,m3:n3), &
!!$                                    (/n2-m2+1,1,n3-m3+1/)), &
!!$                            dat%x13%T_xmean, weight)
!!$      call rhd_mean_x23mean(reshape(T(i1:i1,m2:n2,m3:n3)**4, &
!!$                                    (/n2-m2+1,1,n3-m3+1/)), &
!!$                            dat%x13%T4_xmean, weight)
!!$      call rhd_mean_x23mean(reshape(T(i1:i1,m2:n2,m3:n3)**4 * &
!!$                                    kappa(i1:i1,m2:n2,m3:n3)*model%rho(i1:i1,m2:n2,m3:n3), &
!!$                                    (/n2-m2+1,1,n3-m3+1/)), &
!!$                            dat%x13%T4kaprho_xmean, weight)
!!$      call rhd_mean_x23mean(reshape(Frad2(i1:i1,m2:n2+1,m3:n3), (/n2-m2+2,1,n3-m3+1/)), &
!!$                            dat%x13%fer_xmean, weight)
!!$      !
!!$      if ((radpressure == 'on')  .and. (present(g2))) then
!!$        call rhd_mean_x23mean(reshape(g2(i1:i1,m2:n2,m3:n3), (/n2-m2+1,1,n3-m3+1/)), &
!!$                              dat%x13%grad_xmean, weight)
!!$      endif
!!$      !
!!$      if (par%grav_mode == 'central') then
!!$        ! --- Averages over spheres (only mass and energy) ---
!!$        !
!!$        allocate(bindex(-size(dat%r%xb):size(dat%r%xb),i1:i1,m3:n3))
!!$        !
!!$        ! --- Compute weight arrays ---
!!$        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtrel)
!!$        !
!!$        ! --- Index arrays ---
!!$        call rhd_mean_rindex(slice, dat%r%xb, nbindex, bindex, cindex)
!!$        !
!!$        call rhd_mean_rbmean(m2,n2,i1,i1,m3,n3, size(dat%r%xb), &
!!$                             reshape(Frad2(i1:i1,m2:n2+1,m3:n3), (/n2-m2+2,1,n3-m3+1/)), &
!!$                             dat%r%fer_xmean, &
!!$                             nbindex, bindex, weight)
!!$        deallocate(bindex)
!!$      endif
!!$      !
!!$      call rhd_box_Delete(slice)
!!$      !
!!$      ! --- Deallocate temporary arrays ---
!!$      deallocate(weight, x123weight, nbindex, cindex)
!!$    end do ! i1
!!$  endif
!!$  !
!!$  if (n3 > m3) then
!!$    do i2=m2,n2
!!$      areafraction=dtrel*(model%xb2(i2+1)-model%xb2(i2)) / (model%xb2(n2+1)-model%xb2(m2))
!!$      !
!!$      allocate(weight( i2:i2, m1:n1), x123weight(m3:n3, i2:i2, m1:n1), &
!!$               nbindex(i2:i2, m1:n1), cindex(    m3:n3, i2:i2, m1:n1), &
!!$               kapparho_trans(m3:n3,i2:i2,m1:n1), T_trans(m3:n3,i2:i2,m1:n1), &
!!$               Frad3_trans(m3:n3+1,i2:i2,m1:n1), g3_trans(m3:n3,i2:i2,m1:n1))
!!$      call rhd_box_Make(slice, m1=m3, n1=n3, m2=i2, n2=i2, m3=m1, n3=n1)
!!$      call rhd_box_Copy(model, m1, n1, i2, i2, m3, n3, &
!!$                        slice, m3,     i2,     m3,    trans_mode=3, bulk_mode=0)
!!$      !
!!$      call rhd_mean_weight(slice, areafraction, weight)
!!$      !
!!$      kapparho_trans(:,i2,:)=transpose(kappa(m1:n1,i2,m3:n3)*model%rho(m1:n1,i2,m3:n3))
!!$      T_trans(       :,i2,:)=transpose(T(    m1:n1,i2,m3:n3))
!!$      Frad3_trans(   :,i2,:)=transpose(Frad3(m1:n1,i2,m3:n3+1))
!!$      !
!!$      call rhd_mean_x23mean(kapparho_trans, dat%x12%kapparho_xmean, weight)
!!$      call rhd_mean_x23mean(T_trans       , dat%x12%T_xmean       , weight)
!!$      call rhd_mean_x23mean(T_trans**4    , dat%x12%T4_xmean      , weight)
!!$      call rhd_mean_x23mean(T_trans**4 * kapparho_trans, &
!!$                                            dat%x12%T4kaprho_xmean, weight)
!!$      call rhd_mean_x23mean(Frad3_trans   , dat%x12%fer_xmean     , weight)
!!$      !
!!$      if ((radpressure == 'on') .and. (present(g3))) then
!!$        g3_trans(:,i2,:)=transpose(g3(   m1:n1,i2,m3:n3  ))
!!$        call rhd_mean_x23mean(g3_trans, dat%x12%grad_xmean, weight)
!!$      endif
!!$      !
!!$      if (par%grav_mode == 'central') then
!!$        ! --- Averages over spheres (only mass and energy) ---
!!$        !
!!$        allocate(bindex(-size(dat%r%xb):size(dat%r%xb),i2:i2,m1:n1))
!!$        !
!!$        ! --- Compute weight arrays ---
!!$        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtrel)
!!$        !
!!$        ! --- Index arrays ---
!!$        call rhd_mean_rindex(slice, dat%r%xb, nbindex, bindex, cindex)
!!$        !
!!$        call rhd_mean_rbmean(m3,n3,i2,i2,m1,n1, size(dat%r%xb), Frad3_trans, &
!!$                             dat%r%fer_xmean, &
!!$                             nbindex, bindex, weight)
!!$        deallocate(bindex)
!!$      endif
!!$      !
!!$      call rhd_box_Delete(slice)
!!$      !
!!$      ! --- Deallocate temporary arrays ---
!!$      deallocate(weight, x123weight, nbindex, cindex, &
!!$                 kapparho_trans, T_trans, Frad3_trans, g3_trans)
!!$    end do ! i2
!!$    !
!!$  endif
endif ! (OutIntensity_flag)
!
end subroutine rhd_rad3d_StepOutput


!----------**************--------------------------------------------------------------------------
subroutine rhd_rad3d_Step(model, action, dtime, dtime_limit, &
                          update, output, outintensity, &
                            dat, &
                          outstr,ierr)
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rad3d_Step ('rhd_radiation_3_dimension_Step')
!
! PURPOSE:
!   Perform 3d radiative energy transfer step.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   update:       ('update') logical, optional, if true the internal energy is updated, default=T
!   output:       ('output') logical, optional, if true the output fluxes are computed, default=F
!   outintensity: ('output_intensity') logical, optional, if true the output intensities are
!                 computed, otherwise output fluxes, default=F
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, pv, e, x, time, ...
!   dat:          ('data') derived type with additional output data (optional)
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, recommended maximum radiative time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                   =0:   normal run
!                   1-99: "small" error (e.g. time step limit violation)
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   eosinter_logPTdTOMP:   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   rhd_rad3d_BaseAxisSystem: Generate random or unity basic orthonormal axis system
!   rhd_rad3d_dir:         Compute radiative energy flux for one direction field
!   rhd_rad3d_RayDirections: Create list of basic direction vectors
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TO DO:
!   structure for ray geometry
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Copenhagen) Writing started
!   1999-12-03 (B.F.) Reorganization
!   1999-12-28 (B.F.) Check for maximum iteration number
!   1999-12-29 (B.F.) C_radTsmooth
!   2000-01-02 (B.F.) If output only, set dtrel=1.0
!   2000-02-11 (B.F.) kappa now 3D array, radiation pressure added
!   2000-03-22 (B.F.) Correct weight formula for case of 1 or 2 rays
!   2000-03-24 (B.F.) Radiative "shear" fluxes added
!   2000-03-26 (B.F.) dTdei now 3D array
!   2000-03-27 (B.F.) drk -> selected_real_kind(10)
!   2000-03-28 (B.F.) Radiative "shear" fluxes proportional to delta kappa*rho*T^4
!   2000-03-30 (B.F.) call rhd_rad3d_dirper1
!   2000-05-09 (B.F.) Lambda* iteration
!   2000-05-22 (B.F.) drhoeidtime -> dT
!   2000-06-08 (B.F.) dT is allocated also in explicit case
!   2000-11-21 (B.F. Uppsala) drhoeidtime + 3 cells
!   2000-11-27 (B.F.) drhoeidtime directly from subroutines, not from Frad
!   2000-12-13 (B.F.) New drhoeidtime handling also in implicit case
!   2001-01-08 (B.F.) Diffusion approximation at optical thick points
!   2001-01-18 (B.F.) Some parts put into own routines
!   2001-02-09 (B.F.) Restructuring: explicit and implicit case separated,
!                     some pointer allocations removed (indices)
!   2001-02-12 (B.F.) Formula for emission changed
!   2001-03-19 (B.F.) Formula for emission changed: emission=rho*T^4
!   2001-03-21 (B.F.) Check only for too large NEGATIVE energy changes,
!                     New time step suggestion for iteration mode
!   2001-06-08 (B.F.) Distinction for optically thin/thick regions in formula for dtime_drhoei
!   2001-10-01 (B.F.) Possibility to keep random base axis system for all sub time steps
!   2008-08-12 (B.F. Lyon) Add index parameters to call of eosinter_logPTdT
!   2013-02-17 (B.F.) Rename eosinter_logPTdT -> eosinter_logPTdTOMP
!--------------------------------------------------------------------------------------------------
use opta_module
use gasinter_module
!use const_module, only: sigma_srk => sigma, pi_srk => pi, c_light_srk => c_light
use const_module, only: sigma, pi, c_light, sigopi
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_dat_module
!#
# ifdef rhd_rad3d_step_t01 
!#
use timing_module
!#
# endif
!#
use uio_bulk_module
use uio_filedef_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),           intent(inout) :: model
type(action_type),        intent(in)    :: action
real,                     intent(in)    :: dtime
real,                     intent(out)   :: dtime_limit
logical,        optional, intent(in)    :: update, output, outintensity
type(dat_type), optional, intent(inout) :: dat
integer,        optional, intent(out)   :: ierr
character*(*),  optional, intent(out)   :: outstr
!
! --- Local variables ---
integer, parameter                      :: srk=kind(0.0E+00), drk=selected_real_kind(10)
real (kind=srk), &
  dimension(model%m1  :model%n1, &
            model%m2  :model%n2, &
            model%m3  :model%n3)        :: T, logP, time_rad, kappa, dTdei, dT
real (kind=srk), &
  dimension(model%m1-3:model%n1+3  , &
            model%m2-3:model%n2+3  , &
            model%m3-3:model%n3+3  )    :: emission, drhoeidtime
real (kind=srk), &
  dimension(model%m1-3:model%n1+3+1, &
            model%m2-3:model%n2+3  , &
            model%m3-3:model%n3+3  )    :: Frad1
real (kind=srk), &
  dimension(model%m1-3:model%n1+3  , &
            model%m2-3:model%n2+3+1, &
            model%m3-3:model%n3+3  )    :: Frad2
real (kind=srk), &
  dimension(model%m1-3:model%n1+3  , &
            model%m2-3:model%n2+3  , &
            model%m3-3:model%n3+3+1)    :: Frad3
real (kind=srk), allocatable, &
                       dimension(:,:,:) :: g1, g2, g3
real (kind=srk)                         :: adtau, dx1, dx2, dx3, dx, dx_typ, &
                                           dtime_allsteps, dtime_thisstep, dtime_limit_thisstep, &
                                           dtime_limit_recommend, &
                                           dtime_rad, dtime_drhoei, dtime_min, dtrel
real (kind=srk), dimension(3)           :: dx_dir, r_dir
real (kind=srk), dimension(3,3)         :: M_axis
real (kind=srk), pointer, &
                         dimension(:,:) :: ray_dir0, ray_dir
integer                                 :: i, i_dir, n_dir, i1, i2, i3, &
                                           m1,n1, m2,n2, m3,n3, &
                                           n_subdtime
logical                                 :: exit_flag, &
                                           update_flag, output_flag, OutIntensity_flag, &
                                           periodic_flag, flux_flag
real                                    :: w_dir
character*(3)                           :: Frad_str
integer                                 :: ierr0
character*(80)                          :: outstr0
integer                                 :: m3ray, n3ray
real, parameter                         :: C_radtimesmooth=0.0
!real, parameter                         :: C_radtimesmooth=1.0
!!$integer                                 :: my_thread
!!$integer                                 :: omp_get_thread_num
!!$logical                                 :: omp_in_parallel
!--------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_rad3d_step_t01 
!#
call timing_start('RAD: step')
call timing_start('RAD: step: init')
!#
# endif
!#
write(6,*)
write(6,'(A)') '=== Start of rhd_rad3d_Step ==='
call flush(6)
!
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Default values for keywords ---
if (.not. present(update)) then
  update_flag=.true.
else
  update_flag=update
endif
if (.not. present(output)) then
  output_flag=.false.
else
  output_flag=output
endif
if (.not. present(outintensity)) then
  OutIntensity_flag=.false.
else
  OutIntensity_flag=outintensity
endif
flux_flag=(par%radpressure == 'on') .or. (output_flag) .or. (OutIntensity_flag)
!
! --- Boundary conditions periodic? ---
periodic_flag=((action%bound_l1 == bound_periodic) .or. (action%bound_r1 == bound_periodic) .or. &
               (action%bound_l2 == bound_periodic) .or. (action%bound_r2 == bound_periodic))
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- (Sub) range for rays ---
if (periodic_flag) then
  m3ray=min(m3+max(par%N_radThickPoint,0),n3)
  n3ray=max(n3-max(par%N_radThinPoint ,0),m3)
else
  m3ray=m3
  n3ray=n3
endif
!
if (par%radpressure == 'on') then
  ! --- Allocate additional arrays for acceleration due to radiation ---
  allocate(g1(m1:n1, m2:n2, m3:n3), &
           g2(m1:n1, m2:n2, m3:n3), &
           g3(m1:n1, m2:n2, m3:n3) )
  !$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1)
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        g1(i1,i2,i3)=0.0
        g2(i1,i2,i3)=0.0
        g3(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
endif
!
! --- Typical grid size: grid assumed to be equidistant ---
dx1=(model%xb1(n1+1)-model%xb1(m1   )) / real(n1+1-m1   )
dx2=(model%xb2(n2+1)-model%xb2(m2   )) / real(n2+1-m2   )
dx3=(model%xb3(n3+1)-model%xb3(m3ray)) / real(n3+1-m3ray)
!
! --- Vector through origin and opposite corner of grid ---
dx    =sqrt(dx1**2+dx2**2+dx3**2)
dx_dir=(/ dx1, dx2, dx3 /) / dx
dx_typ=dx/sqrt(3.0)
!
! --- Create base ray star ---
call rhd_rad3d_RayDirections(action%radraystar, ray_dir0)
n_dir=size(ray_dir0,2)
allocate(ray_dir(3,n_dir))
!write(6,'(2A)') 'radraystar: ', action%radraystar
!write(6,*) 'n_dir: ', n_dir
!
dtime_limit=huge(dtime_limit)
dtime_allsteps=0.0
n_subdtime=0
!#
# ifdef rhd_rad3d_step_t01 
!#
call timing_stop('RAD: step: init')
call timing_start('RAD: step: dtime')
!#
# endif
!#
!
! === Loop over all sub time steps ================================================================
DTIME_LOOP: do
  !
!#
# ifdef rhd_rad3d_step_t01 
!#
  call timing_start('RAD: step: dtime: init')
!#
# endif
!#
  if (par%C_radimplicitmu <= 0.0) then
    n_subdtime=n_subdtime+1
    write(6,'(A,I4)') 'n_subdtime: ', n_subdtime
    call flush(6)
    !
    ! --- Test if number of time steps is above absolute limit ---
    if (par%N_radmaxiter > 0) then
      if (n_subdtime > par%N_radmaxiter) then
        ierr0  =1
        outstr0='RAD3D: Number of sub time step above absolute limit'
        dtime_limit=dtime/max(par%N_radmaxiter,2)
        exit DTIME_LOOP
      endif
    endif
  endif
  !
  ! --- Create base axis system (possibly different for all sub time steps) ---
  if ((n_subdtime <= 1) .or. (action%radraybase == 'random')) then
    call rhd_rad3d_BaseAxisSystem(action%radraybase, M_axis)
  endif
  !
  ! --- Determine resulting ray directions ---
  ray_dir=matmul(M_axis,ray_dir0)
  !
  if (periodic_flag) then
    ! --- Periodic side boundaries: ---
    ! --- Check if directions of nearly horizontally rays have to be modified ---
    do i_dir=1,n_dir
      r_dir=ray_dir(:,i_dir)
      !
      ! --- Modify ray direction for nearly horizontal rays ---
      if (((abs(r_dir(1)/dx_dir(1))) >= (abs(r_dir(2)/dx_dir(2)))) .and. &
          ((abs(r_dir(1)/dx_dir(1))) >= (abs(r_dir(3)/dx_dir(3))))) then
        r_dir=r_dir/abs(r_dir(1))*dx_dir(1)
        if (r_dir(3) >= 0.0) then
          r_dir(3)= dx_dir(3)
        else
          r_dir(3)=-dx_dir(3)
        endif
        ray_dir(:,i_dir)=r_dir/sqrt(sum(r_dir**2))
      else if (((abs(r_dir(2)/dx_dir(2))) >= (abs(r_dir(1)/dx_dir(1)))) .and. &
               ((abs(r_dir(2)/dx_dir(2))) >= (abs(r_dir(3)/dx_dir(3))))) then
        r_dir=r_dir/abs(r_dir(2))*dx_dir(2)
        if (r_dir(3) >= 0.0) then
          r_dir(3)= dx_dir(3)
        else
          r_dir(3)=-dx_dir(3)
        endif
        ray_dir(:,i_dir)=r_dir/sqrt(sum(r_dir**2))
      endif
    end do ! i_dir
    w_dir=real(max(n_dir,3))/3.0/sum(ray_dir(3,:)**2)
    !
    write(6,*) 'ray_dir', shape(ray_dir)
    do i_dir=1,n_dir
      write(6,'(3(E13.6,1X))') ray_dir(:,i_dir)
    end do ! i_dir
    !
    write(6,'(A,3F9.5)')       '              weight: ', 3.0*sum(ray_dir(1,:)**2)/real(n_dir) &
                                                       , 3.0*sum(ray_dir(2,:)**2)/real(n_dir) &
                                                       , w_dir
  endif

!!$  write(6,*) 'M_axis: ', shape(M_axis)
!!$  do i=1,3
!!$    write(6,'(3(E13.6,1X))') M_axis(:,i)
!!$  end do ! i
!!$  write(6,*)
!!$  !
!!$  write(6,*) 'ray_dir0', shape(ray_dir0)
!!$  do i_dir=1,n_dir
!!$    write(6,'(3(E13.6,1X))') ray_dir0(:,i_dir)
!!$  end do ! i_dir
!!$  write(6,*)
!!$  !
!!$  write(6,*) 'Projection weights:', sum((ray_dir)**2,2)

  !$OMP PARALLEL DEFAULT(SHARED)
  !
  ! --- Solve EOS, compute minimum time step ---
  !
!#
# ifdef rhd_rad3d_step_t01 
!#
  !$OMP MASTER
  call timing_start('RAD: step: dtime: init: EOS, OPA')
  !$OMP END MASTER
!#
# endif
!#
  ! --- Solve EOS ---
  call eosinter_logPTdTOMP(model%rho, model%ei, &
                           logP, &
                           T, dTdei, m1,n1, m2,n2, m3,n3)
  !
  ! --- Opacities ---
  call sub_xkaros_logPT2kappa(logP,T,0,kappa)
  !
  !$OMP DO SCHEDULE(STATIC), PRIVATE(i2,i1,adtau)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
!!$      emission(i1,i2,i3)=(      dx_typ*kappa(i1,i2,i3)*model%rho(i1,i2,i3)) / &
!!$                         (1.0 + dx_typ*kappa(i1,i2,i3)*model%rho(i1,i2,i3)) *(T(i1,i2,i3)**2)**2
!!$        emission(i1,i2,i3)=model%rho(i1,i2,i3) * (T(i1,i2,i3)**2)**2
!!$        emission(i1,i2,i3)=(((1.0E-04*T(i1,i2,i3))**2)**2)**2
!!$           emission(i1,i2,i3)=((1.0E-04*T(i1,i2,i3))**2)**2
           emission(i1,i2,i3)=(1.0E-04*T(i1,i2,i3))*((1.0E-04*T(i1,i2,i3))**2)**2
        !
        ! --- delta (optical depth) ---
        adtau=kappa(i1,i2,i3)*model%rho(i1,i2,i3) * min(model%xb1(i1+1) - model%xb1(i1), &
                                                        model%xb2(i2+1) - model%xb2(i2), &
                                                        model%xb3(i3+1) - model%xb3(i3))
        !
        ! --- Eddington approximation (PhD 3.31) with k=pi/dx ---
        time_rad(i1,i2,i3)=1.0_srk/(16.0_srk*sigma) / &
                           (dTdei(i1,i2,i3)*kappa(i1,i2,i3)*T(i1,i2,i3)*T(i1,i2,i3)**2)* &
                           (1.0_srk+3.0_srk/(2.0_srk)*adtau**2)
        !!$                  (1.0_srk+3.0_srk/(pi*pi)*real(adtau,kind=srk)**2)
        !!$                  (1.0_srk+3.0_srk*real(adtau2,kind=srk))
        !!$                  (1.0_srk+3.0_srk/2.0_srk*real(adtau2,kind=srk))
        !!$                  exp(real(adtau,kind=srk))
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
!#
# ifdef rhd_rad3d_step_t01 
!#
  !$OMP MASTER
  call timing_stop('RAD: step: dtime: init: EOS, OPA')
  !$OMP END MASTER
!#
# endif
!#
  !$OMP MASTER
write(6,'(A,2E13.6)') 'minmax(T)       ', minval(T), maxval(T)
write(6,'(A,2E13.6)') 'minmax(ei8)     ', minval(model%ei(m1+8:n1-8,m2+8:n2-8,m3+8:n3-8)), &
                                          maxval(model%ei(m1+8:n1-8,m2+8:n2-8,m3+8:n3-8))
write(6,'(A,2E13.6)') 'minmax(logP)    ', minval(logP), maxval(logP)
write(6,'(A,2E13.6)') 'minmax(kappa)   ', minval(kappa), maxval(kappa)
!!$write(6,'(A,2E13.6)') 'minmax(time_rad)', minval(time_rad), maxval(time_rad)
  !
  ! --- Smooth time scale array ---
  if (C_radtimesmooth > 0.0) then
    call rhd_rad3d_smooth(C_radtimesmooth, m1,n1, m2,n2, m3,n3, time_rad, time_rad)
  endif
  !
write(6,'(A,2E13.6)') 'minmax(time_rad)', minval(time_rad), maxval(time_rad)
call flush(6)
!
  ! --- Maximum time step ---
  dtime_min=minval(time_rad)
  dtime_rad=par%C_radCourantMax*dtime_min
  !
  write(6,'(A,1PE14.6)') 'Time step ratio: dtime/dtime_rad: ', dtime/dtime_min
  !
  ! --- Fill buffer zones of emission array ---
  do i=1,3
    emission(m1-i , m2:n2, m3:n3)=emission(m1   , m2:n2, m3:n3)
    emission(n1+i , m2:n2, m3:n3)=emission(n1   , m2:n2, m3:n3)
  end do ! i
  do i=1,3
    emission(m1-3:n1+3, m2-i , m3:n3)=emission(m1-3:n1+3, m2   , m3:n3)
    emission(m1-3:n1+3, n2+i , m3:n3)=emission(m1-3:n1+3, n2   , m3:n3)
  end do ! i
  do i=1,3
    emission(m1-3:n1+3, m2-3:n2+3, m3-i )=emission(m1-3:n1+3, m2-3:n2+3, m3   )
    emission(m1-3:n1+3, m2-3:n2+3, n3+i )=emission(m1-3:n1+3, m2-3:n2+3, n3   )
  end do ! i
  !
  ! --- Smoothing of temperature field ---
  if (par%C_radTsmooth > 0.0) then
    T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1)= &
       (1.0-par%C_radTsmooth) *  T(m1+1:n1-1, m2+1:n2-1, m3+1:n3-1) + &
       par%C_radTsmooth/6.0   * (T(m1+2:n1  , m2+1:n2-1, m3+1:n3-1) + &
                                 T(m1  :n1-2, m2+1:n2-1, m3+1:n3-1) + &
                                 T(m1+1:n1-1, m2+2:n2  , m3+1:n3-1) + &
                                 T(m1+1:n1-1, m2  :n2-2, m3+1:n3-1) + &
                                 T(m1+1:n1-1, m2+1:n2-1, m3+2:n3  ) + &
                                 T(m1+1:n1-1, m2+1:n2-1, m3  :n3-2)  )
  endif
  !$OMP END MASTER
  !
  !$OMP END PARALLEL
!#
# ifdef rhd_rad3d_step_t01 
!#
  call timing_stop('RAD: step: dtime: init')
!#
# endif
!#
  if (par%C_radimplicitmu <= 0.0) then
    ! --- Explicit case ---------------------------------------------------------------------------
    !
!#
#   ifdef rhd_rad3d_step_t01 
!#
    call timing_start('RAD: step: dtime: explicit')
!#
#   endif
!#
    ! --- Compute 3D radiative flux: formal solution
write(6,*) 'Call rhd_rad3d_formal'
call flush(6)
    call rhd_rad3d_formal(action, flux_flag, periodic_flag, &
                          n_dir, dx_dir, ray_dir, w_dir, &
                          m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
                          model%xc1,model%xc2,model%xc3, model%xb1,model%xb2,model%xb3, &
                          model%rho, T, logP, dTdei, kappa, time_rad, emission, &
                          Frad1, Frad2, Frad3, drhoeidtime)
    !
!!$write(6,'(A)') 'formal done'
!!$call flush(6)
    !
    ! --- Check change in internal energy: derive characteristic time scale ---
    ! --- dT=dtime*drhoeidtime(m1:n1,m2:n2,m3:n3)*dTdei/model%rho
    ! --- dtime_drhoei=dtime / max(maxval(abs(dT/T)), 1.0E-06)
!!$write(6,'(A,2E13.5)') 'minmax(drhoeidtime)   : ', minval(drhoeidtime(m1:n1,m2:n2,m3:n3)), &
!!$                                               maxval(drhoeidtime(m1:n1,m2:n2,m3:n3))
!!$write(6,'(A,6I8)') 'minmaxloc(drhoeidtime): ', minloc(drhoeidtime(m1:n1,m2:n2,m3:n3)), &
!!$                                               maxloc(drhoeidtime(m1:n1,m2:n2,m3:n3))
!!$call flush(6)
    if ((par%C_radthintimefac > 0.0) .and. (par%C_radTvisdtau > 0.0)) then
      dtime_drhoei=1.0 / max(maxval(abs( &
                          (1.0+                     par%C_radTvisdtau/(dx_typ*kappa*model%rho))/ &
                          (1.0+par%C_radthintimefac*par%C_radTvisdtau/(dx_typ*kappa*model%rho)) * &
                          drhoeidtime(m1:n1,m2:n2,m3:n3)*dTdei/(model%rho*T))), &
                         1.0E-06/dtime)
    else
      dtime_drhoei=1.0 / max(maxval(abs(drhoeidtime(m1:n1,m2:n2,m3:n3)*dTdei/(model%rho*T))), &
                             1.0E-06/dtime)
!!$    dtime_drhoei=1.0 / max(maxval(-drhoeidtime(m1:n1,m2:n2,m3:n3)*dTdei/(model%rho*T)), &
!!$                           1.0E-06/dtime)
    endif
    !
!#
#   ifdef rhd_rad3d_step_t01 
!#
    call timing_stop('RAD: step: dtime: explicit')
!#
#   endif
!#
    ! --- Time step restrictions in explicit case ---
    !
    ! --- Radiative time scale: absolute upper limit for this sub time step ---
    dtime_rad=par%C_radCourantMax*dtime_min
    dtime_limit_thisstep=dtime_rad
    ! --- Minimum iteration number ---
    if (par%N_radminiter > 0) then
      dtime_limit_thisstep=min(dtime_limit_thisstep, dtime/(real(par%N_radminiter)-0.9))
    endif
    ! --- Time step due to maximum change in internal energy ---
    if (par%C_radmaxeiChange > 0.0) then
      dtime_limit_thisstep=min(dtime_limit_thisstep, par%C_radmaxeiChange*dtime_drhoei)
    endif
    !
    ! --- Test if time step is below absolute limit ---
    if (dtime_limit_thisstep < par%dtime_min_stop) then
      ierr0  =100
      outstr0='RAD3D: Time step below absolute limit'    
      dtime_limit=dtime_limit_thisstep
      exit_flag=.true.
      if (update_flag) exit DTIME_LOOP
    endif
    !
    ! --- Radiative time scale: recommended value for this sub time step ---
    dtime_rad =par%C_radCourant*dtime_min
    dtime_limit_recommend=dtime_rad
    ! --- Minimum iteration number ---
    if (par%N_radminiter > 0) then
      dtime_limit_recommend=min(dtime_limit_recommend, dtime/(real(par%N_radminiter)-0.9))
    endif
    ! --- Time step due to maximum change in internal energy ---
    if (par%C_radmaxeiChange > 0.0) then
      dtime_limit_recommend=min(dtime_limit_recommend, par%C_radmaxeiChange*dtime_drhoei)
    endif
    !
    ! --- Check if current sub time step is large enough to be the last one ---
    if (dtime_limit_thisstep >= (dtime-dtime_allsteps)) then
      dtime_thisstep=dtime-dtime_allsteps
      exit_flag=.true.
    else
      dtime_thisstep=dtime_limit_recommend
      exit_flag=.false.
    endif
    !
    write(6,'(A,4(1PE10.3))') 'dtime_:rad,drhoei,limit_this,all: ', &
                              dtime_rad, dtime_drhoei, dtime_limit_thisstep, dtime_allsteps
    !
    ! --- Suggest new time step for radiation transport ---
    if ((exit_flag) .and. (ierr0 == 0)) then
      if (par%N_raditer >= max(1, par%N_radminiter)) then
        ! --- Time step suggestion for iteration mode ---
!!$        dtime_limit=dtime_limit_recommend*real(par%N_raditer)
        dtime_limit=dtime*real(par%N_raditer)/real(n_subdtime)
      else
        ! --- Time step suggestion in case of no iteration ---
        dtime_limit=dtime_rad
        ! --- Time step due to maximum change in internal energy ---
        if (par%C_radmaxeiChange > 0.0) then
          dtime_limit=min(dtime_limit, par%C_radmaxeiChange*dtime_drhoei)
        endif
      endif
    endif
    !
  else
    ! --- Implicit case: perform Lambda iteration -------------------------------------------------
    !
    dtime_thisstep=dtime
    !
    call rhd_rad3d_lambda(action, dtime, flux_flag, periodic_flag, &
                          n_dir, dx_dir, ray_dir, w_dir, &
                          m1,n1, m2,n2, m3,n3, m3ray, n3ray, &
                          model%xc1,model%xc2,model%xc3, model%xb1,model%xb2,model%xb3, &
                          model%rho, T, logP, dTdei, kappa, time_rad, emission, &
                          dtime_limit, &
                          Frad1, Frad2, Frad3, drhoeidtime, &
                          outstr=outstr0, ierr=ierr0)
    exit_flag=.true.
    if (ierr0 == 4) exit DTIME_LOOP
    !
  endif
  ! --- Explicit or implicit case done: drhoeidtime, dtime_limit computed -------------------------
!#
# ifdef rhd_rad3d_step_t01 
!#
  call timing_start('RAD: step: dtime: final')
!#
# endif
!#
  !
  if (update_flag) then
    dtrel=dtime_thisstep/dtime
  else
    dtrel=1.0
  endif
  !
  ! --- Radiative acceleration ---
  if (par%radpressure == 'on') then
    !$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1)
    !$OMP DO SCHEDULE(STATIC)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          g1(i1,i2,i3)=g1(i1,i2,i3) + dtrel/(2.0*c_light)*kappa(i1,i2,i3)*(Frad1(i1  ,i2  ,i3  ) +&
                                                                           Frad1(i1+1,i2  ,i3  ))
          g2(i1,i2,i3)=g2(i1,i2,i3) + dtrel/(2.0*c_light)*kappa(i1,i2,i3)*(Frad2(i1  ,i2  ,i3  ) +&
                                                                           Frad2(i1  ,i2+1,i3  ))
          g3(i1,i2,i3)=g3(i1,i2,i3) + dtrel/(2.0*c_light)*kappa(i1,i2,i3)*(Frad3(i1  ,i2  ,i3  ) +&
                                                                           Frad3(i1  ,i2  ,i3+1))
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !$OMP END PARALLEL
  endif
  !
  ! --- Update output fluxes (for equidistant grid) ---
  if ((output_flag) .and. present(dat)) then
    if (OutIntensity_flag) then
      Frad_str='Int'
    else
      Frad_str='Fer'
    endif
    write(6,'(A,3(2E11.4,2X))') 'minmax(' // trim(Frad_str) // '1,2,3): ', &
                                                 minval(Frad1(m1:n1+1,m2:n2  ,m3:n3  )), &
                                                 maxval(Frad1(m1:n1+1,m2:n2  ,m3:n3  )), &
                                                 minval(Frad2(m1:n1  ,m2:n2+1,m3:n3  )), &
                                                 maxval(Frad2(m1:n1  ,m2:n2+1,m3:n3  )), &
                                                 minval(Frad3(m1:n1  ,m2:n2  ,m3:n3+1)), &
                                                 maxval(Frad3(m1:n1  ,m2:n2  ,m3:n3+1))

    !
!#
#    ifdef rhd_rad3d_step_t01 
!#
     call timing_start('RAD: step: dtime: final(output)')
!#
#    endif
!#
    if (par%radpressure == 'on') then
      call rhd_rad3d_StepOutput(model, OutIntensity_flag, dtrel, par%radpressure, &
                                kappa, T, Frad1, Frad2, Frad3, &
                                dat, &
                                g1=g1, g2=g2, g3=g3)
    else
      call rhd_rad3d_StepOutput(model, OutIntensity_flag, dtrel, par%radpressure, &
                                kappa, T, Frad1, Frad2, Frad3, &
                                dat)
    endif
!#
#    ifdef rhd_rad3d_step_t01 
!#
     call timing_stop('RAD: step: dtime: final(output)')
!#
#    endif
!#
    !
  endif
  !
  if (update_flag) then
    ! --- Update internal energy ---
    !$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1)
    !$OMP DO SCHEDULE(STATIC)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                             dtime_thisstep*drhoeidtime(i1,i2,i3)/model%rho(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !$OMP END PARALLEL
  else
    exit_flag=.true.
  endif
  !
  ! --- Update sub time step ---
  dtime_allsteps=dtime_allsteps+dtime_thisstep
  !
!#
# ifdef rhd_rad3d_step_t01 
!#
  call timing_stop('RAD: step: dtime: final')
!#
# endif
!#
  ! --- Check for exit ---
  if (exit_flag) exit DTIME_LOOP
  !
end do DTIME_LOOP
! =================================================================================================
!#
# ifdef rhd_rad3d_step_t01 
!#
call timing_stop('RAD: step: dtime')
call timing_start('RAD: step: final')
!#
# endif
!#
!
if ((ierr0 == 0) .and. (update_flag) .and. (par%radpressure == 'on')) then
  ! --- Update velocities ---
  !$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i2, i1)
  !$OMP DO SCHEDULE(STATIC)
  do i3=m3,n3
    do i2=m2,n2
      do i1=m1,n1
        model%v1(i1,i2,i3)=model%v1(i1,i2,i3) + dtime*g1(i1,i2,i3)
        model%v2(i1,i2,i3)=model%v2(i1,i2,i3) + dtime*g2(i1,i2,i3)
        model%v3(i1,i2,i3)=model%v3(i1,i2,i3) + dtime*g3(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
endif
!
if (par%radpressure == 'on') then
  deallocate(g1, g2, g3)
endif
deallocate(ray_dir0, ray_dir)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
write(6,'(A)') '=== End of rhd_rad3d_Step ==='
write(6,*)
!
!#
# ifdef rhd_rad3d_step_t01 
!#
call timing_stop('RAD: step: final')
call timing_stop('RAD: step')
!#
# endif
!#
end subroutine rhd_rad3d_Step

end module rhd_lhdrad_module
