!***************************************************************************************************
!
!   #####   #    #  #####         ####   #    #   ####   #####  ##### #####     ##    #####
!   #    #  #    #  #    #       #       #    #  #    #  #    #   #   #    #   #  #   #    #
!   #    #  ######  #    #        ####   ######  #    #  #    #   #   #    #  #    #  #    #
!   #####   #    #  #    #            #  #    #  #    #  #####    #   #####   ######  #    #
!   #   #   #    #  #    #       #    #  #    #  #    #  #   #    #   #   #   #    #  #    #
!   #    #  #    #  #####  #####  ####   #    #   ####   #    #   #   #    #  #    #  ##### _module
!
!   Radiation-Hydrodynamics Code: short-characteristics radiation-transport module
!
!***************************************************************************************************
!   Fortran90
!   2019-03-19
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Uppsala, Montpellier, Lyon, Naples
!***************************************************************************************************
! MODULES:
!   rhd_shortrad_module:   Short-characteristics radiative transfer routines (global models)
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Type of short-characteristics operator:                                             --- */
!# /*     0: simple test operator, fast but results are utterly useless.                          */
!# /*     1: case distinction with 'if..then..else' construct                                     */
!# /*     2: case distinction with masks (weights 0.0 or 1.0)                                     */
!# /*     3: case distinction with 'if..then..else' construct, S slope reduction                  */
!# /*     4: case distinction with masks (weights 0.0 or 1.0), S slope reduction                  */
!# /*     5: case distinction with 'if..then..else' construct, modified S slope reduct., default  */
!# /*     6: case distinction with masks (weights 0.0 or 1.0), modified S slope reduction         */
# ifndef rhd_shortrad_operator_l01
#   define rhd_shortrad_operator_l01 5
# endif
!#
!# /* --- Calling of short-characteristics operator:                                          --- */
!# /*     0: Call subroutine with scalar step                                                     */
!# /*     1: Include program fragment                                                             */
# ifndef rhd_shortrad_operator_l02
#   define rhd_shortrad_operator_l02 1
# endif
!#
!# /* --- Type of short-characteristics tau-coupling operator:                                --- */
!# /*     Parameters as for rhd_shortrad_operator_l01                                             */
# ifndef rhd_shortrad_dtauop_l01
#   if ((rhd_shortrad_operator_l01 == 1) || (rhd_shortrad_operator_l01 == 3) || (rhd_shortrad_operator_l01 == 5))
#     define rhd_shortrad_dtauop_l01 1
#   else
#     define rhd_shortrad_dtauop_l01 2
#   endif
# endif
!#
!# /* --- Calling of short-characteristics tau-coupling operator:                             --- */
!# /*     Parameters as for rhd_shortrad_operator_l02                                             */
# ifndef rhd_shortrad_dtauop_l02
#   define rhd_shortrad_dtauop_l02 rhd_shortrad_operator_l02
# endif
!#
!# /* --- MPI activation                                                                      --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* --- Routine version for rays in dir1 direction:                                         --- */
!# /*     0: Use routine with permuted indices for rays in x1 direction                           */
!# /*     1: default: transpose arrays and use routine rhd_shortrad_Dir3 for rays in x1 dir.      */
!# /*     2: Transpose arrays serially and use rhd_shortrad_Dir3 for rays in x1 direction         */
# ifndef rhd_shortrad_dir1_l01
#   define rhd_shortrad_dir1_l01 1
# endif
!# /* --- Don't use not-transposed version of rhd_shortrad_Dir1 with MPI --- */
# if (rhd_mpi_l01 == 1) && (rhd_shortrad_dir1_l01 == 0)
#   define rhd_shortrad_dir1_l01 1
# endif
!#
!# /* --- PARALLEL statement relative to outer loop in rhd_shortrad_dirX:                     --- */
!# /*     0: PARALLEL statement inside of outer loop in rhd_shortrad_dirX                         */
!# /*     1: default: PARALLEL statement outside of outer loop in rhd_shortrad_dirX               */
# ifndef rhd_shortrad_dir_l02
#   define rhd_shortrad_dir_l02 1
# endif
!#
!# /* --- Handling of extra arrays to allow partially implicit Lambda* iteration              --- */
!# /*     0: default: Only fully implicit Lambda* iteration allowed                               */
!# /*     1: general case: allow also partially implicit Lambda* iteration                        */
# ifndef rhd_shortrad_lambda_l01
#   define rhd_shortrad_lambda_l01 0
# endif
!#
!# /* --- Allocation of local 3D arrays                                                       --- */
!# /*     0: default: declaration in header of subroutine                                         */
!# /*     1: pointer arrays                                                                       */
!# /*     2: allocatable arrays                                                                   */
# ifndef rhd_arrays_l01
#   define rhd_arrays_l01 0
# endif
!#
!# /* --- Radiation pressure:                                                                 --- */
!# /*     0: Arrays and terms not included                                                        */
!# /*     1: Arrays and terms (partially) included                                                */
# ifndef rhd_shortrad_radpressure_l01
#   define rhd_shortrad_radpressure_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*******************--------------------------------------------------------------------------
module rhd_shortrad_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_module ('rhd_short-characteristics_radiation_module')
!
! PURPOSE:
!   Solve radiative transfer for rhd with short-characteristics method.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   use rhd_shortrad_module
!
! VARIABLES:
!   None
!
! ROUTINES: (all contained routines; public ones are marked with '>')
! > rhd_shortrad_SwitchInfo: Print information about used compiler switches.
!   rhd_shortrad_RayWeight: Compute weights for incoming rays
!   rhd_shortrad_OpTa:     Compute opacities and source function needed for radiative transfer
!   rhd_shortrad_OpTaTest: Compute opacities with various subroutines and measure the timings
!   --- MPI-only routines ---
!   rhd_shortrad_MPIDir2Int1: Transfer x1 boundary intensity between processes for x2 integration
!   rhd_shortrad_MPIDir2Int3: Transfer x3 boundary intensity between processes for x2 integration
!   rhd_shortrad_MPIDir3Int1: Transfer x1 boundary intensity between processes for x3 integration
!   rhd_shortrad_MPIDir3Int2: Transfer x2 boundary intensity between processes for x3 integration
!   -------------------------
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   rhd_shortrad_dtauop:   Compute contribution to radiative time scale from optical thickness
!   rhd_shortrad_DirSimple1: Compute radiative energy flux, simple direction field, main direction 1
!   rhd_shortrad_DirSimple2: Compute radiative energy flux, simple direction field, main direction 2
!   rhd_shortrad_DirSimple3: Compute radiative energy flux, simple direction field, main direction 3
!   rhd_shortrad_Dir1:     Compute radiative energy flux for one direction field, main direction 1
!   rhd_shortrad_Dir2:     Compute radiative energy flux for one direction field, main direction 2
!   rhd_shortrad_Dir3:     Compute radiative energy flux for one direction field, main direction 3
!   rhd_shortrad_tauCouple1: Compute tau coupling to get radiative time scale, direction 1
!   rhd_shortrad_tauCouple2: Compute tau coupling to get radiative time scale, direction 2
!   rhd_shortrad_tauCouple3: Compute tau coupling to get radiative time scale, direction 3
!   rhd_shortrad_Formal:   Compute formal solution: 3D radiative energy flux and change
!   rhd_shortrad_StepOutput: Derive some averages as additional output quantities
!   rhd_shortrad_EOS:      Compute EOS quantities needed for radiative transfer
!   rhd_shortrad_EOSTest:  Compute EOS quantities with various subroutines and measure the timings
!   rhd_shortrad_LocalSource: Perform radiation update step employing local approximation
! > rhd_shortrad_SubStep:  Perform 3d radiative energy transfer step using short characteristics
! > rhd_shortrad_Step:     Perform 3D radiation step w. short-characteristics or local-source scheme
!
! CALLING TREE:
!   rhd_shortrad_SwitchInfo
!   |
!   rhd_shortrad_Step
!   + rhd_shortrad_SubStep
!     * rhd_baserad_RaySystem
!     | + rhd_baserad_RayDirections
!     | + rhd_baserad_BaseAxisSystem
!     |   . rhd_baserad_RotationMatrix
!     !   . rhd_baserad_RotationMatrixAxisAngle
!     !   . rhd_baserad_DirShuffle
!     * rhd_shortrad_EOS
!     | + eosinter_logPTdTOMP
!     * rhd_shortrad_Formal
!     !   rhd_shortrad_OpTa
!     |   . xBnu
!     |   . sub_xkaros_logPT2kappaB
!     |   . sub_xkaros_logPT2kappa
!     |   . kappadust
!     |   . op_dust2
!     | + rhd_shortrad_Dir1
!     |   . rhd_shortrad_DirSimple1
!     |       rhd_shortrad_operator
!     |   . rhd_shortrad_RayWeight
!     |   . rhd_shortrad_operator
!     | + rhd_shortrad_Dir2
!     |   . rhd_shortrad_DirSimple2
!     |       rhd_shortrad_MPIDir2Int1
!     |       rhd_shortrad_MPIDir2Int3
!     |       rhd_shortrad_operator
!     |   . rhd_shortrad_RayWeight
!     |   . rhd_shortrad_MPIDir2Int1
!     |   , rhd_shortrad_MPIDir2Int3
!     |   . rhd_shortrad_operator
!     | + rhd_shortrad_Dir3
!     |   . rhd_shortrad_DirSimple3
!     |       rhd_shortrad_MPIDir3Int1
!     |       rhd_shortrad_MPIDir3Int2
!     |       rhd_shortrad_operator
!     |   . rhd_shortrad_RayWeight
!     |   . rhd_shortrad_MPIDir3Int1
!     |   , rhd_shortrad_MPIDir3Int2
!     |   . rhd_shortrad_operator
!     | + rhd_shortrad_tauCouple1
!     |   . rhd_shortrad_dtauop
!     | + rhd_shortrad_tauCouple2
!     |   . rhd_shortrad_dtauop
!     | + rhd_shortrad_tauCouple3
!     |   . rhd_shortrad_dtauop
!     * rhd_shortrad_StepOutput
!  + rhd_box_MPICopy
!  + rhd_shortrad_LocalSource
!     * eosinter_TdT
!     * rhd_box_RestIndices
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F. Uppsala) First version
!   2001-11-24 (B.F.) First complete version for rays in all directions
!   2002-05-01 (B.F.) Accelerated Lambda iteration becomes more and more useful
!   2002-07-23 (B.F.) Changed weights for individual rays, changed overall weight factor
!   2002-07-24 (B.F.) Overall weight factor to old value
!   2002-08-02 (B.F.) Rays along axes or diagonals get special treatment; allow list of directions
!   2002-08-12 (B.F.) Information about compiler switches
!   2002-11-28 (B.F.) Individual radiation transport operators accessible via include
!   2002-12-02 (B.F.) Mask for optical thick regions is computed once and stored in array thickmask
!   2004-05-26 (B.F.) Radiation pressure array handling
!   2004-05-27 (B.F.) iX +/- 1 -> iXold
!   2005-03-08 (B.F.) Radiation pressure handling removed
!   2008-06-20 (B.F. Uppsala) Lambda* iteration reactivated
!   2008-06-24 (B.F.) Switch: rhd_shortrad_arrays_l01 -> rhd_arrays_l01
!   2008-08-20 (B.F. Lyon) Bugfix: re-insert energy update in downward integration in DirSimple1
!   2008-08-27 (B.F.) Radiation pressure handling re-inserted
!   2013-01-21 (B.F.) Simplify expression for integration index (i123old=i123+di123 -> +-1)
!   2013-01-22 (B.F.) Compute thickmask locally and remove 3D thickmask array
!   2014-02-13 (B.F. Uppsala) Rename some routines rhd_rad3d_XXX -> rhd_shortrad_XXX
!   2014-02-24 (B.F.) Change default value of some compiler switches
!   2014-03-05 (B.F.) First modifications for MPI
!   2014-03-15 (B.F.) New version of boundary-condition handling
!   2014-11-05 (B.F.) Copy additional density arrays into local quc array
!   2015-02-28 (B.F.) Some routines moved into rhd_baserad_module
!   2017-07-03 (B.F.) Continue with modifications for MPI
!   2017-07-27 (B.F.) Put MPI commands into all intensity-integration routines
!   2017-09-07 (B.F.) Prepare averaging routines for MPI
!   2017-09-19 (B.F.) Parallelize averaging routines with OpenMP (and MPI)
!   2017-10-03 (B.F.) Put lateral MPI boundary intensity exchange into separate routines
!   2017-10-04 (B.F.) Outcomment calls for MPI boundary intensity exchange (performance issues)
!   2018-11-28 (B.F.) Add wrapper routine that calls short-characteristics and local-source substeps
!   2018-11-29 (B.F.) Remove compiler switch to control loop splitting for exp(-dtau) computation
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
implicit none
!
private
!
public rhd_shortrad_SwitchInfo, rhd_shortrad_SubStep, rhd_shortrad_Step
!
contains

!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_SwitchInfo ('rhd_short-characteristics_radiation_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-08-12 (B.F.) Written
!   2010-03-31 (B.F.) Remove one space in some output lines
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_shortrad_module ....................'
!
write(ncp,'(A,I6)') '  rhd' // '_shortrad_operator_l01:  ', rhd_shortrad_operator_l01
write(ncp,'(A,I6)') '  rhd' // '_shortrad_operator_l02:  ', rhd_shortrad_operator_l02
write(ncp,'(A,I6)') '  rhd' // '_shortrad_dtauop_l01:    ', rhd_shortrad_dtauop_l01
write(ncp,'(A,I6)') '  rhd' // '_shortrad_dtauop_l02:    ', rhd_shortrad_dtauop_l02
write(ncp,'(A,I6)') '  rhd' // '_shortrad_dir1_l01:      ', rhd_shortrad_dir1_l01
write(ncp,'(A,I6)') '  rhd' // '_shortrad_dir_l02:       ', rhd_shortrad_dir_l02
! write(ncp,'(A,I6)') '  rhd' // '_shortrad_lambda_l01:    ', rhd_shortrad_lambda_l01
write(ncp,'(A,I6)') '  rhd' // '_arrays_l01:             ', rhd_arrays_l01
write(ncp,'(A,I5)') '  rhd' // '_shortrad_radpressure_l01:', rhd_shortrad_radpressure_l01
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
!
!#
# ifdef rhd_shortrad_formal_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_shortrad_formal_t01: ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_shortrad_formal_t01: ', undefstr
!#
# endif
!#
!
! !#
! # ifdef rhd_shortrad_lambda_t01
! !#
! write(ncp,'(A,A)' ) '  rhd' // '_shortrad_lambda_t01: ', defstr
! !#
! # else
! !#
! write(ncp,'(A,A)' ) '  rhd' // '_shortrad_lambda_t01: ', undefstr
! !#
! # endif
! !#
!
!#
# ifdef rhd_shortrad_step_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_shortrad_step_t01:   ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_shortrad_step_t01:   ', undefstr
!#
# endif
!#
!
end subroutine rhd_shortrad_SwitchInfo


!----------**********************-------------------------------------------------------------------
subroutine rhd_shortrad_RayWeight(r_dir1, r_dir2, r_dir3, di2, di3, w00, w01, w10, w11)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_RayWeight ('rhd_short-characteristics_ray_weight')
!
! PURPOSE:
!   Compute weights for incoming rays.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_RayWeight(r_dir1, r_dir2, r_dir3, di2, di3, w00, w01, w10, w11)
!
! INPUT:
!   r_dir1:       ('ray_direction_1') real with 1st component of ray direction (main)
!   r_dir2:       ('ray_direction_2') real with 2nd component of ray direction
!   r_dir3:       ('ray_direction_3') real with 3rd component of ray direction
!
! OUTPUT:
!   w00:          ('weight_00') real, weight of neighbor cell (di2,di3)+(0,0)
!   w10:          ('weight_10') real, weight of neighbor cell (di2,di3)+(1,0)
!   w01:          ('weight_01') real, weight of neighbor cell (di2,di3)+(0,1)
!   w11:          ('weight_11') real, weight of neighbor cell (di2,di3)+(1,1)
!
! LOCAL VARIABLES:
!   wi2:          ('weight_index_2') real, weight of left neighbor cells in x2 direction
!   wi3:          ('weight_index_3') real, weight of left neighbor cells in x3 direction
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions, cells must be cubic: dx1=dx2=dx3.
!   First direction coordinate must be positive: r_dir1 > 0.0.
!
! PROCEDURE:
!   Compute interpolation weights for all 4 parallel rays hitting a cell for given direction
!
!   Weight indices of relevant neighbor cells in old layer:
!
!     ^i3
!     |
!     +------+------+
!     |      |      |
!     |  01  |  11  |
!     |      |      |
!     +------+------+
!     |      |      |
!     |  00  |  10  |
!     |      |      |
!     +------+------+->i2
!
!   Position of relevant neighbor cells in old layer relativ to cell i in new layer:
!
!     di2=-1, di3=-1   +---+---+     di2= 0, di3=-1   +---+---+
!                      |   | i |                      | i |   |
!                      +---+---+                      +---+---+
!                      |   |   |                      |   |   |
!                      +---+---+                      +---+---+
!
!     di2=-1, di3= 0   +---+---+     di2= 0, di3= 0   +---+---+
!                      |   |   |                      |   |   |
!                      +---+---+                      +---+---+
!                      |   | i |                      | i |   |
!                      +---+---+                      +---+---+
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-07-23 (B.F.) Put into own routine; weights squared
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: r_dir1, r_dir2, r_dir3
integer,                   intent(out)  :: di2, di3
real,                      intent(out)  :: w00, w01, w10, w11
!
! --- Local variables ---
real                                    :: wi2, wi3
!---------------------------------------------------------------------------------------------------
!
! --- Find indices of relevant cells in old plane ---
if (r_dir2 < 0) then
  di2=-1
!!$  wi2=-r_dir2/r_dir1
  wi2=(r_dir2/r_dir1)**2
else
  di2= 0
!!$  wi2=1.0 - r_dir2/r_dir1
  wi2=1.0 - (r_dir2/r_dir1)**2
endif
!
if (r_dir3 < 0) then
  di3=-1
!!$  wi3=-r_dir3/r_dir1
  wi3=(r_dir3/r_dir1)**2
else
  di3= 0
!!$  wi3=1.0 - r_dir3/r_dir1
  wi3=1.0 - (r_dir3/r_dir1)**2
endif
!
! --- Determine weights of neighbor cells ---
w00=     wi2 *     wi3
w01=     wi2 *(1.0-wi3)
w10=(1.0-wi2)*     wi3
w11=(1.0-wi2)*(1.0-wi3)
!
end subroutine rhd_shortrad_RayWeight


!----------*****************------------------------------------------------------------------------
subroutine rhd_shortrad_OpTa(m1,n1, m2,n2, m3,n3, nquc, nu, &
                             ComputeIntensity_flag, Computekappa_flag, &
                             T, logP, rho, rhokappa, wTb, S, quc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_OpTa ('rhd_short-characteristics_radiation_Opacity_Table')
!
! PURPOSE:
!   Compute opacities and source function needed for radiative transfer.
!
! CATEGORY:
!   Radiative transfer, opacities
!
! CALLING SEQUENCE:
!   call rhd_shortrad_OpTa(m1,n1, m2,n2, m3,n3, nquc, nu, &
!                          ComputeIntensity_flag, Computekappa_flag, &
!                          T, logP, rho, rhokappa, wTb, S, quc)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nquc:         ('number_quantities_center') integer, number of additional density arrays
!   nu:           ('nu') integer with frequency index
!   ComputeIntensity_flag: ('Compute_Intensity_flag') logical, activate intensity computation
!   Computekappa_flag: ('Compute_kappa_flag') logical, activate computation of new opacity
!   T:            ('Temperature') real 3D array with temperature
!   logP:         ('logarithm_Pressure') real 3D array with natural logarithm of pressure,
!   rho:          ('rho') real 3D array with density, attention: one buffer cell in all directions
!   quc:          ('quantity_center') real 4D array with list of centered variables (optional)
!
! OUTPUT:
!   rhokappa:     ('rho_kappa') real 3D array with opacities
!   wTb:          ('weight_Temperature_b') real 3D array with fraction of B_nu
!   S:            ('Source') real 3D array with source function
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   xBnu:                  Compute the fraction of the Planck function B_nu
!   sub_xkaros_logPT2kappa: Interpolate opacity per mass for given logP, T, and nu bin (1D,2D,3D)
!   sub_xkaros_logPT2rhokap: Interpolate opacity per volume for given logP, T, and nu bin (1D,2D,3D)
!   kappadust:             Compute Rosseland dust opacity for amorph. carbon in Small-Particle Limit
!   op_dust2:              Compute dust opacity for various dust models for one vector
!
! MODULES:
!   op_dust_module:        Dust opacity routines
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   const_module:          Global physical and mathematical constants and units
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The gas opacity data must be read (and stored in opta_module) before calling this routine.
!
! PROCEDURE:
!   Compute source function and (gas + dust) opacities.
!   Gas opacities are computed by interpolation in a table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-03-01 (B.F. Lyon) Written
!   2013-03-12 (B.F.) Call op_dust1
!   2013-03-13 (B.F.) Non-grey opacities
!   2013-12-11 (B.F.) Allow periodic side boundaries
!   2014-11-05 (B.F.) Input array quc gets buffer cells
!---------------------------------------------------------------------------------------------------
use opta_module
use const_module, only: sigopi
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
!#
# if (rhd_box_quc01 != 0)
!#
use op_dust_module
!#
# endif
!#      /* rhd_box_quc01 */
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nquc, nu
logical,                   intent(in)   :: ComputeIntensity_flag, Computekappa_flag
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(in)   :: T, logP, rho
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1, &
                1   :nquc   ), &
                 optional, intent(in)   :: quc
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(out)  :: rhokappa, wTb, S
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real, dimension(m1  :n1     )           :: opadus
!---------------------------------------------------------------------------------------------------
!
!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, i3, opadus)
!
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-1,n3+1
  do i2=m2-1,n2+1
    if ((ComputeIntensity_flag) .and. ((nu == 0) .or. (.not. Computekappa_flag))) then
      ! --- Compute source function S ---
      !
      if (nu == 0) then
        ! --- Compute grey source function ---
        !
        ! --- Compute source function S ---
        do i1=m1-1,n1+1
          S(i1,i2,i3)=sigopi * (T(i1,i2,i3)**2)**2
        end do ! i1
        !
      else ! (nu > 0)
        ! --- Compute non-grey source function ---
        !
        ! --- Compute bin weight ---
        call xBnu(T(:,i2,i3), nu, wTb(:,i2,i3))
        !
        ! --- Compute source function S ---
        do i1=m1-1,n1+1
          S(i1,i2,i3)=sigopi * wTb(i1,i2,i3) * (T(i1,i2,i3)**2)**2
        end do ! i1
        !
      endif ! (nu)
    endif ! (ComputeIntensity_flag)
    !
    if (Computekappa_flag) then
      ! --- Compute opacities ---
      !
      if ((ComputeIntensity_flag) .and. (nu > 0)) then
        !DIR$ FORCEINLINE
        call sub_xkaros_logPT2rhokapB(logP(:,i2,i3), T(:,i2,i3), rho(:,i2,i3), nu, &
                                      rhokappa(:,i2,i3), wTb( :,i2,i3), S(:,i2,i3))
      else
        !DIR$ FORCEINLINE
        call sub_xkaros_logPT2rhokap(logP(:,i2,i3), T(:,i2,i3), rho(:,i2,i3), nu, &
                                     rhokappa(:,i2,i3))
      endif
!#
#     if (rhd_box_quc01 != 0)
!#
      if ((nquc            >  0     ) .and. &
          (par%dustopamode /= 'None')        ) then
        ! --- Compute and add dust opacities ---\label{f90:dustscheme__use5}
        if ((par%C_dust04    >   0.0              ) .and. &
            (par%dustScheme  == 'dust_moment04_c2')        ) then
          ! --- Compute and add dust opacities (amorphous Carbon for AGB models) ---
          !
          !DIR$ FORCEINLINE
          call kappadust(n1-m1+3, T(m1-1:n1+1,i2,i3), &
                         quc(m1-1:n1+1, i2, i3, 4), &
                         opadus)
          rhokappa(m1-1:n1+1,i2,i3)=rhokappa(m1-1:n1+1,i2,i3) + par%C_dust04*opadus
          !
        else ! (par%dustScheme)
          !DIR$ FORCEINLINE
          call op_dust2(T, rho, quc, &
                        nu, m1,n1, m2,n2, m3,n3, nquc, &
                        i2, i3, &
                        rhokappa)
          !
        endif ! (par%dustScheme)
      endif ! (nquc)
!#
#     endif
!#          /* rhd_box_quc01 */
    endif ! (Computekappa_flag)
    !
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
end subroutine rhd_shortrad_OpTa


!----------*********************--------------------------------------------------------------------
subroutine rhd_shortrad_OpTaTest(model, rho, T, logP, rhokappa, wTb, S, &
                                 nu, ComputeIntensity_flag, Computekappa_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_OpTaTest ('rhd_short-characteristics_radiation_Opacity_Table_test')
!
! PURPOSE:
!   Compute opacities with various subroutines and measure the timings.
!
! CATEGORY:
!   Radiative transfer, opacities
!
! CALLING SEQUENCE:
!   call rhd_shortrad_OpTaTest(model, rho, T, logP, rhokappa, wTb, S, &
!                              nu, ComputeIntensity_flag, Computekappa_flag)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!   rho:          ('rho') real 3D array with density, attention: one buffer cell in all directions
!   T:            ('Temperature') real 3D array with temperature, attention: one buffer cell
!   logP:         ('logarithm_Pressure') real 3D array with natural logarithm of pressure,
!                   attention: one buffer cell
!   nu:           ('nu') integer with frequency index
!   ComputeIntensity_flag: ('Compute_Intensity_flag') logical, activate intensity computation
!   Computekappa_flag: ('Compute_kappa_flag') logical, activate computation of new opacity
!
! OUTPUT:
!   rhokappa:     ('rho_kappa') real 3D array with opacities
!   wTb:          ('weight_Temperature_b') real 3D array with fraction of B_nu
!   S:            ('Source') real 3D array with source function
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   kappadust:             Compute Rosseland dust opacity for amorph. carbon in Small-Particle Limit
!   rhd_shortrad_OpTa:     Compute opacities and source function needed for radiative transfer
!   sub_xkaros3lin_logPT2kappa: Compute opacities with linear interpolation
!   sub_xkaros3veclin_logPT2kappa: Compute opacities with linear interpolation, different loops
!   sub_xkaros3veclin2_logPT2kappa: Compute opacities with linear interpolation, different loops
!   sub_xkaros3vec22_logPT2kappa: Compute opacities with cubic interpolation, different loops
!   sub_xkaros3vec2_logPT2kappa: Compute opacities with cubic interpolation, different loops
!   sub_xkaros3vec_logPT2kappa: Compute opacities with cubic interpolation, different loops
!   sub_xkaros_logPT2kappa: Interpolate opacity per mass for given logP, T, and nu bin (1D,2D,3D)
!   sub_xkaros_logPT2rhokap: Interpolate opacity per volume for given logP, T, and nu bin (1D,2D,3D)
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   xBnu:                  Compute the fraction of the Planck function B_nu
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   const_module:          Global physical and mathematical constants and units
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_box_module:        Box-handling routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Timing statistics are gathered.
!   Some output is written to screen.
!
! RESTRICTIONS:
!   A proper opacity table has to be read beforehand.
!   There are several routine calls only for the grey case (nu=0).
!   This routines should not be called during normal production runs: it is meant for
!   performance measurements and testing only.
!
! PROCEDURE:
!   Compute opacity data by various variants of the opacity-table interpolation routine
!   and time the calls.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-02-27 (B.F. Uppsala) Put into separate routine
!---------------------------------------------------------------------------------------------------
use opta_module
use const_module, only: sigopi
use rhd_gl_module
use rhd_box_module
use timing_module
!#
# if (rhd_box_quc01 != 0)
!#
use op_dust_module
!#
# endif
!#      /* rhd_box_quc01 */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real, dimension(model%m1-1:model%n1+1, &
                model%m2-1:model%n2+1, &
                model%m3-1:model%n3+1 ), &
                           intent(in)   :: rho, T, logP
real, dimension(model%m1-1:model%n1+1, &
                model%m2-1:model%n2+1, &
                model%m3-1:model%n3+1   ), &
                           intent(out)  :: rhokappa, wTb, S
integer,                   intent(in)   :: nu
logical,                   intent(in)   :: ComputeIntensity_flag, Computekappa_flag
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3
real, dimension(model%m1  :model%n1   ) :: opadus
real, dimension(model%m1-1:model%n1+1, &
                model%m2-1:model%n2+1, &
                model%m3-1:model%n3+1 ) :: T1, P1
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
if (nu == 0) then
  ! --- Grey opacities ---
  !
  if (ComputeIntensity_flag) then
    ! --- Compute source function S ---
    !
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1,n3+1
      do i2=m2-1,n2+1
        do i1=m1-1,n1+1
          S(i1,i2,i3)=sigopi * (T(i1,i2,i3)**2)**2
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
  endif ! (ComputeIntensity_flag)
  !
  if (Computekappa_flag) then
    ! --- Compute grey opacities ---
    !
    call timing_start('SHC: opa-lin')
    !$OMP PARALLEL DEFAULT(SHARED)
    call sub_xkaros3lin_logPT2kappa(logP, T, 0, rhokappa)
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-lin')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    call timing_start('SHC: opa-veclin')
    !$OMP PARALLEL DEFAULT(SHARED)
    call sub_xkaros3veclin_logPT2kappa(logP, T, 0, rhokappa)
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-veclin')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    call timing_start('SHC: opa-veclin2')
    !$OMP PARALLEL DEFAULT(SHARED)
    call sub_xkaros3veclin2_logPT2kappa(logP, T, 0, rhokappa)
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-veclin2')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    P1=logP*0.0001
    T1=T*0.01+20000.0
    call timing_start('SHC: opa-vec2-scale')
    !$OMP PARALLEL DEFAULT(SHARED)
    call sub_xkaros3vec22_logPT2kappa(P1, T1, 0, rhokappa)
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-vec2-scale')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3), &
               ' : different data'
    !
    call timing_start('SHC: opa-vec2')
    !$OMP PARALLEL DEFAULT(SHARED)
    call sub_xkaros3vec2_logPT2kappa(logP, T, 0, rhokappa)
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-vec2')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    call timing_start('SHC: opa-vec')
    !$OMP PARALLEL DEFAULT(SHARED)
    call sub_xkaros3vec_logPT2kappa(logP, T, 0, rhokappa)
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-vec')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    call timing_start('SHC: opa-xkaros1')
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i3)
    !$OMP DO
    do i3=m3-1,n3+1
      do i2=m2-1,n2+1
        call sub_xkaros_logPT2kappa(logP(:,i2,i3), T(:,i2,i3), 0, rhokappa(:,i2,i3))
      end do ! i2
    end do ! i3
    !$OMP END DO
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-xkaros1')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    call timing_start('SHC: opa-xkaros1-rhokap')
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i3)
    !$OMP DO
    do i3=m3-1,n3+1
      do i2=m2-1,n2+1
        call sub_xkaros_logPT2rhokap(logP(:,i2,i3), T(:,i2,i3), rho(:,i2,i3), 0, rhokappa(:,i2,i3))
        rhokappa(:,i2,i3)=rhokappa(:,i2,i3)/rho(:,i2,i3)
      end do ! i2
    end do ! i3
    !$OMP END DO
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-xkaros1-rhokap')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    call timing_start('SHC: opa-xkaros2')
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i3)
    !$OMP DO
    do i3=m3-1,n3+1
      call sub_xkaros_logPT2kappa(logP(:,:,i3), T(:,:,i3), 0, rhokappa(:,:,i3))
    end do ! i3
    !$OMP END DO
    !$OMP END PARALLEL
    call timing_stop('SHC: opa-xkaros2')
    write(gl%nc_p,*) 'kappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
    ! --- Old standard call ---
    !
    call timing_start('SHC: opa')
    !
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, opadus)
    !
    call sub_xkaros_logPT2rhokap(logP, T, rho, 0, rhokappa)
    !
#   if (rhd_box_quc01 != 0)
    if ((par%C_dust04 > 0.0) .and. (par%dustScheme == 'dust_moment04_c2') .and. &
        (model%nquc > 0)                                                         ) then
      ! --- Compute and add dust opacities ---
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i2=m2,n2
          call kappadust(n1-m1+1, T(m1:n1,i2,i3), model%quc(m1:n1,i2,i3,4), &
                         opadus)
          rhokappa(m1:n1,i2,i3)=rhokappa(m1:n1,i2,i3) + par%C_dust04*opadus
          rhokappa(m1-1 ,i2,i3)=rhokappa(m1-1 ,i2,i3) + par%C_dust04*opadus(m1)
          rhokappa(n1+1 ,i2,i3)=rhokappa(n1+1 ,i2,i3) + par%C_dust04*opadus(n1)
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
#   endif
!#       /* rhd_box_quc01 */
    !$OMP END PARALLEL
    !
    call timing_stop('SHC: opa')
    write(gl%nc_p,*) 'rhokappa: ', &
               rhokappa(m1,m2,m3), rhokappa((m1+n1)/2, (m2+n2)/2, (m3+n3)/2), rhokappa(n1,n2,n3)
    !
  endif ! (Computekappa_flag)
else ! (nu) ----------------------------------------------------------------------------------------
  ! --- Non-grey opacities ---
  !
  !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, opadus)
    !
    if (ComputeIntensity_flag) then
      ! --- Compute bin weight ---
      call xBnu(T, nu, wTb)
      !
!call uio_openwr(nc, 'rhd_short_formal_test.out', form='unformatted', conv='ieee_4', &
!                position='append')
!call uio_wrlabl(nc, dataset_ident, date='now' )
!call uio_wrlabl(nc, box_ident                 )
!call uio_wr(nc, 'z'          , box_id_ident   )
!call uio_wr(nc, xc1          , 'xc1'          )
!call uio_wr(nc, xb1          , 'xb1'          )
!call uio_wr(nc, xc2          , 'xc2'          )
!call uio_wr(nc, xb2          , 'xb2'          )
!call uio_wr(nc, xc3          , 'xc3'          )
!call uio_wr(nc, xb3          , 'xb3'          )
!call uio_wr(nc, wTb          , 'wtb'          )
!call uio_wrlabl(nc, endbox_ident)
!call uio_wrlabl(nc, enddataset_ident)
!call uio_closrd(nc)
    !
    ! --- Compute source function S ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1,n3+1
      do i2=m2-1,n2+1
        do i1=m1-1,n1+1
          S(          i1,i2,i3)=sigopi * wTb(i1,i2,i3) * (T(i1,i2,i3)**2)**2
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  endif ! (ComputeIntensity_flag)
  !
  ! --- Opacities for bin nu ---
  if (Computekappa_flag) then
    call timing_start('SHC: opa')
    !
    call sub_xkaros_logPT2rhokap(logP, T, rho, nu, rhokappa)
    !
#   if (rhd_box_quc01 != 0)
    if ((par%C_dust04 > 0.0) .and. (par%dustScheme == 'dust_moment04_c2') .and. &
        (model%nquc > 0)                                                         ) then
      ! --- Compute and add dust opacities ---
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i2=m2,n2
          call kappadust(n1-m1+1, T(m1:n1,i2,i3), model%quc(m1:n1,i2,i3,4), &
                         opadus)
          rhokappa(m1:n1,i2,i3)=rhokappa(m1:n1,i2,i3) + par%C_dust04*opadus
          rhokappa(m1-1 ,i2,i3)=rhokappa(m1-1 ,i2,i3) + par%C_dust04*opadus(m1)
          rhokappa(n1+1 ,i2,i3)=rhokappa(n1+1 ,i2,i3) + par%C_dust04*opadus(n1)
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
#   endif
!#        /* rhd_box_quc01 */
    call timing_stop('SHC: opa')
    !
  endif ! (Computekappa_flag)
  !
  !$OMP END PARALLEL
endif ! (nu)
!
write(gl%nc_p,*) 'rhokappa: ', rhokappa(m1,m2,m3), &
                         rhokappa((m1+n1)/2,(m2+n2)/2,(m3+n3)/2), &
                         rhokappa(n1,n2,n3)
write(gl%nc_p,*) 'kappa: ', 1.0/rho(m1,m2,m3)*rhokappa(m1,m2,m3), &
                  1.0/rho((m1+n1)/2,(m2+n2)/2,(m3+n3)/2)*rhokappa((m1+n1)/2,(m2+n2)/2,(m3+n3)/2), &
                      1.0/rho(n1,n2,n3)*rhokappa(n1,n2,n3)
write(gl%nc_p,*) 'S:     ', S(m1,m2,m3), &
                         S((m1+n1)/2,(m2+n2)/2,(m3+n3)/2), &
                         S(n1,n2,n3)
call timing_start('SHC: OpTa')
!
call rhd_shortrad_OpTa(m1,n1, m2,n2, m3,n3, model%nquc, nu, &
                       ComputeIntensity_flag, Computekappa_flag, &
                       T, logP, rho, rhokappa, wTb, S, model%quc)
!
call timing_stop('SHC: OpTa')
!
write(gl%nc_p,*) 'rhokappa: ', rhokappa(m1,m2,m3), &
                         rhokappa((m1+n1)/2,(m2+n2)/2,(m3+n3)/2), &
                         rhokappa(n1,n2,n3)
write(gl%nc_p,*) 'kappa: ', 1.0/rho(m1,m2,m3)*rhokappa(m1,m2,m3), &
                  1.0/rho((m1+n1)/2,(m2+n2)/2,(m3+n3)/2)*rhokappa((m1+n1)/2,(m2+n2)/2,(m3+n3)/2), &
                      1.0/rho(n1,n2,n3)*rhokappa(n1,n2,n3)
write(gl%nc_p,*) 'S:     ', S(m1,m2,m3), &
                         S((m1+n1)/2,(m2+n2)/2,(m3+n3)/2), &
                         S(n1,n2,n3)
!
end subroutine rhd_shortrad_OpTaTest


!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ======================================================== */


!----------************************-----------------------------------------------------------------
subroutine rhd_shortrad_MPIDir2Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
                                    I1l, I1r, di1, di1max, bound_l1, bound_r1, i2old, &
                                    tagMPI, RequestArrMPI)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_MPIDir2Int1 ('rhd_short-characteristics_radiation_MPI_Direction_2_Intensity_1')
!
! PURPOSE:
!   Transfer x1 boundary intensity between MPI processes for integration in x2 direction.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_MPIDir2Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!                                 I1l, I1r, di1, di1max, bound_l1, bound_r1, i2old, &
!                                 tagMPI, RequestArrMPI)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   di1:          ('delta_index_1') integer, relative index of leftmost important neighbor cell
!                 in x1 direction (-1, 0, or 1)
!   di1max:       ('delta_index_1_maximum') integer, maximum value of di1:
!                   0: Dir2
!                   1: DirSimple2
!   bound_l1:     ('boundary_left_1') integer, left boundary type for dimension 1;
!                 values: see rhd_action_module
!   bound_r1:     ('boundary_right_1') integer, right boundary type for dimension 1;
!                 values: see rhd_action_module
!   i2old:        ('index_2_old') integer, temporary variable of x2 index of old cell
!   tagMPI:       ('tag_MPI') integer with MPI tag; 1 or 2 will be added
!
! INPUT/OUTPUT:
!   I:            ('Intensity') 3D real array with intensity values at cell centers
!   I1l:          ('Intensity_1_left') 1D real array with left boundary intensities
!   I1r:          ('Intensity_1_right') 1D real array with right boundary intensities
!   RequestArrMPI: ('Request_Array_MPI') 1D integer array with MPI request handles
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   StatusArrMPI: ('Status_Array_MPI') 2D integer array with MPI status
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_WaitAll:           Wait for all given communications to complete
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Communicate with neighbor MPI processes.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Check if there is a relevant internal boundary.
!   If yes, transfer boundary intensity with MPI_ISend, MPI_IRecv, MPI_WaitAll.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-10-03 (B.F. Uppsala) Extracted from rhd_shortrad_Dir2
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use rhd_action_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           di1, di1max, bound_l1, bound_r1, i2old, tagMPI
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(inout):: I
real, dimension(m3-1:n3+1),intent(inout):: I1l, I1r
integer, dimension(4),     intent(inout):: RequestArrMPI
!
! --- Local variables ---
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!---------------------------------------------------------------------------------------------------
!
! --- Issue non-blocking send requests for boundary values to neighbor processes ---
!
! --- Initialize request arrays: nothing has happened, yet ---
RequestArrMPI(1)=MPI_REQUEST_NULL
RequestArrMPI(2)=MPI_REQUEST_NULL
!
if (di1 == -1) then
  if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I1r=I(n1, i2old, :)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
    !   ' in Dir=', action%dimV(1), &
    !   ' tag=', tagMPI+1
    call MPI_ISend(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(1), ierrMPI)
  endif ! (bound_r1)
  !
  if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(2), ierrMPI)
  endif ! (bound_l1)
  !
else if (di1 == di1max) then
  if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I1l=I(m1, i2old, :)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
    !   ' in Dir=', action%dimV(1), &
    !   ' tag=', tagMPI+2
    call MPI_ISend(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(1), ierrMPI)
  endif ! (bound_l1)
  !
  if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(2), ierrMPI)
  endif ! (bound_r1)
  !
endif ! (di1)
!
! --- Wait for non-blocking send/receive requests for boundary values ---
! --- from neighbor processes to finish                               ---
call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
!
end subroutine rhd_shortrad_MPIDir2Int1


!----------************************-----------------------------------------------------------------
subroutine rhd_shortrad_MPIDir2Int3(geo, action, I, m1,n1, m2,n2, m3,n3, &
                                    I3l, I3r, di3, di3max, bound_l3, bound_r3, i2old, &
                                    tagMPI, RequestArrMPI)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_MPIDir2Int3 ('rhd_short-characteristics_radiation_MPI_Direction_2_Intensity_3')
!
! PURPOSE:
!   Transfer x3 boundary intensity between MPI processes for integration in x2 direction.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_MPIDir2Int3(geo, action, I, m1,n1, m2,n2, m3,n3, &
!                                 I3l, I3r, di3, di3max, bound_l3, bound_r3, i2old, &
!                                 tagMPI, RequestArrMPI)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   di3:          ('delta_index_3') integer, relative index of leftmost important neighbor cell
!                 in x3 direction (-1, 0, or 1)
!   di3max:       ('delta_index_3_maximum') integer, maximum value of di3:
!                   0: Dir2
!                   1: DirSimple2
!   bound_l3:     ('boundary_left_3') integer, left boundary type for dimension 3;
!                 values: see rhd_action_module
!   bound_r3:     ('boundary_right_3') integer, right boundary type for dimension 3;
!                 values: see rhd_action_module
!   i2old:        ('index_2_old') integer, temporary variable of x2 index of old cell
!   tagMPI:       ('tag_MPI') integer with MPI tag; 1 or 2 will be added
!
! INPUT/OUTPUT:
!   I:            ('Intensity') 3D real array with intensity values at cell centers
!   I3l:          ('Intensity_3_left') 1D real array with left boundary intensities
!   I3r:          ('Intensity_3_right') 1D real array with right boundary intensities
!   RequestArrMPI: ('Request_Array_MPI') 1D integer array with MPI request handles
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   StatusArrMPI: ('Status_Array_MPI') 2D integer array with MPI status
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_WaitAll:           Wait for all given communications to complete
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Communicate with neighbor MPI processes.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Check if there is a relevant internal boundary.
!   If yes, transfer boundary intensity with MPI_ISend, MPI_IRecv, MPI_WaitAll.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-10-03 (B.F. Uppsala) Extracted from rhd_shortrad_Dir2
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use rhd_action_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           di3, di3max, bound_l3, bound_r3, i2old, tagMPI
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(inout):: I
real, dimension(m1-1:n1+1),intent(inout):: I3l, I3r
integer, dimension(4),     intent(inout):: RequestArrMPI
!
! --- Local variables ---
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!---------------------------------------------------------------------------------------------------
!
! --- Issue non-blocking send requests for boundary values to neighbor processes ---
!
! --- Initialize request arrays: nothing has happened, yet ---
RequestArrMPI(3)=MPI_REQUEST_NULL
RequestArrMPI(4)=MPI_REQUEST_NULL
!
if (di3 == -1) then
  if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I3r=I(:, i2old, n3)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I3r     to=', geo%Proc_r(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', tagMPI+1
    call MPI_ISend(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(3), ierrMPI)
  endif ! (bound_r3)
  !
  if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(4), ierrMPI)
  endif ! (bound_l3)
  !
else if (di3 == di3max) then
  if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I3l=I(:, i2old, m3)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I3l     to=', geo%Proc_l(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', tagMPI+2
    call MPI_ISend(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(3), ierrMPI)
  endif ! (bound_l3)
  !
  if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(4), ierrMPI)
  endif ! (bound_r3)
  !
endif ! (di3)
!
! --- Wait for non-blocking send/receive requests for boundary values ---
! --- from neighbor processes to finish                               ---
call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
!
end subroutine rhd_shortrad_MPIDir2Int3


!----------************************-----------------------------------------------------------------
subroutine rhd_shortrad_MPIDir3Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
                                    I1l, I1r, di1, di1max, bound_l1, bound_r1, i3old, &
                                    tagMPI, RequestArrMPI)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_MPIDir3Int1 ('rhd_short-characteristics_radiation_MPI_Direction_3_Intensity_1')
!
! PURPOSE:
!   Transfer x1 boundary intensity between MPI processes for integration in x3 direction.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_MPIDir3Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!                                 I1l, I1r, di1, di1max, bound_l1, bound_r1, i3old, &
!                                 tagMPI, RequestArrMPI)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   di1:          ('delta_index_1') integer, relative index of leftmost important neighbor cell
!                 in x1 direction (-1, 0, or 1)
!   di1max:       ('delta_index_1_maximum') integer, maximum value of di1:
!                   0: Dir3
!                   1: DirSimple3
!   bound_l1:     ('boundary_left_1') integer, left boundary type for dimension 1;
!                 values: see rhd_action_module
!   bound_r1:     ('boundary_right_1') integer, right boundary type for dimension 1;
!                 values: see rhd_action_module
!   i3old:        ('index_3_old') integer, temporary variable of x3 index of old cell
!   tagMPI:       ('tag_MPI') integer with MPI tag; 1 or 2 will be added
!
! INPUT/OUTPUT:
!   I:            ('Intensity') 3D real array with intensity values at cell centers
!   I1l:          ('Intensity_1_left') 1D real array with left boundary intensities
!   I1r:          ('Intensity_1_right') 1D real array with right boundary intensities
!   RequestArrMPI: ('Request_Array_MPI') 1D integer array with MPI request handles
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   StatusArrMPI: ('Status_Array_MPI') 2D integer array with MPI status
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_WaitAll:           Wait for all given communications to complete
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Communicate with neighbor MPI processes.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Check if there is a relevant internal boundary.
!   If yes, transfer boundary intensity with MPI_ISend, MPI_IRecv, MPI_WaitAll.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-10-01 (B.F. Uppsala) Extracted from rhd_shortrad_Dir3
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use rhd_action_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           di1, di1max, bound_l1, bound_r1, i3old, tagMPI
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(inout):: I
real, dimension(m2-1:n2+1),intent(inout):: I1l, I1r
integer, dimension(4),     intent(inout):: RequestArrMPI
!
! --- Local variables ---
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!---------------------------------------------------------------------------------------------------
!
! --- Issue non-blocking send requests for boundary values to neighbor processes ---
!
! --- Initialize request arrays: nothing has happened, yet ---
RequestArrMPI(1)=MPI_REQUEST_NULL
RequestArrMPI(2)=MPI_REQUEST_NULL
!
if (di1 == -1) then
  if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I1r=I(n1, :, i3old)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
    !   ' in Dir=', action%dimV(1), &
    !   ' tag=', tagMPI+1
    call MPI_ISend(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(1), ierrMPI)
  endif ! (bound_r1)
  !
  if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(2), ierrMPI)
  endif ! (bound_l1)
  !
else if (di1 == di1max) then
  if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I1l=I(m1, :, i3old)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
    !   ' in Dir=', action%dimV(1), &
    !   ' tag=', tagMPI+2
    call MPI_ISend(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(1), ierrMPI)
  endif ! (bound_l1)
  !
  if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(2), ierrMPI)
  endif ! (bound_r1)
  !
endif ! (di1)
!
! --- Wait for non-blocking send/receive requests for boundary values ---
! --- from neighbor processes to finish                               ---
call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
!
end subroutine rhd_shortrad_MPIDir3Int1


!----------************************-----------------------------------------------------------------
subroutine rhd_shortrad_MPIDir3Int2(geo, action, I, m1,n1, m2,n2, m3,n3, &
                                    I2l, I2r, di2, di2max, bound_l2, bound_r2, i3old, &
                                    tagMPI, RequestArrMPI)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_MPIDir3Int2 ('rhd_short-characteristics_radiation_MPI_Direction_3_Intensity_2')
!
! PURPOSE:
!   Transfer x2 boundary intensity between MPI processes for integration in x3 direction.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_MPIDir3Int2(geo, action, I, m1,n1, m2,n2, m3,n3, &
!                                 I2l, I2r, di2, di2max, bound_l2, bound_r2, i3old, &
!                                 tagMPI, RequestArrMPI)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   di2:          ('delta_index_2') integer, relative index of leftmost important neighbor cell
!                 in x2 direction (-1, 0, or 1)
!   di2max:       ('delta_index_2_maximum') integer, maximum value of di2:
!                   0: Dir3
!                   1: DirSimple3
!   bound_l2:     ('boundary_left_2') integer, left boundary type for dimension 2;
!                 values: see rhd_action_module
!   bound_r2:     ('boundary_right_2') integer, right boundary type for dimension 2;
!                 values: see rhd_action_module
!   i3old:        ('index_3_old') integer, temporary variable of x3 index of old cell
!   tagMPI:       ('tag_MPI') integer with MPI tag; 1 or 2 will be added
!
! INPUT/OUTPUT:
!   I:            ('Intensity') 3D real array with intensity values at cell centers
!   I2l:          ('Intensity_2_left') 1D real array with left boundary intensities
!   I2r:          ('Intensity_2_right') 1D real array with right boundary intensities
!   RequestArrMPI: ('Request_Array_MPI') 1D integer array with MPI request handles
!
! LOCAL VARIABLES:
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   StatusArrMPI: ('Status_Array_MPI') 2D integer array with MPI status
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_WaitAll:           Wait for all given communications to complete
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Communicate with neighbor MPI processes.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Check if there is a relevant internal boundary.
!   If yes, transfer boundary intensity with MPI_ISend, MPI_IRecv, MPI_WaitAll.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-10-01 (B.F. Uppsala) Extracted from rhd_shortrad_Dir3
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use rhd_action_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           di2, di2max, bound_l2, bound_r2, i3old, tagMPI
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(inout):: I
real, dimension(m1-1:n1+1),intent(inout):: I2l, I2r
integer, dimension(4),     intent(inout):: RequestArrMPI
!
! --- Local variables ---
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!---------------------------------------------------------------------------------------------------
!
! --- Issue non-blocking send requests for boundary values to neighbor processes ---
!
! --- Initialize request arrays: nothing has happened, yet ---
RequestArrMPI(3)=MPI_REQUEST_NULL
RequestArrMPI(4)=MPI_REQUEST_NULL
!
if (di2 == -1) then
  if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I2r=I(:, n2, i3old)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I2r     to=', geo%Proc_r(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', tagMPI+1
    call MPI_ISend(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(3), ierrMPI)
  endif ! (bound_r2)
  !
  if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                   tagMPI+1, geo%Comm, RequestArrMPI(4), ierrMPI)
  endif ! (bound_l2)
  !
else if (di2 == di2max) then
  if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    I2l=I(:, m2, i3old)
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I2l     to=', geo%Proc_l(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', tagMPI+2
    call MPI_ISend(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(3), ierrMPI)
  endif ! (bound_l2)
  !
  if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: receive boundary values from neighbor process ---
    call MPI_IRecv(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                   tagMPI+2, geo%Comm, RequestArrMPI(4), ierrMPI)
  endif ! (bound_r2)
  !
endif ! (di2)
!
! --- Wait for non-blocking send/receive requests for boundary values ---
! --- from neighbor processes to finish                               ---
call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
!
end subroutine rhd_shortrad_MPIDir3Int2


!#
# endif
!#      /* rhd_mpi_l01 ========================================================================== */


!#
# if (rhd_shortrad_operator_l02 == 0)
!#


!----------*********************--------------------------------------------------------------------
subroutine rhd_shortrad_operator(dtauthick_limit, &
                                 Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
                                             thickmaskold, &
                                       Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
                                             thickmasknew, &
                                 deiold, deinew, Inew)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_operator ('rhd_short-characteristics_radiation_operator')
!
! PURPOSE:
!   Perform the proper short-characteristics step for pair of grid cells.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   call rhd_shortrad_operator(dtauthick_limit, &
!                              Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
!                                            thickmaskold, &
!                                    Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
!                                            thickmasknew, &
!                              deiold, deinew, Inew)
!
! INPUT:
!   dtauthick_limit: ('delta_tau_thick_limit') real, border between optically thin and thick
!   Iold:         ('Intensity_old') real, reduced intensity at old level
!   Sold:         ('Source_old') real, source function at old level
!   dtauhalfold:  ('delta_tau_half_old') half optical depth of old cell
!   exp_mdtauhalfold: ('exponential_minus_delta_tau_half_old') real, exp(-dtauhalfold)
!   expl2t_mdtauhalfold: ('exponential_less_2_terms_minus_delta_tau_half_old') real,
!                        exp(-dtauhalfold)-1+dtauhalfold
!   thickmaskold: ('thick_mask_old') real with mask to distinguish optically thick/thin:
!                 1.0 for optically thick, 0.0 for optically thin region
!   Snew:         ('Source_new') real, source function at new level
!   dtauhalfnew:  ('delta_tau_half_new') half optical depth of new cell
!   exp_mdtauhalfnew: ('exponential_minus_delta_tau_half_new') real, exp(-dtauhalfnew)
!   expl2t_mdtauhalfnew: ('exponential_less_2_terms_minus_delta_tau_half_new') real,
!                        exp(-dtauhalfnew)-1+dtauhalfnew
!   thickmasknew: ('thick_mask_new') real with mask to distinguish optically thick/thin:
!                 1.0 for optically thick, 0.0 for optically thin region
!
! OUTPUT:
!   deiold:       ('delta_energy_old') real, energy change in old interval
!   deinew:       ('delta_energy_new') real, energy change in new interval
!   Inew:         ('Intensity_new') real, reduced intensity at new level
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The routine operates on scalars. It should be inlined to be efficient!
!
! PROCEDURE:
!   The routine solves the equation of radiative transfer
!     dI_org/dtau = - I_org + S
!   for one cell. It uses the reduced intensity
!
!          / = I_org    ,  dtau <  dtauthick_limit
!     I = <
!          \ = I_org - S,  dtau >=  dtauthick_limit
!
!
!           dtauhalfold    dtauhalfnew
!              ->deiold  ->deinew
!               ___-___   ___-___
!              /       \ /       \
!    |--------+---------|---------+---------|-->
!            Iold              ->Inew
!            Sold    Scenter     Snew
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F. Uppsala) First version
!   2001-11-23 (B.F.) Test operator
!   2001-11-30 (B.F.) Reasonable wS for large optical depths
!   2002-03-15 (B.F.) Do S slope reduction in case of cool optically thick cell in thin environment
!   2002-05-10 (B.F.) Tried performance improvements
!   2002-05-24 (B.F.) Modified S slope reduction
!   2002-11-27 (B.F.) Compiler switches within routine
!   2002-11-28 (B.F.) Individual methods accessed via include
!   2002-12-02 (B.F.) thickmask as input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtauthick_limit, &
                                           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, thickmaskold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew, thickmasknew
real,                      intent(out)  :: deiold, deinew, Inew
!
! --- Local variables ---
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
include 'rhd_shortrad_operator00.f90'
# elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
include 'rhd_shortrad_operator01.f90'
# elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
include 'rhd_shortrad_operator02.f90'
# elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
include 'rhd_shortrad_operator03.f90'
# elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
include 'rhd_shortrad_operator04.f90'
# elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
include 'rhd_shortrad_operator05.f90'
# elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
include 'rhd_shortrad_operator06.f90'
# elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
include 'rhd_shortrad_operator08.f90'
# endif /* rhd_shortrad_operator_l01 */
!#
!
end subroutine rhd_shortrad_operator


!#
# endif
!#      /* rhd_shortrad_operator_l02 */
!#
# if (rhd_shortrad_dtauop_l02 == 0)
!#


!----------*******************----------------------------------------------------------------------
subroutine rhd_shortrad_dtauop(dtauthick_limit, &
                               expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
                               expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, thickmasknew, &
                               taucouple)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_dtauop ('rhd_short-characteristics_radiation_delta_tau_operator')
!
! PURPOSE:
!   Compute contribution to radiative relaxation time scale from optical thickness.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   call rhd_shortrad_dtauop(dtauthick_limit, &
!                            expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
!                            expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, thickmasknew, &
!                            taucouple)
!
! INPUT:
!   dtauthick_limit: ('delta_tau_thick_limit') real, border between optically thin and thick
!   dtauhalfold:  ('delta_tau_half_old') half optical depth of old cell
!   exp_mdtauhalfold: ('exponential_minus_delta_tau_half_old') real, exp(-dtauhalfold)
!   expl2t_mdtauhalfold: ('exponential_less_2_terms_minus_delta_tau_half_old') real,
!                       exp(-dtauhalfold)-1+dtauhalfold
!   dtauhalfnew:  ('delta_tau_half_new') half optical depth of new cell
!   exp_mdtauhalfnew: ('exponential_minus_delta_tau_half_new') real, exp(-dtauhalfnew)
!   expl2t_mdtauhalfnew: ('exponential_less_2_terms_minus_delta_tau_half_new') real,
!                        exp(-dtauhalfnew)-1+dtauhalfnew
!   thickmasknew: ('thick_mask_new') real with mask to distinguish optically thick/thin:
!                 1.0 for optically thick, 0.0 for optically thin region
!
! OUTPUT:
!   taucouple:    ('tau_couple') contribution to radiative relaxation time scale from tau coupling
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The routines operates on scalars. It must be inlined to be efficient!
!
! PROCEDURE:
!   Limit cases:
!     optically thin:  taucouple  -->  (dtauhalfnew*(dtauhalfnew+2*dtauhalfold)) /
!                                           (dtauhalfnew + dtauhalfold)
!     optically thick: taucouple  -->  2/(dtauhalfnew + dtauhalfold)
!   This routine computes the "optically coupling" between a cell with optical thickness
!   dtaunew and a neighbor cell with  optical thickness dtauold.
!   This coefficient can be used to compute the time scale for the radiative relaxation of local
!   temperature fluctuations.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-04-10 (B.F. Uppsala) First version
!   2002-05-04 (B.F.) Switch to simple form in optically very thin
!   2002-11-27 (B.F.) All cases merged into one routine
!   2002-12-02 (B.F.) thickmasknew as input parameter
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtauthick_limit, &
                                           expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                             dtauhalfold, &
                                           expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                             dtauhalfnew, thickmasknew
real,                      intent(out)  :: taucouple
!
! --- Local variables ---
logical                                 :: thickflagnew
real                                    :: wthick
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_shortrad_dtauop_l01 == 1)
include 'rhd_shortrad_dtauop01.f90'
# else /* rhd_shortrad_dtauop_l01 */
include 'rhd_shortrad_dtauop02.f90'
# endif /* rhd_shortrad_dtauop_l01 */
!#
end subroutine rhd_shortrad_dtauop


!#
# endif
!#      /* rhd_shortrad_dtauop_l02 */


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_DirSimple1(geo, ComputeIntDown_flag, ComputeIntUp_flag, &
                                   ComputeFlux_flag, ComputePressure_flag, &
                                   r_dir, r_wei, n_dir, &
                                   m1,n1, m2,n2, m3,n3, &
                                   xb1, xb2, xb3, &
                                   S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                   Frad1, Frad2, Frad3, drhoeidtime &
!#
#                                  if (rhd_shortrad_radpressure_l01 != 0)
!#
                                   , drhov1dtime, drhov2dtime, drhov3dtime &
!#
#                                  endif
!#
                                   )
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_DirSimple1 ('rhd_short-characteristics_radiation_Direction_Simple_1')
!
! PURPOSE:
!   Compute radiative energy flux for one simple direction field with main direction 1.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_DirSimple1(geo, ComputeIntDown_flag, ComputeIntUp_flag, &
!                                ComputeFlux_flag, ComputePressure_flag, &
!                                r_dir, r_wei, n_dir, &
!                                m1,n1, m2,n2, m3,n3, &
!                                xb1, xb2, xb3, &
!                                S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                                Frad1, Frad2, Frad3, drhoeidtime &
!                                , drhov1dtime, drhov2dtime, drhov3dtime &
!                                )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   see rhd_shortrad_Dir1
!
! ROUTINES:
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_operator is called frequently and should be inlined.
!
! PROCEDURE:
!   Solve the radiative transfer equation along one direction (both orientations):
!   Call rhd_shortrad_operator for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-08-01 (B.F.) simple2 -> simple1
!   2004-05-26 (B.F.) Bugfix: declaration of deiold00, deinew00, Fradt: m1,n1 -> m2,n2
!   2008-08-20 (B.F. Lyon) Bugfix: re-insert energy update in downward integration
!   2013-01-22 (B.F.) Compute thickmask locally
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2014-02-20 (B.F.) Handle boundaries within integration loops
!   2014-02-28 (B.F.) Handle all boundaries within integration loops
!   2017-07-03 (B.F.) Add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-21 (B.F.) Bugfix: set i1 for boundary conditions
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use const_module, only: fourpi, c_light
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+2),intent(in)   :: xb1
real, dimension(m2-1:n2+2),intent(in)   :: xb2
real, dimension(m3-1:n3+2),intent(in)   :: xb3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1 ), &
                           intent(inout):: Frad3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
!
! --- Local variables ---
real                                    :: deiold, deinew, Inew, &
                                           mu, wFrad, wdivFrad, wdrhov1, wdrhov2, wdrhov3, &
                                           dx1, dx2, dx3
integer                                    di1, di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3
real, dimension(m2-1:n2+1, &
                m3-1:n3+1 )             :: deiold00, deinew00, Fradt
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 )             :: dei000
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 )             :: I
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: I, dei000
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: I, dei000
!#
# endif
!#      /* (rhd_arrays_l01) */
real, parameter                         :: dtauthick_limit=0.17
real, parameter                         :: w12=0.5, w13=1.0/3.0, w16=1.0/6.0
real                                    :: Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew, &
                                           thickmaskold, thickmasknew
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(I(     m1-1:n1+1, m2-1:n2+1, m3-1:n3+1), &
         dei000(m1-1:n1+1, m2-1:n2+1, m3-1:n3+1))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
dx1=(geo%xb_r(1)-geo%xb_l(1))/real(geo%n(1)-geo%m(1)+1)
dx2=(geo%xb_r(2)-geo%xb_l(2))/real(geo%n(2)-geo%m(2)+1)
dx3=(geo%xb_r(3)-geo%xb_l(3))/real(geo%n(3)-geo%m(3)+1)
!
! --- Find indices of relevant cells in old plane ---
di1=1
di2=nint(r_dir(2)/r_dir(1))
di3=nint(r_dir(3)/r_dir(1))
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
mu      =abs(r_dir(1))
wFrad   =fourpi * 0.5 * mu / real(max(n_dir,3))
wdivFrad=wFrad / dx1
wdrhov1 =(wdivFrad / c_light) * r_dir(1)
wdrhov2 =(wdivFrad / c_light) * r_dir(2)
wdrhov3 =(wdivFrad / c_light) * r_dir(3)
!
if (ComputeIntDown_flag) then
  ! === Integration i1: downward ===================================================================
  !
  if ((di2 == 0) .and. (di3 == 0)) then
    ! --- Special case for simple integration along x1 axis ---
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew)
    !
    i1=n1
    i1old=i1+1
    !
    ! === Set boundary intensity ===
    !
    if (i1 == n1) then
      ! --- Set intensity at n1 boundary ---
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3, n3
        do i2=m2, n2
          if (dtauhalf(i1old, i2, i3) > dtauthick_limit) then
            I(         i1old, i2, i3)=-S(i1old, i2, i3)
          else
            I(         i1old, i2, i3)=0.0
          endif
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
    !
    ! === Perform radiation-transport step ===
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3, n3
      do i2=m2, n2
        do i1=n1, m1-1, -1
          i1old=i1+1
          i2old=i2
          i3old=i3
!#
#         if (rhd_shortrad_operator_l02 == 0)
!#
          call rhd_shortrad_operator(dtauthick_limit, &
                 I(                     i1old,i2old,i3old), &
                 S(                     i1old,i2old,i3old), &
                 expl2t_mdtauhalf(      i1old,i2old,i3old), &
                 exp_mdtauhalf(         i1old,i2old,i3old), &
                 dtauhalf(              i1old,i2old,i3old), &
                 0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
                 S(                     i1   ,i2   ,i3   ), &
                 expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
                 exp_mdtauhalf(         i1   ,i2   ,i3   ), &
                 dtauhalf(              i1   ,i2   ,i3   ), &
                 0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                 deiold, deinew, Inew)
!#
#         else /* rhd_shortrad_operator_l02 */
!#
          Iold               =I(               i1old,i2old,i3old)
          Sold               =S(               i1old,i2old,i3old)
          expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
          exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
          dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
          thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
          Snew               =S(               i1   ,i2   ,i3   )
          expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
          exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
          dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
          thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
          !
!#
#         if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
          include 'rhd_shortrad_operator00.f90'
#         elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
          include 'rhd_shortrad_operator01.f90'
#         elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
          include 'rhd_shortrad_operator02.f90'
#         elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator03.f90'
#         elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
          include 'rhd_shortrad_operator04.f90'
#         elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator05.f90'
#         elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
          include 'rhd_shortrad_operator06.f90'
#         elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
          include 'rhd_shortrad_operator08.f90'
#         endif /* rhd_shortrad_operator_l01 */
!#
#         endif /* rhd_shortrad_operator_l02 */
!#
          dei000(i1,i2old,i3old)=deiold
          I(     i1,i2   ,i3   )=Inew
          !
          drhoeidtime( i1old,i2old,i3old)= &
           drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
          drhoeidtime( i1   ,i2   ,i3   )= &
           drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#         if (rhd_shortrad_radpressure_l01 != 0)
!#
          drhov1dtime( i1old,i2old,i3old)= &
           drhov1dtime(i1old,i2old,i3old) + wdrhov1 * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
          drhov1dtime( i1   ,i2   ,i3   )= &
           drhov1dtime(i1   ,i2   ,i3   ) + wdrhov1 * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
!#
#         endif
!#
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    if (ComputeFlux_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i2=m2,n2
          do i1=n1, m1-1, -1
            Frad1( i1+1,i2,i3)= &
             Frad1(i1+1,i2,i3) - wFrad * (I(i1+1,i2,i3) + dei000(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
    !
    !$OMP END PARALLEL
  else ! (di2, di3)
    ! --- Inclined ray ---
!#
#   if (rhd_shortrad_dir_l02 == 1)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 1 */
  !
    do i1=n1, m1-1, -1
!#
#     if (rhd_shortrad_dir_l02 == 0)
!#
      !$OMP PARALLEL DEFAULT(SHARED) &
      !$OMP   PRIVATE(i2, i1old, i2old, i3old, deiold, deinew, Inew, &
      !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
      !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
      !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
      !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew)
!#
#     endif
!#          /* rhd_shortrad_dir_l02 == 0 */
      !
      i1old=i1+1
      !
      ! === Set boundary intensity ===
      !
      if (i1 == n1) then
        ! --- Set intensity at n1 boundary ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3, n3
          do i2=m2, n2
            if (dtauhalf(i1old, i2, i3) > dtauthick_limit) then
              I(         i1old, i2, i3)=-S(i1old, i2, i3)
            else
              I(         i1old, i2, i3)=0.0
            endif
          end do ! i2
        end do ! i3
        !$OMP END DO
      endif
      !
      ! === Perform radiation-transport step ===
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3-max(di3,0), n3-min(di3,0)
        if (i3 == (m3-max(di3,0))) then
          ! --- Set intensity at m3 boundary ---
          where (dtauhalf(i1old, :   , m3-1) > dtauthick_limit)
                        I(i1old, :   , m3-1)=-S(i1old, :   , m3-1)
          elsewhere
                        I(i1old, :   , m3-1)=0.0
          endwhere
        endif
        if (i3 == (n3-min(di3,0))) then
          ! --- Set intensity at n3 boundary ---
          where (dtauhalf(i1old, :   , n3+1) > dtauthick_limit)
                        I(i1old, :   , n3+1)=-S(i1old, :   , n3+1)
          elsewhere
                        I(i1old, :   , n3+1)=0.0
          endwhere
        endif
        ! --- Set intensity at m2 boundary ---
        if (dtauhalf(i1old, m2-1, i3  ) > dtauthick_limit) then
                   I(i1old, m2-1, i3  )=-S(i1old, m2-1, i3  )
        else
                   I(i1old, m2-1, i3  )=0.0
        endif
        ! --- Set intensity at n2 boundary ---
        if (dtauhalf(i1old, n2+1, i3  ) > dtauthick_limit) then
                   I(i1old, n2+1, i3  )=-S(i1old, n2+1, i3  )
        else
                   I(i1old, n2+1, i3  )=0.0
        endif
      end do ! i3
      !$OMP END DO
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3-max(di3,0), n3-min(di3,0)
        do i2=m2-max(di2,0), n2-min(di2,0)
          i2old=i2+di2
          i3old=i3+di3
!#
#         if (rhd_shortrad_operator_l02 == 0)
!#
          call rhd_shortrad_operator(dtauthick_limit, &
                 I(                     i1old,i2old,i3old), &
                 S(                     i1old,i2old,i3old), &
                 expl2t_mdtauhalf(      i1old,i2old,i3old), &
                 exp_mdtauhalf(         i1old,i2old,i3old), &
                 dtauhalf(              i1old,i2old,i3old), &
                 0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
                 S(                     i1   ,i2   ,i3   ), &
                 expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
                 exp_mdtauhalf(         i1   ,i2   ,i3   ), &
                 dtauhalf(              i1   ,i2   ,i3   ), &
                 0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                 deiold, deinew, Inew)
!#
#         else /* rhd_shortrad_operator_l02 */
!#
          Iold               =I(               i1old,i2old,i3old)
          Sold               =S(               i1old,i2old,i3old)
          expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
          exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
          dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
          thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
          Snew               =S(               i1   ,i2   ,i3   )
          expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
          exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
          dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
          thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
          !
!#
#         if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
          include 'rhd_shortrad_operator00.f90'
#         elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
          include 'rhd_shortrad_operator01.f90'
#         elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
          include 'rhd_shortrad_operator02.f90'
#         elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator03.f90'
#         elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
          include 'rhd_shortrad_operator04.f90'
#         elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator05.f90'
#         elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
          include 'rhd_shortrad_operator06.f90'
#         elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
          include 'rhd_shortrad_operator08.f90'
#         endif /* rhd_shortrad_operator_l01 */
!#
#         endif /* rhd_shortrad_operator_l02 */
!#
          deiold00(   i2old,i3old)=deiold
          I(       i1,i2   ,i3   )=Inew
          !
          drhoeidtime( i1old,i2old,i3old)= &
           drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
          drhoeidtime( i1   ,i2   ,i3   )= &
           drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#         if (rhd_shortrad_radpressure_l01 != 0)
!#
          drhov1dtime( i1old,i2old,i3old)= &
           drhov1dtime(i1old,i2old,i3old) + wdrhov1  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
          drhov1dtime( i1   ,i2   ,i3   )= &
           drhov1dtime(i1   ,i2   ,i3   ) + wdrhov1  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
          drhov2dtime( i1old,i2old,i3old)= &
           drhov2dtime(i1old,i2old,i3old) + wdrhov2  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
          drhov2dtime( i1   ,i2   ,i3   )= &
           drhov2dtime(i1   ,i2   ,i3   ) + wdrhov2  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
          drhov3dtime( i1old,i2old,i3old)= &
           drhov3dtime(i1old,i2old,i3old) + wdrhov3  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
          drhov3dtime( i1   ,i2   ,i3   )= &
           drhov3dtime(i1   ,i2   ,i3   ) + wdrhov3  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
!#
#         endif
!#
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      if (ComputeFlux_flag) then
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3-max(di3,0), n3-min(di3,0)
          do i2=m2-max(di2,0), n2-min(di2,0)
            Fradt(i2+di2,i3+di3)=wFrad * (I(i1+1,i2+di2,i3+di3) + deiold00(i2+di2,i3+di3))
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        if (di2 == -1) then
          if (di3 == -1) then
            ! --- -1-1 -----------------------------------------------------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - (w13*(Fradt(i2-1,i3-1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2-1,i3  ) + &
                                               Fradt(i2  ,i3-1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1+1,i2  ,i3  )= &
                 Frad2(i1+1,i2  ,i3  ) + (w16* Fradt(i2-1,i3-1) + &
                                          w13* Fradt(i2-1,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i2-1,i3-1) + &
                                          w16* Fradt(i2-1,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1+1,i2  ,i3  )= &
                 Frad3(i1+1,i2  ,i3  ) + (w16* Fradt(i2-1,i3-1) + &
                                          w13* Fradt(i2  ,i3-1)  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i2-1,i3-1) + &
                                          w16* Fradt(i2  ,i3-1)  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else if (di3 == 0) then
            ! --- -10 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - w12*(Fradt(i2-1,i3  ) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1+1,i2  ,i3  )= &
                 Frad2(i1+1,i2  ,i3  ) + w12* Fradt(i2-1,i3  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) + w12* Fradt(i2-1,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else
            ! --- -11 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - (w13*(Fradt(i2-1,i3+1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2-1,i3  ) + &
                                               Fradt(i2  ,i3+1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1+1,i2  ,i3  )= &
                 Frad2(i1+1,i2  ,i3  ) + (w16* Fradt(i2-1,i3+1) + &
                                          w13* Fradt(i2-1,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i2-1,i3+1) + &
                                          w16* Fradt(i2-1,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1+1,i2  ,i3  )= &
                 Frad3(i1+1,i2  ,i3  ) - (w16* Fradt(i2-1,i3  ) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i2-1,i3  ) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          endif
        else if (di2 == 0) then
          if (di3 == -1) then
            ! --- 0-1 ------------------------------------------------------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - w12*(Fradt(i2  ,i3-1) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1+1,i2  ,i3  )= &
                 Frad3(i1+1,i2  ,i3  ) + w12* Fradt(i2  ,i3-1)
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) + w12* Fradt(i2  ,i3-1)
              end do ! i2
            end do ! i3
            !$OMP END DO
          else if (di3 == 0) then
            ! --- 00 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) -      Fradt(i2  ,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else
            ! --- 01 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - w12*(Fradt(i2  ,i3+1) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1+1,i2  ,i3  )= &
                 Frad3(i1+1,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          endif
        else
          if (di3 == -1) then
            ! --- 1-1 ------------------------------------------------------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - (w13*(Fradt(i2+1,i3-1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2+1,i3  ) + &
                                               Fradt(i2  ,i3-1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1+1,i2  ,i3  )= &
                 Frad2(i1+1,i2  ,i3  ) - (w16* Fradt(i2  ,i3-1) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i2  ,i3-1) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1+1,i2  ,i3  )= &
                 Frad3(i1+1,i2  ,i3  ) + (w16* Fradt(i2+1,i3-1) + &
                                          w13* Fradt(i2  ,i3-1)  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i2+1,i3-1) + &
                                          w16* Fradt(i2  ,i3-1)  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else if (di3 == 0) then
            ! --- 10 -------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - w12*(Fradt(i2+1,i3  ) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1+1,i2  ,i3  )= &
                 Frad2(i1+1,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else
            ! --- 11 -------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1+1,i2  ,i3  )= &
                 Frad1(i1+1,i2  ,i3  ) - (w13*(Fradt(i2+1,i3+1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2+1,i3  ) + &
                                               Fradt(i2  ,i3+1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1+1,i2  ,i3  )= &
                 Frad2(i1+1,i2  ,i3  ) - (w16* Fradt(i2  ,i3+1) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i2  ,i3+1) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1+1,i2  ,i3  )= &
                 Frad3(i1+1,i2  ,i3  ) - (w16* Fradt(i2+1,i3  ) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i2+1,i3  ) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          endif
        endif
        !
      endif ! (ComputeFlux_flag)
      !
!#
#     if (rhd_shortrad_dir_l02 == 0)
!#
      !$OMP END PARALLEL
!#
#     endif
!#          /* rhd_shortrad_dir_l02 == 0 */
    end do ! i1
!#
#   if (rhd_shortrad_dir_l02 == 1)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 1 */
  endif ! (di2, di3)
  !
endif ! (ComputeIntDown_flag)
!
! --- Adjust indices ---
di1=-di1
di2=-di2
di3=-di3
!
if (ComputeIntUp_flag) then
  ! === Integration i1: upward =====================================================================
  !
  if ((di2 == 0) .and. (di3 == 0)) then
    ! --- Special case for simple integration along x1 axis ---
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew)
    !
    i1old=i1-1
    !
    ! === Set boundary intensity ===
    !
    if (i1 == m1) then
      ! --- Set intensity at m1 boundary ---
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3, n3
        do i2=m2, n2
          if (dtauhalf(i1old, i2, i3) > dtauthick_limit) then
            I(         i1old, i2, i3)=-S(i1old, i2, i3)
          else
            I(         i1old, i2, i3)=0.0
          endif
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
    !
    ! === Perform radiation-transport step ===
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3, n3
      do i2=m2, n2
        do i1=m1, n1+1
          i1old=i1-1
          i2old=i2
          i3old=i3
!#
#         if (rhd_shortrad_operator_l02 == 0)
!#
          call rhd_shortrad_operator(dtauthick_limit, &
                 I(                     i1old,i2old,i3old), &
                 S(                     i1old,i2old,i3old), &
                 expl2t_mdtauhalf(      i1old,i2old,i3old), &
                 exp_mdtauhalf(         i1old,i2old,i3old), &
                 dtauhalf(              i1old,i2old,i3old), &
                 0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
                 S(                     i1   ,i2   ,i3   ), &
                 expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
                 exp_mdtauhalf(         i1   ,i2   ,i3   ), &
                 dtauhalf(              i1   ,i2   ,i3   ), &
                 0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                 deiold, deinew, Inew)
!#
#         else /* rhd_shortrad_operator_l02 */
!#
          Iold               =I(               i1old,i2old,i3old)
          Sold               =S(               i1old,i2old,i3old)
          expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
          exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
          dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
          thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
          Snew               =S(               i1   ,i2   ,i3   )
          expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
          exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
          dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
          thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
          !
!#
#         if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
          include 'rhd_shortrad_operator00.f90'
#         elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
          include 'rhd_shortrad_operator01.f90'
#         elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
          include 'rhd_shortrad_operator02.f90'
#         elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator03.f90'
#         elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
          include 'rhd_shortrad_operator04.f90'
#         elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator05.f90'
#         elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
          include 'rhd_shortrad_operator06.f90'
#         elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
          include 'rhd_shortrad_operator08.f90'
#         endif /* rhd_shortrad_operator_l01 */
!#
#         endif /* rhd_shortrad_operator_l02 */
!#
          dei000(i1,i2,i3)=deinew
          I(     i1,i2,i3)=Inew
          !
          drhoeidtime( i1old,i2old,i3old)= &
           drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
          drhoeidtime( i1   ,i2   ,i3   )= &
           drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#         if (rhd_shortrad_radpressure_l01 != 0)
!#
          drhov1dtime( i1old,i2old,i3old)= &
           drhov1dtime(i1old,i2old,i3old) - wdrhov1  * deiold
          drhov1dtime( i1   ,i2   ,i3   )= &
           drhov1dtime(i1   ,i2   ,i3   ) - wdrhov1  * deinew
!#
#         endif
!#
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    if (ComputeFlux_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1, n1+1
            Frad1( i1,i2,i3)= &
             Frad1(i1,i2,i3) + wFrad * (I(i1,i2,i3) - dei000(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
    !
    !$OMP END PARALLEL
  else ! (di2, di3)
    ! --- Inclined ray ---
!#
#   if (rhd_shortrad_dir_l02 == 1)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 1 */
    !
    do i1=m1, n1+1
!#
#     if (rhd_shortrad_dir_l02 == 0)
!#
      !$OMP PARALLEL DEFAULT(SHARED) &
      !$OMP   PRIVATE(i2, i1old, i2old, i3old, deiold, deinew, Inew, &
      !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
      !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
      !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
      !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew)
!#
#     endif
!#          /* rhd_shortrad_dir_l02 == 0 */
      !
      i1old=i1-1
      !
      ! === Set boundary intensity ===
      !
      if (i1 == m1) then
        ! --- Set intensity at m1 boundary ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3, n3
          do i2=m2, n2
            if (dtauhalf(i1old, i2, i3) > dtauthick_limit) then
              I(         i1old, i2, i3)=-S(i1old, i2, i3)
            else
              I(         i1old, i2, i3)=0.0
            endif
          end do ! i2
        end do ! i3
        !$OMP END DO
      endif
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3-max(di3,0), n3-min(di3,0)
        if (i3 == (m3-max(di3,0))) then
          ! --- Set intensity at m3 boundary ---
          where (dtauhalf(i1old, :   , m3-1) > dtauthick_limit)
                        I(i1old, :   , m3-1)=-S(i1old, :   , m3-1)
          elsewhere
                        I(i1old, :   , m3-1)=0.0
          endwhere
        endif
        if (i3 == (n3-min(di3,0))) then
          ! --- Set intensity at n3 boundary ---
          where (dtauhalf(i1old, :   , n3+1) > dtauthick_limit)
                        I(i1old, :   , n3+1)=-S(i1old, :   , n3+1)
          elsewhere
                        I(i1old, :   , n3+1)=0.0
          endwhere
        endif
        ! --- Set intensity at m2 boundary ---
        if (dtauhalf(i1old, m2-1, i3  ) > dtauthick_limit) then
                   I(i1old, m2-1, i3  )=-S(i1old, m2-1, i3  )
        else
                   I(i1old, m2-1, i3  )=0.0
        endif
        ! --- Set intensity at n2 boundary ---
        if (dtauhalf(i1old, n2+1, i3  ) > dtauthick_limit) then
                   I(i1old, n2+1, i3  )=-S(i1old, n2+1, i3  )
        else
                   I(i1old, n2+1, i3  )=0.0
        endif
      end do ! i3
      !$OMP END DO
      !
      ! === Perform radiation-transport step ===
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3-max(di3,0), n3-min(di3,0)
        do i2=m2-max(di2,0), n2-min(di2,0)
          i2old=i2+di2
          i3old=i3+di3
!#
#         if (rhd_shortrad_operator_l02 == 0)
!#
          call rhd_shortrad_operator(dtauthick_limit, &
                 I(                     i1old,i2old,i3old), &
                 S(                     i1old,i2old,i3old), &
                 expl2t_mdtauhalf(      i1old,i2old,i3old), &
                 exp_mdtauhalf(         i1old,i2old,i3old), &
                 dtauhalf(              i1old,i2old,i3old), &
                 0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
                 S(                     i1   ,i2   ,i3   ), &
                 expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
                 exp_mdtauhalf(         i1   ,i2   ,i3   ), &
                 dtauhalf(              i1   ,i2   ,i3   ), &
                 0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                 deiold, deinew, Inew)
!#
#         else /* rhd_shortrad_operator_l02 */
!#
          Iold               =I(               i1old,i2old,i3old)
          Sold               =S(               i1old,i2old,i3old)
          expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
          exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
          dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
          thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
          Snew               =S(               i1   ,i2   ,i3   )
          expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
          exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
          dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
          thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
          !
!#
#         if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
          include 'rhd_shortrad_operator00.f90'
#         elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
          include 'rhd_shortrad_operator01.f90'
#         elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
          include 'rhd_shortrad_operator02.f90'
#         elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator03.f90'
#         elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
          include 'rhd_shortrad_operator04.f90'
#         elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
          include 'rhd_shortrad_operator05.f90'
#         elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
          include 'rhd_shortrad_operator06.f90'
#         elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
          include 'rhd_shortrad_operator08.f90'
#         endif /* rhd_shortrad_operator_l01 */
!#
#         endif /* rhd_shortrad_operator_l02 */
!#
          deinew00(   i2,i3)=deinew
          I(       i1,i2,i3)=Inew
          !
          drhoeidtime( i1old,i2old,i3old)= &
           drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
          drhoeidtime( i1   ,i2   ,i3   )= &
           drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#         if (rhd_shortrad_radpressure_l01 != 0)
!#
          drhov1dtime( i1old,i2old,i3old)= &
           drhov1dtime(i1old,i2old,i3old) - wdrhov1  * deiold
          drhov1dtime( i1   ,i2   ,i3   )= &
           drhov1dtime(i1   ,i2   ,i3   ) - wdrhov1  * deinew
          drhov2dtime( i1old,i2old,i3old)= &
           drhov2dtime(i1old,i2old,i3old) - wdrhov2  * deiold
          drhov2dtime( i1   ,i2   ,i3   )= &
           drhov2dtime(i1   ,i2   ,i3   ) - wdrhov2  * deinew
          drhov3dtime( i1old,i2old,i3old)= &
           drhov3dtime(i1old,i2old,i3old) - wdrhov3  * deiold
          drhov3dtime( i1   ,i2   ,i3   )= &
           drhov3dtime(i1   ,i2   ,i3   ) - wdrhov3  * deinew
!#
#         endif
!#
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      if (ComputeFlux_flag) then
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3-max(di3,0), n3-min(di3,0)
          do i2=m2-max(di2,0), n2-min(di2,0)
            Fradt(i2+di2,i3+di3)=wFrad * (I(i1,i2,i3) - deinew00(i2,i3))
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        if (di2 == -1) then
          if (di3 == -1) then
            ! --- -1-1 -----------------------------------------------------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + (w13*(Fradt(i2-1,i3-1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2-1,i3  ) + &
                                               Fradt(i2  ,i3-1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1-1,i2  ,i3  )= &
                 Frad2(i1-1,i2  ,i3  ) + (w16* Fradt(i2-1,i3-1) + &
                                          w13* Fradt(i2-1,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i2-1,i3-1) + &
                                          w16* Fradt(i2-1,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1-1,i2  ,i3  )= &
                 Frad3(i1-1,i2  ,i3  ) + (w16* Fradt(i2-1,i3-1) + &
                                          w13* Fradt(i2  ,i3-1)  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i2-1,i3-1) + &
                                          w16* Fradt(i2  ,i3-1)  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else if (di3 == 0) then
            ! --- -10 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + w12*(Fradt(i2-1,i3  ) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1-1,i2  ,i3  )= &
                 Frad2(i1-1,i2  ,i3  ) + w12* Fradt(i2-1,i3  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) + w12* Fradt(i2-1,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else
            ! --- -11 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + (w13*(Fradt(i2-1,i3+1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2-1,i3  ) + &
                                               Fradt(i2  ,i3+1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1-1,i2  ,i3  )= &
                 Frad2(i1-1,i2  ,i3  ) + (w16* Fradt(i2-1,i3+1) + &
                                          w13* Fradt(i2-1,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i2-1,i3+1) + &
                                          w16* Fradt(i2-1,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1-1,i2  ,i3  )= &
                 Frad3(i1-1,i2  ,i3  ) - (w16* Fradt(i2-1,i3  ) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i2-1,i3  ) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          endif
        else if (di2 == 0) then
          if (di3 == -1) then
            ! --- 0-1 ------------------------------------------------------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + w12*(Fradt(i2  ,i3-1) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1-1,i2  ,i3  )= &
                 Frad3(i1-1,i2  ,i3  ) + w12* Fradt(i2  ,i3-1)
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) + w12* Fradt(i2  ,i3-1)
              end do ! i2
            end do ! i3
            !$OMP END DO
          else if (di3 == 0) then
            ! --- 00 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) +      Fradt(i2  ,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else
            ! --- 01 ------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + w12*(Fradt(i2  ,i3+1) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1-1,i2  ,i3  )= &
                 Frad3(i1-1,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          endif
        else
          if (di3 == -1) then
            ! --- 1-1 ------------------------------------------------------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + (w13*(Fradt(i2+1,i3-1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2+1,i3  ) + &
                                               Fradt(i2  ,i3-1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1-1,i2  ,i3  )= &
                 Frad2(i1-1,i2  ,i3  ) - (w16* Fradt(i2  ,i3-1) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i2  ,i3-1) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1-1,i2  ,i3  )= &
                 Frad3(i1-1,i2  ,i3  ) + (w16* Fradt(i2+1,i3-1) + &
                                          w13* Fradt(i2  ,i3-1)  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i2+1,i3-1) + &
                                          w16* Fradt(i2  ,i3-1)  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else if (di3 == 0) then
            ! --- 10 -------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + w12*(Fradt(i2+1,i3  ) + &
                                              Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1-1,i2  ,i3  )= &
                 Frad2(i1-1,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) - w12* Fradt(i2  ,i3  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          else
            ! --- 11 -------------------------------
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2
                Frad1( i1  ,i2  ,i3  )= &
                 Frad1(i1  ,i2  ,i3  ) + (w13*(Fradt(i2+1,i3+1) + &
                                               Fradt(i2  ,i3  )  ) + &
                                          w16*(Fradt(i2+1,i3  ) + &
                                               Fradt(i2  ,i3+1)  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3
              do i2=m2,n2+1
                Frad2( i1-1,i2  ,i3  )= &
                 Frad2(i1-1,i2  ,i3  ) - (w16* Fradt(i2  ,i3+1) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad2( i1  ,i2  ,i3  )= &
                 Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i2  ,i3+1) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO NOWAIT
            !$OMP DO SCHEDULE(RUNTIME)
            do i3=m3,n3+1
              do i2=m2,n2
                Frad3( i1-1,i2  ,i3  )= &
                 Frad3(i1-1,i2  ,i3  ) - (w16* Fradt(i2+1,i3  ) + &
                                          w13* Fradt(i2  ,i3  )  )
                Frad3( i1  ,i2  ,i3  )= &
                 Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i2+1,i3  ) + &
                                          w16* Fradt(i2  ,i3  )  )
              end do ! i2
            end do ! i3
            !$OMP END DO
          endif
        endif
        !
      endif ! (ComputeFlux_flag)
      !
!#
#     if (rhd_shortrad_dir_l02 == 0)
!#
      !$OMP END PARALLEL
!#
#     endif
!#          /* rhd_shortrad_dir_l02 == 0 */
    end do ! i1
!#
#   if (rhd_shortrad_dir_l02 == 1)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 1 */
  endif ! (di2, di3)
  !
endif ! (ComputeIntUp_flag)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(I, dei000)
!#
# endif
!#
!
end subroutine rhd_shortrad_DirSimple1


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_DirSimple2(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                                   ComputeFlux_flag, ComputePressure_flag, &
                                   r_dir, r_wei, n_dir, &
                                   m1,n1, m2,n2, m3,n3, &
                                   xb1, xb2, xb3, &
                                   S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                   Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                                  if (rhd_shortrad_radpressure_l01 != 0)
!#
                                   drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                                  endif
!#
                                   RequestSendRecvMPI, I2Send, I2Recv &
                                   )
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_DirSimple2 ('rhd_short-characteristics_radiation_Direction_Simple_2')
!
! PURPOSE:
!   Compute radiative energy flux for one simple direction field with main direction 2.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_DirSimple2(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
!                                ComputeFlux_flag, ComputePressure_flag, &
!                                r_dir, r_wei, n_dir, &
!                                m1,n1, m2,n2, m3,n3, &
!                                xb1, xb2, xb3, &
!                                S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                                Frad1, Frad2, Frad3, drhoeidtime, &
!                                drhov1dtime, drhov2dtime, drhov3dtime, &
!                                RequestSendRecvMPI, I2Send, I2Recv &
!                                )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   RequestSendRecvMPI: ('Request_Send_Receive_MPI') integer vector with MPI requests to transfer
!                 boundary data
!   I2Send:       ('Intensity_2_Send') 2D real array with boundary intensity in MPI buffer
!   I2Recv:       ('Intensity_2_Receive') 2D real array with boundary intensity in MPI buffer
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   see rhd_shortrad_Dir1
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Wait:              Wait for given communication to complete
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_shortrad_MPIDir2Int1: Transfer x1 boundary intensity between processes for x2 integration
!   rhd_shortrad_MPIDir2Int3: Transfer x3 boundary intensity between processes for x2 integration
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Via MPI.
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_operator is called frequently and should be inlined.
!
! PROCEDURE:
!   Solve the radiative transfer equation along one direction (both orientations):
!   Call rhd_shortrad_operator for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-08-01 (B.F.) simple3 -> simple2
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2014-02-20 (B.F.) Handle boundaries within integration loops
!   2014-02-28 (B.F.) Handle all boundaries within integration loops
!   2014-03-15 (B.F.) New version of boundary-condition handling
!   2017-07-03 (B.F.) Add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-27 (B.F.) Add MPI commands
!   2017-10-03 (B.F.) Put lateral MPI boundary intensity exchange into separate routines
!   2017-10-04 (B.F.) Outcomment routine calls for MPI boundary intensity exchange
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use const_module, only: fourpi, c_light
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
!!$use uio_bulk_module
!!$use uio_filedef_module
use timing_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+2),intent(in)   :: xb1
real, dimension(m2-1:n2+2),intent(in)   :: xb2
real, dimension(m3-1:n3+2),intent(in)   :: xb3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1 ), &
                           intent(inout):: Frad3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
integer, dimension(2),     intent(inout):: RequestSendRecvMPI
real, dimension(m1-1:n1+1, m3-1:n3+1), &
                           intent(inout):: I2Send, I2Recv
!
! --- Local variables ---
real                                    :: deiold, deinew, Inew, &
                                           mu, wFrad, wdivFrad, wdrhov1, wdrhov2, wdrhov3, &
                                           dx1, dx2, dx3, &
                                           RabsOut, RcosthetaOut, Dout, tauOut, dxOut, RdOut
integer                                    di1, di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3, nc, &
                                           bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3, &
                                           I1_timing, I3_timing
logical                                 :: BoundHtau_flag
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
logical                                 :: HasBound1Internal_flag, HasBound3Internal_flag
integer                                 :: ierrMPI
integer, dimension(4)                   :: RequestArrMPI
real, dimension(            m3-1:n3+1)  :: I1l, I1r
real, dimension(m1-1:n1+1            )  :: I3l, I3r
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
real, dimension(3)                      :: Rout
real, dimension(m1-1:n1+1, &
                m3-1:n3+1 )             :: deiold00, deinew00, Fradt
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 )             :: I
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: I
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: I
!#
# endif
!#      /* (rhd_arrays_l01) */
real, parameter                         :: dtauthick_limit=0.17
real, parameter                         :: w12=0.5, w13=1.0/3.0, w16=1.0/6.0
real                                    :: Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew, &
                                           thickmaskold, thickmasknew
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(I(m1-1:n1+1, m2-1:n2+1, m3-1:n3+1))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
dx1=(geo%xb_r(1)-geo%xb_l(1))/real(geo%n(1)-geo%m(1)+1)
dx2=(geo%xb_r(2)-geo%xb_l(2))/real(geo%n(2)-geo%m(2)+1)
dx3=(geo%xb_r(3)-geo%xb_l(3))/real(geo%n(3)-geo%m(3)+1)
!
! --- Find indices of relevant cells in old plane ---
di1  =nint(r_dir(1)/r_dir(2))
di2  =1
di3  =nint(r_dir(3)/r_dir(2))
RdOut=1.0
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
mu      =abs(r_dir(2))
wFrad   =fourpi * 0.5 * mu / real(max(n_dir,3))
wdivFrad=wFrad / dx2
wdrhov1 =(wdivFrad / c_light) * r_dir(1)
wdrhov2 =(wdivFrad / c_light) * r_dir(2)
wdrhov3 =(wdivFrad / c_light) * r_dir(3)
!
! --- Check boundary settings ---
BoundHtau_flag=(par%C_radHtautop > 0.0)
! --- Copy boundary-condition parameters ---
bound_l1=action%bound_l1
bound_r1=action%bound_r1
bound_l2=action%bound_l2
bound_r2=action%bound_r2
bound_l3=action%bound_l3
bound_r3=action%bound_r3
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
HasBound1Internal_flag=(((bound_l1 == bound_InternalNext    ) .or. &
                         (bound_l1 == bound_InternalPeriodic) .or. &
                         (bound_r1 == bound_InternalNext    ) .or. &
                         (bound_r1 == bound_InternalPeriodic)       ) .and. &
                        ((di1 /= 0)                                 )        )
HasBound3Internal_flag=(((bound_l3 == bound_InternalNext    ) .or. &
                         (bound_l3 == bound_InternalPeriodic) .or. &
                         (bound_r3 == bound_InternalNext    ) .or. &
                         (bound_r3 == bound_InternalPeriodic)       ) .and. &
                        ((di3 /= 0)                                 )        )
! write(gl%nc_p, '(A,I3,A,L1,2X,L1)') &
!   '#', geo%MyProc, ' HasBound1,3Internal_flag= ', &
!   HasBound1Internal_flag, HasBound3Internal_flag
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ComputeIntDown_flag) then
  ! ### Integration i2: downward ###################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I2 down, di1=', di1, ' di3=', di3
  !
  call timing_GetHandle('SHC:   dir2: D I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir2: D I3 SeRc', I3_timing)
  !
  if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I2Recv from=', geo%Proc_r(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2122
    ! call timing_start('SHC:   dir2: s D I2 Recv')
    call MPI_IRecv(I2Recv, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_r(action%dimV(2)), &
                   2122, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir2: s D I2 Recv')
    call timing_start('SHC:   dir2: s D I2 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: s D I2 Recv Wait')
  endif ! (bound_r2)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir2: s D I2 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: s D I2 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i2=n2, m2-1, -1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i2old=i2+1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i2old == (n2+1)) then
      ! --- Set intensity at starting face for integration (n2 boundary) ---------------------------
      !
      if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I2Recv(i1old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_r2)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              dxOut=xb2(i2old+1)-xb2(i2old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_r2)
      !
    else ! (i2old)
      ! --- Set intensity at m3/n3 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound3Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I3_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int3(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I3l, I3r, di3, 1, bound_l3, bound_r3, i2old, &
!!$                                      2130, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di3 == -1) then
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3r=I(:, i2old, n3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3r     to=', geo%Proc_r(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2131
            call MPI_ISend(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2131, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r3)
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2131, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l3)
          !
        else if (di3 == 1) then
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3l=I(:, i2old, m3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3l     to=', geo%Proc_l(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2132
            call MPI_ISend(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2132, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l3)
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2132, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r3)
          !
        endif ! (di3)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I3_timing)
        !
      endif ! (HasBound3Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        if (i3old == (m3-1)) then
          ! --- Set intensity at m3 side boundary ---
          !
          if (bound_l3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, n3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l3)
        endif ! (i3old)
        !
        if (i3old == (n3+1)) then
          ! --- Set intensity at n3 side boundary ---
          !
          if (bound_r3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, m3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r3)
        endif ! (i3old)
      end do ! i3old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 1, bound_l1, bound_r1, i2old, &
!!$                                      2110, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2111
            call MPI_ISend(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2111, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2111, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 1) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2112
            call MPI_ISend(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2112, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2112, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i3old
      !$OMP END DO
      !
    endif ! (i2old)
    !
    ! === Perform radiation-transport step ========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-max(di3,0), n3-min(di3,0)
      do i1=m1-max(di1,0), n1-min(di1,0)
        i1old=i1+di1
        i3old=i3+di3
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i1old,   i3old)=deiold
        I(       i1   ,i2,i3   )=Inew
        !
        drhoeidtime( i1old,i2old,i3old)= &
         drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
        drhoeidtime( i1   ,i2   ,i3   )= &
         drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#       if (rhd_shortrad_radpressure_l01 != 0)
!#
        drhov1dtime( i1old,i2old,i3old)= &
         drhov1dtime(i1old,i2old,i3old) + wdrhov1  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
        drhov1dtime( i1   ,i2   ,i3   )= &
         drhov1dtime(i1   ,i2   ,i3   ) + wdrhov1  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
        drhov2dtime( i1old,i2old,i3old)= &
         drhov2dtime(i1old,i2old,i3old) + wdrhov2  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
        drhov2dtime( i1   ,i2   ,i3   )= &
         drhov2dtime(i1   ,i2   ,i3   ) + wdrhov2  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
        drhov3dtime( i1old,i2old,i3old)= &
         drhov3dtime(i1old,i2old,i3old) + wdrhov3  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
        drhov3dtime( i1   ,i2   ,i3   )= &
         drhov3dtime(i1   ,i2   ,i3   ) + wdrhov3  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
!#
#       endif
!#
      end do ! i1
    end do ! i3
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i2old == m2) then
      if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I2Send(i1old, i3old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
      endif ! (bound_l2)
    endif ! (i2old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    if (ComputeFlux_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3-max(di3,0), n3-min(di3,0)
        do i1=m1-max(di1,0), n1-min(di1,0)
          Fradt(i1+di1,i3+di3)=wFrad * (I(i1+di1,i2+1,i3+di3) + deiold00(i1+di1,i3+di3))
        end do ! i1
      end do ! i3
      !$OMP END DO
      !
      if (di1 == -1) then
        if (di3 == -1) then
          ! --- -1-1 -------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - (w13*(Fradt(i1-1,i3-1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1-1,i3  ) + &
                                             Fradt(i1  ,i3-1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  ) + (w16* Fradt(i1-1,i3-1) + &
                                        w13* Fradt(i1-1,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i3-1) + &
                                        w16* Fradt(i1-1,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  ) + (w16* Fradt(i1-1,i3-1) + &
                                        w13* Fradt(i1  ,i3-1)  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i3-1) + &
                                        w16* Fradt(i1  ,i3-1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else if (di3 == 0) then
          ! --- -10 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - w12*(Fradt(i1-1,i3  ) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  ) + w12* Fradt(i1-1,i3  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + w12* Fradt(i1-1,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- -11 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - (w13*(Fradt(i1-1,i3+1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1-1,i3  ) + &
                                             Fradt(i1  ,i3+1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  ) + (w16* Fradt(i1-1,i3+1) + &
                                        w13* Fradt(i1-1,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i3+1) + &
                                        w16* Fradt(i1-1,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  ) - (w16* Fradt(i1-1,i3  ) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i1-1,i3  ) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      else if (di1 == 0) then
        if (di3 == -1) then
          ! --- 0-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - w12*(Fradt(i1  ,i3-1) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  ) + w12* Fradt(i1  ,i3-1)
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + w12* Fradt(i1  ,i3-1)
            end do ! i1
          end do ! i3
          !$OMP END DO
        else if (di3 == 0) then
          ! --- 00 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) -      Fradt(i1  ,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- 01 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - w12*(Fradt(i1  ,i3+1) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  ) - w12* Fradt(i1  ,i3  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      else
        if (di3 == -1) then
          ! --- 1-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - (w13*(Fradt(i1+1,i3-1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1+1,i3  ) + &
                                             Fradt(i1  ,i3-1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  ) - (w16* Fradt(i1  ,i3-1) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i3-1) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  ) + (w16* Fradt(i1+1,i3-1) + &
                                        w13* Fradt(i1  ,i3-1)  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i1+1,i3-1) + &
                                        w16* Fradt(i1  ,i3-1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else if (di3 == 0) then
          ! --- 10 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - w12*(Fradt(i1+1,i3  ) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  ) - w12* Fradt(i1  ,i3  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- 11 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  ) - (w13*(Fradt(i1+1,i3+1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1+1,i3  ) + &
                                             Fradt(i1  ,i3+1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  ) - (w16* Fradt(i1  ,i3+1) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i3+1) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  ) - (w16* Fradt(i1+1,i3  ) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i1+1,i3  ) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i2
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I2Send to=', geo%Proc_l(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2122
    call MPI_ISend(I2Send, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_l(action%dimV(2)), &
                   2122, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_l2)
  !
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I2 down'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir2_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i2s down'   , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntDown_flag)
!
! --- Adjust indices ---
di1=-di1
di2=-di2
di3=-di3
RdOut=-1.0
!
if (ComputeIntUp_flag) then
  ! ### Integration i2: upward #####################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I2 up,   di1=', di1, ' di3=', di3
  !
  call timing_GetHandle('SHC:   dir2: U I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir2: U I3 SeRc', I3_timing)
  !
  if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I2Recv from=', geo%Proc_l(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2221
    ! call timing_start('SHC:   dir2: s U I2 Recv')
    call MPI_IRecv(I2Recv, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_l(action%dimV(2)), &
                   2221, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir2: s U I2 Recv')
    call timing_start('SHC:   dir2: s U I2 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: s U I2 Recv Wait')
  endif ! (bound_l2)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir2: s U I2 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: s U I2 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i2=m2, n2+1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i2old=i2-1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i2old == (m2-1)) then
      ! --- Set intensity at starting face for integration (m2 boundary) ---------------------------
      !
      if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I2Recv(i1old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_l2)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              dxOut=xb2(i2old+1)-xb2(i2old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_l2)
      !
    else ! (i2old)
      ! --- Set intensity at m3/n3 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound3Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I3_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int3(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I3l, I3r, di3, 1, bound_l3, bound_r3, i2old, &
!!$                                      2230, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di3 == -1) then
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3r=I(:, i2old, n3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3r     to=', geo%Proc_r(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2231
            call MPI_ISend(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2231, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r3)
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2231, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l3)
          !
        else if (di3 == 1) then
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3l=I(:, i2old, m3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3l     to=', geo%Proc_l(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2232
            call MPI_ISend(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2232, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l3)
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2232, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r3)
          !
        endif ! (di3)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I3_timing)
        !
      endif ! (HasBound3Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        if (i3old == (m3-1)) then
          ! --- Set intensity at m3 side boundary ---
          !
          if (bound_l3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, n3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l3)
        endif ! (i3old)
        !
        if (i3old == (n3+1)) then
          ! --- Set intensity at n3 side boundary ---
          !
          if (bound_r3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, m3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r3)
        endif ! (i3old)
      end do ! i3old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 1, bound_l1, bound_r1, i2old, &
!!$                                      2210, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2211
            call MPI_ISend(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2211, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2211, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 1) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2212
            call MPI_ISend(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2212, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2212, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i3old
      !$OMP END DO
    endif ! (i2old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-max(di3,0), n3-min(di3,0)
      do i1=m1-max(di1,0), n1-min(di1,0)
        i1old=i1+di1
        i3old=i3+di3
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deinew00(i1,   i3)=deinew
        I(       i1,i2,i3)=Inew
        !
        drhoeidtime( i1old,i2old,i3old)= &
         drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
        drhoeidtime( i1   ,i2   ,i3   )= &
         drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#       if (rhd_shortrad_radpressure_l01 != 0)
!#
        drhov1dtime( i1old,i2old,i3old)= &
         drhov1dtime(i1old,i2old,i3old) - wdrhov1  * deiold
        drhov1dtime( i1   ,i2   ,i3   )= &
         drhov1dtime(i1   ,i2   ,i3   ) - wdrhov1  * deinew
        drhov2dtime( i1old,i2old,i3old)= &
         drhov2dtime(i1old,i2old,i3old) - wdrhov2  * deiold
        drhov2dtime( i1   ,i2   ,i3   )= &
         drhov2dtime(i1   ,i2   ,i3   ) - wdrhov2  * deinew
        drhov3dtime( i1old,i2old,i3old)= &
         drhov3dtime(i1old,i2old,i3old) - wdrhov3  * deiold
        drhov3dtime( i1   ,i2   ,i3   )= &
         drhov3dtime(i1   ,i2   ,i3   ) - wdrhov3  * deinew
!#
#       endif
!#
      end do ! i1
    end do ! i3
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i2old == n2) then
      if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I2Send(i1old, i3old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
      endif ! (bound_r2)
    endif ! (i2old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    if (ComputeFlux_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3-max(di3,0), n3-min(di3,0)
        do i1=m1-max(di1,0), n1-min(di1,0)
          Fradt(i1+di1,i3+di3)=wFrad * (I(i1,i2,i3) - deinew00(i1,i3))
        end do ! i1
      end do ! i3
      !$OMP END DO
      !
      if (di1 == -1) then
        if (di3 == -1) then
          ! --- -1-1 -------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13*(Fradt(i1-1,i3-1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1-1,i3  ) + &
                                             Fradt(i1  ,i3-1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2-1,i3  )= &
               Frad1(i1  ,i2-1,i3  ) + (w16* Fradt(i1-1,i3-1) + &
                                        w13* Fradt(i1-1,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i3-1) + &
                                        w16* Fradt(i1-1,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2-1,i3  )= &
               Frad3(i1  ,i2-1,i3  ) + (w16* Fradt(i1-1,i3-1) + &
                                        w13* Fradt(i1  ,i3-1)  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i3-1) + &
                                        w16* Fradt(i1  ,i3-1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else if (di3 == 0) then
          ! --- -10 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + w12*(Fradt(i1-1,i3  ) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2-1,i3  )= &
               Frad1(i1  ,i2-1,i3  ) + w12* Fradt(i1-1,i3  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + w12* Fradt(i1-1,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- -11 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13*(Fradt(i1-1,i3+1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1-1,i3  ) + &
                                             Fradt(i1  ,i3+1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2-1,i3  )= &
               Frad1(i1  ,i2-1,i3  ) + (w16* Fradt(i1-1,i3+1) + &
                                        w13* Fradt(i1-1,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i3+1) + &
                                        w16* Fradt(i1-1,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2-1,i3  )= &
               Frad3(i1  ,i2-1,i3  ) - (w16* Fradt(i1-1,i3  ) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i1-1,i3  ) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      else if (di1 == 0) then
        if (di3 == -1) then
          ! --- 0-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + w12*(Fradt(i1  ,i3-1) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2-1,i3  )= &
               Frad3(i1  ,i2-1,i3  ) + w12* Fradt(i1  ,i3-1)
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + w12* Fradt(i1  ,i3-1)
            end do ! i1
          end do ! i3
          !$OMP END DO
        else if (di3 == 0) then
          ! --- 00 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) +      Fradt(i1  ,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- 01 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + w12*(Fradt(i1  ,i3+1) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2-1,i3  )= &
               Frad3(i1  ,i2-1,i3  ) - w12* Fradt(i1  ,i3  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      else
        if (di3 == -1) then
          ! --- 1-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13*(Fradt(i1+1,i3-1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1+1,i3  ) + &
                                             Fradt(i1  ,i3-1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2-1,i3  )= &
               Frad1(i1  ,i2-1,i3  ) - (w16* Fradt(i1  ,i3-1) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i3-1) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2-1,i3  )= &
               Frad3(i1  ,i2-1,i3  ) + (w16* Fradt(i1+1,i3-1) + &
                                        w13* Fradt(i1  ,i3-1)  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13* Fradt(i1+1,i3-1) + &
                                        w16* Fradt(i1  ,i3-1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else if (di3 == 0) then
          ! --- 10 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + w12*(Fradt(i1+1,i3  ) + &
                                            Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2-1,i3  )= &
               Frad1(i1  ,i2-1,i3  ) - w12* Fradt(i1  ,i3  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i3  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- 11 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13*(Fradt(i1+1,i3+1) + &
                                             Fradt(i1  ,i3  )  ) + &
                                        w16*(Fradt(i1+1,i3  ) + &
                                             Fradt(i1  ,i3+1)  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2-1,i3  )= &
               Frad1(i1  ,i2-1,i3  ) - (w16* Fradt(i1  ,i3+1) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i3+1) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2-1,i3  )= &
               Frad3(i1  ,i2-1,i3  ) - (w16* Fradt(i1+1,i3  ) + &
                                        w13* Fradt(i1  ,i3  )  )
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) - (w13* Fradt(i1+1,i3  ) + &
                                        w16* Fradt(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i2
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I2Send to=', geo%Proc_r(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2221
    call MPI_ISend(I2Send, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_r(action%dimV(2)), &
                   2221, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_r2)
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I2 up'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir2_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i2s up'     , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntUp_flag)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(I)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
end subroutine rhd_shortrad_DirSimple2


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_DirSimple3(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                                   ComputeFlux_flag, ComputePressure_flag, &
                                   r_dir, r_wei, n_dir, &
                                   m1,n1, m2,n2, m3,n3, &
                                   xb1, xb2, xb3, &
                                   S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                   Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                                  if (rhd_shortrad_radpressure_l01 != 0)
!#
                                   drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                                  endif
!#
                                   RequestSendRecvMPI, I3Send, I3Recv &
                                   )
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_DirSimple3 ('rhd_short-characteristics_radiation_Direction_Simple_3')
!
! PURPOSE:
!   Compute radiative energy flux for one simple direction field with main direction 3.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_DirSimple3(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
!                                ComputeFlux_flag, ComputePressure_flag, &
!                                r_dir, r_wei, n_dir, &
!                                m1,n1, m2,n2, m3,n3, &
!                                xb1, xb2, xb3, &
!                                S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                                Frad1, Frad2, Frad3, drhoeidtime, &
!                                drhov1dtime, drhov2dtime, drhov3dtime, &
!                                RequestSendRecvMPI, I3Send, I3Recv &
!                                )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   RequestSendRecvMPI: ('Request_Send_Receive_MPI') integer vector with MPI requests to transfer
!                 boundary data
!   I3Send:       ('Intensity_3_Send') 2D real array with boundary intensity in MPI buffer
!   I3Recv:       ('Intensity_3_Receive') 2D real array with boundary intensity in MPI buffer
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   see rhd_shortrad_Dir1
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Wait:              Wait for given communication to complete
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_shortrad_MPIDir3Int1: Transfer x1 boundary intensity between processes for x3 integration
!   rhd_shortrad_MPIDir3Int2: Transfer x2 boundary intensity between processes for x3 integration
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Via MPI.
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_operator is called frequently and should be inlined.
!
! PROCEDURE:
!   Solve the radiative transfer equation along one direction (both orientations):
!   Call rhd_shortrad_operator for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F. Uppsala) First version
!   2001-11-22 (B.F.) Working version that is vectorizable and parallelizable
!   2002-07-23 (B.F.) Formula for wFrad and wXX changed
!   2002-07-30 (B.F.) Simplified version
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-01-22 (B.F.) New version of operator
!   2013-12-03 (B.F. Uppsala) New version
!   2013-12-06 (B.F.) New parameter r_wei
!   2013-12-11 (B.F.) Allow periodic side boundaries
!   2014-02-20 (B.F.) Handle boundaries within integration loops
!   2014-02-28 (B.F.) Handle all boundaries within integration loops
!   2014-03-06 (B.F.) New parameter 'action'
!   2014-03-15 (B.F.) New version of boundary-condition handling
!   2017-07-03 (B.F.) Add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-05 (B.F.) Add parameters for MPI transfer of top/bottom boundary intensity
!   2017-07-07 (B.F.) Implement MPI transfer of side bounday intensity
!   2017-07-20 (B.F.) Re-arrange MPI send/receive requests
!   2017-07-23 (B.F.) Re-arrange MPI send/receive requests
!   2017-10-01 (B.F.) Put lateral MPI boundary intensity exchange into separate routines
!   2017-10-04 (B.F.) Outcomment routine calls for MPI boundary intensity exchange
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use const_module, only: fourpi, c_light
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
!!$use uio_bulk_module
!!$use uio_filedef_module
use timing_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+2),intent(in)   :: xb1
real, dimension(m2-1:n2+2),intent(in)   :: xb2
real, dimension(m3-1:n3+2),intent(in)   :: xb3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1 ), &
                           intent(inout):: Frad3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
integer, dimension(2),     intent(inout):: RequestSendRecvMPI
real, dimension(m1-1:n1+1, m2-1:n2+1), &
                           intent(inout):: I3Send, I3Recv
!
! --- Local variables ---
real                                    :: deiold, deinew, Inew, &
                                           mu, wFrad, wdivFrad, wdrhov1, wdrhov2, wdrhov3, &
                                           dx1, dx2, dx3, &
                                           RabsOut, RcosthetaOut, Dout, tauOut, dxOut, RdOut
integer                                    di1, di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3, nc, &
                                           bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3, &
                                           I1_timing, I2_timing
logical                                 :: BoundHtau_flag
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
logical                                 :: HasBound1Internal_flag, HasBound2Internal_flag
integer                                 :: ierrMPI
integer, dimension(4)                   :: RequestArrMPI
real, dimension(           m2-1:n2+1)   :: I1l, I1r
real, dimension(m1-1:n1+1           )   :: I2l, I2r
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
real, dimension(3)                      :: Rout
real, dimension(m1-1:n1+1, &
                m2-1:n2+1 )             :: deiold00, deinew00, Fradt
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 )             :: I
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: I
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: I
!#
# endif
!#      /* (rhd_arrays_l01) */
real, parameter                         :: dtauthick_limit=0.17
real, parameter                         :: w12=0.5, w13=1.0/3.0, w16=1.0/6.0
real                                    :: Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew, &
                                           thickmaskold, thickmasknew
!real                                    :: wSold, wSnew, OneOdtauhalfold, OneOdtauhalfnew, Sratio4
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!$ integer                              :: omp_get_thread_num
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(I(m1-1:n1+1, m2-1:n2+1, m3-1:n3+1))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
dx1=    (geo%xb_r(action%dimV(1))-geo%xb_l(action%dimV(1))  ) / &
    real(geo%n(   action%dimV(1))-geo%m(   action%dimV(1))+1)
dx2=    (geo%xb_r(action%dimV(2))-geo%xb_l(action%dimV(2))  ) / &
    real(geo%n(   action%dimV(2))-geo%m(   action%dimV(2))+1)
dx3=    (geo%xb_r(action%dimV(3))-geo%xb_l(action%dimV(3))   )/ &
    real(geo%n(   action%dimV(3))-geo%m(   action%dimV(3))+1)
!
! --- Find indices of relevant cells in old plane ---
di1  =nint(r_dir(1)/r_dir(3))
di2  =nint(r_dir(2)/r_dir(3))
di3  =1
RdOut=1.0
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
mu      =abs(r_dir(3))
wFrad   =fourpi * 0.5 * mu / real(max(n_dir,3))
wdivFrad=wFrad / dx3
wdrhov1 =(wdivFrad / c_light) * r_dir(1)
wdrhov2 =(wdivFrad / c_light) * r_dir(2)
wdrhov3 =(wdivFrad / c_light) * r_dir(3)
!
! --- Check boundary settings ---
BoundHtau_flag=(par%C_radHtautop > 0.0)
if (action%dim == 2) then
  ! --- Transposed arrays: copy boundary-condition parameters ---
  bound_l1=action%bound_l2
  bound_r1=action%bound_r2
  bound_l2=action%bound_l3
  bound_r2=action%bound_r3
  bound_l3=action%bound_l1
  bound_r3=action%bound_r1
else ! (action%dim)
  ! --- Copy boundary-condition parameters ---
  bound_l1=action%bound_l1
  bound_r1=action%bound_r1
  bound_l2=action%bound_l2
  bound_r2=action%bound_r2
  bound_l3=action%bound_l3
  bound_r3=action%bound_r3
endif ! (action%dim)
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
HasBound1Internal_flag=(((bound_l1 == bound_InternalNext    ) .or. &
                         (bound_l1 == bound_InternalPeriodic) .or. &
                         (bound_r1 == bound_InternalNext    ) .or. &
                         (bound_r1 == bound_InternalPeriodic)       ) .and. &
                        ((di1 /= 0)                                          )        )
HasBound2Internal_flag=(((bound_l2 == bound_InternalNext    ) .or. &
                         (bound_l2 == bound_InternalPeriodic) .or. &
                         (bound_r2 == bound_InternalNext    ) .or. &
                         (bound_r2 == bound_InternalPeriodic)       ) .and. &
                        ((di2 /= 0)                                          )        )
! write(gl%nc_p, '(A,I3,A,L1,2X,L1)') &
!   '#', geo%MyProc, ' HasBound1,2Internal_flag= ', &
!   HasBound1Internal_flag, HasBound2Internal_flag
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ComputeIntDown_flag) then
  ! ### Integration i3: downward ###################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I3 down, di1=', di1, ' di2=', di2
  !
  call timing_GetHandle('SHC:   dir3: D I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir3: D I2 SeRc', I2_timing)
  !
  if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I3Recv from=', geo%Proc_r(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !  ' tag=', 3132
    ! call timing_start('SHC:   dir3: s D I3 Recv')
    call MPI_IRecv(I3Recv, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_r(action%dimV(3)), &
                   3132, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir3: s D I3 Recv')
    call timing_start('SHC:   dir3: s D I3 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: s D I3 Recv Wait')
  endif ! (bound_r3)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir3: s D I3 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: s D I3 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i3, i1old, i2old, i3old, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i3=n3, m3-1, -1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i3old=i3+1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i3old == (n3+1)) then
      ! --- Set intensity at starting face for integration (n3 boundary) ---------------------------
      !
      if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I3Recv(i1old, i2old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_r3)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              dxOut=xb3(i3old+1)-xb3(i3old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_r3)
      !
    else ! (i3old)
      ! --- Set intensity at m2/n2 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound2Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I2_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int2(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I2l, I2r, di2, 1, bound_l2, bound_r2, i3old, &
!!$                                      3120, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di2 == -1) then
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2r=I(:, n2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2r     to=', geo%Proc_r(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3121
            call MPI_ISend(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3121, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r2)
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3121, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l2)
          !
        else if (di2 == 1) then
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2l=I(:, m2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2l     to=', geo%Proc_l(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3122
            call MPI_ISend(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3122, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l2)
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3122, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r2)
          !
        endif ! (di2)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I2_timing)
        !
      endif ! (HasBound2Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        if (i2old == (m2-1)) then
          ! --- Set intensity at m2 side boundary ---
          !
          if (bound_l2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, n2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l2)
        endif ! (i2old)
        !
        if (i2old == (n2+1)) then
          ! --- Set intensity at n2 side boundary ---
          !
          if (bound_r2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, m2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r2)
        endif ! (i2old)
      end do ! i2old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 1, bound_l1, bound_r1, i3old, &
!!$                                      3110, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3111
            call MPI_ISend(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3111, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3111, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 1) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3112
            call MPI_ISend(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3112, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3112, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i2old
      !$OMP END DO
      !
    endif ! (i3old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i2=m2-max(di2,0), n2-min(di2,0)
      do i1=m1-max(di1,0), n1-min(di1,0)
        i1old=i1+di1
        i2old=i2+di2
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i1old,i2old   )=deiold
        I(       i1   ,i2   ,i3)=Inew
        !
        drhoeidtime( i1old,i2old,i3old)= &
         drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
        drhoeidtime( i1   ,i2,i3      )= &
         drhoeidtime(i1   ,i2,i3      ) - wdivFrad * deinew
!#
#       if (rhd_shortrad_radpressure_l01 != 0)
!#
        drhov1dtime( i1old,i2old,i3old)= &
         drhov1dtime(i1old,i2old,i3old) + wdrhov1  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
        drhov1dtime( i1   ,i2   ,i3   )= &
         drhov1dtime(i1   ,i2   ,i3   ) + wdrhov1  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
        drhov2dtime( i1old,i2old,i3old)= &
         drhov2dtime(i1old,i2old,i3old) + wdrhov2  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
        drhov2dtime( i1   ,i2   ,i3   )= &
         drhov2dtime(i1   ,i2   ,i3   ) + wdrhov2  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
        drhov3dtime( i1old,i2old,i3old)= &
         drhov3dtime(i1old,i2old,i3old) + wdrhov3  * (deiold + 2.0*thickmaskold*(Sreduce-Sold   ))
        drhov3dtime( i1   ,i2   ,i3   )= &
         drhov3dtime(i1   ,i2   ,i3   ) + wdrhov3  * (deinew + 2.0*thickmasknew*(Snew   -Sreduce))
!#
#       endif
!#
      end do ! i1
    end do ! i2
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i3old == m3) then
      if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I3Send(i1old, i2old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
      endif ! (bound_l3)
    endif ! (i3old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    if (ComputeFlux_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i2=m2-max(di2,0), n2-min(di2,0)
        do i1=m1-max(di1,0), n1-min(di1,0)
          Fradt(i1+di1,i2+di2)=wFrad * (I(i1+di1,i2+di2,i3+1) + deiold00(i1+di1,i2+di2))
        end do ! i1
      end do ! i2
      !$OMP END DO
      !
      if (di1 == -1) then
        if (di2 == -1) then
          ! --- -1-1 -------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - (w13*(Fradt(i1-1,i2-1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1-1,i2  ) + &
                                             Fradt(i1  ,i2-1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1) + (w16* Fradt(i1-1,i2-1) + &
                                        w13* Fradt(i1-1,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i2-1) + &
                                        w16* Fradt(i1-1,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1) + (w16* Fradt(i1-1,i2-1) + &
                                        w13* Fradt(i1  ,i2-1)  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i2-1) + &
                                        w16* Fradt(i1  ,i2-1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else if (di2 == 0) then
          ! --- -10 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - w12*(Fradt(i1-1,i2  ) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1) + w12* Fradt(i1-1,i2  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + w12* Fradt(i1-1,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- -11 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - (w13*(Fradt(i1-1,i2+1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1-1,i2  ) + &
                                             Fradt(i1  ,i2+1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1) + (w16* Fradt(i1-1,i2+1) + &
                                        w13* Fradt(i1-1,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i2+1) + &
                                        w16* Fradt(i1-1,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1) - (w16* Fradt(i1-1,i2  ) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i1-1,i2  ) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      else if (di1 == 0) then
        if (di2 == -1) then
          ! --- 0-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - w12*(Fradt(i1  ,i2-1) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1) + w12* Fradt(i1  ,i2-1)
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + w12* Fradt(i1  ,i2-1)
            end do ! i1
          end do ! i2
          !$OMP END DO
        else if (di2 == 0) then
          ! --- 00 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) -      Fradt(i1  ,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- 01 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - w12*(Fradt(i1  ,i2+1) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1) - w12* Fradt(i1  ,i2  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      else
        if (di2 == -1) then
          ! --- 1-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - (w13*(Fradt(i1+1,i2-1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1+1,i2  ) + &
                                             Fradt(i1  ,i2-1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1) - (w16* Fradt(i1  ,i2-1) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i2-1) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1) + (w16* Fradt(i1+1,i2-1) + &
                                        w13* Fradt(i1  ,i2-1)  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i1+1,i2-1) + &
                                        w16* Fradt(i1  ,i2-1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else if (di2 == 0) then
          ! --- 10 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - w12*(Fradt(i1+1,i2  ) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1) - w12* Fradt(i1  ,i2  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- 11 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1) - (w13*(Fradt(i1+1,i2+1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1+1,i2  ) + &
                                             Fradt(i1  ,i2+1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1) - (w16* Fradt(i1  ,i2+1) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i2+1) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1) - (w16* Fradt(i1+1,i2  ) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i1+1,i2  ) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i3
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I3Send to=', geo%Proc_l(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3132
    call MPI_ISend(I3Send, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_l(action%dimV(3)), &
                   3132, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_l3)
  !
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I3 down'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir3_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i3s down'   , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntDown_flag)
!
! --- Adjust indices ---
di1=-di1
di2=-di2
di3=-di3
RdOut=-1.0
!
if (ComputeIntUp_flag) then
  ! ### Integration i3: upward #####################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I3 up,   di1=', di1, ' di2=', di2
  !
  call timing_GetHandle('SHC:   dir3: U I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir3: U I2 SeRc', I2_timing)
  !
  if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I3Recv from=', geo%Proc_l(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3231
    ! call timing_start('SHC:   dir3: s U I3 Recv')
    call MPI_IRecv(I3Recv, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_l(action%dimV(3)), &
                   3231, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir3: s U I3 Recv')
    call timing_start('SHC:   dir3: s U I3 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: s U I3 Recv Wait')
  endif ! (bound_l3)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir3: s U I3 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: s U I3 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i3, i1old, i2old, i3old, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i3=m3, n3+1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i3old=i3-1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i3old == (m3-1)) then
      ! --- Set intensity at starting face for integration (m3 boundary) ---------------------------
      !
      if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I3Recv(i1old, i2old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_l3)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              dxOut=xb3(i3old+1)-xb3(i3old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_l3)
      !
    else ! (i3old)
      ! --- Set intensity at m2/n2 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound2Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I2_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int2(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I2l, I2r, di2, 1, bound_l2, bound_r2, i3old, &
!!$                                      3220, RequestArrMPI)
!!$        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di2 == -1) then
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2r=I(:, n2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2r     to=', geo%Proc_r(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3221
            call MPI_ISend(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3221, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r2)
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3221, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l2)
          !
        else if (di2 == 1) then
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2l=I(:, m2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2l     to=', geo%Proc_l(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3222
            call MPI_ISend(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3222, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l2)
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3222, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r2)
          !
        endif ! (di2)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I2_timing)
        !
      endif ! (HasBound2Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        if (i2old == (m2-1)) then
          ! --- Set intensity at m2 side boundary ---
          !
          if (bound_l2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, n2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l2)
        endif ! (i2old)
        !
        if (i2old == (n2+1)) then
          ! --- Set intensity at n2 side boundary ---
          !
          if (bound_r2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, m2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r2)
        endif ! (i2old)
      end do ! i2old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 1, bound_l1, bound_r1, i3old, &
!!$                                      3210, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3211
            call MPI_ISend(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3211, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3211, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 1) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3212
            call MPI_ISend(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3212, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3212, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i2old
      !$OMP END DO
    endif ! (i3old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i2=m2-max(di2,0), n2-min(di2,0)
      do i1=m1-max(di1,0), n1-min(di1,0)
        i1old=i1+di1
        i2old=i2+di2
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deinew00(i1,i2   )=deinew
        I(       i1,i2,i3)=Inew
        !
        drhoeidtime( i1old,i2old,i3old)= &
         drhoeidtime(i1old,i2old,i3old) - wdivFrad * deiold
        drhoeidtime( i1   ,i2   ,i3   )= &
         drhoeidtime(i1   ,i2   ,i3   ) - wdivFrad * deinew
!#
#       if (rhd_shortrad_radpressure_l01 != 0)
!#
        drhov1dtime( i1old,i2old,i3old)= &
         drhov1dtime(i1old,i2old,i3old) - wdrhov1  * deiold
        drhov1dtime( i1   ,i2   ,i3   )= &
         drhov1dtime(i1   ,i2   ,i3   ) - wdrhov1  * deinew
        drhov2dtime( i1old,i2old,i3old)= &
         drhov2dtime(i1old,i2old,i3old) - wdrhov2  * deiold
        drhov2dtime( i1   ,i2   ,i3   )= &
         drhov2dtime(i1   ,i2   ,i3   ) - wdrhov2  * deinew
        drhov3dtime( i1old,i2old,i3old)= &
         drhov3dtime(i1old,i2old,i3old) - wdrhov3  * deiold
        drhov3dtime( i1   ,i2   ,i3   )= &
         drhov3dtime(i1   ,i2   ,i3   ) - wdrhov3  * deinew
!#
#       endif
!#
      end do ! i1
    end do ! i2
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i3old == n3) then
      if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I3Send(i1old, i2old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
      endif ! (bound_r3)
    endif ! (i3old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    if (ComputeFlux_flag) then
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2=m2-max(di2,0), n2-min(di2,0)
        do i1=m1-max(di1,0), n1-min(di1,0)
          Fradt(i1+di1,i2+di2)=wFrad * (I(i1,i2,i3) - deinew00(i1,i2))
        end do ! i1
      end do ! i2
      !$OMP END DO
      !
      if (di1 == -1) then
        if (di2 == -1) then
          ! --- -1-1 -------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13*(Fradt(i1-1,i2-1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1-1,i2  ) + &
                                             Fradt(i1  ,i2-1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3-1)= &
               Frad1(i1  ,i2  ,i3-1) + (w16* Fradt(i1-1,i2-1) + &
                                        w13* Fradt(i1-1,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i2-1) + &
                                        w16* Fradt(i1-1,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3-1)= &
               Frad2(i1  ,i2  ,i3-1) + (w16* Fradt(i1-1,i2-1) + &
                                        w13* Fradt(i1  ,i2-1)  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i2-1) + &
                                        w16* Fradt(i1  ,i2-1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else if (di2 == 0) then
          ! --- -10 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + w12*(Fradt(i1-1,i2  ) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3-1)= &
               Frad1(i1  ,i2  ,i3-1) + w12* Fradt(i1-1,i2  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + w12* Fradt(i1-1,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- -11 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13*(Fradt(i1-1,i2+1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1-1,i2  ) + &
                                             Fradt(i1  ,i2+1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3-1)= &
               Frad1(i1  ,i2  ,i3-1) + (w16* Fradt(i1-1,i2+1) + &
                                        w13* Fradt(i1-1,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) + (w13* Fradt(i1-1,i2+1) + &
                                        w16* Fradt(i1-1,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3-1)= &
               Frad2(i1  ,i2  ,i3-1) - (w16* Fradt(i1-1,i2  ) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i1-1,i2  ) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      else if (di1 == 0) then
        if (di2 == -1) then
          ! --- 0-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + w12*(Fradt(i1  ,i2-1) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3-1)= &
               Frad2(i1  ,i2  ,i3-1) + w12* Fradt(i1  ,i2-1)
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + w12* Fradt(i1  ,i2-1)
            end do ! i1
          end do ! i2
          !$OMP END DO
        else if (di2 == 0) then
          ! --- 00 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) +      Fradt(i1  ,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- 01 ------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + w12*(Fradt(i1  ,i2+1) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3-1)= &
               Frad2(i1  ,i2  ,i3-1) - w12* Fradt(i1  ,i2  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      else
        if (di2 == -1) then
          ! --- 1-1 --------------------------------------------------------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13*(Fradt(i1+1,i2-1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1+1,i2  ) + &
                                             Fradt(i1  ,i2-1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3-1)= &
               Frad1(i1  ,i2  ,i3-1) - (w16* Fradt(i1  ,i2-1) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i2-1) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3-1)= &
               Frad2(i1  ,i2  ,i3-1) + (w16* Fradt(i1+1,i2-1) + &
                                        w13* Fradt(i1  ,i2-1)  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) + (w13* Fradt(i1+1,i2-1) + &
                                        w16* Fradt(i1  ,i2-1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else if (di2 == 0) then
          ! --- 10 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + w12*(Fradt(i1+1,i2  ) + &
                                            Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3-1)= &
               Frad1(i1  ,i2  ,i3-1) - w12* Fradt(i1  ,i2  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - w12* Fradt(i1  ,i2  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- 11 -------------------------------
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  ) + (w13*(Fradt(i1+1,i2+1) + &
                                             Fradt(i1  ,i2  )  ) + &
                                        w16*(Fradt(i1+1,i2  ) + &
                                             Fradt(i1  ,i2+1)  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3-1)= &
               Frad1(i1  ,i2  ,i3-1) - (w16* Fradt(i1  ,i2+1) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  ) - (w13* Fradt(i1  ,i2+1) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3-1)= &
               Frad2(i1  ,i2  ,i3-1) - (w16* Fradt(i1+1,i2  ) + &
                                        w13* Fradt(i1  ,i2  )  )
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  ) - (w13* Fradt(i1+1,i2  ) + &
                                        w16* Fradt(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i3
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I3Send to=', geo%Proc_r(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3231
    call MPI_ISend(I3Send, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_r(action%dimV(3)), &
                   3231, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_r3)
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I3 up'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir3_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i3s up'     , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntUp_flag)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(I)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
end subroutine rhd_shortrad_DirSimple3


!----------*****************------------------------------------------------------------------------
subroutine rhd_shortrad_Dir1(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                             ComputeFlux_flag, ComputePressure_flag, &
                             r_dir, r_wei, n_dir, &
                             m1,n1, m2,n2, m3,n3, &
                             xb1, xb2, xb3, &
                             S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                             Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                            if (rhd_shortrad_radpressure_l01 != 0)
!#
                             drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                            endif
!#
                             RequestSendRecvMPI, I1Send, I1Recv &
                             )
!---------------------------------------------------------------------------------------------------

!#
# if (rhd_shortrad_dir1_l01 == 0)
!#                                /* swapped indices */

!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_Dir1 ('rhd_short-characteristics_radiation_Direction_1')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field with main direction 1.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_Dir1(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
!                          ComputeFlux_flag, ComputePressure_flag, &
!                          r_dir, r_wei, n_dir, &
!                          m1,n1, m2,n2, m3,n3, &
!                          xb1, xb2, xb3, &
!                          S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                          Frad1, Frad2, Frad3, drhoeidtime, &
!                          drhov1dtime, drhov2dtime, drhov3dtime, &
!                          RequestSendRecvMPI, I1Send, I1Recv &
!                          )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   RequestSendRecvMPI: ('Request_Send_Receive_MPI') integer vector with MPI requests to transfer
!                 boundary data
!   I1Send:       ('Intensity_1_Send') 2D real array with boundary intensity in MPI buffer
!   I1Recv:       ('Intensity_1_Receive') 2D real array with boundary intensity in MPI buffer
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   w00:          ('weight_00') real, weight of neighbor cell (di2,di3)+(0,0)
!   w10:          ('weight_10') real, weight of neighbor cell (di2,di3)+(1,0)
!   w01:          ('weight_01') real, weight of neighbor cell (di2,di3)+(0,1)
!   w11:          ('weight_11') real, weight of neighbor cell (di2,di3)+(1,1)
!   w:            ('weight') real, temporary variable for weight of current cell pair
!   wt:           ('weight_temporary') real, temporary variable used during value swapping
!   deiold:       ('delta_energy_internal_old') real, intensity change in old interval
!   thickmaskold: ('thick_mask_old') real with mask to distinguish optically thick/thin in old
!                 interval: 1.0 for optically thick, 0.0 for optically thin region
!   deinew:       ('delta_energy_internal_new') real, intensity change in new interval
!   deinew_total: ('delta_energy_internal_new_total') real, intensity change in new cell
!   thickmasknew: ('thick_mask_new') real with mask to distinguish optically thick/thin in new
!                 interval: 1.0 for optically thick, 0.0 for optically thin region
!   Inew:         ('Intensity_new') real, (reduced) intensity at end of new interval
!   Inew_total:   ('Intensity_new_total') real, (reduced) intensity in new cell
!   mu:           ('mu') real, cosine of inclination angle relative to x1 direction
!   wFrad:        ('weight_Flux_radiative') real, weight to transform from ray intensity
!                 to energy flux into cell;
!                 accounts for 'pi', ray inclination and total number of rays
!   wdivFrad:     ('weight_divergence_Flux_radiative') real, weight to transform from change in
!                 ray intensity to energy flux divergence in cell,
!                 accounts for 'pi', ray inclination, total number of rays, and cell size
!   dx1:          ('delta_x_1') real, cell size in x1 direction (taken from xb1)
!   dx2:          ('delta_x_2') real, cell size in x2 direction (taken from xb2)
!   dx3:          ('delta_x_3') real, cell size in x3 direction (taken from xb3)
!   di2:          ('delta_index_2') integer, relative index of leftmost important neighbor cell
!                 in x2 direction (-1 or 0)
!   di3:          ('delta_index_3') integer, relative index of leftmost important neighbor cell
!                 in x3 direction (-1 or 0)
!   i2old:        ('index_2_old') integer, temporary variable of x2 index of old cell
!   i3old:        ('index_3_old') integer, temporary variable of x3 index of old cell
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   deiold00:     ('delta_energy_internal_old_00') 2D real array for intensity change in 00 cell
!                 in old layer
!   deiold10:     ('delta_energy_internal_old_10') 2D real array for intensity change in 10 cell
!                 in old layer
!   deiold01:     ('delta_energy_internal_old_01') 2D real array for intensity change in 01 cell
!                 in old layer
!   deiold11:     ('delta_energy_internal_old_11') 2D real array for intensity change in 11 cell
!                 in old layer
!   deinew00:     ('delta_energy_internal_new_00') 2D real array for intensity change in new layer
!                 induced from 00 cell in old layer
!   deinew10:     ('delta_energy_internal_new_10') 2D real array for intensity change in new layer
!                 induced from 10 cell in old layer
!   deinew01:     ('delta_energy_internal_new_01') 2D real array for intensity change in new layer
!                 induced from 01 cell in old layer
!   deinew11:     ('delta_energy_internal_new_11') 2D real array for intensity change in new layer
!                 induced from 11 cell in old layer
!   deinewa:      ('delta_energy_internal_new_array') 2D real array for total intensity change
!                 for each cell in new layer
!   Inew00:       ('Intensity_new_00') 2D real array for intensity contribution of 00 cell in
!                 old layer to intensity in cell in new layer
!   Inew10:       ('Intensity_new_10') 2D real array for intensity contribution of 10 cell in
!                 old layer to intensity in cell in new layer
!   Inew01:       ('Intensity_new_01') 2D real array for intensity contribution of 01 cell in
!                 old layer to intensity in cell in new layer
!   Inew11:       ('Intensity_new_11') 2D real array for intensity contribution of 11 cell in
!                 old layer to intensity in cell in new layer
!   I:            ('Intensity') 3D real array with intensity values at cell centers
!   nFb:          ('number_Flux_boundary') number of extra boundary cells in flux arrays
!   dtauthick_limit: ('delta_tau_thick_limit') real parameter, border between optically thin
!                    and thick; typically 1/6
!
! ROUTINES:
!   rhd_shortrad_Dir3:     Compute radiative energy flux for one direction field, main direction 3
!   rhd_shortrad_DirSimple1: Compute radiative energy flux, simple direction field, main direction 1
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   rhd_shortrad_RayWeight: Compute weights for incoming rays
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_operator is called frequently and should be inlined.
!
! PROCEDURE:
!   Solve the radiative transfer equation along one direction (both orientations):
!   Call rhd_shortrad_operator for all necessary pairs of cells in a reasonable order.
!
!   Weight indices of relevant neighbor cells in old layer:
!
!     ^i3
!     |
!     +------+------+
!     |      |      |
!     |  01  |  11  |
!     |      |      |
!     +------+------+
!     |      |      |
!     |  00  |  10  |
!     |      |      |
!     +------+------+->i2
!
!   Position of relevant neighbor cells in old layer relativ to cell i in new layer:
!
!     di2=-1, di3=-1   +---+---+     di2= 0, di3=-1   +---+---+
!                      |   | i |                      | i |   |
!                      +---+---+                      +---+---+
!                      |   |   |                      |   |   |
!                      +---+---+                      +---+---+
!
!     di2=-1, di3= 0   +---+---+     di2= 0, di3= 0   +---+---+
!                      |   |   |                      |   |   |
!                      +---+---+                      +---+---+
!                      |   | i |                      | i |   |
!                      +---+---+                      +---+---+
!
!   Side view (di2=-1):
!     The cells of the original Cartesian grid are assumed to be "sheared" to be aligned with
!     the current ray.
!
!                        ----+---------+---------+----
!                          \ |       \ |       \ |
!                           \|        \|        \|
!     \              i1+1    |   ......|....     |      -  Iold, Sold
!      \                     |\   .    |\   .    |\      \_ dtauhalfold
!       \                    | \   .   | \   .   | \     /
!        \ intensity     ----+------===+=======--+----  -
!         \                          \ |   .   \ |       \_ dtauhalfnew
!          \|                         \|    .   \|       /
!          _.        i1                |.........|      -  Inew, Snew
!                                      |\        |\
!                                      | \       | \
!                        ----+---------+--=======+===-
!                               i2-1       i2
!
!   To compute the reduced intensity I in cell (i1,i2,i3) the source function S and the optical
!   depth dtauhalf in that cell are needed, as well as I, S, and dtauhalf in the 4 neighbor cells
!   (i1+1,i2-1,i-3), (i1+1,i2-1,i3), (i1+1,i2,i3-1), (i1+1,i2,i3), for example.
!   To compute the contribution from each of the 4 neighbor cells an individual radiative
!   transfer step (call of rhd_shortrad_operator) is performed for each of the four cell pairs,
!   e.g. for the cell pairs
!     [I,S,dtauhalf]_(i1+1,i2-1,i3-1), [S,dtauhalf]_(i1,i2,i3)  ->  I_(i1,i2,i3)
!     [I,S,dtauhalf]_(i1+1,i2-1,i3  ), [S,dtauhalf]_(i1,i2,i3)  ->  I_(i1,i2,i3)
!     [I,S,dtauhalf]_(i1+1,i2  ,i3-1), [S,dtauhalf]_(i1,i2,i3)  ->  I_(i1,i2,i3)
!     [I,S,dtauhalf]_(i1+1,i2  ,i3  ), [S,dtauhalf]_(i1,i2,i3)  ->  I_(i1,i2,i3)
!
!   The intensity is needed to solve the equation of radiation transfer, but is itself only of
!   secondary interest. The important quantity is the intensity change along each dtauhalf
!   interval, which gives immediately the contribution to the energy change in the corresponding
!   cell.
!
!   The compute the overall flux (and especially its divergence) the intensity for both
!   orientations along a given ray direction is needed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F. Uppsala) First version
!   2001-11-22 (B.F.) Working version that is vectorizable and parallelizable
!   2002-07-23 (B.F.) Formula for wFrad and wXX changed
!   2004-05-26 (B.F.) Bugfix: declaration of deiold00...: m1,m2 -> m2,m3
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2014-02-20 (B.F.) Handle boundaries within integration loops
!   2014-02-28 (B.F.) Handle all boundaries within integration loops
!   2014-03-06 (B.F.) New parameter 'action'
!   2017-07-03 (B.F.) Rename flux_flag, pressure_flag -> ComputeFlux_flag, ComputePressure_flag;
!                     add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-06 (B.F.) Add parameters for MPI transfer of boundary intensity
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use const_module, only: fourpi, c_light
use rhd_action_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+2),intent(in)   :: xb1
real, dimension(m2-1:n2+2),intent(in)   :: xb2
real, dimension(m3-1:n3+2),intent(in)   :: xb3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1 ), &
                           intent(inout):: Frad3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
integer, dimension(2),     intent(inout):: RequestSendRecvMPI
real, dimension(m2-1:n2+1, m3-1:n3+1), &
                           intent(inout):: I1Send, I1Recv
!
! --- Local variables ---
real                                    :: w00, w01, w10, w11, w, wt, &
                                           deinew_total, Inew_total, deiold, deinew, Inew, &
                                           mu, wFrad, wdivFrad, wdrhov1, wdrhov2, wdrhov3, &
                                           dx1, dx2, dx3, deioldsum, deinewsum
integer                                    di1, di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3
real, dimension(m2-1:n2+1, &
                m3-1:n3+1 )             :: deiold00, deiold01, deiold10, deiold11, &
                                           deinewa , &
                                           deinew00, deinew01, deinew10, deinew11, &
                                           Inew00, Inew01, Inew10, Inew11, &
                                           dSold00, dSold01, dSold10, dSold11, &
                                           dSnew00, dSnew01, dSnew10, dSnew11
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 )             :: I
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: I
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: I
!#
# endif
!#      /* (rhd_arrays_l01) */
real, parameter                         :: dtauthick_limit=0.17
real                                    :: Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew, &
                                           thickmaskold, thickmasknew
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!---------------------------------------------------------------------------------------------------
!
if ( (abs(anint(r_dir(2)/r_dir(1))-(r_dir(2)/r_dir(1))) < 0.001) .and. &
     (abs(anint(r_dir(3)/r_dir(1))-(r_dir(3)/r_dir(1))) < 0.001)      ) then
  call rhd_shortrad_DirSimple1(geo, ComputeIntDown_flag, ComputeIntUp_flag, &
                               ComputeFlux_flag, ComputePressure_flag, &
                               r_dir, r_wei, n_dir, &
                               m1,n1, m2,n2, m3,n3, &
                               xb1, xb2, xb3, &
                               S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                               Frad1, Frad2, Frad3, drhoeidtime &
!#
#                              if (rhd_shortrad_radpressure_l01 != 0)
!#
                               , drhov1dtime, drhov2dtime, drhov3dtime &
!#
#                              endif
!#
                               )
else ! (r_dir()/r_dir_(1))
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(I(m1-1:n1+1, m2-1:n2+1, m3-1:n3+1))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
dx1=(geo%xb_r(1)-geo%xb_l(1))/real(geo%n(1)-geo%m(1)+1)
dx2=(geo%xb_r(2)-geo%xb_l(2))/real(geo%n(2)-geo%m(2)+1)
dx3=(geo%xb_r(3)-geo%xb_l(3))/real(geo%n(3)-geo%m(3)+1)
!
! --- Find indices of relevant cells in old plane ---
call rhd_shortrad_RayWeight(r_dir(1), r_dir(2), r_dir(3), di2, di3, w00, w01, w10, w11)
di1=1
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
mu      =abs(r_dir(1))
wFrad   =fourpi * 0.5 * mu / real(max(n_dir,3))
wdivFrad=wFrad / dx1
wdrhov1 =(wdivFrad / c_light) * r_dir(1)
wdrhov2 =(wdivFrad / c_light) * r_dir(2)
wdrhov3 =(wdivFrad / c_light) * r_dir(3)
!
if (ComputeIntDown_flag) then
  ! === Integration i1: downward ===================================================================
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!
  do i1=n1, m1-1, -1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i2, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 ==0 */
    !
    i1old=i1+1
    !
    ! === Set boundary intensity ===
    !
    if (i1 == n1) then
      ! --- Set intensity at n1 boundary ---
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3, n3
        do i2=m2, n2
          if (dtauhalf(i1old, i2, i3) > dtauthick_limit) then
            I(         i1old, i2, i3)=-S(i1old, i2, i3)
          else
            I(         i1old, i2, i3)=0.0
          endif
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
    !
    ! === Perform radiation-transport step ===
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1-di3, n3-di3
      if (i3 == (m3-1-di3)) then
        ! --- Set intensity at m3 boundary ---
        where (dtauhalf(i1old, :   , m3-1) > dtauthick_limit)
                      I(i1old, :   , m3-1)=-S(i1old, :   , m3-1)
        elsewhere
                      I(i1old, :   , m3-1)=0.0
        endwhere
      endif
      if (i3 == (n3-di3)) then
        ! --- Set intensity at n3 boundary ---
        where (dtauhalf(i1old, :   , n3+1) > dtauthick_limit)
                      I(i1old, :   , n3+1)=-S(i1old, :   , n3+1)
        elsewhere
                      I(i1old, :   , n3+1)=0.0
        endwhere
      endif
      ! --- Set intensity at m2 boundary ---
      if (dtauhalf(i1old, m2-1, i3  ) > dtauthick_limit) then
                 I(i1old, m2-1, i3  )=-S(i1old, m2-1, i3  )
      else
                 I(i1old, m2-1, i3  )=0.0
      endif
      ! --- Set intensity at n2 boundary ---
      if (dtauhalf(i1old, n2+1, i3  ) > dtauthick_limit) then
                 I(i1old, n2+1, i3  )=-S(i1old, n2+1, i3  )
      else
                 I(i1old, n2+1, i3  )=0.0
      endif
    end do ! i3
    !$OMP END DO
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1-di3, n3-di3
      do i2=m2-1-di2, n2-di2
        deinew_total=0.0
        Inew_total  =0.0
        !
        i2old=i2+di2
        i3old=i3+di3
        w    =w00
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i2old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold00( i2old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew00( i2   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i2old=i2+di2+1
        i3old=i3+di3
        w    =w10
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold10(i2old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold10( i2old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew10( i2   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i2old=i2+di2
        i3old=i3+di3+1
        w    =w01
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold01(i2old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold01( i2old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew01( i2   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i2old=i2+di2+1
        i3old=i3+di3+1
        w    =w11
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold11(i2old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold11( i2old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew11( i2   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        deinewa(   i2,i3)    =deinew_total
        I(      i1,i2,i3)    =Inew_total
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        drhoeidtime( i1old,i2,i3)= &
         drhoeidtime(i1old,i2,i3) - wdivFrad * &
                           (deiold00(i2,i3) + deiold01(i2,i3) + deiold10(i2,i3) + deiold11(i2,i3))
        drhoeidtime( i1   ,i2,i3)= &
         drhoeidtime(i1   ,i2,i3) - wdivFrad * deinewa(i2,i3)
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i2=m2,n2
          deioldsum=     (deiold00(i2,i3) + deiold01(i2,i3) + deiold10(i2,i3) + deiold11(i2,i3)) + &
                    2.0* ( dSold00(i2,i3) +  dSold01(i2,i3) +  dSold10(i2,i3) +  dSold11(i2,i3))
          drhov1dtime( i1old,i2,i3)= &
           drhov1dtime(i1old,i2,i3) + wdrhov1 * deioldsum
          drhov2dtime( i1old,i2,i3)= &
           drhov2dtime(i1old,i2,i3) + wdrhov2 * deioldsum
          drhov3dtime( i1old,i2,i3)= &
           drhov3dtime(i1old,i2,i3) + wdrhov3 * deioldsum
          deinewsum=      deinewa( i2,i3) + &
                    2.0* ( dSnew00(i2,i3) +  dSnew01(i2,i3) +  dSnew10(i2,i3) +  dSnew11(i2,i3))
          drhov1dtime( i1   ,i2,i3)= &
           drhov1dtime(i1   ,i2,i3) + wdrhov1 * deinewsum
          drhov2dtime( i1   ,i2,i3)= &
           drhov2dtime(i1   ,i2,i3) + wdrhov2 * deinewsum
          drhov3dtime( i1   ,i2,i3)= &
           drhov3dtime(i1   ,i2,i3) + wdrhov3 * deinewsum
        end do ! i2
      end do ! i3
      !$OMP END DO NOWAIT
    endif
!#
#   endif
!#
    !
    if (ComputeFlux_flag) then
      if (di2 == -1) then
        if (di3 == -1) then
          ! --- .. ---
          ! --- x. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1+1,i2  ,i3  )= &
               Frad1(i1+1,i2  ,i3  )-wFrad * (w00*I(i1+1,i2-1,i3-1) + deiold00(i2-1,i3-1) + &
                                              w01*I(i1+1,i2-1,i3  ) + deiold01(i2-1,i3  ) + &
                                              w10*I(i1+1,i2  ,i3-1) + deiold10(i2  ,i3-1) + &
                                              w11*I(i1+1,i2  ,i3  ) + deiold11(i2  ,i3  ))
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1+1,i2  ,i3  )= &
               Frad2(i1+1,i2  ,i3  )+wFrad * (0.5*(w00*I(i1+1,i2-1,i3  ) + deiold00(i2-1,i3  ))+ &
                                              0.5*(w00*I(i1+1,i2-1,i3-1) + deiold00(i2-1,i3-1))+ &
                                                  (w01*I(i1+1,i2-1,i3  ) + deiold01(i2-1,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1+1,i2  ,i3  )= &
               Frad3(i1+1,i2  ,i3  )+wFrad * (0.5*(w00*I(i1+1,i2  ,i3-1) + deiold00(i2  ,i3-1))+ &
                                              0.5*(w00*I(i1+1,i2-1,i3-1) + deiold00(i2-1,i3-1))+ &
                                                  (w10*I(i1+1,i2  ,i3-1) + deiold10(i2  ,i3-1)) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        else
          ! --- x. ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1+1,i2  ,i3  )= &
               Frad1(i1+1,i2  ,i3  )-wFrad * (w00*I(i1+1,i2-1,i3  ) + deiold00(i2-1,i3  ) + &
                                              w01*I(i1+1,i2-1,i3+1) + deiold01(i2-1,i3+1) + &
                                              w10*I(i1+1,i2  ,i3  ) + deiold10(i2  ,i3  ) + &
                                              w11*I(i1+1,i2  ,i3+1) + deiold11(i2  ,i3+1)  )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1+1,i2  ,i3  )= &
               Frad2(i1+1,i2  ,i3  )+wFrad * (0.5*(w01*I(i1+1,i2-1,i3  ) + deiold01(i2-1,i3  ))+ &
                                              0.5*(w01*I(i1+1,i2-1,i3+1) + deiold01(i2-1,i3+1))+ &
                                                  (w00*I(i1+1,i2-1,i3  ) + deiold00(i2-1,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1+1,i2  ,i3  )= &
               Frad3(i1+1,i2  ,i3  )-wFrad * (0.5*(w01*I(i1+1,i2  ,i3  ) + deiold01(i2  ,i3  ))+ &
                                              0.5*(w01*I(i1+1,i2-1,i3  ) + deiold01(i2-1,i3  ))+ &
                                                  (w11*I(i1+1,i2  ,i3  ) + deiold11(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        endif
      else
        if (di3 == -1) then
          ! --- .. ---
          ! --- .x ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1+1,i2  ,i3  )= &
               Frad1(i1+1,i2  ,i3  )-wFrad * (w00*I(i1+1,i2  ,i3-1) + deiold00(i2  ,i3-1) + &
                                              w01*I(i1+1,i2  ,i3  ) + deiold01(i2  ,i3  ) + &
                                              w10*I(i1+1,i2+1,i3-1) + deiold10(i2+1,i3-1) + &
                                              w11*I(i1+1,i2+1,i3  ) + deiold11(i2+1,i3  )  )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1+1,i2  ,i3  )= &
               Frad2(i1+1,i2  ,i3  )-wFrad * (0.5*(w10*I(i1+1,i2  ,i3  ) + deiold10(i2  ,i3  ))+ &
                                              0.5*(w10*I(i1+1,i2  ,i3-1) + deiold10(i2  ,i3-1))+ &
                                                  (w11*I(i1+1,i2  ,i3  ) + deiold11(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1+1,i2  ,i3  )= &
               Frad3(i1+1,i2  ,i3  )+wFrad * (0.5*(w10*I(i1+1,i2  ,i3-1) + deiold10(i2  ,i3-1))+ &
                                              0.5*(w10*I(i1+1,i2+1,i3-1) + deiold10(i2+1,i3-1))+ &
                                                  (w00*I(i1+1,i2  ,i3-1) + deiold00(i2  ,i3-1)) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        else
          ! --- .x ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1+1,i2  ,i3  )= &
               Frad1(i1+1,i2  ,i3  )-wFrad * (w00*I(i1+1,i2  ,i3  ) + deiold00(i2  ,i3  ) + &
                                              w01*I(i1+1,i2  ,i3+1) + deiold01(i2  ,i3+1) + &
                                              w10*I(i1+1,i2+1,i3  ) + deiold10(i2+1,i3  ) + &
                                              w11*I(i1+1,i2+1,i3+1) + deiold11(i2+1,i3+1)  )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1+1,i2  ,i3  )= &
               Frad2(i1+1,i2  ,i3  )-wFrad * (0.5*(w11*I(i1+1,i2  ,i3  ) + deiold11(i2  ,i3  ))+ &
                                              0.5*(w11*I(i1+1,i2  ,i3+1) + deiold11(i2  ,i3+1))+ &
                                                  (w10*I(i1+1,i2  ,i3  ) + deiold10(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1+1,i2  ,i3  )= &
               Frad3(i1+1,i2  ,i3  )-wFrad * (0.5*(w11*I(i1+1,i2  ,i3  ) + deiold11(i2  ,i3  ))+ &
                                              0.5*(w11*I(i1+1,i2+1,i3  ) + deiold11(i2+1,i3  ))+ &
                                                  (w01*I(i1+1,i2  ,i3  ) + deiold01(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i1
!
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
endif ! (ComputeIntDown_flag)
!
! --- Adjust indices ---
di1=  -di1
di2=-1-di2
di3=-1-di3
!
! --- Swap weights ---
wt=w11 ; w11=w00 ; w00=wt
wt=w01 ; w01=w10 ; w10=wt
!
if (ComputeIntUp_flag) then
  ! === Integration i1: upward =====================================================================
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i1=m1, n1+1
!#
# if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i2, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum)
!#
# endif
!#      /* rhd_shortrad_dir_l02 ==0 */
    !
    i1old=i1-1
    !
    ! === Set boundary intensity ===
    !
    if (i1 == m1) then
      ! --- Set intensity at m1 boundary ---
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3, n3
        do i2=m2, n2
          if (dtauhalf(i1old, i2, i3) > dtauthick_limit) then
            I(         i1old, i2, i3)=-S(i1old, i2, i3)
          else
            I(         i1old, i2, i3)=0.0
          endif
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif
    !
    ! === Perform radiation-transport step ===
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1-di3, n3-di3
      if (i3 == (m3-1-di3)) then
        ! --- Set intensity at m3 boundary ---
        where (dtauhalf(i1old, :   , m3-1) > dtauthick_limit)
                      I(i1old, :   , m3-1)=-S(i1old, :   , m3-1)
        elsewhere
                      I(i1old, :   , m3-1)=0.0
        endwhere
      endif
      if (i3 == (n3-di3)) then
        ! --- Set intensity at n3 boundary ---
        where (dtauhalf(i1old, :   , n3+1) > dtauthick_limit)
                      I(i1old, :   , n3+1)=-S(i1old, :   , n3+1)
        elsewhere
                      I(i1old, :   , n3+1)=0.0
        endwhere
      endif
      ! --- Set intensity at m2 boundary ---
      if (dtauhalf(i1old, m2-1, i3  ) > dtauthick_limit) then
                 I(i1old, m2-1, i3  )=-S(i1old, m2-1, i3  )
      else
                 I(i1old, m2-1, i3  )=0.0
      endif
      ! --- Set intensity at n2 boundary ---
      if (dtauhalf(i1old, n2+1, i3  ) > dtauthick_limit) then
                 I(i1old, n2+1, i3  )=-S(i1old, n2+1, i3  )
      else
                 I(i1old, n2+1, i3  )=0.0
      endif
    end do ! i3
    !$OMP END DO
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1-di3, n3-di3
      do i2=m2-1-di2, n2-di2
        Inew_total=0.0
        !
        i2old=i2+di2
        i3old=i3+di3
        w    =w00
!#
#     if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#     else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#     if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#     elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#     elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#     elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#     elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#     elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#     elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#     elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#     endif /* rhd_shortrad_operator_l01 */
!#
#     endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i2old,i3old)=             w*deiold
        deinew00(i2   ,i3   )=             w*deinew
        Inew00(  i2   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
!      dSold00(i2old,i3old) =w*thickmaskold*(Sreduce-Sold   )
!      dSnew00(i2   ,i3   ) =w*thickmasknew*(Snew   -Sreduce)
        !
        i2old=i2+di2+1
        i3old=i3+di3
        w    =w10
!#
#     if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#     else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#     if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#     elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#     elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#     elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#     elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#     elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#     elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#     elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#     endif /* rhd_shortrad_operator_l01 */
!#
#     endif /* rhd_shortrad_operator_l02 */
!#
        deiold10(i2old,i3old)=             w*deiold
        deinew10(i2   ,i3   )=             w*deinew
        Inew10(  i2   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
!      dSold10(i2old,i3old) =w*thickmaskold*(Sreduce-Sold   )
!      dSnew10(i2   ,i3   ) =w*thickmasknew*(Snew   -Sreduce)
        !
        i2old=i2+di2
        i3old=i3+di3+1
        w    =w01
!#
#     if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#     else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#     if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#     elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#     elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#     elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#     elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#     elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#     elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#     elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#     endif /* rhd_shortrad_operator_l01 */
!#
#     endif /* rhd_shortrad_operator_l02 */
!#
        deiold01(i2old,i3old)=             w*deiold
        deinew01(i2   ,i3   )=             w*deinew
        Inew01(  i2   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
!      dSold01(i2old,i3old) =w*thickmaskold*(Sreduce-Sold   )
!      dSnew01(i2   ,i3   ) =w*thickmasknew*(Snew   -Sreduce)
        !
        i2old=i2+di2+1
        i3old=i3+di3+1
        w    =w11
!#
#     if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#     else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#     if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#     elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#     elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#     elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#     elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#     elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#     elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#     elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#     endif /* rhd_shortrad_operator_l01 */
!#
#     endif /* rhd_shortrad_operator_l02 */
!#
        deiold11(i2old,i3old)=             w*deiold
        deinew11(      i2,i3)=             w*deinew
        Inew11(        i2,i3)=               Inew
        Inew_total           =Inew_total + w*Inew
!      dSold11(i2old,i3old) =w*thickmaskold*(Sreduce-Sold   )
!      dSnew11(i2   ,i3   ) =w*thickmasknew*(Snew   -Sreduce)
        !
        I(       i1   ,i2,i3)=Inew_total
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        drhoeidtime( i1old,i2,i3)= &
         drhoeidtime(i1old,i2,i3) - wdivFrad * &
                              (deiold00(i2,i3) + deiold01(i2,i3) + deiold10(i2,i3) + deiold11(i2,i3))
        drhoeidtime( i1   ,i2,i3)= &
         drhoeidtime(i1   ,i2,i3) - wdivFrad * &
                              (deinew00(i2,i3) + deinew01(i2,i3) + deinew10(i2,i3) + deinew11(i2,i3))
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i2=m2,n2
          deioldsum=deiold00(i2,i3) + deiold01(i2,i3) + deiold10(i2,i3) + deiold11(i2,i3)
          drhov1dtime( i1old,i2,i3)= &
           drhov1dtime(i1old,i2,i3) - wdrhov1 * deioldsum
          drhov2dtime( i1old,i2,i3)= &
           drhov2dtime(i1old,i2,i3) - wdrhov2 * deioldsum
          drhov3dtime( i1old,i2,i3)= &
           drhov3dtime(i1old,i2,i3) - wdrhov3 * deioldsum
          deinewsum=deinew00(i2,i3) + deinew01(i2,i3) + deinew10(i2,i3) + deinew11(i2,i3)
          drhov1dtime( i1   ,i2,i3)= &
           drhov1dtime(i1   ,i2,i3) - wdrhov1 * deinewsum
          drhov2dtime( i1   ,i2,i3)= &
           drhov2dtime(i1   ,i2,i3) - wdrhov2 * deinewsum
          drhov3dtime( i1   ,i2,i3)= &
           drhov3dtime(i1   ,i2,i3) - wdrhov3 * deinewsum
        end do ! i2
      end do ! i3
      !$OMP END DO NOWAIT
    endif
!#
# endif
!#
    !
    if (ComputeFlux_flag) then
      if (di2 == -1) then
        if (di3 == -1) then
          ! --- .. ---
          ! --- x. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i2+1,i3+1) - deinew00(i2+1,i3+1) + &
                                              w01*Inew01(i2+1,i3  ) - deinew01(i2+1,i3  ) + &
                                              w10*Inew10(i2  ,i3+1) - deinew10(i2  ,i3+1) + &
                                              w11*Inew11(i2  ,i3  ) - deinew11(i2  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (0.5*(w00*Inew00(i2  ,i3+1) - deinew00(i2  ,i3+1))+ &
                                              0.5*(w00*Inew00(i2  ,i3  ) - deinew00(i2  ,i3  ))+ &
                                                  (w01*Inew01(i2  ,i3  ) - deinew01(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (0.5*(w00*Inew00(i2+1,i3  ) - deinew00(i2+1,i3  ))+ &
                                              0.5*(w00*Inew00(i2  ,i3  ) - deinew00(i2  ,i3  ))+ &
                                                  (w10*Inew10(i2  ,i3  ) - deinew10(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        else
          ! --- x. ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i2+1,i3  ) - deinew00(i2+1,i3  ) + &
                                              w01*Inew01(i2+1,i3-1) - deinew01(i2+1,i3-1) + &
                                              w10*Inew10(i2  ,i3  ) - deinew10(i2  ,i3  ) + &
                                              w11*Inew11(i2  ,i3-1) - deinew11(i2  ,i3-1)  )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (0.5*(w01*Inew01(i2  ,i3-1) - deinew01(i2  ,i3-1))+ &
                                              0.5*(w01*Inew01(i2  ,i3  ) - deinew01(i2  ,i3  ))+ &
                                                  (w00*Inew00(i2  ,i3  ) - deinew00(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )-wFrad * (0.5*(w01*Inew01(i2+1,i3-1) - deinew01(i2+1,i3-1))+ &
                                              0.5*(w01*Inew01(i2  ,i3-1) - deinew01(i2  ,i3-1))+ &
                                                  (w11*Inew11(i2  ,i3-1) - deinew11(i2  ,i3-1)) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        endif
      else
        if (di3 == -1) then
          ! --- .. ---
          ! --- .x ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i2  ,i3+1) - deinew00(i2  ,i3+1) + &
                                              w01*Inew01(i2  ,i3  ) - deinew01(i2  ,i3  ) + &
                                              w10*Inew10(i2-1,i3+1) - deinew10(i2-1,i3+1) + &
                                              w11*Inew11(i2-1,i3  ) - deinew11(i2-1,i3  )  )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )-wFrad * (0.5*(w10*Inew10(i2-1,i3+1) - deinew10(i2-1,i3+1))+ &
                                              0.5*(w10*Inew10(i2-1,i3  ) - deinew10(i2-1,i3  ))+ &
                                                  (w11*Inew11(i2-1,i3  ) - deinew11(i2-1,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (0.5*(w10*Inew10(i2-1,i3  ) - deinew10(i2-1,i3  ))+ &
                                              0.5*(w10*Inew10(i2  ,i3  ) - deinew10(i2  ,i3  ))+ &
                                                  (w00*Inew00(i2  ,i3  ) - deinew00(i2  ,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        else
          ! --- .x ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i2  ,i3  ) - deinew00(i2  ,i3  ) + &
                                              w01*Inew01(i2  ,i3-1) - deinew01(i2  ,i3-1) + &
                                              w10*Inew10(i2-1,i3  ) - deinew10(i2-1,i3  ) + &
                                              w11*Inew11(i2-1,i3-1) - deinew11(i2-1,i3-1)  )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i2=m2,n2+1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )-wFrad * (0.5*(w11*Inew11(i2-1,i3-1) - deinew11(i2-1,i3-1))+ &
                                              0.5*(w11*Inew11(i2-1,i3  ) - deinew11(i2-1,i3  ))+ &
                                                  (w10*Inew10(i2-1,i3  ) - deinew10(i2-1,i3  )) )
            end do ! i2
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i2=m2,n2
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )-wFrad * (0.5*(w11*Inew11(i2-1,i3-1) - deinew11(i2-1,i3-1))+ &
                                              0.5*(w11*Inew11(i2  ,i3-1) - deinew11(i2  ,i3-1))+ &
                                                  (w01*Inew01(i2  ,i3-1) - deinew01(i2  ,i3-1)) )
            end do ! i2
          end do ! i3
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
      !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    end do ! i1
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
    !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
endif ! (ComputeIntUp_flag)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(I)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
endif ! (r_dir()/r_dir_(1))
!
!#
# else
!#     /* rhd_shortrad_dir1_l01 == 1 or 2, swapped arrays */

!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_Dir1 ('rhd_short-characteristics_radiation_Direction_1')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field with main direction 1.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_Dir1(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
!                          ComputeFlux_flag, ComputePressure_flag, &
!                          r_dir, r_wei, n_dir, &
!                          m1,n1, m2,n2, m3,n3, &
!                          xb1, xb2, xb3, &
!                          S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                          Frad1, Frad2, Frad3, drhoeidtime, &
!                          drhov1dtime, drhov2dtime, drhov3dtime, &
!                          RequestSendRecvMPI, I1Send, I1Recv &
!                          )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   RequestSendRecvMPI: ('Request_Send_Receive_MPI') integer vector with MPI requests to transfer
!                 boundary data
!   I1Send:       ('Intensity_1_Send') 2D real array with boundary intensity in MPI buffer
!   I1Recv:       ('Intensity_1_Receive') 2D real array with boundary intensity in MPI buffer
!   I1Send_swap:  ('Intensity_1_Send_swap') 2D real array with boundary int. in MPI buffer
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   nFb:          ('number_Flux_boundary') number of extra boundary cells in flux arrays
!
! ROUTINES:
!   rhd_shortrad_Dir3:     Compute radiative energy flux for one direction field, main direction 3
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Transpose arrays (i1,i2,i3) -> (i2,i3,i1).
!   Call rhd_shortrad_Dir3.
!   Transpose arrays back.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-29 (B.F. Uppsala) First version
!   2004-05-26 (B.F.) NOWAIT statements
!   2013-01-22 (B.F. Lyon) No 3D thickmask array anymore
!   2013-02-06 (B.F.) Insert timing statistics
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2014-03-06 (B.F.) New parameter 'action'
!   2017-07-03 (B.F.) Rename flux_flag, pressure_flag -> ComputeFlux_flag, ComputePressure_flag;
!                     add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-06 (B.F.) Add parameters for MPI transfer of boundary intensity
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_prop_module
!#
# ifdef rhd_shortrad_formal_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+2),intent(in)   :: xb1
real, dimension(m2-1:n2+2),intent(in)   :: xb2
real, dimension(m3-1:n3+2),intent(in)   :: xb3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1 ), &
                           intent(inout):: Frad3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
integer, dimension(2),     intent(inout):: RequestSendRecvMPI
real, dimension(m2-1:n2+1, m3-1:n3+1), &
                           intent(inout):: I1Send, I1Recv
!real, dimension(m2-1:n2+1, m3-1:n3+1), &
!                           intent(inout):: I1Send_swap
!
! --- Local variables ---
real, dimension(m2-1  :n2+1, &
                m3-1  :n3+1, &
                m1-1  :n1+1 )           :: S_swap, dtauhalf_swap, exp_mdtauhalf_swap, &
                                           expl2t_mdtauhalf_swap, drhoeidtime_swap
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m2-1  :n2+1, &
                m3-1  :n3+1, &
                m1-1  :n1+1 )           :: drhov1dtime_swap, drhov2dtime_swap, drhov3dtime_swap
!#
# endif
!#
real, dimension(m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb  , &
                m1-nFb:n1+nFb   )       :: Frad1_swap
real, dimension(m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1, &
                m1-nFb:n1+nFb   )       :: Frad2_swap
real, dimension(m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb  , &
                m1-nFb:n1+nFb+1 )       :: Frad3_swap
integer                                 :: i1,i2,i3
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_start('SHC: formal: dir1: transpose>')
!#
# endif
!#
!
!#
# if (rhd_shortrad_dir1_l01 == 1)
!#                                /* parallel transpose */
!$OMP PARALLEL DEFAULT(SHARED)
!
! --- Transpose arrays ---
!$OMP DO
do i3=m3-1, n3+1
  S_swap(               :,i3,:)=transpose(S(               :,:,i3))
end do ! i3
!$OMP END DO NOWAIT
!$OMP DO
do i3=m3-1, n3+1
  dtauhalf_swap(        :,i3,:)=transpose(dtauhalf(        :,:,i3))
end do ! i3
!$OMP END DO NOWAIT
!$OMP DO
do i3=m3-1, n3+1
  exp_mdtauhalf_swap(   :,i3,:)=transpose(exp_mdtauhalf(   :,:,i3))
end do ! i3
!$OMP END DO NOWAIT
!$OMP DO
do i3=m3-1, n3+1
  expl2t_mdtauhalf_swap(:,i3,:)=transpose(expl2t_mdtauhalf(:,:,i3))
end do ! i3
!$OMP END DO NOWAIT
!
!$OMP DO
do i3=m3-1, n3+1
  drhoeidtime_swap(     :,i3,:)=transpose(drhoeidtime(     :,:,i3))
end do ! i3
!$OMP END DO NOWAIT
!
if (ComputeFlux_flag) then
  !$OMP DO
  do i3=m3-nFb, n3+nFb
    Frad3_swap(         :,i3,:)=transpose(Frad1(           :,:,i3))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-nFb, n3+nFb
    Frad1_swap(         :,i3,:)=transpose(Frad2(           :,:,i3))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-nFb, n3+nFb+1
    Frad2_swap(         :,i3,:)=transpose(Frad3(           :,:,i3))
  end do ! i3
  !$OMP END DO
endif
!
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
if (ComputePressure_flag) then
  !$OMP DO
  do i3=m3-1, n3+1
    drhov3dtime_swap(     :,i3,:)=transpose(drhov1dtime(     :,:,i3))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-1, n3+1
    drhov1dtime_swap(     :,i3,:)=transpose(drhov2dtime(     :,:,i3))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-1, n3+1
    drhov2dtime_swap(     :,i3,:)=transpose(drhov3dtime(     :,:,i3))
  end do ! i3
  !$OMP END DO NOWAIT
endif
!#
# endif
!#
!
!$OMP END PARALLEL
!
!#
# else
!#     /* rhd_shortrad_dir1_l01 == 2, serial transpose */
do i3=m3-1, n3+1
  S_swap(               :,i3,:)=transpose(S(               :,:,i3))
end do ! i3
do i3=m3-1, n3+1
  dtauhalf_swap(        :,i3,:)=transpose(dtauhalf(        :,:,i3))
end do ! i3
do i3=m3-1, n3+1
  exp_mdtauhalf_swap(   :,i3,:)=transpose(exp_mdtauhalf(   :,:,i3))
end do ! i3
do i3=m3-1, n3+1
  expl2t_mdtauhalf_swap(:,i3,:)=transpose(expl2t_mdtauhalf(:,:,i3))
end do ! i3
!
do i3=m3-1, n3+1
  drhoeidtime_swap(     :,i3,:)=transpose(drhoeidtime(     :,:,i3))
end do ! i3
!
if (ComputeFlux_flag) then
  do i3=m3-nFb, n3+nFb
    Frad3_swap(         :,i3,:)=transpose(Frad1(           :,:,i3))
  end do ! i3
  do i3=m3-nFb, n3+nFb
    Frad1_swap(         :,i3,:)=transpose(Frad2(           :,:,i3))
  end do ! i3
  do i3=m3-nFb, n3+nFb+1
    Frad2_swap(         :,i3,:)=transpose(Frad3(           :,:,i3))
  end do ! i3
endif
!
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
if (ComputePressure_flag) then
  do i3=m3-1, n3+1
    drhov3dtime_swap(     :,i3,:)=transpose(drhov1dtime(     :,:,i3))
  end do ! i3
  do i3=m3-1, n3+1
    drhov1dtime_swap(     :,i3,:)=transpose(drhov2dtime(     :,:,i3))
  end do ! i3
  do i3=m3-1, n3+1
    drhov2dtime_swap(     :,i3,:)=transpose(drhov3dtime(     :,:,i3))
  end do ! i3
endif
!#
# endif
!#
!
!#
# endif
!#      /* rhd_shortrad_dir1_l01 */
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_stop('SHC: formal: dir1: transpose>')
!#
# endif
!#
!
! --- Compute radiation transport for swapped arrays ---
call rhd_shortrad_Dir3(geo, rhd_action_New(action, dim=2, dimV=((/2, 3, 1/))), &
                       ComputeIntDown_flag, ComputeIntUp_flag, &
                       ComputeFlux_flag, ComputePressure_flag, &
                       (/r_dir(2), r_dir(3), r_dir(1)/), r_wei, n_dir, &
                       m2,n2, m3,n3, m1,n1, &
                       xb2, xb3, xb1, &
                       S_swap, dtauhalf_swap, exp_mdtauhalf_swap, expl2t_mdtauhalf_swap, &
                       Frad1_swap, Frad2_swap, Frad3_swap, drhoeidtime_swap, &
!#
#                      if (rhd_shortrad_radpressure_l01 != 0)
!#
                       drhov1dtime_swap, drhov2dtime_swap, drhov3dtime_swap, &
!#
#                      endif
!#
                       RequestSendRecvMPI, I1Send, I1Recv &
                       )
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_start('SHC: formal: dir1: transpose<')
!#
# endif
!#
!
!#
# if (rhd_shortrad_dir1_l01 == 1)
!#                                /* parallel transpose */
!
!$OMP PARALLEL DEFAULT(SHARED)
!
! --- Transpose arrays back ---
!$OMP DO
do i3=m3-1, n3+1
  drhoeidtime(     :,:,i3)=transpose(drhoeidtime_swap(     :,i3,:))
end do ! i3
!$OMP END DO NOWAIT
!
if (ComputeFlux_flag) then
  !$OMP DO
  do i3=m3-nFb, n3+nFb
    Frad1(         :,:,i3)=transpose(Frad3_swap(           :,i3,:))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-nFb, n3+nFb
    Frad2(         :,:,i3)=transpose(Frad1_swap(           :,i3,:))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-nFb, n3+nFb+1
    Frad3(         :,:,i3)=transpose(Frad2_swap(           :,i3,:))
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
if (ComputePressure_flag) then
  !$OMP DO
  do i3=m3-1, n3+1
    drhov1dtime(     :,:,i3)=transpose(drhov3dtime_swap(     :,i3,:))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-1, n3+1
    drhov2dtime(     :,:,i3)=transpose(drhov1dtime_swap(     :,i3,:))
  end do ! i3
  !$OMP END DO NOWAIT
  !$OMP DO
  do i3=m3-1, n3+1
    drhov3dtime(     :,:,i3)=transpose(drhov2dtime_swap(     :,i3,:))
  end do ! i3
  !$OMP END DO NOWAIT
endif
!#
# endif
!#
!
!$OMP END PARALLEL
!
!#
# else
!#     /* rhd_shortrad_dir1_l01 == 2, serial transpose */
!
do i3=m3-1, n3+1
  drhoeidtime(     :,:,i3)=transpose(drhoeidtime_swap(     :,i3,:))
end do ! i3
!
if (ComputeFlux_flag) then
  do i3=m3-nFb, n3+nFb
    Frad1(         :,:,i3)=transpose(Frad3_swap(           :,i3,:))
  end do ! i3
  do i3=m3-nFb, n3+nFb
    Frad2(         :,:,i3)=transpose(Frad1_swap(           :,i3,:))
  end do ! i3
  do i3=m3-nFb, n3+nFb+1
    Frad3(         :,:,i3)=transpose(Frad2_swap(           :,i3,:))
  end do ! i3
endif
!
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
if (ComputePressure_flag) then
  do i3=m3-1, n3+1
    drhov1dtime(     :,:,i3)=transpose(drhov3dtime_swap(     :,i3,:))
  end do ! i3
  do i3=m3-1, n3+1
    drhov2dtime(     :,:,i3)=transpose(drhov1dtime_swap(     :,i3,:))
  end do ! i3
  do i3=m3-1, n3+1
    drhov3dtime(     :,:,i3)=transpose(drhov2dtime_swap(     :,i3,:))
  end do ! i3
endif
!#
# endif
!#
!
!#
# endif
!#      /* rhd_shortrad_dir1_l01 */
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_stop('SHC: formal: dir1: transpose<')
!#
# endif
!#
!
!#
# endif
!#      /* rhd_shortrad_dir1_l01 */

end subroutine rhd_shortrad_Dir1


!----------*****************------------------------------------------------------------------------
subroutine rhd_shortrad_Dir2(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                             ComputeFlux_flag, ComputePressure_flag, &
                             r_dir, r_wei, n_dir, &
                             m1,n1, m2,n2, m3,n3, &
                             xb1, xb2, xb3, &
                             S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                             Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                            if (rhd_shortrad_radpressure_l01 != 0)
!#
                             drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                            endif
!#
                             RequestSendRecvMPI, I2Send, I2Recv &
                             )
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_Dir2 ('rhd_short-characteristics_radiation_Direction_2')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field with main direction 2.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_Dir2(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
!                          ComputeFlux_flag, ComputePressure_flag, &
!                          r_dir, r_wei, n_dir, &
!                          m1,n1, m2,n2, m3,n3, &
!                          xb1, xb2, xb3, &
!                          S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                          Frad1, Frad2, Frad3, drhoeidtime, &
!                          drhov1dtime, drhov2dtime, drhov3dtime, &
!                          RequestSendRecvMPI, I2Send, I2Recv &
!                          )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   RequestSendRecvMPI: ('Request_Send_Receive_MPI') integer vector with MPI requests to transfer
!                 boundary data
!   I2Send:       ('Intensity_2_Send') 2D real array with boundary intensity in MPI buffer
!   I2Recv:       ('Intensity_2_Receive') 2D real array with boundary intensity in MPI buffer
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   see rhd_shortrad_Dir1
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Wait:              Wait for given communication to complete
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_shortrad_DirSimple2: Compute radiative energy flux, simple direction field, main direction 2
!   rhd_shortrad_MPIDir2Int1: Transfer x1 boundary intensity between processes for x2 integration
!   rhd_shortrad_MPIDir2Int3: Transfer x3 boundary intensity between processes for x2 integration
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   rhd_shortrad_RayWeight: Compute weights for incoming rays
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Via MPI.
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_operator is called frequently and should be inlined.
!
! PROCEDURE:
!   Solve the radiative transfer equation along one direction (both orientations):
!   Call rhd_shortrad_operator for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F. Uppsala) First version
!   2001-11-22 (B.F.) Working version that is vectorizable and parallelizable
!   2001-11-23 (B.F.) dir3 -> dir2
!   2002-07-23 (B.F.) Formula for wFrad and wXX changed
!   2002-07-24 (B.F.) Error in index range for Frad2-computation removed: do i3=m2,n2 -> do i3=m3,n3
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2014-02-20 (B.F.) Handle boundaries within integration loops
!   2014-02-28 (B.F.) Handle all boundaries within integration loops
!   2014-03-06 (B.F.) New parameter 'action'
!   2014-03-14 (B.F.) New version of boundary-condition handling
!   2017-07-03 (B.F.) Rename flux_flag, pressure_flag -> ComputeFlux_flag, ComputePressure_flag;
!                     add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-27 (B.F.) Add MPI commands
!   2017-10-03 (B.F.) Put lateral MPI boundary intensity exchange into separate routines
!   2017-10-04 (B.F.) Outcomment routine calls for MPI boundary intensity exchange
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use const_module, only: fourpi, c_light
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
!!$use uio_bulk_module
!!$use uio_filedef_module
use timing_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+2),intent(in)   :: xb1
real, dimension(m2-1:n2+2),intent(in)   :: xb2
real, dimension(m3-1:n3+2),intent(in)   :: xb3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb   ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1 ), &
                           intent(inout):: Frad3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#      /* (rhd_shortrad_radpressure_l01) */
integer, dimension(2),     intent(inout):: RequestSendRecvMPI
real, dimension(m1-1:n1+1, m3-1:n3+1), &
                           intent(inout):: I2Send, I2Recv
!
! --- Local variables ---
real                                    :: w00, w01, w10, w11, w, wt, &
                                           deinew_total, Inew_total, deiold, deinew, Inew, &
                                           mu, wFrad, wdivFrad, wdrhov1, wdrhov2, wdrhov3, &
                                           dx1, dx2, dx3, deioldsum, deinewsum, &
                                           RabsOut, RcosthetaOut, Dout, tauOut, dxOut, RdOut
integer                                    di1, di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3, nc, &
                                           bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3, &
                                           I1_timing, I3_timing
logical                                 :: BoundHtau_flag
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
logical                                 :: HasBound1Internal_flag, HasBound3Internal_flag
integer                                 :: ierrMPI
integer, dimension(4)                   :: RequestArrMPI
real, dimension(            m3-1:n3+1)  :: I1l, I1r
real, dimension(m1-1:n1+1            )  :: I3l, I3r
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
real, dimension(3)                      :: Rout
real, dimension(m1-1:n1+1, &
                m3-1:n3+1 )             :: deiold00, deiold01, deiold10, deiold11, &
                                           deinewa , &
                                           deinew00, deinew01, deinew10, deinew11, &
                                           Inew00, Inew01, Inew10, Inew11, &
                                           dSold00, dSold01, dSold10, dSold11, &
                                           dSnew00, dSnew01, dSnew10, dSnew11
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1 )             :: I
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: I
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: I
!#
# endif
!#      /* (rhd_arrays_l01) */
real, parameter                         :: dtauthick_limit=0.17
real                                    :: Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew, &
                                           thickmaskold, thickmasknew
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!---------------------------------------------------------------------------------------------------
!
if ( (abs(anint(r_dir(1)/r_dir(2))-(r_dir(1)/r_dir(2))) < 0.001) .and. &
     (abs(anint(r_dir(3)/r_dir(2))-(r_dir(3)/r_dir(2))) < 0.001)      ) then
  call rhd_shortrad_DirSimple2(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                               ComputeFlux_flag, ComputePressure_flag, &
                               r_dir, r_wei, n_dir, &
                               m1,n1, m2,n2, m3,n3, &
                               xb1, xb2, xb3, &
                               S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                               Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                              if (rhd_shortrad_radpressure_l01 != 0)
!#
                               drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                              endif
!#
                               RequestSendRecvMPI, I2Send, I2Recv &
                               )
else ! (r_dir()/r_dir_(2))
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(I(m1-1:n1+1, m2-1:n2+1, m3-1:n3+1))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
dx1=(geo%xb_r(1)-geo%xb_l(1))/real(geo%n(1)-geo%m(1)+1)
dx2=(geo%xb_r(2)-geo%xb_l(2))/real(geo%n(2)-geo%m(2)+1)
dx3=(geo%xb_r(3)-geo%xb_l(3))/real(geo%n(3)-geo%m(3)+1)
!
! --- Find indices of relevant cells in old plane ---
call rhd_shortrad_RayWeight(r_dir(2), r_dir(1), r_dir(3), di1, di3, w00, w01, w10, w11)
di2  =1
RdOut=1.0
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
mu      =abs(r_dir(2))
wFrad   =fourpi * 0.5 * mu / real(max(n_dir,3))
wdivFrad=wFrad / dx2
wdrhov1 =(wdivFrad / c_light) * r_dir(1)
wdrhov2 =(wdivFrad / c_light) * r_dir(2)
wdrhov3 =(wdivFrad / c_light) * r_dir(3)
!
! --- Check boundary settings ---
BoundHtau_flag=(par%C_radHtautop > 0.0)
! --- Copy boundary-condition parameters ---
bound_l1=action%bound_l1
bound_r1=action%bound_r1
bound_l2=action%bound_l2
bound_r2=action%bound_r2
bound_l3=action%bound_l3
bound_r3=action%bound_r3
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
HasBound1Internal_flag=((bound_l1 == bound_InternalNext    ) .or. &
                        (bound_l1 == bound_InternalPeriodic) .or. &
                        (bound_r1 == bound_InternalNext    ) .or. &
                        (bound_r1 == bound_InternalPeriodic)       )
HasBound3Internal_flag=((bound_l3 == bound_InternalNext    ) .or. &
                        (bound_l3 == bound_InternalPeriodic) .or. &
                        (bound_r3 == bound_InternalNext    ) .or. &
                        (bound_r3 == bound_InternalPeriodic)       )
! write(gl%nc_p, '(A,I3,A,L1,2X,L1)') &
!   '#', geo%MyProc, ' HasBound1,3Internal_flag= ', &
!   HasBound1Internal_flag, HasBound3Internal_flag
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ComputeIntDown_flag) then
  ! ### Integration i2: downward ###################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I2 down, di1=', di1, ' di3=', di3
  !
  call timing_GetHandle('SHC:   dir2: D I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir2: D I3 SeRc', I3_timing)
  !
  if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I2Recv from=', geo%Proc_r(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2122
    ! call timing_start('SHC:   dir2: D I2 Recv')
    call MPI_IRecv(I2Recv, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_r(action%dimV(2)), &
                   2122, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir2: D I2 Recv')
    call timing_start('SHC:   dir2: D I2 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: D I2 Recv Wait')
  endif ! (bound_r2)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir2: D I2 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: D I2 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i2=n2, m2-1, -1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i2old=i2+1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i2old == (n2+1)) then
      ! --- Set intensity at starting face for integration (n2 boundary) ---------------------------
      !
      if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I2Recv(i1old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_r2)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              dxOut=xb2(i2old+1)-xb2(i2old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_r2)
      !
    else ! (i2old)
      ! --- Set intensity at m3/n3 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound3Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I3_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int3(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I3l, I3r, di3, 0, bound_l3, bound_r3, i2old, &
!!$                                      2130, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di3 == -1) then
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3r=I(:, i2old, n3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3r     to=', geo%Proc_r(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2131
            call MPI_ISend(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2131, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r3)
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2131, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l3)
          !
        else if (di3 == 0) then
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3l=I(:, i2old, m3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3l     to=', geo%Proc_l(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2132
            call MPI_ISend(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2132, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l3)
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2132, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r3)
          !
        endif ! (di3)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I3_timing)
        !
      endif ! (HasBound3Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        if (i3old == (m3-1)) then
          ! --- Set intensity at m3 side boundary ---
          !
          if (bound_l3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, n3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l3)
        endif ! (i3old)
        !
        if (i3old == (n3+1)) then
          ! --- Set intensity at n3 side boundary ---
          !
          if (bound_r3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, m3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r3)
        endif ! (i3old)
      end do ! i3old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 0, bound_l1, bound_r1, i2old, &
!!$                                      2110, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2111
            call MPI_ISend(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2111, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2111, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 0) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2112
            call MPI_ISend(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2112, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2112, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1+1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i3old
      !$OMP END DO
      !
    endif ! (i2old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1-di3, n3-di3
      do i1=m1-1-di1, n1-di1
        deinew_total=0.0
        Inew_total  =0.0
        !
        i1old=i1+di1
        i3old=i3+di3
        w    =w00
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i1old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold00( i1old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew00( i1   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i1old=i1+di1+1
        i3old=i3+di3
        w    =w10
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold10(i1old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold10( i1old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew10( i1   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i1old=i1+di1
        i3old=i3+di3+1
        w    =w01
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold01(i1old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold01( i1old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew01( i1   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i1old=i1+di1+1
        i3old=i3+di3+1
        w    =w11
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold11(i1old,i3old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold11( i1old,i3old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew11( i1   ,i3   )=w*thickmasknew*(Snew   -Sreduce)
        !
        deinewa(i1,   i3)    =deinew_total
        I(      i1,i2,i3)    =Inew_total
      end do ! i1
    end do ! i3
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i2old == m2) then
      if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I2Send(i1old, i3old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
      endif ! (bound_l2)
    endif ! (i2old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i1=m1,n1
        ! --- Collect energy changes in old and new plane ---
        drhoeidtime( i1,i2old,i3)= &
         drhoeidtime(i1,i2old,i3) - wdivFrad * &
                           (deiold00(i1,i3) + deiold01(i1,i3) + deiold10(i1,i3) + deiold11(i1,i3))
        drhoeidtime( i1,i2   ,i3)= &
         drhoeidtime(i1,i2   ,i3) - wdivFrad * deinewa(i1,i3)
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
    !
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i1=m1,n1
          ! --- Collect momentum changes in old and new plane ---
          deioldsum=     (deiold00(i1,i3) + deiold01(i1,i3) + deiold10(i1,i3) + deiold11(i1,i3)) + &
                    2.0* ( dSold00(i1,i3) +  dSold01(i1,i3) +  dSold10(i1,i3) +  dSold11(i1,i3))
          drhov1dtime( i1,i2old,i3)= &
           drhov1dtime(i1,i2old,i3) + wdrhov1 * deioldsum
          drhov2dtime( i1,i2old,i3)= &
           drhov2dtime(i1,i2old,i3) + wdrhov2 * deioldsum
          drhov3dtime( i1,i2old,i3)= &
           drhov3dtime(i1,i2old,i3) + wdrhov3 * deioldsum
          deinewsum=      deinewa( i1,i3) + &
                    2.0* ( dSnew00(i1,i3) +  dSnew01(i1,i3) +  dSnew10(i1,i3) +  dSnew11(i1,i3))
          drhov1dtime( i1,i2   ,i3)= &
           drhov1dtime(i1,i2   ,i3) + wdrhov1 * deinewsum
          drhov2dtime( i1,i2   ,i3)= &
           drhov2dtime(i1,i2   ,i3) + wdrhov2 * deinewsum
          drhov3dtime( i1,i2   ,i3)= &
           drhov3dtime(i1,i2   ,i3) + wdrhov3 * deinewsum
        end do ! i1
      end do ! i3
      !$OMP END DO NOWAIT
    endif
!#
#   endif
!#        /* (rhd_shortrad_radpressure_l01) */
    !
    if (ComputeFlux_flag) then
      if (di1 == -1) then
        if (di3 == -1) then
          ! --- .. ---
          ! --- x. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  )-wFrad * (w00*I(i1-1,i2+1,i3-1) + deiold00(i1-1,i3-1) + &
                                              w01*I(i1-1,i2+1,i3  ) + deiold01(i1-1,i3  ) + &
                                              w10*I(i1  ,i2+1,i3-1) + deiold10(i1  ,i3-1) + &
                                              w11*I(i1  ,i2+1,i3  ) + deiold11(i1  ,i3  ))
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  )+wFrad * (0.5*(w00*I(i1-1,i2+1,i3  ) + deiold00(i1-1,i3  ))+ &
                                              0.5*(w00*I(i1-1,i2+1,i3-1) + deiold00(i1-1,i3-1))+ &
                                                  (w01*I(i1-1,i2+1,i3  ) + deiold01(i1-1,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  )+wFrad * (0.5*(w00*I(i1  ,i2+1,i3-1) + deiold00(i1  ,i3-1))+ &
                                              0.5*(w00*I(i1-1,i2+1,i3-1) + deiold00(i1-1,i3-1))+ &
                                                  (w10*I(i1  ,i2+1,i3-1) + deiold10(i1  ,i3-1)) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- x. ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  )-wFrad * (w00*I(i1-1,i2+1,i3  ) + deiold00(i1-1,i3  ) + &
                                              w01*I(i1-1,i2+1,i3+1) + deiold01(i1-1,i3+1) + &
                                              w10*I(i1  ,i2+1,i3  ) + deiold10(i1  ,i3  ) + &
                                              w11*I(i1  ,i2+1,i3+1) + deiold11(i1  ,i3+1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  )+wFrad * (0.5*(w01*I(i1-1,i2+1,i3  ) + deiold01(i1-1,i3  ))+ &
                                              0.5*(w01*I(i1-1,i2+1,i3+1) + deiold01(i1-1,i3+1))+ &
                                                  (w00*I(i1-1,i2+1,i3  ) + deiold00(i1-1,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  )-wFrad * (0.5*(w01*I(i1  ,i2+1,i3  ) + deiold01(i1  ,i3  ))+ &
                                              0.5*(w01*I(i1-1,i2+1,i3  ) + deiold01(i1-1,i3  ))+ &
                                                  (w11*I(i1  ,i2+1,i3  ) + deiold11(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      else
        if (di3 == -1) then
          ! --- .. ---
          ! --- .x ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  )-wFrad * (w00*I(i1  ,i2+1,i3-1) + deiold00(i1  ,i3-1) + &
                                              w01*I(i1  ,i2+1,i3  ) + deiold01(i1  ,i3  ) + &
                                              w10*I(i1+1,i2+1,i3-1) + deiold10(i1+1,i3-1) + &
                                              w11*I(i1+1,i2+1,i3  ) + deiold11(i1+1,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  )-wFrad * (0.5*(w10*I(i1  ,i2+1,i3  ) + deiold10(i1  ,i3  ))+ &
                                              0.5*(w10*I(i1  ,i2+1,i3-1) + deiold10(i1  ,i3-1))+ &
                                                  (w11*I(i1  ,i2+1,i3  ) + deiold11(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  )+wFrad * (0.5*(w10*I(i1  ,i2+1,i3-1) + deiold10(i1  ,i3-1))+ &
                                              0.5*(w10*I(i1+1,i2+1,i3-1) + deiold10(i1+1,i3-1))+ &
                                                  (w00*I(i1  ,i2+1,i3-1) + deiold00(i1  ,i3-1)) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- .x ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2+1,i3  )= &
               Frad2(i1  ,i2+1,i3  )-wFrad * (w00*I(i1  ,i2+1,i3  ) + deiold00(i1  ,i3  ) + &
                                              w01*I(i1  ,i2+1,i3+1) + deiold01(i1  ,i3+1) + &
                                              w10*I(i1+1,i2+1,i3  ) + deiold10(i1+1,i3  ) + &
                                              w11*I(i1+1,i2+1,i3+1) + deiold11(i1+1,i3+1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2+1,i3  )= &
               Frad1(i1  ,i2+1,i3  )-wFrad * (0.5*(w11*I(i1  ,i2+1,i3  ) + deiold11(i1  ,i3  ))+ &
                                              0.5*(w11*I(i1  ,i2+1,i3+1) + deiold11(i1  ,i3+1))+ &
                                                  (w10*I(i1  ,i2+1,i3  ) + deiold10(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2+1,i3  )= &
               Frad3(i1  ,i2+1,i3  )-wFrad * (0.5*(w11*I(i1  ,i2+1,i3  ) + deiold11(i1  ,i3  ))+ &
                                              0.5*(w11*I(i1+1,i2+1,i3  ) + deiold11(i1+1,i3  ))+ &
                                                  (w01*I(i1  ,i2+1,i3  ) + deiold01(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i2
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I2Send to=', geo%Proc_l(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2122
    call MPI_ISend(I2Send, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_l(action%dimV(2)), &
                   2122, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_l2)
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I2 down'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir3_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i2 down'    , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntDown_flag)
!
! --- Adjust indices ---
di1  =-1-di1
di2  =  -di2
di3  =-1-di3
RdOut=-1.0
!
! --- Swap weights ---
wt=w11 ; w11=w00 ; w00=wt
wt=w01 ; w01=w10 ; w10=wt
!
if (ComputeIntUp_flag) then
  ! ### Integration i2: upward #####################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I2 up,   di1=', di1, ' di3=', di3
  !
  call timing_GetHandle('SHC:   dir2: U I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir2: U I3 SeRc', I3_timing)
  !
  if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I2Recv from=', geo%Proc_l(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2221
    ! call timing_start('SHC:   dir2: U I2 Recv')
    call MPI_IRecv(I2Recv, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_l(action%dimV(2)), &
                   2221, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir2: U I2 Recv')
    call timing_start('SHC:   dir2: U I2 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: U I2 Recv Wait')
  endif ! (bound_l2)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir2: U I2 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir2: U I2 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i2old, i3old, Inew_total, w, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i2=m2, n2+1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, Inew_total, w, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i2old=i2-1
    !
    ! === Set boundary intensity ==================================================================
    !
    if (i2old == (m2-1)) then
      ! --- Set intensity at starting face for integration (m2 boundary) ---------------------------
      !
      if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I2Recv(i1old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_l2)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              dxOut=xb2(i2old+1)-xb2(i2old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3old=m3-1, n3+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i3old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_l2)
      !
    else ! (i2old)
      ! --- Set intensity at m3/n3 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound3Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I3_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int3(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I3l, I3r, di3, 0, bound_l3, bound_r3, i2old, &
!!$                                      2230, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di3 == -1) then
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3r=I(:, i2old, n3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3r     to=', geo%Proc_r(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2231
            call MPI_ISend(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2231, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r3)
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2231, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l3)
          !
        else if (di3 == 0) then
          if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I3l=I(:, i2old, m3)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I3l     to=', geo%Proc_l(action%dimV(3)), &
            !   ' in Dir=', action%dimV(3), &
            !   ' tag=', 2232
            call MPI_ISend(I3l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(3)), &
                           2232, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l3)
          if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I3r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(3)), &
                           2232, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r3)
          !
        endif ! (di3)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I3_timing)
        !
      endif ! (HasBound3Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        if (i3old == (m3-1)) then
          ! --- Set intensity at m3 side boundary ---
          !
          if (bound_l3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, n3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l3)
        endif ! (i3old)
        !
        if (i3old == (n3+1)) then
          ! --- Set intensity at n3 side boundary ---
          !
          if (bound_r3 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, i2old, m3)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I3r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r3)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb3(i3old+1)-xb3(i3old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r3)
        endif ! (i3old)
      end do ! i3old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir2Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 0, bound_l1, bound_r1, i2old, &
!!$                                      2210, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2211
            call MPI_ISend(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2211, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2211, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 0) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, i2old, :)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 2212
            call MPI_ISend(I1l, n3-m3+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           2212, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n3-m3+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           2212, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3old=m3-1, n3+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1+1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i3old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i3old
      !$OMP END DO
    endif ! (i2old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-1-di3, n3-di3
      do i1=m1-1-di1, n1-di1
        Inew_total  =0.0
        !
        i1old=i1+di1
        i3old=i3+di3
        w    =w00
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i1old,i3old)=             w*deiold
        deinew00(i1   ,i3   )=             w*deinew
        Inew00(  i1   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        i1old=i1+di1+1
        i3old=i3+di3
        w    =w10
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold10(i1old,i3old)=             w*deiold
        deinew10(i1   ,i3   )=             w*deinew
        Inew10(  i1   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        i1old=i1+di1
        i3old=i3+di3+1
        w    =w01
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold01(i1old,i3old)=             w*deiold
        deinew01(i1   ,i3   )=             w*deinew
        Inew01(  i1   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        i1old=i1+di1+1
        i3old=i3+di3+1
        w    =w11
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold11(i1old,i3old)=             w*deiold
        deinew11(i1   ,i3   )=             w*deinew
        Inew11(  i1   ,i3   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        I(       i1   ,i2,i3)=Inew_total
      end do ! i1
    end do ! i3
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i2old == n2) then
      if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i3old=m3-1, n3+1
          do i1old=m1-1, n1+1
            I2Send(i1old, i3old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i3old
        !$OMP END DO
      endif ! (bound_r2)
    endif ! (i2old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i1=m1,n1
        ! --- Collect energy changes in old and new plane ---
        drhoeidtime( i1,i2old,i3)= &
         drhoeidtime(i1,i2old,i3) - wdivFrad * &
                           (deiold00(i1,i3) + deiold01(i1,i3) + deiold10(i1,i3) + deiold11(i1,i3))
        drhoeidtime( i1,i2   ,i3)= &
         drhoeidtime(i1,i2   ,i3) - wdivFrad * &
                           (deinew00(i1,i3) + deinew01(i1,i3) + deinew10(i1,i3) + deinew11(i1,i3))
      end do ! i1
    end do ! i3
    !$OMP END DO NOWAIT
    !
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=m3,n3
        do i1=m1,n1
          ! --- Collect momentum changes in old and new plane ---
          deioldsum=deiold00(i1,i3) + deiold01(i1,i3) + deiold10(i1,i3) + deiold11(i1,i3)
          drhov1dtime( i1,i2old,i3)= &
           drhov1dtime(i1,i2old,i3) - wdrhov1 * deioldsum
          drhov2dtime( i1,i2old,i3)= &
           drhov2dtime(i1,i2old,i3) - wdrhov2 * deioldsum
          drhov3dtime( i1,i2old,i3)= &
           drhov3dtime(i1,i2old,i3) - wdrhov3 * deioldsum
          deinewsum=deinew00(i1,i3) + deinew01(i1,i3) + deinew10(i1,i3) + deinew11(i1,i3)
          drhov1dtime( i1,i2   ,i3)= &
           drhov1dtime(i1,i2   ,i3) - wdrhov1 * deinewsum
          drhov2dtime( i1,i2   ,i3)= &
           drhov2dtime(i1,i2   ,i3) - wdrhov2 * deinewsum
          drhov3dtime( i1,i2   ,i3)= &
           drhov3dtime(i1,i2   ,i3) - wdrhov3 * deinewsum
        end do ! i1
      end do ! i3
      !$OMP END DO NOWAIT
    endif
!#
#   endif
!#        /* (rhd_shortrad_radpressure_l01) */
    !
    if (ComputeFlux_flag) then
      if (di1 == -1) then
        if (di3 == -1) then
          ! --- .. ---
          ! --- x. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1+1,i3+1) - deinew00(i1+1,i3+1) + &
                                              w01*Inew01(i1+1,i3  ) - deinew01(i1+1,i3  ) + &
                                              w10*Inew10(i1  ,i3+1) - deinew10(i1  ,i3+1) + &
                                              w11*Inew11(i1  ,i3  ) - deinew11(i1  ,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (0.5*(w00*Inew00(i1  ,i3+1) - deinew00(i1  ,i3+1))+ &
                                              0.5*(w00*Inew00(i1  ,i3  ) - deinew00(i1  ,i3  ))+ &
                                                  (w01*Inew01(i1  ,i3  ) - deinew01(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (0.5*(w00*Inew00(i1+1,i3  ) - deinew00(i1+1,i3  ))+ &
                                              0.5*(w00*Inew00(i1  ,i3  ) - deinew00(i1  ,i3  ))+ &
                                                  (w10*Inew10(i1  ,i3  ) - deinew10(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- x. ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1+1,i3  ) - deinew00(i1+1,i3  ) + &
                                              w01*Inew01(i1+1,i3-1) - deinew01(i1+1,i3-1) + &
                                              w10*Inew10(i1  ,i3  ) - deinew10(i1  ,i3  ) + &
                                              w11*Inew11(i1  ,i3-1) - deinew11(i1  ,i3-1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (0.5*(w01*Inew01(i1  ,i3-1) - deinew01(i1  ,i3-1))+ &
                                              0.5*(w01*Inew01(i1  ,i3  ) - deinew01(i1  ,i3  ))+ &
                                                  (w00*Inew00(i1  ,i3  ) - deinew00(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )-wFrad * (0.5*(w01*Inew01(i1+1,i3-1) - deinew01(i1+1,i3-1))+ &
                                              0.5*(w01*Inew01(i1  ,i3-1) - deinew01(i1  ,i3-1))+ &
                                                  (w11*Inew11(i1  ,i3-1) - deinew11(i1  ,i3-1)) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      else
        if (di3 == -1) then
          ! --- .. ---
          ! --- .x ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1  ,i3+1) - deinew00(i1  ,i3+1) + &
                                              w01*Inew01(i1  ,i3  ) - deinew01(i1  ,i3  ) + &
                                              w10*Inew10(i1-1,i3+1) - deinew10(i1-1,i3+1) + &
                                              w11*Inew11(i1-1,i3  ) - deinew11(i1-1,i3  )  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )-wFrad * (0.5*(w10*Inew10(i1-1,i3+1) - deinew10(i1-1,i3+1))+ &
                                              0.5*(w10*Inew10(i1-1,i3  ) - deinew10(i1-1,i3  ))+ &
                                                  (w11*Inew11(i1-1,i3  ) - deinew11(i1-1,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (0.5*(w10*Inew10(i1-1,i3  ) - deinew10(i1-1,i3  ))+ &
                                              0.5*(w10*Inew10(i1  ,i3  ) - deinew10(i1  ,i3  ))+ &
                                                  (w00*Inew00(i1  ,i3  ) - deinew00(i1  ,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        else
          ! --- .x ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1  ,i3  ) - deinew00(i1  ,i3  ) + &
                                              w01*Inew01(i1  ,i3-1) - deinew01(i1  ,i3-1) + &
                                              w10*Inew10(i1-1,i3  ) - deinew10(i1-1,i3  ) + &
                                              w11*Inew11(i1-1,i3-1) - deinew11(i1-1,i3-1)  )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )-wFrad * (0.5*(w11*Inew11(i1-1,i3-1) - deinew11(i1-1,i3-1))+ &
                                              0.5*(w11*Inew11(i1-1,i3  ) - deinew11(i1-1,i3  ))+ &
                                                  (w10*Inew10(i1-1,i3  ) - deinew10(i1-1,i3  )) )
            end do ! i1
          end do ! i3
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3,n3+1
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )-wFrad * (0.5*(w11*Inew11(i1-1,i3-1) - deinew11(i1-1,i3-1))+ &
                                              0.5*(w11*Inew11(i1  ,i3-1) - deinew11(i1  ,i3-1))+ &
                                                  (w01*Inew01(i1  ,i3-1) - deinew01(i1  ,i3-1)) )
            end do ! i1
          end do ! i3
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i2
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I2Send to=', geo%Proc_r(action%dimV(2)), &
    !   ' in Dir=', action%dimV(2), &
    !   ' tag=', 2221
    call MPI_ISend(I2Send, (n1-m1+3)*(n3-m3+3), MPI_REAL, geo%Proc_r(action%dimV(2)), &
                   2221, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_r2)
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I2 up'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir3_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i2 up'      , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntUp_flag)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(I)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
endif ! (r_dir()/r_dir_(2))
!
end subroutine rhd_shortrad_Dir2


!----------*****************------------------------------------------------------------------------
subroutine rhd_shortrad_Dir3(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                             ComputeFlux_flag, ComputePressure_flag, &
                             r_dir, r_wei, n_dir, &
                             m1,n1, m2,n2, m3,n3, &
                             xb1, xb2, xb3, &
                             S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                             Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                            if (rhd_shortrad_radpressure_l01 != 0)
!#
                             drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                            endif
!#
                             RequestSendRecvMPI, I3Send, I3Recv &
                             )

!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_Dir3 ('rhd_short-characteristics_radiation_Direction_3')
!
! PURPOSE:
!   Compute radiative energy flux for one direction field with main direction 3.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_Dir3(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
!                          ComputeFlux_flag, ComputePressure_flag, &
!                          r_dir, r_wei, n_dir, &
!                          m1,n1, m2,n2, m3,n3, &
!                          xb1, xb2, xb3, &
!                          S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                          Frad1, Frad2, Frad3, drhoeidtime, &
!                          drhov1dtime, drhov2dtime, drhov3dtime, &
!                          RequestSendRecvMPI, I3Send, I3Recv &
!                          )
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   ComputeIntDown_flag: ('Compute_Intensity_Down_flag') logical, compute downward intensity
!   ComputeIntUp_flag: ('Compute_Intensity_Up_flag') logical, compute upward intensity
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   RequestSendRecvMPI: ('Request_Send_Receive_MPI') integer vector with MPI requests to transfer
!                 boundary data
!   I3Send:       ('Intensity_3_Send') 2D real array with boundary intensity in MPI buffer
!   I3Recv:       ('Intensity_3_Receive') 2D real array with boundary intensity in MPI buffer
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   see rhd_shortrad_Dir1
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Wait:              Wait for given communication to complete
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_shortrad_DirSimple3: Compute radiative energy flux, simple direction field, main direction 3
!   rhd_shortrad_MPIDir3Int1: Transfer x1 boundary intensity between processes for x3 integration
!   rhd_shortrad_MPIDir3Int2: Transfer x2 boundary intensity between processes for x3 integration
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   rhd_shortrad_RayWeight: Compute weights for incoming rays
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Via MPI.
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_operator is called frequently and should be inlined.
!
! PROCEDURE:
!   Solve the radiative transfer equation along one direction (both orientations):
!   Call rhd_shortrad_operator for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F. Uppsala) First version
!   2001-11-22 (B.F.) Working version that is vectorizable and parallelizable
!   2002-07-23 (B.F.) Formula for wFrad and wXX changed
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-11-12 (B.F.) New version
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2014-02-20 (B.F.) Handle boundaries within integration loops
!   2014-02-28 (B.F.) Handle all boundaries within integration loops
!   2014-03-02 (B.F.) Partial implementation of boundaries accounting for C_radHtautop
!   2014-03-06 (B.F.) New parameter 'action'
!   2014-03-07 (B.F.) Put some boundary-condition handling into include file
!   2014-03-13 (B.F.) New version of boundary-condition handling
!   2017-07-03 (B.F.) Rename flux_flag, pressure_flag -> ComputeFlux_flag, ComputePressure_flag;
!                     add parameters ComputeIntDown_flag, ComputeIntUp_flag
!   2017-07-05 (B.F.) Add parameters for MPI transfer of boundary intensity
!   2017-07-26 (B.F.) Add MPI commands taken from rhd_shortrad_DirSimple3
!   2017-10-01 (B.F.) Put lateral MPI boundary intensity exchange into separate routines
!   2017-10-04 (B.F.) Outcomment routine calls for MPI boundary intensity exchange
!   2017-10-27 (B.F.) Make the number of ghost cells flexible
!   2018-12-02 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use const_module, only: fourpi, c_light
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
!!$use uio_bulk_module
!!$use uio_filedef_module
use timing_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFb=1, nFg=1
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: ComputeIntDown_flag, ComputeIntUp_flag, &
                                           ComputeFlux_flag, ComputePressure_flag
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-nFg:n1+nFg+1), &
                           intent(in)   :: xb1
real, dimension(m2-nFg:n2+nFg+1), &
                           intent(in)   :: xb2
real, dimension(m3-nFg:n3+nFg+1), &
                           intent(in)   :: xb3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(in)   :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb     ), &
                           intent(inout):: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb     ), &
                           intent(inout):: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1   ), &
                           intent(inout):: Frad3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(inout):: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
integer, dimension(2),     intent(inout):: RequestSendRecvMPI
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg   ), &
                           intent(inout):: I3Send, I3Recv
!
! --- Local variables ---
real                                    :: w00, w01, w10, w11, w, wt, &
                                           deinew_total, Inew_total, deiold, deinew, Inew, &
                                           mu, wFrad, wdivFrad, wdrhov1, wdrhov2, wdrhov3, &
                                           dx1, dx2, dx3, deioldsum, deinewsum, &
                                           RabsOut, RcosthetaOut, Dout, tauOut, dxOut, RdOut
integer                                    di1, di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3, nc, &
                                           bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3, &
                                           I1_timing, I2_timing
logical                                 :: BoundHtau_flag
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
logical                                 :: HasBound1Internal_flag, HasBound2Internal_flag
integer                                 :: ierrMPI
integer, dimension(4)                   :: RequestArrMPI
real, dimension(m2-nFg:n2+nFg)          :: I1l, I1r
real, dimension(m1-nFg:n1+nFg)          :: I2l, I2r
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
integer, dimension(MPI_STATUS_SIZE, 2)  :: StatusArrMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
real, dimension(3)                      :: Rout
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg   )       :: deiold00, deiold01, deiold10, deiold11, &
                                           deinewa , &
                                           deinew00, deinew01, deinew10, deinew11, &
                                           Inew00, Inew01, Inew10, Inew11, &
                                           dSold00, dSold01, dSold10, dSold11, &
                                           dSnew00, dSnew01, dSnew10, dSnew11
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   )       :: I
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: I
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: I
!#
# endif
!#      /* (rhd_arrays_l01) */
real, parameter                         :: dtauthick_limit=0.17
real                                    :: Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, &
                                                       dtauhalfold, &
                                                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, &
                                                       dtauhalfnew
real                                    :: Scenter, Sbound, Sreduce, wS, &
                                           dIold, Icenterold, Icenternew, dInew, &
                                           thickmaskold, thickmasknew
logical                                 :: thickflagold, thickflagnew
real, parameter                         :: dtaucoolthick_limit=0.5
!---------------------------------------------------------------------------------------------------
!
if ( (abs(anint(r_dir(1)/r_dir(3))-(r_dir(1)/r_dir(3))) < 0.001) .and. &
     (abs(anint(r_dir(2)/r_dir(3))-(r_dir(2)/r_dir(3))) < 0.001)      ) then
  call rhd_shortrad_DirSimple3(geo, action, ComputeIntDown_flag, ComputeIntUp_flag, &
                               ComputeFlux_flag, ComputePressure_flag, &
                               r_dir, r_wei, n_dir, &
                               m1,n1, m2,n2, m3,n3, &
                               xb1, xb2, xb3, &
                               S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                               Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                              if (rhd_shortrad_radpressure_l01 != 0)
!#
                               drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                              endif
!#
                               RequestSendRecvMPI, I3Send, I3Recv &
                               )
else ! (r_dir()/r_dir_(3))
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(I(m1-nFg:n1+nFg, m2-nFg:n2+nFg, m3-nFg:n3+nFg))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
dx1=    (geo%xb_r(action%dimV(1))-geo%xb_l(action%dimV(1))  ) / &
    real(geo%n(   action%dimV(1))-geo%m(   action%dimV(1))+1)
dx2=    (geo%xb_r(action%dimV(2))-geo%xb_l(action%dimV(2))  ) / &
    real(geo%n(   action%dimV(2))-geo%m(   action%dimV(2))+1)
dx3=    (geo%xb_r(action%dimV(3))-geo%xb_l(action%dimV(3))   )/ &
    real(geo%n(   action%dimV(3))-geo%m(   action%dimV(3))+1)
!
! --- Find indices of relevant cells in old plane ---
call rhd_shortrad_RayWeight(r_dir(3), r_dir(1), r_dir(2), di1, di2, w00, w01, w10, w11)
di3  =1
RdOut=1.0
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
mu      =abs(r_dir(3))
wFrad   =fourpi * 0.5 * mu / real(max(n_dir,3))
wdivFrad=wFrad / dx3
wdrhov1 =(wdivFrad / c_light) * r_dir(1)
wdrhov2 =(wdivFrad / c_light) * r_dir(2)
wdrhov3 =(wdivFrad / c_light) * r_dir(3)
!
! --- Check boundary settings ---
BoundHtau_flag=(par%C_radHtautop > 0.0)
if (action%dim == 2) then
  ! --- Transposed arrays: copy boundary-condition parameters ---
  bound_l1=action%bound_l2
  bound_r1=action%bound_r2
  bound_l2=action%bound_l3
  bound_r2=action%bound_r3
  bound_l3=action%bound_l1
  bound_r3=action%bound_r1
else ! (action%dim)
  ! --- Copy boundary-condition parameters ---
  bound_l1=action%bound_l1
  bound_r1=action%bound_r1
  bound_l2=action%bound_l2
  bound_r2=action%bound_r2
  bound_l3=action%bound_l3
  bound_r3=action%bound_r3
endif ! (action%dim)
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
HasBound1Internal_flag=((bound_l1 == bound_InternalNext    ) .or. &
                        (bound_l1 == bound_InternalPeriodic) .or. &
                        (bound_r1 == bound_InternalNext    ) .or. &
                        (bound_r1 == bound_InternalPeriodic)       )
HasBound2Internal_flag=((bound_l2 == bound_InternalNext    ) .or. &
                        (bound_l2 == bound_InternalPeriodic) .or. &
                        (bound_r2 == bound_InternalNext    ) .or. &
                        (bound_r2 == bound_InternalPeriodic)       )
! write(gl%nc_p, '(A,I3,A,L1,2X,L1)') &
!   '#', geo%MyProc, ' HasBound1,2Internal_flag= ', &
!   HasBound1Internal_flag, HasBound2Internal_flag
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ComputeIntDown_flag) then
  ! ### Integration i3: downward ###################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I3 down, di1=', di1, ' di2=', di2
  !
  call timing_GetHandle('SHC:   dir3: D I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir3: D I2 SeRc', I2_timing)
  !
  if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I3Recv from=', geo%Proc_r(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3132
    ! call timing_start('SHC:   dir3: D I3 Recv')
    call MPI_IRecv(I3Recv, (n1-m1+2*nFg+1)*(n2-m2+2*nFg+1), MPI_REAL, &
                   geo%Proc_r(action%dimV(3)), &
                   3132, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir3: D I3 Recv')
    call timing_start('SHC:   dir3: D I3 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: D I3 Recv Wait')
  endif ! (bound_r3)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir3: D I3 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: D I3 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i3, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i3=n3, m3-1, -1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, deinew_total, Inew_total, w, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i3old=i3+1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i3old == (n3+1)) then
      ! --- Set intensity at starting face for integration (n3 boundary) ---------------------------
      !
      if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I3Recv(i1old, i2old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_r3)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              dxOut=xb3(i3old+1)-xb3(i3old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_r3)
      !
    else ! (i3old)
      ! --- Set intensity at m2/n2 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound2Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I2_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int2(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I2l, I2r, di2, 0, bound_l2, bound_r2, i3old, &
!!$                                      3120, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di2 == -1) then
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2r=I(:, n2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2r     to=', geo%Proc_r(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3121
            call MPI_ISend(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3121, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r2)
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3121, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l2)
          !
        else if (di2 == 0) then
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2l=I(:, m2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2l     to=', geo%Proc_l(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3122
            call MPI_ISend(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3122, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l2)
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3122, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r2)
          !
        endif ! (di2)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I2_timing)
        !
      endif ! (HasBound2Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        if (i2old == (m2-1)) then
          ! --- Set intensity at m2 side boundary ---
          !
          if (bound_l2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, n2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l2)
        endif ! (i2old)
        !
        if (i2old == (n2+1)) then
          ! --- Set intensity at n2 side boundary ---
          !
          if (bound_r2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, m2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r2)
        endif ! (i2old)
      end do ! i2old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 0, bound_l1, bound_r1, i3old, &
!!$                                      3110, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3111
            call MPI_ISend(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3111, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3111, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 0) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3112
            call MPI_ISend(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3112, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3112, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1+1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i2old
      !$OMP END DO
      !
    endif ! (i3old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i2=m2-1-di2, n2-di2
      do i1=m1-1-di1, n1-di1
        deinew_total=0.0
        Inew_total  =0.0
        !
        i1old=i1+di1
        i2old=i2+di2
        w    =w00
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i1old,i2old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold00( i1old,i2old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew00( i1   ,i2   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i1old=i1+di1+1
        i2old=i2+di2
        w    =w10
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold10(i1old,i2old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold10( i1old,i2old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew10( i1   ,i2   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i1old=i1+di1
        i2old=i2+di2+1
        w    =w01
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold01(i1old,i2old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold01( i1old,i2old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew01( i1   ,i2   )=w*thickmasknew*(Snew   -Sreduce)
        !
        i1old=i1+di1+1
        i2old=i2+di2+1
        w    =w11
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold11(i1old,i2old)=               w*deiold
        deinew_total         =deinew_total + w*deinew
        Inew_total           =Inew_total   + w*Inew
        dSold11( i1old,i2old)=w*thickmaskold*(Sreduce-Sold   )
        dSnew11( i1   ,i2   )=w*thickmasknew*(Snew   -Sreduce)
        !
        deinewa(i1,i2)       =deinew_total
        I(      i1,i2,i3)    =Inew_total
      end do ! i1
    end do ! i2
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i3old == m3) then
      if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I3Send(i1old, i2old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
      endif ! (bound_l3)
    endif ! (i3old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i2=m2,n2
      do i1=m1,n1
        ! --- Collect energy changes in old and new plane ---
        drhoeidtime( i1,i2,i3old)= &
         drhoeidtime(i1,i2,i3old) - wdivFrad * &
                           (deiold00(i1,i2) + deiold01(i1,i2) + deiold10(i1,i2) + deiold11(i1,i2))
        drhoeidtime( i1,i2,i3   )= &
         drhoeidtime(i1,i2,i3   ) - wdivFrad * deinewa(i1,i2)
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
    !
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i2=m2,n2
        do i1=m1,n1
          ! --- Collect momentum changes in old and new plane ---
          deioldsum=     (deiold00(i1,i2) + deiold01(i1,i2) + deiold10(i1,i2) + deiold11(i1,i2)) + &
                    2.0* ( dSold00(i1,i2) +  dSold01(i1,i2) +  dSold10(i1,i2) +  dSold11(i1,i2))
          drhov1dtime( i1,i2,i3old)= &
           drhov1dtime(i1,i2,i3old) + wdrhov1 * deioldsum
          drhov2dtime( i1,i2,i3old)= &
           drhov2dtime(i1,i2,i3old) + wdrhov2 * deioldsum
          drhov3dtime( i1,i2,i3old)= &
           drhov3dtime(i1,i2,i3old) + wdrhov3 * deioldsum
          deinewsum=      deinewa( i1,i2) + &
                    2.0* ( dSnew00(i1,i2) +  dSnew01(i1,i2) +  dSnew10(i1,i2) +  dSnew11(i1,i2))
          drhov1dtime( i1,i2,i3   )= &
           drhov1dtime(i1,i2,i3   ) + wdrhov1 * deinewsum
          drhov2dtime( i1,i2,i3   )= &
           drhov2dtime(i1,i2,i3   ) + wdrhov2 * deinewsum
          drhov3dtime( i1,i2,i3   )= &
           drhov3dtime(i1,i2,i3   ) + wdrhov3 * deinewsum
        end do ! i1
      end do ! i2
      !$OMP END DO NOWAIT
    endif
!#
#   endif
!#        /* (rhd_shortrad_radpressure_l01) */
    !
    if (ComputeFlux_flag) then
      if (di1 == -1) then
        if (di2 == -1) then
          ! --- .. ---
          ! --- x. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1)-wFrad * (w00*I(i1-1,i2-1,i3+1) + deiold00(i1-1,i2-1) + &
                                              w01*I(i1-1,i2  ,i3+1) + deiold01(i1-1,i2  ) + &
                                              w10*I(i1  ,i2-1,i3+1) + deiold10(i1  ,i2-1) + &
                                              w11*I(i1  ,i2  ,i3+1) + deiold11(i1  ,i2  ))
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1)+wFrad * (0.5*(w00*I(i1-1,i2  ,i3+1) + deiold00(i1-1,i2  ))+ &
                                              0.5*(w00*I(i1-1,i2-1,i3+1) + deiold00(i1-1,i2-1))+ &
                                                  (w01*I(i1-1,i2  ,i3+1) + deiold01(i1-1,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1)+wFrad * (0.5*(w00*I(i1  ,i2-1,i3+1) + deiold00(i1  ,i2-1))+ &
                                              0.5*(w00*I(i1-1,i2-1,i3+1) + deiold00(i1-1,i2-1))+ &
                                                  (w10*I(i1  ,i2-1,i3+1) + deiold10(i1  ,i2-1)) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- x. ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1)-wFrad * (w00*I(i1-1,i2  ,i3+1) + deiold00(i1-1,i2  ) + &
                                              w01*I(i1-1,i2+1,i3+1) + deiold01(i1-1,i2+1) + &
                                              w10*I(i1  ,i2  ,i3+1) + deiold10(i1  ,i2  ) + &
                                              w11*I(i1  ,i2+1,i3+1) + deiold11(i1  ,i2+1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1)+wFrad * (0.5*(w01*I(i1-1,i2  ,i3+1) + deiold01(i1-1,i2  ))+ &
                                              0.5*(w01*I(i1-1,i2+1,i3+1) + deiold01(i1-1,i2+1))+ &
                                                  (w00*I(i1-1,i2  ,i3+1) + deiold00(i1-1,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1)-wFrad * (0.5*(w01*I(i1  ,i2  ,i3+1) + deiold01(i1  ,i2  ))+ &
                                              0.5*(w01*I(i1-1,i2  ,i3+1) + deiold01(i1-1,i2  ))+ &
                                                  (w11*I(i1  ,i2  ,i3+1) + deiold11(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      else
        if (di2 == -1) then
          ! --- .. ---
          ! --- .x ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1)-wFrad * (w00*I(i1  ,i2-1,i3+1) + deiold00(i1  ,i2-1) + &
                                              w01*I(i1  ,i2  ,i3+1) + deiold01(i1  ,i2  ) + &
                                              w10*I(i1+1,i2-1,i3+1) + deiold10(i1+1,i2-1) + &
                                              w11*I(i1+1,i2  ,i3+1) + deiold11(i1+1,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1)-wFrad * (0.5*(w10*I(i1  ,i2  ,i3+1) + deiold10(i1  ,i2  ))+ &
                                              0.5*(w10*I(i1  ,i2-1,i3+1) + deiold10(i1  ,i2-1))+ &
                                                  (w11*I(i1  ,i2  ,i3+1) + deiold11(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1)+wFrad * (0.5*(w10*I(i1  ,i2-1,i3+1) + deiold10(i1  ,i2-1))+ &
                                              0.5*(w10*I(i1+1,i2-1,i3+1) + deiold10(i1+1,i2-1))+ &
                                                  (w00*I(i1  ,i2-1,i3+1) + deiold00(i1  ,i2-1)) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- .x ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3+1)= &
               Frad3(i1  ,i2  ,i3+1)-wFrad * (w00*I(i1  ,i2  ,i3+1) + deiold00(i1  ,i2  ) + &
                                              w01*I(i1  ,i2+1,i3+1) + deiold01(i1  ,i2+1) + &
                                              w10*I(i1+1,i2  ,i3+1) + deiold10(i1+1,i2  ) + &
                                              w11*I(i1+1,i2+1,i3+1) + deiold11(i1+1,i2+1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3+1)= &
               Frad1(i1  ,i2  ,i3+1)-wFrad * (0.5*(w11*I(i1  ,i2  ,i3+1) + deiold11(i1  ,i2  ))+ &
                                              0.5*(w11*I(i1  ,i2+1,i3+1) + deiold11(i1  ,i2+1))+ &
                                                  (w10*I(i1  ,i2  ,i3+1) + deiold10(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3+1)= &
               Frad2(i1  ,i2  ,i3+1)-wFrad * (0.5*(w11*I(i1  ,i2  ,i3+1) + deiold11(i1  ,i2  ))+ &
                                              0.5*(w11*I(i1+1,i2  ,i3+1) + deiold11(i1+1,i2  ))+ &
                                                  (w01*I(i1  ,i2  ,i3+1) + deiold01(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i3
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I3Send to=', geo%Proc_l(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3132
    call MPI_ISend(I3Send, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_l(action%dimV(3)), &
                   3132, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_l3)
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I3 down'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir3_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i3 down'    , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntDown_flag)
!
! --- Adjust indices ---
di1  =-1-di1
di2  =-1-di2
di3  =  -di3
RdOut=-1.0
!
! --- Swap weights ---
wt=w11 ; w11=w00 ; w00=wt
wt=w01 ; w01=w10 ; w10=wt
!
if (ComputeIntUp_flag) then
  ! ### Integration i3: upward #####################################################################
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! write(gl%nc_p, '(A,I3,A,I3,A,I3)') &
  !   '#', geo%MyProc, ' Start integration I3 up,   di1=', di1, ' di2=', di2
  !
  call timing_GetHandle('SHC:   dir3: U I1 SeRc', I1_timing)
  call timing_GetHandle('SHC:   dir3: U I2 SeRc', I2_timing)
  !
  if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
    ! --- Box has incoming internal boundary: get boundary values from neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Receive I3Recv from=', geo%Proc_l(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3231
    ! call timing_start('SHC:   dir3: U I3 Recv')
    call MPI_IRecv(I3Recv, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_l(action%dimV(3)), &
                   3231, geo%Comm, RequestSendRecvMPI(2), ierrMPI)
    ! call timing_stop( 'SHC:   dir3: U I3 Recv')
    call timing_start('SHC:   dir3: U I3 Recv Wait')
    call MPI_Wait(RequestSendRecvMPI(2), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: U I3 Recv Wait')
  endif ! (bound_l3)
  !
  if (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL) then
    call timing_start('SHC:   dir3: U I3 Send Wait')
    call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
    call timing_stop( 'SHC:   dir3: U I3 Send Wait')
  endif ! (RequestSendRecvMPI(1))
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i3, i1old, i2old, i3old, Inew_total, w, deiold, deinew, Inew, &
  !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
  !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
  !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
  !
  do i3=m3, n3+1
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(i1, i1old, i2old, i3old, Inew_total, w, deiold, deinew, Inew, &
    !$OMP           Iold, Sold, expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
    !$OMP                 Snew, expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
    !$OMP           Scenter, Sbound, Sreduce, wS, dIold, Icenterold, Icenternew, dInew, &
    !$OMP           thickmaskold, thickmasknew, thickflagold, thickflagnew, deioldsum, deinewsum, &
    !$OMP           ROut, RabsOut, RcosthetaOut, Dout, tauOut, dxOut)
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
    !
    i3old=i3-1
    !
    ! === Set boundary intensity ===================================================================
    !
    if (i3old == (m3-1)) then
      ! --- Set intensity at starting face for integration (m3 boundary) ---------------------------
      !
      if ((bound_l3 == bound_InternalNext) .or. (bound_l3 == bound_InternalPeriodic)) then
        ! --- Box has internal boundary: copy boundary values from 2D array ---
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I(i1old, i2old, i3old)=I3Recv(i1old, i2old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      else ! (bound_l3)
        if (BoundHtau_flag) then
          ! --- Account for some intensity entering the box from the outside ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              dxOut=xb3(i3old+1)-xb3(i3old)
              include 'rhd_shortrad_bound01.f90'
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        else ! (.not. BoundHtau_flag)
          ! --- Set intensity to zero ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2old=m2-1, n2+1
            do i1old=m1-1, n1+1
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            end do ! i1old
          end do ! i2old
          !$OMP END DO
        endif ! (BoundHtau_flag)
        !
      endif ! (bound_l3)
      !
    else ! (i3old)
      ! --- Set intensity at m2/n2 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound2Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I2_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int2(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I2l, I2r, di2, 0, bound_l2, bound_r2, i3old, &
!!$                                      3220, RequestArrMPI)
        !
        RequestArrMPI(3)=MPI_REQUEST_NULL
        RequestArrMPI(4)=MPI_REQUEST_NULL
        !
        if (di2 == -1) then
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2r=I(:, n2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2r     to=', geo%Proc_r(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3221
            call MPI_ISend(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3221, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_r2)
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3221, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_l2)
          !
        else if (di2 == 0) then
          if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I2l=I(:, m2, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I2l     to=', geo%Proc_l(action%dimV(2)), &
            !   ' in Dir=', action%dimV(2), &
            !   ' tag=', 3222
            call MPI_ISend(I2l, n1-m1+3, MPI_REAL, geo%Proc_l(action%dimV(2)), &
                           3222, geo%Comm, RequestArrMPI(3), ierrMPI)
          endif ! (bound_l2)
          if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I2r, n1-m1+3, MPI_REAL, geo%Proc_r(action%dimV(2)), &
                           3222, geo%Comm, RequestArrMPI(4), ierrMPI)
          endif ! (bound_r2)
          !
        endif ! (di2)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(3:4), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I2_timing)
        !
      endif ! (HasBound2Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        if (i2old == (m2-1)) then
          ! --- Set intensity at m2 side boundary ---
          !
          if (bound_l2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, n2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l2 == bound_InternalNext) .or. (bound_l2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2l(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_l2)
        endif ! (i2old)
        !
        if (i2old == (n2+1)) then
          ! --- Set intensity at n2 side boundary ---
          !
          if (bound_r2 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I(i1old, m2, i3old)
            end do ! i1old
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r2 == bound_InternalNext) .or. (bound_r2 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            do i1old=m1-1, n1+1
              I(i1old, i2old, i3old)=I2r(i1old)
            end do ! i1old
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r2)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              do i1old=m1-1, n1+1
                dxOut=xb2(i2old+1)-xb2(i2old)
                include 'rhd_shortrad_bound01.f90'
              end do ! i1old
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              do i1old=m1-1, n1+1
                if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                  I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
                else
                  I(         i1old, i2old, i3old)=0.0
                endif
              end do ! i1old
            endif ! (BoundHtau_flag)
          endif ! (bound_r2)
        endif ! (i2old)
      end do ! i2old
      !$OMP END DO
      !
      ! --- Set intensity at m1/n1 side boundaries -------------------------------------------------
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      if (HasBound1Internal_flag) then
        ! --- Issue non-blocking send requests for boundary values to neighbor processes ---
        !
        call timing_start(I1_timing)
        !
!!$        call rhd_shortrad_MPIDir3Int1(geo, action, I, m1,n1, m2,n2, m3,n3, &
!!$                                      I1l, I1r, di1, 0, bound_l1, bound_r1, i3old, &
!!$                                      3210, RequestArrMPI)
        !
        RequestArrMPI(1)=MPI_REQUEST_NULL
        RequestArrMPI(2)=MPI_REQUEST_NULL
        !
        if (di1 == -1) then
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1r=I(n1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1r     to=', geo%Proc_r(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3211
            call MPI_ISend(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3211, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_r1)
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3211, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_l1)
          !
        else if (di1 == 0) then
          if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
            I1l=I(m1, :, i3old)
            ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
            !   '#', geo%MyProc, &
            !   ' Send    I1l     to=', geo%Proc_l(action%dimV(1)), &
            !   ' in Dir=', action%dimV(1), &
            !   ' tag=', 3212
            call MPI_ISend(I1l, n2-m2+3, MPI_REAL, geo%Proc_l(action%dimV(1)), &
                           3212, geo%Comm, RequestArrMPI(1), ierrMPI)
          endif ! (bound_l1)
          if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Box has incoming internal boundary: receive boundary values from neighbor process
            call MPI_IRecv(I1r, n2-m2+3, MPI_REAL, geo%Proc_r(action%dimV(1)), &
                           3212, geo%Comm, RequestArrMPI(2), ierrMPI)
          endif ! (bound_r1)
          !
        endif ! (di1)
        !
        ! --- Wait for non-blocking send/receive requests for boundary values ---
        ! --- from neighbor processes to finish                               ---
        call MPI_WaitAll(2, RequestArrMPI(1:2), StatusArrMPI, ierrMPI)
        !
        call timing_stop(I1_timing)
        !
      endif ! (HasBound1Internal_flag)
      !$OMP END MASTER
      !
      !$OMP BARRIER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i2old=m2-1, n2+1
        i1old=m1+di1
        if (i1old == (m1-1)) then
          ! --- Set intensity at m1 side boundary ---
          !
          if (bound_l1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(n1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_l1 == bound_InternalNext) .or. (bound_l1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1l(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_l1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_l1)
        endif ! (i1old)
        !
        i1old=n1+di1+1
        if (i1old == (n1+1)) then
          ! --- Set intensity at n1 side boundary ---
          !
          if (bound_r1 == bound_periodic) then
            ! --- Handle periodic boundaries ---
            I(i1old, i2old, i3old)=I(m1, i2old, i3old)
!#
#         if (rhd_mpi_l01 == 1)
!#                              /* MPI activated ----------------- */
          else if ((bound_r1 == bound_InternalNext) .or. (bound_r1 == bound_InternalPeriodic)) then
            ! --- Copy boundary values from temporary 1D array ---
            I(i1old, i2old, i3old)=I1r(i2old)
!#
#         endif
!#              /* rhd_mpi_l01 ----------------------------------- */
          else ! (bound_r1)
            ! --- Handle any open boundary ---
            if (BoundHtau_flag) then
              ! --- Account for some intensity entering the box from the outside ---
              dxOut=xb1(i1old+1)-xb1(i1old)
              include 'rhd_shortrad_bound01.f90'
            else ! (.not. BoundHtau_flag)
              ! --- Set intensity to zero ---
              if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
                I(         i1old, i2old, i3old)=-S(i1old, i2old, i3old)
              else
                I(         i1old, i2old, i3old)=0.0
              endif
            endif ! (BoundHtau_flag)
          endif ! (bound_r1)
        endif ! (i1old)
      end do ! i2old
      !$OMP END DO
    endif ! (i3old)
    !
    ! === Perform radiation-transport step =========================================================
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i2=m2-1-di2, n2-di2
      do i1=m1-1-di1, n1-di1
        Inew_total  =0.0
        !
        i1old=i1+di1
        i2old=i2+di2
        w    =w00
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold00(i1old,i2old)=             w*deiold
        deinew00(i1   ,i2   )=             w*deinew
        Inew00(  i1   ,i2   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        i1old=i1+di1+1
        i2old=i2+di2
        w    =w10
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold10(i1old,i2old)=             w*deiold
        deinew10(i1   ,i2   )=             w*deinew
        Inew10(  i1   ,i2   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        i1old=i1+di1
        i2old=i2+di2+1
        w    =w01
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold01(i1old,i2old)=             w*deiold
        deinew01(i1   ,i2   )=             w*deinew
        Inew01(  i1   ,i2   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        i1old=i1+di1+1
        i2old=i2+di2+1
        w    =w11
!#
#       if (rhd_shortrad_operator_l02 == 0)
!#
        call rhd_shortrad_operator(dtauthick_limit, &
               I(                     i1old,i2old,i3old), &
               S(                     i1old,i2old,i3old), &
               expl2t_mdtauhalf(      i1old,i2old,i3old), &
               exp_mdtauhalf(         i1old,i2old,i3old), &
               dtauhalf(              i1old,i2old,i3old), &
               0.5+sign(0.5, dtauhalf(i1old,i2old,i3old)-dtauthick_limit), &
               S(                     i1   ,i2   ,i3   ), &
               expl2t_mdtauhalf(      i1   ,i2   ,i3   ), &
               exp_mdtauhalf(         i1   ,i2   ,i3   ), &
               dtauhalf(              i1   ,i2   ,i3   ), &
               0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
               deiold, deinew, Inew)
!#
#       else /* rhd_shortrad_operator_l02 */
!#
        Iold               =I(               i1old,i2old,i3old)
        Sold               =S(               i1old,i2old,i3old)
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3old)
        thickmaskold       =0.5+sign(0.5, dtauhalfold-dtauthick_limit)
        Snew               =S(               i1   ,i2   ,i3   )
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
        !
!#
#       if (rhd_shortrad_operator_l01 == 0)   /* test 'zero' operator */
        include 'rhd_shortrad_operator00.f90'
#       elif (rhd_shortrad_operator_l01 == 1) /* operator with 'if..then..else' */
        include 'rhd_shortrad_operator01.f90'
#       elif (rhd_shortrad_operator_l01 == 2) /* operator with masks */
        include 'rhd_shortrad_operator02.f90'
#       elif (rhd_shortrad_operator_l01 == 3) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator03.f90'
#       elif (rhd_shortrad_operator_l01 == 4) /* operator with masks */
        include 'rhd_shortrad_operator04.f90'
#       elif (rhd_shortrad_operator_l01 == 5) /* extended operator with 'if..then..else' */
        include 'rhd_shortrad_operator05.f90'
#       elif (rhd_shortrad_operator_l01 == 6) /* default operator with masks */
        include 'rhd_shortrad_operator06.f90'
#       elif (rhd_shortrad_operator_l01 == 8) /* operator with masks */
        include 'rhd_shortrad_operator08.f90'
#       endif /* rhd_shortrad_operator_l01 */
!#
#       endif /* rhd_shortrad_operator_l02 */
!#
        deiold11(i1old,i2old)=             w*deiold
        deinew11(i1   ,i2   )=             w*deinew
        Inew11(  i1   ,i2   )=               Inew
        Inew_total           =Inew_total + w*Inew
        !
        I(       i1   ,i2,i3)=Inew_total
      end do ! i1
    end do ! i2
    !$OMP END DO
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    if (i3old == n3) then
      if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
        ! --- Copy boundary intensity into persistent buffer ---
        !$OMP DO SCHEDULE(RUNTIME)
        do i2old=m2-1, n2+1
          do i1old=m1-1, n1+1
            I3Send(i1old, i2old)=I(i1old, i2old, i3old)
          end do ! i1old
        end do ! i2old
        !$OMP END DO
      endif ! (bound_r3)
    endif ! (i3old)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i2=m2,n2
      do i1=m1,n1
        ! --- Collect energy changes in old and new plane ---
        drhoeidtime( i1,i2,i3old)= &
         drhoeidtime(i1,i2,i3old) - wdivFrad * &
                           (deiold00(i1,i2) + deiold01(i1,i2) + deiold10(i1,i2) + deiold11(i1,i2))
        drhoeidtime( i1,i2,i3   )= &
         drhoeidtime(i1,i2,i3   ) - wdivFrad * &
                           (deinew00(i1,i2) + deinew01(i1,i2) + deinew10(i1,i2) + deinew11(i1,i2))
      end do ! i1
    end do ! i2
    !$OMP END DO NOWAIT
    !
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME)
      do i2=m2,n2
        do i1=m1,n1
          ! --- Collect momentum changes in old and new plane ---
          deioldsum=deiold00(i1,i2) + deiold01(i1,i2) + deiold10(i1,i2) + deiold11(i1,i2)
          drhov1dtime( i1,i2,i3old)= &
           drhov1dtime(i1,i2,i3old) - wdrhov1 * deioldsum
          drhov2dtime( i1,i2,i3old)= &
           drhov2dtime(i1,i2,i3old) - wdrhov2 * deioldsum
          drhov3dtime( i1,i2,i3old)= &
           drhov3dtime(i1,i2,i3old) - wdrhov3 * deioldsum
          deinewsum=deinew00(i1,i2) + deinew01(i1,i2) + deinew10(i1,i2) + deinew11(i1,i2)
          drhov1dtime( i1,i2,i3   )= &
           drhov1dtime(i1,i2,i3   ) - wdrhov1 * deinewsum
          drhov2dtime( i1,i2,i3   )= &
           drhov2dtime(i1,i2,i3   ) - wdrhov2 * deinewsum
          drhov3dtime( i1,i2,i3   )= &
           drhov3dtime(i1,i2,i3   ) - wdrhov3 * deinewsum
        end do ! i1
      end do ! i2
      !$OMP END DO NOWAIT
    endif
!#
#   endif
!#        /* (rhd_shortrad_radpressure_l01) */
    if (ComputeFlux_flag) then
      if (di1 == -1) then
        if (di2 == -1) then
          ! --- .. ---
          ! --- x. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1+1,i2+1) - deinew00(i1+1,i2+1) + &
                                              w01*Inew01(i1+1,i2  ) - deinew01(i1+1,i2  ) + &
                                              w10*Inew10(i1  ,i2+1) - deinew10(i1  ,i2+1) + &
                                              w11*Inew11(i1  ,i2  ) - deinew11(i1  ,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (0.5*(w00*Inew00(i1  ,i2+1) - deinew00(i1  ,i2+1))+ &
                                              0.5*(w00*Inew00(i1  ,i2  ) - deinew00(i1  ,i2  ))+ &
                                                  (w01*Inew01(i1  ,i2  ) - deinew01(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (0.5*(w00*Inew00(i1+1,i2  ) - deinew00(i1+1,i2  ))+ &
                                              0.5*(w00*Inew00(i1  ,i2  ) - deinew00(i1  ,i2  ))+ &
                                                  (w10*Inew10(i1  ,i2  ) - deinew10(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- x. ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1+1,i2  ) - deinew00(i1+1,i2  ) + &
                                              w01*Inew01(i1+1,i2-1) - deinew01(i1+1,i2-1) + &
                                              w10*Inew10(i1  ,i2  ) - deinew10(i1  ,i2  ) + &
                                              w11*Inew11(i1  ,i2-1) - deinew11(i1  ,i2-1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )+wFrad * (0.5*(w01*Inew01(i1  ,i2-1) - deinew01(i1  ,i2-1))+ &
                                              0.5*(w01*Inew01(i1  ,i2  ) - deinew01(i1  ,i2  ))+ &
                                                  (w00*Inew00(i1  ,i2  ) - deinew00(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )-wFrad * (0.5*(w01*Inew01(i1+1,i2-1) - deinew01(i1+1,i2-1))+ &
                                              0.5*(w01*Inew01(i1  ,i2-1) - deinew01(i1  ,i2-1))+ &
                                                  (w11*Inew11(i1  ,i2-1) - deinew11(i1  ,i2-1)) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      else
        if (di2 == -1) then
          ! --- .. ---
          ! --- .x ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1  ,i2+1) - deinew00(i1  ,i2+1) + &
                                              w01*Inew01(i1  ,i2  ) - deinew01(i1  ,i2  ) + &
                                              w10*Inew10(i1-1,i2+1) - deinew10(i1-1,i2+1) + &
                                              w11*Inew11(i1-1,i2  ) - deinew11(i1-1,i2  )  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )-wFrad * (0.5*(w10*Inew10(i1-1,i2+1) - deinew10(i1-1,i2+1))+ &
                                              0.5*(w10*Inew10(i1-1,i2  ) - deinew10(i1-1,i2  ))+ &
                                                  (w11*Inew11(i1-1,i2  ) - deinew11(i1-1,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )+wFrad * (0.5*(w10*Inew10(i1-1,i2  ) - deinew10(i1-1,i2  ))+ &
                                              0.5*(w10*Inew10(i1  ,i2  ) - deinew10(i1  ,i2  ))+ &
                                                  (w00*Inew00(i1  ,i2  ) - deinew00(i1  ,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        else
          ! --- .x ---
          ! --- .. ---
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1
              Frad3( i1  ,i2  ,i3  )= &
               Frad3(i1  ,i2  ,i3  )+wFrad * (w00*Inew00(i1  ,i2  ) - deinew00(i1  ,i2  ) + &
                                              w01*Inew01(i1  ,i2-1) - deinew01(i1  ,i2-1) + &
                                              w10*Inew10(i1-1,i2  ) - deinew10(i1-1,i2  ) + &
                                              w11*Inew11(i1-1,i2-1) - deinew11(i1-1,i2-1)  )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2
            do i1=m1,n1+1
              Frad1( i1  ,i2  ,i3  )= &
               Frad1(i1  ,i2  ,i3  )-wFrad * (0.5*(w11*Inew11(i1-1,i2-1) - deinew11(i1-1,i2-1))+ &
                                              0.5*(w11*Inew11(i1-1,i2  ) - deinew11(i1-1,i2  ))+ &
                                                  (w10*Inew10(i1-1,i2  ) - deinew10(i1-1,i2  )) )
            end do ! i1
          end do ! i2
          !$OMP END DO NOWAIT
          !$OMP DO SCHEDULE(RUNTIME)
          do i2=m2,n2+1
            do i1=m1,n1
              Frad2( i1  ,i2  ,i3  )= &
               Frad2(i1  ,i2  ,i3  )-wFrad * (0.5*(w11*Inew11(i1-1,i2-1) - deinew11(i1-1,i2-1))+ &
                                              0.5*(w11*Inew11(i1  ,i2-1) - deinew11(i1  ,i2-1))+ &
                                                  (w01*Inew01(i1  ,i2-1) - deinew01(i1  ,i2-1)) )
            end do ! i1
          end do ! i2
          !$OMP END DO
        endif
      endif
      !
    endif ! (ComputeFlux_flag)
    !
!#
#   if (rhd_shortrad_dir_l02 == 0)
!#
    !$OMP END PARALLEL
!#
#   endif
!#        /* rhd_shortrad_dir_l02 == 0 */
  end do ! i3
!#
# if (rhd_shortrad_dir_l02 == 1)
!#
  !$OMP END PARALLEL
!#
# endif
!#      /* rhd_shortrad_dir_l02 == 1 */
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if ((bound_r3 == bound_InternalNext) .or. (bound_r3 == bound_InternalPeriodic)) then
    ! --- Box has outgoing internal boundary: send boundary values to neighbor process ---
    ! write(gl%nc_p, '(A,I3,A,I3,A,I1,A,I4)') &
    !   '#', geo%MyProc, &
    !   ' Send    I3Send to=', geo%Proc_r(action%dimV(3)), &
    !   ' in Dir=', action%dimV(3), &
    !   ' tag=', 3231
    call MPI_ISend(I3Send, (n1-m1+3)*(n2-m2+3), MPI_REAL, geo%Proc_r(action%dimV(3)), &
                   3231, geo%Comm, RequestSendRecvMPI(1), ierrMPI)
  endif ! (bound_r3)
  ! write(gl%nc_p, '(A,I3,A)') &
  !   '#', geo%MyProc, ' End integration I3 up'
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!!$  call uio_openwr(nc, 'rhd_short_dir3_test.out', form='unformatted', conv='ieee_4', &
!!$                  position='append')
!!$  call uio_wrlabl(nc, dataset_ident, date='now' )
!!$  call uio_wrlabl(nc, box_ident                 )
!!$  call uio_wr(nc, 'z'          , box_id_ident   )
!!$  call uio_wr(nc, xb1          , 'xb1'          )
!!$  call uio_wr(nc, xb2          , 'xb2'          )
!!$  call uio_wr(nc, xb3          , 'xb3'          )
!!$  call uio_wr(nc, 'i3 up'      , 'dirstr'       )
!!$  call uio_wr(nc, r_dir        , 'r_dir'        )
!!$  call uio_wr(nc, RdOut        , 'rdout'        )
!!$  call uio_wr(nc, S            , 's'            )
!!$  call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$  call uio_wr(nc, I            , 'i'            )
!!$  !call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$  !call uio_wr(nc, Frad1        , 'frad1'        )
!!$  !call uio_wr(nc, Frad2        , 'frad2'        )
!!$  !call uio_wr(nc, Frad3        , 'frad3'        )
!!$  !call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$  !call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$  !call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$  call uio_wrlabl(nc, endbox_ident              )
!!$  call uio_wrlabl(nc, enddataset_ident          )
!!$  call uio_closrd(nc)
  !
endif ! (ComputeIntUp_flag)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(I)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
endif ! (r_dir()/r_dir_(3))
!
end subroutine rhd_shortrad_Dir3


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_tauCouple1(r_dir, r_wei, n_dir, &
                                   m1,n1, m2,n2, m3,n3, &
                                   xb1, xb2, xb3, &
                                   dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                   taucouplesum)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_tauCouple1 ('rhd_short-characteristics_radiation_tau_Couple_1')
!
! PURPOSE:
!   Compute tau coupling to get radiative relaxation time scale.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_tauCouple1(r_dir, r_wei, n_dir, &
!                                m1,n1, m2,n2, m3,n3, &
!                                xb1, xb2, xb3, &
!                                dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                                taucouplesum)
!
! INPUT:
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   taucouplesum: ('tau_couple_sum') real 3D array with tau coupling to all relevant neighbors
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   w00:          ('weight_00') real, weight of neighbor cell (di2,di3)+(0,0)
!   w10:          ('weight_10') real, weight of neighbor cell (di2,di3)+(1,0)
!   w01:          ('weight_01') real, weight of neighbor cell (di2,di3)+(0,1)
!   w11:          ('weight_11') real, weight of neighbor cell (di2,di3)+(1,1)
!   w:            ('weight') real, temporary variable for weight of current cell pair
!   wt:           ('weight_temporary') real, temporary variable used during value swapping
!   taucouple:    ('tau_couple') real, coupling coefficient to one neighbor
!   taucouple_total: ('tau_couple_total') real, tau coupling to all relevant neighbors for given
!                    main direction
!   mu:           ('mu') real, cosine of inclination angle relative to x1 direction
!   geom:         ('geometry') real, geometrical factor for total coupling
!   dx1:          ('delta_x_1') real, cell size in x1 direction (taken from xb1)
!   dx2:          ('delta_x_2') real, cell size in x2 direction (taken from xb2)
!   dx3:          ('delta_x_3') real, cell size in x3 direction (taken from xb3)
!   di2:          ('delta_index_2') integer, relative index of leftmost important neighbor cell
!                 in x2 direction (-1 or 0)
!   di3:          ('delta_index_3') integer, relative index of leftmost important neighbor cell
!                 in x3 direction (-1 or 0)
!   i2old:        ('index_2_old') integer, temporary variable of x2 index of old cell
!   i3old:        ('index_3_old') integer, temporary variable of x3 index of old cell
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   dtauthick_limit: ('delta_tau_thick_limit') real parameter, border between optically thin
!                    and thick; typically 1/6
!
! ROUTINES:
!   rhd_shortrad_dtauop:   Compute contribution to radiative time scale from optical thickness
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_dtauop is called frequently and should be inlined.
!
! PROCEDURE:
!   Call rhd_shortrad_dtauop for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-04-10 (B.F.) Written
!   2002-08-01 (B.F.) Branch for simple rays
!   2003-02-19 (B.F.) Correction: some variables declared private
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2017-10-27 (B.F.) Make the number of ghost cells flexible
!   2018-12-02 (B.F. Uppsala) Derive dx from xb1
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFg=1
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-nFg:n1+nFg+1), &
                           intent(in)   :: xb1
real, dimension(m2-nFg:n2+nFg+1), &
                           intent(in)   :: xb2
real, dimension(m3-nFg:n3+nFg+1), &
                           intent(in)   :: xb3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(in)   :: dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
real, dimension(m1    :n1  , &
                m2    :n2  , &
                m3    :n3     ), &
                           intent(inout):: taucouplesum
!
! --- Local variables ---
real                                    :: w00, w01, w10, w11, w, wt, &
                                           taucouple, taucouple_total, &
                                           mu
integer                                    di2, di3, i1old, i2old, i3old, &
                                           i1, i2, i3
real, parameter                         :: dtauthick_limit=0.17
real                                    :: expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
                                           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew
logical                                 :: thickflagnew
real                                    :: thickmasknew, wthick
!---------------------------------------------------------------------------------------------------
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
!!$dx1=(wg%rad(1)%xb_r(1)-wg%rad(1)%xb_l(1))/real(wg%rad(1)%n(1)-wg%rad(1)%m(1)+1)
!!$dx2=(wg%rad(1)%xb_r(2)-wg%rad(1)%xb_l(2))/real(wg%rad(1)%n(2)-wg%rad(1)%m(2)+1)
!!$dx3=(wg%rad(1)%xb_r(3)-wg%rad(1)%xb_l(3))/real(wg%rad(1)%n(3)-wg%rad(1)%m(3)+1)
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
!
! --- Geometrical weight for ray ---
mu  =abs(r_dir(1))
!geom=8.0*mu/(real(max(n_dir,3)) * dx1)
!
if ( (abs(anint(r_dir(2)/r_dir(1))-(r_dir(2)/r_dir(1))) < 0.001) .and. &
     (abs(anint(r_dir(3)/r_dir(1))-(r_dir(3)/r_dir(1))) < 0.001)      ) then
  ! --- Simple ray direction: along axis or diagonal ---
  !
  ! --- Find indices of relevant cells in old plane ---
  di2=nint(r_dir(2)/r_dir(1))
  di3=nint(r_dir(3)/r_dir(1))
  !
  ! --- Integration ---
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i2old, i3old, taucouple, taucouple_total, &
  !$OMP           expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           thickflagnew, thickmasknew, wthick)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3, n3
    do i2=m2, n2
      do i1=m1, n1
        i2old=i2+di2
        i3old=i3+di3
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1+1,i2old,i3old), &
                            exp_mdtauhalf(i1+1,i2old,i3old), dtauhalf(        i1+1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1+1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1+1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1+1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple
        !
        i2old=i2-di2
        i3old=i3-di3
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1-1,i2old,i3old), &
                            exp_mdtauhalf(i1-1,i2old,i3old), dtauhalf(        i1-1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1-1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1-1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1-1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + taucouple
        !
        !taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + geom*taucouple_total
        taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + &
                               (8.0*mu/(real(max(n_dir,3)))) / (xb1(i1+1)-xb1(i1)) * taucouple_total
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
else
  ! --- General ray direction ---
  !
  ! --- Find indices of relevant cells in old plane ---
  call rhd_shortrad_RayWeight(r_dir(1), r_dir(2), r_dir(3), di2, di3, w00, w01, w10, w11)
  !
  !!$write(gl%nc_p,'(A,5E13.5)') 'mu, w00,w01,w10,w11: ', mu, w00,w01,w10,w11
  !!$write(gl%nc_p,'(A,2I3)')    'di2, di3: ', di2, di3
  !!$call flush(gl%nc_p)
  !
  ! --- Integration ---
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i2old, i3old, taucouple, taucouple_total, w, &
  !$OMP           expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           thickflagnew, thickmasknew, wthick)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3, n3
    do i2=m2, n2
      do i1=m1, n1
        taucouple_total=0.0
        !
        i2old=i2+di2
        i3old=i3+di3
        w    =w00
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1+1,i2old,i3old), &
                            exp_mdtauhalf(i1+1,i2old,i3old), dtauhalf(        i1+1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1+1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1+1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1+1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2+di2+1
        i3old=i3+di3
        w    =w10
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1+1,i2old,i3old), &
                            exp_mdtauhalf(i1+1,i2old,i3old), dtauhalf(        i1+1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1+1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1+1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1+1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2+di2
        i3old=i3+di3+1
        w    =w01
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1+1,i2old,i3old), &
                            exp_mdtauhalf(i1+1,i2old,i3old), dtauhalf(        i1+1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1+1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1+1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1+1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2+di2+1
        i3old=i3+di3+1
        w    =w11
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1+1,i2old,i3old), &
                            exp_mdtauhalf(i1+1,i2old,i3old), dtauhalf(        i1+1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1+1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1+1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1+1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2-di2
        i3old=i3-di3
        w    =w00
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1-1,i2old,i3old), &
                            exp_mdtauhalf(i1-1,i2old,i3old), dtauhalf(        i1-1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1-1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1-1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1-1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2-di2-1
        i3old=i3-di3
        w    =w10
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1-1,i2old,i3old), &
                            exp_mdtauhalf(i1-1,i2old,i3old), dtauhalf(        i1-1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1-1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1-1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1-1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2-di2
        i3old=i3-di3-1
        w    =w01
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1-1,i2old,i3old), &
                            exp_mdtauhalf(i1-1,i2old,i3old), dtauhalf(        i1-1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1-1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1-1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1-1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i2old=i2-di2-1
        i3old=i3-di3-1
        w    =w11
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1-1,i2old,i3old), &
                            exp_mdtauhalf(i1-1,i2old,i3old), dtauhalf(        i1-1,i2old,i3old), &
                                                             expl2t_mdtauhalf(i1  ,i2   ,i3   ), &
                            exp_mdtauhalf(i1  ,i2   ,i3   ), dtauhalf(        i1  ,i2   ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1-1,i2old,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1-1,i2old,i3old)
        dtauhalfold        =dtauhalf(        i1-1,i2old,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1  ,i2   ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1  ,i2   ,i3   )
        dtauhalfnew        =dtauhalf(        i1  ,i2   ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        !taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + geom*taucouple_total
        taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + &
                               (8.0*mu/(real(max(n_dir,3)))) / (xb1(i1+1)-xb1(i1)) * taucouple_total
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
endif
!
end subroutine rhd_shortrad_tauCouple1


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_tauCouple2(r_dir, r_wei, n_dir, &
                                   m1,n1, m2,n2, m3,n3, &
                                   xb1, xb2, xb3, &
                                   dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                   taucouplesum)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_tauCouple2 ('rhd_short-characteristics_radiation_tau_couple_3')
!
! PURPOSE:
!   Compute tau coupling to get radiative relaxation time scale.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_tauCouple2(r_dir, r_wei, n_dir, &
!                                m1,n1, m2,n2, m3,n3, &
!                                xb1, xb2, xb3, &
!                                dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                                taucouplesum)
!
! INPUT:
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   taucouplesum: ('tau_couple_sum') real 3D array with tau coupling to all relevant neighbors
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!   see rhd_shortrad_tauCouple1
!
! ROUTINES:
!   rhd_shortrad_dtauop:   Compute contribution to radiative time scale from optical thickness
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_dtauop is called frequently and should be inlined.
!
! PROCEDURE:
!   Call rhd_shortrad_dtauop for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-04-10 (B.F.) Written
!   2002-08-02 (B.F.) Branch for simple rays
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2017-10-27 (B.F.) Make the number of ghost cells flexible
!   2018-12-02 (B.F. Uppsala) Derive dx from xb2
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFg=1
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-nFg:n1+nFg+1), &
                           intent(in)   :: xb1
real, dimension(m2-nFg:n2+nFg+1), &
                           intent(in)   :: xb2
real, dimension(m3-nFg:n3+nFg+1), &
                           intent(in)   :: xb3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(in)   :: dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
real, dimension(m1    :n1  , &
                m2    :n2  , &
                m3    :n3     ), &
                           intent(inout):: taucouplesum
!
! --- Local variables ---
real                                    :: w00, w01, w10, w11, w, wt, &
                                           taucouple, taucouple_total, &
                                           mu
integer                                    di1, di3, i1old, i2old, i3old, &
                                           i1, i2, i3
real, parameter                         :: dtauthick_limit=0.17
real                                    :: expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
                                           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew
logical                                 :: thickflagnew
real                                    :: thickmasknew, wthick
!---------------------------------------------------------------------------------------------------
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
!!$dx1=(wg%rad(1)%xb_r(1)-wg%rad(1)%xb_l(1))/real(wg%rad(1)%n(1)-wg%rad(1)%m(1)+1)
!!$dx2=(wg%rad(1)%xb_r(2)-wg%rad(1)%xb_l(2))/real(wg%rad(1)%n(2)-wg%rad(1)%m(2)+1)
!!$dx3=(wg%rad(1)%xb_r(3)-wg%rad(1)%xb_l(3))/real(wg%rad(1)%n(3)-wg%rad(1)%m(3)+1)
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
!
! --- Geometrical weight for ray ---
mu  =abs(r_dir(2))
!geom=8.0*mu/(real(max(n_dir,3)) * dx2)
!
if ( (abs(anint(r_dir(1)/r_dir(2))-(r_dir(1)/r_dir(2))) < 0.001) .and. &
     (abs(anint(r_dir(3)/r_dir(2))-(r_dir(3)/r_dir(2))) < 0.001)      ) then
  ! --- Simple ray direction: along axis or diagonal ---
  !
  ! --- Find indices of relevant cells in old plane ---
  di1=nint(r_dir(1)/r_dir(2))
  di3=nint(r_dir(3)/r_dir(2))
  !
  ! --- Integration ---
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i3old, taucouple, taucouple_total, &
  !$OMP           expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           thickflagnew, thickmasknew, wthick)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3, n3
    do i2=m2, n2
      do i1=m1, n1
        i1old=i1+di1
        i3old=i3+di3
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2+1,i3old), &
                            exp_mdtauhalf(i1old,i2+1,i3old), dtauhalf(        i1old,i2+1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2+1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2+1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2+1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple
        !
        i1old=i1-di1
        i3old=i3-di3
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2-1,i3old), &
                            exp_mdtauhalf(i1old,i2-1,i3old), dtauhalf(        i1old,i2-1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2-1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2-1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2-1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + taucouple
        !
        !taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + geom*taucouple_total
        taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + &
                               (8.0*mu/(real(max(n_dir,3)))) / (xb2(i2+1)-xb2(i2)) * taucouple_total
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
else
  ! --- General ray direction ---
  !
  ! --- Find indices of relevant cells in old plane ---
  call rhd_shortrad_RayWeight(r_dir(2), r_dir(1), r_dir(3), di1, di3, w00, w01, w10, w11)
  !
  !!$write(gl%nc_p,'(A,5E13.5)') 'mu, w00,w01,w10,w11: ', mu, w00,w01,w10,w11
  !!$write(gl%nc_p,'(A,2I3)')    'di1, di3: ', di1, di3
  !!$call flush(gl%nc_p)
  !
  ! --- Integration ---
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i1old, i3old, taucouple, taucouple_total, w, &
  !$OMP           expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           thickflagnew, thickmasknew, wthick)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3, n3
    do i2=m2, n2
      do i1=m1, n1
        taucouple_total=0.0
        !
        i1old=i1+di1
        i3old=i3+di3
        w    =w00
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2+1,i3old), &
                            exp_mdtauhalf(i1old,i2+1,i3old), dtauhalf(        i1old,i2+1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2+1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2+1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2+1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1+di1+1
        i3old=i3+di3
        w    =w10
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2+1,i3old), &
                            exp_mdtauhalf(i1old,i2+1,i3old), dtauhalf(        i1old,i2+1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2+1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2+1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2+1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1+di1
        i3old=i3+di3+1
        w    =w01
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2+1,i3old), &
                            exp_mdtauhalf(i1old,i2+1,i3old), dtauhalf(        i1old,i2+1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2+1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2+1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2+1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1+di1+1
        i3old=i3+di3+1
        w    =w11
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2+1,i3old), &
                            exp_mdtauhalf(i1old,i2+1,i3old), dtauhalf(        i1old,i2+1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2+1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2+1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2+1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1
        i3old=i3-di3
        w    =w00
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2-1,i3old), &
                            exp_mdtauhalf(i1old,i2-1,i3old), dtauhalf(        i1old,i2-1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2-1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2-1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2-1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1-1
        i3old=i3-di3
        w    =w10
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2-1,i3old), &
                            exp_mdtauhalf(i1old,i2-1,i3old), dtauhalf(        i1old,i2-1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2-1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2-1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2-1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1
        i3old=i3-di3-1
        w    =w01
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2-1,i3old), &
                            exp_mdtauhalf(i1old,i2-1,i3old), dtauhalf(        i1old,i2-1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2-1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2-1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2-1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1-1
        i3old=i3-di3-1
        w    =w11
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2-1,i3old), &
                            exp_mdtauhalf(i1old,i2-1,i3old), dtauhalf(        i1old,i2-1,i3old), &
                                                             expl2t_mdtauhalf(i1   ,i2  ,i3   ), &
                            exp_mdtauhalf(i1   ,i2  ,i3   ), dtauhalf(        i1   ,i2  ,i3   ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2-1,i3old)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2-1,i3old)
        dtauhalfold        =dtauhalf(        i1old,i2-1,i3old)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2  ,i3   )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2  ,i3   )
        dtauhalfnew        =dtauhalf(        i1   ,i2  ,i3   )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        !taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + geom*taucouple_total
        taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + &
                               (8.0*mu/(real(max(n_dir,3)))) / (xb2(i2+1)-xb2(i2)) * taucouple_total
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !$OMP END PARALLEL
endif
!
end subroutine rhd_shortrad_tauCouple2


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_tauCouple3(r_dir, r_wei, n_dir, &
                                   m1,n1, m2,n2, m3,n3, &
                                   xb1, xb2, xb3, &
                                   dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                   taucouplesum)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_tauCouple3 ('rhd_short-characteristics_radiation_tau_Couple_3')
!
! PURPOSE:
!   Compute tau coupling to get radiative relaxation time scale.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_tauCouple3(r_dir, r_wei, n_dir, &
!                                m1,n1, m2,n2, m3,n3, &
!                                xb1, xb2, xb3, &
!                                dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
!                                taucouplesum)
!
! INPUT:
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   r_wei:        ('ray_weight') real with ray weight
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtauhalf)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtauhalf)-1+dtauhalf
!
! INPUT/OUTPUT:
!   taucouplesum: ('tau_couple_sum') real 3D array with tau coupling to all relevant neighbors
!
! OUTPUT:
!   None (INPUT/OUTPUT only)
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_shortrad_dtauop:   Compute contribution to radiative time scale from optical thickness
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions. Cells must be cubic: dx1=dx2=dx3
!   The routine rhd_shortrad_dtauop is called frequently and should be inlined.
!
! PROCEDURE:
!   Call rhd_shortrad_dtauop for all necessary pairs of cells in a reasonable order.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-04-10 (B.F.) Written
!   2002-08-02 (B.F.) Branch for simple rays
!   2013-01-22 (B.F. Lyon) Compute thickmask locally
!   2013-02-20 (B.F.) Swap i2 and i3 loops
!   2013-12-06 (B.F. Uppsala) New parameter r_wei
!   2017-10-27 (B.F.) Make the number of ghost cells flexible
!   2018-12-02 (B.F. Uppsala) Derive dx from xb3
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFg=1
real, dimension(3),        intent(in)   :: r_dir
real,                      intent(in)   :: r_wei
integer,                   intent(in)   :: n_dir, m1,n1, m2,n2, m3,n3
real, dimension(m1-nFg:n1+nFg+1), &
                           intent(in)   :: xb1
real, dimension(m2-nFg:n2+nFg+1), &
                           intent(in)   :: xb2
real, dimension(m3-nFg:n3+nFg+1), &
                           intent(in)   :: xb3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(in)   :: dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
real, dimension(m1    :n1  , &
                m2    :n2  , &
                m3    :n3     ), &
                           intent(inout):: taucouplesum
!
! --- Local variables ---
real                                    :: w00, w01, w10, w11, w, wt, &
                                           taucouple, taucouple_total, &
                                           mu, geom
integer                                    di1, di2, i1old, i2old, i3old, &
                                           i1, i2, i3
real, parameter                         :: dtauthick_limit=0.17
real                                    :: expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
                                           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew
logical                                 :: thickflagnew
real                                    :: thickmasknew, wthick
!---------------------------------------------------------------------------------------------------
!
! --- Compute typical grid size: grid assumed to be equidistant ---
!!$dx1=(xb1(n1+1)-xb1(m1))/real(n1+1-m1)
!!$dx2=(xb2(n2+1)-xb2(m2))/real(n2+1-m2)
!!$dx3=(xb3(n3+1)-xb3(m3))/real(n3+1-m3)
!!$dx1=(wg%rad(1)%xb_r(1)-wg%rad(1)%xb_l(1))/real(wg%rad(1)%n(1)-wg%rad(1)%m(1)+1)
!!$dx2=(wg%rad(1)%xb_r(2)-wg%rad(1)%xb_l(2))/real(wg%rad(1)%n(2)-wg%rad(1)%m(2)+1)
!!$dx3=(wg%rad(1)%xb_r(3)-wg%rad(1)%xb_l(3))/real(wg%rad(1)%n(3)-wg%rad(1)%m(3)+1)
!
! --- In lhdrad:   wFrad=fourpi/3.0_drk
! ---              Fradb()=Fradb() * (3.0_drk/real(max(n_dir,3), kind=drk)*mu)
! ---              divFrad_ray()=-real(diffFradb(), kind=srk)/dx3
! --- In shortrad: wdivFrad=fourpi/3.0 * 0.5 * (3.0/real(max(n_dir,3), kind=srk)*mu)
!
! --- Geometrical weight for ray ---
mu  =abs(r_dir(3))
!geom=8.0*mu/(real(max(n_dir,3)) * dx3)
!
if ( (abs(anint(r_dir(1)/r_dir(3))-(r_dir(1)/r_dir(3))) < 0.001) .and. &
     (abs(anint(r_dir(2)/r_dir(3))-(r_dir(2)/r_dir(3))) < 0.001)      ) then
  ! --- Simple ray direction: along axis or diagonal ---
  !
  ! --- Find indices of relevant cells in old plane ---
  di1=nint(r_dir(1)/r_dir(3))
  di2=nint(r_dir(2)/r_dir(3))
  !
  ! --- Integration ---
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i3, i1old, i2old, taucouple, taucouple_total, &
  !$OMP           expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           thickflagnew, thickmasknew, wthick)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i2=m2, n2
    do i3=m3, n3
      do i1=m1, n1
        i1old=i1+di1
        i2old=i2+di2
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3+1), &
                            exp_mdtauhalf(i1old,i2old,i3+1), dtauhalf(        i1old,i2old,i3+1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3+1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3+1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3+1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple
        !
        i1old=i1-di1
        i2old=i2-di2
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3-1), &
                            exp_mdtauhalf(i1old,i2old,i3-1), dtauhalf(        i1old,i2old,i3-1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3-1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3-1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3-1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + taucouple
        !
        !taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + geom*taucouple_total
        taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + &
                               (8.0*mu/(real(max(n_dir,3)))) / (xb3(i3+1)-xb3(i3)) * taucouple_total
      end do ! i1
    end do ! i3
  end do ! i2
  !$OMP END DO
  !$OMP END PARALLEL
else
  ! --- General ray direction ---
  !
  ! --- Find indices of relevant cells in old plane ---
  call rhd_shortrad_RayWeight(r_dir(3), r_dir(1), r_dir(2), di1, di2, w00, w01, w10, w11)
  !
  !!$write(gl%nc_p,'(A,5E13.5)') 'mu, w00,w01,w10,w11: ', mu, w00,w01,w10,w11
  !!$write(gl%nc_p,'(A,2I3)')    'di1, di2: ', di1, di2
  !!$call flush(gl%nc_p)
  !
  ! --- Integration ---
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP   PRIVATE(i1, i2, i3, i1old, i2old, taucouple, taucouple_total, w, &
  !$OMP           expl2t_mdtauhalfold, exp_mdtauhalfold, dtauhalfold, &
  !$OMP           expl2t_mdtauhalfnew, exp_mdtauhalfnew, dtauhalfnew, &
  !$OMP           thickflagnew, thickmasknew, wthick)
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i2=m2, n2
    do i3=m3, n3
      do i1=m1, n1
        taucouple_total=0.0
        !
        i1old=i1+di1
        i2old=i2+di2
        w    =w00
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3+1), &
                            exp_mdtauhalf(i1old,i2old,i3+1), dtauhalf(        i1old,i2old,i3+1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3+1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3+1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3+1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1+di1+1
        i2old=i2+di2
        w    =w10
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3+1), &
                            exp_mdtauhalf(i1old,i2old,i3+1), dtauhalf(        i1old,i2old,i3+1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3+1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3+1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3+1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1+di1
        i2old=i2+di2+1
        w    =w01
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3+1), &
                            exp_mdtauhalf(i1old,i2old,i3+1), dtauhalf(        i1old,i2old,i3+1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3+1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3+1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3+1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1+di1+1
        i2old=i2+di2+1
        w    =w11
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3+1), &
                            exp_mdtauhalf(i1old,i2old,i3+1), dtauhalf(        i1old,i2old,i3+1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3+1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3+1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3+1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1
        i2old=i2-di2
        w    =w00
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3-1), &
                            exp_mdtauhalf(i1old,i2old,i3-1), dtauhalf(        i1old,i2old,i3-1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3-1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3-1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3-1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1-1
        i2old=i2-di2
        w    =w10
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3-1), &
                            exp_mdtauhalf(i1old,i2old,i3-1), dtauhalf(        i1old,i2old,i3-1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3-1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3-1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3-1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1
        i2old=i2-di2-1
        w    =w01
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3-1), &
                            exp_mdtauhalf(i1old,i2old,i3-1), dtauhalf(        i1old,i2old,i3-1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3-1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3-1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3-1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        i1old=i1-di1-1
        i2old=i2-di2-1
        w    =w11
!#
#       if (rhd_shortrad_dtauop_l02 == 0)
!#
        call rhd_shortrad_dtauop(dtauthick_limit,            expl2t_mdtauhalf(i1old,i2old,i3-1), &
                            exp_mdtauhalf(i1old,i2old,i3-1), dtauhalf(        i1old,i2old,i3-1), &
                                                             expl2t_mdtauhalf(i1   ,i2   ,i3  ), &
                            exp_mdtauhalf(i1   ,i2   ,i3  ), dtauhalf(        i1   ,i2   ,i3  ), &
                            0.5+sign(0.5, dtauhalf(i1   ,i2   ,i3   )-dtauthick_limit), &
                            taucouple)
!#
#       else /* rhd_shortrad_dtauop_l02 */
!#
        expl2t_mdtauhalfold=expl2t_mdtauhalf(i1old,i2old,i3-1)
        exp_mdtauhalfold   =exp_mdtauhalf(   i1old,i2old,i3-1)
        dtauhalfold        =dtauhalf(        i1old,i2old,i3-1)
        expl2t_mdtauhalfnew=expl2t_mdtauhalf(i1   ,i2   ,i3  )
        exp_mdtauhalfnew   =exp_mdtauhalf(   i1   ,i2   ,i3  )
        dtauhalfnew        =dtauhalf(        i1   ,i2   ,i3  )
!        thickmasknew       =0.5+sign(0.5, dtauhalfnew-dtauthick_limit)
!#
#       if (rhd_shortrad_dtauop_l01 == 1)
        include 'rhd_shortrad_dtauop01.f90'
#       else /* rhd_shortrad_dtauop_l01 */
        include 'rhd_shortrad_dtauop02.f90'
#       endif /* rhd_shortrad_dtauop_l01 */
!#
#       endif /* rhd_shortrad_dtauop_l02 */
!#
        taucouple_total=taucouple_total + w*taucouple
        !
        !taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + geom*taucouple_total
        taucouplesum(i1,i2,i3)=taucouplesum(i1,i2,i3) + &
                               (8.0*mu/(real(max(n_dir,3)))) / (xb3(i3+1)-xb3(i3)) * taucouple_total
      end do ! i1
    end do ! i3
  end do ! i2
  !$OMP END DO
  !$OMP END PARALLEL
endif
!
end subroutine rhd_shortrad_tauCouple3


!----------*******************----------------------------------------------------------------------
subroutine rhd_shortrad_Formal(geo, action, &
                               Output_flag, Computekappa_flag, Computetaufunc_flag, &
                               ComputeIntensity_flag, ComputeFlux_flag, &
                               ComputePressure_flag, &
                               n_dir, dx_dir, ray_dir, ray_wei, &
                               m1,n1, m2,n2, m3,n3, nquc, &
                               xb1, xb2, xb3, &
                               rho, quc, T, logP, &
                               rhokappa, tau_func, Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                              if (rhd_shortrad_radpressure_l01 != 0)
!#
                               drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                              endif
!#
                               ierr, outstr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_Formal ('rhd_short-characteristics_radiation_Formal')
!
! PURPOSE:
!   Compute formal solution: 3D radiative energy flux and change.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_Formal(geo, action, &
!                            Output_flag, Computekappa_flag, Computetaufunc_flag, &
!                            ComputeIntensity_flag, ComputeFlux_flag, ComputePressure_flag, &
!                            n_dir, dx_dir, ray_dir, ray_wei, &
!                            m1,n1, m2,n2, m3,n3, nquc, &
!                            xb1, xb2, xb3, &
!                            rho, quc, T, logP, &
!                            rhokappa, tau_func, Frad1, Frad2, Frad3, drhoeidtime &
!                            , drhov1dtime, drhov2dtime, drhov3dtime &
!                            , ierr, outstr)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   Output_flag:  ('Output_flag') logical to request computation of additional output data
!   Computekappa_flag: ('Compute_kappa_flag') logical, activate computation of new opacity
!   Computetaufunc_flag: ('Compute_tau_function_flag') logical, activate comp. of rad. time scale
!   ComputeIntensity_flag: ('Compute_Intensity_flag') logical, activate intensity computation
!   ComputeFlux_flag: ('Compute_Flux_flag') logical, activate flux computation
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   n_dir:        ('number_directions') integer, total number of rays per sub time step
!   dx_dir:       ('delta_x_directions') 3-element real vector with relative grid step size in
!                 all directions, dx_dir=(/ dx1, dx2, dx3 /) / sqrt(dx1**2+dx2**2+dx3**2)
!   ray_dir:      ('ray_directions') 3xn_dir array with all ray directions,
!                 typically from rhd_shortrad_RayDirections (and a rotation)
!   ray_wei:      ('ray_weights') real N-element vector with ray weights
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nquc:         ('number_quantities_center') integer, number of additional density arrays
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:          ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:          ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:          ('rho') real 3D array with density, attention: one buffer cell in all directions
!   quc:          ('quantity_centered') real 4D array with additional densities
!   T:            ('Temperature') real 3D array with temperature
!   logP:         ('logarithm_Pressure') real 3D array with natural logarithm of pressure,
!
! INPUT/OUTPUT:
!   rhokappa:     ('rho_kappa') real 3D array with opacities
!   tau_func:     ('tau_function') real 3D array with optical geometry function (for time scale)
!
! OUTPUT:
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                   =0:   normal run
!                  100:  Severe error: ray direction incompatible with boundary conditions; exit
!
! LOCAL VARIABLES:
!   i_dir:        ('index_directions') integer, loop index for ray direction number
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   dir:          ('direction') integer, main direction of current ray (1, 2, or 3)
!   nc:           ('number_channel') integer, channel unit number for text output (usually 6)
!   nbin:         ('number_bins') integer, current number of bins (or bands)
!   nu:           ('nu') integer with frequency index
!   nu1:          ('nu_1') integer with lower frequency index
!   nu2:          ('nu_2') integer with upper frequency index
!   oneovermu:    ('one_over_mu') real with 1/mu relative to current main direction
!   oneovermu_last: ('one_over_mu_last') real with 1/mu of previous ray
!   x:            ('x') real temporary variable with dtau/2 for exp(x) computation
!   expmx:        ('exponential_minus_x') real temporary variable with exp(-dtau/2)
!   weight:       ('weight') real temporary variable with large-dtau weight
!   r_wei:        ('ray_weight') real with ray weight
!   Computeexptau_flag: ('Compute_exponential_tau_flag') logical to indicate if (new) delta taus
!                       have to be computed
!   dtauthick_limit: ('delta_tau_thick_limit') real param., border between optically thin and thick
!   ierr0:        ('integer_error_0') integer with error status
!   outstr0:      ('output_string_0') character with error report
!   r_dir:        ('ray_direction') real vector (3 elements) with ray direction
!   dx:           ('delta_x') real vector (3 elements) with typical grid-cell size
!   S:            ('Source') real 3D array with source function
!   dtauhalf:     ('delta_tau_half') real 3D array with half optical depth of cells: dtau/2
!   exp_mdtauhalf: ('exponential_minus_delta_tau_half') real 3D array: exp(-dtau/2)
!   expl2t_mdtauhalf: ('exponential_less_2_terms_minus_delta_tau_half') real 3D array:
!                     exp(-dtau/2)-1+dtau/2
!   wTb:          ('weight_Temperature_b') real 3D array with fraction of B_nu
!   tau_func_total: ('tau_function_total') real 3D array with total optical geometry function
!   omp_MaxThreads: ('openmp_Maximum_Threads') integer with maximum number of threads
!   omp_NumThreads: ('openmp_Number_Threads') integer with number of threads
!
! ROUTINES:
!   opta_Info:             Give stored information about current opacity table
!   rhd_shortrad_Dir1:     Compute radiative energy flux for one direction field, main direction 1
!   rhd_shortrad_Dir2:     Compute radiative energy flux for one direction field, main direction 2
!   rhd_shortrad_Dir3:     Compute radiative energy flux for one direction field, main direction 3
!   rhd_shortrad_OpTa:     Compute opacities and source function needed for radiative transfer
!   rhd_shortrad_tauCouple1: Compute tau coupling to get radiative time scale, direction 1
!   rhd_shortrad_tauCouple2: Compute tau coupling to get radiative time scale, direction 2
!   rhd_shortrad_tauCouple3: Compute tau coupling to get radiative time scale, direction 3
!   sub_xkaros_logPT2rhokap: Interpolate opacity per volume for given logP, T, and nu bin (1D,2D,3D)
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   MPI:                   Standard MPI routines and definitions
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   op_dust_module:        Dust opacity routines
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!   omp_lib:               Library of system OpenMP definitions and routines
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-11-07 (B.F.) Transformed: rhd_rad3d_formal -> rhd_shortrad_Formal
!   2002-04-08 (B.F.) Parameter Computekappa_flag added
!   2002-04-09 (B.F.) Parameter Computekappa_flag also triggers computation of time_rad
!   2002-04-27 (B.F.) time_rad_new -> time_rad
!   2002-04-29 (B.F.) time_rad -> tau_func
!   2002-05-23 (B.F.) Parameter ComputeIntensity_flag added
!   2002-05-25 (B.F.) Parameter Computetaufunc_flag added
!   2002-08-02 (B.F.) Initialize fluxes only if explicitly requested
!   2002-08-12 (B.F.) Reuse delta optical depth if possible
!   2002-10-15 (B.F.) Set change of radiative energy to zero outside sphere with radius par%C_test1
!   2002-10-16 (B.F.) Radiative energy only >0 outside sphere with radius par%C_test1
!   2002-11-08 (B.F.) par%C_test1 -> par%r1_rad
!   2002-12-02 (B.F.) thickmask as precomputed array
!   2003-01-31 (B.F.) Computation of expl2t_mdtauhalf also for large dtauhalf
!   2006-10-09 (B.F. Lyon) Allocate temporary 3D arrays as pointers
!   2006-10-20 (B.F.) Possibility to modify temperature to fake non-grey transport (outcommented)
!   2007-02-27 (B.F.) First steps towards non-grey
!   2009-10-21 (B.F. Garching) Put tau_func_total update outside idir loop
!   2009-10-22 (B.F. Garching) Add parameter Output_flag
!   2013-01-22 (B.F. Lyon) Remove thickmask as precomputed array
!   2013-02-05 (B.F.) Correction in timing calls
!   2013-02-06 (B.F.) Experiments with adjustments of thread number
!   2013-02-18 (B.F.) Move masking of radiative energy change outside r1_rad outside
!   2013-03-13 (B.F.) Call new routine rhd_shortrad_OpTa
!   2013-12-06 (B.F. Uppsala) New parameter ray_wei
!   2014-02-06 (B.F.) Make rhd_shortrad_Dir3_new the standard rhd_shortrad_Dir3
!   2014-11-05 (B.F.) Remove parameters xc1,2,3,model; add parameters nquc, quc
!   2017-07-04 (B.F.) Add some MPI commands
!   2017-10-27 (B.F.) Make the number of ghost cells flexible
!   2018-11-29 (B.F.) Remove compiler switch to control loop splitting for exp(-dtau) computation;
!                     handle non-equidistant grid for rays along axes
!   2018-11-30 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use opta_module
use const_module, only: sigopi
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
!$ use omp_lib
!!$use uio_bulk_module
!!$use uio_filedef_module
!#
# if (rhd_box_quc01 != 0)
!#
use op_dust_module
!#
# endif
!#      /* rhd_box_quc01 */
!#
# ifdef rhd_shortrad_formal_t01
!#
use timing_module
!#
# endif
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFb=1, nFg=1
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: Output_flag, Computekappa_flag, Computetaufunc_flag, &
                                           ComputeIntensity_flag, ComputeFlux_flag, &
                                           ComputePressure_flag
integer,                   intent(in)   :: n_dir
real, dimension(3      ),  intent(in)   :: dx_dir
real, dimension(3,n_dir),  intent(in)   :: ray_dir
real, dimension(  n_dir),  intent(in)   :: ray_wei
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nquc
real, dimension(m1-nFg:n1+nFg+1), &
                           intent(in)   :: xb1
real, dimension(m2-nFg:n2+nFg+1), &
                           intent(in)   :: xb2
real, dimension(m3-nFg:n3+nFg+1), &
                           intent(in)   :: xb3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(in)   :: rho, T, logP
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg, &
                1   :nquc       ), &
                           intent(in)   :: quc
real, dimension(m1  :n1  , &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(inout):: tau_func
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb     ), &
                           intent(out)  :: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb     ), &
                           intent(out)  :: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1   ), &
                           intent(out)  :: Frad3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(inout):: rhokappa
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(out)  :: drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(out)  :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i_dir, i1,i2,i3, dir, nc, &
                                           nbin, nu, nu1, nu2, iDirCall, nDirCall
integer, dimension(2)                   :: RequestSendRecvMPI
real                                    :: oneovermu, oneovermu_last, &
                                           x, expmx, weight, &
                                           r_wei, dxb_var
logical                                 :: Computeexptau_flag, DownFirst_flag, &
                                           ComputeIntDown_flag, ComputeIntUp_flag
real, parameter                         :: dtauthick_limit=0.17
real, parameter                         :: C_dxb1_maxvar=0.01
integer, dimension(3)                   :: PCoords
real, dimension(3)                      :: r_dir, dx, dxb_avg
integer                                 :: ierr0
character(len=80)                       :: outstr0
real, dimension(m1-nFg:n1+nFg)          :: dxb1
real, dimension(m2-nFg:n2+nFg)          :: dxb2
real, dimension(m3-nFg:n3+nFg)          :: dxb3
real, dimension(m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg)          :: I1Send, I1Recv
real, dimension(m1-nFg:n1+nFg, &
                m3-nFg:n3+nFg)          :: I2Send, I2Recv
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg)          :: I3Send, I3Recv
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg )         :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf
!#
# endif
!#      /* (rhd_arrays_l01) */
real, dimension(:,:,:), allocatable     :: wTb, tau_func_total
!$ integer                              :: omp_MaxThreads, omp_NumThreads
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_start('SHC: formal')
!#
# endif
!#
!
! --- Initialize error variables: no error occurred, so far ---
ierr0  =0
outstr0=''
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Initialize MPI request handle ---
RequestSendRecvMPI=MPI_REQUEST_NULL
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
! \label{f90:n_radband__use1}
! --- Set parameters describing number of bands (see rhd_rad_init() in rhd_rad_module.F90) ---
! --- Only partly implemented, no CO, yet                                                  ---
if ((par%N_radband >= 1) .and. (par%N_radband <= 2)) then
  if (    par%N_radband == 1) then
    nbin  =0
    nu1   =0
    nu2   =0
  elseif (par%N_radband == 2) then
    call opta_Info(nbin=nbin)
    nu1   =1
    nu2   =nbin
  endif
  !nnu = nu2 - nu1 + 1
else
  ! --- Error! Invalid value for N_radband ---
  if (gl%nc_m >= 0) then
    write(gl%nc_m,'(A,I7)') 'ERROR> Invalid value for N_radband: ', par%N_radband
  endif
  stop
endif
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D arrays ---
allocate(S(               m1-nFg:n1+nFg, m2-nFg:n2+nFg, m3-nFg:n3+nFg), &
         dtauhalf(        m1-nFg:n1+nFg, m2-nFg:n2+nFg, m3-nFg:n3+nFg), &
         exp_mdtauhalf(   m1-nFg:n1+nFg, m2-nFg:n2+nFg, m3-nFg:n3+nFg), &
         expl2t_mdtauhalf(m1-nFg:n1+nFg, m2-nFg:n2+nFg, m3-nFg:n3+nFg))
!#
# endif
!#      /* (rhd_arrays_l01) */
!
if (nbin > 0) then
  ! --- Allocate additional arrays in case of non-grey opacities ---
  allocate(wTb(           m1-nFg:n1+nFg, m2-nFg:n2+nFg, m3-nFg:n3+nFg), &
           tau_func_total(m1    :n1    , m2    :n2    , m3    :n3    ))
endif
!
! --- Compute typical global grid-cell size: grid assumed to be equidistant ---
dx(:)=(geo%xb_r(:)-geo%xb_l(:)) / real(geo%n(:)-geo%m(:)+1)
!
! --- Compute grid step of local box and determine its variation ---
dxb1      =xb1(m1-nFg+1: n1+nFg+1) - xb1(m1-nFg  : n1+nFg  )
dxb_avg(1)=(xb1(n1+nFg+1) - xb1(m1-nFg  )) / real(n1-m1+1+2*nFg)
dxb_var   =maxval(abs(dxb1-dxb_avg(1))) / dxb_avg(1)
if (dxb_var > C_dxb1_maxvar) dxb_avg(1)=-1.0
!
dxb2      =xb2(m2-nFg+1: n2+nFg+1) - xb2(m2-nFg  : n2+nFg  )
dxb_avg(2)=(xb2(n2+nFg+1) - xb2(m2-nFg  )) / real(n2-m2+1+2*nFg)
dxb_var   =maxval(abs(dxb2-dxb_avg(2))) / dxb_avg(2)
if (dxb_var > C_dxb1_maxvar) dxb_avg(2)=-1.0
!
dxb3      =xb3(m3-nFg+1: n3+nFg+1) - xb3(m3-nFg  : n3+nFg  )
dxb_avg(3)=(xb3(n3+nFg+1) - xb3(m3-nFg  )) / real(n3-m3+1+2*nFg)
dxb_var   =maxval(abs(dxb3-dxb_avg(3))) / dxb_avg(3)
if (dxb_var > C_dxb1_maxvar) dxb_avg(3)=-1.0
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_start('SHC: formal: init: zeros')
!#
# endif
!#
!
!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2)
!
if (ComputeIntensity_flag) then
  if (ComputeFlux_flag) then
    ! --- Set fluxes to zero ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-nFb,n3+nFb
      do i2=m2-nFb,n2+nFb
        do i1=m1-nFb,n1+nFb+1
          Frad1(i1,i2,i3)=0.0
        end do ! i1
      end do ! i2
      do i2=m2-nFb,n2+nFb+1
        do i1=m1-nFb,n1+nFb
          Frad2(i1,i2,i3)=0.0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-nFb,n3+nFb+1
      do i2=m2-nFb,n2+nFb
        do i1=m1-nFb,n1+nFb
          Frad3(i1,i2,i3)=0.0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  endif ! (ComputeFlux_flag)
  !
  ! --- Initialize change in internal energy ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nFg,n3+nFg
    do i2=m2-nFg,n2+nFg
      do i1=m1-nFg,n1+nFg
        drhoeidtime(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif ! (ComputeIntensity_flag)
!
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
! --- Always set radiative force to zero (it might be partially computed even if not requested) ---
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-nFg,n3+nFg
  do i2=m2-nFg,n2+nFg
    do i1=m1-nFg,n1+nFg
      drhov1dtime(i1,i2,i3)=0.0
      drhov2dtime(i1,i2,i3)=0.0
      drhov3dtime(i1,i2,i3)=0.0
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!#
# endif
!#
!
!$OMP END PARALLEL
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_stop('SHC: formal: init: zeros')
call timing_start('SHC: formal: nuloop')
!#
# endif
!#
!
! ### Loop over all frequency bins #################################################################
NULOOP: do nu=nu1, nu2
  !
  if (gl%nc_m >= 0) then
    write(gl%nc_m,'(A,3I4,A)') 'Frequency index nu (nu1, nu2): ', nu, nu1, nu2, ' ================='
  endif
  !
!#
# ifdef rhd_shortrad_formal_t01
!#
  call timing_start('SHC: formal: nuloop: init')
!#
# endif
!#
  !
  ! === Compute opacities and source function ======================================================
  !
  ! --- Compare performance of various interpolation routines ---
  !call rhd_shortrad_OpTaTest(model, rho, T, logP, rhokappa, wTb, S, &
  !                           nu, ComputeIntensity_flag, Computekappa_flag)
  !
  call timing_start('SHC: formal: nuloop: init: OpTa')
  !
  call rhd_shortrad_OpTa(m1,n1, m2,n2, m3,n3, nquc, nu, &
                         ComputeIntensity_flag, Computekappa_flag, &
                         T, logP, rho, rhokappa, wTb, S, quc)
  !
  call timing_stop('SHC: formal: nuloop: init: OpTa')
  !
  ! === Initialize array ===========================================================================
  if (Computetaufunc_flag) then
    ! --- Initialize tau_func array, used for radiative time step estimate ---
    !
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          tau_func(i1,i2,i3)=0.0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
  endif ! (Computetaufunc_flag)
  !
!#
# ifdef rhd_shortrad_formal_t01
!#
  call timing_stop('SHC: formal: nuloop: init')
  call timing_start('SHC: formal: nuloop: dirloop')
!#
# endif /* --- rhd_shortrad_formal_t01 --- */
!#
  !
  ! === Loop over all directions ===================================================================
  do i_dir=1,n_dir
    ! === Perform radiation-transport step for single ray ===
    !
    ! --- Set direction and weight of current ray ---
    r_dir=ray_dir(:,i_dir)
    r_wei=ray_wei(i_dir)
    !
    ! --- Determine main ray direction                                       ---
    ! --- and normalize vector so that main directions coordinate is positiv ---
    if (((abs(r_dir(1)/dx_dir(1))) >= (abs(r_dir(2)/dx_dir(2)))) .and. &
        ((abs(r_dir(1)/dx_dir(1))) >= (abs(r_dir(3)/dx_dir(3))))) then
      dir=1
    else if (((abs(r_dir(2)/dx_dir(2))) >= (abs(r_dir(1)/dx_dir(1)))) .and. &
             ((abs(r_dir(2)/dx_dir(2))) >= (abs(r_dir(3)/dx_dir(3))))) then
      dir=2
    else
      dir=3
    endif
    r_dir=sign(1.0,r_dir(dir)) * r_dir
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I2,A,I2,A,I1,A,3F10.6,2X,F10.6,A)') &
        'Main (', i_dir, '/', n_dir, ') ray direction ', dir, ': ', r_dir, r_wei, ' -----------'
      call flush(gl%nc_m)
    endif
    !
    oneovermu=abs(1.0/r_dir(dir))
    !
    ! --- Check if computation of new values for delta optical depth is necessary ---
    if (i_dir <= 1) then
      Computeexptau_flag=.true.
    else
      Computeexptau_flag= (abs(oneovermu_last-oneovermu) > 2.0E-5)
    endif
    !
    oneovermu_last=oneovermu
    !
    if ((Computeexptau_flag) .or. (any(dxb_avg < 0.0))) then
      ! --- Compute new delta optical depth (otherwise reuse previous values) ---
!#
#     ifdef rhd_shortrad_formal_t01
!#
      call timing_start('SHC: formal: nuloop: dl: exp')
!#
#     endif
!#
      if (any(dxb_avg < 0.0)) then
        if (dir == 1) then
          ! --- Account for general grid: main direction 1 ---
          !
          !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, x, expmx, weight)
          !
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3-nFg,n3+nFg
            do i2=m2-nFg,n2+nFg
              do i1=m1-nFg,n1+nFg
                x                         =(oneovermu*0.5*dxb1(i1)) * rhokappa(i1,i2,i3)
                if (x < 80.0) then
                  expmx                     =exp(-x)
                  weight                    =(( 1.0 - (((expmx**2)**2)**2)**2 )**2)**2
                  expl2t_mdtauhalf(i1,i2,i3)=     weight * (expmx-(1.0-x)) + &
                                             (1.0-weight)* (((((-0.2             *x   ) + 1.0)* &
                                                               (-0.25            *x   ) + 1.0)* &
                                                               (-0.33333333333333*x   ) + 1.0)* &
                                                               ( 0.5             *x**2) &
                                                           )
                else
                  expmx                     =0.0
                  expl2t_mdtauhalf(i1,i2,i3)=-(1.0-x)
                endif
                dtauhalf(i1,i2,i3)        =x
                exp_mdtauhalf(i1,i2,i3)   =expmx
              end do ! i1
            end do ! i2
          end do ! i3
          !$OMP END DO
          !
          !$OMP END PARALLEL
          !
        else if (dir == 2) then
          ! --- Account for general grid: main direction 2 ---
          !
          !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, x, expmx, weight)
          !
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3-nFg,n3+nFg
            do i2=m2-nFg,n2+nFg
              do i1=m1-nFg,n1+nFg
                x                         =(oneovermu*0.5*dxb2(i2)) * rhokappa(i1,i2,i3)
                if (x < 80.0) then
                  expmx                     =exp(-x)
                  weight                    =(( 1.0 - (((expmx**2)**2)**2)**2 )**2)**2
                  expl2t_mdtauhalf(i1,i2,i3)=     weight * (expmx-(1.0-x)) + &
                                             (1.0-weight)* (((((-0.2             *x   ) + 1.0)* &
                                                               (-0.25            *x   ) + 1.0)* &
                                                               (-0.33333333333333*x   ) + 1.0)* &
                                                               ( 0.5             *x**2) &
                                                           )
                else
                  expmx                     =0.0
                  expl2t_mdtauhalf(i1,i2,i3)=-(1.0-x)
                endif
                dtauhalf(i1,i2,i3)        =x
                exp_mdtauhalf(i1,i2,i3)   =expmx
              end do ! i1
            end do ! i2
          end do ! i3
          !$OMP END DO
          !
          !$OMP END PARALLEL
          !
        else ! (dir == 3)
          ! --- Account for general grid: main direction 3 ---
          !
          !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, x, expmx, weight)
          !
          !$OMP DO SCHEDULE(RUNTIME)
          do i3=m3-nFg,n3+nFg
            do i2=m2-nFg,n2+nFg
              do i1=m1-nFg,n1+nFg
                x                         =(oneovermu*0.5*dxb3(i3)) * rhokappa(i1,i2,i3)
                if (x < 80.0) then
                  expmx                     =exp(-x)
                  weight                    =(( 1.0 - (((expmx**2)**2)**2)**2 )**2)**2
                  expl2t_mdtauhalf(i1,i2,i3)=     weight * (expmx-(1.0-x)) + &
                                             (1.0-weight)* (((((-0.2             *x   ) + 1.0)* &
                                                               (-0.25            *x   ) + 1.0)* &
                                                               (-0.33333333333333*x   ) + 1.0)* &
                                                               ( 0.5             *x**2) &
                                                           )
                else
                  expmx                     =0.0
                  expl2t_mdtauhalf(i1,i2,i3)=-(1.0-x)
                endif
                dtauhalf(i1,i2,i3)        =x
                exp_mdtauhalf(i1,i2,i3)   =expmx
              end do ! i1
            end do ! i2
          end do ! i3
          !$OMP END DO
          !
          !$OMP END PARALLEL
          !
        endif ! (dir)
        !
      else ! (dxb_avg(dir) >= 0.0)
        !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, x, expmx, weight)
        !
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3-nFg,n3+nFg
          do i2=m2-nFg,n2+nFg
            do i1=m1-nFg,n1+nFg
              x                         =(oneovermu*0.5*dx(dir)) * rhokappa(i1,i2,i3)
              if (x < 80.0) then
                expmx                     =exp(-x)
                weight                    =(( 1.0 - (((expmx**2)**2)**2)**2 )**2)**2
                expl2t_mdtauhalf(i1,i2,i3)=     weight * (expmx-(1.0-x)) + &
                                           (1.0-weight)* (((((-0.2             *x   ) + 1.0)* &
                                                             (-0.25            *x   ) + 1.0)* &
                                                             (-0.33333333333333*x   ) + 1.0)* &
                                                             ( 0.5             *x**2) &
                                                         )
              else
                expmx                     =0.0
                expl2t_mdtauhalf(i1,i2,i3)=-(1.0-x)
              endif
              dtauhalf(i1,i2,i3)        =x
              exp_mdtauhalf(i1,i2,i3)   =expmx
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP END PARALLEL
      endif ! (dxb_avg(dir) < 0.0)
      !
!#
#     ifdef rhd_shortrad_formal_t01
!#
      call timing_stop('SHC: formal: nuloop: dl: exp')
!#
#     endif
!#
    endif ! (Computeexptau_flag)
    !
    ! --- Check if direction is compatible with side boundary conditions ---
    if ((dir /= 3) .and. (geo%bound_l(1) == bound_periodic)) then
      ! --- Severe error condition: exit gracefully ---
      outstr0='Ray direction incompatible with side boundary conditions'
      ierr0  =100
!#
#     ifdef rhd_shortrad_formal_t01
!#
      call timing_stop('SHC: formal: nuloop: dirloop')
!#
#     endif
!#
      exit NULOOP
    endif
    !
    ! --- Radiative transfer according to main ray direction ---
    if (ComputeIntensity_flag) then
!!$!#
!!$#     ifdef _OPENMP
!!$!#
!!$      !write(6,*) 'SHC: formal: nuloop: dirloop: dirs'
!!$      !call flush(6)
!!$      omp_MaxThreads=omp_get_max_threads()
!!$      !write(6,*) 'omp_get_max_threads():', omp_MaxThreads
!!$      !call flush(6)
!!$      omp_NumThreads=max(omp_MaxThreads/max(par%N_test5, 1), 1)
!!$      !write(6,*) 'omp_NumThreads       :', omp_NumThreads
!!$      !call flush(6)
!!$      call omp_set_num_threads(omp_NumThreads)
!!$      !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$      !call flush(6)
!!$!#
!!$#     endif /* _OPENMP */
!!$!#
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      ! --- Determine if one or two calls to integration routines are necessary ---
      nDirCall=max(geo%nProcsPD(dir), 2)
      !
      ! --- Set the first integration direction ---
      call MPI_Cart_Coords(geo%Comm, geo%MyProc, 3, PCoords, ierrMPI)
      DownFirst_flag=(PCoords(dir) > 0)
      ! write(gl%nc_p, '(A,I3,1X,I3,A,I3,A,L1)') &
      !   '#', geo%MyProc, geo%MaProcWoInCo, ' PCoords=', PCoords(dir), &
      !        ' DownFirst_flag=', DownFirst_flag
!#
#     else
!#         /* MPI not activated ------------------------------ */
      ! --- Onle a single call to integration routines is necessary ---
      nDirCall=1
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      do iDirCall=1,nDirCall
        !
        ! --- Determine for which orientation the intensity has to be computed ---
        ComputeIntDown_flag=((nDirCall == 1) .or. (DownFirst_flag .eqv.  (iDirCall == 1)))
        ComputeIntUp_flag  =((nDirCall == 1) .or. (DownFirst_flag .neqv. (iDirCall == 1)))
        !
        if (dir == 1) then
          ! --- Update fluxes and change of energy and momentum for direction 1 ---
!#
#         ifdef rhd_shortrad_formal_t01
!#
          call timing_start('SHC: formal: nuloop: dl: dir1')
!#
#         endif
!#
          call rhd_shortrad_Dir1(geo, action, &
                                 ComputeIntDown_flag, ComputeIntUp_flag, &
                                 ComputeFlux_flag, ComputePressure_flag, &
                                 r_dir, r_wei, n_dir, &
                                 m1,n1, m2,n2, m3,n3, &
                                 xb1, xb2, xb3, &
                                 S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                 Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                                if (rhd_shortrad_radpressure_l01 != 0)
!#
                                 drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                                endif
!#
                                 RequestSendRecvMPI, I1Send, I1Recv &
                                 )
!#
#         ifdef rhd_shortrad_formal_t01
!#
          call timing_stop('SHC: formal: nuloop: dl: dir1', cycles=2-iDirCall)
!#
#         endif
!#
        else if (dir == 2) then
          ! --- Update fluxes and change of energy and momentum for direction 2 ---
!#
#         ifdef rhd_shortrad_formal_t01
!#
          call timing_start('SHC: formal: nuloop: dl: dir2')
!#
#         endif
!#
          call rhd_shortrad_Dir2(geo, action, &
                                 ComputeIntDown_flag, ComputeIntUp_flag, &
                                 ComputeFlux_flag, ComputePressure_flag, &
                                 r_dir, r_wei, n_dir, &
                                 m1,n1, m2,n2, m3,n3, &
                                 xb1, xb2, xb3, &
                                 S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                 Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                                if (rhd_shortrad_radpressure_l01 != 0)
!#
                                 drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                                endif
!#
                                 RequestSendRecvMPI, I2Send, I2Recv &
                                 )
!#
#         ifdef rhd_shortrad_formal_t01
!#
          call timing_stop('SHC: formal: nuloop: dl: dir2', cycles=2-iDirCall)
!#
#         endif
!#
        else ! (dir == 3)
          ! --- Update fluxes and change of energy and momentum for direction 3 ---
!#
#         ifdef rhd_shortrad_formal_t01
!#
          call timing_start('SHC: formal: nuloop: dl: dir3')
!#
#         endif
!#
          call rhd_shortrad_Dir3(geo, action, &
                                 ComputeIntDown_flag, ComputeIntUp_flag, &
                                 ComputeFlux_flag, ComputePressure_flag, &
                                 r_dir, r_wei, n_dir, &
                                 m1,n1, m2,n2, m3,n3, &
                                 xb1, xb2, xb3, &
                                 S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                 Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                                if (rhd_shortrad_radpressure_l01 != 0)
!#
                                 drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                                endif
!#
                                 RequestSendRecvMPI, I3Send, I3Recv &
                                 )
!#
#         ifdef rhd_shortrad_formal_t01
!#
          call timing_stop('SHC: formal: nuloop: dl: dir3', cycles=2-iDirCall)
!#
#         endif
!#
        endif ! (dir)
        !
!#
#       if (rhd_mpi_l01 == 1)
!#                            /* MPI activated ----------------- */
        if ((.not. (ComputeIntDown_flag .or. ComputeIntUp_flag)) .and. &
            (RequestSendRecvMPI(1) /= MPI_REQUEST_NULL)                       ) then
          call timing_start('SHC: formal Send Wait')
          ! --- Wait for open send request (because no routine was called that would have done it) -
          call MPI_Wait(RequestSendRecvMPI(1), StatusMPI, ierrMPI)
          call timing_stop( 'SHC: formal Send Wait')
        endif ! (RequestSendRecvMPI)
!#
#       endif
!#            /* rhd_mpi_l01 ----------------------------------- */
        !
      end do ! iDirCall
    endif ! (ComputeIntensity_flag)
    !
    ! --- Compute optical-depth factor for radiative time scale ---
    if (Computetaufunc_flag) then
      if (dir == 1) then
        ! --- Add contribution from direction 1 ---
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_start('SHC: formal: nuloop: dl: time1')
!#
#       endif
!#
        call rhd_shortrad_tauCouple1(r_dir, r_wei, n_dir, &
                                     m1,n1, m2,n2, m3,n3, &
                                     xb1, xb2, xb3, &
                                     dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                     tau_func)
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_stop('SHC: formal: nuloop: dl: time1')
!#
#       endif
!#
      else if (dir == 2) then
        ! --- Add contribution from direction 2 ---
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_start('SHC: formal: nuloop: dl: time2')
!#
#       endif
!#
        call rhd_shortrad_tauCouple2(r_dir, r_wei, n_dir, &
                                     m1,n1, m2,n2, m3,n3, &
                                     xb1, xb2, xb3, &
                                     dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                     tau_func)
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_stop('SHC: formal: nuloop: dl: time2')
!#
#       endif
!#
      else ! (dir == 3)
        ! --- Add contribution from direction 3 ---
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_start('SHC: formal: nuloop: dl: time3')
!#
#       endif
!#
        call rhd_shortrad_tauCouple3(r_dir, r_wei, n_dir, &
                                     m1,n1, m2,n2, m3,n3, &
                                     xb1, xb2, xb3, &
                                     dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf, &
                                     tau_func)
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_stop('SHC: formal: nuloop: dl: time3')
!#
#       endif
!#
      endif ! (dir)
      !
    endif ! (Computetaufunc_flag)
    !
!!$!#
!!$#   ifdef _OPENMP
!!$!#
!!$    !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$    call omp_set_num_threads(omp_MaxThreads)
!!$    !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$!#
!!$#   endif /* _OPENMP */
!!$!#
  end do ! i_dir
  ! ================================================================================================
!#
# ifdef rhd_shortrad_formal_t01
!#
  call timing_stop('SHC: formal: nuloop: dirloop')
!#
# endif
!#
  !
  if (Computetaufunc_flag) then
    if (nu > 0) then
      ! --- Add tau_func to tau_func_total ---
      !
!#
#     ifdef rhd_shortrad_formal_t01
!#
      call timing_start('SHC: formal: nuloop: taufunc')
!#
#     endif
!#
      !
      !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, i3)
      !
      if (nu == nu1) then
        ! --- First bin: just copy the weighted contribution ---
        !
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3,n3
          do i2=m2,n2
            do i1=m1,n1
              tau_func_total(i1,i2,i3)=wTb(i1,i2,i3)*tau_func(i1,i2,i3)
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
      else if (nu < nu2) then
        ! --- Inner bin: add the weighted contribution ---
        !
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3,n3
          do i2=m2,n2
            do i1=m1,n1
              tau_func_total(i1,i2,i3)=tau_func_total(i1,i2,i3) + wTb(i1,i2,i3)*tau_func(i1,i2,i3)
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
      else
        ! --- Last bin: add the weighted contribution and put result into tau_func ---
        !
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3,n3
          do i2=m2,n2
            do i1=m1,n1
              tau_func(i1,i2,i3)=tau_func_total(i1,i2,i3) + wTb(i1,i2,i3)*tau_func(i1,i2,i3)
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
      endif
      !
      !$OMP END PARALLEL
      !
!#
#     ifdef rhd_shortrad_formal_t01
!#
      call timing_stop('SHC: formal: nuloop: taufunc')
!#
#     endif
!#
      !
    endif ! (nu)
  endif ! (Computetaufunc_flag)
  !
end do NULOOP ! nu
! ##################################################################################################
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_stop('SHC: formal: nuloop')
!#
# endif
!#
!
if (Output_flag .and. Computekappa_flag .and. (nu2 > 0)) then
  ! --- Compute grey opacities ---
  !
!#
# ifdef rhd_shortrad_formal_t01
!#
  call timing_start('SHC: formal: opagrey')
!#
# endif
!#
  !$OMP PARALLEL DEFAULT(SHARED)
  !
  call sub_xkaros_logPT2rhokap(logP, T, rho, 0, rhokappa)
  !
  !$OMP END PARALLEL
!#
# ifdef rhd_shortrad_formal_t01
!#
  call timing_stop('SHC: formal: opagrey')
!#
# endif
!#
endif ! (Output_flag .and. Computekappa_flag .and. (nu2 > 0))
!
!!$      ! --- Change in internal energy from flux divergence (for tests) ---
!!$      do i3=m3,n3
!!$        do i2=m2,n2
!!$          drhoeidtime(m1:n1,i2,i3)=-( &
!!$                                     (Frad1(m1+1:n1+1,i2  ,i3  )-Frad1(m1:n1,i2,i3))/ &
!!$                                     (xb1(  m1+1:n1+1          )-xb1(  m1:n1      )) + &
!!$                                     (Frad2(m1  :n1  ,i2+1,i3  )-Frad2(m1:n1,i2,i3))/ &
!!$                                     (xb2(            i2+1     )-xb2(        i2   )) + &
!!$                                     (Frad3(m1  :n1  ,i2  ,i3+1)-Frad3(m1:n1,i2,i3))/ &
!!$                                     (xb3(                 i3+1)-xb3(           i3))    )
!!$        end do ! i2
!!$      end do ! i3
!
!#
# ifdef rhd_shortrad_formal_t01
!#
call timing_stop('SHC: formal')
!#
# endif
!#
!!$call uio_openwr(nc, 'rhd_short_formal_test.out', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now' )
!!$call uio_wrlabl(nc, box_ident                 )
!!$call uio_wr(nc, 'z'          , box_id_ident   )
!!$call uio_wr(nc, 0.5*(xb1(m1:n1)+xb1(m1+1:n1+1)), 'xc1'          )
!!$call uio_wr(nc, xb1          , 'xb1'          )
!!$call uio_wr(nc, 0.5*(xb2(m2:n2)+xb2(m2+1:n2+1)), 'xc2'          )
!!$call uio_wr(nc, xb2          , 'xb2'          )
!!$call uio_wr(nc, 0.5*(xb3(m3:n3)+xb3(m3+1:n3+1)), 'xc3'          )
!!$call uio_wr(nc, xb3          , 'xb3'          )
!!$call uio_wr(nc, rho          , 'rho'          )
!!$call uio_wr(nc, logP         , 'logp'         )
!!$call uio_wr(nc, T            , 't'            )
!!$call uio_wr(nc, rhokappa     , 'rhokappa'     )
!!$call uio_wr(nc, dtauhalf     , 'dtauhalf'     )
!!$call uio_wr(nc, Frad1        , 'frad1'        )
!!$call uio_wr(nc, Frad2        , 'frad2'        )
!!$call uio_wr(nc, Frad3        , 'frad3'        )
!!$call uio_wr(nc, drhoeidtime  , 'drhoeidtime'  )
!!$call uio_wr(nc, drhov1dtime  , 'drhov1dtime'  )
!!$call uio_wr(nc, drhov2dtime  , 'drhov2dtime'  )
!!$call uio_wr(nc, drhov3dtime  , 'drhov3dtime'  )
!!$call uio_wrlabl(nc, endbox_ident)
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call uio_closrd(nc)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(S, dtauhalf, exp_mdtauhalf, expl2t_mdtauhalf)
!#
# endif
!#      /* (rhd_arrays_l01) */
if (nbin > 0) then
  ! --- Allocate additional array in case of non-grey opacities ---
  deallocate(wTb, tau_func_total)
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine rhd_shortrad_Formal


!----------***********************------------------------------------------------------------------
subroutine rhd_shortrad_StepOutput(model, geo, m1,n1, m2,n2, m3,n3, n_subdtime, &
                                   OutIntensity_flag, OutFlux_flag, ComputePressure_flag, &
                                   dtimerel, xc1, xc2, xc3, &
                                   rhokappa, T, drhoeidtime, Frad1, Frad2, Frad3, &
!#
#                                  if (rhd_shortrad_radpressure_l01 != 0)
!#
                                   g1, g2, g3, &
!#
#                                  endif
!#
                                   dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_StepOutput ('rhd_short-characteristics_radiation_Step_Output')
!
! PURPOSE:
!   Derive some averages as additional output quantities.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_StepOutput(model, geo, m1,n1, m2,n2, m3,n3, &
!                                OutIntensity_flag, OutFlux_flag, ComputePressure_flag, &
!                                dtimerel, xc1, xc2, xc3, &
!                                rhokappa, T, drhoeidtime, Frad1, Frad2, Frad3, &
!                                g1, g2, g3, &
!                                dat)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   n_subdtime:   ('number_sub_delta_time') integer, number of sub time step, 1=first
!   OutIntensity_flag: ('Output_Intensity_flag') logical, if true the output intensities are
!                      computed, otherwise output fluxes etc.
!   OutFlux_flag: ('Output_Flux_flag') logical, if true (and OutIntensity_flag=.false.) the
!                 surface flux arrays and the various averages of the radiative energy are computed
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   dtimerel:     ('delta_time_relative') real, fraction of current sub time step of full time step
!   xc1:          ('x_center_1') real, 1. coordinate of cell centers
!   xc2:          ('x_center_2') real, 2. coordinate of cell centers
!   xc3:          ('x_center_3') real, 3. coordinate of cell centers
!   rhokappa:     ('rho_kappa') real 3D array with opacities
!   T:            ('Temperature') real 3D array with temperature
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time) real 3D array with change of
!                 internal energy per time due to radiation
!   g1:           ('g_1') real 3D array with radiative acceleration, x1 d. (if ComputePressure_flag)
!   g2:           ('g_2') real 3D array with radiative acceleration, x2 d. (if ComputePressure_flag)
!   g3:           ('g_3') real 3D array with radiative acceleration, x3 d. (if ComputePressure_flag)
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!
! INPUT/OUTPUT:
!   dat:          ('data') derived type with additional output data
!
! LOCAL VARIABLES:
!
! TODO:
!   Account for possibly smaller radiation-transport box for azimuthal averages.
!
! ROUTINES:
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!   rhd_datc_Add:          Add the fields in the structure for cylindrical mean data
!   rhd_datc_Delete:       Delete structure for cylindrical mean data. Deallocate memory
!   rhd_datc_Make:         Create structure of requested size for 2D cylindrical mean data
!   rhd_datx_Add:          Add the fields in the structure for additional data
!   rhd_datx_Delete:       Delete structure for additional data. Deallocate memory
!   rhd_datx_Make:         Create structure for data of requested size
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_c1Index:      Produce index array for azimuthal averaging over cylinders
!   rhd_mean_c1VecProj:    Project vector on radial vector perpendicular to x1 axis
!   rhd_mean_ccamean:      Average cell-centered quantity over cylinders (azimuthal mean)
!   rhd_mean_rb23mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_rcmean:       Average cell-centered quantity over spheres in a symmetric box
!   rhd_mean_rindex:       Produce index arrays for averaging over spheres in a symmetric box
!   rhd_mean_VecProj:      Project vector on radial ray
!   rhd_mean_weight:       Produce weight array for horizontal averaging
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Compute some averages for requested quantities
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-18 (B.F. Copenhagen) Put into own routine
!   2001-07-02 (B.F.) Temperature averages included
!   2004-03-02 (B.F. Uppsala) Temporary array Frad2_tmp used because of an obscure Sun error
!   2006-10-24 (B.F. Lyon) Dimension of arrays kappa and T increased to save temporary copies
!   2012-07-27 (B.F.) Move scale%length to prop%length
!   2013-08-22 (B.F.) Add azimuthal averages of the cell-centered radiative energy flux
!   2014-08-17 (B.F. Uppsala) Add detailed timing information
!   2014-12-07 (B.F.) New parameters: m1,... n3
!   2017-08-12 (B.F.) Check if fer array is associated before computing spherical means
!   2017-08-14 (B.F.) Adapt averaging calls to new datx type
!   2017-08-18 (B.F. Lyon) Take axes in new datx structure from a a separate array
!   2017-08-25 (B.F.) Compute AreaFraction from local and global quantities
!   2017-09-07 (B.F.) Change to averaging routines for asymmetric boxes for new datx type;
!                     Add xc1, xc2, xc3 as input parameters
!   2017-09-19 (B.F. Uppsala) Add OpenMP directives
!   2017-10-25 (B.F.) Make the number of ghost cells flexible
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!   2018-11-29 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!   2019-01-28 (B.F.) Construct weight for horizontal means for wg%hyd geometry
!   2019-02-04 (B.F.) Add parameter OutFlux_flag to control output of radiative flux data
!   2019-02-06 (B.F.) Compute averages of radiative change of internal energy
!   2019-02-09 (B.F.) Add parameter n_subdtime; compute relsurfrad
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_dat_module
use rhd_box_module
use rhd_mean_module
!#
#   ifdef rhd_shortrad_step_t01
!#
use timing_module
!#
#   endif
!#
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFg=1
type(box_type),            intent(in)   :: model
type(geo_type),            intent(in)   :: geo
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, n_subdtime
logical,                   intent(in)   :: OutIntensity_flag, Outflux_flag, ComputePressure_flag
real,                      intent(in)   :: dtimerel
real, dimension(m1-nFg:n1+nFg), &
                           intent(in)   :: xc1
real, dimension(m2-nFg:n2+nFg), &
                           intent(in)   :: xc2
real, dimension(m3-nFg:n3+nFg), &
                           intent(in)   :: xc3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(in)   :: rhokappa, T, drhoeidtime
integer, parameter                      :: nFb=1
real, dimension(m1-nFb:n1+nFb+1, &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb     ), &
                           intent(in)   :: Frad1
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb+1, &
                m3-nFb:n3+nFb     ), &
                           intent(in)   :: Frad2
real, dimension(m1-nFb:n1+nFb  , &
                m2-nFb:n2+nFb  , &
                m3-nFb:n3+nFb+1   ), &
                           intent(in)   :: Frad3
type(dat_type),            intent(inout):: dat
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(m1  :n1, &
                m2  :n2, &
                m3  :n3   ), &
                           intent(in)   :: g1, g2, g3
!#
# endif
!#
!
! --- Local variables ---
integer                                 :: i1, i2, i3, i, m, n
real                                    :: AreaFraction
real   , dimension(  :,:), pointer      :: weight
real   , dimension(:,:,:), pointer      :: x123weight, kapparho_trans, T_trans, drhoeidtime_trans, &
                                           Frad1_trans, Frad2_trans, Frad3_trans, &
                                           g3_trans, gr, Frad2_tmp, Fr, Fa
integer, dimension(  :,:), pointer      :: nbindex, ccindex
integer, dimension(:,:,:), pointer      :: bindex, cindex, nb6index
type(box_type)                          :: slice
type(datx_type)                         :: datx_thread, datr_thread
type(datc_type)                         :: datc_thread
!---------------------------------------------------------------------------------------------------
!
if (OutIntensity_flag) then
  ! === Update just the output intensities with data from a special SHORTrad step ===
  !
  ! --- Update surface intensities ---
  if (associated(dat%int1_l)) then
    dat%int1_l=dat%int1_l + dtimerel*Frad1(m1     , m2:n2  , m3:n3  )
  endif
  if (associated(dat%int1_r)) then
    dat%int1_r=dat%int1_r + dtimerel*Frad1(   n1+1, m2:n2  , m3:n3  )
  endif
  if (associated(dat%int2_l)) then
    dat%int2_l=dat%int2_l + dtimerel*Frad2(m1:n1  , m2     , m3:n3  )
  endif
  if (associated(dat%int2_r)) then
    dat%int2_r=dat%int2_r + dtimerel*Frad2(m1:n1  ,    n2+1, m3:n3  )
  endif
  if (associated(dat%int3_l)) then
    dat%int3_l=dat%int3_l + dtimerel*Frad3(m1:n1  , m2:n2  , m3     )
  endif
  if (associated(dat%int3_r)) then
    dat%int3_r=dat%int3_r + dtimerel*Frad3(m1:n1  , m2:n2  ,    n3+1)
  endif
else ! (.not. OutIntensity_flag)
  ! === Average/update all quantities except intensities from a standard SHORTrad step ===
  !
  if (OutFlux_flag) then
    ! --- Update surface fluxes ---
    if (associated(dat%fer1_l)) then
      dat%fer1_l=dat%fer1_l + dtimerel*Frad1(m1     , m2:n2  , m3:n3  )
      !write(gl%nc_p, '(A,I3,A,A,4(2E11.4,1X))') '#', geo%MyProc, ' StepOutput dat ', &
      !  'minmax:fer1_l: ', minval(dat%fer1_l), maxval(dat%fer1_l), &
      !                     minval(Frad1(m1     , m2:n2  , m3:n3  )), &
      !                     maxval(Frad1(m1     , m2:n2  , m3:n3  ))
    endif
    if (associated(dat%fer1_r)) then
      dat%fer1_r=dat%fer1_r + dtimerel*Frad1(   n1+1, m2:n2  , m3:n3  )
      !write(gl%nc_p, '(A,I3,A,A,4(2E11.4,1X))') '#', geo%MyProc, ' StepOutput dat ', &
      !  'minmax:fer1_r: ', minval(dat%fer1_r), maxval(dat%fer1_r), &
      !                     minval(Frad1(   n1+1, m2:n2  , m3:n3  )), &
      !                     maxval(Frad1(   n1+1, m2:n2  , m3:n3  ))
    endif
    if (associated(dat%fer2_l)) then
      dat%fer2_l=dat%fer2_l + dtimerel*Frad2(m1:n1  , m2     , m3:n3  )
      !write(gl%nc_p, '(A,I3,A,A,2(2E11.4,1X))') '#', geo%MyProc, ' StepOutput dat ', &
      !  'minmax:fer2_l: ', minval(dat%fer2_l), maxval(dat%fer2_l)
    endif
    if (associated(dat%fer2_r)) then
      dat%fer2_r=dat%fer2_r + dtimerel*Frad2(m1:n1  ,    n2+1, m3:n3  )
      !write(gl%nc_p, '(A,I3,A,A,2(2E11.4,1X))') '#', geo%MyProc, ' StepOutput dat ', &
      !  'minmax:fer2_r: ', minval(dat%fer2_r), maxval(dat%fer2_r)
    endif
    if (associated(dat%fer3_l)) then
      dat%fer3_l=dat%fer3_l + dtimerel*Frad3(m1:n1  , m2:n2  , m3     )
      !write(gl%nc_p, '(A,I3,A,A,2(2E11.4,1X))') '#', geo%MyProc, ' StepOutput dat ', &
      !  'minmax:fer3_l: ', minval(dat%fer3_l), maxval(dat%fer3_l)
    endif
    if (associated(dat%fer3_r)) then
      dat%fer3_r=dat%fer3_r + dtimerel*Frad3(m1:n1  , m2:n2  ,    n3+1)
      !write(gl%nc_p, '(A,I3,A,A,2(2E11.4,1X))') '#', geo%MyProc, ' StepOutput dat ', &
      !  'minmax:fer3_r: ', minval(dat%fer3_r), maxval(dat%fer3_r)
    endif
  endif ! (OutFlux_flag)
  !
  if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
    ! --- Make abbreviations ---
    m=dat%r%m
    n=dat%r%n
  endif
  !
  if (dat%x23%na > 0) then
    ! === Average over x2-x3 planes and spherical surfaces =========================================
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_start('SHC: step: dtime: final(outnew1)')
!#
#   endif
!#
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(datx_thread, datr_thread, weight, slice, AreaFraction, i, &
    !$OMP           x123weight, cindex, bindex, nb6index, &
    !$OMP           gr)
    !
    call rhd_datx_Make(datx_thread, dat%x23%m, dat%x23%n, dat%x23%nWave)
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Make(datr_thread, dat%r%m, dat%r%n, dat%r%nWave, radial_flag=.true.)
    endif
    !
    !$OMP DO
    do i2=m2,n2
      ! --- Allocate temporary arrays ---
      allocate(weight(           i2:i2, m3:n3))
      !
      ! --- Create temporary box (only with axes filled) ---
      call rhd_box_Make(slice, m1=m1, n1=n1, m2=i2, n2=i2, m3=m3, n3=n3)
      call rhd_box_Copy(model, m1, n1, i2, i2, m3, n3, &
                        slice, m1,     i2,     m3, &
                        trans_mode=1, bulk_mode=0)
      !
      ! --- Compute weights for horizontal averaging ---
      AreaFraction=dtimerel * &
                   ((model%xb2(i2+1)-model%xb2(i2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                   ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
      call rhd_mean_weight(slice, AreaFraction, weight)
      !
      ! --- Average horizontally ---
      i=dat%x23%i_kapparho
      if (i > 0) &
        call rhd_mean_x23mean(rhokappa(m1:n1  ,i2:i2,m3:n3)   , datx_thread%d(m1:n1  ,i), weight)
      i=dat%x23%i_T
      if (i > 0) &
        call rhd_mean_x23mean(T(       m1:n1  ,i2:i2,m3:n3)   , datx_thread%d(m1:n1  ,i), weight)
      i=dat%x23%i_T2
      if (i > 0) &
        call rhd_mean_x23mean(T(       m1:n1  ,i2:i2,m3:n3)**2, datx_thread%d(m1:n1  ,i), weight)
      i=dat%x23%i_T4
      if (i > 0) &
        call rhd_mean_x23mean(T(       m1:n1  ,i2:i2,m3:n3)**4, datx_thread%d(m1:n1  ,i), weight)
      i=dat%x23%i_T4kaprho
      if (i > 0) &
        call rhd_mean_x23mean(T(       m1:n1  ,i2:i2,m3:n3)**4 * &
                              rhokappa(m1:n1  ,i2:i2,m3:n3)   , datx_thread%d(m1:n1  ,i), weight)
      i=dat%x23%i_der
      if (i > 0) &
        call rhd_mean_x23mean(drhoeidtime(m1:n1  ,i2:i2,m3:n3), datx_thread%d(m1:n1  ,i), weight)
      i=dat%x23%i_fer
      if (Outflux_flag .and. (i > 0)) &
        call rhd_mean_x23mean(Frad1(   m1:n1+1,i2:i2,m3:n3)   , datx_thread%d(m1:n1+1,i), weight)
      !
!#
#     if (rhd_shortrad_radpressure_l01 != 0)
!#
      if (ComputePressure_flag) then
        i=dat%x23%i_grad
        if (i > 0) &
          call rhd_mean_x23mean(g1(    m1:n1  ,i2:i2,m3:n3)   , datx_thread%d(m1:n1  ,i), weight)
      endif
!#
#     endif
!#
      !
      if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
        ! === Average over spheres ===
        !
        ! --- Allocate index and gravity array ---
        allocate(x123weight(  m1:n1     , i2:i2, m3:n3), &
                 cindex(      m1:n1     , i2:i2, m3:n3), &
                 bindex(-(n-m+2):(n-m+2), i2:i2, m3:n3), &
                 nb6index(        6     , i2:i2, m3:n3), &
                 gr(          m1:n1     , i2:i2, m3:n3)   )
        !
        ! --- Compute weight arrays ---
        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtimerel)
        !
        ! --- Generate index arrays ---
        call rhd_mean_a2rindex(m1,n1, i2,i2, m3,n3, &
                               1, geo%m(1), geo%n(1), &
                               xc1(m1-1:n1+1), xc2(i2:i2), xc3(m3:n3), &
                               dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
        !
        ! --- Compute averages of bulk quantities over spheres ---
        i=dat%r%i_kapparho
        if (i > 0) &
          call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                 rhokappa(m1:n1,i2:i2,m3:n3),            datr_thread%d(m:n,i), &
                                 cindex, x123weight)
        i=dat%r%i_T
        if (i > 0) &
          call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                 T(m1:n1,i2:i2,m3:n3),                   datr_thread%d(m:n,i), &
                                 cindex, x123weight)
        i=dat%r%i_T2
        if (i > 0) &
          call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                 T(m1:n1,i2:i2,m3:n3)**2,                datr_thread%d(m:n,i), &
                                 cindex, x123weight)
        i=dat%r%i_T4
        if (i > 0) &
          call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                 T(m1:n1,i2:i2,m3:n3)**4,                datr_thread%d(m:n,i), &
                                 cindex, x123weight)
        i=dat%r%i_T4kaprho
        if (i > 0) &
          call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                 T(m1:n1,i2:i2,m3:n3)**4* &
                                 rhokappa(m1:n1,i2:i2,m3:n3),            datr_thread%d(m:n,i), &
                                 cindex, x123weight)
        i=dat%r%i_der
        if (i > 0) &
          call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                 drhoeidtime(m1:n1,i2:i2,m3:n3),         datr_thread%d(m:n,i), &
                                 cindex, x123weight)
        !
        ! --- Compute contribution of radiative energy flux to radial radiative energy flux ---
        if (Outflux_flag) then
          i=dat%r%i_fer
          if (i > 0) &
            call rhd_mean_rb23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                   Frad1(m1:n1+1,i2:i2,m3:n3),             datr_thread%d(:,i), &
                                   nb6index, bindex, weight)
          i=dat%r%i_relsurfrad
          if ((i > 0) .and. (n_subdtime == 1)) &
            call rhd_mean_rb23absmean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                      Frad1(m1:n1+1,i2:i2,m3:n3),          datr_thread%d(:,i), &
                                      nb6index, bindex, weight/dtimerel, unity=.true.)
        endif
!#
#       if (rhd_shortrad_radpressure_l01 != 0)
!#
        if (ComputePressure_flag) then
          i=dat%r%i_grad
          if (i > 0) then
            call rhd_mean_VecProj(slice, g1(:,i2:i2,:), g2(:,i2:i2,:), g3(:,i2:i2,:), gr)
            call rhd_mean_rc23mean(m1,n1,i2,i2,m3,n3, (n-m+2), &
                                   gr,                                   datr_thread%d(m:n,i), &
                                   cindex, x123weight)
          endif
        endif
!#
#       endif
!#
        !
        ! --- Deallocate temporary arrays ---
        deallocate(x123weight, cindex, bindex, nb6index, gr)
      endif ! ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0))
      !
      ! --- Delete temporary box ---
      call rhd_box_Delete(slice)
      !
      ! --- Deallocate temporary arrays ---
      deallocate(weight)
    end do ! i2
    !$OMP END DO
    !
    !$OMP CRITICAL (X23MEAN)
    call rhd_datx_Add(dat%x23, datx_thread)
    call rhd_datx_Delete(datx_thread)
    !
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Add(dat%r, datr_thread)
      call rhd_datx_Delete(datr_thread)
    endif
    !$OMP END CRITICAL (X23MEAN)
    !
    !$OMP END PARALLEL
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_stop('SHC: step: dtime: final(outnew1)')
!#
#   endif
!#
    !
  endif ! (datx23%na > 0)
  !
  if (dat%x13%na > 0) then
    ! === Average over x1-x3 planes and (Frad and grad only) over spherical surfaces ===============
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_start('SHC: step: dtime: final(outnew2)')
!#
#   endif
!#
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(datx_thread, datr_thread, weight, slice, AreaFraction, i, &
    !$OMP           x123weight, cindex, bindex, nb6index, &
    !$OMP           Frad2_tmp)
    !
    call rhd_datx_Make(datx_thread, dat%x13%m, dat%x13%n, dat%x13%nWave)
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Make(datr_thread, dat%r%m, dat%r%n, dat%r%nWave, radial_flag=.true.)
    endif
    !
    !$OMP DO
    do i1=m1,n1
      ! --- Allocate temporary arrays ---
      allocate(weight(             i1:i1, m3:n3  ), &
               Frad2_tmp( n2-m2+2, 1    , n3-m3+1)   )
      !
      ! --- Create temporary box (only with axes filled) ---
      call rhd_box_Make(slice, m1=m2, n1=n2, m2=i1, n2=i1, m3=m3, n3=n3)
      call rhd_box_Copy(model, i1, i1, m2, n2, m3, n3, &
                        slice, m2,     i1,     m3, &
                        trans_mode=2, bulk_mode=0)
      !
      ! --- Compute weights for horizontal averaging ---
      AreaFraction=dtimerel * &
                   ((model%xb1(i1+1)-model%xb1(i1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                   ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
      call rhd_mean_weight(slice, AreaFraction, weight)
      !
      ! --- Average horizontally ---
      i=dat%x13%i_kapparho
      if (i > 0) &
        call rhd_mean_x23mean(reshape(rhokappa(i1,m2:n2,m3:n3), &
                                      (/n2-m2+1,1,n3-m3+1/)), &
                              datx_thread%d(m2:n2  ,i), weight)
      i=dat%x13%i_T
      if (i > 0) &
        call rhd_mean_x23mean(reshape(T(i1,m2:n2,m3:n3), &
                                      (/n2-m2+1,1,n3-m3+1/)), &
                              datx_thread%d(m2:n2  ,i), weight)
      i=dat%x13%i_T2
      if (i > 0) &
        call rhd_mean_x23mean(reshape(T(i1,m2:n2,m3:n3)**2, &
                                      (/n2-m2+1,1,n3-m3+1/)), &
                              datx_thread%d(m2:n2  ,i), weight)
      i=dat%x13%i_T4
      if (i > 0) &
        call rhd_mean_x23mean(reshape(T(i1,m2:n2,m3:n3)**4, &
                                      (/n2-m2+1,1,n3-m3+1/)), &
                              datx_thread%d(m2:n2  ,i), weight)
      i=dat%x13%i_T4kaprho
      if (i > 0) &
        call rhd_mean_x23mean(reshape(T(i1,m2:n2,m3:n3)**4 * rhokappa(i1,m2:n2,m3:n3), &
                                      (/n2-m2+1,1,n3-m3+1/)), &
                              datx_thread%d(m2:n2  ,i), weight)
      i=dat%x13%i_der
      if (i > 0) &
        call rhd_mean_x23mean(reshape(drhoeidtime(i1,m2:n2,m3:n3), &
                                      (/n2-m2+1,1,n3-m3+1/)), &
                              datx_thread%d(m2:n2  ,i), weight)
      i=dat%x13%i_fer
      if (OutFlux_flag .and. (i > 0)) then
        Frad2_tmp=reshape(Frad2(i1,m2:n2+1,m3:n3), (/n2-m2+2,1,n3-m3+1/))
        call rhd_mean_x23mean(Frad2_tmp, &
                              datx_thread%d(m2:n2+1,i), weight)
      endif
      !
!#
#     if (rhd_shortrad_radpressure_l01 != 0)
!#
      if (ComputePressure_flag) then
        i=dat%x13%i_grad
        if (i > 0) &
          call rhd_mean_x23mean(reshape(g2(i1,m2:n2,m3:n3), (/n2-m2+1,1,n3-m3+1/)), &
                                datx_thread%d(m2:n2  ,i), weight)
      endif
!#
#     endif
!#
      !
      if (OutFlux_flag .and. (par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
        ! === Average over spheres ===
        !
        ! --- Allocate index array ---
        allocate(x123weight(  m2:n2     , i1:i1, m3:n3), &
                 cindex(      m2:n2     , i1:i1, m3:n3), &
                 bindex(-(n-m+2):(n-m+2), i1:i1, m3:n3), &
                 nb6index(       6      , i1:i1, m3:n3)   )
        !
        ! --- Compute weight arrays ---
        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtimerel)
        !
        ! --- Generate index arrays ---
        call rhd_mean_a2rindex(m2,n2, i1,i1, m3,n3, &
                               1, geo%m(2), geo%n(2), &
                               xc2(m2-1:n2+1), xc1(i1:i1), xc3(m3:n3), &
                               dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
        !
        ! --- Compute contribution of radiative energy flux to radial radiative energy flux ---
        i=dat%r%i_fer
        if (i > 0) &
          call rhd_mean_rb23mean(m2,n2,i1,i1,m3,n3, (n-m+2), &
                                 Frad2_tmp,                              datr_thread%d(:,i), &
                                 nb6index, bindex, weight)
        i=dat%r%i_relsurfrad
        if ((i > 0) .and. (n_subdtime == 1)) &
          call rhd_mean_rb23absmean(m2,n2,i1,i1,m3,n3, (n-m+2), &
                                    Frad2_tmp,                              datr_thread%d(:,i), &
                                    nb6index, bindex, weight/dtimerel, unity=.true.)
        !
        ! --- Deallocate temporary array ---
        deallocate(x123weight, cindex, bindex, nb6index)
      endif ! (OutFlux_flag .and. (par%grav_mode == 'central') .and. (dat%r%i_xb > 0))
      !
      ! --- Delete temporary box ---
      call rhd_box_Delete(slice)
      !
      ! --- Deallocate temporary arrays ---
      deallocate(weight, Frad2_tmp)
    end do ! i1
    !$OMP END DO
    !
    !$OMP CRITICAL (X13MEAN)
    call rhd_datx_Add(dat%x13, datx_thread)
    call rhd_datx_Delete(datx_thread)
    !
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Add(dat%r, datr_thread)
      call rhd_datx_Delete(datr_thread)
    endif
    !$OMP END CRITICAL (X13MEAN)
    !
    !$OMP END PARALLEL
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_stop('SHC: step: dtime: final(outnew2)')
!#
#   endif
!#
  endif ! (dat%x13%na > 0)
  !
  if (dat%x12%na > 0) then
    ! === Average over x1-x2 planes, (Frad and grad only) over spherical surfaces, =================
    ! === and azimuthally ===
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_start('SHC: step: dtime: final(outnew3)')
!#
#   endif
!#
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   PRIVATE(datx_thread, datr_thread, weight, slice, AreaFraction, i, &
    !$OMP           x123weight, cindex, bindex, nb6index, &
    !$OMP           kapparho_trans, T_trans, drhoeidtime_trans, Frad3_trans, g3_trans, &
    !$OMP           datc_thread, ccindex, Frad1_trans, Frad2_trans, Fr, Fa, i1, i3)
    !
    call rhd_datx_Make(datx_thread, dat%x12%m, dat%x12%n, dat%x12%nWave)
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Make(datr_thread, dat%r%m, dat%r%n, dat%r%nWave, radial_flag=.true.)
      if (associated(dat%c%d)) then
        call rhd_datc_Make(datc_thread, model, .false.)
      endif
    endif
    !
    !$OMP DO
    do i2=m2,n2
      ! --- Allocate temporary arrays ---
      allocate(weight(                    i2:i2, m1:n1), &
               kapparho_trans(   m3:n3  , i2:i2, m1:n1), &
               T_trans(          m3:n3  , i2:i2, m1:n1), &
               drhoeidtime_trans(m3:n3  , i2:i2, m1:n1), &
               Frad3_trans(      m3:n3+1, i2:i2, m1:n1), &
               g3_trans(         m3:n3  , i2:i2, m1:n1)   )
      !
      ! --- Create temporary box (only with axes filled) ---
      call rhd_box_Make(slice, m1=m3, n1=n3, m2=i2, n2=i2, m3=m1, n3=n1)
      call rhd_box_Copy(model, m1, n1, i2, i2, m3, n3, &
                        slice, m3,     i2,     m1, &
                        trans_mode=3, bulk_mode=0)
      !
      ! --- Compute weights for horizontal averaging ---
      AreaFraction=dtimerel * &
                   ((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                   ((model%xb2(i2+1)-model%xb2(i2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
      call rhd_mean_weight(slice, AreaFraction, weight)
      !
      ! --- Transpose some arrays so that averaging can be done over 2nd and 3rd dimension ---
      kapparho_trans(   :,i2,:)=transpose(rhokappa(   m1:n1,i2,m3:n3  ))
      T_trans(          :,i2,:)=transpose(T(          m1:n1,i2,m3:n3  ))
      drhoeidtime_trans(:,i2,:)=transpose(drhoeidtime(m1:n1,i2,m3:n3  ))
      if (OutFlux_flag) then
        Frad3_trans(    :,i2,:)=transpose(Frad3(      m1:n1,i2,m3:n3+1))
      endif
      !
      ! --- Average horizontally ---
      i=dat%x12%i_kapparho
      if (i > 0) &
        call rhd_mean_x23mean(kapparho_trans, datx_thread%d(m3:n3  ,i), weight)
      i=dat%x12%i_T
      if (i > 0) &
        call rhd_mean_x23mean(T_trans       , datx_thread%d(m3:n3  ,i), weight)
      i=dat%x12%i_T2
      if (i > 0) &
        call rhd_mean_x23mean(T_trans**2    , datx_thread%d(m3:n3  ,i), weight)
      i=dat%x12%i_T4
      if (i > 0) &
        call rhd_mean_x23mean(T_trans**4    , datx_thread%d(m3:n3  ,i), weight)
      i=dat%x12%i_T4kaprho
      if (i > 0) &
        call rhd_mean_x23mean(T_trans**4 * kapparho_trans, &
                                              datx_thread%d(m3:n3  ,i), weight)
      i=dat%x12%i_der
      if (i > 0) &
        call rhd_mean_x23mean(drhoeidtime_trans, &
                                              datx_thread%d(m3:n3  ,i), weight)
      i=dat%x12%i_fer
      if (OutFlux_flag .and. (i > 0)) &
        call rhd_mean_x23mean(Frad3_trans   , datx_thread%d(m3:n3+1,i), weight)
      !
!#
#     if (rhd_shortrad_radpressure_l01 != 0)
!#
      if (ComputePressure_flag) then
        i=dat%x12%i_grad
        if (i > 0) then
          g3_trans(:,i2,:)=transpose(g3(   m1:n1,i2,m3:n3  ))
          call rhd_mean_x23mean(g3_trans,     datx_thread%d(m3:n3  ,i), weight)
        endif
      endif
!#
#     endif
!#
      !
      if (OutFlux_flag .and. (par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
        ! === Average over spheres (and azimuthally) ===
        !
        ! --- Allocate index array ---
        allocate(x123weight(  m3:n3     , i2:i2, m1:n1), &
                 cindex(      m3:n3     , i2:i2, m1:n1), &
                 bindex(-(n-m+2):(n-m+2), i2:i2, m1:n1), &
                 nb6index(        6     , i2:i2, m1:n1)   )
        !
        ! --- Compute weight arrays ---
        call rhd_mean_x23weight(slice, prop%length, weight, x123weight, factor=dtimerel)
        !
        ! --- Generate index arrays ---
        call rhd_mean_a2rindex(m3,n3, i2,i2, m1,n1, &
                               1, geo%m(3), geo%n(3), &
                               xc3(m3-1:n3+1), xc2(i2:i2), xc1(m1:n1), &
                               dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
        !
        ! --- Compute contribution of radiative energy flux to radial radiative energy flux ---
        i=dat%r%i_fer
        if (i > 0) &
          call rhd_mean_rb23mean(m3,n3,i2,i2,m1,n1, (n-m+2), &
                                 Frad3_trans,                            datr_thread%d(:,i), &
                                 nb6index, bindex, weight)
        i=dat%r%i_relsurfrad
        if ((i > 0) .and. (n_subdtime == 1)) &
          call rhd_mean_rb23absmean(m3,n3,i2,i2,m1,n1, (n-m+2), &
                                    Frad3_trans,                            datr_thread%d(:,i), &
                                    nb6index, bindex, weight/dtimerel, unity=.true.)
        !
        if (associated(dat%c%d)) then
          ! === Average azimuthally ===
          !
          ! --- Allocate temporary arrays ---
          allocate(ccindex(          i2:i2,m1:n1), &
                   Frad1_trans(m3:n3,i2:i2,m1:n1), &
                   Frad2_trans(m3:n3,i2:i2,m1:n1), &
                   Fr(         m3:n3,i2:i2,m1:n1), &
                   Fa(         m3:n3,i2:i2,m1:n1)   )
          !
          ! --- Compute index array ---
          call rhd_mean_c1Index(slice, dat%c%d1(:,dat%c%i_xb), ccindex)
          !
          ! --- Compute cell-centered fluxes (average and transpose) ---
          do i1=m1,n1
            do i3=m3,n3
              Frad1_trans(i3,i2,i1)=0.5 * (Frad1(i1,i2,i3)+Frad1(i1+1,i2  ,i3))
              Frad2_trans(i3,i2,i1)=0.5 * (Frad2(i1,i2,i3)+Frad2(i1  ,i2+1,i3))
            end do ! i3
          end do ! i1
          !
          ! --- Project cell-centered fluxes ---
          call rhd_mean_c1VecProj(slice, Frad2_trans, Frad1_trans, Fr, Fa)
          !
          ! --- Average projected cell-centered fluxes ---
          call rhd_mean_ccamean(m3,n3,i2,i2,m1,n1, dat%c%m1,dat%c%n1,dat%c%m2,dat%c%n2,dat%c%na, &
                 ccindex, x123weight, datc_thread%d, dat%c%i_fer1, &
                 Fr)
          call rhd_mean_ccamean(m3,n3,i2,i2,m1,n1, dat%c%m1,dat%c%n1,dat%c%m2,dat%c%n2,dat%c%na, &
                 ccindex, x123weight, datc_thread%d, dat%c%i_fer2, &
                 Fa)
          call rhd_mean_ccamean(m3,n3,i2,i2,m1,n1, dat%c%m1,dat%c%n1,dat%c%m2,dat%c%n2,dat%c%na, &
                 ccindex, x123weight, datc_thread%d, dat%c%i_fer3, &
                 0.5 * (Frad3_trans(m3:n3,i2,m1:n1)+Frad3_trans(m3+1:n3+1,i2,m1:n1)))
          !
          ! --- Deallocate temporary arrays ---
          deallocate(ccindex, Frad1_trans, Frad2_trans, Fr, Fa)
        endif ! (associated(dat%c%d))
        !
        ! --- Deallocate temporary array ---
        deallocate(x123weight, cindex, bindex, nb6index)
      endif ! (OutFlux_flag .and. (par%grav_mode == 'central') .and. (dat%r%i_xb > 0))
      !
      ! --- Delete temporary box ---
      call rhd_box_Delete(slice)
      !
      ! --- Deallocate temporary arrays ---
      deallocate(weight, &
                 kapparho_trans, T_trans, drhoeidtime_trans, Frad3_trans, g3_trans)
    end do ! i2
    !$OMP END DO
    !
    !$OMP CRITICAL (X12MEAN)
    call rhd_datx_Add(dat%x12, datx_thread)
    call rhd_datx_Delete(datx_thread)
    !
    if ((par%grav_mode == 'central') .and. (dat%r%i_xb > 0)) then
      call rhd_datx_Add(dat%r, datr_thread)
      call rhd_datx_Delete(datr_thread)
      !
      if (associated(dat%c%d)) then
        call rhd_datc_Add(dat%c, datc_thread)
        call rhd_datc_Delete(datc_thread)
      endif
    endif
    !$OMP END CRITICAL (X12MEAN)
    !
    !$OMP END PARALLEL
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_stop('SHC: step: dtime: final(outnew3)')
!#
#   endif
!#
  endif ! (dat%x12%na > 0)
  !
endif ! (OutIntensity_flag)
!
end subroutine rhd_shortrad_StepOutput

!#
# ifdef rhd_shortrad_eostest_switch
!#

!----------********************---------------------------------------------------------------------
subroutine rhd_shortrad_EOSTest(model, m1,n1, m2,n2, m3,n3, rho, T, logP, dTdei)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_EOSTest ('rhd_short-characteristics_radiation_Equation_Of_State_Test')
!
! PURPOSE:
!   Compute EOS quantities with various subroutines and measure the timings.
!
! CATEGORY:
!   Radiative transfer, EOS
!
! CALLING SEQUENCE:
!   call rhd_shortrad_EOSTest(model, m1,n1, m2,n2, m3,n3, rho, T, logP, dTdei)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! OUTPUT:
!   rho:          ('rho') real 3D array with density, attention: one buffer cell in all directions
!   T:            ('Temperature') real 3D array with temperature, attention: one buffer cell
!   logP:         ('logarithm_Pressure') real 3D array with natural logarithm of pressure,
!                 attention: one buffer cell
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array with inverse
!                 heat capacity
!
! LOCAL VARIABLES:
!   Ttmp:         ('Temperature_temporary') real 3D array with temperature
!   logPtmp:      ('logarithm_Pressure_temporary') real 3D array with natural logarithm of pressure
!
! ROUTINES:
!   eosinter3_logPTdTnewCv1:
!   eosinter3_logPTdTnewCv2:
!   eosinter_allOMP:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_logPTdT:      Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinter_logPTdTOMP:   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinternewa1:
!   eosinternewb1:
!   eosinternewc1:
!   eosinternewd1:
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   eosinter_PdPTdT:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Timing statistics are gathered.
!   Some output is written to screen.
!
! RESTRICTIONS:
!   This routines should not be called during normal production runs: it is meant for
!   performance measurements and testing only.
!   A proper EOS table has to exist.
!   The number of ghost cells has to be exactly 1.
!
! PROCEDURE:
!   Compute EOS data by various variants of the EOS-table interpolation routine
!   and time the calls.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-05-26 (B.F.) Put into separate routine
!   2008-08-12 (B.F. Lyon) Add index parameters to call of eosinter_logPTdT
!   2013-02-17 (B.F.) Lots of extra calls of various EOS routines
!   2013-02-18 (B.F.) Name change: rhd_shortrad_EOSt -> rhd_shortrad_EOSTest
!   2014-12-07 (B.F.) New parameters: m1,... n3
!---------------------------------------------------------------------------------------------------
use gasinter_module
use rhd_gl_module
use rhd_box_module
!#
# ifdef rhd_shortrad_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real, dimension(m1-1:n1+1, &
                m2-1:n2+1, &
                m3-1:n3+1   ), &
                           intent(out)  :: rho, T, logP, dTdei
!
! --- Local variables ---
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(m1  :n1  , &
                m2  :n2  , &
                m3  :n3     )           :: Ttmp, logPtmp, &
                                           dPdrho, dPde, s
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: Ttmp, logPtmp, dPdrho, dPde, s
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: Ttmp, logPtmp, dPdrho, dPde, s
!#
# endif
!#      /* (rhd_arrays_l01) */
integer                                 :: i1, i2, i3, in1
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D array ---
allocate(Ttmp(   m1:n1, m2:n2, m3:n3), &
         logPtmp(m1:n1, m2:n2, m3:n3), &
         dPdrho( m1:n1, m2:n2, m3:n3), &
         dPde(   m1:n1, m2:n2, m3:n3), &
         s(      m1:n1, m2:n2, m3:n3)   )
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! === Testing only ================================================================
call timing_start('SHC: test: eosinternewa01')
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i1, i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
!    in1=min(i1+15,n1)
  do i2=m2,n2
!    do i1=m1,n1,16
      call eosinternewa1(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                         logP(m1:n1,i2,i3), Ttmp(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
!      call eosinternewa1(model%rho(i1:in1,i2,i3), model%ei(i1:in1,i2,i3), &
!                        logP(i1:in1,i2,i3), Ttmp(i1:in1,i2,i3), dTdei(i1:in1,i2,i3))
!    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
!write(gl%nc_p,*) 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
write(gl%nc_p,'(A,3(1PE14.6))') 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
call timing_stop('SHC: test: eosinternewa01')
!
call timing_start('SHC: test: eosinternewa1')
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i1, i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
!    in1=min(i1+15,n1)
  do i2=m2,n2
!    do i1=m1,n1,16
      call eosinternewa1(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                         logP(m1:n1,i2,i3), Ttmp(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
!      call eosinternewa1(model%rho(i1:in1,i2,i3), model%ei(i1:in1,i2,i3), &
!                        logP(i1:in1,i2,i3), Ttmp(i1:in1,i2,i3), dTdei(i1:in1,i2,i3))
!    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
write(gl%nc_p,'(A,3(1PE14.6))') 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
call timing_stop('SHC: test: eosinternewa1')
!
call timing_start('SHC: test: eosinternewb1')
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
!    in1=min(i1+15,n1)
  do i2=m2,n2
!    do i1=m1,n1,16
      call eosinternewb1(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                        logP(m1:n1,i2,i3), Ttmp(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
!      call eosinternewb1(model%rho(i1:in1,i2,i3), model%ei(i1:in1,i2,i3), &
!                        logP(i1:in1,i2,i3), Ttmp(i1:in1,i2,i3), dTdei(i1:in1,i2,i3))
!    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
write(gl%nc_p,'(A,3(1PE14.6))') 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
call timing_stop('SHC: test: eosinternewb1')
!
!!$call timing_start('SHC: test: eosinternewbscale1')
!!$!$OMP PARALLEL DEFAULT(SHARED)
!!$!
!!$!$OMP DO PRIVATE(i1, i2, i3) SCHEDULE(RUNTIME)
!!$do i3=m3,n3
!!$!    in1=min(i1+15,n1)
!!$  do i2=m2,n2
!!$!    do i1=m1,n1,16
!!$      call eosinternewb1(model%rho(m1:n1,i2,i3), 1.0E+13+0.00001*(model%ei(m1:n1,i2,i3)-1.0E+13), &
!!$                        logP(m1:n1,i2,i3), Ttmp(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
!!$!      call eosinternewb1(model%rho(i1:in1,i2,i3), model%ei(i1:in1,i2,i3), &
!!$!                        logP(i1:in1,i2,i3), Ttmp(i1:in1,i2,i3), dTdei(i1:in1,i2,i3))
!!$!    end do ! i1
!!$  end do ! i2
!!$end do ! i3
!!$!$OMP END DO
!!$!
!!$!$OMP END PARALLEL
!!$write(gl%nc_p,*) 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
!!$call timing_stop('SHC: test: eosinternewbscale1')
!
call timing_start('SHC: test: eosinternewc1')
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    call eosinternewc1(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                       logP(m1:n1,i2,i3), Ttmp(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
write(gl%nc_p,'(A,3(1PE14.6))') 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
call timing_stop('SHC: test: eosinternewc1')
!
call timing_start('SHC: test: eosinternewd1')
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    call eosinternewd1(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                       logP(m1:n1,i2,i3), Ttmp(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
write(gl%nc_p,'(A,3(1PE14.6))') 'Test: ', logP(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3)
call timing_stop('SHC: test: eosinternewd1')
!
call timing_start('SHC: test: eosinter3_logPTdTnCv1')
!$OMP PARALLEL DEFAULT(SHARED)
call eosinter3_logPTdTnewCv1(model%rho, model%ei, &
                             logPtmp, &
                             Ttmp, dTdei(m1:n1,m2:n2,m3:n3), m1,n1, m2,n2, m3,n3)
!$OMP END PARALLEL
write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
                                 logPtmp(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3), 0.0, 0.0, 0.0, &
                                 logPtmp(n1,n2,n3), Ttmp(n1,n2,n3), dTdei(n1,n2,n3), 0.0, 0.0, 0.0
call timing_stop('SHC: test: eosinter3_logPTdTnCv1')
!
call timing_start('SHC: test: eosinter3_logPTdTnCv2')
!$OMP PARALLEL DEFAULT(SHARED)
call eosinter3_logPTdTnewCv2(model%rho, model%ei, &
                             logPtmp, &
                             Ttmp, dTdei(m1:n1,m2:n2,m3:n3), m1,n1, m2,n2, m3,n3)
!$OMP END PARALLEL
write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
                                 logPtmp(m1,m2,m3), Ttmp(m1,m2,m3), dTdei(m1,m2,m3), 0.0, 0.0, 0.0, &
                                 logPtmp(n1,n2,n3), Ttmp(n1,n2,n3), dTdei(n1,n2,n3), 0.0, 0.0, 0.0
call timing_stop('SHC: test: eosinter3_logPTdTnCv2')
!
call timing_start('SHC: test: eosinter_logPTdT1D')
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    call eosinter_logPTdT(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                          logP(m1:n1,i2,i3), T(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
  end do ! i2
end do ! i3
!$OMP END DO
!$OMP END PARALLEL
write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
                                 logP(m1,m2,m3), T(m1,m2,m3), dTdei(m1,m2,m3), 0.0, 0.0, 0.0, &
                                 logP(n1,n2,n3), T(n1,n2,n3), dTdei(n1,n2,n3), 0.0, 0.0, 0.0
call timing_stop('SHC: test: eosinter_logPTdT1D')
!
call timing_start('SHC: test: eosinter_PdP')
call eosinter_PdP(model%rho, model%ei, &
                  logPtmp, dPdrho, dPde)
write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
                          log(logPtmp(m1,m2,m3)), 0.0, 0.0, dPdrho(m1,m2,m3), dPde(m1,m2,m3), 0.0, &
                          log(logPtmp(n1,n2,n3)), 0.0, 0.0, dPdrho(n1,n2,n3), dPde(n1,n2,n3), 0.0
call timing_stop('SHC: test: eosinter_PdP')
!
call timing_start('SHC: test: eosinter_PdPTdT')
call eosinter_PdPTdT(model%rho, model%ei, &
                     logPtmp, dPdrho, dPde, &
                     Ttmp, dTdei(m1:n1,m2:n2,m3:n3))
write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
   log(logPtmp(m1,m2,m3)), Ttmp(m1,m2,m3), dTdei(m1,m2,m3), dPdrho(m1,m2,m3), dPde(m1,m2,m3), 0.0, &
   log(logPtmp(n1,n2,n3)), Ttmp(n1,n2,n3), dTdei(n1,n2,n3), dPdrho(n1,n2,n3), dPde(n1,n2,n3), 0.0
call timing_stop('SHC: test: eosinter_PdPTdT')
!
call timing_start('SHC: test: eosinter_all')
!$OMP PARALLEL DEFAULT(SHARED)
call eosinter_allOMP(model%rho, model%ei, &
                     logPtmp, dPdrho, dPde, &
                     Ttmp, dTdei(m1:n1,m2:n2,m3:n3), &
                     s)
!$OMP END PARALLEL
write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
  log(logPtmp(m1,m2,m3)), Ttmp(m1,m2,m3), dTdei(m1,m2,m3), dPdrho(m1,m2,m3), dPde(m1,m2,m3), &
     s(m1,m2,m3), &
  log(logPtmp(n1,n2,n3)), Ttmp(n1,n2,n3), dTdei(n1,n2,n3), dPdrho(n1,n2,n3), dPde(n1,n2,n3), &
     s(n1,n2,n3)
call timing_stop('SHC: test: eosinter_all')
! === End testing only ============================================================
!
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_start('SHC: step: dtime: init: EOS')
!#
# endif
!#
!
!$OMP PARALLEL DEFAULT(SHARED)
!
! --- Solve EOS ---
!call eosinter_logPTdTOMP(model%rho, model%ei, &
!                         logPtmp, Ttmp, dTdei(m1:n1,m2:n2,m3:n3))
!
!$OMP DO PRIVATE(i2, i3) &
!$OMP SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    ! --- Get EOS data for 1D vector ---
    call eosinter_logPTdT(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                          logP(m1:n1,i2,i3), &
                          T(m1:n1,i2,i3), dTdei(m1:n1,i2,i3))
    !
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: dtime: init: EOS')
call timing_start('SHC: step: dtime: init: EOScp')
!#
# endif
!#
!
!$OMP PARALLEL DEFAULT(SHARED)
!
!$OMP DO PRIVATE(i1, i2, i3) SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      logP(i1,i2,i3)=logPtmp(  i1,i2,i3)
      T(   i1,i2,i3)=Ttmp(     i1,i2,i3)
      rho( i1,i2,i3)=model%rho(i1,i2,i3)
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!$OMP END PARALLEL
!
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: dtime: init: EOScp')
call timing_start('SHC: step: dtime: init: bnd')
!#
# endif
!#
!
! --- Fill buffer zones of T, P, rho arrays ---
T(m1-1     , m2  :n2  , m3:n3)=T(m1       , m2  :n2  , m3:n3)
T(n1+1     , m2  :n2  , m3:n3)=T(n1       , m2  :n2  , m3:n3)
T(m1-1:n1+1, m2-1     , m3:n3)=T(m1-1:n1+1, m2       , m3:n3)
T(m1-1:n1+1, n2+1     , m3:n3)=T(m1-1:n1+1, n2       , m3:n3)
T(m1-1:n1+1, m2-1:n2+1, m3-1 )=T(m1-1:n1+1, m2-1:n2+1, m3   )
T(m1-1:n1+1, m2-1:n2+1, n3+1 )=T(m1-1:n1+1, m2-1:n2+1, n3   )
!
logP(m1-1     , m2  :n2  , m3:n3)=logP(m1       , m2  :n2  , m3:n3)
logP(n1+1     , m2  :n2  , m3:n3)=logP(n1       , m2  :n2  , m3:n3)
logP(m1-1:n1+1, m2-1     , m3:n3)=logP(m1-1:n1+1, m2       , m3:n3)
logP(m1-1:n1+1, n2+1     , m3:n3)=logP(m1-1:n1+1, n2       , m3:n3)
logP(m1-1:n1+1, m2-1:n2+1, m3-1 )=logP(m1-1:n1+1, m2-1:n2+1, m3   )
logP(m1-1:n1+1, m2-1:n2+1, n3+1 )=logP(m1-1:n1+1, m2-1:n2+1, n3   )
!
rho(m1-1     , m2  :n2  , m3:n3)=rho(m1       , m2  :n2  , m3:n3)
rho(n1+1     , m2  :n2  , m3:n3)=rho(n1       , m2  :n2  , m3:n3)
rho(m1-1:n1+1, m2-1     , m3:n3)=rho(m1-1:n1+1, m2       , m3:n3)
rho(m1-1:n1+1, n2+1     , m3:n3)=rho(m1-1:n1+1, n2       , m3:n3)
rho(m1-1:n1+1, m2-1:n2+1, m3-1 )=rho(m1-1:n1+1, m2-1:n2+1, m3   )
rho(m1-1:n1+1, m2-1:n2+1, n3+1 )=rho(m1-1:n1+1, m2-1:n2+1, n3   )
!
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: dtime: init: bnd')
!#
# endif
!#
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
deallocate(Ttmp, logPtmp, dPdrho, dPde, s)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
end subroutine rhd_shortrad_EOSTest

!#
# endif
!#      /* rhd_shortrad_eostest_switch */


!----------****************-------------------------------------------------------------------------
subroutine rhd_shortrad_EOS(action, Copyrho_flag, model, geo, m1,n1, m2,n2, m3,n3, &
                            rho, quc, T, logP, dTdei, T_min, T_max)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_EOS ('rhd_short-characteristics_radiation_Equation_Of_State')
!
! PURPOSE:
!   Compute EOS quantities needed for radiative transfer.
!
! CATEGORY:
!   Radiative transfer, EOS
!
! CALLING SEQUENCE:
!   call rhd_shortrad_EOS(action, Copyrho_flag, model, geo, m1,n1, m2,n2, m3,n3, &
!                         rho, quc, T, logP, dTdei, T_min, T_max)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   Copyrho_flag: ('Copy_rho_flag') logical to indicate if density arrays (rho and quc) have to be
!                 copied
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   rho:          ('rho') real 3D array with density, attention: one buffer cell in all directions
!   quc:          ('quantity_centered') real 4D array with additional densities
!   T:            ('Temperature') real 3D array with temperature, attention: one buffer cell
!   logP:         ('logarithm_Pressure') real 3D array with natural logarithm of pressure,
!                 attention: one buffer cell
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array with inverse
!                 heat capacity
!   T_min:        ('Temperature_minimum') real with minimum temperature in box
!   T_max:        ('Temperature_maximum') real with maximum temperature in box
!
! LOCAL VARIABLES:
!   Ttmp:         ('Temperature_temporary') real 3D array with temperature
!   logPtmp:      ('logarithm_Pressure_temporary') real 3D array with natural logarithm of pressure
!   mgh1:         ('m_ghost_1') integer, number of ghost cells to copy at left side of x1 direction
!   ngh1:         ('m_ghost_1') integer, number of ghost cells to copy at right side of x1 direction
!   mgh2:         ('m_ghost_2') integer, number of ghost cells to copy at left side of x2 direction
!   ngh2:         ('m_ghost_2') integer, number of ghost cells to copy at right side of x2 direction
!   mgh3:         ('m_ghost_3') integer, number of ghost cells to copy at left side of x3 direction
!   ngh3:         ('m_ghost_3') integer, number of ghost cells to copy at right side of x3 direction
!
! ROUTINES:
!   eosinter_logPTdT:      Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   flush:                 Flush output buffer
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   rhd_box_MPIFillGhost:  Fill ghost cells by communicating with neighbor MPI processes
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_MPI_module:        MPI routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   Via MPI.
!
! RESTRICTIONS:
!   A proper EOS table has to exist.
!
! PROCEDURE:
!   Call EOS interpolation routine,
!   determine minimum and maximum temperature,
!   apply temperature limiter if requested,
!   copy arrays into others with extra buffer cells,
!   fill buffer cells.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2004-05-26 (B.F.) Put into separate routine
!   2008-08-12 (B.F. Lyon) Add index parameters to call of eosinter_logPTdT
!   2013-02-18 (B.F.) New version with better cache use, merge with TemperatureLimiter routine
!   2013-12-11 (B.F.) Allow periodic side boundaries
!   2014-11-05 (B.F. Uppsala) Fill additional quc array; new parameter Copyrho_flag
!   2014-12-07 (B.F.) New parameters: m1,... n3
!   2017-07-13 (B.F.) Account for internal boundaries in case of activated MPI
!   2017-07-21 (B.F.) Add (only partly filled) buffer cells to dTdei;
!                     use boundary parameters from action structure
!   2017-09-23 (B.F.) Add timing call around MPI ghost-cell-filling call
!   2017-10-25 (B.F.) Make the number of ghost cells flexible
!   2018-11-29 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!---------------------------------------------------------------------------------------------------
use gasinter_module
use rhd_gl_module
use rhd_box_module
use rhd_prop_module
use rhd_action_module
use rhd_MPI_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!#
# ifdef rhd_shortrad_step_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
integer, parameter                      :: nFg=1
type(action_type),         intent(in)   :: action
logical,                   intent(in)   :: Copyrho_flag
type(box_type),            intent(inout):: model
type(geo_type),            intent(in)   :: geo
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(out)  :: T, logP, dTdei
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg   ), &
                           intent(inout):: rho
real, dimension(m1-nFg:n1+nFg, &
                m2-nFg:n2+nFg, &
                m3-nFg:n3+nFg, &
                model%nquc      ), &
                           intent(inout):: quc
real,                      intent(out)  :: T_min, T_max
!
! --- Local variables ---
integer                                 :: nquc, i1, i2, i3, iquc, &
                                           mgh1,ngh1, mgh2,ngh2, mgh3,ngh3, &
                                           icnt_Tfix, ncnt_Tfix, ierrMPI
real                                    :: T_min1D, T_max1D, TminLimit
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_start('SHC: step: dtime: init: EOS')
!#
# endif
!
! --- Make abbreviations ---
nquc=model%nquc
! \label{f90:c_tminlimit__use1}
TminLimit=par%C_TminLimit
!
! --- Initialize some reduction variables ---
T_min    = 0.5*huge(T_min)
T_max    =-0.5*huge(T_max)
ncnt_Tfix=0
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Account for the larger input box if MPI is activated: there are always enough layers ---
! --- for the ghost cells (already filled, including the axes)                             ---
!
if ((action%bound_l1 == bound_InternalNext    ) .or. &
    (action%bound_l1 == bound_InternalPeriodic)     ) then
  mgh1=nFg
else
  mgh1=0
endif
if ((action%bound_r1 == bound_InternalNext    ) .or. &
    (action%bound_r1 == bound_InternalPeriodic)     ) then
  ngh1=nFg
else
  ngh1=0
endif
!
if ((action%bound_l2 == bound_InternalNext    ) .or. &
    (action%bound_l2 == bound_InternalPeriodic)     ) then
  mgh2=nFg
else
  mgh2=0
endif
if ((action%bound_r2 == bound_InternalNext    ) .or. &
    (action%bound_r2 == bound_InternalPeriodic)     ) then
  ngh2=nFg
else
  ngh2=0
endif
!
if ((action%bound_l3 == bound_InternalNext    ) .or. &
    (action%bound_l3 == bound_InternalPeriodic)     ) then
  mgh3=nFg
else
  mgh3=0
endif
if ((action%bound_r3 == bound_InternalNext    ) .or. &
    (action%bound_r3 == bound_InternalPeriodic)     ) then
  ngh3=nFg
else
  ngh3=0
endif
!
! --- Fill ghost cells with energy data from neighbor processes ---
!
call timing_start('SHC:   EOS: FillGhost')
call rhd_box_MPIFillGhost(geo, model, nFg, &
                          grid_mode=0, bulk_mode=1, ei_mode=1)
call timing_stop('SHC:   EOS: FillGhost')
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
mgh1=0
ngh1=0
mgh2=0
ngh2=0
mgh3=0
ngh3=0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
!#
!$OMP PARALLEL DEFAULT(SHARED)
!
!$OMP DO PRIVATE(i1, i2, i3, iquc, T_min1D, T_max1D) &
!$OMP   REDUCTION(MIN: T_min) REDUCTION(MAX: T_max) REDUCTION(+: ncnt_Tfix) &
!$OMP   SCHEDULE(RUNTIME)
do i3=m3-mgh3,n3+ngh3
  do i2=m2-mgh2,n2+ngh2
    ! --- Get EOS data for 1D vector ---
    call eosinter_logPTdT(model%rho(m1-mgh1:n1+ngh1,i2,i3), model%ei(m1-mgh1:n1+ngh1,i2,i3), &
                          logP(     m1-mgh1:n1+ngh1,i2,i3), &
                          T(        m1-mgh1:n1+ngh1,i2,i3), dTdei(   m1-mgh1:n1+ngh1,i2,i3))
    !
    ! --- Determine minimum and maximum temperature ---
    T_min1D=minval(T(m1-mgh1:n1+ngh1,i2,i3))
    T_max1D=maxval(T(m1-mgh1:n1+ngh1,i2,i3))
    T_min  =min(T_min, T_min1D)
    T_max  =max(T_max, T_max1D)
    !
    ! --- Prevent the gas temperature from dropping below a specified limit ---
    ! --- Count only modifications within box and not those in ghost cells  ---
    if (T_min1D < TminLimit) then
      if ((i2 < m2) .or. (i2 > n2) .or. (i3 < m3) .or. (i3 > n3)) then
        icnt_Tfix=0
      else
        icnt_Tfix=1
      endif
      do i1=m1-mgh1,n1+ngh1
        if (T(i1,i2,i3) < TminLimit) then
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + (TminLimit-T(i1,i2,i3))/dTdei(i1,i2,i3)
          if (.not. ((i1 < m1) .or. (i1 > n1))) then
            ncnt_Tfix=ncnt_Tfix+icnt_Tfix
          endif
        endif ! (T(i1,i2,i3) < TminLimit)
      end do ! i1
    endif ! (T_min1D < TminLimit)
    !
    if (Copyrho_flag) then
      ! --- Copy original rho array into array with buffer zones ---
      rho(m1-mgh1:n1+ngh1,i2,i3)=model%rho(m1-mgh1:n1+ngh1,i2,i3)
      !
      ! --- Copy original quc array into array with buffer zones ---
      do iquc=1,nquc
        quc(m1-mgh1:n1+ngh1,i2,i3,iquc)=model%quc(m1-mgh1:n1+ngh1,i2,i3,iquc)
      end do ! iquc
    endif ! (Copyrho_flag)
    !
    ! --- Fill buffer zones of T, logP, rho, quc arrays ---
    if (action%bound_l1 == bound_periodic) then
      ! --- Treat all side boundaries as periodic ---
      !
      T(   m1-nFg:m1-1  , i2, i3)=T(   n1-nFg+1:n1      , i2, i3)
      T(   n1+1  :n1+nFg, i2, i3)=T(   m1      :m1+nFg-1, i2, i3)
      logP(m1-nFg:m1-1  , i2, i3)=logP(n1-nFg+1:n1      , i2, i3)
      logP(n1+1  :n1+nFg, i2, i3)=logP(m1      :m1+nFg-1, i2, i3)
      if (Copyrho_flag) then
        rho( m1-nFg:m1-1  , i2, i3)=rho( n1-nFg+1:n1      , i2, i3)
        rho( n1+1  :n1+nFg, i2, i3)=rho( m1      :m1+nFg-1, i2, i3)
        if (nquc > 0) then
          quc( m1-nFg:m1-1  , i2, i3, :)=quc( n1-nFg+1:n1      , i2, i3, :)
          quc( n1+1  :n1+nFg, i2, i3, :)=quc( m1      :m1+nFg-1, i2, i3, :)
        endif
      endif ! (Copyrho_flag)
    else ! (action%bound_l1)
      if (.not. ((action%bound_l1 == bound_InternalNext    ) .or. &
                 (action%bound_l1 == bound_InternalPeriodic)       )) then
        ! --- Treat left side boundary as open ---
        !
        T(   m1-nFg:m1-1, i2, i3)=T(   m1, i2, i3)
        logP(m1-nFg:m1-1, i2, i3)=logP(m1, i2, i3)
        if (Copyrho_flag) then
          rho( m1-nFg:m1-1, i2, i3)=rho( m1, i2, i3)
          if (nquc > 0) then
            do iquc=1,nquc
              quc( m1-nFg:m1-1, i2, i3, iquc)=quc( m1, i2, i3, iquc)
            end do ! iquc
          endif
        endif ! (Copyrho_flag)
      endif ! (action%bound_l1)
      !
      if (.not. ((action%bound_r1 == bound_InternalNext    ) .or. &
                 (action%bound_r1 == bound_InternalPeriodic)       )) then
        ! --- Treat right side boundary as open ---
        !
        T(   n1+1:n1+nFg, i2, i3)=T(   n1, i2, i3)
        logP(n1+1:n1+nFg, i2, i3)=logP(n1, i2, i3)
        if (Copyrho_flag) then
          rho( n1+1:n1+nFg, i2, i3)=rho( n1, i2, i3)
          if (nquc > 0) then
            do iquc=1,nquc
              quc( n1+1:n1+nFg, i2, i3, iquc)=quc( n1, i2, i3, iquc)
            end do ! iquc
          endif
        endif ! (Copyrho_flag)
      endif ! (action%bound_r1)
    endif ! (action%bound_l1)
  end do ! i2
end do ! i3
!$OMP END DO
!
! --- Fill rest of buffer zones of T, logP, rho, quc arrays ---
if (action%bound_l2 == bound_periodic) then
  ! --- Treat both side boundaries as periodic ---
  !
  !$OMP DO PRIVATE(i2, i3)
  do i3=m3-mgh3,n3+ngh3
    do i2=m2-nFg,m2-1
      T(   m1-nFg:n1+nFg, i2, i3  )=T(   m1-nFg:n1+nFg, i2+n2-m2+1, i3)
      logP(m1-nFg:n1+nFg, i2, i3  )=logP(m1-nFg:n1+nFg, i2+n2-m2+1, i3)
    end do ! i2
    do i2=n2+1,n2+nFg
      T(   m1-nFg:n1+nFg, i2, i3  )=T(   m1-nFg:n1+nFg, i2-n2+m2-1, i3)
      logP(m1-nFg:n1+nFg, i2, i3  )=logP(m1-nFg:n1+nFg, i2-n2+m2-1, i3)
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  if (Copyrho_flag) then
    if (nquc > 0) then
      !$OMP DO PRIVATE(i2, i3)
      do i3=m3-mgh3,n3+ngh3
        do i2=m2-nFg,m2-1
          quc( m1-nFg:n1+nFg, i2, i3, :)=quc( m1-nFg:n1+nFg, i2+n2-m2+1, i3, :)
        end do ! i2
        do i2=n2+1,n2+nFg
          quc( m1-nFg:n1+nFg, i2, i3, :)=quc( m1-nFg:n1+nFg, i2-n2+m2-1, i3, :)
        end do ! i2
      end do ! i3
      !$OMP END DO NOWAIT
    endif
    !$OMP DO PRIVATE(i2, i3)
    do i3=m3-mgh3,n3+ngh3
      do i2=m2-nFg,m2-1
        rho( m1-nFg:n1+nFg, i2, i3  )=rho( m1-nFg:n1+nFg, i2+n2-m2+1, i3)
      end do ! i2
      do i2=n2+1,n2+nFg
        rho( m1-nFg:n1+nFg, i2, i3  )=rho( m1-nFg:n1+nFg, i2-n2+m2-1, i3)
      end do ! i2
    end do ! i3
    !$OMP END DO
  endif ! (Copyrho_flag)
  !
else ! (action%bound_l2)
  if (.not. ((action%bound_l2 == bound_InternalNext    ) .or. &
             (action%bound_l2 == bound_InternalPeriodic)       )) then
    ! --- Treat left side boundary as open ---
    !
    !$OMP DO PRIVATE(i2, i3)
    do i3=m3-mgh3,n3+ngh3
      do i2=m2-nFg,m2-1
        T(   m1-nFg:n1+nFg, i2, i3  )=T(   m1-nFg:n1+nFg, m2, i3)
        logP(m1-nFg:n1+nFg, i2, i3  )=logP(m1-nFg:n1+nFg, m2, i3)
      end do ! i2
    end do ! i3
    !$OMP END DO
    if (Copyrho_flag) then
      if (nquc > 0) then
        !$OMP DO PRIVATE(i2, i3)
        do i3=m3-mgh3,n3+ngh3
          do i2=m2-nFg,m2-1
            quc( m1-nFg:n1+nFg, i2, i3, :)=quc( m1-nFg:n1+nFg, m2, i3, :)
          end do ! i2
        end do ! i3
        !$OMP END DO NOWAIT
      endif
      !$OMP DO PRIVATE(i2, i3)
      do i3=m3-mgh3,n3+ngh3
        do i2=m2-nFg,m2-1
          rho( m1-nFg:n1+nFg, i2, i3  )=rho( m1-nFg:n1+nFg, m2, i3)
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif ! (Copyrho_flag)
  endif ! (action%bound_l2)
  !
  if (.not. ((action%bound_r2 == bound_InternalNext    ) .or. &
             (action%bound_r2 == bound_InternalPeriodic)       )) then
    ! --- Treat right side boundary as open ---
    !
    !$OMP DO PRIVATE(i2, i3)
    do i3=m3-mgh3,n3+ngh3
      do i2=n2+1,n2+nFg
        T(   m1-nFg:n1+nFg, i2, i3  )=T(   m1-nFg:n1+nFg, n2, i3)
        logP(m1-nFg:n1+nFg, i2, i3  )=logP(m1-nFg:n1+nFg, n2, i3)
      end do ! i2
    end do ! i3
    !$OMP END DO
    if (Copyrho_flag) then
      if (nquc > 0) then
        !$OMP DO PRIVATE(i2, i3)
        do i3=m3-mgh3,n3+ngh3
          do i2=n2+1,n2+nFg
            quc( m1-nFg:n1+nFg, i2, i3, :)=quc( m1-nFg:n1+nFg, n2, i3, :)
          end do ! i2
        end do ! i3
        !$OMP END DO NOWAIT
      endif
      !$OMP DO PRIVATE(i2, i3)
      do i3=m3-mgh3,n3+ngh3
        do i2=n2+1,n2+nFg
          rho( m1-nFg:n1+nFg, i2, i3  )=rho( m1-nFg:n1+nFg, n2, i3)
        end do ! i2
      end do ! i3
      !$OMP END DO
    endif ! (Copyrho_flag)
  endif ! (action%bound_r2)
  !
endif ! (action%bound_l2)
!
if (.not. (action%bound_l3 == bound_InternalNext)) then
  ! --- Treat bottom boundary as open ---
  !$OMP DO PRIVATE(i2, i3)
  do i2=m2-nFg,n2+nFg
    do i3=m3-nFg,m3-1
      T(   m1-nFg:n1+nFg, i2  , i3)=T(   m1-nFg:n1+nFg, i2, m3)
      logP(m1-nFg:n1+nFg, i2  , i3)=logP(m1-nFg:n1+nFg, i2, m3)
    end do ! i3
  end do ! i2
  !$OMP END DO NOWAIT
  if (Copyrho_flag) then
    !$OMP DO PRIVATE(i2, i3)
    do i2=m2-nFg,n2+nFg
      do i3=m3-nFg,m3-1
        rho( m1-nFg:n1+nFg, i2  , i3)=rho( m1-nFg:n1+nFg, i2, m3)
      end do ! i3
    end do ! i2
    !$OMP END DO NOWAIT
    if (nquc > 0) then
      !$OMP DO PRIVATE(i2, i3)
      do i2=m2-nFg,n2+nFg
        do i3=m3-nFg,m3-1
          quc( m1-nFg:n1+nFg, i2  , i3, :)=quc( m1-nFg:n1+nFg, i2, m3, :)
        end do ! i3
      end do ! i2
      !$OMP END DO NOWAIT
    endif
  endif ! (Copyrho_flag)
endif ! (action%bound_l3)
!
if (.not. (action%bound_r3 == bound_InternalNext)) then
  ! --- Treat top boundary as open ---
  !$OMP DO PRIVATE(i2, i3)
  do i2=m2-nFg,n2+nFg
    do i3=n3+1,n3+nFg
      T(   m1-nFg:n1+nFg, i2  , i3)=T(   m1-nFg:n1+nFg, i2, n3)
      logP(m1-nFg:n1+nFg, i2  , i3)=logP(m1-nFg:n1+nFg, i2, n3)
    end do ! i3
  end do ! i2
  !$OMP END DO NOWAIT
  if (Copyrho_flag) then
    !$OMP DO PRIVATE(i2, i3)
    do i2=m2-nFg,n2+nFg
      do i3=n3+1,n3+nFg
        rho( m1-nFg:n1+nFg, i2  , i3)=rho( m1-nFg:n1+nFg, i2, n3)
      end do ! i3
    end do ! i2
    !$OMP END DO NOWAIT
    if (nquc > 0) then
      !$OMP DO PRIVATE(i2, i3)
      do i2=m2-nFg,n2+nFg
        do i3=n3+1,n3+nFg
          quc( m1-nFg:n1+nFg, i2  , i3, :)=quc( m1-nFg:n1+nFg, i2, n3, :)
        end do ! i3
      end do ! i2
      !$OMP END DO NOWAIT
    endif
  endif ! (Copyrho_flag)
endif ! (action%bound_r3)
!
!$OMP END PARALLEL
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: dtime: init: EOS')
!#
# endif
!#
!
!write(gl%nc_p,'(A,12(1PE14.6))') 'Test: ', &
!                                 logP(m1,m2,m3), T(m1,m2,m3), dTdei(m1,m2,m3), 0.0, 0.0, 0.0, &
!                                 logP(n1,n2,n3), T(n1,n2,n3), dTdei(n1,n2,n3), 0.0, 0.0, 0.0
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Compute global minimum and maximum temperatures ---
  call MPI_AllReduce(MPI_IN_PLACE, T_min    , 1, MPI_REAL   , MPI_MIN, geo%Comm, ierrMPI)
  call MPI_AllReduce(MPI_IN_PLACE, T_max    , 1, MPI_REAL   , MPI_MAX, geo%Comm, ierrMPI)
  call MPI_AllReduce(MPI_IN_PLACE, ncnt_Tfix, 1, MPI_INTEGER, MPI_SUM, geo%Comm, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A,2(1PE13.6))') 'minmax(T)       ', T_min, T_max
  if (T_min < TminLimit) then
    write(gl%nc_m,'(A,1PE12.4)') 'Limiting minimum temperature to                 :', TminLimit
    write(gl%nc_m,'(A,I12)'    ) 'Activated temperature fix, no. of affected cells:', ncnt_Tfix
  endif
  call flush(gl%nc_m)
endif
!
end subroutine rhd_shortrad_EOS


!----------************************-----------------------------------------------------------------
subroutine rhd_shortrad_LocalSource(model, geo, action, dtime, dtime_limit, &
                                    Output_flag, &
                                    dat, &
                                    outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_LocalSource ('rhd_short-characteristics_radiation_LocalSource')
!
! PURPOSE:
!   Perform radiation update step employing local approximation.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_LocalSource(model, geo, action, dtime, dtime_limit, &
!                                 Output_flag=Output_flag, &
!                                 dat=dat, &
!                                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data of non-local radiation-transport box
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                 default=.false.
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model structure with input and output data
!   dat:          ('data') derived type with additional output data (optional)
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, recommended maximum radiative time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                   =0:   normal run
!                   1-99: "small" error (e.g. time step limit violation)
!
! ROUTINES:
!   eosinter_logPTdT:      Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinter_TdT:          Solve EOS by bicubic interpolation: rho, e -> T, dTde
!   rhd_box_RestIndices:   Compute indices of boxes surrounding small box in large box
!   rhd_shortrad_StepOutput: Derive some averages as additional output quantities
!   sub_xkaros_logPT2rhokap: Interpolate opacity per volume for given logP, T, and nu bin (1D,2D,3D)
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   MPI:                   Standard MPI routines and definitions
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_MPI_module:        MPI routines specific for RHD code
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! LOCAL VARIABLES:
!   m1:           ('m_number_1') integer, 1. dimension, start index of local box
!   n1:           ('number_1')   integer, 1. dimension, end index of local box
!   m2:           ('m_number_2') integer, 2. dimension, start index of local box
!   n2:           ('number_2')   integer, 2. dimension, end index of local box
!   m3:           ('m_number_3') integer, 3. dimension, start index of local box
!   n3:           ('number_3')   integer, 3. dimension, end index of local box
!   mb1:          ('m_number_block_1') integer, 1. dimension, start index of current block
!   nb1:          ('number_block_1')   integer, 1. dimension, end index of current block
!   mb2:          ('m_number_block_2') integer, 2. dimension, start index of current block
!   nb2:          ('number_block_2')   integer, 2. dimension, end index of current block
!   mb3:          ('m_number_block_3') integer, 3. dimension, start index of current block
!   nb3:          ('number_block_3')   integer, 3. dimension, end index of current block
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   ibl:          ('integer_block') integer with block index (1 <= ibl <= 6)
!   dei:          ('delta_energy_internal') real scalar with local change of internal energy
!   deidtime:     ('delta_energy_internal_delta_time') real scalar with local change of
!                 internal energy per time unit
!   r:            ('radius') real scalar with distance of grid point to origin
!   r2:           ('radius_2') real scalar with squared distance of grid point to origin
!   r_core:       ('radius_core') real scalar with radius of core heating zone
!   T_Target:     ('Temperature_Target') real scalar with local target temperature
!   T_CoreLum:    ('Temperature_Core_Luminosity') real scalar with (fictitous) temperature at
!                 surface of core. A grey temperature in the optically thin at radius r
!                 can be estimated with
!                   T_CoreLum * sqrt(r_core/r)
!   T_min:        ('Temperature_minimum') real with global minimum temperature
!   T_max:        ('Temperature_maximum') real with global maximum temperature
!   T_minblock:   ('Temperature_minimum_block') real with minimum temperature in current sub block
!   T_maxblock:   ('Temperature_maximum_block') real with maximum temperature in current sub block
!   FocL_CoreLum: ('Flux_over_c_light_Core_Luminosity') real scalar with (fictitous) flux over
!                 light speed at surface of core.
!                 The radiative flux in the optically thin at radius r can be estimated with
!                   F_CoreLum * (r_core/r)**2 * c_light
!   FocL:         ('Flux_over_c_light') real scalar with local flux over light speed.
!   n_MonoPerGrain: ('number_Monomers_per_Grain) real scalar with number of monomers per dust grain
!   a_Grain:      ('a_Grain') real scalar with radius of individual dust grain
!   Q_rp:         ('Q_radiation_pressure') real scalar with unitless opacity efficiency factor, see
!               \href{http://adsabs.harvard.edu/abs/2008PhST..133a4007H/abstract}{ADS: Hoefner 2008}
!   kapparho_rp:  ('kappa_rho_radiation_pressure') real scalar with dust scattering opacity
!   dvradialdtime: ('delta_velocity_radial_delta_time') real scalar with radial radiative
!                 acceleration
!   dvradialdtime_max: ('delta_velocity_radial_delta_time_maximum') real scalar with
!                 global maximum of radial radiative acceleration
!   dvradialdtime_maxblock: ('delta_velocity_radial_delta_time_maximum_block') real scalar with
!                 maximum of radial radiative acceleration in current sub block
!   dv1dtime:     ('delta_velocity_1_delta_time') real scalar with 1st component of radiative
!                 acceleration
!   dv2dtime:     ('delta_velocity_2_delta_time') real scalar with 2nd component of radiative
!                 acceleration
!   dv3dtime:     ('delta_velocity_3_delta_time') real scalar with 3rd component of radiative
!                 acceleration
!   Output_flag0: ('Output_flag_0') logical, if true the output quantities are computed and put into
!                 dat structure, that has to be passed
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   T1D:          ('Temperature_1_Dimension') real vector with temperature
!   logP1D:       ('logarithm_Pressure_1_Dimension') real vector with log(Pressure)
!   dTdei1D:      ('delta_Temperature_delta_energy_internal_1_Dimension') real vector with
!                 temperature derivative (inverse heat capacity)
!   T:            ('Temperature') real 3D array with temperature
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array with
!                 temperature derivative (inverse heat capacity)
!   rhokappa:     ('rho_kappa') real 3D array with mean opacities
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time) real 3D array with change of
!                 internal energy per time due to radiation
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change x1
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change x2
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change x3
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!   ierr0:        ('integer_error_0') integer with error status
!   outstr0:      ('output_string_0') character with error report
!   C_deiFraction: ('Constant_delta_energy_internal_Fraction') real parameter with maximum allowed
!                  relative energy change
!   HeH:          ('Helium_Haeufigkeit') real parameter with helium abundance (number fraction)
!   PMass:        ('Proton_Mass') real parameter with proton mass [g]
!   SMass:        ('S_Mass') real parameter
!   ZNL:          ('ZNL') real parameter with minimum number of monomers per grain
!   rhoD_Mg2SiO4: ('rho_Dust_Mg2SiO4') real parameter with material density of forsterite [g/cm^3]
!   AW_Mg2SiO4:   ('Atomic_Weight_Mg2SiO4') real parameter with atomic wweight of forsterite
!   q1_Mg2SiO4:   ('quantity_1_Mg2SiO4') real parameter with unitless coefficient for Eq. (20) in
!               \href{http://adsabs.harvard.edu/abs/2008PhST..133a4007H/abstract}{ADS: Hoefner 2008}
!   q4_Mg2SiO4:   ('quantity_4_Mg2SiO4') real parameter with unitless coefficient for Eq. (20)
!   qmax_Mg2SiO4: ('quantity_max_Mg2SiO4') real parameter with unitless coefficient for Eq. (20)
!   lambdamax_Mg2SiO4: ('lambda_max_Mg2SiO4') real parameter with coefficient for Eq. (20) [cm]
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-12-12 (B.F. Uppsala) First version
!   2018-11-24 (B.F.) Change call to EOS routine
!   2018-11-29 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!   2019-02-05 (B.F.) Reorganize 1D and 3D arrays; compute output data if requested
!   2019-02-06 (B.F.) Add branch for radThinHeatScheme='None' (the default)
!   2019-02-26 (B.F.) Account for C_radthintimeRelax in radThinHeatScheme='ConstantT'
!   2019-02-27 (B.F.) Implement radThinPressScheme='ConstantLum'
!   2019-02-28 (B.F.) Clean up things and implement smooth transition for heating
!   2019-03-01 (B.F.) Properly handle smooth-transition region for dat output quantities
!   2019-03-12 (B.F.) Limit target temperature from below for radThinHeatScheme='ConstantLum'
!   2019-03-19 (B.F.) Limit resulting internal energy from below
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use opta_module
use const_module, only: sigma, c_light, pi, fourpi
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_prop_module
use rhd_dat_module
!$ use omp_lib
use rhd_MPI_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,          optional,intent(in)   :: Output_flag
type(dat_type),   optional,intent(inout):: dat
integer,          optional,intent(out)  :: ierr
character(len=*), optional,intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           mb1,nb1, mb2,nb2, mb3,nb3, &
                                           i1, i2, i3, ibl, nHeat, nPress, imax, imask
real                                       dei, deidtime, fmask, r, r2, r_core, eiTable_min, &
                                           T_Target, T_CoreLum, &
                                           T_min, T_max, T_minblock, T_maxblock, &
                                           FocL_CoreLum, FocL, &
                                           n_MonoPerGrain, a_Grain, Q_rp, kapparho_rp, &
                                           dvradialdtime, dvradialdtime_max, &
                                           dvradialdtime_maxblock, &
                                           dv1dtime, dv2dtime, dv3dtime
logical                                 :: Output_flag0, ComputePressure_flag
integer, parameter                      :: nFg=1
integer, parameter                      :: nFb=1
real, dimension(model%m1-nFg:model%n1+nFg) &
                                        :: rho1D, ei1D, T1D, logP1D, dTdei1D
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(model%m1-nFg:model%n1+nFg, &
                model%m2-nFg:model%n2+nFg, &
                model%m3-nFg:model%n3+nFg) &
                                        :: T, dTdei, rhokappa, drhoeidtime
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(model%m1-nFg:model%n1+nFg, &
                model%m2-nFg:model%n2+nFg, &
                model%m3-nFg:model%n3+nFg) &
                                        :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
real, dimension(model%m1-nFb:model%n1+nFb+1, &
                model%m2-nFb:model%n2+nFb  , &
                model%m3-nFb:model%n3+nFb   ) &
                                        :: Frad1
real, dimension(model%m1-nFb:model%n1+nFb  , &
                model%m2-nFb:model%n2+nFb+1, &
                model%m3-nFb:model%n3+nFb   ) &
                                        :: Frad2
real, dimension(model%m1-nFb:model%n1+nFb  , &
                model%m2-nFb:model%n2+nFb  , &
                model%m3-nFb:model%n3+nFb+1 ) &
                                        :: Frad3
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: T, dTdei, rhokappa, drhoeidtime, &
                                           Frad1, Frad2, Frad3
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(:,:,:), pointer         :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: T, dTdei, rhokappa, drhoeidtime, &
                                           Frad1, Frad2, Frad3
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(:,:,:), allocatable     :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
!#
# endif
!#      /* (rhd_arrays_l01) */
integer                                 :: ierrMPI
integer                                 :: ierr0
character(len=80)                       :: outstr0
real, parameter                         :: C_deiFraction=0.5, &
                                           HeH         =0.1, &
                                           PMass       =1.6726E-24, &
                                           SMass       =PMass * (1.0+4.0*HeH), &
                                           ZNL         =1.0E+03, &
                                           rhoD_Mg2SiO4=3.27, &
                                           AW_Mg2SiO4  =2.0*24.0 + 28.0 + 4.0*16.0, &
                                           q1_Mg2SiO4  =2.00E-03, &
                                           q4_Mg2SiO4  =4.68E+02, &
                                           qmax_Mg2SiO4=1.0, &
                                           lambdamax   =1.08E-4
!---------------------------------------------------------------------------------------------------
!
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A)') '--- Start of LocalSource step' // &
                       '; heating: ' // trim(par%radThinHeatScheme) // &
                       ', pressure: ' // trim(par%radThinPressScheme) // ' ---'
  call flush(gl%nc_m)
endif
!
! --- Initialize error variables: no error occurred, so far ---
ierr0=0
outstr0=''
!
! --- Set default values for optional keywords ---
if (.not. present(Output_flag)) then
  Output_flag0=.false.
else
  Output_flag0=Output_flag
endif
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set size of small box for non-local radiation transport ---
if (par%N_radThinHeatPoint > 0) then
  imax=max(-geo%m(1), geo%n(1), -geo%m(2), geo%n(2), -geo%m(3), geo%n(3))
else
  imax=-1
endif
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D arrays ---
allocate( &
  T(          m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  dTdei(      m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  rhokappa(   m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  drhoeidtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  Frad1(      m1-nFb:n1+nFb+1, m2-nFb:n2+nFb  , m3-nFb:n3+nFb  ), &
  Frad2(      m1-nFb:n1+nFb  , m2-nFb:n2+nFb+1, m3-nFb:n3+nFb  ), &
  Frad3(      m1-nFb:n1+nFb  , m2-nFb:n2+nFb  , m3-nFb:n3+nFb+1)   )
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
allocate( &
  drhov1dtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  drhov2dtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  drhov3dtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  )   )
!#
# endif
!#      /* (rhd_shortrad_radpressure_l01) */
!#
# endif
!#      /* (rhd_arrays_l01) */
!
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
! \label{f90:radpressure__use2}
! ComputePressure_flag=((par%radpressure == 'on') .or. (par%radpressure == 'On'))
ComputePressure_flag=.true.
!#
# else
!#
ComputePressure_flag=.false.
!#
# endif
!#      /* (rhd_shortrad_radpressure_l01) */
!
dtime_limit=huge(dtime_limit)
!
! --- Get minimum value of tabulated internal energy ---
call tabinter_getinfo(x2min=eiTable_min)
!
!
! \label{f90:n_radthinheatpoint__use1}
! --- Set number of additional grid points with optically thin radiative heating ---
nHeat=max(par%N_radThinHeatPoint, 0)
!
! \label{f90:radthinheatscheme__use1}
! ### Treat different cases of local heating and cooling ###########################################
if (par%radThinHeatScheme == 'MinLimitT') then
  ! === Adjust internal energy to limit the temperature from below =================================
  !
  call timing_start('RAD: LocalHeat T limit')
  !
  ! \label{f90:c_tminlimit__use2}
  T_Target=par%C_TminLimit
  !
  T_min= 0.1*huge(T_min)
  T_max=-0.1*huge(T_max)
  !
  do ibl=1,6
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nHeat, geo%n(1)-nHeat, &
                                  geo%m(2)+nHeat, geo%n(2)-nHeat, &
                                  geo%m(3)+nHeat, geo%n(3)-nHeat, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !write(gl%nc_p,'(A,I3,A,I5,3X,6I5,3X,6I5)') &
    !                   '#', wg%hyd%MyProc, &
    !                   ' ibl, m1,n1, m2,n2, m3,n3: ', ibl, &
    !                     mb1,nb1, mb2,nb2, mb3,nb3, &
    !                     m1,n1, m2,n2, m3,n3
    !call flush(gl%nc_p)
    !
    if ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3)) then
      ! --- Handle non-empty sub block ---
      !
      T_minblock= 0.1*huge(T_minblock)
      T_maxblock=-0.1*huge(T_maxblock)
      !
      !$OMP PARALLEL DEFAULT(NONE) &
      !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, model, drhoeidtime, dtime, T_target, eiTable_min) &
      !$OMP   PRIVATE(i1,i2,i3, T1D, dTdei1D, dei) &
      !$OMP   REDUCTION(MIN: T_minblock) REDUCTION(MAX: T_maxblock)
      !
      !$OMP DO SCHEDULE(STATIC)
      do i3=mb3,nb3
        do i2=mb2,nb2
          ! --- Get EOS data for 1D vector ---
          call eosinter_TdT(model%rho(mb1:nb1, i2, i3), model%ei(mb1:nb1, i2, i3), &
                            T1D(      mb1:nb1        ), dTdei1D( mb1:nb1        )   )
          !
          ! --- Determine minimum and maximum temperature within block ---
          T_minblock=min(T_minblock, minval(T1D(mb1:nb1)))
          T_maxblock=max(T_maxblock, maxval(T1D(mb1:nb1)))
          !
          ! --- Compute energy change and update internal energy ---
          do i1=mb1,nb1
            dei                   = (max(T_Target-T1D(i1), 0.0)) / dTdei1D(i1)
            dei                   = max(dei, &
                                        C_deiFraction * (eiTable_min-model%ei(i1,i2,i3)))
            drhoeidtime(i1,i2,i3) = dei * model%rho(i1,i2,i3) / dtime
            model%ei(   i1,i2,i3) = model%ei(i1,i2,i3) + dei
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
      ! --- Determine minimum and maximum temperature within local model ---
      T_min=min(T_min, T_minblock)
      T_max=max(T_max, T_maxblock)
      write(gl%nc_p, '(A,I3,A,I2,A,4(1PE11.4,2X))') &
        '#', geo%MyProc, ' Block: ', ibl, &
        ' LocalSource: Tminmax: ', T_min, T_max, T_minblock, T_maxblock
      !
    endif ! ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3))
  end do ! ibl
  !
  call timing_stop('RAD: LocalHeat T limit')
  !
  ! \label{f90:c_radthintimerelax__use1}
else if ((par%radThinHeatScheme == 'ConstantT') .and. (par%C_radthintimeRelax > 0.0)) then
  ! === Adjust internal energy towards prescribed constant temperature =============================
  !
  call timing_start('RAD: LocalHeat T constant')
  !
  ! \label{f90:c_radthintfactor__use1}
  T_Target=par%C_radthinTFactor*par%Teff
  !
  T_min= 0.1*huge(T_min)
  T_max=-0.1*huge(T_max)
  !
  do ibl=1,6
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nHeat, geo%n(1)-nHeat, &
                                  geo%m(2)+nHeat, geo%n(2)-nHeat, &
                                  geo%m(3)+nHeat, geo%n(3)-nHeat, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    if ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3)) then
      ! --- Handle non-empty sub block ---
      !
      T_minblock= 0.1*huge(T_minblock)
      T_maxblock=-0.1*huge(T_maxblock)
      !
      !$OMP PARALLEL DEFAULT(NONE) &
      !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, model, drhoeidtime, dtime, T_target, &
      !$OMP          imax, par, nHeat, eiTable_min) &
      !$OMP   PRIVATE(i1,i2,i3, T1D, dTdei1D, deidtime, imask, fmask) &
      !$OMP   REDUCTION(MIN: T_minblock) REDUCTION(MAX: T_maxblock)
      !
      !$OMP DO SCHEDULE(STATIC)
      do i3=mb3,nb3
        do i2=mb2,nb2
          ! --- Get EOS data for 1D vector ---
          call eosinter_TdT(model%rho(mb1:nb1, i2, i3), model%ei(mb1:nb1, i2, i3), &
                            T1D(      mb1:nb1        ), dTdei1D( mb1:nb1        )   )
          !
          ! --- Determine minimum and maximum temperature within block ---
          T_minblock=min(T_minblock, minval(T1D(mb1:nb1)))
          T_maxblock=max(T_maxblock, maxval(T1D(mb1:nb1)))
          !
          ! --- Compute energy change and update internal energy ---
          if (imax < 0) then
            ! --- Do not use smooth transition ---
            do i1=mb1,nb1
              ! \label{f90:c_radthintimerelax__use2}
              deidtime              = (T_Target-T1D(i1)) / (dTdei1D(i1)*par%C_radthintimeRelax)
              deidtime              = max(deidtime, &
                                          C_deiFraction/dtime * (eiTable_min-model%ei(i1,i2,i3)))
              drhoeidtime(i1,i2,i3) = deidtime * model%rho(i1,i2,i3)
              model%ei(   i1,i2,i3) = model%ei(i1,i2,i3) + dtime * deidtime
            end do ! i1
          else ! (imax >= 0)
            ! --- Use smooth transition (assume cubic small box) ---
            do i1=mb1,nb1
              deidtime              = (T_Target-T1D(i1)) / (dTdei1D(i1)*par%C_radthintimeRelax)
              deidtime              = max(deidtime, &
                                          C_deiFraction/dtime * (eiTable_min-model%ei(i1,i2,i3)))
              imask                 = max(min(max(abs(i1), abs(i2), abs(i3)), imax+1) - &
                                            (imax-nHeat), &
                                          0)
              fmask                 = real(imask)/real(nHeat+1)
              deidtime              = deidtime * fmask
              drhoeidtime(i1,i2,i3) = deidtime * model%rho(i1,i2,i3)
              model%ei(   i1,i2,i3) = model%ei(i1,i2,i3) + dtime * deidtime
            end do ! i1
          endif ! (imax < 0)
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
      ! --- Determine minimum and maximum temperature within local model ---
      T_min=min(T_min, T_minblock)
      T_max=max(T_max, T_maxblock)
      write(gl%nc_p, '(A,I3,A,I2,A,4(1PE11.4,2X))') &
        '#', geo%MyProc, ' Block: ', ibl, &
        ' LocalSource: Tminmax: ', T_min, T_max, T_minblock, T_maxblock
      !
    endif ! ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3))
  end do ! ibl
  !
  call timing_stop('RAD: LocalHeat T constant')
  !
  ! \label{f90:c_radthintimerelax__use3}
else if ((par%radThinHeatScheme == 'ConstantLum') .and. &
         (par%r0_grav            > 0.0          ) .and. &
         (par%C_radthintimeRelax > 0.0          )        ) then
  ! === Adjust internal energy towards temperature computed from core luminosity ===================
  !
  call timing_start('RAD: LocalHeat L constant')
  !
  ! --- Set core radius ---
  if (par%C_gravCorer0Heat > 0.0) then
    r_core=par%C_gravCorer0Heat*par%r0_grav
  else
    r_core=par%r0_grav
  endif
  ! \label{f90:c_radthintfactor__use2}\label{f90:luminositypervolume__use5}
  ! --- Compute temperature at core radius from core luminosity per volume ---
  T_CoreLum=par%C_radthinTFactor * sqrt(sqrt(1.0/(12.0*sigma) * r_core * par%LuminosityPerVolume))
  !
  T_min= 0.1*huge(T_min)
  T_max=-0.1*huge(T_max)
  !
  do ibl=1,6
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nHeat, geo%n(1)-nHeat, &
                                  geo%m(2)+nHeat, geo%n(2)-nHeat, &
                                  geo%m(3)+nHeat, geo%n(3)-nHeat, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !write(gl%nc_p,'(A,I3,A,I5,3X,6I5,3X,6I5)') &
    !                   '#', wg%hyd%MyProc, &
    !                   ' ibl, m1,n1, m2,n2, m3,n3: ', ibl, &
    !                     mb1,nb1, mb2,nb2, mb3,nb3, &
    !                     m1,n1, m2,n2, m3,n3
    !call flush(gl%nc_p)
    !
    if ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3)) then
      ! --- Handle non-empty sub block ---
      !
      T_minblock= 0.1*huge(T_minblock)
      T_maxblock=-0.1*huge(T_maxblock)
      !
      !$OMP PARALLEL DEFAULT(SHARED) &
      !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, model, drhoeidtime, dtime, eiTable_min) &
      !$OMP   PRIVATE(i1,i2,i3, T1D, dTdei1D, r2, T_Target, deidtime, imask, fmask) &
      !$OMP   REDUCTION(MIN: T_minblock) REDUCTION(MAX: T_maxblock)
      !
      !$OMP DO SCHEDULE(STATIC)
      do i3=mb3,nb3
        do i2=mb2,nb2
          ! --- Get EOS data for 1D vector ---
          call eosinter_TdT(model%rho(mb1:nb1, i2, i3), model%ei(mb1:nb1, i2, i3), &
                            T1D(      mb1:nb1        ), dTdei1D( mb1:nb1        )   )
          !
          ! --- Determine minimum and maximum temperature within block ---
          T_minblock=min(T_minblock, minval(T1D(mb1:nb1)))
          T_maxblock=max(T_maxblock, maxval(T1D(mb1:nb1)))
          !
          ! --- Compute energy change and update internal energy ---
          if (imax < 0) then
            ! --- Do not use smooth transition ---
            do i1=mb1,nb1
              r2                    = model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2
              ! \label{f90:c_tminlimit__use3}
              T_Target              = max(T_CoreLum * sqrt(sqrt(r_core**2/max(r2, r_core**2))), &
                                          par%C_TminLimit)
              ! \label{f90:c_radthintimerelax__use4}
              deidtime              = (T_Target-T1D(i1)) / (dTdei1D(i1)*par%C_radthintimeRelax)
              deidtime              = max(deidtime, &
                                          C_deiFraction/dtime * (eiTable_min-model%ei(i1,i2,i3)))
              drhoeidtime(i1,i2,i3) = deidtime * model%rho(i1,i2,i3)
              model%ei(   i1,i2,i3) = model%ei(i1,i2,i3) + dtime * deidtime
            end do ! i1
          else ! (imax >= 0)
            ! --- Use smooth transition (assume cubic small box) ---
            do i1=mb1,nb1
              r2                    = model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2
              T_Target              = max(T_CoreLum * sqrt(sqrt(r_core**2/max(r2, r_core**2))), &
                                          par%C_TminLimit)
              deidtime              = (T_Target-T1D(i1)) / (dTdei1D(i1)*par%C_radthintimeRelax)
              imask                 = max(min(max(abs(i1), abs(i2), abs(i3)), imax+1) - &
                                            (imax-nHeat), &
                                          0)
              fmask                 = real(imask)/real(nHeat+1)
              deidtime              = deidtime * fmask
              deidtime              = max(deidtime, &
                                          C_deiFraction/dtime * (eiTable_min-model%ei(i1,i2,i3)))
              drhoeidtime(i1,i2,i3) = deidtime * model%rho(i1,i2,i3)
              model%ei(   i1,i2,i3) = model%ei(i1,i2,i3) + dtime * deidtime
            end do ! i1
          endif ! (imax < 0)
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
      ! --- Determine minimum and maximum temperature within local model ---
      T_min=min(T_min, T_minblock)
      T_max=max(T_max, T_maxblock)
      write(gl%nc_p, '(A,I3,A,I2,A,4(1PE11.4,2X))') &
        '#', geo%MyProc, ' Block: ', ibl, &
        ' LocalSource: Tminmax: ', T_min, T_max, T_minblock, T_maxblock
      !
    endif ! ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3))
  end do ! ibl
  !
  ! \label{f90:c_radthintimerelax__use5}
  ! --- Set time step limit to an (sensible but somewhat arbitrary) factor of the ---
  ! --- radiative relaxation time scale                                           ---
  dtime_limit=1.5*par%C_radthintimeRelax
  !
  call timing_stop('RAD: LocalHeat L constant')
  !
else
  ! === Apply no heating or cooling ================================================================
  !
  call timing_start('RAD: LocalHeat None')
  !
  T_min= 0.1*huge(T_min)
  T_max=-0.1*huge(T_max)
  !
  do ibl=1,6
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nHeat, geo%n(1)-nHeat, &
                                  geo%m(2)+nHeat, geo%n(2)-nHeat, &
                                  geo%m(3)+nHeat, geo%n(3)-nHeat, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !write(gl%nc_p,'(A,I3,A,I5,3X,6I5,3X,6I5)') &
    !                   '#', wg%hyd%MyProc, &
    !                   ' ibl, m1,n1, m2,n2, m3,n3: ', ibl, &
    !                     mb1,nb1, mb2,nb2, mb3,nb3, &
    !                     m1,n1, m2,n2, m3,n3
    !call flush(gl%nc_p)
    !
    if ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3)) then
      ! --- Handle non-empty sub block ---
      !
      T_minblock= 0.1*huge(T_minblock)
      T_maxblock=-0.1*huge(T_maxblock)
      !
      !$OMP PARALLEL DEFAULT(NONE) &
      !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, model, drhoeidtime) &
      !$OMP   PRIVATE(i1,i2,i3, rho1D, ei1D, T1D, dTdei1D) &
      !$OMP   REDUCTION(MIN: T_minblock) REDUCTION(MAX: T_maxblock)
      !
      !$OMP DO SCHEDULE(STATIC)
      do i3=mb3,nb3
        do i2=mb2,nb2
          ! --- Get EOS data for 1D vector ---
          call eosinter_TdT(model%rho(mb1:nb1, i2, i3), model%ei(mb1:nb1, i2, i3), &
                            T1D(      mb1:nb1        ), dTdei1D( mb1:nb1        )   )
          !
          ! --- Determine minimum and maximum temperature within block ---
          T_minblock=min(T_minblock, minval(T1D))
          T_maxblock=max(T_maxblock, maxval(T1D))
          !
          ! --- Compute energy change, (here: set it to zero) ---
          do i1=mb1,nb1
            drhoeidtime(i1,i2,i3) = 0.0
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
      ! --- Determine minimum and maximum temperature within local model ---
      T_min=min(T_min, T_minblock)
      T_max=max(T_max, T_maxblock)
      write(gl%nc_p, '(A,I3,A,I2,A,4(1PE11.4,2X))') &
        '#', geo%MyProc, ' Block: ', ibl, &
        ' LocalSource: Tminmax: ', T_min, T_max, T_minblock, T_maxblock
      !
    endif ! ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3))
  end do ! ibl
  !
  call timing_stop('RAD: LocalHeat None')
  !
endif ! (par%radThinHeatScheme)
!
!
! \label{f90:n_radthinpresspoint__use1}
! --- Set number of additional grid points with optically thin radiative acceleration ---
nPress=max(par%N_radThinPressPoint, 0)
!
! \label{f90:radthinpressscheme__use1)
! ### Treat different cases of local acceleration ##################################################
if (par%radThinPressScheme == 'ConstantLum') then
  ! === Compute radial acceleration from flux due to luminosity injected into core =================
  !
  call timing_start('RAD: LocalPress L constant')
  !
  ! --- Initialize maximum value for model ---
  dvradialdtime_max=-0.1*huge(dvradialdtime_max)
  !
  ! --- Set core radius ---
  if (par%C_gravCorer0Heat > 0.0) then
    r_core=par%C_gravCorer0Heat*par%r0_grav
  else
    r_core=par%r0_grav
  endif
  ! \label{f90:luminositypervolume__use6}
  ! --- Compute flux/c_light at core radius from core luminosity per volume ---
  FocL_CoreLum=1.0/3.0 * r_core * par%LuminosityPerVolume / c_light
  !
  do ibl=1,6
    ! --- Initialize maximum value for sub block ---
    dvradialdtime_maxblock=-0.1*huge(dvradialdtime_maxblock)
    !
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nPress, geo%n(1)-nPress, &
                                  geo%m(2)+nPress, geo%n(2)-nPress, &
                                  geo%m(3)+nPress, geo%n(3)-nPress, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, model, dtime, par, r_core) &
#   if (rhd_shortrad_radpressure_l01 != 0)
    !$OMP   SHARED(drhov1dtime, drhov2dtime, drhov3dtime) & 
#   endif
    !$OMP   PRIVATE(i1,i2,i3, &
    !$OMP           n_MonoPerGrain, a_Grain, Q_rp, kapparho_rp, &
    !$OMP           r2, r, FocL, &
    !$OMP           dvradialdtime, dv1dtime, dv2dtime, dv3dtime) &
    !$OMP   REDUCTION(MAX: dvradialdtime_maxblock)
    !
    !$OMP DO SCHEDULE(STATIC)
    do i3=mb3,nb3
      do i2=mb2,nb2
        do i1=mb1,nb1
          ! --- Compute number of monomers per grain ---
          n_MonoPerGrain        = ZNL + &
                                  (SMASS/par%ar_dustGrainAbu(1)) * &
                                    model%quc(i1,i2,i3,1)/model%rho(i1,i2,i3)
          ! --- Compute grain radius ---
          a_Grain               = (((3.0*AW_Mg2SiO4*PMass)/(fourpi*rhoD_Mg2SiO4)) * &
                                   n_MonoPerGrain)**(1.0/3.0)
          ! --- Compute opacity per grain ---
          Q_rp                  = min(q1_Mg2SiO4*(a_Grain/lambdamax)   + &
                                      q4_Mg2SiO4*(a_Grain/lambdamax)**4   , qmax_Mg2SiO4)
          ! --- Compute number density of grains ---
          kapparho_rp           = pi * a_Grain**2 * Q_rp * model%quc(i1,i2,i3,1) / n_MonoPerGrain
          !
          ! --- Compute radius coordinate (squared and linear) outside core ---
          r2                    = max(model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2, &
                                      r_core**2)
          r                     = sqrt(r2)
          ! --- Compute radiative flux over light speed at given radius point ---
          FocL                  = FocL_CoreLum * (r_core**2/r2)
          !
          ! --- Compute radial acceleration ---
          dvradialdtime         = FocL * kapparho_rp / model%rho(i1,i2,i3)
          dvradialdtime_maxblock=max(dvradialdtime, dvradialdtime_maxblock)
          ! --- Project radial acceleration to the three spatial directions ---
          dv1dtime              = (model%xc1(i1)/r) * dvradialdtime
          dv2dtime              = (model%xc2(i2)/r) * dvradialdtime
          dv3dtime              = (model%xc3(i3)/r) * dvradialdtime
          ! --- Apply acceleration ---
          model%v1(i1,i2,i3)    = model%v1(i1,i2,i3) + dtime*dv1dtime
          model%v2(i1,i2,i3)    = model%v2(i1,i2,i3) + dtime*dv2dtime
          model%v3(i1,i2,i3)    = model%v3(i1,i2,i3) + dtime*dv3dtime
!#
#         if (rhd_shortrad_radpressure_l01 != 0)
!#
          ! --- Save mass-weighted acceleration in 3D arrays ---
          drhov1dtime(i1,i2,i3) = model%rho(i1,i2,i3) * dv1dtime
          drhov2dtime(i1,i2,i3) = model%rho(i1,i2,i3) * dv2dtime
          drhov3dtime(i1,i2,i3) = model%rho(i1,i2,i3) * dv3dtime
!#
#         endif
!#
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
    ! --- Determine maximum radiative acceleration within local model ---
    dvradialdtime_max=max(dvradialdtime_max, dvradialdtime_maxblock)
    write(gl%nc_p, '(A,I3,A,I2,A,2(1PE11.4,2X))') &
      '#', geo%MyProc, ' Block: ', ibl, &
      ' LocalSource: dvradialdtimemax: ', dvradialdtime_max, dvradialdtime_maxblock
    !
  end do ! ibl
  !
  call timing_stop('RAD: LocalPress L constant')
  !
else ! (par%radThinPressScheme)
  ! === Apply no acceleration ======================================================================
  !
  call timing_start('RAD: LocalPress None')
  !
  ! --- Set maximum value to zero ---
  dvradialdtime_max=0.0
  !
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
  do ibl=1,6
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nPress, geo%n(1)-nPress, &
                                  geo%m(2)+nPress, geo%n(2)-nPress, &
                                  geo%m(3)+nPress, geo%n(3)-nPress, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !$OMP PARALLEL DEFAULT(SHARED) &
    !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3) &
#   if (rhd_shortrad_radpressure_l01 != 0)
    !$OMP   SHARED(drhov1dtime, drhov2dtime, drhov3dtime) & 
#   endif
    !$OMP   PRIVATE(i1,i2,i3) &
    !$OMP   REDUCTION(MAX: dvradialdtime_maxblock)
    !
    !$OMP DO SCHEDULE(STATIC)
    do i3=mb3,nb3
      do i2=mb2,nb2
        do i1=mb1,nb1
          ! --- Set mass-weighted acceleration in 3D arrays to zero ---
          drhov1dtime(i1,i2,i3) = 0.0
          drhov2dtime(i1,i2,i3) = 0.0
          drhov3dtime(i1,i2,i3) = 0.0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
  end do ! ibl
!#
# endif
!#
  !
  call timing_stop('RAD: LocalPress None')
  !
endif ! (par%radThinPressScheme)
!
!
! ### Compute and print some global values #########################################################
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Compute global minimum and maximum temperatures ---
  call MPI_AllReduce(MPI_IN_PLACE, T_min, 1, MPI_REAL, MPI_MIN, wg%hyd%Comm, ierrMPI)
  call MPI_AllReduce(MPI_IN_PLACE, T_max, 1, MPI_REAL, MPI_MAX, wg%hyd%Comm, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A,2(1PE13.6))') 'LocalSource: minmax(T):          ', T_min, T_max
  call flush(gl%nc_m)
endif
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Compute global maximum radiative acceleration ---
  call MPI_AllReduce(MPI_IN_PLACE, dvradialdtime_max, 1, MPI_REAL, MPI_MAX, wg%hyd%Comm, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A,1PE13.6)')    'LocalSource: max(dvradialdtime): ', dvradialdtime_max
  call flush(gl%nc_m)
endif
!
!
! ### Optional output of averages ##################################################################
if (Output_flag0) then
!#
# ifdef rhd_shortrad_step_t01
!#
  call timing_start('RAD: LocalSource: final(output)')
!#
# endif
!#
  !
  ! --- Compute temperature and grey opacities ---
  !
  do ibl=0,6
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1),geo%n(1), geo%m(2),geo%n(2), geo%m(3),geo%n(3), &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    if ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3)) then
      if (ibl == 0) then
        !$OMP PARALLEL DEFAULT(NONE) &
        !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, T, rhokappa) &
        !$OMP   PRIVATE(i1,i2,i3, logP1D, dTdei1D)
        !
        !$OMP DO SCHEDULE(STATIC)
        do i3=mb3,nb3
          do i2=mb2,nb2
            do i1=mb1,nb1
              ! --- Set temperature and opacity in sub block 0 to zero ---
              T(       i1,i2,i3)=0.0
              rhokappa(i1,i2,i3)=0.0
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP END PARALLEL
      else ! (ibl /= 0)
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_start('RAD: LocalSource: eosopagrey')
!#
#       endif
!#
        !$OMP PARALLEL DEFAULT(NONE) &
        !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, model, T, rhokappa) &
        !$OMP   PRIVATE(i2,i3, logP1D, dTdei1D)
        !
        !$OMP DO SCHEDULE(STATIC)
        do i3=mb3,nb3
          do i2=mb2,nb2
            ! --- Interpolate EOS data for 1D vector ---
            call eosinter_logPTdT(model%rho(mb1:nb1, i2, i3), &
                                  model%ei( mb1:nb1, i2, i3), &
                                  logP1D(   mb1:nb1        ), &
                                  T(        mb1:nb1, i2, i3), &
                                  dTdei1D(  mb1:nb1        )   )
            !
            ! --- Interpolate opacities for 1D vector ---
            call sub_xkaros_logPT2rhokap(logP1D(   mb1:nb1        ), &
                                         T(        mb1:nb1, i2, i3), &
                                         model%rho(mb1:nb1, i2, i3), &
                                         0, &
                                         rhokappa( mb1:nb1, i2, i3)   )
          !
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP END PARALLEL
!#
#       ifdef rhd_shortrad_formal_t01
!#
        call timing_stop('RAD: LocalSource: eosopagrey')
!#
#       endif
!#
      endif ! (ibl == 0)
    endif ! ((nb1 >= mb1) .and. (nb2 >= mb2) .and. (nb3 >= mb3))
  end do ! ibl
  !
  ! --- Set heating in sub block 0 to zero ---
  do ibl=0,0
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nHeat, geo%n(1)-nHeat, &
                                  geo%m(2)+nHeat, geo%n(2)-nHeat, &
                                  geo%m(3)+nHeat, geo%n(3)-nHeat, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !$OMP PARALLEL DEFAULT(NONE) &
    !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, drhoeidtime) &
    !$OMP   PRIVATE(i1,i2,i3)
    !
    !$OMP DO SCHEDULE(STATIC)
    do i3=mb3,nb3
      do i2=mb2,nb2
        do i1=mb1,nb1
          ! --- Compute energy change, (here: set it to zero) ---
          drhoeidtime(i1,i2,i3) = 0.0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
  end do ! ibl
  !
  ! --- Set acceleration in sub block 0 to zero ---
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
  do ibl=0,0
    ! --- Get indices for current sub block ---
    call rhd_box_RestIndices(ibl, m1      ,n1      , m2      ,n2      , m3      ,n3      , &
                                  geo%m(1)+nPress, geo%n(1)-nPress, &
                                  geo%m(2)+nPress, geo%n(2)-nPress, &
                                  geo%m(3)+nPress, geo%n(3)-nPress, &
                                  mb1     ,nb1     , mb2     ,nb2     , mb3     ,nb3        )
    !
    !$OMP PARALLEL DEFAULT(NONE) &
    !$OMP   SHARED(mb1,nb1, mb2,nb2, mb3,nb3, drhov1dtime, drhov2dtime, drhov3dtime) &
    !$OMP   PRIVATE(i1,i2,i3)
    !
    !$OMP DO SCHEDULE(STATIC)
    do i3=mb3,nb3
      do i2=mb2,nb2
        do i1=mb1,nb1
          ! --- Set mass-weighted acceleration in 3D arrays to zero ---
          drhov1dtime(i1,i2,i3) = 0.0
          drhov2dtime(i1,i2,i3) = 0.0
          drhov3dtime(i1,i2,i3) = 0.0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
  end do ! ibl
!#
# endif
!#
  !
  ! --- Compute averages for current sub block ---
  call rhd_shortrad_StepOutput(model, wg%hyd, m1,n1, m2,n2, m3,n3, 1, &
                               .false., .false., ComputePressure_flag, 1.0, &
                               model%xc1(m1-nFg:n1+nFg), &
                               model%xc2(m2-nFg:n2+nFg), &
                               model%xc3(m3-nFg:n3+nFg), &
                               rhokappa, T, drhoeidtime, Frad1, Frad2, Frad3, &
!#
#                              if (rhd_shortrad_radpressure_l01 != 0)
!#
                               drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                              endif
!#
                               dat)
  !
!#
# ifdef rhd_shortrad_step_t01
!#
  call timing_stop('RAD: LocalSource: final(output)')
!#
# endif
!#
endif ! (Output_flag0)
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
deallocate(drhov1dtime, drhov2dtime, drhov3dtime)
!#
# endif
!#
deallocate(T, rhokappa, drhoeidtime, dTdei, Frad1, Frad2, Frad3)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
!if (gl%nc_m >= 0) then
!  write(gl%nc_m,'(A)') '--- End of LocalSource step -----'
!  call flush(gl%nc_m)
!endif
!
end subroutine rhd_shortrad_LocalSource


!----------********************---------------------------------------------------------------------
subroutine rhd_shortrad_SubStep(model, geo, action, dtime, dtime_limit, &
                                Update_flag, Output_flag, OutIntensity_flag, &
                                  dat, &
                                outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_SubStep ('rhd_short-characteristics_radiation_SubStep')
!
! PURPOSE:
!   Perform 3D radiative energy transfer step employing short-characteristics method.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_SubStep(model, geo, action, dtime, dtime_limit, &
!                             Update_flag=Update_flag, Output_flag=Output_flag, &
!                               OutIntensity_flag=OutIntensity_flag, &
!                               dat=dat, &
!                             outstr=outstr,ierr=ierr)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about model domain decomposition
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Update_flag:  ('Update_flag') logical, optional, if true the internal energy is updated,
!                 default=T
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                 default=F
!   OutIntensity_flag: ('Output_Intensity_flag') logical, optional, if true the output intensities
!                 are computed, otherwise output fluxes, default=F
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model structure with input and output data
!   dat:          ('data') derived type with additional output data (optional)
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, recommended maximum radiative time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                   =0:   normal run
!                   1-99: "small" error (e.g. time step limit violation)
!
! LOCAL VARIABLES:
!   T:            ('Temperature') real 3D array with temperature
!   logP:         ('logarithm_Pressure') real 3D array with natural logarithm of pressure,
!   rho:          ('rho') real 3D array with density, attention: one buffer cell in all directions
!   rhokappa:     ('rho_kappa') real 3D array with opacities
!   drhoeidtime:  ('delta_rho_energy_internal_delta_time') real 3D array, radiative energy change
!   drhov1dtime:  ('delta_rho_velocity_1_delta_time') real 3D array, radiative momentum change
!                 in x1 dir
!   drhov2dtime:  ('delta_rho_velocity_2_delta_time') real 3D array, radiative momentum change
!                 in x2 dir
!   drhov3dtime:  ('delta_rho_velocity_3_delta_time') real 3D array, radiative momentum change
!                 in x3 dir
!   dTdei:        ('delta_Temperature_delta_energy_internal') real 3D array with inverse
!                 heat capacity
!   time_rad:     ('time_radiation') real 3D array with estimate for radiative time scale
!   Frad1:        ('Flux_radiative_1') real 3D array with radiative flux in x1 direction
!   Frad2:        ('Flux_radiative_2') real 3D array with radiative flux in x2 direction
!   Frad3:        ('Flux_radiative_3') real 3D array with radiative flux in x3 direction
!   dx1:          ('delta_x_1') real, cell size in x1 direction (taken from xb1)
!   dx2:          ('delta_x_2') real, cell size in x2 direction (taken from xb2)
!   dx3:          ('delta_x_3') real, cell size in x3 direction (taken from xb3)
!   dtime_allsteps: ('delta_time_all_steps') real with time step over all sub steps
!   dtime_thisstep: ('delta_time_this_step') real
!   dtime_limit_thisstep: ('delta_time_limit_this_step') real
!   dtime_limit_recommend: ('delta_time_limit_recommend') real with recommended step size for
!                          next step
!   dtime_rad:    ('delta_time_radiation') real
!   dtime_rhoei:  ('delta_time_rho_energy_internal') real
!   dtime_min:    ('delta_time_minimum') real
!   dtimerel:     ('delta_time_relative') real, fraction of current sub time step of full time step
!   T_min:        ('Temperature_minimum') real minimum temperature
!   T_max:        ('Temperature_maximum') real maximum temperature
!   eiTable_min:  ('energy_internal_Table_minimum') real with minimum allowed internal-energy value
!   dv1dtime_min: ('delta_velocity_1_delta_time_minimum') real with minimum velocity-1 change
!   dv1dtime_max: ('delta_velocity_1_delta_time_maximum') real with maximum velocity-1 change
!   dv2dtime_min: ('delta_velocity_2_delta_time_minimum') real with minimum velocity-2 change
!   dv2dtime_max: ('delta_velocity_2_delta_time_maximum') real with maximum velocity-2 change
!   dv3dtime_min: ('delta_velocity_3_delta_time_minimum') real with minimum velocity-3 change
!   dv3dtime_max: ('delta_velocity_3_delta_time_maximum') real with maximum velocity-3 change
!   dx_dir:       ('delta_x_directions') 3-element real vector with relative grid step size in
!                 all directions, dx_dir=(/ dx1, dx2, dx3 /) / sqrt(dx1**2+dx2**2+dx3**2)
!   M_axis:       ('Matrix_axis') 3x3 real array, specifying the base axes of the ray system
!   ray_dir0:     ('ray_directions_0') 3xn_dir array with all ray directions,
!                 typically from rhd_shortrad_RayDirections
!   ray_dir:      ('ray_directions') 3xn_dir array with all ray directions,
!                 typically from rhd_shortrad_RayDirections (and a rotation)
!   ray_wei:      ('ray_weights') real N-element vector with ray weights
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   n_subdtime:   ('number_sub_delta_time') integer, number of sub time step, 1=first
!   n_dir:        ('number_directions') integer, total number of rays in ray_dir,...
!   exit_flag:    ('exit_flag') logical to indicate exit of sub-time-step loop
!   Update_flag0: ('Update_flag_0') logical, if true the internal energy is updated
!   Output_flag0: ('Output_flag_0') logical, if true the output fluxes are computed
!   OutIntensity_flag0: ('Output_Intensity_flag_0') logical, if true the output intensities are
!                       computed, otherwise output fluxes
!   ComputeFlux_flag: ('ComputeFlux_flag') logical, if true the output fluxes are computed
!   ComputePressure_flag: ('Compute_Pressure_flag') logical, activate computation of rad. acc.
!   dToverTtime_max: ('delta_Temperature_over_Temperature_time_maximum') real with allowed maximum
!                    time step computed from relative temperature change
!   deiovereitime_max: ('delta_energy_internal_over_energy_iternal_time_maximum') real with
!                      allowed maximum time step computed from relative internal-energy change
!   Frad1_min:    ('Flux_radiative_1_minimum') real with minimum radiative flux in x1 direction
!   Frad1_max:    ('Flux_radiative_1_maximum') real with maximum radiative flux in x1 direction
!   Frad2_min:    ('Flux_radiative_2_minimum') real with minimum radiative flux in x2 direction
!   Frad2_max:    ('Flux_radiative_2_maximum') real with maximum radiative flux in x2 direction
!   Frad3_min:    ('Flux_radiative_3_minimum') real with minimum radiative flux in x3 direction
!   Frad3_max:    ('Flux_radiative_3_maximum') real with maximum radiative flux in x3 direction
!   Frad_str:     ('Flux_radiative_string') character with 'Int' or 'Fer' according to
!   ierr0:        ('integer_error_0') integer with error status
!   outstr0:      ('output_string_0') character with error report
!   omp_MaxThreads: ('openmp_Maximum_Threads') integer with maximum number of threads
!   omp_NumThreads: ('openmp_Number_Threads') integer with number of threads
!   T_min_limit:  ('Temperature_minimum_limit') real with allowed lower limit of temperature
!
! ROUTINES:
!   flush:                 Flush output buffer
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_BCast:             Broadcast a message from one process to all others in the group
!   rhd_baserad_RaySystem: Create list of direction vectors for actual rotation matrix
!   rhd_box_MPIFillGhost:  Fill ghost cells by communicating with neighbor MPI processes
!   rhd_MPI_DtimeErrorDist: Distribute time step, error code, and output string among processes
!   rhd_MPI_ErrorDist:     Distribute error code and output string among processes
!   rhd_shortrad_EOS:      Compute EOS quantities needed for radiative transfer
!   rhd_shortrad_Formal:   Compute formal solution: 3D radiative energy flux and change
!   rhd_shortrad_StepOutput: Derive some averages as additional output quantities
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_baserad_module:    Basic routines for radiation transport
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_MPI_module:        MPI routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!   omp_lib:               Library of system OpenMP definitions and routines
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Grid must be equidistant in all directions, cells must be cubic: dx1=dx2=dx3.
!
! PROCEDURE:
!   Compute the energy change per cell from the formal solution and update
!   the internal energy per cell for a number of radiative sub time steps.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-11-10 (B.F. Uppsala) First version
!   2001-11-07 (B.F.) Modified for short-characteristics scheme
!   2002-03-11 (B.F.) Test if minimum temperature is below absolute limit
!   2002-04-16 (B.F.) Parameter model0 added
!   2002-07-30 (B.F.) deallocate(ray_dir0, ray_dir)
!   2002-08-02 (B.F.) Create new ray star in each sub-timestep
!   2002-10-15 (B.F.) In case of too low temperature: decrease dtime only (ierr0=103->2)
!   2003-02-11 (B.F.) trim(Frad_str) -> Frad_str
!   2003-08-26 (B.F.) Bugfix: dtime_min_thread made thread private
!   2003-09-10 (B.F. Lyon) Correction in test if number of time steps is above absolute limit
!   2004-06-18 (B.F.) ComputePressure_flag independent of update_flag
!   2006-10-09 (B.F.) Remove Ptmp, Ttmp; allocate temporary 3D arrays as pointers
!   2006-10-24 (B.F.) Section to determine Frad_minmax parallelized
!   2006-11-17 (B.F.) Bugfix: initialize also Frad2/3_min/max in that section
!   2007-02-10 (B.F. Lyon) Change in dtime_limit suggestion in case of single iteration
!   2008-08-24 (B.F.) Radiation pressure partially re-inserted
!   2008-08-30 (B.F.) Removed from action structure: radpressure
!   2012-03-08 (B.F.) Give less weight to time scales at points outside sphere
!                     with radius par%r1_rad
!   2012-07-09 (B.F.) Call subroutine rhd_shortrad_TemperatureLimiter
!   2013-01-21 (B.F.) Correct timing-statictics calls;
!                     use both dei estimates for new time step if check is requested at all
!   2013-02-06 (B.F.) Experiments with adjustments of thread number
!   2013-02-18 (B.F.) call timing_stop('SHC: step: dtime: init') now after TemperatureLimiter,
!                     use updated EOS routine, parallelize minmax determination of dv1dtime
!   2014-02-21 (B.F. Uppsala) Replace by-hand OpenMP reduction by declared reduction
!   2014-03-04 (B.F.) Axes xb1, xb2, and xb3 get extra ghost cell
!   2014-03-05 (B.F.) Some first MPI-routine calls
!   2014-03-21 (B.F.) Rename some parameters; parameter model0 and call to LambdaIter removed
!   2014-11-05 (B.F.) Copy additional density arrays into local quc array
!   2014-12-10 (B.F.) Base size of arrays on wg%rad(1)
!   2016-03-12 (B.F.) Actually increase the time step outside sphere with radius par%r1_rad (bugfix)
!   2016-03-12 (B.F.) Start implementing accumulated energy change, controlled with par%C_test5
!   2016-03-15 (B.F.) Control accumulated energy change with C_radAccEnergyFraction
!   2016-03-20 (B.F.) Fill ghost cells with data from other MPI processes
!   2017-08-08 (B.F.) Base size arrays again on model
!   2017-09-05 (B.F.) Create temporary xc axes with ghost cells
!   2017-09-23 (B.F.) Add timing call around MPI ghost-cell-filling call
!   2017-10-08 (B.F.) Start making the number of ghost cells flexible
!   2018-11-28 (B.F.) Rename routine rhd_shortrad_Step -> rhd_shortrad_SubStep
!   2018-11-29 (B.F.) Take model MPI geometry data from input parameter instead of global variable
!   2019-02-28 (B.F.) Implement smooth transition to optical-thin heating
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use const_module, only: sigma
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_prop_module
use rhd_dat_module
use rhd_baserad_module
!$ use omp_lib
use rhd_MPI_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(geo_type),            intent(in)   :: geo
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,          optional,intent(in)   :: Update_flag, Output_flag, OutIntensity_flag
type(dat_type),   optional,intent(inout):: dat
integer,          optional,intent(out)  :: ierr
character(len=*), optional,intent(out)  :: outstr
!
! --- Local variables ---
integer, parameter                      :: nFb=1, nFg=1
real, dimension(model%m1-nFg:model%n1+nFg) &
                                        :: xc1
real, dimension(model%m2-nFg:model%n2+nFg) &
                                        :: xc2
real, dimension(model%m3-nFg:model%n3+nFg) &
                                        :: xc3
real, dimension(model%m1-nFg:model%n1+nFg+1) &
                                        :: xb1
real, dimension(model%m2-nFg:model%n2+nFg+1) &
                                        :: xb2
real, dimension(model%m3-nFg:model%n3+nFg+1) &
                                        :: xb3
!#
# if (rhd_arrays_l01 == 0)
!#
real, dimension(model%m1-nFg:model%n1+nFg, &
                model%m2-nFg:model%n2+nFg, &
                model%m3-nFg:model%n3+nFg) &
                                        :: T, logP, dTdei, rho, rhokappa, drhoeidtime, drhoei_Acc
real, dimension(model%m1-nFg:model%n1+nFg, &
                model%m2-nFg:model%n2+nFg, &
                model%m3-nFg:model%n3+nFg, &
                model%nquc           )  :: quc
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(model%m1-nFg:model%n1+nFg, &
                model%m2-nFg:model%n2+nFg, &
                model%m3-nFg:model%n3+nFg) &
                                        :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
real, dimension(model%m1  :model%n1  , &
                model%m2  :model%n2  , &
                model%m3  :model%n3  )  :: time_rad
real, dimension(model%m1-nFb:model%n1+nFb+1, &
                model%m2-nFb:model%n2+nFb  , &
                model%m3-nFb:model%n3+nFb   ) &
                                        :: Frad1
real, dimension(model%m1-nFb:model%n1+nFb  , &
                model%m2-nFb:model%n2+nFb+1, &
                model%m3-nFb:model%n3+nFb   ) &
                                        :: Frad2
real, dimension(model%m1-nFb:model%n1+nFb  , &
                model%m2-nFb:model%n2+nFb  , &
                model%m3-nFb:model%n3+nFb+1 ) &
                                        :: Frad3
!#
# elif (rhd_arrays_l01 == 1)
!#
real, dimension(:,:,:), pointer         :: T, logP, rho, rhokappa, drhoeidtime, &
                                           drhoei_Acc, &
                                           dTdei, time_rad, &
                                           Frad1, &
                                           Frad2, &
                                           Frad3
real, dimension(:,:,:,:), pointer       :: quc
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(:,:,:), pointer         :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
!#
# else
!#     /* (rhd_arrays_l01 == 2) */
real, dimension(:,:,:), allocatable     :: T, logP, rho, rhokappa, drhoeidtime, &
                                           drhoei_Acc, &
                                           dTdei, time_rad, &
                                           Frad1, &
                                           Frad2, &
                                           Frad3
real, dimension(:,:,:,:), allocatable   :: quc
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
real, dimension(:,:,:), allocatable     :: drhov1dtime, drhov2dtime, drhov3dtime
!#
# endif
!#
!#
# endif
!#      /* (rhd_arrays_l01) */
real                                    :: dtime_AllSteps, dtime_ThisStep, dtime_ThisStepOnly, &
                                           dtime_limit_ThisStep, &
                                           dtime_limit_Recommend, &
                                           dtime_rad, dtime_drhoei, dtime_min, &
                                           dtime_Acc, dtime_AccOld, &
                                           dtimerel, &
                                           T_min, T_max, &
                                           eiTable_min, &
                                           dv1dtime_min, dv1dtime_max, &
                                           dv2dtime_min, dv2dtime_max, &
                                           dv3dtime_min, dv3dtime_max, &
                                           drhoeidtime_tmp, drhoei_Acc_tmp, &
                                           fmask
real, dimension(3)                      :: dx, dx_dir
real, dimension(3,3)                    :: M_axis
real, pointer, dimension(:,:)           :: ray_dir0, ray_dir
real, pointer, dimension(:)             :: ray_wei
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3, ng1, ng2, ng3, &
                                           n_subdtime, n_dir, imax, imask
logical                                 :: exit_flag, &
                                           Update_flag0, Output_flag0, OutIntensity_flag0, &
                                           ComputeFlux_flag, ComputePressure_flag
real                                    :: dToverTtime_max, &
                                           deiovereitime_max, &
                                           Frad1_min, Frad1_max, &
                                           Frad2_min, Frad2_max, &
                                           Frad3_min, Frad3_max
character(len=3)                        :: Frad_str
integer                                 :: ierr0
character(len=80)                       :: outstr0
!$ integer                              :: omp_MaxThreads, omp_NumThreads
real, parameter                         :: T_min_limit=100.0
integer                                 :: ierrMPI, ProcErr
!---------------------------------------------------------------------------------------------------
!
call timing_start('SHC: substep')
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_start('SHC: step: init')
!#
# endif
!#
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A)') '--- Start of rhd_shortrad_step ---'
  call flush(gl%nc_m)
endif
!
! --- Initialize error variables: no error occurred, so far ---
ierr0=0
outstr0=''
!
! --- Make some abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set size of small box for non-local radiation transport ---
if (par%N_radThinHeatPoint > 0) then
  imax=max(-geo%m(1), geo%n(1), -geo%m(2), geo%n(2), -geo%m(3), geo%n(3))
else
  imax=-1
endif
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Allocate temporary 3D arrays ---
allocate( &
  T(          m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  logP(       m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  dTdei(      m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  rho(        m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  rhokappa(   m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  drhoeidtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  quc(        m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg, model%nquc), &
  time_rad(   m1    :n1      , m2    :n2      , m3    :n3      ), &
  Frad1(      m1-nFb:n1+nFb+1, m2-nFb:n2+nFb  , m3-nFb:n3+nFb  ), &
  Frad2(      m1-nFb:n1+nFb  , m2-nFb:n2+nFb+1, m3-nFb:n3+nFb  ), &
  Frad3(      m1-nFb:n1+nFb  , m2-nFb:n2+nFb  , m3-nFb:n3+nFb+1)   )
if (par%C_radAccEnergyFraction > 0.0) then
  allocate( &
  drhoei_Acc( m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  )   )
endif
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
allocate( &
  drhov1dtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  drhov2dtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  ), &
  drhov3dtime(m1-nFg:n1+nFg  , m2-nFg:n2+nFg  , m3-nFg:n3+nFg  )   )
!#
# endif
!#      /* (rhd_shortrad_radpressure_l01) */
!#
# endif
!#      /* (rhd_arrays_l01) */
!
! --- Set default values for keywords not present in input list ---
if (.not. present(Update_flag)) then
  Update_flag0=.true.
else
  Update_flag0=Update_flag
endif
if (.not. present(Output_flag)) then
  Output_flag0=.false.
else
  Output_flag0=Output_flag
endif
if (.not. present(OutIntensity_flag)) then
  OutIntensity_flag0=.false.
else
  OutIntensity_flag0=OutIntensity_flag
endif
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
! \label{f90:radpressure__use1}
ComputePressure_flag=((par%radpressure == 'on') .or. (par%radpressure == 'On'))
if (gl%nc_m >= 0) then
  write(gl%nc_m,*) 'ComputePressure_flag: ', ComputePressure_flag
endif
!#
# else
!#
ComputePressure_flag=.false.
!#
# endif
!#      /* (rhd_shortrad_radpressure_l01) */
ComputeFlux_flag=(ComputePressure_flag) .or. (Output_flag0) .or. (OutIntensity_flag0)
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Fill ghost cells with gas and dust density data from neighbor processes ---
!
call timing_start('SHC:   step: FillGhost')
call rhd_box_MPIFillGhost(geo, model, nFg, &
                          grid_mode=0, bulk_mode=1, rho_mode=1, quc_mode=1)
call timing_stop('SHC:   step: FillGhost')
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
! --- Construct local grid axes (with extra ghost cells) ---
! --- Copy the part of the axes that is needed and available ---
ng1=min(nFg,model%nghost1)
xb1(m1-ng1:n1+ng1+1)=model%xb1(m1-ng1:n1+ng1+1)
xc1(m1-ng1:n1+ng1  )=model%xc1(m1-ng1:n1+ng1  )
if (model%nghost1 < nFg) then
  ! --- Extrapolate axis assuming constant grid-cell size ---
  xb1(m1-nFg  :m1-ng1-1)=model%xb1(m1          :m1+nFg-ng1-1)+(model%xb1(m1    )-model%xb1(m1+nFg))
  xb1(n1+ng1+2:n1+nFg+1)=model%xb1(n1-nFg+ng1+2:n1+1        )+(model%xb1(n1+nFg)-model%xb1(n1    ))
  xc1(m1-nFg  :m1-ng1-1)=model%xc1(m1          :m1+nFg-ng1-1)+(model%xb1(m1    )-model%xb1(m1+nFg))
  xc1(n1+ng1+1:n1+nFg  )=model%xc1(n1-nFg+ng1+1:n1          )+(model%xb1(n1+nFg)-model%xb1(n1    ))
endif
!call rhd_box_CheckAxis1D(gl%nc_p, xc1,xb1, m1,n1, nFg, nFg, 'xc1+g: ', 'xb1+g: ', 9, .true., ierr0)
!
ng2=min(nFg,model%nghost2)
xb2(m2-ng2:n2+ng2+1)=model%xb2(m2-ng2:n2+ng2+1)
xc2(m2-ng2:n2+ng2  )=model%xc2(m2-ng2:n2+ng2  )
if (model%nghost2 < nFg) then
  ! --- Extrapolate axis assuming constant grid-cell size ---
  xb2(m2-nFg  :m2-ng2-1)=model%xb2(m2          :m2+nFg-ng2-1)+(model%xb2(m2    )-model%xb2(m2+nFg))
  xb2(n2+ng2+2:n2+nFg+1)=model%xb2(n2-nFg+ng2+2:n2+1        )+(model%xb2(n2+nFg)-model%xb2(n2    ))
  xc2(m2-nFg  :m2-ng2-1)=model%xc2(m2          :m2+nFg-ng2-1)+(model%xb2(m2    )-model%xb2(m2+nFg))
  xc2(n2+ng2+1:n2+nFg  )=model%xc2(n2-nFg+ng2+1:n2          )+(model%xb2(n2+nFg)-model%xb2(n2    ))
endif
!call rhd_box_CheckAxis1D(gl%nc_p, xc2,xb2, m2,n2, nFg, nFg, 'xc2+g: ', 'xb2+g: ', 9, .true., ierr0)
!
ng3=min(nFg,model%nghost3)
xb3(m3-ng3:n3+ng3+1)=model%xb3(m3-ng3:n3+ng3+1)
xc3(m3-ng3:n3+ng3  )=model%xc3(m3-ng3:n3+ng3  )
if (model%nghost3 < nFg) then
  ! --- Extrapolate axis assuming constant grid-cell size ---
  xb3(m3-nFg  :m3-ng3-1)=model%xb3(m3          :m3+nFg-ng3-1)+(model%xb3(m3    )-model%xb3(m3+nFg))
  xb3(n3+ng3+2:n3+nFg+1)=model%xb3(n3-nFg+ng3+2:n3+1        )+(model%xb3(n3+nFg)-model%xb3(n3    ))
  xc3(m3-nFg  :m3-ng3-1)=model%xc3(m3          :m3+nFg-ng3-1)+(model%xb3(m3    )-model%xb3(m3+nFg))
  xc3(n3+ng3+1:n3+nFg  )=model%xc3(n3-nFg+ng3+1:n3          )+(model%xb3(n3+nFg)-model%xb3(n3    ))
endif
!call rhd_box_CheckAxis1D(gl%nc_p, xc3,xb3, m3,n3, nFg, nFg, 'xc3+g: ', 'xb3+g: ', 9, .true., ierr0)
!
! --- Compute typical global grid-cell size: grid assumed to be equidistant ---
!!$dx(1)=(xb1(n1+1)-xb1(m1)) / real(n1+1-m1)
!!$dx(2)=(xb2(n2+1)-xb2(m2)) / real(n2+1-m2)
!!$dx(3)=(xb3(n3+1)-xb3(m3)) / real(n3+1-m3)
dx(:)=(geo%xb_r(:)-geo%xb_l(:)) / real(geo%n(:)-geo%m(:)+1)
!
! --- Construct unit vector through center and corner of grid cell ---
dx_dir=dx / sqrt(dx(1)**2 + dx(2)**2 + dx(3)**2)
!
! --- Get minimum internal energy from EOS table ---
call tabinter_getinfo(x2min=eiTable_min)
!
! --- Initialize ray-direction and -weight arrays ---
nullify(ray_dir0, ray_dir, ray_wei)
!
! --- Initialize time-step limits and counter ---
dtime_limit   =huge(dtime_limit)
dtime_AllSteps=0.0
n_subdtime    =0
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: init')
call timing_start('SHC: step: dtime')
!#
# endif
!#
!
! === Loop over all sub time steps =================================================================
DTIME_LOOP: do
  !
  ! --- Count and check sub time steps ---
  n_subdtime=n_subdtime+1
  if (par%C_radimplicitmu <= 0.0) then
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I4)') 'n_subdtime: ', n_subdtime
      call flush(gl%nc_m)
    endif
    !
    ! \label{f90:n_radmaxiter__use1}\label{f90:n_raditer__use1}
    ! --- Test if number of sub time steps is above absolute limit ---
    if (par%N_radmaxiter > 0) then
      if (n_subdtime > par%N_radmaxiter) then
        ierr0  =1
        outstr0='SHORTrad: Number of sub time steps above absolute limit'
        dtime_limit=dtime * real(max(par%N_raditer  , 1                  )) / &
                            real(max(par%N_raditer+1, par%N_radmaxiter, 2))
        call rhd_MPI_DtimeErrorDist(geo%Comm, geo%MyProc, geo%MaProcWoInCO, &
                                    dtime_limit, ierr0, outstr0, ProcErr)
        exit DTIME_LOOP
      endif ! (n_subdtime)
    endif !  (par%N_radmaxiter)
  endif ! (par%C_radimplicitmu)
  !
!#
# ifdef rhd_shortrad_step_t01
!#
  call timing_start('SHC: step: dtime: init')
!#
# endif
!#
  if (geo%MyProc == geo%MaProcWoInCo) then
    ! --- Set ray directions ---
    call rhd_baserad_RaySystem(action%radraystar, action%radraybase, gl%nc_m, &
                               model%itime, n_subdtime, &
                               n_dir, ray_dir0, ray_dir, ray_wei, M_axis)
  else
    n_dir=0
  endif
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  !
  ! --- Broadcast the ray-system data to all processes ---
  ! write(gl%nc_p, '(A,I3,1X,I3,A,I3)') &
  !   '#', geo%MyProc, geo%MaProcWoInCo, ' n_dir=', n_dir
  call MPI_BCast(  n_dir  , 1      , MPI_INTEGER, geo%MaProcWoInCo, geo%Comm, ierrMPI)
  ! write(gl%nc_p, '(A,I3,1X,I3,A,I3)') &
  !   '#', geo%MyProc, geo%MaProcWoInCo, ' n_dir=', n_dir
  if (n_dir > 0) then
    if (geo%MyProc /= geo%MaProcWoInCo) then
      allocate(ray_dir(3,n_dir))
      allocate(ray_wei(  n_dir))
    endif
    call MPI_BCast(ray_dir, 3*n_dir, MPI_REAL   , geo%MaProcWoInCo, geo%Comm, ierrMPI)
    call MPI_BCast(ray_wei,   n_dir, MPI_REAL   , geo%MaProcWoInCo, geo%Comm, ierrMPI)
  endif
  !
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  ! --- Compute EOS quantities, find minmax temperature, limit temperature, fill buffer layers ---
  call rhd_shortrad_EOS(action, (n_subdtime == 1), model, geo, m1,n1, m2,n2, m3,n3, &
                        rho, quc, T, logP, dTdei, T_min, T_max)
  !
!#
# ifdef rhd_shortrad_step_t01
!#
  call timing_stop('SHC: step: dtime: init')
!#
# endif
!#
  !
  ! --- Test if minimum temperature is below absolute limit ---
  if (T_min < T_min_limit) then
    ierr0  =2
    outstr0='SHORTrad: Minimum temperature below absolute limit'
    dtime_limit=0.8 * dtime
    exit_flag=.true.
    exit DTIME_LOOP
  endif
  !
  if (par%C_radimplicitmu <= 0.0) then
    ! --- Handle explicit case ---------------------------------------------------------------------
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_start('SHC: step: dtime: explicit')
!#
#   endif
!#
    ! --- Compute 3D formal solution, return energy change, fluxes, momentum change ---
    call rhd_shortrad_Formal(geo, action, Output_flag0, &
                             .true., .true., .true., ComputeFlux_flag, ComputePressure_flag, &
                             n_dir, dx_dir, ray_dir, ray_wei, &
                             m1,n1, m2,n2, m3,n3, model%nquc, &
                             xb1, xb2, xb3, &
                             rho, quc, T, logP, &
                             rhokappa, time_rad, Frad1, Frad2, Frad3, drhoeidtime, &
!#
#                            if (rhd_shortrad_radpressure_l01 != 0)
!#
                             drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                            endif
!#
                             ierr=ierr0, outstr=outstr0)
    !
    ! write(gl%nc_p, '(A,I3,1X,I3,A,I3,A)') &
    !   '#', geo%MyProc, geo%MaProcWoInCo, ' ierr=', ierr0, ' outstr='//trim(outstr0)
    ! call MPI_Barrier(geo%Comm, ierrMPI)
    call rhd_MPI_ErrorDist(geo%Comm, geo%MyProc, geo%MaProcWoInCO, &
                           ierr0, outstr0, ProcErr)
    ! write(gl%nc_p, '(A,I3,1X,I3,A,I3,A)') &
    !   '#', geo%MyProc, geo%MaProcWoInCo, ' ierr=', ierr0, ' outstr='//trim(outstr0)
    !
    if (ierr0 >= 100) then
      ! --- A severe error occurred: exit gracefully ---
!#
#     ifdef rhd_shortrad_step_t01
!#
      call timing_stop('SHC: step: dtime: explicit')
!#
#     endif
!#
      exit DTIME_LOOP
    endif
    !
    ! --- Time step restrictions in explicit case:                          ---
    ! --- Check change in internal energy: derive characteristic time scale ---
    ! --- dT=dtime*drhoeidtime*dTdei/model%rho                              ---
    ! --- dtime_drhoei=dtime / max(maxval(abs(dT/T)), 1.0E-06)              ---
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_start('SHC: step: dtime: explicit: fini')
!#
#   endif
!#
    ! --- Initialize reduction variables ---
    dtime_min        =huge(dtime_min)
    dToverTtime_max  =1.0E-06/dtime
    deiovereitime_max=1.0E-06/dtime
    !
!!$!#
!!$#   ifdef _OPENMP
!!$!#
!!$    !write(6,*) 'SHC: step: dtime: explicit: fini'
!!$    !call flush(6)
!!$    omp_MaxThreads=omp_get_max_threads()
!!$    !write(6,*) 'omp_get_max_threads():', omp_MaxThreads
!!$    !call flush(6)
!!$    omp_NumThreads=max(omp_MaxThreads/max(par%N_test5, 1), 1)
!!$    !write(6,*) 'omp_NumThreads       :', omp_NumThreads
!!$    !call flush(6)
!!$    call omp_set_num_threads(omp_NumThreads)
!!$    !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$    !call flush(6)
!!$!#
!!$#   endif /* _OPENMP */
!!$!#
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO SCHEDULE(RUNTIME) &
    !$OMP   PRIVATE(i1,i2,i3, imask, fmask) &
    !$OMP   REDUCTION(MIN: dtime_min) &
    !$OMP   REDUCTION(MAX: dToverTtime_max, deiovereitime_max)
    do i3=m3,n3
      do i2=m2,n2
        ! \label{f90:r1_rad__use1}
        if (par%r1_rad > 0.0) then
          ! --- Give less weight to time scales at points                     ---
          ! --- outside sphere with radius par%r1_rad by increasing time_rad. ---
          ! --- Accept only positive values of change of radiative energy     ---
          ! --- outside sphere with radius par%r1_rad.                        ---
          do i1=m1,n1
            if ((xb1(i1)**2+xb2(i2)**2+xb3(i3)**2) > par%r1_rad**2) then
              time_rad(   i1,i2,i3)=time_rad(i1,i2,i3) * 1.0E+02
              drhoeidtime(i1,i2,i3)=max(drhoeidtime(i1,i2,i3), 0.0)
            endif
          end do ! i1
        endif ! (par%r1_rad > 0.0)
        !
        if (imax >= 0) then
          ! --- Construct and apply mask to allow smooth transition to optically-thin heating ---
          do i1=m1,n1
            imask                =max(min(max(abs(i1), abs(i2), abs(i3)), imax+1) - &
                                        (imax-par%N_radThinHeatPoint), &
                                      0)
            fmask                =1.0 - real(imask)/real(par%N_radThinHeatPoint+1)
            time_rad(   i1,i2,i3)=time_rad(i1,i2,i3) * max(fmask, 1.0E-2)
            drhoeidtime(i1,i2,i3)=drhoeidtime(i1,i2,i3) * fmask
          end do ! i1
        endif ! (imax >= 0)
        !
        do i1=m1,n1
          time_rad(i1,i2,i3)=rho(i1,i2,i3) / &
                             (sigma * T(i1,i2,i3)*T(i1,i2,i3)**2 * dTdei(i1,i2,i3) * &
                              time_rad(i1,i2,i3))
          dtime_min         =min(dtime_min, time_rad(i1,i2,i3))
          dToverTtime_max   =max(dToverTtime_max, &
                                 abs(drhoeidtime(i1,i2,i3))*dTdei(i1,i2,i3) / &
                                   (model%rho(i1,i2,i3)*T(i1,i2,i3)) &
                                )
          deiovereitime_max =max(deiovereitime_max, &
                                 abs(drhoeidtime(i1,i2,i3))/ &
                                   (model%rho(i1,i2,i3)*(model%ei(i1,i2,i3)-eiTable_min)) &
                                )
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
!!$!#
!!$#   ifdef _OPENMP
!!$!#
!!$    !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$    call omp_set_num_threads(omp_MaxThreads)
!!$    !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$!#
!!$#   endif /* _OPENMP */
!!$!#
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_stop('SHC: step: dtime: explicit: fini')
    call timing_stop('SHC: step: dtime: explicit')
!#
#   endif
!#
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Compute global minimum/maximum values ---
    call MPI_AllReduce(MPI_IN_PLACE, dtime_min        , 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, dToverTtime_max  , 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, deiovereitime_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! \label{f90:c_radcourantmax__use1}
    ! --- Compute maximum time step ---
    dtime_drhoei=1.0 / dToverTtime_max
    dtime_rad   =par%C_radCourant*dtime_min
    !
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,1PE14.6)') 'Time step ratio: dtime/dtime_rad: ', dtime/dtime_min
    endif
    !
    ! --- Compute absolute upper limit for this sub time step from radiative time scale ---
    dtime_limit_ThisStep=par%C_radCourantMax*dtime_min
    ! \label{f90:n_radminiter__use1}
    ! --- Check minimum iteration number ---
    if (par%N_radminiter > 0) then
      dtime_limit_ThisStep=min(dtime_limit_ThisStep, dtime/(real(par%N_radminiter)-0.9))
    endif
    ! \label{f90:c_radmaxeichange__use1}
    ! --- Compute time step due to maximum change in internal energy ---
    if (par%C_radmaxeiChange > 0.0) then
      dtime_limit_ThisStep=min(dtime_limit_ThisStep, par%C_radmaxeiChange*dtime_drhoei)
    endif
    !
    ! \label{f90:c_radcourant__use1}
    ! --- Compute recommended value for this sub time step from radiative time scale ---
    dtime_limit_Recommend=dtime_rad
    ! --- Check minimum iteration number ---
    if (par%N_radminiter > 0) then
      dtime_limit_Recommend=min(dtime_limit_Recommend, dtime/(real(par%N_radminiter)-0.9))
    endif
    ! --- Compute time step due to maximum change in internal energy ---
    if (par%C_radmaxeiChange > 0.0) then
      dtime_limit_Recommend=min(dtime_limit_Recommend, par%C_radmaxeiChange*dtime_drhoei    )
      dtime_limit_Recommend=min(dtime_limit_Recommend, 0.8                 /deiovereitime_max)
    endif
    !
    ! --- Check if current sub time step is large enough to be the last one ---
    if (dtime_limit_ThisStep >= (dtime-dtime_AllSteps)) then
      ! --- Take the remaining time interval as current sub time step ---
      dtime_ThisStepOnly=dtime-dtime_AllSteps
      exit_flag=.true.
      !
    else
      ! --- Take the recommended time step as current sub time step ---
      dtime_ThisStepOnly=dtime_limit_Recommend
      exit_flag=.false.
    endif
    !
    ! --- Print time step limits ---
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,4(1PE10.3))')     'dtime_:rad,drhoei,limit_this,all: ', &
                                dtime_rad, dtime_drhoei,dtime_limit_ThisStep, dtime_AllSteps
      write(gl%nc_m,'(A,10X,1(1PE10.3))') '           drhoei(dei/ei):        ', &
                                           1.0/deiovereitime_max
    endif
    !
    ! === Check if the accumulated energy change should be computed and used ===
    if (par%C_radAccEnergyFraction <= 0.0) then
      ! --- Use the energy change from the current sub step only ---
      dtime_ThisStep=dtime_ThisStepOnly
      !
    else ! (par%C_radAccEnergyFraction > 0.0)
      ! --- Use the accumulated energy change ---
      !
      ! --- Add accumulated and current energy change ---
      !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1, i2, i3, drhoeidtime_tmp, drhoei_Acc_tmp)
      !
      if (n_subdtime <= 1) then
        !
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3,n3
          do i2=m2,n2
            do i1=m1,n1
              drhoeidtime_tmp      =drhoeidtime(i1,i2,i3)
              drhoei_Acc( i1,i2,i3)=(dtime_ThisStepOnly*par%C_radAccEnergyFraction) * &
                                                                         drhoeidtime_tmp
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP MASTER
        dtime_AccOld=0.0
        dtime_Acc   =dtime_ThisStepOnly*par%C_radAccEnergyFraction
        !$OMP END MASTER
        !
      else ! (n_subdtime > 1)
        ! --- Use and update existing accumulated energy change ---
        !
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3,n3
          do i2=m2,n2
            do i1=m1,n1
              drhoeidtime_tmp      =drhoeidtime(i1,i2,i3)
              drhoei_Acc_tmp       =drhoei_Acc( i1,i2,i3)
              drhoeidtime(i1,i2,i3)=(                                    drhoei_Acc_tmp + &
                                     dtime_ThisStepOnly                * drhoeidtime_tmp ) / &
                                    (dtime_Acc + dtime_ThisStepOnly)
              drhoei_Acc( i1,i2,i3)=                                     drhoei_Acc_tmp + &
                                    (dtime_ThisStepOnly*par%C_radAccEnergyFraction) * &
                                                                         drhoeidtime_tmp
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP MASTER
        dtime_AccOld=dtime_Acc
        dtime_Acc   =dtime_Acc + dtime_ThisStepOnly*par%C_radAccEnergyFraction
        !$OMP END MASTER
        !
      endif ! (n_subdtime)
      !
      !$OMP END PARALLEL
      !
      ! --- Compute maximum relative temperature and energy change ---
      !
      ! --- Initialize reduction variables ---
      dToverTtime_max  =1.0E-06/dtime
      deiovereitime_max=1.0E-06/dtime
      !
      !$OMP PARALLEL DEFAULT(SHARED) &
      !$OMP   PRIVATE(i1, i2, i3)
      !
      !$OMP DO SCHEDULE(RUNTIME) &
      !$OMP   REDUCTION(MAX: dToverTtime_max, deiovereitime_max)
      do i3=m3,n3
        do i2=m2,n2
          ! \label{f90:r1_rad__use2}
          if (par%r1_rad > 0.0) then
            ! --- Accept only positive values of change of radiative energy     ---
            ! --- outside sphere with radius par%r1_rad.                        ---
            do i1=m1,n1
              if ((xb1(i1)**2+xb2(i2)**2+xb3(i3)**2) > par%r1_rad**2) then
                drhoeidtime(i1,i2,i3)=max(drhoeidtime(i1,i2,i3), 0.0)
              endif
            end do ! i1
          endif
          !
          if (imax >= 0) then
            ! --- Construct and apply mask to allow smooth transition to optically-thin heating ---
            do i1=m1,n1
              imask                =max(min(max(abs(i1), abs(i2), abs(i3)), imax+1) - &
                                          (imax-par%N_radThinHeatPoint), &
                                        0)
              fmask                =1.0 - real(imask)/real(par%N_radThinHeatPoint+1)
              drhoeidtime(i1,i2,i3)=drhoeidtime(i1,i2,i3) * fmask
            end do ! i1
          endif ! (imax >= 0)
          !
          do i1=m1,n1
            dToverTtime_max   =max(dToverTtime_max, &
                                   abs(drhoeidtime(i1,i2,i3))*dTdei(i1,i2,i3) / &
                                     (model%rho(i1,i2,i3)*T(i1,i2,i3)) &
                                  )
            deiovereitime_max =max(deiovereitime_max, &
                                   abs(drhoeidtime(i1,i2,i3))/ &
                                     (model%rho(i1,i2,i3)*(model%ei(i1,i2,i3)-eiTable_min)) &
                                  )
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      ! --- Compute global minimum/maximum values ---
      call MPI_AllReduce(MPI_IN_PLACE, dToverTtime_max  , 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
      call MPI_AllReduce(MPI_IN_PLACE, deiovereitime_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      ! --- Compute maximum time step ---
      dtime_drhoei=1.0 / dToverTtime_max
      !
      ! --- Derive the current sub time step, overrule the previous value ---
      !
      ! --- Compute absolute upper limit for this sub time step ---
      dtime_limit_ThisStep=dtime_AccOld + dtime_ThisStepOnly
      ! \label{f90:n_radminiter__use2}
      ! --- Check minimum iteration number ---
      if (par%N_radminiter > 0) then
        dtime_limit_ThisStep=min(dtime_limit_ThisStep, dtime/(real(par%N_radminiter)-0.9))
      else
        dtime_limit_ThisStep=0.0
      endif
      ! \label{f90:c_radmaxeichange__use2}
      ! --- Compute time step due to maximum change in internal energy ---
      if (par%C_radmaxeiChange > 0.0) then
        dtime_limit_ThisStep=min(dtime_limit_ThisStep, par%C_radmaxeiChange*dtime_drhoei)
      endif
      !
      ! \label{f90:c_radcourant__use2}
      ! --- Compute recommended value for this sub time step ---
      ! --- Check minimum iteration number ---
      dtime_limit_Recommend=dtime_AccOld + dtime_ThisStepOnly
      if (par%N_radminiter > 0) then
        dtime_limit_Recommend=min(dtime_limit_Recommend, dtime/(real(par%N_radminiter)-0.9))
      endif
      ! --- Compute time step due to maximum change in internal energy ---
      if (par%C_radmaxeiChange > 0.0) then
        dtime_limit_Recommend=min(dtime_limit_Recommend, par%C_radmaxeiChange*dtime_drhoei     )
        dtime_limit_Recommend=min(dtime_limit_Recommend, 0.8                 /deiovereitime_max)
      endif
      !
      ! --- Check if current sub time step is large enough to be the last one ---
      if (dtime_limit_ThisStep >= (dtime-dtime_AllSteps)) then
        ! --- Take the remaining time interval as current sub time step ---
        dtime_ThisStep=dtime-dtime_AllSteps
        exit_flag=.true.
        !
      else ! (dtime_limit_ThisStep)
        ! --- Take the recommended time step as current sub time step ---
        dtime_ThisStep=dtime_limit_Recommend
        exit_flag=.false.
      endif ! (dtime_limit_ThisStep)
      !
      ! --- Print total time step limits ---
      if (gl%nc_m >= 0) then
        write(gl%nc_m,'(A,4(1PE10.3))')     'dtime_;rad,drhoei,limit_this,Acc: ', &
                                  dtime_rad, dtime_drhoei,dtime_limit_ThisStep, dtime_Acc
        write(gl%nc_m,'(A,10X,1(1PE10.3))') '           drhoei(dei/ei):        ', &
                                             1.0/deiovereitime_max
      endif
      !
    endif ! (par%C_radAccEnergyFraction)
    !
    ! --- Test if time step is below absolute limit ---
    if (dtime_limit_ThisStep < par%dtime_min_stop) then
      ierr0  =100
      outstr0='SHORTrad: Time step below absolute limit'
      dtime_limit=dtime_limit_ThisStep
      exit_flag=.true.
      if (Update_flag0) exit DTIME_LOOP
    endif
    !
    ! --- Suggest new time step for radiation transport ---
    if ((exit_flag) .and. (ierr0 == 0)) then
      if ((par%N_raditer >= max(1, par%N_radminiter)) .and. &
          ((par%N_raditer > 1) .or. (n_subdtime > 1))         ) then
        ! --- Suggest time step for iteration mode ---
        dtime_limit=dtime*real(par%N_raditer)/real(n_subdtime)
      else ! (par%N_raditer)
        ! --- Suggest time step in case of no iteration ---
        dtime_limit=dtime_rad
        ! --- Compute time-step limit due to maximum change in internal energy ---
        if (par%C_radmaxeiChange > 0.0) then
          dtime_limit=min(dtime_limit, par%C_radmaxeiChange*dtime_drhoei)
        endif
      endif ! (par%N_raditer)
    endif ! (exit_flag)
    !
  else ! (par%C_radimplicitmu > 0.0)
    ! --- Handle implicit case: perform Lambda iteration -------------------------------------------
    !
!!$    call rhd_shortrad_LambdaIter(model, model0, action, dtime, &
!!$                                 Output_flag0, ComputeFlux_flag, ComputePressure_flag, &
!!$                                 n_dir, dx_dir, ray_dir, ray_wei, &
!!$                                 m1,n1, m2,n2, m3,n3, &
!!$                                 model%xc1, model%xc2, model%xc3, xb1, xb2, xb3, &
!!$                                 model%ei, rho, &
!!$                                 rhokappa, time_rad, Frad1, Frad2, Frad3, drhoeidtime, &
!!$!#
!!$#                                if (rhd_shortrad_radpressure_l01 != 0)
!!$!#
!!$                                 drhov1dtime, drhov2dtime, drhov3dtime, &
!!$!#
!!$#                                endif
!!$!#
!!$                                 dtime_limit, ierr=ierr0, outstr=outstr0)
!!$    exit_flag=.true.
!!$    dtime_ThisStep=dtime
  endif ! (par%C_radimplicitmu)
  !
  ! --- Explicit or implicit case done: drhoeidtime, dtime_limit computed --------------------------
!#
# ifdef rhd_shortrad_step_t01
!#
  call timing_start('SHC: step: dtime: final')
!#
# endif
!#
  !
  if (Update_flag0) then
    dtimerel=dtime_ThisStep/dtime
  else
    dtimerel=1.0
  endif
  !
!!$!#
!!$# ifdef _OPENMP
!!$!#
!!$  !write(6,*) 'SHC: step: dtime: final'
!!$  !call flush(6)
!!$  omp_MaxThreads=omp_get_max_threads()
!!$  !write(6,*) 'omp_get_max_threads():', omp_MaxThreads
!!$  !call flush(6)
!!$  omp_NumThreads=max(omp_MaxThreads/max(par%N_test5, 1), 1)
!!$  !write(6,*) 'omp_NumThreads       :', omp_NumThreads
!!$  !call flush(6)
!!$  call omp_set_num_threads(omp_NumThreads)
!!$  !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$  !call flush(6)
!!$!#
!!$# endif /* _OPENMP */
!!$!#
  ! --- Update output fluxes (for equidistant grid) ------------------------------------------------
  if ((Output_flag0) .and. present(dat)) then
    !
    ! === Find minimum and maximum fluxes ===
    !
    Frad1_min=Frad1(m1,m2,m3)
    Frad1_max=Frad1(m1,m2,m3)
    Frad2_min=Frad2(m1,m2,m3)
    Frad2_max=Frad2(m1,m2,m3)
    Frad3_min=Frad3(m1,m2,m3)
    Frad3_max=Frad3(m1,m2,m3)
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    ! --- Find minimum and maximum of radiative fluxes in x1 direction ---
    !
    !$OMP DO SCHEDULE(RUNTIME) &
    !$OMP   REDUCTION(MIN: Frad1_min) &
    !$OMP   REDUCTION(MAX: Frad1_max)
    do i3=m3, n3
      Frad1_min=min(Frad1_min, minval(Frad1(m1:n1+1,m2:n2,i3)))
      Frad1_max=max(Frad1_max, maxval(Frad1(m1:n1+1,m2:n2,i3)))
    end do ! i3
    !$OMP END DO NOWAIT
    !
    ! --- Find minimum and maximum of radiative fluxes in x2 direction ---
    !
    !$OMP DO SCHEDULE(RUNTIME) &
    !$OMP   REDUCTION(MIN: Frad2_min) &
    !$OMP   REDUCTION(MAX: Frad2_max)
    do i3=m3, n3
      Frad2_min=min(Frad2_min, minval(Frad2(m1:n1,m2:n2+1,i3)))
      Frad2_max=max(Frad2_max, maxval(Frad2(m1:n1,m2:n2+1,i3)))
    end do ! i3
    !$OMP END DO NOWAIT
    !
    ! --- Find minimum and maximum of radiative fluxes in x3 direction ---
    !
    !$OMP DO SCHEDULE(RUNTIME) &
    !$OMP   REDUCTION(MIN: Frad3_min) &
    !$OMP   REDUCTION(MAX: Frad3_max)
    do i3=m3, n3+1
      Frad3_min=min(Frad3_min, minval(Frad3(m1:n1,m2:n2,i3)))
      Frad3_max=max(Frad3_max, maxval(Frad3(m1:n1,m2:n2,i3)))
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Compute global minimum/maximum values of radiative fluxes ---
    call MPI_AllReduce(MPI_IN_PLACE, Frad1_min, 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Frad1_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Frad2_min, 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Frad2_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Frad3_min, 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Frad3_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    if (gl%nc_m >= 0) then
      if (OutIntensity_flag0) then
        Frad_str='Int'
      else
        Frad_str='Fer'
      endif
      !
      write(gl%nc_m,'(A,3(2E11.4,2X))') 'minmax(' // Frad_str // '1,2,3): ', &
                                    Frad1_min, Frad1_max, Frad2_min, Frad2_max, Frad3_min, Frad3_max
      call flush(gl%nc_m)
    endif
    !
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_start('SHC: step: dtime: final(output)')
!#
#   endif
!#
    call rhd_shortrad_StepOutput(model, geo, m1,n1, m2,n2, m3,n3, n_subdtime, &
                                 OutIntensity_flag0, .true., ComputePressure_flag, dtimerel, &
                                 xc1, xc2, xc3, &
                                 rhokappa, T, drhoeidtime, Frad1, Frad2, Frad3, &
!#
#                                if (rhd_shortrad_radpressure_l01 != 0)
!#
                                 drhov1dtime, drhov2dtime, drhov3dtime, &
!#
#                                endif
!#
                                 dat)
!#
#   ifdef rhd_shortrad_step_t01
!#
    call timing_stop('SHC: step: dtime: final(output)')
!#
#   endif
!#
    !
  endif ! (Output_flag0)
  !
  ! --- Update energy (and momentum if radiation pressure is activated) ----------------------------
  if (Update_flag0) then
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      dv1dtime_min= 0.1*huge(dv1dtime_min)
      dv1dtime_max=-0.1*huge(dv1dtime_max)
      dv2dtime_min= 0.1*huge(dv2dtime_min)
      dv2dtime_max=-0.1*huge(dv2dtime_max)
      dv3dtime_min= 0.1*huge(dv3dtime_min)
      dv3dtime_max=-0.1*huge(dv3dtime_max)
    endif
!#
#   endif
!#        /* rhd_shortrad_radpressure_l01 */
    ! --- Update internal energy (and velocity if radiation pressure enabled) ---
    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(i1,i2,i3)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                             dtime_ThisStep*drhoeidtime(i1,i2,i3)/model%rho(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
!#
#   if (rhd_shortrad_radpressure_l01 != 0)
!#
    if (ComputePressure_flag) then
      !$OMP DO SCHEDULE(RUNTIME) &
      !$OMP   REDUCTION(MIN: dv1dtime_min, dv2dtime_min, dv3dtime_min) &
      !$OMP   REDUCTION(MAX: dv1dtime_max, dv2dtime_max, dv3dtime_max)
      do i3=m3,n3
        do i2=m2,n2
          do i1=m1,n1
            dv1dtime_min=min(dv1dtime_min, drhov1dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            dv1dtime_max=max(dv1dtime_max, drhov1dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            dv2dtime_min=min(dv2dtime_min, drhov2dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            dv2dtime_max=max(dv2dtime_max, drhov2dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            dv3dtime_min=min(dv3dtime_min, drhov3dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            dv3dtime_max=max(dv3dtime_max, drhov3dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            model%v1(i1,i2,i3)=model%v1(i1,i2,i3) + &
                               dtime_ThisStep*(drhov1dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            model%v2(i1,i2,i3)=model%v2(i1,i2,i3) + &
                               dtime_ThisStep*(drhov2dtime(i1,i2,i3)/model%rho(i1,i2,i3))
            model%v3(i1,i2,i3)=model%v3(i1,i2,i3) + &
                               dtime_ThisStep*(drhov3dtime(i1,i2,i3)/model%rho(i1,i2,i3))
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      !$OMP MASTER
      ! --- Compute global minimum/maximum values of momentum changes ---
      call MPI_AllReduce(MPI_IN_PLACE, dv1dtime_min, 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
      call MPI_AllReduce(MPI_IN_PLACE, dv1dtime_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
      call MPI_AllReduce(MPI_IN_PLACE, dv2dtime_min, 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
      call MPI_AllReduce(MPI_IN_PLACE, dv2dtime_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
      call MPI_AllReduce(MPI_IN_PLACE, dv3dtime_min, 1, MPI_REAL, MPI_MIN, geo%Comm, ierrMPI)
      call MPI_AllReduce(MPI_IN_PLACE, dv3dtime_max, 1, MPI_REAL, MPI_MAX, geo%Comm, ierrMPI)
      !$OMP END MASTER
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
    !
      !
      !$OMP MASTER
      if (gl%nc_m >= 0) then
        write(gl%nc_m,'(A,2(1PE13.6))') 'minmax(dv1dtime): ', dv1dtime_min, dv1dtime_max
        write(gl%nc_m,'(A,2(1PE13.6))') 'minmax(dv2dtime): ', dv2dtime_min, dv2dtime_max
        write(gl%nc_m,'(A,2(1PE13.6))') 'minmax(dv3dtime): ', dv3dtime_min, dv3dtime_max
      endif
      !$OMP END MASTER
      !
    endif ! (ComputePressure_flag)
!#
#   endif
!#        /* rhd_shortrad_radpressure_l01 */
    !
    !$OMP END PARALLEL
  else ! (.not. Update_flag0)
    exit_flag=.true.
  endif ! (Update_flag0)
  !
!!$!#
!!$# ifdef _OPENMP
!!$!#
!!$  !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$  call omp_set_num_threads(omp_MaxThreads)
!!$  !write(6,*) 'omp_get_max_threads():', omp_get_max_threads()
!!$!#
!!$# endif /* _OPENMP */
!!$!#
  ! --- Update sub time step ---
  dtime_AllSteps=dtime_AllSteps+dtime_ThisStep
  !
!#
# ifdef rhd_shortrad_step_t01
!#
  call timing_stop('SHC: step: dtime: final')
!#
# endif
!#
  ! --- Check for exit ---
  if (exit_flag) exit DTIME_LOOP
  !
end do DTIME_LOOP
! ==================================================================================================
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: dtime')
call timing_start('SHC: step: final')
!#
# endif
!#
if (associated(ray_dir0)) deallocate(ray_dir0)
if (associated(ray_dir )) deallocate(ray_dir )
if (associated(ray_wei )) deallocate(ray_wei )
!
!#
# if (rhd_arrays_l01 != 0)
!#
! --- Deallocate temporary 3D arrays ---
!#
# if (rhd_shortrad_radpressure_l01 != 0)
!#
deallocate(drhov1dtime, drhov2dtime, drhov3dtime)
!#
# endif
!#
if (par%C_radAccEnergyFraction > 0.0) then
  deallocate(drhoei_Acc)
endif
deallocate(T, logP, rho, rhokappa, drhoeidtime, quc, dTdei, time_rad, Frad1, Frad2, Frad3)
!#
# endif
!#      /* (rhd_arrays_l01) */
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
!if (gl%nc_m >= 0) then
!  write(gl%nc_m,'(A)') '--- End of rhd_shortrad_step -----'
!  call flush(gl%nc_m)
!endif
!
!#
# ifdef rhd_shortrad_step_t01
!#
call timing_stop('SHC: step: final')
!#
# endif
!#
call timing_stop('SHC: substep')
!
end subroutine rhd_shortrad_SubStep


!----------*****************------------------------------------------------------------------------
subroutine rhd_shortrad_Step(model, action, dtime, dtime_limit, &
                             Update_flag, Output_flag, OutIntensity_flag, &
                               dat, datRadLrg, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_Step ('rhd_short-characteristics_radiation_Step')
!
! PURPOSE:
!   Perform 3D radiation transport step employing short-characteristics and/or local-source scheme.
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call rhd_shortrad_Step(model, action, dtime, dtime_limit, &
!                          Update_flag=Update_flag, Output_flag=Output_flag, &
!                            OutIntensity_flag=OutIntensity_flag, &
!                            dat=dat, datRadLrg=datRadLrg, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Update_flag:  ('Update_flag') logical, optional, if true the internal energy is updated,
!                 default=T
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                 default=F
!   OutIntensity_flag: ('Output_Intensity_flag') logical, optional, if true the output intensities
!                 are computed, otherwise output fluxes, default=F
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model structure with input and output data
!   dat:          ('data') derived type with additional output data for model;
!                 optional, required if Output_flag=.true.
!   datRadLrg:    ('data_Radiation_Large') derived type with additional output data for global
!                 non-local radiation-transport box;
!                 optional, required for MPI master process if Output_flag=.true.
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, recommended maximum radiative time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                   =0:   normal run
!                   1-99: "small" error (e.g. time step limit violation)
!
! LOCAL VARIABLES:
!   dtime_limit_SHORTrad: ('delta_time_limit_SHORT_characteristcs_radiation') real,
!                         time step limit for non-local SHORTrad step
!   dtime_limit_LocalSource: ('delta_time_limit_Local_Source') real,
!                         time step limit for local-source step
!   CentralBox:   ('Central_Box') type(box_type), data for small central SHORTrad box: rho, ei, ...
!   ierr0:        ('integer_error_0') integer with error status
!   outstr0:      ('output_string_0') character with error report
!   ierrMPI:      ('integer_error_MPI') integer with error status of MPI routines
!
! ROUTINES:
!   flush:                 Flush output buffer
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!   rhd_box_MPICopy:       Copy distribution of boxes from one MPI partition to another
!   rhd_dat_AddOverlap:    Add overlap of contents of two structures for additional data
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_shortrad_LocalSource: Perform radiation update step employing local approximation
!   rhd_shortrad_SubStep:  Perform 3d radiative energy transfer step using short characteristics
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_MPI_module:        MPI routines specific for RHD code
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The sub grid on which the SHORTrad step is performed has to be equidistant, cells must be cubic.
!
! PROCEDURE:
!   If no optically thin points exist, just call the SHORTrad routine.
!   Otherwise, copy data to a smaller box, on which the SHORTrad step is performed,
!   and perform the LocalSourse step on the optically thin rest.
!
! EXAMPLE:
!
! TODO:
!   Handle mean data in dat structure
!
! MODIFICATION HISTORY:
!   2018-11-28 (B.F. Uppsala) First version
!   2019-02-01 (B.F.) Add parameter datRadLrg
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_prop_module
use rhd_dat_module
use rhd_MPI_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
use timing_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,          optional,intent(in)   :: Update_flag, Output_flag, OutIntensity_flag
type(dat_type),   optional,intent(inout):: dat, datRadLrg
integer,          optional,intent(out)  :: ierr
character(len=*), optional,intent(out)  :: outstr
!
! --- Local variables ---
integer, parameter                      :: nFb=1, nFg=1
logical                                 :: Output_flag0
real                                    :: dtime_limit_SHORTrad, dtime_limit_LocalSource
type(box_type)                          :: CentralBox
type(dat_type)                          :: CentralDat
integer                                 :: ierr0
character(len=80)                       :: outstr0
integer                                 :: ierrMPI
!---------------------------------------------------------------------------------------------------
!
call timing_start('SHC: step')
!
! --- Set a default value for the output flag if it is not present ---
if (present(Output_flag)) then
  Output_flag0=Output_flag
else
  Output_flag0=.false.
endif
!
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A)') '=== Start of SHORTrad steps ======'
  call flush(gl%nc_m)
endif
!
! --- Initialize error variables: no error occurred, so far ---
ierr0=0
outstr0=''
!
if (par%N_radThinPoint <= 0) then
  ! === Perform just a non-local short-characteristics sub step ====================================
  !
  call rhd_shortrad_SubStep(model, wg%rad(1), action, dtime, dtime_limit, &
                            Update_flag=Update_flag, Output_flag=Output_flag, &
                              OutIntensity_flag=OutIntensity_flag, &
                              dat=dat, &
                            outstr=outstr0,ierr=ierr0)
  !
else ! (par%N_radThinPoint > 0)
  ! === Perform non-local short-characteristics sub step on smaller central box ====================
  ! === and local-source sub step on the rest                                   ====================
  !
  ! --- Perform non-local short-characteristics sub step on smaller central box --------------------
  !
  call timing_start('SHC: Local boxCopy')
  !
  ! --- Initialize box for central SHORTrad step (don't create it, yet) ---
  call rhd_box_Init(CentralBox)
  !
  ! --- Copy data from large hydro box into small SHORTrad box, transfer data with MPI if needed ---
  call rhd_box_MPICopy(model, wg%hyd, CentralBox, wg%rad(1), &
                       SubBoxMode='All', SubBoxName='all', &
                       nghost1=nFg, nghost2=nFg, nghost3=nFg, &
                       rho_mode=1, ei_mode=1, v_mode=0, quc_mode=1)
  !
  call timing_stop('SHC: Local boxCopy')
  !
  if (Output_flag0) then
    ! --- Set data fields to zero, make radial axis ---
    call rhd_dat_Make(CentralDat, CentralBox, min(par%N_radband, 2), action=action, &
                      IncludeIntensity_flag=.false., IncludeFlux_flag=.true.)
    call rhd_dat_Reset(CentralDat)
  endif
  !
  ! --- Perform non-local short-characteristics sub step on smaller central box ---
  call rhd_shortrad_SubStep(CentralBox, wg%rad(1), action, dtime, dtime_limit_SHORTrad, &
                            Update_flag=Update_flag, Output_flag=Output_flag, &
                              OutIntensity_flag=OutIntensity_flag, &
                              dat=Centraldat, &
                            outstr=outstr0,ierr=ierr0)
  !
  call timing_start('SHC: Local boxCopy')
  !
  ! --- Copy data from small SHORTrad box into large hydro box, transfer data with MPI if needed ---
  call rhd_box_MPICopy(CentralBox, wg%rad(1), model, wg%hyd, &
                       SubBoxMode='All', SubBoxName='all', &
                       nghost1=0, nghost2=0, nghost3=0, &
                       rho_mode=0, ei_mode=1, v_mode=0, quc_mode=0)
  !
  ! --- Delete box for central SHORTrad step (previously created by rhd_box_MPICopy) ---
  call rhd_box_Delete(CentralBox)
  !
  call timing_stop('SHC: Local boxCopy', cycles=0)
  !
  if (Output_flag0) then
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    call rhd_dat_MPICollect(wg%rad(1), Centraldat, datRadLrg, &
                            IncludeIntensity_flag=.false., IncludeFlux_flag=.true.)
!#
#   else
!#       /* rhd_mpi_l01 ------------------------------------ */
    call rhd_dat_AddOverlap(dat, Centraldat)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Delete data structure(s) for output of additional information ---
    call rhd_dat_Delete(CentralDat)
  endif
  !
  ! --- Perform local-source sub step on the outer region ------------------------------------------
  !
  if ((Update_flag) .and. (ierr0 == 0)) then
    call rhd_shortrad_LocalSource(model, wg%rad(1), action, dtime, dtime_limit_LocalSource, &
                                  Output_flag=Output_flag, &
                                    dat=dat, &
                                  outstr=outstr0,ierr=ierr0)
  else
    dtime_limit_LocalSource=huge(dtime_limit_LocalSource)
  endif
  !
  ! --- Handle time-step restrictions --------------------------------------------------------------
  !
  dtime_limit=min(dtime_limit_SHORTrad, dtime_limit_LocalSource)
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Compute global minimum/maximum values ---
  call MPI_AllReduce(MPI_IN_PLACE, dtime_limit, 1, MPI_REAL, MPI_MIN, wg%hyd%Comm, &
                     ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
endif ! (par%N_radThinPoint <= 0)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (gl%nc_m >= 0) then
  write(gl%nc_m,'(A)') '=== End of SHORTrad steps ========'
  write(gl%nc_m,*)
  call flush(gl%nc_m)
endif
!
call timing_stop('SHC: step')
!
end subroutine rhd_shortrad_Step

end module rhd_shortrad_module
