!***************************************************************************************************
!
!   #####   #    #  #####          #    #  #   ####
!   #    #  #    #  #    #         #    #  #  #
!   #    #  ######  #    #         #    #  #   ####
!   #####   #    #  #    #         #    #  #       #
!   #   #   #    #  #    #          #  #   #  #    #
!   #    #  #    #  #####  #######   ##    #   #### _module
!
!   Radiation-Hydrodynamics-Code: tensor-viscosity module
!
!***************************************************************************************************
!   Fortran90
!   2018-11-18
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhd_vis_module:        Artificial-tensor-viscosity routines
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Density average: 0:min, 1:0.5+0.5 --- */
# ifndef rhd_vis_density_p01
#   define rhd_vis_density_p01 1
# endif
!#
!# /* --- MPI activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */

!------**************-------------------------------------------------------------------------------
module rhd_vis_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_module ('rhd_viscosity_module')
!
! PURPOSE:
!   Provide artificial tensor viscosity routines for rhd.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   use rhd_vis_module
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_vis_SwitchInfo:    Print information about used compiler switches.
!   rhd_vis_StepOutput:    Put some additional output quantities into dat structure
!   rhd_vis_2dStep:        Update model by applying 2D tensor viscosity (incl. turbulent mixing)
!   rhd_vis_3dStep:        Update model by applying 3D tensor viscosity (incl. turbulent mixing)
!   rhd_vis_p2pStepOutput: Put some additional output quantities into dat structure
!   rhd_vis_p2p2dStep:     Update model by applying 2D point-to-point (tensor) viscosity
!   rhd_vis_p2p3dStep:     Update model by applying 3D point-to-point (tensor) viscosity
!   rhd_vis_Step:          Complete viscosity step
!
! MODIFICATION HISTORY:
!   1999-07-28 (B.F. Copenhagen) Put into own module
!   2001-01-22 (B.F. Uppsala) Version with preprocessor directives
!   2001-01-28 (B.F.) Viscosity with new centering, slight change in names
!   2003-05-11 (B.F. Montpellier) point-to-point viscosity
!   2006-11-22 (B.F. Lyon) Routine to do entire viscosity step, taken from rhd.F90
!   2008-04-27 (B.F.) Expansion viscosity (C_visExpansion) added
!   2008-07-03 (B.F.) Linear viscosity (C_visLinear) added
!   2010-04-17 (B.F.) Cosmetics: unused parameters removed, comments added, routines renamed
!   2010-12-10 (W.S. Exeter) routine rhd_box_Extract moved to rhd_box_module
!   2012-11-19 (B.F.) First steps towards MPI version
!   2014-02-26 (B.F. Uppsala) Changes in OpenMP scheduling
!   2014-08-17 (B.F.) Use global modelVis box
!   2016-04-15 (B.F.) Reorganize global world-geometry variables for MPI
!   2017-08-18 (B.F.) Rename output routines; account for new datx type
!---------------------------------------------------------------------------------------------------
implicit none
!
private
!
public rhd_vis_SwitchInfo, rhd_vis_Step
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_vis_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_SwitchInfo ('rhd_viscosity_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-08-12 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_vis3d_module .......................'
!
write(ncp,'(A,I6)') '  rhd' // '_vis_density_p01:        ', rhd_vis_density_p01
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
!
!#
# ifdef rhd_vis_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_vis_t01:             ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_vis_t01:             ', undefstr
!#
# endif
!#
!
end subroutine rhd_vis_SwitchInfo


!----------******************-----------------------------------------------------------------------
subroutine rhd_vis_StepOutput(model, &
                              AreaFraction1, AreaFraction2, AreaFraction3, &
                              Fev1, Fev2, Fev3, dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_StepOutput ('rhd_viscosity_Step_Output')
!
! PURPOSE:
!   Put some additional output quantities into dat structure.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_StepOutput(model, &
!                           AreaFraction1, AreaFraction2, AreaFraction3, &
!                           Fev1, Fev2, Fev3, dat)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   AreaFraction1: ('Area_Fraction_1') real, optional, weighting factor for chunk in x23 plane
!   AreaFraction2: ('Area_Fraction_2') real, optional, weighting factor for chunk in x13 plane
!   AreaFraction3: ('Area_Fraction_3') real, optional, weighting factor for chunk in x12 plane
!   Fev1:         ('Flux_energy_viscosity_1') real 3D array with viscous energy flux in x1 direction
!   Fev2:         ('Flux_energy_viscosity_2') real 3D array with viscous energy flux in x2 direction
!   Fev3:         ('Flux_energy_viscosity_3') real 3D array with viscous energy flux in x3 direction
!
! INPUT/OUTPUT:
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   weight:       ('weight') real 2D array with column weights
!
! ROUTINES:
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! TODO:
!   Radial flux averages over spheres
!
! MODIFICATION HISTORY:
!   2001-01-03 (B.F.) Compute additional output data
!   2001-01-28 (B.F.) Put into own routine
!   2001-03-12 (B.F.) Bugfix
!   2001-03-13 (B.F.) Simplifications
!   2006-11-15 (B.F. Lyon) Some pointer declarations outcommented
!   2010-04-17 (B.F.) mnX_global from rhd_prop_module
!   2012-07-27 (B.F.) Move scale%length to prop%length
!   2017-08-18 (B.F.) Adapt averaging calls to new datx type;
!                     don't count boundary fluxes twice in new averaging calls
!   2017-08-22 (B.F.) Use wg%hyd instead of prop data;
!                     don't count boundary fluxes twice in old averaging calls
!   2017-08-23 (B.F.) Compute AreaFractions from local and global quantities
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_dat_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real,                      intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2, &
                model%m3  :model%n3), &
                           intent(in)   :: Fev1
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2+1, &
                model%m3  :model%n3), &
                           intent(in)   :: Fev2
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3+1), &
                           intent(in)   :: Fev3
type(dat_type),            intent(inout):: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, di1, di2, di3
real                                    :: AreaFraction12, AreaFraction13, AreaFraction23
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: x12weight
real, dimension(model%m1:model%n1, &
                model%m3:model%n3)      :: x13weight
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: x23weight
real,    dimension(  :,:), pointer      :: weight
!real,    dimension(:,:,:), pointer      :: x123weight
!integer, dimension(:,:,:), pointer      :: nb6index, bindex, cindex
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
!!$write(6,'(A,6I4)') 'VIS: m1,n1, m2,n2, m3,n3: ', m1,n1, m2,n2, m3,n3
!
if ((wg%hyd%n(1) > wg%hyd%m(1)) .and. (dat%x23%na > 0) .and. (dat%x23%i_fev > 0)) then
  ! --- Compute weight ---
  AreaFraction23=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m2,n2, m3,n3, model%xb2(m2:n2+1), model%xb3(m3:n3+1), AreaFraction23, &
                        x23weight)
  ! --- Average ---
  if (n1 < wg%hyd%n(1)) then
    di1=0
  else
    di1=1
  endif
  call rhd_mean_x23mean(Fev1(m1:n1+di1,:,:), dat%x23%d(m1:n1+di1,dat%x23%i_fev), x23weight)
endif
!
if ((wg%hyd%n(2) > wg%hyd%m(2)) .and. (dat%x13%na > 0) .and. (dat%x13%i_fev > 0)) then
  ! --- Compute weight ---
  AreaFraction13=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m1,n1, m3,n3, model%xb1(m1:n1+1), model%xb3(m3:n3+1), AreaFraction13, &
                        x13weight)
  !
  ! --- Average ---
  if (n2 < wg%hyd%n(2)) then
    di2=0
  else
    di2=1
  endif
  call rhd_mean_x13mean(Fev2(:,m2:n2+di2,:), dat%x13%d(m2:n2+di2,dat%x13%i_fev), x13weight)
endif
!
if ((wg%hyd%n(3) > wg%hyd%m(3)) .and. (dat%x12%na > 0) .and. (dat%x12%i_fev > 0)) then
  ! --- Compute weight ---
  AreaFraction12=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
  call rhd_mean_aweight(m1,n1, m2,n2, model%xb1(m1:n1+1), model%xb2(m2:n2+1), AreaFraction12, &
                        x12weight)
  !
  ! --- Average ---
  if (n3 < wg%hyd%n(3)) then
    di3=0
  else
    di3=1
  endif
  call rhd_mean_x12mean(Fev3(:,:,m3:n3+di3), dat%x12%d(m3:n3+di3,dat%x12%i_fev), x12weight)
endif
!
end subroutine rhd_vis_StepOutput


!----------**************---------------------------------------------------------------------------
subroutine rhd_vis_2dStep(model, action, dtime, dtime_limit, Output_flag, &
                          AreaFraction1, AreaFraction2, AreaFraction3, &
                          dat, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_2dStep ('rhd_viscosity_2_dimension_Step')
!
! PURPOSE:
!   Update model by applying 2D tensor viscosity.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_2dStep(model, action, dtime, dtime_limit, &
!                       Output_flag=Output_flag, AreaFraction1=AreaFraction1, &
!                       AreaFraction2=AreaFraction2, AreaFraction3=AreaFraction3, dat=dat, &
!                       outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                   default=.false.
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, proposed maximum time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                    =0: normal run
!                  1-99: "small" error (e.g., time step limit violation)
!                 >=100: "severe" error
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   eosinter_Ts:           Solve EOS by bicubic interpolation: rho, e -> T, s
!   rhd_vis_StepOutput:    Put some additional output quantities into dat structure
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Assumes that m2==n2.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-06-01 (B.F. Copenhagen) First version
!   1999-06-20 (B.F. Kiel) Turbulent mixing added (acting on internal energy gradients)
!   1999-06-28 (B.F. Copenhagen) Turbulent mixing acts on entropy gradients
!   1999-07-06 (B.F.) Corrected dimensions for T, dTdei, s
!   2000-02-09 (B.F.) timing
!   2000-12-30 (B.F. Kiel) Call 3D version of eosinter
!   2001-01-03 (B.F.) Compute additional output data
!   2001-01-25 (B.F. Uppsala) Rearrangement of computation of nu and sigma, sigmaXX introduced
!   2001-01-27 (B.F.) Completely new centering of variables
!   2001-01-29 (B.F.) Bug in computation of dxbX_v and dxX_v and in some indices removed
!   2001-02-05 (B.F.) Bug in computation of flux_rhov13 fixed
!   2001-02-11 (B.F.) 2D version
!   2001-03-01 (B.F.) Bug fix in declaration of T and s arrays
!   2001-06-06 (B.F.) New formula for rhob
!   2002-05-27 (B.F.) dtime_limit -> C_visCourant*dtime_limit; check for minimum temperature
!   2002-12-16 (B.F.) Check minimum internal energy
!   2002-12-21 (B.F.) Increase viscosity in case of small time step (fac_vis_smalldtime)
!   2003-01-22 (B.F.) Viscosity increase outcommented
!   2004-03-08 (B.F. Montpellier) T_min_limit=200 -> 100
!   2008-07-03 (B.F. Lyon) Linear viscosity
!   2010-04-17 (B.F.) Factor fac_vis_smalldtime removed
!   2011-11-23 (B.F.) csound(i1,i2,i3) -> 0.5*(csound(i1,i2,i3)+csound(i1+1,i2,i3))
!   2012-10-15 (B.F.)                  -> 0.5*(csound(i1-1,i2,i3)+csound(i1,i2,i3))
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use const_module, only: R_gas
use rhd_gl_module
use rhd_dat_module
use rhd_mean_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_vis_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: Output_flag
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag0
integer                                 :: m1,n1, m2,n2, m3,n3
real                                    :: one_over_Prturb, AreaFraction10, AreaFraction30
real, dimension(model%m1  :model%n1  )  :: dxb1_v
real, dimension(model%m2  :model%n2  )  :: dxb2_v
real, dimension(model%m3  :model%n3  )  :: dxb3_v
real, dimension(model%m1  :model%n1+1)  :: dx1_v
real, dimension(model%m3  :model%n3+1)  :: dx3_v
integer                                 :: i1, i2, i3
real                                    :: inv_dtime_limit, T_min, einew_min, eiTable_min
real                                    :: dx1, dx2, dx3, dxavg, &
                                           dv1dx1, dv3dx1, &
                                           dv1dx3, dv3dx3, &
                                           divv, nu_total, rhob, &
                                           v1, v3, &
                                           dtimeoverrhodx1, dtimeoverrhodx3, &
                                           visLinear, visArtificial, visExpansion, visSmagorinsky
real, dimension(model%m1-1:model%n1+1, &
                model%m2  :model%n2  , &
                model%m3-1:model%n3+1)  :: T, s, csound
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2, &
                model%m3  :model%n3)    :: flux_rhov11, flux_rhov31, flux_rhoei1
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2+1, &
                model%m3  :model%n3)    ::                           flux_rhoei2
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3+1)  :: flux_rhov13, flux_rhov33, flux_rhoei3
integer                                 :: ierr0
character(len=80)                       :: outstr0
real, parameter                         :: T_min_limit=100.0
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VIS2D:')
!#
# endif
!#
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Set default values for keywords ---
if (.not. present(Output_flag)) then
  Output_flag0=.false.
else
  Output_flag0=Output_flag
endif
if (.not. present(AreaFraction1)) then
  AreaFraction10=1.0
else
  AreaFraction10=AreaFraction1
endif
if (.not. present(AreaFraction3)) then
  AreaFraction30=1.0
else
  AreaFraction30=AreaFraction3
endif
!
! --- No limit on time step ---
dtime_limit=0.1*huge(dtime_limit)
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! \label{f90:c_visprturb__use1}\label{f90:c_vislinear__use1}
if ((par%C_visPrturb > 0.0) .or. (par%C_visLinear > 0.0)) then
  ! --- Additional turbulent mixing and/or linear viscosity ---
  !
  if (par%C_visPrturb > 0.0) then
    one_over_Prturb=1.0/par%C_visPrturb
  else
    one_over_Prturb=0.0
  endif
  !
  ! --- Solve EOS ---
!#
# ifdef rhd_vis_t01
!#
  call timing_start('VIS2D: eosinter')
!#
# endif
!#
  call eosinter_Ts(model%rho(model%m1-1:model%n1+1,m2:n2,model%m3-1:model%n3+1), &
                   model%ei( model%m1-1:model%n1+1,m2:n2,model%m3-1:model%n3+1), &
                   T=T, s=s)
  T_min=minval(T)
  if (T_min < T_min_limit) then
    ! \label{f90:c_viscourant__use1}
    ! --- Minimum temperature too small ---
    dtime_limit=min(par%C_visCourant,0.8) * dtime
    ierr0=2
    outstr0='Minimum temperature too small'
    goto 999
  endif
  !
  if (par%C_visLinear > 0.0) then
    ! --- Use an approximation for sound speed (error is sqrt(Gamma_1/mu)) ---
    csound=sqrt(R_gas*T)
    !
    if (par%C_visPrturb <= 0.0) then
      s=0.0
    endif
  else
    csound=0.0
  endif
!#
# ifdef rhd_vis_t01
!#
  call timing_stop( 'VIS2D: eosinter')
!#
# endif
!#
else
  ! --- No additional turbulent mixing: set factor and arrays to zero ---
  !
  one_over_Prturb=0.0
  T     =0.0
  s     =0.0
  csound=0.0
endif
!
! \label{f90:c_vislinear__use2}\label{f90:c_visartificial__use1}
! --- Normalize viscosity coefficients ---
visLinear     =max(par%C_visLinear     ,0.0)*0.5
visArtificial =max(par%C_visArtificial ,0.0)**2
visExpansion  =max(par%C_visExpansion  ,0.0)**2
visSmagorinsky=max(par%C_visSmagorinsky,0.0)**2
!
! --- Grid size ---
dxb1_v=       model%xb1(m1+1:n1+1) - model%xb1(m1  :n1  )
dxb2_v=       model%xb2(m2+1:n2+1) - model%xb2(m2  :n2  )
dxb3_v=       model%xb3(m3+1:n3+1) - model%xb3(m3  :n3  )
dx1_v =0.5 * (model%xb1(m1+1:n1+2) - model%xb1(m1-1:n1  ))
dx3_v =0.5 * (model%xb3(m3+1:n3+2) - model%xb3(m3-1:n3  ))
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VIS2D: dvdx,nu,fluxes')
!#
# endif
!#
!
inv_dtime_limit=0.0
!
! --- Fluxes in direction 1 ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Relevant spatial steps ---
      dx1=dx1_v( i1)
      dx2=dxb2_v(i2)
      dx3=dxb3_v(i3)
      dxavg=(1.0/3.0)*(dx1+dx2+dx3)
      !
      ! --- Derivatives of velocity ---
      dv1dx1=     (model%v1(i1  ,i2  ,i3  ) - model%v1(i1-1,i2  ,i3  )) / dx1
      dv3dx1=     (model%v3(i1  ,i2  ,i3  ) - model%v3(i1-1,i2  ,i3  )) / dx1
      dv1dx3=0.25*(model%v1(i1-1,i2  ,i3+1) - model%v1(i1-1,i2  ,i3-1) + &
                   model%v1(i1  ,i2  ,i3+1) - model%v1(i1  ,i2  ,i3-1)) / dx3
      dv3dx3=0.25*(model%v3(i1-1,i2  ,i3+1) - model%v3(i1-1,i2  ,i3-1) + &
                   model%v3(i1  ,i2  ,i3+1) - model%v3(i1  ,i2  ,i3-1)) / dx3
      !
      ! --- Divergence ---
      divv=dv1dx1 + dv3dx3
      !
      ! --- Kinematic viscosity ---
      nu_total=dxavg * visLinear * (csound(i1-1,i2,i3)+csound(i1,i2,i3)) + &
               min(dx1, dx2, dx3) * max(dx1, dx2, dx3) * &
               ( visArtificial  * &
                   max(-divv, 0.0) + &
                 visExpansion   * &
                   max( divv, 0.0) + &
                 visSmagorinsky * &
                   sqrt( 2.0 * (dv1dx1**2 + dv3dx3**2) + &
                         (dv1dx3 + dv3dx1)**2 &
                       ) &
               )
      !
      ! --- Density at boundary ---
!#
#     if (rhd_vis_density_p01 == 0)
!#                                  /* 'min' */
      rhob= min(model%rho(i1-1,i2  ,i3  ),  model%rho(i1  ,i2  ,i3  ))
!#
#     else
!#         /* rhd_vis_density_p01 == 1 '0.5+0.5' */
      rhob=0.5*(model%rho(i1-1,i2  ,i3  ) + model%rho(i1  ,i2  ,i3  ))
!#
#     endif
!#          /* rhd_vis_density_p01 */
      !
      ! \label{f90:c_vistensoroff__use1}
      ! --- Momentum Fluxes ---
      flux_rhov11(i1,i2,i3)=-rhob * nu_total * (par%C_visTensordiag * dv1dx1 + &
                                                par%C_visTensordiv  * divv      )
      flux_rhov31(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv1dx3 + dv3dx1)
      !
      ! --- Energy flux ---
      flux_rhoei1(i1,i2,i3)=0.5*( &
                   (model%v1(i1-1,i2  ,i3  ) + model%v1(i1  ,i2  ,i3  )) * flux_rhov11(i1,i2,i3) +&
                   (model%v3(i1-1,i2  ,i3  ) + model%v3(i1  ,i2  ,i3  )) * flux_rhov31(i1,i2,i3) -&
                              one_over_Prturb * rhob * nu_total * &
                                     (T(i1  ,i2  ,i3  ) +        T(i1-1,i2  ,i3  )) * &
                                     (s(i1  ,i2  ,i3  ) -        s(i1-1,i2  ,i3  ))/dx1 &
                                )
      !
      ! --- Maximum time step ---
      inv_dtime_limit=max(inv_dtime_limit, nu_total/dx1**2)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Fluxes in direction 3 ---
do i3=m3,n3+1
  do i2=m2,n2
    do i1=m1,n1
      ! --- Relevant spatial steps ---
      dx1=dxb1_v(i1)
      dx2=dxb2_v(i2)
      dx3=dx3_v( i3)
      dxavg=(1.0/3.0)*(dx1+dx2+dx3)
      !
      ! --- Derivatives of velocity ---
      dv1dx1=0.25*(model%v1(i1+1,i2  ,i3-1) - model%v1(i1-1,i2  ,i3-1) + &
                   model%v1(i1+1,i2  ,i3  ) - model%v1(i1-1,i2  ,i3  )) / dx1
      dv3dx1=0.25*(model%v3(i1+1,i2  ,i3-1) - model%v3(i1-1,i2  ,i3-1) + &
                   model%v3(i1+1,i2  ,i3  ) - model%v3(i1-1,i2  ,i3  )) / dx1
      dv1dx3=     (model%v1(i1  ,i2  ,i3  ) - model%v1(i1  ,i2  ,i3-1)) / dx3
      dv3dx3=     (model%v3(i1  ,i2  ,i3  ) - model%v3(i1  ,i2  ,i3-1)) / dx3
      !
      ! --- Divergence ---
      divv=dv1dx1 + dv3dx3
      !
      ! --- Kinematic viscosity ---
      nu_total=dxavg * visLinear * (csound(i1,i2,i3-1)+csound(i1,i2,i3)) + &
               min(dx1, dx2, dx3) * max(dx1, dx2, dx3) * &
               ( visArtificial  * &
                   max(-divv, 0.0) + &
                 visExpansion   * &
                   max( divv, 0.0) + &
                 visSmagorinsky * &
                   sqrt( 2.0 * (dv1dx1**2 + dv3dx3**2) + &
                         (dv1dx3 + dv3dx1)**2 &
                       ) &
               )
      !
      ! --- Density at boundary ---
!#
#     if (rhd_vis_density_p01 == 0)
!#                                  /* 'min' */
      rhob= min(model%rho(i1  ,i2  ,i3-1),  model%rho(i1  ,i2  ,i3  ))
!#
#     else
!#         /* rhd_vis_density_p01 == 1 '0.5+0.5' */
      rhob=0.5*(model%rho(i1  ,i2  ,i3-1) + model%rho(i1  ,i2  ,i3  ))
!#
#     endif
!#          /* rhd_vis_density_p01 */
      !
      ! \label{f90:c_vistensoroff__use2}
      ! --- Momentum Fluxes ---
      flux_rhov13(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv1dx3 + dv3dx1)
      flux_rhov33(i1,i2,i3)=-rhob * nu_total * (par%C_visTensordiag * dv3dx3 + &
                                                par%C_visTensordiv  * divv      )
      !
      ! --- Energy flux ---
      flux_rhoei3(i1,i2,i3)=0.5*( &
                   (model%v1(i1  ,i2  ,i3-1) + model%v1(i1  ,i2  ,i3  )) * flux_rhov13(i1,i2,i3) +&
                   (model%v3(i1  ,i2  ,i3-1) + model%v3(i1  ,i2  ,i3  )) * flux_rhov33(i1,i2,i3) -&
                              one_over_Prturb * rhob * nu_total * &
                                     (T(i1  ,i2  ,i3  ) +        T(i1  ,i2  ,i3-1)) * &
                                     (s(i1  ,i2  ,i3  ) -        s(i1  ,i2  ,i3-1))/dx3 &
                                )
      !
      ! --- Maximum time step ---
      inv_dtime_limit=max(inv_dtime_limit, nu_total/dx3**2)
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VIS2D: dvdx,nu,fluxes')
call timing_start( 'VIS2D: update')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    !DIR$ IVDEP
    do i1=m1,n1
      ! --- Save old velocities (necessary for update of internal energy) ---
      v1=model%v1(i1,i2,i3)
      v3=model%v3(i1,i2,i3)
      !
      ! --- Abbreviation: dtimeoverrhodx=dt/(rho*dx) ---
      dtimeoverrhodx1=dtime/(model%rho(i1,i2,i3)*dxb1_v(i1))
      dtimeoverrhodx3=dtime/(model%rho(i1,i2,i3)*dxb3_v(i3))
      !
      ! --- Update model: apply fluxes (no mass flux) ---
      model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhov11(i1+1, i2  , i3  ) - &
                                                flux_rhov11(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhov13(i1  , i2  , i3+1) - &
                                                flux_rhov13(i1  , i2  , i3  )) &
                           )
      model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhov31(i1+1, i2  , i3  ) - &
                                                flux_rhov31(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhov33(i1  , i2  , i3+1) - &
                                                flux_rhov33(i1  , i2  , i3  )) &
                           )

      model%ei(i1,i2,i3)=model%ei(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhoei1(i1+1, i2  , i3  ) - &
                                                flux_rhoei1(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhoei3(i1  , i2  , i3+1) - &
                                                flux_rhoei3(i1  , i2  , i3  )) &
                           ) + &
                             0.5 * ( (v1-model%v1(i1,i2,i3)) * &
                                     (v1+model%v1(i1,i2,i3)) + &
                                     (v3-model%v3(i1,i2,i3)) * &
                                     (v3+model%v3(i1,i2,i3)) &
                                   )
    !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Check if internal energy is below minimum table value anywhere ---
!!$einew_min=huge(einew_min)
!!$do i3=m3,n3
!!$  do i2=m2,n2
!!$    do i1=m1,n1
!!$      einew_min=min(einew_min, model%ei(i1,i2,i3))
!!$    end do ! i1
!!$  end do ! i2
!!$end do ! i3
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
call tabinter_getinfo(x2min=eiTable_min)
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*dtime
  outstr0='Internal energy below minimum table value'
  goto 999
endif
!
!#
# ifdef rhd_vis_t01
!#
call timing_stop( 'VIS2D: update')
!#
# endif
!#
! \label{f90:c_viscourantmax__use1}\label{f90:c_viscourant__use2}
! --- Check for time step limit violation ---
if (inv_dtime_limit > 100.0*spacing(0.0)) then
  dtime_limit=par%C_visCourant/inv_dtime_limit
  if (dtime > (par%C_visCourantMax/par%C_visCourant*dtime_limit)) then
    ierr0=1
    outstr0='Time step exceeds viscous time scale'
    goto 999
  endif
endif
!
! --- Output of additional data ---
if ((Output_flag0) .and. present(dat)) then
!#
# ifdef rhd_vis_t01
!#
  call timing_start( 'VIS2D: (output)')
!#
# endif
!#
  call rhd_vis_StepOutput(model, &
                          AreaFraction10, 1.0, AreaFraction30, &
                          flux_rhoei1, flux_rhoei2, flux_rhoei3, dat)
!#
# ifdef rhd_vis_t01
!#
  call timing_stop( 'VIS2D: (output)')
!#
# endif
!#
endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VIS2D:')
!#
# endif
!#
!
end subroutine rhd_vis_2dStep


!----------**************---------------------------------------------------------------------------
subroutine rhd_vis_3dStep(model, action, dtime, dtime_limit, Output_flag, &
                          AreaFraction1, AreaFraction2, AreaFraction3, &
                          dat, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_3dStep ('rhd_viscosity_3_dimension_Step')
!
! PURPOSE:
!   Update model by applying 3D tensor viscosity.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_3dStep(model, action, dtime, dtime_limit, &
!                       Output_flag=Output_flag, AreaFraction1=AreaFraction1, &
!                       AreaFraction2=AreaFraction2, AreaFraction3=AreaFraction3, dat=dat, &
!                       outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                   default=.false.
!   AreaFraction: ('Area_Fraction') real, optional, weighting factor for the current model slice
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, proposed maximum time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                    =0: normal run
!                  1-99: "small" error (e.g., time step limit violation)
!                 >=100: "severe" error
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!
! ROUTINES:
!   eosinter_Ts:           Solve EOS by bicubic interpolation: rho, e -> T, s
!   rhd_vis_StepOutput:    Put some additional output quantities into dat structure
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-06-01 (B.F. Copenhagen) First version
!   1999-06-20 (B.F. Kiel) Turbulent mixing added (acting on internal energy gradients)
!   1999-06-28 (B.F. Copenhagen) Turbulent mixing acts on entropy gradients
!   1999-07-06 (B.F.) Corrected dimensions for T, dTdei, s
!   2000-02-09 (B.F.) timing
!   2000-12-30 (B.F. Kiel) Call 3D version of eosinter
!   2001-01-03 (B.F.) Compute additional output data
!   2001-01-25 (B.F. Uppsala) Rearrangement of computation of nu and sigma, sigmaXX introduced
!   2001-01-27 (B.F.) Completely new centering of variables
!   2001-01-29 (B.F.) Bug in computation of dxbX_v and dxX_v and in some indices removed
!   2001-02-05 (B.F.) Bug in computation of flux_rhov13 removed
!   2001-06-06 (B.F.) New formula for rhob
!   2001-09-04 (B.F.) mnX_global
!   2002-05-27 (B.F.) dtime_limit -> C_visCourant*dtime_limit; check for minimum temperature
!   2002-12-16 (B.F.) Check minimum internal energy
!   2002-12-21 (B.F.) Increase viscosity in case of small time step (fac_vis_smalldtime)
!   2003-01-22 (B.F.) Viscosity increase outcommented
!   2004-03-08 (B.F. Montpellier) T_min_limit=200 -> 100
!   2008-07-04 (B.F. Lyon) Linear viscosity
!   2010-04-17 (B.F.) Parameters mnX_global etc. removed, factor fac_vis_smalldtime removed
!   2011-11-23 (B.F.) csound(i1,i2,i3) -> 0.5*(csound(i1,i2,i3)+csound(i1+1,i2,i3))
!---------------------------------------------------------------------------------------------------
use tabinter_module
use gasinter_module
use const_module, only: R_gas
use rhd_gl_module
use rhd_dat_module
use rhd_mean_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_vis_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: Output_flag
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag0
integer                                 :: m1,n1, m2,n2, m3,n3
real                                    :: one_over_Prturb, &
                                           AreaFraction10, AreaFraction20, AreaFraction30
real, dimension(model%m1  :model%n1  )  :: dxb1_v
real, dimension(model%m2  :model%n2  )  :: dxb2_v
real, dimension(model%m3  :model%n3  )  :: dxb3_v
real, dimension(model%m1  :model%n1+1)  :: dx1_v
real, dimension(model%m2  :model%n2+1)  :: dx2_v
real, dimension(model%m3  :model%n3+1)  :: dx3_v
integer                                 :: i1, i2, i3
real                                    :: inv_dtime_limit, T_min, einew_min, eiTable_min
real                                    :: dx1, dx2, dx3, dxavg, &
                                           dv1dx1, dv2dx1, dv3dx1, &
                                           dv1dx2, dv2dx2, dv3dx2, &
                                           dv1dx3, dv2dx3, dv3dx3, &
                                           divv, nu_total, rhob, &
                                           v1, v2, v3, &
                                           dtimeoverrhodx1, dtimeoverrhodx2, dtimeoverrhodx3, &
                                           visLinear, visArtificial, visExpansion, visSmagorinsky
real, dimension(model%m1-1:model%n1+1, &
                model%m2-1:model%n2+1, &
                model%m3-1:model%n3+1)  :: T, s, csound
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2, &
                model%m3  :model%n3)    :: flux_rhov11, flux_rhov21, flux_rhov31, flux_rhoei1
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2+1, &
                model%m3  :model%n3)    :: flux_rhov12, flux_rhov22, flux_rhov32, flux_rhoei2
real, dimension(model%m1  :model%n1, &
                model%m2  :model%n2, &
                model%m3  :model%n3+1)  :: flux_rhov13, flux_rhov23, flux_rhov33, flux_rhoei3
integer                                 :: ierr0
character(len=80)                       :: outstr0
real, parameter                         :: T_min_limit=100.0
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VIS3D:')
!#
# endif
!#
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Set default values for keywords ---
if (.not. present(Output_flag)) then
  Output_flag0=.false.
else
  Output_flag0=Output_flag
endif
!
if (.not. present(AreaFraction1)) then
  AreaFraction10=1.0
else
  AreaFraction10=AreaFraction1
endif
if (.not. present(AreaFraction2)) then
  AreaFraction20=1.0
else
  AreaFraction20=AreaFraction2
endif
if (.not. present(AreaFraction3)) then
  AreaFraction30=1.0
else
  AreaFraction30=AreaFraction3
endif
!
! --- No limit on time step ---
dtime_limit=0.1*huge(dtime_limit)
!
! \label{f90:c_visprturb__use2}\label{f90:c_vislinear__use3}
if ((par%C_visPrturb > 0.0)  .or. (par%C_visLinear > 0.0)) then
  ! --- Additional turbulent mixing and/or linear viscosity ---
  !
  if (par%C_visPrturb > 0.0) then
    one_over_Prturb=1.0/par%C_visPrturb
  else
    one_over_Prturb=0.0
  endif
  !
  ! --- Solve EOS ---
!#
# ifdef rhd_vis_t01
!#
  call timing_start('VIS3D: eosinter')
!#
# endif
!#
  call eosinter_Ts(model%rho(model%m1-1:model%n1+1,model%m2-1:model%n2+1,model%m3-1:model%n3+1), &
                   model%ei( model%m1-1:model%n1+1,model%m2-1:model%n2+1,model%m3-1:model%n3+1), &
                   T=T, s=s)
  ! \label{f90:c_viscourant__use3}
  T_min=minval(T)
  if (T_min < T_min_limit) then
    ! --- Minimum temperature too small ---
    dtime_limit=min(par%C_visCourant,0.8) * dtime
    ierr0=2
    outstr0='Minimum temperature too small'
    goto 999
  endif
  !
  if (par%C_visLinear > 0.0) then
    ! --- Use an approximation for sound speed (error is sqrt(Gamma_1/mu)) ---
    csound=sqrt(R_gas*T)
    !
    if (par%C_visPrturb <= 0.0) then
      s=0.0
    endif
  else
    csound=0.0
  endif
!#
# ifdef rhd_vis_t01
!#
  call timing_stop( 'VIS3D: eosinter')
!#
# endif
!#
else
  ! --- No additional turbulent mixing: set factor and arrays to zero ---
  !
  one_over_Prturb=0.0
  T     =0.0
  s     =0.0
  csound=0.0
endif
!
! \label{f90:c_vislinear__use4}\label{f90:c_visartificial__use2}
! --- Normalize viscosity coefficients ---
visLinear     =max(par%C_visLinear     ,0.0)*0.5
visArtificial =max(par%C_visArtificial ,0.0)**2
visExpansion  =max(par%C_visExpansion  ,0.0)**2
visSmagorinsky=max(par%C_visSmagorinsky,0.0)**2
!
! --- Grid size ---
dxb1_v=       model%xb1(m1+1:n1+1) - model%xb1(m1  :n1  )
dxb2_v=       model%xb2(m2+1:n2+1) - model%xb2(m2  :n2  )
dxb3_v=       model%xb3(m3+1:n3+1) - model%xb3(m3  :n3  )
dx1_v =0.5 * (model%xb1(m1+1:n1+2) - model%xb1(m1-1:n1  ))
dx2_v =0.5 * (model%xb2(m2+1:n2+2) - model%xb2(m2-1:n2  ))
dx3_v =0.5 * (model%xb3(m3+1:n3+2) - model%xb3(m3-1:n3  ))
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VIS3D: dvdx,nu,fluxes')
!#
# endif
!#
!
inv_dtime_limit=0.0
!
! --- Fluxes in direction 1 ---
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1+1
      ! --- Relevant spatial steps ---
      dx1=dx1_v( i1)
      dx2=dxb2_v(i2)
      dx3=dxb3_v(i3)
      dxavg=(1.0/3.0)*(dx1+dx2+dx3)
      !
      ! --- Derivatives of velocity ---
      dv1dx1=     (model%v1(i1  ,i2  ,i3  ) - model%v1(i1-1,i2  ,i3  )) / dx1
      dv2dx1=     (model%v2(i1  ,i2  ,i3  ) - model%v2(i1-1,i2  ,i3  )) / dx1
      dv3dx1=     (model%v3(i1  ,i2  ,i3  ) - model%v3(i1-1,i2  ,i3  )) / dx1
      dv1dx2=0.25*(model%v1(i1-1,i2+1,i3  ) - model%v1(i1-1,i2-1,i3  ) + &
                   model%v1(i1  ,i2+1,i3  ) - model%v1(i1  ,i2-1,i3  )) / dx2
      dv2dx2=0.25*(model%v2(i1-1,i2+1,i3  ) - model%v2(i1-1,i2-1,i3  ) + &
                   model%v2(i1  ,i2+1,i3  ) - model%v2(i1  ,i2-1,i3  )) / dx2
      dv3dx2=0.25*(model%v3(i1-1,i2+1,i3  ) - model%v3(i1-1,i2-1,i3  ) + &
                   model%v3(i1  ,i2+1,i3  ) - model%v3(i1  ,i2-1,i3  )) / dx2
      dv1dx3=0.25*(model%v1(i1-1,i2  ,i3+1) - model%v1(i1-1,i2  ,i3-1) + &
                   model%v1(i1  ,i2  ,i3+1) - model%v1(i1  ,i2  ,i3-1)) / dx3
      dv2dx3=0.25*(model%v2(i1-1,i2  ,i3+1) - model%v2(i1-1,i2  ,i3-1) + &
                   model%v2(i1  ,i2  ,i3+1) - model%v2(i1  ,i2  ,i3-1)) / dx3
      dv3dx3=0.25*(model%v3(i1-1,i2  ,i3+1) - model%v3(i1-1,i2  ,i3-1) + &
                   model%v3(i1  ,i2  ,i3+1) - model%v3(i1  ,i2  ,i3-1)) / dx3
      !
      ! --- Divergence ---
      divv=dv1dx1 + dv2dx2 + dv3dx3
      !
      ! --- Kinematic viscosity ---
      nu_total=dxavg * visLinear * (csound(i1-1,i2,i3)+csound(i1,i2,i3)) + &
               min(dx1, dx2, dx3) * max(dx1, dx2, dx3) * &
               ( visArtificial  * &
                   max(-divv, 0.0) + &
                 visExpansion   * &
                   max( divv, 0.0) + &
                 visSmagorinsky * &
                   sqrt( 2.0 * (dv1dx1**2 + dv2dx2**2 + dv3dx3**2) + &
                         (dv1dx2 + dv2dx1)**2 + (dv1dx3 + dv3dx1)**2 + (dv2dx3 + dv3dx2)**2 &
                       ) &
               )
      !
      ! --- Density at boundary ---
!#
#     if (rhd_vis_density_p01 == 0)
!#                                  /* 'min' */
      rhob= min(model%rho(i1-1,i2  ,i3  ),  model%rho(i1  ,i2  ,i3  ))
!#
#     else
!#         /* rhd_vis_density_p01 == 1 '0.5+0.5' */
      rhob=0.5*(model%rho(i1-1,i2  ,i3  ) + model%rho(i1  ,i2  ,i3  ))
!#
#     endif
!#          /* rhd_vis_density_p01 */
      !
      ! \label{f90:c_vistensoroff__use3}
      ! --- Momentum Fluxes ---
      flux_rhov11(i1,i2,i3)=-rhob * nu_total * (par%C_visTensordiag * dv1dx1 + &
                                                par%C_visTensordiv  * divv      )
      flux_rhov21(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv1dx2 + dv2dx1)
      flux_rhov31(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv1dx3 + dv3dx1)
      !
      ! --- Energy flux ---
      flux_rhoei1(i1,i2,i3)=0.5*( &
                   (model%v1(i1-1,i2  ,i3  ) + model%v1(i1  ,i2  ,i3  )) * flux_rhov11(i1,i2,i3) + &
                   (model%v2(i1-1,i2  ,i3  ) + model%v2(i1  ,i2  ,i3  )) * flux_rhov21(i1,i2,i3) + &
                   (model%v3(i1-1,i2  ,i3  ) + model%v3(i1  ,i2  ,i3  )) * flux_rhov31(i1,i2,i3) - &
                              one_over_Prturb * rhob * nu_total * &
                                     (T(i1  ,i2  ,i3  ) +        T(i1-1,i2  ,i3  )) * &
                                     (s(i1  ,i2  ,i3  ) -        s(i1-1,i2  ,i3  ))/dx1 &
                                )
      !
      ! --- Maximum time step ---
      inv_dtime_limit=max(inv_dtime_limit, nu_total/dx1**2)
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Fluxes in direction 2 ---
do i3=m3,n3
  do i2=m2,n2+1
    do i1=m1,n1
      ! --- Relevant spatial steps ---
      dx1=dxb1_v(i1)
      dx2=dx2_v( i2)
      dx3=dxb3_v(i3)
      dxavg=(1.0/3.0)*(dx1+dx2+dx3)
      !
      ! --- Derivatives of velocity ---
      dv1dx1=0.25*(model%v1(i1+1,i2-1,i3  ) - model%v1(i1-1,i2-1,i3  ) + &
                   model%v1(i1+1,i2  ,i3  ) - model%v1(i1-1,i2  ,i3  )) / dx1
      dv2dx1=0.25*(model%v2(i1+1,i2-1,i3  ) - model%v2(i1-1,i2-1,i3  ) + &
                   model%v2(i1+1,i2  ,i3  ) - model%v2(i1-1,i2  ,i3  )) / dx1
      dv3dx1=0.25*(model%v3(i1+1,i2-1,i3  ) - model%v3(i1-1,i2-1,i3  ) + &
                   model%v3(i1+1,i2  ,i3  ) - model%v3(i1-1,i2  ,i3  )) / dx1
      dv1dx2=     (model%v1(i1  ,i2  ,i3  ) - model%v1(i1  ,i2-1,i3  )) / dx2
      dv2dx2=     (model%v2(i1  ,i2  ,i3  ) - model%v2(i1  ,i2-1,i3  )) / dx2
      dv3dx2=     (model%v3(i1  ,i2  ,i3  ) - model%v3(i1  ,i2-1,i3  )) / dx2
      dv1dx3=0.25*(model%v1(i1  ,i2-1,i3+1) - model%v1(i1  ,i2-1,i3-1) + &
                   model%v1(i1  ,i2  ,i3+1) - model%v1(i1  ,i2  ,i3-1)) / dx3
      dv2dx3=0.25*(model%v2(i1  ,i2-1,i3+1) - model%v2(i1  ,i2-1,i3-1) + &
                   model%v2(i1  ,i2  ,i3+1) - model%v2(i1  ,i2  ,i3-1)) / dx3
      dv3dx3=0.25*(model%v3(i1  ,i2-1,i3+1) - model%v3(i1  ,i2-1,i3-1) + &
                   model%v3(i1  ,i2  ,i3+1) - model%v3(i1  ,i2  ,i3-1)) / dx3
      !
      ! --- Divergence ---
      divv=dv1dx1 + dv2dx2 + dv3dx3
      !
      ! --- Kinematic viscosity ---
      nu_total=dxavg * visLinear * (csound(i1,i2-1,i3)+csound(i1,i2,i3)) + &
               min(dx1, dx2, dx3) * max(dx1, dx2, dx3) * &
               ( visArtificial  * &
                   max(-divv, 0.0) + &
                 visExpansion   * &
                   max( divv, 0.0) + &
                 visSmagorinsky * &
                   sqrt( 2.0 * (dv1dx1**2 + dv2dx2**2 + dv3dx3**2) + &
                         (dv1dx2 + dv2dx1)**2 + (dv1dx3 + dv3dx1)**2 + (dv2dx3 + dv3dx2)**2 &
                       ) &
               )
      !
      ! --- Density at boundary ---
!#
#     if (rhd_vis_density_p01 == 0)
!#                                  /* 'min' */
      rhob= min(model%rho(i1  ,i2-1,i3  ),  model%rho(i1  ,i2  ,i3  ))
!#
#     else
!#         /* rhd_vis_density_p01 == 1 '0.5+0.5' */
      rhob=0.5*(model%rho(i1  ,i2-1,i3  ) + model%rho(i1  ,i2  ,i3  ))
!#
#     endif
!#          /* rhd_vis_density_p01 */
      !
      ! \label{f90:c_vistensoroff__use4}
      ! --- Momentum Fluxes ---
      flux_rhov12(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv1dx2 + dv2dx1)
      flux_rhov22(i1,i2,i3)=-rhob * nu_total * (par%C_visTensordiag * dv2dx2 + &
                                                par%C_visTensordiv  * divv      )
      flux_rhov32(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv2dx3 + dv3dx2)
      !
      ! --- Energy flux ---
      flux_rhoei2(i1,i2,i3)=0.5*( &
                   (model%v1(i1  ,i2-1,i3  ) + model%v1(i1  ,i2  ,i3  )) * flux_rhov12(i1,i2,i3) + &
                   (model%v2(i1  ,i2-1,i3  ) + model%v2(i1  ,i2  ,i3  )) * flux_rhov22(i1,i2,i3) + &
                   (model%v3(i1  ,i2-1,i3  ) + model%v3(i1  ,i2  ,i3  )) * flux_rhov32(i1,i2,i3) - &
                              one_over_Prturb * rhob * nu_total * &
                                     (T(i1  ,i2  ,i3  ) +        T(i1  ,i2-1,i3  )) * &
                                     (s(i1  ,i2  ,i3  ) -        s(i1  ,i2-1,i3  ))/dx2 &
                                )
      !
      ! --- Maximum time step ---
      inv_dtime_limit=max(inv_dtime_limit, nu_total/dx2**2)
      !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Fluxes in direction 3 ---
do i3=m3,n3+1
  do i2=m2,n2
    do i1=m1,n1
      ! --- Relevant spatial steps ---
      dx1=dxb1_v(i1)
      dx2=dxb2_v(i2)
      dx3=dx3_v( i3)
      dxavg=(1.0/3.0)*(dx1+dx2+dx3)
      !
      ! --- Derivatives of velocity ---
      dv1dx1=0.25*(model%v1(i1+1,i2  ,i3-1) - model%v1(i1-1,i2  ,i3-1) + &
                   model%v1(i1+1,i2  ,i3  ) - model%v1(i1-1,i2  ,i3  )) / dx1
      dv2dx1=0.25*(model%v2(i1+1,i2  ,i3-1) - model%v2(i1-1,i2  ,i3-1) + &
                   model%v2(i1+1,i2  ,i3  ) - model%v2(i1-1,i2  ,i3  )) / dx1
      dv3dx1=0.25*(model%v3(i1+1,i2  ,i3-1) - model%v3(i1-1,i2  ,i3-1) + &
                   model%v3(i1+1,i2  ,i3  ) - model%v3(i1-1,i2  ,i3  )) / dx1
      dv1dx2=0.25*(model%v1(i1  ,i2+1,i3-1) - model%v1(i1  ,i2-1,i3-1) + &
                   model%v1(i1  ,i2+1,i3  ) - model%v1(i1  ,i2-1,i3  )) / dx2
      dv2dx2=0.25*(model%v2(i1  ,i2+1,i3-1) - model%v2(i1  ,i2-1,i3-1) + &
                   model%v2(i1  ,i2+1,i3  ) - model%v2(i1  ,i2-1,i3  )) / dx2
      dv3dx2=0.25*(model%v3(i1  ,i2+1,i3-1) - model%v3(i1  ,i2-1,i3-1) + &
                   model%v3(i1  ,i2+1,i3  ) - model%v3(i1  ,i2-1,i3  )) / dx2
      dv1dx3=     (model%v1(i1  ,i2  ,i3  ) - model%v1(i1  ,i2  ,i3-1)) / dx3
      dv2dx3=     (model%v2(i1  ,i2  ,i3  ) - model%v2(i1  ,i2  ,i3-1)) / dx3
      dv3dx3=     (model%v3(i1  ,i2  ,i3  ) - model%v3(i1  ,i2  ,i3-1)) / dx3
      !
      ! --- Divergence ---
      divv=dv1dx1 + dv2dx2 + dv3dx3
      !
      ! --- Kinematic viscosity ---
      nu_total=dxavg * visLinear * (csound(i1,i2,i3-1)+csound(i1,i2,i3)) + &
               min(dx1, dx2, dx3) * max(dx1, dx2, dx3) * &
               ( visArtificial  * &
                   max(-divv, 0.0) + &
                 visExpansion   * &
                   max( divv, 0.0) + &
                 visSmagorinsky * &
                   sqrt( 2.0 * (dv1dx1**2 + dv2dx2**2 + dv3dx3**2) + &
                         (dv1dx2 + dv2dx1)**2 + (dv1dx3 + dv3dx1)**2 + (dv2dx3 + dv3dx2)**2 &
                       ) &
               )
      !
      ! --- Density at boundary ---
!#
#     if (rhd_vis_density_p01 == 0)
!#                                  /* 'min' */
      rhob= min(model%rho(i1  ,i2  ,i3-1),  model%rho(i1  ,i2  ,i3  ))
!#
#     else
!#         /* rhd_vis_density_p01 == 1 '0.5+0.5' */
      rhob=0.5*(model%rho(i1  ,i2  ,i3-1) + model%rho(i1  ,i2  ,i3  ))
!#
#     endif
!#          /* rhd_vis_density_p01 */
      !
      ! \label{f90:c_vistensoroff__use5}
      ! --- Momentum Fluxes ---
      flux_rhov13(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv1dx3 + dv3dx1)
      flux_rhov23(i1,i2,i3)=-rhob * nu_total *  par%C_visTensoroff  * (dv2dx3 + dv3dx2)
      flux_rhov33(i1,i2,i3)=-rhob * nu_total * (par%C_visTensordiag * dv3dx3 + &
                                                par%C_visTensordiv  * divv      )
      !
      ! --- Energy flux ---
      flux_rhoei3(i1,i2,i3)=0.5*( &
                   (model%v1(i1  ,i2  ,i3-1) + model%v1(i1  ,i2  ,i3  )) * flux_rhov13(i1,i2,i3) + &
                   (model%v2(i1  ,i2  ,i3-1) + model%v2(i1  ,i2  ,i3  )) * flux_rhov23(i1,i2,i3) + &
                   (model%v3(i1  ,i2  ,i3-1) + model%v3(i1  ,i2  ,i3  )) * flux_rhov33(i1,i2,i3) - &
                              one_over_Prturb * rhob * nu_total * &
                                     (T(i1  ,i2  ,i3  ) +        T(i1  ,i2  ,i3-1)) * &
                                     (s(i1  ,i2  ,i3  ) -        s(i1  ,i2  ,i3-1))/dx3 &
                                )
      !
      ! --- Maximum time step ---
      inv_dtime_limit=max(inv_dtime_limit, nu_total/dx3**2)
    end do ! i1
  end do ! i2
end do ! i3
!
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VIS3D: dvdx,nu,fluxes')
call timing_start( 'VIS3D: update')
!#
# endif
!#
do i3=m3,n3
  do i2=m2,n2
    !DIR$ IVDEP
    do i1=m1,n1
      ! --- Save old velocities (necessary for update of internal energy) ---
      v1=model%v1(i1,i2,i3)
      v2=model%v2(i1,i2,i3)
      v3=model%v3(i1,i2,i3)
      !
      ! --- Abbreviation: dtimeoverrhodx=dt/(rho*dx) ---
      dtimeoverrhodx1=dtime/(model%rho(i1,i2,i3)*dxb1_v(i1))
      dtimeoverrhodx2=dtime/(model%rho(i1,i2,i3)*dxb2_v(i2))
      dtimeoverrhodx3=dtime/(model%rho(i1,i2,i3)*dxb3_v(i3))
      !
      ! --- Update model: apply fluxes (no mass flux) ---
      model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhov11(i1+1, i2  , i3  ) - &
                                                flux_rhov11(i1  , i2  , i3  )) + &
                             dtimeoverrhodx2 * (flux_rhov12(i1  , i2+1, i3  ) - &
                                                flux_rhov12(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhov13(i1  , i2  , i3+1) - &
                                                flux_rhov13(i1  , i2  , i3  )) &
                           )
      model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhov21(i1+1, i2  , i3  ) - &
                                                flux_rhov21(i1  , i2  , i3  )) + &
                             dtimeoverrhodx2 * (flux_rhov22(i1  , i2+1, i3  ) - &
                                                flux_rhov22(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhov23(i1  , i2  , i3+1) - &
                                                flux_rhov23(i1  , i2  , i3  )) &
                           )
      model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhov31(i1+1, i2  , i3  ) - &
                                                flux_rhov31(i1  , i2  , i3  )) + &
                             dtimeoverrhodx2 * (flux_rhov32(i1  , i2+1, i3  ) - &
                                                flux_rhov32(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhov33(i1  , i2  , i3+1) - &
                                                flux_rhov33(i1  , i2  , i3  )) &
                           )

      model%ei(i1,i2,i3)=model%ei(i1,i2,i3) - &
                           ( dtimeoverrhodx1 * (flux_rhoei1(i1+1, i2  , i3  ) - &
                                                flux_rhoei1(i1  , i2  , i3  )) + &
                             dtimeoverrhodx2 * (flux_rhoei2(i1  , i2+1, i3  ) - &
                                                flux_rhoei2(i1  , i2  , i3  )) + &
                             dtimeoverrhodx3 * (flux_rhoei3(i1  , i2  , i3+1) - &
                                                flux_rhoei3(i1  , i2  , i3  )) &
                           ) + &
                             0.5 * ( (v1-model%v1(i1,i2,i3)) * &
                                     (v1+model%v1(i1,i2,i3)) + &
                                     (v2-model%v2(i1,i2,i3)) * &
                                     (v2+model%v2(i1,i2,i3)) + &
                                     (v3-model%v3(i1,i2,i3)) * &
                                     (v3+model%v3(i1,i2,i3)) &
                                   )
    !
    end do ! i1
  end do ! i2
end do ! i3
!
! --- Check if internal energy is below minimum table value anywhere ---
!!$einew_min=huge(einew_min)
!!$do i3=m3,n3
!!$  do i2=m2,n2
!!$    do i1=m1,n1
!!$      einew_min=min(einew_min, model%ei(i1,i2,i3))
!!$    end do ! i1
!!$  end do ! i2
!!$end do ! i3
einew_min=minval(model%ei(m1:n1,m2:n2,m3:n3))
call tabinter_getinfo(x2min=eiTable_min)
if (einew_min < eiTable_min) then
  ierr0=7
  dtime_limit=0.7*dtime
  outstr0='Internal energy below minimum table value'
  goto 999
endif
!
!#
# ifdef rhd_vis_t01
!#
call timing_stop( 'VIS3D: update')
!#
# endif
!#
! \label{f90:c_viscourantmax__use2}\label{f90:c_viscourant__use4}
! --- Check for time step limit violation ---
if (inv_dtime_limit > 100.0*spacing(0.0)) then
  dtime_limit=par%C_visCourant/inv_dtime_limit
  if (dtime > (par%C_visCourantMax/par%C_visCourant*dtime_limit)) then
    ierr0=1
    outstr0='Time step exceeds viscous time scale'
    goto 999
  endif
endif
!
!
! --- Output of additional data ---
if ((Output_flag0) .and. present(dat)) then
!#
# ifdef rhd_vis_t01
!#
  call timing_start( 'VIS3D: (output)')
!#
# endif
!#
  call rhd_vis_StepOutput(model, &
                          AreaFraction10, AreaFraction20, AreaFraction30, &
                          flux_rhoei1, flux_rhoei2, flux_rhoei3, dat)
!#
# ifdef rhd_vis_t01
!#
  call timing_stop( 'VIS3D: (output)')
!#
# endif
!#
endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VIS3D:')
!#
# endif
!#
!
end subroutine rhd_vis_3dStep


!----------*********************--------------------------------------------------------------------
subroutine rhd_vis_p2pStepOutput(model, &
                                 AreaFraction1, AreaFraction2, AreaFraction3, &
                                 drhoei, dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_p2pStepOutput ('rhd_viscosity_point_to_point_Step_Output')
!
! PURPOSE:
!   Put some additional output quantities into dat structure.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_p2pStepOutput(model, &
!                              AreaFraction1, AreaFraction2, AreaFraction3, &
!                              drhoei, dat)
!
! INPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   AreaFraction1: ('Area_Fraction_1') real, optional, weighting factor for chunk in x23 plane
!   AreaFraction2: ('Area_Fraction_2') real, optional, weighting factor for chunk in x13 plane
!   AreaFraction3: ('Area_Fraction_3') real, optional, weighting factor for chunk in x12 plane
!   drhoei:       ('delta_rho_ei_internal') real 3D array with viscous viscous energy change
!
! INPUT/OUTPUT:
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!
! LOCAL VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   weight:       ('weight') real 2D array with column weights
!
! ROUTINES:
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_mean_module:       Averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2001-01-03 (B.F.) Compute additional output data
!   2001-01-28 (B.F.) Put into own routine
!   2001-03-12 (B.F.) Bugfix
!   2001-03-13 (B.F.) Simplifications
!   2004-05-12 (B.F. Montpellier) point-to-point version
!   2006-11-15 (B.F. Lyon) Replaced ";" by "!"
!   2017-08-18 (B.F.) Adapt averaging calls to new datx type
!   2017-08-22 (B.F.) Use wg%hyd instead of prop data
!   2017-08-23 (B.F.) Compute AreaFractions from local and global quantities
!   2017-09-16 (B.F. Uppsala) Compute average over spheres
!   2018-05-07 (B.F.) Remove branch with old averaging routines
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_box_module
use rhd_dat_module
use rhd_mean_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: model
real,                      intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
real, dimension(model%m1:model%n1, &
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: drhoei
type(dat_type),            intent(inout):: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, m,n
real                                    :: AreaFraction12, AreaFraction13, AreaFraction23
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: x12weight
real, dimension(model%m1:model%n1, &
                model%m3:model%n3)      :: x13weight
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: x23weight
real, dimension(:,:), pointer           :: weight
real   , dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: x123weight
integer, dimension(6, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: nb6index
integer, dimension(model%m1:model%n1, &
                   model%m2:model%n2, &
                   model%m3:model%n3 )  :: cindex
integer, dimension(:,:,:), allocatable  :: bindex
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
!!$write(6,'(A,6I4)') 'VP2P: m1,n1, m2,n2, m3,n3: ', m1,n1, m2,n2, m3,n3
!
! --- Compute averages ---
!
if ((wg%hyd%n(1) > wg%hyd%m(1)) .and. (dat%x23%na > 0) .and. (dat%x23%i_devp2p > 0)) then
  ! --- Average over x2-x3 planes ---
  !
  ! --- Compute weight ---
  AreaFraction23=((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m2,n2, m3,n3, model%xb2(m2:n2+1), model%xb3(m3:n3+1), AreaFraction23, &
                        x23weight)
  !
  ! --- Average ---
  call rhd_mean_x23mean(drhoei, dat%x23%d(m1:n1,dat%x23%i_devp2p), x23weight)
endif
!
if ((dat%r%na > 0) .and. (dat%r%i_devp2p > 0) .and. (par%grav_mode == 'central')) then
  ! --- Average over spheres ---
  !
  ! --- Make abbreviations ---
  m=dat%r%m
  n=dat%r%n
  !
  ! --- Allocate boundary-index array ---
  allocate(bindex(-(n-m+2):(n-m+2),m2:n2,m3:n3))
  !
  ! --- Compute weight arrays ---
  call rhd_mean_x23weight(model, prop%length, x23weight, x123weight)
  !
  ! --- Compute index arrays ---
  call rhd_mean_a2rindex(m1,n1, m2,n2, m3,n3, &
                         model%nghost1, wg%hyd%m(1), wg%hyd%n(1), &
                         model%xc1, model%xc2(m2:n2), model%xc3(m3:n3), &
                         dat%r%d1(:,dat%r%i_xb), nb6index, bindex, cindex)
  !
  ! --- Average ---
  call rhd_mean_rc23mean(m1,n1, m2,n2, m3,n3, (n-m+2), &
                         drhoei                                , dat%r%d(m:n,dat%r%i_devp2p), &
                         cindex, x123weight)
  ! --- Deallocate boundary-index array ---
  deallocate(bindex)
  !
endif
!
if ((wg%hyd%n(2) > wg%hyd%m(2)) .and. (dat%x13%na > 0) .and. (dat%x13%i_devp2p > 0)) then
  ! --- Average over x1-x3 planes ---
  !
  ! --- Compute weight ---
  AreaFraction13=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb3(n3+1)-model%xb3(m3)) / (wg%hyd%xb_r(3)-wg%hyd%xb_l(3)))
  call rhd_mean_aweight(m1,n1, m3,n3, model%xb1(m1:n1+1), model%xb3(m3:n3+1), AreaFraction13, &
                        x13weight)
  !
  ! --- Average ---
  call rhd_mean_x13mean(drhoei, dat%x13%d(m2:n2,dat%x13%i_devp2p), x13weight)
endif
!
if ((wg%hyd%n(3) > wg%hyd%m(3)) .and. (dat%x12%na > 0) .and. (dat%x12%i_devp2p > 0)) then
  ! --- Average over x1-x2 planes ---
  !
  ! --- Create and compute weight ---
  AreaFraction12=((model%xb1(n1+1)-model%xb1(m1)) / (wg%hyd%xb_r(1)-wg%hyd%xb_l(1))) * &
                 ((model%xb2(n2+1)-model%xb2(m2)) / (wg%hyd%xb_r(2)-wg%hyd%xb_l(2)))
  call rhd_mean_aweight(m1,n1, m2,n2, model%xb1(m1:n1+1), model%xb2(m2:n2+1), AreaFraction12, &
                        x12weight)
  !
  ! --- Average ---
  call rhd_mean_x12mean(drhoei, dat%x12%d(m3:n3,dat%x12%i_devp2p), x12weight)
endif
!
end subroutine rhd_vis_p2pStepOutput


!----------*****************------------------------------------------------------------------------
subroutine rhd_vis_p2p2dStep(model, action, dtime, dtime_limit, nghost, Output_flag, &
                             AreaFraction1, AreaFraction2, AreaFraction3, &
                             dat, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_p2p2dStep ('rhd_viscosity_point_2_point_2_dimension_Step')
!
! PURPOSE:
!   Update model by applying 2D point-to-point (tensor) viscosity.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_p2p2dStep(model, action, dtime, dtime_limit, nghost, Output_flag=Output_flag, &
!                          AreaFraction1=AreaFraction1, AreaFraction2=AreaFraction2, &
!                          AreaFraction3=AreaFraction3, &
!                          dat=dat, outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   nghost:       ('number_ghost_cells') integer with number of ghost cells
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                   default=.false.
!   AreaFraction1: ('Area_Fraction_1') real, optional, weighting factor for chunk in x23 plane
!   AreaFraction2: ('Area_Fraction_2') real, optional, weighting factor for chunk in x13 plane
!   AreaFraction3: ('Area_Fraction_3') real, optional, weighting factor for chunk in x12 plane
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, proposed maximum time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                    =0: normal run
!                  1-99: "small" error (e.g., time step limit violation)
!                 >=100: "severe" error (does not occur in this version)
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   m1l:          ('m_1_limit') integer, lower loop index limit, 1st dimension
!   n1l:          ('n_1_limit') integer, upper loop index limit, 1st dimension
!   i1:           ('index_1') loop index (source cell), 1st dimension
!   di1:          ('delta_index_1') integer, relative index shift of target cell, 1st dimension
!   i1d:          ('index_1_delta') integer, index (target cell), 1st dimension
!   idir:         ('index_direction') integer, index for loop over directions
!   ndir:         ('number_direction') integer, number of directions
!   inv_dtime_limit: ('inverse_delta_time_limit') real, accumulated inverse limit for next time step
!   einew_min:    ('energy_internal_new_minimum') real, total minimum of internal energy after
!                 viscous energy update
!   eiTable_min:  ('energy_internal_Table_minimum') real, minimum energy allowed by EOS table
!   nu_total:     ('nu_total') real, kinematic viscosity
!   rhob:         ('rho_boundary') real, "mean" density at boundary between two cells
!   visP2P:       ('viscosity_Point_to_Point') real, total viscosity factor
!   dx1:          ('delta_x_1') real, distance of cell centers, 1st dimension
!   dx_sqr:       ('delta_x_square') real, square of distance of cell centers
!   dx:           ('delta_x') real, distance of cell centers
!   vol0_scale:   ('volume_0_scale') real, scaled (i.e. not in cm^3) volume of source cell
!   vold_scale:   ('volume_delta_scale') real, scaled (i.e. not in cm^3) volume of target cell
!   area_scale:   ('area_scale') real, scaled area through which the flux goes
!   dx0_update:   ('delta_x_0_update') real, geometrical factor (volume/area)
!                 for update of source cell
!   dxd_update:   ('delta_x_delta_update') real, geometrical factor (volume/area)
!                 for update of target cell
!   dv1:          ('delta_velocity_1') real, 1st component of velocity difference
!                 along line of sight
!   dv:           ('delta_velocity') real, velocity difference along line of sight
!   dvdx:         ('delta_velocity_delta_x') real, velocity gradient along line of sight
!   sv1:          ('sum_velocity_1') real, 1st component of half velocity sum
!                 along line of sight
!   sv:           ('sum_velocity') real, half velocity sum along line of sight
!   v1_old:       ('velocity_1_old') real, velocity before update, 1st component
!   ei_old:       ('energy_internal_old') real, internal energy before update
!   dtimeoverrhodx: ('delta_time_over_rho_dx') real, geometrical update factor (flux -> change)
!   flux_rhov:    ('flux_rho_velocity') real, viscous momentum flux along line of sight
!   flux_rhov1:   ('flux_rho_velocity_1') real, momentum flux along line of sight, 1st component
!   flux_rhoei:   ('flux_rho_energy_internal') real, viscous energy flux along line of sight
!   dxb1_scale:   ('delta_x_boundary_1_scale') real vector, scales grid size, 1st dimension
!   v1:           ('velocity_1') real 3D array, 1st component of velocities
!   ei:           ('energy_internal') real 3D array, internal energy
!   dev:          ('delta_energy_viscous') real 3D array, divergence of viscous energy flux
!   di1arr:       ('delta_index_1_array') integer vector, relative index of target cell, 1st dim.
!   TenCoeffarr:  ('Tensor_Coefficient_array') real array, with weights for each direction
!
! ROUTINES:
!   rhd_vis_p2pStepOutput: Put some additional output quantities into dat structure
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   All output via variables in parameter list.
!
! RESTRICTIONS:
!   Assumes that m2==n2.
!   There should be exactly one layer of (already filled) ghost cells in x1 and x3 direction.
!
! PROCEDURE:
!   The velocity of each cell relative to all immediate neighbors is computed.
!   This relative velocity is used to compute the viscous momentum exchange
!     between each pair of cells ('source' and 'target') -- only along each line of sight.
!
!           target +->  -
!                 /|    |
!                v |    |
!                  v    |
!              ^        | dx3
!          ^  /         |
!          | /          |
!          |/           |
!   source +------>     -
!
!          |-------|
!             dx1
!
!   The viscous energy flux is computed consistently.
!   This should guarantee angular momentum conservation
!     AND the positivity of the viscous energy dissipation.
!
! EXAMPLE:
!
! TODO:
!   spherical mean
!   mean based on fluxes
!   (faster) version with constant geometrical coefficients
!
! MODIFICATION HISTORY:
!   1999-06-01 (B.F. Copenhagen) First Version
!   2004-05-11 (B.F.Montpellier) First draft: point-to-point viscosity
!   2004-05-13 (B.F.) Modifications
!   2004-05-14 (B.F.) More comments
!   2004-05-17 (B.F.) New formula for time step limit; order of time step checks altered
!   2004-05-18 (B.F.) New brackets in update of internal energy
!   2011-09-06 (B.F. Lyon) Smagorinsky-type viscosity
!   2011-11-26 (B.F.) Improved vectorization: more branches
!   2011-11-28 (B.F.) Improved vectorization: compute max. time step in one go, add IVDEP directive
!   2011-12-01 (B.F.) New parameter nghost; artificial viscosity; use MinMod for dvdx
!   2011-12-29 (B.F.) Bugfix: artificial viscosity
!   2012-01-17 (B.F.) visDivrhov
!   2012-01-19 (B.F.) Small rearrangements
!   2012-01-20 (B.F.) Check for negative internal energy changes relative to reference energy
!   2012-07-27 (B.F.) Move scale%length to prop%length
!---------------------------------------------------------------------------------------------------
use tabinter_module
use rhd_gl_module
use rhd_prop_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
!!$use uio_bulk_module
!!$use uio_filedef_module
!#
# ifdef rhd_vis_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
integer,                   intent(in)   :: nghost
logical,         optional, intent(in)   :: Output_flag
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag0
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           m1l, n1l, m3l, n3l, &
                                           i1, i2, i3, &
                                           di1, di2, di3, i1d, i2d, i3d, &
                                           idir, ndir, &
                                           di1SA, di1SB, di3SA, di3SB, &
                                           nc
real                                    :: AreaFraction10, AreaFraction30, &
                                           inv_dtime_limit, einew_min, eiTable_min, &
                                           nu_total, rhob, visP2P, &
                                           dx1, dx2, dx3, oneodx1, oneodx2, oneodx3, &
                                           dx_sqr, dx, vol0_scale, vold_scale, area_scale, &
                                           dx0_update, dxd_update, &
                                           dv1, dv3, dv, dvdx, divv, &
                                           sv1, sv3, sv, &
                                           v1_old, v3_old, &
                                           dtimeoverrhodx, &
                                           flux_rhov, flux_rhov1,flux_rhov3, flux_rhoei, &
                                           visSmagorinsky, visArtificial, visExpansion, visDivrhov
!real                                    :: ei_old
real, dimension(model%m1-1:model%n1+1)  :: dxb1_scale, dv1v, dei1v, &
                                           deltav1, deltav1R, deltav3, deltav3R
!real, dimension(model%m2  :model%n2  )  :: dxb2_scale
real, dimension(model%m3-1:model%n3+1)  :: dxb3_scale, dv3v, dei3v
real, dimension(model%m1-nghost:model%n1+nghost, &
                model%m2       :model%n2  , &
                model%m3-nghost:model%n3+nghost) :: v1, v3, ei, dev
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2  , &
                model%m3  :model%n3+1)  :: nuVis
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2  , &
                model%m3-1:model%n3+1)  :: dv1dx1, dv3dx1
real, dimension(model%m1-1:model%n1+1, &
                model%m2  :model%n2  , &
                model%m3  :model%n3+1)  :: dv1dx3, dv3dx3
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, parameter, dimension(4)        :: di1arr  =(/ 1, 0,  1,-1 /), &
                                           di3arr  =(/ 0, 1,  1, 1 /), &
                                           di1SAarr=(/ 1, 0,  1, 0 /), &
                                           di1SBarr=(/ 1, 1,  1, 0 /), &
                                           di3SAarr=(/ 0, 1,  1, 1 /), &
                                           di3SBarr=(/ 1, 1,  1, 1 /)
real,               dimension(4)        :: TenCoeffarr
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VISp2p2D:')
!#
# endif
!#
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Set default values for keywords ---
if (.not. present(Output_flag)) then
  Output_flag0=.false.
else
  Output_flag0=Output_flag
endif
if (.not. present(AreaFraction1)) then
  AreaFraction10=1.0
else
  AreaFraction10=AreaFraction1
endif
if (.not. present(AreaFraction3)) then
  AreaFraction30=1.0
else
  AreaFraction30=AreaFraction3
endif
!
! --- Give initially no limit on time step ---
dtime_limit    =0.1*huge(dtime_limit)
inv_dtime_limit=0.0
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Compute grid size ---
do i1=m1-1,n1+1
  dxb1_scale=(model%xb1(i1+1)-model%xb1(i1))/prop%length
end do ! i1
do i3=m3-1,n3+1
  dxb3_scale=(model%xb3(i3+1)-model%xb3(i3))/prop%length
end do ! i3
!dxb1_scale=(model%xb1(m1  :n1+2) - model%xb1(m1-1:n1+1))/prop%length
!!dxb2_scale=(model%xb2(m2+1:n2+1) - model%xb2(m2  :n2  ))/prop%length
!dxb3_scale=(model%xb3(m3  :n3+2) - model%xb3(m3-1:n3+1))/prop%length
!!$write(6,'(A,2E13.5)') 'minmax(dxb1_scale): ', minval(dxb1_scale), maxval(dxb1_scale)
!!$!write(6,'(A,2E13.5)') 'minmax(dxb2_scale): ', minval(dxb2_scale), maxval(dxb2_scale)
!!$write(6,'(A,2E13.5)') 'minmax(dxb3_scale): ', minval(dxb3_scale), maxval(dxb3_scale)
!
! \label{f90:c_visp2pincl1__use1}
! --- Define tensor coefficient array ---
TenCoeffarr=(/ 1.0, 1.0, &
               par%C_visP2Pincl1, par%C_visP2Pincl1 /)
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VISp2p2D: dvdx,nu,fluxes,update')
!#
# endif
!#
!
! --- 2D case: di2 is always zero, i2=m2=n2 ---
di2=0
i2 =m2
i2d=i2+di2
!
! \label{f90:c_visp2phypartificial__use1}
! --- Set viscosity coefficients ---
visSmagorinsky=max(par%C_visP2PhypSmagorinsky,0.0)**2
visArtificial =max(par%C_visP2PhypArtificial ,0.0)**2
visExpansion  =max(par%C_visP2PhypExpansion  ,0.0)**2
visDivrhov    =max(par%C_visP2Pdivrhov       ,0.0)**2
!write(6,'(A,3(1PE13.5),I4)') 'VisP2P: visSma,Art,Exp;nghost:', &
!  visSmagorinsky, visArtificial, visExpansion, nghost
!
! --- Save velocities and internal energy ---
v1 =model%v1
v3 =model%v3
ei =model%ei
dev=0.0
!
if ((visSmagorinsky > 0.0) .or. (visArtificial > 0.0) .or. (visExpansion > 0.0)) then
  if (par%A%visP2Phypreconstruction_type == reconstruction_type_Minmod) then
    ! --- Compute MinMod differences ---
    !
    ! --- Compute MinMod differences in x1 direction ---
    do i3=m3-1,n3+1
      do i1=m1-1,n1+1
        ! --- Compute slopes in cells ---
        deltav1(i1)=( model%xb1(i1+1) - model%xb1(i1) ) * &
                        ( min( max((v1( i1  ,i2,i3) - v1( i1-1,i2,i3))/ &
                                   (model%xb1(i1+1      ) - model%xb1(i1-1      )) , 0.0) , &
                               max((v1( i1+1,i2,i3) - v1( i1  ,i2,i3))/ &
                                   (model%xb1(i1+2      ) - model%xb1(i1        )) , 0.0) ) + &
                          max( min((v1( i1  ,i2,i3) - v1( i1-1,i2,i3))/ &
                                   (model%xb1(i1+1      ) - model%xb1(i1-1      )) , 0.0) , &
                               min((v1( i1+1,i2,i3) - v1( i1  ,i2,i3))/ &
                                   (model%xb1(i1+2      ) - model%xb1(i1        )) , 0.0) )  )
        deltav3(i1)=( model%xb1(i1+1) - model%xb1(i1) ) * &
                        ( min( max((v3( i1  ,i2,i3) - v3( i1-1,i2,i3))/ &
                                   (model%xb1(i1+1      ) - model%xb1(i1-1      )) , 0.0) , &
                               max((v3( i1+1,i2,i3) - v3( i1  ,i2,i3))/ &
                                   (model%xb1(i1+2      ) - model%xb1(i1        )) , 0.0) ) + &
                          max( min((v3( i1  ,i2,i3) - v3( i1-1,i2,i3))/ &
                                   (model%xb1(i1+1      ) - model%xb1(i1-1      )) , 0.0) , &
                               min((v3( i1+1,i2,i3) - v3( i1  ,i2,i3))/ &
                                   (model%xb1(i1+2      ) - model%xb1(i1        )) , 0.0) )  )
      end do ! i1
      !
      do i1=m1,n1+1
        ! --- Compute geometrical data ---
        oneodx1=1.0/(model%xc1(i1)-model%xc1(i1-1))
        !
        ! --- Compute jump across cell boundaries after MinMod reconstruction in x1 direction ---
        dv1dx1(i1,i2,i3)=((v1(i1  ,i2,i3  )-deltav1(i1  )) - &
                          (v1(i1-1,i2,i3  )+deltav1(i1-1))  ) * oneodx1
        dv3dx1(i1,i2,i3)=((v3(i1  ,i2,i3  )-deltav3(i1  )) - &
                          (v3(i1-1,i2,i3  )+deltav3(i1-1))  ) * oneodx1
      end do ! i1
    end do ! i3
    !
    ! --- Compute MinMod differences in x3 direction with inner loop over x1 direction ---
    i3=m3-1
    do i1=m1-1,n1+1
      ! --- Compute slopes in cells on left side of boundary ---
      deltav1(i1)=( model%xb3(i3+1) - model%xb3(i3) ) * &
                        ( min( max((v1( i1,i2,i3  ) - v1( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               max((v1( i1,i2,i3+1) - v1( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) ) + &
                          max( min((v1( i1,i2,i3  ) - v1( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               min((v1( i1,i2,i3+1) - v1( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) )  )
      deltav3(i1)=( model%xb3(i3+1) - model%xb3(i3) ) * &
                        ( min( max((v3( i1,i2,i3  ) - v3( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               max((v3( i1,i2,i3+1) - v3( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) ) + &
                          max( min((v3( i1,i2,i3  ) - v3( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               min((v3( i1,i2,i3+1) - v3( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) )  )
    end do ! i1
    do i3=m3,n3+1
      do i1=m1-1,n1+1
        ! --- Compute slopes in cells on right side of boundary ---
        deltav1R(i1)=( model%xb3(i3+1) - model%xb3(i3) ) * &
                        ( min( max((v1( i1,i2,i3  ) - v1( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               max((v1( i1,i2,i3+1) - v1( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) ) + &
                          max( min((v1( i1,i2,i3  ) - v1( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               min((v1( i1,i2,i3+1) - v1( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) )  )
        deltav3R(i1)=( model%xb3(i3+1) - model%xb3(i3) ) * &
                        ( min( max((v3( i1,i2,i3  ) - v3( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               max((v3( i1,i2,i3+1) - v3( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) ) + &
                          max( min((v3( i1,i2,i3  ) - v3( i1,i2,i3-1))/ &
                                   (model%xb3(      i3+1) - model%xb3(      i3-1)) , 0.0) , &
                               min((v3( i1,i2,i3+1) - v3( i1,i2,i3  ))/ &
                                   (model%xb3(      i3+2) - model%xb3(      i3  )) , 0.0) )  )
      end do ! i1
      !
      ! --- Compute geometrical data ---
      oneodx3=1.0/(model%xc3(i3)-model%xc3(i3-1))
      !
      do i1=m1-1,n1+1
        ! --- Compute jump across cell boundaries after MinMod reconstruction in x1 direction ---
        dv1dx3(i1,i2,i3)=((v1(i1,i2,i3  )-deltav1R(i1)) - &
                          (v1(i1,i2,i3-1)+deltav1( i1))  ) * oneodx3
        dv3dx3(i1,i2,i3)=((v3(i1,i2,i3  )-deltav3R(i1)) - &
                          (v3(i1,i2,i3-1)+deltav3( i1))  ) * oneodx3
      end do ! i1
      !
      ! --- Copy slopes for next iteration ---
      deltav1=deltav1R
      deltav3=deltav3R
    end do ! i3
  else
    ! --- Just compute differences (corresponds to constant reconstruction) ---
    !
    do i3=m3-1,n3+1
      do i1=m1,n1+1
        ! --- Compute geometrical data ---
        oneodx1=1.0/(model%xc1(i1)-model%xc1(i1-1))
        !
        ! --- Take discrete derivative in x1 direction ---
        dv1dx1(i1,i2,i3)=(v1(i1  ,i2,i3  )-v1(i1-1,i2,i3  ))*oneodx1
        dv3dx1(i1,i2,i3)=(v3(i1  ,i2,i3  )-v3(i1-1,i2,i3  ))*oneodx1
      end do ! i1
    end do ! i3
    !
    do i3=m3,n3+1
      do i1=m1-1,n1+1
        ! --- Compute geometrical data ---
        oneodx3=1.0/(model%xc3(i3)-model%xc3(i3-1))
        !
        ! --- Take discrete derivative in x3 direction ---
        dv1dx3(i1,i2,i3)=(v1(i1  ,i2,i3  )-v1(i1  ,i2,i3-1))*oneodx3
        dv3dx3(i1,i2,i3)=(v3(i1  ,i2,i3  )-v3(i1  ,i2,i3-1))*oneodx3
      end do ! i1
    end do ! i3
    !
  endif
  !
  if (visSmagorinsky > 0.0) then
    !
    do i3=m3,n3+1
      do i1=m1,n1+1
        ! --- Compute Smagorinsky (square of) magnitude of strain-rate tensor at cell corners ---
        ! --- Factor 1/2 ---
        nuVis(i1,i2,i3)=(0.5*visSmagorinsky) * &
                      sqrt(    dv1dx1(i1  ,i2,i3-1)**2 + &
                               dv1dx1(i1  ,i2,i3  )**2   &
                                 + &
                               dv3dx3(i1-1,i2,i3  )**2 + &
                               dv3dx3(i1  ,i2,i3  )**2 + &
                        0.25*((dv1dx3(i1-1,i2,i3  ) + dv3dx1(i1  ,i2,i3-1))**2 + &
                              (dv1dx3(i1  ,i2,i3  ) + dv3dx1(i1  ,i2,i3-1))**2  &
                                 + &
                              (dv1dx3(i1-1,i2,i3  ) + dv3dx1(i1  ,i2,i3  ))**2 + &
                              (dv1dx3(i1  ,i2,i3  ) + dv3dx1(i1  ,i2,i3  ))**2  ) )
      end do ! i1
    end do ! i3
    !
  else
    nuVis=0.0
  endif
  !
  !write(6,*) 'nuVis :', minval(nuVis), maxval(nuVis)
  !write(6,*) 'dv1dx1:', minval(dv1dx1), maxval(dv1dx1)
  !write(6,*) 'dv3dx3:', minval(dv3dx3), maxval(dv3dx3)
  !
  if ((visArtificial > 0.0) .or. (visExpansion > 0.0)) then
    do i3=m3,n3+1
      do i1=m1,n1+1
        ! --- Compute divergence at cell corners (factor 1/2) ---
        divv=0.25 * (dv1dx1(i1  ,i2,i3-1) + &
                     dv1dx1(i1  ,i2,i3  )  &
                       + &
                     dv3dx3(i1-1,i2,i3  ) + &
                     dv3dx3(i1  ,i2,i3  ))
        !
        ! --- Add contribution to viscosity ---
        nuVis(i1,i2,i3)=nuVis(i1,i2,i3) + visArtificial * max(-divv, 0.0) + &
                                          visExpansion  * max( divv, 0.0)
  !if (abs(divv) > 10.0) write(6,'(3I4,6(1PE12.4))') i1,i2,i3, &
  !     dv1dx1(i1  ,i2,i3-1), dv1dx1(i1  ,i2,i3  ), &
  !     dv3dx3(i1-1,i2,i3  ), dv3dx3(i1  ,i2,i3  ), divv, nuVis(i1,i2,i3)
      end do ! i1
    end do ! i3
  endif
else
  nuVis=0.0
endif
!
if (visDivrhov > 0.0) then
  ! --- Just compute differences (corresponds to constant reconstruction) ---
  ! --- Put drhovdx into dvdx arrays ---
  !
  do i3=m3-1,n3+1
    do i1=m1,n1+1
      ! --- Compute geometrical data ---
      oneodx1=1.0/(model%xc1(i1)-model%xc1(i1-1))
      !
      ! --- Take discrete derivative in x1 direction ---
      dv1dx1(i1,i2,i3)=(model%rho(i1  ,i2,i3  )*v1(i1  ,i2,i3  )- &
                        model%rho(i1-1,i2,i3  )*v1(i1-1,i2,i3  ) ) * oneodx1
    end do ! i1
  end do ! i3
  !
  do i3=m3,n3+1
    do i1=m1-1,n1+1
      ! --- Compute geometrical data ---
      oneodx3=1.0/(model%xc3(i3)-model%xc3(i3-1))
      !
      ! --- Take discrete derivative in x3 direction ---
      dv3dx3(i1,i2,i3)=(model%rho(i1  ,i2,i3  )*v3(i1  ,i2,i3  )- &
                        model%rho(i1  ,i2,i3-1)*v3(i1  ,i2,i3-1) ) * oneodx3
    end do ! i1
  end do ! i3
  !
  do i3=m3,n3+1
    do i1=m1,n1+1
      ! --- Compute divergence at cell corners (factor 1/2) ---
      divv=      (dv1dx1(   i1  ,i2,i3-1) + &
                  dv1dx1(   i1  ,i2,i3  )   &
                    + &
                  dv3dx3(   i1-1,i2,i3  ) + &
                  dv3dx3(   i1  ,i2,i3  )    ) / &
                 (model%rho(i1-1,i2,i3-1) + &
                  model%rho(i1  ,i2,i3-1) + &
                  model%rho(i1-1,i2,i3  ) + &
                  model%rho(i1  ,i2,i3  )    )
      !
      ! --- Add contribution to viscosity ---
      !nuVis(i1,i2,i3)=nuVis(i1,i2,i3) + min(visDivrhov * abs(divv), 1.0E-01)
      nuVis(i1,i2,i3)=nuVis(i1,i2,i3) + visDivrhov * abs(divv)
    end do ! i1
  end do ! i3
  !
endif
!
!write(6,*) 'nuVis :', minval(nuVis), maxval(nuVis)
!
! \label{f90:c_visp2pincl1__use2}
! --- Update for all (2 or 4) directions ---
if (par%C_visP2Pincl1 <= 0.0) then
  ndir=2
else
  ndir=4
endif
do idir=1, ndir
  !
  ! --- Set current direction ---
  di1=di1arr(idir)
  di3=di3arr(idir)
  ! \label{f90:c_visp2pcoeff__use1}
  visP2P=max(par%C_visP2Pcoeff, 0.0)**2
  !
  ! --- Compute loop index limits ---
  m1l=m1-max( di1,0)
  n1l=n1+max(-di1,0)
  m3l=m3-max( di3,0)
  n3l=n3+max(-di3,0)
  !
  ! --- Set indices for averages of Smagorinsky magnitude of the strain-rate tensor ---
  di1SA=di1SAarr(idir)
  di1SB=di1SBarr(idir)
  di3SA=di3SAarr(idir)
  di3SB=di3SBarr(idir)
  !
!!$  write(6,'(A,3I13,E13.5)') 'idir, di1, di3, visP2P:              ', idir, di1, di3, visP2P
  if ((di1 == 1) .and. (di3 == 0)) then
    ! --- Special case: only di1 /= 0; simplify and remove vector dependency ---
    !
    do i3=m3l, n3l
      ! --- Set indices of target cell ---
      i3d=i3
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1+di1
        !
        ! --- Geometrical data ---
        dx1       =model%xc1(i1d)-model%xc1(i1)
!        dx3       =model%xc3(i3d)-model%xc3(i3)
        dx_sqr    =dx1**2
        dx        =dx1
        vol0_scale=dxb1_scale(i1 ) * dxb3_scale(i3 )
        vold_scale=dxb1_scale(i1d) * dxb3_scale(i3d)
        area_scale=0.5*(vol0_scale + vold_scale)/(dx/prop%length)
        dx0_update=vol0_scale*prop%length/area_scale
        dxd_update=vold_scale*prop%length/area_scale
        !
        ! --- Velocity difference along vector connecting points ---
        dv1 =model%v1(i1d,i2d,i3d) - model%v1(i1,i2,i3)
!        dv3 =model%v3(i1d,i2d,i3d) - model%v3(i1,i2,i3)
        dv  =dv1
        dvdx=dv / dx
        sv1 =0.5*(model%v1(i1d,i2d,i3d) + model%v1(i1,i2,i3))
!        sv3 =0.5*(model%v3(i1d,i2d,i3d) + model%v3(i1,i2,i3))
        sv  =sv1
        !
        ! --- Kinematic viscosity ---
        nu_total=dx_sqr * TenCoeffarr(idir) * &
                 (visP2P * abs(dvdx) + &
                  nuVis(i1+di1SA,i2,i3+di3SA) + &
                  nuVis(i1+di1SB,i2,i3+di3SB)  )
        !
        ! --- Density at boundary ---
!#
#       if (rhd_vis_density_p01 == 0)
!#                                    /* 'min' */
        rhob= min(model%rho(i1d,i2d,i3d),  model%rho(i1,i2,i3))
!#
#       else
!#           /* rhd_vis_density_p01 == 1 '0.5+0.5' */
        rhob=0.5*(model%rho(i1d,i2d,i3d) + model%rho(i1,i2,i3))
!#
#       endif
!#            /* rhd_vis_density_p01 */
        !
        ! --- Momentum fluxes and energy flux ---
        flux_rhov =-rhob * nu_total * dvdx
        flux_rhov1=         flux_rhov
!        flux_rhov3=dx3/dx * flux_rhov
        flux_rhoei=sv     * flux_rhov
        !
        ! --- Update: source ---
        dtimeoverrhodx  =dtime/(model%rho(i1 ,i2 ,i3 )*dx0_update)
        v1_old          =v1(i1 ,i2 ,i3 )
!        v3_old          =v3(i1 ,i2 ,i3 )
        v1( i1 ,i2 ,i3 )=v1(i1 ,i2 ,i3 ) -   dtimeoverrhodx * flux_rhov1
!        v3( i1 ,i2 ,i3 )=v3(i1 ,i2 ,i3 ) -   dtimeoverrhodx * flux_rhov3
        ei( i1 ,i2 ,i3 )=ei(i1 ,i2 ,i3 ) - ( dtimeoverrhodx * flux_rhoei - &
                               0.5 * ( (v1_old**2 - v1(i1 ,i2 ,i3 )**2) &
                                     ) &
                                           )
        dev(i1 ,i2 ,i3 )=dev(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhoei
        !
        ! --- Compute and store update of target ---
        dtimeoverrhodx  =dtime/(model%rho(i1d,i2d,i3d)*dxd_update)
        !v1_old          =v1(i1d,i2d,i3d)
        dv1v(i1d       )=dtimeoverrhodx * flux_rhov1
        !ei( i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dtimeoverrhodx * flux_rhoei + &
        !                       0.5 * ( (v1_old**2 - v1(i1d,i2d,i3d)**2) &
        !                             ) &
        !                                   )
        dei1v(i1d      )=dtimeoverrhodx * flux_rhoei
        !dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhoei
        !
        ! --- Maximum time step ---
        inv_dtime_limit=max(inv_dtime_limit, &
                            (nu_total*rhob)/(model%rho(i1 ,i2 ,i3 )*dx0_update*dx), &
                            (nu_total*rhob)/(model%rho(i1d,i2d,i3d)*dxd_update*dx))
      end do ! i1
      !
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1+di1
        !
        v1_old         =v1(i1d,i2d,i3d)
        v1(i1d,i2d,i3d)=v1(i1d,i2d,i3d) + dv1v( i1d)
        ei(i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dei1v(i1d) + &
                               0.5 * ( (v1_old**2 - v1(i1d,i2d,i3d)**2) &
                                     ) &
                                          )
        dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dei1v(i1d)
      end do ! i1
    end do ! i3
  else if ((di1 == 0) .and. (di3 == 1)) then
    ! --- Special case: only di3 /= 0 ---
    !
    do i3=m3l, n3l
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1
        i3d=i3+di3
        !
        ! --- Geometrical data ---
!        dx1       =model%xc1(i1d)-model%xc1(i1)
        dx3       =model%xc3(i3d)-model%xc3(i3)
        dx_sqr    =dx3**2
        dx        =dx3
        vol0_scale=dxb1_scale(i1 ) * dxb3_scale(i3 )
        vold_scale=dxb1_scale(i1d) * dxb3_scale(i3d)
        area_scale=0.5*(vol0_scale + vold_scale)/(dx/prop%length)
        dx0_update=vol0_scale*prop%length/area_scale
        dxd_update=vold_scale*prop%length/area_scale
        !
        ! --- Velocity difference along vector connecting points ---
!        dv1 =model%v1(i1d,i2d,i3d) - model%v1(i1,i2,i3)
        dv3 =model%v3(i1d,i2d,i3d) - model%v3(i1,i2,i3)
        dv  =dv3
        dvdx=dv / dx
!        sv1 =0.5*(model%v1(i1d,i2d,i3d) + model%v1(i1,i2,i3))
        sv3 =0.5*(model%v3(i1d,i2d,i3d) + model%v3(i1,i2,i3))
        sv  =sv3
        !
        ! --- Kinematic viscosity ---
        nu_total=dx_sqr * TenCoeffarr(idir) * &
                 (visP2P * abs(dvdx) + &
                  nuVis(i1+di1SA,i2,i3+di3SA) + &
                  nuVis(i1+di1SB,i2,i3+di3SB)  )
        !
        ! --- Density at boundary ---
!#
#       if (rhd_vis_density_p01 == 0)
!#                                    /* 'min' */
        rhob= min(model%rho(i1d,i2d,i3d),  model%rho(i1,i2,i3))
!#
#       else
!#           /* rhd_vis_density_p01 == 1 '0.5+0.5' */
        rhob=0.5*(model%rho(i1d,i2d,i3d) + model%rho(i1,i2,i3))
!#
#       endif
!#            /* rhd_vis_density_p01 */
        !
        ! --- Momentum fluxes and energy flux ---
        flux_rhov =-rhob * nu_total * dvdx
!        flux_rhov1=dx1/dx * flux_rhov
        flux_rhov3=         flux_rhov
        flux_rhoei=sv     * flux_rhov
        !
        ! --- Update: source ---
        dtimeoverrhodx  =dtime/(model%rho(i1 ,i2 ,i3 )*dx0_update)
!        v1_old          =v1(i1 ,i2 ,i3 )
        v3_old          =v3(i1 ,i2 ,i3 )
!        v1( i1 ,i2 ,i3 )=v1(i1 ,i2 ,i3 ) -   dtimeoverrhodx * flux_rhov1
        v3( i1 ,i2 ,i3 )=v3(i1 ,i2 ,i3 ) -   dtimeoverrhodx * flux_rhov3
        ei( i1 ,i2 ,i3 )=ei(i1 ,i2 ,i3 ) - ( dtimeoverrhodx * flux_rhoei - &
                               0.5 * ( (v3_old**2 - v3(i1 ,i2 ,i3 )**2)  &
                                     ) &
                                           )
        dev(i1 ,i2 ,i3 )=dev(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhoei
        !
        ! --- Update: target ---
        dtimeoverrhodx  =dtime/(model%rho(i1d,i2d,i3d)*dxd_update)
!        v1_old          =v1(i1d,i2d,i3d)
        v3_old          =v3(i1d,i2d,i3d)
!        v1( i1d,i2d,i3d)=v1(i1d,i2d,i3d) +   dtimeoverrhodx * flux_rhov1
        v3( i1d,i2d,i3d)=v3(i1d,i2d,i3d) +   dtimeoverrhodx * flux_rhov3
        ei( i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dtimeoverrhodx * flux_rhoei + &
                               0.5 * ( (v3_old**2 - v3(i1d,i2d,i3d)**2)  &
                                     ) &
                                           )
        dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhoei
        !
        ! --- Maximum time step ---
        inv_dtime_limit=max(inv_dtime_limit, &
                            (nu_total*rhob)/(model%rho(i1 ,i2 ,i3 )*dx0_update*dx), &
                            (nu_total*rhob)/(model%rho(i1d,i2d,i3d)*dxd_update*dx))
      end do ! i1
    end do ! i3
  else if ((di1 /= 0) .and. (di3 /= 0)) then
    ! --- General case ---
    !
    do i3=m3l, n3l
      !DIR$ IVDEP
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1+di1
        i3d=i3+di3
        !
        ! --- Geometrical data ---
        dx1       =model%xc1(i1d)-model%xc1(i1)
        dx3       =model%xc3(i3d)-model%xc3(i3)
        dx_sqr    =dx1**2 + dx3**2
        dx        =sqrt(dx_sqr)
        vol0_scale=dxb1_scale(i1 ) * dxb3_scale(i3 )
        vold_scale=dxb1_scale(i1d) * dxb3_scale(i3d)
        area_scale=0.5*(vol0_scale + vold_scale)/(dx/prop%length)
        dx0_update=vol0_scale*prop%length/area_scale
        dxd_update=vold_scale*prop%length/area_scale
        !
        ! --- Velocity difference along vector connecting points ---
        dv1 =model%v1(i1d,i2d,i3d) - model%v1(i1,i2,i3)
        dv3 =model%v3(i1d,i2d,i3d) - model%v3(i1,i2,i3)
        dv  =(dv1*dx1 + dv3*dx3) / dx
        dvdx=dv / dx
        sv1 =0.5*(model%v1(i1d,i2d,i3d) + model%v1(i1,i2,i3))
        sv3 =0.5*(model%v3(i1d,i2d,i3d) + model%v3(i1,i2,i3))
        sv  =(sv1*dx1 + sv3*dx3) / dx
        !
        ! --- Kinematic viscosity ---
!        nu_total=dx_sqr * visP2P * abs(dvdx)
        nu_total=dx_sqr * TenCoeffarr(idir) * &
                 (visP2P * abs(dvdx) + &
                  nuVis(i1+di1SA,i2,i3+di3SA) + &
                  nuVis(i1+di1SB,i2,i3+di3SB)  )
        !
        ! --- Density at boundary ---
!#
#       if (rhd_vis_density_p01 == 0)
!#                                    /* 'min' */
        rhob= min(model%rho(i1d,i2d,i3d),  model%rho(i1,i2,i3))
!#
#       else
!#           /* rhd_vis_density_p01 == 1 '0.5+0.5' */
        rhob=0.5*(model%rho(i1d,i2d,i3d) + model%rho(i1,i2,i3))
!#
#       endif
!#            /* rhd_vis_density_p01 */
        !
        ! --- Momentum fluxes and energy flux ---
        flux_rhov =-rhob * nu_total * dvdx
        flux_rhov1=dx1/dx * flux_rhov
        flux_rhov3=dx3/dx * flux_rhov
        flux_rhoei=sv     * flux_rhov
        !
        ! --- Update: source ---
        dtimeoverrhodx  =dtime/(model%rho(i1 ,i2 ,i3 )*dx0_update)
        v1_old          =v1(i1 ,i2 ,i3 )
        v3_old          =v3(i1 ,i2 ,i3 )
!        ei_old          =ei(i1 ,i2 ,i3 )
        v1( i1 ,i2 ,i3 )=v1(i1 ,i2 ,i3 ) -   dtimeoverrhodx * flux_rhov1
        v3( i1 ,i2 ,i3 )=v3(i1 ,i2 ,i3 ) -   dtimeoverrhodx * flux_rhov3
        ei( i1 ,i2 ,i3 )=ei(i1 ,i2 ,i3 ) - ( dtimeoverrhodx * flux_rhoei - &
                               0.5 * ( (v1_old**2 - v1(i1 ,i2 ,i3 )**2) + &
                                       (v3_old**2 - v3(i1 ,i2 ,i3 )**2)  &
                                     ) &
                                           )
        dev(i1 ,i2 ,i3 )=dev(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhoei
        !
!!$if ((i1 == 1) .and. (i3 == 25)) then
!!$  write(6,'(A,5E13.5)'    ) 'dx1, dx3, dx, dx0_update, dxd_update:', &
!!$                             dx1, dx3, dx, dx0_update, dxd_update
!!$  write(6,'(A,5E13.5)'    ) 'dv1, dv3, dvdx, dv, sv:              ', dv1, dv3, dvdx, dv, sv
!!$  write(6,'(A,2E13.5)'    ) 'nu_total, rhob:                      ', nu_total, rhob
!!$  write(6,'(A,4E13.5)'    ) 'flux_rhov, flux_rhov1,3, flux_rhoei: ', &
!!$                             flux_rhov, flux_rhov1, flux_rhov3, flux_rhoei
!!$  write(6,'(A,1E13.5)'    ) 'dtimeoverrhodx:                         ', dtimeoverrhodx
!!$  write(6,'(A,5E13.5)'    ) 'model%v1, v1_old, v1, v3_old, v3:    ', &
!!$                             model%v1(i1,i2,i3), v1_old, v1(i1,i2,i3), v3_old, v3(i1,i2,i3)
!!$  write(6,'(A,3E13.5)'    ) 'model%ei, ei_old, ei:                ', &
!!$                             model%ei(i1,i2,i3), ei_old, ei(i1,i2,i3)
!!$endif
        !
        ! --- Update: target ---
        dtimeoverrhodx  =dtime/(model%rho(i1d,i2d,i3d)*dxd_update)
        v1_old          =v1(i1d,i2d,i3d)
        v3_old          =v3(i1d,i2d,i3d)
!        ei_old          =ei(i1d,i2d,i3d)
        v1( i1d,i2d,i3d)=v1(i1d,i2d,i3d) +   dtimeoverrhodx * flux_rhov1
        v3( i1d,i2d,i3d)=v3(i1d,i2d,i3d) +   dtimeoverrhodx * flux_rhov3
        ei( i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dtimeoverrhodx * flux_rhoei + &
                               0.5 * ( (v1_old**2 - v1(i1d,i2d,i3d)**2) + &
                                       (v3_old**2 - v3(i1d,i2d,i3d)**2)  &
                                     ) &
                                           )
        dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhoei
        !
        ! --- Maximum time step ---
        !inv_dtime_limit=max(inv_dtime_limit, nu_total/dx_sqr)
        inv_dtime_limit=max(inv_dtime_limit, &
                            (nu_total*rhob)/(model%rho(i1 ,i2 ,i3 )*dx0_update*dx), &
                            (nu_total*rhob)/(model%rho(i1d,i2d,i3d)*dxd_update*dx))
        !
!!$if ((i1 == 1) .and. (i3 == 25)) then
!!$  write(6,'(A,1E13.5)'    ) 'dtimeoverrhodx:                         ', dtimeoverrhodx
!!$  write(6,'(A,5E13.5)'    ) 'model%v1, v1_old, v1, v3_old, v3:    ', &
!!$                             model%v1(i1d,i2d,i3d), v1_old, v1(i1d,i2d,i3d), v3_old, v3(i1d,i2d,i3d)
!!$  write(6,'(A,3E13.5)'    ) 'model%ei, ei_old, ei:                ', &
!!$                             model%ei(i1d,i2d,i3d), ei_old, ei(i1d,i2d,i3d)
!!$endif
      end do ! i1
    end do ! i3
  else
    stop 'Impossible case in rhd_visP2P'
  endif
end do ! idir
!
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VISp2p2D: dvdx,nu,fluxes,update')
!#
# endif
!#
!
! --- Compute minimum energy change relative to reference ---
call tabinter_getinfo(x2min=eiTable_min)
einew_min=minval((ei(m1:n1,m2:n2,m3:n3)-model%ei(m1:n1,m2:n2,m3:n3))/ &
                 (max(model%ei(m1:n1,m2:n2,m3:n3)-eiTable_min, 0.0) + &
                      model%v1(m1:n1,m2:n2,m3:n3)**2 + &
                      model%v3(m1:n1,m2:n2,m3:n3)**2    ))
!
!!$call uio_openwr(nc, 'rhd_visp2p_test.dat', form='unformatted', conv='ieee_4', &
!!$                position='append')
!!$call uio_wrlabl(nc, dataset_ident, date='now' )
!!$call uio_wrlabl(nc, box_ident                 )
!!$call uio_wr(nc, 'z'          , box_id_ident   )
!!$call uio_wr(nc, model%time   , 'time'         )
!!$call uio_wr(nc, dtime        , 'dtime'        )
!!$call uio_wr(nc, inv_dtime_limit, 'inv_dtime_limit')
!!$call uio_wr(nc, eiTable_min  , 'eiTable_min'  )
!!$call uio_wr(nc, einew_min    , 'einew_min'    )
!!$call uio_wr(nc, spacing(0.0) , 'spacing0'     )
!!$call uio_wr(nc, spacing(1.0) , 'spacing1'     )
!!$call uio_wr(nc, nghost       , 'nghost'       )
!!$call uio_wr(nc, model%xc1    , 'xc1'          )
!!$call uio_wr(nc, model%xb1    , 'xb1'          )
!!$call uio_wr(nc, model%xc2    , 'xc2'          )
!!$call uio_wr(nc, model%xb2    , 'xb2'          )
!!$call uio_wr(nc, model%xc3    , 'xc3'          )
!!$call uio_wr(nc, model%xb3    , 'xb3'          )
!!$call uio_wr(nc, model%v1     , 'v1'           )
!!$call uio_wr(nc, model%v3     , 'v3'           )
!!$call uio_wr(nc, model%ei     , 'ei'           )
!!$call uio_wr(nc, model%rho    , 'rho'          )
!!$call uio_wr(nc, dv1dx1       , 'dv1dx1'       )
!!$call uio_wr(nc, dv3dx1       , 'dv3dx1'       )
!!$call uio_wr(nc, dv1dx3       , 'dv1dx3'       )
!!$call uio_wr(nc, dv3dx3       , 'dv3dx3'       )
!!$call uio_wr(nc, nuVis        , 'nuvis'        )
!!$call uio_wr(nc, v1-model%v1  , 'dv1'          )
!!$call uio_wr(nc, v3-model%v3  , 'dv3'          )
!!$call uio_wr(nc, ei-model%ei  , 'dei'          )
!!$call uio_wrlabl(nc, endbox_ident)
!!$call uio_wrlabl(nc, enddataset_ident)
!!$call uio_closrd(nc)
!
! \label{f90:c_viscourant__use5}\label{f90:c_viscourantmax__use3}
! --- Check for time step limit violation ---
if (inv_dtime_limit > 100.0*spacing(0.0)) then
  dtime_limit=par%C_visCourant/inv_dtime_limit
  if (dtime > (par%C_visCourantMax/par%C_visCourant*dtime_limit)) then
    ierr0=1
    outstr0='Time step exceeds viscous time scale'
    goto 999
  endif
endif
!
!!$einew_min=minval(ei(m1:n1,m2:n2,m3:n3))
!!$call tabinter_getinfo(x2min=eiTable_min)
!!$if (einew_min < eiTable_min) then
!!$  ierr0=7
!!$  dtime_limit=min(dtime_limit, 0.7*dtime)
!!$  outstr0='Internal energy below minimum table value'
!!$  goto 999
!!$endif
!
!einew_min=minval(ei(m1:n1,m2:n2,m3:n3)-model%ei(m1:n1,m2:n2,m3:n3))
!if (einew_min < (-spacing(0.0))) then
if (einew_min < (-100.0*spacing(1.0))) then
  ierr0=8
  dtime_limit=min(dtime_limit, 0.7*dtime)
  write(error_locstr,'(1PE14.5)') einew_min
  outstr0=trim(error_locstr)
  write(error_locstr,'(1PE14.5)') eiTable_min
  outstr0=trim(outstr0) // ' / ' // trim(error_locstr)
  outstr0='Change in internal energy below zero: ' // outstr0
  goto 999
endif
!
! --- Output of additional data ---
if ((Output_flag0) .and. present(dat)) then
!#
# ifdef rhd_vis_t01
!#
  call timing_start( 'VISp2p2D: (output)')
!#
# endif
!#
  call rhd_vis_p2pStepOutput(model, &
                             AreaFraction10, 1.0, AreaFraction30, &
                             model%rho(m1:n1,m2:n2,m3:n3)*dev(m1:n1,m2:n2,m3:n3)/dtime, &
                             dat)
!#
# ifdef rhd_vis_t01
!#
  call timing_stop( 'VISp2p2D: (output)')
!#
# endif
!#
endif
!
! --- Take updated quantities ---
model%v1=v1
model%v3=v3
model%ei=ei
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VISp2p2D:')
!#
# endif
!#
!
end subroutine rhd_vis_p2p2dStep


!----------*****************------------------------------------------------------------------------
subroutine rhd_vis_p2p3dStep(model, action, dtime, dtime_limit, nghost, Output_flag, &
                             AreaFraction1, AreaFraction2, AreaFraction3, &
                             dat, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_p2p3dStep ('rhd_viscosity_point_2_point_3_dimension_Step')
!
! PURPOSE:
!   Update model by applying 3D point-to-point (tensor) viscosity.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_p2p3dStep(model, action, dtime, dtime_limit, nghost, Output_flag=Output_flag, &
!                          AreaFraction1=AreaFraction1, AreaFraction2=AreaFraction2, &
!                          AreaFraction3=AreaFraction3, &
!                          dat=dat, outstr=outstr,ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   nghost:       ('number_ghost_cells') integer with number of ghost cells
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                   default=.false.
!   AreaFraction1: ('Area_Fraction_1') real, optional, weighting factor for chunk in x23 plane
!   AreaFraction2: ('Area_Fraction_2') real, optional, weighting factor for chunk in x13 plane
!   AreaFraction3: ('Area_Fraction_3') real, optional, weighting factor for chunk in x12 plane
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, proposed maximum time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                    =0: normal run
!                  1-99: "small" error (e.g., time step limit violation)
!                 >=100: "severe" error (does not occur in this version)
!
! VARIABLES:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   m1l:          ('m_1_limit') integer, lower loop index limit, 1st dimension
!   n1l:          ('n_1_limit') integer, upper loop index limit, 1st dimension
!   i1:           ('index_1') loop index (source cell), 1st dimension
!   di1:          ('delta_index_1') integer, relative index shift of target cell, 1st dimension
!   i1d:          ('index_1_delta') integer, index (target cell), 1st dimension
!   idir:         ('index_direction') integer, index for loop over directions
!   ndir:         ('number_direction') integer, number of directions
!   inv_dtime_limit: ('inverse_delta_time_limit') real, accumulated inverse limit for next time step
!   einew_min:    ('energy_internal_new_minimum') real, total minimum of internal energy after
!                 viscous energy update
!   eiTable_min:  ('energy_internal_Table_minimum') real, minimum energy allowed by EOS table
!   nu_total:     ('nu_total') real, kinematic viscosity
!   rhob:         ('rho_boundary') real, "mean" density at boundary between two cells
!   visP2P:       ('viscosity_Point_to_Point') real, total viscosity factor
!   dx1:          ('delta_x_1') real, distance of cell centers, 1st dimension
!   dx_sqr:       ('delta_x_square') real, square of distance of cell centers
!   dx:           ('delta_x') real, distance of cell centers
!   vol0_scale:   ('volume_0_scale') real, scaled (i.e. not in cm^3) volume of source cell
!   vold_scale:   ('volume_delta_scale') real, scaled (i.e. not in cm^3) volume of target cell
!   area_scale:   ('area_scale') real, scaled area through which the flux goes
!   dx0_update:   ('delta_x_0_update') real, geometrical factor (volume/area)
!                 for update of source cell
!   dxd_update:   ('delta_x_delta_update') real, geometrical factor (volume/area)
!                 for update of target cell
!   dv1:          ('delta_velocity_1') real, 1st component of velocity difference
!                 along line of sight
!   dv:           ('delta_velocity') real, velocity difference along line of sight
!   dvdx:         ('delta_velocity_delta_x') real, velocity gradient along line of sight
!   sv1:          ('sum_velocity_1') real, 1st component of half velocity sum
!                 along line of sight
!   sv:           ('sum_velocity') real, half velocity sum along line of sight
!   v1_old:       ('velocity_1_old') real, velocity before update, 1st component
!   ei_old:       ('energy_internal_old') real, internal energy before update
!   dtimeoverrhodx: ('delta_time_over_rho_dx') real, geometrical update factor (flux -> change)
!   flux_rhov:    ('flux_rho_velocity') real, viscous momentum flux along line of sight
!   flux_rhov1:   ('flux_rho_velocity_1') real, momentum flux along line of sight, 1st component
!   flux_rhoei:   ('flux_rho_energy_internal') real, viscous energy flux along line of sight
!   dxb1_scale:   ('delta_x_boundary_1_scale') real vector, scales grid size, 1st dimension
!   v1:           ('velocity_1') real 3D array, 1st component of velocities
!   ei:           ('energy_internal') real 3D array, internal energy
!   dev:          ('delta_energy_viscous') real 3D array, divergence of viscous energy flux
!   di1arr:       ('delta_index_1_array') integer vector, relative index of target cell, 1st dim.
!   TenCoeffarr:  ('Tensor_coefficient_array') real array, with weights for each direction
!
! ROUTINES:
!   rhd_vis_p2pStepOutput: Put some additional output quantities into dat structure
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   tabinter_module:       Routines for handling of interpolation coefficients
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   There should be exactly one layer of (already filled) ghost cells in x1, x2, and x3 direction.
!
! PROCEDURE:
!   The velocity of each cell relative to all immediate neighbors is computed.
!   This relative velocity is used to compute the viscous momentum exchange
!     between each pair of cells ('source' and 'target') -- only along each line of sight.
!
!           target +->  -
!                 /|    |
!                v |    |
!                  v    |
!              ^        | dx3
!          ^  /         |
!          | /          |
!          |/           |
!   source +------>     -
!
!          |-------|
!             dx1
!
!   The viscous energy flux is computed consistently.
!   This should guarantee angular momentum conservation
!     AND the positivity of the viscous energy dissipation.
!
! EXAMPLE:
!
! TODO:
!   spherical mean
!   mean based on fluxes
!   (faster) version with constant geometrical coefficients
!   Make separate version for equidistant grid.
!   Check geometry factors for non-equidistant grid.
!
! MODIFICATION HISTORY:
!   1999-06-01 (B.F. Copenhagen) First Version
!   2004-05-11 (B.F.Montpellier) First draft: point-to-point viscosity
!   2004-05-13 (B.F.) Modifications
!   2004-05-14 (B.F.) More comments
!                   3D version
!   2004-05-17 (B.F.) New formula for time step limit; order of time step checks altered
!   2004-05-18 (B.F.) New brackets in update of internal energy
!   2011-11-28 (B.F.) Improved vectorization: more branches, max time step in one go,
!                                             IVDEP directive
!   2012-01-19 (B.F.) New parameter nghost; Smagorinsky, artificial viscosity
!   2012-01-20 (B.F.) Check for negative internal energy changes relative to reference energy
!   2012-04-11 (B.F.) Bugfix: index corrected in computation of vis(Divrhov)
!   2012-07-27 (B.F.) Move scale%length to prop%length
!---------------------------------------------------------------------------------------------------
use tabinter_module
use rhd_gl_module
use rhd_prop_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_vis_t01
!#
use timing_module
!#
# endif
!#
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
integer,                   intent(in)   :: nghost
logical,         optional, intent(in)   :: Output_flag
real,            optional, intent(in)   :: AreaFraction1, AreaFraction2, AreaFraction3
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
logical                                 :: Output_flag0
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           m1l, n1l, m2l, n2l, m3l, n3l, &
                                           i1, i2, i3, &
                                           di1, di2, di3, i1d, i2d, i3d, &
                                           idir, ndir, &
                                           di1SA, di1SB, di1SC, di1SD, &
                                           di2SA, di2SB, di2SC, di2SD, &
                                           di3SA, di3SB, di3SC, di3SD
real                                    :: AreaFraction10, AreaFraction20, AreaFraction30, &
                                           inv_dtime_limit, einew_min, eiTable_min, &
                                           nu_total, rhob, visP2P, &
                                           dx1, dx2, dx3, oneodx1, oneodx2, oneodx3, &
                                           dx_sqr, dx, vol0_scale, vold_scale, area_scale, &
                                           dx0_update, dxd_update, &
                                           dv1, dv2, dv3, dv, dvdx, divv, &
                                           sv1, sv2, sv3, sv, &
                                           v1_old, v2_old, v3_old, &
                                           dtimeoverrhodx, &
                                           flux_rhov, flux_rhoei, &
                                           flux_rhov1,flux_rhov2,flux_rhov3, &
                                           visSmagorinsky, visArtificial, visExpansion, visDivrhov
!real                                    :: ei_old
real, dimension(model%m1-1:model%n1+1)  :: dxb1_scale, dv1v, dei1v
real, dimension(model%m2-1:model%n2+1)  :: dxb2_scale
real, dimension(model%m3-1:model%n3+1)  :: dxb3_scale
real, dimension(model%m1-nghost:model%n1+nghost, &
                model%m2-nghost:model%n2+nghost, &
                model%m3-nghost:model%n3+nghost)  :: v1, v2, v3, ei, dev
real, dimension(model%m1  :model%n1+1, &
                model%m2  :model%n2+1, &
                model%m3  :model%n3+1)  :: nuVis
real, dimension(model%m1  :model%n1+1, &
                model%m2-1:model%n2+1, &
                model%m3-1:model%n3+1)  :: dv1dx1, dv2dx1, dv3dx1
real, dimension(model%m1-1:model%n1+1, &
                model%m2  :model%n2+1, &
                model%m3-1:model%n3+1)  :: dv1dx2, dv2dx2, dv3dx2
real, dimension(model%m1-1:model%n1+1, &
                model%m2-1:model%n2+1, &
                model%m3  :model%n3+1)  :: dv1dx3, dv2dx3, dv3dx3
integer                                 :: ierr0
character(len=80)                       :: outstr0, error_locstr
integer, parameter, dimension(13)       :: di1arr  =(/ 1, 0, 0,  1, 1, 0, 0, 1, 1,  1, 1, 1, 1 /), &
                                           di2arr  =(/ 0, 1, 0,  1,-1, 1, 1, 0, 0,  1, 1,-1,-1 /), &
                                           di3arr  =(/ 0, 0, 1,  0, 0, 1,-1, 1,-1,  1,-1, 1,-1 /), &
                                           di1SAarr=(/ 1, 0, 0,  1, 1, 0, 0, 1, 1,  1, 1, 1, 1 /), &
                                           di1SBarr=(/ 1, 1, 1,  1, 1, 0, 0, 1, 1,  1, 1, 1, 1 /), &
                                           di1SCarr=(/ 1, 0, 0,  1, 1, 1, 1, 1, 1,  1, 1, 1, 1 /), &
                                           di1SDarr=(/ 1, 1, 1,  1, 1, 1, 1, 1, 1,  1, 1, 1, 1 /), &
                                           di2SAarr=(/ 0, 1, 0,  1, 0, 1, 1, 0, 0,  1, 1, 0, 0 /), &
                                           di2SBarr=(/ 1, 1, 0,  1, 0, 1, 1, 0, 0,  1, 1, 0, 0 /), &
                                           di2SCarr=(/ 0, 1, 1,  1, 0, 1, 1, 1, 1,  1, 1, 0, 0 /), &
                                           di2SDarr=(/ 1, 1, 1,  1, 0, 1, 1, 1, 1,  1, 1, 0, 0 /), &
                                           di3SAarr=(/ 0, 0, 1,  0, 0, 1, 0, 1, 0,  1, 0, 1, 0 /), &
                                           di3SBarr=(/ 0, 0, 1,  0, 0, 1, 0, 1, 0,  1, 0, 1, 0 /), &
                                           di3SCarr=(/ 1, 1, 1,  1, 1, 1, 0, 1, 0,  1, 0, 1, 0 /), &
                                           di3SDarr=(/ 1, 1, 1,  1, 1, 1, 0, 1, 0,  1, 0, 1, 0 /)
real,               dimension(13)       :: TenCoeffarr
!---------------------------------------------------------------------------------------------------
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VISp2p3D:')
!#
# endif
!#
! --- No error occurred ---
ierr0=0
outstr0=''
!
! --- Set default values for keywords ---
if (.not. present(Output_flag)) then
  Output_flag0=.false.
else
  Output_flag0=Output_flag
endif
if (.not. present(AreaFraction1)) then
  AreaFraction10=1.0
else
  AreaFraction10=AreaFraction1
endif
if (.not. present(AreaFraction2)) then
  AreaFraction20=1.0
else
  AreaFraction20=AreaFraction2
endif
if (.not. present(AreaFraction3)) then
  AreaFraction30=1.0
else
  AreaFraction30=AreaFraction3
endif
!
! --- Give initially no limit on time step ---
dtime_limit    =0.1*huge(dtime_limit)
inv_dtime_limit=0.0
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Compute grid size ---
do i1=m1-1,n1+1
  dxb1_scale=(model%xb1(i1+1)-model%xb1(i1))/prop%length
end do ! i1
do i2=m2-1,n2+1
  dxb2_scale=(model%xb2(i2+1)-model%xb2(i2))/prop%length
end do ! i3
do i3=m3-1,n3+1
  dxb3_scale=(model%xb3(i3+1)-model%xb3(i3))/prop%length
end do ! i3
!dxb1_scale=(model%xb1(m1  :n1+2) - model%xb1(m1-1:n1+1))/prop%length
!dxb2_scale=(model%xb2(m2  :n2+2) - model%xb2(m2-1:n2+1))/prop%length
!dxb3_scale=(model%xb3(m3  :n3+2) - model%xb3(m3-1:n3+1))/prop%length
!!$write(6,'(A,2E13.5)') 'minmax(dxb1_scale): ', minval(dxb1_scale), maxval(dxb1_scale)
!!$write(6,'(A,2E13.5)') 'minmax(dxb2_scale): ', minval(dxb2_scale), maxval(dxb2_scale)
!!$write(6,'(A,2E13.5)') 'minmax(dxb3_scale): ', minval(dxb3_scale), maxval(dxb3_scale)
!
! \label{f90:c_visp2pincl1__use3}\label{f90:c_visp2pincl2__use1}
! --- Define tensor coefficient array ---
TenCoeffarr=(/ 1.0, 1.0, 1.0, &
               par%C_visP2Pincl1, par%C_visP2Pincl1, par%C_visP2Pincl1, &
               par%C_visP2Pincl1, par%C_visP2Pincl1, par%C_visP2Pincl1, &
               par%C_visP2Pincl2, par%C_visP2Pincl2, par%C_visP2Pincl2, &
               par%C_visP2Pincl2 /)
!
!#
# ifdef rhd_vis_t01
!#
call timing_start('VISp2p3D: dvdx,nu,fluxes,update')
!#
# endif
!#
!
! \label{f90:c_visp2phypartificial__use2}
! --- Set viscosity coefficients ---
visSmagorinsky=max(par%C_visP2PhypSmagorinsky,0.0)**2
visArtificial =max(par%C_visP2PhypArtificial ,0.0)**2
visExpansion  =max(par%C_visP2PhypExpansion  ,0.0)**2
visDivrhov    =max(par%C_visP2Pdivrhov       ,0.0)**2
!write(6,'(A,3(1PE13.5),I4)') 'VisP2P: visSma,Art,Exp;nghost:', &
!  visSmagorinsky, visArtificial, visExpansion, nghost
!
! --- Save velocities and internal energy ---
v1 =model%v1
v2 =model%v2
v3 =model%v3
ei =model%ei
dev=0.0
!
if ((visSmagorinsky > 0.0) .or. (visArtificial > 0.0) .or. (visExpansion > 0.0)) then
  ! --- Compute differences (corresponds to constant reconstruction) ---
  !
  do i3=m3-1,n3+1
   do i2=m2-1,n2+1
    do i1=m1,n1+1
      ! --- Compute geometrical data ---
      oneodx1=1.0/(model%xc1(i1)-model%xc1(i1-1))
      !
      ! --- Take discrete derivative in x1 direction ---
      dv1dx1(i1,i2,i3)=(v1(i1  ,i2  ,i3  )-v1(i1-1,i2  ,i3  ))*oneodx1
      dv2dx1(i1,i2,i3)=(v2(i1  ,i2  ,i3  )-v2(i1-1,i2  ,i3  ))*oneodx1
      dv3dx1(i1,i2,i3)=(v3(i1  ,i2  ,i3  )-v3(i1-1,i2  ,i3  ))*oneodx1
    end do ! i1
   end do ! i2
  end do ! i3
  !
  do i3=m3-1,n3+1
   do i2=m2,n2+1
    do i1=m1-1,n1+1
      ! --- Compute geometrical data ---
      oneodx2=1.0/(model%xc2(i2)-model%xc2(i2-1))
      !
      ! --- Take discrete derivative in x1 direction ---
      dv1dx2(i1,i2,i3)=(v1(i1  ,i2  ,i3  )-v1(i1  ,i2-1,i3  ))*oneodx2
      dv2dx2(i1,i2,i3)=(v2(i1  ,i2  ,i3  )-v2(i1  ,i2-1,i3  ))*oneodx2
      dv3dx2(i1,i2,i3)=(v3(i1  ,i2  ,i3  )-v3(i1  ,i2-1,i3  ))*oneodx2
    end do ! i1
   end do ! i2
  end do ! i3
  !
  do i3=m3,n3+1
   do i2=m2-1,n2+1
    do i1=m1-1,n1+1
      ! --- Compute geometrical data ---
      oneodx3=1.0/(model%xc3(i3)-model%xc3(i3-1))
      !
      ! --- Take discrete derivative in x3 direction ---
      dv1dx3(i1,i2,i3)=(v1(i1  ,i2  ,i3  )-v1(i1  ,i2  ,i3-1))*oneodx3
      dv2dx3(i1,i2,i3)=(v2(i1  ,i2  ,i3  )-v2(i1  ,i2  ,i3-1))*oneodx3
      dv3dx3(i1,i2,i3)=(v3(i1  ,i2  ,i3  )-v3(i1  ,i2  ,i3-1))*oneodx3
    end do ! i1
   end do ! i2
  end do ! i3
  !
  if (visSmagorinsky > 0.0) then
    !
    do i3=m3,n3+1
     do i2=m2,n2+1
      do i1=m1,n1+1
        ! --- Compute Smagorinsky (square of) magnitude of strain-rate tensor at cell edges ---
        ! --- Factor 1/4 ---
        nuVis(i1,i2,i3)=(0.25*visSmagorinsky) * &
                      sqrt( &
                        0.5 *( dv1dx1(i1  ,i2-1,i3-1)**2 + &
                               dv1dx1(i1  ,i2  ,i3-1)**2 + &
                               dv1dx1(i1  ,i2-1,i3  )**2 + &
                               dv1dx1(i1  ,i2  ,i3  )**2   &
                                 + &
                               dv2dx2(i1-1,i2  ,i3-1)**2 + &
                               dv2dx2(i1  ,i2  ,i3-1)**2 + &
                               dv2dx2(i1-1,i2  ,i3  )**2 + &
                               dv2dx2(i1  ,i2  ,i3  )**2   &
                                 + &
                               dv3dx3(i1-1,i2-1,i3  )**2 + &
                               dv3dx3(i1  ,i2-1,i3  )**2 + &
                               dv3dx3(i1-1,i2  ,i3  )**2 + &
                               dv3dx3(i1  ,i2  ,i3  )**2    ) + &
                       0.125*((dv1dx2(i1-1,i2  ,i3-1) + dv2dx1(i1  ,i2-1,i3-1))**2 + &
                              (dv1dx2(i1  ,i2  ,i3-1) + dv2dx1(i1  ,i2-1,i3-1))**2 + &
                              (dv1dx2(i1-1,i2  ,i3-1) + dv2dx1(i1  ,i2  ,i3-1))**2 + &
                              (dv1dx2(i1  ,i2  ,i3-1) + dv2dx1(i1  ,i2  ,i3-1))**2 + &
                              (dv1dx2(i1-1,i2  ,i3  ) + dv2dx1(i1  ,i2-1,i3  ))**2 + &
                              (dv1dx2(i1  ,i2  ,i3  ) + dv2dx1(i1  ,i2-1,i3  ))**2 + &
                              (dv1dx2(i1-1,i2  ,i3  ) + dv2dx1(i1  ,i2  ,i3  ))**2 + &
                              (dv1dx2(i1  ,i2  ,i3  ) + dv2dx1(i1  ,i2  ,i3  ))**2  &
                                 + &
                              (dv1dx3(i1-1,i2-1,i3  ) + dv3dx1(i1  ,i2-1,i3-1))**2 + &
                              (dv1dx3(i1  ,i2-1,i3  ) + dv3dx1(i1  ,i2-1,i3-1))**2 + &
                              (dv1dx3(i1-1,i2-1,i3  ) + dv3dx1(i1  ,i2-1,i3-1))**2 + &
                              (dv1dx3(i1  ,i2-1,i3  ) + dv3dx1(i1  ,i2-1,i3-1))**2 + &
                              (dv1dx3(i1-1,i2  ,i3  ) + dv3dx1(i1  ,i2  ,i3  ))**2 + &
                              (dv1dx3(i1  ,i2  ,i3  ) + dv3dx1(i1  ,i2  ,i3  ))**2 + &
                              (dv1dx3(i1-1,i2  ,i3  ) + dv3dx1(i1  ,i2  ,i3  ))**2 + &
                              (dv1dx3(i1  ,i2  ,i3  ) + dv3dx1(i1  ,i2  ,i3  ))**2  &
                                 + &
                              (dv2dx3(i1-1,i2-1,i3  ) + dv3dx2(i1-1,i2  ,i3-1))**2 + &
                              (dv2dx3(i1-1,i2  ,i3  ) + dv3dx2(i1-1,i2  ,i3-1))**2 + &
                              (dv2dx3(i1-1,i2-1,i3  ) + dv3dx2(i1-1,i2  ,i3  ))**2 + &
                              (dv2dx3(i1-1,i2  ,i3  ) + dv3dx2(i1-1,i2  ,i3  ))**2 + &
                              (dv2dx3(i1  ,i2-1,i3  ) + dv3dx2(i1  ,i2  ,i3-1))**2 + &
                              (dv2dx3(i1  ,i2  ,i3  ) + dv3dx2(i1  ,i2  ,i3-1))**2 + &
                              (dv2dx3(i1  ,i2-1,i3  ) + dv3dx2(i1  ,i2  ,i3  ))**2 + &
                              (dv2dx3(i1  ,i2  ,i3  ) + dv3dx2(i1  ,i2  ,i3  ))**2    ))
      end do ! i1
     end do ! i2
    end do ! i3
    !
  else
    nuVis=0.0
  endif
  !
  !write(6,*) 'nuVis :', minval(nuVis), maxval(nuVis)
  !write(6,*) 'dv1dx1:', minval(dv1dx1), maxval(dv1dx1)
  !write(6,*) 'dv3dx3:', minval(dv3dx3), maxval(dv3dx3)
  !
  if ((visArtificial > 0.0) .or. (visExpansion > 0.0)) then
    do i3=m3,n3+1
     do i2=m2,n2+1
      do i1=m1,n1+1
        ! --- Compute divergence at cell edges (factor 1/4) ---
        divv=0.25 * (dv1dx1(i1  ,i2-1,i3-1) + &
                     dv1dx1(i1  ,i2-1,i3  ) + &
                     dv1dx1(i1  ,i2  ,i3-1) + &
                     dv1dx1(i1  ,i2  ,i3  )   &
                       + &
                     dv2dx2(i1-1,i2  ,i3-1) + &
                     dv2dx2(i1  ,i2  ,i3-1) + &
                     dv2dx2(i1-1,i2  ,i3  ) + &
                     dv2dx2(i1  ,i2  ,i3  )   &
                       + &
                     dv3dx3(i1-1,i2-1,i3  ) + &
                     dv3dx3(i1  ,i2-1,i3  ) + &
                     dv3dx3(i1-1,i2  ,i3  ) + &
                     dv3dx3(i1  ,i2  ,i3  ))
        !
        ! --- Add contribution to viscosity ---
        nuVis(i1,i2,i3)=nuVis(i1,i2,i3) + visArtificial * max(-divv, 0.0) + &
                                          visExpansion  * max( divv, 0.0)
  !if (abs(divv) > 10.0) write(6,'(3I4,6(1PE12.4))') i1,i2,i3, &
  !     dv1dx1(i1  ,i2,i3-1), dv1dx1(i1  ,i2,i3  ), &
  !     dv3dx3(i1-1,i2,i3  ), dv3dx3(i1  ,i2,i3  ), divv, nuVis(i1,i2,i3)
      end do ! i1
     end do ! i2
    end do ! i3
  endif
else
  nuVis=0.0
endif
!
if (visDivrhov > 0.0) then
  ! --- Just compute differences (corresponds to constant reconstruction) ---
  ! --- Put drhovdx into dvdx arrays ---
  !
  do i3=m3-1,n3+1
   do i2=m2-1,n2+1
    do i1=m1,n1+1
      ! --- Compute geometrical data ---
      oneodx1=1.0/(model%xc1(i1)-model%xc1(i1-1))
      !
      ! --- Take discrete derivative in x1 direction ---
      dv1dx1(i1,i2,i3)=(model%rho(i1  ,i2  ,i3  )*v1(i1  ,i2,i3  )- &
                        model%rho(i1-1,i2  ,i3  )*v1(i1-1,i2,i3  ) ) * oneodx1
    end do ! i1
   end do ! i2
  end do ! i3
  !
  do i3=m3-1,n3+1
   do i2=m2,n2+1
    do i1=m1-1,n1+1
      ! --- Compute geometrical data ---
      oneodx2=1.0/(model%xc2(i2)-model%xc2(i2-1))
      !
      ! --- Take discrete derivative in x3 direction ---
      dv2dx2(i1,i2,i3)=(model%rho(i1  ,i2  ,i3  )*v2(i1  ,i2  ,i3  )- &
                        model%rho(i1  ,i2  ,i3  )*v2(i1  ,i2-1,i3  ) ) * oneodx2
    end do ! i1
   end do ! i2
  end do ! i3
  !
  !
  do i3=m3,n3+1
   do i2=m2-1,n2+1
    do i1=m1-1,n1+1
      ! --- Compute geometrical data ---
      oneodx3=1.0/(model%xc3(i3)-model%xc3(i3-1))
      !
      ! --- Take discrete derivative in x3 direction ---
      dv3dx3(i1,i2,i3)=(model%rho(i1  ,i2  ,i3  )*v3(i1  ,i2  ,i3  )- &
                        model%rho(i1  ,i2  ,i3-1)*v3(i1  ,i2  ,i3-1) ) * oneodx3
    end do ! i1
   end do ! i2
  end do ! i3
  !
  do i3=m3,n3+1
   do i2=m2,n2+1
    do i1=m1,n1+1
      ! --- Compute divergence at cell edges (factor 1/4) ---
      divv=0.5 * (dv1dx1(i1  ,i2-1,i3-1) + &
                  dv1dx1(i1  ,i2-1,i3  ) + &
                  dv1dx1(i1  ,i2  ,i3-1) + &
                  dv1dx1(i1  ,i2  ,i3  )   &
                    + &
                  dv2dx2(i1-1,i2  ,i3-1) + &
                  dv2dx2(i1  ,i2  ,i3-1) + &
                  dv2dx2(i1-1,i2  ,i3  ) + &
                  dv2dx2(i1  ,i2  ,i3  )   &
                    + &
                  dv3dx3(i1-1,i2-1,i3  ) + &
                  dv3dx3(i1  ,i2-1,i3  ) + &
                  dv3dx3(i1-1,i2  ,i3  ) + &
                  dv3dx3(i1  ,i2  ,i3  )    ) / &
              (model%rho(i1-1,i2-1,i3-1) + &
               model%rho(i1  ,i2-1,i3-1) + &
               model%rho(i1-1,i2  ,i3-1) + &
               model%rho(i1  ,i2  ,i3-1) + &
               model%rho(i1-1,i2-1,i3  ) + &
               model%rho(i1  ,i2-1,i3  ) + &
               model%rho(i1-1,i2  ,i3  ) + &
               model%rho(i1  ,i2  ,i3  )    )
      !
      ! --- Add contribution to viscosity ---
!      nuVis(i1,i2,i3)=nuVis(i1,i2,i3) + min(visDivrhov * abs(divv), 1.0E-01)
      nuVis(i1,i2,i3)=nuVis(i1,i2,i3) + visDivrhov * abs(divv)
    end do ! i1
   end do ! i2
  end do ! i3
  !
endif
!
! \label{f90:c_visp2pincl1__use4}\label{f90:c_visp2pincl2__use2}
! --- Update for all (3 or 9) directions ---
if (par%C_visP2Pincl1 <= 0.0) then
  ndir=3
else if (par%C_visP2Pincl2 <= 0.0) then
  ndir=9
else
  ndir=13
endif
do idir=1, ndir
  !
  ! --- Set current direction ---
  di1=di1arr(idir)
  di2=di2arr(idir)
  di3=di3arr(idir)
  ! \label{f90:c_visp2pcoeff__use2}
  visP2P=max(par%C_visP2Pcoeff, 0.0)**2
  !
  ! --- Compute loop index limits ---
  m1l=m1-max( di1,0)
  n1l=n1+max(-di1,0)
  m2l=m2-max( di2,0)
  n2l=n2+max(-di2,0)
  m3l=m3-max( di3,0)
  n3l=n3+max(-di3,0)
  !
  ! --- Set indices for averages of Smagorinsky magnitude of the strain-rate tensor ---
  di1SA=di1SAarr(idir)
  di1SB=di1SBarr(idir)
  di1SC=di1SCarr(idir)
  di1SD=di1SDarr(idir)
  di2SA=di2SAarr(idir)
  di2SB=di2SBarr(idir)
  di2SC=di2SCarr(idir)
  di2SD=di2SDarr(idir)
  di3SA=di3SAarr(idir)
  di3SB=di3SBarr(idir)
  di3SC=di3SCarr(idir)
  di3SD=di3SDarr(idir)
  !
!!$  write(6,'(A,4I13,E13.5)') 'idir, di1, di2, di3, visP2P:          ', idir, di1, di2, di3, visP2P
  if ((di1 == 1) .and. (di2 == 0) .and. (di3 == 0)) then
    ! --- Special case: only di1 /= 0; simplify and remove vector dependency ---
    !
    do i3=m3l, n3l
     do i2=m2l, n2l
      ! --- Set indices of target cell ---
      i2d=i2
      i3d=i3
      !DIR$ SIMD
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1+di1
        !
        ! --- Geometrical data ---
        dx1       =model%xc1(i1d)-model%xc1(i1)
!        dx2       =model%xc2(i2d)-model%xc2(i2)
!        dx3       =model%xc3(i3d)-model%xc3(i3)
        dx_sqr    =dx1**2
        dx        =dx1
        vol0_scale=dxb1_scale(i1 ) * dxb2_scale(i2 ) * dxb3_scale(i3 )
        vold_scale=dxb1_scale(i1d) * dxb2_scale(i2d) * dxb3_scale(i3d)
        area_scale=0.5*(vol0_scale + vold_scale)/(dx/prop%length)
        dx0_update=vol0_scale*prop%length/area_scale
        dxd_update=vold_scale*prop%length/area_scale
        !
        ! --- Velocity difference along vector connecting points ---
        dv1 =model%v1(i1d,i2d,i3d) - model%v1(i1,i2,i3)
!        dv2 =model%v2(i1d,i2d,i3d) - model%v2(i1,i2,i3)
!        dv3 =model%v3(i1d,i2d,i3d) - model%v3(i1,i2,i3)
        dv  =dv1
        dvdx=dv / dx
        sv1 =0.5*(model%v1(i1d,i2d,i3d) + model%v1(i1,i2,i3))
!        sv2 =0.5*(model%v2(i1d,i2d,i3d) + model%v2(i1,i2,i3))
!        sv3 =0.5*(model%v3(i1d,i2d,i3d) + model%v3(i1,i2,i3))
        sv  =sv1
        !
        ! --- Kinematic viscosity ---
        nu_total=dx_sqr * TenCoeffarr(idir) * &
                 (visP2P * abs(dvdx) + &
                  nuVis(i1+di1SA,i2+di2SA,i3+di3SA) + &
                  nuVis(i1+di1SB,i2+di2SB,i3+di3SB) + &
                  nuVis(i1+di1SC,i2+di2SC,i3+di3SC) + &
                  nuVis(i1+di1SD,i2+di2SD,i3+di3SD)  )
        !
        ! --- Density at boundary ---
!#
#       if (rhd_vis_density_p01 == 0)
!#                                    /* 'min' */
        rhob= min(model%rho(i1d,i2d,i3d),  model%rho(i1,i2,i3))
!#
#       else
!#           /* rhd_vis_density_p01 == 1 '0.5+0.5' */
        rhob=0.5*(model%rho(i1d,i2d,i3d) + model%rho(i1,i2,i3))
!#
#       endif
!#            /* rhd_vis_density_p01 */
        !
        ! --- Momentum fluxes and energy flux ---
        flux_rhov =-rhob * nu_total * dvdx
        flux_rhov1=         flux_rhov
!        flux_rhov2=dx2/dx * flux_rhov
!        flux_rhov3=dx3/dx * flux_rhov
        flux_rhoei=sv     * flux_rhov
        !
        ! --- Update: source ---
        dtimeoverrhodx  =dtime/(model%rho(i1 ,i2 ,i3 )*dx0_update)
        v1_old          =v1(i1 ,i2 ,i3 )
!        v2_old          =v2(i1 ,i2 ,i3 )
!        v3_old          =v3(i1 ,i2 ,i3 )
        v1( i1 ,i2 ,i3 )=v1(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhov1
!        v2( i1 ,i2 ,i3 )=v2(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhov2
!        v3( i1 ,i2 ,i3 )=v3(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhov3
        ei( i1 ,i2 ,i3 )=ei(i1 ,i2 ,i3 ) - ( dtimeoverrhodx * flux_rhoei - &
                               0.5 * ( (v1_old**2 - v1(i1 ,i2 ,i3 )**2) &
                                     ) &
                                           )
        dev(i1 ,i2 ,i3 )=dev(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhoei
        !
        ! --- Compute and store update of target ---
        dtimeoverrhodx  =dtime/(model%rho(i1d,i2d,i3d)*dxd_update)
        !v1_old          =v1(i1d,i2d,i3d)
        dv1v(i1d       )=dtimeoverrhodx * flux_rhov1
        !ei( i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dtimeoverrhodx * flux_rhoei + &
        !                       0.5 * ( (v1_old**2 - v1(i1d,i2d,i3d)**2) &
        !                             ) &
        !                                   )
        dei1v(i1d      )=dtimeoverrhodx * flux_rhoei
        !dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhoei
        !
        ! --- Maximum time step ---
        inv_dtime_limit=max(inv_dtime_limit, &
                            (nu_total*rhob)/(model%rho(i1 ,i2 ,i3 )*dx0_update*dx), &
                            (nu_total*rhob)/(model%rho(i1d,i2d,i3d)*dxd_update*dx))
        !
      end do ! i1
      !
      !DIR$ SIMD
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1+di1
        !
        v1_old         =v1(i1d,i2d,i3d)
        v1(i1d,i2d,i3d)=v1(i1d,i2d,i3d) + dv1v( i1d)
        ei(i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dei1v(i1d) + &
                               0.5 * ( (v1_old**2 - v1(i1d,i2d,i3d)**2) &
                                     ) &
                                          )
        dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dei1v(i1d)
      end do ! i1
     end do ! i2
    end do ! i3
  else
    ! --- General case: at least one of di2 and di3 is not zero ---
    !
    do i3=m3l, n3l
     do i2=m2l, n2l
      !DIR$ SIMD
      do i1=m1l, n1l
        !
        ! --- Set indices of target cell ---
        i1d=i1+di1
        i2d=i2+di2
        i3d=i3+di3
        !
        ! --- Geometrical data ---
        dx1       =model%xc1(i1d)-model%xc1(i1)
        dx2       =model%xc2(i2d)-model%xc2(i2)
        dx3       =model%xc3(i3d)-model%xc3(i3)
        dx_sqr    =dx1**2 + dx2**2 + dx3**2
        dx        =sqrt(dx_sqr)
        vol0_scale=dxb1_scale(i1 ) * dxb2_scale(i2 ) * dxb3_scale(i3 )
        vold_scale=dxb1_scale(i1d) * dxb2_scale(i2d) * dxb3_scale(i3d)
        area_scale=0.5*(vol0_scale + vold_scale)/(dx/prop%length)
        dx0_update=vol0_scale*prop%length/area_scale
        dxd_update=vold_scale*prop%length/area_scale
        !
        ! --- Velocity difference along vector connecting points ---
        dv1 =model%v1(i1d,i2d,i3d) - model%v1(i1,i2,i3)
        dv2 =model%v2(i1d,i2d,i3d) - model%v2(i1,i2,i3)
        dv3 =model%v3(i1d,i2d,i3d) - model%v3(i1,i2,i3)
        dv  =(dv1*dx1 + dv2*dx2 + dv3*dx3) / dx
        dvdx=dv / dx
        sv1 =0.5*(model%v1(i1d,i2d,i3d) + model%v1(i1,i2,i3))
        sv2 =0.5*(model%v2(i1d,i2d,i3d) + model%v2(i1,i2,i3))
        sv3 =0.5*(model%v3(i1d,i2d,i3d) + model%v3(i1,i2,i3))
        sv  =(sv1*dx1 + sv2*dx2 + sv3*dx3) / dx
        !
        ! --- Kinematic viscosity ---
        nu_total=dx_sqr * TenCoeffarr(idir) * &
                 (visP2P * abs(dvdx) + &
                  nuVis(i1+di1SA,i2+di2SA,i3+di3SA) + &
                  nuVis(i1+di1SB,i2+di2SB,i3+di3SB) + &
                  nuVis(i1+di1SC,i2+di2SC,i3+di3SC) + &
                  nuVis(i1+di1SD,i2+di2SD,i3+di3SD)  )
        !
        ! --- Density at boundary ---
!#
#       if (rhd_vis_density_p01 == 0)
!#                                    /* 'min' */
        rhob= min(model%rho(i1d,i2d,i3d),  model%rho(i1,i2,i3))
!#
#       else
!#           /* rhd_vis_density_p01 == 1 '0.5+0.5' */
        rhob=0.5*(model%rho(i1d,i2d,i3d) + model%rho(i1,i2,i3))
!#
#       endif
!#            /* rhd_vis_density_p01 */
        !
        ! --- Momentum fluxes and energy flux ---
        flux_rhov =-rhob * nu_total * dvdx
        flux_rhov1=dx1/dx * flux_rhov
        flux_rhov2=dx2/dx * flux_rhov
        flux_rhov3=dx3/dx * flux_rhov
        flux_rhoei=sv     * flux_rhov
        !
        ! --- Update: source ---
        dtimeoverrhodx  =dtime/(model%rho(i1 ,i2 ,i3 )*dx0_update)
        v1_old          =v1(i1 ,i2 ,i3 )
        v2_old          =v2(i1 ,i2 ,i3 )
        v3_old          =v3(i1 ,i2 ,i3 )
!        ei_old          =ei(i1 ,i2 ,i3 )
        v1( i1 ,i2 ,i3 )=v1(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhov1
        v2( i1 ,i2 ,i3 )=v2(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhov2
        v3( i1 ,i2 ,i3 )=v3(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhov3
        ei( i1 ,i2 ,i3 )=ei(i1 ,i2 ,i3 ) - ( dtimeoverrhodx * flux_rhoei - &
                               0.5 * ( (v1_old**2 - v1(i1 ,i2 ,i3 )**2) + &
                                       (v2_old**2 - v2(i1 ,i2 ,i3 )**2) + &
                                       (v3_old**2 - v3(i1 ,i2 ,i3 )**2)  &
                                     ) &
                                           )
        dev(i1 ,i2 ,i3 )=dev(i1 ,i2 ,i3 ) - dtimeoverrhodx * flux_rhoei
        !
!!$if ((i1 == 1) .and. (i2 == 10) .and. (i3 == 25)) then
!!$  write(6,'(A,5E13.5)'    ) 'dx1, dx3, dx, dx0_update, dxd_update:', &
!!$                             dx1, dx3, dx, dx0_update, dxd_update
!!$  write(6,'(A,5E13.5)'    ) 'dv1, dv3, dvdx, dv, sv:              ', dv1, dv3, dvdx, dv, sv
!!$  write(6,'(A,2E13.5)'    ) 'nu_total, rhob:                      ', nu_total, rhob
!!$  write(6,'(A,4E13.5)'    ) 'flux_rhov, flux_rhov1,3, flux_rhoei: ', &
!!$                             flux_rhov, flux_rhov1, flux_rhov3, flux_rhoei
!!$  write(6,'(A,1E13.5)'    ) 'dtimeoverrhodx:                         ', dtimeoverrhodx
!!$  write(6,'(A,5E13.5)'    ) 'model%v1, v1_old, v1, v3_old, v3:    ', &
!!$                             model%v1(i1,i2,i3), v1_old, v1(i1,i2,i3), v3_old, v3(i1,i2,i3)
!!$  write(6,'(A,3E13.5)'    ) 'model%ei, ei_old, ei:                ', &
!!$                             model%ei(i1,i2,i3), ei_old, ei(i1,i2,i3)
!!$endif
        !
        ! --- Update: target ---
        dtimeoverrhodx  =dtime/(model%rho(i1d,i2d,i3d)*dxd_update)
        v1_old          =v1(i1d,i2d,i3d)
        v2_old          =v2(i1d,i2d,i3d)
        v3_old          =v3(i1d,i2d,i3d)
!        ei_old          =ei(i1d,i2d,i3d)
        v1( i1d,i2d,i3d)=v1(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhov1
        v2( i1d,i2d,i3d)=v2(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhov2
        v3( i1d,i2d,i3d)=v3(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhov3
        ei( i1d,i2d,i3d)=ei(i1d,i2d,i3d) + ( dtimeoverrhodx * flux_rhoei + &
                               0.5 * ( (v1_old**2 - v1(i1d,i2d,i3d)**2) + &
                                       (v2_old**2 - v2(i1d,i2d,i3d)**2) + &
                                       (v3_old**2 - v3(i1d,i2d,i3d)**2)  &
                                     ) &
                                           )
        dev(i1d,i2d,i3d)=dev(i1d,i2d,i3d) + dtimeoverrhodx * flux_rhoei
        !
        ! --- Maximum time step ---
        !inv_dtime_limit=max(inv_dtime_limit, nu_total/dx_sqr)
        !inv_dtime_limit=max(inv_dtime_limit, &
        !                    (nu_total*rhob)/(model%rho(i1 ,i2 ,i3 )*dx0_update*dx))
        !inv_dtime_limit=max(inv_dtime_limit, &
        !                    (nu_total*rhob)/(model%rho(i1d,i2d,i3d)*dxd_update*dx))
        inv_dtime_limit=max(inv_dtime_limit, &
                            (nu_total*rhob)/(model%rho(i1 ,i2 ,i3 )*dx0_update*dx), &
                            (nu_total*rhob)/(model%rho(i1d,i2d,i3d)*dxd_update*dx))
        !
!!$if ((i1 == 1) .and. (i2 == 10) .and. (i3 == 25)) then
!!$  write(6,'(A,1E13.5)'    ) 'dtimeoverrhodx:                         ', dtimeoverrhodx
!!$  write(6,'(A,5E13.5)'    ) 'model%v1, v1_old, v1, v3_old, v3:    ', &
!!$                             model%v1(i1d,i2d,i3d), v1_old, v1(i1d,i2d,i3d), v3_old, v3(i1d,i2d,i3d)
!!$  write(6,'(A,3E13.5)'    ) 'model%ei, ei_old, ei:                ', &
!!$                             model%ei(i1d,i2d,i3d), ei_old, ei(i1d,i2d,i3d)
!!$endif
      end do ! i1
     end do ! i2
    end do ! i3
  endif
end do ! idir
!
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VISp2p3D: dvdx,nu,fluxes,update')
!#
# endif
!#
!
!write(6,'(A,2E13.5)') 'minmax(dei): ', minval(ei(m1:n1,m2:n2,m3:n3)-model%ei(m1:n1,m2:n2,m3:n3)), &
!                                       maxval(ei(m1:n1,m2:n2,m3:n3)-model%ei(m1:n1,m2:n2,m3:n3))
!
! \label{f90:c_viscourant__use6}\label{f90:c_viscourantmax__use4}
! --- Check for time step limit violation ---
if (inv_dtime_limit > 100.0*spacing(0.0)) then
  dtime_limit=par%C_visCourant/inv_dtime_limit
  if (dtime > (par%C_visCourantMax/par%C_visCourant*dtime_limit)) then
    ierr0=1
    outstr0='Time step exceeds viscous time scale'
    goto 999
  endif
endif
!
!!$einew_min=minval(ei(m1:n1,m2:n2,m3:n3))
!!$call tabinter_getinfo(x2min=eiTable_min)
!!$if (einew_min < eiTable_min) then
!!$  ierr0=7
!!$  dtime_limit=min(dtime_limit, 0.7*dtime)
!!$  outstr0='Internal energy below minimum table value'
!!$  goto 999
!!$endif
!
!einew_min=minval(ei(m1:n1,m2:n2,m3:n3)-model%ei(m1:n1,m2:n2,m3:n3))
!if (einew_min < (-spacing(0.0))) then
!  ierr0=8
!  dtime_limit=min(dtime_limit, 0.7*dtime)
!  outstr0='Change in internal energy below zero'
!  goto 999
!endif
!
call tabinter_getinfo(x2min=eiTable_min)
einew_min=minval((ei(m1:n1,m2:n2,m3:n3)-model%ei(m1:n1,m2:n2,m3:n3))/ &
                 (max(model%ei(m1:n1,m2:n2,m3:n3)-eiTable_min, 0.0) + &
                      model%v1(m1:n1,m2:n2,m3:n3)**2 + &
                      model%v2(m1:n1,m2:n2,m3:n3)**2 + &
                      model%v3(m1:n1,m2:n2,m3:n3)**2    ))
if (einew_min < (-100.0*spacing(1.0))) then
  ierr0=8
  dtime_limit=min(dtime_limit, 0.7*dtime)
  write(error_locstr,'(1PE14.5)') einew_min
  outstr0=trim(error_locstr)
  write(error_locstr,'(1PE14.5)') eiTable_min
  outstr0=trim(outstr0) // ' / ' // trim(error_locstr)
  outstr0='Change in internal energy below zero: ' // outstr0
  goto 999
endif
!
! --- Output of additional data ---
if ((Output_flag0) .and. present(dat)) then
!#
# ifdef rhd_vis_t01
!#
  call timing_start( 'VISp2p3D: (output)')
!#
# endif
!#
  call rhd_vis_p2pStepOutput(model, &
                             AreaFraction10, AreaFraction20, AreaFraction30, &
                             model%rho(m1:n1,m2:n2,m3:n3)*dev(m1:n1,m2:n2,m3:n3)/dtime, &
                             dat)
!#
# ifdef rhd_vis_t01
!#
  call timing_stop( 'VISp2p3D: (output)')
!#
# endif
!#
endif
!
! --- Take updated quantities ---
model%v1=v1
model%v2=v2
model%v3=v3
model%ei=ei
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!#
# ifdef rhd_vis_t01
!#
call timing_stop('VISp2p3D:')
!#
# endif
!#
!
end subroutine rhd_vis_p2p3dStep


!----------************-----------------------------------------------------------------------------
subroutine rhd_vis_Step(model, action, dtime, dtime_limit, &
                        Output_flag, &
                        dat, &
                        outstr, ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_vis_Step ('rhd_viscosity_Step')
!
! PURPOSE:
!   Update model by applying tensor viscosity.
!
! CATEGORY:
!   Hydrodynamics, Viscosity
!
! CALLING SEQUENCE:
!   call rhd_vis_Step(model, action, dtime, dtime_limit, &
!                     Output_flag=Output_flag, &
!                     dat=dat, &
!                     outstr=outstr, ierr=ierr)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   Output_flag:  ('Output_flag') logical, optional, if true the output fluxes are computed,
!                 default=false
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!   dat:          ('data') derived type with additional output data
!
! OUTPUT:
!   dtime_limit:  ('delta_time_limit') real, proposed maximum time step
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                    =0: normal run
!                  1-99: "small" error (e.g., time step limit violation)
!                 >=100: "severe" error (does not occur in this version)
!
! LOCAL VARIABLES:
!   i_chunk:      ('index_chunk') integer, chunk index, used while splitting box into chunks
!   n_chunks:     ('number_chunks') integer, number of chunks the box is to be split into
!   m1,2,3c:      ('m_1,2,3_chunk') integer, first index value of chunk
!   n1,2,3c:      ('n_1,2,3_chunk') integer, last index value of chunk
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_bound_ExtractBoxMPI: Extract part of box, fill boundary ghost cells
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_MPIFillGhost:  Fill ghost cells by communicating with neighbor MPI processes
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_datx_Add:          Add the fields in the structure for additional data
!   rhd_vis_2dStep:        Update model by applying 2D tensor viscosity (incl. turbulent mixing)
!   rhd_vis_3dStep:        Update model by applying 3D tensor viscosity (incl. turbulent mixing)
!   rhd_vis_p2p2dStep:     Update model by applying 2D point-to-point (tensor) viscosity
!   rhd_vis_p2p3dStep:     Update model by applying 3D point-to-point (tensor) viscosity
!   rhd_wrk_module:        Provide persistent work copies of model data
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_bound_module:      Boundary-condition routines for RHD
!   rhd_box_module:        Box-handling routines for RHD
!   rhd_dat_module:        Routines for the collection of additional data
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!
! SIDE EFFECTS:
!   The global variable modelVis is modified.
!
! RESTRICTIONS:
!   The routines will likely not work in 1D.
!   2D models are only allowed with n2=m2.
!
! MODIFICATION HISTORY:
!   2006-11-22 (B.F. Lyon) First version of separate routine, taken from rhd.F90
!   2010-06-15 (B.F.) Create action_current because of bound_inoutflow2
!   2012-05-18 (B.F.) Different action_current also for bound_closedbottom2
!   2012-11-19 (B.F.) MPI version: first steps
!   2012-11-20 (B.F.) MPI version; call rhd_box_ExtractMPI
!   2014-08-17 (B.F.) Use global modelVis box
!   2016-03-20 (B.F. Uppsala) Put previous ghost-cell exchange calls into rhd_boxMPI_FillGhost
!   2017-09-16 (B.F.) Replace model%m3->m3c, model%n3->n3c in some cases;
!                     Use chunks complete in x1 direction instead of x3 direction
!   2017-09-23 (B.F.) Add timing call around MPI ghost-cell-filling calls
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_dat_module
use rhd_action_module
use rhd_box_module
use rhd_wrk_module
use rhd_bound_module
use timing_module
use rhd_MPI_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: dtime_limit
logical,         optional, intent(in)   :: Output_flag
type(dat_type),  optional, intent(inout):: dat
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
type(box_type)                          :: slice
type(dat_type)                          :: dat_thread
type(action_type)                       :: action_current
integer                                 :: i1,i2,i3, i_chunk, n_chunks, m1c,n1c, m2c,n2c, m3c,n3c, &
                                           nghost
real                                    :: dtime_limit_chunk, dtime_limit_thread
character(len=80)                       :: outstr_chunk, outstr_thread
integer                                 :: ierr_chunk, ierr_thread
integer                                 :: ierr0
character(len=80)                       :: outstr0
integer                                 :: ierrMPI, ProcErr
!---------------------------------------------------------------------------------------------------
!
if ((action%bound_l3 == bound_inoutflow2) .or. (action%bound_l3 == bound_inoutflow3)) then
  action_current=rhd_action_New(action, bound_l3=bound_inoutflow)
else if ((action%bound_l3 == bound_closedbottom2)) then
  action_current=rhd_action_New(action, bound_l3=bound_closedbottom)
else
  action_current=action
endif
!
! --- Set time step limit to maximum value ---
dtime_limit=huge(dtime_limit)
!
! ### Classical tensor viscosity step ##############################################################
!
if ((par%C_visSmagorinsky > 0.0) .or. (par%C_visArtificial > 0.0) .or. &
    (par%C_visExpansion   > 0.0) .or. (par%C_visLinear     > 0.0)     ) then
  if (model%n2 == model%m2) then
    ! === Classical 2D tensor viscosity step =======================================================
    !
    call timing_start('Viscosity routines 2D')
    call timing_start('VIS:')
    !
    ! --- Set number of ghost cells ---
    nghost=1
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    call timing_start('VIS:   FillGhost')
    call rhd_box_MPIFillGhost(wg%hyd, model, nghost, &
                              grid_mode=0, bulk_mode=1, rho_mode=1, ei_mode=1, v_mode=1)
    call timing_stop('VIS:   FillGhost')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! \label{f90:n_viscellsperchunk__use1}
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_visCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'VIS2D: N_CellsPerChunk, n_chunks: ', par%N_visCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    if (n_chunks > 1) then
      ! --- Create empty temporary 3D model as backup for current model, if necessary ---
      if (.not. rhd_box_Exist(modelVis)) then
        call timing_start('VIS: make_box(modelVis)')
        call rhd_box_Make(modelVis, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                    m3=model%m3, n3=model%n3)
        call timing_stop('VIS: make_box(modelVis)')
      endif
    endif
    !
    ierr0           =0
    outstr0         =''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, m3c, n3c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (VIS2D_INIT)
    outstr_thread         =''
    ierr_thread           =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (VIS2D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPVIS2D: do i_chunk=1,n_chunks
      ! \label{f90:n_viscellsperchunk__use2}
      ! --- Current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_visCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
                             m1c,n1c, m2c,n2c, m1c=m3c,n1c=n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,3(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c,n1c, ' : ', m2c,n2c, ' : ', m3c,n3c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=1, nghost2=0, nghost3=1)
      call rhd_bound_ExtractBoxMPI(model, action_current, &
                                          m1c     , n1c, m2c     , n2c, m3c     , n3c, 1, &
                                   slice, slice%m1,      slice%m2,      slice%m3           )
      !
      ! --- Actual 2D viscosity step ---
      if (Output_flag) then
        call rhd_vis_2dStep(slice, action_current, dtime, dtime_limit_chunk, &
                            Output_flag=Output_flag, &
                            AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                          (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                          (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                          (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                            AreaFraction2=1.0, &
                            AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                          (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                          (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                          (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                            dat=dat_thread, &
                            outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_vis_2dStep(slice, action_current, dtime, dtime_limit_chunk, &
                            outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelVis, m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPVIS2D ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (VIS2D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (VIS2D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from viscosity step into standard box and delete the former ---
      call timing_start('VIS: copy_box(modelVis)')
      call rhd_box_Copy(modelVis, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                        model   , model%m1,           model%m2,           model%m3, &
                        trans_mode=1)
      call timing_stop('VIS: copy_box(modelVis)')
      ! \label{f90:hdtmpstructkeepmode__useVIS1}
      if (par%hdTmpStructKeepMode == 'Delete') then
        call timing_start('VIS: delete_box(modelVis)')
        call rhd_box_Delete(modelVis)
        call timing_stop('VIS: delete_box(modelVis)')
      endif
    endif
    !
    call timing_stop('VIS:')
    call timing_stop('Viscosity routines 2D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  else ! (model%n2 /= model%m2)
    ! === Classical 3D tensor viscosity step =======================================================
    !
    call timing_start('Viscosity routines 3D')
    call timing_start('VIS:')
    !
    ! --- Set number of ghost cells ---
    nghost=1
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    call timing_start('VIS:   FillGhost')
    call rhd_box_MPIFillGhost(wg%hyd, model, nghost, &
                              grid_mode=0, bulk_mode=1, rho_mode=1, ei_mode=1, v_mode=1)
    call timing_stop('VIS:   FillGhost')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Determine number of chunks ---
!!$    call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
!!$                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
!!$                           m1c,n1c, m2c,n2c, &
!!$                           n_chunks=n_chunks)
    call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
                           model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                           m2c,n2c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'VIS3D: N_CellsPerChunk, n_chunks: ', par%N_visCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    if (n_chunks > 1) then
      ! --- Create empty temporary 3D model as backup for current model, if necessary ---
      if (.not. rhd_box_Exist(modelVis)) then
        call timing_start('VIS: make_box(modelVis)')
        call rhd_box_Make(modelVis, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                    m3=model%m3, n3=model%n3)
        call timing_stop('VIS: make_box(modelVis)')
      endif
    endif
    !
    ierr0           =0
    outstr0         =''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, m3c, n3c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (VIS3D_INIT)
    outstr_thread         =''
    ierr_thread           =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (VIS3D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPVIS3D: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
!!$      call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
!!$                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
!!$                             m1c,n1c, m2c,n2c, m1c=m3c,n1c=n3c)
      call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
                             model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk,&
                             m2c,n2c, m3c,n3c, m1c=m1c,n1c=n1c)
      !!$write(gl%nc_p,'(I3,I6,I6,3(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c,n1c, ' : ', m2c,n2c, ' : ', m3c,n3c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=1, nghost2=1, nghost3=1)
      call rhd_bound_ExtractBoxMPI(model, action_current, &
                                          m1c     , n1c, m2c     , n2c, m3c, n3c, 1, &
                                   slice, slice%m1,      slice%m2,      slice%m3              )
      !
      ! --- Actual 3D viscosity step ---
      if (Output_flag) then
        call rhd_vis_3dStep(slice, action_current, dtime, dtime_limit_chunk, &
                            Output_flag=Output_flag, &
                            AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                          (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                          (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                          (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                            AreaFraction2=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                          (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                          (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                          (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                            AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                          (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                          (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                          (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                            dat=dat_thread, &
                            outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_vis_3dStep(slice, action_current, dtime, dtime_limit_chunk, &
                            outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelVis, m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPVIS3D ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (VIS3D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (VIS3D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from viscosity step into standard box and delete the former ---
      call timing_start('VIS: copy_box(modelVis)')
!!$      call rhd_box_Copy(modelVis, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
!!$                        model   , model%m1,           model%m2,           model%m3,           &
!!$                        grid_mode=0, time_mode=0, trans_mode=1)
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=model%m3,model%n3
        call rhd_box_Copy(modelVis, model%m1, model%n1, model%m2, model%n2, i3, i3, &
                          model   , model%m1,           model%m2,           i3,     &
                          grid_mode=0, time_mode=0, trans_mode=1)
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      call timing_stop('VIS: copy_box(modelVis)')
      ! \label{f90:hdtmpstructkeepmode__useVIS2}
      if (par%hdTmpStructKeepMode == 'Delete') then
        call timing_start('VIS: delete_box(modelVis)')
        call rhd_box_Delete(modelVis)
        call timing_stop('VIS: delete_box(modelVis)')
      endif
    endif
    !
    call timing_stop('VIS:')
    call timing_stop('Viscosity routines 3D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  endif ! (model%n2 == model%m2)
endif
!
! \label{f90:c_visp2pcoeff__use3}\label{f90:c_visp2phypartificial__use3}
! ### P2P tensor viscosity step ####################################################################
!
if ((par%C_visP2Pcoeff         > 0.0) .or. (par%C_visP2PhypSmagorinsky > 0.0) .or. &
    (par%C_visP2PhypArtificial > 0.0) .or. (par%C_visP2PhypExpansion   > 0.0) .or. &
    (par%C_visP2Pdivrhov       > 0.0)                                             ) then
  if (model%n2 == model%m2) then
    ! === 2D P2P tensor viscosity step =============================================================
    !
    call timing_start('Viscosity routines 2D')
    call timing_start('VPP:')
    !
    ! --- Set (compute) number of ghost cells ---
    nghost=par%A%visP2Phypreconstruction_order+1
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    call timing_start('VPP:   FillGhost')
    call rhd_box_MPIFillGhost(wg%hyd, model, nghost, &
                              grid_mode=0, bulk_mode=1, rho_mode=1, ei_mode=1, v_mode=1)
    call timing_stop('VPP:   FillGhost')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Determine number of chunks ---
    call rhd_box_ChunkSize(0, par%N_visCellsPerChunk, &
                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
                           m1c,n1c, m2c,n2c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'VP2P2D: N_CellsPerChunk, n_chunks:', par%N_visCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    if (n_chunks > 1) then
      ! --- Create empty temporary 3D model as backup for current model, if necessary ---
      if (.not. rhd_box_Exist(modelVis)) then
        call timing_start('VPP: make_box(modelVis)')
        call rhd_box_Make(modelVis, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                    m3=model%m3, n3=model%n3)
        call timing_stop('VPP: make_box(modelVis)')
      endif
    endif
    !
    ierr0  =0
    outstr0=''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, m3c, n3c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (VISP2P2D_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (VISP2P2D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPVPP2D: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
      call rhd_box_ChunkSize(0, par%N_visCellsPerChunk, &
                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
                             m1c,n1c, m2c,n2c, m1c=m3c,n1c=n3c)
      !!$write(gl%nc_p,'(I3,I6,I6,3(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c,n1c, ' : ', m2c,n2c, ' : ', m3c,n3c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, nghost2=0, nghost3=nghost)
      call rhd_bound_ExtractBoxMPI(model, action_current, &
                                          m1c     , n1c, m2c     , n2c, m3c     , n3c, 1, &
                                   slice, slice%m1,      slice%m2,      slice%m3           )
      !
      ! --- Actual 2D viscosity step ---
      if (Output_flag) then
        call rhd_vis_p2p2dStep(slice, action_current, dtime, dtime_limit_chunk, &
                               nghost, Output_flag=Output_flag, &
                               AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                             (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                             (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                               AreaFraction2=1.0, &
                               AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               dat=dat_thread, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_vis_p2p2dStep(slice, action_current, dtime, dtime_limit_chunk, &
                               nghost, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelVis, m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPVPP2D ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (VISP2P2D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (VISP2P2D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from viscosity step into standard box and delete the former ---
      call timing_start('VPP: copy_box(modelVis)')
      call rhd_box_Copy(modelVis, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
                        model   , model%m1,           model%m2,           model%m3, &
                        trans_mode=1)
      call timing_stop('VPP: copy_box(modelVis)')
      ! \label{f90:hdtmpstructkeepmode__useVIS3}
      if (par%hdTmpStructKeepMode == 'Delete') then
        call timing_start('VIS: delete_box(modelVis)')
        call rhd_box_Delete(modelVis)
        call timing_stop('VIS: delete_box(modelVis)')
      endif
    endif
    !
    call timing_stop('VPP:')
    call timing_stop('Viscosity routines 2D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  else ! (model%n2 /= model%m2)
    ! === 3D P2P tensor viscosity step =============================================================
    !
    call timing_start('Viscosity routines 3D')
    call timing_start('VPP:')
    !
    ! --- Set (compute) number of ghost cells ---
    nghost=par%A%visP2Phypreconstruction_order+1
    !nghost=1
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Fill ghost cells with data from neighbor processes ---
    !
    call timing_start('VPP:   FillGhost')
    call rhd_box_MPIFillGhost(wg%hyd, model, nghost, &
                              grid_mode=0, bulk_mode=1, rho_mode=1, ei_mode=1, v_mode=1)
    call timing_stop('VPP:   FillGhost')
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Determine number of chunks ---
!!$    call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
!!$                           model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, 1, &
!!$                           m1c,n1c, m2c,n2c, &
!!$                           n_chunks=n_chunks)
    call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
                           model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, 1, &
                           m2c,n2c, m3c,n3c, &
                           n_chunks=n_chunks)
    if (gl%nc_m >= 0) then
      write(gl%nc_m,'(A,I10,I8)') &
        'VP2P3D: N_CellsPerChunk, n_chunks:', par%N_visCellsPerChunk, n_chunks
      call flush(gl%nc_m)
    endif
    !
    if (n_chunks > 1) then
      ! --- Create empty temporary 3D model as backup for current model, if necessary ---
      if (.not. rhd_box_Exist(modelVis)) then
        call timing_start('VPP: make_box(modelVis)')
        call rhd_box_Make(modelVis, m1=model%m1, n1=model%n1, m2=model%m2, n2=model%n2, &
                                    m3=model%m3, n3=model%n3)
        call timing_stop('VPP: make_box(modelVis)')
      endif
    endif
    !
    ierr0  =0
    outstr0=''
    !
    ! vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    !$OMP PARALLEL DEFAULT(SHARED), &
    !$OMP          PRIVATE(dat_thread, slice, &
    !$OMP                  m1c, n1c, m2c, n2c, m3c, n3c, &
    !$OMP                  dtime_limit_chunk, dtime_limit_thread, &
    !$OMP                  outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
    !
    !$OMP CRITICAL (VISP2P3D_INIT)
    outstr_thread     =''
    ierr_thread       =0
    dtime_limit_thread=huge(dtime_limit_thread)
    !
    if (Output_flag) then
      ! --- Thread-local data: set data fields to zero, make radial axis ---
      call rhd_dat_Make(dat_thread, model, min(par%N_radband, 2))
    endif
    !$OMP END CRITICAL (VISP2P3D_INIT)
    !
    !$OMP DO SCHEDULE(RUNTIME)
    CHUNK_LOOPVPP3D: do i_chunk=1,n_chunks
      ! --- Current chunk indices ---
!!$      call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
!!$                             model%m3,model%n3, model%m1,model%n1, model%m2,model%n2, i_chunk,&
!!$                             m1c,n1c, m2c,n2c, m1c=m3c,n1c=n3c)
      call rhd_box_ChunkSize(1, par%N_visCellsPerChunk, &
                             model%m1,model%n1, model%m2,model%n2, model%m3,model%n3, i_chunk,&
                             m2c,n2c, m3c,n3c, m1c=m1c,n1c=n1c)
      !!$write(gl%nc_p,'(I3,I6,I6,3(A,2I10))') &
      !!$  omp_get_thread_num(), i_chunk, n_chunks, ' : ', m1c,n1c, ' : ', m2c,n2c, ' : ', m3c,n3c
      !
      ! --- Create empty sub box and fill it with data ---
      call rhd_box_Make(slice, m1=m1c, n1=n1c, m2=m2c, n2=n2c, m3=m3c, n3=n3c, &
                               nghost1=nghost, nghost2=nghost, nghost3=nghost)
      call rhd_bound_ExtractBoxMPI(model, action_current, &
                                          m1c     , n1c, m2c     , n2c, m3c     , n3c, 1, &
                                   slice, slice%m1,      slice%m2,      slice%m3           )
      !
      ! --- Actual 3D viscosity step ---
      if (Output_flag) then
        call rhd_vis_p2p3dStep(slice, action_current, dtime, dtime_limit_chunk, &
                               nghost, Output_flag=Output_flag, &
                               AreaFraction1=(model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)) * &
                                             (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                             (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                               AreaFraction2=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb3(n3c+1     )-model%xb3(m3c     ))/ &
                                             (model%xb3(model%n3+1)-model%xb3(model%m3)), &
                               AreaFraction3=(model%xb1(n1c+1     )-model%xb1(m1c     ))/ &
                                             (model%xb1(model%n1+1)-model%xb1(model%m1)) * &
                                             (model%xb2(n2c+1     )-model%xb2(m2c     ))/ &
                                             (model%xb2(model%n2+1)-model%xb2(model%m2)), &
                               dat=dat_thread, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      else
        call rhd_vis_p2p3dStep(slice, action_current, dtime, dtime_limit_chunk, &
                               nghost, &
                               outstr=outstr_chunk, ierr=ierr_chunk)
      endif
      !
      ! --- Copy data back into full model and delete temporary box ---
      if (n_chunks > 1) then
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          modelVis, m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      else
        call rhd_box_Copy(slice   , slice%m1, slice%n1, slice%m2, slice%n2, slice%m3, slice%n3, &
                          model   , m1c     ,           m2c     ,           m3c     , &
                          trans_mode=1)
      endif
      call rhd_box_Delete(slice)
      !
      ! --- Handle error conditions and time step ---
      if (ierr_chunk > ierr_thread) then
        ierr_thread  =ierr_chunk
        outstr_thread=outstr_chunk
      endif
      dtime_limit_thread=min(dtime_limit_thread, dtime_limit_chunk)
    end do CHUNK_LOOPVPP3D ! i_chunk
    !$OMP END DO NOWAIT
    !
    !$OMP CRITICAL (VISP2P3D_REDUCTION)
    if (ierr_thread > ierr0) then
      ierr0  =ierr_thread
      outstr0=outstr_thread
    endif
    dtime_limit=min(dtime_limit, dtime_limit_thread)
    !
    if (Output_flag) then
      call rhd_datx_Add(dat%x12, dat_thread%x12)
      call rhd_datx_Add(dat%x13, dat_thread%x13)
      call rhd_datx_Add(dat%x23, dat_thread%x23)
      call rhd_datx_Add(dat%r  , dat_thread%r  )
      call rhd_dat_Delete(dat_thread)
    endif
    !$OMP END CRITICAL (VISP2P3D_REDUCTION)
    !
    !$OMP END PARALLEL
    ! ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    !
    if (n_chunks > 1) then
      ! --- Copy output box from viscosity step into standard box and delete the former ---
      call timing_start('VPP: copy_box(modelVis)')
      !   call rhd_box_Copy(modelVis, model%m1, model%n1, model%m2, model%n2, model%m3, model%n3, &
      !                     model   , model%m1,           model%m2,           model%m3,           &
      !                     grid_mode=0, time_mode=0, trans_mode=1)
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO SCHEDULE(RUNTIME)
      do i3=model%m3,model%n3
        call rhd_box_Copy(modelVis, model%m1, model%n1, model%m2, model%n2, i3, i3, &
                          model   , model%m1,           model%m2,           i3,     &
                          grid_mode=0, time_mode=0, trans_mode=1)
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      call timing_stop('VPP: copy_box(modelVis)')
      ! \label{f90:hdtmpstructkeepmode__useVIS4}
      if (par%hdTmpStructKeepMode == 'Delete') then
        call timing_start('VIS: delete_box(modelVis)')
        call rhd_box_Delete(modelVis)
        call timing_stop('VIS: delete_box(modelVis)')
      endif
    endif
    !
    call timing_stop('VPP:')
    call timing_stop('Viscosity routines 3D')
    !
    ! --- Handle error conditions and time step ---
    if (ierr0 > 0) goto 999
    !
  endif ! (model%n2 == model%m2)
endif
!
999 continue
!
! --- Handle error conditions and time step ---
call rhd_MPI_DtimeErrorDist(wg%hyd%Comm, wg%hyd%MyProc, wg%hyd%MaProcWoInCO, &
                            dtime_limit, ierr0, outstr0, ProcErr)
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_vis_Step

end module rhd_vis_module
