!***************************************************************************************************
!
!   #####  #    # #####         #####   ####  #    # #    # #####  
!   #    # #    # #    #        #    # #    # #    # ##   # #    # 
!   #    # ###### #    #        #####  #    # #    # # #  # #    # 
!   #####  #    # #    #        #    # #    # #    # #  # # #    # 
!   #   #  #    # #    #        #    # #    # #    # #   ## #    # 
!   #    # #    # #####  ###### #####   ####   ####  #    # ##### _module
!
!   Radiation-Hydrodynamics Code: boundary conditions for hydrodynamics solver
!
!***************************************************************************************************
!   Fortran90
!   2018-11-10
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Werner Schaffenberger: Exeter
!   Sven Wedemeyer-Boehm:  Freiburg
!***************************************************************************************************
! MODULES:
!   rhd_bound_module:      Boundary-condition routines for RHD
!
!***************************************************************************************************

!# /* --- Set default values for some preprocessor macros --------------------------------------- */
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Number of magnetic-field arrays --- */
# ifndef rhd_box_bmag01
#   define rhd_box_bmag01 0
# endif
!#
!# /* --- MPI activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------****************-----------------------------------------------------------------------------
module rhd_bound_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_boundary_module ('rhd_boundary_module')
!
! PURPOSE:
!   Provide boundary-condition routines for rhd.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_bound_module
!
! VARIABLES:
!   None
!
! ROUTINES:
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_bound_1DGhostArray: Fill ghost cells of single array according to boundary conditions
!   rhd_bound_1DTransmit:  Adjust values in top or bottom cells for transmitting boundary condition
!   rhd_bound_SwitchInfo:  Print information about used compiler switches
!   rhd_bound_3DTransmit:  Adjust density values in top cells for the open boundary condition
!   rhd_bound_3DInOutFlow: Adjust values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DCenter:    Modify energy in central cells to keep entropy constant
!   rhd_bound_3DHeatBottom: Modify energy in bottom cells to keep entropy constant
!   rhd_bound_ExtractBox:       Extract part of box, fill boundary ghost cells (MHD version; no MPI)
!   rhd_bound_ExtractBoxMPI:    Extract part of box, fill boundary ghost cells (MPI version; no MHD)
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) First version
!   1999-05-11 (B.F. Copenhagen) Routine bound3d_inoutflow added
!   1999-06-10 (B.F.) Routine bound3d_center added
!   1999-09-28 (B.F.) Routine bound1d_transmit added
!   2001-01    (B.F. Uppsala) .f90 -> .F90, preprocessor macros added (for timing routine calls and
!                     routine versions)
!   2012-10-11 (B.F.) Steps towards MPI
!   2016-04-15 (B.F.) Reorganize global world-geometry variables for MPI
!   2017-09-18 (B.F.) Extract routines from rhd_hyd_module and rhd_box_module
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhd_bound_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_SwitchInfo ('rhd_boundary_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to requested output channel.
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_bound_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2017-09-18 (B.F.) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_bound_module .......................'
!
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
write(ncp,'(A,I6)') '  rhd' // '_box_quc01:              ', rhd_box_quc01
write(ncp,'(A,I6)') '  rhd' // '_box_bmag01:             ', rhd_box_bmag01
!
!#
# ifdef rhd_bound_t01
!#
write(ncp,'(A,A)' ) '  rhd' // '_bound_t01:           ', defstr
!#
# else
!#
write(ncp,'(A,A)' ) '  rhd' // '_bound_t01:           ', undefstr
!#
# endif
!#
!
end subroutine rhd_bound_SwitchInfo


!----------*****************------------------------------------------------------------------------
subroutine rhd_bound_1DGhost(model, action, grid_mode, bulk_mode, Hp_l, Hp_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_1DGhost ('rhd_boundary_conditions_1D_Ghost_cells')
!
! PURPOSE:
!   Fill ghost cells according to boundary conditions.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_1DGhost(model, action, grid_mode=grid_mode, bulk_mode=bulk_mode, &
!                          Hp_l=Hp_l, Hp_r=Hp_r)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:1
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default:1
!   Hp_l:         ('height_pressure_left') optional real, pressure scale on left side
!   Hp_r:         ('height_pressure_right') optional real, pressure scale on right side
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, e, v, x, time, ...
!
! OUTPUT:
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_box_module:        Box-handling routines for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The number of ghost zones one each side must not exceed the number of interior grid points.
!
! PROCEDURE:
!   Possible boundary conditions:
!     bound_reflective
!     bound_periodic
!     bound_constant
!     bound_closed
!     bound_closed2
!     bound_inoutflow     (assumed call of rhd_bound3d_inoutflow)
!     bound_inoutflow2    (assumed call of rhd_bound3d_inoutflow)
!     bound_inoutflow3    (assumed call of rhd_bound3d_inoutflow)
!     bound_closedbottom  (with prescribed radiative flux)
!     bound_closedbottom2 (with prescribed radiative flux)
!     bound_transmitting  (constant, rho declines exponentially, with temperature correction)
!     bound_transmitting2 (constant, rho declines exponentially, with temperature correction)
!     bound_transmitting3 (constant, with temperature correction)
!     otherwise: no action
!   Possible magnetic boundary conditions:
!     bound_mag_periodic
!     bound_mag_constant
!     bound_mag_vertical
!     bound_mag_inoutflow
!     bound_mag_oblique
!     bound_mag_fixed
!     bound_mag_reflective
!     bound_mag_vertical2
!     bound_mag_constant2
!     otherwise: no action
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-15 (B.F. Kiel) First version
!   1999-04-13 (B.F. Copenhagen) Boundary condition 'closed'
!   1999-05-11 (B.F.) Boundary condition 4
!   1999-09-27 (B.F.) Boundary condition 6
!   2000-03-25 (B.F.) Keywords grid, bulk
!   2001-02-12 (B.F.) Transmitting boundary: exploit equidistant extrapolated grid and save exp call
!   2001-04-09 (B.F.) Factor in front of Hp
!   2002-05-07 (B.F.) Lower limit for boundary density with action%rho_min instead of par%N_test1
!   2002-10-16 (B.F.) "Extrapolation" of phic_grav: mostly constant
!   2002-11-13 (B.F.) bound_transmitting2
!   2004-03-27 (B.F.) Extrapolation of Bb arrays out-commented (index ranges are wrong!)
!   2006-09-12 (W.S. Freiburg) bound_inoutflow2
!   2008-07-28 (B.F. Lyon) Check for presence of Hp_l and Hp_r moved
!   2008-06-11 (W.S. Freiburg) magnetic boundary conditions
!   2010-03-16 (B.F. Naples) Handle energy_switch depending on boundary condition, with Bc
!   2010-03-25 (B.F.) Bugfix: left MHD oblique boundary condition
!   2010-04-14 (B.F.) Renaming: B_theta, B_phi -> C_magthetaB, C_magphiB
!   2010-04-20 (B.F.) Rename bound1d_ghost -> rhd_bound_1DGhost
!   2010-04-28 (B.F.) bound_transmitting3
!   2010-06-12 (B.F.) bound_inoutflow2, bound_closedbottom2
!   2011-03-28 (W.S.) bound_mag_vertical2
!   2011-04-28 (B.F.) bound_mag_constant2
!   2011-11-15 (B.F. Lyon) bound_inoutflow3
!   2012-05-18 (B.F.) bound_closedbottom2: divide v1 by sqrt(rhoFac)
!   2014-09-21 (B.F. Uppsala) bound_closed2
!   2017-07-25 (B.F.) Change some comments and spaces; add some brackets
!   2017-08-24 (B.F. Lyon) Make some boundary conditions that need two ghost layers work with one:
!                       bound_closed2, bound_closedbottom2, bound_inoutflow2, bound_inoutflow3,
!                       bound_mag_constant2;
!                     Change dependence of velocity on density for bound_closedbottom2;
!                     Remove some outcommented code
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_gl_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
integer,         optional, intent(in)   :: grid_mode, bulk_mode
real, dimension(model%m2:model%n2, &
                model%m3:model%n3), &
                 optional, intent(in)   :: Hp_l, Hp_r
!
! --- Local variables ---
integer                                 :: grid_mode0, bulk_mode0
integer                                 :: i1, bound_l, bound_r, bound_mag_l, bound_mag_r, iquc
integer                                 :: m1, n1, nghost1
real                                    :: fx, fy
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: Hp_l0, Hp_r0, rhoFac, eiAdd
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1     =model%m1
n1     =model%n1
nghost1=model%nghost1
!
! --- Set default values for optional parameters ---
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=1
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l    =action%bound_l1
    bound_r    =action%bound_r1
    bound_mag_l=action%bound_mag_l1
    bound_mag_r=action%bound_mag_r1
  case (2)
    bound_l    =action%bound_l2
    bound_r    =action%bound_r2
    bound_mag_l=action%bound_mag_l2
    bound_mag_r=action%bound_mag_r2
  case (3)
    bound_l    =action%bound_l3
    bound_r    =action%bound_r3
    bound_mag_l=action%bound_mag_l3
    bound_mag_r=action%bound_mag_r3
end select ! (action%dim)
!
! === Fill axes ====================================================================================
!
if (grid_mode0 > 0) then
  ! --- Handle 'left' boundary ---
  select case (bound_l)
    case (bound_reflective)
      ! --- reflective ---
      do i1=1,nghost1
        model%xb1(m1-i1) = model%xb1(m1-i1+1) + (model%xb1(m1+i1-1)-model%xb1(m1+i1))
        model%xc1(m1-i1) = 0.5*(model%xb1(m1-i1)+model%xb1(m1-i1+1))
      end do ! i1
    case (bound_periodic)
      ! --- periodic ---
      model%xb1(m1-nghost1:m1-1) = model%xb1(n1-nghost1+1:n1) + (model%xb1(m1)-model%xb1(n1+1))
      model%xc1(m1-nghost1:m1-1) = model%xc1(n1-nghost1+1:n1) + (model%xb1(m1)-model%xb1(n1+1))
    case (bound_constant, bound_inoutflow, bound_inoutflow2, bound_inoutflow3, &
          bound_transmitting, bound_transmitting2, bound_transmitting3)
      ! --- constant ---
      do i1=1,nghost1
        model%xb1(m1-i1) = model%xb1(m1-i1+1) + (model%xb1(m1)-model%xb1(m1+1))
        model%xc1(m1-i1) = 0.5*(model%xb1(m1-i1)+model%xb1(m1-i1+1))
      end do ! i1
    case (bound_closed, bound_closed2, bound_closedbottom, bound_closedbottom2)
      ! --- closed ---
      do i1=1,nghost1
        model%xb1(m1-i1) = model%xb1(m1-i1+1) + (model%xb1(m1)-model%xb1(m1+1))
        model%xc1(m1-i1) = 0.5*(model%xb1(m1-i1)+model%xb1(m1-i1+1))
      end do ! i1
  end select ! (bound_l)
  !
  ! --- Handle 'right' boundary ---
  select case (bound_r)
    case (bound_reflective)
      ! --- reflective ---
      do i1=1,nghost1
        model%xb1(n1+i1+1) = model%xb1(n1+i1) + (model%xb1(n1-i1+2)-model%xb1(n1-i1+1))
        model%xc1(n1+i1  ) = 0.5*(model%xb1(n1+i1)+model%xb1(n1+i1+1))
      end do ! i1
    case (bound_periodic)
      ! --- periodic ---
      model%xb1(n1+2:n1+nghost1+1) = model%xb1(m1+1:m1+nghost1  ) + (model%xb1(n1+1)-model%xb1(m1))
      model%xc1(n1+1:n1+nghost1  ) = model%xc1(m1  :m1+nghost1-1) + (model%xb1(n1+1)-model%xb1(m1))
    case (bound_constant, bound_inoutflow, &
          bound_transmitting, bound_transmitting2, bound_transmitting3)
      ! --- constant ---
      do i1=1,nghost1
        model%xb1(n1+i1+1) = model%xb1(n1+i1) + (model%xb1(n1+1)-model%xb1(n1))
        model%xc1(n1+i1  ) = 0.5*(model%xb1(n1+i1)+model%xb1(n1+i1+1))
      end do ! i1
    case (bound_closed, bound_closed2, bound_closedbottom, bound_closedbottom2)
      ! --- closed ---
      do i1=1,nghost1
        model%xb1(n1+i1+1) = model%xb1(n1+i1) + (model%xb1(n1+1)-model%xb1(n1))
        model%xc1(n1+i1  ) = 0.5*(model%xb1(n1+i1)+model%xb1(n1+i1+1))
      end do ! i1
  end select ! (bound_r)
endif ! (grid0)
!
! === Handle bulk data =============================================================================
!
if (bulk_mode0 > 0) then
  !
  ! === Handle (non-magnetic) bulk data ============================================================
  !
  ! --- 'Left' boundary ----------------------------------------------------------------------------
  !
  select case (bound_l)
    !============================================================
    case (bound_reflective)
      ! --- reflective ---
      do i1=1,nghost1
        model%rho(      m1-i1,:,:)= model%rho(      m1+i1-1,:,:)
        model%ei(       m1-i1,:,:)= model%ei(       m1+i1-1,:,:)
        model%v1(       m1-i1,:,:)=-model%v1(       m1+i1-1,:,:)
        model%v2(       m1-i1,:,:)= model%v2(       m1+i1-1,:,:)
        model%v3(       m1-i1,:,:)= model%v3(       m1+i1-1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(m1-i,:,:)= model%phic_grav(m1+i1-1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    m1-i1,:,:,iquc)= model%quc( m1+i1-1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
!
      end do ! i1
      !
    !============================================================
    case (bound_periodic)
      ! --- periodic ---
        model%rho(      m1-nghost1:m1-1,:,:)=model%rho(      n1-nghost1+1:n1,:,:)
        model%ei(       m1-nghost1:m1-1,:,:)=model%ei(       n1-nghost1+1:n1,:,:)
        model%v1(       m1-nghost1:m1-1,:,:)=model%v1(       n1-nghost1+1:n1,:,:)
        model%v2(       m1-nghost1:m1-1,:,:)=model%v2(       n1-nghost1+1:n1,:,:)
        model%v3(       m1-nghost1:m1-1,:,:)=model%v3(       n1-nghost1+1:n1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(m1-nghost1:m1-1,:,:)=model%phic_grav(n1-nghost1+1:n1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    m1-nghost1:m1-1,:,:,iquc)=model%quc( n1-nghost1+1:n1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
    !
    !============================================================
    case (bound_constant, bound_inoutflow, bound_transmitting, bound_transmitting2, &
          bound_transmitting3)
      ! --- constant ---
      do i1=1,nghost1
        model%rho(  m1-i1,:,:)=model%rho(m1,:,:)
        model%ei(   m1-i1,:,:)=model%ei( m1,:,:)
        model%v1(   m1-i1,:,:)=model%v1( m1,:,:)
        model%v2(   m1-i1,:,:)=model%v2( m1,:,:)
        model%v3(   m1-i1,:,:)=model%v3( m1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(m1-i,:,:)=model%phic_grav(m1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    m1-i1,:,:,iquc)=model%quc( m1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
      !
      end do ! i1
      if ((bound_l == bound_transmitting) .or. (bound_l == bound_transmitting2)) then
        if (present(Hp_l)) then
          Hp_l0=Hp_l
        else
          Hp_l0=0.0
        endif
        if (minval(Hp_l0) > 0.0) then
          ! \label{f90:c_hptopfactor__use1}
          if (par%C_Hptopfactor < 0.0) then
            rhoFac=exp(-abs(                  (model%xc1(m1-1)-model%xc1(m1))/Hp_l0))
          else
            rhoFac=exp(-abs(par%C_Hptopfactor*(model%xc1(m1-1)-model%xc1(m1))/Hp_l0))
          endif
          do i1=1,nghost1
            model%rho(m1-i1,:,:)=model%rho(m1-i1+1,:,:) * rhoFac
          end do ! i1
          ! \label{f90:rho_min__use1}
          if (par%rho_min > 0.0) then
            do i1=1,nghost1
              model%rho(m1-i1,:,:)=max(model%rho(m1-i1,:,:), par%rho_min)
            end do ! i1
          endif
!#
#         if (rhd_box_quc01 != 0)
!#
          do iquc=1,model%nquc
            do i1=1,nghost1
              model%quc(m1-i1,:,:,iquc)=model%quc(m1,:,:,iquc) * &
                                       (model%rho(m1-i1,:,:)/model%rho(m1,:,:))
            end do ! i1
          end do ! iquc
!#
#         endif
!#              /* rhd_box_quc01 */
        endif
      endif
    !
    !============================================================
    case (bound_closed, bound_closedbottom)
      ! --- closed ---
      do i1=1,nghost1
        model%rho(  m1-i1,:,:)= model%rho(m1,:,:)
        model%ei(   m1-i1,:,:)= model%ei( m1,:,:)
        model%v1(   m1-i1,:,:)=-model%v1( m1,:,:)
        model%v2(   m1-i1,:,:)= model%v2( m1,:,:)
        model%v3(   m1-i1,:,:)= model%v3( m1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(m1-i1,:,:)=model%phic_grav(m1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    m1-i1,:,:,iquc)=model%quc( m1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
      end do ! i1
      !
    !============================================================
    case (bound_closed2)
      ! --- Constant/linear/exponential extrapolation; v1: reflective ---
      rhoFac=model%rho(m1,:,:)/model%rho(min(m1+1,n1),:,:)
      ! eiAdd =model%ei( m1,:,:)-model%ei(min(m1+1,n1),:,:)
      do i1=1,nghost1
        model%rho(  m1-i1,:,:)= model%rho(m1-i1+1,:,:) * rhoFac
        model%ei(   m1-i1,:,:)= model%ei( m1-i1+1,:,:)
        model%v1(   m1-i1,:,:)=-model%v1( m1     ,:,:) *sqrt(model%rho(m1,:,:)/model%rho(m1-i1,:,:))
        model%v2(   m1-i1,:,:)= model%v2( m1     ,:,:)
        model%v3(   m1-i1,:,:)= model%v3( m1     ,:,:)
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(m1-i1,:,:,iquc)=model%quc(m1,:,:,iquc) * &
                                   (model%rho(m1-i1,:,:)/model%rho(m1,:,:))
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
        !
      end do ! i1
      !
    !============================================================
    case (bound_closedbottom2)
      ! --- Constant/linear/exponential extrapolation; v1: reflective ---
      rhoFac=max(model%rho(m1,:,:)/model%rho(min(m1+1,n1),:,:), 1.0)
      eiAdd =max(model%ei( m1,:,:)-model%ei( min(m1+1,n1),:,:), 0.0)
      do i1=1,nghost1
        model%rho(  m1-i1,:,:)= model%rho(m1-i1+1,:,:) * rhoFac
        model%ei(   m1-i1,:,:)= model%ei( m1-i1+1,:,:) + eiAdd
        model%v1(   m1-i1,:,:)=-model%v1( m1     ,:,:) *sqrt(model%rho(m1,:,:)/model%rho(m1-i1,:,:))
        model%v2(   m1-i1,:,:)= model%v2( m1     ,:,:)
        model%v3(   m1-i1,:,:)= model%v3( m1     ,:,:)
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(m1-i1,:,:,iquc)=model%quc(m1,:,:,iquc) * &
                                   (model%rho(m1-i1,:,:)/model%rho(m1,:,:))
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
        !
      end do ! i1
      !
    !============================================================
    case (bound_inoutflow2)
      ! --- Constant/linear/exponential extrapolation ---
      rhoFac=max(model%rho(m1,:,:)/model%rho(min(m1+1,n1),:,:), 1.0)
      eiAdd =max(model%ei( m1,:,:)-model%ei( min(m1+1,n1),:,:), 0.0)
!write(6,'(A, 4(1PE14.6))') 'bound_1Dghost; minmax(rhoFac), minmax(eiAdd):', &
!  minval(rhoFac), maxval(rhoFac), minval(eiAdd), maxval(eiAdd)
      do i1=1,nghost1
        model%rho(  m1-i1,:,:)= model%rho(m1-i1+1,:,:) * rhoFac
        model%ei(   m1-i1,:,:)= model%ei( m1-i1+1,:,:) + eiAdd
        model%v1(   m1-i1,:,:)= model%v1( m1     ,:,:)
        model%v2(   m1-i1,:,:)= model%v2( m1     ,:,:)
        model%v3(   m1-i1,:,:)= model%v3( m1     ,:,:)
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(m1-i1,:,:,iquc)=model%quc(m1,:,:,iquc) * &
                                   (model%rho(m1-i1,:,:)/model%rho(m1,:,:))
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
        !
      end do ! i1
      !
    !============================================================
    case (bound_inoutflow3)
      ! --- Constant/linear/exponential extrapolation ---
      rhoFac=max(model%rho(m1,:,:)/model%rho(min(m1+1,n1),:,:), 1.0)
      eiAdd =max(model%ei( m1,:,:)-model%ei( min(m1+1,n1),:,:), 0.0)
      do i1=1,nghost1
        model%rho(  m1-i1,:,:)= model%rho(m1-i1+1,:,:) * rhoFac
        model%ei(   m1-i1,:,:)= model%ei( m1-i1+1,:,:) + eiAdd
        model%v1(   m1-i1,:,:)= model%v1( m1     ,:,:)
        model%v2(   m1-i1,:,:)= model%v2( m1     ,:,:)
        model%v3(   m1-i1,:,:)= model%v3( m1     ,:,:)
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(m1-i1,:,:,iquc)=model%quc(m1,:,:,iquc) * &
                                   (model%rho(m1-i1,:,:)/model%rho(m1,:,:))
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
        !
      end do ! i1
      !
  end select ! (bound_l)
  !
  ! --- 'Right' boundary ---------------------------------------------------------------------------
  !
  select case (bound_r)
    !============================================================
    case (bound_reflective)
      ! --- reflective ---
      do i1=1,nghost1
        model%rho(  n1+i1,:,:)= model%rho(n1-i1+1,:,:)
        model%ei(   n1+i1,:,:)= model%ei( n1-i1+1,:,:)
        model%v1(   n1+i1,:,:)=-model%v1( n1-i1+1,:,:)
        model%v2(   n1+i1,:,:)= model%v2( n1-i1+1,:,:)
        model%v3(   n1+i1,:,:)= model%v3( n1-i1+1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(n1+i1,:,:)= model%phic_grav(n1-i1+1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    n1+i1,:,:,iquc)= model%quc( n1-i1+1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
!
      end do ! i1
      !
    !============================================================
    case (bound_periodic)
      ! --- periodic ---
        model%rho(  n1+1:n1+nghost1,:,:)=model%rho(m1:m1+nghost1-1,:,:)
        model%ei(   n1+1:n1+nghost1,:,:)=model%ei( m1:m1+nghost1-1,:,:)
        model%v1(   n1+1:n1+nghost1,:,:)=model%v1( m1:m1+nghost1-1,:,:)
        model%v2(   n1+1:n1+nghost1,:,:)=model%v2( m1:m1+nghost1-1,:,:)
        model%v3(   n1+1:n1+nghost1,:,:)=model%v3( m1:m1+nghost1-1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(n1+1:n1+nghost1,:,:)=model%phic_grav(m1:m1+nghost1-1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    n1+1:n1+nghost1,:,:,iquc)=model%quc( m1:m1+nghost1-1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
      !
    !============================================================
    case (bound_constant, bound_inoutflow, bound_transmitting, bound_transmitting2, &
          bound_transmitting3)
      ! --- constant ---
      do i1=1,nghost1
        model%rho(  n1+i1,:,:)=model%rho(n1,:,:)
        model%ei(   n1+i1,:,:)=model%ei( n1,:,:)
        model%v1(   n1+i1,:,:)=model%v1( n1,:,:)
        model%v2(   n1+i1,:,:)=model%v2( n1,:,:)
        model%v3(   n1+i1,:,:)=model%v3( n1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(n1+i1,:,:)=model%phic_grav(n1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    n1+i1,:,:,iquc)=model%quc( n1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
      end do ! i1
      if ((bound_r == bound_transmitting) .or. (bound_r == bound_transmitting2)) then
        if (present(Hp_r)) then
          Hp_r0=Hp_r
        else
          Hp_r0=0.0
        endif
        if (minval(Hp_r0) > 0.0) then
          ! \label{f90:c_hptopfactor__use2}
          if (par%C_Hptopfactor < 0.0) then
            rhoFac=exp(-abs(                  (model%xc1(n1+1)-model%xc1(n1))/Hp_r0))
          else
            rhoFac=exp(-abs(par%C_Hptopfactor*(model%xc1(n1+1)-model%xc1(n1))/Hp_r0))
          endif
          do i1=1,nghost1
            model%rho(n1+i1,:,:)=model%rho(n1+i1-1,:,:) * rhoFac
          end do ! i1
          ! \label{f90:rho_min__use2}
          if (par%rho_min > 0.0) then
            do i1=1,nghost1
              model%rho(n1+i1,:,:)=max(model%rho(n1+i1,:,:), par%rho_min)
            end do ! i1
          endif
!#
#         if (rhd_box_quc01 != 0)
!#
          do iquc=1,model%nquc
            do i1=1,nghost1
              model%quc(n1+i1,:,:,iquc)=model%quc(n1,:,:,iquc) * &
                                       (model%rho(n1+i1,:,:)/model%rho(n1,:,:))
            end do ! i1
          end do ! iquc
!#
#         endif
!#              /* rhd_box_quc01 */
        endif
      endif
      !
    !============================================================
    case (bound_closed, bound_closedbottom)
      ! --- closed ---
      do i1=1,nghost1
        model%rho(  n1+i1,:,:)= model%rho(n1,:,:)
        model%ei(   n1+i1,:,:)= model%ei( n1,:,:)
        model%v1(   n1+i1,:,:)=-model%v1( n1,:,:)
        model%v2(   n1+i1,:,:)= model%v2( n1,:,:)
        model%v3(   n1+i1,:,:)= model%v3( n1,:,:)
!#
#       if (rhd_box_grav01 == 1)
!#
        model%phic_grav(n1+i1,:,:)=model%phic_grav(n1,:,:)
!#
#       endif
!#            /* rhd_box_grav01 */
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(    n1+i1,:,:,iquc)=model%quc( n1,:,:,iquc)
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
      end do ! i1
      !
    !============================================================
    case (bound_closed2)
      ! --- Constant/linear/exponential extrapolation; v1: reflective ---
      rhoFac=model%rho(n1,:,:)/model%rho(max(n1-1,m1),:,:)
      ! eiAdd =model%ei( n1,:,:)-model%ei( max(n1-1,m1),:,:)
      do i1=1,nghost1
        model%rho(  n1+i1,:,:)= model%rho(n1+i1-1,:,:) *rhoFac
        model%ei(   n1+i1,:,:)= model%ei( n1+i1-1,:,:)
        model%v1(   n1+i1,:,:)=-model%v1( n1     ,:,:) *sqrt(model%rho(n1,:,:)/model%rho(n1+i1,:,:))
        model%v2(   n1+i1,:,:)= model%v2( n1     ,:,:)
        model%v3(   n1+i1,:,:)= model%v3( n1     ,:,:)
!#
#       if (rhd_box_quc01 != 0)
!#
        do iquc=1,model%nquc
          model%quc(n1+i1,:,:,iquc)=model%quc(n1,:,:,iquc) * &
                                   (model%rho(n1+i1,:,:)/model%rho(n1,:,:))
        end do ! iquc
!#
#       endif
!#            /* rhd_box_quc01 */
        !
      end do ! i1
      !
  end select ! (bound_r)
  !
  ! === Handle magnetic boundary conditions ========================================================
  !
!#
#  if (rhd_box_bmag01 != 0)
!#
  !
  ! --- 'Left' boundary ----------------------------------------------------------------------------
  !
  select case (bound_mag_l)
    case (bound_mag_periodic)
      ! --- periodic ---
        if (model%nBc > 0) then
          model%Bc1(          m1-nghost1:m1-1,:,:)=model%Bc1(          n1-nghost1+1:n1,:,:)
          model%Bc2(          m1-nghost1:m1-1,:,:)=model%Bc2(          n1-nghost1+1:n1,:,:)
          model%Bc3(          m1-nghost1:m1-1,:,:)=model%Bc3(          n1-nghost1+1:n1,:,:)
          model%energy_switch(m1-nghost1:m1-1,:,:)=model%energy_switch(n1-nghost1+1:n1,:,:)
        endif
    !
    !============================================================
    case (bound_mag_constant, bound_mag_inoutflow, bound_mag_fixed)
      ! --- constant ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          m1-i1,:,:)=model%Bc1(          m1,:,:)
          model%Bc2(          m1-i1,:,:)=model%Bc2(          m1,:,:)
          model%Bc3(          m1-i1,:,:)=model%Bc3(          m1,:,:)
          model%energy_switch(m1-i1,:,:)=model%energy_switch(m1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_constant2)
      ! --- constant2 ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          m1-i1,:,:)=model%Bc1(          m1,:,:) + &
                                     merge((model%Bc1(m1+1 ,:,:)-model%Bc1(m1,:,:)) * &
                                           (model%xc1(m1-i1    )-model%xc1(m1    )) / &
                                           (model%xc1(m1+1     )-model%xc1(m1    )), &
                                           0.0, &
                                           ((m1+1) <= n1))
          model%Bc2(          m1-i1,:,:)=model%Bc2(          m1,:,:)
          model%Bc3(          m1-i1,:,:)=model%Bc3(          m1,:,:)
          model%energy_switch(m1-i1,:,:)=model%energy_switch(m1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_vertical)
      ! --- vertical ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          m1-i1,:,:)= model%Bc1(          m1,:,:)
          model%Bc2(          m1-i1,:,:)=-model%Bc2(          m1,:,:)
          model%Bc3(          m1-i1,:,:)=-model%Bc3(          m1,:,:)
          model%energy_switch(m1-i1,:,:)= model%energy_switch(m1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_vertical2)
      ! --- vertical ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          m1-i1,:,:)= model%Bc1(          m1,:,:)
          model%Bc2(          m1-i1,:,:)=0.0
          model%Bc3(          m1-i1,:,:)=0.0
          model%energy_switch(m1-i1,:,:)= model%energy_switch(m1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_reflective)
      ! --- reflective ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          m1-i1,:,:)=-model%Bc1(          m1,:,:)
          model%Bc2(          m1-i1,:,:)= model%Bc2(          m1,:,:)
          model%Bc3(          m1-i1,:,:)= model%Bc3(          m1,:,:)
          model%energy_switch(m1-i1,:,:)= model%energy_switch(m1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_oblique)
      ! --- oblique ---
      fx = tan(par%C_magthetaB)*cos(par%C_magphiB)
      fy = tan(par%C_magthetaB)*sin(par%C_magphiB)
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          m1-i1,:,:)=model%Bc1(          m1,:,:)
          model%Bc2(          m1-i1,:,:)=model%Bc1(          m1,:,:)*fy
          model%Bc3(          m1-i1,:,:)=model%Bc1(          m1,:,:)*fx
          model%energy_switch(m1-i1,:,:)=model%energy_switch(m1,:,:)
        endif
      end do ! i1
      !
  end select ! (bound_mag_l)
  !
  ! --- 'Right' boundary ---------------------------------------------------------------------------
  !
  select case (bound_mag_r)
    case (bound_mag_periodic)
      ! --- periodic ---
        if (model%nBc > 0) then
          model%Bc1(          n1+1:n1+nghost1,:,:)=model%Bc1(          m1:m1+nghost1-1,:,:)
          model%Bc2(          n1+1:n1+nghost1,:,:)=model%Bc2(          m1:m1+nghost1-1,:,:)
          model%Bc3(          n1+1:n1+nghost1,:,:)=model%Bc3(          m1:m1+nghost1-1,:,:)
          model%energy_switch(n1+1:n1+nghost1,:,:)=model%energy_switch(m1:m1+nghost1-1,:,:)
        endif
      !
    !============================================================
    case (bound_mag_constant, bound_mag_inoutflow, bound_mag_fixed)
      ! --- constant ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          n1+i1,:,:)=model%Bc1(          n1,:,:)
          model%Bc2(          n1+i1,:,:)=model%Bc2(          n1,:,:)
          model%Bc3(          n1+i1,:,:)=model%Bc3(          n1,:,:)
          model%energy_switch(n1+i1,:,:)=model%energy_switch(n1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_constant2)
      ! --- constant2 ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          n1+i1,:,:)=model%Bc1(          n1,:,:) + &
                                     merge((model%Bc1(n1   ,:,:)-model%Bc1(n1-1,:,:)) * &
                                           (model%xc1(n1+i1    )-model%xc1(n1      )) / &
                                           (model%xc1(n1       )-model%xc1(n1-1    )), &
                                           0.0, &
                                           ((n1-1) >= m1))
          model%Bc2(          n1+i1,:,:)=model%Bc2(          n1,:,:)
          model%Bc3(          n1+i1,:,:)=model%Bc3(          n1,:,:)
          model%energy_switch(n1+i1,:,:)=model%energy_switch(n1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_vertical)
      ! --- vertical ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          n1+i1,:,:)= model%Bc1(          n1,:,:)
          model%Bc2(          n1+i1,:,:)=-model%Bc2(          n1,:,:)
          model%Bc3(          n1+i1,:,:)=-model%Bc3(          n1,:,:)
          model%energy_switch(n1+i1,:,:)= model%energy_switch(n1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_vertical2)
      ! --- vertical ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          n1+i1,:,:)= model%Bc1(          n1,:,:)
          model%Bc2(          n1+i1,:,:)=0.0
          model%Bc3(          n1+i1,:,:)=0.0
          model%energy_switch(n1+i1,:,:)= model%energy_switch(n1,:,:)
        endif
      end do ! i1
      !
    !============================================================
    case (bound_mag_reflective)
      ! --- reflective ---
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          n1+i1,:,:)=-model%Bc1(           n1,:,:)
          model%Bc2(          n1+i1,:,:)= model%Bc2(           n1,:,:)
          model%Bc3(          n1+i1,:,:)= model%Bc3(           n1,:,:)
          model%energy_switch(n1+i1,:,:)= model%energy_switch( n1,:,:)
        endif
       end do ! i1
       !
    !============================================================
    case (bound_mag_oblique)
      ! --- oblique ---
      fx = tan(par%C_magthetaB)*cos(par%C_magphiB)
      fy = tan(par%C_magthetaB)*sin(par%C_magphiB)
      do i1=1,nghost1
        if (model%nBc > 0) then
          model%Bc1(          n1+i1,:,:)=model%Bc1(          n1,:,:)
          model%Bc2(          n1+i1,:,:)=model%Bc1(          n1,:,:)*fy
          model%Bc3(          n1+i1,:,:)=model%Bc1(          n1,:,:)*fx
          model%energy_switch(n1+i1,:,:)=model%energy_switch(n1,:,:)
        endif
      end do ! i1
      !
  end select ! (bound_mag_r)
!#
#  endif
!#       /* rhd_box_bmag01 */
endif ! (grid0)
!
end subroutine rhd_bound_1DGhost


!----------**********************-------------------------------------------------------------------
subroutine rhd_bound_1DGhostArray(m1,n1, m2,n2, m3,n3, nghost1, q, action)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_1DGhostArray ('rhd_boundary_conditions_1D_Ghost_cells_Array')
!
! PURPOSE:
!   Fill ghost cells of single array according to boundary conditions.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_1DGhostArray(m1,n1, m2,n2, m3,n3, nghost1, q, action)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, optional (def=0), number of ghost cells, 1. dim
!   action:       ('action') derived type with collection of control parameters
!
! INPUT/OUTPUT:
!   q:            ('quantity') 3D array with scalar field
!
! OUTPUT:
!
! ROUTINES:
!   None:
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The number of ghost zones one each side must not exceed the number of interior grid points.
!   The routine cannot handle properly more complex boundary conditions where quantities
!   are treated differently.
!
! PROCEDURE:
!   Possible boundary conditions:
!     no action
!     bound_reflective
!     bound_periodic
!     bound_constant
!     bound_closed
!     bound_closed2
!     bound_inoutflow     (assumed call of rhd_bound3d_inoutflow)
!     bound_inoutflow2    (assumed call of rhd_bound3d_inoutflow)
!     bound_inoutflow3    (assumed call of rhd_bound3d_inoutflow)
!     bound_closedbottom  (with prescribed radiative flux)
!     bound_closedbottom2 (with prescribed radiative flux)
!     bound_transmitting  (constant, rho declines exponentially, with temperature correction)
!     bound_transmitting2 (constant, rho declines exponentially, with temperature correction)
!     bound_transmitting3 (constant, with temperature correction)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-15 (B.F. Kiel) First version
!   1999-04-13 (B.F. Copenhagen) Boundary condition 'closed'
!   1999-05-11 (B.F.) Boundary condition 4
!   1999-09-27 (B.F.) Boundary condition 6
!   2000-03-25 (B.F.) Keywords grid, bulk
!   2001-02-12 (B.F.) Transmitting boundary: exploit equidistant extrapolated grid and save exp call
!   2001-04-09 (B.F.) Factor in front of Hp
!   2002-05-07 (B.F.) Lower limit for boundary density with action%rho_min instead of par%N_test1
!   2002-10-16 (B.F.) "Extrapolation" of phic_grav: mostly constant
!   2002-11-13 (B.F.) bound_transmitting2
!   2004-03-27 (B.F.) Extrapolation of Bb arrays out-commented (index ranges are wrong!)
!   2006-09-12 (W.S. Freiburg) bound_inoutflow2
!   2008-01-05 (B.F.) Derived from bound1d_ghost
!   2010-04-20 (B.F.) Rename bound1d_ghostarray -> rhd_bound_1DGhostArray
!   2010-04-28 (B.F.) bound_transmitting3
!   2011-12-15 (B.F.) bound_inoutflow3
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1
real, dimension(m1-nghost1:n1+nghost1, &
                m2        :n2, &
                m3        :n3           ), &
                           intent(inout):: q
type(action_type),         intent(in)   :: action
!
! --- Local variables ---
integer                                 :: i, bound_l, bound_r
!---------------------------------------------------------------------------------------------------
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
!================================================================================
! --- Bulk data ---
!
! --- 'Left' boundary ---
select case (bound_l)
  !============================================================
  case (bound_reflective)
    ! --- reflective ---
    do i=1,nghost1
      q(      m1-i,:,:)= q(      m1+i-1,:,:)
    end do ! i
  !============================================================
  case (bound_periodic)
    ! --- periodic ---
      q(      m1-nghost1:m1-1,:,:)=q(      n1-nghost1+1:n1,:,:)
  !============================================================
  case (bound_constant, bound_inoutflow, bound_inoutflow2, bound_inoutflow3, &
        bound_transmitting, bound_transmitting2, bound_transmitting3)
    ! --- constant ---
    do i=1,nghost1
      q(      m1-i,:,:)=q(      m1,:,:)
    end do ! i
  !============================================================
  case (bound_closed, bound_closed2, bound_closedbottom, bound_closedbottom2)
    ! --- closed ---
    do i=1,nghost1
      q(      m1-i,:,:)= q(     m1,:,:)
    end do ! i
end select ! bound_l
!
! --- 'Right' boundary ---
select case (bound_r)
  !============================================================
  case (bound_reflective)
    ! --- reflective ---
    do i=1,nghost1
      q(      n1+i,:,:)= q(      n1-i+1,:,:)
    end do ! i
  !============================================================
  case (bound_periodic)
    ! --- periodic ---
      q(      n1+1:n1+nghost1,:,:)=q(      m1:m1+nghost1-1,:,:)
  !============================================================
  case (bound_constant, bound_inoutflow,  bound_inoutflow2, bound_inoutflow3, &
        bound_transmitting, bound_transmitting2, bound_transmitting3)
    ! --- constant ---
    do i=1,nghost1
      q(      n1+i,:,:)=q(      n1,:,:)
    end do ! i
  !============================================================
  case (bound_closed, bound_closed2, bound_closedbottom, bound_closedbottom2)
    ! --- closed ---
    do i=1,nghost1
      q(      n1+i,:,:)= q(     n1,:,:)
    end do ! i
end select ! bound_r
!
end subroutine rhd_bound_1DGhostArray


!----------********************---------------------------------------------------------------------
subroutine rhd_bound_1DTransmit(model, action, dtime, ComputeDelta_flag, grav_flag, phib_grav, &
                                drhov1_l, drhoeik_l, drhov1_r, drhoeik_r, &
                                Hp_l, Hp_r)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_1DTransmit ('rhd_boundary_conditions_1_Dimension_Transmitting')
!
! PURPOSE:
!   Adjust values in top or bottom cells to fulfill the open transmitting boundary condition.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_1DTransmit(model, action, dtime, ComputeDelta_flag, grav_flag, phib_grav, &
!                             drhov1_l, drhoeik_l, drhov1_r, drhoeik_r, &
!                             Hp_l=Hp_l, Hp_r=Hp_r)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!   grav_flag:    ('gravity_flag') logical, false if gravity is zero everywhere
!   phib_grav:    ('phi_boundary_gravity) real 3D array, gravitational potential, cell boundaries
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   drhov1_l:     ('delta_rho_velocity_1_left') real 2D array with 1-momentum change at left side
!   drhoeik_l:    ('delta_rho_energy_internal_kinetic_left) 2D array with energy change at left side
!   drhov1_r:     ('delta_rho_velocity_1_right') real 2D array with 1-momentum change at right side
!   drhoeik_r:    ('delta_rho_energy_internal_kinetic_right) 2D array with energy change at r. side
!   Hp_l:         ('height_pressure_left') real, averaged pressure scale height on left (m1) side
!   Hp_r:         ('height_pressure_right') real, averaged pressure scale height on right (n1) side
!
! ROUTINES:
!   eosinter_PdPTdT:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines for RHD
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!
! SIDE EFFECTS:
!   Modify global timing statistics counters (if activated during compilation).
!
! RESTRICTIONS:
!   There are no OpenMP parallelization directives: the routine is intended to be used inside
!   an OpenMP parallel block.
!
! PROCEDURE:
!   This routine realises part of the open upper or outer boundary condition.
!   Each side is handled separately.
!   First, the EOS is solved for a boundary layers.
!   The hydrostatic pressure scale is computed for further use (extrapolation of the density)
!   by rhd_bound_1DGhost.
!   In inflow cells in a boundary layer the internal energy is modified to come closer towards
!   a specified temperature.
!   Additionally, in inflow cells the inflow velocity may be reduced to increase stability (usually
!   not used).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-09-28 (B.F. Copenhagen) Written
!   1999-09-29 (B.F.) Correct temperature for BOTH boundaries
!   1999-10-01 (B.F.) Characteristic time local
!   2000-03-22 (B.F.) Reduce inflow velocity, parameters Hp_l, Hp_r added
!   2000-03-25 (B.F.) Parameters Hp_l, Hp_r become arrays
!   2000-03-27 (B.F.) Corrections in computation of Hp_l, Hp_r
!   2001-02-06 (B.F.) Put rho, ei, v1 into temporary arrays; call 2D version of eosinter
!                     compute area not with spread and matmul anymore
!   2001-02-13 (B.F.) Hp_l and Hp_r are computed from local P and rho (and phi_grav) only
!   2001-04-09 (B.F.) Formula for cs changed (possible overflow)
!   2008-02-11 (B.F. Lyon) Apply C_visBound drag force in inflow and outflow regions
!   2008-03-11 (B.F.) Remove factor in C_visBound drag force that enhances effect
!                     in high Mach number flows
!   2010-04-28 (B.F.) bound_transmitting3, C_VisBound formula consistent with MHD version
!   2011-12-05 (B.F.) Don't copy density array back: rho remains unchanged anyway
!   2011-12-12 (B.F.) Change call: eosinter_all -> eosinter_PdPTdT
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_bound_t01
!#
use timing_module
!#
# endif
!#
use gasinter_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: ComputeDelta_flag, grav_flag
real, dimension(model%m1-model%nghost1:&
                model%n1+model%nghost1+1,&
                model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(in)   :: phib_grav
real, dimension(model%m2:model%n2, &
                model%m3:model%n3), &
                           intent(out)  :: drhov1_l, drhoeik_l, drhov1_r, drhoeik_r
real, dimension(model%m2:model%n2, &
                model%m3:model%n3), &
      optional,            intent(out)  :: Hp_l, Hp_r
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3
integer                                 :: bound_l, bound_r
real                                    :: T_inflow
real, dimension(model%m2:model%n2, &
                model%m3:model%n3)      :: rho, ei, v1, &
                                           P, dPdei, dPdrho, T, dTdei, cs, v1org, &
                                           time_char, time_T_corr
logical, dimension(model%m2:model%n2, &
                   model%m3:model%n3)   :: mask_inflow
!---------------------------------------------------------------------------------------------------
!
! --- Look for which dimension boundary conditions have to be applied ---
select case (action%dim)
  case (1)
    bound_l=action%bound_l1
    bound_r=action%bound_r1
  case (2)
    bound_l=action%bound_l2
    bound_r=action%bound_r2
  case (3)
    bound_l=action%bound_l3
    bound_r=action%bound_r3
end select
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
! \label{f90:teff__use2}\label{f90:c_tsurf__use1}
! --- Compute target temperature of inflowing material from effective temperature ---
! --- and reduction factor                                                        ---
T_inflow=par%Teff*par%C_Tsurf
!
if ((bound_l == bound_transmitting ) .or. &
    (bound_l == bound_transmitting2) .or. &
    (bound_l == bound_transmitting3)     ) then
  ! === Lower boundary ===
  !
  i1=m1
  !
  rho=model%rho(i1,:,:)
  ei= model%ei( i1,:,:)
  v1= model%v1( i1,:,:)
  !
  ! --- Solve EOS ---
!#
# ifdef rhd_bound_t01
!#
  call timing_start('HYD: bound1d: eosinter')
!#
# endif
!#
  call eosinter_PdPTdT(rho, ei, &
                       P=P, dPdrho=dPdrho, dPde=dPdei, &
                       T=T,                dTde=dTdei)
!#
# ifdef rhd_bound_t01
!#
  call timing_stop( 'HYD: bound1d: eosinter')
!#
# endif
!#
  cs=sqrt(P*(dPdei/rho)/rho + dPdrho)
  !
  ! --- Compute pressure scale height ---
  if (present(Hp_l)) then
    if ((grav_flag) .and. (bound_l /= bound_transmitting3)) then
      Hp_l=P / (abs(phib_grav(i1+1,:,:)-phib_grav(i1,:,:))/(model%xb1(i1+1)-model%xb1(i1)) * rho)
    else
      Hp_l=0.0
    endif
  endif
  ! \label{f90:c_tchange__use1}
  ! --- Compute characteristic time scales ---
  time_char=(model%xb1(i1+1)-model%xb1(i1)) / (0.01*cs + abs(v1))
  time_T_corr=time_char/par%C_TChange
  !
  ! --- Find inflow columns ---
  mask_inflow=(v1 > 0.0)
  !
  ! --- Modify internal energy to come closer to specified inflow temperature ---
  where (mask_inflow)
    ei=ei + dtime/time_T_corr * (T_inflow-T)/dTdei
  end where
  !
  ! \label{f90:c_visbound__use1}
  if (par%C_visBound > 0.0) then
    ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
    v1org=v1
    where (mask_inflow)
      v1=v1 * max( (1.0-par%C_visBound*dtime/(model%xb1(i1+1)-model%xb1(i1))*cs * &
                      (1.0+(abs(v1)/cs)**2)) &
                   ,0.0)
      ei=ei + 0.5 * (v1org**2 - v1**2)
    end where
  endif
  !
  if (ComputeDelta_flag) then
    ! --- Save change in velocity 1 and energy ---
    drhov1_l =model%rho(i1,:,:) *  (v1-model%v1(i1,:,:))
    drhoeik_l=model%rho(i1,:,:) * ((ei-model%ei(i1,:,:)) + 0.5*(v1**2-model%v1(i1,:,:)**2))
  endif
  !
  !model%rho(i1,:,:)=rho
  model%ei( i1,:,:)=ei
  model%v1( i1,:,:)=v1
  !
endif
!
if ((bound_r == bound_transmitting ) .or. &
    (bound_r == bound_transmitting2) .or. &
    (bound_r == bound_transmitting3)     ) then
  ! === Upper boundary ===
  !
  i1=n1
  !
  rho=model%rho(i1,:,:)
  ei= model%ei( i1,:,:)
  v1= model%v1( i1,:,:)
  !
  ! --- Solve EOS ---
!#
# ifdef rhd_bound_t01
!#
  call timing_start('HYD: bound1d: eosinter')
!#
# endif
!#
  call eosinter_PdPTdT(rho, ei, &
                       P=P, dPdrho=dPdrho, dPde=dPdei, &
                       T=T,                dTde=dTdei)
!#
# ifdef rhd_bound_t01
!#
  call timing_stop( 'HYD: bound1d: eosinter')
!#
# endif
!#
  cs=sqrt(P*(dPdei/rho)/rho + dPdrho)
  !
  ! --- Compute pressure scale height ---
  if (present(Hp_r)) then
    if ((grav_flag) .and. (bound_r /= bound_transmitting3)) then
      Hp_r=P / (abs(phib_grav(i1+1,:,:)-phib_grav(i1,:,:))/(model%xb1(i1+1)-model%xb1(i1)) * rho)
    else
      Hp_r=0.0
    endif
  endif
  ! \label{f90:c_tchange__use2}
  ! --- Compute characteristic time scales ---
  time_char=(model%xb1(i1+1)-model%xb1(i1)) / (0.01*cs + abs(v1))
  time_T_corr=time_char/par%C_TChange
  !
  ! --- Find inflow columns ---
  mask_inflow=(v1 < 0.0)
  !
  ! --- Modify internal energy to come closer to specified inflow temperature ---
  where (mask_inflow)
    ei=ei + dtime/time_T_corr * (T_inflow-T)/dTdei
  end where
  !
  ! \label{f90:c_visbound__use2}
  if (par%C_visBound > 0.0) then
    ! --- Apply additional "viscosity" (kind of braking) in inflow cells ---
    v1org=v1
    where (mask_inflow)
      v1=v1 * max( (1.0-par%C_visBound*dtime/(model%xb1(i1+1)-model%xb1(i1))*cs * &
                     (1.0+(abs(v1)/cs)**2)) &
                   ,0.0)
      ei=ei + 0.5 * (v1org**2 - v1**2)
    end where
  endif
  !
  if (ComputeDelta_flag) then
    ! --- Save change in velocity 1 and energy ---
    drhov1_r =model%rho(i1,:,:) *  (v1-model%v1(i1,:,:))
    drhoeik_r=model%rho(i1,:,:) * ((ei-model%ei(i1,:,:)) + 0.5*(v1**2-model%v1(i1,:,:)**2))
  endif
  !
  !model%rho(i1,:,:)=rho
  model%ei( i1,:,:)=ei
  model%v1( i1,:,:)=v1
  !
endif
!
end subroutine rhd_bound_1DTransmit


!----------********************---------------------------------------------------------------------
subroutine rhd_bound_3DTransmit(model, action, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DTransmit ('rhd_boundary_conditions_3_Dimensions_Transmit')
!
! PURPOSE:
!   Adjust density values in top cells to contribute to the open upper boundary condition.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DTransmit(model, action, dtime)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!
! ROUTINES:
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Comm_Split:        Split a communicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Works only for upper (bound_r3) boundary, not for all 6 sides.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_bound_3DTransmit(model, action, 0.5)
!
! MODIFICATION HISTORY:
!   2010-04-05 (B.F.) First Version
!   2010-04-28 (B.F.) bound_transmitting3
!   2012-11-08 (B.F.) Minor modifications
!   2012-11-09 (B.F.) New parameters in call, MPI version
!   2012-11-12 (B.F.) MPI parameters put into prop structure
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_prop_module
use gasinter_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2, i3
real                                    :: P_mean, dPdrho_mean, dPdei_mean, rho_min, rho_max
real(kind=kind(0.0D+00))                :: rho_mean_0, rho_mean_1, ei_mean, v3abs_mean, &
                                           time_char, area_sum, cs
real(kind=kind(0.0D+00)), &
      dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: area
integer                                 :: ModelHasTop_id, rhdMPI_COMM_HDtop, ierrMPI
logical                                 :: ModelHasTop_flag
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! \label{f90:c_rhochangetop__use1}
if (((wg%hyd%bound_r(3) == bound_transmitting ) .or. &
     (wg%hyd%bound_r(3) == bound_transmitting2) .or. &
     (wg%hyd%bound_r(3) == bound_transmitting3)       ) .and. &
    ( par%grav_mode /= 'central')                       .and. &
    ( par%C_rhoChangeTop > 0.0)                                ) then
  i3=n3
  !
  ! --- Determine if there are heating layers in the box ---
  ModelHasTop_flag=(n3 == wg%hyd%n(3))
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  if (ModelHasTop_flag) then
    ModelHasTop_id=1
  else
    ModelHasTop_id=0
  endif
  !
  write(gl%nc_p, '(A,I3,A,1(2X2I4),7X,I1)') '#', wg%hyd%MyProc, &
    ' Heating layer: ', n3, wg%hyd%m(3), ModelHasTop_id
  !
  ! --- Split processes into two groups with and without top-layer cells, respectively ---
  call MPI_Comm_Split(wg%hyd%Comm, ModelHasTop_id, 0, rhdMPI_COMM_HDtop, ierrMPI)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
  rhdMPI_COMM_HDtop=0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
  if (ModelHasTop_flag) then
    ! === Local box contains global top layer: adjust the density there ===
    !
    ! --- Compute relative cell (column) areas ---
    do i2=m2,n2
      do i1=m1,n1
        area(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2))
      end do ! i1
    end do ! i2
    area_sum=sum(area)
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    call MPI_AllReduce(MPI_IN_PLACE, area_sum, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDtop, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    do i2=m2,n2
      do i1=m1,n1
        area(i1,i2)=area(i1,i2)/area_sum
      end do ! i1
    end do ! i2
    !
    ! --- Compute old minimum/maximum density (just for later output to screen) ---
    rho_min=minval(model%rho(:,:,i3))
    rho_max=maxval(model%rho(:,:,i3))
    !
    ! --- Compute area-averaged density, energy, and absolute v3-velocity ---
    rho_mean_0=sum(area*    model%rho(:,:,i3) )
    ei_mean   =sum(area*    model%ei( :,:,i3) )
    v3abs_mean=sum(area*abs(model%v3( :,:,i3)))
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    call MPI_AllReduce(MPI_IN_PLACE, rho_mean_0, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDtop, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, ei_mean   , 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDtop, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, v3abs_mean, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDtop, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Solve EOS for average and compute sound speed ---
    call eosinter(real(rho_mean_0, kind=kind(0.0)), real(ei_mean, kind=kind(0.0)), &
                  P=P_mean, dPdrho=dPdrho_mean, dPde=dPdei_mean)
    cs=sqrt(real(dPdrho_mean, kind=kind(0.0D+00)) + &
            real(dPdei_mean , kind=kind(0.0D+00))*real(P_mean, kind=kind(0.0D+00))/rho_mean_0**2)
    !
    ! --- Estimate a characteristic time scale ---
    time_char=(model%xb3(i3+1)-model%xb3(i3)) / (0.01*(cs+v3abs_mean))
    !
    ! --- Adjust density ---
    model%rho(:,:,i3)=rho_mean_0 + (model%rho(:,:,i3)-rho_mean_0) * &
                                 exp(-dtime/time_char*log(rho_mean_0/model%rho(:,:,i3)+1.0))
    !
    ! --- Compute area-averaged density ---
    rho_mean_1=sum(area*model%rho(:,:,i3))
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    call MPI_AllReduce(MPI_IN_PLACE, rho_mean_1, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDtop, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Correct density to keep overall mass in layer ---
    model%rho(:,:,i3)=model%rho(:,:,i3)*(rho_mean_0/rho_mean_1)
    !
    ! --- Print result ---
    write(gl%nc_p, '(A,I3,2(A,1PE13.6,1X,1PE13.6))') &
         '#', wg%hyd%MyProc, &
         ' 3DTransmit: min/max rho: ', rho_min, rho_max, &
         ' => ', minval(model%rho(:,:,i3)), maxval(model%rho(:,:,i3))
    !
  endif ! (ModelHasTop_flag)
endif ! (wg%hyd%bound_r(3))
!
end subroutine rhd_bound_3DTransmit


!----------*********************--------------------------------------------------------------------
subroutine rhd_bound_3DInOutFlow(model, action, dtime)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DInOutFlow ('rhd_boundary_conditions_3_Dimensions_In_Out_Flow')
!
! PURPOSE:
!   Adjust values in bottom cells to fulfill the open lower boundary condition.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DInOutFlow(model, action, dtime)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!
! ROUTINES:
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Comm_Split:        Split a communicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   This routine realizes the open lower boundary condition for models of the surface layers
!   deep solar-type convection zones.
!   The values of density, internal energy and velocity in the deepest layer of the model
!   are modified.
!   The extrapolation into the ghost cells at the bottom (later by rhd_bound_1DGhost) should be
!   'constant'.
!   The entropy in the upward streaming cells is modified to push its value closer
!   to the value prescribed by s_InFlow.
!   The densities are adjusted to give a pressure closer to the mean value with no net mass change.
!   The velocities are modified to give zero mass flux.
!
!   New corrections:
!     drho  =  drho/dP|s         * dP  +  drho/ds|P                        * ds
!     dei   =   dei/dP|s         * dP  +   dei/ds|P                        * ds
!   =>
!     drho  =  rho/(Gamma_1*P)   * dP  +  (-rho^2*T/P*(Gamma_3-1)/Gamma_1) * ds
!     dei   =    1/(Gamma_1*rho) * dP  +  T(1-(Gamma_3-1)/Gamma_1))        * ds
!
! EXAMPLE:
!   call rhd_bound_3DInOutFlow(model, action, 0.5)
!
! MODIFICATION HISTORY:
!   1999-05-11 (B.F. Copenhagen) Written
!   1999-05-17 (B.F.) Mass flux correction at end
!   1999-05-19 (B.F.) Correct effect of pressure correction for conserving the mass in bottom layer
!   2000-03-22 (B.F.) Density and energy corrections depend both on pressure and entropy
!   2000-12-14 (B.F.) Pressure correction: ei: time_s_corr => time_P_corr
!   2003-03-06 (M.S.) Pressure forced to be plane-parallel at all times if C_PChange < 0
!   2004-03-03 (B.F.) Area computation without matmul because of Sun compiler problem
!   2004-04-19 (S.W-B, B.F.) case quc_bound=2, scale quc values to new mass density
!   2007-01-27 (B.F. Lyon) Modification of prop structure removed
!   2010-03-24 (B.F. Naples) Bugfix against loss/gain of total mass: new brackets in update formula
!                     normalize area in a separate step; some local variables double precision
!   2011-12-15 (B.F. Lyon) Experimental branch for reduction of vertical velocity amplitude
!   2011-12-27 (B.F.) New Formula for velocity damping
!   2012-10-25 (B.F.) Change type of eps_small
!   2012-11-09 (B.F.) New parameters in call, MPI version, remove timing output
!   2012-11-12 (B.F.) MPI parameters put into prop structure
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use rhd_box_module
use rhd_prop_module
use gasinter_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i1, i2
real(kind=kind(0.0D+00))                :: rhov3_mean, rho_mean_0, rho_mean_1, rho_mean_2, &
                                           v3_correction, P_mean_0, P_mean_1, csv3abs_mean, &
                                           time_char, frac_s_corr, frac_P_corr, area_sum, v3_rms, &
                                           Pocs2_mean, Oocs2_mean
real(kind=kind(0.0D+00)), &
      dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: area
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: P, dPdei, dPdrho, T, dTdei, s, &
                                           Gamma_1, Gamma_3, cs
logical, dimension(model%m1:model%n1, &
                   model%m2:model%n2)   :: mask_inflow
!#
# if (rhd_box_quc01 != 0)
!#
real, dimension(model%m1:model%n1, &
                model%m2:model%n2)      :: rho_old
integer                                 :: iquc, iqucbound2
!#
# endif
!#      /* rhd_box_quc01 */
real(kind=kind(0.0D+00)), parameter     :: eps_small=1.0D+08*tiny(0.0)
integer                                 :: ModelHasBottom_id, rhdMPI_COMM_HDbottom, ierrMPI
logical                                 :: ModelHasBottom_flag
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Determine if there are heating layers in the box ---
ModelHasBottom_flag=(m3 == wg%hyd%m(3))
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
if (ModelHasBottom_flag) then
  ModelHasBottom_id=1
else
  ModelHasBottom_id=0
endif
!
write(gl%nc_p, '(A,I3,A,1(2X2I4),7X,I1)') '#', wg%hyd%MyProc, &
  ' Heating layer: ', m3, wg%hyd%m(3), ModelHasBottom_id
!
! --- Split processes into two groups with and without bottom-layer cells, respectively ---
call MPI_Comm_Split(wg%hyd%Comm, ModelHasBottom_id, 0, rhdMPI_COMM_HDbottom, ierrMPI)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
rhdMPI_COMM_HDbottom=0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ModelHasBottom_flag) then
  ! === Local box contains global bottom layer ===
  !
!#
# if (rhd_box_quc01 != 0)
!#
  if ((model%nquc > 0) .and. (associated(prop%quc_bound))) then
    iqucbound2=0
    do iquc=1,prop%nquc
      if (prop%quc_bound(iquc) == 2) then
        iqucbound2=1
      endif
    end do ! iquc
    !
    ! --- Save initial density in bottom layer ---
    if (iqucbound2 == 1) then
      rho_old=model%rho(m1:n1,m2:n2,m3)
    endif
  endif
!#
# endif
!#      /* rhd_box_quc01 */
  !
  ! --- Compute relative cell (column) areas ---
  do i2=m2,n2
    do i1=m1,n1
      area(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2))
    end do ! i1
  end do ! i2
  area_sum=sum(area)
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  call MPI_AllReduce(MPI_IN_PLACE, area_sum, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                     rhdMPI_COMM_HDbottom, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  do i2=m2,n2
    do i1=m1,n1
      area(i1,i2)=area(i1,i2)/area_sum
    end do ! i1
  end do ! i2
  !
  ! --- Solve EOS ---
  call eosinter_all(model%rho(:,:,m3), model%ei(:,:,m3), &
                    P=P, dPdrho=dPdrho, dPde=dPdei, &
                    T=T,                dTde=dTdei, &
                    s=s)
  !
  ! --- Derive further quantities ---
  Gamma_1=dPdrho*model%rho(:,:,m3)/P + dPdei/model%rho(:,:,m3)
  Gamma_3=dPdei/model%rho(:,:,m3) + 1.0
  cs     =sqrt(P/model%rho(:,:,m3)*Gamma_1)
  !
  ! --- Compute area-averaged density and pressure ---
  rho_mean_0  =sum(area*model%rho(m1:n1,m2:n2,m3))
  P_mean_0    =sum(area*P)
  csv3abs_mean=sum(area*(cs + abs(model%v3(:,:,m3))))
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  call MPI_AllReduce(MPI_IN_PLACE, rho_mean_0  , 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                                 rhdMPI_COMM_HDbottom, ierrMPI)
  call MPI_AllReduce(MPI_IN_PLACE, P_mean_0    , 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                                 rhdMPI_COMM_HDbottom, ierrMPI)
  call MPI_AllReduce(MPI_IN_PLACE, csv3abs_mean, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                                 rhdMPI_COMM_HDbottom, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  ! \label{f90:c_schange__use1}\label{f90:c_pchange__use1}
  ! --- Compute characteristic time scales (scalars) ---
  time_char  =(model%xb3(m3+1)-model%xb3(m3))/csv3abs_mean
  frac_s_corr=dtime/time_char*par%C_sChange
  frac_P_corr=dtime/time_char*par%C_PChange
  !
  ! === Entropy ===================================================================
  !
  ! --- Find inflow columns ---
  mask_inflow=(model%v3(m1:n1,m2:n2,m3) > 0.0)
  !
  ! \label{f90:s_inflow__use1}
  ! --- Modify density and internal energy to come closer to specified inflow entropy ---
  ! --- Change density and internal energy at constant pressure (upflows only)        ---
  where (mask_inflow)
    model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + frac_s_corr * &
                       (-(model%rho(m1:n1,m2:n2,m3)**2)*T/P*(Gamma_3-1.0)/Gamma_1) *(par%s_InFlow-s)
    model%ei( m1:n1,m2:n2,m3)=model%ei( m1:n1,m2:n2,m3) + frac_s_corr * &
                        T*(1.0-(Gamma_3-1.0)/Gamma_1)                              *(par%s_InFlow-s)
  end where
  !
  ! \label{f90:c_pchange__use2}
  ! === Pressure ==================================================================
  if (par%C_PChange < 0.0) then
    ! === Find plane-parallel pressure such that total change of area-averaged density vanishes ===
    !
    ! --- Compute new area-averaged density and other quantities ---
    rho_mean_1=sum(area*model%rho(m1:n1,m2:n2,m3))
    Pocs2_mean=sum(area*(P  /cs**2))
    Oocs2_mean=sum(area*(1.0/cs**2))
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    call MPI_AllReduce(MPI_IN_PLACE, rho_mean_1, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                                 rhdMPI_COMM_HDbottom, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Pocs2_mean, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                                 rhdMPI_COMM_HDbottom, ierrMPI)
    call MPI_AllReduce(MPI_IN_PLACE, Oocs2_mean, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                                 rhdMPI_COMM_HDbottom, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Compute mean plane-parallel pressure ---
    P_mean_1   =(rho_mean_0-rho_mean_1 + Pocs2_mean)/Oocs2_mean
    frac_P_corr=1.0
  else
    P_mean_1   =P_mean_0
  endif
  !
  ! --- Modify density and energy to shift local pressure closer to average ---
  model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + &
                            frac_P_corr * (P_mean_1-P)/cs**2
  model%ei( m1:n1,m2:n2,m3)=model%ei( m1:n1,m2:n2,m3) + &
                            frac_P_corr * (P_mean_1-P)/(Gamma_1*model%rho(m1:n1,m2:n2,m3))
  !
  ! === Density ===================================================================
  !
  ! --- Compute new area-averaged density after corrections ---
  rho_mean_2=sum(area*model%rho(m1:n1,m2:n2,m3))
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  call MPI_AllReduce(MPI_IN_PLACE, rho_mean_2, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                               rhdMPI_COMM_HDbottom, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  ! --- Correct density to give same overall mass in bottom layer ---
  model%rho(m1:n1,m2:n2,m3)=model%rho(m1:n1,m2:n2,m3) + (rho_mean_0-rho_mean_2)
  !
  ! === Velocity ==================================================================
  !
  if ((par%C_v3ChangeLinBottom > 0.0) .or. (par%C_v3ChangeSqrBottom > 0.0)) then
    ! === Reduce the overall velocity amplitude ===
    !
    ! --- Compute the overall rms value of v3 ---
    v3_rms=sum(area*model%v3(m1:n1,m2:n2,m3)**2)
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    call MPI_AllReduce(MPI_IN_PLACE, v3_rms, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                             rhdMPI_COMM_HDbottom, ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    v3_rms=sqrt(v3_rms)
    ! \label{f90:c_v3changelinbottom__use1}
    ! --- Reduce the amplitude of v3 ---
    model%v3(m1:n1,m2:n2,m3)=model%v3(m1:n1,m2:n2,m3) - &
                             model%v3(m1:n1,m2:n2,m3) * &
                               min((dtime/time_char) * &
                                     (par%C_v3ChangeLinBottom + &
                                      par%C_v3ChangeSqrBottom* &
                                        (abs(model%v3(m1:n1,m2:n2,m3))/(v3_rms+eps_small))), &
                                   1.0)
  endif
  !
  ! --- Mass flux correction: Modify vertical velocity to give overall mass flux = 0 ---
  ! --- Average mass flux through bottom cell layer ---
  rhov3_mean=sum(area*(model%rho(m1:n1,m2:n2,m3)*model%v3(m1:n1,m2:n2,m3)))
  !#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  call MPI_AllReduce(MPI_IN_PLACE, rhov3_mean, 1, MPI_DOUBLE_PRECISION, MPI_SUM, &
                                               rhdMPI_COMM_HDbottom, ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  ! --- Compute the correction velocity ---
  v3_correction=rhov3_mean/rho_mean_0
  !
  ! --- Apply the correction ---
  model%v3(m1:n1,m2:n2,m3)=model%v3(m1:n1,m2:n2,m3)-v3_correction
  !
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Correct additional quantities to keep concentrations constant ---
  if ((model%nquc > 0) .and. (associated(prop%quc_bound))) then
    do iquc=1,prop%nquc
      if (prop%quc_bound(iquc) .eq. 2) then
        model%quc(m1:n1,m2:n2,m3,iquc)=model%quc(m1:n1,m2:n2,m3,iquc) * &
                                       model%rho(m1:n1,m2:n2,m3     )/rho_old
      endif
    end do ! iquc
  endif
!#
# endif
!#      /* rhd_box_quc01 */
endif
!
end subroutine rhd_bound_3DInOutFlow


!----------******************-----------------------------------------------------------------------
subroutine rhd_bound_3DCenter(model, action, dtime, luminosity)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DCenter ('rhd_boundary_conditions_3_Dimensions_Center')
!
! PURPOSE:
!   Modify energy in central cells to keep entropy constant.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DCenter(model, action, dtime, luminosity)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   luminosity:   ('luminosity') real, luminosity per core volume [erg/cm^3/s]
!
! ROUTINES:
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Comm_Free:         Free a communicator
!   MPI_Comm_Split:        Split a communicator
!   MPI_Reduce:            Perform a reduction
!   rhd_MPI_AllReduceReals: Reduce and broadcast a number of real scalars in one go
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_MPI_module:        MPI routines specific for RHD code
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   So far, the routines has to be called only for processes that have hydro data.
!   A lateral core-entropy gradient always uses the vertical axis as reference (and not
!   what is specified in the rotation-axis parameter).
!
! PROCEDURE:
!   The internal energy in the core cells is modified to push their entropy closer
!   to the value prescribed by s_InFlow.
!   Alternatively, the internal energy in the core cells is incremented by a fixed amount.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-06-10 (B.F. Copenhagen) Written
!   1999-12-26 (B.F.) Luminosity now per dtime
!   2000-01-24 (B.F.) Use rhd_const_module for pi
!   2001-02-13 (B.F. Uppsala) Use allocatable instead of pointer arrays
!   2002-04-11 (B.F.) Rearrangements,
!                     parameter par%C_test2 used to specify luminosity per core volume
!   2002-05-07 (B.F.) par%C_test2 -> par%LuminosityPerVolume
!   2003-02-13 (B.F.) Core drag force
!   2004-02-27 (B.F.) par%C_test1 -> par%C_CoreDrag
!   2011-06-30 (W.S.) Core radius can be specified independently from inner smoothing
!                     radius of gravity potential, new heating mode core_energy2, apply
!                     heating mode core_entropy1 for LuminosityPerVolume <= 0.0 only when
!                     heat mode not specified
!   2012-07-09 (B.F. Lyon) Account for option hdCoreDragProfile='Linear'
!   2012-10-11 (B.F.) Swap order of loops
!   2012-10-12 (B.F.) Add +-1 in bounding box determination; MPI (+OpenMP) version;
!                     remove timing('HYD: bound3dc: eosinter');
!                     core_energy2: shrink affected radius by sqrt(5.0)
!   2012-10-23 (B.F.) New parameters in call, MPI version
!   2012-11-12 (B.F.) MPI parameters put into prop structure
!   2013-04-10 (B.F.) Bugfix: make CoreDragProfile='Linear' really linear.
!                     CoreDragProfile='Cosine', 'CosSqr'
!   2014-03-01 (B.F.. Uppsala) Account for s_InFlGrad in case 'core_entropy1'
!   2014-03-17 (B.F.) Correct OpenMP calls (missing $), rearrange drag-force loops
!   2014-04-08 (B.F.) Account for hdCoreHeatProfile
!   2014-04-11 (B.F.) Handle 'core_energyentropy1', correctly normalize volume array
!   2014-06-11 (B.F.) Re-implement the constant drag profile; first radial-velocity-only version
!   2014-06-12 (B.F.) par%hdCoreDragProfile = '...-Radial'
!   2014-06-13 (B.F.) par%hdCoreDragProfile = '...-Meridional'; s_InFlGrad: r->r^2 dependence
!   2014-06-16 (B.F.) New parameters s_InFlRadGrad, s_InFlLatGrad replace s_InFlGrad
!   2014-06-20 (B.F.) Replace par%r0_grav by r_core in computation of invtime_s_corr ->
!                     par%C_CoreDrag has to be adjusted
!   2017-11-17 (B.F.) Add option par%hdCoreDragProfile = 'Constant-Radial-vTrans'
!   2017-11-23 (B.F.) Add option par%hdCoreDragProfile = 'Constant-All-vTrans'
!   2018-06-11 (B.F.) New parameter C_gravCorer0Heat=r0_core/r0_grav replaces r0_core
!---------------------------------------------------------------------------------------------------
use const_module, only: pi
use rhd_gl_module
use rhd_action_module
use rhd_prop_module
use rhd_box_module
use gasinter_module
use rhd_MPI_module
!$ use omp_lib
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: luminosity
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           m1i, n1i, m2i, n2i, m3i, n3i, &
                                           i1, i2, i3, &
                                           ModelHasCore_id, &
                                           rhdMPI_COMM_HDcore, ierrMPI
real                                    :: gridsize, dxb3, dxb23, rhoCore_mean, sCore_mean, &
                                           rhov1Core_mean, rhov2Core_mean, rhov3Core_mean, &
                                           v1Core, v2Core, v3Core, &
                                           r_core, C, x, dei0, drhoeiodtime0, v_factor_drag, &
                                           time_s_corr, invtime_v_corr, mylum, luminosityTotal, &
                                           volumeCoreTotal, &
                                           vRadialOverx, vRadial1, vRadial2, vRadial3, &
                                           vRadial1_mean, vRadial2_mean, vRadial3_mean
real,    allocatable, dimension(:)      :: P, dPdrho, dPdei, dTdei
real,    allocatable, dimension(:,:,:)  :: radius2, volume, T, s, cs
logical, allocatable, dimension(:,:,:)  :: mask_core
logical                                 :: ModelHasCore_flag
real                                    :: sin2theta, s_InFlEff
real, parameter                         :: eps_small=1.0E+02*tiny(0.0), C_vTransFactor=50.0
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! \label{f90:r0_grav__use2}\label{f90:c_gravcorer0heat__use1}
! --- Set core radius ---
if (par%C_gravCorer0Heat > 0.0) then
  r_core=par%C_gravCorer0Heat*par%r0_grav
else
  r_core=par%r0_grav
endif
!
! --- Index range for core bounding box ---
m1i=n1+1
do i1=m1,n1
  if (model%xc1(i1) > -r_core) then
    m1i=i1
    exit
  endif
end do ! i1
n1i=m1-1
do i1=n1,m1,-1
  if (model%xc1(i1) < r_core) then
    n1i=i1
    exit
  endif
end do ! i1
m2i=n2+1
do i2=m2,n2
  if (model%xc2(i2) > -r_core) then
    m2i=i2
    exit
  endif
end do ! i2
n2i=m2-1
do i2=n2,m2,-1
  if (model%xc2(i2) < r_core) then
    n2i=i2
    exit
  endif
end do ! i2
m3i=n3+1
do i3=m3,n3
  if (model%xc3(i3) > -r_core) then
    m3i=i3
    exit
  endif
end do ! i3
n3i=m3-1
do i3=n3,m3,-1
  if (model%xc3(i3) < r_core) then
    n3i=i3
    exit
  endif
end do ! i3
!
! --- Determine if the core bounding box is non-empty ---
ModelHasCore_flag=(n1i >= m1i) .and. (n2i >= m2i) .and. (n3i >= m3i)
if (ModelHasCore_flag) then
  ModelHasCore_id=1
else
  ModelHasCore_id=0
endif
!
! write(gl%nc_p, '(A,I3,A,3(2X2I4),7X,I1)') '#', wg%hyd%MyProc, &
!   ' Core bounding box: ', m1i,n1i, m2i,n2i, m3i,n3i, ModelHasCore_id
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Split processes into two groups with and without core cells, respectively ---
call MPI_Comm_Split(wg%hyd%Comm, ModelHasCore_id, 0, rhdMPI_COMM_HDcore, ierrMPI)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
rhdMPI_COMM_HDcore=0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ModelHasCore_flag) then
  !
  ! --- Allocate temporary arrays ---
  allocate(radius2(  m1i:n1i,m2i:n2i,m3i:n3i), &
           volume(   m1i:n1i,m2i:n2i,m3i:n3i), &
           T(        m1i:n1i,m2i:n2i,m3i:n3i), &
           s(        m1i:n1i,m2i:n2i,m3i:n3i), &
           cs(       m1i:n1i,m2i:n2i,m3i:n3i), &
           mask_core(m1i:n1i,m2i:n2i,m3i:n3i))
  !
  ! --- Compute the typical grid size ---
  !gridsize=max((model%xb1(n1+1)-model%xb1(m1))/(n1-m1+1), &
  !             (model%xb2(n2+1)-model%xb2(m2))/(n2-m2+1), &
  !             (model%xb3(n3+1)-model%xb3(m3))/(n3-m3+1))
  gridsize=max((wg%hyd%xb_r(1)-wg%hyd%xb_l(1))/(wg%hyd%n(1)-wg%hyd%m(1)+1), &
               (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))/(wg%hyd%n(2)-wg%hyd%m(2)+1), &
               (wg%hyd%xb_r(3)-wg%hyd%xb_l(3))/(wg%hyd%n(3)-wg%hyd%m(3)+1))
  !
  volumeCoreTotal=0.0
  !
  !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(P, dPdrho, dPdei, dTdei)
  !
  ! --- Compute the square of distance to center, relative volume element ---
  !$OMP DO PRIVATE(dxb3, dxb23, i1, i2) REDUCTION(+: volumeCoreTotal)
  do i3=m3i,n3i
    dxb3=(model%xb3(i3+1)-model%xb3(i3))/(4.0/3.0*pi*r_core)
    do i2=m2i,n2i
      dxb23=(model%xb2(i2+1)-model%xb2(i2))/r_core * dxb3
      do i1=m1i,n1i
        radius2(  i1,i2,i3)=model%xc1(i1)**2 + model%xc2(i2)**2 + model%xc3(i3)**2
        volume(   i1,i2,i3)=(model%xb1(i1+1)-model%xb1(i1)) / r_core * dxb23
        ! --- Construct a mask that indicates affiliation to core ---
        mask_core(i1,i2,i3)=(radius2(i1,i2,i3) <= r_core**2)
        if (mask_core(i1,i2,i3)) then
          volumeCoreTotal=volumeCoreTotal+volume(i1,i2,i3)
        endif
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  ! write(gl%nc_p, '(A,I3,A,1PE14.6,2(3X,2(1PE14.6)))') '#', wg%hyd%MyProc, &
  !   ' grid, minmax(radius2, volume): ', gridsize, &
  !   minval(radius2), maxval(radius2), &
  !   minval(volume), maxval(volume)
  !
  if ((par%LuminosityPerVolume <= 0.0        ) .or. &
      (par%heat_mode == 'core_energyentropy1') .or. &
      (par%heat_mode == 'core_entropy1'      ) .or. &
      (par%C_CoreDrag > 0.0                  )     ) then
    allocate(P(m1i:n1i), dPdrho(m1i:n1i), dPdei(m1i:n1i), dTdei(m1i:n1i))
    !
    ! --- Solve EOS ---
    !$OMP DO PRIVATE(i2)
    do i3=m3i,n3i
      do i2=m2i,n2i
        call eosinter(model%rho(m1i:n1i,i2,i3), model%ei(m1i:n1i,i2,i3), &
                      P=P, dPdrho=dPdrho, dPde=dPdei, &
                      T=T(:,i2,i3),       dTde=dTdei, &
                      s=s(:,i2,i3))
        ! --- Compute sound speed ---
        cs(:,i2,i3)=sqrt(P*dPdei/model%rho(m1i:n1i,i2,i3)**2+dPdrho)
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    deallocate(P, dPdrho, dPdei, dTdei)
    !
    ! write(gl%nc_p, '(A,I3,A,4(3X,2(1PE14.6)))') '#', wg%hyd%MyProc, &
    !   ' minmax(ei,T,s,cs): ', &
    !   minval(model%ei), maxval(model%ei), &
    !   minval(T) , maxval(T), &
    !   minval(s) , maxval(s), &
    !   minval(cs), maxval(cs)
    !
  endif
  !
  !$OMP END PARALLEL
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Sum over all processes (that potentially have core data) ---
  call MPI_AllReduce(MPI_IN_PLACE, volumeCoreTotal, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDcore, &
                     ierrMPI)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
  !
  !$OMP PARALLEL DEFAULT(SHARED)
  !
  !$OMP DO PRIVATE(i1, i2)
  do i3=m3i,n3i
    do i2=m2i,n2i
      do i1=m1i,n1i
        volume(i1,i2,i3)=volume(i1,i2,i3)/volumeCoreTotal
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  !$OMP END PARALLEL
  !
  if (gl%nc_m >= 0) then
    write(gl%nc_m,'(A,1PE15.8)') 'Core volume factor:   ', volumeCoreTotal
  endif
  !
  ! ### Compute energy production in core ##########################################################
  !
  ! \label{f90:heat_mode__use1}
  ! --- Handle the different heating modes ---
  if (par%LuminosityPerVolume <= 0.0 .or. par%heat_mode == 'core_entropy1') then
    ! === Adjust core entropy (via internal energy) ================================================
    !
    luminosity=0.0
    !
    if ((par%s_InFlRadGrad == 0.0) .and. (par%s_InFlLatGrad == 0.0)) then
      ! --- Classic case of isentropic core ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, time_s_corr, dei0) REDUCTION(+: luminosity)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! \label{f90:c_schange__use2}
              ! --- Compute characteristic time scale ---
              time_s_corr=(gridsize/par%C_sChange)/cs(i1,i2,i3)
              !
              ! \label{f90:s_inflow__use2}
              ! --- Get change in internal energy to come closer to ---
              ! --- specified core (inflow) entropy                 ---
              dei0=1.0/time_s_corr * T(i1,i2,i3) * (par%s_InFlow-s(i1,i2,i3))
              !
              ! --- Add change in internal energy ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + dei0*dtime
              !
              ! --- Compute luminosity per volume (core radius) ---
              luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else ! ((par%s_InFlRadGrad /= 0.0) .or. (par%s_InFlLatGrad /= 0.0))
      ! --- Account for entropy gradient with latitude ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, time_s_corr, dei0) REDUCTION(+: luminosity)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute sin^2(latitude) ---
              sin2theta=model%xc3(i3)**2/(radius2(i1,i2,i3)+eps_small)
              !
              ! \label{f90:s_inflow__use3}\label{f90:s_inflradgrad__use1}\label{f90:s_infllatgrad__use1}
              ! --- Compute local target entropy ---
              s_InFlEff=par%s_InFlow + &
                    (sqrt(radius2(i1,i2,i3)/r_core**2)-1.0)                   * par%s_InFlRadGrad+ &
                    (radius2(i1,i2,i3)/r_core**2          ) * (0.5-sin2theta) * par%s_InFlLatGrad
              !
              ! --- Compute characteristic time scale ---
              time_s_corr=(gridsize/par%C_sChange)/cs(i1,i2,i3)
              !
              ! --- Get change in internal energy to come closer to specified target entropy ---
              dei0=1.0/time_s_corr * T(i1,i2,i3) * (s_InFlEff-s(i1,i2,i3))
              !
              ! --- Add change in internal energy ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + dei0*dtime
              !
              ! --- Compute luminosity per volume (core radius) ---
              luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    endif ! (par%s_InFlRadGrad, par%s_InFlLatGrad)
    !
    ! write(gl%nc_p, '(A,I3,A,1(3X,2(1PE14.6)),2(3X,1PE14.6))') '#', wg%hyd%MyProc, &
    !   ' minmax(ei), lumin., dtime:', &
    !   minval(model%ei), maxval(model%ei), luminosity, dtime
    !
  else if (par%heat_mode == 'core_energy2') then
    ! === Add energy in core =======================================================================
    ! \label{f90:luminositypervolume__use1}
    !C=4.0 * par%LuminosityPerVolume / (3.0 * sqrt(pi))
    C=4.0 * 5.0 *sqrt(5.0) * par%LuminosityPerVolume / (3.0 * sqrt(pi))
    luminosity=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, x, dei0) REDUCTION(+: luminosity)
    do i3=m3i,n3i
      do i2=m2i,n2i
        do i1=m1i,n1i
          !
          !x = (model%xc1(i1)**2+model%xc2(i2)**2+model%xc3(i3)**2)/r_core**2
          !if (x < 5.0) then
          !
          ! --- Compute change in internal energy in core ---
          x = 5.0*(model%xc1(i1)**2+model%xc2(i2)**2+model%xc3(i3)**2)/r_core**2
          if (x < 1.0) then
            dei0 = C * exp(-x) / model%rho(i1,i2,i3)
          else
            dei0 = 0.0
          endif
          !
          ! --- Add the change in internal energy ---
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dei0*dtime
          !
          ! --- Compute luminosity per volume (core radius) ---
          ! --- Usually luminosity=par%LuminosityPerVolume, but it's better to check... ---
          !luminosity=luminosity + model%rho(i1,i2,i3)*dei0 * (3.0 / (4.0 * pi)) * &
          !           ((model%xb1(i1+1) - model%xb1(i1  )) / r_core) * &
          !           ((model%xb2(i2+1) - model%xb2(i2  )) / r_core) * &
          !           ((model%xb3(i3+1) - model%xb3(i3  )) / r_core)
          luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  else if (par%heat_mode == 'core_energyentropy1') then
    ! === Add energy in core, adjust entropy towards average in core ===============================
    !
    ! --- Compute average core entropy ---
    !
    sCore_mean=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2) REDUCTION(+: sCore_mean)
    do i3=m3i,n3i
      do i2=m2i,n2i
        do i1=m1i,n1i
          if (mask_core(i1,i2,i3)) then
            sCore_mean=sCore_mean+volume(i1,i2,i3)*s(i1,i2,i3)
          endif
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Sum over all processes (that potentially have core data) ---
    call MPI_AllReduce(MPI_IN_PLACE, sCore_mean, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDcore, &
                       ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! --- Adjust entropy in core towards the mean in core and ---
    ! --- compute resulting luminosity per core volume        ---
    !
    luminosity=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, time_s_corr, dei0) REDUCTION(+: luminosity)
    do i3=m3i,n3i
      do i2=m2i,n2i
        do i1=m1i,n1i
          if (mask_core(i1,i2,i3)) then
            ! --- Compute characteristic time scale ---
            time_s_corr=(gridsize/par%C_sChange)/cs(i1,i2,i3)
            !
            ! --- Get change in internal energy to come closer to specified core (inflow) entropy --
            dei0=1.0/time_s_corr * T(i1,i2,i3) * (sCore_mean-s(i1,i2,i3))
            !
            ! --- Add change in internal energy ---
            model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + dei0*dtime
            !
            ! --- Compute luminosity per volume (core radius) ---
            luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0
          endif
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Sum over all processes (that potentially have core data) ---
    call MPI_AllReduce(MPI_IN_PLACE, luminosity, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDcore, &
                       ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    luminosityTotal=par%LuminosityPerVolume-luminosity
    ! \label{f90:hdcoreheatprofile__use1}
    if (par%hdCoreHeatProfile == 'Constantdrhoei') then
      !
      luminosity=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, drhoeiodtime0) REDUCTION(+: luminosity)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            ! --- Compute change in internal energy ---
            if (mask_core(i1,i2,i3)) then
              drhoeiodtime0=luminosityTotal
              !
              ! --- Add change in internal energy ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dtime*drhoeiodtime0/model%rho(i1,i2,i3)
              !
              ! --- Compute luminosity per volume (from core radius) ---
              luminosity=luminosity + volume(i1,i2,i3)*par%LuminosityPerVolume
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
    else ! (par%hdCoreHeatProfile == 'Constantdei')
      !
      ! --- Compute average core density ---
      !
      rhoCore_mean=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2) REDUCTION(+: rhoCore_mean)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              rhoCore_mean=rhoCore_mean+volume(i1,i2,i3)*model%rho(i1,i2,i3)
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      ! --- Sum over all processes (that potentially have core data) ---
      call MPI_AllReduce(MPI_IN_PLACE, rhoCore_mean, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDcore, &
                         ierrMPI)
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      luminosity=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, dei0) REDUCTION(+: luminosity)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            ! --- Compute change in internal energy ---
            if (mask_core(i1,i2,i3)) then
              dei0=dtime*luminosityTotal/rhoCore_mean
              !
              ! --- Add change in internal energy ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dei0
              !
              ! --- Compute luminosity per volume (core radius) ---
              luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0/dtime
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
    endif ! (par%hdCoreHeatProfile)
    !
  else ! (par%heat_mode == 'core_energy1': default) ---
    ! === Add energy in core =======================================================================
    !
    if (par%hdCoreHeatProfile == 'Constantdrhoei') then
      luminosity=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, drhoeiodtime0) REDUCTION(+: luminosity)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            ! --- Compute change in internal energy ---
            if (mask_core(i1,i2,i3)) then
              drhoeiodtime0=par%LuminosityPerVolume
              !
              ! --- Add change in internal energy ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dtime*drhoeiodtime0/model%rho(i1,i2,i3)
              !
              ! --- Compute luminosity per volume (from core radius) ---
              luminosity=luminosity + volume(i1,i2,i3)*drhoeiodtime0
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
    else ! (par%hdCoreHeatProfile == 'Constantdei')
      !
      ! --- Compute average core density ---
      !
      rhoCore_mean=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2) REDUCTION(+: rhoCore_mean)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              rhoCore_mean=rhoCore_mean+volume(i1,i2,i3)*model%rho(i1,i2,i3)
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      ! --- Sum over all processes (that potentially have core data) ---
      call MPI_AllReduce(MPI_IN_PLACE, rhoCore_mean, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDcore, &
                         ierrMPI)
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      luminosity=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, dei0) REDUCTION(+: luminosity)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            ! --- Compute change in internal energy ---
            if (mask_core(i1,i2,i3)) then
              dei0=dtime*par%LuminosityPerVolume/rhoCore_mean
              !
              ! --- Add change in internal energy ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dei0
              !
              ! --- Compute luminosity per volume (core radius) ---
              luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0/dtime
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
    endif ! (par%hdCoreHeatProfile)
    !
  endif ! (par%heat_mode)
  !
  ! ### Apply core drag force ######################################################################
  !
  ! \label{f90:c_coredrag__use1}
  ! === Apply drag force inside core ===
  if (par%C_CoreDrag > 0.0) then
    !
    ! \label{f90:hdcoredragprofile__use1}
    if (par%hdCoreDragProfile == 'Linear') then
      ! --- Superimpose a radial linear profile: drag force decays with radius ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                             max(1.0-sqrt(radius2(i1,i2,i3))/r_core, 1.0E-6)
              !
              v_factor_drag=max(1.0-dtime*invtime_v_corr, 0.0)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(     i1,i2,i3) + &
                                 0.5*(model%v1(i1,i2,i3)**2 + &
                                      model%v2(i1,i2,i3)**2 + &
                                      model%v3(i1,i2,i3)**2  ) * &
                                 (1.0 - v_factor_drag**2)
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'Linear-Radial') then
      ! --- Superimpose a radial linear profile: drag force decays with radius ---
      ! --- Damp the radial component of the velocity only                     ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                             max(1.0-sqrt(radius2(i1,i2,i3))/r_core, 1.0E-6)
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the radial velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3) + &
                            model%xc3(i3)*model%v3(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*vRadialOverx*model%xc3(i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'Linear-Meridional') then
      ! --- Superimpose a radial linear profile: drag force decays with radius ---
      ! --- Damp the meridional component of the velocity only                 ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                             max(1.0-sqrt(radius2(i1,i2,i3))/r_core, 1.0E-6)
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the meridional velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*model%v3(i1,i2,i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'Cosine') then
      ! --- Superimpose a radial cosine profile: drag force decays with radius ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
            ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                           (0.5+0.5*cos(pi*(min(sqrt(radius2(i1,i2,i3))/r_core, 1.0))))
              !
              v_factor_drag=max(1.0-dtime*invtime_v_corr, 0.0)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(     i1,i2,i3) + &
                                 0.5*(model%v1(i1,i2,i3)**2 + &
                                      model%v2(i1,i2,i3)**2 + &
                                      model%v3(i1,i2,i3)**2  ) * &
                                 (1.0 - v_factor_drag**2)
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'Cosine-Radial') then
      ! --- Superimpose a radial cosine profile: drag force decays with radius ---
      ! --- Damp the radial component of the velocity only                     ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                           (0.5+0.5*cos(pi*(min(sqrt(radius2(i1,i2,i3))/r_core, 1.0))))
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the radial velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3) + &
                            model%xc3(i3)*model%v3(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*vRadialOverx*model%xc3(i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'Cosine-Meridional') then
      ! --- Superimpose a radial cosine profile: drag force decays with radius ---
      ! --- Damp the meridional component of the velocity only                 ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                           (0.5+0.5*cos(pi*(min(sqrt(radius2(i1,i2,i3))/r_core, 1.0))))
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the meridional velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*model%v3(i1,i2,i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'CosSqr') then
      ! --- Superimpose a radial distorted cosine profile: drag force decays with radius ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                             (0.5+0.5*cos(pi*(min(radius2(i1,i2,i3)/r_core**2, 1.0))))
              !
              v_factor_drag=max(1.0-dtime*invtime_v_corr, 0.0)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(     i1,i2,i3) + &
                                 0.5*(model%v1(i1,i2,i3)**2 + &
                                      model%v2(i1,i2,i3)**2 + &
                                      model%v3(i1,i2,i3)**2  ) * &
                                 (1.0 - v_factor_drag**2)
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'CosSqr-Radial') then
      ! --- Superimpose a radial distorted cosine profile: drag force decays with radius ---
      ! --- Damp the radial component of the velocity only                               ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                             (0.5+0.5*cos(pi*(min(radius2(i1,i2,i3)/r_core**2, 1.0))))
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the radial velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3) + &
                            model%xc3(i3)*model%v3(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*vRadialOverx*model%xc3(i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'CosSqr-Meridional') then
      ! --- Superimpose a radial distorted cosine profile: drag force decays with radius ---
      ! --- Damp the meridional component of the velocity only                           ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              invtime_v_corr=invtime_v_corr * &
                             (0.5+0.5*cos(pi*(min(radius2(i1,i2,i3)/r_core**2, 1.0))))
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the meridional velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*model%v3(i1,i2,i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if ((par%hdCoreDragProfile == 'Constant-All-vTrans'   ) .or. &
             (par%hdCoreDragProfile == 'Constant-Radial-vTrans')       ) then
      ! --- Apply a drag force without radial profile                        ---
      ! --- Damp only in case of mean transversal velocity                   ---
      ! --- Damp all velocities or the radial component of the velocity only ---
      !
      ! --- Compute average core density and mass flow ---
      !
      rhoCore_mean  =0.0
      rhov1Core_mean=0.0
      rhov2Core_mean=0.0
      rhov3Core_mean=0.0
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2), &
      !$OMP    REDUCTION(+: rhoCore_mean, rhov1Core_mean, rhov2Core_mean, rhov3Core_mean)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              rhoCore_mean  =rhoCore_mean  +volume(i1,i2,i3)*model%rho(i1,i2,i3)
              rhov1Core_mean=rhov1Core_mean+volume(i1,i2,i3)*model%rho(i1,i2,i3)*model%v1(i1,i2,i3)
              rhov2Core_mean=rhov2Core_mean+volume(i1,i2,i3)*model%rho(i1,i2,i3)*model%v2(i1,i2,i3)
              rhov3Core_mean=rhov3Core_mean+volume(i1,i2,i3)*model%rho(i1,i2,i3)*model%v3(i1,i2,i3)
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      ! --- Sum over all processes (that potentially have core data) ---
      call rhd_MPI_AllReduceReals(MPI_SUM, rhdMPI_COMM_HDcore, r1=rhoCore_mean, &
                                  r2=rhov1Core_mean, r3=rhov2Core_mean, r4=rhov3Core_mean)
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      ! --- Compute core velocity ---
      v1Core=rhov1Core_mean/rhoCore_mean
      v2Core=rhov2Core_mean/rhoCore_mean
      v3Core=rhov3Core_mean/rhoCore_mean
      !
      ! --- Compute and apply drag force to reduce the core linear velocity ---
      !
      vRadial1_mean=0.0
      vRadial2_mean=0.0
      vRadial3_mean=0.0
      !
      if (par%hdCoreDragProfile == 'Constant-All-vTrans') then
        ! --- Apply drag force to all velocity components ---
        !
        !$OMP PARALLEL DEFAULT(SHARED)
        !
        !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
        !$OMP            vRadial1, vRadial2, vRadial3), &
        !$OMP    REDUCTION(+: vRadial1_mean, vRadial2_mean, vRadial3_mean)
        do i3=m3i,n3i
          do i2=m2i,n2i
            do i1=m1i,n1i
              if (mask_core(i1,i2,i3)) then
                ! --- Compute characteristic inverse time scale ---
                invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
                !
                ! --- Compute fraction by which the velocity should be reduced ---
                v_factor_drag=min(dtime*invtime_v_corr, 1.0)
                v_factor_drag=v_factor_drag * &
                              min(max((C_vTransFactor/ &
                                       (cs(i1,i2,i3)* &
                                        sqrt(model%v1(i1,i2,i3)**2 + &
                                             model%v2(i1,i2,i3)**2 + &
                                             model%v3(i1,i2,i3)**2 + eps_small))) * &
                                      (v1Core*model%v1(i1,i2,i3) + &
                                       v2Core*model%v2(i1,i2,i3) + &
                                       v3Core*model%v3(i1,i2,i3)    ), &
                                      0.0), &
                                  1.0)
                !
                ! --- Compute the components of the velocity times the drag factor ---
                vRadial1=v_factor_drag*model%v1(i1,i2,i3)
                vRadial2=v_factor_drag*model%v2(i1,i2,i3)
                vRadial3=v_factor_drag*model%v3(i1,i2,i3)
                !
                ! --- Apply drag force ---
                model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                   (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                    vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                    vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
                model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
                model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
                model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
                !
                ! --- Sum the changes (for output purposes only) ---
                vRadial1_mean=vRadial1_mean + volume(i1,i2,i3)*vRadial1
                vRadial2_mean=vRadial2_mean + volume(i1,i2,i3)*vRadial2
                vRadial3_mean=vRadial3_mean + volume(i1,i2,i3)*vRadial3
              endif
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP END PARALLEL
        !
      else ! (par%hdCoreDragProfile == 'Constant-Radial-vTrans')
        ! --- Apply drag force to radial velocity component only ---
        !
        !$OMP PARALLEL DEFAULT(SHARED)
        !
        !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
        !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3), &
        !$OMP    REDUCTION(+: vRadial1_mean, vRadial2_mean, vRadial3_mean)
        do i3=m3i,n3i
          do i2=m2i,n2i
            do i1=m1i,n1i
              if (mask_core(i1,i2,i3)) then
                ! --- Compute characteristic inverse time scale ---
                invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
                !
                ! --- Compute fraction by which the velocity should be reduced ---
                v_factor_drag=min(dtime*invtime_v_corr, 1.0)
                v_factor_drag=v_factor_drag * &
                              min(max((C_vTransFactor/ &
                                       (cs(i1,i2,i3)* &
                                        sqrt(model%v1(i1,i2,i3)**2 + &
                                             model%v2(i1,i2,i3)**2 + &
                                             model%v3(i1,i2,i3)**2 + eps_small))) * &
                                      (v1Core*model%v1(i1,i2,i3) + &
                                       v2Core*model%v2(i1,i2,i3) + &
                                       v3Core*model%v3(i1,i2,i3)    ), &
                                      0.0), &
                                  1.0)
                !
                ! --- Compute the components of the radial velocity times the drag factor ---
                vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                              model%xc2(i2)*model%v2(i1,i2,i3) + &
                              model%xc3(i3)*model%v3(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
                vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
                vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
                vRadial3=v_factor_drag*vRadialOverx*model%xc3(i3)
                !
                ! --- Apply drag force ---
                model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                   (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                    vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                    vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
                model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
                model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
                model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
                !
                ! --- Sum the changes (for output purposes only) ---
                vRadial1_mean=vRadial1_mean + volume(i1,i2,i3)*vRadial1
                vRadial2_mean=vRadial2_mean + volume(i1,i2,i3)*vRadial2
                vRadial3_mean=vRadial3_mean + volume(i1,i2,i3)*vRadial3
              endif
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
        !
        !$OMP END PARALLEL
        !
      endif ! (par%hdCoreDragProfile)
      !
!#
#     if (rhd_mpi_l01 == 1)
!#                          /* MPI activated ----------------- */
      ! --- Sum over all processes (that potentially have core data) ---
      call rhd_MPI_AllReduceReals(MPI_SUM, rhdMPI_COMM_HDcore, r1=vRadial1_mean, &
                                  r2=vRadial2_mean, r3=vRadial3_mean)
!#
#     endif
!#          /* rhd_mpi_l01 ----------------------------------- */
      !
      if (gl%nc_m >= 0) then
        write(gl%nc_m,'(A,3(1PE13.5),A,3(1PE11.3))') &
             'Core linear velocity:', v1Core, v2Core, v3Core, &
             ' -', vRadial1_mean, vRadial2_mean, vRadial3_mean
      endif
      !
    else if (par%hdCoreDragProfile == 'Constant-Radial') then
      ! --- Apply a drag force without radial profile      ---
      ! --- Damp the radial component of the velocity only ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the radial velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3) + &
                            model%xc3(i3)*model%v3(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*vRadialOverx*model%xc3(i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else if (par%hdCoreDragProfile == 'Constant-Meridional') then
      ! --- Apply a drag force without radial profile          ---
      ! --- Damp the meridional component of the velocity only ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag, &
      !$OMP            vRadialOverx, vRadial1, vRadial2, vRadial3)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              !
              v_factor_drag=min(dtime*invtime_v_corr, 1.0)
              !
              ! --- Compute the components of the meridional velocity times the drag factor ---
              vRadialOverx=(model%xc1(i1)*model%v1(i1,i2,i3) + &
                            model%xc2(i2)*model%v2(i1,i2,i3)    )/(radius2(i1,i2,i3)+eps_small)
              vRadial1=v_factor_drag*vRadialOverx*model%xc1(i1)
              vRadial2=v_factor_drag*vRadialOverx*model%xc2(i2)
              vRadial3=v_factor_drag*model%v3(i1,i2,i3)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + &
                                 (vRadial1*(model%v1(i1,i2,i3)-0.5*vRadial1) + &
                                  vRadial2*(model%v2(i1,i2,i3)-0.5*vRadial2) + &
                                  vRadial3*(model%v3(i1,i2,i3)-0.5*vRadial3)    )
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) - vRadial1
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) - vRadial2
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) - vRadial3
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    else ! (par%hdCoreDragProfile == 'Constant')
      ! --- Apply a drag force without radial profile ---
      !
      !$OMP PARALLEL DEFAULT(SHARED)
      !
      !$OMP DO PRIVATE(i1, i2, invtime_v_corr, v_factor_drag)
      do i3=m3i,n3i
        do i2=m2i,n2i
          do i1=m1i,n1i
            if (mask_core(i1,i2,i3)) then
              ! --- Compute characteristic inverse time scale ---
              invtime_v_corr=cs(i1,i2,i3)*(par%C_CoreDrag/(2.0*r_core))
              !
              v_factor_drag=max(1.0-dtime*invtime_v_corr, 0.0)
              !
              ! --- Apply drag force ---
              model%ei(i1,i2,i3)=model%ei(     i1,i2,i3) + &
                                 0.5*(model%v1(i1,i2,i3)**2 + &
                                      model%v2(i1,i2,i3)**2 + &
                                      model%v3(i1,i2,i3)**2  ) * &
                                 (1.0 - v_factor_drag**2)
              model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
              model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
              model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
            endif
          end do ! i1
        end do ! i2
      end do ! i3
      !$OMP END DO
      !
      !$OMP END PARALLEL
      !
    endif ! (par%hdCoreDragProfile)
    !
  endif ! (par%C_CoreDrag > 0.0)
  !
  ! --- Deallocate temporary arrays ---
  deallocate(radius2, volume, T, s, cs, mask_core)
  !
else ! (ModelHasCore_flag)
  ! --- Box does not contain any part of the core: there is no heating anywhere ---
  luminosity=0.0
  !
endif ! (ModelHasCore_flag)
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Free communicator for core-data group ---
call MPI_Comm_Free(rhdMPI_COMM_HDcore, ierrMPI)
!
! --- Sum over all processes (those without core data have zero luminosity) ---
! write(gl%nc_p, '(A,I3,A,3X,1PE14.6)') '#', wg%hyd%MyProc, &
!       ' mylumin.:', luminosity
mylum=luminosity
call MPI_Reduce(mylum, luminosity, 1, MPI_REAL, MPI_SUM, &
                wg%hyd%MaProcWoInCO, wg%hyd%Comm, ierrMPI)
! write(gl%nc_p, '(A,I3,A,3X,1PE14.6)') '#', wg%hyd%MyProc, &
!      ' lumin.  :', luminosity
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_bound_3DCenter


!----------**********************-------------------------------------------------------------------
subroutine rhd_bound_3DHeatBottom(model, action, dtime, luminosity)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DHeatBottom ('rhd_boundary_conditions_3_Dimensions_Heat_Bottom')
!
! PURPOSE:
!   Modify energy in bottom cells to keep entropy constant.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DHeatBottom(model, action, dtime, luminosity)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   luminosity:   ('luminosity') real, luminosity per core volume [erg/cm^3/s]
!
! ROUTINES:
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   timing_module:         Timing routines and data for performance measurements and benchmarks
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The internal energy in the bottom-layer cells is modified to push their entropy closer
!   to the value prescribed by s_InFlow.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2006-12-14 (B.F. Lyon) First attempt
!   2006-12-15 (B.F.) First running version
!   2006-12-18 (B.F.) r0_grav used in volume computation
!   2007-01-14 (B.F.) Drag force i3 dependend, gradient at bottom of entropy plateau
!   2007-01-15 (B.F.) Drag force and entropy gradient back to normal
!   2007-02-01 (B.F.) Smooth onset of drag force (top), adjust s slower in 2 bottom layers
!                     drag force only vertical
!   2008-02-01 (B.F.) time_s_Factor=1.0 everywhere
!   2007-03-25 (B.F.) Drag force in all three dimensions, again
!   2009-09-23 (B.F.) bottom_entropy2 mode added
!   2009-09-30 (B.F.) bottom_entropy3 mode added, corrected mode bottom_entropy2
!   2010-04-07 (B.F.) Renamed: rhd_bound_HeatBottom => rhd_bound_3DHeatBottom
!   2015-01-13 (B.F.) Add mode bottom_entropy2h
!---------------------------------------------------------------------------------------------------
use const_module, only: sigma, pi
use rhd_gl_module
use rhd_action_module
use rhd_box_module
!#
# ifdef rhd_bound_t01
!#
use timing_module
!#
# endif
!#
use gasinter_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: luminosity
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           n3i, &
                                           i1, i2, i3
real                                    :: gridsize, dxb3, dxb23, rhoCore_mean, v_factor_drag, &
                                           time_s_corr, s_target, dei_scalar, DragFactor, &
                                           time_s_Factor, &
                                           v_TransLayerThickness, s_TransLayerThickness
real(kind=kind(0.0E+00))                :: area_sum
real,    allocatable, dimension(:)      :: P, dPdrho, dPdei, dTdei
real,    allocatable, dimension(:,:,:)  :: volume, T, s, dei, cs
real(kind=kind(0.0E+00)), &
         allocatable, dimension(:)      :: ds_mean
real(kind=kind(0.0E+00)), &
         allocatable, dimension(:,:)    :: area
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! --- Compute index range ---
I3_LOOP: do i3=m3,n3
  if ((model%xc3(i3) - model%xc3(m3)) > par%r0_grav) then
    n3i=i3-1
    exit I3_LOOP
  endif
end do I3_LOOP ! i3
v_TransLayerThickness=(real(n3i-m3)*0.7)+1.0
s_TransLayerThickness=(real(n3i-m3)*1.0)+1.0
!write(6,*) n3i, n3i-m3, v_TransLayerThickness, s_TransLayerThickness
!
! --- Allocate temporary arrays ---
allocate(P(m1:n1), dPdrho(m1:n1), dPdei(m1:n1), dTdei(m1:n1), &
         volume( m1:n1,m2:n2,m3:n3i), &
         T(      m1:n1,m2:n2,m3:n3i), &
         s(      m1:n1,m2:n2,m3:n3i), &
         cs(     m1:n1,m2:n2,m3:n3i), &
         dei(    m1:n1,m2:n2,m3:n3i), &
         ds_mean(            m3:n3i), &
         area(   m1:n1,m2:n2       )   )
!
! --- Solve EOS and compute characteristic time scales ---
gridsize=max((model%xb1(n1+1)-model%xb1(m1))/(n1-m1+1), &
             (model%xb2(n2+1)-model%xb2(m2))/(n2-m2+1), &
             (model%xb3(n3+1)-model%xb3(m3))/(n3-m3+1))
!
! --- Relative volume element ---
do i3=m3,n3i
!  dxb3=(model%xb3(i3+1)-model%xb3(i3))/min(par%r0_grav,(model%xb3(n3i+1)-model%xb3(m3)))
  dxb3=(model%xb3(i3+1)-model%xb3(i3))/par%r0_grav
  do i2=m2,n2
    dxb23=(model%xb2(i2+1)-model%xb2(i2))/(model%xb2(n2+1)-model%xb2(m2)) * dxb3
    volume( :,i2,i3)=(model%xb1(m1+1:n1+1)-model%xb1(m1:n1))/(model%xb1(n1+1)-model%xb1(m1)) * dxb23
  end do ! i2
end do ! i3
!
! --- Solve EOS ---
!#
# ifdef rhd_bound_t01
!#
call timing_start('HYD: bound_HeatBottom: EOS')
!#
# endif
!#
do i3=m3,n3i
  do i2=m2,n2
    call eosinter(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                  P=P, dPdrho=dPdrho, dPde=dPdei, &
                  T=T(:,i2,i3),       dTde=dTdei, &
                  s=s(:,i2,i3))
    ! --- Compute sound speed ---
    cs(:,i2,i3)=sqrt(P*dPdei/model%rho(m1:n1,i2,i3)**2+dPdrho)
  end do ! i2
end do ! i3
!#
# ifdef rhd_bound_t01
!#
call timing_stop( 'HYD: bound_HeatBottom: EOS', cycles=(n2-m2+1)*(n3i-m3+1))
!#
# endif
!#
!
if (par%heat_mode == 'bottom_entropy2h') then
  ! --- Compute relative cell (column) areas ---
  do i2=m2,n2
    do i1=m1,n1
      area(i1,i2)=(model%xb1(i1+1)-model%xb1(i1)) * (model%xb2(i2+1)-model%xb2(i2))
    end do ! i1
  end do ! i2
  !
  area_sum=sum(area)
  !
  do i2=m2,n2
    do i1=m1,n1
      area(i1,i2)=area(i1,i2)/area_sum
    end do ! i1
  end do ! i2
  !
  do i3=m3,n3i
    ds_mean(i3)=sum(area*(s(m1:n1,m2:n2,i3)-par%s_InFlow))
  end do ! i3
endif
!
! --- Handle energy production ---
!
if (((par%heat_mode == 'bottom_entropy1' ) .or. &
     (par%heat_mode == 'bottom_entropy2' ) .or. &
     (par%heat_mode == 'bottom_entropy2h') .or. &
     (par%heat_mode == 'bottom_entropy3' )       ) .and. &
    (par%C_sChange > 0.0                         )        ) then
  ! === Adjust entropy ===
  !
  do i3=m3,n3i
    if (par%heat_mode == 'bottom_entropy2h') then
      s_target=par%s_InFlow + 0.5*ds_mean(i3)
    else
      s_target=par%s_InFlow
    endif
    !
    if ((par%heat_mode == 'bottom_entropy2') .or. (par%heat_mode == 'bottom_entropy2h')) then
      time_s_Factor=max(min(real(n3i-i3+1)/s_TransLayerThickness, 1.0), 0.0)
      !write(6,*) i3, time_s_Factor
    else if (par%heat_mode == 'bottom_entropy3') then
      time_s_Factor=0.5-0.5*cos(pi*max(min(real(n3i-i3+1)/s_TransLayerThickness, 1.0), 0.0))
      !write(6,*) i3, time_s_Factor
    else
      time_s_Factor=1.0
    endif
    do i2=m2,n2
      do i1=m1,n1
        ! --- Compute characteristic time scales ---
        !time_s_corr=time_s_Factor*gridsize/par%C_sChange/cs(i1,i2,i3)
        !time_s_corr=gridsize/(time_s_Factor*par%C_sChange*cs(i1,i2,i3))
        !
        ! \label{f90:c_schange__use3}
        ! --- Change in internal energy to come closer to specified entropy ---
        dei(i1,i2,i3)=dtime/gridsize*(time_s_Factor*par%C_sChange*cs(i1,i2,i3)) * T(i1,i2,i3) * &
                      (s_target-s(i1,i2,i3))
        model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dei(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !
  ! --- Compute luminosity per volume ---
  luminosity=sum(volume*model%rho(m1:n1,m2:n2,m3:n3i)*dei)/dtime
  !
else if (par%heat_mode == 'bottom_energy1') then
  ! === Add energy ===
  !
  rhoCore_mean=sum(volume*model%rho(m1:n1,m2:n2,m3:n3i))
  !
  ! --- Change in internal energy ---
!  dei_scalar=dtime*par%LuminosityPerVolume/rhoCore_mean
  dei_scalar=dtime*sigma*par%Teff**4/((model%xb3(n3i)-model%xb3(m3))*rhoCore_mean)
  model%ei(m1:n1,m2:n2,m3:n3i)=model%ei(m1:n1,m2:n2,m3:n3i)+dei_scalar
  !
  ! --- Compute luminosity per volume (core radius) ---
  ! --- Usually luminosity=par%LuminosityPerVolume, but it's better to check... ---
  luminosity=sum(volume*model%rho(m1:n1,m2:n2,m3:n3i)*dei_scalar)/dtime
  !
else
  write(gl%nc_p,'(A)') 'Unsupported heating mode'
endif
!
! --- Apply drag force ---
!
if (par%C_CoreDrag > 0.0) then
  ! === Apply drag force inside bottom layers ===
  !
  do i3=m3,n3i
    if ((par%heat_mode == 'bottom_entropy2') .or. (par%heat_mode == 'bottom_entropy2h')) then
      DragFactor=max(min(real(n3i-i3+1)/v_TransLayerThickness, 1.0), 0.0)
      !write(6,*) i3, DragFactor
    else if (par%heat_mode == 'bottom_entropy3') then
      DragFactor=0.5-0.5*cos(pi*max(min(real(n3i-i3+1)/v_TransLayerThickness, 1.0), 0.0))
      !write(6,*) i3, DragFactor
    else
      if      ((n3i-i3) == 0) then
        DragFactor=0.15
      else if ((n3i-i3) == 1) then
        DragFactor=0.5
      else if ((n3i-i3) == 2) then
        DragFactor=0.85
      else
        DragFactor=1.0
      endif
    endif
    !
    do i2=m2,n2
      do i1=m1,n1
        time_s_corr=(2.0*par%r0_grav/(DragFactor*par%C_CoreDrag))/cs(i1,i2,i3)
        v_factor_drag=max(1.0-dtime/time_s_corr, 0.0)
        model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + 0.5*( &
                                                     model%v1(i1,i2,i3)**2 + &
                                                     model%v2(i1,i2,i3)**2 + &
                                                     model%v3(i1,i2,i3)**2  &
                                                    ) * &
                                                (1.0 - v_factor_drag**2)
        model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
        model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
        model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
      end do ! i1
    end do ! i2
  end do ! i3
endif
!
! --- Deallocate temporary arrays ---
deallocate(P, dPdrho, dPdei, dTdei, &
           volume, T, s, cs, dei, &
           ds_mean, area)
!
end subroutine rhd_bound_3DHeatBottom


!----------**********************-------------------------------------------------------------------
subroutine rhd_bound_3DHeatBottom_MPI(model, action, dtime, luminosity)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_3DHeatBottom_MPI ('rhd_boundary_conditions_3_Dimensions_Heat_Bottom')
!
! PURPOSE:
!   Modify energy in bottom cells to keep entropy constant.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_bound_3DHeatBottom(model, action, dtime, luminosity)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   dtime:        ('delta_time') real, time step to be used
!
! INPUT/OUTPUT:
!   model:        ('model') derived type, model data: rho, ei, v, x, time, ...
!
! OUTPUT:
!   luminosity:   ('luminosity') real, luminosity per core volume [erg/cm^3/s]
!
! ROUTINES:
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   flush:                 Flush output buffer
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Comm_Free:         Free a communicator
!   MPI_Comm_Split:        Split a communicator
!   MPI_Reduce:            Perform a reduction
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   gasinter_module:       Routines for interpolating GAS or EOS quantities
!   MPI:                   Standard MPI routines and definitions
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   The internal energy in the bottom-layer cells is modified to push their entropy closer
!   to the value prescribed by s_InFlow.
!   Alternatively, the energy change can be specified directly.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2006-12-14 (B.F. Lyon) First attempt
!   2006-12-15 (B.F.) First running version
!   2006-12-18 (B.F.) r0_grav used in volume computation
!   2007-01-14 (B.F.) Drag force i3 dependend, gradient at bottom of entropy plateau
!   2007-01-15 (B.F.) Drag force and entropy gradient back to normal
!   2007-02-01 (B.F.) Smooth onset of drag force (top), adjust s slower in 2 bottom layers,
!                     drag force only vertical
!   2008-02-01 (B.F.) time_s_Factor=1.0 everywhere
!   2007-03-25 (B.F.) Drag force in all three dimensions, again
!   2009-09-23 (B.F.) bottom_entropy2 mode added
!   2009-09-30 (B.F.) bottom_entropy3 mode added, corrected mode bottom_entropy2
!   2010-04-07 (B.F.) Renamed: rhd_bound_HeatBottom => rhd_bound_3DHeatBottom
!   2012-10-30 (B.F.) New parameters in call, first steps towards MPI version,
!                     remove timing('HYD: bound_HeatBottom: EOS')
!   2012-11-07 (B.F.) OpenMP+MPI version; compute heating starting at xb3(m3) and not xc3(m3)
!   2012-11-12 (B.F.) MPI parameters put into prop structure
!   2014-03-17 (B.F.) Correct OpenMP calls (missing $)
!---------------------------------------------------------------------------------------------------
use const_module, only: sigma, pi
use rhd_gl_module
use rhd_prop_module
use rhd_action_module
use rhd_box_module
use gasinter_module
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: model
type(action_type),         intent(in)   :: action
real,                      intent(in)   :: dtime
real,                      intent(out)  :: luminosity
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           m3i, n3i, n3ig, &
                                           i1, i2, i3, &
                                           ModelHasBottom_id, &
                                           rhdMPI_COMM_HDbottom, ierrMPI
real                                    :: gridsize, dxb3, dxb23, rhobottom_mean, &
                                           v_factor_drag, &
                                           time_s_corr, dei0, DragFactor, &
                                           time_s_Factor, &
                                           v_TransLayerThickness, s_TransLayerThickness, mylum
real,    allocatable, dimension(:)      :: P, dPdrho, dPdei, dTdei
real,    allocatable, dimension(:,:,:)  :: volume, T, s, cs
logical                                 :: ModelHasBottom_flag
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
! \label{f90:r0_grav__use3}
! --- Determine the index range for grid points within the heating layers ---
m3i=m3
n3i=n3
I3_LOOP: do i3=m3,n3
!  if ((model%xc3(i3) - model%xc3(m3)) > par%r0_grav) then
  if ((model%xc3(i3) - wg%hyd%xb_l(3)) > par%r0_grav) then
    n3i=i3-1
    exit I3_LOOP
  endif
end do I3_LOOP ! i3
!
! --- Determine if there are heating layers in the box ---
ModelHasBottom_flag=(n3i >= m3i)
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
if (ModelHasBottom_flag) then
  ModelHasBottom_id=1
else
  ModelHasBottom_id=0
endif
!
write(gl%nc_p, '(A,I3,A,1(2X2I4),7X,I1)') '#', wg%hyd%MyProc, &
  ' Heating layer range: ', m3i,n3i, ModelHasBottom_id
! --- Split processes into two groups with and without bottom-layer cells, respectively ---
call MPI_Comm_Split(wg%hyd%Comm, ModelHasBottom_id, 0, rhdMPI_COMM_HDbottom, ierrMPI)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
rhdMPI_COMM_HDbottom=0
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
if (ModelHasBottom_flag) then
  !
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
  ! --- Find and distribute the global extent of the heating layer region ---
  call MPI_AllReduce(n3i, n3ig, 1, MPI_INTEGER, MPI_MAX, rhdMPI_COMM_HDbottom, ierrMPI)
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
  n3ig=n3i
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
  ! --- Compute layer thicknesses from global parameters ---
  v_TransLayerThickness=(real(n3ig-wg%hyd%m(3))*0.7)+1.0
  s_TransLayerThickness=(real(n3ig-wg%hyd%m(3))*1.0)+1.0
  write(gl%nc_p, '(A,I3,A,1(2X2I4),7X,2(1PE14.6))') '#', wg%hyd%MyProc, &
    ' Gl. heat. layer range & thickness: ', m3i,n3ig, v_TransLayerThickness, s_TransLayerThickness
  call flush(gl%nc_p)
  !
  ! --- Allocate temporary arrays ---
  allocate(volume(m1:n1,m2:n2,m3i:n3i), &
           T(     m1:n1,m2:n2,m3i:n3i), &
           s(     m1:n1,m2:n2,m3i:n3i), &
           cs(    m1:n1,m2:n2,m3i:n3i))
  !
  ! --- Compute the typical grid size ---
  !gridsize=max((model%xb1(n1+1)-model%xb1(m1))/(n1-m1+1), &
  !             (model%xb2(n2+1)-model%xb2(m2))/(n2-m2+1), &
  !             (model%xb3(n3+1)-model%xb3(m3))/(n3-m3+1))
  gridsize=max((wg%hyd%xb_r(1)-wg%hyd%xb_l(1))/(wg%hyd%n(1)-wg%hyd%m(1)+1), &
               (wg%hyd%xb_r(2)-wg%hyd%xb_l(2))/(wg%hyd%n(2)-wg%hyd%m(2)+1), &
               (wg%hyd%xb_r(3)-wg%hyd%xb_l(3))/(wg%hyd%n(3)-wg%hyd%m(3)+1))
  !
  !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(P, dPdrho, dPdei, dTdei)
  !
  ! --- Compute relative volume element ---
  !$OMP DO PRIVATE(dxb3, dxb23, i1, i2)
  do i3=m3i,n3i
    dxb3=(model%xb3(i3+1)-model%xb3(i3))/par%r0_grav
    do i2=m2,n2
      dxb23=(model%xb2(i2+1)-model%xb2(i2))/(wg%hyd%xb_r(2)-wg%hyd%xb_l(2))*dxb3
      do i1=m1,n1
        volume(i1,i2,i3)=(model%xb1(i1+1)-model%xb1(i1))/(wg%hyd%xb_r(1)-wg%hyd%xb_l(1))*dxb23
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
  !
  write(gl%nc_p, '(A,I3,A,1PE14.6,1(3X,2(1PE14.6)))') '#', wg%hyd%MyProc, &
    ' grid, minmax(volume): ', gridsize, &
    minval(volume), maxval(volume)
  !
  if ((par%LuminosityPerVolume <= 0.0    ) .or. &
      (par%heat_mode == 'bottom_entropy1') .or. &
      (par%heat_mode == 'bottom_entropy2') .or. &
      (par%heat_mode == 'bottom_entropy3') .or. &
      (par%C_CoreDrag > 0.0              )     ) then
    !
    allocate(P(m1:n1), dPdrho(m1:n1), dPdei(m1:n1), dTdei(m1:n1))
    !
    ! --- Solve EOS ---
    !$OMP DO PRIVATE(i2)
    do i3=m3,n3i
      do i2=m2,n2
        call eosinter(model%rho(m1:n1,i2,i3), model%ei(m1:n1,i2,i3), &
                      P=P, dPdrho=dPdrho, dPde=dPdei, &
                      T=T(:,i2,i3),       dTde=dTdei, &
                      s=s(:,i2,i3))
        ! --- Compute sound speed ---
        cs(:,i2,i3)=sqrt(P*dPdei/model%rho(m1:n1,i2,i3)**2+dPdrho)
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    deallocate(P, dPdrho, dPdei, dTdei)
  endif
  !
  !$OMP END PARALLEL
  !
  ! \label{f90:heat_mode__use2}
  ! --- Compute the energy production rate ---
  !
  if (((par%heat_mode == 'bottom_entropy1') .or. &
       (par%heat_mode == 'bottom_entropy2') .or. &
       (par%heat_mode == 'bottom_entropy3')       ) .and. (par%C_sChange > 0.0)) then
    ! === Adjust the entropy via changing the internal energy ===
    !
    luminosity=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, time_s_Factor, dei0) REDUCTION(+: luminosity)
    do i3=m3,n3i
      if (par%heat_mode == 'bottom_entropy2') then
        time_s_Factor=max(min(real(n3ig-i3+1)/s_TransLayerThickness, 1.0), 0.0)
        !write(6,*) i3, time_s_Factor
      else if (par%heat_mode == 'bottom_entropy3') then
        time_s_Factor=0.5-0.5*cos(pi*max(min(real(n3ig-i3+1)/s_TransLayerThickness, 1.0), 0.0))
        !write(6,*) i3, time_s_Factor
      else
        time_s_Factor=1.0
      endif
      do i2=m2,n2
        do i1=m1,n1
          ! --- Compute characteristic time scales ---
          !time_s_corr=time_s_Factor*gridsize/par%C_sChange/cs(i1,i2,i3)
          !time_s_corr=gridsize/(time_s_Factor*par%C_sChange*cs(i1,i2,i3))
          !
          ! \label{f90:s_inflow__use4}\label{f90:c_schange__use4}
          ! --- Get change in internal energy to come closer to the specified entropy ---
          dei0=1.0/gridsize*(time_s_Factor*par%C_sChange*cs(i1,i2,i3)) * T(i1,i2,i3) * &
               (par%s_InFlow-s(i1,i2,i3))
          !
          ! --- Add change in internal energy ---
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dei0*dtime
          !
          ! --- Compute luminosity per volume ---
          luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  else if (par%heat_mode == 'bottom_energy1') then
    ! === Add energy directly ===
    !
    ! --- Compute average core density ---
    rhobottom_mean=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2) REDUCTION(+: rhobottom_mean)
    do i3=m3i,n3i
      do i2=m2,n2
        do i1=m1,n1
          rhobottom_mean=rhobottom_mean+volume(i1,i2,i3)*model%rho(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
    ! --- Sum over all processes (that potentially contain heating layers) ---
    call MPI_AllReduce(MPI_IN_PLACE, rhobottom_mean, 1, MPI_REAL, MPI_SUM, rhdMPI_COMM_HDbottom, &
                       ierrMPI)
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
    !
    ! \label{f90:teff__use1}
    ! --- Compute the change in internal energy ---
    dei0=sigma*par%Teff**4/((model%xb3(n3i)-model%xb3(m3))*rhobottom_mean)
    luminosity=0.0
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2) REDUCTION(+: luminosity)
    do i3=m3i,n3i
      do i2=m2,n2
        do i1=m1,n1
          ! --- Add the change in internal energy ---
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3)+dei0*dtime
          !
          ! --- Compute luminosity per (heating-layer) volume ---
          luminosity=luminosity + volume(i1,i2,i3)*model%rho(i1,i2,i3)*dei0
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
    !
  else
    write(gl%nc_p,'(A)') 'Unsupported heating mode'
  endif
  !
  ! \label{f90:c_coredrag__use2}
  ! --- Apply drag force ---
  !
  if (par%C_CoreDrag > 0.0) then
    ! === Apply drag force inside bottom layers ===
    !
    !$OMP PARALLEL DEFAULT(SHARED)
    !
    !$OMP DO PRIVATE(i1, i2, DragFactor, time_s_corr, v_factor_drag)
    do i3=m3,n3i
      if (par%heat_mode == 'bottom_entropy2') then
        DragFactor=max(min(real(n3ig-i3+1)/v_TransLayerThickness, 1.0), 0.0)
        !write(6,*) i3, DragFactor
      else if (par%heat_mode == 'bottom_entropy3') then
        DragFactor=0.5-0.5*cos(pi*max(min(real(n3ig-i3+1)/v_TransLayerThickness, 1.0), 0.0))
        !write(6,*) i3, DragFactor
      else
        if      ((n3ig-i3) == 0) then
          DragFactor=0.15
        else if ((n3ig-i3) == 1) then
          DragFactor=0.5
        else if ((n3ig-i3) == 2) then
          DragFactor=0.85
        else
          DragFactor=1.0
        endif
      endif
      !
      do i2=m2,n2
        do i1=m1,n1
          time_s_corr=(2.0*par%r0_grav/(DragFactor*par%C_CoreDrag))/cs(i1,i2,i3)
          v_factor_drag=max(1.0-dtime/time_s_corr, 0.0)
          model%ei(i1,i2,i3)=model%ei(i1,i2,i3) + 0.5*( &
                                                       model%v1(i1,i2,i3)**2 + &
                                                       model%v2(i1,i2,i3)**2 + &
                                                       model%v3(i1,i2,i3)**2  &
                                                      ) * &
                                                  (1.0 - v_factor_drag**2)
          model%v1(i1,i2,i3)=model%v1(i1,i2,i3) * v_factor_drag
          model%v2(i1,i2,i3)=model%v2(i1,i2,i3) * v_factor_drag
          model%v3(i1,i2,i3)=model%v3(i1,i2,i3) * v_factor_drag
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !
    !$OMP END PARALLEL
  endif
  !
  ! --- Deallocate temporary arrays ---
  deallocate(volume, T, s, cs)
  !
else
  ! --- Box does not contain any part of the heating layers: there is no heating anywhere ---
  luminosity=0.0
  !
endif
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Free communicator for bottom-data group ---
call MPI_Comm_Free(rhdMPI_COMM_HDbottom, ierrMPI)
!
! --- Sum over all processes (those without bottom data have zero luminosity) ---
write(gl%nc_p, '(A,I3,A,3X,1PE14.6)') '#', wg%hyd%MyProc, &
      ' mylumin.:', luminosity
mylum=luminosity
call MPI_Reduce(mylum, luminosity, 1, MPI_REAL, MPI_SUM, &
                wg%hyd%MaProcWoInCO, wg%hyd%Comm, ierrMPI)
write(gl%nc_p, '(A,I3,A,3X,1PE14.6)') '#', wg%hyd%MyProc, &
      ' lumin.  :', luminosity
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_bound_3DHeatBottom_MPI


!----------********************---------------------------------------------------------------------
subroutine rhd_bound_ExtractBox(box, action, m1,n1, m2,n2, m3,n3, nghost, &
                                boxout, m1out, m2out, m3out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_ExtractBox ('rhd_boundaries_Extract_Box')
!
! PURPOSE:
!   Extract part of box, fill boundary ghost cells properly.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_bound_ExtractBox(box, action, m1,n1, m2,n2, m3,n3, nghost, &
!                             boxout, m1out, m2out, m3out)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   action:       ('action') derived type with collection of control parameters
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost:       ('number_ghost') integer, number of ghost cells
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines for RHD
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Copy.
!   Time specifications are not copied.
!   Higher-order boundary conditions (e.g., bound_closedbottom2, bound_inoutflow2) might not work.
!   Boxes with extra boundary layers (in the MPI case) are not handled.
!
! PROCEDURE:
!   Grid (1st index):
!
!       +         +      +             +       +        +
!     box%m1  m1-nghost  m1    ...     n1  n1+nghost  box%n1
!             m1eff                        n1eff
!   or e.g.
!        +         +     +             +     +         +
!     m1-nghost  box%m1  m1    ...     n1  box%n1  n1+nghost
!                m1eff                     n1eff
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-05-25 (B.F. Copenhagen) First version
!   1999-05-31 (B.F.) More calls of copy_box
!   1999-06-07 (B.F.) Index correction
!   1999-09-02 (B.F.) 2D case: check 2nd dimension of boxout
!   2010-04-20 (B.F. Lyon) Rename extract_box -> rhd_box_Extract
!   2010-12-10 (W.S. Exeter) moved to rhd_box_module
!   2010-12-23 (W.S.) Modified for handling of MHD boundary conditions
!   2017-09-18 (B.F. Uppsala) Rename rhd_box_Extract -> rhd_bound_ExtractBox
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
type(action_type),         intent(in)   :: action
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1out, m2out, m3out
!
! --- Local variables ---
logical                                 :: quc_flag0, Bc_flag0, Bb_flag0
integer                                 :: n1out, n2out, n3out, &
                                           m1eff0, n1eff0, m2eff0, n2eff0, m3eff0, n3eff0, &
                                           m1effout0, n1effout0, &
                                           m2effout0, n2effout0, &
                                           m3effout0, n3effout0, &
                                           m1eff, n1eff, m2eff, n2eff, m3eff, n3eff, &
                                           m1effout, n1effout, &
                                           m2effout, n2effout, &
                                           m3effout, n3effout, &
                                           n1first, m1last, n2first, m2last, n3first, m3last, &
                                           m1efonew, n1efonew, m2efonew, n2efonew, &
                                           m3efonew, n3efonew, &
                                           nghostrest
type(box_type)                          :: slice
type(action_type)                       :: action_current
!---------------------------------------------------------------------------------------------------
!
if(boxout%nquc > 0) then
  quc_flag0 = .true.
else
  quc_flag0 = .false.
endif
if(boxout%nBc > 0) then
  Bc_flag0 = .true.
else
  Bc_flag0 = .false.
endif
if(boxout%nBb > 0) then
  Bb_flag0 = .true.
else
  Bb_flag0 = .false.
endif
!
n1out=m1out+n1-m1
n2out=m2out+n2-m2
n3out=m3out+n3-m3
!
! --- Determine effective source indices for copying:                                           ---
! --- If requested sub-box lies within larger box, there is no need for extra boundary handling ---
m1eff0=m1-boxout%nghost1
n1eff0=n1+boxout%nghost1
m2eff0=m2-boxout%nghost2
n2eff0=n2+boxout%nghost2
m3eff0=m3-boxout%nghost3
n3eff0=n3+boxout%nghost3
!
m1eff=max(m1eff0,box%m1)
n1eff=min(n1eff0,box%n1)
m2eff=max(m2eff0,box%m2)
n2eff=min(n2eff0,box%n2)
m3eff=max(m3eff0,box%m3)
n3eff=min(n3eff0,box%n3)
!
! --- Corresponding target indices ---
m1effout0=m1out + m1eff0-m1
n1effout0=m1out + n1eff0-m1
m2effout0=m2out + m2eff0-m2
n2effout0=m2out + n2eff0-m2
m3effout0=m3out + m3eff0-m3
n3effout0=m3out + n3eff0-m3
!
m1effout=m1out + m1eff-m1
n1effout=m1out + n1eff-m1
m2effout=m2out + m2eff-m2
n2effout=m2out + n2eff-m2
m3effout=m3out + m3eff-m3
n3effout=m3out + n3eff-m3
!
m1last=box%m1-n1eff+n1eff0-1
m2last=box%m2-n2eff+n2eff0-1
m3last=box%m3-n3eff+n3eff0-1
!
n1first=box%n1-m1eff+m1eff0+1
n2first=box%n2-m2eff+m2eff0+1
n3first=box%n3-m3eff+m3eff0+1
!
! --- Copy HD quantities ---
call rhd_box_Copy(box   , m1eff     , n1eff , m2eff     , n2eff , m3eff     , n3eff , &
                  boxout, m1effout  ,         m2effout  ,         m3effout  )
!
!
! --- Fill ghost cells, assume periodic boundary conditions ---
!
! --- Faces ---
!
if (m1eff > m1eff0) then
  call rhd_box_Copy(box   , n1first   , box%n1, m2eff     , n2eff , m3eff     , n3eff , &
                    boxout, m1effout0 ,         m2effout  ,         m3effout  , &
                    grid_mode=0, flux_mode=0)
    ! --- Copy grid ---
    boxout%xc1(m1effout0:m1effout-1)=(box%xc1(n1first:box%n1) - box%xb1(box%n1+1)) + &
                                      box%xb1(box%m1)
    boxout%xb1(m1effout0:m1effout-1)=(box%xb1(n1first:box%n1) - box%xb1(box%n1+1)) + &
                                      box%xb1(box%m1)
endif
!
if (n1eff0 > n1eff) then
  call rhd_box_Copy(box   , box%m1    , m1last, m2eff     , n2eff , m3eff     , n3eff , &
                    boxout, n1effout+1,         m2effout  ,         m3effout  , &
                    grid_mode=0, flux_mode=0)
    boxout%xc1(n1effout+1:n1effout0  )=(box%xc1(box%m1  :m1last  ) - box%xb1(box%m1)) + &
                                        box%xb1(box%n1+1)
    boxout%xb1(n1effout+2:n1effout0+1)=(box%xb1(box%m1+1:m1last+1) - box%xb1(box%m1)) + &
                                        box%xb1(box%n1+1)
endif
!
if (m2eff > m2eff0) then
  call rhd_box_Copy(box   , m1eff     , n1eff , n2first   , box%n2, m3eff     , n3eff , &
                    boxout, m1effout  ,         m2effout0 ,         m3effout  , &
                    grid_mode=0, flux_mode=0)
    ! --- Copy grid ---
    boxout%xc2(m2effout0:m2effout-1)=(box%xc2(n2first:box%n2) - box%xb2(box%n2+1)) + &
                                      box%xb2(box%m2)
    boxout%xb2(m2effout0:m2effout-1)=(box%xb2(n2first:box%n2) - box%xb2(box%n2+1)) + &
                                      box%xb2(box%m2)
endif
!
if (n2eff0 > n2eff) then
  call rhd_box_Copy(box   , m1eff     , n1eff , box%m2    , m2last, m3eff     , n3eff , &
                    boxout, m1effout  ,         n2effout+1,         m3effout  , &
                    grid_mode=0, flux_mode=0)
    ! --- Copy grid ---
    boxout%xc2(n2effout+1:n2effout0  )=(box%xc2(box%m2  :m2last  ) - box%xb2(box%m2)) + &
                                        box%xb2(box%n2+1)
    boxout%xb2(n2effout+2:n2effout0+1)=(box%xb2(box%m2+1:m2last+1) - box%xb2(box%m2)) + &
                                        box%xb2(box%n2+1)
endif
!
if (m3eff > m3eff0) then
  call rhd_box_Copy(box   , m1eff     , n1eff , m2eff     , n2eff , n3first   , box%n3, &
                    boxout, m1effout  ,         m2effout  ,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
    ! --- Copy grid ---
    boxout%xc3(m3effout0:m3effout-1)=(box%xc3(n3first:box%n3) - box%xb3(box%n3+1)) + &
                                      box%xb3(box%m3)
    boxout%xb3(m3effout0:m3effout-1)=(box%xb3(n3first:box%n3) - box%xb3(box%n3+1)) + &
                                      box%xb3(box%m3)
endif
!
if (n3eff0 > n3eff) then
  call rhd_box_Copy(box   , m1eff     , n1eff , m2eff     , n2eff , box%m3    , m3last, &
                    boxout, m1effout  ,         m2effout  ,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
    ! --- Copy grid ---
    boxout%xc3(n3effout+1:n3effout0  )=(box%xc3(box%m3  :m3last  ) - box%xb3(box%m3)) + &
                                       box%xb3(box%n3+1)
    boxout%xb3(n3effout+2:n3effout0+1)=(box%xb3(box%m3+1:m3last+1) - box%xb3(box%m3)) + &
                                       box%xb3(box%n3+1)
endif
!
! --- Edges ---
!
if (m1eff > m1eff0 .and. m2eff > m2eff0) then
  call rhd_box_Copy(box   , n1first   , box%n1, n2first   , box%n2, m3eff     , n3eff , &
                    boxout, m1effout0 ,         m2effout0 ,         m3effout  , &
                    grid_mode=0, flux_mode=0)
endif
!
if (m1eff > m1eff0 .and. n2eff0 > n2eff) then
  call rhd_box_Copy(box   , n1first   , box%n1, box%m2    , m2last, m3eff     , n3eff , &
                    boxout, m1effout0 ,         n2effout+1,         m3effout  , &
                    grid_mode=0, flux_mode=0)
endif
!
if (m1eff > m1eff0 .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , n1first   , box%n1, m2eff     , n2eff , n3first   , box%n3, &
                    boxout, m1effout0 ,         m2effout  ,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (m1eff > m1eff0 .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , n1first   , box%n1, m2eff     , n2eff , box%m3    , m3last, &
                    boxout, m1effout0 ,         m2effout  ,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. m2eff > m2eff0) then
  call rhd_box_Copy(box   , box%m1    , m1last, n2first   , box%n2, m3eff     , n3eff , &
                    boxout, n1effout+1,         m2effout0 ,         m3effout  , &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. n2eff0 > n2eff) then
  call rhd_box_Copy(box   , box%m1    , m1last, box%m2    , m2last, m3eff     , n3eff , &
                    boxout, n1effout+1,         n2effout+1,         m3effout  , &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , box%m1    , m1last, m2eff     , n2eff , n3first   , box%n3, &
                    boxout, n1effout+1,         m2effout  ,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , box%m1    , m1last, m2eff     , n2eff , box%m3    , m3last, &
                    boxout, n1effout+1,         m2effout  ,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
if (m2eff > m2eff0 .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , m1eff     , n1eff , n2first   , box%n2, n3first   , box%n3, &
                    boxout, m1effout  ,         m2effout0 ,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (m2eff > m2eff0 .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , m1eff     , n1eff , n2first   , box%n2, box%m3    , m3last, &
                    boxout, m1effout  ,         m2effout0 ,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
if (n2eff0 > n2eff .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , m1eff     , n1eff , box%m2    , m2last, n3first   , box%n3, &
                    boxout, m1effout  ,         n2effout+1,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (n2eff0 > n2eff .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , m1eff     , n1eff , box%m2    , m2last, box%m3    , m3last, &
                    boxout, m1effout  ,         n2effout+1,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
! --- Corners ---
!
if (m1eff > m1eff0 .and. m2eff > m2eff0 .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , n1first   , box%n1, n2first   , box%n2, n3first   , box%n3, &
                    boxout, m1effout0 ,         m2effout0 ,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (m1eff > m1eff0 .and. m2eff > m2eff0 .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , n1first   , box%n1, n2first   , box%n2, box%m3    , m3last, &
                    boxout, m1effout0 ,         m2effout0 ,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
if (m1eff > m1eff0 .and. n2eff0 > n2eff .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , n1first   , box%n1, box%m2    , m2last, n3first   , box%n3, &
                    boxout, m1effout0 ,         n2effout+1,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (m1eff > m1eff0 .and. n2eff0 > n2eff .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , n1first   , box%n1, box%m2    , m2last, box%m3    , m3last, &
                    boxout, m1effout0 ,         n2effout+1,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. m2eff > m2eff0 .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , box%m1    , m1last, n2first   , box%n2, n3first   , box%n3, &
                    boxout, n1effout+1,         m2effout0 ,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. m2eff > m2eff0 .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , box%m1    , m1last, n2first   , box%n2, box%m3    , m3last, &
                    boxout, n1effout+1,         m2effout0 ,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. n2eff0 > n2eff .and. m3eff > m3eff0) then
  call rhd_box_Copy(box   , box%m1    , m1last, box%m2    , m2last, n3first   , box%n3, &
                    boxout, n1effout+1,         n2effout+1,         m3effout0 , &
                    grid_mode=0, flux_mode=0)
endif
!
if (n1eff0 > n1eff .and. n2eff0 > n2eff .and. n3eff0 > n3eff) then
  call rhd_box_Copy(box   , box%m1    , m1last, box%m2    , m2last, box%m3    , m3last, &
                    boxout, n1effout+1,         n2effout+1,         n3effout+1, &
                    grid_mode=0, flux_mode=0)
endif
!
!
! --- Check for nonperiodic boundary conditions ---
!
if ((m1eff > m1eff0) .and. &
    ((action%bound_l1 /= bound_periodic) .or. &
     (Bc_flag0 .and. (action%bound_mag_l1 /= bound_mag_periodic)))) then
  !
  ! --- Prepare action variable ---
  if (action%bound_l1 == bound_periodic) then
    action_current = rhd_action_New(action, dim=1, bound_l1=bound_None, bound_r1=bound_None, &
                                    bound_mag_r1=bound_mag_None)
  else if (action%bound_mag_l1 == bound_mag_periodic) then
    action_current = rhd_action_New(action, dim=1, bound_r1=bound_None, &
                                    bound_mag_l1=bound_mag_None, bound_mag_r1 = bound_mag_None)
  else
    action_current = rhd_action_New(action, dim=1, bound_r1=bound_None, bound_mag_r1=bound_mag_None)
  endif
  !
  nghostrest=m1eff-m1eff0
  m1efonew=m1effout+nghostrest-1
  !
  ! --- Standard handling of nonperiodic boundary conditions ---
  call rhd_box_Make(slice , m1=m1effout , n1=m1efonew , &
                            m2=m2effout0, n2=n2effout0, m3=m3effout0, n3=n3effout0, &
                            nghost1=nghostrest, &
                            quc_flag=quc_flag0, Bb_flag=Bb_flag0, Bc_flag=Bc_flag0)
  call rhd_box_Copy(boxout, m1effout0, m1efonew , &
                            m2effout0, n2effout0, m3effout0, n3effout0, &
                    slice , m1effout0, m2effout0, m3effout0, &
                    trans_mode=1, flux_mode=0)
  !
  call rhd_bound_1DGhost(slice, action_current)
  !
  call rhd_box_Copy(slice , m1effout0, m1effout-1, &
                            m2effout0, n2effout0 , m3effout0, n3effout0, &
                    boxout, m1effout0, m2effout0 , m3effout0, &
                    trans_mode=1, flux_mode=0)
  call rhd_box_Delete(slice)
  !
endif
!
if ((n1eff0 > n1eff) .and. &
    ((action%bound_r1 /= bound_periodic) .or. &
     (Bc_flag0 .and. (action%bound_mag_r1 /= bound_mag_periodic)))) then
  !
  ! --- Prepare action variable ---
  if (action%bound_r1 == bound_periodic) then
    action_current = rhd_action_New(action, dim=1, bound_l1=bound_None, bound_r1=bound_None, &
                                    bound_mag_l1=bound_mag_None)
  else if (action%bound_mag_r1 == bound_mag_periodic) then
    action_current = rhd_action_New(action, dim=1, bound_l1=bound_None, &
                                    bound_mag_l1=bound_mag_None, bound_mag_r1 = bound_mag_None)
  else
    action_current = rhd_action_New(action, dim=1, bound_l1=bound_None, bound_mag_l1=bound_mag_None)
  endif
  !ss
  nghostrest=n1eff0-n1eff
  n1efonew=n1effout-nghostrest+1
  !
  ! --- Standard handling of nonperiodic boundary conditions ---
  call rhd_box_Make(slice , m1=n1efonew , n1=n1effout , &
                            m2=m2effout0, n2=n2effout0, m3=m3effout0, n3=n3effout0, &
                            nghost1=nghostrest, &
                            quc_flag=quc_flag0, Bb_flag=Bb_flag0, Bc_flag=Bc_flag0)
  call rhd_box_Copy(boxout, n1efonew , n1effout0, &
                            m2effout0, n2effout0, m3effout0, n3effout0, &
                    slice , n1efonew , m2effout0, m3effout0, &
                    trans_mode=1, flux_mode=0)
  !
  call rhd_bound_1DGhost(slice, action_current)
  !
  call rhd_box_Copy(slice , n1effout+1, n1effout0, m2effout0, n2effout0, m3effout0, n3effout0, &
                    boxout, n1effout+1, m2effout0, m3effout0, &
                    trans_mode=1, flux_mode=0)
  call rhd_box_Delete(slice)
  !
endif
!
if ((m2eff > m2eff0) .and. &
    ((action%bound_l2 /= bound_periodic) .or. &
     (Bc_flag0 .and. (action%bound_mag_l2 /= bound_mag_periodic)))) then
  !
  ! --- Prepare action variable ---
  if (action%bound_l2 == bound_periodic) then
    action_current = rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                    bound_l2=bound_None, bound_r2=bound_None, &
                                    bound_mag_r2=bound_mag_None)
  else if (action%bound_mag_l2 == bound_mag_periodic) then
    action_current = rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                    bound_r2=bound_None, &
                                    bound_mag_l2=bound_mag_None, bound_mag_r2 = bound_mag_None)
  else
    action_current = rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                    bound_r2=bound_None, bound_mag_r2=bound_mag_None)
  endif
  !
  nghostrest=m2eff-m2eff0
  m2efonew=m2effout+nghostrest-1
  !
  ! --- Standard handling of nonperiodic boundary conditions ---
  call rhd_box_Make(slice , m1=m2effout , n1=m2efonew , &
                            m2=m1effout0, n2=n1effout0, m3=m3effout0, n3=n3effout0, &
                            nghost1=nghostrest, &
                            quc_flag=quc_flag0, Bb_flag=Bb_flag0, Bc_flag=Bc_flag0)
  call rhd_box_Copy(boxout, m1effout0, n1effout0           , &
                            m2effout0, m2efonew , m3effout0, n3effout0, &
                    slice , m2effout0, m1effout0, m3effout0, &
                    trans_mode=2, flux_mode=0)
  !
  call rhd_bound_1DGhost(slice, action_current)
  !
  call rhd_box_Copy(slice , m2effout0, m2effout-1, &
                            m1effout0, n1effout0 , m3effout0, n3effout0, &
                    boxout, m1effout0, m2effout0 , m3effout0, &
                    trans_mode=2, flux_mode=0)
  call rhd_box_Delete(slice)
  !
endif
!
if ((n2eff0 > n2eff) .and. &
    ((action%bound_r2 /= bound_periodic) .or. &
     (Bc_flag0 .and. (action%bound_mag_r2 /= bound_mag_periodic)))) then
  !
  ! --- Prepare action variable ---
  if (action%bound_r2 == bound_periodic) then
    action_current = rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                    bound_l2=bound_None, bound_r2=bound_None, &
                                    bound_mag_l2=bound_mag_None)
  else if (action%bound_mag_r2 == bound_mag_periodic) then
    action_current = rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                    bound_l2=bound_None, &
                                    bound_mag_l2=bound_mag_None, bound_mag_r2 = bound_mag_None)
  else
    action_current = rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                    bound_l2=bound_None, bound_mag_l2=bound_mag_None)
  endif
  !
  nghostrest=n2eff0-n2eff
  n2efonew=n2effout-nghostrest+1
  !
  ! --- Standard handling of nonperiodic boundary conditions ---
  call rhd_box_Make(slice , m1=n2efonew , n1=n2effout , &
                            m2=m1effout0, n2=n1effout0, m3=m3effout0, n3=n3effout0, &
                            nghost1=nghostrest, &
                            quc_flag=quc_flag0, Bb_flag=Bb_flag0, Bc_flag=Bc_flag0)
  call rhd_box_Copy(boxout, m1effout0, n1effout0, &
                            n2efonew , n2effout0, m3effout0, n3effout0, &
                    slice , n2efonew , m1effout0, m3effout0, &
                    trans_mode=2, flux_mode=0)
  !
  call rhd_bound_1DGhost(slice, action_current)
  !
  call rhd_box_Copy(slice , n2effout+1, n2effout0, m1effout0, n1effout0, m3effout0, n3effout0, &
                    boxout, m1effout0, n2effout+1, m3effout0, &
                    trans_mode=2, flux_mode=0)
  call rhd_box_Delete(slice)
  !
endif
!
if ((m3eff > m3eff0) .and. &
    ((action%bound_l3 /= bound_periodic) .or. &
     (Bc_flag0 .and. (action%bound_mag_l3 /= bound_mag_periodic)))) then
  !
  ! --- Prepare action variable ---
  if (action%bound_l3 == bound_periodic) then
    action_current = rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                    bound_l3=bound_None, bound_r3=bound_None, &
                                    bound_mag_r3=bound_mag_None)
  else if (action%bound_mag_l3 == bound_mag_periodic) then
    action_current = rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                    bound_r3=bound_None, &
                                    bound_mag_l3=bound_mag_None, bound_mag_r3 = bound_mag_None)
  else
    action_current = rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                    bound_r3=bound_None, bound_mag_r3=bound_mag_None)
  endif
  !
  nghostrest=m3eff-m3eff0
  m3efonew=m3effout+nghostrest-1
  !
  ! --- Standard handling of nonperiodic boundary conditions ---
  call rhd_box_Make(slice , m1=m3effout , n1=m3efonew , &
                            m2=m2effout0, n2=n2effout0, m3=m1effout0, n3=n1effout0, &
                            nghost1=nghostrest, &
                            quc_flag=quc_flag0, Bb_flag=Bb_flag0, Bc_flag=Bc_flag0)
  call rhd_box_Copy(boxout, m1effout0, n1effout0, &
                            m2effout0, n2effout0, m3effout0, m3efonew , &
                    slice , m3effout0, m2effout0, m1effout0, &
                    trans_mode=3, flux_mode=0)
  !
  call rhd_bound_1DGhost(slice, action_current)
  !
  call rhd_box_Copy(slice , m3effout0, m3effout-1, &
                            m2effout0, n2effout0 , m1effout0, n1effout0, &
                    boxout, m1effout0, m2effout0 , m3effout0, &
                    trans_mode=3, flux_mode=0)
  call rhd_box_Delete(slice)
  !
endif
!
if ((n3eff0 > n3eff) .and. &
    ((action%bound_r3 /= bound_periodic) .or. &
     (Bc_flag0 .and. (action%bound_mag_r3 /= bound_mag_periodic)))) then
  !
  ! --- Prepare action variable ---
  if (action%bound_r3 == bound_periodic) then
    action_current = rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                    bound_l3=bound_None, bound_r3=bound_None, &
                                    bound_mag_l3=bound_mag_None)
  else if (action%bound_mag_r3 == bound_mag_periodic) then
    action_current = rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                    bound_l3=bound_None, &
                                    bound_mag_l3=bound_mag_None, bound_mag_r3 = bound_mag_None)
  else
    action_current = rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                    bound_l3=bound_None, bound_mag_l3=bound_mag_None)
  endif
  !
  nghostrest=n3eff0-n3eff
  n3efonew=n3effout-nghostrest+1
  !
  ! --- Standard handling of nonperiodic boundary conditions ---
  call rhd_box_Make(slice , m1=n3efonew , n1=n3effout , &
                            m2=m2effout0, n2=n2effout0, m3=m1effout0, n3=n1effout0, &
                            nghost1=nghostrest, &
                            quc_flag=quc_flag0, Bb_flag=Bb_flag0, Bc_flag=Bc_flag0)
  call rhd_box_Copy(boxout, m1effout0, n1effout0, &
                            m2effout0, n2effout0, n3efonew , n3effout0, &
                    slice , n3efonew , m2effout0, m1effout0, &
                    trans_mode=3, flux_mode=0)
  !
  call rhd_bound_1DGhost(slice, action_current)
  !
  call rhd_box_Copy(slice , n3effout+1, n3effout0, m2effout0, n2effout0, m1effout0, n1effout0, &
                    boxout, m1effout0, m2effout0, n3effout+1, &
                    trans_mode=3, flux_mode=0)
  call rhd_box_Delete(slice)
  !
endif
!
end subroutine rhd_bound_ExtractBox


!----------***********************------------------------------------------------------------------
subroutine rhd_bound_ExtractBoxMPI(box, action, m1,n1, m2,n2, m3,n3, nghost, &
                                   boxout, m1out, m2out, m3out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_bound_ExtractBoxMPI ('rhd_boundaries_Extract_Box_MPI')
!
! PURPOSE:
!   Extract part of box, fill boundary ghost cells properly.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_bound_ExtractBoxMPI(box, action, m1,n1, m2,n2, m3,n3, nghost, &
!                                boxout, m1out, m2out, m3out)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   action:       ('action') derived type with collection of control parameters
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost:       ('number_ghost') integer, number of ghost cells
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_box_Copy:          Copy (part of) box into another. Transpose, if requested
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Make:          Create computational box of requested size
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines for RHD
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Copy.
!   Time specifications are not copied.
!   Magnetic-field arrays are not copied.
!
! PROCEDURE:
!   Grid (1st index):
!
!       +         +      +             +       +        +
!     box%m1  m1-nghost  m1    ...     n1  n1+nghost  box%n1
!             m1eff                        n1eff
!   or e.g.
!        +         +     +             +     +         +
!     m1-nghost  box%m1  m1    ...     n1  box%n1  n1+nghost
!                m1eff                     n1eff
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-05-25 (B.F. Copenhagen) First version
!   1999-05-31 (B.F.) More calls of copy_box
!   1999-06-07 (B.F.) Index correction
!   1999-09-02 (B.F.) 2D case: check 2nd dimension of boxout
!   2010-04-20 (B.F. Lyon) Rename extract_box -> rhd_box_Extract
!   2012-11-19 (B.F.) Modification of an old (pre 2010-12-10) version of rhd_box_Extract
!   2017-08-24 (B.F.) Bugfix: copy at least 2 layers, if possible, for higher-order bnd. conditions
!   2017-09-18 (B.F. Uppsala) Rename rhd_box_ExtractMPI -> rhd_bound_ExtractBoxMPI
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
type(action_type),         intent(in)   :: action
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1out, m2out, m3out
!
! --- Local variables ---
integer                                 :: n1out, n2out, n3out, &
                                           m1eff0, n1eff0, m2eff0, n2eff0, m3eff0, n3eff0, &
                                           m1effout0, n1effout0, &
                                           m2effout0, n2effout0, &
                                           m3effout0, n3effout0, &
                                           m1eff, n1eff, m2eff, n2eff, m3eff, n3eff, &
                                           m1effout, n1effout, &
                                           m2effout, n2effout, &
                                           m3effout, n3effout, &
                                           n1first, m1last, n2first, m2last, n3first, m3last, &
                                           m1efonew, n1efonew, &
                                           m2efonew, n2efonew, &
                                           m3efonew, n3efonew, &
                                           m1copy, n1copy, &
                                           m2copy, n2copy, &
                                           m3copy, n3copy, &
                                           nghostrest, &
                                           grid_mode
type(box_type)                          :: slice
!---------------------------------------------------------------------------------------------------
!
! === Prepare indices of input (source) and output (target) box ====================================
!
! --- Compute the right target boundary values of the output box ---
n1out=m1out+n1-m1
n2out=m2out+n2-m2
n3out=m3out+n3-m3
!
! --- Determine the initial effective source indices for copying:                               ---
! --- If requested sub-box lies within larger box, there is no need for extra boundary handling ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
! --- Account for the larger source box if MPI is activated: there are always enough layers ---
! --- for the ghost cells (already filled, including the axes)                              ---
!
if ((action%bound_l1 == bound_InternalNext    ) .or. &
    (action%bound_l1 == bound_InternalPeriodic)     ) then
  m1eff0=            m1-nghost
else
  m1eff0=max(box%m1, m1-nghost)
endif
if ((action%bound_r1 == bound_InternalNext    ) .or. &
    (action%bound_r1 == bound_InternalPeriodic)     ) then
  n1eff0=            n1+nghost
else
  n1eff0=min(box%n1, n1+nghost)
endif
!
if ((action%bound_l2 == bound_InternalNext    ) .or. &
    (action%bound_l2 == bound_InternalPeriodic)     ) then
  m2eff0=            m2-nghost
else
  m2eff0=max(box%m2, m2-nghost)
endif
if ((action%bound_r2 == bound_InternalNext    ) .or. &
    (action%bound_r2 == bound_InternalPeriodic)     ) then
  n2eff0=            n2+nghost
else
  n2eff0=min(box%n2, n2+nghost)
endif
!
if ((action%bound_l3 == bound_InternalNext    ) .or. &
    (action%bound_l3 == bound_InternalPeriodic)     ) then
  m3eff0=            m3-nghost
else
  m3eff0=max(box%m3, m3-nghost)
endif
if ((action%bound_r3 == bound_InternalNext    ) .or. &
    (action%bound_r3 == bound_InternalPeriodic)     ) then
  n3eff0=            n3+nghost
else
  n3eff0=min(box%n3, n3+nghost)
endif
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
m1eff0=max(box%m1, m1-nghost)
n1eff0=min(box%n1, n1+nghost)
m2eff0=max(box%m2, m2-nghost)
n2eff0=min(box%n2, n2+nghost)
m3eff0=max(box%m3, m3-nghost)
n3eff0=min(box%n3, n3+nghost)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
! --- Set mode to fill axes of "real-outer-boundary" layers ---
grid_mode=1
!
! --- Compute the corresponding initial target indices ---
m1effout0=m1out     + m1eff0-m1
n1effout0=m1effout0 + n1eff0-m1eff0
m2effout0=m2out     + m2eff0-m2
n2effout0=m2effout0 + n2eff0-m2eff0
m3effout0=m3out     + m3eff0-m3
n3effout0=m3effout0 + n3eff0-m3eff0
!
! --- Set the current index values to the initial values ---
m1eff=m1eff0
n1eff=n1eff0
m2eff=m2eff0
n2eff=n2eff0
m3eff=m3eff0
n3eff=n3eff0
m1effout=m1effout0
n1effout=n1effout0
m2effout=m2effout0
n2effout=n2effout0
m3effout=m3effout0
n3effout=n3effout0
!
! === Copy the bulk of the quantities in one go ====================================================
!
! --- Copy HD quantities (internal and MPI-ghost-zone values) ---
call rhd_box_Copy(box   , m1eff   , n1eff, m2eff   , n2eff   , m3eff ,n3eff, &
                  boxout, m1effout,        m2effout,           m3effout)
!
! === Check boundary conditions: periodic boundary conditions first ================================
!
!if (box%n1 > box%m1) then
  ! --- Fill ghost cells at bound_l1 ---
  nghostrest=m1eff - (m1-nghost)
  if ((nghostrest > 0) .and. (action%bound_l1 == bound_periodic)) then
    n1first =box%n1-nghostrest+1
    m1efonew=m1out-nghost
    ! --- Copy HD quantities according to periodic boundary conditions ---
    call rhd_box_Copy(box,    n1first , box%n1  , m2eff   ,n2eff, m3eff   ,n3eff, &
                      boxout, m1efonew,           m2effout,       m3effout, &
                      grid_mode=0)
    !
    if (grid_mode == 1) then
      ! --- Copy grid ---
      boxout%xc1(m1efonew:m1effout-1)=(box%xc1(n1first:box%n1) - box%xb1(box%n1+1)) + &
                                       box%xb1(box%m1)
      boxout%xb1(m1efonew:m1effout-1)=(box%xb1(n1first:box%n1) - box%xb1(box%n1+1)) + &
                                       box%xb1(box%m1)
    endif
    !
    m1eff   =m1   -nghost
    m1effout=m1out-nghost
  endif
  !
  ! --- Fill ghost cells at bound_r1 ---
  nghostrest=(n1+nghost) - n1eff
  if ((nghostrest > 0) .and. (action%bound_r1 == bound_periodic)) then
    m1last  =box%m1+nghostrest-1
    n1efonew=n1out+nghost
    ! --- Copy HD quantities according to periodic boundary conditions ---
    call rhd_box_Copy(box,    box%m1    , m1last, m2eff   ,n2eff, m3eff   ,n3eff, &
                      boxout, n1effout+1,         m2effout,       m3effout, &
                      grid_mode=0)
    !
    if (grid_mode == 1) then
      ! --- Copy grid ---
      boxout%xc1(n1effout+1:n1efonew  )=(box%xc1(box%m1  :m1last  ) - box%xb1(box%m1)) + &
                                         box%xb1(box%n1+1)
      boxout%xb1(n1effout+2:n1efonew+1)=(box%xb1(box%m1+1:m1last+1) - box%xb1(box%m1)) + &
                                         box%xb1(box%n1+1)
    endif
    !
    n1eff   =n1   +nghost
    n1effout=n1out+nghost
  endif
!endif
!
if ((boxout%n2+boxout%nghost2) > boxout%m2) then
  ! --- Fill ghost cells at bound_l2 ---
  nghostrest=m2eff - (m2-nghost)
  if ((nghostrest > 0) .and. (action%bound_l2 == bound_periodic)) then
    n2first =box%n2-nghostrest+1
    m2efonew=m2out-nghost
    ! --- Copy HD quantities according to periodic boundary conditions ---
    call rhd_box_Copy(  box,    m1eff0    ,n1eff0 , n2first , box%n2, m3eff0   ,n3eff0, &
                        boxout, m1effout0 ,         m2efonew,         m3effout0, &
                        grid_mode=0)
    if ((m1eff < m1eff0) .and. (action%bound_l1 == bound_periodic)) then
      call rhd_box_Copy(box   , n1first   , box%n1, n2first , box%n2, m3eff0   ,n3eff0, &
                        boxout, m1effout  ,         m2efonew,         m3effout0, &
                        grid_mode=0)
    endif
    if ((n1eff0 < n1eff) .and. (action%bound_r1 == bound_periodic)) then
      call rhd_box_Copy(box   , box%m1    , m1last, n2first , box%n2, m3eff0   ,n3eff0, &
                        boxout, n1effout0+1,        m2efonew,         m3effout0, &
                        grid_mode=0)
    endif
    !
    if (grid_mode == 1) then
      ! --- Copy grid ---
      boxout%xc2(m2efonew:m2effout-1)=(box%xc2(n2first:box%n2) - box%xb2(box%n2+1)) + &
                                       box%xb2(box%m2)
      boxout%xb2(m2efonew:m2effout-1)=(box%xb2(n2first:box%n2) - box%xb2(box%n2+1)) + &
                                       box%xb2(box%m2)
    endif
    !
    m2eff   =m2   -nghost
    m2effout=m2out-nghost
  endif
  !
  ! --- Fill ghost cells at bound_r2 ---
  nghostrest=(n2+nghost) - n2eff
  if ((nghostrest > 0) .and. (action%bound_r2 == bound_periodic)) then
    m2last  =box%m2+nghostrest-1
    n2efonew=n2out+nghost
    ! --- Copy HD quantities according to periodic boundary conditions ---
    call rhd_box_Copy(  box,    m1eff0    ,n1eff0 , box%m2    , m2last, m3eff0   ,n3eff0, &
                        boxout, m1effout0 ,         n2effout+1,         m3effout0, &
                        grid_mode=0)
    if ((m1eff < m1eff0) .and. (action%bound_l1 == bound_periodic)) then
      call rhd_box_Copy(box   , n1first   , box%n1, box%m2    , m2last, m3eff0   ,n3eff0, &
                        boxout, m1effout  ,         n2effout+1,         m3effout0, &
                        grid_mode=0)
    endif
    if ((n1eff0 < n1eff) .and. (action%bound_r1 == bound_periodic)) then
      call rhd_box_Copy(box   , box%m1    , m1last, box%m2    , m2last, m3eff0   ,n3eff0, &
                        boxout, n1effout0+1,        n2effout+1,         m3effout0, &
                        grid_mode=0)
    endif
    !
    if (grid_mode == 1) then
      ! --- Copy grid ---
      boxout%xc2(n2effout+1:n2efonew  )=(box%xc2(box%m2  :m2last  ) - box%xb2(box%m2)) + &
                                         box%xb2(box%n2+1)
      boxout%xb2(n2effout+2:n2efonew+1)=(box%xb2(box%m2+1:m2last+1) - box%xb2(box%m2)) + &
                                         box%xb2(box%n2+1)
    endif
    !
    n2eff   =n2   +nghost
    n2effout=n2out+nghost
  endif
endif
!
!if (box%n3 > box%m3) then
  ! --- Fill ghost cells at bound_l3 ---
  nghostrest=m3eff - (m3-nghost)
  if ((nghostrest > 0) .and. (action%bound_l3 == bound_periodic)) then
    n3first =box%n3-nghostrest+1
    m3efonew=m3out-nghost
    ! --- Copy HD quantities according to periodic boundary conditions ---
    call rhd_box_Copy(box,    m1eff   ,n1eff, m2eff   ,n2eff, n3first , box%n3, &
                      boxout, m1effout,       m2effout,       m3efonew, &
                      grid_mode=0)
    !
    if (grid_mode == 1) then
      ! --- Copy grid ---
      boxout%xc3(m3efonew:m3effout-1)=(box%xc3(n3first:box%n3) - box%xb3(box%n3+1)) + &
                                       box%xb3(box%m3)
      boxout%xb3(m3efonew:m3effout-1)=(box%xb3(n3first:box%n3) - box%xb3(box%n3+1)) + &
                                       box%xb3(box%m3)
    endif
    !
    m3eff   =m3   -nghost
    m3effout=m3out-nghost
  endif
  !
  ! --- Fill ghost cells at bound_r3 ---
  nghostrest=(n3+nghost) - n3eff
  if ((nghostrest > 0) .and. (action%bound_r3 == bound_periodic)) then
    m3last  =box%m3+nghostrest-1
    n3efonew=n3out+nghost
    ! --- Copy HD quantities according to periodic boundary conditions ---
    call rhd_box_Copy(box,    m1eff   ,n1eff, m2eff   ,n2eff, box%m3    , m3last, &
                      boxout, m1effout,       m2effout,       n3effout+1, &
                      grid_mode=0)
    !
    if (grid_mode == 1) then
      ! --- Copy grid ---
      boxout%xc3(n3effout+1:n3efonew  )=(box%xc3(box%m3  :m3last  ) - box%xb3(box%m3)) + &
                                         box%xb3(box%n3+1)
      boxout%xb3(n3effout+2:n3efonew+1)=(box%xb3(box%m3+1:m3last+1) - box%xb3(box%m3)) + &
                                         box%xb3(box%n3+1)
    endif
    !
    n3eff   =n3   +nghost
    n3effout=n3out+nghost
  endif
!endif
!
! === Handle other boundary conditions =============================================================
!
!if (box%n1 > box%m1) then
  ! --- Fill ghost cells at bound_l1 ---
  nghostrest=m1eff - (m1-nghost)
  if ((nghostrest       > 0                     ) .and. &
      (action%bound_l1 /= bound_periodic        ) .and. &
      (action%bound_l1 /= bound_InternalNext    ) .and. &
      (action%bound_l1 /= bound_InternalPeriodic)     ) then
    n1first =box%n1-nghostrest+1
    m1efonew=m1out-nghost
    n1copy=min(m1effout+max(nghostrest-1, 1), n1)
    ! --- Standard handling of 1D boundary conditions ---
    call rhd_box_Make(slice , m1=m1effout, n1=n1copy, &
                              m2=m2effout, n2=n2effout, m3=m3effout, n3=n3effout, &
                              nghost1=nghostrest)
    call rhd_box_Copy(boxout, m1effout, n1copy, m2effout, n2effout, m3effout, n3effout, &
                      slice , m1effout,         m2effout,           m3effout, &
                      grid_mode=grid_mode, trans_mode=1)
    call rhd_bound_1DGhost(slice, rhd_action_New(action, dim=1, bound_r1=bound_None), &
                           grid_mode=grid_mode)
    call rhd_box_Copy(slice , m1efonew, m1effout-1, m2effout, n2effout, m3effout, n3effout, &
                      boxout, m1efonew,             m2effout,           m3effout, &
                      grid_mode=grid_mode, trans_mode=1)
    call rhd_box_Delete(slice)
    m1eff   =m1   -nghost
    m1effout=m1out-nghost
  endif
  !
  ! --- Fill ghost cells at bound_r1 ---
  nghostrest=(n1+nghost) - n1eff
  if ((nghostrest       > 0                     ) .and. &
      (action%bound_r1 /= bound_periodic        ) .and. &
      (action%bound_r1 /= bound_InternalNext    ) .and. &
      (action%bound_r1 /= bound_InternalPeriodic)     ) then
    m1last  =box%m1+nghostrest-1
    n1efonew=n1out+nghost
    m1copy=max(n1effout-max(nghostrest-1, 1), m1)
    ! --- Standard handling of 1D boundary conditions ---
    call rhd_box_Make(slice, m1=m1copy, n1=n1effout, &
                             m2=m2effout, n2=n2effout, m3=m3effout, n3=n3effout, &
                             nghost1=nghostrest)
    call rhd_box_Copy(boxout, m1copy, n1effout, m2effout, n2effout, m3effout, n3effout, &
                      slice , m1copy,           m2effout,           m3effout, &
                      grid_mode=grid_mode, trans_mode=1)
    call rhd_bound_1DGhost(slice, rhd_action_New(action, dim=1, bound_l1=bound_None), &
                           grid_mode=grid_mode)
    call rhd_box_Copy(slice , n1effout+1, n1efonew, m2effout, n2effout, m3effout, n3effout, &
                      boxout, n1effout+1,           m2effout,           m3effout, &
                      grid_mode=grid_mode, trans_mode=1)
    call rhd_box_Delete(slice)
    n1eff   =n1   +nghost
    n1effout=n1out+nghost
  endif
!endif
!
if ((boxout%n2+boxout%nghost2) > boxout%m2) then
  ! --- Fill ghost cells at bound_l2 ---
  nghostrest=m2eff - (m2-nghost)
  if ((nghostrest       > 0                     ) .and. &
      (action%bound_l2 /= bound_periodic        ) .and. &
      (action%bound_l2 /= bound_InternalNext    ) .and. &
      (action%bound_l2 /= bound_InternalPeriodic)     ) then
    n2first =box%n2-nghostrest+1
    m2efonew=m2out-nghost
    n2copy=min(m2effout+max(nghostrest-1, 1), n2)
!!$write(6,'(A,2(3(2I4,2X),3X))') 'rhd_bound_ExtractBoxMPI: bound_l2: ', &
!!$  m1,n1, m2,n2, m3,n3,  m1effout,n1effout, m2effout,m2copy, m3effout,n3effout
!!$call flush(6)
    ! --- Standard handling of 1D boundary conditions ---
    call rhd_box_Make(slice , m1=m2effout, n1=n2copy, &
                              m2=m1effout, n2=n1effout, m3=m3effout, n3=n3effout, &
                              nghost1=nghostrest)
    call rhd_box_Copy(boxout, m1effout, n1effout, m2effout, n2copy, m3effout, n3effout, &
                      slice,  m2effout,           m1effout,         m3effout, &
                      grid_mode=grid_mode, trans_mode=2)
    call rhd_bound_1DGhost(slice, rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                                 bound_r2=bound_None), grid_mode=grid_mode)
    call rhd_box_Copy(slice , m2efonew, m2effout-1, m1effout, n1effout, m3effout, n3effout, &
                      boxout, m1effout,             m2efonew,           m3effout, &
                      grid_mode=grid_mode, trans_mode=2)
    call rhd_box_Delete(slice)
    m2eff   =m2   -nghost
    m2effout=m2out-nghost
  endif
  !
  ! --- Fill ghost cells at bound_r2 ---
  nghostrest=(n2+nghost) - n2eff
  if ((nghostrest       > 0                     ) .and. &
      (action%bound_r2 /= bound_periodic        ) .and. &
      (action%bound_r2 /= bound_InternalNext    ) .and. &
      (action%bound_r2 /= bound_InternalPeriodic)     ) then
    m2last  =box%m2+nghostrest-1
    n2efonew=n2out+nghost
    m2copy=max(n2effout-max(nghostrest-1, 1), m2)
    ! --- Standard handling of 1D boundary conditions ---
    call rhd_box_Make(slice , m1=m2copy, n1=n2effout, &
                              m2=m1effout, n2=n1effout, m3=m3effout, n3=n3effout, &
                              nghost1=nghostrest)
    call rhd_box_Copy(boxout, m1effout, n1effout, m2copy, n2effout, m3effout, n3effout, &
                      slice , m2copy  ,           m1effout,         m3effout, &
                      grid_mode=grid_mode, trans_mode=2)
    call rhd_bound_1DGhost(slice, rhd_action_New(action, dim=2, dimV=((/2, 1, 3/)), &
                                                 bound_l2=bound_None), grid_mode=grid_mode)
    call rhd_box_Copy(slice , n2effout+1, n2efonew, m1effout, n1effout, m3effout, n3effout, &
                      boxout, m1effout,             n2effout+1,         m3effout, &
                      grid_mode=grid_mode, trans_mode=2)
    call rhd_box_Delete(slice)
    n2eff   =n2   +nghost
    n2effout=n2out+nghost
  endif
endif
!
!
!if (box%n3 > box%m3) then
  ! --- Fill ghost cells at bound_l3 ---
  nghostrest=m3eff - (m3-nghost)
  if ((nghostrest       > 0                     ) .and. &
      (action%bound_l3 /= bound_periodic        ) .and. &
      (action%bound_l3 /= bound_InternalNext    ) .and. &
      (action%bound_l3 /= bound_InternalPeriodic)     ) then
    n3first =box%n3-nghostrest+1
    m3efonew=m3out-nghost
    n3copy=min(m3effout+max(nghostrest-1, 1), n3)
    ! --- Standard handling of 1D boundary conditions ---
    call rhd_box_Make(slice, m1=m3effout, n1=n3copy, &
                             m2=m2effout, n2=n2effout, m3=m1effout, n3=n1effout, &
                             nghost1=nghostrest)
    call rhd_box_Copy(boxout, m1effout, n1effout, m2effout, n2effout, m3effout, n3copy, &
                      slice,  m3effout,           m2effout,           m1effout, &
                      grid_mode=grid_mode, trans_mode=3)
    call rhd_bound_1DGhost(slice, rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                                 bound_r3=bound_None), grid_mode=grid_mode)
    call rhd_box_Copy(slice , m3efonew, m3effout-1, m2effout, n2effout, m1effout, n1effout, &
                      boxout, m1effout,             m2effout,           m3efonew, &
                      grid_mode=grid_mode, trans_mode=3)
    call rhd_box_Delete(slice)
    m3eff   =m3   -nghost
    m3effout=m3out-nghost
  endif
  !
  ! --- Fill ghost cells at bound_r3 ---
  nghostrest=(n3+nghost) - n3eff
  if ((nghostrest       > 0                     ) .and. &
      (action%bound_r3 /= bound_periodic        ) .and. &
      (action%bound_r3 /= bound_InternalNext    ) .and. &
      (action%bound_r3 /= bound_InternalPeriodic)     ) then
    m3last  =box%m3+nghostrest-1
    n3efonew=n3out+nghost
    m3copy=max(n3effout-max(nghostrest-1, 1), m3)
    ! --- Standard handling of 1D boundary conditions ---
    call rhd_box_Make(slice , m1=m3copy, n1=n3effout, &
                              m2=m2effout, n2=n2effout, m3=m1effout, n3=n1effout, &
                              nghost1=nghostrest)
    call rhd_box_Copy(boxout, m1effout, n1effout, m2effout, n2effout, m3copy, n3effout, &
                      slice , m3copy,             m2effout,           m1effout, &
                      grid_mode=grid_mode, trans_mode=3)
    call rhd_bound_1DGhost(slice, rhd_action_New(action, dim=3, dimV=((/3, 2, 1/)), &
                                                 bound_l3=bound_None), grid_mode=grid_mode)
    call rhd_box_Copy(slice , n3effout+1, n3efonew, m2effout, n2effout, m1effout, n1effout, &
                      boxout, m1effout,             m2effout,           n3effout+1, &
                      grid_mode=grid_mode, trans_mode=3)
    call rhd_box_Delete(slice)
    n3eff   =n3   +nghost
    n3effout=n3out+nghost
  endif
!endif
!
end subroutine rhd_bound_ExtractBoxMPI

end module rhd_bound_module
