!***************************************************************************************************
!
!   #####   #    #  #####          #####    ####   #    #
!   #    #  #    #  #    #         #    #  #    #   #  #
!   #    #  ######  #    #         #####   #    #    ##
!   #####   #    #  #    #         #    #  #    #    ##
!   #   #   #    #  #    #         #    #  #    #   #  #
!   #    #  #    #  #####  ####### #####    ####   #    # _module
!
!   Radiation-Hydrodynamics Code: box-handling routines
!
!***************************************************************************************************
!   Fortran90
!   2019-03-08
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Sven Wedemeyer-Boehm:  Freiburg, Oslo
!   Werner Schaffenberger: Potsdam, Freiburg, East Lansing, Exeter
!***************************************************************************************************
! MODULES:
!   rhd_box_module:        Box-handling routines
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Gravitational potential: 0: not in data structure, 1: included in data structure --- */
# ifndef rhd_box_grav01
#   define rhd_box_grav01 0
# endif
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Number of magnetic-field arrays --- */
# ifndef rhd_box_bmag01
#   define rhd_box_bmag01 0
# endif
!#
!# /* --- Type of box array components: 0: classical pointer arrays (default) --- */
!# /* ---                               1: new allocatable arrays (faster)    --- */
# ifndef rhd_box_arrays01
#   define rhd_box_arrays01 0
# endif
# if (rhd_box_arrays01 == 0)
#   define ALLOASSOTED associated
#   define BOXARRAYTYPE pointer
# else
#   define ALLOASSOTED allocated
#   define BOXARRAYTYPE allocatable
# endif
!#
!# /* --- MPI activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------**************-------------------------------------------------------------------------------
module rhd_box_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_module ('rhd_box_module')
!
! PURPOSE:
!   Box handling routines for rhd.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_box_module
!
! TYPES:
!   box_type: Structure containing RHD bulk data together with grid:
!     m1:         ('m_1')      integer, 1. dimension, start index
!     n1:         ('number_1') integer, 1. dimension, end index
!     m2:         ('m_2')      integer, 2. dimension, start index
!     n2:         ('number_2') integer, 2. dimension, end index
!     m3:         ('m_3')      integer, 3. dimension, start index
!     n3:         ('number_3') integer, 3. dimension, end index
!     nghost1:    ('number_ghost_1') integer, 1. dimension, number of ghost cells
!     nghost2:    ('number_ghost_2') integer, 2. dimension, number of ghost cells
!     nghost3:    ('number_ghost_3') integer, 3. dimension, number of ghost cells
!     itime:      ('index_time') integer, time step number
!     ngrav:      ('number_gravity') integer (0 or 1), number of potential arrays
!     nquc:       ('number_quantities_center') integer, number of additional density arrays
!     nBc:        ('number_B_center') integer (0 or not 0), number of cell-centered
!                 magnetic-field arrays
!     nBb:        ('number_B_boundary') integer (0 or not 0), number of boundary
!                 magnetic-field arrays
!     time:       ('time') real, time
!     time_db:    ('time_double') double precision real, time
!     xc1:        ('x_center_1') real, 1. coordinate of cell centers
!     xc2:        ('x_center_2') real, 2. coordinate of cell centers
!     xc3:        ('x_center_3') real, 3. coordinate of cell centers
!     xb1:        ('x_boundary_1') real, 1. coordinate of cell boundaries
!     xb2:        ('x_boundary_2') real, 2. coordinate of cell boundaries
!     xb3:        ('x_boundary_3') real, 3. coordinate of cell boundaries
!     rho:        ('rho') real 3D array, density
!     ei:         ('energy_internal') real 3D array, internal energy density per mass
!     v1:         ('velocity_1') real 3D array, velocity, 1. coordinate
!     v2:         ('velocity_2') real 3D array, velocity, 2. coordinate
!     v3:         ('velocity_3') real 3D array, velocity, 3. coordinate
!     phic_grav:  ('phi_center_gravitation') real 3D array, cell-centered gravitational potential
!     quc:        ('quantity_central') real 4D array, 1D set of cell-centered 3D quantities
!     Bc1:        ('B_center_1') real 3D array, cell-centered magnetic field, 1. component
!     Bc2:        ('B_center_2') real 3D array, cell-centered magnetic field, 2. component
!     Bc3:        ('B_center_3') real 3D array, cell-centered magnetic field, 3. component
!     Bb1:        ('B_boundary_1') real 3D array, cell-boundary magnetic field, 1. component
!     Bb2:        ('B_boundary_2') real 3D array, cell-boundary magnetic field, 2. component
!     Bb3:        ('B_boundary_3') real 3D array, cell-boundary magnetic field, 3. component
!     flux_B12:   ('flux_B_1_2') real 3D array of flux of 1. component of B in x2 direction
!     flux_B13:   ('flux_B_1_2') real 3D array of flux of 1. component of B in x3 direction
!     flux_B21:   ('flux_B_2_1') real 3D array of flux of 2. component of B in x1 direction
!     flux_B23:   ('flux_B_2_3') real 3D array of flux of 2. component of B in x3 direction
!     flux_B31:   ('flux_B_3_1') real 3D array of flux of 3. component of B in x1 direction
!     flux_B32:   ('flux_B_3_2') real 3D array of flux of 3. component of B in x2 direction
!     energy_switch: ('energy_switch') logical 3D array to switch between entropy and energy solver
!
! ROUTINES: (contained)
!   rhd_box_SwitchInfo:    Print information about used compiler switches
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_RestIndices:   Compute indices of boxes surrounding small box in large box
!   rhd_box_SubBoxIndices: Compute indices of box representing the section of both input boxes
!   rhd_box_FillAxes:      Fill axes of box
!   rhd_box_FillGhostAxis1D: Fill ghost cells of axis vectors for real boundaries
!   rhd_box_FillGhostAxes: Fill ghost cells of box axes for real boundaries
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!   rhd_box_ZeroOMP:       Set bulk arrays explicitly to zero
!   rhd_box_FillTestOMP:   Set bulk arrays in box structure to some test values
!   rhd_box_Exist:         Function: Test if box is already filled
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_RemoveBc:      Remove cell-centered magnetic-field arrays from box
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_AppendBc:      Add cell-centered magnetic-field arrays to box
!   rhd_box_Copy:          Copy (part of) box into another. Transpose and shift, if requested
!   rhd_box_AddDelta:      Add delta of (part of) box into another. Transpose, if requested
!   rhd_box_AddDelta2:     Add delta of (part of) box into another
!   rhd_box_AddChange:     Add change of (part of) box into another. Transpose, if requested
!   rhd_box_Add:           Add (part of) box to another. Transpose, if requested
!   rhd_box_Factor:        Multiply contents of box with constant factor
!   rhd_box_AllAddDelta2OMP: Add change to entire box, OpenMP-parallelized if n3>m3
!   rhd_box_AllAddOMP:     Add box to another, OpenMP-parallelized if n3>m3
!   rhd_box_AllAvgOMP:     Average two boxes, OpenMP-parallelized if n3>m3
!   rhd_box_AllCopyOMP:    Copy entire box, OpenMP-parallelized if n3>m3
!   rhd_box_CopyOMP:       Copy (part of) box into another, OpenMP-parallelized if n3>m3
!   rhd_box_AllFactorOMP:  Multiply contents of box with const. factor, OpenMP-parallelized if n3>m3
!   rhd_box_PrintStat:     Write statistics about box to specified channel
!   rhd_box_CheckAxis1D:   Check step size of 1D axis
!   rhd_box_CheckAxes2D:   Check aspect ratio of two axes
!   rhd_box_CheckAxes:     Check axes in box and write result to specified channel
!   rhd_box_MPICopy:       Copy distribution of boxes from one MPI partition to another
!   --- MPI-only routines ---
!   rhd_box_MPICreateType: Create MPI data type for box structure
!   rhd_box_MPIFillGhostStart: Start filling "internal" ghost cells
!   rhd_box_MPIFillGhostEnd: Finalize filling ghost cells
!   rhd_box_MPIFillGhost:  Fill ghost cells by communicating with neighbor MPI processes
!   rhd_box_MPIFillGhostAxes: Fill axes in ghost layers by communicating with neighbor MPI processes
!
! MODIFICATION HISTORY:
!   1997-06-09 (B.F. Kiel) Written
!   1999-05-28 (B.F.) Box routines put into own module, box_type from rhd_gl_module,
!                     bound1d_ghost from rhd_hyd_module
!   1999-12-28 (B.F.) init_box, exist_box
!   2000-12-08 (B.F. Uppsala) rhd_box_chunksize
!   2001-01-22 (B.F.) New version of rhd_box_chunksize
!   2002-10-17 (B.F.) phic_grav, quc
!   2002-10-29 (B.F.) Use rhd_prop_module to find number of fields of additional quantities
!   2005-05-31 (W.S.&SWB) Added magnetic fluxes and entropy switch
!   2007-10-22 (B.F. Lyon) box%time_db introduced
!   2008-03-24 (W.S. East Lansing) box structure: entropy_switch -> energy_switch
!   2008-07-28 (B.F. Lyon) Check for presence of Hp_l and Hp_r moved
!   2009-01-08 (M.S.) Merged for use with latest MHD-module
!   2010-03-15 (B.F. Naples) Temporary fields for MHD handled by appendBc_box
!   2010-03-16 (B.F.) Handle energy_switch in bound1d_ghost properly; routine printstat_box added
!   2010-03-18 (B.F.) Option to switch between pointer and allocatable arrays
!   2010-04-19 (B.F. Lyon) Normalize date; rename routine
!   2010-06-03 (B.F. Freiburg) Additional box-handling routines added required by Runge-Kutta scheme
!   2010-12-10 (W.S. Exeter) Move routine rhd_box_Extract from rhd_vis_module to rhd_box_module
!   2011-02-25 (W.S. Exeter) Provisions for MHD dual-energy method
!   2011-08-03 (B.F.) Additional box-handling routines added required by CTU scheme
!   2012-11-16 (B.F.) Some routines get additional input parameters
!   2013-03-05 (B.F.) Change OpenMP scheduling (from typically GUIDED to RUNTIME), add directives
!   2014-08-17 (B.F. Uppsala) Add global variables
!   2017-07-25 (B.F.) Add routines to fill axis ghost cells and to check axes
!   2017-08-24 (B.F. Lyon) Fix issues in rhd_bound_1DGhost routine;
!                    Bugfix in rhd_box_ExtractMPI: copy enough layers for all boundary conditions
!   2017-09-18 (B.F.) Move boundary-condition routines and rhd_box_Extract to rhd_bound_module;
!                     Move MPI routines for box handling from rhd_MPI_module here and rename them
!   2017-10-07 (B.F. Upsala) Put definitions of global variables into separate module
!   2018-11-24 (B.F.) Add routine rhd_box_FillAxes and complete version of rhd_box_MPICopy
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- Type for box ---
type box_type
  integer                               :: m1, m2, m3, n1, n2, n3
  integer                               :: nghost1, nghost2, nghost3
  integer                               :: itime
  integer                               :: ngrav, nquc, nBc, nBb
  real                                  :: time
  real(kind=kind(0.0D+00))              :: time_db
  real, BOXARRAYTYPE, dimension(:)      :: xc1, xc2, xc3
  real, BOXARRAYTYPE, dimension(:)      :: xb1, xb2, xb3
  real, BOXARRAYTYPE, dimension(:,:,:)  :: rho, ei, v1, v2, v3
!#
# if (rhd_box_grav01 == 1)
!#
  real, BOXARRAYTYPE, dimension(:,:,:)  :: phic_grav
!#
# endif
!#      /* rhd_box_grav01 */
  real, BOXARRAYTYPE, dimension(:,:,:,:):: quc
  real, BOXARRAYTYPE, dimension(:,:,:)  :: Bc1, Bc2, Bc3, Bb1, Bb2, Bb3, &
                                           flux_B12, flux_B13, flux_B21, &
                                           flux_B23, flux_B31, flux_B32
  logical, BOXARRAYTYPE,dimension(:,:,:):: energy_switch
end type box_type
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_box_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_SwitchInfo ('rhd_box_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!
! CATEGORY:
!   I/O
!
! CALLING SEQUENCE:
!   call rhd_box_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   Output is written to channel ncp.
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-10-12 (B.F. Uppsala) Written
!   2010-04-20 (B.F. Lyon) rhd_box_switchinfo -> rhd_box_SwitchInfo
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_box_module .........................'
!
write(ncp,'(A,I6)') '  rhd' // '_box_grav01:             ', rhd_box_grav01
write(ncp,'(A,I6)') '  rhd' // '_box_quc01:              ', rhd_box_quc01
write(ncp,'(A,I6)') '  rhd' // '_box_bmag01:             ', rhd_box_bmag01
write(ncp,'(A,I6)') '  rhd' // '_box_arrays01:           ', rhd_box_arrays01
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
!write(ncp,'(A,A)') '  ALLO' // 'ASSOTED:                ', 'ALLOASSOTED'
!
end subroutine rhd_box_SwitchInfo


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_ChunkSize(edge_mode, n_elements_per_chunk, m1,n1, m2,n2, m3,n3, i_chunk, &
                             m2c,n2c, m3c,n3c, m1c, n1c, &
                             n_chunks, n_chunks_per_slice, n_2_per_chunk, n_3_per_chunk)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_ChunkSize ('rhd_box_Chunk_Size')
!
! PURPOSE:
!   Determine proper size of regular chunk with number of elements close to specified value
!
! CATEGORY:
!   Box handling, index arithmetic
!
! CALLING SEQUENCE:
!   call rhd_box_ChunkSize(edge_mode, n_elements_per_chunk, m1,n1, m2,n2, m3,n3, i_chunk, &
!                          m2c,n2c, m3c,n3c, &
!                          m1c=m1c, n1c=n1c, &
!                          n_chunks=n_chunks, n_chunks_per_slice=n_chunks_per_slice, &
!                          n_2_per_chunk=n_2_per_chunk, n_3_per_chunk=n_3_per_chunk)
!
! INPUT:
!   edge_mode:    ('edge_mode') integer, mode to describe aspect ratio of chunk:
!                   0: always complete columns (n1), prefer complete slices (n2)
!                   1: always complete columns (n1), prefer n2=n3
!                   2: likely partial columns (n1), prefer n1=n2=n3 (under construction)
!   n_elements_per_chunk: ('number_elements_per_chunk') integer, requested total number of
!                         elements per chunk
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   i_chunk:      ('index_chunk') integer, selected chunk number (1 <= i_chunk <= n_chunks)
!
! OUTPUT:
!   m2c:          ('m_number_2_chunk') integer, 2. dimension, start index of chunk
!   n2c:          ('number_2_chunk')   integer, 2. dimension, end index of chunk
!   m3c:          ('m_number_3_chunk') integer, 3. dimension, start index of chunk
!   n3c:          ('number_3_chunk')   integer, 3. dimension, end index of chunk
!   --- optional ---
!   m1c:          ('m_number_1_chunk') integer, 1. dimension, start index of chunk
!   n1c:          ('number_1_chunk')   integer, 1. dimension, end index of chunk
!   n_chunks:     ('number_chunks') integer, number of chunks (sub boxes) the whole box
!                 is split into
!   n_chunks_per_slice: ('number_chunks_per_slice') integer, number of chunks to put side by side
!                       in n2 direction
!   n_2_per_chunk: ('number_2_per_chunk') integer, number of columns of chunk in n2 direction
!   n_3_per_chunk: ('number_3_per_chunk') integer, number of columns of chunk in n3 direction
!
! LOCAL VARIABLES:
!   n_chunks0:    ('number_chunks_0') integer, number of chunks (sub boxes) the whole box
!                 is split into
!   n_chunks_per_slice0: ('number_chunks_per_slice_0') integer, number of chunks to put side by side
!                        in n2 direction
!   n_2_per_chunk0: ('number_2_per_chunk_0') integer, number of columns of chunk in
!                   n2 direction
!   n_3_per_chunk0: ('number_3_per_chunk_0') integer, number of columns of chunk in
!                   n3 direction
!   n_elements_per_box: ('number_elements_per_box') integer, total number of cells/elements
!                       in the box
!   n_columns_per_chunk: ('number_columns_per_chunk') integer, first guess for number of columns
!                        per chunk
!   n_banks:      ('number_banks') integer, number of banks per box;
!                 this is the number of chunks in n3 direction
!   n_slices_per_bank: ('number_slices_per_bank') integer, number of slices per bank
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   There is no check that the input box specifications are sensible,
!   i.e., that the box is non-empty and not insanely large.
!   A box size with more than 2^31 elements might cause an integer overflow.
!
! PROCEDURE:
!   Naming conventions:
!     A box has (n1-m1+1)*(n2-m2+1)*(n3-m3+1) elements.
!     A box has (n2-m2+1)*(n3-m3+1) columns.
!     A column has (n1-m1+1) elements.
!     A box has (n2-m2+1) slices.
!   edge_mode=0,1:
!     An output chunk always contains an integer number of complete columns.
!   edge_mode=0:
!     If an output chunk contains more than one column, it first takes them from the n2 direction.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-12-05 (B.F. Uppsala) Written
!   2000-12-08 (B.F.) Included into RHD code
!   2001-01-21 (B.F.) Parameter edge_mode added
!   2010-04-20 (B.F. Lyon) rhd_box_chunksize -> rhd_box_ChunkSize
!   2013-03-03 (B.F.) Rename: n_cells -> n_elements_per_box; first steps towards edge_mode=2
!   2016-04-23 (B.F. Uppsala) m1c and n1c become optional output parameters
!   2016-03-24 (B.F.) Bugfix in edge_mode=2 branch: compute n_banks
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: edge_mode, &
                                           n_elements_per_chunk, m1,n1, m2,n2, m3,n3, i_chunk
integer,                   intent(out)  :: m2c,n2c, m3c,n3c
integer,         optional, intent(out)  :: m1c,n1c, n_chunks, n_chunks_per_slice, &
                                           n_2_per_chunk, n_3_per_chunk
!
! --- Local variables ---
integer                                 :: n_chunks0, n_chunks_per_slice0, &
                                           n_1_per_chunk0, n_2_per_chunk0, n_3_per_chunk0, &
                                           n_elements_per_box, n_columns_per_chunk, &
                                           n_banks, &
                                           n_slices_per_bank, &
                                           n_chunks_per_column, &
                                           i10, i2, i3, m1c0, n1c0, &
                                           n0, n
!---------------------------------------------------------------------------------------------------
!
n_elements_per_box=(n1-m1+1)*(n2-m2+1)*(n3-m3+1)
!
if (edge_mode == 0) then ! -------------------------------------------------------------------------
  ! --- Prefer slices (n1 complete, n2 >> n3) ---
  !
  if (n_elements_per_chunk <= 0) then
    ! --- Default (old) value, choose something reasonable ---
    !
    n_columns_per_chunk=n2-m2+1
    if (n_columns_per_chunk == 1) n_columns_per_chunk=n3-m3+1
  else
    n_columns_per_chunk=(min(max(n_elements_per_chunk, n1-m1+1), n_elements_per_box)/(n1-m1+1))
  endif
  !
  if (n_columns_per_chunk >= (n2-m2+1)) then
    ! --- At least one complete slice per chunk ---
    !
    n_chunks_per_slice0=1
    n_2_per_chunk0     =(n2-m2+1)/n_chunks_per_slice0
    !
    n_slices_per_bank=n_columns_per_chunk/(n2-m2+1)
    n_banks          =(n3-m3)/n_slices_per_bank + 1
    n_3_per_chunk0   =(n3-m3)/n_banks + 1
    !
    n_chunks0=n_banks
  else
    ! --- More than one chunk per slice ---
    !
    n_slices_per_bank  =1
    n_chunks_per_slice0=(n2-m2)/n_columns_per_chunk + 1
    n_2_per_chunk0     =(n2-m2)/n_chunks_per_slice0 + 1
    !
    n_3_per_chunk0=1
    n_banks       =(n3-m3+1)/n_3_per_chunk0
    !
    n_chunks0=n_chunks_per_slice0 * n_banks
  endif
else if (edge_mode == 2) then ! --------------------------------------------------------------------
  ! --- Prefer cubical chunks (n1 = n2 = n3) ---
  !
  if (n_elements_per_chunk <= 0) then
    ! --- Default value, choose something reasonable ---
    !
    n_1_per_chunk0     =min(16, n1-m1+1)
    n_chunks_per_column=(n1-m1)/n_1_per_chunk0 + 1
    n_1_per_chunk0     =(n1-m1)/n_chunks_per_column + 1
    !
    n_2_per_chunk0     =min(16, n2-m2+1)
    n_chunks_per_slice0=(n2-m2)/n_2_per_chunk0 + 1
    n_2_per_chunk0     =(n2-m2)/n_chunks_per_slice0 + 1
    !
    n_3_per_chunk0     =min(16, n3-m3+1)
    n_slices_per_bank  =(n3-m3)/n_3_per_chunk0 + 1
    n_3_per_chunk0     =(n3-m3)/n_slices_per_bank + 1
    n_banks            =(n3-m3)/n_3_per_chunk0 + 1
  else
    ! --- Choose chunk parameters according to input number of elements per chunk ---
    !
    n0=int(float(min(n_elements_per_chunk, n_elements_per_box))**(1.0/3.0)+0.5)
    !
    n=n0
    n_1_per_chunk0     =min(n, n1-m1+1)
    n_chunks_per_column=(n1-m1)/n_1_per_chunk0 + 1
    n_1_per_chunk0     =(n1-m1)/n_chunks_per_column + 1
    !
    n=n*sqrt(float(n)/float(n_1_per_chunk0))
    n_2_per_chunk0     =min(n, n2-m2+1)
    n_chunks_per_slice0=(n2-m2)/n_2_per_chunk0 + 1
    n_2_per_chunk0     =(n2-m2)/n_chunks_per_slice0 + 1
    !
    n=n*    (float(n)/float(n_2_per_chunk0))
    n_3_per_chunk0     =min(n, n3-m3+1)
    n_slices_per_bank  =(n3-m3)/n_3_per_chunk0 + 1
    n_3_per_chunk0     =(n3-m3)/n_slices_per_bank + 1
    n_banks            =(n3-m3)/n_3_per_chunk0 + 1
  endif
  !
  ! write(6,*) 'n_chunks_per_column=', n_chunks_per_column
  ! write(6,*) 'n_1_per_chunk0     =', n_1_per_chunk0
  ! write(6,*) 'n_chunks_per_slice0=', n_chunks_per_slice0
  ! write(6,*) 'n_2_per_chunk0     =', n_2_per_chunk0
  ! write(6,*) 'n_slices_per_bank  =', n_slices_per_bank
  ! write(6,*) 'n_3_per_chunk0     =', n_3_per_chunk0
  ! write(6,*) 'n_banks            =', n_banks
  !
  n_chunks0=n_chunks_per_column*n_chunks_per_slice0*n_slices_per_bank
  !
else ! (edge_mode == 1) ----------------------------------------------------------------------------
  ! --- Prefer slabs with squarish cross section (n1 complete, n2 = n3) ---
  !
  if (n_elements_per_chunk <= 0) then
    ! --- Default value, choose something reasonable ---
    !
    n_chunks_per_slice0=max(min(4, (n2-m2+1)/8), 1)
    n_banks            =max(min(4, (n3-m3+1)/8), 1)
    !
    n_2_per_chunk0     =(n2-m2)/n_chunks_per_slice0 + 1
    n_3_per_chunk0     =(n3-m3)/n_banks + 1
    !
    n_columns_per_chunk=n_2_per_chunk0*n_3_per_chunk0
    n_chunks0          =((n2-m2)/n_2_per_chunk0 + 1) * ((n3-m3)/n_3_per_chunk0 + 1)
  else
    ! --- Choose chunk parameters according to input number of elements per chunk ---
    !
    n_columns_per_chunk=max(n_elements_per_chunk, n1-m1+1) / (n1-m1+1)
    !
    n_slices_per_bank=min(max(int(sqrt(float(n_columns_per_chunk)+0.5)), 1), n3-m3+1)
    n_banks          =(n3-m3)/n_slices_per_bank + 1
    n_3_per_chunk0   =(n3-m3)/n_banks + 1
    !
    n_2_per_chunk0     =min((n_columns_per_chunk)/n_3_per_chunk0, n2-m2+1)
    n_chunks_per_slice0=(n2-m2)/n_2_per_chunk0 +1
    n_2_per_chunk0     =(n2-m2)/n_chunks_per_slice0 + 1
    !
    n_chunks0=((n2-m2)/n_2_per_chunk0 + 1) * ((n3-m3)/n_3_per_chunk0 + 1)
  endif
endif ! (edge_mode) --------------------------------------------------------------------------------
!
! --- Check if chunk number is within range ---
if ((i_chunk < 1) .or. (i_chunk > n_chunks0)) then
  ! --- Chunk number outside allowed range ---
  !
  m1c0= 0
  n1c0=-1
  m2c=  0
  n2c= -1
  m3c=  0
  n3c= -1
  !m1c0=m1
  !n1c0=m1-1
  !m2c =m2
  !n2c =m2-1
  !m3c =m3
  !n3c =m3-1
else if (edge_mode /= 2) then
  ! --- Chunk number within range, edge_mode=0 or 1 ---
  !
  ! --- Determine current chunk index range ---
  i2 =mod(i_chunk-1,n_chunks_per_slice0)
  i3 =(i_chunk-1)/n_chunks_per_slice0
  !
  ! --- Compute start and end indices of chunk ---
  m1c0=m1
  n1c0=n1
  if (i2 < n_chunks_per_slice0-(n_chunks_per_slice0*n_2_per_chunk0-(n2-m2+1))) then
    m2c =i2*n_2_per_chunk0 + m2
    n2c =min(m2c+n_2_per_chunk0-1, n2)
  else
    n2c =n2-(n_chunks_per_slice0-i2-1)*(n_2_per_chunk0-1)
    m2c =n2c-(n_2_per_chunk0-2)
  endif
  !
  if (i3 < n_banks-(n_banks*n_3_per_chunk0-(n3-m3+1))) then
    m3c =i3*n_3_per_chunk0 + m3
    n3c =min(m3c+n_3_per_chunk0-1, n3)
  else
    n3c =n3-(n_banks-i3-1)*(n_3_per_chunk0-1)
    m3c =n3c-(n_3_per_chunk0-2)
  endif
else ! (edge_mode = 2)
  ! --- Chunk number within range, edge_mode=2 ---
  !
  ! --- Determine current chunk index range ---
  i10=mod( i_chunk-1                                          , n_chunks_per_column)
  i2 =mod((i_chunk-1)/ n_chunks_per_column                    , n_chunks_per_slice0)
  i3 =    (i_chunk-1)/(n_chunks_per_column*n_chunks_per_slice0                     )
  !
  ! --- Compute start and end indices of chunk ---
  if (i10 < n_chunks_per_column-(n_chunks_per_column*n_1_per_chunk0-(n1-m1+1))) then
    m1c0=i10*n_1_per_chunk0 + m1
    n1c0=min(m1c0+n_1_per_chunk0-1, n1)
  else
    n1c0=n1-(n_chunks_per_column-i10-1)*(n_1_per_chunk0-1)
    m1c0=n1c0-(n_1_per_chunk0-2)
  endif
  !
  if (i2 < n_chunks_per_slice0-(n_chunks_per_slice0*n_2_per_chunk0-(n2-m2+1))) then
    m2c =i2*n_2_per_chunk0 + m2
    n2c =min(m2c+n_2_per_chunk0-1, n2)
  else
    n2c =n2-(n_chunks_per_slice0-i2-1)*(n_2_per_chunk0-1)
    m2c =n2c-(n_2_per_chunk0-2)
  endif
  !
  if (i3 < n_banks-(n_banks*n_3_per_chunk0-(n3-m3+1))) then
    m3c =i3*n_3_per_chunk0 + m3
    n3c =min(m3c+n_3_per_chunk0-1, n3)
  else
    n3c =n3-(n_banks-i3-1)*(n_3_per_chunk0-1)
    m3c =n3c-(n_3_per_chunk0-2)
  endif
  !
endif
!
! write(6,*) m1c0,n1c0, m2c,n2c, m3c,n3c
if (present(m1c               )) m1c               =m1c0
if (present(n1c               )) n1c               =n1c0
if (present(n_chunks          )) n_chunks          =n_chunks0
if (present(n_chunks_per_slice)) n_chunks_per_slice=n_chunks_per_slice0
if (present(n_2_per_chunk     )) n_2_per_chunk     =n_2_per_chunk0
if (present(n_3_per_chunk     )) n_3_per_chunk     =n_3_per_chunk0
!
end subroutine rhd_box_ChunkSize


!----------*******************----------------------------------------------------------------------
subroutine rhd_box_RestIndices(ibl, &
                               ml1,nl1, ml2,nl2, ml3,nl3, &
                               ms1,ns1, ms2,ns2, ms3,ns3, &
                               mb1,nb1, mb2,nb2, mb3,nb3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_RestIndices ('rhd_box_Rest_Indices')
!
! PURPOSE:
!   Compute indices of boxes surrounding small box in large box.
!
! CATEGORY:
!   Box handling, index arithmetic
!
! CALLING SEQUENCE:
!   call rhd_box_RestIndices(ibl, &
!                            ml1,nl1, ml2,nl2, ml3,nl3, &
!                            ms1,ns1, ms2,ns2, ms3,ns3, &
!                            mb1,nb1, mb2,nb2, mb3,nb3)
!
! INPUT:
!   ibl:          ('integer_block') integer with block index (1 <= ibl <= 6)
!                   <0: Output empty box
!                    0: Output indices for small box that are inside large box
!                    1: Output indices for bottom (first/left i3) box
!                    2: Output indices for first/left i2 box
!                    3: Output indices for second/right i2 box
!                    4: Output indices for first/left i1 box
!                    5: Output indices for second/right i1 box
!                    6: Output indices for top (second/right i3) box
!                   >6: Output empty box
!
!   ml1:          ('m_number_large_1') integer, 1. dimension, start index of large box
!   nl1:          ('number_large_1')   integer, 1. dimension, end index of large box
!   ml2:          ('m_number_large_2') integer, 2. dimension, start index of large box
!   nl2:          ('number_large_2')   integer, 2. dimension, end index of large box
!   ml3:          ('m_number_large_3') integer, 3. dimension, start index of large box
!   nl3:          ('number_large_3')   integer, 3. dimension, end index of large box
!   ms1:          ('m_number_small_1') integer, 1. dimension, start index of small box
!   ns1:          ('number_small_1')   integer, 1. dimension, end index of small box
!   ms2:          ('m_number_small_2') integer, 2. dimension, start index of small box
!   ns2:          ('number_small_2')   integer, 2. dimension, end index of small box
!   ms3:          ('m_number_small_3') integer, 3. dimension, start index of small box
!   ns3:          ('number_small_3')   integer, 3. dimension, end index of small box
!
! OUTPUT:
!   mb1:          ('m_number_block_1') integer, 1. dimension, start index of output block
!   nb1:          ('number_block_1')   integer, 1. dimension, end index of output block
!   mb2:          ('m_number_block_2') integer, 2. dimension, start index of output block
!   nb2:          ('number_block_2')   integer, 2. dimension, end index of output block
!   mb3:          ('m_number_block_3') integer, 3. dimension, start index of output block
!   nb3:          ('number_block_3')   integer, 3. dimension, end index of output block
!
! LOCAL VARIABLES:
!   None
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   One or more of the output boxes can be empty, indicated by m>n.
!
! PROCEDURE:
!   From a large box (given by indices ml1,...) a small box (given by indices ms1,...) is cut out.
!   The remainder is split into 6 boxes.
!   The coordinates of each remainder box can be accessed by specifying the box index with ibl.
!   Example (a 2D slice through a 3D box):
!      ^ i3
!      |
!      66666666
!      66666666
!      44....55
!      44....55
!      44....55
!      11111111
!      11111111 -> i1
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-12-13 (B.F. Uppsala) Written
!   2016-04-18 (B.F.) Fix index-computation bugs
!   2017-07-28 (B.F.) Implement ibl=0 (output of original small box)
!   2019-03-06 (B.F.) Fix bug with small box partly outside large box; check for empty small box
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ibl, &
                                           ml1,nl1, ml2,nl2, ml3,nl3, &
                                           ms1,ns1, ms2,ns2, ms3,ns3
integer,                   intent(out)  :: mb1,nb1, mb2,nb2, mb3,nb3
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if ((ml1 > nl1) .or. (ml2 > nl2) .or. (ml3 > nl3) .or. &
    (ibl < 0) .or. (ibl > 6)                            ) then
  ! --- The volume of the large box is zero or a non-existing sub box is requested: ---
  ! --- Force the volume of the output box to be zero                               ---
  mb1=ml1+1
  nb1=ml1
  mb2=ml2+1
  nb2=ml2
  mb3=ml3+1
  nb3=ml3
  !
else ! (.not. ((ml1 > nl1) .or. (ml2 > nl2) .or. (ml3 > nl3) .or. (ibl < 0) .or. (ibl > 6)))
  ! --- The volume of the large box is non-zero and the sub-box exists: proceed... ---
  !
  if ((ms1 > ns1) .or. (ms2 > ns2) .or. (ms3 > ns3) .or. &
      (ms1 > nl1) .or. (ns1 < ml1) .or. &
      (ms2 > nl2) .or. (ns2 < ml2) .or. &
      (ms3 > nl3) .or. (ns3 < ml3)                        ) then
    ! --- The small box is empty or completely outside the large box ---
    !
    if (ibl == 1) then
      ! --- Set the output box to be the large box ---
      mb1=ml1
      nb1=nl1
      mb2=ml2
      nb2=nl2
      mb3=ml3
      nb3=nl3
    else ! (ibl /= 1)
      ! --- Set the volume of the output box to zero ---
      mb1=ml1+1
      nb1=ml1
      mb2=ml2+1
      nb2=ml2
      mb3=ml3+1
      nb3=ml3
      !
    endif ! (ibl)
  else ! (ms1,2,3, ns1,2,3)
    ! --- Large and small box overlap at least partially ---
    !
    if (ibl == 0) then
      ! --- Get indices for small box ---
      !
      mb1=max(ml1, ms1  )
      nb1=min(nl1, ns1  )
      mb2=max(ml2, ms2  )
      nb2=min(nl2, ns2  )
      mb3=max(ml3, ms3  )
      nb3=min(nl3, ns3  )
      !
    else if (ibl == 1) then
      ! --- Get indices for bottom (first/left i3) box ---
      !
      mb1=    ml1
      nb1=    nl1
      mb2=    ml2
      nb2=    nl2
      mb3=    ml3
      nb3=min(nl3, ms3-1)
      !
    else if (ibl == 2) then
      ! --- Get indices for first/left i2 box ---
      !
      mb1=    ml1
      nb1=    nl1
      mb2=    ml2
      nb2=min(nl2, ms2-1)
      mb3=max(ml3, ms3  )
      nb3=min(nl3, ns3  )
      !
    else if (ibl == 3) then
      ! --- Get indices for second/right i2 box ---
      !
      mb1=    ml1
      nb1=    nl1
      mb2=max(ml2, ns2+1)
      nb2=    nl2
      mb3=max(ml3, ms3  )
      nb3=min(nl3, ns3  )
      !
    else if (ibl == 4) then
      ! --- Get indices for first/left i1 box ---
      !
      mb1=    ml1
      nb1=min(nl1, ms1-1)
      mb2=max(ml2, ms2  )
      nb2=min(nl2, ns2  )
      mb3=max(ml3, ms3  )
      nb3=min(nl3, ns3  )
      !
    else if (ibl == 5) then
      ! --- Get indices for second/right i1 box ---
      !
      mb1=max(ml1, ns1+1)
      nb1=    nl1
      mb2=max(ml2, ms2  )
      nb2=min(nl2, ns2  )
      mb3=max(ml3, ms3  )
      nb3=min(nl3, ns3  )
      !
    else if (ibl == 6) then
      ! --- Get indices for top (second/right i3) box ---
      !
      mb1=    ml1
      nb1=    nl1
      mb2=    ml2
      nb2=    nl2
      mb3=max(ml3, ns3+1)
      nb3=    nl3
      !
    endif ! (ibl)
  endif ! (ms1,2,3, ns1,2,3)
endif ! ((ml1 > nl1) .or. (ml2 > nl2) .or. (ml3 > nl3) .or. (ibl < 0) .or. (ibl > 6))
!
end subroutine rhd_box_RestIndices


!----------*********************--------------------------------------------------------------------
subroutine rhd_box_SubBoxIndices(ml1,nl1, ml2,nl2, ml3,nl3, &
                                 ms1,ns1, ms2,ns2, ms3,ns3, &
                                 mb1,nb1, mb2,nb2, mb3,nb3, &
                                 SubBoxMode, SubBoxDim, &
                                 IsEmpty_flag, IsComplete_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_SubBoxIndices ('rhd_box_Sub_Box_Indices')
!
! PURPOSE:
!   Compute indices of box representing the section of both input boxes.
!
! CATEGORY:
!   Box handling, index arithmetic
!
! CALLING SEQUENCE:
!   call rhd_box_SubBoxIndices(ml1,nl1, ml2,nl2, ml3,nl3, &
!                              ms1,ns1, ms2,ns2, ms3,ns3, &
!                              mb1,nb1, mb2,nb2, mb3,nb3, &
!                              SubBoxMode=SubBoxMode, SubBoxDim=SubBoxDim, &
!                              IsEmpty_flag=IsEmpty_flag, IsComplete_flag=IsComplete_flag)
!
! INPUT:
!   ml1:          ('m_number_large_1') integer, 1. dimension, start index of large box
!   nl1:          ('number_large_1')   integer, 1. dimension, end index of large box
!   ml2:          ('m_number_large_2') integer, 2. dimension, start index of large box
!   nl2:          ('number_large_2')   integer, 2. dimension, end index of large box
!   ml3:          ('m_number_large_3') integer, 3. dimension, start index of large box
!   nl3:          ('number_large_3')   integer, 3. dimension, end index of large box
!   ms1:          ('m_number_small_1') integer, 1. dimension, start index of small box
!   ns1:          ('number_small_1')   integer, 1. dimension, end index of small box
!   ms2:          ('m_number_small_2') integer, 2. dimension, start index of small box
!   ns2:          ('number_small_2')   integer, 2. dimension, end index of small box
!   ms3:          ('m_number_small_3') integer, 3. dimension, start index of small box
!   ns3:          ('number_small_3')   integer, 3. dimension, end index of small box
!   SubBoxDim:    ('Sub_Box_Dimension') optional 2x3 real array with requested dimension of sub box;
!                 if this array is not present, the dimension is taken from ms1,ns1,...
!   SubBoxMode:   ('Sub_Box_Mode') optional character with mode how to find or interpret
!                 the sub box dimension:
!                   'All'                : Ignore SubBoxDim and just use the entire volume
!                   'CenterSlice-x1'     : Ignore SubBoxDim and take a center slice in x2-x3 plane
!                   'CenterSlice-x2'     : Ignore SubBoxDim and take a center slice in x1-x3 plane
!                   'CenterSlice-x3'     : Ignore SubBoxDim and take a center slice in x1-x2 plane
!                   'Dimensions-absolute': Take sub box with absolute dimensions given by SubBoxDiM
!                   'Dimensions-relative': Take sub box with relative dimensions given by SubBoxDiM
!
! OUTPUT:
!   mb1:          ('m_number_block_1') integer, 1. dimension, start index of output block
!   nb1:          ('number_block_1')   integer, 1. dimension, end index of output block
!   mb2:          ('m_number_block_2') integer, 2. dimension, start index of output block
!   nb2:          ('number_block_2')   integer, 2. dimension, end index of output block
!   mb3:          ('m_number_block_3') integer, 3. dimension, start index of output block
!   nb3:          ('number_block_3')   integer, 3. dimension, end index of output block
!   IsEmpty_flag: ('Is_Empty_flag') optional logical indicating if output sub box is empty
!   IsComplete_flag: ('Is_Complete_flag') optional logical indicating if output sub box contains
!                 the complete large input box
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   rhd_box_RestIndices:   Compute indices of boxes surrounding small box in large box
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The output box can be empty, indicated by m>n and IsEmpty_flag=.true.
!
! PROCEDURE:
!   From a large box (given by indices ml1,...), a small box is cut out.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-01 (B.F. Uppsala) Adapted from rhd_box_Write and rhd_box_RestIndices
!   2017-12-18 (B.F.) Replace merge command by if-then-else construct because of compiler warning
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ml1,nl1, ml2,nl2, ml3,nl3, &
                                           ms1,ns1, ms2,ns2, ms3,ns3
integer,                   intent(out)  :: mb1,nb1, mb2,nb2, mb3,nb3
character(len=*),optional, intent(in)   :: SubBoxMode
integer, dimension(2,3), &
                 optional, intent(in)   :: SubBoxDim
logical,         optional, intent(out)  :: IsEmpty_flag, IsComplete_flag
!
! --- Local variables ---
integer                                 :: ms10,ns10, ms20,ns20, ms30,ns30
logical                                 :: IsEmpty_flag0, IsComplete_flag0
character(len=80)                       :: SubBoxMode0
!---------------------------------------------------------------------------------------------------
!
if ((ml1 > nl1) .or. (ml2 > nl2) .or. (ml3 > nl3)) then
  ! --- The volume of the large box is zero:          ---
  ! --- Force the volume of the output box to be zero ---
  !
  mb1=ml1+1
  nb1=ml1
  mb2=ml2+1
  nb2=ml2
  mb3=ml3+1
  nb3=ml3
  IsEmpty_flag0   =.true.
  IsComplete_flag0=.true.
  !
else ! (.not. ((ml1 > nl1) .or. (ml2 > nl2) .or. (ml3 > nl3)))
  ! --- The volume of the large box is non-zero: determine the indices of the sub box ---
  !
  ! --- Check if control parameter is present and set to the default, if not ---
  if (present(SubBoxMode)) then
    SubBoxMode0=SubBoxMode
  else
    SubBoxMode0=''
  endif
  if (SubBoxMode0 == '') then
    if (present(SubBoxDim)) then
      SubBoxMode0='Dimensions-relative'
    else
      SubBoxMode0='All'
    endif
  endif
  !
  if      (SubBoxMode0 == 'All') then
    ! --- Take the entire large box ---
    !
    mb1=ml1
    nb1=nl1
    mb2=ml2
    nb2=nl2
    mb3=ml3
    nb3=nl3
    IsEmpty_flag0   =.false.
    IsComplete_flag0=.true.
    !
  else if (SubBoxMode0 == 'CenterSlice-x1') then
    ! --- Take center slice in x2-x3 plane ---
    !
    mb1=(ml1+nl1)/2
    nb1=mb1
    mb2=ml2
    nb2=nl2
    mb3=ml3
    nb3=nl3
    IsEmpty_flag0   =.false.
    IsComplete_flag0=((mb1 <= ml1) .and. (nb1 >= nl1))
    !
  else if (SubBoxMode0 == 'CenterSlice-x2') then
    ! --- Take center slice in x1-x3 plane ---
    !
    mb1=ml1
    nb1=nl1
    mb2=(ml2+nl2)/2
    nb2=mb2
    mb3=ml3
    nb3=nl3
    IsEmpty_flag0   =.false.
    IsComplete_flag0=((mb2 <= ml2) .and. (nb2 >= nl2))
    !
  else if (SubBoxMode0 == 'CenterSlice-x3') then
    ! --- Take center slice in x1-x2 plane ---
    !
    mb1=ml1
    nb1=nl1
    mb2=ml2
    nb2=nl2
    mb3=(ml3+nl3)/2
    nb3=mb3
    IsEmpty_flag0   =.false.
    IsComplete_flag0=((mb3 <= ml3) .and. (nb3 >= nl3))
    !
  else if ((SubBoxMode0 == 'Dimensions-absolute') .or. (SubBoxMode0 == 'Dimensions-relative')) then
    ! --- Determine the part of the specified sub box that actually exists in the box ---
    ! --- Interpret the indices as absolute or relative                               ---
    !
    if (present(SubBoxDim)) then
      ms10=SubBoxDim(1,1)
      ns10=SubBoxDim(2,1)
      ms20=SubBoxDim(1,2)
      ns20=SubBoxDim(2,2)
      ms30=SubBoxDim(1,3)
      ns30=SubBoxDim(2,3)
    else
      ms10=ms1
      ns10=ns1
      ms20=ms2
      ns20=ns2
      ms30=ms3
      ns30=ns3
    endif
    !
    if (SubBoxMode0 == 'Dimensions-relative') then
      ! --- Interpret the indices as relative to the left (bottom) layer, starting at 1 ---
      ms10=ms10+ml1-1
      ns10=ns10+ml1-1
      ms20=ms20+ml2-1
      ns20=ns20+ml2-1
      ms30=ms30+ml3-1
      ns30=ns30+ml3-1
    endif
    !
    call rhd_box_RestIndices(0, &
                             ml1 ,nl1 , ml2 ,nl2 , ml3 ,nl3, &
                             ms10,ns10, ms20,ns20, ms30,ns30, &
                             mb1 ,nb1 , mb2 ,nb2 , mb3 ,nb3    )
    !
    IsEmpty_flag0   =((mb1 > nb1) .or. (mb2 > nb2) .or. (mb3 > nb3))
    IsComplete_flag0=((mb1 <= ml1) .and. (nb1 >= nl1) .and. &
                      (mb2 <= ml2) .and. (nb2 >= nl2) .and. &
                      (mb3 <= ml3) .and. (nb3 >= nl3)        )
    !
  else ! (SubBoxMode0)
    ! --- Specify empty volume to prevent any output ---
    !
    mb1=ml1+1
    nb1=ml1
    mb2=ml2+1
    nb2=ml2
    mb3=ml3+1
    nb3=ml3
    IsEmpty_flag0   =.true.
    IsComplete_flag0=.false.
    !
  endif ! (SubBoxMode0)
endif ! ((ml1 > nl1) .or. (ml2 > nl2) .or. (ml3 > nl3))
!
if (present(IsEmpty_flag   )) IsEmpty_flag   =IsEmpty_flag0
if (present(IsComplete_flag)) IsComplete_flag=IsComplete_flag0
!
end subroutine rhd_box_SubBoxIndices


!----------****************-------------------------------------------------------------------------
subroutine rhd_box_FillAxes(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_FillAxes ('rhd_box_Fill_Axes')
!
! PURPOSE:
!   Fill axes of box.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_FillAxes(box)
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The global prop structures has to be initialized.
!   Outer ghost cells are not filled.
!
! PROCEDURE:
!   Copy axes values, including ghost-cell values if available, from global prop structure.
!
! EXAMPLE:
!   call rhd_box_FillAxes(model)
!
! MODIFICATION HISTORY:
!   2018-11-24 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3
!---------------------------------------------------------------------------------------------------
!
! --- Fill x1 axes ---
m1=max(box%m1-box%nghost1, prop%m1)
n1=min(box%n1+box%nghost1, prop%n1)
if (n1 >= m1) then
  box%xc1(m1:n1  )=prop%xc1(m1:n1  )
  box%xb1(m1:n1+1)=prop%xb1(m1:n1+1)
endif
!
! --- Fill x2 axes ---
m2=max(box%m2-box%nghost2, prop%m2)
n2=min(box%n2+box%nghost2, prop%n2)
if (n2 >= m2) then
  box%xc2(m2:n2  )=prop%xc2(m2:n2  )
  box%xb2(m2:n2+1)=prop%xb2(m2:n2+1)
endif
!
! --- Fill x3 axes ---
m3=max(box%m3-box%nghost3, prop%m3)
n3=min(box%n3+box%nghost3, prop%n3)
if (n3 >= m3) then
  box%xc3(m3:n3  )=prop%xc3(m3:n3  )
  box%xb3(m3:n3+1)=prop%xb3(m3:n3+1)
endif
!
end subroutine rhd_box_FillAxes


!----------***********************------------------------------------------------------------------
subroutine rhd_box_FillGhostAxis1D(bound_l1, bound_r1, xc1, xb1, m1, n1, nghost1, ngh1)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_FillGhostAxis1D ('rhd_box_Fill_Ghost-cells_Axis_1_Dimension')
!
! PURPOSE:
!   Fill ghost cells of axis vectors for real boundaries.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_FillGhostAxis1D(bound_l1, bound_r1, xc1, xb1, m1, n1, nghost1, ngh1)
!
! INPUT:
!   bound_l1:     ('boundary_left_1') integer, left boundary type for dimension 1;
!                 values: see rhd_action_module
!   bound_r1:     ('boundary_right_1') integer, right boundary type for dimension 1;
!                 values: see rhd_action_module
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   ngh1:         ('number_ghost_1') integer, 1. dimension,  with number of ghost cells
!                 to be transferred; ngh1<=nghost1
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   xc1:          ('x_center_1') real, 1. coordinate of cell centers
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Ghost cells of MPI-internal boundary are not filled.
!   The axis vectors have to exist, with sizes properly specified by m1, n1, nghost1.
!   This is not checked.
!
! PROCEDURE:
!   The number of specified ghost cells ngh1 can be smaller than the total number nghost1.
!   If the number of specified ghost cells is larger than (or equal to) the total number,
!   all ghost cells are filled.
!   If the number of specified ghost cells is smaller than (or equal to) zero,
!   no ghost cells are filled.
!
! EXAMPLE:
!   call rhd_box_FillGhostAxis1D(bound_periodic, bound_periodic, xc1, xb1, m1, n1, nghost1, 2)
!
! MODIFICATION HISTORY:
!   2017-07-25 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: bound_l1, bound_r1, m1, n1, nghost1, ngh1
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(inout):: xc1
real, dimension(m1-nghost1:n1+nghost1+1), &
                           intent(inout):: xb1
!
! --- Local variables ---
integer                                 :: i1, nghost10
!---------------------------------------------------------------------------------------------------
!
! --- Limit number of ghost cells to allowed values ---
nghost10=max(min(ngh1, nghost1), 0)
!
if (nghost10 > 0) then
  ! --- Handle 'left' boundary ---
  select case (bound_l1)
    case (bound_reflective)
      ! --- reflective ---
      do i1=1,nghost10
        xb1(m1-i1) = xb1(m1-i1+1) + (xb1(m1+i1-1)-xb1(m1+i1))
        xc1(m1-i1) = 0.5*(xb1(m1-i1)+xb1(m1-i1+1))
      end do ! i1
      !
    case (bound_periodic)
      ! --- periodic ---
      xb1(m1-nghost10:m1-1) = xb1(n1-nghost10+1:n1) + (xb1(m1)-xb1(n1+1))
      xc1(m1-nghost10:m1-1) = xc1(n1-nghost10+1:n1) + (xb1(m1)-xb1(n1+1))
      !
    case (bound_InternalNext, bound_InternalPeriodic, bound_None)
      ! --- MPI-internal ---
      !
    case default
      ! --- constant, closed ---
      ! --- case (bound_constant, bound_inoutflow, bound_inoutflow2, bound_inoutflow3, &
      ! ---       bound_transmitting, bound_transmitting2, bound_transmitting3, &
      ! ---       bound_closed, bound_closed2, bound_closedbottom, bound_closedbottom2)
      do i1=1,nghost10
        xb1(m1-i1) = xb1(m1-i1+1) + (xb1(m1)-xb1(m1+1))
        xc1(m1-i1) = 0.5*(xb1(m1-i1)+xb1(m1-i1+1))
      end do ! i1
      !
  end select ! (bound_l1)
  !
  ! --- Handle 'right' boundary ---
  select case (bound_r1)
    case (bound_reflective)
      ! --- reflective ---
      do i1=1,nghost10
        xb1(n1+i1+1) = xb1(n1+i1) + (xb1(n1-i1+2)-xb1(n1-i1+1))
        xc1(n1+i1  ) = 0.5*(xb1(n1+i1)+xb1(n1+i1+1))
      end do ! i1
      !
    case (bound_periodic)
      ! --- periodic ---
      xb1(n1+2:n1+nghost10+1) = xb1(m1+1:m1+nghost10  ) + (xb1(n1+1)-xb1(m1))
      xc1(n1+1:n1+nghost10  ) = xc1(m1  :m1+nghost10-1) + (xb1(n1+1)-xb1(m1))
      !
    case (bound_InternalNext, bound_InternalPeriodic, bound_None)
      ! --- MPI-internal ---
      !
    case default
      ! --- constant, closed ---
      ! --- case (bound_constant, bound_inoutflow, bound_inoutflow2, bound_inoutflow3, &
      ! ---       bound_transmitting, bound_transmitting2, bound_transmitting3, &
      ! ---       bound_closed, bound_closed2, bound_closedbottom, bound_closedbottom2)
      do i1=1,nghost10
        xb1(n1+i1+1) = xb1(n1+i1) + (xb1(n1+1)-xb1(n1))
        xc1(n1+i1  ) = 0.5*(xb1(n1+i1)+xb1(n1+i1+1))
      end do ! i1
      !
  end select ! (bound_r1)
  !
endif ! (nghost10)
!
end subroutine rhd_box_FillGhostAxis1D


!----------*********************--------------------------------------------------------------------
subroutine rhd_box_FillGhostAxes(action, box, ngh1, ngh2, ngh3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_FillGhostAxes ('rhd_box_Fill_Ghost-cells_Axes')
!
! PURPOSE:
!   Fill ghost cells of box axes for real boundaries.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_FillGhostAxes(action, box, ngh1, ngh2, ngh3)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   ngh1:         ('number_ghost_1') integer with number of ghost layers to be transferred
!                 in i1 direction
!   ngh2:         ('number_ghost_2') integer with number of ghost layers to be transferred
!                 in i2 direction
!   ngh3:         ('number_ghost_3') integer with number of ghost layers to be transferred
!                 in i3 direction
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   rhd_box_FillGhostAxis1D: Fill ghost cells of axis vectors for real boundaries
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Ghost cells of MPI-internal boundary are not filled.
!
! PROCEDURE:
!   Fill ghost values for the axes in each dimension one by one by calling
!   rhd_box_FillGhostAxis1D.
!
! EXAMPLE:
!   call rhd_box_FillGhostAxes(action, model, 1, 1, 1)
!
! MODIFICATION HISTORY:
!   2017-07-25 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),         intent(in)   :: action
type(box_type),            intent(inout):: box
integer,                   intent(in)   :: ngh1, ngh2, ngh3
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
call rhd_box_FillGhostAxis1D(action%bound_l1, action%bound_r1, &
                             box%xc1, box%xb1, box%m1, box%n1, box%nghost1, ngh1)
call rhd_box_FillGhostAxis1D(action%bound_l2, action%bound_r2, &
                             box%xc2, box%xb2, box%m2, box%n2, box%nghost2, ngh2)
call rhd_box_FillGhostAxis1D(action%bound_l3, action%bound_r3, &
                             box%xc3, box%xb3, box%m3, box%n3, box%nghost3, ngh3)
!
end subroutine rhd_box_FillGhostAxes


!----------************-----------------------------------------------------------------------------
subroutine rhd_box_Init(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Init ('rhd_box_Initialize')
!
! PURPOSE:
!   Initialize computational box. Nullify pointers.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Init(box)
!
! INPUT:
!   None
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   This routine (or rhd_box_Make) should be the first operation applied to a box.
!
! PROCEDURE:
!   Set scalar values to zero, nullify pointers.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-12-28 (B.F. Kiel) Written
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2010-04-19 (B.F. Lyon) Rename init_box -> rhd_box_Init
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
box%m1=0
box%n1=0
box%m2=0
box%n2=0
box%m3=0
box%n3=0
box%nghost1=0
box%nghost2=0
box%nghost3=0
box%time   =0.0
box%time_db=0.0D+00
box%itime  =0
box%ngrav=0
box%nquc =0
box%nBc  =0
box%nBb  =0
!
!#
# if (rhd_box_arrays01 == 0)
!#
nullify(box%xc1, box%xc2, box%xc3, &
        box%xb1, box%xb2, box%xb3, &
        box%rho, box%ei ,          &
        box%v1 , box%v2 , box%v3)
!#
# if (rhd_box_grav01 == 1)
!#
nullify(box%phic_grav)
!#
# endif
!#      /* rhd_box_grav01 */
nullify(box%quc)
nullify(box%Bc1, box%Bc2, box%Bc3, box%Bb1, box%Bb2, box%Bb3)
nullify(box%flux_B12, box%flux_B13, box%flux_B21, box%flux_B23, box%flux_B31, box%flux_B32)
nullify(box%energy_switch)
!#
# endif
!#      /* rhd_box_arrays01 */
!
end subroutine rhd_box_Init


!----------***************--------------------------------------------------------------------------
subroutine rhd_box_ZeroOMP(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_ZeroOMP ('rhd_box_Zero_OpenMP')
!
! PURPOSE:
!   Set bulk arrays explicitly to zero.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_ZeroOMP(box)
!
! INPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), input computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create box with rhd_box_Make before call of rhd_box_ZeroOMP.
!
! PROCEDURE:
!   Just fill all bulk arrays with zeros.
!   The outer loop uses OpenMP to help distributing the data sensibly accross the cores
!   and hopefully improve data locality.
!
! EXAMPLE:
!   call rhd_box_ZeroOMP(box)
!
! MODIFICATION HISTORY:
!   2013-05-31 (B.F. Lyon) Derived from rhd_box_AllAddOMP
!   2017-09-27 (B.F. Uppsala) Check if rho, ei, v1,v2,v3 arrays exist before setting them to zero
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
!
! --- Local variables ---
integer                                 :: i1, i2, i3, iquc, &
                                           m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=box%m1
n1=box%n1
m2=box%m2
n2=box%n2
m3=box%m3
n3=box%n3
nghost1=box%nghost1
nghost2=box%nghost2
nghost3=box%nghost3
!
! === Handle (M)HD quantities ===
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3, iquc)
!
if (ALLOASSOTED(box%rho)) then
  ! --- Zero density ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1
        box%rho(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if (ALLOASSOTED(box%ei)) then
  ! --- Zero internal energy ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1
        box%ei( i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if (ALLOASSOTED(box%v1)) then
  ! --- Zero velocities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1
        box%v1( i1,i2,i3)=0.0
        box%v2( i1,i2,i3)=0.0
        box%v3( i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
!#
# if (rhd_box_bmag01 == 1)
!#
!
if (ALLOASSOTED(box%Bc1)) then
  ! --- Zero magnetic field c123 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1
        box%Bc1(i1,i2,i3)=0.0
        box%Bc2(i1,i2,i3)=0.0
        box%Bc3(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bb1)) then
  ! --- Zero magnetic field b1 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1+1
        box%Bb1(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bb2)) then
  ! --- Zero magnetic field b2 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do i2=m2-nghost2,n2+nghost2+1
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1
        box%Bb2(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bb3)) then
  ! --- Zero magnetic field b3 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3+1
    do i2=m2-nghost2,n2+nghost2
      !DIR$ IVDEP
      do i1=m1-nghost1,n1+nghost1
        box%Bb3(i1,i2,i3)=0.0
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
! --- ToDo: zero magnetic fluxes, too ---
!
!#
# endif
!#      /* rhd_box_bmag01 */
!#
# if (rhd_box_quc01 != 0)
!#
if (ALLOASSOTED(box%quc)) then
  ! --- Zero additional density arrays ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-nghost3,n3+nghost3
    do iquc=1,box%nquc
      do i2=m2-nghost2,n2+nghost2
        !DIR$ IVDEP
        do i1=m1-nghost1,n1+nghost1
          box%quc(   i1,i2,i3,iquc)=0.0
        end do ! i1
      end do ! i2
    end do ! iquc
  end do ! i3
  !$OMP END DO
  !
endif
!#
# endif
!#      /* rhd_box_quc01 */
!
!$OMP END PARALLEL
!
end subroutine rhd_box_ZeroOMP


!----------*******************----------------------------------------------------------------------
subroutine rhd_box_FillTestOMP(box, AddMode, FillValues, dx, nghost1, nghost2, nghost3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_FillTestOMP ('rhd_box_FillTest_OpenMP')
!
! PURPOSE:
!   Set bulk arrays in box structure to some test values.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_FillTestOMP(box, AddMode=AddMode, FillValues=FillValues, dx=dx, &
!                            nghost1=nghost1, nghost2=nghost2, nghost3=nghost3)
!
! INPUT:
!   AddMode:      ('Add_Mode') optional, integer with mode how to fill or modify the box arrays:
!                   0: Put the values into the arrays, default
!                   1: Add the values to the arrays
!                   2: Subtract the values from the arrays
!   FillValues:   ('Fill_Values') optional real vector with values the arrays should be filled with
!   dx:           ('delta_x) optional real scalar with grid size, default=1.0
!   nghost1:      ('number_ghost_1') integer, optional (def=0), number of ghost cells
!   nghost2:      ('number_ghost_2') integer, optional (def=0), number of ghost cells
!   nghost3:      ('number_ghost_3') integer, optional (def=0), number of ghost cells
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), input computational box
!
! ROUTINES:
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create box with rhd_box_Make before call of rhd_box_FillTestOMP.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_FillTestOMP(box, AddMode=0)
!
! MODIFICATION HISTORY:
!   2016-04-28 (B.F. Uppsala) Derived from rhd_box_ZeroOMP
!   2017-08-27 (B.F. Lyon) Rename input mode
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
integer,         optional, intent(in)   :: AddMode
real, dimension(:), &
                 optional, intent(in)   :: FillValues
real,            optional, intent(in)   :: dx
integer,         optional, intent(in)   :: nghost1, nghost2, nghost3
!
! --- Local variables ---
integer                                 :: i1, i2, i3, iquc, &
                                           m1,n1, m2,n2, m3,n3, ngh1, ngh2, ngh3, &
                                           AddMode0, nFillValues, iFillValues
real                                    :: FillFactor, FillValueCur, dx0
integer, parameter                      :: nFillValuesMax=10000
!---------------------------------------------------------------------------------------------------
!
! === Initialize ===
!
! --- Handle optional parameters ---
if (present(nghost1)) then
  ngh1=min(nghost1, box%nghost1)
else
  ngh1=box%nghost1
endif
if (present(nghost2)) then
  ngh2=min(nghost2, box%nghost2)
else
  ngh2=box%nghost2
endif
if (present(nghost3)) then
  ngh3=min(nghost3, box%nghost3)
else
  ngh3=box%nghost3
endif
if (present(dx)) then
  dx0=dx
else
  dx0=1.0
endif
!
! --- Initialize the arrays and set the factor that determines addition or subtraction ---
if (present(AddMode)) then
  AddMode0=AddMode
else
  AddMode0=0
endif
if      (AddMode0 == 0) then
  call rhd_box_ZeroOMP(box)
  box%xc1=0.0
  box%xb1=0.0
  box%xc2=0.0
  box%xb2=0.0
  box%xc3=0.0
  box%xb3=0.0
  FillFactor= 1.0
else if (AddMode0 == 1) then
  FillFactor= 1.0
else !  (AddMode0 == 2)
  FillFactor=-1.0
endif ! (AddMode0)
!
! --- Make abbreviations ---
m1=box%m1
n1=box%n1
m2=box%m2
n2=box%n2
m3=box%m3
n3=box%n3
if (present(FillValues)) then
  ! --- The input array is present: determine its number of elements ---
  nFillValues=size(FillValues)
else
  ! --- The input array is not present: set the element number to a very high value ---
  nFillValues=nFillValuesMax
endif
!
! --- Initialize the fill-value counter ---
iFillValues=0
!
!
! === Handle axes ===
do i1=m1-ngh1,n1+ngh1
  box%xc1(i1)=box%xc1(i1) + (FillFactor*dx0) * real(i1)
end do ! i1
do i1=m1-ngh1,n1+ngh1+1
  box%xb1(i1)=box%xb1(i1) + (FillFactor*dx0) *(real(i1)-0.5)
end do ! i1
do i2=m2-ngh2,n2+ngh2
  box%xc2(i2)=box%xc2(i2) + (FillFactor*dx0) * real(i2)
end do ! i2
do i2=m2-ngh2,n2+ngh2+1
  box%xb2(i2)=box%xb2(i2) + (FillFactor*dx0) *(real(i2)-0.5)
end do ! i2
do i3=m3-ngh3,n3+ngh3
  box%xc3(i3)=box%xc3(i3) + (FillFactor*dx0) * real(i3)
end do ! i3
do i3=m3-ngh3,n3+ngh3+1
  box%xb3(i3)=box%xb3(i3) + (FillFactor*dx0) *(real(i3)-0.5)
end do ! i3
!
!
! === Handle (M)HD quantities ===
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3)
!
! --- Handle rho array ---
!$OMP MASTER
iFillValues =min(iFillValues+1, nFillValues)
if (present(FillValues)) then
  FillValueCur=FillFactor * FillValues(iFillValues)
else
  FillValueCur=FillFactor * real(iFillValues)
endif
!$OMP END MASTER
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-ngh3,n3+ngh3
  do i2=m2-ngh2,n2+ngh2
    !DIR$ IVDEP
    do i1=m1-ngh1,n1+ngh1
      box%rho(i1,i2,i3)=box%rho(i1,i2,i3) + FillValueCur
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
! --- Handle ei array ---
!$OMP MASTER
iFillValues =min(iFillValues+1, nFillValues)
if (present(FillValues)) then
  FillValueCur=FillFactor * FillValues(iFillValues)
else
  FillValueCur=FillFactor * real(iFillValues)
endif
!$OMP END MASTER
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-ngh3,n3+ngh3
  do i2=m2-ngh2,n2+ngh2
    !DIR$ IVDEP
    do i1=m1-ngh1,n1+ngh1
      box%ei(i1,i2,i3)=box%ei(i1,i2,i3) + FillValueCur
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
! --- Handle v1 array ---
!$OMP MASTER
iFillValues =min(iFillValues+1, nFillValues)
if (present(FillValues)) then
  FillValueCur=FillFactor * FillValues(iFillValues)
else
  FillValueCur=FillFactor * real(iFillValues)
endif
!$OMP END MASTER
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-ngh3,n3+ngh3
  do i2=m2-ngh2,n2+ngh2
    !DIR$ IVDEP
    do i1=m1-ngh1,n1+ngh1
      box%v1(i1,i2,i3)=box%v1(i1,i2,i3) + FillValueCur
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
! --- Handle v2 array ---
!$OMP MASTER
iFillValues =min(iFillValues+1, nFillValues)
if (present(FillValues)) then
  FillValueCur=FillFactor * FillValues(iFillValues)
else
  FillValueCur=FillFactor * real(iFillValues)
endif
!$OMP END MASTER
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-ngh3,n3+ngh3
  do i2=m2-ngh2,n2+ngh2
    !DIR$ IVDEP
    do i1=m1-ngh1,n1+ngh1
      box%v2(i1,i2,i3)=box%v2(i1,i2,i3) + FillValueCur
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
! --- Handle v3 array ---
!$OMP MASTER
iFillValues =min(iFillValues+1, nFillValues)
if (present(FillValues)) then
  FillValueCur=FillFactor * FillValues(iFillValues)
else
  FillValueCur=FillFactor * real(iFillValues)
endif
!$OMP END MASTER
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3-ngh3,n3+ngh3
  do i2=m2-ngh2,n2+ngh2
    !DIR$ IVDEP
    do i1=m1-ngh1,n1+ngh1
      box%v3(i1,i2,i3)=box%v3(i1,i2,i3) + FillValueCur
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO
!
!#
# if (rhd_box_bmag01 == 1)
!#
!
! --- Handle cell-centered magnetic fields c123 ---
if (ALLOASSOTED(box%Bc1)) then
  ! --- Handle Bc1 array ---
  !$OMP MASTER
  iFillValues =min(iFillValues+1, nFillValues)
  if (present(FillValues)) then
    FillValueCur=FillFactor * FillValues(iFillValues)
  else
    FillValueCur=FillFactor * real(iFillValues)
  endif
  !$OMP END MASTER
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-ngh3,n3+ngh3
    do i2=m2-ngh2,n2+ngh2
      !DIR$ IVDEP
      do i1=m1-ngh1,n1+ngh1
        box%Bc1(i1,i2,i3)=box%Bc1(i1,i2,i3) + FillValueCur
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bc2)) then
  ! --- Handle Bc2 array ---
  !$OMP MASTER
  iFillValues =min(iFillValues+1, nFillValues)
  if (present(FillValues)) then
    FillValueCur=FillFactor * FillValues(iFillValues)
  else
    FillValueCur=FillFactor * real(iFillValues)
  endif
  !$OMP END MASTER
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-ngh3,n3+ngh3
    do i2=m2-ngh2,n2+ngh2
      !DIR$ IVDEP
      do i1=m1-ngh1,n1+ngh1
        box%Bc2(i1,i2,i3)=box%Bc2(i1,i2,i3) + FillValueCur
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bc3)) then
  ! --- Handle Bc3 array ---
  !$OMP MASTER
  iFillValues =min(iFillValues+1, nFillValues)
  if (present(FillValues)) then
    FillValueCur=FillFactor * FillValues(iFillValues)
  else
    FillValueCur=FillFactor * real(iFillValues)
  endif
  !$OMP END MASTER
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-ngh3,n3+ngh3
    do i2=m2-ngh2,n2+ngh2
      !DIR$ IVDEP
      do i1=m1-ngh1,n1+ngh1
        box%Bc3(i1,i2,i3)=box%Bc3(i1,i2,i3) + FillValueCur
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
! --- Handle boundary-located magnetic fields b123 ---
if (ALLOASSOTED(box%Bb1)) then
  ! --- Handle Bb1 array ---
  !$OMP MASTER
  iFillValues =min(iFillValues+1, nFillValues)
  if (present(FillValues)) then
    FillValueCur=FillFactor * FillValues(iFillValues)
  else
    FillValueCur=FillFactor * real(iFillValues)
  endif
  !$OMP END MASTER
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-ngh3,n3+ngh3
    do i2=m2-ngh2,n2+ngh2
      !DIR$ IVDEP
      do i1=m1-ngh1,n1+ngh1+1
        box%Bb1(i1,i2,i3)=box%Bb1(i1,i2,i3) + FillValueCur
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bb2)) then
  ! --- Handle Bb2 array ---
  !$OMP MASTER
  iFillValues =min(iFillValues+1, nFillValues)
  if (present(FillValues)) then
    FillValueCur=FillFactor * FillValues(iFillValues)
  else
    FillValueCur=FillFactor * real(iFillValues)
  endif
  !$OMP END MASTER
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-ngh3,n3+ngh3
    do i2=m2-ngh2,n2+ngh2+1
      !DIR$ IVDEP
      do i1=m1-ngh1,n1+ngh1
        box%Bb2(i1,i2,i3)=box%Bb2(i1,i2,i3) + FillValueCur
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
if (ALLOASSOTED(box%Bb3)) then
  ! --- Handle Bb3 array ---
  !$OMP MASTER
  iFillValues =min(iFillValues+1, nFillValues)
  if (present(FillValues)) then
    FillValueCur=FillFactor * FillValues(iFillValues)
  else
    FillValueCur=FillFactor * real(iFillValues)
  endif
  !$OMP END MASTER
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3-ngh3,n3+ngh3+1
    do i2=m2-ngh2,n2+ngh2
      !DIR$ IVDEP
      do i1=m1-ngh1,n1+ngh1
        box%Bb3(i1,i2,i3)=box%Bb3(i1,i2,i3) + FillValueCur
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
endif
!
!#
# endif
!#      /* rhd_box_bmag01 */
!
!$OMP END PARALLEL
!#
# if (rhd_box_quc01 != 0)
!#
! --- Handle additional density arrays ---
if (ALLOASSOTED(box%quc)) then
  ! --- Handle quc arrays ---
  do iquc=1,box%nquc
    iFillValues =min(iFillValues+1, nFillValues)
    if (present(FillValues)) then
      FillValueCur=FillFactor * FillValues(iFillValues)
    else
      FillValueCur=FillFactor * real(iFillValues)
    endif
    !$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3, iquc)
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3-ngh3,n3+ngh3
      do i2=m2-ngh2,n2+ngh2
        !DIR$ IVDEP
        do i1=m1-ngh1,n1+ngh1
          box%quc(i1,i2,i3,iquc)=box%quc(i1,i2,i3,iquc) + FillValueCur
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
    !$OMP END PARALLEL
  end do ! iquc
endif
!#
# endif
!#      /* rhd_box_quc01 */
!
end subroutine rhd_box_FillTestOMP


!----------------*************----------------------------------------------------------------------
logical function rhd_box_Exist(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Exist ('rhd_box_Exist')
!
! PURPOSE:
!   Test if box is already filled.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   flag=rhd_box_Exist(box)
!
! INPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The input variable box has to be at least initialized with rhd_box_Init (or created with
!   rhd_box_Make).
!
! PROCEDURE:
!   This function tests if any of the bulk arrays is already allocated.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-12-28 (B.F. Kiel) Written
!   2010-04-20 (B.F. Lyon) Rename exist_box -> rhd_box_Exist
!   2017-09-27 (B.F. Uppsala) Check also xc1 and xb1
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
rhd_box_Exist=(ALLOASSOTED(box%xc1) .or. ALLOASSOTED(box%xb1) .or. &
               ALLOASSOTED(box%rho) .or. ALLOASSOTED(box%ei ) .or. &
               ALLOASSOTED(box%v1 ) .or. ALLOASSOTED(box%v2 ) .or. ALLOASSOTED(box%v3 ))
!
end function rhd_box_Exist


!----------**************---------------------------------------------------------------------------
subroutine rhd_box_Delete(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Delete ('rhd_box_Delete')
!
! PURPOSE:
!   Delete computational box. Deallocate memory.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Delete(box)
!
! INPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Deallocate all allocated or associated arrays and set all scalars to zero.
!
! EXAMPLE:
!   call rhd_box_Delete(model)
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2010-04-19 (B.F. Lyon) Rename delete_box -> rhd_box_Delete
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
box%m1=0
box%n1=0
box%m2=0
box%n2=0
box%m3=0
box%n3=0
box%nghost1=0
box%nghost2=0
box%nghost3=0
box%time   =0.0
box%time_db=0.0D+00
box%itime  =0
box%ngrav=0
box%nquc =0
box%nBc  =0
box%nBb  =0
!
if (ALLOASSOTED(box%xc1      )) deallocate(box%xc1      )
if (ALLOASSOTED(box%xc2      )) deallocate(box%xc2      )
if (ALLOASSOTED(box%xc3      )) deallocate(box%xc3      )
if (ALLOASSOTED(box%xb1      )) deallocate(box%xb1      )
if (ALLOASSOTED(box%xb2      )) deallocate(box%xb2      )
if (ALLOASSOTED(box%xb3      )) deallocate(box%xb3      )
!
if (ALLOASSOTED(box%rho      )) deallocate(box%rho      )
if (ALLOASSOTED(box%ei       )) deallocate(box%ei       )
if (ALLOASSOTED(box%v1       )) deallocate(box%v1       )
if (ALLOASSOTED(box%v2       )) deallocate(box%v2       )
if (ALLOASSOTED(box%v3       )) deallocate(box%v3       )
!#
# if (rhd_box_grav01 == 1)
!#
if (ALLOASSOTED(box%phic_grav)) deallocate(box%phic_grav)
!#
# endif
!#      /* rhd_box_grav01 */
!#
# if (rhd_box_quc01 != 0)
!#
if (ALLOASSOTED(box%quc      )) deallocate(box%quc      )
!#
# endif
!#      /* rhd_box_quc01 */
!#
# if (rhd_box_bmag01 != 0)
!#
if (ALLOASSOTED(box%Bc1      )) deallocate(box%Bc1      )
if (ALLOASSOTED(box%Bc2      )) deallocate(box%Bc2      )
if (ALLOASSOTED(box%Bc3      )) deallocate(box%Bc3      )
if (ALLOASSOTED(box%Bb1      )) deallocate(box%Bb1      )
if (ALLOASSOTED(box%Bb2      )) deallocate(box%Bb2      )
if (ALLOASSOTED(box%Bb3      )) deallocate(box%Bb3      )
if (ALLOASSOTED(box%flux_B12 )) deallocate(box%flux_B12 )
if (ALLOASSOTED(box%flux_B13 )) deallocate(box%flux_B13 )
if (ALLOASSOTED(box%flux_B21 )) deallocate(box%flux_B21 )
if (ALLOASSOTED(box%flux_B23 )) deallocate(box%flux_B23 )
if (ALLOASSOTED(box%flux_B31 )) deallocate(box%flux_B31 )
if (ALLOASSOTED(box%flux_B32 )) deallocate(box%flux_B32 )
if (ALLOASSOTED(box%energy_switch)) deallocate(box%energy_switch)
!#
# endif
!#      /* rhd_box_bmag01 */
!
end subroutine rhd_box_Delete


!----------****************-------------------------------------------------------------------------
subroutine rhd_box_RemoveBc(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_RemoveBc ('rhd_box_remove_B_center')
!
! PURPOSE:
!   Remove cell-centered magnetic-field arrays from box.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_RemoveBc(box)
!
! INPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Deallocate all allocated magnetic-field arrays and set field-number counter to zero.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-11-04 (B.F.) Written
!   2010-03-15 (B.F. Naples) Handle also additional fields: fluxes, energy_switch
!   2010-04-20 (B.F. Lyon) Rename removeBc_box -> rhd_box_RemoveBc
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
!#
# if (rhd_box_bmag01 != 0)
!#
box%nBc=0
if (ALLOASSOTED(box%Bc1          )) deallocate(box%Bc1          )
if (ALLOASSOTED(box%Bc2          )) deallocate(box%Bc2          )
if (ALLOASSOTED(box%Bc3          )) deallocate(box%Bc3          )
if (ALLOASSOTED(box%flux_B12     )) deallocate(box%flux_B12     )
if (ALLOASSOTED(box%flux_B13     )) deallocate(box%flux_B13     )
if (ALLOASSOTED(box%flux_B21     )) deallocate(box%flux_B21     )
if (ALLOASSOTED(box%flux_B23     )) deallocate(box%flux_B23     )
if (ALLOASSOTED(box%flux_B31     )) deallocate(box%flux_B31     )
if (ALLOASSOTED(box%flux_B32     )) deallocate(box%flux_B32     )
if (ALLOASSOTED(box%energy_switch)) deallocate(box%energy_switch)
!#
# endif
!#      /* rhd_box_bmag01 */
!
end subroutine rhd_box_RemoveBc


!----------************-----------------------------------------------------------------------------
subroutine rhd_box_Make(box, m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, &
                        quc_flag, Bc_flag, Bb_flag, &
                        rho_mode, ei_mode, v_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Make ('rhd_box_Make')
!
! PURPOSE:
!   Create computational box of requested size.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Make(box, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
!                     nghost1=nghost1, nghost2=nghost2, nghost3=nghost3, &
!                     quc_flag=quc_flag, Bc_flag=Bc_flag, Bb_flag=Bb_flag, &
!                     rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode)
!
! INPUT:
!   m1:           ('m_number_1') integer, optional, 1. dimension, start index
!   n1:           ('number_1')   integer, optional, 1. dimension, end index
!   m2:           ('m_number_2') integer, optional, 2. dimension, start index
!   n2:           ('number_2')   integer, optional, 2. dimension, end index
!   m3:           ('m_number_3') integer, optional, 3. dimension, start index
!   n3:           ('number_3')   integer, optional, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, optional (def=0), number of ghost cells
!   nghost2:      ('number_ghost_2') integer, optional (def=0), number of ghost cells
!   nghost3:      ('number_ghost_3') integer, optional (def=0), number of ghost cells
!   quc_flag:     ('quantity_centered_flag') optional logical to request creation of quc array
!   Bc_flag:      ('B_center_flag') optional logical to request creation of
!                 cell-centered magnetic-field arrays (Bc)
!   Bb_flag:      ('B_boundary_flag') optional logical to request creation of
!                 boundary located magnetic-field arrays (Bb)
!   rho_mode:     ('rho_mode') optional integer indicating if rho array should be included:
!                   not present: Include rho array only for bulk_mode=2
!                   0          : Don't include rho array
!                   1:         : Include rho array (default)
!   ei_mode:      ('energy_internal_mode') optional integer indicating if ei array should be
!                 included:
!                   not present: Include ei array only for bulk_mode=2
!                   0          : Don't include ei array
!                   1:         : Include ei array for (default)
!   v_mode:       ('velocity_mode') optional integer indicating if v arrays should be included:
!                   not present: Include v arrays only for bulk_mode=2
!                   0          : Don't include v arrays
!                   1:         : Include v arrays (default)
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   rhd_box_AppendBc:      Add cell-centered magnetic-field arrays to box
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Preventing the allocation of certain bulk quantities by setting either rho_mode, ei_mode, or
!   v_mode to zero, creates a box that is not compatible with all box routines. Most routines
!   assume that all bulk quantities are present. However, at least
!     rhd_box_Init, rhd_box_ZeroOMP, rhd_box_Exist, rhd_box_PrintStat, rhd_box_Delete, 
!     rhd_box_AllCopyOMP, rhd_box_Copy, rhd_box_CopyOMP, rhd_box_MPICopy,
!     rhd_box_MPIFillGhost
!   work.
!
! PROCEDURE:
!   Allocate all requested arrays and nullify all non-requested pointer arrays
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-18 (B.F. Kiel) Written
!   1997-07-01 (B.F.) m1,n1, m2,n2, m3,n3 made optional
!   1997-10-02 (B.F.) Corrections
!   2001-03-13 (B.F. Uppsala) Special version for OpenMP on SGI
!   2003-11-18 (B.F.) Nullify B pointers
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2010-03-15 (B.F. Naples) Temporary fields for MHD handled by appendBc_box
!   2010-04-19 (B.F. Lyon) Rename make_box -> rhd_box_Make
!   2017-09-27 (B.F. Uppsala) Add parameters rho_mode, ei_mode, v_mode as in rhd_box_MPICreateType
!   2019-03-07 (B.F.) Bugfix: nullify pointer arrays that are not allocated (because of XXX_mode=0)
!   2019-03-08 (B.F.) Nullify quc arrays for prop%nquc=0 even if quc_flag=.true.
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
integer,         optional, intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           nghost1, nghost2, nghost3, rho_mode, ei_mode, v_mode
logical,         optional, intent(in)   :: quc_flag, Bc_flag, Bb_flag
type(box_type),            intent(inout):: box
!
! --- Local variables ---
real, dimension(:)    , pointer         :: x
real, dimension(:,:,:), pointer         :: q
!logical, dimension(:,:,:), pointer      :: sw
!#
# if (rhd_box_quc01 != 0)
!#
logical                                 :: quc_flag0
real, dimension(:,:,:,:), pointer       :: q4
!#
# endif
!#      /* rhd_box_quc01 */
!#
# if (rhd_box_bmag01 != 0)
!#
logical                                 :: Bc_flag0, Bb_flag0
!#
# endif
!#      /* rhd_box_bmag01 */
!---------------------------------------------------------------------------------------------------
!
if (present(m1)) box%m1=m1
if (present(n1)) box%n1=n1
if (present(m2)) box%m2=m2
if (present(n2)) box%n2=n2
if (present(m3)) box%m3=m3
if (present(n3)) box%n3=n3
!
if (present(nghost1)) then
  box%nghost1=nghost1
else
  box%nghost1=0
endif
if (present(nghost2)) then
  box%nghost2=nghost2
else
  box%nghost2=0
endif
if (present(nghost3)) then
  box%nghost3=nghost3
else
  box%nghost3=0
endif
!
!#
# if (rhd_box_arrays01 == 0)
!#
! --- Allocate axes ---
allocate(x(box%m1-box%nghost1:box%n1+box%nghost1  ))
box%xc1=>x
allocate(x(box%m2-box%nghost2:box%n2+box%nghost2  ))
box%xc2=>x
allocate(x(box%m3-box%nghost3:box%n3+box%nghost3  ))
box%xc3=>x
allocate(x(box%m1-box%nghost1:box%n1+box%nghost1+1))
box%xb1=>x
allocate(x(box%m2-box%nghost2:box%n2+box%nghost2+1))
box%xb2=>x
allocate(x(box%m3-box%nghost3:box%n3+box%nghost3+1))
box%xb3=>x
! --- Allocate bulk quantities ---
if (merge(rho_mode, 1, present(rho_mode)) > 0) then
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
             box%m2-box%nghost2:box%n2+box%nghost2, &
             box%m3-box%nghost3:box%n3+box%nghost3   ))
  box%rho=>q
else ! (rho_mode)
  nullify(box%rho)
endif ! (rho_mode)
if (merge(ei_mode, 1, present(ei_mode)) > 0) then
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
             box%m2-box%nghost2:box%n2+box%nghost2, &
             box%m3-box%nghost3:box%n3+box%nghost3   ))
  box%ei=>q
else ! (ei_mode)
  nullify(box%ei)
endif ! (ei_mode)
if (merge(v_mode, 1, present(v_mode)) > 0) then
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
             box%m2-box%nghost2:box%n2+box%nghost2, &
             box%m3-box%nghost3:box%n3+box%nghost3   ))
  box%v1=>q
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
             box%m2-box%nghost2:box%n2+box%nghost2, &
             box%m3-box%nghost3:box%n3+box%nghost3   ))
  box%v2=>q
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
             box%m2-box%nghost2:box%n2+box%nghost2, &
             box%m3-box%nghost3:box%n3+box%nghost3   ))
  box%v3=>q
else ! (v_mode)
  nullify(box%v1, box%v2, box%v3)
endif ! (v_mode)
!
! --- Allocate array for gravitational potential ---
!#
# if (rhd_box_grav01 == 1)
!#
box%ngrav=1
allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
           box%m2-box%nghost2:box%n2+box%nghost2, &
           box%m3-box%nghost3:box%n3+box%nghost3   ))
box%phic_grav=>q
!#
# else
!#     /* rhd_box_grav01 */
box%ngrav=0
!#
# endif
!#      /* rhd_box_grav01 */
!
! --- Allocate additional density arrays ---
!#
# if (rhd_box_quc01 != 0)
!#
if (present(quc_flag) .and. (prop%nquc > 0)) then
  quc_flag0=quc_flag
else
  quc_flag0=.false.
endif
if (quc_flag0) then
  box%nquc=prop%nquc
  allocate(q4(box%m1-box%nghost1:box%n1+box%nghost1, &
              box%m2-box%nghost2:box%n2+box%nghost2, &
              box%m3-box%nghost3:box%n3+box%nghost3, &
              box%nquc                                ))
  box%quc=>q4
else
  box%nquc=0
  nullify(box%quc)
endif
!#
# else
!#     /* rhd_box_quc01 */
box%nquc=0
nullify(box%quc)
!#
# endif
!#      /* rhd_box_quc01 */
!
! --- Allocate magnetic field arrays ---
!#
# if (rhd_box_bmag01 == 1)
!#
if (present(Bc_flag)) then
  Bc_flag0=Bc_flag
else
  Bc_flag0=.false.
endif
if (present(Bb_flag)) then
  Bb_flag0=Bb_flag
else
  Bb_flag0=.false.
endif
!
if (Bb_flag0) then
  box%nBb=1
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1+1, &
             box%m2-box%nghost2:box%n2+box%nghost2  , &
             box%m3-box%nghost3:box%n3+box%nghost3     ))
  box%Bb1=>q
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1  , &
             box%m2-box%nghost2:box%n2+box%nghost2+1, &
             box%m3-box%nghost3:box%n3+box%nghost3     ))
  box%Bb2=>q
  allocate(q(box%m1-box%nghost1:box%n1+box%nghost1  , &
             box%m2-box%nghost2:box%n2+box%nghost2  , &
             box%m3-box%nghost3:box%n3+box%nghost3+1   ))
  box%Bb3=>q
else
  box%nBb=0
  nullify(box%Bb1, box%Bb2, box%Bb3)
endif
!
if (Bc_flag0) then
  call rhd_box_AppendBc(box)
else
  box%nBc=0
  nullify(box%Bc1, box%Bc2, box%Bc3)
  nullify(box%flux_B12, box%flux_B13, box%flux_B21, box%flux_B23, box%flux_B31, box%flux_B32)
  nullify(box%energy_switch)
endif
!
!#
# else
!#     /* rhd_box_bmag01 */
box%nBb=0
box%nBc=0
nullify(box%Bb1, box%Bb2, box%Bb3)
nullify(box%Bc1, box%Bc2, box%Bc3)
nullify(box%flux_B12, box%flux_B13, box%flux_B21, box%flux_B23, box%flux_B31, box%flux_B32)
nullify(box%energy_switch)
!#
# endif
!#      /* rhd_box_bmag01 */
!#
# else
!#     /* rhd_box_arrays01 */
!
! --- Allocate axes ---
allocate(box%xc1(box%m1-box%nghost1:box%n1+box%nghost1  ))
allocate(box%xc2(box%m2-box%nghost2:box%n2+box%nghost2  ))
allocate(box%xc3(box%m3-box%nghost3:box%n3+box%nghost3  ))
allocate(box%xb1(box%m1-box%nghost1:box%n1+box%nghost1+1))
allocate(box%xb2(box%m2-box%nghost2:box%n2+box%nghost2+1))
allocate(box%xb3(box%m3-box%nghost3:box%n3+box%nghost3+1))
! --- Allocate bulk quantities ---
if (merge(rho_mode, 1, present(rho_mode)) > 0) then
  allocate(box%rho(box%m1-box%nghost1:box%n1+box%nghost1, &
                   box%m2-box%nghost2:box%n2+box%nghost2, &
                   box%m3-box%nghost3:box%n3+box%nghost3   ))
endif ! (rho_mode)
if (merge(ei_mode, 1, present(ei_mode)) > 0) then
  allocate(box%ei( box%m1-box%nghost1:box%n1+box%nghost1, &
                   box%m2-box%nghost2:box%n2+box%nghost2, &
                   box%m3-box%nghost3:box%n3+box%nghost3   ))
endif ! (ei_mode)
if (merge(v_mode, 1, present(v_mode)) > 0) then
  allocate(box%v1( box%m1-box%nghost1:box%n1+box%nghost1, &
                   box%m2-box%nghost2:box%n2+box%nghost2, &
                   box%m3-box%nghost3:box%n3+box%nghost3   ))
  allocate(box%v2( box%m1-box%nghost1:box%n1+box%nghost1, &
                   box%m2-box%nghost2:box%n2+box%nghost2, &
                   box%m3-box%nghost3:box%n3+box%nghost3   ))
  allocate(box%v3( box%m1-box%nghost1:box%n1+box%nghost1, &
                   box%m2-box%nghost2:box%n2+box%nghost2, &
                   box%m3-box%nghost3:box%n3+box%nghost3   ))
endif ! (v_mode)
!
! --- Allocate array for gravitational potential ---
!#
# if (rhd_box_grav01 == 1)
!#
box%ngrav=1
allocate(box%phic_grav(box%m1-box%nghost1:box%n1+box%nghost1, &
                       box%m2-box%nghost2:box%n2+box%nghost2, &
                       box%m3-box%nghost3:box%n3+box%nghost3   ))
!#
# else
!#     /* rhd_box_grav01 */
box%ngrav=0
!#
# endif
!#      /* rhd_box_grav01 */
!
! --- Allocate additional density arrays ---
!#
# if (rhd_box_quc01 != 0)
!#
if (present(quc_flag) .and. (prop%nquc > 0)) then
  quc_flag0=quc_flag
else
  quc_flag0=.false.
endif
if (quc_flag0) then
  box%nquc=prop%nquc
  allocate(box%quc(box%m1-box%nghost1:box%n1+box%nghost1, &
                   box%m2-box%nghost2:box%n2+box%nghost2, &
                   box%m3-box%nghost3:box%n3+box%nghost3, &
                   box%nquc                                ))
else
  box%nquc=0
endif
!#
# else
!#     /* rhd_box_quc01 */
box%nquc=0
!#
# endif
!#      /* rhd_box_quc01 */
!
! --- Allocate magnetic field arrays ---
!#
# if (rhd_box_bmag01 == 1)
!#
if (present(Bc_flag)) then
  Bc_flag0=Bc_flag
else
  Bc_flag0=.false.
endif
if (present(Bb_flag)) then
  Bb_flag0=Bb_flag
else
  Bb_flag0=.false.
endif
!
if (Bb_flag0) then
  box%nBb=1
  allocate(box%Bb1(box%m1-box%nghost1:box%n1+box%nghost1+1, &
                   box%m2-box%nghost2:box%n2+box%nghost2  , &
                   box%m3-box%nghost3:box%n3+box%nghost3     ))
  allocate(box%Bb2(box%m1-box%nghost1:box%n1+box%nghost1  , &
                   box%m2-box%nghost2:box%n2+box%nghost2+1, &
                   box%m3-box%nghost3:box%n3+box%nghost3     ))
  allocate(box%Bb3(box%m1-box%nghost1:box%n1+box%nghost1  , &
                   box%m2-box%nghost2:box%n2+box%nghost2  , &
                   box%m3-box%nghost3:box%n3+box%nghost3+1   ))
else
  box%nBb=0
endif
!
if (Bc_flag0) then
  call rhd_box_AppendBc(box)
else
  box%nBc=0
endif
!
!#
# else
!#     /* rhd_box_bmag01 */
box%nBb=0
box%nBc=0
!#
# endif
!#      /* rhd_box_bmag01 */
!#
# endif
!#      /* rhd_box_arrays01 */
!
end subroutine rhd_box_Make


!----------****************-------------------------------------------------------------------------
subroutine rhd_box_AppendBc(box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AppendBc ('rhd_box_Append_B_center')
!
! PURPOSE:
!   Add cell-centered magnetic-field arrays to box.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AppendBc(box)
!
! INPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Arrays for cell-centered magnetic fields are created.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2002-11-04 (B.F.) First version
!   2010-03-15 (B.F. Naples) Handle also additional fields: fluxes, energy_switch
!   2010-04-20 (B.F. Lyon) Rename appendBc_box -> rhd_box_AppendBc
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: box
!
! --- Local variables ---
real   , dimension(:,:,:), pointer      :: q
logical, dimension(:,:,:), pointer      :: sw
!---------------------------------------------------------------------------------------------------
!
! --- Magnetic fields ---
!#
# if (rhd_box_arrays01 == 0)
!#
!#
# if (rhd_box_bmag01 == 1)
!#
box%nBc=1
allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
           box%m2-box%nghost2:box%n2+box%nghost2, &
           box%m3-box%nghost3:box%n3+box%nghost3))
box%Bc1=>q
allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
           box%m2-box%nghost2:box%n2+box%nghost2, &
           box%m3-box%nghost3:box%n3+box%nghost3))
box%Bc2=>q
allocate(q(box%m1-box%nghost1:box%n1+box%nghost1, &
           box%m2-box%nghost2:box%n2+box%nghost2, &
           box%m3-box%nghost3:box%n3+box%nghost3))
box%Bc3=>q
allocate(q(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
           box%m2-box%nghost2  :box%n2+box%nghost2+1, &
           box%m3-box%nghost3-2:box%n3+box%nghost3+2))
box%flux_B12=>q
allocate(q(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
           box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
           box%m3-box%nghost3  :box%n3+box%nghost3+1))
box%flux_B13=>q
allocate(q(box%m1-box%nghost1  :box%n1+box%nghost1+1, &
           box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
           box%m3-box%nghost3-2:box%n3+box%nghost3+2))
box%flux_B21=>q
allocate(q(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
           box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
           box%m3-box%nghost3  :box%n3+box%nghost3+1))
box%flux_B23=>q
allocate(q(box%m1-box%nghost1  :box%n1+box%nghost1+1, &
           box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
           box%m3-box%nghost3-2:box%n3+box%nghost3+2))
box%flux_B31=>q
allocate(q(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
           box%m2-box%nghost2  :box%n2+box%nghost2+1, &
           box%m3-box%nghost3-2:box%n3+box%nghost3+2))
box%flux_B32=>q
allocate(sw(box%m1-box%nghost1:box%n1+box%nghost1, &
            box%m2-box%nghost2:box%n2+box%nghost2, &
            box%m3-box%nghost3:box%n3+box%nghost3))
box%energy_switch=>sw
!#
# else
!#     /* rhd_box_bmag01 */
box%nBc=0
nullify(box%Bc1, box%Bc2, box%Bc3)
nullify(box%flux_B12, box%flux_B13, box%flux_B21, box%flux_B23, box%flux_B31, box%flux_B32)
nullify(box%energy_switch)
!#
# endif
!#      /* rhd_box_bmag01 */
!#
# else
!#     /* rhd_box_arrays01 */
!#
# if (rhd_box_bmag01 == 1)
!#
box%nBc=1
allocate(box%Bc1(box%m1-box%nghost1:box%n1+box%nghost1, &
                 box%m2-box%nghost2:box%n2+box%nghost2, &
                 box%m3-box%nghost3:box%n3+box%nghost3))
allocate(box%Bc2(box%m1-box%nghost1:box%n1+box%nghost1, &
                 box%m2-box%nghost2:box%n2+box%nghost2, &
                 box%m3-box%nghost3:box%n3+box%nghost3))
allocate(box%Bc3(box%m1-box%nghost1:box%n1+box%nghost1, &
                 box%m2-box%nghost2:box%n2+box%nghost2, &
                 box%m3-box%nghost3:box%n3+box%nghost3))
allocate(box%flux_B12(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
                      box%m2-box%nghost2  :box%n2+box%nghost2+1, &
                      box%m3-box%nghost3-2:box%n3+box%nghost3+2))
allocate(box%flux_B13(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
                      box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
                      box%m3-box%nghost3  :box%n3+box%nghost3+1))
allocate(box%flux_B21(box%m1-box%nghost1  :box%n1+box%nghost1+1, &
                      box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
                      box%m3-box%nghost3-2:box%n3+box%nghost3+2))
allocate(box%flux_B23(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
                      box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
                      box%m3-box%nghost3  :box%n3+box%nghost3+1))
allocate(box%flux_B31(box%m1-box%nghost1  :box%n1+box%nghost1+1, &
                      box%m2-box%nghost2-2:box%n2+box%nghost2+2, &
                      box%m3-box%nghost3-2:box%n3+box%nghost3+2))
allocate(box%flux_B32(box%m1-box%nghost1-2:box%n1+box%nghost1+2, &
                      box%m2-box%nghost2  :box%n2+box%nghost2+1, &
                      box%m3-box%nghost3-2:box%n3+box%nghost3+2))
allocate(box%energy_switch(box%m1-box%nghost1:box%n1+box%nghost1, &
                           box%m2-box%nghost2:box%n2+box%nghost2, &
                           box%m3-box%nghost3:box%n3+box%nghost3))
!#
# else
!#     /* rhd_box_bmag01 */
box%nBc=0
!#
# endif
!#      /* rhd_box_bmag01 */
!#
# endif
!#      /* rhd_box_arrays01 */
!
end subroutine rhd_box_AppendBc


!----------************-----------------------------------------------------------------------------
subroutine rhd_box_Copy(box, m1,n1, m2,n2, m3,n3, &
                        boxout, m1out, m2out, m3out, &
                        trans_mode, time_mode, grid_mode, bulk_mode, flux_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Copy ('rhd_box_Copy')
!
! PURPOSE:
!   Copy (part of) box into another. Transpose, if requested.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Copy(box, m1,n1, m2,n2, m3,n3, &
!                     boxout, m1out, m2out, m3out, &
!                     trans_mode=trans_mode, time_mode=time_mode, grid_mode=grid_mode, &
!                     bulk_mode=bulk_mode, flux_mode=flux_mode)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!   trans_mode:   ('transpose_mode') optional, integer, (1,2,3), source dimension which will be
!                 1st target dimension, default: 1
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   0: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:1
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default:1
!   flux_mode:    ('flux_mode') optional integer, (0,1), switch for copying flux data, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Copy.
!   Do not use the routine to copy parts within one box.
!   There is no check if the input dimensions are valid for input or output box.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written
!   1997-10-16 (B.F.) In case of transposition, handle momenta properly
!   1999-05-31 (B.F. Copenhagen) Keywords grid and time added
!   2000-02-07 (B.F.) Keyword bulk added
!   2001-01-21 (B.F. Uppsala) Single bulk copy loops for each quantity
!   2001-01-29 (B.F.) Copy entire 3D arrays en block
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2010-03-15 (B.F. Naples) Copy magnetic fluxes and entropy switch only if arrays are associated
!   2010-03-19 (B.F.) Merge copy blocks for Bc's and fluxes
!   2010-04-19 (B.F. Lyon) Rename copy_box -> rhd_box_Copy
!   2010-12-22 (W.S. Exeter) optional parameter flux added
!   2013-03-05 (B.F. Lyon) Add directives to help vectorization
!   2018-11-10 (B.F. Uppsala) Change names of some optional parameters
!   2019-03-08 (B.F.) Check if arrays exist before copying
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1out, m2out, m3out
integer,         optional, intent(in)   :: trans_mode, grid_mode, time_mode, bulk_mode, flux_mode
!
! --- Local variables ---
integer                                 :: trans_mode0, grid_mode0, time_mode0, bulk_mode0, &
                                           flux_mode0
integer                                 :: n1out, n2out, n3out, &
                                           i1, i2, i3, i2out, i3out, iquc
!---------------------------------------------------------------------------------------------------
!
! --- Set default values if necessary ---
if (present(trans_mode)) then
  trans_mode0=trans_mode
else
  trans_mode0=1
endif
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=1
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
if (present(flux_mode)) then
  flux_mode0=flux_mode
else
  flux_mode0=1
endif
!
if (time_mode0 > 0) then
  ! --- Copy time quantities ---
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif
!
if (trans_mode0 == 1) then
  ! === Copy directly ==============================================================================
  !
  ! --- Calculate upper boundary indices of output box ---
  n1out=m1out + n1-m1
  n2out=m2out + n2-m2
  n3out=m3out + n3-m3
  !
  if (grid_mode0 > 0) then
    ! --- Copy axes ---
    !
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- Copy HD quantities ---
    !
    if ((ALLOASSOTED(boxout%rho)) .and. (ALLOASSOTED(box%rho))) then
      ! --- Copy density ---
      !DIR$ IVDEP
      boxout%rho(m1out:n1out,m2out:n2out,m3out:n3out)=box%rho(m1:n1,m2:n2,m3:n3)
    endif
    if ((ALLOASSOTED(boxout%ei)) .and. (ALLOASSOTED(box%ei))) then
      ! --- Copy internal energy ---
      !DIR$ IVDEP
      boxout%ei( m1out:n1out,m2out:n2out,m3out:n3out)=box%ei( m1:n1,m2:n2,m3:n3)
    endif
    if ((ALLOASSOTED(boxout%v1)) .and. (ALLOASSOTED(box%v1))) then
      ! --- Copy velocities ---
      !DIR$ IVDEP
      boxout%v1( m1out:n1out,m2out:n2out,m3out:n3out)=box%v1( m1:n1,m2:n2,m3:n3)
      !DIR$ IVDEP
      boxout%v2( m1out:n1out,m2out:n2out,m3out:n3out)=box%v2( m1:n1,m2:n2,m3:n3)
      !DIR$ IVDEP
      boxout%v3( m1out:n1out,m2out:n2out,m3out:n3out)=box%v3( m1:n1,m2:n2,m3:n3)
    endif
!#
#   if (rhd_box_grav01 == 1)
!#
    if ((ALLOASSOTED(boxout%phic_grav)) .and. (ALLOASSOTED(box%phic_grav))) then
      ! --- Copy gravitational potential ---
      boxout%ngrav=box%ngrav
      !DIR$ IVDEP
      boxout%phic_grav(m1out:n1out,m2out:n2out,m3out:n3out)=box%phic_grav(m1:n1,m2:n2,m3:n3)
    endif
!#
#   endif
!#        /* rhd_box_grav01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      ! --- Copy additional densities ---
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        !DIR$ IVDEP
        boxout%quc(  m1out:n1out,m2out:n2out,m3out:n3out,iquc)=box%quc( m1:n1,m2:n2,m3:n3,iquc)
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    ! --- Copy magnetic field ---
    !
    if ((ALLOASSOTED(boxout%Bb1)) .and. (ALLOASSOTED(box%Bb1))) then
      ! --- Copy boundary-centered magnetic field ---
      boxout%nBb=box%nBb
      !DIR$ IVDEP
      boxout%Bb1(m1out:n1out+1,m2out:n2out  ,m3out:n3out  )=box%Bb1(m1:n1+1,m2:n2  ,m3:n3  )
      !DIR$ IVDEP
      boxout%Bb2(m1out:n1out  ,m2out:n2out+1,m3out:n3out  )=box%Bb2(m1:n1  ,m2:n2+1,m3:n3  )
      !DIR$ IVDEP
      boxout%Bb3(m1out:n1out  ,m2out:n2out  ,m3out:n3out+1)=box%Bb3(m1:n1  ,m2:n2  ,m3:n3+1)
    endif
    if ((ALLOASSOTED(boxout%Bc1)) .and. (ALLOASSOTED(box%Bc1))) then
      ! --- Copy cell-centered magnetic field ---
      boxout%nBc=box%nBc
      !DIR$ IVDEP
      boxout%Bc1(m1out:n1out,m2out:n2out,m3out:n3out)=box%Bc1(m1:n1,m2:n2,m3:n3)
      !DIR$ IVDEP
      boxout%Bc2(m1out:n1out,m2out:n2out,m3out:n3out)=box%Bc2(m1:n1,m2:n2,m3:n3)
      !DIR$ IVDEP
      boxout%Bc3(m1out:n1out,m2out:n2out,m3out:n3out)=box%Bc3(m1:n1,m2:n2,m3:n3)
      !
      if (flux_mode0 > 0) then
        ! --- Copy magnetic fluxes ---
        !DIR$ IVDEP
        boxout%flux_B21(m1out:n1out+1,m2out:n2out,  m3out:n3out  )= &
                                                               box%flux_B21(m1:n1+1,m2:n2  ,m3:n3  )
        !DIR$ IVDEP
        boxout%flux_B31(m1out:n1out+1,m2out:n2out  ,m3out:n3out  )= &
                                                               box%flux_B31(m1:n1+1,m2:n2  ,m3:n3  )
        !DIR$ IVDEP
        boxout%flux_B12(m1out:n1out  ,m2out:n2out+1,m3out:n3out  )= &
                                                               box%flux_B12(m1:n1  ,m2:n2+1,m3:n3  )
        !DIR$ IVDEP
        boxout%flux_B32(m1out:n1out  ,m2out:n2out+1,m3out:n3out  )= &
                                                               box%flux_B32(m1:n1  ,m2:n2+1,m3:n3  )
        !DIR$ IVDEP
        boxout%flux_B13(m1out:n1out  ,m2out:n2out  ,m3out:n3out+1)= &
                                                               box%flux_B13(m1:n1  ,m2:n2  ,m3:n3+1)
        !DIR$ IVDEP
        boxout%flux_B23(m1out:n1out  ,m2out:n2out  ,m3out:n3out+1)= &
                                                               box%flux_B23(m1:n1  ,m2:n2  ,m3:n3+1)
      endif
      !
      !DIR$ IVDEP
      boxout%energy_switch(m1out:n1out,m2out:n2out,m3out:n3out )= &
                                                               box%energy_switch(m1:n1,m2:n2,m3:n3 )
    endif
!#
#   endif
!#        /* rhd_box_bmag01 */
  endif
  !
else if (trans_mode0 == 2) then
  ! === Transpose 2 <-> 1 ==========================================================================
  !
  ! --- Calculate upper boundary indices of output box ---
  n1out=m1out + n2-m2
  n2out=m2out + n1-m1
  n3out=m3out + n3-m3
  !
  if (grid_mode0 > 0) then
    ! --- Copy axes ---
    !
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- Copy HD quantities ---
    !
    if ((ALLOASSOTED(boxout%rho)) .and. (ALLOASSOTED(box%rho))) then
      ! --- Copy density ---
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%rho(m1out:n1out,m2out:n2out,i3out)=transpose(box%rho(m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
    endif
    if ((ALLOASSOTED(boxout%ei)) .and. (ALLOASSOTED(box%ei))) then
      ! --- Copy internal energy ---
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%ei( m1out:n1out,m2out:n2out,i3out)=transpose(box%ei( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
    endif
    if ((ALLOASSOTED(boxout%v1)) .and. (ALLOASSOTED(box%v1))) then
      ! --- Copy velocities ---
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%v1( m1out:n1out,m2out:n2out,i3out)=transpose(box%v2( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%v2( m1out:n1out,m2out:n2out,i3out)=transpose(box%v1( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%v3( m1out:n1out,m2out:n2out,i3out)=transpose(box%v3( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
    endif
!#
#   if (rhd_box_grav01 == 1)
!#
    if ((ALLOASSOTED(boxout%phic_grav)) .and. (ALLOASSOTED(box%phic_grav))) then
      ! --- Copy gravitational potential ---
      boxout%ngrav=box%ngrav
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%phic_grav(m1out:n1out,m2out:n2out,i3out)=transpose(box%phic_grav(m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
    endif
!#
#   endif
!#        /* rhd_box_grav01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      ! --- Copy additional densities ---
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i3=m3
        do i3out=m3out,n3out
          !DIR$ IVDEP
          boxout%quc(m1out:n1out,m2out:n2out,i3out,iquc)=transpose(box%quc(m1:n1,m2:n2,i3,iquc))
          i3=i3+1
        end do ! i3out
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    ! --- Copy magnetic field ---
    !
    if ((ALLOASSOTED(boxout%Bb1)) .and. (ALLOASSOTED(box%Bb1))) then
      ! --- Copy boundary-centered magnetic field ---
      boxout%nBb=box%nBb
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%Bb1( m1out:n1out+1,m2out:n2out  ,i3out)=transpose(box%Bb2( m1:n1  ,m2:n2+1,i3))
        i3=i3+1
      end do ! i3out
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%Bb2( m1out:n1out  ,m2out:n2out+1,i3out)=transpose(box%Bb1( m1:n1+1,m2:n2  ,i3))
        i3=i3+1
      end do ! i3out
      i3=m3
      do i3out=m3out,n3out+1
        !DIR$ IVDEP
        boxout%Bb3( m1out:n1out  ,m2out:n2out  ,i3out)=transpose(box%Bb3( m1:n1  ,m2:n2  ,i3))
        i3=i3+1
      end do ! i3out
    endif
    if ((ALLOASSOTED(boxout%Bc1)) .and. (ALLOASSOTED(box%Bc1))) then
      ! --- Copy cell-centered magnetic field ---
      boxout%nBc=box%nBc
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%Bc1( m1out:n1out,m2out:n2out,i3out)=transpose(box%Bc2( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%Bc2( m1out:n1out,m2out:n2out,i3out)=transpose(box%Bc1( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%Bc3( m1out:n1out,m2out:n2out,i3out)=transpose(box%Bc3( m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
      !
      if (flux_mode0 > 0) then
        ! --- Copy magnetic fluxes ---
        i3=m3
        do i3out=m3out,n3out
          !DIR$ IVDEP
          boxout%flux_B12( m1out:n1out  ,m2out:n2out+1,i3out)= &
                                                        transpose(box%flux_B21( m1:n1+1,m2:n2  ,i3))
          i3=i3+1
        end do ! i3out
        i3=m3
        do i3out=m3out,n3out
          !DIR$ IVDEP
          boxout%flux_B32( m1out:n1out  ,m2out:n2out+1,i3out)= &
                                                        transpose(box%flux_B31( m1:n1+1,m2:n2  ,i3))
          i3=i3+1
        end do ! i3out
      endif
      !
      i3=m3
      do i3out=m3out,n3out
        !DIR$ IVDEP
        boxout%energy_switch(m1out:n1out,m2out:n2out,i3out)= &
                                                        transpose(box%energy_switch(m1:n1,m2:n2,i3))
        i3=i3+1
      end do ! i3out
    endif
!#
#   endif
!#        /* rhd_box_bmag01 */
  endif
  !
else if (trans_mode0 == 3) then
  ! === Transpose 3 <-> 1 ==========================================================================
  !
  ! --- Calculate upper boundary indices of output box ---
  n1out=m1out + n3-m3
  n2out=m2out + n2-m2
  n3out=m3out + n1-m1
  !
  if (grid_mode0 > 0) then
    ! --- Copy axes ---
    !
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb3(m3:n3+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb1(m1:n1+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- Copy HD quantities ---
    !
    if ((ALLOASSOTED(boxout%rho)) .and. (ALLOASSOTED(box%rho))) then
      ! --- Copy density ---
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%rho(m1out:n1out,i2out,m3out:n3out)=transpose(box%rho(m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
    endif
    if ((ALLOASSOTED(boxout%ei)) .and. (ALLOASSOTED(box%ei))) then
      ! --- Copy internal energy ---
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%ei( m1out:n1out,i2out,m3out:n3out)=transpose(box%ei( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
    endif
    if ((ALLOASSOTED(boxout%v1)) .and. (ALLOASSOTED(box%v1))) then
      ! --- Copy velocities ---
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%v1( m1out:n1out,i2out,m3out:n3out)=transpose(box%v3( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%v2( m1out:n1out,i2out,m3out:n3out)=transpose(box%v2( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%v3( m1out:n1out,i2out,m3out:n3out)=transpose(box%v1( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
    endif
!#
#   if (rhd_box_grav01 == 1)
!#
    if ((ALLOASSOTED(boxout%phic_grav)) .and. (ALLOASSOTED(box%phic_grav))) then
      ! --- Copy gravitational potential ---
      boxout%ngrav=box%ngrav
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%phic_grav(m1out:n1out,i2out,m3out:n3out)=transpose(box%phic_grav(m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
    endif
!#
#   endif
!#        /* rhd_box_grav01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      ! --- Copy additional densities ---
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i2=m2
        do i2out=m2out,n2out
          !DIR$ IVDEP
          boxout%quc(m1out:n1out,i2out,m3out:n3out,iquc)=transpose(box%quc(m1:n1,i2,m3:n3,iquc))
          i2=i2+1
        end do ! i2out
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    ! --- Copy magnetic field ---
    !
    if ((ALLOASSOTED(boxout%Bb1)) .and. (ALLOASSOTED(box%Bb1))) then
      ! --- Copy boundary-centered magnetic field ---
      boxout%nBb=box%nBb
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%Bb1( m1out:n1out+1,i2out,m3out:n3out  )=transpose(box%Bb3( m1:n1  ,i2,m3:n3+1))
        i2=i2+1
      end do ! i2out
      i2=m2
      do i2out=m2out,n2out+1
        !DIR$ IVDEP
        boxout%Bb2( m1out:n1out  ,i2out,m3out:n3out  )=transpose(box%Bb2( m1:n1  ,i2,m3:n3  ))
        i2=i2+1
      end do ! i2out
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%Bb3( m1out:n1out  ,i2out,m3out:n3out+1)=transpose(box%Bb1( m1:n1+1,i2,m3:n3  ))
        i2=i2+1
      end do ! i2out
    endif
    !
    if ((ALLOASSOTED(boxout%Bc1)) .and. (ALLOASSOTED(box%Bc1))) then
      ! --- Copy cell-centered magnetic field ---
      boxout%nBc=box%nBc
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%Bc1( m1out:n1out,i2out,m3out:n3out)=transpose(box%Bc3( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%Bc2( m1out:n1out,i2out,m3out:n3out)=transpose(box%Bc2( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%Bc3( m1out:n1out,i2out,m3out:n3out)=transpose(box%Bc1( m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
      !
      if (flux_mode0 > 0) then
        ! --- Copy magnetic fluxes ---
        i2=m2
        do i2out=m2out,n2out
          !DIR$ IVDEP
          boxout%flux_B13( m1out:n1out  ,i2out,m3out:n3out+1)= &
                                          transpose(box%flux_B31( m1:n1+1,i2,m3:n3  ))
          i2=i2+1
        end do ! i2out
        i2=m2
        do i2out=m2out,n2out
          !DIR$ IVDEP
          boxout%flux_B23( m1out:n1out  ,i2out,m3out:n3out+1)= &
                                          transpose(box%flux_B21( m1:n1+1,i2,m3:n3  ))
          i2=i2+1
        end do ! i2out
      endif
      !
      i2=m2
      do i2out=m2out,n2out
        !DIR$ IVDEP
        boxout%energy_switch(m1out:n1out,i2out,m3out:n3out)= &
                                          transpose(box%energy_switch(m1:n1,i2,m3:n3))
        i2=i2+1
      end do ! i2out
    endif
!#
#   endif
!#        /* rhd_box_bmag01 */
  endif
endif
!
end subroutine rhd_box_Copy


!----------****************-------------------------------------------------------------------------
subroutine rhd_box_AddDelta(box, m1,n1, m2,n2, m3,n3, &
                            boxout, m1out, m2out, m3out, eiTable_min, &
                            trans_mode, time_mode, grid_mode, bulk_mode, &
                            outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AddDelta ('rhd_box_Add_Delta')
!
! PURPOSE:
!   Add change of (part of) box into another. Transpose, if requested.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AddDelta(box, m1,n1, m2,n2, m3,n3, &
!                         boxout, m1out, m2out, m3out, eiTable_min, &
!                         trans_mode=trans_mode, time_mode=time_mode, grid_mode=grid_mode, &
!                         bulk_mode=bulk_mode, &
!                         outstr=outstr,ierr=ierr)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box, contains the delta of the
!                 conserved quantities
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!   eiTable_min:  ('energy_internal_Table_minimum') real with minimum allowed internal-energy value
!   trans_mode:   ('transpose_mode') optional, integer, (1,2,3), source dimension (box) which will
!                 be 1st target dimension boxout, default: 1
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   0: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:0
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box, contains the old and new
!                 state variables
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error: (currently none)
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   boxout has to contain the normal variables: rho, v1,2,3, ei
!   box has to contain the changes of the corresponding conserved variables:
!     drho, drhov1,2,3, drhoei
!
! PROCEDURE:
!   Short: boxout=boxout+(box)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written (copy_box)
!   1997-10-16 (B.F.) In case of transposition, handle momenta properly
!   1999-05-31 (B.F. Copenhagen) Keywords grid and time added
!   2000-02-07 (B.F.) Keyword bulk added
!   2001-01-21 (B.F. Uppsala) Single bulk copy loops for each quantity
!   2001-01-29 (B.F.) Copy entire 3D arrays en block
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2007-02-08 (B.F. Lyon) Adapted from copy_box, no grav. potential or magnetic fields
!   2010-04-20 (B.F.) Rename addchange_box -> rhd_box_AddChange
!   2010-05-26 (B.F.) Bugfix: add total energies instead of internal energies only
!   2010-06-13 (B.F.) Cell-centered magnetic field
!   2010-06-15 (B.F.) Bugfix: use vector arrays with correction direction
!   2010-06-16 (B.F.) flux_B arrays are overwritten and not updated
!   2010-07-16 (B.F. Naples) flux_B arrays are directly copied without reference to boxref
!   2011-02-28 (W.S. Exeter) dual-energy method, check if MHD arrays are present
!   2011-08-02 (B.F. Lyon) Taken from rhd_box_AddChange
!   2011-08-03 (B.F.) Perform checks on positivity, new parameters
!   2013-03-05 (B.F.) Add directives to help vectorization
!   2017-05-22 (B.F. Uppsala) Control check for positivity of quc arrays by quc_positive array
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1out, m2out, m3out
real,                      intent(in)   :: eiTable_min
integer,         optional, intent(in)   :: trans_mode, grid_mode, time_mode, bulk_mode
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: trans_mode0, grid_mode0, time_mode0, bulk_mode0
integer                                 :: n1out, n2out, n3out
integer                                 :: i1, i2, i3, i1out, i2out, i3out, iquc
real                                    :: rho, drho, v1, v2, v3, ei, Bc1, Bc2, Bc3, &
                                           rhonew_min, einew_min, qucnew_min
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- Default values ---
ierr0  =0
outstr0=''
if (present(trans_mode)) then
  trans_mode0=trans_mode
else
  trans_mode0=1
endif
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=0
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
!
if (time_mode0 > 0) then
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif
!
if (trans_mode0 == 1) then
  ! --- Copy directly ------------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n1-m1
  n2out=m2out + n2-m2
  n3out=m3out + n3-m3
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    ! --- Check if density is positive everywhere ---
    rhonew_min=huge(rhonew_min)
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i1out-m1out+m1
          i2=i2out-m2out+m2
          i3=i3out-m3out+m3
          !
          rhonew_min=min(rhonew_min, boxout%rho(i1out,i2out,i3out)+ &
                                     box%rho(   i1   ,i2   ,i3   ))
        end do ! i1out
      end do ! i2out
    end do ! i3out
    if (rhonew_min <= 0.0) then
      ierr0  =1
      outstr0='Density becomes non-positive'
      goto 999
    endif
    !
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        !DIR$ IVDEP
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i1out-m1out+m1
          i2=i2out-m2out+m2
          i3=i3out-m3out+m3
          !
          ! --- Bulk HD quantities ---
          rho                          =boxout%rho(i1out,i2out,i3out)
          drho                         =box%rho(   i1   ,i2   ,i3   )
          boxout%rho(i1out,i2out,i3out)=rho + drho
          !
          v1                           =boxout%v1( i1out,i2out,i3out)
          boxout%v1( i1out,i2out,i3out)=v1 + &
                                     (-drho                         *v1                           +&
                                                                     box%v1(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v2                           =boxout%v2( i1out,i2out,i3out)
          boxout%v2( i1out,i2out,i3out)=v2 + &
                                     (-drho                         *v2                           +&
                                                                     box%v2(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v3                           =boxout%v3( i1out,i2out,i3out)
          boxout%v3( i1out,i2out,i3out)=v3 + &
                                     (-drho                         *v3                           +&
                                                                     box%v3(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
!#
#         if (rhd_box_bmag01 == 1)
!#
          ! --- Magnetic field quantities ---
          if (ALLOASSOTED(boxout%Bc1) .and. ALLOASSOTED(box%Bc1)) then
            Bc1                          =boxout%Bc1(      i1out,i2out,i3out)
            boxout%Bc1(i1out,i2out,i3out)=Bc1 + box%Bc1(   i1   ,i2   ,i3   )
          endif
          !
          if (ALLOASSOTED(boxout%Bc2) .and. ALLOASSOTED(box%Bc2)) then
            Bc2                          =boxout%Bc2(      i1out,i2out,i3out)
            boxout%Bc2(i1out,i2out,i3out)=Bc2 + box%Bc2(   i1   ,i2   ,i3   )
          endif
          !
          if (ALLOASSOTED(boxout%Bc3) .and. ALLOASSOTED(box%Bc3)) then
            Bc3                          =boxout%Bc3(      i1out,i2out,i3out)
            boxout%Bc3(i1out,i2out,i3out)=Bc3 + box%Bc3(   i1   ,i2   ,i3   )
          endif
          !
          ei                           =boxout%ei( i1out,i2out,i3out)
          if (ALLOASSOTED(boxout%energy_switch) .and. ALLOASSOTED(box%energy_switch)) then
            if (boxout%energy_switch(i1out,i2out,i3out)) then
              boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out) + &
                                      0.5*((Bc1**2+Bc2**2+Bc3**2             ) - &
                                           (boxout%Bc1( i1out,i2out,i3out)**2 + &
                                            boxout%Bc2( i1out,i2out,i3out)**2 + &
                                            boxout%Bc3( i1out,i2out,i3out)**2) &
                                          )/boxout%rho( i1out,i2out,i3out)
            else
              boxout%ei( i1out,i2out,i3out)=ei + &
                                     (-drho                         *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)
            endif
            !
            boxout%energy_switch(i1out,i2out,i3out)=box%energy_switch(   i1   ,i2   ,i3   )
            !
          else
            boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out)
          endif
!#
#         else
!#             /* rhd_box_bmag01 */
          ei                           =boxout%ei( i1out,i2out,i3out)
          boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out)
!#
#         endif
!#              /* rhd_box_bmag01 */
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
    !
    ! --- Check if internal energy is above minimum table value everywhere ---
    einew_min=minval(boxout%ei(m1out:n1out,m2out:n2out,m3out:n3out))
    if (einew_min < eiTable_min) then
      ierr0  =2
      outstr0='Internal energy below minimum table value'
      goto 999
    endif
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    if (ALLOASSOTED(boxout%flux_B21) .and. ALLOASSOTED(box%flux_B21) .and. &
        ALLOASSOTED(boxout%flux_B31) .and. ALLOASSOTED(box%flux_B31)) then
      do i3out=m3out,n3out
        do i2out=m2out,n2out
          !DIR$ IVDEP
          do i1out=m1out,n1out+1
            ! --- Indices ---
            i1=i1out-m1out+m1
            i2=i2out-m2out+m2
            i3=i3out-m3out+m3
            !
            boxout%flux_B21(i1out,i2out,i3out)=    box%flux_B21(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B21(i1out,i2out,i3out)
            !
            boxout%flux_B31(i1out,i2out,i3out)=    box%flux_B31(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B31(i1out,i2out,i3out)
            !
          end do ! i1out
        end do ! i2out
      end do ! i3out
    endif
!#
#    endif
!#         /* rhd_box_bmag01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      !
      do iquc=1,box%nquc
        !DIR$ IVDEP
        boxout%quc(   m1out:n1out,m2out:n2out,m3out:n3out,iquc)= &
          boxout%quc( m1out:n1out,m2out:n2out,m3out:n3out,iquc) + &
           box%quc(   m1   :n1   ,m2   :n2   ,m3   :n3   ,iquc)
      end do ! iquc
      !
      ! --- Check if additional density arrays are non-negative everywhere ---
      do iquc=1,box%nquc
        if (prop%quc_positive(iquc) > 0) then
          qucnew_min=minval(boxout%quc(m1out:n1out,m2out:n2out,m3out:n3out,iquc))
          if (qucnew_min < 0.0) then
            ierr0  =3
            outstr0='Additional density below zero'
            goto 999
          endif
        endif
      end do ! iquc
    endif
    !
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
  !
else if (trans_mode0 == 2) then
  ! --- Transpose 2 -> 1 ---------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n2-m2
  n2out=m2out + n1-m1
  n3out=m3out + n3-m3
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    ! --- Check if density is positive everywhere ---
    rhonew_min=huge(rhonew_min)
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i1out-m1out+m1
          i2=i2out-m2out+m2
          i3=i3out-m3out+m3
          !
          rhonew_min=min(rhonew_min, boxout%rho(i1out,i2out,i3out)+ &
                                     box%rho(   i1   ,i2   ,i3   ))
        end do ! i1out
      end do ! i2out
    end do ! i3out
    if (rhonew_min <= 0.0) then
      ierr0  =4
      outstr0='Density becomes non-positive'
      goto 999
    endif
    !
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        !DIR$ IVDEP
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i2out-m2out+m1
          i2=i1out-m1out+m2
          i3=i3out-m3out+m3
          !
          ! --- Bulk HD quantities ---
          rho                          =boxout%rho(i1out,i2out,i3out)
          drho                         =box%rho(   i1   ,i2   ,i3   )
          boxout%rho(i1out,i2out,i3out)=rho + drho
          !
          v1                           =boxout%v1( i1out,i2out,i3out)
          boxout%v1( i1out,i2out,i3out)=v1 + &
                                     (-drho                         *v1                           +&
                                                                     box%v2(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v2                           =boxout%v2( i1out,i2out,i3out)
          boxout%v2( i1out,i2out,i3out)=v2 + &
                                     (-drho                         *v2                           +&
                                                                     box%v1(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v3                           =boxout%v3( i1out,i2out,i3out)
          boxout%v3( i1out,i2out,i3out)=v3 + &
                                     (-drho                         *v3                           +&
                                                                     box%v3(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
!#
#         if (rhd_box_bmag01 == 1)
!#
          ! --- Magnetic field quantities ---
          if (ALLOASSOTED(boxout%Bc1) .and. ALLOASSOTED(box%Bc2)) then
            Bc1                          =boxout%Bc1(       i1out,i2out,i3out)
            boxout%Bc1(i1out,i2out,i3out)=Bc1 +  box%Bc2(   i1   ,i2   ,i3   )
          endif
          !
          if (ALLOASSOTED(boxout%Bc2) .and. ALLOASSOTED(box%Bc1)) then
            Bc2                          =boxout%Bc2(       i1out,i2out,i3out)
            boxout%Bc2(i1out,i2out,i3out)=Bc2 + box%Bc1(   i1   ,i2   ,i3   )
          endif
          !
          if (ALLOASSOTED(boxout%Bc3) .and. ALLOASSOTED(box%Bc3)) then
            Bc3                          =boxout%Bc3(       i1out,i2out,i3out)
            boxout%Bc3(i1out,i2out,i3out)=Bc3 + box%Bc3(   i1   ,i2   ,i3   )
          endif
          !
          ei                           =boxout%ei( i1out,i2out,i3out)
          if (ALLOASSOTED(boxout%energy_switch) .and. ALLOASSOTED(box%energy_switch)) then
            if (boxout%energy_switch(i1out,i2out,i3out)) then
              boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out) + &
                                      0.5*((Bc1**2+Bc2**2+Bc3**2             ) - &
                                           (boxout%Bc1( i1out,i2out,i3out)**2 + &
                                            boxout%Bc2( i1out,i2out,i3out)**2 + &
                                            boxout%Bc3( i1out,i2out,i3out)**2) &
                                          )/boxout%rho( i1out,i2out,i3out)
            else
              boxout%ei( i1out,i2out,i3out)=ei + &
                                     (-drho                         *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)
            endif
            !
            boxout%energy_switch(i1out,i2out,i3out)=box%energy_switch(   i1   ,i2   ,i3   )
            !
          else
            boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out)
          endif
!#
#         else
!#             /* rhd_box_bmag01 */
          ei                           =boxout%ei( i1out,i2out,i3out)
          boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out)
!#
#         endif
!#              /* rhd_box_bmag01 */
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
    !
    ! --- Check if internal energy is above minimum table value everywhere ---
    einew_min=minval(boxout%ei(m1out:n1out,m2out:n2out,m3out:n3out))
    if (einew_min < eiTable_min) then
      ierr0  =5
      outstr0='Internal energy below minimum table value'
      goto 999
    endif
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    if (ALLOASSOTED(boxout%flux_B12) .and. ALLOASSOTED(box%flux_B21) .and. &
        ALLOASSOTED(boxout%flux_B32) .and. ALLOASSOTED(box%flux_B31)) then
      do i3out=m3out,n3out
        do i2out=m2out,n2out+1
          !DIR$ IVDEP
          do i1out=m1out,n1out
            ! --- Indices ---
            i1=i2out-m2out+m1
            i2=i1out-m1out+m2
            i3=i3out-m3out+m3
            !
            boxout%flux_B12(i1out,i2out,i3out)=    box%flux_B21(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B12(i1out,i2out,i3out)
            !
            boxout%flux_B32(i1out,i2out,i3out)=    box%flux_B31(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B32(i1out,i2out,i3out)
            !
          end do ! i1out
        end do ! i2out
      end do ! i3out
    endif
!#
#    endif
!#         /* rhd_box_bmag01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i3=m3
        do i3out=m3out,n3out
          !DIR$ IVDEP
          boxout%quc(          m1out:n1out,m2out:n2out,i3out,iquc)= &
            boxout%quc(        m1out:n1out,m2out:n2out,i3out,iquc) + &
             transpose(box%quc(m1   :n1   ,m2   :n2   ,i3   ,iquc))
          i3=i3+1
        end do ! i3out
      end do ! iquc
      !
      ! --- Check if additional density arrays are non-negative everywhere ---
      do iquc=1,box%nquc
        if (prop%quc_positive(iquc) > 0) then
          qucnew_min=minval(boxout%quc(m1out:n1out,m2out:n2out,m3out:n3out,iquc))
          if (qucnew_min < 0.0) then
            ierr0  =6
            outstr0='Additional density below zero'
            goto 999
          endif
        endif
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
  !
else if (trans_mode0 == 3) then
  ! --- Transpose 3 -> 1 ---------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n3-m3
  n2out=m2out + n2-m2
  n3out=m3out + n1-m1
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb3(m3:n3+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb1(m1:n1+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    ! --- Check if density is positive everywhere ---
    rhonew_min=huge(rhonew_min)
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i3out-m3out+m1
          i2=i2out-m2out+m2
          i3=i1out-m1out+m3
          !
          rhonew_min=min(rhonew_min, boxout%rho(i1out,i2out,i3out)+ &
                                     box%rho(   i1   ,i2   ,i3   ))
        end do ! i1out
      end do ! i2out
    end do ! i3out
    if (rhonew_min <= 0.0) then
      ierr0  =7
      outstr0='Density becomes non-positive'
      goto 999
    endif
    !
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        !DIR$ IVDEP
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i3out-m3out+m1
          i2=i2out-m2out+m2
          i3=i1out-m1out+m3
          !
          ! --- Bulk HD quantities ---
          rho                          =boxout%rho(i1out,i2out,i3out)
          drho                         =box%rho(   i1   ,i2   ,i3   )
          boxout%rho(i1out,i2out,i3out)=rho + drho
          !
          v1                           =boxout%v1( i1out,i2out,i3out)
          boxout%v1( i1out,i2out,i3out)=v1 + &
                                     (-drho                         *v1                           +&
                                                                     box%v3(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v2                           =boxout%v2( i1out,i2out,i3out)
          boxout%v2( i1out,i2out,i3out)=v2 + &
                                     (-drho                         *v2                           +&
                                                                     box%v2(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v3                           =boxout%v3( i1out,i2out,i3out)
          boxout%v3( i1out,i2out,i3out)=v3 + &
                                     (-drho                         *v3                           +&
                                                                     box%v1(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
!#
#         if (rhd_box_bmag01 == 1)
!#
          ! --- Magnetic field quantities ---
          if (ALLOASSOTED(boxout%Bc1) .and. ALLOASSOTED(box%Bc3)) then
            Bc1                          =boxout%Bc1(       i1out,i2out,i3out)
            boxout%Bc1(i1out,i2out,i3out)=Bc1 + box%Bc3(   i1   ,i2   ,i3   )
          endif
          !
          if (ALLOASSOTED(boxout%Bc2) .and. ALLOASSOTED(box%Bc2)) then
            Bc2                          =boxout%Bc2(       i1out,i2out,i3out)
            boxout%Bc2(i1out,i2out,i3out)=Bc2 + box%Bc2(   i1   ,i2   ,i3   )
          endif
          !
          if (ALLOASSOTED(boxout%Bc3) .and. ALLOASSOTED(box%Bc1)) then
            Bc3                          =boxout%Bc3(       i1out,i2out,i3out)
            boxout%Bc3(i1out,i2out,i3out)=Bc3 + box%Bc1(   i1   ,i2   ,i3   )
          endif
          !
          ei                           =boxout%ei( i1out,i2out,i3out)
          if (ALLOASSOTED(boxout%energy_switch) .and. ALLOASSOTED(box%energy_switch)) then
            if (boxout%energy_switch(i1out,i2out,i3out)) then
              boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out) + &
                                      0.5*((Bc1**2+Bc2**2+Bc3**2             ) - &
                                           (boxout%Bc1( i1out,i2out,i3out)**2 + &
                                            boxout%Bc2( i1out,i2out,i3out)**2 + &
                                            boxout%Bc3( i1out,i2out,i3out)**2) &
                                          )/boxout%rho( i1out,i2out,i3out)
            else
              boxout%ei( i1out,i2out,i3out)=ei + &
                                     (-drho                         *ei                           +&
                                                                     box%v1(   i1   ,i2   ,i3   ) &
                                     )/boxout%rho(i1out,i2out,i3out)
            endif
            !
            boxout%energy_switch(i1out,i2out,i3out)=box%energy_switch(   i1   ,i2   ,i3   )
            !
          else
            boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out)
          endif
!#
#         else
!#             /* rhd_box_bmag01 */
          ei                           =boxout%ei( i1out,i2out,i3out)
          boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                                                     box%ei(   i1   ,i2   ,i3   ) -&
                                       0.5*drho*(v1**2+v2**2+v3**2) &
                                      )/boxout%rho(     i1out,i2out,i3out)
!#
#         endif
!#              /* rhd_box_bmag01 */
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
    !
    ! --- Check if internal energy is above minimum table value everywhere ---
    einew_min=minval(boxout%ei(m1out:n1out,m2out:n2out,m3out:n3out))
    if (einew_min < eiTable_min) then
      ierr0  =8
      outstr0='Internal energy below minimum table value'
      goto 999
    endif
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    if (ALLOASSOTED(boxout%flux_B23) .and. ALLOASSOTED(box%flux_B21) .and. &
        ALLOASSOTED(boxout%flux_B13) .and. ALLOASSOTED(box%flux_B31)) then
      do i3out=m3out,n3out+1
        do i2out=m2out,n2out
          !DIR$ IVDEP
          do i1out=m1out,n1out
            ! --- Indices ---
            i1=i3out-m3out+m1
            i2=i2out-m2out+m2
            i3=i1out-m1out+m3
            !
            boxout%flux_B23(i1out,i2out,i3out)=    box%flux_B21(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B23(i1out,i2out,i3out)
            !
            boxout%flux_B13(i1out,i2out,i3out)=    box%flux_B31(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B13(i1out,i2out,i3out)
            !
          end do ! i1out
        end do ! i2out
      end do ! i3out
    endif
!#
#    endif
!#         /* rhd_box_bmag01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i2=m2
        do i2out=m2out,n2out
          !DIR$ IVDEP
          boxout%quc(          m1out:n1out,i2out,m3out:n3out,iquc)= &
            boxout%quc(        m1out:n1out,i2out,m3out:n3out,iquc) + &
             transpose(box%quc(m1   :   n1,i2   ,m3   :n3   ,iquc))
          i2=i2+1
        end do ! i2out
      end do ! iquc
      !
      ! --- Check if additional density arrays are non-negative everywhere ---
      do iquc=1,box%nquc
        if (prop%quc_positive(iquc) > 0) then
          qucnew_min=minval(boxout%quc(m1out:n1out,m2out:n2out,m3out:n3out,iquc))
          if (qucnew_min < 0.0) then
            ierr0  =9
            outstr0='Additional density below zero'
            goto 999
          endif
        endif
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_box_AddDelta


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_AddDelta2(box, m1,n1, m2,n2, m3,n3, &
                             boxout, m1out, m2out, m3out, eiTable_min, &
                             time_mode, grid_mode, bulk_mode, &
                             outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AddDelta2 ('rhd_box_Add_Delta_2')
!
! PURPOSE:
!   Add change of (part of) box into another. Transpose, if requested.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AddDelta2(box, m1,n1, m2,n2, m3,n3, &
!                          boxout, m1out, m2out, m3out, eiTable_min, &
!                          time_mode=time_mode, grid_mode=grid_mode, bulk_mode=bulk_mode, &
!                          outstr=outstr,ierr=ierr)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!   eiTable_min:  ('energy_internal_Table_minimum') real with minimum allowed internal-energy value
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   0: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:0
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error: (currently none)
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   box has to contain the normal variables: rho, v1,2,3, ei
!   boxout has to contain the changes of the corresp. conserved variables: drho, drhov1,2,3, drhoei
!
! PROCEDURE:
!   Short: boxout=(boxout)+box
!
! EXAMPLE:
!
! TODO:
!   energy equation
!   magnetic-field case
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written (copy_box)
!   1997-10-16 (B.F.) In case of transposition, handle momenta properly
!   1999-05-31 (B.F. Copenhagen) Keywords grid and time added
!   2000-02-07 (B.F.) Keyword bulk added
!   2001-01-21 (B.F. Uppsala) Single bulk copy loops for each quantity
!   2001-01-29 (B.F.) Copy entire 3D arrays en block
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2007-02-08 (B.F. Lyon) Adapted from copy_box, no grav. potential or magnetic fields
!   2010-04-20 (B.F.) Rename addchange_box -> rhd_box_AddChange
!   2010-05-26 (B.F.) Bugfix: add total energies instead of internal energies only
!   2010-06-13 (B.F.) Cell-centered magnetic field
!   2010-06-15 (B.F.) Bugfix: use vector arrays with correction direction
!   2010-06-16 (B.F.) flux_B arrays are overwritten and not updated
!   2010-07-16 (B.F. Naples) flux_B arrays are directly copied without reference to boxref
!   2011-02-28 (W.S. Exeter) dual-energy method, check if MHD arrays are present
!   2011-08-02 (B.F. Lyon) Taken from rhd_box_AddChange
!   2011-08-03 (B.F.) Perform checks on positivity, new parameters
!   2011-12-15 (B.F.) Written based on rhd_AddDelta; no magnetic fields
!   2013-03-05 (B.F.) Add directives to help vectorization
!   2017-05-22 (B.F. Uppsala) Control check for positivity of quc arrays by quc_positive array
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1out, m2out, m3out
real,                      intent(in)   :: eiTable_min
integer,         optional, intent(in)   :: grid_mode, time_mode, bulk_mode
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: grid_mode0, time_mode0, bulk_mode0
integer                                 :: n1out, n2out, n3out
integer                                 :: i1, i2, i3, i1out, i2out, i3out, iquc
real                                    :: rho, drho, v1, v2, v3, ei, &
                                           rhonew_min, einew_min, qucnew_min
integer                                 :: ierr0
character(len=80)                       :: outstr0
!---------------------------------------------------------------------------------------------------
!
! --- Default values ---
ierr0  =0
outstr0=''
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=0
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
!
if (time_mode0 > 0) then
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif
!
! --- Indices ---
n1out=m1out + n1-m1
n2out=m2out + n2-m2
n3out=m3out + n3-m3
!
! --- Grid ---
if (grid_mode0 > 0) then
  !DIR$ IVDEP
  boxout%xc1(m1out:n1out  )=box%xc1(m1:n1  )
  !DIR$ IVDEP
  boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
  !DIR$ IVDEP
  boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
  !DIR$ IVDEP
  boxout%xb1(m1out:n1out+1)=box%xb1(m1:n1+1)
  !DIR$ IVDEP
  boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
  !DIR$ IVDEP
  boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
endif
!
if (bulk_mode0 > 0) then
  ! --- HD quantities ---
  !
  ! --- Check if density is positive everywhere ---
  rhonew_min=huge(rhonew_min)
  do i3out=m3out,n3out
    do i2out=m2out,n2out
      do i1out=m1out,n1out
        ! --- Indices ---
        i1=i1out-m1out+m1
        i2=i2out-m2out+m2
        i3=i3out-m3out+m3
        !
        rhonew_min=min(rhonew_min, boxout%rho(i1out,i2out,i3out)+ &
                                   box%rho(   i1   ,i2   ,i3   ))
      end do ! i1out
    end do ! i2out
  end do ! i3out
  if (rhonew_min <= 0.0) then
    ierr0  =1
    outstr0='Density becomes non-positive'
    goto 999
  endif
  !
  do i3out=m3out,n3out
    do i2out=m2out,n2out
      !DIR$ IVDEP
      do i1out=m1out,n1out
        ! --- Indices ---
        i1=i1out-m1out+m1
        i2=i2out-m2out+m2
        i3=i3out-m3out+m3
        !
        ! --- Bulk HD quantities ---
        rho                          =box%rho(   i1   ,i2   ,i3   )
        drho                         =boxout%rho(i1out,i2out,i3out)
        boxout%rho(i1out,i2out,i3out)=rho + drho
        !
        v1                           =box%v1(  i1   ,i2   ,i3   )
        boxout%v1( i1out,i2out,i3out)=v1 + &
                                    (-drho                         *v1                           + &
                                                                    boxout%v1(i1out,i2out,i3out) &
                                    )/boxout%rho(i1out,i2out,i3out)
        !
        v2                           =box%v2(  i1   ,i2   ,i3   )
        boxout%v2( i1out,i2out,i3out)=v2 + &
                                    (-drho                         *v2                           + &
                                                                    boxout%v2(i1out,i2out,i3out) &
                                    )/boxout%rho(i1out,i2out,i3out)
        !
        v3                           =box%v3(  i1   ,i2   ,i3   )
        boxout%v3( i1out,i2out,i3out)=v3 + &
                                    (-drho                         *v3                           + &
                                                                    boxout%v3(i1out,i2out,i3out) &
                                    )/boxout%rho(i1out,i2out,i3out)
        !
        ei                           =box%ei(  i1   ,i2   ,i3   )
        boxout%ei( i1out,i2out,i3out)=ei + &
                                    0.5*((v1**2+v2**2+v3**2) - &
                                         (boxout%v1(i1out,i2out,i3out)**2 + &
                                          boxout%v2(i1out,i2out,i3out)**2 + &
                                          boxout%v3(i1out,i2out,i3out)**2) &
                                        ) + &
                                    (-drho                         *ei                           + &
                                                                    boxout%ei(i1out,i2out,i3out) - &
                                     0.5*drho*(v1**2+v2**2+v3**2) &
                                    )/boxout%rho(i1out,i2out,i3out)
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
    !
    ! --- Check if internal energy is above minimum table value everywhere ---
    einew_min=minval(boxout%ei(m1out:n1out,m2out:n2out,m3out:n3out))
    if (einew_min < eiTable_min) then
      ierr0  =2
      outstr0='Internal energy below minimum table value'
      goto 999
    endif
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      !
      do iquc=1,box%nquc
        !DIR$ IVDEP
        boxout%quc(   m1out:n1out,m2out:n2out,m3out:n3out,iquc)= &
          boxout%quc( m1out:n1out,m2out:n2out,m3out:n3out,iquc) + &
           box%quc(   m1   :n1   ,m2   :n2   ,m3   :n3   ,iquc)
      end do ! iquc
      !
      ! --- Check if additional density arrays are non-negative everywhere ---
      do iquc=1,box%nquc
        if (prop%quc_positive(iquc) > 0) then
          qucnew_min=minval(boxout%quc(m1out:n1out,m2out:n2out,m3out:n3out,iquc))
          if (qucnew_min < 0.0) then
            ierr0  =3
            outstr0='Additional density below zero'
            goto 999
          endif
        endif
      end do ! iquc
    endif
    !
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
!
999 if (present(outstr)) outstr=outstr0
    if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_box_AddDelta2


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_AddChange(box, m1,n1, m2,n2, m3,n3, &
                             boxref, boxout, m1out, m2out, m3out, &
                             trans_mode, time_mode, grid_mode, bulk_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AddChange ('rhd_box_Add_Change')
!
! PURPOSE:
!   Add change of (part of) box into another. Transpose, if requested.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AddChange(box, m1,n1, m2,n2, m3,n3, &
!                          boxref, boxout, m1out, m2out, m3out, &
!                          trans_mode=trans_mode, time_mode=time_mode, grid_mode=grid_mode, &
!                          bulk_mode=bulk_mode)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   boxref:       ('box_reference') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!   trans_mode:   ('transpose_mode') optional, integer, (1,2,3), source dimension (box) which will
!                 be 1st target dimension (boxref, boxout), default: 1
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   0: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:0
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Short: boxout=boxout+(box-boxref)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written (copy_box)
!   1997-10-16 (B.F.) In case of transposition, handle momenta properly
!   1999-05-31 (B.F. Copenhagen) Keywords grid and time added
!   2000-02-07 (B.F.) Keyword bulk added
!   2001-01-21 (B.F. Uppsala) Single bulk copy loops for each quantity
!   2001-01-29 (B.F.) Copy entire 3D arrays en block
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2007-02-08 (B.F. Lyon) Adapted from copy_box, no grav. potential or magnetic fields
!   2010-04-20 (B.F.) Rename addchange_box -> rhd_box_AddChange
!   2010-05-26 (B.F.) Bugfix: add total energies instead of internal energies only
!   2010-06-13 (B.F.) Cell-centered magnetic field
!   2010-06-15 (B.F.) Bugfix: use vector arrays with correction direction
!   2010-06-16 (B.F.) flux_B arrays are overwritten and not updated
!   2010-07-16 (B.F. Naples) flux_B arrays are directly copied without reference to boxref
!   2011-02-28 (W.S. Exeter) dual-energy method, check if MHD arrays are present
!   2012-11-28 (W.S.) Bugfix: Error in computation of internal energy corrected
!   2013-03-05 (B.F.) Add directives to help vectorization
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
type(box_type),            intent(in)   :: boxref
integer,                   intent(in)   :: m1out, m2out, m3out
integer,         optional, intent(in)   :: trans_mode, grid_mode, time_mode, bulk_mode
!
! --- Local variables ---
integer                                 :: trans_mode0, grid_mode0, time_mode0, bulk_mode0
integer                                 :: n1out, n2out, n3out
integer                                 :: i1, i2, i3, i1out, i2out, i3out, iquc
real                                    :: rho, drho, v1, v2, v3, ei, Bc1, Bc2, Bc3
!---------------------------------------------------------------------------------------------------
!
! --- Default values ---
if (present(trans_mode)) then
  trans_mode0=trans_mode
else
  trans_mode0=1
endif
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=0
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
!
if (time_mode0 > 0) then
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif
!
if (trans_mode0 == 1) then
  ! --- Copy directly ------------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n1-m1
  n2out=m2out + n2-m2
  n3out=m3out + n3-m3
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        !DIR$ IVDEP
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i1out-m1out+m1
          i2=i2out-m2out+m2
          i3=i3out-m3out+m3
          !
          ! --- Bulk HD quantities ---
          rho                          =boxout%rho(i1out,i2out,i3out)
          drho                         =box%rho(   i1   ,i2   ,i3   ) - &
                                        boxref%rho(i1out,i2out,i3out)
          boxout%rho(i1out,i2out,i3out)=rho + drho
          !
          v1                           =boxout%v1( i1out,i2out,i3out)
          boxout%v1( i1out,i2out,i3out)=v1 + &
                                     (-drho                         *v1                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v1(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v1(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v2                           =boxout%v2( i1out,i2out,i3out)
          boxout%v2( i1out,i2out,i3out)=v2 + &
                                     (-drho                         *v2                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v2(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v2(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v3                           =boxout%v3( i1out,i2out,i3out)
          boxout%v3( i1out,i2out,i3out)=v3 + &
                                     (-drho                         *v3                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v3(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v3(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
!#
#         if (rhd_box_bmag01 == 1)
!#
          ! --- Magnetic field quantities ---
          if (ALLOASSOTED(boxout%Bc1) .and. ALLOASSOTED(box%Bc1)) then
            Bc1                          =boxout%Bc1(       i1out,i2out,i3out)
            boxout%Bc1(i1out,i2out,i3out)=Bc1 + (box%Bc1(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc1(i1out,i2out,i3out))
          endif
          !
          if (ALLOASSOTED(boxout%Bc2) .and. ALLOASSOTED(box%Bc2)) then
            Bc2                          =boxout%Bc2(       i1out,i2out,i3out)
            boxout%Bc2(i1out,i2out,i3out)=Bc2 + (box%Bc2(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc2(i1out,i2out,i3out))
          endif
          !
          if (ALLOASSOTED(boxout%Bc3) .and. ALLOASSOTED(box%Bc3)) then
            Bc3                          =boxout%Bc3(       i1out,i2out,i3out)
            boxout%Bc3(i1out,i2out,i3out)=Bc3 + (box%Bc3(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc3(i1out,i2out,i3out))
          endif
          !
          ei                           =boxout%ei( i1out,i2out,i3out)
          if (ALLOASSOTED(boxout%energy_switch) .and. ALLOASSOTED(box%energy_switch)) then
            if (boxout%energy_switch(i1out,i2out,i3out)) then
              boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out) + &
                                      0.5*((Bc1**2+Bc2**2+Bc3**2             ) - &
                                           (boxout%Bc1( i1out,i2out,i3out)**2 + &
                                            boxout%Bc2( i1out,i2out,i3out)**2 + &
                                            boxout%Bc3( i1out,i2out,i3out)**2) + &
                                           (box%Bc1(    i1   ,i2   ,i3   )**2 + &
                                            box%Bc2(    i1   ,i2   ,i3   )**2 + &
                                            box%Bc3(    i1   ,i2   ,i3   )**2) - &
                                           (boxref%Bc1( i1out,i2out,i3out)**2 + &
                                            boxref%Bc2( i1out,i2out,i3out)**2 + &
                                            boxref%Bc3( i1out,i2out,i3out)**2) &
                                          )/boxout%rho( i1out,i2out,i3out)
            else
              boxout%ei( i1out,i2out,i3out)=ei + &
                                     (-drho                         *ei                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)
            endif
            !
            boxout%energy_switch(i1out,i2out,i3out)=box%energy_switch(   i1   ,i2   ,i3   )
            !
          else
            boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out)
          endif
!#
#         else
!#             /* rhd_box_bmag01 */
          ei                           =boxout%ei( i1out,i2out,i3out)
          boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out)
!#
#         endif
!#              /* rhd_box_bmag01 */
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    if (ALLOASSOTED(boxout%flux_B21) .and. ALLOASSOTED(box%flux_B21) .and. &
        ALLOASSOTED(boxout%flux_B31) .and. ALLOASSOTED(box%flux_B31)) then
      do i3out=m3out,n3out
        do i2out=m2out,n2out
          !DIR$ IVDEP
          do i1out=m1out,n1out+1
            ! --- Indices ---
            i1=i1out-m1out+m1
            i2=i2out-m2out+m2
            i3=i3out-m3out+m3
            !
            boxout%flux_B21(i1out,i2out,i3out)=    box%flux_B21(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B21(i1out,i2out,i3out)
            !
            boxout%flux_B31(i1out,i2out,i3out)=    box%flux_B31(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B31(i1out,i2out,i3out)
            !
          end do ! i1out
        end do ! i2out
      end do ! i3out
    endif
!#
#    endif
!#         /* rhd_box_bmag01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        !DIR$ IVDEP
        boxout%quc(   m1out:n1out,m2out:n2out,m3out:n3out,iquc)= &
          boxout%quc( m1out:n1out,m2out:n2out,m3out:n3out,iquc) + &
          (box%quc(   m1   :n1   ,m2   :n2   ,m3   :n3   ,iquc) - &
           boxref%quc(m1out:n1out,m2out:n2out,m3out:n3out,iquc))
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
  !
else if (trans_mode0 == 2) then
  ! --- Transpose 2 -> 1 ---------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n2-m2
  n2out=m2out + n1-m1
  n3out=m3out + n3-m3
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        !DIR$ IVDEP
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i2out-m2out+m1
          i2=i1out-m1out+m2
          i3=i3out-m3out+m3
          !
          ! --- Bulk HD quantities ---
          rho                          =boxout%rho(i1out,i2out,i3out)
          drho                         =box%rho(   i1   ,i2   ,i3   ) - &
                                        boxref%rho(i1out,i2out,i3out)
          boxout%rho(i1out,i2out,i3out)=rho + drho
          !
          v1                           =boxout%v1( i1out,i2out,i3out)
          boxout%v1( i1out,i2out,i3out)=v1 + &
                                     (-drho                         *v1                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v2(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v1(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v2                           =boxout%v2( i1out,i2out,i3out)
          boxout%v2( i1out,i2out,i3out)=v2 + &
                                     (-drho                         *v2                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v1(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v2(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v3                           =boxout%v3( i1out,i2out,i3out)
          boxout%v3( i1out,i2out,i3out)=v3 + &
                                     (-drho                         *v3                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v3(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v3(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
!#
#         if (rhd_box_bmag01 == 1)
!#
          ! --- Magnetic field quantities ---
          if (ALLOASSOTED(boxout%Bc1) .and. ALLOASSOTED(box%Bc2)) then
            Bc1                          =boxout%Bc1(       i1out,i2out,i3out)
            boxout%Bc1(i1out,i2out,i3out)=Bc1 + (box%Bc2(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc1(i1out,i2out,i3out))
          endif
          !
          if (ALLOASSOTED(boxout%Bc2) .and. ALLOASSOTED(box%Bc1)) then
            Bc2                          =boxout%Bc2(       i1out,i2out,i3out)
            boxout%Bc2(i1out,i2out,i3out)=Bc2 + (box%Bc1(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc2(i1out,i2out,i3out))
          endif
          !
          if (ALLOASSOTED(boxout%Bc3) .and. ALLOASSOTED(box%Bc3)) then
            Bc3                          =boxout%Bc3(       i1out,i2out,i3out)
            boxout%Bc3(i1out,i2out,i3out)=Bc3 + (box%Bc3(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc3(i1out,i2out,i3out))
          endif
          !
          ei                           =boxout%ei( i1out,i2out,i3out)
          if (ALLOASSOTED(boxout%energy_switch) .and. ALLOASSOTED(box%energy_switch)) then
            if (boxout%energy_switch(i1out,i2out,i3out)) then
              boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out) + &
                                      0.5*((Bc1**2+Bc2**2+Bc3**2             ) - &
                                           (boxout%Bc1( i1out,i2out,i3out)**2 + &
                                            boxout%Bc2( i1out,i2out,i3out)**2 + &
                                            boxout%Bc3( i1out,i2out,i3out)**2) + &
                                           (box%Bc1(    i1   ,i2   ,i3   )**2 + &
                                            box%Bc2(    i1   ,i2   ,i3   )**2 + &
                                            box%Bc3(    i1   ,i2   ,i3   )**2) - &
                                           (boxref%Bc1( i1out,i2out,i3out)**2 + &
                                            boxref%Bc2( i1out,i2out,i3out)**2 + &
                                            boxref%Bc3( i1out,i2out,i3out)**2) &
                                          )/boxout%rho( i1out,i2out,i3out)
            else
              boxout%ei( i1out,i2out,i3out)=ei + &
                                     (-drho                         *ei                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)
            endif
            !
            boxout%energy_switch(i1out,i2out,i3out)=box%energy_switch(   i1   ,i2   ,i3   )
            !
          else
            boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out)
          endif
!#
#         else
!#             /* rhd_box_bmag01 */
          ei                           =boxout%ei( i1out,i2out,i3out)
          boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out)
!#
#         endif
!#              /* rhd_box_bmag01 */
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
    !
!#
#   if (rhd_box_bmag01 == 1)
!#
    if (ALLOASSOTED(boxout%flux_B12) .and. ALLOASSOTED(box%flux_B21) .and. &
        ALLOASSOTED(boxout%flux_B32) .and. ALLOASSOTED(box%flux_B31)) then
      do i3out=m3out,n3out
        do i2out=m2out,n2out+1
          !DIR$ IVDEP
          do i1out=m1out,n1out
            ! --- Indices ---
            i1=i2out-m2out+m1
            i2=i1out-m1out+m2
            i3=i3out-m3out+m3
            !
            boxout%flux_B12(i1out,i2out,i3out)=    box%flux_B21(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B12(i1out,i2out,i3out)
            !
            boxout%flux_B32(i1out,i2out,i3out)=    box%flux_B31(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B32(i1out,i2out,i3out)
            !
          end do ! i1out
        end do ! i2out
      end do ! i3out
    endif
!#
#    endif
!#         /* rhd_box_bmag01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i3=m3
        do i3out=m3out,n3out
          !DIR$ IVDEP
          boxout%quc(          m1out:n1out,m2out:n2out,i3out,iquc)= &
            boxout%quc(        m1out:n1out,m2out:n2out,i3out,iquc) + &
            (transpose(box%quc(m1   :n1   ,m2   :n2   ,i3   ,iquc)) - &
             boxref%quc(       m1out:n1out,m2out:n2out,i3out,iquc))
          i3=i3+1
        end do ! i3out
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
  !
else if (trans_mode0 == 3) then
  ! --- Transpose 3 -> 1 ---------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n3-m3
  n2out=m2out + n2-m2
  n3out=m3out + n1-m1
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb3(m3:n3+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb1(m1:n1+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    do i3out=m3out,n3out
      do i2out=m2out,n2out
        !DIR$ IVDEP
        do i1out=m1out,n1out
          ! --- Indices ---
          i1=i3out-m3out+m1
          i2=i2out-m2out+m2
          i3=i1out-m1out+m3
          !
          ! --- Bulk HD quantities ---
          rho                          =boxout%rho(i1out,i2out,i3out)
          drho                         =box%rho(   i1   ,i2   ,i3   ) - &
                                        boxref%rho(i1out,i2out,i3out)
          boxout%rho(i1out,i2out,i3out)=rho + drho
          !
          v1                           =boxout%v1( i1out,i2out,i3out)
          boxout%v1( i1out,i2out,i3out)=v1 + &
                                     (-drho                         *v1                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v3(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v1(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v2                           =boxout%v2( i1out,i2out,i3out)
          boxout%v2( i1out,i2out,i3out)=v2 + &
                                     (-drho                         *v2                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v2(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v2(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
          v3                           =boxout%v3( i1out,i2out,i3out)
          boxout%v3( i1out,i2out,i3out)=v3 + &
                                     (-drho                         *v3                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%v1(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%v3(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)

          !
!#
#         if (rhd_box_bmag01 == 1)
!#
          ! --- Magnetic field quantities ---
          if (ALLOASSOTED(boxout%Bc1) .and. ALLOASSOTED(box%Bc3)) then
            Bc1                          =boxout%Bc1(       i1out,i2out,i3out)
            boxout%Bc1(i1out,i2out,i3out)=Bc1 + (box%Bc3(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc1(i1out,i2out,i3out))
          endif
          !
          if (ALLOASSOTED(boxout%Bc2) .and. ALLOASSOTED(box%Bc2)) then
            Bc2                          =boxout%Bc2(       i1out,i2out,i3out)
            boxout%Bc2(i1out,i2out,i3out)=Bc2 + (box%Bc2(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc2(i1out,i2out,i3out))
          endif
          !
          if (ALLOASSOTED(boxout%Bc3) .and. ALLOASSOTED(box%Bc1)) then
            Bc3                          =boxout%Bc3(       i1out,i2out,i3out)
            boxout%Bc3(i1out,i2out,i3out)=Bc3 + (box%Bc1(   i1   ,i2   ,i3   ) - &
                                                 boxref%Bc3(i1out,i2out,i3out))
          endif
          !
          ei                           =boxout%ei( i1out,i2out,i3out)
          if (ALLOASSOTED(boxout%energy_switch) .and. ALLOASSOTED(box%energy_switch)) then
            if (boxout%energy_switch(i1out,i2out,i3out)) then
              boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out) + &
                                      0.5*((Bc1**2+Bc2**2+Bc3**2             ) - &
                                           (boxout%Bc1( i1out,i2out,i3out)**2 + &
                                            boxout%Bc2( i1out,i2out,i3out)**2 + &
                                            boxout%Bc3( i1out,i2out,i3out)**2) + &
                                           (box%Bc1(    i1   ,i2   ,i3   )**2 + &
                                            box%Bc2(    i1   ,i2   ,i3   )**2 + &
                                            box%Bc3(    i1   ,i2   ,i3   )**2) - &
                                           (boxref%Bc1( i1out,i2out,i3out)**2 + &
                                            boxref%Bc2( i1out,i2out,i3out)**2 + &
                                            boxref%Bc3( i1out,i2out,i3out)**2) &
                                          )/boxout%rho( i1out,i2out,i3out)
            else
              boxout%ei( i1out,i2out,i3out)=ei + &
                                     (-drho                         *ei                           +&
                                      box%rho(    i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                      boxref%rho( i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) &
                                     )/boxout%rho(i1out,i2out,i3out)
            endif
            !
            boxout%energy_switch(i1out,i2out,i3out)=box%energy_switch(   i1   ,i2   ,i3   )
            !
          else
            boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out)
          endif
!#
#         else
!#             /* rhd_box_bmag01 */
          ei                           =boxout%ei( i1out,i2out,i3out)
          boxout%ei( i1out,i2out,i3out)=ei + &
                                      0.5*((v1**2+v2**2+v3**2) - &
                                           (boxout%v1(i1out,i2out,i3out)**2 + &
                                            boxout%v2(i1out,i2out,i3out)**2 + &
                                            boxout%v3(i1out,i2out,i3out)**2) &
                                          ) + &
                                      (-drho                        *ei                           +&
                                       box%rho(   i1   ,i2   ,i3   )*box%ei(   i1   ,i2   ,i3   ) -&
                                       boxref%rho(i1out,i2out,i3out)*boxref%ei(i1out,i2out,i3out) +&
                                       0.5*(-drho                          * &
                                            (v1**2+v2**2+v3**2) + &
                                             box%rho(   i1   ,i2   ,i3   ) * &
                                            (box%v1(    i1   ,i2   ,i3   )**2 + &
                                             box%v2(    i1   ,i2   ,i3   )**2 + &
                                             box%v3(    i1   ,i2   ,i3   )**2) - &
                                             boxref%rho(i1out,i2out,i3out) * &
                                            (boxref%v1( i1out,i2out,i3out)**2 + &
                                             boxref%v2( i1out,i2out,i3out)**2 + &
                                             boxref%v3( i1out,i2out,i3out)**2) &
                                           ) &
                                      )/boxout%rho(     i1out,i2out,i3out)
!#
#         endif
!#              /* rhd_box_bmag01 */
          !
        end do ! i1out
      end do ! i2out
    end do ! i3out
!#
#   if (rhd_box_bmag01 == 1)
!#
    if (ALLOASSOTED(boxout%flux_B23) .and. ALLOASSOTED(box%flux_B21) .and. &
        ALLOASSOTED(boxout%flux_B13) .and. ALLOASSOTED(box%flux_B31)) then
      do i3out=m3out,n3out+1
        do i2out=m2out,n2out
          !DIR$ IVDEP
          do i1out=m1out,n1out
            ! --- Indices ---
            i1=i3out-m3out+m1
            i2=i2out-m2out+m2
            i3=i1out-m1out+m3
            !
            boxout%flux_B23(i1out,i2out,i3out)=    box%flux_B21(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B23(i1out,i2out,i3out)
            !
            boxout%flux_B13(i1out,i2out,i3out)=    box%flux_B31(i1   ,i2   ,i3   )
            !                                   -boxref%flux_B13(i1out,i2out,i3out)
            !
          end do ! i1out
        end do ! i2out
      end do ! i3out
    endif
!#
#   endif
!#        /* rhd_box_bmag01 */
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i2=m2
        do i2out=m2out,n2out
          !DIR$ IVDEP
          boxout%quc(          m1out:n1out,i2out,m3out:n3out,iquc)= &
            boxout%quc(        m1out:n1out,i2out,m3out:n3out,iquc) + &
            (transpose(box%quc(m1   :   n1,i2   ,m3   :n3   ,iquc)) - &
             boxref%quc(       m1out:n1out,i2out,m3out:n3out,iquc))
          i2=i2+1
        end do ! i2out
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
endif
!
end subroutine rhd_box_AddChange


!----------***********------------------------------------------------------------------------------
subroutine rhd_box_Add(box, m1,n1, m2,n2, m3,n3, &
                       boxout, m1out, m2out, m3out, &
                       trans_mode, time_mode, grid_mode, bulk_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Add ('rhd_box_Add')
!
! PURPOSE:
!   Add (part of) box to another. Transpose, if requested.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Add(box, m1,n1, m2,n2, m3,n3, &
!                    boxout, m1out, m2out, m3out, &
!                    trans_mode=trans_mode, time_mode=time_mode, grid_mode=grid_mode, &
!                    bulk_mode=bulk_mode)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   m1out:        ('m_number_1_out') integer, 1. dimension, start index, target
!   m2out:        ('m_number_2_out') integer, 2. dimension, start index, target
!   m3out:        ('m_number_3_out') integer, 3. dimension, start index, target
!   trans_mode:   ('transpose_mode') optional, integer, (1,2,3), source dimension (box) which will
!                 be 1st target dimension (boxref, boxout), default: 1
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   0: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:0
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Add.
!   The routine really just adds the boxes. That makes only sense, if the arrays contain
!   conserved quantities (and not, e.g., the velocities).
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written
!   1997-10-16 (B.F.) In case of transposition, handle momenta properly
!   1999-05-31 (B.F. Copenhagen) Keywords grid and time added
!   2000-02-07 (B.F.) Keyword bulk added
!   2001-01-21 (B.F. Uppsala) Single bulk copy loops for each quantity
!   2001-01-29 (B.F.) Copy entire 3D arrays en block
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2007-02-08 (B.F. Lyon) Adapted from rhd_box_Copy, no grav. potential or magnetic fields
!   2008-01-05 (B.F.) Adapted from addchange_box
!   2010-04-19 (B.F.) Rename add_box -> rhd_box_Add
!   2011-08-08 (B.F.) Transpose e.g. transpose(v2) into v1 and not just transpose(v1)
!   2013-03-05 (B.F.) Add directives to help vectorization
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1out, m2out, m3out
integer,         optional, intent(in)   :: trans_mode, grid_mode, time_mode, bulk_mode
!
! --- Local variables ---
integer                                 :: trans_mode0, grid_mode0, time_mode0, bulk_mode0
integer                                 :: n1out, n2out, n3out
integer                                 :: i1, i2, i3, i2out, i3out, iquc
!---------------------------------------------------------------------------------------------------
!
! --- Default values ---
if (present(trans_mode)) then
  trans_mode0=trans_mode
else
  trans_mode0=1
endif
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=0
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
!
if (time_mode0 > 0) then
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif
!
if (trans_mode0 == 1) then
  ! --- Copy directly ------------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n1-m1
  n2out=m2out + n2-m2
  n3out=m3out + n3-m3
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    !DIR$ IVDEP
    boxout%rho(   m1out:n1out,m2out:n2out,m3out:n3out)= &
       boxout%rho(m1out:n1out,m2out:n2out,m3out:n3out) + &
       box%rho(   m1   :n1   ,m2   :n2   ,m3   :n3   )
    !DIR$ IVDEP
    boxout%ei(    m1out:n1out,m2out:n2out,m3out:n3out)= &
       boxout%ei( m1out:n1out,m2out:n2out,m3out:n3out) + &
       box%ei(    m1   :n1   ,m2   :n2   ,m3   :n3   )
    !DIR$ IVDEP
    boxout%v1(    m1out:n1out,m2out:n2out,m3out:n3out)= &
       boxout%v1( m1out:n1out,m2out:n2out,m3out:n3out) + &
       box%v1(    m1   :n1   ,m2   :n2   ,m3   :n3   )
    !DIR$ IVDEP
    boxout%v2(    m1out:n1out,m2out:n2out,m3out:n3out)= &
       boxout%v2( m1out:n1out,m2out:n2out,m3out:n3out) + &
       box%v2(    m1   :n1   ,m2   :n2   ,m3   :n3   )
    !DIR$ IVDEP
    boxout%v3(    m1out:n1out,m2out:n2out,m3out:n3out)= &
       boxout%v3( m1out:n1out,m2out:n2out,m3out:n3out) + &
       box%v3(    m1   :n1   ,m2   :n2   ,m3   :n3   )
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        !DIR$ IVDEP
        boxout%quc(   m1out:n1out,m2out:n2out,m3out:n3out,iquc)= &
          boxout%quc( m1out:n1out,m2out:n2out,m3out:n3out,iquc) + &
          box%quc(    m1   :n1   ,m2   :n2   ,m3   :n3   ,iquc)
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
  !
else if (trans_mode0 == 2) then
  ! --- Transpose 2 -> 1 ---------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n2-m2
  n2out=m2out + n1-m1
  n3out=m3out + n3-m3
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb1(m1:n1+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb3(m3:n3+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    i3=m3
    do i3out=m3out,n3out
      !DIR$ IVDEP
      boxout%rho(         m1out:n1out,m2out:n2out,i3out)= &
        boxout%rho(       m1out:n1out,m2out:n2out,i3out) + &
        transpose(box%rho(m1   :n1   ,m2   :n2   ,i3   ))
      i3=i3+1
    end do ! i3out
    i3=m3
    do i3out=m3out,n3out
      !DIR$ IVDEP
      boxout%ei(         m1out:n1out,m2out:n2out,i3out)= &
        boxout%ei(       m1out:n1out,m2out:n2out,i3out) + &
        transpose(box%ei(m1   :n1   ,m2   :n2   ,i3   ))
      i3=i3+1
    end do ! i3out
    i3=m3
    do i3out=m3out,n3out
      !DIR$ IVDEP
      boxout%v1(         m1out:n1out,m2out:n2out,i3out)= &
        boxout%v1(       m1out:n1out,m2out:n2out,i3out) + &
        transpose(box%v2(m1   :n1   ,m2   :n2   ,i3   ))
      i3=i3+1
    end do ! i3out
    i3=m3
    do i3out=m3out,n3out
      !DIR$ IVDEP
      boxout%v2(         m1out:n1out,m2out:n2out,i3out)= &
        boxout%v2(       m1out:n1out,m2out:n2out,i3out) + &
        transpose(box%v1(m1   :n1   ,m2   :n2   ,i3   ))
      i3=i3+1
    end do ! i3out
    i3=m3
    do i3out=m3out,n3out
      !DIR$ IVDEP
      boxout%v3(         m1out:n1out,m2out:n2out,i3out)= &
        boxout%v3(       m1out:n1out,m2out:n2out,i3out) + &
        transpose(box%v3(m1   :n1   ,m2   :n2   ,i3   ))
      i3=i3+1
    end do ! i3out
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i3=m3
        do i3out=m3out,n3out
          !DIR$ IVDEP
          boxout%quc(         m1out:n1out,m2out:n2out,i3out,iquc)= &
            boxout%quc(       m1out:n1out,m2out:n2out,i3out,iquc) + &
            transpose(box%quc(m1   :n1   ,m2   :n2   ,i3   ,iquc))
          i3=i3+1
        end do ! i3out
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
  !
else if (trans_mode0 == 3) then
  ! --- Transpose 3 -> 1 ---------------------------------------------------------------------------
  !
  ! --- Indices ---
  n1out=m1out + n3-m3
  n2out=m2out + n2-m2
  n3out=m3out + n1-m1
  !
  ! --- Grid ---
  if (grid_mode0 > 0) then
    !DIR$ IVDEP
    boxout%xc1(m1out:n1out  )=box%xc3(m3:n3  )
    !DIR$ IVDEP
    boxout%xc2(m2out:n2out  )=box%xc2(m2:n2  )
    !DIR$ IVDEP
    boxout%xc3(m3out:n3out  )=box%xc1(m1:n1  )
    !DIR$ IVDEP
    boxout%xb1(m1out:n1out+1)=box%xb3(m3:n3+1)
    !DIR$ IVDEP
    boxout%xb2(m2out:n2out+1)=box%xb2(m2:n2+1)
    !DIR$ IVDEP
    boxout%xb3(m3out:n3out+1)=box%xb1(m1:n1+1)
  endif
  !
  if (bulk_mode0 > 0) then
    ! --- HD quantities ---
    !
    i2=m2
    do i2out=m2out,n2out
      !DIR$ IVDEP
      boxout%rho(         m1out:n1out,i2out,m3out:n3out)= &
        boxout%rho(       m1out:n1out,i2out,m3out:n3out) + &
        transpose(box%rho(m1   :n1   ,i2   ,m3   :n3   ))
      i2=i2+1
    end do ! i2out
    i2=m2
    do i2out=m2out,n2out
      !DIR$ IVDEP
      boxout%ei(         m1out:n1out,i2out,m3out:n3out)= &
        boxout%ei(       m1out:n1out,i2out,m3out:n3out) + &
        transpose(box%ei(m1   :n1   ,i2   ,m3   :n3   ))
      i2=i2+1
    end do ! i2out
    i2=m2
    do i2out=m2out,n2out
      !DIR$ IVDEP
      boxout%v1(         m1out:n1out,i2out,m3out:n3out)= &
        boxout%v1(       m1out:n1out,i2out,m3out:n3out) + &
        transpose(box%v3(m1   :n1   ,i2   ,m3   :n3   ))
      i2=i2+1
    end do ! i2out
    i2=m2
    do i2out=m2out,n2out
      !DIR$ IVDEP
      boxout%v2(         m1out:n1out,i2out,m3out:n3out)= &
        boxout%v2(       m1out:n1out,i2out,m3out:n3out) + &
        transpose(box%v2(m1   :n1   ,i2   ,m3   :n3   ))
      i2=i2+1
    end do ! i2out
    i2=m2
    do i2out=m2out,n2out
      !DIR$ IVDEP
      boxout%v3(         m1out:n1out,i2out,m3out:n3out)= &
        boxout%v3(       m1out:n1out,i2out,m3out:n3out) + &
        transpose(box%v1(m1   :n1   ,i2   ,m3   :n3   ))
      i2=i2+1
    end do ! i2out
    !
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
      boxout%nquc=box%nquc
      do iquc=1,box%nquc
        i2=m2
        do i2out=m2out,n2out
          !DIR$ IVDEP
          boxout%quc(         m1out:n1out,i2out,m3out:n3out,iquc)= &
            boxout%quc(       m1out:n1out,i2out,m3out:n3out,iquc) + &
            transpose(box%quc(m1   :n1   ,i2   ,m3   :n3   ,iquc))
          i2=i2+1
        end do ! i2out
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  endif
endif
!
end subroutine rhd_box_Add


!----------**************---------------------------------------------------------------------------
subroutine rhd_box_Factor(boxout, m1,n1, m2,n2, m3,n3, factor)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_Factor ('rhd_box_Factor')
!
! PURPOSE:
!   Multiply contents of box with constant factor.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_Factor(boxout, m1,n1, m2,n2, m3,n3, factor)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   factor:       ('factor') real factor
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Factor.
!   The routine makes only sense, if the arrays contain
!   conserved quantities (and not e.g. the velocities).
!   No magnetic fields, yet.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_Factor(model, m1,n1, m2,n2, m3,n3, 0.5)
!
! MODIFICATION HISTORY:
!   2011-08-05 (B.F. Lyon) Written
!   2012-11-16 (B.F.) Index limits become input parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real,                      intent(in)   :: factor
!
! --- Local variables ---
integer                                 :: iquc
!---------------------------------------------------------------------------------------------------
!
boxout%rho(   m1:n1,m2:n2,m3:n3)= &
   boxout%rho(m1:n1,m2:n2,m3:n3) * factor
boxout%ei(    m1:n1,m2:n2,m3:n3)= &
   boxout%ei( m1:n1,m2:n2,m3:n3) * factor
boxout%v1(    m1:n1,m2:n2,m3:n3)= &
   boxout%v1( m1:n1,m2:n2,m3:n3) * factor
boxout%v2(    m1:n1,m2:n2,m3:n3)= &
   boxout%v2( m1:n1,m2:n2,m3:n3) * factor
boxout%v3(    m1:n1,m2:n2,m3:n3)= &
   boxout%v3( m1:n1,m2:n2,m3:n3) * factor
    !
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Additional density arrays ---
  if (ALLOASSOTED(boxout%quc)) then
    do iquc=1,boxout%nquc
      boxout%quc(   m1:n1,m2:n2,m3:n3,iquc)= &
        boxout%quc( m1:n1,m2:n2,m3:n3,iquc) * factor
    end do ! iquc
  endif
!#
# endif
!#      /* rhd_box_quc01 */
!
end subroutine rhd_box_Factor


!----------***********************------------------------------------------------------------------
subroutine rhd_box_AllAddDelta2OMP(box, m1,n1, m2,n2, m3,n3, &
                                   boxout, &
                                   eiTable_min, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AllAddDelta2OMP ('rhd_box_All_Add_Delta_2_OpenMP')
!
! PURPOSE:
!   Add change to entire box, OpenMP-parallelized if n3>m3.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AllAddDelta2OMP(box, m1,n1, m2,n2, m3,n3, &
!                                boxout, &
!                                eiTable_min, &
!                                outstr=outstr,ierr=ierr)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   eiTable_min:  ('energy_internal_Table_minimum') real with minimum allowed internal-energy value
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! OUTPUT:
!   outstr:       ('output_string') character, optional
!   ierr:         ('index_error') integer, optional error number:
!                     0: normal run, hydrodynamics step successful completed
!                  1-99: "small" error (e.g. time step limit violation)
!                 >=100: "severe" error: (currently none)
!
! ROUTINES:
!   rhd_box_AddDelta2:     Add delta of (part of) box into another
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create box and boxout with rhd_box_Make first.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-08-08 (B.F. Lyon) Written
!   2011-12-15 (B.F.) Written based on rhd_box_AddDelta
!   2012-11-16 (B.F.) Index limits become input parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
real,                      intent(in)   :: eiTable_min
integer,         optional, intent(out)  :: ierr
character(len=*),optional, intent(out)  :: outstr
!
! --- Local variables ---
integer                                 :: i3
integer                                 :: ierr0, ierr_chunk, ierr_thread
character(len=80)                       :: outstr0, outstr_chunk, outstr_thread
!---------------------------------------------------------------------------------------------------
!
ierr0  =0
outstr0=''
!
if (n2 > m2) then
  !
  !$OMP PARALLEL DEFAULT(SHARED), &
  !$OMP          PRIVATE(i3, outstr_chunk, outstr_thread, ierr_chunk, ierr_thread)
  !
  ! --- Error handling (initialization) ---
  outstr_thread=''
  ierr_thread  =0
  !
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    call rhd_box_AddDelta2(box,   m1, n1, m2, n2, i3, i3, &
                           boxout,m1,     m2,     i3    , &
                           eiTable_min, bulk_mode=1, &
                           outstr=outstr_chunk, ierr=ierr_chunk)
    !
    ! --- Error handling ---
    if (ierr_chunk > ierr_thread) then
      ierr_thread  =ierr_chunk
      outstr_thread=outstr_chunk
    endif
    !
  end do ! i3
  !$OMP END DO NOWAIT
  !
  ! --- Error handling ---
  !$OMP CRITICAL (ADD_REDUCTION)
  if (ierr_thread > ierr0) then
    ierr0  =ierr_thread
    outstr0=outstr_thread
  endif
  !$OMP END CRITICAL (ADD_REDUCTION)
  !
  !$OMP END PARALLEL
  !
else
  call rhd_box_AddDelta2(box,   m1,n1, m2,n2, m3,n3, &
                         boxout,m1,     m2,     m3        , &
                         eiTable_min, bulk_mode=1, &
                         outstr=outstr0, ierr=ierr0)

endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr  )) ierr  =ierr0
!
end subroutine rhd_box_AllAddDelta2OMP


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_AllAddOMP(box, m1,n1, m2,n2, m3,n3, &
                             boxout, time_mode, grid_mode, bulk_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AllAddOMP ('rhd_box_All_Add_OpenMP')
!
! PURPOSE:
!   Add box to another, OpenMP-parallelized if n3>m3.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AllAddOMP(box, m1,n1, m2,n2, m3,n3, &
!                          boxout, &
!                          time_mode=time_mode, grid_mode=grid_mode, bulk_mode=bulk_mode)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   0: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default: 0
!   bulk_mode:    ('bulk_mode') optional integer, (0,1), switch for copying bulk data, default: 1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Add.
!   The routine really just adds the boxes. That makes only sense, if the arrays contain
!   conserved quantities (and not e.g. the velocities).
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-19 (B.F. Kiel) Written
!   1997-10-16 (B.F.) In case of transposition, handle momenta properly
!   1999-05-31 (B.F. Copenhagen) Keywords grid and time added
!   2000-02-07 (B.F.) Keyword bulk added
!   2001-01-21 (B.F. Uppsala) Single bulk copy loops for each quantity
!   2001-01-29 (B.F.) Copy entire 3D arrays en block
!   2005-05-31 (WS&SWB) added magnetic fluxes and entropy switch
!   2007-02-08 (B.F. Lyon) Adapted from rhd_box_Copy, no grav. potential or magnetic fields
!   2008-01-05 (B.F.) Adapted from addchange_box
!   2010-04-19 (B.F.) Rename add_box -> rhd_box_Add
!   2011-08-08 (B.F.) Transpose e.g. transpose(v2) into v1 and not just transpose(v1)
!   2011-12-12 (B.F.) Taken from rhd_box_Add, transposition removed
!   2012-11-16 (B.F.) Index limits become input parameters
!   2013-03-05 (B.F.) Add directives to help vectorization
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
integer,         optional, intent(in)   :: grid_mode, time_mode, bulk_mode
!
! --- Local variables ---
integer                                 :: grid_mode0, time_mode0, bulk_mode0
integer                                 :: i1, i2, i3, iquc
!---------------------------------------------------------------------------------------------------
!
! --- Default values ---
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=0
endif
if (present(bulk_mode)) then
  bulk_mode0=bulk_mode
else
  bulk_mode0=1
endif
!
if (time_mode0 > 0) then
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif ! (time_mode0 > 0)
!
! --- Grid ---
if (grid_mode0 > 0) then
  !DIR$ IVDEP
  boxout%xc1=box%xc1
  !DIR$ IVDEP
  boxout%xc2=box%xc2
  !DIR$ IVDEP
  boxout%xc3=box%xc3
  !DIR$ IVDEP
  boxout%xb1=box%xb1
  !DIR$ IVDEP
  boxout%xb2=box%xb2
  !DIR$ IVDEP
  boxout%xb3=box%xb3
endif ! (grid_mode0 > 0)
!
if (bulk_mode0 > 0) then
  ! --- HD quantities ---
  !$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3)
  !
  ! --- Bulk HD quantities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        !
        boxout%rho(   i1,i2,i3)=boxout%rho(i1,i2,i3) + &
                                box%rho(   i1,i2,i3)
        boxout%ei(    i1,i2,i3)=boxout%ei( i1,i2,i3) + &
                                box%ei(    i1,i2,i3)
        boxout%v1(    i1,i2,i3)=boxout%v1( i1,i2,i3) + &
                                box%v1(    i1,i2,i3)
        boxout%v2(    i1,i2,i3)=boxout%v2( i1,i2,i3) + &
                                box%v2(    i1,i2,i3)
        boxout%v3(    i1,i2,i3)=boxout%v3( i1,i2,i3) + &
                                box%v3(    i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Additional density arrays ---
  if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do iquc=1,box%nquc
        do i2=m2,n2
          !DIR$ IVDEP
          do i1=m1,n1
            !
            boxout%quc(   i1,i2,i3,iquc)=boxout%quc(i1,i2,i3,iquc) + &
                                         box%quc(   i1,i2,i3,iquc)
          end do ! i1
        end do ! i2
      end do ! iquc
    end do ! i3
    !$OMP END DO
    !
    !$OMP MASTER
    boxout%nquc=box%nquc
    !$OMP END MASTER
  endif
!#
# endif
!#      /* rhd_box_quc01 */
  !
  !$OMP END PARALLEL
endif ! (bulk_mode0 > 0)
!
end subroutine rhd_box_AllAddOMP


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_AllAvgOMP(box0, box1, mu)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AllAvgOMP ('rhd_box_All_Average_OpenMP')
!
! PURPOSE:
!   Average two boxes.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AllAvgOMP(box0, box1, mu)
!
! INPUT:
!   box0:         ('box_0') type(box_type), input computational box
!   mu:           ('mu') real, weight factor
!
! INPUT/OUTPUT:
!   box1:         ('box_1') type(box_type), input/output computational box
!
! ROUTINES:
!   rhd_box_AllCopyOMP:    Copy entire box, OpenMP-parallelized if box is 3D
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   mu should be in the range [0.0,1.0].
!   box0 and box1 should have arrays with identical dimension.
!   box1%itime1 is not modified.
!
! PROCEDURE:
!   "box1 = mu * box1 + (1-mu) * box0"
!
! EXAMPLE:
!
! TODO:
!   itime?
!   Check energy update for MHD case.
!   Use q2=q0 + mu*(q1-q0) instead of q2=mu*q1 + (1-mu)*q0.
!   e_2=e_0 + &
!       mu*(rho_1/rho_2)*(e_1-e_0) + &
!       0.5*(mu-mu^2)*(rho_0/rho_2)*(rho_1/rho_2)*(v_1-v_0)^2
!
! MODIFICATION HISTORY:
!   2010-06-03 (B.F. Freiburg) Written
!   2011-02-25 (W.S. Exeter) Pure hydro case, dual-energy method, handling of additional density
!                            arrays
!   2013-03-05 (B.F. Lyon) Add directives to help vectorization
!   2014-09-22 (B.F. Uppsala) Average rho with drho to improve mass conservation
!   2017-07-21 (B.F.) Make mu non-optional; add explicit real conversions
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box0
type(box_type),            intent(inout):: box1
real,                      intent(in)   :: mu
!
! --- Local variables ---
integer                                 :: i1, i2, i3, iquc, m1,n1, m2,n2, m3,n3
real                                    :: rho_2, v1_2, v2_2, v3_2, ei_2, drho
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=box0%m1
n1=box0%n1
m2=box0%m2
n2=box0%n2
m3=box0%m3
n3=box0%n3
!
if (mu == 0.0) then
  ! --- Copy first model ---
  !
  call rhd_box_AllCopyOMP(box0, box1, flux_mode=0)
  !
else if (mu < 1.0) then
  ! --- Average (interpolate between) models ---
  !
  ! --- Time ---
  box1%time_db=box0%time_db + real(mu, kind=kind(0.0D+00))*(box1%time_db-box0%time_db)
  box1%time   =real(box1%time_db, kind=kind(0.0E+00))
  !
  !$OMP PARALLEL DEFAULT(SHARED) &
  !$OMP          PRIVATE(i1, i2, iquc, rho_2, v1_2, v2_2, v3_2, ei_2, drho)
  !
  ! --- Bulk HD quantities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        ! rho_2=(1.0-mu)*box0%rho(i1,i2,i3) + &
        !            mu *box1%rho(i1,i2,i3)
        drho =mu*(box1%rho(i1,i2,i3)-box0%rho(i1,i2,i3) )
        rho_2=box0%rho(i1,i2,i3)+drho
        !
        v1_2 =((1.0-mu)*box0%rho(i1,i2,i3)*box0%v1(i1,i2,i3) + &
                    mu *box1%rho(i1,i2,i3)*box1%v1(i1,i2,i3)  )/rho_2
        v2_2 =((1.0-mu)*box0%rho(i1,i2,i3)*box0%v2(i1,i2,i3) + &
                    mu *box1%rho(i1,i2,i3)*box1%v2(i1,i2,i3)  )/rho_2
        v3_2 =((1.0-mu)*box0%rho(i1,i2,i3)*box0%v3(i1,i2,i3) + &
                    mu *box1%rho(i1,i2,i3)*box1%v3(i1,i2,i3)  )/rho_2
        !
!#
#   if (rhd_box_bmag01 == 1)
!#
        if (ALLOASSOTED(box0%energy_switch) .and. ALLOASSOTED(box1%energy_switch)) then
          if (box1%energy_switch(i1,i2,i3)) then
            ei_2 =((1.0-mu)*  (     box0%rho(i1,i2,i3)* box0%ei(i1,i2,i3) + &
                               0.5 *box0%rho(i1,i2,i3)*(box0%v1(i1,i2,i3)**2+ &
                                                        box0%v2(i1,i2,i3)**2+ &
                                                        box0%v3(i1,i2,i3)**2)) + &
                        mu *  (     box1%rho(i1,i2,i3)* box1%ei(i1,i2,i3) + &
                               0.5 *box1%rho(i1,i2,i3)*(box1%v1(i1,i2,i3)**2+ &
                                                        box1%v2(i1,i2,i3)**2+ &
                                                        box1%v3(i1,i2,i3)**2)) - &
                   0.5 *rho_2*(v1_2**2+v2_2**2+v3_2**2) &
                  +(mu-mu**2)*0.125*(((box1%Bb1(i1+1,i2  ,i3  )+box1%Bb1(i1,i2,i3))- &
                                      (box0%Bb1(i1+1,i2  ,i3  )+box0%Bb1(i1,i2,i3)))**2 + &
                                     ((box1%Bb2(i1  ,i2+1,i3  )+box1%Bb2(i1,i2,i3))- &
                                      (box0%Bb2(i1  ,i2+1,i3  )+box0%Bb2(i1,i2,i3)))**2 + &
                                     ((box1%Bb3(i1  ,i2  ,i3+1)+box1%Bb3(i1,i2,i3))- &
                                      (box0%Bb3(i1  ,i2  ,i3+1)+box0%Bb3(i1,i2,i3)))**2) &
                  )/rho_2
          else
            ei_2 =((1.0-mu)*box0%rho(i1,i2,i3)*box0%ei(i1,i2,i3) + &
                        mu *box1%rho(i1,i2,i3)*box1%ei(i1,i2,i3)  )/rho_2
          endif
        else
          ! --- ToDo: Why no magnetic fields here? ---
          ei_2 =((1.0-mu)*  (     box0%rho(i1,i2,i3)* box0%ei(i1,i2,i3) + &
                             0.5 *box0%rho(i1,i2,i3)*(box0%v1(i1,i2,i3)**2+ &
                                                      box0%v2(i1,i2,i3)**2+ &
                                                      box0%v3(i1,i2,i3)**2)) + &
                      mu *  (     box1%rho(i1,i2,i3)* box1%ei(i1,i2,i3) + &
                             0.5 *box1%rho(i1,i2,i3)*(box1%v1(i1,i2,i3)**2+ &
                                                      box1%v2(i1,i2,i3)**2+ &
                                                      box1%v3(i1,i2,i3)**2)) - &
                 0.5 *rho_2*(v1_2**2+v2_2**2+v3_2**2) &
                )/rho_2
        endif
!#
#   else
!#       /* rhd_box_bmag01: no magnetic fields */
        ! ei_2 =((1.0-mu)*  (     box0%rho(i1,i2,i3)* box0%ei(i1,i2,i3) + &
        !                   0.5 *box0%rho(i1,i2,i3)*(box0%v1(i1,i2,i3)**2+ &
        !                                            box0%v2(i1,i2,i3)**2+ &
        !                                            box0%v3(i1,i2,i3)**2)) + &
        !            mu *  (     box1%rho(i1,i2,i3)* box1%ei(i1,i2,i3) + &
        !                   0.5 *box1%rho(i1,i2,i3)*(box1%v1(i1,i2,i3)**2+ &
        !                                            box1%v2(i1,i2,i3)**2+ &
        !                                            box1%v3(i1,i2,i3)**2)) - &
        !       0.5 *rho_2*(v1_2**2+v2_2**2+v3_2**2) &
        !      )/rho_2
        ! ei_2=box0%ei(i1,i2,i3) + &
        !      mu        *     (box1%rho(i1,i2,i3)/rho_2) * (box1%ei(i1,i2,i3)-box0%ei(i1,i2,i3)) +&
        !      (mu-mu**2)*0.5* (box0%rho(i1,i2,i3)/rho_2) * &
        !                      (box1%rho(i1,i2,i3)/rho_2) * &
        !                     ((box1%v1(i1,i2,i3)-box0%v1(i1,i2,i3))**2 + &
        !                      (box1%v2(i1,i2,i3)-box0%v2(i1,i2,i3))**2 + &
        !                      (box1%v3(i1,i2,i3)-box0%v3(i1,i2,i3))**2    )
        ei_2=box0%ei(i1,i2,i3) + &
             mu * (box1%rho(i1,i2,i3)/rho_2) * &
                  ((box1%ei(i1,i2,i3)-box0%ei(i1,i2,i3)) + &
                   (1.0-mu)*0.5* (box0%rho(i1,i2,i3)/rho_2) * &
                                ((box1%v1(i1,i2,i3)-box0%v1(i1,i2,i3))**2 + &
                                 (box1%v2(i1,i2,i3)-box0%v2(i1,i2,i3))**2 + &
                                 (box1%v3(i1,i2,i3)-box0%v3(i1,i2,i3))**2    ))
!#
#   endif
!#        /* rhd_box_bmag01 */
        !
        box1%rho(i1,i2,i3)=rho_2
        box1%v1(i1,i2,i3) =v1_2
        box1%v2(i1,i2,i3) =v2_2
        box1%v3(i1,i2,i3) =v3_2
        box1%ei(i1,i2,i3) =ei_2
        !
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO
!#
#   if (rhd_box_bmag01 == 1)
!#
  !
  ! --- Magnetic field 1 ---
  if (ALLOASSOTED(box0%Bb1) .and. ALLOASSOTED(box1%Bb1)) then
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1+1
          box1%Bb1(i1,i2,i3)=(1.0-mu)*box0%Bb1(i1,i2,i3) + &
                                  mu *box1%Bb1(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
  endif
  !
  ! --- Magnetic field 2 ---
  if (ALLOASSOTED(box0%Bb2) .and. ALLOASSOTED(box1%Bb2)) then
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2+1
        !DIR$ IVDEP
        do i1=m1,n1
          box1%Bb2(i1,i2,i3)=(1.0-mu)*box0%Bb2(i1,i2,i3) + &
                                  mu *box1%Bb2(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
  endif
  !
  ! --- Magnetic field 3 ---
  if (ALLOASSOTED(box0%Bb3) .and. ALLOASSOTED(box1%Bb3)) then
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3+1
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1
          box1%Bb3(i1,i2,i3)=(1.0-mu)*box0%Bb3(i1,i2,i3) + &
                                  mu *box1%Bb3(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO
  endif
!#
#   endif
!#        /* rhd_box_bmag01 */
!#
#   if (rhd_box_quc01 != 0)
!#
    ! --- Additional density arrays ---
    if ((ALLOASSOTED(box0%quc)) .and. (ALLOASSOTED(box1%quc))) then
      do iquc=1,box1%nquc
        !$OMP DO SCHEDULE(RUNTIME)
        do i3=m3,n3
          do i2=m2,n2
            !DIR$ IVDEP
            do i1=m1,n1
              ! box1%quc(i1,i2,i3,iquc)=(1.0-mu)*box0%quc(i1,i2,i3,iquc) + &
              !                              mu *box1%quc(i1,i2,i3,iquc)
              box1%quc(i1,i2,i3,iquc)=box0%quc(i1,i2,i3,iquc) + &
                                      mu*(box1%quc(i1,i2,i3,iquc)-box0%quc(i1,i2,i3,iquc))
            end do ! i1
          end do ! i2
        end do ! i3
        !$OMP END DO
      end do ! iquc
    endif
!#
#   endif
!#        /* rhd_box_quc01 */
  !
  !$OMP END PARALLEL
endif
!
end subroutine rhd_box_AllAvgOMP


!----------******************-----------------------------------------------------------------------
subroutine rhd_box_AllCopyOMP(box, boxout, grid_mode, flux_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AllCopyOMP ('rhd_box_All_Copy_OpenMP')
!
! PURPOSE:
!   Copy entire box one to one.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AllCopyOMP(box, boxout, grid_mode=grid_mode, flux_mode=flux_mode)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:1
!   flux_mode:    ('flux_mode') optional integer, (0,1), switch for copying fluxes, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_AllCopyOMP.
!   New version requires that box and boxout have identical indices.
!
! PROCEDURE:
!   Copy all requested arrays (without ghost layers) in OpenMP-parallelized loops.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2006-12-14 (B.F. Lyon) Taken from rhd.F90
!   2010-04-19 (B.F.) Rename rhd_box_copyall_OMP -> rhd_box_AllCopyOMP
!   2011-02-25 (W.S. Exeter) Optional parameter flux added
!   2013-06-01 (B.F Lyon) Don't call rhd_box_Copy anymore
!   2017-09-27 (B.F. Uppsala) Check if rho, ei, v1,v2,v3 arrays exist before copying
!   2018-11-10 (B.F.) Change names of some optional parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
type(box_type),            intent(inout):: boxout
integer,         optional, intent(in)   :: grid_mode, flux_mode
!
! --- Local variables ---
integer                                 :: i1, i2, i3, m1,n1, m2,n2, m3,n3, iquc, &
                                           grid_mode0, flux_mode0
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=box%m1
n1=box%n1
m2=box%m2
n2=box%n2
m3=box%m3
n3=box%n3
!
! --- Set default values if necessary ---
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=1
endif
if (present(flux_mode)) then
  flux_mode0=flux_mode
else
  flux_mode0=1
endif
!
! --- Copy time quantities ---
boxout%itime  =box%itime
boxout%time   =box%time
boxout%time_db=box%time_db
!
if (grid_mode0 > 0) then
  ! --- Copy axes ---
  !
  !DIR$ IVDEP
  boxout%xc1(m1:n1  )=box%xc1(m1:n1  )
  !DIR$ IVDEP
  boxout%xc2(m2:n2  )=box%xc2(m2:n2  )
  !DIR$ IVDEP
  boxout%xc3(m3:n3  )=box%xc3(m3:n3  )
  !DIR$ IVDEP
  boxout%xb1(m1:n1+1)=box%xb1(m1:n1+1)
  !DIR$ IVDEP
  boxout%xb2(m2:n2+1)=box%xb2(m2:n2+1)
  !DIR$ IVDEP
  boxout%xb3(m3:n3+1)=box%xb3(m3:n3+1)
endif
!
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3, iquc)
!
if ((ALLOASSOTED(boxout%rho)) .and. (ALLOASSOTED(box%rho))) then
  ! --- Copy density ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%rho(i1,i2,i3)=box%rho(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if ((ALLOASSOTED(boxout%ei)) .and. (ALLOASSOTED(box%ei))) then
  ! --- Copy internal energy ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%ei( i1,i2,i3)=box%ei( i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if ((ALLOASSOTED(boxout%v1)) .and. (ALLOASSOTED(box%v1))) then
  ! --- Copy velocities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%v1( i1,i2,i3)=box%v1( i1,i2,i3)
        boxout%v2( i1,i2,i3)=box%v2( i1,i2,i3)
        boxout%v3( i1,i2,i3)=box%v3( i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
!#
# if (rhd_box_grav01 == 1)
!#
if ((ALLOASSOTED(boxout%phic_grav)) .and. (ALLOASSOTED(box%phic_grav))) then
  ! --- Copy gravitational potential ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%phic_grav(i1,i2,i3)=box%phic_grav(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!#
# endif
!#      /* rhd_box_grav01 */
!#
# if (rhd_box_quc01 != 0)
!#
if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
  ! --- Copy additional densities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do iquc=1,box%nquc
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1
          boxout%quc(i1,i2,i3,iquc)=box%quc(i1,i2,i3,iquc)
        end do ! i1
      end do ! i2
    end do ! iquc
  end do ! i3
  !$OMP END DO NOWAIT
endif
!#
# endif
!#      /* rhd_box_quc01 */
!#
# if (rhd_box_bmag01 == 1)
!#
if (ALLOASSOTED(box%Bb1) .and. ALLOASSOTED(boxout%Bb1)) then
  ! --- Copy magnetic field Bb1 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1+1
        boxout%Bb1(i1,i2,i3)=box%Bb1(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if (ALLOASSOTED(box%Bb2) .and. ALLOASSOTED(boxout%Bb2)) then
  ! --- Copy magnetic field Bb2 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2+1
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%Bb2(i1,i2,i3)=box%Bb2(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if (ALLOASSOTED(box%Bb3) .and. ALLOASSOTED(boxout%Bb3)) then
  ! --- Copy magnetic field Bb3 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3+1
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%Bb3(i1,i2,i3)=box%Bb3(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if ((ALLOASSOTED(boxout%Bc1)) .and. (ALLOASSOTED(box%Bc1))) then
  ! --- Copy magnetic field Bc1, Bc2, Bc3 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%Bc1(i1,i2,i3)=box%Bc1(i1,i2,i3)
        boxout%Bc2(i1,i2,i3)=box%Bc2(i1,i2,i3)
        boxout%Bc3(i1,i2,i3)=box%Bc3(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !
  ! --- Copy energy switch ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%energy_switch(i1,i2,i3)=box%energy_switch(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !
  if (flux_mode0 > 0) then
    ! --- Copy magnetic field fluxes 1 ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1+1
          boxout%flux_B21(i1,i2,i3)=box%flux_B21(i1,i2,i3)
          boxout%flux_B31(i1,i2,i3)=box%flux_B31(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    ! --- Copy magnetic field fluxes 2 ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2+1
        !DIR$ IVDEP
        do i1=m1,n1
          boxout%flux_B12(i1,i2,i3)=box%flux_B12(i1,i2,i3)
          boxout%flux_B32(i1,i2,i3)=box%flux_B32(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    ! --- Copy magnetic field fluxes 3 ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3+1
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1
          boxout%flux_B13(i1,i2,i3)=box%flux_B13(i1,i2,i3)
          boxout%flux_B23(i1,i2,i3)=box%flux_B23(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  endif ! (flux0 > 0)
endif ! "exist(box%Bc1)"
!#
# endif
!#      /* rhd_box_bmag01 */
!
!$OMP END PARALLEL
!
end subroutine rhd_box_AllCopyOMP


!----------***************--------------------------------------------------------------------------
subroutine rhd_box_CopyOMP(box, m1,n1, m2,n2, m3,n3, &
                           boxout, &
                           time_mode, grid_mode, flux_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_CopyOMP ('rhd_box_Copy_OpenMP')
!
! PURPOSE:
!   Copy (part of) box one into another, OpenMP-parallelized if n3>m3.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_CopyOMP(box, m1,n1, m2,n2, m3,n3, &
!                        boxout, &
!                        time_mode=time_mode, grid_mode=grid_mode, flux_mode=flux_mode)
!
! INPUT:
!   box:          ('box') type(box_type), input computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   time_mode:    ('time_mode') optional integer, (0,1), switch for copying of time information,
!                   1: default
!   grid_mode:    ('grid_mode') optional integer, (0,1), switch for copying axes, default:1
!   flux_mode:    ('flux_mode') optional integer, (0,1), switch for copying fluxes, default:1
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_CopyOMP.
!   There is no check if the input dimensions are valid for input or output box.
!
! PROCEDURE:
!   Copy all requested arrays (without ghost layers) in OpenMP-parallelized loops.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2006-12-14 (B.F. Lyon) Taken from rhd.F90
!   2010-04-19 (B.F.) Rename rhd_box_copyall_OMP -> rhd_box_AllCopyOMP
!   2011-02-25 (W.S. Exeter) Optional parameter flux added
!   2013-06-01 (B.F Lyon) Don't call rhd_box_Copy anymore
!   2017-09-27 (B.F. Uppsala) Check if rho, ei, v1,v2,v3 arrays exist before copying
!   2018-11-10 (B.F.) Change names of some optional parameters
!   2017-11-11 (B.F.) Adapt from rhd_box_AllCopyOMP
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
type(box_type),            intent(inout):: boxout
integer,         optional, intent(in)   :: time_mode, grid_mode, flux_mode
!
! --- Local variables ---
integer                                 :: i1, i2, i3, iquc, &
                                           time_mode0, grid_mode0, flux_mode0
!---------------------------------------------------------------------------------------------------
!
! --- Set default values if necessary ---
if (present(time_mode)) then
  time_mode0=time_mode
else
  time_mode0=0
endif
if (present(grid_mode)) then
  grid_mode0=grid_mode
else
  grid_mode0=1
endif
if (present(flux_mode)) then
  flux_mode0=flux_mode
else
  flux_mode0=1
endif
!
if (time_mode0 > 0) then
  ! --- Copy time quantities ---
  boxout%itime  =box%itime
  boxout%time   =box%time
  boxout%time_db=box%time_db
endif
!
if (grid_mode0 > 0) then
  ! --- Copy grid ---
  !DIR$ IVDEP
  boxout%xc1(m1:n1  )=box%xc1(m1:n1  )
  !DIR$ IVDEP
  boxout%xc2(m2:n2  )=box%xc2(m2:n2  )
  !DIR$ IVDEP
  boxout%xc3(m3:n3  )=box%xc3(m3:n3  )
  !DIR$ IVDEP
  boxout%xb1(m1:n1+1)=box%xb1(m1:n1+1)
  !DIR$ IVDEP
  boxout%xb2(m2:n2+1)=box%xb2(m2:n2+1)
  !DIR$ IVDEP
  boxout%xb3(m3:n3+1)=box%xb3(m3:n3+1)
endif
!
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3, iquc)
!
if ((ALLOASSOTED(boxout%rho)) .and. (ALLOASSOTED(box%rho))) then
  ! --- Copy density ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%rho(i1,i2,i3)=box%rho(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if ((ALLOASSOTED(boxout%ei)) .and. (ALLOASSOTED(box%ei))) then
  ! --- Copy internal energy ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%ei( i1,i2,i3)=box%ei( i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if ((ALLOASSOTED(boxout%v1)) .and. (ALLOASSOTED(box%v1))) then
  ! --- Copy velocities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%v1( i1,i2,i3)=box%v1( i1,i2,i3)
        boxout%v2( i1,i2,i3)=box%v2( i1,i2,i3)
        boxout%v3( i1,i2,i3)=box%v3( i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
!#
# if (rhd_box_grav01 == 1)
!#
if ((ALLOASSOTED(boxout%phic_grav)) .and. (ALLOASSOTED(box%phic_grav))) then
  ! --- Copy gravitational potential ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%phic_grav(i1,i2,i3)=box%phic_grav(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!#
# endif
!#      /* rhd_box_grav01 */
!#
# if (rhd_box_quc01 != 0)
!#
if ((ALLOASSOTED(boxout%quc)) .and. (ALLOASSOTED(box%quc))) then
  ! --- Copy additional densities ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do iquc=1,box%nquc
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1
          boxout%quc(i1,i2,i3,iquc)=box%quc(i1,i2,i3,iquc)
        end do ! i1
      end do ! i2
    end do ! iquc
  end do ! i3
  !$OMP END DO NOWAIT
endif
!#
# endif
!#      /* rhd_box_quc01 */
!#
# if (rhd_box_bmag01 == 1)
!#
if (ALLOASSOTED(box%Bb1) .and. ALLOASSOTED(boxout%Bb1)) then
  ! --- Copy magnetic field Bb1 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1+1
        boxout%Bb1(i1,i2,i3)=box%Bb1(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if (ALLOASSOTED(box%Bb2) .and. ALLOASSOTED(boxout%Bb2)) then
  ! --- Copy magnetic field Bb2 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2+1
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%Bb2(i1,i2,i3)=box%Bb2(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if (ALLOASSOTED(box%Bb3) .and. ALLOASSOTED(boxout%Bb3)) then
  ! --- Copy magnetic field Bb3 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3+1
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%Bb3(i1,i2,i3)=box%Bb3(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
endif
!
if ((ALLOASSOTED(boxout%Bc1)) .and. (ALLOASSOTED(box%Bc1))) then
  ! --- Copy magnetic field Bc1, Bc2, Bc3 ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%Bc1(i1,i2,i3)=box%Bc1(i1,i2,i3)
        boxout%Bc2(i1,i2,i3)=box%Bc2(i1,i2,i3)
        boxout%Bc3(i1,i2,i3)=box%Bc3(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !
  ! --- Copy energy switch ---
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do i2=m2,n2
      !DIR$ IVDEP
      do i1=m1,n1
        boxout%energy_switch(i1,i2,i3)=box%energy_switch(i1,i2,i3)
      end do ! i1
    end do ! i2
  end do ! i3
  !$OMP END DO NOWAIT
  !
  if (flux_mode0 > 0) then
    ! --- Copy magnetic field fluxes 1 ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1+1
          boxout%flux_B21(i1,i2,i3)=box%flux_B21(i1,i2,i3)
          boxout%flux_B31(i1,i2,i3)=box%flux_B31(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    ! --- Copy magnetic field fluxes 2 ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3
      do i2=m2,n2+1
        !DIR$ IVDEP
        do i1=m1,n1
          boxout%flux_B12(i1,i2,i3)=box%flux_B12(i1,i2,i3)
          boxout%flux_B32(i1,i2,i3)=box%flux_B32(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
    !
    ! --- Copy magnetic field fluxes 3 ---
    !$OMP DO SCHEDULE(RUNTIME)
    do i3=m3,n3+1
      do i2=m2,n2
        !DIR$ IVDEP
        do i1=m1,n1
          boxout%flux_B13(i1,i2,i3)=box%flux_B13(i1,i2,i3)
          boxout%flux_B23(i1,i2,i3)=box%flux_B23(i1,i2,i3)
        end do ! i1
      end do ! i2
    end do ! i3
    !$OMP END DO NOWAIT
  endif ! (flux0 > 0)
endif ! "exist(box%Bc1)"
!#
# endif
!#      /* rhd_box_bmag01 */
!
!$OMP END PARALLEL
!
end subroutine rhd_box_CopyOMP


!----------********************---------------------------------------------------------------------
subroutine rhd_box_AllFactorOMP(boxout, m1,n1, m2,n2, m3,n3, factor)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_AllFactorOMP ('rhd_box_All_Factor_OpenMP')
!
! PURPOSE:
!   Multiply contents of box with constant factor, OpenMP-parallelized if n3>m3.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_AllFactorOMP(boxout, m1,n1, m2,n2, m3,n3, factor)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   factor:       ('factor') real factor
!
! INPUT/OUTPUT:
!   boxout:       ('box_out') type(box_type), changed computational box
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Create boxout with rhd_box_Make before call of rhd_box_Factor.
!   The routine makes only sense, if the arrays contain
!   conserved quantities (and not, e.g., the velocities).
!   No magnetic fields, yet.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_AllFactorOMP(model, m1,n1, m2,n2, m3,n3, 0.5)
!
! MODIFICATION HISTORY:
!   2011-12-12 (B.F. Lyon) Written based on rhd_box_Factor
!   2012-11-16 (B.F.) Index limits become input parameters
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(inout):: boxout
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
real,                      intent(in)   :: factor
!
! --- Local variables ---
integer                                 :: i1,i2,i3, iquc
!---------------------------------------------------------------------------------------------------
!
!$OMP PARALLEL DEFAULT(SHARED), PRIVATE(i1, i2, i3)
!
! --- Bulk HD quantities ---
!$OMP DO SCHEDULE(RUNTIME)
do i3=m3,n3
  do i2=m2,n2
    do i1=m1,n1
      boxout%rho(i1,i2,i3)=boxout%rho(i1,i2,i3) * factor
      boxout%ei( i1,i2,i3)=boxout%ei( i1,i2,i3) * factor
      boxout%v1( i1,i2,i3)=boxout%v1( i1,i2,i3) * factor
      boxout%v2( i1,i2,i3)=boxout%v2( i1,i2,i3) * factor
      boxout%v3( i1,i2,i3)=boxout%v3( i1,i2,i3) * factor
    end do ! i1
  end do ! i2
end do ! i3
!$OMP END DO NOWAIT
!
!#
# if (rhd_box_quc01 != 0)
!#
! --- Additional density arrays ---
if (ALLOASSOTED(boxout%quc)) then
  !$OMP DO SCHEDULE(RUNTIME)
  do i3=m3,n3
    do iquc=1,boxout%nquc
      do i2=m2,n2
        do i1=m1,n1
          boxout%quc(i1,i2,i3,iquc)=boxout%quc(i1,i2,i3,iquc) * factor
        end do ! i1
      end do ! i2
    end do ! iquc
  end do ! i3
  !$OMP END DO
endif
!#
# endif
!#      /* rhd_box_quc01 */
!
!$OMP END PARALLEL
!
end subroutine rhd_box_AllFactorOMP


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_PrintStat(box, nc, prefix)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_PrintStat ('rhd_box_Print_Statistics')
!
! PURPOSE:
!   Write statistics about box to specified channel.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_PrintStat(box, nc, prefix=prefix)
!
! INPUT:
!   box:          ('box') type(box_type), computational box
!   nc:           ('number_channel') integer with output channel number, e.g. 6
!   prefix:       ('prefix') optional character with prefix string, default=''
!
! OUTPUT:
!
! ROUTINES:
!   flush:                 Flush output buffer
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write output to specified channel.
!
! RESTRICTIONS:
!   The input box has to be initialized.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_PrintStat(model, 6)
!
! MODIFICATION HISTORY:
!   2010-03-16 (B.F. Naples) First version
!   2010-04-19 (B.F. Lyon) Rename printstat_box -> rhd_box_PrintStat
!   2017-08-03 (B.F. Uppsala) Change output message for not allocated or associated arrays
!   2019-03-07 (B.F.) Check additional density arrays
!   2019-03-08 (B.F.) Add parameter prefix
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: nc
character(len=*),optional, intent(in)   :: prefix
!
! --- Local variables ---
character(len=60)                       :: prf
!#
# if (rhd_box_arrays01 == 0)
!#
character(len=*), parameter             :: AlloStr='associated'
!#
# else
!#
character(len=*), parameter             :: AlloStr='allocated'
!#
# endif
!#
!---------------------------------------------------------------------------------------------------
!
! --- Set default value for optional input parameter ---
if (present(prefix)) then
  prf=prefix
else
  prf=''
endif
!
write(nc,'(A,3I6,2X,3I6)'     ) trim(prf)//'m1,2,3, n1,2,3: ', &
                                                      box%m1, box%m2, box%m3, box%n1, box%n2, box%n3
write(nc,'(A,3I6)'            ) trim(prf)//'nghost1,2,3:    ', box%nghost1, box%nghost2, box%nghost3
if (ALLOASSOTED(box%xc1)) then
  write(nc,'(A,I6,14X,I6)'    ) trim(prf)//'xc1:            ', lbound(box%xc1), ubound(box%xc1)
else
  write(nc,'(A,A)'            ) trim(prf)//'xc1:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%xc2)) then
  write(nc,'(A,6X,I6,14X,I6)' ) trim(prf)//'xc2:            ', lbound(box%xc2), ubound(box%xc2)
else
  write(nc,'(A,A)'            ) trim(prf)//'xc2:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%xc3)) then
  write(nc,'(A,12X,I6,14X,I6)') trim(prf)//'xc3:            ', lbound(box%xc3), ubound(box%xc3)
else
  write(nc,'(A,A)'            ) trim(prf)//'xc3:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%xb1)) then
  write(nc,'(A,I6,14X,I6)'    ) trim(prf)//'xb1:            ', lbound(box%xb1), ubound(box%xb1)
else
  write(nc,'(A,A)'            ) trim(prf)//'xb1:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%xb2)) then
  write(nc,'(A,6X,I6,14X,I6)' ) trim(prf)//'xb2:            ', lbound(box%xb2), ubound(box%xb2)
else
  write(nc,'(A,A)'            ) trim(prf)//'xb2:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%xb3)) then
  write(nc,'(A,12X,I6,14X,I6)') trim(prf)//'xb3:            ', lbound(box%xb3), ubound(box%xb3)
else
  write(nc,'(A,A)'            ) trim(prf)//'xb3:            ', 'not '//(AlloStr)
endif
!
if (ALLOASSOTED(box%rho)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'rho:            ', lbound(box%rho), ubound(box%rho)
else
  write(nc,'(A,A)'            ) trim(prf)//'rho:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%ei)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'ei:             ', lbound(box%ei), ubound(box%ei)
else
  write(nc,'(A,A)'            ) trim(prf)//'ei:             ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%v1)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'v1:             ', lbound(box%v1), ubound(box%v1)
else
  write(nc,'(A,A)'            ) trim(prf)//'v1:             ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%v2)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'v2:             ', lbound(box%v2), ubound(box%v2)
else
  write(nc,'(A,A)'            ) trim(prf)//'v2:             ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%v3)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'v3:             ', lbound(box%v3), ubound(box%v3)
else
  write(nc,'(A,A)'            ) trim(prf)//'v3:             ', 'not '//(AlloStr)
endif
!#
# if (rhd_box_quc01 != 0)
!#
if (ALLOASSOTED(box%quc)) then
  write(nc,'(A,3I6,I2,3I6,I2)') trim(prf)//'quc:            ', lbound(box%quc), ubound(box%quc)
else
  write(nc,'(A,A)'            ) trim(prf)//'quc:            ', 'not '//(AlloStr)
endif
!#
# endif
!#      /* rhd_box_quc01 */
!
! --- Magnetic fields ---
!#
# if (rhd_box_bmag01 == 1)
!#
if (ALLOASSOTED(box%Bb1)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'Bb1:            ', lbound(box%Bb1), ubound(box%Bb1)
else
  write(nc,'(A,A)'            ) trim(prf)//'Bb1:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%Bb2)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'Bb2:            ', lbound(box%Bb2), ubound(box%Bb2)
else
  write(nc,'(A,A)'            ) trim(prf)//'Bb2:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%Bb3)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'Bb3:            ', lbound(box%Bb3), ubound(box%Bb3)
else
  write(nc,'(A,A)'            ) trim(prf)//'Bb3:            ', 'not '//(AlloStr)
endif
!
if (ALLOASSOTED(box%Bc1)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'Bc1:            ', lbound(box%Bc1), ubound(box%Bc1)
else
  write(nc,'(A,A)'            ) trim(prf)//'Bc1:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%Bc2)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'Bc2:            ', lbound(box%Bc2), ubound(box%Bc2)
else
  write(nc,'(A,A)'            ) trim(prf)//'Bc2:            ', 'not '//(AlloStr)
endif
if (ALLOASSOTED(box%Bc3)) then
  write(nc,'(A,3I6,2X,3I6)'   ) trim(prf)//'Bc3:            ', lbound(box%Bc3), ubound(box%Bc3)
else
  write(nc,'(A,A)'            ) trim(prf)//'Bc3:            ', 'not '//(AlloStr)
endif

if (ALLOASSOTED(box%energy_switch)) then
  write(nc,'(A,3I6,2X,3I6)'   ) 'trim(prf)//energy_switch:  ', lbound(box%energy_switch), &
                                                    ubound(box%energy_switch)
else
  write(nc,'(A,A)'            ) trim(prf)//'energy_switch:  ', 'not '//(AlloStr)
endif
!#
# endif
!#      /* rhd_box_bmag01 */
!
call flush(nc)
!
end subroutine rhd_box_PrintStat


!----------*******************----------------------------------------------------------------------
subroutine rhd_box_CheckAxis1D(nc, xc1, xb1, m1, n1, nghost1, ngh1, &
                               prefixc, prefixb, OutputLevel, stop_flag, iErr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_CheckAxis1D ('rhd_box_Check_Axis_1_Dimension')
!
! PURPOSE:
!   Check step size of 1D axis.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_CheckAxis1D(nc, xc1, xb1, m1, n1, nghost1, ngh1, &
!                            prefixc, prefixb, OutputLevel, stop_flag, iErr)
!
! INPUT:
!   nc:           ('number_channel') integer with output channel number, e.g. 6
!   xc1:          ('x_center_1') real, 1. coordinate of cell centers
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of allocated ghost cells
!   ngh1:         ('number_ghost_1') integer, 1. dimension, number of ghost cells to be checked
!   prefixc:      ('prefix_center') character printed at the beginning of each line for xc1,
!                 might be empty
!   prefixb:      ('prefix_boundary') character printed at the beginning of each line for xb1,
!                 might be empty
!   OutputLevel:  ('Output_Level') integer to control the amount of output
!   stop_flag:    ('stop_flag') logical, if true an error in the axis parameters causes a stop
!                 to be transferred; ngh1<=nghost1
!
! OUTPUT:
!   iErr:         ('index_Error') integer, number of errors that occurred.
!                 iErr > 0 can only ever occur if stop_flag=.false.
!
! INPUT/OUTPUT:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   No check on the axis dimension specifications are done:
!   That is the duty of the calling routine.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_CheckAxis1D(6, xc1, xb1, m1, n1, nFg, nFg, 'xc1+g: ', 'xb1+g: ', 2, .true., ierr0) 
!
! MODIFICATION HISTORY:
!   2017-07-26 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, m1, n1, nghost1, ngh1, OutputLevel
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: xc1
real, dimension(m1-nghost1:n1+nghost1+1), &
                           intent(in)   :: xb1
character(len=*),          intent(in)   :: prefixc, prefixb
logical,                   intent(in)   :: stop_flag
integer,                   intent(out)  :: iErr
!
! --- Local variables ---
logical                                 :: Error_flag
real                                    :: xc1Left, xc1Right, xc1Min, xc1Max, &
                                           dxc1Min, dxc1Max, dxc1Rat, rdxc1Max, &
                                           xb1Left, xb1Right, xb1Min, xb1Max, &
                                           dxb1Min, dxb1Max, dxb1Rat, rdxb1Max, &
                                           dxcb1rMin, dxcb1rMax, dxbc1rMin, dxbc1rMax
real, dimension(m1-ngh1+1:n1+ngh1  )    :: dxc1
real, dimension(m1-ngh1  :n1+ngh1  )    :: dxb1, dxcb1
real, dimension(m1-ngh1+1:n1+ngh1-1)    :: rdxc1
real, dimension(m1-ngh1  :n1+ngh1-1)    :: rdxb1
real, dimension(m1-ngh1+1:n1+ngh1+1)    :: dxbc1
character(len=60)                       :: CommentStr
!---------------------------------------------------------------------------------------------------
!
! --- Initialize error counter: no error has occurred, so far ---
iErr=0
!
! --- Compute grid range: xc1 ---
xc1Left =xc1(m1-ngh1)
xc1Right=xc1(n1+ngh1)
xc1Min  =minval(xc1(m1-ngh1:n1+ngh1))
xc1Max  =maxval(xc1(n1+ngh1:n1+ngh1))
!
if (OutputLevel > 5) then
  write(nc,'(A,1PE13.6,2X,1PE13.6)') prefixc//'min,max=    ', xc1Min , xc1Max
endif
if (OutputLevel > 7) then
  write(nc,'(A,1PE13.6,2X,1PE13.6)') prefixc//'left,right= ', xc1Left, xc1Right
endif
!
if (((n1+ngh1) > (m1-ngh1)) .and. (xc1Min >= xc1Max)) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixc//'min>=max:   ', xc1Min, ' >= ', xc1Max   , ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xc-axis range'
endif
if (((n1+ngh1) > (m1-ngh1)) .and. (xc1Left >= xc1Right)) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixc//'left>=right:', xc1Left, ' >= ', xc1Right, ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xc-axis range'
endif
if (xc1Left /= xc1Min) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixc//'left/=min:  ', xc1Left, ' /= ', xc1Min  , ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xc-axis range'
endif
if (xc1Right /= xc1Max) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixc//'right/=max: ', xc1Right,' /= ', xc1Max  , ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xc-axis range'
endif
!
! --- Compute grid range: xb1 ---
xb1Left =xb1(m1-ngh1  )
xb1Right=xb1(n1+ngh1+1)
xb1Min  =minval(xb1(m1-ngh1:n1+ngh1+1))
xb1Max  =maxval(xb1(m1-ngh1:n1+ngh1+1))
!
if (OutputLevel > 5) then
  write(nc,'(A,1PE13.6,2X,1PE13.6)') prefixb//'min,max=    ', xb1Min , xb1Max
endif
if (OutputLevel > 7) then
  write(nc,'(A,1PE13.6,2X,1PE13.6)') prefixb//'left,right= ', xb1Left, xb1Right
endif
!
if (((n1+ngh1) > (m1-ngh1)) .and. (xb1Min >= xb1Max)) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixb//'min>=max:   ', xb1Min, ' >= ', xb1Max   , ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xb-axis range'
endif
if (((n1+ngh1) > (m1-ngh1)) .and. (xb1Left >= xb1Right)) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixb//'left>=right:', xb1Left, ' >= ', xb1Right, ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xb-axis range'
endif
if (xb1Left /= xb1Min) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixb//'left/=min:  ', xb1Left, ' /= ', xb1Min  , ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xb-axis range'
endif
if (xb1Right /= xb1Max) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(2(A,1PE13.6),A)'    ) prefixb//'right/=max: ', xb1Right,' /= ', xb1Max  , ', ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xb-axis range'
endif
!
if ((n1+ngh1) > (m1-ngh1)) then
  ! --- Compute grid-step size and check minimum for positivity ---
  !
  dxc1   =xc1(m1-ngh1+1:n1+ngh1  )-xc1(m1-ngh1  :n1+ngh1-1)
  dxc1Min=minval(dxc1)
  dxc1Max=maxval(dxc1)
  dxb1   =xb1(m1-ngh1+1:n1+ngh1+1)-xb1(m1-ngh1  :n1+ngh1  )
  dxb1Min=minval(dxb1)
  dxb1Max=maxval(dxb1)
  !
  if (OutputLevel > 5) then
    write(nc,'(A,1PE13.6,2X,1PE13.6)') prefixc//'dmin,dmax=  ', dxc1Min , dxc1Max
    write(nc,'(A,1PE13.6,2X,1PE13.6)') prefixb//'dmin,dmax=  ', dxb1Min , dxb1Max
  endif
  !
  if (dxc1Min <= 0.0) then
    iErr=iErr+1
    if (OutputLevel > 0) then
      write(nc,'(A,1PE13.6,A)'       ) prefixc//'dmin <= 0:  ', dxc1Min, '<= 0.0, ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xc-axis step size'
  endif
  if (dxb1Min <= 0.0) then
    iErr=iErr+1
    if (OutputLevel > 0) then
      write(nc,'(A,1PE13.6,A)'       ) prefixb//'dmin <= 0:  ', dxb1Min, '<= 0.0, ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xb-axis step size'
  endif
  !
endif
!
if ((n1+ngh1) > (m1-ngh1+1)) then
  if ((dxc1Min > 0.0) .and. (dxb1Min > 0.0) .and. (OutputLevel > 6)) then
    ! --- Compute ratio of maximum and minimum grid-step size and analyze it ---
    !
    dxc1Rat=dxc1Max/dxc1Min
    dxb1Rat=dxb1Max/dxb1Min
    !
    if (dxc1Rat > 16.0) then
      CommentStr=', WARNING: very large'
    else if (dxc1Rat > 4.0) then
      CommentStr=', FROWN: a bit large'
    else if (dxc1Rat > 1.001) then
      CommentStr=''
    else
      CommentStr=', practically equidistant'
    endif
    write(nc,'(A,1PE13.6,A)'         ) prefixc//'dmax/dmin=  ', dxc1Rat, trim(CommentStr)
    !
    if (dxb1Rat > 16.0) then
      CommentStr=', WARNING: very large'
    else if (dxb1Rat > 4.0) then
      CommentStr=', FROWN: a bit large'
    else if (dxb1Rat > 1.001) then
      CommentStr=''
    else
      CommentStr=', practically equidistant'
    endif
    write(nc,'(A,1PE13.6,A)'         ) prefixb//'dmax/dmin=  ', dxb1Rat, trim(CommentStr)
  endif
  !
endif
!
if ((n1+ngh1) > (m1-ngh1+1)) then
  if ((dxc1Min > 0.0) .and. (dxb1Min > 0.0) .and. (OutputLevel > 7)) then
    ! --- Compute maximum change in grid-step size and analyze it ---
    !
    rdxc1=dxc1(m1-ngh1+2:n1+ngh1)/dxc1(m1-ngh1+1:n1+ngh1-1)
    rdxc1Max=max(maxval(rdxc1), 1.0/minval(rdxc1))
    if (rdxc1Max > 2.0) then
      CommentStr=', WARNING: very large'
    else if (rdxc1Max > 1.4) then
      CommentStr=', WARNING: rather large'
    else if (rdxc1Max > 1.1) then
      CommentStr=', FROWN: a bit large'
    else if (rdxc1Max > 1.001) then
      CommentStr=''
    else
      CommentStr=', practically equidistant'
    endif
    write(nc,'(A,1PE13.6,A)'         ) prefixc//'d/d_max=    ', rdxc1Max, trim(CommentStr)
    !
    rdxb1=dxb1(m1-ngh1+1:n1+ngh1)/dxb1(m1-ngh1:n1+ngh1-1)
    rdxb1Max=max(maxval(rdxb1), 1.0/minval(rdxb1))
    if (rdxb1Max > 2.0) then
      CommentStr=', WARNING: very large'
    else if (rdxb1Max > 1.4) then
      CommentStr=', WARNING: rather large'
    else if (rdxb1Max > 1.1) then
      CommentStr=', FROWN: a bit large'
    else if (rdxb1Max > 1.001) then
      CommentStr=''
    else
      CommentStr=', practically equidistant'
    endif
    write(nc,'(A,1PE13.6,A)'         ) prefixb//'d/d_max=    ', rdxb1Max, trim(CommentStr)
    !
  endif
endif
!
! --- Check centering (xc1 relative to xb1 and vice versa) ---
dxb1   =xb1(m1-ngh1+1:n1+ngh1+1)-xb1(m1-ngh1  :n1+ngh1  )
dxcb1  =xc1(m1-ngh1  :n1+ngh1  )-xb1(m1-ngh1  :n1+ngh1  )
dxbc1  =xb1(m1-ngh1+1:n1+ngh1+1)-xc1(m1-ngh1  :n1+ngh1  )
dxcb1rMin=minval(dxcb1/dxb1)
dxcb1rMax=maxval(dxcb1/dxb1)
dxbc1rMin=minval(dxbc1(m1-ngh1+1:n1+ngh1)/dxc1)
dxbc1rMax=maxval(dxbc1(m1-ngh1+1:n1+ngh1)/dxc1)
!
Error_flag=.false.
if ((dxcb1rMin <= 0.0) .or. (dxcb1rMax >= 1.0)) then
  Error_flag=.true.
  CommentStr=', ERROR: xc outside dxb interval'
  iErr=iErr+1
else if ((dxcb1rMin <= 0.25  ) .or. (dxcb1rMax >= 0.75  )) then
  CommentStr=', WARNING: xc far from dxb-interval center'
else if ((dxcb1rMin <= 0.4   ) .or. (dxcb1rMax >= 0.6   )) then
  CommentStr=', FROWN: xc a bit far from dxb-interval center'
else if ((dxcb1rMin <= 0.4998) .or. (dxcb1rMax >= 0.5002)) then
  CommentStr=''
else
  CommentStr=', practically perfectly centered'
endif
if ((OutputLevel > 5) .or. ((OutputLevel > 0) .and. Error_flag)) then
  write(nc,'(A,1PE13.6,2X,1PE13.6,A)') &
                                     prefixc//'xc in dxb=  ', dxcb1rMin, dxcb1rMax, trim(CommentStr)
endif
!
if ((dxbc1rMin <= 0.0) .or. (dxbc1rMax >= 1.0)) then
  Error_flag=.true.
  CommentStr=', ERROR: xb outside dxc interval'
  iErr=iErr+1
else if ((dxbc1rMin <= 0.25  ) .or. (dxbc1rMax >= 0.75  )) then
  CommentStr=', WARNING: xb far from dxc-interval center'
else if ((dxbc1rMin <= 0.4   ) .or. (dxbc1rMax >= 0.6   )) then
  CommentStr=', FROWN: xb a bit far from dxc-interval center'
else if ((dxbc1rMin <= 0.4998) .or. (dxbc1rMax >= 0.5002)) then
  CommentStr=''
else
  CommentStr=', practically perfectly centered'
endif
if ((OutputLevel > 5) .or. ((OutputLevel > 0) .and. Error_flag)) then
  write(nc,'(A,1PE13.6,2X,1PE13.6,A)') &
                                     prefixc//'xb in dxc=  ', dxbc1rMin, dxbc1rMax, trim(CommentStr)
endif
!
if (stop_flag .and. Error_flag) stop 'rhd_box_CheckAxis1D: ERROR in centering of xc/xb points'
!
end subroutine rhd_box_CheckAxis1D


!----------*******************----------------------------------------------------------------------
subroutine rhd_box_CheckAxes2D(nc, xb1, m1, n1, nghost1, ngh1, &
                                   xb2, m2, n2, nghost2, ngh2, &
                               prefix, OutputLevel, stop_flag, iErr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_CheckAxes2D ('rhd_box_Check_Axes_2_Dimensions')
!
! PURPOSE:
!   Check aspect ratio of two axes.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_CheckAxes2D(nc, xb1, m1, n1, nghost1, ngh1, &
!                            xb2, m2, n2, nghost2, ngh2, &
!                            prefix, OutputLevel, stop_flag, iErr)
!
! INPUT:
!   nc:           ('number_channel') integer with output channel number, e.g. 6
!   xc1:          ('x_center_1') real, 1. coordinate of cell centers
!   xb1:          ('x_boundary_1') real, 1. coordinate of cell boundaries
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   ngh1:         ('number_ghost_1') integer, 1. dimension,  with number of ghost cells
!   prefixc:      ('prefix_center') character printed at the beginning of each line for xc1,
!                 might be empty
!   prefixb:      ('prefix_boundary') character printed at the beginning of each line for xb1,
!                 might be empty
!   OutputLevel:  ('Output_Level') integer to control the amount of output
!   stop_flag:    ('stop_flag') logical, if true an error in the axis parameters causes a stop
!                 to be transferred; ngh1<=nghost1
!
! OUTPUT:
!   iErr:         ('index_Error') integer, number of errors that occurred.
!                 iErr > 0 can only ever occur if stop_flag=.false.
!
! INPUT/OUTPUT:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   No check on the axis dimension specifications are done:
!   That is the duty of the calling routine.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-07-26 (B.F. Uppsala) Written
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: nc, &
                                           m1, n1, nghost1, ngh1, &
                                           m2, n2, nghost2, ngh2, OutputLevel
real, dimension(m1-nghost1:n1+nghost1+1), &
                           intent(in)   :: xb1
real, dimension(m2-nghost2:n2+nghost2+1), &
                           intent(in)   :: xb2
character(len=*),          intent(in)   :: prefix
logical,                   intent(in)   :: stop_flag
integer,                   intent(out)  :: iErr
!
! --- Local variables ---
real                                    :: dxb1Min, dxb1Max, &
                                           dxb2Min, dxb2Max, &
                                           dxb2odxb1Min, dxb2odxb1Max
real, dimension(m1-ngh1  :n1+ngh1  )    :: dxb1
real, dimension(m2-ngh2  :n2+ngh2  )    :: dxb2
character(len=60)                       :: CommentStr
!---------------------------------------------------------------------------------------------------
!
! --- Initialize error counter: no error has occurred, so far ---
iErr=0
!
! --- Compute grid-step size ---
dxb1   =xb1(m1-ngh1+1:n1+ngh1+1)-xb1(m1-ngh1  :n1+ngh1  )
dxb1Min=minval(dxb1)
dxb1Max=maxval(dxb1)
dxb2   =xb2(m2-ngh2+1:n2+ngh2+1)-xb2(m2-ngh2  :n2+ngh2  )
dxb2Min=minval(dxb2)
dxb2Max=maxval(dxb2)
!
if ((dxb1Min <= 0.0) .or. (dxb2Min <= 0.0)) then
  iErr=iErr+1
  if (OutputLevel > 0) then
    write(nc,'(A,1PE13.6,2X,1PE13.6,A)') prefix//'dmin <= 0:  ', dxb1Min, dxb2Min, '<= 0.0, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxis1D: ERROR in xb-axis step size'
  !
else
  if (OutputLevel > 3) then
    dxb2odxb1Min=dxb2Min/dxb1Max
    dxb2odxb1Max=dxb2Max/dxb1Min
    !
    if      ((dxb2odxb1Min < 1.0/8.0   ) .or. (dxb2odxb1Max > 8.0   )) then
      CommentStr=', WARNING: extreme cell aspect ratio'
    else if ((dxb2odxb1Min < 1.0/4.0   ) .or. (dxb2odxb1Max > 4.0   )) then
      CommentStr=', WARNING: large cell aspect ratio'
    else if ((dxb2odxb1Min < 1.0/2.5   ) .or. (dxb2odxb1Max > 2.5   )) then
      CommentStr=', FROWN: large cell aspect ratio'
    else if ((dxb2odxb1Min < 1.0/1.0001) .or. (dxb2odxb1Max > 1.0001)) then
      CommentStr=''
    else
      CommentStr=', practically square cells'
    endif
    !
    write(nc,'(A,1PE13.6,2X,1PE13.6,A)') prefix//'A_max,min=  ', dxb2odxb1Min, dxb2odxb1Max, &
                                         trim(CommentStr)
  endif
endif
!
end subroutine rhd_box_CheckAxes2D


!----------*****************------------------------------------------------------------------------
subroutine rhd_box_CheckAxes(box, nc, prefix, OutputLevel, stop_flag, iErr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_CheckAxes ('rhd_box_Check_Axes')
!
! PURPOSE:
!   Check axes in box and write result to specified channel.
!
! CATEGORY:
!   Box handling
!
! CALLING SEQUENCE:
!   call rhd_box_CheckAxes(box, nc, prefix, OutputLevel, stop_flag, iErr)
!
! INPUT:
!   box:          ('box') type(box_type), computational box
!   nc:           ('number_channel') integer with output channel number, e.g. 6
!   prefix:       ('prefix') character printed at the beginning of each line, might be empty
!   OutputLevel:  ('Output_Level') integer to control the amount of output
!   stop_flag:    ('stop_flag') logical, if true an error in the axis parameters causes a stop
!
! OUTPUT:
!   iErr:         ('index_Error') integer, number of errors that occurred.
!                 iErr > 0 can only ever occur if stop_flag=.false.
!
! ROUTINES:
!   flush:                 Flush output buffer
!   rhd_box_CheckAxes2D:   Check aspect ratio of two axes
!   rhd_box_CheckAxis1D:   Check step size of 1D axis
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write output to specified channel.
!
! RESTRICTIONS:
!   The input box has to be initialized.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_box_CheckAxes(model, 6, '#  0 ', 10, .true.)
!
! MODIFICATION HISTORY:
!   2017-07-25 (B.F. Uppsala) First version
!   2017-07-26 (B.F.) Check step of individual axes with subroutine
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: nc, OutputLevel
character(len=*),          intent(in)   :: prefix
logical,                   intent(in)   :: stop_flag
integer,                   intent(out)  :: iErr
!
! --- Local variables ---
integer                                 :: iErr1, iErr2, iErr3
integer, dimension(1)                   :: d
!---------------------------------------------------------------------------------------------------
!
! --- Initialize error counters: no error has occurred, so far ---
iErr =0
iErr1=0
iErr2=0
iErr3=0
!
! --- Check dimension specifications ---------------------------------------------------------------
if (OutputLevel > 4) then
  write(nc,'(A,3I6,2(2X,3I6))'  ) prefix//'m123 n123 ng123=', box%m1, box%m2, box%m3, &
                                                              box%n1, box%n2, box%n3, &
                                                              box%nghost1, box%nghost2, box%nghost3
endif
!
if (box%m1 > box%n1) then
  iErr1=iErr1+1
  if (OutputLevel > 0) then
    write(nc,'(A)'              ) prefix//'m1 > n1, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis dimensions (m1 > n1)'
endif
!
if (box%m2 > box%n2) then
  iErr2=iErr2+1
  if (OutputLevel > 0) then
    write(nc,'(A)'              ) prefix//'m2 > n2, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis dimensions (m2 > n2)'
endif
!
if (box%m3 > box%n3) then
  iErr3=iErr3+1
  if (OutputLevel > 0) then
    write(nc,'(A)'              ) prefix//'m3 > n3, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis dimensions (m3 > n3)'
endif
!
if (box%nghost1 < 0) then
  iErr1=iErr1+1
  if (OutputLevel > 0) then
    write(nc,'(A)'              ) prefix//'nghost1 < 0, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis dimensions (nghost1 < 0)'
endif
!
if (box%nghost2 < 0) then
  iErr2=iErr2+1
  if (OutputLevel > 0) then
    write(nc,'(A)'              ) prefix//'nghost2 < 0, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis dimensions (nghost2 < 0)'
endif
!
if (box%nghost3 < 0) then
  iErr3=iErr3+1
  if (OutputLevel > 0) then
    write(nc,'(A)'              ) prefix//'nghost3 < 0, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis dimensions (nghost3 < 0)'
endif
!
! --- Check association/allocation and print vector boundaries -------------------------------------
if (ALLOASSOTED(box%xc1)) then
  if (OutputLevel > 5) then
    write(nc,'(A,I6,14X,I6)' ) prefix//'xc1: l,ubound=  ', lbound(box%xc1), ubound(box%xc1)
  endif
  ! --- Check consistency of specified dimensions and boundary values ---
  d=lbound(box%xc1)
  if (d(1) /= (box%m1-box%nghost1)) then
    iErr1=iErr1+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xc1: m-ng/=lb:  ', box%m1-box%nghost1  , ' /=', &
                                                              lbound(box%xc1), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in lbound(xc1)'
  endif
  d=ubound(box%xc1)
  if (d(1) /= (box%n1+box%nghost1)) then
    iErr1=iErr1+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xc1: n+ng/=lb:  ', box%m1+box%nghost1  , ' /=', &
                                                              ubound(box%xc1), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in ubound(xc1)'
  endif
else
  iErr1=iErr1+1
  if (OutputLevel > 0) then
    write(nc,'(A,A)'            ) prefix//'xc1:            ', 'not ALLOASSOTED, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis specification'
endif
!
if (ALLOASSOTED(box%xb1)) then
  if (OutputLevel > 5) then
    write(nc,'(A,I6,14X,I6)' ) prefix//'xb1: l,ubound=  ', lbound(box%xb1), ubound(box%xb1)
  endif
  ! --- Check consistency of specified dimensions and boundary values ---
  d=lbound(box%xb1)
  if (d(1) /= (box%m1-box%nghost1)) then
    iErr1=iErr1+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xb1: m-ng/=lb:  ', box%m1-box%nghost1  , ' /=', &
                                                              lbound(box%xb1), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in lbound(xb1)'
  endif
  d=ubound(box%xb1)
  if (d(1) /= (box%n1+box%nghost1+1)) then
    iErr1=iErr1+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xb1: n+ng+1/=lb:', box%m1+box%nghost1+1, ' /=', &
                                                              ubound(box%xb1), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in ubound(xb1)'
  endif
else
  iErr1=iErr1+1
  if (OutputLevel > 0) then
    write(nc,'(A,A)'            ) prefix//'xb1:            ', 'not ALLOASSOTED, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis specification'
endif
!
if (ALLOASSOTED(box%xc2)) then
  if (OutputLevel > 5) then
    write(nc,'(A,6X,I6,14X,I6)' ) prefix//'xc2: l,ubound=  ', lbound(box%xc2), ubound(box%xc2)
  endif
  ! --- Check consistency of specified dimensions and boundary values ---
  d=lbound(box%xc2)
  if (d(1) /= (box%m2-box%nghost2)) then
    iErr2=iErr2+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xc2: m-ng/=lb:  ', box%m2-box%nghost2  , ' /=', &
                                                              lbound(box%xc2), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in lbound(xc2)'
  endif
  d=ubound(box%xc2)
  if (d(1) /= (box%n2+box%nghost2)) then
    iErr2=iErr2+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xc2: n+ng/=lb:  ', box%m2+box%nghost2  , ' /=', &
                                                              ubound(box%xc2), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in ubound(xc2)'
  endif
else
  iErr2=iErr2+1
  if (OutputLevel > 0) then
    write(nc,'(A,A)'            ) prefix//'xc2:            ', 'not ALLOASSOTED, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis specification'
endif
!
if (ALLOASSOTED(box%xb2)) then
  if (OutputLevel > 5) then
    write(nc,'(A,6X,I6,14X,I6)' ) prefix//'xb2: l,ubound=  ', lbound(box%xb2), ubound(box%xb2)
  endif
  ! --- Check consistency of specified dimensions and boundary values ---
  d=lbound(box%xb2)
  if (d(1) /= (box%m2-box%nghost2)) then
    iErr2=iErr2+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xb2: m-ng/=lb:  ', box%m2-box%nghost2  , ' /=', &
                                                              lbound(box%xb2), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in lbound(xb2)'
  endif
  d=ubound(box%xb2)
  if (d(1) /= (box%n2+box%nghost2+1)) then
    iErr2=iErr2+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xb2: n+ng+1/=lb:', box%m2+box%nghost2+1, ' /=', &
                                                              ubound(box%xb2), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in ubound(xb2)'
  endif
else
  iErr2=iErr2+1
  if (OutputLevel > 0) then
    write(nc,'(A,A)'            ) prefix//'xb2:            ', 'not ALLOASSOTED, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis specification'
endif
!
if (ALLOASSOTED(box%xc3)) then
  if (OutputLevel > 5) then
    write(nc,'(A,12X,I6,14X,I6)') prefix//'xc3: l,ubound=  ', lbound(box%xc3), ubound(box%xc3)
  endif
  ! --- Check consistency of specified dimensions and boundary values ---
  d=lbound(box%xc3)
  if (d(1) /= (box%m3-box%nghost3)) then
    iErr3=iErr3+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xc3: m-ng/=lb:  ', box%m3-box%nghost3  , ' /=', &
                                                              lbound(box%xc3), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in lbound(xc3)'
  endif
  d=ubound(box%xc3)
  if (d(1) /= (box%n3+box%nghost3)) then
    iErr3=iErr3+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xc3: n+ng/=lb:  ', box%m3+box%nghost3  , ' /=', &
                                                              ubound(box%xc3), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in ubound(xc3)'
  endif
else
  iErr3=iErr3+1
  if (OutputLevel > 0) then
    write(nc,'(A,A)'            ) prefix//'xc3:            ', 'not ALLOASSOTED, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis specification'
endif
!
if (ALLOASSOTED(box%xb3)) then
  if (OutputLevel > 5) then
    write(nc,'(A,12X,I6,14X,I6)') prefix//'xb3: l,ubound=  ', lbound(box%xb3), ubound(box%xb3)
  endif
  ! --- Check consistency of specified dimensions and boundary values ---
  d=lbound(box%xb3)
  if (d(1) /= (box%m3-box%nghost3)) then
    iErr3=iErr3+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xb3: m-ng/=lb:  ', box%m3-box%nghost3  , ' /=', &
                                                              lbound(box%xb3), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in lbound(xb3)'
  endif
  d=ubound(box%xb3)
  if (d(1) /= (box%n3+box%nghost3+1)) then
    iErr3=iErr3+1
    if (OutputLevel > 0) then
      write(nc,'(2(A,I6),A)'    ) prefix//'xb3: n+ng+1/=lb:', box%m3+box%nghost3+1, ' /=', &
                                                              ubound(box%xb3), ', ERROR'
    endif
    if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in ubound(xb3)'
  endif
else
  iErr3=iErr3+1
  if (OutputLevel > 0) then
    write(nc,'(A,A)'            ) prefix//'xb3:            ', 'not ALLOASSOTED, ERROR'
  endif
  if (stop_flag) stop 'rhd_box_CheckAxes: ERROR in axis specification'
endif
!
! --- Apply arbitrary weight factor to errors ---
iErr1=100*iErr1
iErr2=100*iErr2
iErr3=100*iErr3
!
! === Check grid-step sizes ========================================================================
!
if (iErr1 == 0) then
  if (box%nghost1 > 0) then
    ! --- Check step size of xc1/xb1 axis without and with ghost cells ---
    call rhd_box_CheckAxis1D(nc, box%xc1, box%xb1, box%m1, box%n1, box%nghost1,           0, &
                             prefix//'xc1+0: ', prefix//'xb1+0: ', OutputLevel, stop_flag, iErr)
  endif
  iErr1=iErr1+iErr
  call rhd_box_CheckAxis1D(  nc, box%xc1, box%xb1, box%m1, box%n1, box%nghost1, box%nghost1, &
                             prefix//'xc1+g: ', prefix//'xb1+g: ', OutputLevel, stop_flag, iErr)
  iErr1=iErr1+iErr
endif ! (iErr1)
!
if (iErr2 == 0) then
  if (box%nghost2 > 0) then
    ! --- Check step size of xc2/xb2 axis without and with ghost cells ---
    call rhd_box_CheckAxis1D(nc, box%xc2, box%xb2, box%m2, box%n2, box%nghost2,           0, &
                             prefix//'xc2+0: ', prefix//'xb2+0: ', OutputLevel, stop_flag, iErr)
  endif
  iErr2=iErr2+iErr
  call rhd_box_CheckAxis1D(  nc, box%xc2, box%xb2, box%m2, box%n2, box%nghost2, box%nghost2, &
                             prefix//'xc2+g: ', prefix//'xb2+g: ', OutputLevel, stop_flag, iErr)
  iErr2=iErr2+iErr
endif ! (iErr2)
!
if (iErr3 == 0) then
  if (box%nghost3 > 0) then
    ! --- Check step size of xc3/xb3 axis without and with ghost cells ---
    call rhd_box_CheckAxis1D(nc, box%xc3, box%xb3, box%m3, box%n3, box%nghost3,           0, &
                             prefix//'xc3+0: ', prefix//'xb3+0: ', OutputLevel, stop_flag, iErr)
  endif
  iErr3=iErr3+iErr
  call rhd_box_CheckAxis1D(  nc, box%xc3, box%xb3, box%m3, box%n3, box%nghost3, box%nghost3, &
                             prefix//'xc3+g: ', prefix//'xb3+g: ', OutputLevel, stop_flag, iErr)
  iErr3=iErr3+iErr
endif ! (iErr3)
!
! === Check cell aspect ratios (of axis pairs in 2D) ===============================================
if ((iErr1 == 0) .and. (iErr2 == 0)) then
  call rhd_box_CheckAxes2D(nc, box%xb1, box%m1, box%n1, box%nghost1, box%nghost1, &
                               box%xb2, box%m2, box%n2, box%nghost2, box%nghost2, &
                           prefix//'xb2/1: ', OutputLevel, stop_flag, iErr)
  iErr1=iErr1+iErr
  iErr2=iErr2+iErr
endif
!
if ((iErr1 == 0) .and. (iErr3 == 0)) then
  call rhd_box_CheckAxes2D(nc, box%xb1, box%m1, box%n1, box%nghost1, box%nghost1, &
                               box%xb3, box%m3, box%n3, box%nghost3, box%nghost3, &
                           prefix//'xb3/1: ', OutputLevel, stop_flag, iErr)
  iErr1=iErr1+iErr
  iErr3=iErr3+iErr
endif
!
if ((iErr2 == 0) .and. (iErr3 == 0)) then
  call rhd_box_CheckAxes2D(nc, box%xb2, box%m2, box%n2, box%nghost2, box%nghost2, &
                               box%xb3, box%m3, box%n3, box%nghost3, box%nghost3, &
                           prefix//'xb3/2: ', OutputLevel, stop_flag, iErr)
  iErr2=iErr2+iErr
  iErr3=iErr3+iErr
endif
!
if (OutputLevel > 0) then
  ! --- Flush the output ---
  call flush(nc)
endif
!
! --- Sum the errors (partly with arbitrary weight factor) ---
iErr=iErr1+iErr2+iErr3
!
end subroutine rhd_box_CheckAxes


!----------***************--------------------------------------------------------------------------
subroutine rhd_box_MPICopy(boxSo, geoSo, boxTa, geoTa, &
                           SubBoxDim, SubBoxMode, SubBoxName, nghost1, nghost2, nghost3, &
                           rho_mode, ei_mode, v_mode, quc_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_MPICopy ('rhd_box_MPI_Copy')
!
! PURPOSE:
!   Copy distribution of boxes from one MPI partition to another.
!
! CATEGORY:
!   Box handling, MPI
!
! CALLING SEQUENCE:
!   call rhd_box_MPICopy(boxSo, geoSo, boxTa, geoTa, &
!                        SubBoxDim=SubBoxDim, SubBoxMode=SubBoxMode, SubBoxName=SubBoxName, &
!                        nghost1=nghost1, nghost2=nghost2, nghost3=nghost3, &
!                        rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode)
!
! INPUT:
!   boxSo:        ('box_Source') type(box_type), source/input computational box
!   geoSo:        ('geometry_Source') structure with geometry data about source domain decomposition
!   geoTa:        ('geometry_Target') structure with geometry data about target domain decomposition
!   --- optional: ---
!   SubBoxDim:    ('Sub_Box_Dimension') optional 2x3 real array with output dimension
!   SubBoxMode:   ('Sub_Box_Mode') optional character with mode how to find or interpret
!                 the output dimension
!   SubBoxName:   ('Sub_box_Name') optional character with name of output if SubBoxDim is specified,
!                 e.g., 'Chunk 1'
!   nghost1:      ('number_ghost_1') integer, optional (def=0), number of ghost cells
!   nghost2:      ('number_ghost_2') integer, optional (def=0), number of ghost cells
!   nghost3:      ('number_ghost_3') integer, optional (def=0), number of ghost cells
!   rho_mode:     ('rho_mode') optional integer indicating if rho array should be included:
!                   not present: Include rho array only for bulk_mode=2
!                   0          : Don't include rho array
!                   1:         : Include rho array (default)
!   ei_mode:      ('energy_internal_mode') optional integer indicating if ei array should be
!                 included:
!                   not present: Include ei array only for bulk_mode=2
!                   0          : Don't include ei array
!                   1:         : Include ei array for (default)
!   v_mode:       ('velocity_mode') optional integer indicating if v arrays should be included:
!                   not present: Include v arrays only for bulk_mode=2
!                   0          : Don't include v arrays
!                   1:         : Include v arrays (default)
!   quc_mode:     ('quantity_centered_mode') optional integer indicating if quc arrays should be
!                 included:
!                   not present: Don't include quc arrays
!                   0          : Don't include quc arrays
!                   1:         : Include quc arrays
!
! INPUT/OUTPUT:
!   boxTa:        ('box_Target') type(box_type), target/output computational box
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in Cartesian topology from rank in group
!   MPI_Comm_Group:        Return the group associated with a communicator
!   MPI_Group_Free:        Free a group
!   MPI_Group_Translate_Ranks: Translate the ranks of processes in one group to those in another one
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_box_CopyOMP:       Copy (part of) box into another, OpenMP-parallelized if n3>m3
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_FillAxes:      Fill axes of box
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_MPICreateType: Create MPI data type for box structure
!   rhd_box_SubBoxIndices: Compute indices of box representing the section of both input boxes
!   rhd_box_ZeroOMP:       Set bulk arrays explicitely to zero
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The boxes have to be initialized.
!   Target boxes are created (if not created already) with ghost cells included,
!     but ghost-cell values are not copied.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-09-26 (B.F. Uppsala) First version
!   2018-11-08 (B.F.) Do the group translation
!   2018-11-12 (B.F.) Copy local data with OpenMP
!   2018-11-24 (B.F.) Handle axes
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
use MPI
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: boxSo
type(geo_type),            intent(in)   :: geoSo, geoTa
type(box_type),            intent(inout):: boxTa
character(len=*),optional, intent(in)   :: SubBoxMode, SubBoxName
integer, dimension(2,3), &
                 optional, intent(in)   :: SubBoxDim
integer,         optional, intent(in)   :: nghost1, nghost2, nghost3, &
                                           rho_mode, ei_mode, v_mode, quc_mode
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, &
                                           m1c,n1c, m2c,n2c, m3c,n3c, &
                                           nghost10, nghost20, nghost30
logical                                 :: IsEmpty_flag, quc_flag
!#
#   if (rhd_mpi_l01 == 1)
!#                        /* MPI activated ----------------- */
integer                                 :: ierrMPI, &
                                           iProcSo, nProcsSo, iProcTa, nProcsTa, &
                                           m1b,n1b, m2b,n2b, m3b,n3b, &
                                           rhdMPI_GROUP_WORLD, rhdMPI_GROUP_So, rhdMPI_GROUP_Ta, &
                                           iProcWoSo, iProcWoTa
logical                                 :: IsComplete_flag
integer, dimension(1)                   :: NewRanks
integer, dimension(3)                   :: PCoords
integer, dimension(1:geoSo%nProcsPD(1)* &
                     geoSo%nProcsPD(2)* &
                     geoSo%nProcsPD(3)) :: boxTaTypes, RequestsTa
integer, dimension(MPI_STATUS_SIZE, &
                   1:geoSo%nProcsPD(1)* &
                     geoSo%nProcsPD(2)* &
                     geoSo%nProcsPD(3)) :: StatusTa
integer, dimension(1:geoTa%nProcsPD(1)* &
                     geoTa%nProcsPD(2)* &
                     geoTa%nProcsPD(3)) :: boxSoTypes, RequestsSo
integer, dimension(MPI_STATUS_SIZE, &
                   1:geoTa%nProcsPD(1)* &
                     geoTa%nProcsPD(2)* &
                     geoTa%nProcsPD(3)) :: StatusSo
!#
#   endif
!#        /* rhd_mpi_l01 ----------------------------------- */
!---------------------------------------------------------------------------------------------------
!
! --- Set default values if necessary ---
if (present(nghost1)) then
  nghost10=max(nghost1, 0)
else
  nghost10=1
endif
if (present(nghost2)) then
  nghost20=max(nghost2, 0)
else
  nghost20=1
endif
if (present(nghost3)) then
  nghost30=max(nghost3, 0)
else
  nghost30=1
endif
!
if (present(quc_mode)) then
  quc_flag=(quc_mode > 0)
else
  quc_flag=.false.
endif
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
!
! --- Determine MPI group from MPI communicator ---
call MPI_Comm_Group(MPI_COMM_WORLD, rhdMPI_GROUP_WORLD, ierrMPI)
call MPI_Comm_Group(geoSo%Comm    , rhdMPI_GROUP_So   , ierrMPI)
call MPI_Comm_Group(geoTa%Comm    , rhdMPI_GROUP_Ta   , ierrMPI)
!
! === Find out what to receive from where and issue receive requests ===============================
!
! --- Compute indices of local target box ---
call MPI_Cart_Coords(geoTa%Comm, geoTa%MyProc, 3, PCoords, ierrMPI)
call rhd_geo_Coords2mn(geoTa, PCoords, m1,n1, m2,n2, m3,n3)
!m1=geoTa%m(1) ; n1=geoTa%n(1); m2=geoTa%m(2) ; n2=geoTa%n(2) ; m3=geoTa%m(3) ; n3=geoTa%n(3)
!m1=geoTa%m(1)-nghost10 ; n1=geoTa%n(1)+nghost10
!m2=geoTa%m(2)-nghost20 ; n2=geoTa%n(2)+nghost20
!m3=geoTa%m(3)-nghost20 ; n3=geoTa%n(3)+nghost30
!SubBoxDim0(1,1)=SubBoxDim(1,1)-nghost1 ; SubBoxDim0(2,1)=SubBoxDim(2,1)+nghost1
!SubBoxDim0(1,2)=SubBoxDim(1,2)-nghost2 ; SubBoxDim0(2,2)=SubBoxDim(2,2)+nghost2
!SubBoxDim0(1,3)=SubBoxDim(1,3)-nghost3 ; SubBoxDim0(2,3)=SubBoxDim(2,3)+nghost3
call rhd_box_SubBoxIndices(m1 ,n1 , m2 ,n2 , m3 ,n3 , &
                           0  ,0  , 0  ,0  , 0  ,0  , &
                           m1c,n1c, m2c,n2c, m3c,n3c, &
                           SubBoxMode=SubBoxMode, SubBoxDim=SubBoxDim, &
                           IsEmpty_flag=IsEmpty_flag)
!
!write(gl%nc_p,'(A,I3,1X,A,3(I5,1X,I5,3X),L1)') &
!                         '#', geoTa%MyProc, &
!                         merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
!                         ' target; m1c,n1c, m2c,n2c, m3c,n3c: ', &
!                                   m1c,n1c, m2c,n2c, m3c,n3c, (.not. IsEmpty_flag)
!call flush(gl%nc_p)
!
! --- Initialize the counter for relevant source processes ---
nProcsSo=0
!
if (.not. IsEmpty_flag) then
  ! --- Local target sub box is not empty: data needs to be received ---
  !
  if (.not. rhd_box_Exist(boxTa)) then
    !write(gl%nc_p,'(A,I3,1X,A,3(I5,1X,I5,3X),L1)') &
    !                     '#', geoTa%MyProc, &
    !                     merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
    !                     ' Make  ; m1 ,n1 , m2 ,n2 , m3 ,n3 : ', &
    !                               m1,n1, m2,n2, m3,n3, (.not. IsEmpty_flag)
    !call flush(gl%nc_p)
    !
    ! --- Create box of appropriate size (might be larger than SubBox) ---
    !m1=geoTa%m(1) ; n1=geoTa%n(1); m2=geoTa%m(2) ; n2=geoTa%n(2) ; m3=geoTa%m(3) ; n3=geoTa%n(3)
    call rhd_box_Make(boxTa, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                             nghost1=nghost10, nghost2=nghost20, nghost3=nghost30, &
                             rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_flag=quc_flag)
    !
    ! --- Set bulk arrays explicitly to zero trying to aid proper memory page placement ---
    call rhd_box_ZeroOMP(boxTa)
    !
    ! --- Fill the axes of the box, including MPI-internal and real boundaries ---
    call rhd_box_FillAxes(                       boxTa                              )
!    call rhd_box_MPIFillGhostAxes(action, geoTa, boxTa, nghost10, nghost20, nghost30)
!    call rhd_box_FillGhostAxes(   action,        boxTa, nghost10, nghost20, nghost30)
  endif
  !
  ! --- Loop over the source processes, skip local process ---
  do iProcSo=0, geoSo%nProcsPD(1) * geoSo%nProcsPD(2) * geoSo%nProcsPD(3) - 1
    if (iProcSo /= geoSo%MyProc) then
      ! --- Get remote source-process coordinates ---
      call MPI_Cart_Coords(geoSo%Comm, iProcSo, 3, PCoords, ierrMPI)
      call rhd_geo_Coords2mn(geoSo, PCoords, m1,n1, m2,n2, m3,n3)
      !
      ! --- Compute indices for overlap with local target box ---
      call rhd_box_SubBoxIndices(m1 ,n1 , m2 ,n2 , m3 ,n3 , &
                                 m1c,n1c, m2c,n2c, m3c,n3c, &
                                 m1b,n1b, m2b,n2b, m3b,n3b, &
                                 SubBoxMode='Dimensions-absolute', &
                                 IsEmpty_flag=IsEmpty_flag, IsComplete_flag=IsComplete_flag)
      !
      !write(gl%nc_p,'(A,I3,1X,A,3(I5,1X,I5,3X),L1)') &
      !                   '#', geoSo%MyProc, &
      !                   merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
      !                   ' taFRso; m1b,n1b, m2b,n2b, m3b,n3b: ', &
      !                             m1b,n1b, m2b,n2b, m3b,n3b, (.not. IsEmpty_flag)
      !call flush(gl%nc_p)
      !
      if (.not. IsEmpty_flag) then
        ! --- Determine the world ID of the source process ---
        call MPI_Group_Translate_Ranks(rhdMPI_GROUP_So   , 1, (/ iProcSo /), &
                                       rhdMPI_GROUP_WORLD,    NewRanks     , ierrMPI)
        iProcWoSo=NewRanks(1)
        !
        nProcsSo=nProcsSo+1
        !
        ! --- Create MPI data type for the target box to receive things ---
        call rhd_box_MPICreateType(boxTa, m1b,n1b, m2b,n2b, m3b,n3b, &
                                   boxTaTypes(nProcsSo), &
                                   time_mode=0, grid_mode=0, bulk_mode=2, rho_mode=rho_mode, &
                                   ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode)
        !
        ! --- Request to get the box ---
        call MPI_IRecv(boxTa, 1, boxTaTypes(nProcsSo), iProcWoSo, &
                       MPI_ANY_TAG, MPI_COMM_WORLD, RequestsTa(nProcsSo), ierrMPI)
        !
      endif ! (.not. IsEmpty_flag)
    endif ! (iProcSo /= geoSo%MyProc)
  end do ! iProcSo
endif ! (.not. IsEmpty_flag)
!
! === Find out what to send where and issue send requests ==========================================
!
! --- Compute indices of local source box ---
m1=boxSo%m1 ; n1=boxSo%n1 ; m2=boxSo%m2 ; n2=boxSo%n2 ; m3=boxSo%m3 ; n3=boxSo%n3
!m1=geoSo%m(1) ; n1=geoSo%n(1) ; m2=geoSo%m(2) ; n2=geoSo%n(2) ; m3=geoSo%m(3) ; n3=geoSo%n(3)
call rhd_box_SubBoxIndices(m1 ,n1 , m2 ,n2 , m3 ,n3 , &
                           0  ,0  , 0  ,0  , 0  ,0  , &
                           m1c,n1c, m2c,n2c, m3c,n3c, &
                           SubBoxMode=SubBoxMode, SubBoxDim=SubBoxDim, &
                           IsEmpty_flag=IsEmpty_flag)
!
!write(gl%nc_p,'(A,I3,1X,A,3(I5,1X,I5,3X),L1)') &
!                         '#', geoSo%MyProc, &
!                         merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
!                         ' source; m1c,n1c, m2c,n2c, m3c,n3c: ', &
!                                   m1c,n1c, m2c,n2c, m3c,n3c, (.not. IsEmpty_flag)
!call flush(gl%nc_p)
!
! --- Initialize the counter for relevant target processes ---
nProcsTa=0
!
if (.not. IsEmpty_flag) then
  ! --- Local source sub box is not empty: there is data to send ---
  !
  ! --- Loop over the target processes, skip local process ---
  do iProcTa=0, geoTa%nProcsPD(1) * geoTa%nProcsPD(2) * geoTa%nProcsPD(3) - 1
    if (iProcTa /= geoTa%MyProc) then
      ! --- Get remote target-process coordinates ---
      call MPI_Cart_Coords(geoTa%Comm, iProcTa, 3, PCoords, ierrMPI)
      call rhd_geo_Coords2mn(geoTa, PCoords, m1,n1, m2,n2, m3,n3)
      !m1=m1-nghost10 ; n1=n1+nghost10
      !m2=m2-nghost20 ; n2=n2+nghost20
      !m3=m3-nghost30 ; n3=n3+nghost30
      !
      ! --- Compute indices for overlap with local source box ---
      call rhd_box_SubBoxIndices(m1 ,n1 , m2 ,n2 , m3 ,n3 , &
                                 m1c,n1c, m2c,n2c, m3c,n3c, &
                                 m1b,n1b, m2b,n2b, m3b,n3b, &
                                 SubBoxMode='Dimensions-absolute', &
                                 IsEmpty_flag=IsEmpty_flag, IsComplete_flag=IsComplete_flag)
      !
      !write(gl%nc_p,'(A,I3,1X,A,3(I5,1X,I5,3X),L1)') &
      !                   '#', geoTa%MyProc, &
      !                   merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
      !                   ' soFRta; m1b,n1b, m2b,n2b, m3b,n3b: ', &
      !                             m1b,n1b, m2b,n2b, m3b,n3b, (.not. IsEmpty_flag)
      !call flush(gl%nc_p)
      !
      if (.not. IsEmpty_flag) then
        ! --- Determine the world ID of the target process ---
        call MPI_Group_Translate_Ranks(rhdMPI_GROUP_Ta   , 1, (/ iProcTa /), &
                                       rhdMPI_GROUP_WORLD,    NewRanks     , ierrMPI)
        iProcWoTa=NewRanks(1)
        !
        nProcsTa=nProcsTa+1
        !
        ! --- Create MPI data type ---
        call rhd_box_MPICreateType(boxSo, m1b,n1b, m2b,n2b, m3b,n3b, &
                                   boxSoTypes(nProcsTa), &
                                   time_mode=0, grid_mode=0, bulk_mode=2, rho_mode=rho_mode, &
                                   ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode)
        !
        ! --- Request to send the box ---
        call MPI_ISend(boxSo, 1, boxSoTypes(nProcsTa), iProcWoTa, &
                       MPI_ANY_TAG, MPI_COMM_WORLD, RequestsSo(nProcsTa), ierrMPI)
        !
      endif ! (.not. IsEmpty_flag)
    endif ! (iProcTa /= getTa%MyProc)
  end do ! iProcTa
endif ! (.not. IsEmpty_flag)
!
! === Copy local source box to local target box with OpenMP ========================================
!
! --- Compute indices for overlap of local target box with local source box ---
m1=boxTa%m1 ; n1=boxTa%n1 ; m2=boxTa%m2 ; n2=boxTa%n2 ; m3=boxTa%m3 ; n3=boxTa%n3
!m1=geoTa%m(1) ; n1=geoTa%n(1); m2=geoTa%m(2) ; n2=geoTa%n(2) ; m3=geoTa%m(3) ; n3=geoTa%n(3)
!m1=geoTa%m(1)-nghost10 ; n1=geoTa%n(1)+nghost10
!m2=geoTa%m(2)-nghost20 ; n2=geoTa%n(2)+nghost20
!m3=geoTa%m(3)-nghost20 ; n3=geoTa%n(3)+nghost30
call rhd_box_SubBoxIndices(m1 ,n1 , m2 ,n2 , m3 ,n3 , &
                           m1c,n1c, m2c,n2c, m3c,n3c, &
                           m1b,n1b, m2b,n2b, m3b,n3b, &
                           SubBoxMode='Dimensions-absolute', &
                           IsEmpty_flag=IsEmpty_flag, IsComplete_flag=IsComplete_flag)
!
!write(gl%nc_p,'(A,I3,1X,A,3(I5,1X,I5,3X),L1)') &
!                   '#', geoTa%MyProc, &
!                   merge(trim(SubBoxName), 'Chunk', present(SubBoxName)) // &
!                   ' local ; m1b,n1b, m2b,n2b, m3b,n3b: ', &
!                             m1b,n1b, m2b,n2b, m3b,n3b, (.not. IsEmpty_flag)
!call flush(gl%nc_p)
!
if (.not. IsEmpty_flag) then
  ! --- Copy sub box ---
  call rhd_box_CopyOMP(boxSo, m1b,n1b, m2b,n2b, m3b,n3b, &
                       boxTa, &
                       time_mode=0, grid_mode=0, flux_mode=0)
endif ! (.not. IsEmpty_flag)
!
! === Wait for all transmissions to finish and clear up temporary data =============================
!
! --- Wait for non-blocking send/receive requests ---
if (nProcsSo > 0) then
  call MPI_WaitAll(nProcsSo, boxTaTypes(1:nProcsSo), MPI_STATUSES_IGNORE, ierrMPI)
endif
if (nProcsTa > 0) then
  call MPI_WaitAll(nProcsTa, boxSoTypes(1:nProcsTa), MPI_STATUSES_IGNORE, ierrMPI)
endif
!
! --- Free data types ---
do iProcSo=1, nProcsSo
  call MPI_Type_Free(boxTaTypes(iProcSo), ierrMPI)
end do ! iProcSo
do iProcTa=1, nProcsTa
  call MPI_Type_Free(boxSoTypes(iProcTa), ierrMPI)
end do ! iProcTa
!
! --- Free MPI group handles ---
call MPI_Group_Free(rhdMPI_GROUP_WORLD, ierrMPI)
call MPI_Group_Free(rhdMPI_GROUP_So   , ierrMPI)
call MPI_Group_Free(rhdMPI_GROUP_Ta   , ierrMPI)
!
!#
# else
!#     /* rhd_mpi_l01: MPI not activated ----------------- */
!
if (.not. rhd_box_Exist(boxTa)) then
  ! --- Create box of appropriate size ---
  m1=geoTa%m(1) ; n1=geoTa%n(1); m2=geoTa%m(2) ; n2=geoTa%n(2) ; m3=geoTa%m(3) ; n3=geoTa%n(3)
  call rhd_box_Make(boxTa, m1=m1, n1=n1, m2=m2, n2=n2, m3=m3, n3=n3, &
                           nghost1=nghost10, nghost2=nghost20, nghost3=nghost30, &
                           rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_flag=quc_flag)
  !
  ! --- Set bulk arrays explicitly to zero trying to aid proper memory-page placement ---
  call rhd_box_ZeroOMP(boxTa)
endif
!
! --- Compute indices of local target box ---
m1=geoTa%m(1) ; n1=geoTa%n(1); m2=geoTa%m(2) ; n2=geoTa%n(2) ; m3=geoTa%m(3) ; n3=geoTa%n(3)
!m1=geoTa%m(1)-nghost10 ; n1=geoTa%n(1)+nghost10
!m2=geoTa%m(2)-nghost20 ; n2=geoTa%n(2)+nghost20
!m3=geoTa%m(3)-nghost20 ; n3=geoTa%n(3)+nghost30
!SubBoxDim0(1,1)=SubBoxDim(1,1)-nghost1 ; SubBoxDim0(2,1)=SubBoxDim(2,1)+nghost1
!SubBoxDim0(1,2)=SubBoxDim(1,2)-nghost2 ; SubBoxDim0(2,2)=SubBoxDim(2,2)+nghost2
!SubBoxDim0(1,3)=SubBoxDim(1,3)-nghost3 ; SubBoxDim0(2,3)=SubBoxDim(2,3)+nghost3
call rhd_box_SubBoxIndices(m1 ,n1 , m2 ,n2 , m3 ,n3 , &
                           0  ,0  , 0  ,0  , 0  ,0  , &
                           m1c,n1c, m2c,n2c, m3c,n3c, &
                           SubBoxMode=SubBoxMode, SubBoxDim=SubBoxDim, &
                           IsEmpty_flag=IsEmpty_flag)
!
! --- Copy sub box with OpenMP ---
call rhd_box_CopyOMP(boxSo, m1c,n1c, m2c,n2c, m3c,n3c, &
                     boxTa, &
                     time_mode=1, grid_mode=1, flux_mode=0)
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end subroutine rhd_box_MPICopy

!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ======================================================== */


!----------*********************--------------------------------------------------------------------
subroutine rhd_box_MPICreateType(box, m1,n1, m2,n2, m3,n3, &
                                 rhdMPI_boxDataType, &
                                 time_mode, grid_mode, bulk_mode, &
                                 rho_mode, ei_mode, v_mode, quc_mode, &
                                 x1_mode, bnd1l_mode, bnd1r_mode, &
                                 x2_mode, bnd2l_mode, bnd2r_mode, &
                                 x3_mode, bnd3l_mode, bnd3r_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_MPICreateType ('rhd_box_MPI_Create_Type')
!
! PURPOSE:
!   Create MPI data type for box structure.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_box_MPICreateType(box, m1,n1, m2,n2, m3,n3, &
!                              rhdMPI_boxDataType, &
!                              time_mode, grid_mode, bulk_mode, &
!                              rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, &
!                              quc_mode=quc_mode, &
!                              x1_mode=x1_mode, bnd1l_mode=bnd1l_mode, bnd1r_mode=bnd1r_mode, &
!                              x2_mode=x2_mode, bnd2l_mode=bnd2l_mode, bnd2r_mode=bnd2r_mode, &
!                              x3_mode=x3_mode, bnd3l_mode=bnd3l_mode, bnd3r_mode=bnd3r_mode)
!
! INPUT:
!   box:          ('box') type(box_type), computational box
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   time_mode:    ('time_mode') integer indicating if time data should be included:
!                   0: Don't include time data
!                   1: Include time data
!   grid_mode:    ('grid_mode') integer indicating if axes should be included:
!                   0: Don't include axes
!                   1: Include axes
!   bulk_mode:    ('bulk_mode') integer indicating if bulk data should be included:
!                   0: Don't include any bulk data
!                   1: Include bulk data only if requested by rho_mode, ei_mode, or v_mode
!                   2: Include bulk data unless blocked by rho_mode, ei_mode, v_mode, or quc_mode
!   rho_mode:     ('rho_mode') optional integer indicating if rho array should be included:
!                   not present: Include rho array only for bulk_mode=2
!                   0          : Don't include rho array
!                   1:         : Include rho array for bulk_mode=1 or 2
!   ei_mode:      ('energy_internal_mode') optional integer indicating if ei array should be
!                 included:
!                   not present: Include ei array only for bulk_mode=2
!                   0          : Don't include ei array
!                   1:         : Include ei array for bulk_mode=1 or 2
!   v_mode:       ('velocity_mode') optional integer indicating if v arrays should be included:
!                   not present: Include v arrays only for bulk_mode=2
!                   0          : Don't include v arrays
!                   1:         : Include v arrays for bulk_mode=1 or 2
!   quc_mode:     ('quantity_centered_mode') optional integer indicating if quc arrays should be
!                 included:
!                   not present: Include quc arrays only for bulk_mode=2
!                   0          : Don't include quc arrays
!                   1:         : Include quc arrays for bulk_mode=1 or 2
!   x1_mode:      ('x_1_mode') optional integer indicating if x1 axis should be included:
!                   0: Don't include x1 axis
!                   1: Include x1 axis (default if grid_mode=1)
!   bnd1l_mode:   ('boundary_1_left_mode') optional integer indicating if left-boundary values
!                 of boundary-1-centered quantities (so far xb1) should be included:
!                   0: Don't include left boundary value
!                   1: Include left boundary value (default if x1_mode=1)
!   bnd1r_mode:   ('boundary_1_right_mode') optional integer indicating if right-boundary values
!                 of boundary-1-centered quantities (so far xb1) should be included:
!                   0: Don't include right boundary value
!                   1: Include right boundary value (default if x1_mode=1)
!   x2_mode:      ('x_2_mode') optional integer indicating if x2 axis should be included:
!                   0: Don't include x2 axis
!                   1: Include x2 axis (default if grid_mode=1)
!   bnd2l_mode:   ('boundary_2_left_mode') optional integer indicating if left-boundary values
!                 of boundary-2-centered quantities (so far xb2) should be included:
!                   0: Don't include left boundary value
!                   1: Include left boundary value (default if x2_mode=1)
!   bnd2r_mode:   ('boundary_2_right_mode') optional integer indicating if right-boundary values
!                 of boundary-2-centered quantities (so far xb2) should be included:
!                   0: Don't include right boundary value
!                   1: Include right boundary value (default if x2_mode=1)
!   x3_mode:      ('x_3_mode') optional integer indicating if x3 axis should be included:
!                   0: Don't include x3 axis
!                   1: Include x3 axis (default if grid_mode=1)
!   bnd3l_mode:   ('boundary_3_left_mode') optional integer indicating if left-boundary values
!                 of boundary-3-centered quantities (so far xb3) should be included:
!                   0: Don't include left boundary value
!                   1: Include left boundary value (default if x3_mode=1)
!   bnd3r_mode:   ('boundary_3_right_mode') optional integer indicating if right-boundary values
!                 of boundary-3-centered quantities (so far xb3) should be included:
!                   0: Don't include right boundary value
!                   1: Include right boundary value (default if x3_mode=1)
!
! OUTPUT:
!   rhdMPI_boxDataType: ('rhd_MPI_box_Data_Type') integer with MPI data type handle
!                       for box structure
!
! ROUTINES:
!   flush:                 Flush output buffer
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!   MPI_Get_Address:       Get the address of a location in memory
!   MPI_Type_Commit:       Commit an MPI data type
!   MPI_Type_Create_Struct: Create an MPI structured data type
!   MPI_Type_Create_Subarray: Create an MPI data type describing a sub array
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The created MPI data type has to be freed later on with a call to MPI_Type_Free.
!
! PROCEDURE:
!   Create and fill arrays for displacement, length, and type of variables in box structure.
!   Put the result into MPI structure and give handle to resulting MPI data type as output.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-07-31 (B.F. Lyon) Written
!   2012-11-14 (B.F.) Reordered arguments, additional optional boundary-layer arguments
!   2016-03-19 (B.F. Uppsala) Add comments and modes
!   2017-07-07 (B.F.) Optionally include quc arrays in data type
!   2017-08-12 (B.F.) Free the temporary MPI type after use
!   2017-09-18 (B.F.) Rename rhd_boxMPI_CreateType -> rhd_box_MPICreateType
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(box_type),            intent(in)   :: box
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
integer,                   intent(out)  :: rhdMPI_BoxDataType
integer,                   intent(in)   :: time_mode, grid_mode, bulk_mode
integer,         optional, intent(in)   :: rho_mode, ei_mode, v_mode, quc_mode, &
                                           x1_mode, bnd1l_mode, bnd1r_mode, &
                                           x2_mode, bnd2l_mode, bnd2r_mode, &
                                           x3_mode, bnd3l_mode, bnd3r_mode
!
! --- Local variables ---
integer, parameter                      :: nBlocks=1100
integer                                 :: iquc
integer, dimension(nBlocks)             :: a_BlockLengths, a_Types
integer(kind=MPI_ADDRESS_KIND)          :: Displacement_m1
integer(kind=MPI_ADDRESS_KIND), &
         dimension(nBlocks)             :: a_Displacements
integer                                 :: mt, nt, &
                                           x1_mode0, bnd1l_mode0, bnd1r_mode0, &
                                           x2_mode0, bnd2l_mode0, bnd2r_mode0, &
                                           x3_mode0, bnd3l_mode0, bnd3r_mode0
integer                                 :: ierrMPI, i, TypeMPI, TypequcMPI, MyProc
!---------------------------------------------------------------------------------------------------
!
! --- Set default values for non-present optional parameters ---
if (present(bnd1l_mode)) then
  bnd1l_mode0=bnd1l_mode
else
  bnd1l_mode0=1
endif
if (present(bnd1r_mode)) then
  bnd1r_mode0=bnd1r_mode
else
  bnd1r_mode0=1
endif
if (present(bnd2l_mode)) then
  bnd2l_mode0=bnd2l_mode
else
  bnd2l_mode0=1
endif
if (present(bnd2r_mode)) then
  bnd2r_mode0=bnd2r_mode
else
  bnd2r_mode0=1
endif
if (present(bnd3l_mode)) then
  bnd3l_mode0=bnd3l_mode
else
  bnd3l_mode0=1
endif
if (present(bnd3r_mode)) then
  bnd3r_mode0=bnd3r_mode
else
  bnd3r_mode0=1
endif
!
! --- Set entry counter to zero ---
i=0
!
! === Describe structure the MPI way ===
!
if (time_mode > 0) then
  ! --- Include time data ---
  !
  i=i+1
  call MPI_Get_Address(box%itime   , a_Displacements(i), ierrMPI)
  a_BlockLengths(i)=1
  a_Types(i)       =MPI_INTEGER
  !
  i=i+1
  call MPI_Get_Address(box%time    , a_Displacements(i), ierrMPI)
  a_BlockLengths(i)=1
  a_Types(i)       =MPI_REAL
  !
  i=i+1
  call MPI_Get_Address(box%time_db , a_Displacements(i), ierrMPI)
  a_BlockLengths(i)=1
  a_Types(i)       =MPI_DOUBLE_PRECISION
endif ! (time_mode)
!
if (grid_mode > 0) then
  ! --- Include axis data ---
  !
  ! --- Set default values for non-present optional parameters ---
  if (present(x1_mode)) then
    x1_mode0=x1_mode
  else
    x1_mode0=1
  endif
  if (present(x2_mode)) then
    x2_mode0=x2_mode
  else
    x2_mode0=1
  endif
  if (present(x3_mode)) then
    x3_mode0=x3_mode
  else
    x3_mode0=1
  endif
  !
  if (x1_mode0 > 0) then
    i=i+1
    call MPI_Get_Address(box%xc1(m1) , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=n1-m1+1
    a_Types(i)       =MPI_REAL
    !
    mt=m1
    nt=n1+1
    if (bnd1l_mode0 <= 0) then
      mt=mt+1
    endif
    if (bnd1r_mode0 <= 0) then
      nt=nt-1
    endif
    if (nt >= mt) then
      i=i+1
      call MPI_Get_Address(box%xb1(mt) , a_Displacements(i), ierrMPI)
      a_BlockLengths(i)=nt-mt+1
      a_Types(i)       =MPI_REAL
    endif
  endif ! (x1_mode0)
  !
  if (x2_mode0 > 0) then
    i=i+1
    call MPI_Get_Address(box%xc2(m2) , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=n2-m2+1
    a_Types(i)       =MPI_REAL
    !
    mt=m2
    nt=n2+1
    if (bnd2l_mode0 <= 0) then
      mt=mt+1
    endif
    if (bnd2r_mode0 <= 0) then
      nt=nt-1
    endif
    if (nt >= mt) then
      i=i+1
      call MPI_Get_Address(box%xb2(mt) , a_Displacements(i), ierrMPI)
      a_BlockLengths(i)=nt-mt+1
      a_Types(i)       =MPI_REAL
    endif
  endif ! (x2_mode0)
  !
  if (x3_mode0 > 0) then
    i=i+1
    call MPI_Get_Address(box%xc3(m3) , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=n3-m3+1
    a_Types(i)       =MPI_REAL
    !
    mt=m3
    nt=n3+1
    if (bnd3l_mode0 <= 0) then
      mt=mt+1
    endif
    if (bnd3r_mode0 <= 0) then
      nt=nt-1
    endif
    if (nt >= mt) then
      i=i+1
      call MPI_Get_Address(box%xb3(mt) , a_Displacements(i), ierrMPI)
      a_BlockLengths(i)=nt-mt+1
      a_Types(i)       =MPI_REAL
    endif
  endif ! (x3_mode0)
endif ! (grid_mode)
!
if (bulk_mode > 0) then
  ! --- Include bulk data ---
  !
  call MPI_Comm_Rank(MPI_COMM_WORLD, MyProc, ierrMPI)
!!$  write(6,'(A,I3,A,6I5,3X,3(2X,3I5))') &
!!$           '#', MyProc, ' MPI_Type_Create_Subarray', &
!!$           m1,n1, m2,n2, m3,n3, (/ box%n1-box%m1+1+2*box%nghost1, &
!!$                                   box%n2-box%m2+1+2*box%nghost2, &
!!$                                   box%n3-box%m3+1+2*box%nghost3 /), &
!!$                                (/ n1-m1+1, &
!!$                                   n2-m2+1, &
!!$                                   n3-m3+1/), &
!!$                                (/ m1-box%m1+box%nghost1, &
!!$                                   m2-box%m2+box%nghost2, &
!!$                                   m3-box%m3+box%nghost3 /)
  !
  call MPI_Type_Create_Subarray(3, &
                                (/ box%n1-box%m1+1+2*box%nghost1, &
                                   box%n2-box%m2+1+2*box%nghost2, &
                                   box%n3-box%m3+1+2*box%nghost3 /), &
                                (/ n1-m1+1, &
                                   n2-m2+1, &
                                   n3-m3+1/), &
                                (/ m1-box%m1+box%nghost1, &
                                   m2-box%m2+box%nghost2, &
                                   m3-box%m3+box%nghost3 /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                TypeMPI, &
                                ierrMPI)
  !
  if (merge(rho_mode, bulk_mode-1, present(rho_mode)) > 0) then
    i=i+1
    call MPI_Get_Address(box%rho, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =TypeMPI
  endif ! (rho_mode)
  !
  if (merge(ei_mode, bulk_mode-1, present(ei_mode)) > 0) then
    i=i+1
    call MPI_Get_Address(box%ei , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =TypeMPI
  endif ! (ei_mode)
  !
  if (merge(v_mode, bulk_mode-1, present(v_mode)) > 0) then
    i=i+1
    call MPI_Get_Address(box%v1 , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =TypeMPI
    !
    i=i+1
    call MPI_Get_Address(box%v2 , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =TypeMPI
    !
    i=i+1
    call MPI_Get_Address(box%v3 , a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =TypeMPI
  endif ! (v_mode)
  !
  if ((merge(quc_mode, bulk_mode-1, present(quc_mode)) > 0) .and. &
      (box%nquc > 0)                                               ) then
    i=i+1
    call MPI_Get_Address(box%quc, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=box%nquc
    a_Types(i)       =TypeMPI
    !
!!$    do iquc=1,box%nquc
!!$      i=i+1
!!$      call MPI_Get_Address(box%quc(:,:,:,iquc), a_Displacements(i), ierrMPI)
!!$      a_BlockLengths(i)=1
!!$      a_Types(i)       =TypeMPI
!!$    end do ! iquc
    !
!!$    call MPI_Type_Create_Subarray(4, &
!!$                                  (/ box%n1-box%m1+1+2*box%nghost1, &
!!$                                     box%n2-box%m2+1+2*box%nghost2, &
!!$                                     box%n3-box%m3+1+2*box%nghost3, &
!!$                                     box%nquc /), &
!!$                                  (/ n1-m1+1, &
!!$                                     n2-m2+1, &
!!$                                     n3-m3+1, &
!!$                                     box%nquc /), &
!!$                                  (/ m1-box%m1+box%nghost1, &
!!$                                     m2-box%m2+box%nghost2, &
!!$                                     m3-box%m3+box%nghost3, &
!!$                                     0 /), &
!!$                                  MPI_ORDER_FORTRAN, &
!!$                                  MPI_REAL, &
!!$                                  TypequcMPI, &
!!$                                  ierrMPI)
!!$    i=i+1
!!$    call MPI_Get_Address(box%quc, a_Displacements(i), ierrMPI)
!!$    a_BlockLengths(i)=1
!!$    a_Types(i)       =TypequcMPI
  endif ! (quc_mode)
  !
  ! --- Free the MPI type describing the subarray ---
  !call MPI_Type_Free(TypeMPI, ierrMPI)
  !
endif ! (bulk_mode)
!
! --- Shift displacements ---
call MPI_Get_Address(box%m1, Displacement_m1, ierrMPI)
a_Displacements(1:i)=a_Displacements(1:i)-Displacement_m1
!
! --- Use arrays to create MPI structure data type ---
call MPI_Type_Create_Struct(i, a_BlockLengths(1:i), a_Displacements(1:i), a_Types(1:i), &
                            rhdMPI_BoxDataType, ierrMPI)
!
! --- Commit new MPI data type ---
call MPI_Type_Commit(rhdMPI_boxDataType, ierrMPI)
!
! call flush(6)
!
end subroutine rhd_box_MPICreateType


!----------*************************----------------------------------------------------------------
subroutine rhd_box_MPIFillGhostStart(geo, box, dim, ngh, m1,n1, m2,n2, m3,n3, &
                                     RequestsMPI, rhdMPI_boxGhostTypes, &
                                     grid_mode, bulk_mode, &
                                     rho_mode, ei_mode, v_mode, quc_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_MPIFillGhostStart ('rhd_box_MPI_Fill_Ghost-cells_Start')
!
! PURPOSE:
!   Start filling ghost cells by creating MPI data types and submitting send and receive requests.
!
! CATEGORY:
!   MPI, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_MPIFillGhostStart(geo, box, dim, ngh, m1,n1, m2,n2, m3,n3, &
!                                  RequestsMPI, rhdMPI_boxGhostTypes, &
!                                  grid_mode, bulk_mode, &
!                                  rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, &
!                                  quc_mode=quc_mode)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   dim:          ('dimension') integer with direction (1, 2, or 3) of ghost-layer communication
!   ngh:          ('number_ghost') integer with number of ghost layers to be transferred
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   grid_mode:    ('grid_mode') integer indicating if axes should be included:
!                   0: Don't include axes
!                   1: Include axes
!   bulk_mode:    ('bulk_mode') integer indicating if bulk data should be included:
!                   0: Don't include any bulk data
!                   1: Include bulk data only if requested by rho_mode, ei_mode, or v_mode
!                   2: Include bulk data unless blocked by rho_mode, ei_mode, or v_mode
!   rho_mode:     ('rho_mode') optional integer indicating if rho array should be included:
!                   not present: Include rho array only for bulk_mode=2
!                   0          : Don't include rho array
!                   1:         : Include rho array for bulk_mode=1 or 2
!   ei_mode:      ('energy_internal_mode') optional integer indicating if ei array should be
!                 included:
!                   not present: Include ei array only for bulk_mode=2
!                   0          : Don't include ei array
!                   1:         : Include ei array for bulk_mode=1 or 2
!   v_mode:       ('velocity_mode') optional integer indicating if v arrays should be included:
!                   not present: Include v arrays only for bulk_mode=2
!                   0          : Don't include v arrays
!                   1:         : Include v arrays for bulk_mode=1 or 2
!   quc_mode:     ('quantity_centered_mode') optional integer indicating if quc arrays should be
!                 included:
!                   not present: Include v arrays only for bulk_mode=2
!                   0          : Don't include v arrays
!                   1:         : Include v arrays for bulk_mode=1 or 2
!
! OUTPUT:
!   RequestsMPI:  ('Requests_MPI') 4-element integer vector with MPI-request handles
!   rhdMPI_BoxGhostTypes: ('rhd_MPI_Box_Ghost_Types') 4-element integer vector with MPI-type handles
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Recv:              Perform a blocking receive
!   MPI_Send:              Perform a blocking send
!   MPI_SendRecv:          Perform a combined blocked send and receive
!   MPI_SSend:             Perform a blocking synchronous send
!   rhd_box_MPICreateType: Create MPI data type for box structure
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Create some MPI data types and SendReceive handles.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   ! --- Start filling of ghost layers of model in x1 direction ---
!   call rhd_box_MPIFillGhostStart(geo, box, 1, ngh, &
!                                  box%m1     , box%n1     , &
!                                  box%m2     , box%n2     , &
!                                  box%m3     , box%n3     , &
!                                  RequestsMPI=m0x1_RequestsMPI, &
!                                  rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
!                                  grid_mode=grid_mode, bulk_mode=bulk_mode)
!   ! --- Finalize filling of ghost layers of model in x1 direction ---
!   call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
!
! MODIFICATION HISTORY:
!   2012-11-13 (B.F. Lyon) Written
!   2016-03-20 (B.F. Uppsala) Add geo structure as input parameter
!   2017-07-08 (B.F.) Reorder parameters; pass additional parameters to rhd_boxMPI_CreateType
!   2017-09-18 (B.F.) Rename rhd_boxMPI_FillGhostStart -> rhd_box_MPIFillGhostStart
!   2017-09-30 (B.F.) Merge MPI-code blocks into single block; use variants of MPI send/receive
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(box_type),            intent(inout):: box
integer,                   intent(in)   :: dim, ngh, m1,n1, m2,n2, m3,n3
integer, dimension(4),     intent(out)  :: RequestsMPI, rhdMPI_BoxGhostTypes
integer,                   intent(in)   :: grid_mode, bulk_mode
integer, optional,         intent(in)   :: rho_mode, ei_mode, v_mode, quc_mode
!
! --- Local variables ---
integer                                 :: ierrMPI, iOrder
integer, dimension(MPI_STATUS_SIZE)     :: StatusMPI
!---------------------------------------------------------------------------------------------------
!
! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, &
!      ' Create MPI types for ghost cells'
!
if (dim == 1) then
  ! === Fill ghost layers orthogonal to the x1 direction ===
  !
  ! --- Create MPI box data types (from left to right) ---
  call rhd_box_MPICreateType(box,                               m1-ngh  ,m1-1    , m2,n2, m3,n3, &
         rhdMPI_boxGhostTypes(1), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x2_mode=0, x3_mode=0, bnd1l_mode=1, bnd1r_mode=0)
  call rhd_box_MPICreateType(box,                               m1      ,m1+ngh-1, m2,n2, m3,n3, &
         rhdMPI_boxGhostTypes(2), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x2_mode=0, x3_mode=0, bnd1l_mode=0, bnd1r_mode=1)
  call rhd_box_MPICreateType(box,                               n1-ngh+1,n1      , m2,n2, m3,n3, &
         rhdMPI_boxGhostTypes(3), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x2_mode=0, x3_mode=0, bnd1l_mode=1, bnd1r_mode=0)
  call rhd_box_MPICreateType(box,                               n1+1    ,n1+ngh  , m2,n2, m3,n3, &
         rhdMPI_boxGhostTypes(4), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x2_mode=0, x3_mode=0, bnd1l_mode=0, bnd1r_mode=1)
  !
else if (dim == 2) then
  ! === Fill ghost layers orthogonal to the x2 direction ===
  !
  ! --- Create MPI box data types (from left to right) ---
  call rhd_box_MPICreateType(box,                               m1,n1, m2-ngh  ,m2-1    , m3,n3, &
         rhdMPI_boxGhostTypes(1), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x3_mode=0, bnd2l_mode=1, bnd2r_mode=0)
  call rhd_box_MPICreateType(box,                               m1,n1, m2      ,m2+ngh-1, m3,n3, &
         rhdMPI_boxGhostTypes(2), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x3_mode=0, bnd2l_mode=0, bnd2r_mode=1)
  call rhd_box_MPICreateType(box,                               m1,n1, n2-ngh+1,n2      , m3,n3, &
         rhdMPI_boxGhostTypes(3), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x3_mode=0, bnd2l_mode=1, bnd2r_mode=0)
  call rhd_box_MPICreateType(box,                               m1,n1, n2+1    ,n2+ngh  , m3,n3, &
         rhdMPI_boxGhostTypes(4), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x3_mode=0, bnd2l_mode=0, bnd2r_mode=1)
  !
else ! (dim == 3)
  ! === Fill ghost layers orthogonal to the x3 direction ===
  !
  ! --- Create MPI box data types (from left to right) ---
  call rhd_box_MPICreateType(box,                               m1,n1, m2,n2, m3-ngh  ,m3-1    , &
         rhdMPI_boxGhostTypes(1), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x2_mode=0, bnd3l_mode=1, bnd3r_mode=0)
  call rhd_box_MPICreateType(box,                               m1,n1, m2,n2, m3      ,m3+ngh-1, &
         rhdMPI_boxGhostTypes(2), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x2_mode=0, bnd3l_mode=0, bnd3r_mode=1)
  call rhd_box_MPICreateType(box,                               m1,n1, m2,n2, n3-ngh+1,n3      , &
         rhdMPI_boxGhostTypes(3), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x2_mode=0, bnd3l_mode=1, bnd3r_mode=0)
  call rhd_box_MPICreateType(box,                               m1,n1, m2,n2, n3+1    ,n3+ngh  , &
         rhdMPI_boxGhostTypes(4), &
         time_mode=0, grid_mode=grid_mode, bulk_mode=bulk_mode, &
         rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, quc_mode=quc_mode, &
         x1_mode=0, x2_mode=0, bnd3l_mode=0, bnd3r_mode=1)
  !
endif ! (dim)
!
!!$if (par%N_test1 == 1) then
!!$  ! --- Send and receive data directly with MPI_Send/MPI_Recv        ---
!!$  ! --- Swap the order of send and receive calls to prevent stalling ---
!!$  !
!!$  ! write(gl%nc_p, '(A,I3,A,3I4)') '#', geo%MyProc, &
!!$  !   ' Transfer ghost cells (Send): ', dim, geo%Proc_l(dim), geo%Proc_r(dim)
!!$  !
!!$  ! --- Send from left of boundaries to right of boundaries ---
!!$  do iOrder=0,1
!!$    if (mod(geo%iProcPD(dim),2) == iOrder) then
!!$      call MPI_Send(box, 1, rhdMPI_boxGhostTypes(3), geo%Proc_r(dim), 10*dim+0, &
!!$                    geo%Comm, ierrMPI)
!!$    else
!!$      call MPI_Recv(box, 1, rhdMPI_boxGhostTypes(1), geo%Proc_l(dim), 10*dim+0, &
!!$                    geo%Comm, StatusMPI, ierrMPI)
!!$    endif
!!$  end do ! iOrder
!!$  !
!!$  ! --- Send from right of boundaries to left of boundaries ---
!!$  do iOrder=0,1
!!$    if (mod(geo%iProcPD(dim),2) == iOrder) then
!!$      call MPI_Send(box, 1, rhdMPI_boxGhostTypes(2), geo%Proc_l(dim), 10*dim+1, &
!!$                    geo%Comm, ierrMPI)
!!$    else
!!$      call MPI_Recv(box, 1, rhdMPI_boxGhostTypes(4), geo%Proc_r(dim), 10*dim+1, &
!!$                    geo%Comm, StatusMPI, ierrMPI)
!!$    endif
!!$  end do ! iOrder
!!$  !
!!$else if (par%N_test1 == 2) then
!!$  ! --- Send and receive data directly with MPI_SSend/MPI_Recv       ---
!!$  ! --- Swap the order of send and receive calls to prevent stalling ---
!!$  !
!!$  ! write(gl%nc_p, '(A,I3,A,3I4)') '#', geo%MyProc, &
!!$  !   ' Transfer ghost cells (SSend): ', dim, geo%Proc_l(dim), geo%Proc_r(dim)
!!$  !
!!$  ! --- Send from left of boundaries to right of boundaries ---
!!$  do iOrder=0,1
!!$    if (mod(geo%iProcPD(dim),2) == iOrder) then
!!$      call MPI_SSend(box, 1, rhdMPI_boxGhostTypes(3), geo%Proc_r(dim), 10*dim+0, &
!!$                     geo%Comm, ierrMPI)
!!$    else
!!$      call MPI_Recv( box, 1, rhdMPI_boxGhostTypes(1), geo%Proc_l(dim), 10*dim+0, &
!!$                     geo%Comm, StatusMPI, ierrMPI)
!!$    endif
!!$  end do ! iOrder
!!$  !
!!$  ! --- Send from right of boundaries to left of boundaries ---
!!$  do iOrder=0,1
!!$    if (mod(geo%iProcPD(dim),2) == iOrder) then
!!$      call MPI_SSend(box, 1, rhdMPI_boxGhostTypes(2), geo%Proc_l(dim), 10*dim+1, &
!!$                     geo%Comm, ierrMPI)
!!$    else
!!$      call MPI_Recv( box, 1, rhdMPI_boxGhostTypes(4), geo%Proc_r(dim), 10*dim+1, &
!!$                     geo%Comm, StatusMPI, ierrMPI)
!!$    endif
!!$  end do ! iOrder
!!$  !
!!$else if (par%N_test1 == 3) then
!!$  ! --- Send and receive data directly with MPI_SendRecv ---
!!$  !
!!$  ! write(gl%nc_p, '(A,I3,A,3I4)') '#', geo%MyProc, &
!!$  !   ' Transfer ghost cells (SendRecv): ', dim, geo%Proc_l(dim), geo%Proc_r(dim)
!!$  !
!!$  ! --- Send from left of boundaries to right of boundaries ---
!!$  call MPI_SendRecv(box, 1, rhdMPI_boxGhostTypes(3), geo%Proc_r(dim), 10*dim+0, &
!!$                    box, 1, rhdMPI_boxGhostTypes(1), geo%Proc_l(dim), 10*dim+0, &
!!$                    geo%Comm, StatusMPI, ierrMPI)
!!$  !
!!$  ! --- Send from right of boundaries to left of boundaries ---
!!$  call MPI_SendRecv(box, 1, rhdMPI_boxGhostTypes(2), geo%Proc_l(dim), 10*dim+1, &
!!$                    box, 1, rhdMPI_boxGhostTypes(4), geo%Proc_r(dim), 10*dim+1, &
!!$                    geo%Comm, StatusMPI, ierrMPI)
!!$  !
!!$else ! (par%N_test1)
  ! --- Submit MPI send and receive requests with MPI_ISend/MPI_IRecv                         ---
  ! --- The process has to be finalized with a call to MPI_WaitAll in rhd_box_MPIFillGhostEnd ---
  !
  ! write(gl%nc_p, '(A,I3,A,3I4)') '#', geo%MyProc, &
  !   ' Transfer ghost cells (ISend): ', dim, geo%Proc_l(dim), geo%Proc_r(dim)
  !
  call MPI_IRecv(box, 1, rhdMPI_boxGhostTypes(1), geo%Proc_l(dim), 10*dim+0, &
                 geo%Comm, RequestsMPI(1), ierrMPI)
  call MPI_ISend(box, 1, rhdMPI_boxGhostTypes(3), geo%Proc_r(dim), 10*dim+0, &
                 geo%Comm, RequestsMPI(3), ierrMPI)
  call MPI_ISend(box, 1, rhdMPI_boxGhostTypes(2), geo%Proc_l(dim), 10*dim+1, &
                 geo%Comm, RequestsMPI(2), ierrMPI)
  call MPI_IRecv(box, 1, rhdMPI_boxGhostTypes(4), geo%Proc_r(dim), 10*dim+1, &
                 geo%Comm, RequestsMPI(4), ierrMPI)
  !
!!$endif ! (par%N_test1)
!
end subroutine rhd_box_MPIFillGhostStart


!----------***********************------------------------------------------------------------------
subroutine rhd_box_MPIFillGhostEnd(RequestsMPI, rhdMPI_boxGhostTypes)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_MPIFillGhostEnd ('rhd_box_MPI_Fill_Ghost-cells_End')
!
! PURPOSE:
!   Finalize filling ghost cells.
!
! CATEGORY:
!   MPI, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_MPIFillGhostEnd(RequestsMPI, rhdMPI_boxGhostTypes)
!
! INPUT:
!   RequestsMPI:  ('Requests_MPI') 4-element integer vector with MPI-request handles
!   rhdMPI_BoxGhostTypes: ('rhd_MPI_Box_Ghost_Types') 4-element integer vector with MPI-type handles
!
! OUTPUT:
!   None
!
! ROUTINES:
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Delete some MPI data types and SendReceive handles.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   ! --- Start filling of ghost layers of model in x1 direction ---
!   call rhd_box_MPIFillGhostStart(geo, box, 1, ngh, &
!                                  box%m1     , box%n1     , &
!                                  box%m2     , box%n2     , &
!                                  box%m3     , box%n3     , &
!                                  RequestsMPI=m0x1_RequestsMPI, &
!                                  rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
!                                  grid_mode=grid_mode, bulk_mode=bulk_mode)
!   ! --- Finalize filling of ghost layers of model in x1 direction ---
!   call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
!
! MODIFICATION HISTORY:
!   2012-11-13 (B.F. Lyon) Written
!   2017-09-18 (B.F.) Rename rhd_boxMPI_FillGhostEnd -> rhd_box_MPIFillGhostEnd
!   2017-09-30 (B.F.) Use variants of MPI send/receive
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
integer, dimension(4),     intent(inout):: RequestsMPI, rhdMPI_BoxGhostTypes
!
! --- Local variables ---
integer                                 :: ierrMPI
integer, dimension(MPI_STATUS_SIZE,1:4) :: StatusMPI
!---------------------------------------------------------------------------------------------------
!
! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, &
!   ' Wait until all boundary boxes are sent and received'
!
!!$if      (par%N_test1 == 1) then
!!$  !
!!$else if (par%N_test1 == 2) then
!!$  !
!!$else if (par%N_test1 == 3) then
!!$  !
!!$else ! (par%N_test1)
  ! --- Send and receive data directly with MPI_Send/MPI_Recv ---
  !
  call MPI_WaitAll(4, RequestsMPI, StatusMPI, ierrMPI)
!!$endif ! (par%N_test1)
!
! write(gl%nc_p, '(A,I3,A)') '#', wg%hyd%MyProc, ' Finished waiting for boundary boxes'
!
! --- Delete MPI box data types ---
call MPI_Type_Free(rhdMPI_boxGhostTypes(1), ierrMPI)
call MPI_Type_Free(rhdMPI_boxGhostTypes(2), ierrMPI)
call MPI_Type_Free(rhdMPI_boxGhostTypes(3), ierrMPI)
call MPI_Type_Free(rhdMPI_boxGhostTypes(4), ierrMPI)
!
end subroutine rhd_box_MPIFillGhostEnd


!----------********************---------------------------------------------------------------------
subroutine rhd_box_MPIFillGhost(geo, box, ngh, &
                                grid_mode, bulk_mode, &
                                rho_mode, ei_mode, v_mode, quc_mode)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_MPIFillGhost ('rhd_box_MPI_Fill_Ghost-cells')
!
! PURPOSE:
!   Fill ghost cells by communicating with neighbor MPI processes.
!
! CATEGORY:
!   MPI, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_MPIFillGhost(geo, box, ngh, &
!                             grid_mode, bulk_mode, &
!                             rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, &
!                             quc_mode=quc_mode)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   ngh:          ('number_ghost') integer with number of ghost layers to be transferred
!   grid_mode:    ('grid_mode') integer indicating if axes should be included:
!                   0: Don't include axes
!                   1: Include axes
!   bulk_mode:    ('bulk_mode') integer indicating if bulk data should be included:
!                   0: Don't include any bulk data
!                   1: Include bulk data only if requested by rho_mode, ei_mode, or v_mode
!                   2: Include bulk data unless blocked by rho_mode, ei_mode, or v_mode
!   rho_mode:     ('rho_mode') optional integer indicating if rho array should be included:
!                   not present: Include rho array only for bulk_mode=2
!                   0          : Don't include rho array
!                   1:         : Include rho array for bulk_mode=1 or 2
!   ei_mode:      ('energy_internal_mode') optional integer indicating if ei array should be
!                 included:
!                   not present: Include ei array only for bulk_mode=2
!                   0          : Don't include ei array
!                   1:         : Include ei array for bulk_mode=1 or 2
!   v_mode:       ('velocity_mode') optional integer indicating if v arrays should be included:
!                   not present: Include v arrays only for bulk_mode=2
!                   0          : Don't include v arrays
!                   1:         : Include v arrays for bulk_mode=1 or 2
!   quc_mode:     ('quantity_centered_mode') optional integer indicating if quc arrays should be
!                 included:
!                   not present: Include quc array only for bulk_mode=2
!                   0          : Don't include quc array
!                   1:         : Include quc array for bulk_mode=1 or 2
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   MPI:                   Standard MPI routines and definitions
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   If the box has only one element in one dimension, the corresponding ghost layers are not filled.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2016-03-20 (B.F. Uppsala) Written
!   2017-07-08 (B.F.) Pass additional mode parameters to rhd_boxMPI_FillGhostStart
!   2017-09-18 (B.F.) Rename rhd_boxMPI_FillGhost -> rhd_box_MPIFillGhost
!   2018-11-18 (B.F.) Add shift of axes values for internal periodic boundaries
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(box_type),            intent(inout):: box
integer,                   intent(in)   :: ngh, &
                                           grid_mode, bulk_mode
integer, optional,         intent(in)   :: rho_mode, ei_mode, v_mode, quc_mode
!
! --- Local variables ---
integer                                 :: ierrMPI, ngh1, ngh2
integer, dimension(1:4)                 :: m0x1_rhdMPI_boxGhostTypes, m0x1_RequestsMPI, &
                                           m0x2_rhdMPI_boxGhostTypes, m0x2_RequestsMPI, &
                                           m0x3_rhdMPI_boxGhostTypes, m0x3_RequestsMPI
!---------------------------------------------------------------------------------------------------
!
! --- Fill ghost cells with data from neighbor processes ---
!
if ((box%n1 > box%m1) .and. (ngh > 0)) then
  ! --- Start filling of ghost layers of model in x1 direction ---
  call rhd_box_MPIFillGhostStart(geo, box, 1, ngh, &
                                 box%m1     , box%n1     , &
                                 box%m2     , box%n2     , &
                                 box%m3     , box%n3     , &
                                 RequestsMPI=m0x1_RequestsMPI, &
                                 rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
                                 grid_mode=grid_mode, bulk_mode=bulk_mode, &
                                 rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, &
                                 quc_mode=quc_mode)
  ! --- Finalize filling of ghost layers of model in x1 direction ---
  call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
  !
  ngh1=ngh
  !
  ! --- Account for periodic boundary conditions: shift axes values ---
  if (grid_mode >= 1) then
    if ((geo%bound_l(1) == bound_InternalPeriodic) .and. &
        (box%xb1(box%m1-1) >= box%xb1(box%m1))              ) then
      box%xc1(  box%m1-box%nghost1  :box%m1-1)= &
        box%xc1(box%m1-box%nghost1  :box%m1-1) - (geo%xb_r(1)-geo%xb_l(1))
      box%xb1(  box%m1-box%nghost1  :box%m1-1)= &
        box%xb1(box%m1-box%nghost1  :box%m1-1) - (geo%xb_r(1)-geo%xb_l(1))
    endif
    if ((geo%bound_r(1) == bound_InternalPeriodic) .and. &
        (box%xb1(box%n1+2) <= box%xb1(box%n1+1))            ) then
      box%xc1(  box%n1+1:box%n1+box%nghost1  )= &
        box%xc1(box%n1+1:box%n1+box%nghost1  ) + (geo%xb_r(1)-geo%xb_l(1))
      box%xb1(  box%n1+2:box%n1+box%nghost1+1)= &
        box%xb1(box%n1+2:box%n1+box%nghost1+1) + (geo%xb_r(1)-geo%xb_l(1))
    endif
  endif ! (grid_mode >= 1)
else
  ngh1=0
endif
!
if ((box%n2 > box%m2) .and. (ngh > 0)) then
  ! --- Start filling of ghost layers of model in x2 direction ---
  call rhd_box_MPIFillGhostStart(geo, box, 2, ngh, &
                                 box%m1-ngh1, box%n1+ngh1, &
                                 box%m2     , box%n2     , &
                                 box%m3     , box%n3     , &
                                 RequestsMPI=m0x2_RequestsMPI, &
                                 rhdMPI_boxGhostTypes=m0x2_rhdMPI_boxGhostTypes, &
                                 grid_mode=grid_mode, bulk_mode=bulk_mode, &
                                 rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, &
                                 quc_mode=quc_mode)
  ! --- Finalize filling of ghost layers of model in x2 direction ---
  call rhd_box_MPIFillGhostEnd(m0x2_RequestsMPI, m0x2_rhdMPI_boxGhostTypes)
  !
  ngh2=ngh
  !
  if (grid_mode >= 1) then
    ! --- Account for periodic boundary conditions: shift axes values ---
    if ((geo%bound_l(2) == bound_InternalPeriodic) .and. &
        (box%xb2(box%m2-1) >= box%xb2(box%m2))              ) then
      box%xc2(  box%m2-box%nghost2  :box%m2-1)= &
        box%xc2(box%m2-box%nghost2  :box%m2-1) - (geo%xb_r(2)-geo%xb_l(2))
      box%xb2(  box%m2-box%nghost2  :box%m2-1)= &
        box%xb2(box%m2-box%nghost2  :box%m2-1) - (geo%xb_r(2)-geo%xb_l(2))
    endif
    if ((geo%bound_r(2) == bound_InternalPeriodic) .and. &
        (box%xb2(box%n2+2) <= box%xb2(box%n2+1))            ) then
      box%xc2(  box%n2+1:box%n2+box%nghost2  )= &
        box%xc2(box%n2+1:box%n2+box%nghost2  ) + (geo%xb_r(2)-geo%xb_l(2))
      box%xb2(  box%n2+2:box%n2+box%nghost2+1)= &
        box%xb2(box%n2+2:box%n2+box%nghost2+1) + (geo%xb_r(2)-geo%xb_l(2))
    endif
  endif ! (grid_mode >= 1)
else
  ngh2=0
endif
!
if ((box%n3 > box%m3) .and. (ngh > 0)) then
  ! --- Start filling of ghost layers of model in x3 direction ---
  call rhd_box_MPIFillGhostStart(geo, box, 3, ngh, &
                                 box%m1-ngh1, box%n1+ngh1, &
                                 box%m2-ngh2, box%n2+ngh2, &
                                 box%m3     , box%n3     , &
                                 RequestsMPI=m0x3_RequestsMPI, &
                                 rhdMPI_boxGhostTypes=m0x3_rhdMPI_boxGhostTypes, &
                                 grid_mode=grid_mode, bulk_mode=bulk_mode, &
                                 rho_mode=rho_mode, ei_mode=ei_mode, v_mode=v_mode, &
                                 quc_mode=quc_mode)
  ! --- Finalize filling of ghost layers of model in x3 direction ---
  call rhd_box_MPIFillGhostEnd(m0x3_RequestsMPI, m0x3_rhdMPI_boxGhostTypes)
  !
  if (grid_mode >= 1) then
    ! --- Account for periodic boundary conditions: shift axes values ---
    if ((geo%bound_l(3) == bound_InternalPeriodic) .and. &
        (box%xb3(box%m3-1) >= box%xb3(box%m3))              ) then
      box%xc3(  box%m3-box%nghost3  :box%m3-1)= &
        box%xc3(box%m3-box%nghost3  :box%m3-1) - (geo%xb_r(3)-geo%xb_l(3))
      box%xb3(  box%m3-box%nghost3  :box%m3-1)= &
        box%xb3(box%m3-box%nghost3  :box%m3-1) - (geo%xb_r(3)-geo%xb_l(3))
    endif
    if ((geo%bound_r(3) == bound_InternalPeriodic) .and. &
        (box%xb3(box%n3+2) <= box%xb3(box%n3+1))            ) then
      box%xc3(  box%n3+1:box%n3+box%nghost3  )= &
        box%xc3(box%n3+1:box%n3+box%nghost3  ) + (geo%xb_r(3)-geo%xb_l(3))
      box%xb3(  box%n3+2:box%n3+box%nghost3+1)= &
        box%xb3(box%n3+2:box%n3+box%nghost3+1) + (geo%xb_r(3)-geo%xb_l(3))
    endif
  endif ! (grid_mode >= 1)
endif
!
end subroutine rhd_box_MPIFillGhost


!----------************************-----------------------------------------------------------------
subroutine rhd_box_MPIFillGhostAxes(action, geo, box, ngh1, ngh2, ngh3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_MPIFillGhostAxes ('rhd_box_MPI_Fill_Ghost-cells_Axes')
!
! PURPOSE:
!   Fill axes in ghost layers by communicating with neighbor MPI processes.
!
! CATEGORY:
!   MPI, hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_box_MPIFillGhostAxes(action, geo, box, ngh1, ngh2, ngh3)
!
! INPUT:
!   action:       ('action') derived type with collection of control parameters
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   ngh1:         ('number_ghost_1') integer with number of ghost layers to be transferred
!                 in i1 direction
!   ngh2:         ('number_ghost_2') integer with number of ghost layers to be transferred
!                 in i2 direction
!   ngh3:         ('number_ghost_3') integer with number of ghost layers to be transferred
!                 in i3 direction
!
! OUTPUT:
!
! INPUT/OUTPUT:
!   box:          ('box') type(box_type), computational box
!
! ROUTINES:
!   rhd_box_MPIFillGhostEnd: Finalize filling ghost cells
!   rhd_box_MPIFillGhostStart: Start filling "internal" ghost cells
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Via MPI in subroutines.
!
! RESTRICTIONS:
!   Axes of real, i.e., not internal, boundary layers are not filled.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-07-24 (B.F. Uppsala) Extracted from rhd_MPI_boxRead
!   2017-08-07 (B.F.) Correct only external periodic boundaries
!   2017-09-18 (B.F.) Rename rhd_boxMPI_FillGhostAxes -> rhd_box_MPIFillGhostAxes
!   2018-11-17 (B.F.) Use info from geo parameter and not from prop structure;
!                     Bugfix: correct shift condition for internal periodic boundaries
!---------------------------------------------------------------------------------------------------
use rhd_action_module
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(action_type),         intent(in)   :: action
type(geo_type),            intent(in)   :: geo
type(box_type),            intent(inout):: box
integer,                   intent(in)   :: ngh1, ngh2, ngh3
!
! --- Local variables ---
integer, dimension(1:4)                 :: m0x1_rhdMPI_boxGhostTypes, m0x1_RequestsMPI, &
                                           m0x2_rhdMPI_boxGhostTypes, m0x2_RequestsMPI, &
                                           m0x3_rhdMPI_boxGhostTypes, m0x3_RequestsMPI
!---------------------------------------------------------------------------------------------------
!
if (geo%MyProcIsInCOMM_flag) then
  ! --- Fill axes in ghost layers ------------------------------------------------------------------
  !
  !write(gl%nc_p, '(A,I3,A,4(1PE13.5),2X,4(1PE13.5))') '#', geo%MyProc, ' A: xb1,3:', &
  !  box%xb1(box%m1-box%nghost1),box%xb1(box%m1), box%xb1(box%n1+1),box%xb1(box%n1+1+box%nghost1), &
  !  box%xb3(box%m3-box%nghost3),box%xb3(box%m3), box%xb3(box%n3+1),box%xb3(box%n3+1+box%nghost3)
  !write(gl%nc_p, '(A,I3,A,4(1PE13.5),2X,4(1PE13.5))') '#', geo%MyProc, ' A: xc1,3:', &
  !  box%xc1(box%m1-box%nghost1),box%xc1(box%m1), box%xc1(box%n1  ),box%xc1(box%n1  +box%nghost1), &
  !  box%xc3(box%m3-box%nghost3),box%xc3(box%m3), box%xc3(box%n3  ),box%xc3(box%n3  +box%nghost3)
  !
  ! --- Start filling of ghost layers in all relevant directions ---
  ! --- Axes of real boundary layers are not filled!             ---
  if ((geo%n(1) > geo%m(1)) .and. (ngh1 > 0)) then
    call rhd_box_MPIFillGhostStart(geo, box, 1, ngh1, &
                                   box%m1, box%n1, box%m2, box%n2, box%m3, box%n3, &
                                   RequestsMPI=m0x1_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x1_rhdMPI_boxGhostTypes, &
                                   grid_mode=1, bulk_mode=0)
  endif ! (geo%n(1))
  if ((geo%n(2) > geo%m(2)) .and. (ngh2 > 0)) then
    call rhd_box_MPIFillGhostStart(geo, box, 2, ngh2, &
                                   box%m1, box%n1, box%m2, box%n2, box%m3, box%n3, &
                                   RequestsMPI=m0x2_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x2_rhdMPI_boxGhostTypes, &
                                   grid_mode=1, bulk_mode=0)
  endif ! (geo%n(2))
  if ((geo%n(3) > geo%m(3)) .and. (ngh3 > 0)) then
    call rhd_box_MPIFillGhostStart(geo, box, 3, ngh3, &
                                   box%m1, box%n1, box%m2, box%n2, box%m3, box%n3, &
                                   RequestsMPI=m0x3_RequestsMPI, &
                                   rhdMPI_boxGhostTypes=m0x3_rhdMPI_boxGhostTypes, &
                                   grid_mode=1, bulk_mode=0)
  endif ! (geo%n3)
  !
  if ((geo%n(1) > geo%m(1)) .and. (ngh1 > 0)) then
    ! --- Finalize transfer of x1-axes ghost values ---
    call rhd_box_MPIFillGhostEnd(m0x1_RequestsMPI, m0x1_rhdMPI_boxGhostTypes)
    !
    ! --- Account for periodic boundary conditions: shift axes values ---
    if ((action%bound_l1 == bound_InternalPeriodic) .and. &
        (box%xb1(box%m1-1) >= box%xb1(box%m1))              ) then
      box%xc1(  box%m1-box%nghost1  :box%m1-1)= &
        box%xc1(box%m1-box%nghost1  :box%m1-1) - (geo%xb_r(1)-geo%xb_l(1))
      box%xb1(  box%m1-box%nghost1  :box%m1-1)= &
        box%xb1(box%m1-box%nghost1  :box%m1-1) - (geo%xb_r(1)-geo%xb_l(1))
    endif
    if ((action%bound_r1 == bound_InternalPeriodic) .and. &
        (box%xb1(box%n1+2) <= box%xb1(box%n1+1))            ) then
      box%xc1(  box%n1+1:box%n1+box%nghost1  )= &
        box%xc1(box%n1+1:box%n1+box%nghost1  ) + (geo%xb_r(1)-geo%xb_l(1))
      box%xb1(  box%n1+2:box%n1+box%nghost1+1)= &
        box%xb1(box%n1+2:box%n1+box%nghost1+1) + (geo%xb_r(1)-geo%xb_l(1))
    endif
  endif ! (geo%n(1))
  !
  if ((geo%n(2) > geo%m(2)) .and. (ngh2 > 0)) then
    ! --- Finalize transfer of x2-axes ghost values ---
    call rhd_box_MPIFillGhostEnd(m0x2_RequestsMPI, m0x2_rhdMPI_boxGhostTypes)
    !
    ! --- Account for periodic boundary conditions: shift axes values ---
    if ((action%bound_l2 == bound_InternalPeriodic) .and. &
        (box%xb2(box%m2-1) >= box%xb2(box%m2))              ) then
      box%xc2(  box%m2-box%nghost2  :box%m2-1)= &
        box%xc2(box%m2-box%nghost2  :box%m2-1) - (geo%xb_r(2)-geo%xb_l(2))
      box%xb2(  box%m2-box%nghost2  :box%m2-1)= &
        box%xb2(box%m2-box%nghost2  :box%m2-1) - (geo%xb_r(2)-geo%xb_l(2))
    endif
    if ((action%bound_r2 == bound_InternalPeriodic) .and. &
        (box%xb2(box%n2+2) <= box%xb2(box%n2+1))            ) then
      box%xc2(  box%n2+1:box%n2+box%nghost2  )= &
        box%xc2(box%n2+1:box%n2+box%nghost2  ) + (geo%xb_r(2)-geo%xb_l(2))
      box%xb2(  box%n2+2:box%n2+box%nghost2+1)= &
        box%xb2(box%n2+2:box%n2+box%nghost2+1) + (geo%xb_r(2)-geo%xb_l(2))
    endif
  endif ! (geo%n(2))
  !
  if ((geo%n(3) > geo%m(3)) .and. (ngh3 > 0)) then
    ! --- Finalize transfer of x3-axes ghost values ---
    call rhd_box_MPIFillGhostEnd(m0x3_RequestsMPI, m0x3_rhdMPI_boxGhostTypes)
    !
    ! --- Account for periodic boundary conditions: shift axes values ---
    if ((action%bound_l3 == bound_InternalPeriodic) .and. &
        (box%xb3(box%m3-1) >= box%xb3(box%m3))              ) then
      box%xc3(  box%m3-box%nghost3  :box%m3-1)= &
        box%xc3(box%m3-box%nghost3  :box%m3-1) - (geo%xb_r(3)-geo%xb_l(3))
      box%xb3(  box%m3-box%nghost3  :box%m3-1)= &
        box%xb3(box%m3-box%nghost3  :box%m3-1) - (geo%xb_r(3)-geo%xb_l(3))
    endif
    if ((action%bound_r3 == bound_InternalPeriodic) .and. &
        (box%xb3(box%n3+2) <= box%xb3(box%n3+1))            ) then
      box%xc3(  box%n3+1:box%n3+box%nghost3  )= &
        box%xc3(box%n3+1:box%n3+box%nghost3  ) + (geo%xb_r(3)-geo%xb_l(3))
      box%xb3(  box%n3+2:box%n3+box%nghost3+1)= &
        box%xb3(box%n3+2:box%n3+box%nghost3+1) + (geo%xb_r(3)-geo%xb_l(3))
    endif
  endif ! (geo%n(3))
  !
  !write(gl%nc_p, '(A,I3,A,4(1PE13.5),2X,4(1PE13.5))') '#', geo%MyProc, ' Z: xb1,3:', &
  !  box%xb1(box%m1-box%nghost1),box%xb1(box%m1), box%xb1(box%n1+1),box%xb1(box%n1+1+box%nghost1), &
  !  box%xb3(box%m3-box%nghost3),box%xb3(box%m3), box%xb3(box%n3+1),box%xb3(box%n3+1+box%nghost3)
  !write(gl%nc_p, '(A,I3,A,4(1PE13.5),2X,4(1PE13.5))') '#', geo%MyProc, ' Z: xc1,3:', &
  !  box%xc1(box%m1-box%nghost1),box%xc1(box%m1), box%xc1(box%n1  ),box%xc1(box%n1  +box%nghost1), &
  !  box%xc3(box%m3-box%nghost3),box%xc3(box%m3), box%xc3(box%n3  ),box%xc3(box%n3  +box%nghost3)
  !
endif ! (geo%MyProcIsInCOMM_flag)
!
end subroutine rhd_box_MPIFillGhostAxes

!#
# endif
!#      /* rhd_mpi_l01 ========================================================================== */

end module rhd_box_module
