!***************************************************************************************************
!
!   #####   #    #  #####          #####     ##    #####
!   #    #  #    #  #    #         #    #   #  #     #
!   #    #  ######  #    #         #    #  #    #    #
!   #####   #    #  #    #         #    #  ######    #
!   #   #   #    #  #    #         #    #  #    #    #
!   #    #  #    #  #####  ####### #####   #    #    # _module
!
!   Routines for the collection of additional data
!
!***************************************************************************************************
!   Fortran90
!   2019-03-04
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!   Werner Schaffenberger: Exeter
!***************************************************************************************************
! MODULES:
!   rhd_dat_module:        Routines for the collection of additional data
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Number of additional quantities --- */
# ifndef rhd_box_quc01
#   define rhd_box_quc01 0
# endif
!#
!# /* --- Number of magnetic-field arrays --- */
# ifndef rhd_box_bmag01
#   define rhd_box_bmag01 0
# endif
!#
!# /* --- MPI flag activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------**************-------------------------------------------------------------------------------
module rhd_dat_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_module ('rhd_data_module')
!
! PURPOSE:
!   Handling routines of additional data for rhd.
!
! CATEGORY:
!   Hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_dat_module
!
! TYPES:
!   datx_type:    Substructure for additional RHD data (1D horizontal and radial means)
!     m:            ('m_number') integer, spatial start index
!     n:            ('number')   integer, spatial end index
!     nWave:        ('number_Wave') integer with number of wave components to store
!     na:           ('number_array') number of averaged quantities in array
!     na1:          ('number_array_1') number of 1D quantities (axes) in array
!   datc_type:    Substructure for additional RHD data (2D cylindrical means)
!   dat_type:     Structure to contain additional RHD data
!
! VARIABLES:
!   None
!
! ROUTINES: (contained)
!   rhd_dat_SwitchInfo:    Print information about used compiler switches
!   rhd_datx_SetIndices:   Set indices for quantities in structure for 1D mean data
!   rhd_datx_Make:         Create structure for data of requested size
!   rhd_datx_Reset:        Set fields in structure for additional data to zero
!   rhd_datx_Add:          Add the fields in structure for horizontal/radial mean data
!   rhd_datx_AddOverlap:   Add the overlap of fields in structure for horizontal/radial mean data
!   rhd_datx_Axis:         Construct radial axis for datx
!   rhd_datx_Delete:       Delete structure for additional data. Deallocate memory
!   rhd_datx_Copy:         Copy data structure
!   rhd_datc_SetIndices:   Set indices for quantities in structure for 2D cylindrical mean data
!   rhd_datc_Make:         Create structure of requested size for 2D cylindrical mean data
!   rhd_datc_Reset:        Set fields in structure for cylindrical mean data to zero
!   rhd_datc_Add:          Add the fields in the structure for cylindrical mean data
!   rhd_datc_AddOverlap:   Add the overlap of fields in the structure for cylindrical mean data
!   rhd_datc_Delete:       Delete structure for cylindrical mean data. Deallocate memory
!   rhd_datc_Copy:         Copy data structure
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_dat_Add:           Add contents of two structures for additional data
!   rhd_dat_AddOverlap:    Add overlap of contents of two structures for additional data
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_datx_Write:        Write additional RHD data into file in UIO form
!   rhd_datc_Write:        Write additional RHD data into file in UIO form
!   rhd_dat_Write:         Write additional RHD data into file in UIO form
!   --- MPI-only routines ---
!   rhd_dat_MPICreateType: Create MPI data type for dat structure
!   rhd_datx_MPICollectx:  Collect distributed horizontal-mean data into single structure
!   rhd_datx_MPICollectr:  Collect distributed spherical-mean data into single structure
!   rhd_datc_MPICollect:   Collect distributed azimuthal-mean data into single structure
!   rhd_dat_MPICollectInt: Collect distributed intensities from MPI processes into single structure
!   rhd_dat_MPICollectFer: Collect distributed rad. fluxes from MPI processes into single structure
!   rhd_dat_MPICollect:    Collect distributed data from MPI processes into single structure
!
! PROCEDURE:
!   To add another quantity to datx:
!     1) Mention the new quantity in the history of the module
!     2) Put a suitable index name into the datx_type definition
!     3) Set the index to its appropriate value in rhd_datx_SetIndices (to -1 if not used)
!     4) Make sure that the quantity is actually computed somewhere
!     5) Write (after scaling, if necessary) the result into a file in rhd_datx_Write
!   To add another quantity to datc:
!     1) Mention the new quantity in the history of the module
!     2) Put a suitable index name into the datc_type definition
!     3) Set the index to its appropriate value in rhd_datc_SetIndices
!     4) Make sure that the quantity is actually computed somewhere
!     5) Write the result into a file in rhd_datc_Write
!
! TODO:
!   More quantities: drhoeir, phi, source_rhov, source_rhoei
!   Viscosity in 2D and 3D: fev1, fev2, radial case
!
! MODIFICATION HISTORY:
!   1999-08-20 (B.F. Copenhagen) Written
!   1999-12-08 (B.F.) Variables for intensities added
!   1999-12-26 (B.F.) luminosity
!   2000-01-25 (B.F.) Frad1,2,3_xmean_tmean -> Frad1,2,3_xmean; type datx_type added
!   2000-01-28 (B.F.) Axes xc and xb in structure datx added, fluxes in datx handled
!   2000-02-08 (B.F.) Averaging over spheres ready
!   2001-01-03 (B.F. Uppsala) Entry fev_xmean added
!   2001-03-18 (B.F.) Entries ei_xmean, s_xmean added
!   2001-07-02 (B.F.) Entries T4_xmean, T4kaprho_xmean added
!   2001-09-05 (B.F.) Entries v1_xmean, v2_xmean, v3_xmean added
!   2002-11-09 (B.F.) Entry quc_xmean added
!   2003-08-11 (B.F.) Entry Intxx3_r added
!   2004-09-20 (B.F.) Entry Intxx3_r removed: outcommented
!   2007-12-15 (B.F. Lyon) Entry fquc_xmean added
!   2008-02-26 (B.F.) Entry T2_xmean added
!   2008-02-29 (B.F.) Entries p2_xmean, rho2_xmean, s2_xmean added
!   2010-02-22 (B.F.) Entries Bc1abs_xmean, Bc2abs_xmean, Bc3abs_xmean added
!   2010-12-10 (W.S. Exeter) Entries Bb_xmean, Bbabs_xmean, Bb2_xmean, 
!                                    Bh_xmean, Bhrms_xmean, fem_xmean added
!   2011-01-27 (W.S.) Entry fey_xmean added
!   2012-09-19 (B.F.) Entries v1v1v1_xmean, v1v1v1v1_xmean added
!   2012-11-28 (B.F.) Start putting all averaged into one 2D array; datp_type
!   2011-12-01 (B.F.) Indices moved from datp to datx
!   2013-07-08 (B.F.) New entries MomIne3_xmean, AngMom3_xmean
!   2013-07-30 (B.F.) New entry v1upfrac_xmean
!   2013-07-31 (B.F.) New data type datc for cylindrical (azimuthal) averages
!   2013-08-20 (B.F.) Azimuthal averages usable
!   2014-02-08 (B.F. Uppsala) rhd_dat_module.f90 ->  rhd_dat_module.F90
!   2015-03-21 (B.F.) Give the mean data array its own kind
!   2017-08-13 (B.F.) Allow output of intensity and flux maps under MPI
!   2017-08-18 (B.F. Lyon) Handle axes in datx structure in a separate array
!   2017-09-11 (B.F. Uppsala) Add routine to collect mean data from MPI processes
!   2018-05-07 (B.F.) Remove data structure, output routine, and branches for old averging routines
!   2018-05-31 (B.F.) New entries in datx for scaling and testing: i_runiflux, i_runilum
!   2018-06-02 (B.F.) New entry in datx: i_rhotil
!   2018-06-12 (B.F.) Store (non-)radial wave amplitudes of mass flux; new entries nWave, i_frhowave
!   2018-06-21 (B.F.) New entry in datx: i_frhofrho
!   2018-10-26 (B.F.) Store (non-)radial wave amplitudes of magnetic field; new entry i_Bbwave
!   2019-01-29 (B.F.) Rename and reorganize MPI collect routines
!   2019-01-30 (B.F.) Add indices for dimension of radiative surface fluxes to dat structure
!   2019-02-06 (B.F.) New entry in datx: i_der
!   2019-02-06 (B.F.) New entry in datx: i_relsurfrad
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- Define type for 1D mean data (horizontal and spherical) ---
type datx_type
  integer                               :: m, n, nWave
  integer                               :: na, na1, &
                                           i_xc, &
                                           i_relvol, &
                                           i_rho, i_rho2, &
                                           i_v1, i_v2, i_v3, &
                                           i_v1v1, i_v2v2, i_v3v3, &
                                           i_v1v1v1, i_v1v1v1v1, &
                                           i_v1upfrac, &
                                           i_rhov1, i_rhov2, i_rhov3, &
                                           i_rhov1v1, i_rhov2v2, i_rhov3v3, &
                                           i_rhov1v2, i_rhov1v3, i_rhov2v3, &
                                           i_ei, i_rhoei, i_rhoek, i_rhoeg, &
                                           i_p, i_p2, &
                                           i_T, i_T2, i_T4, i_T4kaprho, &
                                           i_s, i_s2, i_rhos, &
                                           i_Gamma1, i_Gamma3, i_delta, &
                                           i_MomIne3, i_AngMom3, &
                                           i_kapparho, i_grad, &
                                           i_devp2p, i_der, &
                                           i_Bc1, i_Bc2, i_Bc3, &
                                           i_Bc1abs, i_Bc2abs, i_Bc3abs, &
                                           i_Bc12, i_Bc22, i_Bc32, &
                                           i_Bh, i_Bhrms, &
                                           i_Bb, i_Bbabs, i_Bb2, &
                                           i_quc, i_fquc, &
                                           i_xb, &
                                           i_relsurf, i_relsurfrad, &
                                           i_runiflux, i_runilum, &
                                           i_rhotil, i_frho, i_frhofrho, &
                                           i_frhov1, i_frhov2, i_frhov3, &
                                           i_feip, i_fek, i_feg, i_fep, &
                                           i_fev, i_fer, i_fem, i_fey, &
                                           i_frhowave, i_Bbwave
  real, pointer, dimension(:,:)         :: d1
  real(kind=mArk), &
        pointer, dimension(:,:)         :: d
end type datx_type
!
! --- Define type for 2D cylindrical mean data ---
type datc_type
  integer                               :: m1, n1, m2, n2, &
                                           na1, na, &
                                           i_xb, i_xc, &
                                           i_relvol, &
                                           i_rho, i_ei, i_rhoei, &
                                           i_v1, i_v2, i_v3, &
                                           i_v1v1, i_v2v2, i_v3v3, &
                                           i_rhov1, i_rhov2, i_rhov3, &
                                           i_rhov1v1, i_rhov2v2, i_rhov3v3, &
                                           i_rhov1v2, i_rhov1v3, i_rhov2v3, &
                                           i_T, i_P, i_Gamma1, i_Gamma3, i_delta, &
                                           i_s, i_ss, i_rhos, &
                                           i_feip1, i_feip3, &
                                           i_fep1, i_fep3, &
                                           i_fek1, i_fek3, &
                                           i_quc, i_fquc1, i_fquc2, i_fquc3, &
                                           i_Bc1, i_Bc2, i_Bc3, &
                                           i_fer1, i_fer2, i_fer3
  !                                        i_fev1/2, i_r2rho...
  real, pointer, dimension(:,:)         :: d1
  real, pointer, dimension(:,:,:)       :: d
end type datc_type
!
! --- Define container type for all mean data ---
type dat_type
  integer                               :: m1,n1, m2,n2, m3,n3, &
                                           mr1,nr1, mr2,nr2, mr3,nr3
  integer                               :: itime
  real                                  :: time, luminosity
  real, pointer, dimension(:,:)         :: fer1_l, fer1_r, fer2_l, fer2_r, fer3_l, fer3_r, &
                                           Int1_l, Int1_r, Int2_l, Int2_r, Int3_l, Int3_r
  real, pointer, dimension(:,:,:)       :: Intxx3_r
  type(datx_type)                       :: x12, x13, x23, r
  type(datc_type)                       :: c
end type dat_type
!
interface assignment (=)
  module procedure rhd_datx_Copy
end interface ! assignment (=)
!
interface assignment (=)
  module procedure rhd_datc_Copy
end interface ! assignment (=)
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_dat_SwitchInfo(ncp)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_SwitchInfo ('rhd_dat_Switch_Information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   I/O
!
! CALLING SEQUENCE:
!   call rhd_dat_SwitchInfo(ncp)
!
! INPUT:
!   ncp:          ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Write to specified output channel.
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!   call rhd_dat_SwitchInfo(6)
!
! MODIFICATION HISTORY:
!   2002-10-12 (B.F.) Written
!   2014-02-08 (B.F.) rhd_dat_SwitchInfo
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: ncp
!
! --- Local variables ---
!character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!---------------------------------------------------------------------------------------------------
!
write(ncp,'(A)')    'Compiler switches: rhd_dat_module .........................'
!
write(ncp,'(A,I6)') '  rhd' // '_box_quc01:              ', rhd_box_quc01
write(ncp,'(A,I6)') '  rhd' // '_box_bmag01:             ', rhd_box_bmag01
write(ncp,'(A,I6)') '  rhd' // '_mpi_l01:                ', rhd_mpi_l01
!
end subroutine rhd_dat_SwitchInfo


!----------*******************----------------------------------------------------------------------
subroutine rhd_datx_SetIndices(datx, nWave, radial_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_SetIndices ('rhd_data_x_Set_Indices')
!
! PURPOSE:
!   Set indices for quantities in structure for 1D mean data.
!   
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_SetIndices(datx, nWave, radial_flag)
!
! INPUT:
!   nWave:        ('number_Wave') integer with number of wave components to store
!   radial_flag:  ('radial_flag') logical, if set the structure should contain also
!                 axis values
!
! OUTPUT:
!   datx:         ('data_x') type(datx_type), structure with additional output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Set indices for individual quantities in large array.
!
! EXAMPLE:
!   call rhd_datx_SetIndices(datx)
!
! MODIFICATION HISTORY:
!   2013-08-12 (B.F. Lyon) Extracted from rhd_datx_Make
!   2017-08-14 (B.F. Uppsala) Define all indices
!   2017-08-18 (B.F. Lyon) Handle axes in datx structure in a separate array
!   2018-06-12 (B.F. Uppsala) New parameter nWave
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(inout):: datx
integer,                   intent(in)   :: nWave
logical,                   intent(in)   :: radial_flag
!
! --- Local variables ---
integer                                 :: i
!---------------------------------------------------------------------------------------------------
!
! === Handle axes ===
!
! --- Set all indices to a dummy value outside the array range ---
datx%i_xc        =-1
datx%i_xb        =-1
!
! --- For radial data, axes are stored ---
i=0
if (datx%n > datx%m) then
  if (radial_flag) then
    i=i+1 ; datx%i_xc     =i
    i=i+1 ; datx%i_xb     =i
  endif
endif ! (datx%n > datx%m)
datx%na1=i
!
! === Handle quantities to be averaged ===
! --- Give all activated quantities an index in the array, leave the others at -1 ---
!
! --- Set all indices to a dummy value outside the array range ---
datx%nWave       =0
!
datx%i_relvol    =-1
!
datx%i_rho       =-1
datx%i_rho2      =-1
datx%i_v1        =-1
datx%i_v2        =-1
datx%i_v3        =-1
datx%i_v1v1      =-1
datx%i_v2v2      =-1
datx%i_v3v3      =-1
datx%i_v1v1v1    =-1
datx%i_v1v1v1v1  =-1
datx%i_v1upfrac  =-1
datx%i_rhov1     =-1
datx%i_rhov2     =-1
datx%i_rhov3     =-1
datx%i_rhov1v1   =-1
datx%i_rhov2v2   =-1
datx%i_rhov3v3   =-1
datx%i_rhov1v2   =-1
datx%i_rhov1v3   =-1
datx%i_rhov2v3   =-1
datx%i_ei        =-1
datx%i_rhoei     =-1
datx%i_rhoek     =-1
datx%i_rhoeg     =-1
datx%i_p         =-1
datx%i_p2        =-1
datx%i_T         =-1
datx%i_T2        =-1
datx%i_T4        =-1
datx%i_T4kaprho  =-1
datx%i_s         =-1
datx%i_s2        =-1
datx%i_rhos      =-1
datx%i_Gamma1    =-1
datx%i_Gamma3    =-1
datx%i_delta     =-1
datx%i_MomIne3   =-1
datx%i_AngMom3   =-1
datx%i_kapparho  =-1
datx%i_grad      =-1
datx%i_devp2p    =-1
datx%i_der       =-1
!
datx%i_Bc1       =-1
datx%i_Bc2       =-1
datx%i_Bc3       =-1
datx%i_Bc1abs    =-1
datx%i_Bc2abs    =-1
datx%i_Bc3abs    =-1
datx%i_Bc12      =-1
datx%i_Bc22      =-1
datx%i_Bc32      =-1
datx%i_Bh        =-1
datx%i_Bhrms     =-1
!
datx%i_Bb        =-1
datx%i_Bbabs     =-1
datx%i_Bb2       =-1
datx%i_fem       =-1
datx%i_fey       =-1
!
datx%i_quc       =-1
!
datx%i_fquc      =-1
!
datx%i_relsurf   =-1
datx%i_relsurfrad=-1
!
datx%i_runiflux  =-1
datx%i_runilum   =-1
!
datx%i_rhotil    =-1
datx%i_frho      =-1
datx%i_frhofrho  =-1
datx%i_frhov1    =-1
datx%i_frhov2    =-1
datx%i_frhov3    =-1
datx%i_feip      =-1
datx%i_fek       =-1
datx%i_feg       =-1
datx%i_fep       =-1
datx%i_fev       =-1
datx%i_fer       =-1
!
datx%i_frhowave  =-1
datx%i_Bbwave    =-1
!
! --- Give all activated quantities an index in the array, leave the others at -1 ---
datx%nWave=max(nWave, 0)
!
i=0
if (datx%n > datx%m) then
  if (radial_flag) then
    i=i+1 ; datx%i_relvol    =i
    !
    i=i+1 ; datx%i_relsurf   =i
    i=i+1 ; datx%i_relsurfrad=i
    !
    i=i+1 ; datx%i_runiflux  =i
    i=i+1 ; datx%i_runilum   =i
  endif
  !
  i=i+1 ; datx%i_rho     =i
  i=i+1 ; datx%i_rho2    =i
  i=i+1 ; datx%i_v1      =i
  i=i+1 ; datx%i_v2      =i
  i=i+1 ; datx%i_v3      =i
  i=i+1 ; datx%i_v1v1    =i
  i=i+1 ; datx%i_v2v2    =i
  i=i+1 ; datx%i_v3v3    =i
  i=i+1 ; datx%i_v1v1v1  =i
  i=i+1 ; datx%i_v1v1v1v1=i
  i=i+1 ; datx%i_v1upfrac=i
  i=i+1 ; datx%i_rhov1   =i
  i=i+1 ; datx%i_rhov2   =i
  i=i+1 ; datx%i_rhov3   =i
  i=i+1 ; datx%i_rhov1v1 =i
  i=i+1 ; datx%i_rhov2v2 =i
  i=i+1 ; datx%i_rhov3v3 =i
  i=i+1 ; datx%i_rhov1v2 =i
  i=i+1 ; datx%i_rhov1v3 =i
  i=i+1 ; datx%i_rhov2v3 =i
  i=i+1 ; datx%i_ei      =i
  i=i+1 ; datx%i_rhoei   =i
  i=i+1 ; datx%i_rhoek   =i
  i=i+1 ; datx%i_rhoeg   =i
  i=i+1 ; datx%i_p       =i
  i=i+1 ; datx%i_p2      =i
  i=i+1 ; datx%i_T       =i
  i=i+1 ; datx%i_T2      =i
  i=i+1 ; datx%i_T4      =i
  i=i+1 ; datx%i_T4kaprho=i
  i=i+1 ; datx%i_s       =i
  i=i+1 ; datx%i_s2      =i
  i=i+1 ; datx%i_rhos    =i
  i=i+1 ; datx%i_Gamma1  =i
  i=i+1 ; datx%i_Gamma3  =i
  i=i+1 ; datx%i_delta   =i
  i=i+1 ; datx%i_MomIne3 =i
  i=i+1 ; datx%i_AngMom3 =i
  i=i+1 ; datx%i_kapparho=i
  i=i+1 ; datx%i_grad    =i
  i=i+1 ; datx%i_devp2p  =i
  i=i+1 ; datx%i_der     =i
  !
  i=i+1 ; datx%i_rhotil  =i
  i=i+1 ; datx%i_frho    =i
  i=i+1 ; datx%i_frhofrho=i
  i=i+1 ; datx%i_frhov1  =i
  i=i+1 ; datx%i_frhov2  =i
  i=i+1 ; datx%i_frhov3  =i
  i=i+1 ; datx%i_feip    =i
  i=i+1 ; datx%i_fek     =i
  i=i+1 ; datx%i_feg     =i
  i=i+1 ; datx%i_fep     =i
  i=i+1 ; datx%i_fev     =i
  i=i+1 ; datx%i_fer     =i
  !
  if (datx%nWave > 0) then
    i=i+1 ; datx%i_frhowave=i ; i=i+datx%nWave-1
  endif
  !
!#
# if (rhd_box_quc01 != 0)
!#
  if (prop%nquc > 0) then
    ! --- Set array indices for additional densities ---
    i=i+1 ; datx%i_quc     =i ; i=i+prop%nquc-1
    i=i+1 ; datx%i_fquc    =i ; i=i+prop%nquc-1
  endif ! (prop%nquc > 0)
!#
# endif
!#      /* rhd_box_quc01 */
  !
!#
# if (rhd_box_bmag01 != 0)
!#
  if (prop%nBb > 0) then
    ! --- Set array indices for magnetic fields ---
    i=i+1 ; datx%i_Bc1     =i
    i=i+1 ; datx%i_Bc2     =i
    i=i+1 ; datx%i_Bc3     =i
    i=i+1 ; datx%i_Bc1abs  =i
    i=i+1 ; datx%i_Bc2abs  =i
    i=i+1 ; datx%i_Bc3abs  =i
    i=i+1 ; datx%i_Bc12    =i
    i=i+1 ; datx%i_Bc22    =i
    i=i+1 ; datx%i_Bc32    =i
    i=i+1 ; datx%i_Bh      =i
    i=i+1 ; datx%i_Bhrms   =i
    !
    i=i+1 ; datx%i_Bb      =i
    i=i+1 ; datx%i_Bbabs   =i
    i=i+1 ; datx%i_Bb2     =i
    i=i+1 ; datx%i_fem     =i
    i=i+1 ; datx%i_fey     =i
    !
    if (datx%nWave > 0) then
      i=i+1 ; datx%i_Bbwave=i ; i=i+datx%nWave-1
    endif
  endif ! (prop%nBb > 0)
!#
# endif
!#      /* rhd_box_bmag01 */
endif ! (datx%n > datx%m)
!
datx%na=i
!
end subroutine rhd_datx_SetIndices


!----------*************----------------------------------------------------------------------------
subroutine rhd_datx_Make(datx, m, n, nWave, radial_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Make ('rhd_data_x_Make')
!
! PURPOSE:
!   Create structure of requested size for 1D mean data.
!   
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_Make(datx, m, n, nWave, radial_flag=radial_flag)
!
! INPUT:
!   m:            ('m_number') integer, spatial start index
!   n:            ('number')   integer, spatial end index
!   nWave:        ('number_Wave') integer with number of wave components to store
!   radial_flag:  ('radial_flag') optional logical, if set the structure should contain also
!                 axis values
!
! OUTPUT:
!   datx:         ('data_x') type(datx_type), structure with additional output data
!
! ROUTINES:
!   rhd_datx_Reset:        Set fields in structure for additional data to zero.
!   rhd_datx_SetIndices:   Set indices for quantities in structure for 1D mean data
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Pointers are allocated.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Allocate pointer arrays of proper size (or nullify them).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-01-25 (B.F. Copenhagen) Written
!   2001-03-13 (B.F. Uppsala) Special version for OpenMP on SGI
!   2002-11-09 (B.F.) User rhd_prop_module and allocate quc_xmean
!   2002-11-29 (B.F.) Nullify not used quc and Bc arrays
!   2008-03-07 (B.F. Lyon) Nullify datx%fquc_xmean
!   2010-04-20 (B.F.) Rename rhd_dat_Makex -> rhd_datx_Make
!   2012-11-21 (W.S.) Bugfix: allocation for datx%Bb_xmean corrected
!   2012-11-28 (B.F.) Start putting all averaged into one 2D array; datp_type
!   2011-12-01 (B.F.) Indices moved from datp to datx
!   2013-07-31 (B.F.) Create dat%r entries only when needed
!   2017-09-14 (B.F. Uppsala) Don't allocate the arrays in the old format
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!   2018-06-12 (B.F. Uppsala) New parameter nWave
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m, n, nWave
type(datx_type),           intent(out)  :: datx
logical,         optional, intent(in)   :: radial_flag
!
! --- Local variables ---
logical                                 :: radial_flag0
real, dimension(:)    , pointer         :: xx
real(kind=mArk), &
      dimension(:)    , pointer         :: x
real(kind=mArk), &
      dimension(:,:)  , pointer         :: x2
!---------------------------------------------------------------------------------------------------
!
if (present(radial_flag)) then
  radial_flag0=radial_flag
else
  radial_flag0=.false.
endif
!
datx%m=m
datx%n=n
!
datx%na=0
call rhd_datx_SetIndices(datx, nWave, radial_flag0)
if (datx%na > 0) then
  allocate(datx%d(datx%m:datx%n+1, datx%na))
else
  nullify(datx%d)
endif
if (datx%na1 > 0) then
  allocate(datx%d1(datx%m:datx%n+1, datx%na1))
else
  nullify(datx%d1)
endif
!
call rhd_datx_Reset(datx)
!
end subroutine rhd_datx_Make


!----------**************---------------------------------------------------------------------------
subroutine rhd_datx_Reset(datx)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Reset ('rhd_data_x_Reset')
!
! PURPOSE:
!   Set fields in structure for additional data to zero.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_Reset(datx)
!
! INPUT:
!
! INPUT/OUTPUT:
!   datx:         ('data_x') type(datx_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Structure datx has to be initialized (with rhd_datx_Make).
!
! PROCEDURE:
!   Just set all associated arrays to zero.
!
! EXAMPLE:
!   call rhd_datx_Reset(datx)
!
! MODIFICATION HISTORY:
!   2000-01-25 (B.F. Copenhagen) Written
!   2010-04-20 (B.F. Lyon) Rename reset_datx -> rhd_datx_Reset
!   2012-11-28 (B.F.) Reset single large 2D array
!   2013-08-15 (B.F.) Don't reset axes xc and xb
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!---------------------------------------------------------------------------------------------------
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(inout):: datx
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- Set the data array for the averaged quantities to zero and leave the axis intact ---
if (datx%na > 0) datx%d=0.0_mArk
!
end subroutine rhd_datx_Reset


!----------************-----------------------------------------------------------------------------
subroutine rhd_datx_Add(datx1, datx2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Add ('rhd_data_x_Add')
!
! PURPOSE:
!   Add the fields in structure for horizontal/radial mean data.
!
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_Add(datx1, datx2)
!
! INPUT:
!   datx2:        ('data_x_2') type(datx_type), structure for additional data
!
! INPUT/OUTPUT:
!   datx1:        ('data_x_1') type(datx_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Both datx1 and datx2 have to be initialized (with rhd_datx_Make).
!   The array sizes have to agree.
!
! PROCEDURE:
!   "datx1 = datx1 + datx2"
!
! EXAMPLE:
!   call rhd_datx_Add(datx1, datx2)
!
! MODIFICATION HISTORY:
!   2001-01-01 (B.F. Kiel) Written
!   2010-04-20 (B.F. Lyon) Rename add_datx -> rhd_datx_Add
!   2012-11-28 (B.F.) Add large 2D arrays
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(inout):: datx1
type(datx_type),           intent(in)   :: datx2
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if ((datx1%na > 0) .and. (datx2%na > 0)) datx1%d=datx1%d+datx2%d
!
end subroutine rhd_datx_Add


!----------*******************----------------------------------------------------------------------
subroutine rhd_datx_AddOverlap(datx1, datx2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_AddOverlap ('rhd_data_x_Add_Overlap')
!
! PURPOSE:
!   Add the overlap of fields in structure for horizontal/radial mean data.
!
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_AddOverlap(datx1, datx2)
!
! INPUT:
!   datx2:        ('data_x_2') type(datx_type), structure for additional data
!
! INPUT/OUTPUT:
!   datx1:        ('data_x_1') type(datx_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Both datx1 and datx2 have to be initialized (with rhd_datx_Make).
!
! PROCEDURE:
!   "In intersection of datx1 and datx2: datx1 = datx1 + datx2"
!
! EXAMPLE:
!   call rhd_datx_AddOverlap(datx1, datx2)
!
! MODIFICATION HISTORY:
!   2001-01-01 (B.F. Kiel) Written
!   2010-04-20 (B.F. Lyon) Rename add_datx -> rhd_datx_Add
!   2012-11-28 (B.F.) Add large 2D arrays
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!   2019-01-31 (B.F. Uppsala) Derived from rhd_datx_Add
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(inout):: datx1
type(datx_type),           intent(in)   :: datx2
!
! --- Local variables ---
integer                                 :: m, n
!---------------------------------------------------------------------------------------------------
!
! --- Find the intersection of the arrays ---
m=max(datx1%m, datx2%m)
n=min(datx1%n, datx2%n)
!
if ((n >= m) .and. (datx1%na > 0) .and. (datx1%na == datx2%na)) then
  ! --- Add the intersection if non-empty ---
  datx1%d(m:n+1, 1:datx1%na) = datx1%d(m:n+1, 1:datx1%na) +  &
                               datx2%d(m:n+1, 1:datx2%na)
endif
!
end subroutine rhd_datx_AddOverlap


!----------*************----------------------------------------------------------------------------
subroutine rhd_datx_Axis(datr, box)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Axis ('rhd_data_x_Axis')
!
! PURPOSE:
!   Construct radial axis for datr.
!   
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_Axis(datr, box=box)
!
! INPUT:
!   box:          ('box') type(box_type), optional, computational box with bulk data
!   Global_flag:  ('Global_flag') optional logical to indicate that global (from prop structure)
!                 instead of local (from input box) indices should be used; currently ignored
!
! OUTPUT:
!   datr:         ('data_radial') type(datx_type), structure with additional output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The requested number of axis points (in datr%n) should match the axis-generation formula.
!   The current formula works for cubical grids and should make sense for non-cubical grids,
!   although in this case an adaption might be necessary to get optimal results.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_datx_Axis(datr, box)
!
! TODO:
!   Use higher-order interpolation for generation of xc axis.
!
! MODIFICATION HISTORY:
!   2000-02-02 (B.F. Copenhagen) Written
!   2000-04-14 (B.F. Potsdam) Check if xc and xb are associated
!   2001-09-05 (B.F. Uppsala) Increase extent of radial grid
!   2010-04-20 (B.F. Lyon) Rename axis_datx -> rhd_datx_Axis
!   2017-08-13 (B.F. Uppsala) Add keyword Global_flag; rename input parameter datx -> datr
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!   2019-03-04 (B.F.) Ignore keyword Global_flag and construct always global axis;
!                     construct (possibly non-equidistant) axis only from prop%xb1
!---------------------------------------------------------------------------------------------------
use rhd_box_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(inout):: datr
type(box_type),  optional, intent(in)   :: box
!logical,         optional, intent(in)   :: Global_flag
!
! --- Local variables ---
integer                                 :: i, i1
real                                    :: xfactor
!!$logical                                 :: Global_flag0
!---------------------------------------------------------------------------------------------------
!
! --- Set default values for optional parameters ---
!!$if (present(Global_flag)) then
!!$  Global_flag0=Global_flag
!!$else
!!$  Global_flag0=.true.
!!$endif
!
if (datr%n == datr%m) then
  ! --- The axis range is degenerated: fill the axis with dummy values ---
  !
  if (datr%na1 > 0) then
    if (datr%i_xb > 0) then
      datr%d1(:,datr%i_xb)=(/0.0,1.0/)
      if (datr%i_xc > 0) datr%d1(:,datr%i_xc)=0.5
    endif
  endif
  !
else ! (datr%n /= datr%m)
  ! --- Construct radial axis ---
  !
!!$  xfactor=huge(xfactor)
!!$  !
!!$  ! if (Global_flag0) then
!!$    ! --- Base axis on data in global prop structure ---
!!$    if (prop%n1 > prop%m1) xfactor=min(xfactor, prop%xb1(prop%n1+1)-prop%xb1(prop%m1))
!!$    if (prop%n2 > prop%m2) xfactor=min(xfactor, prop%xb2(prop%n2+1)-prop%xb2(prop%m2))
!!$    if (prop%n3 > prop%m3) xfactor=min(xfactor, prop%xb3(prop%n3+1)-prop%xb3(prop%m3))
!!$  ! else
!!$  !   ! --- Base axis on data in input box structure ---
!!$  !   if (box%n1 > box%m1) xfactor=min(xfactor, box%xb1(box%n1+1)-box%xb1(box%m1))
!!$  !   if (box%n2 > box%m2) xfactor=min(xfactor, box%xb2(box%n2+1)-box%xb2(box%m2))
!!$  !   if (box%n3 > box%m3) xfactor=min(xfactor, box%xb3(box%n3+1)-box%xb3(box%m3))
!!$  ! endif
!!$  !
!!$  ! xfactor=1.2 * 0.5 * xfactor / real(datr%n-datr%m+1)
!!$  xfactor=1.5 * 0.5 * xfactor / real(datr%n-datr%m+1)
!!$  !
!!$  if (datr%na1 > 0) then
!!$    if (datr%i_xb > 0) then
!!$      datr%d1(:,datr%i_xb)=xfactor * real((/(i,i=0,datr%n-datr%m+1)/))
!!$      if (datr%i_xc > 0) then
!!$        datr%d1(datr%m:datr%n,datr%i_xc)=0.5 * (datr%d1(datr%m  :datr%n  ,datr%i_xb) + &
!!$                                                datr%d1(datr%m+1:datr%n+1,datr%i_xb)    )
!!$      endif
!!$    endif
!!$  endif
  !
  xfactor=1.5
  !
  if (datr%na1 > 0) then
    if (datr%i_xb > 0) then
      ! --- Create radial xb axis from right half of global xb1 axis: prop%xb1(0:prop%n1) ---
      !
      ! --- Set first point to zero ---
      datr%d1(datr%m, datr%i_xb) = 0.0
      !
      ! --- Fill radial axis with scaled values of half of the grid ---
      do i1=1,prop%n1+1
        datr%d1(2*(i1-prop%n1-1)+datr%n+1, datr%i_xb) = xfactor*prop%xb1(i1)
      enddo ! i1
      !
      ! --- Interpolate the intermediate values ---
      do i=datr%n,datr%m+1,-2
        datr%d1(i, datr%i_xb) = 0.5 * (datr%d1(i-1, datr%i_xb)+datr%d1(i+1, datr%i_xb))
      enddo ! i
      !
      if (datr%i_xc > 0) then
        ! --- Interpolate radial xc axis from radial xb axis ---
        datr%d1(datr%m:datr%n, datr%i_xc)=0.5 * (datr%d1(datr%m  :datr%n  , datr%i_xb) + &
                                                 datr%d1(datr%m+1:datr%n+1, datr%i_xb)    )
      endif
      !
    endif ! (datr%i_xb > 0)
  endif ! (datr%na1 > 0)
  !
endif ! (datr%n == datr%m)
!
end subroutine rhd_datx_Axis


!----------***************--------------------------------------------------------------------------
subroutine rhd_datx_Delete(datx)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Delete ('rhd_data_x_Delete')
!
! PURPOSE:
!   Delete structure for additional data. Deallocate memory.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_Delete(datx)
!
! INPUT:
!
! INPUT/OUTPUT:
!   datx:         ('data_x') type(datx_type), structure for additional data
!
! ROUTINES:
!   rhd_datx_SetIndices:   Set indices for quantities in structure for 1D mean data
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Pointers are deallocated
!
! RESTRICTIONS:
!   Structure datx has to be initialized (with rhd_datx_Make).
!
! PROCEDURE:
!   Deallocate all associated pointer arrays in input structure and set some counters to zero
!
! EXAMPLE:
!   call rhd_datx_Delete(datx)
!
! MODIFICATION HISTORY:
!   2000-01-25 (B.F. Copenhagen) Written
!   2012-11-28 (B.F.) Deallocate single large 2D array
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(inout):: datx
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
datx%m=0
datx%n=0
!
if (datx%na > 0) then
  deallocate(datx%d)
  datx%na=0
endif
if (datx%na1 > 0) then
  deallocate(datx%d1)
  datx%na1=0
endif
!if (.not. (datx%i_relvol > 0)) call rhd_datx_SetIndices(datx, 0, (datx%i_relvol > 0))
!
end subroutine rhd_datx_Delete


!----------*************----------------------------------------------------------------------------
subroutine rhd_datx_Copy(datxout, datxin)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Copy ('rhd_data_x_Copy')
!
! PURPOSE:
!   Copy data structure
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_Copy(datxout, datxin)
!
! INPUT:
!   datxin:       ('data_x_input') type(datx_type), input structure for additional data
!
! OUTPUT:
!   datxout:      ('data_x_output') type(datx_type), output struct. for additional data,
!                 copy of datxin
!
! ROUTINES:
!   rhd_datx_SetIndices:   Set indices for quantities in structure for 1D mean data
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Pointers are allocated (and later deallocated).
!
! RESTRICTIONS:
!   Structure datxout is overwritten without check. It should not exist before.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_datx_Copy(datxout, datxin)
!   datxout=datxin
!
! MODIFICATION HISTORY:
!   2000-02-08 (B.F. Copenhagen) Written
!   2000-11-15 (B.F. Uppsala) Nullify not associated pointers
!   2001-03-13 (B.F.) Special version for OpenMP on SGI
!   2001-03-24 (B.F.) Bug fix
!   2010-04-20 (B.F.) Rename copy_datx -> rhd_datx_Copy
!   2012-11-28 (B.F.) Allocate and copy large 2D array
!   2018-05-07 (B.F.) Remove branch for old averaging data type
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(datx_type),           intent(in)   :: datxin
type(datx_type),           intent(out)  :: datxout
!
! --- Local variables ---
integer                                 :: m, n
real, dimension(:)    , pointer         :: xx
real(kind=mArk), &
      dimension(:)    , pointer         :: x
real(kind=mArk), &
      dimension(:,:)  , pointer         :: x2
!---------------------------------------------------------------------------------------------------
!
datxout%m=datxin%m
datxout%n=datxin%n
m        =datxin%m
n        =datxin%n
!
datxout%na=datxin%na
if (datxin%na > 0) then
  allocate(datxout%d(datxout%m:datxout%n+1, datxout%na))
  datxout%d=datxin%d
  !
  call rhd_datx_SetIndices(datxout, datxin%nWave, (datxin%i_relvol > 0))
else
  nullify(datxout%d)
endif
if (datxin%na1 > 0) then
  allocate(datxout%d1(datxout%m:datxout%n+1, datxin%na1))
  datxout%d1=datxin%d1
else
  nullify(datxout%d1)
endif
!
end subroutine rhd_datx_Copy


!----------*******************----------------------------------------------------------------------
subroutine rhd_datc_SetIndices(datc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_SetIndices ('rhd_data_cylindrical_Set_Indices')
!
! PURPOSE:
!   Set indices for quantities in structure for 2D cylindrical mean data.
!   
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_SetIndices(datc)
!
! OUTPUT:
!   datc:    ('data_cylindrical') type(datc_type), structure with additional output data
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Set indices for individual quantities in large array.
!
! EXAMPLE:
!   call rhd_datc_SetIndices(datc)
!
! MODIFICATION HISTORY:
!   2013-08-09 (B.F. Lyon) Written
!   2018-06-04 (B.F. Uppsala) Initialize i_quc and i_Bc indices also if not needed
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(out)  :: datc
!
! --- Local variables ---
integer                                 :: i
!---------------------------------------------------------------------------------------------------
!
! --- Find indices for 1D mean arrays ---
i=0
i=i+1 ; datc%i_xb      =i
i=i+1 ; datc%i_xc      =i
datc%na1=i
!
! --- Find indices for 2D mean arrays ---
i=0
i=i+1 ; datc%i_relvol  =i
i=i+1 ; datc%i_rho     =i
i=i+1 ; datc%i_ei      =i
i=i+1 ; datc%i_v1      =i
i=i+1 ; datc%i_v2      =i
i=i+1 ; datc%i_v3      =i
i=i+1 ; datc%i_rhoei   =i
i=i+1 ; datc%i_s       =i
i=i+1 ; datc%i_ss      =i
i=i+1 ; datc%i_rhos    =i
i=i+1 ; datc%i_T       =i
i=i+1 ; datc%i_P       =i
i=i+1 ; datc%i_Gamma1  =i
i=i+1 ; datc%i_Gamma3  =i
i=i+1 ; datc%i_delta   =i
i=i+1 ; datc%i_v1v1    =i
i=i+1 ; datc%i_v2v2    =i
i=i+1 ; datc%i_v3v3    =i
i=i+1 ; datc%i_rhov1   =i
i=i+1 ; datc%i_rhov2   =i
i=i+1 ; datc%i_rhov3   =i
i=i+1 ; datc%i_rhov1v1 =i
i=i+1 ; datc%i_rhov2v2 =i
i=i+1 ; datc%i_rhov3v3 =i
i=i+1 ; datc%i_rhov1v2 =i
i=i+1 ; datc%i_rhov1v3 =i
i=i+1 ; datc%i_rhov2v3 =i
i=i+1 ; datc%i_fep1    =i
i=i+1 ; datc%i_fep3    =i
i=i+1 ; datc%i_feip1   =i
i=i+1 ; datc%i_feip3   =i
i=i+1 ; datc%i_fek1    =i
i=i+1 ; datc%i_fek3    =i
i=i+1 ; datc%i_fer1    =i
i=i+1 ; datc%i_fer2    =i
i=i+1 ; datc%i_fer3    =i
!
! --- Set array indices for additional densities ---
datc%i_quc  =-1
datc%i_fquc1=-1
datc%i_fquc2=-1
datc%i_fquc3=-1
!#
# if (rhd_box_quc01 != 0)
!#
if (prop%nquc > 0) then
  i=i+1 ; datc%i_quc   =i ; i=i+prop%nquc-1
  i=i+1 ; datc%i_fquc1 =i ; i=i+prop%nquc-1
  i=i+1 ; datc%i_fquc2 =i ; i=i+prop%nquc-1
  i=i+1 ; datc%i_fquc3 =i ; i=i+prop%nquc-1
endif
!#
# endif
!#      /* rhd_box_quc01 */
!
! --- Set array indices for magnetic fields ---
datc%i_Bc1=-1
datc%i_Bc2=-1
datc%i_Bc3=-1
!#
# if (rhd_box_bmag01 != 0)
!#
if (prop%nBb > 0) then
  i=i+1 ; datc%i_Bc1   =i
  i=i+1 ; datc%i_Bc2   =i
  i=i+1 ; datc%i_Bc3   =i
endif
!#
# endif
!#      /* rhd_box_bmag01 */
!
datc%na=i
!
end subroutine rhd_datc_SetIndices


!----------*************----------------------------------------------------------------------------
subroutine rhd_datc_Make(datc, box, Global_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_Make ('rhd_data_cylindrical_Make')
!
! PURPOSE:
!   Create structure of requested size for 2D cylindrical mean data.
!   
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_Make(datc, box, Global_flag)
!
! INPUT:
!   box:          ('box') type(box_type), computational box with bulk data
!   Global_flag:  ('Global_flag') logical to indicate that global instead of local indices
!                 should be used
!
! OUTPUT:
!   datc:         ('data_cylindrical') type(datc_type), structure with additional output data
!
! ROUTINES:
!   rhd_datc_Reset:        Set fields in structure for cylindrical mean data to zero
!   rhd_datc_SetIndices:   Set indices for quantities in structure for 2D cylindrical mean data
!
! MODULES:
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Pointers are allocated.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Allocate pointer arrays of proper size (or nullify them).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-07-31 (B.F. Lyon) First version
!   2013-08-09 (B.F.) Call rhd_datc_SetIndices
!   2017-08-05 (B.F.) Replace mode with Global_flag
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_box_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(out)  :: datc
type(box_type),            intent(in)   :: box
logical,                   intent(in)   :: Global_flag
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, i
logical                                 :: null_flag
real                                    :: rmin, dr
real, dimension(:,:)    , pointer       :: x2
real, dimension(:,:,:)  , pointer       :: x3
!---------------------------------------------------------------------------------------------------
!
! --- Make abbreviations ---
m1=prop%m1
n1=prop%n1
m2=prop%m2
n2=prop%n2
if (Global_flag) then
  ! --- Use only global model dimensions ---
  m3=prop%m3
  n3=prop%n3
else
  ! --- Use local model dimensions for cylinder height ---
  m3=box%m3
  n3=box%n3
endif
!
!write(6,*) 'rhd_datc_Make: ', Global_flag, m1,n1, m2,n2, m3,n3
!call flush(6)
!
if ((n1 > m1) .and. (n2 > m2) .and. &
    (par%grav_mode == 'central') .and. (par%OutFile_mcyl /= '')) then
  ! --- Set index range ---
  datc%m1=m3
  datc%n1=n3
  datc%m2=1
  !
  ! --- Compute distance from center to nearest wall ---
  rmin=min(-prop%xb_l1, prop%xb_r1, &
           -prop%xb_l2, prop%xb_r2   )
  ! --- Compute maximum radial step width ---
  dr=max((prop%xb_r1-prop%xb_l1)/float(n1-m1+1), &
         (prop%xb_r2-prop%xb_l2)/float(n2-m2+1)   )
  !
  if ((rmin > 0.0) .and. (dr > 0.0)) then
    null_flag=.false.
    !
    ! --- Set quantity indices for mean arrays ---
    call rhd_datc_SetIndices(datc)
    !
    ! --- Compute number of axis values ---
    datc%n2=int(rmin/dr+0.7)
    !
    !write(6,*) 'rhd_datc_Make: ', rmin, dr, rmin/dr, datc%n2
    !call flush(6)
    !
    ! --- Allocate array for 1D (only radial) averages ---
    allocate(x2(                 datc%m2:datc%n2+1, datc%na1))
    datc%d1=>x2
    !
    !write(6,*) 'rhd_datc_Make: ', datc%i_xc, datc%m2, datc%n2, lbound(datc%d1), ubound(datc%d1)
    !call flush(6)
    !
    ! --- Allocate array for standard bulk quantities ---
    allocate(x3(datc%m1:datc%n1, datc%m2:datc%n2  , datc%na ))
    datc%d=>x3
    !
    ! --- Set data arrays to zero ---
    call rhd_datc_Reset(datc)
    !
    ! --- Create axes ---
    datc%d1(datc%m2:datc%n2+1,datc%i_xb)=dr * real((/(i,i=0,datc%n2-datc%m2+1)/))
    datc%d1(datc%m2:datc%n2  ,datc%i_xc)=0.5 * (datc%d1(datc%m2+1:datc%n2+1,datc%i_xb) + &
                                                datc%d1(datc%m2  :datc%n2  ,datc%i_xb)    )
    !
    !write(6,*) 'rhd_datc_Make: ', minval(datc%d1(datc%m2:datc%n2+1,datc%i_xb)), &
    !                              maxval(datc%d1(datc%m2:datc%n2+1,datc%i_xb)), &
    !                              minval(datc%d1(datc%m2:datc%n2  ,datc%i_xc)), &
    !                              maxval(datc%d1(datc%m2:datc%n2  ,datc%i_xc))
    !call flush(6)
  else
    null_flag=.true.
  endif
else
  null_flag=.true.
endif
!
if (null_flag) then
  ! --- No cylindrical means requested: just zero the scalars and nullify the arrays ---
  !
  datc%m1 =1
  datc%n1 =0
  datc%m2 =1
  datc%n2 =0
  datc%na1=0
  datc%na =0
  nullify(datc%d1, datc%d)
endif
!
end subroutine rhd_datc_Make


!----------**************---------------------------------------------------------------------------
subroutine rhd_datc_Reset(datc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_Reset ('rhd_data_cylindrical_Reset')
!
! PURPOSE:
!   Set fields in structure for cylindrical mean data to zero.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_Reset(datc)
!
! INPUT:
!
! INPUT/OUTPUT:
!   datc:         ('data_cylindrical') type(datc_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Structure datc has to be initialized (with rhd_datc_Make).
!
! PROCEDURE:
!   Just set all associated arrays to zero.
!
! EXAMPLE:
!   call rhd_datc_Reset(datc)
!
! MODIFICATION HISTORY:
!   2013-07-31 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(inout):: datc
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
!if (datc%na1 > 0) datc%d1=0.0
if (datc%na  > 0) datc%d =0.0
!
end subroutine rhd_datc_Reset


!----------************-----------------------------------------------------------------------------
subroutine rhd_datc_Add(datc1, datc2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_Add ('rhd_data_cylindrical_Add')
!
! PURPOSE:
!   Add the fields in the structure for cylindrical mean data.
!
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_Add(datc1, datc2)
!
! INPUT:
!   datc2:        ('data_cylindrical_2') type(datc_type), structure for additional data
!
! INPUT/OUTPUT:
!   datc1:        ('data_cylindrical_1') type(datc_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Both datc1 and datc2 have to be initialized (with rhd_datc_Make).
!   The array sizes have to agree.
!
! PROCEDURE:
!   "datc1 = datc1 + datc2"
!
! EXAMPLE:
!   call rhd_datc_Add(datc1, datc2)
!
! MODIFICATION HISTORY:
!   2013-07-31 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(inout):: datc1
type(datc_type),           intent(in)   :: datc2
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (datc1%na1 > 0) datc1%d1=datc2%d1
if (datc1%na  > 0) datc1%d =datc1%d +datc2%d
!
end subroutine rhd_datc_Add


!----------*******************----------------------------------------------------------------------
subroutine rhd_datc_AddOverlap(datc1, datc2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_AddOverlap ('rhd_data_cylindrical_Add_Overlap')
!
! PURPOSE:
!   Add the overlap of fields in the structure for cylindrical mean data.
!
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_AddOverlap(datc1, datc2)
!
! INPUT:
!   datc2:        ('data_cylindrical_2') type(datc_type), structure for additional data
!
! INPUT/OUTPUT:
!   datc1:        ('data_cylindrical_1') type(datc_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Both datc1 and datc2 have to be initialized (with rhd_datc_Make).
!   The array sizes have to agree.
!
! PROCEDURE:
!   "In intersection of datc1 and datc2: datc1 = datc1 + datc2"
!
! EXAMPLE:
!   call rhd_datc_AddOverlap(datc1, datc2)
!
! MODIFICATION HISTORY:
!   2013-07-31 (B.F. Lyon) Written
!   2019-01-31 (B.F. Uppsala) Derived from rhd_datc_Add
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(inout):: datc1
type(datc_type),           intent(in)   :: datc2
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2
!---------------------------------------------------------------------------------------------------
!
! --- Find the intersection of the arrays ---
m1=max(datc1%m1, datc2%m1)
n1=min(datc1%n1, datc2%n1)
m2=max(datc1%m2, datc2%m2)
n2=min(datc1%n2, datc2%n2)
!
if ((n1 >= m1) .and. (m2 >= n2) .and. (datc1%na > 0) .and. (datc1%na == datc2%na)) then
  ! --- Add the intersection if non-empty ---
  datc1%d(m1:n1, m2:n2, 1:datc1%na) = datc1%d(m1:n1, m2:n2, 1:datc1%na) + &
                                      datc2%d(m1:n1, m2:n2, 1:datc2%na)
endif
!
end subroutine rhd_datc_AddOverlap


!----------***************--------------------------------------------------------------------------
subroutine rhd_datc_Delete(datc)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_Delete ('rhd_data_cylindrical_Delete')
!
! PURPOSE:
!   Delete structure for cylindrical mean data. Deallocate memory.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_Delete(datc)
!
! INPUT:
!
! INPUT/OUTPUT:
!   datc:         ('data_cylindrical') type(datc_type), structure for additional data
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Pointers are deallocated.
!
! RESTRICTIONS:
!   Structure datc has to be initialized (with rhd_datc_Make).
!
! PROCEDURE:
!   Deallocate all associated pointer arrays in input structure and set some counters to zero
!
! EXAMPLE:
!   call rhd_datc_Delete(datc)
!
! MODIFICATION HISTORY:
!   2013-07-31 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(inout):: datc
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (datc%na  > 0) deallocate(datc%d )
if (datc%na1 > 0) deallocate(datc%d1)
!
datc%m1 =0
datc%n1 =0
datc%na =0
datc%na1=0
!
end subroutine rhd_datc_Delete


!----------*************----------------------------------------------------------------------------
subroutine rhd_datc_Copy(datcout, datcin)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_Copy ('rhd_data_cylindrical_Copy')
!
! PURPOSE:
!   Copy data structure
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_Copy(datcout, datcin)
!
! INPUT:
!   datcin:       ('data_cylindrical_input') type(datc_type), input structure for additional data
!
! OUTPUT:
!   datcout:      ('data_cylindrical_output') type(datc_type), output structure for additional data,
!                 copy of datcin
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Pointers are allocated (and later deallocated).
!
! RESTRICTIONS:
!   Structure datcout is overwritten without check. It should not exist before.
!   Data indices are not copied.
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_datc_Copy(datcout, datcin)
!   datcout=datcin
!
! TODO:
!   Copy data indices or store them in global structure.
!
! MODIFICATION HISTORY:
!   2013-08-01 (B.F. Lyon) Written
!---------------------------------------------------------------------------------------------------
!use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
type(datc_type),           intent(in)   :: datcin
type(datc_type),           intent(out)  :: datcout
!
! --- Local variables ---
integer                                 :: m, n
real, dimension(:,:)    , pointer       :: x2
real, dimension(:,:,:)  , pointer       :: x3
!---------------------------------------------------------------------------------------------------
!
! --- Copy array-size specifications ---
datcout%m1 =datcin%m1
datcout%n1 =datcin%n1
datcout%m2 =datcin%m2
datcout%n2 =datcin%n2
datcout%na =datcin%na
datcout%na1=datcin%na1
!
! --- Copy data indices ---
! ...
!
! --- Allocate and copy array for 1D (only radial) averages ---
allocate(x2(                       datcout%m2:datcout%n2+1, datcout%na1))
datcout%d1=>x2
datcout%d1=datcin%d1
!
! --- Allocate and copy array for standard bulk quantities ---
allocate(x3(datcout%m1:datcout%n1, datcout%m2:datcout%n2  , datcout%na ))
datcout%d=>x3
datcout%d=datcin%d
!
end subroutine rhd_datc_Copy


!----------************-----------------------------------------------------------------------------
subroutine rhd_dat_Make(dat, box, nband, action, &
                        Global_flag, IncludeIntensity_flag, IncludeFlux_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_Make ('rhd_data_Make')
!
! PURPOSE:
!   Create structure for data of requested size.
!   
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_Make(dat, box, nband, action=action, &
!                     Global_flag=Global_flag, &
!                     IncludeIntensity_flag=IncludeIntensity_flag, &
!                     IncludeFlux_flag=IncludeFlux_flag)
!
! INPUT:
!   box:          ('box') type(box_type), computational box with bulk data
!   nband:        ('number_band') integer, switch for number of opacity bands (1 or 2)
!   action:       ('action') optional, derived type with collection of control parameters
!   Global_flag:  ('Global_flag') optional logical to indicate that global instead of local
!                 indices should be used
!   IncludeIntensity_flag: ('Include_Intensity_flag') optional logical to indicate that surface
!                          intensity arrays should be included in dat structure, default=.false.
!   IncludeFlux_flag: ('Include_Flux_flag') optional logical to indicate that surface flux
!                          arrays should be included in dat structure, default=.false.
!
! OUTPUT:
!   dat:          ('data') type(dat_type), structure with additional output data
!
! VARIABLES:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!
! ROUTINES:
!   opta_Info:             Give stored information about current opacity table
!   rhd_datc_Make:         Create structure of requested size for 2D cylindrical mean data
!   rhd_datx_Axis:         Construct radial axis for datx
!   rhd_datx_Make:         Create structure for data of requested size
!
! MODULES:
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_box_module:        Box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!   Allocate pointers
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   call rhd_dat_Make(dat, box, 1)
!
! MODIFICATION HISTORY:
!   1999-08-20 (B.F. Copenhagen) Written
!   2000-01-25 (B.F.) Parameters m1, ..., n3 not optional anymore
!   2001-03-13 (B.F. Uppsala) Special version for OpenMP on SGI
!   2003-08-11 (B.F.) nullify(fer3_l, fer3_r, Int3_l) for non-central models,
!                     frequency-dependent surface intensity (for non-central models) added
!   2007-03-08 (B.F. Lyon) Determine number of bins with opta_Info
!   2010-04-20 (B.F.) Rename make_dat -> rhd_dat_Make
!   2013-08-09 (B.F.) Replace separate indices by entire box as input parameter; new parameter mode;
!                     call rhd_datx_Axis()
!   2014-03-15 (B.F. Uppsala) New parameter 'action'
!   2014-12-10 (B.F.) Base size of intensity arrays on wg%rad(1)
!   2017-08-05 (B.F.) Replace parameter mode with Global_flag and RadData_flag
!   2017-08-06 (B.F.) Make box and nband optional
!   2017-08-11 (B.F.) Base boundary conditions for global radiation arrays on wg%rad(1)
!   2018-06-12 (B.F.) Reserve 0/0/16 waves in horizontal/vertical/radial datx structures
!   2019-01-28 (B.F.) Create flux arrays for rad(1) and intensity arrays for hyd geometry
!   2019-01-31 (B.F.) Replace RadData_flag by IncludeIntensity_flag and IncludeFlux_flag
!   2019-03-04 (B.F.) Choose number of radial points for spherical averages bases on dir1, only
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_box_module
use rhd_prop_module
use rhd_action_module
use opta_module
!
implicit none
!
! --- I/O parameters ---
type(dat_type),            intent(out)  :: dat
type(box_type),  optional, intent(in)   :: box
integer,         optional, intent(in)   :: nband
type(action_type), &
                 optional, intent(in)   :: action
logical,         optional, intent(in)   :: Global_flag, IncludeIntensity_flag, IncludeFlux_flag
!
! --- Local variables ---
integer                                 :: n, nbin, m1,n1, m2,n2, m3,n3, &
                                           mr1,nr1, mr2,nr2, mr3,nr3, &
                                           bound_l1, bound_r1, &
                                           bound_l2, bound_r2, &
                                           bound_l3, bound_r3
real, dimension(:,:)  , pointer         :: f
real, dimension(:,:,:), pointer         :: ff
logical                                 :: Global_flag0, IncludeIntensity_flag0, IncludeFlux_flag0
!---------------------------------------------------------------------------------------------------
!
! --- Set default values for optional parameters ---
if (present(Global_flag)) then
  Global_flag0=Global_flag
else
  Global_flag0=.false.
endif
if (.not. present(IncludeIntensity_flag)) then
  IncludeIntensity_flag0=.true.
else
  IncludeIntensity_flag0=IncludeIntensity_flag
endif
if (.not. present(IncludeFlux_flag)) then
  IncludeFlux_flag0=.true.
else
  IncludeFlux_flag0=IncludeFlux_flag
endif
!
! === Determine data array sizes and make abbreviations ===
!
if (Global_flag0) then
  ! --- Use global model dimensions ---
  m1=prop%m1
  n1=prop%n1
  m2=prop%m2
  n2=prop%n2
  m3=prop%m3
  n3=prop%n3
  !
else ! (.not. Global_flag0)
  ! --- Use local model dimensions ---
  m1=box%m1
  n1=box%n1
  m2=box%m2
  n2=box%n2
  m3=box%m3
  n3=box%n3
  !
endif ! (Global_flag0)
!
! --- Put box size into dat structure ---
dat%m1=m1
dat%n1=n1
dat%m2=m2
dat%n2=n2
dat%m3=m3
dat%n3=n3
dat%mr1=m1
dat%nr1=n1
dat%mr2=m2
dat%nr2=n2
dat%mr3=m3
dat%nr3=n3
!
! === Create or nullify 2D and 3D arrays for radiation maps ===
!
! --- Nullify flux and intensity arrays ---
nullify(dat%fer1_l, dat%fer1_r, &
        dat%fer2_l, dat%fer2_r, &
        dat%fer3_l, dat%fer3_r, &
        dat%Int1_l, dat%Int1_r, &
        dat%Int2_l, dat%Int2_r, &
        dat%Int3_l, dat%Int3_r, &
        dat%Intxx3_r             )
!
if (IncludeFlux_flag0) then
  ! === Allocate surface flux arrays ===
  !
  ! --- Get index ranges for flux arrays ---
  if (Global_flag0) then
    ! --- Use global range of wg%rad(1) configuration ---
    mr1=wg%rad(1)%m(1)
    nr1=wg%rad(1)%n(1)
    mr2=wg%rad(1)%m(2)
    nr2=wg%rad(1)%n(2)
    mr3=wg%rad(1)%m(3)
    nr3=wg%rad(1)%n(3)
    !
    ! --- Copy boundary-condition parameters from global property structure ---
    bound_l1=wg%rad(1)%bound_l(1)
    bound_r1=wg%rad(1)%bound_r(1)
    bound_l2=wg%rad(1)%bound_l(2)
    bound_r2=wg%rad(1)%bound_r(2)
    bound_l3=wg%rad(1)%bound_l(3)
    bound_r3=wg%rad(1)%bound_r(3)
    !
  else ! (.not. Global_flag0)
    ! --- Use local box ---
    mr1=box%m1
    nr1=box%n1
    mr2=box%m2
    nr2=box%n2
    mr3=box%m3
    nr3=box%n3
    !
    if (present(action)) then
      ! --- Copy boundary-condition parameters from action structure ---
      bound_l1=action%bound_l1
      bound_r1=action%bound_r1
      bound_l2=action%bound_l2
      bound_r2=action%bound_r2
      bound_l3=action%bound_l3
      bound_r3=action%bound_r3
    else ! (.not. (present(action))
      ! --- Copy boundary-condition parameters from global property structure ---
      bound_l1=wg%rad(1)%bound_l(1)
      bound_r1=wg%rad(1)%bound_r(1)
      bound_l2=wg%rad(1)%bound_l(2)
      bound_r2=wg%rad(1)%bound_r(2)
      bound_l3=wg%rad(1)%bound_l(3)
      bound_r3=wg%rad(1)%bound_r(3)
    endif ! (present(action))
    !
  endif ! (Global_flag0)
  !
  ! --- Put box size into dat structure ---
  dat%mr1=mr1
  dat%nr1=nr1
  dat%mr2=mr2
  dat%nr2=nr2
  dat%mr3=mr3
  dat%nr3=nr3
  !
  if (par%grav_mode == 'central') then
    ! --- Allocate arrays for all outside surfaces of a global model ---
    !
    if ((bound_l1 /= bound_InternalNext    ) .and. &
        (bound_l1 /= bound_InternalPeriodic) .and. &
        (wg%rad(1)%n(1) > wg%rad(1)%m(1))           ) then
      ! --- Allocate flux array for left side-1 surface ---
      allocate(f(mr2:nr2, mr3:nr3))
      dat%fer1_l=>f
    endif ! (bound_l1)
    !
    if ((bound_r1 /= bound_InternalNext    ) .and. &
        (bound_r1 /= bound_InternalPeriodic) .and. &
        (wg%rad(1)%n(1) > wg%rad(1)%m(1))           ) then
      ! --- Allocate flux array for right side-1 surface ---
      allocate(f(mr2:nr2, mr3:nr3))
      dat%fer1_r=>f
    endif ! (bound_r1)
    !
    if ((bound_l2 /= bound_InternalNext    ) .and. &
        (bound_l2 /= bound_InternalPeriodic) .and. &
        (wg%rad(1)%n(2) > wg%rad(1)%m(2))           ) then
      ! --- Allocate flux array for left side-2 surface ---
      allocate(f(mr1:nr1, mr3:nr3))
      dat%fer2_l=>f
    endif ! (bound_l2)
    !
    if ((bound_r2 /= bound_InternalNext    ) .and. &
        (bound_r2 /= bound_InternalPeriodic) .and. &
        (wg%rad(1)%n(2) > wg%rad(1)%m(2))           ) then
      ! --- Allocate flux array for right side-2 surface ---
      allocate(f(mr1:nr1, mr3:nr3))
      dat%fer2_r=>f
    endif ! (bound_r2)
    !
    if ((bound_l3 /= bound_InternalNext    ) .and. &
        (bound_l3 /= bound_InternalPeriodic) .and. &
        (wg%rad(1)%n(3) > wg%rad(1)%m(3))           ) then
      ! --- Allocate flux array for left side-3 surface ---
      allocate(f(mr1:nr1, mr2:nr2))
      dat%fer3_l=>f
    endif ! (bound_l3)
    !
    if ((bound_r3 /= bound_InternalNext    ) .and. &
        (bound_r3 /= bound_InternalPeriodic) .and. &
        (wg%rad(1)%n(3) > wg%rad(1)%m(3))           ) then
      ! --- Allocate flux array for right side-3 surface ---
      allocate(f(mr1:nr1, mr2:nr2))
      dat%fer3_r=>f
    endif ! (bound_r3)
  endif ! (par%grav_mode)
endif ! (IncludeFlux_flag0)
!
if (IncludeIntensity_flag0) then
  ! === Allocate surface intensity arrays ===
  !
  ! --- Get index ranges for intensity arrays ---
  if (Global_flag0) then
    ! --- Use global range of wg%hyd configuration ---
    mr1=wg%hyd%m(1)
    nr1=wg%hyd%n(1)
    mr2=wg%hyd%m(2)
    nr2=wg%hyd%n(2)
    mr3=wg%hyd%m(3)
    nr3=wg%hyd%n(3)
    !
    ! --- Copy boundary-condition parameters from global property structure ---
    bound_l1=wg%hyd%bound_l(1)
    bound_r1=wg%hyd%bound_r(1)
    bound_l2=wg%hyd%bound_l(2)
    bound_r2=wg%hyd%bound_r(2)
    bound_l3=wg%hyd%bound_l(3)
    bound_r3=wg%hyd%bound_r(3)
    !
  else ! (.not. Global_flag0)
    ! --- Use local box ---
    mr1=box%m1
    nr1=box%n1
    mr2=box%m2
    nr2=box%n2
    mr3=box%m3
    nr3=box%n3
    !
    if (present(action)) then
      ! --- Copy boundary-condition parameters from action structure ---
      bound_l1=action%bound_l1
      bound_r1=action%bound_r1
      bound_l2=action%bound_l2
      bound_r2=action%bound_r2
      bound_l3=action%bound_l3
      bound_r3=action%bound_r3
    else ! (.not. (present(action))
      ! --- Copy boundary-condition parameters from global property structure ---
      bound_l1=wg%hyd%bound_l(1)
      bound_r1=wg%hyd%bound_r(1)
      bound_l2=wg%hyd%bound_l(2)
      bound_r2=wg%hyd%bound_r(2)
      bound_l3=wg%hyd%bound_l(3)
      bound_r3=wg%hyd%bound_r(3)
    endif ! (present(action))
    !
  endif ! (Global_flag0)
  !
  if (par%grav_mode == 'central') then
    !
    if ((bound_l1 /= bound_InternalNext    ) .and. &
        (bound_l1 /= bound_InternalPeriodic) .and. &
        (wg%hyd%n(1) > wg%hyd%m(1))           ) then
      ! --- Allocate intensity array for left side-1 surface ---
      allocate(f(mr2:nr2, mr3:nr3))
      dat%Int1_l=>f
    endif ! (bound_l1)
    !
    if ((bound_r1 /= bound_InternalNext    ) .and. &
        (bound_r1 /= bound_InternalPeriodic) .and. &
        (wg%hyd%n(1) > wg%hyd%m(1))           ) then
      ! --- Allocate intensity array for right side-1 surface ---
      allocate(f(mr2:nr2, mr3:nr3))
      dat%Int1_r=>f
    endif ! (bound_r1)
    !
    if ((bound_l2 /= bound_InternalNext    ) .and. &
        (bound_l2 /= bound_InternalPeriodic) .and. &
        (wg%hyd%n(2) > wg%hyd%m(2))           ) then
      ! --- Allocate intensity array for left side-2 surface ---
      allocate(f(mr1:nr1, mr3:nr3))
      dat%Int2_l=>f
    endif ! (bound_l2)
    !
    if ((bound_r2 /= bound_InternalNext    ) .and. &
        (bound_r2 /= bound_InternalPeriodic) .and. &
        (wg%hyd%n(2) > wg%hyd%m(2))           ) then
      ! --- Allocate intensity array for right side-2 surface ---
      allocate(f(mr1:nr1, mr3:nr3))
      dat%Int2_r=>f
    endif ! (bound_r2)
    !
    if ((bound_l3 /= bound_InternalNext    ) .and. &
        (bound_l3 /= bound_InternalPeriodic) .and. &
        (wg%hyd%n(3) > wg%hyd%m(3))           ) then
      ! --- Allocate intensity array for left side-3 surface ---
      allocate(f(mr1:nr1, mr2:nr2))
      dat%Int3_l=>f
    endif ! (bound_l3)
    !
    if ((bound_r3 /= bound_InternalNext    ) .and. &
        (bound_r3 /= bound_InternalPeriodic) .and. &
        (wg%hyd%n(3) > wg%hyd%m(3))           ) then
      ! --- Allocate intensity array for right side-3 surface ---
      allocate(f(mr1:nr1, mr2:nr2))
      dat%Int3_r=>f
    endif ! (bound_r3)
    !
  else ! (par%grav_mode /= 'central')
    ! --- Allocate arrays for top of local model ---
    !
    if (nr3 > mr3) then
      ! --- Allocate (grey) intensity array for right (top) side-3 surface ---
      allocate(f(mr1:nr1, mr2:nr2))
      dat%Int3_r=>f
      !
      if ((par%N_radband >= 2) .and. (merge(nband, 1, present(nband)) > 1)) then
        call opta_Info(nbin=nbin)
        if (nbin > 0) then
          ! --- Allocate non-grey intensity array for right (top) side-3 surface ---
          allocate(ff(mr1:nr1, mr2:nr2, nbin))
          dat%Intxx3_r=>ff
        endif ! (nbin)
      endif ! (par%N_radband)
    endif ! (nr3)
    !
  endif ! (par%grav_mode)
endif ! (IncludeIntensity_flag0)
!
! === Create structures for averaged quantities ===
!
! --- Make structures for horizontal averages and set the arrays to zero ---
call rhd_datx_Make(dat%x23, m1, n1, 0)
call rhd_datx_Make(dat%x13, m2, n2, 0)
call rhd_datx_Make(dat%x12, m3, n3, 0)
!
! --- Make structure for averages over spheres ---
! --- Determine number of flux points for averages over spheres ---
!n=min(n1-m1+1, n2-m2+1, n3-m3+1)
!n=min(prop%n1-prop%m1+1, prop%n2-prop%m2+1, prop%n3-prop%m3+1)
!n=prop%n1-prop%m1+1
n=2*prop%n1+1
! --- Make structure for averages over spheres and set the arrays to zero ---
call rhd_datx_Make(dat%r, 1, n, 16, radial_flag=.true.)
! --- Construct radial axis ---
call rhd_datx_Axis(dat%r, box=box)
!
! --- Make structure for azimuthal averages over cylinders and set the arrays to zero ---
call rhd_datc_Make(dat%c, box, Global_flag0)
!
end subroutine rhd_dat_Make


!----------*************----------------------------------------------------------------------------
subroutine rhd_dat_Reset(dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_Reset ('rhd_dat_Reset')
!
! PURPOSE:
!   Set fields in structure for additional data to zero.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_Reset(dat)
!
! INPUT:
!
! INPUT/OUTPUT:
!   dat:          ('dat') type(dat_type), structure for additional data
!
! ROUTINES:
!   rhd_datc_Reset:        Set fields in structure for cylindrical mean data to zero
!   rhd_datx_Reset:        Set fields in structure for additional data to zero
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Structure dat has to be initialized (with rhd_dat_Make).
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-08-20 (B.F. Copenhagen) Written
!   2000-02-07 (B.F.) call reset_datx
!   2010-04-20 (B.F.) Rename reset_dat -> rhd_dat_Reset
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(dat_type),            intent(inout):: dat
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
dat%time      =0.0
dat%itime     =0
dat%luminosity=0.0
!
if (associated(dat%fer1_l)) dat%fer1_l=0.0
if (associated(dat%fer1_r)) dat%fer1_r=0.0
if (associated(dat%fer2_l)) dat%fer2_l=0.0
if (associated(dat%fer2_r)) dat%fer2_r=0.0
if (associated(dat%fer3_l)) dat%fer3_l=0.0
if (associated(dat%fer3_r)) dat%fer3_r=0.0
!
if (associated(dat%Int1_l)) dat%Int1_l=0.0
if (associated(dat%Int1_r)) dat%Int1_r=0.0
if (associated(dat%Int2_l)) dat%Int2_l=0.0
if (associated(dat%Int2_r)) dat%Int2_r=0.0
if (associated(dat%Int3_l)) dat%Int3_l=0.0
if (associated(dat%Int3_r)) dat%Int3_r=0.0
!
if (associated(dat%Intxx3_r)) dat%Intxx3_r=0.0
!
call rhd_datx_Reset(dat%x23)
call rhd_datx_Reset(dat%x13)
call rhd_datx_Reset(dat%x12)
!if (par%grav_mode == 'central') then
  call rhd_datx_Reset(dat%r  )
  call rhd_datc_Reset(dat%c  )
!endif
!
end subroutine rhd_dat_Reset


!----------***********------------------------------------------------------------------------------
subroutine rhd_dat_Add(dat1, dat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_Add ('rhd_data_Add')
!
! PURPOSE:
!   Add contents of two structures for additional data.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_Add(dat1, dat2)
!
! INPUT:
!   dat2:         ('data_2') type(dat_type), structure for additional data
!
! INPUT/OUTPUT:
!   dat1:         ('data_1') type(dat_type), structure for additional data
!
! ROUTINES:
!   rhd_datc_Add:          Add the fields in the structure for cylindrical mean data
!   rhd_datx_Add:          Add the fields in the structure for additional data
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Structures have to be initialized (with rhd_dat_Make).
!
! PROCEDURE:
!   "dat1=dat1+dat2"
!
! EXAMPLE:
!   call rhd_dat_Add(dat1, dat2)
!
! MODIFICATION HISTORY:
!   2001-01-01 (B.F. Kiel) Written
!   2010-04-20 (B.F. Lyon) Rename add_dat to rhd_dat_Add
!   2014-03-15 (B.F. Uppsala) Check association status of both operands
!   2019-02-07 (B.F.) Don't copy luminosity
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(dat_type),            intent(inout):: dat1
type(dat_type),            intent(in)   :: dat2
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
dat1%time      =dat2%time
dat1%itime     =dat2%itime
!dat1%luminosity=dat2%luminosity
!
if ((associated(dat1%fer1_l) .and. (associated(dat2%fer1_l)))) dat1%fer1_l=dat1%fer1_l+dat2%fer1_l
if ((associated(dat1%fer1_r) .and. (associated(dat2%fer1_r)))) dat1%fer1_r=dat1%fer1_r+dat2%fer1_r
if ((associated(dat1%fer2_l) .and. (associated(dat2%fer2_l)))) dat1%fer2_l=dat1%fer2_l+dat2%fer2_l
if ((associated(dat1%fer2_r) .and. (associated(dat2%fer2_r)))) dat1%fer2_r=dat1%fer2_r+dat2%fer2_r
if ((associated(dat1%fer3_l) .and. (associated(dat2%fer3_l)))) dat1%fer3_l=dat1%fer3_l+dat2%fer3_l
if ((associated(dat1%fer3_r) .and. (associated(dat2%fer3_r)))) dat1%fer3_r=dat1%fer3_r+dat2%fer3_r
!
if ((associated(dat1%Int1_l) .and. (associated(dat2%Int1_l)))) dat1%Int1_l=dat1%Int1_l+dat2%Int1_l
if ((associated(dat1%Int1_r) .and. (associated(dat2%Int1_r)))) dat1%Int1_r=dat1%Int1_r+dat2%Int1_r
if ((associated(dat1%Int2_l) .and. (associated(dat2%Int2_l)))) dat1%Int2_l=dat1%Int2_l+dat2%Int2_l
if ((associated(dat1%Int2_r) .and. (associated(dat2%Int2_r)))) dat1%Int2_r=dat1%Int2_r+dat2%Int2_r
if ((associated(dat1%Int3_l) .and. (associated(dat2%Int3_l)))) dat1%Int3_l=dat1%Int3_l+dat2%Int3_l
if ((associated(dat1%Int3_r) .and. (associated(dat2%Int3_r)))) dat1%Int3_r=dat1%Int3_r+dat2%Int3_r
!
if ((associated(dat1%Intxx3_r) .and. (associated(dat2%Intxx3_r)))) &
                                                         dat1%Intxx3_r=dat1%Intxx3_r+dat2%Intxx3_r
!
call rhd_datx_Add(dat1%x23, dat2%x23)
call rhd_datx_Add(dat1%x13, dat2%x13)
call rhd_datx_Add(dat1%x12, dat2%x12)
if (par%grav_mode == 'central') then
  call rhd_datx_Add(dat1%r, dat2%r  )
  call rhd_datc_Add(dat1%c, dat2%c  )
endif
!
end subroutine rhd_dat_Add


!----------******************-----------------------------------------------------------------------
subroutine rhd_dat_AddOverlap(dat1, dat2)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_AddOverlap ('rhd_data_Add_Overlap')
!
! PURPOSE:
!   Add overlap of contents of two structures for additional data.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_AddOverlap(dat1, dat2)
!
! INPUT:
!   dat2:         ('data_2') type(dat_type), structure for additional data
!
! INPUT/OUTPUT:
!   dat1:         ('data_1') type(dat_type), structure for additional data
!
! ROUTINES:
!   rhd_datc_AddOverlap:   Add the overlap of fields in the structure for cylindrical mean data
!   rhd_datx_AddOverlap:   Add the overlap of fields in structure for horizontal/radial mean data
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Structures have to be initialized (with rhd_dat_Make).
!   An empty overlap is silently ignored.
!
! PROCEDURE:
!   "In intersection of dat1 and dat2: dat1 = dat1 + dat2"
!
! EXAMPLE:
!   call rhd_dat_AddOverlap(dat1, dat2)
!
! MODIFICATION HISTORY:
!   2001-01-01 (B.F. Kiel) Written
!   2010-04-20 (B.F. Lyon) Rename add_dat to rhd_dat_Add
!   2014-03-15 (B.F. Uppsala) Check association status of both operands
!   2019-01-31 (B.F.) Derived from rhd_dat_Add
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
type(dat_type),            intent(inout):: dat1
type(dat_type),            intent(in)   :: dat2
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3
!---------------------------------------------------------------------------------------------------
!
! --- Copy scalar data from second data set ---
dat1%time      =dat2%time
dat1%itime     =dat2%itime
dat1%luminosity=dat2%luminosity
!
! --- Find the intersection of the surface flux arrays ---
m1=max(dat1%mr1, dat2%mr1)
n1=min(dat1%nr1, dat2%nr1)
m2=max(dat1%mr2, dat2%mr2)
n2=min(dat1%nr2, dat2%nr2)
m3=max(dat1%mr3, dat2%mr3)
n3=min(dat1%nr3, dat2%nr3)
!
! --- Add the intersection of the surface flux arrays, if non-empty ---
if ((m2 >= n2) .and. (n3 >= m3)) then
  if (associated(dat1%fer1_l) .and. associated(dat2%fer1_l)) then
    dat1%fer1_l(m2:n2, m3:n3) = dat1%fer1_l(m2:n2, m3:n3) + dat2%fer1_l(m2:n2, m3:n3)
  endif
  if (associated(dat1%fer1_r) .and. associated(dat2%fer1_r)) then
    dat1%fer1_r(m2:n2, m3:n3) = dat1%fer1_r(m2:n2, m3:n3) + dat2%fer1_r(m2:n2, m3:n3)
  endif
endif
if ((n1 >= m1) .and. (n3 >= m3)) then
  if (associated(dat1%fer2_l) .and. associated(dat2%fer2_l)) then
    dat1%fer2_l(m1:n1, m3:n3) = dat1%fer2_l(m1:n1, m3:n3) + dat2%fer2_l(m1:n1, m3:n3)
  endif
  if (associated(dat1%fer2_r) .and. associated(dat2%fer2_r)) then
    dat1%fer2_r(m1:n1, m3:n3) = dat1%fer2_r(m1:n1, m3:n3) + dat2%fer2_r(m1:n1, m3:n3)
  endif
endif
if ((n1 >= m1) .and. (m2 >= n2)) then
  if (associated(dat1%fer3_l) .and. associated(dat2%fer3_l)) then
    dat1%fer3_l(m1:n1, m2:n2) = dat1%fer3_l(m1:n1, m2:n2) + dat2%fer3_l(m1:n1, m2:n2)
  endif
  if (associated(dat1%fer3_r) .and. associated(dat2%fer3_r)) then
    dat1%fer3_r(m1:n1, m2:n2) = dat1%fer3_r(m1:n1, m2:n2) + dat2%fer3_r(m1:n1, m2:n2)
  endif
endif
!
! --- Find the intersection of the surface intensity arrays ---
m1=max(dat1%m1, dat2%m1)
n1=min(dat1%n1, dat2%n1)
m2=max(dat1%m2, dat2%m2)
n2=min(dat1%n2, dat2%n2)
m3=max(dat1%m3, dat2%m3)
n3=min(dat1%n3, dat2%n3)
!
! --- Add the intersection of the surface intensity arrays, if non-empty ---
if ((m2 >= n2) .and. (n3 >= m3)) then
  if (associated(dat1%Int1_l) .and. associated(dat2%Int1_l)) then
    dat1%Int1_l(m2:n2, m3:n3) = dat1%Int1_l(m2:n2, m3:n3) + dat2%Int1_l(m2:n2, m3:n3)
  endif
  if (associated(dat1%Int1_r) .and. associated(dat2%Int1_r)) then
    dat1%Int1_r(m2:n2, m3:n3) = dat1%Int1_r(m2:n2, m3:n3) + dat2%Int1_r(m2:n2, m3:n3)
  endif
endif
if ((n1 >= m1) .and. (n3 >= m3)) then
  if (associated(dat1%Int2_l) .and. associated(dat2%Int2_l)) then
    dat1%Int2_l(m1:n1, m3:n3) = dat1%Int2_l(m1:n1, m3:n3) + dat2%Int2_l(m1:n1, m3:n3)
  endif
  if (associated(dat1%Int2_r) .and. associated(dat2%Int2_r)) then
    dat1%Int2_r(m1:n1, m3:n3) = dat1%Int2_r(m1:n1, m3:n3) + dat2%Int2_r(m1:n1, m3:n3)
  endif
endif
if ((n1 >= m1) .and. (m2 >= n2)) then
  if (associated(dat1%Int3_l) .and. associated(dat2%Int3_l)) then
    dat1%Int3_l(m1:n1, m2:n2) = dat1%Int3_l(m1:n1, m2:n2) + dat2%Int3_l(m1:n1, m2:n2)
  endif
  if (associated(dat1%Int3_r) .and. associated(dat2%Int3_r)) then
    dat1%Int3_r(m1:n1, m2:n2) = dat1%Int3_r(m1:n1, m2:n2) + dat2%Int3_r(m1:n1, m2:n2)
  endif
  !
  if (associated(dat1%Intxx3_r) .and. associated(dat2%Intxx3_r)) then
    dat1%Intxx3_r(m1:n1, m2:n2, :) = dat1%Intxx3_r(m1:n1, m2:n2, :) + dat2%Intxx3_r(m1:n1, m2:n2, :)
  endif
endif
!
! --- Add horizontal averages ---
call rhd_datx_AddOverlap(dat1%x23, dat2%x23)
call rhd_datx_AddOverlap(dat1%x13, dat2%x13)
call rhd_datx_AddOverlap(dat1%x12, dat2%x12)
!
if (par%grav_mode == 'central') then
  ! --- Add radial and cylindrical averages ---
  call rhd_datx_AddOverlap(dat1%r, dat2%r  )
  call rhd_datc_AddOverlap(dat1%c, dat2%c  )
endif
!
end subroutine rhd_dat_AddOverlap


!----------**************---------------------------------------------------------------------------
subroutine rhd_dat_Delete(dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_Delete ('rhd_data_Delete')
!
! PURPOSE:
!   Delete structure for additional data. Deallocate memory.
!   
! CATEGORY:
!   Dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_Delete(dat)
!
! INPUT:
!
! INPUT/OUTPUT:
!   dat:         ('data') type(dat_type), structure for additional data
!
! ROUTINES:
!   rhd_datc_Delete:       Delete structure for cylindrical mean data. Deallocate memory
!   rhd_datx_Delete:       Delete structure for additional data. Deallocate memory
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   Pointers are deallocated.
!
! RESTRICTIONS:
!   Structure dat has to be initialized (with rhd_dat_Make).
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1999-08-20 (B.F. Copenhagen) Written
!   2010-04-20 (B.F. Lyon) Rename delete_dat -> rhd_dat_Delete
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
type(dat_type),            intent(inout):: dat
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
dat%m1=0
dat%n1=0
dat%m2=0
dat%n2=0
dat%m3=0
dat%n3=0
dat%mr1=0
dat%nr1=0
dat%mr2=0
dat%nr2=0
dat%mr3=0
dat%nr3=0
dat%time=0.0
dat%itime=0
dat%luminosity=0.0
!
if (associated(dat%fer1_l)) deallocate(dat%fer1_l)
if (associated(dat%fer1_r)) deallocate(dat%fer1_r)
if (associated(dat%fer2_l)) deallocate(dat%fer2_l)
if (associated(dat%fer2_r)) deallocate(dat%fer2_r)
if (associated(dat%fer3_l)) deallocate(dat%fer3_l)
if (associated(dat%fer3_r)) deallocate(dat%fer3_r)
!
if (associated(dat%Int1_l)) deallocate(dat%Int1_l)
if (associated(dat%Int1_r)) deallocate(dat%Int1_r)
if (associated(dat%Int2_l)) deallocate(dat%Int2_l)
if (associated(dat%Int2_r)) deallocate(dat%Int2_r)
if (associated(dat%Int3_l)) deallocate(dat%Int3_l)
if (associated(dat%Int3_r)) deallocate(dat%Int3_r)
!
if (associated(dat%Intxx3_r)) deallocate(dat%Intxx3_r)
!
call rhd_datx_Delete(dat%x23)
call rhd_datx_Delete(dat%x13)
call rhd_datx_Delete(dat%x12)
!if (par%grav_mode == 'central') then
call rhd_datx_Delete(dat%r  )
call rhd_datc_Delete(dat%c  )
!endif
!
end subroutine rhd_dat_Delete


!----------**************---------------------------------------------------------------------------
subroutine rhd_datx_Write(ncout, box_id, box, datx0)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_Write ('rhd_data_x_Write')
!
! PURPOSE:
!   Write additional RHD data into file in UIO form.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_datx_Write(ncout, box_id, box, datx0)
!
! INPUT:
!   ncout:        ('number_channel_output') integer, unit number for data output
!   box_id:       ('box_identifier') character, identifier for box, default='z'
!   box:          ('box') type(box_type), computational box
!   datx0:        ('data_x_0') type(datx_type), structure for additional data
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_datx_Delete:       Delete structure for additional data. Deallocate memory
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_dim2st:            Compose dimension string
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!   uio routines have to be initialized.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2000-02-01 (B.F. Copenhagen) Written
!   2001-07-05 (B.F.) Factor 1.5 in scaling of central fluxes introduced
!   2001-09-05 (B.F.) Radial mass flux actual written
!   2004-05-12 (B.F. Montpellier) p2p viscosity
!   2004-05-13 (B.F.) Check if (datx%kapparho_xmean > 0)
!   2004-06-08 (B.F.) Radiative momentum change -> radiative acceleration
!   2010-04-20 (B.F. Lyon) Rename rhd_wrdatx -> rhd_datx_Write
!   2012-07-27 (B.F.) Move scale%length to prop%length
!   2013-08-15 (B.F.) Radial integration of p2p viscosity in proper branch
!   2013-08-20 (B.F.) Change some capitalization in the names of the output quantities
!   2015-03-21 (B.F. Uppsala) Transform the kind of some output arrays
!   2017-08-14 (B.F.) Take axes from global prop structure and not from input box;
!                     adapt routine to the new datx data type
!   2017-09-12 (B.F.) Rename routine by removing 'New'
!   2017-09-16 (B.F.) Correct radial integration of p2p viscosity (before, it was zero anyway)
!   2018-06-12 (B.F.) Write (non-)radial wave amplitudes of mass flux
!   2018-10-26 (B.F.) Write (non-)radial wave amplitudes of magnetic field
!   2019-02-08 (B.F.) Scale fer with relsurfrad
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use uio_filedef_module
use const_module
use rhd_gl_module
use rhd_box_module
use rhd_prop_module
use rhd_meankind_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(inout):: ncout
character(len=*),          intent(in)   :: box_id
type(box_type),            intent(in)   :: box
type(datx_type),           intent(in)   :: datx0
!
! --- Local variables ---
integer                                 :: m,n, m1,n1, m2,n2, m3,n3, i, iquc, iWave, lWave, mWave
integer, dimension(3)                   :: cindex, bindex
integer                                 :: ntt, bntt
character(len=let)                      :: termt(2,nttmx), btermt(2,nttmx)
character(len=80)                       :: shistr, VarName, lWaveStr, mWaveStr
type(datx_type)                         :: datx
real(kind=mArk), &
      dimension(datx0%m:datx0%n+1)      :: xb3_vol, xb3_surf, xb3_surfrad, fevp2p
real(kind=mArk)                         :: volume, fourpi_mArk, pi_mArk, length
!---------------------------------------------------------------------------------------------------
!
! --- Copy input dataset (the copy will be modified later) ---
datx=datx0
!
! --- Make abbreviations ---
m =datx%m
n =datx%n
!
m1=prop%m1
n1=prop%n1
m2=prop%m2
n2=prop%n2
m3=prop%m3
n3=prop%n3
!
length=real(prop%length, kind=mArk)
!
! === Normalize and process some quantities ========================================================
!
if ((box_id == 'r') .or. (box_id == 'rnew')) then
  ! --- Treat spherical averages ---
  !
  ! --- Normalize spherical averages ---
  do i=1,datx%na
    if ((i == datx%i_relvol    ) .or. &
        (i == datx%i_relsurf   ) .or. &
        (i == datx%i_relsurfrad)       ) then
      ! --- Weight: skip normalization ---
      !
    else if (((i >= datx%i_fquc    )        .and. &
              (i < (datx%i_fquc+prop%nquc)) .and. &
              (0 < datx%i_fquc)                    ) .or. &
             ((i >= datx%i_frhowave)             .and. &
              (i < (datx%i_frhowave+datx%nWave)) .and. &
              (0 <  datx%i_frhowave)             .and. &
              (0 < datx%nWave)                          ) .or. &
             ((i >= datx%i_Bbwave)             .and. &
              (i < (datx%i_Bbwave+datx%nWave)) .and. &
              (0 <  datx%i_Bbwave)             .and. &
              (0 < datx%nWave)                          ) .or. &
              (i == datx%i_runiflux) .or. &
              (i == datx%i_runilum ) .or. &
              (i == datx%i_Bb      ) .or. &
              (i == datx%i_Bbabs   ) .or. &
              (i == datx%i_Bb2     ) .or. &
              (i == datx%i_fem     ) .or. &
              (i == datx%i_fey     ) .or. &
              (i == datx%i_frho    ) .or. &
              (i == datx%i_frhofrho) .or. &
              (i == datx%i_feip    ) .or. &
              (i == datx%i_fek     ) .or. &
              (i == datx%i_feg     ) .or. &
              (i == datx%i_fep     ) .or. &
              (i == datx%i_fev     )       ) then
      ! --- Flux-like surface quantity: normalize with relative surface, ---
      ! --- multiply with 6*pi*r^2/4pi*r^2                               ---
      where (datx%d(:,datx%i_relsurf) > 0.0_mArk)
        datx%d(:,i)=1.5_mArk*datx%d(:,i)/datx%d(:,datx%i_relsurf)
      endwhere
      !
    else if (i == datx%i_fer) then
      ! --- Flux-like surface quantity: normalize with relative surface, ---
      ! --- multiply with 6*pi*r^2/4pi*r^2                               ---
      where (datx%d(:,datx%i_relsurfrad) > 0.0_mArk)
        datx%d(:,i)=1.5_mArk*datx%d(:,i)/datx%d(:,datx%i_relsurfrad)
      endwhere
      !
    else if (i == datx%i_rhotil) then
      ! --- Surface quantity: normalize with relative surface ---
      where (datx%d(:,datx%i_relsurf) > 0.0_mArk)
        datx%d(:,i)=datx%d(:,i)/datx%d(:,datx%i_relsurf)
      endwhere
      !
    else ! (i)
      ! --- Volume quantity: normalize with relative volume ---
      where (datx%d(m:n,datx%i_relvol) > 0.0_mArk)
        datx%d(m:n,i)=datx%d(m:n,i)/datx%d(m:n,datx%i_relvol)
      endwhere
      !
    endif ! (i)
  end do ! i
  !
  ! --- Compute "real" xb from volumes and surface areas ---
  xb3_vol(m)=datx%d1(m,datx%i_xb)
  volume=0.0_mArk
  if (mArk == kind(0.0E+00)) then
    fourpi_mArk=fourpi_srk
    pi_mArk    =pi_srk
  else
    fourpi_mArk=fourpi_drk
    pi_mArk    =pi_drk
  endif
  do i=m,n
    volume      =volume + datx%d(i,datx%i_relvol)
    xb3_vol(i+1)=length * ((volume/(fourpi_mArk/3.0_mArk))**(1.0_mArk/3.0_mArk))
  end do ! i
  !
  xb3_surf(   m)=datx%d1(m,datx%i_xb)
  xb3_surfrad(m)=datx%d1(m,datx%i_xb)
  do i=m+1,n+1
    xb3_surf(   i)=length * sqrt(datx%d(i,datx%i_relsurf   )/(6.0_mArk*pi_mArk))
    xb3_surfrad(i)=length * sqrt(datx%d(i,datx%i_relsurfrad)/(6.0_mArk*pi_mArk))
  end do ! i
  !
endif ! (box_id == 'r')
!
! --- Process special averages ---
if (datx%i_rho2     > 0) datx%d(m:n  ,datx%i_rho2    )=     sqrt(datx%d(m:n  ,datx%i_rho2)    )
if (datx%i_p2       > 0) datx%d(m:n  ,datx%i_p2      )=     sqrt(datx%d(m:n  ,datx%i_p2)      )
if (datx%i_s2       > 0) datx%d(m:n  ,datx%i_s2      )=     sqrt(datx%d(m:n  ,datx%i_s2)      )
if (datx%i_T2       > 0) datx%d(m:n  ,datx%i_T2      )=     sqrt(datx%d(m:n  ,datx%i_T2)      )
if (datx%i_T4       > 0) datx%d(m:n  ,datx%i_T4      )=sqrt(sqrt(datx%d(m:n  ,datx%i_T4)      ))
if (datx%i_frhofrho > 0) datx%d(m:n+1,datx%i_frhofrho)=     sqrt(datx%d(m:n+1,datx%i_frhofrho))
!
if ((datx%i_kapparho > 0) .and. (datx%i_T4kaprho > 0)) then
  if (minval(datx%d(m:n,datx%i_kapparho)) > 0.0_mArk) then
    datx%d(m:n,datx%i_T4kaprho)=sqrt(sqrt(datx%d(m:n,datx%i_T4kaprho) / &
                                          datx%d(m:n,datx%i_kapparho)  ))
  else
    datx%d(m:n,datx%i_T4kaprho)=0.0_mArk
  endif
endif
!
! --- Integrate viscous p2p flux ---
fevp2p(m)=0.0_mArk
if (datx%i_devp2p > 0) then
  select case (box_id)
    case('x23', 'x1', 'z1', 'z1new')
      do i=m+1,n+1
        fevp2p(i)=fevp2p(i-1) - (prop%xb1(i) - prop%xb1(i-1)) * datx%d(i-1,datx%i_devp2p)
      end do ! i
    case('x13', 'x2', 'z2', 'z2new')
      do i=m+1,n+1
        fevp2p(i)=fevp2p(i-1) - (prop%xb2(i) - prop%xb2(i-1)) * datx%d(i-1,datx%i_devp2p)
      end do ! i
    case('x12', 'x3', 'z3', 'z3new')
      do i=m+1,n+1
        fevp2p(i)=fevp2p(i-1) - (prop%xb3(i) - prop%xb3(i-1)) * datx%d(i-1,datx%i_devp2p)
      end do ! i
    case('r', 'rnew')
      do i=m+1,n+1
        !fevp2p(i)=fevp2p(i-1) - (datx%xb(i) - datx%xb(i-1)) * datx%d(i-1,datx%i_devp2p)
        fevp2p(i)=fevp2p(i-1) * (datx%d(i-1,datx%i_relsurf)/datx%d(i,datx%i_relsurf)) - &
                  datx%d(i-1,datx%i_devp2p) * length * &
                    (datx%d(i-1,datx%i_relvol)/(fourpi_mArk*(xb3_surf(i)/length)**2))
      end do ! i
  end select ! (box_id)
endif
!
! --- Transform radiative momentum change to radiative acceleration ---
if (datx%i_grad > 0) datx%d(m:n,datx%i_grad)=datx%d(m:n,datx%i_grad)/datx%d(m:n,datx%i_rho)
!
! === Write general model data (from box and prop) =================================================
!
! --- Write start box label ---
call uio_wrlabl(ncout, box_ident, date='now')
!
call uio_wr(ncout, box_id, box_id_ident, name='Block identification')
!
! --- Write block dimension ---
select case (box_id)
  case('x23', 'x1', 'z1', 'z1new')
    call uio_wr(ncout, reshape((/ m1,n1, m2,m2, m3,m3 /), (/2,3/)), 'dimension', &
                format='I7', nperlin=6)
    cindex=(/n1-m1+1,1,1/)
    bindex=(/n1-m1+2,1,1/)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  0 /), shistr)
    bntt=0
    call uio_adkeyw(btermt,bntt,'ds',shistr)
  case('x13', 'x2', 'z2', 'z2new')
    call uio_wr(ncout, reshape((/ m1,m1, m2,n2, m3,m3 /), (/2,3/)), 'dimension', &
                format='I7', nperlin=6)
    cindex=(/1,n2-m2+1,1/)
    bindex=(/1,n2-m2+2,1/)
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  0 /), shistr)
    bntt=0
    call uio_adkeyw(btermt,bntt,'ds',shistr)
  case('x12', 'x3', 'z3', 'z3new')
    call uio_wr(ncout, reshape((/ m1,m1, m2,m2, m3,n3 /), (/2,3/)), 'dimension', &
                format='I7', nperlin=6)
    cindex=(/1,1,n3-m3+1/)
    bindex=(/1,1,n3-m3+2/)
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  0,  1 /), shistr)
    bntt=0
    call uio_adkeyw(btermt,bntt,'ds',shistr)
  case('r', 'rnew')
    m3=m
    n3=n
    call uio_wr(ncout, reshape((/ m1,m1, m2,m2, m3,n3  /), (/2,3/)), 'dimension', &
                format='I7', nperlin=6)
    cindex=(/1,1,n3-m3+1/)
    bindex=(/1,1,n3-m3+2/)
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  0,  1 /), shistr)
    bntt=0
    call uio_adkeyw(btermt,bntt,'ds',shistr)
end select
!
! --- Write scalars ---
!
! --- Write time ---
call uio_wr(ncout, box%time   , 'time'   , name='time', unit=time_unit)
call uio_wr(ncout, box%time_db, 'time_db', name='time', unit=time_unit)
!
! --- Write time step number ---
call uio_wr(ncout, box%itime, 'itime', name='time step number', unit='1')
!
! --- Write axes ---
!
select case (box_id)
  case('x23', 'x1', 'z1', 'z1new')
    ! --- Write grid: 1D vectors: spread -> 3D ---
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape(prop%xc1                            , (/n1-m1+1,1,1/)), 'xc1', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x1 coordinates of cell centers', unit=x1_unit, &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.5*(prop%xb2(m2)+prop%xb2(n2+1))/), (/1      ,1,1/)), 'xc2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x2 coordinates of cell centers', unit=x2_unit, &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.5*(prop%xb3(m3)+prop%xb3(n3+1))/), (/1      ,1,1/)), 'xc3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x3 coordinates of cell centers', unit=x3_unit, &
                ntt=ntt, termt=termt)
    !
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape(prop%xb1                      , (/n1-m1+2,1,1/)), 'xb1', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x1 coordinates of cell boundaries', unit=x1_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape((/prop%xb2(m2),prop%xb2(n2+1)/), (/1      ,2,1/)), 'xb2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x2 coordinates of cell boundaries', unit=x2_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape((/prop%xb3(m3),prop%xb3(n3+1)/), (/1      ,1,2/)), 'xb3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x3 coordinates of cell boundaries', unit=x3_unit, &
                ntt=ntt, termt=termt)
  !
  case('x13', 'x2', 'z2', 'z2new')
    ! --- Write grid: 1D vectors: spread -> 3D ---
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.5*(prop%xb1(m1)+prop%xb1(n1+1))/), (/1,1      ,1/)), 'xc1', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x1 coordinates of cell centers', unit=x1_unit, &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape(prop%xc2                            , (/1,n2-m2+1,1/)), 'xc2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x2 coordinates of cell centers', unit=x2_unit, &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.5*(prop%xb3(m3)+prop%xb3(n3+1))/), (/1,1      ,1/)), 'xc3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x3 coordinates of cell centers', unit=x3_unit, &
                ntt=ntt, termt=termt)
    !
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/prop%xb1(m1),prop%xb1(n1+1)/), (/2,1      ,1/)), 'xb1', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x1 coordinates of cell boundaries', unit=x1_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape(prop%xb2                      , (/1,n2-m2+2,1/)), 'xb2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x2 coordinates of cell boundaries', unit=x2_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape((/prop%xb3(m3),prop%xb3(n3+1)/), (/1,1      ,2/)), 'xb3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x3 coordinates of cell boundaries', unit=x3_unit, &
                ntt=ntt, termt=termt)
  !
  case('x12', 'x3', 'z3', 'z3new')
    ! --- Write grid: 1D vectors: spread -> 3D ---
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.5*(prop%xb1(m1)+prop%xb1(n1+1))/), (/1,1,1      /)), 'xc1', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x1 coordinates of cell centers', unit=x1_unit, &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.5*(prop%xb2(m2)+prop%xb2(n2+1))/), (/1,1,1      /)), 'xc2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x2 coordinates of cell centers', unit=x2_unit, &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape(prop%xc3                            , (/1,1,n3-m3+1/)), 'xc3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x3 coordinates of cell centers', unit=x3_unit, &
                ntt=ntt, termt=termt)
    !
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/prop%xb1(m1),prop%xb1(n1+1)/), (/2,1,1      /)), 'xb1', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x1 coordinates of cell boundaries', unit=x1_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape((/prop%xb2(m2),prop%xb2(n2+1)/), (/1,2,1      /)), 'xb2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x2 coordinates of cell boundaries', unit=x2_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape(prop%xb3                      , (/1,1,n3-m3+2/)), 'xb3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='x3 coordinates of cell boundaries', unit=x3_unit, &
                ntt=ntt, termt=termt)
  case('r', 'rnew')
    ! --- Write grid: 1D vectors: spread -> 3D ---
    call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/pi/) , (/1,1,1    /)), 'xc1', &
                mode=8, ishift=(/1,1,m3/), &
                name='phi coordinates of cell centers', unit='1', &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.0/), (/1,1,1    /)), 'xc2', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='theta coordinates of cell centers', unit='1', &
                ntt=ntt, termt=termt)
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape(datx%d1(m:n,datx%i_xc), (/1,1,n-m+1/)), 'xc3', &
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell centers', unit=x3_unit, &
                ntt=ntt, termt=termt)
    !
    call uio_wr(ncout, &
                reshape(real(0.5_mArk*(xb3_vol(m:n)+xb3_vol(m+1:n+1)),kind=mOrk),  (/1,1,n-m+1/)), &
                'xc3_volume', &
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell centers (volume preserving)', unit=x3_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, &
                reshape(real(0.5_mArk*(xb3_surf(m:n)+xb3_surf(m+1:n+1)),kind=mOrk), (/1,1,n-m+1/)),&
                'xc3_surface',&
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell centers (surface preserving)', unit=x3_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, &
                reshape(real(0.5_mArk*(xb3_surfrad(m:n)+xb3_surfrad(m+1:n+1)),kind=mOrk), &
                                                                                    (/1,1,n-m+1/)),&
                'xc3_surfacerad',&
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell centers (surface preserving)', unit=x3_unit, &
                ntt=ntt, termt=termt)
    !
    call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
    ntt=0
    call uio_adkeyw(termt,ntt,'ds',shistr)
    call uio_wr(ncout, reshape((/0.0    ,twopi/) , (/2,1,1    /)), 'xb1', &
                mode=8, ishift=(/1,1,m/), &
                name='phi coordinates of cell boundaries', unit='1', &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape((/-halfpi,halfpi/), (/1,2,1    /)), 'xb2', &
                mode=8, ishift=(/1,1,m/), &
                name='theta coordinates of cell boundaries', unit='1', &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape(datx%d1(:,datx%i_xb)           , (/1,1,n-m+2/)), 'xb3', &
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell boundaries', unit=x3_unit, &
                ntt=ntt, termt=termt)
    !
    call uio_wr(ncout, reshape(real(xb3_vol,kind=mOrk), (/1,1,n-m+2/)), 'xb3_volume', &
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell boundaries (volume preserving)', unit=x3_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape(real(xb3_surf,kind=mOrk), (/1,1,n-m+2/)), 'xb3_surface', &
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell boundaries (surface preserving)', unit=x3_unit, &
                ntt=ntt, termt=termt)
    call uio_wr(ncout, reshape(real(xb3_surfrad,kind=mOrk), (/1,1,n-m+2/)), 'xb3_surfacerad', &
                mode=8, ishift=(/1,1,m/), &
                name='Radius coordinates of cell boundaries (surface preserving)', unit=x3_unit, &
                ntt=ntt, termt=termt)
end select ! (box_id)
!
! === Write volume data ============================================================================
!
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rho),kind=mOrk), cindex), 'rho_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Density', unit=rho_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rho2),kind=mOrk), cindex), 'rho_xmean2', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Density', unit=rho_unit)
if ((box_id == 'r') .or. (box_id == 'rnew')) then
  call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_relvol),kind=mOrk), cindex), 'relvol_xmean', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='Relative volume', unit='1')
endif
!
! --- Write vector fields ---
!
select case (box_id)
  case('x23', 'x1', 'z1', 'z1new')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v1),kind=mOrk), cindex), 'v1_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v2),kind=mOrk), cindex), 'v2_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v3),kind=mOrk), cindex), 'v3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v1v1)),kind=mOrk), cindex), 'v1_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v2v2)),kind=mOrk), cindex), 'v2_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v3v3)),kind=mOrk), cindex), 'v3_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real((datx%d(m:n,datx%i_v1v1v1))**(1.0_mArk/3.0_mArk),kind=mOrk), &
                                                                             cindex), 'v1_xmean3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(sqrt(datx%d(m:n,datx%i_v1v1v1v1))),kind=mOrk), cindex), &
                                                                                   'v1_xmean4', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v1upfrac),kind=mOrk), cindex), &
                                                                           'v1upfraction_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1 upward fraction', unit='1')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1),kind=mOrk), cindex), 'rhov1_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x1', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2),kind=mOrk), cindex), 'rhov2_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x2', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3),kind=mOrk), cindex), 'rhov3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x3', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v1),kind=mOrk), cindex),'rhov1v1_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x1', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2v2),kind=mOrk), cindex),'rhov2v2_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x2x2', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3v3),kind=mOrk), cindex),'rhov3v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x3x3', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v2),kind=mOrk), cindex),'rhov1v2_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x2', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v3),kind=mOrk), cindex),'rhov1v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x3', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2v3),kind=mOrk), cindex),'rhov2v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x2x3', unit=rhovv_unit)
!#
#   if (rhd_box_bmag01 != 0)
!#
    if (datx%i_Bc1 > 0) then
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1),kind=mOrk), cindex), 'bc1_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc2),kind=mOrk), cindex), 'bc2_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc3),kind=mOrk), cindex), 'bc3_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1abs),kind=mOrk), cindex),'bc1_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc2abs),kind=mOrk), cindex),'bc2_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc3abs),kind=mOrk), cindex),'bc3_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc12)),kind=mOrk), cindex), &
                                                                                     'bc1_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc22)),kind=mOrk), cindex), &
                                                                                     'bc2_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc32)),kind=mOrk), cindex), &
                                                                                     'bc3_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bb),kind=mOrk), bindex), 'bb1_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bbabs),kind=mOrk), bindex), &
                                                                                   'bb1_xabsmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(:,datx%i_Bb2)) ,kind=mOrk), bindex),'bb1_xmean2',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
    endif ! (datx%i_Bc1 > 0)
!#
#   endif
!#        /* rhd_box_bmag01 */
    !
  case('x13', 'x2', 'z2', 'z2new')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v2),kind=mOrk), cindex), 'v1_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v1),kind=mOrk), cindex), 'v2_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v3),kind=mOrk), cindex), 'v3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v2v2)),kind=mOrk), cindex), 'v1_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v1v1)),kind=mOrk), cindex), 'v2_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v3v3)),kind=mOrk), cindex), 'v3_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real((datx%d(m:n,datx%i_v1v1v1))**(1.0_mArk/3.0_mArk),kind=mOrk), &
                                                                            cindex), 'v2_xmean3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(sqrt(datx%d(m:n,datx%i_v1v1v1v1))),kind=mOrk), cindex), &
                                                                                    'v2_xmean4', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v1upfrac),kind=mOrk), cindex), &
                                                                           'v2upfraction_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2 upward fraction', unit='1')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2),kind=mOrk), cindex), 'rhov1_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x1', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1),kind=mOrk), cindex), 'rhov2_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x2', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3),kind=mOrk), cindex), 'rhov3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x3', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2v2),kind=mOrk), cindex),'rhov1v1_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x1', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v1),kind=mOrk), cindex),'rhov2v2_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x2x2', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3v3),kind=mOrk), cindex),'rhov3v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x3x3', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v2),kind=mOrk), cindex),'rhov1v2_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x2', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2v3),kind=mOrk), cindex),'rhov1v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x3', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v3),kind=mOrk), cindex),'rhov2v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x2x3', unit=rhovv_unit)
!#
#   if (rhd_box_bmag01 != 0)
!#
    if (datx%i_Bc1 > 0) then
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc2),kind=mOrk), cindex), 'bc1_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1),kind=mOrk), cindex), 'bc2_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc3),kind=mOrk), cindex), 'bc3_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc2abs),kind=mOrk), cindex),'bc1_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1abs),kind=mOrk), cindex),'bc2_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc3abs),kind=mOrk), cindex),'bc3_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc22)),kind=mOrk), cindex), &
                                                                                     'bc1_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc12)),kind=mOrk), cindex), &
                                                                                     'bc2_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc32)),kind=mOrk), cindex), &
                                                                                     'bc3_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bb),kind=mOrk), bindex), 'bb2_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bbabs),kind=mOrk), bindex), &
                                                                                   'bb2_xabsmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(:,datx%i_Bb2)) ,kind=mOrk), bindex),'bb2_xmean2',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
    endif ! (datx%i_Bc1 > 0)
!#
#   endif
!#        /* rhd_box_bmag01 */
    !
  case('x12', 'x3', 'z3', 'z3new')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v3),kind=mOrk), cindex), 'v1_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v2),kind=mOrk), cindex), 'v2_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v1),kind=mOrk), cindex), 'v3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v3v3)),kind=mOrk), cindex), 'v1_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x1', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v2v2)),kind=mOrk), cindex), 'v2_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x2', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v1v1)),kind=mOrk), cindex), 'v3_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real((datx%d(m:n,datx%i_v1v1v1))**(1.0_mArk/3.0_mArk),kind=mOrk), &
                                                                          cindex), 'v3_xmean3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(sqrt(datx%d(m:n,datx%i_v1v1v1v1))),kind=mOrk), cindex), &
                                                                                   'v3_xmean4', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3', unit=v_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v1upfrac),kind=mOrk), cindex), &
                                                                           'v3upfraction_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity x3 upward fraction', unit='1')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3),kind=mOrk), cindex), 'rhov1_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x1', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2),kind=mOrk), cindex), 'rhov2_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x2', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1),kind=mOrk), cindex), 'rhov3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux x3', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3v3),kind=mOrk), cindex),'rhov1v1_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x1', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2v2),kind=mOrk), cindex),'rhov2v2_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x2x2', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v1),kind=mOrk), cindex),'rhov3v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x3x3', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov2v3),kind=mOrk), cindex),'rhov1v2_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x2', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v3),kind=mOrk), cindex),'rhov1v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x3', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v2),kind=mOrk), cindex),'rhov2v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x2x3', unit=rhovv_unit)
!#
#   if (rhd_box_bmag01 != 0)
!#
    if (datx%i_Bc1 > 0) then
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc3),kind=mOrk), cindex), 'bc1_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc2),kind=mOrk), cindex), 'bc2_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1),kind=mOrk), cindex), 'bc3_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc3abs),kind=mOrk), cindex),'bc1_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc2abs),kind=mOrk), cindex),'bc2_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1abs),kind=mOrk), cindex),'bc3_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc32)),kind=mOrk), cindex), &
                                                                                     'bc1_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc22)),kind=mOrk), cindex), &
                                                                                     'bc2_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B2_name, unit=prop%B2_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc12)),kind=mOrk), cindex), &
                                                                                     'bc3_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bb),kind=mOrk), bindex), 'bb3_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bbabs),kind=mOrk), bindex), &
                                                                                   'bb3_xabsmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(:,datx%i_Bb2)) ,kind=mOrk), bindex),'bb3_xmean2',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
    endif ! (datx%i_Bc1 > 0)
!#
#   endif
!#        /* rhd_box_bmag01 */
    !
  case('r', 'rnew')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_v3),kind=mOrk), cindex), 'v3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity radial', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v1v1)),kind=mOrk), cindex), 'v1_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity tangential', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v3v3)),kind=mOrk), cindex), 'v3_xmean2',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity radial', unit=v_unit)
    call uio_wr(ncout, reshape(real((datx%d(m:n,datx%i_v1v1v1))**(1.0_mArk/3.0_mArk),kind=mOrk), &
                                                                          cindex), 'v3_xmean3', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity radial', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(sqrt(datx%d(m:n,datx%i_v1v1v1v1))),kind=mOrk), cindex), &
                                                                                   'v3_xmean4', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity radial', unit=v_unit)
    call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_v1upfrac)),kind=mOrk), cindex), &
                                                                             'v3upfraction_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Velocity radial outward fraction', unit='1')
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3),kind=mOrk), cindex), 'rhov3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Mass flux radial', unit=rhov_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov1v1),kind=mOrk), cindex),'rhov1v1_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x1x1 (tangential)', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhov3v3),kind=mOrk), cindex),'rhov3v3_xmean',&
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum flux x3x3 (radial)', unit=rhovv_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_MomIne3),kind=mOrk), cindex), &
                                                                         'momentofinertia3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Moment of inertia', unit=MomIne_unit)
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_AngMom3),kind=mOrk), cindex), &
                                                                         'angularmomentum3_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Angular momentum', unit=AngMom_unit)
!#
#   if (rhd_box_bmag01 != 0)
!#
    if (datx%i_Bc1 > 0) then
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1),kind=mOrk), cindex), 'bc3_xmean', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bc1abs),kind=mOrk), cindex),'bc3_xabsmean',&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc12)),kind=mOrk), cindex), &
                                                                                     'bc1_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B1_name, unit=prop%B1_unit)
      call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bc32)),kind=mOrk), cindex), &
                                                                                     'bc3_xmean2', &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name=prop%B3_name, unit=prop%B3_unit)
    endif ! (datx%i_Bc1 > 0)
!#
#   endif
!#        /* rhd_box_bmag01 */
end select ! (box_id)
!
! --- Write scalar fields ---
!
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_ei),kind=mOrk), cindex), 'ei_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Internal energy', unit=e_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhoei),kind=mOrk), cindex), 'rhoei_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Internal energy', unit=rhoe_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhoek),kind=mOrk), cindex), 'rhoek_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Kinetic energy', unit=rhoe_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhoeg),kind=mOrk), cindex), 'rhoeg_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Gravitational energy', unit=rhoe_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_der),kind=mOrk), cindex), 'drhoerdtime_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Radiative change of internal energy', unit=luminpervol_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_T),kind=mOrk), cindex), 't_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Temperature', unit=T_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_T2),kind=mOrk), cindex), 't_xmean2', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Temperature', unit=T_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_T4),kind=mOrk), cindex), 't_xmean4', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Temperature', unit=T_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_T4kaprho),kind=mOrk), cindex), &
                                                                           't_xmeankapparho', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Temperature', unit=T_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_p),kind=mOrk), cindex), 'p_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Pressure', unit=p_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_p2),kind=mOrk), cindex), 'p_xmean2', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Pressure', unit=p_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_s),kind=mOrk), cindex), 's_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Entropy', unit=s_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_s2),kind=mOrk), cindex), 's_xmean2', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Entropy', unit=s_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_rhos),kind=mOrk), cindex), 'rhos_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Entropy', unit=rhos_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Gamma1),kind=mOrk), cindex), 'gamma1_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='1st adiabatic coefficient', unit='1')
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Gamma3),kind=mOrk), cindex), 'gamma3_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='3rd adiabatic coefficient', unit='1')
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_delta),kind=mOrk), cindex), 'delta_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Expansion coefficient', unit='1')
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_kapparho),kind=mOrk), cindex), 'kapparho_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Absorption coefficient', unit=kapparho_unit)
call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_grad),kind=mOrk), cindex), 'gravrad_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Radiative acceleration', unit=grav_unit)
!#
# if (rhd_box_quc01 != 0)
!#
if (datx%i_quc > 0) then
  do iquc=1,prop%nquc
    call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_quc+(iquc-1)),kind=mOrk), cindex), &
                trim(prop%quc_ident(iquc)) // '_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name=prop%quc_name(iquc), unit=prop%quc_unit(iquc))
  end do ! iquc
endif
!#
# endif
!#      /* rhd_box_quc01 */
!#
# if (rhd_box_bmag01 != 0)
!#
if (datx%i_Bh > 0) then
  call uio_wr(ncout, reshape(real(datx%d(m:n,datx%i_Bh),kind=mOrk), cindex), 'bh_xmean', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='Horizontal magnetic field', unit=prop%B1_unit)
  call uio_wr(ncout, reshape(real(sqrt(datx%d(m:n,datx%i_Bhrms)),kind=mOrk), cindex),'bhrms_xmean',&
              mode=8, ishift=(/m1,m2,m3/), &
              name='RMS horizontal magnetic field', unit=prop%B2_unit)
endif
!#
# endif
!#      /* rhd_box_bmag01 */
!
! --- Write fluxes ---------------------------------------------------------------------------------
!
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frho),kind=mOrk), bindex), 'rhovb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Mass flux', unit=rhov_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhofrho),kind=mOrk), bindex), 'rhovb_xmean2', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Mass flux', unit=rhov_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_rhotil),kind=mOrk), bindex), 'rhotil_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Roe-averaged boundary density', unit=rho_unit, &
            ntt=bntt, termt=btermt)
!#
# if (rhd_box_quc01 != 0)
!#
if (datx%i_fquc > 0) then
  do iquc=1,prop%nquc
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fquc+(iquc-1)),kind=mOrk), bindex), &
                'f' // trim(prop%quc_ident(iquc)) // '_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Flux of ' // prop%quc_name(iquc), &
                unit=trim(prop%quc_unit(iquc)) // '*' // v_unit, &
                ntt=bntt, termt=btermt)
  end do ! iquc
endif
!#
# endif
!#      /* rhd_box_quc01 */
if ((box_id == 'r') .or. (box_id == 'rnew')) then
  call uio_wr(ncout, reshape(real(datx%d(:,datx%i_relsurf),kind=mOrk), bindex), 'relsurf_xmean', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='Relative surface area', unit='1', &
              ntt=bntt, termt=btermt)
  !
  call uio_wr(ncout, reshape(real(datx%d(:,datx%i_relsurfrad),kind=mOrk), bindex), &
                                                                             'relsurfrad_xmean', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='Relative surface area of non-local radiation transport', unit='1', &
              ntt=bntt, termt=btermt)
  !
  if (datx%i_runiflux > 0) then
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_runiflux),kind=mOrk), bindex), &
                                                                        'radialunityflux_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Radial unity flux vector', unit='1', &
                ntt=bntt, termt=btermt)
  endif
  if (datx%i_runilum > 0) then
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_runilum),kind=mOrk), bindex), &
                                                                        'radialunitylum_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Radial unity luminosity vector', unit='1', &
                ntt=bntt, termt=btermt)
  endif
  !
  ! --- Write wave amplitudes: mass flux ---
  if ((datx%nWave > 0) .and. (datx%i_frhowave > 0)) then
    do iWave=0,datx%nWave-1
      lWave=floor(sqrt(real(iWave)+0.1))
      mWave=iWave - lWave**2 - lWave
      !
      write(lWaveStr,'(I3)') lWave
      lWaveStr='00' // adjustl(lWaveStr)
      lWaveStr=lWaveStr(1:3)
      !
      write(mWaveStr,'(I3)') abs(mWave)
      mWaveStr='00' // adjustl(mWaveStr)
      mWaveStr=mWaveStr(1:3)
      if (mWave < 0) then
        mWaveStr='m' // adjustl(mWaveStr)
      else
        mWaveStr='p' // adjustl(mWaveStr)
      endif
      !
      VarName='rhovb' // 'l' // trim(lWaveStr) // 'm' // trim(mWaveStr) // '_xmean'
      !write(6,*) iWave, trim(VarName)
      !
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhowave+iWave),kind=mOrk), bindex), VarName,&
                  mode=8, ishift=(/m1,m2,m3/), &
                  name='Mass flux l=' // trim(lWaveStr) // ' m=' // trim(mWaveStr), unit=rhov_unit,&
                  ntt=bntt, termt=btermt)
    end do ! iWave
  endif ! ((datx%nWave > 0) .and. (datx%i_frhowave > 0))
  !
  ! --- Write wave amplitudes: magnetic field ---
  if ((datx%nWave > 0) .and. (datx%i_Bbwave > 0)) then
    do iWave=0,datx%nWave-1
      lWave=floor(sqrt(real(iWave)+0.1))
      mWave=iWave - lWave**2 - lWave
      !
      write(lWaveStr,'(I3)') lWave
      lWaveStr='00' // adjustl(lWaveStr)
      lWaveStr=lWaveStr(1:3)
      !
      write(mWaveStr,'(I3)') abs(mWave)
      mWaveStr='00' // adjustl(mWaveStr)
      mWaveStr=mWaveStr(1:3)
      if (mWave < 0) then
        mWaveStr='m' // adjustl(mWaveStr)
      else
        mWaveStr='p' // adjustl(mWaveStr)
      endif
      !
      VarName='bb' // 'l' // trim(lWaveStr) // 'm' // trim(mWaveStr) // '_xmean'
      write(6,*) iWave, trim(VarName)
      !
      call uio_wr(ncout, reshape(real(datx%d(:,datx%i_Bbwave+iWave),kind=mOrk), bindex), VarName, &
                  mode=8, ishift=(/m1,m2,m3/), &
                  name='Magnetic field l=' // trim(lWaveStr) // ' m=' // trim(mWaveStr), &
                                                                                unit=prop%B1_unit, &
                  ntt=bntt, termt=btermt)
    end do ! iWave
  endif ! ((datx%nWave > 0) .and. (datx%i_Bbwave > 0))
endif ! (box_id == 'r')
!
select case (box_id)
  case('x23', 'x1', 'z1', 'z1new')
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov1),kind=mOrk), bindex), 'frhov11b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x1 flux x1 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov2),kind=mOrk), bindex), 'frhov21b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x2 flux x1 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov3),kind=mOrk), bindex), 'frhov31b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x3 flux x1 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
  case('x13', 'x2', 'z2', 'z2new')
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov2),kind=mOrk), bindex), 'frhov12b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x1 flux x2 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov1),kind=mOrk), bindex), 'frhov22b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x2 flux x2 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov3),kind=mOrk), bindex), 'frhov32b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x3 flux x2 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
  case('x12', 'x3', 'z3', 'z3new')
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov3),kind=mOrk), bindex), 'frhov13b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x1 flux x3 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov2),kind=mOrk), bindex), 'frhov23b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x2 flux x3 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
    call uio_wr(ncout, reshape(real(datx%d(:,datx%i_frhov1),kind=mOrk), bindex), 'frhov33b_xmean', &
                mode=8, ishift=(/m1,m2,m3/), &
                name='Momentum x3 flux x3 direction', unit=rhoe_unit, &
                ntt=bntt, termt=btermt)
end select ! (box_id)
!
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_feip),kind=mOrk), bindex), 'feipb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Enthalpy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fek),kind=mOrk), bindex), 'fekb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Kinetic energy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_feg),kind=mOrk), bindex), 'fegb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Gravitational energy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fep),kind=mOrk), bindex), 'fepb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Pressure energy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fev),kind=mOrk), bindex), 'fevb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Viscous energy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(fevp2p,kind=mOrk), bindex),            'fevp2pb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Viscous P2P energy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fer),kind=mOrk), bindex), 'ferb_xmean', &
            mode=8, ishift=(/m1,m2,m3/), &
            name='Radiative energy flux', unit=fe_unit, &
            ntt=bntt, termt=btermt)
!#
# if (rhd_box_bmag01 != 0)
!#
if (datx%i_fem > 0) then
  call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fem),kind=mOrk), bindex), 'femb_xmean', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='Poynting flux', unit=fe_unit, &
              ntt=bntt, termt=btermt)
  call uio_wr(ncout, reshape(real(datx%d(:,datx%i_fey),kind=mOrk), bindex), 'feyb_xmean', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='Dissipative Poynting flux', unit=fe_unit, &
              ntt=bntt, termt=btermt)
endif ! (datx%i_fem > 0)
!#
# endif
!#      /* rhd_box_bmag01 */
!
! --- Write end block label ---
call uio_wrlabl(ncout, endbox_ident)
!
! --- Delete temporary dataset ---
call rhd_datx_Delete(datx)
!
end subroutine rhd_datx_Write


!----------**************---------------------------------------------------------------------------
subroutine rhd_datc_Write(command, nc_p, ncout, file, form, conv, &
                          ndes, description, nhis, history, version, &
                          box, datc0)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_Write ('rhd_data_cylindrical_Write')
!
! PURPOSE:
!   Write cylindrical means of RHD data into file in UIO form.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_datc_Write(command, nc_p, ncout, file=file, form=form, conv=conv, &
!                       ndes=ndes, description=description, &
!                       nhis=nhis, history=history, version=version, &
!                       box=box, datc0=datc0)
!
! INPUT:
!   command:      ('command') character, command string:
!                 'open'/'append', 'read', 'model, 'box', 'close'
!   nc_p:         ('number_channel_print') integer, unit number for text output
!   --- optional: ---
!   file:         ('file') character, name of data file
!   form:         ('format') character, name of data file
!   conv:         ('conversion') character, conversion type of data file
!   ndes:         ('number_description') integer, number of lines of description
!   description:  ('description') character, dimension(:), description of model
!   nhis:         ('number_history') integer, number of lines of history
!   history:      ('history') character, dimension(:), history of model
!   version:      ('version') character, string with RHD version
!   box:          ('box') type(box_type), computational box
!   datc0:        ('data_cylindrical_0') type(datc_type), structure for additional data
!
! INPUT/OUTPUT:
!   ncout:        ('number_channel_output') integer, unit number for data output
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_datc_Delete:       Delete structure for cylindrical mean data. Deallocate memory
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_closwr:            Close file after writing
!   uio_dim2st:            Compose dimension string
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!   uio routines have to be initialized.
!
! PROCEDURE:
!   The behaviour of the routines depends on the contents of the command string.
!   It may contain one or more of the following substrings:
!   'open', 'append':
!     The requested file is opened (in UIO format). The file parameter has to
!     be specified. The form and conv parameter may are optional.
!     The file_id and - if specified - the file description and history are
!     written into the file.
!     In the case of the 'open' command a new file is created (old data is overwritten).
!     In the case of the 'append' command data is appended to a (possibly) existing file.
!   'model':
!     Dataset is opened. General model data is written into the file:
!     dataset label, modelboxtype, dimension, time, time step number.
!   'box':
!     The box data are written into the file with the unit ncout.
!     In this case all box data have to be specified.
!   'endmo':
!     Dataset is closed: enddataset label.
!   'close':
!     The file is closed again.
!   One or more models or boxes per model may be written into the file by
!   repeated calls of the routine.
!
! EXAMPLE:
!   --- Write exactly one model into the file ---
!   call rhd_datc_Write('open,model,box,endmo,close', 6, ncout, &
!                       file='model.dat', form='unformatted', conv='ieee_4', &
!                       ndes=ndes, description=description, &
!                       nhis=nhis, history=history, version='testversion', &
!                       box=box, datc0=datc0)
!   --- ---
!   --- Open file, write model(s), close file with separate commands ---
!   call rhd_datc_Write('open', 6, ncout, &
!                       file='model.out', form='unformatted', conv='ieee_4', &
!                       ndes=ndes, description=description, &
!                       nhis=nhis, history=history, version='testversion')
!   call rhd_datc_Write('model,box', 6, ncout, &
!                       box=box, datc0=datc0)
!   --- ... ---
!   call rhd_datc_Write('endmo', 6, ncout)
!   call rhd_datc_Write('close', 6, ncout)
!
! MODIFICATION HISTORY:
!   2013-08-01 (B.F. Lyon) Adapted from rhd_dat_Write and rhd_datx_Write
!   2017-09-13 (B.F. Uppsala) Take xc3 and xb3 from global prop structure and not from input box
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use uio_filedef_module
use const_module
use rhd_box_module
use rhd_prop_module
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(in)   :: nc_p
integer,                   intent(inout):: ncout
character(len=*),optional, intent(in)   :: file, form, conv
integer,         optional, intent(in)   :: ndes
character(len=*), dimension(:), &
                 optional, intent(in)   :: description
integer,         optional, intent(in)   :: nhis
character(len=*), dimension(:), &
                 optional, intent(in)   :: history
character(len=*),optional, intent(in)   :: version
type(box_type),  optional, intent(in)   :: box
type(datc_type), optional, intent(in)   :: datc0
!character(len=*),optional, intent(out)  :: outstr
!integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
!integer                                 :: ierr0
!character(len=80)                       :: outstr0
integer                                 :: m1,n1, m2,n2, na,na1, i, i1, i2, ia, iquc
integer, dimension(3)                   :: cindex
integer                                 :: ntt
character(len=let)                      :: termt(2,nttmx)
character(len=80)                       :: shistr, i_str
type(datc_type)                         :: datc
real, dimension(:  ), allocatable       :: xb1_vol
real, dimension(:,:), allocatable       :: OneOverRelVol
real                                    :: volume
!---------------------------------------------------------------------------------------------------
!
if ((index(command, 'open') /= 0) .or. (index(command, 'append') /= 0)) then
  if (index(command, 'append') /= 0) then
    ! --- Open file for writing in uio-form: append data ---
    call uio_openwr(ncout, file=file, form=form, conv=conv, position='append')
  else
    ! --- Open file for writing in uio-form ---
    call uio_openwr(ncout, file=file, form=form, conv=conv)
  endif
  !
  write(nc_p,'(A,I3)') 'Model file ''' // trim(file) // ''' opened on channel ', ncout
  !
  ! === Write heading information ==================================================================
  !
  ! --- Write file ID ---
  call uio_wr(ncout, 'rhd-data', 'file_id', name='File identification')
  !
  if (present(description) .and. present(ndes)) then
    ! --- Write file description ---
    call uio_wr(ncout, description(1:ndes), 'description', name='File description')
  endif
  !
  if (present(history) .and. present(nhis)) then
    ! --- Write file history ---
    call uio_wr(ncout, history(1:nhis), 'history', name='File history')
  endif
  !
  if (present(version)) then
    ! --- Write program version ---
    call uio_wr(ncout, version, 'version', name='Program version')
  endif
endif ! ((index(command, 'open') /= 0) .or. (index(command, 'append') /= 0))
!
!
if (index(command, 'model') /= 0) then
  ! === Write general model data ===================================================================
  !
  ! --- Write model label ---
  call uio_wrlabl(ncout, dataset_ident, name='RHD model', date='now')
  !
  ! --- Write ID for box hierarchy ---
  call uio_wr(ncout, 'single_box', dataset_id_ident, name='Type of box hierarchy')
  !
  ! --- Write time ---
  call uio_wr(ncout, box%time   , 'modeltime'   , name='time', unit=time_unit)
  call uio_wr(ncout, box%time_db, 'modeltime_db', name='time', unit=time_unit)
  !
  ! --- Write time step number ---
  call uio_wr(ncout, box%itime, 'modelitime', name='time step number', unit='1')
  !
endif ! (index(command, 'model') /= 0)
!
!
if (index(command, 'box') /= 0) then
  ! === Write box ==================================================================================
  !
  !write(6,*) 'rhd_datc_Write: ', minval(datc0%d1(datc0%m2:datc0%n2+1,datc0%i_xb)), &
  !                               maxval(datc0%d1(datc0%m2:datc0%n2+1,datc0%i_xb)), &
  !                               minval(datc0%d1(datc0%m2:datc0%n2  ,datc0%i_xc)), &
  !                               maxval(datc0%d1(datc0%m2:datc0%n2  ,datc0%i_xc))
  m1 =datc0%m1
  n1 =datc0%n1
  m2 =datc0%m2
  n2 =datc0%n2
  na =datc0%na
  na1=datc0%na1
  !
  ! --- Copy input dataset (the copy will be modified later) ---
  datc=datc0
  !
  allocate(xb1_vol(            m2:n2+1), &
           OneOverRelVol(m1:n1,m2:n2  )   )
  !
  ! --- Compute "real" xb from volumes ---
  xb1_vol(m2)=datc%d1(m2,datc0%i_xb)
  volume=0.0
  i1=m1
  do i2=m2,n2
    volume       =volume + datc%d(i1,i2,datc0%i_relvol)
    xb1_vol(i2+1)=prop%length * sqrt(volume/(pi*(prop%xb3(i1+1)-prop%xb3(i1))/prop%length))
  end do ! i
  !
  ! --- Compute inverse of relative volume (for nonzero entries) ---
  do i2=m2,n2
    do i1=m1,n1
      if (datc%d(i1,i2,datc0%i_relvol) > 0.0) then
        OneOverRelVol(i1,i2)=1.0/datc%d(i1,i2,datc0%i_relvol)
      else
        OneOverRelVol(i1,i2)=0.0
      endif
    end do ! i1
  end do ! i2
  !
  ! --- Multiply all quantities with inverse of relative volume ---
  do ia=1,na
    do i2=m2,n2
      do i1=m1,n1
        datc%d(i1,i2,ia)=datc%d(i1,i2,ia)*OneOverRelVol(i1,i2)
      end do ! i1
    end do ! i2
  end do ! ia
  !
  ! === Write box header ===
  !
  ! --- Write start box label ---
  call uio_wrlabl(ncout, box_ident, date='now')
  !
  ! --- Write box ID ---
  call uio_wr(ncout, 'z', box_id_ident, name='Block identification')
  !
  ! --- Write block dimension ---
  call uio_wr(ncout, reshape((/ m2,n2, 1,1, m1,n1 /), (/2,3/)), 'dimension', &
              format='I7', nperlin=6)
  !
  ! === Write scalars ===
  !
  ! --- Write time ---
  call uio_wr(ncout, box%time   , 'time'   , name='time', unit=time_unit)
  call uio_wr(ncout, box%time_db, 'time_db', name='time', unit=time_unit)
  !
  ! --- Write time step number ---
  call uio_wr(ncout, box%itime, 'itime', name='time step number', unit='1')
  !
  ! === Write arrays ===
  !
  ! --- Write grid: 1D vectors: spread -> 3D -------------------------------------------------------
  !
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape(datc%d1(m2:n2,datc0%i_xc), (/n2-m2+1,1,1/)), 'xc1', &
              mode=8, ishift=(/m2,1,1/), &
              name='x1 coordinates of cell centers (radius)', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape(0.5*(xb1_vol(m2:n2)+xb1_vol(m2+1:n2+1)), (/n2-m2+1,1,1/)), &
                                                                      'xc1_volume', &
              mode=8, ishift=(/m2,1,1/), &
              name='Radius coordinates of cell centers (volume preserving)', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape((/0.0/), (/1,1,1/)),                     'xc2', &
              mode=8, ishift=(/1,1,1/), &
              name='phi coordinates of cell centers', unit='1', &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape(prop%xc3, (/1,1,prop%n3-prop%m3+1/)),       'xc3', &
              mode=8, ishift=(/1,1,prop%m3/), &
              name='x3 coordinates of cell centers (height)', unit=x3_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, &
              reshape(datc%d1(m2:n2+1,datc0%i_xb), (/n2-m2+2,1,1/)),  'xb1', &
              mode=8, ishift=(/m2,1,1/), &
              name='x1 coordinates of cell boundaries (radius)', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape(xb1_vol, (/n2-m2+2,1,1/)),               'xb1_volume', &
              mode=8, ishift=(/m2,1,1/), &
              name='Radius coordinates of cell boundaries (volume preserving)', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape((/-pi, pi/), (/2,1,1/)),                 'xb2', &
              mode=8, ishift=(/1,1,1/), &
              name='phi coordinates of cell boundaries', unit='1', &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape(prop%xb3, (/1,1,prop%n3-prop%m3+2/)),       'xb3', &
              mode=8, ishift=(/1,1,prop%m3/), &
              name='x3 coordinates of cell boundaries (height)', unit=x3_unit, &
              ntt=ntt, termt=termt)
  !
  ! --- Write volume data --------------------------------------------------------------------------
  !
  cindex=(/n2-m2+1,1,n1-m1+1/)
  !
  ! --- Write elative volume (to get a proper weight for each radius bin) ---
  call uio_wr(ncout, reshape(transpose(datc0%d(:,:,datc0%i_RelVol)),cindex), 'relvol_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Relative Volume', unit='1')
  !
  ! --- Write basic thermodynamic quantities ---
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rho   )), cindex), 'rho_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Density', unit=rho_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_ei    )), cindex), 'ei_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Internal energy', unit=e_unit)
  !
  ! --- Write additional thermodynamic quantities ---
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhoei )), cindex), 'rhoei_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Internal energy', unit=rhoe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_T     )), cindex), 't_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Temperature', unit=T_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_P     )), cindex), 'p_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Pressure', unit=P_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_Gamma1)), cindex), 'gamma1_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='1st Adiabatic coefficient', unit='1')
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_Gamma3)), cindex), 'gamma3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='3rd Adiabatic coefficient', unit='1')
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_delta )), cindex), 'delta_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Expansion coefficient', unit='1')
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_s     )), cindex), 's_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Entropy', unit=s_unit)
  call uio_wr(ncout, sqrt(reshape(transpose(datc%d(:,:,datc0%i_ss)), cindex)), 's_xmean2', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Entropy', unit=s_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhos  )), cindex), 'rhos_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Entropy', unit=rhos_unit)
  !
  ! --- Write velocities ---
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_v1    )), cindex), 'v1_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Velocity radial', unit=v_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_v2    )), cindex), 'v2_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Velocity azimuthal', unit=v_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_v3    )), cindex), 'v3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Velocity vertical', unit=v_unit)
  call uio_wr(ncout, sqrt(reshape(transpose(datc%d(:,:,datc0%i_v1v1  )), cindex)), 'v1_xmean2', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Velocity radial', unit=v_unit)
  call uio_wr(ncout, sqrt(reshape(transpose(datc%d(:,:,datc0%i_v2v2  )), cindex)), 'v2_xmean2', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Velocity azimuthal', unit=v_unit)
  call uio_wr(ncout, sqrt(reshape(transpose(datc%d(:,:,datc0%i_v3v3  )), cindex)), 'v3_xmean2', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Velocity vertical', unit=v_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov1  )), cindex), 'rhov1_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Mass flux radial', unit=rhov_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov2  )), cindex), 'rhov2_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Mass flux azimuthal', unit=rhov_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov3  )), cindex), 'rhov3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Mass flux vertical', unit=rhov_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov1v1)), cindex), 'rhov1v1_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Momentum flux 11 radial', unit=rhovv_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov2v2)), cindex), 'rhov2v2_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Momentum flux 22 azimuthal', unit=rhovv_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov3v3)), cindex), 'rhov3v3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Momentum flux 33 vertical', unit=rhovv_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov1v2)), cindex), 'rhov1v2_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Momentum flux 12', unit=rhovv_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov1v3)), cindex), 'rhov1v3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Momentum flux 13', unit=rhovv_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_rhov2v3)), cindex), 'rhov2v3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Momentum flux 23', unit=rhovv_unit)
  !
  ! --- Write energy fluxes ---
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fep1  )), cindex), 'fepc1_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Pressure energy flux radial', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fep3  )), cindex), 'fepc3_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Pressure energy flux vertical', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_feip1 )), cindex), 'feipc1_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Enthalpy flux radial', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_feip3 )), cindex), 'feipc3_xmean', &
              mode=8, ishift=(/m2,1,m1/), &
              name='Enthalpy flux vertical', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fek1  )), cindex), 'fekc1_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Kinetic energy flux radial', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fek3  )), cindex), 'fekc3_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Kinetic energy flux vertical', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fer1  )), cindex), 'ferc1_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Radiative energy flux radial', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fer2  )), cindex), 'ferc2_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Radiative energy flux azimuthal', unit=fe_unit)
  call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fer3  )), cindex), 'ferc3_xmean' , &
              mode=8, ishift=(/m2,1,m1/), &
              name='Radiative energy flux vertical', unit=fe_unit)
  !
!#
# if (rhd_box_quc01 != 0)
!#
  ! --- Write additional densities and their fluxes ---
  if (datc0%i_quc > 0) then
    do iquc=1,prop%nquc
      call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_quc-1+iquc  )), cindex), &
                                                    trim(prop%quc_ident(iquc)) // '_xmean', &
                  mode=8, ishift=(/m2,1,m1/), &
                  name=prop%quc_name(iquc), unit=prop%quc_unit(iquc))
      call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fquc1-1+iquc)), cindex), &
                                             'f' // trim(prop%quc_ident(iquc)) // 'c1_xmean', &
                  mode=8, ishift=(/m2,1,m1/), &
                  name='Flux (radial) of ' // prop%quc_name(iquc), &
                  unit=trim(prop%quc_unit(iquc)) // '*' // v_unit)
      call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fquc2-1+iquc)), cindex), &
                                             'f' // trim(prop%quc_ident(iquc)) // 'c2_xmean', &
                  mode=8, ishift=(/m2,1,m1/), &
                  name='Flux (azimuthal) of ' // prop%quc_name(iquc), &
                  unit=trim(prop%quc_unit(iquc)) // '*' // v_unit)
      call uio_wr(ncout, reshape(transpose(datc%d(:,:,datc0%i_fquc3-1+iquc)), cindex), &
                                             'f' // trim(prop%quc_ident(iquc)) // 'c3_xmean', &
                  mode=8, ishift=(/m2,1,m1/), &
                  name='Flux (vertical) of ' // prop%quc_name(iquc), &
                  unit=trim(prop%quc_unit(iquc)) // '*' // v_unit)
    end do ! iquc
  endif ! (datc0%i_quc > 0)
!#
# endif
!#      /* rhd_box_quc01 */
  !
  ! --- Write end block label ---
  call uio_wrlabl(ncout, endbox_ident)
  !
  ! --- Delete temporary dataset ---
  deallocate(xb1_vol, OneOverRelVol)
  call rhd_datc_Delete(datc)
  !
endif ! (index(command, 'box') /= 0)
!
!
! === Finalize =====================================================================================
if (index(command,'endmo') /= 0) then
  ! --- Write end dataset label ---
  call uio_wrlabl(ncout, enddataset_ident, date='now')
endif ! (index(command,'endmo') /= 0)
!
if (index(command,'close') /= 0) then
  ! --- Close file ---
  call uio_closwr(ncout)
endif ! (index(command,'close') /= 0)
!
!if (present(outstr)) outstr=outstr0
!if (present(ierr)) ierr=ierr0
!
end subroutine rhd_datc_Write


!----------*************----------------------------------------------------------------------------
subroutine rhd_dat_Write(command, nc_p, ncout, file, form, conv, &
                         ndes, description, nhis, history, version, &
                         box_id, box, dat)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_Write ('rhd_data_Write')
!
! PURPOSE:
!   Write additional RHD data into file in UIO form.
!   
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_dat_Write(command, nc_p, ncout, file=file, form=form, conv=conv, &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version=version, &
!                      box_id=box_id, box=box, dat=dat)
!
! INPUT:
!   command:      ('command') character, command string:
!                 'open'/'append', 'read', 'model, 'box', 'close'
!   nc_p:         ('number_channel_print') integer, unit number for text output
!   --- optional: ---
!   file:         ('file') character, name of data file
!   form:         ('format') character, name of data file
!   conv:         ('conversion') character, conversion type of data file
!   ndes:         ('number_description') integer, number of lines of description
!   description:  ('description') character, dimension(:), description of model
!   nhis:         ('number_history') integer, number of lines of history
!   history:      ('history') character, dimension(:), history of model
!   version:      ('version') character, string with RHD version
!   box_id:       ('box_identifier') character, identifier for box, default='z'
!   box:          ('box') type(box_type), computational box
!   dat:          ('data') type(dat_type), structure for additional data
!
! INPUT/OUTPUT:
!   ncout:        ('number_channel_output') integer, unit number for data output
!
! OUTPUT:
!   None
!
! ROUTINES:
!   rhd_datx_Write:        Write additional RHD data into file in UIO form
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_closwr:            Close file after writing
!   uio_dim2st:            Compose dimension string
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!
! MODULES:
!   const_module:          Global physical and mathematical constants and units
!   rhd_box_module:        Box handling routines for RHD
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!   uio routines have to be initialized.
!
! PROCEDURE:
!   The behaviour of the routines depends on the contents of the command string.
!   It may contain one or more of the following substrings:
!   'open', 'append':
!     The requested file is opened (in UIO format). The file parameter has to
!     be specified. The form and conv parameter may are optional.
!     The file_id and - if specified - the file description and history are
!     written into the file.
!     In the case of the 'open' command a new file is created (old data is overwritten).
!     In the case of the 'append' command data is appended to a (possibly) existing file.
!   'model':
!     Dataset is opened. General model data is written into the file:
!     dataset label, modelboxtype, dimension, time, time step number.
!   'box':
!     The box data are written into the file with the unit ncout.
!     In this case all box data have to be specified.
!   'endmo':
!     Dataset is closed: enddataset label.
!   'close':
!     The file is closed again.
!   One or more models or boxes per model may be written into the file by
!   repeated calls of the routine.
!
! EXAMPLE:
!   --- Write exactly one model into the file ---
!   call rhd_dat_Write('open,model,box,endmo,close', 6, ncout, &
!                      file='model.dat', form='unformatted', conv='ieee_4', &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version='testversion', &
!                      box=box, dat=dat)
!   --- ---
!   --- Open file, write model(s), close file with separate commands ---
!   call rhd_dat_Write('open', 6, ncout, &
!                      file='model.out', form='unformatted', conv='ieee_4', &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version='testversion')
!   call rhd_dat_Write('model,box', 6, ncout, &
!                      box=box, dat=dat)
!   --- ... ---
!   call rhd_dat_Write('endmo', 6, ncout)
!   call rhd_dat_Write('close', 6, ncout)
!
! MODIFICATION HISTORY:
!   1999-08-20 (B.F. Copenhagen) Written
!   1999-09-28 (B.F.) ferX_r,l
!   2000-01-22 (B.F.) Parameter box_id added
!   2000-04-08 (B.F.) Command 'append' added
!   2004-09-20 (B.F.) len(i_str) -> len(trim(i_str))
!   2006-01-23 (B.F.) Check par%grav_mode
!   2007-10-22 (B.F. Lyon) Output of box%time_db
!   2010-04-20 (B.F.) Rename rhd_wrdat -> rhd_dat_Write
!   2013-08-20 (B.F.) Change some capitalization in the names of the output quantities
!   2014-12-10 (B.F.) Base size of intensity arrays on wg%rad(1)
!   2017-08-12 (B.F. Uppsala) Take axes from global prop structure and not from input box
!   2017-09-12 (B.F.) Rename box IDs, e.g., 'r' -> 'rold', 'rnew' -> 'r'
!   2018-05-07 (B.F.) Remove branch with calls to write old averaging data
!---------------------------------------------------------------------------------------------------
use const_module
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use uio_filedef_module
use rhd_box_module
use rhd_gl_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(in)   :: nc_p
integer,                   intent(inout):: ncout
character(len=*),optional, intent(in)   :: file, form, conv
integer,         optional, intent(in)   :: ndes
character(len=*), dimension(:), &
                 optional, intent(in)   :: description
integer,         optional, intent(in)   :: nhis
character(len=*), dimension(:), &
                 optional, intent(in)   :: history
character(len=*),optional, intent(in)   :: version, box_id
type(box_type),  optional, intent(in)   :: box
type(dat_type),  optional, intent(in)   :: dat
!
! --- Local variables ---
integer                                 :: m1,n1, m2,n2, m3,n3, mr1,nr1, mr2,nr2, mr3,nr3, i
integer                                 :: ntt
character(len=let)                      :: termt(2,nttmx)
character(len=80)                       :: shistr, box_id0, i_str
!---------------------------------------------------------------------------------------------------
!
if (present(dat)) then
  ! --- Make abbreviations ---
  m1=dat%m1
  m2=dat%m2
  m3=dat%m3
  n1=dat%n1
  n2=dat%n2
  n3=dat%n3
endif
!
if ((index(command, 'open') /= 0) .or. (index(command, 'append') /= 0)) then
  if (index(command, 'append') /= 0) then
    ! --- Open file for writing in UIO form: append data ---
    call uio_openwr(ncout, file=file, form=form, conv=conv, position='append')
  else
    ! --- Open file for writing in UIO form ---
    call uio_openwr(ncout, file=file, form=form, conv=conv)
  endif
  !
  write(nc_p,'(A,I3)') 'Model file ''' // trim(file) // ''' opened on channel ', ncout
  !
  ! === Write heading information ==================================================================
  !
  ! --- Write file ID ---
  call uio_wr(ncout, 'rhd-data', 'file_id', name='File identification')
  !
  if (present(description) .and. present(ndes)) then
    ! --- Write file description ---
    call uio_wr(ncout, description(1:ndes), 'description', name='File description')
  endif
  !
  if (present(history) .and. present(nhis)) then
    ! --- Write file history ---
    call uio_wr(ncout, history(1:nhis), 'history', name='File history')
  endif
  !
  if (present(version)) then
    ! --- Write program version ---
    call uio_wr(ncout, version, 'version', name='Program version')
  endif
endif
!
!
if (index(command, 'model') /= 0) then
  ! === Write general model data ===================================================================
  !
  ! --- Write model label ---
  call uio_wrlabl(ncout, dataset_ident, name='RHD model', date='now')
  !
  ! --- Write ID for box hierarchy ---
  call uio_wr(ncout, 'single_box', dataset_id_ident, name='Type of box hierarchy')
  !
  ! --- Write time ---
  call uio_wr(ncout, box%time   , 'modeltime'   , name='time', unit=time_unit)
  call uio_wr(ncout, box%time_db, 'modeltime_db', name='time', unit=time_unit)
  !
  ! --- Write time step number ---
  call uio_wr(ncout, box%itime, 'modelitime', name='time step number', unit='1')
  !
endif
!
!
! === Write additional data (boundary conditions ...) ==============================================
! ...
!
if (index(command, 'box') /= 0) then
  ! === Write bulk model data (from box) ===========================================================
  !
  ! --- Write start box label ---
  call uio_wrlabl(ncout, box_ident, date='now')
  !
  ! --- Write box ID ---
  if (present(box_id)) then
    box_id0=box_id
  else
    box_id0='z'
  endif
  call uio_wr(ncout, box_id0, box_id_ident, name='Block identification')
  !
  ! --- Write block dimension ---
  call uio_wr(ncout, reshape((/ m1,n1, m2,n2, m3,n3 /), (/2,3/)), 'dimension', &
              format='I7', nperlin=6)
  !
  ! --- Write scalars ------------------------------------------------------------------------------
  !
  ! --- Write time ---
  call uio_wr(ncout, box%time   , 'time'   , name='time', unit=time_unit)
  call uio_wr(ncout, box%time_db, 'time_db', name='time', unit=time_unit)
  !
  ! --- Write time step number ---
  call uio_wr(ncout, box%itime, 'itime', name='time step number', unit='1')
  !
  ! --- Write arrays -------------------------------------------------------------------------------
  !
  ! --- Write grid: 1D vectors: spread -> 3D ---
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape(prop%xc1, (/n1-m1+1,1,1/)), 'xc1', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x1 coordinates of cell centers', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape(prop%xc2, (/1,n2-m2+1,1/)), 'xc2', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x2 coordinates of cell centers', unit=x2_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape(prop%xc3, (/1,1,n3-m3+1/)), 'xc3', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x3 coordinates of cell centers', unit=x3_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape(prop%xb1, (/n1-m1+2,1,1/)), 'xb1', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x1 coordinates of cell boundaries', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape(prop%xb2, (/1,n2-m2+2,1/)), 'xb2', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x2 coordinates of cell boundaries', unit=x2_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape(prop%xb3, (/1,1,n3-m3+2/)), 'xb3', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x3 coordinates of cell boundaries', unit=x3_unit, &
              ntt=ntt, termt=termt)
  !
  ! --- Write additional data ---
  !
  ! --- Write luminosity ---
  call uio_wr(ncout, dat%luminosity, 'luminositypervolume', name='luminosity per core volume', &
              unit=luminpervol_unit)
  !
  ! --- Get index ranges for radiative surface flux arrays ---
  mr1=wg%rad(1)%m(1)
  nr1=wg%rad(1)%n(1)
  mr2=wg%rad(1)%m(2)
  nr2=wg%rad(1)%n(2)
  mr3=wg%rad(1)%m(3)
  nr3=wg%rad(1)%n(3)
  !
  ! --- Write surface fluxes ---
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  if (associated(dat%fer1_l)) then
    call uio_wr(ncout, reshape(dat%fer1_l, (/1,nr2-mr2+1,nr3-mr3+1/)), 'ferb1_l', &
                mode=8, ishift=(/mr1,mr2,mr3/), &
                name='Radiative flux x1 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%fer1_r)) then
    call uio_wr(ncout, reshape(dat%fer1_r, (/1,nr2-mr2+1,nr3-mr3+1/)), 'ferb1_r', &
                mode=8, ishift=(/nr1+1,mr2,mr3/), &
                name='Radiative flux x1 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  if (associated(dat%fer2_l)) then
    call uio_wr(ncout, reshape(dat%fer2_l, (/nr1-mr1+1,1,nr3-mr3+1/)), 'ferb2_l', &
                mode=8, ishift=(/mr1,mr2,mr3/), &
                name='Radiative flux x2 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%fer2_r)) then
    call uio_wr(ncout, reshape(dat%fer2_r, (/nr1-mr1+1,1,nr3-mr3+1/)), 'ferb2_r', &
                mode=8, ishift=(/mr1,nr2+1,mr3/), &
                name='Radiative flux x2 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  0,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  if (associated(dat%fer3_l)) then
    call uio_wr(ncout, reshape(dat%fer3_l, (/nr1-mr1+1,nr2-mr2+1,1/)), 'ferb3_l', &
                mode=8, ishift=(/mr1,mr2,mr3/), &
                name='Radiative flux x3 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%fer3_r)) then
    call uio_wr(ncout, reshape(dat%fer3_r, (/nr1-mr1+1,nr2-mr2+1,1/)), 'ferb3_r', &
                mode=8, ishift=(/mr1,mr2,nr3+1/), &
                name='Radiative flux x3 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  ! --- Get index ranges for radiative surface intensity arrays ---
  mr1=wg%hyd%m(1)
  nr1=wg%hyd%n(1)
  mr2=wg%hyd%m(2)
  nr2=wg%hyd%n(2)
  mr3=wg%hyd%m(3)
  nr3=wg%hyd%n(3)
  !
  ! --- Write surface intensities ---
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  if (associated(dat%Int1_l)) then
    call uio_wr(ncout, reshape(dat%Int1_l, (/1,nr2-mr2+1,nr3-mr3+1/)), 'intb1_l', &
                mode=8, ishift=(/mr1,mr2,mr3/), &
                name='Intensity x1 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%Int1_r)) then
    call uio_wr(ncout, reshape(dat%Int1_r, (/1,nr2-mr2+1,nr3-mr3+1/)), 'intb1_r', &
                mode=8, ishift=(/nr1+1,mr2,mr3/), &
                name='Intensity x1 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  if (associated(dat%Int2_l)) then
    call uio_wr(ncout, reshape(dat%Int2_l, (/nr1-mr1+1,1,nr3-mr3+1/)), 'intb2_l', &
                mode=8, ishift=(/mr1,mr2,mr3/), &
                name='Intensity x2 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%Int2_r)) then
    call uio_wr(ncout, reshape(dat%Int2_r, (/nr1-mr1+1,1,nr3-mr3+1/)), 'intb2_r', &
                mode=8, ishift=(/mr1,nr2+1,mr3/), &
                name='Intensity x2 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  0,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  if (associated(dat%Int3_l)) then
    call uio_wr(ncout, reshape(dat%Int3_l, (/nr1-mr1+1,nr2-mr2+1,1/)), 'intb3_l', &
                mode=8, ishift=(/mr1,mr2,mr3/), &
                name='Intensity x3 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%Int3_r)) then
    call uio_wr(ncout, reshape(dat%Int3_r, (/nr1-mr1+1,nr2-mr2+1,1/)), 'intb3_r', &
                mode=8, ishift=(/mr1,mr2,nr3+1/), &
                name='Intensity x3 direction', unit=fe_unit, &
                ntt=ntt, termt=termt)
  endif
  !
  if (associated(dat%Intxx3_r)) then
    do i=1,size(dat%Intxx3_r,3)
      write(i_str,'(I2)') i
      i_str='0' // adjustl(i_str)
      i_str=i_str(len(trim(i_str))-1:len(trim(i_str)))
      call uio_wr(ncout, reshape(dat%Intxx3_r(:,:,i:i), (/nr1-mr1+1,nr2-mr2+1,1/)), &
                    'int' // trim(i_str) // 'b3_r', &
                  mode=8, ishift=(/mr1,mr2,nr3+1/), &
                  name='Bin intensity x3 direction', unit=fe_unit, &
                  ntt=ntt, termt=termt)
    end do ! i
  endif
  !
  ! --- Write end block label ---
  call uio_wrlabl(ncout, endbox_ident)
  !
  ! --- Write sub boxes ----------------------------------------------------------------------------
  !
  if (dat%x23%na > 0) then
    call rhd_datx_Write(ncout, 'z1', box, dat%x23)
  endif
  if (dat%x13%na > 0) then
    call rhd_datx_Write(ncout, 'z2', box, dat%x13)
  endif
  if (dat%x12%na > 0) then
    call rhd_datx_Write(ncout, 'z3', box, dat%x12)
  endif
  !
  if (par%grav_mode == 'central') then
    if (dat%r%na > 0) then
      call rhd_datx_Write(ncout, 'r', box, dat%r)
    endif
  endif
  !
endif ! (index(command, 'box') /= 0)
!
!
! === Finalize =====================================================================================
if (index(command,'endmo') /= 0) then
  ! --- Write end dataset label ---
  call uio_wrlabl(ncout, enddataset_ident, date='now')
endif
!
if (index(command,'close') /= 0) then
  ! --- Close file ---
  call uio_closwr(ncout)
endif
!
!if (present(outstr)) outstr=outstr0
!if (present(ierr)) ierr=ierr0
!
end subroutine rhd_dat_Write


!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ======================================================== */


!----------*********************--------------------------------------------------------------------
subroutine rhd_dat_MPICreateType(dat, m1,n1, m2,n2, m3,n3, &
                                 rhdMPI_datDataType, IncludeIntensity_flag, IncludeFlux_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_MPICreateType ('rhd_dat_MPI_Create_Type')
!
! PURPOSE:
!   Create MPI data type for dat structure.
!
! CATEGORY:
!   MPI
!
! CALLING SEQUENCE:
!   call rhd_dat_MPICreateType(dat, m1,n1, m2,n2, m3,n3, &
!                              rhdMPI_datDataType, &
!                              IncludeIntensity_flag=IncludeIntensity_flag, &
!                              IncludeFlux_flag=IncludeFlux_flag)
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   IncludeIntensity_flag: ('Include_Intensity_flag') optional logical to indicate that surface
!                          intensity arrays should be included in MPI data structure, default=.true.
!   IncludeFlux_flag: ('Include_Flux_flag') optional logical to indicate that surface flux
!                          arrays should be included in MPI data structure, default=.true.
!
! OUTPUT:
!   rhdMPI_datDataType: ('rhd_MPI_dat_Data_Type') integer with MPI data type handle
!                       for dat structure
!
! ROUTINES:
!   flush:                 Flush output buffer
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!   MPI_Get_Address:       Get the address of a location in memory
!   MPI_Type_Commit:       Commit an MPI data type
!   MPI_Type_Create_Struct: Create an MPI structured data type
!   MPI_Type_Create_Subarray: Create an MPI data type describing a sub array
!   opta_Info:             Give stored information about current opacity table
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   opta_module:           Routines for reading, modifying, and interpolating opacities
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The created MPI data type has to be freed later on with a call to MPI_Type_Free.
!
! PROCEDURE:
!   Create and fill arrays for displacement, length, and type of variables in dat structure.
!   Put the result into MPI structure and give handle to resulting MPI data type as output.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-12 (B.F. Uppsala) Written
!   2019-01-29 (B.F.) Add optional input flags
!   2019-01-30 (B.F.) Account for possibly different sizes of flux and intensity arrays
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use opta_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(dat_type),            intent(in)   :: dat
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3
logical, optional,         intent(in)   :: IncludeIntensity_flag, IncludeFlux_flag
integer,                   intent(out)  :: rhdMPI_datDataType
!
! --- Local variables ---
integer, parameter                      :: nBlocks=100
integer, dimension(nBlocks)             :: a_BlockLengths, a_Types
integer(kind=MPI_ADDRESS_KIND)          :: Displacement_m1
integer(kind=MPI_ADDRESS_KIND), &
         dimension(nBlocks)             :: a_Displacements
integer                                 :: ierrMPI, i, MyProc, nbin, &
                                           Type1IntMPI, Type2IntMPI, Type3IntMPI, &
                                           Type1FerMPI, Type2FerMPI, Type3FerMPI
logical                                 :: IncludeIntensity_flag0, IncludeFlux_flag0
!---------------------------------------------------------------------------------------------------
!
! --- Set default values for keywords not present in input list ---
if (.not. present(IncludeIntensity_flag)) then
  IncludeIntensity_flag0=.true.
else
  IncludeIntensity_flag0=IncludeIntensity_flag
endif
if (.not. present(IncludeFlux_flag)) then
  IncludeFlux_flag0=.true.
else
  IncludeFlux_flag0=IncludeFlux_flag
endif
!
! --- Set entry counter to zero ---
i=0
!
! --- Get the process ID, for output only ---
call MPI_Comm_Rank(MPI_COMM_WORLD, MyProc, ierrMPI)
!
! === Describe structure the MPI way ===
!
! --- Handle arrays in x23 plane ---
!
if (IncludeIntensity_flag0) then
  !write(6,'(A,I3,A,4I5,3X,3(2X,2I5))') &
  !         '#', MyProc, ' Int1: MPI_Type_Create_Subarray', &
  !         m2,n2, m3,n3,        (/ dat%n2-dat%m2+1, dat%n3-dat%m3+1 /), &
  !                              (/     n2-m2    +1,     n3-    m3+1 /), &
  !                              (/     m2-dat%m2  ,     m3-dat%m3   /)
  !call flush(6)
  call MPI_Type_Create_Subarray(2, &
                                (/ dat%n2-dat%m2+1, dat%n3-dat%m3+1 /), &
                                (/     n2-    m2+1,     n3-    m3+1 /), &
                                (/     m2-dat%m2  ,     m3-dat%m3   /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                Type1IntMPI, &
                                ierrMPI)
  !
  if ((m1 == dat%m1) .and. associated(dat%Int1_l)) then
    i=i+1
    call MPI_Get_Address(dat%Int1_l, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type1IntMPI
  endif
  !
  if ((n1 == dat%n1) .and. associated(dat%Int1_r)) then
    i=i+1
    call MPI_Get_Address(dat%Int1_r, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type1IntMPI
  endif
endif ! (IncludeIntensity_flag0)
!
if (IncludeFlux_flag0) then
  !write(6,'(A,I3,A,4I5,3X,3(2X,2I5))') &
  !         '#', MyProc, ' Fer1: MPI_Type_Create_Subarray', &
  !         m2,n2, m3,n3,        (/ dat%nr2-dat%mr2+1, dat%nr3-dat%mr3+1 /), &
  !                              (/     n2 -m2     +1,     n3 -    m3+1  /), &
  !                              (/     m2 -dat%mr2  ,     m3 -dat%mr3   /)
  !call flush(6)
  call MPI_Type_Create_Subarray(2, &
                                (/ dat%nr2-dat%mr2+1, dat%nr3-dat%mr3+1 /), &
                                (/     n2 -    m2 +1,     n3 -    m3 +1 /), &
                                (/     m2 -dat%mr2  ,     m3 -dat%mr3   /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                Type1FerMPI, &
                                ierrMPI)
  !
  if ((m1 == dat%mr1) .and. associated(dat%fer1_l)) then
    i=i+1
    call MPI_Get_Address(dat%fer1_l, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type1FerMPI
  endif
  !
  if ((n1 == dat%nr1) .and. associated(dat%fer1_r)) then
    i=i+1
    call MPI_Get_Address(dat%fer1_r, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type1FerMPI
  endif
endif ! (IncludeFlux_flag0)
!
! --- Handle arrays in x13 plane ---
!
if (IncludeIntensity_flag0) then
  !write(6,'(A,I3,A,4I5,3X,3(2X,2I5))') &
  !         '#', MyProc, ' Int2: MPI_Type_Create_Subarray', &
  !         m1,n1, m3,n3,        (/ dat%n1-dat%m1+1, dat%n3-dat%m3+1 /), &
  !                              (/     n1-    m1+1,     n3-    m3+1 /), &
  !                              (/     m1-dat%m1  ,     m3-dat%m3   /)
  !call flush(6)
  call MPI_Type_Create_Subarray(2, &
                                (/ dat%n1-dat%m1+1, dat%n3-dat%m3+1 /), &
                                (/     n1-    m1+1,     n3-    m3+1 /), &
                                (/     m1-dat%m1  ,     m3-dat%m3   /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                Type2IntMPI, &
                                ierrMPI)
  !
  if ((m2 == dat%m2) .and. associated(dat%Int2_l)) then
    i=i+1
    call MPI_Get_Address(dat%Int2_l, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type2IntMPI
  endif
  !
  if ((n2 == dat%n2) .and. associated(dat%Int2_r)) then
    i=i+1
    call MPI_Get_Address(dat%Int2_r, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type2IntMPI
  endif
endif ! (IncludeIntensity_flag0)
!
if (IncludeFlux_flag0) then
  !write(6,'(A,I3,A,4I5,3X,3(2X,2I5))') &
  !         '#', MyProc, ' Fer2: MPI_Type_Create_Subarray', &
  !         m1,n1, m3,n3,        (/ dat%nr1-dat%mr1+1, dat%nr3-dat%mr3+1 /), &
  !                              (/     n1 -     m1+1,     n3 -    m3 +1 /), &
  !                              (/     m1 -dat%mr1  ,     m3 -dat%mr3   /)
  !call flush(6)
  call MPI_Type_Create_Subarray(2, &
                                (/ dat%nr1-dat%mr1+1, dat%nr3-dat%mr3+1 /), &
                                (/     n1 -     m1+1,     n3 -    m3 +1 /), &
                                (/     m1 -dat%mr1  ,     m3 -dat%mr3   /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                Type2FerMPI, &
                                ierrMPI)
  !
  if ((m2 == dat%mr2) .and. associated(dat%fer2_l)) then
    i=i+1
    call MPI_Get_Address(dat%fer2_l, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type2FerMPI
  endif
  !
  if ((n2 == dat%nr2) .and. associated(dat%fer2_r)) then
    i=i+1
    call MPI_Get_Address(dat%fer2_r, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type2FerMPI
  endif
endif ! (IncludeFlux_flag0)
!
! --- Handle arrays in x12 plane ---
!
if (IncludeIntensity_flag0) then
  !write(6,'(A,I3,A,4I5,3X,3(2X,2I5))') &
  !         '#', MyProc, ' Int3: MPI_Type_Create_Subarray', &
  !         m1,n1, m2,n2,        (/ dat%n1-dat%m1+1, dat%n2-dat%m2+1 /), &
  !                              (/     n1-    m1+1,     n2-    m2+1 /), &
  !                              (/     m1-dat%m1  ,     m2-dat%m2   /)
  !call flush(6)
  call MPI_Type_Create_Subarray(2, &
                                (/ dat%n1-dat%m1+1, dat%n2-dat%m2+1 /), &
                                (/     n1-    m1+1,     n2-    m2+1 /), &
                                (/     m1-dat%m1  ,     m2-dat%m2   /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                Type3IntMPI, &
                                ierrMPI)
  !
  if ((m3 == dat%m3) .and. associated(dat%Int3_l)) then
    i=i+1
    call MPI_Get_Address(dat%Int3_l, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type3IntMPI
  endif
  !
  if ((n3 == dat%n3) .and. associated(dat%Int3_r)) then
    i=i+1
    call MPI_Get_Address(dat%Int3_r, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type3IntMPI
  endif
  !
  if ((n3 == dat%n3) .and. associated(dat%Intxx3_r)) then
    call opta_Info(nbin=nbin)
    if (nbin > 0) then
      i=i+1
      call MPI_Get_Address(dat%Intxx3_r, a_Displacements(i), ierrMPI)
      a_BlockLengths(i)=nbin
      a_Types(i)       =Type3IntMPI
    endif
  endif
endif ! (IncludeIntensity_flag0)
!
if (IncludeFlux_flag0) then
  !write(6,'(A,I3,A,4I5,3X,3(2X,2I5))') &
  !         '#', MyProc, ' Fer3: MPI_Type_Create_Subarray', &
  !         m1,n1, m2,n2,        (/ dat%nr1-dat%mr1+1, dat%nr2-dat%mr2+1 /), &
  !                              (/     n1 -    m1 +1,     n2 -    m2 +1 /), &
  !                              (/     m1 -dat%mr1  ,     m2 -dat%mr2   /)
  !call flush(6)
  call MPI_Type_Create_Subarray(2, &
                                (/ dat%nr1-dat%mr1+1, dat%nr2-dat%mr2+1 /), &
                                (/     n1 -    m1 +1,     n2 -    m2 +1 /), &
                                (/     m1 -dat%mr1  ,     m2 -dat%mr2   /), &
                                MPI_ORDER_FORTRAN, &
                                MPI_REAL, &
                                Type3FerMPI, &
                                ierrMPI)
  !
  if ((m3 == dat%mr3) .and. associated(dat%fer3_l)) then
    i=i+1
    call MPI_Get_Address(dat%fer3_l, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type3FerMPI
  endif
  !
  if ((n3 == dat%nr3) .and. associated(dat%fer3_r)) then
    i=i+1
    call MPI_Get_Address(dat%fer3_r, a_Displacements(i), ierrMPI)
    a_BlockLengths(i)=1
    a_Types(i)       =Type3FerMPI
  endif
endif ! (IncludeFlux_flag0)
!
! --- Shift displacements ---
call MPI_Get_Address(dat%m1, Displacement_m1, ierrMPI)
a_Displacements(1:i)=a_Displacements(1:i)-Displacement_m1
!
! --- Use arrays to create MPI structure data type ---
call MPI_Type_Create_Struct(i, a_BlockLengths(1:i), a_Displacements(1:i), a_Types(1:i), &
                            rhdMPI_datDataType, ierrMPI)
!
! --- Commit new MPI data type ---
call MPI_Type_Commit(rhdMPI_datDataType, ierrMPI)
!
! --- Free the MPI types describing the subarrays ---
!if (IncludeIntensity_flag0) then
!  call MPI_Type_Free(Type1IntMPI, ierrMPI)
!  call MPI_Type_Free(Type2IntMPI, ierrMPI)
!  call MPI_Type_Free(Type3IntMPI, ierrMPI)
!endif ! (IncludeIntensity_flag0)
!
!if (IncludeFlux_flag0) then
!  call MPI_Type_Free(Type1FerMPI, ierrMPI)
!  call MPI_Type_Free(Type2FerMPI, ierrMPI)
!  call MPI_Type_Free(Type3FerMPI, ierrMPI)
!endif ! (IncludeFlux_flag0)
!
!call flush(6)
!
end subroutine rhd_dat_MPICreateType


!----------********************---------------------------------------------------------------------
subroutine rhd_datx_MPICollectx(dim, geo, datx, datxLrg)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_MPICollectx ('rhd_data_x_MPI_Collect_x')
!
! PURPOSE:
!   Collect distributed horizontal mean data from MPI processes into single structure.
!
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_MPICollectx(dim, geo, datx, datxLrg)
!
! INPUT:
!   dim:          ('dimension') integer to specify dimension of mean vectors (1, 2, 3)
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   datx:         ('data_x') type(datx_type), structure with additional output data
!                 for each MPI process
!
! INPUT/OUTPUT:
!   datxLrg:      ('data_x_Large') type(dat_x_type), structure with collected additional output data
!                 from all MPI processes, filled only for the master process
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in Cartesian topology from rank in group
!   MPI_Cart_Rank:         Determine process rank in communicator from Cartesian coordinates
!   MPI_Cart_Sub:          Partition a Cartesian communicator group into lower-dimension sub groups
!   MPI_Comm_Free:         Free a communicator
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!   MPI_Gatherv:           Gather varying amounts of data from all processes to the root process
!   MPI_Reduce:            Perform a reduction
!   rhd_geo_Coords2mnVec:  Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, sum data over horizontal planes (MPI reduction step).
!   Secondly, gather data along columns (MPI gather step).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-09-10 (B.F. Uppsala) First outline
!   2017-09-12 (B.F.) Working version
!   2019-01-29 (B.F.) Rename rhd_datx_Collectx to rhd_datx_MPICollectx
!   2019-02-08 (B.F.) Account for array in datxLrg structure larger than in sum of datx structures
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_meankind_module
use MPI
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: dim
type(geo_type),            intent(in)   :: geo
type(datx_type),           intent(in)   :: datx
type(datx_type),           intent(inout):: datxLrg
!
! --- Local variables ---
integer                                 :: ierrMPI, di, dit, iP, &
                                           m1,n1, m2,n2, m3,n3, &
                                           mAlldatx, nAlldatx, &
                                           CommSub12, MyProcSub12, SizeSub12, &
                                           CommSub3 , MyProcSub3 , &
                                           ProcZero, ProcZeroZero
logical, dimension(3)                   :: rhdMPI_RemainDims
real(kind=mArk), &
         dimension(:,:), allocatable    :: dTrans1, dTrans2, dTransLrg
integer, dimension(3)                   :: PCoords, mVec, nVec
integer, dimension(2)                   :: PCoordsSub12
integer, dimension(1)                   :: PCoordsSub3
integer, &
       dimension(0:geo%nProcsPD(dim)-1) :: SizeAr, ShiftAr
!---------------------------------------------------------------------------------------------------
!
! === Reduce all data within planes ================================================================
!
! write(6,'(A,I3,A,I3,2X,6(1PE13.6))') '#', geo%MyProc, &
!    '       rhd_datx_MPICollectx: dim,datx%d:  ', dim, datx%d(datx%m+2,1:6)
!
! --- Check if right boundary values should be included ---
if (datx%n == geo%n(dim)) then
  di=1
else
  di=0
endif
!
allocate(dTrans1(datx%na, datx%m:datx%n+di))
!
! --- Create 1D vector of Cartesian communicators corresponding to planes --
rhdMPI_RemainDims     =.true.
rhdMPI_RemainDims(dim)=.false.
call MPI_Cart_Sub(geo%Comm, rhdMPI_RemainDims, CommSub12, ierrMPI)
call MPI_Comm_Rank(CommSub12, MyProcSub12, ierrMPI)
!
call MPI_Comm_Size(CommSub12, SizeSub12, ierrMPI)
! write(6,'(A,2I3,A,I3,2X,I3)') '#', geo%MyProc, MyProcSub12, &
!    '    rhd_datx_MPICollectx: dim, CommSizeSub12:', dim, SizeSub12
!
! --- Find projection of master process in each plane ---
call MPI_Cart_Coords(geo%Comm, geo%MaProcWoInCO, 3, PCoords, ierrMPI)
if      (dim == 1) then
  PCoordsSub12=(/ PCoords(2), PCoords(3) /)
else if (dim == 2) then
  PCoordsSub12=(/ PCoords(1), PCoords(3) /)
else
  PCoordsSub12=(/ PCoords(1), PCoords(2) /)
endif
call MPI_Cart_Rank(CommSub12, PCoordsSub12, ProcZero, ierrMPI)
!
if ((geo%nProcsPD(1)*geo%nProcsPD(2)*geo%nProcsPD(3)/geo%nProcsPD(dim)) == 1) then
  ! --- There is only one process in this plane: just copy the data ---
  !
  ! write(6,'(A,3I3,A,I3)') '#', geo%MyProc, MyProcSub12, ProcZero, &
  !    ' rhd_datx_MPICollectx: Only one process in plane, dim:', dim
  !
  dTrans1=transpose(datx%d(datx%m:datx%n+di, 1:datx%na))
  !
else
  ! --- There is more than one process in this plane: reduce the data ---
  !
  ! --- Reduce the averages within planes by summing up the individual contributions ---
  allocate(dTrans2(datx%na, datx%m:datx%n+di))
  dTrans2=transpose(datx%d(datx%m:datx%n+di, 1:datx%na))
  dTrans1=0.0_mArk
  call MPI_Reduce(dTrans2, dTrans1, (datx%n-datx%m+di+1)*datx%na, mArkMPI, &
                  MPI_SUM, ProcZero, CommSub12, ierrMPI)
  !
  ! write(6,'(A,3I3,A,I3,2X,6(1PE13.6))') '#', geo%MyProc, MyProcSub12, ProcZero, &
  !    ' rhd_datx_MPICollectx: dim,dTrans2: ', dim, dTrans2(1:6,datx%m+2)
  !
  deallocate(dTrans2)
  !
endif
!
! --- Free the communicators corresponding to planes --
call MPI_Comm_Free(CommSub12, ierrMPI)
!
! write(6,'(A,3I3,A,I3,2X,6(1PE13.6))') '#', geo%MyProc, MyProcSub12, ProcZero, &
!   ' rhd_datx_MPICollectx: dim,dTrans1: ', dim, dTrans1(1:6,datx%m+2)
!
!
! === Gather all data along columns ================================================================
!
if (geo%nProcsPD(dim) == 1) then
  ! --- There is only one process in this dimension: just copy the data ---
  !
  ! write(6,'(A,3I3,A,I3)') '#', geo%MyProc, MyProcSub12, ProcZero, &
  !    ' rhd_datx_MPICollectx: Only one process in vector, dim:', dim
  !
  if (geo%MyProc == geo%MaProcWoInCO) then
    !datxLrg%d=transpose(dTrans1)
    datxLrg%d(datx%m:datx%n+di, 1:datx%na)=transpose(dTrans1)
  endif
  !
else
  ! --- There is more than one process in this dimension: gather the data ---
  !
  ! --- Create 2D array of Cartesian communicators corresponding to columns --
  rhdMPI_RemainDims     =.false.
  rhdMPI_RemainDims(dim)=.true.
  call MPI_Cart_Sub(geo%Comm, rhdMPI_RemainDims, CommSub3, ierrMPI)
  call MPI_Comm_Rank(CommSub3, MyProcSub3, ierrMPI)
  PCoordsSub3=PCoords(dim)
  call MPI_Cart_Rank(CommSub3, PCoordsSub3, ProcZeroZero, ierrMPI)
  !
  if (MyProcSub12 == ProcZero) then
    ! --- Gather data only for columns with reduced data ---
    !
    ! --- Compose arrays for sizes and shifts ---
    do iP=0,geo%nProcsPD(dim)-1
      PCoords(dim)=iP
      call rhd_geo_Coords2mnVec(geo, PCoords, mVec, nVec)
      if (IP == 0) then
        mAlldatx=mVec(dim)
      endif
      if (iP == (geo%nProcsPD(dim)-1)) then
        nAlldatx=nVec(dim)
        dit=1
      else
        dit=0
      endif
      SizeAr( iP)=(nVec(dim)-mVec(dim)+dit+1) * datx%na
      ShiftAr(iP)=(mVec(dim)-geo%m(dim)     ) * datx%na
    end do ! iP
    !
    ! write(6,'(A,3I3,A,I3,2X,8I6)') '#', geo%MyProc, MyProcSub12, ProcZero, &
    !    ' rhd_datx_MPICollectx: dim, SizeAr: ', dim, SizeAr
    ! write(6,'(A,3I3,A,I3,2X,8I6)') '#', geo%MyProc, MyProcSub12, ProcZero, &
    !    ' rhd_datx_MPICollectx: dim, ShiftAr:', dim, ShiftAr
    !
    if (geo%MyProc == geo%MaProcWoInCO) then
      !allocate(dTransLrg(datxLrg%na, datxLrg%m:datxLrg%n+1))
      allocate(dTransLrg(datxLrg%na, mAlldatx:nAlldatx+1))
    else
      ! nullify(dTransLrg)
    endif
    !
    call MPI_Gatherv(dTrans1  , (datx%n-datx%m+di+1)*datx%na, mArkMPI, &
                     dTransLrg, SizeAr, ShiftAr             , mArkMPI, &
                     ProcZeroZero, CommSub3, iErrMPI)
    if (geo%MyProc == geo%MaProcWoInCO) then
      !datxLrg%d=transpose(dTransLrg)
      datxLrg%d( mAlldatx:nAlldatx+1, 1:datxLrg%na)=transpose(dTransLrg)
      deallocate(dTransLrg)
    endif
  endif ! (MyProcSub12 == ProcZero)
  !
  ! --- Free the communicators corresponding to columns --
  call MPI_Comm_Free(CommSub3, ierrMPI)
  !
endif
!
deallocate(dTrans1)
!
end subroutine rhd_datx_MPICollectx


!----------********************---------------------------------------------------------------------
subroutine rhd_datx_MPICollectr(geo, datr, datrLrg)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datx_MPICollectr ('rhd_data_x_MPI_Collect_radial')
!
! PURPOSE:
!   Collect distributed spherical-mean data from MPI processes into single structure.
!
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datx_MPICollectr(geo, datr, datrLrg)
!
! INPUT:
!   dim:          ('dimension') integer to specify dimension of mean vectors (1, 2, 3)
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   datr:         ('data_x') type(datx_type), structure with additional output data
!                 for each MPI process
!
! INPUT/OUTPUT:
!   datrLrg:      ('data_x_Large') type(dat_x_type), structure with collected additional output data
!                 from all MPI processes, filled only for the master process
!
! ROUTINES:
!   MPI_Reduce:            Perform a reduction
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Sum data for all processes (MPI reduction step).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-09-11 (B.F. Uppsala) Written
!   2019-01-29 (B.F.) Rename rhd_datx_Collectr to rhd_datx_MPICollectr
!---------------------------------------------------------------------------------------------------
use rhd_prop_module
use rhd_meankind_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(datx_type),           intent(in)   :: datr
type(datx_type),           intent(inout):: datrLrg
!
! --- Local variables ---
integer                                 :: ierrMPI
real(kind=mArk), &
    dimension(datr%m:datr%n+1, datr%na) :: d
!---------------------------------------------------------------------------------------------------
!
! write(6,'(A,I3,A)') '#', geo%MyProc, '       rhd_datx_MPICollectr'
!
! --- Reduce the averages over spheres by summing up the individual contributions ---
if (geo%MyProc == geo%MaProcWoInCO) then
  datrLrg%d=0.0_mArk
  call MPI_Reduce(datr%d, datrLrg%d, (datr%n-datr%m+2)*datr%na, mArkMPI, MPI_SUM, &
                  geo%MaProcWoInCO, geo%Comm, ierrMPI)
else
  d=datr%d
  call MPI_Reduce(datr%d, d        , (datr%n-datr%m+2)*datr%na, mArkMPI, MPI_SUM, &
                  geo%MaProcWoInCO, geo%Comm, ierrMPI)
endif
!
end subroutine rhd_datx_MPICollectr


!----------*******************----------------------------------------------------------------------
subroutine rhd_datc_MPICollect(geo, datc, datcLrg)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_datc_MPICollect ('rhd_data_cylinder_MPI_Collect')
!
! PURPOSE:
!   Collect distributed azimuthal mean data from MPI processes into single structure.
!
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_datc_MPICollect(geo, datc, datcLrg)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   datc:         ('data_x') type(datc_type), structure with additional output data
!                 for each MPI process
!
! INPUT/OUTPUT:
!   datcLrg:      ('data_x_Large') type(dat_x_type), structure with collected additional output data
!                 from all MPI processes, filled only for the master process
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in Cartesian topology from rank in group
!   MPI_Cart_Rank:         Determine process rank in communicator from Cartesian coordinates
!   MPI_Cart_Sub:          Partition a Cartesian communicator group into lower-dimension sub groups
!   MPI_Comm_Free:         Free a communicator
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!   MPI_Gatherv:           Gather varying amounts of data from all processes to the root process
!   MPI_Reduce:            Perform a reduction
!   rhd_geo_Coords2mnVec:  Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_meankind_module:   Kind parameters for averaging routines
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, sum data over horizontal planes (MPI reduction step).
!   Secondly, gather data along columns (MPI gather step).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-09-13 (B.F. Uppsala) Adapted from rhd_datx_Collectx
!   2019-01-29 (B.F.) Rename rhd_datc_Collect to rhd_datc_MPICollect
!   2019-02-08 (B.F.) Account for array in datcLrg structure larger than in sum of datc structures
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use rhd_meankind_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(datc_type),           intent(in)   :: datc
type(datc_type),           intent(inout):: datcLrg
!
! --- Local variables ---
integer, parameter                      :: dim=3
integer                                 :: ierrMPI, iP, &
                                           m1,n1, m2,n2, m3,n3, i2, &
                                           mAlldatc, nAlldatc, &
                                           CommSub12, MyProcSub12, &
                                           CommSub3 , MyProcSub3 , &
                                           ProcZero, ProcZeroZero
!                                           SizeSub12
logical, dimension(3)                   :: rhdMPI_RemainDims
real,    dimension(:,:,:), allocatable  :: dTrans1, dTrans2, dTransLrg
integer, dimension(3)                   :: PCoords, mVec, nVec
integer, dimension(2)                   :: PCoordsSub12
integer, dimension(1)                   :: PCoordsSub3
integer, &
       dimension(0:geo%nProcsPD(dim)-1) :: SizeAr, ShiftAr
!---------------------------------------------------------------------------------------------------
!
! === Reduce all data within planes ================================================================
!
! write(6,'(A,I3,A,I3,2X,6(1PE13.6))') '#', geo%MyProc, &
!    '       rhd_datc_MPICollect:  dim,datc%d:  ', dim, datc%d(datc%m1+2,datc%m2+2,1:6)
!
allocate(dTrans1(datc%na, datc%m2:datc%n2, datc%m1:datc%n1))
!
! --- Create 1D vector of Cartesian communicators corresponding to planes --
rhdMPI_RemainDims     =.true.
rhdMPI_RemainDims(dim)=.false.
call MPI_Cart_Sub(geo%Comm, rhdMPI_RemainDims, CommSub12, ierrMPI)
call MPI_Comm_Rank(CommSub12, MyProcSub12, ierrMPI)
!
! call MPI_Comm_Size(CommSub12, SizeSub12, ierrMPI)
! write(6,'(A,2I3,A,I3,2X,I3)') '#', geo%MyProc, MyProcSub12, &
!    '    rhd_datc_MPICollect:  dim, CommSizeSub12:', dim, SizeSub12
!
! --- Find projection of master process in each plane ---
call MPI_Cart_Coords(geo%Comm, geo%MaProcWoInCO, 3, PCoords, ierrMPI)
if      (dim == 1) then
  PCoordsSub12=(/ PCoords(2), PCoords(3) /)
else if (dim == 2) then
  PCoordsSub12=(/ PCoords(1), PCoords(3) /)
else
  PCoordsSub12=(/ PCoords(1), PCoords(2) /)
endif
call MPI_Cart_Rank(CommSub12, PCoordsSub12, ProcZero, ierrMPI)
!
if ((geo%nProcsPD(1)*geo%nProcsPD(2)*geo%nProcsPD(3)/geo%nProcsPD(dim)) == 1) then
  ! --- There is only one process in this plane: just copy the data ---
  !
  ! write(6,'(A,3I3,A,I3)') '#', geo%MyProc, MyProcSub12, ProcZero, &
  !    ' rhd_datc_MPICollect:  Only one process in plane, dim:', dim
  !
  do i2=datc%m2,datc%n2
    dTrans1(:,i2,:)=transpose(datc%d(:,i2,:))
  end do ! i2
  !
else
  ! --- There is more than one process in this plane: reduce the data ---
  !
  ! --- Reduce the averages within planes by summing up the individual contributions ---
  allocate(dTrans2(datc%na, datc%m2:datc%n2, datc%m1:datc%n1))
  do i2=datc%m2,datc%n2
    dTrans2(:,i2,:)=transpose(datc%d(:,i2,:))
  end do ! i2
  dTrans1=0.0
  call MPI_Reduce(dTrans2, dTrans1, (datc%n1-datc%m1+1)*(datc%n2-datc%m2+1)*datc%na, MPI_REAL, &
                  MPI_SUM, ProcZero, CommSub12, ierrMPI)
  !
  ! write(6,'(A,3I3,A,I3,2X,6(1PE13.6))') '#', geo%MyProc, MyProcSub12, ProcZero, &
  !    ' rhd_datc_MPICollect:  dim,dTrans2: ', dim, dTrans2(1:6,datc%m2+2,datc%m1+2)
  !
  deallocate(dTrans2)
  !
endif
!
! --- Free the communicators corresponding to planes --
call MPI_Comm_Free(CommSub12, ierrMPI)
!
! write(6,'(A,3I3,A,I3,2X,6(1PE13.6))') '#', geo%MyProc, MyProcSub12, ProcZero, &
!    ' rhd_datc_MPICollect:  dim,dTrans1: ', dim, dTrans1(1:6,datc%m2+2,datc%m1+2)
!
!
! === Gather all data along columns ================================================================
!
if (geo%nProcsPD(dim) == 1) then
  ! --- There is only one process in this dimension: just copy the data ---
  !
  ! write(6,'(A,3I3,A,I3)') '#', geo%MyProc, MyProcSub12, ProcZero, &
  !    ' rhd_datc_MPICollect:  Only one process in vector, dim:', dim
  !
  if (geo%MyProc == geo%MaProcWoInCO) then
    do i2=datc%m2,datc%n2
      !datcLrg%d(:,i2,:)=transpose(dTrans1(:,i2,:))
      datcLrg%d(1:datc%na, i2, datc%m1:datc%n1)=transpose(dTrans1(datc%m1:datc%n1, i2, 1:datc%na))
    end do ! i2
  endif
  !
else
  ! --- There is more than one process in this dimension: gather the data ---
  !
  ! --- Create 2D array of Cartesian communicators corresponding to columns --
  rhdMPI_RemainDims     =.false.
  rhdMPI_RemainDims(dim)=.true.
  call MPI_Cart_Sub(geo%Comm, rhdMPI_RemainDims, CommSub3, ierrMPI)
  call MPI_Comm_Rank(CommSub3, MyProcSub3, ierrMPI)
  PCoordsSub3=PCoords(dim)
  call MPI_Cart_Rank(CommSub3, PCoordsSub3, ProcZeroZero, ierrMPI)
  !
  if (MyProcSub12 == ProcZero) then
    ! --- Gather data only for columns with reduced data ---
    !
    ! --- Compose arrays for sizes and shifts ---
    do iP=0,geo%nProcsPD(dim)-1
      PCoords(dim)=iP
      call rhd_geo_Coords2mnVec(geo, PCoords, mVec, nVec)
      if (IP == 0                    ) mAlldatc=mVec(dim)
      if (IP == (geo%nProcsPD(dim)-1)) nAlldatc=nVec(dim)
      SizeAr( iP)=(nVec(dim)-mVec(dim)+1) * ((datc%n2-datc%m2+1) * datc%na)
      ShiftAr(iP)=(mVec(dim)-geo%m(dim) ) * ((datc%n2-datc%m2+1) * datc%na)
    end do ! iP
    !
    ! write(6,'(A,3I3,A,I3,2X,8I6)') '#', geo%MyProc, MyProcSub12, ProcZero, &
    !    ' rhd_datc_MPICollect:  dim, SizeAr: ', dim, SizeAr
    ! write(6,'(A,3I3,A,I3,2X,8I6)') '#', geo%MyProc, MyProcSub12, ProcZero, &
    !   ' rhd_datc_MPICollect:  dim, ShiftAr:', dim, ShiftAr
    !
    if (geo%MyProc == geo%MaProcWoInCO) then
      !allocate(dTransLrg(datcLrg%na, datcLrg%m2:datcLrg%n2, datcLrg%m1:datcLrg%n1))
      allocate(dTransLrg(datcLrg%na, datcLrg%m2:datcLrg%n2, mAlldatc:nAlldatc))
    else
      ! nullify(dTransLrg)
    endif
    !
    call MPI_Gatherv(dTrans1  , (datc%n1-datc%m1+1)*(datc%n2-datc%m2+1)*datc%na, MPI_REAL, &
                     dTransLrg, SizeAr, ShiftAr                                , MPI_REAL, &
                     ProcZeroZero, CommSub3, iErrMPI)
    if (geo%MyProc == geo%MaProcWoInCO) then
      do i2=datc%m2,datc%n2
        !datcLrg%d(:,i2,:)=transpose(dTransLrg(:,i2,:))
        datcLrg%d(mAlldatc:nAlldatc, i2, 1:datcLrg%na)= &
                     transpose(dTransLrg(1:datcLrg%na, i2, mAlldatc:nAlldatc))
      end do ! i2
      deallocate(dTransLrg)
    endif
  endif ! (MyProcSub12 == ProcZero)
  !
  ! --- Free the communicators corresponding to columns --
  call MPI_Comm_Free(CommSub3, ierrMPI)
  !
endif
!
deallocate(dTrans1)
!
end subroutine rhd_datc_MPICollect


!----------*********************--------------------------------------------------------------------
subroutine rhd_dat_MPICollectInt(geo, dat, datLrg)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_MPICollectInt ('rhd_data_MPI_Collect_Intensity')
!
! PURPOSE:
!   Collect distributed surface intensities from MPI processes into single structure.
!
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_MPICollectInt(geo, dat, datLrg)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   dat:          ('data') type(dat_type), structure with additional output data
!                 for each MPI process
!
! OUTPUT:
!   datLrg:       ('data_Large') type(dat_type), structure with collected additional output data
!                 from all MPI processes, filled only for the master process
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in cartesian topology from rank in group
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_dat_MPICreateType: Create MPI data type for dat structure
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The output datLrg structure has to be created outside with rhd_dat_make,
!   by the master process only.
!
! PROCEDURE:
!   Collect the intensity 2D arrays from all MPI processes into single output structure.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-06 (B.F. Uppsala) First outline
!   2017-09-11 (B.F.) Call routine to collect datx structures
!   2017-09-13 (B.F.) Call routine to collect datc structure
!   2019-01-28 (B.F.) Create datLrg structure outside routine
!   2019-01-29 (B.F.) Extract from rhd_dat_MPICollect
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(dat_type),            intent(in)   :: dat
type(dat_type),            intent(inout):: datLrg
!
! --- Local variables ---
integer                                 :: ierrMPI, NProcs, TaProc, iTaProc, &
                                           m1,n1, m2,n2, m3,n3
logical                                 :: IsEmpty_flag
integer, dimension(3)                   :: PCoords
integer, dimension(:), allocatable      :: rhdMPI_datLrgTypes, rhdMPI_datTypes, RequestsMPI
integer, dimension(:,:), allocatable    :: StatusMPI
!---------------------------------------------------------------------------------------------------
!
if (geo%MyProcIsInCOMM_flag) then
  ! --- Only processes that are member of the current communicator participate ---
  !
  if (geo%MyProc == geo%MaProcWoInCO) then
    ! --- Create the large dat structure by the (global) master process ---
    !
    ! --- Create temporary arrays ---
    call MPI_Comm_Size(geo%Comm, NProcs, ierrMPI)
    iTaProc=NProcs-1
    allocate(rhdMPI_datLrgTypes(        1:iTaProc), &
             RequestsMPI(               1:iTaProc), &
             StatusMPI(MPI_STATUS_SIZE, 1:iTaProc))
    !
    ! --- Loop over the target processes ---
    iTaProc=0
    do TaProc=0,Nprocs-1
      ! --- Get target-process coordinates ---
      call MPI_Cart_Coords(geo%Comm, TaProc, 3, PCoords, ierrMPI)
      call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
      !
      IsEmpty_flag= &
               ((PCoords(1) > 0                  ) .or. (.not. associated(datLrg%Int1_l  ))) .and. &
               ((PCoords(1) < (geo%nProcsPD(1)-1)) .or. (.not. associated(datLrg%Int1_r  ))) .and. &
               ((PCoords(2) > 0                  ) .or. (.not. associated(datLrg%Int2_l  ))) .and. &
               ((PCoords(2) < (geo%nProcsPD(2)-1)) .or. (.not. associated(datLrg%Int2_r  ))) .and. &
               ((PCoords(3) > 0                  ) .or. (.not. associated(datLrg%Int3_l  ))) .and. &
               ((PCoords(3) < (geo%nProcsPD(3)-1)) .or. (.not. associated(datLrg%Int3_r  )) &
                                                   .or. (.not. associated(datLrg%Intxx3_r)))
      !
      if (.not. IsEmpty_flag) then
        if (TaProc == geo%MyProc) then
          ! --- Target is identical to source: just copy the data ---
          !
          ! write(gl%nc_p, '(A,I3,A,I3)') '#', geo%MyProc, &
          !   ' Start copying small local dataset into large one ', TaProc
          if (associated(datLrg%Int1_l) .and. associated(dat%Int1_l)) then
            datLrg%Int1_l(m2:n2, m3:n3)=dat%Int1_l(m2:n2, m3:n3)
          endif
          if (associated(datLrg%Int1_r) .and. associated(dat%Int1_r)) then
            datLrg%Int1_r(m2:n2, m3:n3)=dat%Int1_r(m2:n2, m3:n3)
          endif
          if (associated(datLrg%Int2_l) .and. associated(dat%Int2_l)) then
            datLrg%Int2_l(m1:n1, m3:n3)=dat%Int2_l(m1:n1, m3:n3)
          endif
          if (associated(datLrg%Int2_r) .and. associated(dat%Int2_r)) then
            datLrg%Int2_r(m1:n1, m3:n3)=dat%Int2_r(m1:n1, m3:n3)
          endif
          if (associated(datLrg%Int3_l) .and. associated(dat%Int3_l)) then
            datLrg%Int3_l(m1:n1, m2:n2)=dat%Int3_l(m1:n1, m2:n2)
          endif
          if (associated(datLrg%Int3_r) .and. associated(dat%Int3_r)) then
            datLrg%Int3_r(m1:n1, m2:n2)=dat%Int3_r(m1:n1, m2:n2)
          endif
          !
          if (associated(datLrg%Intxx3_r) .and. associated(dat%Intxx3_r)) then
            datLrg%Intxx3_r(m1:n1, m2:n2, :)=dat%Intxx3_r(m1:n1, m2:n2, :)
          endif
          !
          ! write(gl%nc_p, '(A,I3,A,I3)') '#', geo%MyProc, &
          !   ' Stop copying small local dataset into large one ', TaProc
          !
        else ! (TaProc /= geo%MyProc)
          ! --- Data has to be received from different process ---
          !
          iTaProc=iTaProc+1
          ! --- Create MPI data type ---
          ! write(gl%nc_p, '(A,I3,A,I3)') '#', geo%MyProc, &
          !   ' Create MPI type for part of large dataset ', TaProc
          call rhd_dat_MPICreateType(datLrg, m1,n1, m2,n2, m3,n3, &
                                     rhdMPI_datLrgTypes(iTaProc), &
                                     IncludeIntensity_flag=.true., IncludeFlux_flag=.false.)
          !
          ! --- Get it ---
          ! write(gl%nc_p, '(A,I3,A,2I3)') '#', geo%MyProc, &
          !   ' Receive part of large dataset from ', TaProc
          call MPI_IRecv(datLrg, 1, rhdMPI_datLrgTypes(iTaProc), TaProc, &
                         MPI_ANY_TAG, geo%Comm, RequestsMPI(iTaProc), ierrMPI)
          !
        endif ! (TaProc)
      endif ! (.not. IsEmpty_flag)
      !
    end do ! TaProc
    !
    ! --- Wait until all datasets are copied ---
    ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, ' Wait until all datasets are received'
    call MPI_WaitAll(iTaProc, RequestsMPI, StatusMPI, ierrMPI)
    ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, &
    !   ' Finished waiting that all datasets are received'
    !
    ! --- Free data types ---
    do TaProc=1,iTaProc
      call MPI_Type_Free(rhdMPI_datLrgTypes(TaProc), ierrMPI)
    end do ! TaProc
    !
    ! --- Deallocate arrays ---
    deallocate(rhdMPI_datLrgTypes, &
               RequestsMPI, &
               StatusMPI)
    !
  else ! (geo%MyProc /= 0)
    ! --- The current process is one of the compute processes but definitely not master ---
    !
    ! --- Create temporary arrays ---
    allocate(rhdMPI_datTypes(1:1), &
             RequestsMPI(    1:1), &
             StatusMPI(MPI_STATUS_SIZE, 1))
    !
    ! --- Get local process coordinates ---
    call MPI_Cart_Coords(geo%Comm, geo%MyProc, 3, PCoords, ierrMPI)
    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
    !
    IsEmpty_flag=((PCoords(1) > 0                  ) .or. (.not. associated(dat%Int1_l  ))) .and. &
                 ((PCoords(1) < (geo%nProcsPD(1)-1)) .or. (.not. associated(dat%Int1_r  ))) .and. &
                 ((PCoords(2) > 0                  ) .or. (.not. associated(dat%Int2_l  ))) .and. &
                 ((PCoords(2) < (geo%nProcsPD(2)-1)) .or. (.not. associated(dat%Int2_r  ))) .and. &
                 ((PCoords(3) > 0                  ) .or. (.not. associated(dat%Int3_l  ))) .and. &
                 ((PCoords(3) < (geo%nProcsPD(3)-1)) .or. (.not. associated(dat%Int3_r  )) &
                                                     .or. (.not. associated(dat%Intxx3_r)))
    if (.not. IsEmpty_flag) then
      !
      ! --- Create MPI data type ---
      ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, &
      !   ' Create MPI type for small local dataset'
      call rhd_dat_MPICreateType(dat, m1,n1, m2,n2, m3,n3, &
                                 rhdMPI_datTypes(1), &
                                 IncludeIntensity_flag=.true., IncludeFlux_flag=.false.)

      !
      ! --- Receive it ---
      ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, ' Send small local dataset'
      call MPI_ISend(dat, 1, rhdMPI_datTypes(1), geo%MaProcWoInCO, &
                     geo%MyProc, geo%Comm, RequestsMPI(1), ierrMPI)
      !
      ! --- Wait until all datasets are sent ---
      ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, ' Wait until small local dataset has been sent'
      call MPI_WaitAll(1, RequestsMPI, StatusMPI, ierrMPI)
      ! write(gl%nc_p, '(A,I3,A)') &
      !                      '#', geo%MyProc, ' Finished waiting for small local dataset to be sent'
      !
      ! --- Free data types ---
      call MPI_Type_Free(rhdMPI_datTypes(1), ierrMPI)
      !
      ! --- Deallocate arrays ---
      deallocate(rhdMPI_datTypes, &
                 RequestsMPI, &
                 StatusMPI)
      !
    endif ! (IsEmpty_flag)
  endif ! (geo%MyProc)
  !
endif ! (geo%MyProcIsInCOMM_flag)
!
end subroutine rhd_dat_MPICollectInt


!----------*********************--------------------------------------------------------------------
subroutine rhd_dat_MPICollectFer(geo, dat, datLrg)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_MPICollectFer ('rhd_data_MPI_Collect_Flux_energy_radiative')
!
! PURPOSE:
!   Collect distributed radiative surface fluxes from MPI processes into single structure.
!
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_MPICollectFer(geo, dat, datLrg)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   dat:          ('data') type(dat_type), structure with additional output data
!                 for each MPI process
!
! OUTPUT:
!   datLrg:       ('data_Large') type(dat_type), structure with collected additional output data
!                 from all MPI processes, filled only for the master process
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in cartesian topology from rank in group
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_dat_MPICreateType: Create MPI data type for dat structure
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_datx_MPICollectx:  Collect distributed mean data from MPI processes into single structure
!   rhd_datx_MPICollectr:  Collect distributed spherical-mean data into single structure
!   rhd_datc_MPICollect:   Collect distributed azimuthal mean data into single structure
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The output datLrg structure has to be created outside with rhd_dat_make,
!   by the master process only.
!
! PROCEDURE:
!   Collect radiative surface fluxes from all MPI processes into single output structure.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-06 (B.F. Uppsala) First outline
!   2017-09-11 (B.F.) Call routine to collect datx structures
!   2017-09-13 (B.F.) Call routine to collect datc structure
!   2019-01-28 (B.F.) Create datLrg structure outside routine
!   2019-01-29 (B.F.) Extract from rhd_dat_MPICollect
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(dat_type),            intent(in)   :: dat
type(dat_type),            intent(inout):: datLrg
!
! --- Local variables ---
integer                                 :: ierrMPI, NProcs, TaProc, iTaProc, &
                                           m1,n1, m2,n2, m3,n3
logical                                 :: IsEmpty_flag
integer, dimension(3)                   :: PCoords
integer, dimension(:), allocatable      :: rhdMPI_datLrgTypes, rhdMPI_datTypes, RequestsMPI
integer, dimension(:,:), allocatable    :: StatusMPI
!---------------------------------------------------------------------------------------------------
!
if (geo%MyProcIsInCOMM_flag) then
  ! --- Only processes that are member of the current communicator participate ---
  !
  if (geo%MyProc == geo%MaProcWoInCO) then
    ! --- Create the large dat structure by the (global) master process ---
    !
    ! --- Create temporary arrays ---
    call MPI_Comm_Size(geo%Comm, NProcs, ierrMPI)
    iTaProc=NProcs-1
    allocate(rhdMPI_datLrgTypes(        1:iTaProc), &
             RequestsMPI(               1:iTaProc), &
             StatusMPI(MPI_STATUS_SIZE, 1:iTaProc))
    !
    ! --- Loop over the target processes ---
    iTaProc=0
    do TaProc=0,Nprocs-1
      ! --- Get target-process coordinates ---
      call MPI_Cart_Coords(geo%Comm, TaProc, 3, PCoords, ierrMPI)
      call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
      !write(gl%nc_p, '(A,I3,A,I3,3X,3I3,3X,3(I4,1X,I4,2X))') '#', geo%MyProc, ' from', TaProc, &
      !     PCoords, m1,n1, m2,n2, m3,n3
      !
      IsEmpty_flag= &
               ((PCoords(1) > 0                  ) .or. (.not. associated(datLrg%fer1_l  ))) .and. &
               ((PCoords(1) < (geo%nProcsPD(1)-1)) .or. (.not. associated(datLrg%fer1_r  ))) .and. &
               ((PCoords(2) > 0                  ) .or. (.not. associated(datLrg%fer2_l  ))) .and. &
               ((PCoords(2) < (geo%nProcsPD(2)-1)) .or. (.not. associated(datLrg%fer2_r  ))) .and. &
               ((PCoords(3) > 0                  ) .or. (.not. associated(datLrg%fer3_l  ))) .and. &
               ((PCoords(3) < (geo%nProcsPD(3)-1)) .or. (.not. associated(datLrg%fer3_r  )))
      !
      if (.not. IsEmpty_flag) then
        if (TaProc == geo%MyProc) then
          ! --- Target is identical to source: just copy the data ---
          !
          ! write(gl%nc_p, '(A,I3,A,I3)') '#', geo%MyProc, &
          !   ' Start copying small local dataset into large one ', TaProc
          if (associated(datLrg%fer1_l) .and. associated(dat%fer1_l)) then
            datLrg%fer1_l(m2:n2, m3:n3)=dat%fer1_l(m2:n2, m3:n3)
          endif
          if (associated(datLrg%fer1_r) .and. associated(dat%fer1_r)) then
            datLrg%fer1_r(m2:n2, m3:n3)=dat%fer1_r(m2:n2, m3:n3)
          endif
          if (associated(datLrg%fer2_l) .and. associated(dat%fer2_l)) then
            datLrg%fer2_l(m1:n1, m3:n3)=dat%fer2_l(m1:n1, m3:n3)
          endif
          if (associated(datLrg%fer2_r) .and. associated(dat%fer2_r)) then
            datLrg%fer2_r(m1:n1, m3:n3)=dat%fer2_r(m1:n1, m3:n3)
          endif
          if (associated(datLrg%fer3_l) .and. associated(dat%fer3_l)) then
            datLrg%fer3_l(m1:n1, m2:n2)=dat%fer3_l(m1:n1, m2:n2)
          endif
          if (associated(datLrg%fer3_r) .and. associated(dat%fer3_r)) then
            datLrg%fer3_r(m1:n1, m2:n2)=dat%fer3_r(m1:n1, m2:n2)
          endif
          !
          ! write(gl%nc_p, '(A,I3,A,I3)') '#', geo%MyProc, &
          !   ' Stop copying small local dataset into large one ', TaProc
          !
        else ! (TaProc /= geo%MyProc)
          ! --- Data has to be received from different process ---
          !
          iTaProc=iTaProc+1
          ! --- Create MPI data type ---
          ! write(gl%nc_p, '(A,I3,A,I3)') '#', geo%MyProc, &
          !   ' Create MPI type for part of large dataset ', TaProc
          call rhd_dat_MPICreateType(datLrg, m1,n1, m2,n2, m3,n3, &
                                     rhdMPI_datLrgTypes(iTaProc), &
                                     IncludeIntensity_flag=.false., IncludeFlux_flag=.true.)

          !
          ! --- Get it ---
          ! write(gl%nc_p, '(A,I3,A,2I3)') '#', geo%MyProc, &
          !   ' Receive part of large dataset from ', TaProc
          call MPI_IRecv(datLrg, 1, rhdMPI_datLrgTypes(iTaProc), TaProc, &
                         MPI_ANY_TAG, geo%Comm, RequestsMPI(iTaProc), ierrMPI)
          !
        endif ! (TaProc)
      endif ! (.not. IsEmpty_flag)
      !
    end do ! TaProc
    !
    ! --- Wait until all datasets are copied ---
    ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, ' Wait until all datasets are received'
    call MPI_WaitAll(iTaProc, RequestsMPI, StatusMPI, ierrMPI)
    ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, &
    !   ' Finished waiting that all datasets are received'
    !
    ! --- Free data types ---
    do TaProc=1,iTaProc
      call MPI_Type_Free(rhdMPI_datLrgTypes(TaProc), ierrMPI)
    end do ! TaProc
    !
    ! --- Deallocate arrays ---
    deallocate(rhdMPI_datLrgTypes, &
               RequestsMPI, &
               StatusMPI)
    !
  else ! (geo%MyProc /= 0)
    ! --- The current process is one of the compute processes but definitely not master ---
    !
    ! --- Create temporary arrays ---
    allocate(rhdMPI_datTypes(1:1), &
             RequestsMPI(    1:1), &
             StatusMPI(MPI_STATUS_SIZE, 1))
    !
    ! --- Get local process coordinates ---
    call MPI_Cart_Coords(geo%Comm, geo%MyProc, 3, PCoords, ierrMPI)
    call rhd_geo_Coords2mn(geo, PCoords, m1,n1, m2,n2, m3,n3)
    !
    IsEmpty_flag=((PCoords(1) > 0                  ) .or. (.not. associated(dat%fer1_l  ))) .and. &
                 ((PCoords(1) < (geo%nProcsPD(1)-1)) .or. (.not. associated(dat%fer1_r  ))) .and. &
                 ((PCoords(2) > 0                  ) .or. (.not. associated(dat%fer2_l  ))) .and. &
                 ((PCoords(2) < (geo%nProcsPD(2)-1)) .or. (.not. associated(dat%fer2_r  ))) .and. &
                 ((PCoords(3) > 0                  ) .or. (.not. associated(dat%fer3_l  ))) .and. &
                 ((PCoords(3) < (geo%nProcsPD(3)-1)) .or. (.not. associated(dat%fer3_r  )))
    !
    if (.not. IsEmpty_flag) then
      !
      ! --- Create MPI data type ---
      ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, &
      !   ' Create MPI type for small local dataset'
      call rhd_dat_MPICreateType(dat, m1,n1, m2,n2, m3,n3, &
                                 rhdMPI_datTypes(1), &
                                 IncludeIntensity_flag=.false., IncludeFlux_flag=.true.)

      !
      ! --- Receive it ---
      ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, ' Send small local dataset'
      call MPI_ISend(dat, 1, rhdMPI_datTypes(1), geo%MaProcWoInCO, &
                     geo%MyProc, geo%Comm, RequestsMPI(1), ierrMPI)
      !
      ! --- Wait until all datasets are sent ---
      ! write(gl%nc_p, '(A,I3,A)') '#', geo%MyProc, ' Wait until small local dataset has been sent'
      call MPI_WaitAll(1, RequestsMPI, StatusMPI, ierrMPI)
      ! write(gl%nc_p, '(A,I3,A)') &
      !                      '#', geo%MyProc, ' Finished waiting for small local dataset to be sent'
      !
      ! --- Free data types ---
      call MPI_Type_Free(rhdMPI_datTypes(1), ierrMPI)
      !
      ! --- Deallocate arrays ---
      deallocate(rhdMPI_datTypes, &
                 RequestsMPI, &
                 StatusMPI)
      !
    endif ! (IsEmpty_flag)
  endif ! (geo%MyProc)
  !
endif ! (geo%MyProcIsInCOMM_flag)
!
end subroutine rhd_dat_MPICollectFer


!----------******************-----------------------------------------------------------------------
subroutine rhd_dat_MPICollect(geo, dat, datLrg, &
                              IncludeIntensity_flag, IncludeFlux_flag)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_dat_MPICollect ('rhd_data_MPI_Collect')
!
! PURPOSE:
!   Collect distributed data from MPI processes into single structure.
!
! CATEGORY:
!   dat handling
!
! CALLING SEQUENCE:
!   call rhd_dat_MPICollect(geo, dat, datLrg, &
!                           IncludeIntensity_flag=IncludeIntensity_flag, &
!                           IncludeFlux_flag=IncludeFlux_flag)
!
! INPUT:
!   geo:          ('geometry') structure with geometry data about current domain decomposition
!   dat:          ('data') type(dat_type), structure with additional output data
!                 for each MPI process
!   IncludeIntensity_flag: ('Include_Intensity_flag') optional logical to indicate that surface
!                          intensity arrays should be included in MPI data structure, default=.true.
!   IncludeFlux_flag: ('Include_Flux_flag') optional logical to indicate that surface flux
!                          arrays should be included in MPI data structure, default=.true.
!
! OUTPUT:
!   datLrg:       ('data_Large') type(dat_type), structure with collected additional output data
!                 from all MPI processes, filled only for the master process
!
! ROUTINES:
!   MPI_Cart_Coords:       Determine process coordinates in cartesian topology from rank in group
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Type_Free:         Free an MPI data type
!   MPI_WaitAll:           Wait for all given communications to complete
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_dat_MPICreateType: Create MPI data type for dat structure
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_datx_MPICollectx:  Collect distributed mean data from MPI processes into single structure
!   rhd_datx_MPICollectr:  Collect distributed spherical-mean data into single structure
!   rhd_datc_MPICollect:   Collect distributed azimuthal mean data into single structure
!   rhd_dat_MPICollectInt: Collect distributed intensities from MPI processes into single structure
!   rhd_dat_MPICollectFer: Collect distributed rad. fluxes from MPI processes into single structure
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_prop_module:       Properties of quantities in 'box' structure
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   The output datLrg structure has to be created outside with rhd_dat_make,
!   by the master process only.
!
! PROCEDURE:
!   Collect the various types of data (including surface fluxes and different averages)
!   from all MPI processes into single output structure.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2017-08-06 (B.F. Uppsala) First outline
!   2017-09-11 (B.F.) Call routine to collect datx structures
!   2017-09-13 (B.F.) Call routine to collect datc structure
!   2019-01-28 (B.F.) Create datLrg structure outside routine
!   2019-01-29 (B.F.) Rename rhd_dat_Collect to rhd_dat_MPICollect
!   2019-01-30 (B.F.) Add optional parameters; call subroutines to collect radiation data
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_prop_module
use MPI
!
implicit none
!
! --- I/O parameters ---
type(geo_type),            intent(in)   :: geo
type(dat_type),            intent(in)   :: dat
logical, optional,         intent(in)   :: IncludeIntensity_flag, IncludeFlux_flag
type(dat_type),            intent(inout):: datLrg
!
! --- Local variables ---
integer                                 :: ierrMPI, NProcs, TaProc, iTaProc, &
                                           m1,n1, m2,n2, m3,n3
logical                                 :: IsEmpty_flag, &
                                           IncludeIntensity_flag0, IncludeFlux_flag0
integer, dimension(3)                   :: PCoords
integer, dimension(:), allocatable      :: rhdMPI_datLrgTypes, rhdMPI_datTypes, RequestsMPI
integer, dimension(:,:), allocatable    :: StatusMPI
!---------------------------------------------------------------------------------------------------
!
! --- Set default values for keywords not present in input list ---
if (.not. present(IncludeIntensity_flag)) then
  IncludeIntensity_flag0=.true.
else
  IncludeIntensity_flag0=IncludeIntensity_flag
endif
if (.not. present(IncludeFlux_flag)) then
  IncludeFlux_flag0=.true.
else
  IncludeFlux_flag0=IncludeFlux_flag
endif
!
if (geo%MyProcIsInCOMM_flag) then
  ! --- Only processes that are member of the current communicator participate ---
  !
  if (geo%MyProc == geo%MaProcWoInCO) then
    !
    ! --- Copy scalar(s) ---
    datLrg%luminosity=dat%luminosity
    !
  endif ! (geo%MyProc)
  !
  if (IncludeIntensity_flag0) then
    ! --- Collect surface intensities ---
    call rhd_dat_MPICollectInt(geo, dat, datLrg)
  endif
  !
  if (IncludeFlux_flag0) then
    ! --- Collect surface fluxes ---
    call rhd_dat_MPICollectFer(geo, dat, datLrg)
  endif
  !
  ! --- Collect horizontal means ---
  call rhd_datx_MPICollectx(1, geo, dat%x23, datLrg%x23)
  call rhd_datx_MPICollectx(2, geo, dat%x13, datLrg%x13)
  call rhd_datx_MPICollectx(3, geo, dat%x12, datLrg%x12)
  !
  if (par%grav_mode == 'central') then
    if (dat%r%na > 0) then
      ! --- Collect spherical means ---
      call rhd_datx_MPICollectr(geo, dat%r, datLrg%r)
    endif
    if (dat%c%na > 0) then
      ! --- Collect aximuthal means ---
      call rhd_datc_MPICollect(geo, dat%c, datLrg%c)
    endif
  endif
  !
endif ! (geo%MyProcIsInCOMM_flag)
!
end subroutine rhd_dat_MPICollect

!#
# endif
!#      /* rhd_mpi_l01 ========================================================================== */

end module rhd_dat_module
