!***************************************************************************************************
!
!   #####   #    #  #####         #   ####
!   #    #  #    #  #    #        #  #    #
!   #    #  ######  #    #        #  #    #
!   #####   #    #  #    #        #  #    #
!   #   #   #    #  #    #        #  #    #
!   #    #  #    #  #####  ###### #   #### _module
!
!   Radiation-Hydrodynamics Code: Input/output routines
!
!***************************************************************************************************
!   Fortran90
!   2019-03-05
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Werner Schaffenberger: Potsdam, Freiburg, Exeter
!   Sven Wedemeyer-Boehm:  Kiel, Freiburg
!***************************************************************************************************
! MODULES:
!   rhd_io_module:         Input/output routines
!
!***************************************************************************************************

!# /* --- Default values for some preprocessor macros ------------------------------------------- */
!#
!# /* --- Type of box array components: 0: classical pointer arrays (default) --- */
!# /* ---                               1: new allocatable arrays (faster) --- */
# ifndef rhd_box_arrays01
#   define rhd_box_arrays01 0
# endif
# if (rhd_box_arrays01 == 0)
#   define ALLOASSOTED associated
#   define BOXARRAYTYPE pointer
# else
#   define ALLOASSOTED allocated
#   define BOXARRAYTYPE allocatable
# endif
!#
!# /* --- MPI flag activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*************--------------------------------------------------------------------------------
module rhd_io_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_io_module ('rhd_input_output_module')
!
! PURPOSE:
!   Input/Output routines for rhd.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use rhd_io_module
!
! VARIABLES:
!   None
!
! SUBROUTINES: (contained)
!   rhd_par_RdData:        Read parameters from parameter file in UIO form for RHD code
!   rhd_box_WrData:        Write RHD model into file in UIO form
!   rhd_box_RdData:        Read RHD model from file in UIO form
!
! MODIFICATION HISTORY:
!   1997-06-09 (B.F. Kiel) Written
!   1997-06-21 (B.F.) rhd_wrboxdata
!   1997-10-09 (B.F.) Put into own file
!   2002-10-18 (B.F.) I/O of quc
!   2005-05-31 (W.S.&SWB) MHD parameters added
!   2006-08-07 (SWB)  hion_datapath, hion_atomfile, hion_abufile, hion_edensfile,
!                     hion_pffile, dtime_out_hion, hion_chunks
!   2007-10-22 (B.F. Lyon) box%time_db introduced
!   2010-03-19 (B.F.) Option to switch between pointer and allocatable arrays
!   2010-05-10 (B.F.) Routines renamed
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------**************---------------------------------------------------------------------------
subroutine rhd_par_RdData(parFile, nc_p, n_dustGrainRadiusMax, &
  ndes, description, nhis, history, &
  model, counter, programversion, &
  Teff, heat_mode, &
  grav_mode, grav_type, grav, C_gravmu, C_gravCorePotDepth, C_gravCorePotQuad, &
  mass_star, r0_grav, r1_grav, r2_grav, r1_rad, C_gravCorer0Heat, &
  centrifugal_force, nu_rotation, ar_RotationAxis, &
  side_bound, top_bound, bottom_bound, &
    side_bound_mag_x1, side_bound_mag_x2, top_bound_mag, bottom_bound_mag, &
    s_InFlow, s_InFlRadGrad, s_InFlLatGrad, C_sChange, C_PChange, &
    C_v3ChangeLinBottom, C_v3ChangeSqrBottom, &
    C_TChange, C_rhoChangeTop, C_Tsurf, C_TminLimit, C_TminRate, &
    C_Hptopfactor, rho_min, LuminosityPerVolume, hdCoreDragProfile, hdCoreHeatProfile, C_CoreDrag, &
    B1_inflow, C_magthetaB, C_magphiB, N_magDiffSide, N_magDiffTop, N_magDiffBottom, &
  abuX, abuY, abuZ, Qmol, gamma, &
  EOSFile, EOSPath, kappaMin, kappaMax, OpaFile, OpaPath, &
    chem_abumetal, chem_reacfile, chem_reacpath, &
    hion_datapath, hion_atomfile, hion_abufile, hion_edensfile, &
    hion_pffile, dtime_out_hion, hion_chunks, &
  hdScheme, hdTimeIntegrationScheme, hdsplit, hdTmpStructKeepMode, hdCheckFlux, reconstruction, &
    hdTransVeloMode, hdEntropyWaveMode, hdEnthalpyAvgMode, &
    N_orderConstrainedTransport, N_hydIter, N_hydMaxIter, N_hydRKIter, &
    C_hydTDiffLin, C_hydTDiffMach, C_hydsDiffVelo, C_hydvDiffVelo, C_hydLowMachcsFactor, &
    C_hydLowMachcsBase, C_hydSmallFluctFactor, &
    N_hydCellsPerChunk, N_hydCellsPerChunk2, N_hydCellsPerChunk3, N_visCellsPerChunk, &
    N_resCellsPerChunk, N_radCellsPerChunk, aN_MPIProcsPerDim, &
    C_SlopeRed, C_RecContShift, C_RecContSteep, C_RecWenoWeightCenter, C_RecWenoWeightRight, &
    C_RecWenoPsiFactor, C_hydPredfactor, &
    C_visBound, &
    beta_inv, va_max, &
  dustScheme, dustReconstruction, dustOpaFile, dustopamode, &
    n_dustGrainRadius, ar_dustGrainRadius, n_dustGrainAbu, ar_dustGrainAbu, &
    C_dust01, C_dust02, C_dust03, C_dust04, C_dust05, C_dust06, C_dust07, C_dust08, C_dust09, &
    C_dust10, &
  radScheme, radThinHeatScheme, radThinPressScheme, &
    N_radminiter, N_raditer, N_radmaxiter, N_radtaurefine, &
    N_radThinPoint, N_radThinHeatPoint, N_radThinPressPoint, N_radThickPoint, &
    N_radtheta, N_radphi, N_radsubray, N_radband, &
    N_radrsyslevel, N_radoutput, N_radFeautrier, &
    C_radimplicitmu, C_raditereps, C_raditerstep, &
    C_radkappasmooth, C_radTstep, C_radTsmooth, C_radTintminfac, &
    C_radHtautop, C_radTvisdtau, C_radTvis, &
    C_radthintimefac, C_radthintimeRelax, C_radthinTFactor, &
    C_radtcool, C_radscool, C_raddcool, &
    C_radTinci, C_radDinci, C_radAccEnergyFraction, &
    radraybase, radraystar, radpressure, &
  C_visLinear, C_visSmagorinsky, C_visArtificial, C_visExpansion, C_visPrturb, &
    C_visTensordiag, C_visTensoroff, C_visTensordiv, C_visP2Pcoeff, C_visP2Pincl1, C_visP2Pincl2, &
    C_visP2PhypSmagorinsky, C_visP2PhypArtificial, C_visP2PhypExpansion, C_visP2Pdivrhov, &
    N_visIter, N_visP2PhypSmooth, visP2Phypreconstruction, &
    C_resB, C_resBconstant, C_resEpsilon, &
  starttime, endtime, plustime, time_ScaleStart, time_ScaleEnd, &
    CPUtime, CPUtime_remainlimit, &
    starttimestep, endtimestep, plustimestep, &
  dtime_min, dtime_max, dtime_min_stop, dtime_incmax, dtime_start, &
  C_Courant, C_CourantMax, C_maxeiChange, C_hydExpCourant, C_hydExpCourantMax, &
    C_hydSoundCourant, C_hydSoundCourantMax, &
    C_radCourant, C_radCourantMax, C_radmaxeiChange, &
    C_visCourant, C_visCourantMax, C_resCourant, C_resCourantMax, &
  dtime_out_full, dtime_out_mean, dtime_out_end, dtimestep_out_fine, istep_in_start, &
    N_outslicedim_mean, &
  infile_start, &
    OutFile_end,  OutForm_end,  OutConv_end, &
    OutFile_full, OutForm_full, OutConv_full, &
    OutFile_mean, OutForm_mean, OutConv_mean, &
    OutFile_fine, OutForm_fine, OutConv_fine, &
    OutFile_mcyl, &
    OutFile_chu1, OutFile_chu2, OutFile_chu3, &
    OutFile_out, &
    OutDim_chu1,  OutDim_chu2,  OutDim_chu3, &
  N_test1, N_test2, N_test3, N_test4, N_test5, C_test1, C_test2, C_test3, C_test4, C_test5, &
    strtest1, strtest2, strtest3, strtest4, strtest5, &
  outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_par_RdData ('rhd_parameters_Read_Data')
!
! PURPOSE:
!   Read parameters from parameter file in UIO form for RHD code.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_par_RdData(parFile, nc_p, n_dustGrainRadiusMax, &
!          ndes=ndes, description=description, &
!          nhis=nhis, history=history, &
!          model=model, counter=counter, programversion=programversion, &
!          Teff=Teff, heat_mode=heat_mode, &
!          grav_mode=grav_mode, grav_type=grav_type, grav=grav, C_gravmu=C_gravmu, &
!            C_gravCorePotDepth=C_gravCorePotDepth, C_gravCorePotQuad=C_gravCorePotQuad, &
!            mass_star=mass_star, r0_grav=r0_grav, r1_grav=r1_grav, r2_grav=r2_grav, &
!          r1_rad=r1_rad, C_gravCorer0Heat=C_gravCorer0Heat, &
!          centrifugal_force=centrifugal_force, nu_rotation=nu_rotation, &
!            ar_RotationAxis=ar_RotationAxis, &
!          side_bound=side_bound, top_bound=top_bound, bottom_bound=bottom_bound, &
!          side_bound_mag_x1=side_bound_mag_x1, side_bound_mag_x2=side_bound_mag_x2, &
!          top_bound_mag=top_bound_mag, bottom_bound_mag=bottom_bound_mag, &
!          s_InFlow=s_InFlow, s_InFlRadGrad=s_InFlRadGrad, s_InFlLatGrad=s_InFlLatGrad, &
!            C_sChange=C_sChange, C_PChange=C_PChange, &
!            C_v3ChangeLinBottom=C_v3ChangeLinBottom, C_v3ChangeSqrBottom=C_v3ChangeSqrBottom, &
!            C_TChange=C_TChange, C_rhoChangeTop=C_rhoChangeTop, &
!            C_Tsurf=C_Tsurf, C_TminLimit=C_TminLimit, C_TminRate=C_TminRate, &
!            C_Hptopfactor=C_Hptopfactor, &
!            rho_min=rho_min, LuminosityPerVolume=LuminosityPerVolume, &
!            hdCoreDragProfile=hdCoreDragProfile, hdCoreHeatProfile=hdCoreHeatProfile, &
!            C_CoreDrag=C_CoreDrag, &
!            B1_inflow=B1_inflow, C_magthetaB=C_magthetaB, C_magphiB=C_magphiB, &
!            N_magDiffSide=N_magDiffSide, N_magDiffTop=N_magDiffTop, &
!            N_magDiffBottom=N_magDiffBottom, &
!          abuX=abuX, abuY=abuY, abuZ=abuZ, Qmol=Qmol, gamma=gamma, &
!          EOSFile=EOSFile, EOSPath=EOSPath, &
!          kappaMin=kappaMin, kappaMax=kappaMax, OpaFile=OpaFile, OpaPath=OpaPath, &
!          chem_abumetal=chem_abumetal, chem_reacfile=chem_reacfile, chem_reacpath=chem_reacpath, &
!            hion_datapath=hion_datapath, hion_atomfile=hion_atomfile, hion_abufile=hion_abufile, &
!            hion_edensfile=hion_edensfile, hion_pffile=hion_pffile, &
!            dtime_out_hion=dtime_out_hion, hion_chunks=hion_chunks, &
!          hdScheme=hdScheme, hdTimeIntegrationScheme=hdTimeIntegrationScheme, &
!            hdsplit=hdsplit, hdTmpStructKeepMode=hdTmpStructKeepMode, hdCheckFlux=hdCheckFlux, &
!            reconstruction=reconstruction, &
!            hdTransVeloMode=hdTransVeloMode, hdEntropyWaveMode=hdEntropyWaveMode, &
!            hdEnthalpyAvgMode=hdEnthalpyAvgMode, &
!            N_hydIter=N_hydIter, N_hydMaxIter=N_hydMaxIter, N_hydRKIter=N_hydRKIter, &
!            C_hydTDiffLin=C_hydTDiffLin, C_hydTDiffMach=C_hydTDiffMach, &
!            C_hydsDiffVelo=C_hydsDiffVelo, C_hydvDiffVelo=C_hydvDiffVelo, &
!            C_hydLowMachcsFactor=C_hydLowMachcsFactor, C_hydLowMachcsBase=C_hydLowMachcsBase, &
!            C_hydSmallFluctFactor=C_hydSmallFluctFactor,&
!          N_hydCellsPerChunk=N_hydCellsPerChunk, N_hydCellsPerChunk2=N_hydCellsPerChunk2, &
!            N_hydCellsPerChunk3=N_hydCellsPerChunk3, &
!            N_visCellsPerChunk=N_visCellsPerChunk, N_radCellsPerChunk=N_radCellsPerChunk, &
!            aN_MPIProcsPerDim=aN_MPIProcsPerDim, &
!            C_SlopeRed=C_SlopeRed, &
!            C_RecContShift=C_RecContShift, C_RecContSteep=C_RecContSteep, &
!            C_RecWenoWeightCenter=C_RecWenoWeightCenter,C_RecWenoWeightRight=C_RecWenoWeightRight,&
!            C_RecWenoPsiFactor=C_RecWenoPsiFactor, &
!            C_hydPredfactor=C_hydPredfactor,&
!          C_visBound=C_visBound,&
!          beta_inv=beta_inv, va_max=va_max, &
!          dustScheme=dustScheme, dustReconstruction=dustReconstruction, dustOpaFile=dustOpaFile, &
!            dustopamode=dustopamode, &
!            n_dustGrainRadius=n_dustGrainRadius, ar_dustGrainRadius=ar_dustGrainRadius, &
!            n_dustGrainAbu=n_dustGrainAbu, ar_dustGrainAbu=ar_dustGrainAbu, &
!            C_dust01=C_dust01, C_dust02=C_dust02, &
!            C_dust03=C_dust03, C_dust04=C_dust04, C_dust05=C_dust05, C_dust06=C_dust06, &
!            C_dust07=C_dust07, C_dust08=C_dust08, C_dust09=C_dust09, C_dust10=C_dust10, &
!          radScheme=radScheme, radThinHeatScheme=radThinHeatScheme, &
!            radThinPressScheme=radThinPressScheme, &
!            N_radminiter=N_radminiter, N_raditer=N_raditer, N_radmaxiter=N_radmaxiter, &
!            N_radtaurefine=N_radtaurefine, &
!            N_radThinPoint=N_radThinPoint, N_radThinHeatPoint=N_radThinHeatPoint, &
!            N_radThinPressPoint=N_radThinPressPoint, N_radThickPoint=N_radThickPoint, &
!            N_radthetaN_radtheta, N_radphi=N_radphi, N_radsubray=N_radsubray, &
!            N_radband=N_radband, N_radrsyslevel=N_radrsyslevel, N_radoutput=N_radoutput, &
!            N_radFeautrier=N_radFeautrier, &
!            C_radimplicitmu=C_radimplicitmu, C_raditereps=C_raditereps, &
!            C_raditerstep=C_raditerstep, C_radkappasmooth=C_radkappasmooth, &
!            C_radTstep=C_radTstep, C_radTsmooth=C_radTsmooth, C_radTintminfac=C_radTintminfac, &
!            C_radHtautop=C_radHtautop, C_radTvisdtau=C_radTvisdtau, C_radTvis=C_radTvis, &
!            C_radthintimefac=C_radthintimefac, C_radthintimeRelax=C_radthintimeRelax, &
!            C_radthinTFactor=C_radthinTFactor, &
!            C_radtcool=C_radtcool, C_radscool=C_radscool, &
!            C_raddcool=C_raddcool, C_radTinci=C_radTinci, C_radDinci=C_radDinci, &
!            C_radAccEnergyFraction=C_radAccEnergyFraction, &
!            radraybase=radraybase, radraystar=radraystar, radpressure=radpressure, &
!          C_visLinear=C_visLinear, &
!            C_visSmagorinsky=C_visSmagorinsky, C_visArtificial=C_visArtificial, &
!            C_visExpansion=C_visExpansion, C_visPrturb=C_visPrturb, &
!            C_visTensordiag=C_visTensordiag, C_visTensoroff=C_visTensoroff, &
!            C_visTensordiv=C_visTensordiv, &
!            C_visP2Pcoeff=C_visP2Pcoeff, &
!            C_visP2Pincl1=C_visP2Pincl1, C_visP2Pincl2=C_visP2Pincl2, &
!            C_visP2PhypSmagorinsky=C_visP2PhypSmagorinsky, &
!            C_visP2PhypArtificial=C_visP2PhypArtificial, &
!            C_visP2PhypExpansion=C_visP2PhypExpansion, C_visP2Pdivrhov=C_visP2Pdivrhov, &
!            N_visIter=N_visIter, N_visP2PhypSmooth=N_visP2PhypSmooth, &
!            visP2Phypreconstruction=visP2Phypreconstruction, &
!            C_resB=C_resB, C_resBconstant=C_resBconstant, C_resEpsilon=C_resEpsilon, &
!          starttime=starttime, endtime=endtime, plustime=plustime, &
!            time_ScaleStart=time_ScaleStart, time_ScaleEnd=time_ScaleEnd, &
!            CPUtime=CPUtime, CPUtime_remainlimit=CPUtime_remainlimit, &
!            starttimestep=starttimestep, endtimestep=endtimestep, plustimestep=plustimestep, &
!          dtime_min=dtime_min, dtime_max=dtime_max, &
!            dtime_min_stop=dtime_min_stop, dtime_incmax=dtime_incmax, dtime_start=dtime_start, &
!          C_Courant=C_Courant, C_CourantMax=C_CourantMax, C_maxeiChange=C_maxeiChange, &
!          C_hydExpCourant=C_hydExpCourant, C_hydExpCourantMax=C_hydExpCourantMax, &
!            C_hydSoundCourant=C_hydSoundCourant, C_hydSoundCourantMax=C_hydSoundCourantMax, &
!            C_radCourant=C_radCourant, C_radCourantMax=C_radCourantMax, &
!            C_radmaxeiChange=C_radmaxeiChange, &
!            C_visCourant=C_visCourant, C_visCourantMax=C_visCourantMax, &
!          dtime_out_full=dtime_out_full, dtime_out_mean=dtime_out_mean, &
!            dtime_out_end=dtime_out_end, &
!            dtimestep_out_fine=dtimestep_out_fine, istep_in_start=istep_in_start, &
!          N_outslicedim_mean=N_outslicedim_mean, &
!          infile_start=infile_start, &
!          OutFile_end=OutFile_end, OutForm_end=OutForm_end, &
!            OutConv_end=OutConv_end, &
!            OutFile_full=OutFile_full, OutForm_full=OutForm_full, &
!            OutConv_full=OutConv_full, &
!            OutFile_mean=OutFile_mean, OutForm_mean=OutForm_mean, &
!            OutConv_mean=OutConv_mean, &
!            OutFile_fine=OutFile_fine, OutForm_fine=OutForm_fine, &
!            OutConv_fine=OutConv_fine, &
!            OutFile_mcyl=OutFile_mcyl, &
!            OutFile_chu1=OutFile_chu1, OutDim_chu1=OutDim_chu1, &
!            OutFile_chu2=OutFile_chu2, OutDim_chu2=OutDim_chu2, &
!            OutFile_chu3=OutFile_chu3, OutDim_chu3=OutDim_chu3, &
!            OutFile_out=OutFile_out, &
!          N_test1=N_test1, N_test2=N_test2, N_test3=N_test3, N_test4=N_test4, N_test5=N_test5, &
!            C_test1=C_test1, C_test2=C_test2, C_test3=C_test3, C_test4=C_test4, C_test5=C_test5, &
!            strtest1=strtest1, strtest2=strtest2, strtest3=strtest3, strtest4=strtest4, &
!            strtest5=strtest5, &
!          outstr=outstr,ierr=ierr)
!
! INPUT:
!   parFile:        ('parameter_File') character, name of parameter file
!   nc_p:           ('number_channel_print') integer, logical number for text output
!   n_dustGrainRadiusMax: ('number_dust_Grain_Radius_Maximum') maximum number of dust grain sizes
!
! OUTPUT:
!   ndes:           ('number_description') integer, number of lines of description
!   description:    ('description') character
!   nhis:           ('number_history') integer, number of lines of history
!   history:        ('history') character
!   model:          ('model') character, model name
!   counter:        ('counter') character, e.g. (#aa-bz#)
!   programversion: ('program_version') character to identify the program
!   Teff:           ('Temperature_effective') real, effective temperature [K]
!   heat_mode:      ('heating_mode') character
!                   '': empty:         (default) Try to derive heating mode from s_InFlow and
!                                      LuminosityPerVolume
!                   'bottom_entropy1': Adjust entropy in bottom layers (height given by r0_grav)
!                   'bottom_entropy2': Adjust entropy in bottom layers (height given by r0_grav)
!                                      with linear onset of heating and drag force
!                   'bottom_entropy2h': Adjust entropy in bottom layers (height given by r0_grav)
!                                       with linear onset of heating and drag force, homogenize
!                   'bottom_entropy3': Adjust entropy in bottom layers (height given by r0_grav)
!                                      with smooth onset of heating and drag force
!                   'bottom_energy1':  Add energy in bottom layers (height given by r0_grav)
!                   'core_energy1':  Add energy in core layers (radius given by r0_grav)
!                   'core_energy2':  Add energy in core layers (radius given by C_gravCorer0Heat)
!                   'core_energyentropy1': Adjust entropy towards mean entropy and add energy in
!                                          core layers (radius given by r0_grav)
!                   'core_entropy1': Adjust entropy in core layers (radius given by r0_grav)
!   grav_mode:      ('gravity_mode') character, mode of gravity field:
!                     'constant':     Standard solar setup with constant downward (x3) gravity
!                     'localboxtide': Experimental modification with tide forces
!                     'central':      Standard "star-in-a-box" setup with central potential
!   grav_type:      ('gravity_mode') character, type of gravity field for central potential:
!                     'classic2': Basic form of formula: 1/(r0_grav^2+r^2)^1/2
!                     'classic4': Basic form of formula: 1/(r0_grav^4+r^4)^1/4
!   grav:           ('gravity') real, gravity [cm/s^2], relevant if grav_mode='constant'
!   C_gravmu:       ('Constant_gravity_mu') real, parameter for time-centering of
!                   gravity source terms:
!                   <0.0: use default values (0.0 for RungeKuttaX and 0.5 for all other cases)
!                    0.0: use only values at old sub time step
!                    0.5: use time-centering for each sub time step
!                    1.0: use only values at new sub time step
!   C_gravCorePotDepth: ('Constant_gravity_Core_Potential_Depth') real, factor of
!                   potential deepening in core region for grav_type='classic4' central potential:
!                   <=1.0   : no effect
!                   2.0-4.0 : small sensible values
!                   4.0-10.0: large sensible values
!   C_gravCorePotQuad: ('Constant_gravity_Core_Potential_Quadratic') real, amplitude of
!                   quadratic term  in core region for grav_type='classic4' central potential;
!                   needs grav_type='classic4' and C_gravCorePotDepth>1.0. Values:
!                   <=0.0   : no effect
!                   0.25-0.5: sensible values
!   mass_star:      ('mass_star') real, total mass of star [g], relevant if grav_mode='central'
!   r0_grav:        ('radius_0_gravity') real, central smoothing radius [cm],
!                   relevant if grav_mode='central'
!   r1_grav:        ('radius_1_gravity') real, outer smoothing radius [cm],
!                   relevant if grav_mode='central'
!   r2_grav:        ('radius_2_gravity') real, outer smoothing radius (new central potential) [cm],
!                   relevant if grav_mode='central'
!   r1_rad:         ('radius_1_radiation') real, radius beyond which only positive temperature
!                   changes due to radiation are taken into account,
!                   relevant if grav_mode='central'
!   C_gravCorer0Heat: ('Constant_gravity_Core_radius_0_Heat') real, radius fraction of stellar core
!                   of heating and damping region for central boundary condition
!                   <=0.0    : use default (1.0)
!                   0.3<C<1.5: typical, likely sensible values
!                   1.0      : default
!   centrifugal_force: ('centrifugal_force') charater to switch on the centrifugal force in the
!                   gravity field ('on' default, 'off')
!   nu_rotation:    ('nu_rotation') real, rotational frequency [s^-1], default: 0.0
!   ar_RotationAxis: ('array_Rotation_Axis) 3-element real vector with rotation axis,
!                   default: (0.0, 0.0, 1.0), rotation around z axis
!   abuX:           ('abundance_X') real, hydrogen abundance (number fraction)
!   abuY:           ('abundance_Y') real, helium abundance (number fraction)
!   abuZ:           ('abundance_Z') real, metal abundance (number fraction)
!   Qmol:           ('Q_molecular') real, mean molecular weight [u]
!   gamma:          ('gamma') real, adiabatic coefficient (if constant) [1]
!   EOSFile:        ('Equation_Of_State_File') character, EOS file name
!   EOSPath:        ('Equation_Of_State_Path') character, path of EOS file
!   kappaMin:       ('kappa_Minimum') real, minimum value of opacity (< 0 => ignored)
!   kappaMax:       ('kappa_Maximum') real, maximum value of opacity (< 0 => ignored)
!   OpaFile:        ('Opacity_File') character, opacity file name
!   OpaPath:        ('Opacity_Path') character, path of opacity file
!   chem_abumetal:  ('chemistry_abundance_metal') real, abundance of representative metal for
!                   chemistry routines, (abuZ is the metal abundance in the EOS table)
!   chem_reacfile:  ('chemistry_reaction_file') character, file with chemical reaction network data
!   chem_reacpath:  ('chemistry_reaction_path') character, path to chemical reaction network data
!   hion_datapath:  ('HION data path') character, path of all data files for HION
!   hion_atomfile:  ('HION atom file name') character, data file name for model atom for HION
!   hion_abufile:   ('HION abundance file name') character, data file name for model abundance for
!                   HION
!   hion_edensfile: ('HION electron density file name') character, data file name for model
!                   electron density for HION
!   hion_pffile:    ('HION partition function file name') character, data file name for model
!                   partition function for HION
!   dtime_out_hion: ('delta_time_output_hion') real, number of seconds between
!                   two HION output times
!   hion_chunks:    ('hion_n_chunks') integer, number of computational chunks for HION
!   side_bound:     ('side_boundary') character, side boundary condition:
!                   'reflective'         : closed wall, no gravity, no radiation
!                   'constant'           : open boundary with constant extrapolation of all values,
!                                          no gravity, no radiation
!                   'closed', 'closedtop': closed wall, incl. gravity, open for outward radiation
!                   'closed2', 'closedtop2': closed wall, incl. gravity, open for outward radiation
!                   'closedbottom'       : closed wall, incl. gravity, radiation in diff. approx.
!                   'periodic'           : periodic boundaries for hydrodynamics and radiation
!                   'transmitting'       : transmitting boundary for hydro and outward radiation
!   top_bound:      ('top_boundary') character, top boundary condition:
!                   'reflective'         : closed wall, no gravity, no radiation
!                   'constant'           : open boundary with constant extrapolation of all values,
!                                          no gravity, no radiation
!                   'closed', 'closedtop': closed wall, incl. gravity, open for outward radiation
!                   'closed2', 'closedtop2': closed wall, incl. gravity, open for outward radiation
!                   'periodic'           : periodic boundaries for hydrodynamics and radiation
!                   'transmitting'       : transmitting boundary for hydro and outward radiation
!                   'transmitting2'      : transmitting boundary for hydro and outward radiation,
!                                          version 2
!                   'transmitting3'      : transmitting boundary for MHD and outward radiation,
!                                          version 3
!   bottom_bound:   ('bottom_boundary') character, bottom boundary condition:
!                   'reflective'         : closed wall, no gravity, no radiation
!                   'constant'           : open boundary with constant extrapolation of all values,
!                                          no gravity, no radiation
!                   'closed', 'closedtop': closed wall, incl. gravity, open for outward radiation
!                   'closed2', 'closedtop2': closed wall, incl. gravity, open for outward radiation
!                   'closedbottom'       : closed wall, incl. gravity, radiation in diff. approx.
!                   'closedbottom2'      : closed wall, incl. gravity, radiation in diff. approx.
!                   'periodic'           : periodic boundaries for hydrodynamics and radiation
!                   'transmitting'       : transmitting boundary for hydro and outward radiation
!                   'inoutflow'          : "classical" open lower boundary for deep convection,
!                                          gravity and radiation possible
!                   'inoutflow2'         : modified open lower boundary for deep convection,
!                                          gravity and radiation possible
!                   'inoutflow3'         : modified open lower boundary for deep convection,
!                                          gravity and radiation possible
!   side_bound_mag_x1: ('side_boundary_magnetic_x1') character, side boundary condition for
!                   magnetic field along x-axis:
!                   'constant'           : constant extrapolation of magnetic field
!                   'periodic'           : periodic boundaries for the magnetic field
!   side_bound_mag_x2: ('side_boundary_magnetic_x2') character, side boundary condition for
!                   magnetic field along y-axis:
!                   'constant'           : constant extrapolation of magnetic field
!                   'periodic'           : periodic boundaries for the magnetic field
!   top_bound_mag:  ('top_boundary_magnetic') character, top boundary condition:
!                   'constant'           : constant extrapolation of magnetic field
!                   'periodic'           : periodic boundaries for the magnetic field
!                   'vertical'           : vertical magnetic field at the boundary
!                   'vertical2'          : vertical magnetic field at the boundary (less aggressive)
!   bottom_bound_mag: ('bottom_boundary_magnetic') character, bottom boundary condition:
!                   'constant'           : constant extrapolation of magnetic field
!                   'fixed'              :
!                   'inoutflow'          : advection of magnetic field through an open boundary
!                   'oblique'            : oblique fiels possible
!                   'periodic'           : periodic boundaries for the magnetic field
!                   'vertical'           : vertical magnetic field at the boundary
!                   'vertical2'          : vertical magnetic field at the boundary (less aggressive)
!   s_InFlow:       ('s_In_Flow') real, entropy of material, flowing into the box through bottom
!                   or through the core, [erg/K/g],
!                   if bottom_bound='inoutflow' or heat_mode == 'core_entropy1'
!   s_InFlRadGrad:  ('s_In_Flow_Radial_Gradient') real, radial entropy gradient in core,
!                   default=0.0, [erg/K/g], if heat_mode == 'core_entropy1'
!   s_InFlLatGrad:  ('s_In_Flow_Lateral_Gradient') real, entropy gradient with latitude in core,
!                   default=0.0, [erg/K/g], if heat_mode == 'core_entropy1'
!   C_sChange:      ('Constant_s_Change') real, rate of entropy change at bottom, typically 0.5
!   C_PChange:      ('Constant_Pressure_Change') real, rate of pressure change at bottom,
!                   typically 0.5
!   C_v3ChangeLinBottom: ('Constant_velocity_3_Change_Linear_Bottom') real, rate of vertical
!                   velocity change at open bottom boundary, linear
!   C_v3ChangeSqrBottom: ('Constant_velocity_3_Change_Square_Bottom') real, rate of vertical
!                   velocity change at open bottom boundary, quadratic in v3
!   C_TChange:      ('Constant_Temperature_Change') real, rate of temperature change at top,
!                   typically 0.5
!   C_rhoChangeTop: ('Constant_rho_Change_Top') real, rate of density change at top
!   C_Tsurf:        ('Constant_Temperature_Surface') real, factor between effective temperature
!                   and surface temperature (for transmitting boundary condition), typically=0.81
!   C_TminLimit:    ('Constant_Temperature_minimum_Limit') real, minimum temperature enforced in
!                   MSCrad3 by adding an artificial energy source term (non-conservative)
!   C_Hptopfactor:  ('Constant_Height_pressure_top_factor') real, factor for modification of
!                   local pressure scale height at top (or outer) boundaries:
!                     Hp_actual=Hp/C_Hptopfactor
!                   This scale height is used to extrapolate the density for 'transmitting'
!                   boundary condition.
!                   Values:
!                            C < 0.0: No effect (actually, a value of 1.0 is chosen)
!                     0.0 <= C < 1.0: The pressure scale height is enlarged to account for
!                                     possible effects of turbulent pressure on scale height
!                            C = 1.0: No effect, hydrostatic pressure scale height is used
!                            C > 1.0: Scale height is decreased, not really useful
!   rho_min:        ('rho_minimum') real, minimum density ('transmitting' boundary condition)
!                      0.0: Ignore limits on density
!                     >0.0: Density in outermost layer(s) is set to at least this value
!   LuminosityPerVolume: ('Luminosity_Per_Volume') real, specify energy production rate as
!                   luminosity per volume for 'central' gravity fields
!   hdCoreHeatProfile: ('hydrodynamics_Core_Heat_Profile') character with radial profile
!                   of core heating:
!                   'Constant', 'Constantdei': Constant change of ei, default
!                   'Constantdrhoei':          Constant change of rho*ei
!   hdCoreDragProfile: ('hydrodynamics_Core_Drag_Profile') character with radial profile
!                   of core drag force:
!                   'Constant':      default
!                   'Linear':        linear decay from center to outer boundary of core
!                   'Cosine':        cosine-like decay from center to outer boundary of core
!                   'CosSqr':        cosine^2-like decay from center to outer boundary of core
!                   'Constant-Radial': constant decay acting on radial velocity component only
!                   'Linear-Radial': linear decay acting on radial velocity component only
!                   'Cosine-Radial': cosine-like decay acting on radial velocity component only
!                   'CosSqr-Radial': cosine^2-like decay acting on radial velocity component only
!                   'Constant-Meridional': constant decay acting on meridional velocity comp. only
!                   'Linear-Meridional': linear decay acting on meridional velocity component only
!                   'Cosine-Meridional': cosine-like decay acting on meridional velocity comp. only
!                   'CosSqr-Meridional': cosine^2-like decay acting on meridional velocity
!                                        components only
!                   'Constant-All-vTrans': constant decay acting on all velocity components
!                                          parallel to the transversal core velocity
!                   'Constant-Radial-vTrans': constant decay acting only on radial velocity
!                                             component parallel to the transversal core velocity
!   C_CoreDrag:     ('Constant_Core_Drag') real, parameter for drag force acting in core
!   B1_inflow:      ('B1_inflow') real, strength of inflowing horizontal magnetic field (B1)
!                   at lower boundary (mag_inoutflow)
!   C_magthetaB:    ('Constant_magnetic_theta_B') real, angle between magnetic field and
!                   z-axis (x3-axis) in ghost cells, for boundary condition 'mag_oblique'
!   C_magphiB:      ('Constant_magnetic_phi_B') real, angle between horizontal component of
!                   magnetic field and x-axis in ghost cells, for boundary condition "mag_oblique"
!   N_magDiffSide:  ('Number_magnetic_Diffusion_Sidecells') integer, number of cells for
!                   dissipative layers at side boundaries
!   N_magDiffTop:   ('Number_magnetic_Diffusion_Topcells') integer, number of cells for
!                   dissipative layers at top boundary
!   N_magDiffBottom: ('Number_magnetic_Diffusion_Bottomcells') integer, number of cells for
!                   dissipative layers at bottom boundary
!   hdScheme:       ('hydrodynamics_Scheme') character, hydrodynamics scheme:
!                   'Roe'      : Standard solver for pure hydrodynamics (without magnetic fields)
!                   'RoeMagKin': Test version of kinetic MHD with standard Roe solver, not working
!                   'RoeMHD'   : Test version for MHD
!                   'HLLMHD'   : Standard solver for magneto-hydrodynamics
!                   'CenMHD'   : Slightly modified HLLMHD solver for magneto-hydrodynamics
!                   'None'     : Skip HD or MHD solver entirely
!   hdTimeIntegrationScheme: ('hydrodynamics_Time-Integration_Scheme') character, time-integration
!                   method for MHD scheme
!                   'Single'     : Standard time integration for HD solver
!                   'Hancock'    : Standard time integration for MHD solver
!                   'Euler1'     : 1st-order (unstable) scheme, testing and demo only
!                   'RungeKutta2': 2nd-order Runge-Kutta scheme (needs hdsplit='unsplit')
!                   'RungeKutta3': 3rd-order Runge-Kutta scheme (needs hdsplit='unsplit')
!   hdsplit:        ('hydrodynamics_split') character, hydrodynamics splitting scheme:
!                   '123'    : Directional splitting (default)
!                   'unsplit': Simple unsplit
!                   'CTU'    : Collela's 'Corner Transport Upwind' scheme (1990)
!   hdTmpStructKeepMode: ('hydrodynamics_Temporary_Structures_Keep_Mode') character indicating
!                   if temporary structures should be kept or deleted between calls. Values:
!                     'Keep'  : to improve performance: the new default
!                     'Delete': to save memory: the old mode
!   hdCheckFlux:    ('hydrodynamics_Check_Flux') charater to switch on the CheckFlux routine
!                   in the MHD module ('on' default, 'off')
!   reconstruction: ('reconstruction) character, reconstruction method for HD/MHD scheme
!                   'Constant': Constant reconstruction, 'donor cell' advection, extremely diffusive
!                   'Minmod'  : MinMod slope limiter, stable but rather diffusive scheme
!                   'VanLeer' : van Leer slope limiter, most-often-used scheme
!                   'Superbee': SuperBee slope limiter, steepening by anti-diffusion, testing only
!                   'PP'      : 2nd-order monotonic reconstruction from PPM scheme, low diffusivity
!                   'PPmimo'  : MinMod reconstruction based on boundary values from PP
!                   'FRmimo'  : MinMod reconstruction based on boundary values from FRmono
!                   'FRmono'  : 2nd-order monotonic reconstruction derived from PP, WENO, and MinMod
!                   'FRmono2' : Same as FRmono, only with C_SlopeRed the limit for strong jumps
!                               is Constant and not MinMod
!                   'FRweno'  : 2nd-order non-monotonic version of FR (recommended)
!                   'FRweno2' : Same as FRweno, only with C_SlopeRed the limit for strong jumps
!                               is Constant and not MinMod
!                   'FRcont'  : 2nd-order continuous version of FR, unstable except for low Mach no.
!                   'VaWeno'  : 2nd-order vanilla WENO
!                   'HBweno'  : 2nd-order WENO with handbrake
!   hdTransVeloMode: ('hydrodynamics_Transverse_Velocity_Mode') character, treatment of advection of
!                   transverse velocities
!                   'Normal',' ': flux of transversal momentum with standard Roe treatment (default)
!                   'CA1':        flux of transversal momentum with consistent advection, version 1
!                   'CA2':        flux of transversal momentum with consistent advection, version 2
!                   'CA3':        flux of transversal momentum with consistent advection, version 3
!                   'CA4':        flux of transversal momentum with consistent advection, version 4
!                                 (recommended)
!   hdEntropyWaveMode: ('hydrodynamics_Entropy_Wave_Mode') character, treatment of entropy wave
!                   'Normal', ' ': Entropy wave(s) with standard Roe treatment (default)
!                   'Entropy1'   : Reconstruct entropy and compute flux_3 and flux_6 from it,
!                                  only useful for small Mach numbers
!                   'Entropy2'   : Compute entropy-wave contribution from entropy and pressure
!                   'Entropy3M'  : Compute entropy-wave contribution from entropy, density, and
!                                  pressure; interpolate with Mach number
!                   'Entropy3Ms' : Compute entropy-wave contribution from entropy, density, and
!                                  pressure; interpolate with Mach number and entropy step
!                   'Entropy3gMps':Compute entropy-wave contribution from entropy, density, and
!                                  pressure; interpolate with (direct and reduced) pressure step,
!                                  Mach number, and entropy step (recommended)
!   hdEnthalpyAvgMode: ('hydrodynamics_Enthalpy_Average_Mode') character, mode of enthalpy averaging
!                   for enthalph flux:
!                   'Normal', ' ': Standard Roe treatment (default)
!                   'CA1'        : Use consistent-advection centering for average enthalpy flux
!                   'CA1M'       : Interpolate between consistent-advection and Roe averaging
!                                  with Mach number
!                   'CA1s'       : Interpolate between consistent-advection and Roe averaging
!                                  with entropy jump size
!                   'CA1Ms'      : Interpolate between consistent-advection and Roe averaging
!                                  with Mach number and entropy jump size
!                   'CA1p'       : Interpolate between consistent-advection and Roe averaging
!                                  with pressure jump size
!                   'CA1g'       : Interpolate between consistent-advection and Roe averaging
!                                  with pressure jump size (after hydrostatic pressure correction)
!                   'CA1gMps'    : Interpolate between consistent-advection and Roe averaging
!                                  with pressure jump size (directly and after
!                                  hydrostatic pressure correction),
!                                  Mach number and entropy jump size (recommended)
!                   'CA2'        : Use consistent-advection centering for avg. enthalpy+kinetic flux
!                   'CA2M'       : Interpolate between consistent-advection and Roe averaging
!                                  with Mach number
!                   'CA2s'       : Interpolate between consistent-advection and Roe averaging
!                                  with entropy jump size
!                   'CA2p'       : Interpolate between consistent-advection and Roe averaging
!                                  with pressure jump size
!                   'CA2g'       : Interpolate between consistent-advection and Roe averaging
!                                  with pressure jump size (after hydrostatic pressure correction)
!                   'CA3'        : Use consistent-advection centering for average enthalpy flux,
!                                  use eip_til (and ek_til) for all averaged energies
!                   'CA4'        : Use consistent-advection centering for average enthalpy flux;
!                                  another version
!   N_orderConstrainedTransport: ('Number_order_Constrained_Transport') integer, order of
!                   reconstruction of the electric field in constrained Transport step
!                   1: Simple arithmetic average of fluxes (default)
!                   2: Quadratic interpolation of fluxes to the cell edges
!   N_hydIter:      ('Number_hydrodynamics_Iteration') integer, number of iterations of
!                   hydrodynamics substep.
!                   If more iterations are needed the time step dtime_limit for
!                   the next step is decreased.
!   N_hydMaxIter:   ('Number_hydrodynamics_Maximum_Iteration') integer, absolute maximum number of
!                   iterations of hydrodynamics substep.
!                   If more iterations are needed the computation for the current
!                   time step is stopped and resumed with a smaller one.
!                   If set to zero no upper limit for the iteration number is imposed.
!                   Generally: N_hydIter < N_hydMaxIter
!   N_hydRKIter:    ('Number_hydrodynamics_Runge-Kutta_Iteration') integer, number of
!                   Rungek-Kutta substeps within hydrodynamics substep.
!                   It has a meaning only for the Ros solver
!                   with hdTimeIntegrationScheme='RungeKutta2'.
!                   <=2: Use the default (2): standard predictor-corrector scheme
!                   2<N<=11: Use more sub steps which does not increase the order of the scheme
!                            but the actual time step.
!                   Examples:
!                   2: cost: 2; advance 1; C_hydSoundCourantMax=1/3            =0.333
!                   3: cost: 3; advance 2; C_hydSoundCourantMax=1/3*(3/2)^(1/3)=0.381
!                   4: cost: 4; advance 3; C_hydSoundCourantMax=1/3            =0.333
!                   5: cost: 5; advance 4; C_hydSoundCourantMax=1/3*(5/4)^(1/3)=0.358
!   C_hydTDiffLin:  ('Constant_hydrodynamics_Temperature_Diffusion_Linear) real, energy diffusion
!                   near local temperature minimum, 1
!   C_hydTDiffMach: ('Constant_hydrodynamics_Temperature_Diffusion_Mach) real, energy diffusion
!                   near local temperature minimum, 2
!   C_hydsDiffVelo: ('Constant_hydrodynamics_s_Diffusion_Velocity) real, energy diffusion
!                   down the entropy gradient (i.e., down the step after reconstruction)
!   C_hydvDiffVelo: ('Constant_hydrodynamics_velocity_Diffusion_Velocity) real, momentum diffusion
!                   down transversal velocity steps (i.e., down the step after reconstruction)
!   C_hydLowMachcsFactor: ('Constant_hydrodynamics_Low_Mach-number_c_sound_Factor), real
!                   with factor used in low-Mach-number sound-speed reduction:
!                   <=0.0: Ignore reduction and use old (std. Roe-solver) settings
!                   > 0.0: Artifial sound-speed reduction, reasonable value: 1.0, 2.0, 5.0
!   C_hydLowMachcsBase: ('Constant_hydrodynamics_Low_Mach-number_c_sound_Base), real
!                   with minimum sound-speed fraction:
!                   <1.0E-06: dangerously small values
!                   0.001-0.01: sensible values
!                   1.0: switch off the low-Mach-number sound-speed reduction
!   C_hydSmallFluctFactor: ('Constant_hydrodynamics_Small_Fluctuations_Factor') real
!                   with factor used in computation of fluctuation-size estimator:
!                   <1.0: extend the small-fluctuation treatment towards larger fluctuations,
!                         can result in nicer entropy profile but might be unstable,
!                    1.0: default
!                   >1.0: extend the large-fluctuation treatment towards smaller fluctuations,
!                         improves stability but leads to artefacts in the entropy stratification
!   N_hydCellsPerChunk: ('Number_hydrodynamics_Cells_Per_Chunk') integer, number of cells per box
!                   treated by hydrodynamics scheme at one call,
!                   used to improve cache efficiency,
!                   default: 0 => use one slice at a time
!                   typical values: 1200
!                   older recommendation: 10000 (smaller on cache-limited machines,
!                                                larger for vector computers)
!   N_hydCellsPerChunk2: ('Number_hydrodynamics_Cells_Per_Chunk_2') integer, number of cells per box
!                   in x2 direction,
!                   used to improve cache efficiency and to reduce page misses.
!                   A value larger than N_hydCellsPerChunk can be beneficial.
!                   A value smaller than N_hydCellsPerChunk/2 means that actually
!                   N_hydCellsPerChunk2 number of slices are used for the first-level partitioning
!                   (with OpenMP) and N_hydCellsPerChunk for the second level (without OpenMP)
!                   default: <=0 => N_hydCellsPerChunk2=N_hydCellsPerChunk
!                   2*N_hydCellsPerChunk2 <= N_hydCellsPerChunk: perform 2-levels-of-chunks
!                   partitioning
!                   typical values: 4800 or 1
!   N_hydCellsPerChunk3: ('Number_hydrodynamics_Cells_Per_Chunk_3') integer, number of cells per box
!                   in x3 direction,
!                   used to improve cache efficiency and to reduce page misses.
!                   A value larger than N_hydCellsPerChunk can be beneficial.
!                   A value smaller than N_hydCellsPerChunk/2 means that actually
!                   N_hydCellsPerChunk3 number of slices are used for the first-level partitioning
!                   (with OpenMP) and N_hydCellsPerChunk for the second level (without OpenMP)
!                   default: <=0 => N_hydCellsPerChunk3=N_hydCellsPerChunk
!                   2*N_hydCellsPerChunk3 <= N_hydCellsPerChunk: perform 2-levels-of-chunks
!                   partitioning
!                   typical values: 4800 or 1
!   N_visCellsPerChunk: ('Number_viscosity_Cells_Per_Chunk') integer, number of cells per box
!                   treated by 3D viscosity scheme at one call,
!                   used to improve cache efficiency,
!                   default: 0 => use one slice at a time
!                   typical value: 100000
!   N_resCellsPerChunk: ('Number_resistivity_Cells_Per_Chunk') integer, number of cells per box
!                   treated by 3D resistivity scheme at one call,
!                   used to improve cache efficiency,
!                   default: 0 => use one slice at a time
!                   typical value: 10000 (smaller on cache limited machines,
!                                         larger for vector computers)
!   N_radCellsPerChunk: ('Number_radiation_Cells_Per_Chunk') integer, number of cells per box
!                   treated by radiation transport scheme at one call,
!                   used to improve cache efficiency,
!                   default: 0 => use one slice at a time
!                   typical value: 10000 (smaller on cache limited machines,
!                                         larger for vector computers)
!   aN_MPIProcsPerDim: ('array_Number_MPI_Processes_Per_Dimension') 3-element integer array with
!                   number of MPI processes per dimension
!   C_SlopeRed:     ('Constant_Slope_Reduction') real, parameter for slope reduction in case
!                   of strong density contrast (default: 0.0: off; reasonable value: 0.02)
!   C_RecContShift: ('Constant_Reconstruction_Continuous_Shift') real, parameter 1 to force more
!                   continuous recontruction. This parameter shifts the function in log(Mach)
!                   that controls the interpolation between normal and the continuous
!                   reconstruction.
!                   A small value means to keep essentially the normal reconstruction with steps:
!                     0.0: off (default)
!                     0.01-1.0: reasonable values
!                     1.0E+06: completely continuous reconstruction (set C_RecContSteep=1.0E+06,too)
!   C_RecContSteep: ('Constant_Reconstruction_Continuous_Steepen') real, parameter 2 to force more
!                   continuous recontruction. A change in this parameter does not change the
!                   interpolation function for very small Mach numbers but steepens the transition
!                   between high- and low-Mach-number regime.
!                   A small value means to keep essentially the normal reconstruction with steps:
!                     0.0: off (default)
!                     0.1-10.0: reasonable values
!                     1.0E+06: completely continuous reconstruction (set C_RecContShift=1.0E+06,too)
!  C_RecWenoWeightCenter: ('Constant_Reconstruction_WENO_Weight_Center') real with weight of
!                   center polynomial. Usually 0.5 to 0.8.
!  C_RecWenoWeightRight: ('Constant_Reconstruction_WENO_Weight_Right') real with relative weight of
!                   right polynomial. Usually 0.5 to 0.8.
!                   The weight of the center polynomial is
!                     wC=min(max(C_RecWenoWeightCenter, 0.0), 1.0))
!                   The weight of the left polynomial is
!                     wR=(1.0-wC) * min(max(C_RecWenoWeightRight, 0.0), 1.0)
!                   The weight of the left polynomial is then
!                     wL=1.0-wC-wR
!  C_RecWenoPsiFactor: ('Constant_Reconstruction_WENO_Pairwise_smoothness_indicator_Factor') real
!                   with factor that determines the additional smoothing if the pairwise smoothness
!                   of the candidate polynomials is bad. Typical values:
!                    0.0: No additional smoothing. Avoid except for tests!
!                         Use reconstruction=VAweno, instead!
!                    1.0: Weak smoothing
!                    4.0: Reasonable value
!                   16.0: Strong smoothing
!   C_hydPredfactor: ('Constant_hydrodynamics_Pressure_reduction_factor') real, parameter to
!                   control "hydrostatic pressure reduction" in waves 3 and "6" in Roe solver,
!                   0.0: Deactivation of pressure reduction terms
!                   1.0: Activation of pressure reduction terms (default)
!                   Other values than 0.0 or 1.0 are generally not reasonable.
!   C_visBound:     ('Constant_viscosity_boundary') real, parameter for additional drag viscosity
!                   at boundary, typically 0.01
!   beta_inv:       ('beta_inverse') real, inverse plasma beta for dual energy method
!   va_max:         ('va_maximum') real, maximum Alfven speed to avoid very small time steps
!   dustScheme:     ('dust_Scheme') character, dust treatment scheme:
!                   e.g. ' ', 'none', 'nosource',
!                        'dust_simple_01', 'co_component01_01', 'dust_moment04_01'
!   dustReconstruction: ('dust_Reconstruction) character, reconstruction method for additional
!                   quantities in HD/MHD scheme
!   dustOpaFile:    ('dust_opacity_file') name of opacity file
!   dustopamode:    ('dust_opacity_mode') string with identifier for dust opacity calculation
!   n_dustGrainRadius: ('number_dust_Grain_Radius') integer with size of array ar_dust_Grain_Radius
!   ar_dustGrainRadius: ('array_real_dust_Grain_Radius') real array with dust grain radii
!   n_dustGrainAbu: ('number_dust_Grain_Abundance') integer with size of
!                   array ar_dust_Grain_Abundance
!   ar_dustGrainAbu: ('array_real_dust_Grain_Abundance') real array with dust grain abundances
!   C_dust01,02,..,10: (Constant_dust_XX'), real parameter to control the dust model
!   radScheme:      ('radiation_Scheme') character, radiation transport scheme:
!                   'None'    : No radiation-transport step (default)
!                   'LHDrad'  : Use long-characterstics scheme for global models (not recommended)
!                   'LCFrad'  : Use long-characteristics scheme for local models (previously MSrad)
!                   'MSCrad'  : Use multiple-short-characteristics scheme for local models
!                   'SHORTrad': Use short-characterstics scheme for global models
!   radThinHeatScheme: ('radiation_Thin_Heat_Scheme') character, scheme for local radiative heating
!                   or cooling:
!                   'MinLimitT': Adjust internal energy to limit the temperature from below
!                                given by C_TminLimit
!                   'ConstantT': Adjust internal energy towards prescribed constant temperature
!                                given by Teff*C_Tsurf
!                   'ConstantLum': Adjust internal energy towards prescribed temperature
!                                  falling off with r^(-1/2)
!   radThinPressScheme: ('radiation_Thin_Pressure_Scheme') character, scheme for local radiative
!                   acceleration
!   N_radminiter:   ('Number_radiation_minimum_iteration') integer, minimum number of iterations of
!                   implicit radiative transport substep.
!                   If less iterations are needed for convergence the time step dtime_limit for
!                   the next step is increased.
!   N_raditer:      ('Number_radiation_iteration') integer, number of iterations of
!                   radiative transport substep (explicit case).
!                   Maximum number of iterations of implicit radiative transport substep.
!                   If more iterations are needed for convergence the time step dtime_limit for
!                   the next step is decreased.
!   N_radmaxiter:   ('Number_radiation_maximum_iteration') integer, absolute maximum number of
!                   iterations of implicit radiative transport substep.
!                   If more iterations are needed for convergence the computation for the current
!                   time step is stopped and resumed with a smaller one.
!                   Generally: N_radminiter < N_raditer < N_radmaxiter
!   N_radtaurefine: ('Number_radiation_tau_refine') integer, odd grid refinement factor at large
!                   temperature steps:
!                     1: No refinement
!                     3: reasonable value
!   N_radThinPoint: ('Number_radiation_Thin_Point') integer, number of grid points with
!                   optically thin approximation,
!                   default: 0
!   N_radThinHeatPoint: ('Number_radiation_Thin_Heating_Point') integer, number of additional
!                   grid points with overlapping optically thin and thick approximation for heating;
!                   SHORTrad only;
!                   default: 0
!   N_radThinPressPoint: ('Number_radiation_Thin_Pressure_Point') integer, number of additional
!                   grid points with overlapping optically thin and thick approximation for
!                   radiation pressure; SHORTrad only;
!                   default: 0
!   N_radThickPoint: ('Number_radiation_Thick_Point') integer, number of grid points with
!                   optically thick approximation (diffusion approximation), default: 0
!   N_radtheta:     ('Number_radiation_theta) integer, number of ray sets in theta direction,
!                   typically 2
!   N_radphi:       ('Number_radiation_phi') integer, number of ray sets in phi direction,
!                   typically 2
!   N_radsubray:    ('Number_radiation_sub_ray') integer, number of rays per cell per direction,
!                   typically 2
!   N_radband:      ('Number_radiation_band') integer, number of frequency band scheme
!   N_radrsyslevel: ('Number_radiation_ray_system_level') integer, "zero" point of ray system,
!                   default: 0 (LCFrad only)
!   N_radoutput:    ('Number_radiation_output') integer, controls additional output into 'rhd.qrad',
!                   default: 0 (LCFrad and MSCrad only)
!   N_radFeautrier: ('Number_radiation_Feautrier') integer, control the Feautrier scheme in LCFrad
!                     -1: Feautrier RT on optical depth scale (linear tau-integration, testing)
!                      0: Feautrier RT on optical depth scale ( cubic tau-integration, default)
!                      1: Feautrier RT on geometrical scale   (linear delta tau)
!   C_radimplicitmu: ('Constant_radiation_implicit_mu') real (old: FIMPLC), interpolation parameter
!                   between radiation field at old and new time step (0.0 - 1.0):
!                   =0.0: explicit
!                   >0.0: implicit
!                         0.5: time centered
!                         1.0: fully implicit
!   C_raditereps:   ('Constant_radiation_iteration_epsilon') real, (old: DLIMIT), relative
!                   accuracy of iteration, typically 1.0E-05
!   C_raditerstep:  ('Constant_radiation_iteration_step') real, (old: SRGAM (similar)), fraction
!                   of the computed step that actually is used (0.0 < C <= 1.0):
!                   1.0: full step
!                   0.8: good value
!                   0.2: very cautious stepsize
!   C_radkappasmooth: ('Constant_radiation_kappa_smooth') real, parameter for spatial smoothing of
!                     opacity:
!                     0.0:   no smoothing
!                     0.25:  light smoothing (corresponding to linear interpolation)
!                     0.666: strong smoothing
!                     1.0:   maximum smoothing (perhaps not very reasonable)
!   C_radTstep:     ('Constant_radiation_temperature_step') real, parameter for temperature
!                   interpolation in case of a too strong temperature jump:
!                     0.0: No interpolation
!                     8.0: reasonable value
!   C_radTsmooth:   ('Constant_radiation_Temperature_smooth') real, parameter for spatial
!                   smoothing of temperature:
!                   0.0:   no smoothing
!                   0.5:   reasonable smoothing: T(x) becomes the same weight as the mean of
!                          all 6 neighbor cells
!                   1.0:   maximum smoothing (perhaps not very reasonable)
!   C_radTintminfac: ('Constant_radiation_Temperature_interpolation_minimum_factor') real,
!                   fraction the interpolated temperature may exceed the minimum temperature
!   C_radhtautop:   ('Constant_radiation_height_tau_top') real, scale height of optical depth
!                   at top.
!                     Htau0 >= 0.0: nothing changes with respect to previous versions, i.e.
!                                   tau_0(x,y) =  Htau0 *  kappa(x,y,z_top)
!                     Htau0 < 0.0:  tau_0(x,y) = |Htau0| * kappa(x,y,z_top) * Hp0(x,y,z_top)),
!                                   where Hp0(x,y,z_top) = P(x,y,z_top)/rho(x,y,z_top)/grav
!                                   is the local pressure scale height at the upper boundary.
!                                   Htau0 = -1.0 may be a good choice.
!   C_radTvisdtau:  ('Constant_radiation_Temperature_viscosity_delta_tau') real,
!                   limit in delta tau (rho*kappa*dx) below which the
!                   'radiative temperature viscosity' (=temperature smoothing) is to be applied:
!                   <=0.0: no extra 'viscosity'
!                   0.08:  reasonable value
!   C_radTvis:      ('Constant_radiation_Temperature_viscosity') real, amount of extra
!                   'radiative temperature viscosity':
!                   <=0.0: no extra 'viscosity'
!                   1.6:   reasonable value
!   C_radthintimefac: ('Constant_radiation_thin_time_factor') real, factor by which time scale due
!                   to radiative energy exchange is reduced in optically thin, for LHDrad only
!                   <=0.0: switched off, factor=1.0 assumed
!                   0.5:   reasonable value
!                   1.0:   no effect (same time scale as in optically thick)
!   C_radthintimeRelax: ('Constant_radiation_thin_time_Relaxation') real, relaxation time scale for
!                   optically thin radiation transport (LocalSource in SHORTrad).
!                   <=0.0: switched off, no change of internal energy by optically thin radiation
!                   10000.0: typical time scale for AGB models
!   C_radthinTFactor: ('Constant_radiation_thin_Temperature_Factor') real, factor applied to
!                   temperature in optically thin, a bit similar to C_Tsurf:
!                   1.0      : default
!                   0.0<.<2.0: Reasonable values (rather < 1.0 than > 1.0)
!   C_radTinci:     ('Constant_radiation_Temperature_incident') real, temperature of black body
!                   that emits incident radiation, default: 0.0, (LCFrad and MSCrad only)
!   C_radDinci:     ('Constant_radiation_Dilutionfactor_incident') real, distance factor (R/D)^2
!                   of spherical black body, default: 0.0, (LCFrad and MSCrad only)
!   C_radAccEnergyFraction: ('Constand_radiation_Accumulated_Energy_Fraction') real with
!                   fraction of the current radiative energy change that is put into the
!                   pool of accumulated energy (SHORTrad only):
!                      0.0: No energy change is accumulated; classical behavior (default)
!                     >0.0: Some energy change is accumulated
!                      0.1: 10% of the current energy change is kept (sensible value)
!                      1.0: The total current energy change is kept (absolute maximum)
!   radraybase:     ('radiation_ray_base') character to describe the orientation of the
!                   base axis system (for SHORTrad):
!                     'unity'    : Unity matrix
!                     'random'   : Matrix with random orientation
!                     'randomgroup': At each new time step a new base axis system is
!                                    chosen at random. It is kept for all radiative sub-steps.
!                     'alternate': Alternate between unity and random-orientation matrix
!                     'drift01'  : Slow drift, algorithm 1
!                     'drift02'  : Slow drift, algorithm 2
!                   For LCFrad and MSCrad, the parameter describes the ray directions, i.e., what
!                   radraystar is meant to control:
!                     'lobatto':
!                     'dblgaus':
!                     'dblrdau': recommended
!   radraystar:     ('radiation_ray_star') character to describe the list of the ray directions
!                   (e.g. the number of rays) relative to the base axis system:
!                     'x1'        : Only rays along x1 axis (1 direction)
!                     'x2'        : Only rays along x2 axis (1 direction)
!                     'x3'        : Only rays along x3 axis (1 direction)
!                     'oktaeder'  : Oktaeder: rays along x1, x2, and x3 axis (3 directions)
!                     'tetraeder' : Tetraeder: rays in 4 directions
!                     'cube'      : Cube: rays in 4 directions
!                     'ikosaeder' : Ikosaeder: rays in 6 directions
!                     'dodekaeder': Dodekaeder: rays in 10 directions
!                     'list-01(1)': 'oktaeder-dd1(1)', 'oktaeder-d1d(1)', 'oktaeder-1dd',
!                                   'cube-ddd(1)'
!                     'list-01(2)': 'oktaeder-dd1', 'oktaeder-d1d', 'oktaeder-1dd(2)', 'cube-ddd(2)'
!                     'list-01(3)': 'oktaeder-dd1', 'oktaeder-d1d', 'oktaeder-1dd(3)', 'cube-ddd(3)'
!                     'list-01'   = 'list-01(3)'
!                     'list-02(3)': 'duooktaeder-11x', 'cube-ddd(3)', 'duooktaeder-1x1',
!                                   'cube-ddd(3)', 'duooktaeder-x11'
!                     'list-02'   = 'list-02(3)'
!                     'list-03(3)': 'oktaeder-dd1', 'oktaeder-d1d', 'oktaeder-1dd(3)'
!                     'list-03'   = 'list-03(3)'
!                     'list-04(3)': 'oktaeder-dd1', 'cube-ddd(3)', 'oktaeder-d1d', 'cube-ddd(3)',
!                                   'oktaeder-1dd(3)', 'cube-ddd(3)'
!                     'list-04'   = 'list-04(3)'
!   radpressure:    ('radiation_pressure') character for radiation pressure mode:
!                   'off', 'on'
!   C_visLinear:    ('Constant_viscosity_Linear), real for linear viscosity, default: 0.
!   C_visSmagorinsky: ('Constant_viscosity_Smagorinsky') real, parameter for Smagorinsky-type
!                     eddy viscosity, reasonable range between 0.0 and 1.0, typically 0.5,
!                     default: 0.0
!   C_visArtificial: ('Constant_viscosity_Artificial') real, parameter for artificial viscosity,
!                    reasonable range between 0.0 and 1.0, typically 0.5, default: 0.
!   C_visExpansion:  ('Constant_viscosity_Expansion') real, parameter for expansion viscosity,
!                    default: 0.0
!   C_visPrturb:    ('Constant_viscosity_Prandtl_turbulent') real, Prandtl number for
!                   turbulent mixing: 0.0 (or huge number) -> off
!   C_visTensordiag: ('Constant_viscosity_Tensor_diagonal') real, factor in stress tensor in
!                    front of diagonal terms, typically 1.0
!   C_visTensoroff: ('Constant_viscosity_Tensor_diagonal_off') real, factor in stress tensor in
!                    front of off-diagonal terms, typically 0.5
!   C_visTensordiv: ('Constant_viscosity_Tensor_divergence') real, factor in stress tensor in
!                    front of divergence*UNITY, typically -2./3.
!   C_visP2Pcoeff:  ('Constant_viscosity_Point_to_Point_coefficient') real, point-to-point visocity
!                   coefficient:
!                      0.0: no point-to-point viscosity
!                      1.0: typical value
!   C_visP2Pincl1:  ('Constant_viscosity_Point_to_Point_inclined_1') real, factor for inclined
!                   points: area diagonal (e.g. 110)
!                   typically 1.0
!   C_visP2Pincl2:  ('Constant_viscosity_Point_to_Point_inclined_2') real, factor for inclined
!                   points: space diagonal (e.g. 111)
!                   typically 1.0
!   C_visP2PhypSmagorinsky: ('Constant_viscosity_Point_to_Point_hyper_Smagorinsky') real,
!                   Smagorinsky parameter for point-to-point viscosity,
!                   reasonable range between 0.0 and 1.0, typically 0.5,
!                   default: 0.0
!   C_visP2PhypArtificial: ('Constant_viscosity_Point_to_Point_hyper_Artificial') real,
!                   parameter for point-to-point artificial viscosity,
!                   reasonable range between 0.0 and 1.0, typically 0.5, default: 0.
!   C_visP2PhypExpansion:  ('Constant_viscosity_Point_to_Point_hyper_Expansion') real,
!                   parameter for point-to-point expansion viscosity,
!                    default: 0.0
!   C_visP2Pdivrhov: ('Constant_viscosity_Point_to_Point_divergence_rho_velocity') real,
!                   parameter for point-to-point mass-flux-divergence viscosity,
!                   reasonable range between 0.0 and 1.0, typically 0.5, default: 0.
!   N_visIter:      ('Number_viscosity_Iteration') integer, number of iterations of
!                   tensor-viscosity substep:
!                    0: Do one viscosity substep per global time step after (M)HD substeps (default)
!                   <0: Do one viscosity substep all -N_visIter HD substeps (HD solver only)
!                   >0: Not implemented yet.
!   N_visP2PhypSmooth: ('Number_viscosity_Point_to_Point_hyper_Smooth') integer,
!                   Number of smoothing iterations for point-to-point hyperviscosity
!   visP2Phypreconstruction: ('viscosity_Point_to_Point_hyper_reconstruction') character,
!                   reconstruction method for point-to-point hyperviscosity scheme
!   C_resB       :  ('Constant_resistivity_of_magnetic_field') real, 0.0 or small value
!   C_resBconstant: ('Constant_resistivity_of_magnetic_field_constant') real, 0.0 or small value
!   C_resEpsilon :  ('Constant_resistivity_of_internal_energy') real, 0.0 or small value
!   starttime:      ('start_time') real, if >=0 the initial time is set to this value
!                      starttime <  0.0 -> value ignored
!                      starttime >= 0.0 -> use this value as initial time (ignore value from
!                                          start file)
!   endtime:        ('end_time') real, after this simulation time the simulation exits
!                     endtime < 0.0 -> value ignored
!   plustime:       ('plus_time') real, the simulation is advanced by this time
!                     plustime < 0.0 -> value ignored
!   time_ScaleStart: ('Scale_time_Start') real with beginning of tweak interval,
!                   only relevant for certain tweak modules;
!                   if Scale_time_Start <= time (<= Scale_time_End) some parameters might be
!                   modified by a tweak module
!   time_ScaleEnd:  ('Scale_time_End') real with end of tweak interval,
!                   only relevant for certain tweak modules
!   CPUtime:        ('CPU_time') real, CPU time for this job
!                      CPUtime < 0.0 -> value ignored
!   CPUtime_remainlimit: ('CPU_time_remain_limit') real, CPU time limit,
!                        if there is not enough CPU time left, the simulation exits
!                          CPUtime_remainlimit < 0.0 -> value ignored
!   starttimestep:  ('start_time_step') integer, if >=0 the initial time step is set to this value
!                      starttimestep <  0 -> value ignored
!                      starttimestep >= 0 -> use this value as initial time step (ignore value from
!                                            start file)
!   endtimestep:    ('end_time_step') integer, after this simulation time step
!                   number the simulation exits
!                     endtimestep < 0 -> value ignored
!   plustimestep:   ('plus_time_step') integer, the simulation is advanced by this time step number
!                       plustimestep < 0 -> value ignored
!   dtime_min:      ('delta_time_minimum') real, mimimum allowed time step,
!                   dtime_min<=0 -> no restriction
!   dtime_max:      ('delta_time_maximum') real, maximum allowed time step,
!                   dtime_max<=0 -> no restriction
!   dtime_min_stop: ('delta_time_minimum_stop') real, mimimum allowed time step,
!                     dtime_min <= 0              -> no restriction,
!                     dtime     <  dtime_min_stop -> program stop
!   dtime_incmax:   ('delta_time_increment_maximum') real, maximum allowed relative
!                   time step increment,
!                     dtime_incmax <  1.0 -> no restriction
!                     dtime_incmax >= 1.0 -> dtime_new <= dtime_old*dtime_incmax
!   dtime_start:    ('delta_time_start') real, initial time step
!                     dtime_start <=  0.0 -> take time step from start file (default)
!                     dtime_start >   0.0 -> use this value as initial time step (ignore value from
!                                            start file)
!   C_Courant:      ('Constant_Courant') real, Courant factor, typically 0.5
!   C_CourantMax:   ('Constant_Courant_Maximum') real, maximum Courant factor,
!                   if dtime exceeds C_CourantMax*dtime_hd_limit, the
!                   computation of this time step is stopped and resumed with
!                     dtime=C_Courant*dtime_limit,
!                   typically 0.9
!   C_maxeiChange:  ('Constant_maximum_energy_internal_change') real, maximum amount
!                   of relative internal energy change due to hydro step
!   C_hydExpCourant: ('Constant_hydrodynamics_Expansion_Courant') real, expansion Courant factor,
!                   typically 0.05, default 0.0 (check off)
!   C_hydExpCourantMax: ('Constant_hydrodynamics_Expansion_Courant_Maximum') real,
!                   maximum expansion Courant factor,
!                   typically 0.2, default 0.0 (check off)
!   C_hydSoundCourant: ('Constant_hydrodynamics_Sound_Courant') real, sound-speed Courant factor,
!                   useful for CTU scheme: allows larger normal Courant factor,
!                   typically 0.45, default 0.0 (check off)
!   C_hydSoundCourantMax: ('Constant_hydrodynamics_Sound_Courant_Maximum') real,
!                   useful for CTU scheme: allows larger normal Courant(max) factor,
!                   maximum sound-speed Courant factor,
!                   typically 0.5, default 0.0 (check off)
!   C_radCourant:   ('Constant_radiative_Courant') real, radiative Courant factor, typically 1.0
!   C_radCourantMax: ('Constant_radiative_Courant_Maximum') real, maximum radiative Courant factor,
!                   if dtime exceeds C_CourantMax*dtime_rad_limit, the
!                   computation of this time step is stopped and resumed with
!                     dtime=C_Courant*dtime_limit,
!                   typically 2.0
!   C_radmaxeiChange: ('Constant_radiative_maximum_energy_internal_change') real, maximum amount
!                   of relative internal energy change due to radiative energy transfer
!   C_visCourant:   ('Constant_viscosity_Courant') real, viscous Courant factor, typically 1.0
!   C_visCourantMax: ('Constant_visocsity_Courant_Maximum') real, maximum viscous Courant factor,
!                   if dtime exceeds C_CourantMax*dtime_limit, the
!                   computation of this time step is stopped and resumed with
!                     dtime=C_Courant*dtime_limit,
!                   typically 2.0
!   C_resCourant:   ('Constant_resistivity_Courant') real, resistive Courant factor, typically 0.5
!   C_resCourantMax: ('Constant_resistivity_Courant_Maximum') real, maximum resistive Courant
!                   factor, if dtime exceeds C_CourantMax*dtime_limit, the
!                   computation of this time step is stopped and resumed with
!                     dtime=C_Courant*dtime_limit,
!                   typically 1.0
!   dtime_out_full: ('delta_time_output_full') real, number of seconds between
!                   two full output times
!   dtime_out_mean: ('delta_time_output_mean') real, number of seconds between
!                   two output times with mean values
!   dtime_out_end:  ('delta_time_output_end') real, number of seconds between
!                   two output times with complete models.
!                   Each model and at the end the final model overwrites its predecessor.
!   dtimestep_out_fine: ('delta_time__step_output_fine') integer, number of
!                       time steps between two fine output times
!   istep_in_start: ('index_step_input_start') index, number of dataset to read as start file
!                   1: first dataset, 2: second dataset, ...
!   N_outslicedim_mean: ('number_output_slice_dimension_mean') index of slice in mean output file:
!                   1, 2, 3: output of slice with plane perpendicular to this direction
!                   0      : no output of slice
!   infile_start:   ('input_file_start') character, initial model file name (start)
!   OutFile_end:    ('Output_File_end') character, output model file name (end)
!   OutForm_end:    ('Output_Format_end') character, output file format (end)
!   OutConv_end:    ('Output_Conversion_end') character, output file conversion type (end)
!   OutFile_full:   ('Output_File_full') character, output file name (full)
!   OutForm_full:   ('Output_Format_full') character, output file format (full)
!   OutConv_full:   ('Output_Conversion_full') character, output file conversion type (full)
!   OutFile_mean:   ('Output_File_mean') character, output file name (mean)
!   OutForm_mean:   ('Output_Format_mean') character, output file format (mean)
!   OutConv_mean:   ('Output_Conversion_mean') character, output file conversion type (mean)
!   OutFile_fine:   ('Output_File_fine') character, output file name (fine)
!   OutForm_fine:   ('Output_Format_fine') character, output file format (fine)
!   OutConv_fine:   ('Output_Conversion_fine') character, output file conversion type (fine)
!   OutFile_mcyl:   ('Output_File_mean_cylinder') character, output file name (cylindrical means)
!   OutFile_chu1:   ('Output_File_chunk_1') character, output file name (chunk 1)
!   OutDim_chu1:    ('Output_Dimension_chunk_1) integer array with dimension of output chunk 1
!   OutFile_chu2:   ('Output_File_chunk_2') character, output file name (chunk 2)
!   OutDim_chu2:    ('Output_Dimension_chunk_2) integer array with dimension of output chunk 2
!   OutFile_chu3:   ('Output_File_chunk_3') character, output file name (chunk 3)
!   OutDim_chu3:    ('Output_Dimension_chunk_3) integer array with dimension of output chunk 3
!   OutFile_out:    ('Output_File_output') character, output file name for text (console) messages
!   N_test1:        ('Number_test_1) integer for test purposes
!   N_test2:        ('Number_test_2) integer for test purposes
!   N_test3:        ('Number_test_3) integer for test purposes
!   N_test4:        ('Number_test_4) integer for test purposes
!   N_test5:        ('Number_test_5) integer for test purposes
!   C_test1:        ('Constant_test_1) real number for test purposes
!   C_test2:        ('Constant_test_2) real number for test purposes
!   C_test3:        ('Constant_test_3) real number for test purposes
!   C_test4:        ('Constant_test_4) real number for test purposes
!   C_test5:        ('Constant_test_5) real number for test purposes
!   strtest1:       ('string_test_1) string for test purposes
!   strtest2:       ('string_test_2) string for test purposes
!   strtest3:       ('string_test_3) string for test purposes
!   strtest4:       ('string_test_4) string for test purposes
!   strtest5:       ('string_test_5) string for test purposes
!
! ROUTINES:
!   uio_closrd:            Close file after reading
!   uio_err:               Handle UIO errors
!   uio_filcon:            Determine file contents: list of all entries with its positions
!   uio_openrd:            Open file for reading, read header
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_srhd:              Search entry header in file or in entry list
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!   uio_nam_module:        Definition of names for UIO
!   uio_bulk_module:       Main set of UIO routines
!
! SIDE EFFECTS:
!   Open file, read data from it, close file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Open requested file (in UIO format).
!   Determine file contents.
!   Search each parameter, read it, and put it into variable.
!   Close file again.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-06-09 (B.F. Kiel) Written
!   1997-10-13 (B.F.) Additional parameters
!   1997-10-14 (B.F.) Additional parameters
!   1997-10-24 (B.F.) C_interaction
!   1997-11-14 (B.F.) C_visNeu1, C_visNeu2
!   1998-02-19 (B.F.) xbtotal1 added
!   1998-04-18 (B.F.) Parameters model, counter, programversion, htautop added
!   1998-11-13 (B.F., Copenhagen) Check error condition after opening of file
!   1999-04-07 (B.F.) Parameters mass_star, r0_grav added
!   1999-04-19 (B.F. Kiel) Parameter C_visDrag added
!   1999-05-25 (B.F.) gravcorr_terms
!   1999-05-27 (B.F.) Parameter for tensor viscosity added
!   1999-09-08 (B.F.) Parameter hdScheme added
!   1999-09-28 (B.F.) Parameters C_TChange, C_Tsurf added
!   2000-04-13 (B.F. Potsdam) htautop -> C_radHtautop
!   2001-04-10 (B.F. Uppsala) Parameter C_Hptopfactor, C_radTvisdtau, C_radTvis added
!   2001-06-29 (B.F.) Parameter C_radthintimefac added
!   2001-10-10 (B.F.) Overwrite error information
!   2002-01-18 (B.F.) Missing parameters abuX, abuY, Qmol give no errors anymore
!   2002-04-10 (B.F.) Default value for N_radtaurefine from 1 to 0
!   2003-01-16 (B.F.) Parameter gravcorr_terms removed
!   2003-01-22 (B.F.) Parameter C_SlopeRed added
!   2005-05-31 (W.S.&SWB) MHD parameters added: C_resB, C_resEpsilon
!   2006-08-07 (SWB)  hion_datapath, hion_atomfile, hion_abufile, hion_edensfile,
!                   hion_pffile, dtime_out_hion, hion_chunks
!   2006-11-07 (B.F. Lyon) New parameters added:
!                   N_radrsyslevel, N_radoutput, C_radtcool, C_radscool, C_raddcool,
!                   N_test4, N_test5, C_test4, C_test5, strtest4, strtest5
!   2008-03-13 (B.F.) Parameters C_hydExpCourant, C_hydExpCourantMax added
!   2008-03-17 (B.F.) Parameters C_ReconstructiondTimeFactor, C_hydPredfactor added
!   2008-04-27 (B.F.) Parameter C_visExpansion added
!   2008-06-10 (W.S.) New parameters for magnetic boundary conditions added:
!                     side_bound_mag, top_bound_mag, bottom_bound_mag
!   2008-07-03 (B.F.) Parameter C_visLinear added
!   2008-08-12 (B.F.) Parameters C_radTinci, C_radDinci added
!   2008-08-19 (B.F.) Parameter C_ReconstructiondTimeFactor outcommented
!   2008-10-22 (W.S. Lansing) New parameters: B_theta, B_phi, N_side, N_top, N_bottom, beta_inv
!   2010-02-15 (W.S. Freiburg) side_bound_mag split into side_bound_mag_x1 and side_bound_mag_x2
!   2010-02-16 (W.S. Freiburg) New parameter: va_max
!   2010-04-11 (W.S. Freiburg) New parameter: C_resBconstant
!   2010-04-14 (B.F.) B_theta, B_phi, N_side, N_top, N_bottom =>
!                     C_magthetaB, C_magphiB, N_magDiffSide, N_magDiffTop, N_magDiffBottom
!   2010-05-10 (B.F.) Rename rhd_rdpar -> rhd_par_RdData,
!                     get n_dustGrainRadiusMax as input parameter and not from rhd_gl_module
!   2010-06-03 (B.F. Freiburg) New parameter: hdTimeIntegrationScheme
!   2010-06-09 (B.F. Lyon) New parameter: hdCheckFlux
!   2011-01-06 (W.S. Exeter) New parameters N_resCellsPerChunk, C_resCourant, C_resCourantMax
!   2011-01-27 (W.S. Lyon) New parameter N_orderConstrainedTransport
!   2011-03-31 (B.F., SWB) New parameter C_TminLimit, C_TminRate
!   2011-06-20 (W.S.) New parameters r0_core, centrifugal_force
!   2011-07-17 (B.F.) Parameters C_visNeu1, C_visNeu2, C_visDrag removed
!   2011-08-05 (W.S.) New parameter r2_grav
!   2011-08-26 (B.F.) New parameter hdTransVeloMode
!   2011-09-01 (B.F.) New parameters C_hydTdiff1, C_hydTdiff2
!   2011-09-02 (B.F.) New params C_visP2PhypSmagorinsky, N_visP2PhypSmooth, visP2Phypreconstruction
!   2011-11-18 (B.F.) Parameters C_RecContShift, C_RecContSteep added
!   2011-11-23 (B.F.) New parameter dustReconstruction
!   2011-12-01 (B.F.) New parameters C_visP2PhypArtificial, C_visP2PhypExpansion
!   2011-12-24 (B.F.) New parameters C_hydSoundCourant, C_hydSoundCourantMax
!   2012-01-01 (B.F.) New parameters C_v3ChangeLinBottom, C_v3ChangeSqrBottom
!   2012-01-09 (B.F.) Renamed parameters C_hydTdiff1, C_hydTdiff2 -> C_hydTdiffLin, C_hydTdiffMach
!   2012-01-17 (B.F.) New parameter C_visP2Pdivrhov
!   2012-03-07 (B.F.) New parameter ar_dustGrainAbu
!   2012-04-03 (B.F.) New parameter C_hydsDiffVelo
!   2012-04-04 (B.F.) New parameter C_hydvDiffVelo
!   2012-05-04 (B.F.) New parameter C_hydLowMachcsFactor
!   2012-06-05 (B.F.) New parameters hdEntropyWaveMode, hdEnthalpyAvgMode
!   2012-07-09 (B.F.) New Parameter hdCoreDragProfile
!   2012-07-18 (B.F.) Print only if nc_p >= 0
!   2012-07-30 (B.F.) New parameter aN_MPIProcsPerDim
!   2013-08-07 (B.F.) New parameters: OutFile_chu2/3, OutDim_chu2/3
!   2013-08-08 (B.F.) New parameter: OutFile_mcyl
!   2013-12-05 (B.F.) New parameter: kappaMin
!   2014-03-01 (B.F. Uppsala) New parameter: s_InFlGrad
!   2014-04-08 (B.F.) New parameter: hdCoreHeatProfile
!   2014-06-12 (B.F.) New options to hdCoreDragProfile: '...-Radial'
!   2014-06-13 (B.F.) New options to hdCoreDragProfile: '...-Meridional'
!   2014-06-16 (B.F.) New parameters s_InFlRadGrad, s_InFlLatGrad replace s_InFlGrad
!   2014-07-11 (B.F.) New parameter: C_gravmu
!   2014-08-11 (B.F.) New parameter: C_hydSmallFluctFactor
!   2014-08-15 (B.F.) New parameters: N_hydRKIter, N_hydCellsPerChunk3
!   2014-08-20 (B.F.) New parameters: C_RecWenoWeightCenter, C_RecWenoWeightRight,C_RecWenoPsiFactor
!   2014-08-24 (B.F.) New parameter: N_hydCellsPerChunk2
!   2014-08-25 (B.F.) New parameter: hdTmpStructKeepMode
!   2014-09-22 (B.F.) New parameter: C_hydLowMachcsBase
!   2014-12-13 (B.F.) New parameter: radThinHeatScheme; removed: C_visNeu1, C_visNeu2, C_visDrag
!   2014-12-17 (B.F.) New Parameter: N_visIter
!   2016-03-15 (B.F.) New parameter: C_radAccEnergyFraction
!   2017-08-23 (B.F. Lyon) Bugfix: read 'c_resb' and 'c_resbconstant' instead of 'c_resB' and
!                     'c_resBconstant'
!   2017-10-30 (B.F.) New parameter: N_radFeautrier
!   2017-11-05 (B.F.) nctotal1,2,3 and xbtotal1,2 removed
!   2018-04-24 (B.F.) New parameters: grav_type, C_gravCorePotDepth, C_gravCorePotQuad
!   2018-06-11 (B.F.) New parameter C_gravCorer0Heat=r0_core/r0_grav replaces r0_core
!   2019-02-26 (B.F.) New parameters C_radthintimeRelax, radThinPressScheme, N_radThinHeatPoint,
!                     N_radThinPressPoint
!   2019-03-05 (B.F.) New parameter C_radthinTFactor
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: parFile
integer,                   intent(in)   :: nc_p, n_dustGrainRadiusMax
integer,         optional, intent(out)  :: ndes
character(len=*), dimension(:), &
                 optional, intent(out)  :: description
integer,         optional, intent(out)  :: nhis
character(len=*), dimension(:), &
                 optional, intent(out)  :: history
character(len=*),optional, intent(out)  :: model, counter, programversion
real,            optional, intent(out)  :: Teff
character(len=*),optional, intent(out)  :: heat_mode, grav_mode, grav_type, centrifugal_force
real,            optional, intent(out)  :: grav, mass_star, &
                                           C_gravmu, C_gravCorePotDepth, C_gravCorePotQuad, &
                                           r0_grav, r1_grav, r2_grav, r1_rad, C_gravCorer0Heat, &
                                           nu_rotation
real, dimension(3), &
                 optional, intent(out)  :: ar_RotationAxis
character(len=*),optional, intent(out)  :: side_bound, top_bound, bottom_bound
character(len=*),optional, intent(out)  :: side_bound_mag_x1, side_bound_mag_x2, &
                                           top_bound_mag, bottom_bound_mag, &
                                           hdCoreHeatProfile, hdCoreDragProfile
real,            optional, intent(out)  :: s_InFlow, s_InFlRadGrad, s_InFlLatGrad, &
                                           C_sChange, C_PChange, C_v3ChangeLinBottom, &
                                           C_v3ChangeSqrBottom, &
                                           C_TChange, C_rhoChangeTop, &
                                           C_Tsurf, C_TminLimit, C_TminRate, &
                                           C_Hptopfactor, rho_min, LuminosityPerVolume, C_CoreDrag,&
                                           C_hydTDiffLin, C_hydTDiffMach, &
                                           C_hydsDiffVelo, C_hydvDiffVelo, &
                                           C_hydLowMachcsFactor, C_hydLowMachcsBase, &
                                           C_hydSmallFluctFactor, &
                                           B1_inflow, C_magthetaB, C_magphiB
integer,         optional, intent(out)  :: N_magDiffSide, N_magDiffTop, N_magDiffBottom
real,            optional, intent(out)  :: abuX, abuY, abuZ, Qmol, gamma
character(len=*),optional, intent(out)  :: EOSFile, EOSPath
real,            optional, intent(out)  :: kappaMin, kappaMax
character(len=*),optional, intent(out)  :: OpaFile, OpaPath
real,            optional, intent(out)  :: chem_abumetal
character(len=*),optional, intent(out)  :: chem_reacfile, chem_reacpath
character(len=*),optional, intent(out)  :: hion_datapath, hion_atomfile, hion_abufile, &
                                           hion_edensfile, hion_pffile
real,            optional, intent(out)  :: dtime_out_hion
integer,         optional, intent(out)  :: hion_chunks
character(len=*),optional, intent(out)  :: hdScheme, hdTimeIntegrationScheme, hdsplit, &
                                           hdTmpStructKeepMode, &
                                           hdCheckFlux, reconstruction, &
                                           hdTransVeloMode, hdEntropyWaveMode, hdEnthalpyAvgMode
integer,         optional, intent(out)  :: N_orderConstrainedTransport, &
                                           N_hydIter, N_hydMaxIter, N_hydRKIter, &
                                           N_hydCellsPerChunk, N_hydCellsPerChunk2, &
                                           N_hydCellsPerChunk3, N_visCellsPerChunk, &
                                           N_resCellsPerChunk, &
                                           N_radCellsPerChunk
integer, dimension(3), &
                 optional, intent(out)  :: aN_MPIProcsPerDim
real,            optional, intent(out)  :: C_SlopeRed, C_RecContShift, C_RecContSteep, &
                                           C_RecWenoWeightCenter, C_RecWenoWeightRight, &
                                           C_RecWenoPsiFactor, &
                                           C_hydPredfactor
real,            optional, intent(out)  :: C_visBound
real,            optional, intent(out)  :: beta_inv, va_max
character(len=*),optional, intent(out)  :: dustScheme, dustReconstruction, dustOpaFile, dustopamode
integer,         optional, intent(out)  :: n_dustGrainRadius, n_dustGrainAbu
real, &
  dimension(1:n_dustGrainRadiusMax), &
                 optional, intent(out)  :: ar_dustGrainRadius, ar_dustGrainAbu
real,            optional, intent(out)  :: C_dust01, C_dust02, C_dust03, C_dust04, C_dust05, &
                                           C_dust06, C_dust07, C_dust08, C_dust09, C_dust10
character(len=*),optional, intent(out)  :: radScheme, radThinHeatScheme, radThinPressScheme, &
                                           radraybase, radraystar, radpressure
integer,         optional, intent(out)  :: N_radminiter, N_raditer, N_radmaxiter, N_radtaurefine, &
                                           N_radThinPoint, N_radThinHeatPoint, N_radThinPressPoint,&
                                           N_radThickPoint, &
                                           N_radtheta, N_radphi, N_radsubray, N_radband, &
                                           N_radrsyslevel, N_radoutput, N_radFeautrier
real,            optional, intent(out)  :: C_radimplicitmu, C_raditereps, C_raditerstep, &
                                           C_radkappasmooth,C_radTstep  , C_radTsmooth, &
                                           C_radTintminfac, &
                                           C_radHtautop, C_radTvisdtau, C_radTvis, &
                                           C_radthintimefac, C_radthintimeRelax, C_radthinTFactor, &
                                           C_radtcool, C_radscool, C_raddcool, &
                                           C_radTinci, C_radDinci, C_radAccEnergyFraction
integer,         optional, intent(out)  :: N_visIter, N_visP2PhypSmooth
real,            optional, intent(out)  :: C_visLinear, &
                                           C_visSmagorinsky, C_visArtificial, C_visExpansion, &
                                           C_visPrturb, &
                                           C_visTensordiag, C_visTensoroff, C_visTensordiv, &
                                           C_visP2Pcoeff, C_visP2Pincl1, C_visP2Pincl2, &
                                           C_visP2PhypSmagorinsky, C_visP2PhypArtificial, &
                                           C_visP2PhypExpansion, C_visP2Pdivrhov
real,            optional, intent(out)  :: C_resB, C_resBconstant, C_resEpsilon
character(len=*),optional, intent(out)  :: visP2Phypreconstruction
real,            optional, intent(out)  :: starttime, endtime, plustime, &
                                           time_ScaleStart, time_ScaleEnd, &
                                           CPUtime, CPUtime_remainlimit
integer,         optional, intent(out)  :: starttimestep, endtimestep, plustimestep
real,            optional, intent(out)  :: C_Courant, C_CourantMax, &
                                             C_maxeiChange, &
                                             C_hydExpCourant, C_hydExpCourantMax, &
                                             C_hydSoundCourant, C_hydSoundCourantMax, &
                                           C_radCourant, C_radCourantMax, &
                                             C_radmaxeiChange, &
                                           C_visCourant, C_visCourantMax, &
                                           C_resCourant, C_resCourantMax, &
                                           dtime_min, dtime_max, dtime_min_stop, dtime_incmax, &
                                             dtime_start
real,            optional, intent(out)  :: dtime_out_full, dtime_out_mean, dtime_out_end
integer,         optional, intent(out)  :: dtimestep_out_fine, istep_in_start, N_outslicedim_mean
character(len=*),optional, intent(out)  :: infile_start, &
                                           OutFile_end , OutForm_end , OutConv_end , &
                                           OutFile_full, OutForm_full, OutConv_full, &
                                           OutFile_mean, OutForm_mean, OutConv_mean, &
                                           OutFile_fine, OutForm_fine, OutConv_fine, &
                                           OutFile_mcyl, &
                                           OutFile_chu1, OutFile_chu2, OutFile_chu3, &
                                           OutFile_out
integer, dimension(2,3), &
                 optional, intent(out)  :: OutDim_chu1, OutDim_chu2, OutDim_chu3
integer,         optional, intent(out)  :: N_test1, N_test2, N_test3, N_test4, N_test5
real   ,         optional, intent(out)  :: C_test1, C_test2, C_test3, C_test4, C_test5
character(len=*),optional, intent(out)  :: strtest1, strtest2, strtest3, strtest4, strtest5
character(len=*),optional, intent(out)  :: outstr
integer,         optional, intent(out)  :: ierr
!
! --- Local variables ---
integer                                 :: ierr0
character(len=80)                       :: outstr0
integer                                 :: ncpar
character(len=80)                       :: file_id
integer                                 :: ntt
character(len=let)                      :: termt(2,nttmx)
character(len=let)                      :: ident
integer, parameter                      :: nentmx=200
integer                                 :: nentry
integer                                 :: poslist(nentmx)
character(len=let)                      :: identlist(3,nentmx)
integer                                 :: iup(2)
real                                    :: r1_grav_tmp, r2_grav_tmp
!---------------------------------------------------------------------------------------------------
!
! === Open file ====================================================================================
!
! --- Open parameter file for reading ---
call uio_openrd(ncpar, parFile, outstr=outstr0, ierr=ierr0)
if (ierr0 > 0) goto 900
!
if (nc_p >= 0) then
  write(nc_p,'(A,I3)') 'Parameter file ''' // trim(parFile) // ''' opened on channel ', ncpar
endif
! --- Determine parameter file contents ---
call uio_filcon(ncpar, identlist,poslist,nentry)
!
!if (nc_p >= 0) then
!  write(nc_p,*) 'nentry=', nentry
!  write(nc_p,'(A)') identlist(:,:nentry)
!  write(nc_p,'(I4)') poslist(:nentry)
!endif
!
! --- Read file_id ---
call uio_srhd(ncpar, termt,ntt, type=charna, ident='file_id', &
              identlist=identlist, poslist=poslist, nentry=nentry, &
              outstr=outstr0,ierr=ierr0)
if (ierr0 == 0) then
  call uio_rd(ncpar, termt,ntt, file_id,ident)
  if ((file_id /= 'rhd-parameter') .and. (file_id /= 'hdw-parameter')) then
    outstr0='''' // trim(parFile) // ''' is not RHD parameter file'
    ierr0=1
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
  endif
else
  outstr0='Type of ''' // trim(parFile) // ''' can not be determined'
  ierr0=2
  call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
endif
!
! === Read header ==================================================================================
!
! --- Read description ---
if (present(description) .and. present(ndes)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='description', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, description,ident,mode=2,iup=iup(1:1))
    ndes=iup(1)
  else
    ndes=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)') 'Description: ..............................................'
    write(nc_p,'(A)') description(:ndes)
  endif
endif
!
! --- Read history ---
if (present(history) .and. present(nhis)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='history', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, history,ident,mode=2,iup=iup(1:1))
    nhis=iup(1)
  else
    nhis=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)') 'History: ..................................................'
    write(nc_p,'(A)') history(:nhis)
    write(nc_p,'(A)') '...........................................................'
  endif
endif
!
! === Read model data ==============================================================================
!
! --- Read model name ---
if (present(model)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='model', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, model,ident)
  else
    model=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Model name:         ' // trim(model)
  endif
endif
!
! --- Read file counter ---
if (present(counter)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='counter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, counter,ident)
  else
    counter=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'File counter:       ' // trim(counter)
  endif
endif
!
! --- Read program version ---
if (present(programversion)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='programversion', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, programversion,ident)
  else
    programversion=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Program version:    ' // trim(programversion)
  endif
endif
!
! \label{f90:teff__read}
! --- Read Teff: effective temperature ---
if (present(Teff)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='teff', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, Teff,ident)
  else
    outstr0='temperature is not specified'
    ierr0=3
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,F15.6)')   'Teff:               ', Teff
  endif
endif
!
! \label{f90:heat_mode__read}
! --- Read heat_mode: type of heating ---
if (present(heat_mode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='heat_mode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, heat_mode,ident)
  else
    heat_mode=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Mode of heating:    ' // trim(heat_mode)
  endif
endif
!
! \label{f90:grav_mode__read}
! --- Read grav_mode: type of gravity field ---
if (present(grav_mode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='grav_mode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, grav_mode,ident)
    if (grav_mode == 'Constant') grav_mode='constant'
    if (grav_mode == 'Central' ) grav_mode='central'
  else
    grav_mode='constant'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Mode of gravity:    ' // trim(grav_mode)
  endif
endif
!
! \label{f90:grav__read}
! --- Read grav: gravity ---
if (present(grav)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='grav', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, grav,ident)
  else
    grav=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)')    'grav:               ', grav
  endif
endif
!
! \label{f90:mass_star__read}
! --- Read mass_star: mass of star ---
if (present(mass_star)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='mass_star', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, mass_star,ident)
  else
    mass_star=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'mass_star:          ', mass_star
  endif
endif
!
! \label{f90:c_gravmu__read}
! --- Read C_gravmu: time centering of gravity terms ---
if (present(C_gravmu)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_gravmu', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_gravmu,ident)
  else
    C_gravmu=-1.0
  endif
  C_gravmu=min(C_gravmu, 1.0)
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_gravmu:           ', C_gravmu
  endif
endif
!
! \label{f90:c_gravcorepotdepth__read}
! --- Read C_gravCorePotDepth: depth factor of central potential in core region ---
if (present(C_gravCorePotDepth)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_gravcorepotdepth', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_gravCorePotDepth,ident)
    C_gravCorePotDepth=max(C_gravCorePotDepth, 0.0)
  else
    C_gravCorePotDepth=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_gravCorePotDepth: ', C_gravCorePotDepth
  endif
endif
!
! \label{f90:c_gravcorepotquad__read}
! --- Read C_gravCorePotQuad: amplitude of quadratic term in core region of ---
! --- 'classic4' central potential                                          ---
if (present(C_gravCorePotQuad)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_gravcorepotquad', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_gravCorePotQuad,ident)
    C_gravCorePotQuad=max(C_gravCorePotQuad, 0.0)
  else
    C_gravCorePotQuad=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_gravCorePotQuad:  ', C_gravCorePotQuad
  endif
endif
!
! \label{f90:r0_grav__read}
! --- Read r0_grav ---
if (present(r0_grav)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='r0_grav', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, r0_grav,ident)
  else
    r0_grav=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'r0_grav:            ', r0_grav
  endif
endif
!
! \label{f90:r1_grav__read}
! --- Read r1_grav ---
if (present(r1_grav)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='r1_grav', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, r1_grav,ident)
  else
    r1_grav=0.0
  endif
  r1_grav_tmp=r1_grav
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'r1_grav:            ', r1_grav
  endif
else
  r1_grav_tmp=0.0
endif
!
! \label{f90:r2_grav__read}
! --- Read r2_grav ---
if (present(r2_grav)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='r2_grav', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, r2_grav,ident)
  else
    r2_grav=0.0
  endif
  r2_grav_tmp=r2_grav
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'r2_grav:            ', r2_grav
  endif
else
  r2_grav=0.0
endif
!
! \label{f90:r1_rad__read}
! --- Read r1_rad ---
if (present(r1_rad)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='r1_rad', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, r1_rad,ident)
  else
    r1_rad=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'r1_rad:             ', r1_rad
  endif
endif
!
! \label{f90:c_gravcorer0heat__read}
! --- Read C_gravCorer0Heat ---
if (present(C_gravCorer0Heat)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_gravcorer0heat', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_gravCorer0Heat,ident)
  else
    C_gravCorer0Heat=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_gravCorer0Heat:   ', C_gravCorer0Heat
  endif
endif
!
! \label{f90:grav_type__read}
! --- Read grav_type: type of gravity field ---
if (present(grav_type)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='grav_type', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, grav_type,ident)
    if (grav_type == 'Classic2') grav_type='classic2'
    if (grav_type == 'Classic4') grav_type='classic4'
  else
    grav_type=''
  endif
  !
  if (grav_type == '') then
    if ((r1_grav == 0.0) .and. (r2_grav == 0.0)) then
      ! --- Use the 'old' RSG potential with ^2, computed only from r0_grav ---
      grav_type='classic2'
    else if ((r1_grav == 0.0) .and. (r2_grav /= 0.0)) then
      ! --- Compute the near-polytropic ^2 potential ---
      grav_type='classic2'
      r1_grav=r2_grav
      if (nc_p >= 0) then
        write(nc_p,'(A,1PE15.8,A)') 'r1_grav:            ', r1_grav, ' <=== Overwrite'
      endif
    else
      ! --- Compute the ^4 RSG potential from r0_grav and r1_grav ---
      grav_type='classic4'
    endif
  endif ! (grav_type)
  !
!  grav_type_tmp=grav_type
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Type of gravity:    ' // trim(grav_type)
  endif
!else
!  grav_type_tmp=''
endif
!
! \label{f90:centrifugal_force__read}
! --- Read centrifugal_force: centrifugal-force control parameter ---
if (present(centrifugal_force)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='centrifugal_force', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, centrifugal_force,ident)
    if ((centrifugal_force == 'On')) centrifugal_force='on'
  else
    centrifugal_force='on'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Centrifugal force:  ' // trim(centrifugal_force)
  endif
endif
!
! \label{f90:nu_rotation__read}
! --- Read nu_rotation: rotation frequency ---
if (present(nu_rotation)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='nu_rotation', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, nu_rotation,ident)
  else
    nu_rotation=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'nu_rotation:        ', nu_rotation
  endif
endif
!
! \label{f90:ar_rotationaxis__read}
! --- Read ar_RotationAxis: array with rotation axis ---
if (present(ar_RotationAxis)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='ar_rotationaxis', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  ar_RotationAxis=0.0
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, ar_RotationAxis, ident,mode=2,iup=iup(1:1))
  else
    ar_RotationAxis(3)=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,3(1PE13.6))') 'ar_RotationAxis: ', ar_RotationAxis(1:3)
  endif
endif
!
! \label{f90:side_bound__read}
! --- Read side boundary conditions ---
if (present(side_bound)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='side_bound', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, side_bound,ident)
  else
    side_bound=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Side boundaries:    ' // trim(side_bound)
  endif
endif
!
! \label{f90:top_bound__read}
! --- Read top boundary conditions ---
if (present(top_bound)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='top_bound', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, top_bound,ident)
  else
    top_bound=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Top boundaries:     ' // trim(top_bound)
  endif
endif
!
! \label{f90:bottom_bound__read}
! --- Read bottom boundary conditions ---
if (present(bottom_bound)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='bottom_bound', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, bottom_bound,ident)
  else
    bottom_bound=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Bottom boundaries:  ' // trim(bottom_bound)
  endif
endif
!
! \label{f90:side_bound_mag_x1__read}
! --- Read side boundary conditions for magnetic field ---
if (present(side_bound_mag_x1)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='side_bound_mag_x1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, side_bound_mag_x1,ident)
  else
    side_bound_mag_x1=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Side bound mag x1:  ' // trim(side_bound_mag_x1)
  endif
endif
if (present(side_bound_mag_x2)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='side_bound_mag_x2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, side_bound_mag_x2,ident)
  else
    side_bound_mag_x2=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Side bound mag x2:  ' // trim(side_bound_mag_x2)
  endif
endif
!
! \label{f90:top_bound_mag__read}
! --- Read top boundary conditions for magnetic field ---
if (present(top_bound_mag)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='top_bound_mag', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, top_bound_mag,ident)
  else
    top_bound_mag=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Top bound mag:      ' // trim(top_bound_mag)
  endif
endif
!
! \label{f90:bottom_bound_mag__read}
! --- Read bottom boundary conditions for magnetic field ---
if (present(bottom_bound_mag)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='bottom_bound_mag', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, bottom_bound_mag,ident)
  else
    bottom_bound_mag=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Bottom bound mag:   ' // trim(bottom_bound_mag)
  endif
endif
!
! \label{f90:s_inflow__read}
! --- Read entropy of inflowing material (open lower boundary) ---
if (present(s_InFlow)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='s_inflow', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, s_InFlow,ident)
  else
    s_InFlow=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 's_InFlow:           ', s_InFlow
  endif
endif
!
! \label{f90:s_inflradgrad__read}
! --- Read entropy radial gradient of material in core ---
if (present(s_InFlRadGrad)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='s_inflradgrad', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, s_InFlRadGrad,ident)
  else
    s_InFlRadGrad=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 's_InFlRadGrad:      ', s_InFlRadGrad
  endif
endif
!
! \label{f90:s_infllatgrad__read}
! --- Read entropy lateral gradient of material in core ---
if (present(s_InFlLatGrad)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='s_infllatgrad', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, s_InFlLatGrad,ident)
  else
    s_InFlLatGrad=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 's_InFlLatGrad:      ', s_InFlLatGrad
  endif
endif
!
! \label{f90:c_schange__read}
! --- Read rate of entropy change at bottom (open lower boundary) ---
if (present(C_sChange)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_schange', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_sChange,ident)
  else
    C_sChange=0.5
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_sChange:          ', C_sChange
  endif
endif
!
! \label{f90:c_pchange__read}
! --- Read rate of pressure change at bottom (open lower boundary) ---
if (present(C_PChange)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_pchange', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_PChange,ident)
  else
    C_PChange=0.5
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_PChange:          ', C_PChange
  endif
endif
!
! \label{f90:c_v3changelinbottom__read}
! --- Read rate of vertical velocity change at bottom (open lower boundary) ---
if (present(C_v3ChangeLinBottom)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_v3changelinbottom', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_v3ChangeLinBottom,ident)
  else
    C_v3ChangeLinBottom=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_v3ChangeLinBottom:', C_v3ChangeLinBottom
  endif
endif
!
! \label{f90:c_v3changesqrbottom__read}
! --- Read rate of vertical velocity change at bottom (open lower boundary) ---
if (present(C_v3ChangeSqrBottom)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_v3changesqrbottom', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_v3ChangeSqrBottom,ident)
  else
    C_v3ChangeSqrBottom=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_v3ChangeSqrBottom:', C_v3ChangeSqrBottom
  endif
endif
!
! \label{f90:c_tchange__read}
! --- Read rate of temperature change at top (transmitting boundary) ---
if (present(C_TChange)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_tchange', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_TChange,ident)
  else
    C_TChange=0.5
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_TChange:          ', C_TChange
  endif
endif
!
! \label{f90:c_rhochangetop__read}
! --- Read rate of density change at top (transmitting boundary) ---
if (present(C_rhoChangeTop)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_rhochangetop', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_rhoChangeTop,ident)
  else
    C_rhoChangeTop=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_rhoChangeTop:     ', C_rhoChangeTop
  endif
endif
!
! \label{f90:c_tsurf__read}
! --- Read surface temperature factor ---
if (present(C_Tsurf)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_tsurf', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_Tsurf,ident)
  else
    C_Tsurf=0.81
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_Tsurf:            ', C_Tsurf
  endif
endif
!
! \label{f90:c_tminlimit__read}
! --- Read lower limit temperature ---
if (present(C_TminLimit)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_tminlimit', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_TminLimit,ident)
  else
    C_TminLimit=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_TminLimit:        ', C_TminLimit
  endif
endif
!
! --- Read temperature correction rate ---
if (present(C_TminRate)) then
!  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_tminrate', &
!                identlist=identlist, poslist=poslist, nentry=nentry, &
!                outstr=outstr0,ierr=ierr0)
!  if (ierr0 == 0) then
!    call uio_rd(ncpar, termt,ntt, C_TminRate,ident)
!  else
    C_TminRate=0.0
!  endif
  !if (nc_p >= 0) then
  !  write(nc_p,'(A,1PE13.6)') 'C_TminRate:         ', C_TminRate
  !endif
endif
!
! \label{f90:c_hptopfactor__read}
! --- Read correction factor for surface pressure scale height ---
if (present(C_Hptopfactor)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hptopfactor', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_Hptopfactor,ident)
  else
    C_Hptopfactor=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_Hptopfactor:      ', C_Hptopfactor
  endif
endif
!
! \label{f90:rho_min__read}
! --- Read rho_min: minimum boundary density ---
if (present(rho_min)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='rho_min', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, rho_min,ident)
  else
    rho_min=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'rho_min:            ', rho_min
  endif
endif
!
! \label{f90:luminositypervolume__read}
! --- Read LuminosityPerVolume: luminosity per core volume ---
if (present(LuminosityPerVolume)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='luminositypervolume', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, LuminosityPerVolume,ident)
  else
    LuminosityPerVolume=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'LuminosityPerVolume:', LuminosityPerVolume
  endif
endif
!
! \label{f90:hdcoreheatprofile__read}
! --- Read core heat profile ---
if (present(hdCoreHeatProfile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdcoreheatprofile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdCoreHeatProfile,ident)
    if ((hdCoreHeatProfile == 'constant'      )) hdCoreHeatProfile='Constantdei'
    if ((hdCoreHeatProfile == 'constantdei'   )) hdCoreHeatProfile='Constantdei'
    if ((hdCoreHeatProfile == 'constantdrhoei')) hdCoreHeatProfile='Constantdrhoei'
  else
    hdCoreHeatProfile='Constantdei'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Core heat profile:  ' // trim(hdCoreHeatProfile)
  endif
endif
!
! \label{f90:hdcoredragprofile__read}
! --- Read core drag profile ---
if (present(hdCoreDragProfile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdcoredragprofile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdCoreDragProfile,ident)
    if ((hdCoreDragProfile == 'constant'           )       ) hdCoreDragProfile='Constant'
    if ((hdCoreDragProfile == 'linear'             )       ) hdCoreDragProfile='Linear'
    if ((hdCoreDragProfile == 'cosine'             )       ) hdCoreDragProfile='Cosine'
    if ((hdCoreDragProfile == 'cossqr'             ) .or. &
        (hdCoreDragProfile == 'Cossqr'             )       ) hdCoreDragProfile='CosSqr'
    if ((hdCoreDragProfile == 'constant-radial'    ) .or. &
        (hdCoreDragProfile == 'Constant-radial'    )       ) hdCoreDragProfile='Constant-Radial'
    if ((hdCoreDragProfile == 'linear-radial'      ) .or. &
        (hdCoreDragProfile == 'Linear-radial'      )       ) hdCoreDragProfile='Linear-Radial'
    if ((hdCoreDragProfile == 'cosine-radial'      ) .or. &
        (hdCoreDragProfile == 'Cosine-radial'      )       ) hdCoreDragProfile='Cosine-Radial'
    if ((hdCoreDragProfile == 'cossqr-radial'      ) .or. &
        (hdCoreDragProfile == 'Cossqr-radial'      )       ) hdCoreDragProfile='CosSqr-Radial'
    if ((hdCoreDragProfile == 'constant-meridional') .or. &
        (hdCoreDragProfile == 'Constant-meridional')       ) hdCoreDragProfile='Constant-Meridional'
    if ((hdCoreDragProfile == 'linear-meridional'  ) .or. &
        (hdCoreDragProfile == 'Linear-meridional'  )       ) hdCoreDragProfile='Linear-Meridional'
    if ((hdCoreDragProfile == 'cosine-meridional'  ) .or. &
        (hdCoreDragProfile == 'Cosine-meridional'  )       ) hdCoreDragProfile='Cosine-Meridional'
    if ((hdCoreDragProfile == 'cossqr-meridional'  ) .or. &
        (hdCoreDragProfile == 'Cossqr-meridional'  )       ) hdCoreDragProfile='CosSqr-Meridional'
  else
    hdCoreDragProfile='Constant'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Core drag profile:  ' // trim(hdCoreDragProfile)
  endif
endif
!
! \label{f90:c_coredrag__read}
! --- Read core drag parameter ---
if (present(C_CoreDrag)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_coredrag', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_CoreDrag,ident)
  else
    C_CoreDrag=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_CoreDrag:         ', C_CoreDrag
  endif
endif
!
! \label{f90:b1_inflow__read}
! --- Read horizontal magnetic field component at open lower boundary ---
if (present(B1_inflow)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='b1_inflow', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, B1_inflow, ident)
  else
    B1_inflow=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'B1_inflow:          ', B1_inflow
  endif
endif
!
! \label{f90:c_magthetab__read}
! --- Read theta angle of magnetic field for oblique boundary condition ---
if (present(C_magthetaB)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_magthetab', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_magthetaB, ident)
  else
    C_magthetaB=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_magthetaB:        ', C_magthetaB
  endif
endif
!
! \label{f90:c_magphib__read}
! --- Read phi angle of magnetic field for oblique boundary condition ---
if (present(C_magphiB)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_magphib', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_magphiB, ident)
  else
    C_magphiB=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_magphiB:          ', C_magphiB
  endif
endif
!
! \label{f90:n_magdiffside__read}
! --- Read number of cells for diffusive side layers ---
if (present(N_magDiffSide)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_magdiffside', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_magDiffSide, ident)
    N_magDiffSide=max(N_magDiffSide, 0)
  else
    N_magDiffSide=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_magDiffSide:      ', N_magDiffSide
  endif
endif
!
! \label{f90:n_magdifftop__read}
! --- Read number of cells for diffusive top layer ---
if (present(N_magDiffTop)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_magdifftop', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_magDiffTop, ident)
    N_magDiffTop=max(N_magDiffTop, 0)
  else
    N_magDiffTop=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_magDiffTop:       ', N_magDiffTop
  endif
endif
!
! --- Read number of cells for diffusive bottom layer ---
if (present(N_magDiffBottom)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_magdiffbottom', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_magDiffBottom, ident)
    N_magDiffBottom=max(N_magDiffBottom, 0)
  else
    N_magDiffBottom=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_magDiffBottom:    ', N_magDiffBottom
  endif
endif
!
! --- Read hydrogen abundance ---
if (present(abuX)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='abux', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, abuX,ident)
  else
!!$    outstr0='hydrogen abundance is not specified'
!!$    ierr0=3
!!$    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
    abuX=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,F15.12)')  'abuX:               ', abuX
  endif
endif
!
! --- Read helium abundance ---
if (present(abuY)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='abuy', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, abuY,ident)
  else
!!$    outstr0='helium abundance is not specified'
!!$    ierr0=4
!!$    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
    abuY=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,F15.12)')  'abuY:               ', abuY
  endif
endif
!
! --- Compute metal abundance ---
if (present(abuZ)) then
  if (present(abuX) .and. present(abuY)) then
    abuZ=1.0-abuX-abuY
    if (nc_p >= 0) then
      write(nc_p,'(A,F15.12)')  'abuZ:               ', abuZ
    endif
  else
    outstr0='metal abundance can not be computed'
    ierr0=4
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
  endif
endif
!
! --- Read mean molecular weight ---
if (present(Qmol)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='qmol', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, Qmol,ident)
  else
!!$    outstr0='mean molecular weight is not specified'
!!$    ierr0=5
!!$    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
    Qmol=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'Qmol:               ', Qmol
  endif
endif
!
! --- Read adiabatic coefficient ---
if (present(gamma)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='gamma', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, gamma,ident)
  else
    gamma=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'gamma:              ', gamma
  endif
endif
!
! \label{f90:eosfile__read}
! --- Read EOS file name ---
if (present(EOSFile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='eosfile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, EOSFile,ident)
  else
    outstr0='EOS file name is not specified'
    ierr0=5
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'EOS file name:      ' // trim(EOSFile)
  endif
endif
!
! \label{f90:eospath__read}
! --- Read EOS path ---
if (present(EOSPath)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='eospath', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, EOSPath,ident)
  else
    EOSPath=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'EOS path name:      ' // trim(EOSPath)
  endif
endif
!
! \label{f90:kappamin__read}
! --- Read minimum opacity ---
if (present(kappaMin)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='kappamin', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, kappaMin,ident)
  else
    kappaMin=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'kappaMin:           ', kappaMin
  endif
endif
!
! \label{f90:kappamax__read}
! --- Read maximum opacity ---
if (present(kappaMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='kappamax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, kappaMax,ident)
  else
    kappaMax=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'kappaMax:           ', kappaMax
  endif
endif
!
! \label{f90:opafile__read}
! --- Read opacity file name ---
if (present(OpaFile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='opafile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OpaFile,ident)
  else
    outstr0='opacity file name is not specified'
    ierr0=5
    call uio_err(nc=ncpar,outstr=outstr0,ierr=ierr0,routine='rhd_par_RdData', errlevel=3)
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'opacity file name:  ' // trim(OpaFile)
  endif
endif
!
! \label{f90:opapath__read}
! --- Read opacity path ---
if (present(OpaPath)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='opapath', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OpaPath,ident)
  else
    OpaPath=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'opacity path name:  ' // trim(OpaPath)
  endif
endif
!
! --- Read abundance of representative metal ---
if (present(chem_abumetal)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='chem_abumetal', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, chem_abumetal,ident)
  else
    chem_abumetal=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'chem_abumetal:      ', chem_abumetal
  endif
endif
!
! --- Read chemical reaction network data file name ---
if (present(chem_reacfile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='chem_reacfile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, chem_reacfile,ident)
  else
    chem_reacfile=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'reaction file name: ' // trim(chem_reacfile)
  endif
endif
!
! --- Read chemical reaction network data file path ---
if (present(chem_reacpath)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='chem_reacpath', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, chem_reacpath,ident)
  else
    chem_reacpath=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'reaction path name: ' // trim(chem_reacpath)
  endif
endif
!
! --- Read HION data file path ---
if (present(hion_datapath)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hion_datapath', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hion_datapath,ident)
  else
    hion_datapath=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'HION data path:     ' // trim(hion_datapath)
  endif
endif
!
! --- Read HION file name for model atom ---
if (present(hion_atomfile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hion_atomfile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hion_atomfile,ident)
  else
    hion_atomfile=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'HION atom file:     ' // trim(hion_atomfile)
  endif
endif
!
! --- Read HION file name for abundances ---
if (present(hion_abufile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hion_abufile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hion_abufile,ident)
  else
    hion_abufile=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'HION abundance file:' // trim(hion_abufile)
  endif
endif
!
! --- Read HION file name for electron densities ---
if (present(hion_edensfile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hion_edensfile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hion_edensfile,ident)
  else
    hion_edensfile=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'HION el.dens. file: ' // trim(hion_edensfile)
  endif
endif
!
! --- Read HION file name for partition functions ---
if (present(hion_pffile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hion_pffile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hion_pffile,ident)
  else
    hion_pffile=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'HION part.fnc. file: ' // trim(hion_pffile)
  endif
endif
!
! --- Read HION output time increment ---
if (present(dtime_out_hion)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_out_hion', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_out_hion,ident)
  else
    dtime_out_hion=-1.0 ! no output
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_out_hion:     ', dtime_out_hion
  endif
endif
!
! --- Read HION chunk number ---
if (present(hion_chunks)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='hion_chunks', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hion_chunks,ident)
  else
    hion_chunks=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'hion_chunks:        ', hion_chunks
  endif
endif
!
! \label{f90:hdscheme__read}
! --- Read hydrodynamics scheme ---
if (present(hdScheme)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdscheme', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdScheme,ident)
  else
    hdScheme='Roe'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdScheme:           ' // trim(hdScheme)
  endif
endif
!
! \label{f90:hdtimeintegrationscheme__read}
! --- Read hydrodynamics time-integration scheme ---
if (present(hdTimeIntegrationScheme)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdtimeintegrationscheme', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdTimeIntegrationScheme,ident)
  else
    if (present(hdScheme)) then
      if ((hdScheme == 'roe') .or. (hdScheme == 'Roe') .or. (hdScheme == 'ROE')) then
        hdTimeIntegrationScheme='Single'
      else
        hdTimeIntegrationScheme='Hancock'
      endif
    else
      hdTimeIntegrationScheme='Hancock'
    endif
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdTimeIntegration:  ' // trim(hdTimeIntegrationScheme)
  endif
endif
!
! \label{f90:hdsplit__read}
! --- Read hydrodynamics splitting scheme ---
if (present(hdsplit)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdsplit', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdsplit,ident)
  else
    hdsplit='123'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdsplit:            ' // trim(hdsplit)
  endif
endif
!
! \label{f90:hdtmpstructkeepmode__read}
! --- Read retaining mode for temporary structures ---
if (present(hdTmpStructKeepMode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdtmpstructkeepmode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdTmpStructKeepMode,ident)
    if ((hdTmpStructKeepMode == 'delete') .or. &
        (hdTmpStructKeepMode == 'Delete')     ) then
      hdTmpStructKeepMode='Delete'
    else
      hdTmpStructKeepMode='Keep'
    endif
  else
    hdTmpStructKeepMode='Keep'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdTmpStructKeepMode:' // trim(hdTmpStructKeepMode)
  endif
endif
!
! \label{f90:hdcheckflux__read}
! --- Read if CheckFlux should be activated ---
if (present(hdCheckFlux)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdcheckflux', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdCheckFlux,ident)
    if ((hdCheckFlux == 'off' ) .or. (hdCheckFlux == 'Off' ) .or. (hdCheckFlux == '0') .or. &
        (hdCheckFlux == 'none') .or. (hdCheckFlux == 'None')     ) then
      hdCheckFlux='Off'
    else
      hdCheckFlux='On'
    endif
  else
    hdCheckFlux='On'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdCheckFlux:        ' // trim(hdCheckFlux)
  endif
endif
!
! \label{f90:reconstruction__read}
! --- Read HD reconstruction method ---
if (present(reconstruction)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='reconstruction', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, reconstruction,ident)
  else
    reconstruction='constant'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Reconstruction:     ' // trim(reconstruction)
  endif
endif
!
! \label{f90:hdtransvelomode__read}
! --- Read mode for transverse velocity advection ---
if (present(hdTransVeloMode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdtransvelomode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdTransVeloMode,ident)
    if (hdTransVeloMode == 'normal') hdTransVeloMode='Normal'
  else
    hdTransVeloMode='Normal'
  endif
  if (hdTransVeloMode == '') hdTransVeloMode='Normal'
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdTransVeloMode:    ' // trim(hdTransVeloMode)
  endif
endif
!
! \label{f90:hdentropywavemode__read}
! --- Read mode for entropy-wave treatment ---
if (present(hdEntropyWaveMode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdentropywavemode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdEntropyWaveMode,ident)
    if (hdEntropyWaveMode == 'normal') hdEntropyWaveMode='Normal'
  else
    hdEntropyWaveMode='Normal'
  endif
  if (hdEntropyWaveMode == '') hdEntropyWaveMode='Normal'
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdEntropyWaveMode:  ' // trim(hdEntropyWaveMode)
  endif
endif
!
! \label{f90:hdenthalpyavgmode__read}
! --- Read mode for enthalpy-averaging formula ---
if (present(hdEnthalpyAvgMode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='hdenthalpyavgmode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, hdEnthalpyAvgMode,ident)
    if (hdEnthalpyAvgMode == 'normal') hdEnthalpyAvgMode='Normal'
  else
    hdEnthalpyAvgMode='Normal'
  endif
  if (hdEnthalpyAvgMode == '') hdEnthalpyAvgMode='Normal'
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'hdEnthalpyAvgMode:  ' // trim(hdEnthalpyAvgMode)
  endif
endif
!
! \label{f90:n_orderconstrainedtransport__read}
! --- Read order of constrained transport ---
if (present(N_orderConstrainedTransport)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_orderconstrainedtransport', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_orderConstrainedTransport, ident)
  else
    N_orderConstrainedTransport=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I6)'    ) 'N_orderConstrainedTransport: ', N_orderConstrainedTransport
  endif
endif
!
! \label{f90:n_hyditer__read}
! --- Read number of hydro iterations ---
if (present(N_hydIter)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_hyditer', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_hydIter, ident)
    N_hydIter=max(N_hydIter, 0)
  else
    N_hydIter=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_hydIter:          ', N_hydIter
  endif
endif
!
! \label{f90:n_hydmaxiter__read}
! --- Read maximum number of hydro iterations ---
if (present(N_hydMaxIter)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_hydmaxiter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_hydMaxIter, ident)
    N_hydMaxIter=max(N_hydMaxIter, 0)
  else
    N_hydMaxIter=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_hydMaxIter:       ', N_hydMaxIter
  endif
endif
!
! \label{f90:n_hydrkiter__read}
! --- Read maximum number of hydro iterations ---
if (present(N_hydRKIter)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_hydrkiter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_hydRKIter, ident)
    N_hydRKIter=min(max(N_hydRKIter, 2), 11)
  else
    N_hydRKIter=2
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_hydRKIter:        ', N_hydRKIter
  endif
endif
!
! \label{f90:c_hydtdifflin__read}
! --- Read hydro Temperature diffusion parameter 1 ---
if (present(C_hydTDiffLin)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydtdifflin', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydTDiffLin, ident)
    if (C_hydTDiffLin <= 0.0) C_hydTDiffLin=0.0
  else
    C_hydTDiffLin=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydTDiffLin:      ', C_hydTDiffLin
  endif
endif
!
! \label{f90:c_hydtdiffmach__read}
! --- Read hydro Temperature diffusion parameter 2 ---
if (present(C_hydTDiffMach)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydtdiffmach', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydTDiffMach, ident)
    if (C_hydTDiffMach <= 0.0) C_hydTDiffMach=0.0
  else
    C_hydTDiffMach=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydTDiffMach:     ', C_hydTDiffMach
  endif
endif
!
! \label{f90:c_hydsdiffvelo__read}
! --- Read hydro entropy diffusion parameter ---
if (present(C_hydsDiffVelo)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydsdiffvelo', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydsDiffVelo, ident)
    if (C_hydsDiffVelo <= 0.0) C_hydsDiffVelo=0.0
  else
    C_hydsDiffVelo=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydsDiffVelo:     ', C_hydsDiffVelo
  endif
endif
!
! \label{f90:c_hydvdiffvelo__read}
! --- Read hydro momentum diffusion parameter ---
if (present(C_hydvDiffVelo)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydvdiffvelo', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydvDiffVelo, ident)
    if (C_hydvDiffVelo <= 0.0) C_hydvDiffVelo=0.0
  else
    C_hydvDiffVelo=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydvDiffVelo:     ', C_hydvDiffVelo
  endif
endif
!
! \label{f90:c_hydlowmachcsfactor__read}
! --- Read low-Mach-number sound-speed factor ---
if (present(C_hydLowMachcsFactor)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydlowmachcsfactor', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydLowMachcsFactor, ident)
  else
    C_hydLowMachcsFactor=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydLowMachcsFact.:', C_hydLowMachcsFactor
  endif
endif
!
! \label{f90:c_hydlowmachcsbase__read}
! --- Read low-Mach-number sound-speed base value ---
if (present(C_hydLowMachcsBase)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydlowmachcsbase', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydLowMachcsBase, ident)
    C_hydLowMachcsBase=min(max(C_hydLowMachcsBase, 0.0), 1.0)
  else
    C_hydLowMachcsBase=1.0E-06
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydLowMachcsBase: ', C_hydLowMachcsBase
  endif
endif
!
! \label{f90:c_hydsmallfluctfactor__read}
! --- Read small-fluctuation factor ---
if (present(C_hydSmallFluctFactor)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydsmallfluctfactor', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydSmallFluctFactor, ident)
    C_hydSmallFluctFactor=max(C_hydSmallFluctFactor, 0.0)
  else
    C_hydSmallFluctFactor=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydSmallFluctFac.:', C_hydSmallFluctFactor
  endif
endif
!
! \label{f90:n_hydcellsperchunk__read}
! --- Read number of cells per chunk (hydro) ---
if (present(N_hydCellsPerChunk)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_hydcellsperchunk', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_hydCellsPerChunk, ident)
    N_hydCellsPerChunk=max(N_hydCellsPerChunk, 0)
  else
    N_hydCellsPerChunk=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_hydCellsPerChunk: ', N_hydCellsPerChunk
  endif
endif
!
! \label{f90:n_hydcellsperchunk2__read}
! --- Read number of cells per chunk (hydro) ---
if (present(N_hydCellsPerChunk2)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_hydcellsperchunk2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_hydCellsPerChunk2, ident)
    if (N_hydCellsPerChunk2 <= 0) then
      if (present(N_hydCellsPerChunk)) then
        N_hydCellsPerChunk2=N_hydCellsPerChunk
      else
        N_hydCellsPerChunk2=0
      endif
    else
      N_hydCellsPerChunk2=max(N_hydCellsPerChunk2, 0)
    endif
  else
    if (present(N_hydCellsPerChunk)) then
      N_hydCellsPerChunk2=N_hydCellsPerChunk
    else
      N_hydCellsPerChunk2=0
    endif
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_hydCellsPerChunk2:', N_hydCellsPerChunk2
  endif
endif
!
! \label{f90:n_hydcellsperchunk3__read}
! --- Read number of cells per chunk (hydro) ---
if (present(N_hydCellsPerChunk3)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_hydcellsperchunk3', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_hydCellsPerChunk3, ident)
    if (N_hydCellsPerChunk3 <= 0) then
      if (present(N_hydCellsPerChunk)) then
        N_hydCellsPerChunk3=N_hydCellsPerChunk
      else
        N_hydCellsPerChunk3=0
      endif
    else
      N_hydCellsPerChunk3=max(N_hydCellsPerChunk3, 0)
    endif
  else
    if (present(N_hydCellsPerChunk)) then
      N_hydCellsPerChunk3=N_hydCellsPerChunk
    else
      N_hydCellsPerChunk3=0
    endif
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_hydCellsPerChunk3:', N_hydCellsPerChunk3
  endif
endif
!
! \label{f90:n_viscellsperchunk__read}
! --- Read number of cells per chunk (viscosity) ---
if (present(N_visCellsPerChunk)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_viscellsperchunk', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_visCellsPerChunk, ident)
    if (N_visCellsPerChunk <= 0) N_visCellsPerChunk=0
  else
    N_visCellsPerChunk=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_visCellsPerChunk: ', N_visCellsPerChunk
  endif
endif
!
! \label{f90:n_rescellsperchunk__read}
! --- Read number of cells per chunk (resistivity) ---
if (present(N_resCellsPerChunk)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_rescellsperchunk', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_resCellsPerChunk, ident)
    if (N_resCellsPerChunk <= 0) N_resCellsPerChunk=0
  else
    N_resCellsPerChunk=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_resCellsPerChunk: ', N_resCellsPerChunk
  endif
endif
!
! \label{f90:n_radcellsperchunk__read}
! --- Read number of cells per chunk (radiation transport) ---
if (present(N_radCellsPerChunk)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radCellsPerChunk', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radCellsPerChunk, ident)
    if (N_radCellsPerChunk <= 0) N_radCellsPerChunk=0
  else
    N_radCellsPerChunk=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radCellsPerChunk: ', N_radCellsPerChunk
  endif
endif
!
! \label{f90:an_mpiprocsperdim__read}
! --- Read 3-element integer array with MPI process numbers per dimension ---
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated: read array ------- */
  if (present(aN_MPIProcsPerDim)) then
    call uio_srhd(ncpar, termt,ntt, type=intena, ident='an_mpiprocsperdim', &
                  identlist=identlist, poslist=poslist, nentry=nentry, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncpar, termt,ntt, aN_MPIProcsPerDim, ident)
    else
      aN_MPIProcsPerDim=1
    endif
    if (nc_p >= 0) then
      write(nc_p,'(A,3(I5))'  ) 'aN_MPIProcsPerDim:  ', aN_MPIProcsPerDim
    endif
  endif
!#
# else
!#     /* rhd_mpi_l01: MPI not activated: set array to default values */
  if (present(aN_MPIProcsPerDim)) then
    aN_MPIProcsPerDim=1
  endif
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
! \label{f90:c_slopered__read}
! --- Read slope reduction parameter ---
if (present(C_SlopeRed)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_slopered', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_SlopeRed, ident)
    if (C_SlopeRed <= 0.0) C_SlopeRed=0.0
  else
    C_SlopeRed=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_SlopeRed:         ', C_SlopeRed
  endif
endif
!
! \label{f90:c_reccontshift__read}
! --- Read enhanced reconstruction continuation parameter 1: shift ---
if (present(C_RecContShift)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_reccontshift', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_RecContShift, ident)
    if (C_RecContShift <= 0.0) C_RecContShift=0.0
  else
    C_RecContShift=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_RecContShift:     ', C_RecContShift
  endif
endif
!
! \label{f90:c_reccontsteep__read}
! --- Read enhanced reconstruction continuation parameter 2: steepen ---
if (present(C_RecContSteep)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_reccontsteep', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_RecContSteep, ident)
    if (C_RecContSteep <= 0.0) C_RecContSteep=0.0
  else
    C_RecContSteep=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_RecContSteep:     ', C_RecContSteep
  endif
endif
!
! \label{f90:c_recwenoweightcenter__read}
! --- Read weight of central polynomial for WENO reconstruction ---
if (present(C_RecWenoWeightCenter)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_recwenoweightcenter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_RecWenoWeightCenter, ident)
    C_RecWenoWeightCenter=min(max(C_RecWenoWeightCenter, 0.0), 1.0)
  else
    C_RecWenoWeightCenter=0.6
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_RecWenoWeightCent:', C_RecWenoWeightCenter
  endif
endif
!
! \label{f90:c_recwenoweightright__read}
! --- Read weight of right polynomial for WENO reconstruction ---
if (present(C_RecWenoWeightRight)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_recwenoweightright', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_RecWenoWeightRight, ident)
    C_RecWenoWeightRight=min(max(C_RecWenoWeightRight, 0.0), 1.0)
  else
    C_RecWenoWeightRight=0.6666666
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_RecWenoWeightRigh:', C_RecWenoWeightRight
  endif
endif
!
! \label{f90:c_recwenopsifactor__read}
! --- Read additional smoothing factor in case of low pairwise smoothness for WENO reconstruction --
if (present(C_RecWenoPsiFactor)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_recwenopsifactor', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_RecWenoPsiFactor, ident)
    C_RecWenoPsiFactor=max(C_RecWenoPsiFactor, 0.0)
  else
    C_RecWenoPsiFactor=4.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_RecWenoPsiFactor: ', C_RecWenoPsiFactor
  endif
endif
!
! \label{f90:c_hydpredfactor__read}
! --- Read hydrostatic pressure reduction parameter ---
if (present(C_hydPredfactor)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydpredfactor', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydPredfactor, ident)
    if (C_hydPredfactor <= 0.0) C_hydPredfactor=0.0
  else
    C_hydPredfactor=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydPredfactor:    ', C_hydPredfactor
  endif
endif
!
! \label{f90:c_visbound__read}
! --- Read viscosity constant ---
if (present(C_visBound)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visbound', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visBound, ident)
    if (C_visBound <= 0.0) C_visBound=0.0
  else
    C_visBound=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visBound:         ', C_visBound
  endif
endif
!
! \label{f90:beta_inv__read}
! --- Read inverse plasma beta for dual energy method ---
if (present(beta_inv)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='beta_inv', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, beta_inv, ident)
  else
    beta_inv=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'beta_inv:           ', beta_inv
  endif
endif
!
! \label{f90:va_max__read}
! --- Read maximum Alfven speed ---
if (present(va_max)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='va_max', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, va_max, ident)
  else
    va_max=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'va_max:             ', va_max
  endif
endif
!
! \label{f90:dustscheme__read}
! --- Read dust scheme ---
if (present(dustScheme)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='dustscheme', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dustScheme,ident)
  else
    dustScheme='None'
  endif
  if (dustScheme == 'none') dustScheme='None'
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'dustScheme:         ' // trim(dustScheme)
  endif
endif
!
! \label{f90:dustreconstruction__read}
! --- Read dust (in general: source term) reconstruction method ---
if (present(dustReconstruction)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='dustreconstruction', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dustReconstruction,ident)
  else
    dustReconstruction=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'DustReconstruction: ' // trim(dustReconstruction)
  endif
endif
!
! \label{f90:dustopafile__read}
! --- Read dust opacity file name ---
if (present(dustOpaFile)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='dustopafile', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dustOpaFile,ident)
  else
    dustOpaFile=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'dustOpaFile:        ' // trim(dustOpaFile)
  endif
endif
!
! \label{f90:dustopamode__read}
! --- Read dust opacity mode ---
if (present(dustopamode)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='dustopamode', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dustopamode,ident)
  else
    dustopamode=''
  endif
  if (dustopamode == 'none') dustopamode='None'
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'dustopamode:        ' // trim(dustopamode)
  endif
endif
!
! \label{f90:ar_dustGrainRadius__read}
! --- Read array with dust grain radii ---
if ((present(n_dustGrainRadius)) .and. (present(ar_dustGrainRadius))) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='ar_dustgrainradius', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  ar_dustGrainRadius=0.0
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, ar_dustGrainRadius, ident,mode=2,iup=iup(1:1))
    n_dustGrainRadius=iup(1)
  else
    n_dustGrainRadius=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,5(1PE15.8))') 'ar_dustGrainRadius: ', ar_dustGrainRadius(1:n_dustGrainRadius)
  endif
endif
!
! \label{f90:ar_dustGrainAbu__read}
! --- Read array with dust grain abundances ---
if ((present(n_dustGrainAbu)) .and. (present(ar_dustGrainAbu))) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='ar_dustgrainabu', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  ar_dustGrainAbu=0.0
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, ar_dustGrainAbu, ident,mode=2,iup=iup(1:1))
    n_dustGrainAbu=iup(1)
  else
    n_dustGrainAbu=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,5(1PE15.8))') 'ar_dustGrainAbu:    ', ar_dustGrainAbu(1:n_dustGrainAbu)
  endif
endif
!
! \label{f90:c_dust01__read}
! --- Read dust parameters ---
if (present(C_dust01)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust01', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust01, ident)
  else
    C_dust01=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust01:           ', C_dust01
  endif
endif
if (present(C_dust02)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust02', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust02, ident)
  else
    C_dust02=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust02:           ', C_dust02
  endif
endif
if (present(C_dust03)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust03', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust03, ident)
  else
    C_dust03=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust03:           ', C_dust03
  endif
endif
if (present(C_dust04)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust04', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust04, ident)
  else
    C_dust04=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust04:           ', C_dust04
  endif
endif
if (present(C_dust05)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust05', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust05, ident)
  else
    C_dust05=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust05:           ', C_dust05
  endif
endif
if (present(C_dust06)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust06', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust06, ident)
  else
    C_dust06=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust06:           ', C_dust06
  endif
endif
if (present(C_dust07)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust07', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust07, ident)
  else
    C_dust07=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust07:           ', C_dust07
  endif
endif
if (present(C_dust08)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust08', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust08, ident)
  else
    C_dust08=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust08:           ', C_dust08
  endif
endif
if (present(C_dust09)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust09', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust09, ident)
  else
    C_dust09=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust09:           ', C_dust09
  endif
endif
if (present(C_dust10)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_dust10', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_dust10, ident)
  else
    C_dust10=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_dust10:           ', C_dust10
  endif
endif
!
! \label{f90:radscheme__read}
! --- Read radiation transport scheme ---
if (present(radScheme)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='radscheme', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, radScheme,ident)
  else
    radScheme='None'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'radScheme:          ' // trim(radScheme)
  endif
endif
!
! \label{f90:radthinheatscheme__read}
! --- Read radiation local-source scheme ---
if (present(radThinHeatScheme)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='radthinheatscheme', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, radThinHeatScheme,ident)
  else
    radThinHeatScheme='None'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'radThinHeatScheme:  ' // trim(radThinHeatScheme)
  endif
endif
!
! \label{f90:radthinpressscheme__read}
! --- Read radiation local-source scheme ---
if (present(radThinPressScheme)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='radthinpressscheme', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, radThinPressScheme,ident)
  else
    radThinPressScheme='None'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'radThinPressScheme: ' // trim(radThinPressScheme)
  endif
endif
!
! \label{f90:n_radminiter__read}
! --- Read minimum number of radiation iterations ---
if (present(N_radminiter)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radminiter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radminiter, ident)
    if (N_radminiter <= 0) N_radminiter=1
  else
    N_radminiter=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radminiter:       ', N_radminiter
  endif
endif
!
! \label{f90:n_raditer__read}
! --- Read number of radiation iterations ---
if (present(N_raditer)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_raditer', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_raditer, ident)
    if (N_raditer <= 0) N_raditer=1
  else
    N_raditer=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_raditer:          ', N_raditer
  endif
endif
!
! \label{f90:n_radmaxiter__read}
! --- Read maximum number of radiation iterations ---
if (present(N_radmaxiter)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radmaxiter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radmaxiter, ident)
    if (N_radmaxiter < 0) N_radmaxiter=0
  else
    N_radmaxiter=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radmaxiter:       ', N_radmaxiter
  endif
endif
!
! \label{f90:n_radtaurefine__read}
! --- Grid refinement factor near large temperature steps ---
if (present(N_radtaurefine)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radtaurefine', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radtaurefine, ident)
!!$    N_radtaurefine=((max(N_radtaurefine,1)-1)/2)*2+1
  else
!!$    N_radtaurefine=1
    N_radtaurefine=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radtaurefine:     ', N_radtaurefine
  endif
endif
!
! \label{f90:n_radthinpoint__read}
! --- Number of thin grid points ---
if (present(N_radThinPoint)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radthinpoint', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radThinPoint, ident)
    N_radThinPoint=max(N_radThinPoint, 0)
  else
    N_radThinPoint=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radThinPoint:     ', N_radThinPoint
  endif
endif
!
! \label{f90:n_radthinheatpoint__read}
! --- Number of additional grid points with optically thin heating ---
if (present(N_radThinHeatPoint)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radthinheatpoint', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radThinHeatPoint, ident)
    N_radThinHeatPoint=max(N_radThinHeatPoint, 0)
  else
    N_radThinHeatPoint=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radThinHeatPoint: ', N_radThinHeatPoint
  endif
endif
!
! \label{f90:n_radthinpresspoint__read}
! --- Number of additional grid points with optically thin radiative acceleration ---
if (present(N_radThinPressPoint)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radthinpresspoint', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radThinPressPoint, ident)
    N_radThinPressPoint=max(N_radThinPressPoint, 0)
  else
    N_radThinPressPoint=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radThinPressPoint:', N_radThinPressPoint
  endif
endif
!
! \label{f90:n_radthickpoint__read}
! --- Number of thick grid points ---
if (present(N_radThickPoint)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radthickpoint', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radThickPoint, ident)
  else
    N_radThickPoint=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radThickPoint:    ', N_radThickPoint
  endif
endif
!
! \label{f90:n_radtheta__read}
! --- Number of ray sets in theta direction ---
if (present(N_radtheta)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radtheta', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radtheta, ident)
  else
    N_radtheta=2
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radtheta:         ', N_radtheta
  endif
endif
!
! \label{f90:n_radphi__read}
! --- Number of ray sets in phi direction ---
if (present(N_radphi)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radphi', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radphi, ident)
  else
    N_radphi=2
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radphi:           ', N_radphi
  endif
endif
!
! \label{f90:n_radsubray__read}
! --- Number rays per cell ---
if (present(N_radsubray)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radsubray', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radsubray, ident)
  else
    N_radsubray=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radsubray:        ', N_radsubray
  endif
endif
!
! \label{f90:n_radband__read}
! --- Number of frequency bands ---
if (present(N_radband)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radband', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radband, ident)
  else
    N_radband=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radband:          ', N_radband
  endif
endif
!
! \label{f90:n_radrsyslevel__read}
! --- LCFrad ray control ---
if (present(N_radrsyslevel)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radrsyslevel', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radrsyslevel, ident)
  else
    N_radrsyslevel=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radrsyslevel:     ', N_radrsyslevel
  endif
endif
!
! \label{f90:n_radoutput__read}
! --- LCFrad and MSCrad additional output ---
if (present(N_radoutput)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radoutput', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radoutput, ident)
  else
    N_radoutput=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radoutput:        ', N_radoutput
  endif
endif
!
! \label{f90:n_radfeautrier__read}
! --- LCFrad Feautrier scheme ---
if (present(N_radFeautrier)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_radfeautrier', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_radFeautrier, ident)
  else
    N_radFeautrier=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)'    ) 'N_radFeautrier:     ', N_radFeautrier
  endif
endif
!
! \label{f90:c_radimplicitmu__read}
! --- Read implicitness parameter ---
if (present(C_radimplicitmu)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radimplicitmu', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radimplicitmu, ident)
    if (C_radimplicitmu <= 0.0) C_radimplicitmu=0.0
  else
    C_radimplicitmu=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radimplicitmu:    ', C_radimplicitmu
  endif
endif
!
! \label{f90:c_raditereps__read}
! --- Read radiation iteration accuracy ---
if (present(C_raditereps)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_raditereps', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_raditereps, ident)
    if (C_raditereps <= 0.0) C_raditereps=0.0
  else
    C_raditereps=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_raditereps:       ', C_raditereps
  endif
endif
!
! \label{f90:c_raditerstep__read}
! --- Read radiation iteration step ---
if (present(C_raditerstep)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_raditerstep', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_raditerstep, ident)
    if (C_raditerstep <= 0.0) C_raditerstep=0.0
  else
    C_raditerstep=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_raditerstep:      ', C_raditerstep
  endif
endif
!
! \label{f90:c_radkappasmooth__read}
! --- Opacity smoothing parameter ---
if (present(C_radkappasmooth)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radkappasmooth', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radkappasmooth, ident)
    C_radkappasmooth=min(max(C_radkappasmooth,0.0),1.0)
  else
    C_radkappasmooth=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radkappasmooth:   ', C_radkappasmooth
  endif
endif
!
! \label{f90:c_radtstep__read}
! --- Temperature interpolating parameter ---
if (present(C_radTstep)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtstep', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radTstep, ident)
  else
    C_radTstep=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radTstep:         ', C_radTstep
  endif
endif
!
! \label{f90:c_radtsmooth__read}
! --- Temperature smoothing parameter ---
if (present(C_radTsmooth)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtsmooth', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radTsmooth, ident)
    C_radTsmooth=min(max(C_radTsmooth,0.0),1.0)
  else
    C_radTsmooth=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radTsmooth:       ', C_radTsmooth
  endif
endif
!
! \label{f90:c_radtintminfac__read}
! --- Parameter for temperature interpolation on ray ---
if (present(C_radTintminfac)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtintminfac', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radTintminfac, ident)
    C_radTintminfac=max(C_radTintminfac,0.0)
  else
    C_radTintminfac=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radTintminfac:    ', C_radTintminfac
  endif
endif
!
! \label{f90:c_radhtautop__read}
! --- Read scale height of optical depth at top ---
if (present(C_radHtautop)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radhtautop', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radHtautop,ident)
  else
    call uio_srhd(ncpar, termt,ntt, type=realna, ident='htautop', &
                  identlist=identlist, poslist=poslist, nentry=nentry, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncpar, termt,ntt, C_radHtautop,ident)
    else
      C_radHtautop=0.0
    endif
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE13.6)') 'C_radHtautop:       ', C_radHtautop
  endif
endif
!
! --- Parameter for delta_tau limit for extra 'radiative temperature viscosity' ---
if (present(C_radTvisdtau)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtvisdtau', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radTvisdtau, ident)
  else
    C_radTvisdtau=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radTvisdtau:      ', C_radTvisdtau
  endif
endif
!
! --- Parameter for extra 'radiative temperature viscosity' ---
if (present(C_radTvis)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtvis', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radTvis, ident)
  else
    C_radTvis=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radTvis:          ', C_radTvis
  endif
endif
!
! --- Parameter for 'time scale reduction in optically thin' in LHDrad ---
if (present(C_radthintimefac)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radthintimefac', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radthintimefac, ident)
  else
    C_radthintimefac=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radthintimefac:   ', C_radthintimefac
  endif
endif
!
! \label{f90:c_radthintimerelax__read}
! --- Parameter for 'relaxation time scale in optically thin' in SHORTRAD---
if (present(C_radthintimeRelax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radthintimerelax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radthintimeRelax, ident)
  else
    C_radthintimeRelax=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radthintimeRelax: ', C_radthintimeRelax
  endif
endif
!
! \label{f90:c_radthintfactor__read}
! --- Temperature-correction factor in optically thin in SHORTRAD---
if (present(C_radthinTFactor)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radthintfactor', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radthinTFactor, ident)
  else
    C_radthinTFactor=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radthinTFactor:   ', C_radthinTFactor
  endif
endif
!
! \label{f90:c_radtcool__read}
! --- Parameter for LCFrad and MSCrad surface cooling option ---
if (present(C_radtcool)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtcool', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radtcool, ident)
  else
    C_radtcool=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radtcool:         ', C_radtcool
  endif
endif
!
! \label{f90:c_radscool__read}
! --- Parameter for LCFrad and MSCrad surface cooling option ---
if (present(C_radscool)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radscool', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radscool, ident)
  else
    C_radscool=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radscool:         ', C_radscool
  endif
endif
!
! \label{f90:c_raddcool__read}
! --- Parameter for LCFrad and MSCrad surface cooling option ---
if (present(C_raddcool)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_raddcool', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_raddcool, ident)
  else
    C_raddcool=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_raddcool:         ', C_raddcool
  endif
endif
!
! \label{f90:c_radtinci__read}
! --- Parameter for LCFrad and MCSrad incident radiation ---
if (present(C_radTinci)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radtinci', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radTinci, ident)
  else
    C_radTinci=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radTinci:         ', C_radTinci
  endif
endif
!
! \label{f90:c_raddinci__read}
! --- Parameter for LCFrad and MCSrad incident radiation ---
if (present(C_radDinci)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_raddinci', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radDinci, ident)
  else
    C_radDinci=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radDinci:         ', C_radDinci
  endif
endif
!
! \label{f90:c_radaccenergyfraction__read}
! --- Parameter for LCFrad and MCSrad incident radiation ---
if (present(C_radAccEnergyFraction)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radaccenergyfraction', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radAccEnergyFraction, ident)
    C_radAccEnergyFraction=min(max(C_radAccEnergyFraction, 0.0), 1.0)
  else
    C_radAccEnergyFraction=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radAccEnergyFraction:', C_radAccEnergyFraction
  endif
endif
!
! \label{f90:radraybase__read}
! --- Read parameter for base axis system for ray directions ---
if (present(radraybase)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='radraybase', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, radraybase,ident)
  else
    radraybase='unity'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'radraybase:         ' // trim(radraybase)
  endif
endif
!
! \label{f90:radraystar__read}
! --- Read parameter for list of relative ray directions ---
if (present(radraystar)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='radraystar', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, radraystar,ident)
  else
    radraystar='oktaeder'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'radraystar:         ' // trim(radraystar)
  endif
endif
!
! \label{f90:radpressure__read}
! --- Read radiation pressure mode ---
if (present(radpressure)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='radpressure', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, radpressure,ident)
  else
    radpressure='off'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'radpressure:        ' // trim(radpressure)
  endif
endif
!
! \label{f90:c_vislinear__read}
! --- Read viscosity constant ---
if (present(C_visLinear)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_vislinear', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visLinear, ident)
    if (C_visLinear <= 0.0) C_visLinear=0.0
  else
    C_visLinear=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visLinear:        ', C_visLinear
  endif
endif
!
! \label{f90:c_vissmagorinsky__read}
! --- Read viscosity constant ---
if (present(C_visSmagorinsky)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_vissmagorinsky', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visSmagorinsky, ident)
    if (C_visSmagorinsky <= 0.0) C_visSmagorinsky=0.0
  else
    C_visSmagorinsky=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visSmagorinsky:   ', C_visSmagorinsky
  endif
endif
!
! \label{f90:c_visartificial__read}
! --- Read viscosity constant ---
if (present(C_visArtificial)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visartificial', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visArtificial, ident)
    if (C_visArtificial <= 0.0) C_visArtificial=0.0
  else
    C_visArtificial=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visArtificial:    ', C_visArtificial
  endif
endif
!
! \label{f90:c_visexpansion__read}
! --- Read viscosity constant ---
if (present(C_visExpansion)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visexpansion', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visExpansion, ident)
    if (C_visExpansion <= 0.0) C_visExpansion=0.0
  else
    C_visExpansion=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visExpansion:     ', C_visExpansion
  endif
endif
!
! \label{f90:c_visprturb__read}
! --- Read viscosity constant ---
if (present(C_visPrturb)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visprturb', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visPrturb, ident)
  else
    C_visPrturb=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visPrturb:        ', C_visPrturb
  endif
endif
!
! \label{f90:c_vistensordiag__read}
! --- Read viscosity constant ---
if (present(C_visTensordiag)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_vistensordiag', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visTensordiag, ident)
  else
    C_visTensordiag=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visTensordiag:    ', C_visTensordiag
  endif
endif
!
! \label{f90:c_vistensoroff__read}
! --- Read viscosity constant ---
if (present(C_visTensoroff)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_vistensoroff', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visTensoroff, ident)
  else
    C_visTensoroff=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visTensoroff:     ', C_visTensoroff
  endif
endif
!
! \label{f90:c_vistensordiv__read}
! --- Read viscosity constant ---
if (present(C_visTensordiv)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_vistensordiv', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visTensordiv, ident)
  else
    C_visTensordiv=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visTensordiv:     ', C_visTensordiv
  endif
endif
!
! \label{f90:c_visp2pcoeff__read}
! --- Read point-to-point viscosity constant ---
if (present(C_visP2Pcoeff)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2pcoeff', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2Pcoeff, ident)
  else
    C_visP2Pcoeff=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2Pcoeff:      ', C_visP2Pcoeff
  endif
endif
!
! \label{f90:c_visp2pincl1__read}
! --- Read point-to-point viscosity constant ---
if (present(C_visP2Pincl1)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2pincl1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2Pincl1, ident)
  else
    C_visP2Pincl1=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2Pincl1:      ', C_visP2Pincl1
  endif
endif
!
! \label{f90:c_visp2pincl2__read}
! --- Read point-to-point viscosity constant ---
if (present(C_visP2Pincl2)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2pincl2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2Pincl2, ident)
  else
    C_visP2Pincl2=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2Pincl2:      ', C_visP2Pincl2
  endif
endif
!
! \label{f90:c_visp2phypsmagorinsky__read}
! --- Read point-to-point Smagorinsky viscosity parameter ---
if (present(C_visP2PhypSmagorinsky)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2phypsmagorinsky', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2PhypSmagorinsky, ident)
  else
    C_visP2PhypSmagorinsky=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2PhypSmago.:  ', C_visP2PhypSmagorinsky
  endif
endif
!
! \label{f90:c_visp2phypartificial__read}
! --- Read point-to-point Artificial viscosity parameter ---
if (present(C_visP2PhypArtificial)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2phypartificial', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2PhypArtificial, ident)
  else
    C_visP2PhypArtificial=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2PhypArtif.:  ', C_visP2PhypArtificial
  endif
endif
!
! \label{f90:c_visp2phypexpansion__read}
! --- Read point-to-point Expansion viscosity parameter ---
if (present(C_visP2PhypExpansion)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2phypexpansion', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2PhypExpansion, ident)
  else
    C_visP2PhypExpansion=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2PhypExpan.:  ', C_visP2PhypExpansion
  endif
endif
!
! \label{f90:c_visp2pdivrhov__read}
! --- Read point-to-point mass-flux-divergence viscosity constant ---
if (present(C_visP2Pdivrhov)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_visp2pdivrhov', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visP2Pdivrhov, ident)
  else
    C_visP2Pdivrhov=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visP2Pdivrhov:    ', C_visP2Pdivrhov
  endif
endif
!
! \label{f90:n_visiter__read}
! --- Read number of point-to-point hyper-viscosity smoothing iterations  ---
if (present(N_visIter)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_visiter', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_visIter,ident)
  else
    N_visIter=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_visIter:          ', N_visIter
  endif
endif
!
! --- Read number of point-to-point hyper-viscosity smoothing iterations  ---
!if (present(N_visP2PhypSmooth)) then
!  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_visp2phypsmooth', &
!                identlist=identlist, poslist=poslist, nentry=nentry, &
!                outstr=outstr0,ierr=ierr0)
!  if (ierr0 == 0) then
!    call uio_rd(ncpar, termt,ntt, N_visP2PhypSmooth,ident)
!    N_visP2PhypSmooth=min(max(N_visP2PhypSmooth, 0), 6)
!  else
    N_visP2PhypSmooth=0
!  endif
!  if (nc_p >= 0) then
!    write(nc_p,'(A,I15)')     'N_visP2PhypSmooth:  ', N_visP2PhypSmooth
!  endif
!endif
!
! \label{f90:visp2phypreconstruction__read}
! --- Read visP2Phyp reconstruction method ---
if (present(visP2Phypreconstruction)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='visp2phypreconstruction', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, visP2Phypreconstruction,ident)
  else
   visP2Phypreconstruction='constant'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'visP2Phyp Reconst.: ' // trim(visP2Phypreconstruction)
  endif
endif
!
! \label{f90:c_resB__read}
! --- Read resistivity constant ---
if (present(C_resB)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_resb', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_resB, ident)
  else
    C_resB=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_resB:             ', C_resB
  endif
endif
!
! \label{f90:c_resBconstant__read}
! --- Read resistivity constant ---
if (present(C_resBconstant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_resbconstant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_resBconstant, ident)
  else
    C_resBconstant=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_resBconstant:     ', C_resBconstant
  endif
endif
!
! \label{f90:c_resepsilon__read}
! --- Read resistivity constant ---
if (present(C_resEpsilon)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_resepsilon', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_resEpsilon, ident)
  else
    C_resEpsilon=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_resEpsilon:       ', C_resEpsilon
  endif
endif
!
! \label{f90:starttime__read}
! --- Read start time ---
if (present(starttime)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='starttime', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, starttime,ident)
  else
    starttime=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'starttime:          ', starttime
  endif
endif
!
! \label{f90:endtime__read}
! --- Read time limit for simulation ---
if (present(endtime)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='endtime', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, endtime,ident)
  else
    endtime=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'endtime:            ', endtime
  endif
endif
!
! \label{f90:plustime__read}
! --- Read simulation advance time ---
if (present(plustime)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='plustime', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, plustime,ident)
  else
    plustime=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'plustime:           ', plustime
  endif
endif
!
! \label{f90:time_scalestart__read}
! --- Read start time of scaling interval ---
if (present(time_ScaleStart)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='time_scalestart', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, time_ScaleStart,ident)
  else
    time_ScaleStart=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'time_ScaleStart:    ', time_ScaleStart
  endif
endif
!
! \label{f90:time_scaleend__read}
! --- Read end time of scaling interval ---
if (present(time_ScaleEnd)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='time_scaleend', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, time_ScaleEnd,ident)
  else
    time_ScaleEnd=-2.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'time_ScaleEnd:      ', time_ScaleEnd
  endif
endif
!
! \label{f90:cputime__read}
! --- Read job CPU time ---
if (present(CPUtime)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='cputime', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, CPUtime,ident)
  else
    CPUtime=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'CPUtime:            ', CPUtime
  endif
endif
!
! \label{f90:cputime_remainlimit__read}
! --- Read maximum remaining job CPU time ---
if (present(CPUtime_remainlimit)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='cputime_remainlimit', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, CPUtime_remainlimit,ident)
  else
    CPUtime_remainlimit=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'CPUtime_remainlimit:', CPUtime_remainlimit
  endif
endif
!
! \label{f90:starttimestep__read}
! --- Read start time step ---
if (present(starttimestep)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='starttimestep', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, starttimestep,ident)
  else
    starttimestep=-1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'starttimestep:      ', starttimestep
  endif
endif
!
! \label{f90:endtimestep__read}
! --- Read time step limit for simulation ---
if (present(endtimestep)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='endtimestep', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, endtimestep,ident)
  else
    endtimestep=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'endtimestep:        ', endtimestep
  endif
endif
!
! \label{f90:plustimestep__read}
! --- Read simulation advance time step ---
if (present(plustimestep)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='plustimestep', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, plustimestep,ident)
  else
    plustimestep=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'plustimestep:       ', plustimestep
  endif
endif
!
! \label{f90:dtime_min__read}
! --- Read minimum time step ---
if (present(dtime_min)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_min', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_min, ident)
  else
    dtime_min=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_min:          ', dtime_min
  endif
endif
!
! \label{f90:dtime_max__read}
! --- Read maximum time step ---
if (present(dtime_max)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_max', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_max, ident)
  else
    dtime_max=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_max:          ', dtime_max
  endif
endif
!
! \label{f90:dtime_min_stop__read}
! --- Read minimum time step, stop ---
if (present(dtime_min_stop)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_min_stop', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_min_stop, ident)
  else
    dtime_min_stop=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_min_stop:     ', dtime_min_stop
  endif
endif
!
! \label{f90:dtime_incmax__read}
! --- Read maximum time step increment factor ---
if (present(dtime_incmax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_incmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_incmax, ident)
  else
    dtime_incmax=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_incmax:       ', dtime_incmax
  endif
endif
!
! \label{f90:dtime_start__read}
! --- Read initial time step ---
if (present(dtime_start)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_start', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_start, ident)
  else
    dtime_start=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_start:        ', dtime_start
  endif
endif
!
! \label{f90:c_courant__read}
! --- Read HD Courant factor ---
if (present(C_Courant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_courant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_Courant, ident)
  else
    C_Courant=0.5
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_Courant:          ', C_Courant
  endif
endif
!
! \label{f90:c_courantmax__read}
! --- Read maximum HD Courant factor ---
if (present(C_CourantMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_courantmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_CourantMax, ident)
  else
    C_CourantMax=0.9
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_CourantMax:       ', C_CourantMax
  endif
endif
!
! \label{f90:c_maxeichange__read}
! --- Read maximum allowed hydro energy change ---
if (present(C_maxeiChange)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_maxeichange', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_maxeiChange, ident)
  else
    C_maxeiChange=0.9
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_maxeiChange:      ', C_maxeiChange
  endif
endif
!
! \label{f90:c_hydexpcourant__read}
! --- Read HD expansion Courant factor ---
if (present(C_hydExpCourant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydexpcourant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydExpCourant, ident)
  else
    C_hydExpCourant=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydExpCourant:    ', C_hydExpCourant
  endif
endif
!
! \label{f90:c_hydexpcourantmax__read}
! --- Read maximum HD expansion Courant factor ---
if (present(C_hydExpCourantMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydexpcourantmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydExpCourantMax, ident)
  else
    C_hydExpCourantMax=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydExpCourantMax: ', C_hydExpCourantMax
  endif
endif
!
! \label{f90:c_hydsoundcourant__read}
! --- Read HD sound-speed Courant factor ---
if (present(C_hydSoundCourant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydsoundcourant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydSoundCourant, ident)
  else
    C_hydSoundCourant=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydSoundCourant:  ', C_hydSoundCourant
  endif
endif
!
! \label{f90:c_hydsoundcourantmax__read}
! --- Read maximum HD sound-speed Courant factor ---
if (present(C_hydSoundCourantMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_hydsoundcourantmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_hydSoundCourantMax, ident)
  else
    C_hydSoundCourantMax=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_hydSoundCourantMax', C_hydSoundCourantMax
  endif
endif
!
! \label{f90:c_radcourant__read}
! --- Read RAD Courant factor ---
if (present(C_radCourant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radcourant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radCourant, ident)
  else
    C_radCourant=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radCourant:       ', C_radCourant
  endif
endif
!
! \label{f90:c_radcourantmax__read}
! --- Read maximum RAD Courant factor ---
if (present(C_radCourantMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radcourantmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radCourantMax, ident)
  else
    C_radCourantMax=2.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radCourantMax:    ', C_radCourantMax
  endif
endif
!
! \label{f90:c_radmaxeichange__read}
! --- Read maximum allowed radiative energy change ---
if (present(C_radmaxeiChange)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_radmaxeichange', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_radmaxeiChange, ident)
  else
    C_radmaxeiChange=0.5
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_radmaxeiChange:   ', C_radmaxeiChange
  endif
endif
!
! \label{f90:c_viscourant__read}
! --- Read viscous Courant factor ---
if (present(C_visCourant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_viscourant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visCourant, ident)
  else
    C_visCourant=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visCourant:       ', C_visCourant
  endif
endif
!
! \label{f90:c_viscourantmax__read}
! --- Read maximum viscous Courant factor ---
if (present(C_visCourantMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_viscourantmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_visCourantMax, ident)
  else
    C_visCourantMax=2.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_visCourantMax:    ', C_visCourantMax
  endif
endif
!
! \label{f90:c_rescourant__read}
! --- Read resistive Courant factor ---
if (present(C_resCourant)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_rescourant', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_resCourant, ident)
  else
    C_resCourant=0.5
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_resCourant:       ', C_resCourant
  endif
endif
!
! \label{f90:c_rescourantmax__read}
! --- Read maximum resistive Courant factor ---
if (present(C_resCourantMax)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_rescourantmax', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_resCourantMax, ident)
  else
    C_resCourantMax=1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_resCourantMax:    ', C_resCourantMax
  endif
endif
!
! \label{f90:dtime_out_full__read}
! --- Read output time step (full) ---
if (present(dtime_out_full)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_out_full', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_out_full, ident)
  else
    dtime_out_full=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_out_full:     ', dtime_out_full
  endif
endif
!
! \label{f90:dtime_out_mean__read}
! --- Read output time step (mean) ---
if (present(dtime_out_mean)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_out_mean', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_out_mean, ident)
  else
    dtime_out_mean=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_out_mean:     ', dtime_out_mean
  endif
endif
!
! \label{f90:dtime_out_end__read}
! --- Read output time step (end) ---
if (present(dtime_out_end)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='dtime_out_end', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtime_out_end, ident)
  else
    dtime_out_end=-1.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'dtime_out_end:      ', dtime_out_end
  endif
endif
!
! \label{f90:dtimestep_out_fine__read}
! --- Read output time step (fine) ---
if (present(dtimestep_out_fine)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='dtimestep_out_fine', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, dtimestep_out_fine, ident)
  else
    dtimestep_out_fine=-1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'dtimestep_out_fine: ', dtimestep_out_fine
  endif
endif
!
! \label{f90:istep_in_start__read}
! --- Read input dataset number ---
if (present(istep_in_start)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='istep_in_start', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, istep_in_start, ident)
  else
    istep_in_start=1
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'istep_in_start:     ', istep_in_start
  endif
endif
!
! \label{f90:n_outslicedim_mean__read}
! --- Read dimension of slice in mean output file ---
if (present(N_outslicedim_mean)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_outslicedim_mean', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_outslicedim_mean, ident)
  else
    N_outslicedim_mean=3
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_outslicedim_mean: ', N_outslicedim_mean
  endif
endif
!
! \label{f90:infile_start__read}
! --- Read input file name (start) ---
if (present(infile_start)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='infile_start', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, infile_start,ident)
  else
    infile_start='rhd.sta'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Input file (start):  ' // trim(infile_start)
  endif
endif
!
! \label{f90:outfile_end__read}
! --- Read output file name (end) ---
if (present(OutFile_end)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_end', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_end,ident)
  else
    OutFile_end='rhd.end'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (end):   ' // trim(OutFile_end)
  endif
endif
!
! \label{f90:outform_end__read}
! --- Read output file format (end) ---
if (present(OutForm_end)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outform_end', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutForm_end,ident)
  else
    OutForm_end='unformatted'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output format (end): ' // trim(OutForm_end)
  endif
endif
!
! \label{f90:outconv_end__read}
! --- Read output file conversion (end) ---
if (present(OutConv_end)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outconv_end', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutConv_end,ident)
  else
    OutConv_end=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output conv. (end):  ' // trim(OutConv_end)
  endif
endif
!
! \label{f90:outfile_full__read}
! --- Read output file name (full) ---
if (present(OutFile_full)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_full', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_full,ident)
  else
    OutFile_full='rhd.full'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (full):  ' // trim(OutFile_full)
  endif
endif
!
! \label{f90:outform_full__read}
! --- Read output file format (full) ---
if (present(OutForm_full)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outform_full', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutForm_full,ident)
  else
    OutForm_full='unformatted'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output format (full):' // trim(OutForm_full)
  endif
endif
!
! \label{f90:outconv_full__read}
! --- Read output file conversion (full) ---
if (present(OutConv_full)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outconv_full', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutConv_full,ident)
  else
    OutConv_full=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output conv. (full): ' // trim(OutConv_full)
  endif
endif
!
! \label{f90:outfile_mean__read}
! --- Read output file name (mean) ---
if (present(OutFile_mean)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_mean', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_mean,ident)
  else
    OutFile_mean='rhd.mean'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (mean):  ' // trim(OutFile_mean)
  endif
endif
!
! \label{f90:outform_mean__read}
! --- Read output file format (mean) ---
if (present(OutForm_mean)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outform_mean', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutForm_mean,ident)
  else
    OutForm_mean='unformatted'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output format (mean):' // trim(OutForm_mean)
  endif
endif
!
! \label{f90:outconv_mean__read}
! --- Read output file conversion (mean) ---
if (present(OutConv_mean)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outconv_mean', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutConv_mean,ident)
  else
    OutConv_mean=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output conv. (mean): ' // trim(OutConv_mean)
  endif
endif
!
! \label{f90:outfile_mcyl__read}
! --- Read output file name (cylindrical mean) ---
if (present(OutFile_mcyl)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_mcyl', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_mcyl,ident)
  else
    OutFile_mcyl=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (mcyl):  ' // trim(OutFile_mcyl)
  endif
endif
!
! \label{f90:outfile_fine__read}
! --- Read output file name (fine) ---
if (present(OutFile_fine)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_fine', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_fine,ident)
  else
    OutFile_fine='rhd.fine'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (fine):  ' // trim(OutFile_fine)
  endif
endif
!
! \label{f90:outform_fine__read}
! --- Read output file format (fine) ---
if (present(OutForm_fine)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outform_fine', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutForm_fine,ident)
  else
    OutForm_fine='unformatted'
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output format (fine):' // trim(OutForm_fine)
  endif
endif
!
! \label{f90:outconv_fine__read}
! --- Read output file conversion (fine) ---
if (present(OutConv_fine)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outconv_fine', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutConv_fine,ident)
  else
    OutConv_fine=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output conv. (fine): ' // trim(OutConv_fine)
  endif
endif
!
! \label{f90:outfile_chu1__read}
! --- Read output file name (chunk 1) ---
if (present(OutFile_chu1)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_chu1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_chu1,ident)
  else
    OutFile_chu1=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (chu1):  ' // trim(OutFile_chu1)
  endif
endif
!
! \label{f90:outdim_chu1__read}
! --- Read dimension of output chunk 1 ---
if (present(OutDim_chu1)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='outdim_chu1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutDim_chu1,ident)
  else
    OutDim_chu1=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,3(I5,1X,I5,3X))') 'Output dim. (chu1): ', OutDim_chu1
  endif
endif
!
! \label{f90:outfile_chu2__read}
! --- Read output file name (chunk 2) ---
if (present(OutFile_chu2)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_chu2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_chu2,ident)
  else
    OutFile_chu2=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (chu2):  ' // trim(OutFile_chu2)
  endif
endif
!
! \label{f90:outdim_chu2__read}
! --- Read dimension of output chunk 2 ---
if (present(OutDim_chu2)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='outdim_chu2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutDim_chu2,ident)
  else
    OutDim_chu2=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,3(I5,1X,I5,3X))') 'Output dim. (chu2): ', OutDim_chu2
  endif
endif
!
! \label{f90:outfile_chu3__read}
! --- Read output file name (chunk 3) ---
if (present(OutFile_chu3)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_chu3', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutFile_chu3,ident)
  else
    OutFile_chu3=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Output file (chu3):  ' // trim(OutFile_chu3)
  endif
endif
!
! \label{f90:outdim_chu3__read}
! --- Read dimension of output chunk 3 ---
if (present(OutDim_chu3)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='outdim_chu3', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, OutDim_chu3,ident)
  else
    OutDim_chu3=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,3(I5,1X,I5,3X))') 'Output dim. (chu3): ', OutDim_chu3
  endif
endif
!
! \label{f90:outfile_out__read}
! --- Read output file name (chunk 3) ---
if (present(OutFile_out)) then
!  call uio_srhd(ncpar, termt,ntt, type=charna, ident='outfile_out', &
!                identlist=identlist, poslist=poslist, nentry=nentry, &
!                outstr=outstr0,ierr=ierr0)
!  if (ierr0 == 0) then
!    call uio_rd(ncpar, termt,ntt, OutFile_out,ident)
!  else
    OutFile_out=''
!  endif
!  if (nc_p >= 0) then
!    write(nc_p,'(A)')         'Output file (out):   ' // trim(OutFile_out)
!  endif
endif
!
! === Read additional quantities ===================================================================
!
! \label{f90:n_test1__read}
! --- Read test integer ---
if (present(N_test1)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_test1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_test1,ident)
  else
    N_test1=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_test1:            ', N_test1
  endif
endif
!
! --- Read test integer ---
if (present(N_test2)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_test2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_test2,ident)
  else
    N_test2=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_test2:            ', N_test2
  endif
endif
!
! --- Read test integer ---
if (present(N_test3)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_test3', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_test3,ident)
  else
    N_test3=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_test3:            ', N_test3
  endif
endif
!
! --- Read test integer ---
if (present(N_test4)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_test4', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_test4,ident)
  else
    N_test4=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_test4:            ', N_test4
  endif
endif
!
! --- Read test integer ---
if (present(N_test5)) then
  call uio_srhd(ncpar, termt,ntt, type=intena, ident='n_test5', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, N_test5,ident)
  else
    N_test5=0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,I15)')     'N_test5:            ', N_test5
  endif
endif
!
! \label{f90:c_test1__read}
! --- Read test real ---
if (present(C_test1)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_test1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_test1, ident)
  else
    C_test1=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_test1:            ', C_test1
  endif
endif
!
! --- Read test real ---
if (present(C_test2)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_test2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_test2, ident)
  else
    C_test2=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_test2:            ', C_test2
  endif
endif
!
! --- Read test real ---
if (present(C_test3)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_test3', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_test3, ident)
  else
    C_test3=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_test3:            ', C_test3
  endif
endif
!
! --- Read test real ---
if (present(C_test4)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_test4', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_test4, ident)
  else
    C_test4=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_test4:            ', C_test4
  endif
endif
!
! --- Read test real ---
if (present(C_test5)) then
  call uio_srhd(ncpar, termt,ntt, type=realna, ident='c_test5', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, C_test5, ident)
  else
    C_test5=0.0
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A,1PE15.8)') 'C_test5:            ', C_test5
  endif
endif
!
! \label{f90:strtest1__read}
! --- Read test string ---
if (present(strtest1)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='strtest1', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, strtest1,ident)
  else
    strtest1=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Test string (1):     ' // trim(strtest1)
  endif
endif
!
! --- Read test string ---
if (present(strtest2)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='strtest2', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, strtest2,ident)
  else
    strtest2=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Test string (2):     ' // trim(strtest2)
  endif
endif
!
! --- Read test string ---
if (present(strtest3)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='strtest3', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, strtest3,ident)
  else
    strtest3=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Test string (3):     ' // trim(strtest3)
  endif
endif
!
! --- Read test string ---
if (present(strtest4)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='strtest4', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, strtest4,ident)
  else
    strtest4=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Test string (4):     ' // trim(strtest4)
  endif
endif
!
! --- Read test string ---
if (present(strtest5)) then
  call uio_srhd(ncpar, termt,ntt, type=charna, ident='strtest5', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncpar, termt,ntt, strtest5,ident)
  else
    strtest5=''
  endif
  if (nc_p >= 0) then
    write(nc_p,'(A)')         'Test string (5):     ' // trim(strtest5)
  endif
endif
!
! --- Overwrite error information ---
outstr0='done'
ierr0=0
!
! === Close file ===================================================================================
!
! --- Close parameter file after reading ---
call uio_closrd(ncpar)
!
900 if (present(outstr)) outstr=outstr0
    if (present(ierr)) ierr=ierr0
!
end subroutine rhd_par_RdData


!----------**************---------------------------------------------------------------------------
subroutine rhd_box_WrData(command, nc_p, ncout, file, form, conv, &
                         ndes, description, nhis, history, version, &
                         time_unit, x1_unit, x2_unit, x3_unit, &
                         rho_unit, e_unit, &
                         v_unit, p_unit, &
                         box_id, &
                         m1, m2, m3, n1, n2, n3, &
                         itime, time, time_db, &
                         dtime, time_out_full_last, time_out_mean_last, &
                         xc1, xc2, xc3, xb1, xb2, xb3, &
                         rho, ei, &
                         v1, v2, v3, p, quc, Bc1, Bc2, Bc3, Bb1, Bb2, Bb3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_WrData ('rhd_box_Write_Data')
!
! PURPOSE:
!   Write RHD model box into file in UIO form.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_box_WrData(command, nc_p, ncout, file=file, form=form, conv=conv, &
!                       ndes=ndes, description=description, &
!                       nhis=nhis, history=history, version=version, &
!                       time_unit=time_unit, &
!                       x1_unit=x1_unit, x2_unit=x2_unit, x3_unit=x3_unit, &
!                       rho_unit=rho_unit, e_unit=e_unit, &
!                       v_unit=v_unit, p_unit=p_unit, &
!                       box_id=box_id, &
!                       m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, &
!                       itime=itime, time=time, time_db=time_db, &
!                       dtime=dtime, time_out_full_last=time_out_full_last, &
!                         time_out_mean_last=time_out_mean_last, &
!                       xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, &
!                       rho=rho, ei=ei, &
!                       v1=v1, v2=v2, v3=v3, p=p, quc=quc, &
!                       Bc1=Bc1, Bc2=Bc2, Bc3=Bc3, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3)
!
! INPUT:
!   command:        ('command') character, command string:
!                     'open'  : Open new file and write header
!                     'append': Open existing file
!                     'model' : Write 'dataset' label and general model data at beginning of dataset
!                     'box':    Write box
!                     'endmo':  Write 'end dataset' label
!                     'close':  Close the file
!                   Example: 'open,model,box,endmo,close' or 'model,box,endmo'
!   nc_p:           ('number_channel_print') integer, unit number for text output
!   --- optional: ---
!   file:           ('file') character, name of data file
!   form:           ('format') character, name of data file
!   conv:           ('conversion') character, conversion type of data file
!   ndes:           ('number_description') integer, number of lines of description
!   description:    ('description') character, dimension(:), description of model
!   nhis:           ('number_history') integer, number of lines of history
!   history:        ('history') character, dimension(:), history of model
!   version:        ('version') character, string with RHD version
!   time_unit:      ('time_unit') character, unit of time (e.g. 's')
!   x1_unit:        ('x1_unit') character, unit of 1. space coordinate (e.g. 'cm')
!   x2_unit:        ('x2_unit') character, unit of 2. space coordinate (e.g. 'cm')
!   x3_unit:        ('x3_unit') character, unit of 3. space coordinate (e.g. 'cm')
!   rho_unit:       ('rho_unit') character, unit of density (e.g. 'g/cm^3')
!   e_unit:         ('energy_unit') character, unit of energy per mass (e.g. 'erg/g')
!   v_unit:         ('velocity_unit') character, unit of velocity (e.g. 'cm/s')
!   p_unit:         ('pressure_unit') character, unit of pressure (e.g. 'dyn/cm^2')
!   box_id:         ('box_identifier') character, identifier for box, default='z'
!   m1:             ('m_number_1') integer, 1. dimension, start index
!   n1:             ('number_1')   integer, 1. dimension, end index
!   m2:             ('m_number_2') integer, 2. dimension, start index
!   n2:             ('number_2')   integer, 2. dimension, end index
!   m3:             ('m_number_3') integer, 3. dimension, start index
!   n3:             ('number_3')   integer, 3. dimension, end index
!   itime:          ('index_time') integer, time step number
!   time:           ('time') real, time
!   time_db:        ('time_double') double precision real, time
!   dtime:          ('delta_time') real, time step
!   time_out_full_last: ('time_output_full_last') real, time of last output of full model
!   time_out_mean_last: ('time_output_mean_last') real, time of last output of averaged data
!   xc1:            ('x_center_1') real, 1. coordinate of cell centers
!   xc2:            ('x_center_2') real, 2. coordinate of cell centers
!   xc3:            ('x_center_3') real, 3. coordinate of cell centers
!   xb1:            ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:            ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:            ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:            ('rho') real, dimension(:,:,:), density
!   ei:             ('energy_internal') real, dimension(:,:,:), internal energy density per mass
!   v1:             ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:             ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:             ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   p:              ('pressure') real, dimension(:,:,:), pressure
!   quc:            ('quantity_center')  real, dimension(:,:,:,:), list of centered variables
!   Bc1:            ('B_center_1') real, dimension(:,:,:), cell centered mag. field, 1. component
!   Bc2:            ('B_center_2') real, dimension(:,:,:), cell centered mag. field, 2. component
!   Bc3:            ('B_center_3') real, dimension(:,:,:), cell centered mag. field, 3. component
!   Bb1:            ('B_boundary_1') real, dimension(:,:,:), cell boundary mag. field, 1. component
!   Bb2:            ('B_boundary_2') real, dimension(:,:,:), cell boundary mag. field, 2. component
!   Bb3:            ('B_boundary_3') real, dimension(:,:,:), cell boundary mag. field, 3. component
!
! INPUT/OUTPUT:
!   ncout:          ('number_channel_output') integer, unit number for data output
!
! OUTPUT:
!   None
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_closwr:            Close file after writing
!   uio_dim2st:            Compose dimension string
!   uio_openwr:            Open file for writing, write header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!   UIO routines have to be initialized.
!
! PROCEDURE:
!   The behavior of the routines depends on the contents of the command string.
!   It may contain one or more of the following substrings:
!   'open', 'append':
!     The requested file is opened (in UIO format). The file parameter has to
!     be specified. The form and conv parameter may are optional.
!     The file_id and - if specified - the file description and history are
!     written into the file.
!     In the case of the 'open' command a new file is created (old data is overwritten).
!     In the case of the 'append' command data is appended to a (possibly) existing file.
!   'model':
!     Dataset is opened. General model data is written into the file:
!     dataset label, modelboxtype, dimension, time, time step number.
!   'box':
!     The box data are written into the file with the unit ncout.
!     In this case all box data have to be specified.
!   'endmo':
!     Dataset is closed: enddataset label.
!   'close':
!     The file is closed again.
!   One or more models or boxes per model may be written into the file by
!   repeated calls of the routine.
!
! EXAMPLE:
!   --- Write exactly one model into the file ---
!   call rhd_box_WrData('open,model,box,endmo,close', 6, ncout, &
!                      file='model.out', form='unformatted', conv='ieee_4', &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version='testversion', &
!                      time_unit=time_unit, ... &
!                      rho=rho, ei=ei, v1=v1, v2=v2, v3=v3)
!   --- ---
!   --- Open file, write model(s), close file with separate commands ---
!   call rhd_box_WrData('open', 6, ncout, &
!                      file='model.out', form='unformatted', conv='ieee_4', &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version='testversion')
!   call rhd_box_WrData('model,box', 6, ncout, &
!                      time_unit=time_unit, ... &
!                      rho=rho, ei=ei, v1=v1, v2=v2, v3=v3)
!   --- ... ---
!   call rhd_box_WrData('endmo', 6, ncout)
!   call rhd_box_WrData('close', 6, ncout)
!
! TODO:
!   command string: command -> ',' // trim(command) // ','
!
! MODIFICATION HISTORY:
!   1997-06-21 (B.F. Kiel) Written
!   1997-06-22 (B.F.) Write character block_id
!   1997-06-30 (B.F.) Write label model
!   1997-07-01 (B.F.) rhd_wrmodel -> rhd_wrboxdata, command='model'
!   1997-10-09 (B.F.) rho and eik may be optional, v1,2,3 added
!   1997-10-10 (B.F.) Parameters p and p_unit added
!   1997-10-15 (B.F.) Parameters dtime and time_out_full_last added
!   1997-10-28 (B.F.) Write version
!   1999-03-24 (B.F. Copenhagen) Basic quantities changed, use of uio_filedef_module
!   1999-03-25 (B.F.) eik -> ei
!   1999-04-08 (B.F.) outside of box: time -> modeltime, itime -> modelitime; command 'endmo'
!   1999-05-11 (B.F.) rhd-model -> rhd-data
!   2000-01-22 (B.F.) Parameter box_id added
!   2000-04-08 (B.F.) Command 'append' added
!   2002-02-18 (B.F.) quc
!   2002-11-01 (B.F.) Magnetic field
!   2007-10-22 (B.F. Lyon) time_db introduced
!   2010-03-08 (B.F.) Write magnetic field arrays only if they were in input data
!   2010-05-10 (B.F.) Rename rhd_wrboxdata -> rhd_box_WrData
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use uio_filedef_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(in)   :: nc_p
integer,                   intent(inout):: ncout
character(len=*),optional, intent(in)   :: file, form, conv
integer,         optional, intent(in)   :: ndes
character(len=*), dimension(:), &
                 optional, intent(in)   :: description
integer,         optional, intent(in)   :: nhis
character(len=*), dimension(:), &
                 optional, intent(in)   :: history
character(len=*),optional, intent(in)   :: version
character(len=*),optional, intent(in)   :: time_unit, x1_unit, x2_unit, x3_unit, &
                                           rho_unit, e_unit, v_unit, p_unit, &
                                           box_id
integer,         optional, intent(in)   :: m1, m2, m3, n1, n2, n3
integer,         optional, intent(in)   :: itime
real,            optional, intent(in)   :: time, dtime, time_out_full_last, time_out_mean_last
real(kind=kind(0.0D+00)), &
                 optional, intent(in)   :: time_db
real, dimension(:), &
                 optional, intent(in)   :: xc1, xc2, xc3
real, dimension(:), &
                 optional, intent(in)   :: xb1, xb2, xb3
real, dimension(:,:,:), &
                 optional, intent(in)   :: rho, ei, v1, v2, v3, p
real, dimension(:,:,:,:), &
                optional, BOXARRAYTYPE  :: quc
real, dimension(:,:,:), &
                optional, BOXARRAYTYPE  :: Bc1, Bc2, Bc3, Bb1, Bb2, Bb3
!
! --- Local variables ---
integer                                 :: ntt
character(len=let)                      :: termt(2,nttmx)
character(len=80)                       :: shistr, box_id0, position
integer                                 :: nquc, iquc
character(len=80)                       :: qucstr
!---------------------------------------------------------------------------------------------------
!
if ((index(command, 'open') /= 0) .or. (index(command, 'append') /= 0)) then
  if (index(command, 'append') /= 0) then
    ! --- Open file for writing in uio-form: append data ---
    call uio_openwr(ncout, file=file, form=form, conv=conv, position='append')
  else
    ! --- Open file for writing in uio-form ---
    call uio_openwr(ncout, file=file, form=form, conv=conv)
  endif
  !
  write(nc_p,'(A,I3)') 'Model file ''' // trim(file) // ''' opened on channel ', ncout
  !
  ! --- Write heading information ---
  !
  ! --- Write file_id ---
  call uio_wr(ncout, 'rhd-data', 'file_id', name='File identification')
  !
  if (present(description) .and. present(ndes)) then
    ! --- Write file description ---
    call uio_wr(ncout, description(1:ndes), 'description', name='File description')
  endif
  !
  if (present(history) .and. present(nhis)) then
    ! --- Write file history ---
    call uio_wr(ncout, history(1:nhis), 'history', name='File history')
  endif
  !
  if (present(version)) then
    ! --- Write program version ---
    call uio_wr(ncout, version, 'version', name='Program version')
  endif
endif
!
if (index(command, 'model') /= 0) then
  ! --- Write general model data ---
  !
  ! --- Model label ---
  call uio_wrlabl(ncout, dataset_ident, name='RHD model', date='now')
  !
  ! --- id ---
  call uio_wr(ncout, 'single_box', dataset_id_ident, name='Type of box hierarchy')
  !
  ! --- Time ---
  call uio_wr(ncout, time   , 'modeltime'   , name='time', unit=time_unit)
  call uio_wr(ncout, time_db, 'modeltime_db', name='time', unit=time_unit)
  !
  ! --- Time step number ---
  call uio_wr(ncout, itime, 'modelitime', name='time step number', unit='1')
  !
  ! --- Time step---
  call uio_wr(ncout, dtime, 'dtime', name='time step', unit=time_unit)
  !
  ! --- Time of last output of full model ---
  if (present(time_out_full_last)) then
    call uio_wr(ncout, time_out_full_last, 'time_out_full_last', &
                name='Time of last output of full model', unit=time_unit)
  endif
  !
  ! --- Time of last output of averaged data ---
  if (present(time_out_mean_last)) then
    call uio_wr(ncout, time_out_mean_last, 'time_out_mean_last', &
                name='Time of last output of averaged data', unit=time_unit)
  endif
endif
!
if (index(command, 'box') /= 0) then
  !
  ! --- Write bulk model data (from box) ---
  !
  ! --- Start box label ---
  call uio_wrlabl(ncout, box_ident, date='now')
  !
  ! --- boc_id ---
  if (present(box_id)) then
    box_id0=box_id
  else
    box_id0='z'
  endif
  call uio_wr(ncout, box_id0, box_id_ident, name='Block identification')
  !
  ! --- Block dimension ---
  call uio_wr(ncout, reshape((/ m1,n1, m2,n2, m3,n3 /), (/2,3/)), 'dimension', &
              format='I7', nperlin=6)
  !
  ! --- Scalars ---
  !
  ! --- Time ---
  call uio_wr(ncout, time   , 'time'   , name='time', unit=time_unit)
  call uio_wr(ncout, time_db, 'time_db', name='time', unit=time_unit)
  !
  ! --- Time step number ---
  call uio_wr(ncout, itime, 'itime', name='time step number', unit='1')
  !
  ! --- Arrays: ---
  !
  ! --- Grid: 1D vectors: spread -> 3D ---
  call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape((xc1), (/n1-m1+1,1,1/)), 'xc1', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x1 coordinates of cell centers', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape((xc2), (/1,n2-m2+1,1/)), 'xc2', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x2 coordinates of cell centers', unit=x2_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  0 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape((xc3), (/1,1,n3-m3+1/)), 'xc3', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x3 coordinates of cell centers', unit=x3_unit, &
              ntt=ntt, termt=termt)
  call uio_dim2st((/ 0, 0, 0 /), (/  1,  1,  1 /), shistr)
  ntt=0
  call uio_adkeyw(termt,ntt,'ds',shistr)
  call uio_wr(ncout, reshape((xb1), (/n1-m1+2,1,1/)), 'xb1', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x1 coordinates of cell boundaries', unit=x1_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape((xb2), (/1,n2-m2+2,1/)), 'xb2', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x2 coordinates of cell boundaries', unit=x2_unit, &
              ntt=ntt, termt=termt)
  call uio_wr(ncout, reshape((xb3), (/1,1,n3-m3+2/)), 'xb3', &
              mode=8, ishift=(/m1,m2,m3/), &
              name='x3 coordinates of cell boundaries', unit=x3_unit, &
              ntt=ntt, termt=termt)
  !
  ! --- Volume data ---
  !
  ! --- Density ---
  if (present(rho)) then
    call uio_wr(ncout, rho, 'rho', mode=8, ishift=(/m1,m2,m3/), name='Density', unit=rho_unit)
  endif
  !
  ! --- Energy ---
  if (present(ei)) then
    call uio_wr(ncout, ei, 'ei', mode=8, ishift=(/m1,m2,m3/), &
                name='Internal energy', unit=e_unit)
  endif
  !
  ! --- Velocity ---
  if (present(v1)) then
    call uio_wr(ncout, v1, 'v1', mode=8, ishift=(/m1,m2,m3/), name='Velocity 1', unit=v_unit)
  endif
  if (present(v2)) then
    call uio_wr(ncout, v2, 'v2', mode=8, ishift=(/m1,m2,m3/), name='Velocity 2', unit=v_unit)
  endif
  if (present(v3)) then
    call uio_wr(ncout, v3, 'v3', mode=8, ishift=(/m1,m2,m3/), name='Velocity 3', unit=v_unit)
  endif
  !
  ! --- Pressure ---
  if (present(p)) then
    call uio_wr(ncout, p, 'p', mode=8, ishift=(/m1,m2,m3/), name='Pressure', unit=p_unit)
  endif
  !
  ! --- Additional fields ---
  if (present(quc)) then
    if (ALLOASSOTED(quc)) then
      nquc=size(quc,4)
      do iquc=1,nquc
        write(qucstr,'(I3)') iquc
        qucstr='00' // adjustl(qucstr)
        qucstr='quc' // qucstr(len_trim(qucstr)-2:)
        call uio_wr(ncout, quc(:,:,:,iquc), qucstr, &
                    mode=8, ishift=(/m1,m2,m3/), &
                    name=prop%quc_name(iquc), unit=prop%quc_unit(iquc))
      end do ! iquc
    endif
  endif
  !
  ! --- Magnetic field
  if (present(Bc1)) then
    if (ALLOASSOTED(Bc1)) then
      call uio_wr(ncout, Bc1, 'bc1', mode=8, ishift=(/m1,m2,m3/), name='Magnetic field 1', &
                                             unit=prop%B1_unit)
    endif
  endif
  if (present(Bc2)) then
    if (ALLOASSOTED(Bc2)) then
      call uio_wr(ncout, Bc2, 'bc2', mode=8, ishift=(/m1,m2,m3/), name='Magnetic field 2', &
                                             unit=prop%B2_unit)
    endif
  endif
  if (present(Bc3)) then
    if (ALLOASSOTED(Bc3)) then
      call uio_wr(ncout, Bc3, 'bc3', mode=8, ishift=(/m1,m2,m3/), name='Magnetic field 3', &
                                             unit=prop%B3_unit)
    endif
  endif
  if (present(Bb1)) then
    if (ALLOASSOTED(Bb1) .and. prop%Bb1_IsInFile_flag) then
      call uio_dim2st((/ 0, 0, 0 /), (/  1,  0,  0 /), shistr)
      ntt=0
      call uio_adkeyw(termt,ntt,'ds',shistr)
      call uio_wr(ncout, Bb1, 'bb1', mode=8, ishift=(/m1,m2,m3/), name='Magnetic field 1', &
                                             unit=prop%B1_unit, ntt=ntt, termt=termt)
    endif
  endif
  if (present(Bb2)) then
    if (ALLOASSOTED(Bb2) .and. prop%Bb2_IsInFile_flag) then
      call uio_dim2st((/ 0, 0, 0 /), (/  0,  1,  0 /), shistr)
      ntt=0
      call uio_adkeyw(termt,ntt,'ds',shistr)
      call uio_wr(ncout, Bb2, 'bb2', mode=8, ishift=(/m1,m2,m3/), name='Magnetic field 2', &
                                             unit=prop%B2_unit, ntt=ntt, termt=termt)
    endif
  endif
  if (present(Bb3)) then
    if (ALLOASSOTED(Bb3) .and. prop%Bb3_IsInFile_flag) then
      call uio_dim2st((/ 0, 0, 0 /), (/  0,  0,  1 /), shistr)
      ntt=0
      call uio_adkeyw(termt,ntt,'ds',shistr)
      call uio_wr(ncout, Bb3, 'bb3', mode=8, ishift=(/m1,m2,m3/), name='Magnetic field 3', &
                                             unit=prop%B3_unit, ntt=ntt, termt=termt)
    endif
  endif
  !
  ! --- End block label ---
  call uio_wrlabl(ncout, endbox_ident)
endif
!
if (index(command,'endmo') /= 0) then
  ! --- End dataset label ---
  call uio_wrlabl(ncout, enddataset_ident, date='now')
endif
!
if (index(command,'close') /= 0) then
  ! --- Close file ---
  call uio_closwr(ncout)
endif
!
end subroutine rhd_box_WrData


!----------**************---------------------------------------------------------------------------
subroutine rhd_box_RdData(command, nc_p, ncin, &
                          modellist, timelist, itimelist, nmodel, imodel, &
                          file, &
                          ndes, description, nhis, history, version, &
                          m1, m2, m3, n1, n2, n3, &
                          itime, time, time_db, &
                          dtime, time_out_full_last, time_out_mean_last, &
                          xc1, xc2, xc3, xb1, xb2, xb3, &
                          rho, v1, v2, v3, ei, quc, Bb1, Bb2, Bb3)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_box_RdData ('rhd_box_Read_Data')
!
! PURPOSE:
!   Read RHD model box data from file in UIO form.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call rhd_box_RdData(command, nc_p, ncin, &
!                       modellist, timelist, itimelist, nmodel, imodel, &
!                       file=file, &
!                       ndes=ndes, description=description, &
!                       nhis=nhis, history=history, version=version, &
!                       time_unit=time_unit, &
!                       x1_unit=x1_unit, x2_unit=x2_unit, x3_unit=x3_unit, &
!                       rho_unit=rho_unit, v_unit=v_unit, e_unit=e_unit, &
!                       m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, &
!                       itime=itime, time=time, time_db=time_db, &
!                       dtime=dtime, time_out_full_last=time_out_full_last, &
!                         time_out_mean_last=time_out_mean_last, &
!                       xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, &
!                       rho=rho, v1=v1, v2=v2, v3=v3, ei=ei, quc=quc, &
!                       Bb1=Bb1, Bb2=Bb2, Bb3=Bb3)
!
! INPUT:
!   command:        ('command') character, command string:
!                     'open'  : Open file and read identifiers
!                     'model' : Read general model data at beginning of dataset
!                     'box':    Read box
!                     'close':  Close the file
!                   Example: 'open,model,box,endmo,close' or 'model,box,endmo'
!   nc_p:           ('number_channel_print') integer, unit number for text output
!   file:           ('file') character, name of data file
!
! INPUT/OUTPUT:
!   ncin:           ('number_channel_output') integer, unit number for data output
!
! OUTPUT:
!   --- optional: ---
!   ndes:           ('number_description') integer, number of lines of description
!   description:    ('description') character, dimension(:), description of model
!   nhis:           ('number_history') integer, number of lines of history
!   history:        ('history') character, dimension(:), history of model
!   version:        ('version') character, program version
!   time_unit:      ('time_unit') character, unit of time (e.g. 's')
!   x1_unit:        ('x1_unit') character, unit of 1. space coordinate (e.g. 'cm')
!   x2_unit:        ('x2_unit') character, unit of 2. space coordinate (e.g. 'cm')
!   x3_unit:        ('x3_unit') character, unit of 3. space coordinate (e.g. 'cm')
!   rho_unit:       ('rho_unit') character, unit of density (e.g. 'g/cm^3')
!   v_unit:         ('v_unit') character, unit of velocity (e.g. 'cm/s')
!   e_unit:         ('energy_unit') character, unit of energy per mass (e.g. 'erg/g')
!   m1:             ('m_number_1') integer, 1. dimension, start index
!   n1:             ('number_1')   integer, 1. dimension, end index
!   m2:             ('m_number_2') integer, 2. dimension, start index
!   n2:             ('number_2')   integer, 2. dimension, end index
!   m3:             ('m_number_3') integer, 3. dimension, start index
!   n3:             ('number_3')   integer, 3. dimension, end index
!   itime:          ('index_time') integer, time step number
!   time:           ('time') real, time
!   time_db:        ('time_double') double precision real, time
!   dtime:          ('delta_time') real, time step
!   time_out_full_last: ('time_output_full_last') real, time of last output of full model
!   time_out_mean_last: ('time_output_mean_last') real, time of last output of averaged data
!   xc1:            ('x_center_1') real, 1. coordinate of cell centers
!   xc2:            ('x_center_2') real, 2. coordinate of cell centers
!   xc3:            ('x_center_3') real, 3. coordinate of cell centers
!   xb1:            ('x_boundary_1') real, 1. coordinate of cell boundaries
!   xb2:            ('x_boundary_2') real, 2. coordinate of cell boundaries
!   xb3:            ('x_boundary_3') real, 3. coordinate of cell boundaries
!   rho:            ('rho') real, dimension(:,:,:), density
!   v1:             ('velocity_1') real, dimension(:,:,:), velocity, 1. coordinate
!   v2:             ('velocity_2') real, dimension(:,:,:), velocity, 2. coordinate
!   v3:             ('velocity_3') real, dimension(:,:,:), velocity, 3. coordinate
!   ei:             ('energy_internal') real, dimension(:,:,:), internal energy density
!   quc:            ('quantity_center')  real, dimension(:,:,:,:), list of centered variables
!   Bb1:            ('B_boundary_1') real, dimension(:,:,:), cell boundary mag. field, 1. component
!   Bb2:            ('B_boundary_2') real, dimension(:,:,:), cell boundary mag. field, 2. component
!   Bb3:            ('B_boundary_3') real, dimension(:,:,:), cell boundary mag. field, 3. component
!
! ROUTINES:
!   uio_chpos:             Give current file position or jump to specified position
!   uio_closrd:            Close file after reading
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_openrd:            Open file for reading, read header
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_rdhd:              Read header of variable and extract keywords
!   uio_skipda:            Skip data block
!   uio_srhd:              Search entry header in file or in entry list
!
! MODULES:
!   rhd_prop_module:       Properties of quantities in 'box' structure
!   uio_bulk_module:       Main set of UIO routines
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!   Modify contents of global prop structure.
!
! RESTRICTIONS:
!   UIO routines have to be initialized.
!
! PROCEDURE:
!   The behavior of the routines depends on the contents of the command string.
!   It may contain one or more of the following substrings:
!   'open':
!     The requested file is opened (in UIO format). The file parameter has to
!     be specified.
!     The file_id and - if specified - the file description and history are
!     read from the file.
!   'model','box':
!     The box data are read from the file with the unit ncin.
!   'close':
!     The file is closed again.
!
! EXAMPLE:
!   --- Read exactly one model from the file ---
!   call rhd_box_RdData('open,model,box,close', 6, ncin, &
!                      file='model.in', &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version=version, &
!                      time_unit=time_unit, ... &
!                      rho=rho, v1=v1, v2=v2, v3=v3, ei=ei)
!   --- ---
!   --- Open file, read models, close file with separate commands ---
!   call rhd_box_RdData('open', 6, ncin, &
!                      file='model.in', &
!                      ndes=ndes, description=description, &
!                      nhis=nhis, history=history, version=version)
!   call rhd_box_RdData('model', 6, ncin, &
!                      time_unit=time_unit, ... &
!                      m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, &
!                      itime=itime, time=time, time_db=time_db)
!   call rhd_box_RdData('box', 6, ncin, &
!                      time_unit=time_unit, ... &
!                      rho=rho, v1=v1, v2=v2, v3=v3, ei=ei)
!     ...
!   call rhd_box_RdData('close', 6, ncin)
!
! MODIFICATION HISTORY:
!   1997-06-30 (B.F. Kiel) Written
!   1997-07-01 (B.F.) File format changed
!   1997-10-02 (B.F.) rhd-parameter -> rhd-model, imodel+1, corrections
!   1997-10-14 (B.F.) Parameters dtime and time_out_full_last added
!   1997-10-17 (B.F.) Read arrays with mode=2
!   1997-10-28 (B.F.) Read version
!   1999-03-24 (B.F. Copenhagen) Basic quantities changed, use of uio_filedef_module
!   1999-03-25 (B.F.) eik -> ei
!   1999-03-26 (B.F.) file_id: rhd-data
!   1999-05-11 (B.F.) file_id: rhd-data or rhd-model
!   2000-02-09 (B.F.) Parameter time_out_mean_last added
!   2002-08-09 (B.F.) Some extra 'rewind' commands to account for bug in Intel compiler
!   2002-10-18 (B.F.) Read quc
!   2002-10-29 (B.F.) Read additional information from header of quc arrays
!   2002-11-01 (B.F.) Magnetic field
!   2002-12-09 (B.F.) Check if imodel is within range of datasets
!   2003-09-11 (B.F.) nentmx=1000->10000
!   2007-10-22 (B.F. Lyon) time_db introduced
!   2010-03-08 (B.F.) Do not exit with error message if magnetic field is not specified
!   2010-03-30 (B.F.) Set magnetic field arrays explicitly to zero if they are not read from file
!   2010-04-09 (B.F.) Put m1,...,n3 into prop structure
!   2010-05-10 (B.F.) Rename rhd_rdboxdata -> rhd_box_RdData
!   2014-03-20 (B.F.) Put axes boundary values into prop structure
!   2014-04-06 (B.F.) Put axes vectors into prop structure
!   2017-08-07 (B.F. Uppsala) Deallocate existing global axes; put xc1, xc2, xc3 into prop structure
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use uio_filedef_module
use rhd_prop_module
!
implicit none
!
! --- I/O parameters ---
character(len=*),          intent(in)   :: command
integer,                   intent(in)   :: nc_p
integer,                   intent(inout):: ncin
integer,                   intent(in)   :: imodel
integer,                   intent(inout):: nmodel
integer, dimension(:),     intent(inout):: modellist, itimelist
real,    dimension(:),     intent(inout):: timelist
character(len=*),optional, intent(in)   :: file
integer,         optional, intent(out)  :: ndes
character(len=*), dimension(:), &
                 optional, intent(out)  :: description
integer,         optional, intent(out)  :: nhis
character(len=*), dimension(:), &
                 optional, intent(out)  :: history
character(len=*),optional, intent(out)  :: version
integer,         optional, intent(out)  :: m1, m2, m3, n1, n2, n3
integer,         optional, intent(out)  :: itime
real,            optional, intent(out)  :: time, dtime, time_out_full_last, time_out_mean_last
real(kind=kind(0.0D+00)), &
                 optional, intent(out)  :: time_db
real, dimension(:), &
                 optional, intent(out)  :: xc1, xc2, xc3
real, dimension(:), &
                 optional, intent(out)  :: xb1, xb2, xb3
real, dimension(:,:,:), &
                 optional, intent(out)  :: rho, v1, v2, v3, ei
real, dimension(:,:,:,:), &
                optional, BOXARRAYTYPE  :: quc
real, dimension(:,:,:), &
                optional, BOXARRAYTYPE  :: Bb1, Bb2, Bb3
!
! --- Local variables ---
integer                                 :: ierr0
character(len=80)                       :: outstr0, outstr1
integer                                 :: ntt
character(len=let)                      :: termt(2,nttmx)
character(len=let)                      :: ident, file_id
integer, parameter                      :: nentmx=10000
integer,            save                :: nentry
integer,            save                :: poslist(nentmx)
character(len=let), save                :: identlist(3,nentmx)
integer                                 :: nmodelmx, ncol, ie1, ie2
integer, dimension(2,3)                 :: dim
integer                                 :: iup(2)
logical                                 :: no_time, no_itime
real                                    :: time_out_full_last0, time_tmp
real(kind=kind(0.0D+00))                :: time_db_tmp
integer                                 :: nquc, iquc
character(len=80)                       :: qucstr
!---------------------------------------------------------------------------------------------------
!
nmodelmx=size(modellist,1)
ncol=size(identlist,1)
!
if (index(command,'open') /= 0) then
  ! --- Open file for reading in uio-form ---
  call uio_openrd(ncin, file=file)
  !
  write(nc_p,'(A,I3)') 'Model file ''' // trim(file) // ''' opened on channel ', ncin
  !
  ! --- Determine model file contents ---
  nentry=1
  nmodel=0
  no_time=.false.
  no_itime=.false.
  ENTRYLOOP: do
    ! --- Determine and save current file position ---
    call uio_chpos(ncin,mode='get',posout=poslist(nentry))
    ! --- Read entry header ---
    call uio_rdhd(ncin, termt,ntt, outstr=outstr1)
    if (outstr1 == 'eof') then
      ! --- End of file reached ---
      identlist(1,nentry)='eof'
      identlist(2:,nentry)=''
      outstr0='done'
      ierr0=0
      exit ENTRYLOOP
    else
      ! --- Put entry type, identifier(, and name) into list ---
      identlist(1,nentry)=termt(1,1)
      identlist(2,nentry)=termt(1,2)
      if (ncol >= 3) call uio_exkeyw(termt,ntt,namena,identlist(3,nentry))
      ! --- Check if entry is 'label dataset', 'real time', or 'integer itime' --
      if ((identlist(1,nentry) == lablna       ) .and. &
          (identlist(2,nentry) == dataset_ident)) then
        nmodel=nmodel+1
        if (nmodel > nmodelmx) exit ENTRYLOOP
        modellist(nmodel)=nentry
        no_time=.true.
        no_itime=.true.
      else if ((identlist(1,nentry) == realna) .and. &
               (identlist(2,nentry) == 'time') .and. (no_time)) then
        call uio_rd(ncin, termt,ntt, timelist(nmodel),ident)
        no_time=.false.
      else if ((identlist(1,nentry) == intena)  .and. &
               (identlist(2,nentry) == 'itime') .and. (no_itime)) then
        call uio_rd(ncin, termt,ntt, itimelist(nmodel),ident)
        no_itime=.false.
      else
        ! --- Skip data block ---
        call uio_skipda(ncin, termt,ntt)
      endif
      !
      nentry=nentry+1
      if (nentry > nentmx) then
        ! --- End of entry list reached ---
        nentry=nentmx
        outstr0='too many entries in file'
        ierr0=1
        exit
      endif
    endif
  end do ENTRYLOOP
  !
  !write(nc_p,*) 'nentry=', nentry
  !write(nc_p,'(A)') identlist(:,:nentry)
  !write(nc_p,'(I4)') poslist(:nentry)
  !
  ! --- Patch for Intel compiler ---
  call uio_chpos(ncin, mode='rewind')
  !
  ! --- Read heading information ---
  !
  ! --- Read file_id ---
  call uio_srhd(ncin, termt,ntt, type=charna, ident='file_id', &
                identlist=identlist, poslist=poslist, nentry=nentry, &
                outstr=outstr0,ierr=ierr0)
  if (ierr0 == 0) then
    call uio_rd(ncin, termt,ntt, file_id,ident)
    if ((file_id /= 'rhd-data') .and. (file_id /= 'rhd-model')) then
      outstr0='''' // trim(file) // ''' is not RHD data file'
      ierr0=1
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  else
    outstr0='Type of ''' // trim(file) // ''' can not be determined'
    ierr0=2
    call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
  endif
  !
  ! --- Read description ---
  if (present(description) .and. present(ndes)) then
    call uio_srhd(ncin, termt,ntt, type=charna, ident='description', &
                  identlist=identlist, poslist=poslist, nentry=nentry, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, description,ident,mode=2,iup=iup(1:1))
      ndes=iup(1)
    else
      ndes=0
    endif
    write(nc_p,'(A)') 'Description: ..............................................'
    write(nc_p,'(A)') description(:ndes)
  endif
  !
  ! --- Read history ---
  if (present(history) .and. present(nhis)) then
    call uio_srhd(ncin, termt,ntt, type=charna, ident='history', &
                  identlist=identlist, poslist=poslist, nentry=nentry, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, history,ident,mode=2,iup=iup(1:1))
      nhis=iup(1)
    else
      nhis=0
    endif
    write(nc_p,'(A)') 'History: ..................................................'
    write(nc_p,'(A)') history(:nhis)
    write(nc_p,'(A)') '...........................................................'
  endif
  !
  ! --- Read version ---
  if (present(version)) then
    call uio_srhd(ncin, termt,ntt, type=charna, ident='version', &
                  identlist=identlist, poslist=poslist, nentry=nentry, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, version,ident)
    else
      version=''
    endif
  endif
  !
endif
!
if (index(command,'model') /= 0) then
  ! --- General model data ---
  !
  ! --- Restrict entry range to selected model ---
  if (imodel > nmodel) then
    write(nc_p,'(A,I3,A,I3,A)') 'Requested dataset number (', imodel, &
                                ') larger than number of datasets in file (', nmodel, ')'
    stop
  else if (imodel == nmodel) then
    ie1=modellist(imodel)
    ie2=nentry
  else
    ie1=modellist(imodel)
    ie2=modellist(imodel+1)
  endif
  !
  ! --- Time step ---
  if (present(dtime)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='dtime', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, dtime,ident)
    else
      outstr0='time step is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
    write(nc_p,'(A,1PE15.8)') 'dtime:              ', dtime
  endif
  !
  ! --- Time of last output of full model ---
  !
  time_out_full_last0=-1.0E35
  if (present(time_out_full_last)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='time_out_full_last', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, time_out_full_last,ident)
      time_out_full_last0=time_out_full_last
    else
      outstr0='Time of last output of full model is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
    write(nc_p,'(A,1PE15.8)') 'time_out_full_last: ', time_out_full_last
  endif
  !
  ! --- Time of last output of averaged data ---
  !
  if (present(time_out_mean_last)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='time_out_mean_last', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, time_out_mean_last,ident)
    else
      time_out_mean_last=time_out_full_last0
    endif
    write(nc_p,'(A,1PE15.8)') 'time_out_mean_last: ', time_out_mean_last
  endif
  !
  ! --- Model dimension ---
!  if (present(m1) .or. present(n1) .or. present(m2) .or. present(n2) .or. &
!      present(m3) .or. present(n3)) then
    call uio_srhd(ncin, termt,ntt, type=intena, ident='dimension', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, dim,ident, mode=1)
      if (present(m1)) m1=dim(1,1)
      if (present(n1)) n1=dim(2,1)
      if (present(m2)) m2=dim(1,2)
      if (present(n2)) n2=dim(2,2)
      if (present(m3)) m3=dim(1,3)
      if (present(n3)) n3=dim(2,3)
      prop%m1=dim(1,1)
      prop%n1=dim(2,1)
      prop%m2=dim(1,2)
      prop%n2=dim(2,2)
      prop%m3=dim(1,3)
      prop%n3=dim(2,3)
    else
      outstr0='block dimension is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
    write(nc_p,'(A,3(2I6,2X))') 'Dimension:       ', dim
!  endif
  !
  ! --- Time ---
  if (present(time) .or. present(time_db)) then
    ! --- Try to read double precision time in any case ---
    call uio_srhd(ncin, termt,ntt, type=realna, ident='time_db', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, time_db_tmp, prop%time_ident, name=prop%time_name, &
                                                                 unit=prop%time_unit)
      if (present(time_db)) time_db=time_db_tmp
      if (present(time   )) time   =time_db_tmp
    else
      ! --- Double precision time not present: use single precision time ---
      call uio_srhd(ncin, termt,ntt, type=realna, ident='time', &
                    identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                    nentry=ie2-ie1+1, &
                    outstr=outstr0,ierr=ierr0)
      if (ierr0 == 0) then
        call uio_rd(ncin, termt,ntt, time_tmp, prop%time_ident, name=prop%time_name, &
                                                                unit=prop%time_unit)
        if (present(time_db)) time_db=time_tmp
        if (present(time   )) time   =time_tmp
      else
        outstr0='time is not specified'
        ierr0=3
        call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
      endif
    endif
    write(nc_p,'(A,1PE15.8)') 'Time:               ', time_db
  endif
  !
  ! --- Time step number ---
  if (present(itime)) itime=itimelist(imodel)
  !
endif
!
! --- Read additional data (boundary conditions ...) ---
! ...
!
if (index(command,'box') /= 0) then
  ! --- Read bulk model data (from box) ---
  !
  ! --- Restrict entry range to selected model ---
  ie1=modellist(imodel)
  if (imodel >= nmodel) then
    ie2=nentry
  else
    ie2=modellist(imodel+1)
  endif
  !
  ! --- Search box: increase start index ie1 to position of next box ---
  NEXTBOXLOOP: do
    ie1=ie1+1
    if (ie1 >= ie2) exit NEXTBOXLOOP
    if ((identlist(1,ie1) == lablna) .and. (identlist(2,ie1) == 'box')) exit NEXTBOXLOOP
  end do NEXTBOXLOOP
  !
  ! --- Patch for Intel compiler ---
  call uio_chpos(ncin, mode='rewind')
  !
  call uio_srhd(ncin, termt,ntt, type=lablna, ident='box', &
                identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                nentry=ie2-ie1+1, &
                outstr=outstr0,ierr=ierr0)
  !
  ! --- Block dimension ---
  if (present(m1) .or. present(n1) .or. present(m2) .or. present(n2) .or. &
      present(m3) .or. present(n3)) then
    call uio_srhd(ncin, termt,ntt, type=intena, ident='dimension', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, dim,ident, mode=1)
      if (present(m1)) m1=dim(1,1)
      if (present(n1)) m1=dim(2,1)
      if (present(m2)) m2=dim(1,2)
      if (present(n2)) m2=dim(2,2)
      if (present(m3)) m3=dim(1,3)
      if (present(n3)) m3=dim(2,3)
    else
      outstr0='block dimension is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
    write(nc_p,'(A,6I6)') 'Dimension:', dim
  endif
  !
  ! --- Scalars ---
  !
  ! --- Time ---
  if (present(time) .or. present(time_db)) then
    ! --- Try to read double precision time in any case ---
    call uio_srhd(ncin, termt,ntt, type=realna, ident='time_db', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, time_db_tmp, prop%time_ident, name=prop%time_name, &
                                                                 unit=prop%time_unit)
      if (present(time_db)) time_db=time_db_tmp
      if (present(time   )) time   =time_db_tmp
    else
      ! --- Double precision time not present: use single precision time ---
      call uio_srhd(ncin, termt,ntt, type=realna, ident='time', &
                    identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                    nentry=ie2-ie1+1, &
                    outstr=outstr0,ierr=ierr0)
      if (ierr0 == 0) then
        call uio_rd(ncin, termt,ntt, time_tmp, prop%time_ident, name=prop%time_name, &
                                                                unit=prop%time_unit)
        if (present(time_db)) time_db=time_tmp
        if (present(time   )) time   =time_tmp
      else
        outstr0='time is not specified'
        ierr0=3
        call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
      endif
    endif
    write(nc_p,'(A,1PE15.8)') 'Time:               ', time_db
  endif
  !
  ! --- Time step number ---
  if (present(itime)) itime=itimelist(imodel)
  !
  ! --- Arrays: ---
  !
  ! --- Grid ---
  if (present(xc1)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='xc1', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, xc1,prop%xc1_ident, mode=2, &
                  name=prop%xc1_name, unit=prop%xc1_unit)
      ! --- Put axis into prop structure ---
      if (associated(prop%xc1)) deallocate(prop%xc1)
      allocate(prop%xc1(prop%m1:prop%n1))
      prop%xc1=xc1
    else
      outstr0='xc1 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(xc2)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='xc2', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, xc2,prop%xc2_ident, mode=2, &
                  name=prop%xc2_name, unit=prop%xc2_unit)
      ! --- Put axis into prop structure ---
      if (associated(prop%xc2)) deallocate(prop%xc2)
      allocate(prop%xc2(prop%m2:prop%n2))
      prop%xc2=xc2
    else
      outstr0='xc2 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(xc3)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='xc3', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, xc3,prop%xc3_ident, mode=2, &
                  name=prop%xc3_name, unit=prop%xc3_unit)
      ! --- Put axis into prop structure ---
      if (associated(prop%xc3)) deallocate(prop%xc3)
      allocate(prop%xc3(prop%m3:prop%n3))
      prop%xc3=xc3
    else
      outstr0='xc3 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(xb1)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='xb1', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, xb1,prop%xb1_ident, mode=2, &
                  name=prop%xb1_name, unit=prop%xb1_unit)
      ! --- Put axis and boundary values into prop structure ---
      if (associated(prop%xb1)) deallocate(prop%xb1)
      allocate(prop%xb1(prop%m1:prop%n1+1))
      prop%xb1=xb1
      prop%xb_l1=prop%xb1(prop%m1  )
      prop%xb_r1=prop%xb1(prop%n1+1)
    else
      outstr0='xb1 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(xb2)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='xb2', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, xb2,prop%xb2_ident, mode=2, &
                  name=prop%xb2_name, unit=prop%xb2_unit)
      ! --- Put axis and boundary values into prop structure ---
      if (associated(prop%xb2)) deallocate(prop%xb2)
      allocate(prop%xb2(prop%m2:prop%n2+1))
      prop%xb2=xb2
      prop%xb_l2=prop%xb2(prop%m2  )
      prop%xb_r2=prop%xb2(prop%n2+1)
    else
      outstr0='xb2 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(xb3)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='xb3', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, xb3,prop%xb3_ident, mode=2, &
                  name=prop%xb3_name, unit=prop%xb3_unit)
      ! --- Put axis and boundary values into prop structure ---
      if (associated(prop%xb3)) deallocate(prop%xb3)
      allocate(prop%xb3(prop%m3:prop%n3+1))
      prop%xb3=xb3
      prop%xb_l3=prop%xb3(prop%m3  )
      prop%xb_r3=prop%xb3(prop%n3+1)
    else
      outstr0='xb3 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  !
  ! --- Volume data ---
  !
  ! --- Density ---
  if (present(rho)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='rho', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, rho,prop%rho_ident, mode=2, &
                  name=prop%rho_name, unit=prop%rho_unit)
    else
      outstr0='density is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  !
  ! --- Energy ---
  if (present(ei)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='ei', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, ei,prop%ei_ident, mode=2, name=prop%ei_name, unit=prop%ei_unit)
    else
      outstr0='energy is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  !
  ! --- Velocity ---
  if (present(v1)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='v1', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, v1,prop%v1_ident, mode=2, name=prop%v1_name, unit=prop%v1_unit)
    else
      outstr0='v1 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(v2)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='v2', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, v2,prop%v2_ident, mode=2, name=prop%v2_name, unit=prop%v2_unit)
    else
      outstr0='v2 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  if (present(v3)) then
    call uio_srhd(ncin, termt,ntt, type=realna, ident='v3', &
                  identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                  nentry=ie2-ie1+1, &
                  outstr=outstr0,ierr=ierr0)
    if (ierr0 == 0) then
      call uio_rd(ncin, termt,ntt, v3,prop%v3_ident, mode=2, name=prop%v3_name, unit=prop%v3_unit)
    else
      outstr0='v3 is not specified'
      ierr0=3
      call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
    endif
  endif
  !
  ! --- Additional fields ---
  if (present(quc)) then
    if (ALLOASSOTED(quc)) then
      nquc=size(quc,4)
      do iquc=1,nquc
        write(qucstr,'(I3)') iquc
        qucstr='00' // adjustl(qucstr)
        qucstr='quc' // qucstr(len_trim(qucstr)-2:)
        call uio_srhd(ncin, termt,ntt, type=realna, ident=qucstr, &
                      identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                      nentry=ie2-ie1+1, &
                      outstr=outstr0,ierr=ierr0)
        if (ierr0 == 0) then
          call uio_rd(ncin, termt,ntt, quc(:,:,:,iquc), prop%quc_ident(iquc), mode=2, &
                      name=prop%quc_name(iquc), unit=prop%quc_unit(iquc))
        else
          outstr0=trim(qucstr) // ' is not specified'
          ierr0=3
          call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
        endif
      end do ! iquc
    endif
  endif
  !
  ! --- Magnetic field ---
  if (present(Bb1)) then
    if (ALLOASSOTED(Bb1)) then
      call uio_srhd(ncin, termt,ntt, type=realna, ident='bb1', &
                    identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                    nentry=ie2-ie1+1, &
                    outstr=outstr0,ierr=ierr0)
      if (ierr0 == 0) then
        call uio_rd(ncin, termt,ntt, Bb1,prop%B1_ident, mode=2, &
                    name=prop%B1_name, unit=prop%B1_unit)
        prop%Bb1_IsInFile_flag=.true.
      else
        outstr0='Bb1 is not specified'
        !ierr0=3
        Bb1=0.0
        !call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
        prop%B1_ident='bb1'
        prop%B1_name='Magnetic field 1'
        prop%B1_unit=''
        prop%Bb1_IsInFile_flag=.false.
      endif
    endif
  endif
  if (present(Bb2)) then
    if (ALLOASSOTED(Bb2)) then
      call uio_srhd(ncin, termt,ntt, type=realna, ident='bb2', &
                    identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                    nentry=ie2-ie1+1, &
                    outstr=outstr0,ierr=ierr0)
      if (ierr0 == 0) then
        call uio_rd(ncin, termt,ntt, Bb2,prop%B2_ident, mode=2, &
                    name=prop%B2_name, unit=prop%B2_unit)
        prop%Bb2_IsInFile_flag=.true.
      else
        outstr0='Bb2 is not specified'
        !ierr0=3
        !call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
        Bb2=0.0
        prop%B2_ident='bb2'
        prop%B2_name='Magnetic field 2'
        prop%B2_unit=''
        prop%Bb2_IsInFile_flag=.false.
      endif
    endif
  endif
  if (present(Bb3)) then
    if (ALLOASSOTED(Bb3)) then
      call uio_srhd(ncin, termt,ntt, type=realna, ident='bb3', &
                    identlist=identlist(1:3,ie1:ie2), poslist=poslist(ie1:ie2), &
                    nentry=ie2-ie1+1, &
                    outstr=outstr0,ierr=ierr0)
      if (ierr0 == 0) then
        call uio_rd(ncin, termt,ntt, Bb3,prop%B3_ident, mode=2, &
                    name=prop%B3_name, unit=prop%B3_unit)
        prop%Bb3_IsInFile_flag=.true.
      else
        outstr0='Bb3 is not specified'
        !ierr0=3
        !call uio_err(nc=ncin, outstr=outstr0, ierr=ierr0, routine='rhd_box_RdData', errlevel=3)
        Bb3=0.0
        prop%B3_ident='bb3'
        prop%B3_name='Magnetic field 3'
        prop%B3_unit=''
        prop%Bb3_IsInFile_flag=.false.
      endif
    endif
  endif
endif
!
if (index(command,'close') /= 0) then
  ! --- Close file ---
  call uio_closrd(ncin)
endif
!
end subroutine rhd_box_RdData

end module rhd_io_module
