!***************************************************************************************************
!
!   #####  #    # #####          #####  ######  #### 
!   #    # #    # #    #         #    # #      #    #
!   #    # ###### #    #         #    # #####  #     
!   #####  #    # #    #         #####  #      #     
!   #   #  #    # #    #         #   #  #      #    #
!   #    # #    # #####  ####### #    # ######  #### _module
!
!   Radiation-Hydrodynamics Code: reconstruction routines
!
!***************************************************************************************************
!   Fortran90
!   2018-05-09
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon, Naples
!   Werner Schaffenberger: Potsdam, Freiburg, East Lansing, Exeter
!***************************************************************************************************
! MODULES:
!   rhd_rec_module:        Reconstruction routines
!
!***************************************************************************************************

!------**************-------------------------------------------------------------------------------
module rhd_rec_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_module ('rhd_reconstruction_module')
!
! PURPOSE:
!   Provide reconstruction routines for rhd.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   use rhd_rec_module
!
! VARIABLES:
!   None
!
! ROUTINES: (all contained routines; public ones are marked with '>')
!   rhd_rec_ConstantEq:    Reconstruct partial wave (Constant), compute flux
!   rhd_rec_MinmodEq:      Reconstruct partial wave (Minmod), compute flux of wave, equidistant grid
!   rhd_rec_Minmod:        Reconstruct partial wave (Minmod), compute flux of wave, general grid
!   rhd_rec_vanLeerEq:     Reconstruct partial wave (vanLeer), compute flux on equidistant grid
!   rhd_rec_vanLeer:       Reconstruct partial wave (vanLeer), compute flux on general grid
!   rhd_rec_SuperBeeEq:    Reconstruct partial wave (SuperBee), compute flux on equidistant grid
!   rhd_rec_SuperBee:      Reconstruct partial wave (SuperBee), compute flux on general grid
!   rhd_rec_PPEq:          Reconstruct partial wave (PP), compute flux of wave on equidistant grid
!   rhd_rec_PP:            Reconstruct partial wave (PP), compute flux of wave on general grid
!   rhd_rec_PPmimoEq:      Reconstruct partial wave (PPmimo), compute flux of wave on equidist. grid
!   rhd_rec_PPmimo:        Reconstruct partial wave (PPmimo), compute flux of wave on general grid
!   rhd_rec_PPAWEq:        Reconstruct partial wave (PPAW), compute flux of wave on equidistant grid
!   rhd_rec_PPAW:          Reconstruct partial wave (PPAW), compute flux of wave on general grid
!   rhd_rec_FRmonoEq:      Reconstruct partial wave (FRmono), compute flux of wave on equidist. grid
!   rhd_rec_FRwenoEq:      Reconstruct partial wave (FRweno), compute flux of wave on equidist. grid
!   rhd_rec_FRcontEq:      Reconstruct partial wave (FRcont), compute flux of wave on equidist. grid
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!   rhd_rec_FluxalphaEqFRweno2: Reconstruct partial wave (FRweno2), compute flux on equidistant grid
!   rhd_rec_FluxalphaEqVAweno: Reconstruct partial wave (VAweno), compute flux on equidistant grid
!   rhd_rec_FluxalphaEqHBweno: Reconstruct partial wave (HBweno), compute flux on equidistant grid
! > rhd_rec_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   rhd_rec_FluxalphaVAweno: Reconstruct partial wave (VAweno), compute flux on general grid
!   rhd_rec_FluxalphaHBweno: Reconstruct partial wave (HBweno), compute flux on general grid
! > rhd_rec_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   rhd_rec_VecEqSimple:   Reconstruct vector, compute bnd values with 0th or 1st-order mono. scheme
!   rhd_rec_VecEqPPmimo:   Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecEqPP:       Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRmimo:   Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecEqFRmono:   Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRmono2:  Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqFRweno2:  Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqFRcont:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqVAweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqHBweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
! > rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_VecSimple:     Reconstruct vector, compute bnd values with 0th or 1st-order mono. scheme
!   rhd_rec_VecPPmimo:     Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecPP:         Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRmimo:     Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecFRmono:     Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRmono2:    Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRweno:     Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRweno2:    Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRcont:     Reconstruct vector, compute boundary values with 2nd-order scheme
! > rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!
! MODIFICATION HISTORY:
!   2011-09-08 (B.F. Lyon) Set of routines moved from rhd_hyd_module.F90
!   2011-11-18 (B.F.) Routines rhd_hyd_FluxalphaEq and rhd_hyd_Fluxalpha copied from rhd_hyd_module
!   2012-03-21 (B.F.) First version of routine rhd_hyd_RecVecEq
!   2012-04-16 (B.F.) Some routine names put into CamelCase
!   2013-02-24 (B.F.) Experiments with loop split in WENO schemes
!   2013-04-30 (B.F.) Some reconstruction schemes put into separate routines
!   2014-08-19 (B.F. Uppsala) First version of HBweno and VAweno routines
!   2014-08-20 (B.F.) Use parameters from rhd_gl_module
!   2014-08-26 (B.F.) Some more reconstruction schemes put into separate routines;
!                     only some are public
!   2015-04-21 (B.F.) Rename all subroutines: rhd_hyd_... -> rhd_rec_...
!   2018-05-09 (B.F.) Add dimension parameters to some routines
!---------------------------------------------------------------------------------------------------
implicit none
!
private
!
public rhd_rec_FluxalphaEq, rhd_rec_Fluxalpha, &
       rhd_rec_VecEq      , rhd_rec_Vec
!
contains

!----------******************-----------------------------------------------------------------------
subroutine rhd_rec_ConstantEq(dtime, alpha_p0, dxb1, lambda, &
                              flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_ConstantEq ('rhd_reconstruction_Constant_Equidistant')
!
! PURPOSE:
!   Compute flux of partial wave for constant value of alpha on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_ConstantEq(dtime, alpha_p0, dxb1, lambda, &
!                           flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_p0:       ('alpha_plus_0') real scalar with coefficients at index  0
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=0:
!       reconstruction_type=reconstruction_type_Constant: 
!                           constant values in cell assumed (simple, fast, and very dissipative)
!   The distribution of alpha inside the cell is assumed as constant.
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-12 (B.F.) Special case for this reconstruction, scalar version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_p0
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
! --- 0th order ---
! --- Constant ---
!
! --- Flux ---
flux_alpha_out=lambda * alpha_p0
!
end subroutine rhd_rec_ConstantEq


!----------****************-------------------------------------------------------------------------
subroutine rhd_rec_MinmodEq(dtime, alpha_m1, alpha_p0, alpha_p1, dxb1, lambda, &
                            flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_MinmodEq ('rhd_reconstruction_Minmod_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with Minmod and compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_MinmodEq(dtime, alpha_m1, alpha_p0, alpha_p1, dxb1, lambda, &
!                         flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_Minmod:
!                           Minmod (2nd order, but relatively dissipative)
!   The distribution of alpha inside the cell is reconstructed with a linear function (Minmod).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-06 (B.F.) Loop restructuring and fusion, formula for Minmod changed
!   2001-02-11 (B.F.) Special case for Minmod, scalar version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m1, alpha_p0, alpha_p1
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: delta_alpha
!---------------------------------------------------------------------------------------------------
!
! --- 1st order ---
! --- Minmod ---
!
! --- Slope ---
delta_alpha=( min( max( alpha_p0 - alpha_m1, 0.0 ), &
                   max( alpha_p1 - alpha_p0, 0.0 ) &
                 ) + &
              max( min( alpha_p0 - alpha_m1, 0.0 ), &
                   min( alpha_p1 - alpha_p0, 0.0 ) &
                 ) &
            )
! --- Flux ---
flux_alpha_out=lambda * &
               ( alpha_p0 + delta_alpha * sign(0.5,lambda) * &
                            (1.0 - abs(lambda)*(dtime/dxb1)) &
               )
!
end subroutine rhd_rec_MinmodEq


!----------**************---------------------------------------------------------------------------
subroutine rhd_rec_Minmod(dtime, alpha_m1, alpha_p0, alpha_p1, xb_m1, xb_p0, xb_p1, xb_p2, lambda, &
                          flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_Minmod ('rhd_reconstruction_Minmod')
!
! PURPOSE:
!   Reconstruct partial wave with Minmod and compute flux of partial wave.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_Minmod(dtime, alpha_m1, alpha_p0, alpha_p1, xb_m1, xb_p0, xb_p1, xb_p2, lambda, &
!                       flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   xb_m1:          ('x_boundary_minus_1') real, cell boundary at index -1
!   xb_p0:          ('x_boundary_plus_0') real, cell boundary at index 0
!   xb_p1:          ('x_boundary_plus_1') real, cell boundary at index 1
!   xb_p2:          ('x_boundary_plus_2') real, cell boundary at index 2
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_Minmod:
!                           Minmod (2nd order, but relatively dissipative)
!   The distribution of alpha inside the cell is reconstructed with a linear function (Minmod).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-02-06 (B.F.) Loop restructuring and fusion, formula for Minmod changed
!   2001-02-11 (B.F.) Special case for Minmod, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m1, alpha_p0, alpha_p1
real,                      intent(in)   :: xb_m1, xb_p0, xb_p1, xb_p2
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: delta_alpha
!---------------------------------------------------------------------------------------------------
!
! --- 1st order ---
! --- Minmod ---
!
! --- Slope ---
delta_alpha=2.0 * (xb_p1 - xb_p0) * &
            ( min( max( (alpha_p0 - alpha_m1)/ &
                        (xb_p1    - xb_m1   ), 0.0 &
                      ) , &
                   max( (alpha_p1 - alpha_p0)/ &
                        (xb_p2    - xb_p0   ), 0.0 &
                      ) &
                 ) + &
              max( min( (alpha_p0 - alpha_m1)/ &
                        (xb_p1    - xb_m1   ), 0.0 &
                      ) , &
                   min( (alpha_p1 - alpha_p0)/ &
                        (xb_p2    - xb_p0   ), 0.0 &
                      ) &
                 ) &
            )
! --- Flux ---
flux_alpha_out=lambda * &
               ( alpha_p0 + delta_alpha * sign(0.5,lambda) * &
                            (1.0 - abs(lambda)*(dtime/(xb_p1 - xb_p0))) &
               )
!
end subroutine rhd_rec_Minmod


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_vanLeerEq(dtime, alpha_m1, alpha_p0, alpha_p1, dxb1, lambda, &
                             flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_vanLeerEq ('rhd_reconstruction_van_Leer_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with van Leer slope, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_vanLeerEq(dtime, alpha_m1, alpha_p0, alpha_p1, dxb1, lambda, &
!                          flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_vanLeer:
!                           vanLeer (2nd order, good compromise)
!   The distribution of alpha inside the cell is reconstructed with a linear function (vanLeer).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m1, alpha_p0, alpha_p1
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: delta_alpha, dalphapro
!---------------------------------------------------------------------------------------------------
!
! --- 1st order ---
! --- vanLeer ---
!
! --- Slope ---
dalphapro=(alpha_p0 - alpha_m1) * &
          (alpha_p1 - alpha_p0)
if (dalphapro > 0.0) then
  delta_alpha=2.0 * dalphapro / (alpha_p1 - alpha_m1)
else
  delta_alpha=0.0
endif
!
! --- Flux ---
flux_alpha_out=lambda * &
               ( alpha_p0 + delta_alpha * sign(0.5,lambda) * &
                            (1.0 - abs(lambda)*(dtime/dxb1)) &
               )
!
end subroutine rhd_rec_vanLeerEq


!----------***************--------------------------------------------------------------------------
subroutine rhd_rec_vanLeer(dtime, alpha_m1, alpha_p0, alpha_p1, &
                           xb_m1, xb_p0, xb_p1, xb_p2, lambda,&
                           flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_vanLeer ('rhd_reconstruction_vanLeer')
!
! PURPOSE:
!   Reconstruct partial wave with vanLeer and compute flux of partial wave.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_vanLeer(dtime, alpha_m1, alpha_p0, alpha_p1, xb_m1, xb_p0, xb_p1, xb_p2, lambda, &
!                        flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   xb_m1:          ('x_boundary_minus_1') real, cell boundary at index -1
!   xb_p0:          ('x_boundary_plus_0') real, cell boundary at index 0
!   xb_p1:          ('x_boundary_plus_1') real, cell boundary at index 1
!   xb_p2:          ('x_boundary_plus_2') real, cell boundary at index 2
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_vanLeer:
!                           vanLeer (2nd order, good compromise)
!   The distribution of alpha inside the cell is reconstructed with a linear function (vanLeer).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-02-06 (B.F.) Loop restructuring and fusion, formula for Minmod changed
!   2001-02-11 (B.F.) Special case for Minmod, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m1, alpha_p0, alpha_p1
real,                      intent(in)   :: xb_m1, xb_p0, xb_p1, xb_p2
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: dalphapro, delta_alpha
!---------------------------------------------------------------------------------------------------
!
! --- 1st order ---
! --- vanLeer ---
!
! --- Slope ---
dalphapro=((alpha_p0  - alpha_m1) / &
           (xb_p1     - xb_m1   )) * &
          ((alpha_p1  - alpha_p0) / &
           (xb_p2     - xb_p0   ))
if (dalphapro > 0.0) then
  delta_alpha=4.0 * dalphapro  * (xb_p1 - xb_p0) / &
              ( (alpha_p0  - alpha_m1) / &
                (xb_p1     - xb_m1   ) + &
                (alpha_p1  - alpha_p0) / &
                (xb_p2     - xb_p0   )  )
else
  delta_alpha=0.0
endif
! --- Flux ---
flux_alpha_out=lambda * &
               ( alpha_p0 + delta_alpha * sign(0.5,lambda) * &
                            (1.0 - abs(lambda)*(dtime/(xb_p1 - xb_p0))) &
               )
!
end subroutine rhd_rec_vanLeer


!----------******************-----------------------------------------------------------------------
subroutine rhd_rec_SuperBeeEq(dtime, alpha_m1, alpha_p0, alpha_p1, dxb1, lambda, &
                              flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_SuperBeeEq ('rhd_reconstruction_SuperBee_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with SuperBee slope, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_SuperBeeEq(dtime, alpha_m1, alpha_p0, alpha_p1, dxb1, lambda, &
!                           flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_SuperBee:
!                           SuperBee (sharp shocks and discontinuities, shapes become box-like))
!   The distribution of alpha inside the cell is reconstructed with a linear function (SuperBee).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m1, alpha_p0, alpha_p1
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: delta_alpha
!---------------------------------------------------------------------------------------------------
!
! --- 1st order ---
! --- SuperBee ---
!
! --- Slope ---
delta_alpha= (     sign(1.0,alpha_p0 - alpha_m1) + &
                   sign(1.0,alpha_p1 - alpha_p0) &
             ) * &
             min( abs(      alpha_p0 - alpha_m1  ), &
                  abs(      alpha_p1 - alpha_p0  ), &
                  max(  abs(alpha_p0 - alpha_m1), &
                        abs(alpha_p1 - alpha_p0) &
                     )*0.5 &
                )
!
! --- Flux ---
flux_alpha_out=lambda * &
               ( alpha_p0 + delta_alpha * sign(0.5,lambda) * &
                            (1.0 - abs(lambda)*(dtime/dxb1)) &
               )
!
end subroutine rhd_rec_SuperBeeEq


!----------****************-------------------------------------------------------------------------
subroutine rhd_rec_SuperBee(dtime, alpha_m1, alpha_p0, alpha_p1, &
                            xb_m1, xb_p0, xb_p1, xb_p2, lambda,&
                            flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_SuperBee ('rhd_reconstruction_SuperBee')
!
! PURPOSE:
!   Reconstruct partial wave with SuperBee and compute flux of partial wave.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_SuperBee(dtime, alpha_m1, alpha_p0, alpha_p1, xb_m1, xb_p0, xb_p1, xb_p2, lambda,&
!                         flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   xb_m1:          ('x_boundary_minus_1') real, cell boundary at index -1
!   xb_p0:          ('x_boundary_plus_0') real, cell boundary at index 0
!   xb_p1:          ('x_boundary_plus_1') real, cell boundary at index 1
!   xb_p2:          ('x_boundary_plus_2') real, cell boundary at index 2
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_SuperBee:
!                           SuperBee (sharp shocks and discontinuities, shapes become box-like))
!   The distribution of alpha inside the cell is reconstructed with a linear function (SuperBee).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-02-06 (B.F.) Loop restructuring and fusion, formula for Minmod changed
!   2001-02-11 (B.F.) Special case for Minmod, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m1, alpha_p0, alpha_p1
real,                      intent(in)   :: xb_m1, xb_p0, xb_p1, xb_p2
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: delta_alpha
!---------------------------------------------------------------------------------------------------
!
! --- 1st order ---
! --- SuperBee ---
!
! --- Slope ---
delta_alpha= ( sign(1.0,alpha_p0 - alpha_m1) + &
               sign(1.0,alpha_p1 - alpha_p0) ) * &
             ( xb_p1 - xb_p0 ) * &
             min( abs( alpha_p0 - alpha_m1)/ &
                     ( xb_p0    - xb_m1   ), &
                  abs( alpha_p1 - alpha_p0)/ &
                     ( xb_p1    - xb_p0   ), &
                  max( abs( alpha_p0 - alpha_m1)/ &
                          ( xb_p1    - xb_m1   ), &
                       abs( alpha_p1 - alpha_p0)/ &
                          ( xb_p2    - xb_p0   ) &
                     ) &
                )
! --- Flux ---
flux_alpha_out=lambda * &
               ( alpha_p0 + delta_alpha * sign(0.5,lambda) * &
                            (1.0 - abs(lambda)*(dtime/(xb_p1 - xb_p0))) &
               )
!
end subroutine rhd_rec_SuperBee


!----------************-----------------------------------------------------------------------------
subroutine rhd_rec_PPEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
                        flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_PPEq ('rhd_reconstruction_Piecewise_Parabolic_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_PPEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
!                     flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PP:
!                           PiecewiseParabolic (sophisticated and time-consuming method from
!                           the PPM-scheme without extra sharpening)
!   The distribution of alpha inside the cell is reconstructed with a parabola (PP).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2011-11-21 (B.F.) Replacement: sign(1.0,y) -> sign(1.0,lambda), relevant for RungeKutta
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, a_2, y
!---------------------------------------------------------------------------------------------------
!
! --- 2nd order ---
! --- PP (Piecewise Parabolic) ---
!
! --- Grid ---
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! --- Average slope in zones (1.7) ---
dam1=0.5*(ap0-am2)
dap0=0.5*(ap1-am1)
dap1=0.5*(ap2-ap0)
!
! --- Steepening, monotonizing (1.8) ---
dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
     min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
     min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
     min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
!
! --- Preliminary boundary values (1.6) ---
am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
!
! --- Corrected boundary values (1.10) ---
if ((ap-ap0)*(ap0-am) <= 0.0) then
  ! --- Local extremum in input values ---
  aL=ap0
  aR=ap0
else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
  ! --- Inner extremum in parabola ---
  aL=3.0*ap0-2.0*ap
  aR=ap
else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
  ! --- Inner extremum in parabola ---
  aL=am
  aR=3.0*ap0-2.0*am
else
  ! --- Standard case ---
  aL=am
  aR=ap
endif
!
! --- Coefficients of parabola ---
a_0    =1.5*ap0-0.25*(aL+aR)
delta_a=aR-aL
a_2    =3.0*(aL+aR-2.0*ap0)
!
! --- Compute flux ---
y=lambda*dtime/dxb1
flux_alpha_out=lambda * &
               (a_0 + &
                delta_a * sign(0.5,lambda) * (1.0-abs(y)) + &
                a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
!
end subroutine rhd_rec_PPEq


!----------**********-------------------------------------------------------------------------------
subroutine rhd_rec_PP(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, &
                      xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3, lambda, &
                      flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_PP ('rhd_reconstruction_Piecewise_Parabolic')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_PP(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, &
!                   xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3, lambda, &
!                   flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   xb_m2:          ('x_boundary_minus_2') real, cell boundary at index -2
!   xb_m1:          ('x_boundary_minus_1') real, cell boundary at index -1
!   xb_p0:          ('x_boundary_plus_0') real, cell boundary at index 0
!   xb_p1:          ('x_boundary_plus_1') real, cell boundary at index 1
!   xb_p2:          ('x_boundary_plus_2') real, cell boundary at index 2
!   xb_p3:          ('x_boundary_plus_3') real, cell boundary at index 3
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PP:
!                           PiecewiseParabolic (sophisticated and time-consuming method from
!                           the PPM-scheme without extra sharpening)
!   The distribution of alpha inside the cell is reconstructed with a parabola (PP).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: dxm2, dxm1, dxp0, dxp1, dxp2, &
                                           am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, a_2, y
!---------------------------------------------------------------------------------------------------
!
! --- 2nd order ---
! --- PP (Piecewise Parabolic) ---
!
! --- Grid ---
dxm2=xb_m1-xb_m2
dxm1=xb_p0-xb_m1
dxp0=xb_p1-xb_p0
dxp1=xb_p2-xb_p1
dxp2=xb_p3-xb_p2
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! --- Average slope in zones (1.7) ---
dam1=  dxm1/(dxm2+dxm1+dxp0) * &
     ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
       (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
dap0=  dxp0/(dxm1+dxp0+dxp1) * &
     ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
       (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
dap1=  dxp1/(dxp0+dxp1+dxp2) * &
     ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
       (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
!
! --- Steepening, monotonizing (1.8) ---
dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
     min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
     min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
     min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
!
! --- Preliminary boundary values (1.6) ---
am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
   1.0/(dxm2+dxm1+dxp0+dxp1) * &
   ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
     ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
     (ap0-am1) - &
     dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0)*dap0 + &
     dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0)*dam1  )                
ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
   1.0/(dxm1+dxp0+dxp1+dxp2) * &
   ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
     ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
     (ap1-ap0) - &
     dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1)*dap1 + &
     dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1)*dap0  )
!
! --- Corrected boundary values (1.10) ---
if ((ap-ap0)*(ap0-am) <= 0.0) then
  ! --- Local extremum in input values ---
  aL=ap0
  aR=ap0
else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
  ! --- Inner extremum in parabola ---
  aL=3.0*ap0-2.0*ap
  aR=ap
else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
  ! --- Inner extremum in parabola ---
  aL=am
  aR=3.0*ap0-2.0*am
else
  ! --- Standard case ---
  aL=am
  aR=ap
endif
!
! --- Coefficients of parabola ---
a_0    =1.5*ap0-0.25*(aL+aR)
delta_a=aR-aL
a_2    =3.0*(aL+aR-2.0*ap0)
!
! --- Flux ---
y=lambda*dtime/dxp0
flux_alpha_out=lambda * &
               (a_0 + &
                delta_a * sign(0.5,lambda) * (1.0-abs(y)) + &
                a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
!
end subroutine rhd_rec_PP


!----------****************-------------------------------------------------------------------------
subroutine rhd_rec_PPmimoEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
                            flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_PPmimoEq ('rhd_reconstruction_Piecewise_Parabolic_MinMod_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with linear function, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_PPmimoEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
!                         flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PPmimo:
!                           PiecewiseParabolicLinear
!   The preliminary boundary values are computed with the same formula as used in the PP scheme.
!   However, from those only a linear function is computed -- not a parabola as in PP.
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2011-04-27 (B.F. Lyon) Derived from rhd_hyd_PPEq
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, y
!---------------------------------------------------------------------------------------------------
!
! --- 2nd order ---
! --- PPmimo (Piecewise Parabolic MinMod Linear) ---
!
! --- Grid ---
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! --- Average slope in zones (1.7) ---
dam1=0.5*(ap0-am2)
dap0=0.5*(ap1-am1)
dap1=0.5*(ap2-ap0)
!
! --- Steepening, monotonizing (1.8) ---
dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
     min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
     min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
     min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
!
! --- Preliminary boundary values (1.6) ---
am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
!
! --- Slope ---
delta_a=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
!
! --- Rest of coefficients (of parabola) ---
a_0    =ap0
!
! --- Compute flux ---
y=lambda*dtime/dxb1
flux_alpha_out=lambda * &
               (a_0 + &
                delta_a * sign(0.5,lambda) * (1.0-abs(y)))
!
end subroutine rhd_rec_PPmimoEq


!----------**************---------------------------------------------------------------------------
subroutine rhd_rec_PPmimo(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, &
                          xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3, lambda, &
                          flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_PPmimo ('rhd_reconstruction_Piecewise_Parabolic_MinMod')
!
! PURPOSE:
!   Reconstruct partial wave with linear function, compute flux of partial wave.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_PPmimo(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, &
!                       xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3, lambda, &
!                       flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   xb_m2:          ('x_boundary_minus_2') real, cell boundary at index -2
!   xb_m1:          ('x_boundary_minus_1') real, cell boundary at index -1
!   xb_p0:          ('x_boundary_plus_0') real, cell boundary at index 0
!   xb_p1:          ('x_boundary_plus_1') real, cell boundary at index 1
!   xb_p2:          ('x_boundary_plus_2') real, cell boundary at index 2
!   xb_p3:          ('x_boundary_plus_3') real, cell boundary at index 3
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PPmimo:
!                           PiecewiseParabolicLinear
!   The preliminary boundary values are computed with the same formula as used in the PP scheme.
!   However, from those only a linear function is computed -- not a parabola as in PP.
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2011-04-27 (B.F. Lyon) Derived from rhd_hyd_PPmimo
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: dxm2, dxm1, dxp0, dxp1, dxp2, &
                                           am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, y
!---------------------------------------------------------------------------------------------------
!
! --- 2nd order ---
! --- PPmimo (Piecewise Parabolic Linear) ---
!
! --- Grid ---
dxm2=xb_m1-xb_m2
dxm1=xb_p0-xb_m1
dxp0=xb_p1-xb_p0
dxp1=xb_p2-xb_p1
dxp2=xb_p3-xb_p2
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! --- Average slope in zones (1.7) ---
dam1=  dxm1/(dxm2+dxm1+dxp0) * &
     ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
       (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
dap0=  dxp0/(dxm1+dxp0+dxp1) * &
     ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
       (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
dap1=  dxp1/(dxp0+dxp1+dxp2) * &
     ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
       (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
!
! --- Steepening, monotonizing (1.8) ---
dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
     min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
     min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
     min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
!
! --- Preliminary boundary values (1.6) ---
am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
   1.0/(dxm2+dxm1+dxp0+dxp1) * &
   ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
     ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
     (ap0-am1) - &
     dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0)*dap0 + &
     dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0)*dam1  )                
ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
   1.0/(dxm1+dxp0+dxp1+dxp2) * &
   ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
     ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
     (ap1-ap0) - &
     dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1)*dap1 + &
     dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1)*dap0  )
!
! --- Slope ---
delta_a=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
!
! --- Rest of coefficients (of parabola) ---
a_0    =ap0
!
! --- Compute flux ---
y=lambda*dtime/dxp0
flux_alpha_out=lambda * &
               (a_0 + &
                delta_a * sign(0.5,lambda) * (1.0-abs(y)))
!
end subroutine rhd_rec_PPmimo


!----------**************---------------------------------------------------------------------------
subroutine rhd_rec_PPAWEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
                          flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_PPAWEq ('rhd_reconstruction_Piecewise_Parabolic_Anti_Wiggle_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_PPAWEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
!                       flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   muw:    ('mu_wiggle') real interpolation parameter to measure wiggliness; PP methods;
!           4th order discrete derivative, normalized
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PPAW:
!                           Mix to PP a bit of PPmimo if there are wiggles
!   The distribution of alpha inside the cell is reconstructed with a parabola (PP).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2011-04-27 (B.F. Lyon) Adapted from rhd_hyd_PPEq
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, a_2, y, delta_a_lin, muw
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! --- 2nd order ---
! --- PPAW (Piecewise Parabolic Anti Wiggle) ---
!
! --- Grid ---
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! --- Average slope in zones (1.7) ---
dam1=0.5*(ap0-am2)
dap0=0.5*(ap1-am1)
dap1=0.5*(ap2-ap0)
!
! --- Steepening, monotonizing (1.8) ---
dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
     min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
     min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
     min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
!
! --- Preliminary boundary values (1.6) ---
am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
!
! --- Corrected boundary values (1.10) ---
if ((ap-ap0)*(ap0-am) <= 0.0) then
  ! --- Local extremum in input values ---
  aL=ap0
  aR=ap0
else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
  ! --- Inner extremum in parabola ---
  aL=3.0*ap0-2.0*ap
  aR=ap
else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
  ! --- Inner extremum in parabola ---
  aL=am
  aR=3.0*ap0-2.0*am
else
  ! --- Standard case ---
  aL=am
  aR=ap
endif
!
! --- Slope for linear case ---
delta_a_lin=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
!
! --- Wiggliness ---
muw=abs((   (am1-am2) - 3.0*   (ap0-am1) + 3.0*   (ap1-ap0) -    (ap2-ap1)           )/ &
        (abs(am1-am2) + 3.0*abs(ap0-am1) + 3.0*abs(ap1-ap0) + abs(ap2-ap1) + eps_tiny) )
muw=sqrt(muw)
!
! --- Interpolated coefficients of parabola ---
a_0    =(1.0-muw)*(1.5*ap0-0.25*(aL+aR)) + muw*ap0
delta_a=(1.0-muw)*(aR-aL)                + muw*delta_a_lin
a_2    =(1.0-muw)*(3.0*(aL+aR-2.0*ap0))  + muw*0.0
!
! --- Compute flux ---
y=lambda*dtime/dxb1
flux_alpha_out=lambda * &
               (a_0 + &
                delta_a * sign(0.5,lambda) * (1.0-abs(y)) + &
                a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
!
end subroutine rhd_rec_PPAWEq


!----------************-----------------------------------------------------------------------------
subroutine rhd_rec_PPAW(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, &
                        xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3, lambda, &
                        flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_PPAW ('rhd_reconstruction_Piecewise_Parabolic_Anti_Wiggle')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_PPAW(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, &
!                     xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3, lambda, &
!                     flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   xb_m2:          ('x_boundary_minus_2') real, cell boundary at index -2
!   xb_m1:          ('x_boundary_minus_1') real, cell boundary at index -1
!   xb_p0:          ('x_boundary_plus_0') real, cell boundary at index 0
!   xb_p1:          ('x_boundary_plus_1') real, cell boundary at index 1
!   xb_p2:          ('x_boundary_plus_2') real, cell boundary at index 2
!   xb_p3:          ('x_boundary_plus_3') real, cell boundary at index 3
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   delta_a_lin: ('delta_alpha_linear') real, slope of linear reconstruction; PP methods
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PPAW:
!                           Mix to PP a bit of PPmimo if there are wiggles
!   The distribution of alpha inside the cell is reconstructed with a parabola (PP).
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2011-04-27 (B.F. Lyon) Adapted from rhd_hyd_PP
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: xb_m2, xb_m1, xb_p0, xb_p1, xb_p2, xb_p3
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: dxm2, dxm1, dxp0, dxp1, dxp2, &
                                           am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, a_2, y, delta_a_lin, muw
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! --- 2nd order ---
! --- PPAW (Piecewise Parabolic Anti Wiggle) ---
!
! --- Grid ---
dxm2=xb_m1-xb_m2
dxm1=xb_p0-xb_m1
dxp0=xb_p1-xb_p0
dxp1=xb_p2-xb_p1
dxp2=xb_p3-xb_p2
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! --- Average slope in zones (1.7) ---
dam1=  dxm1/(dxm2+dxm1+dxp0) * &
     ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
       (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
dap0=  dxp0/(dxm1+dxp0+dxp1) * &
     ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
       (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
dap1=  dxp1/(dxp0+dxp1+dxp2) * &
     ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
       (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
!
! --- Steepening, monotonizing (1.8) ---
dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
     min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
     min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
     min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
!
! --- Preliminary boundary values (1.6) ---
am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
   1.0/(dxm2+dxm1+dxp0+dxp1) * &
   ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
     ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
     (ap0-am1) - &
     dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0)*dap0 + &
     dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0)*dam1  )                
ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
   1.0/(dxm1+dxp0+dxp1+dxp2) * &
   ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
     ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
     (ap1-ap0) - &
     dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1)*dap1 + &
     dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1)*dap0  )
!
! --- Corrected boundary values (1.10) ---
if ((ap-ap0)*(ap0-am) <= 0.0) then
  ! --- Local extremum in input values ---
  aL=ap0
  aR=ap0
else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
  ! --- Inner extremum in parabola ---
  aL=3.0*ap0-2.0*ap
  aR=ap
else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
  ! --- Inner extremum in parabola ---
  aL=am
  aR=3.0*ap0-2.0*am
else
  ! --- Standard case ---
  aL=am
  aR=ap
endif
!
! --- Slope for linear case ---
delta_a_lin=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
!
! --- Wiggliness ---
muw=abs((   (am1-am2) - 3.0*   (ap0-am1) + 3.0*   (ap1-ap0) -    (ap2-ap1)           )/ &
        (abs(am1-am2) + 3.0*abs(ap0-am1) + 3.0*abs(ap1-ap0) + abs(ap2-ap1) + eps_tiny) )
muw=sqrt(muw)
!
! --- Interpolated coefficients of parabola ---
a_0    =(1.0-muw)*(1.5*ap0-0.25*(aL+aR)) + muw*ap0
delta_a=(1.0-muw)*(aR-aL)                + muw*delta_a_lin
a_2    =(1.0-muw)*(3.0*(aL+aR-2.0*ap0))  + muw*0.0
!
! --- Flux ---
y=lambda*dtime/dxp0
flux_alpha_out=lambda * &
               (a_0 + &
                delta_a * sign(0.5,lambda) * (1.0-abs(y)) + &
                a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
!
end subroutine rhd_rec_PPAW


!----------****************-------------------------------------------------------------------------
subroutine rhd_rec_FRmonoEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
                            flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FRmonoEq ('rhd_reconstruction_Frankensteins_Reconstruct_method_monotonic_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FRmonoEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
!                         flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_FRM:
!   The distribution of alpha inside the cell is reconstructed with a parabola.
!   The flux (over time dtime) is computed.
!   This is the monotonic version of FRM that fulfills the TVD constraint.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-05-12 (B.F. Lyon) First working IDL version
!   2011-05-18 (B.F.) Fortran version
!   2011-07-19 (B.F.) Bugfix: (ap0-0.5*(ptilR-ptilL)) -> (ap0-0.5*(ptilR+ptilL))
!   2011-11-10 (B.F.) Merge three if branches
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           pc_2m, pc_20, pc_2p, wem, we0, wep, wesum, &
                                           atilL, atilR, f, &
                                           ptilL, ptilR, pc_0, pc_1, pc_2, y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! === 2nd order ===
! === FRM (Frankenstein's Reconstruction Method) ===
!
! --- Set grid (dx=1), assign values as abbreviation ---
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! === Compute interpolation polynomials ========================================
!
! --- Compute boundary values of polynomials that match three adjacent cells ---
atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
!
! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
!
! === Compute preliminary boundary values ======================================
!
! --- Get weights from quadratic coefficients ---
wem=0.5/(abs(pc_2m) + eps_tiny)
we0=0.5/(abs(pc_20) + eps_tiny)
wep=0.5/(abs(pc_2p) + eps_tiny)
!
! --- Compute weighted averages of boundary values in a WENO style ---
atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
!
! --- Monotonize ---
atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
!
! === Construct preliminary boundary values of reconstruction polynomial =======
!
! --- Compute normalized weights of polynomials from quadratic coefficients ---
wem  =0.25/((pc_2m)**2 + eps_tiny)
we0  =0.50/((pc_20)**2 + eps_tiny)
wep  =0.25/((pc_2p)**2 + eps_tiny)
wesum=wem+we0+wep
wem  =wem/wesum
we0  =we0/wesum
wep  =wep/wesum
!
! --- Combine boundary values of polynomials according to normalized weights in a WENO style ---
ptilL=wem*atil2L + we0*atil1L + wep*atil0L
ptilR=wem*atil3R + we0*atil2R + wep*atil1R
!
! === Monotonize boundary values ===============================================
!if ((ap0-atilL)*(atilR-ap0) <= 0.0) then
!  ! --- Local extremum in preliminary boundary values: reconstruction constant ---
!  ptilL=ap0
!  ptilR=ap0
!  !
!else if ((ap0-atilL)*(ap0-ptilL) <= 0.0) then
!  ! --- Wrong sign of ptilL compared to preliminary boundary value: reconstruction constant ---
!  ptilL=ap0
!  ptilR=ap0
!  !
!else if ((ap0-atilR)*(ap0-ptilR) <= 0.0) then
!  ! --- Wrong sign of ptilR compared to preliminary boundary value: reconstruction constant ---
!  ptilL=ap0
!  ptilR=ap0
!  !
if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
    ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
    ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
  ! --- Local extremum in preliminary boundary values or               ---
  ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
  ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
  ! --- Reconstruction constant                                        ---
  ptilL=ap0
  ptilR=ap0
  !
else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
  ! --- Left side has smaller slope ---
  !
  f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
  if (f > 1.0) then
    ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides ---
    !ptilL=(ptilL-ap0)/f                       + ap0
    ptilL=atilL
    ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
  endif
  !
  if (abs(ptilR-ap0) > abs(atilR-ap0)) then
    ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
    ptilR=atilR
  endif
  !
  if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
    ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
    ptilR=3.0*ap0-2.0*ptilL
  endif
  !
else
  ! --- Right side has smaller slope ---
  !
  f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
  if (f > 1.0) then
    ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides ---
    ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
    !ptilR=(ptilR-ap0)/f                       + ap0
    ptilR=atilR
  endif
  !
  if (abs(ptilL-ap0) > abs(atilL-ap0)) then
    ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
    ptilL=atilL
  endif
  !
  if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
    ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
    ptilL=3.0*ap0-2.0*ptilR
  endif
  !
endif
!
! === Compute parabola coefficients from cell integral and boundary values =====
pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
pc_1=ptilR-ptilL
pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
!
! === Compute flux =============================================================
y=lambda*dtime/dxb1
flux_alpha_out=lambda * &
               (pc_0 + &
                pc_1 * 0.5*sign(1.0,lambda) * (1.0 - abs(y)) + &
                pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
!
end subroutine rhd_rec_FRmonoEq


!----------****************-------------------------------------------------------------------------
subroutine rhd_rec_FRwenoEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
                            flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FRwenoEq ('rhd_rec_Frankensteins_Rec_method_weighted_essentially_nonoscillatory_Equ')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave on equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FRwenoEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
!                         flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_FRM:
!   The distribution of alpha inside the cell is reconstructed with a parabola.
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-05-12 (B.F.) First working IDL version
!   2011-05-18 (B.F.) Fortran version
!   2011-07-07 (B.F.) ENO version
!   2011-11-10 (B.F.) Merge three if branches
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           pc_2m, pc_20, pc_2p, wem, we0, wep, wesum, &
                                           atilL, atilR, f, &
                                           ptilL, ptilR, pc_0, pc_1, pc_2, y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! === 2nd order ===
! === FRM (Frankenstein's Reconstruction Method) ===
!
! --- Set grid (dx=1), assign values as abbreviation ---
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! === Compute interpolation polynomials ========================================
!
! --- Compute boundary values of polynomials that match three adjacent cells ---
atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
!
! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
!
! === Compute preliminary boundary values ======================================
!
! --- Get weights from quadratic coefficients ---
wem=0.5/(abs(pc_2m) + eps_tiny)
we0=0.5/(abs(pc_20) + eps_tiny)
wep=0.5/(abs(pc_2p) + eps_tiny)
!
! --- Compute weighted averages of boundary values in a WENO style ---
atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
!
!! --- Monotonize ---
!atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
!atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
!
! === Construct preliminary boundary values of reconstruction polynomial =======
!
! --- Compute normalized weights of polynomials from quadratic coefficients ---
wem  =0.25/((pc_2m)**2 + eps_tiny)
we0  =0.50/((pc_20)**2 + eps_tiny)
wep  =0.25/((pc_2p)**2 + eps_tiny)
wesum=wem+we0+wep
wem  =wem/wesum
we0  =we0/wesum
wep  =wep/wesum
!
! --- Combine boundary values of polynomials according to normalized weights in a WENO style ---
ptilL=wem*atil2L + we0*atil1L + wep*atil0L
ptilR=wem*atil3R + we0*atil2R + wep*atil1R
!
! === Monotonize boundary values ===============================================
if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
    ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
    ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
  ! --- Local extremum in preliminary boundary values or               ---
  ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
  ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
  ! --- Reconstruction constant                                        ---
  ptilL=ap0
  ptilR=ap0
  !
else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
  ! --- Left side has smaller slope ---
  !
  f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
  if (f > 1.0) then
    ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides ---
    !ptilL=(ptilL-ap0)/f                       + ap0
    ptilL=atilL
    ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
  endif
  !
  if (abs(ptilR-ap0) > abs(atilR-ap0)) then
    ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
    ptilR=atilR
  endif
  !
  !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
  !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
  !  ptilR=3.0*ap0-2.0*ptilL
  !endif
  !
else
  ! --- Right side has smaller slope ---
  !
  f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
  if (f > 1.0) then
    ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides ---
    ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
    !ptilR=(ptilR-ap0)/f                       + ap0
    ptilR=atilR
  endif
  !
  if (abs(ptilL-ap0) > abs(atilL-ap0)) then
    ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
    ptilL=atilL
  endif
  !
  !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
  !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
  !  ptilL=3.0*ap0-2.0*ptilR
  !endif
  !
endif
!
! === Compute parabola coefficients from cell integral and boundary values =====
pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
pc_1=ptilR-ptilL
pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
!
! === Compute flux =============================================================
y=lambda*dtime/dxb1
flux_alpha_out=lambda * &
               (pc_0 + &
                pc_1 * 0.5*sign(1.0,lambda) * (1.0 - abs(y)) + &
                pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
!
end subroutine rhd_rec_FRwenoEq


!----------****************-------------------------------------------------------------------------
subroutine rhd_rec_FRcontEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
                            flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FRcontEq ('rhd_reconstruction_Frankensteins_Reconstruct_method_continuous_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave with parabola, compute flux of partial wave on equidistant grid.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FRcontEq(dtime, alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2, dxb1, lambda, &
!                         flux_alpha_out)
!
! INPUT:
!   dtime:          ('delta_time') real, time step
!   alpha_m2:       ('alpha_minus_2') real scalar with coefficients at index -2
!   alpha_m1:       ('alpha_minus_1') real scalar with coefficients at index -1
!   alpha_p0:       ('alpha_plus_0')  real scalar with coefficients at index  0
!   alpha_p1:       ('alpha_plus_1')  real scalar with coefficients at index +1
!   alpha_p2:       ('alpha_plus_2')  real scalar with coefficients at index +2
!   dxb1:           ('delta_x_boundary_1') real, grid cell size
!   lambda:         ('lambda') real with characteristic velocity
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') real scalar with flux
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Special case:
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_FRM:
!   The distribution of alpha inside the cell is reconstructed with a parabola.
!   The flux (over time dtime) is computed.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2011-05-12 (B.F.) First working IDL version
!   2011-05-18 (B.F.) Fortran version
!   2011-07-07 (B.F.) ENO version
!   2011-09-05 (B.F.) Continuous reconstruction
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: dtime
real,                      intent(in)   :: alpha_m2, alpha_m1, alpha_p0, alpha_p1, alpha_p2
real,                      intent(in)   :: dxb1
real,                      intent(in)   :: lambda
real,                      intent(out)  :: flux_alpha_out
!
! --- Local variables ---
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           pc_2m, pc_20, pc_2p, wem, we0, wep, wesum, &
                                           atilL, atilR, f, &
                                           ptilL, ptilR, pc_0, pc_1, pc_2, y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! === 2nd order ===
! === FRM (Frankenstein's Reconstruction Method) ===
!
! --- Set grid (dx=1), assign values as abbreviation ---
am2=alpha_m2
am1=alpha_m1
ap0=alpha_p0
ap1=alpha_p1
ap2=alpha_p2
!
! === Compute interpolation polynomials ========================================
!
! --- Compute boundary values of polynomials that match three adjacent cells ---
atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
!
! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
!
! === Compute preliminary boundary values ======================================
!
! --- Get weights from quadratic coefficients ---
wem=0.5/(abs(pc_2m) + eps_tiny)
we0=0.5/(abs(pc_20) + eps_tiny)
wep=0.5/(abs(pc_2p) + eps_tiny)
!
! --- Compute weighted averages of boundary values in a WENO style ---
atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
!
! --- Monotonize ---
!atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
!atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
!
! --- Set final := preliminary boundary values ---
ptilL=atilL
ptilR=atilR
!
! === Construct preliminary boundary values of reconstruction polynomial =======
!
! --- Compute normalized weights of polynomials from quadratic coefficients ---
!wem  =0.25/((pc_2m)**2 + eps_tiny)
!we0  =0.50/((pc_20)**2 + eps_tiny)
!wep  =0.25/((pc_2p)**2 + eps_tiny)
!wesum=wem+we0+wep
!wem  =wem/wesum
!we0  =we0/wesum
!wep  =wep/wesum
!
! --- Combine boundary values of polynomials according to normalized weights in a WENO style ---
!ptilL=wem*atil2L + we0*atil1L + wep*atil0L
!ptilR=wem*atil3R + we0*atil2R + wep*atil1R
!
! === Skip monotonization of boundary values ===================================
!
! === Compute parabola coefficients from cell integral and boundary values =====
pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
pc_1=ptilR-ptilL
pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
!
! === Compute flux =============================================================
y=lambda*dtime/dxb1
flux_alpha_out=lambda * &
               (pc_0 + &
                pc_1 * 0.5*sign(1.0,lambda) * (1.0 - abs(y)) + &
                pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
!
end subroutine rhd_rec_FRcontEq


!----------*************************----------------------------------------------------------------
subroutine rhd_rec_HBwenoSmoothness(a0, a1, a2, si)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_HBwenoSmoothness ('rhd_reconstruction_HandBrake_weno_Smoothness')
!
! PURPOSE:
!   Compute smoothness indicator for WENO scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_HBwenoSmoothness(a0, a1, a2, si)
!   
! INPUT:
!   a0:           ('a_0') real, value at left boundary of interval
!   a1:           ('a_1') real, value at center of interval
!   a2:           ('a_2') real, value at right boundary of interval
!
! OUTPUT:
!   si:           ('smoothness_indicator') real with smoothness of input polynomial
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The set of three input parameters should describe a parabola.
!   For optimal performance, the routine should be inlined.
!   The relative weight of linear and 2nd-order terms (the factor 8.0 below) is somewhat arbitrary.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-13 (B.F. Uppsala) First version
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
real,                      intent(in)   :: a0, a1, a2
real,                      intent(out)  :: si
!
! --- Local variables ---
real                                    :: si1, si2
!---------------------------------------------------------------------------------------------------
!
!si1=abs(a1-a0) + abs(a2-a1)
!si2=abs((a2-a1)-(a1-a0))
!
!si=si1 + 4.0*si2
!si=0.8*si1 + 6.4*si2
!si=sqrt(si1^2 + (4.0*si2)^2)
!
!si=si1 + 8.0*si2
si=(abs(a1-a0) + abs(a2-a1)) + 8.0*(abs((a2-a1)-(a1-a0)))
!
end subroutine rhd_rec_HBwenoSmoothness


!----------**************************---------------------------------------------------------------
subroutine rhd_rec_FluxalphaEqFRweno2(m1,n1, m2,n2, m3,n3, &
                                      reconstruction_order, &
                                      dtime, &
                                      SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                      alpha, dxb1, lambda, &
                                      flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FluxalphaEqFRweno2 ('rhd_reconstruction_Flux_alpha_Equidistant_Frankenstein_weno_2')
!
! PURPOSE:
!   Reconstruct partial wave and compute resulting flux with 2nd-order scheme for equidistant grid.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FluxalphaEqFRweno2(m1,n1, m2,n2, m3,n3, &
!                                   reconstruction_order, &
!                                   dtime, &
!                                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                                   alpha, dxb1, lambda, &
!                                   flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   alpha:        ('alpha') 4D real array with coeffs., run in cell alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real, grid cell size
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-06 (B.F.) Loop restructuring and fusion, formula for MinMod changed
!   2002-10-23 (B.F.) Removed case distinction for reconstruction_order
!   2011-04-27 (B.F. Lyon) 'PPL', 'PPAW' methods
!   2011-08-13 (B.F.) 'FRmo', 'FRen'
!   2011-11-10 (B.F.) Merge three if branches for 'FR', remove sign bug
!   2011-11-14 (B.F.) New parameters: SlopeRed_flag, SlopeRed
!   2011-11-15 (B.F.) SlopeRed also for FRen, new parameter SlopeCon
!   2011-11-16 (B.F.) New parameter: SlopeCon_flag
!   2011-11-17 (B.F.) Replacement: sign(1.0,y) -> sign(1.0,lambda(i1,i2,i3)), relevant for RungeK.
!   2011-11-18 (B.F.) Remove PPAW branch, add slope reduction and steepening for several cases
!   2011-11-21 (B.F.) New branches, renaming of new schemes including PPL->PPmimo
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2013-02-24 (B.F.) Experiments with loop split in WENO schemes
!   2013-04-30 (B.F.) Derived from rhd_hyd_FluxalphaEq
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           reconstruction_order
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3   ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: alpha
real,                      intent(in)   :: dxb1
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: lambda
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           y, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           pc_2m, pc_20, pc_2p, wem, we0, wep, wesum, &
                                           atilL, atilR, f, &
                                           ptilL, ptilR, pc_0, pc_1, pc_2, pmmtilL, pmmtilR
real, dimension(m1:n1+1)                :: atilLv, atilRv, ptilLv, ptilRv, ptilRnv,ptilLnv, fLv,fRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          atilLv(i1)=atilL
          atilRv(i1)=atilR
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          atilL=atilLv(i1)
          atilR=atilRv(i1)
          !
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ap0=alpha(i1, 0,i2,i3)


          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ap0=alpha(i1, 0,i2,i3)
          !
          atilL=atilLv(i1)
          atilR=atilRv(i1)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          atilLv(i1)=atilL
          atilRv(i1)=atilR
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          atilL=atilLv(i1)
          atilR=atilRv(i1)
          !
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ap0=alpha(i1, 0,i2,i3)
          !


          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ap0=alpha(i1, 0,i2,i3)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


!!$          ! --- Testing: new loop split ---
!!$          ptilLv(i1)=ptilL
!!$          ptilRv(i1)=ptilR
!!$          !
!!$          atilLv(i1)=atilL
!!$          atilRv(i1)=atilR
!!$          !
!!$          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
!!$          fLv(i1)=f
!!$          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
!!$          !
!!$          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
!!$          fRv(i1)=f
!!$          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
!!$        end do ! i1
!!$        !
!!$        !DIR$ SIMD
!!$        do i1=m1,n1+1
!!$          atilL=atilLv(i1)
!!$          atilR=atilRv(i1)
!!$          !
!!$          ptilL=ptilLv(i1)
!!$          ptilR=ptilRv(i1)
!!$          !
!!$          ap0=alpha(i1, 0,i2,i3)
!!$          !


          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilL=ptilLnv(i1)
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !


!!$          ! --- Testing: new loop split ---
!!$          ptilLv(i1)=ptilL
!!$          ptilRv(i1)=ptilR
!!$        end do ! i1
!!$        !
!!$        !DIR$ SIMD
!!$        do i1=m1,n1+1
!!$          ptilL=ptilLv(i1)
!!$          ptilR=ptilRv(i1)
!!$          !
!!$          ap0=alpha(i1, 0,i2,i3)
!!$          !


          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_FluxalphaEqFRweno2


!----------*************************----------------------------------------------------------------
subroutine rhd_rec_FluxalphaEqVAweno(m1,n1, m2,n2, m3,n3, &
                                     reconstruction_order, &
                                     dtime, &
                                     SlopeRed_flag, SlopeRed, &
                                     alpha, dxb1, lambda, &
                                     flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FluxalphaEqVAweno ('rhd_reconstruction_Flux_alpha_Equidistant_VAnilla_weno')
!
! PURPOSE:
!   Reconstruct partial wave and compute resulting flux with 2nd-order scheme for equidistant grid.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FluxalphaEqVAweno(m1,n1, m2,n2, m3,n3, &
!                                  reconstruction_order, &
!                                  dtime, &
!                                  SlopeRed_flag, SlopeRed, &
!                                  alpha, dxb1, lambda, &
!                                  flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   alpha:        ('alpha') 4D real array with coefficients, run in alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real, grid cell size
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-19 (B.F. Uppsala) Derived from rhd_hyd_FluxalphaEqHBweno
!   2014-08-20 (B.F.) Account for parameters from par file
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           reconstruction_order
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3   ), &
                           intent(in)   :: SlopeRed
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: alpha
real,                      intent(in)   :: dxb1
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: lambda
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: wA0, wA1, wA2, &
                                           am2, am1, ap0, ap1, ap2, &
                                                             atil2m1, atil3m1, &
                                                    atil1p0, atil2p0, &
                                           atil0p1, atil1p1, &
                                                             acen2m1, &
                                                    acen1p0, &
                                           acen0p1, &
                                                             si2m1, &
                                                    si1p0  , &
                                           si0p1  , &
                                           wp1L, wp0L, wm1L, wp1R, wp0R, wm1R, &
                                           wisum, &
                                           ptilL, ptilR, &
                                           pc_0, pc_1, pc_2, &
                                           y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    ! \label{f90:c_recwenoweightcenter__use1}
    ! --- Define weight array ---
    ! wA0=0.03 ; wA1=0.9 ; wA2=0.07
    wA1=par%C_RecWenoWeightCenter
    wA2=(1.0-wA1)*par%C_RecWenoWeightRight
    wA0=1.0-wA1-wA2
    !
    if (SlopeRed_flag) then ! ----------------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_FluxalphaEqVAweno


!----------*************************----------------------------------------------------------------
subroutine rhd_rec_FluxalphaEqHBweno(m1,n1, m2,n2, m3,n3, &
                                     reconstruction_order, &
                                     dtime, &
                                     SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                     alpha, dxb1, lambda, &
                                     flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FluxalphaEqHBweno ('rhd_reconstruction_Flux_alpha_Equidistant_HandBrake_weno')
!
! PURPOSE:
!   Reconstruct partial wave and compute resulting flux with 2nd-order scheme for equidistant grid.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FluxalphaEqHBweno(m1,n1, m2,n2, m3,n3, &
!                                  reconstruction_order, &
!                                  dtime, &
!                                  SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                                  alpha, dxb1, lambda, &
!                                  flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   alpha:        ('alpha') 4D real array with coefficients, run in alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real, grid cell size
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-18 (B.F. Uppsala) Derived from rhd_hyd_FluxalphaEqFRweno2 and rhd_hyd_RecVecEqHBweno
!   2014-08-20 (B.F.) Account for parameters from par file
!   2014-11-24 (B.F.) Interpolate differently for SlopeCon_flag case
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           reconstruction_order
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3   ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: alpha
real,                      intent(in)   :: dxb1
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: lambda
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: wA0, wA1, wA2, &
                                           am2, am1, ap0, ap1, ap2, &
                                                    atil1m1, atil2m1, atil3m1, &
                                           atil0p0, atil1p0, atil2p0, atil3p0, &
                                           atil0p1, atil1p1, atil2p1, &
                                                    acen1m1, acen2m1, &
                                           acen0p0, acen1p0, acen2p0, &
                                           acen0p1, acen1p1, &
                                                    si1m1  , si2m1, &
                                           si0p0  , si1p0  , si2p0, &
                                           si0p1  , si1p1  , &
                                           wtil1m1, wtil0p0, wtil1p0, wtil0p1, wtilsum, &
                                           atilL, atilR, amotilL, amotilR, &
                                           wp1L, wp0L, wm1L, wp1R, wp0R, wm1R, wp1, wp0, wm1, &
                                           wisum, &
                                           ptilL, ptilR, ptil0L, ptil0R, &
                                           si01, si02, si12, siTot, &
                                           delta_a, aL, aR, &
                                           pc_0, pc_1, pc_2, &
                                           y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    ! \label{f90:c_recwenoweightcenter__use2}
    ! --- Define weight array ---
    ! wA0=0.03 ; wA1=0.9 ; wA2=0.07
    wA1=par%C_RecWenoWeightCenter
    wA2=(1.0-wA1)*par%C_RecWenoWeightRight
    wA0=1.0-wA1-wA2
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil1m1=  ( 2.0/6.0)*am2 + (5.0/6.0)*am1 - ( 1.0/6.0)*ap0
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil0p0=  (11.0/6.0)*am1 - (7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3p0=  ( 2.0/6.0)*am1 - (7.0/6.0)*ap0 + (11.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2p1=- ( 1.0/6.0)*ap0 + (5.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen1m1=-( 1.0/24.0)*am2 + (26.0/24.0)*am1 - ( 1.0/24.0)*ap0
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen0p0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen2p0=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          acen1p1=-( 1.0/24.0)*ap0 + (26.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil1m1, acen1m1, atil2m1, si1m1)
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil0p0, acen0p0, atil1p0, si0p0)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil2p0, acen2p0, atil3p0, si2p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          call rhd_rec_HBwenoSmoothness(atil1p1, acen1p1, atil2p1, si1p1)
          !
          ! === Compute preliminary (continuous) cell-boundary values ====================
          !
          ! --- Compute weights from smoothness of adjacent polynomials ---
          wtil1m1=0.5/(si1m1+eps_tiny) + 0.5/(si2m1+eps_tiny)
          wtil0p0=0.5/(si0p0+eps_tiny) + 0.5/(si1p0+eps_tiny)
          wtilsum=wtil1m1+wtil0p0
          wtil1m1=wtil1m1/wtilsum
          wtil0p0=wtil0p0/wtilsum
          !
          wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
          wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
          wtilsum=wtil1p0+wtil0p1
          wtil1p0=wtil1p0/wtilsum
          wtil0p1=wtil0p1/wtilsum
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL  =wtil0p0*atil1p0 + wtil1m1*atil2m1
          atilR  =wtil0p1*atil1p1 + wtil1p0*atil2p0
          !
          ! --- Monotonize ---
          amotilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          amotilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wp1  =0.5*(wp1L+wp1R)
          wp0  =0.5*(wp0L+wp0R)
          wm1  =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1  =wp1/wisum
          wp0  =wp0/wisum
          wm1  =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          ptil0L=ptilL
          ptil0R=ptilR
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ! ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ! ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*ptil0L
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*ptil0R
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil1m1=  ( 2.0/6.0)*am2 + (5.0/6.0)*am1 - ( 1.0/6.0)*ap0
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil0p0=  (11.0/6.0)*am1 - (7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3p0=  ( 2.0/6.0)*am1 - (7.0/6.0)*ap0 + (11.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2p1=- ( 1.0/6.0)*ap0 + (5.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen1m1=-( 1.0/24.0)*am2 + (26.0/24.0)*am1 - ( 1.0/24.0)*ap0
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen0p0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen2p0=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          acen1p1=-( 1.0/24.0)*ap0 + (26.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil1m1, acen1m1, atil2m1, si1m1)
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil0p0, acen0p0, atil1p0, si0p0)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil2p0, acen2p0, atil3p0, si2p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          call rhd_rec_HBwenoSmoothness(atil1p1, acen1p1, atil2p1, si1p1)
          !
          ! === Compute preliminary (continuous) cell-boundary values ====================
          !
          ! --- Compute weights from smoothness of adjacent polynomials ---
          wtil1m1=0.5/(si1m1+eps_tiny) + 0.5/(si2m1+eps_tiny)
          wtil0p0=0.5/(si0p0+eps_tiny) + 0.5/(si1p0+eps_tiny)
          wtilsum=wtil1m1+wtil0p0
          wtil1m1=wtil1m1/wtilsum
          wtil0p0=wtil0p0/wtilsum
          !
          wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
          wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
          wtilsum=wtil1p0+wtil0p1
          wtil1p0=wtil1p0/wtilsum
          wtil0p1=wtil0p1/wtilsum
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL  =wtil0p0*atil1p0 + wtil1m1*atil2m1
          atilR  =wtil0p1*atil1p1 + wtil1p0*atil2p0
          !
          ! --- Monotonize ---
          amotilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          amotilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wp1  =0.5*(wp1L+wp1R)
          wp0  =0.5*(wp0L+wp0R)
          wm1  =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1  =wp1/wisum
          wp0  =wp0/wisum
          wm1  =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil1m1=  ( 2.0/6.0)*am2 + (5.0/6.0)*am1 - ( 1.0/6.0)*ap0
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil0p0=  (11.0/6.0)*am1 - (7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3p0=  ( 2.0/6.0)*am1 - (7.0/6.0)*ap0 + (11.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2p1=- ( 1.0/6.0)*ap0 + (5.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen1m1=-( 1.0/24.0)*am2 + (26.0/24.0)*am1 - ( 1.0/24.0)*ap0
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen0p0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen2p0=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          acen1p1=-( 1.0/24.0)*ap0 + (26.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil1m1, acen1m1, atil2m1, si1m1)
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil0p0, acen0p0, atil1p0, si0p0)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil2p0, acen2p0, atil3p0, si2p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          call rhd_rec_HBwenoSmoothness(atil1p1, acen1p1, atil2p1, si1p1)
          !
          ! === Compute preliminary (continuous) cell-boundary values ====================
          !
          ! --- Compute weights from smoothness of adjacent polynomials ---
          wtil1m1=0.5/(si1m1+eps_tiny) + 0.5/(si2m1+eps_tiny)
          wtil0p0=0.5/(si0p0+eps_tiny) + 0.5/(si1p0+eps_tiny)
          wtilsum=wtil1m1+wtil0p0
          wtil1m1=wtil1m1/wtilsum
          wtil0p0=wtil0p0/wtilsum
          !
          wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
          wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
          wtilsum=wtil1p0+wtil0p1
          wtil1p0=wtil1p0/wtilsum
          wtil0p1=wtil0p1/wtilsum
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL  =wtil0p0*atil1p0 + wtil1m1*atil2m1
          atilR  =wtil0p1*atil1p1 + wtil1p0*atil2p0
          !
          ! --- Monotonize ---
          amotilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          amotilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wp1  =0.5*(wp1L+wp1R)
          wp0  =0.5*(wp0L+wp0R)
          wm1  =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1  =wp1/wisum
          wp0  =wp0/wisum
          wm1  =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! \label{f90:c_recwenopsifactor__use1}
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_FluxalphaEqHBweno


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
                               reconstruction_order, reconstruction_type, &
                               dtime, &
                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                               alpha, dxb1, lambda, &
                               flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FluxalphaEq ('rhd_reconstruction_Flux_alpha_Equidistant')
!
! PURPOSE:
!   Reconstruct partial wave and compute resulting flux of partial wave for equidistant grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FluxalphaEq(m1,n1, m2,n2, m3,n3, &
!                            reconstruction_order, reconstruction_type, &
!                            dtime, &
!                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                            alpha, dxb1, lambda, &
!                            flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   alpha:        ('alpha') 4D real array with coeffs., run in cell alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real, grid cell size
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   delta_alpha:  ('delta_alpha') real, slope of reconstruction of linear methods
!   dalphapro:    ('delta_alpha_produce') real, product of slopes as used by VanLeer method
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   dam1:         ('delta_alpha_minus_1') real, slope in cell to the left; PP methods
!   dap0:         ('delta_alpha_plus_0') real, slope in cell; PP methods
!   dap1:         ('delta_alpha_plus_1') real, slope in cell to the right; PP methods
!   am:           ('alpha_minus') real, preliminary boundary value on left side; PP methods
!   ap:           ('alpha_plus') real, preliminary boundary value on right side; PP methods
!   aL:           ('alpha_Left') real, preliminary boundary value on left side; PP methods
!   a_0:          ('alpha_0') real, 0th coefficient of parabola; PP methods
!   delta_a:      ('alpha_0') real, 1st coefficient of parabola (slope); PP methods
!   a_2:          ('alpha_2') real, 2nd coefficient of parabola; PP methods
!   y:            ('y') real, lambda*dtime/dx; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   rhd_rec_FluxalphaEqVAweno: Reconstruct partial wave (VAweno), compute flux on equidistant grid
!   rhd_rec_FluxalphaEqHBweno: Reconstruct partial wave (HBweno), compute flux on equidistant grid
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   Obviously, the grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!   Possible types of reconstruction:
!     reconstruction_order=0:
!       reconstruction_type=reconstruction_type_Constant: 
!         constant values in cell assumed (simple, fast, and very dissipative)
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_Superbee:
!         Superbee (sharp shocks and discontinuities, shapes become box-like)
!       reconstruction_type=reconstruction_type_Minmod:
!         Minmod (2nd order, but relatively dissipative)
!       reconstruction_type=reconstruction_type_VanLeer: VanLeer (good compromise)
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PPmimo:
!         Merely linear reconstruction with boundary values from PP method
!       reconstruction_type=reconstruction_type_PP:
!         PiecewiseParabolic (sophisticated and time-consuming method from
!         the PPM-scheme without extra sharpening)
!       reconstruction_type=reconstruction_type_FRmimo:
!         Merely linear reconstruction with boundary values from FRmono method
!       reconstruction_type=reconstruction_type_FRmono:
!         Monotonic scheme that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature
!       reconstruction_type=reconstruction_type_FRmono2:
!         Monotonic scheme that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature.
!         The slope reduction is more aggressive (dissipative) than for FRmono
!       reconstruction_type=reconstruction_type_FRweno:
!         Non-monotonic scheme that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature
!       reconstruction_type=reconstruction_type_FRweno2:
!         Non-monotonic scheme that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature.
!         The slope reduction is more aggressive (dissipative) than for FRweno
!       reconstruction_type=reconstruction_type_FRcont:
!         Non-monotonic (and fairly unstable) scheme that uses only continuous boundary values
!         (useful for test at small Mach numbers)
!       reconstruction_type=reconstruction_type_VAweno:
!         Non-monotonic vanilla 2nd-order WENO scheme. Not extremely stable but useful.
!         Very accurate if it works.
!       reconstruction_type=reconstruction_type_HBweno:
!         Non-monotonic vanilla 2nd-order scheme, constructed by adding a stabilizing 'handbrake'
!         to a standard WENO scheme. Accuracy and stability depend on control parameters.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-06 (B.F.) Loop restructuring and fusion, formula for MinMod changed
!   2002-10-23 (B.F.) Removed case distinction for reconstruction_order
!   2011-04-27 (B.F. Lyon) 'PPL', 'PPAW' methods
!   2011-08-13 (B.F.) 'FRmo', 'FRen'
!   2011-11-10 (B.F.) Merge three if branches for 'FR', remove sign bug
!   2011-11-14 (B.F.) New parameters: SlopeRed_flag, SlopeRed
!   2011-11-15 (B.F.) SlopeRed also for FRen, new parameter SlopeCon
!   2011-11-16 (B.F.) New parameter: SlopeCon_flag
!   2011-11-17 (B.F.) Replacement: sign(1.0,y) -> sign(1.0,lambda(i1,i2,i3)), relevant for RungeK.
!   2011-11-18 (B.F.) Remove PPAW branch, add slope reduction and steepening for several cases
!   2011-11-21 (B.F.) New branches, renaming of new schemes including PPL->PPmimo
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2013-02-24 (B.F.) Experiments with loop split in WENO schemes
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2  , &
                m3:n3     ), &
                           intent(in)   :: alpha
real,                      intent(in)   :: dxb1
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3     ), &
                           intent(in)   :: lambda
real, dimension(m1:n1+1, &
                m2:n2  , &
                m3:n3     ), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: delta_alpha, dalphapro
real                                    :: am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, a_2, y, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           pc_2m, pc_20, pc_2p, wem, we0, wep, wesum, &
                                           atilL, atilR, f, &
                                           ptilL, ptilR, pc_0, pc_1, pc_2, pmmtilL, pmmtilR
real, dimension(m1:n1+1)                :: atilLv, atilRv, ptilLv, ptilRv !, ptilRnv,ptilLnv, fLv,fRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! --- Reconstruction ---
select case (reconstruction_type)
  !
  ! === 0th order ==================================================================================
  case(reconstruction_type_Constant) ! -------------------------------------------------------------
    ! --- Donor cell with constant reconstruction (delta_alpha=0.0) ---
    !
    ! --- Compute flux (delta_alpha=0.0) ---
    flux_alpha_out(m1:n1+1,:,:)=lambda(m1:n1+1,:,:) * alpha(m1:n1+1,0,:,:)
    !
  ! === 1st order ==================================================================================
  case(reconstruction_type_Minmod) ! ---------------------------------------------------------------
    ! --- Minmod ---
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute slope ---
          delta_alpha=( min( max( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                             max( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                           ) + &
                        max( min( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                             min( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                           ) &
                    )
          !
          ! --- Reduce slope ---
          delta_alpha=SlopeRed(i1,i2,i3)*delta_alpha
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3))* &
                                     (1.0 - abs(lambda(i1,i2,i3))*(dtime/dxb1)) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute slope ---
          delta_alpha=( min( max( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                             max( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                           ) + &
                        max( min( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                             min( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                           ) &
                    )
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*(dtime/dxb1)) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_VanLeer) ! --------------------------------------------------------------
    ! --- Van Leer ---
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute vanLeer slope ---
          dalphapro=(alpha(i1,0,i2,i3) - alpha(i1,-1,i2,i3)) * &
                    (alpha(i1,1,i2,i3) - alpha(i1, 0,i2,i3))
          if (dalphapro > 0.0) then
            delta_alpha=2.0 * dalphapro / (alpha(i1,1,i2,i3) - alpha(i1,-1,i2,i3))
          else
            delta_alpha=0.0
          endif
          !
          ! --- Compute MinMod slope ---
          delta_a=( min( max( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                         max( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                       ) + &
                    max( min( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                         min( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                       ) &
                   )
          !
          ! --- Interpolate between vanLeer and MinMod slope ---
          delta_alpha=SlopeRed(i1,i2,i3)*delta_alpha + (1.0-SlopeRed(i1,i2,i3))*delta_a
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*(dtime/dxb1)) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
!        do i1=m1,n1+1
!          signslope(i1)=sign(1.0, alpha(i1,1,i2,i3) - alpha(i1,-1,i2,i3))
!        end do ! i1
        do i1=m1,n1+1
          ! --- Compute slope ---
          dalphapro=(alpha(i1,0,i2,i3) - alpha(i1,-1,i2,i3)) * &
                    (alpha(i1,1,i2,i3) - alpha(i1, 0,i2,i3))
          if (dalphapro > 0.0) then
            delta_alpha=2.0 * dalphapro / (alpha(i1,1,i2,i3) - alpha(i1,-1,i2,i3))
          else
            delta_alpha=0.0
          endif
!          delta_alpha=2.0 * max(dalphapro, 0.0) * signslope(i1) / &
!                      max(abs(alpha(i1,1,i2,i3) - alpha(i1,-1,i2,i3)), 1.0E-36)
!          delta_alpha=2.0 * max(dalphapro, 0.0) * &
!                      sign(1.0, alpha(i1,1,i2,i3) - alpha(i1,-1,i2,i3)) / &
!                      max(abs(alpha(i1,1,i2,i3) - alpha(i1,-1,i2,i3)), 1.0E-36)
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*(dtime/dxb1)) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_Superbee) ! -------------------------------------------------------------
    ! --- Superbee ---
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute Superbee slope ---
          delta_alpha= (     sign(1.0,alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)) + &
                             sign(1.0,alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)) &
                       ) * &
                       min( abs(      alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)  ), &
                            abs(      alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)  ), &
                            max(  abs(alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)), &
                                  abs(alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)) &
                               )*0.5 &
                          )
          !
          ! --- Compute MinMod slope ---
          delta_a  =( min( max( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                           max( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                         ) + &
                      max( min( alpha(  i1,0,i2,i3) - alpha(i1,-1,i2,i3), 0.0 ), &
                           min( alpha(  i1,1,i2,i3) - alpha(i1, 0,i2,i3), 0.0 ) &
                         ) &
                    )
          !
          ! --- Interpolate between Superbee and MinMod slope ---
          delta_alpha=SlopeRed(i1,i2,i3)*delta_alpha + (1.0-SlopeRed(i1,i2,i3))*delta_a
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*(dtime/dxb1)) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Compute slope ---
          delta_alpha= (     sign(1.0,alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)) + &
                             sign(1.0,alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)) &
                       ) * &
                       min( abs(      alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)  ), &
                            abs(      alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)  ), &
                            max(  abs(alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)), &
                                  abs(alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)) &
                               )*0.5 &
                          )
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*(dtime/dxb1)) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  ! === 2nd order ==================================================================================
  case(reconstruction_type_PPmimo) ! ---------------------------------------------------------------
    ! --- Take boundary values from 'Piecewise Parabolic', do only linear MinMod reconstruction ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Abbreviate cell values (grid is constant) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=0.5*(ap0-am2)
          dap0=0.5*(ap1-am1)
          dap1=0.5*(ap2-ap0)
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1), 2.0*abs(ap0-am1), 2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0), 2.0*abs(ap1-ap0), 2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1), 2.0*abs(ap2-ap1), 2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
          ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          ! --- Compute MinMod boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Interpolate between computed and continuous boundary values ---
          aL=SlopeCon(i1,i2,i3)*aL + (1.0-SlopeCon(i1,i2,i3))*am
          aR=SlopeCon(i1,i2,i3)*aR + (1.0-SlopeCon(i1,i2,i3))*ap
          !
          ! --- Compute parabola coefficients from cell integral and boundary values ---
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Abbreviate cell values (grid is constant) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=0.5*(ap0-am2)
          dap0=0.5*(ap1-am1)
          dap1=0.5*(ap2-ap0)
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1), 2.0*abs(ap0-am1), 2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0), 2.0*abs(ap1-ap0), 2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1), 2.0*abs(ap2-ap1), 2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
          ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Compute (rest of) parabola coefficients from cell integral and boundary values ---
          a_0    =ap0
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Abbreviate cell values (grid is constant) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=0.5*(ap0-am2)
          dap0=0.5*(ap1-am1)
          dap1=0.5*(ap2-ap0)
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1), 2.0*abs(ap0-am1), 2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0), 2.0*abs(ap1-ap0), 2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1), 2.0*abs(ap2-ap1), 2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
          ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Compute (rest of) parabola coefficients from cell integral and boundary values ---
          a_0    =ap0
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_PP) ! -------------------------------------------------------------------
    ! --- Piecewise Parabolic ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Abbreviate cell values (grid is constant) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=0.5*(ap0-am2)
          dap0=0.5*(ap1-am1)
          dap1=0.5*(ap2-ap0)
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1), 2.0*abs(ap0-am1), 2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0), 2.0*abs(ap1-ap0), 2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1), 2.0*abs(ap2-ap1), 2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary (continuous) boundary values (1.6) ---
          am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
          ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
          !
          ! --- Correct boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          ! --- Compute MinMod boundary values ---
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          aL=SlopeRed(i1,i2,i3)*aL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          aR=SlopeRed(i1,i2,i3)*aR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          aL=SlopeCon(i1,i2,i3)*aL + (1.0-SlopeCon(i1,i2,i3))*am
          aR=SlopeCon(i1,i2,i3)*aR + (1.0-SlopeCon(i1,i2,i3))*ap
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Abbreviate cell values (grid is constant) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=0.5*(ap0-am2)
          dap0=0.5*(ap1-am1)
          dap1=0.5*(ap2-ap0)
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1), 2.0*abs(ap0-am1), 2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0), 2.0*abs(ap1-ap0), 2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1), 2.0*abs(ap2-ap1), 2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary (continuous) boundary values (1.6) ---
          am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
          ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          ! --- Compute MinMod boundary values ---
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          aL=SlopeRed(i1,i2,i3)*aL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          aR=SlopeRed(i1,i2,i3)*aR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Abbreviate cell values (grid is constant) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=0.5*(ap0-am2)
          dap0=0.5*(ap1-am1)
          dap1=0.5*(ap2-ap0)
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1), 2.0*abs(ap0-am1), 2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0), 2.0*abs(ap1-ap0), 2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1), 2.0*abs(ap2-ap1), 2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary (continuous) boundary values (1.6) ---
          am=0.5 * (ap0+am1) + 0.16666666666666666666 * (-dap0+dam1)
          ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * (-dap1+dap0)
          !
          ! --- Correct boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! --- Compute parabola coefficients from cell integral and boundary values ---
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_FRmimo) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method MinMod slope ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          ! --- Compute MinMod boundary values ---
          ptilL=ap0-delta_a
          ptilR=ap0+delta_a
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,atilR-ap0)+sign(1.0,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          a_0=ap0
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,atilR-ap0)+sign(1.0,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          a_0=ap0
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRmono) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ! --- to get preliminary boundary values ---
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Monotonize ---
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style ---
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !

!          ptilLv(i1)=ptilL
!          ptilRv(i1)=ptilR
!          atilLv(i1)=atilL
!          atilRv(i1)=atilR
!        end do ! i1
!        !
!        do i1=m1,n1+1
!          ptilL=ptilLv(i1)
!          ptilR=ptilRv(i1)
!          atilL=atilLv(i1)
!          atilR=atilRv(i1)
!          ap0=alpha(i1, 0,i2,i3)

          ! === Monotonize boundary values ===============================================
!          if ((ap0-atilL)*(atilR-ap0) <= 0.0) then
!            ! --- Local extremum in preliminary boundary values: reconstruction constant ---
!            ptilL=ap0
!            ptilR=ap0
!            !
!          else if ((ap0-atilL)*(ap0-ptilL) <= 0.0) then
!            ! --- Wrong sign of ptilL compared to preliminary boundary value: reconstruct constant --
!            ptilL=ap0
!            ptilR=ap0
!            !
!          else if ((ap0-atilR)*(ap0-ptilR) <= 0.0) then
!            ! --- Wrong sign of ptilR compared to preliminary boundary value: reconstruct constant --
!            ptilL=ap0
!            ptilR=ap0
!            !
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_FRmono2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic 2 (more efficient slope reduction) ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ! --- to get preliminary boundary values ---
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Monotonize ---
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides --
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides --
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_FRweno) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_FRweno2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory (more efficient slope reduction) ---
    !
!!$    call rhd_rec_FluxalphaEqFRweno2(m1,n1, m2,n2, m3,n3, &
!!$                                    reconstruction_order, &
!!$                                    dtime, &
!!$                                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!!$                                    alpha, dxb1, lambda, &
!!$                                    flux_alpha_out)
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          atilLv(i1)=atilL
          atilRv(i1)=atilR
          !
          !f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          !fLv(i1)=f
          !ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !!
          !f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          !fRv(i1)=f
          !ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          atilL=atilLv(i1)
          atilR=atilRv(i1)
          !
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ap0=alpha(i1, 0,i2,i3)


          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides --
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilL=ptilLnv(i1)
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ap0=alpha(i1, 0,i2,i3)


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style ---
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_FRcont) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Continuous Essentially Non-oscillatory ---
    !
    do i3=m3,n3
      do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil0L=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1L=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil1R=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2L=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil2R=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3R=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          !
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          pc_2m=      0.5 *am2 -           am1 +       0.5 *ap0
          pc_20=      0.5 *am1 -           ap0 +       0.5 *ap1
          pc_2p=      0.5 *ap0 -           ap1 +       0.5 *ap2
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Set final := preliminary boundary values ---
          ptilL=atilL
          ptilR=atilR
          !
          ! === Skip construction and monotonization of boundary values ==================
          !
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                   pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                   pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
      end do ! i2
    end do ! i3
    !
  case(reconstruction_type_VAweno) ! ---------------------------------------------------------------
    ! --- Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_FluxalphaEqVAweno(m1,n1, m2,n2, m3,n3, &
                                   reconstruction_order, &
                                   dtime, &
                                   SlopeRed_flag, SlopeRed, &
                                   alpha, dxb1, lambda, &
                                   flux_alpha_out)
    !
  case(reconstruction_type_HBweno) ! ---------------------------------------------------------------
    ! --- HandBrake Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_FluxalphaEqHBweno(m1,n1, m2,n2, m3,n3, &
                                   reconstruction_order, &
                                   dtime, &
                                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                   alpha, dxb1, lambda, &
                                   flux_alpha_out)
    !
end select
!
end subroutine rhd_rec_FluxalphaEq


!----------***********************------------------------------------------------------------------
subroutine rhd_rec_FluxalphaVAweno(m1,n1, m2,n2, m3,n3, nghost1, &
                                   reconstruction_order, &
                                   dtime, &
                                   SlopeRed_flag, SlopeRed, &
                                   alpha, dxb1, i_up, lambda, &
                                   flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FluxalphaVAweno ('rhd_reconstruction_Flux_alpha_VAnilla_weno')
!
! PURPOSE:
!   Reconstruct partial wave and compute resulting flux with 2nd-order scheme for general grid.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FluxalphaVAweno(m1,n1, m2,n2, m3,n3, nghost1, &
!                                reconstruction_order, &
!                                dtime, &
!                                SlopeRed_flag, SlopeRed, &
!                                alpha, dxb1, i_up, lambda, &
!                                flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   alpha:        ('alpha') 4D real array with coefficients, run in alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real vector, grid cell size
!   i_up:         ('index_upwind') 3D real array with indices of upwind direction
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The changes of the grid sizes (in x1 direction) should not be too large.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-19 (B.F. Uppsala) Derived from rhd_hyd_FluxalphaEqHBweno
!   2014-08-20 (B.F.) Account for parameters from par file
!   2014-08-25 (B.F.) Derived from rhd_hyd_FluxalphaEqVAweno
!   2014-10-26 (B.F.) New parameter: nghost1
!   2014-12-03 (B.F.) Add parameter dxb1; remove parameter xb1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3   ), &
                           intent(in)   :: SlopeRed
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: alpha
real, dimension(m1-nghost1:n1+nghost1  ), &
                           intent(in)   :: dxb1
integer, dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3     ), &
                           intent(in)   :: i_up
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: lambda
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3, i
real                                    :: wA0, wA1, wA2, &
                                           am2, am1, ap0, ap1, ap2, &
                                                             atil2m1, atil3m1, &
                                                    atil1p0, atil2p0, &
                                           atil0p1, atil1p1, &
                                                             acen2m1, &
                                                    acen1p0, &
                                           acen0p1, &
                                                             si2m1, &
                                                    si1p0  , &
                                           si0p1  , &
                                           wp1L, wp0L, wm1L, wp1R, wp0R, wm1R, &
                                           wisum, &
                                           ptilL, ptilR, &
                                           pc_0, pc_1, pc_2, &
                                           y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    ! --- Define weight array ---
    ! wA0=0.03 ; wA1=0.9 ; wA2=0.07
    wA1=par%C_RecWenoWeightCenter
    wA2=(1.0-wA1)*par%C_RecWenoWeightRight
    wA0=1.0-wA1-wA2
    !
    if (SlopeRed_flag) then ! ----------------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          i   =i_up(i1,i2,i3)
          y   =lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          i   =i_up(i1,i2,i3)
          y   =lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_FluxalphaVAweno


!----------***********************------------------------------------------------------------------
subroutine rhd_rec_FluxalphaHBweno(m1,n1, m2,n2, m3,n3, nghost1, &
                                   reconstruction_order, &
                                   dtime, &
                                   SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                   alpha, dxb1, i_up, lambda, &
                                   flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_FluxalphaHBweno ('rhd_reconstruction_Flux_alpha_HandBrake_weno')
!
! PURPOSE:
!   Reconstruct partial wave and compute resulting flux with 2nd-order scheme for general grid.
!
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_FluxalphaHBweno(m1,n1, m2,n2, m3,n3, nghost1, &
!                                reconstruction_order, &
!                                dtime, &
!                                SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                                alpha, dxb1, i_up, lambda, &
!                                flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   alpha:        ('alpha') 4D real array with coefficients, run in alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real vector, grid cell size
!   i_up:         ('index_upwind') 3D real array with indices of upwind direction
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   The grid (in x1 direction) should be equidistant.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-18 (B.F. Uppsala) Derived from rhd_hyd_FluxalphaEqFRweno2 and rhd_hyd_RecVecEqHBweno
!   2014-08-20 (B.F.) Account for parameters from par file
!   2014-08-25 (B.F.) Derived from rhd_hyd_FluxalphaEqHBweno
!   2014-10-26 (B.F.) New parameter: nghost1
!   2014-11-24 (B.F.) Interpolate differently for SlopeCon_flag case
!   2014-12-03 (B.F.) Add parameter dxb1; remove parameter xb1
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3   ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1:n1+1, &
                -reconstruction_order:reconstruction_order, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: alpha
real, dimension(m1-nghost1:n1+nghost1  ), &
                           intent(in)   :: dxb1
integer, dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3     ), &
                           intent(in)   :: i_up
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(in)   :: lambda
real, dimension(m1:n1+1, &
                m2:n2, &
                m3:n3), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3, i
real                                    :: wA0, wA1, wA2, &
                                           am2, am1, ap0, ap1, ap2, &
                                                    atil1m1, atil2m1, atil3m1, &
                                           atil0p0, atil1p0, atil2p0, atil3p0, &
                                           atil0p1, atil1p1, atil2p1, &
                                                    acen1m1, acen2m1, &
                                           acen0p0, acen1p0, acen2p0, &
                                           acen0p1, acen1p1, &
                                                    si1m1  , si2m1, &
                                           si0p0  , si1p0  , si2p0, &
                                           si0p1  , si1p1  , &
                                           wtil1m1, wtil0p0, wtil1p0, wtil0p1, wtilsum, &
                                           atilL, atilR, amotilL, amotilR, &
                                           wp1L, wp0L, wm1L, wp1R, wp0R, wm1R, wp1, wp0, wm1, &
                                           wisum, &
                                           ptilL, ptilR, ptil0L, ptil0R, &
                                           si01, si02, si12, siTot, &
                                           delta_a, aL, aR, &
                                           pc_0, pc_1, pc_2, &
                                           y
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    ! --- Define weight array ---
    ! wA0=0.03 ; wA1=0.9 ; wA2=0.07
    wA1=par%C_RecWenoWeightCenter
    wA2=(1.0-wA1)*par%C_RecWenoWeightRight
    wA0=1.0-wA1-wA2
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil1m1=  ( 2.0/6.0)*am2 + (5.0/6.0)*am1 - ( 1.0/6.0)*ap0
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil0p0=  (11.0/6.0)*am1 - (7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3p0=  ( 2.0/6.0)*am1 - (7.0/6.0)*ap0 + (11.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2p1=- ( 1.0/6.0)*ap0 + (5.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen1m1=-( 1.0/24.0)*am2 + (26.0/24.0)*am1 - ( 1.0/24.0)*ap0
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen0p0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen2p0=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          acen1p1=-( 1.0/24.0)*ap0 + (26.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil1m1, acen1m1, atil2m1, si1m1)
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil0p0, acen0p0, atil1p0, si0p0)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil2p0, acen2p0, atil3p0, si2p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          call rhd_rec_HBwenoSmoothness(atil1p1, acen1p1, atil2p1, si1p1)
          !
          ! === Compute preliminary (continuous) cell-boundary values ====================
          !
          ! --- Compute weights from smoothness of adjacent polynomials ---
          wtil1m1=0.5/(si1m1+eps_tiny) + 0.5/(si2m1+eps_tiny)
          wtil0p0=0.5/(si0p0+eps_tiny) + 0.5/(si1p0+eps_tiny)
          wtilsum=wtil1m1+wtil0p0
          wtil1m1=wtil1m1/wtilsum
          wtil0p0=wtil0p0/wtilsum
          !
          wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
          wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
          wtilsum=wtil1p0+wtil0p1
          wtil1p0=wtil1p0/wtilsum
          wtil0p1=wtil0p1/wtilsum
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL  =wtil0p0*atil1p0 + wtil1m1*atil2m1
          atilR  =wtil0p1*atil1p1 + wtil1p0*atil2p0
          !
          ! --- Monotonize ---
          amotilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          amotilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wp1  =0.5*(wp1L+wp1R)
          wp0  =0.5*(wp0L+wp0R)
          wm1  =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1  =wp1/wisum
          wp0  =wp0/wisum
          wm1  =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          ptil0L=ptilL
          ptil0R=ptilR
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ! ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ! ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*ptil0L
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*ptil0R
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          i   =i_up(i1,i2,i3)
          y   =lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil1m1=  ( 2.0/6.0)*am2 + (5.0/6.0)*am1 - ( 1.0/6.0)*ap0
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil0p0=  (11.0/6.0)*am1 - (7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3p0=  ( 2.0/6.0)*am1 - (7.0/6.0)*ap0 + (11.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2p1=- ( 1.0/6.0)*ap0 + (5.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen1m1=-( 1.0/24.0)*am2 + (26.0/24.0)*am1 - ( 1.0/24.0)*ap0
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen0p0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen2p0=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          acen1p1=-( 1.0/24.0)*ap0 + (26.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil1m1, acen1m1, atil2m1, si1m1)
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil0p0, acen0p0, atil1p0, si0p0)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil2p0, acen2p0, atil3p0, si2p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          call rhd_rec_HBwenoSmoothness(atil1p1, acen1p1, atil2p1, si1p1)
          !
          ! === Compute preliminary (continuous) cell-boundary values ====================
          !
          ! --- Compute weights from smoothness of adjacent polynomials ---
          wtil1m1=0.5/(si1m1+eps_tiny) + 0.5/(si2m1+eps_tiny)
          wtil0p0=0.5/(si0p0+eps_tiny) + 0.5/(si1p0+eps_tiny)
          wtilsum=wtil1m1+wtil0p0
          wtil1m1=wtil1m1/wtilsum
          wtil0p0=wtil0p0/wtilsum
          !
          wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
          wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
          wtilsum=wtil1p0+wtil0p1
          wtil1p0=wtil1p0/wtilsum
          wtil0p1=wtil0p1/wtilsum
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL  =wtil0p0*atil1p0 + wtil1m1*atil2m1
          atilR  =wtil0p1*atil1p1 + wtil1p0*atil2p0
          !
          ! --- Monotonize ---
          amotilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          amotilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wp1  =0.5*(wp1L+wp1R)
          wp0  =0.5*(wp0L+wp0R)
          wm1  =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1  =wp1/wisum
          wp0  =wp0/wisum
          wm1  =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          i   =i_up(i1,i2,i3)
          y   =lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        !DIR$ SIMD
        do i1=m1,n1+1
          !
          ! --- Set abbreviations for cell values, set grid (dx=1) ---
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          atil1m1=  ( 2.0/6.0)*am2 + (5.0/6.0)*am1 - ( 1.0/6.0)*ap0
          atil2m1=- ( 1.0/6.0)*am2 + (5.0/6.0)*am1 + ( 2.0/6.0)*ap0
          atil3m1=  ( 2.0/6.0)*am2 - (7.0/6.0)*am1 + (11.0/6.0)*ap0
          atil0p0=  (11.0/6.0)*am1 - (7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil1p0=  ( 2.0/6.0)*am1 + (5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
          atil2p0=- ( 1.0/6.0)*am1 + (5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
          atil3p0=  ( 2.0/6.0)*am1 - (7.0/6.0)*ap0 + (11.0/6.0)*ap1
          atil0p1=  (11.0/6.0)*ap0 - (7.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          atil1p1=  ( 2.0/6.0)*ap0 + (5.0/6.0)*ap1 - ( 1.0/6.0)*ap2
          atil2p1=- ( 1.0/6.0)*ap0 + (5.0/6.0)*ap1 + ( 2.0/6.0)*ap2
          !
          ! --- Compute cell-center values ---
          acen1m1=-( 1.0/24.0)*am2 + (26.0/24.0)*am1 - ( 1.0/24.0)*ap0
          acen2m1=-( 1.0/24.0)*am2 + ( 2.0/24.0)*am1 + (23.0/24.0)*ap0
          acen0p0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen1p0=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
          acen2p0=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
          acen0p1= (23.0/24.0)*ap0 + ( 2.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          acen1p1=-( 1.0/24.0)*ap0 + (26.0/24.0)*ap1 - ( 1.0/24.0)*ap2
          !
          ! --- Compute smoothness of all polynomials in all intervals ---
          call rhd_rec_HBwenoSmoothness(atil1m1, acen1m1, atil2m1, si1m1)
          call rhd_rec_HBwenoSmoothness(atil2m1, acen2m1, atil3m1, si2m1)
          call rhd_rec_HBwenoSmoothness(atil0p0, acen0p0, atil1p0, si0p0)
          call rhd_rec_HBwenoSmoothness(atil1p0, acen1p0, atil2p0, si1p0)
          call rhd_rec_HBwenoSmoothness(atil2p0, acen2p0, atil3p0, si2p0)
          call rhd_rec_HBwenoSmoothness(atil0p1, acen0p1, atil1p1, si0p1)
          call rhd_rec_HBwenoSmoothness(atil1p1, acen1p1, atil2p1, si1p1)
          !
          ! === Compute preliminary (continuous) cell-boundary values ====================
          !
          ! --- Compute weights from smoothness of adjacent polynomials ---
          wtil1m1=0.5/(si1m1+eps_tiny) + 0.5/(si2m1+eps_tiny)
          wtil0p0=0.5/(si0p0+eps_tiny) + 0.5/(si1p0+eps_tiny)
          wtilsum=wtil1m1+wtil0p0
          wtil1m1=wtil1m1/wtilsum
          wtil0p0=wtil0p0/wtilsum
          !
          wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
          wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
          wtilsum=wtil1p0+wtil0p1
          wtil1p0=wtil1p0/wtilsum
          wtil0p1=wtil0p1/wtilsum
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL  =wtil0p0*atil1p0 + wtil1m1*atil2m1
          atilR  =wtil0p1*atil1p1 + wtil1p0*atil2p0
          !
          ! --- Monotonize ---
          amotilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          amotilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L =wA0/(si0p1+eps_tiny)
          wp0L =wA1/(si1p0+eps_tiny)
          wm1L =wA2/(si2m1+eps_tiny)
          !
          wp1R =wA2/(si0p1+eps_tiny)
          wp0R =wA1/(si1p0+eps_tiny)
          wm1R =wA0/(si2m1+eps_tiny)
          !
          wp1  =0.5*(wp1L+wp1R)
          wp0  =0.5*(wp0L+wp0R)
          wm1  =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1  =wp1/wisum
          wp0  =wp0/wisum
          wm1  =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L =wp1L/wisum
          wp0L =wp0L/wisum
          wm1L =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R =wp1R/wisum
          wp0R =wp0R/wisum
          wm1R =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          !
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          !
          i   =i_up(i1,i2,i3)
          y   =lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_FluxalphaHBweno


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
                             reconstruction_order, reconstruction_type, &
                             dtime, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             alpha, dxb1, i_up, lambda, &
                             flux_alpha_out)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_Fluxalpha ('rhd_reconstruction_Flux_alpha')
!
! PURPOSE:
!   Reconstruct partial wave and compute the flux on a general grid.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_Fluxalpha(m1,n1, m2,n2, m3,n3, nghost1, &
!                          reconstruction_order, reconstruction_type, &
!                          dtime, &
!                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                          alpha, dxb1, i_up, lambda, &
!                          flux_alpha_out)
!
! INPUT:
!   m1:           ('m_number_1') integer, 1. dimension, start index
!   n1:           ('number_1')   integer, 1. dimension, end index
!   m2:           ('m_number_2') integer, 2. dimension, start index
!   n2:           ('number_2')   integer, 2. dimension, end index
!   m3:           ('m_number_3') integer, 3. dimension, start index
!   n3:           ('number_3')   integer, 3. dimension, end index
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type:  ('reconstruction_type') integer, type of reconstruction scheme
!   dtime:        ('delta_time') real, time step
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   alpha:        ('alpha') 4D real array with coeffs., run in cell alpha(:,0,:,:) is reconstructed
!   dxb1:         ('delta_x_boundary_1') real, cell-boundary step size, 1. coordinate
!   i_up:         ('index_upwind') 3D real array with indices of upwind direction
!   lambda:       ('lambda') 3D real array with characteristic velocities
!
! OUTPUT:
!   flux_alpha_out: ('flux_alpha_out') 3D real array with fluxes
!                   units: [flux_alpha_out] = [lambda] * [alpha]
!
! LOCAL VARIABLES:
!   i1:           ('index_1') integer, loop index for first spatial dimension (x1 direction)
!   i2:           ('index_2') integer, loop index for second spatial dimension (x2 direction)
!   i3:           ('index_3') integer, loop index for third spatial dimension (x3 direction)
!   i:            ('index') integer with spatial upwind index
!   delta_alpha:  ('delta_alpha') real, slope of reconstruction of linear methods
!   dalphapro:    ('delta_alpha_produce') real, product of slopes as used by VanLeer method
!   dxm2:         ('delta_x_minus_2') real, delta x value two cells to the left; PP methods
!   dxm1:         ('delta_x_minus_1') real, delta x value one cell to the left; PP methods
!   dxp0:         ('delta_x_plus_0') real, delta x value in cell; PP methods
!   dxp1:         ('delta_x_plus_1') real, delta x value one cell to the right; PP methods
!   dxp2:         ('delta_x_plus_2') real, delta x value two cells to the right; PP methods
!   am2:          ('alpha_minus_2') real, alpha value two cells to the left; PP methods
!   am1:          ('alpha_minus_1') real, alpha value one cell to the left; PP methods
!   ap0:          ('alpha_plus_0') real, alpha value in cell; PP methods
!   ap1:          ('alpha_plus_1') real, alpha value one cell to the right; PP methods
!   ap2:          ('alpha_plus_2') real, alpha value two cells to the right; PP methods
!   dam1:         ('delta_alpha_minus_1') real, slope in cell to the left; PP methods
!   dap0:         ('delta_alpha_plus_0') real, slope in cell; PP methods
!   dap1:         ('delta_alpha_plus_1') real, slope in cell to the right; PP methods
!   am:           ('alpha_minus') real, preliminary boundary value on left side; PP methods
!   ap:           ('alpha_plus') real, preliminary boundary value on right side; PP methods
!   aL:           ('alpha_Left') real, boundary value on left side; PP methods
!   aR:           ('alpha_Right') real, boundary value on right side; PP methods
!   a_0:          ('alpha_0') real, 0th coefficient of parabola; PP methods
!   delta_a:      ('alpha_0') real, 1st coefficient of parabola (slope); PP methods
!   a_2:          ('alpha_2') real, 2nd coefficient of parabola; PP methods
!   y:            ('y') real, lambda*dtime/dx; PP methods
!   eps_tiny:     ('epsilon_tiny') real, a very small value to prevent division by zero
!
! ROUTINES:
!   rhd_rec_FluxalphaVAweno: Reconstruct partial wave (VAweno), compute flux on general grid
!   rhd_rec_FluxalphaHBweno: Reconstruct partial wave (HBweno), compute flux on general grid
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Possible types of reconstruction:
!     reconstruction_order=0:
!       reconstruction_type=reconstruction_type_Constant: 
!         constant values in cell assumed (simple, fast, and very dissipative)
!     reconstruction_order=1
!       reconstruction_type=reconstruction_type_Superbee:
!         Superbee (sharp shocks and discontinuities, shapes become box-like)
!       reconstruction_type=reconstruction_type_Minmod:
!         Minmod (2nd order, but relatively dissipative)
!       reconstruction_type=reconstruction_type_VanLeer: VanLeer (good compromise)
!     reconstruction_order=2
!       reconstruction_type=reconstruction_type_PPmimo:
!         Merely linear reconstruction with boundary values from PP method
!       reconstruction_type=reconstruction_type_PP:
!         PiecewiseParabolic (sophisticated and time-consuming method from
!         the PPM-scheme without extra sharpening)
!       reconstruction_type=reconstruction_type_FRmimo:
!         Merely linear reconstruction with boundary values from FRmono method
!       reconstruction_type=reconstruction_type_FRmono:
!         Monotonic scheme that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature
!       reconstruction_type=reconstruction_type_FRmono2:
!         Monotonic scheme that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature.
!         The slope reduction is more aggressive (dissipative) than for FRmono
!       reconstruction_type=reconstruction_type_FRweno:
!         Non-monotonic that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature
!       reconstruction_type=reconstruction_type_FRweno2:
!         Non-monotonic that uses PP-style continuous preliminary boundary values
!         and WENO-style parabola with minimal curvature.
!         The slope reduction is more aggressive (dissipative) than for FRweno
!       reconstruction_type=reconstruction_type_FRcont:
!         Non-monotonic (and fairly unstable) scheme that uses only continuous boundary values
!         (useful for test at small Mach numbers)
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   1997-10-08 (B.F.) Minmod, Van Leer
!   1997-10-19 (B.F.) size of xb1 increased
!   1997-10-21 (B.F.) PP
!   1997-10-25 (B.F.) Superbeemod
!   1999-05    (B.F. Copenhagen) Superbeemod... removed
!   2001-02-06 (B.F. Uppsala) Loop restructuring and fusion, formula for MinMod changed
!   2001-02-13 (B.F.) Slight modification in Minmod formula
!   2002-10-23 (B.F.) Removed case distinction for reconstruction_order
!   2011-04-27 (B.F. Lyon) 'PPL', 'PPAW' methods
!   2011-11-08 (B.F.) 'FRmo' method
!   2011-11-14 (B.F.) New parameters: SlopeRed_flag, SlopeRed
!   2011-11-17 (B.F.) Replacement: sign(1.0,y) -> sign(1.0,lambda(i1,i2,i3)), relevant for RungeK.
!   2011-11-18 (B.F.) Remove PPAW branch, add slope reduction and steepening for several cases
!   2011-11-22 (B.F.) New parameters: SlopeCon_flag, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2014-08-25 (B.F. Uppsala) HBweno, VAweno
!   2014-10-26 (B.F.) New parameter: nghost1
!   2014-12-03 (B.F.) Add parameter dxb1; remove parameter xb1
!   2018-05-09 (B.F.) Bugfix: VanLeer, SuperBee with slope reduction: compute MinMod slope correctly
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, &
                                           reconstruction_order, reconstruction_type
real,                      intent(in)   :: dtime
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1:n1+2, &
                m2  :n2  , &
                m3  :n3     ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1:n1+1, &
                -reconstruction_order: &
                 reconstruction_order, &
                m2:n2  , &
                m3:n3     ), &
                           intent(in)   :: alpha
real, dimension(m1-nghost1:n1+nghost1  ), &
                           intent(in)   :: dxb1
integer, dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3     ), &
                           intent(in)   :: i_up
real,    dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3     ), &
                           intent(in)   :: lambda
real,    dimension(m1:n1+1, &
                   m2:n2  , &
                   m3:n3     ), &
                           intent(out)  :: flux_alpha_out
!
! --- Local variables ---
integer                                 :: i1, i2, i3, i
real                                    :: delta_alpha, dalphapro
real                                    :: dxm2, dxm1, dxp0, dxp1, dxp2, &
                                           am2, am1, ap0, ap1, ap2, &
                                           dam1, dap0, dap1, &
                                           ap, am, aL, aR, &
                                           a_0, delta_a, a_2, y, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           pc_2m, pc_20, pc_2p, wem, we0, wep, wesum, &
                                           atilL, atilR, f, &
                                           ptilL, ptilR, pc_0, pc_1, pc_2, &
                                           x2, x3, Di, pmmtilL, pmmtilR
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
! --- Reconstruction ---
select case (reconstruction_type)
  ! === 0th order ==================================================================================
  case(reconstruction_type_Constant) ! -------------------------------------------------------------
    ! --- Donor cell with constant reconstruction (delta_alpha=0.0) ---
    !
    ! --- Compute flux ---
    flux_alpha_out=lambda * alpha(:,0,:,:)
    !
  ! === 1st order ==================================================================================
  case(reconstruction_type_Minmod) ! ---------------------------------------------------------------
    ! --- Minmod ---
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Get upwind index ---
          i=i_up(i1,i2,i3)
          !
          ! --- Compute MinMod slope ---
          delta_alpha=2.0 * dxb1(i) * &
                      ( min( max( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             max( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) + &
                        max( min( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             min( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) &
                      )
          !
          ! --- Reduce slope ---
          delta_alpha=SlopeRed(i1,i2,i3)*delta_alpha
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3))* &
                                     (1.0 - abs(lambda(i1,i2,i3))*dtime/dxb1(i) &
                                     ) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Get upwind index ---
          i=i_up(i1,i2,i3)
          !
          ! --- Compute MinMod slope ---
          delta_alpha=2.0 * dxb1(i) * &
                      ( min( max( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             max( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) + &
                        max( min( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             min( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) &
                      )
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3))* &
                                     (1.0 - abs(lambda(i1,i2,i3))*dtime/dxb1(i) &
                                     ) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_VanLeer) ! --------------------------------------------------------------
    ! --- Van Leer ---
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Get upwind index ---
          i=i_up(i1,i2,i3)
          !
          ! --- Compute vanLeer slope ---
          dalphapro=(alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)) / &
                    (dxb1(    i        ) + dxb1(    i-1      )) * &
                    (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)) / &
                    (dxb1(    i+1      ) + dxb1(    i        )) * &
                    dxb1(i)
          if (dalphapro > 0.0) then
            delta_alpha=4.0 * dalphapro / &
                  ( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)) / &
                    (dxb1(    i        ) + dxb1(    i-1      )) + &
                    (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)) / &
                    (dxb1(    i+1      ) + dxb1(    i        ))    )
          else
            delta_alpha=0.0
          endif
          !
          ! --- Compute MinMod slope ---
          delta_a    =2.0 * dxb1(i) * &
                      ( min( max( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             max( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) + &
                        max( min( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             min( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) &
                      )
          !
          ! --- Interpolate between vanLeer and MinMod slope ---
          delta_alpha=SlopeRed(i1,i2,i3)*delta_alpha + (1.0-SlopeRed(i1,i2,i3))*delta_a
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*dtime/dxb1(i) &
                                     ) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Get upwind index ---
          i=i_up(i1,i2,i3)
          !
          ! --- Compute vanLeer slope ---
          dalphapro=(alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)) / &
                    (dxb1(    i        ) + dxb1(    i-1      )) * &
                    (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)) / &
                    (dxb1(    i+1      ) + dxb1(    i        )) * &
                    dxb1(i)
          if (dalphapro > 0.0) then
            delta_alpha=4.0 * dalphapro / &
                  ( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)) / &
                    (dxb1(    i        ) + dxb1(    i-1      )) + &
                    (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)) / &
                    (dxb1(    i+1      ) + dxb1(    i        ))    )
          else
            delta_alpha=0.0
          endif
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*dtime/dxb1(i) &
                                     ) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  case(reconstruction_type_Superbee) ! -------------------------------------------------------------
    ! --- Superbee ---
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Get upwind index ---
          i=i_up(i1,i2,i3)
          !
          ! --- Compute Suberbee slope ---
          delta_alpha= ( sign(1.0,alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)) + &
                         sign(1.0,alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)) ) * &
                       dxb1(i) * &
                       min( abs( alpha(i1, 0,i2,i3)     - alpha(i1,-1,i2,i3))/ &
                              dxb1(i-1), &
                            abs( alpha(i1, 1,i2,i3)     - alpha(i1, 0,i2,i3))/ &
                              dxb1(i  ), &
                            max( abs( alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3))/ &
                                   (  dxb1(i  )          + dxb1(i-1)         ), &
                                 abs( alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3))/ &
                                   (  dxb1(i+1)          + dxb1(i  )         ) &
                               ) &
                          )
          !
          ! --- Compute MinMod slope ---
          delta_a    =2.0 * dxb1(i) * &
                      ( min( max( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             max( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) + &
                        max( min( (alpha(i1,  0,i2,i3) - alpha(i1, -1,i2,i3)  )/ &
                                  (dxb1(    i        ) + dxb1(    i-1      )  ), 0.0 &
                                ) , &
                             min( (alpha(i1,  1,i2,i3) - alpha(i1,  0,i2,i3)  )/ &
                                  (dxb1(    i+1      ) + dxb1(    i        )  ), 0.0 &
                                ) &
                           ) &
                      )
          !
          ! --- Interpolate between Suberbee and MinMod slope ---
          delta_alpha=SlopeRed(i1,i2,i3)*delta_alpha + (1.0-SlopeRed(i1,i2,i3))*delta_a
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*dtime/dxb1(i) &
                                     ) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip additional slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          ! --- Get upwind index ---
          i=i_up(i1,i2,i3)
          !
          ! --- Compute Suberbee slope ---
          delta_alpha= ( sign(1.0,alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3)) + &
                         sign(1.0,alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3)) ) * &
                       dxb1(i) * &
                       min( abs( alpha(i1, 0,i2,i3)     - alpha(i1,-1,i2,i3))/ &
                              dxb1(i-1), &
                            abs( alpha(i1, 1,i2,i3)     - alpha(i1, 0,i2,i3))/ &
                              dxb1(i  ), &
                            max( abs( alpha(i1, 0,i2,i3) - alpha(i1,-1,i2,i3))/ &
                                   (  dxb1(i  )          + dxb1(i-1)         ), &
                                 abs( alpha(i1, 1,i2,i3) - alpha(i1, 0,i2,i3))/ &
                                   (  dxb1(i+1)          + dxb1(i  )         ) &
                               ) &
                          )
          !
          ! --- Compute flux ---
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   ( alpha(i1,0,i2,i3) + &
                                     delta_alpha * sign(0.5,lambda(i1,i2,i3)) * &
                                     (1.0 - abs(lambda(i1,i2,i3))*dtime/dxb1(i) &
                                     ) &
                                   )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif
    !
  ! === 2nd order ==================================================================================
  case(reconstruction_type_PPmimo) ! ---------------------------------------------------------------
    ! --- Boundary values from 'Piecewise Parabolic', only linear reconstruction ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Set grid spacings, abbreviate cell values ---
          i=i_up(i1,i2,i3)
          dxm2=dxb1(i-2)
          dxm1=dxb1(i-1)
          dxp0=dxb1(i  )
          dxp1=dxb1(i+1)
          dxp2=dxb1(i+2)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=  dxm1/(dxm2+dxm1+dxp0) * &
               ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
                 (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
          dap0=  dxp0/(dxm1+dxp0+dxp1) * &
               ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
                 (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
          dap1=  dxp1/(dxp0+dxp1+dxp2) * &
               ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
                 (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
             1.0/(dxm2+dxm1+dxp0+dxp1) * &
             ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
               ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
                                                  (ap0-am1) - &
               dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0) * dap0 + &
               dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0) * dam1  )
          ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
             1.0/(dxm1+dxp0+dxp1+dxp2) * &
             ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
               ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
                                                  (ap1-ap0) - &
               dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1) * dap1 + &
               dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1) * dap0  )
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          ! --- Compute MinMod boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Interpolate between computed and continuous boundary values ---
          aL=SlopeCon(i1,i2,i3)*aL + (1.0-SlopeCon(i1,i2,i3))*am
          aR=SlopeCon(i1,i2,i3)*aR + (1.0-SlopeCon(i1,i2,i3))*ap
          !
          ! --- Compute parabola coefficients from cell integral and boundary values ---
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxp0
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Set grid spacings, abbreviate cell values ---
          i=i_up(i1,i2,i3)
          dxm2=dxb1(i-2)
          dxm1=dxb1(i-1)
          dxp0=dxb1(i  )
          dxp1=dxb1(i+1)
          dxp2=dxb1(i+2)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=  dxm1/(dxm2+dxm1+dxp0) * &
               ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
                 (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
          dap0=  dxp0/(dxm1+dxp0+dxp1) * &
               ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
                 (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
          dap1=  dxp1/(dxp0+dxp1+dxp2) * &
               ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
                 (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
             1.0/(dxm2+dxm1+dxp0+dxp1) * &
             ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
               ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
                                                  (ap0-am1) - &
               dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0) * dap0 + &
               dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0) * dam1  )
          ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
             1.0/(dxm1+dxp0+dxp1+dxp2) * &
             ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
               ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
                                                  (ap1-ap0) - &
               dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1) * dap1 + &
               dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1) * dap0  )
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,ap-ap0) + sign(1.0,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Compute (rest of) parabola coefficients from cell integral and boundary values ---
          a_0    =ap0
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxp0
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Set grid spacings, abbreviate cell values ---
          i=i_up(i1,i2,i3)
          dxm2=dxb1(i-2)
          dxm1=dxb1(i-1)
          dxp0=dxb1(i  )
          dxp1=dxb1(i+1)
          dxp2=dxb1(i+2)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=  dxm1/(dxm2+dxm1+dxp0) * &
               ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
                 (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
          dap0=  dxp0/(dxm1+dxp0+dxp1) * &
               ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
                 (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
          dap1=  dxp1/(dxp0+dxp1+dxp2) * &
               ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
                 (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
             1.0/(dxm2+dxm1+dxp0+dxp1) * &
             ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
               ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
                                                  (ap0-am1) - &
               dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0) * dap0 + &
               dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0) * dam1  )
          ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
             1.0/(dxm1+dxp0+dxp1+dxp2) * &
             ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
               ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
                                                  (ap1-ap0) - &
               dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1) * dap1 + &
               dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1) * dap0  )
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,ap-ap0)+sign(1.0,ap0-am)) * min(abs(ap-ap0),abs(ap0-am))
          !
          ! --- Compute (rest of) parabola coefficients from cell integral and boundary values ---
          a_0    =ap0
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxp0
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_PP) ! -------------------------------------------------------------------
    ! --- Piecewise Parabolic ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Set grid spacings, abbreviate cell values ---
          i=i_up(i1,i2,i3)
          dxm2=dxb1(i-2)
          dxm1=dxb1(i-1)
          dxp0=dxb1(i  )
          dxp1=dxb1(i+1)
          dxp2=dxb1(i+2)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=  dxm1/(dxm2+dxm1+dxp0) * &
               ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
                 (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
          dap0=  dxp0/(dxm1+dxp0+dxp1) * &
               ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
                 (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
          dap1=  dxp1/(dxp0+dxp1+dxp2) * &
               ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
                 (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
             1.0/(dxm2+dxm1+dxp0+dxp1) * &
             ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
               ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
                                                  (ap0-am1) - &
               dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0) * dap0 + &
               dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0) * dam1  )
          ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
             1.0/(dxm1+dxp0+dxp1+dxp2) * &
             ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
               ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
                                                  (ap1-ap0) - &
               dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1) * dap1 + &
               dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1) * dap0  )
          !
          ! --- Correct boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute MinMod boundary values ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          aL=SlopeRed(i1,i2,i3)*aL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          aR=SlopeRed(i1,i2,i3)*aR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          aL=SlopeCon(i1,i2,i3)*aL + (1.0-SlopeCon(i1,i2,i3))*am
          aR=SlopeCon(i1,i2,i3)*aR + (1.0-SlopeCon(i1,i2,i3))*ap
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxp0
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Set grid spacings, abbreviate cell values ---
          i=i_up(i1,i2,i3)
          dxm2=dxb1(i-2)
          dxm1=dxb1(i-1)
          dxp0=dxb1(i  )
          dxp1=dxb1(i+1)
          dxp2=dxb1(i+2)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=  dxm1/(dxm2+dxm1+dxp0) * &
               ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
                 (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
          dap0=  dxp0/(dxm1+dxp0+dxp1) * &
               ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
                 (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
          dap1=  dxp1/(dxp0+dxp1+dxp2) * &
               ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
                 (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
             1.0/(dxm2+dxm1+dxp0+dxp1) * &
             ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
               ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
                                                  (ap0-am1) - &
               dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0) * dap0 + &
               dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0) * dam1  )
          ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
             1.0/(dxm1+dxp0+dxp1+dxp2) * &
             ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
               ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
                                                  (ap1-ap0) - &
               dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1) * dap1 + &
               dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1) * dap0  )
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute MinMod boundary values ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          aL=SlopeRed(i1,i2,i3)*aL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          aR=SlopeRed(i1,i2,i3)*aR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxp0
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Set grid spacings, abbreviate cell values ---
          i=i_up(i1,i2,i3)
          dxm2=dxb1(i-2)
          dxm1=dxb1(i-1)
          dxp0=dxb1(i  )
          dxp1=dxb1(i+1)
          dxp2=dxb1(i+2)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! --- Average slope in zones (1.7) ---
          dam1=  dxm1/(dxm2+dxm1+dxp0) * &
               ( (2.0*dxm2+dxm1)/(dxp0+dxm1) * (ap0-am1) + &
                 (dxm1+2.0*dxp0)/(dxm2+dxm1) * (am1-am2)  )
          dap0=  dxp0/(dxm1+dxp0+dxp1) * &
               ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
                 (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
          dap1=  dxp1/(dxp0+dxp1+dxp2) * &
               ( (2.0*dxp0+dxp1)/(dxp2+dxp1) * (ap2-ap1) + &
                 (dxp1+2.0*dxp2)/(dxp0+dxp1) * (ap1-ap0)  )
          !
          ! --- Perform steepening, monotonizing (1.8) ---
          dam1=(sign(0.5,ap0-am1) + sign(0.5,am1-am2)) * &
               min(abs(dam1),2.0*abs(ap0-am1),2.0*abs(am1-am2))
          dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
               min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
          dap1=(sign(0.5,ap2-ap1) + sign(0.5,ap1-ap0)) * &
               min(abs(dap1),2.0*abs(ap2-ap1),2.0*abs(ap1-ap0))
          !
          ! --- Compute preliminary boundary values (1.6) ---
          am=am1 + dxm1/(dxm1+dxp0) * (ap0-am1) + &
             1.0/(dxm2+dxm1+dxp0+dxp1) * &
             ( 2.0*dxp0*dxm1/(dxm1+dxp0) * &
               ((dxm2+dxm1)/(2.0*dxm1+dxp0)-(dxp1+dxp0)/(2.0*dxp0+dxm1)) * &
                                                  (ap0-am1) - &
               dxm1*(dxm2+dxm1)/(2.0*dxm1+dxp0) * dap0 + &
               dxp0*(dxp0+dxp1)/(dxm1+2.0*dxp0) * dam1  )
          ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
             1.0/(dxm1+dxp0+dxp1+dxp2) * &
             ( 2.0*dxp1*dxp0/(dxp0+dxp1) * &
               ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0)) * &
                                                  (ap1-ap0) - &
               dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1) * dap1 + &
               dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1) * dap0  )
          !
          ! --- Correct boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! --- Compute coefficients of parabola ---
          a_0    =1.5*ap0-0.25*(aL+aR)
          delta_a=aR-aL
          a_2    =3.0*(aL+aR-2.0*ap0)
          !
          ! --- Compute flux ---
          y=lambda(i1,i2,i3)*dtime/dxp0
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)) + &
                                    a_2 * (0.25-0.5*abs(y)+y*y/3.0) )
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRmimo) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method MinMod slope ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          !pc_2m =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i-1)-xb1(i-2)))**2 * ( &
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          ! --- Compute MinMod boundary values ---
          ptilL=ap0-delta_a
          ptilR=ap0+delta_a
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,atilR-ap0)+sign(1.0,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          delta_a=SlopeRed(i1,i2,i3)*delta_a
          !
          a_0=ap0
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Compute MinMod slope ---
          delta_a=(sign(1.0,atilR-ap0)+sign(1.0,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          a_0=ap0
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (a_0 + &
                                    delta_a * sign(0.5,lambda(i1,i2,i3)) * (1.0-abs(y)))
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRmono) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ! --- to get preliminary boundary values ---
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Monotonize ---
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRmono2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic 2 (more efficient slope reduction) ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ! --- to get preliminary boundary values ---
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Monotonize ---
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/           f             + ap0
            !ptilR=(ptilR-ap0)/((2.0/3.0)*f +(1.0/3.0)) + ap0
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilR=(ptilR-ap0)/f + ap0
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f    =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            !ptilR=(ptilR-ap0)/           f            + ap0
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !f    =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            !f    =max(f,1.0)
            !ptilL=(ptilL-ap0)/f + ap0
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute boundary values of polynomials that match three adjacent cells ---
          ! --- Get coefficients of polynomials (only pc_2 is actually needed) ---
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRweno) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*pmmtilL
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*pmmtilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides --
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRweno2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory (more efficient slope reduction) ---
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          ptilL=SlopeCon(i1,i2,i3)*ptilL + (1.0-SlopeCon(i1,i2,i3))*atilL
          ptilR=SlopeCon(i1,i2,i3)*ptilR + (1.0-SlopeCon(i1,i2,i3))*atilR
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          !! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          ptilL=SlopeRed(i1,i2,i3)*ptilL + (1.0-SlopeRed(i1,i2,i3))*ap0
          ptilR=SlopeRed(i1,i2,i3)*ptilR + (1.0-SlopeRed(i1,i2,i3))*ap0
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.25/((pc_2m)**2 + eps_tiny)
          we0  =0.50/((pc_20)**2 + eps_tiny)
          wep  =0.25/((pc_2p)**2 + eps_tiny)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/f                       + ap0
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              !ptilR=(ptilR-ap0)/f                       + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL))) > ((1.0/6.0)*(ptilR-ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_FRcont) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Continuous Essentially Non-oscillatory ---
    !
      do i3=m3,n3
       do i2=m2,n2
        do i1=m1,n1+1
          !
          ! --- Assign values as abbreviation ---
          i=i_up(i1,i2,i3)
          am2=alpha(i1,-2,i2,i3)
          am1=alpha(i1,-1,i2,i3)
          ap0=alpha(i1, 0,i2,i3)
          ap1=alpha(i1, 1,i2,i3)
          ap2=alpha(i1, 2,i2,i3)
          !
          ! === Compute interpolation polynomials ========================================
          !
          ! --- Compute coefficients for non-equidistant case ---
          !x2=(xb1(i  )-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          !x3=(xb1(i+1)-xb1(i-2))/(xb1(i-1)-xb1(i-2))
          x2=(          dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          x3=(dxb1(i  )+dxb1(i-1)+dxb1(i-2))/dxb1(i-2)
          Di=1.0/(x2*x3*(x3-1.0))
          atil2L=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am2+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * am1+&
             (                     -x2**2                    +x2**3                        ) * ap0 )
          atil3R=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am2+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * am1+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap0 )
          pc_2m =Di * 3.0 * (dxb1(i  )/dxb1(i-2))**2 * ( &
             (-1.0 +x3                                                                     ) * am2+&
             ( 1.0 -x2 -x3                                                                 ) * am1+&
             (      x2                                                                     ) * ap0 )
          !
          !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
          x2=(          dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          x3=(dxb1(i+1)+dxb1(i  )+dxb1(i-1))/dxb1(i-1)
          Di=1.0/(x2*x3*(x3-1.0))
          atil1L=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
          atil2R=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
          pc_20 =Di * 3.0 * (dxb1(i  )/dxb1(i-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
          !
          !x2=(xb1(i+2)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          !x3=(xb1(i+3)-xb1(i  ))/(xb1(i+1)-xb1(i  ))
          x2=(          dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          x3=(dxb1(i+2)+dxb1(i+1)+dxb1(i  ))/dxb1(i  )
          Di=1.0/(x2*x3*(x3-1.0))
          atil0L=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * ap0+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap1+&
             (                                                x2**2                        ) * ap2 )
          atil1R=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * ap0+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap1+&
             (      x2             -x2**2                                                  ) * ap2 )
          pc_2p =Di*3.0*( &
             (-1.0 +x3                                                                     ) * ap0+&
             ( 1.0 -x2 -x3                                                                 ) * ap1+&
             (      x2                                                                     ) * ap2 )
          !
          ! === Compute preliminary boundary values ======================================
          !
          ! --- Get weights from quadratic coefficients ---
          wem=0.5/(abs(pc_2m) + eps_tiny)
          we0=0.5/(abs(pc_20) + eps_tiny)
          wep=0.5/(abs(pc_2p) + eps_tiny)
          !
          ! --- Compute weighted averages of boundary values in a WENO style ---
          atilL=(wem/(wem+we0))*atil2L + (we0/(wem+we0))*atil1L
          atilR=(we0/(we0+wep))*atil2R + (wep/(we0+wep))*atil1R
          !
          ! --- Monotonize ---
          !atilL=min(max(am1,ap0), max(min(am1,ap0), atilL))
          !atilR=min(max(ap0,ap1), max(min(ap0,ap1), atilR))
          !
          ! --- Set final := preliminary boundary values ---
          ptilL=atilL
          ptilR=atilR
          !
          ! === Skip construction and monotonization of boundary values ==================
          !
          !
          ! === Compute parabola coefficients from cell integral and boundary values =====
          pc_0=1.5*ap0 - 0.25*(ptilL+ptilR)
          pc_1=ptilR-ptilL
          pc_2=3.0*(ptilL+ptilR - 2.0*ap0)
          !
          ! === Compute flux =============================================================
          y=lambda(i1,i2,i3)*dtime/dxb1(i)
          flux_alpha_out(i1,i2,i3)=lambda(i1,i2,i3) * &
                                   (pc_0 + &
                                    pc_1 * sign(0.5,lambda(i1,i2,i3)) * (1.0 - abs(y)) + &
                                    pc_2 * (0.25 - 0.5*abs(y) + (1.0/3.0)*y**2) )
        end do ! i1
       end do ! i2
      end do ! i3
    !
  case(reconstruction_type_VAweno) ! ---------------------------------------------------------------
    ! --- Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_FluxalphaVAweno(m1,n1, m2,n2, m3,n3, nghost1, &
                                 reconstruction_order, &
                                 dtime, &
                                 SlopeRed_flag, SlopeRed, &
                                 alpha, dxb1, i_up, lambda, &
                                 flux_alpha_out)
    !
  case(reconstruction_type_HBweno) ! ---------------------------------------------------------------
    ! --- HandBrake Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_FluxalphaHBweno(m1,n1, m2,n2, m3,n3, nghost1, &
                                 reconstruction_order, &
                                 dtime, &
                                 SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                 alpha, dxb1, i_up, lambda, &
                                 flux_alpha_out)
    !
end select
!
end subroutine rhd_rec_Fluxalpha


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqSimple(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               reconstruction_type, &
                               SlopeRed_flag, SlopeRed, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqSimple ('rhd_reconstruction_Vector_Equidistant_Simple')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 0th- or 1st-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqSimple(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            reconstruction_type, &
!                            SlopeRed_flag, SlopeRed, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F.) Adapted from rhd_mhd_PP
!   2011-01-04 (W.S. Exeter) Adapted from rhd_mhd_PPvec
!   2011-06-24 (W.S.) New parameters nghost2, nghost3, d2, d3 for use in unsplit Hancock scheme
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-05-09 (B.F.) Call subroutines with individual reconstructions
!   2013-08-26 (B.F.) Derived from rhd_hyd_RecVecEqPPmimo and rhd_hyd_RecVecEq
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                           reconstruction_type
logical,                   intent(in)   :: SlopeRed_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: delta, delta_a, d1d2, SR
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: d
!---------------------------------------------------------------------------------------------------
!
select case(reconstruction_type)
  !
  ! === 0th order ==================================================================================
  case(reconstruction_type_Constant) ! -------------------------------------------------------------
    ! --- Donor cell with constant reconstruction (delta=0.0) ---
    !
    do i3=m3-d3,n3+d3
      do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          aLv(i1,i2,i3)=av(i1,i2,i3)
          aRv(i1,i2,i3)=av(i1,i2,i3)
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !
  ! === 1st order ==================================================================================
  case(reconstruction_type_Minmod) ! ---------------------------------------------------------------
    ! --- Minmod ---
    !
    ! --- Compute Slope ---
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        d(i1,i2,i3)=av(i1+1,i2,i3)-av(i1  ,i2,i3)
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          delta=(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                 max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Reduce slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta=SR*delta
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          delta=(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                 max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_VanLeer) ! --------------------------------------------------------------
    ! --- Van Leer ---
    !
    ! --- Compute Slope ---
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        d(i1,i2,i3)=av(i1+1,i2,i3)-av(i1  ,i2,i3)
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope: compute vanLeer slope ---
          d1d2=2.0*d(i1-1,i2,i3)*d(i1  ,i2,i3)
          if (d1d2 > 0.0) then
            delta=d1d2/(d(i1-1,i2,i3)+d(i1  ,i2,i3))
          else
            delta=0.0
          endif
          !
          ! --- Compute MinMod slope ---
          delta_a=(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                   max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Interpolate between vanLeer and MinMod slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta=SR*delta + (1.0-SR)*delta_a
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          d1d2=2.0*d(i1-1,i2,i3)*d(i1  ,i2,i3)
          if (d1d2 > 0.0) then
            delta=d1d2/(d(i1-1,i2,i3)+d(i1  ,i2,i3))
          else
            delta=0.0
          endif
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_Superbee) ! -------------------------------------------------------------
    ! --- Superbee ---
    !
    ! --- Compute Slope ---
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        d(i1,i2,i3)=av(i1+1,i2,i3)-av(i1  ,i2,i3)
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope: compute SuperBee slope ---
          delta=(sign(1.0,d(i1-1,i2,i3))+sign(1.0,d(i1  ,i2,i3))) * &
                (min(abs(d(i1-1,i2,i3)), &
                     abs(d(i1  ,i2,i3)), &
                     0.5*max(abs(d(i1-1,i2,i3)), &
                             abs(d(i1  ,i2,i3)))))
          !
          ! --- Compute MinMod slope ---
          delta_a=(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                   max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Interpolate between SuperBee and MinMod slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta=SR*delta + (1.0-SR)*delta_a
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          delta=(sign(1.0,d(i1-1,i2,i3))+sign(1.0,d(i1  ,i2,i3))) * &
                (min(abs(d(i1-1,i2,i3)), &
                     abs(d(i1  ,i2,i3)), &
                     0.5*max(abs(d(i1-1,i2,i3)), &
                             abs(d(i1  ,i2,i3)))))
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end select
!
end subroutine rhd_rec_VecEqSimple


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqPPmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqPPmimo ('rhd_reconstruction_Vector_Equidistant_PiecewiseParabolicminmod')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 1st-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqPPmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmimo and rhd_hyd_RecVecEq
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- Linear reconstructions ---
real                                       delta_a
!                                          --- PP ---
real                                    :: dam1, dap0, dap1, ap, am, aL, aR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: dapv
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: apv
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Average slope in zones (1.7) ---
        dap0=0.5*(ap1-am1)
        !
        ! --- Steepening, monotonizing (1.8) ---
        dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
             min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
        !
        dapv(i1,i2,i3)=dap0
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Slopes ---
        dap0=dapv(i1  ,i2,i3)
        dap1=dapv(i1+1,i2,i3)
        !
        ! --- Preliminary boundary values (1.6) ---
        ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * ( -dap1 + dap0 )
        !
        apv(i1,i2,i3)=ap
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Reduce slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute MinMod boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC=0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          aL=SC*aL + (1.0-SC)*am
          aR=SC*aR + (1.0-SC)*ap
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Reduce slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=ap0-delta_a
          aRv(i1,i2,i3)=ap0+delta_a
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=ap0-delta_a
          aRv(i1,i2,i3)=ap0+delta_a
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecEqPPmimo


!----------***************--------------------------------------------------------------------------
subroutine rhd_rec_VecEqPP(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqPP ('rhd_reconstruction_Vector_Equidistant_PiecewiseParabolic')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqPP(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                        SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                        av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmimo and rhd_hyd_RecVecEq
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- Linear reconstructions ---
real                                       delta_a
!                                          --- PP ---
real                                    :: dam1, dap0, dap1, ap, am, aL, aR, pmmtilL, pmmtilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: dapv
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: apv
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Average slope in zones (1.7) ---
        dap0=0.5*(ap1-am1)
        !
        ! --- Steepening, monotonizing (1.8) ---
        dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
             min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
        !
        dapv(i1,i2,i3)=dap0
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Slopes ---
        dap0=dapv(i1  ,i2,i3)
        dap1=dapv(i1+1,i2,i3)
        !
        ! --- Preliminary boundary values (1.6) ---
        ap=0.5 * (ap1+ap0) + 0.16666666666666666666 * ( -dap1 + dap0 )
        !
        apv(i1,i2,i3)=ap
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute MinMod boundary values ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR=SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          aL=SR*aL + (1.0-SR)*pmmtilL
          aR=SR*aR + (1.0-SR)*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC=0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          aL=SC*aL + (1.0-SC)*am
          aR=SC*aR + (1.0-SC)*ap
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute MinMod boundary values ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR=SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          aL=SR*aL + (1.0-SR)*pmmtilL
          aR=SR*aR + (1.0-SR)*pmmtilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqPP


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqFRmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqFRmimo ('rhd_reconstruction_Vector_Equidistant_Frankenstein_minmod')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 1st-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqFRmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC, &
                                           delta_a, aL, aR
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
        atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
        !
        ! --- Get coefficients of polynomials ---
        pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL=atilv(i1-1,i2,i3)
          atilR=atilv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          !SR     =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute MinMod boundary values ---
          ptilL=ap0-delta_a
          ptilR=ap0+delta_a
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL=atilv(i1-1,i2,i3)
          atilR=atilv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          !SR     =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute MinMod boundary values ---
          ptilL=ap0-delta_a
          ptilR=ap0+delta_a
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL=atilv(i1-1,i2,i3)
          atilR=atilv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqFRmimo


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqFRmono(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqFRmono ('rhd_reconstruction_Vector_Equidistant_Frankenstein_monotonic')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqFRmono(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-26 (B.F.) Derived from rhd_hyd_RecVecEqFRweno
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           pmmtilL, pmmtilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, dimension(m1-1 :n1+1    )         :: ptilLv, ptilRv, ptilRnv, ptilLnv, fLv, fRv, &
                                           pmmtilLv, pmmtilRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
        atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
        !
        ! --- Get coefficients of polynomials ---
        pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          pmmtilLv(i1)=pmmtilL
          pmmtilRv(i1)=pmmtilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          pmmtilL=pmmtilLv(i1)
          pmmtilR=pmmtilRv(i1)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          pmmtilLv(i1)=pmmtilL
          pmmtilRv(i1)=pmmtilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          pmmtilL=pmmtilLv(i1)
          pmmtilR=pmmtilRv(i1)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqFRmono


!----------********************---------------------------------------------------------------------
subroutine rhd_rec_VecEqFRmono2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqFRmono2 ('rhd_reconstruction_Vector_Equidistant_Frankenstein_monotonic_2')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monnotonic 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqFRmono2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                             av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-26 (B.F.) Derived from rhd_hyd_RecVecEqFRweno2
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, dimension(m1-1 :n1+1    )         :: ptilLv, ptilRv, ptilRnv, ptilLnv, fLv, fRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
        atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
        !
        ! --- Get coefficients of polynomials ---
        pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqFRmono2


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqFRweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqFRweno ('rhd_reconstruction_Vector_Equidistant_Frankenstein_weno')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqFRweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-26 (B.F.) Derived from rhd_hyd_RecVecEqFRweno2 and rhd_hyd_RecVecEq
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           pmmtilL, pmmtilR
real, dimension(m1-2: n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2: n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, dimension(m1-1: n1+1    )         :: ptilLv, ptilRv, ptilRnv, ptilLnv, fLv, fRv, &
                                           pmmtilLv, pmmtilRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
        atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
        !
        ! --- Get coefficients of polynomials ---
        pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        !atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          pmmtilLv(i1)=pmmtilL
          pmmtilRv(i1)=pmmtilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          pmmtilL=pmmtilLv(i1)
          pmmtilR=pmmtilRv(i1)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          pmmtilLv(i1)=pmmtilL
          pmmtilRv(i1)=pmmtilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          pmmtilL=pmmtilLv(i1)
          pmmtilR=pmmtilRv(i1)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqFRweno


!----------********************---------------------------------------------------------------------
subroutine rhd_rec_VecEqFRweno2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                                av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqFRweno2 ('rhd_reconstruction_Vector_Equidistant_Frankenstein_weno_2')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqFRweno2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                             av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, dimension(m1-1 :n1+1    )         :: ptilLv, ptilRv, ptilRnv, ptilLnv, fLv, fRv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
        atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
        !
        ! --- Get coefficients of polynomials ---
        pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        !atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !


          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !

          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !

          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !


          ! --- Testing: new loop split ---
          ptilLv(i1)=ptilL
          ptilRv(i1)=ptilR
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          f      =(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
          fLv(i1)=f
          ptilRnv(i1)=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
          !
          f      =(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
          fRv(i1)=f
          ptilLnv(i1)=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
        end do ! i1
        !
        !DIR$ SIMD
        do i1=m1-1,n1+1
          ptilL=ptilLv(i1)
          ptilR=ptilRv(i1)
          !


          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            !f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            f=fLv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              !ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=ptilRnv(i1)
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            !f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            f=fRv(i1)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              !ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilL=ptilLnv(i1)
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqFRweno2


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqFRcont(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqFRcont ('rhd_reconstruction_Vector_Equidistant_Frankenstein_continuous')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqFRcont(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2012-03-21 (B.F. Lyon) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-02-24 (B.F.) Put into separate routine
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        atil0 =  (11.0/6.0)*am1 - ( 7.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil1 =  ( 2.0/6.0)*am1 + ( 5.0/6.0)*ap0 - ( 1.0/6.0)*ap1
        atil2 = -( 1.0/6.0)*am1 + ( 5.0/6.0)*ap0 + ( 2.0/6.0)*ap1
        atil3 =  ( 2.0/6.0)*am1 - ( 7.0/6.0)*ap0 + (11.0/6.0)*ap1
        !
        ! --- Get coefficients of polynomials ---
        pc_20=  0.5 *am1 - ap0 + 0.5 *ap1
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        !atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-1,n1+1
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        ! --- Put left and right boundary values into vectors ---
        aLv(i1,i2,i3)=atilL
        aRv(i1,i2,i3)=atilR
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
end subroutine rhd_rec_VecEqFRcont


!----------*******************----------------------------------------------------------------------
subroutine rhd_rec_VecEqVAweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               SlopeRed_flag, SlopeRed, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqVAweno ('rhd_reconstruction_Vector_Equidistant_VAnilla_weno')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqVAweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            SlopeRed_flag, SlopeRed, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-13 (B.F. Uppsala) Adapted from rhd_hyd_RecVecEqFRweno2
!   2014-08-19 (B.F.) Adapted from rhd_hyd_RecVecEqVAweno
!   2014-08-20 (B.F.) Account for parameters from par file
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: wA0, wA1, wA2, &
                                           am1, ap0, ap1, &
                                           atil0, atil1, atil2, atil3, &
                                           acen0, acen1, acen2, &
                                           si0, si1, si2, &
                                                             atil2m1, atil3m1, &
                                                    atil1p0, atil2p0, &
                                           atil0p1, atil1p1, &
                                           acen2m1, acen1p0, acen0p1, &
                                                             si2m1  , &
                                                    si1p0  , si2p0  , &
                                           si0p1,   si1p1  ,          &
                                           wp1L, wp0L, wm1L, wp1R, wp0R, wm1R, &
                                           wisum, &
                                           ptilL, ptilR, &
                                           SR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, &
                                           si0v, si1v, si2v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    ! \label{f90:c_recwenoweightcenter__use3}
    ! --- Define weight array ---
    ! wA0=0.03 ; wA1=0.9 ; wA2=0.07
    wA1=par%C_RecWenoWeightCenter
    wA2=(1.0-wA1)*par%C_RecWenoWeightRight
    wA0=1.0-wA1-wA2
    !
    ! === Compute interpolation polynomials ========================================
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      !DIR$ SIMD
      do i1=m1-2,n1+2
        !
        ! --- Set abbreviations for cell values, set grid (dx=1) ---
        am1=av(i1-1,i2,i3)
        ap0=av(i1  ,i2,i3)
        ap1=av(i1+1,i2,i3)
        !
        ! --- Compute boundary values of polynomials that match three adjacent cells ---
        atil0= (11.0/ 6.0)*am1 - ( 7.0/ 6.0)*ap0 + ( 2.0/ 6.0)*ap1
        atil1= ( 2.0/ 6.0)*am1 + ( 5.0/ 6.0)*ap0 - ( 1.0/ 6.0)*ap1
        atil2=-( 1.0/ 6.0)*am1 + ( 5.0/ 6.0)*ap0 + ( 2.0/ 6.0)*ap1
        atil3= ( 2.0/ 6.0)*am1 - ( 7.0/ 6.0)*ap0 + (11.0/ 6.0)*ap1
        !
        ! --- Compute cell-center values ---
        acen0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
        acen1=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
        acen2=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
        !
        ! --- Compute smoothness of all polynomials in all intervals ---
        call rhd_rec_HBwenoSmoothness(atil0, acen0, atil1, si0)
        call rhd_rec_HBwenoSmoothness(atil1, acen1, atil2, si1)
        call rhd_rec_HBwenoSmoothness(atil2, acen2, atil3, si2)
        !
        ! --- Put values into vectors ---
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        si0v(i1,i2,i3)=si0
        si1v(i1,i2,i3)=si1
        si2v(i1,i2,i3)=si2
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then ! ----------------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          ! --- Set abbreviations ---
          ap0    =av(i1  ,i2,i3)
          !
          atil0p1=atil0v(i1+1,i2,i3)
          atil1p0=atil1v(i1  ,i2,i3)
          atil1p1=atil1v(i1+1,i2,i3)
          atil2m1=atil2v(i1-1,i2,i3)
          atil2p0=atil2v(i1  ,i2,i3)
          atil3m1=atil3v(i1-1,i2,i3)
          !
          si0p1  =si0v(i1+1,i2,i3)
          si1p0  =si1v(i1  ,i2,i3)
          si2m1  =si2v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L  =wA0/(si0p1+eps_tiny)
          wp0L  =wA1/(si1p0+eps_tiny)
          wm1L  =wA2/(si2m1+eps_tiny)
          !
          wisum=wp1L+wp0L+wm1L
          wp1L  =wp1L/wisum
          wp0L  =wp0L/wisum
          wm1L  =wm1L/wisum
          !
          wp1R  =wA2/(si0p1+eps_tiny)
          wp0R  =wA1/(si1p0+eps_tiny)
          wm1R  =wA0/(si2m1+eps_tiny)
          !
          wisum=wp1R+wp0R+wm1R
          wp1R  =wp1R/wisum
          wp0R  =wp0R/wisum
          wm1R  =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          ! --- Set abbreviations ---
          atil0p1=atil0v(i1+1,i2,i3)
          atil1p0=atil1v(i1  ,i2,i3)
          atil1p1=atil1v(i1+1,i2,i3)
          atil2m1=atil2v(i1-1,i2,i3)
          atil2p0=atil2v(i1  ,i2,i3)
          atil3m1=atil3v(i1-1,i2,i3)
          !
          si0p1  =si0v(i1+1,i2,i3)
          si1p0  =si1v(i1  ,i2,i3)
          si2m1  =si2v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L  =wA0/(si0p1+eps_tiny)
          wp0L  =wA1/(si1p0+eps_tiny)
          wm1L  =wA2/(si2m1+eps_tiny)
          !
          wisum=wp1L+wp0L+wm1L
          wp1L  =wp1L/wisum
          wp0L  =wp0L/wisum
          wm1L  =wm1L/wisum
          !
          wp1R  =wA2/(si0p1+eps_tiny)
          wp0R  =wA1/(si1p0+eps_tiny)
          wm1R  =wA0/(si2m1+eps_tiny)
          !
          wisum=wp1R+wp0R+wm1R
          wp1R  =wp1R/wisum
          wp0R  =wp0R/wisum
          wm1R  =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqVAweno


!----------********************---------------------------------------------------------------------
subroutine rhd_rec_VecEqHBweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                               SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                               av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEqHBweno ('rhd_reconstruction_Vector_Equidistant_HandBrake_weno')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEqHBweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                            av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-13 (B.F. Uppsala) Adapted from rhd_hyd_RecVecEqFRweno2
!   2014-08-20 (B.F.) Account for parameters from par file
!   2014-11-24 (B.F.) Interpolate differently for SlopeCon_flag case
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: wA0, wA1, wA2, &
                                           am1, ap0, ap1, &
                                           atil0, atil1, atil2, atil3, &
                                           acen0, acen1, acen2, &
                                           si0, si1, si2, &
                                                             atil2m1, atil3m1, &
                                                    atil1p0, atil2p0, &
                                           atil0p1, atil1p1, &
                                           acen2m1, acen1p0, acen0p1, &
                                                             si2m1  , &
                                                    si1p0  , si2p0  , &
                                           si0p1,   si1p1  ,          &
                                           wtil1p0, wtil0p1, wtilsum, &
                                           atil, amotil, &
                                           atilL, atilR, amotilL, amotilR, &
                                           wp1L, wp0L, wm1L, wp1R, wp0R, wm1R, wp1, wp0, wm1, &
                                           wisum, &
                                           ptilL, ptilR, ptil0L, ptil0R, &
                                           si01, si02, si12, siTot, &
                                           delta_a, aL, aR, &
                                           SR, SC
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, &
                                           acen0v, acen1v, acen2v, &
                                           si0v, si1v, si2v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv, amotilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    ! \label{f90:c_recwenoweightcenter__use4}
    ! --- Define weight array ---
    ! wA0=0.03 ; wA1=0.9 ; wA2=0.07
    wA1=par%C_RecWenoWeightCenter
    wA2=(1.0-wA1)*par%C_RecWenoWeightRight
    wA0=1.0-wA1-wA2
    !
    ! === Compute interpolation polynomials ========================================
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      !DIR$ SIMD
      do i1=m1-2,n1+2
        !
        ! --- Set abbreviations for cell values, set grid (dx=1) ---
        am1=av(i1-1,i2,i3)
        ap0=av(i1  ,i2,i3)
        ap1=av(i1+1,i2,i3)
        !
        ! --- Compute boundary values of polynomials that match three adjacent cells ---
        atil0= (11.0/ 6.0)*am1 - ( 7.0/ 6.0)*ap0 + ( 2.0/ 6.0)*ap1
        atil1= ( 2.0/ 6.0)*am1 + ( 5.0/ 6.0)*ap0 - ( 1.0/ 6.0)*ap1
        atil2=-( 1.0/ 6.0)*am1 + ( 5.0/ 6.0)*ap0 + ( 2.0/ 6.0)*ap1
        atil3= ( 2.0/ 6.0)*am1 - ( 7.0/ 6.0)*ap0 + (11.0/ 6.0)*ap1
        !
        ! --- Compute cell-center values ---
        acen0= (23.0/24.0)*am1 + ( 2.0/24.0)*ap0 - ( 1.0/24.0)*ap1
        acen1=-( 1.0/24.0)*am1 + (26.0/24.0)*ap0 - ( 1.0/24.0)*ap1
        acen2=-( 1.0/24.0)*am1 + ( 2.0/24.0)*ap0 + (23.0/24.0)*ap1
        !
        ! --- Compute smoothness of all polynomials in all intervals ---
        call rhd_rec_HBwenoSmoothness(atil0, acen0, atil1, si0)
        call rhd_rec_HBwenoSmoothness(atil1, acen1, atil2, si1)
        call rhd_rec_HBwenoSmoothness(atil2, acen2, atil3, si2)
        !
        ! --- Put values into vectors ---
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        acen0v(i1,i2,i3)=acen0
        acen1v(i1,i2,i3)=acen1
        acen2v(i1,i2,i3)=acen2
        !
        si0v(i1,i2,i3)=si0
        si1v(i1,i2,i3)=si1
        si2v(i1,i2,i3)=si2
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    ! === Compute preliminary (continuous) cell-boundary values ====================
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      !DIR$ SIMD
      do i1=m1-2,n1+1
        !
        ! --- Set abbreviations ---
        ap0    =av(i1  ,i2,i3)
        ap1    =av(i1+1,i2,i3)
        !
        atil1p1=atil1v(i1+1,i2,i3)
        atil2p0=atil2v(i1  ,i2,i3)
        !
        si1p0  =si1v(i1  ,i2,i3)
        si2p0  =si2v(i1  ,i2,i3)
        si0p1  =si0v(i1+1,i2,i3)
        si1p1  =si1v(i1+1,i2,i3)
        !
        ! --- Compute weights from smoothness of adjacent polynomials ---
        !
        wtil1p0=0.5/(si1p0+eps_tiny) + 0.5/(si2p0+eps_tiny)
        wtil0p1=0.5/(si0p1+eps_tiny) + 0.5/(si1p1+eps_tiny)
        wtilsum=wtil1p0+wtil0p1
        wtil1p0=wtil1p0/wtilsum
        wtil0p1=wtil0p1/wtilsum
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil   =wtil0p1*atil1p1 + wtil1p0*atil2p0
        !
        ! --- Monotonize ---
        amotil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        ! --- Put values into vectors ---
        atilv(  i1,i2,i3)=atil
        amotilv(i1,i2,i3)=amotil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then ! ----------------------------------------------------------------------
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          ! --- Set abbreviations for cell values ---
          ap0    =av(i1  ,i2,i3)
          !
          atilL  =atilv(i1-1,i2,i3)
          atilR  =atilv(i1  ,i2,i3)
          !
          amotilL=amotilv(i1-1,i2,i3)
          amotilR=amotilv(i1  ,i2,i3)
          !
          atil0p1=atil0v(i1+1,i2,i3)
          atil1p0=atil1v(i1  ,i2,i3)
          atil1p1=atil1v(i1+1,i2,i3)
          atil2m1=atil2v(i1-1,i2,i3)
          atil2p0=atil2v(i1  ,i2,i3)
          atil3m1=atil3v(i1-1,i2,i3)
          !
          acen0p1=acen0v(i1+1,i2,i3)
          acen1p0=acen1v(i1  ,i2,i3)
          acen2m1=acen2v(i1-1,i2,i3)
          !
          si0p1  =si0v(i1+1,i2,i3)
          si1p0  =si1v(i1  ,i2,i3)
          si2m1  =si2v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L  =wA0/(si0p1+eps_tiny)
          wp0L  =wA1/(si1p0+eps_tiny)
          wm1L  =wA2/(si2m1+eps_tiny)
          !
          wp1R  =wA2/(si0p1+eps_tiny)
          wp0R  =wA1/(si1p0+eps_tiny)
          wm1R  =wA0/(si2m1+eps_tiny)
          !
          wp1   =0.5*(wp1L+wp1R)
          wp0   =0.5*(wp0L+wp0R)
          wm1   =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1   =wp1/wisum
          wp0   =wp0/wisum
          wm1   =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L  =wp1L/wisum
          wp0L  =wp0L/wisum
          wm1L  =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R  =wp1R/wisum
          wp0R  =wp0R/wisum
          wm1R  =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          ptil0L=ptilL
          ptil0R=ptilR
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ! ptilL=SC*ptilL + (1.0-SC)*atilL
          ! ptilR=SC*ptilR + (1.0-SC)*atilR
          ptilL=SC*ptilL + (1.0-SC)*ptil0L
          ptilR=SC*ptilR + (1.0-SC)*ptil0R
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then ! -----------------------------------------------------------------
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          ! --- Set abbreviations for cell values ---
          ap0    =av(i1  ,i2,i3)
          !
          amotilL=amotilv(i1-1,i2,i3)
          amotilR=amotilv(i1  ,i2,i3)
          !
          atil0p1=atil0v(i1+1,i2,i3)
          atil1p0=atil1v(i1  ,i2,i3)
          atil1p1=atil1v(i1+1,i2,i3)
          atil2m1=atil2v(i1-1,i2,i3)
          atil2p0=atil2v(i1  ,i2,i3)
          atil3m1=atil3v(i1-1,i2,i3)
          !
          acen0p1=acen0v(i1+1,i2,i3)
          acen1p0=acen1v(i1  ,i2,i3)
          acen2m1=acen2v(i1-1,i2,i3)
          !
          si0p1  =si0v(i1+1,i2,i3)
          si1p0  =si1v(i1  ,i2,i3)
          si2m1  =si2v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L  =wA0/(si0p1+eps_tiny)
          wp0L  =wA1/(si1p0+eps_tiny)
          wm1L  =wA2/(si2m1+eps_tiny)
          !
          wp1R  =wA2/(si0p1+eps_tiny)
          wp0R  =wA1/(si1p0+eps_tiny)
          wm1R  =wA0/(si2m1+eps_tiny)
          !
          wp1   =0.5*(wp1L+wp1R)
          wp0   =0.5*(wp0L+wp0R)
          wm1   =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1   =wp1/wisum
          wp0   =wp0/wisum
          wm1   =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L  =wp1L/wisum
          wp0L  =wp0L/wisum
          wm1L  =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R  =wp1R/wisum
          wp0R  =wp0R/wisum
          wm1R  =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else ! -----------------------------------------------------------------------------------------
      ! --- Skip slope reduction and steepening ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          ! --- Set abbreviations for cell values ---
          ap0    =av(i1  ,i2,i3)
          !
          amotilL=amotilv(i1-1,i2,i3)
          amotilR=amotilv(i1  ,i2,i3)
          !
          atil0p1=atil0v(i1+1,i2,i3)
          atil1p0=atil1v(i1  ,i2,i3)
          atil1p1=atil1v(i1+1,i2,i3)
          atil2m1=atil2v(i1-1,i2,i3)
          atil2p0=atil2v(i1  ,i2,i3)
          atil3m1=atil3v(i1-1,i2,i3)
          !
          acen0p1=acen0v(i1+1,i2,i3)
          acen1p0=acen1v(i1  ,i2,i3)
          acen2m1=acen2v(i1-1,i2,i3)
          !
          si0p1  =si0v(i1+1,i2,i3)
          si1p0  =si1v(i1  ,i2,i3)
          si2m1  =si2v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials ---
          wp1L  =wA0/(si0p1+eps_tiny)
          wp0L  =wA1/(si1p0+eps_tiny)
          wm1L  =wA2/(si2m1+eps_tiny)
          !
          wp1R  =wA2/(si0p1+eps_tiny)
          wp0R  =wA1/(si1p0+eps_tiny)
          wm1R  =wA0/(si2m1+eps_tiny)
          !
          wp1   =0.5*(wp1L+wp1R)
          wp0   =0.5*(wp0L+wp0R)
          wm1   =0.5*(wm1L+wm1R)
          !
          wisum=wp1+wp0+wm1
          wp1   =wp1/wisum
          wp0   =wp0/wisum
          wm1   =wm1/wisum
          !
          wisum=wp1L+wp0L+wm1L
          wp1L  =wp1L/wisum
          wp0L  =wp0L/wisum
          wm1L  =wm1L/wisum
          !
          wisum=wp1R+wp0R+wm1R
          wp1R  =wp1R/wisum
          wp0R  =wp0R/wisum
          wm1R  =wm1R/wisum
          !
          ! --- Combine boundary values of polynomials with normalized weights in a WENO style ---
          ptilL=wm1L*atil2m1 + wp0L*atil1p0 + wp1L*atil0p1
          ptilR=wm1R*atil3m1 + wp0R*atil2p0 + wp1R*atil1p1
          !
          ! === Correct preliminary boundary values according to pairwise smoothness =====
          !
          ! --- Compute smoothness of all pairs of polynomials ---
          call rhd_rec_HBwenoSmoothness(atil0p1-atil1p0, acen0p1-acen1p0, atil1p1-atil2p0, si01)
          call rhd_rec_HBwenoSmoothness(atil0p1-atil2m1, acen0p1-acen2m1, atil1p1-atil3m1, si02)
          call rhd_rec_HBwenoSmoothness(atil1p0-atil2m1, acen1p0-acen2m1, atil2p0-atil3m1, si12)
          !
          ! --- Assemble total pairwise smoothness, scale and limit it ---
          siTot=wp1*wp0*si01/(si0p1+si1p0+eps_tiny) + &
                wp1*wm1*si02/(si0p1+si2m1+eps_tiny) + &
                wp0*wm1*si12/(si1p0+si2m1+eps_tiny)
          ! siTot=min(20.0*siTot, 1.0)
          siTot=min(par%C_RecWenoPsiFactor*siTot, 1.0)
          !
          ! --- Compute half MinMod slope from continuous boundary values ---
          delta_a=(sign(0.5,amotilR-ap0) + sign(0.5,ap0-amotilL)) * &
                  min(abs(amotilR-ap0), abs(ap0-amotilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Correct boundary values ---
          ptilL=siTot*aL + (1.0-siTot)*ptilL
          ptilR=siTot*aR + (1.0-siTot)*ptilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    endif ! ----------------------------------------------------------------------------------------
    !
end subroutine rhd_rec_VecEqHBweno


!----------*************----------------------------------------------------------------------------
subroutine rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                         reconstruction_order, reconstruction_type, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         av, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecEq ('rhd_reconstruction_Vector_Equidistant')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecEq(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                      reconstruction_order, reconstruction_type, &
!                      SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                      av, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell boundaries
!
! ROUTINES:
!   rhd_rec_VecEqSimple:   Reconstruct vector, compute bnd values with 0th or 1st-order mono. scheme
!   rhd_rec_VecEqPPmimo:   Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecEqPP:       Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRmimo:   Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecEqFRmono:   Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRmono2:  Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqFRweno2:  Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqFRcont:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqVAweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqHBweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F.) Adapted from rhd_mhd_PP
!   2011-01-04 (W.S. Exeter) Adapted from rhd_mhd_PPvec
!   2011-06-24 (W.S.) New parameters nghost2, nghost3, d2, d3 for use in unsplit Hancock scheme
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-05-09 (B.F.) Call subroutines with individual reconstructions
!   2014-08-19 (B.F. Uppsala) HBweno
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                           reconstruction_order, reconstruction_type
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (reconstruction_order <= 1) then
  ! === 0th and 1st order ==========================================================================
  call rhd_rec_VecEqSimple(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           reconstruction_type, &
                           SlopeRed_flag, SlopeRed, &
                           av, aLv, aRv)
else ! (reconstruction_order > 1)
 !
 select case(reconstruction_type)
  !
  ! === 2nd order ==================================================================================
  case(reconstruction_type_PPmimo) ! ---------------------------------------------------------------
    ! --- Take boundary values from 'Piecewise Parabolic', do only linear MinMod reconstruction ---
    !
    call rhd_rec_VecEqPPmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_PP) ! -------------------------------------------------------------------
    ! --- Piecewise Parabolic ---
    !
    call rhd_rec_VecEqPP(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         av, aLv, aRv)
    !
  case(reconstruction_type_FRmimo) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method MinMod slope ---
    !
    call rhd_rec_VecEqFRmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_FRmono) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic ---
    !
    call rhd_rec_VecEqFRmono(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_FRmono2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic 2 (more efficient slope reduction) ---
    !
    call rhd_rec_VecEqFRmono2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                              SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                              av, aLv, aRv)
    !
  case(reconstruction_type_FRweno) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory ---
    !
    call rhd_rec_VecEqFRweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_FRweno2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory (more efficient slope reduction) ---
    !
    call rhd_rec_VecEqFRweno2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                              SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                              av, aLv, aRv)
    !
  case(reconstruction_type_FRcont) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Continuous Essentially Non-oscillatory ---
    !
    call rhd_rec_VecEqFRcont(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_VAweno) ! ---------------------------------------------------------------
    ! --- Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_VecEqVAweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeRed, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_HBweno) ! ---------------------------------------------------------------
    ! --- HandBrake Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_VecEqHBweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, aLv, aRv)
    !
 end select ! (reconstruction_type)
endif ! (reconstruction_order)
!
end subroutine rhd_rec_VecEq


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_VecSimple(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             reconstruction_type, &
                             SlopeRed_flag, SlopeRed, &
                             av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecSimple ('rhd_reconstruction_Vector_Simple')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 0th- or 1st-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecSimple(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          reconstruction_type, &
!                          SlopeRed_flag, SlopeRed, &
!                          av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2013-08-26 (B.F.) Derived from rhd_hyd_RecVecEqSimple and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                           reconstruction_type
logical,                   intent(in)   :: SlopeRed_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: delta, delta_a, d1d2, SR
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: d
!---------------------------------------------------------------------------------------------------
!
select case(reconstruction_type)
  !
  ! === 0th order ==================================================================================
  case(reconstruction_type_Constant) ! -------------------------------------------------------------
    ! --- Donor cell with constant reconstruction (delta=0.0) ---
    !
    do i3=m3-d3,n3+d3
      do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          aLv(i1,i2,i3)=av(i1,i2,i3)
          aRv(i1,i2,i3)=av(i1,i2,i3)
          !
        end do ! i1
      end do ! i2
    end do ! i3
    !
  ! === 1st order ==================================================================================
  case(reconstruction_type_Minmod) ! ---------------------------------------------------------------
    ! --- Minmod ---
    !
    ! --- Compute Slope ---
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        d(i1,i2,i3)=2.0*(av(i1+1,i2,i3)-av(i1  ,i2,i3))/(dxb1(i1+1)+dxb1(i1  ))
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          delta=dxb1(i1)*(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                          max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Reduce slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta=SR*delta
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          delta=dxb1(i1)*(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                          max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_VanLeer) ! --------------------------------------------------------------
    ! --- Van Leer ---
    !
    ! --- Compute Slope ---
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        d(i1,i2,i3)=2.0*(av(i1+1,i2,i3)-av(i1  ,i2,i3))/(dxb1(i1+1)+dxb1(i1  ))
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope: compute vanLeer slope ---
          d1d2=2.0*d(i1-1,i2,i3)*d(i1  ,i2,i3)
          if (d1d2 > 0.0) then
            delta=dxb1(i1)*d1d2/(d(i1-1,i2,i3)+d(i1  ,i2,i3))
          else
            delta=0.0
          endif
          !
          ! --- Compute MinMod slope ---
          delta_a=dxb1(i1)*(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                            max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Interpolate between vanLeer and MinMod slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta=SR*delta + (1.0-SR)*delta_a
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          d1d2=2.0*d(i1-1,i2,i3)*d(i1  ,i2,i3)
          if (d1d2 > 0.0) then
            delta=dxb1(i1)*d1d2/(d(i1-1,i2,i3)+d(i1  ,i2,i3))
          else
            delta=0.0
          endif
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
  case(reconstruction_type_Superbee) ! -------------------------------------------------------------
    ! --- Superbee ---
    !
    ! --- Compute Slope ---
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        d(i1,i2,i3)=2.0*(av(i1+1,i2,i3)-av(i1  ,i2,i3))/(dxb1(i1+1)+dxb1(i1  ))
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeRed_flag) then
      ! --- Perform additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope: compute SuperBee slope ---
          delta=(sign(1.0,d(i1-1,i2,i3))+sign(1.0,d(i1  ,i2,i3))) * &
                (min(abs(av(i1  ,i2,i3)-av(i1-1,i2,i3)), &
                     abs(av(i1+1,i2,i3)-av(i1  ,i2,i3)), &
                     dxb1(i1)*0.5*max(abs(d(i1-1,i2,i3)), &
                                      abs(d(i1  ,i2,i3)))))
          !
          ! --- Compute MinMod slope ---
          delta_a=dxb1(i1)*(min(max(d(i1-1,i2,i3),0.0),max(d(i1  ,i2,i3),0.0)) + &
                            max(min(d(i1-1,i2,i3),0.0),min(d(i1  ,i2,i3),0.0)))
          !
          ! --- Interpolate between SuperBee and MinMod slope ---
          !SR   =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta=SR*delta + (1.0-SR)*delta_a
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    else
      ! --- No additional slope reduction ---
      !
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Limit slope ---
          delta=(sign(1.0,d(i1-1,i2,i3))+sign(1.0,d(i1  ,i2,i3))) * &
                (min(abs(av(i1  ,i2,i3)-av(i1-1,i2,i3)), &
                     abs(av(i1+1,i2,i3)-av(i1  ,i2,i3)), &
                     dxb1(i1)*0.5*max(abs(d(i1-1,i2,i3)), &
                                      abs(d(i1  ,i2,i3)))))
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=av(i1,i2,i3)-0.5*delta
          aRv(i1,i2,i3)=av(i1,i2,i3)+0.5*delta
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end select
!
end subroutine rhd_rec_VecSimple


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_VecPPmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecPPmimo ('rhd_reconstruction_Vector_PiecewiseParabolicminmod')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 1st-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecPPmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                          av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-26 (B.F.) Derived from rhd_hyd_RecVecEqPPmimo and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- Linear reconstructions ---
real                                       delta_a
!                                          --- PP ---
real                                    :: dam1, dap0, dap1, ap, am, aL, aR, &
                                           dxm1, dxp0, dxp1, dxp2
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: dapv
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: apv
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Grid size ---
        dxm1=dxb1(i1-1)
        dxp0=dxb1(i1  )
        dxp1=dxb1(i1+1)
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Average slope in zones (1.7) ---
        dap0=  dxp0/(dxm1+dxp0+dxp1) * &
             ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
               (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
        !
        ! --- Steepening, monotonizing (1.8) ---
        dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
             min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
        !
        dapv(i1,i2,i3)=dap0
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Grid size ---
        dxm1=dxb1(i1-1)
        dxp0=dxb1(i1  )
        dxp1=dxb1(i1+1)
        dxp2=dxb1(i1+2)
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Slopes ---
        dap0=dapv(i1  ,i2,i3)
        dap1=dapv(i1+1,i2,i3)
        !
        ! --- Preliminary boundary values (1.6) ---
        ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
           1.0/(dxm1+dxp0+dxp1+dxp2) * &
           ( 2.0*dxp1*dxp0/(dxp0+dxp1) * ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0))*&
               (ap1-ap0) - &
             dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1)*dap1 + &
             dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1)*dap0  )
        !
        apv(i1,i2,i3)=ap
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Reduce slope ---
          !SR     =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR     =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute MinMod boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC=0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          aL=SC*aL + (1.0-SC)*am
          aR=SC*aR + (1.0-SC)*ap
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Reduce slope ---
          !SR     =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR     =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=ap0-delta_a
          aRv(i1,i2,i3)=ap0+delta_a
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,ap-ap0) + sign(0.5,ap0-am)) * min(abs(ap-ap0), abs(ap0-am))
          !
          ! --- Compute left and right boundary values ---
          aLv(i1,i2,i3)=ap0-delta_a
          aRv(i1,i2,i3)=ap0+delta_a
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecPPmimo


!----------*************----------------------------------------------------------------------------
subroutine rhd_rec_VecPP(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                         SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                         av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecPP ('rhd_reconstruction_Vector_PiecewiseParabolic')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecPP(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                      SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                      av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2014-08-26 (B.F.) Derived from rhd_hyd_RecVecPPmimo and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC
!                                          --- Linear reconstructions ---
real                                       delta_a
!                                          --- PP ---
real                                    :: dam1, dap0, dap1, ap, am, aL, aR, pmmtilL, pmmtilR, &
                                           dxm1, dxp0, dxp1, dxp2
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: dapv
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: apv
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Grid size ---
        dxm1=dxb1(i1-1)
        dxp0=dxb1(i1  )
        dxp1=dxb1(i1+1)
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Average slope in zones (1.7) ---
        dap0=  dxp0/(dxm1+dxp0+dxp1) * &
             ( (2.0*dxm1+dxp0)/(dxp1+dxp0) * (ap1-ap0) + &
               (dxp0+2.0*dxp1)/(dxm1+dxp0) * (ap0-am1)  )
        !
        ! --- Steepening, monotonizing (1.8) ---
        dap0=(sign(0.5,ap1-ap0) + sign(0.5,ap0-am1)) * &
             min(abs(dap0),2.0*abs(ap1-ap0),2.0*abs(ap0-am1))
        !
        dapv(i1,i2,i3)=dap0
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Grid size ---
        dxm1=dxb1(i1-1)
        dxp0=dxb1(i1  )
        dxp1=dxb1(i1+1)
        dxp2=dxb1(i1+2)
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Slopes ---
        dap0=dapv(i1  ,i2,i3)
        dap1=dapv(i1+1,i2,i3)
        !
        ! --- Preliminary boundary values (1.6) ---
        ap=ap0 + dxp0/(dxp0+dxp1) * (ap1-ap0) + &
           1.0/(dxm1+dxp0+dxp1+dxp2) * &
           ( 2.0*dxp1*dxp0/(dxp0+dxp1) * ((dxm1+dxp0)/(2.0*dxp0+dxp1)-(dxp2+dxp1)/(2.0*dxp1+dxp0))*&
               (ap1-ap0) - &
             dxp0*(dxm1+dxp0)/(2.0*dxp0+dxp1)*dap1 + &
             dxp1*(dxp1+dxp2)/(dxp0+2.0*dxp1)*dap0  )
        !
        apv(i1,i2,i3)=ap
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute MinMod boundary values ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR=SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          aL=SR*aL + (1.0-SR)*pmmtilL
          aR=SR*aR + (1.0-SR)*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC=0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          aL=SC*aL + (1.0-SC)*am
          aR=SC*aR + (1.0-SC)*ap
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Compute MinMod boundary values ---
          delta_a=(sign(0.5,aR-ap0) + sign(0.5,ap0-aL)) * min(abs(aR-ap0), abs(ap0-aL))
          pmmtilL=ap0-delta_a
          pmmtilR=ap0+delta_a
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR=SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          aL=SR*aL + (1.0-SR)*pmmtilL
          aR=SR*aR + (1.0-SR)*pmmtilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          ! --- Preliminary boundary values ---
          am  =apv(i1-1,i2,i3)
          ap  =apv(i1  ,i2,i3)
          !
          ! --- Corrected boundary values (1.10) ---
          if ((ap-ap0)*(ap0-am) <= 0.0) then
            ! --- Local extremum in input values ---
            aL=ap0
            aR=ap0
          else if (((ap-am)*(ap0-0.5*(ap+am))) > (0.16666666666666666666*((ap-am)**2))) then
            !!$              else if ((ap-am)*(ap0-0.5*(ap+am)) > (ap-am)**2/6.0) then
            ! --- Inner extremum in parabola ---
            aL=3.0*ap0-2.0*ap
            aR=ap
          else if ((-0.16666666666666666666*((ap-am)**2)) > ((ap-am)*(ap0-0.5*(ap+am)))) then
            !!$              else if (-(ap-am)**2/6.0 > (ap-am)*(ap0-0.5*(ap+am))) then
            ! --- Inner extremum in parabola ---
            aL=am
            aR=3.0*ap0-2.0*am
          else
            ! --- Standard case ---
            aL=am
            aR=ap
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecPP


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_VecFRmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecFRmimo ('rhd_reconstruction_Vector_Equidistant_Frankenstein')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 1st-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecFRmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                          av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2014-08-26 (B.F. Uppsala) Derived from rhd_hyd_RecVecEqFRmimo and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC, &
                                           delta_a, aL, aR
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           x2, x3, Di
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Compute geometric coefficients and determinant ---
        !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        x2=(           dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        x3=(dxb1(i1+1)+dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        Di=1.0/(x2*x3*(x3-1.0))
        !
        atil0=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * am1+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap0+&
             (                                                x2**2                        ) * ap1 )
        atil1=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
        atil2=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
        atil3=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am1+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * ap0+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap1 )
        !
        ! --- Get coefficients of polynomials ---
        !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
        pc_20=Di * 3.0 * (dxb1(i1)/dxb1(i1-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL=atilv(i1-1,i2,i3)
          atilR=atilv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          !SR     =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute MinMod boundary values ---
          ptilL=ap0-delta_a
          ptilR=ap0+delta_a
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL=atilv(i1-1,i2,i3)
          atilR=atilv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Reduce slope ---
          !SR     =0.5*(SlopeRed(i1,i2,i3)+SlopeRed(i1+1,i2,i3))
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          delta_a=SR*delta_a
          !
          ! --- Compute MinMod boundary values ---
          ptilL=ap0-delta_a
          ptilR=ap0+delta_a
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL=atilv(i1-1,i2,i3)
          atilR=atilv(i1  ,i2,i3)
          !
          ! --- Compute half MinMod slope ---
          delta_a=(sign(0.5,atilR-ap0)+sign(0.5,ap0-atilL)) * min(abs(atilR-ap0),abs(ap0-atilL))
          !
          ! --- Compute left and right boundary values ---
          aL=ap0-delta_a
          aR=ap0+delta_a
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=aL
          aRv(i1,i2,i3)=aR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecFRmimo


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_VecFRmono(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecFRmono ('rhd_reconstruction_Vector_Equidistant_Frankenstein_monotonic')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecFRmono(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                          av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2014-08-26 (B.F. Uppsala) Derived from rhd_hyd_RecVecFRmimo and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC, &
                                           delta_a, aL, aR
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           x2, x3, Di, &
                                           pmmtilL, pmmtilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Compute geometric coefficients and determinant ---
        !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        x2=(           dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        x3=(dxb1(i1+1)+dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        Di=1.0/(x2*x3*(x3-1.0))
        !
        atil0=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * am1+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap0+&
             (                                                x2**2                        ) * ap1 )
        atil1=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
        atil2=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
        atil3=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am1+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * ap0+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap1 )
        !
        ! --- Get coefficients of polynomials ---
        !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
        pc_20=Di * 3.0 * (dxb1(i1)/dxb1(i1-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecFRmono


!----------******************-----------------------------------------------------------------------
subroutine rhd_rec_VecFRmono2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                              SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                              av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecFRmono2 ('rhd_reconstruction_Vector_Equidistant_Frankenstein_monotonic_2')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with monotonic 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecFRmono2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                           av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2014-08-26 (B.F. Uppsala) Derived from rhd_hyd_RecVecFRmono and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC, &
                                           delta_a, aL, aR
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           x2, x3, Di
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Compute geometric coefficients and determinant ---
        !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        x2=(           dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        x3=(dxb1(i1+1)+dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        Di=1.0/(x2*x3*(x3-1.0))
        !
        atil0=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * am1+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap0+&
             (                                                x2**2                        ) * ap1 )
        atil1=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
        atil2=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
        atil3=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am1+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * ap0+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap1 )
        !
        ! --- Get coefficients of polynomials ---
        !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
        pc_20=Di * 3.0 * (dxb1(i1)/dxb1(i1-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
              ptilR=3.0*ap0-2.0*ptilL
            endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
              ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
              ptilL=3.0*ap0-2.0*ptilR
            endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecFRmono2


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_VecFRweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecFRweno ('rhd_reconstruction_Vector_Equidistant_Frankenstein_weno')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecFRweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                          av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2014-08-26 (B.F. Uppsala) Derived from rhd_hyd_RecVecFRmono and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC, &
                                           delta_a, aL, aR
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           x2, x3, Di, &
                                           pmmtilL, pmmtilR
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Compute geometric coefficients and determinant ---
        !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        x2=(           dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        x3=(dxb1(i1+1)+dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        Di=1.0/(x2*x3*(x3-1.0))
        !
        atil0=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * am1+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap0+&
             (                                                x2**2                        ) * ap1 )
        atil1=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
        atil2=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
        atil3=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am1+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * ap0+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap1 )
        !
        ! --- Get coefficients of polynomials ---
        !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
        pc_20=Di * 3.0 * (dxb1(i1)/dxb1(i1-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        !atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
            ! --- Set MinMod boundary values ---
            pmmtilL=ap0
            pmmtilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=ptilL
            pmmtilR=2.0*ap0-ptilL
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
            ! --- Compute MinMod boundary values ---
            pmmtilL=2.0*ap0-ptilR
            pmmtilR=ptilR
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and MinMod boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*pmmtilL
          ptilR=SR*ptilR + (1.0-SR)*pmmtilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecFRweno


!----------******************-----------------------------------------------------------------------
subroutine rhd_rec_VecFRweno2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                              SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                              av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecFRweno2 ('rhd_reconstruction_Vector_Equidistant_Frankenstein_weno_2')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecFRweno2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                           av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2014-08-26 (B.F. Uppsala) Derived from rhd_hyd_RecVecFRmono2 and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3 ), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1, SR, SC, &
                                           delta_a, aL, aR
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, wem, we0, wep, wesum, f, ptilL, ptilR, &
                                           x2, x3, Di
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      !DIR$ SIMD
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Compute geometric coefficients and determinant ---
        !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        x2=(           dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        x3=(dxb1(i1+1)+dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        Di=1.0/(x2*x3*(x3-1.0))
        !
        atil0=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * am1+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap0+&
             (                                                x2**2                        ) * ap1 )
        atil1=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
        atil2=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
        atil3=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am1+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * ap0+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap1 )
        !
        ! --- Get coefficients of polynomials ---
        !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
        pc_20=Di * 3.0 * (dxb1(i1)/dxb1(i1-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      !DIR$ SIMD
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        !atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    if (SlopeCon_flag) then
      ! --- Continuation flag is set: do both slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Interpolate between computed and continuous boundary values ---
          SC   =0.5*(SlopeCon(i1,i2,i3)+SlopeCon(i1+1,i2,i3))
          ptilL=SC*ptilL + (1.0-SC)*atilL
          ptilR=SC*ptilR + (1.0-SC)*atilR
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else if (SlopeRed_flag) then 
      ! --- Only slope-reduction flag is set: do only slope reduction ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! === Interpolate between "proper" boundary values and others acc. to external weights ===
          !
          ! --- Interpolate between computed and constant boundary values ---
          SR   =SlopeRed(i1,i2,i3)*SlopeRed(i1+1,i2,i3)
          ptilL=SR*ptilL + (1.0-SR)*ap0
          ptilR=SR*ptilR + (1.0-SR)*ap0
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
      !
    else
      ! --- Skip slope reduction and steepening ---
      do i3=m3-d3,n3+d3
       do i2=m2-d2,n2+d2
        !DIR$ SIMD
        do i1=m1-1,n1+1
          !
          atil0L=atil0v(i1+1,i2,i3)
          atil1L=atil1v(i1  ,i2,i3)
          atil1R=atil1v(i1+1,i2,i3)
          atil2L=atil2v(i1-1,i2,i3)
          atil2R=atil2v(i1  ,i2,i3)
          atil3R=atil3v(i1-1,i2,i3)
          !
          ! === Construct preliminary boundary values of reconstruction polynomial =======
          !
          ! --- Compute normalized weights of polynomials from quadratic coefficients ---
          wem  =0.5*we02v(i1-1,i2,i3)
          we0  =    we02v(i1  ,i2,i3)
          wep  =0.5*we02v(i1+1,i2,i3)
          wesum=wem+we0+wep
          wem  =wem/wesum
          we0  =we0/wesum
          wep  =wep/wesum
          !
          ! --- Combine boundary values of polynomials acc. to normalized weights in a WENO style --
          ptilL=wem*atil2L + we0*atil1L + wep*atil0L
          ptilR=wem*atil3R + we0*atil2R + wep*atil1R
          !
          ! --- Cell values ---
          ap0 =av(i1  ,i2,i3)
          !
          atilL = atilv(i1-1,i2,i3)
          atilR = atilv(i1  ,i2,i3)
          !
          ! === Monotonize boundary values ===============================================
          if (((ap0-atilL)*(ap0-atilR) >= 0.0) .or. &
              ((ap0-atilL)*(ap0-ptilL) <= 0.0) .or. &
              ((ap0-atilR)*(ap0-ptilR) <= 0.0)) then
            ! --- Local extremum in preliminary boundary values or               ---
            ! --- Wrong sign of ptilL compared to preliminary boundary value or  ---
            ! --- Wrong sign of ptilR compared to preliminary boundary value or: ---
            ! --- Reconstruction constant                                        ---
            ptilL=ap0
            ptilR=ap0
            !
          else if (abs(atilL-ap0) <= abs(atilR-ap0)) then
            ! --- Left side has smaller slope ---
            !
            f=(abs(ptilL-ap0)+eps_tiny)/(abs(atilL-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude on both sides -
              ptilL=atilL
              ptilR=(ptilR-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
            endif
            !
            if (abs(ptilR-ap0) > abs(atilR-ap0)) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude ---
              ptilR=atilR
            endif
            !
            !if (((-1.0/6.0)*(ptilR-ptilL)**2) > ((ptilR-ptilL)*(ap0-0.5*(ptilR+ptilL)))) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilR and ap0 ---
            !  ptilR=3.0*ap0-2.0*ptilL
            !endif
            !
          else
            ! --- Right side has smaller slope ---
            !
            f=(abs(ptilR-ap0)+eps_tiny)/(abs(atilR-ap0)+eps_tiny)
            if (f > 1.0) then
              ! --- Amplitude of ptilR too large compared to atilR: reduce amplitude on both sides -
              ptilL=(ptilL-ap0)/((2.0/3.0)*f+(1.0/3.0)) + ap0
              ptilR=atilR
            endif
            !
            if (abs(ptilL-ap0) > abs(atilL-ap0)) then
              ! --- Amplitude of ptilL too large compared to atilL: reduce amplitude ---
              ptilL=atilL
            endif
            !
            !if (((ptilR-ptilL)*(ap0-0.5*(ptilR-ptilL))) > ((1.0/6.0)*(ptilR+ptilL)**2)) then
            !  ! --- Inner extremum in parabola: do the PPM restriction step with ptilL and ap0 ---
            !  ptilL=3.0*ap0-2.0*ptilR
            !endif
            !
          endif
          !
          ! --- Put left and right boundary values into vectors ---
          aLv(i1,i2,i3)=ptilL
          aRv(i1,i2,i3)=ptilR
          !
        end do ! i1
       end do ! i2
      end do ! i3
    endif
    !
end subroutine rhd_rec_VecFRweno2


!----------*****************------------------------------------------------------------------------
subroutine rhd_rec_VecFRcont(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_VecFRcont ('rhd_reconstruction_Vector_Equidistant_Frankenstein_continuous')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values with 2nd-order scheme.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_VecFRcont(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                          av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2013-04-30 (B.F.) Derived from rhd_hyd_RecVecEqFRmono and rhd_hyd_RecVecEq
!   2014-08-26 (B.F. Uppsala) Derived from rhd_hyd_RecVecFRmono and rhd_hyd_RecVec
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
integer                                 :: i1, i2, i3
real                                    :: am1, ap0, ap1
!                                          --- FR ---
real                                    :: atil0, atil1, atil2, atil3, &
                                           pc_20, we01, we02, c0, cp, atil, &
                                           atil0L, atil1L, atil1R, atil2L, atil2R, atil3R, &
                                           atilL, atilR, &
                                           x2, x3, Di
real, dimension(m1-2 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atil0v, atil1v, atil2v, atil3v, we01v, we02v
real, dimension(m1-2 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3   )         :: atilv
real, parameter                         :: eps_tiny=10.0*tiny(0.0)
!---------------------------------------------------------------------------------------------------
!
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+2
        !
        ! --- Cell values ---
        am1 =av(i1-1,i2,i3)
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        ! --- Compute geometric coefficients and determinant ---
        !x2=(xb1(i+1)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        !x3=(xb1(i+2)-xb1(i-1))/(xb1(i  )-xb1(i-1))
        x2=(           dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        x3=(dxb1(i1+1)+dxb1(i1  )+dxb1(i1-1))/(dxb1(i1-1))
        Di=1.0/(x2*x3*(x3-1.0))
        !
        atil0=Di*( &
             (     -x2 -x3         +x3**2                    +x2*x3**2                     ) * am1+&
             (      x2 +x3         -x2**2 -x2*x3 -x3**2                                    ) * ap0+&
             (                                                x2**2                        ) * ap1 )
        atil1=Di*( &
             (-1.0 +x2     +2.0*x3 -2.0*x2*x3 -x3**2         +x2*x3**2                     ) * am1+&
             ( 1.0 -2.0*x2 -2.0*x3 +x2**2 +x2*x3 +x3**2                                    ) * ap0+&
             (      x2             -x2**2                                                  ) * ap1 )
        atil2=Di*( &
             (      x2 -x3         -x2**2 +x3**2             +x2**2*x3 -x2*x3**2           ) * am1+&
             (     -x2 +x3         +2.0*x2**2 -x2*x3 -x3**2  -x2**3 -x2**2*x3 +2.0*x2*x3**2) * ap0+&
             (                     -x2**2                    +x2**3                        ) * ap1 )
        atil3=Di*( &
             (     -x2 +x3         +2.0*x2*x3 -2.0*x3**2     -x2*x3**2 +x3**3              ) * am1+&
             (      x2 -x3         -x2**2 -x2*x3 +2.0*x3**2  +2.0*x2**2*x3 -x2*x3**2 -x3**3) * ap0+&
             (                      x2**2-2.0*x2*x3          -2.0*x2**2*x3 +3.0*x2*x3**2   ) * ap1 )
        !
        ! --- Get coefficients of polynomials ---
        !pc_20 =Di * 3.0 * ((xb1(i+1)-xb1(i  ))/(xb1(i  )-xb1(i-1)))**2 * ( &
        pc_20=Di * 3.0 * (dxb1(i1)/dxb1(i1-1))**2 * ( &
             (-1.0 +x3                                                                     ) * am1+&
             ( 1.0 -x2 -x3                                                                 ) * ap0+&
             (      x2                                                                     ) * ap1 )
        !
        ! --- Get weights from quadratic coefficients ---
        we01=0.5/(abs(pc_20) + eps_tiny)
        we02=0.5/((pc_20)**2 + eps_tiny)
        !
        atil0v(i1,i2,i3)=atil0
        atil1v(i1,i2,i3)=atil1
        atil2v(i1,i2,i3)=atil2
        atil3v(i1,i2,i3)=atil3
        !
        we01v(i1,i2,i3)=we01
        we02v(i1,i2,i3)=we02 
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-2,n1+1
        !
        ! --- Cell values ---
        ap0 =av(i1  ,i2,i3)
        ap1 =av(i1+1,i2,i3)
        !
        c0 = we01v(i1  ,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        cp = we01v(i1+1,i2,i3)/(we01v(i1  ,i2,i3)+we01v(i1+1,i2,i3))
        !
        ! --- Compute weighted averages of boundary values in a WENO style ---
        atil = c0*atil2v(i1  ,i2,i3) + cp*atil1v(i1+1,i2,i3)
        !
        ! --- Monotonize ---
        !atil =min(max(ap0,ap1), max(min(ap0,ap1), atil))
        !
        atilv(i1,i2,i3)=atil
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
    do i3=m3-d3,n3+d3
     do i2=m2-d2,n2+d2
      do i1=m1-1,n1+1
        !
        atilL = atilv(i1-1,i2,i3)
        atilR = atilv(i1  ,i2,i3)
        !
        ! --- Put left and right boundary values into vectors ---
        aLv(i1,i2,i3)=atilL
        aRv(i1,i2,i3)=atilR
        !
      end do ! i1
     end do ! i2
    end do ! i3
    !
end subroutine rhd_rec_VecFRcont


!----------***********------------------------------------------------------------------------------
subroutine rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                       reconstruction_order, reconstruction_type, &
                       SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                       av, dxb1, aLv, aRv)
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_rec_Vec ('rhd_reconstruction_Vector')
!
! PURPOSE:
!   Reconstruct input vector and compute boundary values.
!   
! CATEGORY:
!   Interpolation, reconstruction, magneto-hydrodynamics
!
! CALLING SEQUENCE:
!   call rhd_rec_Vec(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
!                    reconstruction_type, &
!                    SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
!                    av, dxb1, aLv, aRv)
!
! INPUT:
!   m1:           ('m_1') integer, lower index, 1st dimension
!   n1:           ('n_1') integer, upper index, 1st dimension
!   m2:           ('m_2') integer, lower index, 2nd dimension
!   n2:           ('n_2') integer, upper index, 2nd dimension
!   m3:           ('m_3') integer, lower index, 3rd dimension
!   n3:           ('n_3') integer, upper index, 3rd dimension
!   nghost1:      ('number_ghost_1') integer, 1. dimension, number of ghost cells
!   nghost2:      ('number_ghost_2') integer, 2. dimension, number of ghost cells
!   nghost3:      ('number_ghost_3') integer, 3. dimension, number of ghost cells
!   d2:           ('d_2') integer, number of additional cells for reconstruction 2. dimension
!   d3:           ('d_3') integer, number of additional cells for reconstruction 3. dimension
!   reconstruction_order: ('reconstruction_order') integer, order of reconstruction scheme
!   reconstruction_type: ('reconstruction_type') integer, type of reconstruction scheme
!   SlopeRed_flag: ('Slope_Reduction_flag') logical indicating if slope reduction should be
!                  performed
!   SlopeCon_flag: ('Slope_Continuous_flag') logical indicating if steepening should be performed
!   SlopeRed:     ('Slope_Reduction') 3D real array with slope-reduction factor
!   SlopeCon:     ('Slope_Continuous') 3D real array with make-continuous factor
!   av:           ('alpha_vector') real array with values to reconstruct
!   dxb1:         ('delta_x_boundary_1') real, vector with grid cell size
!
! OUTPUT:
!   aLv:          ('alpha_Left_vector') real array with reconstructed values at left cell boundaries
!   aRv:          ('alpha_Real_vector') real array with reconstructed values at right cell bnds
!
! ROUTINES:
!   rhd_rec_VecSimple:     Reconstruct vector, compute bnd values with 0th or 1st-order mono. scheme
!   rhd_rec_VecPPmimo:     Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecPP:         Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRmimo:     Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecFRmono:     Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRmono2:    Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRweno:     Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRweno2:    Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRcont:     Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqVAweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqHBweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!
! MODULES:
!   rhd_action_module:     Routines to handle the control structure 'action'
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!    VAweno and HBweno: Just call the equidistant versions.
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1997-07-23 (B.F. Kiel) First version
!   2001-01-20 (B.F. Uppsala) Equidistant version
!   2001-02-11 (B.F.) Special case for this reconstruction, scalar version
!   2001-02-13 (B.F.) Non-equidistant version
!   2010-06-08 (B.F. Lyon) Adapted from rhd_hyd_PP
!   2010-06-09 (B.F.) Adapted from rhd_mhd_PP
!   2011-01-04 (W.S. Exeter) Adapted from rhd_mhd_PPvec
!   2011-06-24 (W.S. Exeter) New parameters nghost2, nghost3, d2, d3 for use in unsplit 
!                            Hancock scheme
!   2012-03-21 (B.F.) Adapted from rhd_mhd_PLEqvec, rhd_mhd_PPEqvec, rhd_mhd_FREqvec
!   2012-03-27 (B.F.) Account for SlopeRed, SlopeCon. Non-equidistant version
!   2012-04-16 (B.F.) Bugfix in declaration of SlopeRed, SlopeCon
!   2012-04-17 (B.F.) Increased array size of SlopeRed, SlopeCon
!   2012-06-15 (B.F.) Bugfix in FRmimo (vanilla) branch
!   2012-10-24 (B.F.) FRmono2, FRweno2
!   2012-10-29 (B.F.) Change formula for combining adjacent slope-reduction factors
!   2014-08-25 (B.F. Uppsala) Just call the equidistant versions of VAweno and HBweno routines
!   2014-08-26 (B.F.) Put sections into individual subroutines
!   2018-05-09 (B.F.) Transform dimension specifications from internal to input parameters
!---------------------------------------------------------------------------------------------------
use rhd_action_module
!
implicit none
!
! --- I/O parameters ---
integer,                   intent(in)   :: m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                                           reconstruction_order, reconstruction_type
logical,                   intent(in)   :: SlopeRed_flag, SlopeCon_flag
real, dimension(m1-1 :n1+2 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(in)   :: SlopeRed, SlopeCon
real, dimension(m1-nghost1:n1+nghost1,&
                m2-nghost2:n2+nghost2,&
                m3-nghost3:n3+nghost3), &
                           intent(in)   :: av
real, dimension(m1-nghost1:n1+nghost1), &
                           intent(in)   :: dxb1
real, dimension(m1-1 :n1+1 , &
                m2-d2:n2+d2, &
                m3-d3:n3+d3), &
                           intent(out)  :: aLv, aRv
!
! --- Local variables ---
!---------------------------------------------------------------------------------------------------
!
if (reconstruction_order <= 1) then
  ! === 0th and 1st order ==========================================================================
  call rhd_rec_VecSimple(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                         reconstruction_type, &
                         SlopeRed_flag, SlopeRed, &
                         av, dxb1, aLv, aRv)
else ! (reconstruction_order > 1)
 !
 select case(reconstruction_type)
  !
  ! === 2nd order ==================================================================================
  case(reconstruction_type_PPmimo) ! ---------------------------------------------------------------
    ! --- Take boundary values from 'Piecewise Parabolic', do only linear MinMod reconstruction ---
    !
    call rhd_rec_VecPPmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_PP) ! -------------------------------------------------------------------
    ! --- Piecewise Parabolic ---
    !
    call rhd_rec_VecPP(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                       SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                       av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_FRmimo) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method MinMod slope ---
    !
    call rhd_rec_VecFRmimo(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_FRmono) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic ---
    !
    call rhd_rec_VecFRmono(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_FRmono2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Monotonic 2 (more efficient slope reduction) ---
    !
    call rhd_rec_VecFRmono2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                            av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_FRweno) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory ---
    !
    call rhd_rec_VecFRweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                           av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_FRweno2) ! --------------------------------------------------------------
    ! --- Frankenstein's Method Essentially Non-oscillatory (more efficient slope reduction) ---
    !
    call rhd_rec_VecFRweno2(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                            SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                            av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_FRcont) ! ---------------------------------------------------------------
    ! --- Frankenstein's Method Continuous Essentially Non-oscillatory continuous ---
    !
    call rhd_rec_VecFRcont(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                           av, dxb1, aLv, aRv)
    !
  case(reconstruction_type_VAweno) ! ---------------------------------------------------------------
    ! --- Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_VecEqVAweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeRed, &
                             av, aLv, aRv)
    !
  case(reconstruction_type_HBweno) ! ---------------------------------------------------------------
    ! --- HandBrake Weighted Essentially Non-Oscillatory ---
    !
    call rhd_rec_VecEqHBweno(m1,n1, m2,n2, m3,n3, nghost1, nghost2, nghost3, d2, d3, &
                             SlopeRed_flag, SlopeCon_flag, SlopeRed, SlopeCon, &
                             av, aLv, aRv)
    !
 end select
endif ! (reconstruction_order)
!
end subroutine rhd_rec_Vec

end module rhd_rec_module
