pro rhdmodeldata, ModelTable

;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhdmodeldata ('model_plot_data_base_de_donnee')
;
; PURPOSE:
;   Provide basic model information.
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;    rhdmodeldata, ModelTable
;
; INPUT:
;   None
;
; OUTPUT:
;   ModelTable: ('model_table') 1D array of structures with model information
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Recompile often to account for changes!
;
; PROCEDURE:
;   The routines just contains an array of structures with components:
;     Model:         't57g44mm00n01',   string with unique name of the model,
;                                       there are lots of naming schemes around
;     Status:        'Finished',        string: 'Planned', 'Running', 'Finished', 'Crashed', 'Finished with problems',...
;     Quality:       0.8,               Real between 0.0 and 1.0 indication the quality or weight of the model.
;                                       Short test runs, newly started runs, and crashed/buggy simulations should get a 0.0.
;                                       A perfectly converged model should get 1.0.
;                                       The range in between is for low-res models, useful simulations with some problems,...
;                                       The idea to allow meaningful plots e.g. for all models with Quality ge 0.5.
;     Type:          'solar reference', string that characterizes the model ('Sun', 'Cepheid', 'Brown Dwarf')
;                                       main properties could be included ('2D'/'3D', 'dust', 'magnetic fields')
;     Parameters:    'High resolution', string with main parameters that characterize the model,
;                                       Usually *not* Teff, grav but e.g.
;                                       'Increased numerical viscosity', 'New open lower boundary'
;     Aim:           'New reference'    string why the model was started.
;                                       Useful while model is running before results are available.
;     Result:        'Well relaxed solar reference model', string with main result
;     Problems:      ''                 string with numerical or technical problems, e.g.
;                                       'Ended with a crash', 'Missing mean files: 03,04', 'Corrupted full file: 05'
;     Author:        'FirstName LastName', string with author name: the one that ran the model
;                                       and can be asked about it
;     Organization:  'ORG',             e.g. 'AIP', 'CIFIST', 'CRAL', 'GRAAL', 'Uppsala University'
;     Computer:      'Machine',         e.g. 'PSMN' 'CINES: Montpellier',
;     nFile:             42,            integer with number of full files, just an indicator how far the run is
;     ParModelIdent:  '_42',            string with final parameter file idendifier. The par file should be
;                                       Model + ParModelIdent + '.par'
;     mMeanAvg:         200,            integer with first snapshot in the mean file sequence,
;                                       useful for final averaging. Zero means: start at the first one.
;                                       The number refers to the element number in the seq.sav file, that contains
;                                       the collected mean file snapshots (see rhd_seqpp_collect.pro).
;     nMeanAvg:       99999,            integer with last snapshot in the mean file sequence,
;                                       useful for final averaging. A large number means: stop with the last one.
;
; TODO:
;   Include toy models (e.g. mini-suns)
;   Include models from all CO5BOLD users
;
;   Additional keywords:
;     SelectedSnapshots
;
; EXAMPLE:
;    rhdmodeldata, ModelTable
;    print, ModelTable.model
;
; MODIFICATION HISTORY:
;   2009-09-28 (B.F.) Last update
;   2009-09-30 (B.F.) Added keywords
;   2009-11-27 (B.F.) Split large array into sub arrays and merge them at the end
;   2012-02-20 (B.F.) Renamed modelplotdata_bd -> rhdmodeldata
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

; ==================================================================================================

ModelTable_SebastianWende=[ $
; --- Goettingen M dwarfs ----------------------------------------------
{ $
Model:         'd3t25g50mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution', $
Aim:           'M-dwarf Teff sequence', $
Result:        'Model converged at 2575K', $
Problems:      'Some mean files missing (only 0-16,49,50 available): the available sequence is very short.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             51, $
mMeanAvg:         172, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t25g50mm00w1hr', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution', $
Aim:           'M-dwarf Teff sequence', $
Result:        'Model converged at 2575K', $
Problems:      'Strange pattern in the upper parts.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             31, $
mMeanAvg:         220, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t25g50mm00w1hrt', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution. High output sampling rage: time resolution 2sec (full files)', $
Aim:           'M-dwarf Teff sequence', $
Result:        'Model converged at 2575K', $
Problems:      'Strange pattern in the upper parts.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:              1, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t25g50mm00w1hrcb', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution. Closed top boundary.', $
Aim:           'Change top boundary from transmitting to close due to strange pattern in the upper parts', $
Result:        'Model converged at 2575K', $
Problems:      'Short run', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:              6, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t28g50mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Some mean files missing.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             16, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t28g50mm00w1hr', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Some mean files missing.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             24, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t28g50mm00w1x2', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution. Larger box (horizontally).', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:              8, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t31g50mm00w1', $
Status:        'Finished', $
Quality:        0.3, $
Type:          '3D M-type dwarf', $
Parameters:    'Low horizontal resolution.', $
Aim:           '', $
Result:        '', $
Problems:      'Temperatures are still dropping. Velocities have been reset. Initial deep entropy different from inflow entropy. mean files 30-32 missing', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             50, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g30mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type giant', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Inflow entropy was reset.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             170, $
mMeanAvg:        1500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g30mm00w1hr', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D M-type giant', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Quite some time steps missing.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             25, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g35mm00w1', $
Status:        'Finished', $
Quality:        0.3, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        'A clean mode is visible.', $
Problems:      '', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             18, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g35mm00w1lg', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type giant', $
Parameters:    'Low resolution. Larger box (horizontally)', $
Aim:           '', $
Result:        'A clean mode is visible.', $
Problems:      '', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             36, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g35mm00w1hrlg', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D M-type giant', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             30, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g40mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type star', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Counter itime is each time reset. Low output sampling rate. Something happens from 09->10: entropy modification.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             15, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g40mm00w1hr', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D M-type star', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Counter itime is each time reset. Low output sampling rate.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             13, $
mMeanAvg:          60, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g45mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Output sampling rate changed twice during the run. Change of thermal structure around itime=560 (.055->.056).', $
Author:        'Sebastian Wende', $
Organization:  'ALTIX GWDG', $
Computer:      '', $
ParModelIdent:   '.00', $
nFile:            141, $
mMeanAvg:        1440, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g45mm00w1hr', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Very short run. Oscillations.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:  '.00', $
nFile:             5, $
mMeanAvg:         10, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Todo: d3t33g45mm00w1hr, d3t33g45mm00w1hrlg (where are the data?)

{ $
Model:         'd3t33g50mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        'Converged. Large cells.', $
Problems:      'Uneven sampling in mean files', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:  '.00', $
nFile:            75, $
mMeanAvg:        400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t33g50mm00w1hr', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Very short run', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:  '.00', $
nFile:             7, $
mMeanAvg:         40, $
nMeanAvg:      99999, $
mTimeAvg:        0.0, $
nTimeAvg:        0.0 $
}, $

{ $
Model:         'd3t33g50mm00w1hrlg', $
Status:        'Finished', $
Quality:        0.4, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution + larger box', $
Aim:           '', $
Result:        '', $
Problems:      'Oscillations. Growing mode? Dark areas in intensity maps.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:  '.00', $
nFile:            24, $
mMeanAvg:        160, $
nMeanAvg:      99999, $
mTimeAvg:        0.0, $
nTimeAvg:        0.0 $
}, $

{ $
Model:         'd3t35g50mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Growing mode', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             41, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t35g50mm00w1lg', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution. Larger box (horizontally)', $
Aim:           '', $
Result:        '', $
Problems:      'Weird mode in intensity maps. Mean file 51 missing.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             10, $
mMeanAvg:          60, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Todo: split (?) d3t35g50mm00w1hr -> d3t35g50mm00w1hrlg, d3t35g50mm00w1hrlgg (still running?)

{ $
Model:         'd3t35g50mm00w1hr', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             42, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Todo: d3t36g50mm00w1XX models (where is it?)

{ $
Model:         'd3t38g49mm00w1', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '3D M-type dwarf', $
Parameters:    'Low resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Initially inflow entropy different from deep entropy. Only the very end of sequence useful.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             46, $
mMeanAvg:         220, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t38g49mm00w1hr', $
Status:        'Finished', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'High resolution', $
Aim:           '', $
Result:        '', $
Problems:      'Some change near upper boundary from n52 on.', $
Author:        'Sebastian Wende', $
Organization:  '', $
Computer:      'ALTIX GWDG', $
ParModelIdent:   '.00', $
nFile:             10, $
mMeanAvg:          70, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_CIFIST=[ $
; --- CIFIST stars ------------------------------------------------------------
{ $
Model:         'd3t28g50mm00du4', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             41, $
mMeanAvg:          0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t28g50mm00du5', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             25, $
mMeanAvg:          0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t36g00mm00n01', $
Status:        'Finished with problems', $
Quality:        0.3, $
Type:          'Local supergiant model', $
Parameters:    'Extremely low gravity', $
Aim:           '', $
Result:        '', $
Problems:      'Experimental', $
Author:        'Hans-Guenter Ludwig', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             89, $
mMeanAvg:        400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t36g00mm00n02', $
Status:        'Finished with problems', $
Quality:        0.3, $
Type:          'Local supergiant model', $
Parameters:    'Extremely low gravity', $
Aim:           '', $
Result:        '', $
Problems:      'Experimental', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            100, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t36g00mm00n03', $
Status:        'Finished with problems', $
Quality:        0.3, $
Type:          'Local supergiant model', $
Parameters:    'Extremely low gravity', $
Aim:           '', $
Result:        '', $
Problems:      'Experimental', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             141, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t36g00mm00n04', $
Status:        'Finished with problems', $
Quality:        0.3, $
Type:          'Local supergiant model', $
Parameters:    'Extremely low gravity', $
Aim:           '', $
Result:        '', $
Problems:      'Experimental', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             33, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t36g10mm00n01', $
Status:        '', $
Quality:        0.5, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             50, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t36g10mm00n02', $
Status:        '', $
Quality:        0.5, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Arunas Kucinskas', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:          88, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t38g40mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:          24, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t38g40mm00n2', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             50, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t40g15mm00n01', $
Status:        '', $
Quality:        0.0, $
Type:          '', $
Parameters:    'Rather large volume', $
Aim:           '', $
Result:        'Lots of small cells.', $
Problems:      'Extremely short run. Not relaxed, yet. Ignore!', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            144, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t40g15mm00n02', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    'Half the horizontal size as d3t40g15mm00n01', $
Aim:           '', $
Result:        'Cells become very large.', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             46, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t40g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            120, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t45g25mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             40, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t45g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            111, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t46g32mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'mean file 75 corrupt', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             89, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g25mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Error in time sequence. Use only [m-z]*.', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            140, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g35mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '.', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            131, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g40mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             96, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Defect mean files (52,76). Not quite converged.', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             20, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g45mm00n02', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Experimental', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             64, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g45mm00n03', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             40, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t50g45mm00n04', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             56, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t55g35mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             72, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t55g40mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            111, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t55g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             82, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t59g35mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             61, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t59g40mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             46, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t59g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '. Mean file (40) is defect.', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             71, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t63g40mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            100, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t63g40mm00n02', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             44, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t63g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            100, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t65g40mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            100, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t65g45mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             80, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t65g45mm00n02', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            100, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t68g43mm00n01', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            305, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t75g40mm00n1', $
Status:        '', $
Quality:        1.0, $
Type:          '', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Hans-Guenter Ludwig', $
Organization:  'CIFIST', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            159, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_AndreaChiavassa=[ $
; --- Supergiants ---
{ $
Model:         'st35gm00n01', $
Status:        '?', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '7 M_Sun', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  'OCA', $
Computer:      '', $
ParModelIdent:  '_029', $
nFile:             29, $
mMeanAvg:          40, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm00n01_old', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '7 M_Sun', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  'OCA', $
Computer:      '', $
ParModelIdent:  '_115', $
nFile:            115, $
mMeanAvg:         700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm00n01_tweak', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '7 M_Sun', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  'OCA', $
Computer:      '', $
ParModelIdent:  '_024', $
nFile:             24, $
mMeanAvg:         120, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n13', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, non-grey greyified opacity tables', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag, Andrea Chiavassa', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_072', $
nFile:             68, $
mMeanAvg:          40, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n14c', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, radiation pressure', $
Aim:           'Produce model with radiation pressure.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag, Andrea Chiavassa', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_199', $
nFile:            199, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n38', $
Status:        'Finished', $
Quality:        0.9, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 401^3, new code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check new code version at high resolution', $
Result:        '', $
Problems:      'In additional files (28-83), there are two gaps, between files 66/67 and 79/80.', $
Author:        'Andrea Chiavassa', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_083', $
nFile:             83, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n38ext', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 501^3, new code version 2017-12', $
Aim:           'Extend the model compared to n38', $
Result:        '', $
Problems:      'file 100 defect', $
Author:        'Andrea Chiavassa', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_111', $
nFile:             28, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n41', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 315^3, rotation period: 20y', $
Aim:           'Get rotating supergiant model', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_999', $
nFile:           1000, $
mMeanAvg:        3992, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n42', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 315^3, rotation period: 10y', $
Aim:           'Get rotating supergiant model', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_255', $
nFile:             56, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n43', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 315^3, rotation period: 30y', $
Aim:           'Get rotating supergiant model', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_244', $
nFile:             45, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with reduced radius', $
Parameters:    '', $
Aim:           'Betelgeuse model with realistic Teff and logg but smaller radius', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag, Andrea Chiavassa', $
Organization:  'MPA', $
Computer:      'IBM Power 6', $
ParModelIdent:  '_029', $
nFile:             27, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g00n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with reduced radius', $
Parameters:    '', $
Aim:           'Betelgeuse model with realistic Teff and logg but smaller radius', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  'MPA', $
Computer:      'IBM Power 6', $
ParModelIdent:  '_014', $
nFile:             15, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g00n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with reduced radius', $
Parameters:    '', $
Aim:           'Betelgeuse model with realistic Teff and logg but smaller radius', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  'MPA', $
Computer:      'IBM Power 6', $
ParModelIdent:  '_012', $
nFile:             12, $
mMeanAvg:         600, $
nMeanAvg:         821, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with reduced radius', $
Parameters:    '', $
Aim:           'Betelgeuse model with realistic Teff and logg but smaller radius', $
Result:        '', $
Problems:      '', $
Author:        'Andrea Chiavassa', $
Organization:  'MPA', $
Computer:      'IBM Power 6', $
ParModelIdent:  '_071', $
nFile:             71, $
mMeanAvg:        4600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g00n05', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Global 3D high-resolution Betelgeuse model with reduced radius', $
Parameters:    '', $
Aim:           'Get high-resolution comparison model', $
Result:        '', $
Problems:      'Available mean file sequence has a gap.', $
Author:        'Andrea Chiavassa', $
Organization:  'MPA', $
Computer:      'IBM Power 6', $
ParModelIdent:  '_303', $
nFile:            191, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_WernerSchaffenberger=[ $
; --- WS models --------------------------------------------------------
{ $
Model:         'st36g45n04', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, from n019 on: damping in outer layers', $
Aim:           'Non-rotating reference model', $
Result:        '', $
Problems:      'Sampling time step changed in between.', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_24',$ 
nFile:             24, $
mMeanAvg:        2130, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n05', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, new code version 2013-03-28: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=6.0E07->2.0E07 at the end.', $
Aim:           'Non-rotating reference model', $
Result:        '', $
Problems:      'Develops global flow through core once core radius is small enough.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_225', $
nFile:            225, $
mMeanAvg:        2400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n06', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, new code version 2013-04-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=3.0E07.', $
Aim:           'Non-rotating reference model', $
Result:        '', $
Problems:      'Develops global flow through core.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_032', $
nFile:             32, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n07', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, new code version 2013-04-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=4.0E07.', $
Aim:           'Non-rotating reference model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_034', $
nFile:             34, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n08', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, new code version 2013-04-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07.', $
Aim:           'Non-rotating reference model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_130', $
nFile:            130, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n09', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.10E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07.', $
Aim:           'Slowly rotating model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n10', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'nu_rotation=0.25E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07.', $
Aim:           'Slowly rotating model', $
Result:        '', $
Problems:      'v1rms near core not yet stationary', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n11', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.10E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Slowly rotating model', $
Result:        'Looks stable and rotationally symmetric', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        3003, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n12', $
Status:        'Finished jm', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.25E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Slowly rotating model', $
Result:        'Looks stable and rotationally symmetric', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        3003, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n13', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.50E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        '', $
Problems:      'Develops deviations from rotational symmetry that appear to be stable.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_400', $
nFile:            400, $
mMeanAvg:        6003, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n14', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.75E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        '', $
Problems:      '(Still) growing fluctuations: deviations from rotational symmetry.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_400', $
nFile:            400, $
mMeanAvg:        3560, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n15', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=1.00E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        '', $
Problems:      'Deviations from rotational symmetry.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        3700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n16', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=1.25E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rapidly rotating model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        2100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n17', $
Status:        'Finished jj', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=1.50E-03, new code version 2013-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rapidly rotating model', $
Result:        '', $
Problems:      'A lot. Strong fluctuations in between. At the end, the direction of rotation reverses, finishes with a crash', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_267', $
nFile:            267, $
mMeanAvg:        1281, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n18', $
Status:        'Finished jg', $
Quality:        0.7, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.05E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Very slowly rotating model', $
Result:        'Differential sub-rotation. The meridional flow is fast and dipolar (downward in the center).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n19', $
Status:        'Finished jh', $
Quality:        0.7, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.10E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Slowly rotating model', $
Result:        'Differential sub-rotation. The meridional flow is slow and dipolar (downward in the center).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        2800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n20', $
Status:        'Finished ji', $
Quality:        0.7, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.15E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Slowly rotating model', $
Result:        'Differential sub-rotation. The meridional flow is bipolar (outward at the equator).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n21', $
Status:        'Finished jj', $
Quality:        0.7, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.20E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Slowly rotating model', $
Result:        'Differential sub-rotation. The meridional flow is bipolar (outward at the equator).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n22', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.30E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Slowly rotating model', $
Result:        'Differential sub-rotation. The meridional flow is bipolar (outward at the equator). The central region of fast rotation becomes more column-like.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n23', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.40E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        'Differential sub-rotation. The meridional flow is bipolar (outward at the equator). The central region of fast rotation becomes even more column-like. This is the fastest-rotating stable model.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        4600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n24', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.50E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        'Differential sub-rotation. The meridional flow is bipolar only in the interior (outward in the equatorial plane). There is an outer meridional plane with inward flow at the equator. The very outer atmosphere super-rotates.', $
Problems:      'Nonradial waves', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        5000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n25', $
Status:        'Finished jj', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.60E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        '', $
Problems:      'Nonradial waves', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n26', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.70E-03, new code version 2013-08-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force.', $
Aim:           'Rotating model', $
Result:        '', $
Problems:      'Nonradial waves', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_127', $
nFile:            127, $
mMeanAvg:        3600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n27', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.40E-03, new code version 2013-10-29: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force. Grid points 255^3 (n23) -> 361^3', $
Aim:           'Higher-resolution rotating model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n28', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, including centrifugal force, without dust', $
Parameters:    'nu_rotation=0.40E-03, new code version 2013-10-29: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07, no core drag force, heat_mode=core_energy1.', $
Aim:           'Rotating model', $
Result:        'The average meridional flow pattern changes from bipolar (n23) to dipolar (n28) and becomes stronger.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n29', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, new code version 2014-06-06: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=2.0E07.', $
Aim:           'Non-rotating reference model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n30', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, new code version 2014-06-06: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, large r0_core=5.0E07.', $
Aim:           'Non-rotating reference model with enforced meridional flow', $
Result:        'The meridional flow has essentially only a bipolar component.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_018', $
nFile:             18, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n31', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'Slow rotation, new code version 2014-06-06: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, small r0_core=2.0E07.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        'The meridional flow has a bipolar and a strong dipolar component.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n32', $
Status:        'Finished jj', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'Slow rotation, new code version 2014-06-06: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, large r0_core=5.0E07.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        'The meridional flow has a bipolar and a strong dipolar component.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n33', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'Slow rotation, new code version 2014-06-11: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, large r0_core=5.0E07, hdCoreDragProfile=Linear-Radial->CosSqr-Meridional.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        'The meridional flow has a dominating bipolar and a dipolar component.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n34', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'Slow rotation, new code version 2014-06-16: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, small r0_core=2.0E07, hdCoreDragProfile=CosSqr-Meridional.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:   '100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n35', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.05E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n36', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 365^3 points, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=3.0E07, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0', $
Aim:           'Non-rotating model, deeper than previous.', $
Result:        '', $
Problems:      'Developes a weak general dipolar flow.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_080', $
nFile:             80, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n37', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.10E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n38', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.15E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n39', $
Status:        'Finished jj', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.20E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n40', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-07-20: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=3.0E07, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0.', $
Aim:           'Non-rotating model, even deeper than previous.', $
Result:        '', $
Problems:      'The entropy plateau is not flat or rather: the entropy is not monotonically decreasing from the center to the surface.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_040', $
nFile:             40, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n41', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.20E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional, increased s_InFlLatGrad.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n42', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.30E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional, increased s_InFlLatGrad.', $
Aim:           'Slowly rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n43', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'nu_rotation=0.40E-03, slow rotation, new code version 2014-06-22: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=5.0E07, hdCoreDragProfile=CosSqr-Meridional, increased s_InFlLatGrad.', $
Aim:           'Rotating model with enforced meridional flow', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n44', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-08-11: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, C_hydSmallFluctFactor=0.4.', $
Aim:           'Non-rotating model. Check influence of smaller C_hydSmallFluctFactor on flatness of entropy plateau', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_030', $
nFile:             30, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n45', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-07-26: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3.', $
Aim:           'Non-rotating model, Check 3-sub-step RungeKutta2.', $
Result:        'Works.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n46', $
Status:        'Finished jj', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-07-27: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3.', $
Aim:           'Non-rotating model, Check bug fix in averaging-weight of thermodynamic variables in rhd_hyd_RoeWavealpha36.', $
Result:        'Hardly any difference to n45.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n47', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-07-27: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, strtest1(tweak)=RadialSmooth.', $
Aim:           'Non-rotating model. Test HBweno and VAweno. Test influence of c_hydsDiffVelo.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_025', $
nFile:             25, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n48', $
Status:        'Finished jf', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 479^3 points, new code version 2014-07-27: RungeKutta, N_RKIter=3, CA4, VAweno, r0_core=1.0E+08.', $
Aim:           'Non-rotating model with higher resolution. Check influence of resolution on flatness of entropy plateau. ', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_030', $
nFile:             30, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49a', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-01: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, interpolate phib_grav linearly from phic_grav.', $
Aim:           'Non-rotating model, search for reason for entropy slope: check influence of potential formula.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49b', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-01: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, switch Pred during compilation.', $
Aim:           'Non-rotating model, search for reason for entropy slope: check influence of hydrostatic-pressure correction.', $
Result:        '', $
Problems:      'End with a crash after a few time steps.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_001', $
nFile:              1, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49c', $
Status:        'Finished jm', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-04: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, CA6gMps', $
Aim:           'Non-rotating model, search for reason for entropy slope: check influence of enthalpy interpolation.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_025', $
nFile:             25, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49d', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-04: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, CA6gMps, C_sChange=0.003->0.001, C_CoreDrag=1.0->0.5 (compared to n49c).', $
Aim:           'Non-rotating model, search for reason for entropy slope: check influence of smoothness of core damping and heating zone.', $
Result:        '', $
Problems:      'Towards the end, there is a dipolar flow pattern visible.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_040', $
nFile:             40, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49e', $
Status:        'Finished ji', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-04: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, CA6gMps, EOS table with worse sampling: eos_mm00_l5.eos -> eos_mm00_l4.eos', $
Aim:           'Non-rotating model, search for reason for entropy slope: check influence of (sampling of) EOS table.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49f', $
Status:        'Finished jj', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-04: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, CA6gMps, EOS table with worse sampling: eos_mm00_l5.eos -> eos_mm00_l6.eos', $
Aim:           'Non-rotating model, search for reason for entropy slope: check influence of (type of) EOS table.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n49g', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'No rotation, 401^3 points, new code version 2014-09-04: RungeKutta, CA4, VAweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor, r0_core=1.0E+08, hdCoreDragProfile=CosSqr-Radial, C_gravmu=1.0, c_hydSmallFluctFactor=0.4, N_RKIter=3, CA6gMps, EOS table with worse sampling: eos_mm00_l5.eos -> eos_mm00_l6.eos, double precision', $
Aim:           'Non-rotating model, search for reason for entropy slope: check effect of double precision', $
Result:        '', $
Problems:      'The entropy in the deeper layers does not drop monotonically outward.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_030', $
nFile:             30, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n04rot', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'Slow rotation', $
Aim:           '', $
Result:        '', $
Problems:      'Parameters changed in between', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_23', $
nFile:             23, $
mMeanAvg:        1900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n04rot2', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'Fast rotation', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_14', $
nFile:             14, $
mMeanAvg:        1000, $
nMeanAvg:        1350, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n04rot3', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'rotation', $
Aim:           '', $
Result:        '', $
Problems:      'Series from _10 to _29 (_00 to _09 has different parameters?).', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_29', $
nFile:             20, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n04rot4', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'rotation', $
Aim:           '', $
Result:        '', $
Problems:      'Series from _10 to _29.', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_29', $
nFile:             20, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n04rot5', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model with rotation, without dust', $
Parameters:    'rotation', $
Aim:           '', $
Result:        '', $
Problems:      'Series from _10 to _29.', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_29', $
nFile:             20, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st36g45n04hll', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D scaled M-dwarf model without rotation, without dust', $
Parameters:    'HLL solver, no rotation, reconstruction: PP', $
Aim:           '', $
Result:        'Works (in principal) and shows convection.', $
Problems:      'Large velocities in the core, ripples in the entropy', $
Author:        'Werner Schaffenberger', $
Organization:  'Uni Exeter', $
Computer:      'Zen', $
ParModelIdent:  '_13', $
nFile:              4, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_MatthiasSteffen=[ $
; --- MS models --------------------------------------------------------
{ $
Model:         'gt57g44n67_3D', $
Status:        'Finished', $
Quality:        0.5, $
Type:          'Early solar model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '', $
nFile:           104, $
mMeanAvg:          0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44n58', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D solar reference model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            31, $
mMeanAvg:          0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44n59', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D solar reference model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:            31, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44n59new3', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D solar model', $
Parameters:    'Roe solver, CTU, ...: current settings (2012.11.05)', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '.60', $
nFile:             20, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44n59x16', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'High-resolution 3D solar model', $
Parameters:    '', $
Aim:           'High-resolution model', $
Result:        '', $
Problems:      'Running very slowly', $
Author:        'Thomas Strauss, Matthias Steffen', $
Organization:  'OAC', $
Computer:      'Cineca', $
ParModelIdent:  '.181', $
nFile:            181, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44b000n59', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D solar model with HLL solver, no magnetic fields', $
Parameters:    'FRweno, Hancock, unsplit', $
Aim:           'Test the new code version', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '.60', $
nFile:          20, $
mMeanAvg:        200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44b010n59', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D solar model with HLL solver, no magnetic fields', $
Parameters:    'FRweno, Hancock, unsplit', $
Aim:           'Test the new code version', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '.60', $
nFile:          20, $
mMeanAvg:        200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3gt57g44n94', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'sun', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '.00', $
nFile:             91, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt57g44n004_2D_RHD', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'sun', $
Parameters:    'Roe solver', $
Aim:           'Reference run with classical HD solver', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt57g44n004_2D_B00', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'sun', $
Parameters:    'HLLMHD solver, B=0', $
Aim:           'Reference run without magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt57g44n004_2D_B10', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'sun', $
Parameters:    'HLLMHD solver, B=10', $
Aim:           'Weak magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_18', $
nFile:             18, $
mMeanAvg:         560, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt57g44n004_2D_B20', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'sun', $
Parameters:    'HLLMHD solver, B=20', $
Aim:           'Weak magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_25', $
nFile:             25, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt57g44n004_2D_B40', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'sun', $
Parameters:    'HLLMHD solver, B=40', $
Aim:           'Medium magnetic field', $
Result:        '', $
Problems:      'mean file data corrupted', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_41', $
nFile:             41, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt57g44n004_2D_B80', $
Status:        'Finished', $
Quality:        0.4, $
Type:          'sun', $
Parameters:    'HLLMHD solver, B=80', $
Aim:           '"Strongest" magnetic field', $
Result:        '', $
Problems:      'mean file data corrupted', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_110', $
nFile:            110, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t116g80wd2', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey white dwarf', $
Parameters:    'Grey opacities, Teff=11600K', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t116g80wd4', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-rey white dwarf', $
Parameters:    'Non-grey opacities, Teff=11600K', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_399', $
nFile:             99, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t120g80wd2', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey white dwarf', $
Parameters:    'Grey opacities, Teff=12000K', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t120g80wd4', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-rey white dwarf', $
Parameters:    'Non-grey opacities, Teff=12000K', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t124g80wd2', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey white dwarf', $
Parameters:    'Grey opacities, Teff=12400K', $
Aim:           '', $
Result:        'Decaying oscillations in mean quantities', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        1800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t124g80wd4', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-rey white dwarf', $
Parameters:    'Non-grey opacities, Teff=12400K', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t128g80wd2', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey white dwarf', $
Parameters:    'Grey opacities, Teff=12800K', $
Aim:           '', $
Result:        'Decaying oscillations in mean quantities', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        2200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t128g80wd4', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey white dwarf', $
Parameters:    'Non-grey opacities, Teff=12800K', $
Aim:           '', $
Result:        'Relaxation grey -> non-grey in about 2 sec.', $
Problems:      'mean files: gap between n027 and n028; n029 and n030 are identical.', $
Author:        'Matthias Steffen', $
Organization:  'AIP', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_OskarSteiner=[ $
; --- Oskar Steiner's models --------------------------------------------------------
{ $
Model:         'd3t32g45', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D M dwarf', $
Parameters:    'HLLMHD solver, open lower and upper boundaries.', $
Aim:           '', $
Result:        '', $
Problems:      'Weird waves visible in intensity plots. First mean file is corrupt.', $
Author:        'Oskar Steiner', $
Organization:  'KIS', $
Computer:      '', $
ParModelIdent:  '_005', $
nFile:              5, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_PierEmmanuelTremblay=[ $
; --- Pier-Emmanuel's models --------------------------------------------------------
{ $
Model:         'd3t040g80wd08_RungeKutta2', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Cool non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'Extend white dwarf grid to low temperatures.', $
Result:        '', $
Problems:      'Old code version. Incompatible parameters. Model blows off.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'STScI', $
Computer:      '', $
ParModelIdent: '', $
nFile:              1, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t040g80wd08e', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Cool non-grey 3D white dwarf', $
Parameters:    'RungeKutta2', $
Aim:           'Extend white dwarf grid to low temperatures.', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'STScI', $
Computer:      '', $
ParModelIdent: '_c', $
nFile:              3, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t040g80wd08f', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Cool non-grey 3D white dwarf', $
Parameters:    'CTU', $
Aim:           'Extend white dwarf grid to low temperatures.', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'STScI', $
Computer:      '', $
ParModelIdent: '', $
nFile:              1, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t040g80wd08g', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Cool non-grey 3D white dwarf', $
Parameters:    'RungeKutta2', $
Aim:           'Extend white dwarf grid to low temperatures.', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'STScI', $
Computer:      '', $
ParModelIdent: '_c', $
nFile:              3, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t060g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'After the first file (_025) the grid was changed', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t070g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'After the first file (_025) the grid was changed', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t080g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'After the first file (_025) the grid was changed', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t090g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'After the first file (_025) the grid was changed', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t095g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'After the first two files (_025,_050) the grid was changed', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         260, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t100g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Several resets, only the sequence in the last file is continuous.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              5, $
mMeanAvg:         520, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t105g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Several resets, only the sequence in the last file is continuous.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              5, $
mMeanAvg:         420, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t110g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Some files are missing, only the sequence in the last file is continuous.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              3, $
mMeanAvg:         180, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t115g80wd08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Several resets, only the sequence in the last file is continuous.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              5, $
mMeanAvg:         320, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t120g80wd08bin8', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t120g80wd08bin11', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t125g80wd08bin8', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Several resets, missing files, only the sequence in the last file is continuous.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              6, $
mMeanAvg:         440, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t125g80wd08bin11', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t130g80wd08bin8', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Several resets, only the sequence in the last file is continuous.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              5, $
mMeanAvg:         410, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t130g80wd08bin11', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D white dwarf', $
Parameters:    '', $
Aim:           'White dwarf grid', $
Result:        '', $
Problems:      'Parameter files with UIO format bugs.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  'LSW Heidelberg', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              3, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t140g80db02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D DB white dwarf', $
Parameters:    '', $
Aim:           'DB white dwarf grid', $
Result:        '', $
Problems:      'The grid changes during the run: only the last 4 files are used. The structure in the upper atmosphere is not well relaxed: the region with flat entropy is likely to expand into even higher layers.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              4, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t180g80db02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D DB white dwarf', $
Parameters:    '', $
Aim:           'DB white dwarf grid', $
Result:        '', $
Problems:      'The grid changes during the run: only the last 2 files are used.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_125', $
nFile:              2, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t220g80db02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D DB white dwarf', $
Parameters:    '', $
Aim:           'DB white dwarf grid', $
Result:        '', $
Problems:      '', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_100', $
nFile:              5, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t340g80db02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Non-grey 3D DB white dwarf', $
Parameters:    '', $
Aim:           'DB white dwarf grid', $
Result:        '', $
Problems:      'The grid changes during the run: only the last file is used.', $
Author:        'Pier-Emmanuel Tremblay', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_050', $
nFile:              1, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]

; ==================================================================================================

ModelTable_SvenWedemeyerBoehm=[ $
; --- Sven's models --------------------------------------------------------
{ $
Model:         'sunmhd3D100', $
Status:        'Finished', $
Quality:        0.3, $
Type:          '3D solar MHD model', $
Parameters:    '200G average vertical magnetic field', $
Aim:           '', $
Result:        '', $
Problems:      'Crash', $
Author:        'Seven Wedemeyer-Boehm', $
Organization:  'ITA University of Oslo', $
Computer:      '', $
ParModelIdent:  '_n018', $
nFile:             18, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'sunmhd3D101', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D solar MHD model', $
Parameters:    '50G average vertical magnetic field, extended', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Seven Wedemeyer-Boehm', $
Organization:  'ITA University of Oslo', $
Computer:      '', $
ParModelIdent:  '_n007', $
nFile:             7, $
mMeanAvg:          0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_AstridWachter=[ $

{ $
Model:         'st28gm06n09e', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dusty AGB star model without radiation pressure', $
Parameters:    'No radiation pressure', $
Aim:           'Comparison run without radiation pressure', $
Result:        '', $
Problems:      '', $
Author:        'Astrid Wachter', $
Organization:  'UAO', $
Computer:      'Mac Darkstar', $
ParModelIdent:   '_28', $
nFile:             28, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n09t4', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dusty AGB star model with radiation pressure', $
Parameters:    'Radiation pressure', $
Aim:           'Run with radiation pressure', $
Result:        '', $
Problems:      '', $
Author:        'Astrid Wachter', $
Organization:  'UAO', $
Computer:      'Mac Darkstar', $
ParModelIdent:   '_55', $
nFile:             55, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_SusanneHoefner=[ $

{ $
Model:         'st28gm06n10a', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'AGB star model without radiation pressure', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Susanne Hoefner', $
Organization:  'UAO', $
Computer:      'Mac Octarine', $
ParModelIdent:   '_03', $
nFile:              3, $
mMeanAvg:          10, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n10d', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'AGB star model without radiation pressure', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Susanne Hoefner', $
Organization:  'UAO', $
Computer:      'Mac Octarine', $
ParModelIdent:   '_02', $
nFile:              2, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n11', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'AGB star model without radiation pressure', $
Parameters:    'New forsterite dust scheme, without dust opacities or radiation pressure, two dust variants', $
Aim:           'Testing of new dust scheme', $
Result:        '', $
Problems:      '', $
Author:        'Susanne Hoefner', $
Organization:  'UAO', $
Computer:      'Mac Octarine', $
ParModelIdent:   '_01', $
nFile:              1, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_FranceAllard=[ $
{ $
Model:         'mt13g50mm00n19', $
Status:        'Finished j6', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf (testing only)', $
Parameters:    'New code version 2013-08-26: double precision', $
Aim:           'Test setup', $
Result:        '', $
Problems:      '', $
Author:        'France Allard', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd2t14g50mm00n01', $
Status:        'Finished j6', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2015-06-15.', $
Aim:           'Get a 2D model with a modern code version and setup.', $
Result:        '', $
Problems:      '', $
Author:        'France Allard', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]


; ==================================================================================================

ModelTable_BerndFreytag=[ $
; --- A-type stars ------------------------------------------------------------
{ $
Model:         'at70g43n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D F-type star', $
Parameters:    '', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_13', $
nFile:             13, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at71g43n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D F-type star', $
Parameters:    '', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at73g43n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D A-type star', $
Parameters:    '', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        2500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at75g44n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D A-type star', $
Parameters:    '', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at77g44n01', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D A-type star', $
Parameters:    '', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at77g44n02', $
Status:        'Finished', $
Quality:        0.6, $
Type:          '2D A-type star', $
Parameters:    'Higher resolution than at77g44n01', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at77g44n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D A-type star', $
Parameters:    'Higher resolution than at77g44n02', $
Aim:           'Continue A-type star sequence towards lower Teff', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at80g40n20', $
Status:        'Finished', $
Quality:        0.3, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Extremely short run', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_04', $
nFile:              4, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at80g40n21', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_36', $
nFile:             36, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at80g40n22', $
Status:        'Finished? Check!', $
Quality:        0.8, $
Type:          '3D A-type star, non-grey', $
Parameters:    'Non-grey', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_24', $
nFile:             24, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at80g44n10_3D', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Matthias Steffen', $
Organization:  '', $
Computer:      '', $
ParModelIdent:   'bz', $
nFile:             52, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at85g40n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Weird features in intensity images', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_07', $
nFile:              7, $
mMeanAvg:         150, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at85g40n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Output of mean diffusive energy flux not correct.', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at85g44n13', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        'Transition 2D->3D', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_24', $
nFile:             24, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at85g44n14', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_12', $
nFile:             12, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at85g44n15', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Short run. File 01.mean is corrupt', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_04', $
nFile:              4, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'at85g44n16', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D A-type star', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Files 07.mean, 11.mean are corrupt.', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Cepheids ----------------------------------------------------------------
{ $
Model:         'gt50g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:        4500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt51g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_44', $
nFile:             44, $
mMeanAvg:        5000, $
nMeanAvg:        9999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt51g20n02', $
Status:        'Finished jw', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check the new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_50', $
nFile:             50, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt52g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_18', $
nFile:             18, $
mMeanAvg:        3600, $
nMeanAvg:        5299, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt52g20n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:         200, $
nMeanAvg:        1099, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt52g20n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:         200, $
nMeanAvg:        1599, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt53g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_13', $
nFile:             13, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt53g20n02', $
Status:        'Finished j9', $
Quality:        0.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check the new code version', $
Result:        '', $
Problems:      'Radiative flux increase and atmospheric expansion towards the end.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:        3000, $
nMeanAvg:        6499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt54g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:         800, $
nMeanAvg:        1499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt54g20n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt54g20n03', $
Status:        'Finished jy', $
Quality:        0.7, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check the new code version', $
Result:        '', $
Problems:      'After running quite some time the simulation finished with a crash.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_43', $
nFile:             43, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt54g20n04', $
Status:        'Finished ju', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-11-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Continue the previous run', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_70', $
nFile:             70, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt54g20n05', $
Status:        'Finished jt', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-12-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, kappamin=0.2', $
Aim:           'Check the influence of klipping the opacities with kappamin=0.2.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_80', $
nFile:             80, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt55g20n01', $
Status:        'Finished jv', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-11-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Get an intermediate-temperature version', $
Result:        '', $
Problems:      'Teff increases at time step around 2900 from 5500K to 5750K.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_70', $
nFile:             70, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt56g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        'There are growing more-or-less plane-parallel oscillations.', $
Problems:      'The depth of the convection zone and the emitted fluxes changes over time.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_14', $
nFile:             14, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt56g20n02', $
Status:        'Finished j8', $
Quality:        0.5, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check the new code version', $
Result:        'There are growing more-or-less plane-parallel oscillations.', $
Problems:      'The model expands, the convective flux goes down. The stellar surface comes close to the top of the box.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_50', $
nFile:             50, $
mMeanAvg:        2400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt56g20n03', $
Status:        'Finished jw', $
Quality:        0.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-10-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps; added 40 points on top of n02', $
Aim:           'Produce vertically extended model', $
Result:        'Plane-parallel pulsations with more or less stable amplitude.', $
Problems:      'At maximum radius, the bottom of the atmosphere comes close to the top of the box.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_50', $
nFile:             50, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt56g20n04', $
Status:        'Finished jr', $
Quality:        0.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-10-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps; added another 40 points on top of n03', $
Aim:           'Produce vertically further extended model', $
Result:        'Pulsations', $
Problems:      'At timestep around 2700, Teff drops a bit to values around 5600K and from then on the pulsation amplitude seems to decline.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_99', $
nFile:             99, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt58g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt58g20n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_07', $
nFile:              4, $
mMeanAvg:         110, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt58g20n03', $
Status:        'Finished jz', $
Quality:        0.8, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check the new code version', $
Result:        'Growing oscillations', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_50', $
nFile:             50, $
mMeanAvg:        1000, $
nMeanAvg:        2100, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt58g20n04', $
Status:        'Finished jz', $
Quality:        0.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-10-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps; added 40 points on top of n03', $
Aim:           'Produce vertically extended model', $
Result:        'Growing oscillations.', $
Problems:      'At maximum radius, the bottom of the atmosphere comes close to the top of the box.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_50', $
nFile:             50, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt58g20n05', $
Status:        'Finished js', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-10-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps; added another 40 points on top of n04', $
Aim:           'Produce vertically further extended model', $
Result:        'Pulsations with amplitudes of plus/minux 45 km/s. Seems relaxed.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_60', $
nFile:             60, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt60g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:         210, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt60g20n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'unsplit hydro', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_08', $
nFile:              8, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt60g20n03', $
Status:        'Finished jx', $
Quality:        0.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check the new code version', $
Result:        'Slowly growing oscillations.', $
Problems:      'The oscillation amplitude becomes so large that the stellar surface hits to top of the box.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:   '_99', $
nFile:             99, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt60g20n04', $
Status:        'Finished jy', $
Quality:        0.0, $
Type:          '2D Cepheid', $
Parameters:    'New code version 2013-10-29: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps; added 20 points on top of gt60g20n03_60.end', $
Aim:           'Produce vertically extended model', $
Result:        'Slowly growing oscillations.', $
Problems:      'Crash at the end (FRweno2 instead of FRweno should solve that). The oscillation amplitude becomes so large that the stellar surface hits to top of the box.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_60', $
nFile:            60, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt62g20n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:         300, $
nMeanAvg:         379, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'gt62g20n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D Cepheid', $
Parameters:    'unsplit hydro', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_09', $
nFile:              9, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Brown dwarfs and M dwarfs -----------------------------------------------
{ $
Model:         'mt07g50mm00n01', $
Status:        'Finished with problems', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Get very cool model', $
Result:        'Description of lower boundary heating region has to be updated for cool models.', $
Problems:      'Warm updrafts start from lower boundary buffer layers.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_13', $
nFile:             13, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt07g50mm00n02', $
Status:        'Finished j0', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code versions 2012-08-31..11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0', $
Aim:           'Get very cool model. Check new code version', $
Result:        'Works well. Initially two cloud convection layers that late merge. No large-scale mode?', $
Problems:      'The effective temperature is far too low. The temperatur limiter hat to be applied.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt07g50mm00n03', $
Status:        'Finished j0', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code versions 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0, reduced viscosity', $
Aim:           'Check influence of viscosity', $
Result:        '', $
Problems:      'Not well converged: still expanding, dust clouds extends much too high up.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:       16000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt08g50mm00n01', $
Status:        'Finished with problems', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Very cool model', $
Result:        'Description of lower boundary heating region has to be updated for cool models.', $
Problems:      'Warm updrafts start from lower boundary buffer layers.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt08g50mm00n02', $
Status:        'Finished j1', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      'Dust cloud layer very high: reaches the top of the box', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_30', $
nFile:             30, $
mMeanAvg:       11000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt08g50mm00n03', $
Status:        'Finished j1', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'n02+38 grid points. New code version 2013-03: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0', $
Aim:           'Check new code version. Get extended model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_25', $
nFile:             25, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g35mm00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower gravity. Incident radiation. Single precision.', $
Aim:           '', $
Result:        'No convection: double precision is needed', $
Problems:      'No convection', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_13', $
nFile:             13, $
mMeanAvg:        5000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n01', $
Status:        'Finished', $
Quality:        0.4, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Very cool model', $
Result:        'Worked. Top boundary possibly to low.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_21', $
nFile:             21, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added points at top.', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:       16000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation (33%)', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:       16000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation (100%)', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:       15000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n05', $
Status:        'Finished', $
Quality:        0.8, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation (300%)', $
Aim:           'Incident radiation', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:       16000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n06', $
Status:        'Finished', $
Quality:        0.8, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation', $
Aim:           'Incident radiation', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:       16000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n07a', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '', $
Parameters:    '', $
Aim:           'Test of new code version 2009.01.05', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n07b', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '', $
Parameters:    '', $
Aim:           'Test of new code version 2009.01.08', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added points at top: 20', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n09', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Higher sampling rate', $
Aim:           'Get model useful for Fourier analysis.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n10', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Modifications of settings for lower boundary and viscosity.', $
Aim:           '', $
Result:        'Reduced p-mode amplitudes.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n11', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Modifications of settings for lower boundary and viscosity. Less extended than n10.', $
Aim:           '', $
Result:        'Only 1-15 (-7499, -27500.0) used for paper.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        4000, $
nMeanAvg:        7499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n12', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Linear viscosity on. Modifications of settings for lower boundary: heat_mode=2', $
Aim:           'Get smoother entropy profile near lower boundary', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n13', $
Status:        'Finished', $
Quality:        0.4, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Linear viscosity on. Modifications of settings for lower boundary: heat_mode=2. hdsplit=123.', $
Aim:           'Check difference: hdsplit=123 versus unsplit', $
Result:        'Agreement is quite OK except for the velocity in the very top layers.', $
Problems:      'Weird velocity beat with period of about 17000 sec in the layers near the top that do not affect the structure very much.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, CAO', $
Computer:      'PSMN', $
ParModelIdent:  '_12', $
nFile:             12, $
mMeanAvg:        1700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt09g50mm00n14', $
Status:        'Finished j2', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_30', $
nFile:             30, $
mMeanAvg:       10000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added points at top.', $
Aim:           'Check if location of upper boundary matters.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller time step.', $
Aim:           'Suppress spurious p-modes.', $
Result:        'Works: smaller time step reduces p-modes. Use only n01-n19 for paper', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_19', $
nFile:             19, $
mMeanAvg:        4000, $
nMeanAvg:        9499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n04', $
Status:        'Finished j3', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-03-01: CTU, CA1, FRweno, equidistant vertical grid.', $
Aim:           'Get 1000K model with new code version and settings.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n05', $
Status:        'Finished j6', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-03-01: CTU, CA1, FRweno, C_RecCont, reduced VisP2P, equidistant vertical grid.', $
Aim:           'Check if non-zero C_RecCont and reduced VisP2P matter compared to n04.', $
Result:        'Maximum of vertical convective velocities lies much further down.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n06', $
Status:        'Finished j8', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-03-01: RungeKutta2, CA1, FRweno, equidistant vertical grid.', $
Aim:           'Check if RungeKutta2 matters compared to n04.', $
Result:        'RungeKutta2 gives slightly larger velocities than CTU (run n04).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n07', $
Status:        'Finished j0', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-04-03: CTU, CA4, FRweno, C_hydsDiffVelo, equidistant vertical grid.', $
Aim:           'Check influence of new energy diffusion down an entropy gradient.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n08', $
Status:        'Finished j1', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-04-04: CTU, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo, equidistant vertical grid.', $
Aim:           'Check influence of new energy diffusion down an entropy step and momentum diffusion down a velocity step.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n09', $
Status:        'Finished j2', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-04-04: CTU, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo (smaller than n08), equidistant vertical grid.', $
Aim:           'Check influence of new energy diffusion down an entropy step and momentum diffusion down a velocity step.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n10', $
Status:        'Finished j4', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version 2012-04-16: call new routine rhd_hyd_FluxState even for dust advection, accidentally: no tweak-damping at top', $
Aim:           'Check implementation of new reconstruction and flux-computation routines.', $
Result:        'Quite some change in vertical and increase in horizontal atmospheric velocity (time scale of hours): switching-off tweak-damping has an effect.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n11', $
Status:        'Finished j3', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-03-01: CTU, CA1, FRweno, equidistant vertical grid: same as n04, just no tweak-damping at top', $
Aim:           'Check influence of tweak damping at top.', $
Result:        'Quite some change in vertical and increase in horizontal atmospheric velocity (time scale of hours): switching-off tweak-damping has an effect.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n12', $
Status:        'Finished j5', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-05-08: RungeKutta2, CA4, FRweno, ', $
Aim:           'Check new low-Mach-number-modified Roe solver.', $
Result:        'The low-Mach-number reduction should not be applied to the pressure terms.', $
Problems:      'The pressure has anoying small-scale small-amplitude disturbances. Entropy and velocity look almost OK.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_02', $
nFile:              2, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n13', $
Status:        'Finished j0', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-05-09: RungeKutta2, CA4, FRweno, ', $
Aim:           'Check new low-Mach-number-modified Roe solver.', $
Result:        'The flow in the low-Mach-number (<0.01) convection zone is now pretty turbulent. The entropy field is "not purely convective".', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n14', $
Status:        'Finished j1', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-05-09: RungeKutta2, CA4, FRweno, ', $
Aim:           'Check new low-Mach-number-modified Roe solver. Compare parameter settings to n13', $
Result:        'Very similar to mt10g50mm00n13. The increased bottom drag force has a small but visible impact, the change viscosity parameters not.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n15', $
Status:        'Finished j0', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-06-15: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1s', $
Aim:           'Check new enthalpy averaging in Roe solver.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n16', $
Status:        'Finished j1', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-06-15: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1s, hdEntropyWaveMode=Entropy1', $
Aim:           'Check new enthalpy averaging and entropy mode in Roe solver.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n17', $
Status:        'Finished j2', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, (new) code version 2012-06-15: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1s, hdEntropyWaveMode=Entropy3', $
Aim:           'Check new enthalpy averaging and entropy mode in Roe solver.', $
Result:        'Works.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt10g50mm00n18', $
Status:        'Finished j3', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g40mm00n01', $
Status:        'Finished', $
Quality:        0.6, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower gravity, 300 points horizontally, single precision.', $
Aim:           'Temperature sequence at logg=4', $
Result:        '', $
Problems:      'Model box does not extend high enough: top of clouds close to top boundary.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_09', $
nFile:              9, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g40mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added 30 points at top to mt11g40mm00n01, single precision.', $
Aim:           'Temperature sequence at logg=4', $
Result:        'Now there is enough space to cover the top of the clouds properly', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_25', $
nFile:             25, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g50mm00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'No convection.', $
Problems:      'No convection.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_13', $
nFile:             13, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g50mm00n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Initially less dust.', $
Result:        'No convection.', $
Problems:      'No convection.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Double precision.', $
Aim:           '', $
Result:        'Convection.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_35', $
nFile:             35, $
mMeanAvg:       10000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g50mm00n04', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Updated energy update.', $
Result:        'Updated energy update makes a difference. Still no convection.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_12', $
nFile:             12, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g50mm00n05', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller time step.', $
Aim:           'Fight pressure waves.', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt11g50mm00n06', $
Status:        'Finished j4', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Go to lower Teff', $
Result:        '', $
Problems:      'None', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:       11000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Larger grains.', $
Aim:           '', $
Result:        'The evil mode appears -- no crash, yet.', $
Problems:      'Dust opacities with bug.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       11999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Dustbin scheme.', $
Aim:           '', $
Result:        'Finished.', $
Problems:      'Seed number is not conserved by scheme.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_21', $
nFile:             21, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation (5%).', $
Aim:           'Incident radiation', $
Result:        'Finished', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_17', $
nFile:             17, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n05', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation (25%).', $
Aim:           'Incident radiation', $
Result:        'Finished', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_17', $
nFile:             17, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n06', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Irridiated Brown Dwarf', $
Parameters:    'Incident radiation (100%).', $
Aim:           'Incident radiation', $
Result:        'Finished', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_17', $
nFile:             17, $
mMeanAvg:       10000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n07', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added 44 points at top.', $
Aim:           '', $
Result:        'Finished', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New dust opacity formula: dustopamode=dualslope01.', $
Aim:           '', $
Result:        'Finished', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_12', $
nFile:             12, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n09', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Modified dustbin scheme.', $
Aim:           '', $
Result:        'Long run. Dust clouds become thin.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_99', $
nFile:             99, $
mMeanAvg:       40000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n10', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller time step.', $
Aim:           'Fight pressure waves.', $
Result:        'Only 01-18 (-17999) used for paper.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       17999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt12g50mm00n11', $
Status:        'Finished j5', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0, double precision', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g35mm00n01', $
Status:        'Finished.', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower gravity, 300 points horizontally.', $
Aim:           'Lower gravity model in gravity sequence', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower gravity, 300 points horizontally.', $
Aim:           'Lower gravity model in gravity sequence', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g40mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New buffer layers at lower boundary: heat_mode=bottom_entropy2; smaller Courant number', $
Aim:           'Check influence of lower boundary', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g45mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower gravity, 300 points horizontally.', $
Aim:           'Lower gravity model in gravity sequence', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:       16000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'No problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Double-precision', $
Aim:           'Compare single precision (n01) and double precision (n02).', $
Result:        'No problems. Hardly any difference to the single-precision run.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Decreased Courant number, added linear viscosity.', $
Aim:           'Suppress spurious p-modes.', $
Result:        'The decreased time step and partly the increased viscosity significantly reduce p-modes.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smoother damping at upper and lower boundary.', $
Aim:           'Test new parameters for upper and lower boundary.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n05', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x270 -> 1000x400', $
Aim:           'Check influence of higher resolution, particularly on overshoot scale height', $
Result:        'Overshoot scale height becomes smaller', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n06', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 1000x400, reconstruction: VanLeer -> MinMod', $
Aim:           'Check influence of reconstruction', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n07', $
Status:        'Finished j0', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x270, new code version: CTU, FRweno', $
Aim:           'Check if CTU and FRweno work.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n08', $
Status:        'Finished j1', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x270, new code version: CTU, FRweno, C_RecCont1,2', $
Aim:           'Check if C_RecCont1,2 work.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n09', $
Status:        'Finished j0', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version: CTU, CA1, FRweno, C_RecCont', $
Aim:           'Test hdTransVeloMode: CA1 versus Normal', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n10', $
Status:        'Finished j1', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version: CTU, CA1, FRweno, C_RecCont; equidistant vertical grid; initially zero velocities', $
Aim:           'Check if equdistant vertical grid and restart from tiny random fluctuations have an impact.', $
Result:        'Time-pattern of horizontal velocity changes over time. Large horizontal velocities towards the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n11', $
Status:        'Finished j2', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version: RK2/unsplit, CA1, FRweno, C_RecCont; equidistant vertical grid; initially zero velocities', $
Aim:           'Test Runge-Kutta scheme.', $
Result:        'Accuracy problems. Gravity waves still there.', $
Problems:      'Likely numerical accuracy errors: convection does not reach bottom, entropy in convection zone increases.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n12', $
Status:        'Finished j3', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version: CTU, CA1, FRweno, C_RecCont; equidistant vertical grid; initially zero velocities', $
Aim:           'Test influence of visP2P settings compared to n10.', $
Result:        'n10: larger photospheric horizontal velocity (v1); n12: slightly larger convective and slightly smaller photospheric velocities (v3rms).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        5000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n13', $
Status:        'Finished j4', $
Quality:        0.5, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x300, new code version: CTU, CA1, FRweno, C_RecCont; equidistant vertical grid; initially zero velocities; 31 points added at top', $
Aim:           'Test influence of position of upper boundary on mode amplitudes.', $
Result:        'The convective velocities are very similar to the ones in n12. The photospheric vertical rms velocities are smaller. That seems to be related to the large average horizontal velocities, there.', $
Problems:      'Large horizontal velocities in the photosphere.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n14', $
Status:        'Finished j0', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x330, new code version: CTU, CA1, FRweno, C_RecCont; 30 points added at top', $
Aim:           'Test influence of position of upper boundary on mode amplitudes.', $
Result:        '', $
Problems:      'The time step becomes small occasionally, which increases noise (and apparently viscosity) due to round-off errors.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n15', $
Status:        'Finished j1', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version 2012-02-09: CTU, CA1, FRweno, C_RecCont; equidistant vertical grid; initially zero velocities', $
Aim:           'Test influence of second-order gravity terms compared to n12.', $
Result:        'The photospheric wave amplitudes are very similar. Convective velocities are similar at the top and are slightly smaller in the deeper layers (by 0.1 dex). Well-defined maximum of convective velocities -- unlike n12.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n16', $
Status:        'Finished j2', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version 2012-02-09: CTU, CA1, PP, equidistant vertical grid; initially zero velocities', $
Aim:           'Test influence of HD reconstruction: PP vs. FRweno compared to n15 and n17.', $
Result:        'Results very similar to n15 and n17. Convective velocities between those of n15 and n17. Wave velocities slightly larger.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n17', $
Status:        'Finished j3', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x268, new code version 2012-02-16: CTU, CA1, FRweno, eqiudistant vertical grid; initially zero velocities', $
Aim:           'Test influence of HD reconstruction: FRweno without extra continuation. Compare to n15.', $
Result:        'Convective velocities slightly smaller than those of n15.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g50mm00n18', $
Status:        'Finished j6', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New code version 2012-11-05: RungeKutta2, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps, C_hydLowMachcsFactor=2.0, double precision', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g55mm00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Higher gravity, 300 points horizontally, single precision', $
Aim:           'Higher gravity model in gravity sequence', $
Result:        'At this Teff/logg single precision suppresses convection.', $
Problems:      'No convection', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt13g55mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Higher gravity, 300 points horizontally, double precision', $
Aim:           'Higher gravity model in gravity sequence', $
Result:        'With double precision convection behaves normally.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:       15000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt14g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Strong horizontal flows in the atmosphere, changing direction with time.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt14g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Strong horizontal flows in the atmosphere, changing direction with time, shorter period than mt14g50mm00n01.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_17', $
nFile:             17, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt14g50mm00n03', $
Status:        'Finished jm', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Get 3D model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Low-gravity model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g40mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'New buffer layers at lower boundary: heat_mode=bottom_entropy2', $
Aim:           'Check if new lower boundary works', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Strong horizontal flows in the atmosphere.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        5000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end. No crash', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        4000, $
nMeanAvg:        7999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller time step.', $
Aim:           '', $
Result:        'Finished without problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower horizontal resolution (400->300).', $
Aim:           '', $
Result:        'No problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n05', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Dustbin scheme.', $
Aim:           '', $
Result:        'Finished without obvious problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n06', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    '3D', $
Aim:           'First 3D model of Brown Dwarf', $
Result:        'Small difference to 2D model', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:             150, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n07', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added 30 points at top.', $
Aim:           '', $
Result:        'Similar vertical velocity profile as shallower model n03.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n08', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Modified dustbin scheme.', $
Aim:           '', $
Result:        'Finished with a crash. Dust settles mostly.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_45', $
nFile:             45, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n09', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower horizontal resolution and higher sampling rate', $
Aim:           '', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n10', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D brown dwarf with rotation', $
Parameters:    '2 hours rotation period', $
Aim:           'Check implementation of local rotation', $
Result:        '', $
Problems:      'Horizontal flow in x1 and x2 direction indicates bug: centrifugal force not switched off.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_035', $
nFile:              35, $
mMeanAvg:          500, $
nMeanAvg:        99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n11', $
Status:        'Finished j4', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x280, new code version 2012-02-16: CTU, CA1, FRweno, equidistant vertical grid.', $
Aim:           'Get 1500K model with new code version and settings.', $
Result:        '', $
Problems:      'Strong horizontal flow in the photosphere.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n12', $
Status:        'Finished j4', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x280, new code version 2012-02-16: CTU, CA1, FRweno, equidistant vertical grid, strong damping of horizontal velocities', $
Aim:           'Try to get rid of dominating horizontal velocity field in the photosphere.', $
Result:        '', $
Problems:      'The horizontal velocity field returns (with the other sign) after an initial damping period.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n13', $
Status:        'Finished jh', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2014-11-27: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of new code version (Runge-Kutta, reconstruction, C_gravmu).', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        2800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n14', $
Status:        'Finished ji', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Increase the horizontal resolution and change the vertical grid as well as the vertical extension to represent overshoot layers and gravity waves better.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n15', $
Status:        'Finished jj', $
Quality:        0.9, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of finer grid (factor 0.7).', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1040, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n16', $
Status:        'Finished ji', $
Quality:        0.4, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of upper boundary: increased vertical extension, removed horizontal drag force.', $
Result:        '', $
Problems:      'The large fluctuations and low temperatures at the top lead to small time steps. The run was stopped after a few jobs. It is not well relaxed.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_010', $
nFile:             10, $
mMeanAvg:         270, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt15g50mm00n17', $
Status:        'Finished ji', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of upper boundary: increased vertical extension, removed horizontal drag force.', $
Result:        'The horizontal velocity fields below the top adjust on a very short time scale to the new boundary conditions. The period of the horizontal velocity fields is about 4000s.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g40mm00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Low gravity', $
Aim:           'Low-gravity model', $
Result:        '', $
Problems:      'Crash', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_02', $
nFile:              2, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n01a', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Experimental', $
Result:        '', $
Problems:      'Experimental.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Start model too hot.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        4000, $
nMeanAvg:        7999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n03', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Strong horizontal flows in the atmosphere. A small-amplitude evil mode appears.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        4000, $
nMeanAvg:        6999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n04', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_08', $
nFile:              8, $
mMeanAvg:         500, $
nMeanAvg:        1999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n05', $
Status:        'Finished', $
Quality:        0.6, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_12', $
nFile:             12, $
mMeanAvg:        2000, $
nMeanAvg:        4999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n06', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Finer, equidistant vertical grid.', $
Aim:           '', $
Result:        'No problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n07', $
Status:        'Finished', $
Quality:        0.4, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller time step.', $
Aim:           '', $
Result:        'Slowly growing (evil?) mode.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_12', $
nFile:             12, $
mMeanAvg:        1000, $
nMeanAvg:        2999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt16g50mm00n08', $
Status:        'Finished jj', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Get 3D model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt17g50mm00n01', $
Status:        'Finished', $
Quality:        0.6, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Damping of plane-parallel waves activated late.', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_14', $
nFile:             14, $
mMeanAvg:        6000, $
nMeanAvg:        7999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt17g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Finer vertical grid.', $
Aim:           '', $
Result:        'Strong horizontal flow. Finished without problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt17g50mm00n03', $
Status:        'Finished jk', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Get 3D model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_110', $
nFile:            110, $
mMeanAvg:        1400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g35mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Low-gravity model.', $
Aim:           'Gravity sequence at Teff=1800K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Low-gravity model.', $
Aim:           'Gravity sequence at Teff=1800K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g40mm00n02', $
Status:        'Finished j9', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Low-gravity model. heat_mode=2, smaller Courant number than n01', $
Aim:           'Check influence of treatment of lower boundary', $
Result:        'Hardly any difference in velocity. Slight difference in entropy at bottom.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g45mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Lower-gravity model.', $
Aim:           'Gravity sequence', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n06', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Damping of plane-parallel waves activated late.', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        6000, $
nMeanAvg:        7999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n07', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Finished without problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n08', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Dustbin scheme.', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n10', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Modified Dustbin scheme.', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:        9000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n11', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D brown dwarf model', $
Parameters:    '', $
Aim:           '3D model at Teff=1800K', $
Result:        'Thick cloud deck with regular wave pattern', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_200', $
nFile:           200, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n12', $
Status:        'Finished j5', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x280, new code version 2012-02-26: CTU, CA1, FRweno, equidistant vertical grid.', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n13', $
Status:        'Finished jk', $
Quality:        0.9, $
Type:          '3D dusty brown dwarf model', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of new grid and new code version.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:           100, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g50mm00n14', $
Status:        'Finished jp', $
Quality:        0.9, $
Type:          '3D dusty brown dwarf model', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of treatment of top boundary.', $
Result:        'The horizontal flow in the upp layers just seems to oscillate back and forth -- without rotation. The horizontal velocities are significantly larger than for n13.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_120', $
nFile:           120, $
mMeanAvg:        1800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n29', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Less damped layers at top (30 -> 8).', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n30', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Damping reduced compared to mt18g55mm00n29. Less damped layers at top (30 -> 8).', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n31', $
Status:        'Finished', $
Quality:        0.4, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Damping reduced compared to mt18g55mm00n30.', $
Aim:           '', $
Result:        'Very short run.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_03', $
nFile:              3, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n32', $
Status:        'Finished', $
Quality:        0.9, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Damping reduced compared to mt18g55mm00n31.', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_45', $
nFile:             45, $
mMeanAvg:        5000, $
nMeanAvg:        6999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n33', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Higher-gravity model. Created from 1D Phoenix stratification.', $
Aim:           'Gravity sequence at Teff=1800K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n33b', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Testrun: 2D dusty Brown Dwarf', $
Parameters:    'Identical to mt18g55mm00n33', $
Aim:           'Test new code version with opacity computation in sub module', $
Result:        '', $
Problems:      'Crash: low temperature and small time step due to wrong tweak module, dust opacities not in mean file', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              1, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt18g55mm00n33c', $
Status:        'Finished', $
Quality:        0.1, $
Type:          'Testrun: 2D dusty Brown Dwarf', $
Parameters:    'Identical to mt18g55mm00n33', $
Aim:           'Test new, corrected code version with opacity computation in sub module', $
Result:        'Stable run. Results first identical to mt18g55mm00n33. Later, the runs deviate.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              1, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt19g50mm00n01', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        1500, $
nMeanAvg:        2199, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt19g50mm00n02', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'A weird (evil?) mode grows and shrinks again.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_23', $
nFile:             23, $
mMeanAvg:        4000, $
nMeanAvg:        7999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt19g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller Courant number.', $
Aim:           '', $
Result:        'Finished without problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n01', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        2000, $
nMeanAvg:        3499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n02', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_09', $
nFile:              9, $
mMeanAvg:        2000, $
nMeanAvg:        3499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n03', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_09', $
nFile:              9, $
mMeanAvg:        1500, $
nMeanAvg:        2999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n04', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Start with late snapshot from mt20g50mm00n03.', $
Result:        'Evil mode grows very slowly.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_04', $
nFile:              4, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n05', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Start with early snapshot from mt20g50mm00n03.', $
Result:        'Finished without problems.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n06', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Linear viscosity', $
Aim:           'Test linear viscosity', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n07', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Linear viscosity', $
Aim:           'Test linear viscosity', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:         600, $
nMeanAvg:         799, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n08', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Linear viscosity: 0.01.', $
Result:        'Evil mode appears but remains under control.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'Mac: orion', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        2000, $
nMeanAvg:        7999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n09', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '3D model with thin clouds, but thicker than at Teff=2200K', $
Result:        'Thin small irregular clouds with granules shining through', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_200', $
nFile:            200, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n10', $
Status:        'Finished j5', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x290, new code version 2012-02-16: CTU, CA1, FRweno, VisP2P, equidistant vertical grid.', $
Aim:           'Get 2000K model with new code version and settings.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n11', $
Status:        'Finished jl', $
Quality:        0.9, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of new grid and new code version.', $
Result:        'Two gravity-wave regimes. Weak mean horizontal flows.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:            100, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt20g50mm00n12', $
Status:        'Finished jl', $
Quality:        0.9, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4(tweak)=0.0, strtest1(tweak)=Vertical.', $
Aim:           'Check influence of upper boundary condition.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:            100, $
mMeanAvg:        1800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Low-gravity model.', $
Aim:           'Temperature sequence at logg=4', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n01', $
Status:        'Finished', $
Quality:        0.1, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end (rather quickly).', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        1200, $
nMeanAvg:        1999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n02', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_08', $
nFile:              8, $
mMeanAvg:        1500, $
nMeanAvg:        3499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n03', $
Status:        'Finished', $
Quality:        0.1, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The change in depth heating volume produces extra waves. The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_02', $
nFile:              2, $
mMeanAvg:         400, $
nMeanAvg:         699, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n04', $
Status:        'Finished', $
Quality:        0.9, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_09', $
nFile:              9, $
mMeanAvg:        2000, $
nMeanAvg:        3499, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n05', $
Status:        'Finished', $
Quality:        0.9, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Smaller time step.', $
Aim:           '', $
Result:        'The evil mode appears at the end.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        2000, $
nMeanAvg:        9999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n06', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Dustbin scheme.', $
Aim:           '', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n07', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Modified dustbin scheme.', $
Aim:           '', $
Result:        'No trends.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_40', $
nFile:             40, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n08', $
Status:        'Finished.', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    '3D', $
Aim:           '3D model of Brown Dwarf with thin cloud layer', $
Result:        'Intermittent patchy clouds over low-contrast granulation', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_50', $
nFile:             50, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n09', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    '3D, rotation period: 2 hours', $
Aim:           'Rotating local 3D model of Brown Dwarf with thin cloud layer', $
Result:        'Slowly rising horizontal velocities v1 above the convection zone, not much effect on granules.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_300', $
nFile:            300, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n10', $
Status:        'Finishded jc', $
Quality:        0.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    '3D, rotation period: 0.5 hours', $
Aim:           'Rotating local 3D model of Brown Dwarf with thin cloud layer', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_200', $
nFile:            200, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n11', $
Status:        'Finished jd', $
Quality:        0.0, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    '3D, rotation period: 0.25 hours', $
Aim:           'Rotating local 3D model of Brown Dwarf with thin cloud layer', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_200', $
nFile:            200, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n12', $
Status:        'Finished j1', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x290, new code version 2012-03-01: CTU, CA1, FRweno, VisP2P, equidistant vertical grid.', $
Aim:           '2200K model with new code version and settings.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:        9999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n13', $
Status:        'Finished jm', $
Quality:        0.9, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of new grid and new code version.', $
Result:        'Two gravity-wave regimes. Weak mean horizontal flows.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt22g50mm00n14', $
Status:        'Finished jm', $
Quality:        0.9, $
Type:          '3D dusty Brown Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4(tweak)=0.0, strtest1(tweak)=Vertical.', $
Aim:           'Check influence of upper boundary condition.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt24g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Temperature sequence at logg=4', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt24g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt24g50mm00n02', $
Status:        'Finished jq', $
Quality:        0.9, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2014-12-21: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0', $
Aim:           'Check influence of new grid and new code version.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:           100, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt24g50mm00n03', $
Status:        'Finished jq', $
Quality:        1.0, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest1=Vertical', $
Aim:           'Check influence of upper boundary condition.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:           100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g35mm00n01', $
Status:        'Finished', $
Quality:        0.6, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Gravity sequence at Teff=2600K', $
Result:        '', $
Problems:      'Low temperatures at upper boundary', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Temperature sequence at logg=4', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_13', $
nFile:             13, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g45mm00n01', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Gravity sequence at Teff=2600K', $
Result:        '', $
Problems:      'Crash due to large horizontal velocities and low temperatures at the top. Run is still usefull.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_09', $
nFile:              9, $
mMeanAvg:        4000, $
nMeanAvg:        7800, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g45mm00n02', $
Status:        'Finished j4', $
Quality:        0.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Top damping: C_test3: 20.0 -> 10.0', $
Aim:           'Gravity sequence at Teff=2600K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Low-Teff opacity table.', $
Aim:           '', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'High-Teff opacity table.', $
Aim:           '', $
Result:        'Slight difference in temperature structure due to different opacity table.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g50mm00n03', $
Status:        'Finished ja', $
Quality:        1.0, $
Type:          '3D dusty M Dwarf', $
Parameters:    '3D, high-Teff opacity table.', $
Aim:           'Get 3D model for conditions where dust just starts to form.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_200', $
nFile:            200, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g50mm00n04', $
Status:        'Finished j2', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Resolution: 400x290, new code version 2012-03-01: CTU, CA1, FRweno, VisP2P, equidistant vertical grid.', $
Aim:           '2600K model with new code version and settings.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g50mm00n05', $
Status:        'Finished jn', $
Quality:        0.9, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2014-12-07: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor', $
Aim:           'Check influence of new grid and new code version.', $
Result:        'Fluctuating global horizontal velocity fields with period of 3.5 h and amplitude of about 0.5 km/s.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_140', $
nFile:            140, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt26g50mm00n06a', $
Status:        'Finished jo', $
Quality:        0.0, $
Type:          '3D dusty M Dwarf', $
Parameters:    '3D', $
Aim:           'Test the new code version including more-frequent tensor-viscosity steps.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_001', $
nFile:              1, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $


{ $
Model:         'mt26g50mm00n07', $
Status:        'Finished jn', $
Quality:        1.0, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest1=Vertical', $
Aim:           'Check influence of extended grid and changed top boundary.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt28g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           'Temperature sequence at logg=4', $
Result:        '', $
Problems:      'Run without turbulent Prandtl number crashed because of low-T spike.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_11', $
nFile:             11, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt28g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_15', $
nFile:             15, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt28g50mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D dusty Brown Dwarf', $
Parameters:    'Added 23 points at top.', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_10', $
nFile:             10, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt28g50mm00n03', $
Status:        'Finished jo', $
Quality:        0.9, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2014-12-21: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, C_test4=0.0', $
Aim:           'Check influence of new grid and new code version.', $
Result:        '', $
Problems:      'It takes quite some time until the large-scale v2 velocity fields develop: they become large only from i_t=5000 on.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_200', $
nFile:           200, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt28g50mm00n04', $
Status:        'Finished jo', $
Quality:        1.0, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest1=Vertical', $
Aim:           'Check influence of deeper bottom and vertical-only damping at upper bouondary.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:           100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt28g50mm00n05', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          '3D dusty M Dwarf', $
Parameters:    'New code version 2015-04-28: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest1=Vertical, opafile=t2800g500mm00_AGSrainout.opta.', $
Aim:           'Check influence of opacity table.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:           100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust', $
Aim:           'Temperature sequence at logg=4', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust, high output sampling rate, very long sequence.', $
Aim:           'Continue Brown Dwarf sequence to higher Teff. Analyse modes.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_20', $
nFile:             20, $
mMeanAvg:        9500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n02', $
Status:        'Finished', $
Quality:        0.1, $
Type:          '2D M-type dwarf', $
Parameters:    'Open lower boundary', $
Aim:           'Compare open and closed lower boundary (mt30g50mm00n01)', $
Result:        'Larger velocity amplitude than n01: growing gravity wave.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust, deeper and wider than n01, bottom_entropy2', $
Aim:           'Check if depth and deep damping matters', $
Result:        'Temperature structure very similar to n01. Velocities slightly smaller.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust, stronger deep damping than n03', $
Aim:           'Check if deep damping matters', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n05', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust, new damping: heat_mode=2. hdsplit=123', $
Aim:           'Reference run to compare with MHD solver result', $
Result:        'Larger vertical wave velocities than n03 or n04', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n06', $
Status:        'Finished', $
Quality:        0.4, $
Type:          '2D M-type dwarf', $
Parameters:    'Comparable to mt30g50mm00n05, but with HLLMHD solver', $
Aim:           'Compare HD and MHD solver results', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n07', $
Status:        'Finished j6', $
Quality:        0.0, $
Type:          '2D M-type dwarf', $
Parameters:    'New scheme: CTU, FRweno, CA1, VisP2P', $
Aim:           'Compare to older runs and model n08 computed with HLLMHD.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_07', $
nFile:              7, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n08', $
Status:        'Finished j7', $
Quality:        0.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Comparable to mt30g50mm00n07, but with HLLMHD solver.', $
Aim:           'Compare HD and MHD solver results', $
Result:        '', $
Problems:      'mean data corrupted: each time step has an arbitrary integer factor applied to all data collected by the HLL module.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n09', $
Status:        'Finished j6', $
Quality:        0.0, $
Type:          '2D M-type dwarf', $
Parameters:    'New scheme: CTU, FRweno, CA1, VisP2P settings as for new brown dwarf models.', $
Aim:           'Check influence of visosity settings and compare to model n07.', $
Result:        'Larger heating near the top increases the local temperature somewhat.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_07', $
nFile:              7, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n10', $
Status:        'Finished je', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-01-03: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0', $
Aim:           'Check influence of new code version and grid.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        2900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n11', $
Status:        'Finished je', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest1=Vertical', $
Aim:           'Check influence of upper boundary.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt30g50mm00n12', $
Status:        'Finished jl', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest1=Vertical, opafile=t3100g500mm00_AGSrainout.opta', $
Aim:           'Check influence of opacity table.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt32g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Low-g model. Horizontally only 300 points, no dust.', $
Aim:           '', $
Result:        'Finished.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt32g40mm00n02', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (faulty)', $
Aim:           'Test new version of buffer layers at closed lower boundary', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt32g40mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (corrected)', $
Aim:           'Test new version of buffer layers at closed lower boundary', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt32g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust', $
Aim:           'Continue Brown Dwarf sequence to higher Teff.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt32g50mm00n02', $
Status:        'Finished jc', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter', $
Aim:           'Get 3D model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        5000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt32g50mm00n03', $
Status:        'Finished jm', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-04-28: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, opafile=t3300g500mm00_AGSrainout.opta', $
Aim:           'Check influence of opacity table.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100',$
nFile:            100, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g25mm00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D M-type star', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2; Prandtl number non-zero. Start model based on AMES-Cond-2000', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              0, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g25mm00n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D M-type star', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2; Prandtl number non-zero. Start model based on BT-NextGen', $
Aim:           'Gravity sequence at Teff=3300K. Check influence of start model.', $
Result:        '', $
Problems:      'PHOENIX model does not go down deep enough. End with crash.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              0, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g30mm00n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '2D M-type star', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        '', $
Problems:      'Weird small-scale fluctuations in upper photosphere: model is too extended', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_01', $
nFile:              1, $
mMeanAvg:          60, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g30mm00n02', $
Status:        'Finished', $
Quality:        0.7, $
Type:          '2D M-type star', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2. Shallower than mt33g30mm00n01. Prandtl number non-zero.', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        '', $
Problems:      'Base PHOENIX model not deep enough. Deep entropy too low. Model too cool.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g35mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type star', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        '', $
Problems:      'Growing wave in 01,02. Stronger damping limits the amplitude.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g40mm00n01', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (corrected)', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        'Maximum of v3rms in convection zone close to bottom.', $
Problems:      'Growing plane-parallel wave', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:        6000, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g40mm00n02', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (corrected)', $
Aim:           'Gravity sequence at Teff=3300K, deeper than n01', $
Result:        '', $
Problems:      'Growing wave', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:       4000, $
nMeanAvg:       6000, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g40mm00n03', $
Status:        'Finished', $
Quality:        0.9, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2, stronger damping than n02', $
Aim:           'Gravity sequence at Teff=3300K, stronger damping than n02', $
Result:        '', $
Problems:      'Wave remains with amplitude roughtly constant', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g40mm00n04', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'bottom_entropy2, even stronger damping than n03', $
Aim:           'Gravity sequence at Teff=3300K, suppress waves', $
Result:        'Only very small difference to mt33g40mm00n03', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g45mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (corrected)', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g45mm00n02', $
Status:        'Finished jg', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2014-12-17: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter', $
Aim:           'Check influence of new grid and new code version.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_140', $
nFile:            140, $
mMeanAvg:        3300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g45mm00n03', $
Status:        'Finished jf', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2014-12-17: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0', $
Aim:           'Check influence of removed energy dissipation due to drag force at top.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_180', $
nFile:            180, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g45mm00n04', $
Status:        'Finished jg', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, strtest=Vertical', $
Aim:           'Check influence removed horizontal drag force at top.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (corrected)', $
Aim:           'Gravity sequence at Teff=3300K', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n02', $
Status:        'Finished jp', $
Quality:        0.4, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2014-12-17: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter', $
Aim:           'Check influence of new grid and new code version.', $
Result:        '', $
Problems:      'Warm filaments rise from lower-boundary layers. The very outer layers are still warming up.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_080', $
nFile:             80, $
mMeanAvg:        2400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n03', $
Status:        'Finished jh', $
Quality:        0.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2014-12-17: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0', $
Aim:           'Check influence of energy dissipation in top drag-force layers.', $
Result:        '', $
Problems:      'Warm filaments rise from lower-boundary layers. The uppermost layers are not yet thermally relaxed.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_050', $
nFile:             50, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n04', $
Status:        'Finished jd', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2014-12-17: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0', $
Aim:           'Check influence of lower-boundary settings.', $
Result:        'Fluctuating (rotating) global horizontal velocity fields with period of 3 h and amplitude of about 1.2 km/s.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_170', $
nFile:            170, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n05', $
Status:        'Finished jh', $
Quality:        0.7, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-01-13: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, C_test4=0.0, heat_mode=bottom_entropy2h', $
Aim:           'Check influence of lower-boundary settings.', $
Result:        '', $
Problems:      'Sequence could be longer. The averaging interval comprises only one half-period of the horizontal-flow oscillation.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n06', $
Status:        'Finished jc', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter, top_bound=transmitting', $
Aim:           'Check influence of upper-boundary settings: closed -> open', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n07', $
Status:        'Finished jb', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter', $
Aim:           'Check influence of upper-boundary settings: 16 points added, only vertical drag', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n08', $
Status:        'Finished jd', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter. Grid size n07: 448^2x320 -> 224^2x208 by coarsening.', $
Aim:           'Check influence of (coarser) resolution. Produce fast running model that can be easily extended horizontally.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n09', $
Status:        'Finished jd', $
Quality:        0.9, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter. Grid size n08: 224^2x208 -> 448^2x208 by stretching.', $
Aim:           'Extended (coarse) model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n10', $
Status:        'Finished jd', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter. Grid size n09: 448^2x208 -> 986^2x208 by replicating.', $
Aim:           'Further extended (coarse) model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n11', $
Status:        'Finished jb', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: CTU, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, (C_hydLowMachcsBase, C_hydLowMachcsFactor), N_visIter', $
Aim:           'Check influence of directional splitting type (here: CTU).', $
Result:        'Works fine. Mean results almost indistinguishable from n07. However, the flow field in the deeper layers does not look as smooth.', $
Problems:      'Instability and crash with C_hydLowMachcsFactor=2.0. Works with 0.0.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n12', $
Status:        'Finished jf', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: 123, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, N_visIter', $
Aim:           'Check influence of directional splitting type.', $
Result:        '', $
Problems:      'Instability and crash with C_hydLowMachcsFactor>0.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n13', $
Status:        'Finished je', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: CTU, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, (C_hydLowMachcsBase, C_hydLowMachcsFactor), N_visIter, open lower boundary', $
Aim:           'Check influence of lower boundary.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt33g50mm00n14', $
Status:        'Finished jn', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter. Different opacity table than n07', $
Aim:           'Check influence of opacity table', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_150', $
nFile:            150, $
mMeanAvg:        2400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g40mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'Horizontally only 300 points, no dust, rather shallow.', $
Aim:           '', $
Result:        '', $
Problems:      'Somewhat too shallow', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g40mm00n02', $
Status:        'Finished', $
Quality:        0.5, $
Type:          '2D M-type dwarf', $
Parameters:    '', $
Aim:           'New heating and drag force at lower boundary: bottom_entropy2 (faulty).', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g40mm00n03', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D M-type dwarf', $
Parameters:    '', $
Aim:           'New heating and drag force at lower boundary: bottom_entropy2 (corrected).', $
Result:        '', $
Problems:      'Slowly growing mode', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g40mm00n04', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D M-type dwarf', $
Parameters:    'Deeper and wider than mt35g40mm00n03, damping at bottom increased', $
Aim:           'Check if new settings suppress growing mode.', $
Result:        '', $
Problems:      'Slowly growing mode', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g50mm00n01', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf ', $
Parameters:    'Horizontally only 300 points, no dust.', $
Aim:           'Extend Brown Dwarf sequence to higher Teff.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g50mm00n02', $
Status:        'Finished jh', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-02-01: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter', $
Aim:           '3D model at 3500K/5.00', $
Result:        'Once the laminar flow in the very top layers if fully developed the temperature at the top rises by about 50K.', $
Problems:      'The laminar flow in x2 direction takes a long time to develop.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        7000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt35g50mm00n03', $
Status:        'Finished jo', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-04-28: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter. Different opacity file than n02, opafile=t3500g500mm00_AGSrainout.opta', $
Aim:           'Check influence of opacity table.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        2800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt40g50mm00n01', $
Status:        'Finished', $
Quality:        0.6, $
Type:          '2D M-type dwarf', $
Parameters:    'New heating and drag force at lower boundary: bottom_entropy2 (n01-03:faulty, n04-..:corrected).', $
Aim:           'Extend Brown Dwarf sequence to higher Teff.', $
Result:        '', $
Problems:      'A bit too narrow', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_06', $
nFile:              6, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt40g50mm00n02', $
Status:        'Finished', $
Quality:        0.8, $
Type:          '2D M-type dwarf', $
Parameters:    'Deeper and wider than mt40g50mm00n01', $
Aim:           'Extend Brown Dwarf sequence to higher Teff.', $
Result:        '', $
Problems:      'Growing wave?', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt40g50mm00n03', $
Status:        'Finished', $
Quality:        1.0, $
Type:          '2D M-type dwarf', $
Parameters:    'More damping than mt40g50mm00n02', $
Aim:           'Extend Brown Dwarf sequence to higher Teff.', $
Result:        'Mode amplitude decreases.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_05', $
nFile:              5, $
mMeanAvg:        6000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt40g50mm00n04', $
Status:        'Finished jp', $
Quality:        1.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-04-28: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter', $
Aim:           '3D model at 4000K/5.00', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        2800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'mt40g50mm00n05', $
Status:        'Finished jq', $
Quality:        0.0, $
Type:          '3D M-type dwarf', $
Parameters:    'New code version 2015-04-28: Runge-Kutta, N_hydRKIter=3, HBweno, c_RecCont, C_gravmu, CA4, CA1gMps, Entropy3gMps, C_hydLowMachcsBase, C_hydLowMachcsFactor, N_visIter; different opacity file than n04, opafile=t3800g490mm00_AGSrainout.opta', $
Aim:           '3D model at 4000K/5.00', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_069', $
nFile:             69, $
mMeanAvg:        1400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Sun ----------------------------------------------------------------------
{ $
Model:         'd3t57g44b0n001', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution', $
Aim:           'Get a start model for MHD runs', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent:  '_072', $
nFile:             72, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n002', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, added 100 points at bottom to n001', $
Aim:           'Deeper model', $
Result:        '', $
Problems:      'The simple extension in the deep layers was not optimal.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_090', $
nFile:             90, $
mMeanAvg:         720, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n003', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, HLLMHD solver, no magnetic field.', $
Aim:           'Reference for models with magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_062', $
nFile:             62, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n004', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'No tweak damping at top.', $
Aim:           'Check influence of top boundary damping layers', $
Result:        'Slightly higher velocity and lower temperature at top', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_045', $
nFile:             45, $
mMeanAvg:         320, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n005', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'HLLMHD, lower resolution: 280x280x140, dx=20km', $
Aim:           'Longer run to check growth of modes', $
Result:        'The amplitude of the kinetic energy of plane-parallel modes varies but does not grow systematically', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_024', $
nFile:             24, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n006', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, deep model, 400^3, rather high resolution, added 100 points at bottom to n002', $
Aim:           'Deeper model', $
Result:        '', $
Problems:      'Deep layers (bottom 50 points or so) are not well relaxed.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_090', $
nFile:             90, $
mMeanAvg:        1100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n007', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, HLLMHD solver, no magnetic field.', $
Aim:           'Reference for models with magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_075', $
nFile:             75, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n008', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, HLLMHD solver, no magnetic field, C_VisBound=0.2.', $
Aim:           'Reference for models with magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_034', $
nFile:             34, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n009', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, HLLMHD solver, no magnetic field, C_VisBound=0.2.', $
Aim:           'Reference for models with magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_027', $
nFile:             27, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n010', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'HLLMHD, lower resolution: 280x280x140, dx=20km, top: transmitting3, C_VisBound', $
Aim:           'Low-resolution reference model with new boundary settings', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_006', $
nFile:             06, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n011', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model', $
Parameters:    'HLLMHD, higher resolution: 560^2x280, dx=10km, top: transmitting3, C_VisBound', $
Aim:           'High-resolution reference model with new boundary settings', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_061', $
nFile:             61, $
mMeanAvg:         720, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n012', $
Status:        'Finished jm', $
Quality:        0.0, $
Type:          'Grey 3D solar model', $
Parameters:    'HLLMHD, lower resolution: 280x280x140, dx=20km, top: transmitting3, C_VisBound', $
Aim:           'Low-resolution reference model with new boundary settings', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_012', $
nFile:             12, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n013', $
Status:        'Finished jl', $
Quality:        0.6, $
Type:          'Grey 3D solar model', $
Parameters:    'Rather high resolution, HLLMHD solver, no magnetic field, C_VisBound=0.3, viscosiy enlarged.', $
Aim:           'Reference for models with magnetic field and large viscosity', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_031', $
nFile:             31, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n014', $
Status:        'Finished jl', $
Quality:        0.7, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, deep model, 400^3, new code version: CTU, FRweno', $
Aim:           'Test the new code version', $
Result:        'Works fine, even with enhanced continuity for low Mach numbers.', $
Problems:      'Time step restriction, damping of v3 at lower boundary', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_054', $
nFile:             54, $
mMeanAvg:        2200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n015', $
Status:        'Finished jk', $
Quality:        0.8, $
Type:          'Grey high-resolution 3D solar model', $
Parameters:    'Grey opacities, 560^2x280, new code version: CTU, FRweno', $
Aim:           'Test the new code version. Get a high-resolution model.', $
Result:        'Looks good. There is lots of fine structure at the intergranular lanes.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         360, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n016', $
Status:        'Finished jm', $
Quality:        0.9, $
Type:          'Grey high-resolution 3D solar model', $
Parameters:    'Grey opacities, 560^2x280, new code version: CTU, PP', $
Aim:           'Test the new code version. Get a high-resolution model. Test PP vs. FRweno.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         360, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n017', $
Status:        'Finished jn', $
Quality:        0.9, $
Type:          'Grey high-resolution 3D solar model', $
Parameters:    'Grey opacities, 560^2x280, new code version: CTU, PP, CA1', $
Aim:           'Test the new code version. Get a high-resolution model. Test CA1 vs. Normal compared to n016.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         360, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n018', $
Status:        'Finished jo', $
Quality:        1.0, $
Type:          'Grey high-resolution 3D solar model', $
Parameters:    'Grey opacities, 560^2x280, new code version: 123, PP, CA1', $
Aim:           'Test the new code version. Get a high-resolution model. Test 123 vs. CTU (with n017).', $
Result:        'Obviously, 123 is faster than CTU. Temperature structures are very similar. There are tiny differences in the velocity averages.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         360, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n019', $
Status:        'Finished jl', $
Quality:        0.8, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, deep model, 400^3, new code version: CTU, CA1, FRweno', $
Aim:           'Test the new code version, particularly CA1 vs Normal.', $
Result:        '', $
Problems:      'Relaxation in the deeper layers not perfect.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:         540, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n020', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Grey super-high-resolution 3D solar model', $
Parameters:    'Grey opacities, 800^2x400, new code version: 123, PP, CA1', $
Aim:           'Get a super-high-resolution model.', $
Result:        'Lots of very fine structure: turbulence, small-scale waves, photospheric shocks, eddies (bright spots).', $
Problems:      'Crash due to low temperatures near strong shock at the top avoided by setting C_TminLimit=1600.0.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n021', $
Status:        'Finished jm', $
Quality:        0.9, $
Type:          'Not so high-resolution 3D solar model', $
Parameters:    'Grey opacities, 400^2x200, new code version: 123, PP, CA1', $
Aim:           'Get a not-quite-so-high-resolution model for comparison purposes.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_050', $
nFile:             50, $
mMeanAvg:         540, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n022', $
Status:        'Finished jo', $
Quality:        0.9, $
Type:          'Not so high-resolution 3D solar model', $
Parameters:    'Grey opacities, 400^2x200, new code version 2012-02-09: 123, PP, CA1', $
Aim:           'Test influence of second-order gravity terms compared to n021.', $
Result:        'Models n021 and n022 have exactly the same convective v3-rms velocities. However, n22 has about 0.015 dex lower wave v3-rms amplitudes and 0.2 des lower v3-mass amplitudes.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_050', $
nFile:             50, $
mMeanAvg:         540, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n023', $
Status:        'Finished jl', $
Quality:        1.0, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, deep model, 400^3, new code version: 123, CA1, FRweno', $
Aim:           'Check influence of 2nd-order gravity terms compared to n019.', $
Result:        'v3-rms velocities slightly smaller. v3-mass velocities visibly smaller. All-in-all better consistency to shallower models, e.g. n022.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_040', $
nFile:             40, $
mMeanAvg:         540, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n024', $
Status:        'Finished jk', $
Quality:        1.0, $
Type:          'Grey super-high-resolution 3D solar model', $
Parameters:    'Grey opacities, 800^2x400, new code version: 123, PP, CA4, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Check if new transversal diffusion has an impact.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n025', $
Status:        'Finished jk', $
Quality:        1.0, $
Type:          'Grey super-high-resolution 3D solar model', $
Parameters:    'Grey opacities, 800^2x400, new code version (2012-07-30): 123, PP, CA4, Entropy3, CA1s, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Check if new enthalpy-averaging and new entropy-wave treatment have an impact.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n026', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, deep model, 400^3, new code version (2012-07-15: the wrong one): 123, FRweno, CA4, Entropy3?, CA1s, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Check if new enthalpy-averaging and new entropy-wave treatment have an impact.', $
Result:        '', $
Problems:      'Empty IDL intens.sav file', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_060', $
nFile:             60, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n027', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, deep model, 400^3, new code version (2012-07-30): 123, FRweno, CA4, Entropy3, CA1s, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Check if new enthalpy-averaging and new entropy-wave treatment have an impact.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_008', $
nFile:              8, $
mMeanAvg:         240, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n029', $
Status:        'Finished jm', $
Quality:        0.0, $
Type:          'Grey deep 3D solar model', $
Parameters:    'Grey opacities, deep model, 400^3, new code version (2012-07-30): c_hydlowmachcsfactor=2.0, RungeKutta2, unsplit, FRweno, CA4, Entropy3, CA1s, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           '', $
Result:        '', $
Problems:      'Weird oscillations.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n030', $
Status:        'Finished jk', $
Quality:        1.0, $
Type:          'Grey super-high-resolution 3D solar model', $
Parameters:    'Grey opacities, 800^2x400, new code version (2012-11-05): 123, FRweno, CA4, Entropy3gMps, CA1gMps, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Check difference between PP and FRweno, check newest code version.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n031', $
Status:        'Finished jn', $
Quality:        1.0, $
Type:          'Grey horizontally enlarged 3D solar model', $
Parameters:    'Grey opacities, 350^2x140, 14Mmx14Mmx2.8Mm, new code version (2013-07-11): 123, PP, CA4, Entropy3gMps, CA1gMps, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Intermediate step to go for (horizontally) larger models.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_050', $
nFile:             50, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n032', $
Status:        'Finished jn', $
Quality:        1.0, $
Type:          'Grey horizontally enlarged 3D solar model', $
Parameters:    'Grey opacities, 700^2x140, 28Mmx28Mmx2.8Mm, new code version (2013-07-30): 123, PP, CA4, Entropy3gMps, CA1gMps, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Intermediate step to go for (horizontally) larger models.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b0n033', $
Status:        'Finished jn', $
Quality:        0.0, $
Type:          'Grey horizontally enlarged 3D solar model', $
Parameters:    'Grey opacities, 1400^2x140, 56Mmx56Mmx2.8Mm, new code version (2013-07-30): 123, PP, CA4, Entropy3gMps, CA1gMps, C_hydsDiffVelo, C_hydvDiffVelo', $
Aim:           'Look for large structures (supergranulation,...)', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:            100, $
mMeanAvg:         540, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n002', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, mean vertical magnetic field=1G', $
Aim:           'MHD model with weak vertical magnetic field', $
Result:        '', $
Problems:      'Increase in magnetic field due to flux through the bottom', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_090', $
nFile:             90, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n003', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Grey opacities, n_radtheta=-1, rather high resolution, mean vertical magnetic field=0.1G', $
Aim:           'MHD model with weak vertical magnetic field', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_090', $
nFile:             90, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n004', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, top/bottom_bound_mag=constant', $
Aim:           'Check for dynamo action', $
Result:        '"Dynamo" at top boundary', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_090', $
nFile:             90, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n005', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, top/bottom_bound_mag=vertical', $
Aim:           'Check for dynamo action. Check influence of boundary conditions', $
Result:        'The "vertical" bundary condition seems to cause less magnetic flux through the boundaries but is less stable than "constant".', $
Problems:      'Ends with crash', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n006', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, top/bottom_bound_mag=vertical, C_VisBound=0.2,..0.5.', $
Aim:           'Check for dynamo action. Check influence of boundary conditions', $
Result:        'Saturation level much lower than for n004', $
Problems:      'Quasi-dynamo action close to the top', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_057', $
nFile:             57, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n007', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, top/bottom_bound_mag=vertical, top_Bound=transmitting3, C_VisBound=0.3.', $
Aim:           'Check for dynamo action. Check influence of boundary conditions', $
Result:        'Total magnetic energy decays with time.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_061', $
nFile:             61, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n008', $
Status:        'Finished jl', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Initially just two "field lines", top/bottom_Bound_mag=vertical, top_Bound=transmitting3, C_VisBound=0.3.', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_036', $
nFile:             36, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n009', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, top/bottom_bound_mag=vertical, top_Bound=transmitting3, C_VisBound=0.3.', $
Aim:           'Check for dynamo action. Lower-resolution run.', $
Result:        'Magnetic field decays on approximately the same time scale as the higher-resolution (dx=14km) run d3t57g44b1n007.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_015', $
nFile:             15, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n010', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, decrease diffusive boundary damping gradually: first at bottom. Many parameter variations during the run.', $
Aim:           'Check for dynamo action. Lower-resolution run.', $
Result:        'At least one damping layer is needed at bottom. Overall, the magnetic field declines.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_016', $
nFile:             16, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n011', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=14km, increased tensor viscosity, 14 extra-diffusive cells at top', $
Aim:           'Check for dynamo action.', $
Result:        'Weak dynamo action, probably due to shear flow in photosphere?', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_121', $
nFile:            121, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n012', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=14km, HLL+predictor-corrector, 14 extra-diffusive cells at top', $
Aim:           'Test the new code version', $
Result:        'Decaying magnetic field', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_061', $
nFile:             61, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n013', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=14km, HLL+predictor-corrector, PP reconstruction, 14 extra-diffusive cells at top', $
Aim:           'Test PP reconstruction, RungeKutta2, split update', $
Result:        'Total magnetic energy fluctuates but remains at approximately the initial level.', $
Problems:      'PP+RungeKutta2 works but has pattern noise in v and B, depending on dtime.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_075', $
nFile:             75, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n015', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=14km, HLL+predictor-corrector, unsplit, VanLeer', $
Aim:           'Test the new code version', $
Result:        '', $
Problems:      'Density, internal energy, and velocities look OK. Growing fluctuations in the magnetic field lead to a crash.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_002', $
nFile:              2, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n016', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, HLLMHD, PP, RungeKutta2, 123', $
Aim:           'Check for dynamo action. Lower-resolution run.', $
Result:        'First some decline, then some growth of magnetic field to comparably small amplitude, 4 extra-diffusive cells at top', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:        4800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n017', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, CentralMHD, PP, RungeKutta2, 123', $
Aim:           'Check for dynamo action. Lower-resolution run.', $
Result:        'Growth of magnetic field', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_018', $
nFile:             18, $
mMeanAvg:        3200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n018', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, HLLMHD, PP, RungeKutta2, 123, tweak-damping', $
Aim:           'Check for dynamo action. Lower-resolution run.', $
Result:        'Growth of magnetic energy', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:        4800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n019', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, HLLMHD, PP, RungeKutta2, 123, tweak-damping of horizontal flows.', $
Aim:           'Check for dynamo action with damped horizontal flows. Lower-resolution run.', $
Result:        '7 hours with roughly constant magnetic energy. Then growth.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:        4800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n020', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, HLLMHD, PP, RungeKutta2, 123, top_bound_mag=constant', $
Aim:           'Check for influence of top boundary on dynamo action. Lower-resolution run.', $
Result:        'Magnetic field grows exponentially without saturating', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:        4800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n021', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123', $
Aim:           'Check new code version', $
Result:        'Significant increase in resolution without the artefacts for RungeKutta. No dynamo.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL, OAC', $
Computer:      'PSMN', $
ParModelIdent: '_030', $
nFile:             30, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n022', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'Random vertical magnetic field, dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=vertical', $
Aim:           'Suppress plane-parallel oscillations and check influence magnetic fields.', $
Result:        'No problems with Intel compiler version 11.1.069. Magnetic field declines even with only 5 diffusive layers at top.', $
Problems:      'Artificial wiggles in the magnetic field with Intel compiler version 12.0.084', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_040', $
nFile:             40, $
mMeanAvg:        2400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n023', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=constant, top_mag=vertical', $
Aim:           'Check influence of magnetic-field boundary settings', $
Result:        'Growing magnetic fields that saturates at about 9% of the kinetic field energy: dynamo.', $
Problems:      'Very long rang: the growth rate is small.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_160', $
nFile:            160, $
mMeanAvg:       13000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n024', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=vertical2, top_mag=vertical', $
Aim:           'Check influence of magnetic-field boundary settings', $
Result:        'Magnetic field energy slowly growing but fluctuating', $
Problems:      'Model is not relaxed. It is not clear if a dynamo is working.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_110', $
nFile:            110, $
mMeanAvg:        8000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n025', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=constant, top_mag=vertical2', $
Aim:           'Check influence of magnetic-field top boundary settings', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_120', $
nFile:            120, $
mMeanAvg:        5800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n026', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=vertical2, top_mag=vertical2', $
Aim:           'Check influence of magnetic-field top boundary settings.', $
Result:        'Overall, the magnetic field decays slowly: no dynamo. Comparing n024 and n026, the upper boundary seems to have some influence.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:            100, $
mMeanAvg:        5800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n027', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=constant, top_mag=vertical2, diff_bot:1->3', $
Aim:           'Check influence of increased number of diffusive layers at bottom.', $
Result:        'After an initial decay, the amplitude settles at a lower level than for n025.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_100', $
nFile:            100, $
mMeanAvg:        4800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n028', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=constant2, top_mag=vertical2, diff_bot:3', $
Aim:           'Check influence magnetic bottom boundary condition (constant -> constant2).', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_121', $
nFile:            121, $
mMeanAvg:        4000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n029', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=constant2, top_mag=vertical2, diff_bot:3', $
Aim:           'Check new reconstruction (PP -> FRmo).', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_001', $
nFile:              1, $
mMeanAvg:          50, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n030', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, PP, Hancock, 123, tweak damping, bot_mag=constant2, top_mag=vertical2, diff_bot:3', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_001', $
nFile:              1, $
mMeanAvg:          50, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'd3t57g44b1n031', $
Status:        'Finished jg', $
Quality:        0.0, $
Type:          'Grey 3D solar model with magnetic field', $
Parameters:    'dx1=dx2=dx3=20km, new code version, HLLMHD, FRmo, Hancock, 123, tweak damping, bot_mag=constant2, top_mag=vertical2, diff_bot:3', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent: '_004', $
nFile:              4, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

; --- Global models: Supergiants, ABG stars, young M or brown dwarfs ------------------------------------
{ $
Model:         'st22g35n01', $
Status:        'Finished jk', $
Quality:        0.0, $
Type:          '3D global brown dwarf model', $
Parameters:    'New code version 2013-03-28: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce non-rotating global toy model cool enough to form dust if activated.', $
Result:        '', $
Problems:      'Small but non-vanishing anisotropy in the velocity field (outward along the axes).', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_016', $
nFile:             16, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n02', $
Status:        'Finished je', $
Quality:        0.0, $
Type:          '3D global brown dwarf model', $
Parameters:    'New code version 2013-04-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce non-rotating global toy model cool enough to form dust if activated.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_045', $
nFile:             45, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n03', $
Status:        'Finished jc', $
Quality:        0.0, $
Type:          '3D global dusty brown dwarf model', $
Parameters:    'New code version 2013-04-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce non-rotating global toy model with dust', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_009', $
nFile:              9, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n04', $
Status:        'Finished jc', $
Quality:        0.0, $
Type:          '3D global dusty brown dwarf model', $
Parameters:    'New code version 2013-04-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce non-rotating global toy model with dust', $
Result:        '', $
Problems:      'Small amounts of dust get very far out. r1_grav somewhat too small?', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_030', $
nFile:             30, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n05', $
Status:        'Finished jc', $
Quality:        0.0, $
Type:          '3D global dusty brown dwarf model', $
Parameters:    'New code version 2013-05-10: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce non-rotating global toy model with dust', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_080', $
nFile:             80, $
mMeanAvg:         982, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n06', $
Status:        'Finished je', $
Quality:        0.8, $
Type:          'Rotating 3D global dusty brown dwarf model', $
Parameters:    'New code version 2013-07: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce rotating global toy model with dust', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_070', $
nFile:             70, $
mMeanAvg:         700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n07', $
Status:        'Finished je', $
Quality:        0.0, $
Type:          'Rotating 3D global dusty brown dwarf model', $
Parameters:    'New code version 2013-07: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce rotating global toy model with dust', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_170', $
nFile:            170, $
mMeanAvg:        2400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g35n08', $
Status:        'Finished PSMN-ja', $
Quality:        0.0, $
Type:          'Rotating 3D global dusty brown dwarf model', $
Parameters:    'New code version 2014-11-03: RungeKutta2(Iter=3), CA4, HBweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor/Base, .', $
Aim:           'Produce rotating global toy model with dust', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st22g40n01', $
Status:        'Finished je', $
Quality:        0.0, $
Type:          '3D global brown dwarf model', $
Parameters:    'New code version 2013-03-28: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Produce global toy model cool enough to form dust if activated.', $
Result:        '', $
Problems:      'Still axis-aligned flow: too low resolution?', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_010', $
nFile:             10, $
mMeanAvg:         170, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    '', $
Aim:           'First global toy model of M dwarf (without dust or rotation)', $
Result:        '', $
Problems:      'Spherical wave causes low temperatures in the photosphere and causes crash', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_008', $
nFile:              8, $
mMeanAvg:          00, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n02', $
Status:        'Finished', $
Quality:        0.3, $
Type:          '3D global M dwarf model', $
Parameters:    'Smaller than st26g35n01', $
Aim:           'First global toy model of M dwarf (without dust or rotation)', $
Result:        '', $
Problems:      'The initial symmetric convection patters decays slowly. Upflow along the axes, downflows along diagonals.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_400', $
nFile:            400, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'MinMod -> PPL, tweak module for low-density damping', $
Aim:           'First global toy model of M dwarf (without dust or rotation)', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n04', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'New code version 2012-04-11: CTU, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo', $
Aim:           'First global toy model of M dwarf (without dust or rotation)', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_060', $
nFile:             60, $
mMeanAvg:        1400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n05', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'New code version 2012-05-09: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor', $
Aim:           'Check new low-Mach-number modification of Roe solver', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_003', $
nFile:              3, $
mMeanAvg:          40, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n06', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'New code version 2012-05-15: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor. New start model with lower external temperature and density', $
Aim:           'Check new low-Mach-number modification of Roe solver', $
Result:        'Some improvements, but still not homogeneous flow across the surface', $
Problems:      'The convective flow has still axis preferences: out at the axes, in at the diagonals.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n07', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'New code version 2012-05-15: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor. New start model with lower external temperature and density', $
Aim:           'Check new enthalpy modification of Roe solver', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_070', $
nFile:             70, $
mMeanAvg:        2100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n08', $
Status:        'Finished jb', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'New code version 2012-05-15: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor. New start model with lower external temperature and density', $
Aim:           'Check new enthalpy/entropy modification of Roe solver', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_070', $
nFile:             70, $
mMeanAvg:        2100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n09', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          '3D global M dwarf model', $
Parameters:    'New code version 2012-11: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_070', $
nFile:             70, $
mMeanAvg:        1700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26g35n10', $
Status:        'Finished je', $
Quality:        0.7, $
Type:          '3D dusty global M dwarf model', $
Parameters:    'New code version 2013-03: RungeKutta, CA4, FRweno, C_hydsDiffVelo+C_hydvDiffVelo+C_hydLowMachcsFactor.', $
Aim:           'Look for dust formation', $
Result:        'Dust forms but is to thin to have any impact.', $
Problems:      'Star too hot. Dust too thin.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_020', $
nFile:             20, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26gm09n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Very short test run', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_03', $
nFile:              3, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26gm09n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_07', $
nFile:              7, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26gm07n001', $
Status:        'Finished mil-jh', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. C_test3=-....', $
Aim:           'Produce model with standard luminosity and mass but larger radius and lower Teff.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st26gm07n002', $
Status:        'Finished mil-jj', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st26gm07n001_050.end. C_test3=-....', $
Aim:           'Produce model with standard luminosity and mass but larger radius and lower Teff.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st27gm06n001', $
Status:        'Finished mil-jf', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm05n002_050.end. Increase the envelope mass', $
Aim:           'Increase radius and decrease Teff.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm07n001', $
Status:        'Finished mil-jk', $
Quality:        0.9, $
Type:          'Grey dust-free 1 M_Sun 10000 L_Sun AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. Luminosity=7000L_Sun ->10000L_Sun', $
Aim:           'Increase luminosity and radius.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:         900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm07n002', $
Status:        'Finished milou-jf (before: rackham-j1)', $
Quality:        0.8, $
Type:          'Grey 1 M_Sun 10000 L_Sun AGB star model', $
Parameters:    '8 MPI processes, 20/16/8 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta', $
Aim:           'Test MPI, check for influence of opacity table, put inflated star into a sufficiently large box', $
Result:        'Envelope is very extended. Star is extremely non-spherical.', $
Problems:      'Some mass is lost through the outer boundary', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham and milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm07n003', $
Status:        'Finished milou-je (before: rackham-j4)', $
Quality:        0.8, $
Type:          'Grey 1 M_Sun 10000 L_Sun AGB star model', $
Parameters:    '8 MPI processes, 20/16/8 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta, increased core size compared to n002', $
Aim:           'Test rhd_scale_module, adapt the core size r0_grav to the radius', $
Result:        'Envelope is very extended. Star is extremely non-spherical.', $
Problems:      'Some mass is lost through the outer boundary', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham and milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm07n004', $
Status:        'Finished milou-je', $
Quality:        0.8, $
Type:          'Grey 1 M_Sun 10000 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 397^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta', $
Aim:           'Put inflated star into a sufficiently large box', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm07n005', $
Status:        'Finished milou-jc', $
Quality:        0.5, $
Type:          'Grey 1 M_Sun 10000 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta', $
Aim:           'Get more compact 1M_Sun/10000L_Sun model', $
Result:        'The molsphere is much larger than the convective envelope.', $
Problems:      'The convective envelope is not much larger than the core. Sometimes dipolar, sometimes bipolar flow. Models needs larger box, higher resolution, smaller core.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_11', $
nFile:             11, $
mMeanAvg:         150, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'crow', $
ParModelIdent:   '_29', $
nFile:             29, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_17', $
nFile:             17, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n05', $
Status:        'Finished', $
Quality:        0.9, $
Type:          'Dusty AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_24', $
nFile:             24, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n06', $
Status:        'Finished', $
Quality:        0.9, $
Type:          'Dusty AGB star model', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_23', $
nFile:             23, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n07', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Dusty AGB star model with radiation pressure', $
Parameters:    '', $
Aim:           'Wind by radiation pressure on dust', $
Result:        '', $
Problems:      'Extremely tiny time step as soon as dust forms, very short run', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_07', $
nFile:              7, $
mMeanAvg:          10, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n12', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Various', $
Aim:           'Test new settings', $
Result:        '', $
Problems:      'The run is not long enough to give useful statistical results.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_013', $
nFile:             13, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n13', $
Status:        'Finished mil-ja', $
Quality:        0.7, $
Type:          'Dust-free AGB star model', $
Parameters:    '', $
Aim:           'First long new-generation AGB model run', $
Result:        'Works. Stellar time=38 yr, mass loss=1.3E-04 MSun/yr, about 21 shocks in outer atmosphere.', $
Problems:      'Some net mass loss, Some rather persistent flow through core.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n14', $
Status:        'Finished mil-jc', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Damp velocities with tweak module, rho_threshhold=10^-9.', $
Aim:           'Try to remove the essentially spherical puslations to see if they come back later.', $
Result:        'The main effect is a reduction in density in the outer layers. The pulsation frequency might increase a bit. Pulsations remain.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n15', $
Status:        'Finished mil-jd', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Damp velocities with tweak module, rho_threshhold=10^-5.', $
Aim:           'Try to remove the essentially spherical puslations to see if they come back later.', $
Result:        'The density in the outer layers drops a lot. All velocities are reduced. The frequency of pulsations increases.', $
Problems:      'Bug resulting in too strong damping in core removed from n28 on. Change leads to transients in mass loss.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_130', $
nFile:            130, $
mMeanAvg:        2200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n16', $
Status:        'Finished mil-je', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n13_050.end. Grid: 281^3->401^3.', $
Aim:           'Produce a higher-resolution reference model.', $
Result:        '', $
Problems:      'File 019.mean is corrupted.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n17', $
Status:        'Finished PSMN-ja', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'C_radHtauTop=10.0E+12', $
Aim:           'Test new code version and particularly new open radiation-transport boundary condition with some infalling intensity.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n18', $
Status:        'Finished PSMN-jb', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Extended model: 281^3 -> 401^3 points, same gridcell size as n13.', $
Aim:           'Check influence of extended grid.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n19', $
Status:        'Finished PSMN-jc', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'C_CoreDrag: 2.0 -> 3.0 compared to n17.', $
Aim:           'Damp global dipolar convective motions even stronger.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n20', $
Status:        'Finished mil-jf', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n19_010.end. Reduce r1_grav to get mass loss.', $
Aim:           'Check if modification of gravitational potential does result in increased mass loss. Check influence on oscillations.', $
Result:        'Strong mass loss: more than 10% of the envelope mass.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_080', $
nFile:             80, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n21', $
Status:        'Finished mil-jg', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n19_010.end. Reduce r1_grav to get mass loss.', $
Aim:           'Try to decrease mass loss by decreasing C_HpTopFactor.', $
Result:        'The mass loss can be reduced by decreasing C_HpTopFactor.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_120', $
nFile:            120, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n22', $
Status:        'Finished mil-ja', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n19_010.end. Use dust-free opacity table and new inner boundary condition.', $
Aim:           'Suppress flow through core and check effect of modified opacity table.', $
Result:        'Not a very strong reduction of the average mass loss.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n23', $
Status:        'Finished mil-jc', $
Quality:        0.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n19_010.end. Use dust-free opacity table and new inner boundary condition with entropy-smoothing.', $
Aim:           'Suppress flow through core and check effect of modified opacity table.', $
Result:        'The average mass loss is significantly reduced.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n24', $
Status:        'Finished mil-jd', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n23_030.end. Linear core drag force, decreased C_HpTopFactor.', $
Aim:           'Get "best" model by combining all optimal parameters, so far.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n25', $
Status:        'Finished PSMN-ja', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Extended model: 281^3 -> 401^3 points, same gridcell size as n13 or n14.', $
Aim:           'Check influence of extended grid for "optimal" parameters.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n26', $
Status:        'Finished mil-ja', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. Linear core drag force, decreased C_HpTopFactor.', $
Aim:           'Check new code version: 2015-01-04, CTU, HBweno.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n27', $
Status:        'Finished mil-jb', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. Linear core drag force, decreased C_HpTopFactor.', $
Aim:           'Check new code version: 2015-01-04, CTU, HBweno, C_hydLowMachcsFactor.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n28', $
Status:        'Finished mil-jc', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. Linear core drag force, decreased C_HpTopFactor.', $
Aim:           'Check new code version: 2015-01-04, RungeKutta2, HBweno, C_hydLowMachcsFactor.', $
Result:        '', $
Problems:      '_047.mean corrupted: recomputed at the end.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n29', $
Status:        'Finished mil-jd', $
Quality:        0.0, $
Type:          'Dust-free rotating AGB star model', $
Parameters:    'Start from st28gm06n26_050.end. Rotation period: 20 yrs.', $
Aim:           'Get rotating AGB-star model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n30', $
Status:        'Finished mil-jc', $
Quality:        0.0, $
Type:          'Dust-free rotating AGB star model', $
Parameters:    'Start from st28gm06n29_040.end. Rotation period: 10 yrs.', $
Aim:           'Get relatively fast-rotating AGB-star model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n031', $
Status:        'Finished mil-ja', $
Quality:        0.9, $
Type:          'Dust-free non-grey AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. Opacity: -> ', $
Aim:           'Get non-grey AGB-star model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n032', $
Status:        'Finished rackham-j0', $
Quality:        0.9, $
Type:          'Dusty large grey AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction)', $
Aim:           'Model the formation of dust clouds', $
Result:        '', $
Problems:      'Bug in the dust module', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n033', $
Status:        'Finished rackham-j1', $
Quality:        0.9, $
Type:          'Dusty non-grey AGB star model', $
Parameters:    'Added dust arrays (forsterite, corundum, no interaction)', $
Aim:           'Get non-grey dusty AGB-star model.', $
Result:        '', $
Problems:      'Bug in the dust module', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n034', $
Status:        'Finished rackham-j2', $
Quality:        0.4, $
Type:          'Dusty large non-grey AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction), from 056 on: MPI with 8 processes', $
Aim:           'Model the formation of dust clouds', $
Result:        '', $
Problems:      'Bug in the dust module', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n036', $
Status:        'Finished milou-jg (before: rackham-j0)', $
Quality:        0.4, $
Type:          'Grey AGB star model', $
Parameters:    '8 MPI processes, 20 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta', $
Aim:           'Test MPI, check for influence of opacity table', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham and milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n037', $
Status:        'Finshed milou-jg', $
Quality:        0.4, $
Type:          'Grey 1.5 M_Sun 10000 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; increase stellar mass and envelope mass gradually', $
Aim:           'Get higher-mass model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham and milou', $
ParModelIdent:  '_250', $
nFile:            250, $
mMeanAvg:        1300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n038', $
Status:        'Finished rackham-j2', $
Quality:        1.0, $
Type:          'Dusty large non-grey 1 M_Sun AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction), bugfix in dust module (over n034), 3 opacity bins', $
Aim:           'Model the formation of dust clouds', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n039', $
Status:        'Finished rackham-j2', $
Quality:        0.9, $
Type:          'Dusty large grey 1 M_Sun AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction)', $
Aim:           'Model the formation of dust clouds; check influence of grey opacities.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n040', $
Status:        'Finished rackham-j3', $
Quality:        0.4, $
Type:          'Dusty large non-grey 1 M_Sun AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction), 2 opacity bins', $
Aim:           'Check influence of number of opacity bins', $
Result:        '', $
Problems:      'Short run', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_040', $
nFile:             40, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n041', $
Status:        'Finished rackham-j4', $
Quality:        0.4, $
Type:          'Dusty large non-grey 1 M_Sun AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction), 4 opacity bins', $
Aim:           'Check influence of number of opacity bins', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_040', $
nFile:             40, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm06n042', $
Status:        'Finished rackham-j5', $
Quality:        0.4, $
Type:          'Dusty large non-grey 1 M_Sun AGB star model', $
Parameters:    'Extended model: 401^3 points, added dust arrays (forsterite, corundum, no interaction), 6 opacity bins', $
Aim:           'Check influence of number of opacity bins', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_040', $
nFile:             40, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n001', $
Status:        'Finished mil-ji', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n24_100.end. Luminosity=7000L_Sun -> 5000L_Sun', $
Aim:           'Decrease luminosity and radius.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n002', $
Status:        'Finished mil-jg', $
Quality:        1.0, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm05n001_040.end. Increase the mass slightly.', $
Aim:           'Increase the radius.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n003', $
Status:        'Finished milou-jd (before: rackham-j3)', $
Quality:        0.9, $
Type:          'Grey AGB star model', $
Parameters:    '8 MPI processes, 20/16 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta', $
Aim:           'Test MPI, check for influence of opacity table', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n004', $
Status:        'Finished milou-jc (before: rackham-j5)', $
Quality:        0.7, $
Type:          'Grey AGB star model', $
Parameters:    '8 MPI processes, 20 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta, decrease the core radius r0_grav', $
Aim:           'Check for influence of core size', $
Result:        'The initially large pulsation amplitude decreases. Still, at least 2 modes are visible in light curves oand radius variations.', $
Problems:      'Strong transient radial pulsations due to too short scaling interval', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n005', $
Status:        'Finished milou-jb', $
Quality:        0.9, $
Type:          'Grey AGB star model', $
Parameters:    '8 MPI processes, 16/8 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta, decrease the core radius r0_grav slowly', $
Aim:           'Check for influence of core size, try to avoid transients my making the transition interval longer', $
Result:        'Superpositon of at least two modes', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        1200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n006', $
Status:        'Finished milou-jd', $
Quality:        0.9, $
Type:          'Grey 1.5 M_Sun AGB star model', $
Parameters:    '8 MPI processes, 16/8 OpenMP threads; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta, increase stellar mass (1->1.5) and envelope mass slowly', $
Aim:           'Create AGB star with slightly higher mass', $
Result:        '', $
Problems:      'During the scaling phase, the entropy drops a lot and recovers only slowly. For this reason, the entropy adjustment was implemented and used in later models.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_350', $
nFile:            350, $
mMeanAvg:        4700, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n007', $
Status:        'Finished milou-jd', $
Quality:        0.0, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; increase stellar mass (1->1.5), envelope mass, and luminosity slowly', $
Aim:           'Create AGB star with slightly higher mass and luminosity', $
Result:        'Large-amplitude (plusminus 1500 L_Sun) radial mode(s)', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        1300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n008', $
Status:        'Finished milou-jb', $
Quality:        0.0, $
Type:          'Grey 1 M_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; increase the envelope mass gradually from n005', $
Aim:           'Get a compact model with lower effective temperature by slightly increasing the radius', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:        1600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n009', $
Status:        'Finished milou-jf', $
Quality:        0.5, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; from n007: hdCoreDragProfile: Linear->Linear-Radial, C_HpTopFator: 0.3->0.2', $
Aim:           'Check influence of boundary condition parameters on pulsations and mass loss', $
Result:        'Rather regular large-amplitude radial pulsations.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_250', $
nFile:            250, $
mMeanAvg:         900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n010', $
Status:        'Finished milou-jb', $
Quality:        0.0, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; from n007: hdCoreDragProfile: Linear->Constant-Radial-vTrans->Constant-All-vTrans', $
Aim:           'Check influence of inner boundary condition on pulsations and mass loss', $
Result:        '', $
Problems:      'Radial-only drag force leads to flow around the core. Drag force on all components works better but shows trend for bipolar flow.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        1720, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n011', $
Status:        'Finished milou-jd', $
Quality:        0.5, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; from n007: hdCoreDragProfile: Constant-All-vTrans; C_CoreDrag=4.0->1.5', $
Aim:           'Check influence of inner boundary condition on pulsations: Reduce C_CoreDrag', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        2200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n012', $
Status:        'Finished milou-jb', $
Quality:        0.8, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; from n011: hdCoreDragProfile: Constant-All-vTrans; C_CoreDrag=1.5->0.5', $
Aim:           'Check influence of inner boundary condition on pulsations: Reduce C_CoreDrag', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        1400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n013', $
Status:        'Finished milou-jc', $
Quality:        0.2, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; from n011: hdCoreDragProfile: Constant-All-vTrans; C_CoreDrag=1.5->0.0', $
Aim:           'Check influence of inner boundary condition on pulsations: Set C_CoreDrag to zero', $
Result:        'After a short time, a strong, persistent dipole flow develops. Radial pulsations seem to become even stronger than in n011.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n014', $
Status:        'Finished rackham-j0', $
Quality:        0.5, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n012: r^12/r^8 potential, C_gravCorePotDepth=1.0->2.0, envelope mass*1.37', $
Aim:           'Check influence of core mass on pulsations: Increase depth of gravitational potential in core', $
Result:        'The initially regular pulsations last but turn into the more irregular ones at nMean=1500.', $
Problems:      'Close to the center in layers with large temperature gradient, a region with non-negligable radiative luminosity develops.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        1500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n015', $
Status:        'Finished rackham-j1', $
Quality:        0.0, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n012: r^8/r^4 potential, C_gravCorePotDepth=1.0->1.275, increase r0_grav', $
Aim:           'Check influence of core mass on pulsations: Change potential formula without increasing the mass', $
Result:        '', $
Problems:      'Forgot to decrease luminositypervolume. Ended up with a blown-up 20000 L_sun object.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_090', $
nFile:             90, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n016', $
Status:        'Finished rackham-j0', $
Quality:        0.8, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n012: r^8/r^4 potential, C_gravCorePotDepth=1.0->1.275, increase r0_grav, decrease luminositypervolume', $
Aim:           'Check influence of core mass on pulsations: Change potential formula without increasing the mass', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n017', $
Status:        'Finished rackham-j1', $
Quality:        0.8, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n016: C_gravCorePotDepth=1.275->2.5, envelope mass*1.50471', $
Aim:           'Check influence of core mass on pulsations.', $
Result:        'Initially, a phase with large pulsation amplitude, followed by a phase with smaller amplitude, followed by a phase with large amplitude.', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n018', $
Status:        'Finished rackham-j2', $
Quality:        0.8, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n017: C_sChange=0.05->0.02->0.01', $
Aim:           'Check influence of core mass on pulsations. In particular, check influence of central entropy smoothing', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        2200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n019', $
Status:        'Finished rackham-j0', $
Quality:        0.8, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n018: C_gravCorer0Heat: 1.0->0.7', $
Aim:           'Check influence of mass of core heating region on pulsations.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n020', $
Status:        'Finished rackham-j1', $
Quality:        0.5, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n019: deeper core potential (2.5->7.0), larger envelope mass, larger core drag.', $
Aim:           'Check influence of core mass on pulsations.', $
Result:        '', $
Problems:      'The steep temperature gradient near the core causes a large radiative luminosity.', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:        2260, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n021', $
Status:        'Finished rackham-j2', $
Quality:        0.8, $
Type:          'Grey 1.5 M_Sun 7500 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n019: slightly deeper core potential (2.5->3.0), slightly larger envelope mass, larger core drag, larger visP2P.', $
Aim:           'Check influence of core mass on pulsations.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n022', $
Status:        'Finished rackham-j0', $
Quality:        0.9, $
Type:          'Grey 1 M_Sun 5000 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n008: changed core drag, larger visP2P, small core heating region.', $
Aim:           'Get model with standard settings but no deepened core.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_300', $
nFile:            300, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st28gm05n023', $
Status:        'Finished rackham-j1', $
Quality:        0.9, $
Type:          'Grey 1.5 M_Sun 7000 L_Sun AGB star model', $
Parameters:    'MPI+OpenMP; 317^3 points; Rosseland mean from non-grey opacity table t2800gm050mm00_coma_opal_paola_oc1rp_b3_n02.opta; compared to n021: smaller luminosity 7500 -> 7000 L_Sun', $
Aim:           'Get model with standard luminosity', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'rackham', $
ParModelIdent:  '_340', $
nFile:            340, $
mMeanAvg:        1300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st29gm06n001', $
Status:        'Finished mil-jb', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm06n26_090.end. M_env decreased by 0.22 dex.', $
Aim:           'Decrease mass to decrease the radius and increase Teff.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st29gm04n001', $
Status:        'Finished mil-ji', $
Quality:        0.9, $
Type:          'Dust-free AGB star model', $
Parameters:    'Start from st28gm05n001_100.end. M_env decreased by 0.11 dex.', $
Aim:           'Decrease mass to decrease the radius and increase Teff, slightly.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'Uppmax', $
Computer:      'milou', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st33gm06n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Mean file data not complete', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      'tacsg2', $
ParModelIdent:   '_21', $
nFile:             21, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st33gm06n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Mean file data not complete', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      'craSHi', $
ParModelIdent:   '_60', $
nFile:             60, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Betelgeuse model with realistic mass', $
Result:        '', $
Problems:      'Very short test run', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:   '_04', $
nFile:              4, $
mMeanAvg:           0, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'GRAAL', $
Computer:      'crow', $
ParModelIdent:   '_30', $
nFile:             30, $
mMeanAvg:         900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'GRAAL', $
Computer:      'PC-Freytag', $
ParModelIdent:   '_09', $
nFile:              9, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n04', $
Status:        'Finished', $
Quality:        0.7, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, finer grid than st35gm03n04', $
Aim:           'Better resolved model', $
Result:        '', $
Problems:      'Not completely relaxed: luminosity is still growing', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      '', $
ParModelIdent:   '_50', $
nFile:             50, $
mMeanAvg:         280, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n05', $
Status:        'Finished', $
Quality:        0.7, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, finer grid than st35gm03n04', $
Aim:           'Better resolved model', $
Result:        'Luminosity with oscillations but no trend', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'GRAAL', $
Computer:      'CINES', $
ParModelIdent:   '_30', $
nFile:             30, $
mMeanAvg:         120, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n06', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_80', $
nFile:             80, $
mMeanAvg:         760, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n07', $
Status:        'Finished', $
Quality:        0.9, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_161', $
nFile:            161, $
mMeanAvg:         660, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n08', $
Status:        'Finished', $
Quality:        0.5, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, further reduced envelope mass, higher resolution', $
Aim:           'Better resolved high-Teff model', $
Result:        '', $
Problems:      'Small time step, very short run', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:   '_21', $
nFile:             21, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n09', $
Status:        'Finished', $
Quality:        0.9, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, decreased luminosity', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:   '_80', $
nFile:             80, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n10', $
Status:        'Finished', $
Quality:        0.8, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, decreased luminosity', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_70', $
nFile:             70, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n11', $
Status:        'Finished', $
Quality:        0.9, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, decreased luminosity, higher resolution', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      '', $
ParModelIdent:  '_75', $
nFile:             75, $
mMeanAvg:         560, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n13', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global non-grey 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, non-grey opacities', $
Aim:           '', $
Result:        '', $
Problems:      'Sequence of mean files is not complete: gap near start. Resolution is low.', $
Author:        'Andrea Chiavassa, Bernd Freytag', $
Organization:  '', $
Computer:      'IBM Power 6', $
ParModelIdent:  '_708', $
nFile:            708, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n15', $
Status:        'Finished jb', $
Quality:        0.0, $
Type:          'Global grey 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, new code version', $
Aim:           'Test the new code version on old model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_008', $
nFile:              8, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n16', $
Status:        'Finished jc', $
Quality:        0.0, $
Type:          'Global grey 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, new code version', $
Aim:           'Test the new code version on old model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_080', $
nFile:             80, $
mMeanAvg:        1000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n17', $
Status:        'Finished jd', $
Quality:        0.0, $
Type:          'Global grey 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, relatively high resolution: 401^3, new code version', $
Aim:           'Test the new code version on high-res version of old model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_200', $
nFile:            200, $
mMeanAvg:         500, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n18', $
Status:        'Finished jb', $
Quality:        1.0, $
Type:          'Global grey 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, new code version', $
Aim:           'Further optimize parameters for new code version.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_090', $
nFile:             90, $
mMeanAvg:         900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm03n19', $
Status:        'Finished jd', $
Quality:        0.0, $
Type:          'Global grey 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun, relatively high resolution: 401^3, new code version', $
Aim:           'Test the new code version on high-res version of old model.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Last run with LHDrad', $
Aim:           '', $
Result:        '', $
Problems:      'Crash', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'craSHi', $
ParModelIdent:   '_50', $
nFile:             50, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'First run with SHORTrad, c_vissmagorinsky=1.2, c_visartificial=1.2', $
Aim:           'Get a stable simulation going', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_27', $
nFile:             27, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n03', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Smaller star with lower mass. Start with hydrostatic sphere and v/10 from st35gm04n02_06.full(n=5)', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_26', $
nFile:             26, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n04', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Start with st35gm04n03_09.end (127^3 -> 171^3)', $
Aim:           'Better resolved model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_38', $
nFile:             38, $
mMeanAvg:         200, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n05', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Prandtl Number: 8 -> 4', $
Aim:           'Check influence of Prandtl number', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'craSHi', $
ParModelIdent:   '_50', $
nFile:             50, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n06', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '127^3->171^3, VanLeer, c_vissmagorinsky=0.6, c_visartificial=0.6, Start with st35gm04n04_33.end', $
Aim:           'Lower viscosity model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_15', $
nFile:             15, $
mMeanAvg:         120, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n07', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'decreased c_schange, s_inflow corrected', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'CRAY SV1', $
ParModelIdent:   '_50', $
nFile:             50, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n08', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Lambda*', $
Aim:           'Accelerate radiation transport', $
Result:        '', $
Problems:      'Very short, no net acceleration', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_04', $
nFile:              4, $
mMeanAvg:         60, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n20', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Simple ray directions; bigfix: correct diffusion limit; radiative energy only >0 outside sphere with radius par%C_test1', $
Aim:           'Accelerate radiation transport', $
Result:        '', $
Problems:      'Very short', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_04', $
nFile:              4, $
mMeanAvg:         20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n21', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Start with st35gm04n06_12.end, SHORTrad, MinMod->VanLeer, extended SHORTrad operator, simple ray directions, correct diffusion limit, Bugfix in Roe1dflux: computation of dxb1_avg & dxb1_var, decreased tensor viscosity', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_13', $
nFile:             13, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n22', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Start with st35gm04n21_06.end, simple ray directions, correct diffusion limit, decreased luminosity, radiation transport radius included, decreased tensor viscosity, check minimum internal energy for allowed range, slope reduction in case of strong density gradient', $
Aim:           '', $
Result:        '', $
Problems:      'Growing global oscillations.', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_42', $
nFile:             42, $
mMeanAvg:          80, $
nMeanAvg:         319, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n23', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity and magnetic fields', $
Parameters:    'Some kind of kinetic approximation', $
Aim:           'Test-run with "magnetic fields"', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_07', $
nFile:              7, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n24', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_13', $
nFile:             13, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n25', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_27', $
nFile:             27, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n26', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_75', $
nFile:             75, $
mMeanAvg:         784, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n27', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Low resolution run with little braking', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_08', $
nFile:              8, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n28', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'HP-UX gunnar', $
ParModelIdent:   '_55', $
nFile:             55, $
mMeanAvg:         920, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n29', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    'Start with st35gm04n26_55.end: 171^3 -> 235^3', $
Aim:           'Highest resolution so far', $
Result:        'Good model', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_30', $
nFile:             30, $
mMeanAvg:         164, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n30', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'crow', $
ParModelIdent:   '_20', $
nFile:             20, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n31', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'crow', $
ParModelIdent:   '_40', $
nFile:             40, $
mMeanAvg:         160, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n32', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_15', $
nFile:             15, $
mMeanAvg:         300, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n33', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:   '_26', $
nFile:             26, $
mMeanAvg:         164, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n34', $
Status:        'Finished', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '315^3 grid points', $
Aim:           'High-resolution model', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'CINES IBM', $
ParModelIdent:   '_36', $
nFile:             36, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

;{ $
;Model:         'st35gm04n35', $
;Status:        'Planned', $
;Quality:        0.0, $
;Type:          'Global 3D Betelgeuse model with low surface gravity', $
;Parameters:    '4xx^3 grid points', $
;Aim:           'Super-high-resolution model', $
;Result:        '', $
;Problems:      '', $
;Author:        'Bernd Freytag', $
;Organization:  '', $
;Computer:      '', $
;nFile:              0, $
;ParModelIdent:   '_01', $
;mMeanAvg:          00, $
;nMeanAvg:       99999 $
;}, $

{ $
Model:         'st35gm04n36', $
Status:        'Finished ja', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 235^3, start with st35gm04n29_30.end; new code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      'Without sufficient core drag force there are large velocities in the center.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_090', $
nFile:             90, $
mMeanAvg:         900, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04n37', $
Status:        'Finished jc', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 315^3, new code version 2012-11-05: CTU, CA4, FRweno, hdEnthalpyAvgMode=CA1gMps, hdEntropyWaveMode=Entropy3gMps', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_180', $
nFile:            180, $
mMeanAvg:         800, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b0n001', $
Status:        'Finished ja', $
Quality:        1.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 315^3, new code version 2012-11-05: HLLMHD, Hancock, unsplit, FRmono', $
Aim:           'Check new code version', $
Result:        '', $
Problems:      'mean files corrupt until 033 inclusively, xb/c3_volume and _surface axes faulty.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_100', $
nFile:            100, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b0n002', $
Status:        'Finished jc', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '5 M_Sun; 315^3, new code version 2013-08-07: HLLMHD, Hancock, 123, FRweno', $
Aim:           'Check new code version; get non-magnetic reference model for b1n003.', $
Result:        '', $
Problems:      '', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_082', $
nFile:             82, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b1n001', $
Status:        'Finished jb', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity and magnetic field', $
Parameters:    '5 M_Sun; 315^3, new code version 2013-03: HLLMHD, Hancock, unsplit, FRmono', $
Aim:           'Create global RSG dynamo', $
Result:        '', $
Problems:      'Fatal crash with high-amplitude 3D-checkerboard pattern in Bb. mean files: xb/c3_surface axes faulty.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_446', $
nFile:            446, $
mMeanAvg:        3000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b1n002', $
Status:        'Finished jf', $
Quality:        0.5, $
Type:          'Global 3D Betelgeuse model with low surface gravity and magnetic field', $
Parameters:    '5 M_Sun; 315^3, new code version 2013-03: HLLMHD, Hancock, unsplit, FRweno', $
Aim:           'Create global RSG dynamo, check influence of reconstruction', $
Result:        'Run finishes without final crash. After growing by about 10^5 the magnetic field energy saturates at about 5% of the total kinetic energy but seems to drop slightly at the very end (by less than a factor 2).', $
Problems:      'mean files: xb/c3_surface axes faulty.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_999', $
nFile:            999, $
mMeanAvg:        4100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b1n003', $
Status:        'Finished jb', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity and magnetic field', $
Parameters:    '5 M_Sun; 315^3, new code version 2013-03: HLLMHD, Hancock, 123, FRweno', $
Aim:           'Create global RSG dynamo, check 123 vs. unsplit', $
Result:        '', $
Problems:      'Fatal crash with high-amplitude 3D-checkerboard pattern in Bb, which is already visible in _500.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_606', $
nFile:            606, $
mMeanAvg:        2000, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b1n004', $
Status:        'Finished ja', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity and magnetic field', $
Parameters:    '5 M_Sun; 315^3, new code version 2013-08-07: HLLMHD, Hancock, 123, FRweno, va_max: 100.0E+05 -> 200.0E+05 compared to n003', $
Aim:           'Check influence of va_max parameter, check performance on new cluster.', $
Result:        '', $
Problems:      'Fatal crash with high-amplitude 3D-checkerboard pattern in Bb.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '141', $
nFile:            141, $
mMeanAvg:         600, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm04b1n005', $
Status:        'Finished jf', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity and magnetic field', $
Parameters:    '5 M_Sun; 315^3, new code version 2013-10-29: HLLMHD, Hancock, unsplit, FRweno', $
Aim:           'Continue run st35gm04b1n002, starting with _990. Check final state of magnetic-field configuration', $
Result:        'The total magnetic-field energy seems to be saturated, with only small fluctuations.', $
Problems:      'Fatal crash with high-amplitude 3D-checkerboard pattern in Bb.', $
Author:        'Bernd Freytag', $
Organization:  'CRAL', $
Computer:      'PSMN', $
ParModelIdent:  '_184', $
nFile:            184, $
mMeanAvg:         800, $
nMeanAvg:        2600, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm06n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Mean file data not complete, very short run, time sequence not monotonic: use only 02,03', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'penguin', $
ParModelIdent:   '_03', $
nFile:              3, $
mMeanAvg:          20, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st35gm06n02', $
Status:        'Finished', $
Quality:        0.3, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '', $
Aim:           '', $
Result:        '', $
Problems:      'Mean file data not complete', $
Author:        'Bernd Freytag', $
Organization:  'UAO', $
Computer:      'craSHi', $
ParModelIdent:   '_20', $
nFile:             20, $
mMeanAvg:         100, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st40gm03n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Higher Teff', $
Result:        '', $
Problems:      'Mean file 04 empty, small time steps, not relaxed', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:  '_17', $
nFile:             17, $
mMeanAvg:          80, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st40gm03n02', $
Status:        'Finished', $
Quality:        0.2, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Higher Teff', $
Result:        'Shallow convection zone', $
Problems:      'Small time steps, not relaxed, model still growing', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:  '_39', $
nFile:             39, $
mMeanAvg:         480, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st40gm03n03', $
Status:        'Finished', $
Quality:        0.2, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Higher Teff', $
Result:        'Shallow convection zone', $
Problems:      'Small time steps, not relaxed, model still growing, increasing luminosity', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:  '_22', $
nFile:             22, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st40gm03n04', $
Status:        'Finished', $
Quality:        0.2, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Higher Teff', $
Result:        'Shallow convection zone', $
Problems:      'Small time steps, not relaxed, decreasing luminosity', $
Author:        'Bernd Freytag', $
Organization:  '', $
Computer:      'Uppmax Aangstroem Cluster SunFire', $
ParModelIdent:  '_99', $
nFile:             99, $
mMeanAvg:         400, $
nMeanAvg:       99999, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st45gm03n01', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Much higher Teff', $
Result:        '', $
Problems:      'Very small time step, star grows', $
Author:        'Bernd Freytag', $
Organization:  'GRAAL', $
Computer:      'PC-Freytag', $
ParModelIdent:   '_09', $
nFile:              9, $
mMeanAvg:         100, $
nMeanAvg:         199, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
}, $

{ $
Model:         'st45gm03n02', $
Status:        'Finished', $
Quality:        0.0, $
Type:          'Global 3D Betelgeuse model with low surface gravity', $
Parameters:    '12 M_Sun', $
Aim:           'Much higher Teff', $
Result:        '', $
Problems:      'Very small time step, star grows', $
Author:        'Bernd Freytag', $
Organization:  'GRAAL', $
Computer:      'PC-Freytag', $
ParModelIdent:   '_07', $
nFile:              7, $
mMeanAvg:         100, $
nMeanAvg:         199, $
mTimeAvg:         0.0, $
nTimeAvg:         0.0 $
} $

]

ModelTable=[ModelTable_BerndFreytag, $
            ModelTable_FranceAllard, $
            ModelTable_AndreaChiavassa, $
            ModelTable_AstridWachter, $
            ModelTable_SusanneHoefner, $
            ModelTable_SebastianWende, $
            ModelTable_WernerSchaffenberger, $
            ModelTable_MatthiasSteffen, $
            ModelTable_OskarSteiner, $
            ModelTable_PierEmmanuelTremblay, $
            ModelTable_SvenWedemeyerBoehm, $
            ModelTable_CIFIST]

end ; rhdmodeldata
