!***************************************************************************************************
!
!   #    # #  ####        #####  #    # #      #    # 
!   #    # # #    #       #    # #    # #      #   #  
!   #    # # #    #       #####  #    # #      ####   
!   #    # # #    #       #    # #    # #      #  #   
!   #    # # #    #       #    # #    # #      #   #  
!    ####  #  #### ###### #####   ####  ###### #    #_module 
!
!   Universal Input/Output Routines: main (bulk) part
!
!***************************************************************************************************
!   This set of input and output routines allows the reading and writing of scalar, array, and
!   table data in header-data units in formatted or unformatted Fortran file format.
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag: Kiel, Copenhagen, Lyon, Uppsala
!***************************************************************************************************
!   Fortran90
!   2016-03-16
!***************************************************************************************************
! MODULES:
!   uio_bulk_module:       Main set of UIO routines
!
!***************************************************************************************************


!------***************------------------------------------------------------------------------------
module uio_bulk_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_bulk_module ('uio_bulk_module')
!
! PURPOSE:
!   Contain main (bulk) part of uio-routines.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_bulk_module
!
! ROUTINES: (CONTAINED)
!   --- Initialization ---
!   uio_init:              Initialization of UIO routines
!   uio_chpos:             Give current file position or jump to specified position
!   --- Open/Close ---
!   uio_openwr:            Open file for writing, write header
!   uio_closwr:            Close file after writing
!   uio_openrd:            Open file for reading, read header
!   uio_closrd:            Close file after reading
!   --- High level writing ---
!   uio_wrfifo:            Write file header
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uio_wrlabl:            Write label
!   --- High level reading ---
!   uio_rdhd:              Read header of variable and extract keywords
!   uio_skipda:            Skip data block
!   uio_rdfifo:            Read file header
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_rdlabl:            Read label
!   --- Useful high level routines ---
!   uio_filcon:            Determine file contents: list of all entries with its positions
!   uio_srhd:              Search entry header in file or in entry list
!   --- Lower level routines ---
!   uio_wrhdln:            Write header of variable, input: line table
!   uio_wrhd:              Write header of variable, input: term table
!   uio_rdhdln:            Read header, output: line table
!
! MODULES:
!   uio_mac_module:        Set of (possibly) machine-dependent UIO routines
!
! MODIFICATION HISTORY:
!   1997-01-13 Routines put into module by Bernd Freytag, Astrophysik Kiel
!   1997-02-10 (B.F.) Automatic routine expansion
!   2005-02-21 (J.L.) new subroutine uio_wrd5
!   2006-10-23 (B.F. Lyon) Check for trivial array bounds and use special branch if possible.
!                     This trick saves stack space: useful on Macs.
!                     It is implemented only for rdr3/4 and wrr3/4 so far.
!   2013-08-24 (B.F.) Change layout of comments
!---------------------------------------------------------------------------------------------------
use uio_mac_module
!
interface uio_wr
  module procedure uio_wrr, uio_wrr1, uio_wrr2, uio_wrr3, uio_wrr4, &
                   uio_wrd, uio_wrd1, uio_wrd2, uio_wrd3, uio_wrd4, uio_wrd5, &
                   uio_wri, uio_wri1, uio_wri2, uio_wri3, uio_wri4, &
                   uio_wrc, uio_wrc1, uio_wrc2, uio_wrc3, uio_wrc4, &
                   uio_wrs, uio_wrs1, uio_wrs2, uio_wrs3, uio_wrs4
end interface ! uio_wr
!
interface uio_rd
  module procedure uio_rdr, uio_rdr1, uio_rdr2, uio_rdr3, uio_rdr4, &
                   uio_rdd, uio_rdd1, uio_rdd2, uio_rdd3, uio_rdd4, &
                   uio_rdi, uio_rdi1, uio_rdi2, uio_rdi3, uio_rdi4, &
                   uio_rdc, uio_rdc1, uio_rdc2, uio_rdc3, uio_rdc4, &
                   uio_rds, uio_rds1, uio_rds2, uio_rds3, uio_rds4
end interface ! uio_rd
!
private uio_wrr, uio_wrr1, uio_wrr2, uio_wrr3, uio_wrr4, &
        uio_wrd, uio_wrd1, uio_wrd2, uio_wrd3, uio_wrd4, uio_wrd5, &
        uio_wri, uio_wri1, uio_wri2, uio_wri3, uio_wri4, &
        uio_wrc, uio_wrc1, uio_wrc2, uio_wrc3, uio_wrc4, &
        uio_wrs, uio_wrs1, uio_wrs2, uio_wrs3, uio_wrs4
!
private uio_rdr, uio_rdr1, uio_rdr2, uio_rdr3, uio_rdr4, &
        uio_rdd, uio_rdd1, uio_rdd2, uio_rdd3, uio_rdd4, &
        uio_rdi, uio_rdi1, uio_rdi2, uio_rdi3, uio_rdi4, &
        uio_rdc, uio_rdc1, uio_rdc2, uio_rdc3, uio_rdc4, &
        uio_rds, uio_rds1, uio_rds2, uio_rds3, uio_rds4
!
contains

!----------********---------------------------------------------------------------------------------
subroutine uio_init(progrm, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_init ('uio_initialize')
!
! PURPOSE:
!   Initialization procedure for input/output routines.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_init(progrm=progrm, outstr=outstr,ierr=ierr)
!
! INPUT:
!   progrm: ('program_name') character, optional, name of main program
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chconv:            Actualize list of conversion types for all channels
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_mkcvls:            Make list with possible conversion types
!
! MODULES:
!   uio_inf_module:        Information about environment
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Variables in the module uio_inf_module are changed. Subroutines change
!   variables in the modules uio_cst_module and uio_cvl_module.
!
! RESTRICTIONS:
!   Don't use the reserved channel units (10 to 80) without uio_init and
!   uio_chunit.
!
! PROCEDURE:
!   Call uio_mkcvls, uio_chunit, and uio_chconv:
!   The program name is saved in global variable in module uio_inf_module.
!   The routine uio_mkcvls creates the conversion type list appropriate for the
!   current machine and sets default write formats for numbers.
!   With uio_chunit and uio_chconv a list of I/O channels (10 to 80) is
!   reserved as usable by uio-routines.
!   Channel 5 is ready for formatted input,
!   channel for formatted output.
!
! EXAMPLE:
!   call uio_init('FORTRANtest', outstr,ierr)
!
! MODIFICATION HISTORY:
!   1995-12-27 (B.F. Kiel) Written
!   1996-02-19 (B.F.) Reserve channels 5/6 for formatted input/output
!   1996-02-21 (B.F.) Call uio_chconv for channels 5/6
!   1996-10-28 (B.F.) More comments
!   1997-01-13 (B.F.) Call uio_chunit with variables
!   1997-01-17 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err.
!   1997-02-19 (B.F.) Channel 5 and 6: conv='native'
!   1997-05-20 (B.F.) "use uio_siz_module" removed
!---------------------------------------------------------------------------------------------------
use uio_inf_module
!
implicit none
!
! --- I/O parameters ---
character*(*), optional, intent(in)     :: progrm
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
!
!---------------------------------------------------------------------------------------------------
!
! --- Initialize error handling: maximum output, sensitive error handling ---
call uio_err(mode=1, outlevel=10,errlevel=1)
!
! --- Save program name in variable in module ---
if (present(progrm)) then
  progr0=progrm
else
  progr0=''
endif
!
! --- Set default write formats for numbers, create conversion list ---
call uio_mkcvls()
!
! --- Initialize list of free unit numbers: reserve units 10 to 80 ---
call uio_chunit('init',     nc1in=10,nc2in=80)
!
! --- Prepare channels 5 and 6 as formatted standard input/output ---
call uio_chunit('setrange', nc1in=5, nc2in=6)
call uio_chunit('set',      nc1in=5)
call uio_chunit('formatted',nc1in=5)
call uio_chconv(5, conv='native')
call uio_chunit('set',      nc1in=6)
call uio_chunit('formatted',nc1in=6)
call uio_chconv(6, conv='native')
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_init


!----------*********--------------------------------------------------------------------------------
subroutine uio_chpos(nc,mode,posin,posout, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_chpos ('uio_channel_position')
!
! PURPOSE:
!   Give current file position or jump to specified position.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_chpos(nc,mode=mode,posin=posin,posout=posout, &
!                  outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, number of channel
!   mode:   ('mode') character, optional:
!                    'get':    give current file position in pos, default
!                    'goto':   jump to file position pos
!                    'rewind': rewind file
!   posin:  ('position_input') integer, optional, file counter position
!
! OUTPUT:
!   posout: ('position_output') integer, optional, file counter position
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!
! MODULES:
!   uio_cst_module:        Channel status information
!
! SIDE EFFECTS:
!   Change file pointer position. Modify variables in module uio_cst_module.
!
! RESTRICTIONS:
!   Choose the file position to jump to carefully. Not every possible number
!   is an reasonable file position. It is always save to jump to a position
!   previously derived with 'get'.
!
! PROCEDURE:
!   mode='get':    Give out file counter position in ncpos(nc) in module
!                  uio_cst_module.
!   mode='goto':   Compare current (ncpos(nc)) and desired (pos) file counter
!                  position. Then go forward (read statements) or backward
!                  (backspace statement).
!   mode='rewind': Rewind file. Set file counter to zero.
!
! EXAMPLE:
! --- Open file ---
!       call uio_openrd(nc,file,form,conv,date, &
!                       system,machin,osrels,osvers,hrdwar,langua, &
!                       progrm, termt,ntt, outstr,ierr)
! --- Get file position ---
!       call uio_chpos(nc,mode='get',posout=pos, outstr=outstr,ierr=ierr)
!       ... Read entries ...
! --- got back to start position ---
!       call uio_chpos(nc,mode='goto',posin=pos, outstr=outstr,ierr=ierr)
!
! MODIFICATION HISTORY:
!   1996-10-21 (B.F. Kiel) Written
!   1996-11-27 (B.F.) Save current position after jumping
!   1997-01-20 (B.F.) Parameters made optional
!---------------------------------------------------------------------------------------------------
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
character*(*), optional, intent(in)     :: mode
integer, optional, intent(in)           :: posin
integer, optional, intent(out)          :: posout
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: p, ierr0
character*(80)                          :: form, mode0, outstr0
!---------------------------------------------------------------------------------------------------
! --- Default values for optional parameter ---
if (present(mode)) then
  mode0=mode
else
  mode0='get'
endif
!
outstr0='done'
ierr0=0
if ((mode0 == 'goto') .and. (present(posin))) then
  ! --- Go to specified file position ---
  !
  ! --- Get channel unit ---
  call uio_chunit('ask', nc1in=nc, outstr=form)
  !
  ! --- Jump ---
  if (posin > ncpos(nc)) then
    ! --- Go forward ---
    if (form == 'unformatted') then
      ! --- Unformatted file ---
      do p=ncpos(nc),posin-1
        read(nc)
      end do ! p
    else
      ! --- Formatted file ---
      do p=ncpos(nc),posin-1
        read(nc,'(A1)')
      end do ! p
    endif
  else if (posin < ncpos(nc)) then
    ! --- Go backward ---
    do p=posin,ncpos(nc)-1
      backspace nc
    end do ! p
  endif
  ncpos(nc)=posin
else if ((mode0 == 'get') .and. (present(posout))) then
  ! --- Give current file position ---
  posout=ncpos(nc)
else if (mode0 == 'rewind') then
  ! --- Rewind file ---
  rewind(nc)
  ncpos(nc)=0
else
  outstr0='error in command specification'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_chpos


!----------**********-------------------------------------------------------------------------------
subroutine uio_openwr(nc,file,form,conv,position,termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_openwr ('uio_open_write')
!
! PURPOSE:
!   Open a file for writing, write header.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_openwr(nc,file, form=form,conv=conv, position=position, &
!                   termt=termt,ntt=ntt, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   file:   ('file_name') character, filename
!   form:   ('file_form') character, optional, 'formatted' or 'unformatted'
!   conv:   ('convert') character, optional, type of conversion procedure
!   position:('position') character, optional, 'rewind', 'append', 'asis'
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
!           optional
!   ntt:    ('number_term_table') integer, number of non-empty lines of
!           termt (ntt >= 0), optional
!
! OUTPUT:
!   nc:     ('number_channel') integer, channel unit number
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chconv:            Actualize list of conversion types for all channels
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_uopen:             Open a file with special handling for conversion type
!   uio_wrfifo:            Write file header
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   A new file is created and opened for writing. The format and
!   conversion type are stored in global variables in modules.
!   The file position counter is set to zero.
!
! RESTRICTIONS:
!   If position='append' the file is opened and positioned at the end,
!   but the internal file position counter is set to zero.
!   Otherwise an old file is overwritten and its contents is deleted.
!
! PROCEDURE:
!   Save form of file and conversion type, open file and write file header.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-16 (B.F. Kiel) Written
!   1995-11-23 (B.F.) Routine uio_getenv included
!   1995-12-05 (B.F.) Additional keywords
!   1995-12-06 (B.F.) 'uiosiz.inc' included
!   1995-12-07 (B.F.) adkeyX used to construct header
!   1995-12-10 (B.F.) assign
!   1995-12-25 (B.F.) New argument: conv, 2 more standard keywords, asnunit
!   1996-01-10 (B.F.) uio_wrfifo used
!   1996-10-19 (B.F.) Call uio_wrfifo without ouconv
!   1996-10-21 (B.F.) Renaming, file position counter
!   1997-01-17 (B.F.) Parameters made optional
!   1997-02-18 (B.F.) form='formatted' -> conv='native'
!   1997-02-19 (B.F.) form='formatted' -> conv='native' (?)
!   1999-05-12 (B.F. Copenhagen) Keyword position added
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(out)                    :: nc
character*(*), intent(in)               :: file
character*(*), optional, intent(in)     :: form, conv, position
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer, optional, intent(in)           :: ntt
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: ntt0
character*(let)                         :: termt0(2,nttmx)
character*(let)                         :: form0, conv0, position0
character*(80)                          :: ouconv
logical                                 :: exist
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
!
! --- Default values for optional position parameter ---
if (present(position)) then
  position0=position
else
  position0='asis'
endif
!
if (position0 == 'append') then
  ! --- Check if file exists ---
  inquire(file=file, exist=exist)
  if (.not. exist) position0='rewind'
endif
!
if (position0 == 'append') then
  ! --- File exists, data should be appended ---
  ! --- Open file for reading to determine file format and conversion type ---
  call uio_openrd(nc,file, form=form0,conv=conv0, outstr=outstr0,ierr=ierr0)
  call uio_closrd(nc)
  if (ierr0 > 0) goto 900
  ntt0=0
else
  ! --- Default values for optional parameter ---
  if (present(form)) then
    form0=form
  else
    form0='formatted'
  endif
  if (present(conv)) then
    conv0=conv
  else
    conv0=''
  endif
  if (present(termt)) then
    termt0=termt
    if (present(ntt)) then
      ntt0=ntt
    else
      ntt0=nttmx
    endif
  else
    ntt0=0
  endif
endif
!
! --- Get channel unit ---
call uio_chunit('get',nc1out=nc)
!
! --- Save form of file ---
call uio_chunit(form0,nc1in=nc)
!
! --- Save conversion type ---
call uio_chconv(nc, conv=conv0,ouconv=ouconv)
!
! --- Set file position counter to zero ---
ncpos(nc)=0
!
! --- Open file on given channel with specified format ---
call uio_uopen(nc,file, form=form0, conv=ouconv, status='unknown', position=position0)
!
if (position0 /= 'append') then
  ! --- Write file header ---
  call uio_wrfifo(nc, termt=termt0,ntt=ntt0, outstr=outstr0,ierr=ierr0)
endif
!
900 continue
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_openwr


!----------**********-------------------------------------------------------------------------------
subroutine uio_closwr(nc, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_closwr ('uio_close_write')
!
! PURPOSE:
!   Close file after writing.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_closwr(nc, outstr=outstr,outstr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_uclose:            Close file with special handling for conversion type
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   A file on the specified channel unit is closed. The channel unit status
!   is set to 'free' and saved.
!
! RESTRICTIONS:
!   Identical to uio_closrd.
!
! PROCEDURE:
!   Close file (with appropriate conversion handling) and free channel unit.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-17 (B.F. Kiel) Written
!   1995-11-23 (B.F.) Routine uio_chunit included
!   1995-12-10 (B.F.) assign
!   1995-12-25 (B.F.) asnunit
!   1995-12-28 (B.F.) call uio_uclose
!   1997-01-17 (B.F.) Parameters outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
integer, optional, intent(out)          :: ierr
character*(*), optional, intent(out)    :: outstr
!
! --- Local variables ---
!
!---------------------------------------------------------------------------------------------------
! --- Close file ---
call uio_uclose(nc)
!
! --- Return channel unit ---
call uio_chunit('return', nc1in=nc)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_closwr


!----------**********-------------------------------------------------------------------------------
subroutine uio_openrd(nc,file, form,conv,date, &
                      system,machin,osrels,osvers,hrdwar,langua, &
                      progrm, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_openrd ('uio_open_read')
!
! PURPOSE:
!   Open a file for reading, read header.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_openrd(nc,file, form=form,conv=conv,date=date, &
!                   system=system,machin=machin,osrels=osrels,osvers=osvers, &
!                   hrdwar=hrdwar,langua=langua, progrm=progrm, &
!                   termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   file:   ('file_name') character, filename
!
! OUTPUT:
!   nc:     ('number_channel') integer, channel unit number
!   form:   ('format') character, 'formatted' or 'unformatted'
!   conv:   ('conversion') character, optional, conversion type
!   date:   ('date') character, optional, date information
!   system: ('system') character, optional, operating system
!                                                   (e.g. SunOS, craSH        )
!   machin: ('machine') character, optional, machine name
!                                                   (e.g. venus, kueste, craSH)
!   osrels: ('operating_system_release') character, operating system release
!           optional                                (e.g. 4.1.3, 8.0.3.0      )
!   osvers: ('operating_system_version') character, operating system version
!           optional                                (e.g. 1,     awa.5        )
!   langua: ('language') character, language of program that wrote file
!           optional                                (e.g. 'FORTRAN77'         )
!   hrdwar: ('hardware') character, empty or machine hardware
!           optional                                (e.g. sun4m, 'CRAY Y-MP'  )
!   progrm: ('program_name') character, optional, name of main program that
!           wrote file
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
!           optional
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt,
!           optional
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chconv:            Actualize list of conversion types for all channels
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_rdfifo:            Read file header
!   uio_rdhd:              Read header of variable and extract keywords
!   uio_uclose:            Close file with special handling for conversion type
!   uio_uopen:             Open a file with special handling for conversion type
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   A file is opened. Its status is stored.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Try to identify format and conversion type of file from first line.
!   Open file with correct format ('formatted' or 'unformatted'). Save
!   conversion type in variable in module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-17 (B.F. Kiel) Written
!   1995-11-23 (B.F.) Routine uio_chunit included
!   1995-12-06 (B.F.) 'uiosiz.inc' included
!   1995-12-08 (B.F.) Output of termt
!   1995-12-28 (B.F.) Conversion type
!   1996-01-03 (B.F.) conv extracted
!   1996-01-05 (B.F.) uio_rdhdln -> uio_rdhd
!   1996-01-10 (B.F.) uio_rdfifo
!   1996-01-30 (B.F.) Parameter langua added
!   1996-02-21 (B.F.) Save conversion type also if form='formatted'
!   1996-09-22 (B.F.) ierr, outstr at end
!   1996-10-21 (B.F.) File position counter
!   1997-01-03 (B.F.) Removable gotos removed
!   1997-01-17 (B.F.) Parameters made optional
!   1997-02-17 (B.F.) termtt introduced because termt may be not present
!   1997-02-19 (B.F.) Determine conversion type even for 'formatted' file
!   1997-02-22 (B.F.) Error handling by uio_err
!   1997-02-23 (B.F.) ierr0 is always set; inquire file before opening
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(out)                    :: nc
character*(*), intent(in)               :: file
character*(*), optional, intent(out)    :: form, conv, date, &
                                           system,machin,osrels,osvers, &
                                           hrdwar,langua,progrm
character*(*), optional, intent(out)    :: termt(2,nttmx)
integer, optional, intent(out)          :: ntt
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
logical                                 :: exist
integer                                 :: n, recog, nttt, ierr0, ierr1
character*(lel)                         :: line
character*(80)                          :: outstr0, form0, conv0, &
                                           sequential, read
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
recog=0
line=''
!
! --- Get channel unit ---
call uio_chunit('get', nc1out=nc)
!
! --- Check if file exists ---
inquire(file=file, exist=exist, sequential=sequential, read=read)
if (.not. exist) then
  outstr0='file ''' // trim(file) // ''' does not exist'
  ierr0=1
  goto 900
else if (sequential == 'NO') then
  outstr0='file ''' // trim(file) // ''' does not allow sequential access'
  ierr0=2
  goto 900
else if (read == 'NO') then
  outstr0='file ''' // trim(file) // ''' does not allow reading'
  ierr0=3
  goto 900
endif
!
! --- Open file 'formatted' and read first line ---
call uio_uopen(nc,file,form='formatted',status='old')
read(nc,'(A)',err=10,end=10) line
goto 20
!
! --- File definitely not 'formatted' ---
10 recog=-1
!
20 call uio_uclose(nc)
!
if (recog >= 0) then
  ! --- File possibly 'formatted' ---
  ! --- Test if there is a readable format description in line ---
  if (index(line,formna // equchr // 'formatted') > 0) then
    ! --- File definitely 'formatted' ---
    recog=1
    form0='formatted'
    outstr0='known format'
    ierr0=0
  endif
endif
!
if (recog /= 1) then
  ! --- File format not recognized yet ---
  ! --- Loop over all conversion types ---
  do n=1,ncv
    recog=0
    ! --- Open file 'unformatted' and read first line ---
    call uio_uopen(nc,file,form='unformatted',conv=cvnals(n),status='old')
    read(nc,err=30,end=30) line
    goto 40
    !
    ! --- File format not recognized yet ---
    30 recog=-1
    !
    40 call uio_uclose(nc)
    ! --- Test if there is format description in line ---
    if (recog >= 0) then
      ! --- File format possibly 'unformatted' with current conversion type ---
      if (index(line,formna//equchr//'unformatted') > 0) then
        ! --- File is indeed 'unformatted' ---
        recog=1
        form0='unformatted'
        outstr0='known format'
        ierr0=0
        exit
      else
        ! --- Unknown format causes open statement to crash ---
        form0='unknown'
        outstr0='unknown format'
        ierr0=1
      endif
    else
      ! --- Unknown format causes open statement to crash ---
      form0='unknown'
      outstr0='unknown format'
      ierr0=1
    endif
  end do ! n
endif
!
! --- Find and save conversion type ---
!if (form0 == 'unformatted') then
!  ! --- File is 'unformatted' but conversion type is still unknown ---
call uio_chconv(nc,conv=line,prefix=trim(convna)//equchr, ouconv=conv0, &
                ierr=ierr1)
if (ierr1 > 0) then
  outstr0='Conversion type not found'
  if (form0 == 'unformatted') call uio_err(outstr=outstr0,ierr=ierr1,nc=nc, &
                                           routine='uio_chconv', errlevel=2)
endif
!else
!  ! --- A 'formatted' file has no/'native' (?) conversion type ---
!  call uio_chconv(nc,conv='native')
!endif
!
! write(6,*) 'Open ', trim(form0), ' file ''', &
!            trim(file), ''' on channel ', nc, ' for reading'
! --- Open file on given channel with specified format and conversion type ---
call uio_uopen(nc,file, form=form0,conv=conv0,status='old')
!
! --- Set file position counter to zero ---
ncpos(nc)=0
!
! --- Save form of file ---
call uio_chunit(form0, nc1in=nc)
!
! --- Read file header completely ---
call uio_rdhd(nc, termtt,nttt)
!
! --- Find keywords and extract their values ---
call uio_rdfifo(nc,termtt,nttt, form=form,conv=conv,date=date, &
                system=system,machin=machin,osrels=osrels,osvers=osvers, &
                hrdwar=hrdwar,langua=langua,progrm=progrm)
!
if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
if (present(ntt)) ntt=nttt
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_openrd', errlevel=3)
!
end subroutine uio_openrd


!----------**********-------------------------------------------------------------------------------
subroutine uio_closrd(nc, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_closrd ('uio_close_read')
!
! PURPOSE:
!   Close a file after reading.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_closrd(nc, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_uclose:            Close file with special handling for conversion type
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Identical to uio_closwr.
!
! PROCEDURE:
!   Close file (with appropriate conversion handling) and free channel.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-17 (B.F. Kiel) Written
!   1995-11-23 (B.F.) Routine uio_chunit included
!   1996-10-14 (B.F.) More comments
!   1997-01-17 (B.F.) outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
integer, optional, intent(out)          :: ierr
character*(*), optional, intent(out)    :: outstr
!
! --- Local variables ---
!
!---------------------------------------------------------------------------------------------------
! --- Close file ---
call uio_uclose(nc)
!
! --- Return unit number ---
call uio_chunit('return', nc1in=nc)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_closrd


!----------**********-------------------------------------------------------------------------------
subroutine uio_wrfifo(nc,termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrfifo ('uio_write_fileform')
!
! PURPOSE:
!   Write file header.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrfifo(nc,termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
!           optional
!   ntt:    ('number_term_table') integer, number of non-empty lines of
!           termt (ntt >= 0), optional
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, contains exit error status, optional
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_getenv:            Get information about environment
!   uio_wrhd:              Write header of variable, input: term table
!   uio_dattim:            Get string with current date and time
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-10 (B.F. Kiel) Written
!   1996-01-12 (B.F.) Correction: ouconv -> conv
!   1996-10-19 (B.F.) conversion type extracted from common block: parameter conv removed
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-11 (B.F.) date not provided from uio_getenv but from uio_dattim
!   1997-01-17 (B.F.) Parameter made optional
!   1997-02-02 (B.F.) uio_getenv with optional parameters
!   1997-02-19 (B.F.) Write conversion even for 'formatted' file
!   1998-02-27 (B.F.) Patch for (conv == 'ieee_4_limit')
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer, optional, intent(in)           :: ntt
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, ierr0
character*(let)                         :: conv, date, system, machin, osrels, &
                                           osvers, hrdwar, progrm, langua, form
character*(80)                          :: outstr0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
! --- Get some information about environment ---
call uio_getenv(system=system, machin=machin, osrels=osrels, osvers=osvers, &
                hrdwar=hrdwar, progrm=progrm, langua=langua)
!write(6,*) system,machin,osrels,osvers,hrdwar,progrm
call uio_dattim(date)
!
! --- Get channel unit ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Assemble file header: standard keywords ---
nttt=0
call uio_adkeyw(termtt,nttt,fifona)
call uio_adkeyw(termtt,nttt,uiona)
call uio_adkeyw(termtt,nttt,formna,keyval=form  ,keydef='formatted')
conv=cvnals(cvnum(nc))
if (conv == 'ieee_4_limit') conv='ieee_4'
call uio_adkeyw(termtt,nttt,convna,keyval=conv)
call uio_adkeyw(termtt,nttt,versna,keyval=versio)
call uio_adkeyw(termtt,nttt,datena,keyval=date)
call uio_adkeyw(termtt,nttt,systna,keyval=system)
call uio_adkeyw(termtt,nttt,machna,keyval=machin)
call uio_adkeyw(termtt,nttt,osrena,keyval=osrels)
call uio_adkeyw(termtt,nttt,osvena,keyval=osvers)
call uio_adkeyw(termtt,nttt,hrdwna,keyval=hrdwar)
call uio_adkeyw(termtt,nttt,langna,keyval=langua)
call uio_adkeyw(termtt,nttt,progna,keyval=progrm)
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_wrfifo


!----------*******----------------------------------------------------------------------------------
subroutine uio_wrr(nc,val,ident, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrr ('uio_write_real')
!
! PURPOSE:
!   Write real.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrr(nc,val,ident, &                 
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0E0)), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=forels(cvnum(nc))
  else if (format == '*') then
    frmt=forels(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=forels(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
wfrmt='(' // trim(frmt) // ')'
call uio_adkeyw(termtt,nttt,bytena,keyval=btrels(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val)
  else
    write(nc) (val)
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val)
  else
    write(nc,wfrmt) val
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrr', errlevel=3)
!
end subroutine uio_wrr


!----------********---------------------------------------------------------------------------------
subroutine uio_wrr1(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrr1 ('uio_write_real_array_1d')
!
! PURPOSE:
!   Write 1D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrr1(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0E0)), dimension(:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(1), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=forels(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // forels(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=forels(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btrels(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1)))
  else
    write(nc) (val(ilow0(1):iup0(1)))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1)))
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1))
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrr1', errlevel=3)
!
end subroutine uio_wrr1


!----------********---------------------------------------------------------------------------------
subroutine uio_wrr2(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrr2 ('uio_write_real_array_2d')
!
! PURPOSE:
!   Write 2D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrr2(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0E0)), dimension(:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(2), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=forels(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // forels(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=forels(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btrels(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  else
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrr2', errlevel=3)
!
end subroutine uio_wrr2


!----------********---------------------------------------------------------------------------------
subroutine uio_wrr3(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrr3 ('uio_write_real_array_3d')
!
! PURPOSE:
!   Write 3D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrr3(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!   2006-10-09 (B.F.) Check for trivial array bounds and use special branch if possible
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0E0)), dimension(:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(3), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
logical                                 :: is_BoundsTrivial
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=forels(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // forels(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=forels(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btrels(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
is_BoundsTrivial=((sum((ilow0-lbound(val))**2)+sum((iup0-ubound(val))**2)) == 0)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    if (is_BoundsTrivial) then
      write(nc) log10(val)
    else
      write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                          ilow0(3):iup0(3)))
    endif
  else
    if (is_BoundsTrivial) then
      write(nc) val
    else
      write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                     ilow0(3):iup0(3)))
    endif
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    if (is_BoundsTrivial) then
      write(nc,wfrmt) log10(val)
    else
      write(nc,wfrmt) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                                ilow0(3):iup0(3)))
    endif
  else
    if (is_BoundsTrivial) then
      write(nc,wfrmt) val
    else
      write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                          ilow0(3):iup0(3))
    endif
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrr3', errlevel=3)
!
end subroutine uio_wrr3


!----------********---------------------------------------------------------------------------------
subroutine uio_wrr4(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrr4 ('uio_write_real_array_4d')
!
! PURPOSE:
!   Write 4D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrr4(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!   2006-10-23 (B.F.) Check for trivial array bounds and use special branch if possible
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0E0)), dimension(:,:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(4), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
logical                                 :: is_BoundsTrivial
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=forels(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // forels(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=forels(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btrels(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
is_BoundsTrivial=((sum((ilow0-lbound(val))**2)+sum((iup0-ubound(val))**2)) == 0)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    if (is_BoundsTrivial) then
      write(nc) log10(val)
    else
      write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                          ilow0(3):iup0(3),ilow0(4):iup0(4)))
    endif
  else
    if (is_BoundsTrivial) then
      write(nc) val
    else
      write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                     ilow0(3):iup0(3),ilow0(4):iup0(4)))
    endif
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    if (is_BoundsTrivial) then
      write(nc,wfrmt) log10(val)
    else
      write(nc,wfrmt) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                                ilow0(3):iup0(3),ilow0(4):iup0(4)))
    endif
  else
    if (is_BoundsTrivial) then
      write(nc,wfrmt) val
    else
      write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                          ilow0(3):iup0(3),ilow0(4):iup0(4))
    endif
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrr4', errlevel=3)
!
end subroutine uio_wrr4


!----------*******----------------------------------------------------------------------------------
subroutine uio_wrd(nc,val,ident, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrd ('uio_write_double_real')
!
! PURPOSE:
!   Write double real.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrd(nc,val,ident, &                 
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') double real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0D0)), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=fodbls(cvnum(nc))
  else if (format == '*') then
    frmt=fodbls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=fodbls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
wfrmt='(' // trim(frmt) // ')'
call uio_adkeyw(termtt,nttt,bytena,keyval=btdbls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val)
  else
    write(nc) (val)
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val)
  else
    write(nc,wfrmt) val
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrd', errlevel=3)
!
end subroutine uio_wrd


!----------********---------------------------------------------------------------------------------
subroutine uio_wrd1(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrd1 ('uio_write_double_real_array_1d')
!
! PURPOSE:
!   Write 1D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrd1(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') double real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0D0)), dimension(:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(1), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=fodbls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // fodbls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=fodbls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btdbls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1)))
  else
    write(nc) (val(ilow0(1):iup0(1)))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1)))
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1))
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrd1', errlevel=3)
!
end subroutine uio_wrd1


!----------********---------------------------------------------------------------------------------
subroutine uio_wrd2(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrd2 ('uio_write_double_real_array_2d')
!
! PURPOSE:
!   Write 2D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrd2(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') double real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0D0)), dimension(:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(2), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=fodbls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // fodbls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=fodbls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btdbls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  else
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrd2', errlevel=3)
!
end subroutine uio_wrd2


!----------********---------------------------------------------------------------------------------
subroutine uio_wrd3(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrd3 ('uio_write_double_real_array_3d')
!
! PURPOSE:
!   Write 3D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrd3(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') double real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0D0)), dimension(:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(3), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=fodbls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // fodbls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=fodbls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btdbls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3)))
  else
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3)))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3)))
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrd3', errlevel=3)
!
end subroutine uio_wrd3


!----------********---------------------------------------------------------------------------------
subroutine uio_wrd4(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrd4 ('uio_write_double_real_array_4d')
!
! PURPOSE:
!   Write 4D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrd4(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') double real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0D0)), dimension(:,:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(4), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=fodbls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // fodbls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=fodbls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btdbls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                        ilow0(3):iup0(3),ilow0(4):iup0(4)))
  else
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                   ilow0(3):iup0(3),ilow0(4):iup0(4)))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                              ilow0(3):iup0(3),ilow0(4):iup0(4)))
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                        ilow0(3):iup0(3),ilow0(4):iup0(4))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrd4', errlevel=3)
!
end subroutine uio_wrd4


!----------********---------------------------------------------------------------------------------
subroutine uio_wrd5(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrd5 ('uio_write_double_real_array_5d')
!
! PURPOSE:
!   Write 5D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrd5(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') double real, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!   2005-03-17 (J.L.) Adapted 4d version to 5d version
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
real (kind(0.0D0)), dimension(:,:,:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(5), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(5)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0, tran0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,realna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,realna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=fodbls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // fodbls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=fodbls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=5
    perlin='5'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=5
  perlin='5'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btdbls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
  tran0=tran
else
  tran0=''
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  if (tran0 == 'log10') then
    write(nc) log10(val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                        ilow0(3):iup0(3),ilow0(4):iup0(4), &
                        ilow0(5):iup0(5)))
  else
    write(nc) (val( ilow0(1):iup0(1), ilow0(2):iup0(2), &
                    ilow0(3):iup0(3), ilow0(4):iup0(4), &
                    ilow0(5):iup0(5)                    ) )
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (tran0 == 'log10') then
    write(nc,wfrmt) log10(val(ilow0(1):iup0(1), ilow0(2):iup0(2), &
                              ilow0(3):iup0(3), ilow0(4):iup0(4), &
                              ilow0(5):iup0(5)                   ) )
  else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                        ilow0(3):iup0(3),ilow0(4):iup0(4),&
                        ilow0(5):iup0(5))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)* &
                       (iup0(5)-ilow0(5)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrd5', errlevel=3)
!
end subroutine uio_wrd5


!----------*******----------------------------------------------------------------------------------
subroutine uio_wri(nc,val,ident, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wri ('uio_write_integer')
!
! PURPOSE:
!   Write integer.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wri(nc,val,ident, &                 
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') integer, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
integer, &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,intena)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,intena)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=foinls(cvnum(nc))
  else if (format == '*') then
    frmt=foinls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=foinls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
wfrmt='(' // trim(frmt) // ')'
call uio_adkeyw(termtt,nttt,bytena,keyval=btinls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
if (form == 'unformatted') then
    write(nc) (val)
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val
  ncpos(nc)=ncpos(nc)+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wri', errlevel=3)
!
end subroutine uio_wri


!----------********---------------------------------------------------------------------------------
subroutine uio_wri1(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wri1 ('uio_write_integer_array_1d')
!
! PURPOSE:
!   Write 1D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wri1(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') integer, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
integer, dimension(:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(1), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)  :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,intena)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,intena)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=foinls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // foinls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=foinls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btinls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1))
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wri1', errlevel=3)
!
end subroutine uio_wri1


!----------********---------------------------------------------------------------------------------
subroutine uio_wri2(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wri2 ('uio_write_integer_array_2d')
!
! PURPOSE:
!   Write 2D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wri2(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') integer, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
integer, dimension(:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(2), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,intena)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,intena)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=foinls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // foinls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=foinls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btinls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wri2', errlevel=3)
!
end subroutine uio_wri2


!----------********---------------------------------------------------------------------------------
subroutine uio_wri3(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wri3 ('uio_write_integer_array_3d')
!
! PURPOSE:
!   Write 3D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wri3(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') integer, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
integer, dimension(:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(3), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,intena)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,intena)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=foinls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // foinls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=foinls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btinls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wri3', errlevel=3)
!
end subroutine uio_wri3


!----------********---------------------------------------------------------------------------------
subroutine uio_wri4(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wri4 ('uio_write_integer_array_4d')
!
! PURPOSE:
!   Write 4D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wri4(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') integer, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
integer, dimension(:,:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(4), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,intena)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,intena)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=foinls(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // foinls(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=foinls(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btinls(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wri4', errlevel=3)
!
end subroutine uio_wri4


!----------*******----------------------------------------------------------------------------------
subroutine uio_wrc(nc,val,ident, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrc ('uio_write_complex')
!
! PURPOSE:
!   Write complex.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrc(nc,val,ident, &                 
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') complex, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
complex (kind(0.0E0)), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,compna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,compna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=focols(cvnum(nc))
  else if (format == '*') then
    frmt=focols(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=focols(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
wfrmt='(' // trim(frmt) // ')'
call uio_adkeyw(termtt,nttt,bytena,keyval=btcols(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
if (form == 'unformatted') then
    write(nc) (val)
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val
  ncpos(nc)=ncpos(nc)+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrc', errlevel=3)
!
end subroutine uio_wrc


!----------********---------------------------------------------------------------------------------
subroutine uio_wrc1(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrc1 ('uio_write_complex_array_1d')
!
! PURPOSE:
!   Write 1D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrc1(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') complex, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
complex (kind(0.0E0)), dimension(:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(1), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,compna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,compna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=focols(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // focols(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=focols(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btcols(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1))
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrc1', errlevel=3)
!
end subroutine uio_wrc1


!----------********---------------------------------------------------------------------------------
subroutine uio_wrc2(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrc2 ('uio_write_complex_array_2d')
!
! PURPOSE:
!   Write 2D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrc2(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') complex, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
complex (kind(0.0E0)), dimension(:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(2), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,compna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,compna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=focols(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // focols(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=focols(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btcols(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrc2', errlevel=3)
!
end subroutine uio_wrc2


!----------********---------------------------------------------------------------------------------
subroutine uio_wrc3(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrc3 ('uio_write_complex_array_3d')
!
! PURPOSE:
!   Write 3D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrc3(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') complex, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
complex (kind(0.0E0)), dimension(:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(3), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,compna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,compna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=focols(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // focols(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=focols(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btcols(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrc3', errlevel=3)
!
end subroutine uio_wrc3


!----------********---------------------------------------------------------------------------------
subroutine uio_wrc4(nc,val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrc4 ('uio_write_complex_array_4d')
!
! PURPOSE:
!   Write 4D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrc4(nc,val,ident, &                 
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') complex, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1996-01-11 (B.F.) format specifies format of single value
!                   Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) wrra -> wrr1
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1997-01-01 (B.F.) 'X' -> '1X'
!   1997-01-22 (B.F.) vector indices, parameters made optional,
!                     more Fortran90 like
!   1997-02-05 (B.F.) Brackets around ... val(...), call uio_err
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-09-11 (B.F.) Check if tran is present...
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
complex (kind(0.0E0)), dimension(:,:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer, dimension(4), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,compna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,compna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
if (present(format)) then
  if (format == '') then
    frmt=focols(cvnum(nc))
  else if (format == '*') then
    frmt='1X,' // focols(cvnum(nc))
  else
    frmt=format
  endif
else
  frmt=focols(cvnum(nc))
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=4
    perlin='4'
  else
    nperlin0=nperlin
    write(perlin,'(I6)') nperlin0
    perlin=adjustl(perlin)
  endif
else
  nperlin0=4
  perlin='4'
endif
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=btcols(cvnum(nc)))
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) then
  call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
else
endif
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
    write(nc) (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4)))
  ncpos(nc)=ncpos(nc)+1
else
    write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrc4', errlevel=3)
!
end subroutine uio_wrc4


!----------*******----------------------------------------------------------------------------------
subroutine uio_wrs(nc,val,ident, nbytes, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrs ('uio_write_string')
!
! PURPOSE:
!   Write character.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrs(nc,val,ident, nbytes=nbytes, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') character, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, optional, number of characters of
!           variable val to be written
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1997-02-05 (B.F.) Merging of uio_wrr and wrs1
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer,       optional, intent(in)     :: nbytes
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0, nlen, nlast
character*(let)                         :: lenval
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,charna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,charna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Format ---
nlen=len(val)
if (present(nbytes)) then
  nbytes0=nbytes
else
  nbytes0=0
endif
if (nbytes0 <= 0) then
  write(lenval,'(I6)') nlen
  nlast=nlen
else
  write(lenval,'(I6)') nbytes0
  nlast=min(nlen,nbytes0)
endif
lenval=adjustl(lenval)
if (present(format)) then
  if (format == '') then
    frmt='A'//lenval
  else if (format == '*') then
    frmt='A'//lenval
  else
    frmt=format
  endif
else
  frmt='A'//lenval
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
wfrmt='(' // trim(frmt) // ')'
call uio_adkeyw(termtt,nttt,bytena,keyval=lenval)
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
if (form == 'unformatted') then
  write(nc)      (val(:nlast))
  ncpos(nc)=ncpos(nc)+1
else
  write(nc,wfrmt) val(:nlast)
  ncpos(nc)=ncpos(nc)+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrs', errlevel=3)
!
end subroutine uio_wrs


!----------********---------------------------------------------------------------------------------
subroutine uio_wrs1(nc,val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrs1 ('uio_write_string_array_1d')
!
! PURPOSE:
!   Write 1D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrs1(nc,val,ident, nbytes=nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') character, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, optional, number of characters of
!           variable val to be written
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1997-02-05 (B.F.) Merging of uio_wrr1 and wrs1
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*), dimension(:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer,       optional, intent(in)     :: nbytes
integer, dimension(1), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0, nlen, nlast
character*(let)                         :: lenval
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,charna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,charna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
nlen=len(val)
if (present(nbytes)) then
  nbytes0=nbytes
else
  nbytes0=0
endif
if (nbytes0 <= 0) then
  write(lenval,'(I6)') nlen
  nlast=nlen
else
  write(lenval,'(I6)') nbytes0
  nlast=min(nlen,nbytes0)
endif
lenval=adjustl(lenval)
if (present(format)) then
  if (format == '') then
    frmt='A'//lenval
  else if (format == '*') then
    frmt='A'//lenval
  else
    frmt=format
  endif
else
  frmt='A'//lenval
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=max(40/nlast,1)
  else
    nperlin0=nperlin
  endif
else
  nperlin0=max(40/nlast,1)
endif
write(perlin,'(I6)') nperlin0
perlin=adjustl(perlin)
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=lenval)
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  write(nc)      (val(ilow0(1):iup0(1))(:nlast))
  ncpos(nc)=ncpos(nc)+1
else
  write(nc,wfrmt) val(ilow0(1):iup0(1))(:nlast)
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrs1', errlevel=3)
!
end subroutine uio_wrs1


!----------********---------------------------------------------------------------------------------
subroutine uio_wrs2(nc,val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrs2 ('uio_write_string_array_2d')
!
! PURPOSE:
!   Write 2D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrs2(nc,val,ident, nbytes=nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') character, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, optional, number of characters of
!           variable val to be written
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1997-02-05 (B.F.) Merging of uio_wrr2 and wrs1
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*), dimension(:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer,       optional, intent(in)     :: nbytes
integer, dimension(2), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0, nlen, nlast
character*(let)                         :: lenval
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,charna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,charna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
nlen=len(val)
if (present(nbytes)) then
  nbytes0=nbytes
else
  nbytes0=0
endif
if (nbytes0 <= 0) then
  write(lenval,'(I6)') nlen
  nlast=nlen
else
  write(lenval,'(I6)') nbytes0
  nlast=min(nlen,nbytes0)
endif
lenval=adjustl(lenval)
if (present(format)) then
  if (format == '') then
    frmt='A'//lenval
  else if (format == '*') then
    frmt='A'//lenval
  else
    frmt=format
  endif
else
  frmt='A'//lenval
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=max(40/nlast,1)
  else
    nperlin0=nperlin
  endif
else
  nperlin0=max(40/nlast,1)
endif
write(perlin,'(I6)') nperlin0
perlin=adjustl(perlin)
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=lenval)
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  write(nc)      (val(ilow0(1):iup0(1),ilow0(2):iup0(2))(:nlast))
  ncpos(nc)=ncpos(nc)+1
else
  write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))(:nlast)
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrs2', errlevel=3)
!
end subroutine uio_wrs2


!----------********---------------------------------------------------------------------------------
subroutine uio_wrs3(nc,val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrs3 ('uio_write_string_array_3d')
!
! PURPOSE:
!   Write 3D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrs3(nc,val,ident, nbytes=nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') character, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, optional, number of characters of
!           variable val to be written
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1997-02-05 (B.F.) Merging of uio_wrr3 and wrs1
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*), dimension(:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer,       optional, intent(in)     :: nbytes
integer, dimension(3), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0, nlen, nlast
character*(let)                         :: lenval
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,charna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,charna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
nlen=len(val)
if (present(nbytes)) then
  nbytes0=nbytes
else
  nbytes0=0
endif
if (nbytes0 <= 0) then
  write(lenval,'(I6)') nlen
  nlast=nlen
else
  write(lenval,'(I6)') nbytes0
  nlast=min(nlen,nbytes0)
endif
lenval=adjustl(lenval)
if (present(format)) then
  if (format == '') then
    frmt='A'//lenval
  else if (format == '*') then
    frmt='A'//lenval
  else
    frmt=format
  endif
else
  frmt='A'//lenval
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=max(40/nlast,1)
  else
    nperlin0=nperlin
  endif
else
  nperlin0=max(40/nlast,1)
endif
write(perlin,'(I6)') nperlin0
perlin=adjustl(perlin)
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=lenval)
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  write(nc)      (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))(:nlast))
  ncpos(nc)=ncpos(nc)+1
else
  write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))(:nlast)
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrs3', errlevel=3)
!
end subroutine uio_wrs3


!----------********---------------------------------------------------------------------------------
subroutine uio_wrs4(nc,val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrs4 ('uio_write_string_array_4d')
!
! PURPOSE:
!   Write 4D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrs4(nc,val,ident, nbytes=nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 termt=termt,ntt=ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   val:    ('value') character, variable to be written
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, optional, number of characters of
!           variable val to be written
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, string with units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date string or 'now'
!   termt:  ('term_table') character array (2,nttmx), optional, additional
!           keywords
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_cvl_module:        UIO convert type list of current machine
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   First, the header is composed and written into file.
!   Then, the data is written in formatted or unformatted form.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-03 (B.F. Kiel) Written
!   1997-02-05 (B.F.) Merging of uio_wrr4 and wrs1
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*), dimension(:,:,:,:), &
                         intent(in)     :: val
character*(*),           intent(in)     :: ident
integer,       optional, intent(in)     :: nbytes
integer, dimension(4), &
               optional, intent(in)     :: ilb, ilow, iup, ishift
integer,       optional, intent(in)     :: nperlin, mode
character*(*), optional, intent(in)     :: format, name, unit, tran, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0, nlen, nlast
character*(let)                         :: lenval
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, frmt, wfrmt, date0
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
! --- Type and identifier ---
nttt=0
call uio_adkeyw(termtt,nttt,charna)
if (ident == '') then
  call uio_adkeyw(termtt,nttt,charna)
else
  call uio_adkeyw(termtt,nttt,ident)
endif
!
! --- Dimension ---
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ilow)) then
  ilow0=ilow
else
  ilow0=ilb0
endif
if (present(iup)) then
  iup0=iup
else
  iup0=ubound(val)-lbound(val)+ilb0
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_dim2st(ilow0, iup0, dimstr, ilbound=ilb0, ishift=ishift0, mode=mode0)
call uio_adkeyw(termtt,nttt,dimna,keyval=dimstr)
!
! --- Format ---
nlen=len(val)
if (present(nbytes)) then
  nbytes0=nbytes
else
  nbytes0=0
endif
if (nbytes0 <= 0) then
  write(lenval,'(I6)') nlen
  nlast=nlen
else
  write(lenval,'(I6)') nbytes0
  nlast=min(nlen,nbytes0)
endif
lenval=adjustl(lenval)
if (present(format)) then
  if (format == '') then
    frmt='A'//lenval
  else if (format == '*') then
    frmt='A'//lenval
  else
    frmt=format
  endif
else
  frmt='A'//lenval
endif
call uio_adkeyw(termtt,nttt,frmtna,keyval=frmt)
if (present(nperlin)) then
  if (nperlin == 0) then
    nperlin0=max(40/nlast,1)
  else
    nperlin0=nperlin
  endif
else
  nperlin0=max(40/nlast,1)
endif
write(perlin,'(I6)') nperlin0
perlin=adjustl(perlin)
wfrmt='(' // trim(perlin) // '(' // trim(frmt) // '))'
call uio_adkeyw(termtt,nttt,plinna,keyval=perlin)
call uio_adkeyw(termtt,nttt,bytena,keyval=lenval)
!
! --- Additional keywords ---
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(unit)) call uio_adkeyw(termtt,nttt,unitna,keyval=unit)
if (present(tran)) call uio_adkeyw(termtt,nttt,tranna,keyval=tran)
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
! --- Write array into file and increment file position counter ---
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if (form == 'unformatted') then
  write(nc)      (val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))(:nlast))
  ncpos(nc)=ncpos(nc)+1
else
  write(nc,wfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))(:nlast)
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_wrs4', errlevel=3)
!
end subroutine uio_wrs4


!----------**********-------------------------------------------------------------------------------
subroutine uio_wrlabl(nc,ident,name,date,termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrlabl ('uio_write_label')
!
! PURPOSE:
!   Write label.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrlabl(nc,ident,name=name,date=date,termt=termt,ntt=ntt, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ident:  ('identifier') character, identifier (mandatory)
!   name:   ('name') character, optional, name of label
!   date:   ('date') character, optional: date string, empty or 'now'
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
!           optional
!   ntt:    ('number_term_table') integer, number of non-empty lines of
!           termt (ntt >= 0), optional
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, contains exit error status, optional
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_dattim:            Get string with current date and time
!   uio_memtrm:            Merge a list of terms (keywords and their values) into a line table
!   uio_wrhd:              Write header of variable, input: term table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data into file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   An entry header is composed and written into file.
!   There is no data block.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-10 (B.F. Kiel) Written
!   1996-10-27 (B.F.) uio_wrhdln -> uio_wrhd
!   1996-12-30 (B.F.) Call uio_dattim
!   1997-01-18 (B.F.) Parameters made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
character*(*), intent(in)               :: ident
character*(*), optional, intent(in)     :: name, date
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer, optional, intent(in)           :: ntt
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nttt, nttn, ierr0
character*(80)                          :: outstr0
character*(let)                         :: form, dat
character*(let)                         :: termtt(2,nttmx)
!---------------------------------------------------------------------------------------------------
!
! --- Get format of output file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Define header ---
nttt=0
call uio_adkeyw(termtt,nttt,lablna)
if (ident /= '') then
  call uio_adkeyw(termtt,nttt,ident)
else
  call uio_adkeyw(termtt,nttt,'mark')
endif
if (present(name)) call uio_adkeyw(termtt,nttt,namena,keyval=name)
if (present(date)) then
  if (date == 'now') then
    call uio_dattim(dat)
  else
    dat=date
  endif
  call uio_adkeyw(termtt,nttt,datena,keyval=dat)
endif
!
! --- Concatenate term tables ---
if (present(termt)) then
  if (present(ntt)) then
    nttn=min(nttt+ntt,nttmx)
  else
    nttn=nttmx
  endif
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Write header into file ---
call uio_wrhd(nc,form, termtt,nttt, outstr=outstr0,ierr=ierr0)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_wrlabl


!----------********---------------------------------------------------------------------------------
subroutine uio_rdhd(nc, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdhd ('uio_read_head')
!
! PURPOSE:
!   Read header of variable and extract keywords.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdhd(nc, termt,ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!   outstr: ('output_string') character, optional, EOF -> 'eof'
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_exmtrm:            Transform a list of items into its components
!   uio_rdhdln:            Read header, output: line table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned properly at beginning of entry header.
!
! PROCEDURE:
!   Read data header from file on unit nc and put keywords into table.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-07 (B.F. Kiel) Written
!   1996-01-05 (B.F.) outstr='eof' if EOF
!   1996-01-27 (B.F.) More comments
!   1997-01-18 (B.F.) outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
integer, intent(out)                    :: ntt
character*(*), intent(out)              :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nlt, ierr0, ierr1
character*(lel)                         :: lineto(nltmx)
character*(let)                         :: form
character*(80)                          :: outstr0, outstr1
!---------------------------------------------------------------------------------------------------
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read file header completely ---
call uio_rdhdln(nc,form,lineto,nlt, outstr=outstr0,ierr=ierr0)
!
! --- Put keywords into table ---
call uio_exmtrm(lineto,nlt, termt,ntt, outstr1,ierr1)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_rdhd


!----------********---------------------------------------------------------------------------------
subroutine uio_srhd(nc, termt,ntt, type,ident, srchls,nsrc, &
                    identlist,poslist,nentry, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_srhd ('uio_search_header')
!
! PURPOSE:
!   Search header of variable and go to its position, read header,
!   and extract keywords. Search directly in file or in entry list.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_srhd(nc, termt,ntt, &
!                 type=type,ident=ident, srchls=srchls,nsrc=nsrc, &
!                 identlist=identlist,poslist=poslist,nentry=nentry, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   type:   ('type') character, optional, type of search keyword
!   ident:  ('identifier') character, optional, identifier (name) of search
!           keyword
!   srchls: ('search_list') character array (2,nsrcmx), optional,list with
!           names oft ypes and/or identifiers of variables to be searched for
!   nsrc:   ('number_search') integer, optional, number of variables to be
!           searched for
!   identlist: ('identifier_list') string array with types and identifiers
!           (and names) of all entries, optional
!   poslist:('position_list') integer array with all entry positions
!           corresponding to identlist, optional
!   nentry: ('number_entry') integer, optional, number of file entries
!
! OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
!           optional
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt,
!           optional
!   outstr: ('output_string') character, optional, EOF -> 'eof'
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chpos:             Give current file position or jump to specified position
!   uio_rdhd:              Read header of variable and extract keywords
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Search header of entry in string list (if parameters identlist, poslist,
!   and nentry, created by uio_filcon, are specified), go to its
!   position, read header, and extract keywords.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-11-27 (B.F. Kiel) Written
!   1997-01-06 (B.F.) gotos removed
!   1997-01-20 (B.F.) Routines srhdex, slhd, fihdex merged; parameters made optional
!   1997-02-23 (B.F.) Error correction
!   1997-09-11 (B.F.) No identlist, EOF -> outstr='eof'
!   1997-11-27 (B.F.) (1:) -> (:)
!   2000-11-26 (B.F. Uppsala) Both search identlist entries might be empty
!---------------------------------------------------------------------------------------------------
use uio_siz_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(out)    :: termt(2,nttmx)
integer,                 intent(out)    :: ntt
character*(*), optional, intent(in)     :: type, ident
integer,       optional, intent(in)     :: nsrc
character*(*), optional, intent(in)     :: srchls(:,:)
character*(*), optional, intent(in)     :: identlist(:,:)
integer,       optional, intent(in)     :: poslist(:)
integer,       optional, intent(in)     :: nentry
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
logical                                 :: found
integer                                 :: n, nent, ierr0, nsrc0, nsrc1
integer, parameter                      :: nsrcmx0=20
character*(80)                          :: outstr0
character*(let)                         :: srchls0(2,nsrcmx0)
!---------------------------------------------------------------------------------------------------
! --- Complete search list ---
nsrc0=0
if (present(type) .or. present(ident)) then
  ! --- Put type and identifier into search list ---
  nsrc0=nsrc0+1
  srchls0(:,nsrc0)=''
  if (present(type)) srchls0(1,nsrc0)=type
  if (present(ident)) srchls0(2,nsrc0)=ident
endif
if (present(srchls)) then
  ! --- Put input search list into actual search list ---
  if (present(nsrc)) then
    nsrc1=nsrc
  else
    nsrc1=size(srchls,2)
  endif
  nsrc1=min(nsrc0+nsrc1,nsrcmx0)
  srchls0(:,nsrc0+1:nsrc0+nsrc1)=srchls(:,1:nsrc1)
  nsrc0=nsrc1
endif
!if (nsrc0 == 0) then
!  nsrc0=1
!  srchls0(:,nsrc0)=''
!endif
!
if (present(identlist) .or. present(poslist) .or. present(nentry)) then
  ! --- Search in list ---
  if (.not. present(identlist) .or. .not. present(poslist) .or. &
      .not. present(nentry)) then
    outstr0='incomplete identlist'
    ierr0=1
  else
    nent=0
    !
    entryloop1: do
      ! --- Skip data block ---
      nent=nent+1
      if (nent > nentry) then
        ! --- End of entry list reached ---
        found=.false.
        ierr0=1
        outstr0='entry not found'
        ntt=0
        exit entryloop1
      endif
      ! --- Read next header ---
      if (identlist(1,nent) == 'eof') then
        ! --- EOF reached ---
        found=.false.
        ierr0=1
        outstr0='eof'
        ntt=0
        exit entryloop1
      endif
      !
      found=.true.
      nsrcloop1: do n=1,nsrc0
        if (srchls0(1,n) == '') then
          if (srchls0(2,n) == '') then
            found=.true.
          else
            found=srchls0(2,n) == identlist(2,nent)
          endif
        else if (srchls0(1,n) /= identlist(1,nent)) then
          found=.false.
        else if (srchls0(2,n) == '') then
          found=.true.
        else
          found=srchls0(2,n) == identlist(2,nent)
        endif
        if (found) exit nsrcloop1
      end do nsrcloop1 ! n
      !
      if (found) then
        exit entryloop1
      endif
    end do entryloop1
    !
    if (found) then
      ! --- Entry found in list ---
      ! --- Goto entry position ---
      call uio_chpos(nc,mode='goto',posin=poslist(nent))
      ! --- Read entry header ---
      call uio_rdhd(nc, termt,ntt, ierr=ierr0)
      if (ierr0 == 0) then
        outstr0='found'
      else
        ntt=0
      endif
    endif
  endif
else
  ! --- Search in file ---
  ! --- Loop over search list until variable found or end of list ---
  entryloop2: do
    ! --- Read next header ---
    call uio_rdhd(nc, termt,ntt, outstr=outstr0)
    if (outstr0 == 'eof') then
      found=.false.
      ierr0=1
      outstr0='eof'
      ntt=0
      exit entryloop2
    endif
    !
    found=.true.
    nsrcloop2: do n=1,nsrc0
      if (srchls0(1,n) == '') then
        if (srchls0(2,n) == '') then
          found=.true.
        else
          found=srchls0(2,n) == termt(1,2)
        endif
      else if (srchls0(1,n) /= termt(1,1)) then
        found=.false.
      else if (srchls0(2,n) == '') then
        found=.true.
      else
        found=srchls0(2,n) == termt(1,2)
      endif
      if (found) exit nsrcloop2
    end do nsrcloop2 ! n
    !
    if (found) then
      exit entryloop2
    endif
    !
    ! --- Skip data block ---
    call uio_skipda(nc, termt,ntt)
  end do entryloop2
  !
  if (found) then
    ! --- Entry found in list ---
    ierr0=0
    outstr0='found'
  endif
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_srhd


!----------**********-------------------------------------------------------------------------------
subroutine uio_filcon(nc, identlist,poslist,nentry, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_filcon ('uio_file_content')
!
! PURPOSE:
!   Determine file content: list of all entries with its positions.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_filcon(nc, identlist,poslist,nentry, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! OUTPUT:
!   identlist: ('identifier_list') string array with types and identifiers
!           (and names) of all entries
!   poslist:('position_list') integer array with all entry positions
!           corresponding to identlist
!   nentry: ('number_entry') integer, number of file entries
!   outstr: ('output_string') character, EOF -> 'eof'
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chpos:             Give current file position or jump to specified position
!   uio_exkeyw:            Extract value of keyword from table
!   uio_rdhd:              Read header of variable and extract keywords
!   uio_skipda:            Skip data block
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read file on unit nc. Return with file position at eof.
!
! RESTRICTIONS:
!   The routine is only useful for uio-files.
!   Before first call of uio_filcon, initialize uio-routines with uio_init.
!
! PROCEDURE:
!   Read all entries one by one and save its type, identifier and position.
!   Start from current position. Return with file position at eof if identlist
!   is long enough. Otherwise at entry last read.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-11-25 (B.F. Kiel) Written
!   1996-12-05 (B.F.) '' -> ' '
!   1997-01-19 (B.F.) Parameter netmx removed; outstr,ierr made optional; ' ' -> ''
!   1997-11-27 (B.F.) (1:) -> (:)
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
integer, intent(out)                    :: nentry
integer, intent(out)                    :: poslist(:)
character*(*), intent(out)              :: identlist(:,:)
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: ntt, ncol, nentmx, ierr0
character*(80)                          :: outstr0, outstr1
character*(let)                         :: termt(2,nttmx)
!---------------------------------------------------------------------------------------------------
ncol=size(identlist,1)
nentmx=size(identlist,2)
!write(6,*) ncol,nentmx
nentry=1
!
do
  ! --- Determine and save current file position ---
  call uio_chpos(nc,mode='get',posout=poslist(nentry))
  ! --- Read entry header ---
  call uio_rdhd(nc, termt,ntt, outstr=outstr1)
  if (outstr1 == 'eof') then
    ! --- End of file reached ---
    identlist(1,nentry)='eof'
    identlist(2:,nentry)=''
    outstr0='done'
    ierr0=0
    exit
  else
    identlist(1,nentry)=termt(1,1)
    identlist(2,nentry)=termt(1,2)
!    write(6,*) nentry, ' ', trim(identlist(1,nentry)), &
!                       ' ', trim(identlist(2,nentry))
    if (ncol >= 3) call uio_exkeyw(termt,ntt,namena,identlist(3,nentry))
    ! --- Skip data block ---
    call uio_skipda(nc, termt,ntt)
    nentry=nentry+1
    if (nentry > nentmx) then
      ! --- End of entry list reached ---
      nentry=nentmx
      outstr0='too many entries in file'
      ierr0=1
      exit
    endif
  endif
end do
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_filcon


!----------**********-------------------------------------------------------------------------------
subroutine uio_skipda(nc,termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_skipda ('uio_skip_data')
!
! PURPOSE:
!   Skip data block
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_skipda(nc,termt,ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Analyze header and skip following data lines or data block.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-12-08 (B.F. Kiel) Written
!   1996-01-09 (B.F.) array
!   1996-01-11 (B.F.) Number of elements per line in array (perlin)
!   1996-01-26 (B.F.) Skipping of table
!   1996-01-29 (B.F.) Skipping of array lines different
!   1996-10-21 (B.F.) File position counter
!   1996-10-28 (B.F.) Renaming
!   1997-01-09 (B.F.) Put termt into termtt
!   1997-01-18 (B.F.) Parameters made optional
!   1997-02-07 (B.F.) st2dim -> uio_st2dim
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc, ntt
character*(*), intent(in)               :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nlt, i, nperlin0, n, nttt
integer, dimension(2)                   :: ilow, iup
character                               :: form*80
character*(let)                         :: type, format, cbyt, &
                                           dimstr, perlin
character*(let)                         :: termtt(2,nttmx)
character*(lel)                         :: lineto(nltmx)
!---------------------------------------------------------------------------------------------------
! --- Make temporary copy from term table --
nttt=ntt
do n=1,ntt
  termtt(1,n)=termt(1,n)
  termtt(2,n)=termt(2,n)
end do ! n
!
! --- Find keywords and extract their values ---
type=termtt(1,1)
!ident=termtt(1,2)
call uio_exkeyw(termtt,nttt, frmtna,format)
call uio_exkeyw(termtt,nttt, bytena,cbyt  )
call uio_exkeyw(termtt,nttt, dimna ,dimstr)
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read data from file: skip ---
if (type == fifona) then
  ! --- fileform: no data ---
  !
else if (type == lablna) then
  ! --- label: no data ---
  !
else if (type == tabna) then
  ! --- Table ---
  call uio_st2dim(dimstr, ilow, iup)
  ! --- Skip entry headers ---
  do i=ilow(1),iup(1)
    call uio_rdhdln(nc,form, lineto,nlt)
  end do ! i
  if (form == 'unformatted') then
    ! --- Skip data headline ---
    read(nc)
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+1
    ! --- Skip table ---
    read(nc)
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+1
  else
    ! --- Skip data headline ---
    read(nc,'(A1)')
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+1
    ! --- Skip table ---
    do i=ilow(2),iup(2)
      read(nc,'(A1)')
    end do ! i
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+iup(2)-ilow(2)+1
  endif
  !
else
  ! --- Integer, real, double precision real, complex, or character data ---
  if (dimstr == '') then
    ! --- Scalar data ---
    if (form == 'unformatted') then
      read(nc)
    else
      read(nc,'(A1)')
    endif
    ! --- Increment file position counter ---
    ncpos(nc)=ncpos(nc)+1
  else
    ! --- Array data ---
    if (form == 'unformatted') then
      read(nc)
      ! --- Increment file position counter ---
      ncpos(nc)=ncpos(nc)+1
    else
      ! --- Parse dimstr, compute array bounds, allow only 1 index for ---
      ! --- lower and upper bound ->                                   ---
      ! --- iup(1)-ilow(1)+1 gives total number of elements            ---
      call uio_st2dim(dimstr, ilow(1:1), iup(1:1))
      call uio_exkeyw(termtt,nttt, plinna ,perlin)
      read(perlin,'(I6)') nperlin0
      do i=0,(iup(1)-ilow(1))/nperlin0
        read(nc,'(A1)')
      end do ! i
      ! --- Increment file position counter ---
      ncpos(nc)=ncpos(nc)+(iup(1)-ilow(1))/nperlin0+1
    endif
  endif
  !
endif
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_skipda


!----------**********-------------------------------------------------------------------------------
subroutine uio_rdfifo(nc,termt,ntt, form,conv,date, &
                      system,machin,osrels,osvers,hrdwar,langua,vers, &
                      progrm, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdfifo ('uio_read_fileform')
!
! PURPOSE:
!   Read file header.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdfifo(nc,termt,ntt, form=form,conv=conv,date=date, &
!                   system=system,machin=machin,osrels=osrels,osvers=osvers, &
!                   hrdwar=hrdwar,langua=langua,vers=vers,progrm=progrm, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number (actually not used)
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   form:   ('format') character, 'formatted' or 'unformatted'
!   conv:   ('conversion') character, conversion type
!   date:   ('date') character, empty or date information
!   system: ('system') character, operating system      (e.g. SunOS, craSH    )
!   machin: ('machine') character, empty or machine name(e.g. venus, craSH    )
!   osrels: ('operating_system_release') character, release of operating system
!                                                       (e.g. 4.1.3, 8.0.3.0  )
!   osvers: ('operating system version') character, version of operating system
!                                                       (e.g. 1,     awa.5    )
!   hrdwar: ('hardware') character, empty or machine hardware
!                                                       (e.g. sun4m, CRAY Y-MP)
!   langua: ('language') character, language of program that wrote file
!                                                       (e.g. 'FORTRAN77'     )
!   vers:   ('version') character, uio-version
!   progrm: ('program_name') character, name of main program
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Extract information from termt by repeated calls of uio_exkeyw.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-10 (B.F. Kiel) Written
!   1996-01-30 (B.F.) Parameter langua added
!   1996-02-01 (B.F.) Parameter progrm added
!   1997-01-17 (B.F.) Parameter made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
character*(*), intent(inout)            :: termt(2,nttmx)
integer, intent(inout)                  :: ntt
character*(*), optional, intent(out)    :: form, conv, date, system, &
                                           machin, osrels, osvers, &
                                           hrdwar, langua, vers, progrm
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nc0
character*(let)                         :: sdum
!---------------------------------------------------------------------------------------------------
! --- Put parameter into variable to prevent compiler from complaining about --
! --- 'Unused dummy variable NC' ---
nc0=nc
!
! --- Find keywords and extract their values ---
termt(1,1)=''
termt(1,2)=''
if (present(form  )) call uio_exkeyw(termt,ntt, formna,form,   mode=1)
if (present(conv  )) call uio_exkeyw(termt,ntt, convna,conv,   mode=1)
if (present(vers  )) call uio_exkeyw(termt,ntt, versna,vers,   mode=1)
if (present(date  )) call uio_exkeyw(termt,ntt, datena,date,   mode=1)
if (present(system)) call uio_exkeyw(termt,ntt, systna,system, mode=1)
if (present(machin)) call uio_exkeyw(termt,ntt, machna,machin, mode=1)
if (present(osrels)) call uio_exkeyw(termt,ntt, osrena,osrels, mode=1)
if (present(osvers)) call uio_exkeyw(termt,ntt, osvena,osvers, mode=1)
if (present(hrdwar)) call uio_exkeyw(termt,ntt, hrdwna,hrdwar, mode=1)
if (present(langua)) call uio_exkeyw(termt,ntt, langna,langua, mode=1)
if (present(progrm)) call uio_exkeyw(termt,ntt, progna,progrm, mode=1)
call uio_exkeyw(termt,ntt, progna,sdum, mode=4)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_rdfifo


!----------*******----------------------------------------------------------------------------------
subroutine uio_rdr(nc,termt,ntt, val,ident, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdr ('uio_read_real')
!
! PURPOSE:
!   Read real.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdr(nc,termt,ntt,val,ident, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0E0)), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(format0) // ')'
endif
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val
  else
    read(nc,rfrmt) val
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val=10.0_ki**val
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdr', errlevel=3)
!
end subroutine uio_rdr


!----------********---------------------------------------------------------------------------------
subroutine uio_rdr1(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdr1 ('uio_read_real_array_1d')
!
! PURPOSE:
!   Read 1D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdr1(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0E0)), dimension(:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(1), &
               optional, intent(out)    :: ilow, iup
integer, dimension(1), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1))
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val(ilow0(1):iup0(1))=10.0_ki**val(ilow0(1):iup0(1))
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdr1', errlevel=3)
!
end subroutine uio_rdr1


!----------********---------------------------------------------------------------------------------
subroutine uio_rdr2(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdr2 ('uio_read_real_array_2d')
!
! PURPOSE:
!   Read 2D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdr2(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0E0)), dimension(:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(2), &
               optional, intent(out)    :: ilow, iup
integer, dimension(2), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val(ilow0(1):iup0(1),ilow0(2):iup0(2))= &
                          10.0_ki**val(ilow0(1):iup0(1),ilow0(2):iup0(2))
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdr2', errlevel=3)
!
end subroutine uio_rdr2


!----------********---------------------------------------------------------------------------------
subroutine uio_rdr3(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdr3 ('uio_read_real_array_3d')
!
! PURPOSE:
!   Read 3D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdr3(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   2006-10-09 (B.F.) Check for trivial array bounds and use special branch if possible
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0E0)), dimension(:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(3), &
               optional, intent(out)    :: ilow, iup
integer, dimension(3), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
logical                                 :: is_BoundsTrivial
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Check if array bounds are trivially comprising the entire array ---
is_BoundsTrivial=((sum((ilow0-lbound(val))**2)+sum((iup0-ubound(val))**2)) == 0)
!
!write(6,*) 'uio_rdr3:'
!write(6,*) ilb0
!write(6,*) ilow0
!write(6,*) iup0
!write(6,*) lbound(val)
!write(6,*) ubound(val)
!write(6,*) is_BoundsTrivial
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  if (is_BoundsTrivial) then
    read(nc) val
  else
    read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                 ilow0(3):iup0(3))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    if (is_BoundsTrivial) then
      read(nc,*) val
    else
      read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                     ilow0(3):iup0(3))
    endif
  else
    if (is_BoundsTrivial) then
      read(nc,rfrmt) val
    else
      read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                         ilow0(3):iup0(3))
    endif
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') then
  if (is_BoundsTrivial) then
    val=10.0_ki**val
  else
    val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
        ilow0(3):iup0(3))=10.0_ki**val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                                       ilow0(3):iup0(3))
  endif
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdr3', errlevel=3)
!
end subroutine uio_rdr3


!----------********---------------------------------------------------------------------------------
subroutine uio_rdr4(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdr4 ('uio_read_real_array_4d')
!
! PURPOSE:
!   Read 4D real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdr4(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   2006-10-23 (B.F.) Check for trivial array bounds and use special branch if possible
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0E0)), dimension(:,:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(4), &
               optional, intent(out)    :: ilow, iup
integer, dimension(4), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
logical                                 :: is_BoundsTrivial
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Check if array bounds are trivially comprising the entire array ---
is_BoundsTrivial=((sum((ilow0-lbound(val))**2)+sum((iup0-ubound(val))**2)) == 0)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  if (is_BoundsTrivial) then
    read(nc) val
  else
    read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                 ilow0(3):iup0(3),ilow0(4):iup0(4))
  endif
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    if (is_BoundsTrivial) then
      read(nc,*) val
    else
      read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                     ilow0(3):iup0(3),ilow0(4):iup0(4))
    endif
  else
    if (is_BoundsTrivial) then
      read(nc,rfrmt) val
    else
      read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                         ilow0(3):iup0(3),ilow0(4):iup0(4))
    endif
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') then
  if (is_BoundsTrivial) then
    val=10.0_ki**val
  else
    val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
        ilow0(3):iup0(3),ilow0(4):iup0(4))=10.0_ki**val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                                                        ilow0(3):iup0(3),ilow0(4):iup0(4))
  endif
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdr4', errlevel=3)
!
end subroutine uio_rdr4


!----------*******----------------------------------------------------------------------------------
subroutine uio_rdd(nc,termt,ntt, val,ident, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdd ('uio_read_double_real')
!
! PURPOSE:
!   Read double real.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdd(nc,termt,ntt,val,ident, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') double real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0D0)), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(format0) // ')'
endif
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val
  else
    read(nc,rfrmt) val
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val=10.0_ki**val
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdd', errlevel=3)
!
end subroutine uio_rdd


!----------********---------------------------------------------------------------------------------
subroutine uio_rdd1(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdd1 ('uio_read_double_real_array_1d')
!
! PURPOSE:
!   Read 1D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdd1(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') double real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0D0)), dimension(:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(1), &
               optional, intent(out)    :: ilow, iup
integer, dimension(1), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1))
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val(ilow0(1):iup0(1))=10.0_ki**val(ilow0(1):iup0(1))
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdd1', errlevel=3)
!
end subroutine uio_rdd1


!----------********---------------------------------------------------------------------------------
subroutine uio_rdd2(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdd2 ('uio_read_double_real_array_2d')
!
! PURPOSE:
!   Read 2D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdd2(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') double real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0D0)), dimension(:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(2), &
               optional, intent(out)    :: ilow, iup
integer, dimension(2), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val(ilow0(1):iup0(1),ilow0(2):iup0(2))= &
                        10.0_ki**val(ilow0(1):iup0(1),ilow0(2):iup0(2))
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdd2', errlevel=3)
!
end subroutine uio_rdd2


!----------********---------------------------------------------------------------------------------
subroutine uio_rdd3(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdd3 ('uio_read_double_real_array_3d')
!
! PURPOSE:
!   Read 3D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdd3(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') double real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0D0)), dimension(:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(3), &
               optional, intent(out)    :: ilow, iup
integer, dimension(3), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
               ilow0(3):iup0(3))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                   ilow0(3):iup0(3))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))=10.0_ki**val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdd3', errlevel=3)
!
end subroutine uio_rdd3


!----------********---------------------------------------------------------------------------------
subroutine uio_rdd4(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdd4 ('uio_read_double_real_array_4d')
!
! PURPOSE:
!   Read 4D double real array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdd4(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') double real, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
real (kind(0.0D0)), dimension(:,:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(4), &
               optional, intent(out)    :: ilow, iup
integer, dimension(4), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= realna) then
  outstr0='type is not ' // trim(realna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                   ilow0(3):iup0(3),ilow0(4):iup0(4))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
! --- Transformation ---
if (tran0 == 'log10') val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                          ilow0(3):iup0(3),ilow0(4):iup0(4))= &
                         10.0_ki**val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                                      ilow0(3):iup0(3),ilow0(4):iup0(4))
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdd4', errlevel=3)
!
end subroutine uio_rdd4


!----------*******----------------------------------------------------------------------------------
subroutine uio_rdi(nc,termt,ntt, val,ident, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdi ('uio_read_integer')
!
! PURPOSE:
!   Read integer.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdi(nc,termt,ntt,val,ident, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') integer, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
integer, &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= intena) then
  outstr0='type is not ' // trim(intena) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(format0) // ')'
endif
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val
  else
    read(nc,rfrmt) val
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdi', errlevel=3)
!
end subroutine uio_rdi


!----------********---------------------------------------------------------------------------------
subroutine uio_rdi1(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdi1 ('uio_read_integer_array_1d')
!
! PURPOSE:
!   Read 1D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdi1(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') integer, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
integer, dimension(:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(1), &
               optional, intent(out)    :: ilow, iup
integer, dimension(1), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= intena) then
  outstr0='type is not ' // trim(intena) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1))
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdi1', errlevel=3)
!
end subroutine uio_rdi1


!----------********---------------------------------------------------------------------------------
subroutine uio_rdi2(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdi2 ('uio_read_integer_array_2d')
!
! PURPOSE:
!   Read 2D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdi2(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') integer, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
integer, dimension(:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(2), &
               optional, intent(out)    :: ilow, iup
integer, dimension(2), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= intena) then
  outstr0='type is not ' // trim(intena) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdi2', errlevel=3)
!
end subroutine uio_rdi2


!----------********---------------------------------------------------------------------------------
subroutine uio_rdi3(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdi3 ('uio_read_integer_array_3d')
!
! PURPOSE:
!   Read 3D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdi3(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') integer, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
integer, dimension(:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(3), &
               optional, intent(out)    :: ilow, iup
integer, dimension(3), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= intena) then
  outstr0='type is not ' // trim(intena) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdi3', errlevel=3)
!
end subroutine uio_rdi3


!----------********---------------------------------------------------------------------------------
subroutine uio_rdi4(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdi4 ('uio_read_integer_array_4d')
!
! PURPOSE:
!   Read 4D integer array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdi4(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') integer, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
integer, dimension(:,:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(4), &
               optional, intent(out)    :: ilow, iup
integer, dimension(4), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= intena) then
  outstr0='type is not ' // trim(intena) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdi4', errlevel=3)
!
end subroutine uio_rdi4


!----------*******----------------------------------------------------------------------------------
subroutine uio_rdc(nc,termt,ntt, val,ident, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdc ('uio_read_complex')
!
! PURPOSE:
!   Read complex.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdc(nc,termt,ntt,val,ident, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') complex, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
complex (kind(0.0E0)), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= compna) then
  outstr0='type is not ' // trim(compna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(format0) // ')'
endif
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val
  else
    read(nc,rfrmt) val
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdc', errlevel=3)
!
end subroutine uio_rdc


!----------********---------------------------------------------------------------------------------
subroutine uio_rdc1(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdc1 ('uio_read_complex_array_1d')
!
! PURPOSE:
!   Read 1D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdc1(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') complex, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
complex (kind(0.0E0)), dimension(:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(1), &
               optional, intent(out)    :: ilow, iup
integer, dimension(1), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= compna) then
  outstr0='type is not ' // trim(compna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1))
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdc1', errlevel=3)
!
end subroutine uio_rdc1


!----------********---------------------------------------------------------------------------------
subroutine uio_rdc2(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdc2 ('uio_read_complex_array_2d')
!
! PURPOSE:
!   Read 2D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdc2(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') complex, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
complex (kind(0.0E0)), dimension(:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(2), &
               optional, intent(out)    :: ilow, iup
integer, dimension(2), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= compna) then
  outstr0='type is not ' // trim(compna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdc2', errlevel=3)
!
end subroutine uio_rdc2


!----------********---------------------------------------------------------------------------------
subroutine uio_rdc3(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdc3 ('uio_read_complex_array_3d')
!
! PURPOSE:
!   Read 3D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdc3(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') complex, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
complex (kind(0.0E0)), dimension(:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(3), &
               optional, intent(out)    :: ilow, iup
integer, dimension(3), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= compna) then
  outstr0='type is not ' // trim(compna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdc3', errlevel=3)
!
end subroutine uio_rdc3


!----------********---------------------------------------------------------------------------------
subroutine uio_rdc4(nc,termt,ntt, val,ident, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdc4 ('uio_read_complex_array_4d')
!
! PURPOSE:
!   Read 4D complex array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdc4(nc,termt,ntt,val,ident, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') complex, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: nc
character*(*),         intent(inout)    :: termt(2,nttmx)
integer,               intent(inout)    :: ntt
complex (kind(0.0E0)), dimension(:,:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer, dimension(4), &
               optional, intent(out)    :: ilow, iup
integer, dimension(4), &
               optional, intent(in)     :: ilb, ishift
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= compna) then
  outstr0='type is not ' // trim(compna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0,  mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rdc4', errlevel=3)
!
end subroutine uio_rdc4


!----------*******----------------------------------------------------------------------------------
subroutine uio_rds(nc,termt,ntt, val,ident, nbytes, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rds ('uio_read_string')
!
! PURPOSE:
!   Read character.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rds(nc,termt,ntt,val,ident, nbytes, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') character, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, number of characters of val
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-05-20 (B.F.) Clear val before reading
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
character*(*), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer,       optional, intent(out)    :: nbytes
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= charna) then
  outstr0='type is not ' // trim(charna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(format0) // ')'
endif
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
read(bytes,'(I6)') nbytes0
if (present(nbytes)) nbytes=nbytes0
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
val=''
if (form == 'unformatted') then
  read(nc) val(:nbytes0)
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(:nbytes0)
  else
    read(nc,rfrmt) val(:nbytes0)
  endif
  ncpos(nc)=ncpos(nc)+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rds', errlevel=3)
!
end subroutine uio_rds


!----------********---------------------------------------------------------------------------------
subroutine uio_rds1(nc,termt,ntt, val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rds1 ('uio_read_string_array_1d')
!
! PURPOSE:
!   Read 1D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rds1(nc,termt,ntt,val,ident, nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') character, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, number of characters of val
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-05-20 (B.F.) Clear val before reading
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
character*(*), dimension(:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer,       optional, intent(out)    :: nbytes
integer, dimension(1), &
               optional, intent(in)     :: ilb, ishift
integer, dimension(1), &
               optional, intent(out)    :: ilow, iup
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0
integer                                 :: nperlin0, mode0
integer, dimension(1)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= charna) then
  outstr0='type is not ' // trim(charna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0, mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
read(bytes,'(I6)') nbytes0
if (present(nbytes)) nbytes=nbytes0
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
val(ilow0(1):iup0(1))=''
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1))(:nbytes0)
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1))(:nbytes0)
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1))(:nbytes0)
  endif
  ncpos(nc)=ncpos(nc)+(iup0(1)-ilow0(1))/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rds1', errlevel=3)
!
end subroutine uio_rds1


!----------********---------------------------------------------------------------------------------
subroutine uio_rds2(nc,termt,ntt, val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rds2 ('uio_read_string_array_2d')
!
! PURPOSE:
!   Read 2D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rds2(nc,termt,ntt,val,ident, nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') character, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, number of characters of val
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-05-20 (B.F.) Clear val before reading
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
character*(*), dimension(:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer,       optional, intent(out)    :: nbytes
integer, dimension(2), &
               optional, intent(in)     :: ilb, ishift
integer, dimension(2), &
               optional, intent(out)    :: ilow, iup
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0
integer                                 :: nperlin0, mode0
integer, dimension(2)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= charna) then
  outstr0='type is not ' // trim(charna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0, mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
read(bytes,'(I6)') nbytes0
if (present(nbytes)) nbytes=nbytes0
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
val(ilow0(1):iup0(1),ilow0(2):iup0(2))=''
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2))(:nbytes0)
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2))(:nbytes0)
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2))(:nbytes0)
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rds2', errlevel=3)
!
end subroutine uio_rds2


!----------********---------------------------------------------------------------------------------
subroutine uio_rds3(nc,termt,ntt, val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rds3 ('uio_read_string_array_3d')
!
! PURPOSE:
!   Read 3D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rds3(nc,termt,ntt,val,ident, nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') character, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, number of characters of val
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-05-20 (B.F.) Clear val before reading
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
character*(*), dimension(:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer,       optional, intent(out)    :: nbytes
integer, dimension(3), &
               optional, intent(in)     :: ilb, ishift
integer, dimension(3), &
               optional, intent(out)    :: ilow, iup
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0
integer                                 :: nperlin0, mode0
integer, dimension(3)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= charna) then
  outstr0='type is not ' // trim(charna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0, mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
read(bytes,'(I6)') nbytes0
if (present(nbytes)) nbytes=nbytes0
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))=''
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))(:nbytes0)
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))(:nbytes0)
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3))(:nbytes0)
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rds3', errlevel=3)
!
end subroutine uio_rds3


!----------********---------------------------------------------------------------------------------
subroutine uio_rds4(nc,termt,ntt, val,ident, nbytes, &
                    ilb,ilow,iup,ishift,mode, nperlin, &
                    format,name, unit,tran, date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rds4 ('uio_read_string_array_4d')
!
! PURPOSE:
!   Read 4D character array.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rds4(nc,termt,ntt,val,ident, nbytes, &
!                 ilb=ilb,ilow=ilow,iup=iup,ishift=ishift,mode=mode, &
!                 nperlin=nperlin, &
!                 format=format,name=name, unit=unit,tran=tran, date=date, &
!                 outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   ilb:    ('index_lower_boundary') integer vector, optional, actual lower
!           indices
!   ishift: ('index_shift') integer vector for shift, optional
!   mode:   ('mode') integer, optional, shift mode: 0, ..., 8
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   val:    ('value') character, variable to be read
!   ident:  ('identifier') character, identifier (mandatory)
!   nbytes: ('number_bytes') integer, number of characters of val
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   nperlin:('number_per_line') integer, optional, array elements per line
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_err:               Handle UIO errors
!   uio_exkeyw:            Extract value of keyword from table
!   uio_st2dim:            Parse dimension string
!   uio_wf2rf:             Produce from write format string read format string
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   File pointer has to be positioned right after entry header, just before
!   data block. termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-04 (B.F. Kiel) Written
!   1996-01-09 (B.F.) Parameter mode added
!                   parameter abb removed, tran added
!   1996-01-11 (B.F.) Number of items per line (perlin) added
!   1996-01-12 (B.F.) More comments
!   1996-01-27 (B.F.) rdra -> rdr1
!                   Parameter position tran shifted
!   1996-10-21 (B.F.) format='*' allowed, file position counter
!   1997-01-24 (B.F.) Parameters made optional
!   1997-02-05 (B.F.) Call uio_err
!   1997-02-14 (B.F.) Use uio_wf2rf
!   1997-02-18 (B.F.) Keyword ilb introduced
!   1997-05-20 (B.F.) Clear val before reading
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
character*(*), dimension(:,:,:,:), &
                         intent(out)    :: val
character*(*),           intent(out)    :: ident
integer,       optional, intent(out)    :: nbytes
integer, dimension(4), &
               optional, intent(in)     :: ilb, ishift
integer, dimension(4), &
               optional, intent(out)    :: ilow, iup
integer,       optional, intent(in)     :: mode
integer,       optional, intent(out)    :: nperlin
character*(*), optional, intent(out)    :: format,name, unit,tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nbytes0
integer                                 :: nperlin0, mode0
integer, dimension(4)                   :: ilb0, ilow0, iup0, ishift0
character*(let)                         :: dimstr, perlin
integer, parameter                      :: ki=kind(val)
character*(let)                         :: bytes, form, rfrmt, tran0, format0
character*(80)                          :: outstr0
integer                                 :: ierr0
!---------------------------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
! --- Analyze header ---
! --- Type and identifier ---
if (termt(1,1) /= charna) then
  outstr0='type is not ' // trim(charna) // ' but ' // trim(termt(1,1))
  ierr0=1
  goto 900
endif
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=1)
if (present(ilb)) then
  ilb0=ilb
else
  ilb0=lbound(val)
endif
if (present(ishift)) then
  ishift0=ishift
else
  ishift0=0
endif
if (present(mode)) then
  mode0=mode
else
  mode0=0
endif
call uio_st2dim(dimstr, ilow0, iup0, mode=mode0, ishift=ishift0, &
                ilbound=ilb0, iubound=ubound(val)-lbound(val)+ilb0)
if (present(ilow)) ilow=ilow0
if (present(iup)) iup=iup0
ilow0=ilow0-ilb0+lbound(val)
iup0=iup0-ilb0+lbound(val)
if ((minval(ilow0-lbound(val)) < 0) .or. (maxval(iup0-ubound(val)) > 0)) then
  outstr0='array ' // trim(ident) // ' is too large'
  ierr0=2
  goto 900
endif
!
! --- Format ---
call uio_exkeyw(termt,ntt, frmtna,format0,mode=1)
if (present(format)) format=format0
call uio_exkeyw(termt,ntt, plinna,perlin, mode=1)
if ((format0 == '') .or. (format0 == '*')) then
  rfrmt='*'
else
  call uio_wf2rf(format0,format0, qmachr=qmachr)
  rfrmt='(' // trim(perlin) // '(' // trim(format0) // '))'
endif
read(perlin,'(I6)') nperlin0
if (present(nperlin)) nperlin=nperlin0
call uio_exkeyw(termt,ntt, bytena,bytes , mode=1)
read(bytes,'(I6)') nbytes0
if (present(nbytes)) nbytes=nbytes0
!
! --- Additional keywords ---
if (present(name)) call uio_exkeyw(termt,ntt, namena,name  , mode=1)
if (present(unit)) call uio_exkeyw(termt,ntt, unitna,unit  , mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date  , mode=1)
call uio_exkeyw(termt,ntt, tranna,tran0 , mode=3)
if (present(tran)) tran=tran0
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=nc, outstr=form)
!
! --- Read value from file and increment file position counter ---
val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))=''
if (form == 'unformatted') then
  read(nc) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))(:nbytes0)
  ncpos(nc)=ncpos(nc)+1
else
  if (rfrmt == '*') then
    read(nc,*) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))(:nbytes0)
  else
    read(nc,rfrmt) val(ilow0(1):iup0(1),ilow0(2):iup0(2), &
                       ilow0(3):iup0(3),ilow0(4):iup0(4))(:nbytes0)
  endif
  ncpos(nc)=ncpos(nc)+((iup0(1)-ilow0(1)+1)*(iup0(2)-ilow0(2)+1)* &
                       (iup0(3)-ilow0(3)+1)*(iup0(4)-ilow0(4)+1)-1)/nperlin0+1
endif
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
if (ierr0 > 0) call uio_err(outstr=outstr0, ierr=ierr0, nc=nc, &
                            routine='uio_rds4', errlevel=3)
!
end subroutine uio_rds4


!----------**********-------------------------------------------------------------------------------
subroutine uio_rdlabl(nc,termt,ntt, ident,name,date, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdlabl ('uio_read_label')
!
! PURPOSE:
!   Read label.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdlabl(nc,termt,ntt, ident,name=name,date=date, &
!                   outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number (actually not used)
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   ident:  ('identifier') character, identifier
!   name:   ('name') character, optional, name of entry
!   date:   ('date') character, optional, date information
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   uio_exkeyw:            Extract value of keyword from table
!
! MODULES:
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Read data from file.
!
! RESTRICTIONS:
!   termt contains entry header information.
!
! PROCEDURE:
!   Extract keywords from term table, read value from file.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-01-10 (B.F. Kiel) Written
!   1997-01-18 (B.F.) Parameters made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
integer, intent(inout)                  :: ntt
character*(*), intent(inout)            :: termt(2,nttmx)
character*(*), intent(out)              :: ident
character*(*), optional, intent(out)    :: name, date
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nc0
character*(1)                           :: sdum
!---------------------------------------------------------------------------------------------------
! --- Put parameter into variable to prevent compiler from complaining about --
! --- Unused dummy variable NC ---
nc0=nc
!
! --- Find keywords and extract their values ---
ident=termt(1,2)
termt(1,1)=''
termt(1,2)=''
if (present(name)) call uio_exkeyw(termt,ntt, namena,name,   mode=1)
if (present(date)) call uio_exkeyw(termt,ntt, datena,date,   mode=1)
call uio_exkeyw(termt,ntt, datena,sdum, mode=4)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_rdlabl


!----------**********-------------------------------------------------------------------------------
subroutine uio_wrhdln(nc,form,linet,nlt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrhdln ('uio_write_head_line')
!
! PURPOSE:
!   Write header of variable, input: line table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrhdln(nc,form,linet,nlt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   form:   ('format') character, 'formatted' or 'unformatted'
!   linet:  ('line_table') string array, nltmx lines
!   nlt:    ('number_line_table') integer, number of non-empty lines of linet
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Write data to file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Write entry header line by line into file according to file format.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-17 (B.F. Kiel) Written
!   1995-12-06 (B.F.) 'uiosiz.inc' included, '(A80)' -> '(A)'
!   1996-10-16 (B.F.) More comments
!   1996-10-28 (B.F.) Renaming
!   1997-01-18 (B.F.) outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc, nlt
character*(*), intent(in)               :: form, linet(nltmx)
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nlti
!
!---------------------------------------------------------------------------------------------------
if (form == 'unformatted') then
  ! --- Write unformatted ---
  do nlti=1,nlt
    write(nc) linet(nlti)
  end do ! nlti
else
  ! --- Write formatted ---
  !         do nlti=1,nlt
  !           write(nc,'(A80)') (linet(nlti),nlti=1,nlt)
  write(nc,'(A)') (linet(nlti),nlti=1,nlt)
  !         end do ! nlti
endif
!
! --- Increment file position counter ---
ncpos(nc)=ncpos(nc)+nlt
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_wrhdln


!----------********---------------------------------------------------------------------------------
subroutine uio_wrhd(nc,form, termt,ntt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_wrhd ('uio_write_head')
!
! PURPOSE:
!   Write header of variable, input: term table.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_wrhd(nc,form, termt,ntt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   form:   ('format') character, 'formatted' or 'unformatted'
!   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!           (ntt >= 0)
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! LOCAL VARIABLES:
!   linet:  ('line_table') string array, nltmx lines
!   nlt:    ('number_line_table') integer, number of non-empty lines of linet
!
! ROUTINES:
!   uio_memtrm:            Merge a list of terms (keywords and their values) into a line table
!   uio_wrhdln:            Write header of variable, input: line table
!
! MODULES:
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Data are written into file on unit nc
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Merge term table (call uio_memtrm) and
!   write entry header into file (call uio_wrhdln).
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1996-10-27 (B.F. Kiel) Written
!   1997-01-18 (B.F.) outstr,ierr made optional
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc, ntt
character*(*), intent(in)               :: form
character*(*), intent(in)               :: termt(2,nttmx)
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: nlt, ierr0
character*(80)                          :: outstr0
character*(lel)                         :: linet(nltmx)
!
!---------------------------------------------------------------------------------------------------
!
! --- Merge header ---
nlt=0
call uio_memtrm(termt,ntt, linet,nlt, outstr0,ierr0)
!
! --- Write header ---
call uio_wrhdln(nc,form, linet,nlt, outstr=outstr0,ierr=ierr0)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_wrhd


!----------**********-------------------------------------------------------------------------------
subroutine uio_rdhdln(nc,form, linet,nlt, outstr,ierr)
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_rdhdln ('uio_read_head_line')
!
! PURPOSE:
!   Read header of entry.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_rdhdln(nc,form, linet,nlt, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   form:   ('format') character, 'formatted' or 'unformatted'
!
! OUTPUT:
!   linet:  ('line_table') string array (nltmx), nltmx lines
!   nlt:    ('number_line_table') integer, number of non-empty lines of linet
!   outstr: ('output_string') character
!   ierr:   ('integer_error') integer, contains exit error status
!            0: correct operation
!           >0: occurrence of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_cst_module:        Channel status information
!   uio_nam_module:        Definition of names for UIO
!   uio_siz_module:        String length, table size for UIO
!
! SIDE EFFECTS:
!   Data are read from file on unit nc.
!
! RESTRICTIONS:
!   File pointer has to be positioned properly on beginning of entry header.
!
! PROCEDURE:
!   Read file line by line until end of header or end of file is reached.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   1995-11-17 (B.F. Kiel) Written
!   1995-12-06 (B.F.) 'uiosiz.inc' included, '(A80)' -> '(A)'
!   1996-01-05 (B.F.) EOF tested
!   1996-10-09 (B.F.) Allow empty lines (treat them as '&'). Don't load lines with single '&'
!   1996-10-15 (B.F.) More comments
!   1996-10-21 (B.F.) File position counter
!   1996-10-28 (B.F.) Renaming
!   1996-11-28 (B.F.) backspace nc if end of file reached
!   1997-01-06 (B.F.) Some gotos removed
!   1997-01-18 (B.F.) outstr,ierr made optional
!   1997-01-19 (B.F.) Remove backspace nc if end of file reached
!---------------------------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_cst_module
!
implicit none
!
! --- I/O parameters ---
integer, intent(in)                     :: nc
integer, intent(out)                    :: nlt
character*(*), intent(in)               :: form
character*(*), intent(out)              :: linet(nltmx)
character*(*), optional, intent(out)    :: outstr
integer, optional, intent(out)          :: ierr
!
! --- Local variables ---
integer                                 :: idxlin, ierr0
character*(lel)                         :: line
character*(80)                          :: outstr0
!---------------------------------------------------------------------------------------------------
ierr0=0
!
nlt=1
do
  ! --- Read line according to current file format ---
  if (form == 'unformatted') then
    read(nc, end=10) line
  else
    read(nc, '(A)', end=10) line
  endif
  goto 20
  !
  ! --- End of file reached ---
  10 nlt=nlt-1
  backspace nc  ! 19.01.97, 13.02.97
  ierr0=0
  outstr0='eof'
  exit
  !
  ! --- Increment file position counter ---
  20 ncpos(nc)=ncpos(nc)+1
  ! --- Delete leading blanks ---
  line=adjustl(line)
  linet(nlt)=line
  ! --- Is there a following header line? ---
  if (line == '') then
    ! --- Empty line ---
  else if (line == lnkchr) then
    ! --- Line with only one link character ---
  else
    idxlin=len_trim(line)
    if (idxlin == 1) then
      outstr0='Only one letter'
      exit
    else if (line(idxlin-1:) == ' ' // lnkchr) then
      ! --- Line with link character at end ---
      nlt=nlt+1
    else
      outstr0='end'
      exit
    endif
  endif
end do
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_rdhdln

end module uio_bulk_module
